const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["assets/dagre-6UL2VRFP-CXCUh4Fa.js","assets/graph-DWQBisi2.js","assets/_baseUniq-DDor4wmy.js","assets/layout-CF3HVlxr.js","assets/_basePickBy-Bp-wv1oY.js","assets/clone-B20-qf8D.js","assets/cose-bilkent-S5V4N54A-w8Z6_s03.js","assets/cytoscape.esm-Dr_J-Pzw.js","assets/c4Diagram-YG6GDRKO-wsxKHdY7.js","assets/chunk-TZMSLE5B-_Su9CZNB.js","assets/flowDiagram-NV44I4VS-Cea10LNu.js","assets/chunk-FMBD7UC4-C0EMQWgw.js","assets/chunk-55IACEB6-Ujowu9We.js","assets/chunk-QN33PNHL-CfSmMJJo.js","assets/channel-Dl-A3qMa.js","assets/erDiagram-Q2GNP2WA-Co43UztB.js","assets/gitGraphDiagram-NY62KEGX-BoXPnSbQ.js","assets/chunk-4BX2VUAB-BKqT35W8.js","assets/chunk-QZHKN3VN-DDCNmJ4Z.js","assets/treemap-75Q7IDZK-CkpjdK7r.js","assets/ganttDiagram-LVOFAZNH-K1v31eUf.js","assets/linear-CEmFeyKG.js","assets/init-Gi6I4Gst.js","assets/defaultLocale-C4B-KCzX.js","assets/infoDiagram-F6ZHWCRC-CqiQjXdQ.js","assets/pieDiagram-ADFJNKIX-DVpm4HYO.js","assets/arc-DmitnDGg.js","assets/ordinal-BENe2yWM.js","assets/quadrantDiagram-AYHSOK5B-B_q4uP4u.js","assets/xychartDiagram-PRI3JC2R-BQB5fhK1.js","assets/requirementDiagram-UZGBJVZJ-D3V1ZBb3.js","assets/sequenceDiagram-WL72ISMW-B4tO_SHN.js","assets/classDiagram-2ON5EDUG-CKcdz_xq.js","assets/chunk-B4BG7PRW-DW00c-ku.js","assets/classDiagram-v2-WZHVMYZB-CKcdz_xq.js","assets/stateDiagram-FKZM4ZOC-DZeTsklT.js","assets/chunk-DI55MBZ5-BeghIf-o.js","assets/stateDiagram-v2-4FDKWEC3-BXv4Etpu.js","assets/journeyDiagram-XKPGCS4Q-C3yL6hVc.js","assets/timeline-definition-IT6M3QCI-EppKg-g9.js","assets/mindmap-definition-VGOIOE7T-1BY12Tdp.js","assets/kanban-definition-3W4ZIXB7-Dnras9Cb.js","assets/sankeyDiagram-TZEHDZUN-D78eCDcZ.js","assets/diagram-S2PKOQOG-BACuWjdy.js","assets/diagram-QEK2KX5R-BrWPzwYr.js","assets/blockDiagram-VD42YOAC-q24NfDqw.js","assets/architectureDiagram-VXUJARFQ-BYI4md1h.js","assets/diagram-PSM6KHXK-BdzOSGFs.js"])))=>i.map(i=>d[i]);
var Xhe=Object.defineProperty;var Zhe=(e,t,r)=>t in e?Xhe(e,t,{enumerable:!0,configurable:!0,writable:!0,value:r}):e[t]=r;var mt=(e,t,r)=>Zhe(e,typeof t!="symbol"?t+"":t,r);function Qhe(e,t){for(var r=0;r<t.length;r++){const n=t[r];if(typeof n!="string"&&!Array.isArray(n)){for(const a in n)if(a!=="default"&&!(a in e)){const i=Object.getOwnPropertyDescriptor(n,a);i&&Object.defineProperty(e,a,i.get?i:{enumerable:!0,get:()=>n[a]})}}}return Object.freeze(Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}))}(function(){const t=document.createElement("link").relList;if(t&&t.supports&&t.supports("modulepreload"))return;for(const a of document.querySelectorAll('link[rel="modulepreload"]'))n(a);new MutationObserver(a=>{for(const i of a)if(i.type==="childList")for(const o of i.addedNodes)o.tagName==="LINK"&&o.rel==="modulepreload"&&n(o)}).observe(document,{childList:!0,subtree:!0});function r(a){const i={};return a.integrity&&(i.integrity=a.integrity),a.referrerPolicy&&(i.referrerPolicy=a.referrerPolicy),a.crossOrigin==="use-credentials"?i.credentials="include":a.crossOrigin==="anonymous"?i.credentials="omit":i.credentials="same-origin",i}function n(a){if(a.ep)return;a.ep=!0;const i=r(a);fetch(a.href,i)}})();var ey=typeof globalThis<"u"?globalThis:typeof window<"u"?window:typeof global<"u"?global:typeof self<"u"?self:{};function jn(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}function epe(e){if(Object.prototype.hasOwnProperty.call(e,"__esModule"))return e;var t=e.default;if(typeof t=="function"){var r=function n(){return this instanceof n?Reflect.construct(t,arguments,this.constructor):t.apply(this,arguments)};r.prototype=t.prototype}else r={};return Object.defineProperty(r,"__esModule",{value:!0}),Object.keys(e).forEach(function(n){var a=Object.getOwnPropertyDescriptor(e,n);Object.defineProperty(r,n,a.get?a:{enumerable:!0,get:function(){return e[n]}})}),r}var mx={exports:{}},kf={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var s9;function tpe(){if(s9)return kf;s9=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.fragment");function r(n,a,i){var o=null;if(i!==void 0&&(o=""+i),a.key!==void 0&&(o=""+a.key),"key"in a){i={};for(var s in a)s!=="key"&&(i[s]=a[s])}else i=a;return a=i.ref,{$$typeof:e,type:n,key:o,ref:a!==void 0?a:null,props:i}}return kf.Fragment=t,kf.jsx=r,kf.jsxs=r,kf}var l9;function rpe(){return l9||(l9=1,mx.exports=tpe()),mx.exports}var x=rpe(),gx={exports:{}},cr={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var u9;function npe(){if(u9)return cr;u9=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.portal"),r=Symbol.for("react.fragment"),n=Symbol.for("react.strict_mode"),a=Symbol.for("react.profiler"),i=Symbol.for("react.consumer"),o=Symbol.for("react.context"),s=Symbol.for("react.forward_ref"),l=Symbol.for("react.suspense"),u=Symbol.for("react.memo"),d=Symbol.for("react.lazy"),p=Symbol.for("react.activity"),f=Symbol.iterator;function m(K){return K===null||typeof K!="object"?null:(K=f&&K[f]||K["@@iterator"],typeof K=="function"?K:null)}var b={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},y=Object.assign,S={};function w(K,G,P){this.props=K,this.context=G,this.refs=S,this.updater=P||b}w.prototype.isReactComponent={},w.prototype.setState=function(K,G){if(typeof K!="object"&&typeof K!="function"&&K!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,K,G,"setState")},w.prototype.forceUpdate=function(K){this.updater.enqueueForceUpdate(this,K,"forceUpdate")};function E(){}E.prototype=w.prototype;function k(K,G,P){this.props=K,this.context=G,this.refs=S,this.updater=P||b}var _=k.prototype=new E;_.constructor=k,y(_,w.prototype),_.isPureReactComponent=!0;var R=Array.isArray;function C(){}var F={H:null,A:null,T:null,S:null},L=Object.prototype.hasOwnProperty;function I(K,G,P){var ne=P.ref;return{$$typeof:e,type:K,key:G,ref:ne!==void 0?ne:null,props:P}}function M(K,G){return I(K.type,G,K.props)}function j(K){return typeof K=="object"&&K!==null&&K.$$typeof===e}function Y(K){var G={"=":"=0",":":"=2"};return"$"+K.replace(/[=:]/g,function(P){return G[P]})}var W=/\/+/g;function $(K,G){return typeof K=="object"&&K!==null&&K.key!=null?Y(""+K.key):G.toString(36)}function X(K){switch(K.status){case"fulfilled":return K.value;case"rejected":throw K.reason;default:switch(typeof K.status=="string"?K.then(C,C):(K.status="pending",K.then(function(G){K.status==="pending"&&(K.status="fulfilled",K.value=G)},function(G){K.status==="pending"&&(K.status="rejected",K.reason=G)})),K.status){case"fulfilled":return K.value;case"rejected":throw K.reason}}throw K}function z(K,G,P,ne,oe){var pe=typeof K;(pe==="undefined"||pe==="boolean")&&(K=null);var ae=!1;if(K===null)ae=!0;else switch(pe){case"bigint":case"string":case"number":ae=!0;break;case"object":switch(K.$$typeof){case e:case t:ae=!0;break;case d:return ae=K._init,z(ae(K._payload),G,P,ne,oe)}}if(ae)return oe=oe(K),ae=ne===""?"."+$(K,0):ne,R(oe)?(P="",ae!=null&&(P=ae.replace(W,"$&/")+"/"),z(oe,G,P,"",function(fe){return fe})):oe!=null&&(j(oe)&&(oe=M(oe,P+(oe.key==null||K&&K.key===oe.key?"":(""+oe.key).replace(W,"$&/")+"/")+ae)),G.push(oe)),1;ae=0;var se=ne===""?".":ne+":";if(R(K))for(var Ee=0;Ee<K.length;Ee++)ne=K[Ee],pe=se+$(ne,Ee),ae+=z(ne,G,P,pe,oe);else if(Ee=m(K),typeof Ee=="function")for(K=Ee.call(K),Ee=0;!(ne=K.next()).done;)ne=ne.value,pe=se+$(ne,Ee++),ae+=z(ne,G,P,pe,oe);else if(pe==="object"){if(typeof K.then=="function")return z(X(K),G,P,ne,oe);throw G=String(K),Error("Objects are not valid as a React child (found: "+(G==="[object Object]"?"object with keys {"+Object.keys(K).join(", ")+"}":G)+"). If you meant to render a collection of children, use an array instead.")}return ae}function H(K,G,P){if(K==null)return K;var ne=[],oe=0;return z(K,ne,"","",function(pe){return G.call(P,pe,oe++)}),ne}function B(K){if(K._status===-1){var G=K._result;G=G(),G.then(function(P){(K._status===0||K._status===-1)&&(K._status=1,K._result=P)},function(P){(K._status===0||K._status===-1)&&(K._status=2,K._result=P)}),K._status===-1&&(K._status=0,K._result=G)}if(K._status===1)return K._result.default;throw K._result}var ee=typeof reportError=="function"?reportError:function(K){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var G=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof K=="object"&&K!==null&&typeof K.message=="string"?String(K.message):String(K),error:K});if(!window.dispatchEvent(G))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",K);return}console.error(K)},J={map:H,forEach:function(K,G,P){H(K,function(){G.apply(this,arguments)},P)},count:function(K){var G=0;return H(K,function(){G++}),G},toArray:function(K){return H(K,function(G){return G})||[]},only:function(K){if(!j(K))throw Error("React.Children.only expected to receive a single React element child.");return K}};return cr.Activity=p,cr.Children=J,cr.Component=w,cr.Fragment=r,cr.Profiler=a,cr.PureComponent=k,cr.StrictMode=n,cr.Suspense=l,cr.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=F,cr.__COMPILER_RUNTIME={__proto__:null,c:function(K){return F.H.useMemoCache(K)}},cr.cache=function(K){return function(){return K.apply(null,arguments)}},cr.cacheSignal=function(){return null},cr.cloneElement=function(K,G,P){if(K==null)throw Error("The argument must be a React element, but you passed "+K+".");var ne=y({},K.props),oe=K.key;if(G!=null)for(pe in G.key!==void 0&&(oe=""+G.key),G)!L.call(G,pe)||pe==="key"||pe==="__self"||pe==="__source"||pe==="ref"&&G.ref===void 0||(ne[pe]=G[pe]);var pe=arguments.length-2;if(pe===1)ne.children=P;else if(1<pe){for(var ae=Array(pe),se=0;se<pe;se++)ae[se]=arguments[se+2];ne.children=ae}return I(K.type,oe,ne)},cr.createContext=function(K){return K={$$typeof:o,_currentValue:K,_currentValue2:K,_threadCount:0,Provider:null,Consumer:null},K.Provider=K,K.Consumer={$$typeof:i,_context:K},K},cr.createElement=function(K,G,P){var ne,oe={},pe=null;if(G!=null)for(ne in G.key!==void 0&&(pe=""+G.key),G)L.call(G,ne)&&ne!=="key"&&ne!=="__self"&&ne!=="__source"&&(oe[ne]=G[ne]);var ae=arguments.length-2;if(ae===1)oe.children=P;else if(1<ae){for(var se=Array(ae),Ee=0;Ee<ae;Ee++)se[Ee]=arguments[Ee+2];oe.children=se}if(K&&K.defaultProps)for(ne in ae=K.defaultProps,ae)oe[ne]===void 0&&(oe[ne]=ae[ne]);return I(K,pe,oe)},cr.createRef=function(){return{current:null}},cr.forwardRef=function(K){return{$$typeof:s,render:K}},cr.isValidElement=j,cr.lazy=function(K){return{$$typeof:d,_payload:{_status:-1,_result:K},_init:B}},cr.memo=function(K,G){return{$$typeof:u,type:K,compare:G===void 0?null:G}},cr.startTransition=function(K){var G=F.T,P={};F.T=P;try{var ne=K(),oe=F.S;oe!==null&&oe(P,ne),typeof ne=="object"&&ne!==null&&typeof ne.then=="function"&&ne.then(C,ee)}catch(pe){ee(pe)}finally{G!==null&&P.types!==null&&(G.types=P.types),F.T=G}},cr.unstable_useCacheRefresh=function(){return F.H.useCacheRefresh()},cr.use=function(K){return F.H.use(K)},cr.useActionState=function(K,G,P){return F.H.useActionState(K,G,P)},cr.useCallback=function(K,G){return F.H.useCallback(K,G)},cr.useContext=function(K){return F.H.useContext(K)},cr.useDebugValue=function(){},cr.useDeferredValue=function(K,G){return F.H.useDeferredValue(K,G)},cr.useEffect=function(K,G){return F.H.useEffect(K,G)},cr.useEffectEvent=function(K){return F.H.useEffectEvent(K)},cr.useId=function(){return F.H.useId()},cr.useImperativeHandle=function(K,G,P){return F.H.useImperativeHandle(K,G,P)},cr.useInsertionEffect=function(K,G){return F.H.useInsertionEffect(K,G)},cr.useLayoutEffect=function(K,G){return F.H.useLayoutEffect(K,G)},cr.useMemo=function(K,G){return F.H.useMemo(K,G)},cr.useOptimistic=function(K,G){return F.H.useOptimistic(K,G)},cr.useReducer=function(K,G,P){return F.H.useReducer(K,G,P)},cr.useRef=function(K){return F.H.useRef(K)},cr.useState=function(K){return F.H.useState(K)},cr.useSyncExternalStore=function(K,G,P){return F.H.useSyncExternalStore(K,G,P)},cr.useTransition=function(){return F.H.useTransition()},cr.version="19.2.0",cr}var c9;function nD(){return c9||(c9=1,gx.exports=npe()),gx.exports}var T=nD();const $e=jn(T),uK=Qhe({__proto__:null,default:$e},[T]);var bx={exports:{}},Cf={},yx={exports:{}},vx={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var d9;function ape(){return d9||(d9=1,(function(e){function t(z,H){var B=z.length;z.push(H);e:for(;0<B;){var ee=B-1>>>1,J=z[ee];if(0<a(J,H))z[ee]=H,z[B]=J,B=ee;else break e}}function r(z){return z.length===0?null:z[0]}function n(z){if(z.length===0)return null;var H=z[0],B=z.pop();if(B!==H){z[0]=B;e:for(var ee=0,J=z.length,K=J>>>1;ee<K;){var G=2*(ee+1)-1,P=z[G],ne=G+1,oe=z[ne];if(0>a(P,B))ne<J&&0>a(oe,P)?(z[ee]=oe,z[ne]=B,ee=ne):(z[ee]=P,z[G]=B,ee=G);else if(ne<J&&0>a(oe,B))z[ee]=oe,z[ne]=B,ee=ne;else break e}}return H}function a(z,H){var B=z.sortIndex-H.sortIndex;return B!==0?B:z.id-H.id}if(e.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var i=performance;e.unstable_now=function(){return i.now()}}else{var o=Date,s=o.now();e.unstable_now=function(){return o.now()-s}}var l=[],u=[],d=1,p=null,f=3,m=!1,b=!1,y=!1,S=!1,w=typeof setTimeout=="function"?setTimeout:null,E=typeof clearTimeout=="function"?clearTimeout:null,k=typeof setImmediate<"u"?setImmediate:null;function _(z){for(var H=r(u);H!==null;){if(H.callback===null)n(u);else if(H.startTime<=z)n(u),H.sortIndex=H.expirationTime,t(l,H);else break;H=r(u)}}function R(z){if(y=!1,_(z),!b)if(r(l)!==null)b=!0,C||(C=!0,Y());else{var H=r(u);H!==null&&X(R,H.startTime-z)}}var C=!1,F=-1,L=5,I=-1;function M(){return S?!0:!(e.unstable_now()-I<L)}function j(){if(S=!1,C){var z=e.unstable_now();I=z;var H=!0;try{e:{b=!1,y&&(y=!1,E(F),F=-1),m=!0;var B=f;try{t:{for(_(z),p=r(l);p!==null&&!(p.expirationTime>z&&M());){var ee=p.callback;if(typeof ee=="function"){p.callback=null,f=p.priorityLevel;var J=ee(p.expirationTime<=z);if(z=e.unstable_now(),typeof J=="function"){p.callback=J,_(z),H=!0;break t}p===r(l)&&n(l),_(z)}else n(l);p=r(l)}if(p!==null)H=!0;else{var K=r(u);K!==null&&X(R,K.startTime-z),H=!1}}break e}finally{p=null,f=B,m=!1}H=void 0}}finally{H?Y():C=!1}}}var Y;if(typeof k=="function")Y=function(){k(j)};else if(typeof MessageChannel<"u"){var W=new MessageChannel,$=W.port2;W.port1.onmessage=j,Y=function(){$.postMessage(null)}}else Y=function(){w(j,0)};function X(z,H){F=w(function(){z(e.unstable_now())},H)}e.unstable_IdlePriority=5,e.unstable_ImmediatePriority=1,e.unstable_LowPriority=4,e.unstable_NormalPriority=3,e.unstable_Profiling=null,e.unstable_UserBlockingPriority=2,e.unstable_cancelCallback=function(z){z.callback=null},e.unstable_forceFrameRate=function(z){0>z||125<z?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):L=0<z?Math.floor(1e3/z):5},e.unstable_getCurrentPriorityLevel=function(){return f},e.unstable_next=function(z){switch(f){case 1:case 2:case 3:var H=3;break;default:H=f}var B=f;f=H;try{return z()}finally{f=B}},e.unstable_requestPaint=function(){S=!0},e.unstable_runWithPriority=function(z,H){switch(z){case 1:case 2:case 3:case 4:case 5:break;default:z=3}var B=f;f=z;try{return H()}finally{f=B}},e.unstable_scheduleCallback=function(z,H,B){var ee=e.unstable_now();switch(typeof B=="object"&&B!==null?(B=B.delay,B=typeof B=="number"&&0<B?ee+B:ee):B=ee,z){case 1:var J=-1;break;case 2:J=250;break;case 5:J=1073741823;break;case 4:J=1e4;break;default:J=5e3}return J=B+J,z={id:d++,callback:H,priorityLevel:z,startTime:B,expirationTime:J,sortIndex:-1},B>ee?(z.sortIndex=B,t(u,z),r(l)===null&&z===r(u)&&(y?(E(F),F=-1):y=!0,X(R,B-ee))):(z.sortIndex=J,t(l,z),b||m||(b=!0,C||(C=!0,Y()))),z},e.unstable_shouldYield=M,e.unstable_wrapCallback=function(z){var H=f;return function(){var B=f;f=H;try{return z.apply(this,arguments)}finally{f=B}}}})(vx)),vx}var h9;function ipe(){return h9||(h9=1,yx.exports=ape()),yx.exports}var Sx={exports:{}},wa={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var p9;function ope(){if(p9)return wa;p9=1;var e=nD();function t(l){var u="https://react.dev/errors/"+l;if(1<arguments.length){u+="?args[]="+encodeURIComponent(arguments[1]);for(var d=2;d<arguments.length;d++)u+="&args[]="+encodeURIComponent(arguments[d])}return"Minified React error #"+l+"; visit "+u+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function r(){}var n={d:{f:r,r:function(){throw Error(t(522))},D:r,C:r,L:r,m:r,X:r,S:r,M:r},p:0,findDOMNode:null},a=Symbol.for("react.portal");function i(l,u,d){var p=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:a,key:p==null?null:""+p,children:l,containerInfo:u,implementation:d}}var o=e.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function s(l,u){if(l==="font")return"";if(typeof u=="string")return u==="use-credentials"?u:""}return wa.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=n,wa.createPortal=function(l,u){var d=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!u||u.nodeType!==1&&u.nodeType!==9&&u.nodeType!==11)throw Error(t(299));return i(l,u,null,d)},wa.flushSync=function(l){var u=o.T,d=n.p;try{if(o.T=null,n.p=2,l)return l()}finally{o.T=u,n.p=d,n.d.f()}},wa.preconnect=function(l,u){typeof l=="string"&&(u?(u=u.crossOrigin,u=typeof u=="string"?u==="use-credentials"?u:"":void 0):u=null,n.d.C(l,u))},wa.prefetchDNS=function(l){typeof l=="string"&&n.d.D(l)},wa.preinit=function(l,u){if(typeof l=="string"&&u&&typeof u.as=="string"){var d=u.as,p=s(d,u.crossOrigin),f=typeof u.integrity=="string"?u.integrity:void 0,m=typeof u.fetchPriority=="string"?u.fetchPriority:void 0;d==="style"?n.d.S(l,typeof u.precedence=="string"?u.precedence:void 0,{crossOrigin:p,integrity:f,fetchPriority:m}):d==="script"&&n.d.X(l,{crossOrigin:p,integrity:f,fetchPriority:m,nonce:typeof u.nonce=="string"?u.nonce:void 0})}},wa.preinitModule=function(l,u){if(typeof l=="string")if(typeof u=="object"&&u!==null){if(u.as==null||u.as==="script"){var d=s(u.as,u.crossOrigin);n.d.M(l,{crossOrigin:d,integrity:typeof u.integrity=="string"?u.integrity:void 0,nonce:typeof u.nonce=="string"?u.nonce:void 0})}}else u==null&&n.d.M(l)},wa.preload=function(l,u){if(typeof l=="string"&&typeof u=="object"&&u!==null&&typeof u.as=="string"){var d=u.as,p=s(d,u.crossOrigin);n.d.L(l,d,{crossOrigin:p,integrity:typeof u.integrity=="string"?u.integrity:void 0,nonce:typeof u.nonce=="string"?u.nonce:void 0,type:typeof u.type=="string"?u.type:void 0,fetchPriority:typeof u.fetchPriority=="string"?u.fetchPriority:void 0,referrerPolicy:typeof u.referrerPolicy=="string"?u.referrerPolicy:void 0,imageSrcSet:typeof u.imageSrcSet=="string"?u.imageSrcSet:void 0,imageSizes:typeof u.imageSizes=="string"?u.imageSizes:void 0,media:typeof u.media=="string"?u.media:void 0})}},wa.preloadModule=function(l,u){if(typeof l=="string")if(u){var d=s(u.as,u.crossOrigin);n.d.m(l,{as:typeof u.as=="string"&&u.as!=="script"?u.as:void 0,crossOrigin:d,integrity:typeof u.integrity=="string"?u.integrity:void 0})}else n.d.m(l)},wa.requestFormReset=function(l){n.d.r(l)},wa.unstable_batchedUpdates=function(l,u){return l(u)},wa.useFormState=function(l,u,d){return o.H.useFormState(l,u,d)},wa.useFormStatus=function(){return o.H.useHostTransitionStatus()},wa.version="19.2.0",wa}var f9;function cK(){if(f9)return Sx.exports;f9=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),Sx.exports=ope(),Sx.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var m9;function spe(){if(m9)return Cf;m9=1;var e=ipe(),t=nD(),r=cK();function n(c){var h="https://react.dev/errors/"+c;if(1<arguments.length){h+="?args[]="+encodeURIComponent(arguments[1]);for(var g=2;g<arguments.length;g++)h+="&args[]="+encodeURIComponent(arguments[g])}return"Minified React error #"+c+"; visit "+h+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function a(c){return!(!c||c.nodeType!==1&&c.nodeType!==9&&c.nodeType!==11)}function i(c){var h=c,g=c;if(c.alternate)for(;h.return;)h=h.return;else{c=h;do h=c,(h.flags&4098)!==0&&(g=h.return),c=h.return;while(c)}return h.tag===3?g:null}function o(c){if(c.tag===13){var h=c.memoizedState;if(h===null&&(c=c.alternate,c!==null&&(h=c.memoizedState)),h!==null)return h.dehydrated}return null}function s(c){if(c.tag===31){var h=c.memoizedState;if(h===null&&(c=c.alternate,c!==null&&(h=c.memoizedState)),h!==null)return h.dehydrated}return null}function l(c){if(i(c)!==c)throw Error(n(188))}function u(c){var h=c.alternate;if(!h){if(h=i(c),h===null)throw Error(n(188));return h!==c?null:c}for(var g=c,v=h;;){var A=g.return;if(A===null)break;var D=A.alternate;if(D===null){if(v=A.return,v!==null){g=v;continue}break}if(A.child===D.child){for(D=A.child;D;){if(D===g)return l(A),c;if(D===v)return l(A),h;D=D.sibling}throw Error(n(188))}if(g.return!==v.return)g=A,v=D;else{for(var V=!1,Q=A.child;Q;){if(Q===g){V=!0,g=A,v=D;break}if(Q===v){V=!0,v=A,g=D;break}Q=Q.sibling}if(!V){for(Q=D.child;Q;){if(Q===g){V=!0,g=D,v=A;break}if(Q===v){V=!0,v=D,g=A;break}Q=Q.sibling}if(!V)throw Error(n(189))}}if(g.alternate!==v)throw Error(n(190))}if(g.tag!==3)throw Error(n(188));return g.stateNode.current===g?c:h}function d(c){var h=c.tag;if(h===5||h===26||h===27||h===6)return c;for(c=c.child;c!==null;){if(h=d(c),h!==null)return h;c=c.sibling}return null}var p=Object.assign,f=Symbol.for("react.element"),m=Symbol.for("react.transitional.element"),b=Symbol.for("react.portal"),y=Symbol.for("react.fragment"),S=Symbol.for("react.strict_mode"),w=Symbol.for("react.profiler"),E=Symbol.for("react.consumer"),k=Symbol.for("react.context"),_=Symbol.for("react.forward_ref"),R=Symbol.for("react.suspense"),C=Symbol.for("react.suspense_list"),F=Symbol.for("react.memo"),L=Symbol.for("react.lazy"),I=Symbol.for("react.activity"),M=Symbol.for("react.memo_cache_sentinel"),j=Symbol.iterator;function Y(c){return c===null||typeof c!="object"?null:(c=j&&c[j]||c["@@iterator"],typeof c=="function"?c:null)}var W=Symbol.for("react.client.reference");function $(c){if(c==null)return null;if(typeof c=="function")return c.$$typeof===W?null:c.displayName||c.name||null;if(typeof c=="string")return c;switch(c){case y:return"Fragment";case w:return"Profiler";case S:return"StrictMode";case R:return"Suspense";case C:return"SuspenseList";case I:return"Activity"}if(typeof c=="object")switch(c.$$typeof){case b:return"Portal";case k:return c.displayName||"Context";case E:return(c._context.displayName||"Context")+".Consumer";case _:var h=c.render;return c=c.displayName,c||(c=h.displayName||h.name||"",c=c!==""?"ForwardRef("+c+")":"ForwardRef"),c;case F:return h=c.displayName||null,h!==null?h:$(c.type)||"Memo";case L:h=c._payload,c=c._init;try{return $(c(h))}catch{}}return null}var X=Array.isArray,z=t.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,H=r.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,B={pending:!1,data:null,method:null,action:null},ee=[],J=-1;function K(c){return{current:c}}function G(c){0>J||(c.current=ee[J],ee[J]=null,J--)}function P(c,h){J++,ee[J]=c.current,c.current=h}var ne=K(null),oe=K(null),pe=K(null),ae=K(null);function se(c,h){switch(P(pe,h),P(oe,c),P(ne,null),h.nodeType){case 9:case 11:c=(c=h.documentElement)&&(c=c.namespaceURI)?RO(c):0;break;default:if(c=h.tagName,h=h.namespaceURI)h=RO(h),c=DO(h,c);else switch(c){case"svg":c=1;break;case"math":c=2;break;default:c=0}}G(ne),P(ne,c)}function Ee(){G(ne),G(oe),G(pe)}function fe(c){c.memoizedState!==null&&P(ae,c);var h=ne.current,g=DO(h,c.type);h!==g&&(P(oe,c),P(ne,g))}function Me(c){oe.current===c&&(G(ne),G(oe)),ae.current===c&&(G(ae),Sf._currentValue=B)}var Oe,xe;function Te(c){if(Oe===void 0)try{throw Error()}catch(g){var h=g.stack.trim().match(/\n( *(at )?)/);Oe=h&&h[1]||"",xe=-1<g.stack.indexOf(`
    at`)?" (<anonymous>)":-1<g.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Oe+c+xe}var Ne=!1;function he(c,h){if(!c||Ne)return"";Ne=!0;var g=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var v={DetermineComponentFrameRoot:function(){try{if(h){var Ue=function(){throw Error()};if(Object.defineProperty(Ue.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(Ue,[])}catch(Ie){var De=Ie}Reflect.construct(c,[],Ue)}else{try{Ue.call()}catch(Ie){De=Ie}c.call(Ue.prototype)}}else{try{throw Error()}catch(Ie){De=Ie}(Ue=c())&&typeof Ue.catch=="function"&&Ue.catch(function(){})}}catch(Ie){if(Ie&&De&&typeof Ie.stack=="string")return[Ie.stack,De.stack]}return[null,null]}};v.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var A=Object.getOwnPropertyDescriptor(v.DetermineComponentFrameRoot,"name");A&&A.configurable&&Object.defineProperty(v.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var D=v.DetermineComponentFrameRoot(),V=D[0],Q=D[1];if(V&&Q){var de=V.split(`
`),_e=Q.split(`
`);for(A=v=0;v<de.length&&!de[v].includes("DetermineComponentFrameRoot");)v++;for(;A<_e.length&&!_e[A].includes("DetermineComponentFrameRoot");)A++;if(v===de.length||A===_e.length)for(v=de.length-1,A=_e.length-1;1<=v&&0<=A&&de[v]!==_e[A];)A--;for(;1<=v&&0<=A;v--,A--)if(de[v]!==_e[A]){if(v!==1||A!==1)do if(v--,A--,0>A||de[v]!==_e[A]){var ze=`
`+de[v].replace(" at new "," at ");return c.displayName&&ze.includes("<anonymous>")&&(ze=ze.replace("<anonymous>",c.displayName)),ze}while(1<=v&&0<=A);break}}}finally{Ne=!1,Error.prepareStackTrace=g}return(g=c?c.displayName||c.name:"")?Te(g):""}function Ze(c,h){switch(c.tag){case 26:case 27:case 5:return Te(c.type);case 16:return Te("Lazy");case 13:return c.child!==h&&h!==null?Te("Suspense Fallback"):Te("Suspense");case 19:return Te("SuspenseList");case 0:case 15:return he(c.type,!1);case 11:return he(c.type.render,!1);case 1:return he(c.type,!0);case 31:return Te("Activity");default:return""}}function Et(c){try{var h="",g=null;do h+=Ze(c,g),g=c,c=c.return;while(c);return h}catch(v){return`
Error generating stack: `+v.message+`
`+v.stack}}var qe=Object.prototype.hasOwnProperty,Ye=e.unstable_scheduleCallback,we=e.unstable_cancelCallback,Qt=e.unstable_shouldYield,or=e.unstable_requestPaint,Pe=e.unstable_now,rt=e.unstable_getCurrentPriorityLevel,ge=e.unstable_ImmediatePriority,Re=e.unstable_UserBlockingPriority,We=e.unstable_NormalPriority,ht=e.unstable_LowPriority,xt=e.unstable_IdlePriority,sr=e.log,ar=e.unstable_setDisableYieldValue,fr=null,Mt=null;function lr(c){if(typeof sr=="function"&&ar(c),Mt&&typeof Mt.setStrictMode=="function")try{Mt.setStrictMode(fr,c)}catch{}}var qt=Math.clz32?Math.clz32:dn,zr=Math.log,nn=Math.LN2;function dn(c){return c>>>=0,c===0?32:31-(zr(c)/nn|0)|0}var Ce=256,st=262144,ft=4194304;function vt(c){var h=c&42;if(h!==0)return h;switch(c&-c){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:return c&261888;case 262144:case 524288:case 1048576:case 2097152:return c&3932160;case 4194304:case 8388608:case 16777216:case 33554432:return c&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return c}}function yr(c,h,g){var v=c.pendingLanes;if(v===0)return 0;var A=0,D=c.suspendedLanes,V=c.pingedLanes;c=c.warmLanes;var Q=v&134217727;return Q!==0?(v=Q&~D,v!==0?A=vt(v):(V&=Q,V!==0?A=vt(V):g||(g=Q&~c,g!==0&&(A=vt(g))))):(Q=v&~D,Q!==0?A=vt(Q):V!==0?A=vt(V):g||(g=v&~c,g!==0&&(A=vt(g)))),A===0?0:h!==0&&h!==A&&(h&D)===0&&(D=A&-A,g=h&-h,D>=g||D===32&&(g&4194048)!==0)?h:A}function Qe(c,h){return(c.pendingLanes&~(c.suspendedLanes&~c.pingedLanes)&h)===0}function zt(c,h){switch(c){case 1:case 2:case 4:case 8:case 64:return h+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return h+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function Nr(){var c=ft;return ft<<=1,(ft&62914560)===0&&(ft=4194304),c}function yn(c){for(var h=[],g=0;31>g;g++)h.push(c);return h}function hn(c,h){c.pendingLanes|=h,h!==268435456&&(c.suspendedLanes=0,c.pingedLanes=0,c.warmLanes=0)}function Ea(c,h,g,v,A,D){var V=c.pendingLanes;c.pendingLanes=g,c.suspendedLanes=0,c.pingedLanes=0,c.warmLanes=0,c.expiredLanes&=g,c.entangledLanes&=g,c.errorRecoveryDisabledLanes&=g,c.shellSuspendCounter=0;var Q=c.entanglements,de=c.expirationTimes,_e=c.hiddenUpdates;for(g=V&~g;0<g;){var ze=31-qt(g),Ue=1<<ze;Q[ze]=0,de[ze]=-1;var De=_e[ze];if(De!==null)for(_e[ze]=null,ze=0;ze<De.length;ze++){var Ie=De[ze];Ie!==null&&(Ie.lane&=-536870913)}g&=~Ue}v!==0&&xa(c,v,0),D!==0&&A===0&&c.tag!==0&&(c.suspendedLanes|=D&~(V&~h))}function xa(c,h,g){c.pendingLanes|=h,c.suspendedLanes&=~h;var v=31-qt(h);c.entangledLanes|=h,c.entanglements[v]=c.entanglements[v]|1073741824|g&261930}function Vn(c,h){var g=c.entangledLanes|=h;for(c=c.entanglements;g;){var v=31-qt(g),A=1<<v;A&h|c[v]&h&&(c[v]|=h),g&=~A}}function kn(c,h){var g=h&-h;return g=(g&42)!==0?1:Nl(g),(g&(c.suspendedLanes|h))!==0?0:g}function Nl(c){switch(c){case 2:c=1;break;case 8:c=4;break;case 32:c=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:c=128;break;case 268435456:c=134217728;break;default:c=0}return c}function fd(c){return c&=-c,2<c?8<c?(c&134217727)!==0?32:268435456:8:2}function Ip(){var c=H.p;return c!==0?c:(c=window.event,c===void 0?32:e9(c.type))}function md(c,h){var g=H.p;try{return H.p=c,h()}finally{H.p=g}}var mo=Math.random().toString(36).slice(2),Xr="__reactFiber$"+mo,ca="__reactProps$"+mo,ws="__reactContainer$"+mo,gd="__reactEvents$"+mo,Xg="__reactListeners$"+mo,sS="__reactHandles$"+mo,Bt="__reactResources$"+mo,be="__reactMarker$"+mo;function at(c){delete c[Xr],delete c[ca],delete c[gd],delete c[Xg],delete c[sS]}function wt(c){var h=c[Xr];if(h)return h;for(var g=c.parentNode;g;){if(h=g[ws]||g[Xr]){if(g=h.alternate,h.child!==null||g!==null&&g.child!==null)for(c=PO(c);c!==null;){if(g=c[Xr])return g;c=PO(c)}return h}c=g,g=c.parentNode}return null}function Hr(c){if(c=c[Xr]||c[ws]){var h=c.tag;if(h===5||h===6||h===13||h===31||h===26||h===27||h===3)return c}return null}function Cn(c){var h=c.tag;if(h===5||h===26||h===27||h===6)return c.stateNode;throw Error(n(33))}function pn(c){var h=c[Bt];return h||(h=c[Bt]={hoistableStyles:new Map,hoistableScripts:new Map}),h}function Mr(c){c[be]=!0}var Ei=new Set,Ll={};function vn(c,h){ks(c,h),ks(c+"Capture",h)}function ks(c,h){for(Ll[c]=h,c=0;c<h.length;c++)Ei.add(h[c])}var Zg=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),SL={},EL={};function Gce(c){return qe.call(EL,c)?!0:qe.call(SL,c)?!1:Zg.test(c)?EL[c]=!0:(SL[c]=!0,!1)}function Qg(c,h,g){if(Gce(h))if(g===null)c.removeAttribute(h);else{switch(typeof g){case"undefined":case"function":case"symbol":c.removeAttribute(h);return;case"boolean":var v=h.toLowerCase().slice(0,5);if(v!=="data-"&&v!=="aria-"){c.removeAttribute(h);return}}c.setAttribute(h,""+g)}}function e0(c,h,g){if(g===null)c.removeAttribute(h);else{switch(typeof g){case"undefined":case"function":case"symbol":case"boolean":c.removeAttribute(h);return}c.setAttribute(h,""+g)}}function Cs(c,h,g,v){if(v===null)c.removeAttribute(g);else{switch(typeof v){case"undefined":case"function":case"symbol":case"boolean":c.removeAttribute(g);return}c.setAttributeNS(h,g,""+v)}}function Gi(c){switch(typeof c){case"bigint":case"boolean":case"number":case"string":case"undefined":return c;case"object":return c;default:return""}}function xL(c){var h=c.type;return(c=c.nodeName)&&c.toLowerCase()==="input"&&(h==="checkbox"||h==="radio")}function jce(c,h,g){var v=Object.getOwnPropertyDescriptor(c.constructor.prototype,h);if(!c.hasOwnProperty(h)&&typeof v<"u"&&typeof v.get=="function"&&typeof v.set=="function"){var A=v.get,D=v.set;return Object.defineProperty(c,h,{configurable:!0,get:function(){return A.call(this)},set:function(V){g=""+V,D.call(this,V)}}),Object.defineProperty(c,h,{enumerable:v.enumerable}),{getValue:function(){return g},setValue:function(V){g=""+V},stopTracking:function(){c._valueTracker=null,delete c[h]}}}}function lS(c){if(!c._valueTracker){var h=xL(c)?"checked":"value";c._valueTracker=jce(c,h,""+c[h])}}function wL(c){if(!c)return!1;var h=c._valueTracker;if(!h)return!0;var g=h.getValue(),v="";return c&&(v=xL(c)?c.checked?"true":"false":c.value),c=v,c!==g?(h.setValue(c),!0):!1}function t0(c){if(c=c||(typeof document<"u"?document:void 0),typeof c>"u")return null;try{return c.activeElement||c.body}catch{return c.body}}var qce=/[\n"\\]/g;function ji(c){return c.replace(qce,function(h){return"\\"+h.charCodeAt(0).toString(16)+" "})}function uS(c,h,g,v,A,D,V,Q){c.name="",V!=null&&typeof V!="function"&&typeof V!="symbol"&&typeof V!="boolean"?c.type=V:c.removeAttribute("type"),h!=null?V==="number"?(h===0&&c.value===""||c.value!=h)&&(c.value=""+Gi(h)):c.value!==""+Gi(h)&&(c.value=""+Gi(h)):V!=="submit"&&V!=="reset"||c.removeAttribute("value"),h!=null?cS(c,V,Gi(h)):g!=null?cS(c,V,Gi(g)):v!=null&&c.removeAttribute("value"),A==null&&D!=null&&(c.defaultChecked=!!D),A!=null&&(c.checked=A&&typeof A!="function"&&typeof A!="symbol"),Q!=null&&typeof Q!="function"&&typeof Q!="symbol"&&typeof Q!="boolean"?c.name=""+Gi(Q):c.removeAttribute("name")}function kL(c,h,g,v,A,D,V,Q){if(D!=null&&typeof D!="function"&&typeof D!="symbol"&&typeof D!="boolean"&&(c.type=D),h!=null||g!=null){if(!(D!=="submit"&&D!=="reset"||h!=null)){lS(c);return}g=g!=null?""+Gi(g):"",h=h!=null?""+Gi(h):g,Q||h===c.value||(c.value=h),c.defaultValue=h}v=v??A,v=typeof v!="function"&&typeof v!="symbol"&&!!v,c.checked=Q?c.checked:!!v,c.defaultChecked=!!v,V!=null&&typeof V!="function"&&typeof V!="symbol"&&typeof V!="boolean"&&(c.name=V),lS(c)}function cS(c,h,g){h==="number"&&t0(c.ownerDocument)===c||c.defaultValue===""+g||(c.defaultValue=""+g)}function bd(c,h,g,v){if(c=c.options,h){h={};for(var A=0;A<g.length;A++)h["$"+g[A]]=!0;for(g=0;g<c.length;g++)A=h.hasOwnProperty("$"+c[g].value),c[g].selected!==A&&(c[g].selected=A),A&&v&&(c[g].defaultSelected=!0)}else{for(g=""+Gi(g),h=null,A=0;A<c.length;A++){if(c[A].value===g){c[A].selected=!0,v&&(c[A].defaultSelected=!0);return}h!==null||c[A].disabled||(h=c[A])}h!==null&&(h.selected=!0)}}function CL(c,h,g){if(h!=null&&(h=""+Gi(h),h!==c.value&&(c.value=h),g==null)){c.defaultValue!==h&&(c.defaultValue=h);return}c.defaultValue=g!=null?""+Gi(g):""}function TL(c,h,g,v){if(h==null){if(v!=null){if(g!=null)throw Error(n(92));if(X(v)){if(1<v.length)throw Error(n(93));v=v[0]}g=v}g==null&&(g=""),h=g}g=Gi(h),c.defaultValue=g,v=c.textContent,v===g&&v!==""&&v!==null&&(c.value=v),lS(c)}function yd(c,h){if(h){var g=c.firstChild;if(g&&g===c.lastChild&&g.nodeType===3){g.nodeValue=h;return}}c.textContent=h}var Wce=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function AL(c,h,g){var v=h.indexOf("--")===0;g==null||typeof g=="boolean"||g===""?v?c.setProperty(h,""):h==="float"?c.cssFloat="":c[h]="":v?c.setProperty(h,g):typeof g!="number"||g===0||Wce.has(h)?h==="float"?c.cssFloat=g:c[h]=(""+g).trim():c[h]=g+"px"}function FL(c,h,g){if(h!=null&&typeof h!="object")throw Error(n(62));if(c=c.style,g!=null){for(var v in g)!g.hasOwnProperty(v)||h!=null&&h.hasOwnProperty(v)||(v.indexOf("--")===0?c.setProperty(v,""):v==="float"?c.cssFloat="":c[v]="");for(var A in h)v=h[A],h.hasOwnProperty(A)&&g[A]!==v&&AL(c,A,v)}else for(var D in h)h.hasOwnProperty(D)&&AL(c,D,h[D])}function dS(c){if(c.indexOf("-")===-1)return!1;switch(c){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var Vce=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),Kce=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function r0(c){return Kce.test(""+c)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":c}function Ts(){}var hS=null;function pS(c){return c=c.target||c.srcElement||window,c.correspondingUseElement&&(c=c.correspondingUseElement),c.nodeType===3?c.parentNode:c}var vd=null,Sd=null;function _L(c){var h=Hr(c);if(h&&(c=h.stateNode)){var g=c[ca]||null;e:switch(c=h.stateNode,h.type){case"input":if(uS(c,g.value,g.defaultValue,g.defaultValue,g.checked,g.defaultChecked,g.type,g.name),h=g.name,g.type==="radio"&&h!=null){for(g=c;g.parentNode;)g=g.parentNode;for(g=g.querySelectorAll('input[name="'+ji(""+h)+'"][type="radio"]'),h=0;h<g.length;h++){var v=g[h];if(v!==c&&v.form===c.form){var A=v[ca]||null;if(!A)throw Error(n(90));uS(v,A.value,A.defaultValue,A.defaultValue,A.checked,A.defaultChecked,A.type,A.name)}}for(h=0;h<g.length;h++)v=g[h],v.form===c.form&&wL(v)}break e;case"textarea":CL(c,g.value,g.defaultValue);break e;case"select":h=g.value,h!=null&&bd(c,!!g.multiple,h,!1)}}}var fS=!1;function RL(c,h,g){if(fS)return c(h,g);fS=!0;try{var v=c(h);return v}finally{if(fS=!1,(vd!==null||Sd!==null)&&(G0(),vd&&(h=vd,c=Sd,Sd=vd=null,_L(h),c)))for(h=0;h<c.length;h++)_L(c[h])}}function Op(c,h){var g=c.stateNode;if(g===null)return null;var v=g[ca]||null;if(v===null)return null;g=v[h];e:switch(h){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(v=!v.disabled)||(c=c.type,v=!(c==="button"||c==="input"||c==="select"||c==="textarea")),c=!v;break e;default:c=!1}if(c)return null;if(g&&typeof g!="function")throw Error(n(231,h,typeof g));return g}var As=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),mS=!1;if(As)try{var Bp={};Object.defineProperty(Bp,"passive",{get:function(){mS=!0}}),window.addEventListener("test",Bp,Bp),window.removeEventListener("test",Bp,Bp)}catch{mS=!1}var Ml=null,gS=null,n0=null;function DL(){if(n0)return n0;var c,h=gS,g=h.length,v,A="value"in Ml?Ml.value:Ml.textContent,D=A.length;for(c=0;c<g&&h[c]===A[c];c++);var V=g-c;for(v=1;v<=V&&h[g-v]===A[D-v];v++);return n0=A.slice(c,1<v?1-v:void 0)}function a0(c){var h=c.keyCode;return"charCode"in c?(c=c.charCode,c===0&&h===13&&(c=13)):c=h,c===10&&(c=13),32<=c||c===13?c:0}function i0(){return!0}function NL(){return!1}function Za(c){function h(g,v,A,D,V){this._reactName=g,this._targetInst=A,this.type=v,this.nativeEvent=D,this.target=V,this.currentTarget=null;for(var Q in c)c.hasOwnProperty(Q)&&(g=c[Q],this[Q]=g?g(D):D[Q]);return this.isDefaultPrevented=(D.defaultPrevented!=null?D.defaultPrevented:D.returnValue===!1)?i0:NL,this.isPropagationStopped=NL,this}return p(h.prototype,{preventDefault:function(){this.defaultPrevented=!0;var g=this.nativeEvent;g&&(g.preventDefault?g.preventDefault():typeof g.returnValue!="unknown"&&(g.returnValue=!1),this.isDefaultPrevented=i0)},stopPropagation:function(){var g=this.nativeEvent;g&&(g.stopPropagation?g.stopPropagation():typeof g.cancelBubble!="unknown"&&(g.cancelBubble=!0),this.isPropagationStopped=i0)},persist:function(){},isPersistent:i0}),h}var Wu={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(c){return c.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},o0=Za(Wu),Pp=p({},Wu,{view:0,detail:0}),Yce=Za(Pp),bS,yS,zp,s0=p({},Pp,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:SS,button:0,buttons:0,relatedTarget:function(c){return c.relatedTarget===void 0?c.fromElement===c.srcElement?c.toElement:c.fromElement:c.relatedTarget},movementX:function(c){return"movementX"in c?c.movementX:(c!==zp&&(zp&&c.type==="mousemove"?(bS=c.screenX-zp.screenX,yS=c.screenY-zp.screenY):yS=bS=0,zp=c),bS)},movementY:function(c){return"movementY"in c?c.movementY:yS}}),LL=Za(s0),Jce=p({},s0,{dataTransfer:0}),Xce=Za(Jce),Zce=p({},Pp,{relatedTarget:0}),vS=Za(Zce),Qce=p({},Wu,{animationName:0,elapsedTime:0,pseudoElement:0}),ede=Za(Qce),tde=p({},Wu,{clipboardData:function(c){return"clipboardData"in c?c.clipboardData:window.clipboardData}}),rde=Za(tde),nde=p({},Wu,{data:0}),ML=Za(nde),ade={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},ide={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},ode={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function sde(c){var h=this.nativeEvent;return h.getModifierState?h.getModifierState(c):(c=ode[c])?!!h[c]:!1}function SS(){return sde}var lde=p({},Pp,{key:function(c){if(c.key){var h=ade[c.key]||c.key;if(h!=="Unidentified")return h}return c.type==="keypress"?(c=a0(c),c===13?"Enter":String.fromCharCode(c)):c.type==="keydown"||c.type==="keyup"?ide[c.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:SS,charCode:function(c){return c.type==="keypress"?a0(c):0},keyCode:function(c){return c.type==="keydown"||c.type==="keyup"?c.keyCode:0},which:function(c){return c.type==="keypress"?a0(c):c.type==="keydown"||c.type==="keyup"?c.keyCode:0}}),ude=Za(lde),cde=p({},s0,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),IL=Za(cde),dde=p({},Pp,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:SS}),hde=Za(dde),pde=p({},Wu,{propertyName:0,elapsedTime:0,pseudoElement:0}),fde=Za(pde),mde=p({},s0,{deltaX:function(c){return"deltaX"in c?c.deltaX:"wheelDeltaX"in c?-c.wheelDeltaX:0},deltaY:function(c){return"deltaY"in c?c.deltaY:"wheelDeltaY"in c?-c.wheelDeltaY:"wheelDelta"in c?-c.wheelDelta:0},deltaZ:0,deltaMode:0}),gde=Za(mde),bde=p({},Wu,{newState:0,oldState:0}),yde=Za(bde),vde=[9,13,27,32],ES=As&&"CompositionEvent"in window,Hp=null;As&&"documentMode"in document&&(Hp=document.documentMode);var Sde=As&&"TextEvent"in window&&!Hp,OL=As&&(!ES||Hp&&8<Hp&&11>=Hp),BL=" ",PL=!1;function zL(c,h){switch(c){case"keyup":return vde.indexOf(h.keyCode)!==-1;case"keydown":return h.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function HL(c){return c=c.detail,typeof c=="object"&&"data"in c?c.data:null}var Ed=!1;function Ede(c,h){switch(c){case"compositionend":return HL(h);case"keypress":return h.which!==32?null:(PL=!0,BL);case"textInput":return c=h.data,c===BL&&PL?null:c;default:return null}}function xde(c,h){if(Ed)return c==="compositionend"||!ES&&zL(c,h)?(c=DL(),n0=gS=Ml=null,Ed=!1,c):null;switch(c){case"paste":return null;case"keypress":if(!(h.ctrlKey||h.altKey||h.metaKey)||h.ctrlKey&&h.altKey){if(h.char&&1<h.char.length)return h.char;if(h.which)return String.fromCharCode(h.which)}return null;case"compositionend":return OL&&h.locale!=="ko"?null:h.data;default:return null}}var wde={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function UL(c){var h=c&&c.nodeName&&c.nodeName.toLowerCase();return h==="input"?!!wde[c.type]:h==="textarea"}function $L(c,h,g,v){vd?Sd?Sd.push(v):Sd=[v]:vd=v,h=J0(h,"onChange"),0<h.length&&(g=new o0("onChange","change",null,g,v),c.push({event:g,listeners:h}))}var Up=null,$p=null;function kde(c){kO(c,0)}function l0(c){var h=Cn(c);if(wL(h))return c}function GL(c,h){if(c==="change")return h}var jL=!1;if(As){var xS;if(As){var wS="oninput"in document;if(!wS){var qL=document.createElement("div");qL.setAttribute("oninput","return;"),wS=typeof qL.oninput=="function"}xS=wS}else xS=!1;jL=xS&&(!document.documentMode||9<document.documentMode)}function WL(){Up&&(Up.detachEvent("onpropertychange",VL),$p=Up=null)}function VL(c){if(c.propertyName==="value"&&l0($p)){var h=[];$L(h,$p,c,pS(c)),RL(kde,h)}}function Cde(c,h,g){c==="focusin"?(WL(),Up=h,$p=g,Up.attachEvent("onpropertychange",VL)):c==="focusout"&&WL()}function Tde(c){if(c==="selectionchange"||c==="keyup"||c==="keydown")return l0($p)}function Ade(c,h){if(c==="click")return l0(h)}function Fde(c,h){if(c==="input"||c==="change")return l0(h)}function _de(c,h){return c===h&&(c!==0||1/c===1/h)||c!==c&&h!==h}var xi=typeof Object.is=="function"?Object.is:_de;function Gp(c,h){if(xi(c,h))return!0;if(typeof c!="object"||c===null||typeof h!="object"||h===null)return!1;var g=Object.keys(c),v=Object.keys(h);if(g.length!==v.length)return!1;for(v=0;v<g.length;v++){var A=g[v];if(!qe.call(h,A)||!xi(c[A],h[A]))return!1}return!0}function KL(c){for(;c&&c.firstChild;)c=c.firstChild;return c}function YL(c,h){var g=KL(c);c=0;for(var v;g;){if(g.nodeType===3){if(v=c+g.textContent.length,c<=h&&v>=h)return{node:g,offset:h-c};c=v}e:{for(;g;){if(g.nextSibling){g=g.nextSibling;break e}g=g.parentNode}g=void 0}g=KL(g)}}function JL(c,h){return c&&h?c===h?!0:c&&c.nodeType===3?!1:h&&h.nodeType===3?JL(c,h.parentNode):"contains"in c?c.contains(h):c.compareDocumentPosition?!!(c.compareDocumentPosition(h)&16):!1:!1}function XL(c){c=c!=null&&c.ownerDocument!=null&&c.ownerDocument.defaultView!=null?c.ownerDocument.defaultView:window;for(var h=t0(c.document);h instanceof c.HTMLIFrameElement;){try{var g=typeof h.contentWindow.location.href=="string"}catch{g=!1}if(g)c=h.contentWindow;else break;h=t0(c.document)}return h}function kS(c){var h=c&&c.nodeName&&c.nodeName.toLowerCase();return h&&(h==="input"&&(c.type==="text"||c.type==="search"||c.type==="tel"||c.type==="url"||c.type==="password")||h==="textarea"||c.contentEditable==="true")}var Rde=As&&"documentMode"in document&&11>=document.documentMode,xd=null,CS=null,jp=null,TS=!1;function ZL(c,h,g){var v=g.window===g?g.document:g.nodeType===9?g:g.ownerDocument;TS||xd==null||xd!==t0(v)||(v=xd,"selectionStart"in v&&kS(v)?v={start:v.selectionStart,end:v.selectionEnd}:(v=(v.ownerDocument&&v.ownerDocument.defaultView||window).getSelection(),v={anchorNode:v.anchorNode,anchorOffset:v.anchorOffset,focusNode:v.focusNode,focusOffset:v.focusOffset}),jp&&Gp(jp,v)||(jp=v,v=J0(CS,"onSelect"),0<v.length&&(h=new o0("onSelect","select",null,h,g),c.push({event:h,listeners:v}),h.target=xd)))}function Vu(c,h){var g={};return g[c.toLowerCase()]=h.toLowerCase(),g["Webkit"+c]="webkit"+h,g["Moz"+c]="moz"+h,g}var wd={animationend:Vu("Animation","AnimationEnd"),animationiteration:Vu("Animation","AnimationIteration"),animationstart:Vu("Animation","AnimationStart"),transitionrun:Vu("Transition","TransitionRun"),transitionstart:Vu("Transition","TransitionStart"),transitioncancel:Vu("Transition","TransitionCancel"),transitionend:Vu("Transition","TransitionEnd")},AS={},QL={};As&&(QL=document.createElement("div").style,"AnimationEvent"in window||(delete wd.animationend.animation,delete wd.animationiteration.animation,delete wd.animationstart.animation),"TransitionEvent"in window||delete wd.transitionend.transition);function Ku(c){if(AS[c])return AS[c];if(!wd[c])return c;var h=wd[c],g;for(g in h)if(h.hasOwnProperty(g)&&g in QL)return AS[c]=h[g];return c}var eM=Ku("animationend"),tM=Ku("animationiteration"),rM=Ku("animationstart"),Dde=Ku("transitionrun"),Nde=Ku("transitionstart"),Lde=Ku("transitioncancel"),nM=Ku("transitionend"),aM=new Map,FS="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");FS.push("scrollEnd");function go(c,h){aM.set(c,h),vn(h,[c])}var u0=typeof reportError=="function"?reportError:function(c){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var h=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof c=="object"&&c!==null&&typeof c.message=="string"?String(c.message):String(c),error:c});if(!window.dispatchEvent(h))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",c);return}console.error(c)},qi=[],kd=0,_S=0;function c0(){for(var c=kd,h=_S=kd=0;h<c;){var g=qi[h];qi[h++]=null;var v=qi[h];qi[h++]=null;var A=qi[h];qi[h++]=null;var D=qi[h];if(qi[h++]=null,v!==null&&A!==null){var V=v.pending;V===null?A.next=A:(A.next=V.next,V.next=A),v.pending=A}D!==0&&iM(g,A,D)}}function d0(c,h,g,v){qi[kd++]=c,qi[kd++]=h,qi[kd++]=g,qi[kd++]=v,_S|=v,c.lanes|=v,c=c.alternate,c!==null&&(c.lanes|=v)}function RS(c,h,g,v){return d0(c,h,g,v),h0(c)}function Yu(c,h){return d0(c,null,null,h),h0(c)}function iM(c,h,g){c.lanes|=g;var v=c.alternate;v!==null&&(v.lanes|=g);for(var A=!1,D=c.return;D!==null;)D.childLanes|=g,v=D.alternate,v!==null&&(v.childLanes|=g),D.tag===22&&(c=D.stateNode,c===null||c._visibility&1||(A=!0)),c=D,D=D.return;return c.tag===3?(D=c.stateNode,A&&h!==null&&(A=31-qt(g),c=D.hiddenUpdates,v=c[A],v===null?c[A]=[h]:v.push(h),h.lane=g|536870912),D):null}function h0(c){if(50<pf)throw pf=0,zE=null,Error(n(185));for(var h=c.return;h!==null;)c=h,h=c.return;return c.tag===3?c.stateNode:null}var Cd={};function Mde(c,h,g,v){this.tag=c,this.key=g,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=h,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=v,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function wi(c,h,g,v){return new Mde(c,h,g,v)}function DS(c){return c=c.prototype,!(!c||!c.isReactComponent)}function Fs(c,h){var g=c.alternate;return g===null?(g=wi(c.tag,h,c.key,c.mode),g.elementType=c.elementType,g.type=c.type,g.stateNode=c.stateNode,g.alternate=c,c.alternate=g):(g.pendingProps=h,g.type=c.type,g.flags=0,g.subtreeFlags=0,g.deletions=null),g.flags=c.flags&65011712,g.childLanes=c.childLanes,g.lanes=c.lanes,g.child=c.child,g.memoizedProps=c.memoizedProps,g.memoizedState=c.memoizedState,g.updateQueue=c.updateQueue,h=c.dependencies,g.dependencies=h===null?null:{lanes:h.lanes,firstContext:h.firstContext},g.sibling=c.sibling,g.index=c.index,g.ref=c.ref,g.refCleanup=c.refCleanup,g}function oM(c,h){c.flags&=65011714;var g=c.alternate;return g===null?(c.childLanes=0,c.lanes=h,c.child=null,c.subtreeFlags=0,c.memoizedProps=null,c.memoizedState=null,c.updateQueue=null,c.dependencies=null,c.stateNode=null):(c.childLanes=g.childLanes,c.lanes=g.lanes,c.child=g.child,c.subtreeFlags=0,c.deletions=null,c.memoizedProps=g.memoizedProps,c.memoizedState=g.memoizedState,c.updateQueue=g.updateQueue,c.type=g.type,h=g.dependencies,c.dependencies=h===null?null:{lanes:h.lanes,firstContext:h.firstContext}),c}function p0(c,h,g,v,A,D){var V=0;if(v=c,typeof c=="function")DS(c)&&(V=1);else if(typeof c=="string")V=zhe(c,g,ne.current)?26:c==="html"||c==="head"||c==="body"?27:5;else e:switch(c){case I:return c=wi(31,g,h,A),c.elementType=I,c.lanes=D,c;case y:return Ju(g.children,A,D,h);case S:V=8,A|=24;break;case w:return c=wi(12,g,h,A|2),c.elementType=w,c.lanes=D,c;case R:return c=wi(13,g,h,A),c.elementType=R,c.lanes=D,c;case C:return c=wi(19,g,h,A),c.elementType=C,c.lanes=D,c;default:if(typeof c=="object"&&c!==null)switch(c.$$typeof){case k:V=10;break e;case E:V=9;break e;case _:V=11;break e;case F:V=14;break e;case L:V=16,v=null;break e}V=29,g=Error(n(130,c===null?"null":typeof c,"")),v=null}return h=wi(V,g,h,A),h.elementType=c,h.type=v,h.lanes=D,h}function Ju(c,h,g,v){return c=wi(7,c,v,h),c.lanes=g,c}function NS(c,h,g){return c=wi(6,c,null,h),c.lanes=g,c}function sM(c){var h=wi(18,null,null,0);return h.stateNode=c,h}function LS(c,h,g){return h=wi(4,c.children!==null?c.children:[],c.key,h),h.lanes=g,h.stateNode={containerInfo:c.containerInfo,pendingChildren:null,implementation:c.implementation},h}var lM=new WeakMap;function Wi(c,h){if(typeof c=="object"&&c!==null){var g=lM.get(c);return g!==void 0?g:(h={value:c,source:h,stack:Et(h)},lM.set(c,h),h)}return{value:c,source:h,stack:Et(h)}}var Td=[],Ad=0,f0=null,qp=0,Vi=[],Ki=0,Il=null,$o=1,Go="";function _s(c,h){Td[Ad++]=qp,Td[Ad++]=f0,f0=c,qp=h}function uM(c,h,g){Vi[Ki++]=$o,Vi[Ki++]=Go,Vi[Ki++]=Il,Il=c;var v=$o;c=Go;var A=32-qt(v)-1;v&=~(1<<A),g+=1;var D=32-qt(h)+A;if(30<D){var V=A-A%5;D=(v&(1<<V)-1).toString(32),v>>=V,A-=V,$o=1<<32-qt(h)+A|g<<A|v,Go=D+c}else $o=1<<D|g<<A|v,Go=c}function MS(c){c.return!==null&&(_s(c,1),uM(c,1,0))}function IS(c){for(;c===f0;)f0=Td[--Ad],Td[Ad]=null,qp=Td[--Ad],Td[Ad]=null;for(;c===Il;)Il=Vi[--Ki],Vi[Ki]=null,Go=Vi[--Ki],Vi[Ki]=null,$o=Vi[--Ki],Vi[Ki]=null}function cM(c,h){Vi[Ki++]=$o,Vi[Ki++]=Go,Vi[Ki++]=Il,$o=h.id,Go=h.overflow,Il=c}var da=null,ln=null,Dr=!1,Ol=null,Yi=!1,OS=Error(n(519));function Bl(c){var h=Error(n(418,1<arguments.length&&arguments[1]!==void 0&&arguments[1]?"text":"HTML",""));throw Wp(Wi(h,c)),OS}function dM(c){var h=c.stateNode,g=c.type,v=c.memoizedProps;switch(h[Xr]=c,h[ca]=v,g){case"dialog":kr("cancel",h),kr("close",h);break;case"iframe":case"object":case"embed":kr("load",h);break;case"video":case"audio":for(g=0;g<mf.length;g++)kr(mf[g],h);break;case"source":kr("error",h);break;case"img":case"image":case"link":kr("error",h),kr("load",h);break;case"details":kr("toggle",h);break;case"input":kr("invalid",h),kL(h,v.value,v.defaultValue,v.checked,v.defaultChecked,v.type,v.name,!0);break;case"select":kr("invalid",h);break;case"textarea":kr("invalid",h),TL(h,v.value,v.defaultValue,v.children)}g=v.children,typeof g!="string"&&typeof g!="number"&&typeof g!="bigint"||h.textContent===""+g||v.suppressHydrationWarning===!0||FO(h.textContent,g)?(v.popover!=null&&(kr("beforetoggle",h),kr("toggle",h)),v.onScroll!=null&&kr("scroll",h),v.onScrollEnd!=null&&kr("scrollend",h),v.onClick!=null&&(h.onclick=Ts),h=!0):h=!1,h||Bl(c,!0)}function hM(c){for(da=c.return;da;)switch(da.tag){case 5:case 31:case 13:Yi=!1;return;case 27:case 3:Yi=!0;return;default:da=da.return}}function Fd(c){if(c!==da)return!1;if(!Dr)return hM(c),Dr=!0,!1;var h=c.tag,g;if((g=h!==3&&h!==27)&&((g=h===5)&&(g=c.type,g=!(g!=="form"&&g!=="button")||ex(c.type,c.memoizedProps)),g=!g),g&&ln&&Bl(c),hM(c),h===13){if(c=c.memoizedState,c=c!==null?c.dehydrated:null,!c)throw Error(n(317));ln=BO(c)}else if(h===31){if(c=c.memoizedState,c=c!==null?c.dehydrated:null,!c)throw Error(n(317));ln=BO(c)}else h===27?(h=ln,Xl(c.type)?(c=ix,ix=null,ln=c):ln=h):ln=da?Xi(c.stateNode.nextSibling):null;return!0}function Xu(){ln=da=null,Dr=!1}function BS(){var c=Ol;return c!==null&&(ri===null?ri=c:ri.push.apply(ri,c),Ol=null),c}function Wp(c){Ol===null?Ol=[c]:Ol.push(c)}var PS=K(null),Zu=null,Rs=null;function Pl(c,h,g){P(PS,h._currentValue),h._currentValue=g}function Ds(c){c._currentValue=PS.current,G(PS)}function zS(c,h,g){for(;c!==null;){var v=c.alternate;if((c.childLanes&h)!==h?(c.childLanes|=h,v!==null&&(v.childLanes|=h)):v!==null&&(v.childLanes&h)!==h&&(v.childLanes|=h),c===g)break;c=c.return}}function HS(c,h,g,v){var A=c.child;for(A!==null&&(A.return=c);A!==null;){var D=A.dependencies;if(D!==null){var V=A.child;D=D.firstContext;e:for(;D!==null;){var Q=D;D=A;for(var de=0;de<h.length;de++)if(Q.context===h[de]){D.lanes|=g,Q=D.alternate,Q!==null&&(Q.lanes|=g),zS(D.return,g,c),v||(V=null);break e}D=Q.next}}else if(A.tag===18){if(V=A.return,V===null)throw Error(n(341));V.lanes|=g,D=V.alternate,D!==null&&(D.lanes|=g),zS(V,g,c),V=null}else V=A.child;if(V!==null)V.return=A;else for(V=A;V!==null;){if(V===c){V=null;break}if(A=V.sibling,A!==null){A.return=V.return,V=A;break}V=V.return}A=V}}function _d(c,h,g,v){c=null;for(var A=h,D=!1;A!==null;){if(!D){if((A.flags&524288)!==0)D=!0;else if((A.flags&262144)!==0)break}if(A.tag===10){var V=A.alternate;if(V===null)throw Error(n(387));if(V=V.memoizedProps,V!==null){var Q=A.type;xi(A.pendingProps.value,V.value)||(c!==null?c.push(Q):c=[Q])}}else if(A===ae.current){if(V=A.alternate,V===null)throw Error(n(387));V.memoizedState.memoizedState!==A.memoizedState.memoizedState&&(c!==null?c.push(Sf):c=[Sf])}A=A.return}c!==null&&HS(h,c,g,v),h.flags|=262144}function m0(c){for(c=c.firstContext;c!==null;){if(!xi(c.context._currentValue,c.memoizedValue))return!0;c=c.next}return!1}function Qu(c){Zu=c,Rs=null,c=c.dependencies,c!==null&&(c.firstContext=null)}function ha(c){return pM(Zu,c)}function g0(c,h){return Zu===null&&Qu(c),pM(c,h)}function pM(c,h){var g=h._currentValue;if(h={context:h,memoizedValue:g,next:null},Rs===null){if(c===null)throw Error(n(308));Rs=h,c.dependencies={lanes:0,firstContext:h},c.flags|=524288}else Rs=Rs.next=h;return g}var Ide=typeof AbortController<"u"?AbortController:function(){var c=[],h=this.signal={aborted:!1,addEventListener:function(g,v){c.push(v)}};this.abort=function(){h.aborted=!0,c.forEach(function(g){return g()})}},Ode=e.unstable_scheduleCallback,Bde=e.unstable_NormalPriority,On={$$typeof:k,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function US(){return{controller:new Ide,data:new Map,refCount:0}}function Vp(c){c.refCount--,c.refCount===0&&Ode(Bde,function(){c.controller.abort()})}var Kp=null,$S=0,Rd=0,Dd=null;function Pde(c,h){if(Kp===null){var g=Kp=[];$S=0,Rd=qE(),Dd={status:"pending",value:void 0,then:function(v){g.push(v)}}}return $S++,h.then(fM,fM),h}function fM(){if(--$S===0&&Kp!==null){Dd!==null&&(Dd.status="fulfilled");var c=Kp;Kp=null,Rd=0,Dd=null;for(var h=0;h<c.length;h++)(0,c[h])()}}function zde(c,h){var g=[],v={status:"pending",value:null,reason:null,then:function(A){g.push(A)}};return c.then(function(){v.status="fulfilled",v.value=h;for(var A=0;A<g.length;A++)(0,g[A])(h)},function(A){for(v.status="rejected",v.reason=A,A=0;A<g.length;A++)(0,g[A])(void 0)}),v}var mM=z.S;z.S=function(c,h){ZI=Pe(),typeof h=="object"&&h!==null&&typeof h.then=="function"&&Pde(c,h),mM!==null&&mM(c,h)};var ec=K(null);function GS(){var c=ec.current;return c!==null?c:Zr.pooledCache}function b0(c,h){h===null?P(ec,ec.current):P(ec,h.pool)}function gM(){var c=GS();return c===null?null:{parent:On._currentValue,pool:c}}var Nd=Error(n(460)),jS=Error(n(474)),y0=Error(n(542)),v0={then:function(){}};function bM(c){return c=c.status,c==="fulfilled"||c==="rejected"}function yM(c,h,g){switch(g=c[g],g===void 0?c.push(h):g!==h&&(h.then(Ts,Ts),h=g),h.status){case"fulfilled":return h.value;case"rejected":throw c=h.reason,SM(c),c;default:if(typeof h.status=="string")h.then(Ts,Ts);else{if(c=Zr,c!==null&&100<c.shellSuspendCounter)throw Error(n(482));c=h,c.status="pending",c.then(function(v){if(h.status==="pending"){var A=h;A.status="fulfilled",A.value=v}},function(v){if(h.status==="pending"){var A=h;A.status="rejected",A.reason=v}})}switch(h.status){case"fulfilled":return h.value;case"rejected":throw c=h.reason,SM(c),c}throw rc=h,Nd}}function tc(c){try{var h=c._init;return h(c._payload)}catch(g){throw g!==null&&typeof g=="object"&&typeof g.then=="function"?(rc=g,Nd):g}}var rc=null;function vM(){if(rc===null)throw Error(n(459));var c=rc;return rc=null,c}function SM(c){if(c===Nd||c===y0)throw Error(n(483))}var Ld=null,Yp=0;function S0(c){var h=Yp;return Yp+=1,Ld===null&&(Ld=[]),yM(Ld,c,h)}function Jp(c,h){h=h.props.ref,c.ref=h!==void 0?h:null}function E0(c,h){throw h.$$typeof===f?Error(n(525)):(c=Object.prototype.toString.call(h),Error(n(31,c==="[object Object]"?"object with keys {"+Object.keys(h).join(", ")+"}":c)))}function EM(c){function h(ve,me){if(c){var Ae=ve.deletions;Ae===null?(ve.deletions=[me],ve.flags|=16):Ae.push(me)}}function g(ve,me){if(!c)return null;for(;me!==null;)h(ve,me),me=me.sibling;return null}function v(ve){for(var me=new Map;ve!==null;)ve.key!==null?me.set(ve.key,ve):me.set(ve.index,ve),ve=ve.sibling;return me}function A(ve,me){return ve=Fs(ve,me),ve.index=0,ve.sibling=null,ve}function D(ve,me,Ae){return ve.index=Ae,c?(Ae=ve.alternate,Ae!==null?(Ae=Ae.index,Ae<me?(ve.flags|=67108866,me):Ae):(ve.flags|=67108866,me)):(ve.flags|=1048576,me)}function V(ve){return c&&ve.alternate===null&&(ve.flags|=67108866),ve}function Q(ve,me,Ae,He){return me===null||me.tag!==6?(me=NS(Ae,ve.mode,He),me.return=ve,me):(me=A(me,Ae),me.return=ve,me)}function de(ve,me,Ae,He){var Ht=Ae.type;return Ht===y?ze(ve,me,Ae.props.children,He,Ae.key):me!==null&&(me.elementType===Ht||typeof Ht=="object"&&Ht!==null&&Ht.$$typeof===L&&tc(Ht)===me.type)?(me=A(me,Ae.props),Jp(me,Ae),me.return=ve,me):(me=p0(Ae.type,Ae.key,Ae.props,null,ve.mode,He),Jp(me,Ae),me.return=ve,me)}function _e(ve,me,Ae,He){return me===null||me.tag!==4||me.stateNode.containerInfo!==Ae.containerInfo||me.stateNode.implementation!==Ae.implementation?(me=LS(Ae,ve.mode,He),me.return=ve,me):(me=A(me,Ae.children||[]),me.return=ve,me)}function ze(ve,me,Ae,He,Ht){return me===null||me.tag!==7?(me=Ju(Ae,ve.mode,He,Ht),me.return=ve,me):(me=A(me,Ae),me.return=ve,me)}function Ue(ve,me,Ae){if(typeof me=="string"&&me!==""||typeof me=="number"||typeof me=="bigint")return me=NS(""+me,ve.mode,Ae),me.return=ve,me;if(typeof me=="object"&&me!==null){switch(me.$$typeof){case m:return Ae=p0(me.type,me.key,me.props,null,ve.mode,Ae),Jp(Ae,me),Ae.return=ve,Ae;case b:return me=LS(me,ve.mode,Ae),me.return=ve,me;case L:return me=tc(me),Ue(ve,me,Ae)}if(X(me)||Y(me))return me=Ju(me,ve.mode,Ae,null),me.return=ve,me;if(typeof me.then=="function")return Ue(ve,S0(me),Ae);if(me.$$typeof===k)return Ue(ve,g0(ve,me),Ae);E0(ve,me)}return null}function De(ve,me,Ae,He){var Ht=me!==null?me.key:null;if(typeof Ae=="string"&&Ae!==""||typeof Ae=="number"||typeof Ae=="bigint")return Ht!==null?null:Q(ve,me,""+Ae,He);if(typeof Ae=="object"&&Ae!==null){switch(Ae.$$typeof){case m:return Ae.key===Ht?de(ve,me,Ae,He):null;case b:return Ae.key===Ht?_e(ve,me,Ae,He):null;case L:return Ae=tc(Ae),De(ve,me,Ae,He)}if(X(Ae)||Y(Ae))return Ht!==null?null:ze(ve,me,Ae,He,null);if(typeof Ae.then=="function")return De(ve,me,S0(Ae),He);if(Ae.$$typeof===k)return De(ve,me,g0(ve,Ae),He);E0(ve,Ae)}return null}function Ie(ve,me,Ae,He,Ht){if(typeof He=="string"&&He!==""||typeof He=="number"||typeof He=="bigint")return ve=ve.get(Ae)||null,Q(me,ve,""+He,Ht);if(typeof He=="object"&&He!==null){switch(He.$$typeof){case m:return ve=ve.get(He.key===null?Ae:He.key)||null,de(me,ve,He,Ht);case b:return ve=ve.get(He.key===null?Ae:He.key)||null,_e(me,ve,He,Ht);case L:return He=tc(He),Ie(ve,me,Ae,He,Ht)}if(X(He)||Y(He))return ve=ve.get(Ae)||null,ze(me,ve,He,Ht,null);if(typeof He.then=="function")return Ie(ve,me,Ae,S0(He),Ht);if(He.$$typeof===k)return Ie(ve,me,Ae,g0(me,He),Ht);E0(me,He)}return null}function kt(ve,me,Ae,He){for(var Ht=null,Or=null,It=me,mr=me=0,_r=null;It!==null&&mr<Ae.length;mr++){It.index>mr?(_r=It,It=null):_r=It.sibling;var Br=De(ve,It,Ae[mr],He);if(Br===null){It===null&&(It=_r);break}c&&It&&Br.alternate===null&&h(ve,It),me=D(Br,me,mr),Or===null?Ht=Br:Or.sibling=Br,Or=Br,It=_r}if(mr===Ae.length)return g(ve,It),Dr&&_s(ve,mr),Ht;if(It===null){for(;mr<Ae.length;mr++)It=Ue(ve,Ae[mr],He),It!==null&&(me=D(It,me,mr),Or===null?Ht=It:Or.sibling=It,Or=It);return Dr&&_s(ve,mr),Ht}for(It=v(It);mr<Ae.length;mr++)_r=Ie(It,ve,mr,Ae[mr],He),_r!==null&&(c&&_r.alternate!==null&&It.delete(_r.key===null?mr:_r.key),me=D(_r,me,mr),Or===null?Ht=_r:Or.sibling=_r,Or=_r);return c&&It.forEach(function(ru){return h(ve,ru)}),Dr&&_s(ve,mr),Ht}function Yt(ve,me,Ae,He){if(Ae==null)throw Error(n(151));for(var Ht=null,Or=null,It=me,mr=me=0,_r=null,Br=Ae.next();It!==null&&!Br.done;mr++,Br=Ae.next()){It.index>mr?(_r=It,It=null):_r=It.sibling;var ru=De(ve,It,Br.value,He);if(ru===null){It===null&&(It=_r);break}c&&It&&ru.alternate===null&&h(ve,It),me=D(ru,me,mr),Or===null?Ht=ru:Or.sibling=ru,Or=ru,It=_r}if(Br.done)return g(ve,It),Dr&&_s(ve,mr),Ht;if(It===null){for(;!Br.done;mr++,Br=Ae.next())Br=Ue(ve,Br.value,He),Br!==null&&(me=D(Br,me,mr),Or===null?Ht=Br:Or.sibling=Br,Or=Br);return Dr&&_s(ve,mr),Ht}for(It=v(It);!Br.done;mr++,Br=Ae.next())Br=Ie(It,ve,mr,Br.value,He),Br!==null&&(c&&Br.alternate!==null&&It.delete(Br.key===null?mr:Br.key),me=D(Br,me,mr),Or===null?Ht=Br:Or.sibling=Br,Or=Br);return c&&It.forEach(function(Jhe){return h(ve,Jhe)}),Dr&&_s(ve,mr),Ht}function Yr(ve,me,Ae,He){if(typeof Ae=="object"&&Ae!==null&&Ae.type===y&&Ae.key===null&&(Ae=Ae.props.children),typeof Ae=="object"&&Ae!==null){switch(Ae.$$typeof){case m:e:{for(var Ht=Ae.key;me!==null;){if(me.key===Ht){if(Ht=Ae.type,Ht===y){if(me.tag===7){g(ve,me.sibling),He=A(me,Ae.props.children),He.return=ve,ve=He;break e}}else if(me.elementType===Ht||typeof Ht=="object"&&Ht!==null&&Ht.$$typeof===L&&tc(Ht)===me.type){g(ve,me.sibling),He=A(me,Ae.props),Jp(He,Ae),He.return=ve,ve=He;break e}g(ve,me);break}else h(ve,me);me=me.sibling}Ae.type===y?(He=Ju(Ae.props.children,ve.mode,He,Ae.key),He.return=ve,ve=He):(He=p0(Ae.type,Ae.key,Ae.props,null,ve.mode,He),Jp(He,Ae),He.return=ve,ve=He)}return V(ve);case b:e:{for(Ht=Ae.key;me!==null;){if(me.key===Ht)if(me.tag===4&&me.stateNode.containerInfo===Ae.containerInfo&&me.stateNode.implementation===Ae.implementation){g(ve,me.sibling),He=A(me,Ae.children||[]),He.return=ve,ve=He;break e}else{g(ve,me);break}else h(ve,me);me=me.sibling}He=LS(Ae,ve.mode,He),He.return=ve,ve=He}return V(ve);case L:return Ae=tc(Ae),Yr(ve,me,Ae,He)}if(X(Ae))return kt(ve,me,Ae,He);if(Y(Ae)){if(Ht=Y(Ae),typeof Ht!="function")throw Error(n(150));return Ae=Ht.call(Ae),Yt(ve,me,Ae,He)}if(typeof Ae.then=="function")return Yr(ve,me,S0(Ae),He);if(Ae.$$typeof===k)return Yr(ve,me,g0(ve,Ae),He);E0(ve,Ae)}return typeof Ae=="string"&&Ae!==""||typeof Ae=="number"||typeof Ae=="bigint"?(Ae=""+Ae,me!==null&&me.tag===6?(g(ve,me.sibling),He=A(me,Ae),He.return=ve,ve=He):(g(ve,me),He=NS(Ae,ve.mode,He),He.return=ve,ve=He),V(ve)):g(ve,me)}return function(ve,me,Ae,He){try{Yp=0;var Ht=Yr(ve,me,Ae,He);return Ld=null,Ht}catch(It){if(It===Nd||It===y0)throw It;var Or=wi(29,It,null,ve.mode);return Or.lanes=He,Or.return=ve,Or}finally{}}}var nc=EM(!0),xM=EM(!1),zl=!1;function qS(c){c.updateQueue={baseState:c.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function WS(c,h){c=c.updateQueue,h.updateQueue===c&&(h.updateQueue={baseState:c.baseState,firstBaseUpdate:c.firstBaseUpdate,lastBaseUpdate:c.lastBaseUpdate,shared:c.shared,callbacks:null})}function Hl(c){return{lane:c,tag:0,payload:null,callback:null,next:null}}function Ul(c,h,g){var v=c.updateQueue;if(v===null)return null;if(v=v.shared,(Ur&2)!==0){var A=v.pending;return A===null?h.next=h:(h.next=A.next,A.next=h),v.pending=h,h=h0(c),iM(c,null,g),h}return d0(c,v,h,g),h0(c)}function Xp(c,h,g){if(h=h.updateQueue,h!==null&&(h=h.shared,(g&4194048)!==0)){var v=h.lanes;v&=c.pendingLanes,g|=v,h.lanes=g,Vn(c,g)}}function VS(c,h){var g=c.updateQueue,v=c.alternate;if(v!==null&&(v=v.updateQueue,g===v)){var A=null,D=null;if(g=g.firstBaseUpdate,g!==null){do{var V={lane:g.lane,tag:g.tag,payload:g.payload,callback:null,next:null};D===null?A=D=V:D=D.next=V,g=g.next}while(g!==null);D===null?A=D=h:D=D.next=h}else A=D=h;g={baseState:v.baseState,firstBaseUpdate:A,lastBaseUpdate:D,shared:v.shared,callbacks:v.callbacks},c.updateQueue=g;return}c=g.lastBaseUpdate,c===null?g.firstBaseUpdate=h:c.next=h,g.lastBaseUpdate=h}var KS=!1;function Zp(){if(KS){var c=Dd;if(c!==null)throw c}}function Qp(c,h,g,v){KS=!1;var A=c.updateQueue;zl=!1;var D=A.firstBaseUpdate,V=A.lastBaseUpdate,Q=A.shared.pending;if(Q!==null){A.shared.pending=null;var de=Q,_e=de.next;de.next=null,V===null?D=_e:V.next=_e,V=de;var ze=c.alternate;ze!==null&&(ze=ze.updateQueue,Q=ze.lastBaseUpdate,Q!==V&&(Q===null?ze.firstBaseUpdate=_e:Q.next=_e,ze.lastBaseUpdate=de))}if(D!==null){var Ue=A.baseState;V=0,ze=_e=de=null,Q=D;do{var De=Q.lane&-536870913,Ie=De!==Q.lane;if(Ie?(Fr&De)===De:(v&De)===De){De!==0&&De===Rd&&(KS=!0),ze!==null&&(ze=ze.next={lane:0,tag:Q.tag,payload:Q.payload,callback:null,next:null});e:{var kt=c,Yt=Q;De=h;var Yr=g;switch(Yt.tag){case 1:if(kt=Yt.payload,typeof kt=="function"){Ue=kt.call(Yr,Ue,De);break e}Ue=kt;break e;case 3:kt.flags=kt.flags&-65537|128;case 0:if(kt=Yt.payload,De=typeof kt=="function"?kt.call(Yr,Ue,De):kt,De==null)break e;Ue=p({},Ue,De);break e;case 2:zl=!0}}De=Q.callback,De!==null&&(c.flags|=64,Ie&&(c.flags|=8192),Ie=A.callbacks,Ie===null?A.callbacks=[De]:Ie.push(De))}else Ie={lane:De,tag:Q.tag,payload:Q.payload,callback:Q.callback,next:null},ze===null?(_e=ze=Ie,de=Ue):ze=ze.next=Ie,V|=De;if(Q=Q.next,Q===null){if(Q=A.shared.pending,Q===null)break;Ie=Q,Q=Ie.next,Ie.next=null,A.lastBaseUpdate=Ie,A.shared.pending=null}}while(!0);ze===null&&(de=Ue),A.baseState=de,A.firstBaseUpdate=_e,A.lastBaseUpdate=ze,D===null&&(A.shared.lanes=0),Wl|=V,c.lanes=V,c.memoizedState=Ue}}function wM(c,h){if(typeof c!="function")throw Error(n(191,c));c.call(h)}function kM(c,h){var g=c.callbacks;if(g!==null)for(c.callbacks=null,c=0;c<g.length;c++)wM(g[c],h)}var Md=K(null),x0=K(0);function CM(c,h){c=Hs,P(x0,c),P(Md,h),Hs=c|h.baseLanes}function YS(){P(x0,Hs),P(Md,Md.current)}function JS(){Hs=x0.current,G(Md),G(x0)}var ki=K(null),Ji=null;function $l(c){var h=c.alternate;P(_n,_n.current&1),P(ki,c),Ji===null&&(h===null||Md.current!==null||h.memoizedState!==null)&&(Ji=c)}function XS(c){P(_n,_n.current),P(ki,c),Ji===null&&(Ji=c)}function TM(c){c.tag===22?(P(_n,_n.current),P(ki,c),Ji===null&&(Ji=c)):Gl()}function Gl(){P(_n,_n.current),P(ki,ki.current)}function Ci(c){G(ki),Ji===c&&(Ji=null),G(_n)}var _n=K(0);function w0(c){for(var h=c;h!==null;){if(h.tag===13){var g=h.memoizedState;if(g!==null&&(g=g.dehydrated,g===null||nx(g)||ax(g)))return h}else if(h.tag===19&&(h.memoizedProps.revealOrder==="forwards"||h.memoizedProps.revealOrder==="backwards"||h.memoizedProps.revealOrder==="unstable_legacy-backwards"||h.memoizedProps.revealOrder==="together")){if((h.flags&128)!==0)return h}else if(h.child!==null){h.child.return=h,h=h.child;continue}if(h===c)break;for(;h.sibling===null;){if(h.return===null||h.return===c)return null;h=h.return}h.sibling.return=h.return,h=h.sibling}return null}var Ns=0,dr=null,Vr=null,Bn=null,k0=!1,Id=!1,ac=!1,C0=0,ef=0,Od=null,Hde=0;function Tn(){throw Error(n(321))}function ZS(c,h){if(h===null)return!1;for(var g=0;g<h.length&&g<c.length;g++)if(!xi(c[g],h[g]))return!1;return!0}function QS(c,h,g,v,A,D){return Ns=D,dr=h,h.memoizedState=null,h.updateQueue=null,h.lanes=0,z.H=c===null||c.memoizedState===null?uI:fE,ac=!1,D=g(v,A),ac=!1,Id&&(D=FM(h,g,v,A)),AM(c),D}function AM(c){z.H=nf;var h=Vr!==null&&Vr.next!==null;if(Ns=0,Bn=Vr=dr=null,k0=!1,ef=0,Od=null,h)throw Error(n(300));c===null||Pn||(c=c.dependencies,c!==null&&m0(c)&&(Pn=!0))}function FM(c,h,g,v){dr=c;var A=0;do{if(Id&&(Od=null),ef=0,Id=!1,25<=A)throw Error(n(301));if(A+=1,Bn=Vr=null,c.updateQueue!=null){var D=c.updateQueue;D.lastEffect=null,D.events=null,D.stores=null,D.memoCache!=null&&(D.memoCache.index=0)}z.H=cI,D=h(g,v)}while(Id);return D}function Ude(){var c=z.H,h=c.useState()[0];return h=typeof h.then=="function"?tf(h):h,c=c.useState()[0],(Vr!==null?Vr.memoizedState:null)!==c&&(dr.flags|=1024),h}function eE(){var c=C0!==0;return C0=0,c}function tE(c,h,g){h.updateQueue=c.updateQueue,h.flags&=-2053,c.lanes&=~g}function rE(c){if(k0){for(c=c.memoizedState;c!==null;){var h=c.queue;h!==null&&(h.pending=null),c=c.next}k0=!1}Ns=0,Bn=Vr=dr=null,Id=!1,ef=C0=0,Od=null}function Ha(){var c={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Bn===null?dr.memoizedState=Bn=c:Bn=Bn.next=c,Bn}function Rn(){if(Vr===null){var c=dr.alternate;c=c!==null?c.memoizedState:null}else c=Vr.next;var h=Bn===null?dr.memoizedState:Bn.next;if(h!==null)Bn=h,Vr=c;else{if(c===null)throw dr.alternate===null?Error(n(467)):Error(n(310));Vr=c,c={memoizedState:Vr.memoizedState,baseState:Vr.baseState,baseQueue:Vr.baseQueue,queue:Vr.queue,next:null},Bn===null?dr.memoizedState=Bn=c:Bn=Bn.next=c}return Bn}function T0(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function tf(c){var h=ef;return ef+=1,Od===null&&(Od=[]),c=yM(Od,c,h),h=dr,(Bn===null?h.memoizedState:Bn.next)===null&&(h=h.alternate,z.H=h===null||h.memoizedState===null?uI:fE),c}function A0(c){if(c!==null&&typeof c=="object"){if(typeof c.then=="function")return tf(c);if(c.$$typeof===k)return ha(c)}throw Error(n(438,String(c)))}function nE(c){var h=null,g=dr.updateQueue;if(g!==null&&(h=g.memoCache),h==null){var v=dr.alternate;v!==null&&(v=v.updateQueue,v!==null&&(v=v.memoCache,v!=null&&(h={data:v.data.map(function(A){return A.slice()}),index:0})))}if(h==null&&(h={data:[],index:0}),g===null&&(g=T0(),dr.updateQueue=g),g.memoCache=h,g=h.data[h.index],g===void 0)for(g=h.data[h.index]=Array(c),v=0;v<c;v++)g[v]=M;return h.index++,g}function Ls(c,h){return typeof h=="function"?h(c):h}function F0(c){var h=Rn();return aE(h,Vr,c)}function aE(c,h,g){var v=c.queue;if(v===null)throw Error(n(311));v.lastRenderedReducer=g;var A=c.baseQueue,D=v.pending;if(D!==null){if(A!==null){var V=A.next;A.next=D.next,D.next=V}h.baseQueue=A=D,v.pending=null}if(D=c.baseState,A===null)c.memoizedState=D;else{h=A.next;var Q=V=null,de=null,_e=h,ze=!1;do{var Ue=_e.lane&-536870913;if(Ue!==_e.lane?(Fr&Ue)===Ue:(Ns&Ue)===Ue){var De=_e.revertLane;if(De===0)de!==null&&(de=de.next={lane:0,revertLane:0,gesture:null,action:_e.action,hasEagerState:_e.hasEagerState,eagerState:_e.eagerState,next:null}),Ue===Rd&&(ze=!0);else if((Ns&De)===De){_e=_e.next,De===Rd&&(ze=!0);continue}else Ue={lane:0,revertLane:_e.revertLane,gesture:null,action:_e.action,hasEagerState:_e.hasEagerState,eagerState:_e.eagerState,next:null},de===null?(Q=de=Ue,V=D):de=de.next=Ue,dr.lanes|=De,Wl|=De;Ue=_e.action,ac&&g(D,Ue),D=_e.hasEagerState?_e.eagerState:g(D,Ue)}else De={lane:Ue,revertLane:_e.revertLane,gesture:_e.gesture,action:_e.action,hasEagerState:_e.hasEagerState,eagerState:_e.eagerState,next:null},de===null?(Q=de=De,V=D):de=de.next=De,dr.lanes|=Ue,Wl|=Ue;_e=_e.next}while(_e!==null&&_e!==h);if(de===null?V=D:de.next=Q,!xi(D,c.memoizedState)&&(Pn=!0,ze&&(g=Dd,g!==null)))throw g;c.memoizedState=D,c.baseState=V,c.baseQueue=de,v.lastRenderedState=D}return A===null&&(v.lanes=0),[c.memoizedState,v.dispatch]}function iE(c){var h=Rn(),g=h.queue;if(g===null)throw Error(n(311));g.lastRenderedReducer=c;var v=g.dispatch,A=g.pending,D=h.memoizedState;if(A!==null){g.pending=null;var V=A=A.next;do D=c(D,V.action),V=V.next;while(V!==A);xi(D,h.memoizedState)||(Pn=!0),h.memoizedState=D,h.baseQueue===null&&(h.baseState=D),g.lastRenderedState=D}return[D,v]}function _M(c,h,g){var v=dr,A=Rn(),D=Dr;if(D){if(g===void 0)throw Error(n(407));g=g()}else g=h();var V=!xi((Vr||A).memoizedState,g);if(V&&(A.memoizedState=g,Pn=!0),A=A.queue,lE(NM.bind(null,v,A,c),[c]),A.getSnapshot!==h||V||Bn!==null&&Bn.memoizedState.tag&1){if(v.flags|=2048,Bd(9,{destroy:void 0},DM.bind(null,v,A,g,h),null),Zr===null)throw Error(n(349));D||(Ns&127)!==0||RM(v,h,g)}return g}function RM(c,h,g){c.flags|=16384,c={getSnapshot:h,value:g},h=dr.updateQueue,h===null?(h=T0(),dr.updateQueue=h,h.stores=[c]):(g=h.stores,g===null?h.stores=[c]:g.push(c))}function DM(c,h,g,v){h.value=g,h.getSnapshot=v,LM(h)&&MM(c)}function NM(c,h,g){return g(function(){LM(h)&&MM(c)})}function LM(c){var h=c.getSnapshot;c=c.value;try{var g=h();return!xi(c,g)}catch{return!0}}function MM(c){var h=Yu(c,2);h!==null&&ni(h,c,2)}function oE(c){var h=Ha();if(typeof c=="function"){var g=c;if(c=g(),ac){lr(!0);try{g()}finally{lr(!1)}}}return h.memoizedState=h.baseState=c,h.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ls,lastRenderedState:c},h}function IM(c,h,g,v){return c.baseState=g,aE(c,Vr,typeof v=="function"?v:Ls)}function $de(c,h,g,v,A){if(D0(c))throw Error(n(485));if(c=h.action,c!==null){var D={payload:A,action:c,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(V){D.listeners.push(V)}};z.T!==null?g(!0):D.isTransition=!1,v(D),g=h.pending,g===null?(D.next=h.pending=D,OM(h,D)):(D.next=g.next,h.pending=g.next=D)}}function OM(c,h){var g=h.action,v=h.payload,A=c.state;if(h.isTransition){var D=z.T,V={};z.T=V;try{var Q=g(A,v),de=z.S;de!==null&&de(V,Q),BM(c,h,Q)}catch(_e){sE(c,h,_e)}finally{D!==null&&V.types!==null&&(D.types=V.types),z.T=D}}else try{D=g(A,v),BM(c,h,D)}catch(_e){sE(c,h,_e)}}function BM(c,h,g){g!==null&&typeof g=="object"&&typeof g.then=="function"?g.then(function(v){PM(c,h,v)},function(v){return sE(c,h,v)}):PM(c,h,g)}function PM(c,h,g){h.status="fulfilled",h.value=g,zM(h),c.state=g,h=c.pending,h!==null&&(g=h.next,g===h?c.pending=null:(g=g.next,h.next=g,OM(c,g)))}function sE(c,h,g){var v=c.pending;if(c.pending=null,v!==null){v=v.next;do h.status="rejected",h.reason=g,zM(h),h=h.next;while(h!==v)}c.action=null}function zM(c){c=c.listeners;for(var h=0;h<c.length;h++)(0,c[h])()}function HM(c,h){return h}function UM(c,h){if(Dr){var g=Zr.formState;if(g!==null){e:{var v=dr;if(Dr){if(ln){t:{for(var A=ln,D=Yi;A.nodeType!==8;){if(!D){A=null;break t}if(A=Xi(A.nextSibling),A===null){A=null;break t}}D=A.data,A=D==="F!"||D==="F"?A:null}if(A){ln=Xi(A.nextSibling),v=A.data==="F!";break e}}Bl(v)}v=!1}v&&(h=g[0])}}return g=Ha(),g.memoizedState=g.baseState=h,v={pending:null,lanes:0,dispatch:null,lastRenderedReducer:HM,lastRenderedState:h},g.queue=v,g=oI.bind(null,dr,v),v.dispatch=g,v=oE(!1),D=pE.bind(null,dr,!1,v.queue),v=Ha(),A={state:h,dispatch:null,action:c,pending:null},v.queue=A,g=$de.bind(null,dr,A,D,g),A.dispatch=g,v.memoizedState=c,[h,g,!1]}function $M(c){var h=Rn();return GM(h,Vr,c)}function GM(c,h,g){if(h=aE(c,h,HM)[0],c=F0(Ls)[0],typeof h=="object"&&h!==null&&typeof h.then=="function")try{var v=tf(h)}catch(V){throw V===Nd?y0:V}else v=h;h=Rn();var A=h.queue,D=A.dispatch;return g!==h.memoizedState&&(dr.flags|=2048,Bd(9,{destroy:void 0},Gde.bind(null,A,g),null)),[v,D,c]}function Gde(c,h){c.action=h}function jM(c){var h=Rn(),g=Vr;if(g!==null)return GM(h,g,c);Rn(),h=h.memoizedState,g=Rn();var v=g.queue.dispatch;return g.memoizedState=c,[h,v,!1]}function Bd(c,h,g,v){return c={tag:c,create:g,deps:v,inst:h,next:null},h=dr.updateQueue,h===null&&(h=T0(),dr.updateQueue=h),g=h.lastEffect,g===null?h.lastEffect=c.next=c:(v=g.next,g.next=c,c.next=v,h.lastEffect=c),c}function qM(){return Rn().memoizedState}function _0(c,h,g,v){var A=Ha();dr.flags|=c,A.memoizedState=Bd(1|h,{destroy:void 0},g,v===void 0?null:v)}function R0(c,h,g,v){var A=Rn();v=v===void 0?null:v;var D=A.memoizedState.inst;Vr!==null&&v!==null&&ZS(v,Vr.memoizedState.deps)?A.memoizedState=Bd(h,D,g,v):(dr.flags|=c,A.memoizedState=Bd(1|h,D,g,v))}function WM(c,h){_0(8390656,8,c,h)}function lE(c,h){R0(2048,8,c,h)}function jde(c){dr.flags|=4;var h=dr.updateQueue;if(h===null)h=T0(),dr.updateQueue=h,h.events=[c];else{var g=h.events;g===null?h.events=[c]:g.push(c)}}function VM(c){var h=Rn().memoizedState;return jde({ref:h,nextImpl:c}),function(){if((Ur&2)!==0)throw Error(n(440));return h.impl.apply(void 0,arguments)}}function KM(c,h){return R0(4,2,c,h)}function YM(c,h){return R0(4,4,c,h)}function JM(c,h){if(typeof h=="function"){c=c();var g=h(c);return function(){typeof g=="function"?g():h(null)}}if(h!=null)return c=c(),h.current=c,function(){h.current=null}}function XM(c,h,g){g=g!=null?g.concat([c]):null,R0(4,4,JM.bind(null,h,c),g)}function uE(){}function ZM(c,h){var g=Rn();h=h===void 0?null:h;var v=g.memoizedState;return h!==null&&ZS(h,v[1])?v[0]:(g.memoizedState=[c,h],c)}function QM(c,h){var g=Rn();h=h===void 0?null:h;var v=g.memoizedState;if(h!==null&&ZS(h,v[1]))return v[0];if(v=c(),ac){lr(!0);try{c()}finally{lr(!1)}}return g.memoizedState=[v,h],v}function cE(c,h,g){return g===void 0||(Ns&1073741824)!==0&&(Fr&261930)===0?c.memoizedState=h:(c.memoizedState=g,c=eO(),dr.lanes|=c,Wl|=c,g)}function eI(c,h,g,v){return xi(g,h)?g:Md.current!==null?(c=cE(c,g,v),xi(c,h)||(Pn=!0),c):(Ns&42)===0||(Ns&1073741824)!==0&&(Fr&261930)===0?(Pn=!0,c.memoizedState=g):(c=eO(),dr.lanes|=c,Wl|=c,h)}function tI(c,h,g,v,A){var D=H.p;H.p=D!==0&&8>D?D:8;var V=z.T,Q={};z.T=Q,pE(c,!1,h,g);try{var de=A(),_e=z.S;if(_e!==null&&_e(Q,de),de!==null&&typeof de=="object"&&typeof de.then=="function"){var ze=zde(de,v);rf(c,h,ze,Fi(c))}else rf(c,h,v,Fi(c))}catch(Ue){rf(c,h,{then:function(){},status:"rejected",reason:Ue},Fi())}finally{H.p=D,V!==null&&Q.types!==null&&(V.types=Q.types),z.T=V}}function qde(){}function dE(c,h,g,v){if(c.tag!==5)throw Error(n(476));var A=rI(c).queue;tI(c,A,h,B,g===null?qde:function(){return nI(c),g(v)})}function rI(c){var h=c.memoizedState;if(h!==null)return h;h={memoizedState:B,baseState:B,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ls,lastRenderedState:B},next:null};var g={};return h.next={memoizedState:g,baseState:g,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Ls,lastRenderedState:g},next:null},c.memoizedState=h,c=c.alternate,c!==null&&(c.memoizedState=h),h}function nI(c){var h=rI(c);h.next===null&&(h=c.alternate.memoizedState),rf(c,h.next.queue,{},Fi())}function hE(){return ha(Sf)}function aI(){return Rn().memoizedState}function iI(){return Rn().memoizedState}function Wde(c){for(var h=c.return;h!==null;){switch(h.tag){case 24:case 3:var g=Fi();c=Hl(g);var v=Ul(h,c,g);v!==null&&(ni(v,h,g),Xp(v,h,g)),h={cache:US()},c.payload=h;return}h=h.return}}function Vde(c,h,g){var v=Fi();g={lane:v,revertLane:0,gesture:null,action:g,hasEagerState:!1,eagerState:null,next:null},D0(c)?sI(h,g):(g=RS(c,h,g,v),g!==null&&(ni(g,c,v),lI(g,h,v)))}function oI(c,h,g){var v=Fi();rf(c,h,g,v)}function rf(c,h,g,v){var A={lane:v,revertLane:0,gesture:null,action:g,hasEagerState:!1,eagerState:null,next:null};if(D0(c))sI(h,A);else{var D=c.alternate;if(c.lanes===0&&(D===null||D.lanes===0)&&(D=h.lastRenderedReducer,D!==null))try{var V=h.lastRenderedState,Q=D(V,g);if(A.hasEagerState=!0,A.eagerState=Q,xi(Q,V))return d0(c,h,A,0),Zr===null&&c0(),!1}catch{}finally{}if(g=RS(c,h,A,v),g!==null)return ni(g,c,v),lI(g,h,v),!0}return!1}function pE(c,h,g,v){if(v={lane:2,revertLane:qE(),gesture:null,action:v,hasEagerState:!1,eagerState:null,next:null},D0(c)){if(h)throw Error(n(479))}else h=RS(c,g,v,2),h!==null&&ni(h,c,2)}function D0(c){var h=c.alternate;return c===dr||h!==null&&h===dr}function sI(c,h){Id=k0=!0;var g=c.pending;g===null?h.next=h:(h.next=g.next,g.next=h),c.pending=h}function lI(c,h,g){if((g&4194048)!==0){var v=h.lanes;v&=c.pendingLanes,g|=v,h.lanes=g,Vn(c,g)}}var nf={readContext:ha,use:A0,useCallback:Tn,useContext:Tn,useEffect:Tn,useImperativeHandle:Tn,useLayoutEffect:Tn,useInsertionEffect:Tn,useMemo:Tn,useReducer:Tn,useRef:Tn,useState:Tn,useDebugValue:Tn,useDeferredValue:Tn,useTransition:Tn,useSyncExternalStore:Tn,useId:Tn,useHostTransitionStatus:Tn,useFormState:Tn,useActionState:Tn,useOptimistic:Tn,useMemoCache:Tn,useCacheRefresh:Tn};nf.useEffectEvent=Tn;var uI={readContext:ha,use:A0,useCallback:function(c,h){return Ha().memoizedState=[c,h===void 0?null:h],c},useContext:ha,useEffect:WM,useImperativeHandle:function(c,h,g){g=g!=null?g.concat([c]):null,_0(4194308,4,JM.bind(null,h,c),g)},useLayoutEffect:function(c,h){return _0(4194308,4,c,h)},useInsertionEffect:function(c,h){_0(4,2,c,h)},useMemo:function(c,h){var g=Ha();h=h===void 0?null:h;var v=c();if(ac){lr(!0);try{c()}finally{lr(!1)}}return g.memoizedState=[v,h],v},useReducer:function(c,h,g){var v=Ha();if(g!==void 0){var A=g(h);if(ac){lr(!0);try{g(h)}finally{lr(!1)}}}else A=h;return v.memoizedState=v.baseState=A,c={pending:null,lanes:0,dispatch:null,lastRenderedReducer:c,lastRenderedState:A},v.queue=c,c=c.dispatch=Vde.bind(null,dr,c),[v.memoizedState,c]},useRef:function(c){var h=Ha();return c={current:c},h.memoizedState=c},useState:function(c){c=oE(c);var h=c.queue,g=oI.bind(null,dr,h);return h.dispatch=g,[c.memoizedState,g]},useDebugValue:uE,useDeferredValue:function(c,h){var g=Ha();return cE(g,c,h)},useTransition:function(){var c=oE(!1);return c=tI.bind(null,dr,c.queue,!0,!1),Ha().memoizedState=c,[!1,c]},useSyncExternalStore:function(c,h,g){var v=dr,A=Ha();if(Dr){if(g===void 0)throw Error(n(407));g=g()}else{if(g=h(),Zr===null)throw Error(n(349));(Fr&127)!==0||RM(v,h,g)}A.memoizedState=g;var D={value:g,getSnapshot:h};return A.queue=D,WM(NM.bind(null,v,D,c),[c]),v.flags|=2048,Bd(9,{destroy:void 0},DM.bind(null,v,D,g,h),null),g},useId:function(){var c=Ha(),h=Zr.identifierPrefix;if(Dr){var g=Go,v=$o;g=(v&~(1<<32-qt(v)-1)).toString(32)+g,h="_"+h+"R_"+g,g=C0++,0<g&&(h+="H"+g.toString(32)),h+="_"}else g=Hde++,h="_"+h+"r_"+g.toString(32)+"_";return c.memoizedState=h},useHostTransitionStatus:hE,useFormState:UM,useActionState:UM,useOptimistic:function(c){var h=Ha();h.memoizedState=h.baseState=c;var g={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return h.queue=g,h=pE.bind(null,dr,!0,g),g.dispatch=h,[c,h]},useMemoCache:nE,useCacheRefresh:function(){return Ha().memoizedState=Wde.bind(null,dr)},useEffectEvent:function(c){var h=Ha(),g={impl:c};return h.memoizedState=g,function(){if((Ur&2)!==0)throw Error(n(440));return g.impl.apply(void 0,arguments)}}},fE={readContext:ha,use:A0,useCallback:ZM,useContext:ha,useEffect:lE,useImperativeHandle:XM,useInsertionEffect:KM,useLayoutEffect:YM,useMemo:QM,useReducer:F0,useRef:qM,useState:function(){return F0(Ls)},useDebugValue:uE,useDeferredValue:function(c,h){var g=Rn();return eI(g,Vr.memoizedState,c,h)},useTransition:function(){var c=F0(Ls)[0],h=Rn().memoizedState;return[typeof c=="boolean"?c:tf(c),h]},useSyncExternalStore:_M,useId:aI,useHostTransitionStatus:hE,useFormState:$M,useActionState:$M,useOptimistic:function(c,h){var g=Rn();return IM(g,Vr,c,h)},useMemoCache:nE,useCacheRefresh:iI};fE.useEffectEvent=VM;var cI={readContext:ha,use:A0,useCallback:ZM,useContext:ha,useEffect:lE,useImperativeHandle:XM,useInsertionEffect:KM,useLayoutEffect:YM,useMemo:QM,useReducer:iE,useRef:qM,useState:function(){return iE(Ls)},useDebugValue:uE,useDeferredValue:function(c,h){var g=Rn();return Vr===null?cE(g,c,h):eI(g,Vr.memoizedState,c,h)},useTransition:function(){var c=iE(Ls)[0],h=Rn().memoizedState;return[typeof c=="boolean"?c:tf(c),h]},useSyncExternalStore:_M,useId:aI,useHostTransitionStatus:hE,useFormState:jM,useActionState:jM,useOptimistic:function(c,h){var g=Rn();return Vr!==null?IM(g,Vr,c,h):(g.baseState=c,[c,g.queue.dispatch])},useMemoCache:nE,useCacheRefresh:iI};cI.useEffectEvent=VM;function mE(c,h,g,v){h=c.memoizedState,g=g(v,h),g=g==null?h:p({},h,g),c.memoizedState=g,c.lanes===0&&(c.updateQueue.baseState=g)}var gE={enqueueSetState:function(c,h,g){c=c._reactInternals;var v=Fi(),A=Hl(v);A.payload=h,g!=null&&(A.callback=g),h=Ul(c,A,v),h!==null&&(ni(h,c,v),Xp(h,c,v))},enqueueReplaceState:function(c,h,g){c=c._reactInternals;var v=Fi(),A=Hl(v);A.tag=1,A.payload=h,g!=null&&(A.callback=g),h=Ul(c,A,v),h!==null&&(ni(h,c,v),Xp(h,c,v))},enqueueForceUpdate:function(c,h){c=c._reactInternals;var g=Fi(),v=Hl(g);v.tag=2,h!=null&&(v.callback=h),h=Ul(c,v,g),h!==null&&(ni(h,c,g),Xp(h,c,g))}};function dI(c,h,g,v,A,D,V){return c=c.stateNode,typeof c.shouldComponentUpdate=="function"?c.shouldComponentUpdate(v,D,V):h.prototype&&h.prototype.isPureReactComponent?!Gp(g,v)||!Gp(A,D):!0}function hI(c,h,g,v){c=h.state,typeof h.componentWillReceiveProps=="function"&&h.componentWillReceiveProps(g,v),typeof h.UNSAFE_componentWillReceiveProps=="function"&&h.UNSAFE_componentWillReceiveProps(g,v),h.state!==c&&gE.enqueueReplaceState(h,h.state,null)}function ic(c,h){var g=h;if("ref"in h){g={};for(var v in h)v!=="ref"&&(g[v]=h[v])}if(c=c.defaultProps){g===h&&(g=p({},g));for(var A in c)g[A]===void 0&&(g[A]=c[A])}return g}function pI(c){u0(c)}function fI(c){console.error(c)}function mI(c){u0(c)}function N0(c,h){try{var g=c.onUncaughtError;g(h.value,{componentStack:h.stack})}catch(v){setTimeout(function(){throw v})}}function gI(c,h,g){try{var v=c.onCaughtError;v(g.value,{componentStack:g.stack,errorBoundary:h.tag===1?h.stateNode:null})}catch(A){setTimeout(function(){throw A})}}function bE(c,h,g){return g=Hl(g),g.tag=3,g.payload={element:null},g.callback=function(){N0(c,h)},g}function bI(c){return c=Hl(c),c.tag=3,c}function yI(c,h,g,v){var A=g.type.getDerivedStateFromError;if(typeof A=="function"){var D=v.value;c.payload=function(){return A(D)},c.callback=function(){gI(h,g,v)}}var V=g.stateNode;V!==null&&typeof V.componentDidCatch=="function"&&(c.callback=function(){gI(h,g,v),typeof A!="function"&&(Vl===null?Vl=new Set([this]):Vl.add(this));var Q=v.stack;this.componentDidCatch(v.value,{componentStack:Q!==null?Q:""})})}function Kde(c,h,g,v,A){if(g.flags|=32768,v!==null&&typeof v=="object"&&typeof v.then=="function"){if(h=g.alternate,h!==null&&_d(h,g,A,!0),g=ki.current,g!==null){switch(g.tag){case 31:case 13:return Ji===null?j0():g.alternate===null&&An===0&&(An=3),g.flags&=-257,g.flags|=65536,g.lanes=A,v===v0?g.flags|=16384:(h=g.updateQueue,h===null?g.updateQueue=new Set([v]):h.add(v),$E(c,v,A)),!1;case 22:return g.flags|=65536,v===v0?g.flags|=16384:(h=g.updateQueue,h===null?(h={transitions:null,markerInstances:null,retryQueue:new Set([v])},g.updateQueue=h):(g=h.retryQueue,g===null?h.retryQueue=new Set([v]):g.add(v)),$E(c,v,A)),!1}throw Error(n(435,g.tag))}return $E(c,v,A),j0(),!1}if(Dr)return h=ki.current,h!==null?((h.flags&65536)===0&&(h.flags|=256),h.flags|=65536,h.lanes=A,v!==OS&&(c=Error(n(422),{cause:v}),Wp(Wi(c,g)))):(v!==OS&&(h=Error(n(423),{cause:v}),Wp(Wi(h,g))),c=c.current.alternate,c.flags|=65536,A&=-A,c.lanes|=A,v=Wi(v,g),A=bE(c.stateNode,v,A),VS(c,A),An!==4&&(An=2)),!1;var D=Error(n(520),{cause:v});if(D=Wi(D,g),hf===null?hf=[D]:hf.push(D),An!==4&&(An=2),h===null)return!0;v=Wi(v,g),g=h;do{switch(g.tag){case 3:return g.flags|=65536,c=A&-A,g.lanes|=c,c=bE(g.stateNode,v,c),VS(g,c),!1;case 1:if(h=g.type,D=g.stateNode,(g.flags&128)===0&&(typeof h.getDerivedStateFromError=="function"||D!==null&&typeof D.componentDidCatch=="function"&&(Vl===null||!Vl.has(D))))return g.flags|=65536,A&=-A,g.lanes|=A,A=bI(A),yI(A,c,g,v),VS(g,A),!1}g=g.return}while(g!==null);return!1}var yE=Error(n(461)),Pn=!1;function pa(c,h,g,v){h.child=c===null?xM(h,null,g,v):nc(h,c.child,g,v)}function vI(c,h,g,v,A){g=g.render;var D=h.ref;if("ref"in v){var V={};for(var Q in v)Q!=="ref"&&(V[Q]=v[Q])}else V=v;return Qu(h),v=QS(c,h,g,V,D,A),Q=eE(),c!==null&&!Pn?(tE(c,h,A),Ms(c,h,A)):(Dr&&Q&&MS(h),h.flags|=1,pa(c,h,v,A),h.child)}function SI(c,h,g,v,A){if(c===null){var D=g.type;return typeof D=="function"&&!DS(D)&&D.defaultProps===void 0&&g.compare===null?(h.tag=15,h.type=D,EI(c,h,D,v,A)):(c=p0(g.type,null,v,h,h.mode,A),c.ref=h.ref,c.return=h,h.child=c)}if(D=c.child,!TE(c,A)){var V=D.memoizedProps;if(g=g.compare,g=g!==null?g:Gp,g(V,v)&&c.ref===h.ref)return Ms(c,h,A)}return h.flags|=1,c=Fs(D,v),c.ref=h.ref,c.return=h,h.child=c}function EI(c,h,g,v,A){if(c!==null){var D=c.memoizedProps;if(Gp(D,v)&&c.ref===h.ref)if(Pn=!1,h.pendingProps=v=D,TE(c,A))(c.flags&131072)!==0&&(Pn=!0);else return h.lanes=c.lanes,Ms(c,h,A)}return vE(c,h,g,v,A)}function xI(c,h,g,v){var A=v.children,D=c!==null?c.memoizedState:null;if(c===null&&h.stateNode===null&&(h.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),v.mode==="hidden"){if((h.flags&128)!==0){if(D=D!==null?D.baseLanes|g:g,c!==null){for(v=h.child=c.child,A=0;v!==null;)A=A|v.lanes|v.childLanes,v=v.sibling;v=A&~D}else v=0,h.child=null;return wI(c,h,D,g,v)}if((g&536870912)!==0)h.memoizedState={baseLanes:0,cachePool:null},c!==null&&b0(h,D!==null?D.cachePool:null),D!==null?CM(h,D):YS(),TM(h);else return v=h.lanes=536870912,wI(c,h,D!==null?D.baseLanes|g:g,g,v)}else D!==null?(b0(h,D.cachePool),CM(h,D),Gl(),h.memoizedState=null):(c!==null&&b0(h,null),YS(),Gl());return pa(c,h,A,g),h.child}function af(c,h){return c!==null&&c.tag===22||h.stateNode!==null||(h.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),h.sibling}function wI(c,h,g,v,A){var D=GS();return D=D===null?null:{parent:On._currentValue,pool:D},h.memoizedState={baseLanes:g,cachePool:D},c!==null&&b0(h,null),YS(),TM(h),c!==null&&_d(c,h,v,!0),h.childLanes=A,null}function L0(c,h){return h=I0({mode:h.mode,children:h.children},c.mode),h.ref=c.ref,c.child=h,h.return=c,h}function kI(c,h,g){return nc(h,c.child,null,g),c=L0(h,h.pendingProps),c.flags|=2,Ci(h),h.memoizedState=null,c}function Yde(c,h,g){var v=h.pendingProps,A=(h.flags&128)!==0;if(h.flags&=-129,c===null){if(Dr){if(v.mode==="hidden")return c=L0(h,v),h.lanes=536870912,af(null,c);if(XS(h),(c=ln)?(c=OO(c,Yi),c=c!==null&&c.data==="&"?c:null,c!==null&&(h.memoizedState={dehydrated:c,treeContext:Il!==null?{id:$o,overflow:Go}:null,retryLane:536870912,hydrationErrors:null},g=sM(c),g.return=h,h.child=g,da=h,ln=null)):c=null,c===null)throw Bl(h);return h.lanes=536870912,null}return L0(h,v)}var D=c.memoizedState;if(D!==null){var V=D.dehydrated;if(XS(h),A)if(h.flags&256)h.flags&=-257,h=kI(c,h,g);else if(h.memoizedState!==null)h.child=c.child,h.flags|=128,h=null;else throw Error(n(558));else if(Pn||_d(c,h,g,!1),A=(g&c.childLanes)!==0,Pn||A){if(v=Zr,v!==null&&(V=kn(v,g),V!==0&&V!==D.retryLane))throw D.retryLane=V,Yu(c,V),ni(v,c,V),yE;j0(),h=kI(c,h,g)}else c=D.treeContext,ln=Xi(V.nextSibling),da=h,Dr=!0,Ol=null,Yi=!1,c!==null&&cM(h,c),h=L0(h,v),h.flags|=4096;return h}return c=Fs(c.child,{mode:v.mode,children:v.children}),c.ref=h.ref,h.child=c,c.return=h,c}function M0(c,h){var g=h.ref;if(g===null)c!==null&&c.ref!==null&&(h.flags|=4194816);else{if(typeof g!="function"&&typeof g!="object")throw Error(n(284));(c===null||c.ref!==g)&&(h.flags|=4194816)}}function vE(c,h,g,v,A){return Qu(h),g=QS(c,h,g,v,void 0,A),v=eE(),c!==null&&!Pn?(tE(c,h,A),Ms(c,h,A)):(Dr&&v&&MS(h),h.flags|=1,pa(c,h,g,A),h.child)}function CI(c,h,g,v,A,D){return Qu(h),h.updateQueue=null,g=FM(h,v,g,A),AM(c),v=eE(),c!==null&&!Pn?(tE(c,h,D),Ms(c,h,D)):(Dr&&v&&MS(h),h.flags|=1,pa(c,h,g,D),h.child)}function TI(c,h,g,v,A){if(Qu(h),h.stateNode===null){var D=Cd,V=g.contextType;typeof V=="object"&&V!==null&&(D=ha(V)),D=new g(v,D),h.memoizedState=D.state!==null&&D.state!==void 0?D.state:null,D.updater=gE,h.stateNode=D,D._reactInternals=h,D=h.stateNode,D.props=v,D.state=h.memoizedState,D.refs={},qS(h),V=g.contextType,D.context=typeof V=="object"&&V!==null?ha(V):Cd,D.state=h.memoizedState,V=g.getDerivedStateFromProps,typeof V=="function"&&(mE(h,g,V,v),D.state=h.memoizedState),typeof g.getDerivedStateFromProps=="function"||typeof D.getSnapshotBeforeUpdate=="function"||typeof D.UNSAFE_componentWillMount!="function"&&typeof D.componentWillMount!="function"||(V=D.state,typeof D.componentWillMount=="function"&&D.componentWillMount(),typeof D.UNSAFE_componentWillMount=="function"&&D.UNSAFE_componentWillMount(),V!==D.state&&gE.enqueueReplaceState(D,D.state,null),Qp(h,v,D,A),Zp(),D.state=h.memoizedState),typeof D.componentDidMount=="function"&&(h.flags|=4194308),v=!0}else if(c===null){D=h.stateNode;var Q=h.memoizedProps,de=ic(g,Q);D.props=de;var _e=D.context,ze=g.contextType;V=Cd,typeof ze=="object"&&ze!==null&&(V=ha(ze));var Ue=g.getDerivedStateFromProps;ze=typeof Ue=="function"||typeof D.getSnapshotBeforeUpdate=="function",Q=h.pendingProps!==Q,ze||typeof D.UNSAFE_componentWillReceiveProps!="function"&&typeof D.componentWillReceiveProps!="function"||(Q||_e!==V)&&hI(h,D,v,V),zl=!1;var De=h.memoizedState;D.state=De,Qp(h,v,D,A),Zp(),_e=h.memoizedState,Q||De!==_e||zl?(typeof Ue=="function"&&(mE(h,g,Ue,v),_e=h.memoizedState),(de=zl||dI(h,g,de,v,De,_e,V))?(ze||typeof D.UNSAFE_componentWillMount!="function"&&typeof D.componentWillMount!="function"||(typeof D.componentWillMount=="function"&&D.componentWillMount(),typeof D.UNSAFE_componentWillMount=="function"&&D.UNSAFE_componentWillMount()),typeof D.componentDidMount=="function"&&(h.flags|=4194308)):(typeof D.componentDidMount=="function"&&(h.flags|=4194308),h.memoizedProps=v,h.memoizedState=_e),D.props=v,D.state=_e,D.context=V,v=de):(typeof D.componentDidMount=="function"&&(h.flags|=4194308),v=!1)}else{D=h.stateNode,WS(c,h),V=h.memoizedProps,ze=ic(g,V),D.props=ze,Ue=h.pendingProps,De=D.context,_e=g.contextType,de=Cd,typeof _e=="object"&&_e!==null&&(de=ha(_e)),Q=g.getDerivedStateFromProps,(_e=typeof Q=="function"||typeof D.getSnapshotBeforeUpdate=="function")||typeof D.UNSAFE_componentWillReceiveProps!="function"&&typeof D.componentWillReceiveProps!="function"||(V!==Ue||De!==de)&&hI(h,D,v,de),zl=!1,De=h.memoizedState,D.state=De,Qp(h,v,D,A),Zp();var Ie=h.memoizedState;V!==Ue||De!==Ie||zl||c!==null&&c.dependencies!==null&&m0(c.dependencies)?(typeof Q=="function"&&(mE(h,g,Q,v),Ie=h.memoizedState),(ze=zl||dI(h,g,ze,v,De,Ie,de)||c!==null&&c.dependencies!==null&&m0(c.dependencies))?(_e||typeof D.UNSAFE_componentWillUpdate!="function"&&typeof D.componentWillUpdate!="function"||(typeof D.componentWillUpdate=="function"&&D.componentWillUpdate(v,Ie,de),typeof D.UNSAFE_componentWillUpdate=="function"&&D.UNSAFE_componentWillUpdate(v,Ie,de)),typeof D.componentDidUpdate=="function"&&(h.flags|=4),typeof D.getSnapshotBeforeUpdate=="function"&&(h.flags|=1024)):(typeof D.componentDidUpdate!="function"||V===c.memoizedProps&&De===c.memoizedState||(h.flags|=4),typeof D.getSnapshotBeforeUpdate!="function"||V===c.memoizedProps&&De===c.memoizedState||(h.flags|=1024),h.memoizedProps=v,h.memoizedState=Ie),D.props=v,D.state=Ie,D.context=de,v=ze):(typeof D.componentDidUpdate!="function"||V===c.memoizedProps&&De===c.memoizedState||(h.flags|=4),typeof D.getSnapshotBeforeUpdate!="function"||V===c.memoizedProps&&De===c.memoizedState||(h.flags|=1024),v=!1)}return D=v,M0(c,h),v=(h.flags&128)!==0,D||v?(D=h.stateNode,g=v&&typeof g.getDerivedStateFromError!="function"?null:D.render(),h.flags|=1,c!==null&&v?(h.child=nc(h,c.child,null,A),h.child=nc(h,null,g,A)):pa(c,h,g,A),h.memoizedState=D.state,c=h.child):c=Ms(c,h,A),c}function AI(c,h,g,v){return Xu(),h.flags|=256,pa(c,h,g,v),h.child}var SE={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function EE(c){return{baseLanes:c,cachePool:gM()}}function xE(c,h,g){return c=c!==null?c.childLanes&~g:0,h&&(c|=Ai),c}function FI(c,h,g){var v=h.pendingProps,A=!1,D=(h.flags&128)!==0,V;if((V=D)||(V=c!==null&&c.memoizedState===null?!1:(_n.current&2)!==0),V&&(A=!0,h.flags&=-129),V=(h.flags&32)!==0,h.flags&=-33,c===null){if(Dr){if(A?$l(h):Gl(),(c=ln)?(c=OO(c,Yi),c=c!==null&&c.data!=="&"?c:null,c!==null&&(h.memoizedState={dehydrated:c,treeContext:Il!==null?{id:$o,overflow:Go}:null,retryLane:536870912,hydrationErrors:null},g=sM(c),g.return=h,h.child=g,da=h,ln=null)):c=null,c===null)throw Bl(h);return ax(c)?h.lanes=32:h.lanes=536870912,null}var Q=v.children;return v=v.fallback,A?(Gl(),A=h.mode,Q=I0({mode:"hidden",children:Q},A),v=Ju(v,A,g,null),Q.return=h,v.return=h,Q.sibling=v,h.child=Q,v=h.child,v.memoizedState=EE(g),v.childLanes=xE(c,V,g),h.memoizedState=SE,af(null,v)):($l(h),wE(h,Q))}var de=c.memoizedState;if(de!==null&&(Q=de.dehydrated,Q!==null)){if(D)h.flags&256?($l(h),h.flags&=-257,h=kE(c,h,g)):h.memoizedState!==null?(Gl(),h.child=c.child,h.flags|=128,h=null):(Gl(),Q=v.fallback,A=h.mode,v=I0({mode:"visible",children:v.children},A),Q=Ju(Q,A,g,null),Q.flags|=2,v.return=h,Q.return=h,v.sibling=Q,h.child=v,nc(h,c.child,null,g),v=h.child,v.memoizedState=EE(g),v.childLanes=xE(c,V,g),h.memoizedState=SE,h=af(null,v));else if($l(h),ax(Q)){if(V=Q.nextSibling&&Q.nextSibling.dataset,V)var _e=V.dgst;V=_e,v=Error(n(419)),v.stack="",v.digest=V,Wp({value:v,source:null,stack:null}),h=kE(c,h,g)}else if(Pn||_d(c,h,g,!1),V=(g&c.childLanes)!==0,Pn||V){if(V=Zr,V!==null&&(v=kn(V,g),v!==0&&v!==de.retryLane))throw de.retryLane=v,Yu(c,v),ni(V,c,v),yE;nx(Q)||j0(),h=kE(c,h,g)}else nx(Q)?(h.flags|=192,h.child=c.child,h=null):(c=de.treeContext,ln=Xi(Q.nextSibling),da=h,Dr=!0,Ol=null,Yi=!1,c!==null&&cM(h,c),h=wE(h,v.children),h.flags|=4096);return h}return A?(Gl(),Q=v.fallback,A=h.mode,de=c.child,_e=de.sibling,v=Fs(de,{mode:"hidden",children:v.children}),v.subtreeFlags=de.subtreeFlags&65011712,_e!==null?Q=Fs(_e,Q):(Q=Ju(Q,A,g,null),Q.flags|=2),Q.return=h,v.return=h,v.sibling=Q,h.child=v,af(null,v),v=h.child,Q=c.child.memoizedState,Q===null?Q=EE(g):(A=Q.cachePool,A!==null?(de=On._currentValue,A=A.parent!==de?{parent:de,pool:de}:A):A=gM(),Q={baseLanes:Q.baseLanes|g,cachePool:A}),v.memoizedState=Q,v.childLanes=xE(c,V,g),h.memoizedState=SE,af(c.child,v)):($l(h),g=c.child,c=g.sibling,g=Fs(g,{mode:"visible",children:v.children}),g.return=h,g.sibling=null,c!==null&&(V=h.deletions,V===null?(h.deletions=[c],h.flags|=16):V.push(c)),h.child=g,h.memoizedState=null,g)}function wE(c,h){return h=I0({mode:"visible",children:h},c.mode),h.return=c,c.child=h}function I0(c,h){return c=wi(22,c,null,h),c.lanes=0,c}function kE(c,h,g){return nc(h,c.child,null,g),c=wE(h,h.pendingProps.children),c.flags|=2,h.memoizedState=null,c}function _I(c,h,g){c.lanes|=h;var v=c.alternate;v!==null&&(v.lanes|=h),zS(c.return,h,g)}function CE(c,h,g,v,A,D){var V=c.memoizedState;V===null?c.memoizedState={isBackwards:h,rendering:null,renderingStartTime:0,last:v,tail:g,tailMode:A,treeForkCount:D}:(V.isBackwards=h,V.rendering=null,V.renderingStartTime=0,V.last=v,V.tail=g,V.tailMode=A,V.treeForkCount=D)}function RI(c,h,g){var v=h.pendingProps,A=v.revealOrder,D=v.tail;v=v.children;var V=_n.current,Q=(V&2)!==0;if(Q?(V=V&1|2,h.flags|=128):V&=1,P(_n,V),pa(c,h,v,g),v=Dr?qp:0,!Q&&c!==null&&(c.flags&128)!==0)e:for(c=h.child;c!==null;){if(c.tag===13)c.memoizedState!==null&&_I(c,g,h);else if(c.tag===19)_I(c,g,h);else if(c.child!==null){c.child.return=c,c=c.child;continue}if(c===h)break e;for(;c.sibling===null;){if(c.return===null||c.return===h)break e;c=c.return}c.sibling.return=c.return,c=c.sibling}switch(A){case"forwards":for(g=h.child,A=null;g!==null;)c=g.alternate,c!==null&&w0(c)===null&&(A=g),g=g.sibling;g=A,g===null?(A=h.child,h.child=null):(A=g.sibling,g.sibling=null),CE(h,!1,A,g,D,v);break;case"backwards":case"unstable_legacy-backwards":for(g=null,A=h.child,h.child=null;A!==null;){if(c=A.alternate,c!==null&&w0(c)===null){h.child=A;break}c=A.sibling,A.sibling=g,g=A,A=c}CE(h,!0,g,null,D,v);break;case"together":CE(h,!1,null,null,void 0,v);break;default:h.memoizedState=null}return h.child}function Ms(c,h,g){if(c!==null&&(h.dependencies=c.dependencies),Wl|=h.lanes,(g&h.childLanes)===0)if(c!==null){if(_d(c,h,g,!1),(g&h.childLanes)===0)return null}else return null;if(c!==null&&h.child!==c.child)throw Error(n(153));if(h.child!==null){for(c=h.child,g=Fs(c,c.pendingProps),h.child=g,g.return=h;c.sibling!==null;)c=c.sibling,g=g.sibling=Fs(c,c.pendingProps),g.return=h;g.sibling=null}return h.child}function TE(c,h){return(c.lanes&h)!==0?!0:(c=c.dependencies,!!(c!==null&&m0(c)))}function Jde(c,h,g){switch(h.tag){case 3:se(h,h.stateNode.containerInfo),Pl(h,On,c.memoizedState.cache),Xu();break;case 27:case 5:fe(h);break;case 4:se(h,h.stateNode.containerInfo);break;case 10:Pl(h,h.type,h.memoizedProps.value);break;case 31:if(h.memoizedState!==null)return h.flags|=128,XS(h),null;break;case 13:var v=h.memoizedState;if(v!==null)return v.dehydrated!==null?($l(h),h.flags|=128,null):(g&h.child.childLanes)!==0?FI(c,h,g):($l(h),c=Ms(c,h,g),c!==null?c.sibling:null);$l(h);break;case 19:var A=(c.flags&128)!==0;if(v=(g&h.childLanes)!==0,v||(_d(c,h,g,!1),v=(g&h.childLanes)!==0),A){if(v)return RI(c,h,g);h.flags|=128}if(A=h.memoizedState,A!==null&&(A.rendering=null,A.tail=null,A.lastEffect=null),P(_n,_n.current),v)break;return null;case 22:return h.lanes=0,xI(c,h,g,h.pendingProps);case 24:Pl(h,On,c.memoizedState.cache)}return Ms(c,h,g)}function DI(c,h,g){if(c!==null)if(c.memoizedProps!==h.pendingProps)Pn=!0;else{if(!TE(c,g)&&(h.flags&128)===0)return Pn=!1,Jde(c,h,g);Pn=(c.flags&131072)!==0}else Pn=!1,Dr&&(h.flags&1048576)!==0&&uM(h,qp,h.index);switch(h.lanes=0,h.tag){case 16:e:{var v=h.pendingProps;if(c=tc(h.elementType),h.type=c,typeof c=="function")DS(c)?(v=ic(c,v),h.tag=1,h=TI(null,h,c,v,g)):(h.tag=0,h=vE(null,h,c,v,g));else{if(c!=null){var A=c.$$typeof;if(A===_){h.tag=11,h=vI(null,h,c,v,g);break e}else if(A===F){h.tag=14,h=SI(null,h,c,v,g);break e}}throw h=$(c)||c,Error(n(306,h,""))}}return h;case 0:return vE(c,h,h.type,h.pendingProps,g);case 1:return v=h.type,A=ic(v,h.pendingProps),TI(c,h,v,A,g);case 3:e:{if(se(h,h.stateNode.containerInfo),c===null)throw Error(n(387));v=h.pendingProps;var D=h.memoizedState;A=D.element,WS(c,h),Qp(h,v,null,g);var V=h.memoizedState;if(v=V.cache,Pl(h,On,v),v!==D.cache&&HS(h,[On],g,!0),Zp(),v=V.element,D.isDehydrated)if(D={element:v,isDehydrated:!1,cache:V.cache},h.updateQueue.baseState=D,h.memoizedState=D,h.flags&256){h=AI(c,h,v,g);break e}else if(v!==A){A=Wi(Error(n(424)),h),Wp(A),h=AI(c,h,v,g);break e}else{switch(c=h.stateNode.containerInfo,c.nodeType){case 9:c=c.body;break;default:c=c.nodeName==="HTML"?c.ownerDocument.body:c}for(ln=Xi(c.firstChild),da=h,Dr=!0,Ol=null,Yi=!0,g=xM(h,null,v,g),h.child=g;g;)g.flags=g.flags&-3|4096,g=g.sibling}else{if(Xu(),v===A){h=Ms(c,h,g);break e}pa(c,h,v,g)}h=h.child}return h;case 26:return M0(c,h),c===null?(g=$O(h.type,null,h.pendingProps,null))?h.memoizedState=g:Dr||(g=h.type,c=h.pendingProps,v=X0(pe.current).createElement(g),v[Xr]=h,v[ca]=c,fa(v,g,c),Mr(v),h.stateNode=v):h.memoizedState=$O(h.type,c.memoizedProps,h.pendingProps,c.memoizedState),null;case 27:return fe(h),c===null&&Dr&&(v=h.stateNode=zO(h.type,h.pendingProps,pe.current),da=h,Yi=!0,A=ln,Xl(h.type)?(ix=A,ln=Xi(v.firstChild)):ln=A),pa(c,h,h.pendingProps.children,g),M0(c,h),c===null&&(h.flags|=4194304),h.child;case 5:return c===null&&Dr&&((A=v=ln)&&(v=The(v,h.type,h.pendingProps,Yi),v!==null?(h.stateNode=v,da=h,ln=Xi(v.firstChild),Yi=!1,A=!0):A=!1),A||Bl(h)),fe(h),A=h.type,D=h.pendingProps,V=c!==null?c.memoizedProps:null,v=D.children,ex(A,D)?v=null:V!==null&&ex(A,V)&&(h.flags|=32),h.memoizedState!==null&&(A=QS(c,h,Ude,null,null,g),Sf._currentValue=A),M0(c,h),pa(c,h,v,g),h.child;case 6:return c===null&&Dr&&((c=g=ln)&&(g=Ahe(g,h.pendingProps,Yi),g!==null?(h.stateNode=g,da=h,ln=null,c=!0):c=!1),c||Bl(h)),null;case 13:return FI(c,h,g);case 4:return se(h,h.stateNode.containerInfo),v=h.pendingProps,c===null?h.child=nc(h,null,v,g):pa(c,h,v,g),h.child;case 11:return vI(c,h,h.type,h.pendingProps,g);case 7:return pa(c,h,h.pendingProps,g),h.child;case 8:return pa(c,h,h.pendingProps.children,g),h.child;case 12:return pa(c,h,h.pendingProps.children,g),h.child;case 10:return v=h.pendingProps,Pl(h,h.type,v.value),pa(c,h,v.children,g),h.child;case 9:return A=h.type._context,v=h.pendingProps.children,Qu(h),A=ha(A),v=v(A),h.flags|=1,pa(c,h,v,g),h.child;case 14:return SI(c,h,h.type,h.pendingProps,g);case 15:return EI(c,h,h.type,h.pendingProps,g);case 19:return RI(c,h,g);case 31:return Yde(c,h,g);case 22:return xI(c,h,g,h.pendingProps);case 24:return Qu(h),v=ha(On),c===null?(A=GS(),A===null&&(A=Zr,D=US(),A.pooledCache=D,D.refCount++,D!==null&&(A.pooledCacheLanes|=g),A=D),h.memoizedState={parent:v,cache:A},qS(h),Pl(h,On,A)):((c.lanes&g)!==0&&(WS(c,h),Qp(h,null,null,g),Zp()),A=c.memoizedState,D=h.memoizedState,A.parent!==v?(A={parent:v,cache:v},h.memoizedState=A,h.lanes===0&&(h.memoizedState=h.updateQueue.baseState=A),Pl(h,On,v)):(v=D.cache,Pl(h,On,v),v!==A.cache&&HS(h,[On],g,!0))),pa(c,h,h.pendingProps.children,g),h.child;case 29:throw h.pendingProps}throw Error(n(156,h.tag))}function Is(c){c.flags|=4}function AE(c,h,g,v,A){if((h=(c.mode&32)!==0)&&(h=!1),h){if(c.flags|=16777216,(A&335544128)===A)if(c.stateNode.complete)c.flags|=8192;else if(aO())c.flags|=8192;else throw rc=v0,jS}else c.flags&=-16777217}function NI(c,h){if(h.type!=="stylesheet"||(h.state.loading&4)!==0)c.flags&=-16777217;else if(c.flags|=16777216,!VO(h))if(aO())c.flags|=8192;else throw rc=v0,jS}function O0(c,h){h!==null&&(c.flags|=4),c.flags&16384&&(h=c.tag!==22?Nr():536870912,c.lanes|=h,Ud|=h)}function of(c,h){if(!Dr)switch(c.tailMode){case"hidden":h=c.tail;for(var g=null;h!==null;)h.alternate!==null&&(g=h),h=h.sibling;g===null?c.tail=null:g.sibling=null;break;case"collapsed":g=c.tail;for(var v=null;g!==null;)g.alternate!==null&&(v=g),g=g.sibling;v===null?h||c.tail===null?c.tail=null:c.tail.sibling=null:v.sibling=null}}function un(c){var h=c.alternate!==null&&c.alternate.child===c.child,g=0,v=0;if(h)for(var A=c.child;A!==null;)g|=A.lanes|A.childLanes,v|=A.subtreeFlags&65011712,v|=A.flags&65011712,A.return=c,A=A.sibling;else for(A=c.child;A!==null;)g|=A.lanes|A.childLanes,v|=A.subtreeFlags,v|=A.flags,A.return=c,A=A.sibling;return c.subtreeFlags|=v,c.childLanes=g,h}function Xde(c,h,g){var v=h.pendingProps;switch(IS(h),h.tag){case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return un(h),null;case 1:return un(h),null;case 3:return g=h.stateNode,v=null,c!==null&&(v=c.memoizedState.cache),h.memoizedState.cache!==v&&(h.flags|=2048),Ds(On),Ee(),g.pendingContext&&(g.context=g.pendingContext,g.pendingContext=null),(c===null||c.child===null)&&(Fd(h)?Is(h):c===null||c.memoizedState.isDehydrated&&(h.flags&256)===0||(h.flags|=1024,BS())),un(h),null;case 26:var A=h.type,D=h.memoizedState;return c===null?(Is(h),D!==null?(un(h),NI(h,D)):(un(h),AE(h,A,null,v,g))):D?D!==c.memoizedState?(Is(h),un(h),NI(h,D)):(un(h),h.flags&=-16777217):(c=c.memoizedProps,c!==v&&Is(h),un(h),AE(h,A,c,v,g)),null;case 27:if(Me(h),g=pe.current,A=h.type,c!==null&&h.stateNode!=null)c.memoizedProps!==v&&Is(h);else{if(!v){if(h.stateNode===null)throw Error(n(166));return un(h),null}c=ne.current,Fd(h)?dM(h):(c=zO(A,v,g),h.stateNode=c,Is(h))}return un(h),null;case 5:if(Me(h),A=h.type,c!==null&&h.stateNode!=null)c.memoizedProps!==v&&Is(h);else{if(!v){if(h.stateNode===null)throw Error(n(166));return un(h),null}if(D=ne.current,Fd(h))dM(h);else{var V=X0(pe.current);switch(D){case 1:D=V.createElementNS("http://www.w3.org/2000/svg",A);break;case 2:D=V.createElementNS("http://www.w3.org/1998/Math/MathML",A);break;default:switch(A){case"svg":D=V.createElementNS("http://www.w3.org/2000/svg",A);break;case"math":D=V.createElementNS("http://www.w3.org/1998/Math/MathML",A);break;case"script":D=V.createElement("div"),D.innerHTML="<script><\/script>",D=D.removeChild(D.firstChild);break;case"select":D=typeof v.is=="string"?V.createElement("select",{is:v.is}):V.createElement("select"),v.multiple?D.multiple=!0:v.size&&(D.size=v.size);break;default:D=typeof v.is=="string"?V.createElement(A,{is:v.is}):V.createElement(A)}}D[Xr]=h,D[ca]=v;e:for(V=h.child;V!==null;){if(V.tag===5||V.tag===6)D.appendChild(V.stateNode);else if(V.tag!==4&&V.tag!==27&&V.child!==null){V.child.return=V,V=V.child;continue}if(V===h)break e;for(;V.sibling===null;){if(V.return===null||V.return===h)break e;V=V.return}V.sibling.return=V.return,V=V.sibling}h.stateNode=D;e:switch(fa(D,A,v),A){case"button":case"input":case"select":case"textarea":v=!!v.autoFocus;break e;case"img":v=!0;break e;default:v=!1}v&&Is(h)}}return un(h),AE(h,h.type,c===null?null:c.memoizedProps,h.pendingProps,g),null;case 6:if(c&&h.stateNode!=null)c.memoizedProps!==v&&Is(h);else{if(typeof v!="string"&&h.stateNode===null)throw Error(n(166));if(c=pe.current,Fd(h)){if(c=h.stateNode,g=h.memoizedProps,v=null,A=da,A!==null)switch(A.tag){case 27:case 5:v=A.memoizedProps}c[Xr]=h,c=!!(c.nodeValue===g||v!==null&&v.suppressHydrationWarning===!0||FO(c.nodeValue,g)),c||Bl(h,!0)}else c=X0(c).createTextNode(v),c[Xr]=h,h.stateNode=c}return un(h),null;case 31:if(g=h.memoizedState,c===null||c.memoizedState!==null){if(v=Fd(h),g!==null){if(c===null){if(!v)throw Error(n(318));if(c=h.memoizedState,c=c!==null?c.dehydrated:null,!c)throw Error(n(557));c[Xr]=h}else Xu(),(h.flags&128)===0&&(h.memoizedState=null),h.flags|=4;un(h),c=!1}else g=BS(),c!==null&&c.memoizedState!==null&&(c.memoizedState.hydrationErrors=g),c=!0;if(!c)return h.flags&256?(Ci(h),h):(Ci(h),null);if((h.flags&128)!==0)throw Error(n(558))}return un(h),null;case 13:if(v=h.memoizedState,c===null||c.memoizedState!==null&&c.memoizedState.dehydrated!==null){if(A=Fd(h),v!==null&&v.dehydrated!==null){if(c===null){if(!A)throw Error(n(318));if(A=h.memoizedState,A=A!==null?A.dehydrated:null,!A)throw Error(n(317));A[Xr]=h}else Xu(),(h.flags&128)===0&&(h.memoizedState=null),h.flags|=4;un(h),A=!1}else A=BS(),c!==null&&c.memoizedState!==null&&(c.memoizedState.hydrationErrors=A),A=!0;if(!A)return h.flags&256?(Ci(h),h):(Ci(h),null)}return Ci(h),(h.flags&128)!==0?(h.lanes=g,h):(g=v!==null,c=c!==null&&c.memoizedState!==null,g&&(v=h.child,A=null,v.alternate!==null&&v.alternate.memoizedState!==null&&v.alternate.memoizedState.cachePool!==null&&(A=v.alternate.memoizedState.cachePool.pool),D=null,v.memoizedState!==null&&v.memoizedState.cachePool!==null&&(D=v.memoizedState.cachePool.pool),D!==A&&(v.flags|=2048)),g!==c&&g&&(h.child.flags|=8192),O0(h,h.updateQueue),un(h),null);case 4:return Ee(),c===null&&YE(h.stateNode.containerInfo),un(h),null;case 10:return Ds(h.type),un(h),null;case 19:if(G(_n),v=h.memoizedState,v===null)return un(h),null;if(A=(h.flags&128)!==0,D=v.rendering,D===null)if(A)of(v,!1);else{if(An!==0||c!==null&&(c.flags&128)!==0)for(c=h.child;c!==null;){if(D=w0(c),D!==null){for(h.flags|=128,of(v,!1),c=D.updateQueue,h.updateQueue=c,O0(h,c),h.subtreeFlags=0,c=g,g=h.child;g!==null;)oM(g,c),g=g.sibling;return P(_n,_n.current&1|2),Dr&&_s(h,v.treeForkCount),h.child}c=c.sibling}v.tail!==null&&Pe()>U0&&(h.flags|=128,A=!0,of(v,!1),h.lanes=4194304)}else{if(!A)if(c=w0(D),c!==null){if(h.flags|=128,A=!0,c=c.updateQueue,h.updateQueue=c,O0(h,c),of(v,!0),v.tail===null&&v.tailMode==="hidden"&&!D.alternate&&!Dr)return un(h),null}else 2*Pe()-v.renderingStartTime>U0&&g!==536870912&&(h.flags|=128,A=!0,of(v,!1),h.lanes=4194304);v.isBackwards?(D.sibling=h.child,h.child=D):(c=v.last,c!==null?c.sibling=D:h.child=D,v.last=D)}return v.tail!==null?(c=v.tail,v.rendering=c,v.tail=c.sibling,v.renderingStartTime=Pe(),c.sibling=null,g=_n.current,P(_n,A?g&1|2:g&1),Dr&&_s(h,v.treeForkCount),c):(un(h),null);case 22:case 23:return Ci(h),JS(),v=h.memoizedState!==null,c!==null?c.memoizedState!==null!==v&&(h.flags|=8192):v&&(h.flags|=8192),v?(g&536870912)!==0&&(h.flags&128)===0&&(un(h),h.subtreeFlags&6&&(h.flags|=8192)):un(h),g=h.updateQueue,g!==null&&O0(h,g.retryQueue),g=null,c!==null&&c.memoizedState!==null&&c.memoizedState.cachePool!==null&&(g=c.memoizedState.cachePool.pool),v=null,h.memoizedState!==null&&h.memoizedState.cachePool!==null&&(v=h.memoizedState.cachePool.pool),v!==g&&(h.flags|=2048),c!==null&&G(ec),null;case 24:return g=null,c!==null&&(g=c.memoizedState.cache),h.memoizedState.cache!==g&&(h.flags|=2048),Ds(On),un(h),null;case 25:return null;case 30:return null}throw Error(n(156,h.tag))}function Zde(c,h){switch(IS(h),h.tag){case 1:return c=h.flags,c&65536?(h.flags=c&-65537|128,h):null;case 3:return Ds(On),Ee(),c=h.flags,(c&65536)!==0&&(c&128)===0?(h.flags=c&-65537|128,h):null;case 26:case 27:case 5:return Me(h),null;case 31:if(h.memoizedState!==null){if(Ci(h),h.alternate===null)throw Error(n(340));Xu()}return c=h.flags,c&65536?(h.flags=c&-65537|128,h):null;case 13:if(Ci(h),c=h.memoizedState,c!==null&&c.dehydrated!==null){if(h.alternate===null)throw Error(n(340));Xu()}return c=h.flags,c&65536?(h.flags=c&-65537|128,h):null;case 19:return G(_n),null;case 4:return Ee(),null;case 10:return Ds(h.type),null;case 22:case 23:return Ci(h),JS(),c!==null&&G(ec),c=h.flags,c&65536?(h.flags=c&-65537|128,h):null;case 24:return Ds(On),null;case 25:return null;default:return null}}function LI(c,h){switch(IS(h),h.tag){case 3:Ds(On),Ee();break;case 26:case 27:case 5:Me(h);break;case 4:Ee();break;case 31:h.memoizedState!==null&&Ci(h);break;case 13:Ci(h);break;case 19:G(_n);break;case 10:Ds(h.type);break;case 22:case 23:Ci(h),JS(),c!==null&&G(ec);break;case 24:Ds(On)}}function sf(c,h){try{var g=h.updateQueue,v=g!==null?g.lastEffect:null;if(v!==null){var A=v.next;g=A;do{if((g.tag&c)===c){v=void 0;var D=g.create,V=g.inst;v=D(),V.destroy=v}g=g.next}while(g!==A)}}catch(Q){Wr(h,h.return,Q)}}function jl(c,h,g){try{var v=h.updateQueue,A=v!==null?v.lastEffect:null;if(A!==null){var D=A.next;v=D;do{if((v.tag&c)===c){var V=v.inst,Q=V.destroy;if(Q!==void 0){V.destroy=void 0,A=h;var de=g,_e=Q;try{_e()}catch(ze){Wr(A,de,ze)}}}v=v.next}while(v!==D)}}catch(ze){Wr(h,h.return,ze)}}function MI(c){var h=c.updateQueue;if(h!==null){var g=c.stateNode;try{kM(h,g)}catch(v){Wr(c,c.return,v)}}}function II(c,h,g){g.props=ic(c.type,c.memoizedProps),g.state=c.memoizedState;try{g.componentWillUnmount()}catch(v){Wr(c,h,v)}}function lf(c,h){try{var g=c.ref;if(g!==null){switch(c.tag){case 26:case 27:case 5:var v=c.stateNode;break;case 30:v=c.stateNode;break;default:v=c.stateNode}typeof g=="function"?c.refCleanup=g(v):g.current=v}}catch(A){Wr(c,h,A)}}function jo(c,h){var g=c.ref,v=c.refCleanup;if(g!==null)if(typeof v=="function")try{v()}catch(A){Wr(c,h,A)}finally{c.refCleanup=null,c=c.alternate,c!=null&&(c.refCleanup=null)}else if(typeof g=="function")try{g(null)}catch(A){Wr(c,h,A)}else g.current=null}function OI(c){var h=c.type,g=c.memoizedProps,v=c.stateNode;try{e:switch(h){case"button":case"input":case"select":case"textarea":g.autoFocus&&v.focus();break e;case"img":g.src?v.src=g.src:g.srcSet&&(v.srcset=g.srcSet)}}catch(A){Wr(c,c.return,A)}}function FE(c,h,g){try{var v=c.stateNode;She(v,c.type,g,h),v[ca]=h}catch(A){Wr(c,c.return,A)}}function BI(c){return c.tag===5||c.tag===3||c.tag===26||c.tag===27&&Xl(c.type)||c.tag===4}function _E(c){e:for(;;){for(;c.sibling===null;){if(c.return===null||BI(c.return))return null;c=c.return}for(c.sibling.return=c.return,c=c.sibling;c.tag!==5&&c.tag!==6&&c.tag!==18;){if(c.tag===27&&Xl(c.type)||c.flags&2||c.child===null||c.tag===4)continue e;c.child.return=c,c=c.child}if(!(c.flags&2))return c.stateNode}}function RE(c,h,g){var v=c.tag;if(v===5||v===6)c=c.stateNode,h?(g.nodeType===9?g.body:g.nodeName==="HTML"?g.ownerDocument.body:g).insertBefore(c,h):(h=g.nodeType===9?g.body:g.nodeName==="HTML"?g.ownerDocument.body:g,h.appendChild(c),g=g._reactRootContainer,g!=null||h.onclick!==null||(h.onclick=Ts));else if(v!==4&&(v===27&&Xl(c.type)&&(g=c.stateNode,h=null),c=c.child,c!==null))for(RE(c,h,g),c=c.sibling;c!==null;)RE(c,h,g),c=c.sibling}function B0(c,h,g){var v=c.tag;if(v===5||v===6)c=c.stateNode,h?g.insertBefore(c,h):g.appendChild(c);else if(v!==4&&(v===27&&Xl(c.type)&&(g=c.stateNode),c=c.child,c!==null))for(B0(c,h,g),c=c.sibling;c!==null;)B0(c,h,g),c=c.sibling}function PI(c){var h=c.stateNode,g=c.memoizedProps;try{for(var v=c.type,A=h.attributes;A.length;)h.removeAttributeNode(A[0]);fa(h,v,g),h[Xr]=c,h[ca]=g}catch(D){Wr(c,c.return,D)}}var Os=!1,zn=!1,DE=!1,zI=typeof WeakSet=="function"?WeakSet:Set,aa=null;function Qde(c,h){if(c=c.containerInfo,ZE=ab,c=XL(c),kS(c)){if("selectionStart"in c)var g={start:c.selectionStart,end:c.selectionEnd};else e:{g=(g=c.ownerDocument)&&g.defaultView||window;var v=g.getSelection&&g.getSelection();if(v&&v.rangeCount!==0){g=v.anchorNode;var A=v.anchorOffset,D=v.focusNode;v=v.focusOffset;try{g.nodeType,D.nodeType}catch{g=null;break e}var V=0,Q=-1,de=-1,_e=0,ze=0,Ue=c,De=null;t:for(;;){for(var Ie;Ue!==g||A!==0&&Ue.nodeType!==3||(Q=V+A),Ue!==D||v!==0&&Ue.nodeType!==3||(de=V+v),Ue.nodeType===3&&(V+=Ue.nodeValue.length),(Ie=Ue.firstChild)!==null;)De=Ue,Ue=Ie;for(;;){if(Ue===c)break t;if(De===g&&++_e===A&&(Q=V),De===D&&++ze===v&&(de=V),(Ie=Ue.nextSibling)!==null)break;Ue=De,De=Ue.parentNode}Ue=Ie}g=Q===-1||de===-1?null:{start:Q,end:de}}else g=null}g=g||{start:0,end:0}}else g=null;for(QE={focusedElem:c,selectionRange:g},ab=!1,aa=h;aa!==null;)if(h=aa,c=h.child,(h.subtreeFlags&1028)!==0&&c!==null)c.return=h,aa=c;else for(;aa!==null;){switch(h=aa,D=h.alternate,c=h.flags,h.tag){case 0:if((c&4)!==0&&(c=h.updateQueue,c=c!==null?c.events:null,c!==null))for(g=0;g<c.length;g++)A=c[g],A.ref.impl=A.nextImpl;break;case 11:case 15:break;case 1:if((c&1024)!==0&&D!==null){c=void 0,g=h,A=D.memoizedProps,D=D.memoizedState,v=g.stateNode;try{var kt=ic(g.type,A);c=v.getSnapshotBeforeUpdate(kt,D),v.__reactInternalSnapshotBeforeUpdate=c}catch(Yt){Wr(g,g.return,Yt)}}break;case 3:if((c&1024)!==0){if(c=h.stateNode.containerInfo,g=c.nodeType,g===9)rx(c);else if(g===1)switch(c.nodeName){case"HEAD":case"HTML":case"BODY":rx(c);break;default:c.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((c&1024)!==0)throw Error(n(163))}if(c=h.sibling,c!==null){c.return=h.return,aa=c;break}aa=h.return}}function HI(c,h,g){var v=g.flags;switch(g.tag){case 0:case 11:case 15:Ps(c,g),v&4&&sf(5,g);break;case 1:if(Ps(c,g),v&4)if(c=g.stateNode,h===null)try{c.componentDidMount()}catch(V){Wr(g,g.return,V)}else{var A=ic(g.type,h.memoizedProps);h=h.memoizedState;try{c.componentDidUpdate(A,h,c.__reactInternalSnapshotBeforeUpdate)}catch(V){Wr(g,g.return,V)}}v&64&&MI(g),v&512&&lf(g,g.return);break;case 3:if(Ps(c,g),v&64&&(c=g.updateQueue,c!==null)){if(h=null,g.child!==null)switch(g.child.tag){case 27:case 5:h=g.child.stateNode;break;case 1:h=g.child.stateNode}try{kM(c,h)}catch(V){Wr(g,g.return,V)}}break;case 27:h===null&&v&4&&PI(g);case 26:case 5:Ps(c,g),h===null&&v&4&&OI(g),v&512&&lf(g,g.return);break;case 12:Ps(c,g);break;case 31:Ps(c,g),v&4&&GI(c,g);break;case 13:Ps(c,g),v&4&&jI(c,g),v&64&&(c=g.memoizedState,c!==null&&(c=c.dehydrated,c!==null&&(g=lhe.bind(null,g),Fhe(c,g))));break;case 22:if(v=g.memoizedState!==null||Os,!v){h=h!==null&&h.memoizedState!==null||zn,A=Os;var D=zn;Os=v,(zn=h)&&!D?zs(c,g,(g.subtreeFlags&8772)!==0):Ps(c,g),Os=A,zn=D}break;case 30:break;default:Ps(c,g)}}function UI(c){var h=c.alternate;h!==null&&(c.alternate=null,UI(h)),c.child=null,c.deletions=null,c.sibling=null,c.tag===5&&(h=c.stateNode,h!==null&&at(h)),c.stateNode=null,c.return=null,c.dependencies=null,c.memoizedProps=null,c.memoizedState=null,c.pendingProps=null,c.stateNode=null,c.updateQueue=null}var fn=null,Qa=!1;function Bs(c,h,g){for(g=g.child;g!==null;)$I(c,h,g),g=g.sibling}function $I(c,h,g){if(Mt&&typeof Mt.onCommitFiberUnmount=="function")try{Mt.onCommitFiberUnmount(fr,g)}catch{}switch(g.tag){case 26:zn||jo(g,h),Bs(c,h,g),g.memoizedState?g.memoizedState.count--:g.stateNode&&(g=g.stateNode,g.parentNode.removeChild(g));break;case 27:zn||jo(g,h);var v=fn,A=Qa;Xl(g.type)&&(fn=g.stateNode,Qa=!1),Bs(c,h,g),bf(g.stateNode),fn=v,Qa=A;break;case 5:zn||jo(g,h);case 6:if(v=fn,A=Qa,fn=null,Bs(c,h,g),fn=v,Qa=A,fn!==null)if(Qa)try{(fn.nodeType===9?fn.body:fn.nodeName==="HTML"?fn.ownerDocument.body:fn).removeChild(g.stateNode)}catch(D){Wr(g,h,D)}else try{fn.removeChild(g.stateNode)}catch(D){Wr(g,h,D)}break;case 18:fn!==null&&(Qa?(c=fn,MO(c.nodeType===9?c.body:c.nodeName==="HTML"?c.ownerDocument.body:c,g.stateNode),Yd(c)):MO(fn,g.stateNode));break;case 4:v=fn,A=Qa,fn=g.stateNode.containerInfo,Qa=!0,Bs(c,h,g),fn=v,Qa=A;break;case 0:case 11:case 14:case 15:jl(2,g,h),zn||jl(4,g,h),Bs(c,h,g);break;case 1:zn||(jo(g,h),v=g.stateNode,typeof v.componentWillUnmount=="function"&&II(g,h,v)),Bs(c,h,g);break;case 21:Bs(c,h,g);break;case 22:zn=(v=zn)||g.memoizedState!==null,Bs(c,h,g),zn=v;break;default:Bs(c,h,g)}}function GI(c,h){if(h.memoizedState===null&&(c=h.alternate,c!==null&&(c=c.memoizedState,c!==null))){c=c.dehydrated;try{Yd(c)}catch(g){Wr(h,h.return,g)}}}function jI(c,h){if(h.memoizedState===null&&(c=h.alternate,c!==null&&(c=c.memoizedState,c!==null&&(c=c.dehydrated,c!==null))))try{Yd(c)}catch(g){Wr(h,h.return,g)}}function ehe(c){switch(c.tag){case 31:case 13:case 19:var h=c.stateNode;return h===null&&(h=c.stateNode=new zI),h;case 22:return c=c.stateNode,h=c._retryCache,h===null&&(h=c._retryCache=new zI),h;default:throw Error(n(435,c.tag))}}function P0(c,h){var g=ehe(c);h.forEach(function(v){if(!g.has(v)){g.add(v);var A=uhe.bind(null,c,v);v.then(A,A)}})}function ei(c,h){var g=h.deletions;if(g!==null)for(var v=0;v<g.length;v++){var A=g[v],D=c,V=h,Q=V;e:for(;Q!==null;){switch(Q.tag){case 27:if(Xl(Q.type)){fn=Q.stateNode,Qa=!1;break e}break;case 5:fn=Q.stateNode,Qa=!1;break e;case 3:case 4:fn=Q.stateNode.containerInfo,Qa=!0;break e}Q=Q.return}if(fn===null)throw Error(n(160));$I(D,V,A),fn=null,Qa=!1,D=A.alternate,D!==null&&(D.return=null),A.return=null}if(h.subtreeFlags&13886)for(h=h.child;h!==null;)qI(h,c),h=h.sibling}var bo=null;function qI(c,h){var g=c.alternate,v=c.flags;switch(c.tag){case 0:case 11:case 14:case 15:ei(h,c),ti(c),v&4&&(jl(3,c,c.return),sf(3,c),jl(5,c,c.return));break;case 1:ei(h,c),ti(c),v&512&&(zn||g===null||jo(g,g.return)),v&64&&Os&&(c=c.updateQueue,c!==null&&(v=c.callbacks,v!==null&&(g=c.shared.hiddenCallbacks,c.shared.hiddenCallbacks=g===null?v:g.concat(v))));break;case 26:var A=bo;if(ei(h,c),ti(c),v&512&&(zn||g===null||jo(g,g.return)),v&4){var D=g!==null?g.memoizedState:null;if(v=c.memoizedState,g===null)if(v===null)if(c.stateNode===null){e:{v=c.type,g=c.memoizedProps,A=A.ownerDocument||A;t:switch(v){case"title":D=A.getElementsByTagName("title")[0],(!D||D[be]||D[Xr]||D.namespaceURI==="http://www.w3.org/2000/svg"||D.hasAttribute("itemprop"))&&(D=A.createElement(v),A.head.insertBefore(D,A.querySelector("head > title"))),fa(D,v,g),D[Xr]=c,Mr(D),v=D;break e;case"link":var V=qO("link","href",A).get(v+(g.href||""));if(V){for(var Q=0;Q<V.length;Q++)if(D=V[Q],D.getAttribute("href")===(g.href==null||g.href===""?null:g.href)&&D.getAttribute("rel")===(g.rel==null?null:g.rel)&&D.getAttribute("title")===(g.title==null?null:g.title)&&D.getAttribute("crossorigin")===(g.crossOrigin==null?null:g.crossOrigin)){V.splice(Q,1);break t}}D=A.createElement(v),fa(D,v,g),A.head.appendChild(D);break;case"meta":if(V=qO("meta","content",A).get(v+(g.content||""))){for(Q=0;Q<V.length;Q++)if(D=V[Q],D.getAttribute("content")===(g.content==null?null:""+g.content)&&D.getAttribute("name")===(g.name==null?null:g.name)&&D.getAttribute("property")===(g.property==null?null:g.property)&&D.getAttribute("http-equiv")===(g.httpEquiv==null?null:g.httpEquiv)&&D.getAttribute("charset")===(g.charSet==null?null:g.charSet)){V.splice(Q,1);break t}}D=A.createElement(v),fa(D,v,g),A.head.appendChild(D);break;default:throw Error(n(468,v))}D[Xr]=c,Mr(D),v=D}c.stateNode=v}else WO(A,c.type,c.stateNode);else c.stateNode=jO(A,v,c.memoizedProps);else D!==v?(D===null?g.stateNode!==null&&(g=g.stateNode,g.parentNode.removeChild(g)):D.count--,v===null?WO(A,c.type,c.stateNode):jO(A,v,c.memoizedProps)):v===null&&c.stateNode!==null&&FE(c,c.memoizedProps,g.memoizedProps)}break;case 27:ei(h,c),ti(c),v&512&&(zn||g===null||jo(g,g.return)),g!==null&&v&4&&FE(c,c.memoizedProps,g.memoizedProps);break;case 5:if(ei(h,c),ti(c),v&512&&(zn||g===null||jo(g,g.return)),c.flags&32){A=c.stateNode;try{yd(A,"")}catch(kt){Wr(c,c.return,kt)}}v&4&&c.stateNode!=null&&(A=c.memoizedProps,FE(c,A,g!==null?g.memoizedProps:A)),v&1024&&(DE=!0);break;case 6:if(ei(h,c),ti(c),v&4){if(c.stateNode===null)throw Error(n(162));v=c.memoizedProps,g=c.stateNode;try{g.nodeValue=v}catch(kt){Wr(c,c.return,kt)}}break;case 3:if(eb=null,A=bo,bo=Z0(h.containerInfo),ei(h,c),bo=A,ti(c),v&4&&g!==null&&g.memoizedState.isDehydrated)try{Yd(h.containerInfo)}catch(kt){Wr(c,c.return,kt)}DE&&(DE=!1,WI(c));break;case 4:v=bo,bo=Z0(c.stateNode.containerInfo),ei(h,c),ti(c),bo=v;break;case 12:ei(h,c),ti(c);break;case 31:ei(h,c),ti(c),v&4&&(v=c.updateQueue,v!==null&&(c.updateQueue=null,P0(c,v)));break;case 13:ei(h,c),ti(c),c.child.flags&8192&&c.memoizedState!==null!=(g!==null&&g.memoizedState!==null)&&(H0=Pe()),v&4&&(v=c.updateQueue,v!==null&&(c.updateQueue=null,P0(c,v)));break;case 22:A=c.memoizedState!==null;var de=g!==null&&g.memoizedState!==null,_e=Os,ze=zn;if(Os=_e||A,zn=ze||de,ei(h,c),zn=ze,Os=_e,ti(c),v&8192)e:for(h=c.stateNode,h._visibility=A?h._visibility&-2:h._visibility|1,A&&(g===null||de||Os||zn||oc(c)),g=null,h=c;;){if(h.tag===5||h.tag===26){if(g===null){de=g=h;try{if(D=de.stateNode,A)V=D.style,typeof V.setProperty=="function"?V.setProperty("display","none","important"):V.display="none";else{Q=de.stateNode;var Ue=de.memoizedProps.style,De=Ue!=null&&Ue.hasOwnProperty("display")?Ue.display:null;Q.style.display=De==null||typeof De=="boolean"?"":(""+De).trim()}}catch(kt){Wr(de,de.return,kt)}}}else if(h.tag===6){if(g===null){de=h;try{de.stateNode.nodeValue=A?"":de.memoizedProps}catch(kt){Wr(de,de.return,kt)}}}else if(h.tag===18){if(g===null){de=h;try{var Ie=de.stateNode;A?IO(Ie,!0):IO(de.stateNode,!1)}catch(kt){Wr(de,de.return,kt)}}}else if((h.tag!==22&&h.tag!==23||h.memoizedState===null||h===c)&&h.child!==null){h.child.return=h,h=h.child;continue}if(h===c)break e;for(;h.sibling===null;){if(h.return===null||h.return===c)break e;g===h&&(g=null),h=h.return}g===h&&(g=null),h.sibling.return=h.return,h=h.sibling}v&4&&(v=c.updateQueue,v!==null&&(g=v.retryQueue,g!==null&&(v.retryQueue=null,P0(c,g))));break;case 19:ei(h,c),ti(c),v&4&&(v=c.updateQueue,v!==null&&(c.updateQueue=null,P0(c,v)));break;case 30:break;case 21:break;default:ei(h,c),ti(c)}}function ti(c){var h=c.flags;if(h&2){try{for(var g,v=c.return;v!==null;){if(BI(v)){g=v;break}v=v.return}if(g==null)throw Error(n(160));switch(g.tag){case 27:var A=g.stateNode,D=_E(c);B0(c,D,A);break;case 5:var V=g.stateNode;g.flags&32&&(yd(V,""),g.flags&=-33);var Q=_E(c);B0(c,Q,V);break;case 3:case 4:var de=g.stateNode.containerInfo,_e=_E(c);RE(c,_e,de);break;default:throw Error(n(161))}}catch(ze){Wr(c,c.return,ze)}c.flags&=-3}h&4096&&(c.flags&=-4097)}function WI(c){if(c.subtreeFlags&1024)for(c=c.child;c!==null;){var h=c;WI(h),h.tag===5&&h.flags&1024&&h.stateNode.reset(),c=c.sibling}}function Ps(c,h){if(h.subtreeFlags&8772)for(h=h.child;h!==null;)HI(c,h.alternate,h),h=h.sibling}function oc(c){for(c=c.child;c!==null;){var h=c;switch(h.tag){case 0:case 11:case 14:case 15:jl(4,h,h.return),oc(h);break;case 1:jo(h,h.return);var g=h.stateNode;typeof g.componentWillUnmount=="function"&&II(h,h.return,g),oc(h);break;case 27:bf(h.stateNode);case 26:case 5:jo(h,h.return),oc(h);break;case 22:h.memoizedState===null&&oc(h);break;case 30:oc(h);break;default:oc(h)}c=c.sibling}}function zs(c,h,g){for(g=g&&(h.subtreeFlags&8772)!==0,h=h.child;h!==null;){var v=h.alternate,A=c,D=h,V=D.flags;switch(D.tag){case 0:case 11:case 15:zs(A,D,g),sf(4,D);break;case 1:if(zs(A,D,g),v=D,A=v.stateNode,typeof A.componentDidMount=="function")try{A.componentDidMount()}catch(_e){Wr(v,v.return,_e)}if(v=D,A=v.updateQueue,A!==null){var Q=v.stateNode;try{var de=A.shared.hiddenCallbacks;if(de!==null)for(A.shared.hiddenCallbacks=null,A=0;A<de.length;A++)wM(de[A],Q)}catch(_e){Wr(v,v.return,_e)}}g&&V&64&&MI(D),lf(D,D.return);break;case 27:PI(D);case 26:case 5:zs(A,D,g),g&&v===null&&V&4&&OI(D),lf(D,D.return);break;case 12:zs(A,D,g);break;case 31:zs(A,D,g),g&&V&4&&GI(A,D);break;case 13:zs(A,D,g),g&&V&4&&jI(A,D);break;case 22:D.memoizedState===null&&zs(A,D,g),lf(D,D.return);break;case 30:break;default:zs(A,D,g)}h=h.sibling}}function NE(c,h){var g=null;c!==null&&c.memoizedState!==null&&c.memoizedState.cachePool!==null&&(g=c.memoizedState.cachePool.pool),c=null,h.memoizedState!==null&&h.memoizedState.cachePool!==null&&(c=h.memoizedState.cachePool.pool),c!==g&&(c!=null&&c.refCount++,g!=null&&Vp(g))}function LE(c,h){c=null,h.alternate!==null&&(c=h.alternate.memoizedState.cache),h=h.memoizedState.cache,h!==c&&(h.refCount++,c!=null&&Vp(c))}function yo(c,h,g,v){if(h.subtreeFlags&10256)for(h=h.child;h!==null;)VI(c,h,g,v),h=h.sibling}function VI(c,h,g,v){var A=h.flags;switch(h.tag){case 0:case 11:case 15:yo(c,h,g,v),A&2048&&sf(9,h);break;case 1:yo(c,h,g,v);break;case 3:yo(c,h,g,v),A&2048&&(c=null,h.alternate!==null&&(c=h.alternate.memoizedState.cache),h=h.memoizedState.cache,h!==c&&(h.refCount++,c!=null&&Vp(c)));break;case 12:if(A&2048){yo(c,h,g,v),c=h.stateNode;try{var D=h.memoizedProps,V=D.id,Q=D.onPostCommit;typeof Q=="function"&&Q(V,h.alternate===null?"mount":"update",c.passiveEffectDuration,-0)}catch(de){Wr(h,h.return,de)}}else yo(c,h,g,v);break;case 31:yo(c,h,g,v);break;case 13:yo(c,h,g,v);break;case 23:break;case 22:D=h.stateNode,V=h.alternate,h.memoizedState!==null?D._visibility&2?yo(c,h,g,v):uf(c,h):D._visibility&2?yo(c,h,g,v):(D._visibility|=2,Pd(c,h,g,v,(h.subtreeFlags&10256)!==0||!1)),A&2048&&NE(V,h);break;case 24:yo(c,h,g,v),A&2048&&LE(h.alternate,h);break;default:yo(c,h,g,v)}}function Pd(c,h,g,v,A){for(A=A&&((h.subtreeFlags&10256)!==0||!1),h=h.child;h!==null;){var D=c,V=h,Q=g,de=v,_e=V.flags;switch(V.tag){case 0:case 11:case 15:Pd(D,V,Q,de,A),sf(8,V);break;case 23:break;case 22:var ze=V.stateNode;V.memoizedState!==null?ze._visibility&2?Pd(D,V,Q,de,A):uf(D,V):(ze._visibility|=2,Pd(D,V,Q,de,A)),A&&_e&2048&&NE(V.alternate,V);break;case 24:Pd(D,V,Q,de,A),A&&_e&2048&&LE(V.alternate,V);break;default:Pd(D,V,Q,de,A)}h=h.sibling}}function uf(c,h){if(h.subtreeFlags&10256)for(h=h.child;h!==null;){var g=c,v=h,A=v.flags;switch(v.tag){case 22:uf(g,v),A&2048&&NE(v.alternate,v);break;case 24:uf(g,v),A&2048&&LE(v.alternate,v);break;default:uf(g,v)}h=h.sibling}}var cf=8192;function zd(c,h,g){if(c.subtreeFlags&cf)for(c=c.child;c!==null;)KI(c,h,g),c=c.sibling}function KI(c,h,g){switch(c.tag){case 26:zd(c,h,g),c.flags&cf&&c.memoizedState!==null&&Hhe(g,bo,c.memoizedState,c.memoizedProps);break;case 5:zd(c,h,g);break;case 3:case 4:var v=bo;bo=Z0(c.stateNode.containerInfo),zd(c,h,g),bo=v;break;case 22:c.memoizedState===null&&(v=c.alternate,v!==null&&v.memoizedState!==null?(v=cf,cf=16777216,zd(c,h,g),cf=v):zd(c,h,g));break;default:zd(c,h,g)}}function YI(c){var h=c.alternate;if(h!==null&&(c=h.child,c!==null)){h.child=null;do h=c.sibling,c.sibling=null,c=h;while(c!==null)}}function df(c){var h=c.deletions;if((c.flags&16)!==0){if(h!==null)for(var g=0;g<h.length;g++){var v=h[g];aa=v,XI(v,c)}YI(c)}if(c.subtreeFlags&10256)for(c=c.child;c!==null;)JI(c),c=c.sibling}function JI(c){switch(c.tag){case 0:case 11:case 15:df(c),c.flags&2048&&jl(9,c,c.return);break;case 3:df(c);break;case 12:df(c);break;case 22:var h=c.stateNode;c.memoizedState!==null&&h._visibility&2&&(c.return===null||c.return.tag!==13)?(h._visibility&=-3,z0(c)):df(c);break;default:df(c)}}function z0(c){var h=c.deletions;if((c.flags&16)!==0){if(h!==null)for(var g=0;g<h.length;g++){var v=h[g];aa=v,XI(v,c)}YI(c)}for(c=c.child;c!==null;){switch(h=c,h.tag){case 0:case 11:case 15:jl(8,h,h.return),z0(h);break;case 22:g=h.stateNode,g._visibility&2&&(g._visibility&=-3,z0(h));break;default:z0(h)}c=c.sibling}}function XI(c,h){for(;aa!==null;){var g=aa;switch(g.tag){case 0:case 11:case 15:jl(8,g,h);break;case 23:case 22:if(g.memoizedState!==null&&g.memoizedState.cachePool!==null){var v=g.memoizedState.cachePool.pool;v!=null&&v.refCount++}break;case 24:Vp(g.memoizedState.cache)}if(v=g.child,v!==null)v.return=g,aa=v;else e:for(g=c;aa!==null;){v=aa;var A=v.sibling,D=v.return;if(UI(v),v===g){aa=null;break e}if(A!==null){A.return=D,aa=A;break e}aa=D}}}var the={getCacheForType:function(c){var h=ha(On),g=h.data.get(c);return g===void 0&&(g=c(),h.data.set(c,g)),g},cacheSignal:function(){return ha(On).controller.signal}},rhe=typeof WeakMap=="function"?WeakMap:Map,Ur=0,Zr=null,wr=null,Fr=0,qr=0,Ti=null,ql=!1,Hd=!1,ME=!1,Hs=0,An=0,Wl=0,sc=0,IE=0,Ai=0,Ud=0,hf=null,ri=null,OE=!1,H0=0,ZI=0,U0=1/0,$0=null,Vl=null,Kn=0,Kl=null,$d=null,Us=0,BE=0,PE=null,QI=null,pf=0,zE=null;function Fi(){return(Ur&2)!==0&&Fr!==0?Fr&-Fr:z.T!==null?qE():Ip()}function eO(){if(Ai===0)if((Fr&536870912)===0||Dr){var c=st;st<<=1,(st&3932160)===0&&(st=262144),Ai=c}else Ai=536870912;return c=ki.current,c!==null&&(c.flags|=32),Ai}function ni(c,h,g){(c===Zr&&(qr===2||qr===9)||c.cancelPendingCommit!==null)&&(Gd(c,0),Yl(c,Fr,Ai,!1)),hn(c,g),((Ur&2)===0||c!==Zr)&&(c===Zr&&((Ur&2)===0&&(sc|=g),An===4&&Yl(c,Fr,Ai,!1)),qo(c))}function tO(c,h,g){if((Ur&6)!==0)throw Error(n(327));var v=!g&&(h&127)===0&&(h&c.expiredLanes)===0||Qe(c,h),A=v?ihe(c,h):UE(c,h,!0),D=v;do{if(A===0){Hd&&!v&&Yl(c,h,0,!1);break}else{if(g=c.current.alternate,D&&!nhe(g)){A=UE(c,h,!1),D=!1;continue}if(A===2){if(D=h,c.errorRecoveryDisabledLanes&D)var V=0;else V=c.pendingLanes&-536870913,V=V!==0?V:V&536870912?536870912:0;if(V!==0){h=V;e:{var Q=c;A=hf;var de=Q.current.memoizedState.isDehydrated;if(de&&(Gd(Q,V).flags|=256),V=UE(Q,V,!1),V!==2){if(ME&&!de){Q.errorRecoveryDisabledLanes|=D,sc|=D,A=4;break e}D=ri,ri=A,D!==null&&(ri===null?ri=D:ri.push.apply(ri,D))}A=V}if(D=!1,A!==2)continue}}if(A===1){Gd(c,0),Yl(c,h,0,!0);break}e:{switch(v=c,D=A,D){case 0:case 1:throw Error(n(345));case 4:if((h&4194048)!==h)break;case 6:Yl(v,h,Ai,!ql);break e;case 2:ri=null;break;case 3:case 5:break;default:throw Error(n(329))}if((h&62914560)===h&&(A=H0+300-Pe(),10<A)){if(Yl(v,h,Ai,!ql),yr(v,0,!0)!==0)break e;Us=h,v.timeoutHandle=NO(rO.bind(null,v,g,ri,$0,OE,h,Ai,sc,Ud,ql,D,"Throttled",-0,0),A);break e}rO(v,g,ri,$0,OE,h,Ai,sc,Ud,ql,D,null,-0,0)}}break}while(!0);qo(c)}function rO(c,h,g,v,A,D,V,Q,de,_e,ze,Ue,De,Ie){if(c.timeoutHandle=-1,Ue=h.subtreeFlags,Ue&8192||(Ue&16785408)===16785408){Ue={stylesheets:null,count:0,imgCount:0,imgBytes:0,suspenseyImages:[],waitingForImages:!0,waitingForViewTransition:!1,unsuspend:Ts},KI(h,D,Ue);var kt=(D&62914560)===D?H0-Pe():(D&4194048)===D?ZI-Pe():0;if(kt=Uhe(Ue,kt),kt!==null){Us=D,c.cancelPendingCommit=kt(cO.bind(null,c,h,D,g,v,A,V,Q,de,ze,Ue,null,De,Ie)),Yl(c,D,V,!_e);return}}cO(c,h,D,g,v,A,V,Q,de)}function nhe(c){for(var h=c;;){var g=h.tag;if((g===0||g===11||g===15)&&h.flags&16384&&(g=h.updateQueue,g!==null&&(g=g.stores,g!==null)))for(var v=0;v<g.length;v++){var A=g[v],D=A.getSnapshot;A=A.value;try{if(!xi(D(),A))return!1}catch{return!1}}if(g=h.child,h.subtreeFlags&16384&&g!==null)g.return=h,h=g;else{if(h===c)break;for(;h.sibling===null;){if(h.return===null||h.return===c)return!0;h=h.return}h.sibling.return=h.return,h=h.sibling}}return!0}function Yl(c,h,g,v){h&=~IE,h&=~sc,c.suspendedLanes|=h,c.pingedLanes&=~h,v&&(c.warmLanes|=h),v=c.expirationTimes;for(var A=h;0<A;){var D=31-qt(A),V=1<<D;v[D]=-1,A&=~V}g!==0&&xa(c,g,h)}function G0(){return(Ur&6)===0?(ff(0),!1):!0}function HE(){if(wr!==null){if(qr===0)var c=wr.return;else c=wr,Rs=Zu=null,rE(c),Ld=null,Yp=0,c=wr;for(;c!==null;)LI(c.alternate,c),c=c.return;wr=null}}function Gd(c,h){var g=c.timeoutHandle;g!==-1&&(c.timeoutHandle=-1,whe(g)),g=c.cancelPendingCommit,g!==null&&(c.cancelPendingCommit=null,g()),Us=0,HE(),Zr=c,wr=g=Fs(c.current,null),Fr=h,qr=0,Ti=null,ql=!1,Hd=Qe(c,h),ME=!1,Ud=Ai=IE=sc=Wl=An=0,ri=hf=null,OE=!1,(h&8)!==0&&(h|=h&32);var v=c.entangledLanes;if(v!==0)for(c=c.entanglements,v&=h;0<v;){var A=31-qt(v),D=1<<A;h|=c[A],v&=~D}return Hs=h,c0(),g}function nO(c,h){dr=null,z.H=nf,h===Nd||h===y0?(h=vM(),qr=3):h===jS?(h=vM(),qr=4):qr=h===yE?8:h!==null&&typeof h=="object"&&typeof h.then=="function"?6:1,Ti=h,wr===null&&(An=1,N0(c,Wi(h,c.current)))}function aO(){var c=ki.current;return c===null?!0:(Fr&4194048)===Fr?Ji===null:(Fr&62914560)===Fr||(Fr&536870912)!==0?c===Ji:!1}function iO(){var c=z.H;return z.H=nf,c===null?nf:c}function oO(){var c=z.A;return z.A=the,c}function j0(){An=4,ql||(Fr&4194048)!==Fr&&ki.current!==null||(Hd=!0),(Wl&134217727)===0&&(sc&134217727)===0||Zr===null||Yl(Zr,Fr,Ai,!1)}function UE(c,h,g){var v=Ur;Ur|=2;var A=iO(),D=oO();(Zr!==c||Fr!==h)&&($0=null,Gd(c,h)),h=!1;var V=An;e:do try{if(qr!==0&&wr!==null){var Q=wr,de=Ti;switch(qr){case 8:HE(),V=6;break e;case 3:case 2:case 9:case 6:ki.current===null&&(h=!0);var _e=qr;if(qr=0,Ti=null,jd(c,Q,de,_e),g&&Hd){V=0;break e}break;default:_e=qr,qr=0,Ti=null,jd(c,Q,de,_e)}}ahe(),V=An;break}catch(ze){nO(c,ze)}while(!0);return h&&c.shellSuspendCounter++,Rs=Zu=null,Ur=v,z.H=A,z.A=D,wr===null&&(Zr=null,Fr=0,c0()),V}function ahe(){for(;wr!==null;)sO(wr)}function ihe(c,h){var g=Ur;Ur|=2;var v=iO(),A=oO();Zr!==c||Fr!==h?($0=null,U0=Pe()+500,Gd(c,h)):Hd=Qe(c,h);e:do try{if(qr!==0&&wr!==null){h=wr;var D=Ti;t:switch(qr){case 1:qr=0,Ti=null,jd(c,h,D,1);break;case 2:case 9:if(bM(D)){qr=0,Ti=null,lO(h);break}h=function(){qr!==2&&qr!==9||Zr!==c||(qr=7),qo(c)},D.then(h,h);break e;case 3:qr=7;break e;case 4:qr=5;break e;case 7:bM(D)?(qr=0,Ti=null,lO(h)):(qr=0,Ti=null,jd(c,h,D,7));break;case 5:var V=null;switch(wr.tag){case 26:V=wr.memoizedState;case 5:case 27:var Q=wr;if(V?VO(V):Q.stateNode.complete){qr=0,Ti=null;var de=Q.sibling;if(de!==null)wr=de;else{var _e=Q.return;_e!==null?(wr=_e,q0(_e)):wr=null}break t}}qr=0,Ti=null,jd(c,h,D,5);break;case 6:qr=0,Ti=null,jd(c,h,D,6);break;case 8:HE(),An=6;break e;default:throw Error(n(462))}}ohe();break}catch(ze){nO(c,ze)}while(!0);return Rs=Zu=null,z.H=v,z.A=A,Ur=g,wr!==null?0:(Zr=null,Fr=0,c0(),An)}function ohe(){for(;wr!==null&&!Qt();)sO(wr)}function sO(c){var h=DI(c.alternate,c,Hs);c.memoizedProps=c.pendingProps,h===null?q0(c):wr=h}function lO(c){var h=c,g=h.alternate;switch(h.tag){case 15:case 0:h=CI(g,h,h.pendingProps,h.type,void 0,Fr);break;case 11:h=CI(g,h,h.pendingProps,h.type.render,h.ref,Fr);break;case 5:rE(h);default:LI(g,h),h=wr=oM(h,Hs),h=DI(g,h,Hs)}c.memoizedProps=c.pendingProps,h===null?q0(c):wr=h}function jd(c,h,g,v){Rs=Zu=null,rE(h),Ld=null,Yp=0;var A=h.return;try{if(Kde(c,A,h,g,Fr)){An=1,N0(c,Wi(g,c.current)),wr=null;return}}catch(D){if(A!==null)throw wr=A,D;An=1,N0(c,Wi(g,c.current)),wr=null;return}h.flags&32768?(Dr||v===1?c=!0:Hd||(Fr&536870912)!==0?c=!1:(ql=c=!0,(v===2||v===9||v===3||v===6)&&(v=ki.current,v!==null&&v.tag===13&&(v.flags|=16384))),uO(h,c)):q0(h)}function q0(c){var h=c;do{if((h.flags&32768)!==0){uO(h,ql);return}c=h.return;var g=Xde(h.alternate,h,Hs);if(g!==null){wr=g;return}if(h=h.sibling,h!==null){wr=h;return}wr=h=c}while(h!==null);An===0&&(An=5)}function uO(c,h){do{var g=Zde(c.alternate,c);if(g!==null){g.flags&=32767,wr=g;return}if(g=c.return,g!==null&&(g.flags|=32768,g.subtreeFlags=0,g.deletions=null),!h&&(c=c.sibling,c!==null)){wr=c;return}wr=c=g}while(c!==null);An=6,wr=null}function cO(c,h,g,v,A,D,V,Q,de){c.cancelPendingCommit=null;do W0();while(Kn!==0);if((Ur&6)!==0)throw Error(n(327));if(h!==null){if(h===c.current)throw Error(n(177));if(D=h.lanes|h.childLanes,D|=_S,Ea(c,g,D,V,Q,de),c===Zr&&(wr=Zr=null,Fr=0),$d=h,Kl=c,Us=g,BE=D,PE=A,QI=v,(h.subtreeFlags&10256)!==0||(h.flags&10256)!==0?(c.callbackNode=null,c.callbackPriority=0,che(We,function(){return mO(),null})):(c.callbackNode=null,c.callbackPriority=0),v=(h.flags&13878)!==0,(h.subtreeFlags&13878)!==0||v){v=z.T,z.T=null,A=H.p,H.p=2,V=Ur,Ur|=4;try{Qde(c,h,g)}finally{Ur=V,H.p=A,z.T=v}}Kn=1,dO(),hO(),pO()}}function dO(){if(Kn===1){Kn=0;var c=Kl,h=$d,g=(h.flags&13878)!==0;if((h.subtreeFlags&13878)!==0||g){g=z.T,z.T=null;var v=H.p;H.p=2;var A=Ur;Ur|=4;try{qI(h,c);var D=QE,V=XL(c.containerInfo),Q=D.focusedElem,de=D.selectionRange;if(V!==Q&&Q&&Q.ownerDocument&&JL(Q.ownerDocument.documentElement,Q)){if(de!==null&&kS(Q)){var _e=de.start,ze=de.end;if(ze===void 0&&(ze=_e),"selectionStart"in Q)Q.selectionStart=_e,Q.selectionEnd=Math.min(ze,Q.value.length);else{var Ue=Q.ownerDocument||document,De=Ue&&Ue.defaultView||window;if(De.getSelection){var Ie=De.getSelection(),kt=Q.textContent.length,Yt=Math.min(de.start,kt),Yr=de.end===void 0?Yt:Math.min(de.end,kt);!Ie.extend&&Yt>Yr&&(V=Yr,Yr=Yt,Yt=V);var ve=YL(Q,Yt),me=YL(Q,Yr);if(ve&&me&&(Ie.rangeCount!==1||Ie.anchorNode!==ve.node||Ie.anchorOffset!==ve.offset||Ie.focusNode!==me.node||Ie.focusOffset!==me.offset)){var Ae=Ue.createRange();Ae.setStart(ve.node,ve.offset),Ie.removeAllRanges(),Yt>Yr?(Ie.addRange(Ae),Ie.extend(me.node,me.offset)):(Ae.setEnd(me.node,me.offset),Ie.addRange(Ae))}}}}for(Ue=[],Ie=Q;Ie=Ie.parentNode;)Ie.nodeType===1&&Ue.push({element:Ie,left:Ie.scrollLeft,top:Ie.scrollTop});for(typeof Q.focus=="function"&&Q.focus(),Q=0;Q<Ue.length;Q++){var He=Ue[Q];He.element.scrollLeft=He.left,He.element.scrollTop=He.top}}ab=!!ZE,QE=ZE=null}finally{Ur=A,H.p=v,z.T=g}}c.current=h,Kn=2}}function hO(){if(Kn===2){Kn=0;var c=Kl,h=$d,g=(h.flags&8772)!==0;if((h.subtreeFlags&8772)!==0||g){g=z.T,z.T=null;var v=H.p;H.p=2;var A=Ur;Ur|=4;try{HI(c,h.alternate,h)}finally{Ur=A,H.p=v,z.T=g}}Kn=3}}function pO(){if(Kn===4||Kn===3){Kn=0,or();var c=Kl,h=$d,g=Us,v=QI;(h.subtreeFlags&10256)!==0||(h.flags&10256)!==0?Kn=5:(Kn=0,$d=Kl=null,fO(c,c.pendingLanes));var A=c.pendingLanes;if(A===0&&(Vl=null),fd(g),h=h.stateNode,Mt&&typeof Mt.onCommitFiberRoot=="function")try{Mt.onCommitFiberRoot(fr,h,void 0,(h.current.flags&128)===128)}catch{}if(v!==null){h=z.T,A=H.p,H.p=2,z.T=null;try{for(var D=c.onRecoverableError,V=0;V<v.length;V++){var Q=v[V];D(Q.value,{componentStack:Q.stack})}}finally{z.T=h,H.p=A}}(Us&3)!==0&&W0(),qo(c),A=c.pendingLanes,(g&261930)!==0&&(A&42)!==0?c===zE?pf++:(pf=0,zE=c):pf=0,ff(0)}}function fO(c,h){(c.pooledCacheLanes&=h)===0&&(h=c.pooledCache,h!=null&&(c.pooledCache=null,Vp(h)))}function W0(){return dO(),hO(),pO(),mO()}function mO(){if(Kn!==5)return!1;var c=Kl,h=BE;BE=0;var g=fd(Us),v=z.T,A=H.p;try{H.p=32>g?32:g,z.T=null,g=PE,PE=null;var D=Kl,V=Us;if(Kn=0,$d=Kl=null,Us=0,(Ur&6)!==0)throw Error(n(331));var Q=Ur;if(Ur|=4,JI(D.current),VI(D,D.current,V,g),Ur=Q,ff(0,!1),Mt&&typeof Mt.onPostCommitFiberRoot=="function")try{Mt.onPostCommitFiberRoot(fr,D)}catch{}return!0}finally{H.p=A,z.T=v,fO(c,h)}}function gO(c,h,g){h=Wi(g,h),h=bE(c.stateNode,h,2),c=Ul(c,h,2),c!==null&&(hn(c,2),qo(c))}function Wr(c,h,g){if(c.tag===3)gO(c,c,g);else for(;h!==null;){if(h.tag===3){gO(h,c,g);break}else if(h.tag===1){var v=h.stateNode;if(typeof h.type.getDerivedStateFromError=="function"||typeof v.componentDidCatch=="function"&&(Vl===null||!Vl.has(v))){c=Wi(g,c),g=bI(2),v=Ul(h,g,2),v!==null&&(yI(g,v,h,c),hn(v,2),qo(v));break}}h=h.return}}function $E(c,h,g){var v=c.pingCache;if(v===null){v=c.pingCache=new rhe;var A=new Set;v.set(h,A)}else A=v.get(h),A===void 0&&(A=new Set,v.set(h,A));A.has(g)||(ME=!0,A.add(g),c=she.bind(null,c,h,g),h.then(c,c))}function she(c,h,g){var v=c.pingCache;v!==null&&v.delete(h),c.pingedLanes|=c.suspendedLanes&g,c.warmLanes&=~g,Zr===c&&(Fr&g)===g&&(An===4||An===3&&(Fr&62914560)===Fr&&300>Pe()-H0?(Ur&2)===0&&Gd(c,0):IE|=g,Ud===Fr&&(Ud=0)),qo(c)}function bO(c,h){h===0&&(h=Nr()),c=Yu(c,h),c!==null&&(hn(c,h),qo(c))}function lhe(c){var h=c.memoizedState,g=0;h!==null&&(g=h.retryLane),bO(c,g)}function uhe(c,h){var g=0;switch(c.tag){case 31:case 13:var v=c.stateNode,A=c.memoizedState;A!==null&&(g=A.retryLane);break;case 19:v=c.stateNode;break;case 22:v=c.stateNode._retryCache;break;default:throw Error(n(314))}v!==null&&v.delete(h),bO(c,g)}function che(c,h){return Ye(c,h)}var V0=null,qd=null,GE=!1,K0=!1,jE=!1,Jl=0;function qo(c){c!==qd&&c.next===null&&(qd===null?V0=qd=c:qd=qd.next=c),K0=!0,GE||(GE=!0,hhe())}function ff(c,h){if(!jE&&K0){jE=!0;do for(var g=!1,v=V0;v!==null;){if(c!==0){var A=v.pendingLanes;if(A===0)var D=0;else{var V=v.suspendedLanes,Q=v.pingedLanes;D=(1<<31-qt(42|c)+1)-1,D&=A&~(V&~Q),D=D&201326741?D&201326741|1:D?D|2:0}D!==0&&(g=!0,EO(v,D))}else D=Fr,D=yr(v,v===Zr?D:0,v.cancelPendingCommit!==null||v.timeoutHandle!==-1),(D&3)===0||Qe(v,D)||(g=!0,EO(v,D));v=v.next}while(g);jE=!1}}function dhe(){yO()}function yO(){K0=GE=!1;var c=0;Jl!==0&&xhe()&&(c=Jl);for(var h=Pe(),g=null,v=V0;v!==null;){var A=v.next,D=vO(v,h);D===0?(v.next=null,g===null?V0=A:g.next=A,A===null&&(qd=g)):(g=v,(c!==0||(D&3)!==0)&&(K0=!0)),v=A}Kn!==0&&Kn!==5||ff(c),Jl!==0&&(Jl=0)}function vO(c,h){for(var g=c.suspendedLanes,v=c.pingedLanes,A=c.expirationTimes,D=c.pendingLanes&-62914561;0<D;){var V=31-qt(D),Q=1<<V,de=A[V];de===-1?((Q&g)===0||(Q&v)!==0)&&(A[V]=zt(Q,h)):de<=h&&(c.expiredLanes|=Q),D&=~Q}if(h=Zr,g=Fr,g=yr(c,c===h?g:0,c.cancelPendingCommit!==null||c.timeoutHandle!==-1),v=c.callbackNode,g===0||c===h&&(qr===2||qr===9)||c.cancelPendingCommit!==null)return v!==null&&v!==null&&we(v),c.callbackNode=null,c.callbackPriority=0;if((g&3)===0||Qe(c,g)){if(h=g&-g,h===c.callbackPriority)return h;switch(v!==null&&we(v),fd(g)){case 2:case 8:g=Re;break;case 32:g=We;break;case 268435456:g=xt;break;default:g=We}return v=SO.bind(null,c),g=Ye(g,v),c.callbackPriority=h,c.callbackNode=g,h}return v!==null&&v!==null&&we(v),c.callbackPriority=2,c.callbackNode=null,2}function SO(c,h){if(Kn!==0&&Kn!==5)return c.callbackNode=null,c.callbackPriority=0,null;var g=c.callbackNode;if(W0()&&c.callbackNode!==g)return null;var v=Fr;return v=yr(c,c===Zr?v:0,c.cancelPendingCommit!==null||c.timeoutHandle!==-1),v===0?null:(tO(c,v,h),vO(c,Pe()),c.callbackNode!=null&&c.callbackNode===g?SO.bind(null,c):null)}function EO(c,h){if(W0())return null;tO(c,h,!0)}function hhe(){khe(function(){(Ur&6)!==0?Ye(ge,dhe):yO()})}function qE(){if(Jl===0){var c=Rd;c===0&&(c=Ce,Ce<<=1,(Ce&261888)===0&&(Ce=256)),Jl=c}return Jl}function xO(c){return c==null||typeof c=="symbol"||typeof c=="boolean"?null:typeof c=="function"?c:r0(""+c)}function wO(c,h){var g=h.ownerDocument.createElement("input");return g.name=h.name,g.value=h.value,c.id&&g.setAttribute("form",c.id),h.parentNode.insertBefore(g,h),c=new FormData(c),g.parentNode.removeChild(g),c}function phe(c,h,g,v,A){if(h==="submit"&&g&&g.stateNode===A){var D=xO((A[ca]||null).action),V=v.submitter;V&&(h=(h=V[ca]||null)?xO(h.formAction):V.getAttribute("formAction"),h!==null&&(D=h,V=null));var Q=new o0("action","action",null,v,A);c.push({event:Q,listeners:[{instance:null,listener:function(){if(v.defaultPrevented){if(Jl!==0){var de=V?wO(A,V):new FormData(A);dE(g,{pending:!0,data:de,method:A.method,action:D},null,de)}}else typeof D=="function"&&(Q.preventDefault(),de=V?wO(A,V):new FormData(A),dE(g,{pending:!0,data:de,method:A.method,action:D},D,de))},currentTarget:A}]})}}for(var WE=0;WE<FS.length;WE++){var VE=FS[WE],fhe=VE.toLowerCase(),mhe=VE[0].toUpperCase()+VE.slice(1);go(fhe,"on"+mhe)}go(eM,"onAnimationEnd"),go(tM,"onAnimationIteration"),go(rM,"onAnimationStart"),go("dblclick","onDoubleClick"),go("focusin","onFocus"),go("focusout","onBlur"),go(Dde,"onTransitionRun"),go(Nde,"onTransitionStart"),go(Lde,"onTransitionCancel"),go(nM,"onTransitionEnd"),ks("onMouseEnter",["mouseout","mouseover"]),ks("onMouseLeave",["mouseout","mouseover"]),ks("onPointerEnter",["pointerout","pointerover"]),ks("onPointerLeave",["pointerout","pointerover"]),vn("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),vn("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),vn("onBeforeInput",["compositionend","keypress","textInput","paste"]),vn("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),vn("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),vn("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var mf="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),ghe=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(mf));function kO(c,h){h=(h&4)!==0;for(var g=0;g<c.length;g++){var v=c[g],A=v.event;v=v.listeners;e:{var D=void 0;if(h)for(var V=v.length-1;0<=V;V--){var Q=v[V],de=Q.instance,_e=Q.currentTarget;if(Q=Q.listener,de!==D&&A.isPropagationStopped())break e;D=Q,A.currentTarget=_e;try{D(A)}catch(ze){u0(ze)}A.currentTarget=null,D=de}else for(V=0;V<v.length;V++){if(Q=v[V],de=Q.instance,_e=Q.currentTarget,Q=Q.listener,de!==D&&A.isPropagationStopped())break e;D=Q,A.currentTarget=_e;try{D(A)}catch(ze){u0(ze)}A.currentTarget=null,D=de}}}}function kr(c,h){var g=h[gd];g===void 0&&(g=h[gd]=new Set);var v=c+"__bubble";g.has(v)||(CO(h,c,2,!1),g.add(v))}function KE(c,h,g){var v=0;h&&(v|=4),CO(g,c,v,h)}var Y0="_reactListening"+Math.random().toString(36).slice(2);function YE(c){if(!c[Y0]){c[Y0]=!0,Ei.forEach(function(g){g!=="selectionchange"&&(ghe.has(g)||KE(g,!1,c),KE(g,!0,c))});var h=c.nodeType===9?c:c.ownerDocument;h===null||h[Y0]||(h[Y0]=!0,KE("selectionchange",!1,h))}}function CO(c,h,g,v){switch(e9(h)){case 2:var A=jhe;break;case 8:A=qhe;break;default:A=cx}g=A.bind(null,h,g,c),A=void 0,!mS||h!=="touchstart"&&h!=="touchmove"&&h!=="wheel"||(A=!0),v?A!==void 0?c.addEventListener(h,g,{capture:!0,passive:A}):c.addEventListener(h,g,!0):A!==void 0?c.addEventListener(h,g,{passive:A}):c.addEventListener(h,g,!1)}function JE(c,h,g,v,A){var D=v;if((h&1)===0&&(h&2)===0&&v!==null)e:for(;;){if(v===null)return;var V=v.tag;if(V===3||V===4){var Q=v.stateNode.containerInfo;if(Q===A)break;if(V===4)for(V=v.return;V!==null;){var de=V.tag;if((de===3||de===4)&&V.stateNode.containerInfo===A)return;V=V.return}for(;Q!==null;){if(V=wt(Q),V===null)return;if(de=V.tag,de===5||de===6||de===26||de===27){v=D=V;continue e}Q=Q.parentNode}}v=v.return}RL(function(){var _e=D,ze=pS(g),Ue=[];e:{var De=aM.get(c);if(De!==void 0){var Ie=o0,kt=c;switch(c){case"keypress":if(a0(g)===0)break e;case"keydown":case"keyup":Ie=ude;break;case"focusin":kt="focus",Ie=vS;break;case"focusout":kt="blur",Ie=vS;break;case"beforeblur":case"afterblur":Ie=vS;break;case"click":if(g.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":Ie=LL;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":Ie=Xce;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":Ie=hde;break;case eM:case tM:case rM:Ie=ede;break;case nM:Ie=fde;break;case"scroll":case"scrollend":Ie=Yce;break;case"wheel":Ie=gde;break;case"copy":case"cut":case"paste":Ie=rde;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":Ie=IL;break;case"toggle":case"beforetoggle":Ie=yde}var Yt=(h&4)!==0,Yr=!Yt&&(c==="scroll"||c==="scrollend"),ve=Yt?De!==null?De+"Capture":null:De;Yt=[];for(var me=_e,Ae;me!==null;){var He=me;if(Ae=He.stateNode,He=He.tag,He!==5&&He!==26&&He!==27||Ae===null||ve===null||(He=Op(me,ve),He!=null&&Yt.push(gf(me,He,Ae))),Yr)break;me=me.return}0<Yt.length&&(De=new Ie(De,kt,null,g,ze),Ue.push({event:De,listeners:Yt}))}}if((h&7)===0){e:{if(De=c==="mouseover"||c==="pointerover",Ie=c==="mouseout"||c==="pointerout",De&&g!==hS&&(kt=g.relatedTarget||g.fromElement)&&(wt(kt)||kt[ws]))break e;if((Ie||De)&&(De=ze.window===ze?ze:(De=ze.ownerDocument)?De.defaultView||De.parentWindow:window,Ie?(kt=g.relatedTarget||g.toElement,Ie=_e,kt=kt?wt(kt):null,kt!==null&&(Yr=i(kt),Yt=kt.tag,kt!==Yr||Yt!==5&&Yt!==27&&Yt!==6)&&(kt=null)):(Ie=null,kt=_e),Ie!==kt)){if(Yt=LL,He="onMouseLeave",ve="onMouseEnter",me="mouse",(c==="pointerout"||c==="pointerover")&&(Yt=IL,He="onPointerLeave",ve="onPointerEnter",me="pointer"),Yr=Ie==null?De:Cn(Ie),Ae=kt==null?De:Cn(kt),De=new Yt(He,me+"leave",Ie,g,ze),De.target=Yr,De.relatedTarget=Ae,He=null,wt(ze)===_e&&(Yt=new Yt(ve,me+"enter",kt,g,ze),Yt.target=Ae,Yt.relatedTarget=Yr,He=Yt),Yr=He,Ie&&kt)t:{for(Yt=bhe,ve=Ie,me=kt,Ae=0,He=ve;He;He=Yt(He))Ae++;He=0;for(var Ht=me;Ht;Ht=Yt(Ht))He++;for(;0<Ae-He;)ve=Yt(ve),Ae--;for(;0<He-Ae;)me=Yt(me),He--;for(;Ae--;){if(ve===me||me!==null&&ve===me.alternate){Yt=ve;break t}ve=Yt(ve),me=Yt(me)}Yt=null}else Yt=null;Ie!==null&&TO(Ue,De,Ie,Yt,!1),kt!==null&&Yr!==null&&TO(Ue,Yr,kt,Yt,!0)}}e:{if(De=_e?Cn(_e):window,Ie=De.nodeName&&De.nodeName.toLowerCase(),Ie==="select"||Ie==="input"&&De.type==="file")var Or=GL;else if(UL(De))if(jL)Or=Fde;else{Or=Tde;var It=Cde}else Ie=De.nodeName,!Ie||Ie.toLowerCase()!=="input"||De.type!=="checkbox"&&De.type!=="radio"?_e&&dS(_e.elementType)&&(Or=GL):Or=Ade;if(Or&&(Or=Or(c,_e))){$L(Ue,Or,g,ze);break e}It&&It(c,De,_e),c==="focusout"&&_e&&De.type==="number"&&_e.memoizedProps.value!=null&&cS(De,"number",De.value)}switch(It=_e?Cn(_e):window,c){case"focusin":(UL(It)||It.contentEditable==="true")&&(xd=It,CS=_e,jp=null);break;case"focusout":jp=CS=xd=null;break;case"mousedown":TS=!0;break;case"contextmenu":case"mouseup":case"dragend":TS=!1,ZL(Ue,g,ze);break;case"selectionchange":if(Rde)break;case"keydown":case"keyup":ZL(Ue,g,ze)}var mr;if(ES)e:{switch(c){case"compositionstart":var _r="onCompositionStart";break e;case"compositionend":_r="onCompositionEnd";break e;case"compositionupdate":_r="onCompositionUpdate";break e}_r=void 0}else Ed?zL(c,g)&&(_r="onCompositionEnd"):c==="keydown"&&g.keyCode===229&&(_r="onCompositionStart");_r&&(OL&&g.locale!=="ko"&&(Ed||_r!=="onCompositionStart"?_r==="onCompositionEnd"&&Ed&&(mr=DL()):(Ml=ze,gS="value"in Ml?Ml.value:Ml.textContent,Ed=!0)),It=J0(_e,_r),0<It.length&&(_r=new ML(_r,c,null,g,ze),Ue.push({event:_r,listeners:It}),mr?_r.data=mr:(mr=HL(g),mr!==null&&(_r.data=mr)))),(mr=Sde?Ede(c,g):xde(c,g))&&(_r=J0(_e,"onBeforeInput"),0<_r.length&&(It=new ML("onBeforeInput","beforeinput",null,g,ze),Ue.push({event:It,listeners:_r}),It.data=mr)),phe(Ue,c,_e,g,ze)}kO(Ue,h)})}function gf(c,h,g){return{instance:c,listener:h,currentTarget:g}}function J0(c,h){for(var g=h+"Capture",v=[];c!==null;){var A=c,D=A.stateNode;if(A=A.tag,A!==5&&A!==26&&A!==27||D===null||(A=Op(c,g),A!=null&&v.unshift(gf(c,A,D)),A=Op(c,h),A!=null&&v.push(gf(c,A,D))),c.tag===3)return v;c=c.return}return[]}function bhe(c){if(c===null)return null;do c=c.return;while(c&&c.tag!==5&&c.tag!==27);return c||null}function TO(c,h,g,v,A){for(var D=h._reactName,V=[];g!==null&&g!==v;){var Q=g,de=Q.alternate,_e=Q.stateNode;if(Q=Q.tag,de!==null&&de===v)break;Q!==5&&Q!==26&&Q!==27||_e===null||(de=_e,A?(_e=Op(g,D),_e!=null&&V.unshift(gf(g,_e,de))):A||(_e=Op(g,D),_e!=null&&V.push(gf(g,_e,de)))),g=g.return}V.length!==0&&c.push({event:h,listeners:V})}var yhe=/\r\n?/g,vhe=/\u0000|\uFFFD/g;function AO(c){return(typeof c=="string"?c:""+c).replace(yhe,`
`).replace(vhe,"")}function FO(c,h){return h=AO(h),AO(c)===h}function Kr(c,h,g,v,A,D){switch(g){case"children":typeof v=="string"?h==="body"||h==="textarea"&&v===""||yd(c,v):(typeof v=="number"||typeof v=="bigint")&&h!=="body"&&yd(c,""+v);break;case"className":e0(c,"class",v);break;case"tabIndex":e0(c,"tabindex",v);break;case"dir":case"role":case"viewBox":case"width":case"height":e0(c,g,v);break;case"style":FL(c,v,D);break;case"data":if(h!=="object"){e0(c,"data",v);break}case"src":case"href":if(v===""&&(h!=="a"||g!=="href")){c.removeAttribute(g);break}if(v==null||typeof v=="function"||typeof v=="symbol"||typeof v=="boolean"){c.removeAttribute(g);break}v=r0(""+v),c.setAttribute(g,v);break;case"action":case"formAction":if(typeof v=="function"){c.setAttribute(g,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof D=="function"&&(g==="formAction"?(h!=="input"&&Kr(c,h,"name",A.name,A,null),Kr(c,h,"formEncType",A.formEncType,A,null),Kr(c,h,"formMethod",A.formMethod,A,null),Kr(c,h,"formTarget",A.formTarget,A,null)):(Kr(c,h,"encType",A.encType,A,null),Kr(c,h,"method",A.method,A,null),Kr(c,h,"target",A.target,A,null)));if(v==null||typeof v=="symbol"||typeof v=="boolean"){c.removeAttribute(g);break}v=r0(""+v),c.setAttribute(g,v);break;case"onClick":v!=null&&(c.onclick=Ts);break;case"onScroll":v!=null&&kr("scroll",c);break;case"onScrollEnd":v!=null&&kr("scrollend",c);break;case"dangerouslySetInnerHTML":if(v!=null){if(typeof v!="object"||!("__html"in v))throw Error(n(61));if(g=v.__html,g!=null){if(A.children!=null)throw Error(n(60));c.innerHTML=g}}break;case"multiple":c.multiple=v&&typeof v!="function"&&typeof v!="symbol";break;case"muted":c.muted=v&&typeof v!="function"&&typeof v!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(v==null||typeof v=="function"||typeof v=="boolean"||typeof v=="symbol"){c.removeAttribute("xlink:href");break}g=r0(""+v),c.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",g);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":v!=null&&typeof v!="function"&&typeof v!="symbol"?c.setAttribute(g,""+v):c.removeAttribute(g);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":v&&typeof v!="function"&&typeof v!="symbol"?c.setAttribute(g,""):c.removeAttribute(g);break;case"capture":case"download":v===!0?c.setAttribute(g,""):v!==!1&&v!=null&&typeof v!="function"&&typeof v!="symbol"?c.setAttribute(g,v):c.removeAttribute(g);break;case"cols":case"rows":case"size":case"span":v!=null&&typeof v!="function"&&typeof v!="symbol"&&!isNaN(v)&&1<=v?c.setAttribute(g,v):c.removeAttribute(g);break;case"rowSpan":case"start":v==null||typeof v=="function"||typeof v=="symbol"||isNaN(v)?c.removeAttribute(g):c.setAttribute(g,v);break;case"popover":kr("beforetoggle",c),kr("toggle",c),Qg(c,"popover",v);break;case"xlinkActuate":Cs(c,"http://www.w3.org/1999/xlink","xlink:actuate",v);break;case"xlinkArcrole":Cs(c,"http://www.w3.org/1999/xlink","xlink:arcrole",v);break;case"xlinkRole":Cs(c,"http://www.w3.org/1999/xlink","xlink:role",v);break;case"xlinkShow":Cs(c,"http://www.w3.org/1999/xlink","xlink:show",v);break;case"xlinkTitle":Cs(c,"http://www.w3.org/1999/xlink","xlink:title",v);break;case"xlinkType":Cs(c,"http://www.w3.org/1999/xlink","xlink:type",v);break;case"xmlBase":Cs(c,"http://www.w3.org/XML/1998/namespace","xml:base",v);break;case"xmlLang":Cs(c,"http://www.w3.org/XML/1998/namespace","xml:lang",v);break;case"xmlSpace":Cs(c,"http://www.w3.org/XML/1998/namespace","xml:space",v);break;case"is":Qg(c,"is",v);break;case"innerText":case"textContent":break;default:(!(2<g.length)||g[0]!=="o"&&g[0]!=="O"||g[1]!=="n"&&g[1]!=="N")&&(g=Vce.get(g)||g,Qg(c,g,v))}}function XE(c,h,g,v,A,D){switch(g){case"style":FL(c,v,D);break;case"dangerouslySetInnerHTML":if(v!=null){if(typeof v!="object"||!("__html"in v))throw Error(n(61));if(g=v.__html,g!=null){if(A.children!=null)throw Error(n(60));c.innerHTML=g}}break;case"children":typeof v=="string"?yd(c,v):(typeof v=="number"||typeof v=="bigint")&&yd(c,""+v);break;case"onScroll":v!=null&&kr("scroll",c);break;case"onScrollEnd":v!=null&&kr("scrollend",c);break;case"onClick":v!=null&&(c.onclick=Ts);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Ll.hasOwnProperty(g))e:{if(g[0]==="o"&&g[1]==="n"&&(A=g.endsWith("Capture"),h=g.slice(2,A?g.length-7:void 0),D=c[ca]||null,D=D!=null?D[g]:null,typeof D=="function"&&c.removeEventListener(h,D,A),typeof v=="function")){typeof D!="function"&&D!==null&&(g in c?c[g]=null:c.hasAttribute(g)&&c.removeAttribute(g)),c.addEventListener(h,v,A);break e}g in c?c[g]=v:v===!0?c.setAttribute(g,""):Qg(c,g,v)}}}function fa(c,h,g){switch(h){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":kr("error",c),kr("load",c);var v=!1,A=!1,D;for(D in g)if(g.hasOwnProperty(D)){var V=g[D];if(V!=null)switch(D){case"src":v=!0;break;case"srcSet":A=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(n(137,h));default:Kr(c,h,D,V,g,null)}}A&&Kr(c,h,"srcSet",g.srcSet,g,null),v&&Kr(c,h,"src",g.src,g,null);return;case"input":kr("invalid",c);var Q=D=V=A=null,de=null,_e=null;for(v in g)if(g.hasOwnProperty(v)){var ze=g[v];if(ze!=null)switch(v){case"name":A=ze;break;case"type":V=ze;break;case"checked":de=ze;break;case"defaultChecked":_e=ze;break;case"value":D=ze;break;case"defaultValue":Q=ze;break;case"children":case"dangerouslySetInnerHTML":if(ze!=null)throw Error(n(137,h));break;default:Kr(c,h,v,ze,g,null)}}kL(c,D,Q,de,_e,V,A,!1);return;case"select":kr("invalid",c),v=V=D=null;for(A in g)if(g.hasOwnProperty(A)&&(Q=g[A],Q!=null))switch(A){case"value":D=Q;break;case"defaultValue":V=Q;break;case"multiple":v=Q;default:Kr(c,h,A,Q,g,null)}h=D,g=V,c.multiple=!!v,h!=null?bd(c,!!v,h,!1):g!=null&&bd(c,!!v,g,!0);return;case"textarea":kr("invalid",c),D=A=v=null;for(V in g)if(g.hasOwnProperty(V)&&(Q=g[V],Q!=null))switch(V){case"value":v=Q;break;case"defaultValue":A=Q;break;case"children":D=Q;break;case"dangerouslySetInnerHTML":if(Q!=null)throw Error(n(91));break;default:Kr(c,h,V,Q,g,null)}TL(c,v,A,D);return;case"option":for(de in g)if(g.hasOwnProperty(de)&&(v=g[de],v!=null))switch(de){case"selected":c.selected=v&&typeof v!="function"&&typeof v!="symbol";break;default:Kr(c,h,de,v,g,null)}return;case"dialog":kr("beforetoggle",c),kr("toggle",c),kr("cancel",c),kr("close",c);break;case"iframe":case"object":kr("load",c);break;case"video":case"audio":for(v=0;v<mf.length;v++)kr(mf[v],c);break;case"image":kr("error",c),kr("load",c);break;case"details":kr("toggle",c);break;case"embed":case"source":case"link":kr("error",c),kr("load",c);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(_e in g)if(g.hasOwnProperty(_e)&&(v=g[_e],v!=null))switch(_e){case"children":case"dangerouslySetInnerHTML":throw Error(n(137,h));default:Kr(c,h,_e,v,g,null)}return;default:if(dS(h)){for(ze in g)g.hasOwnProperty(ze)&&(v=g[ze],v!==void 0&&XE(c,h,ze,v,g,void 0));return}}for(Q in g)g.hasOwnProperty(Q)&&(v=g[Q],v!=null&&Kr(c,h,Q,v,g,null))}function She(c,h,g,v){switch(h){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var A=null,D=null,V=null,Q=null,de=null,_e=null,ze=null;for(Ie in g){var Ue=g[Ie];if(g.hasOwnProperty(Ie)&&Ue!=null)switch(Ie){case"checked":break;case"value":break;case"defaultValue":de=Ue;default:v.hasOwnProperty(Ie)||Kr(c,h,Ie,null,v,Ue)}}for(var De in v){var Ie=v[De];if(Ue=g[De],v.hasOwnProperty(De)&&(Ie!=null||Ue!=null))switch(De){case"type":D=Ie;break;case"name":A=Ie;break;case"checked":_e=Ie;break;case"defaultChecked":ze=Ie;break;case"value":V=Ie;break;case"defaultValue":Q=Ie;break;case"children":case"dangerouslySetInnerHTML":if(Ie!=null)throw Error(n(137,h));break;default:Ie!==Ue&&Kr(c,h,De,Ie,v,Ue)}}uS(c,V,Q,de,_e,ze,D,A);return;case"select":Ie=V=Q=De=null;for(D in g)if(de=g[D],g.hasOwnProperty(D)&&de!=null)switch(D){case"value":break;case"multiple":Ie=de;default:v.hasOwnProperty(D)||Kr(c,h,D,null,v,de)}for(A in v)if(D=v[A],de=g[A],v.hasOwnProperty(A)&&(D!=null||de!=null))switch(A){case"value":De=D;break;case"defaultValue":Q=D;break;case"multiple":V=D;default:D!==de&&Kr(c,h,A,D,v,de)}h=Q,g=V,v=Ie,De!=null?bd(c,!!g,De,!1):!!v!=!!g&&(h!=null?bd(c,!!g,h,!0):bd(c,!!g,g?[]:"",!1));return;case"textarea":Ie=De=null;for(Q in g)if(A=g[Q],g.hasOwnProperty(Q)&&A!=null&&!v.hasOwnProperty(Q))switch(Q){case"value":break;case"children":break;default:Kr(c,h,Q,null,v,A)}for(V in v)if(A=v[V],D=g[V],v.hasOwnProperty(V)&&(A!=null||D!=null))switch(V){case"value":De=A;break;case"defaultValue":Ie=A;break;case"children":break;case"dangerouslySetInnerHTML":if(A!=null)throw Error(n(91));break;default:A!==D&&Kr(c,h,V,A,v,D)}CL(c,De,Ie);return;case"option":for(var kt in g)if(De=g[kt],g.hasOwnProperty(kt)&&De!=null&&!v.hasOwnProperty(kt))switch(kt){case"selected":c.selected=!1;break;default:Kr(c,h,kt,null,v,De)}for(de in v)if(De=v[de],Ie=g[de],v.hasOwnProperty(de)&&De!==Ie&&(De!=null||Ie!=null))switch(de){case"selected":c.selected=De&&typeof De!="function"&&typeof De!="symbol";break;default:Kr(c,h,de,De,v,Ie)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var Yt in g)De=g[Yt],g.hasOwnProperty(Yt)&&De!=null&&!v.hasOwnProperty(Yt)&&Kr(c,h,Yt,null,v,De);for(_e in v)if(De=v[_e],Ie=g[_e],v.hasOwnProperty(_e)&&De!==Ie&&(De!=null||Ie!=null))switch(_e){case"children":case"dangerouslySetInnerHTML":if(De!=null)throw Error(n(137,h));break;default:Kr(c,h,_e,De,v,Ie)}return;default:if(dS(h)){for(var Yr in g)De=g[Yr],g.hasOwnProperty(Yr)&&De!==void 0&&!v.hasOwnProperty(Yr)&&XE(c,h,Yr,void 0,v,De);for(ze in v)De=v[ze],Ie=g[ze],!v.hasOwnProperty(ze)||De===Ie||De===void 0&&Ie===void 0||XE(c,h,ze,De,v,Ie);return}}for(var ve in g)De=g[ve],g.hasOwnProperty(ve)&&De!=null&&!v.hasOwnProperty(ve)&&Kr(c,h,ve,null,v,De);for(Ue in v)De=v[Ue],Ie=g[Ue],!v.hasOwnProperty(Ue)||De===Ie||De==null&&Ie==null||Kr(c,h,Ue,De,v,Ie)}function _O(c){switch(c){case"css":case"script":case"font":case"img":case"image":case"input":case"link":return!0;default:return!1}}function Ehe(){if(typeof performance.getEntriesByType=="function"){for(var c=0,h=0,g=performance.getEntriesByType("resource"),v=0;v<g.length;v++){var A=g[v],D=A.transferSize,V=A.initiatorType,Q=A.duration;if(D&&Q&&_O(V)){for(V=0,Q=A.responseEnd,v+=1;v<g.length;v++){var de=g[v],_e=de.startTime;if(_e>Q)break;var ze=de.transferSize,Ue=de.initiatorType;ze&&_O(Ue)&&(de=de.responseEnd,V+=ze*(de<Q?1:(Q-_e)/(de-_e)))}if(--v,h+=8*(D+V)/(A.duration/1e3),c++,10<c)break}}if(0<c)return h/c/1e6}return navigator.connection&&(c=navigator.connection.downlink,typeof c=="number")?c:5}var ZE=null,QE=null;function X0(c){return c.nodeType===9?c:c.ownerDocument}function RO(c){switch(c){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function DO(c,h){if(c===0)switch(h){case"svg":return 1;case"math":return 2;default:return 0}return c===1&&h==="foreignObject"?0:c}function ex(c,h){return c==="textarea"||c==="noscript"||typeof h.children=="string"||typeof h.children=="number"||typeof h.children=="bigint"||typeof h.dangerouslySetInnerHTML=="object"&&h.dangerouslySetInnerHTML!==null&&h.dangerouslySetInnerHTML.__html!=null}var tx=null;function xhe(){var c=window.event;return c&&c.type==="popstate"?c===tx?!1:(tx=c,!0):(tx=null,!1)}var NO=typeof setTimeout=="function"?setTimeout:void 0,whe=typeof clearTimeout=="function"?clearTimeout:void 0,LO=typeof Promise=="function"?Promise:void 0,khe=typeof queueMicrotask=="function"?queueMicrotask:typeof LO<"u"?function(c){return LO.resolve(null).then(c).catch(Che)}:NO;function Che(c){setTimeout(function(){throw c})}function Xl(c){return c==="head"}function MO(c,h){var g=h,v=0;do{var A=g.nextSibling;if(c.removeChild(g),A&&A.nodeType===8)if(g=A.data,g==="/$"||g==="/&"){if(v===0){c.removeChild(A),Yd(h);return}v--}else if(g==="$"||g==="$?"||g==="$~"||g==="$!"||g==="&")v++;else if(g==="html")bf(c.ownerDocument.documentElement);else if(g==="head"){g=c.ownerDocument.head,bf(g);for(var D=g.firstChild;D;){var V=D.nextSibling,Q=D.nodeName;D[be]||Q==="SCRIPT"||Q==="STYLE"||Q==="LINK"&&D.rel.toLowerCase()==="stylesheet"||g.removeChild(D),D=V}}else g==="body"&&bf(c.ownerDocument.body);g=A}while(g);Yd(h)}function IO(c,h){var g=c;c=0;do{var v=g.nextSibling;if(g.nodeType===1?h?(g._stashedDisplay=g.style.display,g.style.display="none"):(g.style.display=g._stashedDisplay||"",g.getAttribute("style")===""&&g.removeAttribute("style")):g.nodeType===3&&(h?(g._stashedText=g.nodeValue,g.nodeValue=""):g.nodeValue=g._stashedText||""),v&&v.nodeType===8)if(g=v.data,g==="/$"){if(c===0)break;c--}else g!=="$"&&g!=="$?"&&g!=="$~"&&g!=="$!"||c++;g=v}while(g)}function rx(c){var h=c.firstChild;for(h&&h.nodeType===10&&(h=h.nextSibling);h;){var g=h;switch(h=h.nextSibling,g.nodeName){case"HTML":case"HEAD":case"BODY":rx(g),at(g);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(g.rel.toLowerCase()==="stylesheet")continue}c.removeChild(g)}}function The(c,h,g,v){for(;c.nodeType===1;){var A=g;if(c.nodeName.toLowerCase()!==h.toLowerCase()){if(!v&&(c.nodeName!=="INPUT"||c.type!=="hidden"))break}else if(v){if(!c[be])switch(h){case"meta":if(!c.hasAttribute("itemprop"))break;return c;case"link":if(D=c.getAttribute("rel"),D==="stylesheet"&&c.hasAttribute("data-precedence"))break;if(D!==A.rel||c.getAttribute("href")!==(A.href==null||A.href===""?null:A.href)||c.getAttribute("crossorigin")!==(A.crossOrigin==null?null:A.crossOrigin)||c.getAttribute("title")!==(A.title==null?null:A.title))break;return c;case"style":if(c.hasAttribute("data-precedence"))break;return c;case"script":if(D=c.getAttribute("src"),(D!==(A.src==null?null:A.src)||c.getAttribute("type")!==(A.type==null?null:A.type)||c.getAttribute("crossorigin")!==(A.crossOrigin==null?null:A.crossOrigin))&&D&&c.hasAttribute("async")&&!c.hasAttribute("itemprop"))break;return c;default:return c}}else if(h==="input"&&c.type==="hidden"){var D=A.name==null?null:""+A.name;if(A.type==="hidden"&&c.getAttribute("name")===D)return c}else return c;if(c=Xi(c.nextSibling),c===null)break}return null}function Ahe(c,h,g){if(h==="")return null;for(;c.nodeType!==3;)if((c.nodeType!==1||c.nodeName!=="INPUT"||c.type!=="hidden")&&!g||(c=Xi(c.nextSibling),c===null))return null;return c}function OO(c,h){for(;c.nodeType!==8;)if((c.nodeType!==1||c.nodeName!=="INPUT"||c.type!=="hidden")&&!h||(c=Xi(c.nextSibling),c===null))return null;return c}function nx(c){return c.data==="$?"||c.data==="$~"}function ax(c){return c.data==="$!"||c.data==="$?"&&c.ownerDocument.readyState!=="loading"}function Fhe(c,h){var g=c.ownerDocument;if(c.data==="$~")c._reactRetry=h;else if(c.data!=="$?"||g.readyState!=="loading")h();else{var v=function(){h(),g.removeEventListener("DOMContentLoaded",v)};g.addEventListener("DOMContentLoaded",v),c._reactRetry=v}}function Xi(c){for(;c!=null;c=c.nextSibling){var h=c.nodeType;if(h===1||h===3)break;if(h===8){if(h=c.data,h==="$"||h==="$!"||h==="$?"||h==="$~"||h==="&"||h==="F!"||h==="F")break;if(h==="/$"||h==="/&")return null}}return c}var ix=null;function BO(c){c=c.nextSibling;for(var h=0;c;){if(c.nodeType===8){var g=c.data;if(g==="/$"||g==="/&"){if(h===0)return Xi(c.nextSibling);h--}else g!=="$"&&g!=="$!"&&g!=="$?"&&g!=="$~"&&g!=="&"||h++}c=c.nextSibling}return null}function PO(c){c=c.previousSibling;for(var h=0;c;){if(c.nodeType===8){var g=c.data;if(g==="$"||g==="$!"||g==="$?"||g==="$~"||g==="&"){if(h===0)return c;h--}else g!=="/$"&&g!=="/&"||h++}c=c.previousSibling}return null}function zO(c,h,g){switch(h=X0(g),c){case"html":if(c=h.documentElement,!c)throw Error(n(452));return c;case"head":if(c=h.head,!c)throw Error(n(453));return c;case"body":if(c=h.body,!c)throw Error(n(454));return c;default:throw Error(n(451))}}function bf(c){for(var h=c.attributes;h.length;)c.removeAttributeNode(h[0]);at(c)}var Zi=new Map,HO=new Set;function Z0(c){return typeof c.getRootNode=="function"?c.getRootNode():c.nodeType===9?c:c.ownerDocument}var $s=H.d;H.d={f:_he,r:Rhe,D:Dhe,C:Nhe,L:Lhe,m:Mhe,X:Ohe,S:Ihe,M:Bhe};function _he(){var c=$s.f(),h=G0();return c||h}function Rhe(c){var h=Hr(c);h!==null&&h.tag===5&&h.type==="form"?nI(h):$s.r(c)}var Wd=typeof document>"u"?null:document;function UO(c,h,g){var v=Wd;if(v&&typeof h=="string"&&h){var A=ji(h);A='link[rel="'+c+'"][href="'+A+'"]',typeof g=="string"&&(A+='[crossorigin="'+g+'"]'),HO.has(A)||(HO.add(A),c={rel:c,crossOrigin:g,href:h},v.querySelector(A)===null&&(h=v.createElement("link"),fa(h,"link",c),Mr(h),v.head.appendChild(h)))}}function Dhe(c){$s.D(c),UO("dns-prefetch",c,null)}function Nhe(c,h){$s.C(c,h),UO("preconnect",c,h)}function Lhe(c,h,g){$s.L(c,h,g);var v=Wd;if(v&&c&&h){var A='link[rel="preload"][as="'+ji(h)+'"]';h==="image"&&g&&g.imageSrcSet?(A+='[imagesrcset="'+ji(g.imageSrcSet)+'"]',typeof g.imageSizes=="string"&&(A+='[imagesizes="'+ji(g.imageSizes)+'"]')):A+='[href="'+ji(c)+'"]';var D=A;switch(h){case"style":D=Vd(c);break;case"script":D=Kd(c)}Zi.has(D)||(c=p({rel:"preload",href:h==="image"&&g&&g.imageSrcSet?void 0:c,as:h},g),Zi.set(D,c),v.querySelector(A)!==null||h==="style"&&v.querySelector(yf(D))||h==="script"&&v.querySelector(vf(D))||(h=v.createElement("link"),fa(h,"link",c),Mr(h),v.head.appendChild(h)))}}function Mhe(c,h){$s.m(c,h);var g=Wd;if(g&&c){var v=h&&typeof h.as=="string"?h.as:"script",A='link[rel="modulepreload"][as="'+ji(v)+'"][href="'+ji(c)+'"]',D=A;switch(v){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":D=Kd(c)}if(!Zi.has(D)&&(c=p({rel:"modulepreload",href:c},h),Zi.set(D,c),g.querySelector(A)===null)){switch(v){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(g.querySelector(vf(D)))return}v=g.createElement("link"),fa(v,"link",c),Mr(v),g.head.appendChild(v)}}}function Ihe(c,h,g){$s.S(c,h,g);var v=Wd;if(v&&c){var A=pn(v).hoistableStyles,D=Vd(c);h=h||"default";var V=A.get(D);if(!V){var Q={loading:0,preload:null};if(V=v.querySelector(yf(D)))Q.loading=5;else{c=p({rel:"stylesheet",href:c,"data-precedence":h},g),(g=Zi.get(D))&&ox(c,g);var de=V=v.createElement("link");Mr(de),fa(de,"link",c),de._p=new Promise(function(_e,ze){de.onload=_e,de.onerror=ze}),de.addEventListener("load",function(){Q.loading|=1}),de.addEventListener("error",function(){Q.loading|=2}),Q.loading|=4,Q0(V,h,v)}V={type:"stylesheet",instance:V,count:1,state:Q},A.set(D,V)}}}function Ohe(c,h){$s.X(c,h);var g=Wd;if(g&&c){var v=pn(g).hoistableScripts,A=Kd(c),D=v.get(A);D||(D=g.querySelector(vf(A)),D||(c=p({src:c,async:!0},h),(h=Zi.get(A))&&sx(c,h),D=g.createElement("script"),Mr(D),fa(D,"link",c),g.head.appendChild(D)),D={type:"script",instance:D,count:1,state:null},v.set(A,D))}}function Bhe(c,h){$s.M(c,h);var g=Wd;if(g&&c){var v=pn(g).hoistableScripts,A=Kd(c),D=v.get(A);D||(D=g.querySelector(vf(A)),D||(c=p({src:c,async:!0,type:"module"},h),(h=Zi.get(A))&&sx(c,h),D=g.createElement("script"),Mr(D),fa(D,"link",c),g.head.appendChild(D)),D={type:"script",instance:D,count:1,state:null},v.set(A,D))}}function $O(c,h,g,v){var A=(A=pe.current)?Z0(A):null;if(!A)throw Error(n(446));switch(c){case"meta":case"title":return null;case"style":return typeof g.precedence=="string"&&typeof g.href=="string"?(h=Vd(g.href),g=pn(A).hoistableStyles,v=g.get(h),v||(v={type:"style",instance:null,count:0,state:null},g.set(h,v)),v):{type:"void",instance:null,count:0,state:null};case"link":if(g.rel==="stylesheet"&&typeof g.href=="string"&&typeof g.precedence=="string"){c=Vd(g.href);var D=pn(A).hoistableStyles,V=D.get(c);if(V||(A=A.ownerDocument||A,V={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},D.set(c,V),(D=A.querySelector(yf(c)))&&!D._p&&(V.instance=D,V.state.loading=5),Zi.has(c)||(g={rel:"preload",as:"style",href:g.href,crossOrigin:g.crossOrigin,integrity:g.integrity,media:g.media,hrefLang:g.hrefLang,referrerPolicy:g.referrerPolicy},Zi.set(c,g),D||Phe(A,c,g,V.state))),h&&v===null)throw Error(n(528,""));return V}if(h&&v!==null)throw Error(n(529,""));return null;case"script":return h=g.async,g=g.src,typeof g=="string"&&h&&typeof h!="function"&&typeof h!="symbol"?(h=Kd(g),g=pn(A).hoistableScripts,v=g.get(h),v||(v={type:"script",instance:null,count:0,state:null},g.set(h,v)),v):{type:"void",instance:null,count:0,state:null};default:throw Error(n(444,c))}}function Vd(c){return'href="'+ji(c)+'"'}function yf(c){return'link[rel="stylesheet"]['+c+"]"}function GO(c){return p({},c,{"data-precedence":c.precedence,precedence:null})}function Phe(c,h,g,v){c.querySelector('link[rel="preload"][as="style"]['+h+"]")?v.loading=1:(h=c.createElement("link"),v.preload=h,h.addEventListener("load",function(){return v.loading|=1}),h.addEventListener("error",function(){return v.loading|=2}),fa(h,"link",g),Mr(h),c.head.appendChild(h))}function Kd(c){return'[src="'+ji(c)+'"]'}function vf(c){return"script[async]"+c}function jO(c,h,g){if(h.count++,h.instance===null)switch(h.type){case"style":var v=c.querySelector('style[data-href~="'+ji(g.href)+'"]');if(v)return h.instance=v,Mr(v),v;var A=p({},g,{"data-href":g.href,"data-precedence":g.precedence,href:null,precedence:null});return v=(c.ownerDocument||c).createElement("style"),Mr(v),fa(v,"style",A),Q0(v,g.precedence,c),h.instance=v;case"stylesheet":A=Vd(g.href);var D=c.querySelector(yf(A));if(D)return h.state.loading|=4,h.instance=D,Mr(D),D;v=GO(g),(A=Zi.get(A))&&ox(v,A),D=(c.ownerDocument||c).createElement("link"),Mr(D);var V=D;return V._p=new Promise(function(Q,de){V.onload=Q,V.onerror=de}),fa(D,"link",v),h.state.loading|=4,Q0(D,g.precedence,c),h.instance=D;case"script":return D=Kd(g.src),(A=c.querySelector(vf(D)))?(h.instance=A,Mr(A),A):(v=g,(A=Zi.get(D))&&(v=p({},g),sx(v,A)),c=c.ownerDocument||c,A=c.createElement("script"),Mr(A),fa(A,"link",v),c.head.appendChild(A),h.instance=A);case"void":return null;default:throw Error(n(443,h.type))}else h.type==="stylesheet"&&(h.state.loading&4)===0&&(v=h.instance,h.state.loading|=4,Q0(v,g.precedence,c));return h.instance}function Q0(c,h,g){for(var v=g.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),A=v.length?v[v.length-1]:null,D=A,V=0;V<v.length;V++){var Q=v[V];if(Q.dataset.precedence===h)D=Q;else if(D!==A)break}D?D.parentNode.insertBefore(c,D.nextSibling):(h=g.nodeType===9?g.head:g,h.insertBefore(c,h.firstChild))}function ox(c,h){c.crossOrigin==null&&(c.crossOrigin=h.crossOrigin),c.referrerPolicy==null&&(c.referrerPolicy=h.referrerPolicy),c.title==null&&(c.title=h.title)}function sx(c,h){c.crossOrigin==null&&(c.crossOrigin=h.crossOrigin),c.referrerPolicy==null&&(c.referrerPolicy=h.referrerPolicy),c.integrity==null&&(c.integrity=h.integrity)}var eb=null;function qO(c,h,g){if(eb===null){var v=new Map,A=eb=new Map;A.set(g,v)}else A=eb,v=A.get(g),v||(v=new Map,A.set(g,v));if(v.has(c))return v;for(v.set(c,null),g=g.getElementsByTagName(c),A=0;A<g.length;A++){var D=g[A];if(!(D[be]||D[Xr]||c==="link"&&D.getAttribute("rel")==="stylesheet")&&D.namespaceURI!=="http://www.w3.org/2000/svg"){var V=D.getAttribute(h)||"";V=c+V;var Q=v.get(V);Q?Q.push(D):v.set(V,[D])}}return v}function WO(c,h,g){c=c.ownerDocument||c,c.head.insertBefore(g,h==="title"?c.querySelector("head > title"):null)}function zhe(c,h,g){if(g===1||h.itemProp!=null)return!1;switch(c){case"meta":case"title":return!0;case"style":if(typeof h.precedence!="string"||typeof h.href!="string"||h.href==="")break;return!0;case"link":if(typeof h.rel!="string"||typeof h.href!="string"||h.href===""||h.onLoad||h.onError)break;switch(h.rel){case"stylesheet":return c=h.disabled,typeof h.precedence=="string"&&c==null;default:return!0}case"script":if(h.async&&typeof h.async!="function"&&typeof h.async!="symbol"&&!h.onLoad&&!h.onError&&h.src&&typeof h.src=="string")return!0}return!1}function VO(c){return!(c.type==="stylesheet"&&(c.state.loading&3)===0)}function Hhe(c,h,g,v){if(g.type==="stylesheet"&&(typeof v.media!="string"||matchMedia(v.media).matches!==!1)&&(g.state.loading&4)===0){if(g.instance===null){var A=Vd(v.href),D=h.querySelector(yf(A));if(D){h=D._p,h!==null&&typeof h=="object"&&typeof h.then=="function"&&(c.count++,c=tb.bind(c),h.then(c,c)),g.state.loading|=4,g.instance=D,Mr(D);return}D=h.ownerDocument||h,v=GO(v),(A=Zi.get(A))&&ox(v,A),D=D.createElement("link"),Mr(D);var V=D;V._p=new Promise(function(Q,de){V.onload=Q,V.onerror=de}),fa(D,"link",v),g.instance=D}c.stylesheets===null&&(c.stylesheets=new Map),c.stylesheets.set(g,h),(h=g.state.preload)&&(g.state.loading&3)===0&&(c.count++,g=tb.bind(c),h.addEventListener("load",g),h.addEventListener("error",g))}}var lx=0;function Uhe(c,h){return c.stylesheets&&c.count===0&&nb(c,c.stylesheets),0<c.count||0<c.imgCount?function(g){var v=setTimeout(function(){if(c.stylesheets&&nb(c,c.stylesheets),c.unsuspend){var D=c.unsuspend;c.unsuspend=null,D()}},6e4+h);0<c.imgBytes&&lx===0&&(lx=62500*Ehe());var A=setTimeout(function(){if(c.waitingForImages=!1,c.count===0&&(c.stylesheets&&nb(c,c.stylesheets),c.unsuspend)){var D=c.unsuspend;c.unsuspend=null,D()}},(c.imgBytes>lx?50:800)+h);return c.unsuspend=g,function(){c.unsuspend=null,clearTimeout(v),clearTimeout(A)}}:null}function tb(){if(this.count--,this.count===0&&(this.imgCount===0||!this.waitingForImages)){if(this.stylesheets)nb(this,this.stylesheets);else if(this.unsuspend){var c=this.unsuspend;this.unsuspend=null,c()}}}var rb=null;function nb(c,h){c.stylesheets=null,c.unsuspend!==null&&(c.count++,rb=new Map,h.forEach($he,c),rb=null,tb.call(c))}function $he(c,h){if(!(h.state.loading&4)){var g=rb.get(c);if(g)var v=g.get(null);else{g=new Map,rb.set(c,g);for(var A=c.querySelectorAll("link[data-precedence],style[data-precedence]"),D=0;D<A.length;D++){var V=A[D];(V.nodeName==="LINK"||V.getAttribute("media")!=="not all")&&(g.set(V.dataset.precedence,V),v=V)}v&&g.set(null,v)}A=h.instance,V=A.getAttribute("data-precedence"),D=g.get(V)||v,D===v&&g.set(null,A),g.set(V,A),this.count++,v=tb.bind(this),A.addEventListener("load",v),A.addEventListener("error",v),D?D.parentNode.insertBefore(A,D.nextSibling):(c=c.nodeType===9?c.head:c,c.insertBefore(A,c.firstChild)),h.state.loading|=4}}var Sf={$$typeof:k,Provider:null,Consumer:null,_currentValue:B,_currentValue2:B,_threadCount:0};function Ghe(c,h,g,v,A,D,V,Q,de){this.tag=1,this.containerInfo=c,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=yn(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=yn(0),this.hiddenUpdates=yn(null),this.identifierPrefix=v,this.onUncaughtError=A,this.onCaughtError=D,this.onRecoverableError=V,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=de,this.incompleteTransitions=new Map}function KO(c,h,g,v,A,D,V,Q,de,_e,ze,Ue){return c=new Ghe(c,h,g,V,de,_e,ze,Ue,Q),h=1,D===!0&&(h|=24),D=wi(3,null,null,h),c.current=D,D.stateNode=c,h=US(),h.refCount++,c.pooledCache=h,h.refCount++,D.memoizedState={element:v,isDehydrated:g,cache:h},qS(D),c}function YO(c){return c?(c=Cd,c):Cd}function JO(c,h,g,v,A,D){A=YO(A),v.context===null?v.context=A:v.pendingContext=A,v=Hl(h),v.payload={element:g},D=D===void 0?null:D,D!==null&&(v.callback=D),g=Ul(c,v,h),g!==null&&(ni(g,c,h),Xp(g,c,h))}function XO(c,h){if(c=c.memoizedState,c!==null&&c.dehydrated!==null){var g=c.retryLane;c.retryLane=g!==0&&g<h?g:h}}function ux(c,h){XO(c,h),(c=c.alternate)&&XO(c,h)}function ZO(c){if(c.tag===13||c.tag===31){var h=Yu(c,67108864);h!==null&&ni(h,c,67108864),ux(c,67108864)}}function QO(c){if(c.tag===13||c.tag===31){var h=Fi();h=Nl(h);var g=Yu(c,h);g!==null&&ni(g,c,h),ux(c,h)}}var ab=!0;function jhe(c,h,g,v){var A=z.T;z.T=null;var D=H.p;try{H.p=2,cx(c,h,g,v)}finally{H.p=D,z.T=A}}function qhe(c,h,g,v){var A=z.T;z.T=null;var D=H.p;try{H.p=8,cx(c,h,g,v)}finally{H.p=D,z.T=A}}function cx(c,h,g,v){if(ab){var A=dx(v);if(A===null)JE(c,h,v,ib,g),t9(c,v);else if(Vhe(A,c,h,g,v))v.stopPropagation();else if(t9(c,v),h&4&&-1<Whe.indexOf(c)){for(;A!==null;){var D=Hr(A);if(D!==null)switch(D.tag){case 3:if(D=D.stateNode,D.current.memoizedState.isDehydrated){var V=vt(D.pendingLanes);if(V!==0){var Q=D;for(Q.pendingLanes|=2,Q.entangledLanes|=2;V;){var de=1<<31-qt(V);Q.entanglements[1]|=de,V&=~de}qo(D),(Ur&6)===0&&(U0=Pe()+500,ff(0))}}break;case 31:case 13:Q=Yu(D,2),Q!==null&&ni(Q,D,2),G0(),ux(D,2)}if(D=dx(v),D===null&&JE(c,h,v,ib,g),D===A)break;A=D}A!==null&&v.stopPropagation()}else JE(c,h,v,null,g)}}function dx(c){return c=pS(c),hx(c)}var ib=null;function hx(c){if(ib=null,c=wt(c),c!==null){var h=i(c);if(h===null)c=null;else{var g=h.tag;if(g===13){if(c=o(h),c!==null)return c;c=null}else if(g===31){if(c=s(h),c!==null)return c;c=null}else if(g===3){if(h.stateNode.current.memoizedState.isDehydrated)return h.tag===3?h.stateNode.containerInfo:null;c=null}else h!==c&&(c=null)}}return ib=c,null}function e9(c){switch(c){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(rt()){case ge:return 2;case Re:return 8;case We:case ht:return 32;case xt:return 268435456;default:return 32}default:return 32}}var px=!1,Zl=null,Ql=null,eu=null,Ef=new Map,xf=new Map,tu=[],Whe="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function t9(c,h){switch(c){case"focusin":case"focusout":Zl=null;break;case"dragenter":case"dragleave":Ql=null;break;case"mouseover":case"mouseout":eu=null;break;case"pointerover":case"pointerout":Ef.delete(h.pointerId);break;case"gotpointercapture":case"lostpointercapture":xf.delete(h.pointerId)}}function wf(c,h,g,v,A,D){return c===null||c.nativeEvent!==D?(c={blockedOn:h,domEventName:g,eventSystemFlags:v,nativeEvent:D,targetContainers:[A]},h!==null&&(h=Hr(h),h!==null&&ZO(h)),c):(c.eventSystemFlags|=v,h=c.targetContainers,A!==null&&h.indexOf(A)===-1&&h.push(A),c)}function Vhe(c,h,g,v,A){switch(h){case"focusin":return Zl=wf(Zl,c,h,g,v,A),!0;case"dragenter":return Ql=wf(Ql,c,h,g,v,A),!0;case"mouseover":return eu=wf(eu,c,h,g,v,A),!0;case"pointerover":var D=A.pointerId;return Ef.set(D,wf(Ef.get(D)||null,c,h,g,v,A)),!0;case"gotpointercapture":return D=A.pointerId,xf.set(D,wf(xf.get(D)||null,c,h,g,v,A)),!0}return!1}function r9(c){var h=wt(c.target);if(h!==null){var g=i(h);if(g!==null){if(h=g.tag,h===13){if(h=o(g),h!==null){c.blockedOn=h,md(c.priority,function(){QO(g)});return}}else if(h===31){if(h=s(g),h!==null){c.blockedOn=h,md(c.priority,function(){QO(g)});return}}else if(h===3&&g.stateNode.current.memoizedState.isDehydrated){c.blockedOn=g.tag===3?g.stateNode.containerInfo:null;return}}}c.blockedOn=null}function ob(c){if(c.blockedOn!==null)return!1;for(var h=c.targetContainers;0<h.length;){var g=dx(c.nativeEvent);if(g===null){g=c.nativeEvent;var v=new g.constructor(g.type,g);hS=v,g.target.dispatchEvent(v),hS=null}else return h=Hr(g),h!==null&&ZO(h),c.blockedOn=g,!1;h.shift()}return!0}function n9(c,h,g){ob(c)&&g.delete(h)}function Khe(){px=!1,Zl!==null&&ob(Zl)&&(Zl=null),Ql!==null&&ob(Ql)&&(Ql=null),eu!==null&&ob(eu)&&(eu=null),Ef.forEach(n9),xf.forEach(n9)}function sb(c,h){c.blockedOn===h&&(c.blockedOn=null,px||(px=!0,e.unstable_scheduleCallback(e.unstable_NormalPriority,Khe)))}var lb=null;function a9(c){lb!==c&&(lb=c,e.unstable_scheduleCallback(e.unstable_NormalPriority,function(){lb===c&&(lb=null);for(var h=0;h<c.length;h+=3){var g=c[h],v=c[h+1],A=c[h+2];if(typeof v!="function"){if(hx(v||g)===null)continue;break}var D=Hr(g);D!==null&&(c.splice(h,3),h-=3,dE(D,{pending:!0,data:A,method:g.method,action:v},v,A))}}))}function Yd(c){function h(de){return sb(de,c)}Zl!==null&&sb(Zl,c),Ql!==null&&sb(Ql,c),eu!==null&&sb(eu,c),Ef.forEach(h),xf.forEach(h);for(var g=0;g<tu.length;g++){var v=tu[g];v.blockedOn===c&&(v.blockedOn=null)}for(;0<tu.length&&(g=tu[0],g.blockedOn===null);)r9(g),g.blockedOn===null&&tu.shift();if(g=(c.ownerDocument||c).$$reactFormReplay,g!=null)for(v=0;v<g.length;v+=3){var A=g[v],D=g[v+1],V=A[ca]||null;if(typeof D=="function")V||a9(g);else if(V){var Q=null;if(D&&D.hasAttribute("formAction")){if(A=D,V=D[ca]||null)Q=V.formAction;else if(hx(A)!==null)continue}else Q=V.action;typeof Q=="function"?g[v+1]=Q:(g.splice(v,3),v-=3),a9(g)}}}function i9(){function c(D){D.canIntercept&&D.info==="react-transition"&&D.intercept({handler:function(){return new Promise(function(V){return A=V})},focusReset:"manual",scroll:"manual"})}function h(){A!==null&&(A(),A=null),v||setTimeout(g,20)}function g(){if(!v&&!navigation.transition){var D=navigation.currentEntry;D&&D.url!=null&&navigation.navigate(D.url,{state:D.getState(),info:"react-transition",history:"replace"})}}if(typeof navigation=="object"){var v=!1,A=null;return navigation.addEventListener("navigate",c),navigation.addEventListener("navigatesuccess",h),navigation.addEventListener("navigateerror",h),setTimeout(g,100),function(){v=!0,navigation.removeEventListener("navigate",c),navigation.removeEventListener("navigatesuccess",h),navigation.removeEventListener("navigateerror",h),A!==null&&(A(),A=null)}}}function fx(c){this._internalRoot=c}ub.prototype.render=fx.prototype.render=function(c){var h=this._internalRoot;if(h===null)throw Error(n(409));var g=h.current,v=Fi();JO(g,v,c,h,null,null)},ub.prototype.unmount=fx.prototype.unmount=function(){var c=this._internalRoot;if(c!==null){this._internalRoot=null;var h=c.containerInfo;JO(c.current,2,null,c,null,null),G0(),h[ws]=null}};function ub(c){this._internalRoot=c}ub.prototype.unstable_scheduleHydration=function(c){if(c){var h=Ip();c={blockedOn:null,target:c,priority:h};for(var g=0;g<tu.length&&h!==0&&h<tu[g].priority;g++);tu.splice(g,0,c),g===0&&r9(c)}};var o9=t.version;if(o9!=="19.2.0")throw Error(n(527,o9,"19.2.0"));H.findDOMNode=function(c){var h=c._reactInternals;if(h===void 0)throw typeof c.render=="function"?Error(n(188)):(c=Object.keys(c).join(","),Error(n(268,c)));return c=u(h),c=c!==null?d(c):null,c=c===null?null:c.stateNode,c};var Yhe={bundleType:0,version:"19.2.0",rendererPackageName:"react-dom",currentDispatcherRef:z,reconcilerVersion:"19.2.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var cb=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!cb.isDisabled&&cb.supportsFiber)try{fr=cb.inject(Yhe),Mt=cb}catch{}}return Cf.createRoot=function(c,h){if(!a(c))throw Error(n(299));var g=!1,v="",A=pI,D=fI,V=mI;return h!=null&&(h.unstable_strictMode===!0&&(g=!0),h.identifierPrefix!==void 0&&(v=h.identifierPrefix),h.onUncaughtError!==void 0&&(A=h.onUncaughtError),h.onCaughtError!==void 0&&(D=h.onCaughtError),h.onRecoverableError!==void 0&&(V=h.onRecoverableError)),h=KO(c,1,!1,null,null,g,v,null,A,D,V,i9),c[ws]=h.current,YE(c),new fx(h)},Cf.hydrateRoot=function(c,h,g){if(!a(c))throw Error(n(299));var v=!1,A="",D=pI,V=fI,Q=mI,de=null;return g!=null&&(g.unstable_strictMode===!0&&(v=!0),g.identifierPrefix!==void 0&&(A=g.identifierPrefix),g.onUncaughtError!==void 0&&(D=g.onUncaughtError),g.onCaughtError!==void 0&&(V=g.onCaughtError),g.onRecoverableError!==void 0&&(Q=g.onRecoverableError),g.formState!==void 0&&(de=g.formState)),h=KO(c,1,!0,h,g??null,v,A,de,D,V,Q,i9),h.context=YO(null),g=h.current,v=Fi(),v=Nl(v),A=Hl(v),A.callback=null,Ul(g,A,v),g=v,h.current.lanes=g,hn(h,g),qo(h),c[ws]=h.current,YE(c),new ub(h)},Cf.version="19.2.0",Cf}var g9;function lpe(){if(g9)return bx.exports;g9=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),bx.exports=spe(),bx.exports}var upe=lpe();class ui{constructor(t,r,n){this.lexer=void 0,this.start=void 0,this.end=void 0,this.lexer=t,this.start=r,this.end=n}static range(t,r){return r?!t||!t.loc||!r.loc||t.loc.lexer!==r.loc.lexer?null:new ui(t.loc.lexer,t.loc.start,r.loc.end):t&&t.loc}}class Ii{constructor(t,r){this.text=void 0,this.loc=void 0,this.noexpand=void 0,this.treatAsRelax=void 0,this.text=t,this.loc=r}range(t,r){return new Ii(r,ui.range(this,t))}}class nt{constructor(t,r){this.name=void 0,this.position=void 0,this.length=void 0,this.rawMessage=void 0;var n="KaTeX parse error: "+t,a,i,o=r&&r.loc;if(o&&o.start<=o.end){var s=o.lexer.input;a=o.start,i=o.end,a===s.length?n+=" at end of input: ":n+=" at position "+(a+1)+": ";var l=s.slice(a,i).replace(/[^]/g,"$&̲"),u;a>15?u="…"+s.slice(a-15,a):u=s.slice(0,a);var d;i+15<s.length?d=s.slice(i,i+15)+"…":d=s.slice(i),n+=u+l+d}var p=new Error(n);return p.name="ParseError",p.__proto__=nt.prototype,p.position=a,a!=null&&i!=null&&(p.length=i-a),p.rawMessage=t,p}}nt.prototype.__proto__=Error.prototype;var cpe=function(t,r){return t.indexOf(r)!==-1},dpe=function(t,r){return t===void 0?r:t},hpe=/([A-Z])/g,ppe=function(t){return t.replace(hpe,"-$1").toLowerCase()},fpe={"&":"&amp;",">":"&gt;","<":"&lt;",'"':"&quot;","'":"&#x27;"},mpe=/[&><"']/g;function gpe(e){return String(e).replace(mpe,t=>fpe[t])}var dK=function e(t){return t.type==="ordgroup"||t.type==="color"?t.body.length===1?e(t.body[0]):t:t.type==="font"?e(t.body):t},bpe=function(t){var r=dK(t);return r.type==="mathord"||r.type==="textord"||r.type==="atom"},ype=function(t){if(!t)throw new Error("Expected non-null, but got "+String(t));return t},vpe=function(t){var r=/^[\x00-\x20]*([^\\/#?]*?)(:|&#0*58|&#x0*3a|&colon)/i.exec(t);return r?r[2]!==":"||!/^[a-zA-Z][a-zA-Z0-9+\-.]*$/.test(r[1])?null:r[1].toLowerCase():"_relative"},Gt={contains:cpe,deflt:dpe,escape:gpe,hyphenate:ppe,getBaseElem:dK,isCharacterBox:bpe,protocolFromUrl:vpe},gm={displayMode:{type:"boolean",description:"Render math in display mode, which puts the math in display style (so \\int and \\sum are large, for example), and centers the math on the page on its own line.",cli:"-d, --display-mode"},output:{type:{enum:["htmlAndMathml","html","mathml"]},description:"Determines the markup language of the output.",cli:"-F, --format <type>"},leqno:{type:"boolean",description:"Render display math in leqno style (left-justified tags)."},fleqn:{type:"boolean",description:"Render display math flush left."},throwOnError:{type:"boolean",default:!0,cli:"-t, --no-throw-on-error",cliDescription:"Render errors (in the color given by --error-color) instead of throwing a ParseError exception when encountering an error."},errorColor:{type:"string",default:"#cc0000",cli:"-c, --error-color <color>",cliDescription:"A color string given in the format 'rgb' or 'rrggbb' (no #). This option determines the color of errors rendered by the -t option.",cliProcessor:e=>"#"+e},macros:{type:"object",cli:"-m, --macro <def>",cliDescription:"Define custom macro of the form '\\foo:expansion' (use multiple -m arguments for multiple macros).",cliDefault:[],cliProcessor:(e,t)=>(t.push(e),t)},minRuleThickness:{type:"number",description:"Specifies a minimum thickness, in ems, for fraction lines, `\\sqrt` top lines, `{array}` vertical lines, `\\hline`, `\\hdashline`, `\\underline`, `\\overline`, and the borders of `\\fbox`, `\\boxed`, and `\\fcolorbox`.",processor:e=>Math.max(0,e),cli:"--min-rule-thickness <size>",cliProcessor:parseFloat},colorIsTextColor:{type:"boolean",description:"Makes \\color behave like LaTeX's 2-argument \\textcolor, instead of LaTeX's one-argument \\color mode change.",cli:"-b, --color-is-text-color"},strict:{type:[{enum:["warn","ignore","error"]},"boolean","function"],description:"Turn on strict / LaTeX faithfulness mode, which throws an error if the input uses features that are not supported by LaTeX.",cli:"-S, --strict",cliDefault:!1},trust:{type:["boolean","function"],description:"Trust the input, enabling all HTML features such as \\url.",cli:"-T, --trust"},maxSize:{type:"number",default:1/0,description:"If non-zero, all user-specified sizes, e.g. in \\rule{500em}{500em}, will be capped to maxSize ems. Otherwise, elements and spaces can be arbitrarily large",processor:e=>Math.max(0,e),cli:"-s, --max-size <n>",cliProcessor:parseInt},maxExpand:{type:"number",default:1e3,description:"Limit the number of macro expansions to the specified number, to prevent e.g. infinite macro loops. If set to Infinity, the macro expander will try to fully expand as in LaTeX.",processor:e=>Math.max(0,e),cli:"-e, --max-expand <n>",cliProcessor:e=>e==="Infinity"?1/0:parseInt(e)},globalGroup:{type:"boolean",cli:!1}};function Spe(e){if(e.default)return e.default;var t=e.type,r=Array.isArray(t)?t[0]:t;if(typeof r!="string")return r.enum[0];switch(r){case"boolean":return!1;case"string":return"";case"number":return 0;case"object":return{}}}let aD=class{constructor(t){this.displayMode=void 0,this.output=void 0,this.leqno=void 0,this.fleqn=void 0,this.throwOnError=void 0,this.errorColor=void 0,this.macros=void 0,this.minRuleThickness=void 0,this.colorIsTextColor=void 0,this.strict=void 0,this.trust=void 0,this.maxSize=void 0,this.maxExpand=void 0,this.globalGroup=void 0,t=t||{};for(var r in gm)if(gm.hasOwnProperty(r)){var n=gm[r];this[r]=t[r]!==void 0?n.processor?n.processor(t[r]):t[r]:Spe(n)}}reportNonstrict(t,r,n){var a=this.strict;if(typeof a=="function"&&(a=a(t,r,n)),!(!a||a==="ignore")){if(a===!0||a==="error")throw new nt("LaTeX-incompatible input and strict mode is set to 'error': "+(r+" ["+t+"]"),n);a==="warn"?typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to 'warn': "+(r+" ["+t+"]")):typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to "+("unrecognized '"+a+"': "+r+" ["+t+"]"))}}useStrictBehavior(t,r,n){var a=this.strict;if(typeof a=="function")try{a=a(t,r,n)}catch{a="error"}return!a||a==="ignore"?!1:a===!0||a==="error"?!0:a==="warn"?(typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to 'warn': "+(r+" ["+t+"]")),!1):(typeof console<"u"&&console.warn("LaTeX-incompatible input and strict mode is set to "+("unrecognized '"+a+"': "+r+" ["+t+"]")),!1)}isTrusted(t){if(t.url&&!t.protocol){var r=Gt.protocolFromUrl(t.url);if(r==null)return!1;t.protocol=r}var n=typeof this.trust=="function"?this.trust(t):this.trust;return!!n}},nu=class{constructor(t,r,n){this.id=void 0,this.size=void 0,this.cramped=void 0,this.id=t,this.size=r,this.cramped=n}sup(){return es[Epe[this.id]]}sub(){return es[xpe[this.id]]}fracNum(){return es[wpe[this.id]]}fracDen(){return es[kpe[this.id]]}cramp(){return es[Cpe[this.id]]}text(){return es[Tpe[this.id]]}isTight(){return this.size>=2}};var iD=0,ty=1,yh=2,sl=3,Mm=4,ao=5,Oh=6,qa=7,es=[new nu(iD,0,!1),new nu(ty,0,!0),new nu(yh,1,!1),new nu(sl,1,!0),new nu(Mm,2,!1),new nu(ao,2,!0),new nu(Oh,3,!1),new nu(qa,3,!0)],Epe=[Mm,ao,Mm,ao,Oh,qa,Oh,qa],xpe=[ao,ao,ao,ao,qa,qa,qa,qa],wpe=[yh,sl,Mm,ao,Oh,qa,Oh,qa],kpe=[sl,sl,ao,ao,qa,qa,qa,qa],Cpe=[ty,ty,sl,sl,ao,ao,qa,qa],Tpe=[iD,ty,yh,sl,yh,sl,yh,sl],Vt={DISPLAY:es[iD],TEXT:es[yh],SCRIPT:es[Mm],SCRIPTSCRIPT:es[Oh]},E_=[{name:"latin",blocks:[[256,591],[768,879]]},{name:"cyrillic",blocks:[[1024,1279]]},{name:"armenian",blocks:[[1328,1423]]},{name:"brahmic",blocks:[[2304,4255]]},{name:"georgian",blocks:[[4256,4351]]},{name:"cjk",blocks:[[12288,12543],[19968,40879],[65280,65376]]},{name:"hangul",blocks:[[44032,55215]]}];function Ape(e){for(var t=0;t<E_.length;t++)for(var r=E_[t],n=0;n<r.blocks.length;n++){var a=r.blocks[n];if(e>=a[0]&&e<=a[1])return r.name}return null}var b1=[];E_.forEach(e=>e.blocks.forEach(t=>b1.push(...t)));function hK(e){for(var t=0;t<b1.length;t+=2)if(e>=b1[t]&&e<=b1[t+1])return!0;return!1}var Jd=80,Fpe=function(t,r){return"M95,"+(622+t+r)+`
c-2.7,0,-7.17,-2.7,-13.5,-8c-5.8,-5.3,-9.5,-10,-9.5,-14
c0,-2,0.3,-3.3,1,-4c1.3,-2.7,23.83,-20.7,67.5,-54
c44.2,-33.3,65.8,-50.3,66.5,-51c1.3,-1.3,3,-2,5,-2c4.7,0,8.7,3.3,12,10
s173,378,173,378c0.7,0,35.3,-71,104,-213c68.7,-142,137.5,-285,206.5,-429
c69,-144,104.5,-217.7,106.5,-221
l`+t/2.075+" -"+t+`
c5.3,-9.3,12,-14,20,-14
H400000v`+(40+t)+`H845.2724
s-225.272,467,-225.272,467s-235,486,-235,486c-2.7,4.7,-9,7,-19,7
c-6,0,-10,-1,-12,-3s-194,-422,-194,-422s-65,47,-65,47z
M`+(834+t)+" "+r+"h400000v"+(40+t)+"h-400000z"},_pe=function(t,r){return"M263,"+(601+t+r)+`c0.7,0,18,39.7,52,119
c34,79.3,68.167,158.7,102.5,238c34.3,79.3,51.8,119.3,52.5,120
c340,-704.7,510.7,-1060.3,512,-1067
l`+t/2.084+" -"+t+`
c4.7,-7.3,11,-11,19,-11
H40000v`+(40+t)+`H1012.3
s-271.3,567,-271.3,567c-38.7,80.7,-84,175,-136,283c-52,108,-89.167,185.3,-111.5,232
c-22.3,46.7,-33.8,70.3,-34.5,71c-4.7,4.7,-12.3,7,-23,7s-12,-1,-12,-1
s-109,-253,-109,-253c-72.7,-168,-109.3,-252,-110,-252c-10.7,8,-22,16.7,-34,26
c-22,17.3,-33.3,26,-34,26s-26,-26,-26,-26s76,-59,76,-59s76,-60,76,-60z
M`+(1001+t)+" "+r+"h400000v"+(40+t)+"h-400000z"},Rpe=function(t,r){return"M983 "+(10+t+r)+`
l`+t/3.13+" -"+t+`
c4,-6.7,10,-10,18,-10 H400000v`+(40+t)+`
H1013.1s-83.4,268,-264.1,840c-180.7,572,-277,876.3,-289,913c-4.7,4.7,-12.7,7,-24,7
s-12,0,-12,0c-1.3,-3.3,-3.7,-11.7,-7,-25c-35.3,-125.3,-106.7,-373.3,-214,-744
c-10,12,-21,25,-33,39s-32,39,-32,39c-6,-5.3,-15,-14,-27,-26s25,-30,25,-30
c26.7,-32.7,52,-63,76,-91s52,-60,52,-60s208,722,208,722
c56,-175.3,126.3,-397.3,211,-666c84.7,-268.7,153.8,-488.2,207.5,-658.5
c53.7,-170.3,84.5,-266.8,92.5,-289.5z
M`+(1001+t)+" "+r+"h400000v"+(40+t)+"h-400000z"},Dpe=function(t,r){return"M424,"+(2398+t+r)+`
c-1.3,-0.7,-38.5,-172,-111.5,-514c-73,-342,-109.8,-513.3,-110.5,-514
c0,-2,-10.7,14.3,-32,49c-4.7,7.3,-9.8,15.7,-15.5,25c-5.7,9.3,-9.8,16,-12.5,20
s-5,7,-5,7c-4,-3.3,-8.3,-7.7,-13,-13s-13,-13,-13,-13s76,-122,76,-122s77,-121,77,-121
s209,968,209,968c0,-2,84.7,-361.7,254,-1079c169.3,-717.3,254.7,-1077.7,256,-1081
l`+t/4.223+" -"+t+`c4,-6.7,10,-10,18,-10 H400000
v`+(40+t)+`H1014.6
s-87.3,378.7,-272.6,1166c-185.3,787.3,-279.3,1182.3,-282,1185
c-2,6,-10,9,-24,9
c-8,0,-12,-0.7,-12,-2z M`+(1001+t)+" "+r+`
h400000v`+(40+t)+"h-400000z"},Npe=function(t,r){return"M473,"+(2713+t+r)+`
c339.3,-1799.3,509.3,-2700,510,-2702 l`+t/5.298+" -"+t+`
c3.3,-7.3,9.3,-11,18,-11 H400000v`+(40+t)+`H1017.7
s-90.5,478,-276.2,1466c-185.7,988,-279.5,1483,-281.5,1485c-2,6,-10,9,-24,9
c-8,0,-12,-0.7,-12,-2c0,-1.3,-5.3,-32,-16,-92c-50.7,-293.3,-119.7,-693.3,-207,-1200
c0,-1.3,-5.3,8.7,-16,30c-10.7,21.3,-21.3,42.7,-32,64s-16,33,-16,33s-26,-26,-26,-26
s76,-153,76,-153s77,-151,77,-151c0.7,0.7,35.7,202,105,604c67.3,400.7,102,602.7,104,
606zM`+(1001+t)+" "+r+"h400000v"+(40+t)+"H1017.7z"},Lpe=function(t){var r=t/2;return"M400000 "+t+" H0 L"+r+" 0 l65 45 L145 "+(t-80)+" H400000z"},Mpe=function(t,r,n){var a=n-54-r-t;return"M702 "+(t+r)+"H400000"+(40+t)+`
H742v`+a+`l-4 4-4 4c-.667.7 -2 1.5-4 2.5s-4.167 1.833-6.5 2.5-5.5 1-9.5 1
h-12l-28-84c-16.667-52-96.667 -294.333-240-727l-212 -643 -85 170
c-4-3.333-8.333-7.667-13 -13l-13-13l77-155 77-156c66 199.333 139 419.667
219 661 l218 661zM702 `+r+"H400000v"+(40+t)+"H742z"},Ipe=function(t,r,n){r=1e3*r;var a="";switch(t){case"sqrtMain":a=Fpe(r,Jd);break;case"sqrtSize1":a=_pe(r,Jd);break;case"sqrtSize2":a=Rpe(r,Jd);break;case"sqrtSize3":a=Dpe(r,Jd);break;case"sqrtSize4":a=Npe(r,Jd);break;case"sqrtTall":a=Mpe(r,Jd,n)}return a},Ope=function(t,r){switch(t){case"⎜":return"M291 0 H417 V"+r+" H291z M291 0 H417 V"+r+" H291z";case"∣":return"M145 0 H188 V"+r+" H145z M145 0 H188 V"+r+" H145z";case"∥":return"M145 0 H188 V"+r+" H145z M145 0 H188 V"+r+" H145z"+("M367 0 H410 V"+r+" H367z M367 0 H410 V"+r+" H367z");case"⎟":return"M457 0 H583 V"+r+" H457z M457 0 H583 V"+r+" H457z";case"⎢":return"M319 0 H403 V"+r+" H319z M319 0 H403 V"+r+" H319z";case"⎥":return"M263 0 H347 V"+r+" H263z M263 0 H347 V"+r+" H263z";case"⎪":return"M384 0 H504 V"+r+" H384z M384 0 H504 V"+r+" H384z";case"⏐":return"M312 0 H355 V"+r+" H312z M312 0 H355 V"+r+" H312z";case"‖":return"M257 0 H300 V"+r+" H257z M257 0 H300 V"+r+" H257z"+("M478 0 H521 V"+r+" H478z M478 0 H521 V"+r+" H478z");default:return""}},b9={doubleleftarrow:`M262 157
l10-10c34-36 62.7-77 86-123 3.3-8 5-13.3 5-16 0-5.3-6.7-8-20-8-7.3
 0-12.2.5-14.5 1.5-2.3 1-4.8 4.5-7.5 10.5-49.3 97.3-121.7 169.3-217 216-28
 14-57.3 25-88 33-6.7 2-11 3.8-13 5.5-2 1.7-3 4.2-3 7.5s1 5.8 3 7.5
c2 1.7 6.3 3.5 13 5.5 68 17.3 128.2 47.8 180.5 91.5 52.3 43.7 93.8 96.2 124.5
 157.5 9.3 8 15.3 12.3 18 13h6c12-.7 18-4 18-10 0-2-1.7-7-5-15-23.3-46-52-87
-86-123l-10-10h399738v-40H218c328 0 0 0 0 0l-10-8c-26.7-20-65.7-43-117-69 2.7
-2 6-3.7 10-5 36.7-16 72.3-37.3 107-64l10-8h399782v-40z
m8 0v40h399730v-40zm0 194v40h399730v-40z`,doublerightarrow:`M399738 392l
-10 10c-34 36-62.7 77-86 123-3.3 8-5 13.3-5 16 0 5.3 6.7 8 20 8 7.3 0 12.2-.5
 14.5-1.5 2.3-1 4.8-4.5 7.5-10.5 49.3-97.3 121.7-169.3 217-216 28-14 57.3-25 88
-33 6.7-2 11-3.8 13-5.5 2-1.7 3-4.2 3-7.5s-1-5.8-3-7.5c-2-1.7-6.3-3.5-13-5.5-68
-17.3-128.2-47.8-180.5-91.5-52.3-43.7-93.8-96.2-124.5-157.5-9.3-8-15.3-12.3-18
-13h-6c-12 .7-18 4-18 10 0 2 1.7 7 5 15 23.3 46 52 87 86 123l10 10H0v40h399782
c-328 0 0 0 0 0l10 8c26.7 20 65.7 43 117 69-2.7 2-6 3.7-10 5-36.7 16-72.3 37.3
-107 64l-10 8H0v40zM0 157v40h399730v-40zm0 194v40h399730v-40z`,leftarrow:`M400000 241H110l3-3c68.7-52.7 113.7-120
 135-202 4-14.7 6-23 6-25 0-7.3-7-11-21-11-8 0-13.2.8-15.5 2.5-2.3 1.7-4.2 5.8
-5.5 12.5-1.3 4.7-2.7 10.3-4 17-12 48.7-34.8 92-68.5 130S65.3 228.3 18 247
c-10 4-16 7.7-18 11 0 8.7 6 14.3 18 17 47.3 18.7 87.8 47 121.5 85S196 441.3 208
 490c.7 2 1.3 5 2 9s1.2 6.7 1.5 8c.3 1.3 1 3.3 2 6s2.2 4.5 3.5 5.5c1.3 1 3.3
 1.8 6 2.5s6 1 10 1c14 0 21-3.7 21-11 0-2-2-10.3-6-25-20-79.3-65-146.7-135-202
 l-3-3h399890zM100 241v40h399900v-40z`,leftbrace:`M6 548l-6-6v-35l6-11c56-104 135.3-181.3 238-232 57.3-28.7 117
-45 179-50h399577v120H403c-43.3 7-81 15-113 26-100.7 33-179.7 91-237 174-2.7
 5-6 9-10 13-.7 1-7.3 1-20 1H6z`,leftbraceunder:`M0 6l6-6h17c12.688 0 19.313.3 20 1 4 4 7.313 8.3 10 13
 35.313 51.3 80.813 93.8 136.5 127.5 55.688 33.7 117.188 55.8 184.5 66.5.688
 0 2 .3 4 1 18.688 2.7 76 4.3 172 5h399450v120H429l-6-1c-124.688-8-235-61.7
-331-161C60.687 138.7 32.312 99.3 7 54L0 41V6z`,leftgroup:`M400000 80
H435C64 80 168.3 229.4 21 260c-5.9 1.2-18 0-18 0-2 0-3-1-3-3v-38C76 61 257 0
 435 0h399565z`,leftgroupunder:`M400000 262
H435C64 262 168.3 112.6 21 82c-5.9-1.2-18 0-18 0-2 0-3 1-3 3v38c76 158 257 219
 435 219h399565z`,leftharpoon:`M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3
-3.3 10.2-9.5 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5
-18.3 3-21-1.3-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7
-196 228-6.7 4.7-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40z`,leftharpoonplus:`M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3-3.3 10.2-9.5
 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5-18.3 3-21-1.3
-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7-196 228-6.7 4.7
-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40zM0 435v40h400000v-40z
m0 0v40h400000v-40z`,leftharpoondown:`M7 241c-4 4-6.333 8.667-7 14 0 5.333.667 9 2 11s5.333
 5.333 12 10c90.667 54 156 130 196 228 3.333 10.667 6.333 16.333 9 17 2 .667 5
 1 9 1h5c10.667 0 16.667-2 18-6 2-2.667 1-9.667-3-21-32-87.333-82.667-157.667
-152-211l-3-3h399907v-40zM93 281 H400000 v-40L7 241z`,leftharpoondownplus:`M7 435c-4 4-6.3 8.7-7 14 0 5.3.7 9 2 11s5.3 5.3 12
 10c90.7 54 156 130 196 228 3.3 10.7 6.3 16.3 9 17 2 .7 5 1 9 1h5c10.7 0 16.7
-2 18-6 2-2.7 1-9.7-3-21-32-87.3-82.7-157.7-152-211l-3-3h399907v-40H7zm93 0
v40h399900v-40zM0 241v40h399900v-40zm0 0v40h399900v-40z`,lefthook:`M400000 281 H103s-33-11.2-61-33.5S0 197.3 0 164s14.2-61.2 42.5
-83.5C70.8 58.2 104 47 142 47 c16.7 0 25 6.7 25 20 0 12-8.7 18.7-26 20-40 3.3
-68.7 15.7-86 37-10 12-15 25.3-15 40 0 22.7 9.8 40.7 29.5 54 19.7 13.3 43.5 21
 71.5 23h399859zM103 281v-40h399897v40z`,leftlinesegment:`M40 281 V428 H0 V94 H40 V241 H400000 v40z
M40 281 V428 H0 V94 H40 V241 H400000 v40z`,leftmapsto:`M40 281 V448H0V74H40V241H400000v40z
M40 281 V448H0V74H40V241H400000v40z`,leftToFrom:`M0 147h400000v40H0zm0 214c68 40 115.7 95.7 143 167h22c15.3 0 23
-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69-70-101l-7-8h399905v-40H95l7-8
c28.7-32 52-65.7 70-101 10.7-23.3 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 265.3
 68 321 0 361zm0-174v-40h399900v40zm100 154v40h399900v-40z`,longequal:`M0 50 h400000 v40H0z m0 194h40000v40H0z
M0 50 h400000 v40H0z m0 194h40000v40H0z`,midbrace:`M200428 334
c-100.7-8.3-195.3-44-280-108-55.3-42-101.7-93-139-153l-9-14c-2.7 4-5.7 8.7-9 14
-53.3 86.7-123.7 153-211 199-66.7 36-137.3 56.3-212 62H0V214h199568c178.3-11.7
 311.7-78.3 403-201 6-8 9.7-12 11-12 .7-.7 6.7-1 18-1s17.3.3 18 1c1.3 0 5 4 11
 12 44.7 59.3 101.3 106.3 170 141s145.3 54.3 229 60h199572v120z`,midbraceunder:`M199572 214
c100.7 8.3 195.3 44 280 108 55.3 42 101.7 93 139 153l9 14c2.7-4 5.7-8.7 9-14
 53.3-86.7 123.7-153 211-199 66.7-36 137.3-56.3 212-62h199568v120H200432c-178.3
 11.7-311.7 78.3-403 201-6 8-9.7 12-11 12-.7.7-6.7 1-18 1s-17.3-.3-18-1c-1.3 0
-5-4-11-12-44.7-59.3-101.3-106.3-170-141s-145.3-54.3-229-60H0V214z`,oiintSize1:`M512.6 71.6c272.6 0 320.3 106.8 320.3 178.2 0 70.8-47.7 177.6
-320.3 177.6S193.1 320.6 193.1 249.8c0-71.4 46.9-178.2 319.5-178.2z
m368.1 178.2c0-86.4-60.9-215.4-368.1-215.4-306.4 0-367.3 129-367.3 215.4 0 85.8
60.9 214.8 367.3 214.8 307.2 0 368.1-129 368.1-214.8z`,oiintSize2:`M757.8 100.1c384.7 0 451.1 137.6 451.1 230 0 91.3-66.4 228.8
-451.1 228.8-386.3 0-452.7-137.5-452.7-228.8 0-92.4 66.4-230 452.7-230z
m502.4 230c0-111.2-82.4-277.2-502.4-277.2s-504 166-504 277.2
c0 110 84 276 504 276s502.4-166 502.4-276z`,oiiintSize1:`M681.4 71.6c408.9 0 480.5 106.8 480.5 178.2 0 70.8-71.6 177.6
-480.5 177.6S202.1 320.6 202.1 249.8c0-71.4 70.5-178.2 479.3-178.2z
m525.8 178.2c0-86.4-86.8-215.4-525.7-215.4-437.9 0-524.7 129-524.7 215.4 0
85.8 86.8 214.8 524.7 214.8 438.9 0 525.7-129 525.7-214.8z`,oiiintSize2:`M1021.2 53c603.6 0 707.8 165.8 707.8 277.2 0 110-104.2 275.8
-707.8 275.8-606 0-710.2-165.8-710.2-275.8C311 218.8 415.2 53 1021.2 53z
m770.4 277.1c0-131.2-126.4-327.6-770.5-327.6S248.4 198.9 248.4 330.1
c0 130 128.8 326.4 772.7 326.4s770.5-196.4 770.5-326.4z`,rightarrow:`M0 241v40h399891c-47.3 35.3-84 78-110 128
-16.7 32-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20
 11 8 0 13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7
 39-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85
-40.5-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5
-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67
 151.7 139 205zm0 0v40h399900v-40z`,rightbrace:`M400000 542l
-6 6h-17c-12.7 0-19.3-.3-20-1-4-4-7.3-8.3-10-13-35.3-51.3-80.8-93.8-136.5-127.5
s-117.2-55.8-184.5-66.5c-.7 0-2-.3-4-1-18.7-2.7-76-4.3-172-5H0V214h399571l6 1
c124.7 8 235 61.7 331 161 31.3 33.3 59.7 72.7 85 118l7 13v35z`,rightbraceunder:`M399994 0l6 6v35l-6 11c-56 104-135.3 181.3-238 232-57.3
 28.7-117 45-179 50H-300V214h399897c43.3-7 81-15 113-26 100.7-33 179.7-91 237
-174 2.7-5 6-9 10-13 .7-1 7.3-1 20-1h17z`,rightgroup:`M0 80h399565c371 0 266.7 149.4 414 180 5.9 1.2 18 0 18 0 2 0
 3-1 3-3v-38c-76-158-257-219-435-219H0z`,rightgroupunder:`M0 262h399565c371 0 266.7-149.4 414-180 5.9-1.2 18 0 18
 0 2 0 3 1 3 3v38c-76 158-257 219-435 219H0z`,rightharpoon:`M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3
-3.7-15.3-11-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2
-10.7 0-16.7 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58
 69.2 92 94.5zm0 0v40h399900v-40z`,rightharpoonplus:`M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3-3.7-15.3-11
-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2-10.7 0-16.7
 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58 69.2 92 94.5z
m0 0v40h399900v-40z m100 194v40h399900v-40zm0 0v40h399900v-40z`,rightharpoondown:`M399747 511c0 7.3 6.7 11 20 11 8 0 13-.8 15-2.5s4.7-6.8
 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3 8.5-5.8 9.5
-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3-64.7 57-92 95
-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 241v40h399900v-40z`,rightharpoondownplus:`M399747 705c0 7.3 6.7 11 20 11 8 0 13-.8
 15-2.5s4.7-6.8 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3
 8.5-5.8 9.5-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3
-64.7 57-92 95-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 435v40h399900v-40z
m0-194v40h400000v-40zm0 0v40h400000v-40z`,righthook:`M399859 241c-764 0 0 0 0 0 40-3.3 68.7-15.7 86-37 10-12 15-25.3
 15-40 0-22.7-9.8-40.7-29.5-54-19.7-13.3-43.5-21-71.5-23-17.3-1.3-26-8-26-20 0
-13.3 8.7-20 26-20 38 0 71 11.2 99 33.5 0 0 7 5.6 21 16.7 14 11.2 21 33.5 21
 66.8s-14 61.2-42 83.5c-28 22.3-61 33.5-99 33.5L0 241z M0 281v-40h399859v40z`,rightlinesegment:`M399960 241 V94 h40 V428 h-40 V281 H0 v-40z
M399960 241 V94 h40 V428 h-40 V281 H0 v-40z`,rightToFrom:`M400000 167c-70.7-42-118-97.7-142-167h-23c-15.3 0-23 .3-23
 1 0 1.3 5.3 13.7 16 37 18 35.3 41.3 69 70 101l7 8H0v40h399905l-7 8c-28.7 32
-52 65.7-70 101-10.7 23.3-16 35.7-16 37 0 .7 7.7 1 23 1h23c24-69.3 71.3-125 142
-167z M100 147v40h399900v-40zM0 341v40h399900v-40z`,twoheadleftarrow:`M0 167c68 40
 115.7 95.7 143 167h22c15.3 0 23-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69
-70-101l-7-8h125l9 7c50.7 39.3 85 86 103 140h46c0-4.7-6.3-18.7-19-42-18-35.3
-40-67.3-66-96l-9-9h399716v-40H284l9-9c26-28.7 48-60.7 66-96 12.7-23.333 19
-37.333 19-42h-46c-18 54-52.3 100.7-103 140l-9 7H95l7-8c28.7-32 52-65.7 70-101
 10.7-23.333 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 71.3 68 127 0 167z`,twoheadrightarrow:`M400000 167
c-68-40-115.7-95.7-143-167h-22c-15.3 0-23 .3-23 1 0 1.3 5.3 13.7 16 37 18 35.3
 41.3 69 70 101l7 8h-125l-9-7c-50.7-39.3-85-86-103-140h-46c0 4.7 6.3 18.7 19 42
 18 35.3 40 67.3 66 96l9 9H0v40h399716l-9 9c-26 28.7-48 60.7-66 96-12.7 23.333
-19 37.333-19 42h46c18-54 52.3-100.7 103-140l9-7h125l-7 8c-28.7 32-52 65.7-70
 101-10.7 23.333-16 35.7-16 37 0 .7 7.7 1 23 1h22c27.3-71.3 75-127 143-167z`,tilde1:`M200 55.538c-77 0-168 73.953-177 73.953-3 0-7
-2.175-9-5.437L2 97c-1-2-2-4-2-6 0-4 2-7 5-9l20-12C116 12 171 0 207 0c86 0
 114 68 191 68 78 0 168-68 177-68 4 0 7 2 9 5l12 19c1 2.175 2 4.35 2 6.525 0
 4.35-2 7.613-5 9.788l-19 13.05c-92 63.077-116.937 75.308-183 76.128
-68.267.847-113-73.952-191-73.952z`,tilde2:`M344 55.266c-142 0-300.638 81.316-311.5 86.418
-8.01 3.762-22.5 10.91-23.5 5.562L1 120c-1-2-1-3-1-4 0-5 3-9 8-10l18.4-9C160.9
 31.9 283 0 358 0c148 0 188 122 331 122s314-97 326-97c4 0 8 2 10 7l7 21.114
c1 2.14 1 3.21 1 4.28 0 5.347-3 9.626-7 10.696l-22.3 12.622C852.6 158.372 751
 181.476 676 181.476c-149 0-189-126.21-332-126.21z`,tilde3:`M786 59C457 59 32 175.242 13 175.242c-6 0-10-3.457
-11-10.37L.15 138c-1-7 3-12 10-13l19.2-6.4C378.4 40.7 634.3 0 804.3 0c337 0
 411.8 157 746.8 157 328 0 754-112 773-112 5 0 10 3 11 9l1 14.075c1 8.066-.697
 16.595-6.697 17.492l-21.052 7.31c-367.9 98.146-609.15 122.696-778.15 122.696
 -338 0-409-156.573-744-156.573z`,tilde4:`M786 58C457 58 32 177.487 13 177.487c-6 0-10-3.345
-11-10.035L.15 143c-1-7 3-12 10-13l22-6.7C381.2 35 637.15 0 807.15 0c337 0 409
 177 744 177 328 0 754-127 773-127 5 0 10 3 11 9l1 14.794c1 7.805-3 13.38-9
 14.495l-20.7 5.574c-366.85 99.79-607.3 139.372-776.3 139.372-338 0-409
 -175.236-744-175.236z`,vec:`M377 20c0-5.333 1.833-10 5.5-14S391 0 397 0c4.667 0 8.667 1.667 12 5
3.333 2.667 6.667 9 10 19 6.667 24.667 20.333 43.667 41 57 7.333 4.667 11
10.667 11 18 0 6-1 10-3 12s-6.667 5-14 9c-28.667 14.667-53.667 35.667-75 63
-1.333 1.333-3.167 3.5-5.5 6.5s-4 4.833-5 5.5c-1 .667-2.5 1.333-4.5 2s-4.333 1
-7 1c-4.667 0-9.167-1.833-13.5-5.5S337 184 337 178c0-12.667 15.667-32.333 47-59
H213l-171-1c-8.667-6-13-12.333-13-19 0-4.667 4.333-11.333 13-20h359
c-16-25.333-24-45-24-59z`,widehat1:`M529 0h5l519 115c5 1 9 5 9 10 0 1-1 2-1 3l-4 22
c-1 5-5 9-11 9h-2L532 67 19 159h-2c-5 0-9-4-11-9l-5-22c-1-6 2-12 8-13z`,widehat2:`M1181 0h2l1171 176c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 220h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,widehat3:`M1181 0h2l1171 236c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 280h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,widehat4:`M1181 0h2l1171 296c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 340h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,widecheck1:`M529,159h5l519,-115c5,-1,9,-5,9,-10c0,-1,-1,-2,-1,-3l-4,-22c-1,
-5,-5,-9,-11,-9h-2l-512,92l-513,-92h-2c-5,0,-9,4,-11,9l-5,22c-1,6,2,12,8,13z`,widecheck2:`M1181,220h2l1171,-176c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,153l-1167,-153h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,widecheck3:`M1181,280h2l1171,-236c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,213l-1167,-213h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,widecheck4:`M1181,340h2l1171,-296c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,273l-1167,-273h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,baraboveleftarrow:`M400000 620h-399890l3 -3c68.7 -52.7 113.7 -120 135 -202
c4 -14.7 6 -23 6 -25c0 -7.3 -7 -11 -21 -11c-8 0 -13.2 0.8 -15.5 2.5
c-2.3 1.7 -4.2 5.8 -5.5 12.5c-1.3 4.7 -2.7 10.3 -4 17c-12 48.7 -34.8 92 -68.5 130
s-74.2 66.3 -121.5 85c-10 4 -16 7.7 -18 11c0 8.7 6 14.3 18 17c47.3 18.7 87.8 47
121.5 85s56.5 81.3 68.5 130c0.7 2 1.3 5 2 9s1.2 6.7 1.5 8c0.3 1.3 1 3.3 2 6
s2.2 4.5 3.5 5.5c1.3 1 3.3 1.8 6 2.5s6 1 10 1c14 0 21 -3.7 21 -11
c0 -2 -2 -10.3 -6 -25c-20 -79.3 -65 -146.7 -135 -202l-3 -3h399890z
M100 620v40h399900v-40z M0 241v40h399900v-40zM0 241v40h399900v-40z`,rightarrowabovebar:`M0 241v40h399891c-47.3 35.3-84 78-110 128-16.7 32
-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20 11 8 0
13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7 39
-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85-40.5
-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5
-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67
151.7 139 205zm96 379h399894v40H0zm0 0h399904v40H0z`,baraboveshortleftharpoon:`M507,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11
c1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17
c2,0.7,5,1,9,1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21
c-32,-87.3,-82.7,-157.7,-152,-211c0,0,-3,-3,-3,-3l399351,0l0,-40
c-398570,0,-399437,0,-399437,0z M593 435 v40 H399500 v-40z
M0 281 v-40 H399908 v40z M0 281 v-40 H399908 v40z`,rightharpoonaboveshortbar:`M0,241 l0,40c399126,0,399993,0,399993,0
c4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,
-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6
c-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z
M0 241 v40 H399908 v-40z M0 475 v-40 H399500 v40z M0 475 v-40 H399500 v40z`,shortbaraboveleftharpoon:`M7,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11
c1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17c2,0.7,5,1,9,
1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21c-32,-87.3,-82.7,-157.7,
-152,-211c0,0,-3,-3,-3,-3l399907,0l0,-40c-399126,0,-399993,0,-399993,0z
M93 435 v40 H400000 v-40z M500 241 v40 H400000 v-40z M500 241 v40 H400000 v-40z`,shortrightharpoonabovebar:`M53,241l0,40c398570,0,399437,0,399437,0
c4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,
-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6
c-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z
M500 241 v40 H399408 v-40z M500 435 v40 H400000 v-40z`},Bpe=function(t,r){switch(t){case"lbrack":return"M403 1759 V84 H666 V0 H319 V1759 v"+r+` v1759 h347 v-84
H403z M403 1759 V0 H319 V1759 v`+r+" v1759 h84z";case"rbrack":return"M347 1759 V0 H0 V84 H263 V1759 v"+r+` v1759 H0 v84 H347z
M347 1759 V0 H263 V1759 v`+r+" v1759 h84z";case"vert":return"M145 15 v585 v"+r+` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v`+-r+` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M188 15 H145 v585 v`+r+" v585 h43z";case"doublevert":return"M145 15 v585 v"+r+` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v`+-r+` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M188 15 H145 v585 v`+r+` v585 h43z
M367 15 v585 v`+r+` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v`+-r+` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M410 15 H367 v585 v`+r+" v585 h43z";case"lfloor":return"M319 602 V0 H403 V602 v"+r+` v1715 h263 v84 H319z
MM319 602 V0 H403 V602 v`+r+" v1715 H319z";case"rfloor":return"M319 602 V0 H403 V602 v"+r+` v1799 H0 v-84 H319z
MM319 602 V0 H403 V602 v`+r+" v1715 H319z";case"lceil":return"M403 1759 V84 H666 V0 H319 V1759 v"+r+` v602 h84z
M403 1759 V0 H319 V1759 v`+r+" v602 h84z";case"rceil":return"M347 1759 V0 H0 V84 H263 V1759 v"+r+` v602 h84z
M347 1759 V0 h-84 V1759 v`+r+" v602 h84z";case"lparen":return`M863,9c0,-2,-2,-5,-6,-9c0,0,-17,0,-17,0c-12.7,0,-19.3,0.3,-20,1
c-5.3,5.3,-10.3,11,-15,17c-242.7,294.7,-395.3,682,-458,1162c-21.3,163.3,-33.3,349,
-36,557 l0,`+(r+84)+`c0.2,6,0,26,0,60c2,159.3,10,310.7,24,454c53.3,528,210,
949.7,470,1265c4.7,6,9.7,11.7,15,17c0.7,0.7,7,1,19,1c0,0,18,0,18,0c4,-4,6,-7,6,-9
c0,-2.7,-3.3,-8.7,-10,-18c-135.3,-192.7,-235.5,-414.3,-300.5,-665c-65,-250.7,-102.5,
-544.7,-112.5,-882c-2,-104,-3,-167,-3,-189
l0,-`+(r+92)+`c0,-162.7,5.7,-314,17,-454c20.7,-272,63.7,-513,129,-723c65.3,
-210,155.3,-396.3,270,-559c6.7,-9.3,10,-15.3,10,-18z`;case"rparen":return`M76,0c-16.7,0,-25,3,-25,9c0,2,2,6.3,6,13c21.3,28.7,42.3,60.3,
63,95c96.7,156.7,172.8,332.5,228.5,527.5c55.7,195,92.8,416.5,111.5,664.5
c11.3,139.3,17,290.7,17,454c0,28,1.7,43,3.3,45l0,`+(r+9)+`
c-3,4,-3.3,16.7,-3.3,38c0,162,-5.7,313.7,-17,455c-18.7,248,-55.8,469.3,-111.5,664
c-55.7,194.7,-131.8,370.3,-228.5,527c-20.7,34.7,-41.7,66.3,-63,95c-2,3.3,-4,7,-6,11
c0,7.3,5.7,11,17,11c0,0,11,0,11,0c9.3,0,14.3,-0.3,15,-1c5.3,-5.3,10.3,-11,15,-17
c242.7,-294.7,395.3,-681.7,458,-1161c21.3,-164.7,33.3,-350.7,36,-558
l0,-`+(r+144)+`c-2,-159.3,-10,-310.7,-24,-454c-53.3,-528,-210,-949.7,
-470,-1265c-4.7,-6,-9.7,-11.7,-15,-17c-0.7,-0.7,-6.7,-1,-18,-1z`;default:throw new Error("Unknown stretchy delimiter.")}};let ug=class{constructor(t){this.children=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.maxFontSize=void 0,this.style=void 0,this.children=t,this.classes=[],this.height=0,this.depth=0,this.maxFontSize=0,this.style={}}hasClass(t){return Gt.contains(this.classes,t)}toNode(){for(var t=document.createDocumentFragment(),r=0;r<this.children.length;r++)t.appendChild(this.children[r].toNode());return t}toMarkup(){for(var t="",r=0;r<this.children.length;r++)t+=this.children[r].toMarkup();return t}toText(){var t=r=>r.toText();return this.children.map(t).join("")}};var as={"AMS-Regular":{32:[0,0,0,0,.25],65:[0,.68889,0,0,.72222],66:[0,.68889,0,0,.66667],67:[0,.68889,0,0,.72222],68:[0,.68889,0,0,.72222],69:[0,.68889,0,0,.66667],70:[0,.68889,0,0,.61111],71:[0,.68889,0,0,.77778],72:[0,.68889,0,0,.77778],73:[0,.68889,0,0,.38889],74:[.16667,.68889,0,0,.5],75:[0,.68889,0,0,.77778],76:[0,.68889,0,0,.66667],77:[0,.68889,0,0,.94445],78:[0,.68889,0,0,.72222],79:[.16667,.68889,0,0,.77778],80:[0,.68889,0,0,.61111],81:[.16667,.68889,0,0,.77778],82:[0,.68889,0,0,.72222],83:[0,.68889,0,0,.55556],84:[0,.68889,0,0,.66667],85:[0,.68889,0,0,.72222],86:[0,.68889,0,0,.72222],87:[0,.68889,0,0,1],88:[0,.68889,0,0,.72222],89:[0,.68889,0,0,.72222],90:[0,.68889,0,0,.66667],107:[0,.68889,0,0,.55556],160:[0,0,0,0,.25],165:[0,.675,.025,0,.75],174:[.15559,.69224,0,0,.94666],240:[0,.68889,0,0,.55556],295:[0,.68889,0,0,.54028],710:[0,.825,0,0,2.33334],732:[0,.9,0,0,2.33334],770:[0,.825,0,0,2.33334],771:[0,.9,0,0,2.33334],989:[.08167,.58167,0,0,.77778],1008:[0,.43056,.04028,0,.66667],8245:[0,.54986,0,0,.275],8463:[0,.68889,0,0,.54028],8487:[0,.68889,0,0,.72222],8498:[0,.68889,0,0,.55556],8502:[0,.68889,0,0,.66667],8503:[0,.68889,0,0,.44445],8504:[0,.68889,0,0,.66667],8513:[0,.68889,0,0,.63889],8592:[-.03598,.46402,0,0,.5],8594:[-.03598,.46402,0,0,.5],8602:[-.13313,.36687,0,0,1],8603:[-.13313,.36687,0,0,1],8606:[.01354,.52239,0,0,1],8608:[.01354,.52239,0,0,1],8610:[.01354,.52239,0,0,1.11111],8611:[.01354,.52239,0,0,1.11111],8619:[0,.54986,0,0,1],8620:[0,.54986,0,0,1],8621:[-.13313,.37788,0,0,1.38889],8622:[-.13313,.36687,0,0,1],8624:[0,.69224,0,0,.5],8625:[0,.69224,0,0,.5],8630:[0,.43056,0,0,1],8631:[0,.43056,0,0,1],8634:[.08198,.58198,0,0,.77778],8635:[.08198,.58198,0,0,.77778],8638:[.19444,.69224,0,0,.41667],8639:[.19444,.69224,0,0,.41667],8642:[.19444,.69224,0,0,.41667],8643:[.19444,.69224,0,0,.41667],8644:[.1808,.675,0,0,1],8646:[.1808,.675,0,0,1],8647:[.1808,.675,0,0,1],8648:[.19444,.69224,0,0,.83334],8649:[.1808,.675,0,0,1],8650:[.19444,.69224,0,0,.83334],8651:[.01354,.52239,0,0,1],8652:[.01354,.52239,0,0,1],8653:[-.13313,.36687,0,0,1],8654:[-.13313,.36687,0,0,1],8655:[-.13313,.36687,0,0,1],8666:[.13667,.63667,0,0,1],8667:[.13667,.63667,0,0,1],8669:[-.13313,.37788,0,0,1],8672:[-.064,.437,0,0,1.334],8674:[-.064,.437,0,0,1.334],8705:[0,.825,0,0,.5],8708:[0,.68889,0,0,.55556],8709:[.08167,.58167,0,0,.77778],8717:[0,.43056,0,0,.42917],8722:[-.03598,.46402,0,0,.5],8724:[.08198,.69224,0,0,.77778],8726:[.08167,.58167,0,0,.77778],8733:[0,.69224,0,0,.77778],8736:[0,.69224,0,0,.72222],8737:[0,.69224,0,0,.72222],8738:[.03517,.52239,0,0,.72222],8739:[.08167,.58167,0,0,.22222],8740:[.25142,.74111,0,0,.27778],8741:[.08167,.58167,0,0,.38889],8742:[.25142,.74111,0,0,.5],8756:[0,.69224,0,0,.66667],8757:[0,.69224,0,0,.66667],8764:[-.13313,.36687,0,0,.77778],8765:[-.13313,.37788,0,0,.77778],8769:[-.13313,.36687,0,0,.77778],8770:[-.03625,.46375,0,0,.77778],8774:[.30274,.79383,0,0,.77778],8776:[-.01688,.48312,0,0,.77778],8778:[.08167,.58167,0,0,.77778],8782:[.06062,.54986,0,0,.77778],8783:[.06062,.54986,0,0,.77778],8785:[.08198,.58198,0,0,.77778],8786:[.08198,.58198,0,0,.77778],8787:[.08198,.58198,0,0,.77778],8790:[0,.69224,0,0,.77778],8791:[.22958,.72958,0,0,.77778],8796:[.08198,.91667,0,0,.77778],8806:[.25583,.75583,0,0,.77778],8807:[.25583,.75583,0,0,.77778],8808:[.25142,.75726,0,0,.77778],8809:[.25142,.75726,0,0,.77778],8812:[.25583,.75583,0,0,.5],8814:[.20576,.70576,0,0,.77778],8815:[.20576,.70576,0,0,.77778],8816:[.30274,.79383,0,0,.77778],8817:[.30274,.79383,0,0,.77778],8818:[.22958,.72958,0,0,.77778],8819:[.22958,.72958,0,0,.77778],8822:[.1808,.675,0,0,.77778],8823:[.1808,.675,0,0,.77778],8828:[.13667,.63667,0,0,.77778],8829:[.13667,.63667,0,0,.77778],8830:[.22958,.72958,0,0,.77778],8831:[.22958,.72958,0,0,.77778],8832:[.20576,.70576,0,0,.77778],8833:[.20576,.70576,0,0,.77778],8840:[.30274,.79383,0,0,.77778],8841:[.30274,.79383,0,0,.77778],8842:[.13597,.63597,0,0,.77778],8843:[.13597,.63597,0,0,.77778],8847:[.03517,.54986,0,0,.77778],8848:[.03517,.54986,0,0,.77778],8858:[.08198,.58198,0,0,.77778],8859:[.08198,.58198,0,0,.77778],8861:[.08198,.58198,0,0,.77778],8862:[0,.675,0,0,.77778],8863:[0,.675,0,0,.77778],8864:[0,.675,0,0,.77778],8865:[0,.675,0,0,.77778],8872:[0,.69224,0,0,.61111],8873:[0,.69224,0,0,.72222],8874:[0,.69224,0,0,.88889],8876:[0,.68889,0,0,.61111],8877:[0,.68889,0,0,.61111],8878:[0,.68889,0,0,.72222],8879:[0,.68889,0,0,.72222],8882:[.03517,.54986,0,0,.77778],8883:[.03517,.54986,0,0,.77778],8884:[.13667,.63667,0,0,.77778],8885:[.13667,.63667,0,0,.77778],8888:[0,.54986,0,0,1.11111],8890:[.19444,.43056,0,0,.55556],8891:[.19444,.69224,0,0,.61111],8892:[.19444,.69224,0,0,.61111],8901:[0,.54986,0,0,.27778],8903:[.08167,.58167,0,0,.77778],8905:[.08167,.58167,0,0,.77778],8906:[.08167,.58167,0,0,.77778],8907:[0,.69224,0,0,.77778],8908:[0,.69224,0,0,.77778],8909:[-.03598,.46402,0,0,.77778],8910:[0,.54986,0,0,.76042],8911:[0,.54986,0,0,.76042],8912:[.03517,.54986,0,0,.77778],8913:[.03517,.54986,0,0,.77778],8914:[0,.54986,0,0,.66667],8915:[0,.54986,0,0,.66667],8916:[0,.69224,0,0,.66667],8918:[.0391,.5391,0,0,.77778],8919:[.0391,.5391,0,0,.77778],8920:[.03517,.54986,0,0,1.33334],8921:[.03517,.54986,0,0,1.33334],8922:[.38569,.88569,0,0,.77778],8923:[.38569,.88569,0,0,.77778],8926:[.13667,.63667,0,0,.77778],8927:[.13667,.63667,0,0,.77778],8928:[.30274,.79383,0,0,.77778],8929:[.30274,.79383,0,0,.77778],8934:[.23222,.74111,0,0,.77778],8935:[.23222,.74111,0,0,.77778],8936:[.23222,.74111,0,0,.77778],8937:[.23222,.74111,0,0,.77778],8938:[.20576,.70576,0,0,.77778],8939:[.20576,.70576,0,0,.77778],8940:[.30274,.79383,0,0,.77778],8941:[.30274,.79383,0,0,.77778],8994:[.19444,.69224,0,0,.77778],8995:[.19444,.69224,0,0,.77778],9416:[.15559,.69224,0,0,.90222],9484:[0,.69224,0,0,.5],9488:[0,.69224,0,0,.5],9492:[0,.37788,0,0,.5],9496:[0,.37788,0,0,.5],9585:[.19444,.68889,0,0,.88889],9586:[.19444,.74111,0,0,.88889],9632:[0,.675,0,0,.77778],9633:[0,.675,0,0,.77778],9650:[0,.54986,0,0,.72222],9651:[0,.54986,0,0,.72222],9654:[.03517,.54986,0,0,.77778],9660:[0,.54986,0,0,.72222],9661:[0,.54986,0,0,.72222],9664:[.03517,.54986,0,0,.77778],9674:[.11111,.69224,0,0,.66667],9733:[.19444,.69224,0,0,.94445],10003:[0,.69224,0,0,.83334],10016:[0,.69224,0,0,.83334],10731:[.11111,.69224,0,0,.66667],10846:[.19444,.75583,0,0,.61111],10877:[.13667,.63667,0,0,.77778],10878:[.13667,.63667,0,0,.77778],10885:[.25583,.75583,0,0,.77778],10886:[.25583,.75583,0,0,.77778],10887:[.13597,.63597,0,0,.77778],10888:[.13597,.63597,0,0,.77778],10889:[.26167,.75726,0,0,.77778],10890:[.26167,.75726,0,0,.77778],10891:[.48256,.98256,0,0,.77778],10892:[.48256,.98256,0,0,.77778],10901:[.13667,.63667,0,0,.77778],10902:[.13667,.63667,0,0,.77778],10933:[.25142,.75726,0,0,.77778],10934:[.25142,.75726,0,0,.77778],10935:[.26167,.75726,0,0,.77778],10936:[.26167,.75726,0,0,.77778],10937:[.26167,.75726,0,0,.77778],10938:[.26167,.75726,0,0,.77778],10949:[.25583,.75583,0,0,.77778],10950:[.25583,.75583,0,0,.77778],10955:[.28481,.79383,0,0,.77778],10956:[.28481,.79383,0,0,.77778],57350:[.08167,.58167,0,0,.22222],57351:[.08167,.58167,0,0,.38889],57352:[.08167,.58167,0,0,.77778],57353:[0,.43056,.04028,0,.66667],57356:[.25142,.75726,0,0,.77778],57357:[.25142,.75726,0,0,.77778],57358:[.41951,.91951,0,0,.77778],57359:[.30274,.79383,0,0,.77778],57360:[.30274,.79383,0,0,.77778],57361:[.41951,.91951,0,0,.77778],57366:[.25142,.75726,0,0,.77778],57367:[.25142,.75726,0,0,.77778],57368:[.25142,.75726,0,0,.77778],57369:[.25142,.75726,0,0,.77778],57370:[.13597,.63597,0,0,.77778],57371:[.13597,.63597,0,0,.77778]},"Caligraphic-Regular":{32:[0,0,0,0,.25],65:[0,.68333,0,.19445,.79847],66:[0,.68333,.03041,.13889,.65681],67:[0,.68333,.05834,.13889,.52653],68:[0,.68333,.02778,.08334,.77139],69:[0,.68333,.08944,.11111,.52778],70:[0,.68333,.09931,.11111,.71875],71:[.09722,.68333,.0593,.11111,.59487],72:[0,.68333,.00965,.11111,.84452],73:[0,.68333,.07382,0,.54452],74:[.09722,.68333,.18472,.16667,.67778],75:[0,.68333,.01445,.05556,.76195],76:[0,.68333,0,.13889,.68972],77:[0,.68333,0,.13889,1.2009],78:[0,.68333,.14736,.08334,.82049],79:[0,.68333,.02778,.11111,.79611],80:[0,.68333,.08222,.08334,.69556],81:[.09722,.68333,0,.11111,.81667],82:[0,.68333,0,.08334,.8475],83:[0,.68333,.075,.13889,.60556],84:[0,.68333,.25417,0,.54464],85:[0,.68333,.09931,.08334,.62583],86:[0,.68333,.08222,0,.61278],87:[0,.68333,.08222,.08334,.98778],88:[0,.68333,.14643,.13889,.7133],89:[.09722,.68333,.08222,.08334,.66834],90:[0,.68333,.07944,.13889,.72473],160:[0,0,0,0,.25]},"Fraktur-Regular":{32:[0,0,0,0,.25],33:[0,.69141,0,0,.29574],34:[0,.69141,0,0,.21471],38:[0,.69141,0,0,.73786],39:[0,.69141,0,0,.21201],40:[.24982,.74947,0,0,.38865],41:[.24982,.74947,0,0,.38865],42:[0,.62119,0,0,.27764],43:[.08319,.58283,0,0,.75623],44:[0,.10803,0,0,.27764],45:[.08319,.58283,0,0,.75623],46:[0,.10803,0,0,.27764],47:[.24982,.74947,0,0,.50181],48:[0,.47534,0,0,.50181],49:[0,.47534,0,0,.50181],50:[0,.47534,0,0,.50181],51:[.18906,.47534,0,0,.50181],52:[.18906,.47534,0,0,.50181],53:[.18906,.47534,0,0,.50181],54:[0,.69141,0,0,.50181],55:[.18906,.47534,0,0,.50181],56:[0,.69141,0,0,.50181],57:[.18906,.47534,0,0,.50181],58:[0,.47534,0,0,.21606],59:[.12604,.47534,0,0,.21606],61:[-.13099,.36866,0,0,.75623],63:[0,.69141,0,0,.36245],65:[0,.69141,0,0,.7176],66:[0,.69141,0,0,.88397],67:[0,.69141,0,0,.61254],68:[0,.69141,0,0,.83158],69:[0,.69141,0,0,.66278],70:[.12604,.69141,0,0,.61119],71:[0,.69141,0,0,.78539],72:[.06302,.69141,0,0,.7203],73:[0,.69141,0,0,.55448],74:[.12604,.69141,0,0,.55231],75:[0,.69141,0,0,.66845],76:[0,.69141,0,0,.66602],77:[0,.69141,0,0,1.04953],78:[0,.69141,0,0,.83212],79:[0,.69141,0,0,.82699],80:[.18906,.69141,0,0,.82753],81:[.03781,.69141,0,0,.82699],82:[0,.69141,0,0,.82807],83:[0,.69141,0,0,.82861],84:[0,.69141,0,0,.66899],85:[0,.69141,0,0,.64576],86:[0,.69141,0,0,.83131],87:[0,.69141,0,0,1.04602],88:[0,.69141,0,0,.71922],89:[.18906,.69141,0,0,.83293],90:[.12604,.69141,0,0,.60201],91:[.24982,.74947,0,0,.27764],93:[.24982,.74947,0,0,.27764],94:[0,.69141,0,0,.49965],97:[0,.47534,0,0,.50046],98:[0,.69141,0,0,.51315],99:[0,.47534,0,0,.38946],100:[0,.62119,0,0,.49857],101:[0,.47534,0,0,.40053],102:[.18906,.69141,0,0,.32626],103:[.18906,.47534,0,0,.5037],104:[.18906,.69141,0,0,.52126],105:[0,.69141,0,0,.27899],106:[0,.69141,0,0,.28088],107:[0,.69141,0,0,.38946],108:[0,.69141,0,0,.27953],109:[0,.47534,0,0,.76676],110:[0,.47534,0,0,.52666],111:[0,.47534,0,0,.48885],112:[.18906,.52396,0,0,.50046],113:[.18906,.47534,0,0,.48912],114:[0,.47534,0,0,.38919],115:[0,.47534,0,0,.44266],116:[0,.62119,0,0,.33301],117:[0,.47534,0,0,.5172],118:[0,.52396,0,0,.5118],119:[0,.52396,0,0,.77351],120:[.18906,.47534,0,0,.38865],121:[.18906,.47534,0,0,.49884],122:[.18906,.47534,0,0,.39054],160:[0,0,0,0,.25],8216:[0,.69141,0,0,.21471],8217:[0,.69141,0,0,.21471],58112:[0,.62119,0,0,.49749],58113:[0,.62119,0,0,.4983],58114:[.18906,.69141,0,0,.33328],58115:[.18906,.69141,0,0,.32923],58116:[.18906,.47534,0,0,.50343],58117:[0,.69141,0,0,.33301],58118:[0,.62119,0,0,.33409],58119:[0,.47534,0,0,.50073]},"Main-Bold":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.35],34:[0,.69444,0,0,.60278],35:[.19444,.69444,0,0,.95833],36:[.05556,.75,0,0,.575],37:[.05556,.75,0,0,.95833],38:[0,.69444,0,0,.89444],39:[0,.69444,0,0,.31944],40:[.25,.75,0,0,.44722],41:[.25,.75,0,0,.44722],42:[0,.75,0,0,.575],43:[.13333,.63333,0,0,.89444],44:[.19444,.15556,0,0,.31944],45:[0,.44444,0,0,.38333],46:[0,.15556,0,0,.31944],47:[.25,.75,0,0,.575],48:[0,.64444,0,0,.575],49:[0,.64444,0,0,.575],50:[0,.64444,0,0,.575],51:[0,.64444,0,0,.575],52:[0,.64444,0,0,.575],53:[0,.64444,0,0,.575],54:[0,.64444,0,0,.575],55:[0,.64444,0,0,.575],56:[0,.64444,0,0,.575],57:[0,.64444,0,0,.575],58:[0,.44444,0,0,.31944],59:[.19444,.44444,0,0,.31944],60:[.08556,.58556,0,0,.89444],61:[-.10889,.39111,0,0,.89444],62:[.08556,.58556,0,0,.89444],63:[0,.69444,0,0,.54305],64:[0,.69444,0,0,.89444],65:[0,.68611,0,0,.86944],66:[0,.68611,0,0,.81805],67:[0,.68611,0,0,.83055],68:[0,.68611,0,0,.88194],69:[0,.68611,0,0,.75555],70:[0,.68611,0,0,.72361],71:[0,.68611,0,0,.90416],72:[0,.68611,0,0,.9],73:[0,.68611,0,0,.43611],74:[0,.68611,0,0,.59444],75:[0,.68611,0,0,.90138],76:[0,.68611,0,0,.69166],77:[0,.68611,0,0,1.09166],78:[0,.68611,0,0,.9],79:[0,.68611,0,0,.86388],80:[0,.68611,0,0,.78611],81:[.19444,.68611,0,0,.86388],82:[0,.68611,0,0,.8625],83:[0,.68611,0,0,.63889],84:[0,.68611,0,0,.8],85:[0,.68611,0,0,.88472],86:[0,.68611,.01597,0,.86944],87:[0,.68611,.01597,0,1.18888],88:[0,.68611,0,0,.86944],89:[0,.68611,.02875,0,.86944],90:[0,.68611,0,0,.70277],91:[.25,.75,0,0,.31944],92:[.25,.75,0,0,.575],93:[.25,.75,0,0,.31944],94:[0,.69444,0,0,.575],95:[.31,.13444,.03194,0,.575],97:[0,.44444,0,0,.55902],98:[0,.69444,0,0,.63889],99:[0,.44444,0,0,.51111],100:[0,.69444,0,0,.63889],101:[0,.44444,0,0,.52708],102:[0,.69444,.10903,0,.35139],103:[.19444,.44444,.01597,0,.575],104:[0,.69444,0,0,.63889],105:[0,.69444,0,0,.31944],106:[.19444,.69444,0,0,.35139],107:[0,.69444,0,0,.60694],108:[0,.69444,0,0,.31944],109:[0,.44444,0,0,.95833],110:[0,.44444,0,0,.63889],111:[0,.44444,0,0,.575],112:[.19444,.44444,0,0,.63889],113:[.19444,.44444,0,0,.60694],114:[0,.44444,0,0,.47361],115:[0,.44444,0,0,.45361],116:[0,.63492,0,0,.44722],117:[0,.44444,0,0,.63889],118:[0,.44444,.01597,0,.60694],119:[0,.44444,.01597,0,.83055],120:[0,.44444,0,0,.60694],121:[.19444,.44444,.01597,0,.60694],122:[0,.44444,0,0,.51111],123:[.25,.75,0,0,.575],124:[.25,.75,0,0,.31944],125:[.25,.75,0,0,.575],126:[.35,.34444,0,0,.575],160:[0,0,0,0,.25],163:[0,.69444,0,0,.86853],168:[0,.69444,0,0,.575],172:[0,.44444,0,0,.76666],176:[0,.69444,0,0,.86944],177:[.13333,.63333,0,0,.89444],184:[.17014,0,0,0,.51111],198:[0,.68611,0,0,1.04166],215:[.13333,.63333,0,0,.89444],216:[.04861,.73472,0,0,.89444],223:[0,.69444,0,0,.59722],230:[0,.44444,0,0,.83055],247:[.13333,.63333,0,0,.89444],248:[.09722,.54167,0,0,.575],305:[0,.44444,0,0,.31944],338:[0,.68611,0,0,1.16944],339:[0,.44444,0,0,.89444],567:[.19444,.44444,0,0,.35139],710:[0,.69444,0,0,.575],711:[0,.63194,0,0,.575],713:[0,.59611,0,0,.575],714:[0,.69444,0,0,.575],715:[0,.69444,0,0,.575],728:[0,.69444,0,0,.575],729:[0,.69444,0,0,.31944],730:[0,.69444,0,0,.86944],732:[0,.69444,0,0,.575],733:[0,.69444,0,0,.575],915:[0,.68611,0,0,.69166],916:[0,.68611,0,0,.95833],920:[0,.68611,0,0,.89444],923:[0,.68611,0,0,.80555],926:[0,.68611,0,0,.76666],928:[0,.68611,0,0,.9],931:[0,.68611,0,0,.83055],933:[0,.68611,0,0,.89444],934:[0,.68611,0,0,.83055],936:[0,.68611,0,0,.89444],937:[0,.68611,0,0,.83055],8211:[0,.44444,.03194,0,.575],8212:[0,.44444,.03194,0,1.14999],8216:[0,.69444,0,0,.31944],8217:[0,.69444,0,0,.31944],8220:[0,.69444,0,0,.60278],8221:[0,.69444,0,0,.60278],8224:[.19444,.69444,0,0,.51111],8225:[.19444,.69444,0,0,.51111],8242:[0,.55556,0,0,.34444],8407:[0,.72444,.15486,0,.575],8463:[0,.69444,0,0,.66759],8465:[0,.69444,0,0,.83055],8467:[0,.69444,0,0,.47361],8472:[.19444,.44444,0,0,.74027],8476:[0,.69444,0,0,.83055],8501:[0,.69444,0,0,.70277],8592:[-.10889,.39111,0,0,1.14999],8593:[.19444,.69444,0,0,.575],8594:[-.10889,.39111,0,0,1.14999],8595:[.19444,.69444,0,0,.575],8596:[-.10889,.39111,0,0,1.14999],8597:[.25,.75,0,0,.575],8598:[.19444,.69444,0,0,1.14999],8599:[.19444,.69444,0,0,1.14999],8600:[.19444,.69444,0,0,1.14999],8601:[.19444,.69444,0,0,1.14999],8636:[-.10889,.39111,0,0,1.14999],8637:[-.10889,.39111,0,0,1.14999],8640:[-.10889,.39111,0,0,1.14999],8641:[-.10889,.39111,0,0,1.14999],8656:[-.10889,.39111,0,0,1.14999],8657:[.19444,.69444,0,0,.70277],8658:[-.10889,.39111,0,0,1.14999],8659:[.19444,.69444,0,0,.70277],8660:[-.10889,.39111,0,0,1.14999],8661:[.25,.75,0,0,.70277],8704:[0,.69444,0,0,.63889],8706:[0,.69444,.06389,0,.62847],8707:[0,.69444,0,0,.63889],8709:[.05556,.75,0,0,.575],8711:[0,.68611,0,0,.95833],8712:[.08556,.58556,0,0,.76666],8715:[.08556,.58556,0,0,.76666],8722:[.13333,.63333,0,0,.89444],8723:[.13333,.63333,0,0,.89444],8725:[.25,.75,0,0,.575],8726:[.25,.75,0,0,.575],8727:[-.02778,.47222,0,0,.575],8728:[-.02639,.47361,0,0,.575],8729:[-.02639,.47361,0,0,.575],8730:[.18,.82,0,0,.95833],8733:[0,.44444,0,0,.89444],8734:[0,.44444,0,0,1.14999],8736:[0,.69224,0,0,.72222],8739:[.25,.75,0,0,.31944],8741:[.25,.75,0,0,.575],8743:[0,.55556,0,0,.76666],8744:[0,.55556,0,0,.76666],8745:[0,.55556,0,0,.76666],8746:[0,.55556,0,0,.76666],8747:[.19444,.69444,.12778,0,.56875],8764:[-.10889,.39111,0,0,.89444],8768:[.19444,.69444,0,0,.31944],8771:[.00222,.50222,0,0,.89444],8773:[.027,.638,0,0,.894],8776:[.02444,.52444,0,0,.89444],8781:[.00222,.50222,0,0,.89444],8801:[.00222,.50222,0,0,.89444],8804:[.19667,.69667,0,0,.89444],8805:[.19667,.69667,0,0,.89444],8810:[.08556,.58556,0,0,1.14999],8811:[.08556,.58556,0,0,1.14999],8826:[.08556,.58556,0,0,.89444],8827:[.08556,.58556,0,0,.89444],8834:[.08556,.58556,0,0,.89444],8835:[.08556,.58556,0,0,.89444],8838:[.19667,.69667,0,0,.89444],8839:[.19667,.69667,0,0,.89444],8846:[0,.55556,0,0,.76666],8849:[.19667,.69667,0,0,.89444],8850:[.19667,.69667,0,0,.89444],8851:[0,.55556,0,0,.76666],8852:[0,.55556,0,0,.76666],8853:[.13333,.63333,0,0,.89444],8854:[.13333,.63333,0,0,.89444],8855:[.13333,.63333,0,0,.89444],8856:[.13333,.63333,0,0,.89444],8857:[.13333,.63333,0,0,.89444],8866:[0,.69444,0,0,.70277],8867:[0,.69444,0,0,.70277],8868:[0,.69444,0,0,.89444],8869:[0,.69444,0,0,.89444],8900:[-.02639,.47361,0,0,.575],8901:[-.02639,.47361,0,0,.31944],8902:[-.02778,.47222,0,0,.575],8968:[.25,.75,0,0,.51111],8969:[.25,.75,0,0,.51111],8970:[.25,.75,0,0,.51111],8971:[.25,.75,0,0,.51111],8994:[-.13889,.36111,0,0,1.14999],8995:[-.13889,.36111,0,0,1.14999],9651:[.19444,.69444,0,0,1.02222],9657:[-.02778,.47222,0,0,.575],9661:[.19444,.69444,0,0,1.02222],9667:[-.02778,.47222,0,0,.575],9711:[.19444,.69444,0,0,1.14999],9824:[.12963,.69444,0,0,.89444],9825:[.12963,.69444,0,0,.89444],9826:[.12963,.69444,0,0,.89444],9827:[.12963,.69444,0,0,.89444],9837:[0,.75,0,0,.44722],9838:[.19444,.69444,0,0,.44722],9839:[.19444,.69444,0,0,.44722],10216:[.25,.75,0,0,.44722],10217:[.25,.75,0,0,.44722],10815:[0,.68611,0,0,.9],10927:[.19667,.69667,0,0,.89444],10928:[.19667,.69667,0,0,.89444],57376:[.19444,.69444,0,0,0]},"Main-BoldItalic":{32:[0,0,0,0,.25],33:[0,.69444,.11417,0,.38611],34:[0,.69444,.07939,0,.62055],35:[.19444,.69444,.06833,0,.94444],37:[.05556,.75,.12861,0,.94444],38:[0,.69444,.08528,0,.88555],39:[0,.69444,.12945,0,.35555],40:[.25,.75,.15806,0,.47333],41:[.25,.75,.03306,0,.47333],42:[0,.75,.14333,0,.59111],43:[.10333,.60333,.03306,0,.88555],44:[.19444,.14722,0,0,.35555],45:[0,.44444,.02611,0,.41444],46:[0,.14722,0,0,.35555],47:[.25,.75,.15806,0,.59111],48:[0,.64444,.13167,0,.59111],49:[0,.64444,.13167,0,.59111],50:[0,.64444,.13167,0,.59111],51:[0,.64444,.13167,0,.59111],52:[.19444,.64444,.13167,0,.59111],53:[0,.64444,.13167,0,.59111],54:[0,.64444,.13167,0,.59111],55:[.19444,.64444,.13167,0,.59111],56:[0,.64444,.13167,0,.59111],57:[0,.64444,.13167,0,.59111],58:[0,.44444,.06695,0,.35555],59:[.19444,.44444,.06695,0,.35555],61:[-.10889,.39111,.06833,0,.88555],63:[0,.69444,.11472,0,.59111],64:[0,.69444,.09208,0,.88555],65:[0,.68611,0,0,.86555],66:[0,.68611,.0992,0,.81666],67:[0,.68611,.14208,0,.82666],68:[0,.68611,.09062,0,.87555],69:[0,.68611,.11431,0,.75666],70:[0,.68611,.12903,0,.72722],71:[0,.68611,.07347,0,.89527],72:[0,.68611,.17208,0,.8961],73:[0,.68611,.15681,0,.47166],74:[0,.68611,.145,0,.61055],75:[0,.68611,.14208,0,.89499],76:[0,.68611,0,0,.69777],77:[0,.68611,.17208,0,1.07277],78:[0,.68611,.17208,0,.8961],79:[0,.68611,.09062,0,.85499],80:[0,.68611,.0992,0,.78721],81:[.19444,.68611,.09062,0,.85499],82:[0,.68611,.02559,0,.85944],83:[0,.68611,.11264,0,.64999],84:[0,.68611,.12903,0,.7961],85:[0,.68611,.17208,0,.88083],86:[0,.68611,.18625,0,.86555],87:[0,.68611,.18625,0,1.15999],88:[0,.68611,.15681,0,.86555],89:[0,.68611,.19803,0,.86555],90:[0,.68611,.14208,0,.70888],91:[.25,.75,.1875,0,.35611],93:[.25,.75,.09972,0,.35611],94:[0,.69444,.06709,0,.59111],95:[.31,.13444,.09811,0,.59111],97:[0,.44444,.09426,0,.59111],98:[0,.69444,.07861,0,.53222],99:[0,.44444,.05222,0,.53222],100:[0,.69444,.10861,0,.59111],101:[0,.44444,.085,0,.53222],102:[.19444,.69444,.21778,0,.4],103:[.19444,.44444,.105,0,.53222],104:[0,.69444,.09426,0,.59111],105:[0,.69326,.11387,0,.35555],106:[.19444,.69326,.1672,0,.35555],107:[0,.69444,.11111,0,.53222],108:[0,.69444,.10861,0,.29666],109:[0,.44444,.09426,0,.94444],110:[0,.44444,.09426,0,.64999],111:[0,.44444,.07861,0,.59111],112:[.19444,.44444,.07861,0,.59111],113:[.19444,.44444,.105,0,.53222],114:[0,.44444,.11111,0,.50167],115:[0,.44444,.08167,0,.48694],116:[0,.63492,.09639,0,.385],117:[0,.44444,.09426,0,.62055],118:[0,.44444,.11111,0,.53222],119:[0,.44444,.11111,0,.76777],120:[0,.44444,.12583,0,.56055],121:[.19444,.44444,.105,0,.56166],122:[0,.44444,.13889,0,.49055],126:[.35,.34444,.11472,0,.59111],160:[0,0,0,0,.25],168:[0,.69444,.11473,0,.59111],176:[0,.69444,0,0,.94888],184:[.17014,0,0,0,.53222],198:[0,.68611,.11431,0,1.02277],216:[.04861,.73472,.09062,0,.88555],223:[.19444,.69444,.09736,0,.665],230:[0,.44444,.085,0,.82666],248:[.09722,.54167,.09458,0,.59111],305:[0,.44444,.09426,0,.35555],338:[0,.68611,.11431,0,1.14054],339:[0,.44444,.085,0,.82666],567:[.19444,.44444,.04611,0,.385],710:[0,.69444,.06709,0,.59111],711:[0,.63194,.08271,0,.59111],713:[0,.59444,.10444,0,.59111],714:[0,.69444,.08528,0,.59111],715:[0,.69444,0,0,.59111],728:[0,.69444,.10333,0,.59111],729:[0,.69444,.12945,0,.35555],730:[0,.69444,0,0,.94888],732:[0,.69444,.11472,0,.59111],733:[0,.69444,.11472,0,.59111],915:[0,.68611,.12903,0,.69777],916:[0,.68611,0,0,.94444],920:[0,.68611,.09062,0,.88555],923:[0,.68611,0,0,.80666],926:[0,.68611,.15092,0,.76777],928:[0,.68611,.17208,0,.8961],931:[0,.68611,.11431,0,.82666],933:[0,.68611,.10778,0,.88555],934:[0,.68611,.05632,0,.82666],936:[0,.68611,.10778,0,.88555],937:[0,.68611,.0992,0,.82666],8211:[0,.44444,.09811,0,.59111],8212:[0,.44444,.09811,0,1.18221],8216:[0,.69444,.12945,0,.35555],8217:[0,.69444,.12945,0,.35555],8220:[0,.69444,.16772,0,.62055],8221:[0,.69444,.07939,0,.62055]},"Main-Italic":{32:[0,0,0,0,.25],33:[0,.69444,.12417,0,.30667],34:[0,.69444,.06961,0,.51444],35:[.19444,.69444,.06616,0,.81777],37:[.05556,.75,.13639,0,.81777],38:[0,.69444,.09694,0,.76666],39:[0,.69444,.12417,0,.30667],40:[.25,.75,.16194,0,.40889],41:[.25,.75,.03694,0,.40889],42:[0,.75,.14917,0,.51111],43:[.05667,.56167,.03694,0,.76666],44:[.19444,.10556,0,0,.30667],45:[0,.43056,.02826,0,.35778],46:[0,.10556,0,0,.30667],47:[.25,.75,.16194,0,.51111],48:[0,.64444,.13556,0,.51111],49:[0,.64444,.13556,0,.51111],50:[0,.64444,.13556,0,.51111],51:[0,.64444,.13556,0,.51111],52:[.19444,.64444,.13556,0,.51111],53:[0,.64444,.13556,0,.51111],54:[0,.64444,.13556,0,.51111],55:[.19444,.64444,.13556,0,.51111],56:[0,.64444,.13556,0,.51111],57:[0,.64444,.13556,0,.51111],58:[0,.43056,.0582,0,.30667],59:[.19444,.43056,.0582,0,.30667],61:[-.13313,.36687,.06616,0,.76666],63:[0,.69444,.1225,0,.51111],64:[0,.69444,.09597,0,.76666],65:[0,.68333,0,0,.74333],66:[0,.68333,.10257,0,.70389],67:[0,.68333,.14528,0,.71555],68:[0,.68333,.09403,0,.755],69:[0,.68333,.12028,0,.67833],70:[0,.68333,.13305,0,.65277],71:[0,.68333,.08722,0,.77361],72:[0,.68333,.16389,0,.74333],73:[0,.68333,.15806,0,.38555],74:[0,.68333,.14028,0,.525],75:[0,.68333,.14528,0,.76888],76:[0,.68333,0,0,.62722],77:[0,.68333,.16389,0,.89666],78:[0,.68333,.16389,0,.74333],79:[0,.68333,.09403,0,.76666],80:[0,.68333,.10257,0,.67833],81:[.19444,.68333,.09403,0,.76666],82:[0,.68333,.03868,0,.72944],83:[0,.68333,.11972,0,.56222],84:[0,.68333,.13305,0,.71555],85:[0,.68333,.16389,0,.74333],86:[0,.68333,.18361,0,.74333],87:[0,.68333,.18361,0,.99888],88:[0,.68333,.15806,0,.74333],89:[0,.68333,.19383,0,.74333],90:[0,.68333,.14528,0,.61333],91:[.25,.75,.1875,0,.30667],93:[.25,.75,.10528,0,.30667],94:[0,.69444,.06646,0,.51111],95:[.31,.12056,.09208,0,.51111],97:[0,.43056,.07671,0,.51111],98:[0,.69444,.06312,0,.46],99:[0,.43056,.05653,0,.46],100:[0,.69444,.10333,0,.51111],101:[0,.43056,.07514,0,.46],102:[.19444,.69444,.21194,0,.30667],103:[.19444,.43056,.08847,0,.46],104:[0,.69444,.07671,0,.51111],105:[0,.65536,.1019,0,.30667],106:[.19444,.65536,.14467,0,.30667],107:[0,.69444,.10764,0,.46],108:[0,.69444,.10333,0,.25555],109:[0,.43056,.07671,0,.81777],110:[0,.43056,.07671,0,.56222],111:[0,.43056,.06312,0,.51111],112:[.19444,.43056,.06312,0,.51111],113:[.19444,.43056,.08847,0,.46],114:[0,.43056,.10764,0,.42166],115:[0,.43056,.08208,0,.40889],116:[0,.61508,.09486,0,.33222],117:[0,.43056,.07671,0,.53666],118:[0,.43056,.10764,0,.46],119:[0,.43056,.10764,0,.66444],120:[0,.43056,.12042,0,.46389],121:[.19444,.43056,.08847,0,.48555],122:[0,.43056,.12292,0,.40889],126:[.35,.31786,.11585,0,.51111],160:[0,0,0,0,.25],168:[0,.66786,.10474,0,.51111],176:[0,.69444,0,0,.83129],184:[.17014,0,0,0,.46],198:[0,.68333,.12028,0,.88277],216:[.04861,.73194,.09403,0,.76666],223:[.19444,.69444,.10514,0,.53666],230:[0,.43056,.07514,0,.71555],248:[.09722,.52778,.09194,0,.51111],338:[0,.68333,.12028,0,.98499],339:[0,.43056,.07514,0,.71555],710:[0,.69444,.06646,0,.51111],711:[0,.62847,.08295,0,.51111],713:[0,.56167,.10333,0,.51111],714:[0,.69444,.09694,0,.51111],715:[0,.69444,0,0,.51111],728:[0,.69444,.10806,0,.51111],729:[0,.66786,.11752,0,.30667],730:[0,.69444,0,0,.83129],732:[0,.66786,.11585,0,.51111],733:[0,.69444,.1225,0,.51111],915:[0,.68333,.13305,0,.62722],916:[0,.68333,0,0,.81777],920:[0,.68333,.09403,0,.76666],923:[0,.68333,0,0,.69222],926:[0,.68333,.15294,0,.66444],928:[0,.68333,.16389,0,.74333],931:[0,.68333,.12028,0,.71555],933:[0,.68333,.11111,0,.76666],934:[0,.68333,.05986,0,.71555],936:[0,.68333,.11111,0,.76666],937:[0,.68333,.10257,0,.71555],8211:[0,.43056,.09208,0,.51111],8212:[0,.43056,.09208,0,1.02222],8216:[0,.69444,.12417,0,.30667],8217:[0,.69444,.12417,0,.30667],8220:[0,.69444,.1685,0,.51444],8221:[0,.69444,.06961,0,.51444],8463:[0,.68889,0,0,.54028]},"Main-Regular":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.27778],34:[0,.69444,0,0,.5],35:[.19444,.69444,0,0,.83334],36:[.05556,.75,0,0,.5],37:[.05556,.75,0,0,.83334],38:[0,.69444,0,0,.77778],39:[0,.69444,0,0,.27778],40:[.25,.75,0,0,.38889],41:[.25,.75,0,0,.38889],42:[0,.75,0,0,.5],43:[.08333,.58333,0,0,.77778],44:[.19444,.10556,0,0,.27778],45:[0,.43056,0,0,.33333],46:[0,.10556,0,0,.27778],47:[.25,.75,0,0,.5],48:[0,.64444,0,0,.5],49:[0,.64444,0,0,.5],50:[0,.64444,0,0,.5],51:[0,.64444,0,0,.5],52:[0,.64444,0,0,.5],53:[0,.64444,0,0,.5],54:[0,.64444,0,0,.5],55:[0,.64444,0,0,.5],56:[0,.64444,0,0,.5],57:[0,.64444,0,0,.5],58:[0,.43056,0,0,.27778],59:[.19444,.43056,0,0,.27778],60:[.0391,.5391,0,0,.77778],61:[-.13313,.36687,0,0,.77778],62:[.0391,.5391,0,0,.77778],63:[0,.69444,0,0,.47222],64:[0,.69444,0,0,.77778],65:[0,.68333,0,0,.75],66:[0,.68333,0,0,.70834],67:[0,.68333,0,0,.72222],68:[0,.68333,0,0,.76389],69:[0,.68333,0,0,.68056],70:[0,.68333,0,0,.65278],71:[0,.68333,0,0,.78472],72:[0,.68333,0,0,.75],73:[0,.68333,0,0,.36111],74:[0,.68333,0,0,.51389],75:[0,.68333,0,0,.77778],76:[0,.68333,0,0,.625],77:[0,.68333,0,0,.91667],78:[0,.68333,0,0,.75],79:[0,.68333,0,0,.77778],80:[0,.68333,0,0,.68056],81:[.19444,.68333,0,0,.77778],82:[0,.68333,0,0,.73611],83:[0,.68333,0,0,.55556],84:[0,.68333,0,0,.72222],85:[0,.68333,0,0,.75],86:[0,.68333,.01389,0,.75],87:[0,.68333,.01389,0,1.02778],88:[0,.68333,0,0,.75],89:[0,.68333,.025,0,.75],90:[0,.68333,0,0,.61111],91:[.25,.75,0,0,.27778],92:[.25,.75,0,0,.5],93:[.25,.75,0,0,.27778],94:[0,.69444,0,0,.5],95:[.31,.12056,.02778,0,.5],97:[0,.43056,0,0,.5],98:[0,.69444,0,0,.55556],99:[0,.43056,0,0,.44445],100:[0,.69444,0,0,.55556],101:[0,.43056,0,0,.44445],102:[0,.69444,.07778,0,.30556],103:[.19444,.43056,.01389,0,.5],104:[0,.69444,0,0,.55556],105:[0,.66786,0,0,.27778],106:[.19444,.66786,0,0,.30556],107:[0,.69444,0,0,.52778],108:[0,.69444,0,0,.27778],109:[0,.43056,0,0,.83334],110:[0,.43056,0,0,.55556],111:[0,.43056,0,0,.5],112:[.19444,.43056,0,0,.55556],113:[.19444,.43056,0,0,.52778],114:[0,.43056,0,0,.39167],115:[0,.43056,0,0,.39445],116:[0,.61508,0,0,.38889],117:[0,.43056,0,0,.55556],118:[0,.43056,.01389,0,.52778],119:[0,.43056,.01389,0,.72222],120:[0,.43056,0,0,.52778],121:[.19444,.43056,.01389,0,.52778],122:[0,.43056,0,0,.44445],123:[.25,.75,0,0,.5],124:[.25,.75,0,0,.27778],125:[.25,.75,0,0,.5],126:[.35,.31786,0,0,.5],160:[0,0,0,0,.25],163:[0,.69444,0,0,.76909],167:[.19444,.69444,0,0,.44445],168:[0,.66786,0,0,.5],172:[0,.43056,0,0,.66667],176:[0,.69444,0,0,.75],177:[.08333,.58333,0,0,.77778],182:[.19444,.69444,0,0,.61111],184:[.17014,0,0,0,.44445],198:[0,.68333,0,0,.90278],215:[.08333,.58333,0,0,.77778],216:[.04861,.73194,0,0,.77778],223:[0,.69444,0,0,.5],230:[0,.43056,0,0,.72222],247:[.08333,.58333,0,0,.77778],248:[.09722,.52778,0,0,.5],305:[0,.43056,0,0,.27778],338:[0,.68333,0,0,1.01389],339:[0,.43056,0,0,.77778],567:[.19444,.43056,0,0,.30556],710:[0,.69444,0,0,.5],711:[0,.62847,0,0,.5],713:[0,.56778,0,0,.5],714:[0,.69444,0,0,.5],715:[0,.69444,0,0,.5],728:[0,.69444,0,0,.5],729:[0,.66786,0,0,.27778],730:[0,.69444,0,0,.75],732:[0,.66786,0,0,.5],733:[0,.69444,0,0,.5],915:[0,.68333,0,0,.625],916:[0,.68333,0,0,.83334],920:[0,.68333,0,0,.77778],923:[0,.68333,0,0,.69445],926:[0,.68333,0,0,.66667],928:[0,.68333,0,0,.75],931:[0,.68333,0,0,.72222],933:[0,.68333,0,0,.77778],934:[0,.68333,0,0,.72222],936:[0,.68333,0,0,.77778],937:[0,.68333,0,0,.72222],8211:[0,.43056,.02778,0,.5],8212:[0,.43056,.02778,0,1],8216:[0,.69444,0,0,.27778],8217:[0,.69444,0,0,.27778],8220:[0,.69444,0,0,.5],8221:[0,.69444,0,0,.5],8224:[.19444,.69444,0,0,.44445],8225:[.19444,.69444,0,0,.44445],8230:[0,.123,0,0,1.172],8242:[0,.55556,0,0,.275],8407:[0,.71444,.15382,0,.5],8463:[0,.68889,0,0,.54028],8465:[0,.69444,0,0,.72222],8467:[0,.69444,0,.11111,.41667],8472:[.19444,.43056,0,.11111,.63646],8476:[0,.69444,0,0,.72222],8501:[0,.69444,0,0,.61111],8592:[-.13313,.36687,0,0,1],8593:[.19444,.69444,0,0,.5],8594:[-.13313,.36687,0,0,1],8595:[.19444,.69444,0,0,.5],8596:[-.13313,.36687,0,0,1],8597:[.25,.75,0,0,.5],8598:[.19444,.69444,0,0,1],8599:[.19444,.69444,0,0,1],8600:[.19444,.69444,0,0,1],8601:[.19444,.69444,0,0,1],8614:[.011,.511,0,0,1],8617:[.011,.511,0,0,1.126],8618:[.011,.511,0,0,1.126],8636:[-.13313,.36687,0,0,1],8637:[-.13313,.36687,0,0,1],8640:[-.13313,.36687,0,0,1],8641:[-.13313,.36687,0,0,1],8652:[.011,.671,0,0,1],8656:[-.13313,.36687,0,0,1],8657:[.19444,.69444,0,0,.61111],8658:[-.13313,.36687,0,0,1],8659:[.19444,.69444,0,0,.61111],8660:[-.13313,.36687,0,0,1],8661:[.25,.75,0,0,.61111],8704:[0,.69444,0,0,.55556],8706:[0,.69444,.05556,.08334,.5309],8707:[0,.69444,0,0,.55556],8709:[.05556,.75,0,0,.5],8711:[0,.68333,0,0,.83334],8712:[.0391,.5391,0,0,.66667],8715:[.0391,.5391,0,0,.66667],8722:[.08333,.58333,0,0,.77778],8723:[.08333,.58333,0,0,.77778],8725:[.25,.75,0,0,.5],8726:[.25,.75,0,0,.5],8727:[-.03472,.46528,0,0,.5],8728:[-.05555,.44445,0,0,.5],8729:[-.05555,.44445,0,0,.5],8730:[.2,.8,0,0,.83334],8733:[0,.43056,0,0,.77778],8734:[0,.43056,0,0,1],8736:[0,.69224,0,0,.72222],8739:[.25,.75,0,0,.27778],8741:[.25,.75,0,0,.5],8743:[0,.55556,0,0,.66667],8744:[0,.55556,0,0,.66667],8745:[0,.55556,0,0,.66667],8746:[0,.55556,0,0,.66667],8747:[.19444,.69444,.11111,0,.41667],8764:[-.13313,.36687,0,0,.77778],8768:[.19444,.69444,0,0,.27778],8771:[-.03625,.46375,0,0,.77778],8773:[-.022,.589,0,0,.778],8776:[-.01688,.48312,0,0,.77778],8781:[-.03625,.46375,0,0,.77778],8784:[-.133,.673,0,0,.778],8801:[-.03625,.46375,0,0,.77778],8804:[.13597,.63597,0,0,.77778],8805:[.13597,.63597,0,0,.77778],8810:[.0391,.5391,0,0,1],8811:[.0391,.5391,0,0,1],8826:[.0391,.5391,0,0,.77778],8827:[.0391,.5391,0,0,.77778],8834:[.0391,.5391,0,0,.77778],8835:[.0391,.5391,0,0,.77778],8838:[.13597,.63597,0,0,.77778],8839:[.13597,.63597,0,0,.77778],8846:[0,.55556,0,0,.66667],8849:[.13597,.63597,0,0,.77778],8850:[.13597,.63597,0,0,.77778],8851:[0,.55556,0,0,.66667],8852:[0,.55556,0,0,.66667],8853:[.08333,.58333,0,0,.77778],8854:[.08333,.58333,0,0,.77778],8855:[.08333,.58333,0,0,.77778],8856:[.08333,.58333,0,0,.77778],8857:[.08333,.58333,0,0,.77778],8866:[0,.69444,0,0,.61111],8867:[0,.69444,0,0,.61111],8868:[0,.69444,0,0,.77778],8869:[0,.69444,0,0,.77778],8872:[.249,.75,0,0,.867],8900:[-.05555,.44445,0,0,.5],8901:[-.05555,.44445,0,0,.27778],8902:[-.03472,.46528,0,0,.5],8904:[.005,.505,0,0,.9],8942:[.03,.903,0,0,.278],8943:[-.19,.313,0,0,1.172],8945:[-.1,.823,0,0,1.282],8968:[.25,.75,0,0,.44445],8969:[.25,.75,0,0,.44445],8970:[.25,.75,0,0,.44445],8971:[.25,.75,0,0,.44445],8994:[-.14236,.35764,0,0,1],8995:[-.14236,.35764,0,0,1],9136:[.244,.744,0,0,.412],9137:[.244,.745,0,0,.412],9651:[.19444,.69444,0,0,.88889],9657:[-.03472,.46528,0,0,.5],9661:[.19444,.69444,0,0,.88889],9667:[-.03472,.46528,0,0,.5],9711:[.19444,.69444,0,0,1],9824:[.12963,.69444,0,0,.77778],9825:[.12963,.69444,0,0,.77778],9826:[.12963,.69444,0,0,.77778],9827:[.12963,.69444,0,0,.77778],9837:[0,.75,0,0,.38889],9838:[.19444,.69444,0,0,.38889],9839:[.19444,.69444,0,0,.38889],10216:[.25,.75,0,0,.38889],10217:[.25,.75,0,0,.38889],10222:[.244,.744,0,0,.412],10223:[.244,.745,0,0,.412],10229:[.011,.511,0,0,1.609],10230:[.011,.511,0,0,1.638],10231:[.011,.511,0,0,1.859],10232:[.024,.525,0,0,1.609],10233:[.024,.525,0,0,1.638],10234:[.024,.525,0,0,1.858],10236:[.011,.511,0,0,1.638],10815:[0,.68333,0,0,.75],10927:[.13597,.63597,0,0,.77778],10928:[.13597,.63597,0,0,.77778],57376:[.19444,.69444,0,0,0]},"Math-BoldItalic":{32:[0,0,0,0,.25],48:[0,.44444,0,0,.575],49:[0,.44444,0,0,.575],50:[0,.44444,0,0,.575],51:[.19444,.44444,0,0,.575],52:[.19444,.44444,0,0,.575],53:[.19444,.44444,0,0,.575],54:[0,.64444,0,0,.575],55:[.19444,.44444,0,0,.575],56:[0,.64444,0,0,.575],57:[.19444,.44444,0,0,.575],65:[0,.68611,0,0,.86944],66:[0,.68611,.04835,0,.8664],67:[0,.68611,.06979,0,.81694],68:[0,.68611,.03194,0,.93812],69:[0,.68611,.05451,0,.81007],70:[0,.68611,.15972,0,.68889],71:[0,.68611,0,0,.88673],72:[0,.68611,.08229,0,.98229],73:[0,.68611,.07778,0,.51111],74:[0,.68611,.10069,0,.63125],75:[0,.68611,.06979,0,.97118],76:[0,.68611,0,0,.75555],77:[0,.68611,.11424,0,1.14201],78:[0,.68611,.11424,0,.95034],79:[0,.68611,.03194,0,.83666],80:[0,.68611,.15972,0,.72309],81:[.19444,.68611,0,0,.86861],82:[0,.68611,.00421,0,.87235],83:[0,.68611,.05382,0,.69271],84:[0,.68611,.15972,0,.63663],85:[0,.68611,.11424,0,.80027],86:[0,.68611,.25555,0,.67778],87:[0,.68611,.15972,0,1.09305],88:[0,.68611,.07778,0,.94722],89:[0,.68611,.25555,0,.67458],90:[0,.68611,.06979,0,.77257],97:[0,.44444,0,0,.63287],98:[0,.69444,0,0,.52083],99:[0,.44444,0,0,.51342],100:[0,.69444,0,0,.60972],101:[0,.44444,0,0,.55361],102:[.19444,.69444,.11042,0,.56806],103:[.19444,.44444,.03704,0,.5449],104:[0,.69444,0,0,.66759],105:[0,.69326,0,0,.4048],106:[.19444,.69326,.0622,0,.47083],107:[0,.69444,.01852,0,.6037],108:[0,.69444,.0088,0,.34815],109:[0,.44444,0,0,1.0324],110:[0,.44444,0,0,.71296],111:[0,.44444,0,0,.58472],112:[.19444,.44444,0,0,.60092],113:[.19444,.44444,.03704,0,.54213],114:[0,.44444,.03194,0,.5287],115:[0,.44444,0,0,.53125],116:[0,.63492,0,0,.41528],117:[0,.44444,0,0,.68102],118:[0,.44444,.03704,0,.56666],119:[0,.44444,.02778,0,.83148],120:[0,.44444,0,0,.65903],121:[.19444,.44444,.03704,0,.59028],122:[0,.44444,.04213,0,.55509],160:[0,0,0,0,.25],915:[0,.68611,.15972,0,.65694],916:[0,.68611,0,0,.95833],920:[0,.68611,.03194,0,.86722],923:[0,.68611,0,0,.80555],926:[0,.68611,.07458,0,.84125],928:[0,.68611,.08229,0,.98229],931:[0,.68611,.05451,0,.88507],933:[0,.68611,.15972,0,.67083],934:[0,.68611,0,0,.76666],936:[0,.68611,.11653,0,.71402],937:[0,.68611,.04835,0,.8789],945:[0,.44444,0,0,.76064],946:[.19444,.69444,.03403,0,.65972],947:[.19444,.44444,.06389,0,.59003],948:[0,.69444,.03819,0,.52222],949:[0,.44444,0,0,.52882],950:[.19444,.69444,.06215,0,.50833],951:[.19444,.44444,.03704,0,.6],952:[0,.69444,.03194,0,.5618],953:[0,.44444,0,0,.41204],954:[0,.44444,0,0,.66759],955:[0,.69444,0,0,.67083],956:[.19444,.44444,0,0,.70787],957:[0,.44444,.06898,0,.57685],958:[.19444,.69444,.03021,0,.50833],959:[0,.44444,0,0,.58472],960:[0,.44444,.03704,0,.68241],961:[.19444,.44444,0,0,.6118],962:[.09722,.44444,.07917,0,.42361],963:[0,.44444,.03704,0,.68588],964:[0,.44444,.13472,0,.52083],965:[0,.44444,.03704,0,.63055],966:[.19444,.44444,0,0,.74722],967:[.19444,.44444,0,0,.71805],968:[.19444,.69444,.03704,0,.75833],969:[0,.44444,.03704,0,.71782],977:[0,.69444,0,0,.69155],981:[.19444,.69444,0,0,.7125],982:[0,.44444,.03194,0,.975],1009:[.19444,.44444,0,0,.6118],1013:[0,.44444,0,0,.48333],57649:[0,.44444,0,0,.39352],57911:[.19444,.44444,0,0,.43889]},"Math-Italic":{32:[0,0,0,0,.25],48:[0,.43056,0,0,.5],49:[0,.43056,0,0,.5],50:[0,.43056,0,0,.5],51:[.19444,.43056,0,0,.5],52:[.19444,.43056,0,0,.5],53:[.19444,.43056,0,0,.5],54:[0,.64444,0,0,.5],55:[.19444,.43056,0,0,.5],56:[0,.64444,0,0,.5],57:[.19444,.43056,0,0,.5],65:[0,.68333,0,.13889,.75],66:[0,.68333,.05017,.08334,.75851],67:[0,.68333,.07153,.08334,.71472],68:[0,.68333,.02778,.05556,.82792],69:[0,.68333,.05764,.08334,.7382],70:[0,.68333,.13889,.08334,.64306],71:[0,.68333,0,.08334,.78625],72:[0,.68333,.08125,.05556,.83125],73:[0,.68333,.07847,.11111,.43958],74:[0,.68333,.09618,.16667,.55451],75:[0,.68333,.07153,.05556,.84931],76:[0,.68333,0,.02778,.68056],77:[0,.68333,.10903,.08334,.97014],78:[0,.68333,.10903,.08334,.80347],79:[0,.68333,.02778,.08334,.76278],80:[0,.68333,.13889,.08334,.64201],81:[.19444,.68333,0,.08334,.79056],82:[0,.68333,.00773,.08334,.75929],83:[0,.68333,.05764,.08334,.6132],84:[0,.68333,.13889,.08334,.58438],85:[0,.68333,.10903,.02778,.68278],86:[0,.68333,.22222,0,.58333],87:[0,.68333,.13889,0,.94445],88:[0,.68333,.07847,.08334,.82847],89:[0,.68333,.22222,0,.58056],90:[0,.68333,.07153,.08334,.68264],97:[0,.43056,0,0,.52859],98:[0,.69444,0,0,.42917],99:[0,.43056,0,.05556,.43276],100:[0,.69444,0,.16667,.52049],101:[0,.43056,0,.05556,.46563],102:[.19444,.69444,.10764,.16667,.48959],103:[.19444,.43056,.03588,.02778,.47697],104:[0,.69444,0,0,.57616],105:[0,.65952,0,0,.34451],106:[.19444,.65952,.05724,0,.41181],107:[0,.69444,.03148,0,.5206],108:[0,.69444,.01968,.08334,.29838],109:[0,.43056,0,0,.87801],110:[0,.43056,0,0,.60023],111:[0,.43056,0,.05556,.48472],112:[.19444,.43056,0,.08334,.50313],113:[.19444,.43056,.03588,.08334,.44641],114:[0,.43056,.02778,.05556,.45116],115:[0,.43056,0,.05556,.46875],116:[0,.61508,0,.08334,.36111],117:[0,.43056,0,.02778,.57246],118:[0,.43056,.03588,.02778,.48472],119:[0,.43056,.02691,.08334,.71592],120:[0,.43056,0,.02778,.57153],121:[.19444,.43056,.03588,.05556,.49028],122:[0,.43056,.04398,.05556,.46505],160:[0,0,0,0,.25],915:[0,.68333,.13889,.08334,.61528],916:[0,.68333,0,.16667,.83334],920:[0,.68333,.02778,.08334,.76278],923:[0,.68333,0,.16667,.69445],926:[0,.68333,.07569,.08334,.74236],928:[0,.68333,.08125,.05556,.83125],931:[0,.68333,.05764,.08334,.77986],933:[0,.68333,.13889,.05556,.58333],934:[0,.68333,0,.08334,.66667],936:[0,.68333,.11,.05556,.61222],937:[0,.68333,.05017,.08334,.7724],945:[0,.43056,.0037,.02778,.6397],946:[.19444,.69444,.05278,.08334,.56563],947:[.19444,.43056,.05556,0,.51773],948:[0,.69444,.03785,.05556,.44444],949:[0,.43056,0,.08334,.46632],950:[.19444,.69444,.07378,.08334,.4375],951:[.19444,.43056,.03588,.05556,.49653],952:[0,.69444,.02778,.08334,.46944],953:[0,.43056,0,.05556,.35394],954:[0,.43056,0,0,.57616],955:[0,.69444,0,0,.58334],956:[.19444,.43056,0,.02778,.60255],957:[0,.43056,.06366,.02778,.49398],958:[.19444,.69444,.04601,.11111,.4375],959:[0,.43056,0,.05556,.48472],960:[0,.43056,.03588,0,.57003],961:[.19444,.43056,0,.08334,.51702],962:[.09722,.43056,.07986,.08334,.36285],963:[0,.43056,.03588,0,.57141],964:[0,.43056,.1132,.02778,.43715],965:[0,.43056,.03588,.02778,.54028],966:[.19444,.43056,0,.08334,.65417],967:[.19444,.43056,0,.05556,.62569],968:[.19444,.69444,.03588,.11111,.65139],969:[0,.43056,.03588,0,.62245],977:[0,.69444,0,.08334,.59144],981:[.19444,.69444,0,.08334,.59583],982:[0,.43056,.02778,0,.82813],1009:[.19444,.43056,0,.08334,.51702],1013:[0,.43056,0,.05556,.4059],57649:[0,.43056,0,.02778,.32246],57911:[.19444,.43056,0,.08334,.38403]},"SansSerif-Bold":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.36667],34:[0,.69444,0,0,.55834],35:[.19444,.69444,0,0,.91667],36:[.05556,.75,0,0,.55],37:[.05556,.75,0,0,1.02912],38:[0,.69444,0,0,.83056],39:[0,.69444,0,0,.30556],40:[.25,.75,0,0,.42778],41:[.25,.75,0,0,.42778],42:[0,.75,0,0,.55],43:[.11667,.61667,0,0,.85556],44:[.10556,.13056,0,0,.30556],45:[0,.45833,0,0,.36667],46:[0,.13056,0,0,.30556],47:[.25,.75,0,0,.55],48:[0,.69444,0,0,.55],49:[0,.69444,0,0,.55],50:[0,.69444,0,0,.55],51:[0,.69444,0,0,.55],52:[0,.69444,0,0,.55],53:[0,.69444,0,0,.55],54:[0,.69444,0,0,.55],55:[0,.69444,0,0,.55],56:[0,.69444,0,0,.55],57:[0,.69444,0,0,.55],58:[0,.45833,0,0,.30556],59:[.10556,.45833,0,0,.30556],61:[-.09375,.40625,0,0,.85556],63:[0,.69444,0,0,.51945],64:[0,.69444,0,0,.73334],65:[0,.69444,0,0,.73334],66:[0,.69444,0,0,.73334],67:[0,.69444,0,0,.70278],68:[0,.69444,0,0,.79445],69:[0,.69444,0,0,.64167],70:[0,.69444,0,0,.61111],71:[0,.69444,0,0,.73334],72:[0,.69444,0,0,.79445],73:[0,.69444,0,0,.33056],74:[0,.69444,0,0,.51945],75:[0,.69444,0,0,.76389],76:[0,.69444,0,0,.58056],77:[0,.69444,0,0,.97778],78:[0,.69444,0,0,.79445],79:[0,.69444,0,0,.79445],80:[0,.69444,0,0,.70278],81:[.10556,.69444,0,0,.79445],82:[0,.69444,0,0,.70278],83:[0,.69444,0,0,.61111],84:[0,.69444,0,0,.73334],85:[0,.69444,0,0,.76389],86:[0,.69444,.01528,0,.73334],87:[0,.69444,.01528,0,1.03889],88:[0,.69444,0,0,.73334],89:[0,.69444,.0275,0,.73334],90:[0,.69444,0,0,.67223],91:[.25,.75,0,0,.34306],93:[.25,.75,0,0,.34306],94:[0,.69444,0,0,.55],95:[.35,.10833,.03056,0,.55],97:[0,.45833,0,0,.525],98:[0,.69444,0,0,.56111],99:[0,.45833,0,0,.48889],100:[0,.69444,0,0,.56111],101:[0,.45833,0,0,.51111],102:[0,.69444,.07639,0,.33611],103:[.19444,.45833,.01528,0,.55],104:[0,.69444,0,0,.56111],105:[0,.69444,0,0,.25556],106:[.19444,.69444,0,0,.28611],107:[0,.69444,0,0,.53056],108:[0,.69444,0,0,.25556],109:[0,.45833,0,0,.86667],110:[0,.45833,0,0,.56111],111:[0,.45833,0,0,.55],112:[.19444,.45833,0,0,.56111],113:[.19444,.45833,0,0,.56111],114:[0,.45833,.01528,0,.37222],115:[0,.45833,0,0,.42167],116:[0,.58929,0,0,.40417],117:[0,.45833,0,0,.56111],118:[0,.45833,.01528,0,.5],119:[0,.45833,.01528,0,.74445],120:[0,.45833,0,0,.5],121:[.19444,.45833,.01528,0,.5],122:[0,.45833,0,0,.47639],126:[.35,.34444,0,0,.55],160:[0,0,0,0,.25],168:[0,.69444,0,0,.55],176:[0,.69444,0,0,.73334],180:[0,.69444,0,0,.55],184:[.17014,0,0,0,.48889],305:[0,.45833,0,0,.25556],567:[.19444,.45833,0,0,.28611],710:[0,.69444,0,0,.55],711:[0,.63542,0,0,.55],713:[0,.63778,0,0,.55],728:[0,.69444,0,0,.55],729:[0,.69444,0,0,.30556],730:[0,.69444,0,0,.73334],732:[0,.69444,0,0,.55],733:[0,.69444,0,0,.55],915:[0,.69444,0,0,.58056],916:[0,.69444,0,0,.91667],920:[0,.69444,0,0,.85556],923:[0,.69444,0,0,.67223],926:[0,.69444,0,0,.73334],928:[0,.69444,0,0,.79445],931:[0,.69444,0,0,.79445],933:[0,.69444,0,0,.85556],934:[0,.69444,0,0,.79445],936:[0,.69444,0,0,.85556],937:[0,.69444,0,0,.79445],8211:[0,.45833,.03056,0,.55],8212:[0,.45833,.03056,0,1.10001],8216:[0,.69444,0,0,.30556],8217:[0,.69444,0,0,.30556],8220:[0,.69444,0,0,.55834],8221:[0,.69444,0,0,.55834]},"SansSerif-Italic":{32:[0,0,0,0,.25],33:[0,.69444,.05733,0,.31945],34:[0,.69444,.00316,0,.5],35:[.19444,.69444,.05087,0,.83334],36:[.05556,.75,.11156,0,.5],37:[.05556,.75,.03126,0,.83334],38:[0,.69444,.03058,0,.75834],39:[0,.69444,.07816,0,.27778],40:[.25,.75,.13164,0,.38889],41:[.25,.75,.02536,0,.38889],42:[0,.75,.11775,0,.5],43:[.08333,.58333,.02536,0,.77778],44:[.125,.08333,0,0,.27778],45:[0,.44444,.01946,0,.33333],46:[0,.08333,0,0,.27778],47:[.25,.75,.13164,0,.5],48:[0,.65556,.11156,0,.5],49:[0,.65556,.11156,0,.5],50:[0,.65556,.11156,0,.5],51:[0,.65556,.11156,0,.5],52:[0,.65556,.11156,0,.5],53:[0,.65556,.11156,0,.5],54:[0,.65556,.11156,0,.5],55:[0,.65556,.11156,0,.5],56:[0,.65556,.11156,0,.5],57:[0,.65556,.11156,0,.5],58:[0,.44444,.02502,0,.27778],59:[.125,.44444,.02502,0,.27778],61:[-.13,.37,.05087,0,.77778],63:[0,.69444,.11809,0,.47222],64:[0,.69444,.07555,0,.66667],65:[0,.69444,0,0,.66667],66:[0,.69444,.08293,0,.66667],67:[0,.69444,.11983,0,.63889],68:[0,.69444,.07555,0,.72223],69:[0,.69444,.11983,0,.59722],70:[0,.69444,.13372,0,.56945],71:[0,.69444,.11983,0,.66667],72:[0,.69444,.08094,0,.70834],73:[0,.69444,.13372,0,.27778],74:[0,.69444,.08094,0,.47222],75:[0,.69444,.11983,0,.69445],76:[0,.69444,0,0,.54167],77:[0,.69444,.08094,0,.875],78:[0,.69444,.08094,0,.70834],79:[0,.69444,.07555,0,.73611],80:[0,.69444,.08293,0,.63889],81:[.125,.69444,.07555,0,.73611],82:[0,.69444,.08293,0,.64584],83:[0,.69444,.09205,0,.55556],84:[0,.69444,.13372,0,.68056],85:[0,.69444,.08094,0,.6875],86:[0,.69444,.1615,0,.66667],87:[0,.69444,.1615,0,.94445],88:[0,.69444,.13372,0,.66667],89:[0,.69444,.17261,0,.66667],90:[0,.69444,.11983,0,.61111],91:[.25,.75,.15942,0,.28889],93:[.25,.75,.08719,0,.28889],94:[0,.69444,.0799,0,.5],95:[.35,.09444,.08616,0,.5],97:[0,.44444,.00981,0,.48056],98:[0,.69444,.03057,0,.51667],99:[0,.44444,.08336,0,.44445],100:[0,.69444,.09483,0,.51667],101:[0,.44444,.06778,0,.44445],102:[0,.69444,.21705,0,.30556],103:[.19444,.44444,.10836,0,.5],104:[0,.69444,.01778,0,.51667],105:[0,.67937,.09718,0,.23889],106:[.19444,.67937,.09162,0,.26667],107:[0,.69444,.08336,0,.48889],108:[0,.69444,.09483,0,.23889],109:[0,.44444,.01778,0,.79445],110:[0,.44444,.01778,0,.51667],111:[0,.44444,.06613,0,.5],112:[.19444,.44444,.0389,0,.51667],113:[.19444,.44444,.04169,0,.51667],114:[0,.44444,.10836,0,.34167],115:[0,.44444,.0778,0,.38333],116:[0,.57143,.07225,0,.36111],117:[0,.44444,.04169,0,.51667],118:[0,.44444,.10836,0,.46111],119:[0,.44444,.10836,0,.68334],120:[0,.44444,.09169,0,.46111],121:[.19444,.44444,.10836,0,.46111],122:[0,.44444,.08752,0,.43472],126:[.35,.32659,.08826,0,.5],160:[0,0,0,0,.25],168:[0,.67937,.06385,0,.5],176:[0,.69444,0,0,.73752],184:[.17014,0,0,0,.44445],305:[0,.44444,.04169,0,.23889],567:[.19444,.44444,.04169,0,.26667],710:[0,.69444,.0799,0,.5],711:[0,.63194,.08432,0,.5],713:[0,.60889,.08776,0,.5],714:[0,.69444,.09205,0,.5],715:[0,.69444,0,0,.5],728:[0,.69444,.09483,0,.5],729:[0,.67937,.07774,0,.27778],730:[0,.69444,0,0,.73752],732:[0,.67659,.08826,0,.5],733:[0,.69444,.09205,0,.5],915:[0,.69444,.13372,0,.54167],916:[0,.69444,0,0,.83334],920:[0,.69444,.07555,0,.77778],923:[0,.69444,0,0,.61111],926:[0,.69444,.12816,0,.66667],928:[0,.69444,.08094,0,.70834],931:[0,.69444,.11983,0,.72222],933:[0,.69444,.09031,0,.77778],934:[0,.69444,.04603,0,.72222],936:[0,.69444,.09031,0,.77778],937:[0,.69444,.08293,0,.72222],8211:[0,.44444,.08616,0,.5],8212:[0,.44444,.08616,0,1],8216:[0,.69444,.07816,0,.27778],8217:[0,.69444,.07816,0,.27778],8220:[0,.69444,.14205,0,.5],8221:[0,.69444,.00316,0,.5]},"SansSerif-Regular":{32:[0,0,0,0,.25],33:[0,.69444,0,0,.31945],34:[0,.69444,0,0,.5],35:[.19444,.69444,0,0,.83334],36:[.05556,.75,0,0,.5],37:[.05556,.75,0,0,.83334],38:[0,.69444,0,0,.75834],39:[0,.69444,0,0,.27778],40:[.25,.75,0,0,.38889],41:[.25,.75,0,0,.38889],42:[0,.75,0,0,.5],43:[.08333,.58333,0,0,.77778],44:[.125,.08333,0,0,.27778],45:[0,.44444,0,0,.33333],46:[0,.08333,0,0,.27778],47:[.25,.75,0,0,.5],48:[0,.65556,0,0,.5],49:[0,.65556,0,0,.5],50:[0,.65556,0,0,.5],51:[0,.65556,0,0,.5],52:[0,.65556,0,0,.5],53:[0,.65556,0,0,.5],54:[0,.65556,0,0,.5],55:[0,.65556,0,0,.5],56:[0,.65556,0,0,.5],57:[0,.65556,0,0,.5],58:[0,.44444,0,0,.27778],59:[.125,.44444,0,0,.27778],61:[-.13,.37,0,0,.77778],63:[0,.69444,0,0,.47222],64:[0,.69444,0,0,.66667],65:[0,.69444,0,0,.66667],66:[0,.69444,0,0,.66667],67:[0,.69444,0,0,.63889],68:[0,.69444,0,0,.72223],69:[0,.69444,0,0,.59722],70:[0,.69444,0,0,.56945],71:[0,.69444,0,0,.66667],72:[0,.69444,0,0,.70834],73:[0,.69444,0,0,.27778],74:[0,.69444,0,0,.47222],75:[0,.69444,0,0,.69445],76:[0,.69444,0,0,.54167],77:[0,.69444,0,0,.875],78:[0,.69444,0,0,.70834],79:[0,.69444,0,0,.73611],80:[0,.69444,0,0,.63889],81:[.125,.69444,0,0,.73611],82:[0,.69444,0,0,.64584],83:[0,.69444,0,0,.55556],84:[0,.69444,0,0,.68056],85:[0,.69444,0,0,.6875],86:[0,.69444,.01389,0,.66667],87:[0,.69444,.01389,0,.94445],88:[0,.69444,0,0,.66667],89:[0,.69444,.025,0,.66667],90:[0,.69444,0,0,.61111],91:[.25,.75,0,0,.28889],93:[.25,.75,0,0,.28889],94:[0,.69444,0,0,.5],95:[.35,.09444,.02778,0,.5],97:[0,.44444,0,0,.48056],98:[0,.69444,0,0,.51667],99:[0,.44444,0,0,.44445],100:[0,.69444,0,0,.51667],101:[0,.44444,0,0,.44445],102:[0,.69444,.06944,0,.30556],103:[.19444,.44444,.01389,0,.5],104:[0,.69444,0,0,.51667],105:[0,.67937,0,0,.23889],106:[.19444,.67937,0,0,.26667],107:[0,.69444,0,0,.48889],108:[0,.69444,0,0,.23889],109:[0,.44444,0,0,.79445],110:[0,.44444,0,0,.51667],111:[0,.44444,0,0,.5],112:[.19444,.44444,0,0,.51667],113:[.19444,.44444,0,0,.51667],114:[0,.44444,.01389,0,.34167],115:[0,.44444,0,0,.38333],116:[0,.57143,0,0,.36111],117:[0,.44444,0,0,.51667],118:[0,.44444,.01389,0,.46111],119:[0,.44444,.01389,0,.68334],120:[0,.44444,0,0,.46111],121:[.19444,.44444,.01389,0,.46111],122:[0,.44444,0,0,.43472],126:[.35,.32659,0,0,.5],160:[0,0,0,0,.25],168:[0,.67937,0,0,.5],176:[0,.69444,0,0,.66667],184:[.17014,0,0,0,.44445],305:[0,.44444,0,0,.23889],567:[.19444,.44444,0,0,.26667],710:[0,.69444,0,0,.5],711:[0,.63194,0,0,.5],713:[0,.60889,0,0,.5],714:[0,.69444,0,0,.5],715:[0,.69444,0,0,.5],728:[0,.69444,0,0,.5],729:[0,.67937,0,0,.27778],730:[0,.69444,0,0,.66667],732:[0,.67659,0,0,.5],733:[0,.69444,0,0,.5],915:[0,.69444,0,0,.54167],916:[0,.69444,0,0,.83334],920:[0,.69444,0,0,.77778],923:[0,.69444,0,0,.61111],926:[0,.69444,0,0,.66667],928:[0,.69444,0,0,.70834],931:[0,.69444,0,0,.72222],933:[0,.69444,0,0,.77778],934:[0,.69444,0,0,.72222],936:[0,.69444,0,0,.77778],937:[0,.69444,0,0,.72222],8211:[0,.44444,.02778,0,.5],8212:[0,.44444,.02778,0,1],8216:[0,.69444,0,0,.27778],8217:[0,.69444,0,0,.27778],8220:[0,.69444,0,0,.5],8221:[0,.69444,0,0,.5]},"Script-Regular":{32:[0,0,0,0,.25],65:[0,.7,.22925,0,.80253],66:[0,.7,.04087,0,.90757],67:[0,.7,.1689,0,.66619],68:[0,.7,.09371,0,.77443],69:[0,.7,.18583,0,.56162],70:[0,.7,.13634,0,.89544],71:[0,.7,.17322,0,.60961],72:[0,.7,.29694,0,.96919],73:[0,.7,.19189,0,.80907],74:[.27778,.7,.19189,0,1.05159],75:[0,.7,.31259,0,.91364],76:[0,.7,.19189,0,.87373],77:[0,.7,.15981,0,1.08031],78:[0,.7,.3525,0,.9015],79:[0,.7,.08078,0,.73787],80:[0,.7,.08078,0,1.01262],81:[0,.7,.03305,0,.88282],82:[0,.7,.06259,0,.85],83:[0,.7,.19189,0,.86767],84:[0,.7,.29087,0,.74697],85:[0,.7,.25815,0,.79996],86:[0,.7,.27523,0,.62204],87:[0,.7,.27523,0,.80532],88:[0,.7,.26006,0,.94445],89:[0,.7,.2939,0,.70961],90:[0,.7,.24037,0,.8212],160:[0,0,0,0,.25]},"Size1-Regular":{32:[0,0,0,0,.25],40:[.35001,.85,0,0,.45834],41:[.35001,.85,0,0,.45834],47:[.35001,.85,0,0,.57778],91:[.35001,.85,0,0,.41667],92:[.35001,.85,0,0,.57778],93:[.35001,.85,0,0,.41667],123:[.35001,.85,0,0,.58334],125:[.35001,.85,0,0,.58334],160:[0,0,0,0,.25],710:[0,.72222,0,0,.55556],732:[0,.72222,0,0,.55556],770:[0,.72222,0,0,.55556],771:[0,.72222,0,0,.55556],8214:[-99e-5,.601,0,0,.77778],8593:[1e-5,.6,0,0,.66667],8595:[1e-5,.6,0,0,.66667],8657:[1e-5,.6,0,0,.77778],8659:[1e-5,.6,0,0,.77778],8719:[.25001,.75,0,0,.94445],8720:[.25001,.75,0,0,.94445],8721:[.25001,.75,0,0,1.05556],8730:[.35001,.85,0,0,1],8739:[-.00599,.606,0,0,.33333],8741:[-.00599,.606,0,0,.55556],8747:[.30612,.805,.19445,0,.47222],8748:[.306,.805,.19445,0,.47222],8749:[.306,.805,.19445,0,.47222],8750:[.30612,.805,.19445,0,.47222],8896:[.25001,.75,0,0,.83334],8897:[.25001,.75,0,0,.83334],8898:[.25001,.75,0,0,.83334],8899:[.25001,.75,0,0,.83334],8968:[.35001,.85,0,0,.47222],8969:[.35001,.85,0,0,.47222],8970:[.35001,.85,0,0,.47222],8971:[.35001,.85,0,0,.47222],9168:[-99e-5,.601,0,0,.66667],10216:[.35001,.85,0,0,.47222],10217:[.35001,.85,0,0,.47222],10752:[.25001,.75,0,0,1.11111],10753:[.25001,.75,0,0,1.11111],10754:[.25001,.75,0,0,1.11111],10756:[.25001,.75,0,0,.83334],10758:[.25001,.75,0,0,.83334]},"Size2-Regular":{32:[0,0,0,0,.25],40:[.65002,1.15,0,0,.59722],41:[.65002,1.15,0,0,.59722],47:[.65002,1.15,0,0,.81111],91:[.65002,1.15,0,0,.47222],92:[.65002,1.15,0,0,.81111],93:[.65002,1.15,0,0,.47222],123:[.65002,1.15,0,0,.66667],125:[.65002,1.15,0,0,.66667],160:[0,0,0,0,.25],710:[0,.75,0,0,1],732:[0,.75,0,0,1],770:[0,.75,0,0,1],771:[0,.75,0,0,1],8719:[.55001,1.05,0,0,1.27778],8720:[.55001,1.05,0,0,1.27778],8721:[.55001,1.05,0,0,1.44445],8730:[.65002,1.15,0,0,1],8747:[.86225,1.36,.44445,0,.55556],8748:[.862,1.36,.44445,0,.55556],8749:[.862,1.36,.44445,0,.55556],8750:[.86225,1.36,.44445,0,.55556],8896:[.55001,1.05,0,0,1.11111],8897:[.55001,1.05,0,0,1.11111],8898:[.55001,1.05,0,0,1.11111],8899:[.55001,1.05,0,0,1.11111],8968:[.65002,1.15,0,0,.52778],8969:[.65002,1.15,0,0,.52778],8970:[.65002,1.15,0,0,.52778],8971:[.65002,1.15,0,0,.52778],10216:[.65002,1.15,0,0,.61111],10217:[.65002,1.15,0,0,.61111],10752:[.55001,1.05,0,0,1.51112],10753:[.55001,1.05,0,0,1.51112],10754:[.55001,1.05,0,0,1.51112],10756:[.55001,1.05,0,0,1.11111],10758:[.55001,1.05,0,0,1.11111]},"Size3-Regular":{32:[0,0,0,0,.25],40:[.95003,1.45,0,0,.73611],41:[.95003,1.45,0,0,.73611],47:[.95003,1.45,0,0,1.04445],91:[.95003,1.45,0,0,.52778],92:[.95003,1.45,0,0,1.04445],93:[.95003,1.45,0,0,.52778],123:[.95003,1.45,0,0,.75],125:[.95003,1.45,0,0,.75],160:[0,0,0,0,.25],710:[0,.75,0,0,1.44445],732:[0,.75,0,0,1.44445],770:[0,.75,0,0,1.44445],771:[0,.75,0,0,1.44445],8730:[.95003,1.45,0,0,1],8968:[.95003,1.45,0,0,.58334],8969:[.95003,1.45,0,0,.58334],8970:[.95003,1.45,0,0,.58334],8971:[.95003,1.45,0,0,.58334],10216:[.95003,1.45,0,0,.75],10217:[.95003,1.45,0,0,.75]},"Size4-Regular":{32:[0,0,0,0,.25],40:[1.25003,1.75,0,0,.79167],41:[1.25003,1.75,0,0,.79167],47:[1.25003,1.75,0,0,1.27778],91:[1.25003,1.75,0,0,.58334],92:[1.25003,1.75,0,0,1.27778],93:[1.25003,1.75,0,0,.58334],123:[1.25003,1.75,0,0,.80556],125:[1.25003,1.75,0,0,.80556],160:[0,0,0,0,.25],710:[0,.825,0,0,1.8889],732:[0,.825,0,0,1.8889],770:[0,.825,0,0,1.8889],771:[0,.825,0,0,1.8889],8730:[1.25003,1.75,0,0,1],8968:[1.25003,1.75,0,0,.63889],8969:[1.25003,1.75,0,0,.63889],8970:[1.25003,1.75,0,0,.63889],8971:[1.25003,1.75,0,0,.63889],9115:[.64502,1.155,0,0,.875],9116:[1e-5,.6,0,0,.875],9117:[.64502,1.155,0,0,.875],9118:[.64502,1.155,0,0,.875],9119:[1e-5,.6,0,0,.875],9120:[.64502,1.155,0,0,.875],9121:[.64502,1.155,0,0,.66667],9122:[-99e-5,.601,0,0,.66667],9123:[.64502,1.155,0,0,.66667],9124:[.64502,1.155,0,0,.66667],9125:[-99e-5,.601,0,0,.66667],9126:[.64502,1.155,0,0,.66667],9127:[1e-5,.9,0,0,.88889],9128:[.65002,1.15,0,0,.88889],9129:[.90001,0,0,0,.88889],9130:[0,.3,0,0,.88889],9131:[1e-5,.9,0,0,.88889],9132:[.65002,1.15,0,0,.88889],9133:[.90001,0,0,0,.88889],9143:[.88502,.915,0,0,1.05556],10216:[1.25003,1.75,0,0,.80556],10217:[1.25003,1.75,0,0,.80556],57344:[-.00499,.605,0,0,1.05556],57345:[-.00499,.605,0,0,1.05556],57680:[0,.12,0,0,.45],57681:[0,.12,0,0,.45],57682:[0,.12,0,0,.45],57683:[0,.12,0,0,.45]},"Typewriter-Regular":{32:[0,0,0,0,.525],33:[0,.61111,0,0,.525],34:[0,.61111,0,0,.525],35:[0,.61111,0,0,.525],36:[.08333,.69444,0,0,.525],37:[.08333,.69444,0,0,.525],38:[0,.61111,0,0,.525],39:[0,.61111,0,0,.525],40:[.08333,.69444,0,0,.525],41:[.08333,.69444,0,0,.525],42:[0,.52083,0,0,.525],43:[-.08056,.53055,0,0,.525],44:[.13889,.125,0,0,.525],45:[-.08056,.53055,0,0,.525],46:[0,.125,0,0,.525],47:[.08333,.69444,0,0,.525],48:[0,.61111,0,0,.525],49:[0,.61111,0,0,.525],50:[0,.61111,0,0,.525],51:[0,.61111,0,0,.525],52:[0,.61111,0,0,.525],53:[0,.61111,0,0,.525],54:[0,.61111,0,0,.525],55:[0,.61111,0,0,.525],56:[0,.61111,0,0,.525],57:[0,.61111,0,0,.525],58:[0,.43056,0,0,.525],59:[.13889,.43056,0,0,.525],60:[-.05556,.55556,0,0,.525],61:[-.19549,.41562,0,0,.525],62:[-.05556,.55556,0,0,.525],63:[0,.61111,0,0,.525],64:[0,.61111,0,0,.525],65:[0,.61111,0,0,.525],66:[0,.61111,0,0,.525],67:[0,.61111,0,0,.525],68:[0,.61111,0,0,.525],69:[0,.61111,0,0,.525],70:[0,.61111,0,0,.525],71:[0,.61111,0,0,.525],72:[0,.61111,0,0,.525],73:[0,.61111,0,0,.525],74:[0,.61111,0,0,.525],75:[0,.61111,0,0,.525],76:[0,.61111,0,0,.525],77:[0,.61111,0,0,.525],78:[0,.61111,0,0,.525],79:[0,.61111,0,0,.525],80:[0,.61111,0,0,.525],81:[.13889,.61111,0,0,.525],82:[0,.61111,0,0,.525],83:[0,.61111,0,0,.525],84:[0,.61111,0,0,.525],85:[0,.61111,0,0,.525],86:[0,.61111,0,0,.525],87:[0,.61111,0,0,.525],88:[0,.61111,0,0,.525],89:[0,.61111,0,0,.525],90:[0,.61111,0,0,.525],91:[.08333,.69444,0,0,.525],92:[.08333,.69444,0,0,.525],93:[.08333,.69444,0,0,.525],94:[0,.61111,0,0,.525],95:[.09514,0,0,0,.525],96:[0,.61111,0,0,.525],97:[0,.43056,0,0,.525],98:[0,.61111,0,0,.525],99:[0,.43056,0,0,.525],100:[0,.61111,0,0,.525],101:[0,.43056,0,0,.525],102:[0,.61111,0,0,.525],103:[.22222,.43056,0,0,.525],104:[0,.61111,0,0,.525],105:[0,.61111,0,0,.525],106:[.22222,.61111,0,0,.525],107:[0,.61111,0,0,.525],108:[0,.61111,0,0,.525],109:[0,.43056,0,0,.525],110:[0,.43056,0,0,.525],111:[0,.43056,0,0,.525],112:[.22222,.43056,0,0,.525],113:[.22222,.43056,0,0,.525],114:[0,.43056,0,0,.525],115:[0,.43056,0,0,.525],116:[0,.55358,0,0,.525],117:[0,.43056,0,0,.525],118:[0,.43056,0,0,.525],119:[0,.43056,0,0,.525],120:[0,.43056,0,0,.525],121:[.22222,.43056,0,0,.525],122:[0,.43056,0,0,.525],123:[.08333,.69444,0,0,.525],124:[.08333,.69444,0,0,.525],125:[.08333,.69444,0,0,.525],126:[0,.61111,0,0,.525],127:[0,.61111,0,0,.525],160:[0,0,0,0,.525],176:[0,.61111,0,0,.525],184:[.19445,0,0,0,.525],305:[0,.43056,0,0,.525],567:[.22222,.43056,0,0,.525],711:[0,.56597,0,0,.525],713:[0,.56555,0,0,.525],714:[0,.61111,0,0,.525],715:[0,.61111,0,0,.525],728:[0,.61111,0,0,.525],730:[0,.61111,0,0,.525],770:[0,.61111,0,0,.525],771:[0,.61111,0,0,.525],776:[0,.61111,0,0,.525],915:[0,.61111,0,0,.525],916:[0,.61111,0,0,.525],920:[0,.61111,0,0,.525],923:[0,.61111,0,0,.525],926:[0,.61111,0,0,.525],928:[0,.61111,0,0,.525],931:[0,.61111,0,0,.525],933:[0,.61111,0,0,.525],934:[0,.61111,0,0,.525],936:[0,.61111,0,0,.525],937:[0,.61111,0,0,.525],8216:[0,.61111,0,0,.525],8217:[0,.61111,0,0,.525],8242:[0,.61111,0,0,.525],9251:[.11111,.21944,0,0,.525]}},db={slant:[.25,.25,.25],space:[0,0,0],stretch:[0,0,0],shrink:[0,0,0],xHeight:[.431,.431,.431],quad:[1,1.171,1.472],extraSpace:[0,0,0],num1:[.677,.732,.925],num2:[.394,.384,.387],num3:[.444,.471,.504],denom1:[.686,.752,1.025],denom2:[.345,.344,.532],sup1:[.413,.503,.504],sup2:[.363,.431,.404],sup3:[.289,.286,.294],sub1:[.15,.143,.2],sub2:[.247,.286,.4],supDrop:[.386,.353,.494],subDrop:[.05,.071,.1],delim1:[2.39,1.7,1.98],delim2:[1.01,1.157,1.42],axisHeight:[.25,.25,.25],defaultRuleThickness:[.04,.049,.049],bigOpSpacing1:[.111,.111,.111],bigOpSpacing2:[.166,.166,.166],bigOpSpacing3:[.2,.2,.2],bigOpSpacing4:[.6,.611,.611],bigOpSpacing5:[.1,.143,.143],sqrtRuleThickness:[.04,.04,.04],ptPerEm:[10,10,10],doubleRuleSep:[.2,.2,.2],arrayRuleWidth:[.04,.04,.04],fboxsep:[.3,.3,.3],fboxrule:[.04,.04,.04]},y9={Å:"A",Ð:"D",Þ:"o",å:"a",ð:"d",þ:"o",А:"A",Б:"B",В:"B",Г:"F",Д:"A",Е:"E",Ж:"K",З:"3",И:"N",Й:"N",К:"K",Л:"N",М:"M",Н:"H",О:"O",П:"N",Р:"P",С:"C",Т:"T",У:"y",Ф:"O",Х:"X",Ц:"U",Ч:"h",Ш:"W",Щ:"W",Ъ:"B",Ы:"X",Ь:"B",Э:"3",Ю:"X",Я:"R",а:"a",б:"b",в:"a",г:"r",д:"y",е:"e",ж:"m",з:"e",и:"n",й:"n",к:"n",л:"n",м:"m",н:"n",о:"o",п:"n",р:"p",с:"c",т:"o",у:"y",ф:"b",х:"x",ц:"n",ч:"n",ш:"w",щ:"w",ъ:"a",ы:"m",ь:"a",э:"e",ю:"m",я:"r"};function pK(e,t){as[e]=t}function oD(e,t,r){if(!as[t])throw new Error("Font metrics not found for font: "+t+".");var n=e.charCodeAt(0),a=as[t][n];if(!a&&e[0]in y9&&(n=y9[e[0]].charCodeAt(0),a=as[t][n]),!a&&r==="text"&&hK(n)&&(a=as[t][77]),a)return{depth:a[0],height:a[1],italic:a[2],skew:a[3],width:a[4]}}var Ex={};function Ppe(e){var t;if(e>=5?t=0:e>=3?t=1:t=2,!Ex[t]){var r=Ex[t]={cssEmPerMu:db.quad[t]/18};for(var n in db)db.hasOwnProperty(n)&&(r[n]=db[n][t])}return Ex[t]}var zpe=[[1,1,1],[2,1,1],[3,1,1],[4,2,1],[5,2,1],[6,3,1],[7,4,2],[8,6,3],[9,7,6],[10,8,7],[11,10,9]],v9=[.5,.6,.7,.8,.9,1,1.2,1.44,1.728,2.074,2.488],S9=function(t,r){return r.size<2?t:zpe[t-1][r.size-1]};class nl{constructor(t){this.style=void 0,this.color=void 0,this.size=void 0,this.textSize=void 0,this.phantom=void 0,this.font=void 0,this.fontFamily=void 0,this.fontWeight=void 0,this.fontShape=void 0,this.sizeMultiplier=void 0,this.maxSize=void 0,this.minRuleThickness=void 0,this._fontMetrics=void 0,this.style=t.style,this.color=t.color,this.size=t.size||nl.BASESIZE,this.textSize=t.textSize||this.size,this.phantom=!!t.phantom,this.font=t.font||"",this.fontFamily=t.fontFamily||"",this.fontWeight=t.fontWeight||"",this.fontShape=t.fontShape||"",this.sizeMultiplier=v9[this.size-1],this.maxSize=t.maxSize,this.minRuleThickness=t.minRuleThickness,this._fontMetrics=void 0}extend(t){var r={style:this.style,size:this.size,textSize:this.textSize,color:this.color,phantom:this.phantom,font:this.font,fontFamily:this.fontFamily,fontWeight:this.fontWeight,fontShape:this.fontShape,maxSize:this.maxSize,minRuleThickness:this.minRuleThickness};for(var n in t)t.hasOwnProperty(n)&&(r[n]=t[n]);return new nl(r)}havingStyle(t){return this.style===t?this:this.extend({style:t,size:S9(this.textSize,t)})}havingCrampedStyle(){return this.havingStyle(this.style.cramp())}havingSize(t){return this.size===t&&this.textSize===t?this:this.extend({style:this.style.text(),size:t,textSize:t,sizeMultiplier:v9[t-1]})}havingBaseStyle(t){t=t||this.style.text();var r=S9(nl.BASESIZE,t);return this.size===r&&this.textSize===nl.BASESIZE&&this.style===t?this:this.extend({style:t,size:r})}havingBaseSizing(){var t;switch(this.style.id){case 4:case 5:t=3;break;case 6:case 7:t=1;break;default:t=6}return this.extend({style:this.style.text(),size:t})}withColor(t){return this.extend({color:t})}withPhantom(){return this.extend({phantom:!0})}withFont(t){return this.extend({font:t})}withTextFontFamily(t){return this.extend({fontFamily:t,font:""})}withTextFontWeight(t){return this.extend({fontWeight:t,font:""})}withTextFontShape(t){return this.extend({fontShape:t,font:""})}sizingClasses(t){return t.size!==this.size?["sizing","reset-size"+t.size,"size"+this.size]:[]}baseSizingClasses(){return this.size!==nl.BASESIZE?["sizing","reset-size"+this.size,"size"+nl.BASESIZE]:[]}fontMetrics(){return this._fontMetrics||(this._fontMetrics=Ppe(this.size)),this._fontMetrics}getColor(){return this.phantom?"transparent":this.color}}nl.BASESIZE=6;var x_={pt:1,mm:7227/2540,cm:7227/254,in:72.27,bp:803/800,pc:12,dd:1238/1157,cc:14856/1157,nd:685/642,nc:1370/107,sp:1/65536,px:803/800},Hpe={ex:!0,em:!0,mu:!0},fK=function(t){return typeof t!="string"&&(t=t.unit),t in x_||t in Hpe||t==="ex"},xn=function(t,r){var n;if(t.unit in x_)n=x_[t.unit]/r.fontMetrics().ptPerEm/r.sizeMultiplier;else if(t.unit==="mu")n=r.fontMetrics().cssEmPerMu;else{var a;if(r.style.isTight()?a=r.havingStyle(r.style.text()):a=r,t.unit==="ex")n=a.fontMetrics().xHeight;else if(t.unit==="em")n=a.fontMetrics().quad;else throw new nt("Invalid unit: '"+t.unit+"'");a!==r&&(n*=a.sizeMultiplier/r.sizeMultiplier)}return Math.min(t.number*n,r.maxSize)},ct=function(t){return+t.toFixed(4)+"em"},Cu=function(t){return t.filter(r=>r).join(" ")},mK=function(t,r,n){if(this.classes=t||[],this.attributes={},this.height=0,this.depth=0,this.maxFontSize=0,this.style=n||{},r){r.style.isTight()&&this.classes.push("mtight");var a=r.getColor();a&&(this.style.color=a)}},gK=function(t){var r=document.createElement(t);r.className=Cu(this.classes);for(var n in this.style)this.style.hasOwnProperty(n)&&(r.style[n]=this.style[n]);for(var a in this.attributes)this.attributes.hasOwnProperty(a)&&r.setAttribute(a,this.attributes[a]);for(var i=0;i<this.children.length;i++)r.appendChild(this.children[i].toNode());return r},Upe=/[\s"'>/=\x00-\x1f]/,bK=function(t){var r="<"+t;this.classes.length&&(r+=' class="'+Gt.escape(Cu(this.classes))+'"');var n="";for(var a in this.style)this.style.hasOwnProperty(a)&&(n+=Gt.hyphenate(a)+":"+this.style[a]+";");n&&(r+=' style="'+Gt.escape(n)+'"');for(var i in this.attributes)if(this.attributes.hasOwnProperty(i)){if(Upe.test(i))throw new nt("Invalid attribute name '"+i+"'");r+=" "+i+'="'+Gt.escape(this.attributes[i])+'"'}r+=">";for(var o=0;o<this.children.length;o++)r+=this.children[o].toMarkup();return r+="</"+t+">",r};class cg{constructor(t,r,n,a){this.children=void 0,this.attributes=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.width=void 0,this.maxFontSize=void 0,this.style=void 0,mK.call(this,t,n,a),this.children=r||[]}setAttribute(t,r){this.attributes[t]=r}hasClass(t){return Gt.contains(this.classes,t)}toNode(){return gK.call(this,"span")}toMarkup(){return bK.call(this,"span")}}let sD=class{constructor(t,r,n,a){this.children=void 0,this.attributes=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.maxFontSize=void 0,this.style=void 0,mK.call(this,r,a),this.children=n||[],this.setAttribute("href",t)}setAttribute(t,r){this.attributes[t]=r}hasClass(t){return Gt.contains(this.classes,t)}toNode(){return gK.call(this,"a")}toMarkup(){return bK.call(this,"a")}};class $pe{constructor(t,r,n){this.src=void 0,this.alt=void 0,this.classes=void 0,this.height=void 0,this.depth=void 0,this.maxFontSize=void 0,this.style=void 0,this.alt=r,this.src=t,this.classes=["mord"],this.style=n}hasClass(t){return Gt.contains(this.classes,t)}toNode(){var t=document.createElement("img");t.src=this.src,t.alt=this.alt,t.className="mord";for(var r in this.style)this.style.hasOwnProperty(r)&&(t.style[r]=this.style[r]);return t}toMarkup(){var t='<img src="'+Gt.escape(this.src)+'"'+(' alt="'+Gt.escape(this.alt)+'"'),r="";for(var n in this.style)this.style.hasOwnProperty(n)&&(r+=Gt.hyphenate(n)+":"+this.style[n]+";");return r&&(t+=' style="'+Gt.escape(r)+'"'),t+="'/>",t}}var Gpe={î:"ı̂",ï:"ı̈",í:"ı́",ì:"ı̀"};class oo{constructor(t,r,n,a,i,o,s,l){this.text=void 0,this.height=void 0,this.depth=void 0,this.italic=void 0,this.skew=void 0,this.width=void 0,this.maxFontSize=void 0,this.classes=void 0,this.style=void 0,this.text=t,this.height=r||0,this.depth=n||0,this.italic=a||0,this.skew=i||0,this.width=o||0,this.classes=s||[],this.style=l||{},this.maxFontSize=0;var u=Ape(this.text.charCodeAt(0));u&&this.classes.push(u+"_fallback"),/[îïíì]/.test(this.text)&&(this.text=Gpe[this.text])}hasClass(t){return Gt.contains(this.classes,t)}toNode(){var t=document.createTextNode(this.text),r=null;this.italic>0&&(r=document.createElement("span"),r.style.marginRight=ct(this.italic)),this.classes.length>0&&(r=r||document.createElement("span"),r.className=Cu(this.classes));for(var n in this.style)this.style.hasOwnProperty(n)&&(r=r||document.createElement("span"),r.style[n]=this.style[n]);return r?(r.appendChild(t),r):t}toMarkup(){var t=!1,r="<span";this.classes.length&&(t=!0,r+=' class="',r+=Gt.escape(Cu(this.classes)),r+='"');var n="";this.italic>0&&(n+="margin-right:"+this.italic+"em;");for(var a in this.style)this.style.hasOwnProperty(a)&&(n+=Gt.hyphenate(a)+":"+this.style[a]+";");n&&(t=!0,r+=' style="'+Gt.escape(n)+'"');var i=Gt.escape(this.text);return t?(r+=">",r+=i,r+="</span>",r):i}}class pl{constructor(t,r){this.children=void 0,this.attributes=void 0,this.children=t||[],this.attributes=r||{}}toNode(){var t="http://www.w3.org/2000/svg",r=document.createElementNS(t,"svg");for(var n in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,n)&&r.setAttribute(n,this.attributes[n]);for(var a=0;a<this.children.length;a++)r.appendChild(this.children[a].toNode());return r}toMarkup(){var t='<svg xmlns="http://www.w3.org/2000/svg"';for(var r in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,r)&&(t+=" "+r+'="'+Gt.escape(this.attributes[r])+'"');t+=">";for(var n=0;n<this.children.length;n++)t+=this.children[n].toMarkup();return t+="</svg>",t}}class Tu{constructor(t,r){this.pathName=void 0,this.alternate=void 0,this.pathName=t,this.alternate=r}toNode(){var t="http://www.w3.org/2000/svg",r=document.createElementNS(t,"path");return this.alternate?r.setAttribute("d",this.alternate):r.setAttribute("d",b9[this.pathName]),r}toMarkup(){return this.alternate?'<path d="'+Gt.escape(this.alternate)+'"/>':'<path d="'+Gt.escape(b9[this.pathName])+'"/>'}}class w_{constructor(t){this.attributes=void 0,this.attributes=t||{}}toNode(){var t="http://www.w3.org/2000/svg",r=document.createElementNS(t,"line");for(var n in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,n)&&r.setAttribute(n,this.attributes[n]);return r}toMarkup(){var t="<line";for(var r in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,r)&&(t+=" "+r+'="'+Gt.escape(this.attributes[r])+'"');return t+="/>",t}}function E9(e){if(e instanceof oo)return e;throw new Error("Expected symbolNode but got "+String(e)+".")}function jpe(e){if(e instanceof cg)return e;throw new Error("Expected span<HtmlDomNode> but got "+String(e)+".")}var qpe={bin:1,close:1,inner:1,open:1,punct:1,rel:1},Wpe={"accent-token":1,mathord:1,"op-token":1,spacing:1,textord:1},sn={math:{},text:{}};function O(e,t,r,n,a,i){sn[e][a]={font:t,group:r,replace:n},i&&n&&(sn[e][n]=sn[e][a])}var q="math",Ke="text",Z="main",le="ams",bn="accent-token",St="bin",Xa="close",op="inner",Wt="mathord",qn="op-token",Ui="open",Ev="punct",ue="rel",Tl="spacing",ye="textord";O(q,Z,ue,"≡","\\equiv",!0);O(q,Z,ue,"≺","\\prec",!0);O(q,Z,ue,"≻","\\succ",!0);O(q,Z,ue,"∼","\\sim",!0);O(q,Z,ue,"⊥","\\perp");O(q,Z,ue,"⪯","\\preceq",!0);O(q,Z,ue,"⪰","\\succeq",!0);O(q,Z,ue,"≃","\\simeq",!0);O(q,Z,ue,"∣","\\mid",!0);O(q,Z,ue,"≪","\\ll",!0);O(q,Z,ue,"≫","\\gg",!0);O(q,Z,ue,"≍","\\asymp",!0);O(q,Z,ue,"∥","\\parallel");O(q,Z,ue,"⋈","\\bowtie",!0);O(q,Z,ue,"⌣","\\smile",!0);O(q,Z,ue,"⊑","\\sqsubseteq",!0);O(q,Z,ue,"⊒","\\sqsupseteq",!0);O(q,Z,ue,"≐","\\doteq",!0);O(q,Z,ue,"⌢","\\frown",!0);O(q,Z,ue,"∋","\\ni",!0);O(q,Z,ue,"∝","\\propto",!0);O(q,Z,ue,"⊢","\\vdash",!0);O(q,Z,ue,"⊣","\\dashv",!0);O(q,Z,ue,"∋","\\owns");O(q,Z,Ev,".","\\ldotp");O(q,Z,Ev,"⋅","\\cdotp");O(q,Z,ye,"#","\\#");O(Ke,Z,ye,"#","\\#");O(q,Z,ye,"&","\\&");O(Ke,Z,ye,"&","\\&");O(q,Z,ye,"ℵ","\\aleph",!0);O(q,Z,ye,"∀","\\forall",!0);O(q,Z,ye,"ℏ","\\hbar",!0);O(q,Z,ye,"∃","\\exists",!0);O(q,Z,ye,"∇","\\nabla",!0);O(q,Z,ye,"♭","\\flat",!0);O(q,Z,ye,"ℓ","\\ell",!0);O(q,Z,ye,"♮","\\natural",!0);O(q,Z,ye,"♣","\\clubsuit",!0);O(q,Z,ye,"℘","\\wp",!0);O(q,Z,ye,"♯","\\sharp",!0);O(q,Z,ye,"♢","\\diamondsuit",!0);O(q,Z,ye,"ℜ","\\Re",!0);O(q,Z,ye,"♡","\\heartsuit",!0);O(q,Z,ye,"ℑ","\\Im",!0);O(q,Z,ye,"♠","\\spadesuit",!0);O(q,Z,ye,"§","\\S",!0);O(Ke,Z,ye,"§","\\S");O(q,Z,ye,"¶","\\P",!0);O(Ke,Z,ye,"¶","\\P");O(q,Z,ye,"†","\\dag");O(Ke,Z,ye,"†","\\dag");O(Ke,Z,ye,"†","\\textdagger");O(q,Z,ye,"‡","\\ddag");O(Ke,Z,ye,"‡","\\ddag");O(Ke,Z,ye,"‡","\\textdaggerdbl");O(q,Z,Xa,"⎱","\\rmoustache",!0);O(q,Z,Ui,"⎰","\\lmoustache",!0);O(q,Z,Xa,"⟯","\\rgroup",!0);O(q,Z,Ui,"⟮","\\lgroup",!0);O(q,Z,St,"∓","\\mp",!0);O(q,Z,St,"⊖","\\ominus",!0);O(q,Z,St,"⊎","\\uplus",!0);O(q,Z,St,"⊓","\\sqcap",!0);O(q,Z,St,"∗","\\ast");O(q,Z,St,"⊔","\\sqcup",!0);O(q,Z,St,"◯","\\bigcirc",!0);O(q,Z,St,"∙","\\bullet",!0);O(q,Z,St,"‡","\\ddagger");O(q,Z,St,"≀","\\wr",!0);O(q,Z,St,"⨿","\\amalg");O(q,Z,St,"&","\\And");O(q,Z,ue,"⟵","\\longleftarrow",!0);O(q,Z,ue,"⇐","\\Leftarrow",!0);O(q,Z,ue,"⟸","\\Longleftarrow",!0);O(q,Z,ue,"⟶","\\longrightarrow",!0);O(q,Z,ue,"⇒","\\Rightarrow",!0);O(q,Z,ue,"⟹","\\Longrightarrow",!0);O(q,Z,ue,"↔","\\leftrightarrow",!0);O(q,Z,ue,"⟷","\\longleftrightarrow",!0);O(q,Z,ue,"⇔","\\Leftrightarrow",!0);O(q,Z,ue,"⟺","\\Longleftrightarrow",!0);O(q,Z,ue,"↦","\\mapsto",!0);O(q,Z,ue,"⟼","\\longmapsto",!0);O(q,Z,ue,"↗","\\nearrow",!0);O(q,Z,ue,"↩","\\hookleftarrow",!0);O(q,Z,ue,"↪","\\hookrightarrow",!0);O(q,Z,ue,"↘","\\searrow",!0);O(q,Z,ue,"↼","\\leftharpoonup",!0);O(q,Z,ue,"⇀","\\rightharpoonup",!0);O(q,Z,ue,"↙","\\swarrow",!0);O(q,Z,ue,"↽","\\leftharpoondown",!0);O(q,Z,ue,"⇁","\\rightharpoondown",!0);O(q,Z,ue,"↖","\\nwarrow",!0);O(q,Z,ue,"⇌","\\rightleftharpoons",!0);O(q,le,ue,"≮","\\nless",!0);O(q,le,ue,"","\\@nleqslant");O(q,le,ue,"","\\@nleqq");O(q,le,ue,"⪇","\\lneq",!0);O(q,le,ue,"≨","\\lneqq",!0);O(q,le,ue,"","\\@lvertneqq");O(q,le,ue,"⋦","\\lnsim",!0);O(q,le,ue,"⪉","\\lnapprox",!0);O(q,le,ue,"⊀","\\nprec",!0);O(q,le,ue,"⋠","\\npreceq",!0);O(q,le,ue,"⋨","\\precnsim",!0);O(q,le,ue,"⪹","\\precnapprox",!0);O(q,le,ue,"≁","\\nsim",!0);O(q,le,ue,"","\\@nshortmid");O(q,le,ue,"∤","\\nmid",!0);O(q,le,ue,"⊬","\\nvdash",!0);O(q,le,ue,"⊭","\\nvDash",!0);O(q,le,ue,"⋪","\\ntriangleleft");O(q,le,ue,"⋬","\\ntrianglelefteq",!0);O(q,le,ue,"⊊","\\subsetneq",!0);O(q,le,ue,"","\\@varsubsetneq");O(q,le,ue,"⫋","\\subsetneqq",!0);O(q,le,ue,"","\\@varsubsetneqq");O(q,le,ue,"≯","\\ngtr",!0);O(q,le,ue,"","\\@ngeqslant");O(q,le,ue,"","\\@ngeqq");O(q,le,ue,"⪈","\\gneq",!0);O(q,le,ue,"≩","\\gneqq",!0);O(q,le,ue,"","\\@gvertneqq");O(q,le,ue,"⋧","\\gnsim",!0);O(q,le,ue,"⪊","\\gnapprox",!0);O(q,le,ue,"⊁","\\nsucc",!0);O(q,le,ue,"⋡","\\nsucceq",!0);O(q,le,ue,"⋩","\\succnsim",!0);O(q,le,ue,"⪺","\\succnapprox",!0);O(q,le,ue,"≆","\\ncong",!0);O(q,le,ue,"","\\@nshortparallel");O(q,le,ue,"∦","\\nparallel",!0);O(q,le,ue,"⊯","\\nVDash",!0);O(q,le,ue,"⋫","\\ntriangleright");O(q,le,ue,"⋭","\\ntrianglerighteq",!0);O(q,le,ue,"","\\@nsupseteqq");O(q,le,ue,"⊋","\\supsetneq",!0);O(q,le,ue,"","\\@varsupsetneq");O(q,le,ue,"⫌","\\supsetneqq",!0);O(q,le,ue,"","\\@varsupsetneqq");O(q,le,ue,"⊮","\\nVdash",!0);O(q,le,ue,"⪵","\\precneqq",!0);O(q,le,ue,"⪶","\\succneqq",!0);O(q,le,ue,"","\\@nsubseteqq");O(q,le,St,"⊴","\\unlhd");O(q,le,St,"⊵","\\unrhd");O(q,le,ue,"↚","\\nleftarrow",!0);O(q,le,ue,"↛","\\nrightarrow",!0);O(q,le,ue,"⇍","\\nLeftarrow",!0);O(q,le,ue,"⇏","\\nRightarrow",!0);O(q,le,ue,"↮","\\nleftrightarrow",!0);O(q,le,ue,"⇎","\\nLeftrightarrow",!0);O(q,le,ue,"△","\\vartriangle");O(q,le,ye,"ℏ","\\hslash");O(q,le,ye,"▽","\\triangledown");O(q,le,ye,"◊","\\lozenge");O(q,le,ye,"Ⓢ","\\circledS");O(q,le,ye,"®","\\circledR");O(Ke,le,ye,"®","\\circledR");O(q,le,ye,"∡","\\measuredangle",!0);O(q,le,ye,"∄","\\nexists");O(q,le,ye,"℧","\\mho");O(q,le,ye,"Ⅎ","\\Finv",!0);O(q,le,ye,"⅁","\\Game",!0);O(q,le,ye,"‵","\\backprime");O(q,le,ye,"▲","\\blacktriangle");O(q,le,ye,"▼","\\blacktriangledown");O(q,le,ye,"■","\\blacksquare");O(q,le,ye,"⧫","\\blacklozenge");O(q,le,ye,"★","\\bigstar");O(q,le,ye,"∢","\\sphericalangle",!0);O(q,le,ye,"∁","\\complement",!0);O(q,le,ye,"ð","\\eth",!0);O(Ke,Z,ye,"ð","ð");O(q,le,ye,"╱","\\diagup");O(q,le,ye,"╲","\\diagdown");O(q,le,ye,"□","\\square");O(q,le,ye,"□","\\Box");O(q,le,ye,"◊","\\Diamond");O(q,le,ye,"¥","\\yen",!0);O(Ke,le,ye,"¥","\\yen",!0);O(q,le,ye,"✓","\\checkmark",!0);O(Ke,le,ye,"✓","\\checkmark");O(q,le,ye,"ℶ","\\beth",!0);O(q,le,ye,"ℸ","\\daleth",!0);O(q,le,ye,"ℷ","\\gimel",!0);O(q,le,ye,"ϝ","\\digamma",!0);O(q,le,ye,"ϰ","\\varkappa");O(q,le,Ui,"┌","\\@ulcorner",!0);O(q,le,Xa,"┐","\\@urcorner",!0);O(q,le,Ui,"└","\\@llcorner",!0);O(q,le,Xa,"┘","\\@lrcorner",!0);O(q,le,ue,"≦","\\leqq",!0);O(q,le,ue,"⩽","\\leqslant",!0);O(q,le,ue,"⪕","\\eqslantless",!0);O(q,le,ue,"≲","\\lesssim",!0);O(q,le,ue,"⪅","\\lessapprox",!0);O(q,le,ue,"≊","\\approxeq",!0);O(q,le,St,"⋖","\\lessdot");O(q,le,ue,"⋘","\\lll",!0);O(q,le,ue,"≶","\\lessgtr",!0);O(q,le,ue,"⋚","\\lesseqgtr",!0);O(q,le,ue,"⪋","\\lesseqqgtr",!0);O(q,le,ue,"≑","\\doteqdot");O(q,le,ue,"≓","\\risingdotseq",!0);O(q,le,ue,"≒","\\fallingdotseq",!0);O(q,le,ue,"∽","\\backsim",!0);O(q,le,ue,"⋍","\\backsimeq",!0);O(q,le,ue,"⫅","\\subseteqq",!0);O(q,le,ue,"⋐","\\Subset",!0);O(q,le,ue,"⊏","\\sqsubset",!0);O(q,le,ue,"≼","\\preccurlyeq",!0);O(q,le,ue,"⋞","\\curlyeqprec",!0);O(q,le,ue,"≾","\\precsim",!0);O(q,le,ue,"⪷","\\precapprox",!0);O(q,le,ue,"⊲","\\vartriangleleft");O(q,le,ue,"⊴","\\trianglelefteq");O(q,le,ue,"⊨","\\vDash",!0);O(q,le,ue,"⊪","\\Vvdash",!0);O(q,le,ue,"⌣","\\smallsmile");O(q,le,ue,"⌢","\\smallfrown");O(q,le,ue,"≏","\\bumpeq",!0);O(q,le,ue,"≎","\\Bumpeq",!0);O(q,le,ue,"≧","\\geqq",!0);O(q,le,ue,"⩾","\\geqslant",!0);O(q,le,ue,"⪖","\\eqslantgtr",!0);O(q,le,ue,"≳","\\gtrsim",!0);O(q,le,ue,"⪆","\\gtrapprox",!0);O(q,le,St,"⋗","\\gtrdot");O(q,le,ue,"⋙","\\ggg",!0);O(q,le,ue,"≷","\\gtrless",!0);O(q,le,ue,"⋛","\\gtreqless",!0);O(q,le,ue,"⪌","\\gtreqqless",!0);O(q,le,ue,"≖","\\eqcirc",!0);O(q,le,ue,"≗","\\circeq",!0);O(q,le,ue,"≜","\\triangleq",!0);O(q,le,ue,"∼","\\thicksim");O(q,le,ue,"≈","\\thickapprox");O(q,le,ue,"⫆","\\supseteqq",!0);O(q,le,ue,"⋑","\\Supset",!0);O(q,le,ue,"⊐","\\sqsupset",!0);O(q,le,ue,"≽","\\succcurlyeq",!0);O(q,le,ue,"⋟","\\curlyeqsucc",!0);O(q,le,ue,"≿","\\succsim",!0);O(q,le,ue,"⪸","\\succapprox",!0);O(q,le,ue,"⊳","\\vartriangleright");O(q,le,ue,"⊵","\\trianglerighteq");O(q,le,ue,"⊩","\\Vdash",!0);O(q,le,ue,"∣","\\shortmid");O(q,le,ue,"∥","\\shortparallel");O(q,le,ue,"≬","\\between",!0);O(q,le,ue,"⋔","\\pitchfork",!0);O(q,le,ue,"∝","\\varpropto");O(q,le,ue,"◀","\\blacktriangleleft");O(q,le,ue,"∴","\\therefore",!0);O(q,le,ue,"∍","\\backepsilon");O(q,le,ue,"▶","\\blacktriangleright");O(q,le,ue,"∵","\\because",!0);O(q,le,ue,"⋘","\\llless");O(q,le,ue,"⋙","\\gggtr");O(q,le,St,"⊲","\\lhd");O(q,le,St,"⊳","\\rhd");O(q,le,ue,"≂","\\eqsim",!0);O(q,Z,ue,"⋈","\\Join");O(q,le,ue,"≑","\\Doteq",!0);O(q,le,St,"∔","\\dotplus",!0);O(q,le,St,"∖","\\smallsetminus");O(q,le,St,"⋒","\\Cap",!0);O(q,le,St,"⋓","\\Cup",!0);O(q,le,St,"⩞","\\doublebarwedge",!0);O(q,le,St,"⊟","\\boxminus",!0);O(q,le,St,"⊞","\\boxplus",!0);O(q,le,St,"⋇","\\divideontimes",!0);O(q,le,St,"⋉","\\ltimes",!0);O(q,le,St,"⋊","\\rtimes",!0);O(q,le,St,"⋋","\\leftthreetimes",!0);O(q,le,St,"⋌","\\rightthreetimes",!0);O(q,le,St,"⋏","\\curlywedge",!0);O(q,le,St,"⋎","\\curlyvee",!0);O(q,le,St,"⊝","\\circleddash",!0);O(q,le,St,"⊛","\\circledast",!0);O(q,le,St,"⋅","\\centerdot");O(q,le,St,"⊺","\\intercal",!0);O(q,le,St,"⋒","\\doublecap");O(q,le,St,"⋓","\\doublecup");O(q,le,St,"⊠","\\boxtimes",!0);O(q,le,ue,"⇢","\\dashrightarrow",!0);O(q,le,ue,"⇠","\\dashleftarrow",!0);O(q,le,ue,"⇇","\\leftleftarrows",!0);O(q,le,ue,"⇆","\\leftrightarrows",!0);O(q,le,ue,"⇚","\\Lleftarrow",!0);O(q,le,ue,"↞","\\twoheadleftarrow",!0);O(q,le,ue,"↢","\\leftarrowtail",!0);O(q,le,ue,"↫","\\looparrowleft",!0);O(q,le,ue,"⇋","\\leftrightharpoons",!0);O(q,le,ue,"↶","\\curvearrowleft",!0);O(q,le,ue,"↺","\\circlearrowleft",!0);O(q,le,ue,"↰","\\Lsh",!0);O(q,le,ue,"⇈","\\upuparrows",!0);O(q,le,ue,"↿","\\upharpoonleft",!0);O(q,le,ue,"⇃","\\downharpoonleft",!0);O(q,Z,ue,"⊶","\\origof",!0);O(q,Z,ue,"⊷","\\imageof",!0);O(q,le,ue,"⊸","\\multimap",!0);O(q,le,ue,"↭","\\leftrightsquigarrow",!0);O(q,le,ue,"⇉","\\rightrightarrows",!0);O(q,le,ue,"⇄","\\rightleftarrows",!0);O(q,le,ue,"↠","\\twoheadrightarrow",!0);O(q,le,ue,"↣","\\rightarrowtail",!0);O(q,le,ue,"↬","\\looparrowright",!0);O(q,le,ue,"↷","\\curvearrowright",!0);O(q,le,ue,"↻","\\circlearrowright",!0);O(q,le,ue,"↱","\\Rsh",!0);O(q,le,ue,"⇊","\\downdownarrows",!0);O(q,le,ue,"↾","\\upharpoonright",!0);O(q,le,ue,"⇂","\\downharpoonright",!0);O(q,le,ue,"⇝","\\rightsquigarrow",!0);O(q,le,ue,"⇝","\\leadsto");O(q,le,ue,"⇛","\\Rrightarrow",!0);O(q,le,ue,"↾","\\restriction");O(q,Z,ye,"‘","`");O(q,Z,ye,"$","\\$");O(Ke,Z,ye,"$","\\$");O(Ke,Z,ye,"$","\\textdollar");O(q,Z,ye,"%","\\%");O(Ke,Z,ye,"%","\\%");O(q,Z,ye,"_","\\_");O(Ke,Z,ye,"_","\\_");O(Ke,Z,ye,"_","\\textunderscore");O(q,Z,ye,"∠","\\angle",!0);O(q,Z,ye,"∞","\\infty",!0);O(q,Z,ye,"′","\\prime");O(q,Z,ye,"△","\\triangle");O(q,Z,ye,"Γ","\\Gamma",!0);O(q,Z,ye,"Δ","\\Delta",!0);O(q,Z,ye,"Θ","\\Theta",!0);O(q,Z,ye,"Λ","\\Lambda",!0);O(q,Z,ye,"Ξ","\\Xi",!0);O(q,Z,ye,"Π","\\Pi",!0);O(q,Z,ye,"Σ","\\Sigma",!0);O(q,Z,ye,"Υ","\\Upsilon",!0);O(q,Z,ye,"Φ","\\Phi",!0);O(q,Z,ye,"Ψ","\\Psi",!0);O(q,Z,ye,"Ω","\\Omega",!0);O(q,Z,ye,"A","Α");O(q,Z,ye,"B","Β");O(q,Z,ye,"E","Ε");O(q,Z,ye,"Z","Ζ");O(q,Z,ye,"H","Η");O(q,Z,ye,"I","Ι");O(q,Z,ye,"K","Κ");O(q,Z,ye,"M","Μ");O(q,Z,ye,"N","Ν");O(q,Z,ye,"O","Ο");O(q,Z,ye,"P","Ρ");O(q,Z,ye,"T","Τ");O(q,Z,ye,"X","Χ");O(q,Z,ye,"¬","\\neg",!0);O(q,Z,ye,"¬","\\lnot");O(q,Z,ye,"⊤","\\top");O(q,Z,ye,"⊥","\\bot");O(q,Z,ye,"∅","\\emptyset");O(q,le,ye,"∅","\\varnothing");O(q,Z,Wt,"α","\\alpha",!0);O(q,Z,Wt,"β","\\beta",!0);O(q,Z,Wt,"γ","\\gamma",!0);O(q,Z,Wt,"δ","\\delta",!0);O(q,Z,Wt,"ϵ","\\epsilon",!0);O(q,Z,Wt,"ζ","\\zeta",!0);O(q,Z,Wt,"η","\\eta",!0);O(q,Z,Wt,"θ","\\theta",!0);O(q,Z,Wt,"ι","\\iota",!0);O(q,Z,Wt,"κ","\\kappa",!0);O(q,Z,Wt,"λ","\\lambda",!0);O(q,Z,Wt,"μ","\\mu",!0);O(q,Z,Wt,"ν","\\nu",!0);O(q,Z,Wt,"ξ","\\xi",!0);O(q,Z,Wt,"ο","\\omicron",!0);O(q,Z,Wt,"π","\\pi",!0);O(q,Z,Wt,"ρ","\\rho",!0);O(q,Z,Wt,"σ","\\sigma",!0);O(q,Z,Wt,"τ","\\tau",!0);O(q,Z,Wt,"υ","\\upsilon",!0);O(q,Z,Wt,"ϕ","\\phi",!0);O(q,Z,Wt,"χ","\\chi",!0);O(q,Z,Wt,"ψ","\\psi",!0);O(q,Z,Wt,"ω","\\omega",!0);O(q,Z,Wt,"ε","\\varepsilon",!0);O(q,Z,Wt,"ϑ","\\vartheta",!0);O(q,Z,Wt,"ϖ","\\varpi",!0);O(q,Z,Wt,"ϱ","\\varrho",!0);O(q,Z,Wt,"ς","\\varsigma",!0);O(q,Z,Wt,"φ","\\varphi",!0);O(q,Z,St,"∗","*",!0);O(q,Z,St,"+","+");O(q,Z,St,"−","-",!0);O(q,Z,St,"⋅","\\cdot",!0);O(q,Z,St,"∘","\\circ",!0);O(q,Z,St,"÷","\\div",!0);O(q,Z,St,"±","\\pm",!0);O(q,Z,St,"×","\\times",!0);O(q,Z,St,"∩","\\cap",!0);O(q,Z,St,"∪","\\cup",!0);O(q,Z,St,"∖","\\setminus",!0);O(q,Z,St,"∧","\\land");O(q,Z,St,"∨","\\lor");O(q,Z,St,"∧","\\wedge",!0);O(q,Z,St,"∨","\\vee",!0);O(q,Z,ye,"√","\\surd");O(q,Z,Ui,"⟨","\\langle",!0);O(q,Z,Ui,"∣","\\lvert");O(q,Z,Ui,"∥","\\lVert");O(q,Z,Xa,"?","?");O(q,Z,Xa,"!","!");O(q,Z,Xa,"⟩","\\rangle",!0);O(q,Z,Xa,"∣","\\rvert");O(q,Z,Xa,"∥","\\rVert");O(q,Z,ue,"=","=");O(q,Z,ue,":",":");O(q,Z,ue,"≈","\\approx",!0);O(q,Z,ue,"≅","\\cong",!0);O(q,Z,ue,"≥","\\ge");O(q,Z,ue,"≥","\\geq",!0);O(q,Z,ue,"←","\\gets");O(q,Z,ue,">","\\gt",!0);O(q,Z,ue,"∈","\\in",!0);O(q,Z,ue,"","\\@not");O(q,Z,ue,"⊂","\\subset",!0);O(q,Z,ue,"⊃","\\supset",!0);O(q,Z,ue,"⊆","\\subseteq",!0);O(q,Z,ue,"⊇","\\supseteq",!0);O(q,le,ue,"⊈","\\nsubseteq",!0);O(q,le,ue,"⊉","\\nsupseteq",!0);O(q,Z,ue,"⊨","\\models");O(q,Z,ue,"←","\\leftarrow",!0);O(q,Z,ue,"≤","\\le");O(q,Z,ue,"≤","\\leq",!0);O(q,Z,ue,"<","\\lt",!0);O(q,Z,ue,"→","\\rightarrow",!0);O(q,Z,ue,"→","\\to");O(q,le,ue,"≱","\\ngeq",!0);O(q,le,ue,"≰","\\nleq",!0);O(q,Z,Tl," ","\\ ");O(q,Z,Tl," ","\\space");O(q,Z,Tl," ","\\nobreakspace");O(Ke,Z,Tl," ","\\ ");O(Ke,Z,Tl," "," ");O(Ke,Z,Tl," ","\\space");O(Ke,Z,Tl," ","\\nobreakspace");O(q,Z,Tl,null,"\\nobreak");O(q,Z,Tl,null,"\\allowbreak");O(q,Z,Ev,",",",");O(q,Z,Ev,";",";");O(q,le,St,"⊼","\\barwedge",!0);O(q,le,St,"⊻","\\veebar",!0);O(q,Z,St,"⊙","\\odot",!0);O(q,Z,St,"⊕","\\oplus",!0);O(q,Z,St,"⊗","\\otimes",!0);O(q,Z,ye,"∂","\\partial",!0);O(q,Z,St,"⊘","\\oslash",!0);O(q,le,St,"⊚","\\circledcirc",!0);O(q,le,St,"⊡","\\boxdot",!0);O(q,Z,St,"△","\\bigtriangleup");O(q,Z,St,"▽","\\bigtriangledown");O(q,Z,St,"†","\\dagger");O(q,Z,St,"⋄","\\diamond");O(q,Z,St,"⋆","\\star");O(q,Z,St,"◃","\\triangleleft");O(q,Z,St,"▹","\\triangleright");O(q,Z,Ui,"{","\\{");O(Ke,Z,ye,"{","\\{");O(Ke,Z,ye,"{","\\textbraceleft");O(q,Z,Xa,"}","\\}");O(Ke,Z,ye,"}","\\}");O(Ke,Z,ye,"}","\\textbraceright");O(q,Z,Ui,"{","\\lbrace");O(q,Z,Xa,"}","\\rbrace");O(q,Z,Ui,"[","\\lbrack",!0);O(Ke,Z,ye,"[","\\lbrack",!0);O(q,Z,Xa,"]","\\rbrack",!0);O(Ke,Z,ye,"]","\\rbrack",!0);O(q,Z,Ui,"(","\\lparen",!0);O(q,Z,Xa,")","\\rparen",!0);O(Ke,Z,ye,"<","\\textless",!0);O(Ke,Z,ye,">","\\textgreater",!0);O(q,Z,Ui,"⌊","\\lfloor",!0);O(q,Z,Xa,"⌋","\\rfloor",!0);O(q,Z,Ui,"⌈","\\lceil",!0);O(q,Z,Xa,"⌉","\\rceil",!0);O(q,Z,ye,"\\","\\backslash");O(q,Z,ye,"∣","|");O(q,Z,ye,"∣","\\vert");O(Ke,Z,ye,"|","\\textbar",!0);O(q,Z,ye,"∥","\\|");O(q,Z,ye,"∥","\\Vert");O(Ke,Z,ye,"∥","\\textbardbl");O(Ke,Z,ye,"~","\\textasciitilde");O(Ke,Z,ye,"\\","\\textbackslash");O(Ke,Z,ye,"^","\\textasciicircum");O(q,Z,ue,"↑","\\uparrow",!0);O(q,Z,ue,"⇑","\\Uparrow",!0);O(q,Z,ue,"↓","\\downarrow",!0);O(q,Z,ue,"⇓","\\Downarrow",!0);O(q,Z,ue,"↕","\\updownarrow",!0);O(q,Z,ue,"⇕","\\Updownarrow",!0);O(q,Z,qn,"∐","\\coprod");O(q,Z,qn,"⋁","\\bigvee");O(q,Z,qn,"⋀","\\bigwedge");O(q,Z,qn,"⨄","\\biguplus");O(q,Z,qn,"⋂","\\bigcap");O(q,Z,qn,"⋃","\\bigcup");O(q,Z,qn,"∫","\\int");O(q,Z,qn,"∫","\\intop");O(q,Z,qn,"∬","\\iint");O(q,Z,qn,"∭","\\iiint");O(q,Z,qn,"∏","\\prod");O(q,Z,qn,"∑","\\sum");O(q,Z,qn,"⨂","\\bigotimes");O(q,Z,qn,"⨁","\\bigoplus");O(q,Z,qn,"⨀","\\bigodot");O(q,Z,qn,"∮","\\oint");O(q,Z,qn,"∯","\\oiint");O(q,Z,qn,"∰","\\oiiint");O(q,Z,qn,"⨆","\\bigsqcup");O(q,Z,qn,"∫","\\smallint");O(Ke,Z,op,"…","\\textellipsis");O(q,Z,op,"…","\\mathellipsis");O(Ke,Z,op,"…","\\ldots",!0);O(q,Z,op,"…","\\ldots",!0);O(q,Z,op,"⋯","\\@cdots",!0);O(q,Z,op,"⋱","\\ddots",!0);O(q,Z,ye,"⋮","\\varvdots");O(Ke,Z,ye,"⋮","\\varvdots");O(q,Z,bn,"ˊ","\\acute");O(q,Z,bn,"ˋ","\\grave");O(q,Z,bn,"¨","\\ddot");O(q,Z,bn,"~","\\tilde");O(q,Z,bn,"ˉ","\\bar");O(q,Z,bn,"˘","\\breve");O(q,Z,bn,"ˇ","\\check");O(q,Z,bn,"^","\\hat");O(q,Z,bn,"⃗","\\vec");O(q,Z,bn,"˙","\\dot");O(q,Z,bn,"˚","\\mathring");O(q,Z,Wt,"","\\@imath");O(q,Z,Wt,"","\\@jmath");O(q,Z,ye,"ı","ı");O(q,Z,ye,"ȷ","ȷ");O(Ke,Z,ye,"ı","\\i",!0);O(Ke,Z,ye,"ȷ","\\j",!0);O(Ke,Z,ye,"ß","\\ss",!0);O(Ke,Z,ye,"æ","\\ae",!0);O(Ke,Z,ye,"œ","\\oe",!0);O(Ke,Z,ye,"ø","\\o",!0);O(Ke,Z,ye,"Æ","\\AE",!0);O(Ke,Z,ye,"Œ","\\OE",!0);O(Ke,Z,ye,"Ø","\\O",!0);O(Ke,Z,bn,"ˊ","\\'");O(Ke,Z,bn,"ˋ","\\`");O(Ke,Z,bn,"ˆ","\\^");O(Ke,Z,bn,"˜","\\~");O(Ke,Z,bn,"ˉ","\\=");O(Ke,Z,bn,"˘","\\u");O(Ke,Z,bn,"˙","\\.");O(Ke,Z,bn,"¸","\\c");O(Ke,Z,bn,"˚","\\r");O(Ke,Z,bn,"ˇ","\\v");O(Ke,Z,bn,"¨",'\\"');O(Ke,Z,bn,"˝","\\H");O(Ke,Z,bn,"◯","\\textcircled");var yK={"--":!0,"---":!0,"``":!0,"''":!0};O(Ke,Z,ye,"–","--",!0);O(Ke,Z,ye,"–","\\textendash");O(Ke,Z,ye,"—","---",!0);O(Ke,Z,ye,"—","\\textemdash");O(Ke,Z,ye,"‘","`",!0);O(Ke,Z,ye,"‘","\\textquoteleft");O(Ke,Z,ye,"’","'",!0);O(Ke,Z,ye,"’","\\textquoteright");O(Ke,Z,ye,"“","``",!0);O(Ke,Z,ye,"“","\\textquotedblleft");O(Ke,Z,ye,"”","''",!0);O(Ke,Z,ye,"”","\\textquotedblright");O(q,Z,ye,"°","\\degree",!0);O(Ke,Z,ye,"°","\\degree");O(Ke,Z,ye,"°","\\textdegree",!0);O(q,Z,ye,"£","\\pounds");O(q,Z,ye,"£","\\mathsterling",!0);O(Ke,Z,ye,"£","\\pounds");O(Ke,Z,ye,"£","\\textsterling",!0);O(q,le,ye,"✠","\\maltese");O(Ke,le,ye,"✠","\\maltese");var x9='0123456789/@."';for(var xx=0;xx<x9.length;xx++){var w9=x9.charAt(xx);O(q,Z,ye,w9,w9)}var k9='0123456789!@*()-=+";:?/.,';for(var wx=0;wx<k9.length;wx++){var C9=k9.charAt(wx);O(Ke,Z,ye,C9,C9)}var ry="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";for(var kx=0;kx<ry.length;kx++){var hb=ry.charAt(kx);O(q,Z,Wt,hb,hb),O(Ke,Z,ye,hb,hb)}O(q,le,ye,"C","ℂ");O(Ke,le,ye,"C","ℂ");O(q,le,ye,"H","ℍ");O(Ke,le,ye,"H","ℍ");O(q,le,ye,"N","ℕ");O(Ke,le,ye,"N","ℕ");O(q,le,ye,"P","ℙ");O(Ke,le,ye,"P","ℙ");O(q,le,ye,"Q","ℚ");O(Ke,le,ye,"Q","ℚ");O(q,le,ye,"R","ℝ");O(Ke,le,ye,"R","ℝ");O(q,le,ye,"Z","ℤ");O(Ke,le,ye,"Z","ℤ");O(q,Z,Wt,"h","ℎ");O(Ke,Z,Wt,"h","ℎ");var nr="";for(var Ua=0;Ua<ry.length;Ua++){var Fn=ry.charAt(Ua);nr=String.fromCharCode(55349,56320+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56372+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56424+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56580+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56684+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56736+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56788+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56840+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56944+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),Ua<26&&(nr=String.fromCharCode(55349,56632+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr),nr=String.fromCharCode(55349,56476+Ua),O(q,Z,Wt,Fn,nr),O(Ke,Z,ye,Fn,nr))}nr="𝕜";O(q,Z,Wt,"k",nr);O(Ke,Z,ye,"k",nr);for(var lc=0;lc<10;lc++){var au=lc.toString();nr=String.fromCharCode(55349,57294+lc),O(q,Z,Wt,au,nr),O(Ke,Z,ye,au,nr),nr=String.fromCharCode(55349,57314+lc),O(q,Z,Wt,au,nr),O(Ke,Z,ye,au,nr),nr=String.fromCharCode(55349,57324+lc),O(q,Z,Wt,au,nr),O(Ke,Z,ye,au,nr),nr=String.fromCharCode(55349,57334+lc),O(q,Z,Wt,au,nr),O(Ke,Z,ye,au,nr)}var k_="ÐÞþ";for(var Cx=0;Cx<k_.length;Cx++){var pb=k_.charAt(Cx);O(q,Z,Wt,pb,pb),O(Ke,Z,ye,pb,pb)}var fb=[["mathbf","textbf","Main-Bold"],["mathbf","textbf","Main-Bold"],["mathnormal","textit","Math-Italic"],["mathnormal","textit","Math-Italic"],["boldsymbol","boldsymbol","Main-BoldItalic"],["boldsymbol","boldsymbol","Main-BoldItalic"],["mathscr","textscr","Script-Regular"],["","",""],["","",""],["","",""],["mathfrak","textfrak","Fraktur-Regular"],["mathfrak","textfrak","Fraktur-Regular"],["mathbb","textbb","AMS-Regular"],["mathbb","textbb","AMS-Regular"],["mathboldfrak","textboldfrak","Fraktur-Regular"],["mathboldfrak","textboldfrak","Fraktur-Regular"],["mathsf","textsf","SansSerif-Regular"],["mathsf","textsf","SansSerif-Regular"],["mathboldsf","textboldsf","SansSerif-Bold"],["mathboldsf","textboldsf","SansSerif-Bold"],["mathitsf","textitsf","SansSerif-Italic"],["mathitsf","textitsf","SansSerif-Italic"],["","",""],["","",""],["mathtt","texttt","Typewriter-Regular"],["mathtt","texttt","Typewriter-Regular"]],T9=[["mathbf","textbf","Main-Bold"],["","",""],["mathsf","textsf","SansSerif-Regular"],["mathboldsf","textboldsf","SansSerif-Bold"],["mathtt","texttt","Typewriter-Regular"]],Vpe=function(t,r){var n=t.charCodeAt(0),a=t.charCodeAt(1),i=(n-55296)*1024+(a-56320)+65536,o=r==="math"?0:1;if(119808<=i&&i<120484){var s=Math.floor((i-119808)/26);return[fb[s][2],fb[s][o]]}else if(120782<=i&&i<=120831){var l=Math.floor((i-120782)/10);return[T9[l][2],T9[l][o]]}else{if(i===120485||i===120486)return[fb[0][2],fb[0][o]];if(120486<i&&i<120782)return["",""];throw new nt("Unsupported character: "+t)}},xv=function(t,r,n){return sn[n][t]&&sn[n][t].replace&&(t=sn[n][t].replace),{value:t,metrics:oD(t,r,n)}},ko=function(t,r,n,a,i){var o=xv(t,r,n),s=o.metrics;t=o.value;var l;if(s){var u=s.italic;(n==="text"||a&&a.font==="mathit")&&(u=0),l=new oo(t,s.height,s.depth,u,s.skew,s.width,i)}else typeof console<"u"&&console.warn("No character metrics "+("for '"+t+"' in style '"+r+"' and mode '"+n+"'")),l=new oo(t,0,0,0,0,0,i);if(a){l.maxFontSize=a.sizeMultiplier,a.style.isTight()&&l.classes.push("mtight");var d=a.getColor();d&&(l.style.color=d)}return l},Kpe=function(t,r,n,a){return a===void 0&&(a=[]),n.font==="boldsymbol"&&xv(t,"Main-Bold",r).metrics?ko(t,"Main-Bold",r,n,a.concat(["mathbf"])):t==="\\"||sn[r][t].font==="main"?ko(t,"Main-Regular",r,n,a):ko(t,"AMS-Regular",r,n,a.concat(["amsrm"]))},Ype=function(t,r,n,a,i){return i!=="textord"&&xv(t,"Math-BoldItalic",r).metrics?{fontName:"Math-BoldItalic",fontClass:"boldsymbol"}:{fontName:"Main-Bold",fontClass:"mathbf"}},Jpe=function(t,r,n){var a=t.mode,i=t.text,o=["mord"],s=a==="math"||a==="text"&&r.font,l=s?r.font:r.fontFamily,u="",d="";if(i.charCodeAt(0)===55349&&([u,d]=Vpe(i,a)),u.length>0)return ko(i,u,a,r,o.concat(d));if(l){var p,f;if(l==="boldsymbol"){var m=Ype(i,a,r,o,n);p=m.fontName,f=[m.fontClass]}else s?(p=EK[l].fontName,f=[l]):(p=mb(l,r.fontWeight,r.fontShape),f=[l,r.fontWeight,r.fontShape]);if(xv(i,p,a).metrics)return ko(i,p,a,r,o.concat(f));if(yK.hasOwnProperty(i)&&p.slice(0,10)==="Typewriter"){for(var b=[],y=0;y<i.length;y++)b.push(ko(i[y],p,a,r,o.concat(f)));return SK(b)}}if(n==="mathord")return ko(i,"Math-Italic",a,r,o.concat(["mathnormal"]));if(n==="textord"){var S=sn[a][i]&&sn[a][i].font;if(S==="ams"){var w=mb("amsrm",r.fontWeight,r.fontShape);return ko(i,w,a,r,o.concat("amsrm",r.fontWeight,r.fontShape))}else if(S==="main"||!S){var E=mb("textrm",r.fontWeight,r.fontShape);return ko(i,E,a,r,o.concat(r.fontWeight,r.fontShape))}else{var k=mb(S,r.fontWeight,r.fontShape);return ko(i,k,a,r,o.concat(k,r.fontWeight,r.fontShape))}}else throw new Error("unexpected type: "+n+" in makeOrd")},Xpe=(e,t)=>{if(Cu(e.classes)!==Cu(t.classes)||e.skew!==t.skew||e.maxFontSize!==t.maxFontSize)return!1;if(e.classes.length===1){var r=e.classes[0];if(r==="mbin"||r==="mord")return!1}for(var n in e.style)if(e.style.hasOwnProperty(n)&&e.style[n]!==t.style[n])return!1;for(var a in t.style)if(t.style.hasOwnProperty(a)&&e.style[a]!==t.style[a])return!1;return!0},Zpe=e=>{for(var t=0;t<e.length-1;t++){var r=e[t],n=e[t+1];r instanceof oo&&n instanceof oo&&Xpe(r,n)&&(r.text+=n.text,r.height=Math.max(r.height,n.height),r.depth=Math.max(r.depth,n.depth),r.italic=n.italic,e.splice(t+1,1),t--)}return e},lD=function(t){for(var r=0,n=0,a=0,i=0;i<t.children.length;i++){var o=t.children[i];o.height>r&&(r=o.height),o.depth>n&&(n=o.depth),o.maxFontSize>a&&(a=o.maxFontSize)}t.height=r,t.depth=n,t.maxFontSize=a},si=function(t,r,n,a){var i=new cg(t,r,n,a);return lD(i),i},vK=(e,t,r,n)=>new cg(e,t,r,n),Qpe=function(t,r,n){var a=si([t],[],r);return a.height=Math.max(n||r.fontMetrics().defaultRuleThickness,r.minRuleThickness),a.style.borderBottomWidth=ct(a.height),a.maxFontSize=1,a},efe=function(t,r,n,a){var i=new sD(t,r,n,a);return lD(i),i},SK=function(t){var r=new ug(t);return lD(r),r},tfe=function(t,r){return t instanceof ug?si([],[t],r):t},rfe=function(t){if(t.positionType==="individualShift"){for(var r=t.children,n=[r[0]],a=-r[0].shift-r[0].elem.depth,i=a,o=1;o<r.length;o++){var s=-r[o].shift-i-r[o].elem.depth,l=s-(r[o-1].elem.height+r[o-1].elem.depth);i=i+s,n.push({type:"kern",size:l}),n.push(r[o])}return{children:n,depth:a}}var u;if(t.positionType==="top"){for(var d=t.positionData,p=0;p<t.children.length;p++){var f=t.children[p];d-=f.type==="kern"?f.size:f.elem.height+f.elem.depth}u=d}else if(t.positionType==="bottom")u=-t.positionData;else{var m=t.children[0];if(m.type!=="elem")throw new Error('First child must have type "elem".');if(t.positionType==="shift")u=-m.elem.depth-t.positionData;else if(t.positionType==="firstBaseline")u=-m.elem.depth;else throw new Error("Invalid positionType "+t.positionType+".")}return{children:t.children,depth:u}},nfe=function(t,r){for(var{children:n,depth:a}=rfe(t),i=0,o=0;o<n.length;o++){var s=n[o];if(s.type==="elem"){var l=s.elem;i=Math.max(i,l.maxFontSize,l.height)}}i+=2;var u=si(["pstrut"],[]);u.style.height=ct(i);for(var d=[],p=a,f=a,m=a,b=0;b<n.length;b++){var y=n[b];if(y.type==="kern")m+=y.size;else{var S=y.elem,w=y.wrapperClasses||[],E=y.wrapperStyle||{},k=si(w,[u,S],void 0,E);k.style.top=ct(-i-m-S.depth),y.marginLeft&&(k.style.marginLeft=y.marginLeft),y.marginRight&&(k.style.marginRight=y.marginRight),d.push(k),m+=S.height+S.depth}p=Math.min(p,m),f=Math.max(f,m)}var _=si(["vlist"],d);_.style.height=ct(f);var R;if(p<0){var C=si([],[]),F=si(["vlist"],[C]);F.style.height=ct(-p);var L=si(["vlist-s"],[new oo("​")]);R=[si(["vlist-r"],[_,L]),si(["vlist-r"],[F])]}else R=[si(["vlist-r"],[_])];var I=si(["vlist-t"],R);return R.length===2&&I.classes.push("vlist-t2"),I.height=f,I.depth=-p,I},afe=(e,t)=>{var r=si(["mspace"],[],t),n=xn(e,t);return r.style.marginRight=ct(n),r},mb=function(t,r,n){var a="";switch(t){case"amsrm":a="AMS";break;case"textrm":a="Main";break;case"textsf":a="SansSerif";break;case"texttt":a="Typewriter";break;default:a=t}var i;return r==="textbf"&&n==="textit"?i="BoldItalic":r==="textbf"?i="Bold":r==="textit"?i="Italic":i="Regular",a+"-"+i},EK={mathbf:{variant:"bold",fontName:"Main-Bold"},mathrm:{variant:"normal",fontName:"Main-Regular"},textit:{variant:"italic",fontName:"Main-Italic"},mathit:{variant:"italic",fontName:"Main-Italic"},mathnormal:{variant:"italic",fontName:"Math-Italic"},mathsfit:{variant:"sans-serif-italic",fontName:"SansSerif-Italic"},mathbb:{variant:"double-struck",fontName:"AMS-Regular"},mathcal:{variant:"script",fontName:"Caligraphic-Regular"},mathfrak:{variant:"fraktur",fontName:"Fraktur-Regular"},mathscr:{variant:"script",fontName:"Script-Regular"},mathsf:{variant:"sans-serif",fontName:"SansSerif-Regular"},mathtt:{variant:"monospace",fontName:"Typewriter-Regular"}},xK={vec:["vec",.471,.714],oiintSize1:["oiintSize1",.957,.499],oiintSize2:["oiintSize2",1.472,.659],oiiintSize1:["oiiintSize1",1.304,.499],oiiintSize2:["oiiintSize2",1.98,.659]},ife=function(t,r){var[n,a,i]=xK[t],o=new Tu(n),s=new pl([o],{width:ct(a),height:ct(i),style:"width:"+ct(a),viewBox:"0 0 "+1e3*a+" "+1e3*i,preserveAspectRatio:"xMinYMin"}),l=vK(["overlay"],[s],r);return l.height=i,l.style.height=ct(i),l.style.width=ct(a),l},Fe={fontMap:EK,makeSymbol:ko,mathsym:Kpe,makeSpan:si,makeSvgSpan:vK,makeLineSpan:Qpe,makeAnchor:efe,makeFragment:SK,wrapFragment:tfe,makeVList:nfe,makeOrd:Jpe,makeGlue:afe,staticSvg:ife,svgData:xK,tryCombineChars:Zpe},Sn={number:3,unit:"mu"},uc={number:4,unit:"mu"},Gs={number:5,unit:"mu"},ofe={mord:{mop:Sn,mbin:uc,mrel:Gs,minner:Sn},mop:{mord:Sn,mop:Sn,mrel:Gs,minner:Sn},mbin:{mord:uc,mop:uc,mopen:uc,minner:uc},mrel:{mord:Gs,mop:Gs,mopen:Gs,minner:Gs},mopen:{},mclose:{mop:Sn,mbin:uc,mrel:Gs,minner:Sn},mpunct:{mord:Sn,mop:Sn,mrel:Gs,mopen:Sn,mclose:Sn,mpunct:Sn,minner:Sn},minner:{mord:Sn,mop:Sn,mbin:uc,mrel:Gs,mopen:Sn,mpunct:Sn,minner:Sn}},sfe={mord:{mop:Sn},mop:{mord:Sn,mop:Sn},mbin:{},mrel:{},mopen:{},mclose:{mop:Sn},mpunct:{},minner:{mop:Sn}},wK={},ny={},ay={};function gt(e){for(var{type:t,names:r,props:n,handler:a,htmlBuilder:i,mathmlBuilder:o}=e,s={type:t,numArgs:n.numArgs,argTypes:n.argTypes,allowedInArgument:!!n.allowedInArgument,allowedInText:!!n.allowedInText,allowedInMath:n.allowedInMath===void 0?!0:n.allowedInMath,numOptionalArgs:n.numOptionalArgs||0,infix:!!n.infix,primitive:!!n.primitive,handler:a},l=0;l<r.length;++l)wK[r[l]]=s;t&&(i&&(ny[t]=i),o&&(ay[t]=o))}function ed(e){var{type:t,htmlBuilder:r,mathmlBuilder:n}=e;gt({type:t,names:[],props:{numArgs:0},handler(){throw new Error("Should never be called.")},htmlBuilder:r,mathmlBuilder:n})}var iy=function(t){return t.type==="ordgroup"&&t.body.length===1?t.body[0]:t},Ln=function(t){return t.type==="ordgroup"?t.body:[t]},fl=Fe.makeSpan,lfe=["leftmost","mbin","mopen","mrel","mop","mpunct"],ufe=["rightmost","mrel","mclose","mpunct"],cfe={display:Vt.DISPLAY,text:Vt.TEXT,script:Vt.SCRIPT,scriptscript:Vt.SCRIPTSCRIPT},dfe={mord:"mord",mop:"mop",mbin:"mbin",mrel:"mrel",mopen:"mopen",mclose:"mclose",mpunct:"mpunct",minner:"minner"},ta=function(t,r,n,a){a===void 0&&(a=[null,null]);for(var i=[],o=0;o<t.length;o++){var s=Ir(t[o],r);if(s instanceof ug){var l=s.children;i.push(...l)}else i.push(s)}if(Fe.tryCombineChars(i),!n)return i;var u=r;if(t.length===1){var d=t[0];d.type==="sizing"?u=r.havingSize(d.size):d.type==="styling"&&(u=r.havingStyle(cfe[d.style]))}var p=fl([a[0]||"leftmost"],[],r),f=fl([a[1]||"rightmost"],[],r),m=n==="root";return A9(i,(b,y)=>{var S=y.classes[0],w=b.classes[0];S==="mbin"&&Gt.contains(ufe,w)?y.classes[0]="mord":w==="mbin"&&Gt.contains(lfe,S)&&(b.classes[0]="mord")},{node:p},f,m),A9(i,(b,y)=>{var S=C_(y),w=C_(b),E=S&&w?b.hasClass("mtight")?sfe[S][w]:ofe[S][w]:null;if(E)return Fe.makeGlue(E,u)},{node:p},f,m),i},A9=function e(t,r,n,a,i){a&&t.push(a);for(var o=0;o<t.length;o++){var s=t[o],l=kK(s);if(l){e(l.children,r,n,null,i);continue}var u=!s.hasClass("mspace");if(u){var d=r(s,n.node);d&&(n.insertAfter?n.insertAfter(d):(t.unshift(d),o++))}u?n.node=s:i&&s.hasClass("newline")&&(n.node=fl(["leftmost"])),n.insertAfter=(p=>f=>{t.splice(p+1,0,f),o++})(o)}a&&t.pop()},kK=function(t){return t instanceof ug||t instanceof sD||t instanceof cg&&t.hasClass("enclosing")?t:null},hfe=function e(t,r){var n=kK(t);if(n){var a=n.children;if(a.length){if(r==="right")return e(a[a.length-1],"right");if(r==="left")return e(a[0],"left")}}return t},C_=function(t,r){return t?(r&&(t=hfe(t,r)),dfe[t.classes[0]]||null):null},Im=function(t,r){var n=["nulldelimiter"].concat(t.baseSizingClasses());return fl(r.concat(n))},Ir=function(t,r,n){if(!t)return fl();if(ny[t.type]){var a=ny[t.type](t,r);if(n&&r.size!==n.size){a=fl(r.sizingClasses(n),[a],r);var i=r.sizeMultiplier/n.sizeMultiplier;a.height*=i,a.depth*=i}return a}else throw new nt("Got group of unknown type: '"+t.type+"'")};function gb(e,t){var r=fl(["base"],e,t),n=fl(["strut"]);return n.style.height=ct(r.height+r.depth),r.depth&&(n.style.verticalAlign=ct(-r.depth)),r.children.unshift(n),r}function T_(e,t){var r=null;e.length===1&&e[0].type==="tag"&&(r=e[0].tag,e=e[0].body);var n=ta(e,t,"root"),a;n.length===2&&n[1].hasClass("tag")&&(a=n.pop());for(var i=[],o=[],s=0;s<n.length;s++)if(o.push(n[s]),n[s].hasClass("mbin")||n[s].hasClass("mrel")||n[s].hasClass("allowbreak")){for(var l=!1;s<n.length-1&&n[s+1].hasClass("mspace")&&!n[s+1].hasClass("newline");)s++,o.push(n[s]),n[s].hasClass("nobreak")&&(l=!0);l||(i.push(gb(o,t)),o=[])}else n[s].hasClass("newline")&&(o.pop(),o.length>0&&(i.push(gb(o,t)),o=[]),i.push(n[s]));o.length>0&&i.push(gb(o,t));var u;r?(u=gb(ta(r,t,!0)),u.classes=["tag"],i.push(u)):a&&i.push(a);var d=fl(["katex-html"],i);if(d.setAttribute("aria-hidden","true"),u){var p=u.children[0];p.style.height=ct(d.height+d.depth),d.depth&&(p.style.verticalAlign=ct(-d.depth))}return d}function CK(e){return new ug(e)}class Mi{constructor(t,r,n){this.type=void 0,this.attributes=void 0,this.children=void 0,this.classes=void 0,this.type=t,this.attributes={},this.children=r||[],this.classes=n||[]}setAttribute(t,r){this.attributes[t]=r}getAttribute(t){return this.attributes[t]}toNode(){var t=document.createElementNS("http://www.w3.org/1998/Math/MathML",this.type);for(var r in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,r)&&t.setAttribute(r,this.attributes[r]);this.classes.length>0&&(t.className=Cu(this.classes));for(var n=0;n<this.children.length;n++)if(this.children[n]instanceof is&&this.children[n+1]instanceof is){for(var a=this.children[n].toText()+this.children[++n].toText();this.children[n+1]instanceof is;)a+=this.children[++n].toText();t.appendChild(new is(a).toNode())}else t.appendChild(this.children[n].toNode());return t}toMarkup(){var t="<"+this.type;for(var r in this.attributes)Object.prototype.hasOwnProperty.call(this.attributes,r)&&(t+=" "+r+'="',t+=Gt.escape(this.attributes[r]),t+='"');this.classes.length>0&&(t+=' class ="'+Gt.escape(Cu(this.classes))+'"'),t+=">";for(var n=0;n<this.children.length;n++)t+=this.children[n].toMarkup();return t+="</"+this.type+">",t}toText(){return this.children.map(t=>t.toText()).join("")}}class is{constructor(t){this.text=void 0,this.text=t}toNode(){return document.createTextNode(this.text)}toMarkup(){return Gt.escape(this.toText())}toText(){return this.text}}class pfe{constructor(t){this.width=void 0,this.character=void 0,this.width=t,t>=.05555&&t<=.05556?this.character=" ":t>=.1666&&t<=.1667?this.character=" ":t>=.2222&&t<=.2223?this.character=" ":t>=.2777&&t<=.2778?this.character="  ":t>=-.05556&&t<=-.05555?this.character=" ⁣":t>=-.1667&&t<=-.1666?this.character=" ⁣":t>=-.2223&&t<=-.2222?this.character=" ⁣":t>=-.2778&&t<=-.2777?this.character=" ⁣":this.character=null}toNode(){if(this.character)return document.createTextNode(this.character);var t=document.createElementNS("http://www.w3.org/1998/Math/MathML","mspace");return t.setAttribute("width",ct(this.width)),t}toMarkup(){return this.character?"<mtext>"+this.character+"</mtext>":'<mspace width="'+ct(this.width)+'"/>'}toText(){return this.character?this.character:" "}}var tt={MathNode:Mi,TextNode:is,SpaceNode:pfe,newDocumentFragment:CK},so=function(t,r,n){return sn[r][t]&&sn[r][t].replace&&t.charCodeAt(0)!==55349&&!(yK.hasOwnProperty(t)&&n&&(n.fontFamily&&n.fontFamily.slice(4,6)==="tt"||n.font&&n.font.slice(4,6)==="tt"))&&(t=sn[r][t].replace),new tt.TextNode(t)},uD=function(t){return t.length===1?t[0]:new tt.MathNode("mrow",t)},cD=function(t,r){if(r.fontFamily==="texttt")return"monospace";if(r.fontFamily==="textsf")return r.fontShape==="textit"&&r.fontWeight==="textbf"?"sans-serif-bold-italic":r.fontShape==="textit"?"sans-serif-italic":r.fontWeight==="textbf"?"bold-sans-serif":"sans-serif";if(r.fontShape==="textit"&&r.fontWeight==="textbf")return"bold-italic";if(r.fontShape==="textit")return"italic";if(r.fontWeight==="textbf")return"bold";var n=r.font;if(!n||n==="mathnormal")return null;var a=t.mode;if(n==="mathit")return"italic";if(n==="boldsymbol")return t.type==="textord"?"bold":"bold-italic";if(n==="mathbf")return"bold";if(n==="mathbb")return"double-struck";if(n==="mathsfit")return"sans-serif-italic";if(n==="mathfrak")return"fraktur";if(n==="mathscr"||n==="mathcal")return"script";if(n==="mathsf")return"sans-serif";if(n==="mathtt")return"monospace";var i=t.text;if(Gt.contains(["\\imath","\\jmath"],i))return null;sn[a][i]&&sn[a][i].replace&&(i=sn[a][i].replace);var o=Fe.fontMap[n].fontName;return oD(i,o,a)?Fe.fontMap[n].variant:null};function Tx(e){if(!e)return!1;if(e.type==="mi"&&e.children.length===1){var t=e.children[0];return t instanceof is&&t.text==="."}else if(e.type==="mo"&&e.children.length===1&&e.getAttribute("separator")==="true"&&e.getAttribute("lspace")==="0em"&&e.getAttribute("rspace")==="0em"){var r=e.children[0];return r instanceof is&&r.text===","}else return!1}var mi=function(t,r,n){if(t.length===1){var a=tn(t[0],r);return n&&a instanceof Mi&&a.type==="mo"&&(a.setAttribute("lspace","0em"),a.setAttribute("rspace","0em")),[a]}for(var i=[],o,s=0;s<t.length;s++){var l=tn(t[s],r);if(l instanceof Mi&&o instanceof Mi){if(l.type==="mtext"&&o.type==="mtext"&&l.getAttribute("mathvariant")===o.getAttribute("mathvariant")){o.children.push(...l.children);continue}else if(l.type==="mn"&&o.type==="mn"){o.children.push(...l.children);continue}else if(Tx(l)&&o.type==="mn"){o.children.push(...l.children);continue}else if(l.type==="mn"&&Tx(o))l.children=[...o.children,...l.children],i.pop();else if((l.type==="msup"||l.type==="msub")&&l.children.length>=1&&(o.type==="mn"||Tx(o))){var u=l.children[0];u instanceof Mi&&u.type==="mn"&&(u.children=[...o.children,...u.children],i.pop())}else if(o.type==="mi"&&o.children.length===1){var d=o.children[0];if(d instanceof is&&d.text==="̸"&&(l.type==="mo"||l.type==="mi"||l.type==="mn")){var p=l.children[0];p instanceof is&&p.text.length>0&&(p.text=p.text.slice(0,1)+"̸"+p.text.slice(1),i.pop())}}}i.push(l),o=l}return i},Au=function(t,r,n){return uD(mi(t,r,n))},tn=function(t,r){if(!t)return new tt.MathNode("mrow");if(ay[t.type]){var n=ay[t.type](t,r);return n}else throw new nt("Got group of unknown type: '"+t.type+"'")};function F9(e,t,r,n,a){var i=mi(e,r),o;i.length===1&&i[0]instanceof Mi&&Gt.contains(["mrow","mtable"],i[0].type)?o=i[0]:o=new tt.MathNode("mrow",i);var s=new tt.MathNode("annotation",[new tt.TextNode(t)]);s.setAttribute("encoding","application/x-tex");var l=new tt.MathNode("semantics",[o,s]),u=new tt.MathNode("math",[l]);u.setAttribute("xmlns","http://www.w3.org/1998/Math/MathML"),n&&u.setAttribute("display","block");var d=a?"katex":"katex-mathml";return Fe.makeSpan([d],[u])}var TK=function(t){return new nl({style:t.displayMode?Vt.DISPLAY:Vt.TEXT,maxSize:t.maxSize,minRuleThickness:t.minRuleThickness})},AK=function(t,r){if(r.displayMode){var n=["katex-display"];r.leqno&&n.push("leqno"),r.fleqn&&n.push("fleqn"),t=Fe.makeSpan(n,[t])}return t},ffe=function(t,r,n){var a=TK(n),i;if(n.output==="mathml")return F9(t,r,a,n.displayMode,!0);if(n.output==="html"){var o=T_(t,a);i=Fe.makeSpan(["katex"],[o])}else{var s=F9(t,r,a,n.displayMode,!1),l=T_(t,a);i=Fe.makeSpan(["katex"],[s,l])}return AK(i,n)},mfe=function(t,r,n){var a=TK(n),i=T_(t,a),o=Fe.makeSpan(["katex"],[i]);return AK(o,n)},gfe={widehat:"^",widecheck:"ˇ",widetilde:"~",utilde:"~",overleftarrow:"←",underleftarrow:"←",xleftarrow:"←",overrightarrow:"→",underrightarrow:"→",xrightarrow:"→",underbrace:"⏟",overbrace:"⏞",overgroup:"⏠",undergroup:"⏡",overleftrightarrow:"↔",underleftrightarrow:"↔",xleftrightarrow:"↔",Overrightarrow:"⇒",xRightarrow:"⇒",overleftharpoon:"↼",xleftharpoonup:"↼",overrightharpoon:"⇀",xrightharpoonup:"⇀",xLeftarrow:"⇐",xLeftrightarrow:"⇔",xhookleftarrow:"↩",xhookrightarrow:"↪",xmapsto:"↦",xrightharpoondown:"⇁",xleftharpoondown:"↽",xrightleftharpoons:"⇌",xleftrightharpoons:"⇋",xtwoheadleftarrow:"↞",xtwoheadrightarrow:"↠",xlongequal:"=",xtofrom:"⇄",xrightleftarrows:"⇄",xrightequilibrium:"⇌",xleftequilibrium:"⇋","\\cdrightarrow":"→","\\cdleftarrow":"←","\\cdlongequal":"="},bfe=function(t){var r=new tt.MathNode("mo",[new tt.TextNode(gfe[t.replace(/^\\/,"")])]);return r.setAttribute("stretchy","true"),r},yfe={overrightarrow:[["rightarrow"],.888,522,"xMaxYMin"],overleftarrow:[["leftarrow"],.888,522,"xMinYMin"],underrightarrow:[["rightarrow"],.888,522,"xMaxYMin"],underleftarrow:[["leftarrow"],.888,522,"xMinYMin"],xrightarrow:[["rightarrow"],1.469,522,"xMaxYMin"],"\\cdrightarrow":[["rightarrow"],3,522,"xMaxYMin"],xleftarrow:[["leftarrow"],1.469,522,"xMinYMin"],"\\cdleftarrow":[["leftarrow"],3,522,"xMinYMin"],Overrightarrow:[["doublerightarrow"],.888,560,"xMaxYMin"],xRightarrow:[["doublerightarrow"],1.526,560,"xMaxYMin"],xLeftarrow:[["doubleleftarrow"],1.526,560,"xMinYMin"],overleftharpoon:[["leftharpoon"],.888,522,"xMinYMin"],xleftharpoonup:[["leftharpoon"],.888,522,"xMinYMin"],xleftharpoondown:[["leftharpoondown"],.888,522,"xMinYMin"],overrightharpoon:[["rightharpoon"],.888,522,"xMaxYMin"],xrightharpoonup:[["rightharpoon"],.888,522,"xMaxYMin"],xrightharpoondown:[["rightharpoondown"],.888,522,"xMaxYMin"],xlongequal:[["longequal"],.888,334,"xMinYMin"],"\\cdlongequal":[["longequal"],3,334,"xMinYMin"],xtwoheadleftarrow:[["twoheadleftarrow"],.888,334,"xMinYMin"],xtwoheadrightarrow:[["twoheadrightarrow"],.888,334,"xMaxYMin"],overleftrightarrow:[["leftarrow","rightarrow"],.888,522],overbrace:[["leftbrace","midbrace","rightbrace"],1.6,548],underbrace:[["leftbraceunder","midbraceunder","rightbraceunder"],1.6,548],underleftrightarrow:[["leftarrow","rightarrow"],.888,522],xleftrightarrow:[["leftarrow","rightarrow"],1.75,522],xLeftrightarrow:[["doubleleftarrow","doublerightarrow"],1.75,560],xrightleftharpoons:[["leftharpoondownplus","rightharpoonplus"],1.75,716],xleftrightharpoons:[["leftharpoonplus","rightharpoondownplus"],1.75,716],xhookleftarrow:[["leftarrow","righthook"],1.08,522],xhookrightarrow:[["lefthook","rightarrow"],1.08,522],overlinesegment:[["leftlinesegment","rightlinesegment"],.888,522],underlinesegment:[["leftlinesegment","rightlinesegment"],.888,522],overgroup:[["leftgroup","rightgroup"],.888,342],undergroup:[["leftgroupunder","rightgroupunder"],.888,342],xmapsto:[["leftmapsto","rightarrow"],1.5,522],xtofrom:[["leftToFrom","rightToFrom"],1.75,528],xrightleftarrows:[["baraboveleftarrow","rightarrowabovebar"],1.75,901],xrightequilibrium:[["baraboveshortleftharpoon","rightharpoonaboveshortbar"],1.75,716],xleftequilibrium:[["shortbaraboveleftharpoon","shortrightharpoonabovebar"],1.75,716]},vfe=function(t){return t.type==="ordgroup"?t.body.length:1},Sfe=function(t,r){function n(){var s=4e5,l=t.label.slice(1);if(Gt.contains(["widehat","widecheck","widetilde","utilde"],l)){var u=t,d=vfe(u.base),p,f,m;if(d>5)l==="widehat"||l==="widecheck"?(p=420,s=2364,m=.42,f=l+"4"):(p=312,s=2340,m=.34,f="tilde4");else{var b=[1,1,2,2,3,3][d];l==="widehat"||l==="widecheck"?(s=[0,1062,2364,2364,2364][b],p=[0,239,300,360,420][b],m=[0,.24,.3,.3,.36,.42][b],f=l+b):(s=[0,600,1033,2339,2340][b],p=[0,260,286,306,312][b],m=[0,.26,.286,.3,.306,.34][b],f="tilde"+b)}var y=new Tu(f),S=new pl([y],{width:"100%",height:ct(m),viewBox:"0 0 "+s+" "+p,preserveAspectRatio:"none"});return{span:Fe.makeSvgSpan([],[S],r),minWidth:0,height:m}}else{var w=[],E=yfe[l],[k,_,R]=E,C=R/1e3,F=k.length,L,I;if(F===1){var M=E[3];L=["hide-tail"],I=[M]}else if(F===2)L=["halfarrow-left","halfarrow-right"],I=["xMinYMin","xMaxYMin"];else if(F===3)L=["brace-left","brace-center","brace-right"],I=["xMinYMin","xMidYMin","xMaxYMin"];else throw new Error(`Correct katexImagesData or update code here to support
                    `+F+" children.");for(var j=0;j<F;j++){var Y=new Tu(k[j]),W=new pl([Y],{width:"400em",height:ct(C),viewBox:"0 0 "+s+" "+R,preserveAspectRatio:I[j]+" slice"}),$=Fe.makeSvgSpan([L[j]],[W],r);if(F===1)return{span:$,minWidth:_,height:C};$.style.height=ct(C),w.push($)}return{span:Fe.makeSpan(["stretchy"],w,r),minWidth:_,height:C}}}var{span:a,minWidth:i,height:o}=n();return a.height=o,a.style.height=ct(o),i>0&&(a.style.minWidth=ct(i)),a},Efe=function(t,r,n,a,i){var o,s=t.height+t.depth+n+a;if(/fbox|color|angl/.test(r)){if(o=Fe.makeSpan(["stretchy",r],[],i),r==="fbox"){var l=i.color&&i.getColor();l&&(o.style.borderColor=l)}}else{var u=[];/^[bx]cancel$/.test(r)&&u.push(new w_({x1:"0",y1:"0",x2:"100%",y2:"100%","stroke-width":"0.046em"})),/^x?cancel$/.test(r)&&u.push(new w_({x1:"0",y1:"100%",x2:"100%",y2:"0","stroke-width":"0.046em"}));var d=new pl(u,{width:"100%",height:ct(s)});o=Fe.makeSvgSpan([],[d],i)}return o.height=s,o.style.height=ct(s),o},ml={encloseSpan:Efe,mathMLnode:bfe,svgSpan:Sfe};function br(e,t){if(!e||e.type!==t)throw new Error("Expected node of type "+t+", but got "+(e?"node of type "+e.type:String(e)));return e}function dD(e){var t=wv(e);if(!t)throw new Error("Expected node of symbol group type, but got "+(e?"node of type "+e.type:String(e)));return t}function wv(e){return e&&(e.type==="atom"||Wpe.hasOwnProperty(e.type))?e:null}var hD=(e,t)=>{var r,n,a;e&&e.type==="supsub"?(n=br(e.base,"accent"),r=n.base,e.base=r,a=jpe(Ir(e,t)),e.base=n):(n=br(e,"accent"),r=n.base);var i=Ir(r,t.havingCrampedStyle()),o=n.isShifty&&Gt.isCharacterBox(r),s=0;if(o){var l=Gt.getBaseElem(r),u=Ir(l,t.havingCrampedStyle());s=E9(u).skew}var d=n.label==="\\c",p=d?i.height+i.depth:Math.min(i.height,t.fontMetrics().xHeight),f;if(n.isStretchy)f=ml.svgSpan(n,t),f=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:i},{type:"elem",elem:f,wrapperClasses:["svg-align"],wrapperStyle:s>0?{width:"calc(100% - "+ct(2*s)+")",marginLeft:ct(2*s)}:void 0}]},t);else{var m,b;n.label==="\\vec"?(m=Fe.staticSvg("vec",t),b=Fe.svgData.vec[1]):(m=Fe.makeOrd({mode:n.mode,text:n.label},t,"textord"),m=E9(m),m.italic=0,b=m.width,d&&(p+=m.depth)),f=Fe.makeSpan(["accent-body"],[m]);var y=n.label==="\\textcircled";y&&(f.classes.push("accent-full"),p=i.height);var S=s;y||(S-=b/2),f.style.left=ct(S),n.label==="\\textcircled"&&(f.style.top=".2em"),f=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:i},{type:"kern",size:-p},{type:"elem",elem:f}]},t)}var w=Fe.makeSpan(["mord","accent"],[f],t);return a?(a.children[0]=w,a.height=Math.max(w.height,a.height),a.classes[0]="mord",a):w},FK=(e,t)=>{var r=e.isStretchy?ml.mathMLnode(e.label):new tt.MathNode("mo",[so(e.label,e.mode)]),n=new tt.MathNode("mover",[tn(e.base,t),r]);return n.setAttribute("accent","true"),n},xfe=new RegExp(["\\acute","\\grave","\\ddot","\\tilde","\\bar","\\breve","\\check","\\hat","\\vec","\\dot","\\mathring"].map(e=>"\\"+e).join("|"));gt({type:"accent",names:["\\acute","\\grave","\\ddot","\\tilde","\\bar","\\breve","\\check","\\hat","\\vec","\\dot","\\mathring","\\widecheck","\\widehat","\\widetilde","\\overrightarrow","\\overleftarrow","\\Overrightarrow","\\overleftrightarrow","\\overgroup","\\overlinesegment","\\overleftharpoon","\\overrightharpoon"],props:{numArgs:1},handler:(e,t)=>{var r=iy(t[0]),n=!xfe.test(e.funcName),a=!n||e.funcName==="\\widehat"||e.funcName==="\\widetilde"||e.funcName==="\\widecheck";return{type:"accent",mode:e.parser.mode,label:e.funcName,isStretchy:n,isShifty:a,base:r}},htmlBuilder:hD,mathmlBuilder:FK});gt({type:"accent",names:["\\'","\\`","\\^","\\~","\\=","\\u","\\.",'\\"',"\\c","\\r","\\H","\\v","\\textcircled"],props:{numArgs:1,allowedInText:!0,allowedInMath:!0,argTypes:["primitive"]},handler:(e,t)=>{var r=t[0],n=e.parser.mode;return n==="math"&&(e.parser.settings.reportNonstrict("mathVsTextAccents","LaTeX's accent "+e.funcName+" works only in text mode"),n="text"),{type:"accent",mode:n,label:e.funcName,isStretchy:!1,isShifty:!0,base:r}},htmlBuilder:hD,mathmlBuilder:FK});gt({type:"accentUnder",names:["\\underleftarrow","\\underrightarrow","\\underleftrightarrow","\\undergroup","\\underlinesegment","\\utilde"],props:{numArgs:1},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=t[0];return{type:"accentUnder",mode:r.mode,label:n,base:a}},htmlBuilder:(e,t)=>{var r=Ir(e.base,t),n=ml.svgSpan(e,t),a=e.label==="\\utilde"?.12:0,i=Fe.makeVList({positionType:"top",positionData:r.height,children:[{type:"elem",elem:n,wrapperClasses:["svg-align"]},{type:"kern",size:a},{type:"elem",elem:r}]},t);return Fe.makeSpan(["mord","accentunder"],[i],t)},mathmlBuilder:(e,t)=>{var r=ml.mathMLnode(e.label),n=new tt.MathNode("munder",[tn(e.base,t),r]);return n.setAttribute("accentunder","true"),n}});var bb=e=>{var t=new tt.MathNode("mpadded",e?[e]:[]);return t.setAttribute("width","+0.6em"),t.setAttribute("lspace","0.3em"),t};gt({type:"xArrow",names:["\\xleftarrow","\\xrightarrow","\\xLeftarrow","\\xRightarrow","\\xleftrightarrow","\\xLeftrightarrow","\\xhookleftarrow","\\xhookrightarrow","\\xmapsto","\\xrightharpoondown","\\xrightharpoonup","\\xleftharpoondown","\\xleftharpoonup","\\xrightleftharpoons","\\xleftrightharpoons","\\xlongequal","\\xtwoheadrightarrow","\\xtwoheadleftarrow","\\xtofrom","\\xrightleftarrows","\\xrightequilibrium","\\xleftequilibrium","\\\\cdrightarrow","\\\\cdleftarrow","\\\\cdlongequal"],props:{numArgs:1,numOptionalArgs:1},handler(e,t,r){var{parser:n,funcName:a}=e;return{type:"xArrow",mode:n.mode,label:a,body:t[0],below:r[0]}},htmlBuilder(e,t){var r=t.style,n=t.havingStyle(r.sup()),a=Fe.wrapFragment(Ir(e.body,n,t),t),i=e.label.slice(0,2)==="\\x"?"x":"cd";a.classes.push(i+"-arrow-pad");var o;e.below&&(n=t.havingStyle(r.sub()),o=Fe.wrapFragment(Ir(e.below,n,t),t),o.classes.push(i+"-arrow-pad"));var s=ml.svgSpan(e,t),l=-t.fontMetrics().axisHeight+.5*s.height,u=-t.fontMetrics().axisHeight-.5*s.height-.111;(a.depth>.25||e.label==="\\xleftequilibrium")&&(u-=a.depth);var d;if(o){var p=-t.fontMetrics().axisHeight+o.height+.5*s.height+.111;d=Fe.makeVList({positionType:"individualShift",children:[{type:"elem",elem:a,shift:u},{type:"elem",elem:s,shift:l},{type:"elem",elem:o,shift:p}]},t)}else d=Fe.makeVList({positionType:"individualShift",children:[{type:"elem",elem:a,shift:u},{type:"elem",elem:s,shift:l}]},t);return d.children[0].children[0].children[1].classes.push("svg-align"),Fe.makeSpan(["mrel","x-arrow"],[d],t)},mathmlBuilder(e,t){var r=ml.mathMLnode(e.label);r.setAttribute("minsize",e.label.charAt(0)==="x"?"1.75em":"3.0em");var n;if(e.body){var a=bb(tn(e.body,t));if(e.below){var i=bb(tn(e.below,t));n=new tt.MathNode("munderover",[r,i,a])}else n=new tt.MathNode("mover",[r,a])}else if(e.below){var o=bb(tn(e.below,t));n=new tt.MathNode("munder",[r,o])}else n=bb(),n=new tt.MathNode("mover",[r,n]);return n}});var wfe=Fe.makeSpan;function _K(e,t){var r=ta(e.body,t,!0);return wfe([e.mclass],r,t)}function RK(e,t){var r,n=mi(e.body,t);return e.mclass==="minner"?r=new tt.MathNode("mpadded",n):e.mclass==="mord"?e.isCharacterBox?(r=n[0],r.type="mi"):r=new tt.MathNode("mi",n):(e.isCharacterBox?(r=n[0],r.type="mo"):r=new tt.MathNode("mo",n),e.mclass==="mbin"?(r.attributes.lspace="0.22em",r.attributes.rspace="0.22em"):e.mclass==="mpunct"?(r.attributes.lspace="0em",r.attributes.rspace="0.17em"):e.mclass==="mopen"||e.mclass==="mclose"?(r.attributes.lspace="0em",r.attributes.rspace="0em"):e.mclass==="minner"&&(r.attributes.lspace="0.0556em",r.attributes.width="+0.1111em")),r}gt({type:"mclass",names:["\\mathord","\\mathbin","\\mathrel","\\mathopen","\\mathclose","\\mathpunct","\\mathinner"],props:{numArgs:1,primitive:!0},handler(e,t){var{parser:r,funcName:n}=e,a=t[0];return{type:"mclass",mode:r.mode,mclass:"m"+n.slice(5),body:Ln(a),isCharacterBox:Gt.isCharacterBox(a)}},htmlBuilder:_K,mathmlBuilder:RK});var kv=e=>{var t=e.type==="ordgroup"&&e.body.length?e.body[0]:e;return t.type==="atom"&&(t.family==="bin"||t.family==="rel")?"m"+t.family:"mord"};gt({type:"mclass",names:["\\@binrel"],props:{numArgs:2},handler(e,t){var{parser:r}=e;return{type:"mclass",mode:r.mode,mclass:kv(t[0]),body:Ln(t[1]),isCharacterBox:Gt.isCharacterBox(t[1])}}});gt({type:"mclass",names:["\\stackrel","\\overset","\\underset"],props:{numArgs:2},handler(e,t){var{parser:r,funcName:n}=e,a=t[1],i=t[0],o;n!=="\\stackrel"?o=kv(a):o="mrel";var s={type:"op",mode:a.mode,limits:!0,alwaysHandleSupSub:!0,parentIsSupSub:!1,symbol:!1,suppressBaseShift:n!=="\\stackrel",body:Ln(a)},l={type:"supsub",mode:i.mode,base:s,sup:n==="\\underset"?null:i,sub:n==="\\underset"?i:null};return{type:"mclass",mode:r.mode,mclass:o,body:[l],isCharacterBox:Gt.isCharacterBox(l)}},htmlBuilder:_K,mathmlBuilder:RK});gt({type:"pmb",names:["\\pmb"],props:{numArgs:1,allowedInText:!0},handler(e,t){var{parser:r}=e;return{type:"pmb",mode:r.mode,mclass:kv(t[0]),body:Ln(t[0])}},htmlBuilder(e,t){var r=ta(e.body,t,!0),n=Fe.makeSpan([e.mclass],r,t);return n.style.textShadow="0.02em 0.01em 0.04px",n},mathmlBuilder(e,t){var r=mi(e.body,t),n=new tt.MathNode("mstyle",r);return n.setAttribute("style","text-shadow: 0.02em 0.01em 0.04px"),n}});var kfe={">":"\\\\cdrightarrow","<":"\\\\cdleftarrow","=":"\\\\cdlongequal",A:"\\uparrow",V:"\\downarrow","|":"\\Vert",".":"no arrow"},_9=()=>({type:"styling",body:[],mode:"math",style:"display"}),R9=e=>e.type==="textord"&&e.text==="@",Cfe=(e,t)=>(e.type==="mathord"||e.type==="atom")&&e.text===t;function Tfe(e,t,r){var n=kfe[e];switch(n){case"\\\\cdrightarrow":case"\\\\cdleftarrow":return r.callFunction(n,[t[0]],[t[1]]);case"\\uparrow":case"\\downarrow":{var a=r.callFunction("\\\\cdleft",[t[0]],[]),i={type:"atom",text:n,mode:"math",family:"rel"},o=r.callFunction("\\Big",[i],[]),s=r.callFunction("\\\\cdright",[t[1]],[]),l={type:"ordgroup",mode:"math",body:[a,o,s]};return r.callFunction("\\\\cdparent",[l],[])}case"\\\\cdlongequal":return r.callFunction("\\\\cdlongequal",[],[]);case"\\Vert":{var u={type:"textord",text:"\\Vert",mode:"math"};return r.callFunction("\\Big",[u],[])}default:return{type:"textord",text:" ",mode:"math"}}}function Afe(e){var t=[];for(e.gullet.beginGroup(),e.gullet.macros.set("\\cr","\\\\\\relax"),e.gullet.beginGroup();;){t.push(e.parseExpression(!1,"\\\\")),e.gullet.endGroup(),e.gullet.beginGroup();var r=e.fetch().text;if(r==="&"||r==="\\\\")e.consume();else if(r==="\\end"){t[t.length-1].length===0&&t.pop();break}else throw new nt("Expected \\\\ or \\cr or \\end",e.nextToken)}for(var n=[],a=[n],i=0;i<t.length;i++){for(var o=t[i],s=_9(),l=0;l<o.length;l++)if(!R9(o[l]))s.body.push(o[l]);else{n.push(s),l+=1;var u=dD(o[l]).text,d=new Array(2);if(d[0]={type:"ordgroup",mode:"math",body:[]},d[1]={type:"ordgroup",mode:"math",body:[]},!("=|.".indexOf(u)>-1))if("<>AV".indexOf(u)>-1)for(var p=0;p<2;p++){for(var f=!0,m=l+1;m<o.length;m++){if(Cfe(o[m],u)){f=!1,l=m;break}if(R9(o[m]))throw new nt("Missing a "+u+" character to complete a CD arrow.",o[m]);d[p].body.push(o[m])}if(f)throw new nt("Missing a "+u+" character to complete a CD arrow.",o[l])}else throw new nt('Expected one of "<>AV=|." after @',o[l]);var b=Tfe(u,d,e),y={type:"styling",body:[b],mode:"math",style:"display"};n.push(y),s=_9()}i%2===0?n.push(s):n.shift(),n=[],a.push(n)}e.gullet.endGroup(),e.gullet.endGroup();var S=new Array(a[0].length).fill({type:"align",align:"c",pregap:.25,postgap:.25});return{type:"array",mode:"math",body:a,arraystretch:1,addJot:!0,rowGaps:[null],cols:S,colSeparationType:"CD",hLinesBeforeRow:new Array(a.length+1).fill([])}}gt({type:"cdlabel",names:["\\\\cdleft","\\\\cdright"],props:{numArgs:1},handler(e,t){var{parser:r,funcName:n}=e;return{type:"cdlabel",mode:r.mode,side:n.slice(4),label:t[0]}},htmlBuilder(e,t){var r=t.havingStyle(t.style.sup()),n=Fe.wrapFragment(Ir(e.label,r,t),t);return n.classes.push("cd-label-"+e.side),n.style.bottom=ct(.8-n.depth),n.height=0,n.depth=0,n},mathmlBuilder(e,t){var r=new tt.MathNode("mrow",[tn(e.label,t)]);return r=new tt.MathNode("mpadded",[r]),r.setAttribute("width","0"),e.side==="left"&&r.setAttribute("lspace","-1width"),r.setAttribute("voffset","0.7em"),r=new tt.MathNode("mstyle",[r]),r.setAttribute("displaystyle","false"),r.setAttribute("scriptlevel","1"),r}});gt({type:"cdlabelparent",names:["\\\\cdparent"],props:{numArgs:1},handler(e,t){var{parser:r}=e;return{type:"cdlabelparent",mode:r.mode,fragment:t[0]}},htmlBuilder(e,t){var r=Fe.wrapFragment(Ir(e.fragment,t),t);return r.classes.push("cd-vert-arrow"),r},mathmlBuilder(e,t){return new tt.MathNode("mrow",[tn(e.fragment,t)])}});gt({type:"textord",names:["\\@char"],props:{numArgs:1,allowedInText:!0},handler(e,t){for(var{parser:r}=e,n=br(t[0],"ordgroup"),a=n.body,i="",o=0;o<a.length;o++){var s=br(a[o],"textord");i+=s.text}var l=parseInt(i),u;if(isNaN(l))throw new nt("\\@char has non-numeric argument "+i);if(l<0||l>=1114111)throw new nt("\\@char with invalid code point "+i);return l<=65535?u=String.fromCharCode(l):(l-=65536,u=String.fromCharCode((l>>10)+55296,(l&1023)+56320)),{type:"textord",mode:r.mode,text:u}}});var DK=(e,t)=>{var r=ta(e.body,t.withColor(e.color),!1);return Fe.makeFragment(r)},NK=(e,t)=>{var r=mi(e.body,t.withColor(e.color)),n=new tt.MathNode("mstyle",r);return n.setAttribute("mathcolor",e.color),n};gt({type:"color",names:["\\textcolor"],props:{numArgs:2,allowedInText:!0,argTypes:["color","original"]},handler(e,t){var{parser:r}=e,n=br(t[0],"color-token").color,a=t[1];return{type:"color",mode:r.mode,color:n,body:Ln(a)}},htmlBuilder:DK,mathmlBuilder:NK});gt({type:"color",names:["\\color"],props:{numArgs:1,allowedInText:!0,argTypes:["color"]},handler(e,t){var{parser:r,breakOnTokenText:n}=e,a=br(t[0],"color-token").color;r.gullet.macros.set("\\current@color",a);var i=r.parseExpression(!0,n);return{type:"color",mode:r.mode,color:a,body:i}},htmlBuilder:DK,mathmlBuilder:NK});gt({type:"cr",names:["\\\\"],props:{numArgs:0,numOptionalArgs:0,allowedInText:!0},handler(e,t,r){var{parser:n}=e,a=n.gullet.future().text==="["?n.parseSizeGroup(!0):null,i=!n.settings.displayMode||!n.settings.useStrictBehavior("newLineInDisplayMode","In LaTeX, \\\\ or \\newline does nothing in display mode");return{type:"cr",mode:n.mode,newLine:i,size:a&&br(a,"size").value}},htmlBuilder(e,t){var r=Fe.makeSpan(["mspace"],[],t);return e.newLine&&(r.classes.push("newline"),e.size&&(r.style.marginTop=ct(xn(e.size,t)))),r},mathmlBuilder(e,t){var r=new tt.MathNode("mspace");return e.newLine&&(r.setAttribute("linebreak","newline"),e.size&&r.setAttribute("height",ct(xn(e.size,t)))),r}});var A_={"\\global":"\\global","\\long":"\\\\globallong","\\\\globallong":"\\\\globallong","\\def":"\\gdef","\\gdef":"\\gdef","\\edef":"\\xdef","\\xdef":"\\xdef","\\let":"\\\\globallet","\\futurelet":"\\\\globalfuture"},LK=e=>{var t=e.text;if(/^(?:[\\{}$&#^_]|EOF)$/.test(t))throw new nt("Expected a control sequence",e);return t},Ffe=e=>{var t=e.gullet.popToken();return t.text==="="&&(t=e.gullet.popToken(),t.text===" "&&(t=e.gullet.popToken())),t},MK=(e,t,r,n)=>{var a=e.gullet.macros.get(r.text);a==null&&(r.noexpand=!0,a={tokens:[r],numArgs:0,unexpandable:!e.gullet.isExpandable(r.text)}),e.gullet.macros.set(t,a,n)};gt({type:"internal",names:["\\global","\\long","\\\\globallong"],props:{numArgs:0,allowedInText:!0},handler(e){var{parser:t,funcName:r}=e;t.consumeSpaces();var n=t.fetch();if(A_[n.text])return(r==="\\global"||r==="\\\\globallong")&&(n.text=A_[n.text]),br(t.parseFunction(),"internal");throw new nt("Invalid token after macro prefix",n)}});gt({type:"internal",names:["\\def","\\gdef","\\edef","\\xdef"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e){var{parser:t,funcName:r}=e,n=t.gullet.popToken(),a=n.text;if(/^(?:[\\{}$&#^_]|EOF)$/.test(a))throw new nt("Expected a control sequence",n);for(var i=0,o,s=[[]];t.gullet.future().text!=="{";)if(n=t.gullet.popToken(),n.text==="#"){if(t.gullet.future().text==="{"){o=t.gullet.future(),s[i].push("{");break}if(n=t.gullet.popToken(),!/^[1-9]$/.test(n.text))throw new nt('Invalid argument number "'+n.text+'"');if(parseInt(n.text)!==i+1)throw new nt('Argument number "'+n.text+'" out of order');i++,s.push([])}else{if(n.text==="EOF")throw new nt("Expected a macro definition");s[i].push(n.text)}var{tokens:l}=t.gullet.consumeArg();return o&&l.unshift(o),(r==="\\edef"||r==="\\xdef")&&(l=t.gullet.expandTokens(l),l.reverse()),t.gullet.macros.set(a,{tokens:l,numArgs:i,delimiters:s},r===A_[r]),{type:"internal",mode:t.mode}}});gt({type:"internal",names:["\\let","\\\\globallet"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e){var{parser:t,funcName:r}=e,n=LK(t.gullet.popToken());t.gullet.consumeSpaces();var a=Ffe(t);return MK(t,n,a,r==="\\\\globallet"),{type:"internal",mode:t.mode}}});gt({type:"internal",names:["\\futurelet","\\\\globalfuture"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e){var{parser:t,funcName:r}=e,n=LK(t.gullet.popToken()),a=t.gullet.popToken(),i=t.gullet.popToken();return MK(t,n,i,r==="\\\\globalfuture"),t.gullet.pushToken(i),t.gullet.pushToken(a),{type:"internal",mode:t.mode}}});var rm=function(t,r,n){var a=sn.math[t]&&sn.math[t].replace,i=oD(a||t,r,n);if(!i)throw new Error("Unsupported symbol "+t+" and font size "+r+".");return i},pD=function(t,r,n,a){var i=n.havingBaseStyle(r),o=Fe.makeSpan(a.concat(i.sizingClasses(n)),[t],n),s=i.sizeMultiplier/n.sizeMultiplier;return o.height*=s,o.depth*=s,o.maxFontSize=i.sizeMultiplier,o},IK=function(t,r,n){var a=r.havingBaseStyle(n),i=(1-r.sizeMultiplier/a.sizeMultiplier)*r.fontMetrics().axisHeight;t.classes.push("delimcenter"),t.style.top=ct(i),t.height-=i,t.depth+=i},_fe=function(t,r,n,a,i,o){var s=Fe.makeSymbol(t,"Main-Regular",i,a),l=pD(s,r,a,o);return n&&IK(l,a,r),l},Rfe=function(t,r,n,a){return Fe.makeSymbol(t,"Size"+r+"-Regular",n,a)},OK=function(t,r,n,a,i,o){var s=Rfe(t,r,i,a),l=pD(Fe.makeSpan(["delimsizing","size"+r],[s],a),Vt.TEXT,a,o);return n&&IK(l,a,Vt.TEXT),l},Ax=function(t,r,n){var a;r==="Size1-Regular"?a="delim-size1":a="delim-size4";var i=Fe.makeSpan(["delimsizinginner",a],[Fe.makeSpan([],[Fe.makeSymbol(t,r,n)])]);return{type:"elem",elem:i}},Fx=function(t,r,n){var a=as["Size4-Regular"][t.charCodeAt(0)]?as["Size4-Regular"][t.charCodeAt(0)][4]:as["Size1-Regular"][t.charCodeAt(0)][4],i=new Tu("inner",Ope(t,Math.round(1e3*r))),o=new pl([i],{width:ct(a),height:ct(r),style:"width:"+ct(a),viewBox:"0 0 "+1e3*a+" "+Math.round(1e3*r),preserveAspectRatio:"xMinYMin"}),s=Fe.makeSvgSpan([],[o],n);return s.height=r,s.style.height=ct(r),s.style.width=ct(a),{type:"elem",elem:s}},F_=.008,yb={type:"kern",size:-1*F_},Dfe=["|","\\lvert","\\rvert","\\vert"],Nfe=["\\|","\\lVert","\\rVert","\\Vert"],BK=function(t,r,n,a,i,o){var s,l,u,d,p="",f=0;s=u=d=t,l=null;var m="Size1-Regular";t==="\\uparrow"?u=d="⏐":t==="\\Uparrow"?u=d="‖":t==="\\downarrow"?s=u="⏐":t==="\\Downarrow"?s=u="‖":t==="\\updownarrow"?(s="\\uparrow",u="⏐",d="\\downarrow"):t==="\\Updownarrow"?(s="\\Uparrow",u="‖",d="\\Downarrow"):Gt.contains(Dfe,t)?(u="∣",p="vert",f=333):Gt.contains(Nfe,t)?(u="∥",p="doublevert",f=556):t==="["||t==="\\lbrack"?(s="⎡",u="⎢",d="⎣",m="Size4-Regular",p="lbrack",f=667):t==="]"||t==="\\rbrack"?(s="⎤",u="⎥",d="⎦",m="Size4-Regular",p="rbrack",f=667):t==="\\lfloor"||t==="⌊"?(u=s="⎢",d="⎣",m="Size4-Regular",p="lfloor",f=667):t==="\\lceil"||t==="⌈"?(s="⎡",u=d="⎢",m="Size4-Regular",p="lceil",f=667):t==="\\rfloor"||t==="⌋"?(u=s="⎥",d="⎦",m="Size4-Regular",p="rfloor",f=667):t==="\\rceil"||t==="⌉"?(s="⎤",u=d="⎥",m="Size4-Regular",p="rceil",f=667):t==="("||t==="\\lparen"?(s="⎛",u="⎜",d="⎝",m="Size4-Regular",p="lparen",f=875):t===")"||t==="\\rparen"?(s="⎞",u="⎟",d="⎠",m="Size4-Regular",p="rparen",f=875):t==="\\{"||t==="\\lbrace"?(s="⎧",l="⎨",d="⎩",u="⎪",m="Size4-Regular"):t==="\\}"||t==="\\rbrace"?(s="⎫",l="⎬",d="⎭",u="⎪",m="Size4-Regular"):t==="\\lgroup"||t==="⟮"?(s="⎧",d="⎩",u="⎪",m="Size4-Regular"):t==="\\rgroup"||t==="⟯"?(s="⎫",d="⎭",u="⎪",m="Size4-Regular"):t==="\\lmoustache"||t==="⎰"?(s="⎧",d="⎭",u="⎪",m="Size4-Regular"):(t==="\\rmoustache"||t==="⎱")&&(s="⎫",d="⎩",u="⎪",m="Size4-Regular");var b=rm(s,m,i),y=b.height+b.depth,S=rm(u,m,i),w=S.height+S.depth,E=rm(d,m,i),k=E.height+E.depth,_=0,R=1;if(l!==null){var C=rm(l,m,i);_=C.height+C.depth,R=2}var F=y+k+_,L=Math.max(0,Math.ceil((r-F)/(R*w))),I=F+L*R*w,M=a.fontMetrics().axisHeight;n&&(M*=a.sizeMultiplier);var j=I/2-M,Y=[];if(p.length>0){var W=I-y-k,$=Math.round(I*1e3),X=Bpe(p,Math.round(W*1e3)),z=new Tu(p,X),H=(f/1e3).toFixed(3)+"em",B=($/1e3).toFixed(3)+"em",ee=new pl([z],{width:H,height:B,viewBox:"0 0 "+f+" "+$}),J=Fe.makeSvgSpan([],[ee],a);J.height=$/1e3,J.style.width=H,J.style.height=B,Y.push({type:"elem",elem:J})}else{if(Y.push(Ax(d,m,i)),Y.push(yb),l===null){var K=I-y-k+2*F_;Y.push(Fx(u,K,a))}else{var G=(I-y-k-_)/2+2*F_;Y.push(Fx(u,G,a)),Y.push(yb),Y.push(Ax(l,m,i)),Y.push(yb),Y.push(Fx(u,G,a))}Y.push(yb),Y.push(Ax(s,m,i))}var P=a.havingBaseStyle(Vt.TEXT),ne=Fe.makeVList({positionType:"bottom",positionData:j,children:Y},P);return pD(Fe.makeSpan(["delimsizing","mult"],[ne],P),Vt.TEXT,a,o)},_x=80,Rx=.08,Dx=function(t,r,n,a,i){var o=Ipe(t,a,n),s=new Tu(t,o),l=new pl([s],{width:"400em",height:ct(r),viewBox:"0 0 400000 "+n,preserveAspectRatio:"xMinYMin slice"});return Fe.makeSvgSpan(["hide-tail"],[l],i)},Lfe=function(t,r){var n=r.havingBaseSizing(),a=UK("\\surd",t*n.sizeMultiplier,HK,n),i=n.sizeMultiplier,o=Math.max(0,r.minRuleThickness-r.fontMetrics().sqrtRuleThickness),s,l=0,u=0,d=0,p;return a.type==="small"?(d=1e3+1e3*o+_x,t<1?i=1:t<1.4&&(i=.7),l=(1+o+Rx)/i,u=(1+o)/i,s=Dx("sqrtMain",l,d,o,r),s.style.minWidth="0.853em",p=.833/i):a.type==="large"?(d=(1e3+_x)*bm[a.size],u=(bm[a.size]+o)/i,l=(bm[a.size]+o+Rx)/i,s=Dx("sqrtSize"+a.size,l,d,o,r),s.style.minWidth="1.02em",p=1/i):(l=t+o+Rx,u=t+o,d=Math.floor(1e3*t+o)+_x,s=Dx("sqrtTall",l,d,o,r),s.style.minWidth="0.742em",p=1.056),s.height=u,s.style.height=ct(l),{span:s,advanceWidth:p,ruleWidth:(r.fontMetrics().sqrtRuleThickness+o)*i}},PK=["(","\\lparen",")","\\rparen","[","\\lbrack","]","\\rbrack","\\{","\\lbrace","\\}","\\rbrace","\\lfloor","\\rfloor","⌊","⌋","\\lceil","\\rceil","⌈","⌉","\\surd"],Mfe=["\\uparrow","\\downarrow","\\updownarrow","\\Uparrow","\\Downarrow","\\Updownarrow","|","\\|","\\vert","\\Vert","\\lvert","\\rvert","\\lVert","\\rVert","\\lgroup","\\rgroup","⟮","⟯","\\lmoustache","\\rmoustache","⎰","⎱"],zK=["<",">","\\langle","\\rangle","/","\\backslash","\\lt","\\gt"],bm=[0,1.2,1.8,2.4,3],Ife=function(t,r,n,a,i){if(t==="<"||t==="\\lt"||t==="⟨"?t="\\langle":(t===">"||t==="\\gt"||t==="⟩")&&(t="\\rangle"),Gt.contains(PK,t)||Gt.contains(zK,t))return OK(t,r,!1,n,a,i);if(Gt.contains(Mfe,t))return BK(t,bm[r],!1,n,a,i);throw new nt("Illegal delimiter: '"+t+"'")},Ofe=[{type:"small",style:Vt.SCRIPTSCRIPT},{type:"small",style:Vt.SCRIPT},{type:"small",style:Vt.TEXT},{type:"large",size:1},{type:"large",size:2},{type:"large",size:3},{type:"large",size:4}],Bfe=[{type:"small",style:Vt.SCRIPTSCRIPT},{type:"small",style:Vt.SCRIPT},{type:"small",style:Vt.TEXT},{type:"stack"}],HK=[{type:"small",style:Vt.SCRIPTSCRIPT},{type:"small",style:Vt.SCRIPT},{type:"small",style:Vt.TEXT},{type:"large",size:1},{type:"large",size:2},{type:"large",size:3},{type:"large",size:4},{type:"stack"}],Pfe=function(t){if(t.type==="small")return"Main-Regular";if(t.type==="large")return"Size"+t.size+"-Regular";if(t.type==="stack")return"Size4-Regular";throw new Error("Add support for delim type '"+t.type+"' here.")},UK=function(t,r,n,a){for(var i=Math.min(2,3-a.style.size),o=i;o<n.length&&n[o].type!=="stack";o++){var s=rm(t,Pfe(n[o]),"math"),l=s.height+s.depth;if(n[o].type==="small"){var u=a.havingBaseStyle(n[o].style);l*=u.sizeMultiplier}if(l>r)return n[o]}return n[n.length-1]},$K=function(t,r,n,a,i,o){t==="<"||t==="\\lt"||t==="⟨"?t="\\langle":(t===">"||t==="\\gt"||t==="⟩")&&(t="\\rangle");var s;Gt.contains(zK,t)?s=Ofe:Gt.contains(PK,t)?s=HK:s=Bfe;var l=UK(t,r,s,a);return l.type==="small"?_fe(t,l.style,n,a,i,o):l.type==="large"?OK(t,l.size,n,a,i,o):BK(t,r,n,a,i,o)},zfe=function(t,r,n,a,i,o){var s=a.fontMetrics().axisHeight*a.sizeMultiplier,l=901,u=5/a.fontMetrics().ptPerEm,d=Math.max(r-s,n+s),p=Math.max(d/500*l,2*d-u);return $K(t,p,!0,a,i,o)},ll={sqrtImage:Lfe,sizedDelim:Ife,sizeToMaxHeight:bm,customSizedDelim:$K,leftRightDelim:zfe},D9={"\\bigl":{mclass:"mopen",size:1},"\\Bigl":{mclass:"mopen",size:2},"\\biggl":{mclass:"mopen",size:3},"\\Biggl":{mclass:"mopen",size:4},"\\bigr":{mclass:"mclose",size:1},"\\Bigr":{mclass:"mclose",size:2},"\\biggr":{mclass:"mclose",size:3},"\\Biggr":{mclass:"mclose",size:4},"\\bigm":{mclass:"mrel",size:1},"\\Bigm":{mclass:"mrel",size:2},"\\biggm":{mclass:"mrel",size:3},"\\Biggm":{mclass:"mrel",size:4},"\\big":{mclass:"mord",size:1},"\\Big":{mclass:"mord",size:2},"\\bigg":{mclass:"mord",size:3},"\\Bigg":{mclass:"mord",size:4}},Hfe=["(","\\lparen",")","\\rparen","[","\\lbrack","]","\\rbrack","\\{","\\lbrace","\\}","\\rbrace","\\lfloor","\\rfloor","⌊","⌋","\\lceil","\\rceil","⌈","⌉","<",">","\\langle","⟨","\\rangle","⟩","\\lt","\\gt","\\lvert","\\rvert","\\lVert","\\rVert","\\lgroup","\\rgroup","⟮","⟯","\\lmoustache","\\rmoustache","⎰","⎱","/","\\backslash","|","\\vert","\\|","\\Vert","\\uparrow","\\Uparrow","\\downarrow","\\Downarrow","\\updownarrow","\\Updownarrow","."];function Cv(e,t){var r=wv(e);if(r&&Gt.contains(Hfe,r.text))return r;throw r?new nt("Invalid delimiter '"+r.text+"' after '"+t.funcName+"'",e):new nt("Invalid delimiter type '"+e.type+"'",e)}gt({type:"delimsizing",names:["\\bigl","\\Bigl","\\biggl","\\Biggl","\\bigr","\\Bigr","\\biggr","\\Biggr","\\bigm","\\Bigm","\\biggm","\\Biggm","\\big","\\Big","\\bigg","\\Bigg"],props:{numArgs:1,argTypes:["primitive"]},handler:(e,t)=>{var r=Cv(t[0],e);return{type:"delimsizing",mode:e.parser.mode,size:D9[e.funcName].size,mclass:D9[e.funcName].mclass,delim:r.text}},htmlBuilder:(e,t)=>e.delim==="."?Fe.makeSpan([e.mclass]):ll.sizedDelim(e.delim,e.size,t,e.mode,[e.mclass]),mathmlBuilder:e=>{var t=[];e.delim!=="."&&t.push(so(e.delim,e.mode));var r=new tt.MathNode("mo",t);e.mclass==="mopen"||e.mclass==="mclose"?r.setAttribute("fence","true"):r.setAttribute("fence","false"),r.setAttribute("stretchy","true");var n=ct(ll.sizeToMaxHeight[e.size]);return r.setAttribute("minsize",n),r.setAttribute("maxsize",n),r}});function N9(e){if(!e.body)throw new Error("Bug: The leftright ParseNode wasn't fully parsed.")}gt({type:"leftright-right",names:["\\right"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var r=e.parser.gullet.macros.get("\\current@color");if(r&&typeof r!="string")throw new nt("\\current@color set to non-string in \\right");return{type:"leftright-right",mode:e.parser.mode,delim:Cv(t[0],e).text,color:r}}});gt({type:"leftright",names:["\\left"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var r=Cv(t[0],e),n=e.parser;++n.leftrightDepth;var a=n.parseExpression(!1);--n.leftrightDepth,n.expect("\\right",!1);var i=br(n.parseFunction(),"leftright-right");return{type:"leftright",mode:n.mode,body:a,left:r.text,right:i.delim,rightColor:i.color}},htmlBuilder:(e,t)=>{N9(e);for(var r=ta(e.body,t,!0,["mopen","mclose"]),n=0,a=0,i=!1,o=0;o<r.length;o++)r[o].isMiddle?i=!0:(n=Math.max(r[o].height,n),a=Math.max(r[o].depth,a));n*=t.sizeMultiplier,a*=t.sizeMultiplier;var s;if(e.left==="."?s=Im(t,["mopen"]):s=ll.leftRightDelim(e.left,n,a,t,e.mode,["mopen"]),r.unshift(s),i)for(var l=1;l<r.length;l++){var u=r[l],d=u.isMiddle;d&&(r[l]=ll.leftRightDelim(d.delim,n,a,d.options,e.mode,[]))}var p;if(e.right===".")p=Im(t,["mclose"]);else{var f=e.rightColor?t.withColor(e.rightColor):t;p=ll.leftRightDelim(e.right,n,a,f,e.mode,["mclose"])}return r.push(p),Fe.makeSpan(["minner"],r,t)},mathmlBuilder:(e,t)=>{N9(e);var r=mi(e.body,t);if(e.left!=="."){var n=new tt.MathNode("mo",[so(e.left,e.mode)]);n.setAttribute("fence","true"),r.unshift(n)}if(e.right!=="."){var a=new tt.MathNode("mo",[so(e.right,e.mode)]);a.setAttribute("fence","true"),e.rightColor&&a.setAttribute("mathcolor",e.rightColor),r.push(a)}return uD(r)}});gt({type:"middle",names:["\\middle"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var r=Cv(t[0],e);if(!e.parser.leftrightDepth)throw new nt("\\middle without preceding \\left",r);return{type:"middle",mode:e.parser.mode,delim:r.text}},htmlBuilder:(e,t)=>{var r;if(e.delim===".")r=Im(t,[]);else{r=ll.sizedDelim(e.delim,1,t,e.mode,[]);var n={delim:e.delim,options:t};r.isMiddle=n}return r},mathmlBuilder:(e,t)=>{var r=e.delim==="\\vert"||e.delim==="|"?so("|","text"):so(e.delim,e.mode),n=new tt.MathNode("mo",[r]);return n.setAttribute("fence","true"),n.setAttribute("lspace","0.05em"),n.setAttribute("rspace","0.05em"),n}});var fD=(e,t)=>{var r=Fe.wrapFragment(Ir(e.body,t),t),n=e.label.slice(1),a=t.sizeMultiplier,i,o=0,s=Gt.isCharacterBox(e.body);if(n==="sout")i=Fe.makeSpan(["stretchy","sout"]),i.height=t.fontMetrics().defaultRuleThickness/a,o=-.5*t.fontMetrics().xHeight;else if(n==="phase"){var l=xn({number:.6,unit:"pt"},t),u=xn({number:.35,unit:"ex"},t),d=t.havingBaseSizing();a=a/d.sizeMultiplier;var p=r.height+r.depth+l+u;r.style.paddingLeft=ct(p/2+l);var f=Math.floor(1e3*p*a),m=Lpe(f),b=new pl([new Tu("phase",m)],{width:"400em",height:ct(f/1e3),viewBox:"0 0 400000 "+f,preserveAspectRatio:"xMinYMin slice"});i=Fe.makeSvgSpan(["hide-tail"],[b],t),i.style.height=ct(p),o=r.depth+l+u}else{/cancel/.test(n)?s||r.classes.push("cancel-pad"):n==="angl"?r.classes.push("anglpad"):r.classes.push("boxpad");var y=0,S=0,w=0;/box/.test(n)?(w=Math.max(t.fontMetrics().fboxrule,t.minRuleThickness),y=t.fontMetrics().fboxsep+(n==="colorbox"?0:w),S=y):n==="angl"?(w=Math.max(t.fontMetrics().defaultRuleThickness,t.minRuleThickness),y=4*w,S=Math.max(0,.25-r.depth)):(y=s?.2:0,S=y),i=ml.encloseSpan(r,n,y,S,t),/fbox|boxed|fcolorbox/.test(n)?(i.style.borderStyle="solid",i.style.borderWidth=ct(w)):n==="angl"&&w!==.049&&(i.style.borderTopWidth=ct(w),i.style.borderRightWidth=ct(w)),o=r.depth+S,e.backgroundColor&&(i.style.backgroundColor=e.backgroundColor,e.borderColor&&(i.style.borderColor=e.borderColor))}var E;if(e.backgroundColor)E=Fe.makeVList({positionType:"individualShift",children:[{type:"elem",elem:i,shift:o},{type:"elem",elem:r,shift:0}]},t);else{var k=/cancel|phase/.test(n)?["svg-align"]:[];E=Fe.makeVList({positionType:"individualShift",children:[{type:"elem",elem:r,shift:0},{type:"elem",elem:i,shift:o,wrapperClasses:k}]},t)}return/cancel/.test(n)&&(E.height=r.height,E.depth=r.depth),/cancel/.test(n)&&!s?Fe.makeSpan(["mord","cancel-lap"],[E],t):Fe.makeSpan(["mord"],[E],t)},mD=(e,t)=>{var r=0,n=new tt.MathNode(e.label.indexOf("colorbox")>-1?"mpadded":"menclose",[tn(e.body,t)]);switch(e.label){case"\\cancel":n.setAttribute("notation","updiagonalstrike");break;case"\\bcancel":n.setAttribute("notation","downdiagonalstrike");break;case"\\phase":n.setAttribute("notation","phasorangle");break;case"\\sout":n.setAttribute("notation","horizontalstrike");break;case"\\fbox":n.setAttribute("notation","box");break;case"\\angl":n.setAttribute("notation","actuarial");break;case"\\fcolorbox":case"\\colorbox":if(r=t.fontMetrics().fboxsep*t.fontMetrics().ptPerEm,n.setAttribute("width","+"+2*r+"pt"),n.setAttribute("height","+"+2*r+"pt"),n.setAttribute("lspace",r+"pt"),n.setAttribute("voffset",r+"pt"),e.label==="\\fcolorbox"){var a=Math.max(t.fontMetrics().fboxrule,t.minRuleThickness);n.setAttribute("style","border: "+a+"em solid "+String(e.borderColor))}break;case"\\xcancel":n.setAttribute("notation","updiagonalstrike downdiagonalstrike");break}return e.backgroundColor&&n.setAttribute("mathbackground",e.backgroundColor),n};gt({type:"enclose",names:["\\colorbox"],props:{numArgs:2,allowedInText:!0,argTypes:["color","text"]},handler(e,t,r){var{parser:n,funcName:a}=e,i=br(t[0],"color-token").color,o=t[1];return{type:"enclose",mode:n.mode,label:a,backgroundColor:i,body:o}},htmlBuilder:fD,mathmlBuilder:mD});gt({type:"enclose",names:["\\fcolorbox"],props:{numArgs:3,allowedInText:!0,argTypes:["color","color","text"]},handler(e,t,r){var{parser:n,funcName:a}=e,i=br(t[0],"color-token").color,o=br(t[1],"color-token").color,s=t[2];return{type:"enclose",mode:n.mode,label:a,backgroundColor:o,borderColor:i,body:s}},htmlBuilder:fD,mathmlBuilder:mD});gt({type:"enclose",names:["\\fbox"],props:{numArgs:1,argTypes:["hbox"],allowedInText:!0},handler(e,t){var{parser:r}=e;return{type:"enclose",mode:r.mode,label:"\\fbox",body:t[0]}}});gt({type:"enclose",names:["\\cancel","\\bcancel","\\xcancel","\\sout","\\phase"],props:{numArgs:1},handler(e,t){var{parser:r,funcName:n}=e,a=t[0];return{type:"enclose",mode:r.mode,label:n,body:a}},htmlBuilder:fD,mathmlBuilder:mD});gt({type:"enclose",names:["\\angl"],props:{numArgs:1,argTypes:["hbox"],allowedInText:!1},handler(e,t){var{parser:r}=e;return{type:"enclose",mode:r.mode,label:"\\angl",body:t[0]}}});var GK={};function hs(e){for(var{type:t,names:r,props:n,handler:a,htmlBuilder:i,mathmlBuilder:o}=e,s={type:t,numArgs:n.numArgs||0,allowedInText:!1,numOptionalArgs:0,handler:a},l=0;l<r.length;++l)GK[r[l]]=s;i&&(ny[t]=i),o&&(ay[t]=o)}var jK={};function te(e,t){jK[e]=t}function L9(e){var t=[];e.consumeSpaces();var r=e.fetch().text;for(r==="\\relax"&&(e.consume(),e.consumeSpaces(),r=e.fetch().text);r==="\\hline"||r==="\\hdashline";)e.consume(),t.push(r==="\\hdashline"),e.consumeSpaces(),r=e.fetch().text;return t}var Tv=e=>{var t=e.parser.settings;if(!t.displayMode)throw new nt("{"+e.envName+"} can be used only in display mode.")};function gD(e){if(e.indexOf("ed")===-1)return e.indexOf("*")===-1}function Bu(e,t,r){var{hskipBeforeAndAfter:n,addJot:a,cols:i,arraystretch:o,colSeparationType:s,autoTag:l,singleRow:u,emptySingleRow:d,maxNumCols:p,leqno:f}=t;if(e.gullet.beginGroup(),u||e.gullet.macros.set("\\cr","\\\\\\relax"),!o){var m=e.gullet.expandMacroAsText("\\arraystretch");if(m==null)o=1;else if(o=parseFloat(m),!o||o<0)throw new nt("Invalid \\arraystretch: "+m)}e.gullet.beginGroup();var b=[],y=[b],S=[],w=[],E=l!=null?[]:void 0;function k(){l&&e.gullet.macros.set("\\@eqnsw","1",!0)}function _(){E&&(e.gullet.macros.get("\\df@tag")?(E.push(e.subparse([new Ii("\\df@tag")])),e.gullet.macros.set("\\df@tag",void 0,!0)):E.push(!!l&&e.gullet.macros.get("\\@eqnsw")==="1"))}for(k(),w.push(L9(e));;){var R=e.parseExpression(!1,u?"\\end":"\\\\");e.gullet.endGroup(),e.gullet.beginGroup(),R={type:"ordgroup",mode:e.mode,body:R},r&&(R={type:"styling",mode:e.mode,style:r,body:[R]}),b.push(R);var C=e.fetch().text;if(C==="&"){if(p&&b.length===p){if(u||s)throw new nt("Too many tab characters: &",e.nextToken);e.settings.reportNonstrict("textEnv","Too few columns specified in the {array} column argument.")}e.consume()}else if(C==="\\end"){_(),b.length===1&&R.type==="styling"&&R.body[0].body.length===0&&(y.length>1||!d)&&y.pop(),w.length<y.length+1&&w.push([]);break}else if(C==="\\\\"){e.consume();var F=void 0;e.gullet.future().text!==" "&&(F=e.parseSizeGroup(!0)),S.push(F?F.value:null),_(),w.push(L9(e)),b=[],y.push(b),k()}else throw new nt("Expected & or \\\\ or \\cr or \\end",e.nextToken)}return e.gullet.endGroup(),e.gullet.endGroup(),{type:"array",mode:e.mode,addJot:a,arraystretch:o,body:y,cols:i,rowGaps:S,hskipBeforeAndAfter:n,hLinesBeforeRow:w,colSeparationType:s,tags:E,leqno:f}}function bD(e){return e.slice(0,1)==="d"?"display":"text"}var ps=function(t,r){var n,a,i=t.body.length,o=t.hLinesBeforeRow,s=0,l=new Array(i),u=[],d=Math.max(r.fontMetrics().arrayRuleWidth,r.minRuleThickness),p=1/r.fontMetrics().ptPerEm,f=5*p;if(t.colSeparationType&&t.colSeparationType==="small"){var m=r.havingStyle(Vt.SCRIPT).sizeMultiplier;f=.2778*(m/r.sizeMultiplier)}var b=t.colSeparationType==="CD"?xn({number:3,unit:"ex"},r):12*p,y=3*p,S=t.arraystretch*b,w=.7*S,E=.3*S,k=0;function _(Et){for(var qe=0;qe<Et.length;++qe)qe>0&&(k+=.25),u.push({pos:k,isDashed:Et[qe]})}for(_(o[0]),n=0;n<t.body.length;++n){var R=t.body[n],C=w,F=E;s<R.length&&(s=R.length);var L=new Array(R.length);for(a=0;a<R.length;++a){var I=Ir(R[a],r);F<I.depth&&(F=I.depth),C<I.height&&(C=I.height),L[a]=I}var M=t.rowGaps[n],j=0;M&&(j=xn(M,r),j>0&&(j+=E,F<j&&(F=j),j=0)),t.addJot&&(F+=y),L.height=C,L.depth=F,k+=C,L.pos=k,k+=F+j,l[n]=L,_(o[n+1])}var Y=k/2+r.fontMetrics().axisHeight,W=t.cols||[],$=[],X,z,H=[];if(t.tags&&t.tags.some(Et=>Et))for(n=0;n<i;++n){var B=l[n],ee=B.pos-Y,J=t.tags[n],K=void 0;J===!0?K=Fe.makeSpan(["eqn-num"],[],r):J===!1?K=Fe.makeSpan([],[],r):K=Fe.makeSpan([],ta(J,r,!0),r),K.depth=B.depth,K.height=B.height,H.push({type:"elem",elem:K,shift:ee})}for(a=0,z=0;a<s||z<W.length;++a,++z){for(var G=W[z]||{},P=!0;G.type==="separator";){if(P||(X=Fe.makeSpan(["arraycolsep"],[]),X.style.width=ct(r.fontMetrics().doubleRuleSep),$.push(X)),G.separator==="|"||G.separator===":"){var ne=G.separator==="|"?"solid":"dashed",oe=Fe.makeSpan(["vertical-separator"],[],r);oe.style.height=ct(k),oe.style.borderRightWidth=ct(d),oe.style.borderRightStyle=ne,oe.style.margin="0 "+ct(-d/2);var pe=k-Y;pe&&(oe.style.verticalAlign=ct(-pe)),$.push(oe)}else throw new nt("Invalid separator type: "+G.separator);z++,G=W[z]||{},P=!1}if(!(a>=s)){var ae=void 0;(a>0||t.hskipBeforeAndAfter)&&(ae=Gt.deflt(G.pregap,f),ae!==0&&(X=Fe.makeSpan(["arraycolsep"],[]),X.style.width=ct(ae),$.push(X)));var se=[];for(n=0;n<i;++n){var Ee=l[n],fe=Ee[a];if(fe){var Me=Ee.pos-Y;fe.depth=Ee.depth,fe.height=Ee.height,se.push({type:"elem",elem:fe,shift:Me})}}se=Fe.makeVList({positionType:"individualShift",children:se},r),se=Fe.makeSpan(["col-align-"+(G.align||"c")],[se]),$.push(se),(a<s-1||t.hskipBeforeAndAfter)&&(ae=Gt.deflt(G.postgap,f),ae!==0&&(X=Fe.makeSpan(["arraycolsep"],[]),X.style.width=ct(ae),$.push(X)))}}if(l=Fe.makeSpan(["mtable"],$),u.length>0){for(var Oe=Fe.makeLineSpan("hline",r,d),xe=Fe.makeLineSpan("hdashline",r,d),Te=[{type:"elem",elem:l,shift:0}];u.length>0;){var Ne=u.pop(),he=Ne.pos-Y;Ne.isDashed?Te.push({type:"elem",elem:xe,shift:he}):Te.push({type:"elem",elem:Oe,shift:he})}l=Fe.makeVList({positionType:"individualShift",children:Te},r)}if(H.length===0)return Fe.makeSpan(["mord"],[l],r);var Ze=Fe.makeVList({positionType:"individualShift",children:H},r);return Ze=Fe.makeSpan(["tag"],[Ze],r),Fe.makeFragment([l,Ze])},Ufe={c:"center ",l:"left ",r:"right "},fs=function(t,r){for(var n=[],a=new tt.MathNode("mtd",[],["mtr-glue"]),i=new tt.MathNode("mtd",[],["mml-eqn-num"]),o=0;o<t.body.length;o++){for(var s=t.body[o],l=[],u=0;u<s.length;u++)l.push(new tt.MathNode("mtd",[tn(s[u],r)]));t.tags&&t.tags[o]&&(l.unshift(a),l.push(a),t.leqno?l.unshift(i):l.push(i)),n.push(new tt.MathNode("mtr",l))}var d=new tt.MathNode("mtable",n),p=t.arraystretch===.5?.1:.16+t.arraystretch-1+(t.addJot?.09:0);d.setAttribute("rowspacing",ct(p));var f="",m="";if(t.cols&&t.cols.length>0){var b=t.cols,y="",S=!1,w=0,E=b.length;b[0].type==="separator"&&(f+="top ",w=1),b[b.length-1].type==="separator"&&(f+="bottom ",E-=1);for(var k=w;k<E;k++)b[k].type==="align"?(m+=Ufe[b[k].align],S&&(y+="none "),S=!0):b[k].type==="separator"&&S&&(y+=b[k].separator==="|"?"solid ":"dashed ",S=!1);d.setAttribute("columnalign",m.trim()),/[sd]/.test(y)&&d.setAttribute("columnlines",y.trim())}if(t.colSeparationType==="align"){for(var _=t.cols||[],R="",C=1;C<_.length;C++)R+=C%2?"0em ":"1em ";d.setAttribute("columnspacing",R.trim())}else t.colSeparationType==="alignat"||t.colSeparationType==="gather"?d.setAttribute("columnspacing","0em"):t.colSeparationType==="small"?d.setAttribute("columnspacing","0.2778em"):t.colSeparationType==="CD"?d.setAttribute("columnspacing","0.5em"):d.setAttribute("columnspacing","1em");var F="",L=t.hLinesBeforeRow;f+=L[0].length>0?"left ":"",f+=L[L.length-1].length>0?"right ":"";for(var I=1;I<L.length-1;I++)F+=L[I].length===0?"none ":L[I][0]?"dashed ":"solid ";return/[sd]/.test(F)&&d.setAttribute("rowlines",F.trim()),f!==""&&(d=new tt.MathNode("menclose",[d]),d.setAttribute("notation",f.trim())),t.arraystretch&&t.arraystretch<1&&(d=new tt.MathNode("mstyle",[d]),d.setAttribute("scriptlevel","1")),d},qK=function(t,r){t.envName.indexOf("ed")===-1&&Tv(t);var n=[],a=t.envName.indexOf("at")>-1?"alignat":"align",i=t.envName==="split",o=Bu(t.parser,{cols:n,addJot:!0,autoTag:i?void 0:gD(t.envName),emptySingleRow:!0,colSeparationType:a,maxNumCols:i?2:void 0,leqno:t.parser.settings.leqno},"display"),s,l=0,u={type:"ordgroup",mode:t.mode,body:[]};if(r[0]&&r[0].type==="ordgroup"){for(var d="",p=0;p<r[0].body.length;p++){var f=br(r[0].body[p],"textord");d+=f.text}s=Number(d),l=s*2}var m=!l;o.body.forEach(function(w){for(var E=1;E<w.length;E+=2){var k=br(w[E],"styling"),_=br(k.body[0],"ordgroup");_.body.unshift(u)}if(m)l<w.length&&(l=w.length);else{var R=w.length/2;if(s<R)throw new nt("Too many math in a row: "+("expected "+s+", but got "+R),w[0])}});for(var b=0;b<l;++b){var y="r",S=0;b%2===1?y="l":b>0&&m&&(S=1),n[b]={type:"align",align:y,pregap:S,postgap:0}}return o.colSeparationType=m?"align":"alignat",o};hs({type:"array",names:["array","darray"],props:{numArgs:1},handler(e,t){var r=wv(t[0]),n=r?[t[0]]:br(t[0],"ordgroup").body,a=n.map(function(o){var s=dD(o),l=s.text;if("lcr".indexOf(l)!==-1)return{type:"align",align:l};if(l==="|")return{type:"separator",separator:"|"};if(l===":")return{type:"separator",separator:":"};throw new nt("Unknown column alignment: "+l,o)}),i={cols:a,hskipBeforeAndAfter:!0,maxNumCols:a.length};return Bu(e.parser,i,bD(e.envName))},htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["matrix","pmatrix","bmatrix","Bmatrix","vmatrix","Vmatrix","matrix*","pmatrix*","bmatrix*","Bmatrix*","vmatrix*","Vmatrix*"],props:{numArgs:0},handler(e){var t={matrix:null,pmatrix:["(",")"],bmatrix:["[","]"],Bmatrix:["\\{","\\}"],vmatrix:["|","|"],Vmatrix:["\\Vert","\\Vert"]}[e.envName.replace("*","")],r="c",n={hskipBeforeAndAfter:!1,cols:[{type:"align",align:r}]};if(e.envName.charAt(e.envName.length-1)==="*"){var a=e.parser;if(a.consumeSpaces(),a.fetch().text==="["){if(a.consume(),a.consumeSpaces(),r=a.fetch().text,"lcr".indexOf(r)===-1)throw new nt("Expected l or c or r",a.nextToken);a.consume(),a.consumeSpaces(),a.expect("]"),a.consume(),n.cols=[{type:"align",align:r}]}}var i=Bu(e.parser,n,bD(e.envName)),o=Math.max(0,...i.body.map(s=>s.length));return i.cols=new Array(o).fill({type:"align",align:r}),t?{type:"leftright",mode:e.mode,body:[i],left:t[0],right:t[1],rightColor:void 0}:i},htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["smallmatrix"],props:{numArgs:0},handler(e){var t={arraystretch:.5},r=Bu(e.parser,t,"script");return r.colSeparationType="small",r},htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["subarray"],props:{numArgs:1},handler(e,t){var r=wv(t[0]),n=r?[t[0]]:br(t[0],"ordgroup").body,a=n.map(function(o){var s=dD(o),l=s.text;if("lc".indexOf(l)!==-1)return{type:"align",align:l};throw new nt("Unknown column alignment: "+l,o)});if(a.length>1)throw new nt("{subarray} can contain only one column");var i={cols:a,hskipBeforeAndAfter:!1,arraystretch:.5};if(i=Bu(e.parser,i,"script"),i.body.length>0&&i.body[0].length>1)throw new nt("{subarray} can contain only one column");return i},htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["cases","dcases","rcases","drcases"],props:{numArgs:0},handler(e){var t={arraystretch:1.2,cols:[{type:"align",align:"l",pregap:0,postgap:1},{type:"align",align:"l",pregap:0,postgap:0}]},r=Bu(e.parser,t,bD(e.envName));return{type:"leftright",mode:e.mode,body:[r],left:e.envName.indexOf("r")>-1?".":"\\{",right:e.envName.indexOf("r")>-1?"\\}":".",rightColor:void 0}},htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["align","align*","aligned","split"],props:{numArgs:0},handler:qK,htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["gathered","gather","gather*"],props:{numArgs:0},handler(e){Gt.contains(["gather","gather*"],e.envName)&&Tv(e);var t={cols:[{type:"align",align:"c"}],addJot:!0,colSeparationType:"gather",autoTag:gD(e.envName),emptySingleRow:!0,leqno:e.parser.settings.leqno};return Bu(e.parser,t,"display")},htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["alignat","alignat*","alignedat"],props:{numArgs:1},handler:qK,htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["equation","equation*"],props:{numArgs:0},handler(e){Tv(e);var t={autoTag:gD(e.envName),emptySingleRow:!0,singleRow:!0,maxNumCols:1,leqno:e.parser.settings.leqno};return Bu(e.parser,t,"display")},htmlBuilder:ps,mathmlBuilder:fs});hs({type:"array",names:["CD"],props:{numArgs:0},handler(e){return Tv(e),Afe(e.parser)},htmlBuilder:ps,mathmlBuilder:fs});te("\\nonumber","\\gdef\\@eqnsw{0}");te("\\notag","\\nonumber");gt({type:"text",names:["\\hline","\\hdashline"],props:{numArgs:0,allowedInText:!0,allowedInMath:!0},handler(e,t){throw new nt(e.funcName+" valid only within array environment")}});var M9=GK;gt({type:"environment",names:["\\begin","\\end"],props:{numArgs:1,argTypes:["text"]},handler(e,t){var{parser:r,funcName:n}=e,a=t[0];if(a.type!=="ordgroup")throw new nt("Invalid environment name",a);for(var i="",o=0;o<a.body.length;++o)i+=br(a.body[o],"textord").text;if(n==="\\begin"){if(!M9.hasOwnProperty(i))throw new nt("No such environment: "+i,a);var s=M9[i],{args:l,optArgs:u}=r.parseArguments("\\begin{"+i+"}",s),d={mode:r.mode,envName:i,parser:r},p=s.handler(d,l,u);r.expect("\\end",!1);var f=r.nextToken,m=br(r.parseFunction(),"environment");if(m.name!==i)throw new nt("Mismatch: \\begin{"+i+"} matched by \\end{"+m.name+"}",f);return p}return{type:"environment",mode:r.mode,name:i,nameGroup:a}}});var WK=(e,t)=>{var r=e.font,n=t.withFont(r);return Ir(e.body,n)},VK=(e,t)=>{var r=e.font,n=t.withFont(r);return tn(e.body,n)},I9={"\\Bbb":"\\mathbb","\\bold":"\\mathbf","\\frak":"\\mathfrak","\\bm":"\\boldsymbol"};gt({type:"font",names:["\\mathrm","\\mathit","\\mathbf","\\mathnormal","\\mathsfit","\\mathbb","\\mathcal","\\mathfrak","\\mathscr","\\mathsf","\\mathtt","\\Bbb","\\bold","\\frak"],props:{numArgs:1,allowedInArgument:!0},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=iy(t[0]),i=n;return i in I9&&(i=I9[i]),{type:"font",mode:r.mode,font:i.slice(1),body:a}},htmlBuilder:WK,mathmlBuilder:VK});gt({type:"mclass",names:["\\boldsymbol","\\bm"],props:{numArgs:1},handler:(e,t)=>{var{parser:r}=e,n=t[0],a=Gt.isCharacterBox(n);return{type:"mclass",mode:r.mode,mclass:kv(n),body:[{type:"font",mode:r.mode,font:"boldsymbol",body:n}],isCharacterBox:a}}});gt({type:"font",names:["\\rm","\\sf","\\tt","\\bf","\\it","\\cal"],props:{numArgs:0,allowedInText:!0},handler:(e,t)=>{var{parser:r,funcName:n,breakOnTokenText:a}=e,{mode:i}=r,o=r.parseExpression(!0,a),s="math"+n.slice(1);return{type:"font",mode:i,font:s,body:{type:"ordgroup",mode:r.mode,body:o}}},htmlBuilder:WK,mathmlBuilder:VK});var KK=(e,t)=>{var r=t;return e==="display"?r=r.id>=Vt.SCRIPT.id?r.text():Vt.DISPLAY:e==="text"&&r.size===Vt.DISPLAY.size?r=Vt.TEXT:e==="script"?r=Vt.SCRIPT:e==="scriptscript"&&(r=Vt.SCRIPTSCRIPT),r},yD=(e,t)=>{var r=KK(e.size,t.style),n=r.fracNum(),a=r.fracDen(),i;i=t.havingStyle(n);var o=Ir(e.numer,i,t);if(e.continued){var s=8.5/t.fontMetrics().ptPerEm,l=3.5/t.fontMetrics().ptPerEm;o.height=o.height<s?s:o.height,o.depth=o.depth<l?l:o.depth}i=t.havingStyle(a);var u=Ir(e.denom,i,t),d,p,f;e.hasBarLine?(e.barSize?(p=xn(e.barSize,t),d=Fe.makeLineSpan("frac-line",t,p)):d=Fe.makeLineSpan("frac-line",t),p=d.height,f=d.height):(d=null,p=0,f=t.fontMetrics().defaultRuleThickness);var m,b,y;r.size===Vt.DISPLAY.size||e.size==="display"?(m=t.fontMetrics().num1,p>0?b=3*f:b=7*f,y=t.fontMetrics().denom1):(p>0?(m=t.fontMetrics().num2,b=f):(m=t.fontMetrics().num3,b=3*f),y=t.fontMetrics().denom2);var S;if(d){var E=t.fontMetrics().axisHeight;m-o.depth-(E+.5*p)<b&&(m+=b-(m-o.depth-(E+.5*p))),E-.5*p-(u.height-y)<b&&(y+=b-(E-.5*p-(u.height-y)));var k=-(E-.5*p);S=Fe.makeVList({positionType:"individualShift",children:[{type:"elem",elem:u,shift:y},{type:"elem",elem:d,shift:k},{type:"elem",elem:o,shift:-m}]},t)}else{var w=m-o.depth-(u.height-y);w<b&&(m+=.5*(b-w),y+=.5*(b-w)),S=Fe.makeVList({positionType:"individualShift",children:[{type:"elem",elem:u,shift:y},{type:"elem",elem:o,shift:-m}]},t)}i=t.havingStyle(r),S.height*=i.sizeMultiplier/t.sizeMultiplier,S.depth*=i.sizeMultiplier/t.sizeMultiplier;var _;r.size===Vt.DISPLAY.size?_=t.fontMetrics().delim1:r.size===Vt.SCRIPTSCRIPT.size?_=t.havingStyle(Vt.SCRIPT).fontMetrics().delim2:_=t.fontMetrics().delim2;var R,C;return e.leftDelim==null?R=Im(t,["mopen"]):R=ll.customSizedDelim(e.leftDelim,_,!0,t.havingStyle(r),e.mode,["mopen"]),e.continued?C=Fe.makeSpan([]):e.rightDelim==null?C=Im(t,["mclose"]):C=ll.customSizedDelim(e.rightDelim,_,!0,t.havingStyle(r),e.mode,["mclose"]),Fe.makeSpan(["mord"].concat(i.sizingClasses(t)),[R,Fe.makeSpan(["mfrac"],[S]),C],t)},vD=(e,t)=>{var r=new tt.MathNode("mfrac",[tn(e.numer,t),tn(e.denom,t)]);if(!e.hasBarLine)r.setAttribute("linethickness","0px");else if(e.barSize){var n=xn(e.barSize,t);r.setAttribute("linethickness",ct(n))}var a=KK(e.size,t.style);if(a.size!==t.style.size){r=new tt.MathNode("mstyle",[r]);var i=a.size===Vt.DISPLAY.size?"true":"false";r.setAttribute("displaystyle",i),r.setAttribute("scriptlevel","0")}if(e.leftDelim!=null||e.rightDelim!=null){var o=[];if(e.leftDelim!=null){var s=new tt.MathNode("mo",[new tt.TextNode(e.leftDelim.replace("\\",""))]);s.setAttribute("fence","true"),o.push(s)}if(o.push(r),e.rightDelim!=null){var l=new tt.MathNode("mo",[new tt.TextNode(e.rightDelim.replace("\\",""))]);l.setAttribute("fence","true"),o.push(l)}return uD(o)}return r};gt({type:"genfrac",names:["\\dfrac","\\frac","\\tfrac","\\dbinom","\\binom","\\tbinom","\\\\atopfrac","\\\\bracefrac","\\\\brackfrac"],props:{numArgs:2,allowedInArgument:!0},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=t[0],i=t[1],o,s=null,l=null,u="auto";switch(n){case"\\dfrac":case"\\frac":case"\\tfrac":o=!0;break;case"\\\\atopfrac":o=!1;break;case"\\dbinom":case"\\binom":case"\\tbinom":o=!1,s="(",l=")";break;case"\\\\bracefrac":o=!1,s="\\{",l="\\}";break;case"\\\\brackfrac":o=!1,s="[",l="]";break;default:throw new Error("Unrecognized genfrac command")}switch(n){case"\\dfrac":case"\\dbinom":u="display";break;case"\\tfrac":case"\\tbinom":u="text";break}return{type:"genfrac",mode:r.mode,continued:!1,numer:a,denom:i,hasBarLine:o,leftDelim:s,rightDelim:l,size:u,barSize:null}},htmlBuilder:yD,mathmlBuilder:vD});gt({type:"genfrac",names:["\\cfrac"],props:{numArgs:2},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=t[0],i=t[1];return{type:"genfrac",mode:r.mode,continued:!0,numer:a,denom:i,hasBarLine:!0,leftDelim:null,rightDelim:null,size:"display",barSize:null}}});gt({type:"infix",names:["\\over","\\choose","\\atop","\\brace","\\brack"],props:{numArgs:0,infix:!0},handler(e){var{parser:t,funcName:r,token:n}=e,a;switch(r){case"\\over":a="\\frac";break;case"\\choose":a="\\binom";break;case"\\atop":a="\\\\atopfrac";break;case"\\brace":a="\\\\bracefrac";break;case"\\brack":a="\\\\brackfrac";break;default:throw new Error("Unrecognized infix genfrac command")}return{type:"infix",mode:t.mode,replaceWith:a,token:n}}});var O9=["display","text","script","scriptscript"],B9=function(t){var r=null;return t.length>0&&(r=t,r=r==="."?null:r),r};gt({type:"genfrac",names:["\\genfrac"],props:{numArgs:6,allowedInArgument:!0,argTypes:["math","math","size","text","math","math"]},handler(e,t){var{parser:r}=e,n=t[4],a=t[5],i=iy(t[0]),o=i.type==="atom"&&i.family==="open"?B9(i.text):null,s=iy(t[1]),l=s.type==="atom"&&s.family==="close"?B9(s.text):null,u=br(t[2],"size"),d,p=null;u.isBlank?d=!0:(p=u.value,d=p.number>0);var f="auto",m=t[3];if(m.type==="ordgroup"){if(m.body.length>0){var b=br(m.body[0],"textord");f=O9[Number(b.text)]}}else m=br(m,"textord"),f=O9[Number(m.text)];return{type:"genfrac",mode:r.mode,numer:n,denom:a,continued:!1,hasBarLine:d,barSize:p,leftDelim:o,rightDelim:l,size:f}},htmlBuilder:yD,mathmlBuilder:vD});gt({type:"infix",names:["\\above"],props:{numArgs:1,argTypes:["size"],infix:!0},handler(e,t){var{parser:r,funcName:n,token:a}=e;return{type:"infix",mode:r.mode,replaceWith:"\\\\abovefrac",size:br(t[0],"size").value,token:a}}});gt({type:"genfrac",names:["\\\\abovefrac"],props:{numArgs:3,argTypes:["math","size","math"]},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=t[0],i=ype(br(t[1],"infix").size),o=t[2],s=i.number>0;return{type:"genfrac",mode:r.mode,numer:a,denom:o,continued:!1,hasBarLine:s,barSize:i,leftDelim:null,rightDelim:null,size:"auto"}},htmlBuilder:yD,mathmlBuilder:vD});var YK=(e,t)=>{var r=t.style,n,a;e.type==="supsub"?(n=e.sup?Ir(e.sup,t.havingStyle(r.sup()),t):Ir(e.sub,t.havingStyle(r.sub()),t),a=br(e.base,"horizBrace")):a=br(e,"horizBrace");var i=Ir(a.base,t.havingBaseStyle(Vt.DISPLAY)),o=ml.svgSpan(a,t),s;if(a.isOver?(s=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:i},{type:"kern",size:.1},{type:"elem",elem:o}]},t),s.children[0].children[0].children[1].classes.push("svg-align")):(s=Fe.makeVList({positionType:"bottom",positionData:i.depth+.1+o.height,children:[{type:"elem",elem:o},{type:"kern",size:.1},{type:"elem",elem:i}]},t),s.children[0].children[0].children[0].classes.push("svg-align")),n){var l=Fe.makeSpan(["mord",a.isOver?"mover":"munder"],[s],t);a.isOver?s=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:l},{type:"kern",size:.2},{type:"elem",elem:n}]},t):s=Fe.makeVList({positionType:"bottom",positionData:l.depth+.2+n.height+n.depth,children:[{type:"elem",elem:n},{type:"kern",size:.2},{type:"elem",elem:l}]},t)}return Fe.makeSpan(["mord",a.isOver?"mover":"munder"],[s],t)},$fe=(e,t)=>{var r=ml.mathMLnode(e.label);return new tt.MathNode(e.isOver?"mover":"munder",[tn(e.base,t),r])};gt({type:"horizBrace",names:["\\overbrace","\\underbrace"],props:{numArgs:1},handler(e,t){var{parser:r,funcName:n}=e;return{type:"horizBrace",mode:r.mode,label:n,isOver:/^\\over/.test(n),base:t[0]}},htmlBuilder:YK,mathmlBuilder:$fe});gt({type:"href",names:["\\href"],props:{numArgs:2,argTypes:["url","original"],allowedInText:!0},handler:(e,t)=>{var{parser:r}=e,n=t[1],a=br(t[0],"url").url;return r.settings.isTrusted({command:"\\href",url:a})?{type:"href",mode:r.mode,href:a,body:Ln(n)}:r.formatUnsupportedCmd("\\href")},htmlBuilder:(e,t)=>{var r=ta(e.body,t,!1);return Fe.makeAnchor(e.href,[],r,t)},mathmlBuilder:(e,t)=>{var r=Au(e.body,t);return r instanceof Mi||(r=new Mi("mrow",[r])),r.setAttribute("href",e.href),r}});gt({type:"href",names:["\\url"],props:{numArgs:1,argTypes:["url"],allowedInText:!0},handler:(e,t)=>{var{parser:r}=e,n=br(t[0],"url").url;if(!r.settings.isTrusted({command:"\\url",url:n}))return r.formatUnsupportedCmd("\\url");for(var a=[],i=0;i<n.length;i++){var o=n[i];o==="~"&&(o="\\textasciitilde"),a.push({type:"textord",mode:"text",text:o})}var s={type:"text",mode:r.mode,font:"\\texttt",body:a};return{type:"href",mode:r.mode,href:n,body:Ln(s)}}});gt({type:"hbox",names:["\\hbox"],props:{numArgs:1,argTypes:["text"],allowedInText:!0,primitive:!0},handler(e,t){var{parser:r}=e;return{type:"hbox",mode:r.mode,body:Ln(t[0])}},htmlBuilder(e,t){var r=ta(e.body,t,!1);return Fe.makeFragment(r)},mathmlBuilder(e,t){return new tt.MathNode("mrow",mi(e.body,t))}});gt({type:"html",names:["\\htmlClass","\\htmlId","\\htmlStyle","\\htmlData"],props:{numArgs:2,argTypes:["raw","original"],allowedInText:!0},handler:(e,t)=>{var{parser:r,funcName:n,token:a}=e,i=br(t[0],"raw").string,o=t[1];r.settings.strict&&r.settings.reportNonstrict("htmlExtension","HTML extension is disabled on strict mode");var s,l={};switch(n){case"\\htmlClass":l.class=i,s={command:"\\htmlClass",class:i};break;case"\\htmlId":l.id=i,s={command:"\\htmlId",id:i};break;case"\\htmlStyle":l.style=i,s={command:"\\htmlStyle",style:i};break;case"\\htmlData":{for(var u=i.split(","),d=0;d<u.length;d++){var p=u[d].split("=");if(p.length!==2)throw new nt("Error parsing key-value for \\htmlData");l["data-"+p[0].trim()]=p[1].trim()}s={command:"\\htmlData",attributes:l};break}default:throw new Error("Unrecognized html command")}return r.settings.isTrusted(s)?{type:"html",mode:r.mode,attributes:l,body:Ln(o)}:r.formatUnsupportedCmd(n)},htmlBuilder:(e,t)=>{var r=ta(e.body,t,!1),n=["enclosing"];e.attributes.class&&n.push(...e.attributes.class.trim().split(/\s+/));var a=Fe.makeSpan(n,r,t);for(var i in e.attributes)i!=="class"&&e.attributes.hasOwnProperty(i)&&a.setAttribute(i,e.attributes[i]);return a},mathmlBuilder:(e,t)=>Au(e.body,t)});gt({type:"htmlmathml",names:["\\html@mathml"],props:{numArgs:2,allowedInText:!0},handler:(e,t)=>{var{parser:r}=e;return{type:"htmlmathml",mode:r.mode,html:Ln(t[0]),mathml:Ln(t[1])}},htmlBuilder:(e,t)=>{var r=ta(e.html,t,!1);return Fe.makeFragment(r)},mathmlBuilder:(e,t)=>Au(e.mathml,t)});var Nx=function(t){if(/^[-+]? *(\d+(\.\d*)?|\.\d+)$/.test(t))return{number:+t,unit:"bp"};var r=/([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(t);if(!r)throw new nt("Invalid size: '"+t+"' in \\includegraphics");var n={number:+(r[1]+r[2]),unit:r[3]};if(!fK(n))throw new nt("Invalid unit: '"+n.unit+"' in \\includegraphics.");return n};gt({type:"includegraphics",names:["\\includegraphics"],props:{numArgs:1,numOptionalArgs:1,argTypes:["raw","url"],allowedInText:!1},handler:(e,t,r)=>{var{parser:n}=e,a={number:0,unit:"em"},i={number:.9,unit:"em"},o={number:0,unit:"em"},s="";if(r[0])for(var l=br(r[0],"raw").string,u=l.split(","),d=0;d<u.length;d++){var p=u[d].split("=");if(p.length===2){var f=p[1].trim();switch(p[0].trim()){case"alt":s=f;break;case"width":a=Nx(f);break;case"height":i=Nx(f);break;case"totalheight":o=Nx(f);break;default:throw new nt("Invalid key: '"+p[0]+"' in \\includegraphics.")}}}var m=br(t[0],"url").url;return s===""&&(s=m,s=s.replace(/^.*[\\/]/,""),s=s.substring(0,s.lastIndexOf("."))),n.settings.isTrusted({command:"\\includegraphics",url:m})?{type:"includegraphics",mode:n.mode,alt:s,width:a,height:i,totalheight:o,src:m}:n.formatUnsupportedCmd("\\includegraphics")},htmlBuilder:(e,t)=>{var r=xn(e.height,t),n=0;e.totalheight.number>0&&(n=xn(e.totalheight,t)-r);var a=0;e.width.number>0&&(a=xn(e.width,t));var i={height:ct(r+n)};a>0&&(i.width=ct(a)),n>0&&(i.verticalAlign=ct(-n));var o=new $pe(e.src,e.alt,i);return o.height=r,o.depth=n,o},mathmlBuilder:(e,t)=>{var r=new tt.MathNode("mglyph",[]);r.setAttribute("alt",e.alt);var n=xn(e.height,t),a=0;if(e.totalheight.number>0&&(a=xn(e.totalheight,t)-n,r.setAttribute("valign",ct(-a))),r.setAttribute("height",ct(n+a)),e.width.number>0){var i=xn(e.width,t);r.setAttribute("width",ct(i))}return r.setAttribute("src",e.src),r}});gt({type:"kern",names:["\\kern","\\mkern","\\hskip","\\mskip"],props:{numArgs:1,argTypes:["size"],primitive:!0,allowedInText:!0},handler(e,t){var{parser:r,funcName:n}=e,a=br(t[0],"size");if(r.settings.strict){var i=n[1]==="m",o=a.value.unit==="mu";i?(o||r.settings.reportNonstrict("mathVsTextUnits","LaTeX's "+n+" supports only mu units, "+("not "+a.value.unit+" units")),r.mode!=="math"&&r.settings.reportNonstrict("mathVsTextUnits","LaTeX's "+n+" works only in math mode")):o&&r.settings.reportNonstrict("mathVsTextUnits","LaTeX's "+n+" doesn't support mu units")}return{type:"kern",mode:r.mode,dimension:a.value}},htmlBuilder(e,t){return Fe.makeGlue(e.dimension,t)},mathmlBuilder(e,t){var r=xn(e.dimension,t);return new tt.SpaceNode(r)}});gt({type:"lap",names:["\\mathllap","\\mathrlap","\\mathclap"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=t[0];return{type:"lap",mode:r.mode,alignment:n.slice(5),body:a}},htmlBuilder:(e,t)=>{var r;e.alignment==="clap"?(r=Fe.makeSpan([],[Ir(e.body,t)]),r=Fe.makeSpan(["inner"],[r],t)):r=Fe.makeSpan(["inner"],[Ir(e.body,t)]);var n=Fe.makeSpan(["fix"],[]),a=Fe.makeSpan([e.alignment],[r,n],t),i=Fe.makeSpan(["strut"]);return i.style.height=ct(a.height+a.depth),a.depth&&(i.style.verticalAlign=ct(-a.depth)),a.children.unshift(i),a=Fe.makeSpan(["thinbox"],[a],t),Fe.makeSpan(["mord","vbox"],[a],t)},mathmlBuilder:(e,t)=>{var r=new tt.MathNode("mpadded",[tn(e.body,t)]);if(e.alignment!=="rlap"){var n=e.alignment==="llap"?"-1":"-0.5";r.setAttribute("lspace",n+"width")}return r.setAttribute("width","0px"),r}});gt({type:"styling",names:["\\(","$"],props:{numArgs:0,allowedInText:!0,allowedInMath:!1},handler(e,t){var{funcName:r,parser:n}=e,a=n.mode;n.switchMode("math");var i=r==="\\("?"\\)":"$",o=n.parseExpression(!1,i);return n.expect(i),n.switchMode(a),{type:"styling",mode:n.mode,style:"text",body:o}}});gt({type:"text",names:["\\)","\\]"],props:{numArgs:0,allowedInText:!0,allowedInMath:!1},handler(e,t){throw new nt("Mismatched "+e.funcName)}});var P9=(e,t)=>{switch(t.style.size){case Vt.DISPLAY.size:return e.display;case Vt.TEXT.size:return e.text;case Vt.SCRIPT.size:return e.script;case Vt.SCRIPTSCRIPT.size:return e.scriptscript;default:return e.text}};gt({type:"mathchoice",names:["\\mathchoice"],props:{numArgs:4,primitive:!0},handler:(e,t)=>{var{parser:r}=e;return{type:"mathchoice",mode:r.mode,display:Ln(t[0]),text:Ln(t[1]),script:Ln(t[2]),scriptscript:Ln(t[3])}},htmlBuilder:(e,t)=>{var r=P9(e,t),n=ta(r,t,!1);return Fe.makeFragment(n)},mathmlBuilder:(e,t)=>{var r=P9(e,t);return Au(r,t)}});var JK=(e,t,r,n,a,i,o)=>{e=Fe.makeSpan([],[e]);var s=r&&Gt.isCharacterBox(r),l,u;if(t){var d=Ir(t,n.havingStyle(a.sup()),n);u={elem:d,kern:Math.max(n.fontMetrics().bigOpSpacing1,n.fontMetrics().bigOpSpacing3-d.depth)}}if(r){var p=Ir(r,n.havingStyle(a.sub()),n);l={elem:p,kern:Math.max(n.fontMetrics().bigOpSpacing2,n.fontMetrics().bigOpSpacing4-p.height)}}var f;if(u&&l){var m=n.fontMetrics().bigOpSpacing5+l.elem.height+l.elem.depth+l.kern+e.depth+o;f=Fe.makeVList({positionType:"bottom",positionData:m,children:[{type:"kern",size:n.fontMetrics().bigOpSpacing5},{type:"elem",elem:l.elem,marginLeft:ct(-i)},{type:"kern",size:l.kern},{type:"elem",elem:e},{type:"kern",size:u.kern},{type:"elem",elem:u.elem,marginLeft:ct(i)},{type:"kern",size:n.fontMetrics().bigOpSpacing5}]},n)}else if(l){var b=e.height-o;f=Fe.makeVList({positionType:"top",positionData:b,children:[{type:"kern",size:n.fontMetrics().bigOpSpacing5},{type:"elem",elem:l.elem,marginLeft:ct(-i)},{type:"kern",size:l.kern},{type:"elem",elem:e}]},n)}else if(u){var y=e.depth+o;f=Fe.makeVList({positionType:"bottom",positionData:y,children:[{type:"elem",elem:e},{type:"kern",size:u.kern},{type:"elem",elem:u.elem,marginLeft:ct(i)},{type:"kern",size:n.fontMetrics().bigOpSpacing5}]},n)}else return e;var S=[f];if(l&&i!==0&&!s){var w=Fe.makeSpan(["mspace"],[],n);w.style.marginRight=ct(i),S.unshift(w)}return Fe.makeSpan(["mop","op-limits"],S,n)},XK=["\\smallint"],sp=(e,t)=>{var r,n,a=!1,i;e.type==="supsub"?(r=e.sup,n=e.sub,i=br(e.base,"op"),a=!0):i=br(e,"op");var o=t.style,s=!1;o.size===Vt.DISPLAY.size&&i.symbol&&!Gt.contains(XK,i.name)&&(s=!0);var l;if(i.symbol){var u=s?"Size2-Regular":"Size1-Regular",d="";if((i.name==="\\oiint"||i.name==="\\oiiint")&&(d=i.name.slice(1),i.name=d==="oiint"?"\\iint":"\\iiint"),l=Fe.makeSymbol(i.name,u,"math",t,["mop","op-symbol",s?"large-op":"small-op"]),d.length>0){var p=l.italic,f=Fe.staticSvg(d+"Size"+(s?"2":"1"),t);l=Fe.makeVList({positionType:"individualShift",children:[{type:"elem",elem:l,shift:0},{type:"elem",elem:f,shift:s?.08:0}]},t),i.name="\\"+d,l.classes.unshift("mop"),l.italic=p}}else if(i.body){var m=ta(i.body,t,!0);m.length===1&&m[0]instanceof oo?(l=m[0],l.classes[0]="mop"):l=Fe.makeSpan(["mop"],m,t)}else{for(var b=[],y=1;y<i.name.length;y++)b.push(Fe.mathsym(i.name[y],i.mode,t));l=Fe.makeSpan(["mop"],b,t)}var S=0,w=0;return(l instanceof oo||i.name==="\\oiint"||i.name==="\\oiiint")&&!i.suppressBaseShift&&(S=(l.height-l.depth)/2-t.fontMetrics().axisHeight,w=l.italic),a?JK(l,r,n,t,o,w,S):(S&&(l.style.position="relative",l.style.top=ct(S)),l)},dg=(e,t)=>{var r;if(e.symbol)r=new Mi("mo",[so(e.name,e.mode)]),Gt.contains(XK,e.name)&&r.setAttribute("largeop","false");else if(e.body)r=new Mi("mo",mi(e.body,t));else{r=new Mi("mi",[new is(e.name.slice(1))]);var n=new Mi("mo",[so("⁡","text")]);e.parentIsSupSub?r=new Mi("mrow",[r,n]):r=CK([r,n])}return r},Gfe={"∏":"\\prod","∐":"\\coprod","∑":"\\sum","⋀":"\\bigwedge","⋁":"\\bigvee","⋂":"\\bigcap","⋃":"\\bigcup","⨀":"\\bigodot","⨁":"\\bigoplus","⨂":"\\bigotimes","⨄":"\\biguplus","⨆":"\\bigsqcup"};gt({type:"op",names:["\\coprod","\\bigvee","\\bigwedge","\\biguplus","\\bigcap","\\bigcup","\\intop","\\prod","\\sum","\\bigotimes","\\bigoplus","\\bigodot","\\bigsqcup","\\smallint","∏","∐","∑","⋀","⋁","⋂","⋃","⨀","⨁","⨂","⨄","⨆"],props:{numArgs:0},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=n;return a.length===1&&(a=Gfe[a]),{type:"op",mode:r.mode,limits:!0,parentIsSupSub:!1,symbol:!0,name:a}},htmlBuilder:sp,mathmlBuilder:dg});gt({type:"op",names:["\\mathop"],props:{numArgs:1,primitive:!0},handler:(e,t)=>{var{parser:r}=e,n=t[0];return{type:"op",mode:r.mode,limits:!1,parentIsSupSub:!1,symbol:!1,body:Ln(n)}},htmlBuilder:sp,mathmlBuilder:dg});var jfe={"∫":"\\int","∬":"\\iint","∭":"\\iiint","∮":"\\oint","∯":"\\oiint","∰":"\\oiiint"};gt({type:"op",names:["\\arcsin","\\arccos","\\arctan","\\arctg","\\arcctg","\\arg","\\ch","\\cos","\\cosec","\\cosh","\\cot","\\cotg","\\coth","\\csc","\\ctg","\\cth","\\deg","\\dim","\\exp","\\hom","\\ker","\\lg","\\ln","\\log","\\sec","\\sin","\\sinh","\\sh","\\tan","\\tanh","\\tg","\\th"],props:{numArgs:0},handler(e){var{parser:t,funcName:r}=e;return{type:"op",mode:t.mode,limits:!1,parentIsSupSub:!1,symbol:!1,name:r}},htmlBuilder:sp,mathmlBuilder:dg});gt({type:"op",names:["\\det","\\gcd","\\inf","\\lim","\\max","\\min","\\Pr","\\sup"],props:{numArgs:0},handler(e){var{parser:t,funcName:r}=e;return{type:"op",mode:t.mode,limits:!0,parentIsSupSub:!1,symbol:!1,name:r}},htmlBuilder:sp,mathmlBuilder:dg});gt({type:"op",names:["\\int","\\iint","\\iiint","\\oint","\\oiint","\\oiiint","∫","∬","∭","∮","∯","∰"],props:{numArgs:0},handler(e){var{parser:t,funcName:r}=e,n=r;return n.length===1&&(n=jfe[n]),{type:"op",mode:t.mode,limits:!1,parentIsSupSub:!1,symbol:!0,name:n}},htmlBuilder:sp,mathmlBuilder:dg});var ZK=(e,t)=>{var r,n,a=!1,i;e.type==="supsub"?(r=e.sup,n=e.sub,i=br(e.base,"operatorname"),a=!0):i=br(e,"operatorname");var o;if(i.body.length>0){for(var s=i.body.map(p=>{var f=p.text;return typeof f=="string"?{type:"textord",mode:p.mode,text:f}:p}),l=ta(s,t.withFont("mathrm"),!0),u=0;u<l.length;u++){var d=l[u];d instanceof oo&&(d.text=d.text.replace(/\u2212/,"-").replace(/\u2217/,"*"))}o=Fe.makeSpan(["mop"],l,t)}else o=Fe.makeSpan(["mop"],[],t);return a?JK(o,r,n,t,t.style,0,0):o},qfe=(e,t)=>{for(var r=mi(e.body,t.withFont("mathrm")),n=!0,a=0;a<r.length;a++){var i=r[a];if(!(i instanceof tt.SpaceNode))if(i instanceof tt.MathNode)switch(i.type){case"mi":case"mn":case"ms":case"mspace":case"mtext":break;case"mo":{var o=i.children[0];i.children.length===1&&o instanceof tt.TextNode?o.text=o.text.replace(/\u2212/,"-").replace(/\u2217/,"*"):n=!1;break}default:n=!1}else n=!1}if(n){var s=r.map(d=>d.toText()).join("");r=[new tt.TextNode(s)]}var l=new tt.MathNode("mi",r);l.setAttribute("mathvariant","normal");var u=new tt.MathNode("mo",[so("⁡","text")]);return e.parentIsSupSub?new tt.MathNode("mrow",[l,u]):tt.newDocumentFragment([l,u])};gt({type:"operatorname",names:["\\operatorname@","\\operatornamewithlimits"],props:{numArgs:1},handler:(e,t)=>{var{parser:r,funcName:n}=e,a=t[0];return{type:"operatorname",mode:r.mode,body:Ln(a),alwaysHandleSupSub:n==="\\operatornamewithlimits",limits:!1,parentIsSupSub:!1}},htmlBuilder:ZK,mathmlBuilder:qfe});te("\\operatorname","\\@ifstar\\operatornamewithlimits\\operatorname@");ed({type:"ordgroup",htmlBuilder(e,t){return e.semisimple?Fe.makeFragment(ta(e.body,t,!1)):Fe.makeSpan(["mord"],ta(e.body,t,!0),t)},mathmlBuilder(e,t){return Au(e.body,t,!0)}});gt({type:"overline",names:["\\overline"],props:{numArgs:1},handler(e,t){var{parser:r}=e,n=t[0];return{type:"overline",mode:r.mode,body:n}},htmlBuilder(e,t){var r=Ir(e.body,t.havingCrampedStyle()),n=Fe.makeLineSpan("overline-line",t),a=t.fontMetrics().defaultRuleThickness,i=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:r},{type:"kern",size:3*a},{type:"elem",elem:n},{type:"kern",size:a}]},t);return Fe.makeSpan(["mord","overline"],[i],t)},mathmlBuilder(e,t){var r=new tt.MathNode("mo",[new tt.TextNode("‾")]);r.setAttribute("stretchy","true");var n=new tt.MathNode("mover",[tn(e.body,t),r]);return n.setAttribute("accent","true"),n}});gt({type:"phantom",names:["\\phantom"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:r}=e,n=t[0];return{type:"phantom",mode:r.mode,body:Ln(n)}},htmlBuilder:(e,t)=>{var r=ta(e.body,t.withPhantom(),!1);return Fe.makeFragment(r)},mathmlBuilder:(e,t)=>{var r=mi(e.body,t);return new tt.MathNode("mphantom",r)}});gt({type:"hphantom",names:["\\hphantom"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:r}=e,n=t[0];return{type:"hphantom",mode:r.mode,body:n}},htmlBuilder:(e,t)=>{var r=Fe.makeSpan([],[Ir(e.body,t.withPhantom())]);if(r.height=0,r.depth=0,r.children)for(var n=0;n<r.children.length;n++)r.children[n].height=0,r.children[n].depth=0;return r=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:r}]},t),Fe.makeSpan(["mord"],[r],t)},mathmlBuilder:(e,t)=>{var r=mi(Ln(e.body),t),n=new tt.MathNode("mphantom",r),a=new tt.MathNode("mpadded",[n]);return a.setAttribute("height","0px"),a.setAttribute("depth","0px"),a}});gt({type:"vphantom",names:["\\vphantom"],props:{numArgs:1,allowedInText:!0},handler:(e,t)=>{var{parser:r}=e,n=t[0];return{type:"vphantom",mode:r.mode,body:n}},htmlBuilder:(e,t)=>{var r=Fe.makeSpan(["inner"],[Ir(e.body,t.withPhantom())]),n=Fe.makeSpan(["fix"],[]);return Fe.makeSpan(["mord","rlap"],[r,n],t)},mathmlBuilder:(e,t)=>{var r=mi(Ln(e.body),t),n=new tt.MathNode("mphantom",r),a=new tt.MathNode("mpadded",[n]);return a.setAttribute("width","0px"),a}});gt({type:"raisebox",names:["\\raisebox"],props:{numArgs:2,argTypes:["size","hbox"],allowedInText:!0},handler(e,t){var{parser:r}=e,n=br(t[0],"size").value,a=t[1];return{type:"raisebox",mode:r.mode,dy:n,body:a}},htmlBuilder(e,t){var r=Ir(e.body,t),n=xn(e.dy,t);return Fe.makeVList({positionType:"shift",positionData:-n,children:[{type:"elem",elem:r}]},t)},mathmlBuilder(e,t){var r=new tt.MathNode("mpadded",[tn(e.body,t)]),n=e.dy.number+e.dy.unit;return r.setAttribute("voffset",n),r}});gt({type:"internal",names:["\\relax"],props:{numArgs:0,allowedInText:!0,allowedInArgument:!0},handler(e){var{parser:t}=e;return{type:"internal",mode:t.mode}}});gt({type:"rule",names:["\\rule"],props:{numArgs:2,numOptionalArgs:1,allowedInText:!0,allowedInMath:!0,argTypes:["size","size","size"]},handler(e,t,r){var{parser:n}=e,a=r[0],i=br(t[0],"size"),o=br(t[1],"size");return{type:"rule",mode:n.mode,shift:a&&br(a,"size").value,width:i.value,height:o.value}},htmlBuilder(e,t){var r=Fe.makeSpan(["mord","rule"],[],t),n=xn(e.width,t),a=xn(e.height,t),i=e.shift?xn(e.shift,t):0;return r.style.borderRightWidth=ct(n),r.style.borderTopWidth=ct(a),r.style.bottom=ct(i),r.width=n,r.height=a+i,r.depth=-i,r.maxFontSize=a*1.125*t.sizeMultiplier,r},mathmlBuilder(e,t){var r=xn(e.width,t),n=xn(e.height,t),a=e.shift?xn(e.shift,t):0,i=t.color&&t.getColor()||"black",o=new tt.MathNode("mspace");o.setAttribute("mathbackground",i),o.setAttribute("width",ct(r)),o.setAttribute("height",ct(n));var s=new tt.MathNode("mpadded",[o]);return a>=0?s.setAttribute("height",ct(a)):(s.setAttribute("height",ct(a)),s.setAttribute("depth",ct(-a))),s.setAttribute("voffset",ct(a)),s}});function QK(e,t,r){for(var n=ta(e,t,!1),a=t.sizeMultiplier/r.sizeMultiplier,i=0;i<n.length;i++){var o=n[i].classes.indexOf("sizing");o<0?Array.prototype.push.apply(n[i].classes,t.sizingClasses(r)):n[i].classes[o+1]==="reset-size"+t.size&&(n[i].classes[o+1]="reset-size"+r.size),n[i].height*=a,n[i].depth*=a}return Fe.makeFragment(n)}var z9=["\\tiny","\\sixptsize","\\scriptsize","\\footnotesize","\\small","\\normalsize","\\large","\\Large","\\LARGE","\\huge","\\Huge"],Wfe=(e,t)=>{var r=t.havingSize(e.size);return QK(e.body,r,t)};gt({type:"sizing",names:z9,props:{numArgs:0,allowedInText:!0},handler:(e,t)=>{var{breakOnTokenText:r,funcName:n,parser:a}=e,i=a.parseExpression(!1,r);return{type:"sizing",mode:a.mode,size:z9.indexOf(n)+1,body:i}},htmlBuilder:Wfe,mathmlBuilder:(e,t)=>{var r=t.havingSize(e.size),n=mi(e.body,r),a=new tt.MathNode("mstyle",n);return a.setAttribute("mathsize",ct(r.sizeMultiplier)),a}});gt({type:"smash",names:["\\smash"],props:{numArgs:1,numOptionalArgs:1,allowedInText:!0},handler:(e,t,r)=>{var{parser:n}=e,a=!1,i=!1,o=r[0]&&br(r[0],"ordgroup");if(o)for(var s="",l=0;l<o.body.length;++l){var u=o.body[l];if(s=u.text,s==="t")a=!0;else if(s==="b")i=!0;else{a=!1,i=!1;break}}else a=!0,i=!0;var d=t[0];return{type:"smash",mode:n.mode,body:d,smashHeight:a,smashDepth:i}},htmlBuilder:(e,t)=>{var r=Fe.makeSpan([],[Ir(e.body,t)]);if(!e.smashHeight&&!e.smashDepth)return r;if(e.smashHeight&&(r.height=0,r.children))for(var n=0;n<r.children.length;n++)r.children[n].height=0;if(e.smashDepth&&(r.depth=0,r.children))for(var a=0;a<r.children.length;a++)r.children[a].depth=0;var i=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:r}]},t);return Fe.makeSpan(["mord"],[i],t)},mathmlBuilder:(e,t)=>{var r=new tt.MathNode("mpadded",[tn(e.body,t)]);return e.smashHeight&&r.setAttribute("height","0px"),e.smashDepth&&r.setAttribute("depth","0px"),r}});gt({type:"sqrt",names:["\\sqrt"],props:{numArgs:1,numOptionalArgs:1},handler(e,t,r){var{parser:n}=e,a=r[0],i=t[0];return{type:"sqrt",mode:n.mode,body:i,index:a}},htmlBuilder(e,t){var r=Ir(e.body,t.havingCrampedStyle());r.height===0&&(r.height=t.fontMetrics().xHeight),r=Fe.wrapFragment(r,t);var n=t.fontMetrics(),a=n.defaultRuleThickness,i=a;t.style.id<Vt.TEXT.id&&(i=t.fontMetrics().xHeight);var o=a+i/4,s=r.height+r.depth+o+a,{span:l,ruleWidth:u,advanceWidth:d}=ll.sqrtImage(s,t),p=l.height-u;p>r.height+r.depth+o&&(o=(o+p-r.height-r.depth)/2);var f=l.height-r.height-o-u;r.style.paddingLeft=ct(d);var m=Fe.makeVList({positionType:"firstBaseline",children:[{type:"elem",elem:r,wrapperClasses:["svg-align"]},{type:"kern",size:-(r.height+f)},{type:"elem",elem:l},{type:"kern",size:u}]},t);if(e.index){var b=t.havingStyle(Vt.SCRIPTSCRIPT),y=Ir(e.index,b,t),S=.6*(m.height-m.depth),w=Fe.makeVList({positionType:"shift",positionData:-S,children:[{type:"elem",elem:y}]},t),E=Fe.makeSpan(["root"],[w]);return Fe.makeSpan(["mord","sqrt"],[E,m],t)}else return Fe.makeSpan(["mord","sqrt"],[m],t)},mathmlBuilder(e,t){var{body:r,index:n}=e;return n?new tt.MathNode("mroot",[tn(r,t),tn(n,t)]):new tt.MathNode("msqrt",[tn(r,t)])}});var H9={display:Vt.DISPLAY,text:Vt.TEXT,script:Vt.SCRIPT,scriptscript:Vt.SCRIPTSCRIPT};gt({type:"styling",names:["\\displaystyle","\\textstyle","\\scriptstyle","\\scriptscriptstyle"],props:{numArgs:0,allowedInText:!0,primitive:!0},handler(e,t){var{breakOnTokenText:r,funcName:n,parser:a}=e,i=a.parseExpression(!0,r),o=n.slice(1,n.length-5);return{type:"styling",mode:a.mode,style:o,body:i}},htmlBuilder(e,t){var r=H9[e.style],n=t.havingStyle(r).withFont("");return QK(e.body,n,t)},mathmlBuilder(e,t){var r=H9[e.style],n=t.havingStyle(r),a=mi(e.body,n),i=new tt.MathNode("mstyle",a),o={display:["0","true"],text:["0","false"],script:["1","false"],scriptscript:["2","false"]},s=o[e.style];return i.setAttribute("scriptlevel",s[0]),i.setAttribute("displaystyle",s[1]),i}});var Vfe=function(t,r){var n=t.base;if(n)if(n.type==="op"){var a=n.limits&&(r.style.size===Vt.DISPLAY.size||n.alwaysHandleSupSub);return a?sp:null}else if(n.type==="operatorname"){var i=n.alwaysHandleSupSub&&(r.style.size===Vt.DISPLAY.size||n.limits);return i?ZK:null}else{if(n.type==="accent")return Gt.isCharacterBox(n.base)?hD:null;if(n.type==="horizBrace"){var o=!t.sub;return o===n.isOver?YK:null}else return null}else return null};ed({type:"supsub",htmlBuilder(e,t){var r=Vfe(e,t);if(r)return r(e,t);var{base:n,sup:a,sub:i}=e,o=Ir(n,t),s,l,u=t.fontMetrics(),d=0,p=0,f=n&&Gt.isCharacterBox(n);if(a){var m=t.havingStyle(t.style.sup());s=Ir(a,m,t),f||(d=o.height-m.fontMetrics().supDrop*m.sizeMultiplier/t.sizeMultiplier)}if(i){var b=t.havingStyle(t.style.sub());l=Ir(i,b,t),f||(p=o.depth+b.fontMetrics().subDrop*b.sizeMultiplier/t.sizeMultiplier)}var y;t.style===Vt.DISPLAY?y=u.sup1:t.style.cramped?y=u.sup3:y=u.sup2;var S=t.sizeMultiplier,w=ct(.5/u.ptPerEm/S),E=null;if(l){var k=e.base&&e.base.type==="op"&&e.base.name&&(e.base.name==="\\oiint"||e.base.name==="\\oiiint");(o instanceof oo||k)&&(E=ct(-o.italic))}var _;if(s&&l){d=Math.max(d,y,s.depth+.25*u.xHeight),p=Math.max(p,u.sub2);var R=u.defaultRuleThickness,C=4*R;if(d-s.depth-(l.height-p)<C){p=C-(d-s.depth)+l.height;var F=.8*u.xHeight-(d-s.depth);F>0&&(d+=F,p-=F)}var L=[{type:"elem",elem:l,shift:p,marginRight:w,marginLeft:E},{type:"elem",elem:s,shift:-d,marginRight:w}];_=Fe.makeVList({positionType:"individualShift",children:L},t)}else if(l){p=Math.max(p,u.sub1,l.height-.8*u.xHeight);var I=[{type:"elem",elem:l,marginLeft:E,marginRight:w}];_=Fe.makeVList({positionType:"shift",positionData:p,children:I},t)}else if(s)d=Math.max(d,y,s.depth+.25*u.xHeight),_=Fe.makeVList({positionType:"shift",positionData:-d,children:[{type:"elem",elem:s,marginRight:w}]},t);else throw new Error("supsub must have either sup or sub.");var M=C_(o,"right")||"mord";return Fe.makeSpan([M],[o,Fe.makeSpan(["msupsub"],[_])],t)},mathmlBuilder(e,t){var r=!1,n,a;e.base&&e.base.type==="horizBrace"&&(a=!!e.sup,a===e.base.isOver&&(r=!0,n=e.base.isOver)),e.base&&(e.base.type==="op"||e.base.type==="operatorname")&&(e.base.parentIsSupSub=!0);var i=[tn(e.base,t)];e.sub&&i.push(tn(e.sub,t)),e.sup&&i.push(tn(e.sup,t));var o;if(r)o=n?"mover":"munder";else if(e.sub)if(e.sup){var u=e.base;u&&u.type==="op"&&u.limits&&t.style===Vt.DISPLAY||u&&u.type==="operatorname"&&u.alwaysHandleSupSub&&(t.style===Vt.DISPLAY||u.limits)?o="munderover":o="msubsup"}else{var l=e.base;l&&l.type==="op"&&l.limits&&(t.style===Vt.DISPLAY||l.alwaysHandleSupSub)||l&&l.type==="operatorname"&&l.alwaysHandleSupSub&&(l.limits||t.style===Vt.DISPLAY)?o="munder":o="msub"}else{var s=e.base;s&&s.type==="op"&&s.limits&&(t.style===Vt.DISPLAY||s.alwaysHandleSupSub)||s&&s.type==="operatorname"&&s.alwaysHandleSupSub&&(s.limits||t.style===Vt.DISPLAY)?o="mover":o="msup"}return new tt.MathNode(o,i)}});ed({type:"atom",htmlBuilder(e,t){return Fe.mathsym(e.text,e.mode,t,["m"+e.family])},mathmlBuilder(e,t){var r=new tt.MathNode("mo",[so(e.text,e.mode)]);if(e.family==="bin"){var n=cD(e,t);n==="bold-italic"&&r.setAttribute("mathvariant",n)}else e.family==="punct"?r.setAttribute("separator","true"):(e.family==="open"||e.family==="close")&&r.setAttribute("stretchy","false");return r}});var eY={mi:"italic",mn:"normal",mtext:"normal"};ed({type:"mathord",htmlBuilder(e,t){return Fe.makeOrd(e,t,"mathord")},mathmlBuilder(e,t){var r=new tt.MathNode("mi",[so(e.text,e.mode,t)]),n=cD(e,t)||"italic";return n!==eY[r.type]&&r.setAttribute("mathvariant",n),r}});ed({type:"textord",htmlBuilder(e,t){return Fe.makeOrd(e,t,"textord")},mathmlBuilder(e,t){var r=so(e.text,e.mode,t),n=cD(e,t)||"normal",a;return e.mode==="text"?a=new tt.MathNode("mtext",[r]):/[0-9]/.test(e.text)?a=new tt.MathNode("mn",[r]):e.text==="\\prime"?a=new tt.MathNode("mo",[r]):a=new tt.MathNode("mi",[r]),n!==eY[a.type]&&a.setAttribute("mathvariant",n),a}});var Lx={"\\nobreak":"nobreak","\\allowbreak":"allowbreak"},Mx={" ":{},"\\ ":{},"~":{className:"nobreak"},"\\space":{},"\\nobreakspace":{className:"nobreak"}};ed({type:"spacing",htmlBuilder(e,t){if(Mx.hasOwnProperty(e.text)){var r=Mx[e.text].className||"";if(e.mode==="text"){var n=Fe.makeOrd(e,t,"textord");return n.classes.push(r),n}else return Fe.makeSpan(["mspace",r],[Fe.mathsym(e.text,e.mode,t)],t)}else{if(Lx.hasOwnProperty(e.text))return Fe.makeSpan(["mspace",Lx[e.text]],[],t);throw new nt('Unknown type of space "'+e.text+'"')}},mathmlBuilder(e,t){var r;if(Mx.hasOwnProperty(e.text))r=new tt.MathNode("mtext",[new tt.TextNode(" ")]);else{if(Lx.hasOwnProperty(e.text))return new tt.MathNode("mspace");throw new nt('Unknown type of space "'+e.text+'"')}return r}});var U9=()=>{var e=new tt.MathNode("mtd",[]);return e.setAttribute("width","50%"),e};ed({type:"tag",mathmlBuilder(e,t){var r=new tt.MathNode("mtable",[new tt.MathNode("mtr",[U9(),new tt.MathNode("mtd",[Au(e.body,t)]),U9(),new tt.MathNode("mtd",[Au(e.tag,t)])])]);return r.setAttribute("width","100%"),r}});var $9={"\\text":void 0,"\\textrm":"textrm","\\textsf":"textsf","\\texttt":"texttt","\\textnormal":"textrm"},G9={"\\textbf":"textbf","\\textmd":"textmd"},Kfe={"\\textit":"textit","\\textup":"textup"},j9=(e,t)=>{var r=e.font;if(r){if($9[r])return t.withTextFontFamily($9[r]);if(G9[r])return t.withTextFontWeight(G9[r]);if(r==="\\emph")return t.fontShape==="textit"?t.withTextFontShape("textup"):t.withTextFontShape("textit")}else return t;return t.withTextFontShape(Kfe[r])};gt({type:"text",names:["\\text","\\textrm","\\textsf","\\texttt","\\textnormal","\\textbf","\\textmd","\\textit","\\textup","\\emph"],props:{numArgs:1,argTypes:["text"],allowedInArgument:!0,allowedInText:!0},handler(e,t){var{parser:r,funcName:n}=e,a=t[0];return{type:"text",mode:r.mode,body:Ln(a),font:n}},htmlBuilder(e,t){var r=j9(e,t),n=ta(e.body,r,!0);return Fe.makeSpan(["mord","text"],n,r)},mathmlBuilder(e,t){var r=j9(e,t);return Au(e.body,r)}});gt({type:"underline",names:["\\underline"],props:{numArgs:1,allowedInText:!0},handler(e,t){var{parser:r}=e;return{type:"underline",mode:r.mode,body:t[0]}},htmlBuilder(e,t){var r=Ir(e.body,t),n=Fe.makeLineSpan("underline-line",t),a=t.fontMetrics().defaultRuleThickness,i=Fe.makeVList({positionType:"top",positionData:r.height,children:[{type:"kern",size:a},{type:"elem",elem:n},{type:"kern",size:3*a},{type:"elem",elem:r}]},t);return Fe.makeSpan(["mord","underline"],[i],t)},mathmlBuilder(e,t){var r=new tt.MathNode("mo",[new tt.TextNode("‾")]);r.setAttribute("stretchy","true");var n=new tt.MathNode("munder",[tn(e.body,t),r]);return n.setAttribute("accentunder","true"),n}});gt({type:"vcenter",names:["\\vcenter"],props:{numArgs:1,argTypes:["original"],allowedInText:!1},handler(e,t){var{parser:r}=e;return{type:"vcenter",mode:r.mode,body:t[0]}},htmlBuilder(e,t){var r=Ir(e.body,t),n=t.fontMetrics().axisHeight,a=.5*(r.height-n-(r.depth+n));return Fe.makeVList({positionType:"shift",positionData:a,children:[{type:"elem",elem:r}]},t)},mathmlBuilder(e,t){return new tt.MathNode("mpadded",[tn(e.body,t)],["vcenter"])}});gt({type:"verb",names:["\\verb"],props:{numArgs:0,allowedInText:!0},handler(e,t,r){throw new nt("\\verb ended by end of line instead of matching delimiter")},htmlBuilder(e,t){for(var r=q9(e),n=[],a=t.havingStyle(t.style.text()),i=0;i<r.length;i++){var o=r[i];o==="~"&&(o="\\textasciitilde"),n.push(Fe.makeSymbol(o,"Typewriter-Regular",e.mode,a,["mord","texttt"]))}return Fe.makeSpan(["mord","text"].concat(a.sizingClasses(t)),Fe.tryCombineChars(n),a)},mathmlBuilder(e,t){var r=new tt.TextNode(q9(e)),n=new tt.MathNode("mtext",[r]);return n.setAttribute("mathvariant","monospace"),n}});var q9=e=>e.body.replace(/ /g,e.star?"␣":" "),yu=wK,tY=`[ \r
	]`,Yfe="\\\\[a-zA-Z@]+",Jfe="\\\\[^\uD800-\uDFFF]",Xfe="("+Yfe+")"+tY+"*",Zfe=`\\\\(
|[ \r	]+
?)[ \r	]*`,__="[̀-ͯ]",Qfe=new RegExp(__+"+$"),eme="("+tY+"+)|"+(Zfe+"|")+"([!-\\[\\]-‧‪-퟿豈-]"+(__+"*")+"|[\uD800-\uDBFF][\uDC00-\uDFFF]"+(__+"*")+"|\\\\verb\\*([^]).*?\\4|\\\\verb([^*a-zA-Z]).*?\\5"+("|"+Xfe)+("|"+Jfe+")");class W9{constructor(t,r){this.input=void 0,this.settings=void 0,this.tokenRegex=void 0,this.catcodes=void 0,this.input=t,this.settings=r,this.tokenRegex=new RegExp(eme,"g"),this.catcodes={"%":14,"~":13}}setCatcode(t,r){this.catcodes[t]=r}lex(){var t=this.input,r=this.tokenRegex.lastIndex;if(r===t.length)return new Ii("EOF",new ui(this,r,r));var n=this.tokenRegex.exec(t);if(n===null||n.index!==r)throw new nt("Unexpected character: '"+t[r]+"'",new Ii(t[r],new ui(this,r,r+1)));var a=n[6]||n[3]||(n[2]?"\\ ":" ");if(this.catcodes[a]===14){var i=t.indexOf(`
`,this.tokenRegex.lastIndex);return i===-1?(this.tokenRegex.lastIndex=t.length,this.settings.reportNonstrict("commentAtEnd","% comment has no terminating newline; LaTeX would fail because of commenting the end of math mode (e.g. $)")):this.tokenRegex.lastIndex=i+1,this.lex()}return new Ii(a,new ui(this,r,this.tokenRegex.lastIndex))}}class tme{constructor(t,r){t===void 0&&(t={}),r===void 0&&(r={}),this.current=void 0,this.builtins=void 0,this.undefStack=void 0,this.current=r,this.builtins=t,this.undefStack=[]}beginGroup(){this.undefStack.push({})}endGroup(){if(this.undefStack.length===0)throw new nt("Unbalanced namespace destruction: attempt to pop global namespace; please report this as a bug");var t=this.undefStack.pop();for(var r in t)t.hasOwnProperty(r)&&(t[r]==null?delete this.current[r]:this.current[r]=t[r])}endGroups(){for(;this.undefStack.length>0;)this.endGroup()}has(t){return this.current.hasOwnProperty(t)||this.builtins.hasOwnProperty(t)}get(t){return this.current.hasOwnProperty(t)?this.current[t]:this.builtins[t]}set(t,r,n){if(n===void 0&&(n=!1),n){for(var a=0;a<this.undefStack.length;a++)delete this.undefStack[a][t];this.undefStack.length>0&&(this.undefStack[this.undefStack.length-1][t]=r)}else{var i=this.undefStack[this.undefStack.length-1];i&&!i.hasOwnProperty(t)&&(i[t]=this.current[t])}r==null?delete this.current[t]:this.current[t]=r}}var rme=jK;te("\\noexpand",function(e){var t=e.popToken();return e.isExpandable(t.text)&&(t.noexpand=!0,t.treatAsRelax=!0),{tokens:[t],numArgs:0}});te("\\expandafter",function(e){var t=e.popToken();return e.expandOnce(!0),{tokens:[t],numArgs:0}});te("\\@firstoftwo",function(e){var t=e.consumeArgs(2);return{tokens:t[0],numArgs:0}});te("\\@secondoftwo",function(e){var t=e.consumeArgs(2);return{tokens:t[1],numArgs:0}});te("\\@ifnextchar",function(e){var t=e.consumeArgs(3);e.consumeSpaces();var r=e.future();return t[0].length===1&&t[0][0].text===r.text?{tokens:t[1],numArgs:0}:{tokens:t[2],numArgs:0}});te("\\@ifstar","\\@ifnextchar *{\\@firstoftwo{#1}}");te("\\TextOrMath",function(e){var t=e.consumeArgs(2);return e.mode==="text"?{tokens:t[0],numArgs:0}:{tokens:t[1],numArgs:0}});var V9={0:0,1:1,2:2,3:3,4:4,5:5,6:6,7:7,8:8,9:9,a:10,A:10,b:11,B:11,c:12,C:12,d:13,D:13,e:14,E:14,f:15,F:15};te("\\char",function(e){var t=e.popToken(),r,n="";if(t.text==="'")r=8,t=e.popToken();else if(t.text==='"')r=16,t=e.popToken();else if(t.text==="`")if(t=e.popToken(),t.text[0]==="\\")n=t.text.charCodeAt(1);else{if(t.text==="EOF")throw new nt("\\char` missing argument");n=t.text.charCodeAt(0)}else r=10;if(r){if(n=V9[t.text],n==null||n>=r)throw new nt("Invalid base-"+r+" digit "+t.text);for(var a;(a=V9[e.future().text])!=null&&a<r;)n*=r,n+=a,e.popToken()}return"\\@char{"+n+"}"});var SD=(e,t,r,n)=>{var a=e.consumeArg().tokens;if(a.length!==1)throw new nt("\\newcommand's first argument must be a macro name");var i=a[0].text,o=e.isDefined(i);if(o&&!t)throw new nt("\\newcommand{"+i+"} attempting to redefine "+(i+"; use \\renewcommand"));if(!o&&!r)throw new nt("\\renewcommand{"+i+"} when command "+i+" does not yet exist; use \\newcommand");var s=0;if(a=e.consumeArg().tokens,a.length===1&&a[0].text==="["){for(var l="",u=e.expandNextToken();u.text!=="]"&&u.text!=="EOF";)l+=u.text,u=e.expandNextToken();if(!l.match(/^\s*[0-9]+\s*$/))throw new nt("Invalid number of arguments: "+l);s=parseInt(l),a=e.consumeArg().tokens}return o&&n||e.macros.set(i,{tokens:a,numArgs:s}),""};te("\\newcommand",e=>SD(e,!1,!0,!1));te("\\renewcommand",e=>SD(e,!0,!1,!1));te("\\providecommand",e=>SD(e,!0,!0,!0));te("\\message",e=>{var t=e.consumeArgs(1)[0];return console.log(t.reverse().map(r=>r.text).join("")),""});te("\\errmessage",e=>{var t=e.consumeArgs(1)[0];return console.error(t.reverse().map(r=>r.text).join("")),""});te("\\show",e=>{var t=e.popToken(),r=t.text;return console.log(t,e.macros.get(r),yu[r],sn.math[r],sn.text[r]),""});te("\\bgroup","{");te("\\egroup","}");te("~","\\nobreakspace");te("\\lq","`");te("\\rq","'");te("\\aa","\\r a");te("\\AA","\\r A");te("\\textcopyright","\\html@mathml{\\textcircled{c}}{\\char`©}");te("\\copyright","\\TextOrMath{\\textcopyright}{\\text{\\textcopyright}}");te("\\textregistered","\\html@mathml{\\textcircled{\\scriptsize R}}{\\char`®}");te("ℬ","\\mathscr{B}");te("ℰ","\\mathscr{E}");te("ℱ","\\mathscr{F}");te("ℋ","\\mathscr{H}");te("ℐ","\\mathscr{I}");te("ℒ","\\mathscr{L}");te("ℳ","\\mathscr{M}");te("ℛ","\\mathscr{R}");te("ℭ","\\mathfrak{C}");te("ℌ","\\mathfrak{H}");te("ℨ","\\mathfrak{Z}");te("\\Bbbk","\\Bbb{k}");te("·","\\cdotp");te("\\llap","\\mathllap{\\textrm{#1}}");te("\\rlap","\\mathrlap{\\textrm{#1}}");te("\\clap","\\mathclap{\\textrm{#1}}");te("\\mathstrut","\\vphantom{(}");te("\\underbar","\\underline{\\text{#1}}");te("\\not",'\\html@mathml{\\mathrel{\\mathrlap\\@not}}{\\char"338}');te("\\neq","\\html@mathml{\\mathrel{\\not=}}{\\mathrel{\\char`≠}}");te("\\ne","\\neq");te("≠","\\neq");te("\\notin","\\html@mathml{\\mathrel{{\\in}\\mathllap{/\\mskip1mu}}}{\\mathrel{\\char`∉}}");te("∉","\\notin");te("≘","\\html@mathml{\\mathrel{=\\kern{-1em}\\raisebox{0.4em}{$\\scriptsize\\frown$}}}{\\mathrel{\\char`≘}}");te("≙","\\html@mathml{\\stackrel{\\tiny\\wedge}{=}}{\\mathrel{\\char`≘}}");te("≚","\\html@mathml{\\stackrel{\\tiny\\vee}{=}}{\\mathrel{\\char`≚}}");te("≛","\\html@mathml{\\stackrel{\\scriptsize\\star}{=}}{\\mathrel{\\char`≛}}");te("≝","\\html@mathml{\\stackrel{\\tiny\\mathrm{def}}{=}}{\\mathrel{\\char`≝}}");te("≞","\\html@mathml{\\stackrel{\\tiny\\mathrm{m}}{=}}{\\mathrel{\\char`≞}}");te("≟","\\html@mathml{\\stackrel{\\tiny?}{=}}{\\mathrel{\\char`≟}}");te("⟂","\\perp");te("‼","\\mathclose{!\\mkern-0.8mu!}");te("∌","\\notni");te("⌜","\\ulcorner");te("⌝","\\urcorner");te("⌞","\\llcorner");te("⌟","\\lrcorner");te("©","\\copyright");te("®","\\textregistered");te("️","\\textregistered");te("\\ulcorner",'\\html@mathml{\\@ulcorner}{\\mathop{\\char"231c}}');te("\\urcorner",'\\html@mathml{\\@urcorner}{\\mathop{\\char"231d}}');te("\\llcorner",'\\html@mathml{\\@llcorner}{\\mathop{\\char"231e}}');te("\\lrcorner",'\\html@mathml{\\@lrcorner}{\\mathop{\\char"231f}}');te("\\vdots","{\\varvdots\\rule{0pt}{15pt}}");te("⋮","\\vdots");te("\\varGamma","\\mathit{\\Gamma}");te("\\varDelta","\\mathit{\\Delta}");te("\\varTheta","\\mathit{\\Theta}");te("\\varLambda","\\mathit{\\Lambda}");te("\\varXi","\\mathit{\\Xi}");te("\\varPi","\\mathit{\\Pi}");te("\\varSigma","\\mathit{\\Sigma}");te("\\varUpsilon","\\mathit{\\Upsilon}");te("\\varPhi","\\mathit{\\Phi}");te("\\varPsi","\\mathit{\\Psi}");te("\\varOmega","\\mathit{\\Omega}");te("\\substack","\\begin{subarray}{c}#1\\end{subarray}");te("\\colon","\\nobreak\\mskip2mu\\mathpunct{}\\mathchoice{\\mkern-3mu}{\\mkern-3mu}{}{}{:}\\mskip6mu\\relax");te("\\boxed","\\fbox{$\\displaystyle{#1}$}");te("\\iff","\\DOTSB\\;\\Longleftrightarrow\\;");te("\\implies","\\DOTSB\\;\\Longrightarrow\\;");te("\\impliedby","\\DOTSB\\;\\Longleftarrow\\;");te("\\dddot","{\\overset{\\raisebox{-0.1ex}{\\normalsize ...}}{#1}}");te("\\ddddot","{\\overset{\\raisebox{-0.1ex}{\\normalsize ....}}{#1}}");var K9={",":"\\dotsc","\\not":"\\dotsb","+":"\\dotsb","=":"\\dotsb","<":"\\dotsb",">":"\\dotsb","-":"\\dotsb","*":"\\dotsb",":":"\\dotsb","\\DOTSB":"\\dotsb","\\coprod":"\\dotsb","\\bigvee":"\\dotsb","\\bigwedge":"\\dotsb","\\biguplus":"\\dotsb","\\bigcap":"\\dotsb","\\bigcup":"\\dotsb","\\prod":"\\dotsb","\\sum":"\\dotsb","\\bigotimes":"\\dotsb","\\bigoplus":"\\dotsb","\\bigodot":"\\dotsb","\\bigsqcup":"\\dotsb","\\And":"\\dotsb","\\longrightarrow":"\\dotsb","\\Longrightarrow":"\\dotsb","\\longleftarrow":"\\dotsb","\\Longleftarrow":"\\dotsb","\\longleftrightarrow":"\\dotsb","\\Longleftrightarrow":"\\dotsb","\\mapsto":"\\dotsb","\\longmapsto":"\\dotsb","\\hookrightarrow":"\\dotsb","\\doteq":"\\dotsb","\\mathbin":"\\dotsb","\\mathrel":"\\dotsb","\\relbar":"\\dotsb","\\Relbar":"\\dotsb","\\xrightarrow":"\\dotsb","\\xleftarrow":"\\dotsb","\\DOTSI":"\\dotsi","\\int":"\\dotsi","\\oint":"\\dotsi","\\iint":"\\dotsi","\\iiint":"\\dotsi","\\iiiint":"\\dotsi","\\idotsint":"\\dotsi","\\DOTSX":"\\dotsx"};te("\\dots",function(e){var t="\\dotso",r=e.expandAfterFuture().text;return r in K9?t=K9[r]:(r.slice(0,4)==="\\not"||r in sn.math&&Gt.contains(["bin","rel"],sn.math[r].group))&&(t="\\dotsb"),t});var ED={")":!0,"]":!0,"\\rbrack":!0,"\\}":!0,"\\rbrace":!0,"\\rangle":!0,"\\rceil":!0,"\\rfloor":!0,"\\rgroup":!0,"\\rmoustache":!0,"\\right":!0,"\\bigr":!0,"\\biggr":!0,"\\Bigr":!0,"\\Biggr":!0,$:!0,";":!0,".":!0,",":!0};te("\\dotso",function(e){var t=e.future().text;return t in ED?"\\ldots\\,":"\\ldots"});te("\\dotsc",function(e){var t=e.future().text;return t in ED&&t!==","?"\\ldots\\,":"\\ldots"});te("\\cdots",function(e){var t=e.future().text;return t in ED?"\\@cdots\\,":"\\@cdots"});te("\\dotsb","\\cdots");te("\\dotsm","\\cdots");te("\\dotsi","\\!\\cdots");te("\\dotsx","\\ldots\\,");te("\\DOTSI","\\relax");te("\\DOTSB","\\relax");te("\\DOTSX","\\relax");te("\\tmspace","\\TextOrMath{\\kern#1#3}{\\mskip#1#2}\\relax");te("\\,","\\tmspace+{3mu}{.1667em}");te("\\thinspace","\\,");te("\\>","\\mskip{4mu}");te("\\:","\\tmspace+{4mu}{.2222em}");te("\\medspace","\\:");te("\\;","\\tmspace+{5mu}{.2777em}");te("\\thickspace","\\;");te("\\!","\\tmspace-{3mu}{.1667em}");te("\\negthinspace","\\!");te("\\negmedspace","\\tmspace-{4mu}{.2222em}");te("\\negthickspace","\\tmspace-{5mu}{.277em}");te("\\enspace","\\kern.5em ");te("\\enskip","\\hskip.5em\\relax");te("\\quad","\\hskip1em\\relax");te("\\qquad","\\hskip2em\\relax");te("\\tag","\\@ifstar\\tag@literal\\tag@paren");te("\\tag@paren","\\tag@literal{({#1})}");te("\\tag@literal",e=>{if(e.macros.get("\\df@tag"))throw new nt("Multiple \\tag");return"\\gdef\\df@tag{\\text{#1}}"});te("\\bmod","\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}\\mathbin{\\rm mod}\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}");te("\\pod","\\allowbreak\\mathchoice{\\mkern18mu}{\\mkern8mu}{\\mkern8mu}{\\mkern8mu}(#1)");te("\\pmod","\\pod{{\\rm mod}\\mkern6mu#1}");te("\\mod","\\allowbreak\\mathchoice{\\mkern18mu}{\\mkern12mu}{\\mkern12mu}{\\mkern12mu}{\\rm mod}\\,\\,#1");te("\\newline","\\\\\\relax");te("\\TeX","\\textrm{\\html@mathml{T\\kern-.1667em\\raisebox{-.5ex}{E}\\kern-.125emX}{TeX}}");var rY=ct(as["Main-Regular"][84][1]-.7*as["Main-Regular"][65][1]);te("\\LaTeX","\\textrm{\\html@mathml{"+("L\\kern-.36em\\raisebox{"+rY+"}{\\scriptstyle A}")+"\\kern-.15em\\TeX}{LaTeX}}");te("\\KaTeX","\\textrm{\\html@mathml{"+("K\\kern-.17em\\raisebox{"+rY+"}{\\scriptstyle A}")+"\\kern-.15em\\TeX}{KaTeX}}");te("\\hspace","\\@ifstar\\@hspacer\\@hspace");te("\\@hspace","\\hskip #1\\relax");te("\\@hspacer","\\rule{0pt}{0pt}\\hskip #1\\relax");te("\\ordinarycolon",":");te("\\vcentcolon","\\mathrel{\\mathop\\ordinarycolon}");te("\\dblcolon",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-.9mu}\\vcentcolon}}{\\mathop{\\char"2237}}');te("\\coloneqq",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}=}}{\\mathop{\\char"2254}}');te("\\Coloneqq",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}=}}{\\mathop{\\char"2237\\char"3d}}');te("\\coloneq",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}{\\mathop{\\char"3a\\char"2212}}');te("\\Coloneq",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}{\\mathop{\\char"2237\\char"2212}}');te("\\eqqcolon",'\\html@mathml{\\mathrel{=\\mathrel{\\mkern-1.2mu}\\vcentcolon}}{\\mathop{\\char"2255}}');te("\\Eqqcolon",'\\html@mathml{\\mathrel{=\\mathrel{\\mkern-1.2mu}\\dblcolon}}{\\mathop{\\char"3d\\char"2237}}');te("\\eqcolon",'\\html@mathml{\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\vcentcolon}}{\\mathop{\\char"2239}}');te("\\Eqcolon",'\\html@mathml{\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\dblcolon}}{\\mathop{\\char"2212\\char"2237}}');te("\\colonapprox",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\approx}}{\\mathop{\\char"3a\\char"2248}}');te("\\Colonapprox",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\approx}}{\\mathop{\\char"2237\\char"2248}}');te("\\colonsim",'\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\sim}}{\\mathop{\\char"3a\\char"223c}}');te("\\Colonsim",'\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\sim}}{\\mathop{\\char"2237\\char"223c}}');te("∷","\\dblcolon");te("∹","\\eqcolon");te("≔","\\coloneqq");te("≕","\\eqqcolon");te("⩴","\\Coloneqq");te("\\ratio","\\vcentcolon");te("\\coloncolon","\\dblcolon");te("\\colonequals","\\coloneqq");te("\\coloncolonequals","\\Coloneqq");te("\\equalscolon","\\eqqcolon");te("\\equalscoloncolon","\\Eqqcolon");te("\\colonminus","\\coloneq");te("\\coloncolonminus","\\Coloneq");te("\\minuscolon","\\eqcolon");te("\\minuscoloncolon","\\Eqcolon");te("\\coloncolonapprox","\\Colonapprox");te("\\coloncolonsim","\\Colonsim");te("\\simcolon","\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\vcentcolon}");te("\\simcoloncolon","\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\dblcolon}");te("\\approxcolon","\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\vcentcolon}");te("\\approxcoloncolon","\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\dblcolon}");te("\\notni","\\html@mathml{\\not\\ni}{\\mathrel{\\char`∌}}");te("\\limsup","\\DOTSB\\operatorname*{lim\\,sup}");te("\\liminf","\\DOTSB\\operatorname*{lim\\,inf}");te("\\injlim","\\DOTSB\\operatorname*{inj\\,lim}");te("\\projlim","\\DOTSB\\operatorname*{proj\\,lim}");te("\\varlimsup","\\DOTSB\\operatorname*{\\overline{lim}}");te("\\varliminf","\\DOTSB\\operatorname*{\\underline{lim}}");te("\\varinjlim","\\DOTSB\\operatorname*{\\underrightarrow{lim}}");te("\\varprojlim","\\DOTSB\\operatorname*{\\underleftarrow{lim}}");te("\\gvertneqq","\\html@mathml{\\@gvertneqq}{≩}");te("\\lvertneqq","\\html@mathml{\\@lvertneqq}{≨}");te("\\ngeqq","\\html@mathml{\\@ngeqq}{≱}");te("\\ngeqslant","\\html@mathml{\\@ngeqslant}{≱}");te("\\nleqq","\\html@mathml{\\@nleqq}{≰}");te("\\nleqslant","\\html@mathml{\\@nleqslant}{≰}");te("\\nshortmid","\\html@mathml{\\@nshortmid}{∤}");te("\\nshortparallel","\\html@mathml{\\@nshortparallel}{∦}");te("\\nsubseteqq","\\html@mathml{\\@nsubseteqq}{⊈}");te("\\nsupseteqq","\\html@mathml{\\@nsupseteqq}{⊉}");te("\\varsubsetneq","\\html@mathml{\\@varsubsetneq}{⊊}");te("\\varsubsetneqq","\\html@mathml{\\@varsubsetneqq}{⫋}");te("\\varsupsetneq","\\html@mathml{\\@varsupsetneq}{⊋}");te("\\varsupsetneqq","\\html@mathml{\\@varsupsetneqq}{⫌}");te("\\imath","\\html@mathml{\\@imath}{ı}");te("\\jmath","\\html@mathml{\\@jmath}{ȷ}");te("\\llbracket","\\html@mathml{\\mathopen{[\\mkern-3.2mu[}}{\\mathopen{\\char`⟦}}");te("\\rrbracket","\\html@mathml{\\mathclose{]\\mkern-3.2mu]}}{\\mathclose{\\char`⟧}}");te("⟦","\\llbracket");te("⟧","\\rrbracket");te("\\lBrace","\\html@mathml{\\mathopen{\\{\\mkern-3.2mu[}}{\\mathopen{\\char`⦃}}");te("\\rBrace","\\html@mathml{\\mathclose{]\\mkern-3.2mu\\}}}{\\mathclose{\\char`⦄}}");te("⦃","\\lBrace");te("⦄","\\rBrace");te("\\minuso","\\mathbin{\\html@mathml{{\\mathrlap{\\mathchoice{\\kern{0.145em}}{\\kern{0.145em}}{\\kern{0.1015em}}{\\kern{0.0725em}}\\circ}{-}}}{\\char`⦵}}");te("⦵","\\minuso");te("\\darr","\\downarrow");te("\\dArr","\\Downarrow");te("\\Darr","\\Downarrow");te("\\lang","\\langle");te("\\rang","\\rangle");te("\\uarr","\\uparrow");te("\\uArr","\\Uparrow");te("\\Uarr","\\Uparrow");te("\\N","\\mathbb{N}");te("\\R","\\mathbb{R}");te("\\Z","\\mathbb{Z}");te("\\alef","\\aleph");te("\\alefsym","\\aleph");te("\\Alpha","\\mathrm{A}");te("\\Beta","\\mathrm{B}");te("\\bull","\\bullet");te("\\Chi","\\mathrm{X}");te("\\clubs","\\clubsuit");te("\\cnums","\\mathbb{C}");te("\\Complex","\\mathbb{C}");te("\\Dagger","\\ddagger");te("\\diamonds","\\diamondsuit");te("\\empty","\\emptyset");te("\\Epsilon","\\mathrm{E}");te("\\Eta","\\mathrm{H}");te("\\exist","\\exists");te("\\harr","\\leftrightarrow");te("\\hArr","\\Leftrightarrow");te("\\Harr","\\Leftrightarrow");te("\\hearts","\\heartsuit");te("\\image","\\Im");te("\\infin","\\infty");te("\\Iota","\\mathrm{I}");te("\\isin","\\in");te("\\Kappa","\\mathrm{K}");te("\\larr","\\leftarrow");te("\\lArr","\\Leftarrow");te("\\Larr","\\Leftarrow");te("\\lrarr","\\leftrightarrow");te("\\lrArr","\\Leftrightarrow");te("\\Lrarr","\\Leftrightarrow");te("\\Mu","\\mathrm{M}");te("\\natnums","\\mathbb{N}");te("\\Nu","\\mathrm{N}");te("\\Omicron","\\mathrm{O}");te("\\plusmn","\\pm");te("\\rarr","\\rightarrow");te("\\rArr","\\Rightarrow");te("\\Rarr","\\Rightarrow");te("\\real","\\Re");te("\\reals","\\mathbb{R}");te("\\Reals","\\mathbb{R}");te("\\Rho","\\mathrm{P}");te("\\sdot","\\cdot");te("\\sect","\\S");te("\\spades","\\spadesuit");te("\\sub","\\subset");te("\\sube","\\subseteq");te("\\supe","\\supseteq");te("\\Tau","\\mathrm{T}");te("\\thetasym","\\vartheta");te("\\weierp","\\wp");te("\\Zeta","\\mathrm{Z}");te("\\argmin","\\DOTSB\\operatorname*{arg\\,min}");te("\\argmax","\\DOTSB\\operatorname*{arg\\,max}");te("\\plim","\\DOTSB\\mathop{\\operatorname{plim}}\\limits");te("\\bra","\\mathinner{\\langle{#1}|}");te("\\ket","\\mathinner{|{#1}\\rangle}");te("\\braket","\\mathinner{\\langle{#1}\\rangle}");te("\\Bra","\\left\\langle#1\\right|");te("\\Ket","\\left|#1\\right\\rangle");var nY=e=>t=>{var r=t.consumeArg().tokens,n=t.consumeArg().tokens,a=t.consumeArg().tokens,i=t.consumeArg().tokens,o=t.macros.get("|"),s=t.macros.get("\\|");t.macros.beginGroup();var l=p=>f=>{e&&(f.macros.set("|",o),a.length&&f.macros.set("\\|",s));var m=p;if(!p&&a.length){var b=f.future();b.text==="|"&&(f.popToken(),m=!0)}return{tokens:m?a:n,numArgs:0}};t.macros.set("|",l(!1)),a.length&&t.macros.set("\\|",l(!0));var u=t.consumeArg().tokens,d=t.expandTokens([...i,...u,...r]);return t.macros.endGroup(),{tokens:d.reverse(),numArgs:0}};te("\\bra@ket",nY(!1));te("\\bra@set",nY(!0));te("\\Braket","\\bra@ket{\\left\\langle}{\\,\\middle\\vert\\,}{\\,\\middle\\vert\\,}{\\right\\rangle}");te("\\Set","\\bra@set{\\left\\{\\:}{\\;\\middle\\vert\\;}{\\;\\middle\\Vert\\;}{\\:\\right\\}}");te("\\set","\\bra@set{\\{\\,}{\\mid}{}{\\,\\}}");te("\\angln","{\\angl n}");te("\\blue","\\textcolor{##6495ed}{#1}");te("\\orange","\\textcolor{##ffa500}{#1}");te("\\pink","\\textcolor{##ff00af}{#1}");te("\\red","\\textcolor{##df0030}{#1}");te("\\green","\\textcolor{##28ae7b}{#1}");te("\\gray","\\textcolor{gray}{#1}");te("\\purple","\\textcolor{##9d38bd}{#1}");te("\\blueA","\\textcolor{##ccfaff}{#1}");te("\\blueB","\\textcolor{##80f6ff}{#1}");te("\\blueC","\\textcolor{##63d9ea}{#1}");te("\\blueD","\\textcolor{##11accd}{#1}");te("\\blueE","\\textcolor{##0c7f99}{#1}");te("\\tealA","\\textcolor{##94fff5}{#1}");te("\\tealB","\\textcolor{##26edd5}{#1}");te("\\tealC","\\textcolor{##01d1c1}{#1}");te("\\tealD","\\textcolor{##01a995}{#1}");te("\\tealE","\\textcolor{##208170}{#1}");te("\\greenA","\\textcolor{##b6ffb0}{#1}");te("\\greenB","\\textcolor{##8af281}{#1}");te("\\greenC","\\textcolor{##74cf70}{#1}");te("\\greenD","\\textcolor{##1fab54}{#1}");te("\\greenE","\\textcolor{##0d923f}{#1}");te("\\goldA","\\textcolor{##ffd0a9}{#1}");te("\\goldB","\\textcolor{##ffbb71}{#1}");te("\\goldC","\\textcolor{##ff9c39}{#1}");te("\\goldD","\\textcolor{##e07d10}{#1}");te("\\goldE","\\textcolor{##a75a05}{#1}");te("\\redA","\\textcolor{##fca9a9}{#1}");te("\\redB","\\textcolor{##ff8482}{#1}");te("\\redC","\\textcolor{##f9685d}{#1}");te("\\redD","\\textcolor{##e84d39}{#1}");te("\\redE","\\textcolor{##bc2612}{#1}");te("\\maroonA","\\textcolor{##ffbde0}{#1}");te("\\maroonB","\\textcolor{##ff92c6}{#1}");te("\\maroonC","\\textcolor{##ed5fa6}{#1}");te("\\maroonD","\\textcolor{##ca337c}{#1}");te("\\maroonE","\\textcolor{##9e034e}{#1}");te("\\purpleA","\\textcolor{##ddd7ff}{#1}");te("\\purpleB","\\textcolor{##c6b9fc}{#1}");te("\\purpleC","\\textcolor{##aa87ff}{#1}");te("\\purpleD","\\textcolor{##7854ab}{#1}");te("\\purpleE","\\textcolor{##543b78}{#1}");te("\\mintA","\\textcolor{##f5f9e8}{#1}");te("\\mintB","\\textcolor{##edf2df}{#1}");te("\\mintC","\\textcolor{##e0e5cc}{#1}");te("\\grayA","\\textcolor{##f6f7f7}{#1}");te("\\grayB","\\textcolor{##f0f1f2}{#1}");te("\\grayC","\\textcolor{##e3e5e6}{#1}");te("\\grayD","\\textcolor{##d6d8da}{#1}");te("\\grayE","\\textcolor{##babec2}{#1}");te("\\grayF","\\textcolor{##888d93}{#1}");te("\\grayG","\\textcolor{##626569}{#1}");te("\\grayH","\\textcolor{##3b3e40}{#1}");te("\\grayI","\\textcolor{##21242c}{#1}");te("\\kaBlue","\\textcolor{##314453}{#1}");te("\\kaGreen","\\textcolor{##71B307}{#1}");var aY={"^":!0,_:!0,"\\limits":!0,"\\nolimits":!0};class nme{constructor(t,r,n){this.settings=void 0,this.expansionCount=void 0,this.lexer=void 0,this.macros=void 0,this.stack=void 0,this.mode=void 0,this.settings=r,this.expansionCount=0,this.feed(t),this.macros=new tme(rme,r.macros),this.mode=n,this.stack=[]}feed(t){this.lexer=new W9(t,this.settings)}switchMode(t){this.mode=t}beginGroup(){this.macros.beginGroup()}endGroup(){this.macros.endGroup()}endGroups(){this.macros.endGroups()}future(){return this.stack.length===0&&this.pushToken(this.lexer.lex()),this.stack[this.stack.length-1]}popToken(){return this.future(),this.stack.pop()}pushToken(t){this.stack.push(t)}pushTokens(t){this.stack.push(...t)}scanArgument(t){var r,n,a;if(t){if(this.consumeSpaces(),this.future().text!=="[")return null;r=this.popToken(),{tokens:a,end:n}=this.consumeArg(["]"])}else({tokens:a,start:r,end:n}=this.consumeArg());return this.pushToken(new Ii("EOF",n.loc)),this.pushTokens(a),new Ii("",ui.range(r,n))}consumeSpaces(){for(;;){var t=this.future();if(t.text===" ")this.stack.pop();else break}}consumeArg(t){var r=[],n=t&&t.length>0;n||this.consumeSpaces();var a=this.future(),i,o=0,s=0;do{if(i=this.popToken(),r.push(i),i.text==="{")++o;else if(i.text==="}"){if(--o,o===-1)throw new nt("Extra }",i)}else if(i.text==="EOF")throw new nt("Unexpected end of input in a macro argument, expected '"+(t&&n?t[s]:"}")+"'",i);if(t&&n)if((o===0||o===1&&t[s]==="{")&&i.text===t[s]){if(++s,s===t.length){r.splice(-s,s);break}}else s=0}while(o!==0||n);return a.text==="{"&&r[r.length-1].text==="}"&&(r.pop(),r.shift()),r.reverse(),{tokens:r,start:a,end:i}}consumeArgs(t,r){if(r){if(r.length!==t+1)throw new nt("The length of delimiters doesn't match the number of args!");for(var n=r[0],a=0;a<n.length;a++){var i=this.popToken();if(n[a]!==i.text)throw new nt("Use of the macro doesn't match its definition",i)}}for(var o=[],s=0;s<t;s++)o.push(this.consumeArg(r&&r[s+1]).tokens);return o}countExpansion(t){if(this.expansionCount+=t,this.expansionCount>this.settings.maxExpand)throw new nt("Too many expansions: infinite loop or need to increase maxExpand setting")}expandOnce(t){var r=this.popToken(),n=r.text,a=r.noexpand?null:this._getExpansion(n);if(a==null||t&&a.unexpandable){if(t&&a==null&&n[0]==="\\"&&!this.isDefined(n))throw new nt("Undefined control sequence: "+n);return this.pushToken(r),!1}this.countExpansion(1);var i=a.tokens,o=this.consumeArgs(a.numArgs,a.delimiters);if(a.numArgs){i=i.slice();for(var s=i.length-1;s>=0;--s){var l=i[s];if(l.text==="#"){if(s===0)throw new nt("Incomplete placeholder at end of macro body",l);if(l=i[--s],l.text==="#")i.splice(s+1,1);else if(/^[1-9]$/.test(l.text))i.splice(s,2,...o[+l.text-1]);else throw new nt("Not a valid argument number",l)}}}return this.pushTokens(i),i.length}expandAfterFuture(){return this.expandOnce(),this.future()}expandNextToken(){for(;;)if(this.expandOnce()===!1){var t=this.stack.pop();return t.treatAsRelax&&(t.text="\\relax"),t}throw new Error}expandMacro(t){return this.macros.has(t)?this.expandTokens([new Ii(t)]):void 0}expandTokens(t){var r=[],n=this.stack.length;for(this.pushTokens(t);this.stack.length>n;)if(this.expandOnce(!0)===!1){var a=this.stack.pop();a.treatAsRelax&&(a.noexpand=!1,a.treatAsRelax=!1),r.push(a)}return this.countExpansion(r.length),r}expandMacroAsText(t){var r=this.expandMacro(t);return r&&r.map(n=>n.text).join("")}_getExpansion(t){var r=this.macros.get(t);if(r==null)return r;if(t.length===1){var n=this.lexer.catcodes[t];if(n!=null&&n!==13)return}var a=typeof r=="function"?r(this):r;if(typeof a=="string"){var i=0;if(a.indexOf("#")!==-1)for(var o=a.replace(/##/g,"");o.indexOf("#"+(i+1))!==-1;)++i;for(var s=new W9(a,this.settings),l=[],u=s.lex();u.text!=="EOF";)l.push(u),u=s.lex();l.reverse();var d={tokens:l,numArgs:i};return d}return a}isDefined(t){return this.macros.has(t)||yu.hasOwnProperty(t)||sn.math.hasOwnProperty(t)||sn.text.hasOwnProperty(t)||aY.hasOwnProperty(t)}isExpandable(t){var r=this.macros.get(t);return r!=null?typeof r=="string"||typeof r=="function"||!r.unexpandable:yu.hasOwnProperty(t)&&!yu[t].primitive}}var Y9=/^[₊₋₌₍₎₀₁₂₃₄₅₆₇₈₉ₐₑₕᵢⱼₖₗₘₙₒₚᵣₛₜᵤᵥₓᵦᵧᵨᵩᵪ]/,vb=Object.freeze({"₊":"+","₋":"-","₌":"=","₍":"(","₎":")","₀":"0","₁":"1","₂":"2","₃":"3","₄":"4","₅":"5","₆":"6","₇":"7","₈":"8","₉":"9","ₐ":"a","ₑ":"e","ₕ":"h","ᵢ":"i","ⱼ":"j","ₖ":"k","ₗ":"l","ₘ":"m","ₙ":"n","ₒ":"o","ₚ":"p","ᵣ":"r","ₛ":"s","ₜ":"t","ᵤ":"u","ᵥ":"v","ₓ":"x","ᵦ":"β","ᵧ":"γ","ᵨ":"ρ","ᵩ":"ϕ","ᵪ":"χ","⁺":"+","⁻":"-","⁼":"=","⁽":"(","⁾":")","⁰":"0","¹":"1","²":"2","³":"3","⁴":"4","⁵":"5","⁶":"6","⁷":"7","⁸":"8","⁹":"9","ᴬ":"A","ᴮ":"B","ᴰ":"D","ᴱ":"E","ᴳ":"G","ᴴ":"H","ᴵ":"I","ᴶ":"J","ᴷ":"K","ᴸ":"L","ᴹ":"M","ᴺ":"N","ᴼ":"O","ᴾ":"P","ᴿ":"R","ᵀ":"T","ᵁ":"U","ⱽ":"V","ᵂ":"W","ᵃ":"a","ᵇ":"b","ᶜ":"c","ᵈ":"d","ᵉ":"e","ᶠ":"f","ᵍ":"g",ʰ:"h","ⁱ":"i",ʲ:"j","ᵏ":"k",ˡ:"l","ᵐ":"m",ⁿ:"n","ᵒ":"o","ᵖ":"p",ʳ:"r",ˢ:"s","ᵗ":"t","ᵘ":"u","ᵛ":"v",ʷ:"w",ˣ:"x",ʸ:"y","ᶻ":"z","ᵝ":"β","ᵞ":"γ","ᵟ":"δ","ᵠ":"ϕ","ᵡ":"χ","ᶿ":"θ"}),Ix={"́":{text:"\\'",math:"\\acute"},"̀":{text:"\\`",math:"\\grave"},"̈":{text:'\\"',math:"\\ddot"},"̃":{text:"\\~",math:"\\tilde"},"̄":{text:"\\=",math:"\\bar"},"̆":{text:"\\u",math:"\\breve"},"̌":{text:"\\v",math:"\\check"},"̂":{text:"\\^",math:"\\hat"},"̇":{text:"\\.",math:"\\dot"},"̊":{text:"\\r",math:"\\mathring"},"̋":{text:"\\H"},"̧":{text:"\\c"}},J9={á:"á",à:"à",ä:"ä",ǟ:"ǟ",ã:"ã",ā:"ā",ă:"ă",ắ:"ắ",ằ:"ằ",ẵ:"ẵ",ǎ:"ǎ",â:"â",ấ:"ấ",ầ:"ầ",ẫ:"ẫ",ȧ:"ȧ",ǡ:"ǡ",å:"å",ǻ:"ǻ",ḃ:"ḃ",ć:"ć",ḉ:"ḉ",č:"č",ĉ:"ĉ",ċ:"ċ",ç:"ç",ď:"ď",ḋ:"ḋ",ḑ:"ḑ",é:"é",è:"è",ë:"ë",ẽ:"ẽ",ē:"ē",ḗ:"ḗ",ḕ:"ḕ",ĕ:"ĕ",ḝ:"ḝ",ě:"ě",ê:"ê",ế:"ế",ề:"ề",ễ:"ễ",ė:"ė",ȩ:"ȩ",ḟ:"ḟ",ǵ:"ǵ",ḡ:"ḡ",ğ:"ğ",ǧ:"ǧ",ĝ:"ĝ",ġ:"ġ",ģ:"ģ",ḧ:"ḧ",ȟ:"ȟ",ĥ:"ĥ",ḣ:"ḣ",ḩ:"ḩ",í:"í",ì:"ì",ï:"ï",ḯ:"ḯ",ĩ:"ĩ",ī:"ī",ĭ:"ĭ",ǐ:"ǐ",î:"î",ǰ:"ǰ",ĵ:"ĵ",ḱ:"ḱ",ǩ:"ǩ",ķ:"ķ",ĺ:"ĺ",ľ:"ľ",ļ:"ļ",ḿ:"ḿ",ṁ:"ṁ",ń:"ń",ǹ:"ǹ",ñ:"ñ",ň:"ň",ṅ:"ṅ",ņ:"ņ",ó:"ó",ò:"ò",ö:"ö",ȫ:"ȫ",õ:"õ",ṍ:"ṍ",ṏ:"ṏ",ȭ:"ȭ",ō:"ō",ṓ:"ṓ",ṑ:"ṑ",ŏ:"ŏ",ǒ:"ǒ",ô:"ô",ố:"ố",ồ:"ồ",ỗ:"ỗ",ȯ:"ȯ",ȱ:"ȱ",ő:"ő",ṕ:"ṕ",ṗ:"ṗ",ŕ:"ŕ",ř:"ř",ṙ:"ṙ",ŗ:"ŗ",ś:"ś",ṥ:"ṥ",š:"š",ṧ:"ṧ",ŝ:"ŝ",ṡ:"ṡ",ş:"ş",ẗ:"ẗ",ť:"ť",ṫ:"ṫ",ţ:"ţ",ú:"ú",ù:"ù",ü:"ü",ǘ:"ǘ",ǜ:"ǜ",ǖ:"ǖ",ǚ:"ǚ",ũ:"ũ",ṹ:"ṹ",ū:"ū",ṻ:"ṻ",ŭ:"ŭ",ǔ:"ǔ",û:"û",ů:"ů",ű:"ű",ṽ:"ṽ",ẃ:"ẃ",ẁ:"ẁ",ẅ:"ẅ",ŵ:"ŵ",ẇ:"ẇ",ẘ:"ẘ",ẍ:"ẍ",ẋ:"ẋ",ý:"ý",ỳ:"ỳ",ÿ:"ÿ",ỹ:"ỹ",ȳ:"ȳ",ŷ:"ŷ",ẏ:"ẏ",ẙ:"ẙ",ź:"ź",ž:"ž",ẑ:"ẑ",ż:"ż",Á:"Á",À:"À",Ä:"Ä",Ǟ:"Ǟ",Ã:"Ã",Ā:"Ā",Ă:"Ă",Ắ:"Ắ",Ằ:"Ằ",Ẵ:"Ẵ",Ǎ:"Ǎ",Â:"Â",Ấ:"Ấ",Ầ:"Ầ",Ẫ:"Ẫ",Ȧ:"Ȧ",Ǡ:"Ǡ",Å:"Å",Ǻ:"Ǻ",Ḃ:"Ḃ",Ć:"Ć",Ḉ:"Ḉ",Č:"Č",Ĉ:"Ĉ",Ċ:"Ċ",Ç:"Ç",Ď:"Ď",Ḋ:"Ḋ",Ḑ:"Ḑ",É:"É",È:"È",Ë:"Ë",Ẽ:"Ẽ",Ē:"Ē",Ḗ:"Ḗ",Ḕ:"Ḕ",Ĕ:"Ĕ",Ḝ:"Ḝ",Ě:"Ě",Ê:"Ê",Ế:"Ế",Ề:"Ề",Ễ:"Ễ",Ė:"Ė",Ȩ:"Ȩ",Ḟ:"Ḟ",Ǵ:"Ǵ",Ḡ:"Ḡ",Ğ:"Ğ",Ǧ:"Ǧ",Ĝ:"Ĝ",Ġ:"Ġ",Ģ:"Ģ",Ḧ:"Ḧ",Ȟ:"Ȟ",Ĥ:"Ĥ",Ḣ:"Ḣ",Ḩ:"Ḩ",Í:"Í",Ì:"Ì",Ï:"Ï",Ḯ:"Ḯ",Ĩ:"Ĩ",Ī:"Ī",Ĭ:"Ĭ",Ǐ:"Ǐ",Î:"Î",İ:"İ",Ĵ:"Ĵ",Ḱ:"Ḱ",Ǩ:"Ǩ",Ķ:"Ķ",Ĺ:"Ĺ",Ľ:"Ľ",Ļ:"Ļ",Ḿ:"Ḿ",Ṁ:"Ṁ",Ń:"Ń",Ǹ:"Ǹ",Ñ:"Ñ",Ň:"Ň",Ṅ:"Ṅ",Ņ:"Ņ",Ó:"Ó",Ò:"Ò",Ö:"Ö",Ȫ:"Ȫ",Õ:"Õ",Ṍ:"Ṍ",Ṏ:"Ṏ",Ȭ:"Ȭ",Ō:"Ō",Ṓ:"Ṓ",Ṑ:"Ṑ",Ŏ:"Ŏ",Ǒ:"Ǒ",Ô:"Ô",Ố:"Ố",Ồ:"Ồ",Ỗ:"Ỗ",Ȯ:"Ȯ",Ȱ:"Ȱ",Ő:"Ő",Ṕ:"Ṕ",Ṗ:"Ṗ",Ŕ:"Ŕ",Ř:"Ř",Ṙ:"Ṙ",Ŗ:"Ŗ",Ś:"Ś",Ṥ:"Ṥ",Š:"Š",Ṧ:"Ṧ",Ŝ:"Ŝ",Ṡ:"Ṡ",Ş:"Ş",Ť:"Ť",Ṫ:"Ṫ",Ţ:"Ţ",Ú:"Ú",Ù:"Ù",Ü:"Ü",Ǘ:"Ǘ",Ǜ:"Ǜ",Ǖ:"Ǖ",Ǚ:"Ǚ",Ũ:"Ũ",Ṹ:"Ṹ",Ū:"Ū",Ṻ:"Ṻ",Ŭ:"Ŭ",Ǔ:"Ǔ",Û:"Û",Ů:"Ů",Ű:"Ű",Ṽ:"Ṽ",Ẃ:"Ẃ",Ẁ:"Ẁ",Ẅ:"Ẅ",Ŵ:"Ŵ",Ẇ:"Ẇ",Ẍ:"Ẍ",Ẋ:"Ẋ",Ý:"Ý",Ỳ:"Ỳ",Ÿ:"Ÿ",Ỹ:"Ỹ",Ȳ:"Ȳ",Ŷ:"Ŷ",Ẏ:"Ẏ",Ź:"Ź",Ž:"Ž",Ẑ:"Ẑ",Ż:"Ż",ά:"ά",ὰ:"ὰ",ᾱ:"ᾱ",ᾰ:"ᾰ",έ:"έ",ὲ:"ὲ",ή:"ή",ὴ:"ὴ",ί:"ί",ὶ:"ὶ",ϊ:"ϊ",ΐ:"ΐ",ῒ:"ῒ",ῑ:"ῑ",ῐ:"ῐ",ό:"ό",ὸ:"ὸ",ύ:"ύ",ὺ:"ὺ",ϋ:"ϋ",ΰ:"ΰ",ῢ:"ῢ",ῡ:"ῡ",ῠ:"ῠ",ώ:"ώ",ὼ:"ὼ",Ύ:"Ύ",Ὺ:"Ὺ",Ϋ:"Ϋ",Ῡ:"Ῡ",Ῠ:"Ῠ",Ώ:"Ώ",Ὼ:"Ὼ"};let iY=class oY{constructor(t,r){this.mode=void 0,this.gullet=void 0,this.settings=void 0,this.leftrightDepth=void 0,this.nextToken=void 0,this.mode="math",this.gullet=new nme(t,r,this.mode),this.settings=r,this.leftrightDepth=0}expect(t,r){if(r===void 0&&(r=!0),this.fetch().text!==t)throw new nt("Expected '"+t+"', got '"+this.fetch().text+"'",this.fetch());r&&this.consume()}consume(){this.nextToken=null}fetch(){return this.nextToken==null&&(this.nextToken=this.gullet.expandNextToken()),this.nextToken}switchMode(t){this.mode=t,this.gullet.switchMode(t)}parse(){this.settings.globalGroup||this.gullet.beginGroup(),this.settings.colorIsTextColor&&this.gullet.macros.set("\\color","\\textcolor");try{var t=this.parseExpression(!1);return this.expect("EOF"),this.settings.globalGroup||this.gullet.endGroup(),t}finally{this.gullet.endGroups()}}subparse(t){var r=this.nextToken;this.consume(),this.gullet.pushToken(new Ii("}")),this.gullet.pushTokens(t);var n=this.parseExpression(!1);return this.expect("}"),this.nextToken=r,n}parseExpression(t,r){for(var n=[];;){this.mode==="math"&&this.consumeSpaces();var a=this.fetch();if(oY.endOfExpression.indexOf(a.text)!==-1||r&&a.text===r||t&&yu[a.text]&&yu[a.text].infix)break;var i=this.parseAtom(r);if(i){if(i.type==="internal")continue}else break;n.push(i)}return this.mode==="text"&&this.formLigatures(n),this.handleInfixNodes(n)}handleInfixNodes(t){for(var r=-1,n,a=0;a<t.length;a++)if(t[a].type==="infix"){if(r!==-1)throw new nt("only one infix operator per group",t[a].token);r=a,n=t[a].replaceWith}if(r!==-1&&n){var i,o,s=t.slice(0,r),l=t.slice(r+1);s.length===1&&s[0].type==="ordgroup"?i=s[0]:i={type:"ordgroup",mode:this.mode,body:s},l.length===1&&l[0].type==="ordgroup"?o=l[0]:o={type:"ordgroup",mode:this.mode,body:l};var u;return n==="\\\\abovefrac"?u=this.callFunction(n,[i,t[r],o],[]):u=this.callFunction(n,[i,o],[]),[u]}else return t}handleSupSubscript(t){var r=this.fetch(),n=r.text;this.consume(),this.consumeSpaces();var a;do{var i;a=this.parseGroup(t)}while(((i=a)==null?void 0:i.type)==="internal");if(!a)throw new nt("Expected group after '"+n+"'",r);return a}formatUnsupportedCmd(t){for(var r=[],n=0;n<t.length;n++)r.push({type:"textord",mode:"text",text:t[n]});var a={type:"text",mode:this.mode,body:r},i={type:"color",mode:this.mode,color:this.settings.errorColor,body:[a]};return i}parseAtom(t){var r=this.parseGroup("atom",t);if((r==null?void 0:r.type)==="internal"||this.mode==="text")return r;for(var n,a;;){this.consumeSpaces();var i=this.fetch();if(i.text==="\\limits"||i.text==="\\nolimits"){if(r&&r.type==="op"){var o=i.text==="\\limits";r.limits=o,r.alwaysHandleSupSub=!0}else if(r&&r.type==="operatorname")r.alwaysHandleSupSub&&(r.limits=i.text==="\\limits");else throw new nt("Limit controls must follow a math operator",i);this.consume()}else if(i.text==="^"){if(n)throw new nt("Double superscript",i);n=this.handleSupSubscript("superscript")}else if(i.text==="_"){if(a)throw new nt("Double subscript",i);a=this.handleSupSubscript("subscript")}else if(i.text==="'"){if(n)throw new nt("Double superscript",i);var s={type:"textord",mode:this.mode,text:"\\prime"},l=[s];for(this.consume();this.fetch().text==="'";)l.push(s),this.consume();this.fetch().text==="^"&&l.push(this.handleSupSubscript("superscript")),n={type:"ordgroup",mode:this.mode,body:l}}else if(vb[i.text]){var u=Y9.test(i.text),d=[];for(d.push(new Ii(vb[i.text])),this.consume();;){var p=this.fetch().text;if(!vb[p]||Y9.test(p)!==u)break;d.unshift(new Ii(vb[p])),this.consume()}var f=this.subparse(d);u?a={type:"ordgroup",mode:"math",body:f}:n={type:"ordgroup",mode:"math",body:f}}else break}return n||a?{type:"supsub",mode:this.mode,base:r,sup:n,sub:a}:r}parseFunction(t,r){var n=this.fetch(),a=n.text,i=yu[a];if(!i)return null;if(this.consume(),r&&r!=="atom"&&!i.allowedInArgument)throw new nt("Got function '"+a+"' with no arguments"+(r?" as "+r:""),n);if(this.mode==="text"&&!i.allowedInText)throw new nt("Can't use function '"+a+"' in text mode",n);if(this.mode==="math"&&i.allowedInMath===!1)throw new nt("Can't use function '"+a+"' in math mode",n);var{args:o,optArgs:s}=this.parseArguments(a,i);return this.callFunction(a,o,s,n,t)}callFunction(t,r,n,a,i){var o={funcName:t,parser:this,token:a,breakOnTokenText:i},s=yu[t];if(s&&s.handler)return s.handler(o,r,n);throw new nt("No function handler for "+t)}parseArguments(t,r){var n=r.numArgs+r.numOptionalArgs;if(n===0)return{args:[],optArgs:[]};for(var a=[],i=[],o=0;o<n;o++){var s=r.argTypes&&r.argTypes[o],l=o<r.numOptionalArgs;(r.primitive&&s==null||r.type==="sqrt"&&o===1&&i[0]==null)&&(s="primitive");var u=this.parseGroupOfType("argument to '"+t+"'",s,l);if(l)i.push(u);else if(u!=null)a.push(u);else throw new nt("Null argument, please report this as a bug")}return{args:a,optArgs:i}}parseGroupOfType(t,r,n){switch(r){case"color":return this.parseColorGroup(n);case"size":return this.parseSizeGroup(n);case"url":return this.parseUrlGroup(n);case"math":case"text":return this.parseArgumentGroup(n,r);case"hbox":{var a=this.parseArgumentGroup(n,"text");return a!=null?{type:"styling",mode:a.mode,body:[a],style:"text"}:null}case"raw":{var i=this.parseStringGroup("raw",n);return i!=null?{type:"raw",mode:"text",string:i.text}:null}case"primitive":{if(n)throw new nt("A primitive argument cannot be optional");var o=this.parseGroup(t);if(o==null)throw new nt("Expected group as "+t,this.fetch());return o}case"original":case null:case void 0:return this.parseArgumentGroup(n);default:throw new nt("Unknown group type as "+t,this.fetch())}}consumeSpaces(){for(;this.fetch().text===" ";)this.consume()}parseStringGroup(t,r){var n=this.gullet.scanArgument(r);if(n==null)return null;for(var a="",i;(i=this.fetch()).text!=="EOF";)a+=i.text,this.consume();return this.consume(),n.text=a,n}parseRegexGroup(t,r){for(var n=this.fetch(),a=n,i="",o;(o=this.fetch()).text!=="EOF"&&t.test(i+o.text);)a=o,i+=a.text,this.consume();if(i==="")throw new nt("Invalid "+r+": '"+n.text+"'",n);return n.range(a,i)}parseColorGroup(t){var r=this.parseStringGroup("color",t);if(r==null)return null;var n=/^(#[a-f0-9]{3}|#?[a-f0-9]{6}|[a-z]+)$/i.exec(r.text);if(!n)throw new nt("Invalid color: '"+r.text+"'",r);var a=n[0];return/^[0-9a-f]{6}$/i.test(a)&&(a="#"+a),{type:"color-token",mode:this.mode,color:a}}parseSizeGroup(t){var r,n=!1;if(this.gullet.consumeSpaces(),!t&&this.gullet.future().text!=="{"?r=this.parseRegexGroup(/^[-+]? *(?:$|\d+|\d+\.\d*|\.\d*) *[a-z]{0,2} *$/,"size"):r=this.parseStringGroup("size",t),!r)return null;!t&&r.text.length===0&&(r.text="0pt",n=!0);var a=/([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(r.text);if(!a)throw new nt("Invalid size: '"+r.text+"'",r);var i={number:+(a[1]+a[2]),unit:a[3]};if(!fK(i))throw new nt("Invalid unit: '"+i.unit+"'",r);return{type:"size",mode:this.mode,value:i,isBlank:n}}parseUrlGroup(t){this.gullet.lexer.setCatcode("%",13),this.gullet.lexer.setCatcode("~",12);var r=this.parseStringGroup("url",t);if(this.gullet.lexer.setCatcode("%",14),this.gullet.lexer.setCatcode("~",13),r==null)return null;var n=r.text.replace(/\\([#$%&~_^{}])/g,"$1");return{type:"url",mode:this.mode,url:n}}parseArgumentGroup(t,r){var n=this.gullet.scanArgument(t);if(n==null)return null;var a=this.mode;r&&this.switchMode(r),this.gullet.beginGroup();var i=this.parseExpression(!1,"EOF");this.expect("EOF"),this.gullet.endGroup();var o={type:"ordgroup",mode:this.mode,loc:n.loc,body:i};return r&&this.switchMode(a),o}parseGroup(t,r){var n=this.fetch(),a=n.text,i;if(a==="{"||a==="\\begingroup"){this.consume();var o=a==="{"?"}":"\\endgroup";this.gullet.beginGroup();var s=this.parseExpression(!1,o),l=this.fetch();this.expect(o),this.gullet.endGroup(),i={type:"ordgroup",mode:this.mode,loc:ui.range(n,l),body:s,semisimple:a==="\\begingroup"||void 0}}else if(i=this.parseFunction(r,t)||this.parseSymbol(),i==null&&a[0]==="\\"&&!aY.hasOwnProperty(a)){if(this.settings.throwOnError)throw new nt("Undefined control sequence: "+a,n);i=this.formatUnsupportedCmd(a),this.consume()}return i}formLigatures(t){for(var r=t.length-1,n=0;n<r;++n){var a=t[n],i=a.text;i==="-"&&t[n+1].text==="-"&&(n+1<r&&t[n+2].text==="-"?(t.splice(n,3,{type:"textord",mode:"text",loc:ui.range(a,t[n+2]),text:"---"}),r-=2):(t.splice(n,2,{type:"textord",mode:"text",loc:ui.range(a,t[n+1]),text:"--"}),r-=1)),(i==="'"||i==="`")&&t[n+1].text===i&&(t.splice(n,2,{type:"textord",mode:"text",loc:ui.range(a,t[n+1]),text:i+i}),r-=1)}}parseSymbol(){var t=this.fetch(),r=t.text;if(/^\\verb[^a-zA-Z]/.test(r)){this.consume();var n=r.slice(5),a=n.charAt(0)==="*";if(a&&(n=n.slice(1)),n.length<2||n.charAt(0)!==n.slice(-1))throw new nt(`\\verb assertion failed --
                    please report what input caused this bug`);return n=n.slice(1,-1),{type:"verb",mode:"text",body:n,star:a}}J9.hasOwnProperty(r[0])&&!sn[this.mode][r[0]]&&(this.settings.strict&&this.mode==="math"&&this.settings.reportNonstrict("unicodeTextInMathMode",'Accented Unicode text character "'+r[0]+'" used in math mode',t),r=J9[r[0]]+r.slice(1));var i=Qfe.exec(r);i&&(r=r.substring(0,i.index),r==="i"?r="ı":r==="j"&&(r="ȷ"));var o;if(sn[this.mode][r]){this.settings.strict&&this.mode==="math"&&k_.indexOf(r)>=0&&this.settings.reportNonstrict("unicodeTextInMathMode",'Latin-1/Unicode text character "'+r[0]+'" used in math mode',t);var s=sn[this.mode][r].group,l=ui.range(t),u;if(qpe.hasOwnProperty(s)){var d=s;u={type:"atom",mode:this.mode,family:d,loc:l,text:r}}else u={type:s,mode:this.mode,loc:l,text:r};o=u}else if(r.charCodeAt(0)>=128)this.settings.strict&&(hK(r.charCodeAt(0))?this.mode==="math"&&this.settings.reportNonstrict("unicodeTextInMathMode",'Unicode text character "'+r[0]+'" used in math mode',t):this.settings.reportNonstrict("unknownSymbol",'Unrecognized Unicode character "'+r[0]+'"'+(" ("+r.charCodeAt(0)+")"),t)),o={type:"textord",mode:"text",loc:ui.range(t),text:r};else return null;if(this.consume(),i)for(var p=0;p<i[0].length;p++){var f=i[0][p];if(!Ix[f])throw new nt("Unknown accent ' "+f+"'",t);var m=Ix[f][this.mode]||Ix[f].text;if(!m)throw new nt("Accent "+f+" unsupported in "+this.mode+" mode",t);o={type:"accent",mode:this.mode,loc:ui.range(t),label:m,isStretchy:!1,isShifty:!0,base:o}}return o}};iY.endOfExpression=["}","\\endgroup","\\end","\\right","&"];var xD=function(t,r){if(!(typeof t=="string"||t instanceof String))throw new TypeError("KaTeX can only parse string typed expression");var n=new iY(t,r);delete n.gullet.macros.current["\\df@tag"];var a=n.parse();if(delete n.gullet.macros.current["\\current@color"],delete n.gullet.macros.current["\\color"],n.gullet.macros.get("\\df@tag")){if(!r.displayMode)throw new nt("\\tag works only in display equations");a=[{type:"tag",mode:"text",body:a,tag:n.subparse([new Ii("\\df@tag")])}]}return a},wD=function(t,r,n){r.textContent="";var a=Av(t,n).toNode();r.appendChild(a)};typeof document<"u"&&document.compatMode!=="CSS1Compat"&&(typeof console<"u"&&console.warn("Warning: KaTeX doesn't work in quirks mode. Make sure your website has a suitable doctype."),wD=function(){throw new nt("KaTeX doesn't work in quirks mode.")});var sY=function(t,r){var n=Av(t,r).toMarkup();return n},lY=function(t,r){var n=new aD(r);return xD(t,n)},uY=function(t,r,n){if(n.throwOnError||!(t instanceof nt))throw t;var a=Fe.makeSpan(["katex-error"],[new oo(r)]);return a.setAttribute("title",t.toString()),a.setAttribute("style","color:"+n.errorColor),a},Av=function(t,r){var n=new aD(r);try{var a=xD(t,n);return ffe(a,t,n)}catch(i){return uY(i,t,n)}},cY=function(t,r){var n=new aD(r);try{var a=xD(t,n);return mfe(a,t,n)}catch(i){return uY(i,t,n)}},dY="0.16.23",hY={Span:cg,Anchor:sD,SymbolNode:oo,SvgNode:pl,PathNode:Tu,LineNode:w_},Fv={version:dY,render:wD,renderToString:sY,ParseError:nt,SETTINGS_SCHEMA:gm,__parse:lY,__renderToDomTree:Av,__renderToHTMLTree:cY,__setFontMetrics:pK,__defineSymbol:O,__defineFunction:gt,__defineMacro:te,__domTree:hY};const ame=Object.freeze(Object.defineProperty({__proto__:null,ParseError:nt,SETTINGS_SCHEMA:gm,__defineFunction:gt,__defineMacro:te,__defineSymbol:O,__domTree:hY,__parse:lY,__renderToDomTree:Av,__renderToHTMLTree:cY,__setFontMetrics:pK,default:Fv,get render(){return wD},renderToString:sY,version:dY},Symbol.toStringTag,{value:"Module"}));Fv.__defineMacro("\\ce",function(e){return pY(e.consumeArgs(1)[0],"ce")});Fv.__defineMacro("\\pu",function(e){return pY(e.consumeArgs(1)[0],"pu")});Fv.__defineMacro("\\tripledash","{\\vphantom{-}\\raisebox{2.56mu}{$\\mkern2mu\\tiny\\text{-}\\mkern1mu\\text{-}\\mkern1mu\\text{-}\\mkern2mu$}}");var pY=function(t,r){for(var n="",a=t.length&&t[t.length-1].loc.start,i=t.length-1;i>=0;i--)t[i].loc.start>a&&(n+=" ",a=t[i].loc.start),n+=t[i].text,a+=t[i].text.length;var o=en.go(Ge.go(n,r));return o},Ge={go:function(t,r){if(!t)return[];r===void 0&&(r="ce");var n="0",a={};a.parenthesisLevel=0,t=t.replace(/\n/g," "),t=t.replace(/[\u2212\u2013\u2014\u2010]/g,"-"),t=t.replace(/[\u2026]/g,"...");for(var i,o=10,s=[];;){i!==t?(o=10,i=t):o--;var l=Ge.stateMachines[r],u=l.transitions[n]||l.transitions["*"];e:for(var d=0;d<u.length;d++){var p=Ge.patterns.match_(u[d].pattern,t);if(p){for(var f=u[d].task,m=0;m<f.action_.length;m++){var b;if(l.actions[f.action_[m].type_])b=l.actions[f.action_[m].type_](a,p.match_,f.action_[m].option);else if(Ge.actions[f.action_[m].type_])b=Ge.actions[f.action_[m].type_](a,p.match_,f.action_[m].option);else throw["MhchemBugA","mhchem bug A. Please report. ("+f.action_[m].type_+")"];Ge.concatArray(s,b)}if(n=f.nextState||n,t.length>0){if(f.revisit||(t=p.remainder),!f.toContinue)break e}else return s}}if(o<=0)throw["MhchemBugU","mhchem bug U. Please report."]}},concatArray:function(t,r){if(r)if(Array.isArray(r))for(var n=0;n<r.length;n++)t.push(r[n]);else t.push(r)},patterns:{patterns:{empty:/^$/,else:/^./,else2:/^./,space:/^\s/,"space A":/^\s(?=[A-Z\\$])/,space$:/^\s$/,"a-z":/^[a-z]/,x:/^x/,x$:/^x$/,i$:/^i$/,letters:/^(?:[a-zA-Z\u03B1-\u03C9\u0391-\u03A9?@]|(?:\\(?:alpha|beta|gamma|delta|epsilon|zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|omicron|pi|rho|sigma|tau|upsilon|phi|chi|psi|omega|Gamma|Delta|Theta|Lambda|Xi|Pi|Sigma|Upsilon|Phi|Psi|Omega)(?:\s+|\{\}|(?![a-zA-Z]))))+/,"\\greek":/^\\(?:alpha|beta|gamma|delta|epsilon|zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|omicron|pi|rho|sigma|tau|upsilon|phi|chi|psi|omega|Gamma|Delta|Theta|Lambda|Xi|Pi|Sigma|Upsilon|Phi|Psi|Omega)(?:\s+|\{\}|(?![a-zA-Z]))/,"one lowercase latin letter $":/^(?:([a-z])(?:$|[^a-zA-Z]))$/,"$one lowercase latin letter$ $":/^\$(?:([a-z])(?:$|[^a-zA-Z]))\$$/,"one lowercase greek letter $":/^(?:\$?[\u03B1-\u03C9]\$?|\$?\\(?:alpha|beta|gamma|delta|epsilon|zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|omicron|pi|rho|sigma|tau|upsilon|phi|chi|psi|omega)\s*\$?)(?:\s+|\{\}|(?![a-zA-Z]))$/,digits:/^[0-9]+/,"-9.,9":/^[+\-]?(?:[0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+))/,"-9.,9 no missing 0":/^[+\-]?[0-9]+(?:[.,][0-9]+)?/,"(-)(9.,9)(e)(99)":function(t){var r=t.match(/^(\+\-|\+\/\-|\+|\-|\\pm\s?)?([0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+))?(\((?:[0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+))\))?(?:([eE]|\s*(\*|x|\\times|\u00D7)\s*10\^)([+\-]?[0-9]+|\{[+\-]?[0-9]+\}))?/);return r&&r[0]?{match_:r.splice(1),remainder:t.substr(r[0].length)}:null},"(-)(9)^(-9)":function(t){var r=t.match(/^(\+\-|\+\/\-|\+|\-|\\pm\s?)?([0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+)?)\^([+\-]?[0-9]+|\{[+\-]?[0-9]+\})/);return r&&r[0]?{match_:r.splice(1),remainder:t.substr(r[0].length)}:null},"state of aggregation $":function(t){var r=Ge.patterns.findObserveGroups(t,"",/^\([a-z]{1,3}(?=[\),])/,")","");if(r&&r.remainder.match(/^($|[\s,;\)\]\}])/))return r;var n=t.match(/^(?:\((?:\\ca\s?)?\$[amothc]\$\))/);return n?{match_:n[0],remainder:t.substr(n[0].length)}:null},"_{(state of aggregation)}$":/^_\{(\([a-z]{1,3}\))\}/,"{[(":/^(?:\\\{|\[|\()/,")]}":/^(?:\)|\]|\\\})/,", ":/^[,;]\s*/,",":/^[,;]/,".":/^[.]/,". ":/^([.\u22C5\u00B7\u2022])\s*/,"...":/^\.\.\.(?=$|[^.])/,"* ":/^([*])\s*/,"^{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"^{","","","}")},"^($...$)":function(t){return Ge.patterns.findObserveGroups(t,"^","$","$","")},"^a":/^\^([0-9]+|[^\\_])/,"^\\x{}{}":function(t){return Ge.patterns.findObserveGroups(t,"^",/^\\[a-zA-Z]+\{/,"}","","","{","}","",!0)},"^\\x{}":function(t){return Ge.patterns.findObserveGroups(t,"^",/^\\[a-zA-Z]+\{/,"}","")},"^\\x":/^\^(\\[a-zA-Z]+)\s*/,"^(-1)":/^\^(-?\d+)/,"'":/^'/,"_{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"_{","","","}")},"_($...$)":function(t){return Ge.patterns.findObserveGroups(t,"_","$","$","")},_9:/^_([+\-]?[0-9]+|[^\\])/,"_\\x{}{}":function(t){return Ge.patterns.findObserveGroups(t,"_",/^\\[a-zA-Z]+\{/,"}","","","{","}","",!0)},"_\\x{}":function(t){return Ge.patterns.findObserveGroups(t,"_",/^\\[a-zA-Z]+\{/,"}","")},"_\\x":/^_(\\[a-zA-Z]+)\s*/,"^_":/^(?:\^(?=_)|\_(?=\^)|[\^_]$)/,"{}":/^\{\}/,"{...}":function(t){return Ge.patterns.findObserveGroups(t,"","{","}","")},"{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"{","","","}")},"$...$":function(t){return Ge.patterns.findObserveGroups(t,"","$","$","")},"${(...)}$":function(t){return Ge.patterns.findObserveGroups(t,"${","","","}$")},"$(...)$":function(t){return Ge.patterns.findObserveGroups(t,"$","","","$")},"=<>":/^[=<>]/,"#":/^[#\u2261]/,"+":/^\+/,"-$":/^-(?=[\s_},;\]/]|$|\([a-z]+\))/,"-9":/^-(?=[0-9])/,"- orbital overlap":/^-(?=(?:[spd]|sp)(?:$|[\s,;\)\]\}]))/,"-":/^-/,"pm-operator":/^(?:\\pm|\$\\pm\$|\+-|\+\/-)/,operator:/^(?:\+|(?:[\-=<>]|<<|>>|\\approx|\$\\approx\$)(?=\s|$|-?[0-9]))/,arrowUpDown:/^(?:v|\(v\)|\^|\(\^\))(?=$|[\s,;\)\]\}])/,"\\bond{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"\\bond{","","","}")},"->":/^(?:<->|<-->|->|<-|<=>>|<<=>|<=>|[\u2192\u27F6\u21CC])/,CMT:/^[CMT](?=\[)/,"[(...)]":function(t){return Ge.patterns.findObserveGroups(t,"[","","","]")},"1st-level escape":/^(&|\\\\|\\hline)\s*/,"\\,":/^(?:\\[,\ ;:])/,"\\x{}{}":function(t){return Ge.patterns.findObserveGroups(t,"",/^\\[a-zA-Z]+\{/,"}","","","{","}","",!0)},"\\x{}":function(t){return Ge.patterns.findObserveGroups(t,"",/^\\[a-zA-Z]+\{/,"}","")},"\\ca":/^\\ca(?:\s+|(?![a-zA-Z]))/,"\\x":/^(?:\\[a-zA-Z]+\s*|\\[_&{}%])/,orbital:/^(?:[0-9]{1,2}[spdfgh]|[0-9]{0,2}sp)(?=$|[^a-zA-Z])/,others:/^[\/~|]/,"\\frac{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"\\frac{","","","}","{","","","}")},"\\overset{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"\\overset{","","","}","{","","","}")},"\\underset{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"\\underset{","","","}","{","","","}")},"\\underbrace{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"\\underbrace{","","","}_","{","","","}")},"\\color{(...)}0":function(t){return Ge.patterns.findObserveGroups(t,"\\color{","","","}")},"\\color{(...)}{(...)}1":function(t){return Ge.patterns.findObserveGroups(t,"\\color{","","","}","{","","","}")},"\\color(...){(...)}2":function(t){return Ge.patterns.findObserveGroups(t,"\\color","\\","",/^(?=\{)/,"{","","","}")},"\\ce{(...)}":function(t){return Ge.patterns.findObserveGroups(t,"\\ce{","","","}")},oxidation$:/^(?:[+-][IVX]+|\\pm\s*0|\$\\pm\$\s*0)$/,"d-oxidation$":/^(?:[+-]?\s?[IVX]+|\\pm\s*0|\$\\pm\$\s*0)$/,"roman numeral":/^[IVX]+/,"1/2$":/^[+\-]?(?:[0-9]+|\$[a-z]\$|[a-z])\/[0-9]+(?:\$[a-z]\$|[a-z])?$/,amount:function(t){var r;if(r=t.match(/^(?:(?:(?:\([+\-]?[0-9]+\/[0-9]+\)|[+\-]?(?:[0-9]+|\$[a-z]\$|[a-z])\/[0-9]+|[+\-]?[0-9]+[.,][0-9]+|[+\-]?\.[0-9]+|[+\-]?[0-9]+)(?:[a-z](?=\s*[A-Z]))?)|[+\-]?[a-z](?=\s*[A-Z])|\+(?!\s))/),r)return{match_:r[0],remainder:t.substr(r[0].length)};var n=Ge.patterns.findObserveGroups(t,"","$","$","");return n&&(r=n.match_.match(/^\$(?:\(?[+\-]?(?:[0-9]*[a-z]?[+\-])?[0-9]*[a-z](?:[+\-][0-9]*[a-z]?)?\)?|\+|-)\$$/),r)?{match_:r[0],remainder:t.substr(r[0].length)}:null},amount2:function(t){return this.amount(t)},"(KV letters),":/^(?:[A-Z][a-z]{0,2}|i)(?=,)/,formula$:function(t){if(t.match(/^\([a-z]+\)$/))return null;var r=t.match(/^(?:[a-z]|(?:[0-9\ \+\-\,\.\(\)]+[a-z])+[0-9\ \+\-\,\.\(\)]*|(?:[a-z][0-9\ \+\-\,\.\(\)]+)+[a-z]?)$/);return r?{match_:r[0],remainder:t.substr(r[0].length)}:null},uprightEntities:/^(?:pH|pOH|pC|pK|iPr|iBu)(?=$|[^a-zA-Z])/,"/":/^\s*(\/)\s*/,"//":/^\s*(\/\/)\s*/,"*":/^\s*[*.]\s*/},findObserveGroups:function(t,r,n,a,i,o,s,l,u,d){var p=function(k,_){if(typeof _=="string")return k.indexOf(_)!==0?null:_;var R=k.match(_);return R?R[0]:null},f=function(k,_,R){for(var C=0;_<k.length;){var F=k.charAt(_),L=p(k.substr(_),R);if(L!==null&&C===0)return{endMatchBegin:_,endMatchEnd:_+L.length};if(F==="{")C++;else if(F==="}"){if(C===0)throw["ExtraCloseMissingOpen","Extra close brace or missing open brace"];C--}_++}return C>0,null},m=p(t,r);if(m===null||(t=t.substr(m.length),m=p(t,n),m===null))return null;var b=f(t,m.length,a||i);if(b===null)return null;var y=t.substring(0,a?b.endMatchEnd:b.endMatchBegin);if(o||s){var S=this.findObserveGroups(t.substr(b.endMatchEnd),o,s,l,u);if(S===null)return null;var w=[y,S.match_];return{match_:d?w.join(""):w,remainder:S.remainder}}else return{match_:y,remainder:t.substr(b.endMatchEnd)}},match_:function(t,r){var n=Ge.patterns.patterns[t];if(n===void 0)throw["MhchemBugP","mhchem bug P. Please report. ("+t+")"];if(typeof n=="function")return Ge.patterns.patterns[t](r);var a=r.match(n);if(a){var i;return a[2]?i=[a[1],a[2]]:a[1]?i=a[1]:i=a[0],{match_:i,remainder:r.substr(a[0].length)}}return null}},actions:{"a=":function(t,r){t.a=(t.a||"")+r},"b=":function(t,r){t.b=(t.b||"")+r},"p=":function(t,r){t.p=(t.p||"")+r},"o=":function(t,r){t.o=(t.o||"")+r},"q=":function(t,r){t.q=(t.q||"")+r},"d=":function(t,r){t.d=(t.d||"")+r},"rm=":function(t,r){t.rm=(t.rm||"")+r},"text=":function(t,r){t.text_=(t.text_||"")+r},insert:function(t,r,n){return{type_:n}},"insert+p1":function(t,r,n){return{type_:n,p1:r}},"insert+p1+p2":function(t,r,n){return{type_:n,p1:r[0],p2:r[1]}},copy:function(t,r){return r},rm:function(t,r){return{type_:"rm",p1:r||""}},text:function(t,r){return Ge.go(r,"text")},"{text}":function(t,r){var n=["{"];return Ge.concatArray(n,Ge.go(r,"text")),n.push("}"),n},"tex-math":function(t,r){return Ge.go(r,"tex-math")},"tex-math tight":function(t,r){return Ge.go(r,"tex-math tight")},bond:function(t,r,n){return{type_:"bond",kind_:n||r}},"color0-output":function(t,r){return{type_:"color0",color:r[0]}},ce:function(t,r){return Ge.go(r)},"1/2":function(t,r){var n=[];r.match(/^[+\-]/)&&(n.push(r.substr(0,1)),r=r.substr(1));var a=r.match(/^([0-9]+|\$[a-z]\$|[a-z])\/([0-9]+)(\$[a-z]\$|[a-z])?$/);return a[1]=a[1].replace(/\$/g,""),n.push({type_:"frac",p1:a[1],p2:a[2]}),a[3]&&(a[3]=a[3].replace(/\$/g,""),n.push({type_:"tex-math",p1:a[3]})),n},"9,9":function(t,r){return Ge.go(r,"9,9")}},createTransitions:function(t){var r,n,a,i,o={};for(r in t)for(n in t[r])for(a=n.split("|"),t[r][n].stateArray=a,i=0;i<a.length;i++)o[a[i]]=[];for(r in t)for(n in t[r])for(a=t[r][n].stateArray||[],i=0;i<a.length;i++){var s=t[r][n];if(s.action_){s.action_=[].concat(s.action_);for(var l=0;l<s.action_.length;l++)typeof s.action_[l]=="string"&&(s.action_[l]={type_:s.action_[l]})}else s.action_=[];for(var u=r.split("|"),d=0;d<u.length;d++)if(a[i]==="*")for(var p in o)o[p].push({pattern:u[d],task:s});else o[a[i]].push({pattern:u[d],task:s})}return o},stateMachines:{}};Ge.stateMachines={ce:{transitions:Ge.createTransitions({empty:{"*":{action_:"output"}},else:{"0|1|2":{action_:"beginsWithBond=false",revisit:!0,toContinue:!0}},oxidation$:{0:{action_:"oxidation-output"}},CMT:{r:{action_:"rdt=",nextState:"rt"},rd:{action_:"rqt=",nextState:"rdt"}},arrowUpDown:{"0|1|2|as":{action_:["sb=false","output","operator"],nextState:"1"}},uprightEntities:{"0|1|2":{action_:["o=","output"],nextState:"1"}},orbital:{"0|1|2|3":{action_:"o=",nextState:"o"}},"->":{"0|1|2|3":{action_:"r=",nextState:"r"},"a|as":{action_:["output","r="],nextState:"r"},"*":{action_:["output","r="],nextState:"r"}},"+":{o:{action_:"d= kv",nextState:"d"},"d|D":{action_:"d=",nextState:"d"},q:{action_:"d=",nextState:"qd"},"qd|qD":{action_:"d=",nextState:"qd"},dq:{action_:["output","d="],nextState:"d"},3:{action_:["sb=false","output","operator"],nextState:"0"}},amount:{"0|2":{action_:"a=",nextState:"a"}},"pm-operator":{"0|1|2|a|as":{action_:["sb=false","output",{type_:"operator",option:"\\pm"}],nextState:"0"}},operator:{"0|1|2|a|as":{action_:["sb=false","output","operator"],nextState:"0"}},"-$":{"o|q":{action_:["charge or bond","output"],nextState:"qd"},d:{action_:"d=",nextState:"d"},D:{action_:["output",{type_:"bond",option:"-"}],nextState:"3"},q:{action_:"d=",nextState:"qd"},qd:{action_:"d=",nextState:"qd"},"qD|dq":{action_:["output",{type_:"bond",option:"-"}],nextState:"3"}},"-9":{"3|o":{action_:["output",{type_:"insert",option:"hyphen"}],nextState:"3"}},"- orbital overlap":{o:{action_:["output",{type_:"insert",option:"hyphen"}],nextState:"2"},d:{action_:["output",{type_:"insert",option:"hyphen"}],nextState:"2"}},"-":{"0|1|2":{action_:[{type_:"output",option:1},"beginsWithBond=true",{type_:"bond",option:"-"}],nextState:"3"},3:{action_:{type_:"bond",option:"-"}},a:{action_:["output",{type_:"insert",option:"hyphen"}],nextState:"2"},as:{action_:[{type_:"output",option:2},{type_:"bond",option:"-"}],nextState:"3"},b:{action_:"b="},o:{action_:{type_:"- after o/d",option:!1},nextState:"2"},q:{action_:{type_:"- after o/d",option:!1},nextState:"2"},"d|qd|dq":{action_:{type_:"- after o/d",option:!0},nextState:"2"},"D|qD|p":{action_:["output",{type_:"bond",option:"-"}],nextState:"3"}},amount2:{"1|3":{action_:"a=",nextState:"a"}},letters:{"0|1|2|3|a|as|b|p|bp|o":{action_:"o=",nextState:"o"},"q|dq":{action_:["output","o="],nextState:"o"},"d|D|qd|qD":{action_:"o after d",nextState:"o"}},digits:{o:{action_:"q=",nextState:"q"},"d|D":{action_:"q=",nextState:"dq"},q:{action_:["output","o="],nextState:"o"},a:{action_:"o=",nextState:"o"}},"space A":{"b|p|bp":{}},space:{a:{nextState:"as"},0:{action_:"sb=false"},"1|2":{action_:"sb=true"},"r|rt|rd|rdt|rdq":{action_:"output",nextState:"0"},"*":{action_:["output","sb=true"],nextState:"1"}},"1st-level escape":{"1|2":{action_:["output",{type_:"insert+p1",option:"1st-level escape"}]},"*":{action_:["output",{type_:"insert+p1",option:"1st-level escape"}],nextState:"0"}},"[(...)]":{"r|rt":{action_:"rd=",nextState:"rd"},"rd|rdt":{action_:"rq=",nextState:"rdq"}},"...":{"o|d|D|dq|qd|qD":{action_:["output",{type_:"bond",option:"..."}],nextState:"3"},"*":{action_:[{type_:"output",option:1},{type_:"insert",option:"ellipsis"}],nextState:"1"}},". |* ":{"*":{action_:["output",{type_:"insert",option:"addition compound"}],nextState:"1"}},"state of aggregation $":{"*":{action_:["output","state of aggregation"],nextState:"1"}},"{[(":{"a|as|o":{action_:["o=","output","parenthesisLevel++"],nextState:"2"},"0|1|2|3":{action_:["o=","output","parenthesisLevel++"],nextState:"2"},"*":{action_:["output","o=","output","parenthesisLevel++"],nextState:"2"}},")]}":{"0|1|2|3|b|p|bp|o":{action_:["o=","parenthesisLevel--"],nextState:"o"},"a|as|d|D|q|qd|qD|dq":{action_:["output","o=","parenthesisLevel--"],nextState:"o"}},", ":{"*":{action_:["output","comma"],nextState:"0"}},"^_":{"*":{}},"^{(...)}|^($...$)":{"0|1|2|as":{action_:"b=",nextState:"b"},p:{action_:"b=",nextState:"bp"},"3|o":{action_:"d= kv",nextState:"D"},q:{action_:"d=",nextState:"qD"},"d|D|qd|qD|dq":{action_:["output","d="],nextState:"D"}},"^a|^\\x{}{}|^\\x{}|^\\x|'":{"0|1|2|as":{action_:"b=",nextState:"b"},p:{action_:"b=",nextState:"bp"},"3|o":{action_:"d= kv",nextState:"d"},q:{action_:"d=",nextState:"qd"},"d|qd|D|qD":{action_:"d="},dq:{action_:["output","d="],nextState:"d"}},"_{(state of aggregation)}$":{"d|D|q|qd|qD|dq":{action_:["output","q="],nextState:"q"}},"_{(...)}|_($...$)|_9|_\\x{}{}|_\\x{}|_\\x":{"0|1|2|as":{action_:"p=",nextState:"p"},b:{action_:"p=",nextState:"bp"},"3|o":{action_:"q=",nextState:"q"},"d|D":{action_:"q=",nextState:"dq"},"q|qd|qD|dq":{action_:["output","q="],nextState:"q"}},"=<>":{"0|1|2|3|a|as|o|q|d|D|qd|qD|dq":{action_:[{type_:"output",option:2},"bond"],nextState:"3"}},"#":{"0|1|2|3|a|as|o":{action_:[{type_:"output",option:2},{type_:"bond",option:"#"}],nextState:"3"}},"{}":{"*":{action_:{type_:"output",option:1},nextState:"1"}},"{...}":{"0|1|2|3|a|as|b|p|bp":{action_:"o=",nextState:"o"},"o|d|D|q|qd|qD|dq":{action_:["output","o="],nextState:"o"}},"$...$":{a:{action_:"a="},"0|1|2|3|as|b|p|bp|o":{action_:"o=",nextState:"o"},"as|o":{action_:"o="},"q|d|D|qd|qD|dq":{action_:["output","o="],nextState:"o"}},"\\bond{(...)}":{"*":{action_:[{type_:"output",option:2},"bond"],nextState:"3"}},"\\frac{(...)}":{"*":{action_:[{type_:"output",option:1},"frac-output"],nextState:"3"}},"\\overset{(...)}":{"*":{action_:[{type_:"output",option:2},"overset-output"],nextState:"3"}},"\\underset{(...)}":{"*":{action_:[{type_:"output",option:2},"underset-output"],nextState:"3"}},"\\underbrace{(...)}":{"*":{action_:[{type_:"output",option:2},"underbrace-output"],nextState:"3"}},"\\color{(...)}{(...)}1|\\color(...){(...)}2":{"*":{action_:[{type_:"output",option:2},"color-output"],nextState:"3"}},"\\color{(...)}0":{"*":{action_:[{type_:"output",option:2},"color0-output"]}},"\\ce{(...)}":{"*":{action_:[{type_:"output",option:2},"ce"],nextState:"3"}},"\\,":{"*":{action_:[{type_:"output",option:1},"copy"],nextState:"1"}},"\\x{}{}|\\x{}|\\x":{"0|1|2|3|a|as|b|p|bp|o|c0":{action_:["o=","output"],nextState:"3"},"*":{action_:["output","o=","output"],nextState:"3"}},others:{"*":{action_:[{type_:"output",option:1},"copy"],nextState:"3"}},else2:{a:{action_:"a to o",nextState:"o",revisit:!0},as:{action_:["output","sb=true"],nextState:"1",revisit:!0},"r|rt|rd|rdt|rdq":{action_:["output"],nextState:"0",revisit:!0},"*":{action_:["output","copy"],nextState:"3"}}}),actions:{"o after d":function(t,r){var n;if((t.d||"").match(/^[0-9]+$/)){var a=t.d;t.d=void 0,n=this.output(t),t.b=a}else n=this.output(t);return Ge.actions["o="](t,r),n},"d= kv":function(t,r){t.d=r,t.dType="kv"},"charge or bond":function(t,r){if(t.beginsWithBond){var n=[];return Ge.concatArray(n,this.output(t)),Ge.concatArray(n,Ge.actions.bond(t,r,"-")),n}else t.d=r},"- after o/d":function(t,r,n){var a=Ge.patterns.match_("orbital",t.o||""),i=Ge.patterns.match_("one lowercase greek letter $",t.o||""),o=Ge.patterns.match_("one lowercase latin letter $",t.o||""),s=Ge.patterns.match_("$one lowercase latin letter$ $",t.o||""),l=r==="-"&&(a&&a.remainder===""||i||o||s);l&&!t.a&&!t.b&&!t.p&&!t.d&&!t.q&&!a&&o&&(t.o="$"+t.o+"$");var u=[];return l?(Ge.concatArray(u,this.output(t)),u.push({type_:"hyphen"})):(a=Ge.patterns.match_("digits",t.d||""),n&&a&&a.remainder===""?(Ge.concatArray(u,Ge.actions["d="](t,r)),Ge.concatArray(u,this.output(t))):(Ge.concatArray(u,this.output(t)),Ge.concatArray(u,Ge.actions.bond(t,r,"-")))),u},"a to o":function(t){t.o=t.a,t.a=void 0},"sb=true":function(t){t.sb=!0},"sb=false":function(t){t.sb=!1},"beginsWithBond=true":function(t){t.beginsWithBond=!0},"beginsWithBond=false":function(t){t.beginsWithBond=!1},"parenthesisLevel++":function(t){t.parenthesisLevel++},"parenthesisLevel--":function(t){t.parenthesisLevel--},"state of aggregation":function(t,r){return{type_:"state of aggregation",p1:Ge.go(r,"o")}},comma:function(t,r){var n=r.replace(/\s*$/,""),a=n!==r;return a&&t.parenthesisLevel===0?{type_:"comma enumeration L",p1:n}:{type_:"comma enumeration M",p1:n}},output:function(t,r,n){var a;if(!t.r)a=[],!t.a&&!t.b&&!t.p&&!t.o&&!t.q&&!t.d&&!n||(t.sb&&a.push({type_:"entitySkip"}),!t.o&&!t.q&&!t.d&&!t.b&&!t.p&&n!==2?(t.o=t.a,t.a=void 0):!t.o&&!t.q&&!t.d&&(t.b||t.p)?(t.o=t.a,t.d=t.b,t.q=t.p,t.a=t.b=t.p=void 0):t.o&&t.dType==="kv"&&Ge.patterns.match_("d-oxidation$",t.d||"")?t.dType="oxidation":t.o&&t.dType==="kv"&&!t.q&&(t.dType=void 0),a.push({type_:"chemfive",a:Ge.go(t.a,"a"),b:Ge.go(t.b,"bd"),p:Ge.go(t.p,"pq"),o:Ge.go(t.o,"o"),q:Ge.go(t.q,"pq"),d:Ge.go(t.d,t.dType==="oxidation"?"oxidation":"bd"),dType:t.dType}));else{var i;t.rdt==="M"?i=Ge.go(t.rd,"tex-math"):t.rdt==="T"?i=[{type_:"text",p1:t.rd||""}]:i=Ge.go(t.rd);var o;t.rqt==="M"?o=Ge.go(t.rq,"tex-math"):t.rqt==="T"?o=[{type_:"text",p1:t.rq||""}]:o=Ge.go(t.rq),a={type_:"arrow",r:t.r,rd:i,rq:o}}for(var s in t)s!=="parenthesisLevel"&&s!=="beginsWithBond"&&delete t[s];return a},"oxidation-output":function(t,r){var n=["{"];return Ge.concatArray(n,Ge.go(r,"oxidation")),n.push("}"),n},"frac-output":function(t,r){return{type_:"frac-ce",p1:Ge.go(r[0]),p2:Ge.go(r[1])}},"overset-output":function(t,r){return{type_:"overset",p1:Ge.go(r[0]),p2:Ge.go(r[1])}},"underset-output":function(t,r){return{type_:"underset",p1:Ge.go(r[0]),p2:Ge.go(r[1])}},"underbrace-output":function(t,r){return{type_:"underbrace",p1:Ge.go(r[0]),p2:Ge.go(r[1])}},"color-output":function(t,r){return{type_:"color",color1:r[0],color2:Ge.go(r[1])}},"r=":function(t,r){t.r=r},"rdt=":function(t,r){t.rdt=r},"rd=":function(t,r){t.rd=r},"rqt=":function(t,r){t.rqt=r},"rq=":function(t,r){t.rq=r},operator:function(t,r,n){return{type_:"operator",kind_:n||r}}}},a:{transitions:Ge.createTransitions({empty:{"*":{}},"1/2$":{0:{action_:"1/2"}},else:{0:{nextState:"1",revisit:!0}},"$(...)$":{"*":{action_:"tex-math tight",nextState:"1"}},",":{"*":{action_:{type_:"insert",option:"commaDecimal"}}},else2:{"*":{action_:"copy"}}}),actions:{}},o:{transitions:Ge.createTransitions({empty:{"*":{}},"1/2$":{0:{action_:"1/2"}},else:{0:{nextState:"1",revisit:!0}},letters:{"*":{action_:"rm"}},"\\ca":{"*":{action_:{type_:"insert",option:"circa"}}},"\\x{}{}|\\x{}|\\x":{"*":{action_:"copy"}},"${(...)}$|$(...)$":{"*":{action_:"tex-math"}},"{(...)}":{"*":{action_:"{text}"}},else2:{"*":{action_:"copy"}}}),actions:{}},text:{transitions:Ge.createTransitions({empty:{"*":{action_:"output"}},"{...}":{"*":{action_:"text="}},"${(...)}$|$(...)$":{"*":{action_:"tex-math"}},"\\greek":{"*":{action_:["output","rm"]}},"\\,|\\x{}{}|\\x{}|\\x":{"*":{action_:["output","copy"]}},else:{"*":{action_:"text="}}}),actions:{output:function(t){if(t.text_){var r={type_:"text",p1:t.text_};for(var n in t)delete t[n];return r}}}},pq:{transitions:Ge.createTransitions({empty:{"*":{}},"state of aggregation $":{"*":{action_:"state of aggregation"}},i$:{0:{nextState:"!f",revisit:!0}},"(KV letters),":{0:{action_:"rm",nextState:"0"}},formula$:{0:{nextState:"f",revisit:!0}},"1/2$":{0:{action_:"1/2"}},else:{0:{nextState:"!f",revisit:!0}},"${(...)}$|$(...)$":{"*":{action_:"tex-math"}},"{(...)}":{"*":{action_:"text"}},"a-z":{f:{action_:"tex-math"}},letters:{"*":{action_:"rm"}},"-9.,9":{"*":{action_:"9,9"}},",":{"*":{action_:{type_:"insert+p1",option:"comma enumeration S"}}},"\\color{(...)}{(...)}1|\\color(...){(...)}2":{"*":{action_:"color-output"}},"\\color{(...)}0":{"*":{action_:"color0-output"}},"\\ce{(...)}":{"*":{action_:"ce"}},"\\,|\\x{}{}|\\x{}|\\x":{"*":{action_:"copy"}},else2:{"*":{action_:"copy"}}}),actions:{"state of aggregation":function(t,r){return{type_:"state of aggregation subscript",p1:Ge.go(r,"o")}},"color-output":function(t,r){return{type_:"color",color1:r[0],color2:Ge.go(r[1],"pq")}}}},bd:{transitions:Ge.createTransitions({empty:{"*":{}},x$:{0:{nextState:"!f",revisit:!0}},formula$:{0:{nextState:"f",revisit:!0}},else:{0:{nextState:"!f",revisit:!0}},"-9.,9 no missing 0":{"*":{action_:"9,9"}},".":{"*":{action_:{type_:"insert",option:"electron dot"}}},"a-z":{f:{action_:"tex-math"}},x:{"*":{action_:{type_:"insert",option:"KV x"}}},letters:{"*":{action_:"rm"}},"'":{"*":{action_:{type_:"insert",option:"prime"}}},"${(...)}$|$(...)$":{"*":{action_:"tex-math"}},"{(...)}":{"*":{action_:"text"}},"\\color{(...)}{(...)}1|\\color(...){(...)}2":{"*":{action_:"color-output"}},"\\color{(...)}0":{"*":{action_:"color0-output"}},"\\ce{(...)}":{"*":{action_:"ce"}},"\\,|\\x{}{}|\\x{}|\\x":{"*":{action_:"copy"}},else2:{"*":{action_:"copy"}}}),actions:{"color-output":function(t,r){return{type_:"color",color1:r[0],color2:Ge.go(r[1],"bd")}}}},oxidation:{transitions:Ge.createTransitions({empty:{"*":{}},"roman numeral":{"*":{action_:"roman-numeral"}},"${(...)}$|$(...)$":{"*":{action_:"tex-math"}},else:{"*":{action_:"copy"}}}),actions:{"roman-numeral":function(t,r){return{type_:"roman numeral",p1:r||""}}}},"tex-math":{transitions:Ge.createTransitions({empty:{"*":{action_:"output"}},"\\ce{(...)}":{"*":{action_:["output","ce"]}},"{...}|\\,|\\x{}{}|\\x{}|\\x":{"*":{action_:"o="}},else:{"*":{action_:"o="}}}),actions:{output:function(t){if(t.o){var r={type_:"tex-math",p1:t.o};for(var n in t)delete t[n];return r}}}},"tex-math tight":{transitions:Ge.createTransitions({empty:{"*":{action_:"output"}},"\\ce{(...)}":{"*":{action_:["output","ce"]}},"{...}|\\,|\\x{}{}|\\x{}|\\x":{"*":{action_:"o="}},"-|+":{"*":{action_:"tight operator"}},else:{"*":{action_:"o="}}}),actions:{"tight operator":function(t,r){t.o=(t.o||"")+"{"+r+"}"},output:function(t){if(t.o){var r={type_:"tex-math",p1:t.o};for(var n in t)delete t[n];return r}}}},"9,9":{transitions:Ge.createTransitions({empty:{"*":{}},",":{"*":{action_:"comma"}},else:{"*":{action_:"copy"}}}),actions:{comma:function(){return{type_:"commaDecimal"}}}},pu:{transitions:Ge.createTransitions({empty:{"*":{action_:"output"}},space$:{"*":{action_:["output","space"]}},"{[(|)]}":{"0|a":{action_:"copy"}},"(-)(9)^(-9)":{0:{action_:"number^",nextState:"a"}},"(-)(9.,9)(e)(99)":{0:{action_:"enumber",nextState:"a"}},space:{"0|a":{}},"pm-operator":{"0|a":{action_:{type_:"operator",option:"\\pm"},nextState:"0"}},operator:{"0|a":{action_:"copy",nextState:"0"}},"//":{d:{action_:"o=",nextState:"/"}},"/":{d:{action_:"o=",nextState:"/"}},"{...}|else":{"0|d":{action_:"d=",nextState:"d"},a:{action_:["space","d="],nextState:"d"},"/|q":{action_:"q=",nextState:"q"}}}),actions:{enumber:function(t,r){var n=[];return r[0]==="+-"||r[0]==="+/-"?n.push("\\pm "):r[0]&&n.push(r[0]),r[1]&&(Ge.concatArray(n,Ge.go(r[1],"pu-9,9")),r[2]&&(r[2].match(/[,.]/)?Ge.concatArray(n,Ge.go(r[2],"pu-9,9")):n.push(r[2])),r[3]=r[4]||r[3],r[3]&&(r[3]=r[3].trim(),r[3]==="e"||r[3].substr(0,1)==="*"?n.push({type_:"cdot"}):n.push({type_:"times"}))),r[3]&&n.push("10^{"+r[5]+"}"),n},"number^":function(t,r){var n=[];return r[0]==="+-"||r[0]==="+/-"?n.push("\\pm "):r[0]&&n.push(r[0]),Ge.concatArray(n,Ge.go(r[1],"pu-9,9")),n.push("^{"+r[2]+"}"),n},operator:function(t,r,n){return{type_:"operator",kind_:n||r}},space:function(){return{type_:"pu-space-1"}},output:function(t){var r,n=Ge.patterns.match_("{(...)}",t.d||"");n&&n.remainder===""&&(t.d=n.match_);var a=Ge.patterns.match_("{(...)}",t.q||"");if(a&&a.remainder===""&&(t.q=a.match_),t.d&&(t.d=t.d.replace(/\u00B0C|\^oC|\^{o}C/g,"{}^{\\circ}C"),t.d=t.d.replace(/\u00B0F|\^oF|\^{o}F/g,"{}^{\\circ}F")),t.q){t.q=t.q.replace(/\u00B0C|\^oC|\^{o}C/g,"{}^{\\circ}C"),t.q=t.q.replace(/\u00B0F|\^oF|\^{o}F/g,"{}^{\\circ}F");var i={d:Ge.go(t.d,"pu"),q:Ge.go(t.q,"pu")};t.o==="//"?r={type_:"pu-frac",p1:i.d,p2:i.q}:(r=i.d,i.d.length>1||i.q.length>1?r.push({type_:" / "}):r.push({type_:"/"}),Ge.concatArray(r,i.q))}else r=Ge.go(t.d,"pu-2");for(var o in t)delete t[o];return r}}},"pu-2":{transitions:Ge.createTransitions({empty:{"*":{action_:"output"}},"*":{"*":{action_:["output","cdot"],nextState:"0"}},"\\x":{"*":{action_:"rm="}},space:{"*":{action_:["output","space"],nextState:"0"}},"^{(...)}|^(-1)":{1:{action_:"^(-1)"}},"-9.,9":{0:{action_:"rm=",nextState:"0"},1:{action_:"^(-1)",nextState:"0"}},"{...}|else":{"*":{action_:"rm=",nextState:"1"}}}),actions:{cdot:function(){return{type_:"tight cdot"}},"^(-1)":function(t,r){t.rm+="^{"+r+"}"},space:function(){return{type_:"pu-space-2"}},output:function(t){var r=[];if(t.rm){var n=Ge.patterns.match_("{(...)}",t.rm||"");n&&n.remainder===""?r=Ge.go(n.match_,"pu"):r={type_:"rm",p1:t.rm}}for(var a in t)delete t[a];return r}}},"pu-9,9":{transitions:Ge.createTransitions({empty:{0:{action_:"output-0"},o:{action_:"output-o"}},",":{0:{action_:["output-0","comma"],nextState:"o"}},".":{0:{action_:["output-0","copy"],nextState:"o"}},else:{"*":{action_:"text="}}}),actions:{comma:function(){return{type_:"commaDecimal"}},"output-0":function(t){var r=[];if(t.text_=t.text_||"",t.text_.length>4){var n=t.text_.length%3;n===0&&(n=3);for(var a=t.text_.length-3;a>0;a-=3)r.push(t.text_.substr(a,3)),r.push({type_:"1000 separator"});r.push(t.text_.substr(0,n)),r.reverse()}else r.push(t.text_);for(var i in t)delete t[i];return r},"output-o":function(t){var r=[];if(t.text_=t.text_||"",t.text_.length>4){for(var n=t.text_.length-3,a=0;a<n;a+=3)r.push(t.text_.substr(a,3)),r.push({type_:"1000 separator"});r.push(t.text_.substr(a))}else r.push(t.text_);for(var i in t)delete t[i];return r}}}};var en={go:function(t,r){if(!t)return"";for(var n="",a=!1,i=0;i<t.length;i++){var o=t[i];typeof o=="string"?n+=o:(n+=en._go2(o),o.type_==="1st-level escape"&&(a=!0))}return!r&&!a&&n&&(n="{"+n+"}"),n},_goInner:function(t){return t&&en.go(t,!0)},_go2:function(t){var r;switch(t.type_){case"chemfive":r="";var n={a:en._goInner(t.a),b:en._goInner(t.b),p:en._goInner(t.p),o:en._goInner(t.o),q:en._goInner(t.q),d:en._goInner(t.d)};n.a&&(n.a.match(/^[+\-]/)&&(n.a="{"+n.a+"}"),r+=n.a+"\\,"),(n.b||n.p)&&(r+="{\\vphantom{X}}",r+="^{\\hphantom{"+(n.b||"")+"}}_{\\hphantom{"+(n.p||"")+"}}",r+="{\\vphantom{X}}",r+="^{\\smash[t]{\\vphantom{2}}\\mathllap{"+(n.b||"")+"}}",r+="_{\\vphantom{2}\\mathllap{\\smash[t]{"+(n.p||"")+"}}}"),n.o&&(n.o.match(/^[+\-]/)&&(n.o="{"+n.o+"}"),r+=n.o),t.dType==="kv"?((n.d||n.q)&&(r+="{\\vphantom{X}}"),n.d&&(r+="^{"+n.d+"}"),n.q&&(r+="_{\\smash[t]{"+n.q+"}}")):t.dType==="oxidation"?(n.d&&(r+="{\\vphantom{X}}",r+="^{"+n.d+"}"),n.q&&(r+="{\\vphantom{X}}",r+="_{\\smash[t]{"+n.q+"}}")):(n.q&&(r+="{\\vphantom{X}}",r+="_{\\smash[t]{"+n.q+"}}"),n.d&&(r+="{\\vphantom{X}}",r+="^{"+n.d+"}"));break;case"rm":r="\\mathrm{"+t.p1+"}";break;case"text":t.p1.match(/[\^_]/)?(t.p1=t.p1.replace(" ","~").replace("-","\\text{-}"),r="\\mathrm{"+t.p1+"}"):r="\\text{"+t.p1+"}";break;case"roman numeral":r="\\mathrm{"+t.p1+"}";break;case"state of aggregation":r="\\mskip2mu "+en._goInner(t.p1);break;case"state of aggregation subscript":r="\\mskip1mu "+en._goInner(t.p1);break;case"bond":if(r=en._getBond(t.kind_),!r)throw["MhchemErrorBond","mhchem Error. Unknown bond type ("+t.kind_+")"];break;case"frac":var a="\\frac{"+t.p1+"}{"+t.p2+"}";r="\\mathchoice{\\textstyle"+a+"}{"+a+"}{"+a+"}{"+a+"}";break;case"pu-frac":var i="\\frac{"+en._goInner(t.p1)+"}{"+en._goInner(t.p2)+"}";r="\\mathchoice{\\textstyle"+i+"}{"+i+"}{"+i+"}{"+i+"}";break;case"tex-math":r=t.p1+" ";break;case"frac-ce":r="\\frac{"+en._goInner(t.p1)+"}{"+en._goInner(t.p2)+"}";break;case"overset":r="\\overset{"+en._goInner(t.p1)+"}{"+en._goInner(t.p2)+"}";break;case"underset":r="\\underset{"+en._goInner(t.p1)+"}{"+en._goInner(t.p2)+"}";break;case"underbrace":r="\\underbrace{"+en._goInner(t.p1)+"}_{"+en._goInner(t.p2)+"}";break;case"color":r="{\\color{"+t.color1+"}{"+en._goInner(t.color2)+"}}";break;case"color0":r="\\color{"+t.color+"}";break;case"arrow":var o={rd:en._goInner(t.rd),rq:en._goInner(t.rq)},s="\\x"+en._getArrow(t.r);o.rq&&(s+="[{"+o.rq+"}]"),o.rd?s+="{"+o.rd+"}":s+="{}",r=s;break;case"operator":r=en._getOperator(t.kind_);break;case"1st-level escape":r=t.p1+" ";break;case"space":r=" ";break;case"entitySkip":r="~";break;case"pu-space-1":r="~";break;case"pu-space-2":r="\\mkern3mu ";break;case"1000 separator":r="\\mkern2mu ";break;case"commaDecimal":r="{,}";break;case"comma enumeration L":r="{"+t.p1+"}\\mkern6mu ";break;case"comma enumeration M":r="{"+t.p1+"}\\mkern3mu ";break;case"comma enumeration S":r="{"+t.p1+"}\\mkern1mu ";break;case"hyphen":r="\\text{-}";break;case"addition compound":r="\\,{\\cdot}\\,";break;case"electron dot":r="\\mkern1mu \\bullet\\mkern1mu ";break;case"KV x":r="{\\times}";break;case"prime":r="\\prime ";break;case"cdot":r="\\cdot ";break;case"tight cdot":r="\\mkern1mu{\\cdot}\\mkern1mu ";break;case"times":r="\\times ";break;case"circa":r="{\\sim}";break;case"^":r="uparrow";break;case"v":r="downarrow";break;case"ellipsis":r="\\ldots ";break;case"/":r="/";break;case" / ":r="\\,/\\,";break;default:throw["MhchemBugT","mhchem bug T. Please report."]}return r},_getArrow:function(t){switch(t){case"->":return"rightarrow";case"→":return"rightarrow";case"⟶":return"rightarrow";case"<-":return"leftarrow";case"<->":return"leftrightarrow";case"<-->":return"rightleftarrows";case"<=>":return"rightleftharpoons";case"⇌":return"rightleftharpoons";case"<=>>":return"rightequilibrium";case"<<=>":return"leftequilibrium";default:throw["MhchemBugT","mhchem bug T. Please report."]}},_getBond:function(t){switch(t){case"-":return"{-}";case"1":return"{-}";case"=":return"{=}";case"2":return"{=}";case"#":return"{\\equiv}";case"3":return"{\\equiv}";case"~":return"{\\tripledash}";case"~-":return"{\\mathrlap{\\raisebox{-.1em}{$-$}}\\raisebox{.1em}{$\\tripledash$}}";case"~=":return"{\\mathrlap{\\raisebox{-.2em}{$-$}}\\mathrlap{\\raisebox{.2em}{$\\tripledash$}}-}";case"~--":return"{\\mathrlap{\\raisebox{-.2em}{$-$}}\\mathrlap{\\raisebox{.2em}{$\\tripledash$}}-}";case"-~-":return"{\\mathrlap{\\raisebox{-.2em}{$-$}}\\mathrlap{\\raisebox{.2em}{$-$}}\\tripledash}";case"...":return"{{\\cdot}{\\cdot}{\\cdot}}";case"....":return"{{\\cdot}{\\cdot}{\\cdot}{\\cdot}}";case"->":return"{\\rightarrow}";case"<-":return"{\\leftarrow}";case"<":return"{<}";case">":return"{>}";default:throw["MhchemBugT","mhchem bug T. Please report."]}},_getOperator:function(t){switch(t){case"+":return" {}+{} ";case"-":return" {}-{} ";case"=":return" {}={} ";case"<":return" {}<{} ";case">":return" {}>{} ";case"<<":return" {}\\ll{} ";case">>":return" {}\\gg{} ";case"\\pm":return" {}\\pm{} ";case"\\approx":return" {}\\approx{} ";case"$\\approx$":return" {}\\approx{} ";case"v":return" \\downarrow{} ";case"(v)":return" \\downarrow{} ";case"^":return" \\uparrow{} ";case"(^)":return" \\uparrow{} ";default:throw["MhchemBugT","mhchem bug T. Please report."]}}};/**
 * react-router v7.9.4
 *
 * Copyright (c) Remix Software Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */var X9="popstate";function ime(e={}){function t(a,i){let{pathname:o="/",search:s="",hash:l=""}=td(a.location.hash.substring(1));return!o.startsWith("/")&&!o.startsWith(".")&&(o="/"+o),R_("",{pathname:o,search:s,hash:l},i.state&&i.state.usr||null,i.state&&i.state.key||"default")}function r(a,i){let o=a.document.querySelector("base"),s="";if(o&&o.getAttribute("href")){let l=a.location.href,u=l.indexOf("#");s=u===-1?l:l.slice(0,u)}return s+"#"+(typeof i=="string"?i:Om(i))}function n(a,i){Lo(a.pathname.charAt(0)==="/",`relative pathnames are not supported in hash history.push(${JSON.stringify(i)})`)}return sme(t,r,n,e)}function wn(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}function Lo(e,t){if(!e){typeof console<"u"&&console.warn(t);try{throw new Error(t)}catch{}}}function ome(){return Math.random().toString(36).substring(2,10)}function Z9(e,t){return{usr:e.state,key:e.key,idx:t}}function R_(e,t,r=null,n){return{pathname:typeof e=="string"?e:e.pathname,search:"",hash:"",...typeof t=="string"?td(t):t,state:r,key:t&&t.key||n||ome()}}function Om({pathname:e="/",search:t="",hash:r=""}){return t&&t!=="?"&&(e+=t.charAt(0)==="?"?t:"?"+t),r&&r!=="#"&&(e+=r.charAt(0)==="#"?r:"#"+r),e}function td(e){let t={};if(e){let r=e.indexOf("#");r>=0&&(t.hash=e.substring(r),e=e.substring(0,r));let n=e.indexOf("?");n>=0&&(t.search=e.substring(n),e=e.substring(0,n)),e&&(t.pathname=e)}return t}function sme(e,t,r,n={}){let{window:a=document.defaultView,v5Compat:i=!1}=n,o=a.history,s="POP",l=null,u=d();u==null&&(u=0,o.replaceState({...o.state,idx:u},""));function d(){return(o.state||{idx:null}).idx}function p(){s="POP";let S=d(),w=S==null?null:S-u;u=S,l&&l({action:s,location:y.location,delta:w})}function f(S,w){s="PUSH";let E=R_(y.location,S,w);r&&r(E,S),u=d()+1;let k=Z9(E,u),_=y.createHref(E);try{o.pushState(k,"",_)}catch(R){if(R instanceof DOMException&&R.name==="DataCloneError")throw R;a.location.assign(_)}i&&l&&l({action:s,location:y.location,delta:1})}function m(S,w){s="REPLACE";let E=R_(y.location,S,w);r&&r(E,S),u=d();let k=Z9(E,u),_=y.createHref(E);o.replaceState(k,"",_),i&&l&&l({action:s,location:y.location,delta:0})}function b(S){return lme(S)}let y={get action(){return s},get location(){return e(a,o)},listen(S){if(l)throw new Error("A history only accepts one active listener");return a.addEventListener(X9,p),l=S,()=>{a.removeEventListener(X9,p),l=null}},createHref(S){return t(a,S)},createURL:b,encodeLocation(S){let w=b(S);return{pathname:w.pathname,search:w.search,hash:w.hash}},push:f,replace:m,go(S){return o.go(S)}};return y}function lme(e,t=!1){let r="http://localhost";typeof window<"u"&&(r=window.location.origin!=="null"?window.location.origin:window.location.href),wn(r,"No window.location.(origin|href) available to create URL");let n=typeof e=="string"?e:Om(e);return n=n.replace(/ $/,"%20"),!t&&n.startsWith("//")&&(n=r+n),new URL(n,r)}function fY(e,t,r="/"){return ume(e,t,r,!1)}function ume(e,t,r,n){let a=typeof t=="string"?td(t):t,i=gl(a.pathname||"/",r);if(i==null)return null;let o=mY(e);cme(o);let s=null;for(let l=0;s==null&&l<o.length;++l){let u=Eme(i);s=vme(o[l],u,n)}return s}function mY(e,t=[],r=[],n="",a=!1){let i=(o,s,l=a,u)=>{let d={relativePath:u===void 0?o.path||"":u,caseSensitive:o.caseSensitive===!0,childrenIndex:s,route:o};if(d.relativePath.startsWith("/")){if(!d.relativePath.startsWith(n)&&l)return;wn(d.relativePath.startsWith(n),`Absolute route path "${d.relativePath}" nested under path "${n}" is not valid. An absolute child route path must start with the combined path of all its parent routes.`),d.relativePath=d.relativePath.slice(n.length)}let p=ul([n,d.relativePath]),f=r.concat(d);o.children&&o.children.length>0&&(wn(o.index!==!0,`Index routes must not have child routes. Please remove all child routes from route path "${p}".`),mY(o.children,t,f,p,l)),!(o.path==null&&!o.index)&&t.push({path:p,score:bme(p,o.index),routesMeta:f})};return e.forEach((o,s)=>{var l;if(o.path===""||!((l=o.path)!=null&&l.includes("?")))i(o,s);else for(let u of gY(o.path))i(o,s,!0,u)}),t}function gY(e){let t=e.split("/");if(t.length===0)return[];let[r,...n]=t,a=r.endsWith("?"),i=r.replace(/\?$/,"");if(n.length===0)return a?[i,""]:[i];let o=gY(n.join("/")),s=[];return s.push(...o.map(l=>l===""?i:[i,l].join("/"))),a&&s.push(...o),s.map(l=>e.startsWith("/")&&l===""?"/":l)}function cme(e){e.sort((t,r)=>t.score!==r.score?r.score-t.score:yme(t.routesMeta.map(n=>n.childrenIndex),r.routesMeta.map(n=>n.childrenIndex)))}var dme=/^:[\w-]+$/,hme=3,pme=2,fme=1,mme=10,gme=-2,Q9=e=>e==="*";function bme(e,t){let r=e.split("/"),n=r.length;return r.some(Q9)&&(n+=gme),t&&(n+=pme),r.filter(a=>!Q9(a)).reduce((a,i)=>a+(dme.test(i)?hme:i===""?fme:mme),n)}function yme(e,t){return e.length===t.length&&e.slice(0,-1).every((n,a)=>n===t[a])?e[e.length-1]-t[t.length-1]:0}function vme(e,t,r=!1){let{routesMeta:n}=e,a={},i="/",o=[];for(let s=0;s<n.length;++s){let l=n[s],u=s===n.length-1,d=i==="/"?t:t.slice(i.length)||"/",p=oy({path:l.relativePath,caseSensitive:l.caseSensitive,end:u},d),f=l.route;if(!p&&u&&r&&!n[n.length-1].route.index&&(p=oy({path:l.relativePath,caseSensitive:l.caseSensitive,end:!1},d)),!p)return null;Object.assign(a,p.params),o.push({params:a,pathname:ul([i,p.pathname]),pathnameBase:Cme(ul([i,p.pathnameBase])),route:f}),p.pathnameBase!=="/"&&(i=ul([i,p.pathnameBase]))}return o}function oy(e,t){typeof e=="string"&&(e={path:e,caseSensitive:!1,end:!0});let[r,n]=Sme(e.path,e.caseSensitive,e.end),a=t.match(r);if(!a)return null;let i=a[0],o=i.replace(/(.)\/+$/,"$1"),s=a.slice(1);return{params:n.reduce((u,{paramName:d,isOptional:p},f)=>{if(d==="*"){let b=s[f]||"";o=i.slice(0,i.length-b.length).replace(/(.)\/+$/,"$1")}const m=s[f];return p&&!m?u[d]=void 0:u[d]=(m||"").replace(/%2F/g,"/"),u},{}),pathname:i,pathnameBase:o,pattern:e}}function Sme(e,t=!1,r=!0){Lo(e==="*"||!e.endsWith("*")||e.endsWith("/*"),`Route path "${e}" will be treated as if it were "${e.replace(/\*$/,"/*")}" because the \`*\` character must always follow a \`/\` in the pattern. To get rid of this warning, please change the route path to "${e.replace(/\*$/,"/*")}".`);let n=[],a="^"+e.replace(/\/*\*?$/,"").replace(/^\/*/,"/").replace(/[\\.*+^${}|()[\]]/g,"\\$&").replace(/\/:([\w-]+)(\?)?/g,(o,s,l)=>(n.push({paramName:s,isOptional:l!=null}),l?"/?([^\\/]+)?":"/([^\\/]+)")).replace(/\/([\w-]+)\?(\/|$)/g,"(/$1)?$2");return e.endsWith("*")?(n.push({paramName:"*"}),a+=e==="*"||e==="/*"?"(.*)$":"(?:\\/(.+)|\\/*)$"):r?a+="\\/*$":e!==""&&e!=="/"&&(a+="(?:(?=\\/|$))"),[new RegExp(a,t?void 0:"i"),n]}function Eme(e){try{return e.split("/").map(t=>decodeURIComponent(t).replace(/\//g,"%2F")).join("/")}catch(t){return Lo(!1,`The URL path "${e}" could not be decoded because it is a malformed URL segment. This is probably due to a bad percent encoding (${t}).`),e}}function gl(e,t){if(t==="/")return e;if(!e.toLowerCase().startsWith(t.toLowerCase()))return null;let r=t.endsWith("/")?t.length-1:t.length,n=e.charAt(r);return n&&n!=="/"?null:e.slice(r)||"/"}function xme(e,t="/"){let{pathname:r,search:n="",hash:a=""}=typeof e=="string"?td(e):e;return{pathname:r?r.startsWith("/")?r:wme(r,t):t,search:Tme(n),hash:Ame(a)}}function wme(e,t){let r=t.replace(/\/+$/,"").split("/");return e.split("/").forEach(a=>{a===".."?r.length>1&&r.pop():a!=="."&&r.push(a)}),r.length>1?r.join("/"):"/"}function Ox(e,t,r,n){return`Cannot include a '${e}' character in a manually specified \`to.${t}\` field [${JSON.stringify(n)}].  Please separate it out to the \`to.${r}\` field. Alternatively you may provide the full path as a string in <Link to="..."> and the router will parse it for you.`}function kme(e){return e.filter((t,r)=>r===0||t.route.path&&t.route.path.length>0)}function bY(e){let t=kme(e);return t.map((r,n)=>n===t.length-1?r.pathname:r.pathnameBase)}function yY(e,t,r,n=!1){let a;typeof e=="string"?a=td(e):(a={...e},wn(!a.pathname||!a.pathname.includes("?"),Ox("?","pathname","search",a)),wn(!a.pathname||!a.pathname.includes("#"),Ox("#","pathname","hash",a)),wn(!a.search||!a.search.includes("#"),Ox("#","search","hash",a)));let i=e===""||a.pathname==="",o=i?"/":a.pathname,s;if(o==null)s=r;else{let p=t.length-1;if(!n&&o.startsWith("..")){let f=o.split("/");for(;f[0]==="..";)f.shift(),p-=1;a.pathname=f.join("/")}s=p>=0?t[p]:"/"}let l=xme(a,s),u=o&&o!=="/"&&o.endsWith("/"),d=(i||o===".")&&r.endsWith("/");return!l.pathname.endsWith("/")&&(u||d)&&(l.pathname+="/"),l}var ul=e=>e.join("/").replace(/\/\/+/g,"/"),Cme=e=>e.replace(/\/+$/,"").replace(/^\/*/,"/"),Tme=e=>!e||e==="?"?"":e.startsWith("?")?e:"?"+e,Ame=e=>!e||e==="#"?"":e.startsWith("#")?e:"#"+e;function Fme(e){return e!=null&&typeof e.status=="number"&&typeof e.statusText=="string"&&typeof e.internal=="boolean"&&"data"in e}var vY=["POST","PUT","PATCH","DELETE"];new Set(vY);var _me=["GET",...vY];new Set(_me);var lp=T.createContext(null);lp.displayName="DataRouter";var _v=T.createContext(null);_v.displayName="DataRouterState";T.createContext(!1);var SY=T.createContext({isTransitioning:!1});SY.displayName="ViewTransition";var Rme=T.createContext(new Map);Rme.displayName="Fetchers";var Dme=T.createContext(null);Dme.displayName="Await";var ms=T.createContext(null);ms.displayName="Navigation";var hg=T.createContext(null);hg.displayName="Location";var Al=T.createContext({outlet:null,matches:[],isDataRoute:!1});Al.displayName="Route";var kD=T.createContext(null);kD.displayName="RouteError";function Nme(e,{relative:t}={}){wn(pg(),"useHref() may be used only in the context of a <Router> component.");let{basename:r,navigator:n}=T.useContext(ms),{hash:a,pathname:i,search:o}=fg(e,{relative:t}),s=i;return r!=="/"&&(s=i==="/"?r:ul([r,i])),n.createHref({pathname:s,search:o,hash:a})}function pg(){return T.useContext(hg)!=null}function rd(){return wn(pg(),"useLocation() may be used only in the context of a <Router> component."),T.useContext(hg).location}var EY="You should call navigate() in a React.useEffect(), not when your component is first rendered.";function xY(e){T.useContext(ms).static||T.useLayoutEffect(e)}function CD(){let{isDataRoute:e}=T.useContext(Al);return e?qme():Lme()}function Lme(){wn(pg(),"useNavigate() may be used only in the context of a <Router> component.");let e=T.useContext(lp),{basename:t,navigator:r}=T.useContext(ms),{matches:n}=T.useContext(Al),{pathname:a}=rd(),i=JSON.stringify(bY(n)),o=T.useRef(!1);return xY(()=>{o.current=!0}),T.useCallback((l,u={})=>{if(Lo(o.current,EY),!o.current)return;if(typeof l=="number"){r.go(l);return}let d=yY(l,JSON.parse(i),a,u.relative==="path");e==null&&t!=="/"&&(d.pathname=d.pathname==="/"?t:ul([t,d.pathname])),(u.replace?r.replace:r.push)(d,u.state,u)},[t,r,i,a,e])}T.createContext(null);function fg(e,{relative:t}={}){let{matches:r}=T.useContext(Al),{pathname:n}=rd(),a=JSON.stringify(bY(r));return T.useMemo(()=>yY(e,JSON.parse(a),n,t==="path"),[e,a,n,t])}function Mme(e,t){return wY(e,t)}function wY(e,t,r,n,a){var E;wn(pg(),"useRoutes() may be used only in the context of a <Router> component.");let{navigator:i}=T.useContext(ms),{matches:o}=T.useContext(Al),s=o[o.length-1],l=s?s.params:{},u=s?s.pathname:"/",d=s?s.pathnameBase:"/",p=s&&s.route;{let k=p&&p.path||"";kY(u,!p||k.endsWith("*")||k.endsWith("*?"),`You rendered descendant <Routes> (or called \`useRoutes()\`) at "${u}" (under <Route path="${k}">) but the parent route path has no trailing "*". This means if you navigate deeper, the parent won't match anymore and therefore the child routes will never render.

Please change the parent <Route path="${k}"> to <Route path="${k==="/"?"*":`${k}/*`}">.`)}let f=rd(),m;if(t){let k=typeof t=="string"?td(t):t;wn(d==="/"||((E=k.pathname)==null?void 0:E.startsWith(d)),`When overriding the location using \`<Routes location>\` or \`useRoutes(routes, location)\`, the location pathname must begin with the portion of the URL pathname that was matched by all parent routes. The current pathname base is "${d}" but pathname "${k.pathname}" was given in the \`location\` prop.`),m=k}else m=f;let b=m.pathname||"/",y=b;if(d!=="/"){let k=d.replace(/^\//,"").split("/");y="/"+b.replace(/^\//,"").split("/").slice(k.length).join("/")}let S=fY(e,{pathname:y});Lo(p||S!=null,`No routes matched location "${m.pathname}${m.search}${m.hash}" `),Lo(S==null||S[S.length-1].route.element!==void 0||S[S.length-1].route.Component!==void 0||S[S.length-1].route.lazy!==void 0,`Matched leaf route at location "${m.pathname}${m.search}${m.hash}" does not have an element or Component. This means it will render an <Outlet /> with a null value by default resulting in an "empty" page.`);let w=zme(S&&S.map(k=>Object.assign({},k,{params:Object.assign({},l,k.params),pathname:ul([d,i.encodeLocation?i.encodeLocation(k.pathname.replace(/\?/g,"%3F").replace(/#/g,"%23")).pathname:k.pathname]),pathnameBase:k.pathnameBase==="/"?d:ul([d,i.encodeLocation?i.encodeLocation(k.pathnameBase.replace(/\?/g,"%3F").replace(/#/g,"%23")).pathname:k.pathnameBase])})),o,r,n,a);return t&&w?T.createElement(hg.Provider,{value:{location:{pathname:"/",search:"",hash:"",state:null,key:"default",...m},navigationType:"POP"}},w):w}function Ime(){let e=jme(),t=Fme(e)?`${e.status} ${e.statusText}`:e instanceof Error?e.message:JSON.stringify(e),r=e instanceof Error?e.stack:null,n="rgba(200,200,200, 0.5)",a={padding:"0.5rem",backgroundColor:n},i={padding:"2px 4px",backgroundColor:n},o=null;return console.error("Error handled by React Router default ErrorBoundary:",e),o=T.createElement(T.Fragment,null,T.createElement("p",null,"💿 Hey developer 👋"),T.createElement("p",null,"You can provide a way better UX than this when your app throws errors by providing your own ",T.createElement("code",{style:i},"ErrorBoundary")," or"," ",T.createElement("code",{style:i},"errorElement")," prop on your route.")),T.createElement(T.Fragment,null,T.createElement("h2",null,"Unexpected Application Error!"),T.createElement("h3",{style:{fontStyle:"italic"}},t),r?T.createElement("pre",{style:a},r):null,o)}var Ome=T.createElement(Ime,null),Bme=class extends T.Component{constructor(e){super(e),this.state={location:e.location,revalidation:e.revalidation,error:e.error}}static getDerivedStateFromError(e){return{error:e}}static getDerivedStateFromProps(e,t){return t.location!==e.location||t.revalidation!=="idle"&&e.revalidation==="idle"?{error:e.error,location:e.location,revalidation:e.revalidation}:{error:e.error!==void 0?e.error:t.error,location:t.location,revalidation:e.revalidation||t.revalidation}}componentDidCatch(e,t){this.props.unstable_onError?this.props.unstable_onError(e,t):console.error("React Router caught the following error during render",e)}render(){return this.state.error!==void 0?T.createElement(Al.Provider,{value:this.props.routeContext},T.createElement(kD.Provider,{value:this.state.error,children:this.props.component})):this.props.children}};function Pme({routeContext:e,match:t,children:r}){let n=T.useContext(lp);return n&&n.static&&n.staticContext&&(t.route.errorElement||t.route.ErrorBoundary)&&(n.staticContext._deepestRenderedBoundaryId=t.route.id),T.createElement(Al.Provider,{value:e},r)}function zme(e,t=[],r=null,n=null,a=null){if(e==null){if(!r)return null;if(r.errors)e=r.matches;else if(t.length===0&&!r.initialized&&r.matches.length>0)e=r.matches;else return null}let i=e,o=r==null?void 0:r.errors;if(o!=null){let u=i.findIndex(d=>d.route.id&&(o==null?void 0:o[d.route.id])!==void 0);wn(u>=0,`Could not find a matching route for errors on route IDs: ${Object.keys(o).join(",")}`),i=i.slice(0,Math.min(i.length,u+1))}let s=!1,l=-1;if(r)for(let u=0;u<i.length;u++){let d=i[u];if((d.route.HydrateFallback||d.route.hydrateFallbackElement)&&(l=u),d.route.id){let{loaderData:p,errors:f}=r,m=d.route.loader&&!p.hasOwnProperty(d.route.id)&&(!f||f[d.route.id]===void 0);if(d.route.lazy||m){s=!0,l>=0?i=i.slice(0,l+1):i=[i[0]];break}}}return i.reduceRight((u,d,p)=>{let f,m=!1,b=null,y=null;r&&(f=o&&d.route.id?o[d.route.id]:void 0,b=d.route.errorElement||Ome,s&&(l<0&&p===0?(kY("route-fallback",!1,"No `HydrateFallback` element provided to render during initial hydration"),m=!0,y=null):l===p&&(m=!0,y=d.route.hydrateFallbackElement||null)));let S=t.concat(i.slice(0,p+1)),w=()=>{let E;return f?E=b:m?E=y:d.route.Component?E=T.createElement(d.route.Component,null):d.route.element?E=d.route.element:E=u,T.createElement(Pme,{match:d,routeContext:{outlet:u,matches:S,isDataRoute:r!=null},children:E})};return r&&(d.route.ErrorBoundary||d.route.errorElement||p===0)?T.createElement(Bme,{location:r.location,revalidation:r.revalidation,component:b,error:f,children:w(),routeContext:{outlet:null,matches:S,isDataRoute:!0},unstable_onError:n}):w()},null)}function TD(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function Hme(e){let t=T.useContext(lp);return wn(t,TD(e)),t}function Ume(e){let t=T.useContext(_v);return wn(t,TD(e)),t}function $me(e){let t=T.useContext(Al);return wn(t,TD(e)),t}function AD(e){let t=$me(e),r=t.matches[t.matches.length-1];return wn(r.route.id,`${e} can only be used on routes that contain a unique "id"`),r.route.id}function Gme(){return AD("useRouteId")}function jme(){var n;let e=T.useContext(kD),t=Ume("useRouteError"),r=AD("useRouteError");return e!==void 0?e:(n=t.errors)==null?void 0:n[r]}function qme(){let{router:e}=Hme("useNavigate"),t=AD("useNavigate"),r=T.useRef(!1);return xY(()=>{r.current=!0}),T.useCallback(async(a,i={})=>{Lo(r.current,EY),r.current&&(typeof a=="number"?e.navigate(a):await e.navigate(a,{fromRouteId:t,...i}))},[e,t])}var e8={};function kY(e,t,r){!t&&!e8[e]&&(e8[e]=!0,Lo(!1,r))}T.memo(Wme);function Wme({routes:e,future:t,state:r,unstable_onError:n}){return wY(e,void 0,r,n,t)}function D_(e){wn(!1,"A <Route> is only ever to be used as the child of <Routes> element, never rendered directly. Please wrap your <Route> in a <Routes>.")}function Vme({basename:e="/",children:t=null,location:r,navigationType:n="POP",navigator:a,static:i=!1}){wn(!pg(),"You cannot render a <Router> inside another <Router>. You should never have more than one in your app.");let o=e.replace(/^\/*/,"/"),s=T.useMemo(()=>({basename:o,navigator:a,static:i,future:{}}),[o,a,i]);typeof r=="string"&&(r=td(r));let{pathname:l="/",search:u="",hash:d="",state:p=null,key:f="default"}=r,m=T.useMemo(()=>{let b=gl(l,o);return b==null?null:{location:{pathname:b,search:u,hash:d,state:p,key:f},navigationType:n}},[o,l,u,d,p,f,n]);return Lo(m!=null,`<Router basename="${o}"> is not able to match the URL "${l}${u}${d}" because it does not start with the basename, so the <Router> won't render anything.`),m==null?null:T.createElement(ms.Provider,{value:s},T.createElement(hg.Provider,{children:t,value:m}))}function Kme({children:e,location:t}){return Mme(N_(e),t)}function N_(e,t=[]){let r=[];return T.Children.forEach(e,(n,a)=>{if(!T.isValidElement(n))return;let i=[...t,a];if(n.type===T.Fragment){r.push.apply(r,N_(n.props.children,i));return}wn(n.type===D_,`[${typeof n.type=="string"?n.type:n.type.name}] is not a <Route> component. All component children of <Routes> must be a <Route> or <React.Fragment>`),wn(!n.props.index||!n.props.children,"An index route cannot have child routes.");let o={id:n.props.id||i.join("-"),caseSensitive:n.props.caseSensitive,element:n.props.element,Component:n.props.Component,index:n.props.index,path:n.props.path,middleware:n.props.middleware,loader:n.props.loader,action:n.props.action,hydrateFallbackElement:n.props.hydrateFallbackElement,HydrateFallback:n.props.HydrateFallback,errorElement:n.props.errorElement,ErrorBoundary:n.props.ErrorBoundary,hasErrorBoundary:n.props.hasErrorBoundary===!0||n.props.ErrorBoundary!=null||n.props.errorElement!=null,shouldRevalidate:n.props.shouldRevalidate,handle:n.props.handle,lazy:n.props.lazy};n.props.children&&(o.children=N_(n.props.children,i)),r.push(o)}),r}var y1="get",v1="application/x-www-form-urlencoded";function Rv(e){return e!=null&&typeof e.tagName=="string"}function Yme(e){return Rv(e)&&e.tagName.toLowerCase()==="button"}function Jme(e){return Rv(e)&&e.tagName.toLowerCase()==="form"}function Xme(e){return Rv(e)&&e.tagName.toLowerCase()==="input"}function Zme(e){return!!(e.metaKey||e.altKey||e.ctrlKey||e.shiftKey)}function Qme(e,t){return e.button===0&&(!t||t==="_self")&&!Zme(e)}var Sb=null;function ege(){if(Sb===null)try{new FormData(document.createElement("form"),0),Sb=!1}catch{Sb=!0}return Sb}var tge=new Set(["application/x-www-form-urlencoded","multipart/form-data","text/plain"]);function Bx(e){return e!=null&&!tge.has(e)?(Lo(!1,`"${e}" is not a valid \`encType\` for \`<Form>\`/\`<fetcher.Form>\` and will default to "${v1}"`),null):e}function rge(e,t){let r,n,a,i,o;if(Jme(e)){let s=e.getAttribute("action");n=s?gl(s,t):null,r=e.getAttribute("method")||y1,a=Bx(e.getAttribute("enctype"))||v1,i=new FormData(e)}else if(Yme(e)||Xme(e)&&(e.type==="submit"||e.type==="image")){let s=e.form;if(s==null)throw new Error('Cannot submit a <button> or <input type="submit"> without a <form>');let l=e.getAttribute("formaction")||s.getAttribute("action");if(n=l?gl(l,t):null,r=e.getAttribute("formmethod")||s.getAttribute("method")||y1,a=Bx(e.getAttribute("formenctype"))||Bx(s.getAttribute("enctype"))||v1,i=new FormData(s,e),!ege()){let{name:u,type:d,value:p}=e;if(d==="image"){let f=u?`${u}.`:"";i.append(`${f}x`,"0"),i.append(`${f}y`,"0")}else u&&i.append(u,p)}}else{if(Rv(e))throw new Error('Cannot submit element that is not <form>, <button>, or <input type="submit|image">');r=y1,n=null,a=v1,o=e}return i&&a==="text/plain"&&(o=i,i=void 0),{action:n,method:r.toLowerCase(),encType:a,formData:i,body:o}}Object.getOwnPropertyNames(Object.prototype).sort().join("\0");function FD(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}function nge(e,t,r){let n=typeof e=="string"?new URL(e,typeof window>"u"?"server://singlefetch/":window.location.origin):e;return n.pathname==="/"?n.pathname=`_root.${r}`:t&&gl(n.pathname,t)==="/"?n.pathname=`${t.replace(/\/$/,"")}/_root.${r}`:n.pathname=`${n.pathname.replace(/\/$/,"")}.${r}`,n}async function age(e,t){if(e.id in t)return t[e.id];try{let r=await import(e.module);return t[e.id]=r,r}catch(r){return console.error(`Error loading route module \`${e.module}\`, reloading page...`),console.error(r),window.__reactRouterContext&&window.__reactRouterContext.isSpaMode,window.location.reload(),new Promise(()=>{})}}function ige(e){return e==null?!1:e.href==null?e.rel==="preload"&&typeof e.imageSrcSet=="string"&&typeof e.imageSizes=="string":typeof e.rel=="string"&&typeof e.href=="string"}async function oge(e,t,r){let n=await Promise.all(e.map(async a=>{let i=t.routes[a.route.id];if(i){let o=await age(i,r);return o.links?o.links():[]}return[]}));return cge(n.flat(1).filter(ige).filter(a=>a.rel==="stylesheet"||a.rel==="preload").map(a=>a.rel==="stylesheet"?{...a,rel:"prefetch",as:"style"}:{...a,rel:"prefetch"}))}function t8(e,t,r,n,a,i){let o=(l,u)=>r[u]?l.route.id!==r[u].route.id:!0,s=(l,u)=>{var d;return r[u].pathname!==l.pathname||((d=r[u].route.path)==null?void 0:d.endsWith("*"))&&r[u].params["*"]!==l.params["*"]};return i==="assets"?t.filter((l,u)=>o(l,u)||s(l,u)):i==="data"?t.filter((l,u)=>{var p;let d=n.routes[l.route.id];if(!d||!d.hasLoader)return!1;if(o(l,u)||s(l,u))return!0;if(l.route.shouldRevalidate){let f=l.route.shouldRevalidate({currentUrl:new URL(a.pathname+a.search+a.hash,window.origin),currentParams:((p=r[0])==null?void 0:p.params)||{},nextUrl:new URL(e,window.origin),nextParams:l.params,defaultShouldRevalidate:!0});if(typeof f=="boolean")return f}return!0}):[]}function sge(e,t,{includeHydrateFallback:r}={}){return lge(e.map(n=>{let a=t.routes[n.route.id];if(!a)return[];let i=[a.module];return a.clientActionModule&&(i=i.concat(a.clientActionModule)),a.clientLoaderModule&&(i=i.concat(a.clientLoaderModule)),r&&a.hydrateFallbackModule&&(i=i.concat(a.hydrateFallbackModule)),a.imports&&(i=i.concat(a.imports)),i}).flat(1))}function lge(e){return[...new Set(e)]}function uge(e){let t={},r=Object.keys(e).sort();for(let n of r)t[n]=e[n];return t}function cge(e,t){let r=new Set;return new Set(t),e.reduce((n,a)=>{let i=JSON.stringify(uge(a));return r.has(i)||(r.add(i),n.push({key:i,link:a})),n},[])}function CY(){let e=T.useContext(lp);return FD(e,"You must render this element inside a <DataRouterContext.Provider> element"),e}function dge(){let e=T.useContext(_v);return FD(e,"You must render this element inside a <DataRouterStateContext.Provider> element"),e}var _D=T.createContext(void 0);_D.displayName="FrameworkContext";function TY(){let e=T.useContext(_D);return FD(e,"You must render this element inside a <HydratedRouter> element"),e}function hge(e,t){let r=T.useContext(_D),[n,a]=T.useState(!1),[i,o]=T.useState(!1),{onFocus:s,onBlur:l,onMouseEnter:u,onMouseLeave:d,onTouchStart:p}=t,f=T.useRef(null);T.useEffect(()=>{if(e==="render"&&o(!0),e==="viewport"){let y=w=>{w.forEach(E=>{o(E.isIntersecting)})},S=new IntersectionObserver(y,{threshold:.5});return f.current&&S.observe(f.current),()=>{S.disconnect()}}},[e]),T.useEffect(()=>{if(n){let y=setTimeout(()=>{o(!0)},100);return()=>{clearTimeout(y)}}},[n]);let m=()=>{a(!0)},b=()=>{a(!1),o(!1)};return r?e!=="intent"?[i,f,{}]:[i,f,{onFocus:Tf(s,m),onBlur:Tf(l,b),onMouseEnter:Tf(u,m),onMouseLeave:Tf(d,b),onTouchStart:Tf(p,m)}]:[!1,f,{}]}function Tf(e,t){return r=>{e&&e(r),r.defaultPrevented||t(r)}}function pge({page:e,...t}){let{router:r}=CY(),n=T.useMemo(()=>fY(r.routes,e,r.basename),[r.routes,e,r.basename]);return n?T.createElement(mge,{page:e,matches:n,...t}):null}function fge(e){let{manifest:t,routeModules:r}=TY(),[n,a]=T.useState([]);return T.useEffect(()=>{let i=!1;return oge(e,t,r).then(o=>{i||a(o)}),()=>{i=!0}},[e,t,r]),n}function mge({page:e,matches:t,...r}){let n=rd(),{manifest:a,routeModules:i}=TY(),{basename:o}=CY(),{loaderData:s,matches:l}=dge(),u=T.useMemo(()=>t8(e,t,l,a,n,"data"),[e,t,l,a,n]),d=T.useMemo(()=>t8(e,t,l,a,n,"assets"),[e,t,l,a,n]),p=T.useMemo(()=>{if(e===n.pathname+n.search+n.hash)return[];let b=new Set,y=!1;if(t.forEach(w=>{var k;let E=a.routes[w.route.id];!E||!E.hasLoader||(!u.some(_=>_.route.id===w.route.id)&&w.route.id in s&&((k=i[w.route.id])!=null&&k.shouldRevalidate)||E.hasClientLoader?y=!0:b.add(w.route.id))}),b.size===0)return[];let S=nge(e,o,"data");return y&&b.size>0&&S.searchParams.set("_routes",t.filter(w=>b.has(w.route.id)).map(w=>w.route.id).join(",")),[S.pathname+S.search]},[o,s,n,a,u,t,e,i]),f=T.useMemo(()=>sge(d,a),[d,a]),m=fge(d);return T.createElement(T.Fragment,null,p.map(b=>T.createElement("link",{key:b,rel:"prefetch",as:"fetch",href:b,...r})),f.map(b=>T.createElement("link",{key:b,rel:"modulepreload",href:b,...r})),m.map(({key:b,link:y})=>T.createElement("link",{key:b,nonce:r.nonce,...y})))}function gge(...e){return t=>{e.forEach(r=>{typeof r=="function"?r(t):r!=null&&(r.current=t)})}}var AY=typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u";try{AY&&(window.__reactRouterVersion="7.9.4")}catch{}function bge({basename:e,children:t,window:r}){let n=T.useRef();n.current==null&&(n.current=ime({window:r,v5Compat:!0}));let a=n.current,[i,o]=T.useState({action:a.action,location:a.location}),s=T.useCallback(l=>{T.startTransition(()=>o(l))},[o]);return T.useLayoutEffect(()=>a.listen(s),[a,s]),T.createElement(Vme,{basename:e,children:t,location:i.location,navigationType:i.action,navigator:a})}var FY=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,_Y=T.forwardRef(function({onClick:t,discover:r="render",prefetch:n="none",relative:a,reloadDocument:i,replace:o,state:s,target:l,to:u,preventScrollReset:d,viewTransition:p,...f},m){let{basename:b}=T.useContext(ms),y=typeof u=="string"&&FY.test(u),S,w=!1;if(typeof u=="string"&&y&&(S=u,AY))try{let I=new URL(window.location.href),M=u.startsWith("//")?new URL(I.protocol+u):new URL(u),j=gl(M.pathname,b);M.origin===I.origin&&j!=null?u=j+M.search+M.hash:w=!0}catch{Lo(!1,`<Link to="${u}"> contains an invalid URL which will probably break when clicked - please update to a valid URL path.`)}let E=Nme(u,{relative:a}),[k,_,R]=hge(n,f),C=Ege(u,{replace:o,state:s,target:l,preventScrollReset:d,relative:a,viewTransition:p});function F(I){t&&t(I),I.defaultPrevented||C(I)}let L=T.createElement("a",{...f,...R,href:S||E,onClick:w||i?t:F,ref:gge(m,_),target:l,"data-discover":!y&&r==="render"?"true":void 0});return k&&!y?T.createElement(T.Fragment,null,L,T.createElement(pge,{page:E})):L});_Y.displayName="Link";var yge=T.forwardRef(function({"aria-current":t="page",caseSensitive:r=!1,className:n="",end:a=!1,style:i,to:o,viewTransition:s,children:l,...u},d){let p=fg(o,{relative:u.relative}),f=rd(),m=T.useContext(_v),{navigator:b,basename:y}=T.useContext(ms),S=m!=null&&Tge(p)&&s===!0,w=b.encodeLocation?b.encodeLocation(p).pathname:p.pathname,E=f.pathname,k=m&&m.navigation&&m.navigation.location?m.navigation.location.pathname:null;r||(E=E.toLowerCase(),k=k?k.toLowerCase():null,w=w.toLowerCase()),k&&y&&(k=gl(k,y)||k);const _=w!=="/"&&w.endsWith("/")?w.length-1:w.length;let R=E===w||!a&&E.startsWith(w)&&E.charAt(_)==="/",C=k!=null&&(k===w||!a&&k.startsWith(w)&&k.charAt(w.length)==="/"),F={isActive:R,isPending:C,isTransitioning:S},L=R?t:void 0,I;typeof n=="function"?I=n(F):I=[n,R?"active":null,C?"pending":null,S?"transitioning":null].filter(Boolean).join(" ");let M=typeof i=="function"?i(F):i;return T.createElement(_Y,{...u,"aria-current":L,className:I,ref:d,style:M,to:o,viewTransition:s},typeof l=="function"?l(F):l)});yge.displayName="NavLink";var vge=T.forwardRef(({discover:e="render",fetcherKey:t,navigate:r,reloadDocument:n,replace:a,state:i,method:o=y1,action:s,onSubmit:l,relative:u,preventScrollReset:d,viewTransition:p,...f},m)=>{let b=kge(),y=Cge(s,{relative:u}),S=o.toLowerCase()==="get"?"get":"post",w=typeof s=="string"&&FY.test(s),E=k=>{if(l&&l(k),k.defaultPrevented)return;k.preventDefault();let _=k.nativeEvent.submitter,R=(_==null?void 0:_.getAttribute("formmethod"))||o;b(_||k.currentTarget,{fetcherKey:t,method:R,navigate:r,replace:a,state:i,relative:u,preventScrollReset:d,viewTransition:p})};return T.createElement("form",{ref:m,method:S,action:y,onSubmit:n?l:E,...f,"data-discover":!w&&e==="render"?"true":void 0})});vge.displayName="Form";function Sge(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function RY(e){let t=T.useContext(lp);return wn(t,Sge(e)),t}function Ege(e,{target:t,replace:r,state:n,preventScrollReset:a,relative:i,viewTransition:o}={}){let s=CD(),l=rd(),u=fg(e,{relative:i});return T.useCallback(d=>{if(Qme(d,t)){d.preventDefault();let p=r!==void 0?r:Om(l)===Om(u);s(e,{replace:p,state:n,preventScrollReset:a,relative:i,viewTransition:o})}},[l,s,u,r,n,t,e,a,i,o])}var xge=0,wge=()=>`__${String(++xge)}__`;function kge(){let{router:e}=RY("useSubmit"),{basename:t}=T.useContext(ms),r=Gme();return T.useCallback(async(n,a={})=>{let{action:i,method:o,encType:s,formData:l,body:u}=rge(n,t);if(a.navigate===!1){let d=a.fetcherKey||wge();await e.fetch(d,r,a.action||i,{preventScrollReset:a.preventScrollReset,formData:l,body:u,formMethod:a.method||o,formEncType:a.encType||s,flushSync:a.flushSync})}else await e.navigate(a.action||i,{preventScrollReset:a.preventScrollReset,formData:l,body:u,formMethod:a.method||o,formEncType:a.encType||s,replace:a.replace,state:a.state,fromRouteId:r,flushSync:a.flushSync,viewTransition:a.viewTransition})},[e,t,r])}function Cge(e,{relative:t}={}){let{basename:r}=T.useContext(ms),n=T.useContext(Al);wn(n,"useFormAction must be used inside a RouteContext");let[a]=n.matches.slice(-1),i={...fg(e||".",{relative:t})},o=rd();if(e==null){i.search=o.search;let s=new URLSearchParams(i.search),l=s.getAll("index");if(l.some(d=>d==="")){s.delete("index"),l.filter(p=>p).forEach(p=>s.append("index",p));let d=s.toString();i.search=d?`?${d}`:""}}return(!e||e===".")&&a.route.index&&(i.search=i.search?i.search.replace(/^\?/,"?index&"):"?index"),r!=="/"&&(i.pathname=i.pathname==="/"?r:ul([r,i.pathname])),Om(i)}function Tge(e,{relative:t}={}){let r=T.useContext(SY);wn(r!=null,"`useViewTransitionState` must be used within `react-router-dom`'s `RouterProvider`.  Did you accidentally import `RouterProvider` from `react-router`?");let{basename:n}=RY("useViewTransitionState"),a=fg(e,{relative:t});if(!r.isTransitioning)return!1;let i=gl(r.currentLocation.pathname,n)||r.currentLocation.pathname,o=gl(r.nextLocation.pathname,n)||r.nextLocation.pathname;return oy(a.pathname,o)!=null||oy(a.pathname,i)!=null}var mg=cK();const DY=jn(mg),r8=e=>{let t;const r=new Set,n=(u,d)=>{const p=typeof u=="function"?u(t):u;if(!Object.is(p,t)){const f=t;t=d??(typeof p!="object"||p===null)?p:Object.assign({},t,p),r.forEach(m=>m(t,f))}},a=()=>t,s={setState:n,getState:a,getInitialState:()=>l,subscribe:u=>(r.add(u),()=>r.delete(u))},l=t=e(n,a,s);return s},Age=(e=>e?r8(e):r8),Fge=e=>e;function _ge(e,t=Fge){const r=$e.useSyncExternalStore(e.subscribe,$e.useCallback(()=>t(e.getState()),[e,t]),$e.useCallback(()=>t(e.getInitialState()),[e,t]));return $e.useDebugValue(r),r}const n8=e=>{const t=Age(e),r=n=>_ge(t,n);return Object.assign(r,t),r},Dv=(e=>e?n8(e):n8);function NY(e){var t,r,n="";if(typeof e=="string"||typeof e=="number")n+=e;else if(typeof e=="object")if(Array.isArray(e)){var a=e.length;for(t=0;t<a;t++)e[t]&&(r=NY(e[t]))&&(n&&(n+=" "),n+=r)}else for(r in e)e[r]&&(n&&(n+=" "),n+=r);return n}function LY(){for(var e,t,r=0,n="",a=arguments.length;r<a;r++)(e=arguments[r])&&(t=NY(e))&&(n&&(n+=" "),n+=t);return n}const RD="-",Rge=e=>{const t=Nge(e),{conflictingClassGroups:r,conflictingClassGroupModifiers:n}=e;return{getClassGroupId:o=>{const s=o.split(RD);return s[0]===""&&s.length!==1&&s.shift(),MY(s,t)||Dge(o)},getConflictingClassGroupIds:(o,s)=>{const l=r[o]||[];return s&&n[o]?[...l,...n[o]]:l}}},MY=(e,t)=>{var o;if(e.length===0)return t.classGroupId;const r=e[0],n=t.nextPart.get(r),a=n?MY(e.slice(1),n):void 0;if(a)return a;if(t.validators.length===0)return;const i=e.join(RD);return(o=t.validators.find(({validator:s})=>s(i)))==null?void 0:o.classGroupId},a8=/^\[(.+)\]$/,Dge=e=>{if(a8.test(e)){const t=a8.exec(e)[1],r=t==null?void 0:t.substring(0,t.indexOf(":"));if(r)return"arbitrary.."+r}},Nge=e=>{const{theme:t,classGroups:r}=e,n={nextPart:new Map,validators:[]};for(const a in r)L_(r[a],n,a,t);return n},L_=(e,t,r,n)=>{e.forEach(a=>{if(typeof a=="string"){const i=a===""?t:i8(t,a);i.classGroupId=r;return}if(typeof a=="function"){if(Lge(a)){L_(a(n),t,r,n);return}t.validators.push({validator:a,classGroupId:r});return}Object.entries(a).forEach(([i,o])=>{L_(o,i8(t,i),r,n)})})},i8=(e,t)=>{let r=e;return t.split(RD).forEach(n=>{r.nextPart.has(n)||r.nextPart.set(n,{nextPart:new Map,validators:[]}),r=r.nextPart.get(n)}),r},Lge=e=>e.isThemeGetter,Mge=e=>{if(e<1)return{get:()=>{},set:()=>{}};let t=0,r=new Map,n=new Map;const a=(i,o)=>{r.set(i,o),t++,t>e&&(t=0,n=r,r=new Map)};return{get(i){let o=r.get(i);if(o!==void 0)return o;if((o=n.get(i))!==void 0)return a(i,o),o},set(i,o){r.has(i)?r.set(i,o):a(i,o)}}},M_="!",I_=":",Ige=I_.length,Oge=e=>{const{prefix:t,experimentalParseClassName:r}=e;let n=a=>{const i=[];let o=0,s=0,l=0,u;for(let b=0;b<a.length;b++){let y=a[b];if(o===0&&s===0){if(y===I_){i.push(a.slice(l,b)),l=b+Ige;continue}if(y==="/"){u=b;continue}}y==="["?o++:y==="]"?o--:y==="("?s++:y===")"&&s--}const d=i.length===0?a:a.substring(l),p=Bge(d),f=p!==d,m=u&&u>l?u-l:void 0;return{modifiers:i,hasImportantModifier:f,baseClassName:p,maybePostfixModifierPosition:m}};if(t){const a=t+I_,i=n;n=o=>o.startsWith(a)?i(o.substring(a.length)):{isExternal:!0,modifiers:[],hasImportantModifier:!1,baseClassName:o,maybePostfixModifierPosition:void 0}}if(r){const a=n;n=i=>r({className:i,parseClassName:a})}return n},Bge=e=>e.endsWith(M_)?e.substring(0,e.length-1):e.startsWith(M_)?e.substring(1):e,Pge=e=>{const t=Object.fromEntries(e.orderSensitiveModifiers.map(n=>[n,!0]));return n=>{if(n.length<=1)return n;const a=[];let i=[];return n.forEach(o=>{o[0]==="["||t[o]?(a.push(...i.sort(),o),i=[]):i.push(o)}),a.push(...i.sort()),a}},zge=e=>({cache:Mge(e.cacheSize),parseClassName:Oge(e),sortModifiers:Pge(e),...Rge(e)}),Hge=/\s+/,Uge=(e,t)=>{const{parseClassName:r,getClassGroupId:n,getConflictingClassGroupIds:a,sortModifiers:i}=t,o=[],s=e.trim().split(Hge);let l="";for(let u=s.length-1;u>=0;u-=1){const d=s[u],{isExternal:p,modifiers:f,hasImportantModifier:m,baseClassName:b,maybePostfixModifierPosition:y}=r(d);if(p){l=d+(l.length>0?" "+l:l);continue}let S=!!y,w=n(S?b.substring(0,y):b);if(!w){if(!S){l=d+(l.length>0?" "+l:l);continue}if(w=n(b),!w){l=d+(l.length>0?" "+l:l);continue}S=!1}const E=i(f).join(":"),k=m?E+M_:E,_=k+w;if(o.includes(_))continue;o.push(_);const R=a(w,S);for(let C=0;C<R.length;++C){const F=R[C];o.push(k+F)}l=d+(l.length>0?" "+l:l)}return l};function $ge(){let e=0,t,r,n="";for(;e<arguments.length;)(t=arguments[e++])&&(r=IY(t))&&(n&&(n+=" "),n+=r);return n}const IY=e=>{if(typeof e=="string")return e;let t,r="";for(let n=0;n<e.length;n++)e[n]&&(t=IY(e[n]))&&(r&&(r+=" "),r+=t);return r};function Gge(e,...t){let r,n,a,i=o;function o(l){const u=t.reduce((d,p)=>p(d),e());return r=zge(u),n=r.cache.get,a=r.cache.set,i=s,s(l)}function s(l){const u=n(l);if(u)return u;const d=Uge(l,r);return a(l,d),d}return function(){return i($ge.apply(null,arguments))}}const Yn=e=>{const t=r=>r[e]||[];return t.isThemeGetter=!0,t},OY=/^\[(?:(\w[\w-]*):)?(.+)\]$/i,BY=/^\((?:(\w[\w-]*):)?(.+)\)$/i,jge=/^\d+\/\d+$/,qge=/^(\d+(\.\d+)?)?(xs|sm|md|lg|xl)$/,Wge=/\d+(%|px|r?em|[sdl]?v([hwib]|min|max)|pt|pc|in|cm|mm|cap|ch|ex|r?lh|cq(w|h|i|b|min|max))|\b(calc|min|max|clamp)\(.+\)|^0$/,Vge=/^(rgba?|hsla?|hwb|(ok)?(lab|lch)|color-mix)\(.+\)$/,Kge=/^(inset_)?-?((\d+)?\.?(\d+)[a-z]+|0)_-?((\d+)?\.?(\d+)[a-z]+|0)/,Yge=/^(url|image|image-set|cross-fade|element|(repeating-)?(linear|radial|conic)-gradient)\(.+\)$/,Xd=e=>jge.test(e),vr=e=>!!e&&!Number.isNaN(Number(e)),iu=e=>!!e&&Number.isInteger(Number(e)),Px=e=>e.endsWith("%")&&vr(e.slice(0,-1)),js=e=>qge.test(e),Jge=()=>!0,Xge=e=>Wge.test(e)&&!Vge.test(e),PY=()=>!1,Zge=e=>Kge.test(e),Qge=e=>Yge.test(e),e0e=e=>!Tt(e)&&!At(e),t0e=e=>up(e,UY,PY),Tt=e=>OY.test(e),cc=e=>up(e,$Y,Xge),zx=e=>up(e,o0e,vr),o8=e=>up(e,zY,PY),r0e=e=>up(e,HY,Qge),Eb=e=>up(e,GY,Zge),At=e=>BY.test(e),Af=e=>cp(e,$Y),n0e=e=>cp(e,s0e),s8=e=>cp(e,zY),a0e=e=>cp(e,UY),i0e=e=>cp(e,HY),xb=e=>cp(e,GY,!0),up=(e,t,r)=>{const n=OY.exec(e);return n?n[1]?t(n[1]):r(n[2]):!1},cp=(e,t,r=!1)=>{const n=BY.exec(e);return n?n[1]?t(n[1]):r:!1},zY=e=>e==="position"||e==="percentage",HY=e=>e==="image"||e==="url",UY=e=>e==="length"||e==="size"||e==="bg-size",$Y=e=>e==="length",o0e=e=>e==="number",s0e=e=>e==="family-name",GY=e=>e==="shadow",l0e=()=>{const e=Yn("color"),t=Yn("font"),r=Yn("text"),n=Yn("font-weight"),a=Yn("tracking"),i=Yn("leading"),o=Yn("breakpoint"),s=Yn("container"),l=Yn("spacing"),u=Yn("radius"),d=Yn("shadow"),p=Yn("inset-shadow"),f=Yn("text-shadow"),m=Yn("drop-shadow"),b=Yn("blur"),y=Yn("perspective"),S=Yn("aspect"),w=Yn("ease"),E=Yn("animate"),k=()=>["auto","avoid","all","avoid-page","page","left","right","column"],_=()=>["center","top","bottom","left","right","top-left","left-top","top-right","right-top","bottom-right","right-bottom","bottom-left","left-bottom"],R=()=>[..._(),At,Tt],C=()=>["auto","hidden","clip","visible","scroll"],F=()=>["auto","contain","none"],L=()=>[At,Tt,l],I=()=>[Xd,"full","auto",...L()],M=()=>[iu,"none","subgrid",At,Tt],j=()=>["auto",{span:["full",iu,At,Tt]},iu,At,Tt],Y=()=>[iu,"auto",At,Tt],W=()=>["auto","min","max","fr",At,Tt],$=()=>["start","end","center","between","around","evenly","stretch","baseline","center-safe","end-safe"],X=()=>["start","end","center","stretch","center-safe","end-safe"],z=()=>["auto",...L()],H=()=>[Xd,"auto","full","dvw","dvh","lvw","lvh","svw","svh","min","max","fit",...L()],B=()=>[e,At,Tt],ee=()=>[..._(),s8,o8,{position:[At,Tt]}],J=()=>["no-repeat",{repeat:["","x","y","space","round"]}],K=()=>["auto","cover","contain",a0e,t0e,{size:[At,Tt]}],G=()=>[Px,Af,cc],P=()=>["","none","full",u,At,Tt],ne=()=>["",vr,Af,cc],oe=()=>["solid","dashed","dotted","double"],pe=()=>["normal","multiply","screen","overlay","darken","lighten","color-dodge","color-burn","hard-light","soft-light","difference","exclusion","hue","saturation","color","luminosity"],ae=()=>[vr,Px,s8,o8],se=()=>["","none",b,At,Tt],Ee=()=>["none",vr,At,Tt],fe=()=>["none",vr,At,Tt],Me=()=>[vr,At,Tt],Oe=()=>[Xd,"full",...L()];return{cacheSize:500,theme:{animate:["spin","ping","pulse","bounce"],aspect:["video"],blur:[js],breakpoint:[js],color:[Jge],container:[js],"drop-shadow":[js],ease:["in","out","in-out"],font:[e0e],"font-weight":["thin","extralight","light","normal","medium","semibold","bold","extrabold","black"],"inset-shadow":[js],leading:["none","tight","snug","normal","relaxed","loose"],perspective:["dramatic","near","normal","midrange","distant","none"],radius:[js],shadow:[js],spacing:["px",vr],text:[js],"text-shadow":[js],tracking:["tighter","tight","normal","wide","wider","widest"]},classGroups:{aspect:[{aspect:["auto","square",Xd,Tt,At,S]}],container:["container"],columns:[{columns:[vr,Tt,At,s]}],"break-after":[{"break-after":k()}],"break-before":[{"break-before":k()}],"break-inside":[{"break-inside":["auto","avoid","avoid-page","avoid-column"]}],"box-decoration":[{"box-decoration":["slice","clone"]}],box:[{box:["border","content"]}],display:["block","inline-block","inline","flex","inline-flex","table","inline-table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row-group","table-row","flow-root","grid","inline-grid","contents","list-item","hidden"],sr:["sr-only","not-sr-only"],float:[{float:["right","left","none","start","end"]}],clear:[{clear:["left","right","both","none","start","end"]}],isolation:["isolate","isolation-auto"],"object-fit":[{object:["contain","cover","fill","none","scale-down"]}],"object-position":[{object:R()}],overflow:[{overflow:C()}],"overflow-x":[{"overflow-x":C()}],"overflow-y":[{"overflow-y":C()}],overscroll:[{overscroll:F()}],"overscroll-x":[{"overscroll-x":F()}],"overscroll-y":[{"overscroll-y":F()}],position:["static","fixed","absolute","relative","sticky"],inset:[{inset:I()}],"inset-x":[{"inset-x":I()}],"inset-y":[{"inset-y":I()}],start:[{start:I()}],end:[{end:I()}],top:[{top:I()}],right:[{right:I()}],bottom:[{bottom:I()}],left:[{left:I()}],visibility:["visible","invisible","collapse"],z:[{z:[iu,"auto",At,Tt]}],basis:[{basis:[Xd,"full","auto",s,...L()]}],"flex-direction":[{flex:["row","row-reverse","col","col-reverse"]}],"flex-wrap":[{flex:["nowrap","wrap","wrap-reverse"]}],flex:[{flex:[vr,Xd,"auto","initial","none",Tt]}],grow:[{grow:["",vr,At,Tt]}],shrink:[{shrink:["",vr,At,Tt]}],order:[{order:[iu,"first","last","none",At,Tt]}],"grid-cols":[{"grid-cols":M()}],"col-start-end":[{col:j()}],"col-start":[{"col-start":Y()}],"col-end":[{"col-end":Y()}],"grid-rows":[{"grid-rows":M()}],"row-start-end":[{row:j()}],"row-start":[{"row-start":Y()}],"row-end":[{"row-end":Y()}],"grid-flow":[{"grid-flow":["row","col","dense","row-dense","col-dense"]}],"auto-cols":[{"auto-cols":W()}],"auto-rows":[{"auto-rows":W()}],gap:[{gap:L()}],"gap-x":[{"gap-x":L()}],"gap-y":[{"gap-y":L()}],"justify-content":[{justify:[...$(),"normal"]}],"justify-items":[{"justify-items":[...X(),"normal"]}],"justify-self":[{"justify-self":["auto",...X()]}],"align-content":[{content:["normal",...$()]}],"align-items":[{items:[...X(),{baseline:["","last"]}]}],"align-self":[{self:["auto",...X(),{baseline:["","last"]}]}],"place-content":[{"place-content":$()}],"place-items":[{"place-items":[...X(),"baseline"]}],"place-self":[{"place-self":["auto",...X()]}],p:[{p:L()}],px:[{px:L()}],py:[{py:L()}],ps:[{ps:L()}],pe:[{pe:L()}],pt:[{pt:L()}],pr:[{pr:L()}],pb:[{pb:L()}],pl:[{pl:L()}],m:[{m:z()}],mx:[{mx:z()}],my:[{my:z()}],ms:[{ms:z()}],me:[{me:z()}],mt:[{mt:z()}],mr:[{mr:z()}],mb:[{mb:z()}],ml:[{ml:z()}],"space-x":[{"space-x":L()}],"space-x-reverse":["space-x-reverse"],"space-y":[{"space-y":L()}],"space-y-reverse":["space-y-reverse"],size:[{size:H()}],w:[{w:[s,"screen",...H()]}],"min-w":[{"min-w":[s,"screen","none",...H()]}],"max-w":[{"max-w":[s,"screen","none","prose",{screen:[o]},...H()]}],h:[{h:["screen","lh",...H()]}],"min-h":[{"min-h":["screen","lh","none",...H()]}],"max-h":[{"max-h":["screen","lh",...H()]}],"font-size":[{text:["base",r,Af,cc]}],"font-smoothing":["antialiased","subpixel-antialiased"],"font-style":["italic","not-italic"],"font-weight":[{font:[n,At,zx]}],"font-stretch":[{"font-stretch":["ultra-condensed","extra-condensed","condensed","semi-condensed","normal","semi-expanded","expanded","extra-expanded","ultra-expanded",Px,Tt]}],"font-family":[{font:[n0e,Tt,t]}],"fvn-normal":["normal-nums"],"fvn-ordinal":["ordinal"],"fvn-slashed-zero":["slashed-zero"],"fvn-figure":["lining-nums","oldstyle-nums"],"fvn-spacing":["proportional-nums","tabular-nums"],"fvn-fraction":["diagonal-fractions","stacked-fractions"],tracking:[{tracking:[a,At,Tt]}],"line-clamp":[{"line-clamp":[vr,"none",At,zx]}],leading:[{leading:[i,...L()]}],"list-image":[{"list-image":["none",At,Tt]}],"list-style-position":[{list:["inside","outside"]}],"list-style-type":[{list:["disc","decimal","none",At,Tt]}],"text-alignment":[{text:["left","center","right","justify","start","end"]}],"placeholder-color":[{placeholder:B()}],"text-color":[{text:B()}],"text-decoration":["underline","overline","line-through","no-underline"],"text-decoration-style":[{decoration:[...oe(),"wavy"]}],"text-decoration-thickness":[{decoration:[vr,"from-font","auto",At,cc]}],"text-decoration-color":[{decoration:B()}],"underline-offset":[{"underline-offset":[vr,"auto",At,Tt]}],"text-transform":["uppercase","lowercase","capitalize","normal-case"],"text-overflow":["truncate","text-ellipsis","text-clip"],"text-wrap":[{text:["wrap","nowrap","balance","pretty"]}],indent:[{indent:L()}],"vertical-align":[{align:["baseline","top","middle","bottom","text-top","text-bottom","sub","super",At,Tt]}],whitespace:[{whitespace:["normal","nowrap","pre","pre-line","pre-wrap","break-spaces"]}],break:[{break:["normal","words","all","keep"]}],wrap:[{wrap:["break-word","anywhere","normal"]}],hyphens:[{hyphens:["none","manual","auto"]}],content:[{content:["none",At,Tt]}],"bg-attachment":[{bg:["fixed","local","scroll"]}],"bg-clip":[{"bg-clip":["border","padding","content","text"]}],"bg-origin":[{"bg-origin":["border","padding","content"]}],"bg-position":[{bg:ee()}],"bg-repeat":[{bg:J()}],"bg-size":[{bg:K()}],"bg-image":[{bg:["none",{linear:[{to:["t","tr","r","br","b","bl","l","tl"]},iu,At,Tt],radial:["",At,Tt],conic:[iu,At,Tt]},i0e,r0e]}],"bg-color":[{bg:B()}],"gradient-from-pos":[{from:G()}],"gradient-via-pos":[{via:G()}],"gradient-to-pos":[{to:G()}],"gradient-from":[{from:B()}],"gradient-via":[{via:B()}],"gradient-to":[{to:B()}],rounded:[{rounded:P()}],"rounded-s":[{"rounded-s":P()}],"rounded-e":[{"rounded-e":P()}],"rounded-t":[{"rounded-t":P()}],"rounded-r":[{"rounded-r":P()}],"rounded-b":[{"rounded-b":P()}],"rounded-l":[{"rounded-l":P()}],"rounded-ss":[{"rounded-ss":P()}],"rounded-se":[{"rounded-se":P()}],"rounded-ee":[{"rounded-ee":P()}],"rounded-es":[{"rounded-es":P()}],"rounded-tl":[{"rounded-tl":P()}],"rounded-tr":[{"rounded-tr":P()}],"rounded-br":[{"rounded-br":P()}],"rounded-bl":[{"rounded-bl":P()}],"border-w":[{border:ne()}],"border-w-x":[{"border-x":ne()}],"border-w-y":[{"border-y":ne()}],"border-w-s":[{"border-s":ne()}],"border-w-e":[{"border-e":ne()}],"border-w-t":[{"border-t":ne()}],"border-w-r":[{"border-r":ne()}],"border-w-b":[{"border-b":ne()}],"border-w-l":[{"border-l":ne()}],"divide-x":[{"divide-x":ne()}],"divide-x-reverse":["divide-x-reverse"],"divide-y":[{"divide-y":ne()}],"divide-y-reverse":["divide-y-reverse"],"border-style":[{border:[...oe(),"hidden","none"]}],"divide-style":[{divide:[...oe(),"hidden","none"]}],"border-color":[{border:B()}],"border-color-x":[{"border-x":B()}],"border-color-y":[{"border-y":B()}],"border-color-s":[{"border-s":B()}],"border-color-e":[{"border-e":B()}],"border-color-t":[{"border-t":B()}],"border-color-r":[{"border-r":B()}],"border-color-b":[{"border-b":B()}],"border-color-l":[{"border-l":B()}],"divide-color":[{divide:B()}],"outline-style":[{outline:[...oe(),"none","hidden"]}],"outline-offset":[{"outline-offset":[vr,At,Tt]}],"outline-w":[{outline:["",vr,Af,cc]}],"outline-color":[{outline:B()}],shadow:[{shadow:["","none",d,xb,Eb]}],"shadow-color":[{shadow:B()}],"inset-shadow":[{"inset-shadow":["none",p,xb,Eb]}],"inset-shadow-color":[{"inset-shadow":B()}],"ring-w":[{ring:ne()}],"ring-w-inset":["ring-inset"],"ring-color":[{ring:B()}],"ring-offset-w":[{"ring-offset":[vr,cc]}],"ring-offset-color":[{"ring-offset":B()}],"inset-ring-w":[{"inset-ring":ne()}],"inset-ring-color":[{"inset-ring":B()}],"text-shadow":[{"text-shadow":["none",f,xb,Eb]}],"text-shadow-color":[{"text-shadow":B()}],opacity:[{opacity:[vr,At,Tt]}],"mix-blend":[{"mix-blend":[...pe(),"plus-darker","plus-lighter"]}],"bg-blend":[{"bg-blend":pe()}],"mask-clip":[{"mask-clip":["border","padding","content","fill","stroke","view"]},"mask-no-clip"],"mask-composite":[{mask:["add","subtract","intersect","exclude"]}],"mask-image-linear-pos":[{"mask-linear":[vr]}],"mask-image-linear-from-pos":[{"mask-linear-from":ae()}],"mask-image-linear-to-pos":[{"mask-linear-to":ae()}],"mask-image-linear-from-color":[{"mask-linear-from":B()}],"mask-image-linear-to-color":[{"mask-linear-to":B()}],"mask-image-t-from-pos":[{"mask-t-from":ae()}],"mask-image-t-to-pos":[{"mask-t-to":ae()}],"mask-image-t-from-color":[{"mask-t-from":B()}],"mask-image-t-to-color":[{"mask-t-to":B()}],"mask-image-r-from-pos":[{"mask-r-from":ae()}],"mask-image-r-to-pos":[{"mask-r-to":ae()}],"mask-image-r-from-color":[{"mask-r-from":B()}],"mask-image-r-to-color":[{"mask-r-to":B()}],"mask-image-b-from-pos":[{"mask-b-from":ae()}],"mask-image-b-to-pos":[{"mask-b-to":ae()}],"mask-image-b-from-color":[{"mask-b-from":B()}],"mask-image-b-to-color":[{"mask-b-to":B()}],"mask-image-l-from-pos":[{"mask-l-from":ae()}],"mask-image-l-to-pos":[{"mask-l-to":ae()}],"mask-image-l-from-color":[{"mask-l-from":B()}],"mask-image-l-to-color":[{"mask-l-to":B()}],"mask-image-x-from-pos":[{"mask-x-from":ae()}],"mask-image-x-to-pos":[{"mask-x-to":ae()}],"mask-image-x-from-color":[{"mask-x-from":B()}],"mask-image-x-to-color":[{"mask-x-to":B()}],"mask-image-y-from-pos":[{"mask-y-from":ae()}],"mask-image-y-to-pos":[{"mask-y-to":ae()}],"mask-image-y-from-color":[{"mask-y-from":B()}],"mask-image-y-to-color":[{"mask-y-to":B()}],"mask-image-radial":[{"mask-radial":[At,Tt]}],"mask-image-radial-from-pos":[{"mask-radial-from":ae()}],"mask-image-radial-to-pos":[{"mask-radial-to":ae()}],"mask-image-radial-from-color":[{"mask-radial-from":B()}],"mask-image-radial-to-color":[{"mask-radial-to":B()}],"mask-image-radial-shape":[{"mask-radial":["circle","ellipse"]}],"mask-image-radial-size":[{"mask-radial":[{closest:["side","corner"],farthest:["side","corner"]}]}],"mask-image-radial-pos":[{"mask-radial-at":_()}],"mask-image-conic-pos":[{"mask-conic":[vr]}],"mask-image-conic-from-pos":[{"mask-conic-from":ae()}],"mask-image-conic-to-pos":[{"mask-conic-to":ae()}],"mask-image-conic-from-color":[{"mask-conic-from":B()}],"mask-image-conic-to-color":[{"mask-conic-to":B()}],"mask-mode":[{mask:["alpha","luminance","match"]}],"mask-origin":[{"mask-origin":["border","padding","content","fill","stroke","view"]}],"mask-position":[{mask:ee()}],"mask-repeat":[{mask:J()}],"mask-size":[{mask:K()}],"mask-type":[{"mask-type":["alpha","luminance"]}],"mask-image":[{mask:["none",At,Tt]}],filter:[{filter:["","none",At,Tt]}],blur:[{blur:se()}],brightness:[{brightness:[vr,At,Tt]}],contrast:[{contrast:[vr,At,Tt]}],"drop-shadow":[{"drop-shadow":["","none",m,xb,Eb]}],"drop-shadow-color":[{"drop-shadow":B()}],grayscale:[{grayscale:["",vr,At,Tt]}],"hue-rotate":[{"hue-rotate":[vr,At,Tt]}],invert:[{invert:["",vr,At,Tt]}],saturate:[{saturate:[vr,At,Tt]}],sepia:[{sepia:["",vr,At,Tt]}],"backdrop-filter":[{"backdrop-filter":["","none",At,Tt]}],"backdrop-blur":[{"backdrop-blur":se()}],"backdrop-brightness":[{"backdrop-brightness":[vr,At,Tt]}],"backdrop-contrast":[{"backdrop-contrast":[vr,At,Tt]}],"backdrop-grayscale":[{"backdrop-grayscale":["",vr,At,Tt]}],"backdrop-hue-rotate":[{"backdrop-hue-rotate":[vr,At,Tt]}],"backdrop-invert":[{"backdrop-invert":["",vr,At,Tt]}],"backdrop-opacity":[{"backdrop-opacity":[vr,At,Tt]}],"backdrop-saturate":[{"backdrop-saturate":[vr,At,Tt]}],"backdrop-sepia":[{"backdrop-sepia":["",vr,At,Tt]}],"border-collapse":[{border:["collapse","separate"]}],"border-spacing":[{"border-spacing":L()}],"border-spacing-x":[{"border-spacing-x":L()}],"border-spacing-y":[{"border-spacing-y":L()}],"table-layout":[{table:["auto","fixed"]}],caption:[{caption:["top","bottom"]}],transition:[{transition:["","all","colors","opacity","shadow","transform","none",At,Tt]}],"transition-behavior":[{transition:["normal","discrete"]}],duration:[{duration:[vr,"initial",At,Tt]}],ease:[{ease:["linear","initial",w,At,Tt]}],delay:[{delay:[vr,At,Tt]}],animate:[{animate:["none",E,At,Tt]}],backface:[{backface:["hidden","visible"]}],perspective:[{perspective:[y,At,Tt]}],"perspective-origin":[{"perspective-origin":R()}],rotate:[{rotate:Ee()}],"rotate-x":[{"rotate-x":Ee()}],"rotate-y":[{"rotate-y":Ee()}],"rotate-z":[{"rotate-z":Ee()}],scale:[{scale:fe()}],"scale-x":[{"scale-x":fe()}],"scale-y":[{"scale-y":fe()}],"scale-z":[{"scale-z":fe()}],"scale-3d":["scale-3d"],skew:[{skew:Me()}],"skew-x":[{"skew-x":Me()}],"skew-y":[{"skew-y":Me()}],transform:[{transform:[At,Tt,"","none","gpu","cpu"]}],"transform-origin":[{origin:R()}],"transform-style":[{transform:["3d","flat"]}],translate:[{translate:Oe()}],"translate-x":[{"translate-x":Oe()}],"translate-y":[{"translate-y":Oe()}],"translate-z":[{"translate-z":Oe()}],"translate-none":["translate-none"],accent:[{accent:B()}],appearance:[{appearance:["none","auto"]}],"caret-color":[{caret:B()}],"color-scheme":[{scheme:["normal","dark","light","light-dark","only-dark","only-light"]}],cursor:[{cursor:["auto","default","pointer","wait","text","move","help","not-allowed","none","context-menu","progress","cell","crosshair","vertical-text","alias","copy","no-drop","grab","grabbing","all-scroll","col-resize","row-resize","n-resize","e-resize","s-resize","w-resize","ne-resize","nw-resize","se-resize","sw-resize","ew-resize","ns-resize","nesw-resize","nwse-resize","zoom-in","zoom-out",At,Tt]}],"field-sizing":[{"field-sizing":["fixed","content"]}],"pointer-events":[{"pointer-events":["auto","none"]}],resize:[{resize:["none","","y","x"]}],"scroll-behavior":[{scroll:["auto","smooth"]}],"scroll-m":[{"scroll-m":L()}],"scroll-mx":[{"scroll-mx":L()}],"scroll-my":[{"scroll-my":L()}],"scroll-ms":[{"scroll-ms":L()}],"scroll-me":[{"scroll-me":L()}],"scroll-mt":[{"scroll-mt":L()}],"scroll-mr":[{"scroll-mr":L()}],"scroll-mb":[{"scroll-mb":L()}],"scroll-ml":[{"scroll-ml":L()}],"scroll-p":[{"scroll-p":L()}],"scroll-px":[{"scroll-px":L()}],"scroll-py":[{"scroll-py":L()}],"scroll-ps":[{"scroll-ps":L()}],"scroll-pe":[{"scroll-pe":L()}],"scroll-pt":[{"scroll-pt":L()}],"scroll-pr":[{"scroll-pr":L()}],"scroll-pb":[{"scroll-pb":L()}],"scroll-pl":[{"scroll-pl":L()}],"snap-align":[{snap:["start","end","center","align-none"]}],"snap-stop":[{snap:["normal","always"]}],"snap-type":[{snap:["none","x","y","both"]}],"snap-strictness":[{snap:["mandatory","proximity"]}],touch:[{touch:["auto","none","manipulation"]}],"touch-x":[{"touch-pan":["x","left","right"]}],"touch-y":[{"touch-pan":["y","up","down"]}],"touch-pz":["touch-pinch-zoom"],select:[{select:["none","text","all","auto"]}],"will-change":[{"will-change":["auto","scroll","contents","transform",At,Tt]}],fill:[{fill:["none",...B()]}],"stroke-w":[{stroke:[vr,Af,cc,zx]}],stroke:[{stroke:["none",...B()]}],"forced-color-adjust":[{"forced-color-adjust":["auto","none"]}]},conflictingClassGroups:{overflow:["overflow-x","overflow-y"],overscroll:["overscroll-x","overscroll-y"],inset:["inset-x","inset-y","start","end","top","right","bottom","left"],"inset-x":["right","left"],"inset-y":["top","bottom"],flex:["basis","grow","shrink"],gap:["gap-x","gap-y"],p:["px","py","ps","pe","pt","pr","pb","pl"],px:["pr","pl"],py:["pt","pb"],m:["mx","my","ms","me","mt","mr","mb","ml"],mx:["mr","ml"],my:["mt","mb"],size:["w","h"],"font-size":["leading"],"fvn-normal":["fvn-ordinal","fvn-slashed-zero","fvn-figure","fvn-spacing","fvn-fraction"],"fvn-ordinal":["fvn-normal"],"fvn-slashed-zero":["fvn-normal"],"fvn-figure":["fvn-normal"],"fvn-spacing":["fvn-normal"],"fvn-fraction":["fvn-normal"],"line-clamp":["display","overflow"],rounded:["rounded-s","rounded-e","rounded-t","rounded-r","rounded-b","rounded-l","rounded-ss","rounded-se","rounded-ee","rounded-es","rounded-tl","rounded-tr","rounded-br","rounded-bl"],"rounded-s":["rounded-ss","rounded-es"],"rounded-e":["rounded-se","rounded-ee"],"rounded-t":["rounded-tl","rounded-tr"],"rounded-r":["rounded-tr","rounded-br"],"rounded-b":["rounded-br","rounded-bl"],"rounded-l":["rounded-tl","rounded-bl"],"border-spacing":["border-spacing-x","border-spacing-y"],"border-w":["border-w-x","border-w-y","border-w-s","border-w-e","border-w-t","border-w-r","border-w-b","border-w-l"],"border-w-x":["border-w-r","border-w-l"],"border-w-y":["border-w-t","border-w-b"],"border-color":["border-color-x","border-color-y","border-color-s","border-color-e","border-color-t","border-color-r","border-color-b","border-color-l"],"border-color-x":["border-color-r","border-color-l"],"border-color-y":["border-color-t","border-color-b"],translate:["translate-x","translate-y","translate-none"],"translate-none":["translate","translate-x","translate-y","translate-z"],"scroll-m":["scroll-mx","scroll-my","scroll-ms","scroll-me","scroll-mt","scroll-mr","scroll-mb","scroll-ml"],"scroll-mx":["scroll-mr","scroll-ml"],"scroll-my":["scroll-mt","scroll-mb"],"scroll-p":["scroll-px","scroll-py","scroll-ps","scroll-pe","scroll-pt","scroll-pr","scroll-pb","scroll-pl"],"scroll-px":["scroll-pr","scroll-pl"],"scroll-py":["scroll-pt","scroll-pb"],touch:["touch-x","touch-y","touch-pz"],"touch-x":["touch"],"touch-y":["touch"],"touch-pz":["touch"]},conflictingClassGroupModifiers:{"font-size":["leading"]},orderSensitiveModifiers:["*","**","after","backdrop","before","details-content","file","first-letter","first-line","marker","placeholder","selection"]}},u0e=Gge(l0e);function lt(...e){return u0e(LY(e))}function Ff(){const e="0123456789abcdef";let t="#";for(let r=0;r<6;r++)t+=e.charAt(Math.floor(Math.random()*16));return t}function Da(e){return e instanceof Error?e.message:`${e}`}function c0e(e,t){let r=0,n=null;return function(...a){const i=Date.now(),o=t-(i-r);o<=0?(n&&(clearTimeout(n),n=null),r=i,e.apply(this,a)):n||(n=setTimeout(()=>{r=Date.now(),n=null,e.apply(this,a)},o))}}const DD=e=>{const t=e;t.use={};for(const r of Object.keys(t.getState()))t.use[r]=()=>t(n=>n[r]);return t};function jY(e,t){return function(){return e.apply(t,arguments)}}const{toString:d0e}=Object.prototype,{getPrototypeOf:ND}=Object,{iterator:Nv,toStringTag:qY}=Symbol,Lv=(e=>t=>{const r=d0e.call(t);return e[r]||(e[r]=r.slice(8,-1).toLowerCase())})(Object.create(null)),Oo=e=>(e=e.toLowerCase(),t=>Lv(t)===e),Mv=e=>t=>typeof t===e,{isArray:dp}=Array,Bh=Mv("undefined");function gg(e){return e!==null&&!Bh(e)&&e.constructor!==null&&!Bh(e.constructor)&&pi(e.constructor.isBuffer)&&e.constructor.isBuffer(e)}const WY=Oo("ArrayBuffer");function h0e(e){let t;return typeof ArrayBuffer<"u"&&ArrayBuffer.isView?t=ArrayBuffer.isView(e):t=e&&e.buffer&&WY(e.buffer),t}const p0e=Mv("string"),pi=Mv("function"),VY=Mv("number"),bg=e=>e!==null&&typeof e=="object",f0e=e=>e===!0||e===!1,S1=e=>{if(Lv(e)!=="object")return!1;const t=ND(e);return(t===null||t===Object.prototype||Object.getPrototypeOf(t)===null)&&!(qY in e)&&!(Nv in e)},m0e=e=>{if(!bg(e)||gg(e))return!1;try{return Object.keys(e).length===0&&Object.getPrototypeOf(e)===Object.prototype}catch{return!1}},g0e=Oo("Date"),b0e=Oo("File"),y0e=Oo("Blob"),v0e=Oo("FileList"),S0e=e=>bg(e)&&pi(e.pipe),E0e=e=>{let t;return e&&(typeof FormData=="function"&&e instanceof FormData||pi(e.append)&&((t=Lv(e))==="formdata"||t==="object"&&pi(e.toString)&&e.toString()==="[object FormData]"))},x0e=Oo("URLSearchParams"),[w0e,k0e,C0e,T0e]=["ReadableStream","Request","Response","Headers"].map(Oo),A0e=e=>e.trim?e.trim():e.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,"");function yg(e,t,{allOwnKeys:r=!1}={}){if(e===null||typeof e>"u")return;let n,a;if(typeof e!="object"&&(e=[e]),dp(e))for(n=0,a=e.length;n<a;n++)t.call(null,e[n],n,e);else{if(gg(e))return;const i=r?Object.getOwnPropertyNames(e):Object.keys(e),o=i.length;let s;for(n=0;n<o;n++)s=i[n],t.call(null,e[s],s,e)}}function KY(e,t){if(gg(e))return null;t=t.toLowerCase();const r=Object.keys(e);let n=r.length,a;for(;n-- >0;)if(a=r[n],t===a.toLowerCase())return a;return null}const Ec=typeof globalThis<"u"?globalThis:typeof self<"u"?self:typeof window<"u"?window:global,YY=e=>!Bh(e)&&e!==Ec;function O_(){const{caseless:e,skipUndefined:t}=YY(this)&&this||{},r={},n=(a,i)=>{const o=e&&KY(r,i)||i;S1(r[o])&&S1(a)?r[o]=O_(r[o],a):S1(a)?r[o]=O_({},a):dp(a)?r[o]=a.slice():(!t||!Bh(a))&&(r[o]=a)};for(let a=0,i=arguments.length;a<i;a++)arguments[a]&&yg(arguments[a],n);return r}const F0e=(e,t,r,{allOwnKeys:n}={})=>(yg(t,(a,i)=>{r&&pi(a)?e[i]=jY(a,r):e[i]=a},{allOwnKeys:n}),e),_0e=e=>(e.charCodeAt(0)===65279&&(e=e.slice(1)),e),R0e=(e,t,r,n)=>{e.prototype=Object.create(t.prototype,n),e.prototype.constructor=e,Object.defineProperty(e,"super",{value:t.prototype}),r&&Object.assign(e.prototype,r)},D0e=(e,t,r,n)=>{let a,i,o;const s={};if(t=t||{},e==null)return t;do{for(a=Object.getOwnPropertyNames(e),i=a.length;i-- >0;)o=a[i],(!n||n(o,e,t))&&!s[o]&&(t[o]=e[o],s[o]=!0);e=r!==!1&&ND(e)}while(e&&(!r||r(e,t))&&e!==Object.prototype);return t},N0e=(e,t,r)=>{e=String(e),(r===void 0||r>e.length)&&(r=e.length),r-=t.length;const n=e.indexOf(t,r);return n!==-1&&n===r},L0e=e=>{if(!e)return null;if(dp(e))return e;let t=e.length;if(!VY(t))return null;const r=new Array(t);for(;t-- >0;)r[t]=e[t];return r},M0e=(e=>t=>e&&t instanceof e)(typeof Uint8Array<"u"&&ND(Uint8Array)),I0e=(e,t)=>{const n=(e&&e[Nv]).call(e);let a;for(;(a=n.next())&&!a.done;){const i=a.value;t.call(e,i[0],i[1])}},O0e=(e,t)=>{let r;const n=[];for(;(r=e.exec(t))!==null;)n.push(r);return n},B0e=Oo("HTMLFormElement"),P0e=e=>e.toLowerCase().replace(/[-_\s]([a-z\d])(\w*)/g,function(r,n,a){return n.toUpperCase()+a}),l8=(({hasOwnProperty:e})=>(t,r)=>e.call(t,r))(Object.prototype),z0e=Oo("RegExp"),JY=(e,t)=>{const r=Object.getOwnPropertyDescriptors(e),n={};yg(r,(a,i)=>{let o;(o=t(a,i,e))!==!1&&(n[i]=o||a)}),Object.defineProperties(e,n)},H0e=e=>{JY(e,(t,r)=>{if(pi(e)&&["arguments","caller","callee"].indexOf(r)!==-1)return!1;const n=e[r];if(pi(n)){if(t.enumerable=!1,"writable"in t){t.writable=!1;return}t.set||(t.set=()=>{throw Error("Can not rewrite read-only method '"+r+"'")})}})},U0e=(e,t)=>{const r={},n=a=>{a.forEach(i=>{r[i]=!0})};return dp(e)?n(e):n(String(e).split(t)),r},$0e=()=>{},G0e=(e,t)=>e!=null&&Number.isFinite(e=+e)?e:t;function j0e(e){return!!(e&&pi(e.append)&&e[qY]==="FormData"&&e[Nv])}const q0e=e=>{const t=new Array(10),r=(n,a)=>{if(bg(n)){if(t.indexOf(n)>=0)return;if(gg(n))return n;if(!("toJSON"in n)){t[a]=n;const i=dp(n)?[]:{};return yg(n,(o,s)=>{const l=r(o,a+1);!Bh(l)&&(i[s]=l)}),t[a]=void 0,i}}return n};return r(e,0)},W0e=Oo("AsyncFunction"),V0e=e=>e&&(bg(e)||pi(e))&&pi(e.then)&&pi(e.catch),XY=((e,t)=>e?setImmediate:t?((r,n)=>(Ec.addEventListener("message",({source:a,data:i})=>{a===Ec&&i===r&&n.length&&n.shift()()},!1),a=>{n.push(a),Ec.postMessage(r,"*")}))(`axios@${Math.random()}`,[]):r=>setTimeout(r))(typeof setImmediate=="function",pi(Ec.postMessage)),K0e=typeof queueMicrotask<"u"?queueMicrotask.bind(Ec):typeof process<"u"&&process.nextTick||XY,Y0e=e=>e!=null&&pi(e[Nv]),Le={isArray:dp,isArrayBuffer:WY,isBuffer:gg,isFormData:E0e,isArrayBufferView:h0e,isString:p0e,isNumber:VY,isBoolean:f0e,isObject:bg,isPlainObject:S1,isEmptyObject:m0e,isReadableStream:w0e,isRequest:k0e,isResponse:C0e,isHeaders:T0e,isUndefined:Bh,isDate:g0e,isFile:b0e,isBlob:y0e,isRegExp:z0e,isFunction:pi,isStream:S0e,isURLSearchParams:x0e,isTypedArray:M0e,isFileList:v0e,forEach:yg,merge:O_,extend:F0e,trim:A0e,stripBOM:_0e,inherits:R0e,toFlatObject:D0e,kindOf:Lv,kindOfTest:Oo,endsWith:N0e,toArray:L0e,forEachEntry:I0e,matchAll:O0e,isHTMLForm:B0e,hasOwnProperty:l8,hasOwnProp:l8,reduceDescriptors:JY,freezeMethods:H0e,toObjectSet:U0e,toCamelCase:P0e,noop:$0e,toFiniteNumber:G0e,findKey:KY,global:Ec,isContextDefined:YY,isSpecCompliantForm:j0e,toJSONObject:q0e,isAsyncFn:W0e,isThenable:V0e,setImmediate:XY,asap:K0e,isIterable:Y0e};function pr(e,t,r,n,a){Error.call(this),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=new Error().stack,this.message=e,this.name="AxiosError",t&&(this.code=t),r&&(this.config=r),n&&(this.request=n),a&&(this.response=a,this.status=a.status?a.status:null)}Le.inherits(pr,Error,{toJSON:function(){return{message:this.message,name:this.name,description:this.description,number:this.number,fileName:this.fileName,lineNumber:this.lineNumber,columnNumber:this.columnNumber,stack:this.stack,config:Le.toJSONObject(this.config),code:this.code,status:this.status}}});const ZY=pr.prototype,QY={};["ERR_BAD_OPTION_VALUE","ERR_BAD_OPTION","ECONNABORTED","ETIMEDOUT","ERR_NETWORK","ERR_FR_TOO_MANY_REDIRECTS","ERR_DEPRECATED","ERR_BAD_RESPONSE","ERR_BAD_REQUEST","ERR_CANCELED","ERR_NOT_SUPPORT","ERR_INVALID_URL"].forEach(e=>{QY[e]={value:e}});Object.defineProperties(pr,QY);Object.defineProperty(ZY,"isAxiosError",{value:!0});pr.from=(e,t,r,n,a,i)=>{const o=Object.create(ZY);Le.toFlatObject(e,o,function(d){return d!==Error.prototype},u=>u!=="isAxiosError");const s=e&&e.message?e.message:"Error",l=t==null&&e?e.code:t;return pr.call(o,s,l,r,n,a),e&&o.cause==null&&Object.defineProperty(o,"cause",{value:e,configurable:!0}),o.name=e&&e.name||"Error",i&&Object.assign(o,i),o};const J0e=null;function B_(e){return Le.isPlainObject(e)||Le.isArray(e)}function eJ(e){return Le.endsWith(e,"[]")?e.slice(0,-2):e}function u8(e,t,r){return e?e.concat(t).map(function(a,i){return a=eJ(a),!r&&i?"["+a+"]":a}).join(r?".":""):t}function X0e(e){return Le.isArray(e)&&!e.some(B_)}const Z0e=Le.toFlatObject(Le,{},null,function(t){return/^is[A-Z]/.test(t)});function Iv(e,t,r){if(!Le.isObject(e))throw new TypeError("target must be an object");t=t||new FormData,r=Le.toFlatObject(r,{metaTokens:!0,dots:!1,indexes:!1},!1,function(y,S){return!Le.isUndefined(S[y])});const n=r.metaTokens,a=r.visitor||d,i=r.dots,o=r.indexes,l=(r.Blob||typeof Blob<"u"&&Blob)&&Le.isSpecCompliantForm(t);if(!Le.isFunction(a))throw new TypeError("visitor must be a function");function u(b){if(b===null)return"";if(Le.isDate(b))return b.toISOString();if(Le.isBoolean(b))return b.toString();if(!l&&Le.isBlob(b))throw new pr("Blob is not supported. Use a Buffer instead.");return Le.isArrayBuffer(b)||Le.isTypedArray(b)?l&&typeof Blob=="function"?new Blob([b]):Buffer.from(b):b}function d(b,y,S){let w=b;if(b&&!S&&typeof b=="object"){if(Le.endsWith(y,"{}"))y=n?y:y.slice(0,-2),b=JSON.stringify(b);else if(Le.isArray(b)&&X0e(b)||(Le.isFileList(b)||Le.endsWith(y,"[]"))&&(w=Le.toArray(b)))return y=eJ(y),w.forEach(function(k,_){!(Le.isUndefined(k)||k===null)&&t.append(o===!0?u8([y],_,i):o===null?y:y+"[]",u(k))}),!1}return B_(b)?!0:(t.append(u8(S,y,i),u(b)),!1)}const p=[],f=Object.assign(Z0e,{defaultVisitor:d,convertValue:u,isVisitable:B_});function m(b,y){if(!Le.isUndefined(b)){if(p.indexOf(b)!==-1)throw Error("Circular reference detected in "+y.join("."));p.push(b),Le.forEach(b,function(w,E){(!(Le.isUndefined(w)||w===null)&&a.call(t,w,Le.isString(E)?E.trim():E,y,f))===!0&&m(w,y?y.concat(E):[E])}),p.pop()}}if(!Le.isObject(e))throw new TypeError("data must be an object");return m(e),t}function c8(e){const t={"!":"%21","'":"%27","(":"%28",")":"%29","~":"%7E","%20":"+","%00":"\0"};return encodeURIComponent(e).replace(/[!'()~]|%20|%00/g,function(n){return t[n]})}function LD(e,t){this._pairs=[],e&&Iv(e,this,t)}const tJ=LD.prototype;tJ.append=function(t,r){this._pairs.push([t,r])};tJ.toString=function(t){const r=t?function(n){return t.call(this,n,c8)}:c8;return this._pairs.map(function(a){return r(a[0])+"="+r(a[1])},"").join("&")};function Q0e(e){return encodeURIComponent(e).replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",").replace(/%20/g,"+")}function rJ(e,t,r){if(!t)return e;const n=r&&r.encode||Q0e;Le.isFunction(r)&&(r={serialize:r});const a=r&&r.serialize;let i;if(a?i=a(t,r):i=Le.isURLSearchParams(t)?t.toString():new LD(t,r).toString(n),i){const o=e.indexOf("#");o!==-1&&(e=e.slice(0,o)),e+=(e.indexOf("?")===-1?"?":"&")+i}return e}class d8{constructor(){this.handlers=[]}use(t,r,n){return this.handlers.push({fulfilled:t,rejected:r,synchronous:n?n.synchronous:!1,runWhen:n?n.runWhen:null}),this.handlers.length-1}eject(t){this.handlers[t]&&(this.handlers[t]=null)}clear(){this.handlers&&(this.handlers=[])}forEach(t){Le.forEach(this.handlers,function(n){n!==null&&t(n)})}}const nJ={silentJSONParsing:!0,forcedJSONParsing:!0,clarifyTimeoutError:!1},ebe=typeof URLSearchParams<"u"?URLSearchParams:LD,tbe=typeof FormData<"u"?FormData:null,rbe=typeof Blob<"u"?Blob:null,nbe={isBrowser:!0,classes:{URLSearchParams:ebe,FormData:tbe,Blob:rbe},protocols:["http","https","file","blob","url","data"]},MD=typeof window<"u"&&typeof document<"u",P_=typeof navigator=="object"&&navigator||void 0,abe=MD&&(!P_||["ReactNative","NativeScript","NS"].indexOf(P_.product)<0),ibe=typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope&&typeof self.importScripts=="function",obe=MD&&window.location.href||"http://localhost",sbe=Object.freeze(Object.defineProperty({__proto__:null,hasBrowserEnv:MD,hasStandardBrowserEnv:abe,hasStandardBrowserWebWorkerEnv:ibe,navigator:P_,origin:obe},Symbol.toStringTag,{value:"Module"})),Ra={...sbe,...nbe};function lbe(e,t){return Iv(e,new Ra.classes.URLSearchParams,{visitor:function(r,n,a,i){return Ra.isNode&&Le.isBuffer(r)?(this.append(n,r.toString("base64")),!1):i.defaultVisitor.apply(this,arguments)},...t})}function ube(e){return Le.matchAll(/\w+|\[(\w*)]/g,e).map(t=>t[0]==="[]"?"":t[1]||t[0])}function cbe(e){const t={},r=Object.keys(e);let n;const a=r.length;let i;for(n=0;n<a;n++)i=r[n],t[i]=e[i];return t}function aJ(e){function t(r,n,a,i){let o=r[i++];if(o==="__proto__")return!0;const s=Number.isFinite(+o),l=i>=r.length;return o=!o&&Le.isArray(a)?a.length:o,l?(Le.hasOwnProp(a,o)?a[o]=[a[o],n]:a[o]=n,!s):((!a[o]||!Le.isObject(a[o]))&&(a[o]=[]),t(r,n,a[o],i)&&Le.isArray(a[o])&&(a[o]=cbe(a[o])),!s)}if(Le.isFormData(e)&&Le.isFunction(e.entries)){const r={};return Le.forEachEntry(e,(n,a)=>{t(ube(n),a,r,0)}),r}return null}function dbe(e,t,r){if(Le.isString(e))try{return(t||JSON.parse)(e),Le.trim(e)}catch(n){if(n.name!=="SyntaxError")throw n}return(r||JSON.stringify)(e)}const vg={transitional:nJ,adapter:["xhr","http","fetch"],transformRequest:[function(t,r){const n=r.getContentType()||"",a=n.indexOf("application/json")>-1,i=Le.isObject(t);if(i&&Le.isHTMLForm(t)&&(t=new FormData(t)),Le.isFormData(t))return a?JSON.stringify(aJ(t)):t;if(Le.isArrayBuffer(t)||Le.isBuffer(t)||Le.isStream(t)||Le.isFile(t)||Le.isBlob(t)||Le.isReadableStream(t))return t;if(Le.isArrayBufferView(t))return t.buffer;if(Le.isURLSearchParams(t))return r.setContentType("application/x-www-form-urlencoded;charset=utf-8",!1),t.toString();let s;if(i){if(n.indexOf("application/x-www-form-urlencoded")>-1)return lbe(t,this.formSerializer).toString();if((s=Le.isFileList(t))||n.indexOf("multipart/form-data")>-1){const l=this.env&&this.env.FormData;return Iv(s?{"files[]":t}:t,l&&new l,this.formSerializer)}}return i||a?(r.setContentType("application/json",!1),dbe(t)):t}],transformResponse:[function(t){const r=this.transitional||vg.transitional,n=r&&r.forcedJSONParsing,a=this.responseType==="json";if(Le.isResponse(t)||Le.isReadableStream(t))return t;if(t&&Le.isString(t)&&(n&&!this.responseType||a)){const o=!(r&&r.silentJSONParsing)&&a;try{return JSON.parse(t,this.parseReviver)}catch(s){if(o)throw s.name==="SyntaxError"?pr.from(s,pr.ERR_BAD_RESPONSE,this,null,this.response):s}}return t}],timeout:0,xsrfCookieName:"XSRF-TOKEN",xsrfHeaderName:"X-XSRF-TOKEN",maxContentLength:-1,maxBodyLength:-1,env:{FormData:Ra.classes.FormData,Blob:Ra.classes.Blob},validateStatus:function(t){return t>=200&&t<300},headers:{common:{Accept:"application/json, text/plain, */*","Content-Type":void 0}}};Le.forEach(["delete","get","head","post","put","patch"],e=>{vg.headers[e]={}});const hbe=Le.toObjectSet(["age","authorization","content-length","content-type","etag","expires","from","host","if-modified-since","if-unmodified-since","last-modified","location","max-forwards","proxy-authorization","referer","retry-after","user-agent"]),pbe=e=>{const t={};let r,n,a;return e&&e.split(`
`).forEach(function(o){a=o.indexOf(":"),r=o.substring(0,a).trim().toLowerCase(),n=o.substring(a+1).trim(),!(!r||t[r]&&hbe[r])&&(r==="set-cookie"?t[r]?t[r].push(n):t[r]=[n]:t[r]=t[r]?t[r]+", "+n:n)}),t},h8=Symbol("internals");function _f(e){return e&&String(e).trim().toLowerCase()}function E1(e){return e===!1||e==null?e:Le.isArray(e)?e.map(E1):String(e)}function fbe(e){const t=Object.create(null),r=/([^\s,;=]+)\s*(?:=\s*([^,;]+))?/g;let n;for(;n=r.exec(e);)t[n[1]]=n[2];return t}const mbe=e=>/^[-_a-zA-Z0-9^`|~,!#$%&'*+.]+$/.test(e.trim());function Hx(e,t,r,n,a){if(Le.isFunction(n))return n.call(this,t,r);if(a&&(t=r),!!Le.isString(t)){if(Le.isString(n))return t.indexOf(n)!==-1;if(Le.isRegExp(n))return n.test(t)}}function gbe(e){return e.trim().toLowerCase().replace(/([a-z\d])(\w*)/g,(t,r,n)=>r.toUpperCase()+n)}function bbe(e,t){const r=Le.toCamelCase(" "+t);["get","set","has"].forEach(n=>{Object.defineProperty(e,n+r,{value:function(a,i,o){return this[n].call(this,t,a,i,o)},configurable:!0})})}let fi=class{constructor(t){t&&this.set(t)}set(t,r,n){const a=this;function i(s,l,u){const d=_f(l);if(!d)throw new Error("header name must be a non-empty string");const p=Le.findKey(a,d);(!p||a[p]===void 0||u===!0||u===void 0&&a[p]!==!1)&&(a[p||l]=E1(s))}const o=(s,l)=>Le.forEach(s,(u,d)=>i(u,d,l));if(Le.isPlainObject(t)||t instanceof this.constructor)o(t,r);else if(Le.isString(t)&&(t=t.trim())&&!mbe(t))o(pbe(t),r);else if(Le.isObject(t)&&Le.isIterable(t)){let s={},l,u;for(const d of t){if(!Le.isArray(d))throw TypeError("Object iterator must return a key-value pair");s[u=d[0]]=(l=s[u])?Le.isArray(l)?[...l,d[1]]:[l,d[1]]:d[1]}o(s,r)}else t!=null&&i(r,t,n);return this}get(t,r){if(t=_f(t),t){const n=Le.findKey(this,t);if(n){const a=this[n];if(!r)return a;if(r===!0)return fbe(a);if(Le.isFunction(r))return r.call(this,a,n);if(Le.isRegExp(r))return r.exec(a);throw new TypeError("parser must be boolean|regexp|function")}}}has(t,r){if(t=_f(t),t){const n=Le.findKey(this,t);return!!(n&&this[n]!==void 0&&(!r||Hx(this,this[n],n,r)))}return!1}delete(t,r){const n=this;let a=!1;function i(o){if(o=_f(o),o){const s=Le.findKey(n,o);s&&(!r||Hx(n,n[s],s,r))&&(delete n[s],a=!0)}}return Le.isArray(t)?t.forEach(i):i(t),a}clear(t){const r=Object.keys(this);let n=r.length,a=!1;for(;n--;){const i=r[n];(!t||Hx(this,this[i],i,t,!0))&&(delete this[i],a=!0)}return a}normalize(t){const r=this,n={};return Le.forEach(this,(a,i)=>{const o=Le.findKey(n,i);if(o){r[o]=E1(a),delete r[i];return}const s=t?gbe(i):String(i).trim();s!==i&&delete r[i],r[s]=E1(a),n[s]=!0}),this}concat(...t){return this.constructor.concat(this,...t)}toJSON(t){const r=Object.create(null);return Le.forEach(this,(n,a)=>{n!=null&&n!==!1&&(r[a]=t&&Le.isArray(n)?n.join(", "):n)}),r}[Symbol.iterator](){return Object.entries(this.toJSON())[Symbol.iterator]()}toString(){return Object.entries(this.toJSON()).map(([t,r])=>t+": "+r).join(`
`)}getSetCookie(){return this.get("set-cookie")||[]}get[Symbol.toStringTag](){return"AxiosHeaders"}static from(t){return t instanceof this?t:new this(t)}static concat(t,...r){const n=new this(t);return r.forEach(a=>n.set(a)),n}static accessor(t){const n=(this[h8]=this[h8]={accessors:{}}).accessors,a=this.prototype;function i(o){const s=_f(o);n[s]||(bbe(a,o),n[s]=!0)}return Le.isArray(t)?t.forEach(i):i(t),this}};fi.accessor(["Content-Type","Content-Length","Accept","Accept-Encoding","User-Agent","Authorization"]);Le.reduceDescriptors(fi.prototype,({value:e},t)=>{let r=t[0].toUpperCase()+t.slice(1);return{get:()=>e,set(n){this[r]=n}}});Le.freezeMethods(fi);function Ux(e,t){const r=this||vg,n=t||r,a=fi.from(n.headers);let i=n.data;return Le.forEach(e,function(s){i=s.call(r,i,a.normalize(),t?t.status:void 0)}),a.normalize(),i}function iJ(e){return!!(e&&e.__CANCEL__)}function hp(e,t,r){pr.call(this,e??"canceled",pr.ERR_CANCELED,t,r),this.name="CanceledError"}Le.inherits(hp,pr,{__CANCEL__:!0});function oJ(e,t,r){const n=r.config.validateStatus;!r.status||!n||n(r.status)?e(r):t(new pr("Request failed with status code "+r.status,[pr.ERR_BAD_REQUEST,pr.ERR_BAD_RESPONSE][Math.floor(r.status/100)-4],r.config,r.request,r))}function ybe(e){const t=/^([-+\w]{1,25})(:?\/\/|:)/.exec(e);return t&&t[1]||""}function vbe(e,t){e=e||10;const r=new Array(e),n=new Array(e);let a=0,i=0,o;return t=t!==void 0?t:1e3,function(l){const u=Date.now(),d=n[i];o||(o=u),r[a]=l,n[a]=u;let p=i,f=0;for(;p!==a;)f+=r[p++],p=p%e;if(a=(a+1)%e,a===i&&(i=(i+1)%e),u-o<t)return;const m=d&&u-d;return m?Math.round(f*1e3/m):void 0}}function Sbe(e,t){let r=0,n=1e3/t,a,i;const o=(u,d=Date.now())=>{r=d,a=null,i&&(clearTimeout(i),i=null),e(...u)};return[(...u)=>{const d=Date.now(),p=d-r;p>=n?o(u,d):(a=u,i||(i=setTimeout(()=>{i=null,o(a)},n-p)))},()=>a&&o(a)]}const sy=(e,t,r=3)=>{let n=0;const a=vbe(50,250);return Sbe(i=>{const o=i.loaded,s=i.lengthComputable?i.total:void 0,l=o-n,u=a(l),d=o<=s;n=o;const p={loaded:o,total:s,progress:s?o/s:void 0,bytes:l,rate:u||void 0,estimated:u&&s&&d?(s-o)/u:void 0,event:i,lengthComputable:s!=null,[t?"download":"upload"]:!0};e(p)},r)},p8=(e,t)=>{const r=e!=null;return[n=>t[0]({lengthComputable:r,total:e,loaded:n}),t[1]]},f8=e=>(...t)=>Le.asap(()=>e(...t)),Ebe=Ra.hasStandardBrowserEnv?((e,t)=>r=>(r=new URL(r,Ra.origin),e.protocol===r.protocol&&e.host===r.host&&(t||e.port===r.port)))(new URL(Ra.origin),Ra.navigator&&/(msie|trident)/i.test(Ra.navigator.userAgent)):()=>!0,xbe=Ra.hasStandardBrowserEnv?{write(e,t,r,n,a,i){const o=[e+"="+encodeURIComponent(t)];Le.isNumber(r)&&o.push("expires="+new Date(r).toGMTString()),Le.isString(n)&&o.push("path="+n),Le.isString(a)&&o.push("domain="+a),i===!0&&o.push("secure"),document.cookie=o.join("; ")},read(e){const t=document.cookie.match(new RegExp("(^|;\\s*)("+e+")=([^;]*)"));return t?decodeURIComponent(t[3]):null},remove(e){this.write(e,"",Date.now()-864e5)}}:{write(){},read(){return null},remove(){}};function wbe(e){return/^([a-z][a-z\d+\-.]*:)?\/\//i.test(e)}function kbe(e,t){return t?e.replace(/\/?\/$/,"")+"/"+t.replace(/^\/+/,""):e}function sJ(e,t,r){let n=!wbe(t);return e&&(n||r==!1)?kbe(e,t):t}const m8=e=>e instanceof fi?{...e}:e;function Mc(e,t){t=t||{};const r={};function n(u,d,p,f){return Le.isPlainObject(u)&&Le.isPlainObject(d)?Le.merge.call({caseless:f},u,d):Le.isPlainObject(d)?Le.merge({},d):Le.isArray(d)?d.slice():d}function a(u,d,p,f){if(Le.isUndefined(d)){if(!Le.isUndefined(u))return n(void 0,u,p,f)}else return n(u,d,p,f)}function i(u,d){if(!Le.isUndefined(d))return n(void 0,d)}function o(u,d){if(Le.isUndefined(d)){if(!Le.isUndefined(u))return n(void 0,u)}else return n(void 0,d)}function s(u,d,p){if(p in t)return n(u,d);if(p in e)return n(void 0,u)}const l={url:i,method:i,data:i,baseURL:o,transformRequest:o,transformResponse:o,paramsSerializer:o,timeout:o,timeoutMessage:o,withCredentials:o,withXSRFToken:o,adapter:o,responseType:o,xsrfCookieName:o,xsrfHeaderName:o,onUploadProgress:o,onDownloadProgress:o,decompress:o,maxContentLength:o,maxBodyLength:o,beforeRedirect:o,transport:o,httpAgent:o,httpsAgent:o,cancelToken:o,socketPath:o,responseEncoding:o,validateStatus:s,headers:(u,d,p)=>a(m8(u),m8(d),p,!0)};return Le.forEach(Object.keys({...e,...t}),function(d){const p=l[d]||a,f=p(e[d],t[d],d);Le.isUndefined(f)&&p!==s||(r[d]=f)}),r}const lJ=e=>{const t=Mc({},e);let{data:r,withXSRFToken:n,xsrfHeaderName:a,xsrfCookieName:i,headers:o,auth:s}=t;if(t.headers=o=fi.from(o),t.url=rJ(sJ(t.baseURL,t.url,t.allowAbsoluteUrls),e.params,e.paramsSerializer),s&&o.set("Authorization","Basic "+btoa((s.username||"")+":"+(s.password?unescape(encodeURIComponent(s.password)):""))),Le.isFormData(r)){if(Ra.hasStandardBrowserEnv||Ra.hasStandardBrowserWebWorkerEnv)o.setContentType(void 0);else if(Le.isFunction(r.getHeaders)){const l=r.getHeaders(),u=["content-type","content-length"];Object.entries(l).forEach(([d,p])=>{u.includes(d.toLowerCase())&&o.set(d,p)})}}if(Ra.hasStandardBrowserEnv&&(n&&Le.isFunction(n)&&(n=n(t)),n||n!==!1&&Ebe(t.url))){const l=a&&i&&xbe.read(i);l&&o.set(a,l)}return t},Cbe=typeof XMLHttpRequest<"u",Tbe=Cbe&&function(e){return new Promise(function(r,n){const a=lJ(e);let i=a.data;const o=fi.from(a.headers).normalize();let{responseType:s,onUploadProgress:l,onDownloadProgress:u}=a,d,p,f,m,b;function y(){m&&m(),b&&b(),a.cancelToken&&a.cancelToken.unsubscribe(d),a.signal&&a.signal.removeEventListener("abort",d)}let S=new XMLHttpRequest;S.open(a.method.toUpperCase(),a.url,!0),S.timeout=a.timeout;function w(){if(!S)return;const k=fi.from("getAllResponseHeaders"in S&&S.getAllResponseHeaders()),R={data:!s||s==="text"||s==="json"?S.responseText:S.response,status:S.status,statusText:S.statusText,headers:k,config:e,request:S};oJ(function(F){r(F),y()},function(F){n(F),y()},R),S=null}"onloadend"in S?S.onloadend=w:S.onreadystatechange=function(){!S||S.readyState!==4||S.status===0&&!(S.responseURL&&S.responseURL.indexOf("file:")===0)||setTimeout(w)},S.onabort=function(){S&&(n(new pr("Request aborted",pr.ECONNABORTED,e,S)),S=null)},S.onerror=function(_){const R=_&&_.message?_.message:"Network Error",C=new pr(R,pr.ERR_NETWORK,e,S);C.event=_||null,n(C),S=null},S.ontimeout=function(){let _=a.timeout?"timeout of "+a.timeout+"ms exceeded":"timeout exceeded";const R=a.transitional||nJ;a.timeoutErrorMessage&&(_=a.timeoutErrorMessage),n(new pr(_,R.clarifyTimeoutError?pr.ETIMEDOUT:pr.ECONNABORTED,e,S)),S=null},i===void 0&&o.setContentType(null),"setRequestHeader"in S&&Le.forEach(o.toJSON(),function(_,R){S.setRequestHeader(R,_)}),Le.isUndefined(a.withCredentials)||(S.withCredentials=!!a.withCredentials),s&&s!=="json"&&(S.responseType=a.responseType),u&&([f,b]=sy(u,!0),S.addEventListener("progress",f)),l&&S.upload&&([p,m]=sy(l),S.upload.addEventListener("progress",p),S.upload.addEventListener("loadend",m)),(a.cancelToken||a.signal)&&(d=k=>{S&&(n(!k||k.type?new hp(null,e,S):k),S.abort(),S=null)},a.cancelToken&&a.cancelToken.subscribe(d),a.signal&&(a.signal.aborted?d():a.signal.addEventListener("abort",d)));const E=ybe(a.url);if(E&&Ra.protocols.indexOf(E)===-1){n(new pr("Unsupported protocol "+E+":",pr.ERR_BAD_REQUEST,e));return}S.send(i||null)})},Abe=(e,t)=>{const{length:r}=e=e?e.filter(Boolean):[];if(t||r){let n=new AbortController,a;const i=function(u){if(!a){a=!0,s();const d=u instanceof Error?u:this.reason;n.abort(d instanceof pr?d:new hp(d instanceof Error?d.message:d))}};let o=t&&setTimeout(()=>{o=null,i(new pr(`timeout ${t} of ms exceeded`,pr.ETIMEDOUT))},t);const s=()=>{e&&(o&&clearTimeout(o),o=null,e.forEach(u=>{u.unsubscribe?u.unsubscribe(i):u.removeEventListener("abort",i)}),e=null)};e.forEach(u=>u.addEventListener("abort",i));const{signal:l}=n;return l.unsubscribe=()=>Le.asap(s),l}},Fbe=function*(e,t){let r=e.byteLength;if(r<t){yield e;return}let n=0,a;for(;n<r;)a=n+t,yield e.slice(n,a),n=a},_be=async function*(e,t){for await(const r of Rbe(e))yield*Fbe(r,t)},Rbe=async function*(e){if(e[Symbol.asyncIterator]){yield*e;return}const t=e.getReader();try{for(;;){const{done:r,value:n}=await t.read();if(r)break;yield n}}finally{await t.cancel()}},g8=(e,t,r,n)=>{const a=_be(e,t);let i=0,o,s=l=>{o||(o=!0,n&&n(l))};return new ReadableStream({async pull(l){try{const{done:u,value:d}=await a.next();if(u){s(),l.close();return}let p=d.byteLength;if(r){let f=i+=p;r(f)}l.enqueue(new Uint8Array(d))}catch(u){throw s(u),u}},cancel(l){return s(l),a.return()}},{highWaterMark:2})},b8=64*1024,{isFunction:wb}=Le,Dbe=(({Request:e,Response:t})=>({Request:e,Response:t}))(Le.global),{ReadableStream:y8,TextEncoder:v8}=Le.global,S8=(e,...t)=>{try{return!!e(...t)}catch{return!1}},Nbe=e=>{e=Le.merge.call({skipUndefined:!0},Dbe,e);const{fetch:t,Request:r,Response:n}=e,a=t?wb(t):typeof fetch=="function",i=wb(r),o=wb(n);if(!a)return!1;const s=a&&wb(y8),l=a&&(typeof v8=="function"?(b=>y=>b.encode(y))(new v8):async b=>new Uint8Array(await new r(b).arrayBuffer())),u=i&&s&&S8(()=>{let b=!1;const y=new r(Ra.origin,{body:new y8,method:"POST",get duplex(){return b=!0,"half"}}).headers.has("Content-Type");return b&&!y}),d=o&&s&&S8(()=>Le.isReadableStream(new n("").body)),p={stream:d&&(b=>b.body)};a&&["text","arrayBuffer","blob","formData","stream"].forEach(b=>{!p[b]&&(p[b]=(y,S)=>{let w=y&&y[b];if(w)return w.call(y);throw new pr(`Response type '${b}' is not supported`,pr.ERR_NOT_SUPPORT,S)})});const f=async b=>{if(b==null)return 0;if(Le.isBlob(b))return b.size;if(Le.isSpecCompliantForm(b))return(await new r(Ra.origin,{method:"POST",body:b}).arrayBuffer()).byteLength;if(Le.isArrayBufferView(b)||Le.isArrayBuffer(b))return b.byteLength;if(Le.isURLSearchParams(b)&&(b=b+""),Le.isString(b))return(await l(b)).byteLength},m=async(b,y)=>{const S=Le.toFiniteNumber(b.getContentLength());return S??f(y)};return async b=>{let{url:y,method:S,data:w,signal:E,cancelToken:k,timeout:_,onDownloadProgress:R,onUploadProgress:C,responseType:F,headers:L,withCredentials:I="same-origin",fetchOptions:M}=lJ(b),j=t||fetch;F=F?(F+"").toLowerCase():"text";let Y=Abe([E,k&&k.toAbortSignal()],_),W=null;const $=Y&&Y.unsubscribe&&(()=>{Y.unsubscribe()});let X;try{if(C&&u&&S!=="get"&&S!=="head"&&(X=await m(L,w))!==0){let K=new r(y,{method:"POST",body:w,duplex:"half"}),G;if(Le.isFormData(w)&&(G=K.headers.get("content-type"))&&L.setContentType(G),K.body){const[P,ne]=p8(X,sy(f8(C)));w=g8(K.body,b8,P,ne)}}Le.isString(I)||(I=I?"include":"omit");const z=i&&"credentials"in r.prototype,H={...M,signal:Y,method:S.toUpperCase(),headers:L.normalize().toJSON(),body:w,duplex:"half",credentials:z?I:void 0};W=i&&new r(y,H);let B=await(i?j(W,M):j(y,H));const ee=d&&(F==="stream"||F==="response");if(d&&(R||ee&&$)){const K={};["status","statusText","headers"].forEach(oe=>{K[oe]=B[oe]});const G=Le.toFiniteNumber(B.headers.get("content-length")),[P,ne]=R&&p8(G,sy(f8(R),!0))||[];B=new n(g8(B.body,b8,P,()=>{ne&&ne(),$&&$()}),K)}F=F||"text";let J=await p[Le.findKey(p,F)||"text"](B,b);return!ee&&$&&$(),await new Promise((K,G)=>{oJ(K,G,{data:J,headers:fi.from(B.headers),status:B.status,statusText:B.statusText,config:b,request:W})})}catch(z){throw $&&$(),z&&z.name==="TypeError"&&/Load failed|fetch/i.test(z.message)?Object.assign(new pr("Network Error",pr.ERR_NETWORK,b,W),{cause:z.cause||z}):pr.from(z,z&&z.code,b,W)}}},Lbe=new Map,uJ=e=>{let t=e?e.env:{};const{fetch:r,Request:n,Response:a}=t,i=[n,a,r];let o=i.length,s=o,l,u,d=Lbe;for(;s--;)l=i[s],u=d.get(l),u===void 0&&d.set(l,u=s?new Map:Nbe(t)),d=u;return u};uJ();const z_={http:J0e,xhr:Tbe,fetch:{get:uJ}};Le.forEach(z_,(e,t)=>{if(e){try{Object.defineProperty(e,"name",{value:t})}catch{}Object.defineProperty(e,"adapterName",{value:t})}});const E8=e=>`- ${e}`,Mbe=e=>Le.isFunction(e)||e===null||e===!1,cJ={getAdapter:(e,t)=>{e=Le.isArray(e)?e:[e];const{length:r}=e;let n,a;const i={};for(let o=0;o<r;o++){n=e[o];let s;if(a=n,!Mbe(n)&&(a=z_[(s=String(n)).toLowerCase()],a===void 0))throw new pr(`Unknown adapter '${s}'`);if(a&&(Le.isFunction(a)||(a=a.get(t))))break;i[s||"#"+o]=a}if(!a){const o=Object.entries(i).map(([l,u])=>`adapter ${l} `+(u===!1?"is not supported by the environment":"is not available in the build"));let s=r?o.length>1?`since :
`+o.map(E8).join(`
`):" "+E8(o[0]):"as no adapter specified";throw new pr("There is no suitable adapter to dispatch the request "+s,"ERR_NOT_SUPPORT")}return a},adapters:z_};function $x(e){if(e.cancelToken&&e.cancelToken.throwIfRequested(),e.signal&&e.signal.aborted)throw new hp(null,e)}function x8(e){return $x(e),e.headers=fi.from(e.headers),e.data=Ux.call(e,e.transformRequest),["post","put","patch"].indexOf(e.method)!==-1&&e.headers.setContentType("application/x-www-form-urlencoded",!1),cJ.getAdapter(e.adapter||vg.adapter,e)(e).then(function(n){return $x(e),n.data=Ux.call(e,e.transformResponse,n),n.headers=fi.from(n.headers),n},function(n){return iJ(n)||($x(e),n&&n.response&&(n.response.data=Ux.call(e,e.transformResponse,n.response),n.response.headers=fi.from(n.response.headers))),Promise.reject(n)})}const dJ="1.12.2",Ov={};["object","boolean","number","function","string","symbol"].forEach((e,t)=>{Ov[e]=function(n){return typeof n===e||"a"+(t<1?"n ":" ")+e}});const w8={};Ov.transitional=function(t,r,n){function a(i,o){return"[Axios v"+dJ+"] Transitional option '"+i+"'"+o+(n?". "+n:"")}return(i,o,s)=>{if(t===!1)throw new pr(a(o," has been removed"+(r?" in "+r:"")),pr.ERR_DEPRECATED);return r&&!w8[o]&&(w8[o]=!0,console.warn(a(o," has been deprecated since v"+r+" and will be removed in the near future"))),t?t(i,o,s):!0}};Ov.spelling=function(t){return(r,n)=>(console.warn(`${n} is likely a misspelling of ${t}`),!0)};function Ibe(e,t,r){if(typeof e!="object")throw new pr("options must be an object",pr.ERR_BAD_OPTION_VALUE);const n=Object.keys(e);let a=n.length;for(;a-- >0;){const i=n[a],o=t[i];if(o){const s=e[i],l=s===void 0||o(s,i,e);if(l!==!0)throw new pr("option "+i+" must be "+l,pr.ERR_BAD_OPTION_VALUE);continue}if(r!==!0)throw new pr("Unknown option "+i,pr.ERR_BAD_OPTION)}}const x1={assertOptions:Ibe,validators:Ov},Wo=x1.validators;let Fc=class{constructor(t){this.defaults=t||{},this.interceptors={request:new d8,response:new d8}}async request(t,r){try{return await this._request(t,r)}catch(n){if(n instanceof Error){let a={};Error.captureStackTrace?Error.captureStackTrace(a):a=new Error;const i=a.stack?a.stack.replace(/^.+\n/,""):"";try{n.stack?i&&!String(n.stack).endsWith(i.replace(/^.+\n.+\n/,""))&&(n.stack+=`
`+i):n.stack=i}catch{}}throw n}}_request(t,r){typeof t=="string"?(r=r||{},r.url=t):r=t||{},r=Mc(this.defaults,r);const{transitional:n,paramsSerializer:a,headers:i}=r;n!==void 0&&x1.assertOptions(n,{silentJSONParsing:Wo.transitional(Wo.boolean),forcedJSONParsing:Wo.transitional(Wo.boolean),clarifyTimeoutError:Wo.transitional(Wo.boolean)},!1),a!=null&&(Le.isFunction(a)?r.paramsSerializer={serialize:a}:x1.assertOptions(a,{encode:Wo.function,serialize:Wo.function},!0)),r.allowAbsoluteUrls!==void 0||(this.defaults.allowAbsoluteUrls!==void 0?r.allowAbsoluteUrls=this.defaults.allowAbsoluteUrls:r.allowAbsoluteUrls=!0),x1.assertOptions(r,{baseUrl:Wo.spelling("baseURL"),withXsrfToken:Wo.spelling("withXSRFToken")},!0),r.method=(r.method||this.defaults.method||"get").toLowerCase();let o=i&&Le.merge(i.common,i[r.method]);i&&Le.forEach(["delete","get","head","post","put","patch","common"],b=>{delete i[b]}),r.headers=fi.concat(o,i);const s=[];let l=!0;this.interceptors.request.forEach(function(y){typeof y.runWhen=="function"&&y.runWhen(r)===!1||(l=l&&y.synchronous,s.unshift(y.fulfilled,y.rejected))});const u=[];this.interceptors.response.forEach(function(y){u.push(y.fulfilled,y.rejected)});let d,p=0,f;if(!l){const b=[x8.bind(this),void 0];for(b.unshift(...s),b.push(...u),f=b.length,d=Promise.resolve(r);p<f;)d=d.then(b[p++],b[p++]);return d}f=s.length;let m=r;for(;p<f;){const b=s[p++],y=s[p++];try{m=b(m)}catch(S){y.call(this,S);break}}try{d=x8.call(this,m)}catch(b){return Promise.reject(b)}for(p=0,f=u.length;p<f;)d=d.then(u[p++],u[p++]);return d}getUri(t){t=Mc(this.defaults,t);const r=sJ(t.baseURL,t.url,t.allowAbsoluteUrls);return rJ(r,t.params,t.paramsSerializer)}};Le.forEach(["delete","get","head","options"],function(t){Fc.prototype[t]=function(r,n){return this.request(Mc(n||{},{method:t,url:r,data:(n||{}).data}))}});Le.forEach(["post","put","patch"],function(t){function r(n){return function(i,o,s){return this.request(Mc(s||{},{method:t,headers:n?{"Content-Type":"multipart/form-data"}:{},url:i,data:o}))}}Fc.prototype[t]=r(),Fc.prototype[t+"Form"]=r(!0)});let Obe=class hJ{constructor(t){if(typeof t!="function")throw new TypeError("executor must be a function.");let r;this.promise=new Promise(function(i){r=i});const n=this;this.promise.then(a=>{if(!n._listeners)return;let i=n._listeners.length;for(;i-- >0;)n._listeners[i](a);n._listeners=null}),this.promise.then=a=>{let i;const o=new Promise(s=>{n.subscribe(s),i=s}).then(a);return o.cancel=function(){n.unsubscribe(i)},o},t(function(i,o,s){n.reason||(n.reason=new hp(i,o,s),r(n.reason))})}throwIfRequested(){if(this.reason)throw this.reason}subscribe(t){if(this.reason){t(this.reason);return}this._listeners?this._listeners.push(t):this._listeners=[t]}unsubscribe(t){if(!this._listeners)return;const r=this._listeners.indexOf(t);r!==-1&&this._listeners.splice(r,1)}toAbortSignal(){const t=new AbortController,r=n=>{t.abort(n)};return this.subscribe(r),t.signal.unsubscribe=()=>this.unsubscribe(r),t.signal}static source(){let t;return{token:new hJ(function(a){t=a}),cancel:t}}};function Bbe(e){return function(r){return e.apply(null,r)}}function Pbe(e){return Le.isObject(e)&&e.isAxiosError===!0}const H_={Continue:100,SwitchingProtocols:101,Processing:102,EarlyHints:103,Ok:200,Created:201,Accepted:202,NonAuthoritativeInformation:203,NoContent:204,ResetContent:205,PartialContent:206,MultiStatus:207,AlreadyReported:208,ImUsed:226,MultipleChoices:300,MovedPermanently:301,Found:302,SeeOther:303,NotModified:304,UseProxy:305,Unused:306,TemporaryRedirect:307,PermanentRedirect:308,BadRequest:400,Unauthorized:401,PaymentRequired:402,Forbidden:403,NotFound:404,MethodNotAllowed:405,NotAcceptable:406,ProxyAuthenticationRequired:407,RequestTimeout:408,Conflict:409,Gone:410,LengthRequired:411,PreconditionFailed:412,PayloadTooLarge:413,UriTooLong:414,UnsupportedMediaType:415,RangeNotSatisfiable:416,ExpectationFailed:417,ImATeapot:418,MisdirectedRequest:421,UnprocessableEntity:422,Locked:423,FailedDependency:424,TooEarly:425,UpgradeRequired:426,PreconditionRequired:428,TooManyRequests:429,RequestHeaderFieldsTooLarge:431,UnavailableForLegalReasons:451,InternalServerError:500,NotImplemented:501,BadGateway:502,ServiceUnavailable:503,GatewayTimeout:504,HttpVersionNotSupported:505,VariantAlsoNegotiates:506,InsufficientStorage:507,LoopDetected:508,NotExtended:510,NetworkAuthenticationRequired:511};Object.entries(H_).forEach(([e,t])=>{H_[t]=e});function pJ(e){const t=new Fc(e),r=jY(Fc.prototype.request,t);return Le.extend(r,Fc.prototype,t,{allOwnKeys:!0}),Le.extend(r,t,null,{allOwnKeys:!0}),r.create=function(a){return pJ(Mc(e,a))},r}const Mn=pJ(vg);Mn.Axios=Fc;Mn.CanceledError=hp;Mn.CancelToken=Obe;Mn.isCancel=iJ;Mn.VERSION=dJ;Mn.toFormData=Iv;Mn.AxiosError=pr;Mn.Cancel=Mn.CanceledError;Mn.all=function(t){return Promise.all(t)};Mn.spread=Bbe;Mn.isAxiosError=Pbe;Mn.mergeConfig=Mc;Mn.AxiosHeaders=fi;Mn.formToJSON=e=>aJ(Le.isHTMLForm(e)?new FormData(e):e);Mn.getAdapter=cJ.getAdapter;Mn.HttpStatusCode=H_;Mn.default=Mn;const{Axios:kct,AxiosError:Cct,CanceledError:Tct,isCancel:Act,CancelToken:Fct,VERSION:_ct,all:Rct,Cancel:Dct,isAxiosError:Nct,spread:Lct,toFormData:Mct,AxiosHeaders:Ict,HttpStatusCode:Oct,formToJSON:Bct,getAdapter:Pct,mergeConfig:zct}=Mn,ly="",fJ="/webui/",no="ghost",U_="#FFFFFF",zbe="#000000",k8="#000",Hbe="#E2E2E2",$_="#EEEEEE",Ube="#F57F17",$be="#888888",Gbe="#F57F17",jbe="#F57F17",qbe="#F57F17",kb=50,Wbe=500,Vbe="1.0",G_=300,mJ=50,Cb=300,gu=4,uy=20,Kbe=15,C8="*",Ybe={"text/plain":[".txt",".md",".rtf",".odt",".tex",".epub",".html",".htm",".csv",".json",".xml",".yaml",".yml",".log",".conf",".ini",".properties",".sql",".bat",".sh",".c",".cpp",".py",".java",".js",".ts",".swift",".go",".rb",".php",".css",".scss",".less"],"application/pdf":[".pdf"],"application/vnd.openxmlformats-officedocument.wordprocessingml.document":[".docx"],"application/vnd.openxmlformats-officedocument.presentationml.presentation":[".pptx"],"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet":[".xlsx"]},j_={name:"LightRAG",github:"https://github.com/HKUDS/LightRAG"},Jbe="modulepreload",Xbe=function(e){return"/webui/"+e},T8={},Pr=function(t,r,n){let a=Promise.resolve();if(r&&r.length>0){let o=function(u){return Promise.all(u.map(d=>Promise.resolve(d).then(p=>({status:"fulfilled",value:p}),p=>({status:"rejected",reason:p}))))};document.getElementsByTagName("link");const s=document.querySelector("meta[property=csp-nonce]"),l=(s==null?void 0:s.nonce)||(s==null?void 0:s.getAttribute("nonce"));a=o(r.map(u=>{if(u=Xbe(u),u in T8)return;T8[u]=!0;const d=u.endsWith(".css"),p=d?'[rel="stylesheet"]':"";if(document.querySelector(`link[href="${u}"]${p}`))return;const f=document.createElement("link");if(f.rel=d?"stylesheet":Jbe,d||(f.as="script"),f.crossOrigin="",f.href=u,l&&f.setAttribute("nonce",l),document.head.appendChild(f),d)return new Promise((m,b)=>{f.addEventListener("load",m),f.addEventListener("error",()=>b(Error(`Unable to preload CSS for ${u}`)))})}))}function i(o){const s=new Event("vite:preloadError",{cancelable:!0});if(s.payload=o,window.dispatchEvent(s),!s.defaultPrevented)throw o}return a.then(o=>{for(const s of o||[])s.status==="rejected"&&i(s.reason);return t().catch(i)})};function gJ(e,t){let r;try{r=e()}catch{return}return{getItem:a=>{var i;const o=l=>l===null?null:JSON.parse(l,void 0),s=(i=r.getItem(a))!=null?i:null;return s instanceof Promise?s.then(o):o(s)},setItem:(a,i)=>r.setItem(a,JSON.stringify(i,void 0)),removeItem:a=>r.removeItem(a)}}const q_=e=>t=>{try{const r=e(t);return r instanceof Promise?r:{then(n){return q_(n)(r)},catch(n){return this}}}catch(r){return{then(n){return this},catch(n){return q_(n)(r)}}}},Zbe=(e,t)=>(r,n,a)=>{let i={storage:gJ(()=>localStorage),partialize:y=>y,version:0,merge:(y,S)=>({...S,...y}),...t},o=!1;const s=new Set,l=new Set;let u=i.storage;if(!u)return e((...y)=>{console.warn(`[zustand persist middleware] Unable to update item '${i.name}', the given storage is currently unavailable.`),r(...y)},n,a);const d=()=>{const y=i.partialize({...n()});return u.setItem(i.name,{state:y,version:i.version})},p=a.setState;a.setState=(y,S)=>(p(y,S),d());const f=e((...y)=>(r(...y),d()),n,a);a.getInitialState=()=>f;let m;const b=()=>{var y,S;if(!u)return;o=!1,s.forEach(E=>{var k;return E((k=n())!=null?k:f)});const w=((S=i.onRehydrateStorage)==null?void 0:S.call(i,(y=n())!=null?y:f))||void 0;return q_(u.getItem.bind(u))(i.name).then(E=>{if(E)if(typeof E.version=="number"&&E.version!==i.version){if(i.migrate){const k=i.migrate(E.state,E.version);return k instanceof Promise?k.then(_=>[!0,_]):[!0,k]}console.error("State loaded from storage couldn't be migrated since no migrate function was provided")}else return[!1,E.state];return[!1,void 0]}).then(E=>{var k;const[_,R]=E;if(m=i.merge(R,(k=n())!=null?k:f),r(m,!0),_)return d()}).then(()=>{w==null||w(m,void 0),m=n(),o=!0,l.forEach(E=>E(m))}).catch(E=>{w==null||w(void 0,E)})};return a.persist={setOptions:y=>{i={...i,...y},y.storage&&(u=y.storage)},clearStorage:()=>{u==null||u.removeItem(i.name)},getOptions:()=>i,rehydrate:()=>b(),hasHydrated:()=>o,onHydrate:y=>(s.add(y),()=>{s.delete(y)}),onFinishHydration:y=>(l.add(y),()=>{l.delete(y)})},i.skipHydration||b(),m||f},Qbe=Zbe,e1e=Dv()(Qbe(e=>({theme:"system",language:"en",showPropertyPanel:!0,showNodeSearchBar:!0,showLegend:!1,showNodeLabel:!0,enableNodeDrag:!0,showEdgeLabel:!1,enableHideUnselectedEdges:!0,enableEdgeEvents:!1,minEdgeSize:1,maxEdgeSize:1,graphQueryMaxDepth:3,graphMaxNodes:1e3,backendMaxGraphNodes:null,graphLayoutMaxIterations:15,queryLabel:C8,enableHealthCheck:!0,apiKey:null,currentTab:"documents",showFileName:!1,documentsPageSize:10,retrievalHistory:[],userPromptHistory:[],querySettings:{mode:"global",response_type:"Multiple Paragraphs",top_k:40,chunk_top_k:20,max_entity_tokens:6e3,max_relation_tokens:8e3,max_total_tokens:3e4,only_need_context:!1,only_need_prompt:!1,stream:!0,history_turns:0,user_prompt:"",enable_rerank:!0},setTheme:t=>e({theme:t}),setLanguage:t=>{e({language:t}),Pr(async()=>{const{default:r}=await Promise.resolve().then(()=>fut);return{default:r}},[]).then(({default:r})=>{r.language!==t&&r.changeLanguage(t)})},setGraphLayoutMaxIterations:t=>e({graphLayoutMaxIterations:t}),setQueryLabel:t=>e({queryLabel:t}),setGraphQueryMaxDepth:t=>e({graphQueryMaxDepth:t}),setGraphMaxNodes:(t,r=!1)=>{const n=Ve.getState();if(n.graphMaxNodes!==t)if(r){const a=n.queryLabel;e({graphMaxNodes:t,queryLabel:""}),setTimeout(()=>{e({queryLabel:a})},300)}else e({graphMaxNodes:t})},setBackendMaxGraphNodes:t=>e({backendMaxGraphNodes:t}),setMinEdgeSize:t=>e({minEdgeSize:t}),setMaxEdgeSize:t=>e({maxEdgeSize:t}),setEnableHealthCheck:t=>e({enableHealthCheck:t}),setApiKey:t=>e({apiKey:t}),setCurrentTab:t=>e({currentTab:t}),setRetrievalHistory:t=>e({retrievalHistory:t}),updateQuerySettings:t=>{const r={...t};delete r.history_turns,e(n=>({querySettings:{...n.querySettings,...r,history_turns:0}}))},setShowFileName:t=>e({showFileName:t}),setShowLegend:t=>e({showLegend:t}),setDocumentsPageSize:t=>e({documentsPageSize:t}),addUserPromptToHistory:t=>{t.trim()&&e(r=>{const n=[...r.userPromptHistory],a=n.indexOf(t);return a!==-1&&n.splice(a,1),n.unshift(t),n.length>12&&n.splice(12),{userPromptHistory:n}})},setUserPromptHistory:t=>e({userPromptHistory:t}),searchLabelDropdownRefreshTrigger:0,triggerSearchLabelDropdownRefresh:()=>e(t=>({searchLabelDropdownRefreshTrigger:t.searchLabelDropdownRefreshTrigger+1}))}),{name:"settings-storage",storage:gJ(()=>localStorage),version:18,migrate:(e,t)=>(t<2&&(e.showEdgeLabel=!1),t<3&&(e.queryLabel=C8),t<4&&(e.showPropertyPanel=!0,e.showNodeSearchBar=!0,e.showNodeLabel=!0,e.enableHealthCheck=!0,e.apiKey=null),t<5&&(e.currentTab="documents"),t<6&&(e.querySettings={mode:"global",response_type:"Multiple Paragraphs",top_k:10,max_token_for_text_unit:4e3,max_token_for_global_context:4e3,max_token_for_local_context:4e3,only_need_context:!1,only_need_prompt:!1,stream:!0,history_turns:0,hl_keywords:[],ll_keywords:[]},e.retrievalHistory=[]),t<7&&(e.graphQueryMaxDepth=3,e.graphLayoutMaxIterations=15),t<8&&(e.graphMinDegree=0,e.language="en"),t<9&&(e.showFileName=!1),t<10&&(delete e.graphMinDegree,e.graphMaxNodes=1e3),t<11&&(e.minEdgeSize=1,e.maxEdgeSize=1),t<12&&(e.retrievalHistory=[]),t<13&&e.querySettings&&(e.querySettings.user_prompt=""),t<14&&(e.backendMaxGraphNodes=null),t<15&&(e.querySettings={...e.querySettings,mode:"mix",response_type:"Multiple Paragraphs",top_k:40,chunk_top_k:10,max_entity_tokens:1e4,max_relation_tokens:1e4,max_total_tokens:32e3,enable_rerank:!0,history_turns:0}),t<16&&(e.documentsPageSize=10),t<17&&e.querySettings&&(e.querySettings.history_turns=0),t<18&&(e.userPromptHistory=[]),e)})),Ve=DD(e1e);class t1e{constructor(){mt(this,"nodes",[]);mt(this,"edges",[]);mt(this,"nodeIdMap",{});mt(this,"edgeIdMap",{});mt(this,"edgeDynamicIdMap",{});mt(this,"getNode",t=>{const r=this.nodeIdMap[t];if(r!==void 0)return this.nodes[r]});mt(this,"getEdge",(t,r=!0)=>{const n=r?this.edgeDynamicIdMap[t]:this.edgeIdMap[t];if(n!==void 0)return this.edges[n]});mt(this,"buildDynamicMap",()=>{this.edgeDynamicIdMap={};for(let t=0;t<this.edges.length;t++){const r=this.edges[t];this.edgeDynamicIdMap[r.dynamicId]=t}})}}const r1e=Dv()((e,t)=>({selectedNode:null,focusedNode:null,selectedEdge:null,focusedEdge:null,moveToSelectedNode:!1,isFetching:!1,graphIsEmpty:!1,lastSuccessfulQueryLabel:"",graphDataFetchAttempted:!1,labelsFetchAttempted:!1,rawGraph:null,sigmaGraph:null,sigmaInstance:null,typeColorMap:new Map,searchEngine:null,setGraphIsEmpty:r=>e({graphIsEmpty:r}),setLastSuccessfulQueryLabel:r=>e({lastSuccessfulQueryLabel:r}),setIsFetching:r=>e({isFetching:r}),setSelectedNode:(r,n)=>e({selectedNode:r,moveToSelectedNode:n}),setFocusedNode:r=>e({focusedNode:r}),setSelectedEdge:r=>e({selectedEdge:r}),setFocusedEdge:r=>e({focusedEdge:r}),clearSelection:()=>e({selectedNode:null,focusedNode:null,selectedEdge:null,focusedEdge:null}),reset:()=>{e({selectedNode:null,focusedNode:null,selectedEdge:null,focusedEdge:null,rawGraph:null,sigmaGraph:null,searchEngine:null,moveToSelectedNode:!1,graphIsEmpty:!1})},setRawGraph:r=>e({rawGraph:r}),setSigmaGraph:r=>{e({sigmaGraph:r})},setMoveToSelectedNode:r=>e({moveToSelectedNode:r}),setSigmaInstance:r=>e({sigmaInstance:r}),setTypeColorMap:r=>e({typeColorMap:r}),setSearchEngine:r=>e({searchEngine:r}),resetSearchEngine:()=>e({searchEngine:null}),setGraphDataFetchAttempted:r=>e({graphDataFetchAttempted:r}),setLabelsFetchAttempted:r=>e({labelsFetchAttempted:r}),nodeToExpand:null,nodeToPrune:null,triggerNodeExpand:r=>e({nodeToExpand:r}),triggerNodePrune:r=>e({nodeToPrune:r}),graphDataVersion:0,incrementGraphDataVersion:()=>e(r=>({graphDataVersion:r.graphDataVersion+1})),updateNodeAndSelect:async(r,n,a,i)=>{const o=t(),{sigmaGraph:s,rawGraph:l}=o;if(!(!s||!l||!s.hasNode(r)))try{const u=s.getNodeAttributes(r);if(console.log("updateNodeAndSelect",r,n,a,i),r===n&&a==="entity_id"){s.addNode(i,{...u,label:i});const d=[];s.forEachEdge(r,(f,m,b,y)=>{const S=b===r?y:b,w=b===r,E=f,k=l.edgeDynamicIdMap[E],_=s.addEdge(w?i:S,w?S:i,m);k!==void 0&&d.push({originalDynamicId:E,newEdgeId:_,edgeIndex:k}),s.dropEdge(f)}),s.dropNode(r);const p=l.nodeIdMap[r];p!==void 0&&(l.nodes[p].id=i,l.nodes[p].labels=[i],l.nodes[p].properties.entity_id=i,delete l.nodeIdMap[r],l.nodeIdMap[i]=p),d.forEach(({originalDynamicId:f,newEdgeId:m,edgeIndex:b})=>{l.edges[b]&&(l.edges[b].source===r&&(l.edges[b].source=i),l.edges[b].target===r&&(l.edges[b].target=i),l.edges[b].dynamicId=m,delete l.edgeDynamicIdMap[f],l.edgeDynamicIdMap[m]=b)}),e({selectedNode:i,moveToSelectedNode:!0})}else{const d=l.nodeIdMap[String(r)];d!==void 0&&(l.nodes[d].properties[a]=i,a==="entity_id"&&(l.nodes[d].labels=[i],s.setNodeAttribute(String(r),"label",i))),e(p=>({graphDataVersion:p.graphDataVersion+1}))}}catch(u){throw console.error("Error updating node in graph:",u),new Error("Failed to update node in graph")}},updateEdgeAndSelect:async(r,n,a,i,o,s)=>{const l=t(),{sigmaGraph:u,rawGraph:d}=l;if(!(!u||!d))try{const p=d.edgeIdMap[String(r)];p!==void 0&&d.edges[p]&&(d.edges[p].properties[o]=s,n!==void 0&&o==="keywords"&&u.setEdgeAttribute(n,"label",s)),e(f=>({graphDataVersion:f.graphDataVersion+1})),e({selectedEdge:n})}catch(p){throw console.error(`Error updating edge ${a}->${i} in graph:`,p),new Error("Failed to update edge in graph")}}})),pt=DD(r1e);class n1e{constructor(){mt(this,"navigate",null)}setNavigate(t){this.navigate=t}resetAllApplicationState(t=!1){console.log("Resetting all application state...");const r=pt.getState(),n=r.sigmaInstance;r.reset(),r.setGraphDataFetchAttempted(!1),r.setLabelsFetchAttempted(!1),r.setSigmaInstance(null),r.setIsFetching(!1),Xn.getState().clear(),t||Ve.getState().setRetrievalHistory([]),sessionStorage.clear(),n&&(n.getGraph().clear(),n.kill(),pt.getState().setSigmaInstance(null))}navigateToLogin(){if(!this.navigate){console.error("Navigation function not set");return}const t=Oi.getState().username;t&&localStorage.setItem("LIGHTRAG-PREVIOUS-USER",t),this.resetAllApplicationState(!0),Oi.getState().logout(),this.navigate("/login")}navigateToHome(){if(!this.navigate){console.error("Navigation function not set");return}this.navigate("/")}}const Bv=new n1e,bJ="Invalid API Key",yJ="API Key required",Wn=Mn.create({baseURL:ly,headers:{"Content-Type":"application/json"}});Wn.interceptors.request.use(e=>{const t=Ve.getState().apiKey,r=localStorage.getItem("LIGHTRAG-API-TOKEN");return r&&(e.headers.Authorization=`Bearer ${r}`),t&&(e.headers["X-API-Key"]=t),e});Wn.interceptors.response.use(e=>e,e=>{var t,r,n,a;if(e.response){if(((t=e.response)==null?void 0:t.status)===401){if((n=(r=e.config)==null?void 0:r.url)!=null&&n.includes("/login"))throw e;return Bv.navigateToLogin(),Promise.reject(new Error("Authentication required"))}throw new Error(`${e.response.status} ${e.response.statusText}
${JSON.stringify(e.response.data)}
${(a=e.config)==null?void 0:a.url}`)}throw e});const vJ=async(e,t,r)=>(await Wn.get(`/graphs?label=${encodeURIComponent(e)}&max_depth=${t}&max_nodes=${r}`)).data,A8=async(e=G_)=>(await Wn.get(`/graph/label/popular?limit=${e}`)).data,a1e=async(e,t=mJ)=>(await Wn.get(`/graph/label/search?q=${encodeURIComponent(e)}&limit=${t}`)).data,i1e=async()=>{try{return(await Wn.get("/health")).data}catch(e){return{status:"error",message:Da(e)}}},o1e=async()=>(await Wn.post("/documents/scan")).data,s1e=async e=>(await Wn.post("/query",e)).data,l1e=async(e,t,r)=>{const n=Ve.getState().apiKey,a=localStorage.getItem("LIGHTRAG-API-TOKEN"),i={"Content-Type":"application/json",Accept:"application/x-ndjson"};a&&(i.Authorization=`Bearer ${a}`),n&&(i["X-API-Key"]=n);try{const o=await fetch(`${ly}/query/stream`,{method:"POST",headers:i,body:JSON.stringify(e)});if(!o.ok){if(o.status===401)throw Bv.navigateToLogin(),new Error("Authentication required");let d="Unknown error";try{d=await o.text()}catch{}const p=`${ly}/query/stream`;throw new Error(`${o.status} ${o.statusText}
${JSON.stringify({error:d})}
${p}`)}if(!o.body)throw new Error("Response body is null");const s=o.body.getReader(),l=new TextDecoder;let u="";for(;;){const{done:d,value:p}=await s.read();if(d)break;u+=l.decode(p,{stream:!0});const f=u.split(`
`);u=f.pop()||"";for(const m of f)if(m.trim())try{const b=JSON.parse(m);b.response?t(b.response):b.error&&r&&r(b.error)}catch(b){console.error("Error parsing stream chunk:",m,b),r&&r(`Error parsing server response: ${m}`)}}if(u.trim())try{const d=JSON.parse(u);d.response?t(d.response):d.error&&r&&r(d.error)}catch(d){console.error("Error parsing final chunk:",u,d),r&&r(`Error parsing final server response: ${u}`)}}catch(o){const s=Da(o);if(s==="Authentication required"){console.error("Authentication required for stream request"),r&&r("Authentication required");return}const l=s.match(/^(\d{3})\s/);if(l){const u=parseInt(l[1],10);let d=s;switch(u){case 403:d="You do not have permission to access this resource (403 Forbidden)",console.error("Permission denied for stream request:",s);break;case 404:d="The requested resource does not exist (404 Not Found)",console.error("Resource not found for stream request:",s);break;case 429:d="Too many requests, please try again later (429 Too Many Requests)",console.error("Rate limited for stream request:",s);break;case 500:case 502:case 503:case 504:d=`Server error, please try again later (${u})`,console.error("Server error for stream request:",s);break;default:console.error("Stream request failed with status code:",u,s)}r&&r(d);return}if(s.includes("NetworkError")||s.includes("Failed to fetch")||s.includes("Network request failed")){console.error("Network error for stream request:",s),r&&r("Network connection error, please check your internet connection");return}if(s.includes("Error parsing")||s.includes("SyntaxError")){console.error("JSON parsing error in stream:",s),r&&r("Error processing response data");return}console.error("Unhandled stream error:",s),r?r(s):console.error("No error handler provided for stream error:",s)}},u1e=async(e,t)=>{const r=new FormData;return r.append("file",e),(await Wn.post("/documents/upload",r,{headers:{"Content-Type":"multipart/form-data"},onUploadProgress:t!==void 0?a=>{const i=Math.round(a.loaded*100/a.total);t(i)}:void 0})).data},c1e=async()=>(await Wn.delete("/documents")).data,d1e=async()=>(await Wn.post("/documents/clear_cache",{})).data,h1e=async(e,t=!1,r=!1)=>(await Wn.delete("/documents/delete_document",{data:{doc_ids:e,delete_file:t,delete_llm_cache:r}})).data,SJ=async()=>{try{const e=await Wn.get("/auth-status",{timeout:5e3,headers:{Accept:"application/json"}});if((e.headers["content-type"]||"").includes("text/html"))return console.warn("Received HTML response instead of JSON for auth-status endpoint"),{auth_configured:!0,auth_mode:"enabled"};if(e.data&&typeof e.data=="object"&&"auth_configured"in e.data&&typeof e.data.auth_configured=="boolean"){if(e.data.auth_configured)return e.data;if(e.data.access_token&&typeof e.data.access_token=="string")return e.data;console.warn("Auth not configured but no valid access token provided")}return console.warn("Received invalid auth status response:",e.data),{auth_configured:!0,auth_mode:"enabled"}}catch(e){return console.error("Failed to get auth status:",Da(e)),{auth_configured:!0,auth_mode:"enabled"}}},p1e=async()=>(await Wn.get("/documents/pipeline_status")).data,f1e=async()=>(await Wn.post("/documents/cancel_pipeline")).data,m1e=async(e,t)=>{const r=new FormData;return r.append("username",e),r.append("password",t),(await Wn.post("/login",r,{headers:{"Content-Type":"multipart/form-data"}})).data},g1e=async(e,t,r=!1,n=!1)=>(await Wn.post("/graph/entity/edit",{entity_name:e,updated_data:t,allow_rename:r,allow_merge:n})).data,b1e=async(e,t,r)=>(await Wn.post("/graph/relation/edit",{source_id:e,target_id:t,updated_data:r})).data,y1e=async e=>{try{return(await Wn.get(`/graph/entity/exists?name=${encodeURIComponent(e)}`)).data.exists}catch(t){return console.error("Error checking entity name:",t),!1}},Gx=async e=>(await Wn.post("/documents/paginated",e)).data,v1e=Dv()((e,t)=>({health:!0,message:null,messageTitle:null,lastCheckTime:Date.now(),status:null,pipelineBusy:!1,healthCheckIntervalId:null,healthCheckFunction:null,healthCheckIntervalValue:Kbe*1e3,check:async()=>{var n;const r=await i1e();if(r.status==="healthy"){if((r.core_version||r.api_version)&&Oi.getState().setVersion(r.core_version||null,r.api_version||null),("webui_title"in r||"webui_description"in r)&&Oi.getState().setCustomTitle("webui_title"in r?r.webui_title??null:null,"webui_description"in r?r.webui_description??null:null),(n=r.configuration)!=null&&n.max_graph_nodes){const a=parseInt(r.configuration.max_graph_nodes,10);!isNaN(a)&&a>0&&Ve.getState().backendMaxGraphNodes!==a&&(Ve.getState().setBackendMaxGraphNodes(a),Ve.getState().graphMaxNodes>a&&Ve.getState().setGraphMaxNodes(a,!0))}return e({health:!0,message:null,messageTitle:null,lastCheckTime:Date.now(),status:r,pipelineBusy:r.pipeline_busy}),!0}return e({health:!1,message:r.message,messageTitle:"Backend Health Check Error!",lastCheckTime:Date.now(),status:null}),!1},clear:()=>{e({health:!0,message:null,messageTitle:null})},setErrorMessage:(r,n)=>{e({health:!1,message:r,messageTitle:n})},setPipelineBusy:r=>{e({pipelineBusy:r})},setHealthCheckFunction:r=>{e({healthCheckFunction:r})},resetHealthCheckTimer:()=>{const{healthCheckIntervalId:r,healthCheckFunction:n,healthCheckIntervalValue:a}=t();if(r&&clearInterval(r),n){n();const i=setInterval(n,a);e({healthCheckIntervalId:i})}},resetHealthCheckTimerDelayed:r=>{setTimeout(()=>{t().resetHealthCheckTimer()},r)},clearHealthCheckTimer:()=>{const{healthCheckIntervalId:r}=t();r&&(clearInterval(r),e({healthCheckIntervalId:null}))}})),Xn=DD(v1e),EJ=e=>{try{const t=e.split(".");return t.length!==3?{}:JSON.parse(atob(t[1]))}catch(t){return console.error("Error parsing token payload:",t),{}}},xJ=e=>EJ(e).sub||null,S1e=e=>EJ(e).role==="guest",E1e=()=>{const e=localStorage.getItem("LIGHTRAG-API-TOKEN"),t=localStorage.getItem("LIGHTRAG-CORE-VERSION"),r=localStorage.getItem("LIGHTRAG-API-VERSION"),n=localStorage.getItem("LIGHTRAG-WEBUI-TITLE"),a=localStorage.getItem("LIGHTRAG-WEBUI-DESCRIPTION"),i=e?xJ(e):null;return e?{isAuthenticated:!0,isGuestMode:S1e(e),coreVersion:t,apiVersion:r,username:i,webuiTitle:n,webuiDescription:a}:{isAuthenticated:!1,isGuestMode:!1,coreVersion:t,apiVersion:r,username:null,webuiTitle:n,webuiDescription:a}},Oi=Dv(e=>{const t=E1e();return{isAuthenticated:t.isAuthenticated,isGuestMode:t.isGuestMode,coreVersion:t.coreVersion,apiVersion:t.apiVersion,username:t.username,webuiTitle:t.webuiTitle,webuiDescription:t.webuiDescription,login:(r,n=!1,a=null,i=null,o=null,s=null)=>{localStorage.setItem("LIGHTRAG-API-TOKEN",r),a&&localStorage.setItem("LIGHTRAG-CORE-VERSION",a),i&&localStorage.setItem("LIGHTRAG-API-VERSION",i),o?localStorage.setItem("LIGHTRAG-WEBUI-TITLE",o):localStorage.removeItem("LIGHTRAG-WEBUI-TITLE"),s?localStorage.setItem("LIGHTRAG-WEBUI-DESCRIPTION",s):localStorage.removeItem("LIGHTRAG-WEBUI-DESCRIPTION");const l=xJ(r);e({isAuthenticated:!0,isGuestMode:n,username:l,coreVersion:a,apiVersion:i,webuiTitle:o,webuiDescription:s})},logout:()=>{localStorage.removeItem("LIGHTRAG-API-TOKEN");const r=localStorage.getItem("LIGHTRAG-CORE-VERSION"),n=localStorage.getItem("LIGHTRAG-API-VERSION"),a=localStorage.getItem("LIGHTRAG-WEBUI-TITLE"),i=localStorage.getItem("LIGHTRAG-WEBUI-DESCRIPTION");e({isAuthenticated:!1,isGuestMode:!1,username:null,coreVersion:r,apiVersion:n,webuiTitle:a,webuiDescription:i})},setVersion:(r,n)=>{r&&localStorage.setItem("LIGHTRAG-CORE-VERSION",r),n&&localStorage.setItem("LIGHTRAG-API-VERSION",n),e({coreVersion:r,apiVersion:n})},setCustomTitle:(r,n)=>{r?localStorage.setItem("LIGHTRAG-WEBUI-TITLE",r):localStorage.removeItem("LIGHTRAG-WEBUI-TITLE"),n?localStorage.setItem("LIGHTRAG-WEBUI-DESCRIPTION",n):localStorage.removeItem("LIGHTRAG-WEBUI-DESCRIPTION"),e({webuiTitle:r,webuiDescription:n})}}});var x1e=e=>{switch(e){case"success":return C1e;case"info":return A1e;case"warning":return T1e;case"error":return F1e;default:return null}},w1e=Array(12).fill(0),k1e=({visible:e,className:t})=>$e.createElement("div",{className:["sonner-loading-wrapper",t].filter(Boolean).join(" "),"data-visible":e},$e.createElement("div",{className:"sonner-spinner"},w1e.map((r,n)=>$e.createElement("div",{className:"sonner-loading-bar",key:`spinner-bar-${n}`})))),C1e=$e.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 20 20",fill:"currentColor",height:"20",width:"20"},$e.createElement("path",{fillRule:"evenodd",d:"M10 18a8 8 0 100-16 8 8 0 000 16zm3.857-9.809a.75.75 0 00-1.214-.882l-3.483 4.79-1.88-1.88a.75.75 0 10-1.06 1.061l2.5 2.5a.75.75 0 001.137-.089l4-5.5z",clipRule:"evenodd"})),T1e=$e.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 24 24",fill:"currentColor",height:"20",width:"20"},$e.createElement("path",{fillRule:"evenodd",d:"M9.401 3.003c1.155-2 4.043-2 5.197 0l7.355 12.748c1.154 2-.29 4.5-2.599 4.5H4.645c-2.309 0-3.752-2.5-2.598-4.5L9.4 3.003zM12 8.25a.75.75 0 01.75.75v3.75a.75.75 0 01-1.5 0V9a.75.75 0 01.75-.75zm0 8.25a.75.75 0 100-1.5.75.75 0 000 1.5z",clipRule:"evenodd"})),A1e=$e.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 20 20",fill:"currentColor",height:"20",width:"20"},$e.createElement("path",{fillRule:"evenodd",d:"M18 10a8 8 0 11-16 0 8 8 0 0116 0zm-7-4a1 1 0 11-2 0 1 1 0 012 0zM9 9a.75.75 0 000 1.5h.253a.25.25 0 01.244.304l-.459 2.066A1.75 1.75 0 0010.747 15H11a.75.75 0 000-1.5h-.253a.25.25 0 01-.244-.304l.459-2.066A1.75 1.75 0 009.253 9H9z",clipRule:"evenodd"})),F1e=$e.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 20 20",fill:"currentColor",height:"20",width:"20"},$e.createElement("path",{fillRule:"evenodd",d:"M18 10a8 8 0 11-16 0 8 8 0 0116 0zm-8-5a.75.75 0 01.75.75v4.5a.75.75 0 01-1.5 0v-4.5A.75.75 0 0110 5zm0 10a1 1 0 100-2 1 1 0 000 2z",clipRule:"evenodd"})),_1e=$e.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",width:"12",height:"12",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"1.5",strokeLinecap:"round",strokeLinejoin:"round"},$e.createElement("line",{x1:"18",y1:"6",x2:"6",y2:"18"}),$e.createElement("line",{x1:"6",y1:"6",x2:"18",y2:"18"})),R1e=()=>{let[e,t]=$e.useState(document.hidden);return $e.useEffect(()=>{let r=()=>{t(document.hidden)};return document.addEventListener("visibilitychange",r),()=>window.removeEventListener("visibilitychange",r)},[]),e},W_=1,D1e=class{constructor(){this.subscribe=e=>(this.subscribers.push(e),()=>{let t=this.subscribers.indexOf(e);this.subscribers.splice(t,1)}),this.publish=e=>{this.subscribers.forEach(t=>t(e))},this.addToast=e=>{this.publish(e),this.toasts=[...this.toasts,e]},this.create=e=>{var t;let{message:r,...n}=e,a=typeof(e==null?void 0:e.id)=="number"||((t=e.id)==null?void 0:t.length)>0?e.id:W_++,i=this.toasts.find(s=>s.id===a),o=e.dismissible===void 0?!0:e.dismissible;return this.dismissedToasts.has(a)&&this.dismissedToasts.delete(a),i?this.toasts=this.toasts.map(s=>s.id===a?(this.publish({...s,...e,id:a,title:r}),{...s,...e,id:a,dismissible:o,title:r}):s):this.addToast({title:r,...n,dismissible:o,id:a}),a},this.dismiss=e=>(this.dismissedToasts.add(e),e||this.toasts.forEach(t=>{this.subscribers.forEach(r=>r({id:t.id,dismiss:!0}))}),this.subscribers.forEach(t=>t({id:e,dismiss:!0})),e),this.message=(e,t)=>this.create({...t,message:e}),this.error=(e,t)=>this.create({...t,message:e,type:"error"}),this.success=(e,t)=>this.create({...t,type:"success",message:e}),this.info=(e,t)=>this.create({...t,type:"info",message:e}),this.warning=(e,t)=>this.create({...t,type:"warning",message:e}),this.loading=(e,t)=>this.create({...t,type:"loading",message:e}),this.promise=(e,t)=>{if(!t)return;let r;t.loading!==void 0&&(r=this.create({...t,promise:e,type:"loading",message:t.loading,description:typeof t.description!="function"?t.description:void 0}));let n=e instanceof Promise?e:e(),a=r!==void 0,i,o=n.then(async l=>{if(i=["resolve",l],$e.isValidElement(l))a=!1,this.create({id:r,type:"default",message:l});else if(L1e(l)&&!l.ok){a=!1;let u=typeof t.error=="function"?await t.error(`HTTP error! status: ${l.status}`):t.error,d=typeof t.description=="function"?await t.description(`HTTP error! status: ${l.status}`):t.description;this.create({id:r,type:"error",message:u,description:d})}else if(t.success!==void 0){a=!1;let u=typeof t.success=="function"?await t.success(l):t.success,d=typeof t.description=="function"?await t.description(l):t.description;this.create({id:r,type:"success",message:u,description:d})}}).catch(async l=>{if(i=["reject",l],t.error!==void 0){a=!1;let u=typeof t.error=="function"?await t.error(l):t.error,d=typeof t.description=="function"?await t.description(l):t.description;this.create({id:r,type:"error",message:u,description:d})}}).finally(()=>{var l;a&&(this.dismiss(r),r=void 0),(l=t.finally)==null||l.call(t)}),s=()=>new Promise((l,u)=>o.then(()=>i[0]==="reject"?u(i[1]):l(i[1])).catch(u));return typeof r!="string"&&typeof r!="number"?{unwrap:s}:Object.assign(r,{unwrap:s})},this.custom=(e,t)=>{let r=(t==null?void 0:t.id)||W_++;return this.create({jsx:e(r),id:r,...t}),r},this.getActiveToasts=()=>this.toasts.filter(e=>!this.dismissedToasts.has(e.id)),this.subscribers=[],this.toasts=[],this.dismissedToasts=new Set}},li=new D1e,N1e=(e,t)=>{let r=(t==null?void 0:t.id)||W_++;return li.addToast({title:e,...t,id:r}),r},L1e=e=>e&&typeof e=="object"&&"ok"in e&&typeof e.ok=="boolean"&&"status"in e&&typeof e.status=="number",M1e=N1e,I1e=()=>li.toasts,O1e=()=>li.getActiveToasts(),er=Object.assign(M1e,{success:li.success,info:li.info,warning:li.warning,error:li.error,custom:li.custom,message:li.message,promise:li.promise,dismiss:li.dismiss,loading:li.loading},{getHistory:I1e,getToasts:O1e});function B1e(e,{insertAt:t}={}){if(typeof document>"u")return;let r=document.head||document.getElementsByTagName("head")[0],n=document.createElement("style");n.type="text/css",t==="top"&&r.firstChild?r.insertBefore(n,r.firstChild):r.appendChild(n),n.styleSheet?n.styleSheet.cssText=e:n.appendChild(document.createTextNode(e))}B1e(`:where(html[dir="ltr"]),:where([data-sonner-toaster][dir="ltr"]){--toast-icon-margin-start: -3px;--toast-icon-margin-end: 4px;--toast-svg-margin-start: -1px;--toast-svg-margin-end: 0px;--toast-button-margin-start: auto;--toast-button-margin-end: 0;--toast-close-button-start: 0;--toast-close-button-end: unset;--toast-close-button-transform: translate(-35%, -35%)}:where(html[dir="rtl"]),:where([data-sonner-toaster][dir="rtl"]){--toast-icon-margin-start: 4px;--toast-icon-margin-end: -3px;--toast-svg-margin-start: 0px;--toast-svg-margin-end: -1px;--toast-button-margin-start: 0;--toast-button-margin-end: auto;--toast-close-button-start: unset;--toast-close-button-end: 0;--toast-close-button-transform: translate(35%, -35%)}:where([data-sonner-toaster]){position:fixed;width:var(--width);font-family:ui-sans-serif,system-ui,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,Noto Sans,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji;--gray1: hsl(0, 0%, 99%);--gray2: hsl(0, 0%, 97.3%);--gray3: hsl(0, 0%, 95.1%);--gray4: hsl(0, 0%, 93%);--gray5: hsl(0, 0%, 90.9%);--gray6: hsl(0, 0%, 88.7%);--gray7: hsl(0, 0%, 85.8%);--gray8: hsl(0, 0%, 78%);--gray9: hsl(0, 0%, 56.1%);--gray10: hsl(0, 0%, 52.3%);--gray11: hsl(0, 0%, 43.5%);--gray12: hsl(0, 0%, 9%);--border-radius: 8px;box-sizing:border-box;padding:0;margin:0;list-style:none;outline:none;z-index:999999999;transition:transform .4s ease}:where([data-sonner-toaster][data-lifted="true"]){transform:translateY(-10px)}@media (hover: none) and (pointer: coarse){:where([data-sonner-toaster][data-lifted="true"]){transform:none}}:where([data-sonner-toaster][data-x-position="right"]){right:var(--offset-right)}:where([data-sonner-toaster][data-x-position="left"]){left:var(--offset-left)}:where([data-sonner-toaster][data-x-position="center"]){left:50%;transform:translate(-50%)}:where([data-sonner-toaster][data-y-position="top"]){top:var(--offset-top)}:where([data-sonner-toaster][data-y-position="bottom"]){bottom:var(--offset-bottom)}:where([data-sonner-toast]){--y: translateY(100%);--lift-amount: calc(var(--lift) * var(--gap));z-index:var(--z-index);position:absolute;opacity:0;transform:var(--y);filter:blur(0);touch-action:none;transition:transform .4s,opacity .4s,height .4s,box-shadow .2s;box-sizing:border-box;outline:none;overflow-wrap:anywhere}:where([data-sonner-toast][data-styled="true"]){padding:16px;background:var(--normal-bg);border:1px solid var(--normal-border);color:var(--normal-text);border-radius:var(--border-radius);box-shadow:0 4px 12px #0000001a;width:var(--width);font-size:13px;display:flex;align-items:center;gap:6px}:where([data-sonner-toast]:focus-visible){box-shadow:0 4px 12px #0000001a,0 0 0 2px #0003}:where([data-sonner-toast][data-y-position="top"]){top:0;--y: translateY(-100%);--lift: 1;--lift-amount: calc(1 * var(--gap))}:where([data-sonner-toast][data-y-position="bottom"]){bottom:0;--y: translateY(100%);--lift: -1;--lift-amount: calc(var(--lift) * var(--gap))}:where([data-sonner-toast]) :where([data-description]){font-weight:400;line-height:1.4;color:inherit}:where([data-sonner-toast]) :where([data-title]){font-weight:500;line-height:1.5;color:inherit}:where([data-sonner-toast]) :where([data-icon]){display:flex;height:16px;width:16px;position:relative;justify-content:flex-start;align-items:center;flex-shrink:0;margin-left:var(--toast-icon-margin-start);margin-right:var(--toast-icon-margin-end)}:where([data-sonner-toast][data-promise="true"]) :where([data-icon])>svg{opacity:0;transform:scale(.8);transform-origin:center;animation:sonner-fade-in .3s ease forwards}:where([data-sonner-toast]) :where([data-icon])>*{flex-shrink:0}:where([data-sonner-toast]) :where([data-icon]) svg{margin-left:var(--toast-svg-margin-start);margin-right:var(--toast-svg-margin-end)}:where([data-sonner-toast]) :where([data-content]){display:flex;flex-direction:column;gap:2px}[data-sonner-toast][data-styled=true] [data-button]{border-radius:4px;padding-left:8px;padding-right:8px;height:24px;font-size:12px;color:var(--normal-bg);background:var(--normal-text);margin-left:var(--toast-button-margin-start);margin-right:var(--toast-button-margin-end);border:none;cursor:pointer;outline:none;display:flex;align-items:center;flex-shrink:0;transition:opacity .4s,box-shadow .2s}:where([data-sonner-toast]) :where([data-button]):focus-visible{box-shadow:0 0 0 2px #0006}:where([data-sonner-toast]) :where([data-button]):first-of-type{margin-left:var(--toast-button-margin-start);margin-right:var(--toast-button-margin-end)}:where([data-sonner-toast]) :where([data-cancel]){color:var(--normal-text);background:rgba(0,0,0,.08)}:where([data-sonner-toast][data-theme="dark"]) :where([data-cancel]){background:rgba(255,255,255,.3)}:where([data-sonner-toast]) :where([data-close-button]){position:absolute;left:var(--toast-close-button-start);right:var(--toast-close-button-end);top:0;height:20px;width:20px;display:flex;justify-content:center;align-items:center;padding:0;color:var(--gray12);border:1px solid var(--gray4);transform:var(--toast-close-button-transform);border-radius:50%;cursor:pointer;z-index:1;transition:opacity .1s,background .2s,border-color .2s}[data-sonner-toast] [data-close-button]{background:var(--gray1)}:where([data-sonner-toast]) :where([data-close-button]):focus-visible{box-shadow:0 4px 12px #0000001a,0 0 0 2px #0003}:where([data-sonner-toast]) :where([data-disabled="true"]){cursor:not-allowed}:where([data-sonner-toast]):hover :where([data-close-button]):hover{background:var(--gray2);border-color:var(--gray5)}:where([data-sonner-toast][data-swiping="true"]):before{content:"";position:absolute;left:-50%;right:-50%;height:100%;z-index:-1}:where([data-sonner-toast][data-y-position="top"][data-swiping="true"]):before{bottom:50%;transform:scaleY(3) translateY(50%)}:where([data-sonner-toast][data-y-position="bottom"][data-swiping="true"]):before{top:50%;transform:scaleY(3) translateY(-50%)}:where([data-sonner-toast][data-swiping="false"][data-removed="true"]):before{content:"";position:absolute;inset:0;transform:scaleY(2)}:where([data-sonner-toast]):after{content:"";position:absolute;left:0;height:calc(var(--gap) + 1px);bottom:100%;width:100%}:where([data-sonner-toast][data-mounted="true"]){--y: translateY(0);opacity:1}:where([data-sonner-toast][data-expanded="false"][data-front="false"]){--scale: var(--toasts-before) * .05 + 1;--y: translateY(calc(var(--lift-amount) * var(--toasts-before))) scale(calc(-1 * var(--scale)));height:var(--front-toast-height)}:where([data-sonner-toast])>*{transition:opacity .4s}:where([data-sonner-toast][data-expanded="false"][data-front="false"][data-styled="true"])>*{opacity:0}:where([data-sonner-toast][data-visible="false"]){opacity:0;pointer-events:none}:where([data-sonner-toast][data-mounted="true"][data-expanded="true"]){--y: translateY(calc(var(--lift) * var(--offset)));height:var(--initial-height)}:where([data-sonner-toast][data-removed="true"][data-front="true"][data-swipe-out="false"]){--y: translateY(calc(var(--lift) * -100%));opacity:0}:where([data-sonner-toast][data-removed="true"][data-front="false"][data-swipe-out="false"][data-expanded="true"]){--y: translateY(calc(var(--lift) * var(--offset) + var(--lift) * -100%));opacity:0}:where([data-sonner-toast][data-removed="true"][data-front="false"][data-swipe-out="false"][data-expanded="false"]){--y: translateY(40%);opacity:0;transition:transform .5s,opacity .2s}:where([data-sonner-toast][data-removed="true"][data-front="false"]):before{height:calc(var(--initial-height) + 20%)}[data-sonner-toast][data-swiping=true]{transform:var(--y) translateY(var(--swipe-amount-y, 0px)) translate(var(--swipe-amount-x, 0px));transition:none}[data-sonner-toast][data-swiped=true]{user-select:none}[data-sonner-toast][data-swipe-out=true][data-y-position=bottom],[data-sonner-toast][data-swipe-out=true][data-y-position=top]{animation-duration:.2s;animation-timing-function:ease-out;animation-fill-mode:forwards}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=left]{animation-name:swipe-out-left}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=right]{animation-name:swipe-out-right}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=up]{animation-name:swipe-out-up}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=down]{animation-name:swipe-out-down}@keyframes swipe-out-left{0%{transform:var(--y) translate(var(--swipe-amount-x));opacity:1}to{transform:var(--y) translate(calc(var(--swipe-amount-x) - 100%));opacity:0}}@keyframes swipe-out-right{0%{transform:var(--y) translate(var(--swipe-amount-x));opacity:1}to{transform:var(--y) translate(calc(var(--swipe-amount-x) + 100%));opacity:0}}@keyframes swipe-out-up{0%{transform:var(--y) translateY(var(--swipe-amount-y));opacity:1}to{transform:var(--y) translateY(calc(var(--swipe-amount-y) - 100%));opacity:0}}@keyframes swipe-out-down{0%{transform:var(--y) translateY(var(--swipe-amount-y));opacity:1}to{transform:var(--y) translateY(calc(var(--swipe-amount-y) + 100%));opacity:0}}@media (max-width: 600px){[data-sonner-toaster]{position:fixed;right:var(--mobile-offset-right);left:var(--mobile-offset-left);width:100%}[data-sonner-toaster][dir=rtl]{left:calc(var(--mobile-offset-left) * -1)}[data-sonner-toaster] [data-sonner-toast]{left:0;right:0;width:calc(100% - var(--mobile-offset-left) * 2)}[data-sonner-toaster][data-x-position=left]{left:var(--mobile-offset-left)}[data-sonner-toaster][data-y-position=bottom]{bottom:var(--mobile-offset-bottom)}[data-sonner-toaster][data-y-position=top]{top:var(--mobile-offset-top)}[data-sonner-toaster][data-x-position=center]{left:var(--mobile-offset-left);right:var(--mobile-offset-right);transform:none}}[data-sonner-toaster][data-theme=light]{--normal-bg: #fff;--normal-border: var(--gray4);--normal-text: var(--gray12);--success-bg: hsl(143, 85%, 96%);--success-border: hsl(145, 92%, 91%);--success-text: hsl(140, 100%, 27%);--info-bg: hsl(208, 100%, 97%);--info-border: hsl(221, 91%, 91%);--info-text: hsl(210, 92%, 45%);--warning-bg: hsl(49, 100%, 97%);--warning-border: hsl(49, 91%, 91%);--warning-text: hsl(31, 92%, 45%);--error-bg: hsl(359, 100%, 97%);--error-border: hsl(359, 100%, 94%);--error-text: hsl(360, 100%, 45%)}[data-sonner-toaster][data-theme=light] [data-sonner-toast][data-invert=true]{--normal-bg: #000;--normal-border: hsl(0, 0%, 20%);--normal-text: var(--gray1)}[data-sonner-toaster][data-theme=dark] [data-sonner-toast][data-invert=true]{--normal-bg: #fff;--normal-border: var(--gray3);--normal-text: var(--gray12)}[data-sonner-toaster][data-theme=dark]{--normal-bg: #000;--normal-bg-hover: hsl(0, 0%, 12%);--normal-border: hsl(0, 0%, 20%);--normal-border-hover: hsl(0, 0%, 25%);--normal-text: var(--gray1);--success-bg: hsl(150, 100%, 6%);--success-border: hsl(147, 100%, 12%);--success-text: hsl(150, 86%, 65%);--info-bg: hsl(215, 100%, 6%);--info-border: hsl(223, 100%, 12%);--info-text: hsl(216, 87%, 65%);--warning-bg: hsl(64, 100%, 6%);--warning-border: hsl(60, 100%, 12%);--warning-text: hsl(46, 87%, 65%);--error-bg: hsl(358, 76%, 10%);--error-border: hsl(357, 89%, 16%);--error-text: hsl(358, 100%, 81%)}[data-sonner-toaster][data-theme=dark] [data-sonner-toast] [data-close-button]{background:var(--normal-bg);border-color:var(--normal-border);color:var(--normal-text)}[data-sonner-toaster][data-theme=dark] [data-sonner-toast] [data-close-button]:hover{background:var(--normal-bg-hover);border-color:var(--normal-border-hover)}[data-rich-colors=true][data-sonner-toast][data-type=success],[data-rich-colors=true][data-sonner-toast][data-type=success] [data-close-button]{background:var(--success-bg);border-color:var(--success-border);color:var(--success-text)}[data-rich-colors=true][data-sonner-toast][data-type=info],[data-rich-colors=true][data-sonner-toast][data-type=info] [data-close-button]{background:var(--info-bg);border-color:var(--info-border);color:var(--info-text)}[data-rich-colors=true][data-sonner-toast][data-type=warning],[data-rich-colors=true][data-sonner-toast][data-type=warning] [data-close-button]{background:var(--warning-bg);border-color:var(--warning-border);color:var(--warning-text)}[data-rich-colors=true][data-sonner-toast][data-type=error],[data-rich-colors=true][data-sonner-toast][data-type=error] [data-close-button]{background:var(--error-bg);border-color:var(--error-border);color:var(--error-text)}.sonner-loading-wrapper{--size: 16px;height:var(--size);width:var(--size);position:absolute;inset:0;z-index:10}.sonner-loading-wrapper[data-visible=false]{transform-origin:center;animation:sonner-fade-out .2s ease forwards}.sonner-spinner{position:relative;top:50%;left:50%;height:var(--size);width:var(--size)}.sonner-loading-bar{animation:sonner-spin 1.2s linear infinite;background:var(--gray11);border-radius:6px;height:8%;left:-10%;position:absolute;top:-3.9%;width:24%}.sonner-loading-bar:nth-child(1){animation-delay:-1.2s;transform:rotate(.0001deg) translate(146%)}.sonner-loading-bar:nth-child(2){animation-delay:-1.1s;transform:rotate(30deg) translate(146%)}.sonner-loading-bar:nth-child(3){animation-delay:-1s;transform:rotate(60deg) translate(146%)}.sonner-loading-bar:nth-child(4){animation-delay:-.9s;transform:rotate(90deg) translate(146%)}.sonner-loading-bar:nth-child(5){animation-delay:-.8s;transform:rotate(120deg) translate(146%)}.sonner-loading-bar:nth-child(6){animation-delay:-.7s;transform:rotate(150deg) translate(146%)}.sonner-loading-bar:nth-child(7){animation-delay:-.6s;transform:rotate(180deg) translate(146%)}.sonner-loading-bar:nth-child(8){animation-delay:-.5s;transform:rotate(210deg) translate(146%)}.sonner-loading-bar:nth-child(9){animation-delay:-.4s;transform:rotate(240deg) translate(146%)}.sonner-loading-bar:nth-child(10){animation-delay:-.3s;transform:rotate(270deg) translate(146%)}.sonner-loading-bar:nth-child(11){animation-delay:-.2s;transform:rotate(300deg) translate(146%)}.sonner-loading-bar:nth-child(12){animation-delay:-.1s;transform:rotate(330deg) translate(146%)}@keyframes sonner-fade-in{0%{opacity:0;transform:scale(.8)}to{opacity:1;transform:scale(1)}}@keyframes sonner-fade-out{0%{opacity:1;transform:scale(1)}to{opacity:0;transform:scale(.8)}}@keyframes sonner-spin{0%{opacity:1}to{opacity:.15}}@media (prefers-reduced-motion){[data-sonner-toast],[data-sonner-toast]>*,.sonner-loading-bar{transition:none!important;animation:none!important}}.sonner-loader{position:absolute;top:50%;left:50%;transform:translate(-50%,-50%);transform-origin:center;transition:opacity .2s,transform .2s}.sonner-loader[data-visible=false]{opacity:0;transform:scale(.8) translate(-50%,-50%)}
`);function Tb(e){return e.label!==void 0}var P1e=3,z1e="32px",H1e="16px",F8=4e3,U1e=356,$1e=14,G1e=20,j1e=200;function vo(...e){return e.filter(Boolean).join(" ")}function q1e(e){let[t,r]=e.split("-"),n=[];return t&&n.push(t),r&&n.push(r),n}var W1e=e=>{var t,r,n,a,i,o,s,l,u,d,p;let{invert:f,toast:m,unstyled:b,interacting:y,setHeights:S,visibleToasts:w,heights:E,index:k,toasts:_,expanded:R,removeToast:C,defaultRichColors:F,closeButton:L,style:I,cancelButtonStyle:M,actionButtonStyle:j,className:Y="",descriptionClassName:W="",duration:$,position:X,gap:z,loadingIcon:H,expandByDefault:B,classNames:ee,icons:J,closeButtonAriaLabel:K="Close toast",pauseWhenPageIsHidden:G}=e,[P,ne]=$e.useState(null),[oe,pe]=$e.useState(null),[ae,se]=$e.useState(!1),[Ee,fe]=$e.useState(!1),[Me,Oe]=$e.useState(!1),[xe,Te]=$e.useState(!1),[Ne,he]=$e.useState(!1),[Ze,Et]=$e.useState(0),[qe,Ye]=$e.useState(0),we=$e.useRef(m.duration||$||F8),Qt=$e.useRef(null),or=$e.useRef(null),Pe=k===0,rt=k+1<=w,ge=m.type,Re=m.dismissible!==!1,We=m.className||"",ht=m.descriptionClassName||"",xt=$e.useMemo(()=>E.findIndex(Qe=>Qe.toastId===m.id)||0,[E,m.id]),sr=$e.useMemo(()=>{var Qe;return(Qe=m.closeButton)!=null?Qe:L},[m.closeButton,L]),ar=$e.useMemo(()=>m.duration||$||F8,[m.duration,$]),fr=$e.useRef(0),Mt=$e.useRef(0),lr=$e.useRef(0),qt=$e.useRef(null),[zr,nn]=X.split("-"),dn=$e.useMemo(()=>E.reduce((Qe,zt,Nr)=>Nr>=xt?Qe:Qe+zt.height,0),[E,xt]),Ce=R1e(),st=m.invert||f,ft=ge==="loading";Mt.current=$e.useMemo(()=>xt*z+dn,[xt,dn]),$e.useEffect(()=>{we.current=ar},[ar]),$e.useEffect(()=>{se(!0)},[]),$e.useEffect(()=>{let Qe=or.current;if(Qe){let zt=Qe.getBoundingClientRect().height;return Ye(zt),S(Nr=>[{toastId:m.id,height:zt,position:m.position},...Nr]),()=>S(Nr=>Nr.filter(yn=>yn.toastId!==m.id))}},[S,m.id]),$e.useLayoutEffect(()=>{if(!ae)return;let Qe=or.current,zt=Qe.style.height;Qe.style.height="auto";let Nr=Qe.getBoundingClientRect().height;Qe.style.height=zt,Ye(Nr),S(yn=>yn.find(hn=>hn.toastId===m.id)?yn.map(hn=>hn.toastId===m.id?{...hn,height:Nr}:hn):[{toastId:m.id,height:Nr,position:m.position},...yn])},[ae,m.title,m.description,S,m.id]);let vt=$e.useCallback(()=>{fe(!0),Et(Mt.current),S(Qe=>Qe.filter(zt=>zt.toastId!==m.id)),setTimeout(()=>{C(m)},j1e)},[m,C,S,Mt]);$e.useEffect(()=>{if(m.promise&&ge==="loading"||m.duration===1/0||m.type==="loading")return;let Qe;return R||y||G&&Ce?(()=>{if(lr.current<fr.current){let zt=new Date().getTime()-fr.current;we.current=we.current-zt}lr.current=new Date().getTime()})():we.current!==1/0&&(fr.current=new Date().getTime(),Qe=setTimeout(()=>{var zt;(zt=m.onAutoClose)==null||zt.call(m,m),vt()},we.current)),()=>clearTimeout(Qe)},[R,y,m,ge,G,Ce,vt]),$e.useEffect(()=>{m.delete&&vt()},[vt,m.delete]);function yr(){var Qe,zt,Nr;return J!=null&&J.loading?$e.createElement("div",{className:vo(ee==null?void 0:ee.loader,(Qe=m==null?void 0:m.classNames)==null?void 0:Qe.loader,"sonner-loader"),"data-visible":ge==="loading"},J.loading):H?$e.createElement("div",{className:vo(ee==null?void 0:ee.loader,(zt=m==null?void 0:m.classNames)==null?void 0:zt.loader,"sonner-loader"),"data-visible":ge==="loading"},H):$e.createElement(k1e,{className:vo(ee==null?void 0:ee.loader,(Nr=m==null?void 0:m.classNames)==null?void 0:Nr.loader),visible:ge==="loading"})}return $e.createElement("li",{tabIndex:0,ref:or,className:vo(Y,We,ee==null?void 0:ee.toast,(t=m==null?void 0:m.classNames)==null?void 0:t.toast,ee==null?void 0:ee.default,ee==null?void 0:ee[ge],(r=m==null?void 0:m.classNames)==null?void 0:r[ge]),"data-sonner-toast":"","data-rich-colors":(n=m.richColors)!=null?n:F,"data-styled":!(m.jsx||m.unstyled||b),"data-mounted":ae,"data-promise":!!m.promise,"data-swiped":Ne,"data-removed":Ee,"data-visible":rt,"data-y-position":zr,"data-x-position":nn,"data-index":k,"data-front":Pe,"data-swiping":Me,"data-dismissible":Re,"data-type":ge,"data-invert":st,"data-swipe-out":xe,"data-swipe-direction":oe,"data-expanded":!!(R||B&&ae),style:{"--index":k,"--toasts-before":k,"--z-index":_.length-k,"--offset":`${Ee?Ze:Mt.current}px`,"--initial-height":B?"auto":`${qe}px`,...I,...m.style},onDragEnd:()=>{Oe(!1),ne(null),qt.current=null},onPointerDown:Qe=>{ft||!Re||(Qt.current=new Date,Et(Mt.current),Qe.target.setPointerCapture(Qe.pointerId),Qe.target.tagName!=="BUTTON"&&(Oe(!0),qt.current={x:Qe.clientX,y:Qe.clientY}))},onPointerUp:()=>{var Qe,zt,Nr,yn;if(xe||!Re)return;qt.current=null;let hn=Number(((Qe=or.current)==null?void 0:Qe.style.getPropertyValue("--swipe-amount-x").replace("px",""))||0),Ea=Number(((zt=or.current)==null?void 0:zt.style.getPropertyValue("--swipe-amount-y").replace("px",""))||0),xa=new Date().getTime()-((Nr=Qt.current)==null?void 0:Nr.getTime()),Vn=P==="x"?hn:Ea,kn=Math.abs(Vn)/xa;if(Math.abs(Vn)>=G1e||kn>.11){Et(Mt.current),(yn=m.onDismiss)==null||yn.call(m,m),pe(P==="x"?hn>0?"right":"left":Ea>0?"down":"up"),vt(),Te(!0),he(!1);return}Oe(!1),ne(null)},onPointerMove:Qe=>{var zt,Nr,yn,hn;if(!qt.current||!Re||((zt=window.getSelection())==null?void 0:zt.toString().length)>0)return;let Ea=Qe.clientY-qt.current.y,xa=Qe.clientX-qt.current.x,Vn=(Nr=e.swipeDirections)!=null?Nr:q1e(X);!P&&(Math.abs(xa)>1||Math.abs(Ea)>1)&&ne(Math.abs(xa)>Math.abs(Ea)?"x":"y");let kn={x:0,y:0};P==="y"?(Vn.includes("top")||Vn.includes("bottom"))&&(Vn.includes("top")&&Ea<0||Vn.includes("bottom")&&Ea>0)&&(kn.y=Ea):P==="x"&&(Vn.includes("left")||Vn.includes("right"))&&(Vn.includes("left")&&xa<0||Vn.includes("right")&&xa>0)&&(kn.x=xa),(Math.abs(kn.x)>0||Math.abs(kn.y)>0)&&he(!0),(yn=or.current)==null||yn.style.setProperty("--swipe-amount-x",`${kn.x}px`),(hn=or.current)==null||hn.style.setProperty("--swipe-amount-y",`${kn.y}px`)}},sr&&!m.jsx?$e.createElement("button",{"aria-label":K,"data-disabled":ft,"data-close-button":!0,onClick:ft||!Re?()=>{}:()=>{var Qe;vt(),(Qe=m.onDismiss)==null||Qe.call(m,m)},className:vo(ee==null?void 0:ee.closeButton,(a=m==null?void 0:m.classNames)==null?void 0:a.closeButton)},(i=J==null?void 0:J.close)!=null?i:_1e):null,m.jsx||T.isValidElement(m.title)?m.jsx?m.jsx:typeof m.title=="function"?m.title():m.title:$e.createElement($e.Fragment,null,ge||m.icon||m.promise?$e.createElement("div",{"data-icon":"",className:vo(ee==null?void 0:ee.icon,(o=m==null?void 0:m.classNames)==null?void 0:o.icon)},m.promise||m.type==="loading"&&!m.icon?m.icon||yr():null,m.type!=="loading"?m.icon||(J==null?void 0:J[ge])||x1e(ge):null):null,$e.createElement("div",{"data-content":"",className:vo(ee==null?void 0:ee.content,(s=m==null?void 0:m.classNames)==null?void 0:s.content)},$e.createElement("div",{"data-title":"",className:vo(ee==null?void 0:ee.title,(l=m==null?void 0:m.classNames)==null?void 0:l.title)},typeof m.title=="function"?m.title():m.title),m.description?$e.createElement("div",{"data-description":"",className:vo(W,ht,ee==null?void 0:ee.description,(u=m==null?void 0:m.classNames)==null?void 0:u.description)},typeof m.description=="function"?m.description():m.description):null),T.isValidElement(m.cancel)?m.cancel:m.cancel&&Tb(m.cancel)?$e.createElement("button",{"data-button":!0,"data-cancel":!0,style:m.cancelButtonStyle||M,onClick:Qe=>{var zt,Nr;Tb(m.cancel)&&Re&&((Nr=(zt=m.cancel).onClick)==null||Nr.call(zt,Qe),vt())},className:vo(ee==null?void 0:ee.cancelButton,(d=m==null?void 0:m.classNames)==null?void 0:d.cancelButton)},m.cancel.label):null,T.isValidElement(m.action)?m.action:m.action&&Tb(m.action)?$e.createElement("button",{"data-button":!0,"data-action":!0,style:m.actionButtonStyle||j,onClick:Qe=>{var zt,Nr;Tb(m.action)&&((Nr=(zt=m.action).onClick)==null||Nr.call(zt,Qe),!Qe.defaultPrevented&&vt())},className:vo(ee==null?void 0:ee.actionButton,(p=m==null?void 0:m.classNames)==null?void 0:p.actionButton)},m.action.label):null))};function _8(){if(typeof window>"u"||typeof document>"u")return"ltr";let e=document.documentElement.getAttribute("dir");return e==="auto"||!e?window.getComputedStyle(document.documentElement).direction:e}function V1e(e,t){let r={};return[e,t].forEach((n,a)=>{let i=a===1,o=i?"--mobile-offset":"--offset",s=i?H1e:z1e;function l(u){["top","right","bottom","left"].forEach(d=>{r[`${o}-${d}`]=typeof u=="number"?`${u}px`:u})}typeof n=="number"||typeof n=="string"?l(n):typeof n=="object"?["top","right","bottom","left"].forEach(u=>{n[u]===void 0?r[`${o}-${u}`]=s:r[`${o}-${u}`]=typeof n[u]=="number"?`${n[u]}px`:n[u]}):l(s)}),r}var K1e=T.forwardRef(function(e,t){let{invert:r,position:n="bottom-right",hotkey:a=["altKey","KeyT"],expand:i,closeButton:o,className:s,offset:l,mobileOffset:u,theme:d="light",richColors:p,duration:f,style:m,visibleToasts:b=P1e,toastOptions:y,dir:S=_8(),gap:w=$1e,loadingIcon:E,icons:k,containerAriaLabel:_="Notifications",pauseWhenPageIsHidden:R}=e,[C,F]=$e.useState([]),L=$e.useMemo(()=>Array.from(new Set([n].concat(C.filter(G=>G.position).map(G=>G.position)))),[C,n]),[I,M]=$e.useState([]),[j,Y]=$e.useState(!1),[W,$]=$e.useState(!1),[X,z]=$e.useState(d!=="system"?d:typeof window<"u"&&window.matchMedia&&window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light"),H=$e.useRef(null),B=a.join("+").replace(/Key/g,"").replace(/Digit/g,""),ee=$e.useRef(null),J=$e.useRef(!1),K=$e.useCallback(G=>{F(P=>{var ne;return(ne=P.find(oe=>oe.id===G.id))!=null&&ne.delete||li.dismiss(G.id),P.filter(({id:oe})=>oe!==G.id)})},[]);return $e.useEffect(()=>li.subscribe(G=>{if(G.dismiss){F(P=>P.map(ne=>ne.id===G.id?{...ne,delete:!0}:ne));return}setTimeout(()=>{DY.flushSync(()=>{F(P=>{let ne=P.findIndex(oe=>oe.id===G.id);return ne!==-1?[...P.slice(0,ne),{...P[ne],...G},...P.slice(ne+1)]:[G,...P]})})})}),[]),$e.useEffect(()=>{if(d!=="system"){z(d);return}if(d==="system"&&(window.matchMedia&&window.matchMedia("(prefers-color-scheme: dark)").matches?z("dark"):z("light")),typeof window>"u")return;let G=window.matchMedia("(prefers-color-scheme: dark)");try{G.addEventListener("change",({matches:P})=>{z(P?"dark":"light")})}catch{G.addListener(({matches:ne})=>{try{z(ne?"dark":"light")}catch(oe){console.error(oe)}})}},[d]),$e.useEffect(()=>{C.length<=1&&Y(!1)},[C]),$e.useEffect(()=>{let G=P=>{var ne,oe;a.every(pe=>P[pe]||P.code===pe)&&(Y(!0),(ne=H.current)==null||ne.focus()),P.code==="Escape"&&(document.activeElement===H.current||(oe=H.current)!=null&&oe.contains(document.activeElement))&&Y(!1)};return document.addEventListener("keydown",G),()=>document.removeEventListener("keydown",G)},[a]),$e.useEffect(()=>{if(H.current)return()=>{ee.current&&(ee.current.focus({preventScroll:!0}),ee.current=null,J.current=!1)}},[H.current]),$e.createElement("section",{ref:t,"aria-label":`${_} ${B}`,tabIndex:-1,"aria-live":"polite","aria-relevant":"additions text","aria-atomic":"false",suppressHydrationWarning:!0},L.map((G,P)=>{var ne;let[oe,pe]=G.split("-");return C.length?$e.createElement("ol",{key:G,dir:S==="auto"?_8():S,tabIndex:-1,ref:H,className:s,"data-sonner-toaster":!0,"data-theme":X,"data-y-position":oe,"data-lifted":j&&C.length>1&&!i,"data-x-position":pe,style:{"--front-toast-height":`${((ne=I[0])==null?void 0:ne.height)||0}px`,"--width":`${U1e}px`,"--gap":`${w}px`,...m,...V1e(l,u)},onBlur:ae=>{J.current&&!ae.currentTarget.contains(ae.relatedTarget)&&(J.current=!1,ee.current&&(ee.current.focus({preventScroll:!0}),ee.current=null))},onFocus:ae=>{ae.target instanceof HTMLElement&&ae.target.dataset.dismissible==="false"||J.current||(J.current=!0,ee.current=ae.relatedTarget)},onMouseEnter:()=>Y(!0),onMouseMove:()=>Y(!0),onMouseLeave:()=>{W||Y(!1)},onDragEnd:()=>Y(!1),onPointerDown:ae=>{ae.target instanceof HTMLElement&&ae.target.dataset.dismissible==="false"||$(!0)},onPointerUp:()=>$(!1)},C.filter(ae=>!ae.position&&P===0||ae.position===G).map((ae,se)=>{var Ee,fe;return $e.createElement(W1e,{key:ae.id,icons:k,index:se,toast:ae,defaultRichColors:p,duration:(Ee=y==null?void 0:y.duration)!=null?Ee:f,className:y==null?void 0:y.className,descriptionClassName:y==null?void 0:y.descriptionClassName,invert:r,visibleToasts:b,closeButton:(fe=y==null?void 0:y.closeButton)!=null?fe:o,interacting:W,position:G,style:y==null?void 0:y.style,unstyled:y==null?void 0:y.unstyled,classNames:y==null?void 0:y.classNames,cancelButtonStyle:y==null?void 0:y.cancelButtonStyle,actionButtonStyle:y==null?void 0:y.actionButtonStyle,removeToast:K,toasts:C.filter(Me=>Me.position==ae.position),heights:I.filter(Me=>Me.position==ae.position),setHeights:M,expandByDefault:i,gap:w,loadingIcon:E,expanded:j,pauseWhenPageIsHidden:R,swipeDirections:e.swipeDirections})})):null}))});const Y1e={theme:"system",setTheme:()=>null},wJ=T.createContext(Y1e);function kJ({children:e,...t}){const r=Ve.use.theme(),n=Ve.use.setTheme();T.useEffect(()=>{const i=window.document.documentElement;if(i.classList.remove("light","dark"),r==="system"){const o=window.matchMedia("(prefers-color-scheme: dark)"),s=l=>{i.classList.remove("light","dark"),i.classList.add(l.matches?"dark":"light")};return i.classList.add(o.matches?"dark":"light"),o.addEventListener("change",s),()=>o.removeEventListener("change",s)}else i.classList.add(r)},[r]);const a={theme:r,setTheme:n};return x.jsx(wJ.Provider,{...t,value:a,children:e})}const J1e={visibleTabs:{},setTabVisibility:()=>{},isTabVisible:()=>!1},CJ=T.createContext(J1e),X1e=({children:e})=>{const t=Ve.use.currentTab(),[r,n]=T.useState(()=>({documents:!0,"knowledge-graph":!0,retrieval:!0,api:!0}));T.useEffect(()=>{n(i=>({...i,documents:!0,"knowledge-graph":!0,retrieval:!0,api:!0}))},[t]);const a=T.useMemo(()=>({visibleTabs:r,setTabVisibility:(i,o)=>{n(s=>({...s,[i]:o}))},isTabVisible:i=>!!r[i]}),[r]);return x.jsx(CJ.Provider,{value:a,children:e})},Z1e=(e,t,r,n)=>{var i,o,s,l;const a=[r,{code:t,...n||{}}];if((o=(i=e==null?void 0:e.services)==null?void 0:i.logger)!=null&&o.forward)return e.services.logger.forward(a,"warn","react-i18next::",!0);_c(a[0])&&(a[0]=`react-i18next:: ${a[0]}`),(l=(s=e==null?void 0:e.services)==null?void 0:s.logger)!=null&&l.warn?e.services.logger.warn(...a):console!=null&&console.warn&&console.warn(...a)},R8={},V_=(e,t,r,n)=>{_c(r)&&R8[r]||(_c(r)&&(R8[r]=new Date),Z1e(e,t,r,n))},TJ=(e,t)=>()=>{if(e.isInitialized)t();else{const r=()=>{setTimeout(()=>{e.off("initialized",r)},0),t()};e.on("initialized",r)}},K_=(e,t,r)=>{e.loadNamespaces(t,TJ(e,r))},D8=(e,t,r,n)=>{if(_c(r)&&(r=[r]),e.options.preload&&e.options.preload.indexOf(t)>-1)return K_(e,r,n);r.forEach(a=>{e.options.ns.indexOf(a)<0&&e.options.ns.push(a)}),e.loadLanguages(t,TJ(e,n))},Q1e=(e,t,r={})=>!t.languages||!t.languages.length?(V_(t,"NO_LANGUAGES","i18n.languages were undefined or empty",{languages:t.languages}),!0):t.hasLoadedNamespace(e,{lng:r.lng,precheck:(n,a)=>{if(r.bindI18n&&r.bindI18n.indexOf("languageChanging")>-1&&n.services.backendConnector.backend&&n.isLanguageChangingTo&&!a(n.isLanguageChangingTo,e))return!1}}),_c=e=>typeof e=="string",eye=e=>typeof e=="object"&&e!==null,tye=/&(?:amp|#38|lt|#60|gt|#62|apos|#39|quot|#34|nbsp|#160|copy|#169|reg|#174|hellip|#8230|#x2F|#47);/g,rye={"&amp;":"&","&#38;":"&","&lt;":"<","&#60;":"<","&gt;":">","&#62;":">","&apos;":"'","&#39;":"'","&quot;":'"',"&#34;":'"',"&nbsp;":" ","&#160;":" ","&copy;":"©","&#169;":"©","&reg;":"®","&#174;":"®","&hellip;":"…","&#8230;":"…","&#x2F;":"/","&#47;":"/"},nye=e=>rye[e],aye=e=>e.replace(tye,nye);let Y_={bindI18n:"languageChanged",bindI18nStore:"",transEmptyNodeValue:"",transSupportBasicHtmlNodes:!0,transWrapTextNodes:"",transKeepBasicHtmlNodesFor:["br","strong","i","p"],useSuspense:!0,unescape:aye};const iye=(e={})=>{Y_={...Y_,...e}},oye=()=>Y_;let AJ;const sye=e=>{AJ=e},lye=()=>AJ,uye={type:"3rdParty",init(e){iye(e.options.react),sye(e)}},cye=T.createContext();class dye{constructor(){this.usedNamespaces={}}addUsedNamespaces(t){t.forEach(r=>{this.usedNamespaces[r]||(this.usedNamespaces[r]=!0)})}getUsedNamespaces(){return Object.keys(this.usedNamespaces)}}const hye=(e,t)=>{const r=T.useRef();return T.useEffect(()=>{r.current=e},[e,t]),r.current},FJ=(e,t,r,n)=>e.getFixedT(t,r,n),pye=(e,t,r,n)=>T.useCallback(FJ(e,t,r,n),[e,t,r,n]),Sr=(e,t={})=>{var _,R,C,F;const{i18n:r}=t,{i18n:n,defaultNS:a}=T.useContext(cye)||{},i=r||n||lye();if(i&&!i.reportNamespaces&&(i.reportNamespaces=new dye),!i){V_(i,"NO_I18NEXT_INSTANCE","useTranslation: You will need to pass in an i18next instance by using initReactI18next");const L=(M,j)=>_c(j)?j:eye(j)&&_c(j.defaultValue)?j.defaultValue:Array.isArray(M)?M[M.length-1]:M,I=[L,{},!1];return I.t=L,I.i18n={},I.ready=!1,I}(_=i.options.react)!=null&&_.wait&&V_(i,"DEPRECATED_OPTION","useTranslation: It seems you are still using the old wait option, you may migrate to the new useSuspense behaviour.");const o={...oye(),...i.options.react,...t},{useSuspense:s,keyPrefix:l}=o;let u=a||((R=i.options)==null?void 0:R.defaultNS);u=_c(u)?[u]:u||["translation"],(F=(C=i.reportNamespaces).addUsedNamespaces)==null||F.call(C,u);const d=(i.isInitialized||i.initializedStoreOnce)&&u.every(L=>Q1e(L,i,o)),p=pye(i,t.lng||null,o.nsMode==="fallback"?u:u[0],l),f=()=>p,m=()=>FJ(i,t.lng||null,o.nsMode==="fallback"?u:u[0],l),[b,y]=T.useState(f);let S=u.join();t.lng&&(S=`${t.lng}${S}`);const w=hye(S),E=T.useRef(!0);T.useEffect(()=>{const{bindI18n:L,bindI18nStore:I}=o;E.current=!0,!d&&!s&&(t.lng?D8(i,t.lng,u,()=>{E.current&&y(m)}):K_(i,u,()=>{E.current&&y(m)})),d&&w&&w!==S&&E.current&&y(m);const M=()=>{E.current&&y(m)};return L&&(i==null||i.on(L,M)),I&&(i==null||i.store.on(I,M)),()=>{E.current=!1,i&&L&&(L==null||L.split(" ").forEach(j=>i.off(j,M))),I&&i&&I.split(" ").forEach(j=>i.store.off(j,M))}},[i,S]),T.useEffect(()=>{E.current&&d&&y(f)},[i,l,d]);const k=[b,i,d];if(k.t=b,k.i18n=i,k.ready=d,d||!d&&!s)return k;throw new Promise(L=>{t.lng?D8(i,t.lng,u,()=>L()):K_(i,u,()=>L())})};function fye(e,t){const r=T.createContext(t),n=i=>{const{children:o,...s}=i,l=T.useMemo(()=>s,Object.values(s));return x.jsx(r.Provider,{value:l,children:o})};n.displayName=e+"Provider";function a(i){const o=T.useContext(r);if(o)return o;if(t!==void 0)return t;throw new Error(`\`${i}\` must be used within \`${e}\``)}return[n,a]}function Bo(e,t=[]){let r=[];function n(i,o){const s=T.createContext(o),l=r.length;r=[...r,o];const u=p=>{var w;const{scope:f,children:m,...b}=p,y=((w=f==null?void 0:f[e])==null?void 0:w[l])||s,S=T.useMemo(()=>b,Object.values(b));return x.jsx(y.Provider,{value:S,children:m})};u.displayName=i+"Provider";function d(p,f){var y;const m=((y=f==null?void 0:f[e])==null?void 0:y[l])||s,b=T.useContext(m);if(b)return b;if(o!==void 0)return o;throw new Error(`\`${p}\` must be used within \`${i}\``)}return[u,d]}const a=()=>{const i=r.map(o=>T.createContext(o));return function(s){const l=(s==null?void 0:s[e])||i;return T.useMemo(()=>({[`__scope${e}`]:{...s,[e]:l}}),[s,l])}};return a.scopeName=e,[n,mye(a,...t)]}function mye(...e){const t=e[0];if(e.length===1)return t;const r=()=>{const n=e.map(a=>({useScope:a(),scopeName:a.scopeName}));return function(i){const o=n.reduce((s,{useScope:l,scopeName:u})=>{const p=l(i)[`__scope${u}`];return{...s,...p}},{});return T.useMemo(()=>({[`__scope${t.scopeName}`]:o}),[o])}};return r.scopeName=t.scopeName,r}function N8(e,t){if(typeof e=="function")return e(t);e!=null&&(e.current=t)}function Ic(...e){return t=>{let r=!1;const n=e.map(a=>{const i=N8(a,t);return!r&&typeof i=="function"&&(r=!0),i});if(r)return()=>{for(let a=0;a<n.length;a++){const i=n[a];typeof i=="function"?i():N8(e[a],null)}}}}function Rr(...e){return T.useCallback(Ic(...e),e)}function $t(e,t,{checkForDefaultPrevented:r=!0}={}){return function(a){if(e==null||e(a),r===!1||!a.defaultPrevented)return t==null?void 0:t(a)}}var Sa=globalThis!=null&&globalThis.document?T.useLayoutEffect:()=>{},gye=uK[" useId ".trim().toString()]||(()=>{}),bye=0;function La(e){const[t,r]=T.useState(gye());return Sa(()=>{r(n=>n??String(bye++))},[e]),t?`radix-${t}`:""}var yye=uK[" useInsertionEffect ".trim().toString()]||Sa;function bl({prop:e,defaultProp:t,onChange:r=()=>{},caller:n}){const[a,i,o]=vye({defaultProp:t,onChange:r}),s=e!==void 0,l=s?e:a;{const d=T.useRef(e!==void 0);T.useEffect(()=>{const p=d.current;p!==s&&console.warn(`${n} is changing from ${p?"controlled":"uncontrolled"} to ${s?"controlled":"uncontrolled"}. Components should not switch from controlled to uncontrolled (or vice versa). Decide between using a controlled or uncontrolled value for the lifetime of the component.`),d.current=s},[s,n])}const u=T.useCallback(d=>{var p;if(s){const f=Sye(d)?d(e):d;f!==e&&((p=o.current)==null||p.call(o,f))}else i(d)},[s,e,i,o]);return[l,u]}function vye({defaultProp:e,onChange:t}){const[r,n]=T.useState(e),a=T.useRef(r),i=T.useRef(t);return yye(()=>{i.current=t},[t]),T.useEffect(()=>{var o;a.current!==r&&((o=i.current)==null||o.call(i,r),a.current=r)},[r,a]),[r,n,i]}function Sye(e){return typeof e=="function"}function Oc(e){const t=xye(e),r=T.forwardRef((n,a)=>{const{children:i,...o}=n,s=T.Children.toArray(i),l=s.find(wye);if(l){const u=l.props.children,d=s.map(p=>p===l?T.Children.count(u)>1?T.Children.only(null):T.isValidElement(u)?u.props.children:null:p);return x.jsx(t,{...o,ref:a,children:T.isValidElement(u)?T.cloneElement(u,void 0,d):null})}return x.jsx(t,{...o,ref:a,children:i})});return r.displayName=`${e}.Slot`,r}var Eye=Oc("Slot");function xye(e){const t=T.forwardRef((r,n)=>{const{children:a,...i}=r;if(T.isValidElement(a)){const o=Cye(a),s=kye(i,a.props);return a.type!==T.Fragment&&(s.ref=n?Ic(n,o):o),T.cloneElement(a,s)}return T.Children.count(a)>1?T.Children.only(null):null});return t.displayName=`${e}.SlotClone`,t}var _J=Symbol("radix.slottable");function RJ(e){const t=({children:r})=>x.jsx(x.Fragment,{children:r});return t.displayName=`${e}.Slottable`,t.__radixId=_J,t}function wye(e){return T.isValidElement(e)&&typeof e.type=="function"&&"__radixId"in e.type&&e.type.__radixId===_J}function kye(e,t){const r={...t};for(const n in t){const a=e[n],i=t[n];/^on[A-Z]/.test(n)?a&&i?r[n]=(...s)=>{const l=i(...s);return a(...s),l}:a&&(r[n]=a):n==="style"?r[n]={...a,...i}:n==="className"&&(r[n]=[a,i].filter(Boolean).join(" "))}return{...e,...r}}function Cye(e){var n,a;let t=(n=Object.getOwnPropertyDescriptor(e.props,"ref"))==null?void 0:n.get,r=t&&"isReactWarning"in t&&t.isReactWarning;return r?e.ref:(t=(a=Object.getOwnPropertyDescriptor(e,"ref"))==null?void 0:a.get,r=t&&"isReactWarning"in t&&t.isReactWarning,r?e.props.ref:e.props.ref||e.ref)}var Tye=["a","button","div","form","h2","h3","img","input","label","li","nav","ol","p","select","span","svg","ul"],Kt=Tye.reduce((e,t)=>{const r=Oc(`Primitive.${t}`),n=T.forwardRef((a,i)=>{const{asChild:o,...s}=a,l=o?r:t;return typeof window<"u"&&(window[Symbol.for("radix-ui")]=!0),x.jsx(l,{...s,ref:i})});return n.displayName=`Primitive.${t}`,{...e,[t]:n}},{});function Aye(e,t){e&&mg.flushSync(()=>e.dispatchEvent(t))}function Ga(e){const t=T.useRef(e);return T.useEffect(()=>{t.current=e}),T.useMemo(()=>(...r)=>{var n;return(n=t.current)==null?void 0:n.call(t,...r)},[])}function Fye(e,t=globalThis==null?void 0:globalThis.document){const r=Ga(e);T.useEffect(()=>{const n=a=>{a.key==="Escape"&&r(a)};return t.addEventListener("keydown",n,{capture:!0}),()=>t.removeEventListener("keydown",n,{capture:!0})},[r,t])}var _ye="DismissableLayer",J_="dismissableLayer.update",Rye="dismissableLayer.pointerDownOutside",Dye="dismissableLayer.focusOutside",L8,DJ=T.createContext({layers:new Set,layersWithOutsidePointerEventsDisabled:new Set,branches:new Set}),Sg=T.forwardRef((e,t)=>{const{disableOutsidePointerEvents:r=!1,onEscapeKeyDown:n,onPointerDownOutside:a,onFocusOutside:i,onInteractOutside:o,onDismiss:s,...l}=e,u=T.useContext(DJ),[d,p]=T.useState(null),f=(d==null?void 0:d.ownerDocument)??(globalThis==null?void 0:globalThis.document),[,m]=T.useState({}),b=Rr(t,F=>p(F)),y=Array.from(u.layers),[S]=[...u.layersWithOutsidePointerEventsDisabled].slice(-1),w=y.indexOf(S),E=d?y.indexOf(d):-1,k=u.layersWithOutsidePointerEventsDisabled.size>0,_=E>=w,R=Mye(F=>{const L=F.target,I=[...u.branches].some(M=>M.contains(L));!_||I||(a==null||a(F),o==null||o(F),F.defaultPrevented||s==null||s())},f),C=Iye(F=>{const L=F.target;[...u.branches].some(M=>M.contains(L))||(i==null||i(F),o==null||o(F),F.defaultPrevented||s==null||s())},f);return Fye(F=>{E===u.layers.size-1&&(n==null||n(F),!F.defaultPrevented&&s&&(F.preventDefault(),s()))},f),T.useEffect(()=>{if(d)return r&&(u.layersWithOutsidePointerEventsDisabled.size===0&&(L8=f.body.style.pointerEvents,f.body.style.pointerEvents="none"),u.layersWithOutsidePointerEventsDisabled.add(d)),u.layers.add(d),M8(),()=>{r&&u.layersWithOutsidePointerEventsDisabled.size===1&&(f.body.style.pointerEvents=L8)}},[d,f,r,u]),T.useEffect(()=>()=>{d&&(u.layers.delete(d),u.layersWithOutsidePointerEventsDisabled.delete(d),M8())},[d,u]),T.useEffect(()=>{const F=()=>m({});return document.addEventListener(J_,F),()=>document.removeEventListener(J_,F)},[]),x.jsx(Kt.div,{...l,ref:b,style:{pointerEvents:k?_?"auto":"none":void 0,...e.style},onFocusCapture:$t(e.onFocusCapture,C.onFocusCapture),onBlurCapture:$t(e.onBlurCapture,C.onBlurCapture),onPointerDownCapture:$t(e.onPointerDownCapture,R.onPointerDownCapture)})});Sg.displayName=_ye;var Nye="DismissableLayerBranch",Lye=T.forwardRef((e,t)=>{const r=T.useContext(DJ),n=T.useRef(null),a=Rr(t,n);return T.useEffect(()=>{const i=n.current;if(i)return r.branches.add(i),()=>{r.branches.delete(i)}},[r.branches]),x.jsx(Kt.div,{...e,ref:a})});Lye.displayName=Nye;function Mye(e,t=globalThis==null?void 0:globalThis.document){const r=Ga(e),n=T.useRef(!1),a=T.useRef(()=>{});return T.useEffect(()=>{const i=s=>{if(s.target&&!n.current){let l=function(){NJ(Rye,r,u,{discrete:!0})};const u={originalEvent:s};s.pointerType==="touch"?(t.removeEventListener("click",a.current),a.current=l,t.addEventListener("click",a.current,{once:!0})):l()}else t.removeEventListener("click",a.current);n.current=!1},o=window.setTimeout(()=>{t.addEventListener("pointerdown",i)},0);return()=>{window.clearTimeout(o),t.removeEventListener("pointerdown",i),t.removeEventListener("click",a.current)}},[t,r]),{onPointerDownCapture:()=>n.current=!0}}function Iye(e,t=globalThis==null?void 0:globalThis.document){const r=Ga(e),n=T.useRef(!1);return T.useEffect(()=>{const a=i=>{i.target&&!n.current&&NJ(Dye,r,{originalEvent:i},{discrete:!1})};return t.addEventListener("focusin",a),()=>t.removeEventListener("focusin",a)},[t,r]),{onFocusCapture:()=>n.current=!0,onBlurCapture:()=>n.current=!1}}function M8(){const e=new CustomEvent(J_);document.dispatchEvent(e)}function NJ(e,t,r,{discrete:n}){const a=r.originalEvent.target,i=new CustomEvent(e,{bubbles:!1,cancelable:!0,detail:r});t&&a.addEventListener(e,t,{once:!0}),n?Aye(a,i):a.dispatchEvent(i)}var jx="focusScope.autoFocusOnMount",qx="focusScope.autoFocusOnUnmount",I8={bubbles:!1,cancelable:!0},Oye="FocusScope",Pv=T.forwardRef((e,t)=>{const{loop:r=!1,trapped:n=!1,onMountAutoFocus:a,onUnmountAutoFocus:i,...o}=e,[s,l]=T.useState(null),u=Ga(a),d=Ga(i),p=T.useRef(null),f=Rr(t,y=>l(y)),m=T.useRef({paused:!1,pause(){this.paused=!0},resume(){this.paused=!1}}).current;T.useEffect(()=>{if(n){let y=function(k){if(m.paused||!s)return;const _=k.target;s.contains(_)?p.current=_:cu(p.current,{select:!0})},S=function(k){if(m.paused||!s)return;const _=k.relatedTarget;_!==null&&(s.contains(_)||cu(p.current,{select:!0}))},w=function(k){if(document.activeElement===document.body)for(const R of k)R.removedNodes.length>0&&cu(s)};document.addEventListener("focusin",y),document.addEventListener("focusout",S);const E=new MutationObserver(w);return s&&E.observe(s,{childList:!0,subtree:!0}),()=>{document.removeEventListener("focusin",y),document.removeEventListener("focusout",S),E.disconnect()}}},[n,s,m.paused]),T.useEffect(()=>{if(s){B8.add(m);const y=document.activeElement;if(!s.contains(y)){const w=new CustomEvent(jx,I8);s.addEventListener(jx,u),s.dispatchEvent(w),w.defaultPrevented||(Bye($ye(LJ(s)),{select:!0}),document.activeElement===y&&cu(s))}return()=>{s.removeEventListener(jx,u),setTimeout(()=>{const w=new CustomEvent(qx,I8);s.addEventListener(qx,d),s.dispatchEvent(w),w.defaultPrevented||cu(y??document.body,{select:!0}),s.removeEventListener(qx,d),B8.remove(m)},0)}}},[s,u,d,m]);const b=T.useCallback(y=>{if(!r&&!n||m.paused)return;const S=y.key==="Tab"&&!y.altKey&&!y.ctrlKey&&!y.metaKey,w=document.activeElement;if(S&&w){const E=y.currentTarget,[k,_]=Pye(E);k&&_?!y.shiftKey&&w===_?(y.preventDefault(),r&&cu(k,{select:!0})):y.shiftKey&&w===k&&(y.preventDefault(),r&&cu(_,{select:!0})):w===E&&y.preventDefault()}},[r,n,m.paused]);return x.jsx(Kt.div,{tabIndex:-1,...o,ref:f,onKeyDown:b})});Pv.displayName=Oye;function Bye(e,{select:t=!1}={}){const r=document.activeElement;for(const n of e)if(cu(n,{select:t}),document.activeElement!==r)return}function Pye(e){const t=LJ(e),r=O8(t,e),n=O8(t.reverse(),e);return[r,n]}function LJ(e){const t=[],r=document.createTreeWalker(e,NodeFilter.SHOW_ELEMENT,{acceptNode:n=>{const a=n.tagName==="INPUT"&&n.type==="hidden";return n.disabled||n.hidden||a?NodeFilter.FILTER_SKIP:n.tabIndex>=0?NodeFilter.FILTER_ACCEPT:NodeFilter.FILTER_SKIP}});for(;r.nextNode();)t.push(r.currentNode);return t}function O8(e,t){for(const r of e)if(!zye(r,{upTo:t}))return r}function zye(e,{upTo:t}){if(getComputedStyle(e).visibility==="hidden")return!0;for(;e;){if(t!==void 0&&e===t)return!1;if(getComputedStyle(e).display==="none")return!0;e=e.parentElement}return!1}function Hye(e){return e instanceof HTMLInputElement&&"select"in e}function cu(e,{select:t=!1}={}){if(e&&e.focus){const r=document.activeElement;e.focus({preventScroll:!0}),e!==r&&Hye(e)&&t&&e.select()}}var B8=Uye();function Uye(){let e=[];return{add(t){const r=e[0];t!==r&&(r==null||r.pause()),e=P8(e,t),e.unshift(t)},remove(t){var r;e=P8(e,t),(r=e[0])==null||r.resume()}}}function P8(e,t){const r=[...e],n=r.indexOf(t);return n!==-1&&r.splice(n,1),r}function $ye(e){return e.filter(t=>t.tagName!=="A")}var Gye="Portal",zv=T.forwardRef((e,t)=>{var s;const{container:r,...n}=e,[a,i]=T.useState(!1);Sa(()=>i(!0),[]);const o=r||a&&((s=globalThis==null?void 0:globalThis.document)==null?void 0:s.body);return o?DY.createPortal(x.jsx(Kt.div,{...n,ref:t}),o):null});zv.displayName=Gye;function jye(e,t){return T.useReducer((r,n)=>t[r][n]??r,e)}var $i=e=>{const{present:t,children:r}=e,n=qye(t),a=typeof r=="function"?r({present:n.isPresent}):T.Children.only(r),i=Rr(n.ref,Wye(a));return typeof r=="function"||n.isPresent?T.cloneElement(a,{ref:i}):null};$i.displayName="Presence";function qye(e){const[t,r]=T.useState(),n=T.useRef(null),a=T.useRef(e),i=T.useRef("none"),o=e?"mounted":"unmounted",[s,l]=jye(o,{mounted:{UNMOUNT:"unmounted",ANIMATION_OUT:"unmountSuspended"},unmountSuspended:{MOUNT:"mounted",ANIMATION_END:"unmounted"},unmounted:{MOUNT:"mounted"}});return T.useEffect(()=>{const u=Ab(n.current);i.current=s==="mounted"?u:"none"},[s]),Sa(()=>{const u=n.current,d=a.current;if(d!==e){const f=i.current,m=Ab(u);e?l("MOUNT"):m==="none"||(u==null?void 0:u.display)==="none"?l("UNMOUNT"):l(d&&f!==m?"ANIMATION_OUT":"UNMOUNT"),a.current=e}},[e,l]),Sa(()=>{if(t){let u;const d=t.ownerDocument.defaultView??window,p=m=>{const y=Ab(n.current).includes(CSS.escape(m.animationName));if(m.target===t&&y&&(l("ANIMATION_END"),!a.current)){const S=t.style.animationFillMode;t.style.animationFillMode="forwards",u=d.setTimeout(()=>{t.style.animationFillMode==="forwards"&&(t.style.animationFillMode=S)})}},f=m=>{m.target===t&&(i.current=Ab(n.current))};return t.addEventListener("animationstart",f),t.addEventListener("animationcancel",p),t.addEventListener("animationend",p),()=>{d.clearTimeout(u),t.removeEventListener("animationstart",f),t.removeEventListener("animationcancel",p),t.removeEventListener("animationend",p)}}else l("ANIMATION_END")},[t,l]),{isPresent:["mounted","unmountSuspended"].includes(s),ref:T.useCallback(u=>{n.current=u?getComputedStyle(u):null,r(u)},[])}}function Ab(e){return(e==null?void 0:e.animationName)||"none"}function Wye(e){var n,a;let t=(n=Object.getOwnPropertyDescriptor(e.props,"ref"))==null?void 0:n.get,r=t&&"isReactWarning"in t&&t.isReactWarning;return r?e.ref:(t=(a=Object.getOwnPropertyDescriptor(e,"ref"))==null?void 0:a.get,r=t&&"isReactWarning"in t&&t.isReactWarning,r?e.props.ref:e.props.ref||e.ref)}var Wx=0;function ID(){T.useEffect(()=>{const e=document.querySelectorAll("[data-radix-focus-guard]");return document.body.insertAdjacentElement("afterbegin",e[0]??z8()),document.body.insertAdjacentElement("beforeend",e[1]??z8()),Wx++,()=>{Wx===1&&document.querySelectorAll("[data-radix-focus-guard]").forEach(t=>t.remove()),Wx--}},[])}function z8(){const e=document.createElement("span");return e.setAttribute("data-radix-focus-guard",""),e.tabIndex=0,e.style.outline="none",e.style.opacity="0",e.style.position="fixed",e.style.pointerEvents="none",e}var rs=function(){return rs=Object.assign||function(t){for(var r,n=1,a=arguments.length;n<a;n++){r=arguments[n];for(var i in r)Object.prototype.hasOwnProperty.call(r,i)&&(t[i]=r[i])}return t},rs.apply(this,arguments)};function MJ(e,t){var r={};for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&t.indexOf(n)<0&&(r[n]=e[n]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var a=0,n=Object.getOwnPropertySymbols(e);a<n.length;a++)t.indexOf(n[a])<0&&Object.prototype.propertyIsEnumerable.call(e,n[a])&&(r[n[a]]=e[n[a]]);return r}function nd(e,t,r,n){function a(i){return i instanceof r?i:new r(function(o){o(i)})}return new(r||(r=Promise))(function(i,o){function s(d){try{u(n.next(d))}catch(p){o(p)}}function l(d){try{u(n.throw(d))}catch(p){o(p)}}function u(d){d.done?i(d.value):a(d.value).then(s,l)}u((n=n.apply(e,t||[])).next())})}function Vye(e,t,r){if(r||arguments.length===2)for(var n=0,a=t.length,i;n<a;n++)(i||!(n in t))&&(i||(i=Array.prototype.slice.call(t,0,n)),i[n]=t[n]);return e.concat(i||Array.prototype.slice.call(t))}var w1="right-scroll-bar-position",k1="width-before-scroll-bar",Kye="with-scroll-bars-hidden",Yye="--removed-body-scroll-bar-size";function Vx(e,t){return typeof e=="function"?e(t):e&&(e.current=t),e}function Jye(e,t){var r=T.useState(function(){return{value:e,callback:t,facade:{get current(){return r.value},set current(n){var a=r.value;a!==n&&(r.value=n,r.callback(n,a))}}}})[0];return r.callback=t,r.facade}var Xye=typeof window<"u"?T.useLayoutEffect:T.useEffect,H8=new WeakMap;function Zye(e,t){var r=Jye(null,function(n){return e.forEach(function(a){return Vx(a,n)})});return Xye(function(){var n=H8.get(r);if(n){var a=new Set(n),i=new Set(e),o=r.current;a.forEach(function(s){i.has(s)||Vx(s,null)}),i.forEach(function(s){a.has(s)||Vx(s,o)})}H8.set(r,e)},[e]),r}function Qye(e){return e}function eve(e,t){t===void 0&&(t=Qye);var r=[],n=!1,a={read:function(){if(n)throw new Error("Sidecar: could not `read` from an `assigned` medium. `read` could be used only with `useMedium`.");return r.length?r[r.length-1]:e},useMedium:function(i){var o=t(i,n);return r.push(o),function(){r=r.filter(function(s){return s!==o})}},assignSyncMedium:function(i){for(n=!0;r.length;){var o=r;r=[],o.forEach(i)}r={push:function(s){return i(s)},filter:function(){return r}}},assignMedium:function(i){n=!0;var o=[];if(r.length){var s=r;r=[],s.forEach(i),o=r}var l=function(){var d=o;o=[],d.forEach(i)},u=function(){return Promise.resolve().then(l)};u(),r={push:function(d){o.push(d),u()},filter:function(d){return o=o.filter(d),r}}}};return a}function tve(e){e===void 0&&(e={});var t=eve(null);return t.options=rs({async:!0,ssr:!1},e),t}var IJ=function(e){var t=e.sideCar,r=MJ(e,["sideCar"]);if(!t)throw new Error("Sidecar: please provide `sideCar` property to import the right car");var n=t.read();if(!n)throw new Error("Sidecar medium not found");return T.createElement(n,rs({},r))};IJ.isSideCarExport=!0;function rve(e,t){return e.useMedium(t),IJ}var OJ=tve(),Kx=function(){},Hv=T.forwardRef(function(e,t){var r=T.useRef(null),n=T.useState({onScrollCapture:Kx,onWheelCapture:Kx,onTouchMoveCapture:Kx}),a=n[0],i=n[1],o=e.forwardProps,s=e.children,l=e.className,u=e.removeScrollBar,d=e.enabled,p=e.shards,f=e.sideCar,m=e.noIsolation,b=e.inert,y=e.allowPinchZoom,S=e.as,w=S===void 0?"div":S,E=e.gapMode,k=MJ(e,["forwardProps","children","className","removeScrollBar","enabled","shards","sideCar","noIsolation","inert","allowPinchZoom","as","gapMode"]),_=f,R=Zye([r,t]),C=rs(rs({},k),a);return T.createElement(T.Fragment,null,d&&T.createElement(_,{sideCar:OJ,removeScrollBar:u,shards:p,noIsolation:m,inert:b,setCallbacks:i,allowPinchZoom:!!y,lockRef:r,gapMode:E}),o?T.cloneElement(T.Children.only(s),rs(rs({},C),{ref:R})):T.createElement(w,rs({},C,{className:l,ref:R}),s))});Hv.defaultProps={enabled:!0,removeScrollBar:!0,inert:!1};Hv.classNames={fullWidth:k1,zeroRight:w1};var nve=function(){if(typeof __webpack_nonce__<"u")return __webpack_nonce__};function ave(){if(!document)return null;var e=document.createElement("style");e.type="text/css";var t=nve();return t&&e.setAttribute("nonce",t),e}function ive(e,t){e.styleSheet?e.styleSheet.cssText=t:e.appendChild(document.createTextNode(t))}function ove(e){var t=document.head||document.getElementsByTagName("head")[0];t.appendChild(e)}var sve=function(){var e=0,t=null;return{add:function(r){e==0&&(t=ave())&&(ive(t,r),ove(t)),e++},remove:function(){e--,!e&&t&&(t.parentNode&&t.parentNode.removeChild(t),t=null)}}},lve=function(){var e=sve();return function(t,r){T.useEffect(function(){return e.add(t),function(){e.remove()}},[t&&r])}},BJ=function(){var e=lve(),t=function(r){var n=r.styles,a=r.dynamic;return e(n,a),null};return t},uve={left:0,top:0,right:0,gap:0},Yx=function(e){return parseInt(e||"",10)||0},cve=function(e){var t=window.getComputedStyle(document.body),r=t[e==="padding"?"paddingLeft":"marginLeft"],n=t[e==="padding"?"paddingTop":"marginTop"],a=t[e==="padding"?"paddingRight":"marginRight"];return[Yx(r),Yx(n),Yx(a)]},dve=function(e){if(e===void 0&&(e="margin"),typeof window>"u")return uve;var t=cve(e),r=document.documentElement.clientWidth,n=window.innerWidth;return{left:t[0],top:t[1],right:t[2],gap:Math.max(0,n-r+t[2]-t[0])}},hve=BJ(),vh="data-scroll-locked",pve=function(e,t,r,n){var a=e.left,i=e.top,o=e.right,s=e.gap;return r===void 0&&(r="margin"),`
  .`.concat(Kye,` {
   overflow: hidden `).concat(n,`;
   padding-right: `).concat(s,"px ").concat(n,`;
  }
  body[`).concat(vh,`] {
    overflow: hidden `).concat(n,`;
    overscroll-behavior: contain;
    `).concat([t&&"position: relative ".concat(n,";"),r==="margin"&&`
    padding-left: `.concat(a,`px;
    padding-top: `).concat(i,`px;
    padding-right: `).concat(o,`px;
    margin-left:0;
    margin-top:0;
    margin-right: `).concat(s,"px ").concat(n,`;
    `),r==="padding"&&"padding-right: ".concat(s,"px ").concat(n,";")].filter(Boolean).join(""),`
  }
  
  .`).concat(w1,` {
    right: `).concat(s,"px ").concat(n,`;
  }
  
  .`).concat(k1,` {
    margin-right: `).concat(s,"px ").concat(n,`;
  }
  
  .`).concat(w1," .").concat(w1,` {
    right: 0 `).concat(n,`;
  }
  
  .`).concat(k1," .").concat(k1,` {
    margin-right: 0 `).concat(n,`;
  }
  
  body[`).concat(vh,`] {
    `).concat(Yye,": ").concat(s,`px;
  }
`)},U8=function(){var e=parseInt(document.body.getAttribute(vh)||"0",10);return isFinite(e)?e:0},fve=function(){T.useEffect(function(){return document.body.setAttribute(vh,(U8()+1).toString()),function(){var e=U8()-1;e<=0?document.body.removeAttribute(vh):document.body.setAttribute(vh,e.toString())}},[])},mve=function(e){var t=e.noRelative,r=e.noImportant,n=e.gapMode,a=n===void 0?"margin":n;fve();var i=T.useMemo(function(){return dve(a)},[a]);return T.createElement(hve,{styles:pve(i,!t,a,r?"":"!important")})},X_=!1;if(typeof window<"u")try{var Fb=Object.defineProperty({},"passive",{get:function(){return X_=!0,!0}});window.addEventListener("test",Fb,Fb),window.removeEventListener("test",Fb,Fb)}catch{X_=!1}var Zd=X_?{passive:!1}:!1,gve=function(e){return e.tagName==="TEXTAREA"},PJ=function(e,t){if(!(e instanceof Element))return!1;var r=window.getComputedStyle(e);return r[t]!=="hidden"&&!(r.overflowY===r.overflowX&&!gve(e)&&r[t]==="visible")},bve=function(e){return PJ(e,"overflowY")},yve=function(e){return PJ(e,"overflowX")},$8=function(e,t){var r=t.ownerDocument,n=t;do{typeof ShadowRoot<"u"&&n instanceof ShadowRoot&&(n=n.host);var a=zJ(e,n);if(a){var i=HJ(e,n),o=i[1],s=i[2];if(o>s)return!0}n=n.parentNode}while(n&&n!==r.body);return!1},vve=function(e){var t=e.scrollTop,r=e.scrollHeight,n=e.clientHeight;return[t,r,n]},Sve=function(e){var t=e.scrollLeft,r=e.scrollWidth,n=e.clientWidth;return[t,r,n]},zJ=function(e,t){return e==="v"?bve(t):yve(t)},HJ=function(e,t){return e==="v"?vve(t):Sve(t)},Eve=function(e,t){return e==="h"&&t==="rtl"?-1:1},xve=function(e,t,r,n,a){var i=Eve(e,window.getComputedStyle(t).direction),o=i*n,s=r.target,l=t.contains(s),u=!1,d=o>0,p=0,f=0;do{var m=HJ(e,s),b=m[0],y=m[1],S=m[2],w=y-S-i*b;(b||w)&&zJ(e,s)&&(p+=w,f+=b),s instanceof ShadowRoot?s=s.host:s=s.parentNode}while(!l&&s!==document.body||l&&(t.contains(s)||t===s));return(d&&Math.abs(p)<1||!d&&Math.abs(f)<1)&&(u=!0),u},_b=function(e){return"changedTouches"in e?[e.changedTouches[0].clientX,e.changedTouches[0].clientY]:[0,0]},G8=function(e){return[e.deltaX,e.deltaY]},j8=function(e){return e&&"current"in e?e.current:e},wve=function(e,t){return e[0]===t[0]&&e[1]===t[1]},kve=function(e){return`
  .block-interactivity-`.concat(e,` {pointer-events: none;}
  .allow-interactivity-`).concat(e,` {pointer-events: all;}
`)},Cve=0,Qd=[];function Tve(e){var t=T.useRef([]),r=T.useRef([0,0]),n=T.useRef(),a=T.useState(Cve++)[0],i=T.useState(BJ)[0],o=T.useRef(e);T.useEffect(function(){o.current=e},[e]),T.useEffect(function(){if(e.inert){document.body.classList.add("block-interactivity-".concat(a));var y=Vye([e.lockRef.current],(e.shards||[]).map(j8),!0).filter(Boolean);return y.forEach(function(S){return S.classList.add("allow-interactivity-".concat(a))}),function(){document.body.classList.remove("block-interactivity-".concat(a)),y.forEach(function(S){return S.classList.remove("allow-interactivity-".concat(a))})}}},[e.inert,e.lockRef.current,e.shards]);var s=T.useCallback(function(y,S){if("touches"in y&&y.touches.length===2||y.type==="wheel"&&y.ctrlKey)return!o.current.allowPinchZoom;var w=_b(y),E=r.current,k="deltaX"in y?y.deltaX:E[0]-w[0],_="deltaY"in y?y.deltaY:E[1]-w[1],R,C=y.target,F=Math.abs(k)>Math.abs(_)?"h":"v";if("touches"in y&&F==="h"&&C.type==="range")return!1;var L=$8(F,C);if(!L)return!0;if(L?R=F:(R=F==="v"?"h":"v",L=$8(F,C)),!L)return!1;if(!n.current&&"changedTouches"in y&&(k||_)&&(n.current=R),!R)return!0;var I=n.current||R;return xve(I,S,y,I==="h"?k:_)},[]),l=T.useCallback(function(y){var S=y;if(!(!Qd.length||Qd[Qd.length-1]!==i)){var w="deltaY"in S?G8(S):_b(S),E=t.current.filter(function(R){return R.name===S.type&&(R.target===S.target||S.target===R.shadowParent)&&wve(R.delta,w)})[0];if(E&&E.should){S.cancelable&&S.preventDefault();return}if(!E){var k=(o.current.shards||[]).map(j8).filter(Boolean).filter(function(R){return R.contains(S.target)}),_=k.length>0?s(S,k[0]):!o.current.noIsolation;_&&S.cancelable&&S.preventDefault()}}},[]),u=T.useCallback(function(y,S,w,E){var k={name:y,delta:S,target:w,should:E,shadowParent:Ave(w)};t.current.push(k),setTimeout(function(){t.current=t.current.filter(function(_){return _!==k})},1)},[]),d=T.useCallback(function(y){r.current=_b(y),n.current=void 0},[]),p=T.useCallback(function(y){u(y.type,G8(y),y.target,s(y,e.lockRef.current))},[]),f=T.useCallback(function(y){u(y.type,_b(y),y.target,s(y,e.lockRef.current))},[]);T.useEffect(function(){return Qd.push(i),e.setCallbacks({onScrollCapture:p,onWheelCapture:p,onTouchMoveCapture:f}),document.addEventListener("wheel",l,Zd),document.addEventListener("touchmove",l,Zd),document.addEventListener("touchstart",d,Zd),function(){Qd=Qd.filter(function(y){return y!==i}),document.removeEventListener("wheel",l,Zd),document.removeEventListener("touchmove",l,Zd),document.removeEventListener("touchstart",d,Zd)}},[]);var m=e.removeScrollBar,b=e.inert;return T.createElement(T.Fragment,null,b?T.createElement(i,{styles:kve(a)}):null,m?T.createElement(mve,{gapMode:e.gapMode}):null)}function Ave(e){for(var t=null;e!==null;)e instanceof ShadowRoot&&(t=e.host,e=e.host),e=e.parentNode;return t}const Fve=rve(OJ,Tve);var Uv=T.forwardRef(function(e,t){return T.createElement(Hv,rs({},e,{ref:t,sideCar:Fve}))});Uv.classNames=Hv.classNames;var _ve=function(e){if(typeof document>"u")return null;var t=Array.isArray(e)?e[0]:e;return t.ownerDocument.body},eh=new WeakMap,Rb=new WeakMap,Db={},Jx=0,UJ=function(e){return e&&(e.host||UJ(e.parentNode))},Rve=function(e,t){return t.map(function(r){if(e.contains(r))return r;var n=UJ(r);return n&&e.contains(n)?n:(console.error("aria-hidden",r,"in not contained inside",e,". Doing nothing"),null)}).filter(function(r){return!!r})},Dve=function(e,t,r,n){var a=Rve(t,Array.isArray(e)?e:[e]);Db[r]||(Db[r]=new WeakMap);var i=Db[r],o=[],s=new Set,l=new Set(a),u=function(p){!p||s.has(p)||(s.add(p),u(p.parentNode))};a.forEach(u);var d=function(p){!p||l.has(p)||Array.prototype.forEach.call(p.children,function(f){if(s.has(f))d(f);else try{var m=f.getAttribute(n),b=m!==null&&m!=="false",y=(eh.get(f)||0)+1,S=(i.get(f)||0)+1;eh.set(f,y),i.set(f,S),o.push(f),y===1&&b&&Rb.set(f,!0),S===1&&f.setAttribute(r,"true"),b||f.setAttribute(n,"true")}catch(w){console.error("aria-hidden: cannot operate on ",f,w)}})};return d(t),s.clear(),Jx++,function(){o.forEach(function(p){var f=eh.get(p)-1,m=i.get(p)-1;eh.set(p,f),i.set(p,m),f||(Rb.has(p)||p.removeAttribute(n),Rb.delete(p)),m||p.removeAttribute(r)}),Jx--,Jx||(eh=new WeakMap,eh=new WeakMap,Rb=new WeakMap,Db={})}},OD=function(e,t,r){r===void 0&&(r="data-aria-hidden");var n=Array.from(Array.isArray(e)?e:[e]),a=_ve(e);return a?(n.push.apply(n,Array.from(a.querySelectorAll("[aria-live]"))),Dve(n,a,r,"aria-hidden")):function(){return null}},$v="Dialog",[$J,GJ]=Bo($v),[Nve,Po]=$J($v),jJ=e=>{const{__scopeDialog:t,children:r,open:n,defaultOpen:a,onOpenChange:i,modal:o=!0}=e,s=T.useRef(null),l=T.useRef(null),[u,d]=bl({prop:n,defaultProp:a??!1,onChange:i,caller:$v});return x.jsx(Nve,{scope:t,triggerRef:s,contentRef:l,contentId:La(),titleId:La(),descriptionId:La(),open:u,onOpenChange:d,onOpenToggle:T.useCallback(()=>d(p=>!p),[d]),modal:o,children:r})};jJ.displayName=$v;var qJ="DialogTrigger",WJ=T.forwardRef((e,t)=>{const{__scopeDialog:r,...n}=e,a=Po(qJ,r),i=Rr(t,a.triggerRef);return x.jsx(Kt.button,{type:"button","aria-haspopup":"dialog","aria-expanded":a.open,"aria-controls":a.contentId,"data-state":zD(a.open),...n,ref:i,onClick:$t(e.onClick,a.onOpenToggle)})});WJ.displayName=qJ;var BD="DialogPortal",[Lve,VJ]=$J(BD,{forceMount:void 0}),KJ=e=>{const{__scopeDialog:t,forceMount:r,children:n,container:a}=e,i=Po(BD,t);return x.jsx(Lve,{scope:t,forceMount:r,children:T.Children.map(n,o=>x.jsx($i,{present:r||i.open,children:x.jsx(zv,{asChild:!0,container:a,children:o})}))})};KJ.displayName=BD;var cy="DialogOverlay",YJ=T.forwardRef((e,t)=>{const r=VJ(cy,e.__scopeDialog),{forceMount:n=r.forceMount,...a}=e,i=Po(cy,e.__scopeDialog);return i.modal?x.jsx($i,{present:n||i.open,children:x.jsx(Ive,{...a,ref:t})}):null});YJ.displayName=cy;var Mve=Oc("DialogOverlay.RemoveScroll"),Ive=T.forwardRef((e,t)=>{const{__scopeDialog:r,...n}=e,a=Po(cy,r);return x.jsx(Uv,{as:Mve,allowPinchZoom:!0,shards:[a.contentRef],children:x.jsx(Kt.div,{"data-state":zD(a.open),...n,ref:t,style:{pointerEvents:"auto",...n.style}})})}),Bc="DialogContent",JJ=T.forwardRef((e,t)=>{const r=VJ(Bc,e.__scopeDialog),{forceMount:n=r.forceMount,...a}=e,i=Po(Bc,e.__scopeDialog);return x.jsx($i,{present:n||i.open,children:i.modal?x.jsx(Ove,{...a,ref:t}):x.jsx(Bve,{...a,ref:t})})});JJ.displayName=Bc;var Ove=T.forwardRef((e,t)=>{const r=Po(Bc,e.__scopeDialog),n=T.useRef(null),a=Rr(t,r.contentRef,n);return T.useEffect(()=>{const i=n.current;if(i)return OD(i)},[]),x.jsx(XJ,{...e,ref:a,trapFocus:r.open,disableOutsidePointerEvents:!0,onCloseAutoFocus:$t(e.onCloseAutoFocus,i=>{var o;i.preventDefault(),(o=r.triggerRef.current)==null||o.focus()}),onPointerDownOutside:$t(e.onPointerDownOutside,i=>{const o=i.detail.originalEvent,s=o.button===0&&o.ctrlKey===!0;(o.button===2||s)&&i.preventDefault()}),onFocusOutside:$t(e.onFocusOutside,i=>i.preventDefault())})}),Bve=T.forwardRef((e,t)=>{const r=Po(Bc,e.__scopeDialog),n=T.useRef(!1),a=T.useRef(!1);return x.jsx(XJ,{...e,ref:t,trapFocus:!1,disableOutsidePointerEvents:!1,onCloseAutoFocus:i=>{var o,s;(o=e.onCloseAutoFocus)==null||o.call(e,i),i.defaultPrevented||(n.current||(s=r.triggerRef.current)==null||s.focus(),i.preventDefault()),n.current=!1,a.current=!1},onInteractOutside:i=>{var l,u;(l=e.onInteractOutside)==null||l.call(e,i),i.defaultPrevented||(n.current=!0,i.detail.originalEvent.type==="pointerdown"&&(a.current=!0));const o=i.target;((u=r.triggerRef.current)==null?void 0:u.contains(o))&&i.preventDefault(),i.detail.originalEvent.type==="focusin"&&a.current&&i.preventDefault()}})}),XJ=T.forwardRef((e,t)=>{const{__scopeDialog:r,trapFocus:n,onOpenAutoFocus:a,onCloseAutoFocus:i,...o}=e,s=Po(Bc,r),l=T.useRef(null),u=Rr(t,l);return ID(),x.jsxs(x.Fragment,{children:[x.jsx(Pv,{asChild:!0,loop:!0,trapped:n,onMountAutoFocus:a,onUnmountAutoFocus:i,children:x.jsx(Sg,{role:"dialog",id:s.contentId,"aria-describedby":s.descriptionId,"aria-labelledby":s.titleId,"data-state":zD(s.open),...o,ref:u,onDismiss:()=>s.onOpenChange(!1)})}),x.jsxs(x.Fragment,{children:[x.jsx(zve,{titleId:s.titleId}),x.jsx(Uve,{contentRef:l,descriptionId:s.descriptionId})]})]})}),PD="DialogTitle",ZJ=T.forwardRef((e,t)=>{const{__scopeDialog:r,...n}=e,a=Po(PD,r);return x.jsx(Kt.h2,{id:a.titleId,...n,ref:t})});ZJ.displayName=PD;var QJ="DialogDescription",eX=T.forwardRef((e,t)=>{const{__scopeDialog:r,...n}=e,a=Po(QJ,r);return x.jsx(Kt.p,{id:a.descriptionId,...n,ref:t})});eX.displayName=QJ;var tX="DialogClose",rX=T.forwardRef((e,t)=>{const{__scopeDialog:r,...n}=e,a=Po(tX,r);return x.jsx(Kt.button,{type:"button",...n,ref:t,onClick:$t(e.onClick,()=>a.onOpenChange(!1))})});rX.displayName=tX;function zD(e){return e?"open":"closed"}var nX="DialogTitleWarning",[Pve,aX]=fye(nX,{contentName:Bc,titleName:PD,docsSlug:"dialog"}),zve=({titleId:e})=>{const t=aX(nX),r=`\`${t.contentName}\` requires a \`${t.titleName}\` for the component to be accessible for screen reader users.

If you want to hide the \`${t.titleName}\`, you can wrap it with our VisuallyHidden component.

For more information, see https://radix-ui.com/primitives/docs/components/${t.docsSlug}`;return T.useEffect(()=>{e&&(document.getElementById(e)||console.error(r))},[r,e]),null},Hve="DialogDescriptionWarning",Uve=({contentRef:e,descriptionId:t})=>{const n=`Warning: Missing \`Description\` or \`aria-describedby={undefined}\` for {${aX(Hve).contentName}}.`;return T.useEffect(()=>{var i;const a=(i=e.current)==null?void 0:i.getAttribute("aria-describedby");t&&a&&(document.getElementById(t)||console.warn(n))},[n,e,t]),null},HD=jJ,iX=WJ,UD=KJ,Gv=YJ,jv=JJ,$D=ZJ,GD=eX,jD=rX,oX="AlertDialog",[$ve]=Bo(oX,[GJ]),Fl=GJ(),sX=e=>{const{__scopeAlertDialog:t,...r}=e,n=Fl(t);return x.jsx(HD,{...n,...r,modal:!0})};sX.displayName=oX;var Gve="AlertDialogTrigger",jve=T.forwardRef((e,t)=>{const{__scopeAlertDialog:r,...n}=e,a=Fl(r);return x.jsx(iX,{...a,...n,ref:t})});jve.displayName=Gve;var qve="AlertDialogPortal",lX=e=>{const{__scopeAlertDialog:t,...r}=e,n=Fl(t);return x.jsx(UD,{...n,...r})};lX.displayName=qve;var Wve="AlertDialogOverlay",uX=T.forwardRef((e,t)=>{const{__scopeAlertDialog:r,...n}=e,a=Fl(r);return x.jsx(Gv,{...a,...n,ref:t})});uX.displayName=Wve;var Sh="AlertDialogContent",[Vve,Kve]=$ve(Sh),Yve=RJ("AlertDialogContent"),cX=T.forwardRef((e,t)=>{const{__scopeAlertDialog:r,children:n,...a}=e,i=Fl(r),o=T.useRef(null),s=Rr(t,o),l=T.useRef(null);return x.jsx(Pve,{contentName:Sh,titleName:dX,docsSlug:"alert-dialog",children:x.jsx(Vve,{scope:r,cancelRef:l,children:x.jsxs(jv,{role:"alertdialog",...i,...a,ref:s,onOpenAutoFocus:$t(a.onOpenAutoFocus,u=>{var d;u.preventDefault(),(d=l.current)==null||d.focus({preventScroll:!0})}),onPointerDownOutside:u=>u.preventDefault(),onInteractOutside:u=>u.preventDefault(),children:[x.jsx(Yve,{children:n}),x.jsx(Xve,{contentRef:o})]})})})});cX.displayName=Sh;var dX="AlertDialogTitle",hX=T.forwardRef((e,t)=>{const{__scopeAlertDialog:r,...n}=e,a=Fl(r);return x.jsx($D,{...a,...n,ref:t})});hX.displayName=dX;var pX="AlertDialogDescription",fX=T.forwardRef((e,t)=>{const{__scopeAlertDialog:r,...n}=e,a=Fl(r);return x.jsx(GD,{...a,...n,ref:t})});fX.displayName=pX;var Jve="AlertDialogAction",mX=T.forwardRef((e,t)=>{const{__scopeAlertDialog:r,...n}=e,a=Fl(r);return x.jsx(jD,{...a,...n,ref:t})});mX.displayName=Jve;var gX="AlertDialogCancel",bX=T.forwardRef((e,t)=>{const{__scopeAlertDialog:r,...n}=e,{cancelRef:a}=Kve(gX,r),i=Fl(r),o=Rr(t,a);return x.jsx(jD,{...i,...n,ref:o})});bX.displayName=gX;var Xve=({contentRef:e})=>{const t=`\`${Sh}\` requires a description for the component to be accessible for screen reader users.

You can add a description to the \`${Sh}\` by passing a \`${pX}\` component as a child, which also benefits sighted users by adding visible context to the dialog.

Alternatively, you can use your own component as a description by assigning it an \`id\` and passing the same value to the \`aria-describedby\` prop in \`${Sh}\`. If the description is confusing or duplicative for sighted users, you can use the \`@radix-ui/react-visually-hidden\` primitive as a wrapper around your description component.

For more information, see https://radix-ui.com/primitives/docs/components/alert-dialog`;return T.useEffect(()=>{var n;document.getElementById((n=e.current)==null?void 0:n.getAttribute("aria-describedby"))||console.warn(t)},[t,e]),null},Zve=sX,Qve=lX,yX=uX,vX=cX,SX=mX,EX=bX,xX=hX,wX=fX;const q8=e=>typeof e=="boolean"?`${e}`:e===0?"0":e,W8=LY,e2e=(e,t)=>r=>{var n;if((t==null?void 0:t.variants)==null)return W8(e,r==null?void 0:r.class,r==null?void 0:r.className);const{variants:a,defaultVariants:i}=t,o=Object.keys(a).map(u=>{const d=r==null?void 0:r[u],p=i==null?void 0:i[u];if(d===null)return null;const f=q8(d)||q8(p);return a[u][f]}),s=r&&Object.entries(r).reduce((u,d)=>{let[p,f]=d;return f===void 0||(u[p]=f),u},{}),l=t==null||(n=t.compoundVariants)===null||n===void 0?void 0:n.reduce((u,d)=>{let{class:p,className:f,...m}=d;return Object.entries(m).every(b=>{let[y,S]=b;return Array.isArray(S)?S.includes({...i,...s}[y]):{...i,...s}[y]===S})?[...u,p,f]:u},[]);return W8(e,o,l,r==null?void 0:r.class,r==null?void 0:r.className)},t2e=["top","right","bottom","left"],Fu=Math.min,Li=Math.max,dy=Math.round,Nb=Math.floor,ss=e=>({x:e,y:e}),r2e={left:"right",right:"left",bottom:"top",top:"bottom"},n2e={start:"end",end:"start"};function Z_(e,t,r){return Li(e,Fu(t,r))}function yl(e,t){return typeof e=="function"?e(t):e}function vl(e){return e.split("-")[0]}function pp(e){return e.split("-")[1]}function qD(e){return e==="x"?"y":"x"}function WD(e){return e==="y"?"height":"width"}function _u(e){return["top","bottom"].includes(vl(e))?"y":"x"}function VD(e){return qD(_u(e))}function a2e(e,t,r){r===void 0&&(r=!1);const n=pp(e),a=VD(e),i=WD(a);let o=a==="x"?n===(r?"end":"start")?"right":"left":n==="start"?"bottom":"top";return t.reference[i]>t.floating[i]&&(o=hy(o)),[o,hy(o)]}function i2e(e){const t=hy(e);return[Q_(e),t,Q_(t)]}function Q_(e){return e.replace(/start|end/g,t=>n2e[t])}function o2e(e,t,r){const n=["left","right"],a=["right","left"],i=["top","bottom"],o=["bottom","top"];switch(e){case"top":case"bottom":return r?t?a:n:t?n:a;case"left":case"right":return t?i:o;default:return[]}}function s2e(e,t,r,n){const a=pp(e);let i=o2e(vl(e),r==="start",n);return a&&(i=i.map(o=>o+"-"+a),t&&(i=i.concat(i.map(Q_)))),i}function hy(e){return e.replace(/left|right|bottom|top/g,t=>r2e[t])}function l2e(e){return{top:0,right:0,bottom:0,left:0,...e}}function kX(e){return typeof e!="number"?l2e(e):{top:e,right:e,bottom:e,left:e}}function py(e){const{x:t,y:r,width:n,height:a}=e;return{width:n,height:a,top:r,left:t,right:t+n,bottom:r+a,x:t,y:r}}function V8(e,t,r){let{reference:n,floating:a}=e;const i=_u(t),o=VD(t),s=WD(o),l=vl(t),u=i==="y",d=n.x+n.width/2-a.width/2,p=n.y+n.height/2-a.height/2,f=n[s]/2-a[s]/2;let m;switch(l){case"top":m={x:d,y:n.y-a.height};break;case"bottom":m={x:d,y:n.y+n.height};break;case"right":m={x:n.x+n.width,y:p};break;case"left":m={x:n.x-a.width,y:p};break;default:m={x:n.x,y:n.y}}switch(pp(t)){case"start":m[o]-=f*(r&&u?-1:1);break;case"end":m[o]+=f*(r&&u?-1:1);break}return m}const u2e=async(e,t,r)=>{const{placement:n="bottom",strategy:a="absolute",middleware:i=[],platform:o}=r,s=i.filter(Boolean),l=await(o.isRTL==null?void 0:o.isRTL(t));let u=await o.getElementRects({reference:e,floating:t,strategy:a}),{x:d,y:p}=V8(u,n,l),f=n,m={},b=0;for(let y=0;y<s.length;y++){const{name:S,fn:w}=s[y],{x:E,y:k,data:_,reset:R}=await w({x:d,y:p,initialPlacement:n,placement:f,strategy:a,middlewareData:m,rects:u,platform:o,elements:{reference:e,floating:t}});d=E??d,p=k??p,m={...m,[S]:{...m[S],..._}},R&&b<=50&&(b++,typeof R=="object"&&(R.placement&&(f=R.placement),R.rects&&(u=R.rects===!0?await o.getElementRects({reference:e,floating:t,strategy:a}):R.rects),{x:d,y:p}=V8(u,f,l)),y=-1)}return{x:d,y:p,placement:f,strategy:a,middlewareData:m}};async function Bm(e,t){var r;t===void 0&&(t={});const{x:n,y:a,platform:i,rects:o,elements:s,strategy:l}=e,{boundary:u="clippingAncestors",rootBoundary:d="viewport",elementContext:p="floating",altBoundary:f=!1,padding:m=0}=yl(t,e),b=kX(m),S=s[f?p==="floating"?"reference":"floating":p],w=py(await i.getClippingRect({element:(r=await(i.isElement==null?void 0:i.isElement(S)))==null||r?S:S.contextElement||await(i.getDocumentElement==null?void 0:i.getDocumentElement(s.floating)),boundary:u,rootBoundary:d,strategy:l})),E=p==="floating"?{x:n,y:a,width:o.floating.width,height:o.floating.height}:o.reference,k=await(i.getOffsetParent==null?void 0:i.getOffsetParent(s.floating)),_=await(i.isElement==null?void 0:i.isElement(k))?await(i.getScale==null?void 0:i.getScale(k))||{x:1,y:1}:{x:1,y:1},R=py(i.convertOffsetParentRelativeRectToViewportRelativeRect?await i.convertOffsetParentRelativeRectToViewportRelativeRect({elements:s,rect:E,offsetParent:k,strategy:l}):E);return{top:(w.top-R.top+b.top)/_.y,bottom:(R.bottom-w.bottom+b.bottom)/_.y,left:(w.left-R.left+b.left)/_.x,right:(R.right-w.right+b.right)/_.x}}const c2e=e=>({name:"arrow",options:e,async fn(t){const{x:r,y:n,placement:a,rects:i,platform:o,elements:s,middlewareData:l}=t,{element:u,padding:d=0}=yl(e,t)||{};if(u==null)return{};const p=kX(d),f={x:r,y:n},m=VD(a),b=WD(m),y=await o.getDimensions(u),S=m==="y",w=S?"top":"left",E=S?"bottom":"right",k=S?"clientHeight":"clientWidth",_=i.reference[b]+i.reference[m]-f[m]-i.floating[b],R=f[m]-i.reference[m],C=await(o.getOffsetParent==null?void 0:o.getOffsetParent(u));let F=C?C[k]:0;(!F||!await(o.isElement==null?void 0:o.isElement(C)))&&(F=s.floating[k]||i.floating[b]);const L=_/2-R/2,I=F/2-y[b]/2-1,M=Fu(p[w],I),j=Fu(p[E],I),Y=M,W=F-y[b]-j,$=F/2-y[b]/2+L,X=Z_(Y,$,W),z=!l.arrow&&pp(a)!=null&&$!==X&&i.reference[b]/2-($<Y?M:j)-y[b]/2<0,H=z?$<Y?$-Y:$-W:0;return{[m]:f[m]+H,data:{[m]:X,centerOffset:$-X-H,...z&&{alignmentOffset:H}},reset:z}}}),d2e=function(e){return e===void 0&&(e={}),{name:"flip",options:e,async fn(t){var r,n;const{placement:a,middlewareData:i,rects:o,initialPlacement:s,platform:l,elements:u}=t,{mainAxis:d=!0,crossAxis:p=!0,fallbackPlacements:f,fallbackStrategy:m="bestFit",fallbackAxisSideDirection:b="none",flipAlignment:y=!0,...S}=yl(e,t);if((r=i.arrow)!=null&&r.alignmentOffset)return{};const w=vl(a),E=_u(s),k=vl(s)===s,_=await(l.isRTL==null?void 0:l.isRTL(u.floating)),R=f||(k||!y?[hy(s)]:i2e(s)),C=b!=="none";!f&&C&&R.push(...s2e(s,y,b,_));const F=[s,...R],L=await Bm(t,S),I=[];let M=((n=i.flip)==null?void 0:n.overflows)||[];if(d&&I.push(L[w]),p){const $=a2e(a,o,_);I.push(L[$[0]],L[$[1]])}if(M=[...M,{placement:a,overflows:I}],!I.every($=>$<=0)){var j,Y;const $=(((j=i.flip)==null?void 0:j.index)||0)+1,X=F[$];if(X)return{data:{index:$,overflows:M},reset:{placement:X}};let z=(Y=M.filter(H=>H.overflows[0]<=0).sort((H,B)=>H.overflows[1]-B.overflows[1])[0])==null?void 0:Y.placement;if(!z)switch(m){case"bestFit":{var W;const H=(W=M.filter(B=>{if(C){const ee=_u(B.placement);return ee===E||ee==="y"}return!0}).map(B=>[B.placement,B.overflows.filter(ee=>ee>0).reduce((ee,J)=>ee+J,0)]).sort((B,ee)=>B[1]-ee[1])[0])==null?void 0:W[0];H&&(z=H);break}case"initialPlacement":z=s;break}if(a!==z)return{reset:{placement:z}}}return{}}}};function K8(e,t){return{top:e.top-t.height,right:e.right-t.width,bottom:e.bottom-t.height,left:e.left-t.width}}function Y8(e){return t2e.some(t=>e[t]>=0)}const h2e=function(e){return e===void 0&&(e={}),{name:"hide",options:e,async fn(t){const{rects:r}=t,{strategy:n="referenceHidden",...a}=yl(e,t);switch(n){case"referenceHidden":{const i=await Bm(t,{...a,elementContext:"reference"}),o=K8(i,r.reference);return{data:{referenceHiddenOffsets:o,referenceHidden:Y8(o)}}}case"escaped":{const i=await Bm(t,{...a,altBoundary:!0}),o=K8(i,r.floating);return{data:{escapedOffsets:o,escaped:Y8(o)}}}default:return{}}}}};async function p2e(e,t){const{placement:r,platform:n,elements:a}=e,i=await(n.isRTL==null?void 0:n.isRTL(a.floating)),o=vl(r),s=pp(r),l=_u(r)==="y",u=["left","top"].includes(o)?-1:1,d=i&&l?-1:1,p=yl(t,e);let{mainAxis:f,crossAxis:m,alignmentAxis:b}=typeof p=="number"?{mainAxis:p,crossAxis:0,alignmentAxis:null}:{mainAxis:p.mainAxis||0,crossAxis:p.crossAxis||0,alignmentAxis:p.alignmentAxis};return s&&typeof b=="number"&&(m=s==="end"?b*-1:b),l?{x:m*d,y:f*u}:{x:f*u,y:m*d}}const f2e=function(e){return e===void 0&&(e=0),{name:"offset",options:e,async fn(t){var r,n;const{x:a,y:i,placement:o,middlewareData:s}=t,l=await p2e(t,e);return o===((r=s.offset)==null?void 0:r.placement)&&(n=s.arrow)!=null&&n.alignmentOffset?{}:{x:a+l.x,y:i+l.y,data:{...l,placement:o}}}}},m2e=function(e){return e===void 0&&(e={}),{name:"shift",options:e,async fn(t){const{x:r,y:n,placement:a}=t,{mainAxis:i=!0,crossAxis:o=!1,limiter:s={fn:S=>{let{x:w,y:E}=S;return{x:w,y:E}}},...l}=yl(e,t),u={x:r,y:n},d=await Bm(t,l),p=_u(vl(a)),f=qD(p);let m=u[f],b=u[p];if(i){const S=f==="y"?"top":"left",w=f==="y"?"bottom":"right",E=m+d[S],k=m-d[w];m=Z_(E,m,k)}if(o){const S=p==="y"?"top":"left",w=p==="y"?"bottom":"right",E=b+d[S],k=b-d[w];b=Z_(E,b,k)}const y=s.fn({...t,[f]:m,[p]:b});return{...y,data:{x:y.x-r,y:y.y-n,enabled:{[f]:i,[p]:o}}}}}},g2e=function(e){return e===void 0&&(e={}),{options:e,fn(t){const{x:r,y:n,placement:a,rects:i,middlewareData:o}=t,{offset:s=0,mainAxis:l=!0,crossAxis:u=!0}=yl(e,t),d={x:r,y:n},p=_u(a),f=qD(p);let m=d[f],b=d[p];const y=yl(s,t),S=typeof y=="number"?{mainAxis:y,crossAxis:0}:{mainAxis:0,crossAxis:0,...y};if(l){const k=f==="y"?"height":"width",_=i.reference[f]-i.floating[k]+S.mainAxis,R=i.reference[f]+i.reference[k]-S.mainAxis;m<_?m=_:m>R&&(m=R)}if(u){var w,E;const k=f==="y"?"width":"height",_=["top","left"].includes(vl(a)),R=i.reference[p]-i.floating[k]+(_&&((w=o.offset)==null?void 0:w[p])||0)+(_?0:S.crossAxis),C=i.reference[p]+i.reference[k]+(_?0:((E=o.offset)==null?void 0:E[p])||0)-(_?S.crossAxis:0);b<R?b=R:b>C&&(b=C)}return{[f]:m,[p]:b}}}},b2e=function(e){return e===void 0&&(e={}),{name:"size",options:e,async fn(t){var r,n;const{placement:a,rects:i,platform:o,elements:s}=t,{apply:l=()=>{},...u}=yl(e,t),d=await Bm(t,u),p=vl(a),f=pp(a),m=_u(a)==="y",{width:b,height:y}=i.floating;let S,w;p==="top"||p==="bottom"?(S=p,w=f===(await(o.isRTL==null?void 0:o.isRTL(s.floating))?"start":"end")?"left":"right"):(w=p,S=f==="end"?"top":"bottom");const E=y-d.top-d.bottom,k=b-d.left-d.right,_=Fu(y-d[S],E),R=Fu(b-d[w],k),C=!t.middlewareData.shift;let F=_,L=R;if((r=t.middlewareData.shift)!=null&&r.enabled.x&&(L=k),(n=t.middlewareData.shift)!=null&&n.enabled.y&&(F=E),C&&!f){const M=Li(d.left,0),j=Li(d.right,0),Y=Li(d.top,0),W=Li(d.bottom,0);m?L=b-2*(M!==0||j!==0?M+j:Li(d.left,d.right)):F=y-2*(Y!==0||W!==0?Y+W:Li(d.top,d.bottom))}await l({...t,availableWidth:L,availableHeight:F});const I=await o.getDimensions(s.floating);return b!==I.width||y!==I.height?{reset:{rects:!0}}:{}}}};function qv(){return typeof window<"u"}function fp(e){return CX(e)?(e.nodeName||"").toLowerCase():"#document"}function Bi(e){var t;return(e==null||(t=e.ownerDocument)==null?void 0:t.defaultView)||window}function gs(e){var t;return(t=(CX(e)?e.ownerDocument:e.document)||window.document)==null?void 0:t.documentElement}function CX(e){return qv()?e instanceof Node||e instanceof Bi(e).Node:!1}function Mo(e){return qv()?e instanceof Element||e instanceof Bi(e).Element:!1}function us(e){return qv()?e instanceof HTMLElement||e instanceof Bi(e).HTMLElement:!1}function J8(e){return!qv()||typeof ShadowRoot>"u"?!1:e instanceof ShadowRoot||e instanceof Bi(e).ShadowRoot}function Eg(e){const{overflow:t,overflowX:r,overflowY:n,display:a}=Io(e);return/auto|scroll|overlay|hidden|clip/.test(t+n+r)&&!["inline","contents"].includes(a)}function y2e(e){return["table","td","th"].includes(fp(e))}function Wv(e){return[":popover-open",":modal"].some(t=>{try{return e.matches(t)}catch{return!1}})}function KD(e){const t=YD(),r=Mo(e)?Io(e):e;return["transform","translate","scale","rotate","perspective"].some(n=>r[n]?r[n]!=="none":!1)||(r.containerType?r.containerType!=="normal":!1)||!t&&(r.backdropFilter?r.backdropFilter!=="none":!1)||!t&&(r.filter?r.filter!=="none":!1)||["transform","translate","scale","rotate","perspective","filter"].some(n=>(r.willChange||"").includes(n))||["paint","layout","strict","content"].some(n=>(r.contain||"").includes(n))}function v2e(e){let t=Ru(e);for(;us(t)&&!Ph(t);){if(KD(t))return t;if(Wv(t))return null;t=Ru(t)}return null}function YD(){return typeof CSS>"u"||!CSS.supports?!1:CSS.supports("-webkit-backdrop-filter","none")}function Ph(e){return["html","body","#document"].includes(fp(e))}function Io(e){return Bi(e).getComputedStyle(e)}function Vv(e){return Mo(e)?{scrollLeft:e.scrollLeft,scrollTop:e.scrollTop}:{scrollLeft:e.scrollX,scrollTop:e.scrollY}}function Ru(e){if(fp(e)==="html")return e;const t=e.assignedSlot||e.parentNode||J8(e)&&e.host||gs(e);return J8(t)?t.host:t}function TX(e){const t=Ru(e);return Ph(t)?e.ownerDocument?e.ownerDocument.body:e.body:us(t)&&Eg(t)?t:TX(t)}function Pm(e,t,r){var n;t===void 0&&(t=[]),r===void 0&&(r=!0);const a=TX(e),i=a===((n=e.ownerDocument)==null?void 0:n.body),o=Bi(a);if(i){const s=eR(o);return t.concat(o,o.visualViewport||[],Eg(a)?a:[],s&&r?Pm(s):[])}return t.concat(a,Pm(a,[],r))}function eR(e){return e.parent&&Object.getPrototypeOf(e.parent)?e.frameElement:null}function AX(e){const t=Io(e);let r=parseFloat(t.width)||0,n=parseFloat(t.height)||0;const a=us(e),i=a?e.offsetWidth:r,o=a?e.offsetHeight:n,s=dy(r)!==i||dy(n)!==o;return s&&(r=i,n=o),{width:r,height:n,$:s}}function JD(e){return Mo(e)?e:e.contextElement}function Eh(e){const t=JD(e);if(!us(t))return ss(1);const r=t.getBoundingClientRect(),{width:n,height:a,$:i}=AX(t);let o=(i?dy(r.width):r.width)/n,s=(i?dy(r.height):r.height)/a;return(!o||!Number.isFinite(o))&&(o=1),(!s||!Number.isFinite(s))&&(s=1),{x:o,y:s}}const S2e=ss(0);function FX(e){const t=Bi(e);return!YD()||!t.visualViewport?S2e:{x:t.visualViewport.offsetLeft,y:t.visualViewport.offsetTop}}function E2e(e,t,r){return t===void 0&&(t=!1),!r||t&&r!==Bi(e)?!1:t}function Pc(e,t,r,n){t===void 0&&(t=!1),r===void 0&&(r=!1);const a=e.getBoundingClientRect(),i=JD(e);let o=ss(1);t&&(n?Mo(n)&&(o=Eh(n)):o=Eh(e));const s=E2e(i,r,n)?FX(i):ss(0);let l=(a.left+s.x)/o.x,u=(a.top+s.y)/o.y,d=a.width/o.x,p=a.height/o.y;if(i){const f=Bi(i),m=n&&Mo(n)?Bi(n):n;let b=f,y=eR(b);for(;y&&n&&m!==b;){const S=Eh(y),w=y.getBoundingClientRect(),E=Io(y),k=w.left+(y.clientLeft+parseFloat(E.paddingLeft))*S.x,_=w.top+(y.clientTop+parseFloat(E.paddingTop))*S.y;l*=S.x,u*=S.y,d*=S.x,p*=S.y,l+=k,u+=_,b=Bi(y),y=eR(b)}}return py({width:d,height:p,x:l,y:u})}function XD(e,t){const r=Vv(e).scrollLeft;return t?t.left+r:Pc(gs(e)).left+r}function _X(e,t,r){r===void 0&&(r=!1);const n=e.getBoundingClientRect(),a=n.left+t.scrollLeft-(r?0:XD(e,n)),i=n.top+t.scrollTop;return{x:a,y:i}}function x2e(e){let{elements:t,rect:r,offsetParent:n,strategy:a}=e;const i=a==="fixed",o=gs(n),s=t?Wv(t.floating):!1;if(n===o||s&&i)return r;let l={scrollLeft:0,scrollTop:0},u=ss(1);const d=ss(0),p=us(n);if((p||!p&&!i)&&((fp(n)!=="body"||Eg(o))&&(l=Vv(n)),us(n))){const m=Pc(n);u=Eh(n),d.x=m.x+n.clientLeft,d.y=m.y+n.clientTop}const f=o&&!p&&!i?_X(o,l,!0):ss(0);return{width:r.width*u.x,height:r.height*u.y,x:r.x*u.x-l.scrollLeft*u.x+d.x+f.x,y:r.y*u.y-l.scrollTop*u.y+d.y+f.y}}function w2e(e){return Array.from(e.getClientRects())}function k2e(e){const t=gs(e),r=Vv(e),n=e.ownerDocument.body,a=Li(t.scrollWidth,t.clientWidth,n.scrollWidth,n.clientWidth),i=Li(t.scrollHeight,t.clientHeight,n.scrollHeight,n.clientHeight);let o=-r.scrollLeft+XD(e);const s=-r.scrollTop;return Io(n).direction==="rtl"&&(o+=Li(t.clientWidth,n.clientWidth)-a),{width:a,height:i,x:o,y:s}}function C2e(e,t){const r=Bi(e),n=gs(e),a=r.visualViewport;let i=n.clientWidth,o=n.clientHeight,s=0,l=0;if(a){i=a.width,o=a.height;const u=YD();(!u||u&&t==="fixed")&&(s=a.offsetLeft,l=a.offsetTop)}return{width:i,height:o,x:s,y:l}}function T2e(e,t){const r=Pc(e,!0,t==="fixed"),n=r.top+e.clientTop,a=r.left+e.clientLeft,i=us(e)?Eh(e):ss(1),o=e.clientWidth*i.x,s=e.clientHeight*i.y,l=a*i.x,u=n*i.y;return{width:o,height:s,x:l,y:u}}function X8(e,t,r){let n;if(t==="viewport")n=C2e(e,r);else if(t==="document")n=k2e(gs(e));else if(Mo(t))n=T2e(t,r);else{const a=FX(e);n={x:t.x-a.x,y:t.y-a.y,width:t.width,height:t.height}}return py(n)}function RX(e,t){const r=Ru(e);return r===t||!Mo(r)||Ph(r)?!1:Io(r).position==="fixed"||RX(r,t)}function A2e(e,t){const r=t.get(e);if(r)return r;let n=Pm(e,[],!1).filter(s=>Mo(s)&&fp(s)!=="body"),a=null;const i=Io(e).position==="fixed";let o=i?Ru(e):e;for(;Mo(o)&&!Ph(o);){const s=Io(o),l=KD(o);!l&&s.position==="fixed"&&(a=null),(i?!l&&!a:!l&&s.position==="static"&&!!a&&["absolute","fixed"].includes(a.position)||Eg(o)&&!l&&RX(e,o))?n=n.filter(d=>d!==o):a=s,o=Ru(o)}return t.set(e,n),n}function F2e(e){let{element:t,boundary:r,rootBoundary:n,strategy:a}=e;const o=[...r==="clippingAncestors"?Wv(t)?[]:A2e(t,this._c):[].concat(r),n],s=o[0],l=o.reduce((u,d)=>{const p=X8(t,d,a);return u.top=Li(p.top,u.top),u.right=Fu(p.right,u.right),u.bottom=Fu(p.bottom,u.bottom),u.left=Li(p.left,u.left),u},X8(t,s,a));return{width:l.right-l.left,height:l.bottom-l.top,x:l.left,y:l.top}}function _2e(e){const{width:t,height:r}=AX(e);return{width:t,height:r}}function R2e(e,t,r){const n=us(t),a=gs(t),i=r==="fixed",o=Pc(e,!0,i,t);let s={scrollLeft:0,scrollTop:0};const l=ss(0);if(n||!n&&!i)if((fp(t)!=="body"||Eg(a))&&(s=Vv(t)),n){const f=Pc(t,!0,i,t);l.x=f.x+t.clientLeft,l.y=f.y+t.clientTop}else a&&(l.x=XD(a));const u=a&&!n&&!i?_X(a,s):ss(0),d=o.left+s.scrollLeft-l.x-u.x,p=o.top+s.scrollTop-l.y-u.y;return{x:d,y:p,width:o.width,height:o.height}}function Xx(e){return Io(e).position==="static"}function Z8(e,t){if(!us(e)||Io(e).position==="fixed")return null;if(t)return t(e);let r=e.offsetParent;return gs(e)===r&&(r=r.ownerDocument.body),r}function DX(e,t){const r=Bi(e);if(Wv(e))return r;if(!us(e)){let a=Ru(e);for(;a&&!Ph(a);){if(Mo(a)&&!Xx(a))return a;a=Ru(a)}return r}let n=Z8(e,t);for(;n&&y2e(n)&&Xx(n);)n=Z8(n,t);return n&&Ph(n)&&Xx(n)&&!KD(n)?r:n||v2e(e)||r}const D2e=async function(e){const t=this.getOffsetParent||DX,r=this.getDimensions,n=await r(e.floating);return{reference:R2e(e.reference,await t(e.floating),e.strategy),floating:{x:0,y:0,width:n.width,height:n.height}}};function N2e(e){return Io(e).direction==="rtl"}const L2e={convertOffsetParentRelativeRectToViewportRelativeRect:x2e,getDocumentElement:gs,getClippingRect:F2e,getOffsetParent:DX,getElementRects:D2e,getClientRects:w2e,getDimensions:_2e,getScale:Eh,isElement:Mo,isRTL:N2e};function NX(e,t){return e.x===t.x&&e.y===t.y&&e.width===t.width&&e.height===t.height}function M2e(e,t){let r=null,n;const a=gs(e);function i(){var s;clearTimeout(n),(s=r)==null||s.disconnect(),r=null}function o(s,l){s===void 0&&(s=!1),l===void 0&&(l=1),i();const u=e.getBoundingClientRect(),{left:d,top:p,width:f,height:m}=u;if(s||t(),!f||!m)return;const b=Nb(p),y=Nb(a.clientWidth-(d+f)),S=Nb(a.clientHeight-(p+m)),w=Nb(d),k={rootMargin:-b+"px "+-y+"px "+-S+"px "+-w+"px",threshold:Li(0,Fu(1,l))||1};let _=!0;function R(C){const F=C[0].intersectionRatio;if(F!==l){if(!_)return o();F?o(!1,F):n=setTimeout(()=>{o(!1,1e-7)},1e3)}F===1&&!NX(u,e.getBoundingClientRect())&&o(),_=!1}try{r=new IntersectionObserver(R,{...k,root:a.ownerDocument})}catch{r=new IntersectionObserver(R,k)}r.observe(e)}return o(!0),i}function I2e(e,t,r,n){n===void 0&&(n={});const{ancestorScroll:a=!0,ancestorResize:i=!0,elementResize:o=typeof ResizeObserver=="function",layoutShift:s=typeof IntersectionObserver=="function",animationFrame:l=!1}=n,u=JD(e),d=a||i?[...u?Pm(u):[],...Pm(t)]:[];d.forEach(w=>{a&&w.addEventListener("scroll",r,{passive:!0}),i&&w.addEventListener("resize",r)});const p=u&&s?M2e(u,r):null;let f=-1,m=null;o&&(m=new ResizeObserver(w=>{let[E]=w;E&&E.target===u&&m&&(m.unobserve(t),cancelAnimationFrame(f),f=requestAnimationFrame(()=>{var k;(k=m)==null||k.observe(t)})),r()}),u&&!l&&m.observe(u),m.observe(t));let b,y=l?Pc(e):null;l&&S();function S(){const w=Pc(e);y&&!NX(y,w)&&r(),y=w,b=requestAnimationFrame(S)}return r(),()=>{var w;d.forEach(E=>{a&&E.removeEventListener("scroll",r),i&&E.removeEventListener("resize",r)}),p==null||p(),(w=m)==null||w.disconnect(),m=null,l&&cancelAnimationFrame(b)}}const O2e=f2e,B2e=m2e,P2e=d2e,z2e=b2e,H2e=h2e,Q8=c2e,U2e=g2e,$2e=(e,t,r)=>{const n=new Map,a={platform:L2e,...r},i={...a.platform,_c:n};return u2e(e,t,{...a,platform:i})};var C1=typeof document<"u"?T.useLayoutEffect:T.useEffect;function fy(e,t){if(e===t)return!0;if(typeof e!=typeof t)return!1;if(typeof e=="function"&&e.toString()===t.toString())return!0;let r,n,a;if(e&&t&&typeof e=="object"){if(Array.isArray(e)){if(r=e.length,r!==t.length)return!1;for(n=r;n--!==0;)if(!fy(e[n],t[n]))return!1;return!0}if(a=Object.keys(e),r=a.length,r!==Object.keys(t).length)return!1;for(n=r;n--!==0;)if(!{}.hasOwnProperty.call(t,a[n]))return!1;for(n=r;n--!==0;){const i=a[n];if(!(i==="_owner"&&e.$$typeof)&&!fy(e[i],t[i]))return!1}return!0}return e!==e&&t!==t}function LX(e){return typeof window>"u"?1:(e.ownerDocument.defaultView||window).devicePixelRatio||1}function e7(e,t){const r=LX(e);return Math.round(t*r)/r}function Zx(e){const t=T.useRef(e);return C1(()=>{t.current=e}),t}function G2e(e){e===void 0&&(e={});const{placement:t="bottom",strategy:r="absolute",middleware:n=[],platform:a,elements:{reference:i,floating:o}={},transform:s=!0,whileElementsMounted:l,open:u}=e,[d,p]=T.useState({x:0,y:0,strategy:r,placement:t,middlewareData:{},isPositioned:!1}),[f,m]=T.useState(n);fy(f,n)||m(n);const[b,y]=T.useState(null),[S,w]=T.useState(null),E=T.useCallback(B=>{B!==C.current&&(C.current=B,y(B))},[]),k=T.useCallback(B=>{B!==F.current&&(F.current=B,w(B))},[]),_=i||b,R=o||S,C=T.useRef(null),F=T.useRef(null),L=T.useRef(d),I=l!=null,M=Zx(l),j=Zx(a),Y=Zx(u),W=T.useCallback(()=>{if(!C.current||!F.current)return;const B={placement:t,strategy:r,middleware:f};j.current&&(B.platform=j.current),$2e(C.current,F.current,B).then(ee=>{const J={...ee,isPositioned:Y.current!==!1};$.current&&!fy(L.current,J)&&(L.current=J,mg.flushSync(()=>{p(J)}))})},[f,t,r,j,Y]);C1(()=>{u===!1&&L.current.isPositioned&&(L.current.isPositioned=!1,p(B=>({...B,isPositioned:!1})))},[u]);const $=T.useRef(!1);C1(()=>($.current=!0,()=>{$.current=!1}),[]),C1(()=>{if(_&&(C.current=_),R&&(F.current=R),_&&R){if(M.current)return M.current(_,R,W);W()}},[_,R,W,M,I]);const X=T.useMemo(()=>({reference:C,floating:F,setReference:E,setFloating:k}),[E,k]),z=T.useMemo(()=>({reference:_,floating:R}),[_,R]),H=T.useMemo(()=>{const B={position:r,left:0,top:0};if(!z.floating)return B;const ee=e7(z.floating,d.x),J=e7(z.floating,d.y);return s?{...B,transform:"translate("+ee+"px, "+J+"px)",...LX(z.floating)>=1.5&&{willChange:"transform"}}:{position:r,left:ee,top:J}},[r,s,z.floating,d.x,d.y]);return T.useMemo(()=>({...d,update:W,refs:X,elements:z,floatingStyles:H}),[d,W,X,z,H])}const j2e=e=>{function t(r){return{}.hasOwnProperty.call(r,"current")}return{name:"arrow",options:e,fn(r){const{element:n,padding:a}=typeof e=="function"?e(r):e;return n&&t(n)?n.current!=null?Q8({element:n.current,padding:a}).fn(r):{}:n?Q8({element:n,padding:a}).fn(r):{}}}},q2e=(e,t)=>({...O2e(e),options:[e,t]}),W2e=(e,t)=>({...B2e(e),options:[e,t]}),V2e=(e,t)=>({...U2e(e),options:[e,t]}),K2e=(e,t)=>({...P2e(e),options:[e,t]}),Y2e=(e,t)=>({...z2e(e),options:[e,t]}),J2e=(e,t)=>({...H2e(e),options:[e,t]}),X2e=(e,t)=>({...j2e(e),options:[e,t]});var Z2e="Arrow",MX=T.forwardRef((e,t)=>{const{children:r,width:n=10,height:a=5,...i}=e;return x.jsx(Kt.svg,{...i,ref:t,width:n,height:a,viewBox:"0 0 30 10",preserveAspectRatio:"none",children:e.asChild?r:x.jsx("polygon",{points:"0,0 30,0 15,10"})})});MX.displayName=Z2e;var Q2e=MX;function IX(e){const[t,r]=T.useState(void 0);return Sa(()=>{if(e){r({width:e.offsetWidth,height:e.offsetHeight});const n=new ResizeObserver(a=>{if(!Array.isArray(a)||!a.length)return;const i=a[0];let o,s;if("borderBoxSize"in i){const l=i.borderBoxSize,u=Array.isArray(l)?l[0]:l;o=u.inlineSize,s=u.blockSize}else o=e.offsetWidth,s=e.offsetHeight;r({width:o,height:s})});return n.observe(e,{box:"border-box"}),()=>n.unobserve(e)}else r(void 0)},[e]),t}var ZD="Popper",[OX,mp]=Bo(ZD),[eSe,BX]=OX(ZD),PX=e=>{const{__scopePopper:t,children:r}=e,[n,a]=T.useState(null);return x.jsx(eSe,{scope:t,anchor:n,onAnchorChange:a,children:r})};PX.displayName=ZD;var zX="PopperAnchor",HX=T.forwardRef((e,t)=>{const{__scopePopper:r,virtualRef:n,...a}=e,i=BX(zX,r),o=T.useRef(null),s=Rr(t,o),l=T.useRef(null);return T.useEffect(()=>{const u=l.current;l.current=(n==null?void 0:n.current)||o.current,u!==l.current&&i.onAnchorChange(l.current)}),n?null:x.jsx(Kt.div,{...a,ref:s})});HX.displayName=zX;var QD="PopperContent",[tSe,rSe]=OX(QD),UX=T.forwardRef((e,t)=>{var ae,se,Ee,fe,Me,Oe;const{__scopePopper:r,side:n="bottom",sideOffset:a=0,align:i="center",alignOffset:o=0,arrowPadding:s=0,avoidCollisions:l=!0,collisionBoundary:u=[],collisionPadding:d=0,sticky:p="partial",hideWhenDetached:f=!1,updatePositionStrategy:m="optimized",onPlaced:b,...y}=e,S=BX(QD,r),[w,E]=T.useState(null),k=Rr(t,xe=>E(xe)),[_,R]=T.useState(null),C=IX(_),F=(C==null?void 0:C.width)??0,L=(C==null?void 0:C.height)??0,I=n+(i!=="center"?"-"+i:""),M=typeof d=="number"?d:{top:0,right:0,bottom:0,left:0,...d},j=Array.isArray(u)?u:[u],Y=j.length>0,W={padding:M,boundary:j.filter(aSe),altBoundary:Y},{refs:$,floatingStyles:X,placement:z,isPositioned:H,middlewareData:B}=G2e({strategy:"fixed",placement:I,whileElementsMounted:(...xe)=>I2e(...xe,{animationFrame:m==="always"}),elements:{reference:S.anchor},middleware:[q2e({mainAxis:a+L,alignmentAxis:o}),l&&W2e({mainAxis:!0,crossAxis:!1,limiter:p==="partial"?V2e():void 0,...W}),l&&K2e({...W}),Y2e({...W,apply:({elements:xe,rects:Te,availableWidth:Ne,availableHeight:he})=>{const{width:Ze,height:Et}=Te.reference,qe=xe.floating.style;qe.setProperty("--radix-popper-available-width",`${Ne}px`),qe.setProperty("--radix-popper-available-height",`${he}px`),qe.setProperty("--radix-popper-anchor-width",`${Ze}px`),qe.setProperty("--radix-popper-anchor-height",`${Et}px`)}}),_&&X2e({element:_,padding:s}),iSe({arrowWidth:F,arrowHeight:L}),f&&J2e({strategy:"referenceHidden",...W})]}),[ee,J]=jX(z),K=Ga(b);Sa(()=>{H&&(K==null||K())},[H,K]);const G=(ae=B.arrow)==null?void 0:ae.x,P=(se=B.arrow)==null?void 0:se.y,ne=((Ee=B.arrow)==null?void 0:Ee.centerOffset)!==0,[oe,pe]=T.useState();return Sa(()=>{w&&pe(window.getComputedStyle(w).zIndex)},[w]),x.jsx("div",{ref:$.setFloating,"data-radix-popper-content-wrapper":"",style:{...X,transform:H?X.transform:"translate(0, -200%)",minWidth:"max-content",zIndex:oe,"--radix-popper-transform-origin":[(fe=B.transformOrigin)==null?void 0:fe.x,(Me=B.transformOrigin)==null?void 0:Me.y].join(" "),...((Oe=B.hide)==null?void 0:Oe.referenceHidden)&&{visibility:"hidden",pointerEvents:"none"}},dir:e.dir,children:x.jsx(tSe,{scope:r,placedSide:ee,onArrowChange:R,arrowX:G,arrowY:P,shouldHideArrow:ne,children:x.jsx(Kt.div,{"data-side":ee,"data-align":J,...y,ref:k,style:{...y.style,animation:H?void 0:"none"}})})})});UX.displayName=QD;var $X="PopperArrow",nSe={top:"bottom",right:"left",bottom:"top",left:"right"},GX=T.forwardRef(function(t,r){const{__scopePopper:n,...a}=t,i=rSe($X,n),o=nSe[i.placedSide];return x.jsx("span",{ref:i.onArrowChange,style:{position:"absolute",left:i.arrowX,top:i.arrowY,[o]:0,transformOrigin:{top:"",right:"0 0",bottom:"center 0",left:"100% 0"}[i.placedSide],transform:{top:"translateY(100%)",right:"translateY(50%) rotate(90deg) translateX(-50%)",bottom:"rotate(180deg)",left:"translateY(50%) rotate(-90deg) translateX(50%)"}[i.placedSide],visibility:i.shouldHideArrow?"hidden":void 0},children:x.jsx(Q2e,{...a,ref:r,style:{...a.style,display:"block"}})})});GX.displayName=$X;function aSe(e){return e!==null}var iSe=e=>({name:"transformOrigin",options:e,fn(t){var S,w,E;const{placement:r,rects:n,middlewareData:a}=t,o=((S=a.arrow)==null?void 0:S.centerOffset)!==0,s=o?0:e.arrowWidth,l=o?0:e.arrowHeight,[u,d]=jX(r),p={start:"0%",center:"50%",end:"100%"}[d],f=(((w=a.arrow)==null?void 0:w.x)??0)+s/2,m=(((E=a.arrow)==null?void 0:E.y)??0)+l/2;let b="",y="";return u==="bottom"?(b=o?p:`${f}px`,y=`${-l}px`):u==="top"?(b=o?p:`${f}px`,y=`${n.floating.height+l}px`):u==="right"?(b=`${-l}px`,y=o?p:`${m}px`):u==="left"&&(b=`${n.floating.width+l}px`,y=o?p:`${m}px`),{data:{x:b,y}}}});function jX(e){const[t,r="center"]=e.split("-");return[t,r]}var e5=PX,Kv=HX,t5=UX,r5=GX,qX=Object.freeze({position:"absolute",border:0,width:1,height:1,padding:0,margin:-1,overflow:"hidden",clip:"rect(0, 0, 0, 0)",whiteSpace:"nowrap",wordWrap:"normal"}),oSe="VisuallyHidden",WX=T.forwardRef((e,t)=>x.jsx(Kt.span,{...e,ref:t,style:{...qX,...e.style}}));WX.displayName=oSe;var sSe=WX,[Yv]=Bo("Tooltip",[mp]),Jv=mp(),VX="TooltipProvider",lSe=700,tR="tooltip.open",[uSe,n5]=Yv(VX),KX=e=>{const{__scopeTooltip:t,delayDuration:r=lSe,skipDelayDuration:n=300,disableHoverableContent:a=!1,children:i}=e,o=T.useRef(!0),s=T.useRef(!1),l=T.useRef(0);return T.useEffect(()=>{const u=l.current;return()=>window.clearTimeout(u)},[]),x.jsx(uSe,{scope:t,isOpenDelayedRef:o,delayDuration:r,onOpen:T.useCallback(()=>{window.clearTimeout(l.current),o.current=!1},[]),onClose:T.useCallback(()=>{window.clearTimeout(l.current),l.current=window.setTimeout(()=>o.current=!0,n)},[n]),isPointerInTransitRef:s,onPointerInTransitChange:T.useCallback(u=>{s.current=u},[]),disableHoverableContent:a,children:i})};KX.displayName=VX;var zm="Tooltip",[cSe,Xv]=Yv(zm),YX=e=>{const{__scopeTooltip:t,children:r,open:n,defaultOpen:a,onOpenChange:i,disableHoverableContent:o,delayDuration:s}=e,l=n5(zm,e.__scopeTooltip),u=Jv(t),[d,p]=T.useState(null),f=La(),m=T.useRef(0),b=o??l.disableHoverableContent,y=s??l.delayDuration,S=T.useRef(!1),[w,E]=bl({prop:n,defaultProp:a??!1,onChange:F=>{F?(l.onOpen(),document.dispatchEvent(new CustomEvent(tR))):l.onClose(),i==null||i(F)},caller:zm}),k=T.useMemo(()=>w?S.current?"delayed-open":"instant-open":"closed",[w]),_=T.useCallback(()=>{window.clearTimeout(m.current),m.current=0,S.current=!1,E(!0)},[E]),R=T.useCallback(()=>{window.clearTimeout(m.current),m.current=0,E(!1)},[E]),C=T.useCallback(()=>{window.clearTimeout(m.current),m.current=window.setTimeout(()=>{S.current=!0,E(!0),m.current=0},y)},[y,E]);return T.useEffect(()=>()=>{m.current&&(window.clearTimeout(m.current),m.current=0)},[]),x.jsx(e5,{...u,children:x.jsx(cSe,{scope:t,contentId:f,open:w,stateAttribute:k,trigger:d,onTriggerChange:p,onTriggerEnter:T.useCallback(()=>{l.isOpenDelayedRef.current?C():_()},[l.isOpenDelayedRef,C,_]),onTriggerLeave:T.useCallback(()=>{b?R():(window.clearTimeout(m.current),m.current=0)},[R,b]),onOpen:_,onClose:R,disableHoverableContent:b,children:r})})};YX.displayName=zm;var rR="TooltipTrigger",JX=T.forwardRef((e,t)=>{const{__scopeTooltip:r,...n}=e,a=Xv(rR,r),i=n5(rR,r),o=Jv(r),s=T.useRef(null),l=Rr(t,s,a.onTriggerChange),u=T.useRef(!1),d=T.useRef(!1),p=T.useCallback(()=>u.current=!1,[]);return T.useEffect(()=>()=>document.removeEventListener("pointerup",p),[p]),x.jsx(Kv,{asChild:!0,...o,children:x.jsx(Kt.button,{"aria-describedby":a.open?a.contentId:void 0,"data-state":a.stateAttribute,...n,ref:l,onPointerMove:$t(e.onPointerMove,f=>{f.pointerType!=="touch"&&!d.current&&!i.isPointerInTransitRef.current&&(a.onTriggerEnter(),d.current=!0)}),onPointerLeave:$t(e.onPointerLeave,()=>{a.onTriggerLeave(),d.current=!1}),onPointerDown:$t(e.onPointerDown,()=>{a.open&&a.onClose(),u.current=!0,document.addEventListener("pointerup",p,{once:!0})}),onFocus:$t(e.onFocus,()=>{u.current||a.onOpen()}),onBlur:$t(e.onBlur,a.onClose),onClick:$t(e.onClick,a.onClose)})})});JX.displayName=rR;var dSe="TooltipPortal",[Hct,hSe]=Yv(dSe,{forceMount:void 0}),zh="TooltipContent",XX=T.forwardRef((e,t)=>{const r=hSe(zh,e.__scopeTooltip),{forceMount:n=r.forceMount,side:a="top",...i}=e,o=Xv(zh,e.__scopeTooltip);return x.jsx($i,{present:n||o.open,children:o.disableHoverableContent?x.jsx(ZX,{side:a,...i,ref:t}):x.jsx(pSe,{side:a,...i,ref:t})})}),pSe=T.forwardRef((e,t)=>{const r=Xv(zh,e.__scopeTooltip),n=n5(zh,e.__scopeTooltip),a=T.useRef(null),i=Rr(t,a),[o,s]=T.useState(null),{trigger:l,onClose:u}=r,d=a.current,{onPointerInTransitChange:p}=n,f=T.useCallback(()=>{s(null),p(!1)},[p]),m=T.useCallback((b,y)=>{const S=b.currentTarget,w={x:b.clientX,y:b.clientY},E=ySe(w,S.getBoundingClientRect()),k=vSe(w,E),_=SSe(y.getBoundingClientRect()),R=xSe([...k,..._]);s(R),p(!0)},[p]);return T.useEffect(()=>()=>f(),[f]),T.useEffect(()=>{if(l&&d){const b=S=>m(S,d),y=S=>m(S,l);return l.addEventListener("pointerleave",b),d.addEventListener("pointerleave",y),()=>{l.removeEventListener("pointerleave",b),d.removeEventListener("pointerleave",y)}}},[l,d,m,f]),T.useEffect(()=>{if(o){const b=y=>{const S=y.target,w={x:y.clientX,y:y.clientY},E=(l==null?void 0:l.contains(S))||(d==null?void 0:d.contains(S)),k=!ESe(w,o);E?f():k&&(f(),u())};return document.addEventListener("pointermove",b),()=>document.removeEventListener("pointermove",b)}},[l,d,o,u,f]),x.jsx(ZX,{...e,ref:i})}),[fSe,mSe]=Yv(zm,{isInside:!1}),gSe=RJ("TooltipContent"),ZX=T.forwardRef((e,t)=>{const{__scopeTooltip:r,children:n,"aria-label":a,onEscapeKeyDown:i,onPointerDownOutside:o,...s}=e,l=Xv(zh,r),u=Jv(r),{onClose:d}=l;return T.useEffect(()=>(document.addEventListener(tR,d),()=>document.removeEventListener(tR,d)),[d]),T.useEffect(()=>{if(l.trigger){const p=f=>{const m=f.target;m!=null&&m.contains(l.trigger)&&d()};return window.addEventListener("scroll",p,{capture:!0}),()=>window.removeEventListener("scroll",p,{capture:!0})}},[l.trigger,d]),x.jsx(Sg,{asChild:!0,disableOutsidePointerEvents:!1,onEscapeKeyDown:i,onPointerDownOutside:o,onFocusOutside:p=>p.preventDefault(),onDismiss:d,children:x.jsxs(t5,{"data-state":l.stateAttribute,...u,...s,ref:t,style:{...s.style,"--radix-tooltip-content-transform-origin":"var(--radix-popper-transform-origin)","--radix-tooltip-content-available-width":"var(--radix-popper-available-width)","--radix-tooltip-content-available-height":"var(--radix-popper-available-height)","--radix-tooltip-trigger-width":"var(--radix-popper-anchor-width)","--radix-tooltip-trigger-height":"var(--radix-popper-anchor-height)"},children:[x.jsx(gSe,{children:n}),x.jsx(fSe,{scope:r,isInside:!0,children:x.jsx(sSe,{id:l.contentId,role:"tooltip",children:a||n})})]})})});XX.displayName=zh;var QX="TooltipArrow",bSe=T.forwardRef((e,t)=>{const{__scopeTooltip:r,...n}=e,a=Jv(r);return mSe(QX,r).isInside?null:x.jsx(r5,{...a,...n,ref:t})});bSe.displayName=QX;function ySe(e,t){const r=Math.abs(t.top-e.y),n=Math.abs(t.bottom-e.y),a=Math.abs(t.right-e.x),i=Math.abs(t.left-e.x);switch(Math.min(r,n,a,i)){case i:return"left";case a:return"right";case r:return"top";case n:return"bottom";default:throw new Error("unreachable")}}function vSe(e,t,r=5){const n=[];switch(t){case"top":n.push({x:e.x-r,y:e.y+r},{x:e.x+r,y:e.y+r});break;case"bottom":n.push({x:e.x-r,y:e.y-r},{x:e.x+r,y:e.y-r});break;case"left":n.push({x:e.x+r,y:e.y-r},{x:e.x+r,y:e.y+r});break;case"right":n.push({x:e.x-r,y:e.y-r},{x:e.x-r,y:e.y+r});break}return n}function SSe(e){const{top:t,right:r,bottom:n,left:a}=e;return[{x:a,y:t},{x:r,y:t},{x:r,y:n},{x:a,y:n}]}function ESe(e,t){const{x:r,y:n}=e;let a=!1;for(let i=0,o=t.length-1;i<t.length;o=i++){const s=t[i],l=t[o],u=s.x,d=s.y,p=l.x,f=l.y;d>n!=f>n&&r<(p-u)*(n-d)/(f-d)+u&&(a=!a)}return a}function xSe(e){const t=e.slice();return t.sort((r,n)=>r.x<n.x?-1:r.x>n.x?1:r.y<n.y?-1:r.y>n.y?1:0),wSe(t)}function wSe(e){if(e.length<=1)return e.slice();const t=[];for(let n=0;n<e.length;n++){const a=e[n];for(;t.length>=2;){const i=t[t.length-1],o=t[t.length-2];if((i.x-o.x)*(a.y-o.y)>=(i.y-o.y)*(a.x-o.x))t.pop();else break}t.push(a)}t.pop();const r=[];for(let n=e.length-1;n>=0;n--){const a=e[n];for(;r.length>=2;){const i=r[r.length-1],o=r[r.length-2];if((i.x-o.x)*(a.y-o.y)>=(i.y-o.y)*(a.x-o.x))r.pop();else break}r.push(a)}return r.pop(),t.length===1&&r.length===1&&t[0].x===r[0].x&&t[0].y===r[0].y?t:t.concat(r)}var kSe=KX,CSe=YX,TSe=JX,eZ=XX;const ka=kSe,Ca=CSe,Ta=TSe,ASe=e=>typeof e!="string"?e:x.jsx("div",{className:"relative top-0 pt-1 whitespace-pre-wrap break-words",children:e}),ga=T.forwardRef(({className:e,side:t="left",align:r="start",children:n,...a},i)=>{const o=T.useRef(null);return T.useEffect(()=>{o.current&&(o.current.scrollTop=0)},[n]),x.jsx(eZ,{ref:i,side:t,align:r,className:lt("bg-popover text-popover-foreground animate-in fade-in-0 zoom-in-95 data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=closed]:zoom-out-95 data-[side=bottom]:slide-in-from-top-2 data-[side=left]:slide-in-from-right-2 data-[side=right]:slide-in-from-left-2 data-[side=top]:slide-in-from-bottom-2 max-h-[60vh] overflow-y-auto whitespace-pre-wrap break-words rounded-md border px-3 py-2 text-sm shadow-md z-60",e),...a,children:typeof n=="string"?ASe(n):n})});ga.displayName=eZ.displayName;const my=e2e("inline-flex items-center justify-center gap-2 whitespace-nowrap rounded-md text-sm font-medium ring-offset-background transition-colors focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2 disabled:pointer-events-none disabled:opacity-50 [&_svg]:pointer-events-none [&_svg]:size-4 [&_svg]:shrink-0",{variants:{variant:{default:"bg-primary text-primary-foreground hover:bg-primary/90",destructive:"bg-destructive text-destructive-foreground hover:bg-destructive/90",outline:"border border-input bg-background hover:bg-accent hover:text-accent-foreground",secondary:"bg-secondary text-secondary-foreground hover:bg-secondary/80",ghost:"hover:bg-accent hover:text-accent-foreground",link:"text-primary underline-offset-4 hover:underline"},size:{default:"h-10 px-4 py-2",sm:"h-9 rounded-md px-3",lg:"h-11 rounded-md px-8",icon:"size-8"}},defaultVariants:{variant:"default",size:"default"}}),Rt=T.forwardRef(({className:e,variant:t,tooltip:r,size:n,side:a="right",asChild:i=!1,...o},s)=>{const l=i?Eye:"button";return r?x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx(l,{className:lt(my({variant:t,size:n,className:e}),"cursor-pointer"),ref:s,...o})}),x.jsx(ga,{side:a,children:r})]})}):x.jsx(l,{className:lt(my({variant:t,size:n,className:e}),"cursor-pointer"),ref:s,...o})});Rt.displayName="Button";const FSe=Zve,_Se=Qve,tZ=T.forwardRef(({className:e,...t},r)=>x.jsx(yX,{className:lt("data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 fixed inset-0 z-50 bg-black/50",e),...t,ref:r}));tZ.displayName=yX.displayName;const rZ=T.forwardRef(({className:e,...t},r)=>x.jsxs(_Se,{children:[x.jsx(tZ,{}),x.jsx(vX,{ref:r,className:lt("bg-background data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 data-[state=closed]:zoom-out-95 data-[state=open]:zoom-in-95 data-[state=closed]:slide-out-to-top-[48%] data-[state=open]:slide-in-from-top-[48%] fixed top-[50%] left-[50%] z-50 grid w-full max-w-lg translate-x-[-50%] translate-y-[-50%] gap-4 border p-6 shadow-lg duration-200 sm:rounded-lg",e),...t})]}));rZ.displayName=vX.displayName;const nZ=({className:e,...t})=>x.jsx("div",{className:lt("flex flex-col space-y-2 text-center sm:text-left",e),...t});nZ.displayName="AlertDialogHeader";const aZ=T.forwardRef(({className:e,...t},r)=>x.jsx(xX,{ref:r,className:lt("text-lg font-semibold",e),...t}));aZ.displayName=xX.displayName;const iZ=T.forwardRef(({className:e,...t},r)=>x.jsx(wX,{ref:r,className:lt("text-muted-foreground text-sm",e),...t}));iZ.displayName=wX.displayName;const RSe=T.forwardRef(({className:e,...t},r)=>x.jsx(SX,{ref:r,className:lt(my(),e),...t}));RSe.displayName=SX.displayName;const DSe=T.forwardRef(({className:e,...t},r)=>x.jsx(EX,{ref:r,className:lt(my({variant:"outline"}),"mt-2 sm:mt-0",e),...t}));DSe.displayName=EX.displayName;const oa=T.forwardRef(({className:e,type:t,...r},n)=>x.jsx("input",{type:t,className:lt("border-input file:text-foreground placeholder:text-muted-foreground focus-visible:ring-ring flex h-9 rounded-md border bg-transparent px-3 py-1 text-base shadow-sm transition-colors file:border-0 file:bg-transparent file:text-sm file:font-medium focus-visible:ring-1 focus-visible:outline-none disabled:cursor-not-allowed disabled:opacity-50 md:text-sm [&::-webkit-inner-spin-button]:opacity-50 [&::-webkit-outer-spin-button]:opacity-50",e),ref:n,...r}));oa.displayName="Input";const NSe=({open:e,onOpenChange:t})=>{const{t:r}=Sr(),n=Ve.use.apiKey(),[a,i]=T.useState(""),o=Xn.use.message();T.useEffect(()=>{i(n||"")},[n,e]),T.useEffect(()=>{o&&(o.includes(bJ)||o.includes(yJ))&&t(!0)},[o,t]);const s=T.useCallback(()=>{Ve.setState({apiKey:a||null}),t(!1)},[a,t]),l=T.useCallback(u=>{i(u.target.value)},[i]);return x.jsx(FSe,{open:e,onOpenChange:t,children:x.jsxs(rZ,{children:[x.jsxs(nZ,{children:[x.jsx(aZ,{children:r("apiKeyAlert.title")}),x.jsx(iZ,{children:r("apiKeyAlert.description")})]}),x.jsxs("div",{className:"flex flex-col gap-4",children:[x.jsxs("form",{className:"flex gap-2",onSubmit:u=>u.preventDefault(),children:[x.jsx(oa,{type:"password",value:a,onChange:l,placeholder:r("apiKeyAlert.placeholder"),className:"max-h-full w-full min-w-0",autoComplete:"off"}),x.jsx(Rt,{onClick:s,variant:"outline",size:"sm",children:r("apiKeyAlert.save")})]}),o&&x.jsx("div",{className:"text-sm text-red-500",children:o})]})]})})};/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const LSe=e=>e.replace(/([a-z0-9])([A-Z])/g,"$1-$2").toLowerCase(),oZ=(...e)=>e.filter((t,r,n)=>!!t&&t.trim()!==""&&n.indexOf(t)===r).join(" ").trim();/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */var MSe={xmlns:"http://www.w3.org/2000/svg",width:24,height:24,viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:2,strokeLinecap:"round",strokeLinejoin:"round"};/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const ISe=T.forwardRef(({color:e="currentColor",size:t=24,strokeWidth:r=2,absoluteStrokeWidth:n,className:a="",children:i,iconNode:o,...s},l)=>T.createElement("svg",{ref:l,...MSe,width:t,height:t,stroke:e,strokeWidth:n?Number(r)*24/Number(t):r,className:oZ("lucide",a),...s},[...o.map(([u,d])=>T.createElement(u,d)),...Array.isArray(i)?i:[i]]));/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const rr=(e,t)=>{const r=T.forwardRef(({className:n,...a},i)=>T.createElement(ISe,{ref:i,iconNode:t,className:oZ(`lucide-${LSe(e)}`,n),...a}));return r.displayName=`${e}`,r};/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const OSe=[["path",{d:"M22 12h-2.48a2 2 0 0 0-1.93 1.46l-2.35 8.36a.25.25 0 0 1-.48 0L9.24 2.18a.25.25 0 0 0-.48 0l-2.35 8.36A2 2 0 0 1 4.49 12H2",key:"169zse"}]],BSe=rr("Activity",OSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const PSe=[["path",{d:"M17 12H7",key:"16if0g"}],["path",{d:"M19 18H5",key:"18s9l3"}],["path",{d:"M21 6H3",key:"1jwq7v"}]],zSe=rr("AlignCenter",PSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const HSe=[["path",{d:"M15 12H3",key:"6jk70r"}],["path",{d:"M17 18H3",key:"1amg6g"}],["path",{d:"M21 6H3",key:"1jwq7v"}]],USe=rr("AlignLeft",HSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const $Se=[["path",{d:"M21 12H9",key:"dn1m92"}],["path",{d:"M21 18H7",key:"1ygte8"}],["path",{d:"M21 6H3",key:"1jwq7v"}]],GSe=rr("AlignRight",$Se);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const jSe=[["path",{d:"M12 5v14",key:"s699le"}],["path",{d:"m19 12-7 7-7-7",key:"1idqje"}]],Qx=rr("ArrowDown",jSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const qSe=[["path",{d:"m5 12 7-7 7 7",key:"hav0vg"}],["path",{d:"M12 19V5",key:"x0mq9r"}]],ew=rr("ArrowUp",qSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const WSe=[["path",{d:"M12 7v14",key:"1akyts"}],["path",{d:"M3 18a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h5a4 4 0 0 1 4 4 4 4 0 0 1 4-4h5a1 1 0 0 1 1 1v13a1 1 0 0 1-1 1h-6a3 3 0 0 0-3 3 3 3 0 0 0-3-3z",key:"ruj8y"}]],VSe=rr("BookOpen",WSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const KSe=[["path",{d:"M20 6 9 17l-5-5",key:"1gmf2c"}]],a5=rr("Check",KSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const YSe=[["path",{d:"m6 9 6 6 6-6",key:"qrunsl"}]],Zv=rr("ChevronDown",YSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const JSe=[["path",{d:"m15 18-6-6 6-6",key:"1wnfg3"}]],t7=rr("ChevronLeft",JSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const XSe=[["path",{d:"m9 18 6-6-6-6",key:"mthhwq"}]],r7=rr("ChevronRight",XSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const ZSe=[["path",{d:"m18 15-6-6-6 6",key:"153udz"}]],QSe=rr("ChevronUp",ZSe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const eEe=[["path",{d:"m11 17-5-5 5-5",key:"13zhaf"}],["path",{d:"m18 17-5-5 5-5",key:"h8a8et"}]],tEe=rr("ChevronsLeft",eEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const rEe=[["path",{d:"m6 17 5-5-5-5",key:"xnjwq"}],["path",{d:"m13 17 5-5-5-5",key:"17xmmf"}]],nEe=rr("ChevronsRight",rEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const aEe=[["path",{d:"m7 15 5 5 5-5",key:"1hf1tw"}],["path",{d:"m7 9 5-5 5 5",key:"sgt6xg"}]],iEe=rr("ChevronsUpDown",aEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const oEe=[["rect",{width:"14",height:"14",x:"8",y:"8",rx:"2",ry:"2",key:"17jyea"}],["path",{d:"M4 16c-1.1 0-2-.9-2-2V4c0-1.1.9-2 2-2h10c1.1 0 2 .9 2 2",key:"zix9uf"}]],n7=rr("Copy",oEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const sEe=[["path",{d:"m7 21-4.3-4.3c-1-1-1-2.5 0-3.4l9.6-9.6c1-1 2.5-1 3.4 0l5.6 5.6c1 1 1 2.5 0 3.4L13 21",key:"182aya"}],["path",{d:"M22 21H7",key:"t4ddhn"}],["path",{d:"m5 11 9 9",key:"1mo9qw"}]],sZ=rr("Eraser",sEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const lEe=[["path",{d:"M15 2H6a2 2 0 0 0-2 2v16a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V7Z",key:"1rqfz7"}],["path",{d:"M14 2v4a2 2 0 0 0 2 2h4",key:"tnqrlb"}],["path",{d:"M10 9H8",key:"b1mrlr"}],["path",{d:"M16 13H8",key:"t4e002"}],["path",{d:"M16 17H8",key:"z1uh3a"}]],lZ=rr("FileText",lEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const uEe=[["path",{d:"M20 7h-3a2 2 0 0 1-2-2V2",key:"x099mo"}],["path",{d:"M9 18a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h7l4 4v10a2 2 0 0 1-2 2Z",key:"18t6ie"}],["path",{d:"M3 7.6v12.8A1.6 1.6 0 0 0 4.6 22h9.8",key:"1nja0z"}]],cEe=rr("Files",uEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const dEe=[["path",{d:"M3 7V5a2 2 0 0 1 2-2h2",key:"aa7l1z"}],["path",{d:"M17 3h2a2 2 0 0 1 2 2v2",key:"4qcy5o"}],["path",{d:"M21 17v2a2 2 0 0 1-2 2h-2",key:"6vwrx8"}],["path",{d:"M7 21H5a2 2 0 0 1-2-2v-2",key:"ioqczr"}],["rect",{width:"10",height:"8",x:"7",y:"8",rx:"1",key:"vys8me"}]],hEe=rr("Fullscreen",dEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const pEe=[["path",{d:"M6 3v12",key:"qpgusn"}],["path",{d:"M18 9a3 3 0 1 0 0-6 3 3 0 0 0 0 6z",key:"1d02ji"}],["path",{d:"M6 21a3 3 0 1 0 0-6 3 3 0 0 0 0 6z",key:"chk6ph"}],["path",{d:"M15 6a9 9 0 0 0-9 9",key:"or332x"}],["path",{d:"M18 15v6",key:"9wciyi"}],["path",{d:"M21 18h-6",key:"139f0c"}]],fEe=rr("GitBranchPlus",pEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const mEe=[["path",{d:"M15 22v-4a4.8 4.8 0 0 0-1-3.5c3 0 6-2 6-5.5.08-1.25-.27-2.48-1-3.5.28-1.15.28-2.35 0-3.5 0 0-1 0-3 1.5-2.64-.5-5.36-.5-8 0C6 2 5 2 5 2c-.3 1.15-.3 2.35 0 3.5A5.403 5.403 0 0 0 4 9c0 3.5 3 5.5 6 5.5-.39.49-.68 1.05-.85 1.65-.17.6-.22 1.23-.15 1.85v4",key:"tonef"}],["path",{d:"M9 18c-4.51 2-5-2-7-2",key:"9comsn"}]],gEe=rr("Github",mEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const bEe=[["circle",{cx:"12",cy:"5",r:"1",key:"gxeob9"}],["circle",{cx:"19",cy:"5",r:"1",key:"w8mnmm"}],["circle",{cx:"5",cy:"5",r:"1",key:"lttvr7"}],["circle",{cx:"12",cy:"12",r:"1",key:"41hilf"}],["circle",{cx:"19",cy:"12",r:"1",key:"1wjl8i"}],["circle",{cx:"5",cy:"12",r:"1",key:"1pcz8c"}],["circle",{cx:"12",cy:"19",r:"1",key:"lyex9k"}],["circle",{cx:"19",cy:"19",r:"1",key:"shf9b7"}],["circle",{cx:"5",cy:"19",r:"1",key:"bfqh0e"}]],yEe=rr("Grip",bEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const vEe=[["circle",{cx:"12",cy:"12",r:"10",key:"1mglay"}],["path",{d:"M12 16v-4",key:"1dtifu"}],["path",{d:"M12 8h.01",key:"e9boi3"}]],SEe=rr("Info",vEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const EEe=[["path",{d:"M21 12a9 9 0 1 1-6.219-8.56",key:"13zald"}]],i5=rr("LoaderCircle",EEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const xEe=[["path",{d:"M12 2v4",key:"3427ic"}],["path",{d:"m16.2 7.8 2.9-2.9",key:"r700ao"}],["path",{d:"M18 12h4",key:"wj9ykh"}],["path",{d:"m16.2 16.2 2.9 2.9",key:"1bxg5t"}],["path",{d:"M12 18v4",key:"jadmvz"}],["path",{d:"m4.9 19.1 2.9-2.9",key:"bwix9q"}],["path",{d:"M2 12h4",key:"j09sii"}],["path",{d:"m4.9 4.9 2.9 2.9",key:"giyufr"}]],a7=rr("Loader",xEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const wEe=[["path",{d:"M9 21H5a2 2 0 0 1-2-2V5a2 2 0 0 1 2-2h4",key:"1uf3rs"}],["polyline",{points:"16 17 21 12 16 7",key:"1gabdz"}],["line",{x1:"21",x2:"9",y1:"12",y2:"12",key:"1uyos4"}]],kEe=rr("LogOut",wEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const CEe=[["path",{d:"M8 3H5a2 2 0 0 0-2 2v3",key:"1dcmit"}],["path",{d:"M21 8V5a2 2 0 0 0-2-2h-3",key:"1e4gt3"}],["path",{d:"M3 16v3a2 2 0 0 0 2 2h3",key:"wsl5sc"}],["path",{d:"M16 21h3a2 2 0 0 0 2-2v-3",key:"18trek"}]],TEe=rr("Maximize",CEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const AEe=[["path",{d:"M8 3v3a2 2 0 0 1-2 2H3",key:"hohbtr"}],["path",{d:"M21 8h-3a2 2 0 0 1-2-2V3",key:"5jw1f3"}],["path",{d:"M3 16h3a2 2 0 0 1 2 2v3",key:"198tvr"}],["path",{d:"M16 21v-3a2 2 0 0 1 2-2h3",key:"ph8mxp"}]],FEe=rr("Minimize",AEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const _Ee=[["circle",{cx:"13.5",cy:"6.5",r:".5",fill:"currentColor",key:"1okk4w"}],["circle",{cx:"17.5",cy:"10.5",r:".5",fill:"currentColor",key:"f64h9f"}],["circle",{cx:"8.5",cy:"7.5",r:".5",fill:"currentColor",key:"fotxhn"}],["circle",{cx:"6.5",cy:"12.5",r:".5",fill:"currentColor",key:"qy21gx"}],["path",{d:"M12 2C6.5 2 2 6.5 2 12s4.5 10 10 10c.926 0 1.648-.746 1.648-1.688 0-.437-.18-.835-.437-1.125-.29-.289-.438-.652-.438-1.125a1.64 1.64 0 0 1 1.668-1.668h1.996c3.051 0 5.555-2.503 5.555-5.554C21.965 6.012 17.461 2 12 2z",key:"12rzf8"}]],REe=rr("Palette",_Ee);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const DEe=[["rect",{x:"14",y:"4",width:"4",height:"16",rx:"1",key:"zuxfzm"}],["rect",{x:"6",y:"4",width:"4",height:"16",rx:"1",key:"1okwgv"}]],NEe=rr("Pause",DEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const LEe=[["path",{d:"M21.174 6.812a1 1 0 0 0-3.986-3.987L3.842 16.174a2 2 0 0 0-.5.83l-1.321 4.352a.5.5 0 0 0 .623.622l4.353-1.32a2 2 0 0 0 .83-.497z",key:"1a8usu"}],["path",{d:"m15 5 4 4",key:"1mk7zo"}]],MEe=rr("Pencil",LEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const IEe=[["polygon",{points:"6 3 20 12 6 21 6 3",key:"1oa8hb"}]],OEe=rr("Play",IEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const BEe=[["path",{d:"M3 12a9 9 0 0 1 9-9 9.75 9.75 0 0 1 6.74 2.74L21 8",key:"v9h5vc"}],["path",{d:"M21 3v5h-5",key:"1q7to0"}],["path",{d:"M21 12a9 9 0 0 1-9 9 9.75 9.75 0 0 1-6.74-2.74L3 16",key:"3uifl3"}],["path",{d:"M8 16H3v5",key:"1cv678"}]],uZ=rr("RefreshCw",BEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const PEe=[["path",{d:"M3 12a9 9 0 1 0 9-9 9.75 9.75 0 0 0-6.74 2.74L3 8",key:"1357e3"}],["path",{d:"M3 3v5h5",key:"1xhq8a"}]],o5=rr("RotateCcw",PEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const zEe=[["path",{d:"M21 12a9 9 0 1 1-9-9c2.52 0 4.93 1 6.74 2.74L21 8",key:"1p45f6"}],["path",{d:"M21 3v5h-5",key:"1q7to0"}]],HEe=rr("RotateCw",zEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const UEe=[["circle",{cx:"6",cy:"6",r:"3",key:"1lh9wr"}],["path",{d:"M8.12 8.12 12 12",key:"1alkpv"}],["path",{d:"M20 4 8.12 15.88",key:"xgtan2"}],["circle",{cx:"6",cy:"18",r:"3",key:"fqmcym"}],["path",{d:"M14.8 14.8 20 20",key:"ptml3r"}]],$Ee=rr("Scissors",UEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const GEe=[["circle",{cx:"11",cy:"11",r:"8",key:"4ej97u"}],["path",{d:"m21 21-4.3-4.3",key:"1qie3q"}]],jEe=rr("Search",GEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const qEe=[["path",{d:"M14.536 21.686a.5.5 0 0 0 .937-.024l6.5-19a.496.496 0 0 0-.635-.635l-19 6.5a.5.5 0 0 0-.024.937l7.93 3.18a2 2 0 0 1 1.112 1.11z",key:"1ffxy3"}],["path",{d:"m21.854 2.147-10.94 10.939",key:"12cjpa"}]],WEe=rr("Send",qEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const VEe=[["path",{d:"M12.22 2h-.44a2 2 0 0 0-2 2v.18a2 2 0 0 1-1 1.73l-.43.25a2 2 0 0 1-2 0l-.15-.08a2 2 0 0 0-2.73.73l-.22.38a2 2 0 0 0 .73 2.73l.15.1a2 2 0 0 1 1 1.72v.51a2 2 0 0 1-1 1.74l-.15.09a2 2 0 0 0-.73 2.73l.22.38a2 2 0 0 0 2.73.73l.15-.08a2 2 0 0 1 2 0l.43.25a2 2 0 0 1 1 1.73V20a2 2 0 0 0 2 2h.44a2 2 0 0 0 2-2v-.18a2 2 0 0 1 1-1.73l.43-.25a2 2 0 0 1 2 0l.15.08a2 2 0 0 0 2.73-.73l.22-.39a2 2 0 0 0-.73-2.73l-.15-.08a2 2 0 0 1-1-1.74v-.5a2 2 0 0 1 1-1.74l.15-.09a2 2 0 0 0 .73-2.73l-.22-.38a2 2 0 0 0-2.73-.73l-.15.08a2 2 0 0 1-2 0l-.43-.25a2 2 0 0 1-1-1.73V4a2 2 0 0 0-2-2z",key:"1qme2f"}],["circle",{cx:"12",cy:"12",r:"3",key:"1v7zrd"}]],KEe=rr("Settings",VEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const YEe=[["path",{d:"M21 10.5V19a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V5a2 2 0 0 1 2-2h12.5",key:"1uzm8b"}],["path",{d:"m9 11 3 3L22 4",key:"1pflzl"}]],i7=rr("SquareCheckBig",YEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const JEe=[["path",{d:"M3 6h18",key:"d0wm0j"}],["path",{d:"M19 6v14c0 1-1 2-2 2H7c-1 0-2-1-2-2V6",key:"4alrt4"}],["path",{d:"M8 6V4c0-1 1-2 2-2h4c1 0 2 1 2 2v2",key:"v07s0e"}]],XEe=rr("Trash",JEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const ZEe=[["path",{d:"m21.73 18-8-14a2 2 0 0 0-3.48 0l-8 14A2 2 0 0 0 4 21h16a2 2 0 0 0 1.73-3",key:"wmoenq"}],["path",{d:"M12 9v4",key:"juzpu7"}],["path",{d:"M12 17h.01",key:"p32p05"}]],s5=rr("TriangleAlert",ZEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const QEe=[["path",{d:"M9 14 4 9l5-5",key:"102s5s"}],["path",{d:"M4 9h10.5a5.5 5.5 0 0 1 5.5 5.5a5.5 5.5 0 0 1-5.5 5.5H11",key:"f3b9sd"}]],cZ=rr("Undo2",QEe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const exe=[["path",{d:"M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4",key:"ih7n3h"}],["polyline",{points:"17 8 12 3 7 8",key:"t8dd8p"}],["line",{x1:"12",x2:"12",y1:"3",y2:"15",key:"widbto"}]],nR=rr("Upload",exe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const txe=[["path",{d:"M18 6 6 18",key:"1bl5f8"}],["path",{d:"m6 6 12 12",key:"d8bk6v"}]],Qv=rr("X",txe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const rxe=[["path",{d:"M4 14a1 1 0 0 1-.78-1.63l9.9-10.2a.5.5 0 0 1 .86.46l-1.92 6.02A1 1 0 0 0 13 10h7a1 1 0 0 1 .78 1.63l-9.9 10.2a.5.5 0 0 1-.86-.46l1.92-6.02A1 1 0 0 0 11 14z",key:"1xq2db"}]],l5=rr("Zap",rxe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const nxe=[["circle",{cx:"11",cy:"11",r:"8",key:"4ej97u"}],["line",{x1:"21",x2:"16.65",y1:"21",y2:"16.65",key:"13gj7c"}],["line",{x1:"11",x2:"11",y1:"8",y2:"14",key:"1vmskp"}],["line",{x1:"8",x2:"14",y1:"11",y2:"11",key:"durymu"}]],axe=rr("ZoomIn",nxe);/**
 * @license lucide-react v0.475.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const ixe=[["circle",{cx:"11",cy:"11",r:"8",key:"4ej97u"}],["line",{x1:"21",x2:"16.65",y1:"21",y2:"16.65",key:"13gj7c"}],["line",{x1:"8",x2:"14",y1:"11",y2:"11",key:"durymu"}]],oxe=rr("ZoomOut",ixe),Du=HD,u5=iX,sxe=UD,dZ=T.forwardRef(({className:e,...t},r)=>x.jsx(Gv,{ref:r,className:lt("data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 fixed inset-0 z-50 bg-black/30",e),...t}));dZ.displayName=Gv.displayName;const Sl=T.forwardRef(({className:e,children:t,...r},n)=>x.jsxs(sxe,{children:[x.jsx(dZ,{}),x.jsxs(jv,{ref:n,className:lt("bg-background data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 data-[state=closed]:zoom-out-95 data-[state=open]:zoom-in-95 data-[state=closed]:slide-out-to-top-[48%] data-[state=open]:slide-in-from-top-[48%] fixed top-[50%] left-[50%] z-50 grid w-full max-w-lg translate-x-[-50%] translate-y-[-50%] gap-4 border p-6 shadow-lg duration-200 sm:rounded-lg",e),...r,children:[t,x.jsxs(jD,{className:"ring-offset-background focus:ring-ring data-[state=open]:bg-accent data-[state=open]:text-muted-foreground absolute top-4 right-4 rounded-sm opacity-70 transition-opacity hover:opacity-100 focus:ring-2 focus:ring-offset-2 focus:outline-none disabled:pointer-events-none",children:[x.jsx(Qv,{className:"h-4 w-4"}),x.jsx("span",{className:"sr-only",children:"Close"})]})]})]}));Sl.displayName=jv.displayName;const El=({className:e,...t})=>x.jsx("div",{className:lt("flex flex-col space-y-1.5 text-center sm:text-left",e),...t});El.displayName="DialogHeader";const xg=({className:e,...t})=>x.jsx("div",{className:lt("flex flex-col-reverse sm:flex-row sm:justify-end sm:space-x-2",e),...t});xg.displayName="DialogFooter";const xl=T.forwardRef(({className:e,...t},r)=>x.jsx($D,{ref:r,className:lt("text-lg leading-none font-semibold tracking-tight",e),...t}));xl.displayName=$D.displayName;const wl=T.forwardRef(({className:e,...t},r)=>x.jsx(GD,{ref:r,className:lt("text-muted-foreground text-sm",e),...t}));wl.displayName=GD.displayName;const lxe=({status:e})=>{const{t}=Sr();return e?x.jsxs("div",{className:"min-w-[300px] space-y-2 text-xs",children:[x.jsxs("div",{className:"space-y-1",children:[x.jsx("h4",{className:"font-medium",children:t("graphPanel.statusCard.serverInfo")}),x.jsxs("div",{className:"text-foreground grid grid-cols-[160px_1fr] gap-1",children:[x.jsxs("span",{children:[t("graphPanel.statusCard.workingDirectory"),":"]}),x.jsx("span",{className:"truncate",children:e.working_directory}),x.jsxs("span",{children:[t("graphPanel.statusCard.inputDirectory"),":"]}),x.jsx("span",{className:"truncate",children:e.input_directory}),x.jsxs("span",{children:[t("graphPanel.statusCard.summarySettings"),":"]}),x.jsxs("span",{children:[e.configuration.summary_language," / LLM summary on ",e.configuration.force_llm_summary_on_merge.toString()," fragments"]}),x.jsxs("span",{children:[t("graphPanel.statusCard.threshold"),":"]}),x.jsxs("span",{children:["cosine ",e.configuration.cosine_threshold," / rerank_score ",e.configuration.min_rerank_score," / max_related ",e.configuration.related_chunk_number]}),x.jsxs("span",{children:[t("graphPanel.statusCard.maxParallelInsert"),":"]}),x.jsx("span",{children:e.configuration.max_parallel_insert})]})]}),x.jsxs("div",{className:"space-y-1",children:[x.jsx("h4",{className:"font-medium",children:t("graphPanel.statusCard.llmConfig")}),x.jsxs("div",{className:"text-foreground grid grid-cols-[160px_1fr] gap-1",children:[x.jsxs("span",{children:[t("graphPanel.statusCard.llmBindingHost"),":"]}),x.jsx("span",{children:e.configuration.llm_binding_host}),x.jsxs("span",{children:[t("graphPanel.statusCard.llmModel"),":"]}),x.jsxs("span",{children:[e.configuration.llm_binding,": ",e.configuration.llm_model," (#",e.configuration.max_async," Async)"]})]})]}),x.jsxs("div",{className:"space-y-1",children:[x.jsx("h4",{className:"font-medium",children:t("graphPanel.statusCard.embeddingConfig")}),x.jsxs("div",{className:"text-foreground grid grid-cols-[160px_1fr] gap-1",children:[x.jsxs("span",{children:[t("graphPanel.statusCard.embeddingBindingHost"),":"]}),x.jsx("span",{children:e.configuration.embedding_binding_host}),x.jsxs("span",{children:[t("graphPanel.statusCard.embeddingModel"),":"]}),x.jsxs("span",{children:[e.configuration.embedding_binding,": ",e.configuration.embedding_model," (#",e.configuration.embedding_func_max_async," Async * ",e.configuration.embedding_batch_num," batches)"]})]})]}),e.configuration.enable_rerank&&x.jsxs("div",{className:"space-y-1",children:[x.jsx("h4",{className:"font-medium",children:t("graphPanel.statusCard.rerankerConfig")}),x.jsxs("div",{className:"text-foreground grid grid-cols-[160px_1fr] gap-1",children:[x.jsxs("span",{children:[t("graphPanel.statusCard.rerankerBindingHost"),":"]}),x.jsx("span",{children:e.configuration.rerank_binding_host||"-"}),x.jsxs("span",{children:[t("graphPanel.statusCard.rerankerModel"),":"]}),x.jsxs("span",{children:[e.configuration.rerank_binding||"-"," : ",e.configuration.rerank_model||"-"]})]})]}),x.jsxs("div",{className:"space-y-1",children:[x.jsx("h4",{className:"font-medium",children:t("graphPanel.statusCard.storageConfig")}),x.jsxs("div",{className:"text-foreground grid grid-cols-[160px_1fr] gap-1",children:[x.jsxs("span",{children:[t("graphPanel.statusCard.kvStorage"),":"]}),x.jsx("span",{children:e.configuration.kv_storage}),x.jsxs("span",{children:[t("graphPanel.statusCard.docStatusStorage"),":"]}),x.jsx("span",{children:e.configuration.doc_status_storage}),x.jsxs("span",{children:[t("graphPanel.statusCard.graphStorage"),":"]}),x.jsx("span",{children:e.configuration.graph_storage}),x.jsxs("span",{children:[t("graphPanel.statusCard.vectorStorage"),":"]}),x.jsx("span",{children:e.configuration.vector_storage}),x.jsxs("span",{children:[t("graphPanel.statusCard.workspace"),":"]}),x.jsx("span",{children:e.configuration.workspace||"-"}),x.jsxs("span",{children:[t("graphPanel.statusCard.maxGraphNodes"),":"]}),x.jsx("span",{children:e.configuration.max_graph_nodes||"-"}),e.keyed_locks&&x.jsxs(x.Fragment,{children:[x.jsxs("span",{children:[t("graphPanel.statusCard.lockStatus"),":"]}),x.jsxs("span",{children:["mp ",e.keyed_locks.current_status.pending_mp_cleanup,"/",e.keyed_locks.current_status.total_mp_locks," | async ",e.keyed_locks.current_status.pending_async_cleanup,"/",e.keyed_locks.current_status.total_async_locks,"(pid: ",e.keyed_locks.process_id,")"]})]})]})]})]}):x.jsx("div",{className:"text-foreground text-xs",children:t("graphPanel.statusCard.unavailable")})},uxe=({open:e,onOpenChange:t,status:r})=>{const{t:n}=Sr();return x.jsx(Du,{open:e,onOpenChange:t,children:x.jsxs(Sl,{className:"sm:max-w-[700px]",children:[x.jsxs(El,{children:[x.jsx(xl,{children:n("graphPanel.statusDialog.title")}),x.jsx(wl,{children:n("graphPanel.statusDialog.description")})]}),x.jsx(lxe,{status:r})]})})},cxe=()=>{const{t:e}=Sr(),t=Xn.use.health(),r=Xn.use.lastCheckTime(),n=Xn.use.status(),[a,i]=T.useState(!1),[o,s]=T.useState(!1);return T.useEffect(()=>{i(!0);const l=setTimeout(()=>i(!1),300);return()=>clearTimeout(l)},[r]),x.jsxs("div",{className:"fixed right-4 bottom-4 flex items-center gap-2 opacity-80 select-none",children:[x.jsxs("div",{className:"flex cursor-pointer items-center gap-2",onClick:()=>s(!0),children:[x.jsx("div",{className:lt("h-3 w-3 rounded-full transition-all duration-300","shadow-[0_0_8px_rgba(0,0,0,0.2)]",t?"bg-green-500":"bg-red-500",a&&"scale-125",a&&t&&"shadow-[0_0_12px_rgba(34,197,94,0.4)]",a&&!t&&"shadow-[0_0_12px_rgba(239,68,68,0.4)]")}),x.jsx("span",{className:"text-muted-foreground text-xs",children:e(t?"graphPanel.statusIndicator.connected":"graphPanel.statusIndicator.disconnected")})]}),x.jsx(uxe,{open:o,onOpenChange:s,status:n})]})};var e2="Popover",[hZ]=Bo(e2,[mp]),wg=mp(),[dxe,Pu]=hZ(e2),pZ=e=>{const{__scopePopover:t,children:r,open:n,defaultOpen:a,onOpenChange:i,modal:o=!1}=e,s=wg(t),l=T.useRef(null),[u,d]=T.useState(!1),[p,f]=bl({prop:n,defaultProp:a??!1,onChange:i,caller:e2});return x.jsx(e5,{...s,children:x.jsx(dxe,{scope:t,contentId:La(),triggerRef:l,open:p,onOpenChange:f,onOpenToggle:T.useCallback(()=>f(m=>!m),[f]),hasCustomAnchor:u,onCustomAnchorAdd:T.useCallback(()=>d(!0),[]),onCustomAnchorRemove:T.useCallback(()=>d(!1),[]),modal:o,children:r})})};pZ.displayName=e2;var fZ="PopoverAnchor",hxe=T.forwardRef((e,t)=>{const{__scopePopover:r,...n}=e,a=Pu(fZ,r),i=wg(r),{onCustomAnchorAdd:o,onCustomAnchorRemove:s}=a;return T.useEffect(()=>(o(),()=>s()),[o,s]),x.jsx(Kv,{...i,...n,ref:t})});hxe.displayName=fZ;var mZ="PopoverTrigger",gZ=T.forwardRef((e,t)=>{const{__scopePopover:r,...n}=e,a=Pu(mZ,r),i=wg(r),o=Rr(t,a.triggerRef),s=x.jsx(Kt.button,{type:"button","aria-haspopup":"dialog","aria-expanded":a.open,"aria-controls":a.contentId,"data-state":EZ(a.open),...n,ref:o,onClick:$t(e.onClick,a.onOpenToggle)});return a.hasCustomAnchor?s:x.jsx(Kv,{asChild:!0,...i,children:s})});gZ.displayName=mZ;var c5="PopoverPortal",[pxe,fxe]=hZ(c5,{forceMount:void 0}),bZ=e=>{const{__scopePopover:t,forceMount:r,children:n,container:a}=e,i=Pu(c5,t);return x.jsx(pxe,{scope:t,forceMount:r,children:x.jsx($i,{present:r||i.open,children:x.jsx(zv,{asChild:!0,container:a,children:n})})})};bZ.displayName=c5;var Hh="PopoverContent",yZ=T.forwardRef((e,t)=>{const r=fxe(Hh,e.__scopePopover),{forceMount:n=r.forceMount,...a}=e,i=Pu(Hh,e.__scopePopover);return x.jsx($i,{present:n||i.open,children:i.modal?x.jsx(gxe,{...a,ref:t}):x.jsx(bxe,{...a,ref:t})})});yZ.displayName=Hh;var mxe=Oc("PopoverContent.RemoveScroll"),gxe=T.forwardRef((e,t)=>{const r=Pu(Hh,e.__scopePopover),n=T.useRef(null),a=Rr(t,n),i=T.useRef(!1);return T.useEffect(()=>{const o=n.current;if(o)return OD(o)},[]),x.jsx(Uv,{as:mxe,allowPinchZoom:!0,children:x.jsx(vZ,{...e,ref:a,trapFocus:r.open,disableOutsidePointerEvents:!0,onCloseAutoFocus:$t(e.onCloseAutoFocus,o=>{var s;o.preventDefault(),i.current||(s=r.triggerRef.current)==null||s.focus()}),onPointerDownOutside:$t(e.onPointerDownOutside,o=>{const s=o.detail.originalEvent,l=s.button===0&&s.ctrlKey===!0,u=s.button===2||l;i.current=u},{checkForDefaultPrevented:!1}),onFocusOutside:$t(e.onFocusOutside,o=>o.preventDefault(),{checkForDefaultPrevented:!1})})})}),bxe=T.forwardRef((e,t)=>{const r=Pu(Hh,e.__scopePopover),n=T.useRef(!1),a=T.useRef(!1);return x.jsx(vZ,{...e,ref:t,trapFocus:!1,disableOutsidePointerEvents:!1,onCloseAutoFocus:i=>{var o,s;(o=e.onCloseAutoFocus)==null||o.call(e,i),i.defaultPrevented||(n.current||(s=r.triggerRef.current)==null||s.focus(),i.preventDefault()),n.current=!1,a.current=!1},onInteractOutside:i=>{var l,u;(l=e.onInteractOutside)==null||l.call(e,i),i.defaultPrevented||(n.current=!0,i.detail.originalEvent.type==="pointerdown"&&(a.current=!0));const o=i.target;((u=r.triggerRef.current)==null?void 0:u.contains(o))&&i.preventDefault(),i.detail.originalEvent.type==="focusin"&&a.current&&i.preventDefault()}})}),vZ=T.forwardRef((e,t)=>{const{__scopePopover:r,trapFocus:n,onOpenAutoFocus:a,onCloseAutoFocus:i,disableOutsidePointerEvents:o,onEscapeKeyDown:s,onPointerDownOutside:l,onFocusOutside:u,onInteractOutside:d,...p}=e,f=Pu(Hh,r),m=wg(r);return ID(),x.jsx(Pv,{asChild:!0,loop:!0,trapped:n,onMountAutoFocus:a,onUnmountAutoFocus:i,children:x.jsx(Sg,{asChild:!0,disableOutsidePointerEvents:o,onInteractOutside:d,onEscapeKeyDown:s,onPointerDownOutside:l,onFocusOutside:u,onDismiss:()=>f.onOpenChange(!1),children:x.jsx(t5,{"data-state":EZ(f.open),role:"dialog",id:f.contentId,...m,...p,ref:t,style:{...p.style,"--radix-popover-content-transform-origin":"var(--radix-popper-transform-origin)","--radix-popover-content-available-width":"var(--radix-popper-available-width)","--radix-popover-content-available-height":"var(--radix-popper-available-height)","--radix-popover-trigger-width":"var(--radix-popper-anchor-width)","--radix-popover-trigger-height":"var(--radix-popper-anchor-height)"}})})})}),SZ="PopoverClose",yxe=T.forwardRef((e,t)=>{const{__scopePopover:r,...n}=e,a=Pu(SZ,r);return x.jsx(Kt.button,{type:"button",...n,ref:t,onClick:$t(e.onClick,()=>a.onOpenChange(!1))})});yxe.displayName=SZ;var vxe="PopoverArrow",Sxe=T.forwardRef((e,t)=>{const{__scopePopover:r,...n}=e,a=wg(r);return x.jsx(r5,{...a,...n,ref:t})});Sxe.displayName=vxe;function EZ(e){return e?"open":"closed"}var Exe=pZ,xxe=gZ,wxe=bZ,xZ=yZ;const t2=Exe,r2=xxe,kg=T.forwardRef(({className:e,align:t="center",sideOffset:r=4,collisionPadding:n,sticky:a,avoidCollisions:i=!1,...o},s)=>x.jsx(wxe,{children:x.jsx(xZ,{ref:s,align:t,sideOffset:r,collisionPadding:n,sticky:a,avoidCollisions:i,className:lt("bg-popover text-popover-foreground data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 data-[state=closed]:zoom-out-95 data-[state=open]:zoom-in-95 data-[side=bottom]:slide-in-from-top-2 data-[side=left]:slide-in-from-right-2 data-[side=right]:slide-in-from-left-2 data-[side=top]:slide-in-from-bottom-2 z-50 rounded-md border p-4 shadow-md outline-none",e),...o})}));kg.displayName=xZ.displayName;function aR(e,[t,r]){return Math.min(r,Math.max(t,e))}function wZ(e){const t=e+"CollectionProvider",[r,n]=Bo(t),[a,i]=r(t,{collectionRef:{current:null},itemMap:new Map}),o=y=>{const{scope:S,children:w}=y,E=$e.useRef(null),k=$e.useRef(new Map).current;return x.jsx(a,{scope:S,itemMap:k,collectionRef:E,children:w})};o.displayName=t;const s=e+"CollectionSlot",l=Oc(s),u=$e.forwardRef((y,S)=>{const{scope:w,children:E}=y,k=i(s,w),_=Rr(S,k.collectionRef);return x.jsx(l,{ref:_,children:E})});u.displayName=s;const d=e+"CollectionItemSlot",p="data-radix-collection-item",f=Oc(d),m=$e.forwardRef((y,S)=>{const{scope:w,children:E,...k}=y,_=$e.useRef(null),R=Rr(S,_),C=i(d,w);return $e.useEffect(()=>(C.itemMap.set(_,{ref:_,...k}),()=>void C.itemMap.delete(_))),x.jsx(f,{[p]:"",ref:R,children:E})});m.displayName=d;function b(y){const S=i(e+"CollectionConsumer",y);return $e.useCallback(()=>{const E=S.collectionRef.current;if(!E)return[];const k=Array.from(E.querySelectorAll(`[${p}]`));return Array.from(S.itemMap.values()).sort((C,F)=>k.indexOf(C.ref.current)-k.indexOf(F.ref.current))},[S.collectionRef,S.itemMap])}return[{Provider:o,Slot:u,ItemSlot:m},b,n]}var kxe=T.createContext(void 0);function n2(e){const t=T.useContext(kxe);return e||t||"ltr"}function kZ(e){const t=T.useRef({value:e,previous:e});return T.useMemo(()=>(t.current.value!==e&&(t.current.previous=t.current.value,t.current.value=e),t.current.previous),[e])}var Cxe=[" ","Enter","ArrowUp","ArrowDown"],Txe=[" ","Enter"],zc="Select",[a2,i2,Axe]=wZ(zc),[gp]=Bo(zc,[Axe,mp]),o2=mp(),[Fxe,zu]=gp(zc),[_xe,Rxe]=gp(zc),CZ=e=>{const{__scopeSelect:t,children:r,open:n,defaultOpen:a,onOpenChange:i,value:o,defaultValue:s,onValueChange:l,dir:u,name:d,autoComplete:p,disabled:f,required:m,form:b}=e,y=o2(t),[S,w]=T.useState(null),[E,k]=T.useState(null),[_,R]=T.useState(!1),C=n2(u),[F,L]=bl({prop:n,defaultProp:a??!1,onChange:i,caller:zc}),[I,M]=bl({prop:o,defaultProp:s,onChange:l,caller:zc}),j=T.useRef(null),Y=S?b||!!S.closest("form"):!0,[W,$]=T.useState(new Set),X=Array.from(W).map(z=>z.props.value).join(";");return x.jsx(e5,{...y,children:x.jsxs(Fxe,{required:m,scope:t,trigger:S,onTriggerChange:w,valueNode:E,onValueNodeChange:k,valueNodeHasChildren:_,onValueNodeHasChildrenChange:R,contentId:La(),value:I,onValueChange:M,open:F,onOpenChange:L,dir:C,triggerPointerDownPosRef:j,disabled:f,children:[x.jsx(a2.Provider,{scope:t,children:x.jsx(_xe,{scope:e.__scopeSelect,onNativeOptionAdd:T.useCallback(z=>{$(H=>new Set(H).add(z))},[]),onNativeOptionRemove:T.useCallback(z=>{$(H=>{const B=new Set(H);return B.delete(z),B})},[]),children:r})}),Y?x.jsxs(JZ,{"aria-hidden":!0,required:m,tabIndex:-1,name:d,autoComplete:p,value:I,onChange:z=>M(z.target.value),disabled:f,form:b,children:[I===void 0?x.jsx("option",{value:""}):null,Array.from(W)]},X):null]})})};CZ.displayName=zc;var TZ="SelectTrigger",AZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,disabled:n=!1,...a}=e,i=o2(r),o=zu(TZ,r),s=o.disabled||n,l=Rr(t,o.onTriggerChange),u=i2(r),d=T.useRef("touch"),[p,f,m]=ZZ(y=>{const S=u().filter(k=>!k.disabled),w=S.find(k=>k.value===o.value),E=QZ(S,y,w);E!==void 0&&o.onValueChange(E.value)}),b=y=>{s||(o.onOpenChange(!0),m()),y&&(o.triggerPointerDownPosRef.current={x:Math.round(y.pageX),y:Math.round(y.pageY)})};return x.jsx(Kv,{asChild:!0,...i,children:x.jsx(Kt.button,{type:"button",role:"combobox","aria-controls":o.contentId,"aria-expanded":o.open,"aria-required":o.required,"aria-autocomplete":"none",dir:o.dir,"data-state":o.open?"open":"closed",disabled:s,"data-disabled":s?"":void 0,"data-placeholder":XZ(o.value)?"":void 0,...a,ref:l,onClick:$t(a.onClick,y=>{y.currentTarget.focus(),d.current!=="mouse"&&b(y)}),onPointerDown:$t(a.onPointerDown,y=>{d.current=y.pointerType;const S=y.target;S.hasPointerCapture(y.pointerId)&&S.releasePointerCapture(y.pointerId),y.button===0&&y.ctrlKey===!1&&y.pointerType==="mouse"&&(b(y),y.preventDefault())}),onKeyDown:$t(a.onKeyDown,y=>{const S=p.current!=="";!(y.ctrlKey||y.altKey||y.metaKey)&&y.key.length===1&&f(y.key),!(S&&y.key===" ")&&Cxe.includes(y.key)&&(b(),y.preventDefault())})})})});AZ.displayName=TZ;var FZ="SelectValue",_Z=T.forwardRef((e,t)=>{const{__scopeSelect:r,className:n,style:a,children:i,placeholder:o="",...s}=e,l=zu(FZ,r),{onValueNodeHasChildrenChange:u}=l,d=i!==void 0,p=Rr(t,l.onValueNodeChange);return Sa(()=>{u(d)},[u,d]),x.jsx(Kt.span,{...s,ref:p,style:{pointerEvents:"none"},children:XZ(l.value)?x.jsx(x.Fragment,{children:o}):i})});_Z.displayName=FZ;var Dxe="SelectIcon",RZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,children:n,...a}=e;return x.jsx(Kt.span,{"aria-hidden":!0,...a,ref:t,children:n||"▼"})});RZ.displayName=Dxe;var Nxe="SelectPortal",DZ=e=>x.jsx(zv,{asChild:!0,...e});DZ.displayName=Nxe;var Hc="SelectContent",NZ=T.forwardRef((e,t)=>{const r=zu(Hc,e.__scopeSelect),[n,a]=T.useState();if(Sa(()=>{a(new DocumentFragment)},[]),!r.open){const i=n;return i?mg.createPortal(x.jsx(LZ,{scope:e.__scopeSelect,children:x.jsx(a2.Slot,{scope:e.__scopeSelect,children:x.jsx("div",{children:e.children})})}),i):null}return x.jsx(MZ,{...e,ref:t})});NZ.displayName=Hc;var Eo=10,[LZ,Hu]=gp(Hc),Lxe="SelectContentImpl",Mxe=Oc("SelectContent.RemoveScroll"),MZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,position:n="item-aligned",onCloseAutoFocus:a,onEscapeKeyDown:i,onPointerDownOutside:o,side:s,sideOffset:l,align:u,alignOffset:d,arrowPadding:p,collisionBoundary:f,collisionPadding:m,sticky:b,hideWhenDetached:y,avoidCollisions:S,...w}=e,E=zu(Hc,r),[k,_]=T.useState(null),[R,C]=T.useState(null),F=Rr(t,ae=>_(ae)),[L,I]=T.useState(null),[M,j]=T.useState(null),Y=i2(r),[W,$]=T.useState(!1),X=T.useRef(!1);T.useEffect(()=>{if(k)return OD(k)},[k]),ID();const z=T.useCallback(ae=>{const[se,...Ee]=Y().map(Oe=>Oe.ref.current),[fe]=Ee.slice(-1),Me=document.activeElement;for(const Oe of ae)if(Oe===Me||(Oe==null||Oe.scrollIntoView({block:"nearest"}),Oe===se&&R&&(R.scrollTop=0),Oe===fe&&R&&(R.scrollTop=R.scrollHeight),Oe==null||Oe.focus(),document.activeElement!==Me))return},[Y,R]),H=T.useCallback(()=>z([L,k]),[z,L,k]);T.useEffect(()=>{W&&H()},[W,H]);const{onOpenChange:B,triggerPointerDownPosRef:ee}=E;T.useEffect(()=>{if(k){let ae={x:0,y:0};const se=fe=>{var Me,Oe;ae={x:Math.abs(Math.round(fe.pageX)-(((Me=ee.current)==null?void 0:Me.x)??0)),y:Math.abs(Math.round(fe.pageY)-(((Oe=ee.current)==null?void 0:Oe.y)??0))}},Ee=fe=>{ae.x<=10&&ae.y<=10?fe.preventDefault():k.contains(fe.target)||B(!1),document.removeEventListener("pointermove",se),ee.current=null};return ee.current!==null&&(document.addEventListener("pointermove",se),document.addEventListener("pointerup",Ee,{capture:!0,once:!0})),()=>{document.removeEventListener("pointermove",se),document.removeEventListener("pointerup",Ee,{capture:!0})}}},[k,B,ee]),T.useEffect(()=>{const ae=()=>B(!1);return window.addEventListener("blur",ae),window.addEventListener("resize",ae),()=>{window.removeEventListener("blur",ae),window.removeEventListener("resize",ae)}},[B]);const[J,K]=ZZ(ae=>{const se=Y().filter(Me=>!Me.disabled),Ee=se.find(Me=>Me.ref.current===document.activeElement),fe=QZ(se,ae,Ee);fe&&setTimeout(()=>fe.ref.current.focus())}),G=T.useCallback((ae,se,Ee)=>{const fe=!X.current&&!Ee;(E.value!==void 0&&E.value===se||fe)&&(I(ae),fe&&(X.current=!0))},[E.value]),P=T.useCallback(()=>k==null?void 0:k.focus(),[k]),ne=T.useCallback((ae,se,Ee)=>{const fe=!X.current&&!Ee;(E.value!==void 0&&E.value===se||fe)&&j(ae)},[E.value]),oe=n==="popper"?iR:IZ,pe=oe===iR?{side:s,sideOffset:l,align:u,alignOffset:d,arrowPadding:p,collisionBoundary:f,collisionPadding:m,sticky:b,hideWhenDetached:y,avoidCollisions:S}:{};return x.jsx(LZ,{scope:r,content:k,viewport:R,onViewportChange:C,itemRefCallback:G,selectedItem:L,onItemLeave:P,itemTextRefCallback:ne,focusSelectedItem:H,selectedItemText:M,position:n,isPositioned:W,searchRef:J,children:x.jsx(Uv,{as:Mxe,allowPinchZoom:!0,children:x.jsx(Pv,{asChild:!0,trapped:E.open,onMountAutoFocus:ae=>{ae.preventDefault()},onUnmountAutoFocus:$t(a,ae=>{var se;(se=E.trigger)==null||se.focus({preventScroll:!0}),ae.preventDefault()}),children:x.jsx(Sg,{asChild:!0,disableOutsidePointerEvents:!0,onEscapeKeyDown:i,onPointerDownOutside:o,onFocusOutside:ae=>ae.preventDefault(),onDismiss:()=>E.onOpenChange(!1),children:x.jsx(oe,{role:"listbox",id:E.contentId,"data-state":E.open?"open":"closed",dir:E.dir,onContextMenu:ae=>ae.preventDefault(),...w,...pe,onPlaced:()=>$(!0),ref:F,style:{display:"flex",flexDirection:"column",outline:"none",...w.style},onKeyDown:$t(w.onKeyDown,ae=>{const se=ae.ctrlKey||ae.altKey||ae.metaKey;if(ae.key==="Tab"&&ae.preventDefault(),!se&&ae.key.length===1&&K(ae.key),["ArrowUp","ArrowDown","Home","End"].includes(ae.key)){let fe=Y().filter(Me=>!Me.disabled).map(Me=>Me.ref.current);if(["ArrowUp","End"].includes(ae.key)&&(fe=fe.slice().reverse()),["ArrowUp","ArrowDown"].includes(ae.key)){const Me=ae.target,Oe=fe.indexOf(Me);fe=fe.slice(Oe+1)}setTimeout(()=>z(fe)),ae.preventDefault()}})})})})})})});MZ.displayName=Lxe;var Ixe="SelectItemAlignedPosition",IZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,onPlaced:n,...a}=e,i=zu(Hc,r),o=Hu(Hc,r),[s,l]=T.useState(null),[u,d]=T.useState(null),p=Rr(t,F=>d(F)),f=i2(r),m=T.useRef(!1),b=T.useRef(!0),{viewport:y,selectedItem:S,selectedItemText:w,focusSelectedItem:E}=o,k=T.useCallback(()=>{if(i.trigger&&i.valueNode&&s&&u&&y&&S&&w){const F=i.trigger.getBoundingClientRect(),L=u.getBoundingClientRect(),I=i.valueNode.getBoundingClientRect(),M=w.getBoundingClientRect();if(i.dir!=="rtl"){const Me=M.left-L.left,Oe=I.left-Me,xe=F.left-Oe,Te=F.width+xe,Ne=Math.max(Te,L.width),he=window.innerWidth-Eo,Ze=aR(Oe,[Eo,Math.max(Eo,he-Ne)]);s.style.minWidth=Te+"px",s.style.left=Ze+"px"}else{const Me=L.right-M.right,Oe=window.innerWidth-I.right-Me,xe=window.innerWidth-F.right-Oe,Te=F.width+xe,Ne=Math.max(Te,L.width),he=window.innerWidth-Eo,Ze=aR(Oe,[Eo,Math.max(Eo,he-Ne)]);s.style.minWidth=Te+"px",s.style.right=Ze+"px"}const j=f(),Y=window.innerHeight-Eo*2,W=y.scrollHeight,$=window.getComputedStyle(u),X=parseInt($.borderTopWidth,10),z=parseInt($.paddingTop,10),H=parseInt($.borderBottomWidth,10),B=parseInt($.paddingBottom,10),ee=X+z+W+B+H,J=Math.min(S.offsetHeight*5,ee),K=window.getComputedStyle(y),G=parseInt(K.paddingTop,10),P=parseInt(K.paddingBottom,10),ne=F.top+F.height/2-Eo,oe=Y-ne,pe=S.offsetHeight/2,ae=S.offsetTop+pe,se=X+z+ae,Ee=ee-se;if(se<=ne){const Me=j.length>0&&S===j[j.length-1].ref.current;s.style.bottom="0px";const Oe=u.clientHeight-y.offsetTop-y.offsetHeight,xe=Math.max(oe,pe+(Me?P:0)+Oe+H),Te=se+xe;s.style.height=Te+"px"}else{const Me=j.length>0&&S===j[0].ref.current;s.style.top="0px";const xe=Math.max(ne,X+y.offsetTop+(Me?G:0)+pe)+Ee;s.style.height=xe+"px",y.scrollTop=se-ne+y.offsetTop}s.style.margin=`${Eo}px 0`,s.style.minHeight=J+"px",s.style.maxHeight=Y+"px",n==null||n(),requestAnimationFrame(()=>m.current=!0)}},[f,i.trigger,i.valueNode,s,u,y,S,w,i.dir,n]);Sa(()=>k(),[k]);const[_,R]=T.useState();Sa(()=>{u&&R(window.getComputedStyle(u).zIndex)},[u]);const C=T.useCallback(F=>{F&&b.current===!0&&(k(),E==null||E(),b.current=!1)},[k,E]);return x.jsx(Bxe,{scope:r,contentWrapper:s,shouldExpandOnScrollRef:m,onScrollButtonChange:C,children:x.jsx("div",{ref:l,style:{display:"flex",flexDirection:"column",position:"fixed",zIndex:_},children:x.jsx(Kt.div,{...a,ref:p,style:{boxSizing:"border-box",maxHeight:"100%",...a.style}})})})});IZ.displayName=Ixe;var Oxe="SelectPopperPosition",iR=T.forwardRef((e,t)=>{const{__scopeSelect:r,align:n="start",collisionPadding:a=Eo,...i}=e,o=o2(r);return x.jsx(t5,{...o,...i,ref:t,align:n,collisionPadding:a,style:{boxSizing:"border-box",...i.style,"--radix-select-content-transform-origin":"var(--radix-popper-transform-origin)","--radix-select-content-available-width":"var(--radix-popper-available-width)","--radix-select-content-available-height":"var(--radix-popper-available-height)","--radix-select-trigger-width":"var(--radix-popper-anchor-width)","--radix-select-trigger-height":"var(--radix-popper-anchor-height)"}})});iR.displayName=Oxe;var[Bxe,d5]=gp(Hc,{}),oR="SelectViewport",OZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,nonce:n,...a}=e,i=Hu(oR,r),o=d5(oR,r),s=Rr(t,i.onViewportChange),l=T.useRef(0);return x.jsxs(x.Fragment,{children:[x.jsx("style",{dangerouslySetInnerHTML:{__html:"[data-radix-select-viewport]{scrollbar-width:none;-ms-overflow-style:none;-webkit-overflow-scrolling:touch;}[data-radix-select-viewport]::-webkit-scrollbar{display:none}"},nonce:n}),x.jsx(a2.Slot,{scope:r,children:x.jsx(Kt.div,{"data-radix-select-viewport":"",role:"presentation",...a,ref:s,style:{position:"relative",flex:1,overflow:"hidden auto",...a.style},onScroll:$t(a.onScroll,u=>{const d=u.currentTarget,{contentWrapper:p,shouldExpandOnScrollRef:f}=o;if(f!=null&&f.current&&p){const m=Math.abs(l.current-d.scrollTop);if(m>0){const b=window.innerHeight-Eo*2,y=parseFloat(p.style.minHeight),S=parseFloat(p.style.height),w=Math.max(y,S);if(w<b){const E=w+m,k=Math.min(b,E),_=E-k;p.style.height=k+"px",p.style.bottom==="0px"&&(d.scrollTop=_>0?_:0,p.style.justifyContent="flex-end")}}}l.current=d.scrollTop})})})]})});OZ.displayName=oR;var BZ="SelectGroup",[Pxe,zxe]=gp(BZ),PZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,...n}=e,a=La();return x.jsx(Pxe,{scope:r,id:a,children:x.jsx(Kt.div,{role:"group","aria-labelledby":a,...n,ref:t})})});PZ.displayName=BZ;var zZ="SelectLabel",HZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,...n}=e,a=zxe(zZ,r);return x.jsx(Kt.div,{id:a.id,...n,ref:t})});HZ.displayName=zZ;var gy="SelectItem",[Hxe,UZ]=gp(gy),$Z=T.forwardRef((e,t)=>{const{__scopeSelect:r,value:n,disabled:a=!1,textValue:i,...o}=e,s=zu(gy,r),l=Hu(gy,r),u=s.value===n,[d,p]=T.useState(i??""),[f,m]=T.useState(!1),b=Rr(t,E=>{var k;return(k=l.itemRefCallback)==null?void 0:k.call(l,E,n,a)}),y=La(),S=T.useRef("touch"),w=()=>{a||(s.onValueChange(n),s.onOpenChange(!1))};if(n==="")throw new Error("A <Select.Item /> must have a value prop that is not an empty string. This is because the Select value can be set to an empty string to clear the selection and show the placeholder.");return x.jsx(Hxe,{scope:r,value:n,disabled:a,textId:y,isSelected:u,onItemTextChange:T.useCallback(E=>{p(k=>k||((E==null?void 0:E.textContent)??"").trim())},[]),children:x.jsx(a2.ItemSlot,{scope:r,value:n,disabled:a,textValue:d,children:x.jsx(Kt.div,{role:"option","aria-labelledby":y,"data-highlighted":f?"":void 0,"aria-selected":u&&f,"data-state":u?"checked":"unchecked","aria-disabled":a||void 0,"data-disabled":a?"":void 0,tabIndex:a?void 0:-1,...o,ref:b,onFocus:$t(o.onFocus,()=>m(!0)),onBlur:$t(o.onBlur,()=>m(!1)),onClick:$t(o.onClick,()=>{S.current!=="mouse"&&w()}),onPointerUp:$t(o.onPointerUp,()=>{S.current==="mouse"&&w()}),onPointerDown:$t(o.onPointerDown,E=>{S.current=E.pointerType}),onPointerMove:$t(o.onPointerMove,E=>{var k;S.current=E.pointerType,a?(k=l.onItemLeave)==null||k.call(l):S.current==="mouse"&&E.currentTarget.focus({preventScroll:!0})}),onPointerLeave:$t(o.onPointerLeave,E=>{var k;E.currentTarget===document.activeElement&&((k=l.onItemLeave)==null||k.call(l))}),onKeyDown:$t(o.onKeyDown,E=>{var _;((_=l.searchRef)==null?void 0:_.current)!==""&&E.key===" "||(Txe.includes(E.key)&&w(),E.key===" "&&E.preventDefault())})})})})});$Z.displayName=gy;var nm="SelectItemText",GZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,className:n,style:a,...i}=e,o=zu(nm,r),s=Hu(nm,r),l=UZ(nm,r),u=Rxe(nm,r),[d,p]=T.useState(null),f=Rr(t,w=>p(w),l.onItemTextChange,w=>{var E;return(E=s.itemTextRefCallback)==null?void 0:E.call(s,w,l.value,l.disabled)}),m=d==null?void 0:d.textContent,b=T.useMemo(()=>x.jsx("option",{value:l.value,disabled:l.disabled,children:m},l.value),[l.disabled,l.value,m]),{onNativeOptionAdd:y,onNativeOptionRemove:S}=u;return Sa(()=>(y(b),()=>S(b)),[y,S,b]),x.jsxs(x.Fragment,{children:[x.jsx(Kt.span,{id:l.textId,...i,ref:f}),l.isSelected&&o.valueNode&&!o.valueNodeHasChildren?mg.createPortal(i.children,o.valueNode):null]})});GZ.displayName=nm;var jZ="SelectItemIndicator",qZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,...n}=e;return UZ(jZ,r).isSelected?x.jsx(Kt.span,{"aria-hidden":!0,...n,ref:t}):null});qZ.displayName=jZ;var sR="SelectScrollUpButton",WZ=T.forwardRef((e,t)=>{const r=Hu(sR,e.__scopeSelect),n=d5(sR,e.__scopeSelect),[a,i]=T.useState(!1),o=Rr(t,n.onScrollButtonChange);return Sa(()=>{if(r.viewport&&r.isPositioned){let s=function(){const u=l.scrollTop>0;i(u)};const l=r.viewport;return s(),l.addEventListener("scroll",s),()=>l.removeEventListener("scroll",s)}},[r.viewport,r.isPositioned]),a?x.jsx(KZ,{...e,ref:o,onAutoScroll:()=>{const{viewport:s,selectedItem:l}=r;s&&l&&(s.scrollTop=s.scrollTop-l.offsetHeight)}}):null});WZ.displayName=sR;var lR="SelectScrollDownButton",VZ=T.forwardRef((e,t)=>{const r=Hu(lR,e.__scopeSelect),n=d5(lR,e.__scopeSelect),[a,i]=T.useState(!1),o=Rr(t,n.onScrollButtonChange);return Sa(()=>{if(r.viewport&&r.isPositioned){let s=function(){const u=l.scrollHeight-l.clientHeight,d=Math.ceil(l.scrollTop)<u;i(d)};const l=r.viewport;return s(),l.addEventListener("scroll",s),()=>l.removeEventListener("scroll",s)}},[r.viewport,r.isPositioned]),a?x.jsx(KZ,{...e,ref:o,onAutoScroll:()=>{const{viewport:s,selectedItem:l}=r;s&&l&&(s.scrollTop=s.scrollTop+l.offsetHeight)}}):null});VZ.displayName=lR;var KZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,onAutoScroll:n,...a}=e,i=Hu("SelectScrollButton",r),o=T.useRef(null),s=i2(r),l=T.useCallback(()=>{o.current!==null&&(window.clearInterval(o.current),o.current=null)},[]);return T.useEffect(()=>()=>l(),[l]),Sa(()=>{var d;const u=s().find(p=>p.ref.current===document.activeElement);(d=u==null?void 0:u.ref.current)==null||d.scrollIntoView({block:"nearest"})},[s]),x.jsx(Kt.div,{"aria-hidden":!0,...a,ref:t,style:{flexShrink:0,...a.style},onPointerDown:$t(a.onPointerDown,()=>{o.current===null&&(o.current=window.setInterval(n,50))}),onPointerMove:$t(a.onPointerMove,()=>{var u;(u=i.onItemLeave)==null||u.call(i),o.current===null&&(o.current=window.setInterval(n,50))}),onPointerLeave:$t(a.onPointerLeave,()=>{l()})})}),Uxe="SelectSeparator",YZ=T.forwardRef((e,t)=>{const{__scopeSelect:r,...n}=e;return x.jsx(Kt.div,{"aria-hidden":!0,...n,ref:t})});YZ.displayName=Uxe;var uR="SelectArrow",$xe=T.forwardRef((e,t)=>{const{__scopeSelect:r,...n}=e,a=o2(r),i=zu(uR,r),o=Hu(uR,r);return i.open&&o.position==="popper"?x.jsx(r5,{...a,...n,ref:t}):null});$xe.displayName=uR;var Gxe="SelectBubbleInput",JZ=T.forwardRef(({__scopeSelect:e,value:t,...r},n)=>{const a=T.useRef(null),i=Rr(n,a),o=kZ(t);return T.useEffect(()=>{const s=a.current;if(!s)return;const l=window.HTMLSelectElement.prototype,d=Object.getOwnPropertyDescriptor(l,"value").set;if(o!==t&&d){const p=new Event("change",{bubbles:!0});d.call(s,t),s.dispatchEvent(p)}},[o,t]),x.jsx(Kt.select,{...r,style:{...qX,...r.style},ref:i,defaultValue:t})});JZ.displayName=Gxe;function XZ(e){return e===""||e===void 0}function ZZ(e){const t=Ga(e),r=T.useRef(""),n=T.useRef(0),a=T.useCallback(o=>{const s=r.current+o;t(s),(function l(u){r.current=u,window.clearTimeout(n.current),u!==""&&(n.current=window.setTimeout(()=>l(""),1e3))})(s)},[t]),i=T.useCallback(()=>{r.current="",window.clearTimeout(n.current)},[]);return T.useEffect(()=>()=>window.clearTimeout(n.current),[]),[r,a,i]}function QZ(e,t,r){const a=t.length>1&&Array.from(t).every(u=>u===t[0])?t[0]:t,i=r?e.indexOf(r):-1;let o=jxe(e,Math.max(i,0));a.length===1&&(o=o.filter(u=>u!==r));const l=o.find(u=>u.textValue.toLowerCase().startsWith(a.toLowerCase()));return l!==r?l:void 0}function jxe(e,t){return e.map((r,n)=>e[(t+n)%e.length])}var qxe=CZ,eQ=AZ,Wxe=_Z,Vxe=RZ,Kxe=DZ,tQ=NZ,Yxe=OZ,Jxe=PZ,rQ=HZ,nQ=$Z,Xxe=GZ,Zxe=qZ,aQ=WZ,iQ=VZ,oQ=YZ;const Uh=qxe,o7=Jxe,$h=Wxe,Uc=T.forwardRef(({className:e,children:t,...r},n)=>x.jsxs(eQ,{ref:n,className:lt("border-input bg-background ring-offset-background placeholder:text-muted-foreground focus:ring-ring flex h-10 w-full items-center justify-between rounded-md border px-3 py-2 text-sm focus:ring-2 focus:ring-offset-2 focus:outline-none disabled:cursor-not-allowed disabled:opacity-50 [&>span]:line-clamp-1",e),...r,children:[t,x.jsx(Vxe,{asChild:!0,children:x.jsx(Zv,{className:"h-4 w-4 opacity-50"})})]}));Uc.displayName=eQ.displayName;const sQ=T.forwardRef(({className:e,...t},r)=>x.jsx(aQ,{ref:r,className:lt("flex cursor-default items-center justify-center py-1",e),...t,children:x.jsx(QSe,{className:"h-4 w-4"})}));sQ.displayName=aQ.displayName;const lQ=T.forwardRef(({className:e,...t},r)=>x.jsx(iQ,{ref:r,className:lt("flex cursor-default items-center justify-center py-1",e),...t,children:x.jsx(Zv,{className:"h-4 w-4"})}));lQ.displayName=iQ.displayName;const $c=T.forwardRef(({className:e,children:t,position:r="popper",...n},a)=>x.jsx(Kxe,{children:x.jsxs(tQ,{ref:a,className:lt("bg-popover text-popover-foreground data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 data-[state=closed]:zoom-out-95 data-[state=open]:zoom-in-95 data-[side=bottom]:slide-in-from-top-2 data-[side=left]:slide-in-from-right-2 data-[side=right]:slide-in-from-left-2 data-[side=top]:slide-in-from-bottom-2 relative z-50 max-h-96 min-w-[8rem] overflow-hidden rounded-md border shadow-md",r==="popper"&&"data-[side=bottom]:translate-y-1 data-[side=left]:-translate-x-1 data-[side=right]:translate-x-1 data-[side=top]:-translate-y-1",e),position:r,...n,children:[x.jsx(sQ,{}),x.jsx(Yxe,{className:lt("p-1",r==="popper"&&"h-[var(--radix-select-trigger-height)] w-full min-w-[var(--radix-select-trigger-width)]"),children:t}),x.jsx(lQ,{})]})}));$c.displayName=tQ.displayName;const Qxe=T.forwardRef(({className:e,...t},r)=>x.jsx(rQ,{ref:r,className:lt("py-1.5 pr-2 pl-8 text-sm font-semibold",e),...t}));Qxe.displayName=rQ.displayName;const $n=T.forwardRef(({className:e,children:t,...r},n)=>x.jsxs(nQ,{ref:n,className:lt("focus:bg-accent focus:text-accent-foreground relative flex w-full cursor-default items-center rounded-sm py-1.5 pr-2 pl-8 text-sm outline-none select-none data-[disabled]:pointer-events-none data-[disabled]:opacity-50",e),...r,children:[x.jsx("span",{className:"absolute left-2 flex h-3.5 w-3.5 items-center justify-center",children:x.jsx(Zxe,{children:x.jsx(a5,{className:"h-4 w-4"})})}),x.jsx(Xxe,{children:t})]}));$n.displayName=nQ.displayName;const ewe=T.forwardRef(({className:e,...t},r)=>x.jsx(oQ,{ref:r,className:lt("bg-muted -mx-1 my-1 h-px",e),...t}));ewe.displayName=oQ.displayName;function uQ({className:e}){const[t,r]=T.useState(!1),{t:n}=Sr(),a=Ve.use.language(),i=Ve.use.setLanguage(),o=Ve.use.theme(),s=Ve.use.setTheme(),l=T.useCallback(d=>{i(d)},[i]),u=T.useCallback(d=>{s(d)},[s]);return x.jsxs(t2,{open:t,onOpenChange:r,children:[x.jsx(r2,{asChild:!0,children:x.jsx(Rt,{variant:"ghost",size:"icon",className:lt("h-9 w-9",e),children:x.jsx(REe,{className:"h-5 w-5"})})}),x.jsx(kg,{side:"bottom",align:"end",className:"w-56",children:x.jsxs("div",{className:"flex flex-col gap-4",children:[x.jsxs("div",{className:"flex flex-col gap-2",children:[x.jsx("label",{className:"text-sm font-medium",children:n("settings.language")}),x.jsxs(Uh,{value:a,onValueChange:l,children:[x.jsx(Uc,{children:x.jsx($h,{})}),x.jsxs($c,{children:[x.jsx($n,{value:"en",children:"English"}),x.jsx($n,{value:"zh",children:"中文"}),x.jsx($n,{value:"fr",children:"Français"}),x.jsx($n,{value:"ar",children:"العربية"}),x.jsx($n,{value:"zh_TW",children:"繁體中文"})]})]})]}),x.jsxs("div",{className:"flex flex-col gap-2",children:[x.jsx("label",{className:"text-sm font-medium",children:n("settings.theme")}),x.jsxs(Uh,{value:o,onValueChange:u,children:[x.jsx(Uc,{children:x.jsx($h,{})}),x.jsxs($c,{children:[x.jsx($n,{value:"light",children:n("settings.light")}),x.jsx($n,{value:"dark",children:n("settings.dark")}),x.jsx($n,{value:"system",children:n("settings.system")})]})]})]})]})})]})}var tw="rovingFocusGroup.onEntryFocus",twe={bubbles:!1,cancelable:!0},Cg="RovingFocusGroup",[cR,cQ,rwe]=wZ(Cg),[nwe,dQ]=Bo(Cg,[rwe]),[awe,iwe]=nwe(Cg),hQ=T.forwardRef((e,t)=>x.jsx(cR.Provider,{scope:e.__scopeRovingFocusGroup,children:x.jsx(cR.Slot,{scope:e.__scopeRovingFocusGroup,children:x.jsx(owe,{...e,ref:t})})}));hQ.displayName=Cg;var owe=T.forwardRef((e,t)=>{const{__scopeRovingFocusGroup:r,orientation:n,loop:a=!1,dir:i,currentTabStopId:o,defaultCurrentTabStopId:s,onCurrentTabStopIdChange:l,onEntryFocus:u,preventScrollOnEntryFocus:d=!1,...p}=e,f=T.useRef(null),m=Rr(t,f),b=n2(i),[y,S]=bl({prop:o,defaultProp:s??null,onChange:l,caller:Cg}),[w,E]=T.useState(!1),k=Ga(u),_=cQ(r),R=T.useRef(!1),[C,F]=T.useState(0);return T.useEffect(()=>{const L=f.current;if(L)return L.addEventListener(tw,k),()=>L.removeEventListener(tw,k)},[k]),x.jsx(awe,{scope:r,orientation:n,dir:b,loop:a,currentTabStopId:y,onItemFocus:T.useCallback(L=>S(L),[S]),onItemShiftTab:T.useCallback(()=>E(!0),[]),onFocusableItemAdd:T.useCallback(()=>F(L=>L+1),[]),onFocusableItemRemove:T.useCallback(()=>F(L=>L-1),[]),children:x.jsx(Kt.div,{tabIndex:w||C===0?-1:0,"data-orientation":n,...p,ref:m,style:{outline:"none",...e.style},onMouseDown:$t(e.onMouseDown,()=>{R.current=!0}),onFocus:$t(e.onFocus,L=>{const I=!R.current;if(L.target===L.currentTarget&&I&&!w){const M=new CustomEvent(tw,twe);if(L.currentTarget.dispatchEvent(M),!M.defaultPrevented){const j=_().filter(z=>z.focusable),Y=j.find(z=>z.active),W=j.find(z=>z.id===y),X=[Y,W,...j].filter(Boolean).map(z=>z.ref.current);mQ(X,d)}}R.current=!1}),onBlur:$t(e.onBlur,()=>E(!1))})})}),pQ="RovingFocusGroupItem",fQ=T.forwardRef((e,t)=>{const{__scopeRovingFocusGroup:r,focusable:n=!0,active:a=!1,tabStopId:i,children:o,...s}=e,l=La(),u=i||l,d=iwe(pQ,r),p=d.currentTabStopId===u,f=cQ(r),{onFocusableItemAdd:m,onFocusableItemRemove:b,currentTabStopId:y}=d;return T.useEffect(()=>{if(n)return m(),()=>b()},[n,m,b]),x.jsx(cR.ItemSlot,{scope:r,id:u,focusable:n,active:a,children:x.jsx(Kt.span,{tabIndex:p?0:-1,"data-orientation":d.orientation,...s,ref:t,onMouseDown:$t(e.onMouseDown,S=>{n?d.onItemFocus(u):S.preventDefault()}),onFocus:$t(e.onFocus,()=>d.onItemFocus(u)),onKeyDown:$t(e.onKeyDown,S=>{if(S.key==="Tab"&&S.shiftKey){d.onItemShiftTab();return}if(S.target!==S.currentTarget)return;const w=uwe(S,d.orientation,d.dir);if(w!==void 0){if(S.metaKey||S.ctrlKey||S.altKey||S.shiftKey)return;S.preventDefault();let k=f().filter(_=>_.focusable).map(_=>_.ref.current);if(w==="last")k.reverse();else if(w==="prev"||w==="next"){w==="prev"&&k.reverse();const _=k.indexOf(S.currentTarget);k=d.loop?cwe(k,_+1):k.slice(_+1)}setTimeout(()=>mQ(k))}}),children:typeof o=="function"?o({isCurrentTabStop:p,hasTabStop:y!=null}):o})})});fQ.displayName=pQ;var swe={ArrowLeft:"prev",ArrowUp:"prev",ArrowRight:"next",ArrowDown:"next",PageUp:"first",Home:"first",PageDown:"last",End:"last"};function lwe(e,t){return t!=="rtl"?e:e==="ArrowLeft"?"ArrowRight":e==="ArrowRight"?"ArrowLeft":e}function uwe(e,t,r){const n=lwe(e.key,r);if(!(t==="vertical"&&["ArrowLeft","ArrowRight"].includes(n))&&!(t==="horizontal"&&["ArrowUp","ArrowDown"].includes(n)))return swe[n]}function mQ(e,t=!1){const r=document.activeElement;for(const n of e)if(n===r||(n.focus({preventScroll:t}),document.activeElement!==r))return}function cwe(e,t){return e.map((r,n)=>e[(t+n)%e.length])}var dwe=hQ,hwe=fQ,s2="Tabs",[pwe]=Bo(s2,[dQ]),gQ=dQ(),[fwe,h5]=pwe(s2),bQ=T.forwardRef((e,t)=>{const{__scopeTabs:r,value:n,onValueChange:a,defaultValue:i,orientation:o="horizontal",dir:s,activationMode:l="automatic",...u}=e,d=n2(s),[p,f]=bl({prop:n,onChange:a,defaultProp:i??"",caller:s2});return x.jsx(fwe,{scope:r,baseId:La(),value:p,onValueChange:f,orientation:o,dir:d,activationMode:l,children:x.jsx(Kt.div,{dir:d,"data-orientation":o,...u,ref:t})})});bQ.displayName=s2;var yQ="TabsList",vQ=T.forwardRef((e,t)=>{const{__scopeTabs:r,loop:n=!0,...a}=e,i=h5(yQ,r),o=gQ(r);return x.jsx(dwe,{asChild:!0,...o,orientation:i.orientation,dir:i.dir,loop:n,children:x.jsx(Kt.div,{role:"tablist","aria-orientation":i.orientation,...a,ref:t})})});vQ.displayName=yQ;var SQ="TabsTrigger",EQ=T.forwardRef((e,t)=>{const{__scopeTabs:r,value:n,disabled:a=!1,...i}=e,o=h5(SQ,r),s=gQ(r),l=kQ(o.baseId,n),u=CQ(o.baseId,n),d=n===o.value;return x.jsx(hwe,{asChild:!0,...s,focusable:!a,active:d,children:x.jsx(Kt.button,{type:"button",role:"tab","aria-selected":d,"aria-controls":u,"data-state":d?"active":"inactive","data-disabled":a?"":void 0,disabled:a,id:l,...i,ref:t,onMouseDown:$t(e.onMouseDown,p=>{!a&&p.button===0&&p.ctrlKey===!1?o.onValueChange(n):p.preventDefault()}),onKeyDown:$t(e.onKeyDown,p=>{[" ","Enter"].includes(p.key)&&o.onValueChange(n)}),onFocus:$t(e.onFocus,()=>{const p=o.activationMode!=="manual";!d&&!a&&p&&o.onValueChange(n)})})})});EQ.displayName=SQ;var xQ="TabsContent",wQ=T.forwardRef((e,t)=>{const{__scopeTabs:r,value:n,forceMount:a,children:i,...o}=e,s=h5(xQ,r),l=kQ(s.baseId,n),u=CQ(s.baseId,n),d=n===s.value,p=T.useRef(d);return T.useEffect(()=>{const f=requestAnimationFrame(()=>p.current=!1);return()=>cancelAnimationFrame(f)},[]),x.jsx($i,{present:a||d,children:({present:f})=>x.jsx(Kt.div,{"data-state":d?"active":"inactive","data-orientation":s.orientation,role:"tabpanel","aria-labelledby":l,hidden:!f,id:u,tabIndex:0,...o,ref:t,style:{...e.style,animationDuration:p.current?"0s":void 0},children:f&&i})})});wQ.displayName=xQ;function kQ(e,t){return`${e}-trigger-${t}`}function CQ(e,t){return`${e}-content-${t}`}var mwe=bQ,TQ=vQ,AQ=EQ,FQ=wQ;const gwe=mwe,_Q=T.forwardRef(({className:e,...t},r)=>x.jsx(TQ,{ref:r,className:lt("bg-muted text-muted-foreground inline-flex h-10 items-center justify-center rounded-md p-1",e),...t}));_Q.displayName=TQ.displayName;const RQ=T.forwardRef(({className:e,...t},r)=>x.jsx(AQ,{ref:r,className:lt("ring-offset-background focus-visible:ring-ring data-[state=active]:bg-background data-[state=active]:text-foreground inline-flex items-center justify-center rounded-sm px-3 py-1.5 text-sm font-medium whitespace-nowrap transition-all focus-visible:ring-2 focus-visible:ring-offset-2 focus-visible:outline-none disabled:pointer-events-none disabled:opacity-50 data-[state=active]:shadow-sm",e),...t}));RQ.displayName=AQ.displayName;const am=T.forwardRef(({className:e,...t},r)=>x.jsx(FQ,{ref:r,className:lt("ring-offset-background focus-visible:ring-ring focus-visible:ring-2 focus-visible:ring-offset-2 focus-visible:outline-none","data-[state=inactive]:invisible data-[state=active]:visible","h-full w-full",e),forceMount:!0,...t}));am.displayName=FQ.displayName;function Lb({value:e,currentTab:t,children:r}){return x.jsx(RQ,{value:e,className:lt("cursor-pointer px-2 py-1 transition-all",t===e?"!bg-emerald-400 !text-zinc-50":"hover:bg-background/60"),children:r})}function bwe(){const e=Ve.use.currentTab(),{t}=Sr();return x.jsx("div",{className:"flex h-8 self-center",children:x.jsxs(_Q,{className:"h-full gap-2",children:[x.jsx(Lb,{value:"documents",currentTab:e,children:t("header.documents")}),x.jsx(Lb,{value:"knowledge-graph",currentTab:e,children:t("header.knowledgeGraph")}),x.jsx(Lb,{value:"retrieval",currentTab:e,children:t("header.retrieval")}),x.jsx(Lb,{value:"api",currentTab:e,children:t("header.api")})]})})}function ywe(){const{t:e}=Sr(),{isGuestMode:t,coreVersion:r,apiVersion:n,username:a,webuiTitle:i,webuiDescription:o}=Oi(),s=r&&n?`${r}/${n}`:null,l=()=>{Bv.navigateToLogin()};return x.jsxs("header",{className:"border-border/40 bg-background/95 supports-[backdrop-filter]:bg-background/60 sticky top-0 z-50 flex h-10 w-full border-b px-4 backdrop-blur",children:[x.jsxs("div",{className:"min-w-[200px] w-auto flex items-center",children:[x.jsxs("a",{href:fJ,className:"flex items-center gap-2",children:[x.jsx(l5,{className:"size-4 text-emerald-400","aria-hidden":"true"}),x.jsx("span",{className:"font-bold md:inline-block",children:j_.name})]}),i&&x.jsxs("div",{className:"flex items-center",children:[x.jsx("span",{className:"mx-1 text-xs text-gray-500 dark:text-gray-400",children:"|"}),x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("span",{className:"font-medium text-sm cursor-default",children:i})}),o&&x.jsx(ga,{side:"bottom",children:o})]})})]})]}),x.jsxs("div",{className:"flex h-10 flex-1 items-center justify-center",children:[x.jsx(bwe,{}),t&&x.jsx("div",{className:"ml-2 self-center px-2 py-1 text-xs bg-amber-100 text-amber-800 dark:bg-amber-900 dark:text-amber-200 rounded-md",children:e("login.guestMode","Guest Mode")})]}),x.jsx("nav",{className:"w-[200px] flex items-center justify-end",children:x.jsxs("div",{className:"flex items-center gap-2",children:[s&&x.jsxs("span",{className:"text-xs text-gray-500 dark:text-gray-400 mr-1",children:["v",s]}),x.jsx(Rt,{variant:"ghost",size:"icon",side:"bottom",tooltip:e("header.projectRepository"),children:x.jsx("a",{href:j_.github,target:"_blank",rel:"noopener noreferrer",children:x.jsx(gEe,{className:"size-4","aria-hidden":"true"})})}),x.jsx(uQ,{}),!t&&x.jsx(Rt,{variant:"ghost",size:"icon",side:"bottom",tooltip:`${e("header.logout")} (${a})`,onClick:l,children:x.jsx(kEe,{className:"size-4","aria-hidden":"true"})})]})})]})}var Mb={exports:{}},s7;function vwe(){if(s7)return Mb.exports;s7=1;var e=typeof Reflect=="object"?Reflect:null,t=e&&typeof e.apply=="function"?e.apply:function(R,C,F){return Function.prototype.apply.call(R,C,F)},r;e&&typeof e.ownKeys=="function"?r=e.ownKeys:Object.getOwnPropertySymbols?r=function(R){return Object.getOwnPropertyNames(R).concat(Object.getOwnPropertySymbols(R))}:r=function(R){return Object.getOwnPropertyNames(R)};function n(_){console&&console.warn&&console.warn(_)}var a=Number.isNaN||function(R){return R!==R};function i(){i.init.call(this)}Mb.exports=i,Mb.exports.once=w,i.EventEmitter=i,i.prototype._events=void 0,i.prototype._eventsCount=0,i.prototype._maxListeners=void 0;var o=10;function s(_){if(typeof _!="function")throw new TypeError('The "listener" argument must be of type Function. Received type '+typeof _)}Object.defineProperty(i,"defaultMaxListeners",{enumerable:!0,get:function(){return o},set:function(_){if(typeof _!="number"||_<0||a(_))throw new RangeError('The value of "defaultMaxListeners" is out of range. It must be a non-negative number. Received '+_+".");o=_}}),i.init=function(){(this._events===void 0||this._events===Object.getPrototypeOf(this)._events)&&(this._events=Object.create(null),this._eventsCount=0),this._maxListeners=this._maxListeners||void 0},i.prototype.setMaxListeners=function(R){if(typeof R!="number"||R<0||a(R))throw new RangeError('The value of "n" is out of range. It must be a non-negative number. Received '+R+".");return this._maxListeners=R,this};function l(_){return _._maxListeners===void 0?i.defaultMaxListeners:_._maxListeners}i.prototype.getMaxListeners=function(){return l(this)},i.prototype.emit=function(R){for(var C=[],F=1;F<arguments.length;F++)C.push(arguments[F]);var L=R==="error",I=this._events;if(I!==void 0)L=L&&I.error===void 0;else if(!L)return!1;if(L){var M;if(C.length>0&&(M=C[0]),M instanceof Error)throw M;var j=new Error("Unhandled error."+(M?" ("+M.message+")":""));throw j.context=M,j}var Y=I[R];if(Y===void 0)return!1;if(typeof Y=="function")t(Y,this,C);else for(var W=Y.length,$=b(Y,W),F=0;F<W;++F)t($[F],this,C);return!0};function u(_,R,C,F){var L,I,M;if(s(C),I=_._events,I===void 0?(I=_._events=Object.create(null),_._eventsCount=0):(I.newListener!==void 0&&(_.emit("newListener",R,C.listener?C.listener:C),I=_._events),M=I[R]),M===void 0)M=I[R]=C,++_._eventsCount;else if(typeof M=="function"?M=I[R]=F?[C,M]:[M,C]:F?M.unshift(C):M.push(C),L=l(_),L>0&&M.length>L&&!M.warned){M.warned=!0;var j=new Error("Possible EventEmitter memory leak detected. "+M.length+" "+String(R)+" listeners added. Use emitter.setMaxListeners() to increase limit");j.name="MaxListenersExceededWarning",j.emitter=_,j.type=R,j.count=M.length,n(j)}return _}i.prototype.addListener=function(R,C){return u(this,R,C,!1)},i.prototype.on=i.prototype.addListener,i.prototype.prependListener=function(R,C){return u(this,R,C,!0)};function d(){if(!this.fired)return this.target.removeListener(this.type,this.wrapFn),this.fired=!0,arguments.length===0?this.listener.call(this.target):this.listener.apply(this.target,arguments)}function p(_,R,C){var F={fired:!1,wrapFn:void 0,target:_,type:R,listener:C},L=d.bind(F);return L.listener=C,F.wrapFn=L,L}i.prototype.once=function(R,C){return s(C),this.on(R,p(this,R,C)),this},i.prototype.prependOnceListener=function(R,C){return s(C),this.prependListener(R,p(this,R,C)),this},i.prototype.removeListener=function(R,C){var F,L,I,M,j;if(s(C),L=this._events,L===void 0)return this;if(F=L[R],F===void 0)return this;if(F===C||F.listener===C)--this._eventsCount===0?this._events=Object.create(null):(delete L[R],L.removeListener&&this.emit("removeListener",R,F.listener||C));else if(typeof F!="function"){for(I=-1,M=F.length-1;M>=0;M--)if(F[M]===C||F[M].listener===C){j=F[M].listener,I=M;break}if(I<0)return this;I===0?F.shift():y(F,I),F.length===1&&(L[R]=F[0]),L.removeListener!==void 0&&this.emit("removeListener",R,j||C)}return this},i.prototype.off=i.prototype.removeListener,i.prototype.removeAllListeners=function(R){var C,F,L;if(F=this._events,F===void 0)return this;if(F.removeListener===void 0)return arguments.length===0?(this._events=Object.create(null),this._eventsCount=0):F[R]!==void 0&&(--this._eventsCount===0?this._events=Object.create(null):delete F[R]),this;if(arguments.length===0){var I=Object.keys(F),M;for(L=0;L<I.length;++L)M=I[L],M!=="removeListener"&&this.removeAllListeners(M);return this.removeAllListeners("removeListener"),this._events=Object.create(null),this._eventsCount=0,this}if(C=F[R],typeof C=="function")this.removeListener(R,C);else if(C!==void 0)for(L=C.length-1;L>=0;L--)this.removeListener(R,C[L]);return this};function f(_,R,C){var F=_._events;if(F===void 0)return[];var L=F[R];return L===void 0?[]:typeof L=="function"?C?[L.listener||L]:[L]:C?S(L):b(L,L.length)}i.prototype.listeners=function(R){return f(this,R,!0)},i.prototype.rawListeners=function(R){return f(this,R,!1)},i.listenerCount=function(_,R){return typeof _.listenerCount=="function"?_.listenerCount(R):m.call(_,R)},i.prototype.listenerCount=m;function m(_){var R=this._events;if(R!==void 0){var C=R[_];if(typeof C=="function")return 1;if(C!==void 0)return C.length}return 0}i.prototype.eventNames=function(){return this._eventsCount>0?r(this._events):[]};function b(_,R){for(var C=new Array(R),F=0;F<R;++F)C[F]=_[F];return C}function y(_,R){for(;R+1<_.length;R++)_[R]=_[R+1];_.pop()}function S(_){for(var R=new Array(_.length),C=0;C<R.length;++C)R[C]=_[C].listener||_[C];return R}function w(_,R){return new Promise(function(C,F){function L(M){_.removeListener(R,I),F(M)}function I(){typeof _.removeListener=="function"&&_.removeListener("error",L),C([].slice.call(arguments))}k(_,R,I,{once:!0}),R!=="error"&&E(_,L,{once:!0})})}function E(_,R,C){typeof _.on=="function"&&k(_,"error",R,C)}function k(_,R,C,F){if(typeof _.on=="function")F.once?_.once(R,C):_.on(R,C);else if(typeof _.addEventListener=="function")_.addEventListener(R,function L(I){F.once&&_.removeEventListener(R,L),C(I)});else throw new TypeError('The "emitter" argument must be of type EventEmitter. Received type '+typeof _)}return Mb.exports}var DQ=vwe();function Swe(){const e=arguments[0];for(let t=1,r=arguments.length;t<r;t++)if(arguments[t])for(const n in arguments[t])e[n]=arguments[t][n];return e}let Zn=Swe;typeof Object.assign=="function"&&(Zn=Object.assign);function io(e,t,r,n){const a=e._nodes.get(t);let i=null;return a&&(n==="mixed"?i=a.out&&a.out[r]||a.undirected&&a.undirected[r]:n==="directed"?i=a.out&&a.out[r]:i=a.undirected&&a.undirected[r]),i}function va(e){return typeof e=="object"&&e!==null}function NQ(e){let t;for(t in e)return!1;return!0}function Qi(e,t,r){Object.defineProperty(e,t,{enumerable:!1,configurable:!1,writable:!0,value:r})}function So(e,t,r){const n={enumerable:!0,configurable:!0};typeof r=="function"?n.get=r:(n.value=r,n.writable=!1),Object.defineProperty(e,t,n)}function l7(e){return!(!va(e)||e.attributes&&!Array.isArray(e.attributes))}function Ewe(){let e=Math.floor(Math.random()*256)&255;return()=>e++}function cl(){const e=arguments;let t=null,r=-1;return{[Symbol.iterator](){return this},next(){let n=null;do{if(t===null){if(r++,r>=e.length)return{done:!0};t=e[r][Symbol.iterator]()}if(n=t.next(),n.done){t=null;continue}break}while(!0);return n}}}function bp(){return{[Symbol.iterator](){return this},next(){return{done:!0}}}}class p5 extends Error{constructor(t){super(),this.name="GraphError",this.message=t}}class bt extends p5{constructor(t){super(t),this.name="InvalidArgumentsGraphError",typeof Error.captureStackTrace=="function"&&Error.captureStackTrace(this,bt.prototype.constructor)}}class ut extends p5{constructor(t){super(t),this.name="NotFoundGraphError",typeof Error.captureStackTrace=="function"&&Error.captureStackTrace(this,ut.prototype.constructor)}}class Xt extends p5{constructor(t){super(t),this.name="UsageGraphError",typeof Error.captureStackTrace=="function"&&Error.captureStackTrace(this,Xt.prototype.constructor)}}function LQ(e,t){this.key=e,this.attributes=t,this.clear()}LQ.prototype.clear=function(){this.inDegree=0,this.outDegree=0,this.undirectedDegree=0,this.undirectedLoops=0,this.directedLoops=0,this.in={},this.out={},this.undirected={}};function MQ(e,t){this.key=e,this.attributes=t,this.clear()}MQ.prototype.clear=function(){this.inDegree=0,this.outDegree=0,this.directedLoops=0,this.in={},this.out={}};function IQ(e,t){this.key=e,this.attributes=t,this.clear()}IQ.prototype.clear=function(){this.undirectedDegree=0,this.undirectedLoops=0,this.undirected={}};function yp(e,t,r,n,a){this.key=t,this.attributes=a,this.undirected=e,this.source=r,this.target=n}yp.prototype.attach=function(){let e="out",t="in";this.undirected&&(e=t="undirected");const r=this.source.key,n=this.target.key;this.source[e][n]=this,!(this.undirected&&r===n)&&(this.target[t][r]=this)};yp.prototype.attachMulti=function(){let e="out",t="in";const r=this.source.key,n=this.target.key;this.undirected&&(e=t="undirected");const a=this.source[e],i=a[n];if(typeof i>"u"){a[n]=this,this.undirected&&r===n||(this.target[t][r]=this);return}i.previous=this,this.next=i,a[n]=this,this.target[t][r]=this};yp.prototype.detach=function(){const e=this.source.key,t=this.target.key;let r="out",n="in";this.undirected&&(r=n="undirected"),delete this.source[r][t],delete this.target[n][e]};yp.prototype.detachMulti=function(){const e=this.source.key,t=this.target.key;let r="out",n="in";this.undirected&&(r=n="undirected"),this.previous===void 0?this.next===void 0?(delete this.source[r][t],delete this.target[n][e]):(this.next.previous=void 0,this.source[r][t]=this.next,this.target[n][e]=this.next):(this.previous.next=this.next,this.next!==void 0&&(this.next.previous=this.previous))};const OQ=0,BQ=1,xwe=2,PQ=3;function _l(e,t,r,n,a,i,o){let s,l,u,d;if(n=""+n,r===OQ){if(s=e._nodes.get(n),!s)throw new ut(`Graph.${t}: could not find the "${n}" node in the graph.`);u=a,d=i}else if(r===PQ){if(a=""+a,l=e._edges.get(a),!l)throw new ut(`Graph.${t}: could not find the "${a}" edge in the graph.`);const p=l.source.key,f=l.target.key;if(n===p)s=l.target;else if(n===f)s=l.source;else throw new ut(`Graph.${t}: the "${n}" node is not attached to the "${a}" edge (${p}, ${f}).`);u=i,d=o}else{if(l=e._edges.get(n),!l)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`);r===BQ?s=l.source:s=l.target,u=a,d=i}return[s,u,d]}function wwe(e,t,r){e.prototype[t]=function(n,a,i){const[o,s]=_l(this,t,r,n,a,i);return o.attributes[s]}}function kwe(e,t,r){e.prototype[t]=function(n,a){const[i]=_l(this,t,r,n,a);return i.attributes}}function Cwe(e,t,r){e.prototype[t]=function(n,a,i){const[o,s]=_l(this,t,r,n,a,i);return o.attributes.hasOwnProperty(s)}}function Twe(e,t,r){e.prototype[t]=function(n,a,i,o){const[s,l,u]=_l(this,t,r,n,a,i,o);return s.attributes[l]=u,this.emit("nodeAttributesUpdated",{key:s.key,type:"set",attributes:s.attributes,name:l}),this}}function Awe(e,t,r){e.prototype[t]=function(n,a,i,o){const[s,l,u]=_l(this,t,r,n,a,i,o);if(typeof u!="function")throw new bt(`Graph.${t}: updater should be a function.`);const d=s.attributes,p=u(d[l]);return d[l]=p,this.emit("nodeAttributesUpdated",{key:s.key,type:"set",attributes:s.attributes,name:l}),this}}function Fwe(e,t,r){e.prototype[t]=function(n,a,i){const[o,s]=_l(this,t,r,n,a,i);return delete o.attributes[s],this.emit("nodeAttributesUpdated",{key:o.key,type:"remove",attributes:o.attributes,name:s}),this}}function _we(e,t,r){e.prototype[t]=function(n,a,i){const[o,s]=_l(this,t,r,n,a,i);if(!va(s))throw new bt(`Graph.${t}: provided attributes are not a plain object.`);return o.attributes=s,this.emit("nodeAttributesUpdated",{key:o.key,type:"replace",attributes:o.attributes}),this}}function Rwe(e,t,r){e.prototype[t]=function(n,a,i){const[o,s]=_l(this,t,r,n,a,i);if(!va(s))throw new bt(`Graph.${t}: provided attributes are not a plain object.`);return Zn(o.attributes,s),this.emit("nodeAttributesUpdated",{key:o.key,type:"merge",attributes:o.attributes,data:s}),this}}function Dwe(e,t,r){e.prototype[t]=function(n,a,i){const[o,s]=_l(this,t,r,n,a,i);if(typeof s!="function")throw new bt(`Graph.${t}: provided updater is not a function.`);return o.attributes=s(o.attributes),this.emit("nodeAttributesUpdated",{key:o.key,type:"update",attributes:o.attributes}),this}}const Nwe=[{name:e=>`get${e}Attribute`,attacher:wwe},{name:e=>`get${e}Attributes`,attacher:kwe},{name:e=>`has${e}Attribute`,attacher:Cwe},{name:e=>`set${e}Attribute`,attacher:Twe},{name:e=>`update${e}Attribute`,attacher:Awe},{name:e=>`remove${e}Attribute`,attacher:Fwe},{name:e=>`replace${e}Attributes`,attacher:_we},{name:e=>`merge${e}Attributes`,attacher:Rwe},{name:e=>`update${e}Attributes`,attacher:Dwe}];function Lwe(e){Nwe.forEach(function({name:t,attacher:r}){r(e,t("Node"),OQ),r(e,t("Source"),BQ),r(e,t("Target"),xwe),r(e,t("Opposite"),PQ)})}function Mwe(e,t,r){e.prototype[t]=function(n,a){let i;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>2){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const o=""+n,s=""+a;if(a=arguments[2],i=io(this,o,s,r),!i)throw new ut(`Graph.${t}: could not find an edge for the given path ("${o}" - "${s}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,i=this._edges.get(n),!i)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}return i.attributes[a]}}function Iwe(e,t,r){e.prototype[t]=function(n){let a;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>1){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const i=""+n,o=""+arguments[1];if(a=io(this,i,o,r),!a)throw new ut(`Graph.${t}: could not find an edge for the given path ("${i}" - "${o}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,a=this._edges.get(n),!a)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}return a.attributes}}function Owe(e,t,r){e.prototype[t]=function(n,a){let i;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>2){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const o=""+n,s=""+a;if(a=arguments[2],i=io(this,o,s,r),!i)throw new ut(`Graph.${t}: could not find an edge for the given path ("${o}" - "${s}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,i=this._edges.get(n),!i)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}return i.attributes.hasOwnProperty(a)}}function Bwe(e,t,r){e.prototype[t]=function(n,a,i){let o;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>3){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const s=""+n,l=""+a;if(a=arguments[2],i=arguments[3],o=io(this,s,l,r),!o)throw new ut(`Graph.${t}: could not find an edge for the given path ("${s}" - "${l}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,o=this._edges.get(n),!o)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}return o.attributes[a]=i,this.emit("edgeAttributesUpdated",{key:o.key,type:"set",attributes:o.attributes,name:a}),this}}function Pwe(e,t,r){e.prototype[t]=function(n,a,i){let o;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>3){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const s=""+n,l=""+a;if(a=arguments[2],i=arguments[3],o=io(this,s,l,r),!o)throw new ut(`Graph.${t}: could not find an edge for the given path ("${s}" - "${l}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,o=this._edges.get(n),!o)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}if(typeof i!="function")throw new bt(`Graph.${t}: updater should be a function.`);return o.attributes[a]=i(o.attributes[a]),this.emit("edgeAttributesUpdated",{key:o.key,type:"set",attributes:o.attributes,name:a}),this}}function zwe(e,t,r){e.prototype[t]=function(n,a){let i;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>2){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const o=""+n,s=""+a;if(a=arguments[2],i=io(this,o,s,r),!i)throw new ut(`Graph.${t}: could not find an edge for the given path ("${o}" - "${s}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,i=this._edges.get(n),!i)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}return delete i.attributes[a],this.emit("edgeAttributesUpdated",{key:i.key,type:"remove",attributes:i.attributes,name:a}),this}}function Hwe(e,t,r){e.prototype[t]=function(n,a){let i;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>2){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const o=""+n,s=""+a;if(a=arguments[2],i=io(this,o,s,r),!i)throw new ut(`Graph.${t}: could not find an edge for the given path ("${o}" - "${s}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,i=this._edges.get(n),!i)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}if(!va(a))throw new bt(`Graph.${t}: provided attributes are not a plain object.`);return i.attributes=a,this.emit("edgeAttributesUpdated",{key:i.key,type:"replace",attributes:i.attributes}),this}}function Uwe(e,t,r){e.prototype[t]=function(n,a){let i;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>2){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const o=""+n,s=""+a;if(a=arguments[2],i=io(this,o,s,r),!i)throw new ut(`Graph.${t}: could not find an edge for the given path ("${o}" - "${s}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,i=this._edges.get(n),!i)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}if(!va(a))throw new bt(`Graph.${t}: provided attributes are not a plain object.`);return Zn(i.attributes,a),this.emit("edgeAttributesUpdated",{key:i.key,type:"merge",attributes:i.attributes,data:a}),this}}function $we(e,t,r){e.prototype[t]=function(n,a){let i;if(this.type!=="mixed"&&r!=="mixed"&&r!==this.type)throw new Xt(`Graph.${t}: cannot find this type of edges in your ${this.type} graph.`);if(arguments.length>2){if(this.multi)throw new Xt(`Graph.${t}: cannot use a {source,target} combo when asking about an edge's attributes in a MultiGraph since we cannot infer the one you want information about.`);const o=""+n,s=""+a;if(a=arguments[2],i=io(this,o,s,r),!i)throw new ut(`Graph.${t}: could not find an edge for the given path ("${o}" - "${s}").`)}else{if(r!=="mixed")throw new Xt(`Graph.${t}: calling this method with only a key (vs. a source and target) does not make sense since an edge with this key could have the other type.`);if(n=""+n,i=this._edges.get(n),!i)throw new ut(`Graph.${t}: could not find the "${n}" edge in the graph.`)}if(typeof a!="function")throw new bt(`Graph.${t}: provided updater is not a function.`);return i.attributes=a(i.attributes),this.emit("edgeAttributesUpdated",{key:i.key,type:"update",attributes:i.attributes}),this}}const Gwe=[{name:e=>`get${e}Attribute`,attacher:Mwe},{name:e=>`get${e}Attributes`,attacher:Iwe},{name:e=>`has${e}Attribute`,attacher:Owe},{name:e=>`set${e}Attribute`,attacher:Bwe},{name:e=>`update${e}Attribute`,attacher:Pwe},{name:e=>`remove${e}Attribute`,attacher:zwe},{name:e=>`replace${e}Attributes`,attacher:Hwe},{name:e=>`merge${e}Attributes`,attacher:Uwe},{name:e=>`update${e}Attributes`,attacher:$we}];function jwe(e){Gwe.forEach(function({name:t,attacher:r}){r(e,t("Edge"),"mixed"),r(e,t("DirectedEdge"),"directed"),r(e,t("UndirectedEdge"),"undirected")})}const qwe=[{name:"edges",type:"mixed"},{name:"inEdges",type:"directed",direction:"in"},{name:"outEdges",type:"directed",direction:"out"},{name:"inboundEdges",type:"mixed",direction:"in"},{name:"outboundEdges",type:"mixed",direction:"out"},{name:"directedEdges",type:"directed"},{name:"undirectedEdges",type:"undirected"}];function Wwe(e,t,r,n){let a=!1;for(const i in t){if(i===n)continue;const o=t[i];if(a=r(o.key,o.attributes,o.source.key,o.target.key,o.source.attributes,o.target.attributes,o.undirected),e&&a)return o.key}}function Vwe(e,t,r,n){let a,i,o,s=!1;for(const l in t)if(l!==n){a=t[l];do{if(i=a.source,o=a.target,s=r(a.key,a.attributes,i.key,o.key,i.attributes,o.attributes,a.undirected),e&&s)return a.key;a=a.next}while(a!==void 0)}}function rw(e,t){const r=Object.keys(e),n=r.length;let a,i=0;return{[Symbol.iterator](){return this},next(){do if(a)a=a.next;else{if(i>=n)return{done:!0};const o=r[i++];if(o===t){a=void 0;continue}a=e[o]}while(!a);return{done:!1,value:{edge:a.key,attributes:a.attributes,source:a.source.key,target:a.target.key,sourceAttributes:a.source.attributes,targetAttributes:a.target.attributes,undirected:a.undirected}}}}}function Kwe(e,t,r,n){const a=t[r];if(!a)return;const i=a.source,o=a.target;if(n(a.key,a.attributes,i.key,o.key,i.attributes,o.attributes,a.undirected)&&e)return a.key}function Ywe(e,t,r,n){let a=t[r];if(!a)return;let i=!1;do{if(i=n(a.key,a.attributes,a.source.key,a.target.key,a.source.attributes,a.target.attributes,a.undirected),e&&i)return a.key;a=a.next}while(a!==void 0)}function nw(e,t){let r=e[t];if(r.next!==void 0)return{[Symbol.iterator](){return this},next(){if(!r)return{done:!0};const a={edge:r.key,attributes:r.attributes,source:r.source.key,target:r.target.key,sourceAttributes:r.source.attributes,targetAttributes:r.target.attributes,undirected:r.undirected};return r=r.next,{done:!1,value:a}}};let n=!1;return{[Symbol.iterator](){return this},next(){return n===!0?{done:!0}:(n=!0,{done:!1,value:{edge:r.key,attributes:r.attributes,source:r.source.key,target:r.target.key,sourceAttributes:r.source.attributes,targetAttributes:r.target.attributes,undirected:r.undirected}})}}}function Jwe(e,t){if(e.size===0)return[];if(t==="mixed"||t===e.type)return Array.from(e._edges.keys());const r=t==="undirected"?e.undirectedSize:e.directedSize,n=new Array(r),a=t==="undirected",i=e._edges.values();let o=0,s,l;for(;s=i.next(),s.done!==!0;)l=s.value,l.undirected===a&&(n[o++]=l.key);return n}function zQ(e,t,r,n){if(t.size===0)return;const a=r!=="mixed"&&r!==t.type,i=r==="undirected";let o,s,l=!1;const u=t._edges.values();for(;o=u.next(),o.done!==!0;){if(s=o.value,a&&s.undirected!==i)continue;const{key:d,attributes:p,source:f,target:m}=s;if(l=n(d,p,f.key,m.key,f.attributes,m.attributes,s.undirected),e&&l)return d}}function Xwe(e,t){if(e.size===0)return bp();const r=t!=="mixed"&&t!==e.type,n=t==="undirected",a=e._edges.values();return{[Symbol.iterator](){return this},next(){let i,o;for(;;){if(i=a.next(),i.done)return i;if(o=i.value,!(r&&o.undirected!==n))break}return{value:{edge:o.key,attributes:o.attributes,source:o.source.key,target:o.target.key,sourceAttributes:o.source.attributes,targetAttributes:o.target.attributes,undirected:o.undirected},done:!1}}}}function f5(e,t,r,n,a,i){const o=t?Vwe:Wwe;let s;if(r!=="undirected"&&(n!=="out"&&(s=o(e,a.in,i),e&&s)||n!=="in"&&(s=o(e,a.out,i,n?void 0:a.key),e&&s))||r!=="directed"&&(s=o(e,a.undirected,i),e&&s))return s}function Zwe(e,t,r,n){const a=[];return f5(!1,e,t,r,n,function(i){a.push(i)}),a}function Qwe(e,t,r){let n=bp();return e!=="undirected"&&(t!=="out"&&typeof r.in<"u"&&(n=cl(n,rw(r.in))),t!=="in"&&typeof r.out<"u"&&(n=cl(n,rw(r.out,t?void 0:r.key)))),e!=="directed"&&typeof r.undirected<"u"&&(n=cl(n,rw(r.undirected))),n}function m5(e,t,r,n,a,i,o){const s=r?Ywe:Kwe;let l;if(t!=="undirected"&&(typeof a.in<"u"&&n!=="out"&&(l=s(e,a.in,i,o),e&&l)||typeof a.out<"u"&&n!=="in"&&(n||a.key!==i)&&(l=s(e,a.out,i,o),e&&l))||t!=="directed"&&typeof a.undirected<"u"&&(l=s(e,a.undirected,i,o),e&&l))return l}function eke(e,t,r,n,a){const i=[];return m5(!1,e,t,r,n,a,function(o){i.push(o)}),i}function tke(e,t,r,n){let a=bp();return e!=="undirected"&&(typeof r.in<"u"&&t!=="out"&&n in r.in&&(a=cl(a,nw(r.in,n))),typeof r.out<"u"&&t!=="in"&&n in r.out&&(t||r.key!==n)&&(a=cl(a,nw(r.out,n)))),e!=="directed"&&typeof r.undirected<"u"&&n in r.undirected&&(a=cl(a,nw(r.undirected,n))),a}function rke(e,t){const{name:r,type:n,direction:a}=t;e.prototype[r]=function(i,o){if(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)return[];if(!arguments.length)return Jwe(this,n);if(arguments.length===1){i=""+i;const s=this._nodes.get(i);if(typeof s>"u")throw new ut(`Graph.${r}: could not find the "${i}" node in the graph.`);return Zwe(this.multi,n==="mixed"?this.type:n,a,s)}if(arguments.length===2){i=""+i,o=""+o;const s=this._nodes.get(i);if(!s)throw new ut(`Graph.${r}:  could not find the "${i}" source node in the graph.`);if(!this._nodes.has(o))throw new ut(`Graph.${r}:  could not find the "${o}" target node in the graph.`);return eke(n,this.multi,a,s,o)}throw new bt(`Graph.${r}: too many arguments (expecting 0, 1 or 2 and got ${arguments.length}).`)}}function nke(e,t){const{name:r,type:n,direction:a}=t,i="forEach"+r[0].toUpperCase()+r.slice(1,-1);e.prototype[i]=function(u,d,p){if(!(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)){if(arguments.length===1)return p=u,zQ(!1,this,n,p);if(arguments.length===2){u=""+u,p=d;const f=this._nodes.get(u);if(typeof f>"u")throw new ut(`Graph.${i}: could not find the "${u}" node in the graph.`);return f5(!1,this.multi,n==="mixed"?this.type:n,a,f,p)}if(arguments.length===3){u=""+u,d=""+d;const f=this._nodes.get(u);if(!f)throw new ut(`Graph.${i}:  could not find the "${u}" source node in the graph.`);if(!this._nodes.has(d))throw new ut(`Graph.${i}:  could not find the "${d}" target node in the graph.`);return m5(!1,n,this.multi,a,f,d,p)}throw new bt(`Graph.${i}: too many arguments (expecting 1, 2 or 3 and got ${arguments.length}).`)}};const o="map"+r[0].toUpperCase()+r.slice(1);e.prototype[o]=function(){const u=Array.prototype.slice.call(arguments),d=u.pop();let p;if(u.length===0){let f=0;n!=="directed"&&(f+=this.undirectedSize),n!=="undirected"&&(f+=this.directedSize),p=new Array(f);let m=0;u.push((b,y,S,w,E,k,_)=>{p[m++]=d(b,y,S,w,E,k,_)})}else p=[],u.push((f,m,b,y,S,w,E)=>{p.push(d(f,m,b,y,S,w,E))});return this[i].apply(this,u),p};const s="filter"+r[0].toUpperCase()+r.slice(1);e.prototype[s]=function(){const u=Array.prototype.slice.call(arguments),d=u.pop(),p=[];return u.push((f,m,b,y,S,w,E)=>{d(f,m,b,y,S,w,E)&&p.push(f)}),this[i].apply(this,u),p};const l="reduce"+r[0].toUpperCase()+r.slice(1);e.prototype[l]=function(){let u=Array.prototype.slice.call(arguments);if(u.length<2||u.length>4)throw new bt(`Graph.${l}: invalid number of arguments (expecting 2, 3 or 4 and got ${u.length}).`);if(typeof u[u.length-1]=="function"&&typeof u[u.length-2]!="function")throw new bt(`Graph.${l}: missing initial value. You must provide it because the callback takes more than one argument and we cannot infer the initial value from the first iteration, as you could with a simple array.`);let d,p;u.length===2?(d=u[0],p=u[1],u=[]):u.length===3?(d=u[1],p=u[2],u=[u[0]]):u.length===4&&(d=u[2],p=u[3],u=[u[0],u[1]]);let f=p;return u.push((m,b,y,S,w,E,k)=>{f=d(f,m,b,y,S,w,E,k)}),this[i].apply(this,u),f}}function ake(e,t){const{name:r,type:n,direction:a}=t,i="find"+r[0].toUpperCase()+r.slice(1,-1);e.prototype[i]=function(l,u,d){if(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)return!1;if(arguments.length===1)return d=l,zQ(!0,this,n,d);if(arguments.length===2){l=""+l,d=u;const p=this._nodes.get(l);if(typeof p>"u")throw new ut(`Graph.${i}: could not find the "${l}" node in the graph.`);return f5(!0,this.multi,n==="mixed"?this.type:n,a,p,d)}if(arguments.length===3){l=""+l,u=""+u;const p=this._nodes.get(l);if(!p)throw new ut(`Graph.${i}:  could not find the "${l}" source node in the graph.`);if(!this._nodes.has(u))throw new ut(`Graph.${i}:  could not find the "${u}" target node in the graph.`);return m5(!0,n,this.multi,a,p,u,d)}throw new bt(`Graph.${i}: too many arguments (expecting 1, 2 or 3 and got ${arguments.length}).`)};const o="some"+r[0].toUpperCase()+r.slice(1,-1);e.prototype[o]=function(){const l=Array.prototype.slice.call(arguments),u=l.pop();return l.push((p,f,m,b,y,S,w)=>u(p,f,m,b,y,S,w)),!!this[i].apply(this,l)};const s="every"+r[0].toUpperCase()+r.slice(1,-1);e.prototype[s]=function(){const l=Array.prototype.slice.call(arguments),u=l.pop();return l.push((p,f,m,b,y,S,w)=>!u(p,f,m,b,y,S,w)),!this[i].apply(this,l)}}function ike(e,t){const{name:r,type:n,direction:a}=t,i=r.slice(0,-1)+"Entries";e.prototype[i]=function(o,s){if(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)return bp();if(!arguments.length)return Xwe(this,n);if(arguments.length===1){o=""+o;const l=this._nodes.get(o);if(!l)throw new ut(`Graph.${i}: could not find the "${o}" node in the graph.`);return Qwe(n,a,l)}if(arguments.length===2){o=""+o,s=""+s;const l=this._nodes.get(o);if(!l)throw new ut(`Graph.${i}:  could not find the "${o}" source node in the graph.`);if(!this._nodes.has(s))throw new ut(`Graph.${i}:  could not find the "${s}" target node in the graph.`);return tke(n,a,l,s)}throw new bt(`Graph.${i}: too many arguments (expecting 0, 1 or 2 and got ${arguments.length}).`)}}function oke(e){qwe.forEach(t=>{rke(e,t),nke(e,t),ake(e,t),ike(e,t)})}const ske=[{name:"neighbors",type:"mixed"},{name:"inNeighbors",type:"directed",direction:"in"},{name:"outNeighbors",type:"directed",direction:"out"},{name:"inboundNeighbors",type:"mixed",direction:"in"},{name:"outboundNeighbors",type:"mixed",direction:"out"},{name:"directedNeighbors",type:"directed"},{name:"undirectedNeighbors",type:"undirected"}];function l2(){this.A=null,this.B=null}l2.prototype.wrap=function(e){this.A===null?this.A=e:this.B===null&&(this.B=e)};l2.prototype.has=function(e){return this.A!==null&&e in this.A||this.B!==null&&e in this.B};function Rf(e,t,r,n,a){for(const i in n){const o=n[i],s=o.source,l=o.target,u=s===r?l:s;if(t&&t.has(u.key))continue;const d=a(u.key,u.attributes);if(e&&d)return u.key}}function g5(e,t,r,n,a){if(t!=="mixed"){if(t==="undirected")return Rf(e,null,n,n.undirected,a);if(typeof r=="string")return Rf(e,null,n,n[r],a)}const i=new l2;let o;if(t!=="undirected"){if(r!=="out"){if(o=Rf(e,null,n,n.in,a),e&&o)return o;i.wrap(n.in)}if(r!=="in"){if(o=Rf(e,i,n,n.out,a),e&&o)return o;i.wrap(n.out)}}if(t!=="directed"&&(o=Rf(e,i,n,n.undirected,a),e&&o))return o}function lke(e,t,r){if(e!=="mixed"){if(e==="undirected")return Object.keys(r.undirected);if(typeof t=="string")return Object.keys(r[t])}const n=[];return g5(!1,e,t,r,function(a){n.push(a)}),n}function Df(e,t,r){const n=Object.keys(r),a=n.length;let i=0;return{[Symbol.iterator](){return this},next(){let o=null;do{if(i>=a)return e&&e.wrap(r),{done:!0};const s=r[n[i++]],l=s.source,u=s.target;if(o=l===t?u:l,e&&e.has(o.key)){o=null;continue}}while(o===null);return{done:!1,value:{neighbor:o.key,attributes:o.attributes}}}}}function uke(e,t,r){if(e!=="mixed"){if(e==="undirected")return Df(null,r,r.undirected);if(typeof t=="string")return Df(null,r,r[t])}let n=bp();const a=new l2;return e!=="undirected"&&(t!=="out"&&(n=cl(n,Df(a,r,r.in))),t!=="in"&&(n=cl(n,Df(a,r,r.out)))),e!=="directed"&&(n=cl(n,Df(a,r,r.undirected))),n}function cke(e,t){const{name:r,type:n,direction:a}=t;e.prototype[r]=function(i){if(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)return[];i=""+i;const o=this._nodes.get(i);if(typeof o>"u")throw new ut(`Graph.${r}: could not find the "${i}" node in the graph.`);return lke(n==="mixed"?this.type:n,a,o)}}function dke(e,t){const{name:r,type:n,direction:a}=t,i="forEach"+r[0].toUpperCase()+r.slice(1,-1);e.prototype[i]=function(u,d){if(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)return;u=""+u;const p=this._nodes.get(u);if(typeof p>"u")throw new ut(`Graph.${i}: could not find the "${u}" node in the graph.`);g5(!1,n==="mixed"?this.type:n,a,p,d)};const o="map"+r[0].toUpperCase()+r.slice(1);e.prototype[o]=function(u,d){const p=[];return this[i](u,(f,m)=>{p.push(d(f,m))}),p};const s="filter"+r[0].toUpperCase()+r.slice(1);e.prototype[s]=function(u,d){const p=[];return this[i](u,(f,m)=>{d(f,m)&&p.push(f)}),p};const l="reduce"+r[0].toUpperCase()+r.slice(1);e.prototype[l]=function(u,d,p){if(arguments.length<3)throw new bt(`Graph.${l}: missing initial value. You must provide it because the callback takes more than one argument and we cannot infer the initial value from the first iteration, as you could with a simple array.`);let f=p;return this[i](u,(m,b)=>{f=d(f,m,b)}),f}}function hke(e,t){const{name:r,type:n,direction:a}=t,i=r[0].toUpperCase()+r.slice(1,-1),o="find"+i;e.prototype[o]=function(u,d){if(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)return;u=""+u;const p=this._nodes.get(u);if(typeof p>"u")throw new ut(`Graph.${o}: could not find the "${u}" node in the graph.`);return g5(!0,n==="mixed"?this.type:n,a,p,d)};const s="some"+i;e.prototype[s]=function(u,d){return!!this[o](u,d)};const l="every"+i;e.prototype[l]=function(u,d){return!this[o](u,(f,m)=>!d(f,m))}}function pke(e,t){const{name:r,type:n,direction:a}=t,i=r.slice(0,-1)+"Entries";e.prototype[i]=function(o){if(n!=="mixed"&&this.type!=="mixed"&&n!==this.type)return bp();o=""+o;const s=this._nodes.get(o);if(typeof s>"u")throw new ut(`Graph.${i}: could not find the "${o}" node in the graph.`);return uke(n==="mixed"?this.type:n,a,s)}}function fke(e){ske.forEach(t=>{cke(e,t),dke(e,t),hke(e,t),pke(e,t)})}function Ib(e,t,r,n,a){const i=n._nodes.values(),o=n.type;let s,l,u,d,p,f;for(;s=i.next(),s.done!==!0;){let m=!1;if(l=s.value,o!=="undirected"){d=l.out;for(u in d){p=d[u];do f=p.target,m=!0,a(l.key,f.key,l.attributes,f.attributes,p.key,p.attributes,p.undirected),p=p.next;while(p)}}if(o!=="directed"){d=l.undirected;for(u in d)if(!(t&&l.key>u)){p=d[u];do f=p.target,f.key!==u&&(f=p.source),m=!0,a(l.key,f.key,l.attributes,f.attributes,p.key,p.attributes,p.undirected),p=p.next;while(p)}}r&&!m&&a(l.key,null,l.attributes,null,null,null,null)}}function mke(e,t){const r={key:e};return NQ(t.attributes)||(r.attributes=Zn({},t.attributes)),r}function gke(e,t,r){const n={key:t,source:r.source.key,target:r.target.key};return NQ(r.attributes)||(n.attributes=Zn({},r.attributes)),e==="mixed"&&r.undirected&&(n.undirected=!0),n}function bke(e){if(!va(e))throw new bt('Graph.import: invalid serialized node. A serialized node should be a plain object with at least a "key" property.');if(!("key"in e))throw new bt("Graph.import: serialized node is missing its key.");if("attributes"in e&&(!va(e.attributes)||e.attributes===null))throw new bt("Graph.import: invalid attributes. Attributes should be a plain object, null or omitted.")}function yke(e){if(!va(e))throw new bt('Graph.import: invalid serialized edge. A serialized edge should be a plain object with at least a "source" & "target" property.');if(!("source"in e))throw new bt("Graph.import: serialized edge is missing its source.");if(!("target"in e))throw new bt("Graph.import: serialized edge is missing its target.");if("attributes"in e&&(!va(e.attributes)||e.attributes===null))throw new bt("Graph.import: invalid attributes. Attributes should be a plain object, null or omitted.");if("undirected"in e&&typeof e.undirected!="boolean")throw new bt("Graph.import: invalid undirectedness information. Undirected should be boolean or omitted.")}const vke=Ewe(),Ske=new Set(["directed","undirected","mixed"]),u7=new Set(["domain","_events","_eventsCount","_maxListeners"]),Eke=[{name:e=>`${e}Edge`,generateKey:!0},{name:e=>`${e}DirectedEdge`,generateKey:!0,type:"directed"},{name:e=>`${e}UndirectedEdge`,generateKey:!0,type:"undirected"},{name:e=>`${e}EdgeWithKey`},{name:e=>`${e}DirectedEdgeWithKey`,type:"directed"},{name:e=>`${e}UndirectedEdgeWithKey`,type:"undirected"}],xke={allowSelfLoops:!0,multi:!1,type:"mixed"};function wke(e,t,r){if(r&&!va(r))throw new bt(`Graph.addNode: invalid attributes. Expecting an object but got "${r}"`);if(t=""+t,r=r||{},e._nodes.has(t))throw new Xt(`Graph.addNode: the "${t}" node already exist in the graph.`);const n=new e.NodeDataClass(t,r);return e._nodes.set(t,n),e.emit("nodeAdded",{key:t,attributes:r}),n}function c7(e,t,r){const n=new e.NodeDataClass(t,r);return e._nodes.set(t,n),e.emit("nodeAdded",{key:t,attributes:r}),n}function HQ(e,t,r,n,a,i,o,s){if(!n&&e.type==="undirected")throw new Xt(`Graph.${t}: you cannot add a directed edge to an undirected graph. Use the #.addEdge or #.addUndirectedEdge instead.`);if(n&&e.type==="directed")throw new Xt(`Graph.${t}: you cannot add an undirected edge to a directed graph. Use the #.addEdge or #.addDirectedEdge instead.`);if(s&&!va(s))throw new bt(`Graph.${t}: invalid attributes. Expecting an object but got "${s}"`);if(i=""+i,o=""+o,s=s||{},!e.allowSelfLoops&&i===o)throw new Xt(`Graph.${t}: source & target are the same ("${i}"), thus creating a loop explicitly forbidden by this graph 'allowSelfLoops' option set to false.`);const l=e._nodes.get(i),u=e._nodes.get(o);if(!l)throw new ut(`Graph.${t}: source node "${i}" not found.`);if(!u)throw new ut(`Graph.${t}: target node "${o}" not found.`);const d={key:null,undirected:n,source:i,target:o,attributes:s};if(r)a=e._edgeKeyGenerator();else if(a=""+a,e._edges.has(a))throw new Xt(`Graph.${t}: the "${a}" edge already exists in the graph.`);if(!e.multi&&(n?typeof l.undirected[o]<"u":typeof l.out[o]<"u"))throw new Xt(`Graph.${t}: an edge linking "${i}" to "${o}" already exists. If you really want to add multiple edges linking those nodes, you should create a multi graph by using the 'multi' option.`);const p=new yp(n,a,l,u,s);e._edges.set(a,p);const f=i===o;return n?(l.undirectedDegree++,u.undirectedDegree++,f&&(l.undirectedLoops++,e._undirectedSelfLoopCount++)):(l.outDegree++,u.inDegree++,f&&(l.directedLoops++,e._directedSelfLoopCount++)),e.multi?p.attachMulti():p.attach(),n?e._undirectedSize++:e._directedSize++,d.key=a,e.emit("edgeAdded",d),a}function kke(e,t,r,n,a,i,o,s,l){if(!n&&e.type==="undirected")throw new Xt(`Graph.${t}: you cannot merge/update a directed edge to an undirected graph. Use the #.mergeEdge/#.updateEdge or #.addUndirectedEdge instead.`);if(n&&e.type==="directed")throw new Xt(`Graph.${t}: you cannot merge/update an undirected edge to a directed graph. Use the #.mergeEdge/#.updateEdge or #.addDirectedEdge instead.`);if(s){if(l){if(typeof s!="function")throw new bt(`Graph.${t}: invalid updater function. Expecting a function but got "${s}"`)}else if(!va(s))throw new bt(`Graph.${t}: invalid attributes. Expecting an object but got "${s}"`)}i=""+i,o=""+o;let u;if(l&&(u=s,s=void 0),!e.allowSelfLoops&&i===o)throw new Xt(`Graph.${t}: source & target are the same ("${i}"), thus creating a loop explicitly forbidden by this graph 'allowSelfLoops' option set to false.`);let d=e._nodes.get(i),p=e._nodes.get(o),f,m;if(!r&&(f=e._edges.get(a),f)){if((f.source.key!==i||f.target.key!==o)&&(!n||f.source.key!==o||f.target.key!==i))throw new Xt(`Graph.${t}: inconsistency detected when attempting to merge the "${a}" edge with "${i}" source & "${o}" target vs. ("${f.source.key}", "${f.target.key}").`);m=f}if(!m&&!e.multi&&d&&(m=n?d.undirected[o]:d.out[o]),m){const E=[m.key,!1,!1,!1];if(l?!u:!s)return E;if(l){const k=m.attributes;m.attributes=u(k),e.emit("edgeAttributesUpdated",{type:"replace",key:m.key,attributes:m.attributes})}else Zn(m.attributes,s),e.emit("edgeAttributesUpdated",{type:"merge",key:m.key,attributes:m.attributes,data:s});return E}s=s||{},l&&u&&(s=u(s));const b={key:null,undirected:n,source:i,target:o,attributes:s};if(r)a=e._edgeKeyGenerator();else if(a=""+a,e._edges.has(a))throw new Xt(`Graph.${t}: the "${a}" edge already exists in the graph.`);let y=!1,S=!1;d||(d=c7(e,i,{}),y=!0,i===o&&(p=d,S=!0)),p||(p=c7(e,o,{}),S=!0),f=new yp(n,a,d,p,s),e._edges.set(a,f);const w=i===o;return n?(d.undirectedDegree++,p.undirectedDegree++,w&&(d.undirectedLoops++,e._undirectedSelfLoopCount++)):(d.outDegree++,p.inDegree++,w&&(d.directedLoops++,e._directedSelfLoopCount++)),e.multi?f.attachMulti():f.attach(),n?e._undirectedSize++:e._directedSize++,b.key=a,e.emit("edgeAdded",b),[a,!0,y,S]}function th(e,t){e._edges.delete(t.key);const{source:r,target:n,attributes:a}=t,i=t.undirected,o=r===n;i?(r.undirectedDegree--,n.undirectedDegree--,o&&(r.undirectedLoops--,e._undirectedSelfLoopCount--)):(r.outDegree--,n.inDegree--,o&&(r.directedLoops--,e._directedSelfLoopCount--)),e.multi?t.detachMulti():t.detach(),i?e._undirectedSize--:e._directedSize--,e.emit("edgeDropped",{key:t.key,attributes:a,source:r.key,target:n.key,undirected:i})}class Jr extends DQ.EventEmitter{constructor(t){if(super(),t=Zn({},xke,t),typeof t.multi!="boolean")throw new bt(`Graph.constructor: invalid 'multi' option. Expecting a boolean but got "${t.multi}".`);if(!Ske.has(t.type))throw new bt(`Graph.constructor: invalid 'type' option. Should be one of "mixed", "directed" or "undirected" but got "${t.type}".`);if(typeof t.allowSelfLoops!="boolean")throw new bt(`Graph.constructor: invalid 'allowSelfLoops' option. Expecting a boolean but got "${t.allowSelfLoops}".`);const r=t.type==="mixed"?LQ:t.type==="directed"?MQ:IQ;Qi(this,"NodeDataClass",r);const n="geid_"+vke()+"_";let a=0;const i=()=>{let o;do o=n+a++;while(this._edges.has(o));return o};Qi(this,"_attributes",{}),Qi(this,"_nodes",new Map),Qi(this,"_edges",new Map),Qi(this,"_directedSize",0),Qi(this,"_undirectedSize",0),Qi(this,"_directedSelfLoopCount",0),Qi(this,"_undirectedSelfLoopCount",0),Qi(this,"_edgeKeyGenerator",i),Qi(this,"_options",t),u7.forEach(o=>Qi(this,o,this[o])),So(this,"order",()=>this._nodes.size),So(this,"size",()=>this._edges.size),So(this,"directedSize",()=>this._directedSize),So(this,"undirectedSize",()=>this._undirectedSize),So(this,"selfLoopCount",()=>this._directedSelfLoopCount+this._undirectedSelfLoopCount),So(this,"directedSelfLoopCount",()=>this._directedSelfLoopCount),So(this,"undirectedSelfLoopCount",()=>this._undirectedSelfLoopCount),So(this,"multi",this._options.multi),So(this,"type",this._options.type),So(this,"allowSelfLoops",this._options.allowSelfLoops),So(this,"implementation",()=>"graphology")}_resetInstanceCounters(){this._directedSize=0,this._undirectedSize=0,this._directedSelfLoopCount=0,this._undirectedSelfLoopCount=0}hasNode(t){return this._nodes.has(""+t)}hasDirectedEdge(t,r){if(this.type==="undirected")return!1;if(arguments.length===1){const n=""+t,a=this._edges.get(n);return!!a&&!a.undirected}else if(arguments.length===2){t=""+t,r=""+r;const n=this._nodes.get(t);return n?n.out.hasOwnProperty(r):!1}throw new bt(`Graph.hasDirectedEdge: invalid arity (${arguments.length}, instead of 1 or 2). You can either ask for an edge id or for the existence of an edge between a source & a target.`)}hasUndirectedEdge(t,r){if(this.type==="directed")return!1;if(arguments.length===1){const n=""+t,a=this._edges.get(n);return!!a&&a.undirected}else if(arguments.length===2){t=""+t,r=""+r;const n=this._nodes.get(t);return n?n.undirected.hasOwnProperty(r):!1}throw new bt(`Graph.hasDirectedEdge: invalid arity (${arguments.length}, instead of 1 or 2). You can either ask for an edge id or for the existence of an edge between a source & a target.`)}hasEdge(t,r){if(arguments.length===1){const n=""+t;return this._edges.has(n)}else if(arguments.length===2){t=""+t,r=""+r;const n=this._nodes.get(t);return n?typeof n.out<"u"&&n.out.hasOwnProperty(r)||typeof n.undirected<"u"&&n.undirected.hasOwnProperty(r):!1}throw new bt(`Graph.hasEdge: invalid arity (${arguments.length}, instead of 1 or 2). You can either ask for an edge id or for the existence of an edge between a source & a target.`)}directedEdge(t,r){if(this.type==="undirected")return;if(t=""+t,r=""+r,this.multi)throw new Xt("Graph.directedEdge: this method is irrelevant with multigraphs since there might be multiple edges between source & target. See #.directedEdges instead.");const n=this._nodes.get(t);if(!n)throw new ut(`Graph.directedEdge: could not find the "${t}" source node in the graph.`);if(!this._nodes.has(r))throw new ut(`Graph.directedEdge: could not find the "${r}" target node in the graph.`);const a=n.out&&n.out[r]||void 0;if(a)return a.key}undirectedEdge(t,r){if(this.type==="directed")return;if(t=""+t,r=""+r,this.multi)throw new Xt("Graph.undirectedEdge: this method is irrelevant with multigraphs since there might be multiple edges between source & target. See #.undirectedEdges instead.");const n=this._nodes.get(t);if(!n)throw new ut(`Graph.undirectedEdge: could not find the "${t}" source node in the graph.`);if(!this._nodes.has(r))throw new ut(`Graph.undirectedEdge: could not find the "${r}" target node in the graph.`);const a=n.undirected&&n.undirected[r]||void 0;if(a)return a.key}edge(t,r){if(this.multi)throw new Xt("Graph.edge: this method is irrelevant with multigraphs since there might be multiple edges between source & target. See #.edges instead.");t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.edge: could not find the "${t}" source node in the graph.`);if(!this._nodes.has(r))throw new ut(`Graph.edge: could not find the "${r}" target node in the graph.`);const a=n.out&&n.out[r]||n.undirected&&n.undirected[r]||void 0;if(a)return a.key}areDirectedNeighbors(t,r){t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.areDirectedNeighbors: could not find the "${t}" node in the graph.`);return this.type==="undirected"?!1:r in n.in||r in n.out}areOutNeighbors(t,r){t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.areOutNeighbors: could not find the "${t}" node in the graph.`);return this.type==="undirected"?!1:r in n.out}areInNeighbors(t,r){t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.areInNeighbors: could not find the "${t}" node in the graph.`);return this.type==="undirected"?!1:r in n.in}areUndirectedNeighbors(t,r){t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.areUndirectedNeighbors: could not find the "${t}" node in the graph.`);return this.type==="directed"?!1:r in n.undirected}areNeighbors(t,r){t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.areNeighbors: could not find the "${t}" node in the graph.`);return this.type!=="undirected"&&(r in n.in||r in n.out)||this.type!=="directed"&&r in n.undirected}areInboundNeighbors(t,r){t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.areInboundNeighbors: could not find the "${t}" node in the graph.`);return this.type!=="undirected"&&r in n.in||this.type!=="directed"&&r in n.undirected}areOutboundNeighbors(t,r){t=""+t,r=""+r;const n=this._nodes.get(t);if(!n)throw new ut(`Graph.areOutboundNeighbors: could not find the "${t}" node in the graph.`);return this.type!=="undirected"&&r in n.out||this.type!=="directed"&&r in n.undirected}inDegree(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.inDegree: could not find the "${t}" node in the graph.`);return this.type==="undirected"?0:r.inDegree}outDegree(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.outDegree: could not find the "${t}" node in the graph.`);return this.type==="undirected"?0:r.outDegree}directedDegree(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.directedDegree: could not find the "${t}" node in the graph.`);return this.type==="undirected"?0:r.inDegree+r.outDegree}undirectedDegree(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.undirectedDegree: could not find the "${t}" node in the graph.`);return this.type==="directed"?0:r.undirectedDegree}inboundDegree(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.inboundDegree: could not find the "${t}" node in the graph.`);let n=0;return this.type!=="directed"&&(n+=r.undirectedDegree),this.type!=="undirected"&&(n+=r.inDegree),n}outboundDegree(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.outboundDegree: could not find the "${t}" node in the graph.`);let n=0;return this.type!=="directed"&&(n+=r.undirectedDegree),this.type!=="undirected"&&(n+=r.outDegree),n}degree(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.degree: could not find the "${t}" node in the graph.`);let n=0;return this.type!=="directed"&&(n+=r.undirectedDegree),this.type!=="undirected"&&(n+=r.inDegree+r.outDegree),n}inDegreeWithoutSelfLoops(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.inDegreeWithoutSelfLoops: could not find the "${t}" node in the graph.`);return this.type==="undirected"?0:r.inDegree-r.directedLoops}outDegreeWithoutSelfLoops(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.outDegreeWithoutSelfLoops: could not find the "${t}" node in the graph.`);return this.type==="undirected"?0:r.outDegree-r.directedLoops}directedDegreeWithoutSelfLoops(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.directedDegreeWithoutSelfLoops: could not find the "${t}" node in the graph.`);return this.type==="undirected"?0:r.inDegree+r.outDegree-r.directedLoops*2}undirectedDegreeWithoutSelfLoops(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.undirectedDegreeWithoutSelfLoops: could not find the "${t}" node in the graph.`);return this.type==="directed"?0:r.undirectedDegree-r.undirectedLoops*2}inboundDegreeWithoutSelfLoops(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.inboundDegreeWithoutSelfLoops: could not find the "${t}" node in the graph.`);let n=0,a=0;return this.type!=="directed"&&(n+=r.undirectedDegree,a+=r.undirectedLoops*2),this.type!=="undirected"&&(n+=r.inDegree,a+=r.directedLoops),n-a}outboundDegreeWithoutSelfLoops(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.outboundDegreeWithoutSelfLoops: could not find the "${t}" node in the graph.`);let n=0,a=0;return this.type!=="directed"&&(n+=r.undirectedDegree,a+=r.undirectedLoops*2),this.type!=="undirected"&&(n+=r.outDegree,a+=r.directedLoops),n-a}degreeWithoutSelfLoops(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.degreeWithoutSelfLoops: could not find the "${t}" node in the graph.`);let n=0,a=0;return this.type!=="directed"&&(n+=r.undirectedDegree,a+=r.undirectedLoops*2),this.type!=="undirected"&&(n+=r.inDegree+r.outDegree,a+=r.directedLoops*2),n-a}source(t){t=""+t;const r=this._edges.get(t);if(!r)throw new ut(`Graph.source: could not find the "${t}" edge in the graph.`);return r.source.key}target(t){t=""+t;const r=this._edges.get(t);if(!r)throw new ut(`Graph.target: could not find the "${t}" edge in the graph.`);return r.target.key}extremities(t){t=""+t;const r=this._edges.get(t);if(!r)throw new ut(`Graph.extremities: could not find the "${t}" edge in the graph.`);return[r.source.key,r.target.key]}opposite(t,r){t=""+t,r=""+r;const n=this._edges.get(r);if(!n)throw new ut(`Graph.opposite: could not find the "${r}" edge in the graph.`);const a=n.source.key,i=n.target.key;if(t===a)return i;if(t===i)return a;throw new ut(`Graph.opposite: the "${t}" node is not attached to the "${r}" edge (${a}, ${i}).`)}hasExtremity(t,r){t=""+t,r=""+r;const n=this._edges.get(t);if(!n)throw new ut(`Graph.hasExtremity: could not find the "${t}" edge in the graph.`);return n.source.key===r||n.target.key===r}isUndirected(t){t=""+t;const r=this._edges.get(t);if(!r)throw new ut(`Graph.isUndirected: could not find the "${t}" edge in the graph.`);return r.undirected}isDirected(t){t=""+t;const r=this._edges.get(t);if(!r)throw new ut(`Graph.isDirected: could not find the "${t}" edge in the graph.`);return!r.undirected}isSelfLoop(t){t=""+t;const r=this._edges.get(t);if(!r)throw new ut(`Graph.isSelfLoop: could not find the "${t}" edge in the graph.`);return r.source===r.target}addNode(t,r){return wke(this,t,r).key}mergeNode(t,r){if(r&&!va(r))throw new bt(`Graph.mergeNode: invalid attributes. Expecting an object but got "${r}"`);t=""+t,r=r||{};let n=this._nodes.get(t);return n?(r&&(Zn(n.attributes,r),this.emit("nodeAttributesUpdated",{type:"merge",key:t,attributes:n.attributes,data:r})),[t,!1]):(n=new this.NodeDataClass(t,r),this._nodes.set(t,n),this.emit("nodeAdded",{key:t,attributes:r}),[t,!0])}updateNode(t,r){if(r&&typeof r!="function")throw new bt(`Graph.updateNode: invalid updater function. Expecting a function but got "${r}"`);t=""+t;let n=this._nodes.get(t);if(n){if(r){const i=n.attributes;n.attributes=r(i),this.emit("nodeAttributesUpdated",{type:"replace",key:t,attributes:n.attributes})}return[t,!1]}const a=r?r({}):{};return n=new this.NodeDataClass(t,a),this._nodes.set(t,n),this.emit("nodeAdded",{key:t,attributes:a}),[t,!0]}dropNode(t){t=""+t;const r=this._nodes.get(t);if(!r)throw new ut(`Graph.dropNode: could not find the "${t}" node in the graph.`);let n;if(this.type!=="undirected"){for(const a in r.out){n=r.out[a];do th(this,n),n=n.next;while(n)}for(const a in r.in){n=r.in[a];do th(this,n),n=n.next;while(n)}}if(this.type!=="directed")for(const a in r.undirected){n=r.undirected[a];do th(this,n),n=n.next;while(n)}this._nodes.delete(t),this.emit("nodeDropped",{key:t,attributes:r.attributes})}dropEdge(t){let r;if(arguments.length>1){const n=""+arguments[0],a=""+arguments[1];if(r=io(this,n,a,this.type),!r)throw new ut(`Graph.dropEdge: could not find the "${n}" -> "${a}" edge in the graph.`)}else if(t=""+t,r=this._edges.get(t),!r)throw new ut(`Graph.dropEdge: could not find the "${t}" edge in the graph.`);return th(this,r),this}dropDirectedEdge(t,r){if(arguments.length<2)throw new Xt("Graph.dropDirectedEdge: it does not make sense to try and drop a directed edge by key. What if the edge with this key is undirected? Use #.dropEdge for this purpose instead.");if(this.multi)throw new Xt("Graph.dropDirectedEdge: cannot use a {source,target} combo when dropping an edge in a MultiGraph since we cannot infer the one you want to delete as there could be multiple ones.");t=""+t,r=""+r;const n=io(this,t,r,"directed");if(!n)throw new ut(`Graph.dropDirectedEdge: could not find a "${t}" -> "${r}" edge in the graph.`);return th(this,n),this}dropUndirectedEdge(t,r){if(arguments.length<2)throw new Xt("Graph.dropUndirectedEdge: it does not make sense to drop a directed edge by key. What if the edge with this key is undirected? Use #.dropEdge for this purpose instead.");if(this.multi)throw new Xt("Graph.dropUndirectedEdge: cannot use a {source,target} combo when dropping an edge in a MultiGraph since we cannot infer the one you want to delete as there could be multiple ones.");const n=io(this,t,r,"undirected");if(!n)throw new ut(`Graph.dropUndirectedEdge: could not find a "${t}" -> "${r}" edge in the graph.`);return th(this,n),this}clear(){this._edges.clear(),this._nodes.clear(),this._resetInstanceCounters(),this.emit("cleared")}clearEdges(){const t=this._nodes.values();let r;for(;r=t.next(),r.done!==!0;)r.value.clear();this._edges.clear(),this._resetInstanceCounters(),this.emit("edgesCleared")}getAttribute(t){return this._attributes[t]}getAttributes(){return this._attributes}hasAttribute(t){return this._attributes.hasOwnProperty(t)}setAttribute(t,r){return this._attributes[t]=r,this.emit("attributesUpdated",{type:"set",attributes:this._attributes,name:t}),this}updateAttribute(t,r){if(typeof r!="function")throw new bt("Graph.updateAttribute: updater should be a function.");const n=this._attributes[t];return this._attributes[t]=r(n),this.emit("attributesUpdated",{type:"set",attributes:this._attributes,name:t}),this}removeAttribute(t){return delete this._attributes[t],this.emit("attributesUpdated",{type:"remove",attributes:this._attributes,name:t}),this}replaceAttributes(t){if(!va(t))throw new bt("Graph.replaceAttributes: provided attributes are not a plain object.");return this._attributes=t,this.emit("attributesUpdated",{type:"replace",attributes:this._attributes}),this}mergeAttributes(t){if(!va(t))throw new bt("Graph.mergeAttributes: provided attributes are not a plain object.");return Zn(this._attributes,t),this.emit("attributesUpdated",{type:"merge",attributes:this._attributes,data:t}),this}updateAttributes(t){if(typeof t!="function")throw new bt("Graph.updateAttributes: provided updater is not a function.");return this._attributes=t(this._attributes),this.emit("attributesUpdated",{type:"update",attributes:this._attributes}),this}updateEachNodeAttributes(t,r){if(typeof t!="function")throw new bt("Graph.updateEachNodeAttributes: expecting an updater function.");if(r&&!l7(r))throw new bt("Graph.updateEachNodeAttributes: invalid hints. Expecting an object having the following shape: {attributes?: [string]}");const n=this._nodes.values();let a,i;for(;a=n.next(),a.done!==!0;)i=a.value,i.attributes=t(i.key,i.attributes);this.emit("eachNodeAttributesUpdated",{hints:r||null})}updateEachEdgeAttributes(t,r){if(typeof t!="function")throw new bt("Graph.updateEachEdgeAttributes: expecting an updater function.");if(r&&!l7(r))throw new bt("Graph.updateEachEdgeAttributes: invalid hints. Expecting an object having the following shape: {attributes?: [string]}");const n=this._edges.values();let a,i,o,s;for(;a=n.next(),a.done!==!0;)i=a.value,o=i.source,s=i.target,i.attributes=t(i.key,i.attributes,o.key,s.key,o.attributes,s.attributes,i.undirected);this.emit("eachEdgeAttributesUpdated",{hints:r||null})}forEachAdjacencyEntry(t){if(typeof t!="function")throw new bt("Graph.forEachAdjacencyEntry: expecting a callback.");Ib(!1,!1,!1,this,t)}forEachAdjacencyEntryWithOrphans(t){if(typeof t!="function")throw new bt("Graph.forEachAdjacencyEntryWithOrphans: expecting a callback.");Ib(!1,!1,!0,this,t)}forEachAssymetricAdjacencyEntry(t){if(typeof t!="function")throw new bt("Graph.forEachAssymetricAdjacencyEntry: expecting a callback.");Ib(!1,!0,!1,this,t)}forEachAssymetricAdjacencyEntryWithOrphans(t){if(typeof t!="function")throw new bt("Graph.forEachAssymetricAdjacencyEntryWithOrphans: expecting a callback.");Ib(!1,!0,!0,this,t)}nodes(){return Array.from(this._nodes.keys())}forEachNode(t){if(typeof t!="function")throw new bt("Graph.forEachNode: expecting a callback.");const r=this._nodes.values();let n,a;for(;n=r.next(),n.done!==!0;)a=n.value,t(a.key,a.attributes)}findNode(t){if(typeof t!="function")throw new bt("Graph.findNode: expecting a callback.");const r=this._nodes.values();let n,a;for(;n=r.next(),n.done!==!0;)if(a=n.value,t(a.key,a.attributes))return a.key}mapNodes(t){if(typeof t!="function")throw new bt("Graph.mapNode: expecting a callback.");const r=this._nodes.values();let n,a;const i=new Array(this.order);let o=0;for(;n=r.next(),n.done!==!0;)a=n.value,i[o++]=t(a.key,a.attributes);return i}someNode(t){if(typeof t!="function")throw new bt("Graph.someNode: expecting a callback.");const r=this._nodes.values();let n,a;for(;n=r.next(),n.done!==!0;)if(a=n.value,t(a.key,a.attributes))return!0;return!1}everyNode(t){if(typeof t!="function")throw new bt("Graph.everyNode: expecting a callback.");const r=this._nodes.values();let n,a;for(;n=r.next(),n.done!==!0;)if(a=n.value,!t(a.key,a.attributes))return!1;return!0}filterNodes(t){if(typeof t!="function")throw new bt("Graph.filterNodes: expecting a callback.");const r=this._nodes.values();let n,a;const i=[];for(;n=r.next(),n.done!==!0;)a=n.value,t(a.key,a.attributes)&&i.push(a.key);return i}reduceNodes(t,r){if(typeof t!="function")throw new bt("Graph.reduceNodes: expecting a callback.");if(arguments.length<2)throw new bt("Graph.reduceNodes: missing initial value. You must provide it because the callback takes more than one argument and we cannot infer the initial value from the first iteration, as you could with a simple array.");let n=r;const a=this._nodes.values();let i,o;for(;i=a.next(),i.done!==!0;)o=i.value,n=t(n,o.key,o.attributes);return n}nodeEntries(){const t=this._nodes.values();return{[Symbol.iterator](){return this},next(){const r=t.next();if(r.done)return r;const n=r.value;return{value:{node:n.key,attributes:n.attributes},done:!1}}}}export(){const t=new Array(this._nodes.size);let r=0;this._nodes.forEach((a,i)=>{t[r++]=mke(i,a)});const n=new Array(this._edges.size);return r=0,this._edges.forEach((a,i)=>{n[r++]=gke(this.type,i,a)}),{options:{type:this.type,multi:this.multi,allowSelfLoops:this.allowSelfLoops},attributes:this.getAttributes(),nodes:t,edges:n}}import(t,r=!1){if(t instanceof Jr)return t.forEachNode((l,u)=>{r?this.mergeNode(l,u):this.addNode(l,u)}),t.forEachEdge((l,u,d,p,f,m,b)=>{r?b?this.mergeUndirectedEdgeWithKey(l,d,p,u):this.mergeDirectedEdgeWithKey(l,d,p,u):b?this.addUndirectedEdgeWithKey(l,d,p,u):this.addDirectedEdgeWithKey(l,d,p,u)}),this;if(!va(t))throw new bt("Graph.import: invalid argument. Expecting a serialized graph or, alternatively, a Graph instance.");if(t.attributes){if(!va(t.attributes))throw new bt("Graph.import: invalid attributes. Expecting a plain object.");r?this.mergeAttributes(t.attributes):this.replaceAttributes(t.attributes)}let n,a,i,o,s;if(t.nodes){if(i=t.nodes,!Array.isArray(i))throw new bt("Graph.import: invalid nodes. Expecting an array.");for(n=0,a=i.length;n<a;n++){o=i[n],bke(o);const{key:l,attributes:u}=o;r?this.mergeNode(l,u):this.addNode(l,u)}}if(t.edges){let l=!1;if(this.type==="undirected"&&(l=!0),i=t.edges,!Array.isArray(i))throw new bt("Graph.import: invalid edges. Expecting an array.");for(n=0,a=i.length;n<a;n++){s=i[n],yke(s);const{source:u,target:d,attributes:p,undirected:f=l}=s;let m;"key"in s?(m=r?f?this.mergeUndirectedEdgeWithKey:this.mergeDirectedEdgeWithKey:f?this.addUndirectedEdgeWithKey:this.addDirectedEdgeWithKey,m.call(this,s.key,u,d,p)):(m=r?f?this.mergeUndirectedEdge:this.mergeDirectedEdge:f?this.addUndirectedEdge:this.addDirectedEdge,m.call(this,u,d,p))}}return this}nullCopy(t){const r=new Jr(Zn({},this._options,t));return r.replaceAttributes(Zn({},this.getAttributes())),r}emptyCopy(t){const r=this.nullCopy(t);return this._nodes.forEach((n,a)=>{const i=Zn({},n.attributes);n=new r.NodeDataClass(a,i),r._nodes.set(a,n)}),r}copy(t){if(t=t||{},typeof t.type=="string"&&t.type!==this.type&&t.type!=="mixed")throw new Xt(`Graph.copy: cannot create an incompatible copy from "${this.type}" type to "${t.type}" because this would mean losing information about the current graph.`);if(typeof t.multi=="boolean"&&t.multi!==this.multi&&t.multi!==!0)throw new Xt("Graph.copy: cannot create an incompatible copy by downgrading a multi graph to a simple one because this would mean losing information about the current graph.");if(typeof t.allowSelfLoops=="boolean"&&t.allowSelfLoops!==this.allowSelfLoops&&t.allowSelfLoops!==!0)throw new Xt("Graph.copy: cannot create an incompatible copy from a graph allowing self loops to one that does not because this would mean losing information about the current graph.");const r=this.emptyCopy(t),n=this._edges.values();let a,i;for(;a=n.next(),a.done!==!0;)i=a.value,HQ(r,"copy",!1,i.undirected,i.key,i.source.key,i.target.key,Zn({},i.attributes));return r}toJSON(){return this.export()}toString(){return"[object Graph]"}inspect(){const t={};this._nodes.forEach((i,o)=>{t[o]=i.attributes});const r={},n={};this._edges.forEach((i,o)=>{const s=i.undirected?"--":"->";let l="",u=i.source.key,d=i.target.key,p;i.undirected&&u>d&&(p=u,u=d,d=p);const f=`(${u})${s}(${d})`;o.startsWith("geid_")?this.multi&&(typeof n[f]>"u"?n[f]=0:n[f]++,l+=`${n[f]}. `):l+=`[${o}]: `,l+=f,r[l]=i.attributes});const a={};for(const i in this)this.hasOwnProperty(i)&&!u7.has(i)&&typeof this[i]!="function"&&typeof i!="symbol"&&(a[i]=this[i]);return a.attributes=this._attributes,a.nodes=t,a.edges=r,Qi(a,"constructor",this.constructor),a}}typeof Symbol<"u"&&(Jr.prototype[Symbol.for("nodejs.util.inspect.custom")]=Jr.prototype.inspect);Eke.forEach(e=>{["add","merge","update"].forEach(t=>{const r=e.name(t),n=t==="add"?HQ:kke;e.generateKey?Jr.prototype[r]=function(a,i,o){return n(this,r,!0,(e.type||this.type)==="undirected",null,a,i,o,t==="update")}:Jr.prototype[r]=function(a,i,o,s){return n(this,r,!1,(e.type||this.type)==="undirected",a,i,o,s,t==="update")}})});Lwe(Jr);jwe(Jr);oke(Jr);fke(Jr);class UQ extends Jr{constructor(t){const r=Zn({type:"directed"},t);if("multi"in r&&r.multi!==!1)throw new bt("DirectedGraph.from: inconsistent indication that the graph should be multi in given options!");if(r.type!=="directed")throw new bt('DirectedGraph.from: inconsistent "'+r.type+'" type in given options!');super(r)}}class Gh extends Jr{constructor(t){const r=Zn({type:"undirected"},t);if("multi"in r&&r.multi!==!1)throw new bt("UndirectedGraph.from: inconsistent indication that the graph should be multi in given options!");if(r.type!=="undirected")throw new bt('UndirectedGraph.from: inconsistent "'+r.type+'" type in given options!');super(r)}}class $Q extends Jr{constructor(t){const r=Zn({multi:!0},t);if("multi"in r&&r.multi!==!0)throw new bt("MultiGraph.from: inconsistent indication that the graph should be simple in given options!");super(r)}}class GQ extends Jr{constructor(t){const r=Zn({type:"directed",multi:!0},t);if("multi"in r&&r.multi!==!0)throw new bt("MultiDirectedGraph.from: inconsistent indication that the graph should be simple in given options!");if(r.type!=="directed")throw new bt('MultiDirectedGraph.from: inconsistent "'+r.type+'" type in given options!');super(r)}}class jQ extends Jr{constructor(t){const r=Zn({type:"undirected",multi:!0},t);if("multi"in r&&r.multi!==!0)throw new bt("MultiUndirectedGraph.from: inconsistent indication that the graph should be simple in given options!");if(r.type!=="undirected")throw new bt('MultiUndirectedGraph.from: inconsistent "'+r.type+'" type in given options!');super(r)}}function vp(e){e.from=function(t,r){const n=Zn({},t.options,r),a=new e(n);return a.import(t),a}}vp(Jr);vp(UQ);vp(Gh);vp($Q);vp(GQ);vp(jQ);Jr.Graph=Jr;Jr.DirectedGraph=UQ;Jr.UndirectedGraph=Gh;Jr.MultiGraph=$Q;Jr.MultiDirectedGraph=GQ;Jr.MultiUndirectedGraph=jQ;Jr.InvalidArgumentsGraphError=bt;Jr.NotFoundGraphError=ut;Jr.UsageGraphError=Xt;function Cke(e,t){if(typeof e!="object"||!e)return e;var r=e[Symbol.toPrimitive];if(r!==void 0){var n=r.call(e,t);if(typeof n!="object")return n;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}function ym(e){var t=Cke(e,"string");return typeof t=="symbol"?t:t+""}function la(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function d7(e,t){for(var r=0;r<t.length;r++){var n=t[r];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,ym(n.key),n)}}function ua(e,t,r){return t&&d7(e.prototype,t),r&&d7(e,r),Object.defineProperty(e,"prototype",{writable:!1}),e}function jh(e){return jh=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(t){return t.__proto__||Object.getPrototypeOf(t)},jh(e)}function qQ(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(qQ=function(){return!!e})()}function Tke(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function Ake(e,t){if(t&&(typeof t=="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return Tke(e)}function gi(e,t,r){return t=jh(t),Ake(e,qQ()?Reflect.construct(t,r||[],jh(e).constructor):t.apply(e,r))}function dR(e,t){return dR=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,n){return r.__proto__=n,r},dR(e,t)}function bi(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&dR(e,t)}function Fke(e){if(Array.isArray(e))return e}function _ke(e,t){var r=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(r!=null){var n,a,i,o,s=[],l=!0,u=!1;try{if(i=(r=r.call(e)).next,t===0){if(Object(r)!==r)return;l=!1}else for(;!(l=(n=i.call(r)).done)&&(s.push(n.value),s.length!==t);l=!0);}catch(d){u=!0,a=d}finally{try{if(!l&&r.return!=null&&(o=r.return(),Object(o)!==o))return}finally{if(u)throw a}}return s}}function hR(e,t){(t==null||t>e.length)&&(t=e.length);for(var r=0,n=Array(t);r<t;r++)n[r]=e[r];return n}function WQ(e,t){if(e){if(typeof e=="string")return hR(e,t);var r={}.toString.call(e).slice(8,-1);return r==="Object"&&e.constructor&&(r=e.constructor.name),r==="Map"||r==="Set"?Array.from(e):r==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r)?hR(e,t):void 0}}function Rke(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function qh(e,t){return Fke(e)||_ke(e,t)||WQ(e,t)||Rke()}var aw={black:"#000000",silver:"#C0C0C0",gray:"#808080",grey:"#808080",white:"#FFFFFF",maroon:"#800000",red:"#FF0000",purple:"#800080",fuchsia:"#FF00FF",green:"#008000",lime:"#00FF00",olive:"#808000",yellow:"#FFFF00",navy:"#000080",blue:"#0000FF",teal:"#008080",aqua:"#00FFFF",darkblue:"#00008B",mediumblue:"#0000CD",darkgreen:"#006400",darkcyan:"#008B8B",deepskyblue:"#00BFFF",darkturquoise:"#00CED1",mediumspringgreen:"#00FA9A",springgreen:"#00FF7F",cyan:"#00FFFF",midnightblue:"#191970",dodgerblue:"#1E90FF",lightseagreen:"#20B2AA",forestgreen:"#228B22",seagreen:"#2E8B57",darkslategray:"#2F4F4F",darkslategrey:"#2F4F4F",limegreen:"#32CD32",mediumseagreen:"#3CB371",turquoise:"#40E0D0",royalblue:"#4169E1",steelblue:"#4682B4",darkslateblue:"#483D8B",mediumturquoise:"#48D1CC",indigo:"#4B0082",darkolivegreen:"#556B2F",cadetblue:"#5F9EA0",cornflowerblue:"#6495ED",rebeccapurple:"#663399",mediumaquamarine:"#66CDAA",dimgray:"#696969",dimgrey:"#696969",slateblue:"#6A5ACD",olivedrab:"#6B8E23",slategray:"#708090",slategrey:"#708090",lightslategray:"#778899",lightslategrey:"#778899",mediumslateblue:"#7B68EE",lawngreen:"#7CFC00",chartreuse:"#7FFF00",aquamarine:"#7FFFD4",skyblue:"#87CEEB",lightskyblue:"#87CEFA",blueviolet:"#8A2BE2",darkred:"#8B0000",darkmagenta:"#8B008B",saddlebrown:"#8B4513",darkseagreen:"#8FBC8F",lightgreen:"#90EE90",mediumpurple:"#9370DB",darkviolet:"#9400D3",palegreen:"#98FB98",darkorchid:"#9932CC",yellowgreen:"#9ACD32",sienna:"#A0522D",brown:"#A52A2A",darkgray:"#A9A9A9",darkgrey:"#A9A9A9",lightblue:"#ADD8E6",greenyellow:"#ADFF2F",paleturquoise:"#AFEEEE",lightsteelblue:"#B0C4DE",powderblue:"#B0E0E6",firebrick:"#B22222",darkgoldenrod:"#B8860B",mediumorchid:"#BA55D3",rosybrown:"#BC8F8F",darkkhaki:"#BDB76B",mediumvioletred:"#C71585",indianred:"#CD5C5C",peru:"#CD853F",chocolate:"#D2691E",tan:"#D2B48C",lightgray:"#D3D3D3",lightgrey:"#D3D3D3",thistle:"#D8BFD8",orchid:"#DA70D6",goldenrod:"#DAA520",palevioletred:"#DB7093",crimson:"#DC143C",gainsboro:"#DCDCDC",plum:"#DDA0DD",burlywood:"#DEB887",lightcyan:"#E0FFFF",lavender:"#E6E6FA",darksalmon:"#E9967A",violet:"#EE82EE",palegoldenrod:"#EEE8AA",lightcoral:"#F08080",khaki:"#F0E68C",aliceblue:"#F0F8FF",honeydew:"#F0FFF0",azure:"#F0FFFF",sandybrown:"#F4A460",wheat:"#F5DEB3",beige:"#F5F5DC",whitesmoke:"#F5F5F5",mintcream:"#F5FFFA",ghostwhite:"#F8F8FF",salmon:"#FA8072",antiquewhite:"#FAEBD7",linen:"#FAF0E6",lightgoldenrodyellow:"#FAFAD2",oldlace:"#FDF5E6",magenta:"#FF00FF",deeppink:"#FF1493",orangered:"#FF4500",tomato:"#FF6347",hotpink:"#FF69B4",coral:"#FF7F50",darkorange:"#FF8C00",lightsalmon:"#FFA07A",orange:"#FFA500",lightpink:"#FFB6C1",pink:"#FFC0CB",gold:"#FFD700",peachpuff:"#FFDAB9",navajowhite:"#FFDEAD",moccasin:"#FFE4B5",bisque:"#FFE4C4",mistyrose:"#FFE4E1",blanchedalmond:"#FFEBCD",papayawhip:"#FFEFD5",lavenderblush:"#FFF0F5",seashell:"#FFF5EE",cornsilk:"#FFF8DC",lemonchiffon:"#FFFACD",floralwhite:"#FFFAF0",snow:"#FFFAFA",lightyellow:"#FFFFE0",ivory:"#FFFFF0"},VQ=new Int8Array(4),T1=new Int32Array(VQ.buffer,0,1),KQ=new Float32Array(VQ.buffer,0,1),Dke=/^\s*rgba?\s*\(/,Nke=/^\s*rgba?\s*\(\s*([0-9]*)\s*,\s*([0-9]*)\s*,\s*([0-9]*)(?:\s*,\s*(.*)?)?\)\s*$/;function Lke(e){var t=0,r=0,n=0,a=1;if(e[0]==="#")e.length===4?(t=parseInt(e.charAt(1)+e.charAt(1),16),r=parseInt(e.charAt(2)+e.charAt(2),16),n=parseInt(e.charAt(3)+e.charAt(3),16)):(t=parseInt(e.charAt(1)+e.charAt(2),16),r=parseInt(e.charAt(3)+e.charAt(4),16),n=parseInt(e.charAt(5)+e.charAt(6),16)),e.length===9&&(a=parseInt(e.charAt(7)+e.charAt(8),16)/255);else if(Dke.test(e)){var i=e.match(Nke);i&&(t=+i[1],r=+i[2],n=+i[3],i[4]&&(a=+i[4]))}return{r:t,g:r,b:n,a}}var xh={};for(var Ob in aw)xh[Ob]=bs(aw[Ob]),xh[aw[Ob]]=xh[Ob];function YQ(e,t,r,n,a){return T1[0]=n<<24|r<<16|t<<8|e,T1[0]=T1[0]&4278190079,KQ[0]}function bs(e){if(e=e.toLowerCase(),typeof xh[e]<"u")return xh[e];var t=Lke(e),r=t.r,n=t.g,a=t.b,i=t.a;i=i*255|0;var o=YQ(r,n,a,i);return xh[e]=o,o}function Mke(e,t){KQ[0]=bs(e);var r=T1[0],n=r&255,a=r>>8&255,i=r>>16&255,o=r>>24&255;return[n,a,i,o]}var iw={};function JQ(e){if(typeof iw[e]<"u")return iw[e];var t=(e&16711680)>>>16,r=(e&65280)>>>8,n=e&255,a=255,i=YQ(t,r,n,a);return iw[e]=i,i}function h7(e,t,r,n){return r+(t<<8)+(e<<16)}function p7(e,t,r,n,a,i){var o=Math.floor(r/i*a),s=Math.floor(e.drawingBufferHeight/i-n/i*a),l=new Uint8Array(4);e.bindFramebuffer(e.FRAMEBUFFER,t),e.readPixels(o,s,1,1,e.RGBA,e.UNSIGNED_BYTE,l);var u=qh(l,4),d=u[0],p=u[1],f=u[2],m=u[3];return[d,p,f,m]}function Je(e,t,r){return(t=ym(t))in e?Object.defineProperty(e,t,{value:r,enumerable:!0,configurable:!0,writable:!0}):e[t]=r,e}function f7(e,t){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),r.push.apply(r,n)}return r}function Ct(e){for(var t=1;t<arguments.length;t++){var r=arguments[t]!=null?arguments[t]:{};t%2?f7(Object(r),!0).forEach(function(n){Je(e,n,r[n])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):f7(Object(r)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))})}return e}function Ike(e,t){for(;!{}.hasOwnProperty.call(e,t)&&(e=jh(e))!==null;);return e}function pR(){return pR=typeof Reflect<"u"&&Reflect.get?Reflect.get.bind():function(e,t,r){var n=Ike(e,t);if(n){var a=Object.getOwnPropertyDescriptor(n,t);return a.get?a.get.call(arguments.length<3?e:r):a.value}},pR.apply(null,arguments)}function XQ(e,t,r,n){var a=pR(jh(e.prototype),t,r);return typeof a=="function"?function(i){return a.apply(r,i)}:a}function Oke(e){return e.normalized?1:e.size}function ow(e){var t=0;return e.forEach(function(r){return t+=Oke(r)}),t}function ZQ(e,t,r){var n=e==="VERTEX"?t.VERTEX_SHADER:t.FRAGMENT_SHADER,a=t.createShader(n);if(a===null)throw new Error("loadShader: error while creating the shader");t.shaderSource(a,r),t.compileShader(a);var i=t.getShaderParameter(a,t.COMPILE_STATUS);if(!i){var o=t.getShaderInfoLog(a);throw t.deleteShader(a),new Error(`loadShader: error while compiling the shader:
`.concat(o,`
`).concat(r))}return a}function Bke(e,t){return ZQ("VERTEX",e,t)}function Pke(e,t){return ZQ("FRAGMENT",e,t)}function zke(e,t){var r=e.createProgram();if(r===null)throw new Error("loadProgram: error while creating the program.");var n,a;for(n=0,a=t.length;n<a;n++)e.attachShader(r,t[n]);e.linkProgram(r);var i=e.getProgramParameter(r,e.LINK_STATUS);if(!i)throw e.deleteProgram(r),new Error("loadProgram: error while linking the program.");return r}function m7(e){var t=e.gl,r=e.buffer,n=e.program,a=e.vertexShader,i=e.fragmentShader;t.deleteShader(a),t.deleteShader(i),t.deleteProgram(n),t.deleteBuffer(r)}function g7(e){return e%1===0?e.toFixed(1):e.toString()}var b7=`#define PICKING_MODE
`,Hke=Je(Je(Je(Je(Je(Je(Je(Je({},WebGL2RenderingContext.BOOL,1),WebGL2RenderingContext.BYTE,1),WebGL2RenderingContext.UNSIGNED_BYTE,1),WebGL2RenderingContext.SHORT,2),WebGL2RenderingContext.UNSIGNED_SHORT,2),WebGL2RenderingContext.INT,4),WebGL2RenderingContext.UNSIGNED_INT,4),WebGL2RenderingContext.FLOAT,4),QQ=(function(){function e(t,r,n){la(this,e),Je(this,"array",new Float32Array),Je(this,"constantArray",new Float32Array),Je(this,"capacity",0),Je(this,"verticesCount",0);var a=this.getDefinition();if(this.VERTICES=a.VERTICES,this.VERTEX_SHADER_SOURCE=a.VERTEX_SHADER_SOURCE,this.FRAGMENT_SHADER_SOURCE=a.FRAGMENT_SHADER_SOURCE,this.UNIFORMS=a.UNIFORMS,this.ATTRIBUTES=a.ATTRIBUTES,this.METHOD=a.METHOD,this.CONSTANT_ATTRIBUTES="CONSTANT_ATTRIBUTES"in a?a.CONSTANT_ATTRIBUTES:[],this.CONSTANT_DATA="CONSTANT_DATA"in a?a.CONSTANT_DATA:[],this.isInstanced="CONSTANT_ATTRIBUTES"in a,this.ATTRIBUTES_ITEMS_COUNT=ow(this.ATTRIBUTES),this.STRIDE=this.VERTICES*this.ATTRIBUTES_ITEMS_COUNT,this.renderer=n,this.normalProgram=this.getProgramInfo("normal",t,a.VERTEX_SHADER_SOURCE,a.FRAGMENT_SHADER_SOURCE,null),this.pickProgram=r?this.getProgramInfo("pick",t,b7+a.VERTEX_SHADER_SOURCE,b7+a.FRAGMENT_SHADER_SOURCE,r):null,this.isInstanced){var i=ow(this.CONSTANT_ATTRIBUTES);if(this.CONSTANT_DATA.length!==this.VERTICES)throw new Error("Program: error while getting constant data (expected ".concat(this.VERTICES," items, received ").concat(this.CONSTANT_DATA.length," instead)"));this.constantArray=new Float32Array(this.CONSTANT_DATA.length*i);for(var o=0;o<this.CONSTANT_DATA.length;o++){var s=this.CONSTANT_DATA[o];if(s.length!==i)throw new Error("Program: error while getting constant data (one vector has ".concat(s.length," items instead of ").concat(i,")"));for(var l=0;l<s.length;l++)this.constantArray[o*i+l]=s[l]}this.STRIDE=this.ATTRIBUTES_ITEMS_COUNT}}return ua(e,[{key:"kill",value:function(){m7(this.normalProgram),this.pickProgram&&(m7(this.pickProgram),this.pickProgram=null)}},{key:"getProgramInfo",value:function(r,n,a,i,o){var s=this.getDefinition(),l=n.createBuffer();if(l===null)throw new Error("Program: error while creating the WebGL buffer.");var u=Bke(n,a),d=Pke(n,i),p=zke(n,[u,d]),f={};s.UNIFORMS.forEach(function(y){var S=n.getUniformLocation(p,y);S&&(f[y]=S)});var m={};s.ATTRIBUTES.forEach(function(y){m[y.name]=n.getAttribLocation(p,y.name)});var b;if("CONSTANT_ATTRIBUTES"in s&&(s.CONSTANT_ATTRIBUTES.forEach(function(y){m[y.name]=n.getAttribLocation(p,y.name)}),b=n.createBuffer(),b===null))throw new Error("Program: error while creating the WebGL constant buffer.");return{name:r,program:p,gl:n,frameBuffer:o,buffer:l,constantBuffer:b||{},uniformLocations:f,attributeLocations:m,isPicking:r==="pick",vertexShader:u,fragmentShader:d}}},{key:"bindProgram",value:function(r){var n=this,a=0,i=r.gl,o=r.buffer;this.isInstanced?(i.bindBuffer(i.ARRAY_BUFFER,r.constantBuffer),a=0,this.CONSTANT_ATTRIBUTES.forEach(function(s){return a+=n.bindAttribute(s,r,a,!1)}),i.bufferData(i.ARRAY_BUFFER,this.constantArray,i.STATIC_DRAW),i.bindBuffer(i.ARRAY_BUFFER,r.buffer),a=0,this.ATTRIBUTES.forEach(function(s){return a+=n.bindAttribute(s,r,a,!0)}),i.bufferData(i.ARRAY_BUFFER,this.array,i.DYNAMIC_DRAW)):(i.bindBuffer(i.ARRAY_BUFFER,o),a=0,this.ATTRIBUTES.forEach(function(s){return a+=n.bindAttribute(s,r,a)}),i.bufferData(i.ARRAY_BUFFER,this.array,i.DYNAMIC_DRAW)),i.bindBuffer(i.ARRAY_BUFFER,null)}},{key:"unbindProgram",value:function(r){var n=this;this.isInstanced?(this.CONSTANT_ATTRIBUTES.forEach(function(a){return n.unbindAttribute(a,r,!1)}),this.ATTRIBUTES.forEach(function(a){return n.unbindAttribute(a,r,!0)})):this.ATTRIBUTES.forEach(function(a){return n.unbindAttribute(a,r)})}},{key:"bindAttribute",value:function(r,n,a,i){var o=Hke[r.type];if(typeof o!="number")throw new Error('Program.bind: yet unsupported attribute type "'.concat(r.type,'"'));var s=n.attributeLocations[r.name],l=n.gl;if(s!==-1){l.enableVertexAttribArray(s);var u=this.isInstanced?(i?this.ATTRIBUTES_ITEMS_COUNT:ow(this.CONSTANT_ATTRIBUTES))*Float32Array.BYTES_PER_ELEMENT:this.ATTRIBUTES_ITEMS_COUNT*Float32Array.BYTES_PER_ELEMENT;if(l.vertexAttribPointer(s,r.size,r.type,r.normalized||!1,u,a),this.isInstanced&&i)if(l instanceof WebGL2RenderingContext)l.vertexAttribDivisor(s,1);else{var d=l.getExtension("ANGLE_instanced_arrays");d&&d.vertexAttribDivisorANGLE(s,1)}}return r.size*o}},{key:"unbindAttribute",value:function(r,n,a){var i=n.attributeLocations[r.name],o=n.gl;if(i!==-1&&(o.disableVertexAttribArray(i),this.isInstanced&&a))if(o instanceof WebGL2RenderingContext)o.vertexAttribDivisor(i,0);else{var s=o.getExtension("ANGLE_instanced_arrays");s&&s.vertexAttribDivisorANGLE(i,0)}}},{key:"reallocate",value:function(r){r!==this.capacity&&(this.capacity=r,this.verticesCount=this.VERTICES*r,this.array=new Float32Array(this.isInstanced?this.capacity*this.ATTRIBUTES_ITEMS_COUNT:this.verticesCount*this.ATTRIBUTES_ITEMS_COUNT))}},{key:"hasNothingToRender",value:function(){return this.verticesCount===0}},{key:"renderProgram",value:function(r,n){var a=n.gl,i=n.program;a.enable(a.BLEND),a.useProgram(i),this.setUniforms(r,n),this.drawWebGL(this.METHOD,n)}},{key:"render",value:function(r){this.hasNothingToRender()||(this.pickProgram&&(this.pickProgram.gl.viewport(0,0,r.width*r.pixelRatio/r.downSizingRatio,r.height*r.pixelRatio/r.downSizingRatio),this.bindProgram(this.pickProgram),this.renderProgram(Ct(Ct({},r),{},{pixelRatio:r.pixelRatio/r.downSizingRatio}),this.pickProgram),this.unbindProgram(this.pickProgram)),this.normalProgram.gl.viewport(0,0,r.width*r.pixelRatio,r.height*r.pixelRatio),this.bindProgram(this.normalProgram),this.renderProgram(r,this.normalProgram),this.unbindProgram(this.normalProgram))}},{key:"drawWebGL",value:function(r,n){var a=n.gl,i=n.frameBuffer;if(a.bindFramebuffer(a.FRAMEBUFFER,i),!this.isInstanced)a.drawArrays(r,0,this.verticesCount);else if(a instanceof WebGL2RenderingContext)a.drawArraysInstanced(r,0,this.VERTICES,this.capacity);else{var o=a.getExtension("ANGLE_instanced_arrays");o&&o.drawArraysInstancedANGLE(r,0,this.VERTICES,this.capacity)}}}])})(),b5=(function(e){function t(){return la(this,t),gi(this,t,arguments)}return bi(t,e),ua(t,[{key:"kill",value:function(){XQ(t,"kill",this)([])}},{key:"process",value:function(n,a,i){var o=a*this.STRIDE;if(i.hidden){for(var s=o+this.STRIDE;o<s;o++)this.array[o]=0;return}return this.processVisibleItem(JQ(n),o,i)}}])})(QQ),Tg=(function(e){function t(){var r;la(this,t);for(var n=arguments.length,a=new Array(n),i=0;i<n;i++)a[i]=arguments[i];return r=gi(this,t,[].concat(a)),Je(r,"drawLabel",void 0),r}return bi(t,e),ua(t,[{key:"kill",value:function(){XQ(t,"kill",this)([])}},{key:"process",value:function(n,a,i,o,s){var l=a*this.STRIDE;if(s.hidden||i.hidden||o.hidden){for(var u=l+this.STRIDE;l<u;l++)this.array[l]=0;return}return this.processVisibleItem(JQ(n),l,i,o,s)}}])})(QQ);function eee(e,t){return(function(){function r(n,a,i){la(this,r),Je(this,"drawLabel",t),this.programs=e.map(function(o){return new o(n,a,i)})}return ua(r,[{key:"reallocate",value:function(a){this.programs.forEach(function(i){return i.reallocate(a)})}},{key:"process",value:function(a,i,o,s,l){this.programs.forEach(function(u){return u.process(a,i,o,s,l)})}},{key:"render",value:function(a){this.programs.forEach(function(i){return i.render(a)})}},{key:"kill",value:function(){this.programs.forEach(function(a){return a.kill()})}}])})()}function Uke(e,t,r,n,a){var i=a.edgeLabelSize,o=a.edgeLabelFont,s=a.edgeLabelWeight,l=a.edgeLabelColor.attribute?t[a.edgeLabelColor.attribute]||a.edgeLabelColor.color||"#000":a.edgeLabelColor.color,u=t.label;if(u){e.fillStyle=l,e.font="".concat(s," ").concat(i,"px ").concat(o);var d=r.size,p=n.size,f=r.x,m=r.y,b=n.x,y=n.y,S=(f+b)/2,w=(m+y)/2,E=b-f,k=y-m,_=Math.sqrt(E*E+k*k);if(!(_<d+p)){f+=E*d/_,m+=k*d/_,b-=E*p/_,y-=k*p/_,S=(f+b)/2,w=(m+y)/2,E=b-f,k=y-m,_=Math.sqrt(E*E+k*k);var R=e.measureText(u).width;if(R>_){var C="…";for(u=u+C,R=e.measureText(u).width;R>_&&u.length>1;)u=u.slice(0,-2)+C,R=e.measureText(u).width;if(u.length<4)return}var F;E>0?k>0?F=Math.acos(E/_):F=Math.asin(k/_):k>0?F=Math.acos(E/_)+Math.PI:F=Math.asin(E/_)+Math.PI/2,e.save(),e.translate(S,w),e.rotate(F),e.fillText(u,-R/2,t.size/2+i),e.restore()}}}function tee(e,t,r){if(t.label){var n=r.labelSize,a=r.labelFont,i=r.labelWeight,o=r.labelColor.attribute?t[r.labelColor.attribute]||r.labelColor.color||"#000":r.labelColor.color;e.fillStyle=o,e.font="".concat(i," ").concat(n,"px ").concat(a),e.fillText(t.label,t.x+t.size+3,t.y+n/3)}}function $ke(e,t,r){var n=r.labelSize,a=r.labelFont,i=r.labelWeight;e.font="".concat(i," ").concat(n,"px ").concat(a),e.fillStyle="#FFF",e.shadowOffsetX=0,e.shadowOffsetY=0,e.shadowBlur=8,e.shadowColor="#000";var o=2;if(typeof t.label=="string"){var s=e.measureText(t.label).width,l=Math.round(s+5),u=Math.round(n+2*o),d=Math.max(t.size,n/2)+o,p=Math.asin(u/2/d),f=Math.sqrt(Math.abs(Math.pow(d,2)-Math.pow(u/2,2)));e.beginPath(),e.moveTo(t.x+f,t.y+u/2),e.lineTo(t.x+d+l,t.y+u/2),e.lineTo(t.x+d+l,t.y-u/2),e.lineTo(t.x+f,t.y-u/2),e.arc(t.x,t.y,d,p,-p),e.closePath(),e.fill()}else e.beginPath(),e.arc(t.x,t.y,t.size+o,0,Math.PI*2),e.closePath(),e.fill();e.shadowOffsetX=0,e.shadowOffsetY=0,e.shadowBlur=0,tee(e,t,r)}var Gke=`
precision highp float;

varying vec4 v_color;
varying vec2 v_diffVector;
varying float v_radius;

uniform float u_correctionRatio;

const vec4 transparent = vec4(0.0, 0.0, 0.0, 0.0);

void main(void) {
  float border = u_correctionRatio * 2.0;
  float dist = length(v_diffVector) - v_radius + border;

  // No antialiasing for picking mode:
  #ifdef PICKING_MODE
  if (dist > border)
    gl_FragColor = transparent;
  else
    gl_FragColor = v_color;

  #else
  float t = 0.0;
  if (dist > border)
    t = 1.0;
  else if (dist > 0.0)
    t = dist / border;

  gl_FragColor = mix(v_color, transparent, t);
  #endif
}
`,jke=Gke,qke=`
attribute vec4 a_id;
attribute vec4 a_color;
attribute vec2 a_position;
attribute float a_size;
attribute float a_angle;

uniform mat3 u_matrix;
uniform float u_sizeRatio;
uniform float u_correctionRatio;

varying vec4 v_color;
varying vec2 v_diffVector;
varying float v_radius;
varying float v_border;

const float bias = 255.0 / 254.0;

void main() {
  float size = a_size * u_correctionRatio / u_sizeRatio * 4.0;
  vec2 diffVector = size * vec2(cos(a_angle), sin(a_angle));
  vec2 position = a_position + diffVector;
  gl_Position = vec4(
    (u_matrix * vec3(position, 1)).xy,
    0,
    1
  );

  v_diffVector = diffVector;
  v_radius = size / 2.0;

  #ifdef PICKING_MODE
  // For picking mode, we use the ID as the color:
  v_color = a_id;
  #else
  // For normal mode, we use the color:
  v_color = a_color;
  #endif

  v_color.a *= bias;
}
`,Wke=qke,ree=WebGLRenderingContext,y7=ree.UNSIGNED_BYTE,sw=ree.FLOAT,Vke=["u_sizeRatio","u_correctionRatio","u_matrix"],Ag=(function(e){function t(){return la(this,t),gi(this,t,arguments)}return bi(t,e),ua(t,[{key:"getDefinition",value:function(){return{VERTICES:3,VERTEX_SHADER_SOURCE:Wke,FRAGMENT_SHADER_SOURCE:jke,METHOD:WebGLRenderingContext.TRIANGLES,UNIFORMS:Vke,ATTRIBUTES:[{name:"a_position",size:2,type:sw},{name:"a_size",size:1,type:sw},{name:"a_color",size:4,type:y7,normalized:!0},{name:"a_id",size:4,type:y7,normalized:!0}],CONSTANT_ATTRIBUTES:[{name:"a_angle",size:1,type:sw}],CONSTANT_DATA:[[t.ANGLE_1],[t.ANGLE_2],[t.ANGLE_3]]}}},{key:"processVisibleItem",value:function(n,a,i){var o=this.array,s=bs(i.color);o[a++]=i.x,o[a++]=i.y,o[a++]=i.size,o[a++]=s,o[a++]=n}},{key:"setUniforms",value:function(n,a){var i=a.gl,o=a.uniformLocations,s=o.u_sizeRatio,l=o.u_correctionRatio,u=o.u_matrix;i.uniform1f(l,n.correctionRatio),i.uniform1f(s,n.sizeRatio),i.uniformMatrix3fv(u,!1,n.matrix)}}])})(b5);Je(Ag,"ANGLE_1",0);Je(Ag,"ANGLE_2",2*Math.PI/3);Je(Ag,"ANGLE_3",4*Math.PI/3);var Kke=`
precision mediump float;

varying vec4 v_color;

void main(void) {
  gl_FragColor = v_color;
}
`,Yke=Kke,Jke=`
attribute vec2 a_position;
attribute vec2 a_normal;
attribute float a_radius;
attribute vec3 a_barycentric;

#ifdef PICKING_MODE
attribute vec4 a_id;
#else
attribute vec4 a_color;
#endif

uniform mat3 u_matrix;
uniform float u_sizeRatio;
uniform float u_correctionRatio;
uniform float u_minEdgeThickness;
uniform float u_lengthToThicknessRatio;
uniform float u_widenessToThicknessRatio;

varying vec4 v_color;

const float bias = 255.0 / 254.0;

void main() {
  float minThickness = u_minEdgeThickness;

  float normalLength = length(a_normal);
  vec2 unitNormal = a_normal / normalLength;

  // These first computations are taken from edge.vert.glsl and
  // edge.clamped.vert.glsl. Please read it to get better comments on what's
  // happening:
  float pixelsThickness = max(normalLength / u_sizeRatio, minThickness);
  float webGLThickness = pixelsThickness * u_correctionRatio;
  float webGLNodeRadius = a_radius * 2.0 * u_correctionRatio / u_sizeRatio;
  float webGLArrowHeadLength = webGLThickness * u_lengthToThicknessRatio * 2.0;
  float webGLArrowHeadThickness = webGLThickness * u_widenessToThicknessRatio;

  float da = a_barycentric.x;
  float db = a_barycentric.y;
  float dc = a_barycentric.z;

  vec2 delta = vec2(
      da * (webGLNodeRadius * unitNormal.y)
    + db * ((webGLNodeRadius + webGLArrowHeadLength) * unitNormal.y + webGLArrowHeadThickness * unitNormal.x)
    + dc * ((webGLNodeRadius + webGLArrowHeadLength) * unitNormal.y - webGLArrowHeadThickness * unitNormal.x),

      da * (-webGLNodeRadius * unitNormal.x)
    + db * (-(webGLNodeRadius + webGLArrowHeadLength) * unitNormal.x + webGLArrowHeadThickness * unitNormal.y)
    + dc * (-(webGLNodeRadius + webGLArrowHeadLength) * unitNormal.x - webGLArrowHeadThickness * unitNormal.y)
  );

  vec2 position = (u_matrix * vec3(a_position + delta, 1)).xy;

  gl_Position = vec4(position, 0, 1);

  #ifdef PICKING_MODE
  // For picking mode, we use the ID as the color:
  v_color = a_id;
  #else
  // For normal mode, we use the color:
  v_color = a_color;
  #endif

  v_color.a *= bias;
}
`,Xke=Jke,nee=WebGLRenderingContext,v7=nee.UNSIGNED_BYTE,Bb=nee.FLOAT,Zke=["u_matrix","u_sizeRatio","u_correctionRatio","u_minEdgeThickness","u_lengthToThicknessRatio","u_widenessToThicknessRatio"],Fg={extremity:"target",lengthToThicknessRatio:2.5,widenessToThicknessRatio:2};function by(e){var t=Ct(Ct({},Fg),e||{});return(function(r){function n(){return la(this,n),gi(this,n,arguments)}return bi(n,r),ua(n,[{key:"getDefinition",value:function(){return{VERTICES:3,VERTEX_SHADER_SOURCE:Xke,FRAGMENT_SHADER_SOURCE:Yke,METHOD:WebGLRenderingContext.TRIANGLES,UNIFORMS:Zke,ATTRIBUTES:[{name:"a_position",size:2,type:Bb},{name:"a_normal",size:2,type:Bb},{name:"a_radius",size:1,type:Bb},{name:"a_color",size:4,type:v7,normalized:!0},{name:"a_id",size:4,type:v7,normalized:!0}],CONSTANT_ATTRIBUTES:[{name:"a_barycentric",size:3,type:Bb}],CONSTANT_DATA:[[1,0,0],[0,1,0],[0,0,1]]}}},{key:"processVisibleItem",value:function(i,o,s,l,u){if(t.extremity==="source"){var d=[l,s];s=d[0],l=d[1]}var p=u.size||1,f=l.size||1,m=s.x,b=s.y,y=l.x,S=l.y,w=bs(u.color),E=y-m,k=S-b,_=E*E+k*k,R=0,C=0;_&&(_=1/Math.sqrt(_),R=-k*_*p,C=E*_*p);var F=this.array;F[o++]=y,F[o++]=S,F[o++]=-R,F[o++]=-C,F[o++]=f,F[o++]=w,F[o++]=i}},{key:"setUniforms",value:function(i,o){var s=o.gl,l=o.uniformLocations,u=l.u_matrix,d=l.u_sizeRatio,p=l.u_correctionRatio,f=l.u_minEdgeThickness,m=l.u_lengthToThicknessRatio,b=l.u_widenessToThicknessRatio;s.uniformMatrix3fv(u,!1,i.matrix),s.uniform1f(d,i.sizeRatio),s.uniform1f(p,i.correctionRatio),s.uniform1f(f,i.minEdgeThickness),s.uniform1f(m,t.lengthToThicknessRatio),s.uniform1f(b,t.widenessToThicknessRatio)}}])})(Tg)}by();var Qke=`
precision mediump float;

varying vec4 v_color;
varying vec2 v_normal;
varying float v_thickness;
varying float v_feather;

const vec4 transparent = vec4(0.0, 0.0, 0.0, 0.0);

void main(void) {
  // We only handle antialiasing for normal mode:
  #ifdef PICKING_MODE
  gl_FragColor = v_color;
  #else
  float dist = length(v_normal) * v_thickness;

  float t = smoothstep(
    v_thickness - v_feather,
    v_thickness,
    dist
  );

  gl_FragColor = mix(v_color, transparent, t);
  #endif
}
`,y5=Qke,eCe=`
attribute vec4 a_id;
attribute vec4 a_color;
attribute vec2 a_normal;
attribute float a_normalCoef;
attribute vec2 a_positionStart;
attribute vec2 a_positionEnd;
attribute float a_positionCoef;
attribute float a_radius;
attribute float a_radiusCoef;

uniform mat3 u_matrix;
uniform float u_zoomRatio;
uniform float u_sizeRatio;
uniform float u_pixelRatio;
uniform float u_correctionRatio;
uniform float u_minEdgeThickness;
uniform float u_lengthToThicknessRatio;
uniform float u_feather;

varying vec4 v_color;
varying vec2 v_normal;
varying float v_thickness;
varying float v_feather;

const float bias = 255.0 / 254.0;

void main() {
  float minThickness = u_minEdgeThickness;

  float radius = a_radius * a_radiusCoef;
  vec2 normal = a_normal * a_normalCoef;
  vec2 position = a_positionStart * (1.0 - a_positionCoef) + a_positionEnd * a_positionCoef;

  float normalLength = length(normal);
  vec2 unitNormal = normal / normalLength;

  // These first computations are taken from edge.vert.glsl. Please read it to
  // get better comments on what's happening:
  float pixelsThickness = max(normalLength, minThickness * u_sizeRatio);
  float webGLThickness = pixelsThickness * u_correctionRatio / u_sizeRatio;

  // Here, we move the point to leave space for the arrow head:
  float direction = sign(radius);
  float webGLNodeRadius = direction * radius * 2.0 * u_correctionRatio / u_sizeRatio;
  float webGLArrowHeadLength = webGLThickness * u_lengthToThicknessRatio * 2.0;

  vec2 compensationVector = vec2(-direction * unitNormal.y, direction * unitNormal.x) * (webGLNodeRadius + webGLArrowHeadLength);

  // Here is the proper position of the vertex
  gl_Position = vec4((u_matrix * vec3(position + unitNormal * webGLThickness + compensationVector, 1)).xy, 0, 1);

  v_thickness = webGLThickness / u_zoomRatio;

  v_normal = unitNormal;

  v_feather = u_feather * u_correctionRatio / u_zoomRatio / u_pixelRatio * 2.0;

  #ifdef PICKING_MODE
  // For picking mode, we use the ID as the color:
  v_color = a_id;
  #else
  // For normal mode, we use the color:
  v_color = a_color;
  #endif

  v_color.a *= bias;
}
`,tCe=eCe,aee=WebGLRenderingContext,S7=aee.UNSIGNED_BYTE,dc=aee.FLOAT,rCe=["u_matrix","u_zoomRatio","u_sizeRatio","u_correctionRatio","u_pixelRatio","u_feather","u_minEdgeThickness","u_lengthToThicknessRatio"],nCe={lengthToThicknessRatio:Fg.lengthToThicknessRatio};function iee(e){var t=Ct(Ct({},nCe),{});return(function(r){function n(){return la(this,n),gi(this,n,arguments)}return bi(n,r),ua(n,[{key:"getDefinition",value:function(){return{VERTICES:6,VERTEX_SHADER_SOURCE:tCe,FRAGMENT_SHADER_SOURCE:y5,METHOD:WebGLRenderingContext.TRIANGLES,UNIFORMS:rCe,ATTRIBUTES:[{name:"a_positionStart",size:2,type:dc},{name:"a_positionEnd",size:2,type:dc},{name:"a_normal",size:2,type:dc},{name:"a_color",size:4,type:S7,normalized:!0},{name:"a_id",size:4,type:S7,normalized:!0},{name:"a_radius",size:1,type:dc}],CONSTANT_ATTRIBUTES:[{name:"a_positionCoef",size:1,type:dc},{name:"a_normalCoef",size:1,type:dc},{name:"a_radiusCoef",size:1,type:dc}],CONSTANT_DATA:[[0,1,0],[0,-1,0],[1,1,1],[1,1,1],[0,-1,0],[1,-1,-1]]}}},{key:"processVisibleItem",value:function(i,o,s,l,u){var d=u.size||1,p=s.x,f=s.y,m=l.x,b=l.y,y=bs(u.color),S=m-p,w=b-f,E=l.size||1,k=S*S+w*w,_=0,R=0;k&&(k=1/Math.sqrt(k),_=-w*k*d,R=S*k*d);var C=this.array;C[o++]=p,C[o++]=f,C[o++]=m,C[o++]=b,C[o++]=_,C[o++]=R,C[o++]=y,C[o++]=i,C[o++]=E}},{key:"setUniforms",value:function(i,o){var s=o.gl,l=o.uniformLocations,u=l.u_matrix,d=l.u_zoomRatio,p=l.u_feather,f=l.u_pixelRatio,m=l.u_correctionRatio,b=l.u_sizeRatio,y=l.u_minEdgeThickness,S=l.u_lengthToThicknessRatio;s.uniformMatrix3fv(u,!1,i.matrix),s.uniform1f(d,i.zoomRatio),s.uniform1f(b,i.sizeRatio),s.uniform1f(m,i.correctionRatio),s.uniform1f(f,i.pixelRatio),s.uniform1f(p,i.antiAliasingFeather),s.uniform1f(y,i.minEdgeThickness),s.uniform1f(S,t.lengthToThicknessRatio)}}])})(Tg)}iee();function aCe(e){return eee([iee(),by(e)])}var iCe=aCe(),oee=iCe,oCe=`
attribute vec4 a_id;
attribute vec4 a_color;
attribute vec2 a_normal;
attribute float a_normalCoef;
attribute vec2 a_positionStart;
attribute vec2 a_positionEnd;
attribute float a_positionCoef;

uniform mat3 u_matrix;
uniform float u_sizeRatio;
uniform float u_zoomRatio;
uniform float u_pixelRatio;
uniform float u_correctionRatio;
uniform float u_minEdgeThickness;
uniform float u_feather;

varying vec4 v_color;
varying vec2 v_normal;
varying float v_thickness;
varying float v_feather;

const float bias = 255.0 / 254.0;

void main() {
  float minThickness = u_minEdgeThickness;

  vec2 normal = a_normal * a_normalCoef;
  vec2 position = a_positionStart * (1.0 - a_positionCoef) + a_positionEnd * a_positionCoef;

  float normalLength = length(normal);
  vec2 unitNormal = normal / normalLength;

  // We require edges to be at least "minThickness" pixels thick *on screen*
  // (so we need to compensate the size ratio):
  float pixelsThickness = max(normalLength, minThickness * u_sizeRatio);

  // Then, we need to retrieve the normalized thickness of the edge in the WebGL
  // referential (in a ([0, 1], [0, 1]) space), using our "magic" correction
  // ratio:
  float webGLThickness = pixelsThickness * u_correctionRatio / u_sizeRatio;

  // Here is the proper position of the vertex
  gl_Position = vec4((u_matrix * vec3(position + unitNormal * webGLThickness, 1)).xy, 0, 1);

  // For the fragment shader though, we need a thickness that takes the "magic"
  // correction ratio into account (as in webGLThickness), but so that the
  // antialiasing effect does not depend on the zoom level. So here's yet
  // another thickness version:
  v_thickness = webGLThickness / u_zoomRatio;

  v_normal = unitNormal;

  v_feather = u_feather * u_correctionRatio / u_zoomRatio / u_pixelRatio * 2.0;

  #ifdef PICKING_MODE
  // For picking mode, we use the ID as the color:
  v_color = a_id;
  #else
  // For normal mode, we use the color:
  v_color = a_color;
  #endif

  v_color.a *= bias;
}
`,sCe=oCe,see=WebGLRenderingContext,E7=see.UNSIGNED_BYTE,Nf=see.FLOAT,lCe=["u_matrix","u_zoomRatio","u_sizeRatio","u_correctionRatio","u_pixelRatio","u_feather","u_minEdgeThickness"],uCe=(function(e){function t(){return la(this,t),gi(this,t,arguments)}return bi(t,e),ua(t,[{key:"getDefinition",value:function(){return{VERTICES:6,VERTEX_SHADER_SOURCE:sCe,FRAGMENT_SHADER_SOURCE:y5,METHOD:WebGLRenderingContext.TRIANGLES,UNIFORMS:lCe,ATTRIBUTES:[{name:"a_positionStart",size:2,type:Nf},{name:"a_positionEnd",size:2,type:Nf},{name:"a_normal",size:2,type:Nf},{name:"a_color",size:4,type:E7,normalized:!0},{name:"a_id",size:4,type:E7,normalized:!0}],CONSTANT_ATTRIBUTES:[{name:"a_positionCoef",size:1,type:Nf},{name:"a_normalCoef",size:1,type:Nf}],CONSTANT_DATA:[[0,1],[0,-1],[1,1],[1,1],[0,-1],[1,-1]]}}},{key:"processVisibleItem",value:function(n,a,i,o,s){var l=s.size||1,u=i.x,d=i.y,p=o.x,f=o.y,m=bs(s.color),b=p-u,y=f-d,S=b*b+y*y,w=0,E=0;S&&(S=1/Math.sqrt(S),w=-y*S*l,E=b*S*l);var k=this.array;k[a++]=u,k[a++]=d,k[a++]=p,k[a++]=f,k[a++]=w,k[a++]=E,k[a++]=m,k[a++]=n}},{key:"setUniforms",value:function(n,a){var i=a.gl,o=a.uniformLocations,s=o.u_matrix,l=o.u_zoomRatio,u=o.u_feather,d=o.u_pixelRatio,p=o.u_correctionRatio,f=o.u_sizeRatio,m=o.u_minEdgeThickness;i.uniformMatrix3fv(s,!1,n.matrix),i.uniform1f(l,n.zoomRatio),i.uniform1f(f,n.sizeRatio),i.uniform1f(p,n.correctionRatio),i.uniform1f(d,n.pixelRatio),i.uniform1f(u,n.antiAliasingFeather),i.uniform1f(m,n.minEdgeThickness)}}])})(Tg),v5=(function(e){function t(){var r;return la(this,t),r=gi(this,t),r.rawEmitter=r,r}return bi(t,e),ua(t)})(DQ.EventEmitter),lw,x7;function ho(){return x7||(x7=1,lw=function(t){return t!==null&&typeof t=="object"&&typeof t.addUndirectedEdgeWithKey=="function"&&typeof t.dropNode=="function"&&typeof t.multi=="boolean"}),lw}var cCe=ho();const dCe=jn(cCe);var hCe=function(t){return t},pCe=function(t){return t*t},fCe=function(t){return t*(2-t)},mCe=function(t){return(t*=2)<1?.5*t*t:-.5*(--t*(t-2)-1)},gCe=function(t){return t*t*t},bCe=function(t){return--t*t*t+1},yCe=function(t){return(t*=2)<1?.5*t*t*t:.5*((t-=2)*t*t+2)},lee={linear:hCe,quadraticIn:pCe,quadraticOut:fCe,quadraticInOut:mCe,cubicIn:gCe,cubicOut:bCe,cubicInOut:yCe},uee={easing:"quadraticInOut",duration:150};function cee(e,t,r,n){var a=Object.assign({},uee,r),i=typeof a.easing=="function"?a.easing:lee[a.easing],o=Date.now(),s={};for(var l in t){var u=t[l];s[l]={};for(var d in u)s[l][d]=e.getNodeAttribute(l,d)}var p=null,f=function(){p=null;var b=(Date.now()-o)/a.duration;if(b>=1){for(var y in t){var S=t[y];for(var w in S)e.setNodeAttribute(y,w,S[w])}return}b=i(b);for(var E in t){var k=t[E],_=s[E];for(var R in k)e.setNodeAttribute(E,R,k[R]*b+_[R]*(1-b))}p=requestAnimationFrame(f)};return f(),function(){p&&cancelAnimationFrame(p)}}function xo(){return Float32Array.of(1,0,0,0,1,0,0,0,1)}function Pb(e,t,r){return e[0]=t,e[4]=typeof r=="number"?r:t,e}function w7(e,t){var r=Math.sin(t),n=Math.cos(t);return e[0]=n,e[1]=r,e[3]=-r,e[4]=n,e}function k7(e,t,r){return e[6]=t,e[7]=r,e}function ou(e,t){var r=e[0],n=e[1],a=e[2],i=e[3],o=e[4],s=e[5],l=e[6],u=e[7],d=e[8],p=t[0],f=t[1],m=t[2],b=t[3],y=t[4],S=t[5],w=t[6],E=t[7],k=t[8];return e[0]=p*r+f*i+m*l,e[1]=p*n+f*o+m*u,e[2]=p*a+f*s+m*d,e[3]=b*r+y*i+S*l,e[4]=b*n+y*o+S*u,e[5]=b*a+y*s+S*d,e[6]=w*r+E*i+k*l,e[7]=w*n+E*o+k*u,e[8]=w*a+E*s+k*d,e}function fR(e,t){var r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:1,n=e[0],a=e[1],i=e[3],o=e[4],s=e[6],l=e[7],u=t.x,d=t.y;return{x:u*n+d*i+s*r,y:u*a+d*o+l*r}}function vCe(e,t){var r=e.height/e.width,n=t.height/t.width;return r<1&&n>1||r>1&&n<1?1:Math.min(Math.max(n,1/n),Math.max(1/r,r))}function Lf(e,t,r,n,a){var i=e.angle,o=e.ratio,s=e.x,l=e.y,u=t.width,d=t.height,p=xo(),f=Math.min(u,d)-2*n,m=vCe(t,r);return a?(ou(p,k7(xo(),s,l)),ou(p,Pb(xo(),o)),ou(p,w7(xo(),i)),ou(p,Pb(xo(),u/f/2/m,d/f/2/m))):(ou(p,Pb(xo(),2*(f/u)*m,2*(f/d)*m)),ou(p,w7(xo(),-i)),ou(p,Pb(xo(),1/o)),ou(p,k7(xo(),-s,-l))),p}function SCe(e,t,r){var n=fR(e,{x:Math.cos(t.angle),y:Math.sin(t.angle)},0),a=n.x,i=n.y;return 1/Math.sqrt(Math.pow(a,2)+Math.pow(i,2))/r.width}function ECe(e){if(!e.order)return{x:[0,1],y:[0,1]};var t=1/0,r=-1/0,n=1/0,a=-1/0;return e.forEachNode(function(i,o){var s=o.x,l=o.y;s<t&&(t=s),s>r&&(r=s),l<n&&(n=l),l>a&&(a=l)}),{x:[t,r],y:[n,a]}}function xCe(e){if(!dCe(e))throw new Error("Sigma: invalid graph instance.");e.forEachNode(function(t,r){if(!Number.isFinite(r.x)||!Number.isFinite(r.y))throw new Error("Sigma: Coordinates of node ".concat(t," are invalid. A node must have a numeric 'x' and 'y' attribute."))})}function wCe(e,t,r){var n=document.createElement(e);if(t)for(var a in t)n.style[a]=t[a];if(r)for(var i in r)n.setAttribute(i,r[i]);return n}function C7(){return typeof window.devicePixelRatio<"u"?window.devicePixelRatio:1}function T7(e,t,r){return r.sort(function(n,a){var i=t(n)||0,o=t(a)||0;return i<o?-1:i>o?1:0})}function A7(e){var t=qh(e.x,2),r=t[0],n=t[1],a=qh(e.y,2),i=a[0],o=a[1],s=Math.max(n-r,o-i),l=(n+r)/2,u=(o+i)/2;(s===0||Math.abs(s)===1/0||isNaN(s))&&(s=1),isNaN(l)&&(l=0),isNaN(u)&&(u=0);var d=function(f){return{x:.5+(f.x-l)/s,y:.5+(f.y-u)/s}};return d.applyTo=function(p){p.x=.5+(p.x-l)/s,p.y=.5+(p.y-u)/s},d.inverse=function(p){return{x:l+s*(p.x-.5),y:u+s*(p.y-.5)}},d.ratio=s,d}function mR(e){"@babel/helpers - typeof";return mR=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},mR(e)}function F7(e,t){var r=t.size;if(r!==0){var n=e.length;e.length+=r;var a=0;t.forEach(function(i){e[n+a]=i,a++})}}function uw(e){e=e||{};for(var t=0,r=arguments.length<=1?0:arguments.length-1;t<r;t++){var n=t+1<1||arguments.length<=t+1?void 0:arguments[t+1];n&&Object.assign(e,n)}return e}var S5={hideEdgesOnMove:!1,hideLabelsOnMove:!1,renderLabels:!0,renderEdgeLabels:!1,enableEdgeEvents:!1,defaultNodeColor:"#999",defaultNodeType:"circle",defaultEdgeColor:"#ccc",defaultEdgeType:"line",labelFont:"Arial",labelSize:14,labelWeight:"normal",labelColor:{color:"#000"},edgeLabelFont:"Arial",edgeLabelSize:14,edgeLabelWeight:"normal",edgeLabelColor:{attribute:"color"},stagePadding:30,defaultDrawEdgeLabel:Uke,defaultDrawNodeLabel:tee,defaultDrawNodeHover:$ke,minEdgeThickness:1.7,antiAliasingFeather:1,dragTimeout:100,draggedEventsTolerance:3,inertiaDuration:200,inertiaRatio:3,zoomDuration:250,zoomingRatio:1.7,doubleClickTimeout:300,doubleClickZoomingRatio:2.2,doubleClickZoomingDuration:200,tapMoveTolerance:10,zoomToSizeRatioFunction:Math.sqrt,itemSizesReference:"screen",autoRescale:!0,autoCenter:!0,labelDensity:1,labelGridCellSize:100,labelRenderedSizeThreshold:6,nodeReducer:null,edgeReducer:null,zIndex:!1,minCameraRatio:null,maxCameraRatio:null,enableCameraZooming:!0,enableCameraPanning:!0,enableCameraRotation:!0,cameraPanBoundaries:null,allowInvalidContainer:!1,nodeProgramClasses:{},nodeHoverProgramClasses:{},edgeProgramClasses:{}},kCe={circle:Ag},CCe={arrow:oee,line:uCe};function cw(e){if(typeof e.labelDensity!="number"||e.labelDensity<0)throw new Error("Settings: invalid `labelDensity`. Expecting a positive number.");var t=e.minCameraRatio,r=e.maxCameraRatio;if(typeof t=="number"&&typeof r=="number"&&r<t)throw new Error("Settings: invalid camera ratio boundaries. Expecting `maxCameraRatio` to be greater than `minCameraRatio`.")}function TCe(e){var t=uw({},S5,e);return t.nodeProgramClasses=uw({},kCe,t.nodeProgramClasses),t.edgeProgramClasses=uw({},CCe,t.edgeProgramClasses),t}var zb=1.5,_7=(function(e){function t(){var r;return la(this,t),r=gi(this,t),Je(r,"x",.5),Je(r,"y",.5),Je(r,"angle",0),Je(r,"ratio",1),Je(r,"minRatio",null),Je(r,"maxRatio",null),Je(r,"enabledZooming",!0),Je(r,"enabledPanning",!0),Je(r,"enabledRotation",!0),Je(r,"clean",null),Je(r,"nextFrame",null),Je(r,"previousState",null),Je(r,"enabled",!0),r.previousState=r.getState(),r}return bi(t,e),ua(t,[{key:"enable",value:function(){return this.enabled=!0,this}},{key:"disable",value:function(){return this.enabled=!1,this}},{key:"getState",value:function(){return{x:this.x,y:this.y,angle:this.angle,ratio:this.ratio}}},{key:"hasState",value:function(n){return this.x===n.x&&this.y===n.y&&this.ratio===n.ratio&&this.angle===n.angle}},{key:"getPreviousState",value:function(){var n=this.previousState;return n?{x:n.x,y:n.y,angle:n.angle,ratio:n.ratio}:null}},{key:"getBoundedRatio",value:function(n){var a=n;return typeof this.minRatio=="number"&&(a=Math.max(a,this.minRatio)),typeof this.maxRatio=="number"&&(a=Math.min(a,this.maxRatio)),a}},{key:"validateState",value:function(n){var a={};return this.enabledPanning&&typeof n.x=="number"&&(a.x=n.x),this.enabledPanning&&typeof n.y=="number"&&(a.y=n.y),this.enabledZooming&&typeof n.ratio=="number"&&(a.ratio=this.getBoundedRatio(n.ratio)),this.enabledRotation&&typeof n.angle=="number"&&(a.angle=n.angle),this.clean?this.clean(Ct(Ct({},this.getState()),a)):a}},{key:"isAnimated",value:function(){return!!this.nextFrame}},{key:"setState",value:function(n){if(!this.enabled)return this;this.previousState=this.getState();var a=this.validateState(n);return typeof a.x=="number"&&(this.x=a.x),typeof a.y=="number"&&(this.y=a.y),typeof a.ratio=="number"&&(this.ratio=a.ratio),typeof a.angle=="number"&&(this.angle=a.angle),this.hasState(this.previousState)||this.emit("updated",this.getState()),this}},{key:"updateState",value:function(n){return this.setState(n(this.getState())),this}},{key:"animate",value:function(n){var a=this,i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},o=arguments.length>2?arguments[2]:void 0;if(!o)return new Promise(function(m){return a.animate(n,i,m)});if(this.enabled){var s=Ct(Ct({},uee),i),l=this.validateState(n),u=typeof s.easing=="function"?s.easing:lee[s.easing],d=Date.now(),p=this.getState(),f=function(){var b=(Date.now()-d)/s.duration;if(b>=1){a.nextFrame=null,a.setState(l),a.animationCallback&&(a.animationCallback.call(null),a.animationCallback=void 0);return}var y=u(b),S={};typeof l.x=="number"&&(S.x=p.x+(l.x-p.x)*y),typeof l.y=="number"&&(S.y=p.y+(l.y-p.y)*y),a.enabledRotation&&typeof l.angle=="number"&&(S.angle=p.angle+(l.angle-p.angle)*y),typeof l.ratio=="number"&&(S.ratio=p.ratio+(l.ratio-p.ratio)*y),a.setState(S),a.nextFrame=requestAnimationFrame(f)};this.nextFrame?(cancelAnimationFrame(this.nextFrame),this.animationCallback&&this.animationCallback.call(null),this.nextFrame=requestAnimationFrame(f)):f(),this.animationCallback=o}}},{key:"animatedZoom",value:function(n){return n?typeof n=="number"?this.animate({ratio:this.ratio/n}):this.animate({ratio:this.ratio/(n.factor||zb)},n):this.animate({ratio:this.ratio/zb})}},{key:"animatedUnzoom",value:function(n){return n?typeof n=="number"?this.animate({ratio:this.ratio*n}):this.animate({ratio:this.ratio*(n.factor||zb)},n):this.animate({ratio:this.ratio*zb})}},{key:"animatedReset",value:function(n){return this.animate({x:.5,y:.5,ratio:1,angle:0},n)}},{key:"copy",value:function(){return t.from(this.getState())}}],[{key:"from",value:function(n){var a=new t;return a.setState(n)}}])})(v5);function Ao(e,t){var r=t.getBoundingClientRect();return{x:e.clientX-r.left,y:e.clientY-r.top}}function Xs(e,t){var r=Ct(Ct({},Ao(e,t)),{},{sigmaDefaultPrevented:!1,preventSigmaDefault:function(){r.sigmaDefaultPrevented=!0},original:e});return r}function Mf(e){var t="x"in e?e:Ct(Ct({},e.touches[0]||e.previousTouches[0]),{},{original:e.original,sigmaDefaultPrevented:e.sigmaDefaultPrevented,preventSigmaDefault:function(){e.sigmaDefaultPrevented=!0,t.sigmaDefaultPrevented=!0}});return t}function ACe(e,t){return Ct(Ct({},Xs(e,t)),{},{delta:dee(e)})}var FCe=2;function A1(e){for(var t=[],r=0,n=Math.min(e.length,FCe);r<n;r++)t.push(e[r]);return t}function If(e,t,r){var n={touches:A1(e.touches).map(function(a){return Ao(a,r)}),previousTouches:t.map(function(a){return Ao(a,r)}),sigmaDefaultPrevented:!1,preventSigmaDefault:function(){n.sigmaDefaultPrevented=!0},original:e};return n}function dee(e){if(typeof e.deltaY<"u")return e.deltaY*-3/360;if(typeof e.detail<"u")return e.detail/-9;throw new Error("Captor: could not extract delta from event.")}var hee=(function(e){function t(r,n){var a;return la(this,t),a=gi(this,t),a.container=r,a.renderer=n,a}return bi(t,e),ua(t)})(v5),_Ce=["doubleClickTimeout","doubleClickZoomingDuration","doubleClickZoomingRatio","dragTimeout","draggedEventsTolerance","inertiaDuration","inertiaRatio","zoomDuration","zoomingRatio"],RCe=_Ce.reduce(function(e,t){return Ct(Ct({},e),{},Je({},t,S5[t]))},{}),DCe=(function(e){function t(r,n){var a;return la(this,t),a=gi(this,t,[r,n]),Je(a,"enabled",!0),Je(a,"draggedEvents",0),Je(a,"downStartTime",null),Je(a,"lastMouseX",null),Je(a,"lastMouseY",null),Je(a,"isMouseDown",!1),Je(a,"isMoving",!1),Je(a,"movingTimeout",null),Je(a,"startCameraState",null),Je(a,"clicks",0),Je(a,"doubleClickTimeout",null),Je(a,"currentWheelDirection",0),Je(a,"settings",RCe),a.handleClick=a.handleClick.bind(a),a.handleRightClick=a.handleRightClick.bind(a),a.handleDown=a.handleDown.bind(a),a.handleUp=a.handleUp.bind(a),a.handleMove=a.handleMove.bind(a),a.handleWheel=a.handleWheel.bind(a),a.handleLeave=a.handleLeave.bind(a),a.handleEnter=a.handleEnter.bind(a),r.addEventListener("click",a.handleClick,{capture:!1}),r.addEventListener("contextmenu",a.handleRightClick,{capture:!1}),r.addEventListener("mousedown",a.handleDown,{capture:!1}),r.addEventListener("wheel",a.handleWheel,{capture:!1}),r.addEventListener("mouseleave",a.handleLeave,{capture:!1}),r.addEventListener("mouseenter",a.handleEnter,{capture:!1}),document.addEventListener("mousemove",a.handleMove,{capture:!1}),document.addEventListener("mouseup",a.handleUp,{capture:!1}),a}return bi(t,e),ua(t,[{key:"kill",value:function(){var n=this.container;n.removeEventListener("click",this.handleClick),n.removeEventListener("contextmenu",this.handleRightClick),n.removeEventListener("mousedown",this.handleDown),n.removeEventListener("wheel",this.handleWheel),n.removeEventListener("mouseleave",this.handleLeave),n.removeEventListener("mouseenter",this.handleEnter),document.removeEventListener("mousemove",this.handleMove),document.removeEventListener("mouseup",this.handleUp)}},{key:"handleClick",value:function(n){var a=this;if(this.enabled){if(this.clicks++,this.clicks===2)return this.clicks=0,typeof this.doubleClickTimeout=="number"&&(clearTimeout(this.doubleClickTimeout),this.doubleClickTimeout=null),this.handleDoubleClick(n);setTimeout(function(){a.clicks=0,a.doubleClickTimeout=null},this.settings.doubleClickTimeout),this.draggedEvents<this.settings.draggedEventsTolerance&&this.emit("click",Xs(n,this.container))}}},{key:"handleRightClick",value:function(n){this.enabled&&this.emit("rightClick",Xs(n,this.container))}},{key:"handleDoubleClick",value:function(n){if(this.enabled){n.preventDefault(),n.stopPropagation();var a=Xs(n,this.container);if(this.emit("doubleClick",a),!a.sigmaDefaultPrevented){var i=this.renderer.getCamera(),o=i.getBoundedRatio(i.getState().ratio/this.settings.doubleClickZoomingRatio);i.animate(this.renderer.getViewportZoomedState(Ao(n,this.container),o),{easing:"quadraticInOut",duration:this.settings.doubleClickZoomingDuration})}}}},{key:"handleDown",value:function(n){if(this.enabled){if(n.button===0){this.startCameraState=this.renderer.getCamera().getState();var a=Ao(n,this.container),i=a.x,o=a.y;this.lastMouseX=i,this.lastMouseY=o,this.draggedEvents=0,this.downStartTime=Date.now(),this.isMouseDown=!0}this.emit("mousedown",Xs(n,this.container))}}},{key:"handleUp",value:function(n){var a=this;if(!(!this.enabled||!this.isMouseDown)){var i=this.renderer.getCamera();this.isMouseDown=!1,typeof this.movingTimeout=="number"&&(clearTimeout(this.movingTimeout),this.movingTimeout=null);var o=Ao(n,this.container),s=o.x,l=o.y,u=i.getState(),d=i.getPreviousState()||{x:0,y:0};this.isMoving?i.animate({x:u.x+this.settings.inertiaRatio*(u.x-d.x),y:u.y+this.settings.inertiaRatio*(u.y-d.y)},{duration:this.settings.inertiaDuration,easing:"quadraticOut"}):(this.lastMouseX!==s||this.lastMouseY!==l)&&i.setState({x:u.x,y:u.y}),this.isMoving=!1,setTimeout(function(){var p=a.draggedEvents>0;a.draggedEvents=0,p&&a.renderer.getSetting("hideEdgesOnMove")&&a.renderer.refresh()},0),this.emit("mouseup",Xs(n,this.container))}}},{key:"handleMove",value:function(n){var a=this;if(this.enabled){var i=Xs(n,this.container);if(this.emit("mousemovebody",i),(n.target===this.container||n.composedPath()[0]===this.container)&&this.emit("mousemove",i),!i.sigmaDefaultPrevented&&this.isMouseDown){this.isMoving=!0,this.draggedEvents++,typeof this.movingTimeout=="number"&&clearTimeout(this.movingTimeout),this.movingTimeout=window.setTimeout(function(){a.movingTimeout=null,a.isMoving=!1},this.settings.dragTimeout);var o=this.renderer.getCamera(),s=Ao(n,this.container),l=s.x,u=s.y,d=this.renderer.viewportToFramedGraph({x:this.lastMouseX,y:this.lastMouseY}),p=this.renderer.viewportToFramedGraph({x:l,y:u}),f=d.x-p.x,m=d.y-p.y,b=o.getState(),y=b.x+f,S=b.y+m;o.setState({x:y,y:S}),this.lastMouseX=l,this.lastMouseY=u,n.preventDefault(),n.stopPropagation()}}}},{key:"handleLeave",value:function(n){this.emit("mouseleave",Xs(n,this.container))}},{key:"handleEnter",value:function(n){this.emit("mouseenter",Xs(n,this.container))}},{key:"handleWheel",value:function(n){var a=this,i=this.renderer.getCamera();if(!(!this.enabled||!i.enabledZooming)){var o=dee(n);if(o){var s=ACe(n,this.container);if(this.emit("wheel",s),s.sigmaDefaultPrevented){n.preventDefault(),n.stopPropagation();return}var l=i.getState().ratio,u=o>0?1/this.settings.zoomingRatio:this.settings.zoomingRatio,d=i.getBoundedRatio(l*u),p=o>0?1:-1,f=Date.now();l!==d&&(n.preventDefault(),n.stopPropagation(),!(this.currentWheelDirection===p&&this.lastWheelTriggerTime&&f-this.lastWheelTriggerTime<this.settings.zoomDuration/5)&&(i.animate(this.renderer.getViewportZoomedState(Ao(n,this.container),d),{easing:"quadraticOut",duration:this.settings.zoomDuration},function(){a.currentWheelDirection=0}),this.currentWheelDirection=p,this.lastWheelTriggerTime=f))}}}},{key:"setSettings",value:function(n){this.settings=n}}])})(hee),NCe=["dragTimeout","inertiaDuration","inertiaRatio","doubleClickTimeout","doubleClickZoomingRatio","doubleClickZoomingDuration","tapMoveTolerance"],LCe=NCe.reduce(function(e,t){return Ct(Ct({},e),{},Je({},t,S5[t]))},{}),MCe=(function(e){function t(r,n){var a;return la(this,t),a=gi(this,t,[r,n]),Je(a,"enabled",!0),Je(a,"isMoving",!1),Je(a,"hasMoved",!1),Je(a,"touchMode",0),Je(a,"startTouchesPositions",[]),Je(a,"lastTouches",[]),Je(a,"lastTap",null),Je(a,"settings",LCe),a.handleStart=a.handleStart.bind(a),a.handleLeave=a.handleLeave.bind(a),a.handleMove=a.handleMove.bind(a),r.addEventListener("touchstart",a.handleStart,{capture:!1}),r.addEventListener("touchcancel",a.handleLeave,{capture:!1}),document.addEventListener("touchend",a.handleLeave,{capture:!1,passive:!1}),document.addEventListener("touchmove",a.handleMove,{capture:!1,passive:!1}),a}return bi(t,e),ua(t,[{key:"kill",value:function(){var n=this.container;n.removeEventListener("touchstart",this.handleStart),n.removeEventListener("touchcancel",this.handleLeave),document.removeEventListener("touchend",this.handleLeave),document.removeEventListener("touchmove",this.handleMove)}},{key:"getDimensions",value:function(){return{width:this.container.offsetWidth,height:this.container.offsetHeight}}},{key:"handleStart",value:function(n){var a=this;if(this.enabled){n.preventDefault();var i=A1(n.touches);if(this.touchMode=i.length,this.startCameraState=this.renderer.getCamera().getState(),this.startTouchesPositions=i.map(function(m){return Ao(m,a.container)}),this.touchMode===2){var o=qh(this.startTouchesPositions,2),s=o[0],l=s.x,u=s.y,d=o[1],p=d.x,f=d.y;this.startTouchesAngle=Math.atan2(f-u,p-l),this.startTouchesDistance=Math.sqrt(Math.pow(p-l,2)+Math.pow(f-u,2))}this.emit("touchdown",If(n,this.lastTouches,this.container)),this.lastTouches=i,this.lastTouchesPositions=this.startTouchesPositions}}},{key:"handleLeave",value:function(n){if(!(!this.enabled||!this.startTouchesPositions.length)){switch(n.cancelable&&n.preventDefault(),this.movingTimeout&&(this.isMoving=!1,clearTimeout(this.movingTimeout)),this.touchMode){case 2:if(n.touches.length===1){this.handleStart(n),n.preventDefault();break}case 1:if(this.isMoving){var a=this.renderer.getCamera(),i=a.getState(),o=a.getPreviousState()||{x:0,y:0};a.animate({x:i.x+this.settings.inertiaRatio*(i.x-o.x),y:i.y+this.settings.inertiaRatio*(i.y-o.y)},{duration:this.settings.inertiaDuration,easing:"quadraticOut"})}this.hasMoved=!1,this.isMoving=!1,this.touchMode=0;break}if(this.emit("touchup",If(n,this.lastTouches,this.container)),!n.touches.length){var s=Ao(this.lastTouches[0],this.container),l=this.startTouchesPositions[0],u=Math.pow(s.x-l.x,2)+Math.pow(s.y-l.y,2);if(!n.touches.length&&u<Math.pow(this.settings.tapMoveTolerance,2))if(this.lastTap&&Date.now()-this.lastTap.time<this.settings.doubleClickTimeout){var d=If(n,this.lastTouches,this.container);if(this.emit("doubletap",d),this.lastTap=null,!d.sigmaDefaultPrevented){var p=this.renderer.getCamera(),f=p.getBoundedRatio(p.getState().ratio/this.settings.doubleClickZoomingRatio);p.animate(this.renderer.getViewportZoomedState(s,f),{easing:"quadraticInOut",duration:this.settings.doubleClickZoomingDuration})}}else{var m=If(n,this.lastTouches,this.container);this.emit("tap",m),this.lastTap={time:Date.now(),position:m.touches[0]||m.previousTouches[0]}}}this.lastTouches=A1(n.touches),this.startTouchesPositions=[]}}},{key:"handleMove",value:function(n){var a=this;if(!(!this.enabled||!this.startTouchesPositions.length)){n.preventDefault();var i=A1(n.touches),o=i.map(function(K){return Ao(K,a.container)}),s=this.lastTouches;this.lastTouches=i,this.lastTouchesPositions=o;var l=If(n,s,this.container);if(this.emit("touchmove",l),!l.sigmaDefaultPrevented&&(this.hasMoved||(this.hasMoved=o.some(function(K,G){var P=a.startTouchesPositions[G];return P&&(K.x!==P.x||K.y!==P.y)})),!!this.hasMoved)){this.isMoving=!0,this.movingTimeout&&clearTimeout(this.movingTimeout),this.movingTimeout=window.setTimeout(function(){a.isMoving=!1},this.settings.dragTimeout);var u=this.renderer.getCamera(),d=this.startCameraState,p=this.renderer.getSetting("stagePadding");switch(this.touchMode){case 1:{var f=this.renderer.viewportToFramedGraph((this.startTouchesPositions||[])[0]),m=f.x,b=f.y,y=this.renderer.viewportToFramedGraph(o[0]),S=y.x,w=y.y;u.setState({x:d.x+m-S,y:d.y+b-w});break}case 2:{var E={x:.5,y:.5,angle:0,ratio:1},k=o[0],_=k.x,R=k.y,C=o[1],F=C.x,L=C.y,I=Math.atan2(L-R,F-_)-this.startTouchesAngle,M=Math.hypot(L-R,F-_)/this.startTouchesDistance,j=u.getBoundedRatio(d.ratio/M);E.ratio=j,E.angle=d.angle+I;var Y=this.getDimensions(),W=this.renderer.viewportToFramedGraph((this.startTouchesPositions||[])[0],{cameraState:d}),$=Math.min(Y.width,Y.height)-2*p,X=$/Y.width,z=$/Y.height,H=j/$,B=_-$/2/X,ee=R-$/2/z,J=[B*Math.cos(-E.angle)-ee*Math.sin(-E.angle),ee*Math.cos(-E.angle)+B*Math.sin(-E.angle)];B=J[0],ee=J[1],E.x=W.x-B*H,E.y=W.y+ee*H,u.setState(E);break}}}}}},{key:"setSettings",value:function(n){this.settings=n}}])})(hee);function ICe(e){if(Array.isArray(e))return hR(e)}function OCe(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function BCe(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function R7(e){return ICe(e)||OCe(e)||WQ(e)||BCe()}function PCe(e,t){if(e==null)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)!==-1)continue;r[n]=e[n]}return r}function dw(e,t){if(e==null)return{};var r,n,a=PCe(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)r=i[n],t.indexOf(r)===-1&&{}.propertyIsEnumerable.call(e,r)&&(a[r]=e[r])}return a}var D7=(function(){function e(t,r){la(this,e),this.key=t,this.size=r}return ua(e,null,[{key:"compare",value:function(r,n){return r.size>n.size?-1:r.size<n.size||r.key>n.key?1:-1}}])})(),N7=(function(){function e(){la(this,e),Je(this,"width",0),Je(this,"height",0),Je(this,"cellSize",0),Je(this,"columns",0),Je(this,"rows",0),Je(this,"cells",{})}return ua(e,[{key:"resizeAndClear",value:function(r,n){this.width=r.width,this.height=r.height,this.cellSize=n,this.columns=Math.ceil(r.width/n),this.rows=Math.ceil(r.height/n),this.cells={}}},{key:"getIndex",value:function(r){var n=Math.floor(r.x/this.cellSize),a=Math.floor(r.y/this.cellSize);return a*this.columns+n}},{key:"add",value:function(r,n,a){var i=new D7(r,n),o=this.getIndex(a),s=this.cells[o];s||(s=[],this.cells[o]=s),s.push(i)}},{key:"organize",value:function(){for(var r in this.cells){var n=this.cells[r];n.sort(D7.compare)}}},{key:"getLabelsToDisplay",value:function(r,n){var a=this.cellSize*this.cellSize,i=a/r/r,o=i*n/a,s=Math.ceil(o),l=[];for(var u in this.cells)for(var d=this.cells[u],p=0;p<Math.min(s,d.length);p++)l.push(d[p].key);return l}}])})();function zCe(e){var t=e.graph,r=e.hoveredNode,n=e.highlightedNodes,a=e.displayedNodeLabels,i=[];return t.forEachEdge(function(o,s,l,u){(l===r||u===r||n.has(l)||n.has(u)||a.has(l)&&a.has(u))&&i.push(o)}),i}var L7=150,M7=50,al=Object.prototype.hasOwnProperty;function HCe(e,t,r){if(!al.call(r,"x")||!al.call(r,"y"))throw new Error('Sigma: could not find a valid position (x, y) for node "'.concat(t,'". All your nodes must have a number "x" and "y". Maybe your forgot to apply a layout or your "nodeReducer" is not returning the correct data?'));return r.color||(r.color=e.defaultNodeColor),!r.label&&r.label!==""&&(r.label=null),r.label!==void 0&&r.label!==null?r.label=""+r.label:r.label=null,r.size||(r.size=2),al.call(r,"hidden")||(r.hidden=!1),al.call(r,"highlighted")||(r.highlighted=!1),al.call(r,"forceLabel")||(r.forceLabel=!1),(!r.type||r.type==="")&&(r.type=e.defaultNodeType),r.zIndex||(r.zIndex=0),r}function UCe(e,t,r){return r.color||(r.color=e.defaultEdgeColor),r.label||(r.label=""),r.size||(r.size=.5),al.call(r,"hidden")||(r.hidden=!1),al.call(r,"forceLabel")||(r.forceLabel=!1),(!r.type||r.type==="")&&(r.type=e.defaultEdgeType),r.zIndex||(r.zIndex=0),r}var $Ce=(function(e){function t(r,n){var a,i=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{};if(la(this,t),a=gi(this,t),Je(a,"elements",{}),Je(a,"canvasContexts",{}),Je(a,"webGLContexts",{}),Je(a,"pickingLayers",new Set),Je(a,"textures",{}),Je(a,"frameBuffers",{}),Je(a,"activeListeners",{}),Je(a,"labelGrid",new N7),Je(a,"nodeDataCache",{}),Je(a,"edgeDataCache",{}),Je(a,"nodeProgramIndex",{}),Je(a,"edgeProgramIndex",{}),Je(a,"nodesWithForcedLabels",new Set),Je(a,"edgesWithForcedLabels",new Set),Je(a,"nodeExtent",{x:[0,1],y:[0,1]}),Je(a,"nodeZExtent",[1/0,-1/0]),Je(a,"edgeZExtent",[1/0,-1/0]),Je(a,"matrix",xo()),Je(a,"invMatrix",xo()),Je(a,"correctionRatio",1),Je(a,"customBBox",null),Je(a,"normalizationFunction",A7({x:[0,1],y:[0,1]})),Je(a,"graphToViewportRatio",1),Je(a,"itemIDsIndex",{}),Je(a,"nodeIndices",{}),Je(a,"edgeIndices",{}),Je(a,"width",0),Je(a,"height",0),Je(a,"pixelRatio",C7()),Je(a,"pickingDownSizingRatio",2*a.pixelRatio),Je(a,"displayedNodeLabels",new Set),Je(a,"displayedEdgeLabels",new Set),Je(a,"highlightedNodes",new Set),Je(a,"hoveredNode",null),Je(a,"hoveredEdge",null),Je(a,"renderFrame",null),Je(a,"renderHighlightedNodesFrame",null),Je(a,"needToProcess",!1),Je(a,"checkEdgesEventsFrame",null),Je(a,"nodePrograms",{}),Je(a,"nodeHoverPrograms",{}),Je(a,"edgePrograms",{}),a.settings=TCe(i),cw(a.settings),xCe(r),!(n instanceof HTMLElement))throw new Error("Sigma: container should be an html element.");a.graph=r,a.container=n,a.createWebGLContext("edges",{picking:i.enableEdgeEvents}),a.createCanvasContext("edgeLabels"),a.createWebGLContext("nodes",{picking:!0}),a.createCanvasContext("labels"),a.createCanvasContext("hovers"),a.createWebGLContext("hoverNodes"),a.createCanvasContext("mouse",{style:{touchAction:"none",userSelect:"none"}}),a.resize();for(var o in a.settings.nodeProgramClasses)a.registerNodeProgram(o,a.settings.nodeProgramClasses[o],a.settings.nodeHoverProgramClasses[o]);for(var s in a.settings.edgeProgramClasses)a.registerEdgeProgram(s,a.settings.edgeProgramClasses[s]);return a.camera=new _7,a.bindCameraHandlers(),a.mouseCaptor=new DCe(a.elements.mouse,a),a.mouseCaptor.setSettings(a.settings),a.touchCaptor=new MCe(a.elements.mouse,a),a.touchCaptor.setSettings(a.settings),a.bindEventHandlers(),a.bindGraphHandlers(),a.handleSettingsUpdate(),a.refresh(),a}return bi(t,e),ua(t,[{key:"registerNodeProgram",value:function(n,a,i){return this.nodePrograms[n]&&this.nodePrograms[n].kill(),this.nodeHoverPrograms[n]&&this.nodeHoverPrograms[n].kill(),this.nodePrograms[n]=new a(this.webGLContexts.nodes,this.frameBuffers.nodes,this),this.nodeHoverPrograms[n]=new(i||a)(this.webGLContexts.hoverNodes,null,this),this}},{key:"registerEdgeProgram",value:function(n,a){return this.edgePrograms[n]&&this.edgePrograms[n].kill(),this.edgePrograms[n]=new a(this.webGLContexts.edges,this.frameBuffers.edges,this),this}},{key:"unregisterNodeProgram",value:function(n){if(this.nodePrograms[n]){var a=this.nodePrograms,i=a[n],o=dw(a,[n].map(ym));i.kill(),this.nodePrograms=o}if(this.nodeHoverPrograms[n]){var s=this.nodeHoverPrograms,l=s[n],u=dw(s,[n].map(ym));l.kill(),this.nodePrograms=u}return this}},{key:"unregisterEdgeProgram",value:function(n){if(this.edgePrograms[n]){var a=this.edgePrograms,i=a[n],o=dw(a,[n].map(ym));i.kill(),this.edgePrograms=o}return this}},{key:"resetWebGLTexture",value:function(n){var a=this.webGLContexts[n],i=this.frameBuffers[n],o=this.textures[n];o&&a.deleteTexture(o);var s=a.createTexture();return a.bindFramebuffer(a.FRAMEBUFFER,i),a.bindTexture(a.TEXTURE_2D,s),a.texImage2D(a.TEXTURE_2D,0,a.RGBA,this.width,this.height,0,a.RGBA,a.UNSIGNED_BYTE,null),a.framebufferTexture2D(a.FRAMEBUFFER,a.COLOR_ATTACHMENT0,a.TEXTURE_2D,s,0),this.textures[n]=s,this}},{key:"bindCameraHandlers",value:function(){var n=this;return this.activeListeners.camera=function(){n.scheduleRender()},this.camera.on("updated",this.activeListeners.camera),this}},{key:"unbindCameraHandlers",value:function(){return this.camera.removeListener("updated",this.activeListeners.camera),this}},{key:"getNodeAtPosition",value:function(n){var a=n.x,i=n.y,o=p7(this.webGLContexts.nodes,this.frameBuffers.nodes,a,i,this.pixelRatio,this.pickingDownSizingRatio),s=h7.apply(void 0,R7(o)),l=this.itemIDsIndex[s];return l&&l.type==="node"?l.id:null}},{key:"bindEventHandlers",value:function(){var n=this;this.activeListeners.handleResize=function(){n.scheduleRefresh()},window.addEventListener("resize",this.activeListeners.handleResize),this.activeListeners.handleMove=function(i){var o=Mf(i),s={event:o,preventSigmaDefault:function(){o.preventSigmaDefault()}},l=n.getNodeAtPosition(o);if(l&&n.hoveredNode!==l&&!n.nodeDataCache[l].hidden){n.hoveredNode&&n.emit("leaveNode",Ct(Ct({},s),{},{node:n.hoveredNode})),n.hoveredNode=l,n.emit("enterNode",Ct(Ct({},s),{},{node:l})),n.scheduleHighlightedNodesRender();return}if(n.hoveredNode&&n.getNodeAtPosition(o)!==n.hoveredNode){var u=n.hoveredNode;n.hoveredNode=null,n.emit("leaveNode",Ct(Ct({},s),{},{node:u})),n.scheduleHighlightedNodesRender();return}if(n.settings.enableEdgeEvents){var d=n.hoveredNode?null:n.getEdgeAtPoint(s.event.x,s.event.y);d!==n.hoveredEdge&&(n.hoveredEdge&&n.emit("leaveEdge",Ct(Ct({},s),{},{edge:n.hoveredEdge})),d&&n.emit("enterEdge",Ct(Ct({},s),{},{edge:d})),n.hoveredEdge=d)}},this.activeListeners.handleMoveBody=function(i){var o=Mf(i);n.emit("moveBody",{event:o,preventSigmaDefault:function(){o.preventSigmaDefault()}})},this.activeListeners.handleLeave=function(i){var o=Mf(i),s={event:o,preventSigmaDefault:function(){o.preventSigmaDefault()}};n.hoveredNode&&(n.emit("leaveNode",Ct(Ct({},s),{},{node:n.hoveredNode})),n.scheduleHighlightedNodesRender()),n.settings.enableEdgeEvents&&n.hoveredEdge&&(n.emit("leaveEdge",Ct(Ct({},s),{},{edge:n.hoveredEdge})),n.scheduleHighlightedNodesRender()),n.emit("leaveStage",Ct({},s))},this.activeListeners.handleEnter=function(i){var o=Mf(i),s={event:o,preventSigmaDefault:function(){o.preventSigmaDefault()}};n.emit("enterStage",Ct({},s))};var a=function(o){return function(s){var l=Mf(s),u={event:l,preventSigmaDefault:function(){l.preventSigmaDefault()}},d=n.getNodeAtPosition(l);if(d)return n.emit("".concat(o,"Node"),Ct(Ct({},u),{},{node:d}));if(n.settings.enableEdgeEvents){var p=n.getEdgeAtPoint(l.x,l.y);if(p)return n.emit("".concat(o,"Edge"),Ct(Ct({},u),{},{edge:p}))}return n.emit("".concat(o,"Stage"),u)}};return this.activeListeners.handleClick=a("click"),this.activeListeners.handleRightClick=a("rightClick"),this.activeListeners.handleDoubleClick=a("doubleClick"),this.activeListeners.handleWheel=a("wheel"),this.activeListeners.handleDown=a("down"),this.activeListeners.handleUp=a("up"),this.mouseCaptor.on("mousemove",this.activeListeners.handleMove),this.mouseCaptor.on("mousemovebody",this.activeListeners.handleMoveBody),this.mouseCaptor.on("click",this.activeListeners.handleClick),this.mouseCaptor.on("rightClick",this.activeListeners.handleRightClick),this.mouseCaptor.on("doubleClick",this.activeListeners.handleDoubleClick),this.mouseCaptor.on("wheel",this.activeListeners.handleWheel),this.mouseCaptor.on("mousedown",this.activeListeners.handleDown),this.mouseCaptor.on("mouseup",this.activeListeners.handleUp),this.mouseCaptor.on("mouseleave",this.activeListeners.handleLeave),this.mouseCaptor.on("mouseenter",this.activeListeners.handleEnter),this.touchCaptor.on("touchdown",this.activeListeners.handleDown),this.touchCaptor.on("touchdown",this.activeListeners.handleMove),this.touchCaptor.on("touchup",this.activeListeners.handleUp),this.touchCaptor.on("touchmove",this.activeListeners.handleMove),this.touchCaptor.on("tap",this.activeListeners.handleClick),this.touchCaptor.on("doubletap",this.activeListeners.handleDoubleClick),this.touchCaptor.on("touchmove",this.activeListeners.handleMoveBody),this}},{key:"bindGraphHandlers",value:function(){var n=this,a=this.graph,i=new Set(["x","y","zIndex","type"]);return this.activeListeners.eachNodeAttributesUpdatedGraphUpdate=function(o){var s,l=(s=o.hints)===null||s===void 0?void 0:s.attributes;n.graph.forEachNode(function(d){return n.updateNode(d)});var u=!l||l.some(function(d){return i.has(d)});n.refresh({partialGraph:{nodes:a.nodes()},skipIndexation:!u,schedule:!0})},this.activeListeners.eachEdgeAttributesUpdatedGraphUpdate=function(o){var s,l=(s=o.hints)===null||s===void 0?void 0:s.attributes;n.graph.forEachEdge(function(d){return n.updateEdge(d)});var u=l&&["zIndex","type"].some(function(d){return l==null?void 0:l.includes(d)});n.refresh({partialGraph:{edges:a.edges()},skipIndexation:!u,schedule:!0})},this.activeListeners.addNodeGraphUpdate=function(o){var s=o.key;n.addNode(s),n.refresh({partialGraph:{nodes:[s]},skipIndexation:!1,schedule:!0})},this.activeListeners.updateNodeGraphUpdate=function(o){var s=o.key;n.refresh({partialGraph:{nodes:[s]},skipIndexation:!1,schedule:!0})},this.activeListeners.dropNodeGraphUpdate=function(o){var s=o.key;n.removeNode(s),n.refresh({schedule:!0})},this.activeListeners.addEdgeGraphUpdate=function(o){var s=o.key;n.addEdge(s),n.refresh({partialGraph:{edges:[s]},schedule:!0})},this.activeListeners.updateEdgeGraphUpdate=function(o){var s=o.key;n.refresh({partialGraph:{edges:[s]},skipIndexation:!1,schedule:!0})},this.activeListeners.dropEdgeGraphUpdate=function(o){var s=o.key;n.removeEdge(s),n.refresh({schedule:!0})},this.activeListeners.clearEdgesGraphUpdate=function(){n.clearEdgeState(),n.clearEdgeIndices(),n.refresh({schedule:!0})},this.activeListeners.clearGraphUpdate=function(){n.clearEdgeState(),n.clearNodeState(),n.clearEdgeIndices(),n.clearNodeIndices(),n.refresh({schedule:!0})},a.on("nodeAdded",this.activeListeners.addNodeGraphUpdate),a.on("nodeDropped",this.activeListeners.dropNodeGraphUpdate),a.on("nodeAttributesUpdated",this.activeListeners.updateNodeGraphUpdate),a.on("eachNodeAttributesUpdated",this.activeListeners.eachNodeAttributesUpdatedGraphUpdate),a.on("edgeAdded",this.activeListeners.addEdgeGraphUpdate),a.on("edgeDropped",this.activeListeners.dropEdgeGraphUpdate),a.on("edgeAttributesUpdated",this.activeListeners.updateEdgeGraphUpdate),a.on("eachEdgeAttributesUpdated",this.activeListeners.eachEdgeAttributesUpdatedGraphUpdate),a.on("edgesCleared",this.activeListeners.clearEdgesGraphUpdate),a.on("cleared",this.activeListeners.clearGraphUpdate),this}},{key:"unbindGraphHandlers",value:function(){var n=this.graph;n.removeListener("nodeAdded",this.activeListeners.addNodeGraphUpdate),n.removeListener("nodeDropped",this.activeListeners.dropNodeGraphUpdate),n.removeListener("nodeAttributesUpdated",this.activeListeners.updateNodeGraphUpdate),n.removeListener("eachNodeAttributesUpdated",this.activeListeners.eachNodeAttributesUpdatedGraphUpdate),n.removeListener("edgeAdded",this.activeListeners.addEdgeGraphUpdate),n.removeListener("edgeDropped",this.activeListeners.dropEdgeGraphUpdate),n.removeListener("edgeAttributesUpdated",this.activeListeners.updateEdgeGraphUpdate),n.removeListener("eachEdgeAttributesUpdated",this.activeListeners.eachEdgeAttributesUpdatedGraphUpdate),n.removeListener("edgesCleared",this.activeListeners.clearEdgesGraphUpdate),n.removeListener("cleared",this.activeListeners.clearGraphUpdate)}},{key:"getEdgeAtPoint",value:function(n,a){var i=p7(this.webGLContexts.edges,this.frameBuffers.edges,n,a,this.pixelRatio,this.pickingDownSizingRatio),o=h7.apply(void 0,R7(i)),s=this.itemIDsIndex[o];return s&&s.type==="edge"?s.id:null}},{key:"process",value:function(){var n=this;this.emit("beforeProcess");var a=this.graph,i=this.settings,o=this.getDimensions();if(this.nodeExtent=ECe(this.graph),!this.settings.autoRescale){var s=o.width,l=o.height,u=this.nodeExtent,d=u.x,p=u.y;this.nodeExtent={x:[(d[0]+d[1])/2-s/2,(d[0]+d[1])/2+s/2],y:[(p[0]+p[1])/2-l/2,(p[0]+p[1])/2+l/2]}}this.normalizationFunction=A7(this.customBBox||this.nodeExtent);var f=new _7,m=Lf(f.getState(),o,this.getGraphDimensions(),this.getStagePadding());this.labelGrid.resizeAndClear(o,i.labelGridCellSize);for(var b={},y={},S={},w={},E=1,k=a.nodes(),_=0,R=k.length;_<R;_++){var C=k[_],F=this.nodeDataCache[C],L=a.getNodeAttributes(C);F.x=L.x,F.y=L.y,this.normalizationFunction.applyTo(F),typeof F.label=="string"&&!F.hidden&&this.labelGrid.add(C,F.size,this.framedGraphToViewport(F,{matrix:m})),b[F.type]=(b[F.type]||0)+1}this.labelGrid.organize();for(var I in this.nodePrograms){if(!al.call(this.nodePrograms,I))throw new Error('Sigma: could not find a suitable program for node type "'.concat(I,'"!'));this.nodePrograms[I].reallocate(b[I]||0),b[I]=0}this.settings.zIndex&&this.nodeZExtent[0]!==this.nodeZExtent[1]&&(k=T7(this.nodeZExtent,function(oe){return n.nodeDataCache[oe].zIndex},k));for(var M=0,j=k.length;M<j;M++){var Y=k[M];y[Y]=E,w[y[Y]]={type:"node",id:Y},E++;var W=this.nodeDataCache[Y];this.addNodeToProgram(Y,y[Y],b[W.type]++)}for(var $={},X=a.edges(),z=0,H=X.length;z<H;z++){var B=X[z],ee=this.edgeDataCache[B];$[ee.type]=($[ee.type]||0)+1}this.settings.zIndex&&this.edgeZExtent[0]!==this.edgeZExtent[1]&&(X=T7(this.edgeZExtent,function(oe){return n.edgeDataCache[oe].zIndex},X));for(var J in this.edgePrograms){if(!al.call(this.edgePrograms,J))throw new Error('Sigma: could not find a suitable program for edge type "'.concat(J,'"!'));this.edgePrograms[J].reallocate($[J]||0),$[J]=0}for(var K=0,G=X.length;K<G;K++){var P=X[K];S[P]=E,w[S[P]]={type:"edge",id:P},E++;var ne=this.edgeDataCache[P];this.addEdgeToProgram(P,S[P],$[ne.type]++)}return this.itemIDsIndex=w,this.nodeIndices=y,this.edgeIndices=S,this.emit("afterProcess"),this}},{key:"handleSettingsUpdate",value:function(n){var a=this,i=this.settings;if(this.camera.minRatio=i.minCameraRatio,this.camera.maxRatio=i.maxCameraRatio,this.camera.enabledZooming=i.enableCameraZooming,this.camera.enabledPanning=i.enableCameraPanning,this.camera.enabledRotation=i.enableCameraRotation,i.cameraPanBoundaries?this.camera.clean=function(d){return a.cleanCameraState(d,i.cameraPanBoundaries&&mR(i.cameraPanBoundaries)==="object"?i.cameraPanBoundaries:{})}:this.camera.clean=null,this.camera.setState(this.camera.validateState(this.camera.getState())),n){if(n.edgeProgramClasses!==i.edgeProgramClasses){for(var o in i.edgeProgramClasses)i.edgeProgramClasses[o]!==n.edgeProgramClasses[o]&&this.registerEdgeProgram(o,i.edgeProgramClasses[o]);for(var s in n.edgeProgramClasses)i.edgeProgramClasses[s]||this.unregisterEdgeProgram(s)}if(n.nodeProgramClasses!==i.nodeProgramClasses||n.nodeHoverProgramClasses!==i.nodeHoverProgramClasses){for(var l in i.nodeProgramClasses)(i.nodeProgramClasses[l]!==n.nodeProgramClasses[l]||i.nodeHoverProgramClasses[l]!==n.nodeHoverProgramClasses[l])&&this.registerNodeProgram(l,i.nodeProgramClasses[l],i.nodeHoverProgramClasses[l]);for(var u in n.nodeProgramClasses)i.nodeProgramClasses[u]||this.unregisterNodeProgram(u)}}return this.mouseCaptor.setSettings(this.settings),this.touchCaptor.setSettings(this.settings),this}},{key:"cleanCameraState",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},i=a.tolerance,o=i===void 0?0:i,s=a.boundaries,l=Ct({},n),u=s||this.nodeExtent,d=qh(u.x,2),p=d[0],f=d[1],m=qh(u.y,2),b=m[0],y=m[1],S=[this.graphToViewport({x:p,y:b},{cameraState:n}),this.graphToViewport({x:f,y:b},{cameraState:n}),this.graphToViewport({x:p,y},{cameraState:n}),this.graphToViewport({x:f,y},{cameraState:n})],w=1/0,E=-1/0,k=1/0,_=-1/0;S.forEach(function($){var X=$.x,z=$.y;w=Math.min(w,X),E=Math.max(E,X),k=Math.min(k,z),_=Math.max(_,z)});var R=E-w,C=_-k,F=this.getDimensions(),L=F.width,I=F.height,M=0,j=0;if(R>=L?E<L-o?M=E-(L-o):w>o&&(M=w-o):E>L+o?M=E-(L+o):w<-o&&(M=w+o),C>=I?_<I-o?j=_-(I-o):k>o&&(j=k-o):_>I+o?j=_-(I+o):k<-o&&(j=k+o),M||j){var Y=this.viewportToFramedGraph({x:0,y:0},{cameraState:n}),W=this.viewportToFramedGraph({x:M,y:j},{cameraState:n});M=W.x-Y.x,j=W.y-Y.y,l.x+=M,l.y+=j}return l}},{key:"renderLabels",value:function(){if(!this.settings.renderLabels)return this;var n=this.camera.getState(),a=this.labelGrid.getLabelsToDisplay(n.ratio,this.settings.labelDensity);F7(a,this.nodesWithForcedLabels),this.displayedNodeLabels=new Set;for(var i=this.canvasContexts.labels,o=0,s=a.length;o<s;o++){var l=a[o],u=this.nodeDataCache[l];if(!this.displayedNodeLabels.has(l)&&!u.hidden){var d=this.framedGraphToViewport(u),p=d.x,f=d.y,m=this.scaleSize(u.size);if(!(!u.forceLabel&&m<this.settings.labelRenderedSizeThreshold)&&!(p<-L7||p>this.width+L7||f<-M7||f>this.height+M7)){this.displayedNodeLabels.add(l);var b=this.settings.defaultDrawNodeLabel,y=this.nodePrograms[u.type],S=(y==null?void 0:y.drawLabel)||b;S(i,Ct(Ct({key:l},u),{},{size:m,x:p,y:f}),this.settings)}}}return this}},{key:"renderEdgeLabels",value:function(){if(!this.settings.renderEdgeLabels)return this;var n=this.canvasContexts.edgeLabels;n.clearRect(0,0,this.width,this.height);var a=zCe({graph:this.graph,hoveredNode:this.hoveredNode,displayedNodeLabels:this.displayedNodeLabels,highlightedNodes:this.highlightedNodes});F7(a,this.edgesWithForcedLabels);for(var i=new Set,o=0,s=a.length;o<s;o++){var l=a[o],u=this.graph.extremities(l),d=this.nodeDataCache[u[0]],p=this.nodeDataCache[u[1]],f=this.edgeDataCache[l];if(!i.has(l)&&!(f.hidden||d.hidden||p.hidden)){var m=this.settings.defaultDrawEdgeLabel,b=this.edgePrograms[f.type],y=(b==null?void 0:b.drawLabel)||m;y(n,Ct(Ct({key:l},f),{},{size:this.scaleSize(f.size)}),Ct(Ct(Ct({key:u[0]},d),this.framedGraphToViewport(d)),{},{size:this.scaleSize(d.size)}),Ct(Ct(Ct({key:u[1]},p),this.framedGraphToViewport(p)),{},{size:this.scaleSize(p.size)}),this.settings),i.add(l)}}return this.displayedEdgeLabels=i,this}},{key:"renderHighlightedNodes",value:function(){var n=this,a=this.canvasContexts.hovers;a.clearRect(0,0,this.width,this.height);var i=function(m){var b=n.nodeDataCache[m],y=n.framedGraphToViewport(b),S=y.x,w=y.y,E=n.scaleSize(b.size),k=n.settings.defaultDrawNodeHover,_=n.nodePrograms[b.type],R=(_==null?void 0:_.drawHover)||k;R(a,Ct(Ct({key:m},b),{},{size:E,x:S,y:w}),n.settings)},o=[];this.hoveredNode&&!this.nodeDataCache[this.hoveredNode].hidden&&o.push(this.hoveredNode),this.highlightedNodes.forEach(function(f){f!==n.hoveredNode&&o.push(f)}),o.forEach(function(f){return i(f)});var s={};o.forEach(function(f){var m=n.nodeDataCache[f].type;s[m]=(s[m]||0)+1});for(var l in this.nodeHoverPrograms)this.nodeHoverPrograms[l].reallocate(s[l]||0),s[l]=0;o.forEach(function(f){var m=n.nodeDataCache[f];n.nodeHoverPrograms[m.type].process(0,s[m.type]++,m)}),this.webGLContexts.hoverNodes.clear(this.webGLContexts.hoverNodes.COLOR_BUFFER_BIT);var u=this.getRenderParams();for(var d in this.nodeHoverPrograms){var p=this.nodeHoverPrograms[d];p.render(u)}}},{key:"scheduleHighlightedNodesRender",value:function(){var n=this;this.renderHighlightedNodesFrame||this.renderFrame||(this.renderHighlightedNodesFrame=requestAnimationFrame(function(){n.renderHighlightedNodesFrame=null,n.renderHighlightedNodes(),n.renderEdgeLabels()}))}},{key:"render",value:function(){var n=this;this.emit("beforeRender");var a=function(){return n.emit("afterRender"),n};if(this.renderFrame&&(cancelAnimationFrame(this.renderFrame),this.renderFrame=null),this.resize(),this.needToProcess&&this.process(),this.needToProcess=!1,this.clear(),this.pickingLayers.forEach(function(S){return n.resetWebGLTexture(S)}),!this.graph.order)return a();var i=this.mouseCaptor,o=this.camera.isAnimated()||i.isMoving||i.draggedEvents||i.currentWheelDirection,s=this.camera.getState(),l=this.getDimensions(),u=this.getGraphDimensions(),d=this.getStagePadding();this.matrix=Lf(s,l,u,d),this.invMatrix=Lf(s,l,u,d,!0),this.correctionRatio=SCe(this.matrix,s,l),this.graphToViewportRatio=this.getGraphToViewportRatio();var p=this.getRenderParams();for(var f in this.nodePrograms){var m=this.nodePrograms[f];m.render(p)}if(!this.settings.hideEdgesOnMove||!o)for(var b in this.edgePrograms){var y=this.edgePrograms[b];y.render(p)}return this.settings.hideLabelsOnMove&&o||(this.renderLabels(),this.renderEdgeLabels(),this.renderHighlightedNodes()),a()}},{key:"addNode",value:function(n){var a=Object.assign({},this.graph.getNodeAttributes(n));this.settings.nodeReducer&&(a=this.settings.nodeReducer(n,a));var i=HCe(this.settings,n,a);this.nodeDataCache[n]=i,this.nodesWithForcedLabels.delete(n),i.forceLabel&&!i.hidden&&this.nodesWithForcedLabels.add(n),this.highlightedNodes.delete(n),i.highlighted&&!i.hidden&&this.highlightedNodes.add(n),this.settings.zIndex&&(i.zIndex<this.nodeZExtent[0]&&(this.nodeZExtent[0]=i.zIndex),i.zIndex>this.nodeZExtent[1]&&(this.nodeZExtent[1]=i.zIndex))}},{key:"updateNode",value:function(n){this.addNode(n);var a=this.nodeDataCache[n];this.normalizationFunction.applyTo(a)}},{key:"removeNode",value:function(n){delete this.nodeDataCache[n],delete this.nodeProgramIndex[n],this.highlightedNodes.delete(n),this.hoveredNode===n&&(this.hoveredNode=null),this.nodesWithForcedLabels.delete(n)}},{key:"addEdge",value:function(n){var a=Object.assign({},this.graph.getEdgeAttributes(n));this.settings.edgeReducer&&(a=this.settings.edgeReducer(n,a));var i=UCe(this.settings,n,a);this.edgeDataCache[n]=i,this.edgesWithForcedLabels.delete(n),i.forceLabel&&!i.hidden&&this.edgesWithForcedLabels.add(n),this.settings.zIndex&&(i.zIndex<this.edgeZExtent[0]&&(this.edgeZExtent[0]=i.zIndex),i.zIndex>this.edgeZExtent[1]&&(this.edgeZExtent[1]=i.zIndex))}},{key:"updateEdge",value:function(n){this.addEdge(n)}},{key:"removeEdge",value:function(n){delete this.edgeDataCache[n],delete this.edgeProgramIndex[n],this.hoveredEdge===n&&(this.hoveredEdge=null),this.edgesWithForcedLabels.delete(n)}},{key:"clearNodeIndices",value:function(){this.labelGrid=new N7,this.nodeExtent={x:[0,1],y:[0,1]},this.nodeDataCache={},this.edgeProgramIndex={},this.nodesWithForcedLabels=new Set,this.nodeZExtent=[1/0,-1/0],this.highlightedNodes=new Set}},{key:"clearEdgeIndices",value:function(){this.edgeDataCache={},this.edgeProgramIndex={},this.edgesWithForcedLabels=new Set,this.edgeZExtent=[1/0,-1/0]}},{key:"clearIndices",value:function(){this.clearEdgeIndices(),this.clearNodeIndices()}},{key:"clearNodeState",value:function(){this.displayedNodeLabels=new Set,this.highlightedNodes=new Set,this.hoveredNode=null}},{key:"clearEdgeState",value:function(){this.displayedEdgeLabels=new Set,this.highlightedNodes=new Set,this.hoveredEdge=null}},{key:"clearState",value:function(){this.clearEdgeState(),this.clearNodeState()}},{key:"addNodeToProgram",value:function(n,a,i){var o=this.nodeDataCache[n],s=this.nodePrograms[o.type];if(!s)throw new Error('Sigma: could not find a suitable program for node type "'.concat(o.type,'"!'));s.process(a,i,o),this.nodeProgramIndex[n]=i}},{key:"addEdgeToProgram",value:function(n,a,i){var o=this.edgeDataCache[n],s=this.edgePrograms[o.type];if(!s)throw new Error('Sigma: could not find a suitable program for edge type "'.concat(o.type,'"!'));var l=this.graph.extremities(n),u=this.nodeDataCache[l[0]],d=this.nodeDataCache[l[1]];s.process(a,i,u,d,o),this.edgeProgramIndex[n]=i}},{key:"getRenderParams",value:function(){return{matrix:this.matrix,invMatrix:this.invMatrix,width:this.width,height:this.height,pixelRatio:this.pixelRatio,zoomRatio:this.camera.ratio,cameraAngle:this.camera.angle,sizeRatio:1/this.scaleSize(),correctionRatio:this.correctionRatio,downSizingRatio:this.pickingDownSizingRatio,minEdgeThickness:this.settings.minEdgeThickness,antiAliasingFeather:this.settings.antiAliasingFeather}}},{key:"getStagePadding",value:function(){var n=this.settings,a=n.stagePadding,i=n.autoRescale;return i&&a||0}},{key:"createLayer",value:function(n,a){var i=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{};if(this.elements[n])throw new Error('Sigma: a layer named "'.concat(n,'" already exists'));var o=wCe(a,{position:"absolute"},{class:"sigma-".concat(n)});return i.style&&Object.assign(o.style,i.style),this.elements[n]=o,"beforeLayer"in i&&i.beforeLayer?this.elements[i.beforeLayer].before(o):"afterLayer"in i&&i.afterLayer?this.elements[i.afterLayer].after(o):this.container.appendChild(o),o}},{key:"createCanvas",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};return this.createLayer(n,"canvas",a)}},{key:"createCanvasContext",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},i=this.createCanvas(n,a),o={preserveDrawingBuffer:!1,antialias:!1};return this.canvasContexts[n]=i.getContext("2d",o),this}},{key:"createWebGLContext",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},i=(a==null?void 0:a.canvas)||this.createCanvas(n,a);a.hidden&&i.remove();var o=Ct({preserveDrawingBuffer:!1,antialias:!1},a),s;s=i.getContext("webgl2",o),s||(s=i.getContext("webgl",o)),s||(s=i.getContext("experimental-webgl",o));var l=s;if(this.webGLContexts[n]=l,l.blendFunc(l.ONE,l.ONE_MINUS_SRC_ALPHA),a.picking){this.pickingLayers.add(n);var u=l.createFramebuffer();if(!u)throw new Error("Sigma: cannot create a new frame buffer for layer ".concat(n));this.frameBuffers[n]=u}return l}},{key:"killLayer",value:function(n){var a=this.elements[n];if(!a)throw new Error("Sigma: cannot kill layer ".concat(n,", which does not exist"));if(this.webGLContexts[n]){var i,o=this.webGLContexts[n];(i=o.getExtension("WEBGL_lose_context"))===null||i===void 0||i.loseContext(),delete this.webGLContexts[n]}else this.canvasContexts[n]&&delete this.canvasContexts[n];return a.remove(),delete this.elements[n],this}},{key:"getCamera",value:function(){return this.camera}},{key:"setCamera",value:function(n){this.unbindCameraHandlers(),this.camera=n,this.bindCameraHandlers()}},{key:"getContainer",value:function(){return this.container}},{key:"getGraph",value:function(){return this.graph}},{key:"setGraph",value:function(n){n!==this.graph&&(this.hoveredNode&&!n.hasNode(this.hoveredNode)&&(this.hoveredNode=null),this.hoveredEdge&&!n.hasEdge(this.hoveredEdge)&&(this.hoveredEdge=null),this.unbindGraphHandlers(),this.checkEdgesEventsFrame!==null&&(cancelAnimationFrame(this.checkEdgesEventsFrame),this.checkEdgesEventsFrame=null),this.graph=n,this.bindGraphHandlers(),this.refresh())}},{key:"getMouseCaptor",value:function(){return this.mouseCaptor}},{key:"getTouchCaptor",value:function(){return this.touchCaptor}},{key:"getDimensions",value:function(){return{width:this.width,height:this.height}}},{key:"getGraphDimensions",value:function(){var n=this.customBBox||this.nodeExtent;return{width:n.x[1]-n.x[0]||1,height:n.y[1]-n.y[0]||1}}},{key:"getNodeDisplayData",value:function(n){var a=this.nodeDataCache[n];return a?Object.assign({},a):void 0}},{key:"getEdgeDisplayData",value:function(n){var a=this.edgeDataCache[n];return a?Object.assign({},a):void 0}},{key:"getNodeDisplayedLabels",value:function(){return new Set(this.displayedNodeLabels)}},{key:"getEdgeDisplayedLabels",value:function(){return new Set(this.displayedEdgeLabels)}},{key:"getSettings",value:function(){return Ct({},this.settings)}},{key:"getSetting",value:function(n){return this.settings[n]}},{key:"setSetting",value:function(n,a){var i=Ct({},this.settings);return this.settings[n]=a,cw(this.settings),this.handleSettingsUpdate(i),this.scheduleRefresh(),this}},{key:"updateSetting",value:function(n,a){return this.setSetting(n,a(this.settings[n])),this}},{key:"setSettings",value:function(n){var a=Ct({},this.settings);return this.settings=Ct(Ct({},this.settings),n),cw(this.settings),this.handleSettingsUpdate(a),this.scheduleRefresh(),this}},{key:"resize",value:function(n){var a=this.width,i=this.height;if(this.width=this.container.offsetWidth,this.height=this.container.offsetHeight,this.pixelRatio=C7(),this.width===0)if(this.settings.allowInvalidContainer)this.width=1;else throw new Error("Sigma: Container has no width. You can set the allowInvalidContainer setting to true to stop seeing this error.");if(this.height===0)if(this.settings.allowInvalidContainer)this.height=1;else throw new Error("Sigma: Container has no height. You can set the allowInvalidContainer setting to true to stop seeing this error.");if(!n&&a===this.width&&i===this.height)return this;for(var o in this.elements){var s=this.elements[o];s.style.width=this.width+"px",s.style.height=this.height+"px"}for(var l in this.canvasContexts)this.elements[l].setAttribute("width",this.width*this.pixelRatio+"px"),this.elements[l].setAttribute("height",this.height*this.pixelRatio+"px"),this.pixelRatio!==1&&this.canvasContexts[l].scale(this.pixelRatio,this.pixelRatio);for(var u in this.webGLContexts){this.elements[u].setAttribute("width",this.width*this.pixelRatio+"px"),this.elements[u].setAttribute("height",this.height*this.pixelRatio+"px");var d=this.webGLContexts[u];if(d.viewport(0,0,this.width*this.pixelRatio,this.height*this.pixelRatio),this.pickingLayers.has(u)){var p=this.textures[u];p&&d.deleteTexture(p)}}return this.emit("resize"),this}},{key:"clear",value:function(){return this.emit("beforeClear"),this.webGLContexts.nodes.bindFramebuffer(WebGLRenderingContext.FRAMEBUFFER,null),this.webGLContexts.nodes.clear(WebGLRenderingContext.COLOR_BUFFER_BIT),this.webGLContexts.edges.bindFramebuffer(WebGLRenderingContext.FRAMEBUFFER,null),this.webGLContexts.edges.clear(WebGLRenderingContext.COLOR_BUFFER_BIT),this.webGLContexts.hoverNodes.clear(WebGLRenderingContext.COLOR_BUFFER_BIT),this.canvasContexts.labels.clearRect(0,0,this.width,this.height),this.canvasContexts.hovers.clearRect(0,0,this.width,this.height),this.canvasContexts.edgeLabels.clearRect(0,0,this.width,this.height),this.emit("afterClear"),this}},{key:"refresh",value:function(n){var a=this,i=(n==null?void 0:n.skipIndexation)!==void 0?n==null?void 0:n.skipIndexation:!1,o=(n==null?void 0:n.schedule)!==void 0?n.schedule:!1,s=!n||!n.partialGraph;if(s)this.clearEdgeIndices(),this.clearNodeIndices(),this.graph.forEachNode(function(_){return a.addNode(_)}),this.graph.forEachEdge(function(_){return a.addEdge(_)});else{for(var l,u,d=((l=n.partialGraph)===null||l===void 0?void 0:l.nodes)||[],p=0,f=(d==null?void 0:d.length)||0;p<f;p++){var m=d[p];if(this.updateNode(m),i){var b=this.nodeProgramIndex[m];if(b===void 0)throw new Error('Sigma: node "'.concat(m,`" can't be repaint`));this.addNodeToProgram(m,this.nodeIndices[m],b)}}for(var y=(n==null||(u=n.partialGraph)===null||u===void 0?void 0:u.edges)||[],S=0,w=y.length;S<w;S++){var E=y[S];if(this.updateEdge(E),i){var k=this.edgeProgramIndex[E];if(k===void 0)throw new Error('Sigma: edge "'.concat(E,`" can't be repaint`));this.addEdgeToProgram(E,this.edgeIndices[E],k)}}}return(s||!i)&&(this.needToProcess=!0),o?this.scheduleRender():this.render(),this}},{key:"scheduleRender",value:function(){var n=this;return this.renderFrame||(this.renderFrame=requestAnimationFrame(function(){n.render()})),this}},{key:"scheduleRefresh",value:function(n){return this.refresh(Ct(Ct({},n),{},{schedule:!0}))}},{key:"getViewportZoomedState",value:function(n,a){var i=this.camera.getState(),o=i.ratio,s=i.angle,l=i.x,u=i.y,d=this.settings,p=d.minCameraRatio,f=d.maxCameraRatio;typeof f=="number"&&(a=Math.min(a,f)),typeof p=="number"&&(a=Math.max(a,p));var m=a/o,b={x:this.width/2,y:this.height/2},y=this.viewportToFramedGraph(n),S=this.viewportToFramedGraph(b);return{angle:s,x:(y.x-S.x)*(1-m)+l,y:(y.y-S.y)*(1-m)+u,ratio:a}}},{key:"viewRectangle",value:function(){var n=this.viewportToFramedGraph({x:0,y:0}),a=this.viewportToFramedGraph({x:this.width,y:0}),i=this.viewportToFramedGraph({x:0,y:this.height});return{x1:n.x,y1:n.y,x2:a.x,y2:a.y,height:a.y-i.y}}},{key:"framedGraphToViewport",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},i=!!a.cameraState||!!a.viewportDimensions||!!a.graphDimensions,o=a.matrix?a.matrix:i?Lf(a.cameraState||this.camera.getState(),a.viewportDimensions||this.getDimensions(),a.graphDimensions||this.getGraphDimensions(),a.padding||this.getStagePadding()):this.matrix,s=fR(o,n);return{x:(1+s.x)*this.width/2,y:(1-s.y)*this.height/2}}},{key:"viewportToFramedGraph",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},i=!!a.cameraState||!!a.viewportDimensions||!a.graphDimensions,o=a.matrix?a.matrix:i?Lf(a.cameraState||this.camera.getState(),a.viewportDimensions||this.getDimensions(),a.graphDimensions||this.getGraphDimensions(),a.padding||this.getStagePadding(),!0):this.invMatrix,s=fR(o,{x:n.x/this.width*2-1,y:1-n.y/this.height*2});return isNaN(s.x)&&(s.x=0),isNaN(s.y)&&(s.y=0),s}},{key:"viewportToGraph",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};return this.normalizationFunction.inverse(this.viewportToFramedGraph(n,a))}},{key:"graphToViewport",value:function(n){var a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};return this.framedGraphToViewport(this.normalizationFunction(n),a)}},{key:"getGraphToViewportRatio",value:function(){var n={x:0,y:0},a={x:1,y:1},i=Math.sqrt(Math.pow(n.x-a.x,2)+Math.pow(n.y-a.y,2)),o=this.graphToViewport(n),s=this.graphToViewport(a),l=Math.sqrt(Math.pow(o.x-s.x,2)+Math.pow(o.y-s.y,2));return l/i}},{key:"getBBox",value:function(){return this.nodeExtent}},{key:"getCustomBBox",value:function(){return this.customBBox}},{key:"setCustomBBox",value:function(n){return this.customBBox=n,this.scheduleRender(),this}},{key:"kill",value:function(){this.emit("kill"),this.removeAllListeners(),this.unbindCameraHandlers(),window.removeEventListener("resize",this.activeListeners.handleResize),this.mouseCaptor.kill(),this.touchCaptor.kill(),this.unbindGraphHandlers(),this.clearIndices(),this.clearState(),this.nodeDataCache={},this.edgeDataCache={},this.highlightedNodes.clear(),this.renderFrame&&(cancelAnimationFrame(this.renderFrame),this.renderFrame=null),this.renderHighlightedNodesFrame&&(cancelAnimationFrame(this.renderHighlightedNodesFrame),this.renderHighlightedNodesFrame=null);for(var n=this.container;n.firstChild;)n.removeChild(n.firstChild);for(var a in this.nodePrograms)this.nodePrograms[a].kill();for(var i in this.nodeHoverPrograms)this.nodeHoverPrograms[i].kill();for(var o in this.edgePrograms)this.edgePrograms[o].kill();this.nodePrograms={},this.nodeHoverPrograms={},this.edgePrograms={};for(var s in this.elements)this.killLayer(s);this.canvasContexts={},this.webGLContexts={},this.elements={}}},{key:"scaleSize",value:function(){var n=arguments.length>0&&arguments[0]!==void 0?arguments[0]:1,a=arguments.length>1&&arguments[1]!==void 0?arguments[1]:this.camera.ratio;return n/this.settings.zoomToSizeRatioFunction(a)*(this.getSetting("itemSizesReference")==="positions"?a*this.graphToViewportRatio:1)}},{key:"getCanvases",value:function(){var n={};for(var a in this.elements)this.elements[a]instanceof HTMLCanvasElement&&(n[a]=this.elements[a]);return n}}])})(v5);const pee=T.createContext(null),GCe=pee.Provider;function E5(){const e=T.useContext(pee);if(e==null)throw new Error("No context provided: useSigmaContext() can only be used in a descendant of <SigmaContainer>");return e}function zo(){return E5().sigma}function fee(){const{sigma:e}=E5();return T.useCallback((t=>{e&&Object.keys(t).forEach((r=>{e.setSetting(r,t[r])}))}),[e])}function u2(e){return new Set(Object.keys(e))}const I7=u2({clickNode:!0,rightClickNode:!0,downNode:!0,enterNode:!0,leaveNode:!0,doubleClickNode:!0,wheelNode:!0,clickEdge:!0,rightClickEdge:!0,downEdge:!0,enterEdge:!0,leaveEdge:!0,doubleClickEdge:!0,wheelEdge:!0,clickStage:!0,rightClickStage:!0,downStage:!0,doubleClickStage:!0,wheelStage:!0,beforeRender:!0,afterRender:!0,kill:!0,upStage:!0,upEdge:!0,upNode:!0,enterStage:!0,leaveStage:!0,resize:!0,afterClear:!0,afterProcess:!0,beforeClear:!0,beforeProcess:!0,moveBody:!0}),O7=u2({click:!0,rightClick:!0,doubleClick:!0,mouseup:!0,mousedown:!0,mousemove:!0,mousemovebody:!0,mouseleave:!0,mouseenter:!0,wheel:!0}),B7=u2({touchup:!0,touchdown:!0,touchmove:!0,touchmovebody:!0,tap:!0,doubletap:!0}),P7=u2({updated:!0});function mee(){const e=zo(),t=fee(),[r,n]=T.useState({});return T.useEffect((()=>{if(!e||!r)return;const a=r,i=Object.keys(a);return i.forEach((o=>{const s=a[o];I7.has(o)&&e.on(o,s),O7.has(o)&&e.getMouseCaptor().on(o,s),B7.has(o)&&e.getTouchCaptor().on(o,s),P7.has(o)&&e.getCamera().on(o,s)})),()=>{e&&i.forEach((o=>{const s=a[o];I7.has(o)&&e.off(o,s),O7.has(o)&&e.getMouseCaptor().off(o,s),B7.has(o)&&e.getTouchCaptor().off(o,s),P7.has(o)&&e.getCamera().off(o,s)}))}}),[e,r,t]),n}function _g(e,t){if(e===t)return!0;if(typeof e=="object"&&e!=null&&typeof t=="object"&&t!=null){if(Object.keys(e).length!=Object.keys(t).length)return!1;for(const r in e)if(!Object.hasOwn(t,r)||!_g(e[r],t[r]))return!1;return!0}return!1}function gee(e){const t=zo(),[r,n]=T.useState(e||{});T.useEffect((()=>{n((u=>_g(u,e||{})?u:e||{}))}),[e]);const a=T.useCallback((u=>{t.getCamera().animatedZoom(Object.assign(Object.assign({},r),u))}),[t,r]),i=T.useCallback((u=>{t.getCamera().animatedUnzoom(Object.assign(Object.assign({},r),u))}),[t,r]),o=T.useCallback((u=>{t.getCamera().animatedReset(Object.assign(Object.assign({},r),u))}),[t,r]),s=T.useCallback(((u,d)=>{t.getCamera().animate(u,Object.assign(Object.assign({},r),d))}),[t,r]),l=T.useCallback(((u,d)=>{const p=t.getNodeDisplayData(u);p?t.getCamera().animate(p,Object.assign(Object.assign({},r),d)):console.warn(`Node ${u} not found`)}),[t,r]);return{zoomIn:a,zoomOut:i,reset:o,goto:s,gotoNode:l}}function jCe(e){const t=E5(),[r,n]=T.useState(!1),[a,i]=T.useState(t.container),o=T.useCallback((()=>n((s=>!s))),[]);return T.useEffect((()=>(document.addEventListener("fullscreenchange",o),()=>document.removeEventListener("fullscreenchange",o))),[o]),T.useEffect((()=>{i(t.container)}),[e,t.container]),{toggle:T.useCallback((()=>{var s;s=a,document.fullscreenElement!==s?s.requestFullscreen():document.exitFullscreen&&document.exitFullscreen()}),[a]),isFullScreen:r}}const qCe=T.forwardRef((({graph:e,id:t,className:r,style:n,settings:a={},children:i},o)=>{const s=T.useRef(null),l=T.useRef(null),u={className:`react-sigma ${r||""}`,id:t,style:n},[d,p]=T.useState(null),[f,m]=T.useState(a);T.useEffect((()=>{m((S=>_g(S,a)?S:a))}),[a]),T.useEffect((()=>{p((S=>{let w=null;if(l.current!==null){let E=new Jr;e&&(E=typeof e=="function"?new e:e);let k=null;S&&(k=S.getCamera().getState(),S.kill()),w=new $Ce(E,l.current,f),k&&w.getCamera().setState(k)}return w}))}),[l,e,f]),T.useImperativeHandle(o,(()=>d),[d]);const b=T.useMemo((()=>d&&s.current?{sigma:d,container:s.current}:null),[d,s]),y=b!==null?$e.createElement(GCe,{value:b},i):null;return $e.createElement("div",Object.assign({},u,{ref:s}),$e.createElement("div",{className:"sigma-container",ref:l}),y)}));var WCe=`
precision mediump float;

varying vec4 v_color;
varying float v_border;

const float radius = 0.5;
const vec4 transparent = vec4(0.0, 0.0, 0.0, 0.0);

void main(void) {
  vec2 m = gl_PointCoord - vec2(0.5, 0.5);
  float dist = radius - length(m);

  // No antialiasing for picking mode:
  #ifdef PICKING_MODE
  if (dist > v_border)
    gl_FragColor = v_color;
  else
    gl_FragColor = transparent;

  #else
  float t = 0.0;
  if (dist > v_border)
    t = 1.0;
  else if (dist > 0.0)
    t = dist / v_border;

  gl_FragColor = mix(transparent, v_color, t);
  #endif
}
`,VCe=WCe,KCe=`
attribute vec4 a_id;
attribute vec4 a_color;
attribute vec2 a_position;
attribute float a_size;

uniform float u_sizeRatio;
uniform float u_pixelRatio;
uniform mat3 u_matrix;

varying vec4 v_color;
varying float v_border;

const float bias = 255.0 / 254.0;

void main() {
  gl_Position = vec4(
    (u_matrix * vec3(a_position, 1)).xy,
    0,
    1
  );

  // Multiply the point size twice:
  //  - x SCALING_RATIO to correct the canvas scaling
  //  - x 2 to correct the formulae
  gl_PointSize = a_size / u_sizeRatio * u_pixelRatio * 2.0;

  v_border = (0.5 / a_size) * u_sizeRatio;

  #ifdef PICKING_MODE
  // For picking mode, we use the ID as the color:
  v_color = a_id;
  #else
  // For normal mode, we use the color:
  v_color = a_color;
  #endif

  v_color.a *= bias;
}
`,YCe=KCe,bee=WebGLRenderingContext,z7=bee.UNSIGNED_BYTE,H7=bee.FLOAT,JCe=["u_sizeRatio","u_pixelRatio","u_matrix"],XCe=(function(e){function t(){return la(this,t),gi(this,t,arguments)}return bi(t,e),ua(t,[{key:"getDefinition",value:function(){return{VERTICES:1,VERTEX_SHADER_SOURCE:YCe,FRAGMENT_SHADER_SOURCE:VCe,METHOD:WebGLRenderingContext.POINTS,UNIFORMS:JCe,ATTRIBUTES:[{name:"a_position",size:2,type:H7},{name:"a_size",size:1,type:H7},{name:"a_color",size:4,type:z7,normalized:!0},{name:"a_id",size:4,type:z7,normalized:!0}]}}},{key:"processVisibleItem",value:function(n,a,i){var o=this.array;o[a++]=i.x,o[a++]=i.y,o[a++]=i.size,o[a++]=bs(i.color),o[a++]=n}},{key:"setUniforms",value:function(n,a){var i=n.sizeRatio,o=n.pixelRatio,s=n.matrix,l=a.gl,u=a.uniformLocations,d=u.u_sizeRatio,p=u.u_pixelRatio,f=u.u_matrix;l.uniform1f(p,o),l.uniform1f(d,i),l.uniformMatrix3fv(f,!1,s)}}])})(b5),ZCe=`
attribute vec4 a_id;
attribute vec4 a_color;
attribute vec2 a_normal;
attribute float a_normalCoef;
attribute vec2 a_positionStart;
attribute vec2 a_positionEnd;
attribute float a_positionCoef;
attribute float a_sourceRadius;
attribute float a_targetRadius;
attribute float a_sourceRadiusCoef;
attribute float a_targetRadiusCoef;

uniform mat3 u_matrix;
uniform float u_zoomRatio;
uniform float u_sizeRatio;
uniform float u_pixelRatio;
uniform float u_correctionRatio;
uniform float u_minEdgeThickness;
uniform float u_lengthToThicknessRatio;
uniform float u_feather;

varying vec4 v_color;
varying vec2 v_normal;
varying float v_thickness;
varying float v_feather;

const float bias = 255.0 / 254.0;

void main() {
  float minThickness = u_minEdgeThickness;

  vec2 normal = a_normal * a_normalCoef;
  vec2 position = a_positionStart * (1.0 - a_positionCoef) + a_positionEnd * a_positionCoef;

  float normalLength = length(normal);
  vec2 unitNormal = normal / normalLength;

  // These first computations are taken from edge.vert.glsl. Please read it to
  // get better comments on what's happening:
  float pixelsThickness = max(normalLength, minThickness * u_sizeRatio);
  float webGLThickness = pixelsThickness * u_correctionRatio / u_sizeRatio;

  // Here, we move the point to leave space for the arrow heads:
  // Source arrow head
  float sourceRadius = a_sourceRadius * a_sourceRadiusCoef;
  float sourceDirection = sign(sourceRadius);
  float webGLSourceRadius = sourceDirection * sourceRadius * 2.0 * u_correctionRatio / u_sizeRatio;
  float webGLSourceArrowHeadLength = webGLThickness * u_lengthToThicknessRatio * 2.0;
  vec2 sourceCompensationVector =
    vec2(-sourceDirection * unitNormal.y, sourceDirection * unitNormal.x)
    * (webGLSourceRadius + webGLSourceArrowHeadLength);
    
  // Target arrow head
  float targetRadius = a_targetRadius * a_targetRadiusCoef;
  float targetDirection = sign(targetRadius);
  float webGLTargetRadius = targetDirection * targetRadius * 2.0 * u_correctionRatio / u_sizeRatio;
  float webGLTargetArrowHeadLength = webGLThickness * u_lengthToThicknessRatio * 2.0;
  vec2 targetCompensationVector =
  vec2(-targetDirection * unitNormal.y, targetDirection * unitNormal.x)
    * (webGLTargetRadius + webGLTargetArrowHeadLength);

  // Here is the proper position of the vertex
  gl_Position = vec4((u_matrix * vec3(position + unitNormal * webGLThickness + sourceCompensationVector + targetCompensationVector, 1)).xy, 0, 1);

  v_thickness = webGLThickness / u_zoomRatio;

  v_normal = unitNormal;

  v_feather = u_feather * u_correctionRatio / u_zoomRatio / u_pixelRatio * 2.0;

  #ifdef PICKING_MODE
  // For picking mode, we use the ID as the color:
  v_color = a_id;
  #else
  // For normal mode, we use the color:
  v_color = a_color;
  #endif

  v_color.a *= bias;
}
`,QCe=ZCe,yee=WebGLRenderingContext,U7=yee.UNSIGNED_BYTE,qs=yee.FLOAT,eTe=["u_matrix","u_zoomRatio","u_sizeRatio","u_correctionRatio","u_pixelRatio","u_feather","u_minEdgeThickness","u_lengthToThicknessRatio"],tTe={lengthToThicknessRatio:Fg.lengthToThicknessRatio};function vee(e){var t=Ct(Ct({},tTe),{});return(function(r){function n(){return la(this,n),gi(this,n,arguments)}return bi(n,r),ua(n,[{key:"getDefinition",value:function(){return{VERTICES:6,VERTEX_SHADER_SOURCE:QCe,FRAGMENT_SHADER_SOURCE:y5,METHOD:WebGLRenderingContext.TRIANGLES,UNIFORMS:eTe,ATTRIBUTES:[{name:"a_positionStart",size:2,type:qs},{name:"a_positionEnd",size:2,type:qs},{name:"a_normal",size:2,type:qs},{name:"a_color",size:4,type:U7,normalized:!0},{name:"a_id",size:4,type:U7,normalized:!0},{name:"a_sourceRadius",size:1,type:qs},{name:"a_targetRadius",size:1,type:qs}],CONSTANT_ATTRIBUTES:[{name:"a_positionCoef",size:1,type:qs},{name:"a_normalCoef",size:1,type:qs},{name:"a_sourceRadiusCoef",size:1,type:qs},{name:"a_targetRadiusCoef",size:1,type:qs}],CONSTANT_DATA:[[0,1,-1,0],[0,-1,1,0],[1,1,0,1],[1,1,0,1],[0,-1,1,0],[1,-1,0,-1]]}}},{key:"processVisibleItem",value:function(i,o,s,l,u){var d=u.size||1,p=s.x,f=s.y,m=l.x,b=l.y,y=bs(u.color),S=m-p,w=b-f,E=s.size||1,k=l.size||1,_=S*S+w*w,R=0,C=0;_&&(_=1/Math.sqrt(_),R=-w*_*d,C=S*_*d);var F=this.array;F[o++]=p,F[o++]=f,F[o++]=m,F[o++]=b,F[o++]=R,F[o++]=C,F[o++]=y,F[o++]=i,F[o++]=E,F[o++]=k}},{key:"setUniforms",value:function(i,o){var s=o.gl,l=o.uniformLocations,u=l.u_matrix,d=l.u_zoomRatio,p=l.u_feather,f=l.u_pixelRatio,m=l.u_correctionRatio,b=l.u_sizeRatio,y=l.u_minEdgeThickness,S=l.u_lengthToThicknessRatio;s.uniformMatrix3fv(u,!1,i.matrix),s.uniform1f(d,i.zoomRatio),s.uniform1f(b,i.sizeRatio),s.uniform1f(m,i.correctionRatio),s.uniform1f(f,i.pixelRatio),s.uniform1f(p,i.antiAliasingFeather),s.uniform1f(y,i.minEdgeThickness),s.uniform1f(S,t.lengthToThicknessRatio)}}])})(Tg)}vee();function rTe(e){return eee([vee(),by(e),by(Ct(Ct({},e),{},{extremity:"source"}))])}rTe();function nTe(e){if(Array.isArray(e))return e}function aTe(e,t){var r=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(r!=null){var n,a,i,o,s=[],l=!0,u=!1;try{if(i=(r=r.call(e)).next,t!==0)for(;!(l=(n=i.call(r)).done)&&(s.push(n.value),s.length!==t);l=!0);}catch(d){u=!0,a=d}finally{try{if(!l&&r.return!=null&&(o=r.return(),Object(o)!==o))return}finally{if(u)throw a}}return s}}function gR(e,t){(t==null||t>e.length)&&(t=e.length);for(var r=0,n=Array(t);r<t;r++)n[r]=e[r];return n}function See(e,t){if(e){if(typeof e=="string")return gR(e,t);var r={}.toString.call(e).slice(8,-1);return r==="Object"&&e.constructor&&(r=e.constructor.name),r==="Map"||r==="Set"?Array.from(e):r==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r)?gR(e,t):void 0}}function iTe(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function oTe(e,t){return nTe(e)||aTe(e,t)||See(e,t)||iTe()}function sTe(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function lTe(e,t){if(typeof e!="object"||!e)return e;var r=e[Symbol.toPrimitive];if(r!==void 0){var n=r.call(e,t);if(typeof n!="object")return n;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function Eee(e){var t=lTe(e,"string");return typeof t=="symbol"?t:t+""}function uTe(e,t){for(var r=0;r<t.length;r++){var n=t[r];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,Eee(n.key),n)}}function cTe(e,t,r){return t&&uTe(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function yy(e){return yy=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(t){return t.__proto__||Object.getPrototypeOf(t)},yy(e)}function xee(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(xee=function(){return!!e})()}function bR(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function dTe(e,t){if(t&&(typeof t=="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return bR(e)}function hTe(e,t,r){return t=yy(t),dTe(e,xee()?Reflect.construct(t,r||[],yy(e).constructor):t.apply(e,r))}function yR(e,t){return yR=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,n){return r.__proto__=n,r},yR(e,t)}function pTe(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&yR(e,t)}function uh(e,t,r){return(t=Eee(t))in e?Object.defineProperty(e,t,{value:r,enumerable:!0,configurable:!0,writable:!0}):e[t]=r,e}function fTe(e){if(Array.isArray(e))return gR(e)}function mTe(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function gTe(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function hw(e){return fTe(e)||mTe(e)||See(e)||gTe()}function $7(e,t){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),r.push.apply(r,n)}return r}function G7(e){for(var t=1;t<arguments.length;t++){var r=arguments[t]!=null?arguments[t]:{};t%2?$7(Object(r),!0).forEach(function(n){uh(e,n,r[n])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):$7(Object(r)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))})}return e}var bTe="relative",yTe={drawLabel:void 0,drawHover:void 0,borders:[{size:{value:.1},color:{attribute:"borderColor"}},{size:{fill:!0},color:{attribute:"color"}}]},vTe="#000000";function STe(e){var t=e.borders,r=g7(t.filter(function(a){var i=a.size;return"fill"in i}).length),n=`
precision highp float;

varying vec2 v_diffVector;
varying float v_radius;

#ifdef PICKING_MODE
varying vec4 v_color;
#else
// For normal mode, we use the border colors defined in the program:
`.concat(t.flatMap(function(a,i){var o=a.size;return"attribute"in o?["varying float v_borderSize_".concat(i+1,";")]:[]}).join(`
`),`
`).concat(t.flatMap(function(a,i){var o=a.color;return"attribute"in o?["varying vec4 v_borderColor_".concat(i+1,";")]:"value"in o?["uniform vec4 u_borderColor_".concat(i+1,";")]:[]}).join(`
`),`
#endif

uniform float u_correctionRatio;

const float bias = 255.0 / 254.0;
const vec4 transparent = vec4(0.0, 0.0, 0.0, 0.0);

void main(void) {
  float dist = length(v_diffVector);
  float aaBorder = 2.0 * u_correctionRatio;
  float v_borderSize_0 = v_radius;
  vec4 v_borderColor_0 = transparent;

  // No antialiasing for picking mode:
  #ifdef PICKING_MODE
  if (dist > v_radius)
    gl_FragColor = transparent;
  else {
    gl_FragColor = v_color;
    gl_FragColor.a *= bias;
  }
  #else
  // Sizes:
`).concat(t.flatMap(function(a,i){var o=a.size;if("fill"in o)return[];o=o;var s="attribute"in o?"v_borderSize_".concat(i+1):g7(o.value),l=(o.mode||bTe)==="pixels"?"u_correctionRatio":"v_radius";return["  float borderSize_".concat(i+1," = ").concat(l," * ").concat(s,";")]}).join(`
`),`
  // Now, let's split the remaining space between "fill" borders:
  float fillBorderSize = (v_radius - (`).concat(t.flatMap(function(a,i){var o=a.size;return"fill"in o?[]:["borderSize_".concat(i+1)]}).join(" + "),") ) / ").concat(r,`;
`).concat(t.flatMap(function(a,i){var o=a.size;return"fill"in o?["  float borderSize_".concat(i+1," = fillBorderSize;")]:[]}).join(`
`),`

  // Finally, normalize all border sizes, to start from the full size and to end with the smallest:
  float adjustedBorderSize_0 = v_radius;
`).concat(t.map(function(a,i){return"  float adjustedBorderSize_".concat(i+1," = adjustedBorderSize_").concat(i," - borderSize_").concat(i+1,";")}).join(`
`),`

  // Colors:
  vec4 borderColor_0 = transparent;
`).concat(t.map(function(a,i){var o=a.color,s=[];return"attribute"in o?s.push("  vec4 borderColor_".concat(i+1," = v_borderColor_").concat(i+1,";")):"transparent"in o?s.push("  vec4 borderColor_".concat(i+1," = vec4(0.0, 0.0, 0.0, 0.0);")):s.push("  vec4 borderColor_".concat(i+1," = u_borderColor_").concat(i+1,";")),s.push("  borderColor_".concat(i+1,".a *= bias;")),s.push("  if (borderSize_".concat(i+1," <= 1.0 * u_correctionRatio) { borderColor_").concat(i+1," = borderColor_").concat(i,"; }")),s.join(`
`)}).join(`
`),`
  if (dist > adjustedBorderSize_0) {
    gl_FragColor = borderColor_0;
  } else `).concat(t.map(function(a,i){return"if (dist > adjustedBorderSize_".concat(i,` - aaBorder) {
    gl_FragColor = mix(borderColor_`).concat(i+1,", borderColor_").concat(i,", (dist - adjustedBorderSize_").concat(i,` + aaBorder) / aaBorder);
  } else if (dist > adjustedBorderSize_`).concat(i+1,`) {
    gl_FragColor = borderColor_`).concat(i+1,`;
  } else `)}).join(""),` { /* Nothing to add here */ }
  #endif
}
`);return n}function ETe(e){var t=e.borders,r=`
attribute vec2 a_position;
attribute float a_size;
attribute float a_angle;

uniform mat3 u_matrix;
uniform float u_sizeRatio;
uniform float u_correctionRatio;

varying vec2 v_diffVector;
varying float v_radius;

#ifdef PICKING_MODE
attribute vec4 a_id;
varying vec4 v_color;
#else
`.concat(t.flatMap(function(n,a){var i=n.size;return"attribute"in i?["attribute float a_borderSize_".concat(a+1,";"),"varying float v_borderSize_".concat(a+1,";")]:[]}).join(`
`),`
`).concat(t.flatMap(function(n,a){var i=n.color;return"attribute"in i?["attribute vec4 a_borderColor_".concat(a+1,";"),"varying vec4 v_borderColor_".concat(a+1,";")]:[]}).join(`
`),`
#endif

const float bias = 255.0 / 254.0;
const vec4 transparent = vec4(0.0, 0.0, 0.0, 0.0);

void main() {
  float size = a_size * u_correctionRatio / u_sizeRatio * 4.0;
  vec2 diffVector = size * vec2(cos(a_angle), sin(a_angle));
  vec2 position = a_position + diffVector;
  gl_Position = vec4(
    (u_matrix * vec3(position, 1)).xy,
    0,
    1
  );

  v_radius = size / 2.0;
  v_diffVector = diffVector;

  #ifdef PICKING_MODE
  v_color = a_id;
  #else
`).concat(t.flatMap(function(n,a){var i=n.size;return"attribute"in i?["  v_borderSize_".concat(a+1," = a_borderSize_").concat(a+1,";")]:[]}).join(`
`),`
`).concat(t.flatMap(function(n,a){var i=n.color;return"attribute"in i?["  v_borderColor_".concat(a+1," = a_borderColor_").concat(a+1,";")]:[]}).join(`
`),`
  #endif
}
`);return r}var wee=WebGLRenderingContext,j7=wee.UNSIGNED_BYTE,Hb=wee.FLOAT;function xTe(e){var t,r=G7(G7({},yTe),{}),n=r.borders,a=r.drawLabel,i=r.drawHover,o=["u_sizeRatio","u_correctionRatio","u_matrix"].concat(hw(n.flatMap(function(s,l){var u=s.color;return"value"in u?["u_borderColor_".concat(l+1)]:[]})));return t=(function(s){pTe(l,s);function l(){var u;sTe(this,l);for(var d=arguments.length,p=new Array(d),f=0;f<d;f++)p[f]=arguments[f];return u=hTe(this,l,[].concat(p)),uh(bR(u),"drawLabel",a),uh(bR(u),"drawHover",i),u}return cTe(l,[{key:"getDefinition",value:function(){return{VERTICES:3,VERTEX_SHADER_SOURCE:ETe(r),FRAGMENT_SHADER_SOURCE:STe(r),METHOD:WebGLRenderingContext.TRIANGLES,UNIFORMS:o,ATTRIBUTES:[{name:"a_position",size:2,type:Hb},{name:"a_id",size:4,type:j7,normalized:!0},{name:"a_size",size:1,type:Hb}].concat(hw(n.flatMap(function(d,p){var f=d.color;return"attribute"in f?[{name:"a_borderColor_".concat(p+1),size:4,type:j7,normalized:!0}]:[]})),hw(n.flatMap(function(d,p){var f=d.size;return"attribute"in f?[{name:"a_borderSize_".concat(p+1),size:1,type:Hb}]:[]}))),CONSTANT_ATTRIBUTES:[{name:"a_angle",size:1,type:Hb}],CONSTANT_DATA:[[l.ANGLE_1],[l.ANGLE_2],[l.ANGLE_3]]}}},{key:"processVisibleItem",value:function(d,p,f){var m=this.array;m[p++]=f.x,m[p++]=f.y,m[p++]=d,m[p++]=f.size,n.forEach(function(b){var y=b.color;"attribute"in y&&(m[p++]=bs(f[y.attribute]||y.defaultValue||vTe))}),n.forEach(function(b){var y=b.size;"attribute"in y&&(m[p++]=f[y.attribute]||y.defaultValue)})}},{key:"setUniforms",value:function(d,p){var f=p.gl,m=p.uniformLocations,b=m.u_sizeRatio,y=m.u_correctionRatio,S=m.u_matrix;f.uniform1f(y,d.correctionRatio),f.uniform1f(b,d.sizeRatio),f.uniformMatrix3fv(S,!1,d.matrix),n.forEach(function(w,E){var k=w.color;if("value"in k){var _=m["u_borderColor_".concat(E+1)],R=Mke(k.value),C=oTe(R,4),F=C[0],L=C[1],I=C[2],M=C[3];f.uniform4f(_,F/255,L/255,I/255,M/255)}})}}]),l})(b5),uh(t,"ANGLE_1",0),uh(t,"ANGLE_2",2*Math.PI/3),uh(t,"ANGLE_3",4*Math.PI/3),t}var wTe=xTe();function kTe(e,t){if(typeof e!="object"||!e)return e;var r=e[Symbol.toPrimitive];if(r!==void 0){var n=r.call(e,t);if(typeof n!="object")return n;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function kee(e){var t=kTe(e,"string");return typeof t=="symbol"?t:t+""}function Cee(e,t,r){return(t=kee(t))in e?Object.defineProperty(e,t,{value:r,enumerable:!0,configurable:!0,writable:!0}):e[t]=r,e}function q7(e,t){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),r.push.apply(r,n)}return r}function vy(e){for(var t=1;t<arguments.length;t++){var r=arguments[t]!=null?arguments[t]:{};t%2?q7(Object(r),!0).forEach(function(n){Cee(e,n,r[n])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):q7(Object(r)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))})}return e}function CTe(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function TTe(e,t){for(var r=0;r<t.length;r++){var n=t[r];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,kee(n.key),n)}}function ATe(e,t,r){return t&&TTe(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function Sy(e){return Sy=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(t){return t.__proto__||Object.getPrototypeOf(t)},Sy(e)}function Tee(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(Tee=function(){return!!e})()}function Aee(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function FTe(e,t){if(t&&(typeof t=="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return Aee(e)}function _Te(e,t,r){return t=Sy(t),FTe(e,Tee()?Reflect.construct(t,r||[],Sy(e).constructor):t.apply(e,r))}function vR(e,t){return vR=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,n){return r.__proto__=n,r},vR(e,t)}function RTe(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&vR(e,t)}function SR(e,t){(t==null||t>e.length)&&(t=e.length);for(var r=0,n=Array(t);r<t;r++)n[r]=e[r];return n}function DTe(e){if(Array.isArray(e))return SR(e)}function NTe(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function LTe(e,t){if(e){if(typeof e=="string")return SR(e,t);var r={}.toString.call(e).slice(8,-1);return r==="Object"&&e.constructor&&(r=e.constructor.name),r==="Map"||r==="Set"?Array.from(e):r==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r)?SR(e,t):void 0}}function MTe(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function pw(e){return DTe(e)||NTe(e)||LTe(e)||MTe()}function Fee(e,t,r,n){var a=Math.pow(1-e,2)*t.x+2*(1-e)*e*r.x+Math.pow(e,2)*n.x,i=Math.pow(1-e,2)*t.y+2*(1-e)*e*r.y+Math.pow(e,2)*n.y;return{x:a,y:i}}function ITe(e,t,r){for(var n=20,a=0,i=e,o=0;o<n;o++){var s=Fee((o+1)/n,e,t,r);a+=Math.sqrt(Math.pow(i.x-s.x,2)+Math.pow(i.y-s.y,2)),i=s}return a}function OTe(e){var t=e.curvatureAttribute,r=e.defaultCurvature,n=e.keepLabelUpright,a=n===void 0?!0:n;return function(i,o,s,l,u){var d=u.edgeLabelSize,p=o[t]||r,f=u.edgeLabelFont,m=u.edgeLabelWeight,b=u.edgeLabelColor.attribute?o[u.edgeLabelColor.attribute]||u.edgeLabelColor.color||"#000":u.edgeLabelColor.color,y=o.label;if(y){i.fillStyle=b,i.font="".concat(m," ").concat(d,"px ").concat(f);var S=!a||s.x<l.x,w=S?s.x:l.x,E=S?s.y:l.y,k=S?l.x:s.x,_=S?l.y:s.y,R=(w+k)/2,C=(E+_)/2,F=k-w,L=_-E,I=Math.sqrt(Math.pow(F,2)+Math.pow(L,2)),M=S?1:-1,j=R+L*p*M,Y=C-F*p*M,W=o.size*.7+5,$={x:Y-E,y:-(j-w)},X=Math.sqrt(Math.pow($.x,2)+Math.pow($.y,2)),z={x:_-Y,y:-(k-j)},H=Math.sqrt(Math.pow(z.x,2)+Math.pow(z.y,2));w+=W*$.x/X,E+=W*$.y/X,k+=W*z.x/H,_+=W*z.y/H,j+=W*L/I,Y-=W*F/I;var B={x:j,y:Y},ee={x:w,y:E},J={x:k,y:_},K=ITe(ee,B,J);if(!(K<s.size+l.size)){var G=i.measureText(y).width,P=K-s.size-l.size;if(G>P){var ne="…";for(y=y+ne,G=i.measureText(y).width;G>P&&y.length>1;)y=y.slice(0,-2)+ne,G=i.measureText(y).width;if(y.length<4)return}for(var oe={},pe=0,ae=y.length;pe<ae;pe++){var se=y[pe];oe[se]||(oe[se]=i.measureText(se).width*(1+p*.35))}for(var Ee=.5-G/K/2,fe=0,Me=y.length;fe<Me;fe++){var Oe=y[fe],xe=Fee(Ee,ee,B,J),Te=2*(1-Ee)*(j-w)+2*Ee*(k-j),Ne=2*(1-Ee)*(Y-E)+2*Ee*(_-Y),he=Math.atan2(Ne,Te);i.save(),i.translate(xe.x,xe.y),i.rotate(he),i.fillText(Oe,0,0),i.restore(),Ee+=oe[Oe]/K}}}}}function BTe(e){var t=e.arrowHead,r=(t==null?void 0:t.extremity)==="target"||(t==null?void 0:t.extremity)==="both",n=(t==null?void 0:t.extremity)==="source"||(t==null?void 0:t.extremity)==="both",a=`
precision highp float;

varying vec4 v_color;
varying float v_thickness;
varying float v_feather;
varying vec2 v_cpA;
varying vec2 v_cpB;
varying vec2 v_cpC;
`.concat(r?`
varying float v_targetSize;
varying vec2 v_targetPoint;`:"",`
`).concat(n?`
varying float v_sourceSize;
varying vec2 v_sourcePoint;`:"",`
`).concat(t?`
uniform float u_lengthToThicknessRatio;
uniform float u_widenessToThicknessRatio;`:"",`

float det(vec2 a, vec2 b) {
  return a.x * b.y - b.x * a.y;
}

vec2 getDistanceVector(vec2 b0, vec2 b1, vec2 b2) {
  float a = det(b0, b2), b = 2.0 * det(b1, b0), d = 2.0 * det(b2, b1);
  float f = b * d - a * a;
  vec2 d21 = b2 - b1, d10 = b1 - b0, d20 = b2 - b0;
  vec2 gf = 2.0 * (b * d21 + d * d10 + a * d20);
  gf = vec2(gf.y, -gf.x);
  vec2 pp = -f * gf / dot(gf, gf);
  vec2 d0p = b0 - pp;
  float ap = det(d0p, d20), bp = 2.0 * det(d10, d0p);
  float t = clamp((ap + bp) / (2.0 * a + b + d), 0.0, 1.0);
  return mix(mix(b0, b1, t), mix(b1, b2, t), t);
}

float distToQuadraticBezierCurve(vec2 p, vec2 b0, vec2 b1, vec2 b2) {
  return length(getDistanceVector(b0 - p, b1 - p, b2 - p));
}

const vec4 transparent = vec4(0.0, 0.0, 0.0, 0.0);

void main(void) {
  float dist = distToQuadraticBezierCurve(gl_FragCoord.xy, v_cpA, v_cpB, v_cpC);
  float thickness = v_thickness;
`).concat(r?`
  float distToTarget = length(gl_FragCoord.xy - v_targetPoint);
  float targetArrowLength = v_targetSize + thickness * u_lengthToThicknessRatio;
  if (distToTarget < targetArrowLength) {
    thickness = (distToTarget - v_targetSize) / (targetArrowLength - v_targetSize) * u_widenessToThicknessRatio * thickness;
  }`:"",`
`).concat(n?`
  float distToSource = length(gl_FragCoord.xy - v_sourcePoint);
  float sourceArrowLength = v_sourceSize + thickness * u_lengthToThicknessRatio;
  if (distToSource < sourceArrowLength) {
    thickness = (distToSource - v_sourceSize) / (sourceArrowLength - v_sourceSize) * u_widenessToThicknessRatio * thickness;
  }`:"",`

  float halfThickness = thickness / 2.0;
  if (dist < halfThickness) {
    #ifdef PICKING_MODE
    gl_FragColor = v_color;
    #else
    float t = smoothstep(
      halfThickness - v_feather,
      halfThickness,
      dist
    );

    gl_FragColor = mix(v_color, transparent, t);
    #endif
  } else {
    gl_FragColor = transparent;
  }
}
`);return a}function PTe(e){var t=e.arrowHead,r=(t==null?void 0:t.extremity)==="target"||(t==null?void 0:t.extremity)==="both",n=(t==null?void 0:t.extremity)==="source"||(t==null?void 0:t.extremity)==="both",a=`
attribute vec4 a_id;
attribute vec4 a_color;
attribute float a_direction;
attribute float a_thickness;
attribute vec2 a_source;
attribute vec2 a_target;
attribute float a_current;
attribute float a_curvature;
`.concat(r?`attribute float a_targetSize;
`:"",`
`).concat(n?`attribute float a_sourceSize;
`:"",`

uniform mat3 u_matrix;
uniform float u_sizeRatio;
uniform float u_pixelRatio;
uniform vec2 u_dimensions;
uniform float u_minEdgeThickness;
uniform float u_feather;

varying vec4 v_color;
varying float v_thickness;
varying float v_feather;
varying vec2 v_cpA;
varying vec2 v_cpB;
varying vec2 v_cpC;
`).concat(r?`
varying float v_targetSize;
varying vec2 v_targetPoint;`:"",`
`).concat(n?`
varying float v_sourceSize;
varying vec2 v_sourcePoint;`:"",`
`).concat(t?`
uniform float u_widenessToThicknessRatio;`:"",`

const float bias = 255.0 / 254.0;
const float epsilon = 0.7;

vec2 clipspaceToViewport(vec2 pos, vec2 dimensions) {
  return vec2(
    (pos.x + 1.0) * dimensions.x / 2.0,
    (pos.y + 1.0) * dimensions.y / 2.0
  );
}

vec2 viewportToClipspace(vec2 pos, vec2 dimensions) {
  return vec2(
    pos.x / dimensions.x * 2.0 - 1.0,
    pos.y / dimensions.y * 2.0 - 1.0
  );
}

void main() {
  float minThickness = u_minEdgeThickness;

  // Selecting the correct position
  // Branchless "position = a_source if a_current == 1.0 else a_target"
  vec2 position = a_source * max(0.0, a_current) + a_target * max(0.0, 1.0 - a_current);
  position = (u_matrix * vec3(position, 1)).xy;

  vec2 source = (u_matrix * vec3(a_source, 1)).xy;
  vec2 target = (u_matrix * vec3(a_target, 1)).xy;

  vec2 viewportPosition = clipspaceToViewport(position, u_dimensions);
  vec2 viewportSource = clipspaceToViewport(source, u_dimensions);
  vec2 viewportTarget = clipspaceToViewport(target, u_dimensions);

  vec2 delta = viewportTarget.xy - viewportSource.xy;
  float len = length(delta);
  vec2 normal = vec2(-delta.y, delta.x) * a_direction;
  vec2 unitNormal = normal / len;
  float boundingBoxThickness = len * a_curvature;

  float curveThickness = max(minThickness, a_thickness / u_sizeRatio);
  v_thickness = curveThickness * u_pixelRatio;
  v_feather = u_feather;

  v_cpA = viewportSource;
  v_cpB = 0.5 * (viewportSource + viewportTarget) + unitNormal * a_direction * boundingBoxThickness;
  v_cpC = viewportTarget;

  vec2 viewportOffsetPosition = (
    viewportPosition +
    unitNormal * (boundingBoxThickness / 2.0 + sign(boundingBoxThickness) * (`).concat(t?"curveThickness * u_widenessToThicknessRatio":"curveThickness",` + epsilon)) *
    max(0.0, a_direction) // NOTE: cutting the bounding box in half to avoid overdraw
  );

  position = viewportToClipspace(viewportOffsetPosition, u_dimensions);
  gl_Position = vec4(position, 0, 1);
    
`).concat(r?`
  v_targetSize = a_targetSize * u_pixelRatio / u_sizeRatio;
  v_targetPoint = viewportTarget;
`:"",`
`).concat(n?`
  v_sourceSize = a_sourceSize * u_pixelRatio / u_sizeRatio;
  v_sourcePoint = viewportSource;
`:"",`

  #ifdef PICKING_MODE
  // For picking mode, we use the ID as the color:
  v_color = a_id;
  #else
  // For normal mode, we use the color:
  v_color = a_color;
  #endif

  v_color.a *= bias;
}
`);return a}var _ee=.25,zTe={arrowHead:null,curvatureAttribute:"curvature",defaultCurvature:_ee},Ree=WebGLRenderingContext,W7=Ree.UNSIGNED_BYTE,su=Ree.FLOAT;function c2(e){var t=vy(vy({},zTe),e||{}),r=t,n=r.arrowHead,a=r.curvatureAttribute,i=r.drawLabel,o=(n==null?void 0:n.extremity)==="target"||(n==null?void 0:n.extremity)==="both",s=(n==null?void 0:n.extremity)==="source"||(n==null?void 0:n.extremity)==="both",l=["u_matrix","u_sizeRatio","u_dimensions","u_pixelRatio","u_feather","u_minEdgeThickness"].concat(pw(n?["u_lengthToThicknessRatio","u_widenessToThicknessRatio"]:[]));return(function(u){RTe(d,u);function d(){var p;CTe(this,d);for(var f=arguments.length,m=new Array(f),b=0;b<f;b++)m[b]=arguments[b];return p=_Te(this,d,[].concat(m)),Cee(Aee(p),"drawLabel",i||OTe(t)),p}return ATe(d,[{key:"getDefinition",value:function(){return{VERTICES:6,VERTEX_SHADER_SOURCE:PTe(t),FRAGMENT_SHADER_SOURCE:BTe(t),METHOD:WebGLRenderingContext.TRIANGLES,UNIFORMS:l,ATTRIBUTES:[{name:"a_source",size:2,type:su},{name:"a_target",size:2,type:su}].concat(pw(o?[{name:"a_targetSize",size:1,type:su}]:[]),pw(s?[{name:"a_sourceSize",size:1,type:su}]:[]),[{name:"a_thickness",size:1,type:su},{name:"a_curvature",size:1,type:su},{name:"a_color",size:4,type:W7,normalized:!0},{name:"a_id",size:4,type:W7,normalized:!0}]),CONSTANT_ATTRIBUTES:[{name:"a_current",size:1,type:su},{name:"a_direction",size:1,type:su}],CONSTANT_DATA:[[0,1],[0,-1],[1,1],[0,-1],[1,1],[1,-1]]}}},{key:"processVisibleItem",value:function(f,m,b,y,S){var w,E=S.size||1,k=b.x,_=b.y,R=y.x,C=y.y,F=bs(S.color),L=(w=S[a])!==null&&w!==void 0?w:_ee,I=this.array;I[m++]=k,I[m++]=_,I[m++]=R,I[m++]=C,o&&(I[m++]=y.size),s&&(I[m++]=b.size),I[m++]=E,I[m++]=L,I[m++]=F,I[m++]=f}},{key:"setUniforms",value:function(f,m){var b=m.gl,y=m.uniformLocations,S=y.u_matrix,w=y.u_pixelRatio,E=y.u_feather,k=y.u_sizeRatio,_=y.u_dimensions,R=y.u_minEdgeThickness;if(b.uniformMatrix3fv(S,!1,f.matrix),b.uniform1f(w,f.pixelRatio),b.uniform1f(k,f.sizeRatio),b.uniform1f(E,f.antiAliasingFeather),b.uniform2f(_,f.width*f.pixelRatio,f.height*f.pixelRatio),b.uniform1f(R,f.minEdgeThickness),n){var C=y.u_lengthToThicknessRatio,F=y.u_widenessToThicknessRatio;b.uniform1f(C,n.lengthToThicknessRatio),b.uniform1f(F,n.widenessToThicknessRatio)}}}]),d})(Tg)}c2();var HTe=c2({arrowHead:Fg});c2({arrowHead:vy(vy({},Fg),{},{extremity:"both"})});const UTe=({node:e,move:t})=>{const r=zo(),{gotoNode:n}=gee();return T.useEffect(()=>{const a=r.getGraph();if(t){if(e&&a.hasNode(e))try{a.setNodeAttribute(e,"highlighted",!0),n(e)}catch(i){console.error("Error focusing on node:",i)}else r.setCustomBBox(null),r.getCamera().animate({x:.5,y:.5,ratio:1},{duration:0});pt.getState().setMoveToSelectedNode(!1)}else if(e&&a.hasNode(e))try{a.setNodeAttribute(e,"highlighted",!0)}catch(i){console.error("Error highlighting node:",i)}return()=>{if(e&&a.hasNode(e))try{a.setNodeAttribute(e,"highlighted",!1)}catch(i){console.error("Error cleaning up node highlight:",i)}}},[e,t,r,n]),null};function Sp(e,t){const r=zo(),n=T.useRef(t);return _g(n.current,t)||(n.current=t),{positions:T.useCallback((()=>n.current?e(r.getGraph(),n.current):{}),[r,n,e]),assign:T.useCallback((()=>{n.current&&e.assign(r.getGraph(),n.current)}),[r,n,e])}}function x5(e,t){const r=zo(),[n,a]=T.useState(!1),[i,o]=T.useState(null),s=T.useRef(t);return _g(s.current,t)||(s.current=t),T.useEffect((()=>{a(!1);let l=null;return s.current&&(l=new e(r.getGraph(),s.current)),o(l),()=>{l!==null&&l.kill()}}),[r,s,o,a,e]),{stop:T.useCallback((()=>{i&&(i.stop(),a(!1))}),[i,a]),start:T.useCallback((()=>{i&&(i.start(),a(!0))}),[i,a]),kill:T.useCallback((()=>{i&&i.kill(),a(!1)}),[i,a]),isRunning:n}}var fw,V7;function Rg(){if(V7)return fw;V7=1;function e(r){return!r||typeof r!="object"||typeof r=="function"||Array.isArray(r)||r instanceof Set||r instanceof Map||r instanceof RegExp||r instanceof Date}function t(r,n){r=r||{};var a={};for(var i in n){var o=r[i],s=n[i];if(!e(s)){a[i]=t(o,s);continue}o===void 0?a[i]=s:a[i]=o}return a}return fw=t,fw}var mw,K7;function $Te(){if(K7)return mw;K7=1;function e(r){return function(n,a){return n+Math.floor(r()*(a-n+1))}}var t=e(Math.random);return t.createRandom=e,mw=t,mw}var gw,Y7;function GTe(){if(Y7)return gw;Y7=1;var e=$Te().createRandom;function t(n){var a=e(n);return function(i){for(var o=i.length,s=o-1,l=-1;++l<o;){var u=a(l,s),d=i[u];i[u]=i[l],i[l]=d}}}var r=t(Math.random);return r.createShuffleInPlace=t,gw=r,gw}var bw,J7;function jTe(){if(J7)return bw;J7=1;var e=Rg(),t=ho(),r=GTe(),n={attributes:{x:"x",y:"y"},center:0,hierarchyAttributes:[],rng:Math.random,scale:1};function a(F,L,I,M,j){this.wrappedCircle=j||null,this.children={},this.countChildren=0,this.id=F||null,this.next=null,this.previous=null,this.x=L||null,this.y=I||null,j?this.r=1010101:this.r=M||999}a.prototype.hasChildren=function(){return this.countChildren>0},a.prototype.addChild=function(F,L){this.children[F]=L,++this.countChildren},a.prototype.getChild=function(F){if(!this.children.hasOwnProperty(F)){var L=new a;this.children[F]=L,++this.countChildren}return this.children[F]},a.prototype.applyPositionToChildren=function(){if(this.hasChildren()){var F=this;for(var L in F.children){var I=F.children[L];I.x+=F.x,I.y+=F.y,I.applyPositionToChildren()}}};function i(F,L,I){for(var M in L.children){var j=L.children[M];j.hasChildren()?i(F,j,I):I[j.id]={x:j.x,y:j.y}}}function o(F,L){var I=F.r-L.r,M=L.x-F.x,j=L.y-F.y;return I<0||I*I<M*M+j*j}function s(F,L){var I=F.r-L.r+1e-6,M=L.x-F.x,j=L.y-F.y;return I>0&&I*I>M*M+j*j}function l(F,L){for(var I=0;I<L.length;++I)if(!s(F,L[I]))return!1;return!0}function u(F){return new a(null,F.x,F.y,F.r)}function d(F,L){var I=F.x,M=F.y,j=F.r,Y=L.x,W=L.y,$=L.r,X=Y-I,z=W-M,H=$-j,B=Math.sqrt(X*X+z*z);return new a(null,(I+Y+X/B*H)/2,(M+W+z/B*H)/2,(B+j+$)/2)}function p(F,L,I){var M=F.x,j=F.y,Y=F.r,W=L.x,$=L.y,X=L.r,z=I.x,H=I.y,B=I.r,ee=M-W,J=M-z,K=j-$,G=j-H,P=X-Y,ne=B-Y,oe=M*M+j*j-Y*Y,pe=oe-W*W-$*$+X*X,ae=oe-z*z-H*H+B*B,se=J*K-ee*G,Ee=(K*ae-G*pe)/(se*2)-M,fe=(G*P-K*ne)/se,Me=(J*pe-ee*ae)/(se*2)-j,Oe=(ee*ne-J*P)/se,xe=fe*fe+Oe*Oe-1,Te=2*(Y+Ee*fe+Me*Oe),Ne=Ee*Ee+Me*Me-Y*Y,he=-(xe?(Te+Math.sqrt(Te*Te-4*xe*Ne))/(2*xe):Ne/Te);return new a(null,M+Ee+fe*he,j+Me+Oe*he,he)}function f(F){switch(F.length){case 1:return u(F[0]);case 2:return d(F[0],F[1]);case 3:return p(F[0],F[1],F[2]);default:throw new Error("graphology-layout/circlepack: Invalid basis length "+F.length)}}function m(F,L){var I,M;if(l(L,F))return[L];for(I=0;I<F.length;++I)if(o(L,F[I])&&l(d(F[I],L),F))return[F[I],L];for(I=0;I<F.length-1;++I)for(M=I+1;M<F.length;++M)if(o(d(F[I],F[M]),L)&&o(d(F[I],L),F[M])&&o(d(F[M],L),F[I])&&l(p(F[I],F[M],L),F))return[F[I],F[M],L];throw new Error("graphology-layout/circlepack: extendBasis failure !")}function b(F){var L=F.wrappedCircle,I=F.next.wrappedCircle,M=L.r+I.r,j=(L.x*I.r+I.x*L.r)/M,Y=(L.y*I.r+I.y*L.r)/M;return j*j+Y*Y}function y(F,L){var I=0,M=F.slice(),j=F.length,Y=[],W,$;for(L(M);I<j;)W=M[I],$&&s($,W)?++I:(Y=m(Y,W),$=f(Y),I=0);return $}function S(F,L,I){var M=F.x-L.x,j,Y,W=F.y-L.y,$,X,z=M*M+W*W;z?(Y=L.r+I.r,Y*=Y,X=F.r+I.r,X*=X,Y>X?(j=(z+X-Y)/(2*z),$=Math.sqrt(Math.max(0,X/z-j*j)),I.x=F.x-j*M-$*W,I.y=F.y-j*W+$*M):(j=(z+Y-X)/(2*z),$=Math.sqrt(Math.max(0,Y/z-j*j)),I.x=L.x+j*M-$*W,I.y=L.y+j*W+$*M)):(I.x=L.x+I.r,I.y=L.y)}function w(F,L){var I=F.r+L.r-1e-6,M=L.x-F.x,j=L.y-F.y;return I>0&&I*I>M*M+j*j}function E(F,L){var I=F.length;if(I===0)return 0;var M,j,Y,W,$,X,z,H,B,ee;if(M=F[0],M.x=0,M.y=0,I<=1)return M.r;if(j=F[1],M.x=-j.r,j.x=M.r,j.y=0,I<=2)return M.r+j.r;Y=F[2],S(j,M,Y),M=new a(null,null,null,null,M),j=new a(null,null,null,null,j),Y=new a(null,null,null,null,Y),M.next=Y.previous=j,j.next=M.previous=Y,Y.next=j.previous=M;e:for(X=3;X<I;++X){Y=F[X],S(M.wrappedCircle,j.wrappedCircle,Y),Y=new a(null,null,null,null,Y),z=j.next,H=M.previous,B=j.wrappedCircle.r,ee=M.wrappedCircle.r;do if(B<=ee){if(w(z.wrappedCircle,Y.wrappedCircle)){j=z,M.next=j,j.previous=M,--X;continue e}B+=z.wrappedCircle.r,z=z.next}else{if(w(H.wrappedCircle,Y.wrappedCircle)){M=H,M.next=j,j.previous=M,--X;continue e}ee+=H.wrappedCircle.r,H=H.previous}while(z!==H.next);for(Y.previous=M,Y.next=j,M.next=j.previous=j=Y,W=b(M);(Y=Y.next)!==j;)($=b(Y))<W&&(M=Y,W=$);j=M.next}M=[j.wrappedCircle],Y=j;for(var J=1e4;(Y=Y.next)!==j&&--J!==0;)M.push(Y.wrappedCircle);for(Y=y(M,L),X=0;X<I;++X)M=F[X],M.x-=Y.x,M.y-=Y.y;return Y.r}function k(F,L){var I=0;if(F.hasChildren()){for(var M in F.children){var j=F.children[M];j.hasChildren()&&(j.r=k(j,L))}I=E(Object.values(F.children),L)}return I}function _(F,L){k(F,L);for(var I in F.children){var M=F.children[I];M.applyPositionToChildren()}}function R(F,L,I){if(!t(L))throw new Error("graphology-layout/circlepack: the given graph is not a valid graphology instance.");I=e(I,n);var M={},j={},Y=L.nodes(),W=I.center,$=I.hierarchyAttributes,X=r.createShuffleInPlace(I.rng),z=I.scale,H=new a;L.forEachNode(function(P,ne){var oe=ne.size?ne.size:1,pe=new a(P,null,null,oe),ae=H;$.forEach(function(se){var Ee=ne[se];ae=ae.getChild(Ee)}),ae.addChild(P,pe)}),_(H,X),i(L,H,M);var B=Y.length,ee,J,K;for(K=0;K<B;K++){var G=Y[K];ee=W+z*M[G].x,J=W+z*M[G].y,j[G]={x:ee,y:J},F&&(L.setNodeAttribute(G,I.attributes.x,ee),L.setNodeAttribute(G,I.attributes.y,J))}return j}var C=R.bind(null,!1);return C.assign=R.bind(null,!0),bw=C,bw}var qTe=jTe();const WTe=jn(qTe);function VTe(e={}){return Sp(WTe,e)}var yw,X7;function KTe(){if(X7)return yw;X7=1;var e=Rg(),t=ho(),r={dimensions:["x","y"],center:.5,scale:1};function n(i,o,s){if(!t(o))throw new Error("graphology-layout/random: the given graph is not a valid graphology instance.");s=e(s,r);var l=s.dimensions;if(!Array.isArray(l)||l.length!==2)throw new Error("graphology-layout/random: given dimensions are invalid.");var u=s.center,d=s.scale,p=Math.PI*2,f=(u-.5)*d,m=o.order,b=l[0],y=l[1];function S(k,_){return _[b]=d*Math.cos(k*p/m)+f,_[y]=d*Math.sin(k*p/m)+f,_}var w=0;if(!i){var E={};return o.forEachNode(function(k){E[k]=S(w++,{})}),E}o.updateEachNodeAttributes(function(k,_){return S(w++,_),_},{attributes:l})}var a=n.bind(null,!1);return a.assign=n.bind(null,!0),yw=a,yw}var YTe=KTe();const JTe=jn(YTe);function XTe(e={}){return Sp(JTe,e)}var Of={},Z7;function w5(){if(Z7)return Of;Z7=1;function e(n){return typeof n!="number"||isNaN(n)?1:n}function t(n,a){var i={},o=function(u){return typeof u>"u"?a:u};typeof a=="function"&&(o=a);var s=function(u){return o(u[n])},l=function(){return o(void 0)};return typeof n=="string"?(i.fromAttributes=s,i.fromGraph=function(u,d){return s(u.getNodeAttributes(d))},i.fromEntry=function(u,d){return s(d)}):typeof n=="function"?(i.fromAttributes=function(){throw new Error("graphology-utils/getters/createNodeValueGetter: irrelevant usage.")},i.fromGraph=function(u,d){return o(n(d,u.getNodeAttributes(d)))},i.fromEntry=function(u,d){return o(n(u,d))}):(i.fromAttributes=l,i.fromGraph=l,i.fromEntry=l),i}function r(n,a){var i={},o=function(u){return typeof u>"u"?a:u};typeof a=="function"&&(o=a);var s=function(u){return o(u[n])},l=function(){return o(void 0)};return typeof n=="string"?(i.fromAttributes=s,i.fromGraph=function(u,d){return s(u.getEdgeAttributes(d))},i.fromEntry=function(u,d){return s(d)},i.fromPartialEntry=i.fromEntry,i.fromMinimalEntry=i.fromEntry):typeof n=="function"?(i.fromAttributes=function(){throw new Error("graphology-utils/getters/createEdgeValueGetter: irrelevant usage.")},i.fromGraph=function(u,d){var p=u.extremities(d);return o(n(d,u.getEdgeAttributes(d),p[0],p[1],u.getNodeAttributes(p[0]),u.getNodeAttributes(p[1]),u.isUndirected(d)))},i.fromEntry=function(u,d,p,f,m,b,y){return o(n(u,d,p,f,m,b,y))},i.fromPartialEntry=function(u,d,p,f){return o(n(u,d,p,f))},i.fromMinimalEntry=function(u,d){return o(n(u,d))}):(i.fromAttributes=l,i.fromGraph=l,i.fromEntry=l,i.fromMinimalEntry=l),i}return Of.createNodeValueGetter=t,Of.createEdgeValueGetter=r,Of.createEdgeWeightGetter=function(n){return r(n,e)},Of}var vw,Q7;function Dee(){if(Q7)return vw;Q7=1;const{createNodeValueGetter:e,createEdgeValueGetter:t}=w5();return vw=function(n,a,i){const{nodeXAttribute:o,nodeYAttribute:s}=i,{attraction:l,repulsion:u,gravity:d,inertia:p,maxMove:f}=i.settings;let{shouldSkipNode:m,shouldSkipEdge:b,isNodeFixed:y}=i;y=e(y),m=e(m,!1),b=t(b,!1);const S=n.filterNodes((k,_)=>!m.fromEntry(k,_)),w=S.length;for(let k=0;k<w;k++){const _=S[k],R=n.getNodeAttributes(_),C=a[_];C?a[_]={dx:C.dx*p,dy:C.dy*p,x:R[o]||0,y:R[s]||0}:a[_]={dx:0,dy:0,x:R[o]||0,y:R[s]||0}}if(u)for(let k=0;k<w;k++){const _=S[k],R=a[_];for(let C=k+1;C<w;C++){const F=S[C],L=a[F],I=L.x-R.x,M=L.y-R.y,j=Math.sqrt(I*I+M*M)||1,Y=u/j*I,W=u/j*M;R.dx-=Y,R.dy-=W,L.dx+=Y,L.dy+=W}}if(l&&n.forEachEdge((k,_,R,C,F,L,I)=>{if(R===C||m.fromEntry(R,F)||m.fromEntry(C,L)||b.fromEntry(k,_,R,C,F,L,I))return;const M=a[R],j=a[C],Y=j.x-M.x,W=j.y-M.y,$=Math.sqrt(Y*Y+W*W)||1,X=l*$*Y,z=l*$*W;M.dx+=X,M.dy+=z,j.dx-=X,j.dy-=z}),d)for(let k=0;k<w;k++){const _=S[k],R=a[_],{x:C,y:F}=R,L=Math.sqrt(C*C+F*F)||1;a[_].dx-=C*d*L,a[_].dy-=F*d*L}const E=!1;for(let k=0;k<w;k++){const _=S[k],R=a[_],C=Math.sqrt(R.dx*R.dx+R.dy*R.dy);C>f&&(R.dx*=f/C,R.dy*=f/C),y.fromGraph(n,_)?R.fixed=!0:(R.x+=R.dx,R.y+=R.dy,R.fixed=!1)}return{converged:E}},vw}var Ub={},eB;function Nee(){return eB||(eB=1,Ub.assignLayoutChanges=function(e,t,r){const{nodeXAttribute:n,nodeYAttribute:a}=r;e.updateEachNodeAttributes((i,o)=>{const s=t[i];return!s||s.fixed||(o[n]=s.x,o[a]=s.y),o},{attributes:["x","y"]})},Ub.collectLayoutChanges=function(e){const t={};for(const r in e){const n=e[r];t[r]={x:n.x,y:n.y}}return t}),Ub}var Sw,tB;function Lee(){return tB||(tB=1,Sw={nodeXAttribute:"x",nodeYAttribute:"y",isNodeFixed:"fixed",shouldSkipNode:null,shouldSkipEdge:null,settings:{attraction:5e-4,repulsion:.1,gravity:1e-4,inertia:.6,maxMove:200}}),Sw}var Ew,rB;function ZTe(){if(rB)return Ew;rB=1;const e=ho(),t=Rg(),r=Dee(),n=Nee(),a=Lee();function i(s,l,u){if(!e(l))throw new Error("graphology-layout-force: the given graph is not a valid graphology instance.");typeof u=="number"?u={maxIterations:u}:u=u||{};const d=u.maxIterations;if(u=t(u,a),typeof d!="number"||d<=0)throw new Error("graphology-layout-force: you should provide a positive number of maximum iterations.");const p={};let f=null,m;for(m=0;m<d&&(f=r(l,p,u),!f.converged);m++);if(s){n.assignLayoutChanges(l,p,u);return}return n.collectLayoutChanges(p)}const o=i.bind(null,!1);return o.assign=i.bind(null,!0),Ew=o,Ew}var QTe=ZTe();const eAe=jn(QTe);var xw,nB;function tAe(){if(nB)return xw;nB=1;const e=ho(),t=Rg(),r=Dee(),n=Nee(),a=Lee();function i(o,s){if(!e(o))throw new Error("graphology-layout-force/worker: the given graph is not a valid graphology instance.");s=t(s,a),this.callbacks={},s.onConverged&&(this.callbacks.onConverged=s.onConverged),this.graph=o,this.params=s,this.nodeStates={},this.frameID=null,this.running=!1,this.killed=!1}return i.prototype.isRunning=function(){return this.running},i.prototype.runFrame=function(){let{converged:o}=r(this.graph,this.nodeStates,this.params);n.assignLayoutChanges(this.graph,this.nodeStates,this.params),o=!1,o?(this.callbacks.onConverged&&this.callbacks.onConverged(),this.stop()):this.frameID=window.requestAnimationFrame(()=>this.runFrame())},i.prototype.stop=function(){return this.running=!1,this.frameID!==null&&(window.cancelAnimationFrame(this.frameID),this.frameID=null),this},i.prototype.start=function(){if(this.killed)throw new Error("graphology-layout-force/worker.start: layout was killed.");this.running||(this.running=!0,this.runFrame())},i.prototype.kill=function(){this.stop(),delete this.nodeStates,this.killed=!0},xw=i,xw}var rAe=tAe();const nAe=jn(rAe);function aAe(e={maxIterations:100}){return Sp(eAe,e)}function iAe(e={}){return x5(nAe,e)}var ww,aB;function oAe(){if(aB)return ww;aB=1;var e=0,t=1,r=2,n=3,a=4,i=5,o=6,s=7,l=8,u=9,d=0,p=1,f=2,m=0,b=1,y=2,S=3,w=4,E=5,k=6,_=7,R=8,C=3,F=10,L=3,I=9,M=10;return ww=function(Y,W,$){var X,z,H,B,ee,J,K,G,P,ne,oe=W.length,pe=$.length,ae=Y.adjustSizes,se=Y.barnesHutTheta*Y.barnesHutTheta,Ee,fe,Me,Oe,xe,Te,Ne,he=[];for(H=0;H<oe;H+=F)W[H+a]=W[H+r],W[H+i]=W[H+n],W[H+r]=0,W[H+n]=0;if(Y.outboundAttractionDistribution){for(Ee=0,H=0;H<oe;H+=F)Ee+=W[H+o];Ee/=oe/F}if(Y.barnesHutOptimize){var Ze=1/0,Et=-1/0,qe=1/0,Ye=-1/0,we,Qt,or;for(H=0;H<oe;H+=F)Ze=Math.min(Ze,W[H+e]),Et=Math.max(Et,W[H+e]),qe=Math.min(qe,W[H+t]),Ye=Math.max(Ye,W[H+t]);var Pe=Et-Ze,rt=Ye-qe;for(Pe>rt?(qe-=(Pe-rt)/2,Ye=qe+Pe):(Ze-=(rt-Pe)/2,Et=Ze+rt),he[0+m]=-1,he[0+b]=(Ze+Et)/2,he[0+y]=(qe+Ye)/2,he[0+S]=Math.max(Et-Ze,Ye-qe),he[0+w]=-1,he[0+E]=-1,he[0+k]=0,he[0+_]=0,he[0+R]=0,X=1,H=0;H<oe;H+=F)for(z=0,or=C;;)if(he[z+E]>=0){W[H+e]<he[z+b]?W[H+t]<he[z+y]?we=he[z+E]:we=he[z+E]+I:W[H+t]<he[z+y]?we=he[z+E]+I*2:we=he[z+E]+I*3,he[z+_]=(he[z+_]*he[z+k]+W[H+e]*W[H+o])/(he[z+k]+W[H+o]),he[z+R]=(he[z+R]*he[z+k]+W[H+t]*W[H+o])/(he[z+k]+W[H+o]),he[z+k]+=W[H+o],z=we;continue}else if(he[z+m]<0){he[z+m]=H;break}else{if(he[z+E]=X*I,G=he[z+S]/2,P=he[z+E],he[P+m]=-1,he[P+b]=he[z+b]-G,he[P+y]=he[z+y]-G,he[P+S]=G,he[P+w]=P+I,he[P+E]=-1,he[P+k]=0,he[P+_]=0,he[P+R]=0,P+=I,he[P+m]=-1,he[P+b]=he[z+b]-G,he[P+y]=he[z+y]+G,he[P+S]=G,he[P+w]=P+I,he[P+E]=-1,he[P+k]=0,he[P+_]=0,he[P+R]=0,P+=I,he[P+m]=-1,he[P+b]=he[z+b]+G,he[P+y]=he[z+y]-G,he[P+S]=G,he[P+w]=P+I,he[P+E]=-1,he[P+k]=0,he[P+_]=0,he[P+R]=0,P+=I,he[P+m]=-1,he[P+b]=he[z+b]+G,he[P+y]=he[z+y]+G,he[P+S]=G,he[P+w]=he[z+w],he[P+E]=-1,he[P+k]=0,he[P+_]=0,he[P+R]=0,X+=4,W[he[z+m]+e]<he[z+b]?W[he[z+m]+t]<he[z+y]?we=he[z+E]:we=he[z+E]+I:W[he[z+m]+t]<he[z+y]?we=he[z+E]+I*2:we=he[z+E]+I*3,he[z+k]=W[he[z+m]+o],he[z+_]=W[he[z+m]+e],he[z+R]=W[he[z+m]+t],he[we+m]=he[z+m],he[z+m]=-1,W[H+e]<he[z+b]?W[H+t]<he[z+y]?Qt=he[z+E]:Qt=he[z+E]+I:W[H+t]<he[z+y]?Qt=he[z+E]+I*2:Qt=he[z+E]+I*3,we===Qt)if(or--){z=we;continue}else{or=C;break}he[Qt+m]=H;break}}if(Y.barnesHutOptimize)for(fe=Y.scalingRatio,H=0;H<oe;H+=F)for(z=0;;)if(he[z+E]>=0)if(Te=Math.pow(W[H+e]-he[z+_],2)+Math.pow(W[H+t]-he[z+R],2),ne=he[z+S],4*ne*ne/Te<se){if(Me=W[H+e]-he[z+_],Oe=W[H+t]-he[z+R],ae===!0?Te>0?(Ne=fe*W[H+o]*he[z+k]/Te,W[H+r]+=Me*Ne,W[H+n]+=Oe*Ne):Te<0&&(Ne=-fe*W[H+o]*he[z+k]/Math.sqrt(Te),W[H+r]+=Me*Ne,W[H+n]+=Oe*Ne):Te>0&&(Ne=fe*W[H+o]*he[z+k]/Te,W[H+r]+=Me*Ne,W[H+n]+=Oe*Ne),z=he[z+w],z<0)break;continue}else{z=he[z+E];continue}else{if(J=he[z+m],J>=0&&J!==H&&(Me=W[H+e]-W[J+e],Oe=W[H+t]-W[J+t],Te=Me*Me+Oe*Oe,ae===!0?Te>0?(Ne=fe*W[H+o]*W[J+o]/Te,W[H+r]+=Me*Ne,W[H+n]+=Oe*Ne):Te<0&&(Ne=-fe*W[H+o]*W[J+o]/Math.sqrt(Te),W[H+r]+=Me*Ne,W[H+n]+=Oe*Ne):Te>0&&(Ne=fe*W[H+o]*W[J+o]/Te,W[H+r]+=Me*Ne,W[H+n]+=Oe*Ne)),z=he[z+w],z<0)break;continue}else for(fe=Y.scalingRatio,B=0;B<oe;B+=F)for(ee=0;ee<B;ee+=F)Me=W[B+e]-W[ee+e],Oe=W[B+t]-W[ee+t],ae===!0?(Te=Math.sqrt(Me*Me+Oe*Oe)-W[B+l]-W[ee+l],Te>0?(Ne=fe*W[B+o]*W[ee+o]/Te/Te,W[B+r]+=Me*Ne,W[B+n]+=Oe*Ne,W[ee+r]-=Me*Ne,W[ee+n]-=Oe*Ne):Te<0&&(Ne=100*fe*W[B+o]*W[ee+o],W[B+r]+=Me*Ne,W[B+n]+=Oe*Ne,W[ee+r]-=Me*Ne,W[ee+n]-=Oe*Ne)):(Te=Math.sqrt(Me*Me+Oe*Oe),Te>0&&(Ne=fe*W[B+o]*W[ee+o]/Te/Te,W[B+r]+=Me*Ne,W[B+n]+=Oe*Ne,W[ee+r]-=Me*Ne,W[ee+n]-=Oe*Ne));for(P=Y.gravity/Y.scalingRatio,fe=Y.scalingRatio,H=0;H<oe;H+=F)Ne=0,Me=W[H+e],Oe=W[H+t],Te=Math.sqrt(Math.pow(Me,2)+Math.pow(Oe,2)),Y.strongGravityMode?Te>0&&(Ne=fe*W[H+o]*P):Te>0&&(Ne=fe*W[H+o]*P/Te),W[H+r]-=Me*Ne,W[H+n]-=Oe*Ne;for(fe=1*(Y.outboundAttractionDistribution?Ee:1),K=0;K<pe;K+=L)B=$[K+d],ee=$[K+p],G=$[K+f],xe=Math.pow(G,Y.edgeWeightInfluence),Me=W[B+e]-W[ee+e],Oe=W[B+t]-W[ee+t],ae===!0?(Te=Math.sqrt(Me*Me+Oe*Oe)-W[B+l]-W[ee+l],Y.linLogMode?Y.outboundAttractionDistribution?Te>0&&(Ne=-fe*xe*Math.log(1+Te)/Te/W[B+o]):Te>0&&(Ne=-fe*xe*Math.log(1+Te)/Te):Y.outboundAttractionDistribution?Te>0&&(Ne=-fe*xe/W[B+o]):Te>0&&(Ne=-fe*xe)):(Te=Math.sqrt(Math.pow(Me,2)+Math.pow(Oe,2)),Y.linLogMode?Y.outboundAttractionDistribution?Te>0&&(Ne=-fe*xe*Math.log(1+Te)/Te/W[B+o]):Te>0&&(Ne=-fe*xe*Math.log(1+Te)/Te):Y.outboundAttractionDistribution?(Te=1,Ne=-fe*xe/W[B+o]):(Te=1,Ne=-fe*xe)),Te>0&&(W[B+r]+=Me*Ne,W[B+n]+=Oe*Ne,W[ee+r]-=Me*Ne,W[ee+n]-=Oe*Ne);var ge,Re,We,ht,xt,sr;if(ae===!0)for(H=0;H<oe;H+=F)W[H+u]!==1&&(ge=Math.sqrt(Math.pow(W[H+r],2)+Math.pow(W[H+n],2)),ge>M&&(W[H+r]=W[H+r]*M/ge,W[H+n]=W[H+n]*M/ge),Re=W[H+o]*Math.sqrt((W[H+a]-W[H+r])*(W[H+a]-W[H+r])+(W[H+i]-W[H+n])*(W[H+i]-W[H+n])),We=Math.sqrt((W[H+a]+W[H+r])*(W[H+a]+W[H+r])+(W[H+i]+W[H+n])*(W[H+i]+W[H+n]))/2,ht=.1*Math.log(1+We)/(1+Math.sqrt(Re)),xt=W[H+e]+W[H+r]*(ht/Y.slowDown),W[H+e]=xt,sr=W[H+t]+W[H+n]*(ht/Y.slowDown),W[H+t]=sr);else for(H=0;H<oe;H+=F)W[H+u]!==1&&(Re=W[H+o]*Math.sqrt((W[H+a]-W[H+r])*(W[H+a]-W[H+r])+(W[H+i]-W[H+n])*(W[H+i]-W[H+n])),We=Math.sqrt((W[H+a]+W[H+r])*(W[H+a]+W[H+r])+(W[H+i]+W[H+n])*(W[H+i]+W[H+n]))/2,ht=W[H+s]*Math.log(1+We)/(1+Math.sqrt(Re)),W[H+s]=Math.min(1,Math.sqrt(ht*(Math.pow(W[H+r],2)+Math.pow(W[H+n],2))/(1+Math.sqrt(Re)))),xt=W[H+e]+W[H+r]*(ht/Y.slowDown),W[H+e]=xt,sr=W[H+t]+W[H+n]*(ht/Y.slowDown),W[H+t]=sr);return{}},ww}var Ws={},iB;function Mee(){if(iB)return Ws;iB=1;var e=10,t=3;return Ws.assign=function(r){r=r||{};var n=Array.prototype.slice.call(arguments).slice(1),a,i,o;for(a=0,o=n.length;a<o;a++)if(n[a])for(i in n[a])r[i]=n[a][i];return r},Ws.validateSettings=function(r){return"linLogMode"in r&&typeof r.linLogMode!="boolean"?{message:"the `linLogMode` setting should be a boolean."}:"outboundAttractionDistribution"in r&&typeof r.outboundAttractionDistribution!="boolean"?{message:"the `outboundAttractionDistribution` setting should be a boolean."}:"adjustSizes"in r&&typeof r.adjustSizes!="boolean"?{message:"the `adjustSizes` setting should be a boolean."}:"edgeWeightInfluence"in r&&typeof r.edgeWeightInfluence!="number"?{message:"the `edgeWeightInfluence` setting should be a number."}:"scalingRatio"in r&&!(typeof r.scalingRatio=="number"&&r.scalingRatio>=0)?{message:"the `scalingRatio` setting should be a number >= 0."}:"strongGravityMode"in r&&typeof r.strongGravityMode!="boolean"?{message:"the `strongGravityMode` setting should be a boolean."}:"gravity"in r&&!(typeof r.gravity=="number"&&r.gravity>=0)?{message:"the `gravity` setting should be a number >= 0."}:"slowDown"in r&&!(typeof r.slowDown=="number"||r.slowDown>=0)?{message:"the `slowDown` setting should be a number >= 0."}:"barnesHutOptimize"in r&&typeof r.barnesHutOptimize!="boolean"?{message:"the `barnesHutOptimize` setting should be a boolean."}:"barnesHutTheta"in r&&!(typeof r.barnesHutTheta=="number"&&r.barnesHutTheta>=0)?{message:"the `barnesHutTheta` setting should be a number >= 0."}:null},Ws.graphToByteArrays=function(r,n){var a=r.order,i=r.size,o={},s,l=new Float32Array(a*e),u=new Float32Array(i*t);return s=0,r.forEachNode(function(d,p){o[d]=s,l[s]=p.x,l[s+1]=p.y,l[s+2]=0,l[s+3]=0,l[s+4]=0,l[s+5]=0,l[s+6]=1,l[s+7]=1,l[s+8]=p.size||1,l[s+9]=p.fixed?1:0,s+=e}),s=0,r.forEachEdge(function(d,p,f,m,b,y,S){var w=o[f],E=o[m],k=n(d,p,f,m,b,y,S);l[w+6]+=k,l[E+6]+=k,u[s]=w,u[s+1]=E,u[s+2]=k,s+=t}),{nodes:l,edges:u}},Ws.assignLayoutChanges=function(r,n,a){var i=0;r.updateEachNodeAttributes(function(o,s){return s.x=n[i],s.y=n[i+1],i+=e,a?a(o,s):s})},Ws.readGraphPositions=function(r,n){var a=0;r.forEachNode(function(i,o){n[a]=o.x,n[a+1]=o.y,a+=e})},Ws.collectLayoutChanges=function(r,n,a){for(var i=r.nodes(),o={},s=0,l=0,u=n.length;s<u;s+=e){if(a){var d=Object.assign({},r.getNodeAttributes(i[l]));d.x=n[s],d.y=n[s+1],d=a(i[l],d),o[i[l]]={x:d.x,y:d.y}}else o[i[l]]={x:n[s],y:n[s+1]};l++}return o},Ws.createWorker=function(n){var a=window.URL||window.webkitURL,i=n.toString(),o=a.createObjectURL(new Blob(["("+i+").call(this);"],{type:"text/javascript"})),s=new Worker(o);return a.revokeObjectURL(o),s},Ws}var kw,oB;function Iee(){return oB||(oB=1,kw={linLogMode:!1,outboundAttractionDistribution:!1,adjustSizes:!1,edgeWeightInfluence:1,scalingRatio:1,strongGravityMode:!1,gravity:1,slowDown:1,barnesHutOptimize:!1,barnesHutTheta:.5}),kw}var Cw,sB;function sAe(){if(sB)return Cw;sB=1;var e=ho(),t=w5().createEdgeWeightGetter,r=oAe(),n=Mee(),a=Iee();function i(l,u,d){if(!e(u))throw new Error("graphology-layout-forceatlas2: the given graph is not a valid graphology instance.");typeof d=="number"&&(d={iterations:d});var p=d.iterations;if(typeof p!="number")throw new Error("graphology-layout-forceatlas2: invalid number of iterations.");if(p<=0)throw new Error("graphology-layout-forceatlas2: you should provide a positive number of iterations.");var f=t("getEdgeWeight"in d?d.getEdgeWeight:"weight").fromEntry,m=typeof d.outputReducer=="function"?d.outputReducer:null,b=n.assign({},a,d.settings),y=n.validateSettings(b);if(y)throw new Error("graphology-layout-forceatlas2: "+y.message);var S=n.graphToByteArrays(u,f),w;for(w=0;w<p;w++)r(b,S.nodes,S.edges);if(l){n.assignLayoutChanges(u,S.nodes,m);return}return n.collectLayoutChanges(u,S.nodes)}function o(l){var u=typeof l=="number"?l:l.order;return{barnesHutOptimize:u>2e3,strongGravityMode:!0,gravity:.05,scalingRatio:10,slowDown:1+Math.log(u)}}var s=i.bind(null,!1);return s.assign=i.bind(null,!0),s.inferSettings=o,Cw=s,Cw}var lAe=sAe();const uAe=jn(lAe);var Tw,lB;function cAe(){return lB||(lB=1,Tw=function(){var t,r,n={};(function(){var i=0,o=1,s=2,l=3,u=4,d=5,p=6,f=7,m=8,b=9,y=0,S=1,w=2,E=0,k=1,_=2,R=3,C=4,F=5,L=6,I=7,M=8,j=3,Y=10,W=3,$=9,X=10;n.exports=function(H,B,ee){var J,K,G,P,ne,oe,pe,ae,se,Ee,fe=B.length,Me=ee.length,Oe=H.adjustSizes,xe=H.barnesHutTheta*H.barnesHutTheta,Te,Ne,he,Ze,Et,qe,Ye,we=[];for(G=0;G<fe;G+=Y)B[G+u]=B[G+s],B[G+d]=B[G+l],B[G+s]=0,B[G+l]=0;if(H.outboundAttractionDistribution){for(Te=0,G=0;G<fe;G+=Y)Te+=B[G+p];Te/=fe/Y}if(H.barnesHutOptimize){var Qt=1/0,or=-1/0,Pe=1/0,rt=-1/0,ge,Re,We;for(G=0;G<fe;G+=Y)Qt=Math.min(Qt,B[G+i]),or=Math.max(or,B[G+i]),Pe=Math.min(Pe,B[G+o]),rt=Math.max(rt,B[G+o]);var ht=or-Qt,xt=rt-Pe;for(ht>xt?(Pe-=(ht-xt)/2,rt=Pe+ht):(Qt-=(xt-ht)/2,or=Qt+xt),we[0+E]=-1,we[0+k]=(Qt+or)/2,we[0+_]=(Pe+rt)/2,we[0+R]=Math.max(or-Qt,rt-Pe),we[0+C]=-1,we[0+F]=-1,we[0+L]=0,we[0+I]=0,we[0+M]=0,J=1,G=0;G<fe;G+=Y)for(K=0,We=j;;)if(we[K+F]>=0){B[G+i]<we[K+k]?B[G+o]<we[K+_]?ge=we[K+F]:ge=we[K+F]+$:B[G+o]<we[K+_]?ge=we[K+F]+$*2:ge=we[K+F]+$*3,we[K+I]=(we[K+I]*we[K+L]+B[G+i]*B[G+p])/(we[K+L]+B[G+p]),we[K+M]=(we[K+M]*we[K+L]+B[G+o]*B[G+p])/(we[K+L]+B[G+p]),we[K+L]+=B[G+p],K=ge;continue}else if(we[K+E]<0){we[K+E]=G;break}else{if(we[K+F]=J*$,ae=we[K+R]/2,se=we[K+F],we[se+E]=-1,we[se+k]=we[K+k]-ae,we[se+_]=we[K+_]-ae,we[se+R]=ae,we[se+C]=se+$,we[se+F]=-1,we[se+L]=0,we[se+I]=0,we[se+M]=0,se+=$,we[se+E]=-1,we[se+k]=we[K+k]-ae,we[se+_]=we[K+_]+ae,we[se+R]=ae,we[se+C]=se+$,we[se+F]=-1,we[se+L]=0,we[se+I]=0,we[se+M]=0,se+=$,we[se+E]=-1,we[se+k]=we[K+k]+ae,we[se+_]=we[K+_]-ae,we[se+R]=ae,we[se+C]=se+$,we[se+F]=-1,we[se+L]=0,we[se+I]=0,we[se+M]=0,se+=$,we[se+E]=-1,we[se+k]=we[K+k]+ae,we[se+_]=we[K+_]+ae,we[se+R]=ae,we[se+C]=we[K+C],we[se+F]=-1,we[se+L]=0,we[se+I]=0,we[se+M]=0,J+=4,B[we[K+E]+i]<we[K+k]?B[we[K+E]+o]<we[K+_]?ge=we[K+F]:ge=we[K+F]+$:B[we[K+E]+o]<we[K+_]?ge=we[K+F]+$*2:ge=we[K+F]+$*3,we[K+L]=B[we[K+E]+p],we[K+I]=B[we[K+E]+i],we[K+M]=B[we[K+E]+o],we[ge+E]=we[K+E],we[K+E]=-1,B[G+i]<we[K+k]?B[G+o]<we[K+_]?Re=we[K+F]:Re=we[K+F]+$:B[G+o]<we[K+_]?Re=we[K+F]+$*2:Re=we[K+F]+$*3,ge===Re)if(We--){K=ge;continue}else{We=j;break}we[Re+E]=G;break}}if(H.barnesHutOptimize)for(Ne=H.scalingRatio,G=0;G<fe;G+=Y)for(K=0;;)if(we[K+F]>=0)if(qe=Math.pow(B[G+i]-we[K+I],2)+Math.pow(B[G+o]-we[K+M],2),Ee=we[K+R],4*Ee*Ee/qe<xe){if(he=B[G+i]-we[K+I],Ze=B[G+o]-we[K+M],Oe===!0?qe>0?(Ye=Ne*B[G+p]*we[K+L]/qe,B[G+s]+=he*Ye,B[G+l]+=Ze*Ye):qe<0&&(Ye=-Ne*B[G+p]*we[K+L]/Math.sqrt(qe),B[G+s]+=he*Ye,B[G+l]+=Ze*Ye):qe>0&&(Ye=Ne*B[G+p]*we[K+L]/qe,B[G+s]+=he*Ye,B[G+l]+=Ze*Ye),K=we[K+C],K<0)break;continue}else{K=we[K+F];continue}else{if(oe=we[K+E],oe>=0&&oe!==G&&(he=B[G+i]-B[oe+i],Ze=B[G+o]-B[oe+o],qe=he*he+Ze*Ze,Oe===!0?qe>0?(Ye=Ne*B[G+p]*B[oe+p]/qe,B[G+s]+=he*Ye,B[G+l]+=Ze*Ye):qe<0&&(Ye=-Ne*B[G+p]*B[oe+p]/Math.sqrt(qe),B[G+s]+=he*Ye,B[G+l]+=Ze*Ye):qe>0&&(Ye=Ne*B[G+p]*B[oe+p]/qe,B[G+s]+=he*Ye,B[G+l]+=Ze*Ye)),K=we[K+C],K<0)break;continue}else for(Ne=H.scalingRatio,P=0;P<fe;P+=Y)for(ne=0;ne<P;ne+=Y)he=B[P+i]-B[ne+i],Ze=B[P+o]-B[ne+o],Oe===!0?(qe=Math.sqrt(he*he+Ze*Ze)-B[P+m]-B[ne+m],qe>0?(Ye=Ne*B[P+p]*B[ne+p]/qe/qe,B[P+s]+=he*Ye,B[P+l]+=Ze*Ye,B[ne+s]-=he*Ye,B[ne+l]-=Ze*Ye):qe<0&&(Ye=100*Ne*B[P+p]*B[ne+p],B[P+s]+=he*Ye,B[P+l]+=Ze*Ye,B[ne+s]-=he*Ye,B[ne+l]-=Ze*Ye)):(qe=Math.sqrt(he*he+Ze*Ze),qe>0&&(Ye=Ne*B[P+p]*B[ne+p]/qe/qe,B[P+s]+=he*Ye,B[P+l]+=Ze*Ye,B[ne+s]-=he*Ye,B[ne+l]-=Ze*Ye));for(se=H.gravity/H.scalingRatio,Ne=H.scalingRatio,G=0;G<fe;G+=Y)Ye=0,he=B[G+i],Ze=B[G+o],qe=Math.sqrt(Math.pow(he,2)+Math.pow(Ze,2)),H.strongGravityMode?qe>0&&(Ye=Ne*B[G+p]*se):qe>0&&(Ye=Ne*B[G+p]*se/qe),B[G+s]-=he*Ye,B[G+l]-=Ze*Ye;for(Ne=1*(H.outboundAttractionDistribution?Te:1),pe=0;pe<Me;pe+=W)P=ee[pe+y],ne=ee[pe+S],ae=ee[pe+w],Et=Math.pow(ae,H.edgeWeightInfluence),he=B[P+i]-B[ne+i],Ze=B[P+o]-B[ne+o],Oe===!0?(qe=Math.sqrt(he*he+Ze*Ze)-B[P+m]-B[ne+m],H.linLogMode?H.outboundAttractionDistribution?qe>0&&(Ye=-Ne*Et*Math.log(1+qe)/qe/B[P+p]):qe>0&&(Ye=-Ne*Et*Math.log(1+qe)/qe):H.outboundAttractionDistribution?qe>0&&(Ye=-Ne*Et/B[P+p]):qe>0&&(Ye=-Ne*Et)):(qe=Math.sqrt(Math.pow(he,2)+Math.pow(Ze,2)),H.linLogMode?H.outboundAttractionDistribution?qe>0&&(Ye=-Ne*Et*Math.log(1+qe)/qe/B[P+p]):qe>0&&(Ye=-Ne*Et*Math.log(1+qe)/qe):H.outboundAttractionDistribution?(qe=1,Ye=-Ne*Et/B[P+p]):(qe=1,Ye=-Ne*Et)),qe>0&&(B[P+s]+=he*Ye,B[P+l]+=Ze*Ye,B[ne+s]-=he*Ye,B[ne+l]-=Ze*Ye);var sr,ar,fr,Mt,lr,qt;if(Oe===!0)for(G=0;G<fe;G+=Y)B[G+b]!==1&&(sr=Math.sqrt(Math.pow(B[G+s],2)+Math.pow(B[G+l],2)),sr>X&&(B[G+s]=B[G+s]*X/sr,B[G+l]=B[G+l]*X/sr),ar=B[G+p]*Math.sqrt((B[G+u]-B[G+s])*(B[G+u]-B[G+s])+(B[G+d]-B[G+l])*(B[G+d]-B[G+l])),fr=Math.sqrt((B[G+u]+B[G+s])*(B[G+u]+B[G+s])+(B[G+d]+B[G+l])*(B[G+d]+B[G+l]))/2,Mt=.1*Math.log(1+fr)/(1+Math.sqrt(ar)),lr=B[G+i]+B[G+s]*(Mt/H.slowDown),B[G+i]=lr,qt=B[G+o]+B[G+l]*(Mt/H.slowDown),B[G+o]=qt);else for(G=0;G<fe;G+=Y)B[G+b]!==1&&(ar=B[G+p]*Math.sqrt((B[G+u]-B[G+s])*(B[G+u]-B[G+s])+(B[G+d]-B[G+l])*(B[G+d]-B[G+l])),fr=Math.sqrt((B[G+u]+B[G+s])*(B[G+u]+B[G+s])+(B[G+d]+B[G+l])*(B[G+d]+B[G+l]))/2,Mt=B[G+f]*Math.log(1+fr)/(1+Math.sqrt(ar)),B[G+f]=Math.min(1,Math.sqrt(Mt*(Math.pow(B[G+s],2)+Math.pow(B[G+l],2))/(1+Math.sqrt(ar)))),lr=B[G+i]+B[G+s]*(Mt/H.slowDown),B[G+i]=lr,qt=B[G+o]+B[G+l]*(Mt/H.slowDown),B[G+o]=qt);return{}}})();var a=n.exports;self.addEventListener("message",function(i){var o=i.data;t=new Float32Array(o.nodes),o.edges&&(r=new Float32Array(o.edges)),a(o.settings,t,r),self.postMessage({nodes:t.buffer},[t.buffer])})}),Tw}var Aw,uB;function dAe(){if(uB)return Aw;uB=1;var e=cAe(),t=ho(),r=w5().createEdgeWeightGetter,n=Mee(),a=Iee();function i(o,s){if(s=s||{},!t(o))throw new Error("graphology-layout-forceatlas2/worker: the given graph is not a valid graphology instance.");var l=r("getEdgeWeight"in s?s.getEdgeWeight:"weight").fromEntry,u=n.assign({},a,s.settings),d=n.validateSettings(u);if(d)throw new Error("graphology-layout-forceatlas2/worker: "+d.message);this.worker=null,this.graph=o,this.settings=u,this.getEdgeWeight=l,this.matrices=null,this.running=!1,this.killed=!1,this.outputReducer=typeof s.outputReducer=="function"?s.outputReducer:null,this.handleMessage=this.handleMessage.bind(this);var p=void 0,f=this;this.handleGraphUpdate=function(){f.worker&&f.worker.terminate(),p&&clearTimeout(p),p=setTimeout(function(){p=void 0,f.spawnWorker()},0)},o.on("nodeAdded",this.handleGraphUpdate),o.on("edgeAdded",this.handleGraphUpdate),o.on("nodeDropped",this.handleGraphUpdate),o.on("edgeDropped",this.handleGraphUpdate),this.spawnWorker()}return i.prototype.isRunning=function(){return this.running},i.prototype.spawnWorker=function(){this.worker&&this.worker.terminate(),this.worker=n.createWorker(e),this.worker.addEventListener("message",this.handleMessage),this.running&&(this.running=!1,this.start())},i.prototype.handleMessage=function(o){if(this.running){var s=new Float32Array(o.data.nodes);n.assignLayoutChanges(this.graph,s,this.outputReducer),this.outputReducer&&n.readGraphPositions(this.graph,s),this.matrices.nodes=s,this.askForIterations()}},i.prototype.askForIterations=function(o){var s=this.matrices,l={settings:this.settings,nodes:s.nodes.buffer},u=[s.nodes.buffer];return o&&(l.edges=s.edges.buffer,u.push(s.edges.buffer)),this.worker.postMessage(l,u),this},i.prototype.start=function(){if(this.killed)throw new Error("graphology-layout-forceatlas2/worker.start: layout was killed.");return this.running?this:(this.matrices=n.graphToByteArrays(this.graph,this.getEdgeWeight),this.running=!0,this.askForIterations(!0),this)},i.prototype.stop=function(){return this.running=!1,this},i.prototype.kill=function(){if(this.killed)return this;this.running=!1,this.killed=!0,this.matrices=null,this.worker.terminate(),this.graph.removeListener("nodeAdded",this.handleGraphUpdate),this.graph.removeListener("edgeAdded",this.handleGraphUpdate),this.graph.removeListener("nodeDropped",this.handleGraphUpdate),this.graph.removeListener("edgeDropped",this.handleGraphUpdate)},Aw=i,Aw}var hAe=dAe();const pAe=jn(hAe);function Oee(e={iterations:100}){return Sp(uAe,e)}function fAe(e={}){return x5(pAe,e)}var Fw,cB;function mAe(){if(cB)return Fw;cB=1;var e=0,t=1,r=2,n=3;function a(o,s){return o+"§"+s}function i(){return .01*(.5-Math.random())}return Fw=function(s,l){var u=s.margin,d=s.ratio,p=s.expansion,f=s.gridSize,m=s.speed,b,y,S,w,E,k,_=!0,R=l.length,C=R/n|0,F=new Float32Array(C),L=new Float32Array(C),I=1/0,M=1/0,j=-1/0,Y=-1/0;for(b=0;b<R;b+=n)S=l[b+e],w=l[b+t],k=l[b+r]*d+u,I=Math.min(I,S-k),j=Math.max(j,S+k),M=Math.min(M,w-k),Y=Math.max(Y,w+k);var W=j-I,$=Y-M,X=(I+j)/2,z=(M+Y)/2;I=X-p*W/2,j=X+p*W/2,M=z-p*$/2,Y=z+p*$/2;var H=new Array(f*f),B=H.length,ee;for(ee=0;ee<B;ee++)H[ee]=[];var J,K,G,P,ne,oe,pe,ae,se,Ee;for(b=0;b<R;b+=n)for(S=l[b+e],w=l[b+t],k=l[b+r]*d+u,J=S-k,K=S+k,G=w-k,P=w+k,ne=Math.floor(f*(J-I)/(j-I)),oe=Math.floor(f*(K-I)/(j-I)),pe=Math.floor(f*(G-M)/(Y-M)),ae=Math.floor(f*(P-M)/(Y-M)),se=ne;se<=oe;se++)for(Ee=pe;Ee<=ae;Ee++)H[se*f+Ee].push(b);var fe,Me=new Set,Oe,xe,Te,Ne,he,Ze,Et,qe,Ye,we,Qt,or,Pe;for(ee=0;ee<B;ee++)for(fe=H[ee],b=0,E=fe.length;b<E;b++)for(Oe=fe[b],Te=l[Oe+e],he=l[Oe+t],Et=l[Oe+r],y=b+1;y<E;y++)xe=fe[y],Ye=a(Oe,xe),!(B>1&&Me.has(Ye))&&(B>1&&Me.add(Ye),Ne=l[xe+e],Ze=l[xe+t],qe=l[xe+r],we=Ne-Te,Qt=Ze-he,or=Math.sqrt(we*we+Qt*Qt),Pe=or<Et*d+u+(qe*d+u),Pe&&(_=!1,xe=xe/n|0,or>0?(F[xe]+=we/or*(1+Et),L[xe]+=Qt/or*(1+Et)):(F[xe]+=W*i(),L[xe]+=$*i())));for(b=0,y=0;b<R;b+=n,y++)l[b+e]+=F[y]*.1*m,l[b+t]+=L[y]*.1*m;return{converged:_}},Fw}var hc={},dB;function Bee(){if(dB)return hc;dB=1;var e=3;return hc.validateSettings=function(t){return"gridSize"in t&&typeof t.gridSize!="number"||t.gridSize<=0?{message:"the `gridSize` setting should be a positive number."}:"margin"in t&&typeof t.margin!="number"||t.margin<0?{message:"the `margin` setting should be 0 or a positive number."}:"expansion"in t&&typeof t.expansion!="number"||t.expansion<=0?{message:"the `expansion` setting should be a positive number."}:"ratio"in t&&typeof t.ratio!="number"||t.ratio<=0?{message:"the `ratio` setting should be a positive number."}:"speed"in t&&typeof t.speed!="number"||t.speed<=0?{message:"the `speed` setting should be a positive number."}:null},hc.graphToByteArray=function(t,r){var n=t.order,a=new Float32Array(n*e),i=0;return t.forEachNode(function(o,s){typeof r=="function"&&(s=r(o,s)),a[i]=s.x,a[i+1]=s.y,a[i+2]=s.size||1,i+=e}),a},hc.assignLayoutChanges=function(t,r,n){var a=0;t.forEachNode(function(i){var o={x:r[a],y:r[a+1]};typeof n=="function"&&(o=n(i,o)),t.mergeNodeAttributes(i,o),a+=e})},hc.collectLayoutChanges=function(t,r,n){var a={},i=0;return t.forEachNode(function(o){var s={x:r[i],y:r[i+1]};typeof n=="function"&&(s=n(o,s)),a[o]=s,i+=e}),a},hc.createWorker=function(r){var n=window.URL||window.webkitURL,a=r.toString(),i=n.createObjectURL(new Blob(["("+a+").call(this);"],{type:"text/javascript"})),o=new Worker(i);return n.revokeObjectURL(i),o},hc}var _w,hB;function Pee(){return hB||(hB=1,_w={gridSize:20,margin:5,expansion:1.1,ratio:1,speed:3}),_w}var Rw,pB;function gAe(){if(pB)return Rw;pB=1;var e=ho(),t=mAe(),r=Bee(),n=Pee(),a=500;function i(s,l,u){if(!e(l))throw new Error("graphology-layout-noverlap: the given graph is not a valid graphology instance.");typeof u=="number"?u={maxIterations:u}:u=u||{};var d=u.maxIterations||a;if(typeof d!="number"||d<=0)throw new Error("graphology-layout-force: you should provide a positive number of maximum iterations.");var p=Object.assign({},n,u.settings),f=r.validateSettings(p);if(f)throw new Error("graphology-layout-noverlap: "+f.message);var m=r.graphToByteArray(l,u.inputReducer),b=!1,y;for(y=0;y<d&&!b;y++)b=t(p,m).converged;if(s){r.assignLayoutChanges(l,m,u.outputReducer);return}return r.collectLayoutChanges(l,m,u.outputReducer)}var o=i.bind(null,!1);return o.assign=i.bind(null,!0),Rw=o,Rw}var bAe=gAe();const yAe=jn(bAe);var Dw,fB;function vAe(){return fB||(fB=1,Dw=function(){var t,r={};(function(){var a=0,i=1,o=2,s=3;function l(d,p){return d+"§"+p}function u(){return .01*(.5-Math.random())}r.exports=function(p,f){var m=p.margin,b=p.ratio,y=p.expansion,S=p.gridSize,w=p.speed,E,k,_,R,C,F,L=!0,I=f.length,M=I/s|0,j=new Float32Array(M),Y=new Float32Array(M),W=1/0,$=1/0,X=-1/0,z=-1/0;for(E=0;E<I;E+=s)_=f[E+a],R=f[E+i],F=f[E+o]*b+m,W=Math.min(W,_-F),X=Math.max(X,_+F),$=Math.min($,R-F),z=Math.max(z,R+F);var H=X-W,B=z-$,ee=(W+X)/2,J=($+z)/2;W=ee-y*H/2,X=ee+y*H/2,$=J-y*B/2,z=J+y*B/2;var K=new Array(S*S),G=K.length,P;for(P=0;P<G;P++)K[P]=[];var ne,oe,pe,ae,se,Ee,fe,Me,Oe,xe;for(E=0;E<I;E+=s)for(_=f[E+a],R=f[E+i],F=f[E+o]*b+m,ne=_-F,oe=_+F,pe=R-F,ae=R+F,se=Math.floor(S*(ne-W)/(X-W)),Ee=Math.floor(S*(oe-W)/(X-W)),fe=Math.floor(S*(pe-$)/(z-$)),Me=Math.floor(S*(ae-$)/(z-$)),Oe=se;Oe<=Ee;Oe++)for(xe=fe;xe<=Me;xe++)K[Oe*S+xe].push(E);var Te,Ne=new Set,he,Ze,Et,qe,Ye,we,Qt,or,Pe,rt,ge,Re,We;for(P=0;P<G;P++)for(Te=K[P],E=0,C=Te.length;E<C;E++)for(he=Te[E],Et=f[he+a],Ye=f[he+i],Qt=f[he+o],k=E+1;k<C;k++)Ze=Te[k],Pe=l(he,Ze),!(G>1&&Ne.has(Pe))&&(G>1&&Ne.add(Pe),qe=f[Ze+a],we=f[Ze+i],or=f[Ze+o],rt=qe-Et,ge=we-Ye,Re=Math.sqrt(rt*rt+ge*ge),We=Re<Qt*b+m+(or*b+m),We&&(L=!1,Ze=Ze/s|0,Re>0?(j[Ze]+=rt/Re*(1+Qt),Y[Ze]+=ge/Re*(1+Qt)):(j[Ze]+=H*u(),Y[Ze]+=B*u())));for(E=0,k=0;E<I;E+=s,k++)f[E+a]+=j[k]*.1*w,f[E+i]+=Y[k]*.1*w;return{converged:L}}})();var n=r.exports;self.addEventListener("message",function(a){var i=a.data;t=new Float32Array(i.nodes);var o=n(i.settings,t);self.postMessage({result:o,nodes:t.buffer},[t.buffer])})}),Dw}var Nw,mB;function SAe(){if(mB)return Nw;mB=1;var e=vAe(),t=ho(),r=Bee(),n=Pee();function a(i,o){if(o=o||{},!t(i))throw new Error("graphology-layout-noverlap/worker: the given graph is not a valid graphology instance.");var s=Object.assign({},n,o.settings),l=r.validateSettings(s);if(l)throw new Error("graphology-layout-noverlap/worker: "+l.message);this.worker=null,this.graph=i,this.settings=s,this.matrices=null,this.running=!1,this.killed=!1,this.inputReducer=o.inputReducer,this.outputReducer=o.outputReducer,this.callbacks={onConverged:typeof o.onConverged=="function"?o.onConverged:null},this.handleMessage=this.handleMessage.bind(this);var u=!1,d=this;this.handleAddition=function(){u||(u=!0,d.spawnWorker(),setTimeout(function(){u=!1},0))},i.on("nodeAdded",this.handleAddition),i.on("edgeAdded",this.handleAddition),this.spawnWorker()}return a.prototype.isRunning=function(){return this.running},a.prototype.spawnWorker=function(){this.worker&&this.worker.terminate(),this.worker=r.createWorker(e),this.worker.addEventListener("message",this.handleMessage),this.running&&(this.running=!1,this.start())},a.prototype.handleMessage=function(i){if(this.running){var o=new Float32Array(i.data.nodes);if(r.assignLayoutChanges(this.graph,o,this.outputReducer),this.matrices.nodes=o,i.data.result.converged){this.callbacks.onConverged&&this.callbacks.onConverged(),this.stop();return}this.askForIterations()}},a.prototype.askForIterations=function(){var i=this.matrices,o={settings:this.settings,nodes:i.nodes.buffer},s=[i.nodes.buffer];return this.worker.postMessage(o,s),this},a.prototype.start=function(){if(this.killed)throw new Error("graphology-layout-noverlap/worker.start: layout was killed.");return this.running?this:(this.matrices={nodes:r.graphToByteArray(this.graph,this.inputReducer)},this.running=!0,this.askForIterations(),this)},a.prototype.stop=function(){return this.running=!1,this},a.prototype.kill=function(){if(this.killed)return this;this.running=!1,this.killed=!0,this.matrices=null,this.worker.terminate(),this.graph.removeListener("nodeAdded",this.handleAddition),this.graph.removeListener("edgeAdded",this.handleAddition)},Nw=a,Nw}var EAe=SAe();const xAe=jn(EAe);function wAe(e={}){return Sp(yAe,e)}function kAe(e={}){return x5(xAe,e)}var Lw,gB;function CAe(){if(gB)return Lw;gB=1;var e=Rg(),t=ho(),r={dimensions:["x","y"],center:.5,rng:Math.random,scale:1};function n(i,o,s){if(!t(o))throw new Error("graphology-layout/random: the given graph is not a valid graphology instance.");s=e(s,r);var l=s.dimensions;if(!Array.isArray(l)||l.length<1)throw new Error("graphology-layout/random: given dimensions are invalid.");var u=l.length,d=s.center,p=s.rng,f=s.scale,m=(d-.5)*f;function b(S){for(var w=0;w<u;w++)S[l[w]]=p()*f+m;return S}if(!i){var y={};return o.forEachNode(function(S){y[S]=b({})}),y}o.updateEachNodeAttributes(function(S,w){return b(w),w},{attributes:l})}var a=n.bind(null,!1);return a.assign=n.bind(null,!0),Lw=a,Lw}var TAe=CAe();const AAe=jn(TAe);function FAe(e={}){return Sp(AAe,e)}var bB=1,_Ae=.9,RAe=.8,DAe=.17,Mw=.1,Iw=.999,NAe=.9999,LAe=.99,MAe=/[\\\/_+.#"@\[\(\{&]/,IAe=/[\\\/_+.#"@\[\(\{&]/g,OAe=/[\s-]/,zee=/[\s-]/g;function ER(e,t,r,n,a,i,o){if(i===t.length)return a===e.length?bB:LAe;var s=`${a},${i}`;if(o[s]!==void 0)return o[s];for(var l=n.charAt(i),u=r.indexOf(l,a),d=0,p,f,m,b;u>=0;)p=ER(e,t,r,n,u+1,i+1,o),p>d&&(u===a?p*=bB:MAe.test(e.charAt(u-1))?(p*=RAe,m=e.slice(a,u-1).match(IAe),m&&a>0&&(p*=Math.pow(Iw,m.length))):OAe.test(e.charAt(u-1))?(p*=_Ae,b=e.slice(a,u-1).match(zee),b&&a>0&&(p*=Math.pow(Iw,b.length))):(p*=DAe,a>0&&(p*=Math.pow(Iw,u-a))),e.charAt(u)!==t.charAt(i)&&(p*=NAe)),(p<Mw&&r.charAt(u-1)===n.charAt(i+1)||n.charAt(i+1)===n.charAt(i)&&r.charAt(u-1)!==n.charAt(i))&&(f=ER(e,t,r,n,u+1,i+2,o),f*Mw>p&&(p=f*Mw)),p>d&&(d=p),u=r.indexOf(l,u+1);return o[s]=d,d}function yB(e){return e.toLowerCase().replace(zee," ")}function BAe(e,t,r){return e=r&&r.length>0?`${e+" "+r.join(" ")}`:e,ER(e,t,yB(e),yB(t),0,0,{})}var Bf='[cmdk-group=""]',Ow='[cmdk-group-items=""]',PAe='[cmdk-group-heading=""]',Hee='[cmdk-item=""]',vB=`${Hee}:not([aria-disabled="true"])`,xR="cmdk-item-select",ch="data-value",zAe=(e,t,r)=>BAe(e,t,r),Uee=T.createContext(void 0),Dg=()=>T.useContext(Uee),$ee=T.createContext(void 0),k5=()=>T.useContext($ee),Gee=T.createContext(void 0),jee=T.forwardRef((e,t)=>{let r=dh(()=>{var G,P;return{search:"",value:(P=(G=e.value)!=null?G:e.defaultValue)!=null?P:"",selectedItemId:void 0,filtered:{count:0,items:new Map,groups:new Set}}}),n=dh(()=>new Set),a=dh(()=>new Map),i=dh(()=>new Map),o=dh(()=>new Set),s=qee(e),{label:l,children:u,value:d,onValueChange:p,filter:f,shouldFilter:m,loop:b,disablePointerSelection:y=!1,vimBindings:S=!0,...w}=e,E=La(),k=La(),_=La(),R=T.useRef(null),C=JAe();Gc(()=>{if(d!==void 0){let G=d.trim();r.current.value=G,F.emit()}},[d]),Gc(()=>{C(6,W)},[]);let F=T.useMemo(()=>({subscribe:G=>(o.current.add(G),()=>o.current.delete(G)),snapshot:()=>r.current,setState:(G,P,ne)=>{var oe,pe,ae,se;if(!Object.is(r.current[G],P)){if(r.current[G]=P,G==="search")Y(),M(),C(1,j);else if(G==="value"){if(document.activeElement.hasAttribute("cmdk-input")||document.activeElement.hasAttribute("cmdk-root")){let Ee=document.getElementById(_);Ee?Ee.focus():(oe=document.getElementById(E))==null||oe.focus()}if(C(7,()=>{var Ee;r.current.selectedItemId=(Ee=$())==null?void 0:Ee.id,F.emit()}),ne||C(5,W),((pe=s.current)==null?void 0:pe.value)!==void 0){let Ee=P??"";(se=(ae=s.current).onValueChange)==null||se.call(ae,Ee);return}}F.emit()}},emit:()=>{o.current.forEach(G=>G())}}),[]),L=T.useMemo(()=>({value:(G,P,ne)=>{var oe;P!==((oe=i.current.get(G))==null?void 0:oe.value)&&(i.current.set(G,{value:P,keywords:ne}),r.current.filtered.items.set(G,I(P,ne)),C(2,()=>{M(),F.emit()}))},item:(G,P)=>(n.current.add(G),P&&(a.current.has(P)?a.current.get(P).add(G):a.current.set(P,new Set([G]))),C(3,()=>{Y(),M(),r.current.value||j(),F.emit()}),()=>{i.current.delete(G),n.current.delete(G),r.current.filtered.items.delete(G);let ne=$();C(4,()=>{Y(),(ne==null?void 0:ne.getAttribute("id"))===G&&j(),F.emit()})}),group:G=>(a.current.has(G)||a.current.set(G,new Set),()=>{i.current.delete(G),a.current.delete(G)}),filter:()=>s.current.shouldFilter,label:l||e["aria-label"],getDisablePointerSelection:()=>s.current.disablePointerSelection,listId:E,inputId:_,labelId:k,listInnerRef:R}),[]);function I(G,P){var ne,oe;let pe=(oe=(ne=s.current)==null?void 0:ne.filter)!=null?oe:zAe;return G?pe(G,r.current.search,P):0}function M(){if(!r.current.search||s.current.shouldFilter===!1)return;let G=r.current.filtered.items,P=[];r.current.filtered.groups.forEach(oe=>{let pe=a.current.get(oe),ae=0;pe.forEach(se=>{let Ee=G.get(se);ae=Math.max(Ee,ae)}),P.push([oe,ae])});let ne=R.current;X().sort((oe,pe)=>{var ae,se;let Ee=oe.getAttribute("id"),fe=pe.getAttribute("id");return((ae=G.get(fe))!=null?ae:0)-((se=G.get(Ee))!=null?se:0)}).forEach(oe=>{let pe=oe.closest(Ow);pe?pe.appendChild(oe.parentElement===pe?oe:oe.closest(`${Ow} > *`)):ne.appendChild(oe.parentElement===ne?oe:oe.closest(`${Ow} > *`))}),P.sort((oe,pe)=>pe[1]-oe[1]).forEach(oe=>{var pe;let ae=(pe=R.current)==null?void 0:pe.querySelector(`${Bf}[${ch}="${encodeURIComponent(oe[0])}"]`);ae==null||ae.parentElement.appendChild(ae)})}function j(){let G=X().find(ne=>ne.getAttribute("aria-disabled")!=="true"),P=G==null?void 0:G.getAttribute(ch);F.setState("value",P||void 0)}function Y(){var G,P,ne,oe;if(!r.current.search||s.current.shouldFilter===!1){r.current.filtered.count=n.current.size;return}r.current.filtered.groups=new Set;let pe=0;for(let ae of n.current){let se=(P=(G=i.current.get(ae))==null?void 0:G.value)!=null?P:"",Ee=(oe=(ne=i.current.get(ae))==null?void 0:ne.keywords)!=null?oe:[],fe=I(se,Ee);r.current.filtered.items.set(ae,fe),fe>0&&pe++}for(let[ae,se]of a.current)for(let Ee of se)if(r.current.filtered.items.get(Ee)>0){r.current.filtered.groups.add(ae);break}r.current.filtered.count=pe}function W(){var G,P,ne;let oe=$();oe&&(((G=oe.parentElement)==null?void 0:G.firstChild)===oe&&((ne=(P=oe.closest(Bf))==null?void 0:P.querySelector(PAe))==null||ne.scrollIntoView({block:"nearest"})),oe.scrollIntoView({block:"nearest"}))}function $(){var G;return(G=R.current)==null?void 0:G.querySelector(`${Hee}[aria-selected="true"]`)}function X(){var G;return Array.from(((G=R.current)==null?void 0:G.querySelectorAll(vB))||[])}function z(G){let P=X()[G];P&&F.setState("value",P.getAttribute(ch))}function H(G){var P;let ne=$(),oe=X(),pe=oe.findIndex(se=>se===ne),ae=oe[pe+G];(P=s.current)!=null&&P.loop&&(ae=pe+G<0?oe[oe.length-1]:pe+G===oe.length?oe[0]:oe[pe+G]),ae&&F.setState("value",ae.getAttribute(ch))}function B(G){let P=$(),ne=P==null?void 0:P.closest(Bf),oe;for(;ne&&!oe;)ne=G>0?KAe(ne,Bf):YAe(ne,Bf),oe=ne==null?void 0:ne.querySelector(vB);oe?F.setState("value",oe.getAttribute(ch)):H(G)}let ee=()=>z(X().length-1),J=G=>{G.preventDefault(),G.metaKey?ee():G.altKey?B(1):H(1)},K=G=>{G.preventDefault(),G.metaKey?z(0):G.altKey?B(-1):H(-1)};return T.createElement(Kt.div,{ref:t,tabIndex:-1,...w,"cmdk-root":"",onKeyDown:G=>{var P;(P=w.onKeyDown)==null||P.call(w,G);let ne=G.nativeEvent.isComposing||G.keyCode===229;if(!(G.defaultPrevented||ne))switch(G.key){case"n":case"j":{S&&G.ctrlKey&&J(G);break}case"ArrowDown":{J(G);break}case"p":case"k":{S&&G.ctrlKey&&K(G);break}case"ArrowUp":{K(G);break}case"Home":{G.preventDefault(),z(0);break}case"End":{G.preventDefault(),ee();break}case"Enter":{G.preventDefault();let oe=$();if(oe){let pe=new Event(xR);oe.dispatchEvent(pe)}}}}},T.createElement("label",{"cmdk-label":"",htmlFor:L.inputId,id:L.labelId,style:ZAe},l),d2(e,G=>T.createElement($ee.Provider,{value:F},T.createElement(Uee.Provider,{value:L},G))))}),HAe=T.forwardRef((e,t)=>{var r,n;let a=La(),i=T.useRef(null),o=T.useContext(Gee),s=Dg(),l=qee(e),u=(n=(r=l.current)==null?void 0:r.forceMount)!=null?n:o==null?void 0:o.forceMount;Gc(()=>{if(!u)return s.item(a,o==null?void 0:o.id)},[u]);let d=Wee(a,i,[e.value,e.children,i],e.keywords),p=k5(),f=Nu(C=>C.value&&C.value===d.current),m=Nu(C=>u||s.filter()===!1?!0:C.search?C.filtered.items.get(a)>0:!0);T.useEffect(()=>{let C=i.current;if(!(!C||e.disabled))return C.addEventListener(xR,b),()=>C.removeEventListener(xR,b)},[m,e.onSelect,e.disabled]);function b(){var C,F;y(),(F=(C=l.current).onSelect)==null||F.call(C,d.current)}function y(){p.setState("value",d.current,!0)}if(!m)return null;let{disabled:S,value:w,onSelect:E,forceMount:k,keywords:_,...R}=e;return T.createElement(Kt.div,{ref:Ic(i,t),...R,id:a,"cmdk-item":"",role:"option","aria-disabled":!!S,"aria-selected":!!f,"data-disabled":!!S,"data-selected":!!f,onPointerMove:S||s.getDisablePointerSelection()?void 0:y,onClick:S?void 0:b},e.children)}),UAe=T.forwardRef((e,t)=>{let{heading:r,children:n,forceMount:a,...i}=e,o=La(),s=T.useRef(null),l=T.useRef(null),u=La(),d=Dg(),p=Nu(m=>a||d.filter()===!1?!0:m.search?m.filtered.groups.has(o):!0);Gc(()=>d.group(o),[]),Wee(o,s,[e.value,e.heading,l]);let f=T.useMemo(()=>({id:o,forceMount:a}),[a]);return T.createElement(Kt.div,{ref:Ic(s,t),...i,"cmdk-group":"",role:"presentation",hidden:p?void 0:!0},r&&T.createElement("div",{ref:l,"cmdk-group-heading":"","aria-hidden":!0,id:u},r),d2(e,m=>T.createElement("div",{"cmdk-group-items":"",role:"group","aria-labelledby":r?u:void 0},T.createElement(Gee.Provider,{value:f},m))))}),$Ae=T.forwardRef((e,t)=>{let{alwaysRender:r,...n}=e,a=T.useRef(null),i=Nu(o=>!o.search);return!r&&!i?null:T.createElement(Kt.div,{ref:Ic(a,t),...n,"cmdk-separator":"",role:"separator"})}),GAe=T.forwardRef((e,t)=>{let{onValueChange:r,...n}=e,a=e.value!=null,i=k5(),o=Nu(u=>u.search),s=Nu(u=>u.selectedItemId),l=Dg();return T.useEffect(()=>{e.value!=null&&i.setState("search",e.value)},[e.value]),T.createElement(Kt.input,{ref:t,...n,"cmdk-input":"",autoComplete:"off",autoCorrect:"off",spellCheck:!1,"aria-autocomplete":"list",role:"combobox","aria-expanded":!0,"aria-controls":l.listId,"aria-labelledby":l.labelId,"aria-activedescendant":s,id:l.inputId,type:"text",value:a?e.value:o,onChange:u=>{a||i.setState("search",u.target.value),r==null||r(u.target.value)}})}),jAe=T.forwardRef((e,t)=>{let{children:r,label:n="Suggestions",...a}=e,i=T.useRef(null),o=T.useRef(null),s=Nu(u=>u.selectedItemId),l=Dg();return T.useEffect(()=>{if(o.current&&i.current){let u=o.current,d=i.current,p,f=new ResizeObserver(()=>{p=requestAnimationFrame(()=>{let m=u.offsetHeight;d.style.setProperty("--cmdk-list-height",m.toFixed(1)+"px")})});return f.observe(u),()=>{cancelAnimationFrame(p),f.unobserve(u)}}},[]),T.createElement(Kt.div,{ref:Ic(i,t),...a,"cmdk-list":"",role:"listbox",tabIndex:-1,"aria-activedescendant":s,"aria-label":n,id:l.listId},d2(e,u=>T.createElement("div",{ref:Ic(o,l.listInnerRef),"cmdk-list-sizer":""},u)))}),qAe=T.forwardRef((e,t)=>{let{open:r,onOpenChange:n,overlayClassName:a,contentClassName:i,container:o,...s}=e;return T.createElement(HD,{open:r,onOpenChange:n},T.createElement(UD,{container:o},T.createElement(Gv,{"cmdk-overlay":"",className:a}),T.createElement(jv,{"aria-label":e.label,"cmdk-dialog":"",className:i},T.createElement(jee,{ref:t,...s}))))}),WAe=T.forwardRef((e,t)=>Nu(r=>r.filtered.count===0)?T.createElement(Kt.div,{ref:t,...e,"cmdk-empty":"",role:"presentation"}):null),VAe=T.forwardRef((e,t)=>{let{progress:r,children:n,label:a="Loading...",...i}=e;return T.createElement(Kt.div,{ref:t,...i,"cmdk-loading":"",role:"progressbar","aria-valuenow":r,"aria-valuemin":0,"aria-valuemax":100,"aria-label":a},d2(e,o=>T.createElement("div",{"aria-hidden":!0},o)))}),yi=Object.assign(jee,{List:jAe,Item:HAe,Input:GAe,Group:UAe,Separator:$Ae,Dialog:qAe,Empty:WAe,Loading:VAe});function KAe(e,t){let r=e.nextElementSibling;for(;r;){if(r.matches(t))return r;r=r.nextElementSibling}}function YAe(e,t){let r=e.previousElementSibling;for(;r;){if(r.matches(t))return r;r=r.previousElementSibling}}function qee(e){let t=T.useRef(e);return Gc(()=>{t.current=e}),t}var Gc=typeof window>"u"?T.useEffect:T.useLayoutEffect;function dh(e){let t=T.useRef();return t.current===void 0&&(t.current=e()),t}function Nu(e){let t=k5(),r=()=>e(t.snapshot());return T.useSyncExternalStore(t.subscribe,r,r)}function Wee(e,t,r,n=[]){let a=T.useRef(),i=Dg();return Gc(()=>{var o;let s=(()=>{var u;for(let d of r){if(typeof d=="string")return d.trim();if(typeof d=="object"&&"current"in d)return d.current?(u=d.current.textContent)==null?void 0:u.trim():a.current}})(),l=n.map(u=>u.trim());i.value(e,s,l),(o=t.current)==null||o.setAttribute(ch,s),a.current=s}),a}var JAe=()=>{let[e,t]=T.useState(),r=dh(()=>new Map);return Gc(()=>{r.current.forEach(n=>n()),r.current=new Map},[e]),(n,a)=>{r.current.set(n,a),t({})}};function XAe(e){let t=e.type;return typeof t=="function"?t(e.props):"render"in t?t.render(e.props):e}function d2({asChild:e,children:t},r){return e&&T.isValidElement(t)?T.cloneElement(XAe(t),{ref:t.ref},r(t.props.children)):r(t)}var ZAe={position:"absolute",width:"1px",height:"1px",padding:"0",margin:"-1px",overflow:"hidden",clip:"rect(0, 0, 0, 0)",whiteSpace:"nowrap",borderWidth:"0"};const h2=T.forwardRef(({className:e,...t},r)=>x.jsx(yi,{ref:r,className:lt("bg-popover text-popover-foreground flex h-full w-full flex-col overflow-hidden rounded-md",e),...t}));h2.displayName=yi.displayName;const C5=T.forwardRef(({className:e,...t},r)=>x.jsxs("div",{className:"flex items-center border-b px-3","cmdk-input-wrapper":"",children:[x.jsx(jEe,{className:"mr-2 h-4 w-4 shrink-0 opacity-50"}),x.jsx(yi.Input,{ref:r,className:lt("placeholder:text-muted-foreground flex h-11 w-full rounded-md bg-transparent py-3 text-sm outline-none disabled:cursor-not-allowed disabled:opacity-50",e),...t})]}));C5.displayName=yi.Input.displayName;const p2=T.forwardRef(({className:e,...t},r)=>x.jsx(yi.List,{ref:r,className:lt("max-h-[300px] overflow-x-hidden overflow-y-auto",e),...t}));p2.displayName=yi.List.displayName;const T5=T.forwardRef((e,t)=>x.jsx(yi.Empty,{ref:t,className:"py-6 text-center text-sm",...e}));T5.displayName=yi.Empty.displayName;const Ep=T.forwardRef(({className:e,...t},r)=>x.jsx(yi.Group,{ref:r,className:lt("text-foreground [&_[cmdk-group-heading]]:text-muted-foreground overflow-hidden p-1 [&_[cmdk-group-heading]]:px-2 [&_[cmdk-group-heading]]:py-1.5 [&_[cmdk-group-heading]]:text-xs [&_[cmdk-group-heading]]:font-medium",e),...t}));Ep.displayName=yi.Group.displayName;const QAe=T.forwardRef(({className:e,...t},r)=>x.jsx(yi.Separator,{ref:r,className:lt("bg-border -mx-1 h-px",e),...t}));QAe.displayName=yi.Separator.displayName;const xp=T.forwardRef(({className:e,...t},r)=>x.jsx(yi.Item,{ref:r,className:lt("data-[selected='true']:bg-accent data-[selected=true]:text-accent-foreground relative flex cursor-default items-center gap-2 rounded-sm px-2 py-1.5 text-sm outline-none select-none data-[disabled=true]:pointer-events-none data-[disabled=true]:opacity-50 [&_svg]:pointer-events-none [&_svg]:size-4 [&_svg]:shrink-0",e),...t}));xp.displayName=yi.Item.displayName;const eFe=({layout:e,autoRunFor:t,mainLayout:r})=>{const n=zo(),[a,i]=T.useState(!1),o=T.useRef(null),{t:s}=Sr(),l=T.useCallback(()=>{if(n)try{const d=n.getGraph();if(!d||d.order===0)return;const p=r.positions();cee(d,p,{duration:300})}catch(d){console.error("Error updating positions:",d),o.current&&(window.clearInterval(o.current),o.current=null,i(!1))}},[n,r]),u=T.useCallback(()=>{if(a){console.log("Stopping layout animation"),o.current&&(window.clearInterval(o.current),o.current=null);try{typeof e.kill=="function"?(e.kill(),console.log("Layout algorithm killed")):typeof e.stop=="function"&&(e.stop(),console.log("Layout algorithm stopped"))}catch(d){console.error("Error stopping layout algorithm:",d)}i(!1)}else console.log("Starting layout animation"),l(),o.current=window.setInterval(()=>{l()},200),i(!0),setTimeout(()=>{if(o.current){console.log("Auto-stopping layout animation after 3 seconds"),window.clearInterval(o.current),o.current=null,i(!1);try{typeof e.kill=="function"?e.kill():typeof e.stop=="function"&&e.stop()}catch(d){console.error("Error stopping layout algorithm:",d)}}},3e3)},[a,e,l]);return T.useEffect(()=>{if(!n){console.log("No sigma instance available");return}let d=null;return t!==void 0&&t>-1&&n.getGraph().order>0&&(console.log("Auto-starting layout animation"),l(),o.current=window.setInterval(()=>{l()},200),i(!0),t>0&&(d=window.setTimeout(()=>{console.log("Auto-stopping layout animation after timeout"),o.current&&(window.clearInterval(o.current),o.current=null),i(!1)},t))),()=>{o.current&&(window.clearInterval(o.current),o.current=null),d&&window.clearTimeout(d),i(!1)}},[t,n,l]),x.jsx(Rt,{size:"icon",onClick:u,tooltip:s(a?"graphPanel.sideBar.layoutsControl.stopAnimation":"graphPanel.sideBar.layoutsControl.startAnimation"),variant:no,children:a?x.jsx(NEe,{}):x.jsx(OEe,{})})},tFe=()=>{const e=zo(),{t}=Sr(),[r,n]=T.useState("Circular"),[a,i]=T.useState(!1),o=Ve.use.graphLayoutMaxIterations(),s=XTe(),l=VTe(),u=FAe(),d=wAe({maxIterations:o,settings:{margin:5,expansion:1.1,gridSize:1,ratio:1,speed:3}}),p=aAe({maxIterations:o,settings:{attraction:3e-4,repulsion:.02,gravity:.02,inertia:.4,maxMove:100}}),f=Oee({iterations:o}),m=kAe(),b=iAe(),y=fAe(),S=T.useMemo(()=>({Circular:{layout:s},Circlepack:{layout:l},Random:{layout:u},Noverlaps:{layout:d,worker:m},"Force Directed":{layout:p,worker:b},"Force Atlas":{layout:f,worker:y}}),[l,s,p,f,d,u,b,m,y]),w=T.useCallback(E=>{console.debug("Running layout:",E);const{positions:k}=S[E].layout;try{const _=e.getGraph();if(!_){console.error("No graph available");return}const R=k();console.log("Positions calculated, animating nodes"),cee(_,R,{duration:400}),n(E)}catch(_){console.error("Error running layout:",_)}},[S,e]);return x.jsxs("div",{children:[x.jsx("div",{children:S[r]&&"worker"in S[r]&&x.jsx(eFe,{layout:S[r].worker,mainLayout:S[r].layout})}),x.jsx("div",{children:x.jsxs(t2,{open:a,onOpenChange:i,children:[x.jsx(r2,{asChild:!0,children:x.jsx(Rt,{size:"icon",variant:no,onClick:()=>i(E=>!E),tooltip:t("graphPanel.sideBar.layoutsControl.layoutGraph"),children:x.jsx(yEe,{})})}),x.jsx(kg,{side:"right",align:"start",sideOffset:8,collisionPadding:5,sticky:"always",className:"p-1 min-w-auto",children:x.jsx(h2,{children:x.jsx(p2,{children:x.jsx(Ep,{children:Object.keys(S).map(E=>x.jsx(xp,{onSelect:()=>{w(E)},className:"cursor-pointer text-xs",children:t(`graphPanel.sideBar.layoutsControl.layouts.${E}`)},E))})})})})]})})]})},A5=()=>{const e=T.useContext(wJ);if(e===void 0)throw new Error("useTheme must be used within a ThemeProvider");return e},$b=e=>!!(e.type.startsWith("mouse")&&e.buttons!==0),rFe=({disableHoverEffect:e})=>{const t=zo(),r=mee(),n=fee(),a=Ve.use.graphLayoutMaxIterations(),{assign:i}=Oee({iterations:a}),{theme:o}=A5(),s=Ve.use.enableHideUnselectedEdges(),l=Ve.use.enableEdgeEvents(),u=Ve.use.showEdgeLabel(),d=Ve.use.showNodeLabel(),p=Ve.use.minEdgeSize(),f=Ve.use.maxEdgeSize(),m=pt.use.selectedNode(),b=pt.use.focusedNode(),y=pt.use.selectedEdge(),S=pt.use.focusedEdge(),w=pt.use.sigmaGraph(),[E,k]=T.useState(()=>window.matchMedia("(prefers-color-scheme: dark)").matches);return T.useEffect(()=>{if(o==="system"){const _=window.matchMedia("(prefers-color-scheme: dark)"),R=C=>k(C.matches);return _.addEventListener("change",R),()=>_.removeEventListener("change",R)}},[o]),T.useEffect(()=>{if(w&&t){try{typeof t.setGraph=="function"?(t.setGraph(w),console.log("Binding graph to sigma instance")):(t.graph=w,console.warn("Simgma missing setGraph function, set graph property directly"))}catch(_){console.error("Error setting graph on sigma instance:",_)}i(),console.log("Initial layout applied to graph")}},[t,w,i,a]),T.useEffect(()=>{t&&(pt.getState().sigmaInstance||(console.log("Setting sigma instance from GraphControl"),pt.getState().setSigmaInstance(t)))},[t]),T.useEffect(()=>{const{setFocusedNode:_,setSelectedNode:R,setFocusedEdge:C,setSelectedEdge:F,clearSelection:L}=pt.getState(),I={enterNode:M=>{$b(M.event.original)||t.getGraph().hasNode(M.node)&&_(M.node)},leaveNode:M=>{$b(M.event.original)||_(null)},clickNode:M=>{t.getGraph().hasNode(M.node)&&(R(M.node),F(null))},clickStage:()=>L()};return l&&(I.clickEdge=M=>{F(M.edge),R(null)},I.enterEdge=M=>{$b(M.event.original)||C(M.edge)},I.leaveEdge=M=>{$b(M.event.original)||C(null)}),r(I),()=>{try{console.log("Cleaning up graph event listeners")}catch(M){console.warn("Error cleaning up graph event listeners:",M)}}},[r,l,t]),T.useEffect(()=>{if(t&&w){const _=t.getGraph();let R=Number.MAX_SAFE_INTEGER,C=0;_.forEachEdge(L=>{const I=_.getEdgeAttribute(L,"originalWeight")||1;typeof I=="number"&&(R=Math.min(R,I),C=Math.max(C,I))});const F=C-R;if(F>0){const L=f-p;_.forEachEdge(I=>{const M=_.getEdgeAttribute(I,"originalWeight")||1;if(typeof M=="number"){const j=p+L*Math.pow((M-R)/F,.5);_.setEdgeAttribute(I,"size",j)}})}else _.forEachEdge(L=>{_.setEdgeAttribute(L,"size",p)});t.refresh()}},[t,w,p,f]),T.useEffect(()=>{const _=o==="dark"||o==="system"&&window.document.documentElement.classList.contains("dark"),R=_?U_:void 0,C=_?$be:void 0;n({enableEdgeEvents:l,renderEdgeLabels:u,renderLabels:d,nodeReducer:(F,L)=>{const I=t.getGraph();if(!I.hasNode(F))return console.warn(`Node ${F} not found in graph during theme switch, returning default data`),{...L,highlighted:!1,labelColor:R};const M={...L,highlighted:L.highlighted||!1,labelColor:R};if(!e){M.highlighted=!1;const j=b||m,Y=S||y;if(j&&I.hasNode(j))try{(F===j||I.neighbors(j).includes(F))&&(M.highlighted=!0,F===m&&(M.borderColor=Ube))}catch(W){return console.error("Error in nodeReducer:",W),{...L,highlighted:!1,labelColor:R}}else if(Y&&I.hasEdge(Y))try{I.extremities(Y).includes(F)&&(M.highlighted=!0,M.size=3)}catch(W){return console.error("Error accessing edge extremities in nodeReducer:",W),{...L,highlighted:!1,labelColor:R}}else return M;M.highlighted?_&&(M.labelColor=zbe):M.color=Hbe}return M},edgeReducer:(F,L)=>{const I=t.getGraph();if(!I.hasEdge(F))return console.warn(`Edge ${F} not found in graph during theme switch, returning default data`),{...L,hidden:!1,labelColor:R,color:C};const M={...L,hidden:!1,labelColor:R,color:C};if(!e){const j=b||m,Y=_?jbe:qbe;if(j&&I.hasNode(j))try{s?I.extremities(F).includes(j)||(M.hidden=!0):I.extremities(F).includes(j)&&(M.color=Y)}catch(W){return console.error("Error in edgeReducer:",W),{...L,hidden:!1,labelColor:R,color:C}}else{const W=y&&I.hasEdge(y)?y:null,$=S&&I.hasEdge(S)?S:null;(W||$)&&(F===W?M.color=Gbe:F===$?M.color=Y:s&&(M.hidden=!0))}}return M}})},[m,b,y,S,n,t,e,o,E,s,l,u,d]),null},nFe=()=>{const{zoomIn:e,zoomOut:t,reset:r}=gee({duration:200,factor:1.5}),n=zo(),{t:a}=Sr(),i=T.useCallback(()=>e(),[e]),o=T.useCallback(()=>t(),[t]),s=T.useCallback(()=>{if(n)try{n.setCustomBBox(null),n.refresh();const d=n.getGraph();if(!(d!=null&&d.order)||d.nodes().length===0){r();return}n.getCamera().animate({x:.5,y:.5,ratio:1.1},{duration:1e3})}catch(d){console.error("Error resetting zoom:",d),r()}},[n,r]),l=T.useCallback(()=>{if(!n)return;const d=n.getCamera(),f=d.angle+Math.PI/8;d.animate({angle:f},{duration:200})},[n]),u=T.useCallback(()=>{if(!n)return;const d=n.getCamera(),f=d.angle-Math.PI/8;d.animate({angle:f},{duration:200})},[n]);return x.jsxs(x.Fragment,{children:[x.jsx(Rt,{variant:no,onClick:l,tooltip:a("graphPanel.sideBar.zoomControl.rotateCamera"),size:"icon",children:x.jsx(HEe,{})}),x.jsx(Rt,{variant:no,onClick:u,tooltip:a("graphPanel.sideBar.zoomControl.rotateCameraCounterClockwise"),size:"icon",children:x.jsx(o5,{})}),x.jsx(Rt,{variant:no,onClick:s,tooltip:a("graphPanel.sideBar.zoomControl.resetZoom"),size:"icon",children:x.jsx(hEe,{})}),x.jsx(Rt,{variant:no,onClick:i,tooltip:a("graphPanel.sideBar.zoomControl.zoomIn"),size:"icon",children:x.jsx(axe,{})}),x.jsx(Rt,{variant:no,onClick:o,tooltip:a("graphPanel.sideBar.zoomControl.zoomOut"),size:"icon",children:x.jsx(oxe,{})})]})},aFe=()=>{const{isFullScreen:e,toggle:t}=jCe(),{t:r}=Sr();return x.jsx(x.Fragment,{children:e?x.jsx(Rt,{variant:no,onClick:t,tooltip:r("graphPanel.sideBar.fullScreenControl.windowed"),size:"icon",children:x.jsx(FEe,{})}):x.jsx(Rt,{variant:no,onClick:t,tooltip:r("graphPanel.sideBar.fullScreenControl.fullScreen"),size:"icon",children:x.jsx(TEe,{})})})};var f2="Checkbox",[iFe]=Bo(f2),[oFe,F5]=iFe(f2);function sFe(e){const{__scopeCheckbox:t,checked:r,children:n,defaultChecked:a,disabled:i,form:o,name:s,onCheckedChange:l,required:u,value:d="on",internal_do_not_use_render:p}=e,[f,m]=bl({prop:r,defaultProp:a??!1,onChange:l,caller:f2}),[b,y]=T.useState(null),[S,w]=T.useState(null),E=T.useRef(!1),k=b?!!o||!!b.closest("form"):!0,_={checked:f,disabled:i,setChecked:m,control:b,setControl:y,name:s,form:o,value:d,hasConsumerStoppedPropagationRef:E,required:u,defaultChecked:wu(a)?!1:a,isFormControl:k,bubbleInput:S,setBubbleInput:w};return x.jsx(oFe,{scope:t,..._,children:lFe(p)?p(_):n})}var Vee="CheckboxTrigger",Kee=T.forwardRef(({__scopeCheckbox:e,onKeyDown:t,onClick:r,...n},a)=>{const{control:i,value:o,disabled:s,checked:l,required:u,setControl:d,setChecked:p,hasConsumerStoppedPropagationRef:f,isFormControl:m,bubbleInput:b}=F5(Vee,e),y=Rr(a,d),S=T.useRef(l);return T.useEffect(()=>{const w=i==null?void 0:i.form;if(w){const E=()=>p(S.current);return w.addEventListener("reset",E),()=>w.removeEventListener("reset",E)}},[i,p]),x.jsx(Kt.button,{type:"button",role:"checkbox","aria-checked":wu(l)?"mixed":l,"aria-required":u,"data-state":Qee(l),"data-disabled":s?"":void 0,disabled:s,value:o,...n,ref:y,onKeyDown:$t(t,w=>{w.key==="Enter"&&w.preventDefault()}),onClick:$t(r,w=>{p(E=>wu(E)?!0:!E),b&&m&&(f.current=w.isPropagationStopped(),f.current||w.stopPropagation())})})});Kee.displayName=Vee;var _5=T.forwardRef((e,t)=>{const{__scopeCheckbox:r,name:n,checked:a,defaultChecked:i,required:o,disabled:s,value:l,onCheckedChange:u,form:d,...p}=e;return x.jsx(sFe,{__scopeCheckbox:r,checked:a,defaultChecked:i,disabled:s,required:o,onCheckedChange:u,name:n,form:d,value:l,internal_do_not_use_render:({isFormControl:f})=>x.jsxs(x.Fragment,{children:[x.jsx(Kee,{...p,ref:t,__scopeCheckbox:r}),f&&x.jsx(Zee,{__scopeCheckbox:r})]})})});_5.displayName=f2;var Yee="CheckboxIndicator",Jee=T.forwardRef((e,t)=>{const{__scopeCheckbox:r,forceMount:n,...a}=e,i=F5(Yee,r);return x.jsx($i,{present:n||wu(i.checked)||i.checked===!0,children:x.jsx(Kt.span,{"data-state":Qee(i.checked),"data-disabled":i.disabled?"":void 0,...a,ref:t,style:{pointerEvents:"none",...e.style}})})});Jee.displayName=Yee;var Xee="CheckboxBubbleInput",Zee=T.forwardRef(({__scopeCheckbox:e,...t},r)=>{const{control:n,hasConsumerStoppedPropagationRef:a,checked:i,defaultChecked:o,required:s,disabled:l,name:u,value:d,form:p,bubbleInput:f,setBubbleInput:m}=F5(Xee,e),b=Rr(r,m),y=kZ(i),S=IX(n);T.useEffect(()=>{const E=f;if(!E)return;const k=window.HTMLInputElement.prototype,R=Object.getOwnPropertyDescriptor(k,"checked").set,C=!a.current;if(y!==i&&R){const F=new Event("click",{bubbles:C});E.indeterminate=wu(i),R.call(E,wu(i)?!1:i),E.dispatchEvent(F)}},[f,y,i,a]);const w=T.useRef(wu(i)?!1:i);return x.jsx(Kt.input,{type:"checkbox","aria-hidden":!0,defaultChecked:o??w.current,required:s,disabled:l,name:u,value:d,form:p,...t,tabIndex:-1,ref:b,style:{...t.style,...S,position:"absolute",pointerEvents:"none",opacity:0,margin:0,transform:"translateX(-100%)"}})});Zee.displayName=Xee;function lFe(e){return typeof e=="function"}function wu(e){return e==="indeterminate"}function Qee(e){return wu(e)?"indeterminate":e?"checked":"unchecked"}const ol=T.forwardRef(({className:e,...t},r)=>x.jsx(_5,{ref:r,className:lt("peer border-primary ring-offset-background focus-visible:ring-ring data-[state=checked]:bg-muted data-[state=checked]:text-muted-foreground h-4 w-4 shrink-0 rounded-sm border focus-visible:ring-2 focus-visible:ring-offset-2 focus-visible:outline-none disabled:cursor-not-allowed disabled:opacity-50",e),...t,children:x.jsx(Jee,{className:lt("flex items-center justify-center text-current"),children:x.jsx(a5,{className:"h-4 w-4"})})}));ol.displayName=_5.displayName;var uFe="Separator",SB="horizontal",cFe=["horizontal","vertical"],ete=T.forwardRef((e,t)=>{const{decorative:r,orientation:n=SB,...a}=e,i=dFe(n)?n:SB,s=r?{role:"none"}:{"aria-orientation":i==="vertical"?i:void 0,role:"separator"};return x.jsx(Kt.div,{"data-orientation":i,...s,...a,ref:t})});ete.displayName=uFe;function dFe(e){return cFe.includes(e)}var tte=ete;const im=T.forwardRef(({className:e,orientation:t="horizontal",decorative:r=!0,...n},a)=>x.jsx(tte,{ref:a,decorative:r,orientation:t,className:lt("bg-border shrink-0",t==="horizontal"?"h-[1px] w-full":"h-full w-[1px]",e),...n}));im.displayName=tte.displayName;var hFe=[{name:"Aegean Airlines",iataCode:"A3"},{name:"Aeroflot",iataCode:"SU"},{name:"Aerolineas Argentinas",iataCode:"AR"},{name:"Aeromexico",iataCode:"AM"},{name:"Air Algerie",iataCode:"AH"},{name:"Air Arabia",iataCode:"G9"},{name:"Air Canada",iataCode:"AC"},{name:"Air China",iataCode:"CA"},{name:"Air Europa",iataCode:"UX"},{name:"Air France",iataCode:"AF"},{name:"Air India",iataCode:"AI"},{name:"Air Mauritius",iataCode:"MK"},{name:"Air New Zealand",iataCode:"NZ"},{name:"Air Niugini",iataCode:"PX"},{name:"Air Tahiti",iataCode:"VT"},{name:"Air Tahiti Nui",iataCode:"TN"},{name:"Air Transat",iataCode:"TS"},{name:"AirAsia X",iataCode:"D7"},{name:"AirAsia",iataCode:"AK"},{name:"Aircalin",iataCode:"SB"},{name:"Alaska Airlines",iataCode:"AS"},{name:"Alitalia",iataCode:"AZ"},{name:"All Nippon Airways",iataCode:"NH"},{name:"Allegiant Air",iataCode:"G4"},{name:"American Airlines",iataCode:"AA"},{name:"Asiana Airlines",iataCode:"OZ"},{name:"Avianca",iataCode:"AV"},{name:"Azul Linhas Aereas Brasileiras",iataCode:"AD"},{name:"Azur Air",iataCode:"ZF"},{name:"Beijing Capital Airlines",iataCode:"JD"},{name:"Boliviana de Aviacion",iataCode:"OB"},{name:"British Airways",iataCode:"BA"},{name:"Cathay Pacific",iataCode:"CX"},{name:"Cebu Pacific Air",iataCode:"5J"},{name:"China Airlines",iataCode:"CI"},{name:"China Eastern Airlines",iataCode:"MU"},{name:"China Southern Airlines",iataCode:"CZ"},{name:"Condor",iataCode:"DE"},{name:"Copa Airlines",iataCode:"CM"},{name:"Delta Air Lines",iataCode:"DL"},{name:"Easyfly",iataCode:"VE"},{name:"EasyJet",iataCode:"U2"},{name:"EcoJet",iataCode:"8J"},{name:"Egyptair",iataCode:"MS"},{name:"El Al",iataCode:"LY"},{name:"Emirates Airlines",iataCode:"EK"},{name:"Ethiopian Airlines",iataCode:"ET"},{name:"Etihad Airways",iataCode:"EY"},{name:"EVA Air",iataCode:"BR"},{name:"Fiji Airways",iataCode:"FJ"},{name:"Finnair",iataCode:"AY"},{name:"Flybondi",iataCode:"FO"},{name:"Flydubai",iataCode:"FZ"},{name:"FlySafair",iataCode:"FA"},{name:"Frontier Airlines",iataCode:"F9"},{name:"Garuda Indonesia",iataCode:"GA"},{name:"Go First",iataCode:"G8"},{name:"Gol Linhas Aereas Inteligentes",iataCode:"G3"},{name:"Hainan Airlines",iataCode:"HU"},{name:"Hawaiian Airlines",iataCode:"HA"},{name:"IndiGo Airlines",iataCode:"6E"},{name:"Japan Airlines",iataCode:"JL"},{name:"Jeju Air",iataCode:"7C"},{name:"Jet2",iataCode:"LS"},{name:"JetBlue Airways",iataCode:"B6"},{name:"JetSMART",iataCode:"JA"},{name:"Juneyao Airlines",iataCode:"HO"},{name:"Kenya Airways",iataCode:"KQ"},{name:"KLM Royal Dutch Airlines",iataCode:"KL"},{name:"Korean Air",iataCode:"KE"},{name:"Kulula.com",iataCode:"MN"},{name:"LATAM Airlines",iataCode:"LA"},{name:"Lion Air",iataCode:"JT"},{name:"LOT Polish Airlines",iataCode:"LO"},{name:"Lufthansa",iataCode:"LH"},{name:"Libyan Airlines",iataCode:"LN"},{name:"Linea Aerea Amaszonas",iataCode:"Z8"},{name:"Malaysia Airlines",iataCode:"MH"},{name:"Nordwind Airlines",iataCode:"N4"},{name:"Norwegian Air Shuttle",iataCode:"DY"},{name:"Oman Air",iataCode:"WY"},{name:"Pakistan International Airlines",iataCode:"PK"},{name:"Pegasus Airlines",iataCode:"PC"},{name:"Philippine Airlines",iataCode:"PR"},{name:"Qantas Group",iataCode:"QF"},{name:"Qatar Airways",iataCode:"QR"},{name:"Republic Airways",iataCode:"YX"},{name:"Royal Air Maroc",iataCode:"AT"},{name:"Ryanair",iataCode:"FR"},{name:"S7 Airlines",iataCode:"S7"},{name:"SAS",iataCode:"SK"},{name:"Satena",iataCode:"9R"},{name:"Saudia",iataCode:"SV"},{name:"Shandong Airlines",iataCode:"SC"},{name:"Sichuan Airlines",iataCode:"3U"},{name:"Singapore Airlines",iataCode:"SQ"},{name:"Sky Airline",iataCode:"H2"},{name:"SkyWest Airlines",iataCode:"OO"},{name:"South African Airways",iataCode:"SA"},{name:"Southwest Airlines",iataCode:"WN"},{name:"SpiceJet",iataCode:"SG"},{name:"Spirit Airlines",iataCode:"NK"},{name:"Spring Airlines",iataCode:"9S"},{name:"SriLankan Airlines",iataCode:"UL"},{name:"Star Peru",iataCode:"2I"},{name:"Sun Country Airlines",iataCode:"SY"},{name:"SunExpress",iataCode:"XQ"},{name:"TAP Air Portugal",iataCode:"TP"},{name:"Thai AirAsia",iataCode:"FD"},{name:"Thai Airways",iataCode:"TG"},{name:"TUI Airways",iataCode:"BY"},{name:"Tunisair",iataCode:"TU"},{name:"Turkish Airlines",iataCode:"TK"},{name:"Ukraine International",iataCode:"PS"},{name:"United Airlines",iataCode:"UA"},{name:"Ural Airlines",iataCode:"U6"},{name:"VietJet Air",iataCode:"VJ"},{name:"Vietnam Airlines",iataCode:"VN"},{name:"Virgin Atlantic Airways",iataCode:"VS"},{name:"Virgin Australia",iataCode:"VA"},{name:"VivaAerobus",iataCode:"VB"},{name:"VOEPASS Linhas Aereas",iataCode:"2Z"},{name:"Volaris",iataCode:"Y4"},{name:"WestJet",iataCode:"WS"},{name:"Wingo",iataCode:"P5"},{name:"Wizz Air",iataCode:"W6"}],pFe=[{name:"Aerospatiale/BAC Concorde",iataTypeCode:"SSC"},{name:"Airbus A300",iataTypeCode:"AB3"},{name:"Airbus A310",iataTypeCode:"310"},{name:"Airbus A310-200",iataTypeCode:"312"},{name:"Airbus A310-300",iataTypeCode:"313"},{name:"Airbus A318",iataTypeCode:"318"},{name:"Airbus A319",iataTypeCode:"319"},{name:"Airbus A319neo",iataTypeCode:"31N"},{name:"Airbus A320",iataTypeCode:"320"},{name:"Airbus A320neo",iataTypeCode:"32N"},{name:"Airbus A321",iataTypeCode:"321"},{name:"Airbus A321neo",iataTypeCode:"32Q"},{name:"Airbus A330",iataTypeCode:"330"},{name:"Airbus A330-200",iataTypeCode:"332"},{name:"Airbus A330-300",iataTypeCode:"333"},{name:"Airbus A330-800neo",iataTypeCode:"338"},{name:"Airbus A330-900neo",iataTypeCode:"339"},{name:"Airbus A340",iataTypeCode:"340"},{name:"Airbus A340-200",iataTypeCode:"342"},{name:"Airbus A340-300",iataTypeCode:"343"},{name:"Airbus A340-500",iataTypeCode:"345"},{name:"Airbus A340-600",iataTypeCode:"346"},{name:"Airbus A350",iataTypeCode:"350"},{name:"Airbus A350-900",iataTypeCode:"359"},{name:"Airbus A350-1000",iataTypeCode:"351"},{name:"Airbus A380",iataTypeCode:"380"},{name:"Airbus A380-800",iataTypeCode:"388"},{name:"Antonov An-12",iataTypeCode:"ANF"},{name:"Antonov An-24",iataTypeCode:"AN4"},{name:"Antonov An-26",iataTypeCode:"A26"},{name:"Antonov An-28",iataTypeCode:"A28"},{name:"Antonov An-30",iataTypeCode:"A30"},{name:"Antonov An-32",iataTypeCode:"A32"},{name:"Antonov An-72",iataTypeCode:"AN7"},{name:"Antonov An-124 Ruslan",iataTypeCode:"A4F"},{name:"Antonov An-140",iataTypeCode:"A40"},{name:"Antonov An-148",iataTypeCode:"A81"},{name:"Antonov An-158",iataTypeCode:"A58"},{name:"Antonov An-225 Mriya",iataTypeCode:"A5F"},{name:"Boeing 707",iataTypeCode:"703"},{name:"Boeing 717",iataTypeCode:"717"},{name:"Boeing 720B",iataTypeCode:"B72"},{name:"Boeing 727",iataTypeCode:"727"},{name:"Boeing 727-100",iataTypeCode:"721"},{name:"Boeing 727-200",iataTypeCode:"722"},{name:"Boeing 737 MAX 7",iataTypeCode:"7M7"},{name:"Boeing 737 MAX 8",iataTypeCode:"7M8"},{name:"Boeing 737 MAX 9",iataTypeCode:"7M9"},{name:"Boeing 737 MAX 10",iataTypeCode:"7MJ"},{name:"Boeing 737",iataTypeCode:"737"},{name:"Boeing 737-100",iataTypeCode:"731"},{name:"Boeing 737-200",iataTypeCode:"732"},{name:"Boeing 737-300",iataTypeCode:"733"},{name:"Boeing 737-400",iataTypeCode:"734"},{name:"Boeing 737-500",iataTypeCode:"735"},{name:"Boeing 737-600",iataTypeCode:"736"},{name:"Boeing 737-700",iataTypeCode:"73G"},{name:"Boeing 737-800",iataTypeCode:"738"},{name:"Boeing 737-900",iataTypeCode:"739"},{name:"Boeing 747",iataTypeCode:"747"},{name:"Boeing 747-100",iataTypeCode:"741"},{name:"Boeing 747-200",iataTypeCode:"742"},{name:"Boeing 747-300",iataTypeCode:"743"},{name:"Boeing 747-400",iataTypeCode:"744"},{name:"Boeing 747-400D",iataTypeCode:"74J"},{name:"Boeing 747-8",iataTypeCode:"748"},{name:"Boeing 747SP",iataTypeCode:"74L"},{name:"Boeing 747SR",iataTypeCode:"74R"},{name:"Boeing 757",iataTypeCode:"757"},{name:"Boeing 757-200",iataTypeCode:"752"},{name:"Boeing 757-300",iataTypeCode:"753"},{name:"Boeing 767",iataTypeCode:"767"},{name:"Boeing 767-200",iataTypeCode:"762"},{name:"Boeing 767-300",iataTypeCode:"763"},{name:"Boeing 767-400",iataTypeCode:"764"},{name:"Boeing 777",iataTypeCode:"777"},{name:"Boeing 777-200",iataTypeCode:"772"},{name:"Boeing 777-200LR",iataTypeCode:"77L"},{name:"Boeing 777-300",iataTypeCode:"773"},{name:"Boeing 777-300ER",iataTypeCode:"77W"},{name:"Boeing 787",iataTypeCode:"787"},{name:"Boeing 787-8",iataTypeCode:"788"},{name:"Boeing 787-9",iataTypeCode:"789"},{name:"Boeing 787-10",iataTypeCode:"781"},{name:"Canadair Challenger",iataTypeCode:"CCJ"},{name:"Canadair CL-44",iataTypeCode:"CL4"},{name:"Canadair Regional Jet 100",iataTypeCode:"CR1"},{name:"Canadair Regional Jet 200",iataTypeCode:"CR2"},{name:"Canadair Regional Jet 700",iataTypeCode:"CR7"},{name:"Canadair Regional Jet 705",iataTypeCode:"CRA"},{name:"Canadair Regional Jet 900",iataTypeCode:"CR9"},{name:"Canadair Regional Jet 1000",iataTypeCode:"CRK"},{name:"De Havilland Canada DHC-2 Beaver",iataTypeCode:"DHP"},{name:"De Havilland Canada DHC-2 Turbo-Beaver",iataTypeCode:"DHR"},{name:"De Havilland Canada DHC-3 Otter",iataTypeCode:"DHL"},{name:"De Havilland Canada DHC-4 Caribou",iataTypeCode:"DHC"},{name:"De Havilland Canada DHC-6 Twin Otter",iataTypeCode:"DHT"},{name:"De Havilland Canada DHC-7 Dash 7",iataTypeCode:"DH7"},{name:"De Havilland Canada DHC-8-100 Dash 8 / 8Q",iataTypeCode:"DH1"},{name:"De Havilland Canada DHC-8-200 Dash 8 / 8Q",iataTypeCode:"DH2"},{name:"De Havilland Canada DHC-8-300 Dash 8 / 8Q",iataTypeCode:"DH3"},{name:"De Havilland Canada DHC-8-400 Dash 8Q",iataTypeCode:"DH4"},{name:"De Havilland DH.104 Dove",iataTypeCode:"DHD"},{name:"De Havilland DH.114 Heron",iataTypeCode:"DHH"},{name:"Douglas DC-3",iataTypeCode:"D3F"},{name:"Douglas DC-6",iataTypeCode:"D6F"},{name:"Douglas DC-8-50",iataTypeCode:"D8T"},{name:"Douglas DC-8-62",iataTypeCode:"D8L"},{name:"Douglas DC-8-72",iataTypeCode:"D8Q"},{name:"Douglas DC-9-10",iataTypeCode:"D91"},{name:"Douglas DC-9-20",iataTypeCode:"D92"},{name:"Douglas DC-9-30",iataTypeCode:"D93"},{name:"Douglas DC-9-40",iataTypeCode:"D94"},{name:"Douglas DC-9-50",iataTypeCode:"D95"},{name:"Douglas DC-10",iataTypeCode:"D10"},{name:"Douglas DC-10-10",iataTypeCode:"D1X"},{name:"Douglas DC-10-30",iataTypeCode:"D1Y"},{name:"Embraer 170",iataTypeCode:"E70"},{name:"Embraer 175",iataTypeCode:"E75"},{name:"Embraer 190",iataTypeCode:"E90"},{name:"Embraer 195",iataTypeCode:"E95"},{name:"Embraer E190-E2",iataTypeCode:"290"},{name:"Embraer E195-E2",iataTypeCode:"295"},{name:"Embraer EMB.110 Bandeirante",iataTypeCode:"EMB"},{name:"Embraer EMB.120 Brasilia",iataTypeCode:"EM2"},{name:"Embraer Legacy 600",iataTypeCode:"ER3"},{name:"Embraer Phenom 100",iataTypeCode:"EP1"},{name:"Embraer Phenom 300",iataTypeCode:"EP3"},{name:"Embraer RJ135",iataTypeCode:"ER3"},{name:"Embraer RJ140",iataTypeCode:"ERD"},{name:"Embraer RJ145 Amazon",iataTypeCode:"ER4"},{name:"Ilyushin IL18",iataTypeCode:"IL8"},{name:"Ilyushin IL62",iataTypeCode:"IL6"},{name:"Ilyushin IL76",iataTypeCode:"IL7"},{name:"Ilyushin IL86",iataTypeCode:"ILW"},{name:"Ilyushin IL96-300",iataTypeCode:"I93"},{name:"Ilyushin IL114",iataTypeCode:"I14"},{name:"Lockheed L-182 / 282 / 382 (L-100) Hercules",iataTypeCode:"LOH"},{name:"Lockheed L-188 Electra",iataTypeCode:"LOE"},{name:"Lockheed L-1011 Tristar",iataTypeCode:"L10"},{name:"Lockheed L-1049 Super Constellation",iataTypeCode:"L49"},{name:"McDonnell Douglas MD11",iataTypeCode:"M11"},{name:"McDonnell Douglas MD80",iataTypeCode:"M80"},{name:"McDonnell Douglas MD81",iataTypeCode:"M81"},{name:"McDonnell Douglas MD82",iataTypeCode:"M82"},{name:"McDonnell Douglas MD83",iataTypeCode:"M83"},{name:"McDonnell Douglas MD87",iataTypeCode:"M87"},{name:"McDonnell Douglas MD88",iataTypeCode:"M88"},{name:"McDonnell Douglas MD90",iataTypeCode:"M90"},{name:"Sukhoi Superjet 100-95",iataTypeCode:"SU9"},{name:"Tupolev Tu-134",iataTypeCode:"TU3"},{name:"Tupolev Tu-154",iataTypeCode:"TU5"},{name:"Tupolev Tu-204",iataTypeCode:"T20"},{name:"Yakovlev Yak-40",iataTypeCode:"YK4"},{name:"Yakovlev Yak-42",iataTypeCode:"YK2"}],fFe=[{name:"Adelaide International Airport",iataCode:"ADL"},{name:"Adolfo Suarez Madrid-Barajas Airport",iataCode:"MAD"},{name:"Aeroparque Jorge Newbery Airport",iataCode:"AEP"},{name:"Afonso Pena International Airport",iataCode:"CWB"},{name:"Alfonso Bonilla Aragon International Airport",iataCode:"CLO"},{name:"Amsterdam Airport Schiphol",iataCode:"AMS"},{name:"Arturo Merino Benitez International Airport",iataCode:"SCL"},{name:"Auckland International Airport",iataCode:"AKL"},{name:"Beijing Capital International Airport",iataCode:"PEK"},{name:"Belem Val de Cans International Airport",iataCode:"BEL"},{name:"Belo Horizonte Tancredo Neves International Airport",iataCode:"CNF"},{name:"Berlin-Tegel Airport",iataCode:"TXL"},{name:"Bole International Airport",iataCode:"ADD"},{name:"Brasilia-Presidente Juscelino Kubitschek International Airport",iataCode:"BSB"},{name:"Brisbane International Airport",iataCode:"BNE"},{name:"Brussels Airport",iataCode:"BRU"},{name:"Cairns Airport",iataCode:"CNS"},{name:"Cairo International Airport",iataCode:"CAI"},{name:"Canberra Airport",iataCode:"CBR"},{name:"Capetown International Airport",iataCode:"CPT"},{name:"Charles de Gaulle International Airport",iataCode:"CDG"},{name:"Charlotte Douglas International Airport",iataCode:"CLT"},{name:"Chengdu Shuangliu International Airport",iataCode:"CTU"},{name:"Chhatrapati Shivaji International Airport",iataCode:"BOM"},{name:"Chicago O'Hare International Airport",iataCode:"ORD"},{name:"Chongqing Jiangbei International Airport",iataCode:"CKG"},{name:"Christchurch International Airport",iataCode:"CHC"},{name:"Copenhagen Kastrup Airport",iataCode:"CPH"},{name:"Dallas Fort Worth International Airport",iataCode:"DFW"},{name:"Daniel K. Inouye International Airport",iataCode:"HNL"},{name:"Denver International Airport",iataCode:"DEN"},{name:"Don Mueang International Airport",iataCode:"DMK"},{name:"Dubai International Airport",iataCode:"DXB"},{name:"Dublin Airport",iataCode:"DUB"},{name:"Dusseldorf Airport",iataCode:"DUS"},{name:"El Dorado International Airport",iataCode:"BOG"},{name:"Eleftherios Venizelos International Airport",iataCode:"ATH"},{name:"Faa'a International Airport",iataCode:"PPT"},{name:"Fort Lauderdale Hollywood International Airport",iataCode:"FLL"},{name:"Fortaleza Pinto Martins International Airport",iataCode:"FOR"},{name:"Frankfurt am Main Airport",iataCode:"FRA"},{name:"George Bush Intercontinental Houston Airport",iataCode:"IAH"},{name:"Gold Coast Airport",iataCode:"OOL"},{name:"Guarulhos - Governador Andre Franco Montoro International Airport",iataCode:"GRU"},{name:"Hartsfield-Jackson Atlanta International Airport",iataCode:"ATL"},{name:"Helsinki Vantaa Airport",iataCode:"HEL"},{name:"Hobart International Airport",iataCode:"HBA"},{name:"Hong Kong International Airport",iataCode:"HKG"},{name:"Houari Boumediene Airport",iataCode:"ALG"},{name:"Hurgada International Airport",iataCode:"HRG"},{name:"Incheon International Airport",iataCode:"ICN"},{name:"Indira Gandhi International Airport",iataCode:"DEL"},{name:"Istanbul Airport",iataCode:"IST"},{name:"Jacksons International Airport",iataCode:"POM"},{name:"Jeju International Airport",iataCode:"CJU"},{name:"John F Kennedy International Airport",iataCode:"JFK"},{name:"Jorge Chavez International Airport",iataCode:"LIM"},{name:"Jose Maria Cordova International Airport",iataCode:"MDE"},{name:"Josep Tarradellas Barcelona-El Prat Airport",iataCode:"BCN"},{name:"Kahului Airport",iataCode:"OGG"},{name:"King Abdulaziz International Airport",iataCode:"JED"},{name:"Kuala Lumpur International Airport",iataCode:"KUL"},{name:"Kunming Changshui International Airport",iataCode:"KMG"},{name:"La Tontouta International Airport",iataCode:"NOU"},{name:"Leonardo da Vinci-Fiumicino Airport",iataCode:"FCO"},{name:"London Heathrow Airport",iataCode:"LHR"},{name:"Los Angeles International Airport",iataCode:"LAX"},{name:"McCarran International Airport",iataCode:"LAS"},{name:"Melbourne International Airport",iataCode:"MEL"},{name:"Mexico City International Airport",iataCode:"MEX"},{name:"Miami International Airport",iataCode:"MIA"},{name:"Ministro Pistarini International Airport",iataCode:"EZE"},{name:"Minneapolis-St Paul International/Wold-Chamberlain Airport",iataCode:"MSP"},{name:"Mohammed V International Airport",iataCode:"CMN"},{name:"Moscow Domodedovo Airport",iataCode:"DME"},{name:"Munich Airport",iataCode:"MUC"},{name:"Murtala Muhammed International Airport",iataCode:"LOS"},{name:"Nadi International Airport",iataCode:"NAN"},{name:"Nairobi Jomo Kenyatta International Airport",iataCode:"NBO"},{name:"Narita International Airport",iataCode:"NRT"},{name:"Newark Liberty International Airport",iataCode:"EWR"},{name:"Ninoy Aquino International Airport",iataCode:"MNL"},{name:"Noumea Magenta Airport",iataCode:"GEA"},{name:"O. R. Tambo International Airport",iataCode:"JNB"},{name:"Orlando International Airport",iataCode:"MCO"},{name:"Oslo Lufthavn",iataCode:"OSL"},{name:"Perth Airport",iataCode:"PER"},{name:"Phoenix Sky Harbor International Airport",iataCode:"PHX"},{name:"Recife Guararapes-Gilberto Freyre International Airport",iataCode:"REC"},{name:"Rio de Janeiro Galeao International Airport",iataCode:"GIG"},{name:"Salgado Filho International Airport",iataCode:"POA"},{name:"Salvador Deputado Luis Eduardo Magalhaes International Airport",iataCode:"SSA"},{name:"San Francisco International Airport",iataCode:"SFO"},{name:"Santos Dumont Airport",iataCode:"SDU"},{name:"Sao Paulo-Congonhas Airport",iataCode:"CGH"},{name:"Seattle Tacoma International Airport",iataCode:"SEA"},{name:"Shanghai Hongqiao International Airport",iataCode:"SHA"},{name:"Shanghai Pudong International Airport",iataCode:"PVG"},{name:"Shenzhen Bao'an International Airport",iataCode:"SZX"},{name:"Sheremetyevo International Airport",iataCode:"SVO"},{name:"Singapore Changi Airport",iataCode:"SIN"},{name:"Soekarno-Hatta International Airport",iataCode:"CGK"},{name:'Stockholm-Arlanda Airport"',iataCode:"ARN"},{name:"Suvarnabhumi Airport",iataCode:"BKK"},{name:"Sydney Kingsford Smith International Airport",iataCode:"SYD"},{name:"Taiwan Taoyuan International Airport",iataCode:"TPE"},{name:"Tan Son Nhat International Airport",iataCode:"SGN"},{name:"Tokyo Haneda International Airport",iataCode:"HND"},{name:"Toronto Pearson International Airport",iataCode:"YYZ"},{name:"Tunis Carthage International Airport",iataCode:"TUN"},{name:"Vancouver International Airport",iataCode:"YVR"},{name:"Vienna International Airport",iataCode:"VIE"},{name:"Viracopos International Airport",iataCode:"VCP"},{name:"Vnukovo International Airport",iataCode:"VKO"},{name:"Wellington International Airport",iataCode:"WLG"},{name:"Xi'an Xianyang International Airport",iataCode:"XIY"},{name:"Zhukovsky International Airport",iataCode:"ZIA"},{name:"Zurich Airport",iataCode:"ZRH"}],mFe={airline:hFe,airplane:pFe,airport:fFe},gFe=mFe,bFe=["American black bear","Asian black bear","Brown bear","Giant panda","Polar bear","Sloth bear","Spectacled bear","Sun bear"],yFe=["Abert's Towhee","Acadian Flycatcher","Acorn Woodpecker","Alder Flycatcher","Aleutian Tern","Allen's Hummingbird","Altamira Oriole","American Avocet","American Bittern","American Black Duck","American Coot","American Crow","American Dipper","American Golden-Plover","American Goldfinch","American Kestrel","American Oystercatcher","American Pipit","American Redstart","American Robin","American Tree Sparrow","American White Pelican","American Wigeon","American Woodcock","Ancient Murrelet","Anhinga","Anna's Hummingbird","Antillean Nighthawk","Antillean Palm Swift","Aplomado Falcon","Arctic Loon","Arctic Tern","Arctic Warbler","Ash-throated Flycatcher","Ashy Storm-Petrel","Asian Brown Flycatcher","Atlantic Puffin","Audubon's Oriole","Audubon's Shearwater","Aztec Thrush","Azure Gallinule","Bachman's Sparrow","Bachman's Warbler","Bahama Mockingbird","Bahama Swallow","Bahama Woodstar","Baikal Teal","Baird's Sandpiper","Baird's Sparrow","Bald Eagle","Baltimore Oriole","Bananaquit","Band-rumped Storm-Petrel","Band-tailed Gull","Band-tailed Pigeon","Bank Swallow","Bar-tailed Godwit","Barn Owl","Barn Swallow","Barnacle Goose","Barred Owl","Barrow's Goldeneye","Bay-breasted Warbler","Bean Goose","Bell's Vireo","Belted Kingfisher","Bendire's Thrasher","Berylline Hummingbird","Bewick's Wren","Bicknell's Thrush","Black Catbird","Black Guillemot","Black Noddy","Black Oystercatcher","Black Phoebe","Black Rail","Black Rosy-Finch","Black Scoter","Black Skimmer","Black Storm-Petrel","Black Swift","Black Tern","Black Turnstone","Black Vulture","Black-and-white Warbler","Black-backed Wagtail","Black-backed Woodpecker","Black-bellied Plover","Black-bellied Whistling-Duck","Black-billed Cuckoo","Black-billed Magpie","Black-browed Albatross","Black-capped Chickadee","Black-capped Gnatcatcher","Black-capped Petrel","Black-capped Vireo","Black-chinned Hummingbird","Black-chinned Sparrow","Black-crowned Night-Heron","Black-faced Grassquit","Black-footed Albatross","Black-headed Grosbeak","Black-headed Gull","Black-legged Kittiwake","Black-necked Stilt","Black-tailed Gnatcatcher","Black-tailed Godwit","Black-tailed Gull","Black-throated Blue Warbler","Black-throated Gray Warbler","Black-throated Green Warbler","Black-throated Sparrow","Black-vented Oriole","Black-vented Shearwater","Black-whiskered Vireo","Black-winged Stilt","Blackburnian Warbler","Blackpoll Warbler","Blue Bunting","Blue Grosbeak","Blue Grouse","Blue Jay","Blue Mockingbird","Blue-footed Booby","Blue-gray Gnatcatcher","Blue-headed Vireo","Blue-throated Hummingbird","Blue-winged Teal","Blue-winged Warbler","Bluethroat","Boat-tailed Grackle","Bobolink","Bohemian Waxwing","Bonaparte's Gull","Boreal Chickadee","Boreal Owl","Botteri's Sparrow","Brambling","Brandt's Cormorant","Brant","Brewer's Blackbird","Brewer's Sparrow","Bridled Tern","Bridled Titmouse","Bristle-thighed Curlew","Broad-billed Hummingbird","Broad-billed Sandpiper","Broad-tailed Hummingbird","Broad-winged Hawk","Bronzed Cowbird","Brown Booby","Brown Creeper","Brown Jay","Brown Noddy","Brown Pelican","Brown Shrike","Brown Thrasher","Brown-capped Rosy-Finch","Brown-chested Martin","Brown-crested Flycatcher","Brown-headed Cowbird","Brown-headed Nuthatch","Budgerigar","Buff-bellied Hummingbird","Buff-breasted Flycatcher","Buff-breasted Sandpiper","Buff-collared Nightjar","Bufflehead","Buller's Shearwater","Bullock's Oriole","Bumblebee Hummingbird","Burrowing Owl","Bushtit","Cactus Wren","California Condor","California Gnatcatcher","California Gull","California Quail","California Thrasher","California Towhee","Calliope Hummingbird","Canada Goose","Canada Warbler","Canvasback","Canyon Towhee","Canyon Wren","Cape May Warbler","Caribbean Elaenia","Carolina Chickadee","Carolina Parakeet","Carolina Wren","Caspian Tern","Cassin's Auklet","Cassin's Finch","Cassin's Kingbird","Cassin's Sparrow","Cassin's Vireo","Cattle Egret","Cave Swallow","Cedar Waxwing","Cerulean Warbler","Chestnut-backed Chickadee","Chestnut-collared Longspur","Chestnut-sided Warbler","Chihuahuan Raven","Chimney Swift","Chinese Egret","Chipping Sparrow","Chuck-will's-widow","Chukar","Cinnamon Hummingbird","Cinnamon Teal","Citrine Wagtail","Clapper Rail","Clark's Grebe","Clark's Nutcracker","Clay-colored Robin","Clay-colored Sparrow","Cliff Swallow","Colima Warbler","Collared Forest-Falcon","Collared Plover","Common Black-Hawk","Common Chaffinch","Common Crane","Common Cuckoo","Common Eider","Common Goldeneye","Common Grackle","Common Greenshank","Common Ground-Dove","Common House-Martin","Common Loon","Common Merganser","Common Moorhen","Common Murre","Common Nighthawk","Common Pauraque","Common Pochard","Common Poorwill","Common Raven","Common Redpoll","Common Ringed Plover","Common Rosefinch","Common Sandpiper","Common Snipe","Common Swift","Common Tern","Common Yellowthroat","Connecticut Warbler","Cook's Petrel","Cooper's Hawk","Cordilleran Flycatcher","Corn Crake","Cory's Shearwater","Costa's Hummingbird","Couch's Kingbird","Crane Hawk","Craveri's Murrelet","Crescent-chested Warbler","Crested Auklet","Crested Caracara","Crested Myna","Crimson-collared Grosbeak","Crissal Thrasher","Cuban Martin","Curlew Sandpiper","Curve-billed Thrasher","Dark-eyed Junco","Dickcissel","Double-crested Cormorant","Double-striped Thick-knee","Dovekie","Downy Woodpecker","Dunlin","Dusky Flycatcher","Dusky Thrush","Dusky Warbler","Dusky-capped Flycatcher","Eared Grebe","Eared Trogon","Eastern Bluebird","Eastern Kingbird","Eastern Meadowlark","Eastern Phoebe","Eastern Screech-Owl","Eastern Towhee","Eastern Wood-Pewee","Elegant Tern","Elegant Trogon","Elf Owl","Emperor Goose","Eskimo Curlew","Eurasian Blackbird","Eurasian Bullfinch","Eurasian Collared-Dove","Eurasian Coot","Eurasian Curlew","Eurasian Dotterel","Eurasian Hobby","Eurasian Jackdaw","Eurasian Kestrel","Eurasian Oystercatcher","Eurasian Siskin","Eurasian Tree Sparrow","Eurasian Wigeon","Eurasian Woodcock","Eurasian Wryneck","European Golden-Plover","European Starling","European Storm-Petrel","European Turtle-Dove","Evening Grosbeak","Eyebrowed Thrush","Falcated Duck","Fan-tailed Warbler","Far Eastern Curlew","Ferruginous Hawk","Ferruginous Pygmy-Owl","Field Sparrow","Fieldfare","Fish Crow","Five-striped Sparrow","Flame-colored Tanager","Flammulated Owl","Flesh-footed Shearwater","Florida Scrub-Jay","Fork-tailed Flycatcher","Fork-tailed Storm-Petrel","Fork-tailed Swift","Forster's Tern","Fox Sparrow","Franklin's Gull","Fulvous Whistling-Duck","Gadwall","Gambel's Quail","Garganey","Gila Woodpecker","Gilded Flicker","Glaucous Gull","Glaucous-winged Gull","Glossy Ibis","Golden Eagle","Golden-cheeked Warbler","Golden-crowned Kinglet","Golden-crowned Sparrow","Golden-crowned Warbler","Golden-fronted Woodpecker","Golden-winged Warbler","Grace's Warbler","Grasshopper Sparrow","Gray Bunting","Gray Catbird","Gray Flycatcher","Gray Hawk","Gray Jay","Gray Kingbird","Gray Partridge","Gray Silky-flycatcher","Gray Vireo","Gray Wagtail","Gray-breasted Martin","Gray-cheeked Thrush","Gray-crowned Rosy-Finch","Gray-crowned Yellowthroat","Gray-headed Chickadee","Gray-spotted Flycatcher","Gray-tailed Tattler","Great Auk","Great Black-backed Gull","Great Blue Heron","Great Cormorant","Great Crested Flycatcher","Great Egret","Great Frigatebird","Great Gray Owl","Great Horned Owl","Great Kiskadee","Great Knot","Great Skua","Great Spotted Woodpecker","Great-tailed Grackle","Greater Flamingo","Greater Pewee","Greater Prairie-chicken","Greater Roadrunner","Greater Scaup","Greater Shearwater","Greater White-fronted Goose","Greater Yellowlegs","Green Heron","Green Jay","Green Kingfisher","Green Sandpiper","Green Violet-ear","Green-breasted Mango","Green-tailed Towhee","Green-winged Teal","Greenish Elaenia","Groove-billed Ani","Gull-billed Tern","Gyrfalcon","Hairy Woodpecker","Hammond's Flycatcher","Harlequin Duck","Harris's Hawk","Harris's Sparrow","Hawfinch","Heermann's Gull","Henslow's Sparrow","Hepatic Tanager","Herald Petrel","Hermit Thrush","Hermit Warbler","Herring Gull","Himalayan Snowcock","Hoary Redpoll","Hooded Merganser","Hooded Oriole","Hooded Warbler","Hook-billed Kite","Hoopoe","Horned Grebe","Horned Lark","Horned Puffin","House Finch","House Sparrow","House Wren","Hudsonian Godwit","Hutton's Vireo","Iceland Gull","Inca Dove","Indigo Bunting","Island Scrub-Jay","Ivory Gull","Ivory-billed Woodpecker","Jabiru","Jack Snipe","Jungle Nightjar","Juniper Titmouse","Kentucky Warbler","Key West Quail-Dove","Killdeer","King Eider","King Rail","Kirtland's Warbler","Kittlitz's Murrelet","La Sagra's Flycatcher","Labrador Duck","Ladder-backed Woodpecker","Lanceolated Warbler","Lapland Longspur","Large-billed Tern","Lark Bunting","Lark Sparrow","Laughing Gull","Lawrence's Goldfinch","Laysan Albatross","Lazuli Bunting","Le Conte's Sparrow","Le Conte's Thrasher","Leach's Storm-Petrel","Least Auklet","Least Bittern","Least Flycatcher","Least Grebe","Least Sandpiper","Least Storm-Petrel","Least Tern","Lesser Black-backed Gull","Lesser Frigatebird","Lesser Goldfinch","Lesser Nighthawk","Lesser Prairie-chicken","Lesser Scaup","Lesser White-fronted Goose","Lesser Yellowlegs","Lewis's Woodpecker","Limpkin","Lincoln's Sparrow","Little Blue Heron","Little Bunting","Little Curlew","Little Egret","Little Gull","Little Ringed Plover","Little Shearwater","Little Stint","Loggerhead Kingbird","Loggerhead Shrike","Long-billed Curlew","Long-billed Dowitcher","Long-billed Murrelet","Long-billed Thrasher","Long-eared Owl","Long-tailed Jaeger","Long-toed Stint","Louisiana Waterthrush","Lucifer Hummingbird","Lucy's Warbler","MacGillivray's Warbler","Magnificent Frigatebird","Magnificent Hummingbird","Magnolia Warbler","Mallard","Mangrove Cuckoo","Manx Shearwater","Marbled Godwit","Marbled Murrelet","Marsh Sandpiper","Marsh Wren","Masked Booby","Masked Duck","Masked Tityra","McCown's Longspur","McKay's Bunting","Merlin","Mew Gull","Mexican Chickadee","Mexican Jay","Middendorff's Grasshopper-Warbler","Mississippi Kite","Mongolian Plover","Monk Parakeet","Montezuma Quail","Mottled Duck","Mottled Owl","Mottled Petrel","Mountain Bluebird","Mountain Chickadee","Mountain Plover","Mountain Quail","Mourning Dove","Mourning Warbler","Mugimaki Flycatcher","Murphy's Petrel","Muscovy Duck","Mute Swan","Narcissus Flycatcher","Nashville Warbler","Nelson's Sharp-tailed Sparrow","Neotropic Cormorant","Northern Beardless-Tyrannulet","Northern Bobwhite","Northern Cardinal","Northern Flicker","Northern Fulmar","Northern Gannet","Northern Goshawk","Northern Harrier","Northern Hawk Owl","Northern Jacana","Northern Lapwing","Northern Mockingbird","Northern Parula","Northern Pintail","Northern Pygmy-Owl","Northern Rough-winged Swallow","Northern Saw-whet Owl","Northern Shoveler","Northern Shrike","Northern Waterthrush","Northern Wheatear","Northwestern Crow","Nuttall's Woodpecker","Nutting's Flycatcher","Oak Titmouse","Oldsquaw","Olive Sparrow","Olive Warbler","Olive-backed Pipit","Olive-sided Flycatcher","Orange-crowned Warbler","Orchard Oriole","Oriental Cuckoo","Oriental Greenfinch","Oriental Pratincole","Oriental Scops-Owl","Oriental Turtle-Dove","Osprey","Ovenbird","Pacific Golden-Plover","Pacific Loon","Pacific-slope Flycatcher","Paint-billed Crake","Painted Bunting","Painted Redstart","Pallas's Bunting","Palm Warbler","Parakeet Auklet","Parasitic Jaeger","Passenger Pigeon","Pechora Pipit","Pectoral Sandpiper","Pelagic Cormorant","Peregrine Falcon","Phainopepla","Philadelphia Vireo","Pied-billed Grebe","Pigeon Guillemot","Pileated Woodpecker","Pin-tailed Snipe","Pine Bunting","Pine Grosbeak","Pine Siskin","Pine Warbler","Pink-footed Goose","Pink-footed Shearwater","Pinyon Jay","Piping Plover","Plain Chachalaca","Plain-capped Starthroat","Plumbeous Vireo","Pomarine Jaeger","Prairie Falcon","Prairie Warbler","Prothonotary Warbler","Purple Finch","Purple Gallinule","Purple Martin","Purple Sandpiper","Pygmy Nuthatch","Pyrrhuloxia","Razorbill","Red Crossbill","Red Knot","Red Phalarope","Red-bellied Woodpecker","Red-billed Pigeon","Red-billed Tropicbird","Red-breasted Flycatcher","Red-breasted Merganser","Red-breasted Nuthatch","Red-breasted Sapsucker","Red-cockaded Woodpecker","Red-crowned Parrot","Red-eyed Vireo","Red-faced Cormorant","Red-faced Warbler","Red-flanked Bluetail","Red-footed Booby","Red-headed Woodpecker","Red-legged Kittiwake","Red-naped Sapsucker","Red-necked Grebe","Red-necked Phalarope","Red-necked Stint","Red-shouldered Hawk","Red-tailed Hawk","Red-tailed Tropicbird","Red-throated Loon","Red-throated Pipit","Red-whiskered Bulbul","Red-winged Blackbird","Reddish Egret","Redhead","Redwing","Reed Bunting","Rhinoceros Auklet","Ring-billed Gull","Ring-necked Duck","Ring-necked Pheasant","Ringed Kingfisher","Roadside Hawk","Rock Dove","Rock Ptarmigan","Rock Sandpiper","Rock Wren","Rose-breasted Grosbeak","Rose-throated Becard","Roseate Spoonbill","Roseate Tern","Ross's Goose","Ross's Gull","Rough-legged Hawk","Royal Tern","Ruby-crowned Kinglet","Ruby-throated Hummingbird","Ruddy Duck","Ruddy Ground-Dove","Ruddy Quail-Dove","Ruddy Turnstone","Ruff","Ruffed Grouse","Rufous Hummingbird","Rufous-backed Robin","Rufous-capped Warbler","Rufous-crowned Sparrow","Rufous-winged Sparrow","Rustic Bunting","Rusty Blackbird","Sabine's Gull","Sage Grouse","Sage Sparrow","Sage Thrasher","Saltmarsh Sharp-tailed Sparrow","Sanderling","Sandhill Crane","Sandwich Tern","Savannah Sparrow","Say's Phoebe","Scaled Quail","Scaly-naped Pigeon","Scarlet Ibis","Scarlet Tanager","Scissor-tailed Flycatcher","Scott's Oriole","Seaside Sparrow","Sedge Wren","Semipalmated Plover","Semipalmated Sandpiper","Sharp-shinned Hawk","Sharp-tailed Grouse","Sharp-tailed Sandpiper","Shiny Cowbird","Short-billed Dowitcher","Short-eared Owl","Short-tailed Albatross","Short-tailed Hawk","Short-tailed Shearwater","Shy Albatross","Siberian Accentor","Siberian Blue Robin","Siberian Flycatcher","Siberian Rubythroat","Sky Lark","Slate-throated Redstart","Slaty-backed Gull","Slender-billed Curlew","Smew","Smith's Longspur","Smooth-billed Ani","Snail Kite","Snow Bunting","Snow Goose","Snowy Egret","Snowy Owl","Snowy Plover","Solitary Sandpiper","Song Sparrow","Sooty Shearwater","Sooty Tern","Sora","South Polar Skua","Southern Martin","Spectacled Eider","Spoonbill Sandpiper","Spot-billed Duck","Spot-breasted Oriole","Spotted Dove","Spotted Owl","Spotted Rail","Spotted Redshank","Spotted Sandpiper","Spotted Towhee","Sprague's Pipit","Spruce Grouse","Stejneger's Petrel","Steller's Eider","Steller's Jay","Steller's Sea-Eagle","Stilt Sandpiper","Stonechat","Streak-backed Oriole","Streaked Shearwater","Strickland's Woodpecker","Stripe-headed Tanager","Sulphur-bellied Flycatcher","Summer Tanager","Surf Scoter","Surfbird","Swainson's Hawk","Swainson's Thrush","Swainson's Warbler","Swallow-tailed Kite","Swamp Sparrow","Tamaulipas Crow","Tawny-shouldered Blackbird","Temminck's Stint","Tennessee Warbler","Terek Sandpiper","Thayer's Gull","Thick-billed Kingbird","Thick-billed Murre","Thick-billed Parrot","Thick-billed Vireo","Three-toed Woodpecker","Townsend's Solitaire","Townsend's Warbler","Tree Pipit","Tree Swallow","Tricolored Blackbird","Tricolored Heron","Tropical Kingbird","Tropical Parula","Trumpeter Swan","Tufted Duck","Tufted Flycatcher","Tufted Puffin","Tufted Titmouse","Tundra Swan","Turkey Vulture","Upland Sandpiper","Varied Bunting","Varied Thrush","Variegated Flycatcher","Vaux's Swift","Veery","Verdin","Vermilion Flycatcher","Vesper Sparrow","Violet-crowned Hummingbird","Violet-green Swallow","Virginia Rail","Virginia's Warbler","Wandering Albatross","Wandering Tattler","Warbling Vireo","Wedge-rumped Storm-Petrel","Wedge-tailed Shearwater","Western Bluebird","Western Grebe","Western Gull","Western Kingbird","Western Meadowlark","Western Reef-Heron","Western Sandpiper","Western Screech-Owl","Western Scrub-Jay","Western Tanager","Western Wood-Pewee","Whimbrel","Whip-poor-will","Whiskered Auklet","Whiskered Screech-Owl","Whiskered Tern","White Ibis","White Wagtail","White-breasted Nuthatch","White-cheeked Pintail","White-chinned Petrel","White-collared Seedeater","White-collared Swift","White-crowned Pigeon","White-crowned Sparrow","White-eared Hummingbird","White-eyed Vireo","White-faced Ibis","White-faced Storm-Petrel","White-headed Woodpecker","White-rumped Sandpiper","White-tailed Eagle","White-tailed Hawk","White-tailed Kite","White-tailed Ptarmigan","White-tailed Tropicbird","White-throated Needletail","White-throated Robin","White-throated Sparrow","White-throated Swift","White-tipped Dove","White-winged Crossbill","White-winged Dove","White-winged Parakeet","White-winged Scoter","White-winged Tern","Whooper Swan","Whooping Crane","Wild Turkey","Willet","Williamson's Sapsucker","Willow Flycatcher","Willow Ptarmigan","Wilson's Phalarope","Wilson's Plover","Wilson's Storm-Petrel","Wilson's Warbler","Winter Wren","Wood Duck","Wood Sandpiper","Wood Stork","Wood Thrush","Wood Warbler","Worm-eating Warbler","Worthen's Sparrow","Wrentit","Xantus's Hummingbird","Xantus's Murrelet","Yellow Bittern","Yellow Grosbeak","Yellow Rail","Yellow Wagtail","Yellow Warbler","Yellow-bellied Flycatcher","Yellow-bellied Sapsucker","Yellow-billed Cuckoo","Yellow-billed Loon","Yellow-billed Magpie","Yellow-breasted Bunting","Yellow-breasted Chat","Yellow-crowned Night-Heron","Yellow-eyed Junco","Yellow-faced Grassquit","Yellow-footed Gull","Yellow-green Vireo","Yellow-headed Blackbird","Yellow-legged Gull","Yellow-nosed Albatross","Yellow-rumped Warbler","Yellow-throated Vireo","Yellow-throated Warbler","Yucatan Vireo","Zenaida Dove","Zone-tailed Hawk"],vFe=["Abyssinian","American Bobtail","American Curl","American Shorthair","American Wirehair","Balinese","Bengal","Birman","Bombay","British Shorthair","Burmese","Chartreux","Chausie","Cornish Rex","Devon Rex","Donskoy","Egyptian Mau","Exotic Shorthair","Havana","Highlander","Himalayan","Japanese Bobtail","Korat","Kurilian Bobtail","LaPerm","Maine Coon","Manx","Minskin","Munchkin","Nebelung","Norwegian Forest Cat","Ocicat","Ojos Azules","Oriental","Persian","Peterbald","Pixiebob","Ragdoll","Russian Blue","Savannah","Scottish Fold","Selkirk Rex","Serengeti","Siamese","Siberian","Singapura","Snowshoe","Sokoke","Somali","Sphynx","Thai","Tonkinese","Toyger","Turkish Angora","Turkish Van"],SFe=["Amazon River Dolphin","Arnoux's Beaked Whale","Atlantic Humpbacked Dolphin","Atlantic Spotted Dolphin","Atlantic White-Sided Dolphin","Australian Snubfin Dolphin","Australian humpback Dolphin","Blue Whale","Bottlenose Dolphin","Bryde’s whale","Burrunan Dolphin","Chilean Dolphin","Chinese River Dolphin","Chinese White Dolphin","Clymene Dolphin","Commerson’s Dolphin","Costero","Dusky Dolphin","False Killer Whale","Fin Whale","Fraser’s Dolphin","Ganges River Dolphin","Guiana Dolphin","Heaviside’s Dolphin","Hector’s Dolphin","Hourglass Dolphin","Humpback whale","Indo-Pacific Bottlenose Dolphin","Indo-Pacific Hump-backed Dolphin","Irrawaddy Dolphin","Killer Whale (Orca)","La Plata Dolphin","Long-Beaked Common Dolphin","Long-finned Pilot Whale","Longman's Beaked Whale","Melon-headed Whale","Northern Rightwhale Dolphin","Omura’s whale","Pacific White-Sided Dolphin","Pantropical Spotted Dolphin","Peale’s Dolphin","Pygmy Killer Whale","Risso’s Dolphin","Rough-Toothed Dolphin","Sei Whale","Short-Beaked Common Dolphin","Short-finned Pilot Whale","Southern Bottlenose Whale","Southern Rightwhale Dolphin","Sperm Whale","Spinner Dolphin","Striped Dolphin","Tucuxi","White-Beaked Dolphin"],EFe=["Aberdeen Angus","Abergele","Abigar","Abondance","Abyssinian Shorthorned Zebu","Aceh","Achham","Adamawa","Adaptaur","Afar","Africangus","Afrikaner","Agerolese","Alambadi","Alatau","Albanian","Albera","Alderney","Alentejana","Aleutian wild cattle","Aliad Dinka","Alistana-Sanabresa","Allmogekor","Alur","American","American Angus","American Beef Friesian","American Brown Swiss","American Milking Devon","American White Park","Amerifax","Amrit Mahal","Amsterdam Island cattle","Anatolian Black","Andalusian Black","Andalusian Blond","Andalusian Grey","Angeln","Angoni","Ankina","Ankole","Ankole-Watusi","Aracena","Arado","Argentine Criollo","Argentine Friesian","Armorican","Arouquesa","Arsi","Asturian Mountain","Asturian Valley","Aubrac","Aulie-Ata","Aure et Saint-Girons","Australian Braford","Australian Brangus","Australian Charbray","Australian Friesian Sahiwal","Australian Lowline","Australian Milking Zebu","Australian Shorthorn","Austrian Simmental","Austrian Yellow","Avileña-Negra Ibérica","Avétonou","Aweil Dinka","Ayrshire","Azaouak","Azebuado","Azerbaijan Zebu","Azores","Bachaur cattle","Baherie cattle","Bakosi cattle","Balancer","Baoule","Bargur cattle","Barrosã","Barzona","Bazadaise","Beef Freisian","Beefalo","Beefmaker","Beefmaster","Begayt","Belgian Blue","Belgian Red","Belgian Red Pied","Belgian White-and-Red","Belmont Red","Belted Galloway","Bernese","Berrenda cattle","Betizu","Bianca Modenese","Blaarkop","Black Angus","Black Baldy","Black Hereford","Blanca Cacereña","Blanco Orejinegro BON","Blonde d'Aquitaine","Blue Albion","Blue Grey","Bohuskulla","Bonsmara","Boran","Boškarin","Braford","Brahman","Brahmousin","Brangus","Braunvieh","Brava","Breed","British Friesian","British White","Brown Carpathian","Brown Caucasian","Brown Swiss","Bue Lingo","Burlina","Bushuyev","Butana cattle","Buša cattle","Cachena","Caldelana","Camargue","Campbell Island cattle","Canadian Speckle Park","Canadienne","Canaria","Canchim","Caracu","Carinthian Blondvieh","Carora","Charbray","Charolais","Chateaubriand","Chiangus","Chianina","Chillingham cattle","Chinese Black Pied","Cholistani","Coloursided White Back","Commercial","Corriente","Corsican cattle","Costeño con Cuernos","Crioulo Lageano","Cárdena Andaluza","Dajal","Dangi cattle","Danish Black-Pied","Danish Jersey","Danish Red","Deep Red cattle","Deoni","Devon","Dexter cattle","Dhanni","Doayo cattle","Doela","Drakensberger","Droughtmaster","Dulong'","Dutch Belted","Dutch Friesian","Dwarf Lulu","Dølafe","East Anatolian Red","Eastern Finncattle","Eastern Red Polled","Enderby Island cattle","English Longhorn","Ennstaler Bergscheck","Estonian Holstein","Estonian Native","Estonian Red cattle","Finncattle","Finnish Ayrshire","Finnish Holstein-Friesian","Fjäll","Fleckvieh","Florida Cracker cattle","Fogera","French Simmental","Fribourgeoise","Friesian Red and White","Fulani Sudanese","Fēng Cattle","Galician Blond","Galloway cattle","Gangatiri","Gaolao","Garvonesa","Gascon cattle","Gelbvieh","Georgian Mountain cattle","German Angus","German Black Pied Dairy","German Black Pied cattle","German Red Pied","Gir","Glan cattle","Gloucester","Gobra","Greek Shorthorn","Greek Steppe","Greyman cattle","Gudali","Guernsey cattle","Guzerá","Hallikar4","Hanwoo","Hariana cattle","Hartón del Valle","Harzer Rotvieh","Hays Converter","Heck cattle","Hereford","Herens","Highland cattle","Hinterwald","Holando-Argentino","Holstein Friesian cattle","Horro","Hungarian Grey","Huáng Cattle","Hybridmaster","Iberian cattle","Icelandic","Illawarra cattle","Improved Red and White","Indo-Brazilian","Irish Moiled","Israeli Holstein","Israeli Red","Istoben cattle","Istrian cattle","Jamaica Black","Jamaica Hope","Jamaica Red","Japanese Brown","Jarmelista","Javari cattle","Jersey cattle","Jutland cattle","Kabin Buri cattle","Kalmyk cattle","Kamphaeng Saen cattle","Kangayam","Kankrej","Karan Swiss","Kasaragod Dwarf cattle","Kathiawadi","Kazakh Whiteheaded","Kenana cattle","Kenkatha cattle","Kerry cattle","Kherigarh","Khillari cattle","Kholomogory","Korat Wagyu","Kostroma cattle","Krishna Valley cattle","Kurgan cattle","Kuri","La Reina cattle","Lakenvelder cattle","Lampurger","Latvian Blue","Latvian Brown","Latvian Danish Red","Lebedyn","Levantina","Limia cattle","Limousin","Limpurger","Lincoln Red","Lineback","Lithuanian Black-and-White","Lithuanian Light Grey","Lithuanian Red","Lithuanian White-Backed","Lohani cattle","Lourdais","Lucerna cattle","Luing","Madagascar Zebu","Madura","Maine-Anjou","Malnad Gidda","Malvi","Mandalong Special","Mantequera Leonesa","Maramureş Brown","Marchigiana","Maremmana","Marinhoa","Maronesa","Masai","Mashona","Menorquina","Mertolenga","Meuse-Rhine-Issel","Mewati","Milking Shorthorn","Minhota","Mirandesa","Mirkadim","Mocăniţă","Mollie","Monchina","Mongolian","Montbéliarde","Morucha","Murboden","Murnau-Werdenfels","Murray Grey","Muturu","N'Dama","Nagori","Negra Andaluza","Nelore","Nguni","Nimari","Normande","North Bengal Grey","Northern Finncattle","Northern Shorthorn","Norwegian Red","Ongole","Original Simmental","Pajuna","Palmera","Pantaneiro","Parda Alpina","Parthenaise","Pasiega","Pembroke","Philippine Native","Pie Rouge des Plaines","Piedmontese cattle","Pineywoods","Pinzgauer","Pirenaica","Podolac","Podolica","Polish Black-and-White","Polish Red","Poll Shorthorn","Polled Hereford","Polled Shorthorn","Ponwar","Preta","Pulikulam","Punganur","Pustertaler Sprinzen","Qinchaun","Queensland Miniature Boran","RX3","Ramo Grande","Randall","Raramuri Criollo","Rathi","Raya","Red Angus","Red Brangus","Red Chittagong","Red Fulani","Red Gorbatov","Red Holstein","Red Kandhari","Red Mingrelian","Red Poll","Red Polled Østland","Red Sindhi","Retinta","Riggit Galloway","Ringamåla","Rohjan","Romagnola","Romanian Bălţata","Romanian Steppe Gray","Romosinuano","Russian Black Pied","Rätisches Grauvieh","Sahiwal","Salers","Salorn","Sanga","Sanhe","Santa Cruz","Santa Gertrudis","Sayaguesa","Schwyz","Selembu","Senepol","Serbian Pied","Serbian Steppe","Sheko","Shetland","Shorthorn","Siboney de Cuba","Simbrah","Simford","Simmental","Siri","South Devon","Spanish Fighting Bull","Speckle Park","Square Meater","Sussex","Swedish Friesian","Swedish Polled","Swedish Red Pied","Swedish Red Polled","Swedish Red-and-White","Tabapuã","Tarentaise","Tasmanian Grey","Tauros","Telemark","Texas Longhorn","Texon","Thai Black","Thai Fighting Bull","Thai Friesian","Thai Milking Zebu","Tharparkar","Tswana","Tudanca","Tuli","Tulim","Turkish Grey Steppe","Tux-Zillertal","Tyrol Grey","Ukrainian Grey","Umblachery","Valdostana Castana","Valdostana Pezzata Nera","Valdostana Pezzata Rossa","Vaynol","Vechur8","Vestland Fjord","Vestland Red Polled","Vianesa","Volinian Beef","Vorderwald","Vosgienne","Väneko","Waguli","Wagyu","Wangus","Welsh Black","Western Finncattle","White Cáceres","White Fulani","White Lamphun","White Park","Whitebred Shorthorn","Xingjiang Brown","Yakutian","Yanbian","Yanhuang","Yurino","Zebu","Évolène cattle","Żubroń"],xFe=["African Slender-snouted Crocodile","Alligator mississippiensis","American Crocodile","Australian Freshwater Crocodile","Black Caiman","Broad-snouted Caiman","Chinese Alligator","Cuban Crocodile","Cuvier’s Dwarf Caiman","Dwarf Crocodile","Gharial","Morelet’s Crocodile","Mugger Crocodile","New Guinea Freshwater Crocodile","Nile Crocodile","Orinoco Crocodile","Philippine Crocodile","Saltwater Crocodile","Schneider’s Smooth-fronted Caiman","Siamese Crocodile","Spectacled Caiman","Tomistoma","West African Crocodile","Yacare Caiman"],wFe=["Affenpinscher","Afghan Hound","Aidi","Airedale Terrier","Akbash","Akita","Alano Español","Alapaha Blue Blood Bulldog","Alaskan Husky","Alaskan Klee Kai","Alaskan Malamute","Alopekis","Alpine Dachsbracke","American Bulldog","American Bully","American Cocker Spaniel","American English Coonhound","American Foxhound","American Hairless Terrier","American Pit Bull Terrier","American Staffordshire Terrier","American Water Spaniel","Andalusian Hound","Anglo-Français de Petite Vénerie","Appenzeller Sennenhund","Ariegeois","Armant","Armenian Gampr dog","Artois Hound","Australian Cattle Dog","Australian Kelpie","Australian Shepherd","Australian Stumpy Tail Cattle Dog","Australian Terrier","Austrian Black and Tan Hound","Austrian Pinscher","Azawakh","Bakharwal dog","Banjara Hound","Barbado da Terceira","Barbet","Basenji","Basque Shepherd Dog","Basset Artésien Normand","Basset Bleu de Gascogne","Basset Fauve de Bretagne","Basset Hound","Bavarian Mountain Hound","Beagle","Beagle-Harrier","Bearded Collie","Beauceron","Bedlington Terrier","Belgian Shepherd","Bergamasco Shepherd","Berger Picard","Bernese Mountain Dog","Bhotia","Bichon Frisé","Billy","Black Mouth Cur","Black Norwegian Elkhound","Black Russian Terrier","Black and Tan Coonhound","Bloodhound","Blue Lacy","Blue Picardy Spaniel","Bluetick Coonhound","Boerboel","Bohemian Shepherd","Bolognese","Border Collie","Border Terrier","Borzoi","Bosnian Coarse-haired Hound","Boston Terrier","Bouvier des Ardennes","Bouvier des Flandres","Boxer","Boykin Spaniel","Bracco Italiano","Braque Francais","Braque Saint-Germain","Braque d'Auvergne","Braque de l'Ariège","Braque du Bourbonnais","Briard","Briquet Griffon Vendéen","Brittany","Broholmer","Bruno Jura Hound","Brussels Griffon","Bucovina Shepherd Dog","Bull Arab","Bull Terrier","Bulldog","Bullmastiff","Bully Kutta","Burgos Pointer","Cairn Terrier","Campeiro Bulldog","Can de Chira","Canaan Dog","Canadian Eskimo Dog","Cane Corso","Cane Paratore","Cane di Oropa","Cantabrian Water Dog","Cardigan Welsh Corgi","Carea Castellano Manchego","Carolina Dog","Carpathian Shepherd Dog","Catahoula Leopard Dog","Catalan Sheepdog","Caucasian Shepherd Dog","Cavalier King Charles Spaniel","Central Asian Shepherd Dog","Cesky Fousek","Cesky Terrier","Chesapeake Bay Retriever","Chien Français Blanc et Noir","Chien Français Blanc et Orange","Chien Français Tricolore","Chihuahua","Chilean Terrier","Chinese Chongqing Dog","Chinese Crested Dog","Chinook","Chippiparai","Chongqing dog","Chortai","Chow Chow","Cimarrón Uruguayo","Cirneco dell'Etna","Clumber Spaniel","Colombian fino hound","Coton de Tulear","Cretan Hound","Croatian Sheepdog","Curly-Coated Retriever","Cursinu","Czechoslovakian Wolfdog","Cão Fila de São Miguel","Cão da Serra de Aires","Cão de Castro Laboreiro","Cão de Gado Transmontano","Dachshund","Dalmatian","Dandie Dinmont Terrier","Danish-Swedish Farmdog","Denmark Feist","Dingo","Doberman Pinscher","Dogo Argentino","Dogo Guatemalteco","Dogo Sardesco","Dogue Brasileiro","Dogue de Bordeaux","Drentse Patrijshond","Drever","Dunker","Dutch Shepherd","Dutch Smoushond","East European Shepherd","East Siberian Laika","English Cocker Spaniel","English Foxhound","English Mastiff","English Setter","English Shepherd","English Springer Spaniel","English Toy Terrier","Entlebucher Mountain Dog","Estonian Hound","Estrela Mountain Dog","Eurasier","Field Spaniel","Fila Brasileiro","Finnish Hound","Finnish Lapphund","Finnish Spitz","Flat-Coated Retriever","French Bulldog","French Spaniel","Galgo Español","Galician Shepherd Dog","Garafian Shepherd","Gascon Saintongeois","Georgian Shepherd","German Hound","German Longhaired Pointer","German Pinscher","German Roughhaired Pointer","German Shepherd Dog","German Shorthaired Pointer","German Spaniel","German Spitz","German Wirehaired Pointer","Giant Schnauzer","Glen of Imaal Terrier","Golden Retriever","Gordon Setter","Gończy Polski","Grand Anglo-Français Blanc et Noir","Grand Anglo-Français Blanc et Orange","Grand Anglo-Français Tricolore","Grand Basset Griffon Vendéen","Grand Bleu de Gascogne","Grand Griffon Vendéen","Great Dane","Greater Swiss Mountain Dog","Greek Harehound","Greek Shepherd","Greenland Dog","Greyhound","Griffon Bleu de Gascogne","Griffon Fauve de Bretagne","Griffon Nivernais","Gull Dong","Gull Terrier","Hamiltonstövare","Hanover Hound","Harrier","Havanese","Hierran Wolfdog","Hokkaido","Hovawart","Huntaway","Hygen Hound","Hällefors Elkhound","Ibizan Hound","Icelandic Sheepdog","Indian Spitz","Indian pariah dog","Irish Red and White Setter","Irish Setter","Irish Terrier","Irish Water Spaniel","Irish Wolfhound","Istrian Coarse-haired Hound","Istrian Shorthaired Hound","Italian Greyhound","Jack Russell Terrier","Jagdterrier","Japanese Chin","Japanese Spitz","Japanese Terrier","Jindo","Jonangi","Kai Ken","Kaikadi","Kangal Shepherd Dog","Kanni","Karakachan dog","Karelian Bear Dog","Kars","Karst Shepherd","Keeshond","Kerry Beagle","Kerry Blue Terrier","King Charles Spaniel","King Shepherd","Kintamani","Kishu","Kokoni","Kombai","Komondor","Kooikerhondje","Koolie","Koyun dog","Kromfohrländer","Kuchi","Kuvasz","Labrador Retriever","Lagotto Romagnolo","Lakeland Terrier","Lancashire Heeler","Landseer","Lapponian Herder","Large Münsterländer","Leonberger","Levriero Sardo","Lhasa Apso","Lithuanian Hound","Lupo Italiano","Löwchen","Mackenzie River Husky","Magyar agár","Mahratta Greyhound","Maltese","Manchester Terrier","Maremmano-Abruzzese Sheepdog","McNab dog","Miniature American Shepherd","Miniature Bull Terrier","Miniature Fox Terrier","Miniature Pinscher","Miniature Schnauzer","Molossus of Epirus","Montenegrin Mountain Hound","Mountain Cur","Mountain Feist","Mucuchies","Mudhol Hound","Mudi","Neapolitan Mastiff","New Guinea Singing Dog","New Zealand Heading Dog","Newfoundland","Norfolk Terrier","Norrbottenspets","Northern Inuit Dog","Norwegian Buhund","Norwegian Elkhound","Norwegian Lundehund","Norwich Terrier","Nova Scotia Duck Tolling Retriever","Old Croatian Sighthound","Old Danish Pointer","Old English Sheepdog","Old English Terrier","Olde English Bulldogge","Otterhound","Pachon Navarro","Paisley Terrier","Pampas Deerhound","Papillon","Parson Russell Terrier","Pastore della Lessinia e del Lagorai","Patagonian Sheepdog","Patterdale Terrier","Pekingese","Pembroke Welsh Corgi","Perro Majorero","Perro de Pastor Mallorquin","Perro de Presa Canario","Perro de Presa Mallorquin","Peruvian Inca Orchid","Petit Basset Griffon Vendéen","Petit Bleu de Gascogne","Phalène","Pharaoh Hound","Phu Quoc Ridgeback","Picardy Spaniel","Plott Hound","Plummer Terrier","Podenco Canario","Podenco Valenciano","Pointer","Poitevin","Polish Greyhound","Polish Hound","Polish Lowland Sheepdog","Polish Tatra Sheepdog","Pomeranian","Pont-Audemer Spaniel","Poodle","Porcelaine","Portuguese Podengo","Portuguese Pointer","Portuguese Water Dog","Posavac Hound","Pražský Krysařík","Pshdar Dog","Pudelpointer","Pug","Puli","Pumi","Pungsan Dog","Pyrenean Mastiff","Pyrenean Mountain Dog","Pyrenean Sheepdog","Rafeiro do Alentejo","Rajapalayam","Rampur Greyhound","Rat Terrier","Ratonero Bodeguero Andaluz","Ratonero Mallorquin","Ratonero Murciano de Huerta","Ratonero Valenciano","Redbone Coonhound","Rhodesian Ridgeback","Romanian Mioritic Shepherd Dog","Romanian Raven Shepherd Dog","Rottweiler","Rough Collie","Russian Spaniel","Russian Toy","Russo-European Laika","Saarloos Wolfdog","Sabueso Español","Saint Bernard","Saint Hubert Jura Hound","Saint-Usuge Spaniel","Saluki","Samoyed","Sapsali","Sarabi dog","Sardinian Shepherd Dog","Schapendoes","Schillerstövare","Schipperke","Schweizer Laufhund","Schweizerischer Niederlaufhund","Scottish Deerhound","Scottish Terrier","Sealyham Terrier","Segugio Italiano","Segugio Maremmano","Segugio dell'Appennino","Seppala Siberian Sleddog","Serbian Hound","Serbian Tricolour Hound","Serrano Bulldog","Shar Pei","Shetland Sheepdog","Shiba Inu","Shih Tzu","Shikoku","Shiloh Shepherd","Siberian Husky","Silken Windhound","Silky Terrier","Sinhala Hound","Skye Terrier","Sloughi","Slovakian Wirehaired Pointer","Slovenský Cuvac","Slovenský Kopov","Smalandstövare","Small Greek domestic dog","Small Münsterländer","Smooth Collie","Smooth Fox Terrier","Soft-Coated Wheaten Terrier","South Russian Ovcharka","Spanish Mastiff","Spanish Water Dog","Spinone Italiano","Sporting Lucas Terrier","Stabyhoun","Staffordshire Bull Terrier","Standard Schnauzer","Stephens Stock","Styrian Coarse-haired Hound","Sussex Spaniel","Swedish Elkhound","Swedish Lapphund","Swedish Vallhund","Swedish White Elkhound","Taigan","Taiwan Dog","Tamaskan Dog","Teddy Roosevelt Terrier","Telomian","Tenterfield Terrier","Terrier Brasileiro","Thai Bangkaew Dog","Thai Ridgeback","Tibetan Mastiff","Tibetan Spaniel","Tibetan Terrier","Tornjak","Tosa","Toy Fox Terrier","Toy Manchester Terrier","Transylvanian Hound","Treeing Cur","Treeing Feist","Treeing Tennessee Brindle","Treeing Walker Coonhound","Trigg Hound","Tyrolean Hound","Vikhan","Villano de Las Encartaciones","Villanuco de Las Encartaciones","Vizsla","Volpino Italiano","Weimaraner","Welsh Sheepdog","Welsh Springer Spaniel","Welsh Terrier","West Highland White Terrier","West Siberian Laika","Westphalian Dachsbracke","Wetterhoun","Whippet","White Shepherd","White Swiss Shepherd Dog","Wire Fox Terrier","Wirehaired Pointing Griffon","Wirehaired Vizsla","Xiasi Dog","Xoloitzcuintli","Yakutian Laika","Yorkshire Terrier","Šarplaninac"],kFe=["Alaska pollock","Albacore","Amur catfish","Araucanian herring","Argentine hake","Asari","Asian swamp eel","Atlantic cod","Atlantic herring","Atlantic horse mackerel","Atlantic mackerel","Atlantic menhaden","Atlantic salmon","Bigeye scad","Bigeye tuna","Bighead carp","Black carp","Blood cockle","Blue swimming crab","Blue whiting","Bombay-duck","Bonga shad","California pilchard","Cape horse mackerel","Capelin","Catla","Channel catfish","Chilean jack mackerel","Chinese perch","Chinese softshell turtle","Chub mackerel","Chum salmon","Common carp","Crucian carp","Daggertooth pike conger","European anchovy","European pilchard","European sprat","Filipino Venus","Gazami crab","Goldstripe sardinella","Grass carp","Gulf menhaden","Haddock","Hilsa shad","Indian mackerel","Indian oil sardine","Iridescent shark","Japanese anchovy","Japanese cockle","Japanese common catfish","Japanese flying squid","Japanese jack mackerel","Japanese littleneck","Japanese pilchard","Jumbo flying squid","Kawakawa","Korean bullhead","Largehead hairtail","Longtail tuna","Madeiran sardinella","Mandarin fish","Milkfish","Mrigal carp","Narrow-barred Spanish mackerel","Nile perch","Nile tilapia","North Pacific hake","Northern snakehead","Pacific anchoveta","Pacific cod","Pacific herring","Pacific sand lance","Pacific sandlance","Pacific saury","Pacific thread herring","Peruvian anchoveta","Pink salmon","Pollock","Pond loach","Rainbow trout","Rohu","Round sardinella","Short mackerel","Silver carp","Silver cyprinid","Skipjack tuna","Southern African anchovy","Southern rough shrimp","Whiteleg shrimp","Wuchang bream","Yellow croaker","Yellowfin tuna","Yellowhead catfish","Yellowstripe scad"],CFe=["Abaco Barb","Abtenauer","Abyssinian","Aegidienberger","Akhal-Teke","Albanian Horse","Altai Horse","Altèr Real","American Albino","American Cream Draft","American Indian Horse","American Paint Horse","American Quarter Horse","American Saddlebred","American Warmblood","Andalusian Horse","Andravida Horse","Anglo-Arabian","Anglo-Arabo-Sardo","Anglo-Kabarda","Appaloosa","AraAppaloosa","Arabian Horse","Ardennes Horse","Arenberg-Nordkirchen","Argentine Criollo","Asian wild Horse","Assateague Horse","Asturcón","Augeron","Australian Brumby","Australian Draught Horse","Australian Stock Horse","Austrian Warmblood","Auvergne Horse","Auxois","Azerbaijan Horse","Azteca Horse","Baise Horse","Bale","Balearic Horse","Balikun Horse","Baluchi Horse","Banker Horse","Barb Horse","Bardigiano","Bashkir Curly","Basque Mountain Horse","Bavarian Warmblood","Belgian Half-blood","Belgian Horse","Belgian Warmblood","Bhutia Horse","Black Forest Horse","Blazer Horse","Boerperd","Borana","Boulonnais Horse","Brabant","Brandenburger","Brazilian Sport Horse","Breton Horse","Brumby","Budyonny Horse","Burguete Horse","Burmese Horse","Byelorussian Harness Horse","Calabrese Horse","Camargue Horse","Camarillo White Horse","Campeiro","Campolina","Canadian Horse","Canadian Pacer","Carolina Marsh Tacky","Carthusian Horse","Caspian Horse","Castilian Horse","Castillonnais","Catria Horse","Cavallo Romano della Maremma Laziale","Cerbat Mustang","Chickasaw Horse","Chilean Corralero","Choctaw Horse","Cleveland Bay","Clydesdale Horse","Cob","Coldblood Trotter","Colonial Spanish Horse","Colorado Ranger","Comtois Horse","Corsican Horse","Costa Rican Saddle Horse","Cretan Horse","Criollo Horse","Croatian Coldblood","Cuban Criollo","Cumberland Island Horse","Curly Horse","Czech Warmblood","Daliboz","Danish Warmblood","Danube Delta Horse","Dole Gudbrandsdal","Don","Dongola Horse","Draft Trotter","Dutch Harness Horse","Dutch Heavy Draft","Dutch Warmblood","Dzungarian Horse","East Bulgarian","East Friesian Horse","Estonian Draft","Estonian Horse","Falabella","Faroese","Finnhorse","Fjord Horse","Fleuve","Florida Cracker Horse","Foutanké","Frederiksborg Horse","Freiberger","French Trotter","Friesian Cross","Friesian Horse","Friesian Sporthorse","Furioso-North Star","Galiceño","Galician Pony","Gelderland Horse","Georgian Grande Horse","German Warmblood","Giara Horse","Gidran","Groningen Horse","Gypsy Horse","Hackney Horse","Haflinger","Hanoverian Horse","Heck Horse","Heihe Horse","Henson Horse","Hequ Horse","Hirzai","Hispano-Bretón","Holsteiner Horse","Horro","Hungarian Warmblood","Icelandic Horse","Iomud","Irish Draught","Irish Sport Horse sometimes called Irish Hunter","Italian Heavy Draft","Italian Trotter","Jaca Navarra","Jeju Horse","Jutland Horse","Kabarda Horse","Kafa","Kaimanawa Horses","Kalmyk Horse","Karabair","Karabakh Horse","Karachai Horse","Karossier","Kathiawari","Kazakh Horse","Kentucky Mountain Saddle Horse","Kiger Mustang","Kinsky Horse","Kisber Felver","Kiso Horse","Kladruber","Knabstrupper","Konik","Kundudo","Kustanair","Kyrgyz Horse","Latvian Horse","Lipizzan","Lithuanian Heavy Draught","Lokai","Losino Horse","Lusitano","Lyngshest","M'Bayar","M'Par","Mallorquín","Malopolski","Mangalarga","Mangalarga Marchador","Maremmano","Marismeño Horse","Marsh Tacky","Marwari Horse","Mecklenburger","Menorquín","Messara Horse","Metis Trotter","Mezőhegyesi Sport Horse","Međimurje Horse","Miniature Horse","Misaki Horse","Missouri Fox Trotter","Monchina","Mongolian Horse","Mongolian Wild Horse","Monterufolino","Morab","Morgan Horse","Mountain Pleasure Horse","Moyle Horse","Murakoz Horse","Murgese","Mustang Horse","Mérens Horse","Namib Desert Horse","Nangchen Horse","National Show Horse","Nez Perce Horse","Nivernais Horse","Nokota Horse","Noma","Nonius Horse","Nooitgedachter","Nordlandshest","Noriker Horse","Norman Cob","North American Single-Footer Horse","North Swedish Horse","Norwegian Coldblood Trotter","Norwegian Fjord","Novokirghiz","Oberlander Horse","Ogaden","Oldenburg Horse","Orlov trotter","Ostfriesen","Paint","Pampa Horse","Paso Fino","Pentro Horse","Percheron","Persano Horse","Peruvian Paso","Pintabian","Pleven Horse","Poitevin Horse","Posavac Horse","Pottok","Pryor Mountain Mustang","Przewalski's Horse","Pura Raza Española","Purosangue Orientale","Qatgani","Quarab","Quarter Horse","Racking Horse","Retuerta Horse","Rhenish German Coldblood","Rhinelander Horse","Riwoche Horse","Rocky Mountain Horse","Romanian Sporthorse","Rottaler","Russian Don","Russian Heavy Draft","Russian Trotter","Saddlebred","Salerno Horse","Samolaco Horse","San Fratello Horse","Sarcidano Horse","Sardinian Anglo-Arab","Schleswig Coldblood","Schwarzwälder Kaltblut","Selale","Sella Italiano","Selle Français","Shagya Arabian","Shan Horse","Shire Horse","Siciliano Indigeno","Silesian Horse","Sokolsky Horse","Sorraia","South German Coldblood","Soviet Heavy Draft","Spanish Anglo-Arab","Spanish Barb","Spanish Jennet Horse","Spanish Mustang","Spanish Tarpan","Spanish-Norman Horse","Spiti Horse","Spotted Saddle Horse","Standardbred Horse","Suffolk Punch","Swedish Ardennes","Swedish Warmblood","Swedish coldblood trotter","Swiss Warmblood","Taishū Horse","Takhi","Tawleed","Tchernomor","Tennessee Walking Horse","Tersk Horse","Thoroughbred","Tiger Horse","Tinker Horse","Tolfetano","Tori Horse","Trait Du Nord","Trakehner","Tsushima","Tuigpaard","Ukrainian Riding Horse","Unmol Horse","Uzunyayla","Ventasso Horse","Virginia Highlander","Vlaamperd","Vladimir Heavy Draft","Vyatka","Waler","Waler Horse","Walkaloosa","Warlander","Warmblood","Welsh Cob","Westphalian Horse","Wielkopolski","Württemberger","Xilingol Horse","Yakutian Horse","Yili Horse","Yonaguni Horse","Zaniskari","Zhemaichu","Zweibrücker","Žemaitukas"],TFe=["Acacia-ants","Acorn-plum gall","Aerial yellowjacket","Africanized honey bee","Allegheny mound ant","Almond stone wasp","Ant","Arboreal ant","Argentine ant","Asian paper wasp","Baldfaced hornet","Bee","Bigheaded ant","Black and yellow mud dauber","Black carpenter ant","Black imported fire ant","Blue horntail woodwasp","Blue orchard bee","Braconid wasp","Bumble bee","Carpenter ant","Carpenter wasp","Chalcid wasp","Cicada killer","Citrus blackfly parasitoid","Common paper wasp","Crazy ant","Cuckoo wasp","Cynipid gall wasp","Eastern Carpenter bee","Eastern yellowjacket","Elm sawfly","Encyrtid wasp","Erythrina gall wasp","Eulophid wasp","European hornet","European imported fire ant","False honey ant","Fire ant","Forest bachac","Forest yellowjacket","German yellowjacket","Ghost ant","Giant ichneumon wasp","Giant resin bee","Giant wood wasp","Golden northern bumble bee","Golden paper wasp","Gouty oak gall","Grass Carrying Wasp","Great black wasp","Great golden digger wasp","Hackberry nipple gall parasitoid","Honey bee","Horned oak gall","Horse guard wasp","Hunting wasp","Ichneumonid wasp","Keyhole wasp","Knopper gall","Large garden bumble bee","Large oak-apple gall","Leafcutting bee","Little fire ant","Little yellow ant","Long-horned bees","Long-legged ant","Macao paper wasp","Mallow bee","Marble gall","Mossyrose gall wasp","Mud-daubers","Multiflora rose seed chalcid","Oak apple gall wasp","Oak rough bulletgall wasp","Oak saucer gall","Oak shoot sawfly","Odorous house ant","Orange-tailed bumble bee","Orangetailed potter wasp","Oriental chestnut gall wasp","Paper wasp","Pavement ant","Pigeon tremex","Pip gall wasp","Prairie yellowjacket","Pteromalid wasp","Pyramid ant","Raspberry Horntail","Red ant","Red carpenter ant","Red harvester ant","Red imported fire ant","Red wasp","Red wood ant","Red-tailed wasp","Reddish carpenter ant","Rough harvester ant","Sawfly parasitic wasp","Scale parasitoid","Silky ant","Sirex woodwasp","Siricid woodwasp","Smaller yellow ant","Southeastern blueberry bee","Southern fire ant","Southern yellowjacket","Sphecid wasp","Stony gall","Sweat bee","Texas leafcutting ant","Tiphiid wasp","Torymid wasp","Tramp ant","Valentine ant","Velvet ant","Vespid wasp","Weevil parasitoid","Western harvester ant","Western paper wasp","Western thatching ant","Western yellowjacket","White-horned horntail","Willow shoot sawfly","Woodwasp","Wool sower gall maker","Yellow Crazy Ant","Yellow and black potter wasp","Yellow-horned horntail"],AFe=["Asiatic Lion","Barbary Lion","Cape lion","Masai Lion","Northeast Congo Lion","Transvaal lion","West African Lion"],FFe=["Ace","Archie","Bailey","Bandit","Bella","Bentley","Bruno","Buddy","Charlie","Coco","Cookie","Cooper","Daisy","Dixie","Finn","Ginger","Gracie","Gus","Hank","Jack","Jax","Joey","Kobe","Leo","Lola","Louie","Lucy","Maggie","Max","Mia","Milo","Molly","Murphey","Nala","Nova","Ollie","Oreo","Rosie","Scout","Stella","Teddy","Tuffy"],_Fe=["American","American Chinchilla","American Fuzzy Lop","American Sable","Argente Brun","Belgian Hare","Beveren","Blanc de Hotot","Britannia Petite","Californian","Champagne D’Argent","Checkered Giant","Cinnamon","Crème D’Argent","Dutch","Dwarf Hotot","English Angora","English Lop","English Spot","Flemish Giant","Florida White","French Angora","French Lop","Giant Angora","Giant Chinchilla","Harlequin","Havana","Himalayan","Holland Lop","Jersey Wooly","Lilac","Lionhead","Mini Lop","Mini Rex","Mini Satin","Netherland Dwarf","New Zealand","Palomino","Polish","Rex","Rhinelander","Satin","Satin Angora","Silver","Silver Fox","Silver Marten","Standard Chinchilla","Tan","Thrianta"],RFe=["Abrocoma","Abrocoma schistacea","Aconaemys","Aconaemys porteri","African brush-tailed porcupine","Andean mountain cavy","Argentine tuco-tuco","Ashy chinchilla rat","Asiatic brush-tailed porcupine","Atherurus","Azara's agouti","Azara's tuco-tuco","Bahia porcupine","Bathyergus","Bathyergus janetta","Bathyergus suillus","Bennett's chinchilla rat","Bicolored-spined porcupine","Black agouti","Black dwarf porcupine","Black-rumped agouti","Black-tailed hairy dwarf porcupine","Bolivian chinchilla rat","Bolivian tuco-tuco","Bonetto's tuco-tuco","Brandt's yellow-toothed cavy","Brazilian guinea pig","Brazilian porcupine","Brazilian tuco-tuco","Bridge's degu","Brown hairy dwarf porcupine","Budin's chinchilla rat, A. budini","Cape porcupine","Catamarca tuco-tuco","Cavia","Central American agouti","Chacoan tuco-tuco","Chilean rock rat","Chinchilla","Coendou","Coiban agouti","Colburn's tuco-tuco","Collared tuco-tuco","Common degu","Common yellow-toothed cavy","Conover's tuco-tuco","Coruro","Crested agouti","Crested porcupine","Cryptomys","Cryptomys bocagei","Cryptomys damarensis","Cryptomys foxi","Cryptomys hottentotus","Cryptomys mechowi","Cryptomys ochraceocinereus","Cryptomys zechi","Ctenomys","Cuniculus","Cuscomys","Cuscomys ashanika","Dactylomys","Dactylomys boliviensis","Dactylomys dactylinus","Dactylomys peruanus","Dasyprocta","Domestic guinea pig","Emily's tuco-tuco","Erethizon","Famatina chinchilla rat","Frosted hairy dwarf porcupine","Fukomys","Fukomys amatus","Fukomys anselli","Fukomys bocagei","Fukomys damarensis","Fukomys darlingi","Fukomys foxi","Fukomys ilariae","Fukomys kafuensis","Fukomys mechowii","Fukomys micklemi","Fukomys occlusus","Fukomys ochraceocinereus","Fukomys whytei","Fukomys zechi","Furtive tuco-tuco","Galea","Georychus","Georychus capensis","Golden viscacha-rat","Goya tuco-tuco","Greater guinea pig","Green acouchi","Haig's tuco-tuco","Heliophobius","Heliophobius argenteocinereus","Heterocephalus","Heterocephalus glaber","Highland tuco-tuco","Hystrix","Indian porcupine","Isla Mocha degu","Kalinowski agouti","Kannabateomys","Kannabateomys amblyonyx","Lagidium","Lagostomus","Lewis' tuco-tuco","Long-tailed chinchilla","Long-tailed porcupine","Los Chalchaleros' viscacha-rat","Lowland paca","Magellanic tuco-tuco","Malayan porcupine","Maule tuco-tuco","Mendoza tuco-tuco","Mexican agouti","Mexican hairy dwarf porcupine","Microcavia","Montane guinea pig","Moon-toothed degu","Mottled tuco-tuco","Mountain degu","Mountain paca","Mountain viscacha-rat","Myoprocta","Natterer's tuco-tuco","North American porcupine","Northern viscacha","Octodon","Octodontomys","Octomys","Olallamys","Olallamys albicauda","Olallamys edax","Orinoco agouti","Paraguaian hairy dwarf porcupine","Pearson's tuco-tuco","Peruvian tuco-tuco","Philippine porcupine","Pipanacoctomys","Plains viscacha","Plains viscacha-rat","Porteous' tuco-tuco","Punta de Vacas chinchilla rat","Red acouchi","Red-rumped agouti","Reddish tuco-tuco","Rio Negro tuco-tuco","Robust tuco-tuco","Roosmalen's dwarf porcupine","Rothschild's porcupine","Ruatan Island agouti","Sage's rock rat","Salinoctomys","Salta tuco-tuco","San Luis tuco-tuco","Santa Catarina's guinea pig","Shiny guinea pig","Shipton's mountain cavy","Short-tailed chinchilla","Silky tuco-tuco","Social tuco-tuco","Southern mountain cavy","Southern tuco-tuco","Southern viscacha","Spalacopus","Spix's yellow-toothed cavy","Steinbach's tuco-tuco","Streaked dwarf porcupine","Strong tuco-tuco","Stump-tailed porcupine","Sumatran porcupine","Sunda porcupine","Talas tuco-tuco","Tawny tuco-tuco","Thick-spined porcupine","Tiny tuco-tuco","Trichys","Tucuman tuco-tuco","Tympanoctomys","Uspallata chinchilla rat","White-toothed tuco-tuco","Wolffsohn's viscacha"],DFe=["Abaco Island boa","Aesculapian snake","African beaked snake","African puff adder","African rock python","African twig snake","African wolf snake","Amazon tree boa","Amazonian palm viper","American Vine Snake","American copperhead","Amethystine python","Anaconda","Andaman cat snake","Andaman cobra","Angolan python","Annulated sea snake","Arabian cobra","Arafura file snake","Arizona black rattlesnake","Arizona coral snake","Aruba rattlesnake","Asian Vine Snake, Whip Snake","Asian cobra","Asian keelback","Asian pipe snake","Asp","Asp viper","Assam keelback","Australian copperhead","Australian scrub python","Baird's rat snake","Baja California lyresnake","Ball Python","Ball python","Bamboo pitviper","Bamboo viper","Banded Flying Snake","Banded cat-eyed snake","Banded krait","Banded pitviper","Banded water cobra","Barbour's pit viper","Barred wolf snake","Beaked sea snake","Beauty rat snake","Beddome's cat snake","Beddome's coral snake","Bimini racer","Bird snake","Bismarck ringed python","Black headed python","Black krait","Black mamba","Black rat snake","Black snake","Black tree cobra","Black-banded trinket snake","Black-headed snake","Black-necked cobra","Black-necked spitting cobra","Black-speckled palm-pitviper","Black-striped keelback","Black-tailed horned pit viper","Blanding's tree snake","Blind snake","Blonde hognose snake","Blood python","Blue krait","Blunt-headed tree snake","Bluntnose viper","Boa","Boa constrictor","Bocourt's water snake","Boelen python","Boiga","Bolivian anaconda","Boomslang","Bornean pitviper","Borneo short-tailed python","Brahminy blind snake","Brazilian coral snake","Brazilian mud Viper","Brazilian smooth snake","Bredl's python","Brongersma's pitviper","Brown snake","Brown spotted pitviper[4]","Brown tree snake","Brown water python","Brown white-lipped python","Buff striped keelback","Bull snake","Burmese keelback","Burmese krait","Burmese python","Burrowing cobra","Burrowing viper","Bush viper","Bushmaster","Buttermilk racer","Calabar python","California kingsnake","Canebrake","Cantil","Cantor's pitviper","Cape cobra","Cape coral snake","Cape gopher snake","Carpet viper","Cascabel","Caspian cobra","Cat snake","Cat-eyed night snake","Cat-eyed snake","Central American lyre snake","Central ranges taipan","Centralian carpet python","Ceylon krait","Chappell Island tiger snake","Checkered garter snake","Checkered keelback","Chicken snake","Chihuahuan ridge-nosed rattlesnake","Children's python","Chinese tree viper","Coachwhip snake","Coastal carpet python","Coastal taipan","Cobra","Collett's snake","Colorado desert sidewinder","Common adder","Common cobra","Common garter snake","Common ground snake","Common keelback","Common lancehead","Common tiger snake","Common worm snake","Congo snake","Congo water cobra","Copperhead","Coral snake","Corn snake","Coronado Island rattlesnake","Cottonmouth","Crossed viper","Crowned snake","Cuban boa","Cuban wood snake","Cyclades blunt-nosed viper","Dauan Island water python","De Schauensee's anaconda","Death Adder","Desert death adder","Desert kingsnake","Desert woma python","Diamond python","Dog-toothed cat snake","Down's tiger snake","Dubois's sea snake","Dumeril's boa","Durango rock rattlesnake","Dusky pigmy rattlesnake","Dusty hognose snake","Dwarf beaked snake","Dwarf boa","Dwarf pipe snake","Dwarf sand adder","Eastern brown snake","Eastern coral snake","Eastern diamondback rattlesnake","Eastern green mamba","Eastern hognose snake","Eastern lyre snake","Eastern mud snake","Eastern racer","Eastern tiger snake","Eastern water cobra","Eastern yellowbelly sad racer","Egg-eater","Egyptian asp","Egyptian cobra","Elegant pitviper","Emerald tree boa","Equatorial spitting cobra","European asp","European smooth snake","Eyelash palm-pitviper","Eyelash pit viper","Eyelash viper","False cobra","False horned viper","False water cobra","Fan-Si-Pan horned pitviper","Fea's viper","Fer-de-lance","Fierce snake","Fifty pacer","Fishing snake","Flat-nosed pitviper","Flinders python","Flying snake","Forest cobra","Forest flame snake","Forsten's cat snake","Fox snake, three species of Pantherophis","Gaboon viper","Garter snake","Giant Malagasy hognose snake","Godman's pit viper","Gold tree cobra","Gold-ringed cat snake","Golden tree snake","Grand Canyon rattlesnake","Grass snake","Gray cat snake","Great Basin rattlesnake","Great Lakes bush viper","Great Plains rat snake","Green anaconda","Green cat-eyed snake","Green mamba","Green palm viper","Green rat snake","Green snake","Green tree pit viper","Green tree python","Grey Lora","Grey-banded kingsnake","Ground snake","Guatemalan palm viper","Guatemalan tree viper","Habu","Habu pit viper","Hagen's pitviper","Hairy bush viper","Halmahera python","Hardwicke's sea snake","Harlequin coral snake","High Woods coral snake","Hill keelback","Himalayan keelback","Hogg Island boa","Hognose snake","Hognosed viper","Honduran palm viper","Hook Nosed Sea Snake","Hopi rattlesnake","Horned adder","Horned desert viper","Horned viper","Horseshoe pitviper","Hundred pacer","Hutton's tree viper","Ikaheka snake","Indian cobra","Indian flying snake","Indian krait","Indian python","Indian tree viper","Indigo snake","Indochinese spitting cobra","Indonesian water python","Inland carpet python","Inland taipan","Jamaican Tree Snake","Jamaican boa","Jan's hognose snake","Japanese forest rat snake","Japanese rat snake","Japanese striped snake","Javan spitting cobra","Jerdon's pitviper","Jumping viper","Jungle carpet python","Kanburian pit viper","Kaulback's lance-headed pitviper","Kayaudi dwarf reticulated python","Kaznakov's viper","Keelback","Kham Plateau pitviper","Khasi Hills keelback","King Island tiger snake","King brown","King cobra","King rat snake","King snake","Krait","Krefft's tiger snake","Lance-headed rattlesnake","Lancehead","Large shield snake","Large-eyed pitviper","Large-scaled tree viper","Leaf viper","Leaf-nosed viper","Lesser black krait","Levant viper","Long-nosed adder","Long-nosed tree snake","Long-nosed viper","Long-nosed whip snake","Long-tailed rattlesnake","Longnosed worm snake","Lora","Lyre snake","Machete savane","Macklot's python","Madagascar ground boa","Madagascar tree boa","Malabar rock pitviper","Malayan krait","Malayan long-glanded coral snake","Malayan pit viper","Malcolm's tree viper","Mamba","Mamushi","Manchurian Black Water Snake","Mandalay cobra","Mandarin rat snake","Mangrove pit viper","Mangrove snake","Mangshan pitviper","Many-banded krait","Many-banded tree snake","Many-horned adder","Many-spotted cat snake","Massasauga rattlesnake","McMahon's viper","Mexican black kingsnake","Mexican green rattlesnake","Mexican hognose snake","Mexican palm-pitviper","Mexican parrot snake","Mexican racer","Mexican vine snake","Mexican west coast rattlesnake","Midget faded rattlesnake","Milk snake","Moccasin snake","Modest keelback","Mojave desert sidewinder","Mojave rattlesnake","Mole viper","Mollucan python","Moluccan flying snake","Montpellier snake","Motuo bamboo pitviper","Mountain adder","Mozambique spitting cobra","Mud adder","Mud snake","Mussurana","Namaqua dwarf adder","Namib dwarf sand adder","Narrowhead Garter Snake","New Guinea carpet python","Nichell snake","Nicobar Island keelback","Nicobar bamboo pitviper","Night snake","Nightingale adder","Nilgiri keelback","Nitsche's bush viper","Nitsche's tree viper","North Philippine cobra","North eastern king snake","Northeastern hill krait","Northern black-tailed rattlesnake","Northern tree snake","Northern water snake","Northern white-lipped python","Northwestern carpet python","Nose-horned viper","Nubian spitting cobra","Oaxacan small-headed rattlesnake","Oenpelli python","Olive python","Olive sea snake","Orange-collared keelback","Ornate flying snake","Palestine viper","Pallas' viper","Palm viper","Papuan python","Paradise flying snake","Parrot snake","Patchnose snake","Paupan taipan","Pelagic sea snake","Peninsula tiger snake","Peringuey's adder","Perrotet's shieldtail snake","Persian rat snake","Philippine cobra","Philippine pitviper","Pine snake","Pipe snake","Pit viper","Pointed-scaled pit viper[5]","Pope's tree viper","Portuguese viper","Prairie kingsnake","Puerto Rican boa","Puff adder","Pygmy python","Python","Queen snake","Racer","Raddysnake","Rainbow boa","Rat snake","Rattler","Rattlesnake","Red blood python","Red diamond rattlesnake","Red spitting cobra","Red-backed rat snake","Red-bellied black snake","Red-headed krait","Red-necked keelback","Red-tailed bamboo pitviper","Red-tailed boa","Red-tailed pipe snake","Reticulated python","Rhinoceros viper","Rhombic night adder","Ribbon snake","Rinkhals","Rinkhals cobra","River jack","Rosy boa","Rough green snake","Rough-scaled bush viper","Rough-scaled python","Rough-scaled tree viper","Royal python","Rubber boa","Rufous beaked snake","Rungwe tree viper","San Francisco garter snake","Sand adder","Sand boa","Savu python","Saw-scaled viper","Scarlet kingsnake","Schlegel's viper","Schultze's pitviper","Sea snake","Sedge viper","Selayer reticulated python","Sharp-nosed viper","Shield-nosed cobra","Shield-tailed snake","Siamese palm viper","Side-striped palm-pitviper","Sidewinder","Sikkim keelback","Sinai desert cobra","Sind krait","Small-eyed snake","Smooth green snake","Smooth snake","Snorkel viper","Snouted cobra","Sonoran sidewinder","South American hognose snake","South eastern corn snake","Southern Indonesian spitting cobra","Southern Pacific rattlesnake","Southern Philippine cobra","Southern black racer","Southern white-lipped python","Southwestern black spitting cobra","Southwestern blackhead snake","Southwestern carpet python","Southwestern speckled rattlesnake","Speckle-bellied keelback","Speckled kingsnake","Spectacled cobra","Spiny bush viper","Spitting cobra","Spotted python","Sri Lankan pit viper","Stejneger's bamboo pitviper","Stiletto snake","Stimson's python","Stoke's sea snake","Storm water cobra","Striped snake","Sumatran short-tailed python","Sumatran tree viper","Sunbeam snake","Taipan","Taiwan cobra","Tan racer","Tancitaran dusky rattlesnake","Tanimbar python","Tasmanian tiger snake","Tawny cat snake","Temple pit viper","Temple viper","Tentacled snake","Texas Coral Snake","Texas blind snake","Texas garter snake","Texas lyre snake","Texas night snake","Thai cobra","Three-lined ground snake","Tibetan bamboo pitviper","Tic polonga","Tiger pit viper","Tiger rattlesnake","Tiger snake","Tigre snake","Timber rattlesnake","Timor python","Titanboa","Tree boa","Tree snake","Tree viper","Trinket snake","Tropical rattlesnake","Twig snake","Twin Headed King Snake","Twin-Barred tree snake","Twin-spotted rat snake","Twin-spotted rattlesnake","Undulated pit viper","Uracoan rattlesnake","Ursini's viper","Urutu","Vine snake","Viper","Viper Adder","Vipera ammodytes","Wagler's pit viper","Wart snake","Water adder","Water moccasin","Water snake","West Indian racer","Western blind snake","Western carpet python","Western coral snake","Western diamondback rattlesnake","Western green mamba","Western ground snake","Western hog-nosed viper","Western mud snake","Western tiger snake","Western woma python","Wetar Island python","Whip snake","White-lipped keelback","White-lipped python","White-lipped tree viper","Wirot's pit viper","Wolf snake","Woma python","Worm snake","Wutu","Wynaad keelback","Yarara","Yellow anaconda","Yellow-banded sea snake","Yellow-bellied sea snake","Yellow-lined palm viper","Yellow-lipped sea snake","Yellow-striped rat snake","Yunnan keelback","Zebra snake","Zebra spitting cobra"],NFe=["bat","bear","bee","bird","butterfly","cat","cow","crocodile","deer","dog","dolphin","eagle","elephant","fish","flamingo","fox","frog","gecko","giraffe","gorilla","hamster","hippopotamus","horse","kangaroo","koala","lion","monkey","ostrich","panda","parrot","peacock","penguin","polar bear","rabbit","rhinoceros","sea lion","shark","snake","squirrel","tiger","turtle","whale","wolf","zebra"],LFe={bear:bFe,bird:yFe,cat:vFe,cetacean:SFe,cow:EFe,crocodilia:xFe,dog:wFe,fish:kFe,horse:CFe,insect:TFe,lion:AFe,pet_name:FFe,rabbit:_Fe,rodent:RFe,snake:DFe,type:NFe},MFe=LFe,IFe=["{{person.name}}","{{company.name}}"],OFe=["Redhold","Treeflex","Trippledex","Kanlam","Bigtax","Daltfresh","Toughjoyfax","Mat Lam Tam","Otcom","Tres-Zap","Y-Solowarm","Tresom","Voltsillam","Biodex","Greenlam","Viva","Matsoft","Temp","Zoolab","Subin","Rank","Job","Stringtough","Tin","It","Home Ing","Zamit","Sonsing","Konklab","Alpha","Latlux","Voyatouch","Alphazap","Holdlamis","Zaam-Dox","Sub-Ex","Quo Lux","Bamity","Ventosanzap","Lotstring","Hatity","Tempsoft","Overhold","Fixflex","Konklux","Zontrax","Tampflex","Span","Namfix","Transcof","Stim","Fix San","Sonair","Stronghold","Fintone","Y-find","Opela","Lotlux","Ronstring","Zathin","Duobam","Keylex"],BFe=["0.#.#","0.##","#.##","#.#","#.#.#"],PFe={author:IFe,name:OFe,version:BFe},zFe=PFe,HFe=["A.A. Milne","Agatha Christie","Alan Moore and Dave Gibbons","Albert Camus","Aldous Huxley","Alexander Pope","Alexandre Dumas","Alice Walker","Andrew Lang","Anne Frank","Anthony Burgess","Anthony Trollope","Antoine de Saint-Exupéry","Anton Chekhov","Anton Pavlovich Chekhov","Arthur Conan Doyle","Arthur Schopenhauer","Aylmer Maude","Ayn Rand","Beatrix Potter","Benjamin Disraeli","Benjamin Jowett","Bernard Shaw","Bertrand Russell","Bhagavanlal Indrajit","Boris Pasternak","Bram Stoker","Brian Evenson","C.E. Brock","C.S. Lewis","Carson McCallers","Charles Dickens","Charles E. Derbyshire","Charlotte Brontë","Charlotte Perkins Gilman","Chinua Achebe","Clifford R. Adams","Constance Garnett","Cormac McCarthy","D.H. Lawrence","Dan Brown","Daniel Defoe","Dante Alighieri","Dashiell Hammett","David Widger","David Wyllie","Dean Koontz","Don DeLillo","E.M. Forster","Edgar Allan Poe","Edgar Rice Burroughs","Elizabeth Cleghorn Gaskell","Elizabeth Von Arnim","Emily Brontë","Erich Remarque","Ernest Hemingway","Evelyn Waugh","F. Scott Fitzgerald","Ford Madox Ford","Frances Hodgson Burnett","Frank Herbert","Frank T. Merrill","Franz Kafka","Friedrich Wilhelm Nietzsche","Fyodor Dostoyevsky","G.K. Chesterton","Gabriel Garcia Marquez","Geoffrey Chaucer","George Eliot","George Grossmith","George Orwell","George R. R. Martin","George Saunders","Grady Ward","Graham Greene","Gustave Doré","Gustave Flaubert","Guy de Maupassant","Günter Grass","H.G. Wells","H.P. Lovecraft","Harper Lee","Harriet Beecher Stowe","Haruki Murakami","Henrik Ibsen","Henry David Thoreau","Henry Fielding","Henry James","Henry Miller","Henry Morley","Herman Melville","Hermann Broch","Homer","Honoré de Balzac","Ian McEwan","Isabel Florence Hapgood","Italo Calvino","J.D. Salinger","J.K. Rowling","J.M. Barrie","J.R.R. Tolkien","Jack Kerouac","Jack London","Jacob Grimm","Jacqueline Crooks","James Baldwin","James Dickey","James Ellroy","James Joyce","James Patterson","Jane Austen","Johann Wolfgang von Goethe","John Bunyan","John Camden Hotten","John Dos Passos","John Green","John Grisham","John Kennedy Toole","John Milton","John Ormsby","John Steinbeck","John Updike","Jonathan Franzen","Jonathan Swift","Joseph Conrad","Joseph Heller","José Rizal","Judy Blume","Jules Verne","Junot Diaz","Karl Marx","Kazuo Ishiguro","Ken Follett","Ken Kesey","Kenneth Grahame","Khaled Hosseini","Kingsley Amis","Kurt Vonnegut","L. Frank Baum","L.M. Montgomery","Laurence Sterne","Leo Tolstoy","Lewis Carroll","Louisa May Alcott","Louise Maude","Malcolm Lowry","Marcel Proust","Margaret Atwood","Margaret Mitchell","Marilynne Robinson","Mark Twain","Martin Amis","Mary Shelley","Michael Chabon","Miguel de Cervantes","Mikhail Bulgakov","Muriel Spark","Nancy Mitford","Nathanael West","Nathaniel Hawthorne","Neil Gaiman","Niccolo Machiavelli","Norman Mailer","Oscar Levy","Oscar Wilde","P.G. Wodehouse","Paulo Coelho","Peter Carey","Philip Pullman","Philip Roth","Plato","R.L. Stine","Rachel Kushner","Ralph Ellison","Ray Bradbury","Raymond Chandler","Richard Wagner","Richard Wright","Richard Yates","Roald Dahl","Robert Graves","Robert Louis Stevenson","Robert Penn Warren","Rudyard Kipling","Salman Rushdie","Samuel Beckett","Samuel Butler","Samuel Richardson","Saul Bellow","Shivaram Parashuram Bhide","Sir Arthur Conan Doyle","Sir Richard Francis Burton","Stendhal","Stephen Hawking","Stephen King","Sun Tzu","Suzanne Collins","T. Smollett","T.S. Eliot","Theodore Alois Buckley","Theodore Dreiser","Thomas Hardy","Thomas Love Peacock","Thomas Mann","Toni Morrison","Truman Capote","V.S. Naipaul","Vance Packard","Vatsyayana","Victor Hugo","Virgil","Virginia Woolf","Vladimir Nabokov","Voltaire","W.G. Sebald","W.K. Marriott","Walker Percy","Walt Whitman","Walter Scott","Wilhelm Grimm","Wilkie Collins","William Faulkner","William Gibson","William Golding","William Makepeace Thackeray","William Shakespeare","Zadie Smith"],UFe=["Audiobook","Ebook","Hardcover","Paperback"],$Fe=["Adventure","Biography","Business","Children's Literature","Classic","Comedy","Comic","Detective","Drama","Fantasy","Graphic Novel","Historical Fiction","Horror","Memoir","Mystery","Mythology","Philosophy","Poetry","Psychology","Religion","Romance","Science Fiction","Thriller","Western","Young Adult"],GFe=["Academic Press","Ace Books","Addison-Wesley","Adis International","Airiti Press","Allen Ltd","Andrews McMeel Publishing","Anova Books","Anvil Press Poetry","Applewood Books","Apress","Athabasca University Press","Atheneum Books","Atheneum Publishers","Atlantic Books","Atlas Press","BBC Books","Ballantine Books","Banner of Truth Trust","Bantam Books","Bantam Spectra","Barrie & Jenkins","Basic Books","Belknap Press","Bella Books","Bellevue Literary Press","Berg Publishers","Berkley Books","Bison Books","Black Dog Publishing","Black Library","Black Sparrow Books","Blackie and Son Limited","Blackstaff Press","Blackwell Publishing","Bloodaxe Books","Bloomsbury Publishing Plc","Blue Ribbon Books","Book League of America","Book Works","Booktrope","Borgo Press","Bowes & Bowes","Boydell & Brewer","Breslov Research Institute","Brill","Brimstone Press","Broadview Press","Burns & Oates","Butterworth-Heinemann","Caister Academic Press","Cambridge University Press","Candlewick Press","Canongate Books","Carcanet Press","Carlton Books","Carlton Publishing Group","Carnegie Mellon University Press","Casemate Publishers","Cengage Learning","Central European University Press","Chambers Harrap","Charles Scribner's Sons","Chatto and Windus","Chick Publications","Chronicle Books","Churchill Livingstone","Cisco Press","City Lights Publishers","Cloverdale Corporation","D. Appleton & Company","D. Reidel","DAW Books","Da Capo Press","Daedalus Publishing","Dalkey Archive Press","Darakwon Press","David & Charles","Dedalus Books","Del Rey Books","E. P. Dutton","ECW Press","Earthscan","Edupedia Publications","Eel Pie Publishing","Eerdmans Publishing","Ellora's Cave","Elsevier","Emerald Group Publishing","Etruscan Press","FabJob","Faber and Faber","Fairview Press","Farrar, Straus & Giroux","Fearless Books","Felony & Mayhem Press","Firebrand Books","Flame Tree Publishing","Focal Press","G-Unit Books","G. P. Putnam's Sons","Gaspereau Press","Gay Men's Press","Gefen Publishing House","George H. Doran Company","George Newnes","George Routledge & Sons","Godwit Press","Golden Cockerel Press","HMSO","Hachette Book Group USA","Hackett Publishing Company","Hamish Hamilton","Happy House","Harcourt Assessment","Harcourt Trade Publishers","Harlequin Enterprises Ltd","Harper & Brothers","Harper & Row","HarperCollins","HarperPrism","HarperTrophy","Harry N. Abrams, Inc.","Harvard University Press","Harvest House","Harvill Press at Random House","Hawthorne Books","Hay House","Haynes Manuals","Heyday Books","Hodder & Stoughton","Hodder Headline","Hogarth Press","Holland Park Press","Holt McDougal","Horizon Scientific Press","Ian Allan Publishing","Ignatius Press","Imperial War Museum","Indiana University Press","J. M. Dent","Jaico Publishing House","Jarrolds Publishing","John Blake Publishing","Karadi Tales","Kensington Books","Kessinger Publishing","Kodansha","Kogan Page","Koren Publishers Jerusalem","Ladybird Books","Leaf Books","Leafwood Publishers","Left Book Club","Legend Books","Lethe Press","Libertas Academica","Liberty Fund","Library of America","Lion Hudson","Macmillan Publishers","Mainstream Publishing","Manchester University Press","Mandrake Press","Mandrake of Oxford","Manning Publications","Manor House Publishing","Mapin Publishing","Marion Boyars Publishers","Mark Batty Publisher","Marshall Cavendish","Marshall Pickering","Martinus Nijhoff Publishers","Mascot Books","Matthias Media","McClelland and Stewart","McFarland & Company","McGraw Hill Financial","McGraw-Hill Education","Medknow Publications","Naiad Press","Nauka","NavPress","New Directions Publishing","New English Library","New Holland Publishers","New Village Press","Newnes","No Starch Press","Nonesuch Press","O'Reilly Media","Oberon Books","Open Court Publishing Company","Open University Press","Orchard Books","Orion Books","Packt Publishing","Palgrave Macmillan","Pan Books","Pantheon Books at Random House","Papadakis Publisher","Parachute Publishing","Parragon","Pathfinder Press","Paulist Press","Pavilion Books","Peace Hill Press","Pecan Grove Press","Pen and Sword Books","Penguin Books","Random House","Reed Elsevier","Reed Publishing","SAGE Publications","Salt Publishing","Sams Publishing","Schocken Books","Scholastic Press","Seagull Books","Secker & Warburg","Shambhala Publications","Shire Books","Shoemaker & Hoard Publishers","Shuter & Shooter Publishers","Sidgwick & Jackson","Signet Books","Simon & Schuster","St. Martin's Press","T & T Clark","Tachyon Publications","Tammi","Target Books","Tarpaulin Sky Press","Tartarus Press","Tate Publishing & Enterprises","Taunton Press","Taylor & Francis","Ten Speed Press","UCL Press","Unfinished Monument Press","United States Government Publishing Office","University of Akron Press","University of Alaska Press","University of California Press","University of Chicago Press","University of Michigan Press","University of Minnesota Press","University of Nebraska Press","Velazquez Press","Verso Books","Victor Gollancz Ltd","Viking Press","Vintage Books","Vintage Books at Random House","Virago Press","Virgin Publishing","Voyager Books","Zed Books","Ziff Davis Media","Zondervan"],jFe=["A Song of Ice and Fire","Anna Karenina","Colonel Race","Discworld","Dune","Harry Potter","Hercule Poirot","His Dark Materials","Jane Austen Murder Mysteries","Little Women","Outlander","Percy Jackson","Sherlock Holmes","The Arc of a Scythe","The Bartimaeus Trilogy","The Border Trilogy","The Chronicles of Narnia","The Dark Tower","The Dresden Files","The Eighth Life","The Foundation Series","The Hitchhiker's Guide to the Galaxy","The Hunger Games","The Infinity Cycle","The Inheritance Cycle","The Lord of the Rings","The Maze Runner","The Prison Healer","The Red Rising Saga","The Southern Reach","The Wheel of Time","Thursday Next Series","Twilight","War and Peace"],qFe=["1984","20,000 Leagues Under the Sea","A Bend in the River","A Brief History of Time","A Clockwork Orange","A Confederacy of Dunces","A Doll's House","A Handful of Dust","A Modest Proposal","A Passage to India","A Portrait of the Artist as a Young Man","A Room with a View","A Study in Scarlet","A Tale of Two Cities","A Wrinkle in Time","Absalom, Absalom!","Adventures of Huckleberry Finn","Alice's Adventures in Wonderland","All Quiet on the Western Front","All the King's Men","American Pastoral","An American Tragedy","And Then There Were None","Animal Farm","Anna Karenina","Anne of Green Gables","Are You There God? It's Me, Margaret","As I Lay Dying","Atlas Shrugged","Atonement","Austerlitz","Beloved","Beyond Good and Evil","Bible","Bleak House","Blood Meridian","Brave New World","Brideshead Revisited","Candide","Carmilla","Catch-22","Charlie and the Chocolate Factory","Charlotte's Web","Clarissa","Cranford","Crime and Punishment","Dao De Jing: A Minimalist Translation","David Copperfield","Deliverance","Don Quixote","Dora","Dr. Zhivago","Dracula","Dubliners","Dune","East of Eden","Emma","Fahrenheit 451","Faust","For Whom the Bell Tolls","Frankenstein","Freakonomics","Go Tell It on the Mountain","Gone with the Wind","Great Expectations","Grimms' Fairy Tales","Gulliver's Travels","Hamlet","Harry Potter and the Sorcerer's Stone","Heart of Darkness","Herzog","His Dark Materials","Hitting the line","Housekeeping","I, Claudius","If on a Winter's Night a Traveler","In Cold Blood","In Search of Lost Time","Invisible Man","It","Jane Eyre","Josefine Mutzenbacher","Jude the Obscure","L.A. Confidential","Leaves of Grass","Les Miserables","Life of Pi","Little Women","Lolita","Long Walk to Freedom","Lord Jim","Lord of the Flies","Lucky Jim","Madame Bovary","Malone Dies","Meditations","Men Without Women","Metamorphosis","Middlemarch","Midnight's Children","Moby Dick","Money","Mrs. Dalloway","My Bondage and My Freedom","My Life","Native Son","Neuromancer","Never Let Me Go","Nightmare Abbey","Nineteen Eighty Four","Nostromo","Notes from the Underground","Of Mice and Men","Oliver Twist","On the Duty of Civil Disobedience","On the Road","One Flew Over the Cuckoo's Nest","One Hundred Years of Solitude","One Thousand and One Nights","Oscar and Lucinda","Pale Fire","Paradise Lost","Peter Pan","Portnoy's Complaint","Pride and Prejudice","Rabbit, Run","Republic","Revolutionary Road","Robinson Crusoe","Romeo and Juliet","Ruth Fielding in Alaska","Scoop","Second Treatise of Government","Slaughterhouse Five","Stories of Anton Chekhov","Sybil","Tess of the d'Urbervilles","The Adventures of Augie March","The Adventures of Huckleberry Finn","The Adventures of Sherlock Holmes","The Adventures of Tom Sawyer","The Aeneid","The Alchemist","The Ambassadors","The Art of War","The Big Sleep","The Black Sheep","The Blue Castle","The Brief Wondrous Life of Oscar Wao","The Brothers Karamazov","The Call of the Wild","The Canterbury Tales","The Catcher in the Rye","The Color Purple","The Complete Works of Edgar Allen Poe","The Corrections","The Count of Monte Cristo","The Day of the Locust","The Diary of a Nobody","The Diary of a Young Girl","The Divine Comedy","The Enchanted April","The Fountainhead","The Golden Bowl","The Golden Notebook","The Good Soldier","The Grapes of Wrath","The Great Gatsby","The Handmaid's Tale","The Heart is a Lonely Hunter","The Heart of the Matter","The Hobbit","The Hound of the Baskervilles","The Idiot","The Iliad","The King in Yellow","The Kite Runner","The Lion, the Witch, and the Wardrobe","The Little Prince","The Lord of the Rings","The Magic Mountain","The Maltese Falcon","The Master and Margarita","The Moviegoer","The Naked and the Dead","The Odyssey","The Old Man and the Sea","The Pickwick Papers","The Picture of Dorian Gray","The Pilgrim's Progress","The Pillars of the Earth","The Plague","The Portrait of a Lady","The Prime of Miss Jean Brodie","The Prince","The Problems of Philosophy","The Prophet","The Pursuit of Love","The Rainbow","The Red and the Black","The Remains of the Day","The Republic","The Scarlet Letter","The Sleepwalkers","The Sound and the Fury","The Stand","The Strange Case of Dr. Jekyll and Mr. Hyde","The Stranger","The Sun Also Rises","The Thirty-Nine Steps","The Three Musketeers","The Time Machine","The Tin Drum","The Trial","The War of the Worlds","The Waste Land","The Way We Live Now","The Wind in the Willows","The Woman in White","The Wonderful Wizard of Oz","The Works of Edgar Allan Poe","The Yellow Wallpaper","Things Fall Apart","Tinker, Tailor, Soldier, Spy","To Kill a Mockingbird","To the Lighthouse","Tom Jones","Treasure Island","Tristram Shandy","Tropic of Cancer","U.S.A. Trilogy","Ulysses","Uncle Tom's Cabin","Under the Volcano","Underworld","Vanity Fair","Walden","War and Peace","Watchmen","Winnie-the-Pooh","Wuthering Heights"],WFe={author:HFe,format:UFe,genre:$Fe,publisher:GFe,series:jFe,title:qFe},VFe=WFe,KFe=["###-###-####","(###) ###-####","1-###-###-####","###.###.####"],YFe={formats:KFe},JFe=YFe,XFe=["azure","black","blue","cyan","fuchsia","gold","green","grey","indigo","ivory","lavender","lime","magenta","maroon","mint green","olive","orange","orchid","pink","plum","purple","red","salmon","silver","sky blue","tan","teal","turquoise","violet","white","yellow"],ZFe={human:XFe},QFe=ZFe,e3e=["Automotive","Baby","Beauty","Books","Clothing","Computers","Electronics","Games","Garden","Grocery","Health","Home","Industrial","Jewelry","Kids","Movies","Music","Outdoors","Shoes","Sports","Tools","Toys"],t3e=["Discover the {{animal.type}}-like agility of our {{commerce.product}}, perfect for {{word.adjective}} users","Discover the {{word.adjective}} new {{commerce.product}} with an exciting mix of {{commerce.productMaterial}} ingredients","Ergonomic {{commerce.product}} made with {{commerce.productMaterial}} for all-day {{word.adjective}} support","Experience the {{color.human}} brilliance of our {{commerce.product}}, perfect for {{word.adjective}} environments","Featuring {{science.chemical_element.name}}-enhanced technology, our {{commerce.product}} offers unparalleled {{word.adjective}} performance","Innovative {{commerce.product}} featuring {{word.adjective}} technology and {{commerce.productMaterial}} construction","Introducing the {{location.country}}-inspired {{commerce.product}}, blending {{word.adjective}} style with local craftsmanship","New {{color.human}} {{commerce.product}} with ergonomic design for {{word.adjective}} comfort",'New {{commerce.product}} model with {{number.int({"min": 1, "max": 100})}} GB RAM, {{number.int({"min": 1, "max": 1000})}} GB storage, and {{word.adjective}} features',"Our {{animal.type}}-friendly {{commerce.product}} ensures {{word.adjective}} comfort for your pets","Our {{food.adjective}}-inspired {{commerce.product}} brings a taste of luxury to your {{word.adjective}} lifestyle","Professional-grade {{commerce.product}} perfect for {{word.adjective}} training and recreational use","Savor the {{food.adjective}} essence in our {{commerce.product}}, designed for {{word.adjective}} culinary adventures","Stylish {{commerce.product}} designed to make you stand out with {{word.adjective}} looks","The sleek and {{word.adjective}} {{commerce.product}} comes with {{color.human}} LED lighting for smart functionality","The {{color.human}} {{commerce.product}} combines {{location.country}} aesthetics with {{science.chemical_element.name}}-based durability","The {{company.catchPhrase}} {{commerce.product}} offers reliable performance and {{word.adjective}} design","The {{person.firstName}} {{commerce.product}} is the latest in a series of {{word.adjective}} products from {{company.name}}","{{commerce.productAdjective}} {{commerce.product}} designed with {{commerce.productMaterial}} for {{word.adjective}} performance","{{company.name}}'s most advanced {{commerce.product}} technology increases {{word.adjective}} capabilities"],r3e={adjective:["Awesome","Bespoke","Electronic","Elegant","Ergonomic","Fantastic","Fresh","Frozen","Generic","Gorgeous","Handcrafted","Handmade","Incredible","Intelligent","Licensed","Luxurious","Modern","Oriental","Practical","Recycled","Refined","Rustic","Sleek","Small","Soft","Tasty","Unbranded"],material:["Aluminum","Bamboo","Bronze","Ceramic","Concrete","Cotton","Gold","Granite","Marble","Metal","Plastic","Rubber","Silk","Steel","Wooden"],product:["Bacon","Ball","Bike","Car","Chair","Cheese","Chicken","Chips","Computer","Fish","Gloves","Hat","Keyboard","Mouse","Pants","Pizza","Salad","Sausages","Shirt","Shoes","Soap","Table","Towels","Tuna"]},n3e={department:e3e,product_description:t3e,product_name:r3e},a3e=n3e,i3e=["AI-driven","Adaptive","Advanced","Automated","Balanced","Business-focused","Centralized","Compatible","Configurable","Cross-platform","Customer-focused","Customizable","Decentralized","Devolved","Digitized","Distributed","Diverse","Enhanced","Ergonomic","Exclusive","Expanded","Extended","Face to face","Focused","Front-line","Fully-configurable","Fundamental","Future-proofed","Grass-roots","Horizontal","Immersive","Implemented","Innovative","Integrated","Intuitive","Managed","Monitored","Multi-tiered","Networked","Open-architected","Open-source","Operative","Optimized","Optional","Organic","Organized","Persevering","Persistent","Phased","Polarised","Proactive","Profit-focused","Profound","Programmable","Progressive","Public-key","Quality-focused","Reactive","Realigned","Reduced","Reverse-engineered","Robust","Seamless","Secured","Self-enabling","Sharable","Smart","Stand-alone","Streamlined","Sustainable","Synchronised","Team-oriented","Total","Triple-buffered","Universal","Upgradable","User-centric","User-friendly","Versatile","Virtual","Visionary"],o3e=["24/7","AI-driven","B2B","B2C","back-end","best-of-breed","bleeding-edge","collaborative","compelling","cross-media","cross-platform","customized","cutting-edge","decentralized","distributed","dynamic","efficient","end-to-end","enterprise","extensible","frictionless","front-end","generative","global","granular","holistic","immersive","impactful","innovative","integrated","interactive","intuitive","killer","leading-edge","magnetic","mission-critical","next-generation","one-to-one","open-source","out-of-the-box","plug-and-play","proactive","quantum","real-time","revolutionary","rich","robust","scalable","seamless","smart","sticky","strategic","sustainable","synergistic","transparent","turn-key","ubiquitous","user-centric","value-added","vertical","viral","virtual","visionary","world-class"],s3e=["AI","ROI","applications","architectures","blockchains","channels","communities","content","convergence","deliverables","e-commerce","experiences","functionalities","infrastructures","initiatives","interfaces","large language models","lifetime value","markets","methodologies","metrics","mindshare","models","networks","niches","paradigms","partnerships","platforms","relationships","schemas","smart contracts","solutions","supply-chains","synergies","systems","technologies","users","web services"],l3e=["aggregate","architect","benchmark","brand","collaborate","cultivate","deliver","deploy","disintermediate","drive","embrace","empower","enable","engage","engineer","enhance","evolve","expedite","exploit","extend","facilitate","gamify","generate","grow","harness","implement","incentivize","incubate","innovate","integrate","iterate","leverage","maximize","mesh","monetize","optimize","orchestrate","productize","redefine","reinvent","repurpose","revolutionize","scale","seize","simplify","strategize","streamline","syndicate","synthesize","target","transform","transition","unleash","utilize","visualize","whiteboard"],u3e=["24 hour","24/7","AI-powered","actuating","analyzing","asymmetric","asynchronous","attitude-oriented","bifurcated","bottom-line","clear-thinking","client-driven","client-server","cloud-native","coherent","cohesive","composite","content-based","context-sensitive","contextually-based","data-driven","dedicated","demand-driven","directional","discrete","disintermediate","dynamic","eco-centric","empowering","encompassing","executive","explicit","exuding","fault-tolerant","fresh-thinking","full-range","global","heuristic","high-level","holistic","homogeneous","human-resource","hybrid","immersive","impactful","incremental","intangible","interactive","intermediate","leading edge","local","logistical","maximized","methodical","mission-critical","mobile","modular","motivating","national","needs-based","neutral","next generation","optimal","optimizing","radical","real-time","reciprocal","regional","resilient","responsive","scalable","secondary","stable","static","sustainable","system-worthy","systematic","systemic","tangible","tertiary","transitional","uniform","user-facing","value-added","well-modulated","zero administration","zero defect","zero tolerance","zero trust"],c3e=["Group","Inc","LLC","and Sons"],d3e=["{{person.last_name.generic}} - {{person.last_name.generic}}","{{person.last_name.generic}} {{company.legal_entity_type}}","{{person.last_name.generic}}, {{person.last_name.generic}} and {{person.last_name.generic}}"],h3e=["ability","access","adapter","algorithm","alliance","analyzer","application","approach","architecture","archive","array","artificial intelligence","attitude","benchmark","budgetary management","capability","capacity","challenge","circuit","collaboration","complexity","concept","conglomeration","contingency","core","customer loyalty","data-warehouse","database","definition","emulation","encoding","encryption","firmware","flexibility","focus group","forecast","frame","framework","function","functionalities","generative AI","hardware","help-desk","hierarchy","hub","implementation","infrastructure","initiative","installation","instruction set","interface","internet solution","intranet","knowledge base","knowledge user","leverage","local area network","matrices","matrix","methodology","microservice","middleware","migration","model","moderator","monitoring","moratorium","neural-net","open architecture","orchestration","paradigm","parallelism","policy","portal","pricing structure","process improvement","product","productivity","project","projection","protocol","service-desk","software","solution","standardization","strategy","structure","success","support","synergy","system engine","task-force","throughput","time-frame","toolset","utilisation","website","workforce"],p3e={adjective:i3e,buzz_adjective:o3e,buzz_noun:s3e,buzz_verb:l3e,descriptor:u3e,legal_entity_type:c3e,name_pattern:d3e,noun:h3e},f3e=p3e,m3e=["avatar","category","comment","createdAt","email","group","id","name","password","phone","status","title","token","updatedAt"],g3e={column:m3e},b3e=g3e,y3e={wide:["April","August","December","February","January","July","June","March","May","November","October","September"],abbr:["Apr","Aug","Dec","Feb","Jan","Jul","Jun","Mar","May","Nov","Oct","Sep"]},v3e={wide:["Friday","Monday","Saturday","Sunday","Thursday","Tuesday","Wednesday"],abbr:["Fri","Mon","Sat","Sun","Thu","Tue","Wed"]},S3e={month:y3e,weekday:v3e},E3e=S3e,x3e=["Auto Loan","Checking","Credit Card","Home Loan","Investment","Money Market","Personal Loan","Savings"],w3e=["34##-######-####L","37##-######-####L"],k3e=["30[0-5]#-######-###L","36##-######-###L","54##-####-####-###L"],C3e=["6011-####-####-###L","64[4-9]#-####-####-###L","65##-####-####-###L"],T3e=["3528-####-####-###L","3529-####-####-###L","35[3-8]#-####-####-###L"],A3e=["2[221-720]-####-####-###L","5[1-5]##-####-####-###L"],F3e=["4###########L","4###-####-####-###L"],_3e={american_express:w3e,diners_club:k3e,discover:C3e,jcb:T3e,mastercard:A3e,visa:F3e},R3e=_3e,D3e=[{name:"UAE Dirham",code:"AED",symbol:"",numericCode:"784"},{name:"Afghani",code:"AFN",symbol:"؋",numericCode:"971"},{name:"Lek",code:"ALL",symbol:"Lek",numericCode:"008"},{name:"Armenian Dram",code:"AMD",symbol:"",numericCode:"051"},{name:"Netherlands Antillian Guilder",code:"ANG",symbol:"ƒ",numericCode:"532"},{name:"Kwanza",code:"AOA",symbol:"",numericCode:"973"},{name:"Argentine Peso",code:"ARS",symbol:"$",numericCode:"032"},{name:"Australian Dollar",code:"AUD",symbol:"$",numericCode:"036"},{name:"Aruban Guilder",code:"AWG",symbol:"ƒ",numericCode:"533"},{name:"Azerbaijanian Manat",code:"AZN",symbol:"ман",numericCode:"944"},{name:"Convertible Marks",code:"BAM",symbol:"KM",numericCode:"977"},{name:"Barbados Dollar",code:"BBD",symbol:"$",numericCode:"052"},{name:"Taka",code:"BDT",symbol:"",numericCode:"050"},{name:"Bulgarian Lev",code:"BGN",symbol:"лв",numericCode:"975"},{name:"Bahraini Dinar",code:"BHD",symbol:"",numericCode:"048"},{name:"Burundi Franc",code:"BIF",symbol:"",numericCode:"108"},{name:"Bermudian Dollar (customarily known as Bermuda Dollar)",code:"BMD",symbol:"$",numericCode:"060"},{name:"Brunei Dollar",code:"BND",symbol:"$",numericCode:"096"},{name:"Boliviano boliviano",code:"BOB",symbol:"Bs",numericCode:"068"},{name:"Brazilian Real",code:"BRL",symbol:"R$",numericCode:"986"},{name:"Bahamian Dollar",code:"BSD",symbol:"$",numericCode:"044"},{name:"Pula",code:"BWP",symbol:"P",numericCode:"072"},{name:"Belarusian Ruble",code:"BYN",symbol:"Rbl",numericCode:"933"},{name:"Belize Dollar",code:"BZD",symbol:"BZ$",numericCode:"084"},{name:"Canadian Dollar",code:"CAD",symbol:"$",numericCode:"124"},{name:"Congolese Franc",code:"CDF",symbol:"",numericCode:"976"},{name:"Swiss Franc",code:"CHF",symbol:"CHF",numericCode:"756"},{name:"Chilean Peso",code:"CLP",symbol:"$",numericCode:"152"},{name:"Yuan Renminbi",code:"CNY",symbol:"¥",numericCode:"156"},{name:"Colombian Peso",code:"COP",symbol:"$",numericCode:"170"},{name:"Costa Rican Colon",code:"CRC",symbol:"₡",numericCode:"188"},{name:"Cuban Peso",code:"CUP",symbol:"₱",numericCode:"192"},{name:"Cape Verde Escudo",code:"CVE",symbol:"",numericCode:"132"},{name:"Czech Koruna",code:"CZK",symbol:"Kč",numericCode:"203"},{name:"Djibouti Franc",code:"DJF",symbol:"",numericCode:"262"},{name:"Danish Krone",code:"DKK",symbol:"kr",numericCode:"208"},{name:"Dominican Peso",code:"DOP",symbol:"RD$",numericCode:"214"},{name:"Algerian Dinar",code:"DZD",symbol:"",numericCode:"012"},{name:"Egyptian Pound",code:"EGP",symbol:"£",numericCode:"818"},{name:"Nakfa",code:"ERN",symbol:"",numericCode:"232"},{name:"Ethiopian Birr",code:"ETB",symbol:"",numericCode:"230"},{name:"Euro",code:"EUR",symbol:"€",numericCode:"978"},{name:"Fiji Dollar",code:"FJD",symbol:"$",numericCode:"242"},{name:"Falkland Islands Pound",code:"FKP",symbol:"£",numericCode:"238"},{name:"Pound Sterling",code:"GBP",symbol:"£",numericCode:"826"},{name:"Lari",code:"GEL",symbol:"",numericCode:"981"},{name:"Cedi",code:"GHS",symbol:"",numericCode:"936"},{name:"Gibraltar Pound",code:"GIP",symbol:"£",numericCode:"292"},{name:"Dalasi",code:"GMD",symbol:"",numericCode:"270"},{name:"Guinea Franc",code:"GNF",symbol:"",numericCode:"324"},{name:"Quetzal",code:"GTQ",symbol:"Q",numericCode:"320"},{name:"Guyana Dollar",code:"GYD",symbol:"$",numericCode:"328"},{name:"Hong Kong Dollar",code:"HKD",symbol:"$",numericCode:"344"},{name:"Lempira",code:"HNL",symbol:"L",numericCode:"340"},{name:"Gourde",code:"HTG",symbol:"",numericCode:"332"},{name:"Forint",code:"HUF",symbol:"Ft",numericCode:"348"},{name:"Rupiah",code:"IDR",symbol:"Rp",numericCode:"360"},{name:"New Israeli Sheqel",code:"ILS",symbol:"₪",numericCode:"376"},{name:"Bhutanese Ngultrum",code:"BTN",symbol:"Nu",numericCode:"064"},{name:"Indian Rupee",code:"INR",symbol:"₹",numericCode:"356"},{name:"Iraqi Dinar",code:"IQD",symbol:"",numericCode:"368"},{name:"Iranian Rial",code:"IRR",symbol:"﷼",numericCode:"364"},{name:"Iceland Krona",code:"ISK",symbol:"kr",numericCode:"352"},{name:"Jamaican Dollar",code:"JMD",symbol:"J$",numericCode:"388"},{name:"Jordanian Dinar",code:"JOD",symbol:"",numericCode:"400"},{name:"Yen",code:"JPY",symbol:"¥",numericCode:"392"},{name:"Kenyan Shilling",code:"KES",symbol:"",numericCode:"404"},{name:"Som",code:"KGS",symbol:"лв",numericCode:"417"},{name:"Riel",code:"KHR",symbol:"៛",numericCode:"116"},{name:"Comoro Franc",code:"KMF",symbol:"",numericCode:"174"},{name:"North Korean Won",code:"KPW",symbol:"₩",numericCode:"408"},{name:"Won",code:"KRW",symbol:"₩",numericCode:"410"},{name:"Kuwaiti Dinar",code:"KWD",symbol:"",numericCode:"414"},{name:"Cayman Islands Dollar",code:"KYD",symbol:"$",numericCode:"136"},{name:"Tenge",code:"KZT",symbol:"лв",numericCode:"398"},{name:"Kip",code:"LAK",symbol:"₭",numericCode:"418"},{name:"Lebanese Pound",code:"LBP",symbol:"£",numericCode:"422"},{name:"Sri Lanka Rupee",code:"LKR",symbol:"₨",numericCode:"144"},{name:"Liberian Dollar",code:"LRD",symbol:"$",numericCode:"430"},{name:"Libyan Dinar",code:"LYD",symbol:"",numericCode:"434"},{name:"Moroccan Dirham",code:"MAD",symbol:"",numericCode:"504"},{name:"Moldovan Leu",code:"MDL",symbol:"",numericCode:"498"},{name:"Malagasy Ariary",code:"MGA",symbol:"",numericCode:"969"},{name:"Denar",code:"MKD",symbol:"ден",numericCode:"807"},{name:"Kyat",code:"MMK",symbol:"",numericCode:"104"},{name:"Tugrik",code:"MNT",symbol:"₮",numericCode:"496"},{name:"Pataca",code:"MOP",symbol:"",numericCode:"446"},{name:"Ouguiya",code:"MRU",symbol:"",numericCode:"929"},{name:"Mauritius Rupee",code:"MUR",symbol:"₨",numericCode:"480"},{name:"Rufiyaa",code:"MVR",symbol:"",numericCode:"462"},{name:"Kwacha",code:"MWK",symbol:"",numericCode:"454"},{name:"Mexican Peso",code:"MXN",symbol:"$",numericCode:"484"},{name:"Malaysian Ringgit",code:"MYR",symbol:"RM",numericCode:"458"},{name:"Metical",code:"MZN",symbol:"MT",numericCode:"943"},{name:"Naira",code:"NGN",symbol:"₦",numericCode:"566"},{name:"Cordoba Oro",code:"NIO",symbol:"C$",numericCode:"558"},{name:"Norwegian Krone",code:"NOK",symbol:"kr",numericCode:"578"},{name:"Nepalese Rupee",code:"NPR",symbol:"₨",numericCode:"524"},{name:"New Zealand Dollar",code:"NZD",symbol:"$",numericCode:"554"},{name:"Rial Omani",code:"OMR",symbol:"﷼",numericCode:"512"},{name:"Balboa",code:"PAB",symbol:"B/.",numericCode:"590"},{name:"Nuevo Sol",code:"PEN",symbol:"S/.",numericCode:"604"},{name:"Kina",code:"PGK",symbol:"",numericCode:"598"},{name:"Philippine Peso",code:"PHP",symbol:"Php",numericCode:"608"},{name:"Pakistan Rupee",code:"PKR",symbol:"₨",numericCode:"586"},{name:"Zloty",code:"PLN",symbol:"zł",numericCode:"985"},{name:"Guarani",code:"PYG",symbol:"Gs",numericCode:"600"},{name:"Qatari Rial",code:"QAR",symbol:"﷼",numericCode:"634"},{name:"New Leu",code:"RON",symbol:"lei",numericCode:"946"},{name:"Serbian Dinar",code:"RSD",symbol:"Дин.",numericCode:"941"},{name:"Russian Ruble",code:"RUB",symbol:"руб",numericCode:"643"},{name:"Rwanda Franc",code:"RWF",symbol:"",numericCode:"646"},{name:"Saudi Riyal",code:"SAR",symbol:"﷼",numericCode:"682"},{name:"Solomon Islands Dollar",code:"SBD",symbol:"$",numericCode:"090"},{name:"Seychelles Rupee",code:"SCR",symbol:"₨",numericCode:"690"},{name:"Sudanese Pound",code:"SDG",symbol:"",numericCode:"938"},{name:"Swedish Krona",code:"SEK",symbol:"kr",numericCode:"752"},{name:"Singapore Dollar",code:"SGD",symbol:"$",numericCode:"702"},{name:"Saint Helena Pound",code:"SHP",symbol:"£",numericCode:"654"},{name:"Leone",code:"SLE",symbol:"",numericCode:"925"},{name:"Somali Shilling",code:"SOS",symbol:"S",numericCode:"706"},{name:"Surinam Dollar",code:"SRD",symbol:"$",numericCode:"968"},{name:"South Sudanese pound",code:"SSP",symbol:"",numericCode:"728"},{name:"Dobra",code:"STN",symbol:"Db",numericCode:"930"},{name:"Syrian Pound",code:"SYP",symbol:"£",numericCode:"760"},{name:"Lilangeni",code:"SZL",symbol:"",numericCode:"748"},{name:"Baht",code:"THB",symbol:"฿",numericCode:"764"},{name:"Somoni",code:"TJS",symbol:"",numericCode:"972"},{name:"Manat",code:"TMT",symbol:"",numericCode:"934"},{name:"Tunisian Dinar",code:"TND",symbol:"",numericCode:"788"},{name:"Pa'anga",code:"TOP",symbol:"",numericCode:"776"},{name:"Turkish Lira",code:"TRY",symbol:"₺",numericCode:"949"},{name:"Trinidad and Tobago Dollar",code:"TTD",symbol:"TT$",numericCode:"780"},{name:"New Taiwan Dollar",code:"TWD",symbol:"NT$",numericCode:"901"},{name:"Tanzanian Shilling",code:"TZS",symbol:"",numericCode:"834"},{name:"Hryvnia",code:"UAH",symbol:"₴",numericCode:"980"},{name:"Uganda Shilling",code:"UGX",symbol:"",numericCode:"800"},{name:"US Dollar",code:"USD",symbol:"$",numericCode:"840"},{name:"Peso Uruguayo",code:"UYU",symbol:"$U",numericCode:"858"},{name:"Uzbekistan Sum",code:"UZS",symbol:"лв",numericCode:"860"},{name:"Venezuelan bolívar",code:"VES",symbol:"Bs",numericCode:"928"},{name:"Dong",code:"VND",symbol:"₫",numericCode:"704"},{name:"Vatu",code:"VUV",symbol:"",numericCode:"548"},{name:"Tala",code:"WST",symbol:"",numericCode:"882"},{name:"CFA Franc BEAC",code:"XAF",symbol:"",numericCode:"950"},{name:"East Caribbean Dollar",code:"XCD",symbol:"$",numericCode:"951"},{name:"CFA Franc BCEAO",code:"XOF",symbol:"",numericCode:"952"},{name:"CFP Franc",code:"XPF",symbol:"",numericCode:"953"},{name:"Yemeni Rial",code:"YER",symbol:"﷼",numericCode:"886"},{name:"Rand",code:"ZAR",symbol:"R",numericCode:"710"},{name:"Lesotho Loti",code:"LSL",symbol:"",numericCode:"426"},{name:"Namibia Dollar",code:"NAD",symbol:"N$",numericCode:"516"},{name:"Zambian Kwacha",code:"ZMW",symbol:"K",numericCode:"967"},{name:"Zimbabwe Dollar",code:"ZWL",symbol:"",numericCode:"932"}],N3e=["A {{finance.transactionType}} for {{finance.currencyCode}} {{finance.amount}} was made at {{company.name}} via card ending ****{{string.numeric(4)}} on account ***{{string.numeric(4)}}.","A {{finance.transactionType}} of {{finance.currencyCode}} {{finance.amount}} occurred at {{company.name}} using a card ending in ****{{string.numeric(4)}} for account ***{{string.numeric(4)}}.","Payment of {{finance.currencyCode}} {{finance.amount}} for {{finance.transactionType}} at {{company.name}}, processed with card ending ****{{string.numeric(4)}} linked to account ***{{string.numeric(4)}}.","Transaction alert: {{finance.transactionType}} at {{company.name}} using card ending ****{{string.numeric(4)}} for an amount of {{finance.currencyCode}} {{finance.amount}} on account ***{{string.numeric(4)}}.","You made a {{finance.transactionType}} of {{finance.currencyCode}} {{finance.amount}} at {{company.name}} using card ending in ****{{string.numeric(4)}} from account ***{{string.numeric(4)}}.","Your {{finance.transactionType}} of {{finance.currencyCode}} {{finance.amount}} at {{company.name}} was successful. Charged via card ****{{string.numeric(4)}} to account ***{{string.numeric(4)}}.","{{finance.transactionType}} at {{company.name}} with a card ending in ****{{string.numeric(4)}} for {{finance.currencyCode}} {{finance.amount}} from account ***{{string.numeric(4)}}.","{{finance.transactionType}} confirmed at {{company.name}} for {{finance.currencyCode}} {{finance.amount}}, card ending in ****{{string.numeric(4)}} associated with account ***{{string.numeric(4)}}.","{{finance.transactionType}} of {{finance.currencyCode}} {{finance.amount}} at {{company.name}} charged to account ending in {{string.numeric(4)}} using card ending in ****{{string.numeric(4)}}.","{{finance.transactionType}} processed at {{company.name}} for {{finance.currencyCode}} {{finance.amount}}, using card ending ****{{string.numeric(4)}}. Account: ***{{string.numeric(4)}}.","{{finance.transactionType}} transaction at {{company.name}} using card ending with ****{{string.numeric(4)}} for {{finance.currencyCode}} {{finance.amount}} in account ***{{string.numeric(4)}}."],L3e=["deposit","invoice","payment","withdrawal"],M3e={account_type:x3e,credit_card:R3e,currency:D3e,transaction_description_pattern:N3e,transaction_type:L3e},I3e=M3e,O3e=["bitter","creamy","crispy","crunchy","delicious","fluffy","fresh","golden","juicy","moist","rich","salty","savory","smoky","sour","spicy","sweet","tangy","tender","zesty"],B3e=["A classic pie filled with delicious {{food.meat}} and {{food.adjective}} {{food.ingredient}}, baked in a {{food.adjective}} pastry crust and topped with a golden-brown lattice.","A delightful tart combining {{food.adjective}} {{food.vegetable}} and sweet {{food.fruit}}, set in a buttery pastry shell and finished with a hint of {{food.spice}}.","A heartwarming {{food.ethnic_category}} soup, featuring fresh {{food.ingredient}} and an aromatic blend of traditional spices.","A robust {{food.adjective}} stew featuring {{food.ethnic_category}} flavors, loaded with {{food.adjective}} meat, {{food.adjective}} vegetables, and a {{food.adjective}}, {{food.adjective}} broth.","A simple {{food.fruit}} pie. No fancy stuff. Just pie.","A slow-roasted {{animal.bird}} with a {{food.adjective}}, {{food.adjective}} exterior. Stuffed with {{food.fruit}} and covered in {{food.fruit}} sauce. Sides with {{food.vegetable}} puree and wild {{food.vegetable}}.","A special {{color.human}} {{food.ingredient}} from {{location.country}}. To support the strong flavor it is sided with a tablespoon of {{food.spice}}.","A succulent {{food.meat}} steak, encased in a {{food.adjective}} {{food.spice}} crust, served with a side of {{food.spice}} mashed {{food.vegetable}}.","An exquisite {{food.meat}} roast, infused with the essence of {{food.fruit}}, slow-roasted to bring out its natural flavors and served with a side of creamy {{food.vegetable}}","Baked {{food.ingredient}}-stuffed {{food.meat}}, seasoned with {{food.spice}} and {{food.adjective}} herbs, accompanied by roasted {{food.vegetable}} medley.","Crispy fried {{food.meat}} bites, seasoned with {{food.spice}} and served with a tangy {{food.fruit}} dipping sauce.","Fresh mixed greens tossed with {{food.spice}}-rubbed {{food.meat}}, {{food.vegetable}}, and a light dressing.","Fresh {{food.ingredient}} with a pinch of {{food.spice}}, topped by a caramelized {{food.fruit}} with whipped cream","Grilled {{food.meat}} kebabs, marinated in {{food.ethnic_category}} spices and served with a fresh {{food.vegetable}} and {{food.fruit}} salad.","Hearty {{food.ingredient}} and {{food.meat}} stew, slow-cooked with {{food.spice}} and {{food.vegetable}} for a comforting, flavorful meal.","Juicy {{food.meat}}, grilled to your liking and drizzled with a bold {{food.spice}} sauce, served alongside roasted {{food.vegetable}}.","Our {{food.adjective}} {{food.meat}}, slow-cooked to perfection, accompanied by steamed {{food.vegetable}} and a rich, savory gravy.","Tender {{food.meat}} skewers, glazed with a sweet and tangy {{food.fruit}} sauce, served over a bed of fragrant jasmine rice.","Tenderly braised {{food.meat}} in a rich {{food.spice}} and {{food.vegetable}} sauce, served with a side of creamy {{food.vegetable}}.","Three {{food.ingredient}} with {{food.vegetable}}, {{food.vegetable}}, {{food.vegetable}}, {{food.vegetable}} and {{food.ingredient}}. With a side of baked {{food.fruit}}, and your choice of {{food.ingredient}} or {{food.ingredient}}.",'{{number.int({"min":1, "max":99})}}-day aged {{food.meat}} steak, with choice of {{number.int({"min":2, "max":4})}} sides.'],P3e=["California maki","Peking duck","Philadelphia maki","arepas","barbecue ribs","bruschette with tomato","bunny chow","caesar salad","caprese salad","cauliflower penne","cheeseburger","chicken fajitas","chicken milanese","chicken parm","chicken wings","chilli con carne","ebiten maki","fettuccine alfredo","fish and chips","french fries with sausages","french toast","hummus","katsu curry","kebab","lasagne","linguine with clams","massaman curry","meatballs with sauce","mushroom risotto","pappardelle alla bolognese","pasta and beans","pasta carbonara","pasta with tomato and basil","pho","pierogi","pizza","poke","pork belly buns","pork sausage roll","poutine","ricotta stuffed ravioli","risotto with seafood","salmon nigiri","scotch eggs","seafood paella","som tam","souvlaki","stinky tofu","sushi","tacos","teriyaki chicken donburi","tiramisù","tuna sashimi","vegetable soup"],z3e=["{{food.adjective}} {{food.ethnic_category}} stew","{{food.adjective}} {{food.meat}} with {{food.vegetable}}","{{food.ethnic_category}} {{food.ingredient}} soup","{{food.fruit}} and {{food.fruit}} tart","{{food.fruit}} pie","{{food.fruit}}-glazed {{food.meat}} skewers","{{food.fruit}}-infused {{food.meat}} roast","{{food.ingredient}} and {{food.meat}} pie","{{food.ingredient}}-infused {{food.meat}}","{{food.meat}} steak","{{food.meat}} with {{food.fruit}} sauce","{{food.spice}}-crusted {{food.meat}}","{{food.spice}}-rubbed {{food.meat}} salad","{{food.vegetable}} salad","{{person.first_name.generic}}'s special {{food.ingredient}}"],H3e=["Ainu","Albanian","American","Andhra","Anglo-Indian","Arab","Argentine","Armenian","Assyrian","Awadhi","Azerbaijani","Balochi","Bangladeshi","Bashkir","Belarusian","Bengali","Berber","Brazilian","British","Buddhist","Bulgarian","Cajun","Cantonese","Caribbean","Chechen","Chinese","Chinese Islamic","Circassian","Crimean Tatar","Cypriot","Czech","Danish","Egyptian","English","Eritrean","Estonian","Ethiopian","Filipino","French","Georgian","German","Goan","Goan Catholic","Greek","Gujarati","Hyderabad","Indian","Indian Chinese","Indian Singaporean","Indonesian","Inuit","Irish","Italian","Italian-American","Jamaican","Japanese","Jewish - Israeli","Karnataka","Kazakh","Keralite","Korean","Kurdish","Laotian","Latvian","Lebanese","Lithuanian","Louisiana Creole","Maharashtrian","Malay","Malaysian Chinese","Malaysian Indian","Mangalorean","Mediterranean","Mennonite","Mexican","Mordovian","Mughal","Native American","Nepalese","New Mexican","Odia","Pakistani","Parsi","Pashtun","Pennsylvania Dutch","Peranakan","Persian","Peruvian","Polish","Portuguese","Punjabi","Québécois","Rajasthani","Romani","Romanian","Russian","Sami","Serbian","Sindhi","Slovak","Slovenian","Somali","South Indian","Soviet","Spanish","Sri Lankan","Taiwanese","Tamil","Tatar","Texan","Thai","Turkish","Udupi","Ukrainian","Vietnamese","Yamal","Zambian","Zanzibari"],U3e=["apple","apricot","aubergine","avocado","banana","berry","blackberry","blood orange","blueberry","bush tomato","butternut pumpkin","cantaloupe","cavalo","cherry","corella pear","cranberry","cumquat","currant","custard apple","custard apples daikon","date","dragonfruit","dried apricot","elderberry","feijoa","fig","fingerlime","goji berry","grape","grapefruit","guava","honeydew melon","incaberry","jarrahdale pumpkin","juniper berry","kiwi fruit","kiwiberry","lemon","lime","longan","loquat","lychee","mandarin","mango","mangosteen","melon","mulberry","nashi pear","nectarine","olive","orange","papaw","papaya","passionfruit","peach","pear","pineapple","plum","pomegranate","prune","rockmelon","snowpea","sprout","starfruit","strawberry","sultana","tangelo","tomato","watermelon"],$3e=["achacha","adzuki beans","agar","agave syrup","ajowan seed","albacore tuna","alfalfa","allspice","almond oil","almonds","amaranth","amchur","anchovies","aniseed","annatto seed","apple cider vinegar","apple juice","apple juice concentrate","apples","apricots","arborio rice","arrowroot","artichoke","arugula","asafoetida","asian greens","asian noodles","asparagus","aubergine","avocado","avocado oil","avocado spread","bacon","baking powder","baking soda","balsamic vinegar","bamboo shoots","banana","barberry","barley","barramundi","basil basmati rice","bay leaves","bean shoots","bean sprouts","beans","beef","beef stock","beetroot","berries","besan","black eyed beans","blackberries","blood oranges","blue cheese","blue eye trevalla","blue swimmer crab","blueberries","bocconcini","bok choy","bonito flakes","bonza","borlotti beans","bran","brazil nut","bread","brie","broccoli","broccolini","brown flour","brown mushrooms","brown rice","brown rice vinegar","brussels sprouts","buckwheat","buckwheat flour","buckwheat noodles","bulghur","bush tomato","butter","butter beans","buttermilk","butternut lettuce","butternut pumpkin","cabbage","cacao","cake","calamari","camellia tea oil","camembert","camomile","candle nut","cannellini beans","canola oil","cantaloupe","capers","capsicum","caraway seed","cardamom","carob carrot","carrot","cashews","cassia bark","cauliflower","cavalo","cayenne","celery","celery seed","cheddar","cherries","chestnut","chia seeds","chicken","chicken stock","chickory","chickpea","chilli pepper","chinese cabbage","chinese five spice","chives","choy sum","cinnamon","clams","cloves","cocoa powder","coconut","coconut oil","coconut water","coffee","common cultivated mushrooms","corella pear","coriander leaves","coriander seed","corn oil","corn syrup","corn tortilla","cornichons","cornmeal","cos lettuce","cottage cheese","cous cous","crabs","cranberry","cream","cream cheese","cucumber","cumin","cumquat","currants","curry leaves","curry powder","custard apples","dandelion","dark chocolate","dashi","dates","dill","dragonfruit","dried apricots","dried chinese broccoli","duck","edam","edamame","eggplant","eggs","elderberry","endive","english spinach","enoki mushrooms","extra virgin olive oil","farmed prawns","feijoa","fennel","fennel seeds","fenugreek","feta","figs","file powder","fingerlime","fish sauce","fish stock","flat mushrooms","flathead","flaxseed","flaxseed oil","flounder","flour","freekeh","french eschallots","fresh chillies","fromage blanc","fruit","galangal","garam masala","garlic","goat cheese","goat milk","goji berry","grape seed oil","grapefruit","grapes","green beans","green pepper","green tea","green tea noodles","greenwheat freekeh","gruyere","guava","gula melaka","haloumi","ham","haricot beans","harissa","hazelnut","hijiki","hiramasa kingfish","hokkien noodles","honey","honeydew melon","horseradish","hot smoked salmon","hummus","iceberg lettuce","incaberries","jarrahdale pumpkin","jasmine rice","jelly","jerusalem artichoke","jewfish","jicama","juniper berries","kale","kangaroo","kecap manis","kenchur","kidney beans","kidneys","kiwi berries","kiwi fruit","kohlrabi","kokam","kombu","koshihikari rice","kudzu","kumera","lamb","lavender flowers","leeks","lemon","lemongrass","lentils","lettuce","licorice","lime leaves","limes","liver","lobster","longan","loquats","lotus root","lychees","macadamia nut","macadamia oil","mace","mackerel","mahi mahi","mahlab","malt vinegar","mandarins","mango","mangosteens","maple syrup","margarine","marigold","marjoram","mastic","melon","milk","milk chocolate","mint","miso","molasses","monkfish","morwong","mountain bread","mozzarella","muesli","mulberries","mullet","mung beans","mussels","mustard","mustard seed","nashi pear","nasturtium","nectarines","nori","nutmeg","nutritional yeast","nuts","oat flour","oatmeal","oats","octopus","okra","olive oil","olives","omega spread","onion","oranges","oregano","oyster mushrooms","oyster sauce","oysters","pandanus leaves","papaw","papaya","paprik","parmesan cheese","parrotfish","parsley","parsnip","passionfruit","pasta","peaches","peanuts","pear","pear juice","pears","peas","pecan nut","pecorino","pepitas","peppercorns","peppermint","peppers","persimmon","pine nut","pineapple","pinto beans","pistachio nut","plums","polenta","pomegranate","poppy seed","porcini mushrooms","pork","potato flour","potatoes","provolone","prunes","pumpkin","pumpkin seed","purple carrot","purple rice","quark","quince","quinoa","radicchio","radish","raisin","raspberry","red cabbage","red lentils","red pepper","red wine","red wine vinegar","redfish","rhubarb","rice flour","rice noodles","rice paper","rice syrup","ricemilk","ricotta","rockmelon","rose water","rosemary","rye","rye bread","safflower oil","saffron","sage","sake","salmon","sardines","sausages","scallops","sea salt","semolina","sesame oil","sesame seeds","shark","shiitake mushrooms","silverbeet","slivered almonds","smoked trout","snapper","snowpea sprouts","snowpeas","soba","sour dough bread","soy","soy beans","soy flour","soy milk","soy sauce","soymilk","spearmint","spelt","spelt bread","spinach","spring onions","sprouts","squash","squid","star anise","star fruit","starfruit","stevia","strawberries","sugar","sultanas","sun-dried tomatoes","sunflower oil","sunflower seeds","sweet chilli sauce","sweet potato","swiss chard","swordfish","szechuan pepperberry","tabasco","tahini","taleggio cheese","tamari","tamarillo","tangelo","tapioca","tapioca flour","tarragon","tea","tea oil","tempeh","thyme","tinned","tofu","tom yum","tomatoes","trout","tuna","turkey","turmeric","turnips","unbleached flour","vanilla beans","vegetable oil","vegetable spaghetti","vegetable stock","vermicelli noodles","vinegar","wakame","walnut","warehou","wasabi","water","watercress","watermelon","wattleseed","wheat","wheatgrass juice","white bread","white flour","white rice","white wine","white wine vinegar","whiting wild rice","wholegrain bread","wholemeal","wholewheat flour","william pear","yeast","yellow papaw","yellowtail kingfish","yoghurt","yogurt","zucchini"],G3e=["beef","chicken","crocodile","duck","emu","goose","kangaroo","lamb","ostrich","pigeon","pork","quail","rabbit","salmon","turkey","venison"],j3e=["achiote seed","ajwain seed","ajwan seed","allspice","amchoor","anise","anise star","aniseed","annatto seed","arrowroot","asafoetida","baharat","balti masala","balti stir fry mix","basil","bay leaves","bbq","caraway seed","cardamom","cassia","cayenne pepper","celery","chamomile","chervil","chilli","chilli pepper","chillies","china star","chives","cinnamon","cloves","colombo","coriander","cumin","curly leaf parsley","curry","dhansak","dill","fennel seed","fenugreek","fines herbes","five spice","french lavender","galangal","garam masala","garlic","german chamomile","ginger","green cardamom","herbes de provence","jalfrezi","jerk","kaffir leaves","korma","lavender","lemon grass","lemon pepper","lime leaves","liquorice root","mace","mango","marjoram","methi","mint","mustard","nutmeg","onion seed","orange zest","oregano","paprika","parsley","pepper","peppercorns","pimento","piri piri","poppy seed","pot marjoram","poudre de colombo","ras-el-hanout","rice paper","rogan josh","rose baie","rosemary","saffron","sage","sesame seed","spearmint","sumac","sweet basil","sweet laurel","tagine","tandoori masala","tarragon","thyme","tikka masala","turmeric","vanilla","zahtar"],q3e=["artichoke","arugula","asian greens","asparagus","bean shoots","bean sprouts","beans","beetroot","bok choy","broccoli","broccolini","brussels sprouts","butternut lettuce","cabbage","capers","carob carrot","carrot","cauliflower","celery","chilli pepper","chinese cabbage","chives","cornichons","cos lettuce","cucumber","dried chinese broccoli","eggplant","endive","english spinach","french eschallots","fresh chillies","garlic","green beans","green pepper","hijiki","iceberg lettuce","jerusalem artichoke","jicama","kale","kohlrabi","leeks","lettuce","okra","onion","parsnip","peas","peppers","potatoes","pumpkin","purple carrot","radicchio","radish","raspberry","red cabbage","red pepper","rhubarb","snowpea sprouts","spinach","squash","sun dried tomatoes","sweet potato","swiss chard","turnips","zucchini"],W3e={adjective:O3e,description_pattern:B3e,dish:P3e,dish_pattern:z3e,ethnic_category:H3e,fruit:U3e,ingredient:$3e,meat:G3e,spice:j3e,vegetable:q3e},V3e=W3e,K3e=["1080p","auxiliary","back-end","bluetooth","cross-platform","digital","haptic","mobile","multi-byte","neural","online","open-source","optical","primary","redundant","solid state","virtual","wireless"],Y3e=["backing up","bypassing","calculating","compressing","connecting","copying","generating","hacking","indexing","navigating","overriding","parsing","programming","quantifying","synthesizing","transmitting"],J3e=["alarm","application","array","bandwidth","bus","capacitor","card","circuit","driver","feed","firewall","hard drive","interface","matrix","microchip","monitor","panel","pixel","port","program","protocol","sensor","system","transmitter"],X3e=["I'll {{verb}} the {{adjective}} {{abbreviation}} {{noun}}, that should {{noun}} the {{abbreviation}} {{noun}}!","If we {{verb}} the {{noun}}, we can get to the {{abbreviation}} {{noun}} through the {{adjective}} {{abbreviation}} {{noun}}!","The {{abbreviation}} {{noun}} is down, {{verb}} the {{adjective}} {{noun}} so we can {{verb}} the {{abbreviation}} {{noun}}!","Try to {{verb}} the {{abbreviation}} {{noun}}, maybe it will {{verb}} the {{adjective}} {{noun}}!","Use the {{adjective}} {{abbreviation}} {{noun}}, then you can {{verb}} the {{adjective}} {{noun}}!","We need to {{verb}} the {{adjective}} {{abbreviation}} {{noun}}!","You can't {{verb}} the {{noun}} without {{ingverb}} the {{adjective}} {{abbreviation}} {{noun}}!","{{ingverb}} the {{noun}} won't do anything, we need to {{verb}} the {{adjective}} {{abbreviation}} {{noun}}!"],Z3e=["back up","bypass","calculate","compress","connect","copy","generate","hack","index","input","navigate","override","parse","program","quantify","reboot","synthesize","transmit"],Q3e={adjective:K3e,ingverb:Y3e,noun:J3e,phrase:X3e,verb:Z3e},e4e=Q3e,t4e=["biz","com","info","name","net","org"],r4e=["example.com","example.net","example.org"],n4e=["gmail.com","hotmail.com","yahoo.com"],a4e={domain_suffix:t4e,example_email:r4e,free_email:n4e},i4e=a4e,o4e=["#####","####","###"],s4e=["Abilene","Akron","Alafaya","Alameda","Albany","Albuquerque","Alexandria","Alhambra","Aliso Viejo","Allen","Allentown","Aloha","Alpharetta","Altadena","Altamonte Springs","Altoona","Amarillo","Ames","Anaheim","Anchorage","Anderson","Ankeny","Ann Arbor","Annandale","Antelope","Antioch","Apex","Apopka","Apple Valley","Appleton","Arcadia","Arden-Arcade","Arecibo","Arlington","Arlington Heights","Arvada","Ashburn","Asheville","Aspen Hill","Atascocita","Athens-Clarke County","Atlanta","Attleboro","Auburn","Augusta-Richmond County","Aurora","Austin","Avondale","Azusa","Bakersfield","Baldwin Park","Baltimore","Barnstable Town","Bartlett","Baton Rouge","Battle Creek","Bayamon","Bayonne","Baytown","Beaumont","Beavercreek","Beaverton","Bedford","Bel Air South","Bell Gardens","Belleville","Bellevue","Bellflower","Bellingham","Bend","Bentonville","Berkeley","Berwyn","Bethesda","Bethlehem","Billings","Biloxi","Binghamton","Birmingham","Bismarck","Blacksburg","Blaine","Bloomington","Blue Springs","Boca Raton","Boise City","Bolingbrook","Bonita Springs","Bossier City","Boston","Bothell","Boulder","Bountiful","Bowie","Bowling Green","Boynton Beach","Bozeman","Bradenton","Brandon","Brentwood","Bridgeport","Bristol","Brockton","Broken Arrow","Brookhaven","Brookline","Brooklyn Park","Broomfield","Brownsville","Bryan","Buckeye","Buena Park","Buffalo","Buffalo Grove","Burbank","Burien","Burke","Burleson","Burlington","Burnsville","Caguas","Caldwell","Camarillo","Cambridge","Camden","Canton","Cape Coral","Carlsbad","Carmel","Carmichael","Carolina","Carrollton","Carson","Carson City","Cary","Casa Grande","Casas Adobes","Casper","Castle Rock","Castro Valley","Catalina Foothills","Cathedral City","Catonsville","Cedar Hill","Cedar Park","Cedar Rapids","Centennial","Centreville","Ceres","Cerritos","Champaign","Chandler","Chapel Hill","Charleston","Charlotte","Charlottesville","Chattanooga","Cheektowaga","Chesapeake","Chesterfield","Cheyenne","Chicago","Chico","Chicopee","Chino","Chino Hills","Chula Vista","Cicero","Cincinnati","Citrus Heights","Clarksville","Clearwater","Cleveland","Cleveland Heights","Clifton","Clovis","Coachella","Coconut Creek","Coeur d'Alene","College Station","Collierville","Colorado Springs","Colton","Columbia","Columbus","Commerce City","Compton","Concord","Conroe","Conway","Coon Rapids","Coral Gables","Coral Springs","Corona","Corpus Christi","Corvallis","Costa Mesa","Council Bluffs","Country Club","Covina","Cranston","Cupertino","Cutler Bay","Cuyahoga Falls","Cypress","Dale City","Dallas","Daly City","Danbury","Danville","Davenport","Davie","Davis","Dayton","Daytona Beach","DeKalb","DeSoto","Dearborn","Dearborn Heights","Decatur","Deerfield Beach","Delano","Delray Beach","Deltona","Denton","Denver","Des Moines","Des Plaines","Detroit","Diamond Bar","Doral","Dothan","Downers Grove","Downey","Draper","Dublin","Dubuque","Duluth","Dundalk","Dunwoody","Durham","Eagan","East Hartford","East Honolulu","East Lansing","East Los Angeles","East Orange","East Providence","Eastvale","Eau Claire","Eden Prairie","Edina","Edinburg","Edmond","El Cajon","El Centro","El Dorado Hills","El Monte","El Paso","Elgin","Elizabeth","Elk Grove","Elkhart","Ellicott City","Elmhurst","Elyria","Encinitas","Enid","Enterprise","Erie","Escondido","Euclid","Eugene","Euless","Evanston","Evansville","Everett","Fairfield","Fall River","Fargo","Farmington","Farmington Hills","Fayetteville","Federal Way","Findlay","Fishers","Flagstaff","Flint","Florence-Graham","Florin","Florissant","Flower Mound","Folsom","Fond du Lac","Fontana","Fort Collins","Fort Lauderdale","Fort Myers","Fort Pierce","Fort Smith","Fort Wayne","Fort Worth","Fountain Valley","Fountainebleau","Framingham","Franklin","Frederick","Freeport","Fremont","Fresno","Frisco","Fullerton","Gainesville","Gaithersburg","Galveston","Garden Grove","Gardena","Garland","Gary","Gastonia","Georgetown","Germantown","Gilbert","Gilroy","Glen Burnie","Glendale","Glendora","Glenview","Goodyear","Grand Forks","Grand Island","Grand Junction","Grand Prairie","Grand Rapids","Grapevine","Great Falls","Greeley","Green Bay","Greensboro","Greenville","Greenwood","Gresham","Guaynabo","Gulfport","Hacienda Heights","Hackensack","Haltom City","Hamilton","Hammond","Hampton","Hanford","Harlingen","Harrisburg","Harrisonburg","Hartford","Hattiesburg","Haverhill","Hawthorne","Hayward","Hemet","Hempstead","Henderson","Hendersonville","Hesperia","Hialeah","Hicksville","High Point","Highland","Highlands Ranch","Hillsboro","Hilo","Hoboken","Hoffman Estates","Hollywood","Homestead","Honolulu","Hoover","Houston","Huntersville","Huntington","Huntington Beach","Huntington Park","Huntsville","Hutchinson","Idaho Falls","Independence","Indianapolis","Indio","Inglewood","Iowa City","Irondequoit","Irvine","Irving","Jackson","Jacksonville","Janesville","Jefferson City","Jeffersonville","Jersey City","Johns Creek","Johnson City","Joliet","Jonesboro","Joplin","Jupiter","Jurupa Valley","Kalamazoo","Kannapolis","Kansas City","Kearny","Keller","Kendale Lakes","Kendall","Kenner","Kennewick","Kenosha","Kent","Kentwood","Kettering","Killeen","Kingsport","Kirkland","Kissimmee","Knoxville","Kokomo","La Crosse","La Habra","La Mesa","La Mirada","Lacey","Lafayette","Laguna Niguel","Lake Charles","Lake Elsinore","Lake Forest","Lake Havasu City","Lake Ridge","Lakeland","Lakeville","Lakewood","Lancaster","Lansing","Laredo","Largo","Las Cruces","Las Vegas","Lauderhill","Lawrence","Lawton","Layton","League City","Lee's Summit","Leesburg","Lehi","Lehigh Acres","Lenexa","Levittown","Lewisville","Lexington-Fayette","Lincoln","Linden","Little Rock","Littleton","Livermore","Livonia","Lodi","Logan","Lombard","Lompoc","Long Beach","Longmont","Longview","Lorain","Los Angeles","Louisville/Jefferson County","Loveland","Lowell","Lubbock","Lynchburg","Lynn","Lynwood","Macon-Bibb County","Madera","Madison","Malden","Manchester","Manhattan","Mansfield","Manteca","Maple Grove","Margate","Maricopa","Marietta","Marysville","Mayaguez","McAllen","McKinney","McLean","Medford","Melbourne","Memphis","Menifee","Mentor","Merced","Meriden","Meridian","Mesa","Mesquite","Metairie","Methuen Town","Miami","Miami Beach","Miami Gardens","Middletown","Midland","Midwest City","Milford","Millcreek","Milpitas","Milwaukee","Minneapolis","Minnetonka","Minot","Miramar","Mishawaka","Mission","Mission Viejo","Missoula","Missouri City","Mobile","Modesto","Moline","Monroe","Montebello","Monterey Park","Montgomery","Moore","Moreno Valley","Morgan Hill","Mount Pleasant","Mount Prospect","Mount Vernon","Mountain View","Muncie","Murfreesboro","Murray","Murrieta","Nampa","Napa","Naperville","Nashua","Nashville-Davidson","National City","New Bedford","New Braunfels","New Britain","New Brunswick","New Haven","New Orleans","New Rochelle","New York","Newark","Newport Beach","Newport News","Newton","Niagara Falls","Noblesville","Norfolk","Normal","Norman","North Bethesda","North Charleston","North Highlands","North Las Vegas","North Lauderdale","North Little Rock","North Miami","North Miami Beach","North Port","North Richland Hills","Norwalk","Novato","Novi","O'Fallon","Oak Lawn","Oak Park","Oakland","Oakland Park","Ocala","Oceanside","Odessa","Ogden","Oklahoma City","Olathe","Olympia","Omaha","Ontario","Orange","Orem","Orland Park","Orlando","Oro Valley","Oshkosh","Overland Park","Owensboro","Oxnard","Palatine","Palm Bay","Palm Beach Gardens","Palm Coast","Palm Desert","Palm Harbor","Palm Springs","Palmdale","Palo Alto","Paradise","Paramount","Parker","Parma","Pasadena","Pasco","Passaic","Paterson","Pawtucket","Peabody","Pearl City","Pearland","Pembroke Pines","Pensacola","Peoria","Perris","Perth Amboy","Petaluma","Pflugerville","Pharr","Philadelphia","Phoenix","Pico Rivera","Pine Bluff","Pine Hills","Pinellas Park","Pittsburg","Pittsburgh","Pittsfield","Placentia","Plainfield","Plano","Plantation","Pleasanton","Plymouth","Pocatello","Poinciana","Pomona","Pompano Beach","Ponce","Pontiac","Port Arthur","Port Charlotte","Port Orange","Port St. Lucie","Portage","Porterville","Portland","Portsmouth","Potomac","Poway","Providence","Provo","Pueblo","Quincy","Racine","Raleigh","Rancho Cordova","Rancho Cucamonga","Rancho Palos Verdes","Rancho Santa Margarita","Rapid City","Reading","Redding","Redlands","Redmond","Redondo Beach","Redwood City","Reno","Renton","Reston","Revere","Rialto","Richardson","Richland","Richmond","Rio Rancho","Riverside","Riverton","Riverview","Roanoke","Rochester","Rochester Hills","Rock Hill","Rockford","Rocklin","Rockville","Rockwall","Rocky Mount","Rogers","Rohnert Park","Rosemead","Roseville","Roswell","Round Rock","Rowland Heights","Rowlett","Royal Oak","Sacramento","Saginaw","Salem","Salina","Salinas","Salt Lake City","Sammamish","San Angelo","San Antonio","San Bernardino","San Bruno","San Buenaventura (Ventura)","San Clemente","San Diego","San Francisco","San Jacinto","San Jose","San Juan","San Leandro","San Luis Obispo","San Marcos","San Mateo","San Rafael","San Ramon","San Tan Valley","Sandy","Sandy Springs","Sanford","Santa Ana","Santa Barbara","Santa Clara","Santa Clarita","Santa Cruz","Santa Fe","Santa Maria","Santa Monica","Santa Rosa","Santee","Sarasota","Savannah","Sayreville","Schaumburg","Schenectady","Scottsdale","Scranton","Seattle","Severn","Shawnee","Sheboygan","Shoreline","Shreveport","Sierra Vista","Silver Spring","Simi Valley","Sioux City","Sioux Falls","Skokie","Smyrna","Somerville","South Bend","South Gate","South Hill","South Jordan","South San Francisco","South Valley","South Whittier","Southaven","Southfield","Sparks","Spokane","Spokane Valley","Spring","Spring Hill","Spring Valley","Springdale","Springfield","St. Charles","St. Clair Shores","St. Cloud","St. George","St. Joseph","St. Louis","St. Louis Park","St. Paul","St. Peters","St. Petersburg","Stamford","State College","Sterling Heights","Stillwater","Stockton","Stratford","Strongsville","Suffolk","Sugar Land","Summerville","Sunnyvale","Sunrise","Sunrise Manor","Surprise","Syracuse","Tacoma","Tallahassee","Tamarac","Tamiami","Tampa","Taunton","Taylor","Taylorsville","Temecula","Tempe","Temple","Terre Haute","Texas City","The Hammocks","The Villages","The Woodlands","Thornton","Thousand Oaks","Tigard","Tinley Park","Titusville","Toledo","Toms River","Tonawanda","Topeka","Torrance","Town 'n' Country","Towson","Tracy","Trenton","Troy","Trujillo Alto","Tuckahoe","Tucson","Tulare","Tulsa","Turlock","Tuscaloosa","Tustin","Twin Falls","Tyler","Union City","University","Upland","Urbana","Urbandale","Utica","Vacaville","Valdosta","Vallejo","Vancouver","Victoria","Victorville","Vineland","Virginia Beach","Visalia","Vista","Waco","Waipahu","Waldorf","Walnut Creek","Waltham","Warner Robins","Warren","Warwick","Washington","Waterbury","Waterloo","Watsonville","Waukegan","Waukesha","Wauwatosa","Wellington","Wesley Chapel","West Allis","West Babylon","West Covina","West Des Moines","West Hartford","West Haven","West Jordan","West Lafayette","West New York","West Palm Beach","West Sacramento","West Seneca","West Valley City","Westfield","Westland","Westminster","Weston","Weymouth Town","Wheaton","White Plains","Whittier","Wichita","Wichita Falls","Wilmington","Wilson","Winston-Salem","Woodbury","Woodland","Worcester","Wylie","Wyoming","Yakima","Yonkers","Yorba Linda","York","Youngstown","Yuba City","Yucaipa","Yuma"],l4e=["{{location.city_prefix}} {{person.first_name.generic}}{{location.city_suffix}}","{{location.city_prefix}} {{person.first_name.generic}}","{{person.first_name.generic}}{{location.city_suffix}}","{{person.last_name.generic}}{{location.city_suffix}}","{{location.city_name}}"],u4e=["North","East","West","South","New","Lake","Port","Fort"],c4e=["town","ton","land","ville","berg","burgh","boro","borough","bury","view","port","mouth","stad","stead","furt","chester","cester","fort","field","haven","side","shire","worth"],d4e=["Africa","Antarctica","Asia","Australia","Europe","North America","South America"],h4e=["Afghanistan","Aland Islands","Albania","Algeria","American Samoa","Andorra","Angola","Anguilla","Antarctica","Antigua and Barbuda","Argentina","Armenia","Aruba","Australia","Austria","Azerbaijan","Bahamas","Bahrain","Bangladesh","Barbados","Belarus","Belgium","Belize","Benin","Bermuda","Bhutan","Bolivia","Bonaire, Sint Eustatius and Saba","Bosnia and Herzegovina","Botswana","Bouvet Island","Brazil","British Indian Ocean Territory (Chagos Archipelago)","Brunei Darussalam","Bulgaria","Burkina Faso","Burundi","Cambodia","Cameroon","Canada","Cape Verde","Cayman Islands","Central African Republic","Chad","Chile","China","Christmas Island","Cocos (Keeling) Islands","Colombia","Comoros","Congo","Cook Islands","Costa Rica","Cote d'Ivoire","Croatia","Cuba","Curacao","Cyprus","Czechia","Democratic Republic of the Congo","Denmark","Djibouti","Dominica","Dominican Republic","Ecuador","Egypt","El Salvador","Equatorial Guinea","Eritrea","Estonia","Eswatini","Ethiopia","Faroe Islands","Falkland Islands (Malvinas)","Fiji","Finland","France","French Guiana","French Polynesia","French Southern Territories","Gabon","Gambia","Georgia","Germany","Ghana","Gibraltar","Greece","Greenland","Grenada","Guadeloupe","Guam","Guatemala","Guernsey","Guinea","Guinea-Bissau","Guyana","Haiti","Heard Island and McDonald Islands","Holy See (Vatican City State)","Honduras","Hong Kong","Hungary","Iceland","India","Indonesia","Iran","Iraq","Ireland","Isle of Man","Israel","Italy","Jamaica","Japan","Jersey","Jordan","Kazakhstan","Kenya","Kiribati","Democratic People's Republic of Korea","Republic of Korea","Kuwait","Kyrgyz Republic","Lao People's Democratic Republic","Latvia","Lebanon","Lesotho","Liberia","Libyan Arab Jamahiriya","Liechtenstein","Lithuania","Luxembourg","Macao","Madagascar","Malawi","Malaysia","Maldives","Mali","Malta","Marshall Islands","Martinique","Mauritania","Mauritius","Mayotte","Mexico","Micronesia","Moldova","Monaco","Mongolia","Montenegro","Montserrat","Morocco","Mozambique","Myanmar","Namibia","Nauru","Nepal","Netherlands","New Caledonia","New Zealand","Nicaragua","Niger","Nigeria","Niue","Norfolk Island","North Macedonia","Northern Mariana Islands","Norway","Oman","Pakistan","Palau","Palestine","Panama","Papua New Guinea","Paraguay","Peru","Philippines","Pitcairn Islands","Poland","Portugal","Puerto Rico","Qatar","Reunion","Romania","Russian Federation","Rwanda","Saint Barthelemy","Saint Helena","Saint Kitts and Nevis","Saint Lucia","Saint Martin","Saint Pierre and Miquelon","Saint Vincent and the Grenadines","Samoa","San Marino","Sao Tome and Principe","Saudi Arabia","Senegal","Serbia","Seychelles","Sierra Leone","Singapore","Sint Maarten","Slovakia","Slovenia","Solomon Islands","Somalia","South Africa","South Georgia and the South Sandwich Islands","South Sudan","Spain","Sri Lanka","Sudan","Suriname","Svalbard & Jan Mayen Islands","Sweden","Switzerland","Syrian Arab Republic","Taiwan","Tajikistan","Tanzania","Thailand","Timor-Leste","Togo","Tokelau","Tonga","Trinidad and Tobago","Tunisia","Turkey","Turkmenistan","Turks and Caicos Islands","Tuvalu","Uganda","Ukraine","United Arab Emirates","United Kingdom","United States of America","United States Minor Outlying Islands","Uruguay","Uzbekistan","Vanuatu","Venezuela","Vietnam","Virgin Islands, British","Virgin Islands, U.S.","Wallis and Futuna","Western Sahara","Yemen","Zambia","Zimbabwe"],p4e=["Adams County","Calhoun County","Carroll County","Clark County","Clay County","Crawford County","Douglas County","Fayette County","Franklin County","Grant County","Greene County","Hamilton County","Hancock County","Henry County","Jackson County","Jefferson County","Johnson County","Lake County","Lawrence County","Lee County","Lincoln County","Logan County","Madison County","Marion County","Marshall County","Monroe County","Montgomery County","Morgan County","Perry County","Pike County","Polk County","Scott County","Union County","Warren County","Washington County","Wayne County","Avon","Bedfordshire","Berkshire","Borders","Buckinghamshire","Cambridgeshire","Central","Cheshire","Cleveland","Clwyd","Cornwall","County Antrim","County Armagh","County Down","County Fermanagh","County Londonderry","County Tyrone","Cumbria","Derbyshire","Devon","Dorset","Dumfries and Galloway","Durham","Dyfed","East Sussex","Essex","Fife","Gloucestershire","Grampian","Greater Manchester","Gwent","Gwynedd County","Hampshire","Herefordshire","Hertfordshire","Highlands and Islands","Humberside","Isle of Wight","Kent","Lancashire","Leicestershire","Lincolnshire","Lothian","Merseyside","Mid Glamorgan","Norfolk","North Yorkshire","Northamptonshire","Northumberland","Nottinghamshire","Oxfordshire","Powys","Rutland","Shropshire","Somerset","South Glamorgan","South Yorkshire","Staffordshire","Strathclyde","Suffolk","Surrey","Tayside","Tyne and Wear","Warwickshire","West Glamorgan","West Midlands","West Sussex","West Yorkshire","Wiltshire","Worcestershire"],f4e={cardinal:["North","East","South","West"],cardinal_abbr:["N","E","S","W"],ordinal:["Northeast","Northwest","Southeast","Southwest"],ordinal_abbr:["NE","NW","SE","SW"]},m4e=[{name:"Afrikaans",alpha2:"af",alpha3:"afr"},{name:"Azerbaijani",alpha2:"az",alpha3:"aze"},{name:"Maldivian",alpha2:"dv",alpha3:"div"},{name:"Farsi/Persian",alpha2:"fa",alpha3:"fas"},{name:"Latvian",alpha2:"lv",alpha3:"lav"},{name:"Indonesian",alpha2:"id",alpha3:"ind"},{name:"Nepali",alpha2:"ne",alpha3:"nep"},{name:"Thai",alpha2:"th",alpha3:"tha"},{name:"Uzbek",alpha2:"uz",alpha3:"uzb"},{name:"Yoruba",alpha2:"yo",alpha3:"yor"},{name:"Pashto",alpha2:"ps",alpha3:"pus"},{name:"English",alpha2:"en",alpha3:"eng"},{name:"Urdu",alpha2:"ur",alpha3:"urd"},{name:"German",alpha2:"de",alpha3:"deu"},{name:"French",alpha2:"fr",alpha3:"fra"},{name:"Spanish",alpha2:"es",alpha3:"spa"},{name:"Italian",alpha2:"it",alpha3:"ita"},{name:"Dutch",alpha2:"nl",alpha3:"nld"},{name:"Russian",alpha2:"ru",alpha3:"rus"},{name:"Portuguese",alpha2:"pt",alpha3:"por"},{name:"Polish",alpha2:"pl",alpha3:"pol"},{name:"Arabic",alpha2:"ar",alpha3:"ara"},{name:"Japanese",alpha2:"ja",alpha3:"jpn"},{name:"Chinese",alpha2:"zh",alpha3:"zho"},{name:"Hindi",alpha2:"hi",alpha3:"hin"},{name:"Bengali",alpha2:"bn",alpha3:"ben"},{name:"Gujarati",alpha2:"gu",alpha3:"guj"},{name:"Tamil",alpha2:"ta",alpha3:"tam"},{name:"Telugu",alpha2:"te",alpha3:"tel"},{name:"Punjabi",alpha2:"pa",alpha3:"pan"},{name:"Vietnamese",alpha2:"vi",alpha3:"vie"},{name:"Korean",alpha2:"ko",alpha3:"kor"},{name:"Turkish",alpha2:"tr",alpha3:"tur"},{name:"Swedish",alpha2:"sv",alpha3:"swe"},{name:"Greek",alpha2:"el",alpha3:"ell"},{name:"Czech",alpha2:"cs",alpha3:"ces"},{name:"Hungarian",alpha2:"hu",alpha3:"hun"},{name:"Romanian",alpha2:"ro",alpha3:"ron"},{name:"Ukrainian",alpha2:"uk",alpha3:"ukr"},{name:"Norwegian",alpha2:"no",alpha3:"nor"},{name:"Serbian",alpha2:"sr",alpha3:"srp"},{name:"Croatian",alpha2:"hr",alpha3:"hrv"},{name:"Slovak",alpha2:"sk",alpha3:"slk"},{name:"Slovenian",alpha2:"sl",alpha3:"slv"},{name:"Icelandic",alpha2:"is",alpha3:"isl"},{name:"Finnish",alpha2:"fi",alpha3:"fin"},{name:"Danish",alpha2:"da",alpha3:"dan"},{name:"Swahili",alpha2:"sw",alpha3:"swa"},{name:"Bashkir",alpha2:"ba",alpha3:"bak"},{name:"Basque",alpha2:"eu",alpha3:"eus"},{name:"Catalan",alpha2:"ca",alpha3:"cat"},{name:"Galician",alpha2:"gl",alpha3:"glg"},{name:"Esperanto",alpha2:"eo",alpha3:"epo"},{name:"Fijian",alpha2:"fj",alpha3:"fij"},{name:"Malagasy",alpha2:"mg",alpha3:"mlg"},{name:"Maltese",alpha2:"mt",alpha3:"mlt"},{name:"Albanian",alpha2:"sq",alpha3:"sqi"},{name:"Armenian",alpha2:"hy",alpha3:"hye"},{name:"Georgian",alpha2:"ka",alpha3:"kat"},{name:"Macedonian",alpha2:"mk",alpha3:"mkd"},{name:"Kazakh",alpha2:"kk",alpha3:"kaz"},{name:"Haitian Creole",alpha2:"ht",alpha3:"hat"},{name:"Mongolian",alpha2:"mn",alpha3:"mon"},{name:"Kyrgyz",alpha2:"ky",alpha3:"kir"},{name:"Finnish",alpha2:"fi",alpha3:"fin"},{name:"Tagalog",alpha2:"tl",alpha3:"tgl"},{name:"Malay",alpha2:"ms",alpha3:"msa"},{name:"Tajik",alpha2:"tg",alpha3:"tgk"},{name:"Swati",alpha2:"ss",alpha3:"ssw"},{name:"Tatar",alpha2:"tt",alpha3:"tat"},{name:"Zulu",alpha2:"zu",alpha3:"zul"}],g4e=["#####","#####-####"],b4e=["Apt. ###","Suite ###"],y4e=["Alabama","Alaska","Arizona","Arkansas","California","Colorado","Connecticut","Delaware","Florida","Georgia","Hawaii","Idaho","Illinois","Indiana","Iowa","Kansas","Kentucky","Louisiana","Maine","Maryland","Massachusetts","Michigan","Minnesota","Mississippi","Missouri","Montana","Nebraska","Nevada","New Hampshire","New Jersey","New Mexico","New York","North Carolina","North Dakota","Ohio","Oklahoma","Oregon","Pennsylvania","Rhode Island","South Carolina","South Dakota","Tennessee","Texas","Utah","Vermont","Virginia","Washington","West Virginia","Wisconsin","Wyoming"],v4e=["AL","AK","AZ","AR","CA","CO","CT","DE","FL","GA","HI","ID","IL","IN","IA","KS","KY","LA","ME","MD","MA","MI","MN","MS","MO","MT","NE","NV","NH","NJ","NM","NY","NC","ND","OH","OK","OR","PA","RI","SC","SD","TN","TX","UT","VT","VA","WA","WV","WI","WY"],S4e={normal:"{{location.buildingNumber}} {{location.street}}",full:"{{location.buildingNumber}} {{location.street}} {{location.secondaryAddress}}"},E4e=["10th Street","11th Street","12th Street","13th Street","14th Street","15th Street","16th Street","1st Avenue","1st Street","2nd Avenue","2nd Street","3rd Avenue","3rd Street","4th Avenue","4th Street","5th Avenue","5th Street","6th Avenue","6th Street","7th Avenue","7th Street","8th Avenue","8th Street","9th Street","A Street","Abbey Road","Adams Avenue","Adams Street","Airport Road","Albany Road","Albert Road","Albion Street","Alexandra Road","Alfred Street","Alma Street","Ash Close","Ash Grove","Ash Road","Ash Street","Aspen Close","Atlantic Avenue","Avenue Road","Back Lane","Baker Street","Balmoral Road","Barn Close","Barton Road","Bath Road","Bath Street","Bay Street","Beach Road","Bedford Road","Beech Close","Beech Drive","Beech Grove","Beech Road","Beechwood Avenue","Bell Lane","Belmont Road","Birch Avenue","Birch Close","Birch Grove","Birch Road","Blind Lane","Bluebell Close","Boundary Road","Bramble Close","Bramley Close","Bridge Road","Bridge Street","Broad Lane","Broad Street","Broadway","Broadway Avenue","Broadway Street","Brook Lane","Brook Road","Brook Street","Brookside","Buckingham Road","Cambridge Street","Canal Street","Castle Close","Castle Lane","Castle Road","Castle Street","Cavendish Road","Cedar Avenue","Cedar Close","Cedar Grove","Cedar Road","Cedar Street","Cemetery Road","Center Avenue","Center Road","Center Street","Central Avenue","Central Street","Chapel Close","Chapel Hill","Chapel Road","Chapel Street","Charles Street","Cherry Close","Cherry Street","Cherry Tree Close","Chester Road","Chestnut Close","Chestnut Drive","Chestnut Grove","Chestnut Street","Church Avenue","Church Close","Church Hill","Church Lane","Church Path","Church Road","Church Street","Church View","Church Walk","Claremont Road","Clarence Road","Clarence Street","Clarendon Road","Clark Street","Clay Lane","Cleveland Street","Cliff Road","Clifton Road","Clinton Street","College Avenue","College Street","Columbia Avenue","Commerce Street","Commercial Road","Commercial Street","Common Lane","Coronation Avenue","Coronation Road","County Line Road","County Road","Court Street","Cow Lane","Crescent Road","Cromwell Road","Cross Lane","Cross Street","Crown Street","Cumberland Street","Dale Street","Dark Lane","Davis Street","Depot Street","Derby Road","Derwent Close","Devonshire Road","Division Street","Douglas Road","Duke Street","E 10th Street","E 11th Street","E 12th Street","E 14th Street","E 1st Street","E 2nd Street","E 3rd Street","E 4th Avenue","E 4th Street","E 5th Street","E 6th Avenue","E 6th Street","E 7th Street","E 8th Street","E 9th Street","E Bridge Street","E Broad Street","E Broadway","E Broadway Street","E Cedar Street","E Center Street","E Central Avenue","E Church Street","E Elm Street","E Franklin Street","E Front Street","E Grand Avenue","E High Street","E Jackson Street","E Jefferson Street","E Main","E Main Street","E Maple Street","E Market Street","E North Street","E Oak Street","E Park Avenue","E Pine Street","E River Road","E South Street","E State Street","E Union Street","E Walnut Street","E Washington Avenue","E Washington Street","E Water Street","East Avenue","East Road","East Street","Edward Street","Elm Close","Elm Grove","Elm Road","Elm Street","Euclid Avenue","Fairfield Road","Farm Close","Ferry Road","Field Close","Field Lane","First Avenue","First Street","Fore Street","Forest Avenue","Forest Road","Fourth Avenue","Franklin Avenue","Franklin Road","Franklin Street","Front Street","Frontage Road","Garden Close","Garden Street","George Street","Gladstone Road","Glebe Close","Gloucester Road","Gordon Road","Gordon Street","Grand Avenue","Grange Avenue","Grange Close","Grange Road","Grant Street","Green Close","Green Lane","Green Street","Greenville Road","Greenway","Greenwood Road","Grove Lane","Grove Road","Grove Street","Hall Lane","Hall Street","Harrison Avenue","Harrison Street","Hawthorn Avenue","Hawthorn Close","Hazel Close","Hazel Grove","Heath Road","Heather Close","Henry Street","Heron Close","Hickory Street","High Road","High Street","Highfield Avenue","Highfield Close","Highfield Road","Highland Avenue","Hill Road","Hill Street","Hillside","Hillside Avenue","Hillside Close","Hillside Road","Holly Close","Honeysuckle Close","Howard Road","Howard Street","Jackson Avenue","Jackson Street","James Street","Jefferson Avenue","Jefferson Street","Johnson Street","Jubilee Close","Juniper Close","Kent Road","Kestrel Close","King Street","King's Road","Kingfisher Close","Kings Highway","Kingsway","Laburnum Grove","Lafayette Street","Lake Avenue","Lake Drive","Lake Road","Lake Street","Lancaster Road","Lansdowne Road","Larch Close","Laurel Close","Lawrence Street","Lee Street","Liberty Street","Lime Grove","Lincoln Avenue","Lincoln Highway","Lincoln Road","Lincoln Street","Locust Street","Lodge Close","Lodge Lane","London Road","Long Lane","Low Road","Madison Avenue","Madison Street","Main","Main Avenue","Main Road","Main Street","Main Street E","Main Street N","Main Street S","Main Street W","Manchester Road","Manor Close","Manor Drive","Manor Gardens","Manor Road","Manor Way","Maple Avenue","Maple Close","Maple Drive","Maple Road","Maple Street","Market Place","Market Square","Market Street","Marlborough Road","Marsh Lane","Martin Luther King Boulevard","Martin Luther King Drive","Martin Luther King Jr Boulevard","Mary Street","Mayfield Road","Meadow Close","Meadow Drive","Meadow Lane","Meadow View","Meadow Way","Memorial Drive","Middle Street","Mill Close","Mill Lane","Mill Road","Mill Street","Milton Road","Milton Street","Monroe Street","Moor Lane","Moss Lane","Mount Pleasant","Mount Street","Mulberry Street","N 1st Street","N 2nd Street","N 3rd Street","N 4th Street","N 5th Street","N 6th Street","N 7th Street","N 8th Street","N 9th Street","N Bridge Street","N Broad Street","N Broadway","N Broadway Street","N Cedar Street","N Center Street","N Central Avenue","N Chestnut Street","N Church Street","N College Street","N Court Street","N Division Street","N East Street","N Elm Street","N Franklin Street","N Front Street","N Harrison Street","N High Street","N Jackson Street","N Jefferson Street","N Lincoln Street","N Locust Street","N Main","N Main Avenue","N Main Street","N Maple Street","N Market Street","N Monroe Street","N Oak Street","N Park Street","N Pearl Street","N Pine Street","N Poplar Street","N Railroad Street","N State Street","N Union Street","N Walnut Street","N Washington Avenue","N Washington Street","N Water Street","Nelson Road","Nelson Street","New Lane","New Road","New Street","Newton Road","Nightingale Close","Norfolk Road","North Avenue","North Lane","North Road","North Street","Northfield Road","Oak Avenue","Oak Drive","Oak Lane","Oak Road","Oak Street","Oakfield Road","Oaklands","Old Lane","Old Military Road","Old Road","Old State Road","Orchard Drive","Orchard Lane","Orchard Road","Orchard Street","Oxford Road","Oxford Street","Park Avenue","Park Crescent","Park Drive","Park Lane","Park Place","Park Road","Park Street","Park View","Parkside","Pearl Street","Pennsylvania Avenue","Pine Close","Pine Grove","Pine Street","Pinfold Lane","Pleasant Street","Poplar Avenue","Poplar Close","Poplar Road","Poplar Street","Post Road","Pound Lane","Princes Street","Princess Street","Priory Close","Priory Road","Prospect Avenue","Prospect Place","Prospect Road","Prospect Street","Quarry Lane","Quarry Road","Queen's Road","Railroad Avenue","Railroad Street","Railway Street","Rectory Close","Rectory Lane","Richmond Close","Richmond Road","Ridge Road","River Road","River Street","Riverside","Riverside Avenue","Riverside Drive","Roman Road","Roman Way","Rowan Close","Russell Street","S 10th Street","S 14th Street","S 1st Avenue","S 1st Street","S 2nd Street","S 3rd Street","S 4th Street","S 5th Street","S 6th Street","S 7th Street","S 8th Street","S 9th Street","S Bridge Street","S Broad Street","S Broadway","S Broadway Street","S Center Street","S Central Avenue","S Chestnut Street","S Church Street","S College Street","S Division Street","S East Street","S Elm Street","S Franklin Street","S Front Street","S Grand Avenue","S High Street","S Jackson Street","S Jefferson Street","S Lincoln Street","S Main","S Main Avenue","S Main Street","S Maple Street","S Market Street","S Mill Street","S Monroe Street","S Oak Street","S Park Street","S Pine Street","S Railroad Street","S State Street","S Union Street","S Walnut Street","S Washington Avenue","S Washington Street","S Water Street","S West Street","Salisbury Road","Sandringham Road","Sandy Lane","School Close","School Lane","School Road","School Street","Second Avenue","Silver Street","Skyline Drive","Smith Street","Somerset Road","South Avenue","South Drive","South Road","South Street","South View","Spring Gardens","Spring Street","Springfield Close","Springfield Road","Spruce Street","St Andrew's Road","St Andrews Close","St George's Road","St John's Road","St Mary's Close","St Mary's Road","Stanley Road","Stanley Street","State Avenue","State Line Road","State Road","State Street","Station Road","Station Street","Stoney Lane","Sycamore Avenue","Sycamore Close","Sycamore Drive","Sycamore Street","Talbot Road","Tennyson Road","The Avenue","The Beeches","The Causeway","The Chase","The Coppice","The Copse","The Crescent","The Croft","The Dell","The Drive","The Fairway","The Glebe","The Grange","The Green","The Grove","The Hawthorns","The Lane","The Laurels","The Limes","The Maltings","The Meadows","The Mews","The Mount","The Oaks","The Orchard","The Oval","The Paddock","The Paddocks","The Poplars","The Ridgeway","The Ridings","The Rise","The Sidings","The Spinney","The Square","The Willows","The Woodlands","Third Avenue","Third Street","Tower Road","Trinity Road","Tudor Close","Union Avenue","Union Street","University Avenue","University Drive","Valley Road","Veterans Memorial Drive","Veterans Memorial Highway","Vicarage Close","Vicarage Lane","Vicarage Road","Victoria Place","Victoria Road","Victoria Street","Vine Street","W 10th Street","W 11th Street","W 12th Street","W 14th Street","W 1st Street","W 2nd Street","W 3rd Street","W 4th Avenue","W 4th Street","W 5th Street","W 6th Avenue","W 6th Street","W 7th Street","W 8th Street","W 9th Street","W Bridge Street","W Broad Street","W Broadway","W Broadway Avenue","W Broadway Street","W Center Street","W Central Avenue","W Chestnut Street","W Church Street","W Division Street","W Elm Street","W Franklin Street","W Front Street","W Grand Avenue","W High Street","W Jackson Street","W Jefferson Street","W Lake Street","W Main","W Main Street","W Maple Street","W Market Street","W Monroe Street","W North Street","W Oak Street","W Park Street","W Pine Street","W River Road","W South Street","W State Street","W Union Street","W Walnut Street","W Washington Avenue","W Washington Street","Walnut Close","Walnut Street","Warren Close","Warren Road","Washington Avenue","Washington Boulevard","Washington Road","Washington Street","Water Lane","Water Street","Waterloo Road","Waterside","Watery Lane","Waverley Road","Well Lane","Wellington Road","Wellington Street","West Avenue","West End","West Lane","West Road","West Street","West View","Western Avenue","Western Road","Westfield Road","Westgate","William Street","Willow Close","Willow Drive","Willow Grove","Willow Road","Willow Street","Windermere Road","Windmill Close","Windmill Lane","Windsor Avenue","Windsor Close","Windsor Drive","Wood Lane","Wood Street","Woodland Close","Woodland Road","Woodlands","Woodlands Avenue","Woodlands Close","Woodlands Road","Woodside","Woodside Road","Wren Close","Yew Tree Close","York Road","York Street"],x4e=["{{person.first_name.generic}} {{location.street_suffix}}","{{person.last_name.generic}} {{location.street_suffix}}","{{location.street_name}}"],w4e=["Alley","Avenue","Branch","Bridge","Brook","Brooks","Burg","Burgs","Bypass","Camp","Canyon","Cape","Causeway","Center","Centers","Circle","Circles","Cliff","Cliffs","Club","Common","Corner","Corners","Course","Court","Courts","Cove","Coves","Creek","Crescent","Crest","Crossing","Crossroad","Curve","Dale","Dam","Divide","Drive","Drives","Estate","Estates","Expressway","Extension","Extensions","Fall","Falls","Ferry","Field","Fields","Flat","Flats","Ford","Fords","Forest","Forge","Forges","Fork","Forks","Fort","Freeway","Garden","Gardens","Gateway","Glen","Glens","Green","Greens","Grove","Groves","Harbor","Harbors","Haven","Heights","Highway","Hill","Hills","Hollow","Inlet","Island","Islands","Isle","Junction","Junctions","Key","Keys","Knoll","Knolls","Lake","Lakes","Land","Landing","Lane","Light","Lights","Loaf","Lock","Locks","Lodge","Loop","Mall","Manor","Manors","Meadow","Meadows","Mews","Mill","Mills","Mission","Motorway","Mount","Mountain","Mountains","Neck","Orchard","Oval","Overpass","Park","Parks","Parkway","Parkways","Pass","Passage","Path","Pike","Pine","Pines","Place","Plain","Plains","Plaza","Point","Points","Port","Ports","Prairie","Radial","Ramp","Ranch","Rapid","Rapids","Rest","Ridge","Ridges","River","Road","Roads","Route","Row","Rue","Run","Shoal","Shoals","Shore","Shores","Skyway","Spring","Springs","Spur","Spurs","Square","Squares","Station","Stravenue","Stream","Street","Streets","Summit","Terrace","Throughway","Trace","Track","Trafficway","Trail","Tunnel","Turnpike","Underpass","Union","Unions","Valley","Valleys","Via","Viaduct","View","Views","Village","Villages","Ville","Vista","Walk","Walks","Wall","Way","Ways","Well","Wells"],k4e={building_number:o4e,city_name:s4e,city_pattern:l4e,city_prefix:u4e,city_suffix:c4e,continent:d4e,country:h4e,county:p4e,direction:f4e,language:m4e,postcode:g4e,secondary_address:b4e,state:y4e,state_abbr:v4e,street_address:S4e,street_name:E4e,street_pattern:x4e,street_suffix:w4e},C4e=k4e,T4e=["a","ab","abbas","abduco","abeo","abscido","absconditus","absens","absorbeo","absque","abstergo","absum","abundans","abutor","accedo","accendo","acceptus","accommodo","accusamus","accusantium","accusator","acer","acerbitas","acervus","acidus","acies","acquiro","acsi","ad","adamo","adaugeo","addo","adduco","ademptio","adeo","adeptio","adfectus","adfero","adficio","adflicto","adhaero","adhuc","adicio","adimpleo","adinventitias","adipisci","adipiscor","adiuvo","administratio","admiratio","admitto","admoneo","admoveo","adnuo","adopto","adsidue","adstringo","adsuesco","adsum","adulatio","adulescens","aduro","advenio","adversus","advoco","aedificium","aeger","aegre","aegrotatio","aegrus","aeneus","aequitas","aequus","aer","aestas","aestivus","aestus","aetas","aeternus","ager","aggero","aggredior","agnitio","agnosco","ago","ait","aiunt","alias","alienus","alii","alioqui","aliqua","aliquam","aliquid","alius","allatus","alo","alter","altus","alveus","amaritudo","ambitus","ambulo","amet","amicitia","amiculum","amissio","amita","amitto","amo","amor","amoveo","amplexus","amplitudo","amplus","ancilla","angelus","angulus","angustus","animadverto","animi","animus","annus","anser","ante","antea","antepono","antiquus","aperiam","aperio","aperte","apostolus","apparatus","appello","appono","appositus","approbo","apto","aptus","apud","aqua","ara","aranea","arbitro","arbor","arbustum","arca","arceo","arcesso","architecto","arcus","argentum","argumentum","arguo","arma","armarium","aro","ars","articulus","artificiose","arto","arx","ascisco","ascit","asper","asperiores","aspernatur","aspicio","asporto","assentator","assumenda","astrum","at","atavus","ater","atque","atqui","atrocitas","atrox","attero","attollo","attonbitus","auctor","auctus","audacia","audax","audentia","audeo","audio","auditor","aufero","aureus","aurum","aut","autem","autus","auxilium","avaritia","avarus","aveho","averto","baiulus","balbus","barba","bardus","basium","beatae","beatus","bellicus","bellum","bene","beneficium","benevolentia","benigne","bestia","bibo","bis","blandior","blanditiis","bonus","bos","brevis","cado","caecus","caelestis","caelum","calamitas","calcar","calco","calculus","callide","campana","candidus","canis","canonicus","canto","capillus","capio","capitulus","capto","caput","carbo","carcer","careo","caries","cariosus","caritas","carmen","carpo","carus","casso","caste","casus","catena","caterva","cattus","cauda","causa","caute","caveo","cavus","cedo","celebrer","celer","celo","cena","cenaculum","ceno","censura","centum","cerno","cernuus","certe","certus","cervus","cetera","charisma","chirographum","cibo","cibus","cicuta","cilicium","cimentarius","ciminatio","cinis","circumvenio","cito","civis","civitas","clam","clamo","claro","clarus","claudeo","claustrum","clementia","clibanus","coadunatio","coaegresco","coepi","coerceo","cogito","cognatus","cognomen","cogo","cohaero","cohibeo","cohors","colligo","collum","colo","color","coma","combibo","comburo","comedo","comes","cometes","comis","comitatus","commemoro","comminor","commodi","commodo","communis","comparo","compello","complectus","compono","comprehendo","comptus","conatus","concedo","concido","conculco","condico","conduco","confero","confido","conforto","confugo","congregatio","conicio","coniecto","conitor","coniuratio","conor","conqueror","conscendo","consectetur","consequatur","consequuntur","conservo","considero","conspergo","constans","consuasor","contabesco","contego","contigo","contra","conturbo","conventus","convoco","copia","copiose","cornu","corona","corporis","corpus","correptius","corrigo","corroboro","corrumpo","corrupti","coruscus","cotidie","crapula","cras","crastinus","creator","creber","crebro","credo","creo","creptio","crepusculum","cresco","creta","cribro","crinis","cruciamentum","crudelis","cruentus","crur","crustulum","crux","cubicularis","cubitum","cubo","cui","cuius","culpa","culpo","cultellus","cultura","cum","cumque","cunabula","cunae","cunctatio","cupiditas","cupiditate","cupio","cuppedia","cupressus","cur","cura","curatio","curia","curiositas","curis","curo","curriculum","currus","cursim","curso","cursus","curto","curtus","curvo","custodia","damnatio","damno","dapifer","debeo","debilito","debitis","decens","decerno","decet","decimus","decipio","decor","decretum","decumbo","dedecor","dedico","deduco","defaeco","defendo","defero","defessus","defetiscor","deficio","defleo","defluo","defungo","degenero","degero","degusto","deinde","delectatio","delectus","delego","deleniti","deleo","delibero","delicate","delinquo","deludo","demens","demergo","demitto","demo","demonstro","demoror","demulceo","demum","denego","denique","dens","denuncio","denuo","deorsum","depereo","depono","depopulo","deporto","depraedor","deprecator","deprimo","depromo","depulso","deputo","derelinquo","derideo","deripio","deserunt","desidero","desino","desipio","desolo","desparatus","despecto","dicta","dignissimos","distinctio","dolor","dolore","dolorem","doloremque","dolores","doloribus","dolorum","ducimus","ea","eaque","earum","eius","eligendi","enim","eos","error","esse","est","et","eum","eveniet","ex","excepturi","exercitationem","expedita","explicabo","facere","facilis","fuga","fugiat","fugit","harum","hic","id","illo","illum","impedit","in","incidunt","infit","inflammatio","inventore","ipsa","ipsam","ipsum","iste","itaque","iure","iusto","labore","laboriosam","laborum","laudantium","libero","magnam","magni","maiores","maxime","minima","minus","modi","molestiae","molestias","mollitia","nam","natus","necessitatibus","nemo","neque","nesciunt","nihil","nisi","nobis","non","nostrum","nulla","numquam","occaecati","ocer","odio","odit","officia","officiis","omnis","optio","paens","pariatur","patior","patria","patrocinor","patruus","pauci","paulatim","pauper","pax","peccatus","pecco","pecto","pectus","pecus","peior","pel","perferendis","perspiciatis","placeat","porro","possimus","praesentium","provident","quae","quaerat","quam","quas","quasi","qui","quia","quibusdam","quidem","quis","quisquam","quo","quod","quos","ratione","recusandae","reiciendis","rem","repellat","repellendus","reprehenderit","repudiandae","rerum","saepe","sapiente","sed","sequi","similique","sint","sit","socius","sodalitas","sol","soleo","solio","solitudo","solium","sollers","sollicito","solum","solus","soluta","solutio","solvo","somniculosus","somnus","sonitus","sono","sophismata","sopor","sordeo","sortitus","spargo","speciosus","spectaculum","speculum","sperno","spero","spes","spiculum","spiritus","spoliatio","sponte","stabilis","statim","statua","stella","stillicidium","stipes","stips","sto","strenuus","strues","studio","stultus","suadeo","suasoria","sub","subito","subiungo","sublime","subnecto","subseco","substantia","subvenio","succedo","succurro","sufficio","suffoco","suffragium","suggero","sui","sulum","sum","summa","summisse","summopere","sumo","sumptus","sunt","supellex","super","suppellex","supplanto","suppono","supra","surculus","surgo","sursum","suscipio","suscipit","suspendo","sustineo","suus","synagoga","tabella","tabernus","tabesco","tabgo","tabula","taceo","tactus","taedium","talio","talis","talus","tam","tamdiu","tamen","tametsi","tamisium","tamquam","tandem","tantillus","tantum","tardus","tego","temeritas","temperantia","templum","tempora","tempore","temporibus","temptatio","tempus","tenax","tendo","teneo","tener","tenetur","tenuis","tenus","tepesco","tepidus","ter","terebro","teres","terga","tergeo","tergiversatio","tergo","tergum","termes","terminatio","tero","terra","terreo","territo","terror","tersus","tertius","testimonium","texo","textilis","textor","textus","thalassinus","theatrum","theca","thema","theologus","thermae","thesaurus","thesis","thorax","thymbra","thymum","tibi","timidus","timor","titulus","tolero","tollo","tondeo","tonsor","torqueo","torrens","tot","totam","totidem","toties","totus","tracto","trado","traho","trans","tredecim","tremo","trepide","tres","tribuo","tricesimus","triduana","tripudio","tristis","triumphus","trucido","truculenter","tubineus","tui","tum","tumultus","tunc","turba","turbo","turpis","tutamen","tutis","tyrannus","uberrime","ubi","ulciscor","ullam","ullus","ulterius","ultio","ultra","umbra","umerus","umquam","una","unde","undique","universe","unus","urbanus","urbs","uredo","usitas","usque","ustilo","ustulo","usus","ut","uter","uterque","utilis","utique","utor","utpote","utrimque","utroque","utrum","uxor","vaco","vacuus","vado","vae","valde","valens","valeo","valetudo","validus","vallum","vapulus","varietas","varius","vehemens","vel","velit","velociter","velum","velut","venia","veniam","venio","ventito","ventosus","ventus","venustas","ver","verbera","verbum","vere","verecundia","vereor","vergo","veritas","veritatis","vero","versus","verto","verumtamen","verus","vesco","vesica","vesper","vespillo","vester","vestigium","vestrum","vetus","via","vicinus","vicissitudo","victoria","victus","videlicet","video","viduo","vigilo","vigor","vilicus","vilis","vilitas","villa","vinco","vinculum","vindico","vinitor","vinum","vir","virga","virgo","viridis","viriliter","virtus","vis","viscus","vita","vitae","vitiosus","vitium","vito","vivo","vix","vobis","vociferor","voco","volaticus","volo","volubilis","voluntarius","volup","voluptas","voluptate","voluptatem","voluptates","voluptatibus","voluptatum","volutabrum","volva","vomer","vomica","vomito","vorago","vorax","voro","vos","votum","voveo","vox","vulariter","vulgaris","vulgivagus","vulgo","vulgus","vulnero","vulnus","vulpes","vulticulus","xiphias"],A4e={word:T4e},F4e=A4e,_4e={title:"English",code:"en",language:"en",endonym:"English",dir:"ltr",script:"Latn"},R4e=_4e,D4e=['"Awaken, My Love!"',"(What's The Story) Morning Glory?","- Tragedy +","13 Reasons Why (Season 3)","21st Century Breakdown","30 De Febrero","432 Hz Deep Healing","5-Star","528 Hz Meditation Music","54+1","8 Mile","808s & Heartbreak","9 To 5 And Odd Jobs","A Beautiful Lie","A Day At The Races","A Day Without Rain","A Fever You Can't Sweat Out","A Gangsta's Pain","A Gift & A Curse","A Hard Day's Night","A Head Full Of Dreams","A Kind Of Magic","A Million Ways To Murder","A Moment Apart","A Song For Every Moon","A Thousand Suns","A Winter Romance","ABBA","AI YoungBoy","AJ Tracey","Act One","After Hours","Agent Provocateur","All About You","All I Know So Far: Setlist","All Or Nothing","All Out","All Over The Place","All Stand Together","All The Lost Souls","All The Things I Never Said","All Things Must Pass","Alleen","Alright, Still","Alta Suciedad","America","American Heartbreak","American Teen","And Justice For None","Animal Songs","Another Friday Night","Anything Goes","Ao Vivo Em São Paulo","Ao Vivo No Ibirapuera","Apricot Princess","Aqui E Agora (Ao Vivo)","Arcane League Of Legends","Ardipithecus","Aretha Now","Around The Fur","Arrival","Artist 2.0","As She Pleases","Ascend","Ashlyn","Astro Lounge","At Night, Alone.","At. Long. Last. ASAP","Atlas","Audioslave","Aura","Austin","Awake","Away From The Sun","Ayayay!","Baby On Baby","Back For Everything","Back From The Edge","Back In Black","Back To Black","Back To The Game","Bad","Bahía Ducati","Baila","Barbie The Album","Battleground","Bayou Country","Bcos U Will Never B Free","Be","Be Here Now","Beautiful Mind","Beautiful Thugger Girls","Beautiful Trauma","Beauty And The Beast","Beggars Banquet","Being Funny In A Foreign Language","Berlin Lebt","Berry Is On Top","Best White Noise For Baby Sleep - Loopable With No Fade","Big Baby DRAM","Bigger, Better, Faster, More!","Billy Talent II","Black Star Elephant","Blackout","Blank Face LP","Bleach","Blizzard Of Ozz","Blonde","Blood Sugar Sex Magik","Bloom","Blowin' Your Mind!","Blu Celeste","Blue","Blue Banisters","Blue Hawaii","Blue Neighbourhood","Bluebird Days","Bobby Tarantino","Bobby Tarantino II","Bon Iver","Born Pink","Born To Run","Brand New Eyes","Break The Cycle","Breakfast In America","Breakthrough","Brett Young","Bridge Over Troubled Water","Bright: The Album","Brol","Buds","Buena Vista Social Club","Built On Glass","Bury Me At Makeout Creek","Busyhead","By The Way","CB6","CNCO","California Sunrise","Californication","Call Me Irresponsible","Calm","Camino Palmero","Camp","Caracal","Carbon Fiber Hits","Carnival","Carry On","Cartel De Santa","Certified Lover Boy","Chaaama","Chama Meu Nome","Chapter 1: Snake Oil","Chapter 2: Swamp Savant","Chapter One","Charlie's Angels","Cherry Bomb","Chief","Chocolate Factory","Chosen","Chris Brown","Christina Aguilera","Chromatica","Church","City Of Evil","Clandestino","Clouds","Coco","Collision Course","Colour Vision","Combat Rock","Come Around Sundown","Come Away With Me","Come Home The Kids Miss You","Come What(ever) May","Commando","Common Sense","Communion","Conditions","Confident","Confrontation","Control The Streets, Volume 2","Corinne Bailey Rae","Costello Music","Cottonwood","Covers, Vol. 2","Cozy Tapes Vol. 2: Too Cozy","Crash Talk","Crazy Love","Crazysexycool","Crowded House","Cruisin' With Junior H","Culture","Current Mood","DS2","Dale","Danger Days: The True Lives Of The Fabulous Killjoys","Dangerous Woman","Dangerous: The Double Album","Dark Horse","Day69","Daydream","De Fiesta","De Viaje","DeAnn","Death Race For Love","Delirium","Delta","Demidevil","Depression Cherry","Descendants","Desgenerados Mixtape","Destin","Destiny Fulfilled","Desvelado","Detroit 2","Dex Meets Dexter","Dharma","Die A Legend","Different World","Dig Your Roots","Digital Druglord","Dirt","Disclaimer I / II","Discovery","Disraeli Gears","Disumano","Dizzy Up The Girl","Don't Play That Song","Donda","Donde Quiero Estar","Doo-Wops & Hooligans","Down The Way","Dr. Feelgood","Dream Your Life Away","Dreaming Out Loud","Drip Harder","Drive","Drones","Dropped Outta College","Drowning","Dua Warna Cinta","Dulce Beat","Dusty In Memphis","Dutty Rock","Dying To Live","ENR","East Atlanta Love Letter","Editorial","Edna","El Abayarde","El Amor En Los Tiempos Del Perreo","El Camino","El Comienzo","El Dorado","El Karma","El Mal Querer","El Malo","El Trabajo Es La Suerte","El Viaje De Copperpot","Electric Ladyland","Emotion","En Tus Planes","Endless Summer Vacation","Enter The Wu-Tang (36 Chambers)","Equals (=)","Estrella","Euphoria","Europop","Evermore","Every Kingdom","Everyday Life","Evolve","Expectations","Face Yourself","Facelift","Fallin'","Fancy You","Fantasía","Favourite Worst Nightmare","Fear Of The Dark","Fearless","Feel Something","Feels Like Home","Femme Fatale","Ferxxocalipsis","Fifty Shades Darker","Fifty Shades Freed","Fifty Shades Of Grey","Final (Vol.1)","Finding Beauty In Negative Spaces","Fine Line","First Impressions Of Earth","First Steps","Five Seconds Flat","Folklore","For Emma, Forever Ago","Forajido EP 1","Forever","Forever Young","Formula Of Love: O+T=<3","Free 6lack","Freudian","Frozen II","Full Moon Fever","Funhouse","Funk Wav Bounces Vol.1","Future History","FutureSex/LoveSounds","Fuzzybrain","Gallery","Gangsta's Paradise","Gemini","Gemini Rights","Generationwhy","Get A Grip","Get Up","Gettin' Old","Girl","Gladiator","Glisten","Globalization","Gloria","Glory Days","God's Project","Gold Skies","Golden","Good Evening","Good Thing","Goodbye Yellow Brick Road","Gossip Columns","Got Your Six","Graceland","Graduation","Grand Champ","Grandson, Vol. 1","Green River","Guerra","Ha*Ash Primera Fila - Hecho Realidad","Haiz","Hamilton","Happy Endings","Harry Styles","Hasta La Raíz","Hatful Of Hollow","Head In The Clouds","Heard It In A Past Life","Heart Shaped World","Heartbeat City","Heartbreak On A Full Moon / Cuffing Season - 12 Days Of Christmas","Heaven Or Hell","Heaven knows","Hellbilly Deluxe","Hellboy","Help!","Her Loss","Here Comes The Cowboy","Hey World","High School Musical","High Tide In The Snake's Nest","Historias De Un Capricornio","Hndrxx","Hombres G (Devuélveme A Mi Chica)","Homerun","Homework","Hot Fuss","Hot Pink","Hot Sauce / Hello Future","Hot Space","Hotel Diablo","Houses Of The Holy","How Big, How Blue, How Beautiful","How I'm Feeling","How To Be Human","How To Save A Life","How To: Friend, Love, Freefall","Hozier","Human","Huncho Jack, Jack Huncho","Hunter Hayes","Hysteria","I Am...Sasha Fierce","I Can't Handle Change","I Met You When I Was 18. (The Playlist)","I Never Liked You","I Never Loved A Man The Way I Love You","I See You","I Think You Think Too Much Of Me","I Used To Know Her","I Used To Think I Could Fly","I'm Comin' Over","Ich & Keine Maske","If You Can Believe Your Eyes & Ears","Il Ballo Della Vita","Ill Communication","Imagination & The Misfit Kid","Imagine","Immortalized","In A Perfect World...","In Colour","In My Own Words","In Rainbows","In Return","In The Lonely Hour","Infest","Innuendo","Inter Shibuya - La Mafia","Interstellar","Is This It","It Was Written","It's Not Me, It's You","It's Only Me","Ivory","JackBoys","Jamie","Jazz","Jibrail & Iblis","Jordi","Jordin Sparks","Jose","Just As I Am","Just Cause Y'all Waited 2","Just Like You","Justified","K-12 / After School","K.I.D.S.","K.O.","K.O.B. Live","KG0516","KOD","Kane Brown","Kid A","Kid Krow","Kids See Ghosts","Kids in Love","Kinks (You Really Got Me)","Know-It-All","Konvicted","Kring","LANY","LM5","La Criatura","La Flaca","La Melodia De La Calle","La Revolucion","Lady Lady","Lady Wood","Langit Mong Bughaw","Las Que No Iban A Salir","Last Day Of Summer","Last Year Was Complicated","Layers","Layover","Lazarus","Led Zeppelin","Left Of The Middle","Leftoverture","Legends Never Die","Let's Skip To The Wedding","Let's Talk About Love","Licensed To Ill","Life In Cartoon Motion","Life Thru A Lens","Lifelines","Like..?","Lil Big Pac","Lil Boat","Lil Boat 2","Lil Boat 3.5","Lil Kiwi","Lil Pump","Limon Y Sal","Listen Without Prejudice","Little Voice","Live On Red Barn Radio I & II","Lo Que Andábamos Buscando","Lofi Fruits Music 2021","London Calling","Los Campeones Del Pueblo","Los Extraterrestres","Los Favoritos 2","Lost","Lost In Love","Loud","Love Sick","Love Story","Love Stuff","Love Yourself: Tear","Lover","Luca Brasi 2: Gangsta Grillz","Lust For Life","Luv Is Rage","M!ssundaztood","Ma Fleur","Made In Lagos","Mafia Bidness","Magazines Or Novels","Mainstream Sellout","Majestic","Make It Big","Make Yourself","Making Mirrors","Mamma Mia! Here We Go Again","Man Of The Woods","Manic","Me And My Gang","Meduza","Meet The Orphans","Meet The Woo","Melim","Mellon Collie And The Infinite Sadness","Melly vs. Melvin","Memories...Do Not Open","Menagerie","Midnights","Minecraft - Volume Alpha","Minutes To Midnight","Mix Pa Llorar En Tu Cuarto","Modo Avión","Monkey Business","Mono.","Montana","Montevallo","Moosetape","Morning View","Motivan2","Moving Pictures","Mr. Davis","Mr. Misunderstood","Mulan","Mura Masa","Music From The Edge Of Heaven","Music Of The Sun","My House","My Kinda Party","My Krazy Life","My Liver Will Handle What My Heart Can't","My Moment","My Own Lane","My Turn","My Worlds","Na Praia (Ao Vivo)","Nakamura","Nation Of Two","Navegando","Need You Now","Neon Future III","Neotheater","Never Trust A Happy Song","New English","News Of The World","Nicole","Night & Day","Nimmerland","Nimrod","Nine Track Mind","No Angel","No Me Pidas Perdón","No More Drama","No Protection","No Strings Attached","No Time To Die","Nobody Is Listening","Non Stop Erotic Cabaret","Non-Fiction","Northsbest","Nostalgia","Nostalgia, Ultra","Notes On A Conditional Form","Now Or Never","O Embaixador (Ao Vivo)","O My Heart","OK Computer","Ocean","Ocean Avenue","Ocean Eyes","Odisea","Oh My My","Oh, What A Life","On The 6","One In A Million","One More Light","One Of These Nights","Open Up And Say...Ahh!","Ordinary Man","Origins","Out Of The Blue","Over It","OzuTochi","PTSD","Pa Las Baby's Y Belikeada","Pa Que Hablen","Pa' Luego Es Tarde","Pa' Otro La 'O","Pablo Honey","Pain Is Love","Pain Is Temporary","Painting Pictures","Palmen Aus Plastik 2","Para Mi Ex","Para Siempre","Partners In Crime","Pawn Shop","Pegasus / Neon Shark VS Pegasus","Pet Sounds","Piece By Piece","Pier Pressure","Pineapple Sunrise","Piseiro 2020 Ao Vivo","Planet Pit","Plans","Play Deep","Playa Saturno","Por Primera Vez","Por Vida","Positions","Post Human: Survival Horror","Poster Girl","Prazer, Eu Sou Ferrugem (Ao Vivo)","Pretty Girls Like Trap Music","Pretty. Odd.","Prince Royce","Prisma","Prometo","Providence","Puberty 2","Punisher","Purgatory","Purple Rain","Que Bendición","Queen Of The Clouds","Quiero Volver","R&G (Rhythm & Gangsta): The Masterpiece","Raise!","Ransom 2","Rapunzel","Rare","Re Mida","Ready To Die","Realer","Rebelde","Reclassified","Recovery","Recuerden Mi Estilo","Reggatta De Blanc","Regulate… G Funk Era","Reik","Reise, Reise","Relapse","Relaxing Piano Lullabies And Natural Sleep Aid For Baby Sleep Music","Religiously. The Album.","Replay","Results May Vary","Revenge","Revolve","Revolver","Ricky Martin","Rien 100 Rien","Ripcord","Rise And Fall, Rage And Grace","Rise Of An Empire","Robin Hood: Prince Of Thieves","Rock N Roll Jesus","Romance","Romances","Ronan","Royal Blood","Rumours","Sad Boyz 4 Life II","San Lucas","Santana World","Saturation III","Sauce Boyz","Savage Mode","Saxobeats","Scarlet","Schwarzes Herz","Seal The Deal & Let's Boogie","Section.80","Segundo Romance","Sehnsucht","Shake The Snow Globe","Shang-Chi And The Legend Of The Ten Rings: The Album","Sheer Heart Attack","Shiesty Season","Shock Value","Shoot For The Stars, Aim For The Moon","Signed Sealed And Delivered","Signos","Silent Alarm","Simplemente Gracias","Sin Bandera","Sing Me A Lullaby, My Sweet Temptation","Sinner","Sirio","Sit Still, Look Pretty","Skin","Slowhand","Smash","Smithereens","Snow Cougar","Social Cues","Some Girls","Song Hits From Holiday Inn","Songs For Dads","Songs For The Deaf","Songs For You, Truths For Me","Songs In The Key Of Life","Souled Out","Sounds Of Silence","Soy Como Quiero Ser","Speak Now","Speak Your Mind","Speakerboxxx/The Love Below","Spider-Man: Into The Spider-Verse","Split Decision","Square Up","SremmLife","Starboy","Stay +","Stay Dangerous","Staying At Tamara's","Steppenwolf","Stick Season","Still Bill","Straight Outta Compton","Strange Trails","Stronger","Suavemente","Sublime","Suck It and See","Sucker","Sueños","Sugar","Summer Forever","Summer,","Sunset Season","Sunshine On Leith","Surfer Rosa","Sweet Talker","SweetSexySavage","System Of A Down","TA13OO","Talk That Talk","Talking Heads: 77","Tangled Up","Tango In The Night","Taxi Driver","Taylor Swift","Tell Me It's Real","Ten","Ten Summoner's Tales","Terra Sem Cep (Ao Vivo)","Terral","Testing","Tha Carter III","Thank Me Later","That's Christmas To Me","The Academy","The Adventures Of Bobby Ray","The Album","The Andy Williams Christmas Album","The Aviary","The Balcony","The Battle Of Los Angeles","The Beatles (White Album)","The Beginning","The Better Life","The Big Day","The Book","The Breakfast Club","The Cars","The Colour And The Shape","The Death Of Peace Of Mind","The Diary Of Alicia Keys","The Documentary","The Emancipation Of Mimi","The Eminem Show","The End Of Everything","The Final Countdown","The Forever Story","The Foundation","The Goat","The Golden Child","The Good Parts","The Greatest Showman: Reimagined","The Green Trip","The Hardest Love","The Head And The Heart","The Human Condition","The Infamous","The Lady Killer","The Last Don II","The Lion King","The Lockdown Sessions","The London Sessions","The Lord Of The Rings: The Fellowship Of The Ring","The Lost Boy","The Magic Of Christmas / The Christmas Song","The Marshall Mathers LP","The Martin Garrix Collection","The Melodic Blue","The Mockingbird & The Crow","The Pains Of Growing","The Papercut Chronicles","The Perfect Luv Tape","The Pinkprint","The Predator","The Queen Is Dead","The ReVe Festival: Finale","The Rise And Fall Of Ziggy Stardust And The Spiders From Mars","The Rising Tied","The River","The Stone Roses","The Story Of Us","The Stranger","The Sufferer & The Witness","The Sun's Tirade","The Temptations Sing Smokey","The Time Of Our Lives","The Way It Is","The Wonderful World Of Sam Cooke","The Writing's On The Wall","The Young And The Hopeless","Therapy","Therapy Session","There Is More (Live)","There Is Nothing Left To Lose","These Things Happen","Third Eye Blind","This Is Me...Then","This Unruly Mess I've Made","Threat to Survival","Thrill Of The Chase","Time","Timelezz","To Let A Good Thing Die","To Pimp A Butterfly","Toast To Our Differences","Todos Os Cantos, Vol. 1 (Ao Vivo)","Too Hard","Torches X","Total Xanarchy","Toto IV","Toulouse Street","Tourist History","Toxicity","Tragic Kingdom","Tranquility Base Hotel & Casino","Traumazine","Traveler","Tres Hombres","Trip At Knight","Tron: Legacy","True Blue","True Colors","Trustfall","Tu Veneno Mortal","Tudo Em Paz","Ubuntu","Ugly Is Beautiful","Ultra 2021","Una Mattina","Unbroken","Uncovered","Under Pressure","Unsponsored Content","Unstoppable","Unwritten","Urban Flora","Urban Hymns","Use Your Illusion I","Veneer","Versions Of Me","Vibes","Vice Versa","Vices & Virtues","Victory","Vida","Viejo Marihuano","Visualízate","Walk Away","Walk Me Home...","Watch The Throne","Wave","We Broke The Rules","We Love You Tecca","We Love You Tecca 2","Weezer (Green Album)","Welcome To The Madhouse","Westlife","What A Time To Be Alive","What Do You Think About The Car?","What Is Love?","What Makes You Country","What Separates Me From You","What You See Is What You Get / What You See Ain't Always What You Get","When It's Dark Out","When We All Fall Asleep, Where Do We Go?","Where The Light Is","While The World Was Burning","White Pony","Whitney","Who Really Cares","Who You Are","Who's Next","Wide Open","Wilder Mind","Wildfire","Willy And The Poor Boys","Wings / You Never Walk Alone","Wish","Wish You Were Here","Without Warning","Wonder","X&Y","XOXO","Y Que Quede Claro","YBN: The Mixtape","Yo Creo","You Will Regret","Youngblood","Younger Now","Youth"],N4e=["$NOT","$uicideboy$","(G)I-DLE","*NSYNC","2 Chainz","21 Savage","6LACK","? & The Mysterians","A Boogie Wit da Hoodie","A Taste of Honey","A Tribe Called Quest","A-Ha","ABBA","AC/DC","AJ Tracey","ATEEZ","Ace of Base","Adele","Ado","Aerosmith","Agust D","Aitana","Al Dexter & his Troopers","Al Green","Al Jolson","Al Martino","Alan Jackson","Alannah Myles","Alec Benjamin","Alejandro Sanz","Alesso","Alfredo Olivas","Ali Gatie","Alice In Chains","Alina Baraz","All Time Low","All-4-One","All-American Rejects","Alok","America","American Quartet","Amii Stewart","Amitabh Bhattacharya","Ana Castela","Anderson .Paak","Andy Grammer","Angus & Julia Stone","Anirudh Ravichander","Anita Ward","Anitta","Anton Karas","Anuel AA","Arcade Fire","Archie Bell & The Drells","Archies","Aretha Franklin","Arizona Zervas","Armin van Buuren","Arthur Conley","Artie Shaw","Asake","Asees Kaur","Association","Atif Aslam","Audioslave","Aventura","Avril Lavigne","Aya Nakamura","B J Thomas","B.o.B","BLACKPINK","BONES","BROCKHAMPTON","BTS","Baby Keem","Bachman-Turner Overdrive","Backstreet Boys","Bad Bunny","Badshah","Bailey Zimmerman","Banda El Recodo","Barbra Streisand","Barry White","Bazzi","Bebe Rexha","Becky G","Becky Hill","Bee Gees","Ben Bernie","Ben Howard","Ben Selvin","Berlin","Bessie Smith","Bethel Music","Bette Midler","Beyonce","Bibi Blocksberg","Bibi und Tina","BigXthaPlug","Bill Doggett","Bill Haley & his Comets","Bill Withers","Billy Davis Jr","Billy Joel","Billy Paul","Billy Preston","Billy Swan","Birdy","Bizarrap","Blake Shelton","Blur","Bob Marley & The Wailers","Bob Seger","Bobby Darin","Bobby Lewis","Bobby McFerrin","Bobby Vinton","Boney M.","Bonez MC","Bonnie Tyler","Booba","Boston","BoyWithUke","Boyce Avenue","Bradley Cooper","Bread","Brent Faiyaz","Brett Young","Bring Me The Horizon","Britney Spears","Brooks & Dunn","Bruce Channel","Bruno & Marrone","Bryan Adams","Bryce Vine","Buddy Holly","Burna Boy","C. Tangana","CKay","CRO","Camilo","Capital Bra","Captain & Tennille","Cardi B","Carin Leon","Carlos Vives","Carly Simon","Carpenters","Cavetown","Celine Dion","Central Cee","Chaka Khan","Champs","Charlie Rich","Chayanne","Cheat Codes","Cher","Chic","Chicago","Chris Brown","Chris Isaak","Chris Young","Christina Aguilera","Christina Perri","Christopher Cross","Chuck Berry","Ciara","Cigarettes After Sex","Cliff Edwards (Ukelele Ike)","Cody Johnson","Colbie Caillat","Colby O'Donis","Cole Swindell","Coleman Hawkins","Contours","Coolio","Count Basie","Cris Mj","Culture Club","Cyndi Lauper","D-Block Europe","DAY6","DJ Khaled","DJ Luian","DJ Nelson","DMX","DNCE","DaVido","Dadju","Daft Punk","Dan + Shay","Daniel Powter","Danny Ocean","Darius Rucker","Dave","David Bowie","David Guetta","Daya","Dean Martin","Deee-Lite","Deep Purple","Deftones","Demi Lovato","Dennis Lloyd","Denzel Curry","Dermot Kennedy","Desiigner","Devo","Dewa 19","Dexys Midnight Runners","Diddy","Dido","Die drei !!!","Diego & Victor Hugo","Diljit Dosanjh","Dimitri Vegas & Like Mike","Dinah Shore","Dionne Warwick","Dire Straits","Disclosure","Dixie Cups","Doja Cat","Dolly Parton","Don Diablo","Don Henley","Don McLean","Don Omar","Donna Summer","Donovan","Dr. Dre","Drake","Dreamville","Dua Lipa","EMF","ENHYPEN","Earth, Wind & Fire","Ed Sheeran","Eddie Cantor","Eddie Cochran","Eddy Howard","Edgar Winter Group","Edwin Hawkins Singers","Edwin Starr","El Alfa","Eladio Carrion","Electric Light Orchestra","Elevation Worship","Ella Henderson","Ellie Goulding","Elton John","Elvis Presley","Empire of the Sun","En Vogue","Enrique Iglesias","Eslabon Armado","Ethel Waters","Etta James","Evanescence","Exile","Extreme","Faith Hill","Fall Out Boy","Fanny Brice","Farruko","Fats Domino","Fats Waller","Feid","Felix Jaehn","Fergie","Fetty Wap","Fiersa Besari","Fifth Harmony","Fine Young Cannibals","Five Finger Death Punch","Fleetwood Mac","Flo-Rida","Florence + The Machine","Flume","Foo Fighters","Foreigner","Foster The People","Four Aces","Frank Ocean","Frank Sinatra","Frankie Avalon","Frankie Valli","Fred Astaire","Freda Payne","Freddie Dredd","Freddy Fender","French Montana","Fuerza Regida","Fujii Kaze","Future","G-Eazy","Garfunkel and Oates","Gary Lewis & The Playboys","Gary Numan","Gene Autry","Gene Chandler","Gene Vincent","George Michael","George Strait","Gera MX","Ghost","Ghostemane","Gigi D'Agostino","Gladys Knight & The Pips","Glass Animals","Glee Cast","Gloria Gaynor","Godsmack","Gorillaz","Gotye","Grand Funk Railroad","Green Day","Grouplove","Grupo Firme","Grupo Marca Registrada","Gryffin","Gucci Mane","Guess Who","Gunna","Gusttavo Lima","Guy Mitchell","Gwen Stefani","Gzuz","H.E.R.","HARDY","Hailee Steinfeld","Halsey","Hans Zimmer","Harris Jayaraj","Harry Chapin","Harry James","Harry Nilsson","Harry Styles","Hayley Williams","Herb Alpert","Herman's Hermits","Hillsong UNITED","Hillsong Worship","Hollywood Undead","Honey Cone","Hoobastank","Hues Corporation","I Prevail","ITZY","IVE","Ice Cube","Ice Spice","Iggy Azalea","Imagine Dragons","Incubus","Internet Money","Isaac Hayes","J Geils Band","J. Cole","JAY-Z","JJ Lin","JP Saxe","JVKE","Jack Harlow","Jack Johnson","Jackie Wilson","Jacquees","James Arthur","James Brown","James TW","James Taylor","Jamie Foxx","Janet Jackson","Janis Joplin","Jason Aldean","Jason Mraz","Jay Chou","Jay Sean","Jay Wheeler","Jaymes Young","Jean Knight","Jeezy","Jennifer Lopez","Jennifer Warnes","Jeremih","Jeremy Zucker","Jerry Lee Lewis","Jerry Murad's Harmonicats","Jess Glynne","Jessie J","Jewel","Jimi Hendrix","Jimin","Jimmie Rodgers","Jimmy Dean","Jo Stafford","Joan Jett & The Blackhearts","Joao Gilberto","Joel Corry","John Fred & The Playboy Band","John Legend","John Mayer","John Williams","Johnnie Ray","Johnnie Taylor","Johnny Cash","Johnny Horton","Johnny Mathis","Johnny Mercer","Johnny Nash","Joji","Jon Bellion","Jonas Blue","Jonas Brothers","Joni James","Jorja Smith","Juan Gabriel","Juan Luis Guerra 4.40","Juanes","Juice Newton","Julia Michaels","Justin Bieber","Justin Quiles","KALEO","KAROL G","KAYTRANADA","KK","KSI","KYLE","Kacey Musgraves","Kane Brown","Kanye West","Karan Aujla","Kate Smith","Katy Perry","Kay Kyser","Ke$ha","Kehlani","Kelly Clarkson","Kenny Chesney","Kenny Loggins","Kenny Rogers","Kenshi Yonezu","Kenya Grace","Kevin Gates","Key Glock","Khalid","Kim Carnes","Kim Petras","Kimbra","Kina","King Gnu","Kings of Leon","Kingsmen","Kitty Kallen","Kodak Black","Kodaline","Kollegah","Kool & The Gang","Kungs","Kygo","Kylie Minogue","LE SSERAFIM","LISA","LMFAO","LUDMILLA","La Adictiva Banda San José de Mesillas","La Oreja de Van Gogh","Labrinth","Lady Antebellum","Lady GaGa","Lainey Wilson","Lana Del Rey","Latto","Lauryn Hill","Lauv","League of Legends","Lee Brice","Leon Bridges","Leona Lewis","Lesley Gore","Leslie Odom Jr.","Liam Payne","Lifehouse","Lil Baby","Lil Dicky","Lil Durk","Lil Mosey","Lil Nas X","Lil Pump","Lil Skies","Lil Tjay","Lil Uzi Vert","Lil Yachty","Lil' Kim","Lil' Wayne","Lin-Manuel Miranda","Linkin Park","Lionel Richie","Lipps Inc","Lisa Loeb","Little Peggy March","Little Richard","Lofi Fruits Music","Lord Huron","Los Del Rio","Los Dos Carnales","Los Lobos","Los Temerarios","Los Tigres Del Norte","Los Tucanes De Tijuana","Lou Reed","Loud Luxury","Louis Jordan","Louis Tomlinson","Love Unlimited","Lovin' Spoonful","Luan Santana","Luciano","Luis Miguel","Luis R Conriquez","Lulu","Lunay","Lupe Fiasco","M","MAX","MC Hammer","MC Ryan SP","MKTO","Mabel","Machine Gun Kelly","Madison Beer","Madonna","Mahalini","Major Lazer","Mambo Kingz","Maneskin","Marco Antonio Solís","Margaret Whiting","Maria Becerra","Mario","Mario Lanza","Mark Ronson","Maroon 5","Marshmello","Martin Garrix","Mary Ford","Mary J Blige","Mary J. Blige","Mary Wells","Matoma","Mau y Ricky","Meek Mill","Megadeth","Melanie","Melanie Martinez","Melendi","Men At Work","Metro Boomin","Michael Bublé","Michael Jackson","Michael McDonald","Michael Sembello","Miguel","Mike Posner","Miley Cyrus","Milky Chance","Minnie Riperton","Miracle Tones","Miranda Lambert","Mohit Chauhan","Mon Laferte","Moneybagg Yo","Monsta X","Mora","Morad","Morat","Mother Mother","Motley Crue","Ms. Lauryn Hill","Mumford & Sons","Muse","Mya","Myke Towers","NCT 127","NCT DREAM","NEFFEX","Nadin Amizah","Nancy Sinatra","Nat King Cole","Nate Smith","Natti Natasha","Nayer","Neil Diamond","Neil Sedaka","Nekfeu","Nelly","New Vaudeville Band","Next","Nickelback","Nicki Minaj","Nicki Nicole","Nicky Jam","Nina Simone","Ninho","Nipsey Hussle","Nirvana","Niska","No Doubt","Norah Jones","Normani","OMI","ONE OK ROCK","Oasis","Official HIGE DANdism","Offset","Old Dominion","Oliver Heldens","Olivia Rodrigo","Omah Lay","One Direction","Otis Redding","OutKast","Owl City","P Diddy","P!nk","PLK","PNL","Pamungkas","Passenger","Pat Boone","Patsy Cline","Patti LaBelle","Patti Page","Paul & Paula","Paul Revere & the Raiders","Paul Robeson","Paul Russell","Paul Whiteman","Paula Abdul","Peaches & Herb","Pearl Jam","Pee Wee Hunt","Pee Wee King","Pentatonix","Percy Faith","Percy Sledge","Peso Pluma","Peter Cetera","Peter Gabriel","Peter, Paul & Mary","Pharrell Williams","Pierce The Veil","Pineapple StormTv","Pink Floyd","Pink Sweat$","Piso 21","Pitbull","Plan B","Player","Polo G","Pop Smoke","Portugal. The Man","Pouya","Prince","Prince Royce","Pusha T","Quality Control","Queen","Quinn XCII","R. Kelly","RAF Camora","RAYE","REM","REO Speedwagon","Radiohead","Rag'n'Bone Man","Rage Against The Machine","Rahat Fateh Ali Khan","Rainbow Kitten Surprise","Rammstein","Rauw Alejandro","Ray Charles","Ray Parker Jr","Ray Stevens","Red Foley","Red Hot Chili Peppers","Red Velvet","Regard","Regina Belle","Reik","Rels B","Rema","Ricardo Arjona","Rich The Kid","Rick Astley","Rick Dees & his Cast of Idiots","Rick Ross","Rick Springfield","Ricky Martin","Ricky Nelson","Rihanna","Rita Ora","Ritchie Valens","Rizky Febian","Rob Thomas","Roberta Flack","Robin Schulz","Robin Thicke","Rod Stewart","Rod Wave","Roddy Ricch","Roger Miller","Romeo Santos","Rosemary Clooney","Roxette","Roy Acuff","Roy Orbison","Rudimental","Ruel","Ruth B.","Ryan Lewis","SCH","SEVENTEEN","SWV","Sabaton","Sabrina Carpenter","Sachet Tandon","Sachin-Jigar","Sade","Sam Cooke","Sam Feldt","Sam Hunt","Sam Smith","Sam The Sham & The Pharaohs","Sammy Davis Jr","Sammy Kaye","Santana","Sasha Alex Sloan","Savage Garden","Saweetie","Scorpions","Sean Kingston","Sean Paul","Sebastian Yatra","Sech","Seeb","Sezen Aksu","Sfera Ebbasta","Shaggy","Shania Twain","Shawn Mendes","Sheena Easton","Shinedown","Shubh","Sia","Sid Sriram","Sidhu Moose Wala","Silk","Silver Convention","Simon & Garfunkel","Sinead O'Connor","Sir Mix-a-Lot","Sister Sledge","Ski Mask The Slump God","Skillet","Skrillex","Sleeping At Last","Smokey Robinson","Snoop Dogg","Snow Patrol","Soda Stereo","Sonu Nigam","Sophie Ellis-Bextor","Spencer Davis Group","Spice Girls","Stan Getz","Starland Vocal Band","Stephen Sanchez","Steve Aoki","Steve Lacy","Steve Winwood","Stevie B","Sting","Stormzy","Strawberry Alarm Clock","Stray Kids","Stromae","Sublime","Sum 41","Summer Walker","Supertramp","Survivor","Swedish House Mafia","System Of A Down","T-Pain","T.I.","TAEYEON","TKKG","TLC","TOMORROW X TOGETHER","TOTO","TWICE","Tag Team","Tainy","Tammi Terrell","Tanishk Bagchi","Tate McRae","Taylor Swift","Tears For Fears","Tems","Tennessee Ernie Ford","Terence Trent D'Arby","Teresa Brewer","Terry Jacks","The Ames Brothers","The Animals","The B52s","The Bangles","The Beatles","The Black Eyed Peas","The Black Keys","The Box Tops","The Chainsmokers","The Chiffons","The Chordettes","The Clash","The Coasters","The Commodores","The Cowsills","The Cranberries","The Crew-Cuts","The Cure","The Detroit Spinners","The Diamonds","The Doobie Brothers","The Doors","The Drifters","The Emotions","The Eurythmics","The Fireballs","The Flamingos","The Foundations","The Four Seasons","The Fray","The Game","The Go Gos","The Goo Goo Dolls","The Head And The Heart","The Hollies","The Ink Spots","The Isley Brothers","The Jackson 5","The Kid LAROI","The Killers","The Kingston Trio","The Kooks","The Lemon Pipers","The Living Tombstone","The Lumineers","The Mamas & The Papas","The Marvelettes","The McCoys","The Mills Brothers","The Miracles","The Monkees","The Moody Blues","The National","The Neighbourhood","The Notorious B.I.G.","The O'Jays","The Offspring","The Osmonds","The Partridge Family","The Penguins","The Pet Shop Boys","The Platters","The Righteous Brothers","The Rolling Stones","The Ronettes","The Score","The Script","The Seekers","The Shangri-Las","The Smashing Pumpkins","The Staple Singers","The Strokes","The Supremes","The Temptations","The Turtles","The Vamps","The Verve","The Village People","The Weavers","The White Stripes","The Young Rascals","The Zombies","Thelma Houston","Thomas Rhett","Three Days Grace","Three Dog Night","Three Man Down","Timbaland","Timmy Trumpet","Toby Keith","Tom Jones","Tom Petty and the Heartbreakers","Tommy Dorsey","Tommy Edwards","Tommy James & the Shondells","Tone Loc","Toni Braxton","Topic","Tory Lanez","Tove Lo","Trevor Daniel","Trey Songz","Trippie Redd","Trueno","Tulsi Kumar","Tulus","Twenty One Pilots","Two Feet","Ty Dolla $ign","Tyga","Tyler Hubbard","U2","UB40","UZI","Ufo361","Upchurch","V","Vampire Weekend","Van McCoy","Van Morrison","Vance Joy","Vanessa Carlton","Vanessa Williams","Vera Lynn","Vernon Dalhart","Vicente Fernandez","Vishal-Shekhar","Volbeat","WILLOW","Wale","Wallows","Weezer","Wham!","Whitney Houston","Why Don't We","Wilbert Harrison","Wilson Phillips","Wiz Khalifa","Woody Guthrie","Wyclef Jean","XXXTENTACION","Xavi","YG","YNW Melly","YOASOBI","Yandel","Years & Years","Yeat","Yo Gotti","Young Dolph","Young Miko","Young Thug","YoungBoy Never Broke Again","Yung Gravy","Yuuri","Yuvan Shankar Raja","ZAYN","ZZ Top","Zac Brown Band","Zach Bryan","Zara Larsson","aespa","benny blanco","blink-182","d4vd","deadmau5","girl in red","gnash","iann dior","will.i.am"],L4e=["Acid House","Acid Jazz","Acid Rock","Acoustic","Acoustic Blues","Afro-Pop","Afrobeat","Alt-Rock","Alternative","Ambient","American Trad Rock","Americana","Anime","Arena Rock","Art-Rock","Avant-Garde","Avant-Punk","Baladas y Boleros","Barbershop","Baroque","Bebop","Big Band","Black Metal","Blue Note","Bluegrass","Blues","Boogaloo","Bop","Bossa Nova","Bounce","Brazilian Funk","Breakbeat","Britpop","CCM","Cajun","Cantopop","Celtic","Celtic Folk","Chamber Music","Chant","Chanukah","Chicago Blues","Chicago House","Chicano","Children’s Music","Chill","Choral","Christian","Christmas","Classical","Club","College Rock","Conjunto","Cool Jazz","Country","Crunk","Dance","Dancehall","Death Metal","Deep House","Delta Blues","Detroit Techno","Dirty South","Disco","Disney","Dixieland","Doo-wop","Downtempo","Dream Pop","Drill","Drinking Songs","Drone","Drum'n'bass","Dub","Dubstep","EDM","Early Music","East Coast Rap","Easter","Easy Listening","Eclectic","Electric Blues","Electro","Electronic","Electronica","Emo","Enka","Environmental","Ethio-jazz","Experimental","Experimental Rock","Flamenco","Folk","Folk-Rock","Forro","French Pop","Funk","Fusion","Gangsta Rap","Garage","German Folk","German Pop","Glam Rock","Gospel","Goth","Grime","Grindcore","Groove","Grunge","Hair Metal","Halloween","Happy","Hard Bop","Hard Dance","Hard Rock","Hardcore","Hardcore Punk","Hardcore Rap","Hardstyle","Healing","Heavy Metal","High Classical","Hip Hop","Holiday","Honky Tonk","House","IDM","Impressionist","Indie","Industrial","Instrumental","J-Dance","J-Idol","J-Pop","J-Punk","J-Rock","J-Ska","J-Synth","Jackin House","Jam Bands","Japanese Pop","Jazz","Jungle","K-Pop","Karaoke","Kayokyoku","Kids","Kitsch","Klezmer","Krautrock","Latin","Latin Jazz","Latin Rap","Local","Lounge","Lullabies","MPB","Mainstream Jazz","Malay","Mandopop","March","Mariachi","Mawwal","Medieval","Meditation","Metal","Metalcore","Minimal Techno","Minimalism","Modern","Motown","Mugham","Musicals","Musique Concrète","Nature","Neo-Soul","Nerdcore","New Acoustic","New Age","New Mex","New Wave","No Wave","Noise","Nordic","Novelty","OPM","Oi!","Old School Rap","Opera","Orchestral","Original Score","Outlaw Country","Pagode","Party","Piano","Polka","Pop","Pop Film","Pop Latino","Post Dubstep","Power Pop","Praise & Worship","Progressive House","Progressive Rock","Proto-punk","Psych Rock","Psychedelic","Punk","Punk Rock","Qawwali","Quiet Storm","R&B","Ragtime","Rainy Day","Rap","Reggae","Reggaeton","Regional Mexicano","Relaxation","Renaissance","Retro","Rock","Rockabilly","Rocksteady","Romance","Romantic","Roots Reggae","Roots Rock","SKA","Sad","Salsa","Samba","Second Line","Sertanejo","Shaabi","Shoegaze","Sleep","Smooth Jazz","Soft Rock","Soul","Soundtrack","Southern Gospel","Southern Rock","Space Rock","Stage And Screen","Steampunk","Summer","Surf","Swamp Pop","Swing","Synth Pop","Tango","Techno","Teen Pop","Tejano","Tex-Mex","Thanksgiving","Traditional","Trance","Trip Hop","Tropical","Underground Rap","Urban","Urban Cowboy","West Coast Rap","Western Swing","World","Worldbeat","Zydeco"],M4e=["(Everything I Do) I Do it For You","(Ghost) Riders in the Sky","(I Can't Get No) Satisfaction","(I've Got a Gal In) Kalamazoo","(I've Had) the Time of My Life","(It's No) Sin","(Just Like) Starting Over","(Let Me Be Your) Teddy Bear","(Put Another Nickel In) Music! Music! Music!","(Sexual) Healing","(Sittin' On) the Dock of the Bay","(They Long to Be) Close to You","(You Keep Me) Hangin' On","(You're My) Soul & Inspiration","(Your Love Keeps Lifting Me) Higher & Higher","12th Street Rag","1999","19th Nervous Breakdown","50 Ways to Leave Your Lover","9 to 5","96 Tears","A Boy Named Sue","A Hard Day's Night","A String of Pearls","A Thousand Miles","A Tree in the Meadow","A Whiter Shade of Pale","A Whole New World (Aladdin's Theme)","A Woman in Love","A-Tisket A-Tasket","ABC","Abracadabra","Ac-cent-tchu-ate the Positive","Addicted to Love","After You've Gone","Afternoon Delight","Again","Against All Odds (Take a Look At Me Now)","Ain't Misbehavin'","Ain't No Mountain High Enough","Ain't No Sunshine","Ain't That a Shame","Airplanes","All Along the Watchtower","All I Have to Do is Dream","All I Wanna Do","All My Lovin' (You're Never Gonna Get It)","All Night Long (All Night)","All Out of Love","All Shook Up","All You Need is Love","Alone","Alone Again (Naturally)","Always On My Mind","American Pie","American Woman","Angie","Another Brick in the Wall (part 2)","Another Day in Paradise","Another Night","Another One Bites the Dust","Apologize","April Showers","Aquarius/Let The Sunshine In","Are You Lonesome Tonight?","Arthur's Theme (Best That You Can Do)","As Time Goes By","At Last","At the Hop","Auf Wiederseh'n Sweetheart","Baby Baby","Baby Come Back","Baby Got Back","Baby Love","Baby One More Time","Bad Day","Bad Girls","Bad Moon Rising","Bad Romance","Bad, Bad Leroy Brown","Baker Street","Ball of Confusion (That's What the World is Today)","Ballad of the Green Berets","Ballerina","Band On the Run","Band of Gold","Battle of New Orleans","Be Bop a Lula","Be My Baby","Be My Love","Beat It","Beautiful Day","Beauty & the Beast","Because I Love You (The Postman Song)","Because You Loved Me","Because of You","Before The Next Teardrop Falls","Begin the Beguine","Behind Closed Doors","Being With You","Believe","Ben","Bennie & the Jets","Besame Mucho","Best of My Love","Bette Davis Eyes","Big Bad John","Big Girls Don't Cry","Billie Jean","Bitter Sweet Symphony","Black Or White","Black Velvet","Blaze of Glory","Bleeding Love","Blue Suede Shoes","Blue Tango","Blueberry Hill","Blurred Lines","Body & Soul","Bohemian Rhapsody","Boogie Oogie Oogie","Boogie Woogie Bugle Boy","Boom Boom Pow","Born in the USA","Born to Be Wild","Born to Run","Boulevard of Broken Dreams","Brand New Key","Brandy (You're A Fine Girl)","Breaking Up is Hard to Do","Breathe","Bridge Over Troubled Water","Brother Louie","Brother, Can You Spare a Dime?","Brown Eyed Girl","Brown Sugar","Build Me Up Buttercup","Burn","Buttons & Bows","Bye Bye Love","Bye Bye, Blackbird","Bye, Bye, Bye","Caldonia Boogie (What Makes Your Big Head So Hard)","California Dreamin'","California Girls","Call Me","Call Me Maybe","Can You Feel the Love Tonight","Can't Buy Me Love","Can't Get Enough of Your Love, Babe","Can't Help Falling in Love","Candle in the Wind '97","Candy Man","Car Wash","Careless Whisper","Cars","Cat's in the Cradle","Cathy's Clown","Celebration","Centerfold","Chain of Fools","Chances Are","Change the World","Chapel of Love","Chattanooga Choo Choo","Chattanoogie Shoe-Shine Boy","Check On It","Cheek to Cheek","Cherish","Cherry Pink & Apple Blossom White","Cold, Cold Heart","Colors of the Wind","Come On Eileen","Come On-a My House","Come Together","Coming Up","Cracklin' Rosie","Crazy","Crazy For You","Crazy Little Thing Called Love","Crazy in Love","Creep","Crimson & Clover","Crocodile Rock","Cry","Cry Like a Baby","Crying","Da Doo Ron Ron (When He Walked Me Home)","Dance to the Music","Dancing Queen","Dancing in the Dark","Dancing in the Street","Dardanella","Daydream Believer","December 1963 (Oh What a Night)","Delicado","Dilemma","Disco Duck","Disco Lady","Disturbia","Dizzy","Do That to Me One More Time","Do Wah Diddy Diddy","Do Ya Think I'm Sexy?","Do You Love Me?","Don't Be Cruel","Don't Fence Me In","Don't Go Breaking My Heart","Don't Leave Me This Way","Don't Let the Stars Get in Your Eyes","Don't Let the Sun Go Down On Me","Don't Speak","Don't Stop 'Til You Get Enough","Don't Worry Be Happy","Don't You (Forget About Me)","Don't You Want Me","Doo Wop (That Thing)","Down","Down Hearted Blues","Down Under","Downtown","Dreamlover","Dreams","Drop it Like It's Hot","Drops of Jupiter (Tell Me)","Duke of Earl","E.T.","Earth Angel","Ebony & Ivory","Eight Days a Week","Empire State Of Mind","End of the Road","Endless Love","Escape (The Pina Colada Song)","Eve of Destruction","Every Breath You Take","Every Little Thing She Does is Magic","Everybody Loves Somebody","Everybody Wants to Rule the World","Everyday People","Eye of the Tiger","Faith","Fallin'","Fame","Family Affair","Fantasy","Fast Car","Feel Good Inc","Feel Like Making Love","Fire & Rain","Firework","Flashdance. What a Feeling","Fly Robin Fly","Foolish Games","Footloose","For What It's Worth (Stop, Hey What's That Sound)","Fortunate Son","Frankenstein","Freak Me","Freebird","Frenesi","Funkytown","Gangsta's Paradise","Georgia On My Mind","Georgy Girl","Get Back","Get Down Tonight","Get Off of My Cloud","Ghostbusters","Gimme Some Lovin'","Girls Just Wanna Have Fun","Give Me Everything","Gives You Hell","Glamorous","Glory of Love","Go Your Own Way","God Bless America","God Bless the Child","Gold Digger","Gonna Make You Sweat (Everybody Dance Now)","Good Lovin'","Good Times","Good Vibrations","Goodbye Yellow Brick Road","Goodnight, Irene","Got to Give it Up","Grease","Great Balls of Fire","Greatest Love of All","Green Onions","Green River","Green Tambourine","Grenade","Groove is in the Heart","Groovin'","Gypsies, Tramps & Thieves","Hair","Hang On Sloopy","Hanging by a Moment","Hanky Panky","Happy Days Are Here Again","Happy Together","Harbour Lights","Hard to Say I'm Sorry","Harper Valley PTA","Have You Ever Really Loved a Woman?","He'll Have to Go","He's So Fine","He's a Rebel","Heart of Glass","Heart of Gold","Heartbreak Hotel","Hello Dolly","Hello, I Love You, Won't You Tell Me Your Name?","Help Me, Rhonda","Help!","Here Without You","Here in My Heart","Hero","Hey Baby","Hey Jude","Hey Paula","Hey There","Hey There Delilah","Hey Ya!","Higher Love","Hips don't lie","Hit the Road, Jack","Hold On","Hollaback Girl","Honey","Honky Tonk","Honky Tonk Woman","Horse With No Name","Hot Child In The City","Hot Stuff","Hotel California","Hound Dog","House of the Rising Sun","How Deep is Your Love?","How Do I Live?","How Do You Mend a Broken Heart","How High the Moon","How Much is That Doggy in the Window?","How Will I Know","How You Remind Me","How to Save a Life","Hungry Heart","Hurt So Good","I Believe I Can Fly","I Can Dream, Can't I?","I Can Help","I Can See Clearly Now","I Can't Get Next to You","I Can't Get Started","I Can't Go For That (No Can Do)","I Can't Help Myself (Sugar Pie, Honey Bunch)","I Can't Stop Loving You","I Don't Want to Miss a Thing","I Fall to Pieces","I Feel Fine","I Feel For You","I Feel Love","I Get Around","I Got You (I Feel Good)","I Got You Babe","I Gotta Feeling","I Heard it Through the Grapevine","I Honestly Love You","I Just Called to Say I Love You","I Just Wanna Be Your Everything","I Kissed A Girl","I Love Rock 'n' Roll","I Need You Now","I Only Have Eyes For You","I Shot the Sheriff","I Still Haven't Found What I'm Looking For","I Swear","I Think I Love You","I Walk the Line","I Wanna Dance With Somebody (Who Loves Me)","I Wanna Love You","I Want You Back","I Want to Hold Your Hand","I Want to Know What Love Is","I Went to Your Wedding","I Will Always Love You","I Will Follow Him","I Will Survive","I Write the Songs","I'll Be Missing You","I'll Be There","I'll Make Love to You","I'll Never Smile Again","I'll Take You There","I'll Walk Alone","I'll be seeing you","I'm Looking Over a Four Leaf Clover","I'm So Lonesome I Could Cry","I'm Sorry","I'm Walking Behind You","I'm Your Boogie Man","I'm Yours","I'm a Believer","I've Heard That Song Before","If (They Made Me a King)","If I Didn't Care","If You Don't Know Me By Now","If You Leave Me Now","Imagine","In Da Club","In the End","In the Ghetto","In the Mood","In the Summertime","In the Year 2525 (Exordium & Terminus)","Incense & Peppermints","Indian Reservation (The Lament Of The Cherokee Reservation Indian)","Instant Karma","Iris","Ironic","Irreplaceable","It Had to Be You","It's All in the Game","It's My Party","It's Now Or Never","It's Still Rock 'n' Roll to Me","It's Too Late","Jack & Diane","Jailhouse Rock","Jessie's Girl","Jive Talkin'","Johnny B Goode","Joy to the World","Judy in Disguise (With Glasses)","Jump","Jumpin' Jack Flash","Just Dance","Just My Imagination (Running Away With Me)","Just the Way You Are","Kansas City","Karma Chameleon","Keep On Loving You","Killing Me Softly With His Song","King of the Road","Kiss","Kiss & Say Goodbye","Kiss From a Rose","Kiss Me","Kiss On My List","Kiss You All Over","Knock On Wood","Knock Three Times","Kokomo","Kryptonite","Kung Fu Fighting","La Bamba","Lady","Lady Marmalade (Voulez-Vous Coucher Aver Moi Ce Soir?)","Last Train to Clarksville","Layla","Le Freak","Leader of the Pack","Lean On Me","Leaving, on a Jet Plane","Let Me Call You Sweetheart","Let Me Love You","Let it Be","Let it Snow! Let it Snow! Let it Snow!","Let's Dance","Let's Get it On","Let's Groove","Let's Hear it For the Boy","Let's Stay Together","Light My Fire","Lights","Like a Prayer","Like a Rolling Stone","Like a Virgin","Little Darlin'","Little Things Mean a Lot","Live & Let Die","Livin' La Vida Loca","Livin' On a Prayer","Living For the City","Locked Out Of Heaven","Lola","Lonely Boy","Long Cool Woman in a Black Dress","Long Tall Sally","Look Away","Lookin' Out My Back Door","Lose Yourself","Losing My Religion","Louie Louie","Love Child","Love Hangover","Love In This Club","Love Is Blue (L'Amour Est Bleu)","Love Letters in the Sand","Love Me Do","Love Me Tender","Love Shack","Love Theme From 'A Star is Born' (Evergreen)","Love Train","Love Will Keep Us Together","Love is a Many Splendoured Thing","Love to Love You Baby","Love's Theme","Loving You","Low","Macarena","Mack the Knife","Maggie May","Magic","Magic Carpet Ride","Make Love to Me","Make it With You","Makin' Whoopee","Mama Told Me Not to Come","Man in the Mirror","Manana (Is Soon Enough For Me)","Maneater","Maniac","Maybellene","Me & Bobby McGee","Me & Mrs Jones","Memories Are Made of This","Mercy Mercy Me (The Ecology)","Mickey","Midnight Train to Georgia","Minnie the Moocher","Miss You","Miss You Much","Mister Sandman","Mmmbop","Mona Lisa","Monday Monday","Money For Nothing","Mony Mony","Mood Indigo","Moonlight Cocktail","Moonlight Serenade","More Than Words","More Than a Feeling","Morning Train (Nine to Five)","Mr Big Stuff","Mr Brightside","Mr Tambourine Man","Mrs Brown You've Got a Lovely Daughter","Mrs Robinson","Mule Train","Music","My Blue Heaven","My Boyfriend's Back","My Eyes Adored You","My Girl","My Guy","My Heart Will Go On","My Life","My Love","My Man","My Prayer","My Sharona","My Sweet Lord","Na Na Hey Hey (Kiss Him Goodbye)","Nature Boy","Near You","Need You Now","Need You Tonight","Never Gonna Give You Up","Night & Day","Night Fever","Nights in White Satin","No One","No Scrubs","Nobody Does it Better","Nothin' on You","Nothing Compares 2 U","Nothing's Gonna Stop Us Now","Ode To Billie Joe","Oh Happy Day","Oh My Papa (O Mein Papa)","Oh, Pretty Woman","Ol' Man River","Ole Buttermilk Sky","On Bended Knee","On My Own","On the Atchison, Topeka & the Santa Fe","One","One Bad Apple","One More Try","One O'Clock Jump","One Sweet Day","One of These Nights","One of Us","Only The Lonely (Know The Way I Feel)","Only You (And You Alone)","Open Arms","Over There","Over the Rainbow","Paint it Black","Papa Don't Preach","Papa Was a Rolling Stone","Papa's Got a Brand New Bag","Paper Doll","Paper Planes","Paperback Writer","Party Rock Anthem","Peg o' My Heart","Peggy Sue","Pennies From Heaven","Penny Lane","People","People Got to Be Free","Personality","Philadelphia Freedom","Physical","Piano Man","Pick Up the Pieces","Pistol Packin' Mama","Play That Funky Music","Please Mr Postman","Poker Face","Pon De Replay","Pony Time","Pop Muzik","Prisoner of Love","Private Eyes","Promiscuous","Proud Mary","Purple Haze","Purple Rain","Puttin' on the Ritz","Que sera sera (Whatever will be will be)","Queen of Hearts","Rag Doll","Rag Mop","Rags to Riches","Raindrops Keep Falling On My Head","Rapture","Ray of Light","Reach Out (I'll Be There)","Red Red Wine","Rehab","Respect","Return to Sender","Reunited","Revolution","Rhapsody in Blue","Rhinestone Cowboy","Rich Girl","Riders On the Storm","Right Back Where We Started From","Ring My Bell","Ring of Fire","Rock Around the Clock","Rock With You","Rock Your Baby","Rock the Boat","Rock the Casbah","Roll Over Beethoven","Roll With It","Rolling In The Deep","Rosanna","Roses Are Red","Royals","Ruby Tuesday","Rudolph, the Red-Nosed Reindeer","Rum & Coca-Cola","Runaround Sue","Runaway","Running Scared","Rush Rush","Sailing","Save the Best For Last","Save the Last Dance For Me","Say It Right","Say My Name","Say Say Say","Say You, Say Me","School's Out","Seasons in the Sun","Secret Love","Sentimental Journey","Sexyback","Sh-Boom (Life Could Be a Dream)","Shadow Dancing","Shake Down","Shake You Down","She Drives Me Crazy","She Loves You","She's a Lady","Shining Star","Shop Around","Shout","Silly Love Songs","Since U Been Gone","Sing, Sing, Sing (With A Swing)","Singing The Blues","Single Ladies (Put A Ring On It)","Sir Duke","Sixteen Tons","Sledgehammer","Sleep Walk","Sleepy Lagoon","Slow Poke","Smells Like Teen Spirit","Smoke Gets in Your Eyes","Smoke On the Water","Smoke! Smoke! Smoke! (That Cigarette)","Smooth","So Much in Love","Soldier Boy","Some Enchanted Evening","Some of These Days","Somebody That I Used to Know","Somebody to Love","Someday","Somethin' Stupid","Something","Soul Man","Spanish Harlem","Spill the Wine","Spinning Wheel","Spirit in the Sky","St George & the Dragonette","St Louis Blues","Stagger Lee","Stairway to Heaven","Stand By Me","Stardust","Stars & Stripes Forever","Stay (I Missed You)","Stayin' Alive","Stop! in the Name of Love","Stormy Weather (Keeps Rainin' All the Time)","Straight Up","Strange Fruit","Stranger On the Shore","Strangers in the Night","Strawberry Fields Forever","Streets of Philadelphia","Stronger","Stuck On You","Sugar Shack","Sugar Sugar","Summer in the City","Summertime Blues","Sunday, Monday or Always","Sunshine Superman","Sunshine of Your Love","Superstar","Superstition","Surfin' USA","Suspicious Minds","Swanee","Sweet Caroline (Good Times Never Seemed So Good)","Sweet Child O' Mine","Sweet Dreams (Are Made of This)","Sweet Georgia Brown","Sweet Home Alabama","Sweet Soul Music","Swinging On a Star","T For Texas (Blue Yodel No 1)","TSOP (The Sound of Philadelphia)","Take Me Home, Country Roads","Take My Breath Away","Take On Me","Take The 'A' Train","Take a Bow","Tammy","Tangerine","Tears in Heaven","Tears of a Clown","Temperature","Tennessee Waltz","Tequila","Tha Crossroads","Thank You (Falettinme be Mice Elf Again)","That Lucky Old Sun (Just Rolls Around Heaven All Day)","That Old Black Magic","That'll Be the Day","That's Amore","That's What Friends Are For","That's the Way (I Like It)","That's the Way Love Goes","The Boy is Mine","The Boys of Summer","The Christmas Song (Chestnuts Roasting On An Open Fire)","The End of the World","The First Time Ever I Saw Your Face","The Girl From Ipanema","The Glow-Worm","The Great Pretender","The Gypsy","The Hustle","The Joker","The Last Dance","The Letter","The Loco-Motion","The Long & Winding Road","The Love You Save","The Morning After","The Power of Love","The Prisoner's Song","The Reason","The Rose","The Sign","The Song From Moulin Rouge (Where Is Your Heart)","The Sounds of Silence","The Streak","The Sweet Escape","The Thing","The Tide is High","The Tracks of My Tears","The Twist","The Wanderer","The Way We Were","The Way You Look Tonight","The Way You Move","Theme From 'A Summer Place'","Theme From 'Greatest American Hero' (Believe It Or Not)","Theme From 'Shaft'","There goes my baby","These Boots Are Made For Walking","Third Man Theme","This Diamond Ring","This Guy's in Love With You","This Land is Your Land","This Love","This Ole House","This Used to Be My Playground","Three Coins in the Fountain","Three Times a Lady","Thrift Shop","Thriller","Ticket to Ride","Tie a Yellow Ribbon 'round the Old Oak Tree","Tiger Rag","Tighten Up","Tik-Toc","Till I Waltz Again With You","Till The End of Time","Time After Time","Time of the Season","To Sir, with Love","Tom Dooley","Tonight's the Night (Gonna Be Alright)","Too Close","Too Young","Tossing & Turning","Total Eclipse of the Heart","Touch Me","Toxic","Travellin' Band","Travellin' Man","Truly Madly Deeply","Turn! Turn! Turn! (To Everything There is a Season)","Tutti Frutti","Twist & Shout","Two Hearts","U Can't Touch This","U Got it Bad","Umbrella","Un-Break My Heart","Unbelievable","Unchained Melody","Uncle Albert (Admiral Halsey)","Under the Boardwalk","Under the Bridge","Unforgettable","Up Around the Bend","Up Up & Away","Up Where We Belong","Upside Down","Use Somebody","Vaya Con Dios (may God Be With You)","Venus","Vision of Love","Viva La Vida","Vogue","Volare","Wabash Cannonball","Waiting For a Girl Like You","Wake Me Up Before You Go Go","Wake Up Little Susie","Walk Don't Run","Walk Like a Man","Walk Like an Egyptian","Walk On By","Walk On the Wild Side","Walk This Way","Wannabe","Want Ads","Wanted","War","Waterfalls","Wayward Wind","We Are Family","We Are Young","We Are the Champions","We Are the World","We Belong Together","We Built This City","We Can Work it Out","We Didn't Start the Fire","We Found Love","We Got The Beat","We Will Rock You","We've Only Just Begun","Weak","Wedding Bell Blues","West End Blues","West End Girls","What Goes Around Comes Around","What a Fool Believes","What'd I Say","What's Going On?","What's Love Got to Do With It?","Whatcha Say","Wheel of Fortune","When Doves Cry","When You Wish Upon a Star","When a Man Loves a Woman","Where Did Our Love Go","Where is the Love?","Whip It","Whispering","White Christmas","White Rabbit","Whole Lotta Love","Whole Lotta Shakin' Goin' On","Whoomp! (There it Is)","Why Do Fools Fall in Love?","Why Don't You Believe Me?","Wichita Lineman","Wicked Game","Wild Thing","Wild Wild West","Will It Go Round In Circles","Will You Love Me Tomorrow","Winchester Cathedral","Wind Beneath My Wings","Wipe Out","Wishing Well","With Or Without You","Without Me","Without You","Woman","Won't Get Fooled Again","Wooly Bully","Working My Way Back to You","YMCA","Yakety Yak","Yeah!","Yellow Rose of Texas","Yesterday","You Ain't Seen Nothin' Yet","You Always Hurt the One You Love","You Are the Sunshine of My Life","You Belong With Me","You Belong to Me","You Can't Hurry Love","You Don't Bring Me Flowers","You Don't Have to Be a Star (To Be in My Show)","You Light Up My Life","You Make Me Feel Brand New","You Make Me Feel Like Dancing","You Really Got Me","You Send Me","You Sexy Thing","You Were Meant for Me","You make Me Wanna","You'll Never Know","You're Beautiful","You're So Vain","You're Still the One","You're the One That I Want","You've Got a Friend","You've Lost That Lovin' Feelin'","Your Cheatin' Heart","Your Song"],I4e={album:D4e,artist:N4e,genre:L4e,song_name:M4e},O4e=I4e,B4e=["activist","artist","author","blogger","business owner","coach","creator","designer","developer","dreamer","educator","engineer","entrepreneur","environmentalist","film lover","filmmaker","foodie","founder","friend","gamer","geek","grad","inventor","leader","model","musician","nerd","parent","patriot","person","philosopher","photographer","public speaker","scientist","singer","streamer","student","teacher","traveler","veteran","writer"],P4e=["{{person.bio_part}}","{{person.bio_part}}, {{person.bio_part}}","{{person.bio_part}}, {{person.bio_part}}, {{person.bio_part}}","{{person.bio_part}}, {{person.bio_part}}, {{person.bio_part}} {{internet.emoji}}","{{word.noun}} {{person.bio_supporter}}","{{word.noun}} {{person.bio_supporter}}  {{internet.emoji}}","{{word.noun}} {{person.bio_supporter}}, {{person.bio_part}}","{{word.noun}} {{person.bio_supporter}}, {{person.bio_part}} {{internet.emoji}}"],z4e=["advocate","devotee","enthusiast","fan","junkie","lover","supporter"],H4e={generic:["Aaliyah","Aaron","Abagail","Abbey","Abbie","Abbigail","Abby","Abdiel","Abdul","Abdullah","Abe","Abel","Abelardo","Abigail","Abigale","Abigayle","Abner","Abraham","Ada","Adah","Adalberto","Adaline","Adam","Adan","Addie","Addison","Adela","Adelbert","Adele","Adelia","Adeline","Adell","Adella","Adelle","Aditya","Adolf","Adolfo","Adolph","Adolphus","Adonis","Adrain","Adrian","Adriana","Adrianna","Adriel","Adrien","Adrienne","Afton","Aglae","Agnes","Agustin","Agustina","Ahmad","Ahmed","Aida","Aidan","Aiden","Aileen","Aimee","Aisha","Aiyana","Akeem","Al","Alaina","Alan","Alana","Alanis","Alanna","Alayna","Alba","Albert","Alberta","Albertha","Alberto","Albin","Albina","Alda","Alden","Alec","Aleen","Alejandra","Alejandrin","Alek","Alena","Alene","Alessandra","Alessandro","Alessia","Aletha","Alex","Alexa","Alexander","Alexandra","Alexandre","Alexandrea","Alexandria","Alexandrine","Alexandro","Alexane","Alexanne","Alexie","Alexis","Alexys","Alexzander","Alf","Alfonso","Alfonzo","Alford","Alfred","Alfreda","Alfredo","Ali","Alia","Alice","Alicia","Alisa","Alisha","Alison","Alivia","Aliya","Aliyah","Aliza","Alize","Allan","Allen","Allene","Allie","Allison","Ally","Alphonso","Alta","Althea","Alva","Alvah","Alvena","Alvera","Alverta","Alvina","Alvis","Alyce","Alycia","Alysa","Alysha","Alyson","Alysson","Amalia","Amanda","Amani","Amara","Amari","Amaya","Amber","Ambrose","Amelia","Amelie","Amely","America","Americo","Amie","Amina","Amir","Amira","Amiya","Amos","Amparo","Amy","Amya","Ana","Anabel","Anabelle","Anahi","Anais","Anastacio","Anastasia","Anderson","Andre","Andreane","Andreanne","Andres","Andrew","Andy","Angel","Angela","Angelica","Angelina","Angeline","Angelita","Angelo","Angie","Angus","Anibal","Anika","Anissa","Anita","Aniya","Aniyah","Anjali","Anna","Annabel","Annabell","Annabelle","Annalise","Annamae","Annamarie","Anne","Annetta","Annette","Annie","Ansel","Ansley","Anthony","Antoinette","Antone","Antonetta","Antonette","Antonia","Antonietta","Antonina","Antonio","Antwan","Antwon","Anya","April","Ara","Araceli","Aracely","Arch","Archibald","Ardella","Arden","Ardith","Arely","Ari","Ariane","Arianna","Aric","Ariel","Arielle","Arjun","Arlene","Arlie","Arlo","Armand","Armando","Armani","Arnaldo","Arne","Arno","Arnold","Arnoldo","Arnulfo","Aron","Art","Arthur","Arturo","Arvel","Arvid","Arvilla","Aryanna","Asa","Asha","Ashlee","Ashleigh","Ashley","Ashly","Ashlynn","Ashton","Ashtyn","Asia","Assunta","Astrid","Athena","Aubree","Aubrey","Audie","Audra","Audreanne","Audrey","August","Augusta","Augustine","Augustus","Aurelia","Aurelie","Aurelio","Aurore","Austen","Austin","Austyn","Autumn","Ava","Avery","Avis","Axel","Ayana","Ayden","Ayla","Aylin","Baby","Bailee","Bailey","Barbara","Barney","Baron","Barrett","Barry","Bart","Bartholome","Barton","Baylee","Beatrice","Beau","Beaulah","Bell","Bella","Belle","Ben","Benedict","Benjamin","Bennett","Bennie","Benny","Benton","Berenice","Bernadette","Bernadine","Bernard","Bernardo","Berneice","Bernhard","Bernice","Bernie","Berniece","Bernita","Berry","Bert","Berta","Bertha","Bertram","Bertrand","Beryl","Bessie","Beth","Bethany","Bethel","Betsy","Bette","Bettie","Betty","Bettye","Beulah","Beverly","Bianka","Bill","Billie","Billy","Birdie","Blair","Blaise","Blake","Blanca","Blanche","Blaze","Bo","Bobbie","Bobby","Bonita","Bonnie","Boris","Boyd","Brad","Braden","Bradford","Bradley","Bradly","Brady","Braeden","Brain","Brandi","Brando","Brandon","Brandt","Brandy","Brandyn","Brannon","Branson","Brant","Braulio","Braxton","Brayan","Breana","Breanna","Breanne","Brenda","Brendan","Brenden","Brendon","Brenna","Brennan","Brennon","Brent","Bret","Brett","Bria","Brian","Briana","Brianne","Brice","Bridget","Bridgette","Bridie","Brielle","Brigitte","Brionna","Brisa","Britney","Brittany","Brock","Broderick","Brody","Brook","Brooke","Brooklyn","Brooks","Brown","Bruce","Bryana","Bryce","Brycen","Bryon","Buck","Bud","Buddy","Buford","Bulah","Burdette","Burley","Burnice","Buster","Cade","Caden","Caesar","Caitlyn","Cale","Caleb","Caleigh","Cali","Calista","Callie","Camden","Cameron","Camila","Camilla","Camille","Camren","Camron","Camryn","Camylle","Candace","Candelario","Candice","Candida","Candido","Cara","Carey","Carissa","Carlee","Carleton","Carley","Carli","Carlie","Carlo","Carlos","Carlotta","Carmel","Carmela","Carmella","Carmelo","Carmen","Carmine","Carol","Carolanne","Carole","Carolina","Caroline","Carolyn","Carolyne","Carrie","Carroll","Carson","Carter","Cary","Casandra","Casey","Casimer","Casimir","Casper","Cassandra","Cassandre","Cassidy","Cassie","Catalina","Caterina","Catharine","Catherine","Cathrine","Cathryn","Cathy","Cayla","Ceasar","Cecelia","Cecil","Cecile","Cecilia","Cedrick","Celestine","Celestino","Celia","Celine","Cesar","Chad","Chadd","Chadrick","Chaim","Chance","Chandler","Chanel","Chanelle","Charity","Charlene","Charles","Charley","Charlie","Charlotte","Chase","Chasity","Chauncey","Chaya","Chaz","Chelsea","Chelsey","Chelsie","Chesley","Chester","Chet","Cheyanne","Cheyenne","Chloe","Chris","Christ","Christa","Christelle","Christian","Christiana","Christina","Christine","Christop","Christophe","Christopher","Christy","Chyna","Ciara","Cicero","Cielo","Cierra","Cindy","Citlalli","Clair","Claire","Clara","Clarabelle","Clare","Clarissa","Clark","Claud","Claude","Claudia","Claudie","Claudine","Clay","Clemens","Clement","Clementina","Clementine","Clemmie","Cleo","Cleora","Cleta","Cletus","Cleve","Cleveland","Clifford","Clifton","Clint","Clinton","Clotilde","Clovis","Cloyd","Clyde","Coby","Cody","Colby","Cole","Coleman","Colin","Colleen","Collin","Colt","Colten","Colton","Columbus","Concepcion","Conner","Connie","Connor","Conor","Conrad","Constance","Constantin","Consuelo","Cooper","Cora","Coralie","Corbin","Cordelia","Cordell","Cordia","Cordie","Corene","Corine","Cornelius","Cornell","Corrine","Cortez","Cortney","Cory","Coty","Courtney","Coy","Craig","Crawford","Creola","Cristal","Cristian","Cristina","Cristobal","Cristopher","Cruz","Crystal","Crystel","Cullen","Curt","Curtis","Cydney","Cynthia","Cyril","Cyrus","D'angelo","Dagmar","Dahlia","Daija","Daisha","Daisy","Dakota","Dale","Dallas","Dallin","Dalton","Damaris","Dameon","Damian","Damien","Damion","Damon","Dan","Dana","Dandre","Dane","Dangelo","Danial","Daniela","Daniella","Danielle","Danika","Dannie","Danny","Dante","Danyka","Daphne","Daphnee","Daphney","Darby","Daren","Darian","Dariana","Darien","Dario","Darion","Darius","Darlene","Daron","Darrel","Darrell","Darren","Darrick","Darrin","Darrion","Darron","Darryl","Darwin","Daryl","Dashawn","Dasia","Dave","David","Davin","Davion","Davon","Davonte","Dawn","Dawson","Dax","Dayana","Dayna","Dayne","Dayton","Dean","Deangelo","Deanna","Deborah","Declan","Dedric","Dedrick","Dee","Deion","Deja","Dejah","Dejon","Dejuan","Delaney","Delbert","Delfina","Delia","Delilah","Dell","Della","Delmer","Delores","Delpha","Delphia","Delphine","Delta","Demarco","Demarcus","Demario","Demetris","Demetrius","Demond","Dena","Denis","Dennis","Deon","Deondre","Deontae","Deonte","Dereck","Derek","Derick","Deron","Derrick","Deshaun","Deshawn","Desiree","Desmond","Dessie","Destany","Destin","Destinee","Destiney","Destini","Destiny","Devan","Devante","Deven","Devin","Devon","Devonte","Devyn","Dewayne","Dewitt","Dexter","Diamond","Diana","Dianna","Diego","Dillan","Dillon","Dimitri","Dina","Dino","Dion","Dixie","Dock","Dolly","Dolores","Domenic","Domenica","Domenick","Domenico","Domingo","Dominic","Dominique","Don","Donald","Donato","Donavon","Donna","Donnell","Donnie","Donny","Dora","Dorcas","Dorian","Doris","Dorothea","Dorothy","Dorris","Dortha","Dorthy","Doug","Douglas","Dovie","Doyle","Drake","Drew","Duane","Dudley","Dulce","Duncan","Durward","Dustin","Dusty","Dwight","Dylan","Earl","Earlene","Earline","Earnest","Earnestine","Easter","Easton","Ebba","Ebony","Ed","Eda","Edd","Eddie","Eden","Edgar","Edgardo","Edison","Edmond","Edmund","Edna","Eduardo","Edward","Edwardo","Edwin","Edwina","Edyth","Edythe","Effie","Efrain","Efren","Eileen","Einar","Eino","Eladio","Elaina","Elbert","Elda","Eldon","Eldora","Eldred","Eldridge","Eleanora","Eleanore","Eleazar","Electa","Elena","Elenor","Elenora","Eleonore","Elfrieda","Eli","Elian","Eliane","Elias","Eliezer","Elijah","Elinor","Elinore","Elisa","Elisabeth","Elise","Eliseo","Elisha","Elissa","Eliza","Elizabeth","Ella","Ellen","Ellie","Elliot","Elliott","Ellis","Ellsworth","Elmer","Elmira","Elmo","Elmore","Elna","Elnora","Elody","Eloisa","Eloise","Elouise","Eloy","Elroy","Elsa","Else","Elsie","Elta","Elton","Elva","Elvera","Elvie","Elvis","Elwin","Elwyn","Elyse","Elyssa","Elza","Emanuel","Emelia","Emelie","Emely","Emerald","Emerson","Emery","Emie","Emil","Emile","Emilia","Emiliano","Emilie","Emilio","Emily","Emma","Emmalee","Emmanuel","Emmanuelle","Emmet","Emmett","Emmie","Emmitt","Emmy","Emory","Ena","Enid","Enoch","Enola","Enos","Enrico","Enrique","Ephraim","Era","Eriberto","Eric","Erica","Erich","Erick","Ericka","Erik","Erika","Erin","Erling","Erna","Ernest","Ernestina","Ernestine","Ernesto","Ernie","Ervin","Erwin","Eryn","Esmeralda","Esperanza","Esta","Esteban","Estefania","Estel","Estell","Estella","Estelle","Estevan","Esther","Estrella","Etha","Ethan","Ethel","Ethelyn","Ethyl","Ettie","Eudora","Eugene","Eugenia","Eula","Eulah","Eulalia","Euna","Eunice","Eusebio","Eva","Evalyn","Evan","Evangeline","Evans","Eve","Eveline","Evelyn","Everardo","Everett","Everette","Evert","Evie","Ewald","Ewell","Ezekiel","Ezequiel","Ezra","Fabian","Fabiola","Fae","Fannie","Fanny","Fatima","Faustino","Fausto","Favian","Fay","Faye","Federico","Felicia","Felicita","Felicity","Felipa","Felipe","Felix","Felton","Fermin","Fern","Fernando","Ferne","Fidel","Filiberto","Filomena","Finn","Fiona","Flavie","Flavio","Fleta","Fletcher","Flo","Florence","Florencio","Florian","Florida","Florine","Flossie","Floy","Floyd","Ford","Forest","Forrest","Foster","Frances","Francesca","Francesco","Francis","Francisca","Francisco","Franco","Frank","Frankie","Franz","Fred","Freda","Freddie","Freddy","Frederic","Frederick","Frederik","Frederique","Fredrick","Fredy","Freeda","Freeman","Freida","Frida","Frieda","Friedrich","Fritz","Furman","Gabe","Gabriel","Gabriella","Gabrielle","Gaetano","Gage","Gail","Gardner","Garett","Garfield","Garland","Garnet","Garnett","Garret","Garrett","Garrick","Garrison","Garry","Garth","Gaston","Gavin","Gayle","Gene","General","Genesis","Genevieve","Gennaro","Genoveva","Geo","Geoffrey","George","Georgette","Georgiana","Georgianna","Geovanni","Geovanny","Geovany","Gerald","Geraldine","Gerard","Gerardo","Gerda","Gerhard","Germaine","German","Gerry","Gerson","Gertrude","Gia","Gianni","Gideon","Gilbert","Gilberto","Gilda","Giles","Gillian","Gina","Gino","Giovani","Giovanna","Giovanni","Giovanny","Gisselle","Giuseppe","Gladyce","Gladys","Glen","Glenda","Glenna","Glennie","Gloria","Godfrey","Golda","Golden","Gonzalo","Gordon","Grace","Gracie","Graciela","Grady","Graham","Grant","Granville","Grayce","Grayson","Green","Greg","Gregg","Gregoria","Gregorio","Gregory","Greta","Gretchen","Greyson","Griffin","Grover","Guadalupe","Gudrun","Guido","Guillermo","Guiseppe","Gunnar","Gunner","Gus","Gussie","Gust","Gustave","Guy","Gwen","Gwendolyn","Hadley","Hailee","Hailey","Hailie","Hal","Haleigh","Haley","Halie","Halle","Hallie","Hank","Hanna","Hannah","Hans","Hardy","Harley","Harmon","Harmony","Harold","Harrison","Harry","Harvey","Haskell","Hassan","Hassie","Hattie","Haven","Hayden","Haylee","Hayley","Haylie","Hazel","Hazle","Heath","Heather","Heaven","Heber","Hector","Heidi","Helen","Helena","Helene","Helga","Hellen","Helmer","Heloise","Henderson","Henri","Henriette","Henry","Herbert","Herman","Hermann","Hermina","Herminia","Herminio","Hershel","Herta","Hertha","Hester","Hettie","Hilario","Hilbert","Hilda","Hildegard","Hillard","Hillary","Hilma","Hilton","Hipolito","Hiram","Hobart","Holden","Hollie","Hollis","Holly","Hope","Horace","Horacio","Hortense","Hosea","Houston","Howard","Howell","Hoyt","Hubert","Hudson","Hugh","Hulda","Humberto","Hunter","Hyman","Ian","Ibrahim","Icie","Ida","Idell","Idella","Ignacio","Ignatius","Ike","Ila","Ilene","Iliana","Ima","Imani","Imelda","Immanuel","Imogene","Ines","Irma","Irving","Irwin","Isaac","Isabel","Isabell","Isabella","Isabelle","Isac","Isadore","Isai","Isaiah","Isaias","Isidro","Ismael","Isobel","Isom","Israel","Issac","Itzel","Iva","Ivah","Ivory","Ivy","Izabella","Izaiah","Jabari","Jace","Jacey","Jacinthe","Jacinto","Jack","Jackeline","Jackie","Jacklyn","Jackson","Jacky","Jaclyn","Jacquelyn","Jacques","Jacynthe","Jada","Jade","Jaden","Jadon","Jadyn","Jaeden","Jaida","Jaiden","Jailyn","Jaime","Jairo","Jakayla","Jake","Jakob","Jaleel","Jalen","Jalon","Jalyn","Jamaal","Jamal","Jamar","Jamarcus","Jamel","Jameson","Jamey","Jamie","Jamil","Jamir","Jamison","Jammie","Jan","Jana","Janae","Jane","Janelle","Janessa","Janet","Janice","Janick","Janie","Janis","Janiya","Jannie","Jany","Jaquan","Jaquelin","Jaqueline","Jared","Jaren","Jarod","Jaron","Jarred","Jarrell","Jarret","Jarrett","Jarrod","Jarvis","Jasen","Jasmin","Jason","Jasper","Jaunita","Javier","Javon","Javonte","Jay","Jayce","Jaycee","Jayda","Jayde","Jayden","Jaydon","Jaylan","Jaylen","Jaylin","Jaylon","Jayme","Jayne","Jayson","Jazlyn","Jazmin","Jazmyn","Jazmyne","Jean","Jeanette","Jeanie","Jeanne","Jed","Jedediah","Jedidiah","Jeff","Jefferey","Jeffery","Jeffrey","Jeffry","Jena","Jenifer","Jennie","Jennifer","Jennings","Jennyfer","Jensen","Jerad","Jerald","Jeramie","Jeramy","Jerel","Jeremie","Jeremy","Jermain","Jermaine","Jermey","Jerod","Jerome","Jeromy","Jerrell","Jerrod","Jerrold","Jerry","Jess","Jesse","Jessica","Jessie","Jessika","Jessy","Jessyca","Jesus","Jett","Jettie","Jevon","Jewel","Jewell","Jillian","Jimmie","Jimmy","Jo","Joan","Joana","Joanie","Joanne","Joannie","Joanny","Joany","Joaquin","Jocelyn","Jodie","Jody","Joe","Joel","Joelle","Joesph","Joey","Johan","Johann","Johanna","Johathan","John","Johnathan","Johnathon","Johnnie","Johnny","Johnpaul","Johnson","Jolie","Jon","Jonas","Jonatan","Jonathan","Jonathon","Jordan","Jordane","Jordi","Jordon","Jordy","Jordyn","Jorge","Jose","Josefa","Josefina","Joseph","Josephine","Josh","Joshua","Joshuah","Josiah","Josiane","Josianne","Josie","Josue","Jovan","Jovani","Jovanny","Jovany","Joy","Joyce","Juana","Juanita","Judah","Judd","Jude","Judge","Judson","Judy","Jules","Julia","Julian","Juliana","Julianne","Julie","Julien","Juliet","Julio","Julius","June","Junior","Junius","Justen","Justice","Justina","Justine","Juston","Justus","Justyn","Juvenal","Juwan","Kacey","Kaci","Kacie","Kade","Kaden","Kadin","Kaela","Kaelyn","Kaia","Kailee","Kailey","Kailyn","Kaitlin","Kaitlyn","Kale","Kaleb","Kaleigh","Kaley","Kali","Kallie","Kameron","Kamille","Kamren","Kamron","Kamryn","Kane","Kara","Kareem","Karelle","Karen","Kari","Kariane","Karianne","Karina","Karine","Karl","Karlee","Karley","Karli","Karlie","Karolann","Karson","Kasandra","Kasey","Kassandra","Katarina","Katelin","Katelyn","Katelynn","Katharina","Katherine","Katheryn","Kathleen","Kathlyn","Kathryn","Kathryne","Katlyn","Katlynn","Katrina","Katrine","Kattie","Kavon","Kay","Kaya","Kaycee","Kayden","Kayla","Kaylah","Kaylee","Kayleigh","Kayley","Kayli","Kaylie","Kaylin","Keagan","Keanu","Keara","Keaton","Keegan","Keeley","Keely","Keenan","Keira","Keith","Kellen","Kelley","Kelli","Kellie","Kelly","Kelsi","Kelsie","Kelton","Kelvin","Ken","Kendall","Kendra","Kendrick","Kenna","Kennedi","Kennedy","Kenneth","Kennith","Kenny","Kenton","Kenya","Kenyatta","Kenyon","Keon","Keshaun","Keshawn","Keven","Kevin","Kevon","Keyon","Keyshawn","Khalid","Khalil","Kian","Kiana","Kianna","Kiara","Kiarra","Kiel","Kiera","Kieran","Kiley","Kim","Kimberly","King","Kip","Kira","Kirk","Kirsten","Kirstin","Kitty","Kobe","Koby","Kody","Kolby","Kole","Korbin","Korey","Kory","Kraig","Kris","Krista","Kristian","Kristin","Kristina","Kristofer","Kristoffer","Kristopher","Kristy","Krystal","Krystel","Krystina","Kurt","Kurtis","Kyla","Kyle","Kylee","Kyleigh","Kyler","Kylie","Kyra","Lacey","Lacy","Ladarius","Lafayette","Laila","Laisha","Lamar","Lambert","Lamont","Lance","Landen","Lane","Laney","Larissa","Laron","Larry","Larue","Laura","Laurel","Lauren","Laurence","Lauretta","Lauriane","Laurianne","Laurie","Laurine","Laury","Lauryn","Lavada","Lavern","Laverna","Laverne","Lavina","Lavinia","Lavon","Lavonne","Lawrence","Lawson","Layla","Layne","Lazaro","Lea","Leann","Leanna","Leanne","Leatha","Leda","Lee","Leif","Leila","Leilani","Lela","Lelah","Leland","Lelia","Lempi","Lemuel","Lenna","Lennie","Lenny","Lenora","Lenore","Leo","Leola","Leon","Leonard","Leonardo","Leone","Leonel","Leonie","Leonor","Leonora","Leopold","Leopoldo","Leora","Lera","Lesley","Leslie","Lesly","Lessie","Lester","Leta","Letha","Letitia","Levi","Lew","Lewis","Lexi","Lexie","Lexus","Lia","Liam","Liana","Libbie","Libby","Lila","Lilian","Liliana","Liliane","Lilla","Lillian","Lilliana","Lillie","Lilly","Lily","Lilyan","Lina","Lincoln","Linda","Lindsay","Lindsey","Linnea","Linnie","Linwood","Lionel","Lisa","Lisandro","Lisette","Litzy","Liza","Lizeth","Lizzie","Llewellyn","Lloyd","Logan","Lois","Lola","Lolita","Loma","Lon","London","Lonie","Lonnie","Lonny","Lonzo","Lora","Loraine","Loren","Lorena","Lorenz","Lorenza","Lorenzo","Lori","Lorine","Lorna","Lottie","Lou","Louie","Louisa","Lourdes","Louvenia","Lowell","Loy","Loyal","Loyce","Lucas","Luciano","Lucie","Lucienne","Lucile","Lucinda","Lucio","Lucious","Lucius","Lucy","Ludie","Ludwig","Lue","Luella","Luigi","Luis","Luisa","Lukas","Lula","Lulu","Luna","Lupe","Lura","Lurline","Luther","Luz","Lyda","Lydia","Lyla","Lynn","Lyric","Lysanne","Mabel","Mabelle","Mable","Mac","Macey","Maci","Macie","Mack","Mackenzie","Macy","Madaline","Madalyn","Maddison","Madeline","Madelyn","Madelynn","Madge","Madie","Madilyn","Madisen","Madison","Madisyn","Madonna","Madyson","Mae","Maegan","Maeve","Mafalda","Magali","Magdalen","Magdalena","Maggie","Magnolia","Magnus","Maia","Maida","Maiya","Major","Makayla","Makenna","Makenzie","Malachi","Malcolm","Malika","Malinda","Mallie","Mallory","Malvina","Mandy","Manley","Manuel","Manuela","Mara","Marc","Marcel","Marcelina","Marcelino","Marcella","Marcelle","Marcellus","Marcelo","Marcia","Marco","Marcos","Marcus","Margaret","Margarete","Margarett","Margaretta","Margarette","Margarita","Marge","Margie","Margot","Margret","Marguerite","Maria","Mariah","Mariam","Marian","Mariana","Mariane","Marianna","Marianne","Mariano","Maribel","Marie","Mariela","Marielle","Marietta","Marilie","Marilou","Marilyne","Marina","Mario","Marion","Marisa","Marisol","Maritza","Marjolaine","Marjorie","Marjory","Mark","Markus","Marlee","Marlen","Marlene","Marley","Marlin","Marlon","Marques","Marquis","Marquise","Marshall","Marta","Martin","Martina","Martine","Marty","Marvin","Mary","Maryam","Maryjane","Maryse","Mason","Mateo","Mathew","Mathias","Mathilde","Matilda","Matilde","Matt","Matteo","Mattie","Maud","Maude","Maudie","Maureen","Maurice","Mauricio","Maurine","Maverick","Mavis","Max","Maxie","Maxime","Maximilian","Maximillia","Maximillian","Maximo","Maximus","Maxine","Maxwell","May","Maya","Maybell","Maybelle","Maye","Maymie","Maynard","Mayra","Mazie","Mckayla","Mckenna","Mckenzie","Meagan","Meaghan","Meda","Megane","Meggie","Meghan","Mekhi","Melany","Melba","Melisa","Melissa","Mellie","Melody","Melvin","Melvina","Melyna","Melyssa","Mercedes","Meredith","Merl","Merle","Merlin","Merritt","Mertie","Mervin","Meta","Mia","Micaela","Micah","Michael","Michaela","Michale","Micheal","Michel","Michele","Michelle","Miguel","Mikayla","Mike","Mikel","Milan","Miles","Milford","Miller","Millie","Milo","Milton","Mina","Minerva","Minnie","Miracle","Mireille","Mireya","Misael","Missouri","Misty","Mitchel","Mitchell","Mittie","Modesta","Modesto","Mohamed","Mohammad","Mohammed","Moises","Mollie","Molly","Mona","Monica","Monique","Monroe","Monserrat","Monserrate","Montana","Monte","Monty","Morgan","Moriah","Morris","Mortimer","Morton","Mose","Moses","Moshe","Mossie","Mozell","Mozelle","Muhammad","Muriel","Murl","Murphy","Murray","Mustafa","Mya","Myah","Mylene","Myles","Myra","Myriam","Myrl","Myrna","Myron","Myrtice","Myrtie","Myrtis","Myrtle","Nadia","Nakia","Name","Nannie","Naomi","Naomie","Napoleon","Narciso","Nash","Nasir","Nat","Natalia","Natalie","Natasha","Nathan","Nathanael","Nathanial","Nathaniel","Nathen","Nayeli","Neal","Ned","Nedra","Neha","Neil","Nelda","Nella","Nelle","Nellie","Nels","Nelson","Neoma","Nestor","Nettie","Neva","Newell","Newton","Nia","Nicholas","Nicholaus","Nichole","Nick","Nicklaus","Nickolas","Nico","Nicola","Nicolas","Nicole","Nicolette","Nigel","Nikita","Nikki","Nikko","Niko","Nikolas","Nils","Nina","Noah","Noble","Noe","Noel","Noelia","Noemi","Noemie","Noemy","Nola","Nolan","Nona","Nora","Norbert","Norberto","Norene","Norma","Norris","Norval","Norwood","Nova","Novella","Nya","Nyah","Nyasia","Obie","Oceane","Ocie","Octavia","Oda","Odell","Odessa","Odie","Ofelia","Okey","Ola","Olaf","Ole","Olen","Oleta","Olga","Olin","Oliver","Ollie","Oma","Omari","Omer","Ona","Onie","Opal","Ophelia","Ora","Oral","Oran","Oren","Orie","Orin","Orion","Orland","Orlando","Orlo","Orpha","Orrin","Orval","Orville","Osbaldo","Osborne","Oscar","Osvaldo","Oswald","Oswaldo","Otha","Otho","Otilia","Otis","Ottilie","Ottis","Otto","Ova","Owen","Ozella","Pablo","Paige","Palma","Pamela","Pansy","Paolo","Paris","Parker","Pascale","Pasquale","Pat","Patience","Patricia","Patrick","Patsy","Pattie","Paul","Paula","Pauline","Paxton","Payton","Pearl","Pearlie","Pearline","Pedro","Peggie","Penelope","Percival","Percy","Perry","Pete","Peter","Petra","Peyton","Philip","Phoebe","Phyllis","Pierce","Pierre","Pietro","Pink","Pinkie","Piper","Polly","Porter","Precious","Presley","Preston","Price","Prince","Princess","Priscilla","Providenci","Prudence","Queen","Queenie","Quentin","Quincy","Quinn","Quinten","Quinton","Rachael","Rachel","Rachelle","Rae","Raegan","Rafael","Rafaela","Raheem","Rahsaan","Rahul","Raina","Raleigh","Ralph","Ramiro","Ramon","Ramona","Randal","Randall","Randi","Randy","Ransom","Raoul","Raphael","Raphaelle","Raquel","Rashad","Rashawn","Rasheed","Raul","Raven","Ray","Raymond","Raymundo","Reagan","Reanna","Reba","Rebeca","Rebecca","Rebeka","Rebekah","Reece","Reed","Reese","Regan","Reggie","Reginald","Reid","Reilly","Reina","Reinhold","Remington","Rene","Renee","Ressie","Reta","Retha","Retta","Reuben","Reva","Rex","Rey","Reyes","Reymundo","Reyna","Reynold","Rhea","Rhett","Rhianna","Rhiannon","Rhoda","Ricardo","Richard","Richie","Richmond","Rick","Rickey","Rickie","Ricky","Rico","Rigoberto","Riley","Rita","River","Robb","Robbie","Robert","Roberta","Roberto","Robin","Robyn","Rocio","Rocky","Rod","Roderick","Rodger","Rodolfo","Rodrick","Rodrigo","Roel","Rogelio","Roger","Rogers","Rolando","Rollin","Roma","Romaine","Roman","Ron","Ronaldo","Ronny","Roosevelt","Rory","Rosa","Rosalee","Rosalia","Rosalind","Rosalinda","Rosalyn","Rosamond","Rosanna","Rosario","Roscoe","Rose","Rosella","Roselyn","Rosemarie","Rosemary","Rosendo","Rosetta","Rosie","Rosina","Roslyn","Ross","Rossie","Rowan","Rowena","Rowland","Roxane","Roxanne","Roy","Royal","Royce","Rozella","Ruben","Rubie","Ruby","Rubye","Rudolph","Rudy","Rupert","Russ","Russel","Russell","Rusty","Ruth","Ruthe","Ruthie","Ryan","Ryann","Ryder","Rylan","Rylee","Ryleigh","Ryley","Sabina","Sabrina","Sabryna","Sadie","Sadye","Sage","Saige","Sallie","Sally","Salma","Salvador","Salvatore","Sam","Samanta","Samantha","Samara","Samir","Sammie","Sammy","Samson","Sandra","Sandrine","Sandy","Sanford","Santa","Santiago","Santina","Santino","Santos","Sarah","Sarai","Sarina","Sasha","Saul","Savanah","Savanna","Savannah","Savion","Scarlett","Schuyler","Scot","Scottie","Scotty","Seamus","Sean","Sebastian","Sedrick","Selena","Selina","Selmer","Serena","Serenity","Seth","Shad","Shaina","Shakira","Shana","Shane","Shanel","Shanelle","Shania","Shanie","Shaniya","Shanna","Shannon","Shanny","Shanon","Shany","Sharon","Shaun","Shawn","Shawna","Shaylee","Shayna","Shayne","Shea","Sheila","Sheldon","Shemar","Sheridan","Sherman","Sherwood","Shirley","Shyann","Shyanne","Sibyl","Sid","Sidney","Sienna","Sierra","Sigmund","Sigrid","Sigurd","Silas","Sim","Simeon","Simone","Sincere","Sister","Skye","Skyla","Skylar","Sofia","Soledad","Solon","Sonia","Sonny","Sonya","Sophia","Sophie","Spencer","Stacey","Stacy","Stan","Stanford","Stanley","Stanton","Stefan","Stefanie","Stella","Stephan","Stephania","Stephanie","Stephany","Stephen","Stephon","Sterling","Steve","Stevie","Stewart","Stone","Stuart","Summer","Sunny","Susan","Susana","Susanna","Susie","Suzanne","Sven","Syble","Sydnee","Sydney","Sydni","Sydnie","Sylvan","Sylvester","Sylvia","Tabitha","Tad","Talia","Talon","Tamara","Tamia","Tania","Tanner","Tanya","Tara","Taryn","Tate","Tatum","Tatyana","Taurean","Tavares","Taya","Taylor","Teagan","Ted","Telly","Terence","Teresa","Terrance","Terrell","Terrence","Terrill","Terry","Tess","Tessie","Tevin","Thad","Thaddeus","Thalia","Thea","Thelma","Theo","Theodora","Theodore","Theresa","Therese","Theresia","Theron","Thomas","Thora","Thurman","Tia","Tiana","Tianna","Tiara","Tierra","Tiffany","Tillman","Timmothy","Timmy","Timothy","Tina","Tito","Titus","Tobin","Toby","Tod","Tom","Tomas","Tomasa","Tommie","Toney","Toni","Tony","Torey","Torrance","Torrey","Toy","Trace","Tracey","Tracy","Travis","Travon","Tre","Tremaine","Tremayne","Trent","Trenton","Tressa","Tressie","Treva","Trever","Trevion","Trevor","Trey","Trinity","Trisha","Tristian","Tristin","Triston","Troy","Trudie","Trycia","Trystan","Turner","Twila","Tyler","Tyra","Tyree","Tyreek","Tyrel","Tyrell","Tyrese","Tyrique","Tyshawn","Tyson","Ubaldo","Ulices","Ulises","Una","Unique","Urban","Uriah","Uriel","Ursula","Vada","Valentin","Valentina","Valentine","Valerie","Vallie","Van","Vance","Vanessa","Vaughn","Veda","Velda","Vella","Velma","Velva","Vena","Verda","Verdie","Vergie","Verla","Verlie","Vern","Verna","Verner","Vernice","Vernie","Vernon","Verona","Veronica","Vesta","Vicenta","Vicente","Vickie","Vicky","Victor","Victoria","Vida","Vidal","Vilma","Vince","Vincent","Vincenza","Vincenzo","Vinnie","Viola","Violet","Violette","Virgie","Virgil","Virginia","Virginie","Vita","Vito","Viva","Vivian","Viviane","Vivianne","Vivien","Vivienne","Vladimir","Wade","Waino","Waldo","Walker","Wallace","Walter","Walton","Wanda","Ward","Warren","Watson","Wava","Waylon","Wayne","Webster","Weldon","Wellington","Wendell","Wendy","Werner","Westley","Weston","Whitney","Wilber","Wilbert","Wilburn","Wiley","Wilford","Wilfred","Wilfredo","Wilfrid","Wilhelm","Wilhelmine","Will","Willa","Willard","William","Willie","Willis","Willow","Willy","Wilma","Wilmer","Wilson","Wilton","Winfield","Winifred","Winnifred","Winona","Winston","Woodrow","Wyatt","Wyman","Xander","Xavier","Xzavier","Yadira","Yasmeen","Yasmin","Yasmine","Yazmin","Yesenia","Yessenia","Yolanda","Yoshiko","Yvette","Yvonne","Zachariah","Zachary","Zachery","Zack","Zackary","Zackery","Zakary","Zander","Zane","Zaria","Zechariah","Zelda","Zella","Zelma","Zena","Zetta","Zion","Zita","Zoe","Zoey","Zoie","Zoila","Zola","Zora","Zula"],female:["Ada","Adrienne","Agnes","Alberta","Alexandra","Alexis","Alice","Alicia","Alison","Allison","Alma","Alyssa","Amanda","Amber","Amelia","Amy","Ana","Andrea","Angel","Angela","Angelica","Angelina","Angie","Anita","Ann","Anna","Anne","Annette","Annie","Antoinette","Antonia","April","Arlene","Ashley","Audrey","Barbara","Beatrice","Becky","Belinda","Bernadette","Bernice","Bertha","Bessie","Beth","Bethany","Betsy","Betty","Beulah","Beverly","Billie","Blanca","Blanche","Bobbie","Bonnie","Brandi","Brandy","Brenda","Bridget","Brittany","Brooke","Camille","Candace","Candice","Carla","Carmen","Carol","Carole","Caroline","Carolyn","Carrie","Casey","Cassandra","Catherine","Cathy","Cecelia","Cecilia","Celia","Charlene","Charlotte","Chelsea","Cheryl","Christie","Christina","Christine","Christy","Cindy","Claire","Clara","Claudia","Colleen","Connie","Constance","Cora","Courtney","Cristina","Crystal","Cynthia","Daisy","Dana","Danielle","Darla","Darlene","Dawn","Deanna","Debbie","Deborah","Debra","Delia","Della","Delores","Denise","Desiree","Diana","Diane","Dianna","Dianne","Dixie","Dolores","Donna","Dora","Doreen","Doris","Dorothy","Ebony","Edith","Edna","Eileen","Elaine","Eleanor","Elena","Elisa","Elizabeth","Ella","Ellen","Eloise","Elsa","Elsie","Elvira","Emily","Emma","Erica","Erika","Erin","Erma","Ernestine","Essie","Estelle","Esther","Ethel","Eula","Eunice","Eva","Evelyn","Faith","Fannie","Faye","Felicia","Flora","Florence","Frances","Francis","Freda","Gail","Gayle","Geneva","Genevieve","Georgia","Geraldine","Gertrude","Gina","Ginger","Gladys","Glenda","Gloria","Grace","Gretchen","Guadalupe","Gwen","Gwendolyn","Hannah","Harriet","Hattie","Hazel","Heather","Heidi","Helen","Henrietta","Hilda","Holly","Hope","Ida","Inez","Irene","Iris","Irma","Isabel","Jackie","Jacqueline","Jacquelyn","Jaime","Jamie","Jan","Jana","Jane","Janet","Janice","Janie","Janis","Jasmine","Jean","Jeanette","Jeanne","Jeannette","Jeannie","Jenna","Jennie","Jennifer","Jenny","Jessica","Jessie","Jill","Jo","Joan","Joann","Joanna","Joanne","Jodi","Jody","Johanna","Johnnie","Josefina","Josephine","Joy","Joyce","Juana","Juanita","Judith","Judy","Julia","Julie","June","Kara","Karen","Kari","Karla","Kate","Katherine","Kathleen","Kathryn","Kathy","Katie","Katrina","Kay","Kayla","Kelley","Kelli","Kellie","Kelly","Kendra","Kerry","Kim","Kimberly","Krista","Kristen","Kristi","Kristie","Kristin","Kristina","Kristine","Kristy","Krystal","Lana","Latoya","Laura","Lauren","Laurie","Laverne","Leah","Lee","Leigh","Lela","Lena","Leona","Leslie","Leticia","Lila","Lillian","Lillie","Linda","Lindsay","Lindsey","Lisa","Lois","Lola","Lora","Lorena","Lorene","Loretta","Lori","Lorraine","Louise","Lucia","Lucille","Lucy","Lula","Luz","Lydia","Lynda","Lynette","Lynn","Lynne","Mabel","Mable","Madeline","Mae","Maggie","Mamie","Mandy","Marcella","Marcia","Margaret","Margarita","Margie","Marguerite","Maria","Marian","Marianne","Marie","Marilyn","Marion","Marjorie","Marlene","Marsha","Marta","Martha","Mary","Maryann","Mattie","Maureen","Maxine","May","Megan","Meghan","Melanie","Melba","Melinda","Melissa","Melody","Mercedes","Meredith","Michele","Michelle","Mildred","Mindy","Minnie","Miranda","Miriam","Misty","Molly","Mona","Monica","Monique","Muriel","Myra","Myrtle","Nadine","Nancy","Naomi","Natalie","Natasha","Nellie","Nettie","Nichole","Nicole","Nina","Nora","Norma","Olga","Olive","Olivia","Ollie","Opal","Ora","Pam","Pamela","Pat","Patricia","Patsy","Patti","Patty","Paula","Paulette","Pauline","Pearl","Peggy","Penny","Phyllis","Priscilla","Rachael","Rachel","Ramona","Raquel","Rebecca","Regina","Renee","Rhonda","Rita","Roberta","Robin","Robyn","Rochelle","Rosa","Rosalie","Rose","Rosemarie","Rosemary","Rosie","Roxanne","Ruby","Ruth","Sabrina","Sadie","Sally","Samantha","Sandra","Sandy","Sara","Sarah","Shannon","Shari","Sharon","Shawna","Sheila","Shelia","Shelley","Shelly","Sheri","Sherri","Sherry","Sheryl","Shirley","Silvia","Sonia","Sonja","Sonya","Sophia","Sophie","Stacey","Stacy","Stella","Stephanie","Sue","Susan","Susie","Suzanne","Sylvia","Tabitha","Tamara","Tami","Tammy","Tanya","Tara","Tasha","Teresa","Teri","Terri","Terry","Thelma","Theresa","Tiffany","Tina","Toni","Tonya","Tracey","Traci","Tracy","Tricia","Valerie","Vanessa","Velma","Vera","Verna","Veronica","Vicki","Vickie","Vicky","Victoria","Viola","Violet","Virginia","Vivian","Wanda","Wendy","Whitney","Willie","Wilma","Winifred","Yolanda","Yvette","Yvonne"],male:["Aaron","Abel","Abraham","Adam","Adrian","Al","Alan","Albert","Alberto","Alejandro","Alex","Alexander","Alfonso","Alfred","Alfredo","Allan","Allen","Alonzo","Alton","Alvin","Amos","Andre","Andres","Andrew","Andy","Angel","Angelo","Anthony","Antonio","Archie","Armando","Arnold","Arthur","Arturo","Aubrey","Austin","Barry","Ben","Benjamin","Bennie","Benny","Bernard","Bert","Bill","Billy","Blake","Bob","Bobby","Boyd","Brad","Bradford","Bradley","Brandon","Brendan","Brent","Brett","Brian","Bruce","Bryan","Bryant","Byron","Caleb","Calvin","Cameron","Carl","Carlos","Carlton","Carroll","Cary","Casey","Cecil","Cedric","Cesar","Chad","Charles","Charlie","Chester","Chris","Christian","Christopher","Clarence","Clark","Claude","Clay","Clayton","Clifford","Clifton","Clint","Clinton","Clyde","Cody","Colin","Conrad","Corey","Cornelius","Cory","Courtney","Craig","Curtis","Dale","Dallas","Damon","Dan","Dana","Daniel","Danny","Darin","Darnell","Darrel","Darrell","Darren","Darrin","Darryl","Daryl","Dave","David","Dean","Delbert","Dennis","Derek","Derrick","Devin","Dewey","Dexter","Domingo","Dominic","Dominick","Don","Donald","Donnie","Doug","Douglas","Doyle","Drew","Duane","Dustin","Dwayne","Dwight","Earl","Earnest","Ed","Eddie","Edgar","Edmond","Edmund","Eduardo","Edward","Edwin","Elbert","Elias","Elijah","Ellis","Elmer","Emanuel","Emilio","Emmett","Enrique","Eric","Erick","Erik","Ernest","Ernesto","Ervin","Eugene","Evan","Everett","Felipe","Felix","Fernando","Floyd","Forrest","Francis","Francisco","Frank","Frankie","Franklin","Fred","Freddie","Frederick","Fredrick","Gabriel","Garrett","Garry","Gary","Gene","Geoffrey","George","Gerald","Gerard","Gerardo","Gilbert","Gilberto","Glen","Glenn","Gordon","Grady","Grant","Greg","Gregg","Gregory","Guadalupe","Guillermo","Gustavo","Guy","Harold","Harry","Harvey","Hector","Henry","Herbert","Herman","Homer","Horace","Howard","Hubert","Hugh","Hugo","Ian","Ignacio","Ira","Irvin","Irving","Isaac","Ismael","Israel","Ivan","Jack","Jackie","Jacob","Jaime","Jake","James","Jamie","Jan","Jared","Jason","Javier","Jay","Jean","Jeff","Jeffery","Jeffrey","Jerald","Jeremiah","Jeremy","Jermaine","Jerome","Jerry","Jesse","Jessie","Jesus","Jim","Jimmie","Jimmy","Jody","Joe","Joel","Joey","John","Johnathan","Johnnie","Johnny","Jon","Jonathan","Jonathon","Jordan","Jorge","Jose","Joseph","Josh","Joshua","Juan","Julian","Julio","Julius","Justin","Karl","Keith","Kelly","Kelvin","Ken","Kenneth","Kenny","Kent","Kerry","Kevin","Kim","Kirk","Kristopher","Kurt","Kyle","Lamar","Lance","Larry","Laurence","Lawrence","Lee","Leland","Leo","Leon","Leonard","Leroy","Leslie","Lester","Levi","Lewis","Lionel","Lloyd","Lonnie","Loren","Lorenzo","Louis","Lowell","Lucas","Luis","Luke","Luther","Lyle","Lynn","Mack","Malcolm","Manuel","Marc","Marco","Marcos","Marcus","Mario","Marion","Mark","Marlon","Marshall","Martin","Marty","Marvin","Mathew","Matt","Matthew","Maurice","Max","Melvin","Merle","Michael","Micheal","Miguel","Mike","Milton","Mitchell","Morris","Moses","Myron","Nathan","Nathaniel","Neal","Neil","Nelson","Nicholas","Nick","Nicolas","Noah","Noel","Norman","Oliver","Omar","Orlando","Orville","Oscar","Otis","Owen","Pablo","Pat","Patrick","Paul","Pedro","Percy","Perry","Pete","Peter","Phil","Philip","Phillip","Preston","Rafael","Ralph","Ramiro","Ramon","Randal","Randall","Randolph","Randy","Raul","Ray","Raymond","Reginald","Rene","Rex","Ricardo","Richard","Rick","Rickey","Ricky","Robert","Roberto","Robin","Roderick","Rodney","Rodolfo","Rogelio","Roger","Roland","Rolando","Roman","Ron","Ronald","Ronnie","Roosevelt","Ross","Roy","Ruben","Rudolph","Rudy","Rufus","Russell","Ryan","Salvador","Salvatore","Sam","Sammy","Samuel","Santiago","Santos","Saul","Scott","Sean","Sergio","Seth","Shane","Shannon","Shaun","Shawn","Sheldon","Sherman","Sidney","Simon","Spencer","Stanley","Stephen","Steve","Steven","Stewart","Stuart","Sylvester","Taylor","Ted","Terence","Terrance","Terrell","Terrence","Terry","Theodore","Thomas","Tim","Timmy","Timothy","Toby","Todd","Tom","Tomas","Tommie","Tommy","Tony","Tracy","Travis","Trevor","Troy","Tyler","Tyrone","Van","Vernon","Victor","Vincent","Virgil","Wade","Wallace","Walter","Warren","Wayne","Wendell","Wesley","Wilbert","Wilbur","Wilfred","Willard","William","Willie","Willis","Wilson","Winston","Wm","Woodrow","Zachary"]},U4e=["Agender","Androgyne","Androgynous","Bigender","Cis female","Cis male","Cis man","Cis woman","Cis","Cisgender female","Cisgender male","Cisgender man","Cisgender woman","Cisgender","Demi-boy","Demi-girl","Demi-man","Demi-woman","Demiflux","Demigender","F2M","FTM","Female to male trans man","Female to male transgender man","Female to male transsexual man","Female to male","Gender fluid","Gender neutral","Gender nonconforming","Gender questioning","Gender variant","Genderflux","Genderqueer","Hermaphrodite","Intersex man","Intersex person","Intersex woman","Intersex","M2F","MTF","Male to female trans woman","Male to female transgender woman","Male to female transsexual woman","Male to female","Man","Multigender","Neither","Neutrois","Non-binary","Omnigender","Other","Pangender","Polygender","T* man","T* woman","Trans female","Trans male","Trans man","Trans person","Trans woman","Trans","Transsexual female","Transsexual male","Transsexual man","Transsexual person","Transsexual woman","Transsexual","Transgender female","Transgender person","Transmasculine","Trigender","Two* person","Two-spirit person","Two-spirit","Woman","Xenogender"],$4e=["Solutions","Program","Brand","Security","Research","Marketing","Directives","Implementation","Integration","Functionality","Response","Paradigm","Tactics","Identity","Markets","Group","Division","Applications","Optimization","Operations","Infrastructure","Intranet","Communications","Web","Branding","Quality","Assurance","Mobility","Accounts","Data","Creative","Configuration","Accountability","Interactions","Factors","Usability","Metrics"],G4e=["Lead","Senior","Direct","Corporate","Dynamic","Future","Product","National","Regional","District","Central","Global","Customer","Investor","International","Legacy","Forward","Internal","Human","Chief","Principal"],j4e=["{{person.jobDescriptor}} {{person.jobArea}} {{person.jobType}}"],q4e=["Supervisor","Associate","Executive","Liaison","Officer","Manager","Engineer","Specialist","Director","Coordinator","Administrator","Architect","Analyst","Designer","Planner","Orchestrator","Technician","Developer","Producer","Consultant","Assistant","Facilitator","Agent","Representative","Strategist"],W4e={generic:["Abbott","Abernathy","Abshire","Adams","Altenwerth","Anderson","Ankunding","Armstrong","Auer","Aufderhar","Bahringer","Bailey","Balistreri","Barrows","Bartell","Bartoletti","Barton","Bashirian","Batz","Bauch","Baumbach","Bayer","Beahan","Beatty","Bechtelar","Becker","Bednar","Beer","Beier","Berge","Bergnaum","Bergstrom","Bernhard","Bernier","Bins","Blanda","Blick","Block","Bode","Boehm","Bogan","Bogisich","Borer","Bosco","Botsford","Boyer","Boyle","Bradtke","Brakus","Braun","Breitenberg","Brekke","Brown","Bruen","Buckridge","Carroll","Carter","Cartwright","Casper","Cassin","Champlin","Christiansen","Cole","Collier","Collins","Conn","Connelly","Conroy","Considine","Corkery","Cormier","Corwin","Cremin","Crist","Crona","Cronin","Crooks","Cruickshank","Cummerata","Cummings","D'Amore","Dach","Daniel","Dare","Daugherty","Davis","Deckow","Denesik","Dibbert","Dickens","Dicki","Dickinson","Dietrich","Donnelly","Dooley","Douglas","Doyle","DuBuque","Durgan","Ebert","Effertz","Emard","Emmerich","Erdman","Ernser","Fadel","Fahey","Farrell","Fay","Feeney","Feest","Feil","Ferry","Fisher","Flatley","Frami","Franecki","Franey","Friesen","Fritsch","Funk","Gerhold","Gerlach","Gibson","Gislason","Gleason","Gleichner","Glover","Goldner","Goodwin","Gorczany","Gottlieb","Goyette","Grady","Graham","Grant","Green","Greenfelder","Greenholt","Grimes","Gulgowski","Gusikowski","Gutkowski","Gutmann","Haag","Hackett","Hagenes","Hahn","Haley","Halvorson","Hamill","Hammes","Hand","Hane","Hansen","Harber","Harris","Hartmann","Harvey","Hauck","Hayes","Heaney","Heathcote","Hegmann","Heidenreich","Heller","Herman","Hermann","Hermiston","Herzog","Hessel","Hettinger","Hickle","Hilll","Hills","Hilpert","Hintz","Hirthe","Hodkiewicz","Hoeger","Homenick","Hoppe","Howe","Howell","Hudson","Huel","Huels","Hyatt","Jacobi","Jacobs","Jacobson","Jakubowski","Jaskolski","Jast","Jenkins","Jerde","Johns","Johnson","Johnston","Jones","Kassulke","Kautzer","Keebler","Keeling","Kemmer","Kerluke","Kertzmann","Kessler","Kiehn","Kihn","Kilback","King","Kirlin","Klein","Kling","Klocko","Koch","Koelpin","Koepp","Kohler","Konopelski","Koss","Kovacek","Kozey","Krajcik","Kreiger","Kris","Kshlerin","Kub","Kuhic","Kuhlman","Kuhn","Kulas","Kunde","Kunze","Kuphal","Kutch","Kuvalis","Labadie","Lakin","Lang","Langosh","Langworth","Larkin","Larson","Leannon","Lebsack","Ledner","Leffler","Legros","Lehner","Lemke","Lesch","Leuschke","Lind","Lindgren","Littel","Little","Lockman","Lowe","Lubowitz","Lueilwitz","Luettgen","Lynch","MacGyver","Macejkovic","Maggio","Mann","Mante","Marks","Marquardt","Marvin","Mayer","Mayert","McClure","McCullough","McDermott","McGlynn","McKenzie","McLaughlin","Medhurst","Mertz","Metz","Miller","Mills","Mitchell","Moen","Mohr","Monahan","Moore","Morar","Morissette","Mosciski","Mraz","Mueller","Muller","Murazik","Murphy","Murray","Nader","Nicolas","Nienow","Nikolaus","Nitzsche","Nolan","O'Connell","O'Conner","O'Hara","O'Keefe","O'Kon","O'Reilly","Oberbrunner","Okuneva","Olson","Ondricka","Orn","Ortiz","Osinski","Pacocha","Padberg","Pagac","Parisian","Parker","Paucek","Pfannerstill","Pfeffer","Pollich","Pouros","Powlowski","Predovic","Price","Prohaska","Prosacco","Purdy","Quigley","Quitzon","Rath","Ratke","Rau","Raynor","Reichel","Reichert","Reilly","Reinger","Rempel","Renner","Reynolds","Rice","Rippin","Ritchie","Robel","Roberts","Rodriguez","Rogahn","Rohan","Rolfson","Romaguera","Roob","Rosenbaum","Rowe","Ruecker","Runolfsdottir","Runolfsson","Runte","Russel","Rutherford","Ryan","Sanford","Satterfield","Sauer","Sawayn","Schaden","Schaefer","Schamberger","Schiller","Schimmel","Schinner","Schmeler","Schmidt","Schmitt","Schneider","Schoen","Schowalter","Schroeder","Schulist","Schultz","Schumm","Schuppe","Schuster","Senger","Shanahan","Shields","Simonis","Sipes","Skiles","Smith","Smitham","Spencer","Spinka","Sporer","Stamm","Stanton","Stark","Stehr","Steuber","Stiedemann","Stokes","Stoltenberg","Stracke","Streich","Stroman","Strosin","Swaniawski","Swift","Terry","Thiel","Thompson","Tillman","Torp","Torphy","Towne","Toy","Trantow","Tremblay","Treutel","Tromp","Turcotte","Turner","Ullrich","Upton","Vandervort","Veum","Volkman","Von","VonRueden","Waelchi","Walker","Walsh","Walter","Ward","Waters","Watsica","Weber","Wehner","Weimann","Weissnat","Welch","West","White","Wiegand","Wilderman","Wilkinson","Will","Williamson","Willms","Windler","Wintheiser","Wisoky","Wisozk","Witting","Wiza","Wolf","Wolff","Wuckert","Wunsch","Wyman","Yost","Yundt","Zboncak","Zemlak","Ziemann","Zieme","Zulauf"]},V4e={generic:[{value:"{{person.last_name.generic}}",weight:95},{value:"{{person.last_name.generic}}-{{person.last_name.generic}}",weight:5}]},K4e={generic:["Addison","Alex","Anderson","Angel","Arden","August","Austin","Avery","Bailey","Billie","Blake","Bowie","Brooklyn","Cameron","Charlie","Corey","Dakota","Drew","Elliott","Ellis","Emerson","Finley","Gray","Greer","Harper","Hayden","Jaden","James","Jamie","Jordan","Jules","Kai","Kendall","Kennedy","Kyle","Leslie","Logan","London","Marlowe","Micah","Nico","Noah","North","Parker","Phoenix","Quinn","Reagan","Reese","Reign","Riley","River","Robin","Rory","Rowan","Ryan","Sage","Sasha","Sawyer","Shawn","Shiloh","Skyler","Taylor"],female:["Abigail","Adele","Alex","Alice","Alisha","Amber","Amelia","Amora","Anaïs","Angelou","Anika","Anise","Annabel","Anne","Aphrodite","Aretha","Arya","Ashton","Aster","Audrey","Avery","Bailee","Bay","Belle","Beth","Billie","Blair","Blaise","Blake","Blanche","Blue","Bree","Brielle","Brienne","Brooke","Caleen","Candice","Caprice","Carelyn","Caylen","Celine","Cerise","Cia","Claire","Claudia","Clementine","Coral","Coraline","Dahlia","Dakota","Dawn","Della","Demi","Denise","Denver","Devine","Devon","Diana","Dylan","Ebony","Eden","Eleanor","Elein","Elizabeth","Ellen","Elodie","Eloise","Ember","Emma","Erin","Eyre","Faith","Farrah","Fawn","Fayre","Fern","France","Francis","Frida","Genisis","Georgia","Grace","Gwen","Harley","Harper","Hazel","Helen","Hippolyta","Holly","Hope","Imani","Iowa","Ireland","Irene","Iris","Isa","Isla","Ivy","Jade","Jane","Jazz","Jean","Jess","Jett","Jo","Joan","Jolie","Jordan","Josie","Journey","Joy","Jules","Julien","Juliet","Juniper","Justice","Kali","Karma","Kat","Kate","Kennedy","Keva","Kylie","Lake","Lane","Lark","Layla","Lee","Leigh","Leona","Lexi","London","Lou","Louise","Love","Luna","Lux","Lynn","Lyric","Maddie","Mae","Marie","Matilda","Maude","Maybel","Meadow","Medusa","Mercy","Michelle","Mirabel","Monroe","Morgan","Nalia","Naomi","Nova","Olive","Paige","Parker","Pax","Pearl","Penelope","Phoenix","Quinn","Rae","Rain","Raven","Ray","Raye","Rebel","Reese","Reeve","Regan","Riley","River","Robin","Rory","Rose","Royal","Ruth","Rylie","Sage","Sam","Saturn","Scout","Serena","Sky","Skylar","Sofia","Sophia","Storm","Sue","Suzanne","Sydney","Taylen","Taylor","Teagan","Tempest","Tenley","Thea","Trinity","Valerie","Venus","Vera","Violet","Willow","Winter","Xena","Zaylee","Zion","Zoe"],male:["Ace","Aiden","Alexander","Ander","Anthony","Asher","August","Aziel","Bear","Beckham","Benjamin","Buddy","Calvin","Carter","Charles","Christopher","Clyde","Cooper","Daniel","David","Dior","Dylan","Elijah","Ellis","Emerson","Ethan","Ezra","Fletcher","Flynn","Gabriel","Grayson","Gus","Hank","Harrison","Hendrix","Henry","Houston","Hudson","Hugh","Isaac","Jack","Jackson","Jacob","Jakobe","James","Jaxon","Jaxtyn","Jayden","John","Joseph","Josiah","Jude","Julian","Karsyn","Kenji","Kobe","Kylo","Lennon","Leo","Levi","Liam","Lincoln","Logan","Louis","Lucas","Lucky","Luke","Mason","Mateo","Matthew","Maverick","Michael","Monroe","Nixon","Ocean","Oliver","Otis","Otto","Owen","Ozzy","Parker","Rocky","Samuel","Sebastian","Sonny","Teddy","Theo","Theodore","Thomas","Truett","Walter","Warren","Watson","William","Wison","Wyatt","Ziggy","Zyair"]},Y4e=[{value:"{{person.firstName}} {{person.lastName}}",weight:49},{value:"{{person.prefix}} {{person.firstName}} {{person.lastName}}",weight:7},{value:"{{person.firstName}} {{person.lastName}} {{person.suffix}}",weight:7},{value:"{{person.prefix}} {{person.firstName}} {{person.lastName}} {{person.suffix}}",weight:1}],J4e={generic:["Dr.","Miss","Mr.","Mrs.","Ms."],female:["Dr.","Miss","Mrs.","Ms."],male:["Dr.","Mr."]},X4e=["female","male"],Z4e=["Jr.","Sr.","I","II","III","IV","V","MD","DDS","PhD","DVM"],Q4e=["Aquarius","Pisces","Aries","Taurus","Gemini","Cancer","Leo","Virgo","Libra","Scorpio","Sagittarius","Capricorn"],e_e={bio_part:B4e,bio_pattern:P4e,bio_supporter:z4e,first_name:H4e,gender:U4e,job_area:$4e,job_descriptor:G4e,job_title_pattern:j4e,job_type:q4e,last_name:W4e,last_name_pattern:V4e,middle_name:K4e,name:Y4e,prefix:J4e,sex:X4e,suffix:Z4e,western_zodiac_sign:Q4e},t_e=e_e,r_e=["!##-!##-####","(!##) !##-####","1-!##-!##-####","!##.!##.####","!##-!##-#### x###","(!##) !##-#### x###","1-!##-!##-#### x###","!##.!##.#### x###","!##-!##-#### x####","(!##) !##-#### x####","1-!##-!##-#### x####","!##.!##.#### x####","!##-!##-#### x#####","(!##) !##-#### x#####","1-!##-!##-#### x#####","!##.!##.#### x#####"],n_e=["+1!##!######"],a_e=["(!##) !##-####"],i_e={human:r_e,international:n_e,national:a_e},o_e=i_e,s_e={format:o_e},l_e=s_e,u_e=[{symbol:"H",name:"Hydrogen",atomicNumber:1},{symbol:"He",name:"Helium",atomicNumber:2},{symbol:"Li",name:"Lithium",atomicNumber:3},{symbol:"Be",name:"Beryllium",atomicNumber:4},{symbol:"B",name:"Boron",atomicNumber:5},{symbol:"C",name:"Carbon",atomicNumber:6},{symbol:"N",name:"Nitrogen",atomicNumber:7},{symbol:"O",name:"Oxygen",atomicNumber:8},{symbol:"F",name:"Fluorine",atomicNumber:9},{symbol:"Ne",name:"Neon",atomicNumber:10},{symbol:"Na",name:"Sodium",atomicNumber:11},{symbol:"Mg",name:"Magnesium",atomicNumber:12},{symbol:"Al",name:"Aluminium",atomicNumber:13},{symbol:"Si",name:"Silicon",atomicNumber:14},{symbol:"P",name:"Phosphorus",atomicNumber:15},{symbol:"S",name:"Sulfur",atomicNumber:16},{symbol:"Cl",name:"Chlorine",atomicNumber:17},{symbol:"Ar",name:"Argon",atomicNumber:18},{symbol:"K",name:"Potassium",atomicNumber:19},{symbol:"Ca",name:"Calcium",atomicNumber:20},{symbol:"Sc",name:"Scandium",atomicNumber:21},{symbol:"Ti",name:"Titanium",atomicNumber:22},{symbol:"V",name:"Vanadium",atomicNumber:23},{symbol:"Cr",name:"Chromium",atomicNumber:24},{symbol:"Mn",name:"Manganese",atomicNumber:25},{symbol:"Fe",name:"Iron",atomicNumber:26},{symbol:"Co",name:"Cobalt",atomicNumber:27},{symbol:"Ni",name:"Nickel",atomicNumber:28},{symbol:"Cu",name:"Copper",atomicNumber:29},{symbol:"Zn",name:"Zinc",atomicNumber:30},{symbol:"Ga",name:"Gallium",atomicNumber:31},{symbol:"Ge",name:"Germanium",atomicNumber:32},{symbol:"As",name:"Arsenic",atomicNumber:33},{symbol:"Se",name:"Selenium",atomicNumber:34},{symbol:"Br",name:"Bromine",atomicNumber:35},{symbol:"Kr",name:"Krypton",atomicNumber:36},{symbol:"Rb",name:"Rubidium",atomicNumber:37},{symbol:"Sr",name:"Strontium",atomicNumber:38},{symbol:"Y",name:"Yttrium",atomicNumber:39},{symbol:"Zr",name:"Zirconium",atomicNumber:40},{symbol:"Nb",name:"Niobium",atomicNumber:41},{symbol:"Mo",name:"Molybdenum",atomicNumber:42},{symbol:"Tc",name:"Technetium",atomicNumber:43},{symbol:"Ru",name:"Ruthenium",atomicNumber:44},{symbol:"Rh",name:"Rhodium",atomicNumber:45},{symbol:"Pd",name:"Palladium",atomicNumber:46},{symbol:"Ag",name:"Silver",atomicNumber:47},{symbol:"Cd",name:"Cadmium",atomicNumber:48},{symbol:"In",name:"Indium",atomicNumber:49},{symbol:"Sn",name:"Tin",atomicNumber:50},{symbol:"Sb",name:"Antimony",atomicNumber:51},{symbol:"Te",name:"Tellurium",atomicNumber:52},{symbol:"I",name:"Iodine",atomicNumber:53},{symbol:"Xe",name:"Xenon",atomicNumber:54},{symbol:"Cs",name:"Caesium",atomicNumber:55},{symbol:"Ba",name:"Barium",atomicNumber:56},{symbol:"La",name:"Lanthanum",atomicNumber:57},{symbol:"Ce",name:"Cerium",atomicNumber:58},{symbol:"Pr",name:"Praseodymium",atomicNumber:59},{symbol:"Nd",name:"Neodymium",atomicNumber:60},{symbol:"Pm",name:"Promethium",atomicNumber:61},{symbol:"Sm",name:"Samarium",atomicNumber:62},{symbol:"Eu",name:"Europium",atomicNumber:63},{symbol:"Gd",name:"Gadolinium",atomicNumber:64},{symbol:"Tb",name:"Terbium",atomicNumber:65},{symbol:"Dy",name:"Dysprosium",atomicNumber:66},{symbol:"Ho",name:"Holmium",atomicNumber:67},{symbol:"Er",name:"Erbium",atomicNumber:68},{symbol:"Tm",name:"Thulium",atomicNumber:69},{symbol:"Yb",name:"Ytterbium",atomicNumber:70},{symbol:"Lu",name:"Lutetium",atomicNumber:71},{symbol:"Hf",name:"Hafnium",atomicNumber:72},{symbol:"Ta",name:"Tantalum",atomicNumber:73},{symbol:"W",name:"Tungsten",atomicNumber:74},{symbol:"Re",name:"Rhenium",atomicNumber:75},{symbol:"Os",name:"Osmium",atomicNumber:76},{symbol:"Ir",name:"Iridium",atomicNumber:77},{symbol:"Pt",name:"Platinum",atomicNumber:78},{symbol:"Au",name:"Gold",atomicNumber:79},{symbol:"Hg",name:"Mercury",atomicNumber:80},{symbol:"Tl",name:"Thallium",atomicNumber:81},{symbol:"Pb",name:"Lead",atomicNumber:82},{symbol:"Bi",name:"Bismuth",atomicNumber:83},{symbol:"Po",name:"Polonium",atomicNumber:84},{symbol:"At",name:"Astatine",atomicNumber:85},{symbol:"Rn",name:"Radon",atomicNumber:86},{symbol:"Fr",name:"Francium",atomicNumber:87},{symbol:"Ra",name:"Radium",atomicNumber:88},{symbol:"Ac",name:"Actinium",atomicNumber:89},{symbol:"Th",name:"Thorium",atomicNumber:90},{symbol:"Pa",name:"Protactinium",atomicNumber:91},{symbol:"U",name:"Uranium",atomicNumber:92},{symbol:"Np",name:"Neptunium",atomicNumber:93},{symbol:"Pu",name:"Plutonium",atomicNumber:94},{symbol:"Am",name:"Americium",atomicNumber:95},{symbol:"Cm",name:"Curium",atomicNumber:96},{symbol:"Bk",name:"Berkelium",atomicNumber:97},{symbol:"Cf",name:"Californium",atomicNumber:98},{symbol:"Es",name:"Einsteinium",atomicNumber:99},{symbol:"Fm",name:"Fermium",atomicNumber:100},{symbol:"Md",name:"Mendelevium",atomicNumber:101},{symbol:"No",name:"Nobelium",atomicNumber:102},{symbol:"Lr",name:"Lawrencium",atomicNumber:103},{symbol:"Rf",name:"Rutherfordium",atomicNumber:104},{symbol:"Db",name:"Dubnium",atomicNumber:105},{symbol:"Sg",name:"Seaborgium",atomicNumber:106},{symbol:"Bh",name:"Bohrium",atomicNumber:107},{symbol:"Hs",name:"Hassium",atomicNumber:108},{symbol:"Mt",name:"Meitnerium",atomicNumber:109},{symbol:"Ds",name:"Darmstadtium",atomicNumber:110},{symbol:"Rg",name:"Roentgenium",atomicNumber:111},{symbol:"Cn",name:"Copernicium",atomicNumber:112},{symbol:"Nh",name:"Nihonium",atomicNumber:113},{symbol:"Fl",name:"Flerovium",atomicNumber:114},{symbol:"Mc",name:"Moscovium",atomicNumber:115},{symbol:"Lv",name:"Livermorium",atomicNumber:116},{symbol:"Ts",name:"Tennessine",atomicNumber:117},{symbol:"Og",name:"Oganesson",atomicNumber:118}],c_e=[{name:"meter",symbol:"m"},{name:"second",symbol:"s"},{name:"mole",symbol:"mol"},{name:"ampere",symbol:"A"},{name:"kelvin",symbol:"K"},{name:"candela",symbol:"cd"},{name:"kilogram",symbol:"kg"},{name:"radian",symbol:"rad"},{name:"hertz",symbol:"Hz"},{name:"newton",symbol:"N"},{name:"pascal",symbol:"Pa"},{name:"joule",symbol:"J"},{name:"watt",symbol:"W"},{name:"coulomb",symbol:"C"},{name:"volt",symbol:"V"},{name:"ohm",symbol:"Ω"},{name:"tesla",symbol:"T"},{name:"degree Celsius",symbol:"°C"},{name:"lumen",symbol:"lm"},{name:"becquerel",symbol:"Bq"},{name:"gray",symbol:"Gy"},{name:"sievert",symbol:"Sv"},{name:"steradian",symbol:"sr"},{name:"farad",symbol:"F"},{name:"siemens",symbol:"S"},{name:"weber",symbol:"Wb"},{name:"henry",symbol:"H"},{name:"lux",symbol:"lx"},{name:"katal",symbol:"kat"}],d_e={chemical_element:u_e,unit:c_e},h_e=d_e,p_e=["ants","bats","bears","bees","birds","buffalo","cats","chickens","cattle","dogs","dolphins","ducks","elephants","fishes","foxes","frogs","geese","goats","horses","kangaroos","lions","monkeys","owls","oxen","penguins","people","pigs","rabbits","sheep","tigers","whales","wolves","zebras","banshees","crows","black cats","chimeras","ghosts","conspirators","dragons","dwarves","elves","enchanters","exorcists","sons","foes","giants","gnomes","goblins","gooses","griffins","lycanthropes","nemesis","ogres","oracles","prophets","sorcerors","spiders","spirits","vampires","warlocks","vixens","werewolves","witches","worshipers","zombies","druids"],f_e=["{{location.state}} {{team.creature}}"],m_e={creature:p_e,name:f_e},g_e=m_e,b_e=["Adventure Road Bicycle","BMX Bicycle","City Bicycle","Cruiser Bicycle","Cyclocross Bicycle","Dual-Sport Bicycle","Fitness Bicycle","Flat-Foot Comfort Bicycle","Folding Bicycle","Hybrid Bicycle","Mountain Bicycle","Recumbent Bicycle","Road Bicycle","Tandem Bicycle","Touring Bicycle","Track/Fixed-Gear Bicycle","Triathlon/Time Trial Bicycle","Tricycle"],y_e=["Diesel","Electric","Gasoline","Hybrid"],v_e=["Aston Martin","Audi","BMW","BYD","Bentley","Bugatti","Cadillac","Chevrolet","Chrysler","Citroën","Dodge","Ferrari","Fiat","Ford","Honda","Hyundai","Jaguar","Jeep","Kia","Lamborghini","Land Rover","MG","Mahindra & Mahindra","Maruti","Maserati","Mazda","Mercedes Benz","Mini","Mitsubishi","NIO","Nissan","Peugeot","Polestar","Porsche","Renault","Rivian","Rolls Royce","Skoda","Smart","Subaru","Suzuki","Tata","Tesla","Toyota","Vauxhall","Volkswagen","Volvo"],S_e=["1","2","911","A4","A8","ATS","Accord","Alpine","Altima","Aventador","Beetle","CTS","CX-9","Camaro","Camry","Challenger","Charger","Civic","Colorado","Corvette","Countach","Cruze","Durango","El Camino","Element","Escalade","Expedition","Explorer","F-150","Fiesta","Focus","Fortwo","Golf","Grand Caravan","Grand Cherokee","Impala","Jetta","Land Cruiser","LeBaron","Malibu","Mercielago","Model 3","Model S","Model T","Model X","Model Y","Mustang","PT Cruiser","Prius","Ranchero","Roadster","Sentra","Silverado","Spyder","Taurus","V90","Volt","Wrangler","XC90","XTS"],E_e=["Cargo Van","Convertible","Coupe","Crew Cab Pickup","Extended Cab Pickup","Hatchback","Minivan","Passenger Van","SUV","Sedan","Wagon"],x_e={bicycle_type:b_e,fuel:y_e,manufacturer:v_e,model:S_e,type:E_e},w_e=x_e,k_e=["abandoned","able","acceptable","acclaimed","accomplished","accurate","aching","acidic","actual","admired","adolescent","advanced","affectionate","afraid","aged","aggravating","aggressive","agile","agitated","agreeable","ajar","alarmed","alert","alienated","alive","all","altruistic","amazing","ambitious","ample","amused","angelic","anguished","animated","annual","another","antique","any","apprehensive","appropriate","apt","arid","artistic","ashamed","assured","astonishing","athletic","austere","authentic","authorized","avaricious","average","aware","awesome","awful","babyish","back","bad","baggy","bare","basic","beloved","beneficial","best","better","big","biodegradable","bitter","black","black-and-white","blank","blaring","bleak","blind","blond","blue","blushing","bogus","boiling","bony","boring","bossy","both","bouncy","bowed","brave","breakable","bright","brilliant","brisk","broken","brown","bruised","bulky","burdensome","burly","bustling","busy","buttery","buzzing","calculating","candid","carefree","careless","caring","cautious","cavernous","celebrated","charming","cheap","cheerful","chilly","chubby","circular","classic","clean","clear","clear-cut","close","closed","cloudy","clueless","clumsy","cluttered","coarse","colorful","colorless","colossal","comfortable","common","compassionate","competent","complete","complicated","concerned","concrete","confused","considerate","content","cool","cooperative","coordinated","corny","corrupt","courageous","courteous","crafty","crazy","creamy","creative","criminal","critical","crooked","crowded","cruel","crushing","cuddly","cultivated","cumbersome","curly","cute","damaged","damp","dapper","dark","darling","dazzling","dead","deadly","deafening","dearest","decent","decisive","deep","defenseless","defensive","deficient","definite","definitive","delectable","delicious","delirious","dense","dental","dependable","dependent","descriptive","deserted","determined","devoted","different","difficult","digital","diligent","dim","direct","dirty","discrete","disloyal","dismal","distant","distinct","distorted","doting","downright","drab","dramatic","dreary","dual","dull","dutiful","each","early","earnest","easy","ecstatic","edible","educated","elastic","elderly","electric","elegant","elementary","elliptical","eminent","emotional","empty","enchanted","enchanting","energetic","enlightened","enraged","entire","equatorial","essential","esteemed","ethical","everlasting","every","evil","exalted","excellent","excitable","excited","exhausted","exotic","expensive","experienced","expert","extra-large","extroverted","failing","faint","fair","fake","familiar","fantastic","far","far-flung","far-off","faraway","fat","fatal","fatherly","favorable","favorite","fearless","feline","filthy","fine","finished","firm","first","firsthand","fixed","flashy","flawed","flawless","flickering","flimsy","flowery","fluffy","flustered","focused","fond","foolhardy","foolish","forceful","formal","forsaken","fortunate","fragrant","frail","frank","free","french","frequent","friendly","frightened","frilly","frivolous","frizzy","front","frozen","frugal","fruitful","functional","funny","fussy","fuzzy","gaseous","general","gentle","genuine","gifted","gigantic","giving","glaring","glass","gleaming","glittering","gloomy","glorious","glossy","glum","golden","good","good-natured","gorgeous","graceful","gracious","grandiose","granular","grave","gray","great","greedy","grim","grimy","gripping","grizzled","grouchy","grounded","growing","grown","grubby","gruesome","grumpy","guilty","gullible","gummy","hairy","handsome","handy","happy","happy-go-lucky","hard-to-find","harmful","hasty","hateful","haunting","heartfelt","heavenly","heavy","hefty","helpful","helpless","hidden","hoarse","hollow","homely","honorable","honored","hopeful","hospitable","hot","huge","humble","humiliating","hungry","hurtful","husky","icy","ideal","idealistic","idolized","ignorant","ill","ill-fated","illiterate","illustrious","imaginary","imaginative","immaculate","immediate","immense","impartial","impassioned","impeccable","impish","impolite","important","impossible","impractical","impressionable","impressive","improbable","impure","inborn","incomparable","incomplete","inconsequential","indelible","indolent","inexperienced","infamous","infatuated","inferior","infinite","informal","innocent","insecure","insidious","insignificant","insistent","instructive","intelligent","intent","interesting","internal","international","intrepid","ironclad","irresponsible","jagged","jam-packed","jaunty","jealous","jittery","joyful","joyous","jubilant","judicious","juicy","jumbo","junior","juvenile","kaleidoscopic","key","knotty","knowledgeable","known","kooky","kosher","lanky","last","lasting","late","lavish","lawful","lazy","leading","lean","left","legal","light","lighthearted","likable","likely","limited","limp","limping","linear","lined","liquid","little","live","lively","livid","lone","lonely","long","long-term","lost","lovable","lovely","low","lucky","lumbering","lumpy","lustrous","mad","made-up","magnificent","majestic","major","male","mammoth","married","marvelous","massive","mature","meager","mealy","mean","measly","meaty","mediocre","medium","memorable","menacing","merry","messy","metallic","mild","milky","mindless","minor","minty","miserable","miserly","misguided","mixed","moist","monstrous","monthly","monumental","moral","motionless","muddy","muffled","multicolored","mundane","murky","mushy","musty","muted","mysterious","narrow","natural","naughty","nautical","near","neat","necessary","needy","negative","neglected","negligible","neighboring","nervous","new","next","nice","nifty","nimble","nippy","nocturnal","normal","noted","noteworthy","noxious","numb","nutritious","obedient","oblong","obvious","odd","oddball","official","oily","old","old-fashioned","only","optimal","optimistic","orange","orderly","ordinary","ornate","ornery","other","our","outgoing","outlandish","outlying","outrageous","outstanding","oval","overcooked","overdue","palatable","pale","paltry","parallel","parched","partial","passionate","pastel","peaceful","peppery","perfumed","perky","personal","pertinent","pessimistic","petty","phony","physical","pink","pitiful","plain","pleasant","pleased","pleasing","plump","pointed","pointless","polished","polite","political","poor","portly","posh","possible","potable","powerful","powerless","practical","precious","present","prestigious","pretty","pricey","prickly","primary","prime","private","probable","productive","profitable","profuse","proper","proud","prudent","punctual","puny","pure","purple","pushy","putrid","puzzled","qualified","quarrelsome","quarterly","queasy","querulous","questionable","quick","quick-witted","quiet","quintessential","quixotic","radiant","ragged","rapid","rare","raw","realistic","reasonable","recent","reckless","rectangular","red","reflecting","regal","regular","remarkable","remorseful","repentant","respectful","responsible","rewarding","rich","right","rigid","ripe","roasted","robust","rosy","rotating","rotten","rough","round","rowdy","royal","rubbery","ruddy","rundown","runny","rural","rusty","sad","salty","same","sandy","sarcastic","sardonic","scaly","scared","scary","scented","scientific","scornful","scratchy","second","second-hand","secondary","secret","self-assured","self-reliant","selfish","sentimental","separate","serene","serpentine","severe","shabby","shadowy","shady","shallow","shameful","shameless","shimmering","shiny","shocked","shoddy","short","short-term","showy","shrill","shy","sick","silent","silky","silver","similar","simple","simplistic","sinful","sizzling","skeletal","sleepy","slight","slimy","slow","slushy","small","smart","smoggy","smooth","smug","snappy","snarling","sneaky","sniveling","snoopy","sociable","soft","soggy","somber","some","sophisticated","sore","sorrowful","soulful","soupy","sour","spanish","sparkling","sparse","specific","speedy","spherical","spiffy","spirited","spiteful","splendid","spotless","square","squeaky","squiggly","stable","staid","stained","stale","standard","stark","steel","steep","sticky","stiff","stingy","stormy","straight","strange","strict","strident","striking","strong","stunning","stupendous","sturdy","stylish","subdued","submissive","substantial","subtle","suburban","sudden","sugary","sunny","super","superb","superficial","superior","supportive","sure-footed","surprised","svelte","sweet","swift","talkative","tall","tame","tangible","tasty","tattered","taut","tedious","teeming","tempting","tender","tense","tepid","terrible","that","these","thick","thin","thorny","thorough","those","thrifty","tidy","tight","timely","tinted","tiny","tired","torn","total","tough","tragic","trained","triangular","tricky","trim","trivial","troubled","true","trusting","trustworthy","trusty","turbulent","twin","ugly","ultimate","unaware","uncomfortable","uncommon","unconscious","understated","uneven","unfinished","unfit","unfortunate","unhappy","unhealthy","uniform","unimportant","unique","unkempt","unknown","unlawful","unlined","unlucky","unpleasant","unrealistic","unripe","unruly","unselfish","unsightly","unsteady","unsung","untidy","untimely","untried","untrue","unused","unusual","unwelcome","unwieldy","unwilling","unwritten","upbeat","upright","upset","urban","usable","useless","utilized","utter","vague","vain","valuable","variable","vast","velvety","vengeful","vibrant","victorious","violent","vivacious","vivid","voluminous","warlike","warm","warmhearted","warped","wasteful","waterlogged","watery","wavy","wealthy","weary","webbed","wee","weekly","weighty","weird","well-documented","well-groomed","well-lit","well-made","well-off","well-to-do","well-worn","which","whimsical","whirlwind","whispered","white","whole","whopping","wicked","wide","wide-eyed","wiggly","willing","wilted","winding","windy","winged","wise","witty","wobbly","woeful","wonderful","wordy","worldly","worse","worst","worthless","worthwhile","worthy","wrathful","wretched","writhing","wrong","wry","yearly","yellow","yellowish","young","youthful","yummy","zany","zealous","zesty"],C_e=["abnormally","absentmindedly","accidentally","acidly","actually","adventurously","afterwards","almost","always","angrily","annually","anxiously","arrogantly","awkwardly","badly","bashfully","beautifully","bitterly","bleakly","blindly","blissfully","boastfully","boldly","bravely","briefly","brightly","briskly","broadly","busily","calmly","carefully","carelessly","cautiously","certainly","cheerfully","clearly","cleverly","closely","coaxingly","colorfully","commonly","continually","coolly","correctly","courageously","crossly","cruelly","curiously","daily","daintily","dearly","deceivingly","deeply","defiantly","deliberately","delightfully","diligently","dimly","doubtfully","dreamily","easily","elegantly","energetically","enormously","enthusiastically","equally","especially","even","evenly","eventually","exactly","excitedly","extremely","fairly","faithfully","famously","far","fast","fatally","ferociously","fervently","fiercely","fondly","foolishly","fortunately","frankly","frantically","freely","frenetically","frightfully","fully","furiously","generally","generously","gently","gladly","gleefully","gracefully","gratefully","greatly","greedily","happily","hastily","healthily","heavily","helpfully","helplessly","highly","honestly","hopelessly","hourly","hungrily","immediately","innocently","inquisitively","instantly","intensely","intently","interestingly","inwardly","irritably","jaggedly","jealously","joshingly","jovially","joyfully","joyously","jubilantly","judgementally","justly","keenly","kiddingly","kindheartedly","kindly","kissingly","knavishly","knottily","knowingly","knowledgeably","kookily","lazily","less","lightly","likely","limply","lively","loftily","longingly","loosely","loudly","lovingly","loyally","madly","majestically","meaningfully","mechanically","merrily","miserably","mockingly","monthly","more","mortally","mostly","mysteriously","naturally","nearly","neatly","needily","nervously","never","nicely","noisily","not","obediently","obnoxiously","oddly","offensively","officially","often","only","openly","optimistically","overconfidently","owlishly","painfully","partially","patiently","perfectly","physically","playfully","politely","poorly","positively","potentially","powerfully","promptly","properly","punctually","quaintly","quarrelsomely","queasily","questionably","questioningly","quicker","quickly","quietly","quirkily","quizzically","rapidly","rarely","readily","really","reassuringly","recklessly","regularly","reluctantly","repeatedly","reproachfully","restfully","righteously","rightfully","rigidly","roughly","rudely","sadly","safely","scarcely","scarily","searchingly","sedately","seemingly","seldom","selfishly","separately","seriously","shakily","sharply","sheepishly","shrilly","shyly","silently","sleepily","slowly","smoothly","softly","solemnly","solidly","sometimes","soon","speedily","stealthily","sternly","strictly","successfully","suddenly","surprisingly","suspiciously","sweetly","swiftly","sympathetically","tenderly","tensely","terribly","thankfully","thoroughly","thoughtfully","tightly","tomorrow","too","tremendously","triumphantly","truly","truthfully","ultimately","unabashedly","unaccountably","unbearably","unethically","unexpectedly","unfortunately","unimpressively","unnaturally","unnecessarily","upbeat","upliftingly","upright","upside-down","upward","upwardly","urgently","usefully","uselessly","usually","utterly","vacantly","vaguely","vainly","valiantly","vastly","verbally","very","viciously","victoriously","violently","vivaciously","voluntarily","warmly","weakly","wearily","well","wetly","wholly","wildly","willfully","wisely","woefully","wonderfully","worriedly","wrongly","yawningly","yearly","yearningly","yesterday","yieldingly","youthfully"],T_e=["after","although","and","as","because","before","but","consequently","even","finally","for","furthermore","hence","how","however","if","inasmuch","incidentally","indeed","instead","lest","likewise","meanwhile","nor","now","once","or","provided","since","so","supposing","than","that","though","till","unless","until","what","when","whenever","where","whereas","wherever","whether","which","while","who","whoever","whose","why","yet"],A_e=["yuck","oh","phooey","blah","boo","whoa","yowza","huzzah","boohoo","fooey","geez","pfft","ew","ah","yum","brr","hm","yahoo","aha","woot","drat","gah","meh","psst","aw","ugh","yippee","eek","gee","bah","gadzooks","duh","ha","mmm","ouch","phew","ack","uh-huh","gosh","hmph","pish","zowie","er","ick","oof","um"],F_e=["CD","SUV","abacus","academics","accelerator","accompanist","account","accountability","acquaintance","ad","adaptation","address","adrenalin","adult","advancement","advertisement","adviser","affect","affiliate","aftermath","agreement","airbus","aircraft","airline","airmail","airman","airport","alb","alert","allegation","alliance","alligator","allocation","almighty","amendment","amnesty","analogy","angle","annual","antelope","anticodon","apparatus","appliance","approach","apricot","arcade","archaeology","armchair","armoire","asset","assist","atrium","attraction","availability","avalanche","awareness","babushka","backbone","backburn","bakeware","bandwidth","bar","barge","baritone","barracks","baseboard","basket","bathhouse","bathrobe","battle","begonia","behest","bell","bench","bend","beret","best-seller","bid","bidet","bin","birdbath","birdcage","birth","blight","blossom","blowgun","bob","bog","bonfire","bonnet","bookcase","bookend","boulevard","bourgeoisie","bowler","bowling","boyfriend","brace","bracelet","bran","breastplate","brief","brochure","brook","brush","bug","bump","bungalow","cafe","cake","calculus","cannon","cantaloupe","cap","cappelletti","captain","caption","carboxyl","cardboard","carnival","case","casement","cash","casket","cassava","castanet","catalyst","cauliflower","cellar","celsius","cemetery","ceramic","ceramics","certification","chainstay","chairperson","challenge","championship","chap","chapel","character","characterization","charlatan","charm","chasuble","cheese","cheetah","chiffonier","chops","chow","cinder","cinema","circumference","citizen","clamp","clavicle","cleaner","climb","co-producer","coal","coast","cod","coil","coin","coliseum","collaboration","collectivization","colon","colonialism","comestible","commercial","commodity","community","comparison","completion","complication","compromise","concentration","configuration","confusion","conservation","conservative","consistency","contractor","contrail","convection","conversation","cook","coordination","cop-out","cope","cork","cornet","corporation","corral","cosset","costume","couch","council","councilman","countess","courtroom","cow","creator","creature","crest","cricket","crocodile","cross-contamination","cruelty","cuckoo","curl","custody","custom","cutlet","cutover","cycle","daddy","dandelion","dash","daughter","dead","decision","deck","declaration","decongestant","decryption","deduction","deed","deer","defendant","density","department","dependency","deployment","depot","derby","descendant","descent","design","designation","desk","detective","devastation","developing","developmental","devil","diagram","digestive","digit","dime","director","disadvantage","disappointment","disclosure","disconnection","discourse","dish","disk","disposer","distinction","diver","diversity","dividend","divine","doing","doorpost","doubter","draft","draw","dream","dredger","dress","drive","drug","duffel","dulcimer","dusk","duster","dwell","e-mail","earth","ecliptic","ectoderm","edge","editor","effector","eggplant","electronics","elevation","elevator","elver","embarrassment","embossing","emergent","encouragement","entry","epic","equal","essence","eternity","ethyl","euphonium","event","exasperation","excess","executor","exhaust","expansion","expense","experience","exploration","extension","extent","exterior","eyebrow","eyeliner","farm","farmer","fat","fax","feather","fedora","fellow","fen","fencing","ferret","festival","fibre","filter","final","finding","finer","finger","fireplace","fisherman","fishery","fit","flame","flat","fledgling","flight","flint","flood","flu","fog","fold","folklore","follower","following","foodstuffs","footrest","forage","forager","forgery","fork","formamide","formation","formula","fort","fowl","fraudster","freckle","freezing","freight","fuel","fun","fund","fundraising","futon","gallery","galoshes","gastropod","gazebo","gerbil","ghost","giant","gift","giggle","glider","gloom","goat","godfather","godparent","going","goodwill","governance","government","gown","gradient","graffiti","grandpa","grandson","granny","grass","gray","gripper","grouper","guacamole","guard","guidance","guide","gym","gymnast","habit","haircut","halt","hamburger","hammock","handful","handle","handover","harp","haversack","hawk","heartache","heartbeat","heating","hello","help","hepatitis","heroine","hexagon","hierarchy","hippodrome","honesty","hoof","hope","horde","hornet","horst","hose","hospitalization","hovel","hovercraft","hubris","humidity","humor","hundred","hunger","hunt","husband","hutch","hydrant","hydrocarbon","hydrolyse","hydrolyze","hyena","hygienic","hyphenation","ice-cream","icebreaker","igloo","ignorance","illusion","impact","import","importance","impostor","in-joke","incandescence","independence","individual","information","injunction","innovation","insolence","inspection","instance","institute","instruction","instructor","integer","intellect","intent","interchange","interior","intervention","interviewer","invite","iridescence","issue","jacket","jazz","jellyfish","jet","jogging","joy","juggernaut","jump","jungle","junior","jury","kettledrum","kick","kielbasa","kinase","king","kiss","kit","knickers","knight","knitting","knuckle","label","labourer","lace","lady","lamp","language","larva","lashes","laughter","lava","lawmaker","lay","leading","league","legend","legging","legislature","lender","license","lid","lieu","lifestyle","lift","linseed","litter","loaf","lobster","longboat","lotion","lounge","louse","lox","loyalty","luck","lyre","maestro","mainstream","maintainer","majority","makeover","making","mallard","management","manner","mantua","marathon","march","marimba","marketplace","marksman","markup","marten","massage","masterpiece","mathematics","meadow","meal","meander","meatloaf","mechanic","median","membership","mentor","merit","metabolite","metal","middle","midwife","milestone","millet","minion","minister","minor","minority","mixture","mobility","molasses","mom","moment","monasticism","monocle","monster","morbidity","morning","mortise","mountain","mouser","mousse","mozzarella","muscat","mythology","napkin","necklace","nectarine","negotiation","nephew","nerve","netsuke","newsletter","newsprint","newsstand","nightlife","noon","nougat","nucleotidase","nudge","numeracy","numeric","nun","obedience","obesity","object","obligation","ocelot","octave","offset","oil","omelet","onset","opera","operating","optimal","orchid","order","ostrich","other","outlaw","outrun","outset","overcoat","overheard","overload","ownership","pacemaker","packaging","paintwork","palate","pants","pantyhose","papa","parade","parsnip","partridge","passport","pasta","patroller","pear","pearl","pecan","pendant","peninsula","pension","peony","pepper","perfection","permafrost","perp","petal","petticoat","pharmacopoeia","phrase","pick","piglet","pigpen","pigsty","pile","pillbox","pillow","pilot","pine","pinstripe","place","plain","planula","plastic","platter","platypus","pleasure","pliers","plugin","plumber","pneumonia","pocket-watch","poetry","polarisation","polyester","pomelo","pop","poppy","popularity","populist","porter","possession","postbox","precedent","premeditation","premier","premise","premium","pressure","presume","priesthood","printer","privilege","procurement","produce","programme","prohibition","promise","pronoun","providence","provider","provision","publication","publicity","pulse","punctuation","pupil","puppet","puritan","quart","quinoa","quit","railway","range","rationale","ravioli","rawhide","reach","reasoning","reboot","receptor","recommendation","reconsideration","recovery","redesign","relative","release","remark","reorganisation","repeat","replacement","reporter","representation","republican","request","requirement","reservation","resolve","resource","responsibility","restaurant","retention","retrospectivity","reward","ribbon","rim","riser","roadway","role","rosemary","roundabout","rubric","ruin","rule","runway","rust","safe","sailor","saloon","sand","sandbar","sanity","sarong","sauerkraut","saw","scaffold","scale","scarification","scenario","schedule","schnitzel","scholarship","scorn","scorpion","scout","scrap","scratch","seafood","seagull","seal","season","secrecy","secret","section","sediment","self-confidence","sermon","sesame","settler","shadowbox","shark","shipper","shore","shoulder","sideboard","siege","sightseeing","signature","silk","simple","singing","skean","skeleton","skyline","skyscraper","slide","slime","slipper","smog","smoke","sock","soliloquy","solution","solvency","someplace","sonar","sonata","sonnet","soup","soybean","space","spear","spirit","spork","sport","spring","sprinkles","squid","stall","starboard","statue","status","stay","steak","steeple","step","step-mother","sticker","stir-fry","stitcher","stock","stool","story","strait","stranger","strategy","straw","stump","subexpression","submitter","subsidy","substitution","suitcase","summary","summer","sunbeam","sundae","supplier","surface","sushi","suspension","sustenance","swanling","swath","sweatshop","swim","swine","swing","switch","switchboard","swordfish","synergy","t-shirt","tabletop","tackle","tail","tapioca","taro","tarragon","taxicab","teammate","technician","technologist","tectonics","tenant","tenement","tennis","tentacle","teriyaki","term","testimonial","testing","thigh","thongs","thorn","thread","thunderbolt","thyme","tinderbox","toaster","tomatillo","tomb","tomography","tool","tooth","toothbrush","toothpick","topsail","traditionalism","traffic","translation","transom","transparency","trash","travel","tray","trench","tribe","tributary","trick","trolley","tuba","tuber","tune-up","turret","tusk","tuxedo","typeface","typewriter","unblinking","underneath","underpants","understanding","unibody","unique","unit","utilization","valentine","validity","valley","valuable","vanadyl","vein","velocity","venom","version","verve","vestment","veto","viability","vibraphone","vibration","vicinity","video","violin","vision","vista","vol","volleyball","wafer","waist","wallaby","warming","wasabi","waterspout","wear","wedding","whack","whale","wheel","widow","wilderness","willow","window","wombat","word","worth","wriggler","yak","yarmulke","yeast","yin","yogurt","zebra","zen"],__e=["a","abaft","aboard","about","above","absent","across","afore","after","against","along","alongside","amid","amidst","among","amongst","an","anenst","anti","apropos","apud","around","as","aside","astride","at","athwart","atop","barring","before","behind","below","beneath","beside","besides","between","beyond","but","by","circa","concerning","considering","despite","down","during","except","excepting","excluding","failing","following","for","forenenst","from","given","in","including","inside","into","lest","like","mid","midst","minus","modulo","near","next","notwithstanding","of","off","on","onto","opposite","out","outside","over","pace","past","per","plus","pro","qua","regarding","round","sans","save","since","than","the","through","throughout","till","times","to","toward","towards","under","underneath","unlike","until","unto","up","upon","versus","via","vice","with","within","without","worth"],R_e=["abnegate","abscond","abseil","absolve","accentuate","accept","access","accessorise","accompany","account","accredit","achieve","acknowledge","acquire","adjourn","adjudge","admonish","adumbrate","advocate","afford","airbrush","ameliorate","amend","amount","anaesthetise","analyse","anesthetize","anneal","annex","antagonize","ape","apologise","apostrophize","appertain","appreciate","appropriate","approximate","arbitrate","archive","arraign","arrange","ascertain","ascribe","assail","atomize","attend","attest","attribute","augment","avow","axe","baa","banish","bank","baptise","battle","beard","beep","behold","belabor","bemuse","besmirch","bestride","better","bewail","bicycle","bide","bind","biodegrade","blacken","blaspheme","bleach","blend","blink","bliss","bloom","bludgeon","bobble","boggle","bolster","book","boom","bootleg","border","bore","boss","braid","brand","brandish","break","breed","broadcast","broadside","brood","browse","buck","burgeon","bus","butter","buzzing","camouflage","cannibalise","canter","cap","capitalise","capitalize","capsize","card","carouse","carp","carpool","catalog","catalyze","catch","categorise","cease","celebrate","censor","certify","char","charter","chase","chatter","chime","chip","christen","chromakey","chunder","chunter","cinch","circle","circulate","circumnavigate","clamor","clamour","claw","cleave","clinch","clinking","clone","clonk","coagulate","coexist","coincide","collaborate","colligate","colorize","colour","comb","come","commandeer","commemorate","communicate","compete","conceal","conceptualize","conclude","concrete","condense","cone","confide","confirm","confiscate","confound","confute","congregate","conjecture","connect","consign","construe","contradict","contrast","contravene","controvert","convalesce","converse","convince","convoke","coop","cop","corner","covenant","cow","crackle","cram","crank","creak","creaking","cripple","croon","cross","crumble","crystallize","culminate","culture","curry","curse","customise","cycle","dally","dampen","darn","debit","debut","decide","decode","decouple","decriminalize","deduce","deduct","deflate","deflect","deform","defrag","degenerate","degrease","delete","delight","deliquesce","demob","demobilise","democratize","demonstrate","denitrify","deny","depart","depend","deplore","deploy","deprave","depute","dereference","describe","desecrate","deselect","destock","detain","develop","devise","dial","dicker","digitize","dilate","disapprove","disarm","disbar","discontinue","disgorge","dishearten","dishonor","disinherit","dislocate","dispense","display","dispose","disrespect","dissemble","ditch","divert","dock","doodle","downchange","downshift","dowse","draft","drag","drain","dramatize","drowse","drum","dwell","economise","edge","efface","egg","eke","electrify","embalm","embed","embody","emboss","emerge","emphasise","emphasize","emulsify","encode","endow","enfold","engage","engender","enhance","enlist","enrage","enrich","enroll","entice","entomb","entrench","entwine","equate","essay","etch","eulogise","even","evince","exacerbate","exaggerate","exalt","exempt","exonerate","expatiate","explode","expostulate","extract","extricate","eyeglasses","fabricate","facilitate","factorise","factorize","fail","fall","familiarize","fashion","father","fathom","fax","federate","feminize","fence","fess","fictionalize","fiddle","fidget","fill","flash","fleck","flight","floodlight","floss","fluctuate","fluff","fly","focalise","foot","forearm","forecast","foretell","forgather","forgo","fork","form","forswear","founder","fraternise","fray","frizz","fumigate","function","furlough","fuss","gad","gallivant","galvanize","gape","garage","garrote","gasp","gestate","give","glimmer","glisten","gloat","gloss","glow","gnash","gnaw","goose","govern","grade","graduate","graft","grok","guest","guilt","gulp","gum","gurn","gust","gut","guzzle","ham","harangue","harvest","hassle","haul","haze","headline","hearten","heighten","highlight","hoick","hold","hole","hollow","holster","home","homeschool","hoot","horn","horse","hotfoot","house","hover","howl","huddle","huff","hunger","hunt","husk","hype","hypothesise","hypothesize","idle","ignite","imagineer","impact","impanel","implode","incinerate","incline","inculcate","industrialize","ingratiate","inhibit","inject","innovate","inscribe","insert","insist","inspect","institute","institutionalize","intend","intermarry","intermesh","intermix","internalise","internalize","internationalize","intrigue","inure","inveigle","inventory","investigate","irk","iterate","jaywalk","jell","jeopardise","jiggle","jive","joint","jot","jut","keel","knife","knit","know","kowtow","lack","lampoon","large","leap","lecture","legitimize","lend","libel","liberalize","license","ligate","list","lobotomise","lock","log","loose","low","lowball","machine","magnetize","major","make","malfunction","manage","manipulate","maroon","masculinize","mash","mask","masquerade","massage","masticate","materialise","matter","maul","memorise","merge","mesh","metabolise","microblog","microchip","micromanage","militate","mill","minister","minor","misappropriate","miscalculate","misfire","misjudge","miskey","mismatch","mispronounce","misread","misreport","misspend","mob","mobilise","mobilize","moisten","mooch","moor","moralise","mortar","mosh","mothball","motivate","motor","mould","mount","muddy","mummify","mutate","mystify","nab","narrate","narrowcast","nasalise","nauseate","navigate","neaten","neck","neglect","norm","notarize","object","obscure","observe","obsess","obstruct","obtrude","offend","offset","option","orchestrate","orient","orientate","outbid","outdo","outfit","outflank","outfox","outnumber","outrank","outrun","outsource","overburden","overcharge","overcook","overdub","overfeed","overload","overplay","overproduce","overreact","override","overspend","overstay","overtrain","overvalue","overwork","own","oxidise","oxidize","oxygenate","pace","pack","pale","pant","paralyse","parody","part","pause","pave","penalise","persecute","personalise","perspire","pertain","peter","pike","pillory","pinion","pip","pity","pivot","pixellate","plagiarise","plait","plan","please","pluck","ponder","popularize","portray","prance","preclude","preheat","prejudge","preregister","presell","preside","pretend","print","prioritize","probate","probe","proceed","procrastinate","profane","progress","proliferate","proofread","propound","proselytise","provision","pry","publicize","puff","pull","pulp","pulverize","purse","put","putrefy","quadruple","quaff","quantify","quarrel","quash","quaver","question","quiet","quintuple","quip","quit","rag","rally","ramp","randomize","rationalise","rationalize","ravage","ravel","react","readies","readjust","readmit","ready","reapply","rear","reassemble","rebel","reboot","reborn","rebound","rebuff","rebuild","rebuke","recede","reckon","reclassify","recompense","reconstitute","record","recount","redact","redevelop","redound","redraw","redress","reel","refer","reference","refine","reflate","refute","regulate","reiterate","rejigger","rejoin","rekindle","relaunch","relieve","remand","remark","reopen","reorient","replicate","repossess","represent","reprimand","reproach","reprove","repurpose","requite","reschedule","resort","respray","restructure","retool","retract","revere","revitalise","revoke","reword","rewrite","ride","ridge","rim","ring","rise","rival","roger","rosin","rot","rout","row","rue","rule","safeguard","sashay","sate","satirise","satirize","satisfy","saturate","savour","scale","scamper","scar","scare","scarper","scent","schematise","scheme","schlep","scoff","scoop","scope","scotch","scowl","scrabble","scram","scramble","scrape","screw","scruple","scrutinise","scuffle","scuttle","search","secularize","see","segregate","sell","sense","sensitize","sequester","serenade","serialize","serve","service","settle","sew","shaft","sham","shampoo","shanghai","shear","sheathe","shell","shinny","shirk","shoot","shoulder","shout","shovel","showboat","shred","shrill","shudder","shush","sidetrack","sign","silt","sin","singe","sit","sizzle","skateboard","ski","slake","slap","slather","sleet","slink","slip","slope","slump","smarten","smuggle","snack","sneak","sniff","snoop","snow","snowplow","snuggle","soap","solace","solder","solicit","source","spark","spattering","spectacles","spectate","spellcheck","spew","spice","spirit","splash","splay","split","splosh","splurge","spook","square","squirm","stabilise","stable","stack","stage","stake","starch","state","statement","stiffen","stigmatize","sting","stint","stoop","store","storyboard","stratify","structure","stuff","stunt","substantiate","subtract","suckle","suffice","suffocate","summarise","sun","sunbathe","sunder","sup","surge","surprise","swat","swathe","sway","swear","swelter","swerve","swill","swing","symbolise","synthesise","syringe","table","tabulate","tag","tame","tank","tankful","tarry","task","taxicab","team","telescope","tenant","terraform","terrorise","testify","think","throbbing","thump","tighten","toady","toe","tough","tousle","traduce","train","transcend","transplant","trash","treasure","treble","trek","trial","tromp","trouser","trust","tune","tut","twine","twist","typify","unbalance","uncork","uncover","underachieve","undergo","underplay","unearth","unfreeze","unfurl","unlearn","unscramble","unzip","uproot","upsell","usher","vacation","vamoose","vanish","vary","veg","venture","verify","vet","veto","volunteer","vulgarise","waft","wallop","waltz","warp","wash","waver","weary","weatherize","wedge","weep","weight","welcome","westernise","westernize","while","whine","whisper","whistle","whitewash","whup","wilt","wing","wire","wisecrack","wolf","wound","wring","writ","yak","yawn","yearn","yuppify"],D_e={adjective:k_e,adverb:C_e,conjunction:T_e,interjection:A_e,noun:F_e,preposition:__e,verb:R_e},N_e=D_e,L_e={airline:gFe,animal:MFe,app:zFe,book:VFe,cell_phone:JFe,color:QFe,commerce:a3e,company:f3e,database:b3e,date:E3e,finance:I3e,food:V3e,hacker:e4e,internet:i4e,location:C4e,lorem:F4e,metadata:R4e,music:O4e,person:t_e,phone_number:l_e,science:h_e,team:g_e,vehicle:w_e,word:N_e},rte=L_e,Jt=class extends Error{};function M_e(e){let t=Object.getPrototypeOf(e);do{for(let r of Object.getOwnPropertyNames(t))typeof e[r]=="function"&&r!=="constructor"&&(e[r]=e[r].bind(e));t=Object.getPrototypeOf(t)}while(t!==Object.prototype)}var ad=class{constructor(t){this.faker=t,M_e(this)}},In=class extends ad{constructor(t){super(t),this.faker=t}},nte=(e=>(e.Narrowbody="narrowbody",e.Regional="regional",e.Widebody="widebody",e))(nte||{}),I_e=["0","1","2","3","4","5","6","7","8","9"],O_e=["0","O","1","I","L"],B_e={regional:20,narrowbody:35,widebody:60},P_e={regional:["A","B","C","D"],narrowbody:["A","B","C","D","E","F"],widebody:["A","B","C","D","E","F","G","H","J","K"]},z_e=class extends In{airport(){return this.faker.helpers.arrayElement(this.faker.definitions.airline.airport)}airline(){return this.faker.helpers.arrayElement(this.faker.definitions.airline.airline)}airplane(){return this.faker.helpers.arrayElement(this.faker.definitions.airline.airplane)}recordLocator(t={}){let{allowNumerics:r=!1,allowVisuallySimilarCharacters:n=!1}=t,a=[];return r||a.push(...I_e),n||a.push(...O_e),this.faker.string.alphanumeric({length:6,casing:"upper",exclude:a})}seat(t={}){let{aircraftType:r="narrowbody"}=t,n=B_e[r],a=P_e[r],i=this.faker.number.int({min:1,max:n}),o=this.faker.helpers.arrayElement(a);return`${i}${o}`}aircraftType(){return this.faker.helpers.enumValue(nte)}flightNumber(t={}){let{length:r={min:1,max:4},addLeadingZeros:n=!1}=t,a=this.faker.string.numeric({length:r,allowLeadingZeros:!1});return n?a.padStart(4,"0"):a}},ate=(e=>(e.SRGB="sRGB",e.DisplayP3="display-p3",e.REC2020="rec2020",e.A98RGB="a98-rgb",e.ProphotoRGB="prophoto-rgb",e))(ate||{}),ite=(e=>(e.RGB="rgb",e.RGBA="rgba",e.HSL="hsl",e.HSLA="hsla",e.HWB="hwb",e.CMYK="cmyk",e.LAB="lab",e.LCH="lch",e.COLOR="color",e))(ite||{});function H_e(e,t){let{prefix:r,casing:n}=t;switch(n){case"upper":{e=e.toUpperCase();break}case"lower":{e=e.toLowerCase();break}}return r&&(e=r+e),e}function ote(e){return e.map(t=>{if(t%1!==0){let r=new ArrayBuffer(4);new DataView(r).setFloat32(0,t);let n=new Uint8Array(r);return ote([...n]).replaceAll(" ","")}return(t>>>0).toString(2).padStart(8,"0")}).join(" ")}function _i(e){return Math.round(e*100)}function U_e(e,t="rgb",r="sRGB"){switch(t){case"rgba":return`rgba(${e[0]}, ${e[1]}, ${e[2]}, ${e[3]})`;case"color":return`color(${r} ${e[0]} ${e[1]} ${e[2]})`;case"cmyk":return`cmyk(${_i(e[0])}%, ${_i(e[1])}%, ${_i(e[2])}%, ${_i(e[3])}%)`;case"hsl":return`hsl(${e[0]}deg ${_i(e[1])}% ${_i(e[2])}%)`;case"hsla":return`hsl(${e[0]}deg ${_i(e[1])}% ${_i(e[2])}% / ${_i(e[3])})`;case"hwb":return`hwb(${e[0]} ${_i(e[1])}% ${_i(e[2])}%)`;case"lab":return`lab(${_i(e[0])}% ${e[1]} ${e[2]})`;case"lch":return`lch(${_i(e[0])}% ${e[1]} ${e[2]})`;case"rgb":return`rgb(${e[0]}, ${e[1]}, ${e[2]})`}}function pc(e,t,r="rgb",n="sRGB"){switch(t){case"css":return U_e(e,r,n);case"binary":return ote(e);case"decimal":return e}}var $_e=class extends In{human(){return this.faker.helpers.arrayElement(this.faker.definitions.color.human)}space(){return this.faker.helpers.arrayElement(this.faker.definitions.color.space)}cssSupportedFunction(){return this.faker.helpers.enumValue(ite)}cssSupportedSpace(){return this.faker.helpers.enumValue(ate)}rgb(t={}){let{format:r="hex",includeAlpha:n=!1,prefix:a="#",casing:i="lower"}=t,o,s="rgb";return r==="hex"?(o=this.faker.string.hexadecimal({length:n?8:6,prefix:""}),o=H_e(o,{prefix:a,casing:i}),o):(o=Array.from({length:3},()=>this.faker.number.int(255)),n&&(o.push(this.faker.number.float({multipleOf:.01})),s="rgba"),pc(o,r,s))}cmyk(t={}){let{format:r="decimal"}=t,n=Array.from({length:4},()=>this.faker.number.float({multipleOf:.01}));return pc(n,r,"cmyk")}hsl(t={}){let{format:r="decimal",includeAlpha:n=!1}=t,a=[this.faker.number.int(360)];for(let i=0;i<(t!=null&&t.includeAlpha?3:2);i++)a.push(this.faker.number.float({multipleOf:.01}));return pc(a,r,n?"hsla":"hsl")}hwb(t={}){let{format:r="decimal"}=t,n=[this.faker.number.int(360)];for(let a=0;a<2;a++)n.push(this.faker.number.float({multipleOf:.01}));return pc(n,r,"hwb")}lab(t={}){let{format:r="decimal"}=t,n=[this.faker.number.float({multipleOf:1e-6})];for(let a=0;a<2;a++)n.push(this.faker.number.float({min:-100,max:100,multipleOf:1e-4}));return pc(n,r,"lab")}lch(t={}){let{format:r="decimal"}=t,n=[this.faker.number.float({multipleOf:1e-6})];for(let a=0;a<2;a++)n.push(this.faker.number.float({max:230,multipleOf:.1}));return pc(n,r,"lch")}colorByCSSColorSpace(t={}){let{format:r="decimal",space:n="sRGB"}=t,a=Array.from({length:3},()=>this.faker.number.float({multipleOf:1e-4}));return pc(a,r,"color",n)}},ste=(e=>(e.Legacy="legacy",e.Segwit="segwit",e.Bech32="bech32",e.Taproot="taproot",e))(ste||{}),G_e=(e=>(e.Mainnet="mainnet",e.Testnet="testnet",e))(G_e||{}),j_e={legacy:{prefix:{mainnet:"1",testnet:"m"},length:{min:26,max:34},casing:"mixed",exclude:"0OIl"},segwit:{prefix:{mainnet:"3",testnet:"2"},length:{min:26,max:34},casing:"mixed",exclude:"0OIl"},bech32:{prefix:{mainnet:"bc1",testnet:"tb1"},length:{min:42,max:42},casing:"lower",exclude:"1bBiIoO"},taproot:{prefix:{mainnet:"bc1p",testnet:"tb1p"},length:{min:62,max:62},casing:"lower",exclude:"1bBiIoO"}},lte=typeof Buffer>"u"||!ute("base64")?e=>{let t=new TextEncoder().encode(e),r=Array.from(t,n=>String.fromCodePoint(n)).join("");return btoa(r)}:e=>Buffer.from(e).toString("base64"),EB=typeof Buffer>"u"||!ute("base64url")?e=>lte(e).replaceAll("+","-").replaceAll("/","_").replaceAll(/=+$/g,""):e=>Buffer.from(e).toString("base64url");function ute(e){try{return typeof Buffer.from("test").toString(e)=="string"}catch{return!1}}function jc(e){let{deprecated:t,since:r,until:n,proposed:a}=e,i=`[@faker-js/faker]: ${t} is deprecated`;r&&(i+=` since v${r}`),n&&(i+=` and will be removed in v${n}`),a&&(i+=`. Please use ${a} instead`),console.warn(`${i}.`)}var q_e=Object.fromEntries([["А","A"],["а","a"],["Б","B"],["б","b"],["В","V"],["в","v"],["Г","G"],["г","g"],["Д","D"],["д","d"],["ъе","ye"],["Ъе","Ye"],["ъЕ","yE"],["ЪЕ","YE"],["Е","E"],["е","e"],["Ё","Yo"],["ё","yo"],["Ж","Zh"],["ж","zh"],["З","Z"],["з","z"],["И","I"],["и","i"],["ый","iy"],["Ый","Iy"],["ЫЙ","IY"],["ыЙ","iY"],["Й","Y"],["й","y"],["К","K"],["к","k"],["Л","L"],["л","l"],["М","M"],["м","m"],["Н","N"],["н","n"],["О","O"],["о","o"],["П","P"],["п","p"],["Р","R"],["р","r"],["С","S"],["с","s"],["Т","T"],["т","t"],["У","U"],["у","u"],["Ф","F"],["ф","f"],["Х","Kh"],["х","kh"],["Ц","Ts"],["ц","ts"],["Ч","Ch"],["ч","ch"],["Ш","Sh"],["ш","sh"],["Щ","Sch"],["щ","sch"],["Ъ",""],["ъ",""],["Ы","Y"],["ы","y"],["Ь",""],["ь",""],["Э","E"],["э","e"],["Ю","Yu"],["ю","yu"],["Я","Ya"],["я","ya"]]),W_e=Object.fromEntries([["α","a"],["β","v"],["γ","g"],["δ","d"],["ε","e"],["ζ","z"],["η","i"],["θ","th"],["ι","i"],["κ","k"],["λ","l"],["μ","m"],["ν","n"],["ξ","ks"],["ο","o"],["π","p"],["ρ","r"],["σ","s"],["τ","t"],["υ","y"],["φ","f"],["χ","x"],["ψ","ps"],["ω","o"],["ά","a"],["έ","e"],["ί","i"],["ό","o"],["ύ","y"],["ή","i"],["ώ","o"],["ς","s"],["ϊ","i"],["ΰ","y"],["ϋ","y"],["ΐ","i"],["Α","A"],["Β","B"],["Γ","G"],["Δ","D"],["Ε","E"],["Ζ","Z"],["Η","I"],["Θ","TH"],["Ι","I"],["Κ","K"],["Λ","L"],["Μ","M"],["Ν","N"],["Ξ","KS"],["Ο","O"],["Π","P"],["Ρ","R"],["Σ","S"],["Τ","T"],["Υ","Y"],["Φ","F"],["Χ","X"],["Ψ","PS"],["Ω","O"],["Ά","A"],["Έ","E"],["Ί","I"],["Ό","O"],["Ύ","Y"],["Ή","I"],["Ώ","O"],["Ϊ","I"],["Ϋ","Y"]]),V_e=Object.fromEntries([["ء","e"],["آ","a"],["أ","a"],["ؤ","w"],["إ","i"],["ئ","y"],["ا","a"],["ب","b"],["ة","t"],["ت","t"],["ث","th"],["ج","j"],["ح","h"],["خ","kh"],["د","d"],["ذ","dh"],["ر","r"],["ز","z"],["س","s"],["ش","sh"],["ص","s"],["ض","d"],["ط","t"],["ظ","z"],["ع","e"],["غ","gh"],["ـ","_"],["ف","f"],["ق","q"],["ك","k"],["ل","l"],["م","m"],["ن","n"],["ه","h"],["و","w"],["ى","a"],["ي","y"],["َ‎","a"],["ُ","u"],["ِ‎","i"]]),K_e=Object.fromEntries([["ա","a"],["Ա","A"],["բ","b"],["Բ","B"],["գ","g"],["Գ","G"],["դ","d"],["Դ","D"],["ե","ye"],["Ե","Ye"],["զ","z"],["Զ","Z"],["է","e"],["Է","E"],["ը","y"],["Ը","Y"],["թ","t"],["Թ","T"],["ժ","zh"],["Ժ","Zh"],["ի","i"],["Ի","I"],["լ","l"],["Լ","L"],["խ","kh"],["Խ","Kh"],["ծ","ts"],["Ծ","Ts"],["կ","k"],["Կ","K"],["հ","h"],["Հ","H"],["ձ","dz"],["Ձ","Dz"],["ղ","gh"],["Ղ","Gh"],["ճ","tch"],["Ճ","Tch"],["մ","m"],["Մ","M"],["յ","y"],["Յ","Y"],["ն","n"],["Ն","N"],["շ","sh"],["Շ","Sh"],["ո","vo"],["Ո","Vo"],["չ","ch"],["Չ","Ch"],["պ","p"],["Պ","P"],["ջ","j"],["Ջ","J"],["ռ","r"],["Ռ","R"],["ս","s"],["Ս","S"],["վ","v"],["Վ","V"],["տ","t"],["Տ","T"],["ր","r"],["Ր","R"],["ց","c"],["Ց","C"],["ու","u"],["ՈՒ","U"],["Ու","U"],["փ","p"],["Փ","P"],["ք","q"],["Ք","Q"],["օ","o"],["Օ","O"],["ֆ","f"],["Ֆ","F"],["և","yev"]]),Y_e=Object.fromEntries([["چ","ch"],["ک","k"],["گ","g"],["پ","p"],["ژ","zh"],["ی","y"]]),J_e=Object.fromEntries([["א","a"],["ב","b"],["ג","g"],["ד","d"],["ה","h"],["ו","v"],["ז","z"],["ח","ch"],["ט","t"],["י","y"],["כ","k"],["ך","kh"],["ל","l"],["ם","m"],["מ","m"],["ן","n"],["נ","n"],["ס","s"],["ע","a"],["פ","f"],["ף","ph"],["צ","ts"],["ץ","ts"],["ק","k"],["ר","r"],["ש","sh"],["ת","t"],["ו","v"]]),xB={...q_e,...W_e,...V_e,...Y_e,...K_e,...J_e},X_e=(e=>(e.Any="any",e.Loopback="loopback",e.PrivateA="private-a",e.PrivateB="private-b",e.PrivateC="private-c",e.TestNet1="test-net-1",e.TestNet2="test-net-2",e.TestNet3="test-net-3",e.LinkLocal="link-local",e.Multicast="multicast",e))(X_e||{}),Z_e={any:"0.0.0.0/0",loopback:"127.0.0.0/8","private-a":"10.0.0.0/8","private-b":"172.16.0.0/12","private-c":"192.168.0.0/16","test-net-1":"192.0.2.0/24","test-net-2":"198.51.100.0/24","test-net-3":"203.0.113.0/24","link-local":"169.254.0.0/16",multicast:"224.0.0.0/4"};function wB(e){return/^[a-z][a-z-]*[a-z]$/i.exec(e)!==null}function kB(e,t){let r=e.helpers.slugify(t);if(wB(r))return r;let n=e.helpers.slugify(e.lorem.word());return wB(n)?n:e.string.alpha({casing:"lower",length:e.number.int({min:4,max:8})})}function Bw(e,t){return Math.floor((e.number.int(256)+t)/2).toString(16).padStart(2,"0")}var Q_e=class extends In{email(t={}){let{firstName:r,lastName:n,provider:a=this.faker.helpers.arrayElement(this.faker.definitions.internet.free_email),allowSpecialCharacters:i=!1}=t,o=this.username({firstName:r,lastName:n});if(o=o.replaceAll(/[^A-Za-z0-9._+-]+/g,""),o=o.substring(0,50),i){let s=[..."._-"],l=[...".!#$%&'*+-/=?^_`{|}~"];o=o.replace(this.faker.helpers.arrayElement(s),this.faker.helpers.arrayElement(l))}return o=o.replaceAll(/\.{2,}/g,"."),o=o.replace(/^\./,""),o=o.replace(/\.$/,""),`${o}@${a}`}exampleEmail(t={}){let{firstName:r,lastName:n,allowSpecialCharacters:a=!1}=t,i=this.faker.helpers.arrayElement(this.faker.definitions.internet.example_email);return this.email({firstName:r,lastName:n,provider:i,allowSpecialCharacters:a})}userName(t={}){return jc({deprecated:"faker.internet.userName()",proposed:"faker.internet.username()",since:"9.1.0",until:"10.0.0"}),this.username(t)}username(t={}){let{firstName:r=this.faker.person.firstName(),lastName:n=this.faker.person.lastName(),lastName:a}=t,i=this.faker.helpers.arrayElement([".","_"]),o=this.faker.number.int(99),s=[()=>`${r}${i}${n}${o}`,()=>`${r}${i}${n}`];a||s.push(()=>`${r}${o}`);let l=this.faker.helpers.arrayElement(s)();return l=l.normalize("NFKD").replaceAll(/[\u0300-\u036F]/g,""),l=[...l].map(u=>{if(xB[u])return xB[u];let d=u.codePointAt(0)??Number.NaN;return d<128?u:d.toString(36)}).join(""),l=l.toString().replaceAll("'",""),l=l.replaceAll(" ",""),l}displayName(t={}){let{firstName:r=this.faker.person.firstName(),lastName:n=this.faker.person.lastName()}=t,a=this.faker.helpers.arrayElement([".","_"]),i=this.faker.number.int(99),o=[()=>`${r}${i}`,()=>`${r}${a}${n}`,()=>`${r}${a}${n}${i}`],s=this.faker.helpers.arrayElement(o)();return s=s.toString().replaceAll("'",""),s=s.replaceAll(" ",""),s}protocol(){let t=["http","https"];return this.faker.helpers.arrayElement(t)}httpMethod(){let t=["GET","POST","PUT","DELETE","PATCH"];return this.faker.helpers.arrayElement(t)}httpStatusCode(t={}){let{types:r=Object.keys(this.faker.definitions.internet.http_status_code)}=t,n=this.faker.helpers.arrayElement(r);return this.faker.helpers.arrayElement(this.faker.definitions.internet.http_status_code[n])}url(t={}){let{appendSlash:r=this.faker.datatype.boolean(),protocol:n="https"}=t;return`${n}://${this.domainName()}${r?"/":""}`}domainName(){return`${this.domainWord()}.${this.domainSuffix()}`}domainSuffix(){return this.faker.helpers.arrayElement(this.faker.definitions.internet.domain_suffix)}domainWord(){let t=kB(this.faker,this.faker.word.adjective()),r=kB(this.faker,this.faker.word.noun());return`${t}-${r}`.toLowerCase()}ip(){return this.faker.datatype.boolean()?this.ipv4():this.ipv6()}ipv4(t={}){let{network:r="any",cidrBlock:n=Z_e[r]}=t;if(!/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\/\d{1,2}$/.test(n))throw new Jt(`Invalid CIDR block provided: ${n}. Must be in the format x.x.x.x/y.`);let[a,i]=n.split("/"),o=4294967295>>>Number.parseInt(i),[s,l,u,d]=a.split(".").map(Number),p=(s<<24|l<<16|u<<8|d)&~o,f=this.faker.number.int(o),m=p|f;return[m>>>24&255,m>>>16&255,m>>>8&255,m&255].join(".")}ipv6(){return Array.from({length:8},()=>this.faker.string.hexadecimal({length:4,casing:"lower",prefix:""})).join(":")}port(){return this.faker.number.int(65535)}userAgent(){return this.faker.helpers.fake(this.faker.definitions.internet.user_agent_pattern)}color(t={}){jc({deprecated:"faker.internet.color()",proposed:"faker.color.rgb()",since:"9.6.0",until:"10.0.0"});let{redBase:r=0,greenBase:n=0,blueBase:a=0}=t,i=Bw(this.faker,r),o=Bw(this.faker,n),s=Bw(this.faker,a);return`#${i}${o}${s}`}mac(t={}){typeof t=="string"&&(t={separator:t});let{separator:r=":"}=t,n,a="";for([":","-",""].includes(r)||(r=":"),n=0;n<12;n++)a+=this.faker.number.hex(15),n%2===1&&n!==11&&(a+=r);return a}password(t={}){let r=/[aeiouAEIOU]$/,n=/[bcdfghjklmnpqrstvwxyzBCDFGHJKLMNPQRSTVWXYZ]$/,a=(u,d,p,f)=>{if(f.length>=u)return f;d&&(p=n.test(f)?r:n);let m=this.faker.number.int(94)+33,b=String.fromCodePoint(m);return d&&(b=b.toLowerCase()),p.test(b)?a(u,d,p,f+b):a(u,d,p,f)},{length:i=15,memorable:o=!1,pattern:s=/\w/,prefix:l=""}=t;return a(i,o,s,l)}emoji(t={}){let{types:r=Object.keys(this.faker.definitions.internet.emoji)}=t,n=this.faker.helpers.arrayElement(r);return this.faker.helpers.arrayElement(this.faker.definitions.internet.emoji[n])}jwtAlgorithm(){return this.faker.helpers.arrayElement(this.faker.definitions.internet.jwt_algorithm)}jwt(t={}){let{refDate:r=this.faker.defaultRefDate()}=t,n=this.faker.date.recent({refDate:r}),{header:a={alg:this.jwtAlgorithm(),typ:"JWT"},payload:i={iat:Math.round(n.valueOf()/1e3),exp:Math.round(this.faker.date.soon({refDate:n}).valueOf()/1e3),nbf:Math.round(this.faker.date.anytime({refDate:r}).valueOf()/1e3),iss:this.faker.company.name(),sub:this.faker.string.uuid(),aud:this.faker.string.uuid(),jti:this.faker.string.uuid()}}=t,o=EB(JSON.stringify(a)),s=EB(JSON.stringify(i)),l=this.faker.string.alphanumeric(64);return`${o}.${s}.${l}`}},cte=(e=>(e.Female="female",e.Male="male",e))(cte||{});function Pf(e,t,r){let{generic:n,female:a,male:i}=r;switch(t){case"female":return a??n;case"male":return i??n;default:return n??e.helpers.arrayElement([a,i])??[]}}var eRe=class extends In{firstName(t){return this.faker.helpers.arrayElement(Pf(this.faker,t,this.faker.definitions.person.first_name))}lastName(t){var r;if(((r=this.faker.rawDefinitions.person)==null?void 0:r.last_name_pattern)!=null){let n=this.faker.helpers.weightedArrayElement(Pf(this.faker,t,this.faker.rawDefinitions.person.last_name_pattern));return this.faker.helpers.fake(n)}return this.faker.helpers.arrayElement(Pf(this.faker,t,this.faker.definitions.person.last_name))}middleName(t){return this.faker.helpers.arrayElement(Pf(this.faker,t,this.faker.definitions.person.middle_name))}fullName(t={}){let{sex:r=this.faker.helpers.arrayElement(["female","male"]),firstName:n=this.firstName(r),lastName:a=this.lastName(r)}=t,i=this.faker.helpers.weightedArrayElement(this.faker.definitions.person.name);return this.faker.helpers.mustache(i,{"person.prefix":()=>this.prefix(r),"person.firstName":()=>n,"person.middleName":()=>this.middleName(r),"person.lastName":()=>a,"person.suffix":()=>this.suffix()})}gender(){return this.faker.helpers.arrayElement(this.faker.definitions.person.gender)}sex(){return this.faker.helpers.arrayElement(this.faker.definitions.person.sex)}sexType(){return this.faker.helpers.enumValue(cte)}bio(){let{bio_pattern:t}=this.faker.definitions.person;return this.faker.helpers.fake(t)}prefix(t){return this.faker.helpers.arrayElement(Pf(this.faker,t,this.faker.definitions.person.prefix))}suffix(){return this.faker.helpers.arrayElement(this.faker.definitions.person.suffix)}jobTitle(){return this.faker.helpers.fake(this.faker.definitions.person.job_title_pattern)}jobDescriptor(){return this.faker.helpers.arrayElement(this.faker.definitions.person.job_descriptor)}jobArea(){return this.faker.helpers.arrayElement(this.faker.definitions.person.job_area)}jobType(){return this.faker.helpers.arrayElement(this.faker.definitions.person.job_type)}zodiacSign(){return this.faker.helpers.arrayElement(this.faker.definitions.person.western_zodiac_sign)}},tRe=23283064365386963e-26,rRe=1/9007199254740992,{imul:wR,trunc:kR}=Math;function CB(e){return typeof e=="number"?dte(e):nRe(e)}function dte(e){let t=Array.from({length:624});t[0]=e;for(let r=1;r!==624;++r){let n=t[r-1]^t[r-1]>>>30;t[r]=kR(wR(1812433253,n)+r)}return t}function nRe(e){let t=dte(19650218),r=1,n=0;for(let a=Math.max(624,e.length);a!==0;--a){let i=t[r-1]^t[r-1]>>>30;t[r]=kR((t[r]^wR(i,1664525))+e[n]+n),r++,n++,r>=624&&(t[0]=t[623],r=1),n>=e.length&&(n=0)}for(let a=623;a!==0;a--)t[r]=kR((t[r]^wR(t[r-1]^t[r-1]>>>30,1566083941))-r),r++,r>=624&&(t[0]=t[623],r=1);return t[0]=2147483648,t}function Pw(e){for(let r=0;r!==227;++r){let n=(e[r]&2147483648)+(e[r+1]&2147483647);e[r]=e[r+397]^n>>>1^-(n&1)&2567483615}for(let r=227;r!==623;++r){let n=(e[r]&2147483648)+(e[r+1]&2147483647);e[r]=e[r+397-624]^n>>>1^-(n&1)&2567483615}let t=(e[623]&2147483648)+(e[0]&2147483647);return e[623]=e[396]^t>>>1^-(t&1)&2567483615,e}var aRe=class{constructor(t=Math.random()*Number.MAX_SAFE_INTEGER,r=Pw(CB(t)),n=0){this.states=r,this.index=n}nextU32(){let t=this.states[this.index];return t^=this.states[this.index]>>>11,t^=t<<7&2636928640,t^=t<<15&4022730752,t^=t>>>18,++this.index>=624&&(this.states=Pw(this.states),this.index=0),t>>>0}nextF32(){return this.nextU32()*tRe}nextU53(){let t=this.nextU32()>>>5,r=this.nextU32()>>>6;return t*67108864+r}nextF53(){return this.nextU53()*rRe}seed(t){this.states=Pw(CB(t)),this.index=0}};function hte(){return Math.ceil(Math.random()*Number.MAX_SAFE_INTEGER)}function iRe(e=hte()){let t=new aRe(e);return{next(){return t.nextF53()},seed(r){t.seed(r)}}}var oRe=class extends ad{boolean(t={}){typeof t=="number"&&(t={probability:t});let{probability:r=.5}=t;return r<=0?!1:r>=1?!0:this.faker.number.float()<r}};function Zs(e,t="refDate"){let r=new Date(e);if(Number.isNaN(r.valueOf()))throw new Jt(`Invalid ${t} date: ${e.toString()}`);return r}var Ey=()=>{throw new Jt("You cannot edit the locale data on the faker instance")};function sRe(e){let t={};return new Proxy(e,{has(){return!0},get(r,n){return typeof n=="symbol"||n==="nodeType"?r[n]:n in t?t[n]:t[n]=lRe(n,r[n])},set:Ey,deleteProperty:Ey})}function CR(e,...t){if(e===null)throw new Jt(`The locale data for '${t.join(".")}' aren't applicable to this locale.
  If you think this is a bug, please report it at: https://github.com/faker-js/faker`);if(e===void 0)throw new Jt(`The locale data for '${t.join(".")}' are missing in this locale.
  If this is a custom Faker instance, please make sure all required locales are used e.g. '[de_AT, de, en, base]'.
  Please contribute the missing data to the project or use a locale/Faker instance that has these data.
  For more information see https://fakerjs.dev/guide/localization.html`)}function lRe(e,t={}){return new Proxy(t,{has(r,n){return r[n]!=null},get(r,n){let a=r[n];return typeof n=="symbol"||n==="nodeType"||CR(a,e,n.toString()),a},set:Ey,deleteProperty:Ey})}var pte=class extends ad{anytime(t={}){let{refDate:r=this.faker.defaultRefDate()}=t,n=Zs(r).getTime();return this.between({from:n-1e3*60*60*24*365,to:n+1e3*60*60*24*365})}past(t={}){let{years:r=1,refDate:n=this.faker.defaultRefDate()}=t;if(r<=0)throw new Jt("Years must be greater than 0.");let a=Zs(n).getTime();return this.between({from:a-r*365*24*3600*1e3,to:a-1e3})}future(t={}){let{years:r=1,refDate:n=this.faker.defaultRefDate()}=t;if(r<=0)throw new Jt("Years must be greater than 0.");let a=Zs(n).getTime();return this.between({from:a+1e3,to:a+r*365*24*3600*1e3})}between(t){if(t==null||t.from==null||t.to==null)throw new Jt("Must pass an options object with `from` and `to` values.");let{from:r,to:n}=t,a=Zs(r,"from").getTime(),i=Zs(n,"to").getTime();if(a>i)throw new Jt("`from` date must be before `to` date.");return new Date(this.faker.number.int({min:a,max:i}))}betweens(t){if(t==null||t.from==null||t.to==null)throw new Jt("Must pass an options object with `from` and `to` values.");let{from:r,to:n,count:a=3}=t;return this.faker.helpers.multiple(()=>this.between({from:r,to:n}),{count:a}).sort((i,o)=>i.getTime()-o.getTime())}recent(t={}){let{days:r=1,refDate:n=this.faker.defaultRefDate()}=t;if(r<=0)throw new Jt("Days must be greater than 0.");let a=Zs(n).getTime();return this.between({from:a-r*24*3600*1e3,to:a-1e3})}soon(t={}){let{days:r=1,refDate:n=this.faker.defaultRefDate()}=t;if(r<=0)throw new Jt("Days must be greater than 0.");let a=Zs(n).getTime();return this.between({from:a+1e3,to:a+r*24*3600*1e3})}birthdate(t={}){let{mode:r="age",min:n=18,max:a=80,refDate:i=this.faker.defaultRefDate(),mode:o,min:s,max:l}=t;if([s,l,o].filter(p=>p!=null).length%3!==0)throw new Jt("The 'min', 'max', and 'mode' options must be set together.");let u=Zs(i),d=u.getUTCFullYear();switch(r){case"age":{let p=new Date(u).setUTCFullYear(d-a-1)+864e5,f=new Date(u).setUTCFullYear(d-n);if(p>f)throw new Jt(`Max age ${a} should be greater than or equal to min age ${n}.`);return this.between({from:p,to:f})}case"year":{let p=new Date(Date.UTC(0,0,2)).setUTCFullYear(n),f=new Date(Date.UTC(0,11,30)).setUTCFullYear(a);if(p>f)throw new Jt(`Max year ${a} should be greater than or equal to min year ${n}.`);return this.between({from:p,to:f})}}}},uRe=class extends pte{constructor(t){super(t),this.faker=t}month(t={}){let{abbreviated:r=!1,context:n=!1}=t,a=this.faker.definitions.date.month,i;r?i=n&&a.abbr_context!=null?"abbr_context":"abbr":i=n&&a.wide_context!=null?"wide_context":"wide";let o=a[i];return CR(o,"date.month",i),this.faker.helpers.arrayElement(o)}weekday(t={}){let{abbreviated:r=!1,context:n=!1}=t,a=this.faker.definitions.date.weekday,i;r?i=n&&a.abbr_context!=null?"abbr_context":"abbr":i=n&&a.wide_context!=null?"wide_context":"wide";let o=a[i];return CR(o,"date.weekday",i),this.faker.helpers.arrayElement(o)}timeZone(){return this.faker.helpers.arrayElement(this.faker.definitions.date.time_zone)}},cRe=/\.|\(/;function dRe(e,t,r=[t,t.rawDefinitions]){if(e.length===0)throw new Jt("Eval expression cannot be empty.");if(r.length===0)throw new Jt("Eval entrypoints cannot be empty.");let n=r,a=e;do{let o;a.startsWith("(")?[o,n]=hRe(a,n,e):[o,n]=fRe(a,n),a=a.substring(o),n=n.filter(s=>s!=null).map(s=>Array.isArray(s)?t.helpers.arrayElement(s):s)}while(a.length>0&&n.length>0);if(n.length===0)throw new Jt(`Cannot resolve expression '${e}'`);let i=n[0];return typeof i=="function"?i():i}function hRe(e,t,r){let[n,a]=pRe(e),i=e[n+1];switch(i){case".":case"(":case void 0:break;default:throw new Jt(`Expected dot ('.'), open parenthesis ('('), or nothing after function call but got '${i}'`)}return[n+(i==="."?2:1),t.map(o=>typeof o=="function"?o(...a):(console.warn(`[@faker-js/faker]: Invoking expressions which are not functions is deprecated since v9.0 and will be removed in v10.0.
Please remove the parentheses or replace the expression with an actual function.
${r}
${" ".repeat(r.length-e.length)}^`),o))]}function pRe(e){let t=e.indexOf(")",1);if(t===-1)throw new Jt(`Missing closing parenthesis in '${e}'`);for(;t!==-1;){let n=e.substring(1,t);try{return[t,JSON.parse(`[${n}]`)]}catch{if(!n.includes("'")&&!n.includes('"'))try{return[t,JSON.parse(`["${n}"]`)]}catch{}}t=e.indexOf(")",t+1)}t=e.lastIndexOf(")");let r=e.substring(1,t);return[t,[r]]}function fRe(e,t){let r=cRe.exec(e),n=((r==null?void 0:r[0])??"")===".",a=(r==null?void 0:r.index)??e.length,i=e.substring(0,a);if(i.length===0)throw new Jt(`Expression parts cannot be empty in '${e}'`);let o=e[a+1];if(n&&(o==null||o==="."||o==="("))throw new Jt(`Found dot without property name in '${e}'`);return[a+(n?1:0),t.map(s=>mRe(s,i))]}function mRe(e,t){switch(typeof e){case"function":{try{e=e()}catch{return}return e==null?void 0:e[t]}case"object":return e==null?void 0:e[t];default:return}}function gRe(e){let t=bRe(e.replace(/L?$/,"0"));return t===0?0:10-t}function bRe(e){e=e.replaceAll(/[\s-]/g,"");let t=0,r=!1;for(let n=e.length-1;n>=0;n--){let a=Number.parseInt(e[n]);r&&(a*=2,a>9&&(a=a%10+1)),t+=a,r=!r}return t%10}function TB(e,t,r,n){let a=1;if(t)switch(t){case"?":{a=e.datatype.boolean()?0:1;break}case"*":{let i=1;for(;e.datatype.boolean();)i*=2;a=e.number.int({min:0,max:i});break}case"+":{let i=1;for(;e.datatype.boolean();)i*=2;a=e.number.int({min:1,max:i});break}default:throw new Jt("Unknown quantifier symbol provided.")}else r!=null&&n!=null?a=e.number.int({min:Number.parseInt(r),max:Number.parseInt(n)}):r!=null&&n==null&&(a=Number.parseInt(r));return a}function yRe(e,t=""){let r=/(.)\{(\d+),(\d+)\}/,n=/(.)\{(\d+)\}/,a=/\[(\d+)-(\d+)\]/,i,o,s,l,u=r.exec(t);for(;u!=null;)i=Number.parseInt(u[2]),o=Number.parseInt(u[3]),i>o&&(s=o,o=i,i=s),l=e.number.int({min:i,max:o}),t=t.slice(0,u.index)+u[1].repeat(l)+t.slice(u.index+u[0].length),u=r.exec(t);for(u=n.exec(t);u!=null;)l=Number.parseInt(u[2]),t=t.slice(0,u.index)+u[1].repeat(l)+t.slice(u.index+u[0].length),u=n.exec(t);for(u=a.exec(t);u!=null;)i=Number.parseInt(u[1]),o=Number.parseInt(u[2]),i>o&&(s=o,o=i,i=s),t=t.slice(0,u.index)+e.number.int({min:i,max:o}).toString()+t.slice(u.index+u[0].length),u=a.exec(t);return t}function fte(e,t="",r="#"){let n="";for(let a=0;a<t.length;a++)t.charAt(a)===r?n+=e.number.int(9):t.charAt(a)==="!"?n+=e.number.int({min:2,max:9}):n+=t.charAt(a);return n}var mte=class extends ad{slugify(t=""){return t.normalize("NFKD").replaceAll(/[\u0300-\u036F]/g,"").replaceAll(" ","-").replaceAll(/[^\w.-]+/g,"")}replaceSymbols(t=""){let r=["A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"],n="";for(let a=0;a<t.length;a++)t.charAt(a)==="#"?n+=this.faker.number.int(9):t.charAt(a)==="?"?n+=this.arrayElement(r):t.charAt(a)==="*"?n+=this.faker.datatype.boolean()?this.arrayElement(r):this.faker.number.int(9):n+=t.charAt(a);return n}replaceCreditCardSymbols(t="6453-####-####-####-###L",r="#"){t=yRe(this.faker,t),t=fte(this.faker,t,r);let n=gRe(t);return t.replace("L",String(n))}fromRegExp(t){var f;let r=!1;t instanceof RegExp&&(r=t.flags.includes("i"),t=t.toString(),t=((f=/\/(.+?)\//.exec(t))==null?void 0:f[1])??"");let n,a,i,o=/([.A-Za-z0-9])(?:\{(\d+)(?:,(\d+)|)\}|(\?|\*|\+))(?![^[]*]|[^{]*})/,s=o.exec(t);for(;s!=null;){let m=s[2],b=s[3],y=s[4];i=TB(this.faker,y,m,b);let S;s[1]==="."?S=this.faker.string.alphanumeric(i):r?S=this.faker.string.fromCharacters([s[1].toLowerCase(),s[1].toUpperCase()],i):S=s[1].repeat(i),t=t.slice(0,s.index)+S+t.slice(s.index+s[0].length),s=o.exec(t)}let l=/(\d-\d|\w-\w|\d|\w|[-!@#$&()`.+,/"])/,u=/\[(\^|)(-|)(.+?)\](?:\{(\d+)(?:,(\d+)|)\}|(\?|\*|\+)|)/;for(s=u.exec(t);s!=null;){let m=s[1]==="^",b=s[2]==="-",y=s[4],S=s[5],w=s[6],E=[],k=s[3],_=l.exec(k);for(b&&E.push(45);_!=null;){if(_[0].includes("-")){let C=_[0].split("-").map(F=>F.codePointAt(0)??Number.NaN);if(n=C[0],a=C[1],n>a)throw new Jt("Character range provided is out of order.");for(let F=n;F<=a;F++)if(r&&Number.isNaN(Number(String.fromCodePoint(F)))){let L=String.fromCodePoint(F);E.push(L.toUpperCase().codePointAt(0)??Number.NaN,L.toLowerCase().codePointAt(0)??Number.NaN)}else E.push(F)}else r&&Number.isNaN(Number(_[0]))?E.push(_[0].toUpperCase().codePointAt(0)??Number.NaN,_[0].toLowerCase().codePointAt(0)??Number.NaN):E.push(_[0].codePointAt(0)??Number.NaN);k=k.substring(_[0].length),_=l.exec(k)}if(i=TB(this.faker,w,y,S),m){let C=-1;for(let F=48;F<=57;F++){if(C=E.indexOf(F),C>-1){E.splice(C,1);continue}E.push(F)}for(let F=65;F<=90;F++){if(C=E.indexOf(F),C>-1){E.splice(C,1);continue}E.push(F)}for(let F=97;F<=122;F++){if(C=E.indexOf(F),C>-1){E.splice(C,1);continue}E.push(F)}}let R=this.multiple(()=>String.fromCodePoint(this.arrayElement(E)),{count:i}).join("");t=t.slice(0,s.index)+R+t.slice(s.index+s[0].length),s=u.exec(t)}let d=/(.)\{(\d+),(\d+)\}/;for(s=d.exec(t);s!=null;){if(n=Number.parseInt(s[2]),a=Number.parseInt(s[3]),n>a)throw new Jt("Numbers out of order in {} quantifier.");i=this.faker.number.int({min:n,max:a}),t=t.slice(0,s.index)+s[1].repeat(i)+t.slice(s.index+s[0].length),s=d.exec(t)}let p=/(.)\{(\d+)\}/;for(s=p.exec(t);s!=null;)i=Number.parseInt(s[2]),t=t.slice(0,s.index)+s[1].repeat(i)+t.slice(s.index+s[0].length),s=p.exec(t);return t}shuffle(t,r={}){let{inplace:n=!1}=r;n||(t=[...t]);for(let a=t.length-1;a>0;--a){let i=this.faker.number.int(a);[t[a],t[i]]=[t[i],t[a]]}return t}uniqueArray(t,r){if(Array.isArray(t)){let a=[...new Set(t)];return this.shuffle(a).splice(0,r)}let n=new Set;try{if(typeof t=="function"){let a=1e3*r,i=0;for(;n.size<r&&i<a;)n.add(t()),i++}}catch{}return[...n]}mustache(t,r){if(t==null)return"";for(let n in r){let a=new RegExp(`{{${n}}}`,"g"),i=r[n];typeof i=="string"&&(i=i.replaceAll("$","$$$$")),t=t.replace(a,i)}return t}maybe(t,r={}){if(this.faker.datatype.boolean(r))return t()}objectKey(t){let r=Object.keys(t);return this.arrayElement(r)}objectValue(t){let r=this.faker.helpers.objectKey(t);return t[r]}objectEntry(t){let r=this.faker.helpers.objectKey(t);return[r,t[r]]}arrayElement(t){if(t.length===0)throw new Jt("Cannot get value from empty dataset.");let r=t.length>1?this.faker.number.int({max:t.length-1}):0;return t[r]}weightedArrayElement(t){if(t.length===0)throw new Jt("weightedArrayElement expects an array with at least one element");if(!t.every(i=>i.weight>0))throw new Jt("weightedArrayElement expects an array of { weight, value } objects where weight is a positive number");let r=t.reduce((i,{weight:o})=>i+o,0),n=this.faker.number.float({min:0,max:r}),a=0;for(let{weight:i,value:o}of t)if(a+=i,n<a)return o;return t.at(-1).value}arrayElements(t,r){if(t.length===0)return[];let n=this.rangeToNumber(r??{min:1,max:t.length});if(n>=t.length)return this.shuffle(t);if(n<=0)return[];let a=[...t],i=t.length,o=i-n,s,l;for(;i-- >o;)l=this.faker.number.int(i),s=a[l],a[l]=a[i],a[i]=s;return a.slice(o)}enumValue(t){let r=Object.keys(t).filter(a=>Number.isNaN(Number(a))),n=this.arrayElement(r);return t[n]}rangeToNumber(t){return typeof t=="number"?t:this.faker.number.int(t)}multiple(t,r={}){let n=this.rangeToNumber(r.count??3);return n<=0?[]:Array.from({length:n},t)}},vRe=class extends mte{constructor(t){super(t),this.faker=t}fake(t){t=typeof t=="string"?t:this.arrayElement(t);let r=t.search(/{{[a-z]/),n=t.indexOf("}}",r);if(r===-1||n===-1)return t;let a=t.substring(r+2,n+2).replace("}}","").replace("{{",""),i=dRe(a,this.faker),o=String(i),s=t.substring(0,r)+o+t.substring(n+2);return this.fake(s)}},gte=class extends ad{latitude(t={}){let{max:r=90,min:n=-90,precision:a=4}=t;return this.faker.number.float({min:n,max:r,fractionDigits:a})}longitude(t={}){let{max:r=180,min:n=-180,precision:a=4}=t;return this.faker.number.float({max:r,min:n,fractionDigits:a})}nearbyGPSCoordinate(t={}){let{origin:r,radius:n=10,isMetric:a=!1}=t;if(r==null)return[this.latitude(),this.longitude()];let i=this.faker.number.float({max:2*Math.PI,fractionDigits:5}),o=a?n:n*1.60934,s=this.faker.number.float({max:o,fractionDigits:3})*.995,l=4e4/360,u=s/l,d=[r[0]+Math.sin(i)*u,r[1]+Math.cos(i)*u];return d[0]=d[0]%180,(d[0]<-90||d[0]>90)&&(d[0]=Math.sign(d[0])*180-d[0],d[1]+=180),d[1]=(d[1]%360+540)%360-180,[d[0],d[1]]}},SRe=class extends gte{constructor(t){super(t),this.faker=t}zipCode(t={}){typeof t=="string"&&(t={format:t});let{state:r}=t;if(r!=null){let a=this.faker.definitions.location.postcode_by_state[r];if(a==null)throw new Jt(`No zip code definition found for state "${r}"`);return this.faker.helpers.fake(a)}let{format:n=this.faker.definitions.location.postcode}=t;return typeof n=="string"&&(n=[n]),n=this.faker.helpers.arrayElement(n),this.faker.helpers.replaceSymbols(n)}city(){return this.faker.helpers.fake(this.faker.definitions.location.city_pattern)}buildingNumber(){return this.faker.helpers.arrayElement(this.faker.definitions.location.building_number).replaceAll(/#+/g,t=>this.faker.string.numeric({length:t.length,allowLeadingZeros:!1}))}street(){return this.faker.helpers.fake(this.faker.definitions.location.street_pattern)}streetAddress(t={}){typeof t=="boolean"&&(t={useFullAddress:t});let{useFullAddress:r}=t,n=this.faker.definitions.location.street_address[r?"full":"normal"];return this.faker.helpers.fake(n)}secondaryAddress(){return this.faker.helpers.fake(this.faker.definitions.location.secondary_address).replaceAll(/#+/g,t=>this.faker.string.numeric({length:t.length,allowLeadingZeros:!1}))}county(){return this.faker.helpers.arrayElement(this.faker.definitions.location.county)}country(){return this.faker.helpers.arrayElement(this.faker.definitions.location.country)}continent(){return this.faker.helpers.arrayElement(this.faker.definitions.location.continent)}countryCode(t={}){typeof t=="string"&&(t={variant:t});let{variant:r="alpha-2"}=t,n=(()=>{switch(r){case"numeric":return"numeric";case"alpha-3":return"alpha3";case"alpha-2":return"alpha2"}})();return this.faker.helpers.arrayElement(this.faker.definitions.location.country_code)[n]}state(t={}){let{abbreviated:r=!1}=t,n=r?this.faker.definitions.location.state_abbr:this.faker.definitions.location.state;return this.faker.helpers.arrayElement(n)}direction(t={}){let{abbreviated:r=!1}=t;return r?this.faker.helpers.arrayElement([...this.faker.definitions.location.direction.cardinal_abbr,...this.faker.definitions.location.direction.ordinal_abbr]):this.faker.helpers.arrayElement([...this.faker.definitions.location.direction.cardinal,...this.faker.definitions.location.direction.ordinal])}cardinalDirection(t={}){let{abbreviated:r=!1}=t;return r?this.faker.helpers.arrayElement(this.faker.definitions.location.direction.cardinal_abbr):this.faker.helpers.arrayElement(this.faker.definitions.location.direction.cardinal)}ordinalDirection(t={}){let{abbreviated:r=!1}=t;return r?this.faker.helpers.arrayElement(this.faker.definitions.location.direction.ordinal_abbr):this.faker.helpers.arrayElement(this.faker.definitions.location.direction.ordinal)}timeZone(){return this.faker.helpers.arrayElement(this.faker.definitions.location.time_zone)}language(){return this.faker.helpers.arrayElement(this.faker.definitions.location.language)}},ERe=class extends ad{int(t={}){typeof t=="number"&&(t={max:t});let{min:r=0,max:n=Number.MAX_SAFE_INTEGER,multipleOf:a=1}=t;if(!Number.isInteger(a))throw new Jt("multipleOf should be an integer.");if(a<=0)throw new Jt("multipleOf should be greater than 0.");let i=Math.ceil(r/a),o=Math.floor(n/a);if(i===o)return i*a;if(o<i)throw n>=r?new Jt(`No suitable integer value between ${r} and ${n} found.`):new Jt(`Max ${n} should be greater than min ${r}.`);let s=this.faker._randomizer.next(),l=o-i+1;return Math.floor(s*l+i)*a}float(t={}){typeof t=="number"&&(t={max:t});let{min:r=0,max:n=1,fractionDigits:a,multipleOf:i,multipleOf:o=a==null?void 0:10**-a}=t;if(n<r)throw new Jt(`Max ${n} should be greater than min ${r}.`);if(a!=null){if(i!=null)throw new Jt("multipleOf and fractionDigits cannot be set at the same time.");if(!Number.isInteger(a))throw new Jt("fractionDigits should be an integer.");if(a<0)throw new Jt("fractionDigits should be greater than or equal to 0.")}if(o!=null){if(o<=0)throw new Jt("multipleOf should be greater than 0.");let s=Math.log10(o),l=o<1&&Number.isInteger(s)?10**-s:1/o;return this.int({min:r*l,max:n*l})/l}return this.faker._randomizer.next()*(n-r)+r}binary(t={}){typeof t=="number"&&(t={max:t});let{min:r=0,max:n=1}=t;return this.int({max:n,min:r}).toString(2)}octal(t={}){typeof t=="number"&&(t={max:t});let{min:r=0,max:n=7}=t;return this.int({max:n,min:r}).toString(8)}hex(t={}){typeof t=="number"&&(t={max:t});let{min:r=0,max:n=15}=t;return this.int({max:n,min:r}).toString(16)}bigInt(t={}){(typeof t=="bigint"||typeof t=="number"||typeof t=="string"||typeof t=="boolean")&&(t={max:t});let r=BigInt(t.min??0),n=BigInt(t.max??r+BigInt(999999999999999)),a=BigInt(t.multipleOf??1);if(n<r)throw new Jt(`Max ${n} should be larger than min ${r}.`);if(a<=BigInt(0))throw new Jt("multipleOf should be greater than 0.");let i=r/a+(r%a>0n?1n:0n),o=n/a-(n%a<0n?1n:0n);if(i===o)return i*a;if(o<i)throw new Jt(`No suitable bigint value between ${r} and ${n} found.`);let s=o-i+1n,l=BigInt(this.faker.string.numeric({length:s.toString(10).length,allowLeadingZeros:!0}))%s;return(i+l)*a}romanNumeral(t={}){typeof t=="number"&&(t={max:t});let{min:r=1,max:n=3999}=t;if(r<1)throw new Jt(`Min value ${r} should be 1 or greater.`);if(n>3999)throw new Jt(`Max value ${n} should be 3999 or less.`);let a=this.int({min:r,max:n}),i=[["M",1e3],["CM",900],["D",500],["CD",400],["C",100],["XC",90],["L",50],["XL",40],["X",10],["IX",9],["V",5],["IV",4],["I",1]],o="";for(let[s,l]of i)o+=s.repeat(Math.floor(a/l)),a%=l;return o}},bte="0123456789ABCDEFGHJKMNPQRSTVWXYZ";function xRe(e){let t=e.valueOf(),r="";for(let n=10;n>0;n--){let a=t%32;r=bte[a]+r,t=(t-a)/32}return r}var Gb=[..."ABCDEFGHIJKLMNOPQRSTUVWXYZ"],jb=[..."abcdefghijklmnopqrstuvwxyz"],AB=[..."0123456789"],wRe=class extends ad{fromCharacters(t,r=1){if(r=this.faker.helpers.rangeToNumber(r),r<=0)return"";if(typeof t=="string"&&(t=[...t]),t.length===0)throw new Jt("Unable to generate string: No characters to select from.");return this.faker.helpers.multiple(()=>this.faker.helpers.arrayElement(t),{count:r}).join("")}alpha(t={}){typeof t=="number"&&(t={length:t});let r=this.faker.helpers.rangeToNumber(t.length??1);if(r<=0)return"";let{casing:n="mixed"}=t,{exclude:a=[]}=t;typeof a=="string"&&(a=[...a]);let i;switch(n){case"upper":{i=[...Gb];break}case"lower":{i=[...jb];break}case"mixed":{i=[...jb,...Gb];break}}return i=i.filter(o=>!a.includes(o)),this.fromCharacters(i,r)}alphanumeric(t={}){typeof t=="number"&&(t={length:t});let r=this.faker.helpers.rangeToNumber(t.length??1);if(r<=0)return"";let{casing:n="mixed"}=t,{exclude:a=[]}=t;typeof a=="string"&&(a=[...a]);let i=[...AB];switch(n){case"upper":{i.push(...Gb);break}case"lower":{i.push(...jb);break}case"mixed":{i.push(...jb,...Gb);break}}return i=i.filter(o=>!a.includes(o)),this.fromCharacters(i,r)}binary(t={}){let{prefix:r="0b"}=t,n=r;return n+=this.fromCharacters(["0","1"],t.length??1),n}octal(t={}){let{prefix:r="0o"}=t,n=r;return n+=this.fromCharacters(["0","1","2","3","4","5","6","7"],t.length??1),n}hexadecimal(t={}){let{casing:r="mixed",prefix:n="0x"}=t,a=this.faker.helpers.rangeToNumber(t.length??1);if(a<=0)return n;let i=this.fromCharacters(["0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f","A","B","C","D","E","F"],a);return r==="upper"?i=i.toUpperCase():r==="lower"&&(i=i.toLowerCase()),`${n}${i}`}numeric(t={}){typeof t=="number"&&(t={length:t});let r=this.faker.helpers.rangeToNumber(t.length??1);if(r<=0)return"";let{allowLeadingZeros:n=!0}=t,{exclude:a=[]}=t;typeof a=="string"&&(a=[...a]);let i=AB.filter(s=>!a.includes(s));if(i.length===0||i.length===1&&!n&&i[0]==="0")throw new Jt("Unable to generate numeric string, because all possible digits are excluded.");let o="";return!n&&!a.includes("0")&&(o+=this.faker.helpers.arrayElement(i.filter(s=>s!=="0"))),o+=this.fromCharacters(i,r-o.length),o}sample(t=10){t=this.faker.helpers.rangeToNumber(t);let r={min:33,max:125},n="";for(;n.length<t;)n+=String.fromCodePoint(this.faker.number.int(r));return n}uuid(){return"xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replaceAll("x",()=>this.faker.number.hex({min:0,max:15})).replaceAll("y",()=>this.faker.number.hex({min:8,max:11}))}ulid(t={}){let{refDate:r=this.faker.defaultRefDate()}=t,n=Zs(r);return xRe(n)+this.fromCharacters(bte,16)}nanoid(t=21){if(t=this.faker.helpers.rangeToNumber(t),t<=0)return"";let r=[{value:()=>this.alphanumeric(1),weight:62},{value:()=>this.faker.helpers.arrayElement(["_","-"]),weight:2}],n="";for(;n.length<t;){let a=this.faker.helpers.weightedArrayElement(r);n+=a()}return n}symbol(t=1){return this.fromCharacters(["!",'"',"#","$","%","&","'","(",")","*","+",",","-",".","/",":",";","<","=",">","?","@","[","\\","]","^","_","`","{","|","}","~"],t)}},yte=class{constructor(t={}){mt(this,"_defaultRefDate",()=>new Date);mt(this,"_randomizer");mt(this,"datatype",new oRe(this));mt(this,"date",new pte(this));mt(this,"helpers",new mte(this));mt(this,"location",new gte(this));mt(this,"number",new ERe(this));mt(this,"string",new wRe(this));let{randomizer:r,seed:n}=t;r!=null&&n!=null&&r.seed(n),this._randomizer=r??iRe(n)}get defaultRefDate(){return this._defaultRefDate}setDefaultRefDate(t=()=>new Date){typeof t=="function"?this._defaultRefDate=t:this._defaultRefDate=()=>new Date(t)}seed(t=hte()){return this._randomizer.seed(t),t}};new yte;function kRe(e){let t={};for(let r of e)for(let n in r){let a=r[n];t[n]===void 0?t[n]={...a}:t[n]={...a,...t[n]}}return t}var CRe=class extends In{dog(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.dog)}cat(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.cat)}snake(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.snake)}bear(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.bear)}lion(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.lion)}cetacean(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.cetacean)}horse(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.horse)}bird(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.bird)}cow(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.cow)}fish(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.fish)}crocodilia(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.crocodilia)}insect(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.insect)}rabbit(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.rabbit)}rodent(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.rodent)}type(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.type)}petName(){return this.faker.helpers.arrayElement(this.faker.definitions.animal.pet_name)}},TRe=class extends In{author(){return this.faker.helpers.arrayElement(this.faker.definitions.book.author)}format(){return this.faker.helpers.arrayElement(this.faker.definitions.book.format)}genre(){return this.faker.helpers.arrayElement(this.faker.definitions.book.genre)}publisher(){return this.faker.helpers.arrayElement(this.faker.definitions.book.publisher)}series(){return this.faker.helpers.arrayElement(this.faker.definitions.book.series)}title(){return this.faker.helpers.arrayElement(this.faker.definitions.book.title)}},ARe={0:[[1999999,2],[2279999,3],[2289999,4],[3689999,3],[3699999,4],[6389999,3],[6397999,4],[6399999,7],[6449999,3],[6459999,7],[6479999,3],[6489999,7],[6549999,3],[6559999,4],[6999999,3],[8499999,4],[8999999,5],[9499999,6],[9999999,7]],1:[[99999,3],[299999,2],[349999,3],[399999,4],[499999,3],[699999,2],[999999,4],[3979999,3],[5499999,4],[6499999,5],[6799999,4],[6859999,5],[7139999,4],[7169999,3],[7319999,4],[7399999,7],[7749999,5],[7753999,7],[7763999,5],[7764999,7],[7769999,5],[7782999,7],[7899999,5],[7999999,4],[8004999,5],[8049999,5],[8379999,5],[8384999,7],[8671999,5],[8675999,4],[8697999,5],[9159999,6],[9165059,7],[9168699,6],[9169079,7],[9195999,6],[9196549,7],[9729999,6],[9877999,4],[9911499,6],[9911999,7],[9989899,6],[9999999,7]]},FRe=class extends In{department(){return this.faker.helpers.arrayElement(this.faker.definitions.commerce.department)}productName(){return`${this.productAdjective()} ${this.productMaterial()} ${this.product()}`}price(t={}){let{dec:r=2,max:n=1e3,min:a=1,symbol:i=""}=t;if(a<0||n<0)return`${i}0`;if(a===n)return`${i}${a.toFixed(r)}`;let o=this.faker.number.float({min:a,max:n,fractionDigits:r});if(r===0)return`${i}${o.toFixed(r)}`;let s=o*10**r%10,l=this.faker.helpers.weightedArrayElement([{weight:5,value:9},{weight:3,value:5},{weight:1,value:0},{weight:1,value:this.faker.number.int({min:0,max:9})}]),u=(1/10)**r,d=s*u,p=l*u,f=o-d+p;return a<=f&&f<=n?`${i}${f.toFixed(r)}`:`${i}${o.toFixed(r)}`}productAdjective(){return this.faker.helpers.arrayElement(this.faker.definitions.commerce.product_name.adjective)}productMaterial(){return this.faker.helpers.arrayElement(this.faker.definitions.commerce.product_name.material)}product(){return this.faker.helpers.arrayElement(this.faker.definitions.commerce.product_name.product)}productDescription(){return this.faker.helpers.fake(this.faker.definitions.commerce.product_description)}isbn(t={}){var y;typeof t=="number"&&(t={variant:t});let{variant:r=13,separator:n="-"}=t,a="978",[i,o]=this.faker.helpers.objectEntry(ARe),s=this.faker.string.numeric(8),l=Number.parseInt(s.slice(0,-1)),u=(y=o.find(([S])=>l<=S))==null?void 0:y[1];if(!u)throw new Jt(`Unable to find a registrant length for the group ${i}`);let d=s.slice(0,u),p=s.slice(u),f=[a,i,d,p];r===10&&f.shift();let m=f.join(""),b=0;for(let S=0;S<r-1;S++){let w=r===10?S+1:S%2?3:1;b+=w*Number.parseInt(m[S])}return b=r===10?b%11:(10-b%10)%10,f.push(b===10?"X":b.toString()),f.join(n)}},_Re=class extends In{name(){return this.faker.helpers.fake(this.faker.definitions.company.name_pattern)}catchPhrase(){return[this.catchPhraseAdjective(),this.catchPhraseDescriptor(),this.catchPhraseNoun()].join(" ")}buzzPhrase(){return[this.buzzVerb(),this.buzzAdjective(),this.buzzNoun()].join(" ")}catchPhraseAdjective(){return this.faker.helpers.arrayElement(this.faker.definitions.company.adjective)}catchPhraseDescriptor(){return this.faker.helpers.arrayElement(this.faker.definitions.company.descriptor)}catchPhraseNoun(){return this.faker.helpers.arrayElement(this.faker.definitions.company.noun)}buzzAdjective(){return this.faker.helpers.arrayElement(this.faker.definitions.company.buzz_adjective)}buzzVerb(){return this.faker.helpers.arrayElement(this.faker.definitions.company.buzz_verb)}buzzNoun(){return this.faker.helpers.arrayElement(this.faker.definitions.company.buzz_noun)}},RRe=class extends In{column(){return this.faker.helpers.arrayElement(this.faker.definitions.database.column)}type(){return this.faker.helpers.arrayElement(this.faker.definitions.database.type)}collation(){return this.faker.helpers.arrayElement(this.faker.definitions.database.collation)}engine(){return this.faker.helpers.arrayElement(this.faker.definitions.database.engine)}mongodbObjectId(){return this.faker.string.hexadecimal({length:24,casing:"lower",prefix:""})}},DRe={alpha:["A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"],formats:[{country:"AL",total:28,bban:[{type:"n",count:8},{type:"c",count:16}],format:"ALkk bbbs sssx cccc cccc cccc cccc"},{country:"AD",total:24,bban:[{type:"n",count:8},{type:"c",count:12}],format:"ADkk bbbb ssss cccc cccc cccc"},{country:"AT",total:20,bban:[{type:"n",count:5},{type:"n",count:11}],format:"ATkk bbbb bccc cccc cccc"},{country:"AZ",total:28,bban:[{type:"a",count:4},{type:"n",count:20}],format:"AZkk bbbb cccc cccc cccc cccc cccc"},{country:"BH",total:22,bban:[{type:"a",count:4},{type:"c",count:14}],format:"BHkk bbbb cccc cccc cccc cc"},{country:"BE",total:16,bban:[{type:"n",count:3},{type:"n",count:9}],format:"BEkk bbbc cccc ccxx"},{country:"BA",total:20,bban:[{type:"n",count:6},{type:"n",count:10}],format:"BAkk bbbs sscc cccc ccxx"},{country:"BR",total:29,bban:[{type:"n",count:13},{type:"n",count:10},{type:"a",count:1},{type:"c",count:1}],format:"BRkk bbbb bbbb ssss sccc cccc ccct n"},{country:"BG",total:22,bban:[{type:"a",count:4},{type:"n",count:6},{type:"c",count:8}],format:"BGkk bbbb ssss ddcc cccc cc"},{country:"CR",total:22,bban:[{type:"n",count:1},{type:"n",count:3},{type:"n",count:14}],format:"CRkk xbbb cccc cccc cccc cc"},{country:"HR",total:21,bban:[{type:"n",count:7},{type:"n",count:10}],format:"HRkk bbbb bbbc cccc cccc c"},{country:"CY",total:28,bban:[{type:"n",count:8},{type:"c",count:16}],format:"CYkk bbbs ssss cccc cccc cccc cccc"},{country:"CZ",total:24,bban:[{type:"n",count:10},{type:"n",count:10}],format:"CZkk bbbb ssss sscc cccc cccc"},{country:"DK",total:18,bban:[{type:"n",count:4},{type:"n",count:10}],format:"DKkk bbbb cccc cccc cc"},{country:"DO",total:28,bban:[{type:"a",count:4},{type:"n",count:20}],format:"DOkk bbbb cccc cccc cccc cccc cccc"},{country:"TL",total:23,bban:[{type:"n",count:3},{type:"n",count:16}],format:"TLkk bbbc cccc cccc cccc cxx"},{country:"EE",total:20,bban:[{type:"n",count:4},{type:"n",count:12}],format:"EEkk bbss cccc cccc cccx"},{country:"FO",total:18,bban:[{type:"n",count:4},{type:"n",count:10}],format:"FOkk bbbb cccc cccc cx"},{country:"FI",total:18,bban:[{type:"n",count:6},{type:"n",count:8}],format:"FIkk bbbb bbcc cccc cx"},{country:"FR",total:27,bban:[{type:"n",count:10},{type:"c",count:11},{type:"n",count:2}],format:"FRkk bbbb bggg ggcc cccc cccc cxx"},{country:"GE",total:22,bban:[{type:"a",count:2},{type:"n",count:16}],format:"GEkk bbcc cccc cccc cccc cc"},{country:"DE",total:22,bban:[{type:"n",count:8},{type:"n",count:10}],format:"DEkk bbbb bbbb cccc cccc cc"},{country:"GI",total:23,bban:[{type:"a",count:4},{type:"c",count:15}],format:"GIkk bbbb cccc cccc cccc ccc"},{country:"GR",total:27,bban:[{type:"n",count:7},{type:"c",count:16}],format:"GRkk bbbs sssc cccc cccc cccc ccc"},{country:"GL",total:18,bban:[{type:"n",count:4},{type:"n",count:10}],format:"GLkk bbbb cccc cccc cc"},{country:"GT",total:28,bban:[{type:"c",count:4},{type:"c",count:4},{type:"c",count:16}],format:"GTkk bbbb mmtt cccc cccc cccc cccc"},{country:"HU",total:28,bban:[{type:"n",count:8},{type:"n",count:16}],format:"HUkk bbbs sssk cccc cccc cccc cccx"},{country:"IS",total:26,bban:[{type:"n",count:6},{type:"n",count:16}],format:"ISkk bbbb sscc cccc iiii iiii ii"},{country:"IE",total:22,bban:[{type:"a",count:4},{type:"n",count:6},{type:"n",count:8}],format:"IEkk aaaa bbbb bbcc cccc cc"},{country:"IL",total:23,bban:[{type:"n",count:6},{type:"n",count:13}],format:"ILkk bbbn nncc cccc cccc ccc"},{country:"IT",total:27,bban:[{type:"a",count:1},{type:"n",count:10},{type:"c",count:12}],format:"ITkk xaaa aabb bbbc cccc cccc ccc"},{country:"JO",total:30,bban:[{type:"a",count:4},{type:"n",count:4},{type:"n",count:18}],format:"JOkk bbbb nnnn cccc cccc cccc cccc cc"},{country:"KZ",total:20,bban:[{type:"n",count:3},{type:"c",count:13}],format:"KZkk bbbc cccc cccc cccc"},{country:"XK",total:20,bban:[{type:"n",count:4},{type:"n",count:12}],format:"XKkk bbbb cccc cccc cccc"},{country:"KW",total:30,bban:[{type:"a",count:4},{type:"c",count:22}],format:"KWkk bbbb cccc cccc cccc cccc cccc cc"},{country:"LV",total:21,bban:[{type:"a",count:4},{type:"c",count:13}],format:"LVkk bbbb cccc cccc cccc c"},{country:"LB",total:28,bban:[{type:"n",count:4},{type:"c",count:20}],format:"LBkk bbbb cccc cccc cccc cccc cccc"},{country:"LI",total:21,bban:[{type:"n",count:5},{type:"c",count:12}],format:"LIkk bbbb bccc cccc cccc c"},{country:"LT",total:20,bban:[{type:"n",count:5},{type:"n",count:11}],format:"LTkk bbbb bccc cccc cccc"},{country:"LU",total:20,bban:[{type:"n",count:3},{type:"c",count:13}],format:"LUkk bbbc cccc cccc cccc"},{country:"MK",total:19,bban:[{type:"n",count:3},{type:"c",count:10},{type:"n",count:2}],format:"MKkk bbbc cccc cccc cxx"},{country:"MT",total:31,bban:[{type:"a",count:4},{type:"n",count:5},{type:"c",count:18}],format:"MTkk bbbb ssss sccc cccc cccc cccc ccc"},{country:"MR",total:27,bban:[{type:"n",count:10},{type:"n",count:13}],format:"MRkk bbbb bsss sscc cccc cccc cxx"},{country:"MU",total:30,bban:[{type:"a",count:4},{type:"n",count:4},{type:"n",count:15},{type:"a",count:3}],format:"MUkk bbbb bbss cccc cccc cccc 000d dd"},{country:"MC",total:27,bban:[{type:"n",count:10},{type:"c",count:11},{type:"n",count:2}],format:"MCkk bbbb bsss sscc cccc cccc cxx"},{country:"MD",total:24,bban:[{type:"c",count:2},{type:"c",count:18}],format:"MDkk bbcc cccc cccc cccc cccc"},{country:"ME",total:22,bban:[{type:"n",count:3},{type:"n",count:15}],format:"MEkk bbbc cccc cccc cccc xx"},{country:"NL",total:18,bban:[{type:"a",count:4},{type:"n",count:10}],format:"NLkk bbbb cccc cccc cc"},{country:"NO",total:15,bban:[{type:"n",count:4},{type:"n",count:7}],format:"NOkk bbbb cccc ccx"},{country:"PK",total:24,bban:[{type:"a",count:4},{type:"n",count:16}],format:"PKkk bbbb cccc cccc cccc cccc"},{country:"PS",total:29,bban:[{type:"a",count:4},{type:"n",count:9},{type:"n",count:12}],format:"PSkk bbbb xxxx xxxx xccc cccc cccc c"},{country:"PL",total:28,bban:[{type:"n",count:8},{type:"n",count:16}],format:"PLkk bbbs sssx cccc cccc cccc cccc"},{country:"PT",total:25,bban:[{type:"n",count:8},{type:"n",count:13}],format:"PTkk bbbb ssss cccc cccc cccx x"},{country:"QA",total:29,bban:[{type:"a",count:4},{type:"c",count:21}],format:"QAkk bbbb cccc cccc cccc cccc cccc c"},{country:"RO",total:24,bban:[{type:"a",count:4},{type:"c",count:16}],format:"ROkk bbbb cccc cccc cccc cccc"},{country:"SM",total:27,bban:[{type:"a",count:1},{type:"n",count:10},{type:"c",count:12}],format:"SMkk xaaa aabb bbbc cccc cccc ccc"},{country:"SA",total:24,bban:[{type:"n",count:2},{type:"c",count:18}],format:"SAkk bbcc cccc cccc cccc cccc"},{country:"RS",total:22,bban:[{type:"n",count:3},{type:"n",count:15}],format:"RSkk bbbc cccc cccc cccc xx"},{country:"SK",total:24,bban:[{type:"n",count:10},{type:"n",count:10}],format:"SKkk bbbb ssss sscc cccc cccc"},{country:"SI",total:19,bban:[{type:"n",count:5},{type:"n",count:10}],format:"SIkk bbss sccc cccc cxx"},{country:"ES",total:24,bban:[{type:"n",count:10},{type:"n",count:10}],format:"ESkk bbbb gggg xxcc cccc cccc"},{country:"SE",total:24,bban:[{type:"n",count:3},{type:"n",count:17}],format:"SEkk bbbc cccc cccc cccc cccc"},{country:"CH",total:21,bban:[{type:"n",count:5},{type:"c",count:12}],format:"CHkk bbbb bccc cccc cccc c"},{country:"TN",total:24,bban:[{type:"n",count:5},{type:"n",count:15}],format:"TNkk bbss sccc cccc cccc cccc"},{country:"TR",total:26,bban:[{type:"n",count:5},{type:"n",count:1},{type:"n",count:16}],format:"TRkk bbbb bxcc cccc cccc cccc cc"},{country:"AE",total:23,bban:[{type:"n",count:3},{type:"n",count:16}],format:"AEkk bbbc cccc cccc cccc ccc"},{country:"GB",total:22,bban:[{type:"a",count:4},{type:"n",count:6},{type:"n",count:8}],format:"GBkk bbbb ssss sscc cccc cc"},{country:"VG",total:24,bban:[{type:"a",count:4},{type:"n",count:16}],format:"VGkk bbbb cccc cccc cccc cccc"}],iso3166:["AD","AE","AF","AG","AI","AL","AM","AO","AQ","AR","AS","AT","AU","AW","AX","AZ","BA","BB","BD","BE","BF","BG","BH","BI","BJ","BL","BM","BN","BO","BQ","BR","BS","BT","BV","BW","BY","BZ","CA","CC","CD","CF","CG","CH","CI","CK","CL","CM","CN","CO","CR","CU","CV","CW","CX","CY","CZ","DE","DJ","DK","DM","DO","DZ","EC","EE","EG","EH","ER","ES","ET","FI","FJ","FK","FM","FO","FR","GA","GB","GD","GE","GF","GG","GH","GI","GL","GM","GN","GP","GQ","GR","GS","GT","GU","GW","GY","HK","HM","HN","HR","HT","HU","ID","IE","IL","IM","IN","IO","IQ","IR","IS","IT","JE","JM","JO","JP","KE","KG","KH","KI","KM","KN","KP","KR","KW","KY","KZ","LA","LB","LC","LI","LK","LR","LS","LT","LU","LV","LY","MA","MC","MD","ME","MF","MG","MH","MK","ML","MM","MN","MO","MP","MQ","MR","MS","MT","MU","MV","MW","MX","MY","MZ","NA","NC","NE","NF","NG","NI","NL","NO","NP","NR","NU","NZ","OM","PA","PE","PF","PG","PH","PK","PL","PM","PN","PR","PS","PT","PW","PY","QA","RE","RO","RS","RU","RW","SA","SB","SC","SD","SE","SG","SH","SI","SJ","SK","SL","SM","SN","SO","SR","SS","ST","SV","SX","SY","SZ","TC","TD","TF","TG","TH","TJ","TK","TL","TM","TN","TO","TR","TT","TV","TW","TZ","UA","UG","UM","US","UY","UZ","VA","VC","VE","VG","VI","VN","VU","WF","WS","XK","YE","YT","ZA","ZM","ZW"],mod97:e=>{let t=0;for(let r of e)t=(t*10+ +r)%97;return t},pattern10:["01","02","03","04","05","06","07","08","09"],pattern100:["001","002","003","004","005","006","007","008","009"],toDigitString:e=>e.replaceAll(/[A-Z]/gi,t=>String((t.toUpperCase().codePointAt(0)??Number.NaN)-55))},Vs=DRe;function NRe(e){let t="";for(let r=0;r<e.length;r+=4)t+=`${e.substring(r,r+4)} `;return t.trimEnd()}var LRe=class extends In{accountNumber(e={}){typeof e=="number"&&(e={length:e});let{length:t=8}=e;return this.faker.string.numeric({length:t,allowLeadingZeros:!0})}accountName(){return[this.faker.helpers.arrayElement(this.faker.definitions.finance.account_type),"Account"].join(" ")}routingNumber(){let e=this.faker.string.numeric({length:8,allowLeadingZeros:!0}),t=0;for(let r=0;r<e.length;r+=3)t+=Number(e[r])*3,t+=Number(e[r+1])*7,t+=Number(e[r+2])||0;return`${e}${Math.ceil(t/10)*10-t}`}maskedNumber(e={}){jc({deprecated:"faker.finance.maskedNumber()",proposed:"faker.finance.iban().replace(/(?<=.{4})\\w(?=.{2})/g, '*') or a similar approach",since:"9.3.0",until:"10.0.0"}),typeof e=="number"&&(e={length:e});let{ellipsis:t=!0,length:r=4,parens:n=!0}=e,a=this.faker.string.numeric({length:r});return t&&(a=`...${a}`),n&&(a=`(${a})`),a}amount(e={}){let{autoFormat:t=!1,dec:r=2,max:n=1e3,min:a=0,symbol:i=""}=e,o=this.faker.number.float({max:n,min:a,fractionDigits:r}),s=t?o.toLocaleString(void 0,{minimumFractionDigits:r}):o.toFixed(r);return i+s}transactionType(){return this.faker.helpers.arrayElement(this.faker.definitions.finance.transaction_type)}currency(){return this.faker.helpers.arrayElement(this.faker.definitions.finance.currency)}currencyCode(){return this.currency().code}currencyName(){return this.currency().name}currencySymbol(){let e;do e=this.currency().symbol;while(e.length===0);return e}currencyNumericCode(){return this.currency().numericCode}bitcoinAddress(e={}){let{type:t=this.faker.helpers.enumValue(ste),network:r="mainnet"}=e,n=j_e[t],a=n.prefix[r],i=this.faker.number.int(n.length),o=this.faker.string.alphanumeric({length:i-a.length,casing:n.casing,exclude:n.exclude});return a+o}litecoinAddress(){let e=this.faker.number.int({min:26,max:33});return this.faker.string.fromCharacters("LM3")+this.faker.string.fromCharacters("123456789abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ",e-1)}creditCardNumber(e={}){typeof e=="string"&&(e={issuer:e});let{issuer:t=""}=e,r,n=this.faker.definitions.finance.credit_card,a=t.toLowerCase();if(a in n)r=this.faker.helpers.arrayElement(n[a]);else if(t.includes("#"))r=t;else{let i=this.faker.helpers.objectValue(n);r=this.faker.helpers.arrayElement(i)}return r=r.replaceAll("/",""),this.faker.helpers.replaceCreditCardSymbols(r)}creditCardCVV(){return this.faker.string.numeric({length:3,allowLeadingZeros:!0})}creditCardIssuer(){return this.faker.helpers.objectKey(this.faker.definitions.finance.credit_card)}pin(e={}){typeof e=="number"&&(e={length:e});let{length:t=4}=e;if(t<1)throw new Jt("minimum length is 1");return this.faker.string.numeric({length:t,allowLeadingZeros:!0})}ethereumAddress(){return this.faker.string.hexadecimal({length:40,casing:"lower"})}iban(e={}){let{countryCode:t,formatted:r=!1}=e,n=t?Vs.formats.find(l=>l.country===t):this.faker.helpers.arrayElement(Vs.formats);if(!n)throw new Jt(`Country code ${t} not supported.`);let a="",i=0;for(let l of n.bban){let u=l.count;for(i+=l.count;u>0;)l.type==="a"?a+=this.faker.helpers.arrayElement(Vs.alpha):l.type==="c"?this.faker.datatype.boolean(.8)?a+=this.faker.number.int(9):a+=this.faker.helpers.arrayElement(Vs.alpha):u>=3&&this.faker.datatype.boolean(.3)?this.faker.datatype.boolean()?(a+=this.faker.helpers.arrayElement(Vs.pattern100),u-=2):(a+=this.faker.helpers.arrayElement(Vs.pattern10),u--):a+=this.faker.number.int(9),u--;a=a.substring(0,i)}let o=98-Vs.mod97(Vs.toDigitString(`${a}${n.country}00`));o<10&&(o=`0${o}`);let s=`${n.country}${o}${a}`;return r?NRe(s):s}bic(e={}){let{includeBranchCode:t=this.faker.datatype.boolean()}=e,r=this.faker.string.alpha({length:4,casing:"upper"}),n=this.faker.helpers.arrayElement(Vs.iso3166),a=this.faker.string.alphanumeric({length:2,casing:"upper"}),i=t?this.faker.datatype.boolean()?this.faker.string.alphanumeric({length:3,casing:"upper"}):"XXX":"";return`${r}${n}${a}${i}`}transactionDescription(){return this.faker.helpers.fake(this.faker.definitions.finance.transaction_description_pattern)}};function FB(e){return e.split(" ").map(t=>t.charAt(0).toUpperCase()+t.slice(1)).join(" ")}var MRe=class extends In{adjective(){return this.faker.helpers.arrayElement(this.faker.definitions.food.adjective)}description(){return this.faker.helpers.fake(this.faker.definitions.food.description_pattern)}dish(){return this.faker.datatype.boolean()?FB(this.faker.helpers.fake(this.faker.definitions.food.dish_pattern)):FB(this.faker.helpers.arrayElement(this.faker.definitions.food.dish))}ethnicCategory(){return this.faker.helpers.arrayElement(this.faker.definitions.food.ethnic_category)}fruit(){return this.faker.helpers.arrayElement(this.faker.definitions.food.fruit)}ingredient(){return this.faker.helpers.arrayElement(this.faker.definitions.food.ingredient)}meat(){return this.faker.helpers.arrayElement(this.faker.definitions.food.meat)}spice(){return this.faker.helpers.arrayElement(this.faker.definitions.food.spice)}vegetable(){return this.faker.helpers.arrayElement(this.faker.definitions.food.vegetable)}},IRe=" ",ORe=class extends In{branch(){let t=this.faker.hacker.noun().replace(" ","-"),r=this.faker.hacker.verb().replace(" ","-");return`${t}-${r}`}commitEntry(t={}){let{merge:r=this.faker.datatype.boolean({probability:.2}),eol:n="CRLF",refDate:a}=t,i=[`commit ${this.faker.git.commitSha()}`];r&&i.push(`Merge: ${this.commitSha({length:7})} ${this.commitSha({length:7})}`);let o=this.faker.person.firstName(),s=this.faker.person.lastName(),l=this.faker.person.fullName({firstName:o,lastName:s}),u=this.faker.internet.username({firstName:o,lastName:s}),d=this.faker.helpers.arrayElement([l,u]),p=this.faker.internet.email({firstName:o,lastName:s});d=d.replaceAll(/^[.,:;"\\']|[<>\n]|[.,:;"\\']$/g,""),i.push(`Author: ${d} <${p}>`,`Date: ${this.commitDate({refDate:a})}`,"",`${IRe.repeat(4)}${this.commitMessage()}`,"");let f=n==="CRLF"?`\r
`:`
`;return i.join(f)}commitMessage(){return`${this.faker.hacker.verb()} ${this.faker.hacker.adjective()} ${this.faker.hacker.noun()}`}commitDate(t={}){let{refDate:r=this.faker.defaultRefDate()}=t,n=["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],a=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],i=this.faker.date.recent({days:1,refDate:r}),o=n[i.getUTCDay()],s=a[i.getUTCMonth()],l=i.getUTCDate(),u=i.getUTCHours().toString().padStart(2,"0"),d=i.getUTCMinutes().toString().padStart(2,"0"),p=i.getUTCSeconds().toString().padStart(2,"0"),f=i.getUTCFullYear(),m=this.faker.number.int({min:-11,max:12}),b=Math.abs(m).toString().padStart(2,"0"),y="00",S=m>=0?"+":"-";return`${o} ${s} ${l} ${u}:${d}:${p} ${f} ${S}${b}${y}`}commitSha(t={}){let{length:r=40}=t;return this.faker.string.hexadecimal({length:r,casing:"lower",prefix:""})}},BRe=class extends In{abbreviation(){return this.faker.helpers.arrayElement(this.faker.definitions.hacker.abbreviation)}adjective(){return this.faker.helpers.arrayElement(this.faker.definitions.hacker.adjective)}noun(){return this.faker.helpers.arrayElement(this.faker.definitions.hacker.noun)}verb(){return this.faker.helpers.arrayElement(this.faker.definitions.hacker.verb)}ingverb(){return this.faker.helpers.arrayElement(this.faker.definitions.hacker.ingverb)}phrase(){let t={abbreviation:this.abbreviation,adjective:this.adjective,ingverb:this.ingverb,noun:this.noun,verb:this.verb},r=this.faker.helpers.arrayElement(this.faker.definitions.hacker.phrase);return this.faker.helpers.mustache(r,t)}},PRe=class extends In{avatar(){return this.faker.helpers.arrayElement([this.personPortrait,this.avatarGitHub])()}avatarGitHub(){return`https://avatars.githubusercontent.com/u/${this.faker.number.int(1e8)}`}personPortrait(t={}){let{sex:r=this.faker.person.sexType(),size:n=512}=t;return`https://cdn.jsdelivr.net/gh/faker-js/assets-person-portrait/${r}/${n}/${this.faker.number.int({min:0,max:99})}.jpg`}avatarLegacy(){return jc({deprecated:"faker.image.avatarLegacy()",proposed:"faker.image.avatar() or faker.image.personPortrait()",since:"9.0.2",until:"10.0.0"}),`https://cloudflare-ipfs.com/ipfs/Qmd3W5DuhgHirLHGVixi6V76LhCkZUz6pnFt5AJBiyvHye/avatar/${this.faker.number.int(1249)}.jpg`}url(t={}){let{width:r=this.faker.number.int({min:1,max:3999}),height:n=this.faker.number.int({min:1,max:3999})}=t;return this.faker.helpers.arrayElement([this.urlLoremFlickr,({width:a,height:i})=>this.urlPicsumPhotos({width:a,height:i,grayscale:!1,blur:0})])({width:r,height:n})}urlLoremFlickr(t={}){let{width:r=this.faker.number.int({min:1,max:3999}),height:n=this.faker.number.int({min:1,max:3999}),category:a}=t;return`https://loremflickr.com/${r}/${n}${a==null?"":`/${a}`}?lock=${this.faker.number.int()}`}urlPicsumPhotos(t={}){let{width:r=this.faker.number.int({min:1,max:3999}),height:n=this.faker.number.int({min:1,max:3999}),grayscale:a=this.faker.datatype.boolean(),blur:i=this.faker.number.int({max:10})}=t,o=`https://picsum.photos/seed/${this.faker.string.alphanumeric({length:{min:5,max:10}})}/${r}/${n}`,s=typeof i=="number"&&i>=1&&i<=10;return(a||s)&&(o+="?",a&&(o+="grayscale"),a&&s&&(o+="&"),s&&(o+=`blur=${i}`)),o}urlPlaceholder(t={}){jc({deprecated:"faker.image.urlPlaceholder()",proposed:"faker.image.url() or faker.image.dataUri()",since:"9.4.0",until:"10.0.0"});let{width:r=this.faker.number.int({min:1,max:3500}),height:n=this.faker.number.int({min:1,max:3500}),backgroundColor:a=this.faker.color.rgb({format:"hex",prefix:""}),textColor:i=this.faker.color.rgb({format:"hex",prefix:""}),format:o=this.faker.helpers.arrayElement(["gif","jpeg","jpg","png","webp"]),text:s=this.faker.lorem.words()}=t,l="https://via.placeholder.com";return l+=`/${r}`,l+=`x${n}`,l+=`/${a}`,l+=`/${i}`,l+=`.${o}`,l+=`?text=${encodeURIComponent(s)}`,l}dataUri(t={}){let{width:r=this.faker.number.int({min:1,max:3999}),height:n=this.faker.number.int({min:1,max:3999}),color:a=this.faker.color.rgb(),type:i=this.faker.helpers.arrayElement(["svg-uri","svg-base64"])}=t,o=`<svg xmlns="http://www.w3.org/2000/svg" version="1.1" baseProfile="full" width="${r}" height="${n}"><rect width="100%" height="100%" fill="${a}"/><text x="${r/2}" y="${n/2}" font-size="20" alignment-baseline="middle" text-anchor="middle" fill="white">${r}x${n}</text></svg>`;return i==="svg-uri"?`data:image/svg+xml;charset=UTF-8,${encodeURIComponent(o)}`:`data:image/svg+xml;base64,${lte(o)}`}};function zRe(e,t,r=n=>n){let n={};for(let a of e){let i=t(a);n[i]===void 0&&(n[i]=[]),n[i].push(r(a))}return n}var zw={fail:()=>{throw new Jt("No words found that match the given length.")},closest:(e,t)=>{let r=zRe(e,s=>s.length),n=Object.keys(r).map(Number),a=Math.min(...n),i=Math.max(...n),o=Math.min(t.min-a,i-t.max);return e.filter(s=>s.length===t.min-o||s.length===t.max+o)},shortest:e=>{let t=Math.min(...e.map(r=>r.length));return e.filter(r=>r.length===t)},longest:e=>{let t=Math.max(...e.map(r=>r.length));return e.filter(r=>r.length===t)},"any-length":e=>[...e]};function du(e){let{wordList:t,length:r,strategy:n="any-length"}=e;if(r!=null){let a=typeof r=="number"?o=>o.length===r:o=>o.length>=r.min&&o.length<=r.max,i=t.filter(a);return i.length>0?i:typeof r=="number"?zw[n](t,{min:r,max:r}):zw[n](t,r)}else if(n==="shortest"||n==="longest")return zw[n](t);return[...t]}var HRe=class extends In{word(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.lorem.word}))}words(t=3){return this.faker.helpers.multiple(()=>this.word(),{count:t}).join(" ")}sentence(t={min:3,max:10}){let r=this.words(t);return`${r.charAt(0).toUpperCase()+r.substring(1)}.`}slug(t=3){let r=this.words(t);return this.faker.helpers.slugify(r)}sentences(t={min:2,max:6},r=" "){return this.faker.helpers.multiple(()=>this.sentence(),{count:t}).join(r)}paragraph(t=3){return this.sentences(t)}paragraphs(t=3,r=`
`){return this.faker.helpers.multiple(()=>this.paragraph(),{count:t}).join(r)}text(){let t=["sentence","sentences","paragraph","paragraphs","lines"],r=this.faker.helpers.arrayElement(t);return this[r]()}lines(t={min:1,max:5}){return this.sentences(t,`
`)}},URe=class extends In{album(){return this.faker.helpers.arrayElement(this.faker.definitions.music.album)}artist(){return this.faker.helpers.arrayElement(this.faker.definitions.music.artist)}genre(){return this.faker.helpers.arrayElement(this.faker.definitions.music.genre)}songName(){return this.faker.helpers.arrayElement(this.faker.definitions.music.song_name)}},$Re=class extends In{number(t={}){let{style:r="human"}=t,n=this.faker.definitions.phone_number.format[r];if(!n)throw new Error(`No definitions for ${r} in this locale`);let a=this.faker.helpers.arrayElement(n);return fte(this.faker,a)}imei(){return this.faker.helpers.replaceCreditCardSymbols("##-######-######-L","#")}},GRe=class extends In{chemicalElement(){return this.faker.helpers.arrayElement(this.faker.definitions.science.chemical_element)}unit(){return this.faker.helpers.arrayElement(this.faker.definitions.science.unit)}},jRe=["video","audio","image","text","application"],qRe=["application/pdf","audio/mpeg","audio/wav","image/png","image/jpeg","image/gif","video/mp4","video/mpeg","text/html"],WRe=["en","wl","ww"],_B={index:"o",slot:"s",mac:"x",pci:"p"},VRe=["SUN","MON","TUE","WED","THU","FRI","SAT"],KRe=class extends In{fileName(t={}){let{extensionCount:r=1}=t,n=this.faker.word.words().toLowerCase().replaceAll(/\W/g,"_"),a=this.faker.helpers.multiple(()=>this.fileExt(),{count:r}).join(".");return a.length===0?n:`${n}.${a}`}commonFileName(t){return`${this.fileName({extensionCount:0})}.${t||this.commonFileExt()}`}mimeType(){let t=Object.keys(this.faker.definitions.system.mime_type);return this.faker.helpers.arrayElement(t)}commonFileType(){return this.faker.helpers.arrayElement(jRe)}commonFileExt(){return this.fileExt(this.faker.helpers.arrayElement(qRe))}fileType(){let t=this.faker.definitions.system.mime_type,r=new Set(Object.keys(t).map(n=>n.split("/")[0]));return this.faker.helpers.arrayElement([...r])}fileExt(t){let r=this.faker.definitions.system.mime_type;if(typeof t=="string")return this.faker.helpers.arrayElement(r[t].extensions);let n=new Set(Object.values(r).flatMap(({extensions:a})=>a));return this.faker.helpers.arrayElement([...n])}directoryPath(){let t=this.faker.definitions.system.directory_path;return this.faker.helpers.arrayElement(t)}filePath(){return`${this.directoryPath()}/${this.fileName()}`}semver(){return[this.faker.number.int(9),this.faker.number.int(20),this.faker.number.int(20)].join(".")}networkInterface(t={}){let{interfaceType:r=this.faker.helpers.arrayElement(WRe),interfaceSchema:n=this.faker.helpers.objectKey(_B)}=t,a,i="";switch(n){case"index":{a=this.faker.string.numeric();break}case"slot":{a=`${this.faker.string.numeric()}${this.faker.helpers.maybe(()=>`f${this.faker.string.numeric()}`)??""}${this.faker.helpers.maybe(()=>`d${this.faker.string.numeric()}`)??""}`;break}case"mac":{a=this.faker.internet.mac("");break}case"pci":{i=this.faker.helpers.maybe(()=>`P${this.faker.string.numeric()}`)??"",a=`${this.faker.string.numeric()}s${this.faker.string.numeric()}${this.faker.helpers.maybe(()=>`f${this.faker.string.numeric()}`)??""}${this.faker.helpers.maybe(()=>`d${this.faker.string.numeric()}`)??""}`;break}}return`${i}${r}${_B[n]}${a}`}cron(t={}){let{includeYear:r=!1,includeNonStandard:n=!1}=t,a=[this.faker.number.int(59),"*"],i=[this.faker.number.int(23),"*"],o=[this.faker.number.int({min:1,max:31}),"*","?"],s=[this.faker.number.int({min:1,max:12}),"*"],l=[this.faker.number.int(6),this.faker.helpers.arrayElement(VRe),"*","?"],u=[this.faker.number.int({min:1970,max:2099}),"*"],d=this.faker.helpers.arrayElement(a),p=this.faker.helpers.arrayElement(i),f=this.faker.helpers.arrayElement(o),m=this.faker.helpers.arrayElement(s),b=this.faker.helpers.arrayElement(l),y=this.faker.helpers.arrayElement(u),S=`${d} ${p} ${f} ${m} ${b}`;r&&(S+=` ${y}`);let w=["@annually","@daily","@hourly","@monthly","@reboot","@weekly","@yearly"];return!n||this.faker.datatype.boolean()?S:this.faker.helpers.arrayElement(w)}},YRe=class extends In{vehicle(){return`${this.manufacturer()} ${this.model()}`}manufacturer(){return this.faker.helpers.arrayElement(this.faker.definitions.vehicle.manufacturer)}model(){return this.faker.helpers.arrayElement(this.faker.definitions.vehicle.model)}type(){return this.faker.helpers.arrayElement(this.faker.definitions.vehicle.type)}fuel(){return this.faker.helpers.arrayElement(this.faker.definitions.vehicle.fuel)}vin(){let t=["o","i","q","O","I","Q"];return`${this.faker.string.alphanumeric({length:10,casing:"upper",exclude:t})}${this.faker.string.alpha({length:1,casing:"upper",exclude:t})}${this.faker.string.alphanumeric({length:1,casing:"upper",exclude:t})}${this.faker.string.numeric({length:5,allowLeadingZeros:!0})}`}color(){return this.faker.color.human()}vrm(){return`${this.faker.string.alpha({length:2,casing:"upper"})}${this.faker.string.numeric({length:2,allowLeadingZeros:!0})}${this.faker.string.alpha({length:3,casing:"upper"})}`}bicycle(){return this.faker.helpers.arrayElement(this.faker.definitions.vehicle.bicycle_type)}},JRe=class extends In{adjective(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.word.adjective}))}adverb(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.word.adverb}))}conjunction(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.word.conjunction}))}interjection(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.word.interjection}))}noun(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.word.noun}))}preposition(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.word.preposition}))}verb(t={}){return typeof t=="number"&&(t={length:t}),this.faker.helpers.arrayElement(du({...t,wordList:this.faker.definitions.word.verb}))}sample(t={}){let r=this.faker.helpers.shuffle([this.adjective,this.adverb,this.conjunction,this.interjection,this.noun,this.preposition,this.verb]);for(let n of r)try{return n(t)}catch{continue}throw new Jt("No matching word data available for the current locale")}words(t={}){typeof t=="number"&&(t={count:t});let{count:r={min:1,max:3}}=t;return this.faker.helpers.multiple(()=>this.sample(),{count:r}).join(" ")}},vte=class extends yte{constructor(r){super({randomizer:r.randomizer,seed:r.seed});mt(this,"rawDefinitions");mt(this,"definitions");mt(this,"airline",new z_e(this));mt(this,"animal",new CRe(this));mt(this,"book",new TRe(this));mt(this,"color",new $_e(this));mt(this,"commerce",new FRe(this));mt(this,"company",new _Re(this));mt(this,"database",new RRe(this));mt(this,"date",new uRe(this));mt(this,"finance",new LRe(this));mt(this,"food",new MRe(this));mt(this,"git",new ORe(this));mt(this,"hacker",new BRe(this));mt(this,"helpers",new vRe(this));mt(this,"image",new PRe(this));mt(this,"internet",new Q_e(this));mt(this,"location",new SRe(this));mt(this,"lorem",new HRe(this));mt(this,"music",new URe(this));mt(this,"person",new eRe(this));mt(this,"phone",new $Re(this));mt(this,"science",new GRe(this));mt(this,"system",new KRe(this));mt(this,"vehicle",new YRe(this));mt(this,"word",new JRe(this));let{locale:n}=r;if(Array.isArray(n)){if(n.length===0)throw new Jt("The locale option must contain at least one locale definition.");n=kRe(n)}this.rawDefinitions=n,this.definitions=sRe(this.rawDefinitions)}get address(){return jc({deprecated:"faker.address",proposed:"faker.location",since:"8.0",until:"10.0"}),this.location}get name(){return jc({deprecated:"faker.name",proposed:"faker.person",since:"8.0",until:"10.0"}),this.person}getMetadata(){return this.rawDefinitions.metadata??{}}},XRe=["Academy Color Encoding System (ACES)","Adobe RGB","Adobe Wide Gamut RGB","British Standard Colour (BS)","CIE 1931 XYZ","CIELAB","CIELUV","CIEUVW","CMY","CMYK","DCI-P3","Display-P3","Federal Standard 595C","HKS","HSL","HSLA","HSLuv","HSV","HWB","LCh","LMS","Munsell Color System","Natural Color System (NSC)","Pantone Matching System (PMS)","ProPhoto RGB Color Space","RAL","RG","RGBA","RGK","Rec. 2020","Rec. 2100","Rec. 601","Rec. 709","Uniform Color Spaces (UCSs)","YDbDr","YIQ","YPbPr","sRGB","sYCC","scRGB","xvYCC"],ZRe={space:XRe},QRe=ZRe,e6e=["ascii_bin","ascii_general_ci","cp1250_bin","cp1250_general_ci","utf8_bin","utf8_general_ci","utf8_unicode_ci"],t6e=["ARCHIVE","BLACKHOLE","CSV","InnoDB","MEMORY","MyISAM"],r6e=["bigint","binary","bit","blob","boolean","date","datetime","decimal","double","enum","float","geometry","int","mediumint","point","real","serial","set","smallint","text","time","timestamp","tinyint","varchar"],n6e={collation:e6e,engine:t6e,type:r6e},a6e=n6e,Ste=["Africa/Abidjan","Africa/Accra","Africa/Addis_Ababa","Africa/Algiers","Africa/Asmara","Africa/Bamako","Africa/Bangui","Africa/Banjul","Africa/Bissau","Africa/Blantyre","Africa/Brazzaville","Africa/Bujumbura","Africa/Cairo","Africa/Casablanca","Africa/Ceuta","Africa/Conakry","Africa/Dakar","Africa/Dar_es_Salaam","Africa/Djibouti","Africa/Douala","Africa/El_Aaiun","Africa/Freetown","Africa/Gaborone","Africa/Harare","Africa/Johannesburg","Africa/Juba","Africa/Kampala","Africa/Khartoum","Africa/Kigali","Africa/Kinshasa","Africa/Lagos","Africa/Libreville","Africa/Lome","Africa/Luanda","Africa/Lubumbashi","Africa/Lusaka","Africa/Malabo","Africa/Maputo","Africa/Maseru","Africa/Mbabane","Africa/Mogadishu","Africa/Monrovia","Africa/Nairobi","Africa/Ndjamena","Africa/Niamey","Africa/Nouakchott","Africa/Ouagadougou","Africa/Porto-Novo","Africa/Sao_Tome","Africa/Tripoli","Africa/Tunis","Africa/Windhoek","America/Adak","America/Anchorage","America/Anguilla","America/Antigua","America/Araguaina","America/Argentina/Buenos_Aires","America/Argentina/Catamarca","America/Argentina/Cordoba","America/Argentina/Jujuy","America/Argentina/La_Rioja","America/Argentina/Mendoza","America/Argentina/Rio_Gallegos","America/Argentina/Salta","America/Argentina/San_Juan","America/Argentina/San_Luis","America/Argentina/Tucuman","America/Argentina/Ushuaia","America/Aruba","America/Asuncion","America/Atikokan","America/Bahia","America/Bahia_Banderas","America/Barbados","America/Belem","America/Belize","America/Blanc-Sablon","America/Boa_Vista","America/Bogota","America/Boise","America/Cambridge_Bay","America/Campo_Grande","America/Cancun","America/Caracas","America/Cayenne","America/Cayman","America/Chicago","America/Chihuahua","America/Ciudad_Juarez","America/Costa_Rica","America/Creston","America/Cuiaba","America/Curacao","America/Danmarkshavn","America/Dawson","America/Dawson_Creek","America/Denver","America/Detroit","America/Dominica","America/Edmonton","America/Eirunepe","America/El_Salvador","America/Fort_Nelson","America/Fortaleza","America/Glace_Bay","America/Goose_Bay","America/Grand_Turk","America/Grenada","America/Guadeloupe","America/Guatemala","America/Guayaquil","America/Guyana","America/Halifax","America/Havana","America/Hermosillo","America/Indiana/Indianapolis","America/Indiana/Knox","America/Indiana/Marengo","America/Indiana/Petersburg","America/Indiana/Tell_City","America/Indiana/Vevay","America/Indiana/Vincennes","America/Indiana/Winamac","America/Inuvik","America/Iqaluit","America/Jamaica","America/Juneau","America/Kentucky/Louisville","America/Kentucky/Monticello","America/Kralendijk","America/La_Paz","America/Lima","America/Los_Angeles","America/Lower_Princes","America/Maceio","America/Managua","America/Manaus","America/Marigot","America/Martinique","America/Matamoros","America/Mazatlan","America/Menominee","America/Merida","America/Metlakatla","America/Mexico_City","America/Miquelon","America/Moncton","America/Monterrey","America/Montevideo","America/Montserrat","America/Nassau","America/New_York","America/Nome","America/Noronha","America/North_Dakota/Beulah","America/North_Dakota/Center","America/North_Dakota/New_Salem","America/Nuuk","America/Ojinaga","America/Panama","America/Paramaribo","America/Phoenix","America/Port-au-Prince","America/Port_of_Spain","America/Porto_Velho","America/Puerto_Rico","America/Punta_Arenas","America/Rankin_Inlet","America/Recife","America/Regina","America/Resolute","America/Rio_Branco","America/Santarem","America/Santiago","America/Santo_Domingo","America/Sao_Paulo","America/Scoresbysund","America/Sitka","America/St_Barthelemy","America/St_Johns","America/St_Kitts","America/St_Lucia","America/St_Thomas","America/St_Vincent","America/Swift_Current","America/Tegucigalpa","America/Thule","America/Tijuana","America/Toronto","America/Tortola","America/Vancouver","America/Whitehorse","America/Winnipeg","America/Yakutat","America/Yellowknife","Antarctica/Casey","Antarctica/Davis","Antarctica/DumontDUrville","Antarctica/Macquarie","Antarctica/Mawson","Antarctica/McMurdo","Antarctica/Palmer","Antarctica/Rothera","Antarctica/Syowa","Antarctica/Troll","Antarctica/Vostok","Arctic/Longyearbyen","Asia/Aden","Asia/Almaty","Asia/Amman","Asia/Anadyr","Asia/Aqtau","Asia/Aqtobe","Asia/Ashgabat","Asia/Atyrau","Asia/Baghdad","Asia/Bahrain","Asia/Baku","Asia/Bangkok","Asia/Barnaul","Asia/Beirut","Asia/Bishkek","Asia/Brunei","Asia/Chita","Asia/Choibalsan","Asia/Colombo","Asia/Damascus","Asia/Dhaka","Asia/Dili","Asia/Dubai","Asia/Dushanbe","Asia/Famagusta","Asia/Gaza","Asia/Hebron","Asia/Ho_Chi_Minh","Asia/Hong_Kong","Asia/Hovd","Asia/Irkutsk","Asia/Jakarta","Asia/Jayapura","Asia/Jerusalem","Asia/Kabul","Asia/Kamchatka","Asia/Karachi","Asia/Kathmandu","Asia/Khandyga","Asia/Kolkata","Asia/Krasnoyarsk","Asia/Kuala_Lumpur","Asia/Kuching","Asia/Kuwait","Asia/Macau","Asia/Magadan","Asia/Makassar","Asia/Manila","Asia/Muscat","Asia/Nicosia","Asia/Novokuznetsk","Asia/Novosibirsk","Asia/Omsk","Asia/Oral","Asia/Phnom_Penh","Asia/Pontianak","Asia/Pyongyang","Asia/Qatar","Asia/Qostanay","Asia/Qyzylorda","Asia/Riyadh","Asia/Sakhalin","Asia/Samarkand","Asia/Seoul","Asia/Shanghai","Asia/Singapore","Asia/Srednekolymsk","Asia/Taipei","Asia/Tashkent","Asia/Tbilisi","Asia/Tehran","Asia/Thimphu","Asia/Tokyo","Asia/Tomsk","Asia/Ulaanbaatar","Asia/Urumqi","Asia/Ust-Nera","Asia/Vientiane","Asia/Vladivostok","Asia/Yakutsk","Asia/Yangon","Asia/Yekaterinburg","Asia/Yerevan","Atlantic/Azores","Atlantic/Bermuda","Atlantic/Canary","Atlantic/Cape_Verde","Atlantic/Faroe","Atlantic/Madeira","Atlantic/Reykjavik","Atlantic/South_Georgia","Atlantic/St_Helena","Atlantic/Stanley","Australia/Adelaide","Australia/Brisbane","Australia/Broken_Hill","Australia/Darwin","Australia/Eucla","Australia/Hobart","Australia/Lindeman","Australia/Lord_Howe","Australia/Melbourne","Australia/Perth","Australia/Sydney","Europe/Amsterdam","Europe/Andorra","Europe/Astrakhan","Europe/Athens","Europe/Belgrade","Europe/Berlin","Europe/Bratislava","Europe/Brussels","Europe/Bucharest","Europe/Budapest","Europe/Busingen","Europe/Chisinau","Europe/Copenhagen","Europe/Dublin","Europe/Gibraltar","Europe/Guernsey","Europe/Helsinki","Europe/Isle_of_Man","Europe/Istanbul","Europe/Jersey","Europe/Kaliningrad","Europe/Kirov","Europe/Kyiv","Europe/Lisbon","Europe/Ljubljana","Europe/London","Europe/Luxembourg","Europe/Madrid","Europe/Malta","Europe/Mariehamn","Europe/Minsk","Europe/Monaco","Europe/Moscow","Europe/Oslo","Europe/Paris","Europe/Podgorica","Europe/Prague","Europe/Riga","Europe/Rome","Europe/Samara","Europe/San_Marino","Europe/Sarajevo","Europe/Saratov","Europe/Simferopol","Europe/Skopje","Europe/Sofia","Europe/Stockholm","Europe/Tallinn","Europe/Tirane","Europe/Ulyanovsk","Europe/Vaduz","Europe/Vatican","Europe/Vienna","Europe/Vilnius","Europe/Volgograd","Europe/Warsaw","Europe/Zagreb","Europe/Zurich","Indian/Antananarivo","Indian/Chagos","Indian/Christmas","Indian/Cocos","Indian/Comoro","Indian/Kerguelen","Indian/Mahe","Indian/Maldives","Indian/Mauritius","Indian/Mayotte","Indian/Reunion","Pacific/Apia","Pacific/Auckland","Pacific/Bougainville","Pacific/Chatham","Pacific/Chuuk","Pacific/Easter","Pacific/Efate","Pacific/Fakaofo","Pacific/Fiji","Pacific/Funafuti","Pacific/Galapagos","Pacific/Gambier","Pacific/Guadalcanal","Pacific/Guam","Pacific/Honolulu","Pacific/Kanton","Pacific/Kiritimati","Pacific/Kosrae","Pacific/Kwajalein","Pacific/Majuro","Pacific/Marquesas","Pacific/Midway","Pacific/Nauru","Pacific/Niue","Pacific/Norfolk","Pacific/Noumea","Pacific/Pago_Pago","Pacific/Palau","Pacific/Pitcairn","Pacific/Pohnpei","Pacific/Port_Moresby","Pacific/Rarotonga","Pacific/Saipan","Pacific/Tahiti","Pacific/Tarawa","Pacific/Tongatapu","Pacific/Wake","Pacific/Wallis"],i6e={time_zone:Ste},o6e=i6e,s6e=["ADP","AGP","AI","API","ASCII","CLI","COM","CSS","DNS","DRAM","EXE","FTP","GB","HDD","HEX","HTTP","IB","IP","JBOD","JSON","OCR","PCI","PNG","RAM","RSS","SAS","SCSI","SDD","SMS","SMTP","SQL","SSD","SSL","TCP","THX","TLS","UDP","USB","UTF8","VGA","XML","XSS"],l6e={abbreviation:s6e},u6e=l6e,c6e={smiley:["☠️","☹️","☺️","❣️","❤️","❤️‍🔥","❤️‍🩹","👁️‍🗨️","👹","👺","👻","👽","👾","👿","💀","💋","💌","💓","💔","💕","💖","💗","💘","💙","💚","💛","💜","💝","💞","💟","💢","💣","💤","💥","💦","💨","💩","💫","💬","💭","💯","🕳️","🖤","🗨️","🗯️","😀","😁","😂","😃","😄","😅","😆","😇","😈","😉","😊","😋","😌","😍","😎","😏","😐","😑","😒","😓","😔","😕","😖","😗","😘","😙","😚","😛","😜","😝","😞","😟","😠","😡","😢","😣","😤","😥","😦","😧","😨","😩","😪","😫","😬","😭","😮","😮‍💨","😯","😰","😱","😲","😳","😴","😵","😵‍💫","😶","😶‍🌫️","😷","😸","😹","😺","😻","😼","😽","😾","😿","🙀","🙁","🙂","🙃","🙄","🙈","🙉","🙊","🤍","🤎","🤐","🤑","🤒","🤓","🤔","🤕","🤖","🤗","🤠","🤡","🤢","🤣","🤤","🤥","🤧","🤨","🤩","🤪","🤫","🤬","🤭","🤮","🤯","🥰","🥱","🥲","🥳","🥴","🥵","🥶","🥸","🥺","🧐","🧡"],body:["☝🏻","☝🏼","☝🏽","☝🏾","☝🏿","☝️","✊","✊🏻","✊🏼","✊🏽","✊🏾","✊🏿","✋","✋🏻","✋🏼","✋🏽","✋🏾","✋🏿","✌🏻","✌🏼","✌🏽","✌🏾","✌🏿","✌️","✍🏻","✍🏼","✍🏽","✍🏾","✍🏿","✍️","👀","👁️","👂","👂🏻","👂🏼","👂🏽","👂🏾","👂🏿","👃","👃🏻","👃🏼","👃🏽","👃🏾","👃🏿","👄","👅","👆","👆🏻","👆🏼","👆🏽","👆🏾","👆🏿","👇","👇🏻","👇🏼","👇🏽","👇🏾","👇🏿","👈","👈🏻","👈🏼","👈🏽","👈🏾","👈🏿","👉","👉🏻","👉🏼","👉🏽","👉🏾","👉🏿","👊","👊🏻","👊🏼","👊🏽","👊🏾","👊🏿","👋","👋🏻","👋🏼","👋🏽","👋🏾","👋🏿","👌","👌🏻","👌🏼","👌🏽","👌🏾","👌🏿","👍","👍🏻","👍🏼","👍🏽","👍🏾","👍🏿","👎","👎🏻","👎🏼","👎🏽","👎🏾","👎🏿","👏","👏🏻","👏🏼","👏🏽","👏🏾","👏🏿","👐","👐🏻","👐🏼","👐🏽","👐🏾","👐🏿","💅","💅🏻","💅🏼","💅🏽","💅🏾","💅🏿","💪","💪🏻","💪🏼","💪🏽","💪🏾","💪🏿","🖐🏻","🖐🏼","🖐🏽","🖐🏾","🖐🏿","🖐️","🖕","🖕🏻","🖕🏼","🖕🏽","🖕🏾","🖕🏿","🖖","🖖🏻","🖖🏼","🖖🏽","🖖🏾","🖖🏿","🙌","🙌🏻","🙌🏼","🙌🏽","🙌🏾","🙌🏿","🙏","🙏🏻","🙏🏼","🙏🏽","🙏🏾","🙏🏿","🤌","🤌🏻","🤌🏼","🤌🏽","🤌🏾","🤌🏿","🤏","🤏🏻","🤏🏼","🤏🏽","🤏🏾","🤏🏿","🤘","🤘🏻","🤘🏼","🤘🏽","🤘🏾","🤘🏿","🤙","🤙🏻","🤙🏼","🤙🏽","🤙🏾","🤙🏿","🤚","🤚🏻","🤚🏼","🤚🏽","🤚🏾","🤚🏿","🤛","🤛🏻","🤛🏼","🤛🏽","🤛🏾","🤛🏿","🤜","🤜🏻","🤜🏼","🤜🏽","🤜🏾","🤜🏿","🤝","🤞","🤞🏻","🤞🏼","🤞🏽","🤞🏾","🤞🏿","🤟","🤟🏻","🤟🏼","🤟🏽","🤟🏾","🤟🏿","🤲","🤲🏻","🤲🏼","🤲🏽","🤲🏾","🤲🏿","🤳","🤳🏻","🤳🏼","🤳🏽","🤳🏾","🤳🏿","🦴","🦵","🦵🏻","🦵🏼","🦵🏽","🦵🏾","🦵🏿","🦶","🦶🏻","🦶🏼","🦶🏽","🦶🏾","🦶🏿","🦷","🦻","🦻🏻","🦻🏼","🦻🏽","🦻🏾","🦻🏿","🦾","🦿","🧠","🫀","🫁"],person:["🎅","🎅🏻","🎅🏼","🎅🏽","🎅🏾","🎅🏿","👦","👦🏻","👦🏼","👦🏽","👦🏾","👦🏿","👧","👧🏻","👧🏼","👧🏽","👧🏾","👧🏿","👨","👨‍⚕️","👨‍⚖️","👨‍✈️","👨‍🌾","👨‍🍳","👨‍🍼","👨‍🎓","👨‍🎤","👨‍🎨","👨‍🏫","👨‍🏭","👨‍💻","👨‍💼","👨‍🔧","👨‍🔬","👨‍🚀","👨‍🚒","👨‍🦰","👨‍🦱","👨‍🦲","👨‍🦳","👨🏻","👨🏻‍⚕️","👨🏻‍⚖️","👨🏻‍✈️","👨🏻‍🌾","👨🏻‍🍳","👨🏻‍🍼","👨🏻‍🎓","👨🏻‍🎤","👨🏻‍🎨","👨🏻‍🏫","👨🏻‍🏭","👨🏻‍💻","👨🏻‍💼","👨🏻‍🔧","👨🏻‍🔬","👨🏻‍🚀","👨🏻‍🚒","👨🏻‍🦰","👨🏻‍🦱","👨🏻‍🦲","👨🏻‍🦳","👨🏼","👨🏼‍⚕️","👨🏼‍⚖️","👨🏼‍✈️","👨🏼‍🌾","👨🏼‍🍳","👨🏼‍🍼","👨🏼‍🎓","👨🏼‍🎤","👨🏼‍🎨","👨🏼‍🏫","👨🏼‍🏭","👨🏼‍💻","👨🏼‍💼","👨🏼‍🔧","👨🏼‍🔬","👨🏼‍🚀","👨🏼‍🚒","👨🏼‍🦰","👨🏼‍🦱","👨🏼‍🦲","👨🏼‍🦳","👨🏽","👨🏽‍⚕️","👨🏽‍⚖️","👨🏽‍✈️","👨🏽‍🌾","👨🏽‍🍳","👨🏽‍🍼","👨🏽‍🎓","👨🏽‍🎤","👨🏽‍🎨","👨🏽‍🏫","👨🏽‍🏭","👨🏽‍💻","👨🏽‍💼","👨🏽‍🔧","👨🏽‍🔬","👨🏽‍🚀","👨🏽‍🚒","👨🏽‍🦰","👨🏽‍🦱","👨🏽‍🦲","👨🏽‍🦳","👨🏾","👨🏾‍⚕️","👨🏾‍⚖️","👨🏾‍✈️","👨🏾‍🌾","👨🏾‍🍳","👨🏾‍🍼","👨🏾‍🎓","👨🏾‍🎤","👨🏾‍🎨","👨🏾‍🏫","👨🏾‍🏭","👨🏾‍💻","👨🏾‍💼","👨🏾‍🔧","👨🏾‍🔬","👨🏾‍🚀","👨🏾‍🚒","👨🏾‍🦰","👨🏾‍🦱","👨🏾‍🦲","👨🏾‍🦳","👨🏿","👨🏿‍⚕️","👨🏿‍⚖️","👨🏿‍✈️","👨🏿‍🌾","👨🏿‍🍳","👨🏿‍🍼","👨🏿‍🎓","👨🏿‍🎤","👨🏿‍🎨","👨🏿‍🏫","👨🏿‍🏭","👨🏿‍💻","👨🏿‍💼","👨🏿‍🔧","👨🏿‍🔬","👨🏿‍🚀","👨🏿‍🚒","👨🏿‍🦰","👨🏿‍🦱","👨🏿‍🦲","👨🏿‍🦳","👩","👩‍⚕️","👩‍⚖️","👩‍✈️","👩‍🌾","👩‍🍳","👩‍🍼","👩‍🎓","👩‍🎤","👩‍🎨","👩‍🏫","👩‍🏭","👩‍💻","👩‍💼","👩‍🔧","👩‍🔬","👩‍🚀","👩‍🚒","👩‍🦰","👩‍🦱","👩‍🦲","👩‍🦳","👩🏻","👩🏻‍⚕️","👩🏻‍⚖️","👩🏻‍✈️","👩🏻‍🌾","👩🏻‍🍳","👩🏻‍🍼","👩🏻‍🎓","👩🏻‍🎤","👩🏻‍🎨","👩🏻‍🏫","👩🏻‍🏭","👩🏻‍💻","👩🏻‍💼","👩🏻‍🔧","👩🏻‍🔬","👩🏻‍🚀","👩🏻‍🚒","👩🏻‍🦰","👩🏻‍🦱","👩🏻‍🦲","👩🏻‍🦳","👩🏼","👩🏼‍⚕️","👩🏼‍⚖️","👩🏼‍✈️","👩🏼‍🌾","👩🏼‍🍳","👩🏼‍🍼","👩🏼‍🎓","👩🏼‍🎤","👩🏼‍🎨","👩🏼‍🏫","👩🏼‍🏭","👩🏼‍💻","👩🏼‍💼","👩🏼‍🔧","👩🏼‍🔬","👩🏼‍🚀","👩🏼‍🚒","👩🏼‍🦰","👩🏼‍🦱","👩🏼‍🦲","👩🏼‍🦳","👩🏽","👩🏽‍⚕️","👩🏽‍⚖️","👩🏽‍✈️","👩🏽‍🌾","👩🏽‍🍳","👩🏽‍🍼","👩🏽‍🎓","👩🏽‍🎤","👩🏽‍🎨","👩🏽‍🏫","👩🏽‍🏭","👩🏽‍💻","👩🏽‍💼","👩🏽‍🔧","👩🏽‍🔬","👩🏽‍🚀","👩🏽‍🚒","👩🏽‍🦰","👩🏽‍🦱","👩🏽‍🦲","👩🏽‍🦳","👩🏾","👩🏾‍⚕️","👩🏾‍⚖️","👩🏾‍✈️","👩🏾‍🌾","👩🏾‍🍳","👩🏾‍🍼","👩🏾‍🎓","👩🏾‍🎤","👩🏾‍🎨","👩🏾‍🏫","👩🏾‍🏭","👩🏾‍💻","👩🏾‍💼","👩🏾‍🔧","👩🏾‍🔬","👩🏾‍🚀","👩🏾‍🚒","👩🏾‍🦰","👩🏾‍🦱","👩🏾‍🦲","👩🏾‍🦳","👩🏿","👩🏿‍⚕️","👩🏿‍⚖️","👩🏿‍✈️","👩🏿‍🌾","👩🏿‍🍳","👩🏿‍🍼","👩🏿‍🎓","👩🏿‍🎤","👩🏿‍🎨","👩🏿‍🏫","👩🏿‍🏭","👩🏿‍💻","👩🏿‍💼","👩🏿‍🔧","👩🏿‍🔬","👩🏿‍🚀","👩🏿‍🚒","👩🏿‍🦰","👩🏿‍🦱","👩🏿‍🦲","👩🏿‍🦳","👮","👮‍♀️","👮‍♂️","👮🏻","👮🏻‍♀️","👮🏻‍♂️","👮🏼","👮🏼‍♀️","👮🏼‍♂️","👮🏽","👮🏽‍♀️","👮🏽‍♂️","👮🏾","👮🏾‍♀️","👮🏾‍♂️","👮🏿","👮🏿‍♀️","👮🏿‍♂️","👰","👰‍♀️","👰‍♂️","👰🏻","👰🏻‍♀️","👰🏻‍♂️","👰🏼","👰🏼‍♀️","👰🏼‍♂️","👰🏽","👰🏽‍♀️","👰🏽‍♂️","👰🏾","👰🏾‍♀️","👰🏾‍♂️","👰🏿","👰🏿‍♀️","👰🏿‍♂️","👱","👱‍♀️","👱‍♂️","👱🏻","👱🏻‍♀️","👱🏻‍♂️","👱🏼","👱🏼‍♀️","👱🏼‍♂️","👱🏽","👱🏽‍♀️","👱🏽‍♂️","👱🏾","👱🏾‍♀️","👱🏾‍♂️","👱🏿","👱🏿‍♀️","👱🏿‍♂️","👲","👲🏻","👲🏼","👲🏽","👲🏾","👲🏿","👳","👳‍♀️","👳‍♂️","👳🏻","👳🏻‍♀️","👳🏻‍♂️","👳🏼","👳🏼‍♀️","👳🏼‍♂️","👳🏽","👳🏽‍♀️","👳🏽‍♂️","👳🏾","👳🏾‍♀️","👳🏾‍♂️","👳🏿","👳🏿‍♀️","👳🏿‍♂️","👴","👴🏻","👴🏼","👴🏽","👴🏾","👴🏿","👵","👵🏻","👵🏼","👵🏽","👵🏾","👵🏿","👶","👶🏻","👶🏼","👶🏽","👶🏾","👶🏿","👷","👷‍♀️","👷‍♂️","👷🏻","👷🏻‍♀️","👷🏻‍♂️","👷🏼","👷🏼‍♀️","👷🏼‍♂️","👷🏽","👷🏽‍♀️","👷🏽‍♂️","👷🏾","👷🏾‍♀️","👷🏾‍♂️","👷🏿","👷🏿‍♀️","👷🏿‍♂️","👸","👸🏻","👸🏼","👸🏽","👸🏾","👸🏿","👼","👼🏻","👼🏼","👼🏽","👼🏾","👼🏿","💁","💁‍♀️","💁‍♂️","💁🏻","💁🏻‍♀️","💁🏻‍♂️","💁🏼","💁🏼‍♀️","💁🏼‍♂️","💁🏽","💁🏽‍♀️","💁🏽‍♂️","💁🏾","💁🏾‍♀️","💁🏾‍♂️","💁🏿","💁🏿‍♀️","💁🏿‍♂️","💂","💂‍♀️","💂‍♂️","💂🏻","💂🏻‍♀️","💂🏻‍♂️","💂🏼","💂🏼‍♀️","💂🏼‍♂️","💂🏽","💂🏽‍♀️","💂🏽‍♂️","💂🏾","💂🏾‍♀️","💂🏾‍♂️","💂🏿","💂🏿‍♀️","💂🏿‍♂️","💆","💆‍♀️","💆‍♂️","💆🏻","💆🏻‍♀️","💆🏻‍♂️","💆🏼","💆🏼‍♀️","💆🏼‍♂️","💆🏽","💆🏽‍♀️","💆🏽‍♂️","💆🏾","💆🏾‍♀️","💆🏾‍♂️","💆🏿","💆🏿‍♀️","💆🏿‍♂️","💇","💇🏻","💇🏼","💇🏽","🕵🏻","🕵🏻‍♀️","🕵🏻‍♂️","🕵🏼","🕵🏼‍♀️","🕵🏼‍♂️","🕵🏽","🕵🏽‍♀️","🕵🏽‍♂️","🕵🏾","🕵🏾‍♀️","🕵🏾‍♂️","🕵🏿","🕵🏿‍♀️","🕵🏿‍♂️","🕵️","🕵️‍♀️","🕵️‍♂️","🙅","🙅‍♀️","🙅‍♂️","🙅🏻","🙅🏻‍♀️","🙅🏻‍♂️","🙅🏼","🙅🏼‍♀️","🙅🏼‍♂️","🙅🏽","🙅🏽‍♀️","🙅🏽‍♂️","🙅🏾","🙅🏾‍♀️","🙅🏾‍♂️","🙅🏿","🙅🏿‍♀️","🙅🏿‍♂️","🙆","🙆‍♀️","🙆‍♂️","🙆🏻","🙆🏻‍♀️","🙆🏻‍♂️","🙆🏼","🙆🏼‍♀️","🙆🏼‍♂️","🙆🏽","🙆🏽‍♀️","🙆🏽‍♂️","🙆🏾","🙆🏾‍♀️","🙆🏾‍♂️","🙆🏿","🙆🏿‍♀️","🙆🏿‍♂️","🙇","🙇‍♀️","🙇‍♂️","🙇🏻","🙇🏻‍♀️","🙇🏻‍♂️","🙇🏼","🙇🏼‍♀️","🙇🏼‍♂️","🙇🏽","🙇🏽‍♀️","🙇🏽‍♂️","🙇🏾","🙇🏾‍♀️","🙇🏾‍♂️","🙇🏿","🙇🏿‍♀️","🙇🏿‍♂️","🙋","🙋‍♀️","🙋‍♂️","🙋🏻","🙋🏻‍♀️","🙋🏻‍♂️","🙋🏼","🙋🏼‍♀️","🙋🏼‍♂️","🙋🏽","🙋🏽‍♀️","🙋🏽‍♂️","🙋🏾","🙋🏾‍♀️","🙋🏾‍♂️","🙋🏿","🙋🏿‍♀️","🙋🏿‍♂️","🙍","🙍‍♀️","🙍‍♂️","🙍🏻","🙍🏻‍♀️","🙍🏻‍♂️","🙍🏼","🙍🏼‍♀️","🙍🏼‍♂️","🙍🏽","🙍🏽‍♀️","🙍🏽‍♂️","🙍🏾","🙍🏾‍♀️","🙍🏾‍♂️","🙍🏿","🙍🏿‍♀️","🙍🏿‍♂️","🙎","🙎‍♀️","🙎‍♂️","🙎🏻","🙎🏻‍♀️","🙎🏻‍♂️","🙎🏼","🙎🏼‍♀️","🙎🏼‍♂️","🙎🏽","🙎🏽‍♀️","🙎🏽‍♂️","🙎🏾","🙎🏾‍♀️","🙎🏾‍♂️","🙎🏿","🙎🏿‍♀️","🙎🏿‍♂️","🤦","🤦‍♀️","🤦‍♂️","🤦🏻","🤦🏻‍♀️","🤦🏻‍♂️","🤦🏼","🤦🏼‍♀️","🤦🏼‍♂️","🤦🏽","🤦🏽‍♀️","🤦🏽‍♂️","🤦🏾","🤦🏾‍♀️","🤦🏾‍♂️","🤦🏿","🤦🏿‍♀️","🤦🏿‍♂️","🤰","🤰🏻","🤰🏼","🤰🏽","🤰🏾","🤰🏿","🤱","🤱🏻","🤱🏼","🤱🏽","🤱🏾","🤱🏿","🤴","🤴🏻","🤴🏼","🤴🏽","🤴🏾","🤴🏿","🤵","🤵‍♀️","🤵‍♂️","🤵🏻","🤵🏻‍♀️","🤵🏻‍♂️","🤵🏼","🤵🏼‍♀️","🤵🏼‍♂️","🤵🏽","🤵🏽‍♀️","🤵🏽‍♂️","🤵🏾","🤵🏾‍♀️","🤵🏾‍♂️","🤵🏿","🤵🏿‍♀️","🤵🏿‍♂️","🤶","🤶🏻","🤶🏼","🤶🏽","🤶🏾","🤶🏿","🤷","🤷‍♀️","🤷‍♂️","🤷🏻","🤷🏻‍♀️","🤷🏻‍♂️","🤷🏼","🤷🏼‍♀️","🤷🏼‍♂️","🤷🏽","🤷🏽‍♀️","🤷🏽‍♂️","🤷🏾","🤷🏾‍♀️","🤷🏾‍♂️","🤷🏿","🤷🏿‍♀️","🤷🏿‍♂️","🥷","🥷🏻","🥷🏼","🥷🏽","🥷🏾","🥷🏿","🦸","🦸‍♀️","🦸‍♂️","🦸🏻","🦸🏻‍♀️","🦸🏻‍♂️","🦸🏼","🦸🏼‍♀️","🦸🏼‍♂️","🦸🏽","🦸🏽‍♀️","🦸🏽‍♂️","🦸🏾","🦸🏾‍♀️","🦸🏾‍♂️","🦸🏿","🦸🏿‍♀️","🦸🏿‍♂️","🦹","🦹‍♀️","🦹‍♂️","🦹🏻","🦹🏻‍♀️","🦹🏻‍♂️","🦹🏼","🦹🏼‍♀️","🦹🏼‍♂️","🦹🏽","🦹🏽‍♀️","🦹🏽‍♂️","🦹🏾","🦹🏾‍♀️","🦹🏾‍♂️","🦹🏿","🦹🏿‍♀️","🦹🏿‍♂️","🧏","🧏‍♀️","🧏‍♂️","🧏🏻","🧏🏻‍♀️","🧏🏻‍♂️","🧏🏼","🧏🏼‍♀️","🧏🏼‍♂️","🧏🏽","🧏🏽‍♀️","🧏🏽‍♂️","🧏🏾","🧏🏾‍♀️","🧏🏾‍♂️","🧏🏿","🧏🏿‍♀️","🧏🏿‍♂️","🧑","🧑‍⚕️","🧑‍⚖️","🧑‍✈️","🧑‍🌾","🧑‍🍳","🧑‍🍼","🧑‍🎄","🧑‍🎓","🧑‍🎤","🧑‍🎨","🧑‍🏫","🧑‍🏭","🧑‍💻","🧑‍💼","🧑‍🔧","🧑‍🔬","🧑‍🚀","🧑‍🚒","🧑‍🦰","🧑‍🦱","🧑‍🦲","🧑‍🦳","🧑🏻","🧑🏻‍⚕️","🧑🏻‍⚖️","🧑🏻‍✈️","🧑🏻‍🌾","🧑🏻‍🍳","🧑🏻‍🍼","🧑🏻‍🎄","🧑🏻‍🎓","🧑🏻‍🎤","🧑🏻‍🎨","🧑🏻‍🏫","🧑🏻‍🏭","🧑🏻‍💻","🧑🏻‍💼","🧑🏻‍🔧","🧑🏻‍🔬","🧑🏻‍🚀","🧑🏻‍🚒","🧑🏻‍🦰","🧑🏻‍🦱","🧑🏻‍🦲","🧑🏻‍🦳","🧑🏼","🧑🏼‍⚕️","🧑🏼‍⚖️","🧑🏼‍✈️","🧑🏼‍🌾","🧑🏼‍🍳","🧑🏼‍🍼","🧑🏼‍🎄","🧑🏼‍🎓","🧑🏼‍🎤","🧑🏼‍🎨","🧑🏼‍🏫","🧑🏼‍🏭","🧑🏼‍💻","🧑🏼‍💼","🧑🏼‍🔧","🧑🏼‍🔬","🧑🏼‍🚀","🧑🏼‍🚒","🧑🏼‍🦰","🧑🏼‍🦱","🧑🏼‍🦲","🧑🏼‍🦳","🧑🏽","🧑🏽‍⚕️","🧑🏽‍⚖️","🧑🏽‍✈️","🧑🏽‍🌾","🧑🏽‍🍳","🧑🏽‍🍼","🧑🏽‍🎄","🧑🏽‍🎓","🧑🏽‍🎤","🧑🏽‍🎨","🧑🏽‍🏫","🧑🏽‍🏭","🧑🏽‍💻","🧑🏽‍💼","🧑🏽‍🔧","🧑🏽‍🔬","🧑🏽‍🚀","🧑🏽‍🚒","🧑🏽‍🦰","🧑🏽‍🦱","🧑🏽‍🦲","🧑🏽‍🦳","🧑🏾","🧑🏾‍⚕️","🧑🏾‍⚖️","🧑🏾‍✈️","🧑🏾‍🌾","🧑🏾‍🍳","🧑🏾‍🍼","🧑🏾‍🎄","🧑🏾‍🎓","🧑🏾‍🎤","🧑🏾‍🎨","🧑🏾‍🏫","🧑🏾‍🏭","🧑🏾‍💻","🧑🏾‍💼","🧑🏾‍🔧","🧑🏾‍🔬","🧑🏾‍🚀","🧑🏾‍🚒","🧑🏾‍🦰","🧑🏾‍🦱","🧑🏾‍🦲","🧑🏾‍🦳","🧑🏿","🧑🏿‍⚕️","🧑🏿‍⚖️","🧑🏿‍✈️","🧑🏿‍🌾","🧑🏿‍🍳","🧑🏿‍🍼","🧑🏿‍🎄","🧑🏿‍🎓","🧑🏿‍🎤","🧑🏿‍🎨","🧑🏿‍🏫","🧑🏿‍🏭","🧑🏿‍💻","🧑🏿‍💼","🧑🏿‍🔧","🧑🏿‍🔬","🧑🏿‍🚀","🧑🏿‍🚒","🧑🏿‍🦰","🧑🏿‍🦱","🧑🏿‍🦲","🧑🏿‍🦳","🧒","🧒🏻","🧒🏼","🧒🏽","🧒🏾","🧒🏿","🧓","🧓🏻","🧓🏼","🧓🏽","🧓🏾","🧓🏿","🧔","🧔‍♀️","🧔‍♂️","🧔🏻","🧔🏻‍♀️","🧔🏻‍♂️","🧔🏼","🧔🏼‍♀️","🧔🏼‍♂️","🧔🏽","🧔🏽‍♀️","🧔🏽‍♂️","🧔🏾","🧔🏾‍♀️","🧔🏾‍♂️","🧔🏿","🧔🏿‍♀️","🧔🏿‍♂️","🧕","🧕🏻","🧕🏼","🧕🏽","🧕🏾","🧕🏿","🧙","🧙‍♀️","🧙‍♂️","🧙🏻","🧙🏻‍♀️","🧙🏻‍♂️","🧙🏼","🧙🏼‍♀️","🧙🏼‍♂️","🧙🏽","🧙🏽‍♀️","🧙🏽‍♂️","🧙🏾","🧙🏾‍♀️","🧙🏾‍♂️","🧙🏿","🧙🏿‍♀️","🧙🏿‍♂️","🧚","🧚‍♀️","🧚‍♂️","🧚🏻","🧚🏻‍♀️","🧚🏻‍♂️","🧚🏼","🧚🏼‍♀️","🧚🏼‍♂️","🧚🏽","🧚🏽‍♀️","🧚🏽‍♂️","🧚🏾","🧚🏾‍♀️","🧚🏾‍♂️","🧚🏿","🧚🏿‍♀️","🧚🏿‍♂️","🧛","🧛‍♀️","🧛‍♂️","🧛🏻","🧛🏻‍♀️","🧛🏻‍♂️","🧛🏼","🧛🏼‍♀️","🧛🏼‍♂️","🧛🏽","🧛🏽‍♀️","🧛🏽‍♂️","🧛🏾","🧛🏾‍♀️","🧛🏾‍♂️","🧛🏿","🧛🏿‍♀️","🧛🏿‍♂️","🧜","🧜‍♀️","🧜‍♂️","🧜🏻","🧜🏻‍♀️","🧜🏻‍♂️","🧜🏼","🧜🏼‍♀️","🧜🏼‍♂️","🧜🏽","🧜🏽‍♀️","🧜🏽‍♂️","🧜🏾","🧜🏾‍♀️","🧜🏾‍♂️","🧜🏿","🧜🏿‍♀️","🧜🏿‍♂️","🧝","🧝‍♀️","🧝‍♂️","🧝🏻","🧝🏻‍♀️","🧝🏻‍♂️","🧝🏼","🧝🏼‍♀️","🧝🏼‍♂️","🧝🏽","🧝🏽‍♀️","🧝🏽‍♂️","🧝🏾","🧝🏾‍♀️","🧝🏾‍♂️","🧝🏿","🧝🏿‍♀️","🧝🏿‍♂️","🧞","🧞‍♀️","🧞‍♂️","🧟","🧟‍♀️","🧟‍♂️"],nature:["☘️","🌱","🌲","🌳","🌴","🌵","🌷","🌸","🌹","🌺","🌻","🌼","🌾","🌿","🍀","🍁","🍂","🍃","🏵️","🐀","🐁","🐂","🐃","🐄","🐅","🐆","🐇","🐈","🐈‍⬛","🐉","🐊","🐋","🐌","🐍","🐎","🐏","🐐","🐑","🐒","🐓","🐔","🐕","🐕‍🦺","🐖","🐗","🐘","🐙","🐚","🐛","🐜","🐝","🐞","🐟","🐠","🐡","🐢","🐣","🐤","🐥","🐦","🐧","🐨","🐩","🐪","🐫","🐬","🐭","🐮","🐯","🐰","🐱","🐲","🐳","🐴","🐵","🐶","🐷","🐸","🐹","🐺","🐻","🐻‍❄️","🐼","🐽","🐾","🐿️","💐","💮","🕊️","🕷️","🕸️","🥀","🦁","🦂","🦃","🦄","🦅","🦆","🦇","🦈","🦉","🦊","🦋","🦌","🦍","🦎","🦏","🦒","🦓","🦔","🦕","🦖","🦗","🦘","🦙","🦚","🦛","🦜","🦝","🦟","🦠","🦡","🦢","🦣","🦤","🦥","🦦","🦧","🦨","🦩","🦫","🦬","🦭","🦮","🪰","🪱","🪲","🪳","🪴","🪶"],food:["☕","🌭","🌮","🌯","🌰","🌶️","🌽","🍄","🍅","🍆","🍇","🍈","🍉","🍊","🍋","🍌","🍍","🍎","🍏","🍐","🍑","🍒","🍓","🍔","🍕","🍖","🍗","🍘","🍙","🍚","🍛","🍜","🍝","🍞","🍟","🍠","🍡","🍢","🍣","🍤","🍥","🍦","🍧","🍨","🍩","🍪","🍫","🍬","🍭","🍮","🍯","🍰","🍱","🍲","🍳","🍴","🍵","🍶","🍷","🍸","🍹","🍺","🍻","🍼","🍽️","🍾","🍿","🎂","🏺","🔪","🥂","🥃","🥄","🥐","🥑","🥒","🥓","🥔","🥕","🥖","🥗","🥘","🥙","🥚","🥛","🥜","🥝","🥞","🥟","🥠","🥡","🥢","🥣","🥤","🥥","🥦","🥧","🥨","🥩","🥪","🥫","🥬","🥭","🥮","🥯","🦀","🦐","🦑","🦞","🦪","🧀","🧁","🧂","🧃","🧄","🧅","🧆","🧇","🧈","🧉","🧊","🧋","🫐","🫑","🫒","🫓","🫔","🫕","🫖"],travel:["⌚","⌛","⏰","⏱️","⏲️","⏳","☀️","☁️","☂️","☃️","☄️","☔","♨️","⚓","⚡","⛄","⛅","⛈️","⛩️","⛪","⛰️","⛱️","⛲","⛴️","⛵","⛺","⛽","✈️","❄️","⭐","🌀","🌁","🌂","🌃","🌄","🌅","🌆","🌇","🌈","🌉","🌊","🌋","🌌","🌍","🌎","🌏","🌐","🌑","🌒","🌓","🌔","🌕","🌖","🌗","🌘","🌙","🌚","🌛","🌜","🌝","🌞","🌟","🌠","🌡️","🌤️","🌥️","🌦️","🌧️","🌨️","🌩️","🌪️","🌫️","🌬️","🎠","🎡","🎢","🎪","🏍️","🏎️","🏔️","🏕️","🏖️","🏗️","🏘️","🏙️","🏚️","🏛️","🏜️","🏝️","🏞️","🏟️","🏠","🏡","🏢","🏣","🏤","🏥","🏦","🏨","🏩","🏪","🏫","🏬","🏭","🏯","🏰","💈","💒","💧","💺","🔥","🕋","🕌","🕍","🕐","🕑","🕒","🕓","🕔","🕕","🕖","🕗","🕘","🕙","🕚","🕛","🕜","🕝","🕞","🕟","🕠","🕡","🕢","🕣","🕤","🕥","🕦","🕧","🕰️","🗺️","🗻","🗼","🗽","🗾","🚀","🚁","🚂","🚃","🚄","🚅","🚆","🚇","🚈","🚉","🚊","🚋","🚌","🚍","🚎","🚏","🚐","🚑","🚒","🚓","🚔","🚕","🚖","🚗","🚘","🚙","🚚","🚛","🚜","🚝","🚞","🚟","🚠","🚡","🚢","🚤","🚥","🚦","🚧","🚨","🚲","🛎️","🛑","🛕","🛖","🛢️","🛣️","🛤️","🛥️","🛩️","🛫","🛬","🛰️","🛳️","🛴","🛵","🛶","🛸","🛹","🛺","🛻","🛼","🦼","🦽","🧭","🧱","🧳","🪂","🪐","🪨","🪵"],activity:["♟️","♠️","♣️","♥️","♦️","⚽","⚾","⛳","⛸️","✨","🀄","🃏","🎀","🎁","🎃","🎄","🎆","🎇","🎈","🎉","🎊","🎋","🎍","🎎","🎏","🎐","🎑","🎖️","🎗️","🎟️","🎣","🎨","🎫","🎭","🎮","🎯","🎰","🎱","🎲","🎳","🎴","🎽","🎾","🎿","🏀","🏅","🏆","🏈","🏉","🏏","🏐","🏑","🏒","🏓","🏸","🔮","🕹️","🖼️","🛷","🤿","🥅","🥇","🥈","🥉","🥊","🥋","🥌","🥍","🥎","🥏","🧧","🧨","🧩","🧵","🧶","🧸","🧿","🪀","🪁","🪄","🪅","🪆","🪡","🪢"],object:["⌨️","☎️","⚒️","⚔️","⚖️","⚗️","⚙️","⚰️","⚱️","⛏️","⛑️","⛓️","✂️","✉️","✏️","✒️","🎒","🎓","🎙️","🎚️","🎛️","🎞️","🎤","🎥","🎧","🎩","🎬","🎵","🎶","🎷","🎸","🎹","🎺","🎻","🎼","🏮","🏷️","🏹","👑","👒","👓","👔","👕","👖","👗","👘","👙","👚","👛","👜","👝","👞","👟","👠","👡","👢","💄","💉","💊","💍","💎","💡","💰","💳","💴","💵","💶","💷","💸","💹","💻","💼","💽","💾","💿","📀","📁","📂","📃","📄","📅","📆","📇","📈","📉","📊","📋","📌","📍","📎","📏","📐","📑","📒","📓","📔","📕","📖","📗","📘","📙","📚","📜","📝","📞","📟","📠","📡","📢","📣","📤","📥","📦","📧","📨","📩","📪","📫","📬","📭","📮","📯","📰","📱","📲","📷","📸","📹","📺","📻","📼","📽️","📿","🔇","🔈","🔉","🔊","🔋","🔌","🔍","🔎","🔏","🔐","🔑","🔒","🔓","🔔","🔕","🔖","🔗","🔦","🔧","🔨","🔩","🔫","🔬","🔭","🕯️","🕶️","🖇️","🖊️","🖋️","🖌️","🖍️","🖥️","🖨️","🖱️","🖲️","🗂️","🗃️","🗄️","🗑️","🗒️","🗓️","🗜️","🗝️","🗞️","🗡️","🗳️","🗿","🚪","🚬","🚽","🚿","🛁","🛋️","🛍️","🛏️","🛒","🛗","🛠️","🛡️","🥁","🥻","🥼","🥽","🥾","🥿","🦯","🦺","🧢","🧣","🧤","🧥","🧦","🧪","🧫","🧬","🧮","🧯","🧰","🧲","🧴","🧷","🧹","🧺","🧻","🧼","🧽","🧾","🩰","🩱","🩲","🩳","🩴","🩸","🩹","🩺","🪃","🪑","🪒","🪓","🪔","🪕","🪖","🪗","🪘","🪙","🪚","🪛","🪜","🪝","🪞","🪟","🪠","🪣","🪤","🪥","🪦","🪧"],symbol:["#️⃣","*️⃣","0️⃣","1️⃣","2️⃣","3️⃣","4️⃣","5️⃣","6️⃣","7️⃣","8️⃣","9️⃣","©️","®️","‼️","⁉️","™️","ℹ️","↔️","↕️","↖️","↗️","↘️","↙️","↩️","↪️","⏏️","⏩","⏪","⏫","⏬","⏭️","⏮️","⏯️","⏸️","⏹️","⏺️","Ⓜ️","▪️","▫️","▶️","◀️","◻️","◼️","◽","◾","☑️","☢️","☣️","☦️","☪️","☮️","☯️","☸️","♀️","♂️","♈","♉","♊","♋","♌","♍","♎","♏","♐","♑","♒","♓","♻️","♾️","♿","⚕️","⚛️","⚜️","⚠️","⚧️","⚪","⚫","⛎","⛔","✅","✔️","✖️","✝️","✡️","✳️","✴️","❇️","❌","❎","❓","❔","❕","❗","➕","➖","➗","➡️","➰","➿","⤴️","⤵️","⬅️","⬆️","⬇️","⬛","⬜","⭕","〰️","〽️","㊗️","㊙️","🅰️","🅱️","🅾️","🅿️","🆎","🆑","🆒","🆓","🆔","🆕","🆖","🆗","🆘","🆙","🆚","🈁","🈂️","🈚","🈯","🈲","🈳","🈴","🈵","🈶","🈷️","🈸","🈹","🈺","🉐","🉑","🎦","🏧","💠","💱","💲","📛","📳","📴","📵","📶","🔀","🔁","🔂","🔃","🔄","🔅","🔆","🔘","🔙","🔚","🔛","🔜","🔝","🔞","🔟","🔠","🔡","🔢","🔣","🔤","🔯","🔰","🔱","🔲","🔳","🔴","🔵","🔶","🔷","🔸","🔹","🔺","🔻","🔼","🔽","🕉️","🕎","🚫","🚭","🚮","🚯","🚰","🚱","🚳","🚷","🚸","🚹","🚺","🚻","🚼","🚾","🛂","🛃","🛄","🛅","🛐","🟠","🟡","🟢","🟣","🟤","🟥","🟦","🟧","🟨","🟩","🟪","🟫"],flag:["🇦🇨","🇦🇩","🇦🇪","🇦🇫","🇦🇬","🇦🇮","🇦🇱","🇦🇲","🇦🇴","🇦🇶","🇦🇷","🇦🇸","🇦🇹","🇦🇺","🇦🇼","🇦🇽","🇦🇿","🇧🇦","🇧🇧","🇧🇩","🇧🇪","🇧🇫","🇧🇬","🇧🇭","🇧🇮","🇧🇯","🇧🇱","🇧🇲","🇧🇳","🇧🇴","🇧🇶","🇧🇷","🇧🇸","🇧🇹","🇧🇻","🇧🇼","🇧🇾","🇧🇿","🇨🇦","🇨🇨","🇨🇩","🇨🇫","🇨🇬","🇨🇭","🇨🇮","🇨🇰","🇨🇱","🇨🇲","🇨🇳","🇨🇴","🇨🇵","🇨🇷","🇨🇺","🇨🇻","🇨🇼","🇨🇽","🇨🇾","🇨🇿","🇩🇪","🇩🇬","🇩🇯","🇩🇰","🇩🇲","🇩🇴","🇩🇿","🇪🇦","🇪🇨","🇪🇪","🇪🇬","🇪🇭","🇪🇷","🇪🇸","🇪🇹","🇪🇺","🇫🇮","🇫🇯","🇫🇰","🇫🇲","🇫🇴","🇫🇷","🇬🇦","🇬🇧","🇬🇩","🇬🇪","🇬🇫","🇬🇬","🇬🇭","🇬🇮","🇬🇱","🇬🇲","🇬🇳","🇬🇵","🇬🇶","🇬🇷","🇬🇸","🇬🇹","🇬🇺","🇬🇼","🇬🇾","🇭🇰","🇭🇲","🇭🇳","🇭🇷","🇭🇹","🇭🇺","🇮🇨","🇮🇩","🇮🇪","🇮🇱","🇮🇲","🇮🇳","🇮🇴","🇮🇶","🇮🇷","🇮🇸","🇮🇹","🇯🇪","🇯🇲","🇯🇴","🇯🇵","🇰🇪","🇰🇬","🇰🇭","🇰🇮","🇰🇲","🇰🇳","🇰🇵","🇰🇷","🇰🇼","🇰🇾","🇰🇿","🇱🇦","🇱🇧","🇱🇨","🇱🇮","🇱🇰","🇱🇷","🇱🇸","🇱🇹","🇱🇺","🇱🇻","🇱🇾","🇲🇦","🇲🇨","🇲🇩","🇲🇪","🇲🇫","🇲🇬","🇲🇭","🇲🇰","🇲🇱","🇲🇲","🇲🇳","🇲🇴","🇲🇵","🇲🇶","🇲🇷","🇲🇸","🇲🇹","🇲🇺","🇲🇻","🇲🇼","🇲🇽","🇲🇾","🇲🇿","🇳🇦","🇳🇨","🇳🇪","🇳🇫","🇳🇬","🇳🇮","🇳🇱","🇳🇴","🇳🇵","🇳🇷","🇳🇺","🇳🇿","🇴🇲","🇵🇦","🇵🇪","🇵🇫","🇵🇬","🇵🇭","🇵🇰","🇵🇱","🇵🇲","🇵🇳","🇵🇷","🇵🇸","🇵🇹","🇵🇼","🇵🇾","🇶🇦","🇷🇪","🇷🇴","🇷🇸","🇷🇺","🇷🇼","🇸🇦","🇸🇧","🇸🇨","🇸🇩","🇸🇪","🇸🇬","🇸🇭","🇸🇮","🇸🇯","🇸🇰","🇸🇱","🇸🇲","🇸🇳","🇸🇴","🇸🇷","🇸🇸","🇸🇹","🇸🇻","🇸🇽","🇸🇾","🇸🇿","🇹🇦","🇹🇨","🇹🇩","🇹🇫","🇹🇬","🇹🇭","🇹🇯","🇹🇰","🇹🇱","🇹🇲","🇹🇳","🇹🇴","🇹🇷","🇹🇹","🇹🇻","🇹🇼","🇹🇿","🇺🇦","🇺🇬","🇺🇲","🇺🇳","🇺🇸","🇺🇾","🇺🇿","🇻🇦","🇻🇨","🇻🇪","🇻🇬","🇻🇮","🇻🇳","🇻🇺","🇼🇫","🇼🇸","🇽🇰","🇾🇪","🇾🇹","🇿🇦","🇿🇲","🇿🇼","🎌","🏁","🏳️","🏳️‍⚧️","🏳️‍🌈","🏴","🏴‍☠️","🚩"]},d6e={informational:[100,101,102,103],success:[200,201,202,203,204,205,206,207,208,226],redirection:[300,301,302,303,304,305,306,307,308],clientError:[400,401,402,403,404,405,406,407,408,409,410,411,412,413,414,415,416,417,418,421,422,423,424,425,426,428,429,431,451],serverError:[500,501,502,503,504,505,506,507,508,510,511]},h6e=["ES256","ES384","ES512","HS256","HS384","HS512","PS256","PS384","PS512","RS256","RS384","RS512","none"],p6e=["FakerBot/{{system.semver}}","Googlebot/2.1 (+http://www.google.com/bot.html)",'Mozilla/5.0 (Linux; Android {{number.int({"min":5,"max":13})}}; {{helpers.arrayElement(["SM-G998U","SM-G998B","SM-G998N","SM-G998P","SM-T800"])}}) AppleWebKit/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}} (KHTML, like Gecko) Chrome/{{number.int({"min":55,"max":131})}}.{{system.semver}} Mobile Safari/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}}','Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:{{number.int({"min":75, "max":133})}}.0) Gecko/20100101 Firefox/{{number.int({"min":75, "max":133})}}.0','Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}}.{{number.int({"min":0,"max":99})}} (KHTML, like Gecko) Version/16.1 Safari/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}}.{{number.int({"min":0,"max":99})}}','Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_15_7) AppleWebKit/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}}.{{number.int({"min":0,"max":99})}} (KHTML, like Gecko) Chrome/{{number.int({"min":55,"max":131})}}.{{system.semver}} Safari/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}}.{{number.int({"min":0,"max":99})}}','Mozilla/5.0 (Windows NT {{helpers.arrayElement(["5.1","5.2","6.0","6.1","6.2","6.3","10.0"])}}; Win64; x64) AppleWebKit/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}} (KHTML, like Gecko) Chrome/{{number.int({"min":55,"max":131})}}.{{system.semver}} Safari/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}} Edg/{{number.int({"min":110,"max":131})}}.{{system.semver}}','Mozilla/5.0 (X11; Linux x86_64; rv:{{number.int({"min":75,"max":133})}}.0) Gecko/20100101 Firefox/{{number.int({"min":75,"max":133})}}.0','Mozilla/5.0 (compatible; MSIE {{number.int({"min":6,"max":10})}}.0; Windows NT {{helpers.arrayElement(["5.1","5.2","6.0","6.1","6.2","6.3","10.0"])}}; Trident/{{number.int({"min":4,"max":7})}}.0)','Mozilla/5.0 (iPhone; CPU iPhone OS {{number.int({"min":10,"max":18})}}_{{number.int({"min":0,"max":4})}} like Mac OS X) AppleWebKit/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}}.{{number.int({"min":0,"max":99})}} (KHTML, like Gecko) Version/{{number.int({"min":10,"max":18})}}_{{number.int({"min":0,"max":4})}} Mobile/15E148 Safari/{{number.int({"min":536,"max":605})}}.{{number.int({"min":0,"max":99})}}'],f6e={emoji:c6e,http_status_code:d6e,jwt_algorithm:h6e,user_agent_pattern:p6e},m6e=f6e,g6e=[{alpha2:"AD",alpha3:"AND",numeric:"020"},{alpha2:"AE",alpha3:"ARE",numeric:"784"},{alpha2:"AF",alpha3:"AFG",numeric:"004"},{alpha2:"AG",alpha3:"ATG",numeric:"028"},{alpha2:"AI",alpha3:"AIA",numeric:"660"},{alpha2:"AL",alpha3:"ALB",numeric:"008"},{alpha2:"AM",alpha3:"ARM",numeric:"051"},{alpha2:"AO",alpha3:"AGO",numeric:"024"},{alpha2:"AQ",alpha3:"ATA",numeric:"010"},{alpha2:"AR",alpha3:"ARG",numeric:"032"},{alpha2:"AS",alpha3:"ASM",numeric:"016"},{alpha2:"AT",alpha3:"AUT",numeric:"040"},{alpha2:"AU",alpha3:"AUS",numeric:"036"},{alpha2:"AW",alpha3:"ABW",numeric:"533"},{alpha2:"AX",alpha3:"ALA",numeric:"248"},{alpha2:"AZ",alpha3:"AZE",numeric:"031"},{alpha2:"BA",alpha3:"BIH",numeric:"070"},{alpha2:"BB",alpha3:"BRB",numeric:"052"},{alpha2:"BD",alpha3:"BGD",numeric:"050"},{alpha2:"BE",alpha3:"BEL",numeric:"056"},{alpha2:"BF",alpha3:"BFA",numeric:"854"},{alpha2:"BG",alpha3:"BGR",numeric:"100"},{alpha2:"BH",alpha3:"BHR",numeric:"048"},{alpha2:"BI",alpha3:"BDI",numeric:"108"},{alpha2:"BJ",alpha3:"BEN",numeric:"204"},{alpha2:"BL",alpha3:"BLM",numeric:"652"},{alpha2:"BM",alpha3:"BMU",numeric:"060"},{alpha2:"BN",alpha3:"BRN",numeric:"096"},{alpha2:"BO",alpha3:"BOL",numeric:"068"},{alpha2:"BQ",alpha3:"BES",numeric:"535"},{alpha2:"BR",alpha3:"BRA",numeric:"076"},{alpha2:"BS",alpha3:"BHS",numeric:"044"},{alpha2:"BT",alpha3:"BTN",numeric:"064"},{alpha2:"BV",alpha3:"BVT",numeric:"074"},{alpha2:"BW",alpha3:"BWA",numeric:"072"},{alpha2:"BY",alpha3:"BLR",numeric:"112"},{alpha2:"BZ",alpha3:"BLZ",numeric:"084"},{alpha2:"CA",alpha3:"CAN",numeric:"124"},{alpha2:"CC",alpha3:"CCK",numeric:"166"},{alpha2:"CD",alpha3:"COD",numeric:"180"},{alpha2:"CF",alpha3:"CAF",numeric:"140"},{alpha2:"CG",alpha3:"COG",numeric:"178"},{alpha2:"CH",alpha3:"CHE",numeric:"756"},{alpha2:"CI",alpha3:"CIV",numeric:"384"},{alpha2:"CK",alpha3:"COK",numeric:"184"},{alpha2:"CL",alpha3:"CHL",numeric:"152"},{alpha2:"CM",alpha3:"CMR",numeric:"120"},{alpha2:"CN",alpha3:"CHN",numeric:"156"},{alpha2:"CO",alpha3:"COL",numeric:"170"},{alpha2:"CR",alpha3:"CRI",numeric:"188"},{alpha2:"CU",alpha3:"CUB",numeric:"192"},{alpha2:"CV",alpha3:"CPV",numeric:"132"},{alpha2:"CW",alpha3:"CUW",numeric:"531"},{alpha2:"CX",alpha3:"CXR",numeric:"162"},{alpha2:"CY",alpha3:"CYP",numeric:"196"},{alpha2:"CZ",alpha3:"CZE",numeric:"203"},{alpha2:"DE",alpha3:"DEU",numeric:"276"},{alpha2:"DJ",alpha3:"DJI",numeric:"262"},{alpha2:"DK",alpha3:"DNK",numeric:"208"},{alpha2:"DM",alpha3:"DMA",numeric:"212"},{alpha2:"DO",alpha3:"DOM",numeric:"214"},{alpha2:"DZ",alpha3:"DZA",numeric:"012"},{alpha2:"EC",alpha3:"ECU",numeric:"218"},{alpha2:"EE",alpha3:"EST",numeric:"233"},{alpha2:"EG",alpha3:"EGY",numeric:"818"},{alpha2:"EH",alpha3:"ESH",numeric:"732"},{alpha2:"ER",alpha3:"ERI",numeric:"232"},{alpha2:"ES",alpha3:"ESP",numeric:"724"},{alpha2:"ET",alpha3:"ETH",numeric:"231"},{alpha2:"FI",alpha3:"FIN",numeric:"246"},{alpha2:"FJ",alpha3:"FJI",numeric:"242"},{alpha2:"FK",alpha3:"FLK",numeric:"238"},{alpha2:"FM",alpha3:"FSM",numeric:"583"},{alpha2:"FO",alpha3:"FRO",numeric:"234"},{alpha2:"FR",alpha3:"FRA",numeric:"250"},{alpha2:"GA",alpha3:"GAB",numeric:"266"},{alpha2:"GB",alpha3:"GBR",numeric:"826"},{alpha2:"GD",alpha3:"GRD",numeric:"308"},{alpha2:"GE",alpha3:"GEO",numeric:"268"},{alpha2:"GF",alpha3:"GUF",numeric:"254"},{alpha2:"GG",alpha3:"GGY",numeric:"831"},{alpha2:"GH",alpha3:"GHA",numeric:"288"},{alpha2:"GI",alpha3:"GIB",numeric:"292"},{alpha2:"GL",alpha3:"GRL",numeric:"304"},{alpha2:"GM",alpha3:"GMB",numeric:"270"},{alpha2:"GN",alpha3:"GIN",numeric:"324"},{alpha2:"GP",alpha3:"GLP",numeric:"312"},{alpha2:"GQ",alpha3:"GNQ",numeric:"226"},{alpha2:"GR",alpha3:"GRC",numeric:"300"},{alpha2:"GS",alpha3:"SGS",numeric:"239"},{alpha2:"GT",alpha3:"GTM",numeric:"320"},{alpha2:"GU",alpha3:"GUM",numeric:"316"},{alpha2:"GW",alpha3:"GNB",numeric:"624"},{alpha2:"GY",alpha3:"GUY",numeric:"328"},{alpha2:"HK",alpha3:"HKG",numeric:"344"},{alpha2:"HM",alpha3:"HMD",numeric:"334"},{alpha2:"HN",alpha3:"HND",numeric:"340"},{alpha2:"HR",alpha3:"HRV",numeric:"191"},{alpha2:"HT",alpha3:"HTI",numeric:"332"},{alpha2:"HU",alpha3:"HUN",numeric:"348"},{alpha2:"ID",alpha3:"IDN",numeric:"360"},{alpha2:"IE",alpha3:"IRL",numeric:"372"},{alpha2:"IL",alpha3:"ISR",numeric:"376"},{alpha2:"IM",alpha3:"IMN",numeric:"833"},{alpha2:"IN",alpha3:"IND",numeric:"356"},{alpha2:"IO",alpha3:"IOT",numeric:"086"},{alpha2:"IQ",alpha3:"IRQ",numeric:"368"},{alpha2:"IR",alpha3:"IRN",numeric:"364"},{alpha2:"IS",alpha3:"ISL",numeric:"352"},{alpha2:"IT",alpha3:"ITA",numeric:"380"},{alpha2:"JE",alpha3:"JEY",numeric:"832"},{alpha2:"JM",alpha3:"JAM",numeric:"388"},{alpha2:"JO",alpha3:"JOR",numeric:"400"},{alpha2:"JP",alpha3:"JPN",numeric:"392"},{alpha2:"KE",alpha3:"KEN",numeric:"404"},{alpha2:"KG",alpha3:"KGZ",numeric:"417"},{alpha2:"KH",alpha3:"KHM",numeric:"116"},{alpha2:"KI",alpha3:"KIR",numeric:"296"},{alpha2:"KM",alpha3:"COM",numeric:"174"},{alpha2:"KN",alpha3:"KNA",numeric:"659"},{alpha2:"KP",alpha3:"PRK",numeric:"408"},{alpha2:"KR",alpha3:"KOR",numeric:"410"},{alpha2:"KW",alpha3:"KWT",numeric:"414"},{alpha2:"KY",alpha3:"CYM",numeric:"136"},{alpha2:"KZ",alpha3:"KAZ",numeric:"398"},{alpha2:"LA",alpha3:"LAO",numeric:"418"},{alpha2:"LB",alpha3:"LBN",numeric:"422"},{alpha2:"LC",alpha3:"LCA",numeric:"662"},{alpha2:"LI",alpha3:"LIE",numeric:"438"},{alpha2:"LK",alpha3:"LKA",numeric:"144"},{alpha2:"LR",alpha3:"LBR",numeric:"430"},{alpha2:"LS",alpha3:"LSO",numeric:"426"},{alpha2:"LT",alpha3:"LTU",numeric:"440"},{alpha2:"LU",alpha3:"LUX",numeric:"442"},{alpha2:"LV",alpha3:"LVA",numeric:"428"},{alpha2:"LY",alpha3:"LBY",numeric:"434"},{alpha2:"MA",alpha3:"MAR",numeric:"504"},{alpha2:"MC",alpha3:"MCO",numeric:"492"},{alpha2:"MD",alpha3:"MDA",numeric:"498"},{alpha2:"ME",alpha3:"MNE",numeric:"499"},{alpha2:"MF",alpha3:"MAF",numeric:"663"},{alpha2:"MG",alpha3:"MDG",numeric:"450"},{alpha2:"MH",alpha3:"MHL",numeric:"584"},{alpha2:"MK",alpha3:"MKD",numeric:"807"},{alpha2:"ML",alpha3:"MLI",numeric:"466"},{alpha2:"MM",alpha3:"MMR",numeric:"104"},{alpha2:"MN",alpha3:"MNG",numeric:"496"},{alpha2:"MO",alpha3:"MAC",numeric:"446"},{alpha2:"MP",alpha3:"MNP",numeric:"580"},{alpha2:"MQ",alpha3:"MTQ",numeric:"474"},{alpha2:"MR",alpha3:"MRT",numeric:"478"},{alpha2:"MS",alpha3:"MSR",numeric:"500"},{alpha2:"MT",alpha3:"MLT",numeric:"470"},{alpha2:"MU",alpha3:"MUS",numeric:"480"},{alpha2:"MV",alpha3:"MDV",numeric:"462"},{alpha2:"MW",alpha3:"MWI",numeric:"454"},{alpha2:"MX",alpha3:"MEX",numeric:"484"},{alpha2:"MY",alpha3:"MYS",numeric:"458"},{alpha2:"MZ",alpha3:"MOZ",numeric:"508"},{alpha2:"NA",alpha3:"NAM",numeric:"516"},{alpha2:"NC",alpha3:"NCL",numeric:"540"},{alpha2:"NE",alpha3:"NER",numeric:"562"},{alpha2:"NF",alpha3:"NFK",numeric:"574"},{alpha2:"NG",alpha3:"NGA",numeric:"566"},{alpha2:"NI",alpha3:"NIC",numeric:"558"},{alpha2:"NL",alpha3:"NLD",numeric:"528"},{alpha2:"NO",alpha3:"NOR",numeric:"578"},{alpha2:"NP",alpha3:"NPL",numeric:"524"},{alpha2:"NR",alpha3:"NRU",numeric:"520"},{alpha2:"NU",alpha3:"NIU",numeric:"570"},{alpha2:"NZ",alpha3:"NZL",numeric:"554"},{alpha2:"OM",alpha3:"OMN",numeric:"512"},{alpha2:"PA",alpha3:"PAN",numeric:"591"},{alpha2:"PE",alpha3:"PER",numeric:"604"},{alpha2:"PF",alpha3:"PYF",numeric:"258"},{alpha2:"PG",alpha3:"PNG",numeric:"598"},{alpha2:"PH",alpha3:"PHL",numeric:"608"},{alpha2:"PK",alpha3:"PAK",numeric:"586"},{alpha2:"PL",alpha3:"POL",numeric:"616"},{alpha2:"PM",alpha3:"SPM",numeric:"666"},{alpha2:"PN",alpha3:"PCN",numeric:"612"},{alpha2:"PR",alpha3:"PRI",numeric:"630"},{alpha2:"PS",alpha3:"PSE",numeric:"275"},{alpha2:"PT",alpha3:"PRT",numeric:"620"},{alpha2:"PW",alpha3:"PLW",numeric:"585"},{alpha2:"PY",alpha3:"PRY",numeric:"600"},{alpha2:"QA",alpha3:"QAT",numeric:"634"},{alpha2:"RE",alpha3:"REU",numeric:"638"},{alpha2:"RO",alpha3:"ROU",numeric:"642"},{alpha2:"RS",alpha3:"SRB",numeric:"688"},{alpha2:"RU",alpha3:"RUS",numeric:"643"},{alpha2:"RW",alpha3:"RWA",numeric:"646"},{alpha2:"SA",alpha3:"SAU",numeric:"682"},{alpha2:"SB",alpha3:"SLB",numeric:"090"},{alpha2:"SC",alpha3:"SYC",numeric:"690"},{alpha2:"SD",alpha3:"SDN",numeric:"729"},{alpha2:"SE",alpha3:"SWE",numeric:"752"},{alpha2:"SG",alpha3:"SGP",numeric:"702"},{alpha2:"SH",alpha3:"SHN",numeric:"654"},{alpha2:"SI",alpha3:"SVN",numeric:"705"},{alpha2:"SJ",alpha3:"SJM",numeric:"744"},{alpha2:"SK",alpha3:"SVK",numeric:"703"},{alpha2:"SL",alpha3:"SLE",numeric:"694"},{alpha2:"SM",alpha3:"SMR",numeric:"674"},{alpha2:"SN",alpha3:"SEN",numeric:"686"},{alpha2:"SO",alpha3:"SOM",numeric:"706"},{alpha2:"SR",alpha3:"SUR",numeric:"740"},{alpha2:"SS",alpha3:"SSD",numeric:"728"},{alpha2:"ST",alpha3:"STP",numeric:"678"},{alpha2:"SV",alpha3:"SLV",numeric:"222"},{alpha2:"SX",alpha3:"SXM",numeric:"534"},{alpha2:"SY",alpha3:"SYR",numeric:"760"},{alpha2:"SZ",alpha3:"SWZ",numeric:"748"},{alpha2:"TC",alpha3:"TCA",numeric:"796"},{alpha2:"TD",alpha3:"TCD",numeric:"148"},{alpha2:"TF",alpha3:"ATF",numeric:"260"},{alpha2:"TG",alpha3:"TGO",numeric:"768"},{alpha2:"TH",alpha3:"THA",numeric:"764"},{alpha2:"TJ",alpha3:"TJK",numeric:"762"},{alpha2:"TK",alpha3:"TKL",numeric:"772"},{alpha2:"TL",alpha3:"TLS",numeric:"626"},{alpha2:"TM",alpha3:"TKM",numeric:"795"},{alpha2:"TN",alpha3:"TUN",numeric:"788"},{alpha2:"TO",alpha3:"TON",numeric:"776"},{alpha2:"TR",alpha3:"TUR",numeric:"792"},{alpha2:"TT",alpha3:"TTO",numeric:"780"},{alpha2:"TV",alpha3:"TUV",numeric:"798"},{alpha2:"TW",alpha3:"TWN",numeric:"158"},{alpha2:"TZ",alpha3:"TZA",numeric:"834"},{alpha2:"UA",alpha3:"UKR",numeric:"804"},{alpha2:"UG",alpha3:"UGA",numeric:"800"},{alpha2:"UM",alpha3:"UMI",numeric:"581"},{alpha2:"US",alpha3:"USA",numeric:"840"},{alpha2:"UY",alpha3:"URY",numeric:"858"},{alpha2:"UZ",alpha3:"UZB",numeric:"860"},{alpha2:"VA",alpha3:"VAT",numeric:"336"},{alpha2:"VC",alpha3:"VCT",numeric:"670"},{alpha2:"VE",alpha3:"VEN",numeric:"862"},{alpha2:"VG",alpha3:"VGB",numeric:"092"},{alpha2:"VI",alpha3:"VIR",numeric:"850"},{alpha2:"VN",alpha3:"VNM",numeric:"704"},{alpha2:"VU",alpha3:"VUT",numeric:"548"},{alpha2:"WF",alpha3:"WLF",numeric:"876"},{alpha2:"WS",alpha3:"WSM",numeric:"882"},{alpha2:"YE",alpha3:"YEM",numeric:"887"},{alpha2:"YT",alpha3:"MYT",numeric:"175"},{alpha2:"ZA",alpha3:"ZAF",numeric:"710"},{alpha2:"ZM",alpha3:"ZMB",numeric:"894"},{alpha2:"ZW",alpha3:"ZWE",numeric:"716"}],b6e={country_code:g6e,time_zone:Ste},y6e=b6e,v6e={title:"Base",code:"base"},S6e=v6e,E6e=["/Applications","/bin","/boot","/boot/defaults","/dev","/etc","/etc/defaults","/etc/mail","/etc/namedb","/etc/periodic","/etc/ppp","/home","/home/user","/home/user/dir","/lib","/Library","/lost+found","/media","/mnt","/net","/Network","/opt","/opt/bin","/opt/include","/opt/lib","/opt/sbin","/opt/share","/private","/private/tmp","/private/var","/proc","/rescue","/root","/sbin","/selinux","/srv","/sys","/System","/tmp","/Users","/usr","/usr/X11R6","/usr/bin","/usr/include","/usr/lib","/usr/libdata","/usr/libexec","/usr/local/bin","/usr/local/src","/usr/obj","/usr/ports","/usr/sbin","/usr/share","/usr/src","/var","/var/log","/var/mail","/var/spool","/var/tmp","/var/yp"],x6e={"application/epub+zip":{extensions:["epub"]},"application/gzip":{extensions:["gz"]},"application/java-archive":{extensions:["jar","war","ear"]},"application/json":{extensions:["json","map"]},"application/ld+json":{extensions:["jsonld"]},"application/msword":{extensions:["doc","dot"]},"application/octet-stream":{extensions:["bin","dms","lrf","mar","so","dist","distz","pkg","bpk","dump","elc","deploy","exe","dll","deb","dmg","iso","img","msi","msp","msm","buffer"]},"application/ogg":{extensions:["ogx"]},"application/pdf":{extensions:["pdf"]},"application/rtf":{extensions:["rtf"]},"application/vnd.amazon.ebook":{extensions:["azw"]},"application/vnd.apple.installer+xml":{extensions:["mpkg"]},"application/vnd.mozilla.xul+xml":{extensions:["xul"]},"application/vnd.ms-excel":{extensions:["xls","xlm","xla","xlc","xlt","xlw"]},"application/vnd.ms-fontobject":{extensions:["eot"]},"application/vnd.ms-powerpoint":{extensions:["ppt","pps","pot"]},"application/vnd.oasis.opendocument.presentation":{extensions:["odp"]},"application/vnd.oasis.opendocument.spreadsheet":{extensions:["ods"]},"application/vnd.oasis.opendocument.text":{extensions:["odt"]},"application/vnd.openxmlformats-officedocument.presentationml.presentation":{extensions:["pptx"]},"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet":{extensions:["xlsx"]},"application/vnd.openxmlformats-officedocument.wordprocessingml.document":{extensions:["docx"]},"application/vnd.rar":{extensions:["rar"]},"application/vnd.visio":{extensions:["vsd","vst","vss","vsw"]},"application/x-7z-compressed":{extensions:["7z"]},"application/x-abiword":{extensions:["abw"]},"application/x-bzip":{extensions:["bz"]},"application/x-bzip2":{extensions:["bz2","boz"]},"application/x-csh":{extensions:["csh"]},"application/x-freearc":{extensions:["arc"]},"application/x-httpd-php":{extensions:["php"]},"application/x-sh":{extensions:["sh"]},"application/x-tar":{extensions:["tar"]},"application/xhtml+xml":{extensions:["xhtml","xht"]},"application/xml":{extensions:["xml","xsl","xsd","rng"]},"application/zip":{extensions:["zip"]},"audio/3gpp":{extensions:["3gpp"]},"audio/3gpp2":{extensions:["3g2"]},"audio/aac":{extensions:["aac"]},"audio/midi":{extensions:["mid","midi","kar","rmi"]},"audio/mpeg":{extensions:["mpga","mp2","mp2a","mp3","m2a","m3a"]},"audio/ogg":{extensions:["oga","ogg","spx","opus"]},"audio/opus":{extensions:["opus"]},"audio/wav":{extensions:["wav"]},"audio/webm":{extensions:["weba"]},"font/otf":{extensions:["otf"]},"font/ttf":{extensions:["ttf"]},"font/woff":{extensions:["woff"]},"font/woff2":{extensions:["woff2"]},"image/avif":{extensions:["avif"]},"image/bmp":{extensions:["bmp"]},"image/gif":{extensions:["gif"]},"image/jpeg":{extensions:["jpeg","jpg","jpe"]},"image/png":{extensions:["png"]},"image/svg+xml":{extensions:["svg","svgz"]},"image/tiff":{extensions:["tif","tiff"]},"image/vnd.microsoft.icon":{extensions:["ico"]},"image/webp":{extensions:["webp"]},"text/calendar":{extensions:["ics","ifb"]},"text/css":{extensions:["css"]},"text/csv":{extensions:["csv"]},"text/html":{extensions:["html","htm","shtml"]},"text/javascript":{extensions:["js","mjs"]},"text/plain":{extensions:["txt","text","conf","def","list","log","in","ini"]},"video/3gpp":{extensions:["3gp","3gpp"]},"video/3gpp2":{extensions:["3g2"]},"video/mp2t":{extensions:["ts"]},"video/mp4":{extensions:["mp4","mp4v","mpg4"]},"video/mpeg":{extensions:["mpeg","mpg","mpe","m1v","m2v"]},"video/ogg":{extensions:["ogv"]},"video/webm":{extensions:["webm"]},"video/x-msvideo":{extensions:["avi"]}},w6e={directory_path:E6e,mime_type:x6e},k6e=w6e,C6e={color:QRe,database:a6e,date:o6e,hacker:u6e,internet:m6e,location:y6e,metadata:S6e,system:k6e},T6e=C6e,A6e=new vte({locale:[rte,T6e]}),Hw,RB;function F6e(){return RB||(RB=1,Hw=function(t){return t!==null&&typeof t=="function"&&typeof t.prototype=="object"&&typeof t.prototype.addUndirectedEdgeWithKey=="function"&&typeof t.prototype.dropNode=="function"}),Hw}var Vo={},Uw,DB;function _6e(){if(DB)return Uw;DB=1;var e=ho();return Uw=function(r){if(!e(r))throw new Error("graphology-metrics/simple-size: the given graph is not a valid graphology instance.");if(!r.multi)return r.size;var n=0,a=0;function i(){n++}function o(){a++}return r.forEachNode(function(s){r.type!=="directed"&&r.forEachUndirectedNeighbor(s,i),r.type!=="undirected"&&r.forEachOutNeighbor(s,o)}),n/2+a},Uw}var NB;function R6e(){if(NB)return Vo;NB=1;var e=ho(),t=_6e();function r(o,s){return 2*s/(o*(o-1))}function n(o,s){return s/(o*(o-1))}function a(o,s){var l=o*(o-1);return s/(l+l/2)}function i(o,s,l){var u,d;if(arguments.length>3){if(u=l,d=arguments[3],typeof u!="number"||u<0)throw new Error("graphology-metrics/density: given order is not a valid number.");if(typeof d!="number"||d<0)throw new Error("graphology-metrics/density: given size is not a valid number.")}else{if(!e(l))throw new Error("graphology-metrics/density: given graph is not a valid graphology instance.");u=l.order,d=l.size,l.multi&&s===!1&&(d=t(l))}if(u<2)return 0;o===null&&(o=l.type),s===null&&(s=l.multi);var p;return o==="undirected"?p=r:o==="directed"?p=n:p=a,p(u,d)}return Vo.abstractDensity=i,Vo.density=i.bind(null,null,null),Vo.directedDensity=i.bind(null,"directed",!1),Vo.undirectedDensity=i.bind(null,"undirected",!1),Vo.mixedDensity=i.bind(null,"mixed",!1),Vo.multiDirectedDensity=i.bind(null,"directed",!0),Vo.multiUndirectedDensity=i.bind(null,"undirected",!0),Vo.multiMixedDensity=i.bind(null,"mixed",!0),Vo}var $w,LB;function D6e(){if(LB)return $w;LB=1;var e=F6e(),t=R6e().abstractDensity;function r(a,i){if(!e(a))throw new Error("graphology-generators/random/erdos-renyi: invalid Graph constructor.");var o=i.order,s=i.probability,l=i.rng||Math.random,u=new a;if(typeof i.approximateSize=="number"&&(s=t(u.type,!1,o,i.approximateSize)),typeof o!="number"||o<=0)throw new Error("graphology-generators/random/erdos-renyi: invalid `order`. Should be a positive number.");if(typeof s!="number"||s<0||s>1)throw new Error("graphology-generators/random/erdos-renyi: invalid `probability`. Should be a number between 0 and 1. Or maybe you gave an `approximateSize` exceeding the graph's density.");if(typeof l!="function")throw new Error("graphology-generators/random/erdos-renyi: invalid `rng`. Should be a function.");var d,p;for(d=0;d<o;d++)u.addNode(d);if(s<=0)return u;for(d=0;d<o;d++)for(p=d+1;p<o;p++)u.type!=="directed"&&l()<s&&u.addUndirectedEdge(d,p),u.type!=="undirected"&&(l()<s&&u.addDirectedEdge(d,p),l()<s&&u.addDirectedEdge(p,d));return u}function n(a,i){if(!e(a))throw new Error("graphology-generators/random/erdos-renyi: invalid Graph constructor.");var o=i.order,s=i.probability,l=i.rng||Math.random,u=new a;if(typeof i.approximateSize=="number"&&(s=t(u.type,!1,o,i.approximateSize)),typeof o!="number"||o<=0)throw new Error("graphology-generators/random/erdos-renyi: invalid `order`. Should be a positive number.");if(typeof s!="number"||s<0||s>1)throw new Error("graphology-generators/random/erdos-renyi: invalid `probability`. Should be a number between 0 and 1. Or maybe you gave an `approximateSize` exceeding the graph's density.");if(typeof l!="function")throw new Error("graphology-generators/random/erdos-renyi: invalid `rng`. Should be a function.");for(var d=0;d<o;d++)u.addNode(d);if(s<=0)return u;var p=-1,f=Math.log(1-s),m,b;if(u.type!=="undirected")for(b=0;b<o;){for(m=Math.log(1-l()),p+=1+(m/f|0),b===p&&p++;b<o&&o<=p;)p-=o,b++,b===p&&p++;b<o&&u.addDirectedEdge(b,p)}if(p=-1,u.type!=="directed")for(b=1;b<o;){for(m=Math.log(1-l()),p+=1+(m/f|0);p>=b&&b<o;)p-=b,b++;b<o&&u.addUndirectedEdge(b,p)}return u}return r.sparse=n,$w=r,$w}var N6e=D6e();const L6e=jn(N6e);var F1={exports:{}},M6e=F1.exports,MB;function I6e(){return MB||(MB=1,(function(e){(function(t,r,n){function a(l){var u=this,d=s();u.next=function(){var p=2091639*u.s0+u.c*23283064365386963e-26;return u.s0=u.s1,u.s1=u.s2,u.s2=p-(u.c=p|0)},u.c=1,u.s0=d(" "),u.s1=d(" "),u.s2=d(" "),u.s0-=d(l),u.s0<0&&(u.s0+=1),u.s1-=d(l),u.s1<0&&(u.s1+=1),u.s2-=d(l),u.s2<0&&(u.s2+=1),d=null}function i(l,u){return u.c=l.c,u.s0=l.s0,u.s1=l.s1,u.s2=l.s2,u}function o(l,u){var d=new a(l),p=u&&u.state,f=d.next;return f.int32=function(){return d.next()*4294967296|0},f.double=function(){return f()+(f()*2097152|0)*11102230246251565e-32},f.quick=f,p&&(typeof p=="object"&&i(p,d),f.state=function(){return i(d,{})}),f}function s(){var l=4022871197,u=function(d){d=String(d);for(var p=0;p<d.length;p++){l+=d.charCodeAt(p);var f=.02519603282416938*l;l=f>>>0,f-=l,f*=l,l=f>>>0,f-=l,l+=f*4294967296}return(l>>>0)*23283064365386963e-26};return u}r&&r.exports?r.exports=o:this.alea=o})(M6e,e)})(F1)),F1.exports}var _1={exports:{}},O6e=_1.exports,IB;function B6e(){return IB||(IB=1,(function(e){(function(t,r,n){function a(s){var l=this,u="";l.x=0,l.y=0,l.z=0,l.w=0,l.next=function(){var p=l.x^l.x<<11;return l.x=l.y,l.y=l.z,l.z=l.w,l.w^=l.w>>>19^p^p>>>8},s===(s|0)?l.x=s:u+=s;for(var d=0;d<u.length+64;d++)l.x^=u.charCodeAt(d)|0,l.next()}function i(s,l){return l.x=s.x,l.y=s.y,l.z=s.z,l.w=s.w,l}function o(s,l){var u=new a(s),d=l&&l.state,p=function(){return(u.next()>>>0)/4294967296};return p.double=function(){do var f=u.next()>>>11,m=(u.next()>>>0)/4294967296,b=(f+m)/(1<<21);while(b===0);return b},p.int32=u.next,p.quick=p,d&&(typeof d=="object"&&i(d,u),p.state=function(){return i(u,{})}),p}r&&r.exports?r.exports=o:this.xor128=o})(O6e,e)})(_1)),_1.exports}var R1={exports:{}},P6e=R1.exports,OB;function z6e(){return OB||(OB=1,(function(e){(function(t,r,n){function a(s){var l=this,u="";l.next=function(){var p=l.x^l.x>>>2;return l.x=l.y,l.y=l.z,l.z=l.w,l.w=l.v,(l.d=l.d+362437|0)+(l.v=l.v^l.v<<4^(p^p<<1))|0},l.x=0,l.y=0,l.z=0,l.w=0,l.v=0,s===(s|0)?l.x=s:u+=s;for(var d=0;d<u.length+64;d++)l.x^=u.charCodeAt(d)|0,d==u.length&&(l.d=l.x<<10^l.x>>>4),l.next()}function i(s,l){return l.x=s.x,l.y=s.y,l.z=s.z,l.w=s.w,l.v=s.v,l.d=s.d,l}function o(s,l){var u=new a(s),d=l&&l.state,p=function(){return(u.next()>>>0)/4294967296};return p.double=function(){do var f=u.next()>>>11,m=(u.next()>>>0)/4294967296,b=(f+m)/(1<<21);while(b===0);return b},p.int32=u.next,p.quick=p,d&&(typeof d=="object"&&i(d,u),p.state=function(){return i(u,{})}),p}r&&r.exports?r.exports=o:this.xorwow=o})(P6e,e)})(R1)),R1.exports}var D1={exports:{}},H6e=D1.exports,BB;function U6e(){return BB||(BB=1,(function(e){(function(t,r,n){function a(s){var l=this;l.next=function(){var d=l.x,p=l.i,f,m;return f=d[p],f^=f>>>7,m=f^f<<24,f=d[p+1&7],m^=f^f>>>10,f=d[p+3&7],m^=f^f>>>3,f=d[p+4&7],m^=f^f<<7,f=d[p+7&7],f=f^f<<13,m^=f^f<<9,d[p]=m,l.i=p+1&7,m};function u(d,p){var f,m=[];if(p===(p|0))m[0]=p;else for(p=""+p,f=0;f<p.length;++f)m[f&7]=m[f&7]<<15^p.charCodeAt(f)+m[f+1&7]<<13;for(;m.length<8;)m.push(0);for(f=0;f<8&&m[f]===0;++f);for(f==8?m[7]=-1:m[f],d.x=m,d.i=0,f=256;f>0;--f)d.next()}u(l,s)}function i(s,l){return l.x=s.x.slice(),l.i=s.i,l}function o(s,l){s==null&&(s=+new Date);var u=new a(s),d=l&&l.state,p=function(){return(u.next()>>>0)/4294967296};return p.double=function(){do var f=u.next()>>>11,m=(u.next()>>>0)/4294967296,b=(f+m)/(1<<21);while(b===0);return b},p.int32=u.next,p.quick=p,d&&(d.x&&i(d,u),p.state=function(){return i(u,{})}),p}r&&r.exports?r.exports=o:this.xorshift7=o})(H6e,e)})(D1)),D1.exports}var N1={exports:{}},$6e=N1.exports,PB;function G6e(){return PB||(PB=1,(function(e){(function(t,r,n){function a(s){var l=this;l.next=function(){var d=l.w,p=l.X,f=l.i,m,b;return l.w=d=d+1640531527|0,b=p[f+34&127],m=p[f=f+1&127],b^=b<<13,m^=m<<17,b^=b>>>15,m^=m>>>12,b=p[f]=b^m,l.i=f,b+(d^d>>>16)|0};function u(d,p){var f,m,b,y,S,w=[],E=128;for(p===(p|0)?(m=p,p=null):(p=p+"\0",m=0,E=Math.max(E,p.length)),b=0,y=-32;y<E;++y)p&&(m^=p.charCodeAt((y+32)%p.length)),y===0&&(S=m),m^=m<<10,m^=m>>>15,m^=m<<4,m^=m>>>13,y>=0&&(S=S+1640531527|0,f=w[y&127]^=m+S,b=f==0?b+1:0);for(b>=128&&(w[(p&&p.length||0)&127]=-1),b=127,y=512;y>0;--y)m=w[b+34&127],f=w[b=b+1&127],m^=m<<13,f^=f<<17,m^=m>>>15,f^=f>>>12,w[b]=m^f;d.w=S,d.X=w,d.i=b}u(l,s)}function i(s,l){return l.i=s.i,l.w=s.w,l.X=s.X.slice(),l}function o(s,l){s==null&&(s=+new Date);var u=new a(s),d=l&&l.state,p=function(){return(u.next()>>>0)/4294967296};return p.double=function(){do var f=u.next()>>>11,m=(u.next()>>>0)/4294967296,b=(f+m)/(1<<21);while(b===0);return b},p.int32=u.next,p.quick=p,d&&(d.X&&i(d,u),p.state=function(){return i(u,{})}),p}r&&r.exports?r.exports=o:this.xor4096=o})($6e,e)})(N1)),N1.exports}var L1={exports:{}},j6e=L1.exports,zB;function q6e(){return zB||(zB=1,(function(e){(function(t,r,n){function a(s){var l=this,u="";l.next=function(){var p=l.b,f=l.c,m=l.d,b=l.a;return p=p<<25^p>>>7^f,f=f-m|0,m=m<<24^m>>>8^b,b=b-p|0,l.b=p=p<<20^p>>>12^f,l.c=f=f-m|0,l.d=m<<16^f>>>16^b,l.a=b-p|0},l.a=0,l.b=0,l.c=-1640531527,l.d=1367130551,s===Math.floor(s)?(l.a=s/4294967296|0,l.b=s|0):u+=s;for(var d=0;d<u.length+20;d++)l.b^=u.charCodeAt(d)|0,l.next()}function i(s,l){return l.a=s.a,l.b=s.b,l.c=s.c,l.d=s.d,l}function o(s,l){var u=new a(s),d=l&&l.state,p=function(){return(u.next()>>>0)/4294967296};return p.double=function(){do var f=u.next()>>>11,m=(u.next()>>>0)/4294967296,b=(f+m)/(1<<21);while(b===0);return b},p.int32=u.next,p.quick=p,d&&(typeof d=="object"&&i(d,u),p.state=function(){return i(u,{})}),p}r&&r.exports?r.exports=o:this.tychei=o})(j6e,e)})(L1)),L1.exports}var M1={exports:{}};const W6e={},V6e=Object.freeze(Object.defineProperty({__proto__:null,default:W6e},Symbol.toStringTag,{value:"Module"})),K6e=epe(V6e);var Y6e=M1.exports,HB;function J6e(){return HB||(HB=1,(function(e){(function(t,r,n){var a=256,i=6,o=52,s="random",l=n.pow(a,i),u=n.pow(2,o),d=u*2,p=a-1,f;function m(_,R,C){var F=[];R=R==!0?{entropy:!0}:R||{};var L=w(S(R.entropy?[_,k(r)]:_??E(),3),F),I=new b(F),M=function(){for(var j=I.g(i),Y=l,W=0;j<u;)j=(j+W)*a,Y*=a,W=I.g(1);for(;j>=d;)j/=2,Y/=2,W>>>=1;return(j+W)/Y};return M.int32=function(){return I.g(4)|0},M.quick=function(){return I.g(4)/4294967296},M.double=M,w(k(I.S),r),(R.pass||C||function(j,Y,W,$){return $&&($.S&&y($,I),j.state=function(){return y(I,{})}),W?(n[s]=j,Y):j})(M,L,"global"in R?R.global:this==n,R.state)}function b(_){var R,C=_.length,F=this,L=0,I=F.i=F.j=0,M=F.S=[];for(C||(_=[C++]);L<a;)M[L]=L++;for(L=0;L<a;L++)M[L]=M[I=p&I+_[L%C]+(R=M[L])],M[I]=R;(F.g=function(j){for(var Y,W=0,$=F.i,X=F.j,z=F.S;j--;)Y=z[$=p&$+1],W=W*a+z[p&(z[$]=z[X=p&X+Y])+(z[X]=Y)];return F.i=$,F.j=X,W})(a)}function y(_,R){return R.i=_.i,R.j=_.j,R.S=_.S.slice(),R}function S(_,R){var C=[],F=typeof _,L;if(R&&F=="object")for(L in _)try{C.push(S(_[L],R-1))}catch{}return C.length?C:F=="string"?_:_+"\0"}function w(_,R){for(var C=_+"",F,L=0;L<C.length;)R[p&L]=p&(F^=R[p&L]*19)+C.charCodeAt(L++);return k(R)}function E(){try{var _;return f&&(_=f.randomBytes)?_=_(a):(_=new Uint8Array(a),(t.crypto||t.msCrypto).getRandomValues(_)),k(_)}catch{var R=t.navigator,C=R&&R.plugins;return[+new Date,t,C,t.screen,k(r)]}}function k(_){return String.fromCharCode.apply(0,_)}if(w(n.random(),r),e.exports){e.exports=m;try{f=K6e}catch{}}else n["seed"+s]=m})(typeof self<"u"?self:Y6e,[],Math)})(M1)),M1.exports}var Gw,UB;function X6e(){if(UB)return Gw;UB=1;var e=I6e(),t=B6e(),r=z6e(),n=U6e(),a=G6e(),i=q6e(),o=J6e();return o.alea=e,o.xor128=t,o.xorwow=r,o.xorshift7=n,o.xor4096=a,o.tychei=i,Gw=o,Gw}var Z6e=X6e();const xy=jn(Z6e),Q6e=()=>{const[e,t]=T.useState(A6e);T.useEffect(()=>{const a=new URLSearchParams(document.location.search).get("seed");if(a){xy(a,{global:!0});const i=new vte({locale:rte});i.seed(Math.random()),t(i)}},[]);const r=T.useCallback(()=>{pt.getState().reset();const n=L6e(Gh,{order:100,probability:.1});return n.nodes().forEach(a=>{n.mergeNodeAttributes(a,{label:e.person.fullName(),size:e.number.int({min:gu,max:uy}),color:Ff(),x:Math.random(),y:Math.random(),borderColor:Ff(),borderSize:e.number.float({min:0,max:1,multipleOf:.1}),pictoColor:Ff(),image:e.image.urlLoremFlickr()})}),n.edges().forEach(a=>{n.mergeEdgeAttributes(a,{label:e.lorem.words(e.number.int({min:1,max:3})),size:e.number.float({min:1,max:5}),color:Ff()})}),n},[e]);return{faker:e,randomColor:Ff,randomGraph:r}},lu=({checked:e,onCheckedChange:t,label:r})=>{const n=`checkbox-${r.toLowerCase().replace(/\s+/g,"-")}`;return x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx(ol,{id:n,checked:e,onCheckedChange:t}),x.jsx("label",{htmlFor:n,className:"text-sm leading-none font-medium peer-disabled:cursor-not-allowed peer-disabled:opacity-70",children:r})]})},jw=({value:e,onEditFinished:t,label:r,min:n,max:a,defaultValue:i})=>{const{t:o}=Sr(),[s,l]=T.useState(e),u=`input-${r.toLowerCase().replace(/\s+/g,"-")}`;T.useEffect(()=>{l(e)},[e]);const d=T.useCallback(m=>{const b=m.target.value.trim();if(b.length===0){l(null);return}const y=Number.parseInt(b);if(!isNaN(y)&&y!==s){if(n!==void 0&&y<n||a!==void 0&&y>a)return;l(y)}},[s,n,a]),p=T.useCallback(()=>{s!==null&&e!==s&&t(s)},[e,s,t]),f=T.useCallback(()=>{i!==void 0&&e!==i&&(l(i),t(i))},[i,e,t]);return x.jsxs("div",{className:"flex flex-col gap-2",children:[x.jsx("label",{htmlFor:u,className:"text-sm leading-none font-medium peer-disabled:cursor-not-allowed peer-disabled:opacity-70",children:r}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{id:u,type:"number",value:s===null?"":s,onChange:d,className:"h-6 w-full min-w-0 pr-1",min:n,max:a,onBlur:p,onKeyDown:m=>{m.key==="Enter"&&p()}}),i!==void 0&&x.jsx(Rt,{variant:"ghost",size:"icon",className:"h-6 w-6 flex-shrink-0 hover:bg-muted text-muted-foreground hover:text-foreground",onClick:f,type:"button",title:o("graphPanel.sideBar.settings.resetToDefault"),children:x.jsx(cZ,{className:"h-3.5 w-3.5"})})]})]})};function eDe(){const[e,t]=T.useState(!1),r=Ve.use.showPropertyPanel(),n=Ve.use.showNodeSearchBar(),a=Ve.use.showNodeLabel(),i=Ve.use.enableEdgeEvents(),o=Ve.use.enableNodeDrag(),s=Ve.use.enableHideUnselectedEdges(),l=Ve.use.showEdgeLabel(),u=Ve.use.minEdgeSize(),d=Ve.use.maxEdgeSize(),p=Ve.use.graphQueryMaxDepth(),f=Ve.use.graphMaxNodes(),m=Ve.use.backendMaxGraphNodes(),b=Ve.use.graphLayoutMaxIterations(),y=Ve.use.enableHealthCheck(),{randomGraph:S}=Q6e(),w=T.useCallback(()=>Ve.setState($=>({enableNodeDrag:!$.enableNodeDrag})),[]),E=T.useCallback(()=>Ve.setState($=>({enableEdgeEvents:!$.enableEdgeEvents})),[]),k=T.useCallback(()=>Ve.setState($=>({enableHideUnselectedEdges:!$.enableHideUnselectedEdges})),[]),_=T.useCallback(()=>Ve.setState($=>({showEdgeLabel:!$.showEdgeLabel})),[]),R=T.useCallback(()=>Ve.setState($=>({showPropertyPanel:!$.showPropertyPanel})),[]),C=T.useCallback(()=>Ve.setState($=>({showNodeSearchBar:!$.showNodeSearchBar})),[]),F=T.useCallback(()=>Ve.setState($=>({showNodeLabel:!$.showNodeLabel})),[]),L=T.useCallback(()=>Ve.setState($=>({enableHealthCheck:!$.enableHealthCheck})),[]),I=T.useCallback($=>{if($<1)return;Ve.setState({graphQueryMaxDepth:$});const X=Ve.getState().queryLabel;Ve.getState().setQueryLabel(""),setTimeout(()=>{Ve.getState().setQueryLabel(X)},300)},[]),M=T.useCallback($=>{const X=m||1e3;$<1||$>X||Ve.getState().setGraphMaxNodes($,!0)},[m]),j=T.useCallback($=>{$<1||Ve.setState({graphLayoutMaxIterations:$})},[]);T.useCallback(()=>{const $=S();pt.getState().setSigmaGraph($)},[S]);const{t:Y}=Sr(),W=()=>t(!1);return x.jsx(x.Fragment,{children:x.jsxs(t2,{open:e,onOpenChange:t,children:[x.jsx(r2,{asChild:!0,children:x.jsx(Rt,{variant:no,tooltip:Y("graphPanel.sideBar.settings.settings"),size:"icon",children:x.jsx(KEe,{})})}),x.jsx(kg,{side:"right",align:"end",sideOffset:8,collisionPadding:5,className:"p-2 max-w-[200px]",onCloseAutoFocus:$=>$.preventDefault(),children:x.jsxs("div",{className:"flex flex-col gap-2",children:[x.jsx(lu,{checked:y,onCheckedChange:L,label:Y("graphPanel.sideBar.settings.healthCheck")}),x.jsx(im,{}),x.jsx(lu,{checked:r,onCheckedChange:R,label:Y("graphPanel.sideBar.settings.showPropertyPanel")}),x.jsx(lu,{checked:n,onCheckedChange:C,label:Y("graphPanel.sideBar.settings.showSearchBar")}),x.jsx(im,{}),x.jsx(lu,{checked:a,onCheckedChange:F,label:Y("graphPanel.sideBar.settings.showNodeLabel")}),x.jsx(lu,{checked:o,onCheckedChange:w,label:Y("graphPanel.sideBar.settings.nodeDraggable")}),x.jsx(im,{}),x.jsx(lu,{checked:l,onCheckedChange:_,label:Y("graphPanel.sideBar.settings.showEdgeLabel")}),x.jsx(lu,{checked:s,onCheckedChange:k,label:Y("graphPanel.sideBar.settings.hideUnselectedEdges")}),x.jsx(lu,{checked:i,onCheckedChange:E,label:Y("graphPanel.sideBar.settings.edgeEvents")}),x.jsxs("div",{className:"flex flex-col gap-2",children:[x.jsx("label",{htmlFor:"edge-size-min",className:"text-sm leading-none font-medium peer-disabled:cursor-not-allowed peer-disabled:opacity-70",children:Y("graphPanel.sideBar.settings.edgeSizeRange")}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx(oa,{id:"edge-size-min",type:"number",value:u,onChange:$=>{const X=Number($.target.value);!isNaN(X)&&X>=1&&X<=d&&Ve.setState({minEdgeSize:X})},className:"h-6 w-16 min-w-0 pr-1",min:1,max:Math.min(d,10)}),x.jsx("span",{children:"-"}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{id:"edge-size-max",type:"number",value:d,onChange:$=>{const X=Number($.target.value);!isNaN(X)&&X>=u&&X>=1&&X<=10&&Ve.setState({maxEdgeSize:X})},className:"h-6 w-16 min-w-0 pr-1",min:u,max:10}),x.jsx(Rt,{variant:"ghost",size:"icon",className:"h-6 w-6 flex-shrink-0 hover:bg-muted text-muted-foreground hover:text-foreground",onClick:()=>Ve.setState({minEdgeSize:1,maxEdgeSize:5}),type:"button",title:Y("graphPanel.sideBar.settings.resetToDefault"),children:x.jsx(cZ,{className:"h-3.5 w-3.5"})})]})]})]}),x.jsx(im,{}),x.jsx(jw,{label:Y("graphPanel.sideBar.settings.maxQueryDepth"),min:1,value:p,defaultValue:3,onEditFinished:I}),x.jsx(jw,{label:`${Y("graphPanel.sideBar.settings.maxNodes")} (≤ ${m||1e3})`,min:1,max:m||1e3,value:f,defaultValue:m||1e3,onEditFinished:M}),x.jsx(jw,{label:Y("graphPanel.sideBar.settings.maxLayoutIterations"),min:1,max:30,value:b,defaultValue:15,onEditFinished:j}),!1,x.jsx(Rt,{onClick:W,variant:"outline",size:"sm",className:"ml-auto px-4",children:Y("graphPanel.sideBar.settings.save")})]})})]})})}const tDe="ENTRIES",Ete="KEYS",xte="VALUES",ya="";class qw{constructor(t,r){const n=t._tree,a=Array.from(n.keys());this.set=t,this._type=r,this._path=a.length>0?[{node:n,keys:a}]:[]}next(){const t=this.dive();return this.backtrack(),t}dive(){if(this._path.length===0)return{done:!0,value:void 0};const{node:t,keys:r}=rh(this._path);if(rh(r)===ya)return{done:!1,value:this.result()};const n=t.get(rh(r));return this._path.push({node:n,keys:Array.from(n.keys())}),this.dive()}backtrack(){if(this._path.length===0)return;const t=rh(this._path).keys;t.pop(),!(t.length>0)&&(this._path.pop(),this.backtrack())}key(){return this.set._prefix+this._path.map(({keys:t})=>rh(t)).filter(t=>t!==ya).join("")}value(){return rh(this._path).node.get(ya)}result(){switch(this._type){case xte:return this.value();case Ete:return this.key();default:return[this.key(),this.value()]}}[Symbol.iterator](){return this}}const rh=e=>e[e.length-1],rDe=(e,t,r)=>{const n=new Map;if(t===void 0)return n;const a=t.length+1,i=a+r,o=new Uint8Array(i*a).fill(r+1);for(let s=0;s<a;++s)o[s]=s;for(let s=1;s<i;++s)o[s*a]=s;return wte(e,t,r,n,o,1,a,""),n},wte=(e,t,r,n,a,i,o,s)=>{const l=i*o;e:for(const u of e.keys())if(u===ya){const d=a[l-1];d<=r&&n.set(s,[e.get(u),d])}else{let d=i;for(let p=0;p<u.length;++p,++d){const f=u[p],m=o*d,b=m-o;let y=a[m];const S=Math.max(0,d-r-1),w=Math.min(o-1,d+r);for(let E=S;E<w;++E){const k=f!==t[E],_=a[b+E]+ +k,R=a[b+E+1]+1,C=a[m+E]+1,F=a[m+E+1]=Math.min(_,R,C);F<y&&(y=F)}if(y>r)continue e}wte(e.get(u),t,r,n,a,d,o,s+u)}};class vu{constructor(t=new Map,r=""){this._size=void 0,this._tree=t,this._prefix=r}atPrefix(t){if(!t.startsWith(this._prefix))throw new Error("Mismatched prefix");const[r,n]=wy(this._tree,t.slice(this._prefix.length));if(r===void 0){const[a,i]=R5(n);for(const o of a.keys())if(o!==ya&&o.startsWith(i)){const s=new Map;return s.set(o.slice(i.length),a.get(o)),new vu(s,t)}}return new vu(r,t)}clear(){this._size=void 0,this._tree.clear()}delete(t){return this._size=void 0,nDe(this._tree,t)}entries(){return new qw(this,tDe)}forEach(t){for(const[r,n]of this)t(r,n,this)}fuzzyGet(t,r){return rDe(this._tree,t,r)}get(t){const r=TR(this._tree,t);return r!==void 0?r.get(ya):void 0}has(t){const r=TR(this._tree,t);return r!==void 0&&r.has(ya)}keys(){return new qw(this,Ete)}set(t,r){if(typeof t!="string")throw new Error("key must be a string");return this._size=void 0,Ww(this._tree,t).set(ya,r),this}get size(){if(this._size)return this._size;this._size=0;const t=this.entries();for(;!t.next().done;)this._size+=1;return this._size}update(t,r){if(typeof t!="string")throw new Error("key must be a string");this._size=void 0;const n=Ww(this._tree,t);return n.set(ya,r(n.get(ya))),this}fetch(t,r){if(typeof t!="string")throw new Error("key must be a string");this._size=void 0;const n=Ww(this._tree,t);let a=n.get(ya);return a===void 0&&n.set(ya,a=r()),a}values(){return new qw(this,xte)}[Symbol.iterator](){return this.entries()}static from(t){const r=new vu;for(const[n,a]of t)r.set(n,a);return r}static fromObject(t){return vu.from(Object.entries(t))}}const wy=(e,t,r=[])=>{if(t.length===0||e==null)return[e,r];for(const n of e.keys())if(n!==ya&&t.startsWith(n))return r.push([e,n]),wy(e.get(n),t.slice(n.length),r);return r.push([e,t]),wy(void 0,"",r)},TR=(e,t)=>{if(t.length===0||e==null)return e;for(const r of e.keys())if(r!==ya&&t.startsWith(r))return TR(e.get(r),t.slice(r.length))},Ww=(e,t)=>{const r=t.length;e:for(let n=0;e&&n<r;){for(const i of e.keys())if(i!==ya&&t[n]===i[0]){const o=Math.min(r-n,i.length);let s=1;for(;s<o&&t[n+s]===i[s];)++s;const l=e.get(i);if(s===i.length)e=l;else{const u=new Map;u.set(i.slice(s),l),e.set(t.slice(n,n+s),u),e.delete(i),e=u}n+=s;continue e}const a=new Map;return e.set(t.slice(n),a),a}return e},nDe=(e,t)=>{const[r,n]=wy(e,t);if(r!==void 0){if(r.delete(ya),r.size===0)kte(n);else if(r.size===1){const[a,i]=r.entries().next().value;Cte(n,a,i)}}},kte=e=>{if(e.length===0)return;const[t,r]=R5(e);if(t.delete(r),t.size===0)kte(e.slice(0,-1));else if(t.size===1){const[n,a]=t.entries().next().value;n!==ya&&Cte(e.slice(0,-1),n,a)}},Cte=(e,t,r)=>{if(e.length===0)return;const[n,a]=R5(e);n.set(a+t,r),n.delete(a)},R5=e=>e[e.length-1],D5="or",Tte="and",aDe="and_not";class Rc{constructor(t){if((t==null?void 0:t.fields)==null)throw new Error('MiniSearch: option "fields" must be provided');const r=t.autoVacuum==null||t.autoVacuum===!0?Yw:t.autoVacuum;this._options={...Kw,...t,autoVacuum:r,searchOptions:{...$B,...t.searchOptions||{}},autoSuggestOptions:{...uDe,...t.autoSuggestOptions||{}}},this._index=new vu,this._documentCount=0,this._documentIds=new Map,this._idToShortId=new Map,this._fieldIds={},this._fieldLength=new Map,this._avgFieldLength=[],this._nextId=0,this._storedFields=new Map,this._dirtCount=0,this._currentVacuum=null,this._enqueuedVacuum=null,this._enqueuedVacuumConditions=FR,this.addFields(this._options.fields)}add(t){const{extractField:r,stringifyField:n,tokenize:a,processTerm:i,fields:o,idField:s}=this._options,l=r(t,s);if(l==null)throw new Error(`MiniSearch: document does not have ID field "${s}"`);if(this._idToShortId.has(l))throw new Error(`MiniSearch: duplicate ID ${l}`);const u=this.addDocumentId(l);this.saveStoredFields(u,t);for(const d of o){const p=r(t,d);if(p==null)continue;const f=a(n(p,d),d),m=this._fieldIds[d],b=new Set(f).size;this.addFieldLength(u,m,this._documentCount-1,b);for(const y of f){const S=i(y,d);if(Array.isArray(S))for(const w of S)this.addTerm(m,u,w);else S&&this.addTerm(m,u,S)}}}addAll(t){for(const r of t)this.add(r)}addAllAsync(t,r={}){const{chunkSize:n=10}=r,a={chunk:[],promise:Promise.resolve()},{chunk:i,promise:o}=t.reduce(({chunk:s,promise:l},u,d)=>(s.push(u),(d+1)%n===0?{chunk:[],promise:l.then(()=>new Promise(p=>setTimeout(p,0))).then(()=>this.addAll(s))}:{chunk:s,promise:l}),a);return o.then(()=>this.addAll(i))}remove(t){const{tokenize:r,processTerm:n,extractField:a,stringifyField:i,fields:o,idField:s}=this._options,l=a(t,s);if(l==null)throw new Error(`MiniSearch: document does not have ID field "${s}"`);const u=this._idToShortId.get(l);if(u==null)throw new Error(`MiniSearch: cannot remove document with ID ${l}: it is not in the index`);for(const d of o){const p=a(t,d);if(p==null)continue;const f=r(i(p,d),d),m=this._fieldIds[d],b=new Set(f).size;this.removeFieldLength(u,m,this._documentCount,b);for(const y of f){const S=n(y,d);if(Array.isArray(S))for(const w of S)this.removeTerm(m,u,w);else S&&this.removeTerm(m,u,S)}}this._storedFields.delete(u),this._documentIds.delete(u),this._idToShortId.delete(l),this._fieldLength.delete(u),this._documentCount-=1}removeAll(t){if(t)for(const r of t)this.remove(r);else{if(arguments.length>0)throw new Error("Expected documents to be present. Omit the argument to remove all documents.");this._index=new vu,this._documentCount=0,this._documentIds=new Map,this._idToShortId=new Map,this._fieldLength=new Map,this._avgFieldLength=[],this._storedFields=new Map,this._nextId=0}}discard(t){const r=this._idToShortId.get(t);if(r==null)throw new Error(`MiniSearch: cannot discard document with ID ${t}: it is not in the index`);this._idToShortId.delete(t),this._documentIds.delete(r),this._storedFields.delete(r),(this._fieldLength.get(r)||[]).forEach((n,a)=>{this.removeFieldLength(r,a,this._documentCount,n)}),this._fieldLength.delete(r),this._documentCount-=1,this._dirtCount+=1,this.maybeAutoVacuum()}maybeAutoVacuum(){if(this._options.autoVacuum===!1)return;const{minDirtFactor:t,minDirtCount:r,batchSize:n,batchWait:a}=this._options.autoVacuum;this.conditionalVacuum({batchSize:n,batchWait:a},{minDirtCount:r,minDirtFactor:t})}discardAll(t){const r=this._options.autoVacuum;try{this._options.autoVacuum=!1;for(const n of t)this.discard(n)}finally{this._options.autoVacuum=r}this.maybeAutoVacuum()}replace(t){const{idField:r,extractField:n}=this._options,a=n(t,r);this.discard(a),this.add(t)}vacuum(t={}){return this.conditionalVacuum(t)}conditionalVacuum(t,r){return this._currentVacuum?(this._enqueuedVacuumConditions=this._enqueuedVacuumConditions&&r,this._enqueuedVacuum!=null?this._enqueuedVacuum:(this._enqueuedVacuum=this._currentVacuum.then(()=>{const n=this._enqueuedVacuumConditions;return this._enqueuedVacuumConditions=FR,this.performVacuuming(t,n)}),this._enqueuedVacuum)):this.vacuumConditionsMet(r)===!1?Promise.resolve():(this._currentVacuum=this.performVacuuming(t),this._currentVacuum)}async performVacuuming(t,r){const n=this._dirtCount;if(this.vacuumConditionsMet(r)){const a=t.batchSize||AR.batchSize,i=t.batchWait||AR.batchWait;let o=1;for(const[s,l]of this._index){for(const[u,d]of l)for(const[p]of d)this._documentIds.has(p)||(d.size<=1?l.delete(u):d.delete(p));this._index.get(s).size===0&&this._index.delete(s),o%a===0&&await new Promise(u=>setTimeout(u,i)),o+=1}this._dirtCount-=n}await null,this._currentVacuum=this._enqueuedVacuum,this._enqueuedVacuum=null}vacuumConditionsMet(t){if(t==null)return!0;let{minDirtCount:r,minDirtFactor:n}=t;return r=r||Yw.minDirtCount,n=n||Yw.minDirtFactor,this.dirtCount>=r&&this.dirtFactor>=n}get isVacuuming(){return this._currentVacuum!=null}get dirtCount(){return this._dirtCount}get dirtFactor(){return this._dirtCount/(1+this._documentCount+this._dirtCount)}has(t){return this._idToShortId.has(t)}getStoredFields(t){const r=this._idToShortId.get(t);if(r!=null)return this._storedFields.get(r)}search(t,r={}){const{searchOptions:n}=this._options,a={...n,...r},i=this.executeQuery(t,r),o=[];for(const[s,{score:l,terms:u,match:d}]of i){const p=u.length||1,f={id:this._documentIds.get(s),score:l*p,terms:Object.keys(d),queryTerms:u,match:d};Object.assign(f,this._storedFields.get(s)),(a.filter==null||a.filter(f))&&o.push(f)}return t===Rc.wildcard&&a.boostDocument==null||o.sort(jB),o}autoSuggest(t,r={}){r={...this._options.autoSuggestOptions,...r};const n=new Map;for(const{score:i,terms:o}of this.search(t,r)){const s=o.join(" "),l=n.get(s);l!=null?(l.score+=i,l.count+=1):n.set(s,{score:i,terms:o,count:1})}const a=[];for(const[i,{score:o,terms:s,count:l}]of n)a.push({suggestion:i,terms:s,score:o/l});return a.sort(jB),a}get documentCount(){return this._documentCount}get termCount(){return this._index.size}static loadJSON(t,r){if(r==null)throw new Error("MiniSearch: loadJSON should be given the same options used when serializing the index");return this.loadJS(JSON.parse(t),r)}static async loadJSONAsync(t,r){if(r==null)throw new Error("MiniSearch: loadJSON should be given the same options used when serializing the index");return this.loadJSAsync(JSON.parse(t),r)}static getDefault(t){if(Kw.hasOwnProperty(t))return Vw(Kw,t);throw new Error(`MiniSearch: unknown option "${t}"`)}static loadJS(t,r){const{index:n,documentIds:a,fieldLength:i,storedFields:o,serializationVersion:s}=t,l=this.instantiateMiniSearch(t,r);l._documentIds=qb(a),l._fieldLength=qb(i),l._storedFields=qb(o);for(const[u,d]of l._documentIds)l._idToShortId.set(d,u);for(const[u,d]of n){const p=new Map;for(const f of Object.keys(d)){let m=d[f];s===1&&(m=m.ds),p.set(parseInt(f,10),qb(m))}l._index.set(u,p)}return l}static async loadJSAsync(t,r){const{index:n,documentIds:a,fieldLength:i,storedFields:o,serializationVersion:s}=t,l=this.instantiateMiniSearch(t,r);l._documentIds=await Wb(a),l._fieldLength=await Wb(i),l._storedFields=await Wb(o);for(const[d,p]of l._documentIds)l._idToShortId.set(p,d);let u=0;for(const[d,p]of n){const f=new Map;for(const m of Object.keys(p)){let b=p[m];s===1&&(b=b.ds),f.set(parseInt(m,10),await Wb(b))}++u%1e3===0&&await Ate(0),l._index.set(d,f)}return l}static instantiateMiniSearch(t,r){const{documentCount:n,nextId:a,fieldIds:i,averageFieldLength:o,dirtCount:s,serializationVersion:l}=t;if(l!==1&&l!==2)throw new Error("MiniSearch: cannot deserialize an index created with an incompatible version");const u=new Rc(r);return u._documentCount=n,u._nextId=a,u._idToShortId=new Map,u._fieldIds=i,u._avgFieldLength=o,u._dirtCount=s||0,u._index=new vu,u}executeQuery(t,r={}){if(t===Rc.wildcard)return this.executeWildcardQuery(r);if(typeof t!="string"){const f={...r,...t,queries:void 0},m=t.queries.map(b=>this.executeQuery(b,f));return this.combineResults(m,f.combineWith)}const{tokenize:n,processTerm:a,searchOptions:i}=this._options,o={tokenize:n,processTerm:a,...i,...r},{tokenize:s,processTerm:l}=o,p=s(t).flatMap(f=>l(f)).filter(f=>!!f).map(lDe(o)).map(f=>this.executeQuerySpec(f,o));return this.combineResults(p,o.combineWith)}executeQuerySpec(t,r){const n={...this._options.searchOptions,...r},a=(n.fields||this._options.fields).reduce((y,S)=>({...y,[S]:Vw(n.boost,S)||1}),{}),{boostDocument:i,weights:o,maxFuzzy:s,bm25:l}=n,{fuzzy:u,prefix:d}={...$B.weights,...o},p=this._index.get(t.term),f=this.termResults(t.term,t.term,1,t.termBoost,p,a,i,l);let m,b;if(t.prefix&&(m=this._index.atPrefix(t.term)),t.fuzzy){const y=t.fuzzy===!0?.2:t.fuzzy,S=y<1?Math.min(s,Math.round(t.term.length*y)):y;S&&(b=this._index.fuzzyGet(t.term,S))}if(m)for(const[y,S]of m){const w=y.length-t.term.length;if(!w)continue;b==null||b.delete(y);const E=d*y.length/(y.length+.3*w);this.termResults(t.term,y,E,t.termBoost,S,a,i,l,f)}if(b)for(const y of b.keys()){const[S,w]=b.get(y);if(!w)continue;const E=u*y.length/(y.length+w);this.termResults(t.term,y,E,t.termBoost,S,a,i,l,f)}return f}executeWildcardQuery(t){const r=new Map,n={...this._options.searchOptions,...t};for(const[a,i]of this._documentIds){const o=n.boostDocument?n.boostDocument(i,"",this._storedFields.get(a)):1;r.set(a,{score:o,terms:[],match:{}})}return r}combineResults(t,r=D5){if(t.length===0)return new Map;const n=r.toLowerCase(),a=iDe[n];if(!a)throw new Error(`Invalid combination operator: ${r}`);return t.reduce(a)||new Map}toJSON(){const t=[];for(const[r,n]of this._index){const a={};for(const[i,o]of n)a[i]=Object.fromEntries(o);t.push([r,a])}return{documentCount:this._documentCount,nextId:this._nextId,documentIds:Object.fromEntries(this._documentIds),fieldIds:this._fieldIds,fieldLength:Object.fromEntries(this._fieldLength),averageFieldLength:this._avgFieldLength,storedFields:Object.fromEntries(this._storedFields),dirtCount:this._dirtCount,index:t,serializationVersion:2}}termResults(t,r,n,a,i,o,s,l,u=new Map){if(i==null)return u;for(const d of Object.keys(o)){const p=o[d],f=this._fieldIds[d],m=i.get(f);if(m==null)continue;let b=m.size;const y=this._avgFieldLength[f];for(const S of m.keys()){if(!this._documentIds.has(S)){this.removeTerm(f,S,r),b-=1;continue}const w=s?s(this._documentIds.get(S),r,this._storedFields.get(S)):1;if(!w)continue;const E=m.get(S),k=this._fieldLength.get(S)[f],_=sDe(E,b,this._documentCount,k,y,l),R=n*a*p*w*_,C=u.get(S);if(C){C.score+=R,cDe(C.terms,t);const F=Vw(C.match,r);F?F.push(d):C.match[r]=[d]}else u.set(S,{score:R,terms:[t],match:{[r]:[d]}})}}return u}addTerm(t,r,n){const a=this._index.fetch(n,qB);let i=a.get(t);if(i==null)i=new Map,i.set(r,1),a.set(t,i);else{const o=i.get(r);i.set(r,(o||0)+1)}}removeTerm(t,r,n){if(!this._index.has(n)){this.warnDocumentChanged(r,t,n);return}const a=this._index.fetch(n,qB),i=a.get(t);i==null||i.get(r)==null?this.warnDocumentChanged(r,t,n):i.get(r)<=1?i.size<=1?a.delete(t):i.delete(r):i.set(r,i.get(r)-1),this._index.get(n).size===0&&this._index.delete(n)}warnDocumentChanged(t,r,n){for(const a of Object.keys(this._fieldIds))if(this._fieldIds[a]===r){this._options.logger("warn",`MiniSearch: document with ID ${this._documentIds.get(t)} has changed before removal: term "${n}" was not present in field "${a}". Removing a document after it has changed can corrupt the index!`,"version_conflict");return}}addDocumentId(t){const r=this._nextId;return this._idToShortId.set(t,r),this._documentIds.set(r,t),this._documentCount+=1,this._nextId+=1,r}addFields(t){for(let r=0;r<t.length;r++)this._fieldIds[t[r]]=r}addFieldLength(t,r,n,a){let i=this._fieldLength.get(t);i==null&&this._fieldLength.set(t,i=[]),i[r]=a;const s=(this._avgFieldLength[r]||0)*n+a;this._avgFieldLength[r]=s/(n+1)}removeFieldLength(t,r,n,a){if(n===1){this._avgFieldLength[r]=0;return}const i=this._avgFieldLength[r]*n-a;this._avgFieldLength[r]=i/(n-1)}saveStoredFields(t,r){const{storeFields:n,extractField:a}=this._options;if(n==null||n.length===0)return;let i=this._storedFields.get(t);i==null&&this._storedFields.set(t,i={});for(const o of n){const s=a(r,o);s!==void 0&&(i[o]=s)}}}Rc.wildcard=Symbol("*");const Vw=(e,t)=>Object.prototype.hasOwnProperty.call(e,t)?e[t]:void 0,iDe={[D5]:(e,t)=>{for(const r of t.keys()){const n=e.get(r);if(n==null)e.set(r,t.get(r));else{const{score:a,terms:i,match:o}=t.get(r);n.score=n.score+a,n.match=Object.assign(n.match,o),GB(n.terms,i)}}return e},[Tte]:(e,t)=>{const r=new Map;for(const n of t.keys()){const a=e.get(n);if(a==null)continue;const{score:i,terms:o,match:s}=t.get(n);GB(a.terms,o),r.set(n,{score:a.score+i,terms:a.terms,match:Object.assign(a.match,s)})}return r},[aDe]:(e,t)=>{for(const r of t.keys())e.delete(r);return e}},oDe={k:1.2,b:.7,d:.5},sDe=(e,t,r,n,a,i)=>{const{k:o,b:s,d:l}=i;return Math.log(1+(r-t+.5)/(t+.5))*(l+e*(o+1)/(e+o*(1-s+s*n/a)))},lDe=e=>(t,r,n)=>{const a=typeof e.fuzzy=="function"?e.fuzzy(t,r,n):e.fuzzy||!1,i=typeof e.prefix=="function"?e.prefix(t,r,n):e.prefix===!0,o=typeof e.boostTerm=="function"?e.boostTerm(t,r,n):1;return{term:t,fuzzy:a,prefix:i,termBoost:o}},Kw={idField:"id",extractField:(e,t)=>e[t],stringifyField:(e,t)=>e.toString(),tokenize:e=>e.split(dDe),processTerm:e=>e.toLowerCase(),fields:void 0,searchOptions:void 0,storeFields:[],logger:(e,t)=>{typeof(console==null?void 0:console[e])=="function"&&console[e](t)},autoVacuum:!0},$B={combineWith:D5,prefix:!1,fuzzy:!1,maxFuzzy:6,boost:{},weights:{fuzzy:.45,prefix:.375},bm25:oDe},uDe={combineWith:Tte,prefix:(e,t,r)=>t===r.length-1},AR={batchSize:1e3,batchWait:10},FR={minDirtFactor:.1,minDirtCount:20},Yw={...AR,...FR},cDe=(e,t)=>{e.includes(t)||e.push(t)},GB=(e,t)=>{for(const r of t)e.includes(r)||e.push(r)},jB=({score:e},{score:t})=>t-e,qB=()=>new Map,qb=e=>{const t=new Map;for(const r of Object.keys(e))t.set(parseInt(r,10),e[r]);return t},Wb=async e=>{const t=new Map;let r=0;for(const n of Object.keys(e))t.set(parseInt(n,10),e[n]),++r%1e3===0&&await Ate(0);return t},Ate=e=>new Promise(t=>setTimeout(t,e)),dDe=/[\n\r\p{Z}\p{P}]+/u,hDe={index:new Rc({fields:[]})};T.createContext(hDe);const WB=({label:e,color:t,hidden:r,labels:n={}})=>$e.createElement("div",{className:"node"},$e.createElement("span",{className:"render "+(r?"circle":"disc"),style:{backgroundColor:t||"#000"}}),$e.createElement("span",{className:`label ${r?"text-muted":""} ${e?"":"text-italic"}`},e||n.no_label||"No label")),pDe=({label:e,color:t,source:r,target:n,hidden:a,directed:i,labels:o={}})=>$e.createElement("div",{className:"edge"},$e.createElement(WB,Object.assign({},r,{labels:o})),$e.createElement("div",{className:"body"},$e.createElement("div",{className:"render"},$e.createElement("span",{className:a?"dotted":"dash",style:{borderColor:t||"#000"}})," ",i&&$e.createElement("span",{className:"arrow",style:{borderTopColor:t||"#000"}})),$e.createElement("span",{className:`label ${a?"text-muted":""} ${e?"":"fst-italic"}`},e||o.no_label||"No label")),$e.createElement(WB,Object.assign({},n,{labels:o}))),fDe=({id:e,labels:t})=>{const r=zo(),n=T.useMemo((()=>{const a=r.getGraph().getEdgeAttributes(e),i=r.getSetting("nodeReducer"),o=r.getSetting("edgeReducer"),s=r.getGraph().getNodeAttributes(r.getGraph().source(e)),l=r.getGraph().getNodeAttributes(r.getGraph().target(e));return Object.assign(Object.assign(Object.assign({color:r.getSetting("defaultEdgeColor"),directed:r.getGraph().isDirected(e)},a),o?o(e,a):{}),{source:Object.assign(Object.assign({color:r.getSetting("defaultNodeColor")},s),i?i(e,s):{}),target:Object.assign(Object.assign({color:r.getSetting("defaultNodeColor")},l),i?i(e,l):{})})}),[r,e]);return $e.createElement(pDe,Object.assign({},n,{labels:t}))};function N5(e,t){const[r,n]=T.useState(e);return T.useEffect(()=>{const a=setTimeout(()=>{n(e)},t);return()=>{clearTimeout(a)}},[e,t]),r}function mDe({fetcher:e,preload:t,filterFn:r,renderOption:n,getOptionValue:a,notFound:i,loadingSkeleton:o,ariaLabel:s,placeholder:l="Select...",value:u,onChange:d,onFocus:p,disabled:f=!1,className:m,noResultsMessage:b}){const[y,S]=T.useState(!1),[w,E]=T.useState(!1),[k,_]=T.useState([]),[R,C]=T.useState(!1),[F,L]=T.useState(null),[I,M]=T.useState(""),j=N5(I,t?0:150),Y=T.useRef(null);T.useEffect(()=>{S(!0)},[]),T.useEffect(()=>{const H=B=>{Y.current&&!Y.current.contains(B.target)&&w&&E(!1)};return document.addEventListener("mousedown",H),()=>{document.removeEventListener("mousedown",H)}},[w]);const W=T.useCallback(async H=>{try{C(!0),L(null);const B=await e(H);_(B)}catch(B){L(B instanceof Error?B.message:"Failed to fetch options")}finally{C(!1)}},[e]);T.useEffect(()=>{y&&(t?j&&_(H=>H.filter(B=>r?r(B,j):!0)):W(j))},[y,j,t,r,W]),T.useEffect(()=>{!y||!u||W(u)},[y,u,W]);const $=T.useCallback(H=>{d(H),requestAnimationFrame(()=>{const B=document.activeElement;B==null||B.blur(),E(!1)})},[d]),X=T.useCallback(()=>{E(!0),W(I)},[I,W]),z=T.useCallback(H=>{H.target.closest(".cmd-item")&&H.preventDefault()},[]);return x.jsx("div",{ref:Y,className:lt(f&&"cursor-not-allowed opacity-50",m),onMouseDown:z,children:x.jsxs(h2,{shouldFilter:!1,className:"bg-transparent",children:[x.jsxs("div",{children:[x.jsx(C5,{placeholder:l,value:I,className:"max-h-8","aria-label":s,onFocus:X,onValueChange:H=>{M(H),w||E(!0)}}),R&&x.jsx("div",{className:"absolute top-1/2 right-2 flex -translate-y-1/2 transform items-center",children:x.jsx(i5,{className:"h-4 w-4 animate-spin"})})]}),x.jsxs(p2,{hidden:!w,children:[F&&x.jsx("div",{className:"text-destructive p-4 text-center",children:F}),R&&k.length===0&&(o||x.jsx(gDe,{})),!R&&!F&&k.length===0&&(i||x.jsx(T5,{children:b||"No results found."})),x.jsx(Ep,{children:k.map((H,B)=>x.jsxs($e.Fragment,{children:[x.jsx(xp,{value:a(H),onSelect:$,onMouseMove:()=>p(a(H)),className:"truncate cmd-item",children:n(H)},a(H)+`${B}`),B!==k.length-1&&x.jsx("div",{className:"bg-foreground/10 h-[1px]"},`divider-${B}`)]},a(H)+`-fragment-${B}`))})]})]})})}function gDe(){return x.jsx(Ep,{children:x.jsx(xp,{disabled:!0,children:x.jsxs("div",{className:"flex w-full items-center gap-2",children:[x.jsx("div",{className:"bg-muted h-6 w-6 animate-pulse rounded-full"}),x.jsxs("div",{className:"flex flex-1 flex-col gap-1",children:[x.jsx("div",{className:"bg-muted h-4 w-24 animate-pulse rounded"}),x.jsx("div",{className:"bg-muted h-3 w-16 animate-pulse rounded"})]})]})})})}const Jw="__message_item",bDe=({id:e})=>{const t=pt.use.sigmaGraph();if(!(t!=null&&t.hasNode(e)))return null;const r=t.getNodeAttribute(e,"label")||e,n=t.getNodeAttribute(e,"color")||"#666",a=t.getNodeAttribute(e,"size")||4;return x.jsxs("div",{className:"flex items-center gap-2 p-2 text-sm",children:[x.jsx("div",{className:"rounded-full flex-shrink-0",style:{width:Math.max(8,Math.min(a*2,16)),height:Math.max(8,Math.min(a*2,16)),backgroundColor:n}}),x.jsx("span",{className:"truncate",children:r})]})};function yDe(e){return x.jsxs("div",{children:[e.type==="nodes"&&x.jsx(bDe,{id:e.id}),e.type==="edges"&&x.jsx(fDe,{id:e.id}),e.type==="message"&&x.jsx("div",{children:e.message})]})}const vDe=({onChange:e,onFocus:t,value:r})=>{const{t:n}=Sr(),a=pt.use.sigmaGraph(),i=pt.use.searchEngine();T.useEffect(()=>{a&&pt.getState().resetSearchEngine()},[a]),T.useEffect(()=>{if(!a||a.nodes().length===0||i)return;const s=new Rc({idField:"id",fields:["label"],searchOptions:{prefix:!0,fuzzy:.2,boost:{label:2}}}),l=a.nodes().filter(u=>a.hasNode(u)).map(u=>({id:u,label:a.getNodeAttribute(u,"label")}));l.length>0&&s.addAll(l),pt.getState().setSearchEngine(s)},[a,i]);const o=T.useCallback(async s=>{if(t&&t(null),!a||!i)return[];if(a.nodes().length===0)return[];if(!s)return a.nodes().filter(d=>a.hasNode(d)).slice(0,kb).map(d=>({id:d,type:"nodes"}));let l=i.search(s).filter(u=>a.hasNode(u.id)).map(u=>({id:u.id,type:"nodes"}));if(l.length<5){const u=new Set(l.map(p=>p.id)),d=a.nodes().filter(p=>{if(u.has(p)||!a.hasNode(p))return!1;const f=a.getNodeAttribute(p,"label");return f&&typeof f=="string"&&!f.toLowerCase().startsWith(s.toLowerCase())&&f.toLowerCase().includes(s.toLowerCase())}).map(p=>({id:p,type:"nodes"}));l=[...l,...d]}return l.length<=kb?l:[...l.slice(0,kb),{type:"message",id:Jw,message:n("graphPanel.search.message",{count:l.length-kb})}]},[a,i,t,n]);return x.jsx(mDe,{className:"bg-background/60 w-24 rounded-xl border-1 opacity-60 backdrop-blur-lg transition-all hover:w-fit hover:opacity-100 w-full",fetcher:o,renderOption:yDe,getOptionValue:s=>s.id,value:r&&r.type!=="message"?r.id:null,onChange:s=>{s!==Jw&&e(s?{id:s,type:"nodes"}:null)},onFocus:s=>{s!==Jw&&t&&t(s?{id:s,type:"nodes"}:null)},ariaLabel:n("graphPanel.search.placeholder"),placeholder:n("graphPanel.search.placeholder"),noResultsMessage:n("graphPanel.search.placeholder")})},SDe=({...e})=>x.jsx(vDe,{...e});function EDe({fetcher:e,preload:t,filterFn:r,renderOption:n,getOptionValue:a,getDisplayValue:i,notFound:o,loadingSkeleton:s,ariaLabel:l,placeholder:u="Select...",searchPlaceholder:d,value:p,onChange:f,disabled:m=!1,className:b,triggerClassName:y,searchInputClassName:S,noResultsMessage:w,triggerTooltip:E,clearable:k=!0,debounceTime:_=150}){const[R,C]=T.useState(!1),[F,L]=T.useState(!1),[I,M]=T.useState([]),[j,Y]=T.useState(!1),[W,$]=T.useState(null),[X,z]=T.useState(p),[H,B]=T.useState(null),[ee,J]=T.useState(""),K=N5(ee,t?0:_),[G,P]=T.useState([]),[ne,oe]=T.useState(null);T.useEffect(()=>{C(!0),z(p)},[p]),T.useEffect(()=>{p&&(!I.length||!H)?oe(x.jsx("div",{children:p})):H&&oe(null)},[p,I.length,H]),T.useEffect(()=>{if(p&&I.length>0){const ae=I.find(se=>a(se)===p);ae&&B(ae)}},[p,I,a]),T.useEffect(()=>{R||(async()=>{try{Y(!0),$(null);const se=await e("");P(se),M(se)}catch(se){$(se instanceof Error?se.message:"Failed to fetch options")}finally{Y(!1)}})()},[R,e]),T.useEffect(()=>{const ae=async()=>{try{Y(!0),$(null);const se=await e(K);P(se),M(se)}catch(se){$(se instanceof Error?se.message:"Failed to fetch options")}finally{Y(!1)}};R&&t?t&&M(K?G.filter(se=>r?r(se,K):!0):G):ae()},[e,K,R,t,r]);const pe=T.useCallback(ae=>{const se=k&&ae===X?"":ae;z(se),B(I.find(Ee=>a(Ee)===se)||null),f(se),L(!1)},[X,f,k,I,a]);return x.jsxs(t2,{open:F,onOpenChange:L,children:[x.jsx(r2,{asChild:!0,children:x.jsxs(Rt,{variant:"outline",role:"combobox","aria-expanded":F,"aria-label":l,className:lt("justify-between",m&&"cursor-not-allowed opacity-50",y),disabled:m,tooltip:E,side:"bottom",children:[p==="*"?x.jsx("div",{children:"*"}):H?i(H):ne||u,x.jsx(iEe,{className:"opacity-50",size:10})]})}),x.jsx(kg,{className:lt("p-0",b),onCloseAutoFocus:ae=>ae.preventDefault(),align:"start",sideOffset:8,collisionPadding:5,children:x.jsxs(h2,{shouldFilter:!1,children:[x.jsxs("div",{className:"relative w-full border-b",children:[x.jsx(C5,{placeholder:d||"Search...",value:ee,onValueChange:ae=>{J(ae)},className:S}),j&&I.length>0&&x.jsx("div",{className:"absolute top-1/2 right-2 flex -translate-y-1/2 transform items-center",children:x.jsx(i5,{className:"h-4 w-4 animate-spin"})})]}),x.jsxs(p2,{children:[W&&x.jsx("div",{className:"text-destructive p-4 text-center",children:W}),j&&I.length===0&&(s||x.jsx(xDe,{})),!j&&!W&&I.length===0&&(o||x.jsx(T5,{children:w||"No results found."})),x.jsx(Ep,{children:I.map(ae=>{const se=a(ae),Ee=ee.trim()===""?"":se;return x.jsxs(xp,{value:Ee,onSelect:()=>{pe(se)},className:"truncate",children:[n(ae),x.jsx(a5,{className:lt("ml-auto h-3 w-3",X===se?"opacity-100":"opacity-0")})]},se)})})]})]})})]})}function xDe(){return x.jsx(Ep,{children:x.jsx(xp,{disabled:!0,children:x.jsxs("div",{className:"flex w-full items-center gap-2",children:[x.jsx("div",{className:"bg-muted h-6 w-6 animate-pulse rounded-full"}),x.jsxs("div",{className:"flex flex-1 flex-col gap-1",children:[x.jsx("div",{className:"bg-muted h-4 w-24 animate-pulse rounded"}),x.jsx("div",{className:"bg-muted h-3 w-16 animate-pulse rounded"})]})]})})})}class _a{static getHistory(){try{const t=localStorage.getItem(this.STORAGE_KEY);if(!t)return[];const r=JSON.parse(t);return r.version!==this.VERSION?(console.warn(`Search history version mismatch. Expected ${this.VERSION}, got ${r.version}. Clearing history.`),this.clearHistory(),[]):Array.isArray(r.items)?r.items.sort((n,a)=>a.lastAccessed!==n.lastAccessed?a.lastAccessed-n.lastAccessed:(a.accessCount||0)-(n.accessCount||0)):(console.warn("Invalid search history format. Clearing history."),this.clearHistory(),[])}catch(t){return console.error("Error reading search history:",t),this.clearHistory(),[]}}static addToHistory(t){if(!(!t||typeof t!="string"||t.trim()===""))try{const r=this.getHistory(),n=Date.now(),a=t.trim(),i=r.findIndex(s=>s.label===a);if(i>=0){const s=r[i];s.lastAccessed=n,s.accessCount=(s.accessCount||0)+1,r.splice(i,1),r.unshift(s)}else r.unshift({label:a,lastAccessed:n,accessCount:1});r.length>this.MAX_HISTORY&&r.splice(this.MAX_HISTORY);const o={items:r,version:this.VERSION};localStorage.setItem(this.STORAGE_KEY,JSON.stringify(o))}catch(r){console.error("Error saving search history:",r)}}static clearHistory(){try{localStorage.removeItem(this.STORAGE_KEY)}catch(t){console.error("Error clearing search history:",t)}}static async initializeWithDefaults(t){if(this.getHistory().length===0&&t.length>0)try{const n=Date.now(),i={items:t.map((o,s)=>({label:o.trim(),lastAccessed:n-s,accessCount:0})),version:this.VERSION};localStorage.setItem(this.STORAGE_KEY,JSON.stringify(i))}catch(n){console.error("Error initializing search history with defaults:",n)}}static getRecentSearches(t=10){return this.getHistory().filter(n=>n.accessCount>0).slice(0,t)}static getPopularRecommendations(t){const n=this.getHistory().filter(a=>a.accessCount===0);return t?n.slice(0,t):n}static getHistoryLabels(t){const n=this.getHistory().map(a=>a.label);return t?n.slice(0,t):n}static hasLabel(t){return!t||typeof t!="string"?!1:this.getHistory().some(n=>n.label===t.trim())}static removeLabel(t){if(!(!t||typeof t!="string"))try{const r=this.getHistory(),n=t.trim(),a=r.filter(i=>i.label!==n);if(a.length!==r.length){const i={items:a,version:this.VERSION};localStorage.setItem(this.STORAGE_KEY,JSON.stringify(i))}}catch(r){console.error("Error removing label from search history:",r)}}static getStats(){const t=this.getHistory(),r=t.filter(i=>i.accessCount>0).length,n=t.filter(i=>i.accessCount===0).length;let a=0;try{const i=localStorage.getItem(this.STORAGE_KEY);a=i?i.length:0}catch{}return{totalItems:t.length,recentSearches:r,popularRecommendations:n,storageSize:a}}}mt(_a,"STORAGE_KEY","lightrag_search_history"),mt(_a,"MAX_HISTORY",Wbe),mt(_a,"VERSION",Vbe);const wDe=()=>{const{t:e}=Sr(),t=Ve.use.queryLabel(),r=Ve.use.searchLabelDropdownRefreshTrigger(),[n,a]=T.useState(!1),[i,o]=T.useState(0),[s,l]=T.useState(0),u=T.useCallback(()=>n?e("graphPanel.graphLabels.refreshingTooltip"):!t||t==="*"?e("graphPanel.graphLabels.refreshGlobalTooltip"):e("graphPanel.graphLabels.refreshCurrentLabelTooltip",{label:t}),[t,e,n]);T.useEffect(()=>{(async()=>{if(_a.getHistory().length===0)try{const b=await A8(G_);await _a.initializeWithDefaults(b)}catch(b){console.error("Failed to initialize search history:",b)}})()},[]),T.useEffect(()=>{l(f=>f+1)},[t]),T.useEffect(()=>{r>0&&l(f=>f+1)},[r]);const d=T.useCallback(async f=>{let m=[];if(!f||f.trim()===""||f.trim()==="*")m=_a.getHistoryLabels(Cb);else try{const y=await a1e(f.trim(),mJ);m=y.length<=Cb?y:[...y.slice(0,Cb),"..."]}catch(y){console.error("Search API failed, falling back to local history search:",y);const S=_a.getHistory(),w=f.toLowerCase().trim();m=S.filter(E=>E.label.toLowerCase().includes(w)).map(E=>E.label).slice(0,Cb)}return["*",...m.filter(y=>y!=="*")]},[i]),p=T.useCallback(async()=>{a(!0),pt.getState().setTypeColorMap(new Map);try{let f=t;if((!f||f.trim()==="")&&(Ve.getState().setQueryLabel("*"),f="*"),f&&f!=="*")console.log(`Refreshing current label: ${f}`),pt.getState().setGraphDataFetchAttempted(!1),pt.getState().setLastSuccessfulQueryLabel(""),pt.getState().incrementGraphDataVersion();else{console.log("Refreshing global data and popular labels");try{const m=await A8(G_);if(_a.clearHistory(),m.length===0){const b=["entity","relationship","document","concept"];await _a.initializeWithDefaults(b)}else await _a.initializeWithDefaults(m)}catch(m){console.error("Failed to reload popular labels:",m);const b=["entity","relationship","document"];_a.clearHistory(),await _a.initializeWithDefaults(b)}pt.getState().setGraphDataFetchAttempted(!1),pt.getState().setLastSuccessfulQueryLabel(""),pt.getState().incrementGraphDataVersion(),await new Promise(m=>setTimeout(m,0)),o(m=>m+1),l(m=>m+1)}}catch(f){console.error("Error during refresh:",f)}finally{a(!1)}},[t]);return x.jsxs("div",{className:"flex items-center",children:[x.jsx(Rt,{size:"icon",variant:no,onClick:p,tooltip:u(),className:"mr-2",disabled:n,children:x.jsx(uZ,{className:`h-4 w-4 ${n?"animate-spin":""}`})}),x.jsx("div",{className:"w-full min-w-[280px] max-w-[500px]",children:x.jsx(EDe,{className:"min-w-[300px]",triggerClassName:"max-h-8 w-full overflow-hidden",searchInputClassName:"max-h-8",triggerTooltip:e("graphPanel.graphLabels.selectTooltip"),fetcher:d,renderOption:f=>x.jsx("div",{className:"truncate",title:f,children:f}),getOptionValue:f=>f,getDisplayValue:f=>x.jsx("div",{className:"min-w-0 flex-1 truncate text-left",title:f,children:f}),notFound:x.jsx("div",{className:"py-6 text-center text-sm",children:e("graphPanel.graphLabels.noLabels")}),ariaLabel:e("graphPanel.graphLabels.label"),placeholder:e("graphPanel.graphLabels.placeholder"),searchPlaceholder:e("graphPanel.graphLabels.placeholder"),noResultsMessage:e("graphPanel.graphLabels.noLabels"),value:t!==null?t:"*",onChange:f=>{const m=Ve.getState().queryLabel;f==="..."&&(f="*"),f===m&&f!=="*"&&(f="*"),f&&f!=="*"&&f!=="..."&&f.trim()!==""&&_a.addToHistory(f),pt.getState().setGraphDataFetchAttempted(!1),Ve.getState().setQueryLabel(f),pt.getState().incrementGraphDataVersion()},clearable:!1,debounceTime:500},s)})]})},Fte=({text:e,className:t,tooltipClassName:r,tooltip:n,side:a,onClick:i})=>n?x.jsx(ka,{delayDuration:200,children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{className:lt(t,i!==void 0?"cursor-pointer":void 0),onClick:i,children:e})}),x.jsx(ga,{side:a,className:r,children:n})]})}):x.jsx("label",{className:lt(t,i!==void 0?"cursor-pointer":void 0),onClick:i,children:e}),kDe={unknown:"unknown",未知:"unknown",other:"other",其它:"other",concept:"concept",object:"concept",type:"concept",category:"concept",model:"concept",project:"concept",condition:"concept",rule:"concept",regulation:"concept",article:"concept",law:"concept",legalclause:"concept",policy:"concept",disease:"concept",概念:"concept",对象:"concept",类别:"concept",分类:"concept",模型:"concept",项目:"concept",条件:"concept",规则:"concept",法律:"concept",法律条款:"concept",条文:"concept",政策:"policy",疾病:"concept",method:"method",process:"method",方法:"method",过程:"method",artifact:"artifact",technology:"artifact",tech:"artifact",product:"artifact",equipment:"artifact",device:"artifact",stuff:"artifact",component:"artifact",material:"artifact",chemical:"artifact",drug:"artifact",medicine:"artifact",food:"artifact",weapon:"artifact",arms:"artifact",人工制品:"artifact",人造物品:"artifact",技术:"technology",科技:"technology",产品:"artifact",设备:"artifact",装备:"artifact",物品:"artifact",材料:"artifact",化学:"artifact",药物:"artifact",食品:"artifact",武器:"artifact",军火:"artifact",naturalobject:"naturalobject",natural:"naturalobject",phenomena:"naturalobject",substance:"naturalobject",plant:"naturalobject",自然对象:"naturalobject",自然物体:"naturalobject",自然现象:"naturalobject",物质:"naturalobject",物体:"naturalobject",data:"data",figure:"data",value:"data",数据:"data",数字:"data",数值:"data",content:"content",book:"content",video:"content",内容:"content",作品:"content",书籍:"content",视频:"content",organization:"organization",org:"organization",company:"organization",组织:"organization",公司:"organization",机构:"organization",组织机构:"organization",event:"event",事件:"event",activity:"event",活动:"event",person:"person",people:"person",human:"person",role:"person",人物:"person",人类:"person",人:"person",角色:"person",creature:"creature",animal:"creature",beings:"creature",being:"creature",alien:"creature",ghost:"creature",动物:"creature",生物:"creature",神仙:"creature",鬼怪:"creature",妖怪:"creature",location:"location",geography:"location",geo:"location",place:"location",address:"location",地点:"location",位置:"location",地址:"location",地理:"location",地域:"location"},VB={person:"#4169E1",creature:"#bd7ebe",organization:"#00cc00",location:"#cf6d17",event:"#00bfa0",concept:"#e3493b",method:"#b71c1c",content:"#0f558a",data:"#0000ff",artifact:"#4421af",naturalobject:"#b2e061",other:"#f4d371",unknown:"#b0b0b0"},CDe=["#84a3e1","#5a2c6d","#2F4F4F","#003366","#9b3a31","#00CED1","#b300b3","#0f705d","#ff99cc","#6ef7b3","#cd071e"],KB=e=>{const t="#5D6D7E",r=e?e.toLowerCase():"unknown",n=pt.getState().typeColorMap,a=kDe[r],i=a||r;if(n.has(i))return n.get(i)||t;if(a){const d=VB[a],p=new Map(n);return p.set(a,d),pt.setState({typeColorMap:p}),d}const o=new Set(Array.from(n.entries()).filter(([,d])=>!Object.values(VB).includes(d)).map(([,d])=>d)),l=CDe.find(d=>!o.has(d))||t,u=new Map(n);return u.set(r,l),pt.setState({typeColorMap:u}),l},TDe=e=>{if(!e)return console.log("Graph validation failed: graph is null"),!1;if(!Array.isArray(e.nodes)||!Array.isArray(e.edges))return console.log("Graph validation failed: nodes or edges is not an array"),!1;if(e.nodes.length===0)return console.log("Graph validation failed: nodes array is empty"),!1;for(const t of e.nodes)if(!t.id||!t.labels||!t.properties)return console.log("Graph validation failed: invalid node structure"),!1;for(const t of e.edges)if(!t.id||!t.source||!t.target)return console.log("Graph validation failed: invalid edge structure"),!1;for(const t of e.edges){const r=e.getNode(t.source),n=e.getNode(t.target);if(r==null||n==null)return console.log("Graph validation failed: edge references non-existent node"),!1}return console.log("Graph validation passed"),!0},ADe=async(e,t,r)=>{let n=null;pt.getState().setLabelsFetchAttempted(!0);const a=e||"*";try{console.log(`Fetching graph label: ${a}, depth: ${t}, nodes: ${r}`),n=await vJ(a,t,r)}catch(o){return Xn.getState().setErrorMessage(Da(o),"Query Graphs Error!"),null}let i=null;if(n){const o={},s={};for(let p=0;p<n.nodes.length;p++){const f=n.nodes[p];o[f.id]=p,f.x=Math.random(),f.y=Math.random(),f.degree=0,f.size=10}for(let p=0;p<n.edges.length;p++){const f=n.edges[p];s[f.id]=p;const m=o[f.source],b=o[f.target];if(m!==void 0&&b!==void 0){const y=n.nodes[m];if(!y){console.error(`Source node ${f.source} is undefined`);continue}const S=n.nodes[b];if(!S){console.error(`Target node ${f.target} is undefined`);continue}y.degree+=1,S.degree+=1}}let l=Number.MAX_SAFE_INTEGER,u=0;for(const p of n.nodes)l=Math.min(l,p.degree),u=Math.max(u,p.degree);const d=u-l;if(d>0){const p=uy-gu;for(const f of n.nodes)f.size=Math.round(gu+p*Math.pow((f.degree-l)/d,.5))}i=new t1e,i.nodes=n.nodes,i.edges=n.edges,i.nodeIdMap=o,i.edgeIdMap=s,TDe(i)||(i=null,console.warn("Invalid graph data")),console.log("Graph data loaded")}return{rawGraph:i,is_truncated:n.is_truncated}},FDe=e=>{var s,l;const t=Ve.getState().minEdgeSize,r=Ve.getState().maxEdgeSize;if(!e||!e.nodes.length)return console.log("No graph data available, skipping sigma graph creation"),null;const n=new Gh;for(const u of(e==null?void 0:e.nodes)??[]){xy(u.id+Date.now().toString(),{global:!0});const d=Math.random(),p=Math.random();n.addNode(u.id,{label:u.labels.join(", "),color:u.color,x:d,y:p,size:u.size,borderColor:$_,borderSize:.2})}for(const u of(e==null?void 0:e.edges)??[]){const d=((s=u.properties)==null?void 0:s.weight)!==void 0?Number(u.properties.weight):1;u.dynamicId=n.addEdge(u.source,u.target,{label:((l=u.properties)==null?void 0:l.keywords)||void 0,size:d,originalWeight:d,type:"curvedNoArrow"})}let a=Number.MAX_SAFE_INTEGER,i=0;n.forEachEdge(u=>{const d=n.getEdgeAttribute(u,"originalWeight")||1;a=Math.min(a,d),i=Math.max(i,d)});const o=i-a;if(o>0){const u=r-t;n.forEachEdge(d=>{const p=n.getEdgeAttribute(d,"originalWeight")||1,f=t+u*Math.pow((p-a)/o,.5);n.setEdgeAttribute(d,"size",f)})}else n.forEachEdge(u=>{n.setEdgeAttribute(u,"size",t)});return n},_De=()=>{const{t:e}=Sr(),t=Ve.use.queryLabel(),r=pt.use.rawGraph(),n=pt.use.sigmaGraph(),a=Ve.use.graphQueryMaxDepth(),i=Ve.use.graphMaxNodes(),o=pt.use.isFetching(),s=pt.use.nodeToExpand(),l=pt.use.nodeToPrune(),u=pt.use.graphDataVersion(),d=T.useRef(!1),p=T.useRef(!1),f=T.useRef(!1),m=T.useCallback(E=>(r==null?void 0:r.getNode(E))||null,[r]),b=T.useCallback((E,k=!0)=>(r==null?void 0:r.getEdge(E,k))||null,[r]),y=T.useRef(!1);T.useEffect(()=>{if(!t&&(r!==null||n!==null)){const E=pt.getState();E.reset(),E.setGraphDataFetchAttempted(!1),E.setLabelsFetchAttempted(!1),d.current=!1,p.current=!1}},[t,r,n]),T.useEffect(()=>{if(!y.current&&!(!t&&f.current)&&!o&&!pt.getState().graphDataFetchAttempted){y.current=!0,pt.getState().setGraphDataFetchAttempted(!0);const E=pt.getState();E.setIsFetching(!0),E.clearSelection(),E.sigmaGraph&&E.sigmaGraph.forEachNode(F=>{var L;(L=E.sigmaGraph)==null||L.setNodeAttribute(F,"highlighted",!1)}),console.log("Preparing graph data...");const k=t,_=a,R=i;let C;k?C=ADe(k,_,R):(console.log("Query label is empty, show empty graph"),C=Promise.resolve({rawGraph:null,is_truncated:!1})),C.then(F=>{const L=pt.getState(),I=F==null?void 0:F.rawGraph;if(I&&I.nodes&&I.nodes.forEach(M=>{var Y;const j=(Y=M.properties)==null?void 0:Y.entity_type;M.color=KB(j)}),F!=null&&F.is_truncated&&er.info(e("graphPanel.dataIsTruncated","Graph data is truncated to Max Nodes")),L.reset(),!I||!I.nodes||I.nodes.length===0){const M=new Gh;M.addNode("empty-graph-node",{label:e("graphPanel.emptyGraph"),color:"#5D6D7E",x:.5,y:.5,size:15,borderColor:$_,borderSize:.2}),L.setSigmaGraph(M),L.setRawGraph(null),L.setGraphIsEmpty(!0);const j=Xn.getState().message,Y=j&&j.includes("Authentication required");!Y&&k&&Ve.getState().setQueryLabel(""),Y?console.log("Keep queryLabel for post-login reload"):L.setLastSuccessfulQueryLabel(""),console.log(`Graph data is empty, created graph with empty graph node. Auth error: ${Y}`)}else{const M=FDe(I);I.buildDynamicMap(),L.setSigmaGraph(M),L.setRawGraph(I),L.setGraphIsEmpty(!1),L.setLastSuccessfulQueryLabel(k),L.setMoveToSelectedNode(!0)}d.current=!0,p.current=!0,y.current=!1,L.setIsFetching(!1),(!I||!I.nodes||I.nodes.length===0)&&!k&&(f.current=!0)}).catch(F=>{console.error("Error fetching graph data:",F);const L=pt.getState();L.setIsFetching(!1),d.current=!1,y.current=!1,L.setGraphDataFetchAttempted(!1),L.setLastSuccessfulQueryLabel("")})}},[t,a,i,o,e,u]),T.useEffect(()=>{s&&((async k=>{var _,R,C,F,L,I;if(!(!k||!n||!r))try{const M=r.getNode(k);if(!M){console.error("Node not found:",k);return}const j=M.labels[0];if(!j){console.error("Node has no label:",k);return}const Y=await vJ(j,2,1e3);if(!Y||!Y.nodes||!Y.edges){console.error("Failed to fetch extended graph");return}const W=[];for(const xe of Y.nodes){xy(xe.id,{global:!0});const Te=(_=xe.properties)==null?void 0:_.entity_type,Ne=KB(Te);W.push({id:xe.id,labels:xe.labels,properties:xe.properties,size:10,x:Math.random(),y:Math.random(),color:Ne,degree:0})}const $=[];for(const xe of Y.edges)$.push({id:xe.id,source:xe.source,target:xe.target,type:xe.type,properties:xe.properties,dynamicId:""});const X={};n.forEachNode(xe=>{X[xe]={x:n.getNodeAttribute(xe,"x"),y:n.getNodeAttribute(xe,"y")}});const z=new Set(n.nodes()),H=new Set,B=new Set,ee=1;let J=0,K=Number.MAX_SAFE_INTEGER,G=0;n.forEachNode(xe=>{const Te=n.degree(xe);J=Math.max(J,Te)}),n.forEachEdge(xe=>{const Te=n.getEdgeAttribute(xe,"originalWeight")||1;K=Math.min(K,Te),G=Math.max(G,Te)});for(const xe of W){if(z.has(xe.id))continue;$.some(Ne=>Ne.source===k&&Ne.target===xe.id||Ne.target===k&&Ne.source===xe.id)&&H.add(xe.id)}const P=new Map,ne=new Map,oe=new Set;for(const xe of $){const Te=z.has(xe.source)||H.has(xe.source),Ne=z.has(xe.target)||H.has(xe.target);Te&&Ne?(B.add(xe.id),H.has(xe.source)?P.set(xe.source,(P.get(xe.source)||0)+1):z.has(xe.source)&&ne.set(xe.source,(ne.get(xe.source)||0)+1),H.has(xe.target)?P.set(xe.target,(P.get(xe.target)||0)+1):z.has(xe.target)&&ne.set(xe.target,(ne.get(xe.target)||0)+1)):(n.hasNode(xe.source)?oe.add(xe.source):H.has(xe.source)&&(oe.add(xe.source),P.set(xe.source,(P.get(xe.source)||0)+1)),n.hasNode(xe.target)?oe.add(xe.target):H.has(xe.target)&&(oe.add(xe.target),P.set(xe.target,(P.get(xe.target)||0)+1)))}const pe=(xe,Te,Ne,he)=>{const Ze=he-Ne||1,Et=uy-gu;for(const qe of Te)if(xe.hasNode(qe)){let Ye=xe.degree(qe);Ye+=1;const we=Math.min(Ye,he+1),Qt=Math.round(gu+Et*Math.pow((we-Ne)/Ze,.5));xe.setNodeAttribute(qe,"size",Qt)}},ae=(xe,Te,Ne)=>{const he=Ve.getState().minEdgeSize,Ze=Ve.getState().maxEdgeSize,Et=Ne-Te||1,qe=Ze-he;xe.forEachEdge(Ye=>{const we=xe.getEdgeAttribute(Ye,"originalWeight")||1,Qt=he+qe*Math.pow((we-Te)/Et,.5);xe.setEdgeAttribute(Ye,"size",Qt)})};if(H.size===0){pe(n,oe,ee,J),er.info(e("graphPanel.propertiesView.node.noNewNodes"));return}for(const[,xe]of P.entries())J=Math.max(J,xe);for(const[xe,Te]of ne.entries()){const he=n.degree(xe)+Te;J=Math.max(J,he)}const se=J-ee||1,Ee=uy-gu,fe=((R=pt.getState().sigmaInstance)==null?void 0:R.getCamera().ratio)||1,Me=Math.max(Math.sqrt(M.size)*4,Math.sqrt(H.size)*3)/fe;xy(Date.now().toString(),{global:!0});const Oe=Math.random()*2*Math.PI;console.log("nodeSize:",M.size,"nodesToAdd:",H.size),console.log("cameraRatio:",Math.round(fe*100)/100,"spreadFactor:",Math.round(Me*100)/100);for(const xe of H){const Te=W.find(we=>we.id===xe),Ne=P.get(xe)||0,he=Math.min(Ne,J+1),Ze=Math.round(gu+Ee*Math.pow((he-ee)/se,.5)),Et=2*Math.PI*(Array.from(H).indexOf(xe)/H.size),qe=((C=X[xe])==null?void 0:C.x)||X[M.id].x+Math.cos(Oe+Et)*Me,Ye=((F=X[xe])==null?void 0:F.y)||X[M.id].y+Math.sin(Oe+Et)*Me;n.addNode(xe,{label:Te.labels.join(", "),color:Te.color,x:qe,y:Ye,size:Ze,borderColor:$_,borderSize:.2}),r.getNode(xe)||(Te.size=Ze,Te.x=qe,Te.y=Ye,Te.degree=Ne,r.nodes.push(Te),r.nodeIdMap[xe]=r.nodes.length-1)}for(const xe of B){const Te=$.find(he=>he.id===xe);if(n.hasEdge(Te.source,Te.target))continue;const Ne=((L=Te.properties)==null?void 0:L.weight)!==void 0?Number(Te.properties.weight):1;K=Math.min(K,Ne),G=Math.max(G,Ne),Te.dynamicId=n.addEdge(Te.source,Te.target,{label:((I=Te.properties)==null?void 0:I.keywords)||void 0,size:Ne,originalWeight:Ne,type:"curvedNoArrow"}),r.getEdge(Te.id,!1)?console.error("Edge already exists in rawGraph:",Te.id):(r.edges.push(Te),r.edgeIdMap[Te.id]=r.edges.length-1,r.edgeDynamicIdMap[Te.dynamicId]=r.edges.length-1)}if(r.buildDynamicMap(),pt.getState().resetSearchEngine(),pe(n,oe,ee,J),ae(n,K,G),n.hasNode(k)){const xe=n.degree(k),Te=Math.min(xe,J+1),Ne=Math.round(gu+Ee*Math.pow((Te-ee)/se,.5));n.setNodeAttribute(k,"size",Ne),M.size=Ne,M.degree=xe}}catch(M){console.error("Error expanding node:",M)}})(s),window.setTimeout(()=>{pt.getState().triggerNodeExpand(null)},0))},[s,n,r,e]);const S=T.useCallback((E,k)=>{const _=new Set([E]);return k.forEachNode(R=>{if(R===E)return;const C=k.neighbors(R);C.length===1&&C[0]===E&&_.add(R)}),_},[]);return T.useEffect(()=>{l&&((k=>{if(!(!k||!n||!r))try{const _=pt.getState();if(!n.hasNode(k)){console.error("Node not found:",k);return}const R=S(k,n);if(R.size===n.nodes().length){er.error(e("graphPanel.propertiesView.node.deleteAllNodesError"));return}_.clearSelection();for(const C of R){n.dropNode(C);const F=r.nodeIdMap[C];if(F!==void 0){const L=r.edges.filter(I=>I.source===C||I.target===C);for(const I of L){const M=r.edgeIdMap[I.id];if(M!==void 0){r.edges.splice(M,1);for(const[j,Y]of Object.entries(r.edgeIdMap))Y>M&&(r.edgeIdMap[j]=Y-1);delete r.edgeIdMap[I.id],delete r.edgeDynamicIdMap[I.dynamicId]}}r.nodes.splice(F,1);for(const[I,M]of Object.entries(r.nodeIdMap))M>F&&(r.nodeIdMap[I]=M-1);delete r.nodeIdMap[C]}}r.buildDynamicMap(),pt.getState().resetSearchEngine(),R.size>1&&er.info(e("graphPanel.propertiesView.node.nodesRemoved",{count:R.size}))}catch(_){console.error("Error pruning node:",_)}})(l),window.setTimeout(()=>{pt.getState().triggerNodePrune(null)},0))},[l,n,r,S,e]),{lightrageGraph:T.useCallback(()=>{if(n)return n;console.log("Creating new Sigma graph instance");const E=new Gh;return pt.getState().setSigmaGraph(E),E},[n]),getNode:m,getEdge:b}},RDe=({name:e})=>{const{t}=Sr(),r=n=>{const a=`graphPanel.propertiesView.node.propertyNames.${n}`,i=t(a);return i===a?n:i};return x.jsx("span",{className:"text-primary/60 tracking-wide whitespace-nowrap",children:r(e)})},DDe=({onClick:e})=>x.jsx("div",{children:x.jsx(MEe,{className:"h-3 w-3 text-gray-500 hover:text-gray-700 cursor-pointer",onClick:e})}),NDe=({value:e,onClick:t,tooltip:r})=>x.jsx("div",{className:"flex items-center gap-1 overflow-hidden",children:x.jsx(Fte,{className:"hover:bg-primary/20 rounded p-1 overflow-hidden text-ellipsis whitespace-nowrap",tooltipClassName:"max-w-80 -translate-x-15",text:e,tooltip:r||(typeof e=="string"?e:JSON.stringify(e,null,2)),side:"left",onClick:t})}),LDe=({isOpen:e,onClose:t,onSave:r,propertyName:n,initialValue:a,isSubmitting:i=!1,errorMessage:o=null})=>{const{t:s}=Sr(),[l,u]=T.useState(""),[d,p]=T.useState(!1);T.useEffect(()=>{e&&(u(a),p(!1))},[e,a]);const f=y=>{const S=`graphPanel.propertiesView.node.propertyNames.${y}`,w=s(S);return w===S?y:w},m=y=>{switch(y){case"description":return{className:"max-h-[50vh] min-h-[10em] resize-y",style:{height:"70vh",minHeight:"20em",resize:"vertical"}};case"entity_id":return{rows:2,className:"",style:{}};case"keywords":return{rows:4,className:"",style:{}};default:return{rows:5,className:"",style:{}}}},b=async()=>{l.trim()!==""&&await r(l,n==="entity_id"?{allowMerge:d}:void 0)};return x.jsx(Du,{open:e,onOpenChange:y=>!y&&t(),children:x.jsxs(Sl,{className:"sm:max-w-md",children:[x.jsxs(El,{children:[x.jsx(xl,{children:s("graphPanel.propertiesView.editProperty",{property:f(n)})}),x.jsx(wl,{children:s("graphPanel.propertiesView.editPropertyDescription")})]}),o&&x.jsx("div",{className:"bg-destructive/15 text-destructive px-4 py-2 rounded-md text-sm",children:o}),x.jsx("div",{className:"grid gap-4 py-4",children:(()=>{const y=m(n);return n==="description"?x.jsx("textarea",{value:l,onChange:S=>u(S.target.value),className:`border-input focus-visible:ring-ring flex w-full rounded-md border bg-transparent px-3 py-2 text-sm shadow-sm transition-colors focus-visible:ring-1 focus-visible:outline-none disabled:cursor-not-allowed disabled:opacity-50 ${y.className}`,style:y.style,disabled:i}):x.jsx("textarea",{value:l,onChange:S=>u(S.target.value),rows:y.rows,className:`border-input focus-visible:ring-ring flex w-full rounded-md border bg-transparent px-3 py-2 text-sm shadow-sm transition-colors focus-visible:ring-1 focus-visible:outline-none disabled:cursor-not-allowed disabled:opacity-50 ${y.className}`,disabled:i})})()}),n==="entity_id"&&x.jsx("div",{className:"rounded-md border border-border bg-muted/20 p-3",children:x.jsxs("label",{className:"flex items-start gap-2 text-sm font-medium",children:[x.jsx(ol,{id:"allow-merge",checked:d,disabled:i,onCheckedChange:y=>p(y===!0)}),x.jsxs("div",{children:[x.jsx("span",{children:s("graphPanel.propertiesView.mergeOptionLabel")}),x.jsx("p",{className:"text-xs font-normal text-muted-foreground",children:s("graphPanel.propertiesView.mergeOptionDescription")})]})]})}),x.jsxs(xg,{children:[x.jsx(Rt,{type:"button",variant:"outline",onClick:t,disabled:i,children:s("common.cancel")}),x.jsx(Rt,{type:"button",onClick:b,disabled:i,children:i?x.jsxs(x.Fragment,{children:[x.jsx("span",{className:"mr-2",children:x.jsxs("svg",{className:"animate-spin h-4 w-4",xmlns:"http://www.w3.org/2000/svg",fill:"none",viewBox:"0 0 24 24",children:[x.jsx("circle",{className:"opacity-25",cx:"12",cy:"12",r:"10",stroke:"currentColor",strokeWidth:"4"}),x.jsx("path",{className:"opacity-75",fill:"currentColor",d:"M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"})]})}),s("common.saving")]}):s("common.save")})]})]})})},MDe=({mergeDialogOpen:e,mergeDialogInfo:t,onOpenChange:r,onRefresh:n})=>{const{t:a}=Sr(),i=Ve.use.queryLabel();return x.jsx(Du,{open:e,onOpenChange:r,children:x.jsxs(Sl,{children:[x.jsxs(El,{children:[x.jsx(xl,{children:a("graphPanel.propertiesView.mergeDialog.title")}),x.jsx(wl,{children:a("graphPanel.propertiesView.mergeDialog.description",{source:(t==null?void 0:t.sourceEntity)??"",target:(t==null?void 0:t.targetEntity)??""})})]}),x.jsx("p",{className:"text-sm text-muted-foreground",children:a("graphPanel.propertiesView.mergeDialog.refreshHint")}),x.jsxs(xg,{className:"mt-4 flex-col gap-2 sm:flex-row sm:justify-end",children:[i!==(t==null?void 0:t.sourceEntity)&&x.jsx(Rt,{type:"button",variant:"outline",onClick:()=>n(!1),children:a("graphPanel.propertiesView.mergeDialog.keepCurrentStart")}),x.jsx(Rt,{type:"button",onClick:()=>n(!0),children:a("graphPanel.propertiesView.mergeDialog.useMergedStart")})]})]})})},IDe=({name:e,value:t,onClick:r,nodeId:n,edgeId:a,entityId:i,dynamicId:o,entityType:s,sourceId:l,targetId:u,onValueChange:d,isEditable:p=!1,tooltip:f})=>{const{t:m}=Sr(),[b,y]=T.useState(!1),[S,w]=T.useState(!1),[E,k]=T.useState(t),[_,R]=T.useState(null),[C,F]=T.useState(!1),[L,I]=T.useState(null);T.useEffect(()=>{k(t)},[t]);const M=()=>{p&&!b&&(y(!0),R(null))},j=()=>{y(!1),R(null)},Y=async($,X)=>{if(S||$===String(E)){y(!1),R(null);return}w(!0),R(null);try{if(s==="node"&&i&&n){let z={[e]:$};const H=(X==null?void 0:X.allowMerge)??!1;if(e==="entity_id"){if(!H&&await y1e($)){const P=m("graphPanel.propertiesView.errors.duplicateName");R(P),er.error(P);return}z={entity_name:$}}const ee=(await g1e(i,z,!0,H)).operation_summary,J=(ee==null?void 0:ee.operation_status)||"complete_success",K=(ee==null?void 0:ee.final_entity)??$;if(J==="success"){if(ee!=null&&ee.merged)I({targetEntity:K,sourceEntity:i}),F(!0),_a.removeLabel(i),er.success(m("graphPanel.propertiesView.success.entityMerged"));else{try{await pt.getState().updateNodeAndSelect(n,i,e,K)}catch(P){throw console.error("Error updating node in graph:",P),new Error("Failed to update node in graph")}if(e==="entity_id"){const P=Ve.getState().queryLabel;_a.removeLabel(i),_a.addToHistory(K),Ve.getState().triggerSearchLabelDropdownRefresh(),P===i&&Ve.getState().setQueryLabel(K)}er.success(m("graphPanel.propertiesView.success.entityUpdated"))}const G=e==="entity_id"?K:$;k(G),d==null||d(G)}else if(J==="partial_success"){const G=(ee==null?void 0:ee.merge_error)||"Unknown error",P=m("graphPanel.propertiesView.errors.updateSuccessButMergeFailed",{error:G});R(P),er.error(P);return}else{if((ee==null?void 0:ee.merge_status)==="failed"){const G=(ee==null?void 0:ee.merge_error)||"Unknown error",P=m("graphPanel.propertiesView.errors.mergeFailed",{error:G});R(P),er.error(P)}else{const G=m("graphPanel.propertiesView.errors.updateFailed");R(G),er.error(G)}return}}else if(s==="edge"&&l&&u&&a&&o){const z={[e]:$};await b1e(l,u,z);try{await pt.getState().updateEdgeAndSelect(a,o,l,u,e,$)}catch(H){throw console.error(`Error updating edge ${l}->${u} in graph:`,H),new Error("Failed to update edge in graph")}er.success(m("graphPanel.propertiesView.success.relationUpdated")),k($),d==null||d($)}y(!1)}catch(z){console.error("Error updating property:",z);const H=z instanceof Error?z.message:m("graphPanel.propertiesView.errors.updateFailed");R(H),er.error(H);return}finally{w(!1)}},W=$=>{const X=L,z=pt.getState(),H=Ve.getState(),B=H.queryLabel;z.clearSelection(),z.setGraphDataFetchAttempted(!1),z.setLastSuccessfulQueryLabel(""),$&&(X!=null&&X.targetEntity)?H.setQueryLabel(X.targetEntity):(H.setQueryLabel(""),setTimeout(()=>{H.setQueryLabel(B)},50)),z.incrementGraphDataVersion(),F(!1),I(null),er.info(m("graphPanel.propertiesView.mergeDialog.refreshing"))};return x.jsxs("div",{className:"flex items-center gap-1 overflow-hidden",children:[x.jsx(RDe,{name:e}),x.jsx(DDe,{onClick:M}),":",x.jsx(NDe,{value:E,onClick:r,tooltip:f||(typeof E=="string"?E:JSON.stringify(E,null,2))}),x.jsx(LDe,{isOpen:b,onClose:j,onSave:Y,propertyName:e,initialValue:String(E),isSubmitting:S,errorMessage:_}),x.jsx(MDe,{mergeDialogOpen:C,mergeDialogInfo:L,onOpenChange:$=>{F($),$||I(null)},onRefresh:W})]})},ODe=()=>{const{getNode:e,getEdge:t}=_De(),r=pt.use.selectedNode(),n=pt.use.focusedNode(),a=pt.use.selectedEdge(),i=pt.use.focusedEdge(),o=pt.use.graphDataVersion(),[s,l]=T.useState(null),[u,d]=T.useState(null);return T.useEffect(()=>{let p=null,f=null;n?(p="node",f=e(n)):r?(p="node",f=e(r)):i?(p="edge",f=t(i,!0)):a&&(p="edge",f=t(a,!0)),f?(p=="node"?l(BDe(f)):l(PDe(f)),d(p)):(l(null),d(null))},[n,r,i,a,o,l,d,e,t]),s?x.jsx("div",{className:"bg-background/80 max-w-xs rounded-lg border-2 p-2 text-xs backdrop-blur-lg",children:u=="node"?x.jsx(zDe,{node:s}):x.jsx(HDe,{edge:s})}):x.jsx(x.Fragment,{})},BDe=e=>{const t=pt.getState(),r=[];if(t.sigmaGraph&&t.rawGraph)try{if(!t.sigmaGraph.hasNode(e.id))return console.warn("Node not found in sigmaGraph:",e.id),{...e,relationships:[]};const n=t.sigmaGraph.edges(e.id);for(const a of n){if(!t.sigmaGraph.hasEdge(a))continue;const i=t.rawGraph.getEdge(a,!0);if(i){const s=e.id===i.source?i.target:i.source;if(!t.sigmaGraph.hasNode(s))continue;const l=t.rawGraph.getNode(s);l&&r.push({type:"Neighbour",id:s,label:l.properties.entity_id?l.properties.entity_id:l.labels.join(", ")})}}}catch(n){console.error("Error refining node properties:",n)}return{...e,relationships:r}},PDe=e=>{const t=pt.getState();let r,n;if(t.sigmaGraph&&t.rawGraph)try{if(!t.sigmaGraph.hasEdge(e.dynamicId))return console.warn("Edge not found in sigmaGraph:",e.id,"dynamicId:",e.dynamicId),{...e,sourceNode:void 0,targetNode:void 0};t.sigmaGraph.hasNode(e.source)&&(r=t.rawGraph.getNode(e.source)),t.sigmaGraph.hasNode(e.target)&&(n=t.rawGraph.getNode(e.target))}catch(a){console.error("Error refining edge properties:",a)}return{...e,sourceNode:r,targetNode:n}},ns=({name:e,value:t,onClick:r,tooltip:n,nodeId:a,edgeId:i,dynamicId:o,entityId:s,entityType:l,sourceId:u,targetId:d,isEditable:p=!1,truncate:f})=>{const{t:m}=Sr(),b=E=>{const k=`graphPanel.propertiesView.node.propertyNames.${E}`,_=m(k);return _===k?E:_},y=E=>typeof E=="string"?E.replace(/<SEP>/g,`;
`):typeof E=="string"?E:JSON.stringify(E,null,2),S=y(t);let w=n||y(t);return e==="source_id"&&f&&(w+=`
(Truncated: ${f})`),p&&(e==="description"||e==="entity_id"||e==="entity_type"||e==="keywords")?x.jsx(IDe,{name:e,value:t,onClick:r,nodeId:a,entityId:s,edgeId:i,dynamicId:o,entityType:l,sourceId:u,targetId:d,isEditable:!0,tooltip:n||(typeof t=="string"?t:JSON.stringify(t,null,2))}):x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsxs("span",{className:"text-primary/60 tracking-wide whitespace-nowrap",children:[b(e),e==="source_id"&&f&&x.jsx("sup",{className:"text-red-500",children:"†"})]}),":",x.jsx(Fte,{className:"hover:bg-primary/20 rounded p-1 overflow-hidden text-ellipsis",tooltipClassName:"max-w-96 -translate-x-13",text:S,tooltip:w,side:"left",onClick:r})]})},zDe=({node:e})=>{const{t}=Sr(),r=()=>{pt.getState().triggerNodeExpand(e.id)},n=()=>{pt.getState().triggerNodePrune(e.id)};return x.jsxs("div",{className:"flex flex-col gap-2",children:[x.jsxs("div",{className:"flex justify-between items-center",children:[x.jsx("h3",{className:"text-md pl-1 font-bold tracking-wide text-blue-700",children:t("graphPanel.propertiesView.node.title")}),x.jsxs("div",{className:"flex gap-3",children:[x.jsx(Rt,{size:"icon",variant:"ghost",className:"h-7 w-7 border border-gray-400 hover:bg-gray-200 dark:border-gray-600 dark:hover:bg-gray-700",onClick:r,tooltip:t("graphPanel.propertiesView.node.expandNode"),children:x.jsx(fEe,{className:"h-4 w-4 text-gray-700 dark:text-gray-300"})}),x.jsx(Rt,{size:"icon",variant:"ghost",className:"h-7 w-7 border border-gray-400 hover:bg-gray-200 dark:border-gray-600 dark:hover:bg-gray-700",onClick:n,tooltip:t("graphPanel.propertiesView.node.pruneNode"),children:x.jsx($Ee,{className:"h-4 w-4 text-gray-900 dark:text-gray-300"})})]})]}),x.jsxs("div",{className:"bg-primary/5 max-h-96 overflow-auto rounded p-1",children:[x.jsx(ns,{name:t("graphPanel.propertiesView.node.id"),value:String(e.id)}),x.jsx(ns,{name:t("graphPanel.propertiesView.node.labels"),value:e.labels.join(", "),onClick:()=>{pt.getState().setSelectedNode(e.id,!0)}}),x.jsx(ns,{name:t("graphPanel.propertiesView.node.degree"),value:e.degree})]}),x.jsx("h3",{className:"text-md pl-1 font-bold tracking-wide text-amber-700",children:t("graphPanel.propertiesView.node.properties")}),x.jsx("div",{className:"bg-primary/5 max-h-96 overflow-auto rounded p-1",children:Object.keys(e.properties).sort().map(a=>a==="created_at"||a==="truncate"?null:x.jsx(ns,{name:a,value:e.properties[a],nodeId:String(e.id),entityId:e.properties.entity_id,entityType:"node",isEditable:a==="description"||a==="entity_id"||a==="entity_type",truncate:e.properties.truncate},a))}),e.relationships.length>0&&x.jsxs(x.Fragment,{children:[x.jsx("h3",{className:"text-md pl-1 font-bold tracking-wide text-emerald-700",children:t("graphPanel.propertiesView.node.relationships")}),x.jsx("div",{className:"bg-primary/5 max-h-96 overflow-auto rounded p-1",children:e.relationships.map(({type:a,id:i,label:o})=>x.jsx(ns,{name:a,value:o,onClick:()=>{pt.getState().setSelectedNode(i,!0)}},i))})]})]})},HDe=({edge:e})=>{const{t}=Sr();return x.jsxs("div",{className:"flex flex-col gap-2",children:[x.jsx("h3",{className:"text-md pl-1 font-bold tracking-wide text-violet-700",children:t("graphPanel.propertiesView.edge.title")}),x.jsxs("div",{className:"bg-primary/5 max-h-96 overflow-auto rounded p-1",children:[x.jsx(ns,{name:t("graphPanel.propertiesView.edge.id"),value:e.id}),e.type&&x.jsx(ns,{name:t("graphPanel.propertiesView.edge.type"),value:e.type}),x.jsx(ns,{name:t("graphPanel.propertiesView.edge.source"),value:e.sourceNode?e.sourceNode.labels.join(", "):e.source,onClick:()=>{pt.getState().setSelectedNode(e.source,!0)}}),x.jsx(ns,{name:t("graphPanel.propertiesView.edge.target"),value:e.targetNode?e.targetNode.labels.join(", "):e.target,onClick:()=>{pt.getState().setSelectedNode(e.target,!0)}})]}),x.jsx("h3",{className:"text-md pl-1 font-bold tracking-wide text-amber-700",children:t("graphPanel.propertiesView.edge.properties")}),x.jsx("div",{className:"bg-primary/5 max-h-96 overflow-auto rounded p-1",children:Object.keys(e.properties).sort().map(r=>{var n,a;return r==="created_at"||r==="truncate"?null:x.jsx(ns,{name:r,value:e.properties[r],edgeId:String(e.id),dynamicId:String(e.dynamicId),entityType:"edge",sourceId:((n=e.sourceNode)==null?void 0:n.properties.entity_id)||e.source,targetId:((a=e.targetNode)==null?void 0:a.properties.entity_id)||e.target,isEditable:r==="description"||r==="keywords",truncate:e.properties.truncate},r)})})]})},UDe=()=>{const{t:e}=Sr(),t=Ve.use.graphQueryMaxDepth(),r=Ve.use.graphMaxNodes();return x.jsxs("div",{className:"absolute bottom-4 left-[calc(1rem+2.5rem)] flex items-center gap-2 text-xs text-gray-400",children:[x.jsxs("div",{children:[e("graphPanel.sideBar.settings.depth"),": ",t]}),x.jsxs("div",{children:[e("graphPanel.sideBar.settings.max"),": ",r]})]})},qc=T.forwardRef(({className:e,...t},r)=>x.jsx("div",{ref:r,className:lt("bg-card text-card-foreground rounded-xl border shadow",e),...t}));qc.displayName="Card";const Hm=T.forwardRef(({className:e,...t},r)=>x.jsx("div",{ref:r,className:lt("flex flex-col space-y-1.5 p-6",e),...t}));Hm.displayName="CardHeader";const Um=T.forwardRef(({className:e,...t},r)=>x.jsx("div",{ref:r,className:lt("leading-none font-semibold tracking-tight",e),...t}));Um.displayName="CardTitle";const m2=T.forwardRef(({className:e,...t},r)=>x.jsx("div",{ref:r,className:lt("text-muted-foreground text-sm",e),...t}));m2.displayName="CardDescription";const $m=T.forwardRef(({className:e,...t},r)=>x.jsx("div",{ref:r,className:lt("p-6 pt-0",e),...t}));$m.displayName="CardContent";const $De=T.forwardRef(({className:e,...t},r)=>x.jsx("div",{ref:r,className:lt("flex items-center p-6 pt-0",e),...t}));$De.displayName="CardFooter";function GDe(e,t){return T.useReducer((r,n)=>t[r][n]??r,e)}var L5="ScrollArea",[_te]=Bo(L5),[jDe,po]=_te(L5),Rte=T.forwardRef((e,t)=>{const{__scopeScrollArea:r,type:n="hover",dir:a,scrollHideDelay:i=600,...o}=e,[s,l]=T.useState(null),[u,d]=T.useState(null),[p,f]=T.useState(null),[m,b]=T.useState(null),[y,S]=T.useState(null),[w,E]=T.useState(0),[k,_]=T.useState(0),[R,C]=T.useState(!1),[F,L]=T.useState(!1),I=Rr(t,j=>l(j)),M=n2(a);return x.jsx(jDe,{scope:r,type:n,dir:M,scrollHideDelay:i,scrollArea:s,viewport:u,onViewportChange:d,content:p,onContentChange:f,scrollbarX:m,onScrollbarXChange:b,scrollbarXEnabled:R,onScrollbarXEnabledChange:C,scrollbarY:y,onScrollbarYChange:S,scrollbarYEnabled:F,onScrollbarYEnabledChange:L,onCornerWidthChange:E,onCornerHeightChange:_,children:x.jsx(Kt.div,{dir:M,...o,ref:I,style:{position:"relative","--radix-scroll-area-corner-width":w+"px","--radix-scroll-area-corner-height":k+"px",...e.style}})})});Rte.displayName=L5;var Dte="ScrollAreaViewport",Nte=T.forwardRef((e,t)=>{const{__scopeScrollArea:r,children:n,nonce:a,...i}=e,o=po(Dte,r),s=T.useRef(null),l=Rr(t,s,o.onViewportChange);return x.jsxs(x.Fragment,{children:[x.jsx("style",{dangerouslySetInnerHTML:{__html:"[data-radix-scroll-area-viewport]{scrollbar-width:none;-ms-overflow-style:none;-webkit-overflow-scrolling:touch;}[data-radix-scroll-area-viewport]::-webkit-scrollbar{display:none}"},nonce:a}),x.jsx(Kt.div,{"data-radix-scroll-area-viewport":"",...i,ref:l,style:{overflowX:o.scrollbarXEnabled?"scroll":"hidden",overflowY:o.scrollbarYEnabled?"scroll":"hidden",...e.style},children:x.jsx("div",{ref:o.onContentChange,style:{minWidth:"100%",display:"table"},children:n})})]})});Nte.displayName=Dte;var ys="ScrollAreaScrollbar",M5=T.forwardRef((e,t)=>{const{forceMount:r,...n}=e,a=po(ys,e.__scopeScrollArea),{onScrollbarXEnabledChange:i,onScrollbarYEnabledChange:o}=a,s=e.orientation==="horizontal";return T.useEffect(()=>(s?i(!0):o(!0),()=>{s?i(!1):o(!1)}),[s,i,o]),a.type==="hover"?x.jsx(qDe,{...n,ref:t,forceMount:r}):a.type==="scroll"?x.jsx(WDe,{...n,ref:t,forceMount:r}):a.type==="auto"?x.jsx(Lte,{...n,ref:t,forceMount:r}):a.type==="always"?x.jsx(I5,{...n,ref:t}):null});M5.displayName=ys;var qDe=T.forwardRef((e,t)=>{const{forceMount:r,...n}=e,a=po(ys,e.__scopeScrollArea),[i,o]=T.useState(!1);return T.useEffect(()=>{const s=a.scrollArea;let l=0;if(s){const u=()=>{window.clearTimeout(l),o(!0)},d=()=>{l=window.setTimeout(()=>o(!1),a.scrollHideDelay)};return s.addEventListener("pointerenter",u),s.addEventListener("pointerleave",d),()=>{window.clearTimeout(l),s.removeEventListener("pointerenter",u),s.removeEventListener("pointerleave",d)}}},[a.scrollArea,a.scrollHideDelay]),x.jsx($i,{present:r||i,children:x.jsx(Lte,{"data-state":i?"visible":"hidden",...n,ref:t})})}),WDe=T.forwardRef((e,t)=>{const{forceMount:r,...n}=e,a=po(ys,e.__scopeScrollArea),i=e.orientation==="horizontal",o=b2(()=>l("SCROLL_END"),100),[s,l]=GDe("hidden",{hidden:{SCROLL:"scrolling"},scrolling:{SCROLL_END:"idle",POINTER_ENTER:"interacting"},interacting:{SCROLL:"interacting",POINTER_LEAVE:"idle"},idle:{HIDE:"hidden",SCROLL:"scrolling",POINTER_ENTER:"interacting"}});return T.useEffect(()=>{if(s==="idle"){const u=window.setTimeout(()=>l("HIDE"),a.scrollHideDelay);return()=>window.clearTimeout(u)}},[s,a.scrollHideDelay,l]),T.useEffect(()=>{const u=a.viewport,d=i?"scrollLeft":"scrollTop";if(u){let p=u[d];const f=()=>{const m=u[d];p!==m&&(l("SCROLL"),o()),p=m};return u.addEventListener("scroll",f),()=>u.removeEventListener("scroll",f)}},[a.viewport,i,l,o]),x.jsx($i,{present:r||s!=="hidden",children:x.jsx(I5,{"data-state":s==="hidden"?"hidden":"visible",...n,ref:t,onPointerEnter:$t(e.onPointerEnter,()=>l("POINTER_ENTER")),onPointerLeave:$t(e.onPointerLeave,()=>l("POINTER_LEAVE"))})})}),Lte=T.forwardRef((e,t)=>{const r=po(ys,e.__scopeScrollArea),{forceMount:n,...a}=e,[i,o]=T.useState(!1),s=e.orientation==="horizontal",l=b2(()=>{if(r.viewport){const u=r.viewport.offsetWidth<r.viewport.scrollWidth,d=r.viewport.offsetHeight<r.viewport.scrollHeight;o(s?u:d)}},10);return Wh(r.viewport,l),Wh(r.content,l),x.jsx($i,{present:n||i,children:x.jsx(I5,{"data-state":i?"visible":"hidden",...a,ref:t})})}),I5=T.forwardRef((e,t)=>{const{orientation:r="vertical",...n}=e,a=po(ys,e.__scopeScrollArea),i=T.useRef(null),o=T.useRef(0),[s,l]=T.useState({content:0,viewport:0,scrollbar:{size:0,paddingStart:0,paddingEnd:0}}),u=Pte(s.viewport,s.content),d={...n,sizes:s,onSizesChange:l,hasThumb:u>0&&u<1,onThumbChange:f=>i.current=f,onThumbPointerUp:()=>o.current=0,onThumbPointerDown:f=>o.current=f};function p(f,m){return ZDe(f,o.current,s,m)}return r==="horizontal"?x.jsx(VDe,{...d,ref:t,onThumbPositionChange:()=>{if(a.viewport&&i.current){const f=a.viewport.scrollLeft,m=YB(f,s,a.dir);i.current.style.transform=`translate3d(${m}px, 0, 0)`}},onWheelScroll:f=>{a.viewport&&(a.viewport.scrollLeft=f)},onDragScroll:f=>{a.viewport&&(a.viewport.scrollLeft=p(f,a.dir))}}):r==="vertical"?x.jsx(KDe,{...d,ref:t,onThumbPositionChange:()=>{if(a.viewport&&i.current){const f=a.viewport.scrollTop,m=YB(f,s);i.current.style.transform=`translate3d(0, ${m}px, 0)`}},onWheelScroll:f=>{a.viewport&&(a.viewport.scrollTop=f)},onDragScroll:f=>{a.viewport&&(a.viewport.scrollTop=p(f))}}):null}),VDe=T.forwardRef((e,t)=>{const{sizes:r,onSizesChange:n,...a}=e,i=po(ys,e.__scopeScrollArea),[o,s]=T.useState(),l=T.useRef(null),u=Rr(t,l,i.onScrollbarXChange);return T.useEffect(()=>{l.current&&s(getComputedStyle(l.current))},[l]),x.jsx(Ite,{"data-orientation":"horizontal",...a,ref:u,sizes:r,style:{bottom:0,left:i.dir==="rtl"?"var(--radix-scroll-area-corner-width)":0,right:i.dir==="ltr"?"var(--radix-scroll-area-corner-width)":0,"--radix-scroll-area-thumb-width":g2(r)+"px",...e.style},onThumbPointerDown:d=>e.onThumbPointerDown(d.x),onDragScroll:d=>e.onDragScroll(d.x),onWheelScroll:(d,p)=>{if(i.viewport){const f=i.viewport.scrollLeft+d.deltaX;e.onWheelScroll(f),Hte(f,p)&&d.preventDefault()}},onResize:()=>{l.current&&i.viewport&&o&&n({content:i.viewport.scrollWidth,viewport:i.viewport.offsetWidth,scrollbar:{size:l.current.clientWidth,paddingStart:Cy(o.paddingLeft),paddingEnd:Cy(o.paddingRight)}})}})}),KDe=T.forwardRef((e,t)=>{const{sizes:r,onSizesChange:n,...a}=e,i=po(ys,e.__scopeScrollArea),[o,s]=T.useState(),l=T.useRef(null),u=Rr(t,l,i.onScrollbarYChange);return T.useEffect(()=>{l.current&&s(getComputedStyle(l.current))},[l]),x.jsx(Ite,{"data-orientation":"vertical",...a,ref:u,sizes:r,style:{top:0,right:i.dir==="ltr"?0:void 0,left:i.dir==="rtl"?0:void 0,bottom:"var(--radix-scroll-area-corner-height)","--radix-scroll-area-thumb-height":g2(r)+"px",...e.style},onThumbPointerDown:d=>e.onThumbPointerDown(d.y),onDragScroll:d=>e.onDragScroll(d.y),onWheelScroll:(d,p)=>{if(i.viewport){const f=i.viewport.scrollTop+d.deltaY;e.onWheelScroll(f),Hte(f,p)&&d.preventDefault()}},onResize:()=>{l.current&&i.viewport&&o&&n({content:i.viewport.scrollHeight,viewport:i.viewport.offsetHeight,scrollbar:{size:l.current.clientHeight,paddingStart:Cy(o.paddingTop),paddingEnd:Cy(o.paddingBottom)}})}})}),[YDe,Mte]=_te(ys),Ite=T.forwardRef((e,t)=>{const{__scopeScrollArea:r,sizes:n,hasThumb:a,onThumbChange:i,onThumbPointerUp:o,onThumbPointerDown:s,onThumbPositionChange:l,onDragScroll:u,onWheelScroll:d,onResize:p,...f}=e,m=po(ys,r),[b,y]=T.useState(null),S=Rr(t,I=>y(I)),w=T.useRef(null),E=T.useRef(""),k=m.viewport,_=n.content-n.viewport,R=Ga(d),C=Ga(l),F=b2(p,10);function L(I){if(w.current){const M=I.clientX-w.current.left,j=I.clientY-w.current.top;u({x:M,y:j})}}return T.useEffect(()=>{const I=M=>{const j=M.target;(b==null?void 0:b.contains(j))&&R(M,_)};return document.addEventListener("wheel",I,{passive:!1}),()=>document.removeEventListener("wheel",I,{passive:!1})},[k,b,_,R]),T.useEffect(C,[n,C]),Wh(b,F),Wh(m.content,F),x.jsx(YDe,{scope:r,scrollbar:b,hasThumb:a,onThumbChange:Ga(i),onThumbPointerUp:Ga(o),onThumbPositionChange:C,onThumbPointerDown:Ga(s),children:x.jsx(Kt.div,{...f,ref:S,style:{position:"absolute",...f.style},onPointerDown:$t(e.onPointerDown,I=>{I.button===0&&(I.target.setPointerCapture(I.pointerId),w.current=b.getBoundingClientRect(),E.current=document.body.style.webkitUserSelect,document.body.style.webkitUserSelect="none",m.viewport&&(m.viewport.style.scrollBehavior="auto"),L(I))}),onPointerMove:$t(e.onPointerMove,L),onPointerUp:$t(e.onPointerUp,I=>{const M=I.target;M.hasPointerCapture(I.pointerId)&&M.releasePointerCapture(I.pointerId),document.body.style.webkitUserSelect=E.current,m.viewport&&(m.viewport.style.scrollBehavior=""),w.current=null})})})}),ky="ScrollAreaThumb",Ote=T.forwardRef((e,t)=>{const{forceMount:r,...n}=e,a=Mte(ky,e.__scopeScrollArea);return x.jsx($i,{present:r||a.hasThumb,children:x.jsx(JDe,{ref:t,...n})})}),JDe=T.forwardRef((e,t)=>{const{__scopeScrollArea:r,style:n,...a}=e,i=po(ky,r),o=Mte(ky,r),{onThumbPositionChange:s}=o,l=Rr(t,p=>o.onThumbChange(p)),u=T.useRef(void 0),d=b2(()=>{u.current&&(u.current(),u.current=void 0)},100);return T.useEffect(()=>{const p=i.viewport;if(p){const f=()=>{if(d(),!u.current){const m=QDe(p,s);u.current=m,s()}};return s(),p.addEventListener("scroll",f),()=>p.removeEventListener("scroll",f)}},[i.viewport,d,s]),x.jsx(Kt.div,{"data-state":o.hasThumb?"visible":"hidden",...a,ref:l,style:{width:"var(--radix-scroll-area-thumb-width)",height:"var(--radix-scroll-area-thumb-height)",...n},onPointerDownCapture:$t(e.onPointerDownCapture,p=>{const m=p.target.getBoundingClientRect(),b=p.clientX-m.left,y=p.clientY-m.top;o.onThumbPointerDown({x:b,y})}),onPointerUp:$t(e.onPointerUp,o.onThumbPointerUp)})});Ote.displayName=ky;var O5="ScrollAreaCorner",Bte=T.forwardRef((e,t)=>{const r=po(O5,e.__scopeScrollArea),n=!!(r.scrollbarX&&r.scrollbarY);return r.type!=="scroll"&&n?x.jsx(XDe,{...e,ref:t}):null});Bte.displayName=O5;var XDe=T.forwardRef((e,t)=>{const{__scopeScrollArea:r,...n}=e,a=po(O5,r),[i,o]=T.useState(0),[s,l]=T.useState(0),u=!!(i&&s);return Wh(a.scrollbarX,()=>{var p;const d=((p=a.scrollbarX)==null?void 0:p.offsetHeight)||0;a.onCornerHeightChange(d),l(d)}),Wh(a.scrollbarY,()=>{var p;const d=((p=a.scrollbarY)==null?void 0:p.offsetWidth)||0;a.onCornerWidthChange(d),o(d)}),u?x.jsx(Kt.div,{...n,ref:t,style:{width:i,height:s,position:"absolute",right:a.dir==="ltr"?0:void 0,left:a.dir==="rtl"?0:void 0,bottom:0,...e.style}}):null});function Cy(e){return e?parseInt(e,10):0}function Pte(e,t){const r=e/t;return isNaN(r)?0:r}function g2(e){const t=Pte(e.viewport,e.content),r=e.scrollbar.paddingStart+e.scrollbar.paddingEnd,n=(e.scrollbar.size-r)*t;return Math.max(n,18)}function ZDe(e,t,r,n="ltr"){const a=g2(r),i=a/2,o=t||i,s=a-o,l=r.scrollbar.paddingStart+o,u=r.scrollbar.size-r.scrollbar.paddingEnd-s,d=r.content-r.viewport,p=n==="ltr"?[0,d]:[d*-1,0];return zte([l,u],p)(e)}function YB(e,t,r="ltr"){const n=g2(t),a=t.scrollbar.paddingStart+t.scrollbar.paddingEnd,i=t.scrollbar.size-a,o=t.content-t.viewport,s=i-n,l=r==="ltr"?[0,o]:[o*-1,0],u=aR(e,l);return zte([0,o],[0,s])(u)}function zte(e,t){return r=>{if(e[0]===e[1]||t[0]===t[1])return t[0];const n=(t[1]-t[0])/(e[1]-e[0]);return t[0]+n*(r-e[0])}}function Hte(e,t){return e>0&&e<t}var QDe=(e,t=()=>{})=>{let r={left:e.scrollLeft,top:e.scrollTop},n=0;return(function a(){const i={left:e.scrollLeft,top:e.scrollTop},o=r.left!==i.left,s=r.top!==i.top;(o||s)&&t(),r=i,n=window.requestAnimationFrame(a)})(),()=>window.cancelAnimationFrame(n)};function b2(e,t){const r=Ga(e),n=T.useRef(0);return T.useEffect(()=>()=>window.clearTimeout(n.current),[]),T.useCallback(()=>{window.clearTimeout(n.current),n.current=window.setTimeout(r,t)},[r,t])}function Wh(e,t){const r=Ga(t);Sa(()=>{let n=0;if(e){const a=new ResizeObserver(()=>{cancelAnimationFrame(n),n=window.requestAnimationFrame(r)});return a.observe(e),()=>{window.cancelAnimationFrame(n),a.unobserve(e)}}},[e,r])}var Ute=Rte,e5e=Nte,t5e=Bte;const B5=T.forwardRef(({className:e,children:t,...r},n)=>x.jsxs(Ute,{ref:n,className:lt("relative overflow-hidden",e),...r,children:[x.jsx(e5e,{className:"h-full w-full rounded-[inherit]",children:t}),x.jsx($te,{}),x.jsx(t5e,{})]}));B5.displayName=Ute.displayName;const $te=T.forwardRef(({className:e,orientation:t="vertical",...r},n)=>x.jsx(M5,{ref:n,orientation:t,className:lt("flex touch-none transition-colors select-none",t==="vertical"&&"h-full w-2.5 border-l border-l-transparent p-[1px]",t==="horizontal"&&"h-2.5 flex-col border-t border-t-transparent p-[1px]",e),...r,children:x.jsx(Ote,{className:"bg-border relative flex-1 rounded-full"})}));$te.displayName=M5.displayName;const r5e=({className:e})=>{const{t}=Sr(),r=pt.use.typeColorMap();return!r||r.size===0?null:x.jsxs(qc,{className:`p-2 max-w-xs ${e}`,children:[x.jsx("h3",{className:"text-sm font-medium mb-2",children:t("graphPanel.legend")}),x.jsx(B5,{className:"max-h-80",children:x.jsx("div",{className:"flex flex-col gap-1",children:Array.from(r.entries()).map(([n,a])=>x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx("div",{className:"w-4 h-4 rounded-full",style:{backgroundColor:a}}),x.jsx("span",{className:"text-xs truncate",title:n,children:t(`graphPanel.nodeTypes.${n.toLowerCase().replace(/\s+/g,"")}`,n)})]},n))})})]})},n5e=()=>{const{t:e}=Sr(),t=Ve.use.showLegend(),r=Ve.use.setShowLegend(),n=T.useCallback(()=>{r(!t)},[t,r]);return x.jsx(Rt,{variant:no,onClick:n,tooltip:e("graphPanel.sideBar.legendControl.toggleLegend"),size:"icon",children:x.jsx(VSe,{})})},a5e=e=>({allowInvalidContainer:!0,defaultNodeType:"default",defaultEdgeType:"curvedNoArrow",renderEdgeLabels:!1,edgeProgramClasses:{arrow:oee,curvedArrow:HTe,curvedNoArrow:c2()},nodeProgramClasses:{default:wTe,circel:Ag,point:XCe},labelGridCellSize:60,labelRenderedSizeThreshold:12,enableEdgeEvents:!0,labelColor:{color:e?U_:k8,attribute:"labelColor"},edgeLabelColor:{color:e?U_:k8,attribute:"labelColor"},edgeLabelSize:8,labelSize:12}),i5e=()=>{const e=mee(),t=zo(),[r,n]=T.useState(null);return T.useEffect(()=>{e({downNode:a=>{n(a.node),t.getGraph().setNodeAttribute(a.node,"highlighted",!0)},mousemovebody:a=>{if(!r)return;const i=t.viewportToGraph(a);t.getGraph().setNodeAttribute(r,"x",i.x),t.getGraph().setNodeAttribute(r,"y",i.y),a.preventSigmaDefault(),a.original.preventDefault(),a.original.stopPropagation()},mouseup:()=>{r&&(n(null),t.getGraph().removeNodeAttribute(r,"highlighted"))},mousedown:a=>{a.original.buttons!==0&&!t.getCustomBBox()&&t.setCustomBBox(t.getBBox())}})},[e,t,r]),null},o5e=()=>{const[e,t]=T.useState(!1),r=T.useRef(null),n=T.useRef(""),a=pt.use.selectedNode(),i=pt.use.focusedNode(),o=pt.use.moveToSelectedNode(),s=pt.use.isFetching(),l=Ve.use.showPropertyPanel(),u=Ve.use.showNodeSearchBar(),d=Ve.use.enableNodeDrag(),p=Ve.use.showLegend(),f=Ve.use.theme(),m=T.useMemo(()=>a5e(f==="dark"),[f]);T.useEffect(()=>{if(n.current&&n.current!==f){t(!0),console.log("Theme switching detected:",n.current,"->",f);const k=setTimeout(()=>{t(!1),console.log("Theme switching completed")},150);return()=>clearTimeout(k)}n.current=f,console.log("Initialized sigma settings for theme:",f)},[f]),T.useEffect(()=>()=>{const E=pt.getState().sigmaInstance;if(E)try{E.kill(),pt.getState().setSigmaInstance(null),console.log("Cleared sigma instance on Graphviewer unmount")}catch(k){console.error("Error cleaning up sigma instance:",k)}},[]);const b=T.useCallback(E=>{E===null?pt.getState().setFocusedNode(null):E.type==="nodes"&&pt.getState().setFocusedNode(E.id)},[]),y=T.useCallback(E=>{E===null?pt.getState().setSelectedNode(null):E.type==="nodes"&&pt.getState().setSelectedNode(E.id,!0)},[]),S=T.useMemo(()=>i??a,[i,a]),w=T.useMemo(()=>a?{type:"nodes",id:a}:null,[a]);return x.jsxs("div",{className:"relative h-full w-full overflow-hidden",children:[x.jsxs(qCe,{settings:m,className:"!bg-background !size-full overflow-hidden",ref:r,children:[x.jsx(rFe,{}),d&&x.jsx(i5e,{}),x.jsx(UTe,{node:S,move:o}),x.jsxs("div",{className:"absolute top-2 left-2 flex items-start gap-2",children:[x.jsx(wDe,{}),u&&!e&&x.jsx(SDe,{value:w,onFocus:b,onChange:y})]}),x.jsxs("div",{className:"bg-background/60 absolute bottom-2 left-2 flex flex-col rounded-xl border-2 backdrop-blur-lg",children:[x.jsx(tFe,{}),x.jsx(nFe,{}),x.jsx(aFe,{}),x.jsx(n5e,{}),x.jsx(eDe,{})]}),l&&x.jsx("div",{className:"absolute top-2 right-2 z-10",children:x.jsx(ODe,{})}),p&&x.jsx("div",{className:"absolute bottom-10 right-2 z-0",children:x.jsx(r5e,{className:"bg-background/60 backdrop-blur-lg"})}),x.jsx(UDe,{})]}),(s||e)&&x.jsx("div",{className:"absolute inset-0 flex items-center justify-center bg-background/80 z-10",children:x.jsxs("div",{className:"text-center",children:[x.jsx("div",{className:"mb-2 h-8 w-8 animate-spin rounded-full border-4 border-primary border-t-transparent mx-auto"}),x.jsx("p",{children:e?"Switching Theme...":"Loading Graph Data..."})]})})]})},Gte=T.forwardRef(({className:e,...t},r)=>x.jsx("div",{className:"relative w-full overflow-auto",children:x.jsx("table",{ref:r,className:lt("w-full caption-bottom text-sm",e),...t})}));Gte.displayName="Table";const jte=T.forwardRef(({className:e,...t},r)=>x.jsx("thead",{ref:r,className:lt("[&_tr]:border-b",e),...t}));jte.displayName="TableHeader";const qte=T.forwardRef(({className:e,...t},r)=>x.jsx("tbody",{ref:r,className:lt("[&_tr:last-child]:border-0",e),...t}));qte.displayName="TableBody";const s5e=T.forwardRef(({className:e,...t},r)=>x.jsx("tfoot",{ref:r,className:lt("bg-muted/50 border-t font-medium [&>tr]:last:border-b-0",e),...t}));s5e.displayName="TableFooter";const _R=T.forwardRef(({className:e,...t},r)=>x.jsx("tr",{ref:r,className:lt("hover:bg-muted/50 data-[state=selected]:bg-muted border-b transition-colors",e),...t}));_R.displayName="TableRow";const Qs=T.forwardRef(({className:e,...t},r)=>x.jsx("th",{ref:r,className:lt("text-muted-foreground h-10 px-2 text-left align-middle font-medium [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px]",e),...t}));Qs.displayName="TableHead";const el=T.forwardRef(({className:e,...t},r)=>x.jsx("td",{ref:r,className:lt("p-2 align-middle [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px]",e),...t}));el.displayName="TableCell";const l5e=T.forwardRef(({className:e,...t},r)=>x.jsx("caption",{ref:r,className:lt("text-muted-foreground mt-4 text-sm",e),...t}));l5e.displayName="TableCaption";function u5e({title:e,description:t,icon:r=cEe,action:n,className:a,...i}){return x.jsxs(qc,{className:lt("flex w-full flex-col items-center justify-center space-y-6 bg-transparent p-16",a),...i,children:[x.jsx("div",{className:"mr-4 shrink-0 rounded-full border border-dashed p-4",children:x.jsx(r,{className:"text-muted-foreground size-8","aria-hidden":"true"})}),x.jsxs("div",{className:"flex flex-col items-center gap-1.5 text-center",children:[x.jsx(Um,{children:e}),t?x.jsx(m2,{children:t}):null]}),n||null]})}var Xw={exports:{}},Zw,JB;function c5e(){if(JB)return Zw;JB=1;var e="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED";return Zw=e,Zw}var Qw,XB;function d5e(){if(XB)return Qw;XB=1;var e=c5e();function t(){}function r(){}return r.resetWarningCache=t,Qw=function(){function n(o,s,l,u,d,p){if(p!==e){var f=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw f.name="Invariant Violation",f}}n.isRequired=n;function a(){return n}var i={array:n,bigint:n,bool:n,func:n,number:n,object:n,string:n,symbol:n,any:n,arrayOf:a,element:n,elementType:n,instanceOf:a,node:n,objectOf:a,oneOf:a,oneOfType:a,shape:a,exact:a,checkPropTypes:r,resetWarningCache:t};return i.PropTypes=i,i},Qw}var ZB;function h5e(){return ZB||(ZB=1,Xw.exports=d5e()()),Xw.exports}var p5e=h5e();const Qr=jn(p5e),f5e=new Map([["1km","application/vnd.1000minds.decision-model+xml"],["3dml","text/vnd.in3d.3dml"],["3ds","image/x-3ds"],["3g2","video/3gpp2"],["3gp","video/3gp"],["3gpp","video/3gpp"],["3mf","model/3mf"],["7z","application/x-7z-compressed"],["7zip","application/x-7z-compressed"],["123","application/vnd.lotus-1-2-3"],["aab","application/x-authorware-bin"],["aac","audio/x-acc"],["aam","application/x-authorware-map"],["aas","application/x-authorware-seg"],["abw","application/x-abiword"],["ac","application/vnd.nokia.n-gage.ac+xml"],["ac3","audio/ac3"],["acc","application/vnd.americandynamics.acc"],["ace","application/x-ace-compressed"],["acu","application/vnd.acucobol"],["acutc","application/vnd.acucorp"],["adp","audio/adpcm"],["aep","application/vnd.audiograph"],["afm","application/x-font-type1"],["afp","application/vnd.ibm.modcap"],["ahead","application/vnd.ahead.space"],["ai","application/pdf"],["aif","audio/x-aiff"],["aifc","audio/x-aiff"],["aiff","audio/x-aiff"],["air","application/vnd.adobe.air-application-installer-package+zip"],["ait","application/vnd.dvb.ait"],["ami","application/vnd.amiga.ami"],["amr","audio/amr"],["apk","application/vnd.android.package-archive"],["apng","image/apng"],["appcache","text/cache-manifest"],["application","application/x-ms-application"],["apr","application/vnd.lotus-approach"],["arc","application/x-freearc"],["arj","application/x-arj"],["asc","application/pgp-signature"],["asf","video/x-ms-asf"],["asm","text/x-asm"],["aso","application/vnd.accpac.simply.aso"],["asx","video/x-ms-asf"],["atc","application/vnd.acucorp"],["atom","application/atom+xml"],["atomcat","application/atomcat+xml"],["atomdeleted","application/atomdeleted+xml"],["atomsvc","application/atomsvc+xml"],["atx","application/vnd.antix.game-component"],["au","audio/x-au"],["avi","video/x-msvideo"],["avif","image/avif"],["aw","application/applixware"],["azf","application/vnd.airzip.filesecure.azf"],["azs","application/vnd.airzip.filesecure.azs"],["azv","image/vnd.airzip.accelerator.azv"],["azw","application/vnd.amazon.ebook"],["b16","image/vnd.pco.b16"],["bat","application/x-msdownload"],["bcpio","application/x-bcpio"],["bdf","application/x-font-bdf"],["bdm","application/vnd.syncml.dm+wbxml"],["bdoc","application/x-bdoc"],["bed","application/vnd.realvnc.bed"],["bh2","application/vnd.fujitsu.oasysprs"],["bin","application/octet-stream"],["blb","application/x-blorb"],["blorb","application/x-blorb"],["bmi","application/vnd.bmi"],["bmml","application/vnd.balsamiq.bmml+xml"],["bmp","image/bmp"],["book","application/vnd.framemaker"],["box","application/vnd.previewsystems.box"],["boz","application/x-bzip2"],["bpk","application/octet-stream"],["bpmn","application/octet-stream"],["bsp","model/vnd.valve.source.compiled-map"],["btif","image/prs.btif"],["buffer","application/octet-stream"],["bz","application/x-bzip"],["bz2","application/x-bzip2"],["c","text/x-c"],["c4d","application/vnd.clonk.c4group"],["c4f","application/vnd.clonk.c4group"],["c4g","application/vnd.clonk.c4group"],["c4p","application/vnd.clonk.c4group"],["c4u","application/vnd.clonk.c4group"],["c11amc","application/vnd.cluetrust.cartomobile-config"],["c11amz","application/vnd.cluetrust.cartomobile-config-pkg"],["cab","application/vnd.ms-cab-compressed"],["caf","audio/x-caf"],["cap","application/vnd.tcpdump.pcap"],["car","application/vnd.curl.car"],["cat","application/vnd.ms-pki.seccat"],["cb7","application/x-cbr"],["cba","application/x-cbr"],["cbr","application/x-cbr"],["cbt","application/x-cbr"],["cbz","application/x-cbr"],["cc","text/x-c"],["cco","application/x-cocoa"],["cct","application/x-director"],["ccxml","application/ccxml+xml"],["cdbcmsg","application/vnd.contact.cmsg"],["cda","application/x-cdf"],["cdf","application/x-netcdf"],["cdfx","application/cdfx+xml"],["cdkey","application/vnd.mediastation.cdkey"],["cdmia","application/cdmi-capability"],["cdmic","application/cdmi-container"],["cdmid","application/cdmi-domain"],["cdmio","application/cdmi-object"],["cdmiq","application/cdmi-queue"],["cdr","application/cdr"],["cdx","chemical/x-cdx"],["cdxml","application/vnd.chemdraw+xml"],["cdy","application/vnd.cinderella"],["cer","application/pkix-cert"],["cfs","application/x-cfs-compressed"],["cgm","image/cgm"],["chat","application/x-chat"],["chm","application/vnd.ms-htmlhelp"],["chrt","application/vnd.kde.kchart"],["cif","chemical/x-cif"],["cii","application/vnd.anser-web-certificate-issue-initiation"],["cil","application/vnd.ms-artgalry"],["cjs","application/node"],["cla","application/vnd.claymore"],["class","application/octet-stream"],["clkk","application/vnd.crick.clicker.keyboard"],["clkp","application/vnd.crick.clicker.palette"],["clkt","application/vnd.crick.clicker.template"],["clkw","application/vnd.crick.clicker.wordbank"],["clkx","application/vnd.crick.clicker"],["clp","application/x-msclip"],["cmc","application/vnd.cosmocaller"],["cmdf","chemical/x-cmdf"],["cml","chemical/x-cml"],["cmp","application/vnd.yellowriver-custom-menu"],["cmx","image/x-cmx"],["cod","application/vnd.rim.cod"],["coffee","text/coffeescript"],["com","application/x-msdownload"],["conf","text/plain"],["cpio","application/x-cpio"],["cpp","text/x-c"],["cpt","application/mac-compactpro"],["crd","application/x-mscardfile"],["crl","application/pkix-crl"],["crt","application/x-x509-ca-cert"],["crx","application/x-chrome-extension"],["cryptonote","application/vnd.rig.cryptonote"],["csh","application/x-csh"],["csl","application/vnd.citationstyles.style+xml"],["csml","chemical/x-csml"],["csp","application/vnd.commonspace"],["csr","application/octet-stream"],["css","text/css"],["cst","application/x-director"],["csv","text/csv"],["cu","application/cu-seeme"],["curl","text/vnd.curl"],["cww","application/prs.cww"],["cxt","application/x-director"],["cxx","text/x-c"],["dae","model/vnd.collada+xml"],["daf","application/vnd.mobius.daf"],["dart","application/vnd.dart"],["dataless","application/vnd.fdsn.seed"],["davmount","application/davmount+xml"],["dbf","application/vnd.dbf"],["dbk","application/docbook+xml"],["dcr","application/x-director"],["dcurl","text/vnd.curl.dcurl"],["dd2","application/vnd.oma.dd2+xml"],["ddd","application/vnd.fujixerox.ddd"],["ddf","application/vnd.syncml.dmddf+xml"],["dds","image/vnd.ms-dds"],["deb","application/x-debian-package"],["def","text/plain"],["deploy","application/octet-stream"],["der","application/x-x509-ca-cert"],["dfac","application/vnd.dreamfactory"],["dgc","application/x-dgc-compressed"],["dic","text/x-c"],["dir","application/x-director"],["dis","application/vnd.mobius.dis"],["disposition-notification","message/disposition-notification"],["dist","application/octet-stream"],["distz","application/octet-stream"],["djv","image/vnd.djvu"],["djvu","image/vnd.djvu"],["dll","application/octet-stream"],["dmg","application/x-apple-diskimage"],["dmn","application/octet-stream"],["dmp","application/vnd.tcpdump.pcap"],["dms","application/octet-stream"],["dna","application/vnd.dna"],["doc","application/msword"],["docm","application/vnd.ms-word.template.macroEnabled.12"],["docx","application/vnd.openxmlformats-officedocument.wordprocessingml.document"],["dot","application/msword"],["dotm","application/vnd.ms-word.template.macroEnabled.12"],["dotx","application/vnd.openxmlformats-officedocument.wordprocessingml.template"],["dp","application/vnd.osgi.dp"],["dpg","application/vnd.dpgraph"],["dra","audio/vnd.dra"],["drle","image/dicom-rle"],["dsc","text/prs.lines.tag"],["dssc","application/dssc+der"],["dtb","application/x-dtbook+xml"],["dtd","application/xml-dtd"],["dts","audio/vnd.dts"],["dtshd","audio/vnd.dts.hd"],["dump","application/octet-stream"],["dvb","video/vnd.dvb.file"],["dvi","application/x-dvi"],["dwd","application/atsc-dwd+xml"],["dwf","model/vnd.dwf"],["dwg","image/vnd.dwg"],["dxf","image/vnd.dxf"],["dxp","application/vnd.spotfire.dxp"],["dxr","application/x-director"],["ear","application/java-archive"],["ecelp4800","audio/vnd.nuera.ecelp4800"],["ecelp7470","audio/vnd.nuera.ecelp7470"],["ecelp9600","audio/vnd.nuera.ecelp9600"],["ecma","application/ecmascript"],["edm","application/vnd.novadigm.edm"],["edx","application/vnd.novadigm.edx"],["efif","application/vnd.picsel"],["ei6","application/vnd.pg.osasli"],["elc","application/octet-stream"],["emf","image/emf"],["eml","message/rfc822"],["emma","application/emma+xml"],["emotionml","application/emotionml+xml"],["emz","application/x-msmetafile"],["eol","audio/vnd.digital-winds"],["eot","application/vnd.ms-fontobject"],["eps","application/postscript"],["epub","application/epub+zip"],["es","application/ecmascript"],["es3","application/vnd.eszigno3+xml"],["esa","application/vnd.osgi.subsystem"],["esf","application/vnd.epson.esf"],["et3","application/vnd.eszigno3+xml"],["etx","text/x-setext"],["eva","application/x-eva"],["evy","application/x-envoy"],["exe","application/octet-stream"],["exi","application/exi"],["exp","application/express"],["exr","image/aces"],["ext","application/vnd.novadigm.ext"],["ez","application/andrew-inset"],["ez2","application/vnd.ezpix-album"],["ez3","application/vnd.ezpix-package"],["f","text/x-fortran"],["f4v","video/mp4"],["f77","text/x-fortran"],["f90","text/x-fortran"],["fbs","image/vnd.fastbidsheet"],["fcdt","application/vnd.adobe.formscentral.fcdt"],["fcs","application/vnd.isac.fcs"],["fdf","application/vnd.fdf"],["fdt","application/fdt+xml"],["fe_launch","application/vnd.denovo.fcselayout-link"],["fg5","application/vnd.fujitsu.oasysgp"],["fgd","application/x-director"],["fh","image/x-freehand"],["fh4","image/x-freehand"],["fh5","image/x-freehand"],["fh7","image/x-freehand"],["fhc","image/x-freehand"],["fig","application/x-xfig"],["fits","image/fits"],["flac","audio/x-flac"],["fli","video/x-fli"],["flo","application/vnd.micrografx.flo"],["flv","video/x-flv"],["flw","application/vnd.kde.kivio"],["flx","text/vnd.fmi.flexstor"],["fly","text/vnd.fly"],["fm","application/vnd.framemaker"],["fnc","application/vnd.frogans.fnc"],["fo","application/vnd.software602.filler.form+xml"],["for","text/x-fortran"],["fpx","image/vnd.fpx"],["frame","application/vnd.framemaker"],["fsc","application/vnd.fsc.weblaunch"],["fst","image/vnd.fst"],["ftc","application/vnd.fluxtime.clip"],["fti","application/vnd.anser-web-funds-transfer-initiation"],["fvt","video/vnd.fvt"],["fxp","application/vnd.adobe.fxp"],["fxpl","application/vnd.adobe.fxp"],["fzs","application/vnd.fuzzysheet"],["g2w","application/vnd.geoplan"],["g3","image/g3fax"],["g3w","application/vnd.geospace"],["gac","application/vnd.groove-account"],["gam","application/x-tads"],["gbr","application/rpki-ghostbusters"],["gca","application/x-gca-compressed"],["gdl","model/vnd.gdl"],["gdoc","application/vnd.google-apps.document"],["geo","application/vnd.dynageo"],["geojson","application/geo+json"],["gex","application/vnd.geometry-explorer"],["ggb","application/vnd.geogebra.file"],["ggt","application/vnd.geogebra.tool"],["ghf","application/vnd.groove-help"],["gif","image/gif"],["gim","application/vnd.groove-identity-message"],["glb","model/gltf-binary"],["gltf","model/gltf+json"],["gml","application/gml+xml"],["gmx","application/vnd.gmx"],["gnumeric","application/x-gnumeric"],["gpg","application/gpg-keys"],["gph","application/vnd.flographit"],["gpx","application/gpx+xml"],["gqf","application/vnd.grafeq"],["gqs","application/vnd.grafeq"],["gram","application/srgs"],["gramps","application/x-gramps-xml"],["gre","application/vnd.geometry-explorer"],["grv","application/vnd.groove-injector"],["grxml","application/srgs+xml"],["gsf","application/x-font-ghostscript"],["gsheet","application/vnd.google-apps.spreadsheet"],["gslides","application/vnd.google-apps.presentation"],["gtar","application/x-gtar"],["gtm","application/vnd.groove-tool-message"],["gtw","model/vnd.gtw"],["gv","text/vnd.graphviz"],["gxf","application/gxf"],["gxt","application/vnd.geonext"],["gz","application/gzip"],["gzip","application/gzip"],["h","text/x-c"],["h261","video/h261"],["h263","video/h263"],["h264","video/h264"],["hal","application/vnd.hal+xml"],["hbci","application/vnd.hbci"],["hbs","text/x-handlebars-template"],["hdd","application/x-virtualbox-hdd"],["hdf","application/x-hdf"],["heic","image/heic"],["heics","image/heic-sequence"],["heif","image/heif"],["heifs","image/heif-sequence"],["hej2","image/hej2k"],["held","application/atsc-held+xml"],["hh","text/x-c"],["hjson","application/hjson"],["hlp","application/winhlp"],["hpgl","application/vnd.hp-hpgl"],["hpid","application/vnd.hp-hpid"],["hps","application/vnd.hp-hps"],["hqx","application/mac-binhex40"],["hsj2","image/hsj2"],["htc","text/x-component"],["htke","application/vnd.kenameaapp"],["htm","text/html"],["html","text/html"],["hvd","application/vnd.yamaha.hv-dic"],["hvp","application/vnd.yamaha.hv-voice"],["hvs","application/vnd.yamaha.hv-script"],["i2g","application/vnd.intergeo"],["icc","application/vnd.iccprofile"],["ice","x-conference/x-cooltalk"],["icm","application/vnd.iccprofile"],["ico","image/x-icon"],["ics","text/calendar"],["ief","image/ief"],["ifb","text/calendar"],["ifm","application/vnd.shana.informed.formdata"],["iges","model/iges"],["igl","application/vnd.igloader"],["igm","application/vnd.insors.igm"],["igs","model/iges"],["igx","application/vnd.micrografx.igx"],["iif","application/vnd.shana.informed.interchange"],["img","application/octet-stream"],["imp","application/vnd.accpac.simply.imp"],["ims","application/vnd.ms-ims"],["in","text/plain"],["ini","text/plain"],["ink","application/inkml+xml"],["inkml","application/inkml+xml"],["install","application/x-install-instructions"],["iota","application/vnd.astraea-software.iota"],["ipfix","application/ipfix"],["ipk","application/vnd.shana.informed.package"],["irm","application/vnd.ibm.rights-management"],["irp","application/vnd.irepository.package+xml"],["iso","application/x-iso9660-image"],["itp","application/vnd.shana.informed.formtemplate"],["its","application/its+xml"],["ivp","application/vnd.immervision-ivp"],["ivu","application/vnd.immervision-ivu"],["jad","text/vnd.sun.j2me.app-descriptor"],["jade","text/jade"],["jam","application/vnd.jam"],["jar","application/java-archive"],["jardiff","application/x-java-archive-diff"],["java","text/x-java-source"],["jhc","image/jphc"],["jisp","application/vnd.jisp"],["jls","image/jls"],["jlt","application/vnd.hp-jlyt"],["jng","image/x-jng"],["jnlp","application/x-java-jnlp-file"],["joda","application/vnd.joost.joda-archive"],["jp2","image/jp2"],["jpe","image/jpeg"],["jpeg","image/jpeg"],["jpf","image/jpx"],["jpg","image/jpeg"],["jpg2","image/jp2"],["jpgm","video/jpm"],["jpgv","video/jpeg"],["jph","image/jph"],["jpm","video/jpm"],["jpx","image/jpx"],["js","application/javascript"],["json","application/json"],["json5","application/json5"],["jsonld","application/ld+json"],["jsonl","application/jsonl"],["jsonml","application/jsonml+json"],["jsx","text/jsx"],["jxr","image/jxr"],["jxra","image/jxra"],["jxrs","image/jxrs"],["jxs","image/jxs"],["jxsc","image/jxsc"],["jxsi","image/jxsi"],["jxss","image/jxss"],["kar","audio/midi"],["karbon","application/vnd.kde.karbon"],["kdb","application/octet-stream"],["kdbx","application/x-keepass2"],["key","application/x-iwork-keynote-sffkey"],["kfo","application/vnd.kde.kformula"],["kia","application/vnd.kidspiration"],["kml","application/vnd.google-earth.kml+xml"],["kmz","application/vnd.google-earth.kmz"],["kne","application/vnd.kinar"],["knp","application/vnd.kinar"],["kon","application/vnd.kde.kontour"],["kpr","application/vnd.kde.kpresenter"],["kpt","application/vnd.kde.kpresenter"],["kpxx","application/vnd.ds-keypoint"],["ksp","application/vnd.kde.kspread"],["ktr","application/vnd.kahootz"],["ktx","image/ktx"],["ktx2","image/ktx2"],["ktz","application/vnd.kahootz"],["kwd","application/vnd.kde.kword"],["kwt","application/vnd.kde.kword"],["lasxml","application/vnd.las.las+xml"],["latex","application/x-latex"],["lbd","application/vnd.llamagraphics.life-balance.desktop"],["lbe","application/vnd.llamagraphics.life-balance.exchange+xml"],["les","application/vnd.hhe.lesson-player"],["less","text/less"],["lgr","application/lgr+xml"],["lha","application/octet-stream"],["link66","application/vnd.route66.link66+xml"],["list","text/plain"],["list3820","application/vnd.ibm.modcap"],["listafp","application/vnd.ibm.modcap"],["litcoffee","text/coffeescript"],["lnk","application/x-ms-shortcut"],["log","text/plain"],["lostxml","application/lost+xml"],["lrf","application/octet-stream"],["lrm","application/vnd.ms-lrm"],["ltf","application/vnd.frogans.ltf"],["lua","text/x-lua"],["luac","application/x-lua-bytecode"],["lvp","audio/vnd.lucent.voice"],["lwp","application/vnd.lotus-wordpro"],["lzh","application/octet-stream"],["m1v","video/mpeg"],["m2a","audio/mpeg"],["m2v","video/mpeg"],["m3a","audio/mpeg"],["m3u","text/plain"],["m3u8","application/vnd.apple.mpegurl"],["m4a","audio/x-m4a"],["m4p","application/mp4"],["m4s","video/iso.segment"],["m4u","application/vnd.mpegurl"],["m4v","video/x-m4v"],["m13","application/x-msmediaview"],["m14","application/x-msmediaview"],["m21","application/mp21"],["ma","application/mathematica"],["mads","application/mads+xml"],["maei","application/mmt-aei+xml"],["mag","application/vnd.ecowin.chart"],["maker","application/vnd.framemaker"],["man","text/troff"],["manifest","text/cache-manifest"],["map","application/json"],["mar","application/octet-stream"],["markdown","text/markdown"],["mathml","application/mathml+xml"],["mb","application/mathematica"],["mbk","application/vnd.mobius.mbk"],["mbox","application/mbox"],["mc1","application/vnd.medcalcdata"],["mcd","application/vnd.mcd"],["mcurl","text/vnd.curl.mcurl"],["md","text/markdown"],["mdb","application/x-msaccess"],["mdi","image/vnd.ms-modi"],["mdx","text/mdx"],["me","text/troff"],["mesh","model/mesh"],["meta4","application/metalink4+xml"],["metalink","application/metalink+xml"],["mets","application/mets+xml"],["mfm","application/vnd.mfmp"],["mft","application/rpki-manifest"],["mgp","application/vnd.osgeo.mapguide.package"],["mgz","application/vnd.proteus.magazine"],["mid","audio/midi"],["midi","audio/midi"],["mie","application/x-mie"],["mif","application/vnd.mif"],["mime","message/rfc822"],["mj2","video/mj2"],["mjp2","video/mj2"],["mjs","application/javascript"],["mk3d","video/x-matroska"],["mka","audio/x-matroska"],["mkd","text/x-markdown"],["mks","video/x-matroska"],["mkv","video/x-matroska"],["mlp","application/vnd.dolby.mlp"],["mmd","application/vnd.chipnuts.karaoke-mmd"],["mmf","application/vnd.smaf"],["mml","text/mathml"],["mmr","image/vnd.fujixerox.edmics-mmr"],["mng","video/x-mng"],["mny","application/x-msmoney"],["mobi","application/x-mobipocket-ebook"],["mods","application/mods+xml"],["mov","video/quicktime"],["movie","video/x-sgi-movie"],["mp2","audio/mpeg"],["mp2a","audio/mpeg"],["mp3","audio/mpeg"],["mp4","video/mp4"],["mp4a","audio/mp4"],["mp4s","application/mp4"],["mp4v","video/mp4"],["mp21","application/mp21"],["mpc","application/vnd.mophun.certificate"],["mpd","application/dash+xml"],["mpe","video/mpeg"],["mpeg","video/mpeg"],["mpg","video/mpeg"],["mpg4","video/mp4"],["mpga","audio/mpeg"],["mpkg","application/vnd.apple.installer+xml"],["mpm","application/vnd.blueice.multipass"],["mpn","application/vnd.mophun.application"],["mpp","application/vnd.ms-project"],["mpt","application/vnd.ms-project"],["mpy","application/vnd.ibm.minipay"],["mqy","application/vnd.mobius.mqy"],["mrc","application/marc"],["mrcx","application/marcxml+xml"],["ms","text/troff"],["mscml","application/mediaservercontrol+xml"],["mseed","application/vnd.fdsn.mseed"],["mseq","application/vnd.mseq"],["msf","application/vnd.epson.msf"],["msg","application/vnd.ms-outlook"],["msh","model/mesh"],["msi","application/x-msdownload"],["msl","application/vnd.mobius.msl"],["msm","application/octet-stream"],["msp","application/octet-stream"],["msty","application/vnd.muvee.style"],["mtl","model/mtl"],["mts","model/vnd.mts"],["mus","application/vnd.musician"],["musd","application/mmt-usd+xml"],["musicxml","application/vnd.recordare.musicxml+xml"],["mvb","application/x-msmediaview"],["mvt","application/vnd.mapbox-vector-tile"],["mwf","application/vnd.mfer"],["mxf","application/mxf"],["mxl","application/vnd.recordare.musicxml"],["mxmf","audio/mobile-xmf"],["mxml","application/xv+xml"],["mxs","application/vnd.triscape.mxs"],["mxu","video/vnd.mpegurl"],["n-gage","application/vnd.nokia.n-gage.symbian.install"],["n3","text/n3"],["nb","application/mathematica"],["nbp","application/vnd.wolfram.player"],["nc","application/x-netcdf"],["ncx","application/x-dtbncx+xml"],["nfo","text/x-nfo"],["ngdat","application/vnd.nokia.n-gage.data"],["nitf","application/vnd.nitf"],["nlu","application/vnd.neurolanguage.nlu"],["nml","application/vnd.enliven"],["nnd","application/vnd.noblenet-directory"],["nns","application/vnd.noblenet-sealer"],["nnw","application/vnd.noblenet-web"],["npx","image/vnd.net-fpx"],["nq","application/n-quads"],["nsc","application/x-conference"],["nsf","application/vnd.lotus-notes"],["nt","application/n-triples"],["ntf","application/vnd.nitf"],["numbers","application/x-iwork-numbers-sffnumbers"],["nzb","application/x-nzb"],["oa2","application/vnd.fujitsu.oasys2"],["oa3","application/vnd.fujitsu.oasys3"],["oas","application/vnd.fujitsu.oasys"],["obd","application/x-msbinder"],["obgx","application/vnd.openblox.game+xml"],["obj","model/obj"],["oda","application/oda"],["odb","application/vnd.oasis.opendocument.database"],["odc","application/vnd.oasis.opendocument.chart"],["odf","application/vnd.oasis.opendocument.formula"],["odft","application/vnd.oasis.opendocument.formula-template"],["odg","application/vnd.oasis.opendocument.graphics"],["odi","application/vnd.oasis.opendocument.image"],["odm","application/vnd.oasis.opendocument.text-master"],["odp","application/vnd.oasis.opendocument.presentation"],["ods","application/vnd.oasis.opendocument.spreadsheet"],["odt","application/vnd.oasis.opendocument.text"],["oga","audio/ogg"],["ogex","model/vnd.opengex"],["ogg","audio/ogg"],["ogv","video/ogg"],["ogx","application/ogg"],["omdoc","application/omdoc+xml"],["onepkg","application/onenote"],["onetmp","application/onenote"],["onetoc","application/onenote"],["onetoc2","application/onenote"],["opf","application/oebps-package+xml"],["opml","text/x-opml"],["oprc","application/vnd.palm"],["opus","audio/ogg"],["org","text/x-org"],["osf","application/vnd.yamaha.openscoreformat"],["osfpvg","application/vnd.yamaha.openscoreformat.osfpvg+xml"],["osm","application/vnd.openstreetmap.data+xml"],["otc","application/vnd.oasis.opendocument.chart-template"],["otf","font/otf"],["otg","application/vnd.oasis.opendocument.graphics-template"],["oth","application/vnd.oasis.opendocument.text-web"],["oti","application/vnd.oasis.opendocument.image-template"],["otp","application/vnd.oasis.opendocument.presentation-template"],["ots","application/vnd.oasis.opendocument.spreadsheet-template"],["ott","application/vnd.oasis.opendocument.text-template"],["ova","application/x-virtualbox-ova"],["ovf","application/x-virtualbox-ovf"],["owl","application/rdf+xml"],["oxps","application/oxps"],["oxt","application/vnd.openofficeorg.extension"],["p","text/x-pascal"],["p7a","application/x-pkcs7-signature"],["p7b","application/x-pkcs7-certificates"],["p7c","application/pkcs7-mime"],["p7m","application/pkcs7-mime"],["p7r","application/x-pkcs7-certreqresp"],["p7s","application/pkcs7-signature"],["p8","application/pkcs8"],["p10","application/x-pkcs10"],["p12","application/x-pkcs12"],["pac","application/x-ns-proxy-autoconfig"],["pages","application/x-iwork-pages-sffpages"],["pas","text/x-pascal"],["paw","application/vnd.pawaafile"],["pbd","application/vnd.powerbuilder6"],["pbm","image/x-portable-bitmap"],["pcap","application/vnd.tcpdump.pcap"],["pcf","application/x-font-pcf"],["pcl","application/vnd.hp-pcl"],["pclxl","application/vnd.hp-pclxl"],["pct","image/x-pict"],["pcurl","application/vnd.curl.pcurl"],["pcx","image/x-pcx"],["pdb","application/x-pilot"],["pde","text/x-processing"],["pdf","application/pdf"],["pem","application/x-x509-user-cert"],["pfa","application/x-font-type1"],["pfb","application/x-font-type1"],["pfm","application/x-font-type1"],["pfr","application/font-tdpfr"],["pfx","application/x-pkcs12"],["pgm","image/x-portable-graymap"],["pgn","application/x-chess-pgn"],["pgp","application/pgp"],["php","application/x-httpd-php"],["php3","application/x-httpd-php"],["php4","application/x-httpd-php"],["phps","application/x-httpd-php-source"],["phtml","application/x-httpd-php"],["pic","image/x-pict"],["pkg","application/octet-stream"],["pki","application/pkixcmp"],["pkipath","application/pkix-pkipath"],["pkpass","application/vnd.apple.pkpass"],["pl","application/x-perl"],["plb","application/vnd.3gpp.pic-bw-large"],["plc","application/vnd.mobius.plc"],["plf","application/vnd.pocketlearn"],["pls","application/pls+xml"],["pm","application/x-perl"],["pml","application/vnd.ctc-posml"],["png","image/png"],["pnm","image/x-portable-anymap"],["portpkg","application/vnd.macports.portpkg"],["pot","application/vnd.ms-powerpoint"],["potm","application/vnd.ms-powerpoint.presentation.macroEnabled.12"],["potx","application/vnd.openxmlformats-officedocument.presentationml.template"],["ppa","application/vnd.ms-powerpoint"],["ppam","application/vnd.ms-powerpoint.addin.macroEnabled.12"],["ppd","application/vnd.cups-ppd"],["ppm","image/x-portable-pixmap"],["pps","application/vnd.ms-powerpoint"],["ppsm","application/vnd.ms-powerpoint.slideshow.macroEnabled.12"],["ppsx","application/vnd.openxmlformats-officedocument.presentationml.slideshow"],["ppt","application/powerpoint"],["pptm","application/vnd.ms-powerpoint.presentation.macroEnabled.12"],["pptx","application/vnd.openxmlformats-officedocument.presentationml.presentation"],["pqa","application/vnd.palm"],["prc","application/x-pilot"],["pre","application/vnd.lotus-freelance"],["prf","application/pics-rules"],["provx","application/provenance+xml"],["ps","application/postscript"],["psb","application/vnd.3gpp.pic-bw-small"],["psd","application/x-photoshop"],["psf","application/x-font-linux-psf"],["pskcxml","application/pskc+xml"],["pti","image/prs.pti"],["ptid","application/vnd.pvi.ptid1"],["pub","application/x-mspublisher"],["pvb","application/vnd.3gpp.pic-bw-var"],["pwn","application/vnd.3m.post-it-notes"],["pya","audio/vnd.ms-playready.media.pya"],["pyv","video/vnd.ms-playready.media.pyv"],["qam","application/vnd.epson.quickanime"],["qbo","application/vnd.intu.qbo"],["qfx","application/vnd.intu.qfx"],["qps","application/vnd.publishare-delta-tree"],["qt","video/quicktime"],["qwd","application/vnd.quark.quarkxpress"],["qwt","application/vnd.quark.quarkxpress"],["qxb","application/vnd.quark.quarkxpress"],["qxd","application/vnd.quark.quarkxpress"],["qxl","application/vnd.quark.quarkxpress"],["qxt","application/vnd.quark.quarkxpress"],["ra","audio/x-realaudio"],["ram","audio/x-pn-realaudio"],["raml","application/raml+yaml"],["rapd","application/route-apd+xml"],["rar","application/x-rar"],["ras","image/x-cmu-raster"],["rcprofile","application/vnd.ipunplugged.rcprofile"],["rdf","application/rdf+xml"],["rdz","application/vnd.data-vision.rdz"],["relo","application/p2p-overlay+xml"],["rep","application/vnd.businessobjects"],["res","application/x-dtbresource+xml"],["rgb","image/x-rgb"],["rif","application/reginfo+xml"],["rip","audio/vnd.rip"],["ris","application/x-research-info-systems"],["rl","application/resource-lists+xml"],["rlc","image/vnd.fujixerox.edmics-rlc"],["rld","application/resource-lists-diff+xml"],["rm","audio/x-pn-realaudio"],["rmi","audio/midi"],["rmp","audio/x-pn-realaudio-plugin"],["rms","application/vnd.jcp.javame.midlet-rms"],["rmvb","application/vnd.rn-realmedia-vbr"],["rnc","application/relax-ng-compact-syntax"],["rng","application/xml"],["roa","application/rpki-roa"],["roff","text/troff"],["rp9","application/vnd.cloanto.rp9"],["rpm","audio/x-pn-realaudio-plugin"],["rpss","application/vnd.nokia.radio-presets"],["rpst","application/vnd.nokia.radio-preset"],["rq","application/sparql-query"],["rs","application/rls-services+xml"],["rsa","application/x-pkcs7"],["rsat","application/atsc-rsat+xml"],["rsd","application/rsd+xml"],["rsheet","application/urc-ressheet+xml"],["rss","application/rss+xml"],["rtf","text/rtf"],["rtx","text/richtext"],["run","application/x-makeself"],["rusd","application/route-usd+xml"],["rv","video/vnd.rn-realvideo"],["s","text/x-asm"],["s3m","audio/s3m"],["saf","application/vnd.yamaha.smaf-audio"],["sass","text/x-sass"],["sbml","application/sbml+xml"],["sc","application/vnd.ibm.secure-container"],["scd","application/x-msschedule"],["scm","application/vnd.lotus-screencam"],["scq","application/scvp-cv-request"],["scs","application/scvp-cv-response"],["scss","text/x-scss"],["scurl","text/vnd.curl.scurl"],["sda","application/vnd.stardivision.draw"],["sdc","application/vnd.stardivision.calc"],["sdd","application/vnd.stardivision.impress"],["sdkd","application/vnd.solent.sdkm+xml"],["sdkm","application/vnd.solent.sdkm+xml"],["sdp","application/sdp"],["sdw","application/vnd.stardivision.writer"],["sea","application/octet-stream"],["see","application/vnd.seemail"],["seed","application/vnd.fdsn.seed"],["sema","application/vnd.sema"],["semd","application/vnd.semd"],["semf","application/vnd.semf"],["senmlx","application/senml+xml"],["sensmlx","application/sensml+xml"],["ser","application/java-serialized-object"],["setpay","application/set-payment-initiation"],["setreg","application/set-registration-initiation"],["sfd-hdstx","application/vnd.hydrostatix.sof-data"],["sfs","application/vnd.spotfire.sfs"],["sfv","text/x-sfv"],["sgi","image/sgi"],["sgl","application/vnd.stardivision.writer-global"],["sgm","text/sgml"],["sgml","text/sgml"],["sh","application/x-sh"],["shar","application/x-shar"],["shex","text/shex"],["shf","application/shf+xml"],["shtml","text/html"],["sid","image/x-mrsid-image"],["sieve","application/sieve"],["sig","application/pgp-signature"],["sil","audio/silk"],["silo","model/mesh"],["sis","application/vnd.symbian.install"],["sisx","application/vnd.symbian.install"],["sit","application/x-stuffit"],["sitx","application/x-stuffitx"],["siv","application/sieve"],["skd","application/vnd.koan"],["skm","application/vnd.koan"],["skp","application/vnd.koan"],["skt","application/vnd.koan"],["sldm","application/vnd.ms-powerpoint.slide.macroenabled.12"],["sldx","application/vnd.openxmlformats-officedocument.presentationml.slide"],["slim","text/slim"],["slm","text/slim"],["sls","application/route-s-tsid+xml"],["slt","application/vnd.epson.salt"],["sm","application/vnd.stepmania.stepchart"],["smf","application/vnd.stardivision.math"],["smi","application/smil"],["smil","application/smil"],["smv","video/x-smv"],["smzip","application/vnd.stepmania.package"],["snd","audio/basic"],["snf","application/x-font-snf"],["so","application/octet-stream"],["spc","application/x-pkcs7-certificates"],["spdx","text/spdx"],["spf","application/vnd.yamaha.smaf-phrase"],["spl","application/x-futuresplash"],["spot","text/vnd.in3d.spot"],["spp","application/scvp-vp-response"],["spq","application/scvp-vp-request"],["spx","audio/ogg"],["sql","application/x-sql"],["src","application/x-wais-source"],["srt","application/x-subrip"],["sru","application/sru+xml"],["srx","application/sparql-results+xml"],["ssdl","application/ssdl+xml"],["sse","application/vnd.kodak-descriptor"],["ssf","application/vnd.epson.ssf"],["ssml","application/ssml+xml"],["sst","application/octet-stream"],["st","application/vnd.sailingtracker.track"],["stc","application/vnd.sun.xml.calc.template"],["std","application/vnd.sun.xml.draw.template"],["stf","application/vnd.wt.stf"],["sti","application/vnd.sun.xml.impress.template"],["stk","application/hyperstudio"],["stl","model/stl"],["stpx","model/step+xml"],["stpxz","model/step-xml+zip"],["stpz","model/step+zip"],["str","application/vnd.pg.format"],["stw","application/vnd.sun.xml.writer.template"],["styl","text/stylus"],["stylus","text/stylus"],["sub","text/vnd.dvb.subtitle"],["sus","application/vnd.sus-calendar"],["susp","application/vnd.sus-calendar"],["sv4cpio","application/x-sv4cpio"],["sv4crc","application/x-sv4crc"],["svc","application/vnd.dvb.service"],["svd","application/vnd.svd"],["svg","image/svg+xml"],["svgz","image/svg+xml"],["swa","application/x-director"],["swf","application/x-shockwave-flash"],["swi","application/vnd.aristanetworks.swi"],["swidtag","application/swid+xml"],["sxc","application/vnd.sun.xml.calc"],["sxd","application/vnd.sun.xml.draw"],["sxg","application/vnd.sun.xml.writer.global"],["sxi","application/vnd.sun.xml.impress"],["sxm","application/vnd.sun.xml.math"],["sxw","application/vnd.sun.xml.writer"],["t","text/troff"],["t3","application/x-t3vm-image"],["t38","image/t38"],["taglet","application/vnd.mynfc"],["tao","application/vnd.tao.intent-module-archive"],["tap","image/vnd.tencent.tap"],["tar","application/x-tar"],["tcap","application/vnd.3gpp2.tcap"],["tcl","application/x-tcl"],["td","application/urc-targetdesc+xml"],["teacher","application/vnd.smart.teacher"],["tei","application/tei+xml"],["teicorpus","application/tei+xml"],["tex","application/x-tex"],["texi","application/x-texinfo"],["texinfo","application/x-texinfo"],["text","text/plain"],["tfi","application/thraud+xml"],["tfm","application/x-tex-tfm"],["tfx","image/tiff-fx"],["tga","image/x-tga"],["tgz","application/x-tar"],["thmx","application/vnd.ms-officetheme"],["tif","image/tiff"],["tiff","image/tiff"],["tk","application/x-tcl"],["tmo","application/vnd.tmobile-livetv"],["toml","application/toml"],["torrent","application/x-bittorrent"],["tpl","application/vnd.groove-tool-template"],["tpt","application/vnd.trid.tpt"],["tr","text/troff"],["tra","application/vnd.trueapp"],["trig","application/trig"],["trm","application/x-msterminal"],["ts","video/mp2t"],["tsd","application/timestamped-data"],["tsv","text/tab-separated-values"],["ttc","font/collection"],["ttf","font/ttf"],["ttl","text/turtle"],["ttml","application/ttml+xml"],["twd","application/vnd.simtech-mindmapper"],["twds","application/vnd.simtech-mindmapper"],["txd","application/vnd.genomatix.tuxedo"],["txf","application/vnd.mobius.txf"],["txt","text/plain"],["u8dsn","message/global-delivery-status"],["u8hdr","message/global-headers"],["u8mdn","message/global-disposition-notification"],["u8msg","message/global"],["u32","application/x-authorware-bin"],["ubj","application/ubjson"],["udeb","application/x-debian-package"],["ufd","application/vnd.ufdl"],["ufdl","application/vnd.ufdl"],["ulx","application/x-glulx"],["umj","application/vnd.umajin"],["unityweb","application/vnd.unity"],["uoml","application/vnd.uoml+xml"],["uri","text/uri-list"],["uris","text/uri-list"],["urls","text/uri-list"],["usdz","model/vnd.usdz+zip"],["ustar","application/x-ustar"],["utz","application/vnd.uiq.theme"],["uu","text/x-uuencode"],["uva","audio/vnd.dece.audio"],["uvd","application/vnd.dece.data"],["uvf","application/vnd.dece.data"],["uvg","image/vnd.dece.graphic"],["uvh","video/vnd.dece.hd"],["uvi","image/vnd.dece.graphic"],["uvm","video/vnd.dece.mobile"],["uvp","video/vnd.dece.pd"],["uvs","video/vnd.dece.sd"],["uvt","application/vnd.dece.ttml+xml"],["uvu","video/vnd.uvvu.mp4"],["uvv","video/vnd.dece.video"],["uvva","audio/vnd.dece.audio"],["uvvd","application/vnd.dece.data"],["uvvf","application/vnd.dece.data"],["uvvg","image/vnd.dece.graphic"],["uvvh","video/vnd.dece.hd"],["uvvi","image/vnd.dece.graphic"],["uvvm","video/vnd.dece.mobile"],["uvvp","video/vnd.dece.pd"],["uvvs","video/vnd.dece.sd"],["uvvt","application/vnd.dece.ttml+xml"],["uvvu","video/vnd.uvvu.mp4"],["uvvv","video/vnd.dece.video"],["uvvx","application/vnd.dece.unspecified"],["uvvz","application/vnd.dece.zip"],["uvx","application/vnd.dece.unspecified"],["uvz","application/vnd.dece.zip"],["vbox","application/x-virtualbox-vbox"],["vbox-extpack","application/x-virtualbox-vbox-extpack"],["vcard","text/vcard"],["vcd","application/x-cdlink"],["vcf","text/x-vcard"],["vcg","application/vnd.groove-vcard"],["vcs","text/x-vcalendar"],["vcx","application/vnd.vcx"],["vdi","application/x-virtualbox-vdi"],["vds","model/vnd.sap.vds"],["vhd","application/x-virtualbox-vhd"],["vis","application/vnd.visionary"],["viv","video/vnd.vivo"],["vlc","application/videolan"],["vmdk","application/x-virtualbox-vmdk"],["vob","video/x-ms-vob"],["vor","application/vnd.stardivision.writer"],["vox","application/x-authorware-bin"],["vrml","model/vrml"],["vsd","application/vnd.visio"],["vsf","application/vnd.vsf"],["vss","application/vnd.visio"],["vst","application/vnd.visio"],["vsw","application/vnd.visio"],["vtf","image/vnd.valve.source.texture"],["vtt","text/vtt"],["vtu","model/vnd.vtu"],["vxml","application/voicexml+xml"],["w3d","application/x-director"],["wad","application/x-doom"],["wadl","application/vnd.sun.wadl+xml"],["war","application/java-archive"],["wasm","application/wasm"],["wav","audio/x-wav"],["wax","audio/x-ms-wax"],["wbmp","image/vnd.wap.wbmp"],["wbs","application/vnd.criticaltools.wbs+xml"],["wbxml","application/wbxml"],["wcm","application/vnd.ms-works"],["wdb","application/vnd.ms-works"],["wdp","image/vnd.ms-photo"],["weba","audio/webm"],["webapp","application/x-web-app-manifest+json"],["webm","video/webm"],["webmanifest","application/manifest+json"],["webp","image/webp"],["wg","application/vnd.pmi.widget"],["wgt","application/widget"],["wks","application/vnd.ms-works"],["wm","video/x-ms-wm"],["wma","audio/x-ms-wma"],["wmd","application/x-ms-wmd"],["wmf","image/wmf"],["wml","text/vnd.wap.wml"],["wmlc","application/wmlc"],["wmls","text/vnd.wap.wmlscript"],["wmlsc","application/vnd.wap.wmlscriptc"],["wmv","video/x-ms-wmv"],["wmx","video/x-ms-wmx"],["wmz","application/x-msmetafile"],["woff","font/woff"],["woff2","font/woff2"],["word","application/msword"],["wpd","application/vnd.wordperfect"],["wpl","application/vnd.ms-wpl"],["wps","application/vnd.ms-works"],["wqd","application/vnd.wqd"],["wri","application/x-mswrite"],["wrl","model/vrml"],["wsc","message/vnd.wfa.wsc"],["wsdl","application/wsdl+xml"],["wspolicy","application/wspolicy+xml"],["wtb","application/vnd.webturbo"],["wvx","video/x-ms-wvx"],["x3d","model/x3d+xml"],["x3db","model/x3d+fastinfoset"],["x3dbz","model/x3d+binary"],["x3dv","model/x3d-vrml"],["x3dvz","model/x3d+vrml"],["x3dz","model/x3d+xml"],["x32","application/x-authorware-bin"],["x_b","model/vnd.parasolid.transmit.binary"],["x_t","model/vnd.parasolid.transmit.text"],["xaml","application/xaml+xml"],["xap","application/x-silverlight-app"],["xar","application/vnd.xara"],["xav","application/xcap-att+xml"],["xbap","application/x-ms-xbap"],["xbd","application/vnd.fujixerox.docuworks.binder"],["xbm","image/x-xbitmap"],["xca","application/xcap-caps+xml"],["xcs","application/calendar+xml"],["xdf","application/xcap-diff+xml"],["xdm","application/vnd.syncml.dm+xml"],["xdp","application/vnd.adobe.xdp+xml"],["xdssc","application/dssc+xml"],["xdw","application/vnd.fujixerox.docuworks"],["xel","application/xcap-el+xml"],["xenc","application/xenc+xml"],["xer","application/patch-ops-error+xml"],["xfdf","application/vnd.adobe.xfdf"],["xfdl","application/vnd.xfdl"],["xht","application/xhtml+xml"],["xhtml","application/xhtml+xml"],["xhvml","application/xv+xml"],["xif","image/vnd.xiff"],["xl","application/excel"],["xla","application/vnd.ms-excel"],["xlam","application/vnd.ms-excel.addin.macroEnabled.12"],["xlc","application/vnd.ms-excel"],["xlf","application/xliff+xml"],["xlm","application/vnd.ms-excel"],["xls","application/vnd.ms-excel"],["xlsb","application/vnd.ms-excel.sheet.binary.macroEnabled.12"],["xlsm","application/vnd.ms-excel.sheet.macroEnabled.12"],["xlsx","application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"],["xlt","application/vnd.ms-excel"],["xltm","application/vnd.ms-excel.template.macroEnabled.12"],["xltx","application/vnd.openxmlformats-officedocument.spreadsheetml.template"],["xlw","application/vnd.ms-excel"],["xm","audio/xm"],["xml","application/xml"],["xns","application/xcap-ns+xml"],["xo","application/vnd.olpc-sugar"],["xop","application/xop+xml"],["xpi","application/x-xpinstall"],["xpl","application/xproc+xml"],["xpm","image/x-xpixmap"],["xpr","application/vnd.is-xpr"],["xps","application/vnd.ms-xpsdocument"],["xpw","application/vnd.intercon.formnet"],["xpx","application/vnd.intercon.formnet"],["xsd","application/xml"],["xsl","application/xml"],["xslt","application/xslt+xml"],["xsm","application/vnd.syncml+xml"],["xspf","application/xspf+xml"],["xul","application/vnd.mozilla.xul+xml"],["xvm","application/xv+xml"],["xvml","application/xv+xml"],["xwd","image/x-xwindowdump"],["xyz","chemical/x-xyz"],["xz","application/x-xz"],["yaml","text/yaml"],["yang","application/yang"],["yin","application/yin+xml"],["yml","text/yaml"],["ymp","text/x-suse-ymp"],["z","application/x-compress"],["z1","application/x-zmachine"],["z2","application/x-zmachine"],["z3","application/x-zmachine"],["z4","application/x-zmachine"],["z5","application/x-zmachine"],["z6","application/x-zmachine"],["z7","application/x-zmachine"],["z8","application/x-zmachine"],["zaz","application/vnd.zzazz.deck+xml"],["zip","application/zip"],["zir","application/vnd.zul"],["zirz","application/vnd.zul"],["zmm","application/vnd.handheld-entertainment+xml"],["zsh","text/x-scriptzsh"]]);function Vh(e,t,r){const n=m5e(e),{webkitRelativePath:a}=e,i=typeof t=="string"?t:typeof a=="string"&&a.length>0?a:`./${e.name}`;return typeof n.path!="string"&&QB(n,"path",i),QB(n,"relativePath",i),n}function m5e(e){const{name:t}=e;if(t&&t.lastIndexOf(".")!==-1&&!e.type){const n=t.split(".").pop().toLowerCase(),a=f5e.get(n);a&&Object.defineProperty(e,"type",{value:a,writable:!1,configurable:!1,enumerable:!0})}return e}function QB(e,t,r){Object.defineProperty(e,t,{value:r,writable:!1,configurable:!1,enumerable:!0})}const g5e=[".DS_Store","Thumbs.db"];function b5e(e){return nd(this,void 0,void 0,function*(){return Ty(e)&&y5e(e.dataTransfer)?x5e(e.dataTransfer,e.type):v5e(e)?S5e(e):Array.isArray(e)&&e.every(t=>"getFile"in t&&typeof t.getFile=="function")?E5e(e):[]})}function y5e(e){return Ty(e)}function v5e(e){return Ty(e)&&Ty(e.target)}function Ty(e){return typeof e=="object"&&e!==null}function S5e(e){return RR(e.target.files).map(t=>Vh(t))}function E5e(e){return nd(this,void 0,void 0,function*(){return(yield Promise.all(e.map(r=>r.getFile()))).map(r=>Vh(r))})}function x5e(e,t){return nd(this,void 0,void 0,function*(){if(e.items){const r=RR(e.items).filter(a=>a.kind==="file");if(t!=="drop")return r;const n=yield Promise.all(r.map(w5e));return eP(Wte(n))}return eP(RR(e.files).map(r=>Vh(r)))})}function eP(e){return e.filter(t=>g5e.indexOf(t.name)===-1)}function RR(e){if(e===null)return[];const t=[];for(let r=0;r<e.length;r++){const n=e[r];t.push(n)}return t}function w5e(e){if(typeof e.webkitGetAsEntry!="function")return tP(e);const t=e.webkitGetAsEntry();return t&&t.isDirectory?Vte(t):tP(e,t)}function Wte(e){return e.reduce((t,r)=>[...t,...Array.isArray(r)?Wte(r):[r]],[])}function tP(e,t){return nd(this,void 0,void 0,function*(){var r;if(globalThis.isSecureContext&&typeof e.getAsFileSystemHandle=="function"){const i=yield e.getAsFileSystemHandle();if(i===null)throw new Error(`${e} is not a File`);if(i!==void 0){const o=yield i.getFile();return o.handle=i,Vh(o)}}const n=e.getAsFile();if(!n)throw new Error(`${e} is not a File`);return Vh(n,(r=t==null?void 0:t.fullPath)!==null&&r!==void 0?r:void 0)})}function k5e(e){return nd(this,void 0,void 0,function*(){return e.isDirectory?Vte(e):C5e(e)})}function Vte(e){const t=e.createReader();return new Promise((r,n)=>{const a=[];function i(){t.readEntries(o=>nd(this,void 0,void 0,function*(){if(o.length){const s=Promise.all(o.map(k5e));a.push(s),i()}else try{const s=yield Promise.all(a);r(s)}catch(s){n(s)}}),o=>{n(o)})}i()})}function C5e(e){return nd(this,void 0,void 0,function*(){return new Promise((t,r)=>{e.file(n=>{const a=Vh(n,e.fullPath);t(a)},n=>{r(n)})})})}var Vb={},rP;function T5e(){return rP||(rP=1,Vb.__esModule=!0,Vb.default=function(e,t){if(e&&t){var r=Array.isArray(t)?t:t.split(",");if(r.length===0)return!0;var n=e.name||"",a=(e.type||"").toLowerCase(),i=a.replace(/\/.*$/,"");return r.some(function(o){var s=o.trim().toLowerCase();return s.charAt(0)==="."?n.toLowerCase().endsWith(s):s.endsWith("/*")?i===s.replace(/\/.*$/,""):a===s})}return!0}),Vb}var A5e=T5e();const ek=jn(A5e);function nP(e){return R5e(e)||_5e(e)||Yte(e)||F5e()}function F5e(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function _5e(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function R5e(e){if(Array.isArray(e))return DR(e)}function aP(e,t){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),r.push.apply(r,n)}return r}function iP(e){for(var t=1;t<arguments.length;t++){var r=arguments[t]!=null?arguments[t]:{};t%2?aP(Object(r),!0).forEach(function(n){Kte(e,n,r[n])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):aP(Object(r)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))})}return e}function Kte(e,t,r){return t in e?Object.defineProperty(e,t,{value:r,enumerable:!0,configurable:!0,writable:!0}):e[t]=r,e}function Gm(e,t){return L5e(e)||N5e(e,t)||Yte(e,t)||D5e()}function D5e(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function Yte(e,t){if(e){if(typeof e=="string")return DR(e,t);var r=Object.prototype.toString.call(e).slice(8,-1);if(r==="Object"&&e.constructor&&(r=e.constructor.name),r==="Map"||r==="Set")return Array.from(e);if(r==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r))return DR(e,t)}}function DR(e,t){(t==null||t>e.length)&&(t=e.length);for(var r=0,n=new Array(t);r<t;r++)n[r]=e[r];return n}function N5e(e,t){var r=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(r!=null){var n=[],a=!0,i=!1,o,s;try{for(r=r.call(e);!(a=(o=r.next()).done)&&(n.push(o.value),!(t&&n.length===t));a=!0);}catch(l){i=!0,s=l}finally{try{!a&&r.return!=null&&r.return()}finally{if(i)throw s}}return n}}function L5e(e){if(Array.isArray(e))return e}var M5e=typeof ek=="function"?ek:ek.default,I5e="file-invalid-type",O5e="file-too-large",B5e="file-too-small",P5e="too-many-files",z5e=function(){var t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"",r=t.split(","),n=r.length>1?"one of ".concat(r.join(", ")):r[0];return{code:I5e,message:"File type must be ".concat(n)}},oP=function(t){return{code:O5e,message:"File is larger than ".concat(t," ").concat(t===1?"byte":"bytes")}},sP=function(t){return{code:B5e,message:"File is smaller than ".concat(t," ").concat(t===1?"byte":"bytes")}},H5e={code:P5e,message:"Too many files"};function Jte(e,t){var r=e.type==="application/x-moz-file"||M5e(e,t);return[r,r?null:z5e(t)]}function Xte(e,t,r){if(Sc(e.size))if(Sc(t)&&Sc(r)){if(e.size>r)return[!1,oP(r)];if(e.size<t)return[!1,sP(t)]}else{if(Sc(t)&&e.size<t)return[!1,sP(t)];if(Sc(r)&&e.size>r)return[!1,oP(r)]}return[!0,null]}function Sc(e){return e!=null}function U5e(e){var t=e.files,r=e.accept,n=e.minSize,a=e.maxSize,i=e.multiple,o=e.maxFiles,s=e.validator;return!i&&t.length>1||i&&o>=1&&t.length>o?!1:t.every(function(l){var u=Jte(l,r),d=Gm(u,1),p=d[0],f=Xte(l,n,a),m=Gm(f,1),b=m[0],y=s?s(l):null;return p&&b&&!y})}function Ay(e){return typeof e.isPropagationStopped=="function"?e.isPropagationStopped():typeof e.cancelBubble<"u"?e.cancelBubble:!1}function Kb(e){return e.dataTransfer?Array.prototype.some.call(e.dataTransfer.types,function(t){return t==="Files"||t==="application/x-moz-file"}):!!e.target&&!!e.target.files}function lP(e){e.preventDefault()}function $5e(e){return e.indexOf("MSIE")!==-1||e.indexOf("Trident/")!==-1}function G5e(e){return e.indexOf("Edge/")!==-1}function j5e(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:window.navigator.userAgent;return $5e(e)||G5e(e)}function Ko(){for(var e=arguments.length,t=new Array(e),r=0;r<e;r++)t[r]=arguments[r];return function(n){for(var a=arguments.length,i=new Array(a>1?a-1:0),o=1;o<a;o++)i[o-1]=arguments[o];return t.some(function(s){return!Ay(n)&&s&&s.apply(void 0,[n].concat(i)),Ay(n)})}}function q5e(){return"showOpenFilePicker"in window}function W5e(e){if(Sc(e)){var t=Object.entries(e).filter(function(r){var n=Gm(r,2),a=n[0],i=n[1],o=!0;return Zte(a)||(console.warn('Skipped "'.concat(a,'" because it is not a valid MIME type. Check https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types for a list of valid MIME types.')),o=!1),(!Array.isArray(i)||!i.every(Qte))&&(console.warn('Skipped "'.concat(a,'" because an invalid file extension was provided.')),o=!1),o}).reduce(function(r,n){var a=Gm(n,2),i=a[0],o=a[1];return iP(iP({},r),{},Kte({},i,o))},{});return[{description:"Files",accept:t}]}return e}function V5e(e){if(Sc(e))return Object.entries(e).reduce(function(t,r){var n=Gm(r,2),a=n[0],i=n[1];return[].concat(nP(t),[a],nP(i))},[]).filter(function(t){return Zte(t)||Qte(t)}).join(",")}function K5e(e){return e instanceof DOMException&&(e.name==="AbortError"||e.code===e.ABORT_ERR)}function Y5e(e){return e instanceof DOMException&&(e.name==="SecurityError"||e.code===e.SECURITY_ERR)}function Zte(e){return e==="audio/*"||e==="video/*"||e==="image/*"||e==="text/*"||e==="application/*"||/\w+\/[-+.\w]+/g.test(e)}function Qte(e){return/^.*\.[\w]+$/.test(e)}var J5e=["children"],X5e=["open"],Z5e=["refKey","role","onKeyDown","onFocus","onBlur","onClick","onDragEnter","onDragOver","onDragLeave","onDrop"],Q5e=["refKey","onChange","onClick"];function eNe(e){return nNe(e)||rNe(e)||ere(e)||tNe()}function tNe(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function rNe(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function nNe(e){if(Array.isArray(e))return NR(e)}function tk(e,t){return oNe(e)||iNe(e,t)||ere(e,t)||aNe()}function aNe(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function ere(e,t){if(e){if(typeof e=="string")return NR(e,t);var r=Object.prototype.toString.call(e).slice(8,-1);if(r==="Object"&&e.constructor&&(r=e.constructor.name),r==="Map"||r==="Set")return Array.from(e);if(r==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r))return NR(e,t)}}function NR(e,t){(t==null||t>e.length)&&(t=e.length);for(var r=0,n=new Array(t);r<t;r++)n[r]=e[r];return n}function iNe(e,t){var r=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(r!=null){var n=[],a=!0,i=!1,o,s;try{for(r=r.call(e);!(a=(o=r.next()).done)&&(n.push(o.value),!(t&&n.length===t));a=!0);}catch(l){i=!0,s=l}finally{try{!a&&r.return!=null&&r.return()}finally{if(i)throw s}}return n}}function oNe(e){if(Array.isArray(e))return e}function uP(e,t){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),r.push.apply(r,n)}return r}function En(e){for(var t=1;t<arguments.length;t++){var r=arguments[t]!=null?arguments[t]:{};t%2?uP(Object(r),!0).forEach(function(n){LR(e,n,r[n])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):uP(Object(r)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))})}return e}function LR(e,t,r){return t in e?Object.defineProperty(e,t,{value:r,enumerable:!0,configurable:!0,writable:!0}):e[t]=r,e}function Fy(e,t){if(e==null)return{};var r=sNe(e,t),n,a;if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(a=0;a<i.length;a++)n=i[a],!(t.indexOf(n)>=0)&&Object.prototype.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}function sNe(e,t){if(e==null)return{};var r={},n=Object.keys(e),a,i;for(i=0;i<n.length;i++)a=n[i],!(t.indexOf(a)>=0)&&(r[a]=e[a]);return r}var y2=T.forwardRef(function(e,t){var r=e.children,n=Fy(e,J5e),a=lNe(n),i=a.open,o=Fy(a,X5e);return T.useImperativeHandle(t,function(){return{open:i}},[i]),$e.createElement(T.Fragment,null,r(En(En({},o),{},{open:i})))});y2.displayName="Dropzone";var tre={disabled:!1,getFilesFromEvent:b5e,maxSize:1/0,minSize:0,multiple:!0,maxFiles:0,preventDropOnDocument:!0,noClick:!1,noKeyboard:!1,noDrag:!1,noDragEventsBubbling:!1,validator:null,useFsAccessApi:!1,autoFocus:!1};y2.defaultProps=tre;y2.propTypes={children:Qr.func,accept:Qr.objectOf(Qr.arrayOf(Qr.string)),multiple:Qr.bool,preventDropOnDocument:Qr.bool,noClick:Qr.bool,noKeyboard:Qr.bool,noDrag:Qr.bool,noDragEventsBubbling:Qr.bool,minSize:Qr.number,maxSize:Qr.number,maxFiles:Qr.number,disabled:Qr.bool,getFilesFromEvent:Qr.func,onFileDialogCancel:Qr.func,onFileDialogOpen:Qr.func,useFsAccessApi:Qr.bool,autoFocus:Qr.bool,onDragEnter:Qr.func,onDragLeave:Qr.func,onDragOver:Qr.func,onDrop:Qr.func,onDropAccepted:Qr.func,onDropRejected:Qr.func,onError:Qr.func,validator:Qr.func};var MR={isFocused:!1,isFileDialogActive:!1,isDragActive:!1,isDragAccept:!1,isDragReject:!1,acceptedFiles:[],fileRejections:[]};function lNe(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},t=En(En({},tre),e),r=t.accept,n=t.disabled,a=t.getFilesFromEvent,i=t.maxSize,o=t.minSize,s=t.multiple,l=t.maxFiles,u=t.onDragEnter,d=t.onDragLeave,p=t.onDragOver,f=t.onDrop,m=t.onDropAccepted,b=t.onDropRejected,y=t.onFileDialogCancel,S=t.onFileDialogOpen,w=t.useFsAccessApi,E=t.autoFocus,k=t.preventDropOnDocument,_=t.noClick,R=t.noKeyboard,C=t.noDrag,F=t.noDragEventsBubbling,L=t.onError,I=t.validator,M=T.useMemo(function(){return V5e(r)},[r]),j=T.useMemo(function(){return W5e(r)},[r]),Y=T.useMemo(function(){return typeof S=="function"?S:cP},[S]),W=T.useMemo(function(){return typeof y=="function"?y:cP},[y]),$=T.useRef(null),X=T.useRef(null),z=T.useReducer(uNe,MR),H=tk(z,2),B=H[0],ee=H[1],J=B.isFocused,K=B.isFileDialogActive,G=T.useRef(typeof window<"u"&&window.isSecureContext&&w&&q5e()),P=function(){!G.current&&K&&setTimeout(function(){if(X.current){var rt=X.current.files;rt.length||(ee({type:"closeDialog"}),W())}},300)};T.useEffect(function(){return window.addEventListener("focus",P,!1),function(){window.removeEventListener("focus",P,!1)}},[X,K,W,G]);var ne=T.useRef([]),oe=function(rt){$.current&&$.current.contains(rt.target)||(rt.preventDefault(),ne.current=[])};T.useEffect(function(){return k&&(document.addEventListener("dragover",lP,!1),document.addEventListener("drop",oe,!1)),function(){k&&(document.removeEventListener("dragover",lP),document.removeEventListener("drop",oe))}},[$,k]),T.useEffect(function(){return!n&&E&&$.current&&$.current.focus(),function(){}},[$,E,n]);var pe=T.useCallback(function(Pe){L?L(Pe):console.error(Pe)},[L]),ae=T.useCallback(function(Pe){Pe.preventDefault(),Pe.persist(),Ye(Pe),ne.current=[].concat(eNe(ne.current),[Pe.target]),Kb(Pe)&&Promise.resolve(a(Pe)).then(function(rt){if(!(Ay(Pe)&&!F)){var ge=rt.length,Re=ge>0&&U5e({files:rt,accept:M,minSize:o,maxSize:i,multiple:s,maxFiles:l,validator:I}),We=ge>0&&!Re;ee({isDragAccept:Re,isDragReject:We,isDragActive:!0,type:"setDraggedFiles"}),u&&u(Pe)}}).catch(function(rt){return pe(rt)})},[a,u,pe,F,M,o,i,s,l,I]),se=T.useCallback(function(Pe){Pe.preventDefault(),Pe.persist(),Ye(Pe);var rt=Kb(Pe);if(rt&&Pe.dataTransfer)try{Pe.dataTransfer.dropEffect="copy"}catch{}return rt&&p&&p(Pe),!1},[p,F]),Ee=T.useCallback(function(Pe){Pe.preventDefault(),Pe.persist(),Ye(Pe);var rt=ne.current.filter(function(Re){return $.current&&$.current.contains(Re)}),ge=rt.indexOf(Pe.target);ge!==-1&&rt.splice(ge,1),ne.current=rt,!(rt.length>0)&&(ee({type:"setDraggedFiles",isDragActive:!1,isDragAccept:!1,isDragReject:!1}),Kb(Pe)&&d&&d(Pe))},[$,d,F]),fe=T.useCallback(function(Pe,rt){var ge=[],Re=[];Pe.forEach(function(We){var ht=Jte(We,M),xt=tk(ht,2),sr=xt[0],ar=xt[1],fr=Xte(We,o,i),Mt=tk(fr,2),lr=Mt[0],qt=Mt[1],zr=I?I(We):null;if(sr&&lr&&!zr)ge.push(We);else{var nn=[ar,qt];zr&&(nn=nn.concat(zr)),Re.push({file:We,errors:nn.filter(function(dn){return dn})})}}),(!s&&ge.length>1||s&&l>=1&&ge.length>l)&&(ge.forEach(function(We){Re.push({file:We,errors:[H5e]})}),ge.splice(0)),ee({acceptedFiles:ge,fileRejections:Re,isDragReject:Re.length>0,type:"setFiles"}),f&&f(ge,Re,rt),Re.length>0&&b&&b(Re,rt),ge.length>0&&m&&m(ge,rt)},[ee,s,M,o,i,l,f,m,b,I]),Me=T.useCallback(function(Pe){Pe.preventDefault(),Pe.persist(),Ye(Pe),ne.current=[],Kb(Pe)&&Promise.resolve(a(Pe)).then(function(rt){Ay(Pe)&&!F||fe(rt,Pe)}).catch(function(rt){return pe(rt)}),ee({type:"reset"})},[a,fe,pe,F]),Oe=T.useCallback(function(){if(G.current){ee({type:"openDialog"}),Y();var Pe={multiple:s,types:j};window.showOpenFilePicker(Pe).then(function(rt){return a(rt)}).then(function(rt){fe(rt,null),ee({type:"closeDialog"})}).catch(function(rt){K5e(rt)?(W(rt),ee({type:"closeDialog"})):Y5e(rt)?(G.current=!1,X.current?(X.current.value=null,X.current.click()):pe(new Error("Cannot open the file picker because the https://developer.mozilla.org/en-US/docs/Web/API/File_System_Access_API is not supported and no <input> was provided."))):pe(rt)});return}X.current&&(ee({type:"openDialog"}),Y(),X.current.value=null,X.current.click())},[ee,Y,W,w,fe,pe,j,s]),xe=T.useCallback(function(Pe){!$.current||!$.current.isEqualNode(Pe.target)||(Pe.key===" "||Pe.key==="Enter"||Pe.keyCode===32||Pe.keyCode===13)&&(Pe.preventDefault(),Oe())},[$,Oe]),Te=T.useCallback(function(){ee({type:"focus"})},[]),Ne=T.useCallback(function(){ee({type:"blur"})},[]),he=T.useCallback(function(){_||(j5e()?setTimeout(Oe,0):Oe())},[_,Oe]),Ze=function(rt){return n?null:rt},Et=function(rt){return R?null:Ze(rt)},qe=function(rt){return C?null:Ze(rt)},Ye=function(rt){F&&rt.stopPropagation()},we=T.useMemo(function(){return function(){var Pe=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},rt=Pe.refKey,ge=rt===void 0?"ref":rt,Re=Pe.role,We=Pe.onKeyDown,ht=Pe.onFocus,xt=Pe.onBlur,sr=Pe.onClick,ar=Pe.onDragEnter,fr=Pe.onDragOver,Mt=Pe.onDragLeave,lr=Pe.onDrop,qt=Fy(Pe,Z5e);return En(En(LR({onKeyDown:Et(Ko(We,xe)),onFocus:Et(Ko(ht,Te)),onBlur:Et(Ko(xt,Ne)),onClick:Ze(Ko(sr,he)),onDragEnter:qe(Ko(ar,ae)),onDragOver:qe(Ko(fr,se)),onDragLeave:qe(Ko(Mt,Ee)),onDrop:qe(Ko(lr,Me)),role:typeof Re=="string"&&Re!==""?Re:"presentation"},ge,$),!n&&!R?{tabIndex:0}:{}),qt)}},[$,xe,Te,Ne,he,ae,se,Ee,Me,R,C,n]),Qt=T.useCallback(function(Pe){Pe.stopPropagation()},[]),or=T.useMemo(function(){return function(){var Pe=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},rt=Pe.refKey,ge=rt===void 0?"ref":rt,Re=Pe.onChange,We=Pe.onClick,ht=Fy(Pe,Q5e),xt=LR({accept:M,multiple:s,type:"file",style:{border:0,clip:"rect(0, 0, 0, 0)",clipPath:"inset(50%)",height:"1px",margin:"0 -1px -1px 0",overflow:"hidden",padding:0,position:"absolute",width:"1px",whiteSpace:"nowrap"},onChange:Ze(Ko(Re,Me)),onClick:Ze(Ko(We,Qt)),tabIndex:-1},ge,X);return En(En({},xt),ht)}},[X,r,s,Me,n]);return En(En({},B),{},{isFocused:J&&!n,getRootProps:we,getInputProps:or,rootRef:$,inputRef:X,open:Ze(Oe)})}function uNe(e,t){switch(t.type){case"focus":return En(En({},e),{},{isFocused:!0});case"blur":return En(En({},e),{},{isFocused:!1});case"openDialog":return En(En({},MR),{},{isFileDialogActive:!0});case"closeDialog":return En(En({},e),{},{isFileDialogActive:!1});case"setDraggedFiles":return En(En({},e),{},{isDragActive:t.isDragActive,isDragAccept:t.isDragAccept,isDragReject:t.isDragReject});case"setFiles":return En(En({},e),{},{acceptedFiles:t.acceptedFiles,fileRejections:t.fileRejections,isDragReject:t.isDragReject});case"reset":return En({},MR);default:return e}}function cP(){}function IR(e,t={}){const{decimals:r=0,sizeType:n="normal"}=t,a=["Bytes","KB","MB","GB","TB"],i=["Bytes","KiB","MiB","GiB","TiB"];if(e===0)return"0 Byte";const o=Math.floor(Math.log(e)/Math.log(1024));return`${(e/Math.pow(1024,o)).toFixed(r)} ${n==="accurate"?i[o]??"Bytes":a[o]??"Bytes"}`}function cNe(e){const{t}=Sr(),{value:r,onValueChange:n,onUpload:a,onReject:i,progresses:o,fileErrors:s,accept:l=Ybe,maxSize:u=1024*1024*200,maxFileCount:d=1,multiple:p=!1,disabled:f=!1,description:m,className:b,...y}=e,[S,w]=bl({prop:r,onChange:n}),E=T.useCallback((R,C)=>{const F=((S==null?void 0:S.length)??0)+R.length+C.length;if(!p&&d===1&&R.length+C.length>1){er.error(t("documentPanel.uploadDocuments.fileUploader.singleFileLimit"));return}if(F>d){er.error(t("documentPanel.uploadDocuments.fileUploader.maxFilesLimit",{count:d}));return}C.length>0&&(i?i(C):C.forEach(({file:Y})=>{er.error(t("documentPanel.uploadDocuments.fileUploader.fileRejected",{name:Y.name}))}));const L=R.map(Y=>Object.assign(Y,{preview:URL.createObjectURL(Y)})),I=C.map(({file:Y})=>Object.assign(Y,{preview:URL.createObjectURL(Y),rejected:!0})),M=[...L,...I],j=S?[...S,...M]:M;if(w(j),a&&R.length>0){const Y=R.filter(W=>{var H;if(!W.name)return!1;const $=`.${((H=W.name.split(".").pop())==null?void 0:H.toLowerCase())||""}`,X=Object.entries(l||{}).some(([B,ee])=>W.type===B||Array.isArray(ee)&&ee.includes($)),z=W.size<=u;return X&&z});Y.length>0&&a(Y)}},[S,d,p,a,i,w,t,l,u]);function k(R){if(!S)return;const C=S.filter((F,L)=>L!==R);w(C),n==null||n(C)}T.useEffect(()=>()=>{S&&S.forEach(R=>{rre(R)&&URL.revokeObjectURL(R.preview)})},[]);const _=f||((S==null?void 0:S.length)??0)>=d;return x.jsxs("div",{className:"relative flex flex-col gap-6 overflow-hidden",children:[x.jsx(y2,{onDrop:E,noClick:!1,noKeyboard:!1,maxSize:u,maxFiles:d,multiple:d>1||p,disabled:_,validator:R=>{var L;if(!R.name)return{code:"invalid-file-name",message:t("documentPanel.uploadDocuments.fileUploader.invalidFileName",{fallback:"Invalid file name"})};const C=`.${((L=R.name.split(".").pop())==null?void 0:L.toLowerCase())||""}`;return Object.entries(l||{}).some(([I,M])=>R.type===I||Array.isArray(M)&&M.includes(C))?R.size>u?{code:"file-too-large",message:t("documentPanel.uploadDocuments.fileUploader.fileTooLarge",{maxSize:IR(u)})}:null:{code:"file-invalid-type",message:t("documentPanel.uploadDocuments.fileUploader.unsupportedType")}},children:({getRootProps:R,getInputProps:C,isDragActive:F})=>x.jsxs("div",{...R(),className:lt("group border-muted-foreground/25 hover:bg-muted/25 relative grid h-52 w-full cursor-pointer place-items-center rounded-lg border-2 border-dashed px-5 py-2.5 text-center transition","ring-offset-background focus-visible:ring-ring focus-visible:ring-2 focus-visible:ring-offset-2 focus-visible:outline-none",F&&"border-muted-foreground/50",_&&"pointer-events-none opacity-60",b),...y,children:[x.jsx("input",{...C()}),F?x.jsxs("div",{className:"flex flex-col items-center justify-center gap-4 sm:px-5",children:[x.jsx("div",{className:"rounded-full border border-dashed p-3",children:x.jsx(nR,{className:"text-muted-foreground size-7","aria-hidden":"true"})}),x.jsx("p",{className:"text-muted-foreground font-medium",children:t("documentPanel.uploadDocuments.fileUploader.dropHere")})]}):x.jsxs("div",{className:"flex flex-col items-center justify-center gap-4 sm:px-5",children:[x.jsx("div",{className:"rounded-full border border-dashed p-3",children:x.jsx(nR,{className:"text-muted-foreground size-7","aria-hidden":"true"})}),x.jsxs("div",{className:"flex flex-col gap-px",children:[x.jsx("p",{className:"text-muted-foreground font-medium",children:t("documentPanel.uploadDocuments.fileUploader.dragAndDrop")}),m?x.jsx("p",{className:"text-muted-foreground/70 text-sm",children:m}):x.jsxs("p",{className:"text-muted-foreground/70 text-sm",children:[t("documentPanel.uploadDocuments.fileUploader.uploadDescription",{count:d,isMultiple:d===1/0,maxSize:IR(u)}),t("documentPanel.uploadDocuments.fileTypes")]})]})]})]})}),S!=null&&S.length?x.jsx(B5,{className:"h-fit w-full px-3",children:x.jsx("div",{className:"flex max-h-48 flex-col gap-4",children:S==null?void 0:S.map((R,C)=>x.jsx(dNe,{file:R,onRemove:()=>k(C),progress:o==null?void 0:o[R.name],error:s==null?void 0:s[R.name]},C))})}):null]})}function dP({value:e,error:t}){return x.jsx("div",{className:"relative h-2 w-full",children:x.jsx("div",{className:"h-full w-full overflow-hidden rounded-full bg-secondary",children:x.jsx("div",{className:lt("h-full transition-all",t?"bg-red-400":"bg-primary"),style:{width:`${e}%`}})})})}function dNe({file:e,progress:t,error:r,onRemove:n}){const{t:a}=Sr();return x.jsxs("div",{className:"relative flex items-center gap-2.5",children:[x.jsxs("div",{className:"flex flex-1 gap-2.5",children:[r?x.jsx(lZ,{className:"text-red-400 size-10","aria-hidden":"true"}):rre(e)?x.jsx(hNe,{file:e}):null,x.jsxs("div",{className:"flex w-full flex-col gap-2",children:[x.jsxs("div",{className:"flex flex-col gap-px",children:[x.jsx("p",{className:"text-foreground/80 line-clamp-1 text-sm font-medium",children:e.name}),x.jsx("p",{className:"text-muted-foreground text-xs",children:IR(e.size)})]}),r?x.jsxs("div",{className:"text-red-400 text-sm",children:[x.jsx("div",{className:"relative mb-2",children:x.jsx(dP,{value:100,error:!0})}),x.jsx("p",{children:r})]}):t?x.jsx(dP,{value:t}):null]})]}),x.jsx("div",{className:"flex items-center gap-2",children:x.jsxs(Rt,{type:"button",variant:"outline",size:"icon",className:"size-7",onClick:n,children:[x.jsx(Qv,{className:"size-4","aria-hidden":"true"}),x.jsx("span",{className:"sr-only",children:a("documentPanel.uploadDocuments.fileUploader.removeFile")})]})})]})}function rre(e){return"preview"in e&&typeof e.preview=="string"}function hNe({file:e}){return e.type.startsWith("image/")?x.jsx("div",{className:"aspect-square shrink-0 rounded-md object-cover"}):x.jsx(lZ,{className:"text-muted-foreground size-10","aria-hidden":"true"})}function pNe({onDocumentsUploaded:e}){const{t}=Sr(),[r,n]=T.useState(!1),[a,i]=T.useState(!1),[o,s]=T.useState({}),[l,u]=T.useState({}),d=T.useCallback(f=>{f.forEach(({file:m,errors:b})=>{var S;let y=((S=b[0])==null?void 0:S.message)||t("documentPanel.uploadDocuments.fileUploader.fileRejected",{name:m.name});y.includes("file-invalid-type")&&(y=t("documentPanel.uploadDocuments.fileUploader.unsupportedType")),s(w=>({...w,[m.name]:100})),u(w=>({...w,[m.name]:y}))})},[s,u,t]),p=T.useCallback(async f=>{var y,S;i(!0);let m=!1;u(w=>{const E={...w};return f.forEach(k=>{delete E[k.name]}),E});const b=er.loading(t("documentPanel.uploadDocuments.batch.uploading"));try{const w={},E=new Intl.Collator(["zh-CN","en"],{sensitivity:"accent",numeric:!0}),k=[...f].sort((R,C)=>E.compare(R.name,C.name));for(const R of k)try{s(F=>({...F,[R.name]:0}));const C=await u1e(R,F=>{console.debug(t("documentPanel.uploadDocuments.single.uploading",{name:R.name,percent:F})),s(L=>({...L,[R.name]:F}))});C.status==="duplicated"?(w[R.name]=t("documentPanel.uploadDocuments.fileUploader.duplicateFile"),u(F=>({...F,[R.name]:t("documentPanel.uploadDocuments.fileUploader.duplicateFile")}))):C.status!=="success"?(w[R.name]=C.message,u(F=>({...F,[R.name]:C.message}))):m=!0}catch(C){console.error(`Upload failed for ${R.name}:`,C);let F=Da(C);if(C&&typeof C=="object"&&"response"in C){const L=C;((y=L.response)==null?void 0:y.status)===400&&(F=((S=L.response.data)==null?void 0:S.detail)||F),s(I=>({...I,[R.name]:100}))}w[R.name]=F,u(L=>({...L,[R.name]:F}))}Object.keys(w).length>0?er.error(t("documentPanel.uploadDocuments.batch.error"),{id:b}):er.success(t("documentPanel.uploadDocuments.batch.success"),{id:b}),m&&e&&e().catch(R=>{console.error("Error refreshing documents:",R)})}catch(w){console.error("Unexpected error during upload:",w),er.error(t("documentPanel.uploadDocuments.generalError",{error:Da(w)}),{id:b})}finally{i(!1)}},[i,s,u,t,e]);return x.jsxs(Du,{open:r,onOpenChange:f=>{a||(f||(s({}),u({})),n(f))},children:[x.jsx(u5,{asChild:!0,children:x.jsxs(Rt,{variant:"default",side:"bottom",tooltip:t("documentPanel.uploadDocuments.tooltip"),size:"sm",children:[x.jsx(nR,{})," ",t("documentPanel.uploadDocuments.button")]})}),x.jsxs(Sl,{className:"sm:max-w-xl",onCloseAutoFocus:f=>f.preventDefault(),children:[x.jsxs(El,{children:[x.jsx(xl,{children:t("documentPanel.uploadDocuments.title")}),x.jsx(wl,{children:t("documentPanel.uploadDocuments.description")})]}),x.jsx(cNe,{maxFileCount:1/0,maxSize:200*1024*1024,description:t("documentPanel.uploadDocuments.fileTypes"),onUpload:p,onReject:d,progresses:o,fileErrors:l,disabled:a})]})]})}const hP=({htmlFor:e,className:t,children:r,...n})=>x.jsx("label",{htmlFor:e,className:t,...n,children:r});function fNe({onDocumentsCleared:e}){const{t}=Sr(),[r,n]=T.useState(!1),[a,i]=T.useState(""),[o,s]=T.useState(!1),[l,u]=T.useState(!1),d=T.useRef(null),p=a.toLowerCase()==="yes",f=3e4;T.useEffect(()=>{r||(i(""),s(!1),u(!1),d.current&&(clearTimeout(d.current),d.current=null))},[r]),T.useEffect(()=>()=>{d.current&&clearTimeout(d.current)},[]);const m=T.useCallback(async()=>{if(!(!p||l)){u(!0),d.current=setTimeout(()=>{l&&(er.error(t("documentPanel.clearDocuments.timeout")),u(!1),i(""))},f);try{const b=await c1e();if(b.status!=="success"){er.error(t("documentPanel.clearDocuments.failed",{message:b.message})),i("");return}if(er.success(t("documentPanel.clearDocuments.success")),o)try{await d1e(),er.success(t("documentPanel.clearDocuments.cacheCleared"))}catch(y){er.error(t("documentPanel.clearDocuments.cacheClearFailed",{error:Da(y)}))}e&&e().catch(console.error),n(!1)}catch(b){er.error(t("documentPanel.clearDocuments.error",{error:Da(b)})),i("")}finally{d.current&&(clearTimeout(d.current),d.current=null),u(!1)}}},[p,l,o,n,t,e,f]);return x.jsxs(Du,{open:r,onOpenChange:n,children:[x.jsx(u5,{asChild:!0,children:x.jsxs(Rt,{variant:"outline",side:"bottom",tooltip:t("documentPanel.clearDocuments.tooltip"),size:"sm",children:[x.jsx(sZ,{})," ",t("documentPanel.clearDocuments.button")]})}),x.jsxs(Sl,{className:"sm:max-w-xl",onCloseAutoFocus:b=>b.preventDefault(),children:[x.jsxs(El,{children:[x.jsxs(xl,{className:"flex items-center gap-2 text-red-500 dark:text-red-400 font-bold",children:[x.jsx(s5,{className:"h-5 w-5"}),t("documentPanel.clearDocuments.title")]}),x.jsx(wl,{className:"pt-2",children:t("documentPanel.clearDocuments.description")})]}),x.jsx("div",{className:"text-red-500 dark:text-red-400 font-semibold mb-4",children:t("documentPanel.clearDocuments.warning")}),x.jsx("div",{className:"mb-4",children:t("documentPanel.clearDocuments.confirm")}),x.jsxs("div",{className:"space-y-4",children:[x.jsxs("div",{className:"space-y-2",children:[x.jsx(hP,{htmlFor:"confirm-text",className:"text-sm font-medium",children:t("documentPanel.clearDocuments.confirmPrompt")}),x.jsx(oa,{id:"confirm-text",value:a,onChange:b=>i(b.target.value),placeholder:t("documentPanel.clearDocuments.confirmPlaceholder"),className:"w-full",disabled:l})]}),x.jsxs("div",{className:"flex items-center space-x-2",children:[x.jsx(ol,{id:"clear-cache",checked:o,onCheckedChange:b=>s(b===!0),disabled:l}),x.jsx(hP,{htmlFor:"clear-cache",className:"text-sm font-medium cursor-pointer",children:t("documentPanel.clearDocuments.clearCache")})]})]}),x.jsxs(xg,{children:[x.jsx(Rt,{variant:"outline",onClick:()=>n(!1),disabled:l,children:t("common.cancel")}),x.jsx(Rt,{variant:"destructive",onClick:m,disabled:!p||l,children:l?x.jsxs(x.Fragment,{children:[x.jsx(i5,{className:"mr-2 h-4 w-4 animate-spin"}),t("documentPanel.clearDocuments.clearing")]}):t("documentPanel.clearDocuments.confirmButton")})]})]})]})}const rk=({htmlFor:e,className:t,children:r,...n})=>x.jsx("label",{htmlFor:e,className:t,...n,children:r});function mNe({selectedDocIds:e,onDocumentsDeleted:t}){const{t:r}=Sr(),[n,a]=T.useState(!1),[i,o]=T.useState(""),[s,l]=T.useState(!1),[u,d]=T.useState(!1),[p,f]=T.useState(!1),m=i.toLowerCase()==="yes"&&!u;T.useEffect(()=>{n||(o(""),l(!1),f(!1),d(!1))},[n]);const b=T.useCallback(async()=>{if(!(!m||e.length===0)){d(!0);try{const y=await h1e(e,s,p);if(y.status==="deletion_started")er.success(r("documentPanel.deleteDocuments.success",{count:e.length}));else if(y.status==="busy"){er.error(r("documentPanel.deleteDocuments.busy")),o(""),d(!1);return}else if(y.status==="not_allowed"){er.error(r("documentPanel.deleteDocuments.notAllowed")),o(""),d(!1);return}else{er.error(r("documentPanel.deleteDocuments.failed",{message:y.message})),o(""),d(!1);return}t&&t().catch(console.error),a(!1)}catch(y){er.error(r("documentPanel.deleteDocuments.error",{error:Da(y)})),o("")}finally{d(!1)}}},[m,e,s,p,a,r,t]);return x.jsxs(Du,{open:n,onOpenChange:a,children:[x.jsx(u5,{asChild:!0,children:x.jsxs(Rt,{variant:"destructive",side:"bottom",tooltip:r("documentPanel.deleteDocuments.tooltip",{count:e.length}),size:"sm",children:[x.jsx(XEe,{})," ",r("documentPanel.deleteDocuments.button")]})}),x.jsxs(Sl,{className:"sm:max-w-xl",onCloseAutoFocus:y=>y.preventDefault(),children:[x.jsxs(El,{children:[x.jsxs(xl,{className:"flex items-center gap-2 text-red-500 dark:text-red-400 font-bold",children:[x.jsx(s5,{className:"h-5 w-5"}),r("documentPanel.deleteDocuments.title")]}),x.jsx(wl,{className:"pt-2",children:r("documentPanel.deleteDocuments.description",{count:e.length})})]}),x.jsx("div",{className:"text-red-500 dark:text-red-400 font-semibold mb-4",children:r("documentPanel.deleteDocuments.warning")}),x.jsx("div",{className:"mb-4",children:r("documentPanel.deleteDocuments.confirm",{count:e.length})}),x.jsxs("div",{className:"space-y-4",children:[x.jsxs("div",{className:"space-y-2",children:[x.jsx(rk,{htmlFor:"confirm-text",className:"text-sm font-medium",children:r("documentPanel.deleteDocuments.confirmPrompt")}),x.jsx(oa,{id:"confirm-text",value:i,onChange:y=>o(y.target.value),placeholder:r("documentPanel.deleteDocuments.confirmPlaceholder"),className:"w-full",disabled:u})]}),x.jsxs("div",{className:"flex items-center space-x-2",children:[x.jsx("input",{type:"checkbox",id:"delete-file",checked:s,onChange:y=>l(y.target.checked),disabled:u,className:"h-4 w-4 text-red-600 focus:ring-red-500 border-gray-300 rounded"}),x.jsx(rk,{htmlFor:"delete-file",className:"text-sm font-medium cursor-pointer",children:r("documentPanel.deleteDocuments.deleteFileOption")})]}),x.jsxs("div",{className:"flex items-center space-x-2",children:[x.jsx("input",{type:"checkbox",id:"delete-llm-cache",checked:p,onChange:y=>f(y.target.checked),disabled:u,className:"h-4 w-4 text-red-600 focus:ring-red-500 border-gray-300 rounded"}),x.jsx(rk,{htmlFor:"delete-llm-cache",className:"text-sm font-medium cursor-pointer",children:r("documentPanel.deleteDocuments.deleteLLMCacheOption")})]})]}),x.jsxs(xg,{children:[x.jsx(Rt,{variant:"outline",onClick:()=>a(!1),disabled:u,children:r("common.cancel")}),x.jsx(Rt,{variant:"destructive",onClick:b,disabled:!m,children:r(u?"documentPanel.deleteDocuments.deleting":"documentPanel.deleteDocuments.confirmButton")})]})]})]})}const pP=[{value:10,label:"10"},{value:20,label:"20"},{value:50,label:"50"},{value:100,label:"100"},{value:200,label:"200"}];function gNe({currentPage:e,totalPages:t,pageSize:r,totalCount:n,onPageChange:a,onPageSizeChange:i,isLoading:o=!1,compact:s=!1,className:l}){const{t:u}=Sr(),[d,p]=T.useState(e.toString());T.useEffect(()=>{p(e.toString())},[e]);const f=T.useCallback(_=>{p(_)},[]),m=T.useCallback(()=>{const _=parseInt(d,10);!isNaN(_)&&_>=1&&_<=t?a(_):p(e.toString())},[d,t,a,e]),b=T.useCallback(_=>{_.key==="Enter"&&m()},[m]),y=T.useCallback(_=>{const R=parseInt(_,10);isNaN(R)||i(R)},[i]),S=T.useCallback(()=>{e>1&&!o&&a(1)},[e,a,o]),w=T.useCallback(()=>{e>1&&!o&&a(e-1)},[e,a,o]),E=T.useCallback(()=>{e<t&&!o&&a(e+1)},[e,t,a,o]),k=T.useCallback(()=>{e<t&&!o&&a(t)},[e,t,a,o]);return t<=1?null:s?x.jsxs("div",{className:lt("flex items-center gap-2",l),children:[x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(Rt,{variant:"outline",size:"sm",onClick:w,disabled:e<=1||o,className:"h-8 w-8 p-0",children:x.jsx(t7,{className:"h-4 w-4"})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{type:"text",value:d,onChange:_=>f(_.target.value),onBlur:m,onKeyPress:b,disabled:o,className:"h-8 w-12 text-center text-sm"}),x.jsxs("span",{className:"text-sm text-gray-500",children:["/ ",t]})]}),x.jsx(Rt,{variant:"outline",size:"sm",onClick:E,disabled:e>=t||o,className:"h-8 w-8 p-0",children:x.jsx(r7,{className:"h-4 w-4"})})]}),x.jsxs(Uh,{value:r.toString(),onValueChange:y,disabled:o,children:[x.jsx(Uc,{className:"h-8 w-16",children:x.jsx($h,{})}),x.jsx($c,{children:pP.map(_=>x.jsx($n,{value:_.value.toString(),children:_.label},_.value))})]})]}):x.jsxs("div",{className:lt("flex items-center justify-between gap-4",l),children:[x.jsx("div",{className:"text-sm text-gray-500",children:u("pagination.showing",{start:Math.min((e-1)*r+1,n),end:Math.min(e*r,n),total:n})}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(Rt,{variant:"outline",size:"sm",onClick:S,disabled:e<=1||o,className:"h-8 w-8 p-0",tooltip:u("pagination.firstPage"),children:x.jsx(tEe,{className:"h-4 w-4"})}),x.jsx(Rt,{variant:"outline",size:"sm",onClick:w,disabled:e<=1||o,className:"h-8 w-8 p-0",tooltip:u("pagination.prevPage"),children:x.jsx(t7,{className:"h-4 w-4"})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx("span",{className:"text-sm",children:u("pagination.page")}),x.jsx(oa,{type:"text",value:d,onChange:_=>f(_.target.value),onBlur:m,onKeyPress:b,disabled:o,className:"h-8 w-16 text-center text-sm"}),x.jsxs("span",{className:"text-sm",children:["/ ",t]})]}),x.jsx(Rt,{variant:"outline",size:"sm",onClick:E,disabled:e>=t||o,className:"h-8 w-8 p-0",tooltip:u("pagination.nextPage"),children:x.jsx(r7,{className:"h-4 w-4"})}),x.jsx(Rt,{variant:"outline",size:"sm",onClick:k,disabled:e>=t||o,className:"h-8 w-8 p-0",tooltip:u("pagination.lastPage"),children:x.jsx(nEe,{className:"h-4 w-4"})})]}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx("span",{className:"text-sm",children:u("pagination.pageSize")}),x.jsxs(Uh,{value:r.toString(),onValueChange:y,disabled:o,children:[x.jsx(Uc,{className:"h-8 w-16",children:x.jsx($h,{})}),x.jsx($c,{children:pP.map(_=>x.jsx($n,{value:_.value.toString(),children:_.label},_.value))})]})]})]})]})}function bNe({open:e,onOpenChange:t}){var y;const{t:r}=Sr(),[n,a]=T.useState(null),[i,o]=T.useState("center"),[s,l]=T.useState(!1),[u,d]=T.useState(!1),p=T.useRef(null);T.useEffect(()=>{e?(o("center"),l(!1)):d(!1)},[e]),T.useEffect(()=>{const S=p.current;!S||s||(S.scrollTop=S.scrollHeight)},[n==null?void 0:n.history_messages,s]);const f=()=>{const S=p.current;if(!S)return;const w=Math.abs(S.scrollHeight-S.scrollTop-S.clientHeight)<1;l(!w)};T.useEffect(()=>{if(!e)return;const S=async()=>{try{const E=await p1e();a(E)}catch(E){er.error(r("documentPanel.pipelineStatus.errors.fetchFailed",{error:Da(E)}))}};S();const w=setInterval(S,2e3);return()=>clearInterval(w)},[e,r]);const m=async()=>{d(!1);try{const S=await f1e();S.status==="cancellation_requested"?er.success(r("documentPanel.pipelineStatus.cancelSuccess")):S.status==="not_busy"&&er.info(r("documentPanel.pipelineStatus.cancelNotBusy"))}catch(S){er.error(r("documentPanel.pipelineStatus.cancelFailed",{error:Da(S)}))}},b=(n==null?void 0:n.busy)===!0&&!(n!=null&&n.cancellation_requested);return x.jsxs(Du,{open:e,onOpenChange:t,children:[x.jsxs(Sl,{className:lt("sm:max-w-[800px] transition-all duration-200 fixed",i==="left"&&"!left-[25%] !translate-x-[-50%] !mx-4",i==="center"&&"!left-1/2 !-translate-x-1/2",i==="right"&&"!left-[75%] !translate-x-[-50%] !mx-4"),children:[x.jsx(wl,{className:"sr-only",children:n!=null&&n.job_name?`${r("documentPanel.pipelineStatus.jobName")}: ${n.job_name}, ${r("documentPanel.pipelineStatus.progress")}: ${n.cur_batch}/${n.batchs}`:r("documentPanel.pipelineStatus.noActiveJob")}),x.jsxs(El,{className:"flex flex-row items-center",children:[x.jsx(xl,{className:"flex-1",children:r("documentPanel.pipelineStatus.title")}),x.jsxs("div",{className:"flex items-center gap-2 mr-8",children:[x.jsx(Rt,{variant:"ghost",size:"icon",className:lt("h-6 w-6",i==="left"&&"bg-zinc-200 text-zinc-800 hover:bg-zinc-300 dark:bg-zinc-700 dark:text-zinc-200 dark:hover:bg-zinc-600"),onClick:()=>o("left"),children:x.jsx(USe,{className:"h-4 w-4"})}),x.jsx(Rt,{variant:"ghost",size:"icon",className:lt("h-6 w-6",i==="center"&&"bg-zinc-200 text-zinc-800 hover:bg-zinc-300 dark:bg-zinc-700 dark:text-zinc-200 dark:hover:bg-zinc-600"),onClick:()=>o("center"),children:x.jsx(zSe,{className:"h-4 w-4"})}),x.jsx(Rt,{variant:"ghost",size:"icon",className:lt("h-6 w-6",i==="right"&&"bg-zinc-200 text-zinc-800 hover:bg-zinc-300 dark:bg-zinc-700 dark:text-zinc-200 dark:hover:bg-zinc-600"),onClick:()=>o("right"),children:x.jsx(GSe,{className:"h-4 w-4"})})]})]}),x.jsxs("div",{className:"space-y-4 pt-4",children:[x.jsxs("div",{className:"flex flex-wrap items-center justify-between gap-4",children:[x.jsxs("div",{className:"flex items-center gap-4",children:[x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsxs("div",{className:"text-sm font-medium",children:[r("documentPanel.pipelineStatus.busy"),":"]}),x.jsx("div",{className:`h-2 w-2 rounded-full ${n!=null&&n.busy?"bg-green-500":"bg-gray-300"}`})]}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsxs("div",{className:"text-sm font-medium",children:[r("documentPanel.pipelineStatus.requestPending"),":"]}),x.jsx("div",{className:`h-2 w-2 rounded-full ${n!=null&&n.request_pending?"bg-green-500":"bg-gray-300"}`})]}),(n==null?void 0:n.cancellation_requested)&&x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsxs("div",{className:"text-sm font-medium",children:[r("documentPanel.pipelineStatus.cancellationRequested"),":"]}),x.jsx("div",{className:"h-2 w-2 rounded-full bg-red-500"})]})]}),(n==null?void 0:n.busy)&&x.jsx(Rt,{variant:"destructive",size:"sm",disabled:!b,onClick:()=>d(!0),title:n!=null&&n.cancellation_requested?r("documentPanel.pipelineStatus.cancelInProgress"):r("documentPanel.pipelineStatus.cancelTooltip"),children:r("documentPanel.pipelineStatus.cancelButton")})]}),x.jsxs("div",{className:"rounded-md border p-3 space-y-2",children:[x.jsxs("div",{children:[r("documentPanel.pipelineStatus.jobName"),": ",(n==null?void 0:n.job_name)||"-"]}),x.jsxs("div",{className:"flex justify-between",children:[x.jsxs("span",{children:[r("documentPanel.pipelineStatus.startTime"),": ",n!=null&&n.job_start?new Date(n.job_start).toLocaleString(void 0,{year:"numeric",month:"numeric",day:"numeric",hour:"numeric",minute:"numeric",second:"numeric"}):"-"]}),x.jsxs("span",{children:[r("documentPanel.pipelineStatus.progress"),": ",n?`${n.cur_batch}/${n.batchs} ${r("documentPanel.pipelineStatus.unit")}`:"-"]})]})]}),x.jsxs("div",{className:"space-y-2",children:[x.jsxs("div",{className:"text-sm font-medium",children:[r("documentPanel.pipelineStatus.pipelineMessages"),":"]}),x.jsx("div",{ref:p,onScroll:f,className:"font-mono text-xs rounded-md bg-zinc-800 text-zinc-100 p-3 overflow-y-auto overflow-x-hidden min-h-[7.5em] max-h-[40vh]",children:(y=n==null?void 0:n.history_messages)!=null&&y.length?n.history_messages.map((S,w)=>x.jsx("div",{className:"whitespace-pre-wrap break-all",children:S},w)):"-"})]})]})]}),x.jsx(Du,{open:u,onOpenChange:d,children:x.jsxs(Sl,{className:"sm:max-w-[425px]",children:[x.jsxs(El,{children:[x.jsx(xl,{children:r("documentPanel.pipelineStatus.cancelConfirmTitle")}),x.jsx(wl,{children:r("documentPanel.pipelineStatus.cancelConfirmDescription")})]}),x.jsxs("div",{className:"flex justify-end gap-3 mt-4",children:[x.jsx(Rt,{variant:"outline",onClick:()=>d(!1),children:r("common.cancel")}),x.jsx(Rt,{variant:"destructive",onClick:m,children:r("documentPanel.pipelineStatus.cancelConfirmButton")})]})]})})]})}const bu=(e,...t)=>{for(const r of t){const n=e[r];if(typeof n=="number")return n}return 0},nk=e=>bu(e,"PROCESSING","processing")>0||bu(e,"PENDING","pending")>0||bu(e,"PREPROCESSED","preprocessed")>0,ak=(e,t=20)=>{if(!e.file_path||typeof e.file_path!="string"||e.file_path.trim()==="")return e.id;const r=e.file_path.split("/"),n=r[r.length-1];return!n||n.trim()===""?e.id:n.length>t?n.slice(0,t)+"...":n},yNe=e=>{const t={...e};if(t.processing_start_time&&typeof t.processing_start_time=="number"){const a=new Date(t.processing_start_time*1e3);isNaN(a.getTime())||(t.processing_start_time=a.toLocaleString())}if(t.processing_end_time&&typeof t.processing_end_time=="number"){const a=new Date(t.processing_end_time*1e3);isNaN(a.getTime())||(t.processing_end_time=a.toLocaleString())}return JSON.stringify(t,null,2).split(`
`).slice(1,-1).map(a=>a.replace(/^ {2}/,"")).join(`
`)},vNe=`
/* Tooltip styles */
.tooltip-container {
  position: relative;
  overflow: visible !important;
}

.tooltip {
  position: fixed; /* Use fixed positioning to escape overflow constraints */
  z-index: 9999; /* Ensure tooltip appears above all other elements */
  max-width: 600px;
  white-space: normal;
  word-break: break-word;
  overflow-wrap: break-word;
  border-radius: 0.375rem;
  padding: 0.5rem 0.75rem;
  font-size: 0.75rem; /* 12px */
  background-color: rgba(0, 0, 0, 0.95);
  color: white;
  box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
  pointer-events: none; /* Prevent tooltip from interfering with mouse events */
  opacity: 0;
  visibility: hidden;
  transition: opacity 0.15s, visibility 0.15s;
}

.tooltip.visible {
  opacity: 1;
  visibility: visible;
}

.dark .tooltip {
  background-color: rgba(255, 255, 255, 0.95);
  color: black;
}

.tooltip pre {
  white-space: pre-wrap;
  word-break: break-word;
  overflow-wrap: break-word;
}

/* Position tooltip helper class */
.tooltip-helper {
  position: absolute;
  visibility: hidden;
  pointer-events: none;
  top: 0;
  left: 0;
  width: 100%;
  height: 0;
}

@keyframes pulse {
  0% {
    background-color: rgb(255 0 0 / 0.1);
    border-color: rgb(255 0 0 / 0.2);
  }
  50% {
    background-color: rgb(255 0 0 / 0.2);
    border-color: rgb(255 0 0 / 0.4);
  }
  100% {
    background-color: rgb(255 0 0 / 0.1);
    border-color: rgb(255 0 0 / 0.2);
  }
}

.dark .pipeline-busy {
  animation: dark-pulse 2s infinite;
}

@keyframes dark-pulse {
  0% {
    background-color: rgb(255 0 0 / 0.2);
    border-color: rgb(255 0 0 / 0.4);
  }
  50% {
    background-color: rgb(255 0 0 / 0.3);
    border-color: rgb(255 0 0 / 0.6);
  }
  100% {
    background-color: rgb(255 0 0 / 0.2);
    border-color: rgb(255 0 0 / 0.4);
  }
}

.pipeline-busy {
  animation: pulse 2s infinite;
  border: 1px solid;
}
`;function SNe(){const e=T.useRef(!0);T.useEffect(()=>{e.current=!0;const Ce=()=>{e.current=!1};return window.addEventListener("beforeunload",Ce),()=>{e.current=!1,window.removeEventListener("beforeunload",Ce)}},[]);const[t,r]=T.useState(!1),{t:n,i18n:a}=Sr(),i=Xn.use.health(),o=Xn.use.pipelineBusy(),[s,l]=T.useState(null),u=Ve.use.currentTab(),d=Ve.use.showFileName(),p=Ve.use.setShowFileName(),f=Ve.use.documentsPageSize(),m=Ve.use.setDocumentsPageSize(),[b,y]=T.useState([]),[S,w]=T.useState({page:1,page_size:f,total_count:0,total_pages:0,has_next:!1,has_prev:!1}),[E,k]=T.useState({all:0}),[_,R]=T.useState(!1),[C,F]=T.useState("updated_at"),[L,I]=T.useState("desc"),[M,j]=T.useState("all"),[Y,W]=T.useState({all:1,processed:1,preprocessed:1,processing:1,pending:1,failed:1}),[$,X]=T.useState([]),z=$.length>0,H=T.useRef(void 0),B=T.useRef(null),[ee,J]=T.useState({count:0,lastError:null,isBackingOff:!1}),[K,G]=T.useState({isOpen:!1,failureCount:0,lastFailureTime:null,nextRetryTime:null}),P=T.useCallback((Ce,st)=>{X(ft=>st?[...ft,Ce]:ft.filter(vt=>vt!==Ce))},[]),ne=T.useCallback(()=>{X([])},[]),oe=Ce=>{let st=Ce;Ce==="id"&&(st=d?"file_path":"id");const ft=C===st&&L==="desc"?"asc":"desc";F(st),I(ft),w(vt=>({...vt,page:1})),W({all:1,processed:1,preprocessed:1,processing:1,pending:1,failed:1})},pe=T.useCallback(Ce=>[...Ce].sort((st,ft)=>{let vt,yr;C==="id"&&d?(vt=ak(st),yr=ak(ft)):C==="id"?(vt=st.id,yr=ft.id):(vt=new Date(st[C]).getTime(),yr=new Date(ft[C]).getTime());const Qe=L==="asc"?1:-1;return typeof vt=="string"&&typeof yr=="string"?Qe*vt.localeCompare(yr):Qe*(vt>yr?1:vt<yr?-1:0)}),[C,L,d]),ae=T.useMemo(()=>{if(b&&b.length>0)return b.map(st=>({...st,status:st.status}));if(!s)return null;const Ce=[];return M==="all"?Object.entries(s.statuses).forEach(([st,ft])=>{ft.forEach(vt=>{Ce.push({...vt,status:st})})}):(s.statuses[M]||[]).forEach(ft=>{Ce.push({...ft,status:M})}),C&&L?pe(Ce):Ce},[b,s,C,L,M,pe]),se=T.useMemo(()=>(ae==null?void 0:ae.map(Ce=>Ce.id))||[],[ae]),Ee=T.useMemo(()=>se.filter(Ce=>$.includes(Ce)).length,[se,$]),fe=T.useMemo(()=>se.length>0&&Ee===se.length,[se,Ee]),Me=T.useMemo(()=>Ee>0,[Ee]),Oe=T.useCallback(()=>{X(se)},[se]),xe=T.useCallback(()=>Me?fe?{text:n("documentPanel.selectDocuments.deselectAll",{count:se.length}),action:ne,icon:Qv}:{text:n("documentPanel.selectDocuments.selectCurrentPage",{count:se.length}),action:Oe,icon:i7}:{text:n("documentPanel.selectDocuments.selectCurrentPage",{count:se.length}),action:Oe,icon:i7},[Me,fe,se.length,Oe,ne,n]),Te=T.useMemo(()=>{if(!s)return{all:0};const Ce={all:0};return Object.entries(s.statuses).forEach(([st,ft])=>{Ce[st]=ft.length,Ce.all+=ft.length}),Ce},[s]),Ne=bu(E,"PROCESSED","processed")||Te.processed||0,he=bu(E,"PREPROCESSED","preprocessed")||Te.preprocessed||0,Ze=bu(E,"PROCESSING","processing")||Te.processing||0,Et=bu(E,"PENDING","pending")||Te.pending||0,qe=bu(E,"FAILED","failed")||Te.failed||0,Ye=T.useRef({processed:0,preprocessed:0,processing:0,pending:0,failed:0});T.useEffect(()=>{const Ce=document.createElement("style");return Ce.textContent=vNe,document.head.appendChild(Ce),()=>{document.head.removeChild(Ce)}},[]);const we=T.useRef(null);T.useEffect(()=>{if(!s)return;const Ce=()=>{document.querySelectorAll(".tooltip-container").forEach(yr=>{const Qe=yr.querySelector(".tooltip");if(!Qe||!Qe.classList.contains("visible"))return;const zt=yr.getBoundingClientRect();Qe.style.left=`${zt.left}px`,Qe.style.top=`${zt.top-5}px`,Qe.style.transform="translateY(-100%)"})},st=vt=>{const Qe=vt.target.closest(".tooltip-container");if(!Qe)return;const zt=Qe.querySelector(".tooltip");zt&&(zt.classList.add("visible"),Ce())},ft=vt=>{const Qe=vt.target.closest(".tooltip-container");if(!Qe)return;const zt=Qe.querySelector(".tooltip");zt&&zt.classList.remove("visible")};return document.addEventListener("mouseover",st),document.addEventListener("mouseout",ft),()=>{document.removeEventListener("mouseover",st),document.removeEventListener("mouseout",ft)}},[s]);const Qt=T.useCallback(Ce=>{w(Ce.pagination),y(Ce.documents),k(Ce.status_counts);const st={statuses:{processed:Ce.documents.filter(ft=>ft.status==="processed"),preprocessed:Ce.documents.filter(ft=>ft.status==="preprocessed"),processing:Ce.documents.filter(ft=>ft.status==="processing"),pending:Ce.documents.filter(ft=>ft.status==="pending"),failed:Ce.documents.filter(ft=>ft.status==="failed")}};l(Ce.pagination.total_count>0?st:null)},[]),or=T.useCallback((Ce,st=3e4,ft="Request timeout")=>{const vt=new Promise((yr,Qe)=>{setTimeout(()=>Qe(new Error(ft)),st)});return Promise.race([Ce,vt])},[]),Pe=T.useCallback(Ce=>{var st;return Ce.name==="AbortError"?{type:"cancelled",shouldRetry:!1,shouldShowToast:!1}:Ce.message==="Request timeout"?{type:"timeout",shouldRetry:!0,shouldShowToast:!0}:(st=Ce.message)!=null&&st.includes("Network Error")||Ce.code==="NETWORK_ERROR"?{type:"network",shouldRetry:!0,shouldShowToast:!0}:Ce.status>=500?{type:"server",shouldRetry:!0,shouldShowToast:!0}:Ce.status>=400&&Ce.status<500?{type:"client",shouldRetry:!1,shouldShowToast:!0}:{type:"unknown",shouldRetry:!0,shouldShowToast:!0}},[]),rt=T.useCallback(()=>{if(!K.isOpen)return!1;const Ce=Date.now();return K.nextRetryTime&&Ce>=K.nextRetryTime?(G(st=>({...st,isOpen:!1,failureCount:Math.max(0,st.failureCount-1)})),!1):!0},[K]),ge=T.useCallback(Ce=>{const st=Date.now();G(ft=>{const vt=ft.failureCount+1,yr=vt>=3;return{isOpen:yr,failureCount:vt,lastFailureTime:st,nextRetryTime:yr?st+Math.pow(2,vt)*1e3:null}}),J(ft=>({count:ft.count+1,lastError:Ce,isBackingOff:!0}))},[]),Re=T.useCallback(()=>{G({isOpen:!1,failureCount:0,lastFailureTime:null,nextRetryTime:null}),J({count:0,lastError:null,isBackingOff:!1})},[]),We=T.useCallback(async(Ce,st)=>{try{if(!e.current)return;R(!0);const ft=st?1:Ce||S.page,vt={status_filter:M==="all"?null:M,page:ft,page_size:S.page_size,sort_field:C,sort_direction:L},yr=await or(Gx(vt),3e4,"Document fetch timeout");if(!e.current)return;if(yr.documents.length===0&&yr.pagination.total_count>0){const Qe=Math.max(1,yr.pagination.total_pages);if(ft!==Qe){const zt={...vt,page:Qe},Nr=await or(Gx(zt),3e4,"Document fetch timeout");if(!e.current)return;W(yn=>({...yn,[M]:Qe})),Qt(Nr);return}}ft!==S.page&&W(Qe=>({...Qe,[M]:ft})),Qt(yr)}catch(ft){if(e.current){const vt=Pe(ft);vt.shouldShowToast&&er.error(n("documentPanel.documentManager.errors.loadFailed",{error:Da(ft)})),vt.shouldRetry&&ge(ft)}}finally{e.current&&R(!1)}},[M,S.page,S.page_size,C,L,n,Qt,or,Pe,ge]),ht=T.useCallback(async(Ce,st,ft)=>{w(vt=>({...vt,page:Ce,page_size:st})),await We(Ce)},[We]),xt=T.useCallback(async()=>{await ht(S.page,S.page_size,M)},[ht,S.page,S.page_size,M]),sr=T.useCallback(()=>{B.current&&(clearInterval(B.current),B.current=null)},[]),ar=T.useCallback(Ce=>{sr(),B.current=setInterval(async()=>{try{if(rt())return;e.current&&(await xt(),Re())}catch(st){if(e.current){const ft=Pe(st);if(R(!1),ft.shouldShowToast&&er.error(n("documentPanel.documentManager.errors.scanProgressFailed",{error:Da(st)})),ft.shouldRetry){ge(st);const vt=Math.min(Math.pow(2,ee.count)*1e3,3e4);ee.count<3&&setTimeout(()=>{e.current&&J(yr=>({...yr,isBackingOff:!1}))},vt)}else sr()}}},Ce)},[xt,n,sr,rt,Re,ge,Pe,ee.count]),fr=T.useCallback(async()=>{try{if(!e.current)return;const{status:Ce,message:st,track_id:ft}=await o1e();if(!e.current)return;er.message(st||Ce),Xn.getState().resetHealthCheckTimerDelayed(1e3),ar(2e3),setTimeout(()=>{if(e.current&&u==="documents"&&i){const yr=nk(E)?5e3:3e4;ar(yr)}},15e3)}catch(Ce){e.current&&er.error(n("documentPanel.documentManager.errors.scanFailed",{error:Da(Ce)}))}},[n,ar,u,i,E]),Mt=T.useCallback(Ce=>{Ce!==S.page_size&&(m(Ce),W({all:1,processed:1,preprocessed:1,processing:1,pending:1,failed:1}),w(st=>({...st,page:1,page_size:Ce})))},[S.page_size,m]),lr=T.useCallback(async()=>{try{R(!0);const Ce={status_filter:M==="all"?null:M,page:1,page_size:S.page_size,sort_field:C,sort_direction:L},st=await Gx(Ce);if(!e.current)return;if(st.pagination.total_count<S.page_size&&S.page_size!==10)Mt(10);else{w(st.pagination),y(st.documents),k(st.status_counts);const ft={statuses:{processed:st.documents.filter(vt=>vt.status==="processed"),preprocessed:st.documents.filter(vt=>vt.status==="preprocessed"),processing:st.documents.filter(vt=>vt.status==="processing"),pending:st.documents.filter(vt=>vt.status==="pending"),failed:st.documents.filter(vt=>vt.status==="failed")}};st.pagination.total_count>0?l(ft):l(null)}}catch(Ce){e.current&&er.error(n("documentPanel.documentManager.errors.loadFailed",{error:Da(Ce)}))}finally{e.current&&R(!1)}},[M,S.page_size,C,L,Mt,n]);T.useEffect(()=>{if(H.current!==void 0&&H.current!==o&&u==="documents"&&i&&e.current){We();const st=nk(E)?5e3:3e4;ar(st)}H.current=o},[o,u,i,We,E,ar]),T.useEffect(()=>{if(u!=="documents"||!i){sr();return}const st=nk(E)?5e3:3e4;return ar(st),()=>{sr()}},[i,n,u,E,ar,sr]),T.useEffect(()=>{var ft,vt,yr,Qe,zt,Nr,yn,hn,Ea,xa;if(!s)return;const Ce={processed:((vt=(ft=s==null?void 0:s.statuses)==null?void 0:ft.processed)==null?void 0:vt.length)||0,preprocessed:((Qe=(yr=s==null?void 0:s.statuses)==null?void 0:yr.preprocessed)==null?void 0:Qe.length)||0,processing:((Nr=(zt=s==null?void 0:s.statuses)==null?void 0:zt.processing)==null?void 0:Nr.length)||0,pending:((hn=(yn=s==null?void 0:s.statuses)==null?void 0:yn.pending)==null?void 0:hn.length)||0,failed:((xa=(Ea=s==null?void 0:s.statuses)==null?void 0:Ea.failed)==null?void 0:xa.length)||0};Object.keys(Ce).some(Vn=>Ce[Vn]!==Ye.current[Vn])&&e.current&&Xn.getState().check(),Ye.current=Ce},[s]);const qt=T.useCallback(Ce=>{Ce!==S.page&&(W(st=>({...st,[M]:Ce})),w(st=>({...st,page:Ce})))},[S.page,M]),zr=T.useCallback(Ce=>{if(Ce===M)return;W(ft=>({...ft,[M]:S.page}));const st=Y[Ce];j(Ce),w(ft=>({...ft,page:st}))},[M,S.page,Y]),nn=T.useCallback(async()=>{X([]),Xn.getState().resetHealthCheckTimerDelayed(1e3),ar(2e3)},[ar]),dn=T.useCallback(async()=>{if(sr(),k({all:0,processed:0,processing:0,pending:0,failed:0}),e.current)try{await xt()}catch(Ce){console.error("Error fetching documents after clear:",Ce)}u==="documents"&&i&&e.current&&ar(3e4)},[sr,k,xt,u,i,ar]);return T.useEffect(()=>{if(C==="id"||C==="file_path"){const Ce=d?"file_path":"id";C!==Ce&&F(Ce)}},[d,C]),T.useEffect(()=>{X([])},[S.page,M,C,L]),T.useEffect(()=>{u==="documents"&&ht(S.page,S.page_size,M)},[u,S.page,S.page_size,M,C,L,ht]),x.jsxs(qc,{className:"!rounded-none !overflow-hidden flex flex-col h-full min-h-0",children:[x.jsx(Hm,{className:"py-2 px-6",children:x.jsx(Um,{className:"text-lg",children:n("documentPanel.documentManager.title")})}),x.jsxs($m,{className:"flex-1 flex flex-col min-h-0 overflow-auto",children:[x.jsxs("div",{className:"flex justify-between items-center gap-2 mb-2",children:[x.jsxs("div",{className:"flex gap-2",children:[x.jsxs(Rt,{variant:"outline",onClick:fr,side:"bottom",tooltip:n("documentPanel.documentManager.scanTooltip"),size:"sm",children:[x.jsx(uZ,{})," ",n("documentPanel.documentManager.scanButton")]}),x.jsxs(Rt,{variant:"outline",onClick:()=>r(!0),side:"bottom",tooltip:n("documentPanel.documentManager.pipelineStatusTooltip"),size:"sm",className:lt(o&&"pipeline-busy"),children:[x.jsx(BSe,{})," ",n("documentPanel.documentManager.pipelineStatusButton")]})]}),S.total_pages>1&&x.jsx(gNe,{currentPage:S.page,totalPages:S.total_pages,pageSize:S.page_size,totalCount:S.total_count,onPageChange:qt,onPageSizeChange:Mt,isLoading:_,compact:!0}),x.jsxs("div",{className:"flex gap-2",children:[z&&x.jsx(mNe,{selectedDocIds:$,onDocumentsDeleted:nn}),z&&Me?(()=>{const Ce=xe(),st=Ce.icon;return x.jsxs(Rt,{variant:"outline",size:"sm",onClick:Ce.action,side:"bottom",tooltip:Ce.text,children:[x.jsx(st,{className:"h-4 w-4"}),Ce.text]})})():z?null:x.jsx(fNe,{onDocumentsCleared:dn}),x.jsx(pNe,{onDocumentsUploaded:xt}),x.jsx(bNe,{open:t,onOpenChange:r})]})]}),x.jsxs(qc,{className:"flex-1 flex flex-col border rounded-md min-h-0 mb-2",children:[x.jsxs(Hm,{className:"flex-none py-2 px-4",children:[x.jsxs("div",{className:"flex justify-between items-center",children:[x.jsx(Um,{children:n("documentPanel.documentManager.uploadedTitle")}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsxs("div",{className:"flex gap-1",dir:a.dir(),children:[x.jsxs(Rt,{size:"sm",variant:M==="all"?"secondary":"outline",onClick:()=>zr("all"),disabled:_,className:lt(M==="all"&&"bg-gray-100 dark:bg-gray-900 font-medium border border-gray-400 dark:border-gray-500 shadow-sm"),children:[n("documentPanel.documentManager.status.all")," (",E.all||Te.all,")"]}),x.jsxs(Rt,{size:"sm",variant:M==="processed"?"secondary":"outline",onClick:()=>zr("processed"),disabled:_,className:lt(Ne>0?"text-green-600":"text-gray-500",M==="processed"&&"bg-green-100 dark:bg-green-900/30 font-medium border border-green-400 dark:border-green-600 shadow-sm"),children:[n("documentPanel.documentManager.status.completed")," (",Ne,")"]}),x.jsxs(Rt,{size:"sm",variant:M==="preprocessed"?"secondary":"outline",onClick:()=>zr("preprocessed"),disabled:_,className:lt(he>0?"text-purple-600":"text-gray-500",M==="preprocessed"&&"bg-purple-100 dark:bg-purple-900/30 font-medium border border-purple-400 dark:border-purple-600 shadow-sm"),children:[n("documentPanel.documentManager.status.preprocessed")," (",he,")"]}),x.jsxs(Rt,{size:"sm",variant:M==="processing"?"secondary":"outline",onClick:()=>zr("processing"),disabled:_,className:lt(Ze>0?"text-blue-600":"text-gray-500",M==="processing"&&"bg-blue-100 dark:bg-blue-900/30 font-medium border border-blue-400 dark:border-blue-600 shadow-sm"),children:[n("documentPanel.documentManager.status.processing")," (",Ze,")"]}),x.jsxs(Rt,{size:"sm",variant:M==="pending"?"secondary":"outline",onClick:()=>zr("pending"),disabled:_,className:lt(Et>0?"text-yellow-600":"text-gray-500",M==="pending"&&"bg-yellow-100 dark:bg-yellow-900/30 font-medium border border-yellow-400 dark:border-yellow-600 shadow-sm"),children:[n("documentPanel.documentManager.status.pending")," (",Et,")"]}),x.jsxs(Rt,{size:"sm",variant:M==="failed"?"secondary":"outline",onClick:()=>zr("failed"),disabled:_,className:lt(qe>0?"text-red-600":"text-gray-500",M==="failed"&&"bg-red-100 dark:bg-red-900/30 font-medium border border-red-400 dark:border-red-600 shadow-sm"),children:[n("documentPanel.documentManager.status.failed")," (",qe,")"]})]}),x.jsx(Rt,{variant:"ghost",size:"sm",onClick:lr,disabled:_,side:"bottom",tooltip:n("documentPanel.documentManager.refreshTooltip"),children:x.jsx(o5,{className:"h-4 w-4"})})]}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx("label",{htmlFor:"toggle-filename-btn",className:"text-sm text-gray-500",children:n("documentPanel.documentManager.fileNameLabel")}),x.jsx(Rt,{id:"toggle-filename-btn",variant:"outline",size:"sm",onClick:()=>p(!d),className:"border-gray-200 dark:border-gray-700 hover:bg-gray-100 dark:hover:bg-gray-800",children:n(d?"documentPanel.documentManager.hideButton":"documentPanel.documentManager.showButton")})]})]}),x.jsx(m2,{"aria-hidden":"true",className:"hidden",children:n("documentPanel.documentManager.uploadedDescription")})]}),x.jsxs($m,{className:"flex-1 relative p-0",ref:we,children:[!s&&x.jsx("div",{className:"absolute inset-0 p-0",children:x.jsx(u5e,{title:n("documentPanel.documentManager.emptyTitle"),description:n("documentPanel.documentManager.emptyDescription")})}),s&&x.jsx("div",{className:"absolute inset-0 flex flex-col p-0",children:x.jsx("div",{className:"absolute inset-[-1px] flex flex-col p-0 border rounded-md border-gray-200 dark:border-gray-700 overflow-hidden",children:x.jsxs(Gte,{className:"w-full",children:[x.jsx(jte,{className:"sticky top-0 bg-background z-10 shadow-sm",children:x.jsxs(_R,{className:"border-b bg-card/95 backdrop-blur supports-[backdrop-filter]:bg-card/75 shadow-[inset_0_-1px_0_rgba(0,0,0,0.1)]",children:[x.jsx(Qs,{onClick:()=>oe("id"),className:"cursor-pointer hover:bg-gray-200 dark:hover:bg-gray-800 select-none",children:x.jsxs("div",{className:"flex items-center",children:[n(d?"documentPanel.documentManager.columns.fileName":"documentPanel.documentManager.columns.id"),(C==="id"&&!d||C==="file_path"&&d)&&x.jsx("span",{className:"ml-1",children:L==="asc"?x.jsx(ew,{size:14}):x.jsx(Qx,{size:14})})]})}),x.jsx(Qs,{children:n("documentPanel.documentManager.columns.summary")}),x.jsx(Qs,{children:n("documentPanel.documentManager.columns.status")}),x.jsx(Qs,{children:n("documentPanel.documentManager.columns.length")}),x.jsx(Qs,{children:n("documentPanel.documentManager.columns.chunks")}),x.jsx(Qs,{onClick:()=>oe("created_at"),className:"cursor-pointer hover:bg-gray-200 dark:hover:bg-gray-800 select-none",children:x.jsxs("div",{className:"flex items-center",children:[n("documentPanel.documentManager.columns.created"),C==="created_at"&&x.jsx("span",{className:"ml-1",children:L==="asc"?x.jsx(ew,{size:14}):x.jsx(Qx,{size:14})})]})}),x.jsx(Qs,{onClick:()=>oe("updated_at"),className:"cursor-pointer hover:bg-gray-200 dark:hover:bg-gray-800 select-none",children:x.jsxs("div",{className:"flex items-center",children:[n("documentPanel.documentManager.columns.updated"),C==="updated_at"&&x.jsx("span",{className:"ml-1",children:L==="asc"?x.jsx(ew,{size:14}):x.jsx(Qx,{size:14})})]})}),x.jsx(Qs,{className:"w-16 text-center",children:n("documentPanel.documentManager.columns.select")})]})}),x.jsx(qte,{className:"text-sm overflow-auto",children:ae&&ae.map(Ce=>x.jsxs(_R,{children:[x.jsx(el,{className:"truncate font-mono overflow-visible max-w-[250px]",children:d?x.jsxs(x.Fragment,{children:[x.jsxs("div",{className:"group relative overflow-visible tooltip-container",children:[x.jsx("div",{className:"truncate",children:ak(Ce,30)}),x.jsx("div",{className:"invisible group-hover:visible tooltip",children:Ce.file_path})]}),x.jsx("div",{className:"text-xs text-gray-500",children:Ce.id})]}):x.jsxs("div",{className:"group relative overflow-visible tooltip-container",children:[x.jsx("div",{className:"truncate",children:Ce.id}),x.jsx("div",{className:"invisible group-hover:visible tooltip",children:Ce.file_path})]})}),x.jsx(el,{className:"max-w-xs min-w-45 truncate overflow-visible",children:x.jsxs("div",{className:"group relative overflow-visible tooltip-container",children:[x.jsx("div",{className:"truncate",children:Ce.content_summary}),x.jsx("div",{className:"invisible group-hover:visible tooltip",children:Ce.content_summary})]})}),x.jsx(el,{children:x.jsxs("div",{className:"group relative flex items-center overflow-visible tooltip-container",children:[Ce.status==="processed"&&x.jsx("span",{className:"text-green-600",children:n("documentPanel.documentManager.status.completed")}),Ce.status==="preprocessed"&&x.jsx("span",{className:"text-purple-600",children:n("documentPanel.documentManager.status.preprocessed")}),Ce.status==="processing"&&x.jsx("span",{className:"text-blue-600",children:n("documentPanel.documentManager.status.processing")}),Ce.status==="pending"&&x.jsx("span",{className:"text-yellow-600",children:n("documentPanel.documentManager.status.pending")}),Ce.status==="failed"&&x.jsx("span",{className:"text-red-600",children:n("documentPanel.documentManager.status.failed")}),Ce.error_msg?x.jsx(s5,{className:"ml-2 h-4 w-4 text-yellow-500"}):Ce.metadata&&Object.keys(Ce.metadata).length>0&&x.jsx(SEe,{className:"ml-2 h-4 w-4 text-blue-500"}),(Ce.error_msg||Ce.metadata&&Object.keys(Ce.metadata).length>0||Ce.track_id)&&x.jsxs("div",{className:"invisible group-hover:visible tooltip",children:[Ce.track_id&&x.jsxs("div",{className:"mt-1",children:["Track ID: ",Ce.track_id]}),Ce.metadata&&Object.keys(Ce.metadata).length>0&&x.jsx("pre",{children:yNe(Ce.metadata)}),Ce.error_msg&&x.jsx("pre",{children:Ce.error_msg})]})]})}),x.jsx(el,{children:Ce.content_length??"-"}),x.jsx(el,{children:Ce.chunks_count??"-"}),x.jsx(el,{className:"truncate",children:new Date(Ce.created_at).toLocaleString()}),x.jsx(el,{className:"truncate",children:new Date(Ce.updated_at).toLocaleString()}),x.jsx(el,{className:"text-center",children:x.jsx(ol,{checked:$.includes(Ce.id),onCheckedChange:st=>P(Ce.id,st===!0),className:"mx-auto"})})]},Ce.id))})]})})})]})]})]})]})}const nre=T.forwardRef(({className:e,...t},r)=>x.jsx("textarea",{className:lt("border-input file:text-foreground placeholder:text-muted-foreground focus-visible:ring-ring flex min-h-[60px] w-full rounded-md border bg-transparent px-3 py-2 text-base shadow-sm transition-colors file:border-0 file:bg-transparent file:text-sm file:font-medium focus-visible:ring-1 focus-visible:outline-none disabled:cursor-not-allowed disabled:opacity-50 md:text-sm resize-none",e),ref:r,...t}));nre.displayName="Textarea";function ENe({value:e,onChange:t,placeholder:r,className:n,id:a,history:i,onSelectFromHistory:o,onDeleteFromHistory:s}){const[l,u]=T.useState(!1),[d,p]=T.useState(-1),[f,m]=T.useState(!1),b=T.useRef(null),y=T.useRef(null);T.useEffect(()=>{const F=L=>{b.current&&!b.current.contains(L.target)&&(u(!1),p(-1))};return document.addEventListener("mousedown",F),()=>{document.removeEventListener("mousedown",F)}},[]);const S=T.useCallback(F=>{if(!l){F.key==="ArrowDown"&&i.length>0&&(F.preventDefault(),u(!0),p(0));return}switch(F.key){case"ArrowDown":F.preventDefault(),p(L=>L<i.length-1?L+1:L);break;case"ArrowUp":F.preventDefault(),p(L=>L>0?L-1:-1),d===0&&p(-1);break;case"Enter":if(d>=0&&d<i.length){F.preventDefault();const L=i[d];o(L),u(!1),p(-1)}break;case"Escape":F.preventDefault(),u(!1),p(-1);break}},[l,d,i,o]),w=()=>{i.length>0&&(u(!l),p(-1))},E=F=>{var L;o(F),u(!1),p(-1),(L=y.current)==null||L.focus()},k=F=>{t(F.target.value)},_=()=>{m(!0)},R=()=>{m(!1)},C=T.useCallback((F,L)=>{L.stopPropagation(),s==null||s(F),i.length===1?(u(!1),p(-1)):d===F?p(I=>I>0?I-1:-1):d>F&&p(I=>I-1)},[s,i.length,d]);return x.jsxs("div",{className:"relative",ref:b,onMouseEnter:_,onMouseLeave:R,children:[x.jsxs("div",{className:"relative",children:[x.jsx(oa,{ref:y,id:a,value:e,onChange:k,onKeyDown:S,onClick:w,placeholder:r,autoComplete:"off",className:lt(f&&i.length>0?"pr-5":"pr-2","w-full",n)}),f&&i.length>0&&x.jsx("button",{type:"button",onClick:w,className:"absolute right-2 top-1/2 -translate-y-1/2 p-0 rounded hover:bg-gray-100 dark:hover:bg-gray-800 transition-colors",tabIndex:-1,children:x.jsx(Zv,{className:lt("h-3 w-3 transition-transform duration-200 text-gray-500",l&&"rotate-180")})})]}),l&&i.length>0&&x.jsx("div",{className:"absolute top-full left-0 right-0 z-50 mt-0.5 bg-gray-100 dark:bg-gray-900 border border-gray-300 dark:border-gray-700 rounded-md shadow-lg max-h-96 overflow-auto min-w-0",children:i.map((F,L)=>x.jsxs("div",{className:lt("flex items-center justify-between pl-3 pr-1 py-2 text-sm hover:bg-gray-200 dark:hover:bg-gray-700 transition-colors","border-b border-gray-100 dark:border-gray-700 last:border-b-0","focus-within:bg-gray-100 dark:focus-within:bg-gray-700",d===L&&"bg-gray-100 dark:bg-gray-700"),children:[x.jsx("button",{type:"button",onClick:()=>E(F),className:"flex-1 text-left truncate focus:outline-none mr-0",title:F,children:F}),s&&x.jsx("button",{type:"button",onClick:I=>C(L,I),className:"flex-shrink-0 p-0 rounded hover:bg-red-100 dark:hover:bg-red-900 transition-colors focus:outline-none ml-auto",title:"Delete this history item",children:x.jsx(Qv,{className:"h-3 w-3 text-gray-400 hover:text-red-500"})})]},L))})]})}function xNe(){const{t:e}=Sr(),t=Ve(u=>u.querySettings),r=Ve(u=>u.userPromptHistory),n=T.useCallback((u,d)=>{Ve.getState().updateQuerySettings({[u]:d})},[]),a=T.useCallback(u=>{n("user_prompt",u)},[n]),i=T.useCallback(u=>{const d=[...r];d.splice(u,1),Ve.getState().setUserPromptHistory(d)},[r]),o=T.useMemo(()=>({mode:"mix",response_type:"Multiple Paragraphs",top_k:40,chunk_top_k:20,max_entity_tokens:6e3,max_relation_tokens:8e3,max_total_tokens:3e4}),[]),s=T.useCallback(u=>{n(u,o[u])},[n,o]),l=({onClick:u,title:d})=>x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("button",{type:"button",onClick:u,className:"mr-1 p-1 rounded hover:bg-gray-100 dark:hover:bg-gray-800 transition-colors",title:d,children:x.jsx(o5,{className:"h-3 w-3 text-gray-500 hover:text-gray-700 dark:text-gray-400 dark:hover:text-gray-200"})})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:d})})]})});return x.jsxs(qc,{className:"flex shrink-0 flex-col w-[280px]",children:[x.jsxs(Hm,{className:"px-4 pt-4 pb-2",children:[x.jsx(Um,{children:e("retrievePanel.querySettings.parametersTitle")}),x.jsx(m2,{className:"sr-only",children:e("retrievePanel.querySettings.parametersDescription")})]}),x.jsx($m,{className:"m-0 flex grow flex-col p-0 text-xs",children:x.jsx("div",{className:"relative size-full",children:x.jsxs("div",{className:"absolute inset-0 flex flex-col gap-2 overflow-auto px-2 pr-2",children:[x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"user_prompt",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.userPrompt")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.userPromptTooltip")})})]})}),x.jsx("div",{children:x.jsx(ENe,{id:"user_prompt",value:t.user_prompt||"",onChange:u=>n("user_prompt",u),onSelectFromHistory:a,onDeleteFromHistory:i,history:r,placeholder:e("retrievePanel.querySettings.userPromptPlaceholder"),className:"h-9"})})]}),x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"query_mode_select",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.queryMode")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.queryModeTooltip")})})]})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsxs(Uh,{value:t.mode,onValueChange:u=>n("mode",u),children:[x.jsx(Uc,{id:"query_mode_select",className:"hover:bg-primary/5 h-9 cursor-pointer focus:ring-0 focus:ring-offset-0 focus:outline-0 active:right-0 flex-1 text-left [&>span]:break-all [&>span]:line-clamp-1",children:x.jsx($h,{})}),x.jsx($c,{children:x.jsxs(o7,{children:[x.jsx($n,{value:"naive",children:e("retrievePanel.querySettings.queryModeOptions.naive")}),x.jsx($n,{value:"local",children:e("retrievePanel.querySettings.queryModeOptions.local")}),x.jsx($n,{value:"global",children:e("retrievePanel.querySettings.queryModeOptions.global")}),x.jsx($n,{value:"hybrid",children:e("retrievePanel.querySettings.queryModeOptions.hybrid")}),x.jsx($n,{value:"mix",children:e("retrievePanel.querySettings.queryModeOptions.mix")}),x.jsx($n,{value:"bypass",children:e("retrievePanel.querySettings.queryModeOptions.bypass")})]})})]}),x.jsx(l,{onClick:()=>s("mode"),title:"Reset to default (Mix)"})]})]}),x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"response_format_select",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.responseFormat")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.responseFormatTooltip")})})]})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsxs(Uh,{value:t.response_type,onValueChange:u=>n("response_type",u),children:[x.jsx(Uc,{id:"response_format_select",className:"hover:bg-primary/5 h-9 cursor-pointer focus:ring-0 focus:ring-offset-0 focus:outline-0 active:right-0 flex-1 text-left [&>span]:break-all [&>span]:line-clamp-1",children:x.jsx($h,{})}),x.jsx($c,{children:x.jsxs(o7,{children:[x.jsx($n,{value:"Multiple Paragraphs",children:e("retrievePanel.querySettings.responseFormatOptions.multipleParagraphs")}),x.jsx($n,{value:"Single Paragraph",children:e("retrievePanel.querySettings.responseFormatOptions.singleParagraph")}),x.jsx($n,{value:"Bullet Points",children:e("retrievePanel.querySettings.responseFormatOptions.bulletPoints")})]})})]}),x.jsx(l,{onClick:()=>s("response_type"),title:"Reset to default (Multiple Paragraphs)"})]})]}),x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"top_k",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.topK")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.topKTooltip")})})]})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{id:"top_k",type:"number",value:t.top_k??"",onChange:u=>{const d=u.target.value;n("top_k",d===""?"":parseInt(d)||0)},onBlur:u=>{const d=u.target.value;(d===""||isNaN(parseInt(d)))&&n("top_k",40)},min:1,placeholder:e("retrievePanel.querySettings.topKPlaceholder"),className:"h-9 flex-1 pr-2 [&::-webkit-outer-spin-button]:appearance-none [&::-webkit-inner-spin-button]:appearance-none [-moz-appearance:textfield]"}),x.jsx(l,{onClick:()=>s("top_k"),title:"Reset to default"})]})]}),x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"chunk_top_k",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.chunkTopK")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.chunkTopKTooltip")})})]})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{id:"chunk_top_k",type:"number",value:t.chunk_top_k??"",onChange:u=>{const d=u.target.value;n("chunk_top_k",d===""?"":parseInt(d)||0)},onBlur:u=>{const d=u.target.value;(d===""||isNaN(parseInt(d)))&&n("chunk_top_k",20)},min:1,placeholder:e("retrievePanel.querySettings.chunkTopKPlaceholder"),className:"h-9 flex-1 pr-2 [&::-webkit-outer-spin-button]:appearance-none [&::-webkit-inner-spin-button]:appearance-none [-moz-appearance:textfield]"}),x.jsx(l,{onClick:()=>s("chunk_top_k"),title:"Reset to default"})]})]}),x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"max_entity_tokens",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.maxEntityTokens")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.maxEntityTokensTooltip")})})]})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{id:"max_entity_tokens",type:"number",value:t.max_entity_tokens??"",onChange:u=>{const d=u.target.value;n("max_entity_tokens",d===""?"":parseInt(d)||0)},onBlur:u=>{const d=u.target.value;(d===""||isNaN(parseInt(d)))&&n("max_entity_tokens",6e3)},min:1,placeholder:e("retrievePanel.querySettings.maxEntityTokensPlaceholder"),className:"h-9 flex-1 pr-2 [&::-webkit-outer-spin-button]:appearance-none [&::-webkit-inner-spin-button]:appearance-none [-moz-appearance:textfield]"}),x.jsx(l,{onClick:()=>s("max_entity_tokens"),title:"Reset to default"})]})]}),x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"max_relation_tokens",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.maxRelationTokens")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.maxRelationTokensTooltip")})})]})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{id:"max_relation_tokens",type:"number",value:t.max_relation_tokens??"",onChange:u=>{const d=u.target.value;n("max_relation_tokens",d===""?"":parseInt(d)||0)},onBlur:u=>{const d=u.target.value;(d===""||isNaN(parseInt(d)))&&n("max_relation_tokens",8e3)},min:1,placeholder:e("retrievePanel.querySettings.maxRelationTokensPlaceholder"),className:"h-9 flex-1 pr-2 [&::-webkit-outer-spin-button]:appearance-none [&::-webkit-inner-spin-button]:appearance-none [-moz-appearance:textfield]"}),x.jsx(l,{onClick:()=>s("max_relation_tokens"),title:"Reset to default"})]})]}),x.jsxs(x.Fragment,{children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"max_total_tokens",className:"ml-1 cursor-help",children:e("retrievePanel.querySettings.maxTotalTokens")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.maxTotalTokensTooltip")})})]})}),x.jsxs("div",{className:"flex items-center gap-1",children:[x.jsx(oa,{id:"max_total_tokens",type:"number",value:t.max_total_tokens??"",onChange:u=>{const d=u.target.value;n("max_total_tokens",d===""?"":parseInt(d)||0)},onBlur:u=>{const d=u.target.value;(d===""||isNaN(parseInt(d)))&&n("max_total_tokens",3e4)},min:1,placeholder:e("retrievePanel.querySettings.maxTotalTokensPlaceholder"),className:"h-9 flex-1 pr-2 [&::-webkit-outer-spin-button]:appearance-none [&::-webkit-inner-spin-button]:appearance-none [-moz-appearance:textfield]"}),x.jsx(l,{onClick:()=>s("max_total_tokens"),title:"Reset to default"})]})]}),x.jsxs(x.Fragment,{children:[x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"enable_rerank",className:"flex-1 ml-1 cursor-help",children:e("retrievePanel.querySettings.enableRerank")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.enableRerankTooltip")})})]})}),x.jsx(ol,{className:"mr-10 cursor-pointer",id:"enable_rerank",checked:t.enable_rerank,onCheckedChange:u=>n("enable_rerank",u)})]}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"only_need_context",className:"flex-1 ml-1 cursor-help",children:e("retrievePanel.querySettings.onlyNeedContext")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.onlyNeedContextTooltip")})})]})}),x.jsx(ol,{className:"mr-10 cursor-pointer",id:"only_need_context",checked:t.only_need_context,onCheckedChange:u=>{n("only_need_context",u),u&&n("only_need_prompt",!1)}})]}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"only_need_prompt",className:"flex-1 ml-1 cursor-help",children:e("retrievePanel.querySettings.onlyNeedPrompt")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.onlyNeedPromptTooltip")})})]})}),x.jsx(ol,{className:"mr-10 cursor-pointer",id:"only_need_prompt",checked:t.only_need_prompt,onCheckedChange:u=>{n("only_need_prompt",u),u&&n("only_need_context",!1)}})]}),x.jsxs("div",{className:"flex items-center gap-2",children:[x.jsx(ka,{children:x.jsxs(Ca,{children:[x.jsx(Ta,{asChild:!0,children:x.jsx("label",{htmlFor:"stream",className:"flex-1 ml-1 cursor-help",children:e("retrievePanel.querySettings.streamResponse")})}),x.jsx(ga,{side:"left",children:x.jsx("p",{children:e("retrievePanel.querySettings.streamResponseTooltip")})})]})}),x.jsx(ol,{className:"mr-10 cursor-pointer",id:"stream",checked:t.stream,onCheckedChange:u=>n("stream",u)})]})]})]})})})]})}function fP(e){const t=[],r=String(e||"");let n=r.indexOf(","),a=0,i=!1;for(;!i;){n===-1&&(n=r.length,i=!0);const o=r.slice(a,n).trim();(o||!i)&&t.push(o),a=n+1,n=r.indexOf(",",a)}return t}function are(e,t){const r={};return(e[e.length-1]===""?[...e,""]:e).join((r.padRight?" ":"")+","+(r.padLeft===!1?"":" ")).trim()}const wNe=/^[$_\p{ID_Start}][$_\u{200C}\u{200D}\p{ID_Continue}]*$/u,kNe=/^[$_\p{ID_Start}][-$_\u{200C}\u{200D}\p{ID_Continue}]*$/u,CNe={};function mP(e,t){return(CNe.jsx?kNe:wNe).test(e)}const TNe=/[ \t\n\f\r]/g;function ANe(e){return typeof e=="object"?e.type==="text"?gP(e.value):!1:gP(e)}function gP(e){return e.replace(TNe,"")===""}let Ng=class{constructor(t,r,n){this.property=t,this.normal=r,n&&(this.space=n)}};Ng.prototype.property={};Ng.prototype.normal={};Ng.prototype.space=null;function ire(e,t){const r={},n={};let a=-1;for(;++a<e.length;)Object.assign(r,e[a].property),Object.assign(n,e[a].normal);return new Ng(r,n,t)}function OR(e){return e.toLowerCase()}let fo=class{constructor(t,r){this.property=t,this.attribute=r}};fo.prototype.space=null;fo.prototype.boolean=!1;fo.prototype.booleanish=!1;fo.prototype.overloadedBoolean=!1;fo.prototype.number=!1;fo.prototype.commaSeparated=!1;fo.prototype.spaceSeparated=!1;fo.prototype.commaOrSpaceSeparated=!1;fo.prototype.mustUseProperty=!1;fo.prototype.defined=!1;let FNe=0;const hr=id(),Hn=id(),ore=id(),it=id(),an=id(),wh=id(),Di=id();function id(){return 2**++FNe}const BR=Object.freeze(Object.defineProperty({__proto__:null,boolean:hr,booleanish:Hn,commaOrSpaceSeparated:Di,commaSeparated:wh,number:it,overloadedBoolean:ore,spaceSeparated:an},Symbol.toStringTag,{value:"Module"})),ik=Object.keys(BR);let P5=class extends fo{constructor(t,r,n,a){let i=-1;if(super(t,r),bP(this,"space",a),typeof n=="number")for(;++i<ik.length;){const o=ik[i];bP(this,ik[i],(n&BR[o])===BR[o])}}};P5.prototype.defined=!0;function bP(e,t,r){r&&(e[t]=r)}const _Ne={}.hasOwnProperty;function wp(e){const t={},r={};let n;for(n in e.properties)if(_Ne.call(e.properties,n)){const a=e.properties[n],i=new P5(n,e.transform(e.attributes||{},n),a,e.space);e.mustUseProperty&&e.mustUseProperty.includes(n)&&(i.mustUseProperty=!0),t[n]=i,r[OR(n)]=n,r[OR(i.attribute)]=n}return new Ng(t,r,e.space)}const sre=wp({space:"xlink",transform(e,t){return"xlink:"+t.slice(5).toLowerCase()},properties:{xLinkActuate:null,xLinkArcRole:null,xLinkHref:null,xLinkRole:null,xLinkShow:null,xLinkTitle:null,xLinkType:null}}),lre=wp({space:"xml",transform(e,t){return"xml:"+t.slice(3).toLowerCase()},properties:{xmlLang:null,xmlBase:null,xmlSpace:null}});function ure(e,t){return t in e?e[t]:t}function cre(e,t){return ure(e,t.toLowerCase())}const dre=wp({space:"xmlns",attributes:{xmlnsxlink:"xmlns:xlink"},transform:cre,properties:{xmlns:null,xmlnsXLink:null}}),hre=wp({transform(e,t){return t==="role"?t:"aria-"+t.slice(4).toLowerCase()},properties:{ariaActiveDescendant:null,ariaAtomic:Hn,ariaAutoComplete:null,ariaBusy:Hn,ariaChecked:Hn,ariaColCount:it,ariaColIndex:it,ariaColSpan:it,ariaControls:an,ariaCurrent:null,ariaDescribedBy:an,ariaDetails:null,ariaDisabled:Hn,ariaDropEffect:an,ariaErrorMessage:null,ariaExpanded:Hn,ariaFlowTo:an,ariaGrabbed:Hn,ariaHasPopup:null,ariaHidden:Hn,ariaInvalid:null,ariaKeyShortcuts:null,ariaLabel:null,ariaLabelledBy:an,ariaLevel:it,ariaLive:null,ariaModal:Hn,ariaMultiLine:Hn,ariaMultiSelectable:Hn,ariaOrientation:null,ariaOwns:an,ariaPlaceholder:null,ariaPosInSet:it,ariaPressed:Hn,ariaReadOnly:Hn,ariaRelevant:null,ariaRequired:Hn,ariaRoleDescription:an,ariaRowCount:it,ariaRowIndex:it,ariaRowSpan:it,ariaSelected:Hn,ariaSetSize:it,ariaSort:null,ariaValueMax:it,ariaValueMin:it,ariaValueNow:it,ariaValueText:null,role:null}}),RNe=wp({space:"html",attributes:{acceptcharset:"accept-charset",classname:"class",htmlfor:"for",httpequiv:"http-equiv"},transform:cre,mustUseProperty:["checked","multiple","muted","selected"],properties:{abbr:null,accept:wh,acceptCharset:an,accessKey:an,action:null,allow:null,allowFullScreen:hr,allowPaymentRequest:hr,allowUserMedia:hr,alt:null,as:null,async:hr,autoCapitalize:null,autoComplete:an,autoFocus:hr,autoPlay:hr,blocking:an,capture:null,charSet:null,checked:hr,cite:null,className:an,cols:it,colSpan:null,content:null,contentEditable:Hn,controls:hr,controlsList:an,coords:it|wh,crossOrigin:null,data:null,dateTime:null,decoding:null,default:hr,defer:hr,dir:null,dirName:null,disabled:hr,download:ore,draggable:Hn,encType:null,enterKeyHint:null,fetchPriority:null,form:null,formAction:null,formEncType:null,formMethod:null,formNoValidate:hr,formTarget:null,headers:an,height:it,hidden:hr,high:it,href:null,hrefLang:null,htmlFor:an,httpEquiv:an,id:null,imageSizes:null,imageSrcSet:null,inert:hr,inputMode:null,integrity:null,is:null,isMap:hr,itemId:null,itemProp:an,itemRef:an,itemScope:hr,itemType:an,kind:null,label:null,lang:null,language:null,list:null,loading:null,loop:hr,low:it,manifest:null,max:null,maxLength:it,media:null,method:null,min:null,minLength:it,multiple:hr,muted:hr,name:null,nonce:null,noModule:hr,noValidate:hr,onAbort:null,onAfterPrint:null,onAuxClick:null,onBeforeMatch:null,onBeforePrint:null,onBeforeToggle:null,onBeforeUnload:null,onBlur:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onContextLost:null,onContextMenu:null,onContextRestored:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnded:null,onError:null,onFocus:null,onFormData:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLanguageChange:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadEnd:null,onLoadStart:null,onMessage:null,onMessageError:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRejectionHandled:null,onReset:null,onResize:null,onScroll:null,onScrollEnd:null,onSecurityPolicyViolation:null,onSeeked:null,onSeeking:null,onSelect:null,onSlotChange:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnhandledRejection:null,onUnload:null,onVolumeChange:null,onWaiting:null,onWheel:null,open:hr,optimum:it,pattern:null,ping:an,placeholder:null,playsInline:hr,popover:null,popoverTarget:null,popoverTargetAction:null,poster:null,preload:null,readOnly:hr,referrerPolicy:null,rel:an,required:hr,reversed:hr,rows:it,rowSpan:it,sandbox:an,scope:null,scoped:hr,seamless:hr,selected:hr,shadowRootClonable:hr,shadowRootDelegatesFocus:hr,shadowRootMode:null,shape:null,size:it,sizes:null,slot:null,span:it,spellCheck:Hn,src:null,srcDoc:null,srcLang:null,srcSet:null,start:it,step:null,style:null,tabIndex:it,target:null,title:null,translate:null,type:null,typeMustMatch:hr,useMap:null,value:Hn,width:it,wrap:null,writingSuggestions:null,align:null,aLink:null,archive:an,axis:null,background:null,bgColor:null,border:it,borderColor:null,bottomMargin:it,cellPadding:null,cellSpacing:null,char:null,charOff:null,classId:null,clear:null,code:null,codeBase:null,codeType:null,color:null,compact:hr,declare:hr,event:null,face:null,frame:null,frameBorder:null,hSpace:it,leftMargin:it,link:null,longDesc:null,lowSrc:null,marginHeight:it,marginWidth:it,noResize:hr,noHref:hr,noShade:hr,noWrap:hr,object:null,profile:null,prompt:null,rev:null,rightMargin:it,rules:null,scheme:null,scrolling:Hn,standby:null,summary:null,text:null,topMargin:it,valueType:null,version:null,vAlign:null,vLink:null,vSpace:it,allowTransparency:null,autoCorrect:null,autoSave:null,disablePictureInPicture:hr,disableRemotePlayback:hr,prefix:null,property:null,results:it,security:null,unselectable:null}}),DNe=wp({space:"svg",attributes:{accentHeight:"accent-height",alignmentBaseline:"alignment-baseline",arabicForm:"arabic-form",baselineShift:"baseline-shift",capHeight:"cap-height",className:"class",clipPath:"clip-path",clipRule:"clip-rule",colorInterpolation:"color-interpolation",colorInterpolationFilters:"color-interpolation-filters",colorProfile:"color-profile",colorRendering:"color-rendering",crossOrigin:"crossorigin",dataType:"datatype",dominantBaseline:"dominant-baseline",enableBackground:"enable-background",fillOpacity:"fill-opacity",fillRule:"fill-rule",floodColor:"flood-color",floodOpacity:"flood-opacity",fontFamily:"font-family",fontSize:"font-size",fontSizeAdjust:"font-size-adjust",fontStretch:"font-stretch",fontStyle:"font-style",fontVariant:"font-variant",fontWeight:"font-weight",glyphName:"glyph-name",glyphOrientationHorizontal:"glyph-orientation-horizontal",glyphOrientationVertical:"glyph-orientation-vertical",hrefLang:"hreflang",horizAdvX:"horiz-adv-x",horizOriginX:"horiz-origin-x",horizOriginY:"horiz-origin-y",imageRendering:"image-rendering",letterSpacing:"letter-spacing",lightingColor:"lighting-color",markerEnd:"marker-end",markerMid:"marker-mid",markerStart:"marker-start",navDown:"nav-down",navDownLeft:"nav-down-left",navDownRight:"nav-down-right",navLeft:"nav-left",navNext:"nav-next",navPrev:"nav-prev",navRight:"nav-right",navUp:"nav-up",navUpLeft:"nav-up-left",navUpRight:"nav-up-right",onAbort:"onabort",onActivate:"onactivate",onAfterPrint:"onafterprint",onBeforePrint:"onbeforeprint",onBegin:"onbegin",onCancel:"oncancel",onCanPlay:"oncanplay",onCanPlayThrough:"oncanplaythrough",onChange:"onchange",onClick:"onclick",onClose:"onclose",onCopy:"oncopy",onCueChange:"oncuechange",onCut:"oncut",onDblClick:"ondblclick",onDrag:"ondrag",onDragEnd:"ondragend",onDragEnter:"ondragenter",onDragExit:"ondragexit",onDragLeave:"ondragleave",onDragOver:"ondragover",onDragStart:"ondragstart",onDrop:"ondrop",onDurationChange:"ondurationchange",onEmptied:"onemptied",onEnd:"onend",onEnded:"onended",onError:"onerror",onFocus:"onfocus",onFocusIn:"onfocusin",onFocusOut:"onfocusout",onHashChange:"onhashchange",onInput:"oninput",onInvalid:"oninvalid",onKeyDown:"onkeydown",onKeyPress:"onkeypress",onKeyUp:"onkeyup",onLoad:"onload",onLoadedData:"onloadeddata",onLoadedMetadata:"onloadedmetadata",onLoadStart:"onloadstart",onMessage:"onmessage",onMouseDown:"onmousedown",onMouseEnter:"onmouseenter",onMouseLeave:"onmouseleave",onMouseMove:"onmousemove",onMouseOut:"onmouseout",onMouseOver:"onmouseover",onMouseUp:"onmouseup",onMouseWheel:"onmousewheel",onOffline:"onoffline",onOnline:"ononline",onPageHide:"onpagehide",onPageShow:"onpageshow",onPaste:"onpaste",onPause:"onpause",onPlay:"onplay",onPlaying:"onplaying",onPopState:"onpopstate",onProgress:"onprogress",onRateChange:"onratechange",onRepeat:"onrepeat",onReset:"onreset",onResize:"onresize",onScroll:"onscroll",onSeeked:"onseeked",onSeeking:"onseeking",onSelect:"onselect",onShow:"onshow",onStalled:"onstalled",onStorage:"onstorage",onSubmit:"onsubmit",onSuspend:"onsuspend",onTimeUpdate:"ontimeupdate",onToggle:"ontoggle",onUnload:"onunload",onVolumeChange:"onvolumechange",onWaiting:"onwaiting",onZoom:"onzoom",overlinePosition:"overline-position",overlineThickness:"overline-thickness",paintOrder:"paint-order",panose1:"panose-1",pointerEvents:"pointer-events",referrerPolicy:"referrerpolicy",renderingIntent:"rendering-intent",shapeRendering:"shape-rendering",stopColor:"stop-color",stopOpacity:"stop-opacity",strikethroughPosition:"strikethrough-position",strikethroughThickness:"strikethrough-thickness",strokeDashArray:"stroke-dasharray",strokeDashOffset:"stroke-dashoffset",strokeLineCap:"stroke-linecap",strokeLineJoin:"stroke-linejoin",strokeMiterLimit:"stroke-miterlimit",strokeOpacity:"stroke-opacity",strokeWidth:"stroke-width",tabIndex:"tabindex",textAnchor:"text-anchor",textDecoration:"text-decoration",textRendering:"text-rendering",transformOrigin:"transform-origin",typeOf:"typeof",underlinePosition:"underline-position",underlineThickness:"underline-thickness",unicodeBidi:"unicode-bidi",unicodeRange:"unicode-range",unitsPerEm:"units-per-em",vAlphabetic:"v-alphabetic",vHanging:"v-hanging",vIdeographic:"v-ideographic",vMathematical:"v-mathematical",vectorEffect:"vector-effect",vertAdvY:"vert-adv-y",vertOriginX:"vert-origin-x",vertOriginY:"vert-origin-y",wordSpacing:"word-spacing",writingMode:"writing-mode",xHeight:"x-height",playbackOrder:"playbackorder",timelineBegin:"timelinebegin"},transform:ure,properties:{about:Di,accentHeight:it,accumulate:null,additive:null,alignmentBaseline:null,alphabetic:it,amplitude:it,arabicForm:null,ascent:it,attributeName:null,attributeType:null,azimuth:it,bandwidth:null,baselineShift:null,baseFrequency:null,baseProfile:null,bbox:null,begin:null,bias:it,by:null,calcMode:null,capHeight:it,className:an,clip:null,clipPath:null,clipPathUnits:null,clipRule:null,color:null,colorInterpolation:null,colorInterpolationFilters:null,colorProfile:null,colorRendering:null,content:null,contentScriptType:null,contentStyleType:null,crossOrigin:null,cursor:null,cx:null,cy:null,d:null,dataType:null,defaultAction:null,descent:it,diffuseConstant:it,direction:null,display:null,dur:null,divisor:it,dominantBaseline:null,download:hr,dx:null,dy:null,edgeMode:null,editable:null,elevation:it,enableBackground:null,end:null,event:null,exponent:it,externalResourcesRequired:null,fill:null,fillOpacity:it,fillRule:null,filter:null,filterRes:null,filterUnits:null,floodColor:null,floodOpacity:null,focusable:null,focusHighlight:null,fontFamily:null,fontSize:null,fontSizeAdjust:null,fontStretch:null,fontStyle:null,fontVariant:null,fontWeight:null,format:null,fr:null,from:null,fx:null,fy:null,g1:wh,g2:wh,glyphName:wh,glyphOrientationHorizontal:null,glyphOrientationVertical:null,glyphRef:null,gradientTransform:null,gradientUnits:null,handler:null,hanging:it,hatchContentUnits:null,hatchUnits:null,height:null,href:null,hrefLang:null,horizAdvX:it,horizOriginX:it,horizOriginY:it,id:null,ideographic:it,imageRendering:null,initialVisibility:null,in:null,in2:null,intercept:it,k:it,k1:it,k2:it,k3:it,k4:it,kernelMatrix:Di,kernelUnitLength:null,keyPoints:null,keySplines:null,keyTimes:null,kerning:null,lang:null,lengthAdjust:null,letterSpacing:null,lightingColor:null,limitingConeAngle:it,local:null,markerEnd:null,markerMid:null,markerStart:null,markerHeight:null,markerUnits:null,markerWidth:null,mask:null,maskContentUnits:null,maskUnits:null,mathematical:null,max:null,media:null,mediaCharacterEncoding:null,mediaContentEncodings:null,mediaSize:it,mediaTime:null,method:null,min:null,mode:null,name:null,navDown:null,navDownLeft:null,navDownRight:null,navLeft:null,navNext:null,navPrev:null,navRight:null,navUp:null,navUpLeft:null,navUpRight:null,numOctaves:null,observer:null,offset:null,onAbort:null,onActivate:null,onAfterPrint:null,onBeforePrint:null,onBegin:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnd:null,onEnded:null,onError:null,onFocus:null,onFocusIn:null,onFocusOut:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadStart:null,onMessage:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onMouseWheel:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRepeat:null,onReset:null,onResize:null,onScroll:null,onSeeked:null,onSeeking:null,onSelect:null,onShow:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnload:null,onVolumeChange:null,onWaiting:null,onZoom:null,opacity:null,operator:null,order:null,orient:null,orientation:null,origin:null,overflow:null,overlay:null,overlinePosition:it,overlineThickness:it,paintOrder:null,panose1:null,path:null,pathLength:it,patternContentUnits:null,patternTransform:null,patternUnits:null,phase:null,ping:an,pitch:null,playbackOrder:null,pointerEvents:null,points:null,pointsAtX:it,pointsAtY:it,pointsAtZ:it,preserveAlpha:null,preserveAspectRatio:null,primitiveUnits:null,propagate:null,property:Di,r:null,radius:null,referrerPolicy:null,refX:null,refY:null,rel:Di,rev:Di,renderingIntent:null,repeatCount:null,repeatDur:null,requiredExtensions:Di,requiredFeatures:Di,requiredFonts:Di,requiredFormats:Di,resource:null,restart:null,result:null,rotate:null,rx:null,ry:null,scale:null,seed:null,shapeRendering:null,side:null,slope:null,snapshotTime:null,specularConstant:it,specularExponent:it,spreadMethod:null,spacing:null,startOffset:null,stdDeviation:null,stemh:null,stemv:null,stitchTiles:null,stopColor:null,stopOpacity:null,strikethroughPosition:it,strikethroughThickness:it,string:null,stroke:null,strokeDashArray:Di,strokeDashOffset:null,strokeLineCap:null,strokeLineJoin:null,strokeMiterLimit:it,strokeOpacity:it,strokeWidth:null,style:null,surfaceScale:it,syncBehavior:null,syncBehaviorDefault:null,syncMaster:null,syncTolerance:null,syncToleranceDefault:null,systemLanguage:Di,tabIndex:it,tableValues:null,target:null,targetX:it,targetY:it,textAnchor:null,textDecoration:null,textRendering:null,textLength:null,timelineBegin:null,title:null,transformBehavior:null,type:null,typeOf:Di,to:null,transform:null,transformOrigin:null,u1:null,u2:null,underlinePosition:it,underlineThickness:it,unicode:null,unicodeBidi:null,unicodeRange:null,unitsPerEm:it,values:null,vAlphabetic:it,vMathematical:it,vectorEffect:null,vHanging:it,vIdeographic:it,version:null,vertAdvY:it,vertOriginX:it,vertOriginY:it,viewBox:null,viewTarget:null,visibility:null,width:null,widths:null,wordSpacing:null,writingMode:null,x:null,x1:null,x2:null,xChannelSelector:null,xHeight:it,y:null,y1:null,y2:null,yChannelSelector:null,z:null,zoomAndPan:null}}),NNe=/^data[-\w.:]+$/i,yP=/-[a-z]/g,LNe=/[A-Z]/g;function pre(e,t){const r=OR(t);let n=t,a=fo;if(r in e.normal)return e.property[e.normal[r]];if(r.length>4&&r.slice(0,4)==="data"&&NNe.test(t)){if(t.charAt(4)==="-"){const i=t.slice(5).replace(yP,INe);n="data"+i.charAt(0).toUpperCase()+i.slice(1)}else{const i=t.slice(4);if(!yP.test(i)){let o=i.replace(LNe,MNe);o.charAt(0)!=="-"&&(o="-"+o),t="data"+o}}a=P5}return new a(n,t)}function MNe(e){return"-"+e.toLowerCase()}function INe(e){return e.charAt(1).toUpperCase()}const ONe={classId:"classID",dataType:"datatype",itemId:"itemID",strokeDashArray:"strokeDasharray",strokeDashOffset:"strokeDashoffset",strokeLineCap:"strokeLinecap",strokeLineJoin:"strokeLinejoin",strokeMiterLimit:"strokeMiterlimit",typeOf:"typeof",xLinkActuate:"xlinkActuate",xLinkArcRole:"xlinkArcrole",xLinkHref:"xlinkHref",xLinkRole:"xlinkRole",xLinkShow:"xlinkShow",xLinkTitle:"xlinkTitle",xLinkType:"xlinkType",xmlnsXLink:"xmlnsXlink"},fre=ire([lre,sre,dre,hre,RNe],"html"),Lg=ire([lre,sre,dre,hre,DNe],"svg");function vP(e){const t=String(e||"").trim();return t?t.split(/[ \t\n\r\f]+/g):[]}function mre(e){return e.join(" ").trim()}var nh={},ok,SP;function BNe(){if(SP)return ok;SP=1;var e=/\/\*[^*]*\*+([^/*][^*]*\*+)*\//g,t=/\n/g,r=/^\s*/,n=/^(\*?[-#/*\\\w]+(\[[0-9a-z_-]+\])?)\s*/,a=/^:\s*/,i=/^((?:'(?:\\'|.)*?'|"(?:\\"|.)*?"|\([^)]*?\)|[^};])+)/,o=/^[;\s]*/,s=/^\s+|\s+$/g,l=`
`,u="/",d="*",p="",f="comment",m="declaration";ok=function(y,S){if(typeof y!="string")throw new TypeError("First argument must be a string");if(!y)return[];S=S||{};var w=1,E=1;function k(W){var $=W.match(t);$&&(w+=$.length);var X=W.lastIndexOf(l);E=~X?W.length-X:E+W.length}function _(){var W={line:w,column:E};return function($){return $.position=new R(W),L(),$}}function R(W){this.start=W,this.end={line:w,column:E},this.source=S.source}R.prototype.content=y;function C(W){var $=new Error(S.source+":"+w+":"+E+": "+W);if($.reason=W,$.filename=S.source,$.line=w,$.column=E,$.source=y,!S.silent)throw $}function F(W){var $=W.exec(y);if($){var X=$[0];return k(X),y=y.slice(X.length),$}}function L(){F(r)}function I(W){var $;for(W=W||[];$=M();)$!==!1&&W.push($);return W}function M(){var W=_();if(!(u!=y.charAt(0)||d!=y.charAt(1))){for(var $=2;p!=y.charAt($)&&(d!=y.charAt($)||u!=y.charAt($+1));)++$;if($+=2,p===y.charAt($-1))return C("End of comment missing");var X=y.slice(2,$-2);return E+=2,k(X),y=y.slice($),E+=2,W({type:f,comment:X})}}function j(){var W=_(),$=F(n);if($){if(M(),!F(a))return C("property missing ':'");var X=F(i),z=W({type:m,property:b($[0].replace(e,p)),value:X?b(X[0].replace(e,p)):p});return F(o),z}}function Y(){var W=[];I(W);for(var $;$=j();)$!==!1&&(W.push($),I(W));return W}return L(),Y()};function b(y){return y?y.replace(s,p):p}return ok}var EP;function PNe(){if(EP)return nh;EP=1;var e=nh&&nh.__importDefault||function(n){return n&&n.__esModule?n:{default:n}};Object.defineProperty(nh,"__esModule",{value:!0}),nh.default=r;var t=e(BNe());function r(n,a){var i=null;if(!n||typeof n!="string")return i;var o=(0,t.default)(n),s=typeof a=="function";return o.forEach(function(l){if(l.type==="declaration"){var u=l.property,d=l.value;s?a(u,d,l):d&&(i=i||{},i[u]=d)}}),i}return nh}var zNe=PNe();const xP=jn(zNe),HNe=xP.default||xP,v2=gre("end"),vs=gre("start");function gre(e){return t;function t(r){const n=r&&r.position&&r.position[e]||{};if(typeof n.line=="number"&&n.line>0&&typeof n.column=="number"&&n.column>0)return{line:n.line,column:n.column,offset:typeof n.offset=="number"&&n.offset>-1?n.offset:void 0}}}function UNe(e){const t=vs(e),r=v2(e);if(t&&r)return{start:t,end:r}}function vm(e){return!e||typeof e!="object"?"":"position"in e||"type"in e?wP(e.position):"start"in e||"end"in e?wP(e):"line"in e||"column"in e?PR(e):""}function PR(e){return kP(e&&e.line)+":"+kP(e&&e.column)}function wP(e){return PR(e&&e.start)+"-"+PR(e&&e.end)}function kP(e){return e&&typeof e=="number"?e:1}class Ba extends Error{constructor(t,r,n){super(),typeof r=="string"&&(n=r,r=void 0);let a="",i={},o=!1;if(r&&("line"in r&&"column"in r?i={place:r}:"start"in r&&"end"in r?i={place:r}:"type"in r?i={ancestors:[r],place:r.position}:i={...r}),typeof t=="string"?a=t:!i.cause&&t&&(o=!0,a=t.message,i.cause=t),!i.ruleId&&!i.source&&typeof n=="string"){const l=n.indexOf(":");l===-1?i.ruleId=n:(i.source=n.slice(0,l),i.ruleId=n.slice(l+1))}if(!i.place&&i.ancestors&&i.ancestors){const l=i.ancestors[i.ancestors.length-1];l&&(i.place=l.position)}const s=i.place&&"start"in i.place?i.place.start:i.place;this.ancestors=i.ancestors||void 0,this.cause=i.cause||void 0,this.column=s?s.column:void 0,this.fatal=void 0,this.file,this.message=a,this.line=s?s.line:void 0,this.name=vm(i.place)||"1:1",this.place=i.place||void 0,this.reason=this.message,this.ruleId=i.ruleId||void 0,this.source=i.source||void 0,this.stack=o&&i.cause&&typeof i.cause.stack=="string"?i.cause.stack:"",this.actual,this.expected,this.note,this.url}}Ba.prototype.file="";Ba.prototype.name="";Ba.prototype.reason="";Ba.prototype.message="";Ba.prototype.stack="";Ba.prototype.column=void 0;Ba.prototype.line=void 0;Ba.prototype.ancestors=void 0;Ba.prototype.cause=void 0;Ba.prototype.fatal=void 0;Ba.prototype.place=void 0;Ba.prototype.ruleId=void 0;Ba.prototype.source=void 0;const z5={}.hasOwnProperty,$Ne=new Map,GNe=/[A-Z]/g,jNe=/-([a-z])/g,qNe=new Set(["table","tbody","thead","tfoot","tr"]),WNe=new Set(["td","th"]),bre="https://github.com/syntax-tree/hast-util-to-jsx-runtime";function yre(e,t){if(!t||t.Fragment===void 0)throw new TypeError("Expected `Fragment` in options");const r=t.filePath||void 0;let n;if(t.development){if(typeof t.jsxDEV!="function")throw new TypeError("Expected `jsxDEV` in options when `development: true`");n=eLe(r,t.jsxDEV)}else{if(typeof t.jsx!="function")throw new TypeError("Expected `jsx` in production options");if(typeof t.jsxs!="function")throw new TypeError("Expected `jsxs` in production options");n=QNe(r,t.jsx,t.jsxs)}const a={Fragment:t.Fragment,ancestors:[],components:t.components||{},create:n,elementAttributeNameCase:t.elementAttributeNameCase||"react",evaluater:t.createEvaluater?t.createEvaluater():void 0,filePath:r,ignoreInvalidStyle:t.ignoreInvalidStyle||!1,passKeys:t.passKeys!==!1,passNode:t.passNode||!1,schema:t.space==="svg"?Lg:fre,stylePropertyNameCase:t.stylePropertyNameCase||"dom",tableCellAlignToStyle:t.tableCellAlignToStyle!==!1},i=vre(a,e,void 0);return i&&typeof i!="string"?i:a.create(e,a.Fragment,{children:i||void 0},void 0)}function vre(e,t,r){if(t.type==="element")return VNe(e,t,r);if(t.type==="mdxFlowExpression"||t.type==="mdxTextExpression")return KNe(e,t);if(t.type==="mdxJsxFlowElement"||t.type==="mdxJsxTextElement")return JNe(e,t,r);if(t.type==="mdxjsEsm")return YNe(e,t);if(t.type==="root")return XNe(e,t,r);if(t.type==="text")return ZNe(e,t)}function VNe(e,t,r){const n=e.schema;let a=n;t.tagName.toLowerCase()==="svg"&&n.space==="html"&&(a=Lg,e.schema=a),e.ancestors.push(t);const i=Ere(e,t.tagName,!1),o=tLe(e,t);let s=U5(e,t);return qNe.has(t.tagName)&&(s=s.filter(function(l){return typeof l=="string"?!ANe(l):!0})),Sre(e,o,i,t),H5(o,s),e.ancestors.pop(),e.schema=n,e.create(t,i,o,r)}function KNe(e,t){if(t.data&&t.data.estree&&e.evaluater){const n=t.data.estree.body[0];return n.type,e.evaluater.evaluateExpression(n.expression)}jm(e,t.position)}function YNe(e,t){if(t.data&&t.data.estree&&e.evaluater)return e.evaluater.evaluateProgram(t.data.estree);jm(e,t.position)}function JNe(e,t,r){const n=e.schema;let a=n;t.name==="svg"&&n.space==="html"&&(a=Lg,e.schema=a),e.ancestors.push(t);const i=t.name===null?e.Fragment:Ere(e,t.name,!0),o=rLe(e,t),s=U5(e,t);return Sre(e,o,i,t),H5(o,s),e.ancestors.pop(),e.schema=n,e.create(t,i,o,r)}function XNe(e,t,r){const n={};return H5(n,U5(e,t)),e.create(t,e.Fragment,n,r)}function ZNe(e,t){return t.value}function Sre(e,t,r,n){typeof r!="string"&&r!==e.Fragment&&e.passNode&&(t.node=n)}function H5(e,t){if(t.length>0){const r=t.length>1?t:t[0];r&&(e.children=r)}}function QNe(e,t,r){return n;function n(a,i,o,s){const u=Array.isArray(o.children)?r:t;return s?u(i,o,s):u(i,o)}}function eLe(e,t){return r;function r(n,a,i,o){const s=Array.isArray(i.children),l=vs(n);return t(a,i,o,s,{columnNumber:l?l.column-1:void 0,fileName:e,lineNumber:l?l.line:void 0},void 0)}}function tLe(e,t){const r={};let n,a;for(a in t.properties)if(a!=="children"&&z5.call(t.properties,a)){const i=nLe(e,a,t.properties[a]);if(i){const[o,s]=i;e.tableCellAlignToStyle&&o==="align"&&typeof s=="string"&&WNe.has(t.tagName)?n=s:r[o]=s}}if(n){const i=r.style||(r.style={});i[e.stylePropertyNameCase==="css"?"text-align":"textAlign"]=n}return r}function rLe(e,t){const r={};for(const n of t.attributes)if(n.type==="mdxJsxExpressionAttribute")if(n.data&&n.data.estree&&e.evaluater){const i=n.data.estree.body[0];i.type;const o=i.expression;o.type;const s=o.properties[0];s.type,Object.assign(r,e.evaluater.evaluateExpression(s.argument))}else jm(e,t.position);else{const a=n.name;let i;if(n.value&&typeof n.value=="object")if(n.value.data&&n.value.data.estree&&e.evaluater){const s=n.value.data.estree.body[0];s.type,i=e.evaluater.evaluateExpression(s.expression)}else jm(e,t.position);else i=n.value===null?!0:n.value;r[a]=i}return r}function U5(e,t){const r=[];let n=-1;const a=e.passKeys?new Map:$Ne;for(;++n<t.children.length;){const i=t.children[n];let o;if(e.passKeys){const l=i.type==="element"?i.tagName:i.type==="mdxJsxFlowElement"||i.type==="mdxJsxTextElement"?i.name:void 0;if(l){const u=a.get(l)||0;o=l+"-"+u,a.set(l,u+1)}}const s=vre(e,i,o);s!==void 0&&r.push(s)}return r}function nLe(e,t,r){const n=pre(e.schema,t);if(!(r==null||typeof r=="number"&&Number.isNaN(r))){if(Array.isArray(r)&&(r=n.commaSeparated?are(r):mre(r)),n.property==="style"){let a=typeof r=="object"?r:aLe(e,String(r));return e.stylePropertyNameCase==="css"&&(a=iLe(a)),["style",a]}return[e.elementAttributeNameCase==="react"&&n.space?ONe[n.property]||n.property:n.attribute,r]}}function aLe(e,t){const r={};try{HNe(t,n)}catch(a){if(!e.ignoreInvalidStyle){const i=a,o=new Ba("Cannot parse `style` attribute",{ancestors:e.ancestors,cause:i,ruleId:"style",source:"hast-util-to-jsx-runtime"});throw o.file=e.filePath||void 0,o.url=bre+"#cannot-parse-style-attribute",o}}return r;function n(a,i){let o=a;o.slice(0,2)!=="--"&&(o.slice(0,4)==="-ms-"&&(o="ms-"+o.slice(4)),o=o.replace(jNe,sLe)),r[o]=i}}function Ere(e,t,r){let n;if(!r)n={type:"Literal",value:t};else if(t.includes(".")){const a=t.split(".");let i=-1,o;for(;++i<a.length;){const s=mP(a[i])?{type:"Identifier",name:a[i]}:{type:"Literal",value:a[i]};o=o?{type:"MemberExpression",object:o,property:s,computed:!!(i&&s.type==="Literal"),optional:!1}:s}n=o}else n=mP(t)&&!/^[a-z]/.test(t)?{type:"Identifier",name:t}:{type:"Literal",value:t};if(n.type==="Literal"){const a=n.value;return z5.call(e.components,a)?e.components[a]:a}if(e.evaluater)return e.evaluater.evaluateExpression(n);jm(e)}function jm(e,t){const r=new Ba("Cannot handle MDX estrees without `createEvaluater`",{ancestors:e.ancestors,place:t,ruleId:"mdx-estree",source:"hast-util-to-jsx-runtime"});throw r.file=e.filePath||void 0,r.url=bre+"#cannot-handle-mdx-estrees-without-createevaluater",r}function iLe(e){const t={};let r;for(r in e)z5.call(e,r)&&(t[oLe(r)]=e[r]);return t}function oLe(e){let t=e.replace(GNe,lLe);return t.slice(0,3)==="ms-"&&(t="-"+t),t}function sLe(e,t){return t.toUpperCase()}function lLe(e){return"-"+e.toLowerCase()}const sk={action:["form"],cite:["blockquote","del","ins","q"],data:["object"],formAction:["button","input"],href:["a","area","base","link"],icon:["menuitem"],itemId:null,manifest:["html"],ping:["a","area"],poster:["video"],src:["audio","embed","iframe","img","input","script","source","track","video"]},uLe={};function $5(e,t){const r=uLe,n=typeof r.includeImageAlt=="boolean"?r.includeImageAlt:!0,a=typeof r.includeHtml=="boolean"?r.includeHtml:!0;return xre(e,n,a)}function xre(e,t,r){if(cLe(e)){if("value"in e)return e.type==="html"&&!r?"":e.value;if(t&&"alt"in e&&e.alt)return e.alt;if("children"in e)return CP(e.children,t,r)}return Array.isArray(e)?CP(e,t,r):""}function CP(e,t,r){const n=[];let a=-1;for(;++a<e.length;)n[a]=xre(e[a],t,r);return n.join("")}function cLe(e){return!!(e&&typeof e=="object")}const TP=document.createElement("i");function G5(e){const t="&"+e+";";TP.innerHTML=t;const r=TP.textContent;return r.charCodeAt(r.length-1)===59&&e!=="semi"||r===t?!1:r}function Pi(e,t,r,n){const a=e.length;let i=0,o;if(t<0?t=-t>a?0:a+t:t=t>a?a:t,r=r>0?r:0,n.length<1e4)o=Array.from(n),o.unshift(t,r),e.splice(...o);else for(r&&e.splice(t,r);i<n.length;)o=n.slice(i,i+1e4),o.unshift(t,0),e.splice(...o),i+=1e4,t+=1e4}function to(e,t){return e.length>0?(Pi(e,e.length,0,t),e):t}const AP={}.hasOwnProperty;function wre(e){const t={};let r=-1;for(;++r<e.length;)dLe(t,e[r]);return t}function dLe(e,t){let r;for(r in t){const a=(AP.call(e,r)?e[r]:void 0)||(e[r]={}),i=t[r];let o;if(i)for(o in i){AP.call(a,o)||(a[o]=[]);const s=i[o];hLe(a[o],Array.isArray(s)?s:s?[s]:[])}}}function hLe(e,t){let r=-1;const n=[];for(;++r<t.length;)(t[r].add==="after"?e:n).push(t[r]);Pi(e,0,0,n)}function kre(e,t){const r=Number.parseInt(e,t);return r<9||r===11||r>13&&r<32||r>126&&r<160||r>55295&&r<57344||r>64975&&r<65008||(r&65535)===65535||(r&65535)===65534||r>1114111?"�":String.fromCodePoint(r)}function Ro(e){return e.replace(/[\t\n\r ]+/g," ").replace(/^ | $/g,"").toLowerCase().toUpperCase()}const ja=Uu(/[A-Za-z]/),Na=Uu(/[\dA-Za-z]/),pLe=Uu(/[#-'*+\--9=?A-Z^-~]/);function _y(e){return e!==null&&(e<32||e===127)}const zR=Uu(/\d/),fLe=Uu(/[\dA-Fa-f]/),mLe=Uu(/[!-/:-@[-`{-~]/);function Ot(e){return e!==null&&e<-2}function rn(e){return e!==null&&(e<0||e===32)}function Tr(e){return e===-2||e===-1||e===32}const S2=Uu(new RegExp("\\p{P}|\\p{S}","u")),Wc=Uu(/\s/);function Uu(e){return t;function t(r){return r!==null&&r>-1&&e.test(String.fromCharCode(r))}}function kp(e){const t=[];let r=-1,n=0,a=0;for(;++r<e.length;){const i=e.charCodeAt(r);let o="";if(i===37&&Na(e.charCodeAt(r+1))&&Na(e.charCodeAt(r+2)))a=2;else if(i<128)/[!#$&-;=?-Z_a-z~]/.test(String.fromCharCode(i))||(o=String.fromCharCode(i));else if(i>55295&&i<57344){const s=e.charCodeAt(r+1);i<56320&&s>56319&&s<57344?(o=String.fromCharCode(i,s),a=1):o="�"}else o=String.fromCharCode(i);o&&(t.push(e.slice(n,r),encodeURIComponent(o)),n=r+a+1,o=""),a&&(r+=a,a=0)}return t.join("")+e.slice(n)}function xr(e,t,r,n){const a=n?n-1:Number.POSITIVE_INFINITY;let i=0;return o;function o(l){return Tr(l)?(e.enter(r),s(l)):t(l)}function s(l){return Tr(l)&&i++<a?(e.consume(l),s):(e.exit(r),t(l))}}const gLe={tokenize:bLe};function bLe(e){const t=e.attempt(this.parser.constructs.contentInitial,n,a);let r;return t;function n(s){if(s===null){e.consume(s);return}return e.enter("lineEnding"),e.consume(s),e.exit("lineEnding"),xr(e,t,"linePrefix")}function a(s){return e.enter("paragraph"),i(s)}function i(s){const l=e.enter("chunkText",{contentType:"text",previous:r});return r&&(r.next=l),r=l,o(s)}function o(s){if(s===null){e.exit("chunkText"),e.exit("paragraph"),e.consume(s);return}return Ot(s)?(e.consume(s),e.exit("chunkText"),i):(e.consume(s),o)}}const yLe={tokenize:vLe},FP={tokenize:SLe};function vLe(e){const t=this,r=[];let n=0,a,i,o;return s;function s(k){if(n<r.length){const _=r[n];return t.containerState=_[1],e.attempt(_[0].continuation,l,u)(k)}return u(k)}function l(k){if(n++,t.containerState._closeFlow){t.containerState._closeFlow=void 0,a&&E();const _=t.events.length;let R=_,C;for(;R--;)if(t.events[R][0]==="exit"&&t.events[R][1].type==="chunkFlow"){C=t.events[R][1].end;break}w(n);let F=_;for(;F<t.events.length;)t.events[F][1].end={...C},F++;return Pi(t.events,R+1,0,t.events.slice(_)),t.events.length=F,u(k)}return s(k)}function u(k){if(n===r.length){if(!a)return f(k);if(a.currentConstruct&&a.currentConstruct.concrete)return b(k);t.interrupt=!!(a.currentConstruct&&!a._gfmTableDynamicInterruptHack)}return t.containerState={},e.check(FP,d,p)(k)}function d(k){return a&&E(),w(n),f(k)}function p(k){return t.parser.lazy[t.now().line]=n!==r.length,o=t.now().offset,b(k)}function f(k){return t.containerState={},e.attempt(FP,m,b)(k)}function m(k){return n++,r.push([t.currentConstruct,t.containerState]),f(k)}function b(k){if(k===null){a&&E(),w(0),e.consume(k);return}return a=a||t.parser.flow(t.now()),e.enter("chunkFlow",{_tokenizer:a,contentType:"flow",previous:i}),y(k)}function y(k){if(k===null){S(e.exit("chunkFlow"),!0),w(0),e.consume(k);return}return Ot(k)?(e.consume(k),S(e.exit("chunkFlow")),n=0,t.interrupt=void 0,s):(e.consume(k),y)}function S(k,_){const R=t.sliceStream(k);if(_&&R.push(null),k.previous=i,i&&(i.next=k),i=k,a.defineSkip(k.start),a.write(R),t.parser.lazy[k.start.line]){let C=a.events.length;for(;C--;)if(a.events[C][1].start.offset<o&&(!a.events[C][1].end||a.events[C][1].end.offset>o))return;const F=t.events.length;let L=F,I,M;for(;L--;)if(t.events[L][0]==="exit"&&t.events[L][1].type==="chunkFlow"){if(I){M=t.events[L][1].end;break}I=!0}for(w(n),C=F;C<t.events.length;)t.events[C][1].end={...M},C++;Pi(t.events,L+1,0,t.events.slice(F)),t.events.length=C}}function w(k){let _=r.length;for(;_-- >k;){const R=r[_];t.containerState=R[1],R[0].exit.call(t,e)}r.length=k}function E(){a.write([null]),i=void 0,a=void 0,t.containerState._closeFlow=void 0}}function SLe(e,t,r){return xr(e,e.attempt(this.parser.constructs.document,t,r),"linePrefix",this.parser.constructs.disable.null.includes("codeIndented")?void 0:4)}function Kh(e){if(e===null||rn(e)||Wc(e))return 1;if(S2(e))return 2}function E2(e,t,r){const n=[];let a=-1;for(;++a<e.length;){const i=e[a].resolveAll;i&&!n.includes(i)&&(t=i(t,r),n.push(i))}return t}const HR={name:"attention",resolveAll:ELe,tokenize:xLe};function ELe(e,t){let r=-1,n,a,i,o,s,l,u,d;for(;++r<e.length;)if(e[r][0]==="enter"&&e[r][1].type==="attentionSequence"&&e[r][1]._close){for(n=r;n--;)if(e[n][0]==="exit"&&e[n][1].type==="attentionSequence"&&e[n][1]._open&&t.sliceSerialize(e[n][1]).charCodeAt(0)===t.sliceSerialize(e[r][1]).charCodeAt(0)){if((e[n][1]._close||e[r][1]._open)&&(e[r][1].end.offset-e[r][1].start.offset)%3&&!((e[n][1].end.offset-e[n][1].start.offset+e[r][1].end.offset-e[r][1].start.offset)%3))continue;l=e[n][1].end.offset-e[n][1].start.offset>1&&e[r][1].end.offset-e[r][1].start.offset>1?2:1;const p={...e[n][1].end},f={...e[r][1].start};_P(p,-l),_P(f,l),o={type:l>1?"strongSequence":"emphasisSequence",start:p,end:{...e[n][1].end}},s={type:l>1?"strongSequence":"emphasisSequence",start:{...e[r][1].start},end:f},i={type:l>1?"strongText":"emphasisText",start:{...e[n][1].end},end:{...e[r][1].start}},a={type:l>1?"strong":"emphasis",start:{...o.start},end:{...s.end}},e[n][1].end={...o.start},e[r][1].start={...s.end},u=[],e[n][1].end.offset-e[n][1].start.offset&&(u=to(u,[["enter",e[n][1],t],["exit",e[n][1],t]])),u=to(u,[["enter",a,t],["enter",o,t],["exit",o,t],["enter",i,t]]),u=to(u,E2(t.parser.constructs.insideSpan.null,e.slice(n+1,r),t)),u=to(u,[["exit",i,t],["enter",s,t],["exit",s,t],["exit",a,t]]),e[r][1].end.offset-e[r][1].start.offset?(d=2,u=to(u,[["enter",e[r][1],t],["exit",e[r][1],t]])):d=0,Pi(e,n-1,r-n+3,u),r=n+u.length-d-2;break}}for(r=-1;++r<e.length;)e[r][1].type==="attentionSequence"&&(e[r][1].type="data");return e}function xLe(e,t){const r=this.parser.constructs.attentionMarkers.null,n=this.previous,a=Kh(n);let i;return o;function o(l){return i=l,e.enter("attentionSequence"),s(l)}function s(l){if(l===i)return e.consume(l),s;const u=e.exit("attentionSequence"),d=Kh(l),p=!d||d===2&&a||r.includes(l),f=!a||a===2&&d||r.includes(n);return u._open=!!(i===42?p:p&&(a||!f)),u._close=!!(i===42?f:f&&(d||!p)),t(l)}}function _P(e,t){e.column+=t,e.offset+=t,e._bufferIndex+=t}const wLe={name:"autolink",tokenize:kLe};function kLe(e,t,r){let n=0;return a;function a(m){return e.enter("autolink"),e.enter("autolinkMarker"),e.consume(m),e.exit("autolinkMarker"),e.enter("autolinkProtocol"),i}function i(m){return ja(m)?(e.consume(m),o):m===64?r(m):u(m)}function o(m){return m===43||m===45||m===46||Na(m)?(n=1,s(m)):u(m)}function s(m){return m===58?(e.consume(m),n=0,l):(m===43||m===45||m===46||Na(m))&&n++<32?(e.consume(m),s):(n=0,u(m))}function l(m){return m===62?(e.exit("autolinkProtocol"),e.enter("autolinkMarker"),e.consume(m),e.exit("autolinkMarker"),e.exit("autolink"),t):m===null||m===32||m===60||_y(m)?r(m):(e.consume(m),l)}function u(m){return m===64?(e.consume(m),d):pLe(m)?(e.consume(m),u):r(m)}function d(m){return Na(m)?p(m):r(m)}function p(m){return m===46?(e.consume(m),n=0,d):m===62?(e.exit("autolinkProtocol").type="autolinkEmail",e.enter("autolinkMarker"),e.consume(m),e.exit("autolinkMarker"),e.exit("autolink"),t):f(m)}function f(m){if((m===45||Na(m))&&n++<63){const b=m===45?f:p;return e.consume(m),b}return r(m)}}const Mg={partial:!0,tokenize:CLe};function CLe(e,t,r){return n;function n(i){return Tr(i)?xr(e,a,"linePrefix")(i):a(i)}function a(i){return i===null||Ot(i)?t(i):r(i)}}const Cre={continuation:{tokenize:ALe},exit:FLe,name:"blockQuote",tokenize:TLe};function TLe(e,t,r){const n=this;return a;function a(o){if(o===62){const s=n.containerState;return s.open||(e.enter("blockQuote",{_container:!0}),s.open=!0),e.enter("blockQuotePrefix"),e.enter("blockQuoteMarker"),e.consume(o),e.exit("blockQuoteMarker"),i}return r(o)}function i(o){return Tr(o)?(e.enter("blockQuotePrefixWhitespace"),e.consume(o),e.exit("blockQuotePrefixWhitespace"),e.exit("blockQuotePrefix"),t):(e.exit("blockQuotePrefix"),t(o))}}function ALe(e,t,r){const n=this;return a;function a(o){return Tr(o)?xr(e,i,"linePrefix",n.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(o):i(o)}function i(o){return e.attempt(Cre,t,r)(o)}}function FLe(e){e.exit("blockQuote")}const Tre={name:"characterEscape",tokenize:_Le};function _Le(e,t,r){return n;function n(i){return e.enter("characterEscape"),e.enter("escapeMarker"),e.consume(i),e.exit("escapeMarker"),a}function a(i){return mLe(i)?(e.enter("characterEscapeValue"),e.consume(i),e.exit("characterEscapeValue"),e.exit("characterEscape"),t):r(i)}}const Are={name:"characterReference",tokenize:RLe};function RLe(e,t,r){const n=this;let a=0,i,o;return s;function s(p){return e.enter("characterReference"),e.enter("characterReferenceMarker"),e.consume(p),e.exit("characterReferenceMarker"),l}function l(p){return p===35?(e.enter("characterReferenceMarkerNumeric"),e.consume(p),e.exit("characterReferenceMarkerNumeric"),u):(e.enter("characterReferenceValue"),i=31,o=Na,d(p))}function u(p){return p===88||p===120?(e.enter("characterReferenceMarkerHexadecimal"),e.consume(p),e.exit("characterReferenceMarkerHexadecimal"),e.enter("characterReferenceValue"),i=6,o=fLe,d):(e.enter("characterReferenceValue"),i=7,o=zR,d(p))}function d(p){if(p===59&&a){const f=e.exit("characterReferenceValue");return o===Na&&!G5(n.sliceSerialize(f))?r(p):(e.enter("characterReferenceMarker"),e.consume(p),e.exit("characterReferenceMarker"),e.exit("characterReference"),t)}return o(p)&&a++<i?(e.consume(p),d):r(p)}}const RP={partial:!0,tokenize:NLe},DP={concrete:!0,name:"codeFenced",tokenize:DLe};function DLe(e,t,r){const n=this,a={partial:!0,tokenize:R};let i=0,o=0,s;return l;function l(C){return u(C)}function u(C){const F=n.events[n.events.length-1];return i=F&&F[1].type==="linePrefix"?F[2].sliceSerialize(F[1],!0).length:0,s=C,e.enter("codeFenced"),e.enter("codeFencedFence"),e.enter("codeFencedFenceSequence"),d(C)}function d(C){return C===s?(o++,e.consume(C),d):o<3?r(C):(e.exit("codeFencedFenceSequence"),Tr(C)?xr(e,p,"whitespace")(C):p(C))}function p(C){return C===null||Ot(C)?(e.exit("codeFencedFence"),n.interrupt?t(C):e.check(RP,y,_)(C)):(e.enter("codeFencedFenceInfo"),e.enter("chunkString",{contentType:"string"}),f(C))}function f(C){return C===null||Ot(C)?(e.exit("chunkString"),e.exit("codeFencedFenceInfo"),p(C)):Tr(C)?(e.exit("chunkString"),e.exit("codeFencedFenceInfo"),xr(e,m,"whitespace")(C)):C===96&&C===s?r(C):(e.consume(C),f)}function m(C){return C===null||Ot(C)?p(C):(e.enter("codeFencedFenceMeta"),e.enter("chunkString",{contentType:"string"}),b(C))}function b(C){return C===null||Ot(C)?(e.exit("chunkString"),e.exit("codeFencedFenceMeta"),p(C)):C===96&&C===s?r(C):(e.consume(C),b)}function y(C){return e.attempt(a,_,S)(C)}function S(C){return e.enter("lineEnding"),e.consume(C),e.exit("lineEnding"),w}function w(C){return i>0&&Tr(C)?xr(e,E,"linePrefix",i+1)(C):E(C)}function E(C){return C===null||Ot(C)?e.check(RP,y,_)(C):(e.enter("codeFlowValue"),k(C))}function k(C){return C===null||Ot(C)?(e.exit("codeFlowValue"),E(C)):(e.consume(C),k)}function _(C){return e.exit("codeFenced"),t(C)}function R(C,F,L){let I=0;return M;function M(X){return C.enter("lineEnding"),C.consume(X),C.exit("lineEnding"),j}function j(X){return C.enter("codeFencedFence"),Tr(X)?xr(C,Y,"linePrefix",n.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(X):Y(X)}function Y(X){return X===s?(C.enter("codeFencedFenceSequence"),W(X)):L(X)}function W(X){return X===s?(I++,C.consume(X),W):I>=o?(C.exit("codeFencedFenceSequence"),Tr(X)?xr(C,$,"whitespace")(X):$(X)):L(X)}function $(X){return X===null||Ot(X)?(C.exit("codeFencedFence"),F(X)):L(X)}}}function NLe(e,t,r){const n=this;return a;function a(o){return o===null?r(o):(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),i)}function i(o){return n.parser.lazy[n.now().line]?r(o):t(o)}}const lk={name:"codeIndented",tokenize:MLe},LLe={partial:!0,tokenize:ILe};function MLe(e,t,r){const n=this;return a;function a(u){return e.enter("codeIndented"),xr(e,i,"linePrefix",5)(u)}function i(u){const d=n.events[n.events.length-1];return d&&d[1].type==="linePrefix"&&d[2].sliceSerialize(d[1],!0).length>=4?o(u):r(u)}function o(u){return u===null?l(u):Ot(u)?e.attempt(LLe,o,l)(u):(e.enter("codeFlowValue"),s(u))}function s(u){return u===null||Ot(u)?(e.exit("codeFlowValue"),o(u)):(e.consume(u),s)}function l(u){return e.exit("codeIndented"),t(u)}}function ILe(e,t,r){const n=this;return a;function a(o){return n.parser.lazy[n.now().line]?r(o):Ot(o)?(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),a):xr(e,i,"linePrefix",5)(o)}function i(o){const s=n.events[n.events.length-1];return s&&s[1].type==="linePrefix"&&s[2].sliceSerialize(s[1],!0).length>=4?t(o):Ot(o)?a(o):r(o)}}const OLe={name:"codeText",previous:PLe,resolve:BLe,tokenize:zLe};function BLe(e){let t=e.length-4,r=3,n,a;if((e[r][1].type==="lineEnding"||e[r][1].type==="space")&&(e[t][1].type==="lineEnding"||e[t][1].type==="space")){for(n=r;++n<t;)if(e[n][1].type==="codeTextData"){e[r][1].type="codeTextPadding",e[t][1].type="codeTextPadding",r+=2,t-=2;break}}for(n=r-1,t++;++n<=t;)a===void 0?n!==t&&e[n][1].type!=="lineEnding"&&(a=n):(n===t||e[n][1].type==="lineEnding")&&(e[a][1].type="codeTextData",n!==a+2&&(e[a][1].end=e[n-1][1].end,e.splice(a+2,n-a-2),t-=n-a-2,n=a+2),a=void 0);return e}function PLe(e){return e!==96||this.events[this.events.length-1][1].type==="characterEscape"}function zLe(e,t,r){let n=0,a,i;return o;function o(p){return e.enter("codeText"),e.enter("codeTextSequence"),s(p)}function s(p){return p===96?(e.consume(p),n++,s):(e.exit("codeTextSequence"),l(p))}function l(p){return p===null?r(p):p===32?(e.enter("space"),e.consume(p),e.exit("space"),l):p===96?(i=e.enter("codeTextSequence"),a=0,d(p)):Ot(p)?(e.enter("lineEnding"),e.consume(p),e.exit("lineEnding"),l):(e.enter("codeTextData"),u(p))}function u(p){return p===null||p===32||p===96||Ot(p)?(e.exit("codeTextData"),l(p)):(e.consume(p),u)}function d(p){return p===96?(e.consume(p),a++,d):a===n?(e.exit("codeTextSequence"),e.exit("codeText"),t(p)):(i.type="codeTextData",u(p))}}class HLe{constructor(t){this.left=t?[...t]:[],this.right=[]}get(t){if(t<0||t>=this.left.length+this.right.length)throw new RangeError("Cannot access index `"+t+"` in a splice buffer of size `"+(this.left.length+this.right.length)+"`");return t<this.left.length?this.left[t]:this.right[this.right.length-t+this.left.length-1]}get length(){return this.left.length+this.right.length}shift(){return this.setCursor(0),this.right.pop()}slice(t,r){const n=r??Number.POSITIVE_INFINITY;return n<this.left.length?this.left.slice(t,n):t>this.left.length?this.right.slice(this.right.length-n+this.left.length,this.right.length-t+this.left.length).reverse():this.left.slice(t).concat(this.right.slice(this.right.length-n+this.left.length).reverse())}splice(t,r,n){const a=r||0;this.setCursor(Math.trunc(t));const i=this.right.splice(this.right.length-a,Number.POSITIVE_INFINITY);return n&&zf(this.left,n),i.reverse()}pop(){return this.setCursor(Number.POSITIVE_INFINITY),this.left.pop()}push(t){this.setCursor(Number.POSITIVE_INFINITY),this.left.push(t)}pushMany(t){this.setCursor(Number.POSITIVE_INFINITY),zf(this.left,t)}unshift(t){this.setCursor(0),this.right.push(t)}unshiftMany(t){this.setCursor(0),zf(this.right,t.reverse())}setCursor(t){if(!(t===this.left.length||t>this.left.length&&this.right.length===0||t<0&&this.left.length===0))if(t<this.left.length){const r=this.left.splice(t,Number.POSITIVE_INFINITY);zf(this.right,r.reverse())}else{const r=this.right.splice(this.left.length+this.right.length-t,Number.POSITIVE_INFINITY);zf(this.left,r.reverse())}}}function zf(e,t){let r=0;if(t.length<1e4)e.push(...t);else for(;r<t.length;)e.push(...t.slice(r,r+1e4)),r+=1e4}function Fre(e){const t={};let r=-1,n,a,i,o,s,l,u;const d=new HLe(e);for(;++r<d.length;){for(;r in t;)r=t[r];if(n=d.get(r),r&&n[1].type==="chunkFlow"&&d.get(r-1)[1].type==="listItemPrefix"&&(l=n[1]._tokenizer.events,i=0,i<l.length&&l[i][1].type==="lineEndingBlank"&&(i+=2),i<l.length&&l[i][1].type==="content"))for(;++i<l.length&&l[i][1].type!=="content";)l[i][1].type==="chunkText"&&(l[i][1]._isInFirstContentOfListItem=!0,i++);if(n[0]==="enter")n[1].contentType&&(Object.assign(t,ULe(d,r)),r=t[r],u=!0);else if(n[1]._container){for(i=r,a=void 0;i--;)if(o=d.get(i),o[1].type==="lineEnding"||o[1].type==="lineEndingBlank")o[0]==="enter"&&(a&&(d.get(a)[1].type="lineEndingBlank"),o[1].type="lineEnding",a=i);else if(o[1].type!=="linePrefix")break;a&&(n[1].end={...d.get(a)[1].start},s=d.slice(a,r),s.unshift(n),d.splice(a,r-a+1,s))}}return Pi(e,0,Number.POSITIVE_INFINITY,d.slice(0)),!u}function ULe(e,t){const r=e.get(t)[1],n=e.get(t)[2];let a=t-1;const i=[],o=r._tokenizer||n.parser[r.contentType](r.start),s=o.events,l=[],u={};let d,p,f=-1,m=r,b=0,y=0;const S=[y];for(;m;){for(;e.get(++a)[1]!==m;);i.push(a),m._tokenizer||(d=n.sliceStream(m),m.next||d.push(null),p&&o.defineSkip(m.start),m._isInFirstContentOfListItem&&(o._gfmTasklistFirstContentOfListItem=!0),o.write(d),m._isInFirstContentOfListItem&&(o._gfmTasklistFirstContentOfListItem=void 0)),p=m,m=m.next}for(m=r;++f<s.length;)s[f][0]==="exit"&&s[f-1][0]==="enter"&&s[f][1].type===s[f-1][1].type&&s[f][1].start.line!==s[f][1].end.line&&(y=f+1,S.push(y),m._tokenizer=void 0,m.previous=void 0,m=m.next);for(o.events=[],m?(m._tokenizer=void 0,m.previous=void 0):S.pop(),f=S.length;f--;){const w=s.slice(S[f],S[f+1]),E=i.pop();l.push([E,E+w.length-1]),e.splice(E,2,w)}for(l.reverse(),f=-1;++f<l.length;)u[b+l[f][0]]=b+l[f][1],b+=l[f][1]-l[f][0]-1;return u}const $Le={resolve:jLe,tokenize:qLe},GLe={partial:!0,tokenize:WLe};function jLe(e){return Fre(e),e}function qLe(e,t){let r;return n;function n(s){return e.enter("content"),r=e.enter("chunkContent",{contentType:"content"}),a(s)}function a(s){return s===null?i(s):Ot(s)?e.check(GLe,o,i)(s):(e.consume(s),a)}function i(s){return e.exit("chunkContent"),e.exit("content"),t(s)}function o(s){return e.consume(s),e.exit("chunkContent"),r.next=e.enter("chunkContent",{contentType:"content",previous:r}),r=r.next,a}}function WLe(e,t,r){const n=this;return a;function a(o){return e.exit("chunkContent"),e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),xr(e,i,"linePrefix")}function i(o){if(o===null||Ot(o))return r(o);const s=n.events[n.events.length-1];return!n.parser.constructs.disable.null.includes("codeIndented")&&s&&s[1].type==="linePrefix"&&s[2].sliceSerialize(s[1],!0).length>=4?t(o):e.interrupt(n.parser.constructs.flow,r,t)(o)}}function _re(e,t,r,n,a,i,o,s,l){const u=l||Number.POSITIVE_INFINITY;let d=0;return p;function p(w){return w===60?(e.enter(n),e.enter(a),e.enter(i),e.consume(w),e.exit(i),f):w===null||w===32||w===41||_y(w)?r(w):(e.enter(n),e.enter(o),e.enter(s),e.enter("chunkString",{contentType:"string"}),y(w))}function f(w){return w===62?(e.enter(i),e.consume(w),e.exit(i),e.exit(a),e.exit(n),t):(e.enter(s),e.enter("chunkString",{contentType:"string"}),m(w))}function m(w){return w===62?(e.exit("chunkString"),e.exit(s),f(w)):w===null||w===60||Ot(w)?r(w):(e.consume(w),w===92?b:m)}function b(w){return w===60||w===62||w===92?(e.consume(w),m):m(w)}function y(w){return!d&&(w===null||w===41||rn(w))?(e.exit("chunkString"),e.exit(s),e.exit(o),e.exit(n),t(w)):d<u&&w===40?(e.consume(w),d++,y):w===41?(e.consume(w),d--,y):w===null||w===32||w===40||_y(w)?r(w):(e.consume(w),w===92?S:y)}function S(w){return w===40||w===41||w===92?(e.consume(w),y):y(w)}}function Rre(e,t,r,n,a,i){const o=this;let s=0,l;return u;function u(m){return e.enter(n),e.enter(a),e.consume(m),e.exit(a),e.enter(i),d}function d(m){return s>999||m===null||m===91||m===93&&!l||m===94&&!s&&"_hiddenFootnoteSupport"in o.parser.constructs?r(m):m===93?(e.exit(i),e.enter(a),e.consume(m),e.exit(a),e.exit(n),t):Ot(m)?(e.enter("lineEnding"),e.consume(m),e.exit("lineEnding"),d):(e.enter("chunkString",{contentType:"string"}),p(m))}function p(m){return m===null||m===91||m===93||Ot(m)||s++>999?(e.exit("chunkString"),d(m)):(e.consume(m),l||(l=!Tr(m)),m===92?f:p)}function f(m){return m===91||m===92||m===93?(e.consume(m),s++,p):p(m)}}function Dre(e,t,r,n,a,i){let o;return s;function s(f){return f===34||f===39||f===40?(e.enter(n),e.enter(a),e.consume(f),e.exit(a),o=f===40?41:f,l):r(f)}function l(f){return f===o?(e.enter(a),e.consume(f),e.exit(a),e.exit(n),t):(e.enter(i),u(f))}function u(f){return f===o?(e.exit(i),l(o)):f===null?r(f):Ot(f)?(e.enter("lineEnding"),e.consume(f),e.exit("lineEnding"),xr(e,u,"linePrefix")):(e.enter("chunkString",{contentType:"string"}),d(f))}function d(f){return f===o||f===null||Ot(f)?(e.exit("chunkString"),u(f)):(e.consume(f),f===92?p:d)}function p(f){return f===o||f===92?(e.consume(f),d):d(f)}}function Sm(e,t){let r;return n;function n(a){return Ot(a)?(e.enter("lineEnding"),e.consume(a),e.exit("lineEnding"),r=!0,n):Tr(a)?xr(e,n,r?"linePrefix":"lineSuffix")(a):t(a)}}const VLe={name:"definition",tokenize:YLe},KLe={partial:!0,tokenize:JLe};function YLe(e,t,r){const n=this;let a;return i;function i(m){return e.enter("definition"),o(m)}function o(m){return Rre.call(n,e,s,r,"definitionLabel","definitionLabelMarker","definitionLabelString")(m)}function s(m){return a=Ro(n.sliceSerialize(n.events[n.events.length-1][1]).slice(1,-1)),m===58?(e.enter("definitionMarker"),e.consume(m),e.exit("definitionMarker"),l):r(m)}function l(m){return rn(m)?Sm(e,u)(m):u(m)}function u(m){return _re(e,d,r,"definitionDestination","definitionDestinationLiteral","definitionDestinationLiteralMarker","definitionDestinationRaw","definitionDestinationString")(m)}function d(m){return e.attempt(KLe,p,p)(m)}function p(m){return Tr(m)?xr(e,f,"whitespace")(m):f(m)}function f(m){return m===null||Ot(m)?(e.exit("definition"),n.parser.defined.push(a),t(m)):r(m)}}function JLe(e,t,r){return n;function n(s){return rn(s)?Sm(e,a)(s):r(s)}function a(s){return Dre(e,i,r,"definitionTitle","definitionTitleMarker","definitionTitleString")(s)}function i(s){return Tr(s)?xr(e,o,"whitespace")(s):o(s)}function o(s){return s===null||Ot(s)?t(s):r(s)}}const XLe={name:"hardBreakEscape",tokenize:ZLe};function ZLe(e,t,r){return n;function n(i){return e.enter("hardBreakEscape"),e.consume(i),a}function a(i){return Ot(i)?(e.exit("hardBreakEscape"),t(i)):r(i)}}const QLe={name:"headingAtx",resolve:eMe,tokenize:tMe};function eMe(e,t){let r=e.length-2,n=3,a,i;return e[n][1].type==="whitespace"&&(n+=2),r-2>n&&e[r][1].type==="whitespace"&&(r-=2),e[r][1].type==="atxHeadingSequence"&&(n===r-1||r-4>n&&e[r-2][1].type==="whitespace")&&(r-=n+1===r?2:4),r>n&&(a={type:"atxHeadingText",start:e[n][1].start,end:e[r][1].end},i={type:"chunkText",start:e[n][1].start,end:e[r][1].end,contentType:"text"},Pi(e,n,r-n+1,[["enter",a,t],["enter",i,t],["exit",i,t],["exit",a,t]])),e}function tMe(e,t,r){let n=0;return a;function a(d){return e.enter("atxHeading"),i(d)}function i(d){return e.enter("atxHeadingSequence"),o(d)}function o(d){return d===35&&n++<6?(e.consume(d),o):d===null||rn(d)?(e.exit("atxHeadingSequence"),s(d)):r(d)}function s(d){return d===35?(e.enter("atxHeadingSequence"),l(d)):d===null||Ot(d)?(e.exit("atxHeading"),t(d)):Tr(d)?xr(e,s,"whitespace")(d):(e.enter("atxHeadingText"),u(d))}function l(d){return d===35?(e.consume(d),l):(e.exit("atxHeadingSequence"),s(d))}function u(d){return d===null||d===35||rn(d)?(e.exit("atxHeadingText"),s(d)):(e.consume(d),u)}}const rMe=["address","article","aside","base","basefont","blockquote","body","caption","center","col","colgroup","dd","details","dialog","dir","div","dl","dt","fieldset","figcaption","figure","footer","form","frame","frameset","h1","h2","h3","h4","h5","h6","head","header","hr","html","iframe","legend","li","link","main","menu","menuitem","nav","noframes","ol","optgroup","option","p","param","search","section","summary","table","tbody","td","tfoot","th","thead","title","tr","track","ul"],NP=["pre","script","style","textarea"],nMe={concrete:!0,name:"htmlFlow",resolveTo:oMe,tokenize:sMe},aMe={partial:!0,tokenize:uMe},iMe={partial:!0,tokenize:lMe};function oMe(e){let t=e.length;for(;t--&&!(e[t][0]==="enter"&&e[t][1].type==="htmlFlow"););return t>1&&e[t-2][1].type==="linePrefix"&&(e[t][1].start=e[t-2][1].start,e[t+1][1].start=e[t-2][1].start,e.splice(t-2,2)),e}function sMe(e,t,r){const n=this;let a,i,o,s,l;return u;function u(P){return d(P)}function d(P){return e.enter("htmlFlow"),e.enter("htmlFlowData"),e.consume(P),p}function p(P){return P===33?(e.consume(P),f):P===47?(e.consume(P),i=!0,y):P===63?(e.consume(P),a=3,n.interrupt?t:J):ja(P)?(e.consume(P),o=String.fromCharCode(P),S):r(P)}function f(P){return P===45?(e.consume(P),a=2,m):P===91?(e.consume(P),a=5,s=0,b):ja(P)?(e.consume(P),a=4,n.interrupt?t:J):r(P)}function m(P){return P===45?(e.consume(P),n.interrupt?t:J):r(P)}function b(P){const ne="CDATA[";return P===ne.charCodeAt(s++)?(e.consume(P),s===ne.length?n.interrupt?t:Y:b):r(P)}function y(P){return ja(P)?(e.consume(P),o=String.fromCharCode(P),S):r(P)}function S(P){if(P===null||P===47||P===62||rn(P)){const ne=P===47,oe=o.toLowerCase();return!ne&&!i&&NP.includes(oe)?(a=1,n.interrupt?t(P):Y(P)):rMe.includes(o.toLowerCase())?(a=6,ne?(e.consume(P),w):n.interrupt?t(P):Y(P)):(a=7,n.interrupt&&!n.parser.lazy[n.now().line]?r(P):i?E(P):k(P))}return P===45||Na(P)?(e.consume(P),o+=String.fromCharCode(P),S):r(P)}function w(P){return P===62?(e.consume(P),n.interrupt?t:Y):r(P)}function E(P){return Tr(P)?(e.consume(P),E):M(P)}function k(P){return P===47?(e.consume(P),M):P===58||P===95||ja(P)?(e.consume(P),_):Tr(P)?(e.consume(P),k):M(P)}function _(P){return P===45||P===46||P===58||P===95||Na(P)?(e.consume(P),_):R(P)}function R(P){return P===61?(e.consume(P),C):Tr(P)?(e.consume(P),R):k(P)}function C(P){return P===null||P===60||P===61||P===62||P===96?r(P):P===34||P===39?(e.consume(P),l=P,F):Tr(P)?(e.consume(P),C):L(P)}function F(P){return P===l?(e.consume(P),l=null,I):P===null||Ot(P)?r(P):(e.consume(P),F)}function L(P){return P===null||P===34||P===39||P===47||P===60||P===61||P===62||P===96||rn(P)?R(P):(e.consume(P),L)}function I(P){return P===47||P===62||Tr(P)?k(P):r(P)}function M(P){return P===62?(e.consume(P),j):r(P)}function j(P){return P===null||Ot(P)?Y(P):Tr(P)?(e.consume(P),j):r(P)}function Y(P){return P===45&&a===2?(e.consume(P),z):P===60&&a===1?(e.consume(P),H):P===62&&a===4?(e.consume(P),K):P===63&&a===3?(e.consume(P),J):P===93&&a===5?(e.consume(P),ee):Ot(P)&&(a===6||a===7)?(e.exit("htmlFlowData"),e.check(aMe,G,W)(P)):P===null||Ot(P)?(e.exit("htmlFlowData"),W(P)):(e.consume(P),Y)}function W(P){return e.check(iMe,$,G)(P)}function $(P){return e.enter("lineEnding"),e.consume(P),e.exit("lineEnding"),X}function X(P){return P===null||Ot(P)?W(P):(e.enter("htmlFlowData"),Y(P))}function z(P){return P===45?(e.consume(P),J):Y(P)}function H(P){return P===47?(e.consume(P),o="",B):Y(P)}function B(P){if(P===62){const ne=o.toLowerCase();return NP.includes(ne)?(e.consume(P),K):Y(P)}return ja(P)&&o.length<8?(e.consume(P),o+=String.fromCharCode(P),B):Y(P)}function ee(P){return P===93?(e.consume(P),J):Y(P)}function J(P){return P===62?(e.consume(P),K):P===45&&a===2?(e.consume(P),J):Y(P)}function K(P){return P===null||Ot(P)?(e.exit("htmlFlowData"),G(P)):(e.consume(P),K)}function G(P){return e.exit("htmlFlow"),t(P)}}function lMe(e,t,r){const n=this;return a;function a(o){return Ot(o)?(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),i):r(o)}function i(o){return n.parser.lazy[n.now().line]?r(o):t(o)}}function uMe(e,t,r){return n;function n(a){return e.enter("lineEnding"),e.consume(a),e.exit("lineEnding"),e.attempt(Mg,t,r)}}const cMe={name:"htmlText",tokenize:dMe};function dMe(e,t,r){const n=this;let a,i,o;return s;function s(J){return e.enter("htmlText"),e.enter("htmlTextData"),e.consume(J),l}function l(J){return J===33?(e.consume(J),u):J===47?(e.consume(J),R):J===63?(e.consume(J),k):ja(J)?(e.consume(J),L):r(J)}function u(J){return J===45?(e.consume(J),d):J===91?(e.consume(J),i=0,b):ja(J)?(e.consume(J),E):r(J)}function d(J){return J===45?(e.consume(J),m):r(J)}function p(J){return J===null?r(J):J===45?(e.consume(J),f):Ot(J)?(o=p,H(J)):(e.consume(J),p)}function f(J){return J===45?(e.consume(J),m):p(J)}function m(J){return J===62?z(J):J===45?f(J):p(J)}function b(J){const K="CDATA[";return J===K.charCodeAt(i++)?(e.consume(J),i===K.length?y:b):r(J)}function y(J){return J===null?r(J):J===93?(e.consume(J),S):Ot(J)?(o=y,H(J)):(e.consume(J),y)}function S(J){return J===93?(e.consume(J),w):y(J)}function w(J){return J===62?z(J):J===93?(e.consume(J),w):y(J)}function E(J){return J===null||J===62?z(J):Ot(J)?(o=E,H(J)):(e.consume(J),E)}function k(J){return J===null?r(J):J===63?(e.consume(J),_):Ot(J)?(o=k,H(J)):(e.consume(J),k)}function _(J){return J===62?z(J):k(J)}function R(J){return ja(J)?(e.consume(J),C):r(J)}function C(J){return J===45||Na(J)?(e.consume(J),C):F(J)}function F(J){return Ot(J)?(o=F,H(J)):Tr(J)?(e.consume(J),F):z(J)}function L(J){return J===45||Na(J)?(e.consume(J),L):J===47||J===62||rn(J)?I(J):r(J)}function I(J){return J===47?(e.consume(J),z):J===58||J===95||ja(J)?(e.consume(J),M):Ot(J)?(o=I,H(J)):Tr(J)?(e.consume(J),I):z(J)}function M(J){return J===45||J===46||J===58||J===95||Na(J)?(e.consume(J),M):j(J)}function j(J){return J===61?(e.consume(J),Y):Ot(J)?(o=j,H(J)):Tr(J)?(e.consume(J),j):I(J)}function Y(J){return J===null||J===60||J===61||J===62||J===96?r(J):J===34||J===39?(e.consume(J),a=J,W):Ot(J)?(o=Y,H(J)):Tr(J)?(e.consume(J),Y):(e.consume(J),$)}function W(J){return J===a?(e.consume(J),a=void 0,X):J===null?r(J):Ot(J)?(o=W,H(J)):(e.consume(J),W)}function $(J){return J===null||J===34||J===39||J===60||J===61||J===96?r(J):J===47||J===62||rn(J)?I(J):(e.consume(J),$)}function X(J){return J===47||J===62||rn(J)?I(J):r(J)}function z(J){return J===62?(e.consume(J),e.exit("htmlTextData"),e.exit("htmlText"),t):r(J)}function H(J){return e.exit("htmlTextData"),e.enter("lineEnding"),e.consume(J),e.exit("lineEnding"),B}function B(J){return Tr(J)?xr(e,ee,"linePrefix",n.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(J):ee(J)}function ee(J){return e.enter("htmlTextData"),o(J)}}const j5={name:"labelEnd",resolveAll:mMe,resolveTo:gMe,tokenize:bMe},hMe={tokenize:yMe},pMe={tokenize:vMe},fMe={tokenize:SMe};function mMe(e){let t=-1;const r=[];for(;++t<e.length;){const n=e[t][1];if(r.push(e[t]),n.type==="labelImage"||n.type==="labelLink"||n.type==="labelEnd"){const a=n.type==="labelImage"?4:2;n.type="data",t+=a}}return e.length!==r.length&&Pi(e,0,e.length,r),e}function gMe(e,t){let r=e.length,n=0,a,i,o,s;for(;r--;)if(a=e[r][1],i){if(a.type==="link"||a.type==="labelLink"&&a._inactive)break;e[r][0]==="enter"&&a.type==="labelLink"&&(a._inactive=!0)}else if(o){if(e[r][0]==="enter"&&(a.type==="labelImage"||a.type==="labelLink")&&!a._balanced&&(i=r,a.type!=="labelLink")){n=2;break}}else a.type==="labelEnd"&&(o=r);const l={type:e[i][1].type==="labelLink"?"link":"image",start:{...e[i][1].start},end:{...e[e.length-1][1].end}},u={type:"label",start:{...e[i][1].start},end:{...e[o][1].end}},d={type:"labelText",start:{...e[i+n+2][1].end},end:{...e[o-2][1].start}};return s=[["enter",l,t],["enter",u,t]],s=to(s,e.slice(i+1,i+n+3)),s=to(s,[["enter",d,t]]),s=to(s,E2(t.parser.constructs.insideSpan.null,e.slice(i+n+4,o-3),t)),s=to(s,[["exit",d,t],e[o-2],e[o-1],["exit",u,t]]),s=to(s,e.slice(o+1)),s=to(s,[["exit",l,t]]),Pi(e,i,e.length,s),e}function bMe(e,t,r){const n=this;let a=n.events.length,i,o;for(;a--;)if((n.events[a][1].type==="labelImage"||n.events[a][1].type==="labelLink")&&!n.events[a][1]._balanced){i=n.events[a][1];break}return s;function s(f){return i?i._inactive?p(f):(o=n.parser.defined.includes(Ro(n.sliceSerialize({start:i.end,end:n.now()}))),e.enter("labelEnd"),e.enter("labelMarker"),e.consume(f),e.exit("labelMarker"),e.exit("labelEnd"),l):r(f)}function l(f){return f===40?e.attempt(hMe,d,o?d:p)(f):f===91?e.attempt(pMe,d,o?u:p)(f):o?d(f):p(f)}function u(f){return e.attempt(fMe,d,p)(f)}function d(f){return t(f)}function p(f){return i._balanced=!0,r(f)}}function yMe(e,t,r){return n;function n(p){return e.enter("resource"),e.enter("resourceMarker"),e.consume(p),e.exit("resourceMarker"),a}function a(p){return rn(p)?Sm(e,i)(p):i(p)}function i(p){return p===41?d(p):_re(e,o,s,"resourceDestination","resourceDestinationLiteral","resourceDestinationLiteralMarker","resourceDestinationRaw","resourceDestinationString",32)(p)}function o(p){return rn(p)?Sm(e,l)(p):d(p)}function s(p){return r(p)}function l(p){return p===34||p===39||p===40?Dre(e,u,r,"resourceTitle","resourceTitleMarker","resourceTitleString")(p):d(p)}function u(p){return rn(p)?Sm(e,d)(p):d(p)}function d(p){return p===41?(e.enter("resourceMarker"),e.consume(p),e.exit("resourceMarker"),e.exit("resource"),t):r(p)}}function vMe(e,t,r){const n=this;return a;function a(s){return Rre.call(n,e,i,o,"reference","referenceMarker","referenceString")(s)}function i(s){return n.parser.defined.includes(Ro(n.sliceSerialize(n.events[n.events.length-1][1]).slice(1,-1)))?t(s):r(s)}function o(s){return r(s)}}function SMe(e,t,r){return n;function n(i){return e.enter("reference"),e.enter("referenceMarker"),e.consume(i),e.exit("referenceMarker"),a}function a(i){return i===93?(e.enter("referenceMarker"),e.consume(i),e.exit("referenceMarker"),e.exit("reference"),t):r(i)}}const EMe={name:"labelStartImage",resolveAll:j5.resolveAll,tokenize:xMe};function xMe(e,t,r){const n=this;return a;function a(s){return e.enter("labelImage"),e.enter("labelImageMarker"),e.consume(s),e.exit("labelImageMarker"),i}function i(s){return s===91?(e.enter("labelMarker"),e.consume(s),e.exit("labelMarker"),e.exit("labelImage"),o):r(s)}function o(s){return s===94&&"_hiddenFootnoteSupport"in n.parser.constructs?r(s):t(s)}}const wMe={name:"labelStartLink",resolveAll:j5.resolveAll,tokenize:kMe};function kMe(e,t,r){const n=this;return a;function a(o){return e.enter("labelLink"),e.enter("labelMarker"),e.consume(o),e.exit("labelMarker"),e.exit("labelLink"),i}function i(o){return o===94&&"_hiddenFootnoteSupport"in n.parser.constructs?r(o):t(o)}}const uk={name:"lineEnding",tokenize:CMe};function CMe(e,t){return r;function r(n){return e.enter("lineEnding"),e.consume(n),e.exit("lineEnding"),xr(e,t,"linePrefix")}}const I1={name:"thematicBreak",tokenize:TMe};function TMe(e,t,r){let n=0,a;return i;function i(u){return e.enter("thematicBreak"),o(u)}function o(u){return a=u,s(u)}function s(u){return u===a?(e.enter("thematicBreakSequence"),l(u)):n>=3&&(u===null||Ot(u))?(e.exit("thematicBreak"),t(u)):r(u)}function l(u){return u===a?(e.consume(u),n++,l):(e.exit("thematicBreakSequence"),Tr(u)?xr(e,s,"whitespace")(u):s(u))}}const oi={continuation:{tokenize:RMe},exit:NMe,name:"list",tokenize:_Me},AMe={partial:!0,tokenize:LMe},FMe={partial:!0,tokenize:DMe};function _Me(e,t,r){const n=this,a=n.events[n.events.length-1];let i=a&&a[1].type==="linePrefix"?a[2].sliceSerialize(a[1],!0).length:0,o=0;return s;function s(m){const b=n.containerState.type||(m===42||m===43||m===45?"listUnordered":"listOrdered");if(b==="listUnordered"?!n.containerState.marker||m===n.containerState.marker:zR(m)){if(n.containerState.type||(n.containerState.type=b,e.enter(b,{_container:!0})),b==="listUnordered")return e.enter("listItemPrefix"),m===42||m===45?e.check(I1,r,u)(m):u(m);if(!n.interrupt||m===49)return e.enter("listItemPrefix"),e.enter("listItemValue"),l(m)}return r(m)}function l(m){return zR(m)&&++o<10?(e.consume(m),l):(!n.interrupt||o<2)&&(n.containerState.marker?m===n.containerState.marker:m===41||m===46)?(e.exit("listItemValue"),u(m)):r(m)}function u(m){return e.enter("listItemMarker"),e.consume(m),e.exit("listItemMarker"),n.containerState.marker=n.containerState.marker||m,e.check(Mg,n.interrupt?r:d,e.attempt(AMe,f,p))}function d(m){return n.containerState.initialBlankLine=!0,i++,f(m)}function p(m){return Tr(m)?(e.enter("listItemPrefixWhitespace"),e.consume(m),e.exit("listItemPrefixWhitespace"),f):r(m)}function f(m){return n.containerState.size=i+n.sliceSerialize(e.exit("listItemPrefix"),!0).length,t(m)}}function RMe(e,t,r){const n=this;return n.containerState._closeFlow=void 0,e.check(Mg,a,i);function a(s){return n.containerState.furtherBlankLines=n.containerState.furtherBlankLines||n.containerState.initialBlankLine,xr(e,t,"listItemIndent",n.containerState.size+1)(s)}function i(s){return n.containerState.furtherBlankLines||!Tr(s)?(n.containerState.furtherBlankLines=void 0,n.containerState.initialBlankLine=void 0,o(s)):(n.containerState.furtherBlankLines=void 0,n.containerState.initialBlankLine=void 0,e.attempt(FMe,t,o)(s))}function o(s){return n.containerState._closeFlow=!0,n.interrupt=void 0,xr(e,e.attempt(oi,t,r),"linePrefix",n.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(s)}}function DMe(e,t,r){const n=this;return xr(e,a,"listItemIndent",n.containerState.size+1);function a(i){const o=n.events[n.events.length-1];return o&&o[1].type==="listItemIndent"&&o[2].sliceSerialize(o[1],!0).length===n.containerState.size?t(i):r(i)}}function NMe(e){e.exit(this.containerState.type)}function LMe(e,t,r){const n=this;return xr(e,a,"listItemPrefixWhitespace",n.parser.constructs.disable.null.includes("codeIndented")?void 0:5);function a(i){const o=n.events[n.events.length-1];return!Tr(i)&&o&&o[1].type==="listItemPrefixWhitespace"?t(i):r(i)}}const LP={name:"setextUnderline",resolveTo:MMe,tokenize:IMe};function MMe(e,t){let r=e.length,n,a,i;for(;r--;)if(e[r][0]==="enter"){if(e[r][1].type==="content"){n=r;break}e[r][1].type==="paragraph"&&(a=r)}else e[r][1].type==="content"&&e.splice(r,1),!i&&e[r][1].type==="definition"&&(i=r);const o={type:"setextHeading",start:{...e[a][1].start},end:{...e[e.length-1][1].end}};return e[a][1].type="setextHeadingText",i?(e.splice(a,0,["enter",o,t]),e.splice(i+1,0,["exit",e[n][1],t]),e[n][1].end={...e[i][1].end}):e[n][1]=o,e.push(["exit",o,t]),e}function IMe(e,t,r){const n=this;let a;return i;function i(u){let d=n.events.length,p;for(;d--;)if(n.events[d][1].type!=="lineEnding"&&n.events[d][1].type!=="linePrefix"&&n.events[d][1].type!=="content"){p=n.events[d][1].type==="paragraph";break}return!n.parser.lazy[n.now().line]&&(n.interrupt||p)?(e.enter("setextHeadingLine"),a=u,o(u)):r(u)}function o(u){return e.enter("setextHeadingLineSequence"),s(u)}function s(u){return u===a?(e.consume(u),s):(e.exit("setextHeadingLineSequence"),Tr(u)?xr(e,l,"lineSuffix")(u):l(u))}function l(u){return u===null||Ot(u)?(e.exit("setextHeadingLine"),t(u)):r(u)}}const OMe={tokenize:BMe};function BMe(e){const t=this,r=e.attempt(Mg,n,e.attempt(this.parser.constructs.flowInitial,a,xr(e,e.attempt(this.parser.constructs.flow,a,e.attempt($Le,a)),"linePrefix")));return r;function n(i){if(i===null){e.consume(i);return}return e.enter("lineEndingBlank"),e.consume(i),e.exit("lineEndingBlank"),t.currentConstruct=void 0,r}function a(i){if(i===null){e.consume(i);return}return e.enter("lineEnding"),e.consume(i),e.exit("lineEnding"),t.currentConstruct=void 0,r}}const PMe={resolveAll:Lre()},zMe=Nre("string"),HMe=Nre("text");function Nre(e){return{resolveAll:Lre(e==="text"?UMe:void 0),tokenize:t};function t(r){const n=this,a=this.parser.constructs[e],i=r.attempt(a,o,s);return o;function o(d){return u(d)?i(d):s(d)}function s(d){if(d===null){r.consume(d);return}return r.enter("data"),r.consume(d),l}function l(d){return u(d)?(r.exit("data"),i(d)):(r.consume(d),l)}function u(d){if(d===null)return!0;const p=a[d];let f=-1;if(p)for(;++f<p.length;){const m=p[f];if(!m.previous||m.previous.call(n,n.previous))return!0}return!1}}}function Lre(e){return t;function t(r,n){let a=-1,i;for(;++a<=r.length;)i===void 0?r[a]&&r[a][1].type==="data"&&(i=a,a++):(!r[a]||r[a][1].type!=="data")&&(a!==i+2&&(r[i][1].end=r[a-1][1].end,r.splice(i+2,a-i-2),a=i+2),i=void 0);return e?e(r,n):r}}function UMe(e,t){let r=0;for(;++r<=e.length;)if((r===e.length||e[r][1].type==="lineEnding")&&e[r-1][1].type==="data"){const n=e[r-1][1],a=t.sliceStream(n);let i=a.length,o=-1,s=0,l;for(;i--;){const u=a[i];if(typeof u=="string"){for(o=u.length;u.charCodeAt(o-1)===32;)s++,o--;if(o)break;o=-1}else if(u===-2)l=!0,s++;else if(u!==-1){i++;break}}if(s){const u={type:r===e.length||l||s<2?"lineSuffix":"hardBreakTrailing",start:{_bufferIndex:i?o:n.start._bufferIndex+o,_index:n.start._index+i,line:n.end.line,column:n.end.column-s,offset:n.end.offset-s},end:{...n.end}};n.end={...u.start},n.start.offset===n.end.offset?Object.assign(n,u):(e.splice(r,0,["enter",u,t],["exit",u,t]),r+=2)}r++}return e}const $Me={42:oi,43:oi,45:oi,48:oi,49:oi,50:oi,51:oi,52:oi,53:oi,54:oi,55:oi,56:oi,57:oi,62:Cre},GMe={91:VLe},jMe={[-2]:lk,[-1]:lk,32:lk},qMe={35:QLe,42:I1,45:[LP,I1],60:nMe,61:LP,95:I1,96:DP,126:DP},WMe={38:Are,92:Tre},VMe={[-5]:uk,[-4]:uk,[-3]:uk,33:EMe,38:Are,42:HR,60:[wLe,cMe],91:wMe,92:[XLe,Tre],93:j5,95:HR,96:OLe},KMe={null:[HR,PMe]},YMe={null:[42,95]},JMe={null:[]},XMe=Object.freeze(Object.defineProperty({__proto__:null,attentionMarkers:YMe,contentInitial:GMe,disable:JMe,document:$Me,flow:qMe,flowInitial:jMe,insideSpan:KMe,string:WMe,text:VMe},Symbol.toStringTag,{value:"Module"}));function ZMe(e,t,r){let n={_bufferIndex:-1,_index:0,line:r&&r.line||1,column:r&&r.column||1,offset:r&&r.offset||0};const a={},i=[];let o=[],s=[];const l={attempt:F(R),check:F(C),consume:E,enter:k,exit:_,interrupt:F(C,{interrupt:!0})},u={code:null,containerState:{},defineSkip:y,events:[],now:b,parser:e,previous:null,sliceSerialize:f,sliceStream:m,write:p};let d=t.tokenize.call(u,l);return t.resolveAll&&i.push(t),u;function p(j){return o=to(o,j),S(),o[o.length-1]!==null?[]:(L(t,0),u.events=E2(i,u.events,u),u.events)}function f(j,Y){return eIe(m(j),Y)}function m(j){return QMe(o,j)}function b(){const{_bufferIndex:j,_index:Y,line:W,column:$,offset:X}=n;return{_bufferIndex:j,_index:Y,line:W,column:$,offset:X}}function y(j){a[j.line]=j.column,M()}function S(){let j;for(;n._index<o.length;){const Y=o[n._index];if(typeof Y=="string")for(j=n._index,n._bufferIndex<0&&(n._bufferIndex=0);n._index===j&&n._bufferIndex<Y.length;)w(Y.charCodeAt(n._bufferIndex));else w(Y)}}function w(j){d=d(j)}function E(j){Ot(j)?(n.line++,n.column=1,n.offset+=j===-3?2:1,M()):j!==-1&&(n.column++,n.offset++),n._bufferIndex<0?n._index++:(n._bufferIndex++,n._bufferIndex===o[n._index].length&&(n._bufferIndex=-1,n._index++)),u.previous=j}function k(j,Y){const W=Y||{};return W.type=j,W.start=b(),u.events.push(["enter",W,u]),s.push(W),W}function _(j){const Y=s.pop();return Y.end=b(),u.events.push(["exit",Y,u]),Y}function R(j,Y){L(j,Y.from)}function C(j,Y){Y.restore()}function F(j,Y){return W;function W($,X,z){let H,B,ee,J;return Array.isArray($)?G($):"tokenize"in $?G([$]):K($);function K(pe){return ae;function ae(se){const Ee=se!==null&&pe[se],fe=se!==null&&pe.null,Me=[...Array.isArray(Ee)?Ee:Ee?[Ee]:[],...Array.isArray(fe)?fe:fe?[fe]:[]];return G(Me)(se)}}function G(pe){return H=pe,B=0,pe.length===0?z:P(pe[B])}function P(pe){return ae;function ae(se){return J=I(),ee=pe,pe.partial||(u.currentConstruct=pe),pe.name&&u.parser.constructs.disable.null.includes(pe.name)?oe():pe.tokenize.call(Y?Object.assign(Object.create(u),Y):u,l,ne,oe)(se)}}function ne(pe){return j(ee,J),X}function oe(pe){return J.restore(),++B<H.length?P(H[B]):z}}}function L(j,Y){j.resolveAll&&!i.includes(j)&&i.push(j),j.resolve&&Pi(u.events,Y,u.events.length-Y,j.resolve(u.events.slice(Y),u)),j.resolveTo&&(u.events=j.resolveTo(u.events,u))}function I(){const j=b(),Y=u.previous,W=u.currentConstruct,$=u.events.length,X=Array.from(s);return{from:$,restore:z};function z(){n=j,u.previous=Y,u.currentConstruct=W,u.events.length=$,s=X,M()}}function M(){n.line in a&&n.column<2&&(n.column=a[n.line],n.offset+=a[n.line]-1)}}function QMe(e,t){const r=t.start._index,n=t.start._bufferIndex,a=t.end._index,i=t.end._bufferIndex;let o;if(r===a)o=[e[r].slice(n,i)];else{if(o=e.slice(r,a),n>-1){const s=o[0];typeof s=="string"?o[0]=s.slice(n):o.shift()}i>0&&o.push(e[a].slice(0,i))}return o}function eIe(e,t){let r=-1;const n=[];let a;for(;++r<e.length;){const i=e[r];let o;if(typeof i=="string")o=i;else switch(i){case-5:{o="\r";break}case-4:{o=`
`;break}case-3:{o=`\r
`;break}case-2:{o=t?" ":"	";break}case-1:{if(!t&&a)continue;o=" ";break}default:o=String.fromCharCode(i)}a=i===-2,n.push(o)}return n.join("")}function tIe(e){const n={constructs:wre([XMe,...(e||{}).extensions||[]]),content:a(gLe),defined:[],document:a(yLe),flow:a(OMe),lazy:{},string:a(zMe),text:a(HMe)};return n;function a(i){return o;function o(s){return ZMe(n,i,s)}}}function rIe(e){for(;!Fre(e););return e}const MP=/[\0\t\n\r]/g;function nIe(){let e=1,t="",r=!0,n;return a;function a(i,o,s){const l=[];let u,d,p,f,m;for(i=t+(typeof i=="string"?i.toString():new TextDecoder(o||void 0).decode(i)),p=0,t="",r&&(i.charCodeAt(0)===65279&&p++,r=void 0);p<i.length;){if(MP.lastIndex=p,u=MP.exec(i),f=u&&u.index!==void 0?u.index:i.length,m=i.charCodeAt(f),!u){t=i.slice(p);break}if(m===10&&p===f&&n)l.push(-3),n=void 0;else switch(n&&(l.push(-5),n=void 0),p<f&&(l.push(i.slice(p,f)),e+=f-p),m){case 0:{l.push(65533),e++;break}case 9:{for(d=Math.ceil(e/4)*4,l.push(-2);e++<d;)l.push(-1);break}case 10:{l.push(-4),e=1;break}default:n=!0,e=1}p=f+1}return s&&(n&&l.push(-5),t&&l.push(t),l.push(null)),l}}const aIe=/\\([!-/:-@[-`{-~])|&(#(?:\d{1,7}|x[\da-f]{1,6})|[\da-z]{1,31});/gi;function iIe(e){return e.replace(aIe,oIe)}function oIe(e,t,r){if(t)return t;if(r.charCodeAt(0)===35){const a=r.charCodeAt(1),i=a===120||a===88;return kre(r.slice(i?2:1),i?16:10)}return G5(r)||e}const Mre={}.hasOwnProperty;function sIe(e,t,r){return typeof t!="string"&&(r=t,t=void 0),lIe(r)(rIe(tIe(r).document().write(nIe()(e,t,!0))))}function lIe(e){const t={transforms:[],canContainEols:["emphasis","fragment","heading","paragraph","strong"],enter:{autolink:i(qe),autolinkProtocol:I,autolinkEmail:I,atxHeading:i(Ne),blockQuote:i(fe),characterEscape:I,characterReference:I,codeFenced:i(Me),codeFencedFenceInfo:o,codeFencedFenceMeta:o,codeIndented:i(Me,o),codeText:i(Oe,o),codeTextData:I,data:I,codeFlowValue:I,definition:i(xe),definitionDestinationString:o,definitionLabelString:o,definitionTitleString:o,emphasis:i(Te),hardBreakEscape:i(he),hardBreakTrailing:i(he),htmlFlow:i(Ze,o),htmlFlowData:I,htmlText:i(Ze,o),htmlTextData:I,image:i(Et),label:o,link:i(qe),listItem:i(we),listItemValue:f,listOrdered:i(Ye,p),listUnordered:i(Ye),paragraph:i(Qt),reference:P,referenceString:o,resourceDestinationString:o,resourceTitleString:o,setextHeading:i(Ne),strong:i(or),thematicBreak:i(rt)},exit:{atxHeading:l(),atxHeadingSequence:R,autolink:l(),autolinkEmail:Ee,autolinkProtocol:se,blockQuote:l(),characterEscapeValue:M,characterReferenceMarkerHexadecimal:oe,characterReferenceMarkerNumeric:oe,characterReferenceValue:pe,characterReference:ae,codeFenced:l(S),codeFencedFence:y,codeFencedFenceInfo:m,codeFencedFenceMeta:b,codeFlowValue:M,codeIndented:l(w),codeText:l(X),codeTextData:M,data:M,definition:l(),definitionDestinationString:_,definitionLabelString:E,definitionTitleString:k,emphasis:l(),hardBreakEscape:l(Y),hardBreakTrailing:l(Y),htmlFlow:l(W),htmlFlowData:M,htmlText:l($),htmlTextData:M,image:l(H),label:ee,labelText:B,lineEnding:j,link:l(z),listItem:l(),listOrdered:l(),listUnordered:l(),paragraph:l(),referenceString:ne,resourceDestinationString:J,resourceTitleString:K,resource:G,setextHeading:l(L),setextHeadingLineSequence:F,setextHeadingText:C,strong:l(),thematicBreak:l()}};Ire(t,(e||{}).mdastExtensions||[]);const r={};return n;function n(ge){let Re={type:"root",children:[]};const We={stack:[Re],tokenStack:[],config:t,enter:s,exit:u,buffer:o,resume:d,data:r},ht=[];let xt=-1;for(;++xt<ge.length;)if(ge[xt][1].type==="listOrdered"||ge[xt][1].type==="listUnordered")if(ge[xt][0]==="enter")ht.push(xt);else{const sr=ht.pop();xt=a(ge,sr,xt)}for(xt=-1;++xt<ge.length;){const sr=t[ge[xt][0]];Mre.call(sr,ge[xt][1].type)&&sr[ge[xt][1].type].call(Object.assign({sliceSerialize:ge[xt][2].sliceSerialize},We),ge[xt][1])}if(We.tokenStack.length>0){const sr=We.tokenStack[We.tokenStack.length-1];(sr[1]||IP).call(We,void 0,sr[0])}for(Re.position={start:uu(ge.length>0?ge[0][1].start:{line:1,column:1,offset:0}),end:uu(ge.length>0?ge[ge.length-2][1].end:{line:1,column:1,offset:0})},xt=-1;++xt<t.transforms.length;)Re=t.transforms[xt](Re)||Re;return Re}function a(ge,Re,We){let ht=Re-1,xt=-1,sr=!1,ar,fr,Mt,lr;for(;++ht<=We;){const qt=ge[ht];switch(qt[1].type){case"listUnordered":case"listOrdered":case"blockQuote":{qt[0]==="enter"?xt++:xt--,lr=void 0;break}case"lineEndingBlank":{qt[0]==="enter"&&(ar&&!lr&&!xt&&!Mt&&(Mt=ht),lr=void 0);break}case"linePrefix":case"listItemValue":case"listItemMarker":case"listItemPrefix":case"listItemPrefixWhitespace":break;default:lr=void 0}if(!xt&&qt[0]==="enter"&&qt[1].type==="listItemPrefix"||xt===-1&&qt[0]==="exit"&&(qt[1].type==="listUnordered"||qt[1].type==="listOrdered")){if(ar){let zr=ht;for(fr=void 0;zr--;){const nn=ge[zr];if(nn[1].type==="lineEnding"||nn[1].type==="lineEndingBlank"){if(nn[0]==="exit")continue;fr&&(ge[fr][1].type="lineEndingBlank",sr=!0),nn[1].type="lineEnding",fr=zr}else if(!(nn[1].type==="linePrefix"||nn[1].type==="blockQuotePrefix"||nn[1].type==="blockQuotePrefixWhitespace"||nn[1].type==="blockQuoteMarker"||nn[1].type==="listItemIndent"))break}Mt&&(!fr||Mt<fr)&&(ar._spread=!0),ar.end=Object.assign({},fr?ge[fr][1].start:qt[1].end),ge.splice(fr||ht,0,["exit",ar,qt[2]]),ht++,We++}if(qt[1].type==="listItemPrefix"){const zr={type:"listItem",_spread:!1,start:Object.assign({},qt[1].start),end:void 0};ar=zr,ge.splice(ht,0,["enter",zr,qt[2]]),ht++,We++,Mt=void 0,lr=!0}}}return ge[Re][1]._spread=sr,We}function i(ge,Re){return We;function We(ht){s.call(this,ge(ht),ht),Re&&Re.call(this,ht)}}function o(){this.stack.push({type:"fragment",children:[]})}function s(ge,Re,We){this.stack[this.stack.length-1].children.push(ge),this.stack.push(ge),this.tokenStack.push([Re,We||void 0]),ge.position={start:uu(Re.start),end:void 0}}function l(ge){return Re;function Re(We){ge&&ge.call(this,We),u.call(this,We)}}function u(ge,Re){const We=this.stack.pop(),ht=this.tokenStack.pop();if(ht)ht[0].type!==ge.type&&(Re?Re.call(this,ge,ht[0]):(ht[1]||IP).call(this,ge,ht[0]));else throw new Error("Cannot close `"+ge.type+"` ("+vm({start:ge.start,end:ge.end})+"): it’s not open");We.position.end=uu(ge.end)}function d(){return $5(this.stack.pop())}function p(){this.data.expectingFirstListItemValue=!0}function f(ge){if(this.data.expectingFirstListItemValue){const Re=this.stack[this.stack.length-2];Re.start=Number.parseInt(this.sliceSerialize(ge),10),this.data.expectingFirstListItemValue=void 0}}function m(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.lang=ge}function b(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.meta=ge}function y(){this.data.flowCodeInside||(this.buffer(),this.data.flowCodeInside=!0)}function S(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.value=ge.replace(/^(\r?\n|\r)|(\r?\n|\r)$/g,""),this.data.flowCodeInside=void 0}function w(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.value=ge.replace(/(\r?\n|\r)$/g,"")}function E(ge){const Re=this.resume(),We=this.stack[this.stack.length-1];We.label=Re,We.identifier=Ro(this.sliceSerialize(ge)).toLowerCase()}function k(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.title=ge}function _(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.url=ge}function R(ge){const Re=this.stack[this.stack.length-1];if(!Re.depth){const We=this.sliceSerialize(ge).length;Re.depth=We}}function C(){this.data.setextHeadingSlurpLineEnding=!0}function F(ge){const Re=this.stack[this.stack.length-1];Re.depth=this.sliceSerialize(ge).codePointAt(0)===61?1:2}function L(){this.data.setextHeadingSlurpLineEnding=void 0}function I(ge){const We=this.stack[this.stack.length-1].children;let ht=We[We.length-1];(!ht||ht.type!=="text")&&(ht=Pe(),ht.position={start:uu(ge.start),end:void 0},We.push(ht)),this.stack.push(ht)}function M(ge){const Re=this.stack.pop();Re.value+=this.sliceSerialize(ge),Re.position.end=uu(ge.end)}function j(ge){const Re=this.stack[this.stack.length-1];if(this.data.atHardBreak){const We=Re.children[Re.children.length-1];We.position.end=uu(ge.end),this.data.atHardBreak=void 0;return}!this.data.setextHeadingSlurpLineEnding&&t.canContainEols.includes(Re.type)&&(I.call(this,ge),M.call(this,ge))}function Y(){this.data.atHardBreak=!0}function W(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.value=ge}function $(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.value=ge}function X(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.value=ge}function z(){const ge=this.stack[this.stack.length-1];if(this.data.inReference){const Re=this.data.referenceType||"shortcut";ge.type+="Reference",ge.referenceType=Re,delete ge.url,delete ge.title}else delete ge.identifier,delete ge.label;this.data.referenceType=void 0}function H(){const ge=this.stack[this.stack.length-1];if(this.data.inReference){const Re=this.data.referenceType||"shortcut";ge.type+="Reference",ge.referenceType=Re,delete ge.url,delete ge.title}else delete ge.identifier,delete ge.label;this.data.referenceType=void 0}function B(ge){const Re=this.sliceSerialize(ge),We=this.stack[this.stack.length-2];We.label=iIe(Re),We.identifier=Ro(Re).toLowerCase()}function ee(){const ge=this.stack[this.stack.length-1],Re=this.resume(),We=this.stack[this.stack.length-1];if(this.data.inReference=!0,We.type==="link"){const ht=ge.children;We.children=ht}else We.alt=Re}function J(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.url=ge}function K(){const ge=this.resume(),Re=this.stack[this.stack.length-1];Re.title=ge}function G(){this.data.inReference=void 0}function P(){this.data.referenceType="collapsed"}function ne(ge){const Re=this.resume(),We=this.stack[this.stack.length-1];We.label=Re,We.identifier=Ro(this.sliceSerialize(ge)).toLowerCase(),this.data.referenceType="full"}function oe(ge){this.data.characterReferenceType=ge.type}function pe(ge){const Re=this.sliceSerialize(ge),We=this.data.characterReferenceType;let ht;We?(ht=kre(Re,We==="characterReferenceMarkerNumeric"?10:16),this.data.characterReferenceType=void 0):ht=G5(Re);const xt=this.stack[this.stack.length-1];xt.value+=ht}function ae(ge){const Re=this.stack.pop();Re.position.end=uu(ge.end)}function se(ge){M.call(this,ge);const Re=this.stack[this.stack.length-1];Re.url=this.sliceSerialize(ge)}function Ee(ge){M.call(this,ge);const Re=this.stack[this.stack.length-1];Re.url="mailto:"+this.sliceSerialize(ge)}function fe(){return{type:"blockquote",children:[]}}function Me(){return{type:"code",lang:null,meta:null,value:""}}function Oe(){return{type:"inlineCode",value:""}}function xe(){return{type:"definition",identifier:"",label:null,title:null,url:""}}function Te(){return{type:"emphasis",children:[]}}function Ne(){return{type:"heading",depth:0,children:[]}}function he(){return{type:"break"}}function Ze(){return{type:"html",value:""}}function Et(){return{type:"image",title:null,url:"",alt:null}}function qe(){return{type:"link",title:null,url:"",children:[]}}function Ye(ge){return{type:"list",ordered:ge.type==="listOrdered",start:null,spread:ge._spread,children:[]}}function we(ge){return{type:"listItem",spread:ge._spread,checked:null,children:[]}}function Qt(){return{type:"paragraph",children:[]}}function or(){return{type:"strong",children:[]}}function Pe(){return{type:"text",value:""}}function rt(){return{type:"thematicBreak"}}}function uu(e){return{line:e.line,column:e.column,offset:e.offset}}function Ire(e,t){let r=-1;for(;++r<t.length;){const n=t[r];Array.isArray(n)?Ire(e,n):uIe(e,n)}}function uIe(e,t){let r;for(r in t)if(Mre.call(t,r))switch(r){case"canContainEols":{const n=t[r];n&&e[r].push(...n);break}case"transforms":{const n=t[r];n&&e[r].push(...n);break}case"enter":case"exit":{const n=t[r];n&&Object.assign(e[r],n);break}}}function IP(e,t){throw e?new Error("Cannot close `"+e.type+"` ("+vm({start:e.start,end:e.end})+"): a different token (`"+t.type+"`, "+vm({start:t.start,end:t.end})+") is open"):new Error("Cannot close document, a token (`"+t.type+"`, "+vm({start:t.start,end:t.end})+") is still open")}function cIe(e){const t=this;t.parser=r;function r(n){return sIe(n,{...t.data("settings"),...e,extensions:t.data("micromarkExtensions")||[],mdastExtensions:t.data("fromMarkdownExtensions")||[]})}}function dIe(e,t){const r={type:"element",tagName:"blockquote",properties:{},children:e.wrap(e.all(t),!0)};return e.patch(t,r),e.applyData(t,r)}function hIe(e,t){const r={type:"element",tagName:"br",properties:{},children:[]};return e.patch(t,r),[e.applyData(t,r),{type:"text",value:`
`}]}function pIe(e,t){const r=t.value?t.value+`
`:"",n={};t.lang&&(n.className=["language-"+t.lang]);let a={type:"element",tagName:"code",properties:n,children:[{type:"text",value:r}]};return t.meta&&(a.data={meta:t.meta}),e.patch(t,a),a=e.applyData(t,a),a={type:"element",tagName:"pre",properties:{},children:[a]},e.patch(t,a),a}function fIe(e,t){const r={type:"element",tagName:"del",properties:{},children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}function mIe(e,t){const r={type:"element",tagName:"em",properties:{},children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}function gIe(e,t){const r=typeof e.options.clobberPrefix=="string"?e.options.clobberPrefix:"user-content-",n=String(t.identifier).toUpperCase(),a=kp(n.toLowerCase()),i=e.footnoteOrder.indexOf(n);let o,s=e.footnoteCounts.get(n);s===void 0?(s=0,e.footnoteOrder.push(n),o=e.footnoteOrder.length):o=i+1,s+=1,e.footnoteCounts.set(n,s);const l={type:"element",tagName:"a",properties:{href:"#"+r+"fn-"+a,id:r+"fnref-"+a+(s>1?"-"+s:""),dataFootnoteRef:!0,ariaDescribedBy:["footnote-label"]},children:[{type:"text",value:String(o)}]};e.patch(t,l);const u={type:"element",tagName:"sup",properties:{},children:[l]};return e.patch(t,u),e.applyData(t,u)}function bIe(e,t){const r={type:"element",tagName:"h"+t.depth,properties:{},children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}function yIe(e,t){if(e.options.allowDangerousHtml){const r={type:"raw",value:t.value};return e.patch(t,r),e.applyData(t,r)}}function Ore(e,t){const r=t.referenceType;let n="]";if(r==="collapsed"?n+="[]":r==="full"&&(n+="["+(t.label||t.identifier)+"]"),t.type==="imageReference")return[{type:"text",value:"!["+t.alt+n}];const a=e.all(t),i=a[0];i&&i.type==="text"?i.value="["+i.value:a.unshift({type:"text",value:"["});const o=a[a.length-1];return o&&o.type==="text"?o.value+=n:a.push({type:"text",value:n}),a}function vIe(e,t){const r=String(t.identifier).toUpperCase(),n=e.definitionById.get(r);if(!n)return Ore(e,t);const a={src:kp(n.url||""),alt:t.alt};n.title!==null&&n.title!==void 0&&(a.title=n.title);const i={type:"element",tagName:"img",properties:a,children:[]};return e.patch(t,i),e.applyData(t,i)}function SIe(e,t){const r={src:kp(t.url)};t.alt!==null&&t.alt!==void 0&&(r.alt=t.alt),t.title!==null&&t.title!==void 0&&(r.title=t.title);const n={type:"element",tagName:"img",properties:r,children:[]};return e.patch(t,n),e.applyData(t,n)}function EIe(e,t){const r={type:"text",value:t.value.replace(/\r?\n|\r/g," ")};e.patch(t,r);const n={type:"element",tagName:"code",properties:{},children:[r]};return e.patch(t,n),e.applyData(t,n)}function xIe(e,t){const r=String(t.identifier).toUpperCase(),n=e.definitionById.get(r);if(!n)return Ore(e,t);const a={href:kp(n.url||"")};n.title!==null&&n.title!==void 0&&(a.title=n.title);const i={type:"element",tagName:"a",properties:a,children:e.all(t)};return e.patch(t,i),e.applyData(t,i)}function wIe(e,t){const r={href:kp(t.url)};t.title!==null&&t.title!==void 0&&(r.title=t.title);const n={type:"element",tagName:"a",properties:r,children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}function kIe(e,t,r){const n=e.all(t),a=r?CIe(r):Bre(t),i={},o=[];if(typeof t.checked=="boolean"){const d=n[0];let p;d&&d.type==="element"&&d.tagName==="p"?p=d:(p={type:"element",tagName:"p",properties:{},children:[]},n.unshift(p)),p.children.length>0&&p.children.unshift({type:"text",value:" "}),p.children.unshift({type:"element",tagName:"input",properties:{type:"checkbox",checked:t.checked,disabled:!0},children:[]}),i.className=["task-list-item"]}let s=-1;for(;++s<n.length;){const d=n[s];(a||s!==0||d.type!=="element"||d.tagName!=="p")&&o.push({type:"text",value:`
`}),d.type==="element"&&d.tagName==="p"&&!a?o.push(...d.children):o.push(d)}const l=n[n.length-1];l&&(a||l.type!=="element"||l.tagName!=="p")&&o.push({type:"text",value:`
`});const u={type:"element",tagName:"li",properties:i,children:o};return e.patch(t,u),e.applyData(t,u)}function CIe(e){let t=!1;if(e.type==="list"){t=e.spread||!1;const r=e.children;let n=-1;for(;!t&&++n<r.length;)t=Bre(r[n])}return t}function Bre(e){const t=e.spread;return t??e.children.length>1}function TIe(e,t){const r={},n=e.all(t);let a=-1;for(typeof t.start=="number"&&t.start!==1&&(r.start=t.start);++a<n.length;){const o=n[a];if(o.type==="element"&&o.tagName==="li"&&o.properties&&Array.isArray(o.properties.className)&&o.properties.className.includes("task-list-item")){r.className=["contains-task-list"];break}}const i={type:"element",tagName:t.ordered?"ol":"ul",properties:r,children:e.wrap(n,!0)};return e.patch(t,i),e.applyData(t,i)}function AIe(e,t){const r={type:"element",tagName:"p",properties:{},children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}function FIe(e,t){const r={type:"root",children:e.wrap(e.all(t))};return e.patch(t,r),e.applyData(t,r)}function _Ie(e,t){const r={type:"element",tagName:"strong",properties:{},children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}function RIe(e,t){const r=e.all(t),n=r.shift(),a=[];if(n){const o={type:"element",tagName:"thead",properties:{},children:e.wrap([n],!0)};e.patch(t.children[0],o),a.push(o)}if(r.length>0){const o={type:"element",tagName:"tbody",properties:{},children:e.wrap(r,!0)},s=vs(t.children[1]),l=v2(t.children[t.children.length-1]);s&&l&&(o.position={start:s,end:l}),a.push(o)}const i={type:"element",tagName:"table",properties:{},children:e.wrap(a,!0)};return e.patch(t,i),e.applyData(t,i)}function DIe(e,t,r){const n=r?r.children:void 0,i=(n?n.indexOf(t):1)===0?"th":"td",o=r&&r.type==="table"?r.align:void 0,s=o?o.length:t.children.length;let l=-1;const u=[];for(;++l<s;){const p=t.children[l],f={},m=o?o[l]:void 0;m&&(f.align=m);let b={type:"element",tagName:i,properties:f,children:[]};p&&(b.children=e.all(p),e.patch(p,b),b=e.applyData(p,b)),u.push(b)}const d={type:"element",tagName:"tr",properties:{},children:e.wrap(u,!0)};return e.patch(t,d),e.applyData(t,d)}function NIe(e,t){const r={type:"element",tagName:"td",properties:{},children:e.all(t)};return e.patch(t,r),e.applyData(t,r)}const OP=9,BP=32;function LIe(e){const t=String(e),r=/\r?\n|\r/g;let n=r.exec(t),a=0;const i=[];for(;n;)i.push(PP(t.slice(a,n.index),a>0,!0),n[0]),a=n.index+n[0].length,n=r.exec(t);return i.push(PP(t.slice(a),a>0,!1)),i.join("")}function PP(e,t,r){let n=0,a=e.length;if(t){let i=e.codePointAt(n);for(;i===OP||i===BP;)n++,i=e.codePointAt(n)}if(r){let i=e.codePointAt(a-1);for(;i===OP||i===BP;)a--,i=e.codePointAt(a-1)}return a>n?e.slice(n,a):""}function MIe(e,t){const r={type:"text",value:LIe(String(t.value))};return e.patch(t,r),e.applyData(t,r)}function IIe(e,t){const r={type:"element",tagName:"hr",properties:{},children:[]};return e.patch(t,r),e.applyData(t,r)}const OIe={blockquote:dIe,break:hIe,code:pIe,delete:fIe,emphasis:mIe,footnoteReference:gIe,heading:bIe,html:yIe,imageReference:vIe,image:SIe,inlineCode:EIe,linkReference:xIe,link:wIe,listItem:kIe,list:TIe,paragraph:AIe,root:FIe,strong:_Ie,table:RIe,tableCell:NIe,tableRow:DIe,text:MIe,thematicBreak:IIe,toml:Yb,yaml:Yb,definition:Yb,footnoteDefinition:Yb};function Yb(){}const Pre=-1,x2=0,Em=1,Ry=2,q5=3,W5=4,V5=5,K5=6,zre=7,Hre=8,zP=typeof self=="object"?self:globalThis,BIe=(e,t)=>{const r=(a,i)=>(e.set(i,a),a),n=a=>{if(e.has(a))return e.get(a);const[i,o]=t[a];switch(i){case x2:case Pre:return r(o,a);case Em:{const s=r([],a);for(const l of o)s.push(n(l));return s}case Ry:{const s=r({},a);for(const[l,u]of o)s[n(l)]=n(u);return s}case q5:return r(new Date(o),a);case W5:{const{source:s,flags:l}=o;return r(new RegExp(s,l),a)}case V5:{const s=r(new Map,a);for(const[l,u]of o)s.set(n(l),n(u));return s}case K5:{const s=r(new Set,a);for(const l of o)s.add(n(l));return s}case zre:{const{name:s,message:l}=o;return r(new zP[s](l),a)}case Hre:return r(BigInt(o),a);case"BigInt":return r(Object(BigInt(o)),a);case"ArrayBuffer":return r(new Uint8Array(o).buffer,o);case"DataView":{const{buffer:s}=new Uint8Array(o);return r(new DataView(s),o)}}return r(new zP[i](o),a)};return n},HP=e=>BIe(new Map,e)(0),ah="",{toString:PIe}={},{keys:zIe}=Object,Hf=e=>{const t=typeof e;if(t!=="object"||!e)return[x2,t];const r=PIe.call(e).slice(8,-1);switch(r){case"Array":return[Em,ah];case"Object":return[Ry,ah];case"Date":return[q5,ah];case"RegExp":return[W5,ah];case"Map":return[V5,ah];case"Set":return[K5,ah];case"DataView":return[Em,r]}return r.includes("Array")?[Em,r]:r.includes("Error")?[zre,r]:[Ry,r]},Jb=([e,t])=>e===x2&&(t==="function"||t==="symbol"),HIe=(e,t,r,n)=>{const a=(o,s)=>{const l=n.push(o)-1;return r.set(s,l),l},i=o=>{if(r.has(o))return r.get(o);let[s,l]=Hf(o);switch(s){case x2:{let d=o;switch(l){case"bigint":s=Hre,d=o.toString();break;case"function":case"symbol":if(e)throw new TypeError("unable to serialize "+l);d=null;break;case"undefined":return a([Pre],o)}return a([s,d],o)}case Em:{if(l){let f=o;return l==="DataView"?f=new Uint8Array(o.buffer):l==="ArrayBuffer"&&(f=new Uint8Array(o)),a([l,[...f]],o)}const d=[],p=a([s,d],o);for(const f of o)d.push(i(f));return p}case Ry:{if(l)switch(l){case"BigInt":return a([l,o.toString()],o);case"Boolean":case"Number":case"String":return a([l,o.valueOf()],o)}if(t&&"toJSON"in o)return i(o.toJSON());const d=[],p=a([s,d],o);for(const f of zIe(o))(e||!Jb(Hf(o[f])))&&d.push([i(f),i(o[f])]);return p}case q5:return a([s,o.toISOString()],o);case W5:{const{source:d,flags:p}=o;return a([s,{source:d,flags:p}],o)}case V5:{const d=[],p=a([s,d],o);for(const[f,m]of o)(e||!(Jb(Hf(f))||Jb(Hf(m))))&&d.push([i(f),i(m)]);return p}case K5:{const d=[],p=a([s,d],o);for(const f of o)(e||!Jb(Hf(f)))&&d.push(i(f));return p}}const{message:u}=o;return a([s,{name:l,message:u}],o)};return i},UP=(e,{json:t,lossy:r}={})=>{const n=[];return HIe(!(t||r),!!t,new Map,n)(e),n},Yh=typeof structuredClone=="function"?(e,t)=>t&&("json"in t||"lossy"in t)?HP(UP(e,t)):structuredClone(e):(e,t)=>HP(UP(e,t));function UIe(e,t){const r=[{type:"text",value:"↩"}];return t>1&&r.push({type:"element",tagName:"sup",properties:{},children:[{type:"text",value:String(t)}]}),r}function $Ie(e,t){return"Back to reference "+(e+1)+(t>1?"-"+t:"")}function GIe(e){const t=typeof e.options.clobberPrefix=="string"?e.options.clobberPrefix:"user-content-",r=e.options.footnoteBackContent||UIe,n=e.options.footnoteBackLabel||$Ie,a=e.options.footnoteLabel||"Footnotes",i=e.options.footnoteLabelTagName||"h2",o=e.options.footnoteLabelProperties||{className:["sr-only"]},s=[];let l=-1;for(;++l<e.footnoteOrder.length;){const u=e.footnoteById.get(e.footnoteOrder[l]);if(!u)continue;const d=e.all(u),p=String(u.identifier).toUpperCase(),f=kp(p.toLowerCase());let m=0;const b=[],y=e.footnoteCounts.get(p);for(;y!==void 0&&++m<=y;){b.length>0&&b.push({type:"text",value:" "});let E=typeof r=="string"?r:r(l,m);typeof E=="string"&&(E={type:"text",value:E}),b.push({type:"element",tagName:"a",properties:{href:"#"+t+"fnref-"+f+(m>1?"-"+m:""),dataFootnoteBackref:"",ariaLabel:typeof n=="string"?n:n(l,m),className:["data-footnote-backref"]},children:Array.isArray(E)?E:[E]})}const S=d[d.length-1];if(S&&S.type==="element"&&S.tagName==="p"){const E=S.children[S.children.length-1];E&&E.type==="text"?E.value+=" ":S.children.push({type:"text",value:" "}),S.children.push(...b)}else d.push(...b);const w={type:"element",tagName:"li",properties:{id:t+"fn-"+f},children:e.wrap(d,!0)};e.patch(u,w),s.push(w)}if(s.length!==0)return{type:"element",tagName:"section",properties:{dataFootnotes:!0,className:["footnotes"]},children:[{type:"element",tagName:i,properties:{...Yh(o),id:"footnote-label"},children:[{type:"text",value:a}]},{type:"text",value:`
`},{type:"element",tagName:"ol",properties:{},children:e.wrap(s,!0)},{type:"text",value:`
`}]}}const w2=(function(e){if(e==null)return VIe;if(typeof e=="function")return k2(e);if(typeof e=="object")return Array.isArray(e)?jIe(e):qIe(e);if(typeof e=="string")return WIe(e);throw new Error("Expected function, string, or object as test")});function jIe(e){const t=[];let r=-1;for(;++r<e.length;)t[r]=w2(e[r]);return k2(n);function n(...a){let i=-1;for(;++i<t.length;)if(t[i].apply(this,a))return!0;return!1}}function qIe(e){const t=e;return k2(r);function r(n){const a=n;let i;for(i in e)if(a[i]!==t[i])return!1;return!0}}function WIe(e){return k2(t);function t(r){return r&&r.type===e}}function k2(e){return t;function t(r,n,a){return!!(KIe(r)&&e.call(this,r,typeof n=="number"?n:void 0,a||void 0))}}function VIe(){return!0}function KIe(e){return e!==null&&typeof e=="object"&&"type"in e}const Ure=[],YIe=!0,UR=!1,JIe="skip";function $re(e,t,r,n){let a;typeof t=="function"&&typeof r!="function"?(n=r,r=t):a=t;const i=w2(a),o=n?-1:1;s(e,void 0,[])();function s(l,u,d){const p=l&&typeof l=="object"?l:{};if(typeof p.type=="string"){const m=typeof p.tagName=="string"?p.tagName:typeof p.name=="string"?p.name:void 0;Object.defineProperty(f,"name",{value:"node ("+(l.type+(m?"<"+m+">":""))+")"})}return f;function f(){let m=Ure,b,y,S;if((!t||i(l,u,d[d.length-1]||void 0))&&(m=XIe(r(l,d)),m[0]===UR))return m;if("children"in l&&l.children){const w=l;if(w.children&&m[0]!==JIe)for(y=(n?w.children.length:-1)+o,S=d.concat(w);y>-1&&y<w.children.length;){const E=w.children[y];if(b=s(E,y,S)(),b[0]===UR)return b;y=typeof b[1]=="number"?b[1]:y+o}}return m}}}function XIe(e){return Array.isArray(e)?e:typeof e=="number"?[YIe,e]:e==null?Ure:[e]}function Ig(e,t,r,n){let a,i,o;typeof t=="function"&&typeof r!="function"?(i=void 0,o=t,a=r):(i=t,o=r,a=n),$re(e,i,s,a);function s(l,u){const d=u[u.length-1],p=d?d.children.indexOf(l):void 0;return o(l,p,d)}}const $R={}.hasOwnProperty,ZIe={};function QIe(e,t){const r=t||ZIe,n=new Map,a=new Map,i=new Map,o={...OIe,...r.handlers},s={all:u,applyData:tOe,definitionById:n,footnoteById:a,footnoteCounts:i,footnoteOrder:[],handlers:o,one:l,options:r,patch:eOe,wrap:nOe};return Ig(e,function(d){if(d.type==="definition"||d.type==="footnoteDefinition"){const p=d.type==="definition"?n:a,f=String(d.identifier).toUpperCase();p.has(f)||p.set(f,d)}}),s;function l(d,p){const f=d.type,m=s.handlers[f];if($R.call(s.handlers,f)&&m)return m(s,d,p);if(s.options.passThrough&&s.options.passThrough.includes(f)){if("children"in d){const{children:y,...S}=d,w=Yh(S);return w.children=s.all(d),w}return Yh(d)}return(s.options.unknownHandler||rOe)(s,d,p)}function u(d){const p=[];if("children"in d){const f=d.children;let m=-1;for(;++m<f.length;){const b=s.one(f[m],d);if(b){if(m&&f[m-1].type==="break"&&(!Array.isArray(b)&&b.type==="text"&&(b.value=$P(b.value)),!Array.isArray(b)&&b.type==="element")){const y=b.children[0];y&&y.type==="text"&&(y.value=$P(y.value))}Array.isArray(b)?p.push(...b):p.push(b)}}}return p}}function eOe(e,t){e.position&&(t.position=UNe(e))}function tOe(e,t){let r=t;if(e&&e.data){const n=e.data.hName,a=e.data.hChildren,i=e.data.hProperties;if(typeof n=="string")if(r.type==="element")r.tagName=n;else{const o="children"in r?r.children:[r];r={type:"element",tagName:n,properties:{},children:o}}r.type==="element"&&i&&Object.assign(r.properties,Yh(i)),"children"in r&&r.children&&a!==null&&a!==void 0&&(r.children=a)}return r}function rOe(e,t){const r=t.data||{},n="value"in t&&!($R.call(r,"hProperties")||$R.call(r,"hChildren"))?{type:"text",value:t.value}:{type:"element",tagName:"div",properties:{},children:e.all(t)};return e.patch(t,n),e.applyData(t,n)}function nOe(e,t){const r=[];let n=-1;for(t&&r.push({type:"text",value:`
`});++n<e.length;)n&&r.push({type:"text",value:`
`}),r.push(e[n]);return t&&e.length>0&&r.push({type:"text",value:`
`}),r}function $P(e){let t=0,r=e.charCodeAt(t);for(;r===9||r===32;)t++,r=e.charCodeAt(t);return e.slice(t)}function GP(e,t){const r=QIe(e,t),n=r.one(e,void 0),a=GIe(r),i=Array.isArray(n)?{type:"root",children:n}:n||{type:"root",children:[]};return a&&i.children.push({type:"text",value:`
`},a),i}function aOe(e,t){return e&&"run"in e?async function(r,n){const a=GP(r,{file:n,...t});await e.run(a,n)}:function(r,n){return GP(r,{file:n,...e||t})}}function jP(e){if(e)throw e}var ck,qP;function iOe(){if(qP)return ck;qP=1;var e=Object.prototype.hasOwnProperty,t=Object.prototype.toString,r=Object.defineProperty,n=Object.getOwnPropertyDescriptor,a=function(u){return typeof Array.isArray=="function"?Array.isArray(u):t.call(u)==="[object Array]"},i=function(u){if(!u||t.call(u)!=="[object Object]")return!1;var d=e.call(u,"constructor"),p=u.constructor&&u.constructor.prototype&&e.call(u.constructor.prototype,"isPrototypeOf");if(u.constructor&&!d&&!p)return!1;var f;for(f in u);return typeof f>"u"||e.call(u,f)},o=function(u,d){r&&d.name==="__proto__"?r(u,d.name,{enumerable:!0,configurable:!0,value:d.newValue,writable:!0}):u[d.name]=d.newValue},s=function(u,d){if(d==="__proto__")if(e.call(u,d)){if(n)return n(u,d).value}else return;return u[d]};return ck=function l(){var u,d,p,f,m,b,y=arguments[0],S=1,w=arguments.length,E=!1;for(typeof y=="boolean"&&(E=y,y=arguments[1]||{},S=2),(y==null||typeof y!="object"&&typeof y!="function")&&(y={});S<w;++S)if(u=arguments[S],u!=null)for(d in u)p=s(y,d),f=s(u,d),y!==f&&(E&&f&&(i(f)||(m=a(f)))?(m?(m=!1,b=p&&a(p)?p:[]):b=p&&i(p)?p:{},o(y,{name:d,newValue:l(E,b,f)})):typeof f<"u"&&o(y,{name:d,newValue:f}));return y},ck}var oOe=iOe();const dk=jn(oOe);function GR(e){if(typeof e!="object"||e===null)return!1;const t=Object.getPrototypeOf(e);return(t===null||t===Object.prototype||Object.getPrototypeOf(t)===null)&&!(Symbol.toStringTag in e)&&!(Symbol.iterator in e)}function sOe(){const e=[],t={run:r,use:n};return t;function r(...a){let i=-1;const o=a.pop();if(typeof o!="function")throw new TypeError("Expected function as last argument, not "+o);s(null,...a);function s(l,...u){const d=e[++i];let p=-1;if(l){o(l);return}for(;++p<a.length;)(u[p]===null||u[p]===void 0)&&(u[p]=a[p]);a=u,d?lOe(d,s)(...u):o(null,...u)}}function n(a){if(typeof a!="function")throw new TypeError("Expected `middelware` to be a function, not "+a);return e.push(a),t}}function lOe(e,t){let r;return n;function n(...o){const s=e.length>o.length;let l;s&&o.push(a);try{l=e.apply(this,o)}catch(u){const d=u;if(s&&r)throw d;return a(d)}s||(l&&l.then&&typeof l.then=="function"?l.then(i,a):l instanceof Error?a(l):i(l))}function a(o,...s){r||(r=!0,t(o,...s))}function i(o){a(null,o)}}const Zo={basename:uOe,dirname:cOe,extname:dOe,join:hOe,sep:"/"};function uOe(e,t){if(t!==void 0&&typeof t!="string")throw new TypeError('"ext" argument must be a string');Og(e);let r=0,n=-1,a=e.length,i;if(t===void 0||t.length===0||t.length>e.length){for(;a--;)if(e.codePointAt(a)===47){if(i){r=a+1;break}}else n<0&&(i=!0,n=a+1);return n<0?"":e.slice(r,n)}if(t===e)return"";let o=-1,s=t.length-1;for(;a--;)if(e.codePointAt(a)===47){if(i){r=a+1;break}}else o<0&&(i=!0,o=a+1),s>-1&&(e.codePointAt(a)===t.codePointAt(s--)?s<0&&(n=a):(s=-1,n=o));return r===n?n=o:n<0&&(n=e.length),e.slice(r,n)}function cOe(e){if(Og(e),e.length===0)return".";let t=-1,r=e.length,n;for(;--r;)if(e.codePointAt(r)===47){if(n){t=r;break}}else n||(n=!0);return t<0?e.codePointAt(0)===47?"/":".":t===1&&e.codePointAt(0)===47?"//":e.slice(0,t)}function dOe(e){Og(e);let t=e.length,r=-1,n=0,a=-1,i=0,o;for(;t--;){const s=e.codePointAt(t);if(s===47){if(o){n=t+1;break}continue}r<0&&(o=!0,r=t+1),s===46?a<0?a=t:i!==1&&(i=1):a>-1&&(i=-1)}return a<0||r<0||i===0||i===1&&a===r-1&&a===n+1?"":e.slice(a,r)}function hOe(...e){let t=-1,r;for(;++t<e.length;)Og(e[t]),e[t]&&(r=r===void 0?e[t]:r+"/"+e[t]);return r===void 0?".":pOe(r)}function pOe(e){Og(e);const t=e.codePointAt(0)===47;let r=fOe(e,!t);return r.length===0&&!t&&(r="."),r.length>0&&e.codePointAt(e.length-1)===47&&(r+="/"),t?"/"+r:r}function fOe(e,t){let r="",n=0,a=-1,i=0,o=-1,s,l;for(;++o<=e.length;){if(o<e.length)s=e.codePointAt(o);else{if(s===47)break;s=47}if(s===47){if(!(a===o-1||i===1))if(a!==o-1&&i===2){if(r.length<2||n!==2||r.codePointAt(r.length-1)!==46||r.codePointAt(r.length-2)!==46){if(r.length>2){if(l=r.lastIndexOf("/"),l!==r.length-1){l<0?(r="",n=0):(r=r.slice(0,l),n=r.length-1-r.lastIndexOf("/")),a=o,i=0;continue}}else if(r.length>0){r="",n=0,a=o,i=0;continue}}t&&(r=r.length>0?r+"/..":"..",n=2)}else r.length>0?r+="/"+e.slice(a+1,o):r=e.slice(a+1,o),n=o-a-1;a=o,i=0}else s===46&&i>-1?i++:i=-1}return r}function Og(e){if(typeof e!="string")throw new TypeError("Path must be a string. Received "+JSON.stringify(e))}const mOe={cwd:gOe};function gOe(){return"/"}function jR(e){return!!(e!==null&&typeof e=="object"&&"href"in e&&e.href&&"protocol"in e&&e.protocol&&e.auth===void 0)}function bOe(e){if(typeof e=="string")e=new URL(e);else if(!jR(e)){const t=new TypeError('The "path" argument must be of type string or an instance of URL. Received `'+e+"`");throw t.code="ERR_INVALID_ARG_TYPE",t}if(e.protocol!=="file:"){const t=new TypeError("The URL must be of scheme file");throw t.code="ERR_INVALID_URL_SCHEME",t}return yOe(e)}function yOe(e){if(e.hostname!==""){const n=new TypeError('File URL host must be "localhost" or empty on darwin');throw n.code="ERR_INVALID_FILE_URL_HOST",n}const t=e.pathname;let r=-1;for(;++r<t.length;)if(t.codePointAt(r)===37&&t.codePointAt(r+1)===50){const n=t.codePointAt(r+2);if(n===70||n===102){const a=new TypeError("File URL path must not include encoded / characters");throw a.code="ERR_INVALID_FILE_URL_PATH",a}}return decodeURIComponent(t)}const hk=["history","path","basename","stem","extname","dirname"];class Gre{constructor(t){let r;t?jR(t)?r={path:t}:typeof t=="string"||vOe(t)?r={value:t}:r=t:r={},this.cwd="cwd"in r?"":mOe.cwd(),this.data={},this.history=[],this.messages=[],this.value,this.map,this.result,this.stored;let n=-1;for(;++n<hk.length;){const i=hk[n];i in r&&r[i]!==void 0&&r[i]!==null&&(this[i]=i==="history"?[...r[i]]:r[i])}let a;for(a in r)hk.includes(a)||(this[a]=r[a])}get basename(){return typeof this.path=="string"?Zo.basename(this.path):void 0}set basename(t){fk(t,"basename"),pk(t,"basename"),this.path=Zo.join(this.dirname||"",t)}get dirname(){return typeof this.path=="string"?Zo.dirname(this.path):void 0}set dirname(t){WP(this.basename,"dirname"),this.path=Zo.join(t||"",this.basename)}get extname(){return typeof this.path=="string"?Zo.extname(this.path):void 0}set extname(t){if(pk(t,"extname"),WP(this.dirname,"extname"),t){if(t.codePointAt(0)!==46)throw new Error("`extname` must start with `.`");if(t.includes(".",1))throw new Error("`extname` cannot contain multiple dots")}this.path=Zo.join(this.dirname,this.stem+(t||""))}get path(){return this.history[this.history.length-1]}set path(t){jR(t)&&(t=bOe(t)),fk(t,"path"),this.path!==t&&this.history.push(t)}get stem(){return typeof this.path=="string"?Zo.basename(this.path,this.extname):void 0}set stem(t){fk(t,"stem"),pk(t,"stem"),this.path=Zo.join(this.dirname||"",t+(this.extname||""))}fail(t,r,n){const a=this.message(t,r,n);throw a.fatal=!0,a}info(t,r,n){const a=this.message(t,r,n);return a.fatal=void 0,a}message(t,r,n){const a=new Ba(t,r,n);return this.path&&(a.name=this.path+":"+a.name,a.file=this.path),a.fatal=!1,this.messages.push(a),a}toString(t){return this.value===void 0?"":typeof this.value=="string"?this.value:new TextDecoder(t||void 0).decode(this.value)}}function pk(e,t){if(e&&e.includes(Zo.sep))throw new Error("`"+t+"` cannot be a path: did not expect `"+Zo.sep+"`")}function fk(e,t){if(!e)throw new Error("`"+t+"` cannot be empty")}function WP(e,t){if(!e)throw new Error("Setting `"+t+"` requires `path` to be set too")}function vOe(e){return!!(e&&typeof e=="object"&&"byteLength"in e&&"byteOffset"in e)}const SOe=(function(e){const n=this.constructor.prototype,a=n[e],i=function(){return a.apply(i,arguments)};return Object.setPrototypeOf(i,n),i}),EOe={}.hasOwnProperty;class Y5 extends SOe{constructor(){super("copy"),this.Compiler=void 0,this.Parser=void 0,this.attachers=[],this.compiler=void 0,this.freezeIndex=-1,this.frozen=void 0,this.namespace={},this.parser=void 0,this.transformers=sOe()}copy(){const t=new Y5;let r=-1;for(;++r<this.attachers.length;){const n=this.attachers[r];t.use(...n)}return t.data(dk(!0,{},this.namespace)),t}data(t,r){return typeof t=="string"?arguments.length===2?(bk("data",this.frozen),this.namespace[t]=r,this):EOe.call(this.namespace,t)&&this.namespace[t]||void 0:t?(bk("data",this.frozen),this.namespace=t,this):this.namespace}freeze(){if(this.frozen)return this;const t=this;for(;++this.freezeIndex<this.attachers.length;){const[r,...n]=this.attachers[this.freezeIndex];if(n[0]===!1)continue;n[0]===!0&&(n[0]=void 0);const a=r.call(t,...n);typeof a=="function"&&this.transformers.use(a)}return this.frozen=!0,this.freezeIndex=Number.POSITIVE_INFINITY,this}parse(t){this.freeze();const r=Xb(t),n=this.parser||this.Parser;return mk("parse",n),n(String(r),r)}process(t,r){const n=this;return this.freeze(),mk("process",this.parser||this.Parser),gk("process",this.compiler||this.Compiler),r?a(void 0,r):new Promise(a);function a(i,o){const s=Xb(t),l=n.parse(s);n.run(l,s,function(d,p,f){if(d||!p||!f)return u(d);const m=p,b=n.stringify(m,f);kOe(b)?f.value=b:f.result=b,u(d,f)});function u(d,p){d||!p?o(d):i?i(p):r(void 0,p)}}}processSync(t){let r=!1,n;return this.freeze(),mk("processSync",this.parser||this.Parser),gk("processSync",this.compiler||this.Compiler),this.process(t,a),KP("processSync","process",r),n;function a(i,o){r=!0,jP(i),n=o}}run(t,r,n){VP(t),this.freeze();const a=this.transformers;return!n&&typeof r=="function"&&(n=r,r=void 0),n?i(void 0,n):new Promise(i);function i(o,s){const l=Xb(r);a.run(t,l,u);function u(d,p,f){const m=p||t;d?s(d):o?o(m):n(void 0,m,f)}}}runSync(t,r){let n=!1,a;return this.run(t,r,i),KP("runSync","run",n),a;function i(o,s){jP(o),a=s,n=!0}}stringify(t,r){this.freeze();const n=Xb(r),a=this.compiler||this.Compiler;return gk("stringify",a),VP(t),a(t,n)}use(t,...r){const n=this.attachers,a=this.namespace;if(bk("use",this.frozen),t!=null)if(typeof t=="function")l(t,r);else if(typeof t=="object")Array.isArray(t)?s(t):o(t);else throw new TypeError("Expected usable value, not `"+t+"`");return this;function i(u){if(typeof u=="function")l(u,[]);else if(typeof u=="object")if(Array.isArray(u)){const[d,...p]=u;l(d,p)}else o(u);else throw new TypeError("Expected usable value, not `"+u+"`")}function o(u){if(!("plugins"in u)&&!("settings"in u))throw new Error("Expected usable value but received an empty preset, which is probably a mistake: presets typically come with `plugins` and sometimes with `settings`, but this has neither");s(u.plugins),u.settings&&(a.settings=dk(!0,a.settings,u.settings))}function s(u){let d=-1;if(u!=null)if(Array.isArray(u))for(;++d<u.length;){const p=u[d];i(p)}else throw new TypeError("Expected a list of plugins, not `"+u+"`")}function l(u,d){let p=-1,f=-1;for(;++p<n.length;)if(n[p][0]===u){f=p;break}if(f===-1)n.push([u,...d]);else if(d.length>0){let[m,...b]=d;const y=n[f][1];GR(y)&&GR(m)&&(m=dk(!0,y,m)),n[f]=[u,m,...b]}}}}const xOe=new Y5().freeze();function mk(e,t){if(typeof t!="function")throw new TypeError("Cannot `"+e+"` without `parser`")}function gk(e,t){if(typeof t!="function")throw new TypeError("Cannot `"+e+"` without `compiler`")}function bk(e,t){if(t)throw new Error("Cannot call `"+e+"` on a frozen processor.\nCreate a new processor first, by calling it: use `processor()` instead of `processor`.")}function VP(e){if(!GR(e)||typeof e.type!="string")throw new TypeError("Expected node, got `"+e+"`")}function KP(e,t,r){if(!r)throw new Error("`"+e+"` finished async. Use `"+t+"` instead")}function Xb(e){return wOe(e)?e:new Gre(e)}function wOe(e){return!!(e&&typeof e=="object"&&"message"in e&&"messages"in e)}function kOe(e){return typeof e=="string"||COe(e)}function COe(e){return!!(e&&typeof e=="object"&&"byteLength"in e&&"byteOffset"in e)}const TOe="https://github.com/remarkjs/react-markdown/blob/main/changelog.md",YP=[],JP={allowDangerousHtml:!0},AOe=/^(https?|ircs?|mailto|xmpp)$/i,FOe=[{from:"astPlugins",id:"remove-buggy-html-in-markdown-parser"},{from:"allowDangerousHtml",id:"remove-buggy-html-in-markdown-parser"},{from:"allowNode",id:"replace-allownode-allowedtypes-and-disallowedtypes",to:"allowElement"},{from:"allowedTypes",id:"replace-allownode-allowedtypes-and-disallowedtypes",to:"allowedElements"},{from:"disallowedTypes",id:"replace-allownode-allowedtypes-and-disallowedtypes",to:"disallowedElements"},{from:"escapeHtml",id:"remove-buggy-html-in-markdown-parser"},{from:"includeElementIndex",id:"#remove-includeelementindex"},{from:"includeNodeIndex",id:"change-includenodeindex-to-includeelementindex"},{from:"linkTarget",id:"remove-linktarget"},{from:"plugins",id:"change-plugins-to-remarkplugins",to:"remarkPlugins"},{from:"rawSourcePos",id:"#remove-rawsourcepos"},{from:"renderers",id:"change-renderers-to-components",to:"components"},{from:"source",id:"change-source-to-children",to:"children"},{from:"sourcePos",id:"#remove-sourcepos"},{from:"transformImageUri",id:"#add-urltransform",to:"urlTransform"},{from:"transformLinkUri",id:"#add-urltransform",to:"urlTransform"}];function XP(e){const t=_Oe(e),r=ROe(e);return DOe(t.runSync(t.parse(r),r),e)}function _Oe(e){const t=e.rehypePlugins||YP,r=e.remarkPlugins||YP,n=e.remarkRehypeOptions?{...e.remarkRehypeOptions,...JP}:JP;return xOe().use(cIe).use(r).use(aOe,n).use(t)}function ROe(e){const t=e.children||"",r=new Gre;return typeof t=="string"&&(r.value=t),r}function DOe(e,t){const r=t.allowedElements,n=t.allowElement,a=t.components,i=t.disallowedElements,o=t.skipHtml,s=t.unwrapDisallowed,l=t.urlTransform||NOe;for(const d of FOe)Object.hasOwn(t,d.from)&&(""+d.from+(d.to?"use `"+d.to+"` instead":"remove it")+TOe+d.id,void 0);return t.className&&(e={type:"element",tagName:"div",properties:{className:t.className},children:e.type==="root"?e.children:[e]}),Ig(e,u),yre(e,{Fragment:x.Fragment,components:a,ignoreInvalidStyle:!0,jsx:x.jsx,jsxs:x.jsxs,passKeys:!0,passNode:!0});function u(d,p,f){if(d.type==="raw"&&f&&typeof p=="number")return o?f.children.splice(p,1):f.children[p]={type:"text",value:d.value},p;if(d.type==="element"){let m;for(m in sk)if(Object.hasOwn(sk,m)&&Object.hasOwn(d.properties,m)){const b=d.properties[m],y=sk[m];(y===null||y.includes(d.tagName))&&(d.properties[m]=l(String(b||""),m,d))}}if(d.type==="element"){let m=r?!r.includes(d.tagName):i?i.includes(d.tagName):!1;if(!m&&n&&typeof p=="number"&&(m=!n(d,p,f)),m&&f&&typeof p=="number")return s&&d.children?f.children.splice(p,1,...d.children):f.children.splice(p,1),p}}}function NOe(e){const t=e.indexOf(":"),r=e.indexOf("?"),n=e.indexOf("#"),a=e.indexOf("/");return t===-1||a!==-1&&t>a||r!==-1&&t>r||n!==-1&&t>n||AOe.test(e.slice(0,t))?e:""}function ZP(e,t){const r=String(e);if(typeof t!="string")throw new TypeError("Expected character");let n=0,a=r.indexOf(t);for(;a!==-1;)n++,a=r.indexOf(t,a+t.length);return n}function LOe(e){if(typeof e!="string")throw new TypeError("Expected a string");return e.replace(/[|\\{}()[\]^$+*?.]/g,"\\$&").replace(/-/g,"\\x2d")}function MOe(e,t,r){const a=w2((r||{}).ignore||[]),i=IOe(t);let o=-1;for(;++o<i.length;)$re(e,"text",s);function s(u,d){let p=-1,f;for(;++p<d.length;){const m=d[p],b=f?f.children:void 0;if(a(m,b?b.indexOf(m):void 0,f))return;f=m}if(f)return l(u,d)}function l(u,d){const p=d[d.length-1],f=i[o][0],m=i[o][1];let b=0;const S=p.children.indexOf(u);let w=!1,E=[];f.lastIndex=0;let k=f.exec(u.value);for(;k;){const _=k.index,R={index:k.index,input:k.input,stack:[...d,u]};let C=m(...k,R);if(typeof C=="string"&&(C=C.length>0?{type:"text",value:C}:void 0),C===!1?f.lastIndex=_+1:(b!==_&&E.push({type:"text",value:u.value.slice(b,_)}),Array.isArray(C)?E.push(...C):C&&E.push(C),b=_+k[0].length,w=!0),!f.global)break;k=f.exec(u.value)}return w?(b<u.value.length&&E.push({type:"text",value:u.value.slice(b)}),p.children.splice(S,1,...E)):E=[u],S+E.length}}function IOe(e){const t=[];if(!Array.isArray(e))throw new TypeError("Expected find and replace tuple or list of tuples");const r=!e[0]||Array.isArray(e[0])?e:[e];let n=-1;for(;++n<r.length;){const a=r[n];t.push([OOe(a[0]),BOe(a[1])])}return t}function OOe(e){return typeof e=="string"?new RegExp(LOe(e),"g"):e}function BOe(e){return typeof e=="function"?e:function(){return e}}const yk="phrasing",vk=["autolink","link","image","label"];function POe(){return{transforms:[qOe],enter:{literalAutolink:HOe,literalAutolinkEmail:Sk,literalAutolinkHttp:Sk,literalAutolinkWww:Sk},exit:{literalAutolink:jOe,literalAutolinkEmail:GOe,literalAutolinkHttp:UOe,literalAutolinkWww:$Oe}}}function zOe(){return{unsafe:[{character:"@",before:"[+\\-.\\w]",after:"[\\-.\\w]",inConstruct:yk,notInConstruct:vk},{character:".",before:"[Ww]",after:"[\\-.\\w]",inConstruct:yk,notInConstruct:vk},{character:":",before:"[ps]",after:"\\/",inConstruct:yk,notInConstruct:vk}]}}function HOe(e){this.enter({type:"link",title:null,url:"",children:[]},e)}function Sk(e){this.config.enter.autolinkProtocol.call(this,e)}function UOe(e){this.config.exit.autolinkProtocol.call(this,e)}function $Oe(e){this.config.exit.data.call(this,e);const t=this.stack[this.stack.length-1];t.type,t.url="http://"+this.sliceSerialize(e)}function GOe(e){this.config.exit.autolinkEmail.call(this,e)}function jOe(e){this.exit(e)}function qOe(e){MOe(e,[[/(https?:\/\/|www(?=\.))([-.\w]+)([^ \t\r\n]*)/gi,WOe],[new RegExp("(?<=^|\\s|\\p{P}|\\p{S})([-.\\w+]+)@([-\\w]+(?:\\.[-\\w]+)+)","gu"),VOe]],{ignore:["link","linkReference"]})}function WOe(e,t,r,n,a){let i="";if(!jre(a)||(/^w/i.test(t)&&(r=t+r,t="",i="http://"),!KOe(r)))return!1;const o=YOe(r+n);if(!o[0])return!1;const s={type:"link",title:null,url:i+t+o[0],children:[{type:"text",value:t+o[0]}]};return o[1]?[s,{type:"text",value:o[1]}]:s}function VOe(e,t,r,n){return!jre(n,!0)||/[-\d_]$/.test(r)?!1:{type:"link",title:null,url:"mailto:"+t+"@"+r,children:[{type:"text",value:t+"@"+r}]}}function KOe(e){const t=e.split(".");return!(t.length<2||t[t.length-1]&&(/_/.test(t[t.length-1])||!/[a-zA-Z\d]/.test(t[t.length-1]))||t[t.length-2]&&(/_/.test(t[t.length-2])||!/[a-zA-Z\d]/.test(t[t.length-2])))}function YOe(e){const t=/[!"&'),.:;<>?\]}]+$/.exec(e);if(!t)return[e,void 0];e=e.slice(0,t.index);let r=t[0],n=r.indexOf(")");const a=ZP(e,"(");let i=ZP(e,")");for(;n!==-1&&a>i;)e+=r.slice(0,n+1),r=r.slice(n+1),n=r.indexOf(")"),i++;return[e,r]}function jre(e,t){const r=e.input.charCodeAt(e.index-1);return(e.index===0||Wc(r)||S2(r))&&(!t||r!==47)}qre.peek=a9e;function JOe(){this.buffer()}function XOe(e){this.enter({type:"footnoteReference",identifier:"",label:""},e)}function ZOe(){this.buffer()}function QOe(e){this.enter({type:"footnoteDefinition",identifier:"",label:"",children:[]},e)}function e9e(e){const t=this.resume(),r=this.stack[this.stack.length-1];r.type,r.identifier=Ro(this.sliceSerialize(e)).toLowerCase(),r.label=t}function t9e(e){this.exit(e)}function r9e(e){const t=this.resume(),r=this.stack[this.stack.length-1];r.type,r.identifier=Ro(this.sliceSerialize(e)).toLowerCase(),r.label=t}function n9e(e){this.exit(e)}function a9e(){return"["}function qre(e,t,r,n){const a=r.createTracker(n);let i=a.move("[^");const o=r.enter("footnoteReference"),s=r.enter("reference");return i+=a.move(r.safe(r.associationId(e),{after:"]",before:i})),s(),o(),i+=a.move("]"),i}function i9e(){return{enter:{gfmFootnoteCallString:JOe,gfmFootnoteCall:XOe,gfmFootnoteDefinitionLabelString:ZOe,gfmFootnoteDefinition:QOe},exit:{gfmFootnoteCallString:e9e,gfmFootnoteCall:t9e,gfmFootnoteDefinitionLabelString:r9e,gfmFootnoteDefinition:n9e}}}function o9e(e){let t=!1;return e&&e.firstLineBlank&&(t=!0),{handlers:{footnoteDefinition:r,footnoteReference:qre},unsafe:[{character:"[",inConstruct:["label","phrasing","reference"]}]};function r(n,a,i,o){const s=i.createTracker(o);let l=s.move("[^");const u=i.enter("footnoteDefinition"),d=i.enter("label");return l+=s.move(i.safe(i.associationId(n),{before:l,after:"]"})),d(),l+=s.move("]:"),n.children&&n.children.length>0&&(s.shift(4),l+=s.move((t?`
`:" ")+i.indentLines(i.containerFlow(n,s.current()),t?Wre:s9e))),u(),l}}function s9e(e,t,r){return t===0?e:Wre(e,t,r)}function Wre(e,t,r){return(r?"":"    ")+e}const l9e=["autolink","destinationLiteral","destinationRaw","reference","titleQuote","titleApostrophe"];Vre.peek=p9e;function u9e(){return{canContainEols:["delete"],enter:{strikethrough:d9e},exit:{strikethrough:h9e}}}function c9e(){return{unsafe:[{character:"~",inConstruct:"phrasing",notInConstruct:l9e}],handlers:{delete:Vre}}}function d9e(e){this.enter({type:"delete",children:[]},e)}function h9e(e){this.exit(e)}function Vre(e,t,r,n){const a=r.createTracker(n),i=r.enter("strikethrough");let o=a.move("~~");return o+=r.containerPhrasing(e,{...a.current(),before:o,after:"~"}),o+=a.move("~~"),i(),o}function p9e(){return"~"}function f9e(e){return e.length}function m9e(e,t){const r=t||{},n=(r.align||[]).concat(),a=r.stringLength||f9e,i=[],o=[],s=[],l=[];let u=0,d=-1;for(;++d<e.length;){const y=[],S=[];let w=-1;for(e[d].length>u&&(u=e[d].length);++w<e[d].length;){const E=g9e(e[d][w]);if(r.alignDelimiters!==!1){const k=a(E);S[w]=k,(l[w]===void 0||k>l[w])&&(l[w]=k)}y.push(E)}o[d]=y,s[d]=S}let p=-1;if(typeof n=="object"&&"length"in n)for(;++p<u;)i[p]=QP(n[p]);else{const y=QP(n);for(;++p<u;)i[p]=y}p=-1;const f=[],m=[];for(;++p<u;){const y=i[p];let S="",w="";y===99?(S=":",w=":"):y===108?S=":":y===114&&(w=":");let E=r.alignDelimiters===!1?1:Math.max(1,l[p]-S.length-w.length);const k=S+"-".repeat(E)+w;r.alignDelimiters!==!1&&(E=S.length+E+w.length,E>l[p]&&(l[p]=E),m[p]=E),f[p]=k}o.splice(1,0,f),s.splice(1,0,m),d=-1;const b=[];for(;++d<o.length;){const y=o[d],S=s[d];p=-1;const w=[];for(;++p<u;){const E=y[p]||"";let k="",_="";if(r.alignDelimiters!==!1){const R=l[p]-(S[p]||0),C=i[p];C===114?k=" ".repeat(R):C===99?R%2?(k=" ".repeat(R/2+.5),_=" ".repeat(R/2-.5)):(k=" ".repeat(R/2),_=k):_=" ".repeat(R)}r.delimiterStart!==!1&&!p&&w.push("|"),r.padding!==!1&&!(r.alignDelimiters===!1&&E==="")&&(r.delimiterStart!==!1||p)&&w.push(" "),r.alignDelimiters!==!1&&w.push(k),w.push(E),r.alignDelimiters!==!1&&w.push(_),r.padding!==!1&&w.push(" "),(r.delimiterEnd!==!1||p!==u-1)&&w.push("|")}b.push(r.delimiterEnd===!1?w.join("").replace(/ +$/,""):w.join(""))}return b.join(`
`)}function g9e(e){return e==null?"":String(e)}function QP(e){const t=typeof e=="string"?e.codePointAt(0):0;return t===67||t===99?99:t===76||t===108?108:t===82||t===114?114:0}const ez={}.hasOwnProperty;function Kre(e,t){const r=t||{};function n(a,...i){let o=n.invalid;const s=n.handlers;if(a&&ez.call(a,e)){const l=String(a[e]);o=ez.call(s,l)?s[l]:n.unknown}if(o)return o.call(this,a,...i)}return n.handlers=r.handlers||{},n.invalid=r.invalid,n.unknown=r.unknown,n}function b9e(e,t,r,n){const a=r.enter("blockquote"),i=r.createTracker(n);i.move("> "),i.shift(2);const o=r.indentLines(r.containerFlow(e,i.current()),y9e);return a(),o}function y9e(e,t,r){return">"+(r?"":" ")+e}function v9e(e,t){return tz(e,t.inConstruct,!0)&&!tz(e,t.notInConstruct,!1)}function tz(e,t,r){if(typeof t=="string"&&(t=[t]),!t||t.length===0)return r;let n=-1;for(;++n<t.length;)if(e.includes(t[n]))return!0;return!1}function rz(e,t,r,n){let a=-1;for(;++a<r.unsafe.length;)if(r.unsafe[a].character===`
`&&v9e(r.stack,r.unsafe[a]))return/[ \t]/.test(n.before)?"":" ";return`\\
`}function Yre(e,t){const r=String(e);let n=r.indexOf(t),a=n,i=0,o=0;if(typeof t!="string")throw new TypeError("Expected substring");for(;n!==-1;)n===a?++i>o&&(o=i):i=1,a=n+t.length,n=r.indexOf(t,a);return o}function S9e(e,t){return!!(t.options.fences===!1&&e.value&&!e.lang&&/[^ \r\n]/.test(e.value)&&!/^[\t ]*(?:[\r\n]|$)|(?:^|[\r\n])[\t ]*$/.test(e.value))}function E9e(e){const t=e.options.fence||"`";if(t!=="`"&&t!=="~")throw new Error("Cannot serialize code with `"+t+"` for `options.fence`, expected `` ` `` or `~`");return t}function x9e(e,t,r,n){const a=E9e(r),i=e.value||"",o=a==="`"?"GraveAccent":"Tilde";if(S9e(e,r)){const p=r.enter("codeIndented"),f=r.indentLines(i,w9e);return p(),f}const s=r.createTracker(n),l=a.repeat(Math.max(Yre(i,a)+1,3)),u=r.enter("codeFenced");let d=s.move(l);if(e.lang){const p=r.enter(`codeFencedLang${o}`);d+=s.move(r.safe(e.lang,{before:d,after:" ",encode:["`"],...s.current()})),p()}if(e.lang&&e.meta){const p=r.enter(`codeFencedMeta${o}`);d+=s.move(" "),d+=s.move(r.safe(e.meta,{before:d,after:`
`,encode:["`"],...s.current()})),p()}return d+=s.move(`
`),i&&(d+=s.move(i+`
`)),d+=s.move(l),u(),d}function w9e(e,t,r){return(r?"":"    ")+e}function J5(e){const t=e.options.quote||'"';if(t!=='"'&&t!=="'")throw new Error("Cannot serialize title with `"+t+"` for `options.quote`, expected `\"`, or `'`");return t}function k9e(e,t,r,n){const a=J5(r),i=a==='"'?"Quote":"Apostrophe",o=r.enter("definition");let s=r.enter("label");const l=r.createTracker(n);let u=l.move("[");return u+=l.move(r.safe(r.associationId(e),{before:u,after:"]",...l.current()})),u+=l.move("]: "),s(),!e.url||/[\0- \u007F]/.test(e.url)?(s=r.enter("destinationLiteral"),u+=l.move("<"),u+=l.move(r.safe(e.url,{before:u,after:">",...l.current()})),u+=l.move(">")):(s=r.enter("destinationRaw"),u+=l.move(r.safe(e.url,{before:u,after:e.title?" ":`
`,...l.current()}))),s(),e.title&&(s=r.enter(`title${i}`),u+=l.move(" "+a),u+=l.move(r.safe(e.title,{before:u,after:a,...l.current()})),u+=l.move(a),s()),o(),u}function C9e(e){const t=e.options.emphasis||"*";if(t!=="*"&&t!=="_")throw new Error("Cannot serialize emphasis with `"+t+"` for `options.emphasis`, expected `*`, or `_`");return t}function qm(e){return"&#x"+e.toString(16).toUpperCase()+";"}function Dy(e,t,r){const n=Kh(e),a=Kh(t);return n===void 0?a===void 0?r==="_"?{inside:!0,outside:!0}:{inside:!1,outside:!1}:a===1?{inside:!0,outside:!0}:{inside:!1,outside:!0}:n===1?a===void 0?{inside:!1,outside:!1}:a===1?{inside:!0,outside:!0}:{inside:!1,outside:!1}:a===void 0?{inside:!1,outside:!1}:a===1?{inside:!0,outside:!1}:{inside:!1,outside:!1}}Jre.peek=T9e;function Jre(e,t,r,n){const a=C9e(r),i=r.enter("emphasis"),o=r.createTracker(n),s=o.move(a);let l=o.move(r.containerPhrasing(e,{after:a,before:s,...o.current()}));const u=l.charCodeAt(0),d=Dy(n.before.charCodeAt(n.before.length-1),u,a);d.inside&&(l=qm(u)+l.slice(1));const p=l.charCodeAt(l.length-1),f=Dy(n.after.charCodeAt(0),p,a);f.inside&&(l=l.slice(0,-1)+qm(p));const m=o.move(a);return i(),r.attentionEncodeSurroundingInfo={after:f.outside,before:d.outside},s+l+m}function T9e(e,t,r){return r.options.emphasis||"*"}function A9e(e,t){let r=!1;return Ig(e,function(n){if("value"in n&&/\r?\n|\r/.test(n.value)||n.type==="break")return r=!0,UR}),!!((!e.depth||e.depth<3)&&$5(e)&&(t.options.setext||r))}function F9e(e,t,r,n){const a=Math.max(Math.min(6,e.depth||1),1),i=r.createTracker(n);if(A9e(e,r)){const d=r.enter("headingSetext"),p=r.enter("phrasing"),f=r.containerPhrasing(e,{...i.current(),before:`
`,after:`
`});return p(),d(),f+`
`+(a===1?"=":"-").repeat(f.length-(Math.max(f.lastIndexOf("\r"),f.lastIndexOf(`
`))+1))}const o="#".repeat(a),s=r.enter("headingAtx"),l=r.enter("phrasing");i.move(o+" ");let u=r.containerPhrasing(e,{before:"# ",after:`
`,...i.current()});return/^[\t ]/.test(u)&&(u=qm(u.charCodeAt(0))+u.slice(1)),u=u?o+" "+u:o,r.options.closeAtx&&(u+=" "+o),l(),s(),u}Xre.peek=_9e;function Xre(e){return e.value||""}function _9e(){return"<"}Zre.peek=R9e;function Zre(e,t,r,n){const a=J5(r),i=a==='"'?"Quote":"Apostrophe",o=r.enter("image");let s=r.enter("label");const l=r.createTracker(n);let u=l.move("![");return u+=l.move(r.safe(e.alt,{before:u,after:"]",...l.current()})),u+=l.move("]("),s(),!e.url&&e.title||/[\0- \u007F]/.test(e.url)?(s=r.enter("destinationLiteral"),u+=l.move("<"),u+=l.move(r.safe(e.url,{before:u,after:">",...l.current()})),u+=l.move(">")):(s=r.enter("destinationRaw"),u+=l.move(r.safe(e.url,{before:u,after:e.title?" ":")",...l.current()}))),s(),e.title&&(s=r.enter(`title${i}`),u+=l.move(" "+a),u+=l.move(r.safe(e.title,{before:u,after:a,...l.current()})),u+=l.move(a),s()),u+=l.move(")"),o(),u}function R9e(){return"!"}Qre.peek=D9e;function Qre(e,t,r,n){const a=e.referenceType,i=r.enter("imageReference");let o=r.enter("label");const s=r.createTracker(n);let l=s.move("![");const u=r.safe(e.alt,{before:l,after:"]",...s.current()});l+=s.move(u+"]["),o();const d=r.stack;r.stack=[],o=r.enter("reference");const p=r.safe(r.associationId(e),{before:l,after:"]",...s.current()});return o(),r.stack=d,i(),a==="full"||!u||u!==p?l+=s.move(p+"]"):a==="shortcut"?l=l.slice(0,-1):l+=s.move("]"),l}function D9e(){return"!"}ene.peek=N9e;function ene(e,t,r){let n=e.value||"",a="`",i=-1;for(;new RegExp("(^|[^`])"+a+"([^`]|$)").test(n);)a+="`";for(/[^ \r\n]/.test(n)&&(/^[ \r\n]/.test(n)&&/[ \r\n]$/.test(n)||/^`|`$/.test(n))&&(n=" "+n+" ");++i<r.unsafe.length;){const o=r.unsafe[i],s=r.compilePattern(o);let l;if(o.atBreak)for(;l=s.exec(n);){let u=l.index;n.charCodeAt(u)===10&&n.charCodeAt(u-1)===13&&u--,n=n.slice(0,u)+" "+n.slice(l.index+1)}}return a+n+a}function N9e(){return"`"}function tne(e,t){const r=$5(e);return!!(!t.options.resourceLink&&e.url&&!e.title&&e.children&&e.children.length===1&&e.children[0].type==="text"&&(r===e.url||"mailto:"+r===e.url)&&/^[a-z][a-z+.-]+:/i.test(e.url)&&!/[\0- <>\u007F]/.test(e.url))}rne.peek=L9e;function rne(e,t,r,n){const a=J5(r),i=a==='"'?"Quote":"Apostrophe",o=r.createTracker(n);let s,l;if(tne(e,r)){const d=r.stack;r.stack=[],s=r.enter("autolink");let p=o.move("<");return p+=o.move(r.containerPhrasing(e,{before:p,after:">",...o.current()})),p+=o.move(">"),s(),r.stack=d,p}s=r.enter("link"),l=r.enter("label");let u=o.move("[");return u+=o.move(r.containerPhrasing(e,{before:u,after:"](",...o.current()})),u+=o.move("]("),l(),!e.url&&e.title||/[\0- \u007F]/.test(e.url)?(l=r.enter("destinationLiteral"),u+=o.move("<"),u+=o.move(r.safe(e.url,{before:u,after:">",...o.current()})),u+=o.move(">")):(l=r.enter("destinationRaw"),u+=o.move(r.safe(e.url,{before:u,after:e.title?" ":")",...o.current()}))),l(),e.title&&(l=r.enter(`title${i}`),u+=o.move(" "+a),u+=o.move(r.safe(e.title,{before:u,after:a,...o.current()})),u+=o.move(a),l()),u+=o.move(")"),s(),u}function L9e(e,t,r){return tne(e,r)?"<":"["}nne.peek=M9e;function nne(e,t,r,n){const a=e.referenceType,i=r.enter("linkReference");let o=r.enter("label");const s=r.createTracker(n);let l=s.move("[");const u=r.containerPhrasing(e,{before:l,after:"]",...s.current()});l+=s.move(u+"]["),o();const d=r.stack;r.stack=[],o=r.enter("reference");const p=r.safe(r.associationId(e),{before:l,after:"]",...s.current()});return o(),r.stack=d,i(),a==="full"||!u||u!==p?l+=s.move(p+"]"):a==="shortcut"?l=l.slice(0,-1):l+=s.move("]"),l}function M9e(){return"["}function X5(e){const t=e.options.bullet||"*";if(t!=="*"&&t!=="+"&&t!=="-")throw new Error("Cannot serialize items with `"+t+"` for `options.bullet`, expected `*`, `+`, or `-`");return t}function I9e(e){const t=X5(e),r=e.options.bulletOther;if(!r)return t==="*"?"-":"*";if(r!=="*"&&r!=="+"&&r!=="-")throw new Error("Cannot serialize items with `"+r+"` for `options.bulletOther`, expected `*`, `+`, or `-`");if(r===t)throw new Error("Expected `bullet` (`"+t+"`) and `bulletOther` (`"+r+"`) to be different");return r}function O9e(e){const t=e.options.bulletOrdered||".";if(t!=="."&&t!==")")throw new Error("Cannot serialize items with `"+t+"` for `options.bulletOrdered`, expected `.` or `)`");return t}function ane(e){const t=e.options.rule||"*";if(t!=="*"&&t!=="-"&&t!=="_")throw new Error("Cannot serialize rules with `"+t+"` for `options.rule`, expected `*`, `-`, or `_`");return t}function B9e(e,t,r,n){const a=r.enter("list"),i=r.bulletCurrent;let o=e.ordered?O9e(r):X5(r);const s=e.ordered?o==="."?")":".":I9e(r);let l=t&&r.bulletLastUsed?o===r.bulletLastUsed:!1;if(!e.ordered){const d=e.children?e.children[0]:void 0;if((o==="*"||o==="-")&&d&&(!d.children||!d.children[0])&&r.stack[r.stack.length-1]==="list"&&r.stack[r.stack.length-2]==="listItem"&&r.stack[r.stack.length-3]==="list"&&r.stack[r.stack.length-4]==="listItem"&&r.indexStack[r.indexStack.length-1]===0&&r.indexStack[r.indexStack.length-2]===0&&r.indexStack[r.indexStack.length-3]===0&&(l=!0),ane(r)===o&&d){let p=-1;for(;++p<e.children.length;){const f=e.children[p];if(f&&f.type==="listItem"&&f.children&&f.children[0]&&f.children[0].type==="thematicBreak"){l=!0;break}}}}l&&(o=s),r.bulletCurrent=o;const u=r.containerFlow(e,n);return r.bulletLastUsed=o,r.bulletCurrent=i,a(),u}function P9e(e){const t=e.options.listItemIndent||"one";if(t!=="tab"&&t!=="one"&&t!=="mixed")throw new Error("Cannot serialize items with `"+t+"` for `options.listItemIndent`, expected `tab`, `one`, or `mixed`");return t}function z9e(e,t,r,n){const a=P9e(r);let i=r.bulletCurrent||X5(r);t&&t.type==="list"&&t.ordered&&(i=(typeof t.start=="number"&&t.start>-1?t.start:1)+(r.options.incrementListMarker===!1?0:t.children.indexOf(e))+i);let o=i.length+1;(a==="tab"||a==="mixed"&&(t&&t.type==="list"&&t.spread||e.spread))&&(o=Math.ceil(o/4)*4);const s=r.createTracker(n);s.move(i+" ".repeat(o-i.length)),s.shift(o);const l=r.enter("listItem"),u=r.indentLines(r.containerFlow(e,s.current()),d);return l(),u;function d(p,f,m){return f?(m?"":" ".repeat(o))+p:(m?i:i+" ".repeat(o-i.length))+p}}function H9e(e,t,r,n){const a=r.enter("paragraph"),i=r.enter("phrasing"),o=r.containerPhrasing(e,n);return i(),a(),o}const U9e=w2(["break","delete","emphasis","footnote","footnoteReference","image","imageReference","inlineCode","inlineMath","link","linkReference","mdxJsxTextElement","mdxTextExpression","strong","text","textDirective"]);function $9e(e,t,r,n){return(e.children.some(function(o){return U9e(o)})?r.containerPhrasing:r.containerFlow).call(r,e,n)}function G9e(e){const t=e.options.strong||"*";if(t!=="*"&&t!=="_")throw new Error("Cannot serialize strong with `"+t+"` for `options.strong`, expected `*`, or `_`");return t}ine.peek=j9e;function ine(e,t,r,n){const a=G9e(r),i=r.enter("strong"),o=r.createTracker(n),s=o.move(a+a);let l=o.move(r.containerPhrasing(e,{after:a,before:s,...o.current()}));const u=l.charCodeAt(0),d=Dy(n.before.charCodeAt(n.before.length-1),u,a);d.inside&&(l=qm(u)+l.slice(1));const p=l.charCodeAt(l.length-1),f=Dy(n.after.charCodeAt(0),p,a);f.inside&&(l=l.slice(0,-1)+qm(p));const m=o.move(a+a);return i(),r.attentionEncodeSurroundingInfo={after:f.outside,before:d.outside},s+l+m}function j9e(e,t,r){return r.options.strong||"*"}function q9e(e,t,r,n){return r.safe(e.value,n)}function W9e(e){const t=e.options.ruleRepetition||3;if(t<3)throw new Error("Cannot serialize rules with repetition `"+t+"` for `options.ruleRepetition`, expected `3` or more");return t}function V9e(e,t,r){const n=(ane(r)+(r.options.ruleSpaces?" ":"")).repeat(W9e(r));return r.options.ruleSpaces?n.slice(0,-1):n}const one={blockquote:b9e,break:rz,code:x9e,definition:k9e,emphasis:Jre,hardBreak:rz,heading:F9e,html:Xre,image:Zre,imageReference:Qre,inlineCode:ene,link:rne,linkReference:nne,list:B9e,listItem:z9e,paragraph:H9e,root:$9e,strong:ine,text:q9e,thematicBreak:V9e};function K9e(){return{enter:{table:Y9e,tableData:nz,tableHeader:nz,tableRow:X9e},exit:{codeText:Z9e,table:J9e,tableData:Ek,tableHeader:Ek,tableRow:Ek}}}function Y9e(e){const t=e._align;this.enter({type:"table",align:t.map(function(r){return r==="none"?null:r}),children:[]},e),this.data.inTable=!0}function J9e(e){this.exit(e),this.data.inTable=void 0}function X9e(e){this.enter({type:"tableRow",children:[]},e)}function Ek(e){this.exit(e)}function nz(e){this.enter({type:"tableCell",children:[]},e)}function Z9e(e){let t=this.resume();this.data.inTable&&(t=t.replace(/\\([\\|])/g,Q9e));const r=this.stack[this.stack.length-1];r.type,r.value=t,this.exit(e)}function Q9e(e,t){return t==="|"?t:e}function e8e(e){const t=e||{},r=t.tableCellPadding,n=t.tablePipeAlign,a=t.stringLength,i=r?" ":"|";return{unsafe:[{character:"\r",inConstruct:"tableCell"},{character:`
`,inConstruct:"tableCell"},{atBreak:!0,character:"|",after:"[	 :-]"},{character:"|",inConstruct:"tableCell"},{atBreak:!0,character:":",after:"-"},{atBreak:!0,character:"-",after:"[:|-]"}],handlers:{inlineCode:f,table:o,tableCell:l,tableRow:s}};function o(m,b,y,S){return u(d(m,y,S),m.align)}function s(m,b,y,S){const w=p(m,y,S),E=u([w]);return E.slice(0,E.indexOf(`
`))}function l(m,b,y,S){const w=y.enter("tableCell"),E=y.enter("phrasing"),k=y.containerPhrasing(m,{...S,before:i,after:i});return E(),w(),k}function u(m,b){return m9e(m,{align:b,alignDelimiters:n,padding:r,stringLength:a})}function d(m,b,y){const S=m.children;let w=-1;const E=[],k=b.enter("table");for(;++w<S.length;)E[w]=p(S[w],b,y);return k(),E}function p(m,b,y){const S=m.children;let w=-1;const E=[],k=b.enter("tableRow");for(;++w<S.length;)E[w]=l(S[w],m,b,y);return k(),E}function f(m,b,y){let S=one.inlineCode(m,b,y);return y.stack.includes("tableCell")&&(S=S.replace(/\|/g,"\\$&")),S}}function t8e(){return{exit:{taskListCheckValueChecked:az,taskListCheckValueUnchecked:az,paragraph:n8e}}}function r8e(){return{unsafe:[{atBreak:!0,character:"-",after:"[:|-]"}],handlers:{listItem:a8e}}}function az(e){const t=this.stack[this.stack.length-2];t.type,t.checked=e.type==="taskListCheckValueChecked"}function n8e(e){const t=this.stack[this.stack.length-2];if(t&&t.type==="listItem"&&typeof t.checked=="boolean"){const r=this.stack[this.stack.length-1];r.type;const n=r.children[0];if(n&&n.type==="text"){const a=t.children;let i=-1,o;for(;++i<a.length;){const s=a[i];if(s.type==="paragraph"){o=s;break}}o===r&&(n.value=n.value.slice(1),n.value.length===0?r.children.shift():r.position&&n.position&&typeof n.position.start.offset=="number"&&(n.position.start.column++,n.position.start.offset++,r.position.start=Object.assign({},n.position.start)))}}this.exit(e)}function a8e(e,t,r,n){const a=e.children[0],i=typeof e.checked=="boolean"&&a&&a.type==="paragraph",o="["+(e.checked?"x":" ")+"] ",s=r.createTracker(n);i&&s.move(o);let l=one.listItem(e,t,r,{...n,...s.current()});return i&&(l=l.replace(/^(?:[*+-]|\d+\.)([\r\n]| {1,3})/,u)),l;function u(d){return d+o}}function i8e(){return[POe(),i9e(),u9e(),K9e(),t8e()]}function o8e(e){return{extensions:[zOe(),o9e(e),c9e(),e8e(e),r8e()]}}const s8e={tokenize:p8e,partial:!0},sne={tokenize:f8e,partial:!0},lne={tokenize:m8e,partial:!0},une={tokenize:g8e,partial:!0},l8e={tokenize:b8e,partial:!0},cne={name:"wwwAutolink",tokenize:d8e,previous:hne},dne={name:"protocolAutolink",tokenize:h8e,previous:pne},Rl={name:"emailAutolink",tokenize:c8e,previous:fne},Ss={};function u8e(){return{text:Ss}}let fc=48;for(;fc<123;)Ss[fc]=Rl,fc++,fc===58?fc=65:fc===91&&(fc=97);Ss[43]=Rl;Ss[45]=Rl;Ss[46]=Rl;Ss[95]=Rl;Ss[72]=[Rl,dne];Ss[104]=[Rl,dne];Ss[87]=[Rl,cne];Ss[119]=[Rl,cne];function c8e(e,t,r){const n=this;let a,i;return o;function o(p){return!qR(p)||!fne.call(n,n.previous)||Z5(n.events)?r(p):(e.enter("literalAutolink"),e.enter("literalAutolinkEmail"),s(p))}function s(p){return qR(p)?(e.consume(p),s):p===64?(e.consume(p),l):r(p)}function l(p){return p===46?e.check(l8e,d,u)(p):p===45||p===95||Na(p)?(i=!0,e.consume(p),l):d(p)}function u(p){return e.consume(p),a=!0,l}function d(p){return i&&a&&ja(n.previous)?(e.exit("literalAutolinkEmail"),e.exit("literalAutolink"),t(p)):r(p)}}function d8e(e,t,r){const n=this;return a;function a(o){return o!==87&&o!==119||!hne.call(n,n.previous)||Z5(n.events)?r(o):(e.enter("literalAutolink"),e.enter("literalAutolinkWww"),e.check(s8e,e.attempt(sne,e.attempt(lne,i),r),r)(o))}function i(o){return e.exit("literalAutolinkWww"),e.exit("literalAutolink"),t(o)}}function h8e(e,t,r){const n=this;let a="",i=!1;return o;function o(p){return(p===72||p===104)&&pne.call(n,n.previous)&&!Z5(n.events)?(e.enter("literalAutolink"),e.enter("literalAutolinkHttp"),a+=String.fromCodePoint(p),e.consume(p),s):r(p)}function s(p){if(ja(p)&&a.length<5)return a+=String.fromCodePoint(p),e.consume(p),s;if(p===58){const f=a.toLowerCase();if(f==="http"||f==="https")return e.consume(p),l}return r(p)}function l(p){return p===47?(e.consume(p),i?u:(i=!0,l)):r(p)}function u(p){return p===null||_y(p)||rn(p)||Wc(p)||S2(p)?r(p):e.attempt(sne,e.attempt(lne,d),r)(p)}function d(p){return e.exit("literalAutolinkHttp"),e.exit("literalAutolink"),t(p)}}function p8e(e,t,r){let n=0;return a;function a(o){return(o===87||o===119)&&n<3?(n++,e.consume(o),a):o===46&&n===3?(e.consume(o),i):r(o)}function i(o){return o===null?r(o):t(o)}}function f8e(e,t,r){let n,a,i;return o;function o(u){return u===46||u===95?e.check(une,l,s)(u):u===null||rn(u)||Wc(u)||u!==45&&S2(u)?l(u):(i=!0,e.consume(u),o)}function s(u){return u===95?n=!0:(a=n,n=void 0),e.consume(u),o}function l(u){return a||n||!i?r(u):t(u)}}function m8e(e,t){let r=0,n=0;return a;function a(o){return o===40?(r++,e.consume(o),a):o===41&&n<r?i(o):o===33||o===34||o===38||o===39||o===41||o===42||o===44||o===46||o===58||o===59||o===60||o===63||o===93||o===95||o===126?e.check(une,t,i)(o):o===null||rn(o)||Wc(o)?t(o):(e.consume(o),a)}function i(o){return o===41&&n++,e.consume(o),a}}function g8e(e,t,r){return n;function n(s){return s===33||s===34||s===39||s===41||s===42||s===44||s===46||s===58||s===59||s===63||s===95||s===126?(e.consume(s),n):s===38?(e.consume(s),i):s===93?(e.consume(s),a):s===60||s===null||rn(s)||Wc(s)?t(s):r(s)}function a(s){return s===null||s===40||s===91||rn(s)||Wc(s)?t(s):n(s)}function i(s){return ja(s)?o(s):r(s)}function o(s){return s===59?(e.consume(s),n):ja(s)?(e.consume(s),o):r(s)}}function b8e(e,t,r){return n;function n(i){return e.consume(i),a}function a(i){return Na(i)?r(i):t(i)}}function hne(e){return e===null||e===40||e===42||e===95||e===91||e===93||e===126||rn(e)}function pne(e){return!ja(e)}function fne(e){return!(e===47||qR(e))}function qR(e){return e===43||e===45||e===46||e===95||Na(e)}function Z5(e){let t=e.length,r=!1;for(;t--;){const n=e[t][1];if((n.type==="labelLink"||n.type==="labelImage")&&!n._balanced){r=!0;break}if(n._gfmAutolinkLiteralWalkedInto){r=!1;break}}return e.length>0&&!r&&(e[e.length-1][1]._gfmAutolinkLiteralWalkedInto=!0),r}const y8e={tokenize:T8e,partial:!0};function v8e(){return{document:{91:{name:"gfmFootnoteDefinition",tokenize:w8e,continuation:{tokenize:k8e},exit:C8e}},text:{91:{name:"gfmFootnoteCall",tokenize:x8e},93:{name:"gfmPotentialFootnoteCall",add:"after",tokenize:S8e,resolveTo:E8e}}}}function S8e(e,t,r){const n=this;let a=n.events.length;const i=n.parser.gfmFootnotes||(n.parser.gfmFootnotes=[]);let o;for(;a--;){const l=n.events[a][1];if(l.type==="labelImage"){o=l;break}if(l.type==="gfmFootnoteCall"||l.type==="labelLink"||l.type==="label"||l.type==="image"||l.type==="link")break}return s;function s(l){if(!o||!o._balanced)return r(l);const u=Ro(n.sliceSerialize({start:o.end,end:n.now()}));return u.codePointAt(0)!==94||!i.includes(u.slice(1))?r(l):(e.enter("gfmFootnoteCallLabelMarker"),e.consume(l),e.exit("gfmFootnoteCallLabelMarker"),t(l))}}function E8e(e,t){let r=e.length;for(;r--;)if(e[r][1].type==="labelImage"&&e[r][0]==="enter"){e[r][1];break}e[r+1][1].type="data",e[r+3][1].type="gfmFootnoteCallLabelMarker";const n={type:"gfmFootnoteCall",start:Object.assign({},e[r+3][1].start),end:Object.assign({},e[e.length-1][1].end)},a={type:"gfmFootnoteCallMarker",start:Object.assign({},e[r+3][1].end),end:Object.assign({},e[r+3][1].end)};a.end.column++,a.end.offset++,a.end._bufferIndex++;const i={type:"gfmFootnoteCallString",start:Object.assign({},a.end),end:Object.assign({},e[e.length-1][1].start)},o={type:"chunkString",contentType:"string",start:Object.assign({},i.start),end:Object.assign({},i.end)},s=[e[r+1],e[r+2],["enter",n,t],e[r+3],e[r+4],["enter",a,t],["exit",a,t],["enter",i,t],["enter",o,t],["exit",o,t],["exit",i,t],e[e.length-2],e[e.length-1],["exit",n,t]];return e.splice(r,e.length-r+1,...s),e}function x8e(e,t,r){const n=this,a=n.parser.gfmFootnotes||(n.parser.gfmFootnotes=[]);let i=0,o;return s;function s(p){return e.enter("gfmFootnoteCall"),e.enter("gfmFootnoteCallLabelMarker"),e.consume(p),e.exit("gfmFootnoteCallLabelMarker"),l}function l(p){return p!==94?r(p):(e.enter("gfmFootnoteCallMarker"),e.consume(p),e.exit("gfmFootnoteCallMarker"),e.enter("gfmFootnoteCallString"),e.enter("chunkString").contentType="string",u)}function u(p){if(i>999||p===93&&!o||p===null||p===91||rn(p))return r(p);if(p===93){e.exit("chunkString");const f=e.exit("gfmFootnoteCallString");return a.includes(Ro(n.sliceSerialize(f)))?(e.enter("gfmFootnoteCallLabelMarker"),e.consume(p),e.exit("gfmFootnoteCallLabelMarker"),e.exit("gfmFootnoteCall"),t):r(p)}return rn(p)||(o=!0),i++,e.consume(p),p===92?d:u}function d(p){return p===91||p===92||p===93?(e.consume(p),i++,u):u(p)}}function w8e(e,t,r){const n=this,a=n.parser.gfmFootnotes||(n.parser.gfmFootnotes=[]);let i,o=0,s;return l;function l(b){return e.enter("gfmFootnoteDefinition")._container=!0,e.enter("gfmFootnoteDefinitionLabel"),e.enter("gfmFootnoteDefinitionLabelMarker"),e.consume(b),e.exit("gfmFootnoteDefinitionLabelMarker"),u}function u(b){return b===94?(e.enter("gfmFootnoteDefinitionMarker"),e.consume(b),e.exit("gfmFootnoteDefinitionMarker"),e.enter("gfmFootnoteDefinitionLabelString"),e.enter("chunkString").contentType="string",d):r(b)}function d(b){if(o>999||b===93&&!s||b===null||b===91||rn(b))return r(b);if(b===93){e.exit("chunkString");const y=e.exit("gfmFootnoteDefinitionLabelString");return i=Ro(n.sliceSerialize(y)),e.enter("gfmFootnoteDefinitionLabelMarker"),e.consume(b),e.exit("gfmFootnoteDefinitionLabelMarker"),e.exit("gfmFootnoteDefinitionLabel"),f}return rn(b)||(s=!0),o++,e.consume(b),b===92?p:d}function p(b){return b===91||b===92||b===93?(e.consume(b),o++,d):d(b)}function f(b){return b===58?(e.enter("definitionMarker"),e.consume(b),e.exit("definitionMarker"),a.includes(i)||a.push(i),xr(e,m,"gfmFootnoteDefinitionWhitespace")):r(b)}function m(b){return t(b)}}function k8e(e,t,r){return e.check(Mg,t,e.attempt(y8e,t,r))}function C8e(e){e.exit("gfmFootnoteDefinition")}function T8e(e,t,r){const n=this;return xr(e,a,"gfmFootnoteDefinitionIndent",5);function a(i){const o=n.events[n.events.length-1];return o&&o[1].type==="gfmFootnoteDefinitionIndent"&&o[2].sliceSerialize(o[1],!0).length===4?t(i):r(i)}}function A8e(e){let r=(e||{}).singleTilde;const n={name:"strikethrough",tokenize:i,resolveAll:a};return r==null&&(r=!0),{text:{126:n},insideSpan:{null:[n]},attentionMarkers:{null:[126]}};function a(o,s){let l=-1;for(;++l<o.length;)if(o[l][0]==="enter"&&o[l][1].type==="strikethroughSequenceTemporary"&&o[l][1]._close){let u=l;for(;u--;)if(o[u][0]==="exit"&&o[u][1].type==="strikethroughSequenceTemporary"&&o[u][1]._open&&o[l][1].end.offset-o[l][1].start.offset===o[u][1].end.offset-o[u][1].start.offset){o[l][1].type="strikethroughSequence",o[u][1].type="strikethroughSequence";const d={type:"strikethrough",start:Object.assign({},o[u][1].start),end:Object.assign({},o[l][1].end)},p={type:"strikethroughText",start:Object.assign({},o[u][1].end),end:Object.assign({},o[l][1].start)},f=[["enter",d,s],["enter",o[u][1],s],["exit",o[u][1],s],["enter",p,s]],m=s.parser.constructs.insideSpan.null;m&&Pi(f,f.length,0,E2(m,o.slice(u+1,l),s)),Pi(f,f.length,0,[["exit",p,s],["enter",o[l][1],s],["exit",o[l][1],s],["exit",d,s]]),Pi(o,u-1,l-u+3,f),l=u+f.length-2;break}}for(l=-1;++l<o.length;)o[l][1].type==="strikethroughSequenceTemporary"&&(o[l][1].type="data");return o}function i(o,s,l){const u=this.previous,d=this.events;let p=0;return f;function f(b){return u===126&&d[d.length-1][1].type!=="characterEscape"?l(b):(o.enter("strikethroughSequenceTemporary"),m(b))}function m(b){const y=Kh(u);if(b===126)return p>1?l(b):(o.consume(b),p++,m);if(p<2&&!r)return l(b);const S=o.exit("strikethroughSequenceTemporary"),w=Kh(b);return S._open=!w||w===2&&!!y,S._close=!y||y===2&&!!w,s(b)}}}class F8e{constructor(){this.map=[]}add(t,r,n){_8e(this,t,r,n)}consume(t){if(this.map.sort(function(i,o){return i[0]-o[0]}),this.map.length===0)return;let r=this.map.length;const n=[];for(;r>0;)r-=1,n.push(t.slice(this.map[r][0]+this.map[r][1]),this.map[r][2]),t.length=this.map[r][0];n.push(t.slice()),t.length=0;let a=n.pop();for(;a;){for(const i of a)t.push(i);a=n.pop()}this.map.length=0}}function _8e(e,t,r,n){let a=0;if(!(r===0&&n.length===0)){for(;a<e.map.length;){if(e.map[a][0]===t){e.map[a][1]+=r,e.map[a][2].push(...n);return}a+=1}e.map.push([t,r,n])}}function R8e(e,t){let r=!1;const n=[];for(;t<e.length;){const a=e[t];if(r){if(a[0]==="enter")a[1].type==="tableContent"&&n.push(e[t+1][1].type==="tableDelimiterMarker"?"left":"none");else if(a[1].type==="tableContent"){if(e[t-1][1].type==="tableDelimiterMarker"){const i=n.length-1;n[i]=n[i]==="left"?"center":"right"}}else if(a[1].type==="tableDelimiterRow")break}else a[0]==="enter"&&a[1].type==="tableDelimiterRow"&&(r=!0);t+=1}return n}function D8e(){return{flow:{null:{name:"table",tokenize:N8e,resolveAll:L8e}}}}function N8e(e,t,r){const n=this;let a=0,i=0,o;return s;function s(M){let j=n.events.length-1;for(;j>-1;){const $=n.events[j][1].type;if($==="lineEnding"||$==="linePrefix")j--;else break}const Y=j>-1?n.events[j][1].type:null,W=Y==="tableHead"||Y==="tableRow"?C:l;return W===C&&n.parser.lazy[n.now().line]?r(M):W(M)}function l(M){return e.enter("tableHead"),e.enter("tableRow"),u(M)}function u(M){return M===124||(o=!0,i+=1),d(M)}function d(M){return M===null?r(M):Ot(M)?i>1?(i=0,n.interrupt=!0,e.exit("tableRow"),e.enter("lineEnding"),e.consume(M),e.exit("lineEnding"),m):r(M):Tr(M)?xr(e,d,"whitespace")(M):(i+=1,o&&(o=!1,a+=1),M===124?(e.enter("tableCellDivider"),e.consume(M),e.exit("tableCellDivider"),o=!0,d):(e.enter("data"),p(M)))}function p(M){return M===null||M===124||rn(M)?(e.exit("data"),d(M)):(e.consume(M),M===92?f:p)}function f(M){return M===92||M===124?(e.consume(M),p):p(M)}function m(M){return n.interrupt=!1,n.parser.lazy[n.now().line]?r(M):(e.enter("tableDelimiterRow"),o=!1,Tr(M)?xr(e,b,"linePrefix",n.parser.constructs.disable.null.includes("codeIndented")?void 0:4)(M):b(M))}function b(M){return M===45||M===58?S(M):M===124?(o=!0,e.enter("tableCellDivider"),e.consume(M),e.exit("tableCellDivider"),y):R(M)}function y(M){return Tr(M)?xr(e,S,"whitespace")(M):S(M)}function S(M){return M===58?(i+=1,o=!0,e.enter("tableDelimiterMarker"),e.consume(M),e.exit("tableDelimiterMarker"),w):M===45?(i+=1,w(M)):M===null||Ot(M)?_(M):R(M)}function w(M){return M===45?(e.enter("tableDelimiterFiller"),E(M)):R(M)}function E(M){return M===45?(e.consume(M),E):M===58?(o=!0,e.exit("tableDelimiterFiller"),e.enter("tableDelimiterMarker"),e.consume(M),e.exit("tableDelimiterMarker"),k):(e.exit("tableDelimiterFiller"),k(M))}function k(M){return Tr(M)?xr(e,_,"whitespace")(M):_(M)}function _(M){return M===124?b(M):M===null||Ot(M)?!o||a!==i?R(M):(e.exit("tableDelimiterRow"),e.exit("tableHead"),t(M)):R(M)}function R(M){return r(M)}function C(M){return e.enter("tableRow"),F(M)}function F(M){return M===124?(e.enter("tableCellDivider"),e.consume(M),e.exit("tableCellDivider"),F):M===null||Ot(M)?(e.exit("tableRow"),t(M)):Tr(M)?xr(e,F,"whitespace")(M):(e.enter("data"),L(M))}function L(M){return M===null||M===124||rn(M)?(e.exit("data"),F(M)):(e.consume(M),M===92?I:L)}function I(M){return M===92||M===124?(e.consume(M),L):L(M)}}function L8e(e,t){let r=-1,n=!0,a=0,i=[0,0,0,0],o=[0,0,0,0],s=!1,l=0,u,d,p;const f=new F8e;for(;++r<e.length;){const m=e[r],b=m[1];m[0]==="enter"?b.type==="tableHead"?(s=!1,l!==0&&(iz(f,t,l,u,d),d=void 0,l=0),u={type:"table",start:Object.assign({},b.start),end:Object.assign({},b.end)},f.add(r,0,[["enter",u,t]])):b.type==="tableRow"||b.type==="tableDelimiterRow"?(n=!0,p=void 0,i=[0,0,0,0],o=[0,r+1,0,0],s&&(s=!1,d={type:"tableBody",start:Object.assign({},b.start),end:Object.assign({},b.end)},f.add(r,0,[["enter",d,t]])),a=b.type==="tableDelimiterRow"?2:d?3:1):a&&(b.type==="data"||b.type==="tableDelimiterMarker"||b.type==="tableDelimiterFiller")?(n=!1,o[2]===0&&(i[1]!==0&&(o[0]=o[1],p=Zb(f,t,i,a,void 0,p),i=[0,0,0,0]),o[2]=r)):b.type==="tableCellDivider"&&(n?n=!1:(i[1]!==0&&(o[0]=o[1],p=Zb(f,t,i,a,void 0,p)),i=o,o=[i[1],r,0,0])):b.type==="tableHead"?(s=!0,l=r):b.type==="tableRow"||b.type==="tableDelimiterRow"?(l=r,i[1]!==0?(o[0]=o[1],p=Zb(f,t,i,a,r,p)):o[1]!==0&&(p=Zb(f,t,o,a,r,p)),a=0):a&&(b.type==="data"||b.type==="tableDelimiterMarker"||b.type==="tableDelimiterFiller")&&(o[3]=r)}for(l!==0&&iz(f,t,l,u,d),f.consume(t.events),r=-1;++r<t.events.length;){const m=t.events[r];m[0]==="enter"&&m[1].type==="table"&&(m[1]._align=R8e(t.events,r))}return e}function Zb(e,t,r,n,a,i){const o=n===1?"tableHeader":n===2?"tableDelimiter":"tableData",s="tableContent";r[0]!==0&&(i.end=Object.assign({},hh(t.events,r[0])),e.add(r[0],0,[["exit",i,t]]));const l=hh(t.events,r[1]);if(i={type:o,start:Object.assign({},l),end:Object.assign({},l)},e.add(r[1],0,[["enter",i,t]]),r[2]!==0){const u=hh(t.events,r[2]),d=hh(t.events,r[3]),p={type:s,start:Object.assign({},u),end:Object.assign({},d)};if(e.add(r[2],0,[["enter",p,t]]),n!==2){const f=t.events[r[2]],m=t.events[r[3]];if(f[1].end=Object.assign({},m[1].end),f[1].type="chunkText",f[1].contentType="text",r[3]>r[2]+1){const b=r[2]+1,y=r[3]-r[2]-1;e.add(b,y,[])}}e.add(r[3]+1,0,[["exit",p,t]])}return a!==void 0&&(i.end=Object.assign({},hh(t.events,a)),e.add(a,0,[["exit",i,t]]),i=void 0),i}function iz(e,t,r,n,a){const i=[],o=hh(t.events,r);a&&(a.end=Object.assign({},o),i.push(["exit",a,t])),n.end=Object.assign({},o),i.push(["exit",n,t]),e.add(r+1,0,i)}function hh(e,t){const r=e[t],n=r[0]==="enter"?"start":"end";return r[1][n]}const M8e={name:"tasklistCheck",tokenize:O8e};function I8e(){return{text:{91:M8e}}}function O8e(e,t,r){const n=this;return a;function a(l){return n.previous!==null||!n._gfmTasklistFirstContentOfListItem?r(l):(e.enter("taskListCheck"),e.enter("taskListCheckMarker"),e.consume(l),e.exit("taskListCheckMarker"),i)}function i(l){return rn(l)?(e.enter("taskListCheckValueUnchecked"),e.consume(l),e.exit("taskListCheckValueUnchecked"),o):l===88||l===120?(e.enter("taskListCheckValueChecked"),e.consume(l),e.exit("taskListCheckValueChecked"),o):r(l)}function o(l){return l===93?(e.enter("taskListCheckMarker"),e.consume(l),e.exit("taskListCheckMarker"),e.exit("taskListCheck"),s):r(l)}function s(l){return Ot(l)?t(l):Tr(l)?e.check({tokenize:B8e},t,r)(l):r(l)}}function B8e(e,t,r){return xr(e,n,"whitespace");function n(a){return a===null?r(a):t(a)}}function P8e(e){return wre([u8e(),v8e(),A8e(e),D8e(),I8e()])}const z8e={};function oz(e){const t=this,r=e||z8e,n=t.data(),a=n.micromarkExtensions||(n.micromarkExtensions=[]),i=n.fromMarkdownExtensions||(n.fromMarkdownExtensions=[]),o=n.toMarkdownExtensions||(n.toMarkdownExtensions=[]);a.push(P8e(r)),i.push(i8e()),o.push(o8e(r))}function sz(e){const t=this;t.compiler=r;function r(n,a){return yre(n,{filePath:a.path,...e})}}class Bg{constructor(t,r,n){this.normal=r,this.property=t,n&&(this.space=n)}}Bg.prototype.normal={};Bg.prototype.property={};Bg.prototype.space=void 0;function mne(e,t){const r={},n={};for(const a of e)Object.assign(r,a.property),Object.assign(n,a.normal);return new Bg(r,n,t)}function Wm(e){return e.toLowerCase()}class vi{constructor(t,r){this.attribute=r,this.property=t}}vi.prototype.attribute="";vi.prototype.booleanish=!1;vi.prototype.boolean=!1;vi.prototype.commaOrSpaceSeparated=!1;vi.prototype.commaSeparated=!1;vi.prototype.defined=!1;vi.prototype.mustUseProperty=!1;vi.prototype.number=!1;vi.prototype.overloadedBoolean=!1;vi.prototype.property="";vi.prototype.spaceSeparated=!1;vi.prototype.space=void 0;let H8e=0;const gr=od(),Un=od(),WR=od(),ot=od(),on=od(),kh=od(),Ni=od();function od(){return 2**++H8e}const VR=Object.freeze(Object.defineProperty({__proto__:null,boolean:gr,booleanish:Un,commaOrSpaceSeparated:Ni,commaSeparated:kh,number:ot,overloadedBoolean:WR,spaceSeparated:on},Symbol.toStringTag,{value:"Module"})),xk=Object.keys(VR);class Q5 extends vi{constructor(t,r,n,a){let i=-1;if(super(t,r),lz(this,"space",a),typeof n=="number")for(;++i<xk.length;){const o=xk[i];lz(this,xk[i],(n&VR[o])===VR[o])}}}Q5.prototype.defined=!0;function lz(e,t,r){r&&(e[t]=r)}function Cp(e){const t={},r={};for(const[n,a]of Object.entries(e.properties)){const i=new Q5(n,e.transform(e.attributes||{},n),a,e.space);e.mustUseProperty&&e.mustUseProperty.includes(n)&&(i.mustUseProperty=!0),t[n]=i,r[Wm(n)]=n,r[Wm(i.attribute)]=n}return new Bg(t,r,e.space)}const gne=Cp({properties:{ariaActiveDescendant:null,ariaAtomic:Un,ariaAutoComplete:null,ariaBusy:Un,ariaChecked:Un,ariaColCount:ot,ariaColIndex:ot,ariaColSpan:ot,ariaControls:on,ariaCurrent:null,ariaDescribedBy:on,ariaDetails:null,ariaDisabled:Un,ariaDropEffect:on,ariaErrorMessage:null,ariaExpanded:Un,ariaFlowTo:on,ariaGrabbed:Un,ariaHasPopup:null,ariaHidden:Un,ariaInvalid:null,ariaKeyShortcuts:null,ariaLabel:null,ariaLabelledBy:on,ariaLevel:ot,ariaLive:null,ariaModal:Un,ariaMultiLine:Un,ariaMultiSelectable:Un,ariaOrientation:null,ariaOwns:on,ariaPlaceholder:null,ariaPosInSet:ot,ariaPressed:Un,ariaReadOnly:Un,ariaRelevant:null,ariaRequired:Un,ariaRoleDescription:on,ariaRowCount:ot,ariaRowIndex:ot,ariaRowSpan:ot,ariaSelected:Un,ariaSetSize:ot,ariaSort:null,ariaValueMax:ot,ariaValueMin:ot,ariaValueNow:ot,ariaValueText:null,role:null},transform(e,t){return t==="role"?t:"aria-"+t.slice(4).toLowerCase()}});function bne(e,t){return t in e?e[t]:t}function yne(e,t){return bne(e,t.toLowerCase())}const U8e=Cp({attributes:{acceptcharset:"accept-charset",classname:"class",htmlfor:"for",httpequiv:"http-equiv"},mustUseProperty:["checked","multiple","muted","selected"],properties:{abbr:null,accept:kh,acceptCharset:on,accessKey:on,action:null,allow:null,allowFullScreen:gr,allowPaymentRequest:gr,allowUserMedia:gr,alt:null,as:null,async:gr,autoCapitalize:null,autoComplete:on,autoFocus:gr,autoPlay:gr,blocking:on,capture:null,charSet:null,checked:gr,cite:null,className:on,cols:ot,colSpan:null,content:null,contentEditable:Un,controls:gr,controlsList:on,coords:ot|kh,crossOrigin:null,data:null,dateTime:null,decoding:null,default:gr,defer:gr,dir:null,dirName:null,disabled:gr,download:WR,draggable:Un,encType:null,enterKeyHint:null,fetchPriority:null,form:null,formAction:null,formEncType:null,formMethod:null,formNoValidate:gr,formTarget:null,headers:on,height:ot,hidden:WR,high:ot,href:null,hrefLang:null,htmlFor:on,httpEquiv:on,id:null,imageSizes:null,imageSrcSet:null,inert:gr,inputMode:null,integrity:null,is:null,isMap:gr,itemId:null,itemProp:on,itemRef:on,itemScope:gr,itemType:on,kind:null,label:null,lang:null,language:null,list:null,loading:null,loop:gr,low:ot,manifest:null,max:null,maxLength:ot,media:null,method:null,min:null,minLength:ot,multiple:gr,muted:gr,name:null,nonce:null,noModule:gr,noValidate:gr,onAbort:null,onAfterPrint:null,onAuxClick:null,onBeforeMatch:null,onBeforePrint:null,onBeforeToggle:null,onBeforeUnload:null,onBlur:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onContextLost:null,onContextMenu:null,onContextRestored:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnded:null,onError:null,onFocus:null,onFormData:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLanguageChange:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadEnd:null,onLoadStart:null,onMessage:null,onMessageError:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRejectionHandled:null,onReset:null,onResize:null,onScroll:null,onScrollEnd:null,onSecurityPolicyViolation:null,onSeeked:null,onSeeking:null,onSelect:null,onSlotChange:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnhandledRejection:null,onUnload:null,onVolumeChange:null,onWaiting:null,onWheel:null,open:gr,optimum:ot,pattern:null,ping:on,placeholder:null,playsInline:gr,popover:null,popoverTarget:null,popoverTargetAction:null,poster:null,preload:null,readOnly:gr,referrerPolicy:null,rel:on,required:gr,reversed:gr,rows:ot,rowSpan:ot,sandbox:on,scope:null,scoped:gr,seamless:gr,selected:gr,shadowRootClonable:gr,shadowRootDelegatesFocus:gr,shadowRootMode:null,shape:null,size:ot,sizes:null,slot:null,span:ot,spellCheck:Un,src:null,srcDoc:null,srcLang:null,srcSet:null,start:ot,step:null,style:null,tabIndex:ot,target:null,title:null,translate:null,type:null,typeMustMatch:gr,useMap:null,value:Un,width:ot,wrap:null,writingSuggestions:null,align:null,aLink:null,archive:on,axis:null,background:null,bgColor:null,border:ot,borderColor:null,bottomMargin:ot,cellPadding:null,cellSpacing:null,char:null,charOff:null,classId:null,clear:null,code:null,codeBase:null,codeType:null,color:null,compact:gr,declare:gr,event:null,face:null,frame:null,frameBorder:null,hSpace:ot,leftMargin:ot,link:null,longDesc:null,lowSrc:null,marginHeight:ot,marginWidth:ot,noResize:gr,noHref:gr,noShade:gr,noWrap:gr,object:null,profile:null,prompt:null,rev:null,rightMargin:ot,rules:null,scheme:null,scrolling:Un,standby:null,summary:null,text:null,topMargin:ot,valueType:null,version:null,vAlign:null,vLink:null,vSpace:ot,allowTransparency:null,autoCorrect:null,autoSave:null,disablePictureInPicture:gr,disableRemotePlayback:gr,prefix:null,property:null,results:ot,security:null,unselectable:null},space:"html",transform:yne}),$8e=Cp({attributes:{accentHeight:"accent-height",alignmentBaseline:"alignment-baseline",arabicForm:"arabic-form",baselineShift:"baseline-shift",capHeight:"cap-height",className:"class",clipPath:"clip-path",clipRule:"clip-rule",colorInterpolation:"color-interpolation",colorInterpolationFilters:"color-interpolation-filters",colorProfile:"color-profile",colorRendering:"color-rendering",crossOrigin:"crossorigin",dataType:"datatype",dominantBaseline:"dominant-baseline",enableBackground:"enable-background",fillOpacity:"fill-opacity",fillRule:"fill-rule",floodColor:"flood-color",floodOpacity:"flood-opacity",fontFamily:"font-family",fontSize:"font-size",fontSizeAdjust:"font-size-adjust",fontStretch:"font-stretch",fontStyle:"font-style",fontVariant:"font-variant",fontWeight:"font-weight",glyphName:"glyph-name",glyphOrientationHorizontal:"glyph-orientation-horizontal",glyphOrientationVertical:"glyph-orientation-vertical",hrefLang:"hreflang",horizAdvX:"horiz-adv-x",horizOriginX:"horiz-origin-x",horizOriginY:"horiz-origin-y",imageRendering:"image-rendering",letterSpacing:"letter-spacing",lightingColor:"lighting-color",markerEnd:"marker-end",markerMid:"marker-mid",markerStart:"marker-start",navDown:"nav-down",navDownLeft:"nav-down-left",navDownRight:"nav-down-right",navLeft:"nav-left",navNext:"nav-next",navPrev:"nav-prev",navRight:"nav-right",navUp:"nav-up",navUpLeft:"nav-up-left",navUpRight:"nav-up-right",onAbort:"onabort",onActivate:"onactivate",onAfterPrint:"onafterprint",onBeforePrint:"onbeforeprint",onBegin:"onbegin",onCancel:"oncancel",onCanPlay:"oncanplay",onCanPlayThrough:"oncanplaythrough",onChange:"onchange",onClick:"onclick",onClose:"onclose",onCopy:"oncopy",onCueChange:"oncuechange",onCut:"oncut",onDblClick:"ondblclick",onDrag:"ondrag",onDragEnd:"ondragend",onDragEnter:"ondragenter",onDragExit:"ondragexit",onDragLeave:"ondragleave",onDragOver:"ondragover",onDragStart:"ondragstart",onDrop:"ondrop",onDurationChange:"ondurationchange",onEmptied:"onemptied",onEnd:"onend",onEnded:"onended",onError:"onerror",onFocus:"onfocus",onFocusIn:"onfocusin",onFocusOut:"onfocusout",onHashChange:"onhashchange",onInput:"oninput",onInvalid:"oninvalid",onKeyDown:"onkeydown",onKeyPress:"onkeypress",onKeyUp:"onkeyup",onLoad:"onload",onLoadedData:"onloadeddata",onLoadedMetadata:"onloadedmetadata",onLoadStart:"onloadstart",onMessage:"onmessage",onMouseDown:"onmousedown",onMouseEnter:"onmouseenter",onMouseLeave:"onmouseleave",onMouseMove:"onmousemove",onMouseOut:"onmouseout",onMouseOver:"onmouseover",onMouseUp:"onmouseup",onMouseWheel:"onmousewheel",onOffline:"onoffline",onOnline:"ononline",onPageHide:"onpagehide",onPageShow:"onpageshow",onPaste:"onpaste",onPause:"onpause",onPlay:"onplay",onPlaying:"onplaying",onPopState:"onpopstate",onProgress:"onprogress",onRateChange:"onratechange",onRepeat:"onrepeat",onReset:"onreset",onResize:"onresize",onScroll:"onscroll",onSeeked:"onseeked",onSeeking:"onseeking",onSelect:"onselect",onShow:"onshow",onStalled:"onstalled",onStorage:"onstorage",onSubmit:"onsubmit",onSuspend:"onsuspend",onTimeUpdate:"ontimeupdate",onToggle:"ontoggle",onUnload:"onunload",onVolumeChange:"onvolumechange",onWaiting:"onwaiting",onZoom:"onzoom",overlinePosition:"overline-position",overlineThickness:"overline-thickness",paintOrder:"paint-order",panose1:"panose-1",pointerEvents:"pointer-events",referrerPolicy:"referrerpolicy",renderingIntent:"rendering-intent",shapeRendering:"shape-rendering",stopColor:"stop-color",stopOpacity:"stop-opacity",strikethroughPosition:"strikethrough-position",strikethroughThickness:"strikethrough-thickness",strokeDashArray:"stroke-dasharray",strokeDashOffset:"stroke-dashoffset",strokeLineCap:"stroke-linecap",strokeLineJoin:"stroke-linejoin",strokeMiterLimit:"stroke-miterlimit",strokeOpacity:"stroke-opacity",strokeWidth:"stroke-width",tabIndex:"tabindex",textAnchor:"text-anchor",textDecoration:"text-decoration",textRendering:"text-rendering",transformOrigin:"transform-origin",typeOf:"typeof",underlinePosition:"underline-position",underlineThickness:"underline-thickness",unicodeBidi:"unicode-bidi",unicodeRange:"unicode-range",unitsPerEm:"units-per-em",vAlphabetic:"v-alphabetic",vHanging:"v-hanging",vIdeographic:"v-ideographic",vMathematical:"v-mathematical",vectorEffect:"vector-effect",vertAdvY:"vert-adv-y",vertOriginX:"vert-origin-x",vertOriginY:"vert-origin-y",wordSpacing:"word-spacing",writingMode:"writing-mode",xHeight:"x-height",playbackOrder:"playbackorder",timelineBegin:"timelinebegin"},properties:{about:Ni,accentHeight:ot,accumulate:null,additive:null,alignmentBaseline:null,alphabetic:ot,amplitude:ot,arabicForm:null,ascent:ot,attributeName:null,attributeType:null,azimuth:ot,bandwidth:null,baselineShift:null,baseFrequency:null,baseProfile:null,bbox:null,begin:null,bias:ot,by:null,calcMode:null,capHeight:ot,className:on,clip:null,clipPath:null,clipPathUnits:null,clipRule:null,color:null,colorInterpolation:null,colorInterpolationFilters:null,colorProfile:null,colorRendering:null,content:null,contentScriptType:null,contentStyleType:null,crossOrigin:null,cursor:null,cx:null,cy:null,d:null,dataType:null,defaultAction:null,descent:ot,diffuseConstant:ot,direction:null,display:null,dur:null,divisor:ot,dominantBaseline:null,download:gr,dx:null,dy:null,edgeMode:null,editable:null,elevation:ot,enableBackground:null,end:null,event:null,exponent:ot,externalResourcesRequired:null,fill:null,fillOpacity:ot,fillRule:null,filter:null,filterRes:null,filterUnits:null,floodColor:null,floodOpacity:null,focusable:null,focusHighlight:null,fontFamily:null,fontSize:null,fontSizeAdjust:null,fontStretch:null,fontStyle:null,fontVariant:null,fontWeight:null,format:null,fr:null,from:null,fx:null,fy:null,g1:kh,g2:kh,glyphName:kh,glyphOrientationHorizontal:null,glyphOrientationVertical:null,glyphRef:null,gradientTransform:null,gradientUnits:null,handler:null,hanging:ot,hatchContentUnits:null,hatchUnits:null,height:null,href:null,hrefLang:null,horizAdvX:ot,horizOriginX:ot,horizOriginY:ot,id:null,ideographic:ot,imageRendering:null,initialVisibility:null,in:null,in2:null,intercept:ot,k:ot,k1:ot,k2:ot,k3:ot,k4:ot,kernelMatrix:Ni,kernelUnitLength:null,keyPoints:null,keySplines:null,keyTimes:null,kerning:null,lang:null,lengthAdjust:null,letterSpacing:null,lightingColor:null,limitingConeAngle:ot,local:null,markerEnd:null,markerMid:null,markerStart:null,markerHeight:null,markerUnits:null,markerWidth:null,mask:null,maskContentUnits:null,maskUnits:null,mathematical:null,max:null,media:null,mediaCharacterEncoding:null,mediaContentEncodings:null,mediaSize:ot,mediaTime:null,method:null,min:null,mode:null,name:null,navDown:null,navDownLeft:null,navDownRight:null,navLeft:null,navNext:null,navPrev:null,navRight:null,navUp:null,navUpLeft:null,navUpRight:null,numOctaves:null,observer:null,offset:null,onAbort:null,onActivate:null,onAfterPrint:null,onBeforePrint:null,onBegin:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnd:null,onEnded:null,onError:null,onFocus:null,onFocusIn:null,onFocusOut:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadStart:null,onMessage:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onMouseWheel:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRepeat:null,onReset:null,onResize:null,onScroll:null,onSeeked:null,onSeeking:null,onSelect:null,onShow:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnload:null,onVolumeChange:null,onWaiting:null,onZoom:null,opacity:null,operator:null,order:null,orient:null,orientation:null,origin:null,overflow:null,overlay:null,overlinePosition:ot,overlineThickness:ot,paintOrder:null,panose1:null,path:null,pathLength:ot,patternContentUnits:null,patternTransform:null,patternUnits:null,phase:null,ping:on,pitch:null,playbackOrder:null,pointerEvents:null,points:null,pointsAtX:ot,pointsAtY:ot,pointsAtZ:ot,preserveAlpha:null,preserveAspectRatio:null,primitiveUnits:null,propagate:null,property:Ni,r:null,radius:null,referrerPolicy:null,refX:null,refY:null,rel:Ni,rev:Ni,renderingIntent:null,repeatCount:null,repeatDur:null,requiredExtensions:Ni,requiredFeatures:Ni,requiredFonts:Ni,requiredFormats:Ni,resource:null,restart:null,result:null,rotate:null,rx:null,ry:null,scale:null,seed:null,shapeRendering:null,side:null,slope:null,snapshotTime:null,specularConstant:ot,specularExponent:ot,spreadMethod:null,spacing:null,startOffset:null,stdDeviation:null,stemh:null,stemv:null,stitchTiles:null,stopColor:null,stopOpacity:null,strikethroughPosition:ot,strikethroughThickness:ot,string:null,stroke:null,strokeDashArray:Ni,strokeDashOffset:null,strokeLineCap:null,strokeLineJoin:null,strokeMiterLimit:ot,strokeOpacity:ot,strokeWidth:null,style:null,surfaceScale:ot,syncBehavior:null,syncBehaviorDefault:null,syncMaster:null,syncTolerance:null,syncToleranceDefault:null,systemLanguage:Ni,tabIndex:ot,tableValues:null,target:null,targetX:ot,targetY:ot,textAnchor:null,textDecoration:null,textRendering:null,textLength:null,timelineBegin:null,title:null,transformBehavior:null,type:null,typeOf:Ni,to:null,transform:null,transformOrigin:null,u1:null,u2:null,underlinePosition:ot,underlineThickness:ot,unicode:null,unicodeBidi:null,unicodeRange:null,unitsPerEm:ot,values:null,vAlphabetic:ot,vMathematical:ot,vectorEffect:null,vHanging:ot,vIdeographic:ot,version:null,vertAdvY:ot,vertOriginX:ot,vertOriginY:ot,viewBox:null,viewTarget:null,visibility:null,width:null,widths:null,wordSpacing:null,writingMode:null,x:null,x1:null,x2:null,xChannelSelector:null,xHeight:ot,y:null,y1:null,y2:null,yChannelSelector:null,z:null,zoomAndPan:null},space:"svg",transform:bne}),vne=Cp({properties:{xLinkActuate:null,xLinkArcRole:null,xLinkHref:null,xLinkRole:null,xLinkShow:null,xLinkTitle:null,xLinkType:null},space:"xlink",transform(e,t){return"xlink:"+t.slice(5).toLowerCase()}}),Sne=Cp({attributes:{xmlnsxlink:"xmlns:xlink"},properties:{xmlnsXLink:null,xmlns:null},space:"xmlns",transform:yne}),Ene=Cp({properties:{xmlBase:null,xmlLang:null,xmlSpace:null},space:"xml",transform(e,t){return"xml:"+t.slice(3).toLowerCase()}}),G8e=/[A-Z]/g,uz=/-[a-z]/g,j8e=/^data[-\w.:]+$/i;function xne(e,t){const r=Wm(t);let n=t,a=vi;if(r in e.normal)return e.property[e.normal[r]];if(r.length>4&&r.slice(0,4)==="data"&&j8e.test(t)){if(t.charAt(4)==="-"){const i=t.slice(5).replace(uz,W8e);n="data"+i.charAt(0).toUpperCase()+i.slice(1)}else{const i=t.slice(4);if(!uz.test(i)){let o=i.replace(G8e,q8e);o.charAt(0)!=="-"&&(o="-"+o),t="data"+o}}a=Q5}return new a(n,t)}function q8e(e){return"-"+e.toLowerCase()}function W8e(e){return e.charAt(1).toUpperCase()}const eN=mne([gne,U8e,vne,Sne,Ene],"html"),tN=mne([gne,$8e,vne,Sne,Ene],"svg"),cz=/[#.]/g;function V8e(e,t){const r=e||"",n={};let a=0,i,o;for(;a<r.length;){cz.lastIndex=a;const s=cz.exec(r),l=r.slice(a,s?s.index:r.length);l&&(i?i==="#"?n.id=l:Array.isArray(n.className)?n.className.push(l):n.className=[l]:o=l,a+=l.length),s&&(i=s[0],a++)}return{type:"element",tagName:o||t||"div",properties:n,children:[]}}function wne(e,t,r){const n=r?X8e(r):void 0;function a(i,o,...s){let l;if(i==null){l={type:"root",children:[]};const u=o;s.unshift(u)}else{l=V8e(i,t);const u=l.tagName.toLowerCase(),d=n?n.get(u):void 0;if(l.tagName=d||u,K8e(o))s.unshift(o);else for(const[p,f]of Object.entries(o))Y8e(e,l.properties,p,f)}for(const u of s)KR(l.children,u);return l.type==="element"&&l.tagName==="template"&&(l.content={type:"root",children:l.children},l.children=[]),l}return a}function K8e(e){if(e===null||typeof e!="object"||Array.isArray(e))return!0;if(typeof e.type!="string")return!1;const t=e,r=Object.keys(e);for(const n of r){const a=t[n];if(a&&typeof a=="object"){if(!Array.isArray(a))return!0;const i=a;for(const o of i)if(typeof o!="number"&&typeof o!="string")return!0}}return!!("children"in e&&Array.isArray(e.children))}function Y8e(e,t,r,n){const a=xne(e,r);let i;if(n!=null){if(typeof n=="number"){if(Number.isNaN(n))return;i=n}else typeof n=="boolean"?i=n:typeof n=="string"?a.spaceSeparated?i=vP(n):a.commaSeparated?i=fP(n):a.commaOrSpaceSeparated?i=vP(fP(n).join(" ")):i=dz(a,a.property,n):Array.isArray(n)?i=[...n]:i=a.property==="style"?J8e(n):String(n);if(Array.isArray(i)){const o=[];for(const s of i)o.push(dz(a,a.property,s));i=o}a.property==="className"&&Array.isArray(t.className)&&(i=t.className.concat(i)),t[a.property]=i}}function KR(e,t){if(t!=null)if(typeof t=="number"||typeof t=="string")e.push({type:"text",value:String(t)});else if(Array.isArray(t))for(const r of t)KR(e,r);else if(typeof t=="object"&&"type"in t)t.type==="root"?KR(e,t.children):e.push(t);else throw new Error("Expected node, nodes, or string, got `"+t+"`")}function dz(e,t,r){if(typeof r=="string"){if(e.number&&r&&!Number.isNaN(Number(r)))return Number(r);if((e.boolean||e.overloadedBoolean)&&(r===""||Wm(r)===Wm(t)))return!0}return r}function J8e(e){const t=[];for(const[r,n]of Object.entries(e))t.push([r,n].join(": "));return t.join("; ")}function X8e(e){const t=new Map;for(const r of e)t.set(r.toLowerCase(),r);return t}const Z8e=["altGlyph","altGlyphDef","altGlyphItem","animateColor","animateMotion","animateTransform","clipPath","feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feDropShadow","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence","foreignObject","glyphRef","linearGradient","radialGradient","solidColor","textArea","textPath"],Q8e=wne(eN,"div"),e7e=wne(tN,"g",Z8e);function t7e(e){const t=String(e),r=[];return{toOffset:a,toPoint:n};function n(i){if(typeof i=="number"&&i>-1&&i<=t.length){let o=0;for(;;){let s=r[o];if(s===void 0){const l=hz(t,r[o-1]);s=l===-1?t.length+1:l+1,r[o]=s}if(s>i)return{line:o+1,column:i-(o>0?r[o-1]:0)+1,offset:i};o++}}}function a(i){if(i&&typeof i.line=="number"&&typeof i.column=="number"&&!Number.isNaN(i.line)&&!Number.isNaN(i.column)){for(;r.length<i.line;){const s=r[r.length-1],l=hz(t,s),u=l===-1?t.length+1:l+1;if(s===u)break;r.push(u)}const o=(i.line>1?r[i.line-2]:0)+i.column-1;if(o<r[i.line-1])return o}}}function hz(e,t){const r=e.indexOf("\r",t),n=e.indexOf(`
`,t);return n===-1?r:r===-1||r+1===n?n:r<n?r:n}const xc={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"},kne={}.hasOwnProperty,r7e=Object.prototype;function n7e(e,t){const r=t||{};return rN({file:r.file||void 0,location:!1,schema:r.space==="svg"?tN:eN,verbose:r.verbose||!1},e)}function rN(e,t){let r;switch(t.nodeName){case"#comment":{const n=t;return r={type:"comment",value:n.data},O1(e,n,r),r}case"#document":case"#document-fragment":{const n=t,a="mode"in n?n.mode==="quirks"||n.mode==="limited-quirks":!1;if(r={type:"root",children:Cne(e,t.childNodes),data:{quirksMode:a}},e.file&&e.location){const i=String(e.file),o=t7e(i),s=o.toPoint(0),l=o.toPoint(i.length);r.position={start:s,end:l}}return r}case"#documentType":{const n=t;return r={type:"doctype"},O1(e,n,r),r}case"#text":{const n=t;return r={type:"text",value:n.value},O1(e,n,r),r}default:return r=a7e(e,t),r}}function Cne(e,t){let r=-1;const n=[];for(;++r<t.length;){const a=rN(e,t[r]);n.push(a)}return n}function a7e(e,t){const r=e.schema;e.schema=t.namespaceURI===xc.svg?tN:eN;let n=-1;const a={};for(;++n<t.attrs.length;){const s=t.attrs[n],l=(s.prefix?s.prefix+":":"")+s.name;kne.call(r7e,l)||(a[l]=s.value)}const o=(e.schema.space==="svg"?e7e:Q8e)(t.tagName,a,Cne(e,t.childNodes));if(O1(e,t,o),o.tagName==="template"){const s=t,l=s.sourceCodeLocation,u=l&&l.startTag&&mh(l.startTag),d=l&&l.endTag&&mh(l.endTag),p=rN(e,s.content);u&&d&&e.file&&(p.position={start:u.end,end:d.start}),o.content=p}return e.schema=r,o}function O1(e,t,r){if("sourceCodeLocation"in t&&t.sourceCodeLocation&&e.file){const n=i7e(e,r,t.sourceCodeLocation);n&&(e.location=!0,r.position=n)}}function i7e(e,t,r){const n=mh(r);if(t.type==="element"){const a=t.children[t.children.length-1];if(n&&!r.endTag&&a&&a.position&&a.position.end&&(n.end=Object.assign({},a.position.end)),e.verbose){const i={};let o;if(r.attrs)for(o in r.attrs)kne.call(r.attrs,o)&&(i[xne(e.schema,o).property]=mh(r.attrs[o]));r.startTag;const s=mh(r.startTag),l=r.endTag?mh(r.endTag):void 0,u={opening:s};l&&(u.closing=l),u.properties=i,t.data={position:u}}}return n}function mh(e){const t=pz({line:e.startLine,column:e.startCol,offset:e.startOffset}),r=pz({line:e.endLine,column:e.endCol,offset:e.endOffset});return t||r?{start:t,end:r}:void 0}function pz(e){return e.line&&e.column?e:void 0}const o7e={},s7e={}.hasOwnProperty,Tne=Kre("type",{handlers:{root:u7e,element:f7e,text:h7e,comment:p7e,doctype:d7e}});function l7e(e,t){const n=(t||o7e).space;return Tne(e,n==="svg"?Lg:fre)}function u7e(e,t){const r={nodeName:"#document",mode:(e.data||{}).quirksMode?"quirks":"no-quirks",childNodes:[]};return r.childNodes=nN(e.children,r,t),Tp(e,r),r}function c7e(e,t){const r={nodeName:"#document-fragment",childNodes:[]};return r.childNodes=nN(e.children,r,t),Tp(e,r),r}function d7e(e){const t={nodeName:"#documentType",name:"html",publicId:"",systemId:"",parentNode:null};return Tp(e,t),t}function h7e(e){const t={nodeName:"#text",value:e.value,parentNode:null};return Tp(e,t),t}function p7e(e){const t={nodeName:"#comment",data:e.value,parentNode:null};return Tp(e,t),t}function f7e(e,t){const r=t;let n=r;e.type==="element"&&e.tagName.toLowerCase()==="svg"&&r.space==="html"&&(n=Lg);const a=[];let i;if(e.properties){for(i in e.properties)if(i!=="children"&&s7e.call(e.properties,i)){const l=m7e(n,i,e.properties[i]);l&&a.push(l)}}const o=n.space,s={nodeName:e.tagName,tagName:e.tagName,attrs:a,namespaceURI:xc[o],childNodes:[],parentNode:null};return s.childNodes=nN(e.children,s,n),Tp(e,s),e.tagName==="template"&&e.content&&(s.content=c7e(e.content,n)),s}function m7e(e,t,r){const n=pre(e,t);if(r===!1||r===null||r===void 0||typeof r=="number"&&Number.isNaN(r)||!r&&n.boolean)return;Array.isArray(r)&&(r=n.commaSeparated?are(r):mre(r));const a={name:n.attribute,value:r===!0?"":String(r)};if(n.space&&n.space!=="html"&&n.space!=="svg"){const i=a.name.indexOf(":");i<0?a.prefix="":(a.name=a.name.slice(i+1),a.prefix=n.attribute.slice(0,i)),a.namespace=xc[n.space]}return a}function nN(e,t,r){let n=-1;const a=[];if(e)for(;++n<e.length;){const i=Tne(e[n],r);i.parentNode=t,a.push(i)}return a}function Tp(e,t){const r=e.position;r&&r.start&&r.end&&(r.start.offset,r.end.offset,t.sourceCodeLocation={startLine:r.start.line,startCol:r.start.column,startOffset:r.start.offset,endLine:r.end.line,endCol:r.end.column,endOffset:r.end.offset})}const g7e=["area","base","basefont","bgsound","br","col","command","embed","frame","hr","image","img","input","keygen","link","meta","param","source","track","wbr"],b7e=new Set([65534,65535,131070,131071,196606,196607,262142,262143,327678,327679,393214,393215,458750,458751,524286,524287,589822,589823,655358,655359,720894,720895,786430,786431,851966,851967,917502,917503,983038,983039,1048574,1048575,1114110,1114111]),mn="�";var re;(function(e){e[e.EOF=-1]="EOF",e[e.NULL=0]="NULL",e[e.TABULATION=9]="TABULATION",e[e.CARRIAGE_RETURN=13]="CARRIAGE_RETURN",e[e.LINE_FEED=10]="LINE_FEED",e[e.FORM_FEED=12]="FORM_FEED",e[e.SPACE=32]="SPACE",e[e.EXCLAMATION_MARK=33]="EXCLAMATION_MARK",e[e.QUOTATION_MARK=34]="QUOTATION_MARK",e[e.AMPERSAND=38]="AMPERSAND",e[e.APOSTROPHE=39]="APOSTROPHE",e[e.HYPHEN_MINUS=45]="HYPHEN_MINUS",e[e.SOLIDUS=47]="SOLIDUS",e[e.DIGIT_0=48]="DIGIT_0",e[e.DIGIT_9=57]="DIGIT_9",e[e.SEMICOLON=59]="SEMICOLON",e[e.LESS_THAN_SIGN=60]="LESS_THAN_SIGN",e[e.EQUALS_SIGN=61]="EQUALS_SIGN",e[e.GREATER_THAN_SIGN=62]="GREATER_THAN_SIGN",e[e.QUESTION_MARK=63]="QUESTION_MARK",e[e.LATIN_CAPITAL_A=65]="LATIN_CAPITAL_A",e[e.LATIN_CAPITAL_Z=90]="LATIN_CAPITAL_Z",e[e.RIGHT_SQUARE_BRACKET=93]="RIGHT_SQUARE_BRACKET",e[e.GRAVE_ACCENT=96]="GRAVE_ACCENT",e[e.LATIN_SMALL_A=97]="LATIN_SMALL_A",e[e.LATIN_SMALL_Z=122]="LATIN_SMALL_Z"})(re||(re={}));const ai={DASH_DASH:"--",CDATA_START:"[CDATA[",DOCTYPE:"doctype",SCRIPT:"script",PUBLIC:"public",SYSTEM:"system"};function Ane(e){return e>=55296&&e<=57343}function y7e(e){return e>=56320&&e<=57343}function v7e(e,t){return(e-55296)*1024+9216+t}function Fne(e){return e!==32&&e!==10&&e!==13&&e!==9&&e!==12&&e>=1&&e<=31||e>=127&&e<=159}function _ne(e){return e>=64976&&e<=65007||b7e.has(e)}var Be;(function(e){e.controlCharacterInInputStream="control-character-in-input-stream",e.noncharacterInInputStream="noncharacter-in-input-stream",e.surrogateInInputStream="surrogate-in-input-stream",e.nonVoidHtmlElementStartTagWithTrailingSolidus="non-void-html-element-start-tag-with-trailing-solidus",e.endTagWithAttributes="end-tag-with-attributes",e.endTagWithTrailingSolidus="end-tag-with-trailing-solidus",e.unexpectedSolidusInTag="unexpected-solidus-in-tag",e.unexpectedNullCharacter="unexpected-null-character",e.unexpectedQuestionMarkInsteadOfTagName="unexpected-question-mark-instead-of-tag-name",e.invalidFirstCharacterOfTagName="invalid-first-character-of-tag-name",e.unexpectedEqualsSignBeforeAttributeName="unexpected-equals-sign-before-attribute-name",e.missingEndTagName="missing-end-tag-name",e.unexpectedCharacterInAttributeName="unexpected-character-in-attribute-name",e.unknownNamedCharacterReference="unknown-named-character-reference",e.missingSemicolonAfterCharacterReference="missing-semicolon-after-character-reference",e.unexpectedCharacterAfterDoctypeSystemIdentifier="unexpected-character-after-doctype-system-identifier",e.unexpectedCharacterInUnquotedAttributeValue="unexpected-character-in-unquoted-attribute-value",e.eofBeforeTagName="eof-before-tag-name",e.eofInTag="eof-in-tag",e.missingAttributeValue="missing-attribute-value",e.missingWhitespaceBetweenAttributes="missing-whitespace-between-attributes",e.missingWhitespaceAfterDoctypePublicKeyword="missing-whitespace-after-doctype-public-keyword",e.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers="missing-whitespace-between-doctype-public-and-system-identifiers",e.missingWhitespaceAfterDoctypeSystemKeyword="missing-whitespace-after-doctype-system-keyword",e.missingQuoteBeforeDoctypePublicIdentifier="missing-quote-before-doctype-public-identifier",e.missingQuoteBeforeDoctypeSystemIdentifier="missing-quote-before-doctype-system-identifier",e.missingDoctypePublicIdentifier="missing-doctype-public-identifier",e.missingDoctypeSystemIdentifier="missing-doctype-system-identifier",e.abruptDoctypePublicIdentifier="abrupt-doctype-public-identifier",e.abruptDoctypeSystemIdentifier="abrupt-doctype-system-identifier",e.cdataInHtmlContent="cdata-in-html-content",e.incorrectlyOpenedComment="incorrectly-opened-comment",e.eofInScriptHtmlCommentLikeText="eof-in-script-html-comment-like-text",e.eofInDoctype="eof-in-doctype",e.nestedComment="nested-comment",e.abruptClosingOfEmptyComment="abrupt-closing-of-empty-comment",e.eofInComment="eof-in-comment",e.incorrectlyClosedComment="incorrectly-closed-comment",e.eofInCdata="eof-in-cdata",e.absenceOfDigitsInNumericCharacterReference="absence-of-digits-in-numeric-character-reference",e.nullCharacterReference="null-character-reference",e.surrogateCharacterReference="surrogate-character-reference",e.characterReferenceOutsideUnicodeRange="character-reference-outside-unicode-range",e.controlCharacterReference="control-character-reference",e.noncharacterCharacterReference="noncharacter-character-reference",e.missingWhitespaceBeforeDoctypeName="missing-whitespace-before-doctype-name",e.missingDoctypeName="missing-doctype-name",e.invalidCharacterSequenceAfterDoctypeName="invalid-character-sequence-after-doctype-name",e.duplicateAttribute="duplicate-attribute",e.nonConformingDoctype="non-conforming-doctype",e.missingDoctype="missing-doctype",e.misplacedDoctype="misplaced-doctype",e.endTagWithoutMatchingOpenElement="end-tag-without-matching-open-element",e.closingOfElementWithOpenChildElements="closing-of-element-with-open-child-elements",e.disallowedContentInNoscriptInHead="disallowed-content-in-noscript-in-head",e.openElementsLeftAfterEof="open-elements-left-after-eof",e.abandonedHeadElementChild="abandoned-head-element-child",e.misplacedStartTagForHeadElement="misplaced-start-tag-for-head-element",e.nestedNoscriptInHead="nested-noscript-in-head",e.eofInElementThatCanContainOnlyText="eof-in-element-that-can-contain-only-text"})(Be||(Be={}));const S7e=65536;class E7e{constructor(t){this.handler=t,this.html="",this.pos=-1,this.lastGapPos=-2,this.gapStack=[],this.skipNextNewLine=!1,this.lastChunkWritten=!1,this.endOfChunkHit=!1,this.bufferWaterline=S7e,this.isEol=!1,this.lineStartPos=0,this.droppedBufferSize=0,this.line=1,this.lastErrOffset=-1}get col(){return this.pos-this.lineStartPos+ +(this.lastGapPos!==this.pos)}get offset(){return this.droppedBufferSize+this.pos}getError(t,r){const{line:n,col:a,offset:i}=this,o=a+r,s=i+r;return{code:t,startLine:n,endLine:n,startCol:o,endCol:o,startOffset:s,endOffset:s}}_err(t){this.handler.onParseError&&this.lastErrOffset!==this.offset&&(this.lastErrOffset=this.offset,this.handler.onParseError(this.getError(t,0)))}_addGap(){this.gapStack.push(this.lastGapPos),this.lastGapPos=this.pos}_processSurrogate(t){if(this.pos!==this.html.length-1){const r=this.html.charCodeAt(this.pos+1);if(y7e(r))return this.pos++,this._addGap(),v7e(t,r)}else if(!this.lastChunkWritten)return this.endOfChunkHit=!0,re.EOF;return this._err(Be.surrogateInInputStream),t}willDropParsedChunk(){return this.pos>this.bufferWaterline}dropParsedChunk(){this.willDropParsedChunk()&&(this.html=this.html.substring(this.pos),this.lineStartPos-=this.pos,this.droppedBufferSize+=this.pos,this.pos=0,this.lastGapPos=-2,this.gapStack.length=0)}write(t,r){this.html.length>0?this.html+=t:this.html=t,this.endOfChunkHit=!1,this.lastChunkWritten=r}insertHtmlAtCurrentPos(t){this.html=this.html.substring(0,this.pos+1)+t+this.html.substring(this.pos+1),this.endOfChunkHit=!1}startsWith(t,r){if(this.pos+t.length>this.html.length)return this.endOfChunkHit=!this.lastChunkWritten,!1;if(r)return this.html.startsWith(t,this.pos);for(let n=0;n<t.length;n++)if((this.html.charCodeAt(this.pos+n)|32)!==t.charCodeAt(n))return!1;return!0}peek(t){const r=this.pos+t;if(r>=this.html.length)return this.endOfChunkHit=!this.lastChunkWritten,re.EOF;const n=this.html.charCodeAt(r);return n===re.CARRIAGE_RETURN?re.LINE_FEED:n}advance(){if(this.pos++,this.isEol&&(this.isEol=!1,this.line++,this.lineStartPos=this.pos),this.pos>=this.html.length)return this.endOfChunkHit=!this.lastChunkWritten,re.EOF;let t=this.html.charCodeAt(this.pos);return t===re.CARRIAGE_RETURN?(this.isEol=!0,this.skipNextNewLine=!0,re.LINE_FEED):t===re.LINE_FEED&&(this.isEol=!0,this.skipNextNewLine)?(this.line--,this.skipNextNewLine=!1,this._addGap(),this.advance()):(this.skipNextNewLine=!1,Ane(t)&&(t=this._processSurrogate(t)),this.handler.onParseError===null||t>31&&t<127||t===re.LINE_FEED||t===re.CARRIAGE_RETURN||t>159&&t<64976||this._checkForProblematicCharacters(t),t)}_checkForProblematicCharacters(t){Fne(t)?this._err(Be.controlCharacterInInputStream):_ne(t)&&this._err(Be.noncharacterInInputStream)}retreat(t){for(this.pos-=t;this.pos<this.lastGapPos;)this.lastGapPos=this.gapStack.pop(),this.pos--;this.isEol=!1}}var Er;(function(e){e[e.CHARACTER=0]="CHARACTER",e[e.NULL_CHARACTER=1]="NULL_CHARACTER",e[e.WHITESPACE_CHARACTER=2]="WHITESPACE_CHARACTER",e[e.START_TAG=3]="START_TAG",e[e.END_TAG=4]="END_TAG",e[e.COMMENT=5]="COMMENT",e[e.DOCTYPE=6]="DOCTYPE",e[e.EOF=7]="EOF",e[e.HIBERNATION=8]="HIBERNATION"})(Er||(Er={}));function Rne(e,t){for(let r=e.attrs.length-1;r>=0;r--)if(e.attrs[r].name===t)return e.attrs[r].value;return null}const x7e=new Uint16Array('ᵁ<Õıʊҝջאٵ۞ޢߖࠏ੊ઑඡ๭༉༦჊ረዡᐕᒝᓃᓟᔥ\0\0\0\0\0\0ᕫᛍᦍᰒᷝ὾⁠↰⊍⏀⏻⑂⠤⤒ⴈ⹈⿎〖㊺㘹㞬㣾㨨㩱㫠㬮ࠀEMabcfglmnoprstu\\bfms¦³¹ÈÏlig耻Æ䃆P耻&䀦cute耻Á䃁reve;䄂Āiyx}rc耻Â䃂;䐐r;쀀𝔄rave耻À䃀pha;䎑acr;䄀d;橓Āgp¡on;䄄f;쀀𝔸plyFunction;恡ing耻Å䃅Ācs¾Ãr;쀀𝒜ign;扔ilde耻Ã䃃ml耻Ä䃄ЀaceforsuåûþėĜĢħĪĀcrêòkslash;或Ŷöø;櫧ed;挆y;䐑ƀcrtąċĔause;戵noullis;愬a;䎒r;쀀𝔅pf;쀀𝔹eve;䋘còēmpeq;扎܀HOacdefhilorsuōőŖƀƞƢƵƷƺǜȕɳɸɾcy;䐧PY耻©䂩ƀcpyŝŢźute;䄆Ā;iŧŨ拒talDifferentialD;慅leys;愭ȀaeioƉƎƔƘron;䄌dil耻Ç䃇rc;䄈nint;戰ot;䄊ĀdnƧƭilla;䂸terDot;䂷òſi;䎧rcleȀDMPTǇǋǑǖot;抙inus;抖lus;投imes;抗oĀcsǢǸkwiseContourIntegral;戲eCurlyĀDQȃȏoubleQuote;思uote;怙ȀlnpuȞȨɇɕonĀ;eȥȦ户;橴ƀgitȯȶȺruent;扡nt;戯ourIntegral;戮ĀfrɌɎ;愂oduct;成nterClockwiseContourIntegral;戳oss;樯cr;쀀𝒞pĀ;Cʄʅ拓ap;才րDJSZacefiosʠʬʰʴʸˋ˗ˡ˦̳ҍĀ;oŹʥtrahd;椑cy;䐂cy;䐅cy;䐏ƀgrsʿ˄ˇger;怡r;憡hv;櫤Āayː˕ron;䄎;䐔lĀ;t˝˞戇a;䎔r;쀀𝔇Āaf˫̧Ācm˰̢riticalȀADGT̖̜̀̆cute;䂴oŴ̋̍;䋙bleAcute;䋝rave;䁠ilde;䋜ond;拄ferentialD;慆Ѱ̽\0\0\0͔͂\0Ѕf;쀀𝔻ƀ;DE͈͉͍䂨ot;惜qual;扐blèCDLRUVͣͲ΂ϏϢϸontourIntegraìȹoɴ͹\0\0ͻ»͉nArrow;懓Āeo·ΤftƀARTΐΖΡrrow;懐ightArrow;懔eåˊngĀLRΫτeftĀARγιrrow;柸ightArrow;柺ightArrow;柹ightĀATϘϞrrow;懒ee;抨pɁϩ\0\0ϯrrow;懑ownArrow;懕erticalBar;戥ǹABLRTaВЪаўѿͼrrowƀ;BUНОТ憓ar;椓pArrow;懵reve;䌑eft˒к\0ц\0ѐightVector;楐eeVector;楞ectorĀ;Bљњ憽ar;楖ightǔѧ\0ѱeeVector;楟ectorĀ;BѺѻ懁ar;楗eeĀ;A҆҇护rrow;憧ĀctҒҗr;쀀𝒟rok;䄐ࠀNTacdfglmopqstuxҽӀӄӋӞӢӧӮӵԡԯԶՒ՝ՠեG;䅊H耻Ð䃐cute耻É䃉ƀaiyӒӗӜron;䄚rc耻Ê䃊;䐭ot;䄖r;쀀𝔈rave耻È䃈ement;戈ĀapӺӾcr;䄒tyɓԆ\0\0ԒmallSquare;旻erySmallSquare;斫ĀgpԦԪon;䄘f;쀀𝔼silon;䎕uĀaiԼՉlĀ;TՂՃ橵ilde;扂librium;懌Āci՗՚r;愰m;橳a;䎗ml耻Ë䃋Āipժկsts;戃onentialE;慇ʀcfiosօֈ֍ֲ׌y;䐤r;쀀𝔉lledɓ֗\0\0֣mallSquare;旼erySmallSquare;斪Ͱֺ\0ֿ\0\0ׄf;쀀𝔽All;戀riertrf;愱cò׋؀JTabcdfgorstר׬ׯ׺؀ؒؖ؛؝أ٬ٲcy;䐃耻>䀾mmaĀ;d׷׸䎓;䏜reve;䄞ƀeiy؇،ؐdil;䄢rc;䄜;䐓ot;䄠r;쀀𝔊;拙pf;쀀𝔾eater̀EFGLSTصلَٖٛ٦qualĀ;Lؾؿ扥ess;招ullEqual;执reater;檢ess;扷lantEqual;橾ilde;扳cr;쀀𝒢;扫ЀAacfiosuڅڋږڛڞڪھۊRDcy;䐪Āctڐڔek;䋇;䁞irc;䄤r;愌lbertSpace;愋ǰگ\0ڲf;愍izontalLine;攀Āctۃۅòکrok;䄦mpńېۘownHumðįqual;扏܀EJOacdfgmnostuۺ۾܃܇܎ܚܞܡܨ݄ݸދޏޕcy;䐕lig;䄲cy;䐁cute耻Í䃍Āiyܓܘrc耻Î䃎;䐘ot;䄰r;愑rave耻Ì䃌ƀ;apܠܯܿĀcgܴܷr;䄪inaryI;慈lieóϝǴ݉\0ݢĀ;eݍݎ戬Āgrݓݘral;戫section;拂isibleĀCTݬݲomma;恣imes;恢ƀgptݿރވon;䄮f;쀀𝕀a;䎙cr;愐ilde;䄨ǫޚ\0ޞcy;䐆l耻Ï䃏ʀcfosuެ޷޼߂ߐĀiyޱ޵rc;䄴;䐙r;쀀𝔍pf;쀀𝕁ǣ߇\0ߌr;쀀𝒥rcy;䐈kcy;䐄΀HJacfosߤߨ߽߬߱ࠂࠈcy;䐥cy;䐌ppa;䎚Āey߶߻dil;䄶;䐚r;쀀𝔎pf;쀀𝕂cr;쀀𝒦րJTaceflmostࠥࠩࠬࡐࡣ঳সে্਷ੇcy;䐉耻<䀼ʀcmnpr࠷࠼ࡁࡄࡍute;䄹bda;䎛g;柪lacetrf;愒r;憞ƀaeyࡗ࡜ࡡron;䄽dil;䄻;䐛Āfsࡨ॰tԀACDFRTUVarࡾࢩࢱࣦ࣠ࣼयज़ΐ४Ānrࢃ࢏gleBracket;柨rowƀ;BR࢙࢚࢞憐ar;懤ightArrow;懆eiling;挈oǵࢷ\0ࣃbleBracket;柦nǔࣈ\0࣒eeVector;楡ectorĀ;Bࣛࣜ懃ar;楙loor;挊ightĀAV࣯ࣵrrow;憔ector;楎Āerँगeƀ;AVउऊऐ抣rrow;憤ector;楚iangleƀ;BEतथऩ抲ar;槏qual;抴pƀDTVषूौownVector;楑eeVector;楠ectorĀ;Bॖॗ憿ar;楘ectorĀ;B॥०憼ar;楒ightáΜs̀EFGLSTॾঋকঝঢভqualGreater;拚ullEqual;扦reater;扶ess;檡lantEqual;橽ilde;扲r;쀀𝔏Ā;eঽা拘ftarrow;懚idot;䄿ƀnpw৔ਖਛgȀLRlr৞৷ਂਐeftĀAR০৬rrow;柵ightArrow;柷ightArrow;柶eftĀarγਊightáοightáϊf;쀀𝕃erĀLRਢਬeftArrow;憙ightArrow;憘ƀchtਾੀੂòࡌ;憰rok;䅁;扪Ѐacefiosuਗ਼੝੠੷੼અઋ઎p;椅y;䐜Ādl੥੯iumSpace;恟lintrf;愳r;쀀𝔐nusPlus;戓pf;쀀𝕄cò੶;䎜ҀJacefostuણધભીଔଙඑ඗ඞcy;䐊cute;䅃ƀaey઴હાron;䅇dil;䅅;䐝ƀgswે૰଎ativeƀMTV૓૟૨ediumSpace;怋hiĀcn૦૘ë૙eryThiî૙tedĀGL૸ଆreaterGreateòٳessLesóੈLine;䀊r;쀀𝔑ȀBnptଢନଷ଺reak;恠BreakingSpace;䂠f;愕ڀ;CDEGHLNPRSTV୕ୖ୪୼஡௫ఄ౞಄ದ೘ൡඅ櫬Āou୛୤ngruent;扢pCap;扭oubleVerticalBar;戦ƀlqxஃஊ஛ement;戉ualĀ;Tஒஓ扠ilde;쀀≂̸ists;戄reater΀;EFGLSTஶஷ஽௉௓௘௥扯qual;扱ullEqual;쀀≧̸reater;쀀≫̸ess;批lantEqual;쀀⩾̸ilde;扵umpń௲௽ownHump;쀀≎̸qual;쀀≏̸eĀfsఊధtTriangleƀ;BEచఛడ拪ar;쀀⧏̸qual;括s̀;EGLSTవశ఼ౄోౘ扮qual;扰reater;扸ess;쀀≪̸lantEqual;쀀⩽̸ilde;扴estedĀGL౨౹reaterGreater;쀀⪢̸essLess;쀀⪡̸recedesƀ;ESಒಓಛ技qual;쀀⪯̸lantEqual;拠ĀeiಫಹverseElement;戌ghtTriangleƀ;BEೋೌ೒拫ar;쀀⧐̸qual;拭ĀquೝഌuareSuĀbp೨೹setĀ;E೰ೳ쀀⊏̸qual;拢ersetĀ;Eഃആ쀀⊐̸qual;拣ƀbcpഓതൎsetĀ;Eഛഞ쀀⊂⃒qual;抈ceedsȀ;ESTലള഻െ抁qual;쀀⪰̸lantEqual;拡ilde;쀀≿̸ersetĀ;E൘൛쀀⊃⃒qual;抉ildeȀ;EFT൮൯൵ൿ扁qual;扄ullEqual;扇ilde;扉erticalBar;戤cr;쀀𝒩ilde耻Ñ䃑;䎝܀Eacdfgmoprstuvලෂ෉෕ෛ෠෧෼ขภยา฿ไlig;䅒cute耻Ó䃓Āiy෎ීrc耻Ô䃔;䐞blac;䅐r;쀀𝔒rave耻Ò䃒ƀaei෮ෲ෶cr;䅌ga;䎩cron;䎟pf;쀀𝕆enCurlyĀDQฎบoubleQuote;怜uote;怘;橔Āclวฬr;쀀𝒪ash耻Ø䃘iŬื฼de耻Õ䃕es;樷ml耻Ö䃖erĀBP๋๠Āar๐๓r;怾acĀek๚๜;揞et;掴arenthesis;揜Ҁacfhilors๿ງຊຏຒດຝະ໼rtialD;戂y;䐟r;쀀𝔓i;䎦;䎠usMinus;䂱Āipຢອncareplanåڝf;愙Ȁ;eio຺ູ໠໤檻cedesȀ;EST່້໏໚扺qual;檯lantEqual;扼ilde;找me;怳Ādp໩໮uct;戏ortionĀ;aȥ໹l;戝Āci༁༆r;쀀𝒫;䎨ȀUfos༑༖༛༟OT耻"䀢r;쀀𝔔pf;愚cr;쀀𝒬؀BEacefhiorsu༾གྷཇའཱིྦྷྪྭ႖ႩႴႾarr;椐G耻®䂮ƀcnrཎནབute;䅔g;柫rĀ;tཛྷཝ憠l;椖ƀaeyཧཬཱron;䅘dil;䅖;䐠Ā;vླྀཹ愜erseĀEUྂྙĀlq྇ྎement;戋uilibrium;懋pEquilibrium;楯r»ཹo;䎡ghtЀACDFTUVa࿁࿫࿳ဢဨၛႇϘĀnr࿆࿒gleBracket;柩rowƀ;BL࿜࿝࿡憒ar;懥eftArrow;懄eiling;按oǵ࿹\0စbleBracket;柧nǔည\0နeeVector;楝ectorĀ;Bဝသ懂ar;楕loor;挋Āerိ၃eƀ;AVဵံြ抢rrow;憦ector;楛iangleƀ;BEၐၑၕ抳ar;槐qual;抵pƀDTVၣၮၸownVector;楏eeVector;楜ectorĀ;Bႂႃ憾ar;楔ectorĀ;B႑႒懀ar;楓Āpuႛ႞f;愝ndImplies;楰ightarrow;懛ĀchႹႼr;愛;憱leDelayed;槴ڀHOacfhimoqstuფჱჷჽᄙᄞᅑᅖᅡᅧᆵᆻᆿĀCcჩხHcy;䐩y;䐨FTcy;䐬cute;䅚ʀ;aeiyᄈᄉᄎᄓᄗ檼ron;䅠dil;䅞rc;䅜;䐡r;쀀𝔖ortȀDLRUᄪᄴᄾᅉownArrow»ОeftArrow»࢚ightArrow»࿝pArrow;憑gma;䎣allCircle;战pf;쀀𝕊ɲᅭ\0\0ᅰt;戚areȀ;ISUᅻᅼᆉᆯ斡ntersection;抓uĀbpᆏᆞsetĀ;Eᆗᆘ抏qual;抑ersetĀ;Eᆨᆩ抐qual;抒nion;抔cr;쀀𝒮ar;拆ȀbcmpᇈᇛሉላĀ;sᇍᇎ拐etĀ;Eᇍᇕqual;抆ĀchᇠህeedsȀ;ESTᇭᇮᇴᇿ扻qual;檰lantEqual;扽ilde;承Tháྌ;我ƀ;esሒሓሣ拑rsetĀ;Eሜም抃qual;抇et»ሓրHRSacfhiorsሾቄ቉ቕ቞ቱቶኟዂወዑORN耻Þ䃞ADE;愢ĀHc቎ቒcy;䐋y;䐦Ābuቚቜ;䀉;䎤ƀaeyብቪቯron;䅤dil;䅢;䐢r;쀀𝔗Āeiቻ኉ǲኀ\0ኇefore;戴a;䎘Ācn኎ኘkSpace;쀀  Space;怉ldeȀ;EFTካኬኲኼ戼qual;扃ullEqual;扅ilde;扈pf;쀀𝕋ipleDot;惛Āctዖዛr;쀀𝒯rok;䅦ૡዷጎጚጦ\0ጬጱ\0\0\0\0\0ጸጽ፷ᎅ\0᏿ᐄᐊᐐĀcrዻጁute耻Ú䃚rĀ;oጇገ憟cir;楉rǣጓ\0጖y;䐎ve;䅬Āiyጞጣrc耻Û䃛;䐣blac;䅰r;쀀𝔘rave耻Ù䃙acr;䅪Ādiፁ፩erĀBPፈ፝Āarፍፐr;䁟acĀekፗፙ;揟et;掵arenthesis;揝onĀ;P፰፱拃lus;抎Āgp፻፿on;䅲f;쀀𝕌ЀADETadps᎕ᎮᎸᏄϨᏒᏗᏳrrowƀ;BDᅐᎠᎤar;椒ownArrow;懅ownArrow;憕quilibrium;楮eeĀ;AᏋᏌ报rrow;憥ownáϳerĀLRᏞᏨeftArrow;憖ightArrow;憗iĀ;lᏹᏺ䏒on;䎥ing;䅮cr;쀀𝒰ilde;䅨ml耻Ü䃜ҀDbcdefosvᐧᐬᐰᐳᐾᒅᒊᒐᒖash;披ar;櫫y;䐒ashĀ;lᐻᐼ抩;櫦Āerᑃᑅ;拁ƀbtyᑌᑐᑺar;怖Ā;iᑏᑕcalȀBLSTᑡᑥᑪᑴar;戣ine;䁼eparator;杘ilde;所ThinSpace;怊r;쀀𝔙pf;쀀𝕍cr;쀀𝒱dash;抪ʀcefosᒧᒬᒱᒶᒼirc;䅴dge;拀r;쀀𝔚pf;쀀𝕎cr;쀀𝒲Ȁfiosᓋᓐᓒᓘr;쀀𝔛;䎞pf;쀀𝕏cr;쀀𝒳ҀAIUacfosuᓱᓵᓹᓽᔄᔏᔔᔚᔠcy;䐯cy;䐇cy;䐮cute耻Ý䃝Āiyᔉᔍrc;䅶;䐫r;쀀𝔜pf;쀀𝕐cr;쀀𝒴ml;䅸ЀHacdefosᔵᔹᔿᕋᕏᕝᕠᕤcy;䐖cute;䅹Āayᕄᕉron;䅽;䐗ot;䅻ǲᕔ\0ᕛoWidtè૙a;䎖r;愨pf;愤cr;쀀𝒵௡ᖃᖊᖐ\0ᖰᖶᖿ\0\0\0\0ᗆᗛᗫᙟ᙭\0ᚕ᚛ᚲᚹ\0ᚾcute耻á䃡reve;䄃̀;Ediuyᖜᖝᖡᖣᖨᖭ戾;쀀∾̳;房rc耻â䃢te肻´̆;䐰lig耻æ䃦Ā;r²ᖺ;쀀𝔞rave耻à䃠ĀepᗊᗖĀfpᗏᗔsym;愵èᗓha;䎱ĀapᗟcĀclᗤᗧr;䄁g;樿ɤᗰ\0\0ᘊʀ;adsvᗺᗻᗿᘁᘇ戧nd;橕;橜lope;橘;橚΀;elmrszᘘᘙᘛᘞᘿᙏᙙ戠;榤e»ᘙsdĀ;aᘥᘦ戡ѡᘰᘲᘴᘶᘸᘺᘼᘾ;榨;榩;榪;榫;榬;榭;榮;榯tĀ;vᙅᙆ戟bĀ;dᙌᙍ抾;榝Āptᙔᙗh;戢»¹arr;捼Āgpᙣᙧon;䄅f;쀀𝕒΀;Eaeiop዁ᙻᙽᚂᚄᚇᚊ;橰cir;橯;扊d;手s;䀧roxĀ;e዁ᚒñᚃing耻å䃥ƀctyᚡᚦᚨr;쀀𝒶;䀪mpĀ;e዁ᚯñʈilde耻ã䃣ml耻ä䃤Āciᛂᛈoninôɲnt;樑ࠀNabcdefiklnoprsu᛭ᛱᜰ᜼ᝃᝈ᝸᝽០៦ᠹᡐᜍ᤽᥈ᥰot;櫭Ācrᛶ᜞kȀcepsᜀᜅᜍᜓong;扌psilon;䏶rime;怵imĀ;e᜚᜛戽q;拍Ŷᜢᜦee;抽edĀ;gᜬᜭ挅e»ᜭrkĀ;t፜᜷brk;掶Āoyᜁᝁ;䐱quo;怞ʀcmprtᝓ᝛ᝡᝤᝨausĀ;eĊĉptyv;榰séᜌnoõēƀahwᝯ᝱ᝳ;䎲;愶een;扬r;쀀𝔟g΀costuvwឍឝឳេ៕៛៞ƀaiuបពរðݠrc;旯p»፱ƀdptឤឨឭot;樀lus;樁imes;樂ɱឹ\0\0ើcup;樆ar;昅riangleĀdu៍្own;施p;斳plus;樄eåᑄåᒭarow;植ƀako៭ᠦᠵĀcn៲ᠣkƀlst៺֫᠂ozenge;槫riangleȀ;dlr᠒᠓᠘᠝斴own;斾eft;旂ight;斸k;搣Ʊᠫ\0ᠳƲᠯ\0ᠱ;斒;斑4;斓ck;斈ĀeoᠾᡍĀ;qᡃᡆ쀀=⃥uiv;쀀≡⃥t;挐Ȁptwxᡙᡞᡧᡬf;쀀𝕓Ā;tᏋᡣom»Ꮜtie;拈؀DHUVbdhmptuvᢅᢖᢪᢻᣗᣛᣬ᣿ᤅᤊᤐᤡȀLRlrᢎᢐᢒᢔ;敗;敔;敖;敓ʀ;DUduᢡᢢᢤᢦᢨ敐;敦;敩;敤;敧ȀLRlrᢳᢵᢷᢹ;敝;敚;敜;教΀;HLRhlrᣊᣋᣍᣏᣑᣓᣕ救;敬;散;敠;敫;敢;敟ox;槉ȀLRlrᣤᣦᣨᣪ;敕;敒;攐;攌ʀ;DUduڽ᣷᣹᣻᣽;敥;敨;攬;攴inus;抟lus;択imes;抠ȀLRlrᤙᤛᤝ᤟;敛;敘;攘;攔΀;HLRhlrᤰᤱᤳᤵᤷ᤻᤹攂;敪;敡;敞;攼;攤;攜Āevģ᥂bar耻¦䂦Ȁceioᥑᥖᥚᥠr;쀀𝒷mi;恏mĀ;e᜚᜜lƀ;bhᥨᥩᥫ䁜;槅sub;柈Ŭᥴ᥾lĀ;e᥹᥺怢t»᥺pƀ;Eeįᦅᦇ;檮Ā;qۜۛೡᦧ\0᧨ᨑᨕᨲ\0ᨷᩐ\0\0᪴\0\0᫁\0\0ᬡᬮ᭍᭒\0᯽\0ᰌƀcpr᦭ᦲ᧝ute;䄇̀;abcdsᦿᧀᧄ᧊᧕᧙戩nd;橄rcup;橉Āau᧏᧒p;橋p;橇ot;橀;쀀∩︀Āeo᧢᧥t;恁îړȀaeiu᧰᧻ᨁᨅǰ᧵\0᧸s;橍on;䄍dil耻ç䃧rc;䄉psĀ;sᨌᨍ橌m;橐ot;䄋ƀdmnᨛᨠᨦil肻¸ƭptyv;榲t脀¢;eᨭᨮ䂢räƲr;쀀𝔠ƀceiᨽᩀᩍy;䑇ckĀ;mᩇᩈ朓ark»ᩈ;䏇r΀;Ecefms᩟᩠ᩢᩫ᪤᪪᪮旋;槃ƀ;elᩩᩪᩭ䋆q;扗eɡᩴ\0\0᪈rrowĀlr᩼᪁eft;憺ight;憻ʀRSacd᪒᪔᪖᪚᪟»ཇ;擈st;抛irc;抚ash;抝nint;樐id;櫯cir;槂ubsĀ;u᪻᪼晣it»᪼ˬ᫇᫔᫺\0ᬊonĀ;eᫍᫎ䀺Ā;qÇÆɭ᫙\0\0᫢aĀ;t᫞᫟䀬;䁀ƀ;fl᫨᫩᫫戁îᅠeĀmx᫱᫶ent»᫩eóɍǧ᫾\0ᬇĀ;dኻᬂot;橭nôɆƀfryᬐᬔᬗ;쀀𝕔oäɔ脀©;sŕᬝr;愗Āaoᬥᬩrr;憵ss;朗Ācuᬲᬷr;쀀𝒸Ābpᬼ᭄Ā;eᭁᭂ櫏;櫑Ā;eᭉᭊ櫐;櫒dot;拯΀delprvw᭠᭬᭷ᮂᮬᯔ᯹arrĀlr᭨᭪;椸;椵ɰ᭲\0\0᭵r;拞c;拟arrĀ;p᭿ᮀ憶;椽̀;bcdosᮏᮐᮖᮡᮥᮨ截rcap;橈Āauᮛᮞp;橆p;橊ot;抍r;橅;쀀∪︀Ȁalrv᮵ᮿᯞᯣrrĀ;mᮼᮽ憷;椼yƀevwᯇᯔᯘqɰᯎ\0\0ᯒreã᭳uã᭵ee;拎edge;拏en耻¤䂤earrowĀlrᯮ᯳eft»ᮀight»ᮽeäᯝĀciᰁᰇoninôǷnt;戱lcty;挭ঀAHabcdefhijlorstuwz᰸᰻᰿ᱝᱩᱵᲊᲞᲬᲷ᳻᳿ᴍᵻᶑᶫᶻ᷆᷍rò΁ar;楥Ȁglrs᱈ᱍ᱒᱔ger;怠eth;愸òᄳhĀ;vᱚᱛ怐»ऊūᱡᱧarow;椏aã̕Āayᱮᱳron;䄏;䐴ƀ;ao̲ᱼᲄĀgrʿᲁr;懊tseq;橷ƀglmᲑᲔᲘ耻°䂰ta;䎴ptyv;榱ĀirᲣᲨsht;楿;쀀𝔡arĀlrᲳᲵ»ࣜ»သʀaegsv᳂͸᳖᳜᳠mƀ;oș᳊᳔ndĀ;ș᳑uit;晦amma;䏝in;拲ƀ;io᳧᳨᳸䃷de脀÷;o᳧ᳰntimes;拇nø᳷cy;䑒cɯᴆ\0\0ᴊrn;挞op;挍ʀlptuwᴘᴝᴢᵉᵕlar;䀤f;쀀𝕕ʀ;emps̋ᴭᴷᴽᵂqĀ;d͒ᴳot;扑inus;戸lus;戔quare;抡blebarwedgåúnƀadhᄮᵝᵧownarrowóᲃarpoonĀlrᵲᵶefôᲴighôᲶŢᵿᶅkaro÷གɯᶊ\0\0ᶎrn;挟op;挌ƀcotᶘᶣᶦĀryᶝᶡ;쀀𝒹;䑕l;槶rok;䄑Ādrᶰᶴot;拱iĀ;fᶺ᠖斿Āah᷀᷃ròЩaòྦangle;榦Āci᷒ᷕy;䑟grarr;柿ऀDacdefglmnopqrstuxḁḉḙḸոḼṉṡṾấắẽỡἪἷὄ὎὚ĀDoḆᴴoôᲉĀcsḎḔute耻é䃩ter;橮ȀaioyḢḧḱḶron;䄛rĀ;cḭḮ扖耻ê䃪lon;払;䑍ot;䄗ĀDrṁṅot;扒;쀀𝔢ƀ;rsṐṑṗ檚ave耻è䃨Ā;dṜṝ檖ot;檘Ȁ;ilsṪṫṲṴ檙nters;揧;愓Ā;dṹṺ檕ot;檗ƀapsẅẉẗcr;䄓tyƀ;svẒẓẕ戅et»ẓpĀ1;ẝẤĳạả;怄;怅怃ĀgsẪẬ;䅋p;怂ĀgpẴẸon;䄙f;쀀𝕖ƀalsỄỎỒrĀ;sỊị拕l;槣us;橱iƀ;lvỚớở䎵on»ớ;䏵ȀcsuvỪỳἋἣĀioữḱrc»Ḯɩỹ\0\0ỻíՈantĀglἂἆtr»ṝess»Ṻƀaeiἒ἖Ἒls;䀽st;扟vĀ;DȵἠD;橸parsl;槥ĀDaἯἳot;打rr;楱ƀcdiἾὁỸr;愯oô͒ĀahὉὋ;䎷耻ð䃰Āmrὓὗl耻ë䃫o;悬ƀcipὡὤὧl;䀡sôծĀeoὬὴctatioîՙnentialåչৡᾒ\0ᾞ\0ᾡᾧ\0\0ῆῌ\0ΐ\0ῦῪ \0 ⁚llingdotseñṄy;䑄male;晀ƀilrᾭᾳ῁lig;耀ﬃɩᾹ\0\0᾽g;耀ﬀig;耀ﬄ;쀀𝔣lig;耀ﬁlig;쀀fjƀaltῙ῜ῡt;晭ig;耀ﬂns;斱of;䆒ǰ΅\0ῳf;쀀𝕗ĀakֿῷĀ;vῼ´拔;櫙artint;樍Āao‌⁕Ācs‑⁒α‚‰‸⁅⁈\0⁐β•‥‧‪‬\0‮耻½䂽;慓耻¼䂼;慕;慙;慛Ƴ‴\0‶;慔;慖ʴ‾⁁\0\0⁃耻¾䂾;慗;慜5;慘ƶ⁌\0⁎;慚;慝8;慞l;恄wn;挢cr;쀀𝒻ࢀEabcdefgijlnorstv₂₉₟₥₰₴⃰⃵⃺⃿℃ℒℸ̗ℾ⅒↞Ā;lٍ₇;檌ƀcmpₐₕ₝ute;䇵maĀ;dₜ᳚䎳;檆reve;䄟Āiy₪₮rc;䄝;䐳ot;䄡Ȁ;lqsؾق₽⃉ƀ;qsؾٌ⃄lanô٥Ȁ;cdl٥⃒⃥⃕c;檩otĀ;o⃜⃝檀Ā;l⃢⃣檂;檄Ā;e⃪⃭쀀⋛︀s;檔r;쀀𝔤Ā;gٳ؛mel;愷cy;䑓Ȁ;Eajٚℌℎℐ;檒;檥;檤ȀEaesℛℝ℩ℴ;扩pĀ;p℣ℤ檊rox»ℤĀ;q℮ℯ檈Ā;q℮ℛim;拧pf;쀀𝕘Āci⅃ⅆr;愊mƀ;el٫ⅎ⅐;檎;檐茀>;cdlqr׮ⅠⅪⅮⅳⅹĀciⅥⅧ;檧r;橺ot;拗Par;榕uest;橼ʀadelsↄⅪ←ٖ↛ǰ↉\0↎proø₞r;楸qĀlqؿ↖lesó₈ií٫Āen↣↭rtneqq;쀀≩︀Å↪ԀAabcefkosy⇄⇇⇱⇵⇺∘∝∯≨≽ròΠȀilmr⇐⇔⇗⇛rsðᒄf»․ilôکĀdr⇠⇤cy;䑊ƀ;cwࣴ⇫⇯ir;楈;憭ar;意irc;䄥ƀalr∁∎∓rtsĀ;u∉∊晥it»∊lip;怦con;抹r;쀀𝔥sĀew∣∩arow;椥arow;椦ʀamopr∺∾≃≞≣rr;懿tht;戻kĀlr≉≓eftarrow;憩ightarrow;憪f;쀀𝕙bar;怕ƀclt≯≴≸r;쀀𝒽asè⇴rok;䄧Ābp⊂⊇ull;恃hen»ᱛૡ⊣\0⊪\0⊸⋅⋎\0⋕⋳\0\0⋸⌢⍧⍢⍿\0⎆⎪⎴cute耻í䃭ƀ;iyݱ⊰⊵rc耻î䃮;䐸Ācx⊼⊿y;䐵cl耻¡䂡ĀfrΟ⋉;쀀𝔦rave耻ì䃬Ȁ;inoܾ⋝⋩⋮Āin⋢⋦nt;樌t;戭fin;槜ta;愩lig;䄳ƀaop⋾⌚⌝ƀcgt⌅⌈⌗r;䄫ƀelpܟ⌏⌓inåގarôܠh;䄱f;抷ed;䆵ʀ;cfotӴ⌬⌱⌽⍁are;愅inĀ;t⌸⌹戞ie;槝doô⌙ʀ;celpݗ⍌⍐⍛⍡al;抺Āgr⍕⍙eróᕣã⍍arhk;樗rod;樼Ȁcgpt⍯⍲⍶⍻y;䑑on;䄯f;쀀𝕚a;䎹uest耻¿䂿Āci⎊⎏r;쀀𝒾nʀ;EdsvӴ⎛⎝⎡ӳ;拹ot;拵Ā;v⎦⎧拴;拳Ā;iݷ⎮lde;䄩ǫ⎸\0⎼cy;䑖l耻ï䃯̀cfmosu⏌⏗⏜⏡⏧⏵Āiy⏑⏕rc;䄵;䐹r;쀀𝔧ath;䈷pf;쀀𝕛ǣ⏬\0⏱r;쀀𝒿rcy;䑘kcy;䑔Ѐacfghjos␋␖␢␧␭␱␵␻ppaĀ;v␓␔䎺;䏰Āey␛␠dil;䄷;䐺r;쀀𝔨reen;䄸cy;䑅cy;䑜pf;쀀𝕜cr;쀀𝓀஀ABEHabcdefghjlmnoprstuv⑰⒁⒆⒍⒑┎┽╚▀♎♞♥♹♽⚚⚲⛘❝❨➋⟀⠁⠒ƀart⑷⑺⑼rò৆òΕail;椛arr;椎Ā;gঔ⒋;檋ar;楢ॣ⒥\0⒪\0⒱\0\0\0\0\0⒵Ⓔ\0ⓆⓈⓍ\0⓹ute;䄺mptyv;榴raîࡌbda;䎻gƀ;dlࢎⓁⓃ;榑åࢎ;檅uo耻«䂫rЀ;bfhlpst࢙ⓞⓦⓩ⓫⓮⓱⓵Ā;f࢝ⓣs;椟s;椝ë≒p;憫l;椹im;楳l;憢ƀ;ae⓿─┄檫il;椙Ā;s┉┊檭;쀀⪭︀ƀabr┕┙┝rr;椌rk;杲Āak┢┬cĀek┨┪;䁻;䁛Āes┱┳;榋lĀdu┹┻;榏;榍Ȁaeuy╆╋╖╘ron;䄾Ādi═╔il;䄼ìࢰâ┩;䐻Ȁcqrs╣╦╭╽a;椶uoĀ;rนᝆĀdu╲╷har;楧shar;楋h;憲ʀ;fgqs▋▌উ◳◿扤tʀahlrt▘▤▷◂◨rrowĀ;t࢙□aé⓶arpoonĀdu▯▴own»њp»०eftarrows;懇ightƀahs◍◖◞rrowĀ;sࣴࢧarpoonó྘quigarro÷⇰hreetimes;拋ƀ;qs▋ও◺lanôবʀ;cdgsব☊☍☝☨c;檨otĀ;o☔☕橿Ā;r☚☛檁;檃Ā;e☢☥쀀⋚︀s;檓ʀadegs☳☹☽♉♋pproøⓆot;拖qĀgq♃♅ôউgtò⒌ôছiíলƀilr♕࣡♚sht;楼;쀀𝔩Ā;Eজ♣;檑š♩♶rĀdu▲♮Ā;l॥♳;楪lk;斄cy;䑙ʀ;achtੈ⚈⚋⚑⚖rò◁orneòᴈard;楫ri;旺Āio⚟⚤dot;䅀ustĀ;a⚬⚭掰che»⚭ȀEaes⚻⚽⛉⛔;扨pĀ;p⛃⛄檉rox»⛄Ā;q⛎⛏檇Ā;q⛎⚻im;拦Ѐabnoptwz⛩⛴⛷✚✯❁❇❐Ānr⛮⛱g;柬r;懽rëࣁgƀlmr⛿✍✔eftĀar০✇ightá৲apsto;柼ightá৽parrowĀlr✥✩efô⓭ight;憬ƀafl✶✹✽r;榅;쀀𝕝us;樭imes;樴š❋❏st;戗áፎƀ;ef❗❘᠀旊nge»❘arĀ;l❤❥䀨t;榓ʀachmt❳❶❼➅➇ròࢨorneòᶌarĀ;d྘➃;業;怎ri;抿̀achiqt➘➝ੀ➢➮➻quo;怹r;쀀𝓁mƀ;egল➪➬;檍;檏Ābu┪➳oĀ;rฟ➹;怚rok;䅂萀<;cdhilqrࠫ⟒☹⟜⟠⟥⟪⟰Āci⟗⟙;檦r;橹reå◲mes;拉arr;楶uest;橻ĀPi⟵⟹ar;榖ƀ;ef⠀भ᠛旃rĀdu⠇⠍shar;楊har;楦Āen⠗⠡rtneqq;쀀≨︀Å⠞܀Dacdefhilnopsu⡀⡅⢂⢎⢓⢠⢥⢨⣚⣢⣤ઃ⣳⤂Dot;戺Ȁclpr⡎⡒⡣⡽r耻¯䂯Āet⡗⡙;時Ā;e⡞⡟朠se»⡟Ā;sျ⡨toȀ;dluျ⡳⡷⡻owîҌefôएðᏑker;斮Āoy⢇⢌mma;権;䐼ash;怔asuredangle»ᘦr;쀀𝔪o;愧ƀcdn⢯⢴⣉ro耻µ䂵Ȁ;acdᑤ⢽⣀⣄sôᚧir;櫰ot肻·Ƶusƀ;bd⣒ᤃ⣓戒Ā;uᴼ⣘;横ţ⣞⣡p;櫛ò−ðઁĀdp⣩⣮els;抧f;쀀𝕞Āct⣸⣽r;쀀𝓂pos»ᖝƀ;lm⤉⤊⤍䎼timap;抸ఀGLRVabcdefghijlmoprstuvw⥂⥓⥾⦉⦘⧚⧩⨕⨚⩘⩝⪃⪕⪤⪨⬄⬇⭄⭿⮮ⰴⱧⱼ⳩Āgt⥇⥋;쀀⋙̸Ā;v⥐௏쀀≫⃒ƀelt⥚⥲⥶ftĀar⥡⥧rrow;懍ightarrow;懎;쀀⋘̸Ā;v⥻ే쀀≪⃒ightarrow;懏ĀDd⦎⦓ash;抯ash;抮ʀbcnpt⦣⦧⦬⦱⧌la»˞ute;䅄g;쀀∠⃒ʀ;Eiop඄⦼⧀⧅⧈;쀀⩰̸d;쀀≋̸s;䅉roø඄urĀ;a⧓⧔普lĀ;s⧓ସǳ⧟\0⧣p肻 ଷmpĀ;e௹ఀʀaeouy⧴⧾⨃⨐⨓ǰ⧹\0⧻;橃on;䅈dil;䅆ngĀ;dൾ⨊ot;쀀⩭̸p;橂;䐽ash;怓΀;Aadqsxஒ⨩⨭⨻⩁⩅⩐rr;懗rĀhr⨳⨶k;椤Ā;oᏲᏰot;쀀≐̸uiöୣĀei⩊⩎ar;椨í஘istĀ;s஠டr;쀀𝔫ȀEest௅⩦⩹⩼ƀ;qs஼⩭௡ƀ;qs஼௅⩴lanô௢ií௪Ā;rஶ⪁»ஷƀAap⪊⪍⪑rò⥱rr;憮ar;櫲ƀ;svྍ⪜ྌĀ;d⪡⪢拼;拺cy;䑚΀AEadest⪷⪺⪾⫂⫅⫶⫹rò⥦;쀀≦̸rr;憚r;急Ȁ;fqs఻⫎⫣⫯tĀar⫔⫙rro÷⫁ightarro÷⪐ƀ;qs఻⪺⫪lanôౕĀ;sౕ⫴»శiíౝĀ;rవ⫾iĀ;eచథiäඐĀpt⬌⬑f;쀀𝕟膀¬;in⬙⬚⬶䂬nȀ;Edvஉ⬤⬨⬮;쀀⋹̸ot;쀀⋵̸ǡஉ⬳⬵;拷;拶iĀ;vಸ⬼ǡಸ⭁⭃;拾;拽ƀaor⭋⭣⭩rȀ;ast୻⭕⭚⭟lleì୻l;쀀⫽⃥;쀀∂̸lint;樔ƀ;ceಒ⭰⭳uåಥĀ;cಘ⭸Ā;eಒ⭽ñಘȀAait⮈⮋⮝⮧rò⦈rrƀ;cw⮔⮕⮙憛;쀀⤳̸;쀀↝̸ghtarrow»⮕riĀ;eೋೖ΀chimpqu⮽⯍⯙⬄୸⯤⯯Ȁ;cerല⯆ഷ⯉uå൅;쀀𝓃ortɭ⬅\0\0⯖ará⭖mĀ;e൮⯟Ā;q൴൳suĀbp⯫⯭å೸åഋƀbcp⯶ⰑⰙȀ;Ees⯿ⰀഢⰄ抄;쀀⫅̸etĀ;eഛⰋqĀ;qണⰀcĀ;eലⰗñസȀ;EesⰢⰣൟⰧ抅;쀀⫆̸etĀ;e൘ⰮqĀ;qൠⰣȀgilrⰽⰿⱅⱇìௗlde耻ñ䃱çృiangleĀlrⱒⱜeftĀ;eచⱚñదightĀ;eೋⱥñ೗Ā;mⱬⱭ䎽ƀ;esⱴⱵⱹ䀣ro;愖p;怇ҀDHadgilrsⲏⲔⲙⲞⲣⲰⲶⳓⳣash;抭arr;椄p;쀀≍⃒ash;抬ĀetⲨⲬ;쀀≥⃒;쀀>⃒nfin;槞ƀAetⲽⳁⳅrr;椂;쀀≤⃒Ā;rⳊⳍ쀀<⃒ie;쀀⊴⃒ĀAtⳘⳜrr;椃rie;쀀⊵⃒im;쀀∼⃒ƀAan⳰⳴ⴂrr;懖rĀhr⳺⳽k;椣Ā;oᏧᏥear;椧ቓ᪕\0\0\0\0\0\0\0\0\0\0\0\0\0ⴭ\0ⴸⵈⵠⵥ⵲ⶄᬇ\0\0ⶍⶫ\0ⷈⷎ\0ⷜ⸙⸫⸾⹃Ācsⴱ᪗ute耻ó䃳ĀiyⴼⵅrĀ;c᪞ⵂ耻ô䃴;䐾ʀabios᪠ⵒⵗǈⵚlac;䅑v;樸old;榼lig;䅓Ācr⵩⵭ir;榿;쀀𝔬ͯ⵹\0\0⵼\0ⶂn;䋛ave耻ò䃲;槁Ābmⶈ෴ar;榵Ȁacitⶕ⶘ⶥⶨrò᪀Āir⶝ⶠr;榾oss;榻nå๒;槀ƀaeiⶱⶵⶹcr;䅍ga;䏉ƀcdnⷀⷅǍron;䎿;榶pf;쀀𝕠ƀaelⷔ⷗ǒr;榷rp;榹΀;adiosvⷪⷫⷮ⸈⸍⸐⸖戨rò᪆Ȁ;efmⷷⷸ⸂⸅橝rĀ;oⷾⷿ愴f»ⷿ耻ª䂪耻º䂺gof;抶r;橖lope;橗;橛ƀclo⸟⸡⸧ò⸁ash耻ø䃸l;折iŬⸯ⸴de耻õ䃵esĀ;aǛ⸺s;樶ml耻ö䃶bar;挽ૡ⹞\0⹽\0⺀⺝\0⺢⺹\0\0⻋ຜ\0⼓\0\0⼫⾼\0⿈rȀ;astЃ⹧⹲຅脀¶;l⹭⹮䂶leìЃɩ⹸\0\0⹻m;櫳;櫽y;䐿rʀcimpt⺋⺏⺓ᡥ⺗nt;䀥od;䀮il;怰enk;怱r;쀀𝔭ƀimo⺨⺰⺴Ā;v⺭⺮䏆;䏕maô੶ne;明ƀ;tv⺿⻀⻈䏀chfork»´;䏖Āau⻏⻟nĀck⻕⻝kĀ;h⇴⻛;愎ö⇴sҀ;abcdemst⻳⻴ᤈ⻹⻽⼄⼆⼊⼎䀫cir;樣ir;樢Āouᵀ⼂;樥;橲n肻±ຝim;樦wo;樧ƀipu⼙⼠⼥ntint;樕f;쀀𝕡nd耻£䂣Ԁ;Eaceinosu່⼿⽁⽄⽇⾁⾉⾒⽾⾶;檳p;檷uå໙Ā;c໎⽌̀;acens່⽙⽟⽦⽨⽾pproø⽃urlyeñ໙ñ໎ƀaes⽯⽶⽺pprox;檹qq;檵im;拨iíໟmeĀ;s⾈ຮ怲ƀEas⽸⾐⽺ð⽵ƀdfp໬⾙⾯ƀals⾠⾥⾪lar;挮ine;挒urf;挓Ā;t໻⾴ï໻rel;抰Āci⿀⿅r;쀀𝓅;䏈ncsp;怈̀fiopsu⿚⋢⿟⿥⿫⿱r;쀀𝔮pf;쀀𝕢rime;恗cr;쀀𝓆ƀaeo⿸〉〓tĀei⿾々rnionóڰnt;樖stĀ;e【】䀿ñἙô༔઀ABHabcdefhilmnoprstux぀けさすムㄎㄫㅇㅢㅲㆎ㈆㈕㈤㈩㉘㉮㉲㊐㊰㊷ƀartぇおがròႳòϝail;検aròᱥar;楤΀cdenqrtとふへみわゔヌĀeuねぱ;쀀∽̱te;䅕iãᅮmptyv;榳gȀ;del࿑らるろ;榒;榥å࿑uo耻»䂻rր;abcfhlpstw࿜ガクシスゼゾダッデナp;極Ā;f࿠ゴs;椠;椳s;椞ë≝ð✮l;楅im;楴l;憣;憝Āaiパフil;椚oĀ;nホボ戶aló༞ƀabrョリヮrò៥rk;杳ĀakンヽcĀekヹ・;䁽;䁝Āes㄂㄄;榌lĀduㄊㄌ;榎;榐Ȁaeuyㄗㄜㄧㄩron;䅙Ādiㄡㄥil;䅗ì࿲âヺ;䑀Ȁclqsㄴㄷㄽㅄa;椷dhar;楩uoĀ;rȎȍh;憳ƀacgㅎㅟངlȀ;ipsླྀㅘㅛႜnåႻarôྩt;断ƀilrㅩဣㅮsht;楽;쀀𝔯ĀaoㅷㆆrĀduㅽㅿ»ѻĀ;l႑ㆄ;楬Ā;vㆋㆌ䏁;䏱ƀgns㆕ㇹㇼht̀ahlrstㆤㆰ㇂㇘㇤㇮rrowĀ;t࿜ㆭaéトarpoonĀduㆻㆿowîㅾp»႒eftĀah㇊㇐rrowó࿪arpoonóՑightarrows;應quigarro÷ニhreetimes;拌g;䋚ingdotseñἲƀahm㈍㈐㈓rò࿪aòՑ;怏oustĀ;a㈞㈟掱che»㈟mid;櫮Ȁabpt㈲㈽㉀㉒Ānr㈷㈺g;柭r;懾rëဃƀafl㉇㉊㉎r;榆;쀀𝕣us;樮imes;樵Āap㉝㉧rĀ;g㉣㉤䀩t;榔olint;樒arò㇣Ȁachq㉻㊀Ⴜ㊅quo;怺r;쀀𝓇Ābu・㊊oĀ;rȔȓƀhir㊗㊛㊠reåㇸmes;拊iȀ;efl㊪ၙᠡ㊫方tri;槎luhar;楨;愞ൡ㋕㋛㋟㌬㌸㍱\0㍺㎤\0\0㏬㏰\0㐨㑈㑚㒭㒱㓊㓱\0㘖\0\0㘳cute;䅛quï➺Ԁ;Eaceinpsyᇭ㋳㋵㋿㌂㌋㌏㌟㌦㌩;檴ǰ㋺\0㋼;檸on;䅡uåᇾĀ;dᇳ㌇il;䅟rc;䅝ƀEas㌖㌘㌛;檶p;檺im;择olint;樓iíሄ;䑁otƀ;be㌴ᵇ㌵担;橦΀Aacmstx㍆㍊㍗㍛㍞㍣㍭rr;懘rĀhr㍐㍒ë∨Ā;oਸ਼਴t耻§䂧i;䀻war;椩mĀin㍩ðnuóñt;朶rĀ;o㍶⁕쀀𝔰Ȁacoy㎂㎆㎑㎠rp;景Āhy㎋㎏cy;䑉;䑈rtɭ㎙\0\0㎜iäᑤaraì⹯耻­䂭Āgm㎨㎴maƀ;fv㎱㎲㎲䏃;䏂Ѐ;deglnprካ㏅㏉㏎㏖㏞㏡㏦ot;橪Ā;q኱ኰĀ;E㏓㏔檞;檠Ā;E㏛㏜檝;檟e;扆lus;樤arr;楲aròᄽȀaeit㏸㐈㐏㐗Āls㏽㐄lsetmé㍪hp;樳parsl;槤Ādlᑣ㐔e;挣Ā;e㐜㐝檪Ā;s㐢㐣檬;쀀⪬︀ƀflp㐮㐳㑂tcy;䑌Ā;b㐸㐹䀯Ā;a㐾㐿槄r;挿f;쀀𝕤aĀdr㑍ЂesĀ;u㑔㑕晠it»㑕ƀcsu㑠㑹㒟Āau㑥㑯pĀ;sᆈ㑫;쀀⊓︀pĀ;sᆴ㑵;쀀⊔︀uĀbp㑿㒏ƀ;esᆗᆜ㒆etĀ;eᆗ㒍ñᆝƀ;esᆨᆭ㒖etĀ;eᆨ㒝ñᆮƀ;afᅻ㒦ְrť㒫ֱ»ᅼaròᅈȀcemt㒹㒾㓂㓅r;쀀𝓈tmîñiì㐕aræᆾĀar㓎㓕rĀ;f㓔ឿ昆Āan㓚㓭ightĀep㓣㓪psiloîỠhé⺯s»⡒ʀbcmnp㓻㕞ሉ㖋㖎Ҁ;Edemnprs㔎㔏㔑㔕㔞㔣㔬㔱㔶抂;櫅ot;檽Ā;dᇚ㔚ot;櫃ult;櫁ĀEe㔨㔪;櫋;把lus;檿arr;楹ƀeiu㔽㕒㕕tƀ;en㔎㕅㕋qĀ;qᇚ㔏eqĀ;q㔫㔨m;櫇Ābp㕚㕜;櫕;櫓c̀;acensᇭ㕬㕲㕹㕻㌦pproø㋺urlyeñᇾñᇳƀaes㖂㖈㌛pproø㌚qñ㌗g;晪ڀ123;Edehlmnps㖩㖬㖯ሜ㖲㖴㗀㗉㗕㗚㗟㗨㗭耻¹䂹耻²䂲耻³䂳;櫆Āos㖹㖼t;檾ub;櫘Ā;dሢ㗅ot;櫄sĀou㗏㗒l;柉b;櫗arr;楻ult;櫂ĀEe㗤㗦;櫌;抋lus;櫀ƀeiu㗴㘉㘌tƀ;enሜ㗼㘂qĀ;qሢ㖲eqĀ;q㗧㗤m;櫈Ābp㘑㘓;櫔;櫖ƀAan㘜㘠㘭rr;懙rĀhr㘦㘨ë∮Ā;oਫ਩war;椪lig耻ß䃟௡㙑㙝㙠ዎ㙳㙹\0㙾㛂\0\0\0\0\0㛛㜃\0㜉㝬\0\0\0㞇ɲ㙖\0\0㙛get;挖;䏄rë๟ƀaey㙦㙫㙰ron;䅥dil;䅣;䑂lrec;挕r;쀀𝔱Ȁeiko㚆㚝㚵㚼ǲ㚋\0㚑eĀ4fኄኁaƀ;sv㚘㚙㚛䎸ym;䏑Ācn㚢㚲kĀas㚨㚮pproø዁im»ኬsðኞĀas㚺㚮ð዁rn耻þ䃾Ǭ̟㛆⋧es膀×;bd㛏㛐㛘䃗Ā;aᤏ㛕r;樱;樰ƀeps㛡㛣㜀á⩍Ȁ;bcf҆㛬㛰㛴ot;挶ir;櫱Ā;o㛹㛼쀀𝕥rk;櫚á㍢rime;怴ƀaip㜏㜒㝤dåቈ΀adempst㜡㝍㝀㝑㝗㝜㝟ngleʀ;dlqr㜰㜱㜶㝀㝂斵own»ᶻeftĀ;e⠀㜾ñम;扜ightĀ;e㊪㝋ñၚot;旬inus;樺lus;樹b;槍ime;樻ezium;揢ƀcht㝲㝽㞁Āry㝷㝻;쀀𝓉;䑆cy;䑛rok;䅧Āio㞋㞎xô᝷headĀlr㞗㞠eftarro÷ࡏightarrow»ཝऀAHabcdfghlmoprstuw㟐㟓㟗㟤㟰㟼㠎㠜㠣㠴㡑㡝㡫㢩㣌㣒㣪㣶ròϭar;楣Ācr㟜㟢ute耻ú䃺òᅐrǣ㟪\0㟭y;䑞ve;䅭Āiy㟵㟺rc耻û䃻;䑃ƀabh㠃㠆㠋ròᎭlac;䅱aòᏃĀir㠓㠘sht;楾;쀀𝔲rave耻ù䃹š㠧㠱rĀlr㠬㠮»ॗ»ႃlk;斀Āct㠹㡍ɯ㠿\0\0㡊rnĀ;e㡅㡆挜r»㡆op;挏ri;旸Āal㡖㡚cr;䅫肻¨͉Āgp㡢㡦on;䅳f;쀀𝕦̀adhlsuᅋ㡸㡽፲㢑㢠ownáᎳarpoonĀlr㢈㢌efô㠭ighô㠯iƀ;hl㢙㢚㢜䏅»ᏺon»㢚parrows;懈ƀcit㢰㣄㣈ɯ㢶\0\0㣁rnĀ;e㢼㢽挝r»㢽op;挎ng;䅯ri;旹cr;쀀𝓊ƀdir㣙㣝㣢ot;拰lde;䅩iĀ;f㜰㣨»᠓Āam㣯㣲rò㢨l耻ü䃼angle;榧ހABDacdeflnoprsz㤜㤟㤩㤭㦵㦸㦽㧟㧤㧨㧳㧹㧽㨁㨠ròϷarĀ;v㤦㤧櫨;櫩asèϡĀnr㤲㤷grt;榜΀eknprst㓣㥆㥋㥒㥝㥤㦖appá␕othinçẖƀhir㓫⻈㥙opô⾵Ā;hᎷ㥢ïㆍĀiu㥩㥭gmá㎳Ābp㥲㦄setneqĀ;q㥽㦀쀀⊊︀;쀀⫋︀setneqĀ;q㦏㦒쀀⊋︀;쀀⫌︀Āhr㦛㦟etá㚜iangleĀlr㦪㦯eft»थight»ၑy;䐲ash»ံƀelr㧄㧒㧗ƀ;beⷪ㧋㧏ar;抻q;扚lip;拮Ābt㧜ᑨaòᑩr;쀀𝔳tré㦮suĀbp㧯㧱»ജ»൙pf;쀀𝕧roð໻tré㦴Ācu㨆㨋r;쀀𝓋Ābp㨐㨘nĀEe㦀㨖»㥾nĀEe㦒㨞»㦐igzag;榚΀cefoprs㨶㨻㩖㩛㩔㩡㩪irc;䅵Ādi㩀㩑Ābg㩅㩉ar;機eĀ;qᗺ㩏;扙erp;愘r;쀀𝔴pf;쀀𝕨Ā;eᑹ㩦atèᑹcr;쀀𝓌ૣណ㪇\0㪋\0㪐㪛\0\0㪝㪨㪫㪯\0\0㫃㫎\0㫘ៜ៟tré៑r;쀀𝔵ĀAa㪔㪗ròσrò৶;䎾ĀAa㪡㪤ròθrò৫að✓is;拻ƀdptឤ㪵㪾Āfl㪺ឩ;쀀𝕩imåឲĀAa㫇㫊ròώròਁĀcq㫒ីr;쀀𝓍Āpt៖㫜ré។Ѐacefiosu㫰㫽㬈㬌㬑㬕㬛㬡cĀuy㫶㫻te耻ý䃽;䑏Āiy㬂㬆rc;䅷;䑋n耻¥䂥r;쀀𝔶cy;䑗pf;쀀𝕪cr;쀀𝓎Ācm㬦㬩y;䑎l耻ÿ䃿Ԁacdefhiosw㭂㭈㭔㭘㭤㭩㭭㭴㭺㮀cute;䅺Āay㭍㭒ron;䅾;䐷ot;䅼Āet㭝㭡træᕟa;䎶r;쀀𝔷cy;䐶grarr;懝pf;쀀𝕫cr;쀀𝓏Ājn㮅㮇;怍j;怌'.split("").map(e=>e.charCodeAt(0))),w7e=new Map([[0,65533],[128,8364],[130,8218],[131,402],[132,8222],[133,8230],[134,8224],[135,8225],[136,710],[137,8240],[138,352],[139,8249],[140,338],[142,381],[145,8216],[146,8217],[147,8220],[148,8221],[149,8226],[150,8211],[151,8212],[152,732],[153,8482],[154,353],[155,8250],[156,339],[158,382],[159,376]]);function k7e(e){var t;return e>=55296&&e<=57343||e>1114111?65533:(t=w7e.get(e))!==null&&t!==void 0?t:e}var sa;(function(e){e[e.NUM=35]="NUM",e[e.SEMI=59]="SEMI",e[e.EQUALS=61]="EQUALS",e[e.ZERO=48]="ZERO",e[e.NINE=57]="NINE",e[e.LOWER_A=97]="LOWER_A",e[e.LOWER_F=102]="LOWER_F",e[e.LOWER_X=120]="LOWER_X",e[e.LOWER_Z=122]="LOWER_Z",e[e.UPPER_A=65]="UPPER_A",e[e.UPPER_F=70]="UPPER_F",e[e.UPPER_Z=90]="UPPER_Z"})(sa||(sa={}));const C7e=32;var Su;(function(e){e[e.VALUE_LENGTH=49152]="VALUE_LENGTH",e[e.BRANCH_LENGTH=16256]="BRANCH_LENGTH",e[e.JUMP_TABLE=127]="JUMP_TABLE"})(Su||(Su={}));function YR(e){return e>=sa.ZERO&&e<=sa.NINE}function T7e(e){return e>=sa.UPPER_A&&e<=sa.UPPER_F||e>=sa.LOWER_A&&e<=sa.LOWER_F}function A7e(e){return e>=sa.UPPER_A&&e<=sa.UPPER_Z||e>=sa.LOWER_A&&e<=sa.LOWER_Z||YR(e)}function F7e(e){return e===sa.EQUALS||A7e(e)}var ia;(function(e){e[e.EntityStart=0]="EntityStart",e[e.NumericStart=1]="NumericStart",e[e.NumericDecimal=2]="NumericDecimal",e[e.NumericHex=3]="NumericHex",e[e.NamedEntity=4]="NamedEntity"})(ia||(ia={}));var il;(function(e){e[e.Legacy=0]="Legacy",e[e.Strict=1]="Strict",e[e.Attribute=2]="Attribute"})(il||(il={}));class _7e{constructor(t,r,n){this.decodeTree=t,this.emitCodePoint=r,this.errors=n,this.state=ia.EntityStart,this.consumed=1,this.result=0,this.treeIndex=0,this.excess=1,this.decodeMode=il.Strict}startEntity(t){this.decodeMode=t,this.state=ia.EntityStart,this.result=0,this.treeIndex=0,this.excess=1,this.consumed=1}write(t,r){switch(this.state){case ia.EntityStart:return t.charCodeAt(r)===sa.NUM?(this.state=ia.NumericStart,this.consumed+=1,this.stateNumericStart(t,r+1)):(this.state=ia.NamedEntity,this.stateNamedEntity(t,r));case ia.NumericStart:return this.stateNumericStart(t,r);case ia.NumericDecimal:return this.stateNumericDecimal(t,r);case ia.NumericHex:return this.stateNumericHex(t,r);case ia.NamedEntity:return this.stateNamedEntity(t,r)}}stateNumericStart(t,r){return r>=t.length?-1:(t.charCodeAt(r)|C7e)===sa.LOWER_X?(this.state=ia.NumericHex,this.consumed+=1,this.stateNumericHex(t,r+1)):(this.state=ia.NumericDecimal,this.stateNumericDecimal(t,r))}addToNumericResult(t,r,n,a){if(r!==n){const i=n-r;this.result=this.result*Math.pow(a,i)+Number.parseInt(t.substr(r,i),a),this.consumed+=i}}stateNumericHex(t,r){const n=r;for(;r<t.length;){const a=t.charCodeAt(r);if(YR(a)||T7e(a))r+=1;else return this.addToNumericResult(t,n,r,16),this.emitNumericEntity(a,3)}return this.addToNumericResult(t,n,r,16),-1}stateNumericDecimal(t,r){const n=r;for(;r<t.length;){const a=t.charCodeAt(r);if(YR(a))r+=1;else return this.addToNumericResult(t,n,r,10),this.emitNumericEntity(a,2)}return this.addToNumericResult(t,n,r,10),-1}emitNumericEntity(t,r){var n;if(this.consumed<=r)return(n=this.errors)===null||n===void 0||n.absenceOfDigitsInNumericCharacterReference(this.consumed),0;if(t===sa.SEMI)this.consumed+=1;else if(this.decodeMode===il.Strict)return 0;return this.emitCodePoint(k7e(this.result),this.consumed),this.errors&&(t!==sa.SEMI&&this.errors.missingSemicolonAfterCharacterReference(),this.errors.validateNumericCharacterReference(this.result)),this.consumed}stateNamedEntity(t,r){const{decodeTree:n}=this;let a=n[this.treeIndex],i=(a&Su.VALUE_LENGTH)>>14;for(;r<t.length;r++,this.excess++){const o=t.charCodeAt(r);if(this.treeIndex=R7e(n,a,this.treeIndex+Math.max(1,i),o),this.treeIndex<0)return this.result===0||this.decodeMode===il.Attribute&&(i===0||F7e(o))?0:this.emitNotTerminatedNamedEntity();if(a=n[this.treeIndex],i=(a&Su.VALUE_LENGTH)>>14,i!==0){if(o===sa.SEMI)return this.emitNamedEntityData(this.treeIndex,i,this.consumed+this.excess);this.decodeMode!==il.Strict&&(this.result=this.treeIndex,this.consumed+=this.excess,this.excess=0)}}return-1}emitNotTerminatedNamedEntity(){var t;const{result:r,decodeTree:n}=this,a=(n[r]&Su.VALUE_LENGTH)>>14;return this.emitNamedEntityData(r,a,this.consumed),(t=this.errors)===null||t===void 0||t.missingSemicolonAfterCharacterReference(),this.consumed}emitNamedEntityData(t,r,n){const{decodeTree:a}=this;return this.emitCodePoint(r===1?a[t]&~Su.VALUE_LENGTH:a[t+1],n),r===3&&this.emitCodePoint(a[t+2],n),n}end(){var t;switch(this.state){case ia.NamedEntity:return this.result!==0&&(this.decodeMode!==il.Attribute||this.result===this.treeIndex)?this.emitNotTerminatedNamedEntity():0;case ia.NumericDecimal:return this.emitNumericEntity(0,2);case ia.NumericHex:return this.emitNumericEntity(0,3);case ia.NumericStart:return(t=this.errors)===null||t===void 0||t.absenceOfDigitsInNumericCharacterReference(this.consumed),0;case ia.EntityStart:return 0}}}function R7e(e,t,r,n){const a=(t&Su.BRANCH_LENGTH)>>7,i=t&Su.JUMP_TABLE;if(a===0)return i!==0&&n===i?r:-1;if(i){const l=n-i;return l<0||l>=a?-1:e[r+l]-1}let o=r,s=o+a-1;for(;o<=s;){const l=o+s>>>1,u=e[l];if(u<n)o=l+1;else if(u>n)s=l-1;else return e[l+a]}return-1}var Xe;(function(e){e.HTML="http://www.w3.org/1999/xhtml",e.MATHML="http://www.w3.org/1998/Math/MathML",e.SVG="http://www.w3.org/2000/svg",e.XLINK="http://www.w3.org/1999/xlink",e.XML="http://www.w3.org/XML/1998/namespace",e.XMLNS="http://www.w3.org/2000/xmlns/"})(Xe||(Xe={}));var Dc;(function(e){e.TYPE="type",e.ACTION="action",e.ENCODING="encoding",e.PROMPT="prompt",e.NAME="name",e.COLOR="color",e.FACE="face",e.SIZE="size"})(Dc||(Dc={}));var ro;(function(e){e.NO_QUIRKS="no-quirks",e.QUIRKS="quirks",e.LIMITED_QUIRKS="limited-quirks"})(ro||(ro={}));var ke;(function(e){e.A="a",e.ADDRESS="address",e.ANNOTATION_XML="annotation-xml",e.APPLET="applet",e.AREA="area",e.ARTICLE="article",e.ASIDE="aside",e.B="b",e.BASE="base",e.BASEFONT="basefont",e.BGSOUND="bgsound",e.BIG="big",e.BLOCKQUOTE="blockquote",e.BODY="body",e.BR="br",e.BUTTON="button",e.CAPTION="caption",e.CENTER="center",e.CODE="code",e.COL="col",e.COLGROUP="colgroup",e.DD="dd",e.DESC="desc",e.DETAILS="details",e.DIALOG="dialog",e.DIR="dir",e.DIV="div",e.DL="dl",e.DT="dt",e.EM="em",e.EMBED="embed",e.FIELDSET="fieldset",e.FIGCAPTION="figcaption",e.FIGURE="figure",e.FONT="font",e.FOOTER="footer",e.FOREIGN_OBJECT="foreignObject",e.FORM="form",e.FRAME="frame",e.FRAMESET="frameset",e.H1="h1",e.H2="h2",e.H3="h3",e.H4="h4",e.H5="h5",e.H6="h6",e.HEAD="head",e.HEADER="header",e.HGROUP="hgroup",e.HR="hr",e.HTML="html",e.I="i",e.IMG="img",e.IMAGE="image",e.INPUT="input",e.IFRAME="iframe",e.KEYGEN="keygen",e.LABEL="label",e.LI="li",e.LINK="link",e.LISTING="listing",e.MAIN="main",e.MALIGNMARK="malignmark",e.MARQUEE="marquee",e.MATH="math",e.MENU="menu",e.META="meta",e.MGLYPH="mglyph",e.MI="mi",e.MO="mo",e.MN="mn",e.MS="ms",e.MTEXT="mtext",e.NAV="nav",e.NOBR="nobr",e.NOFRAMES="noframes",e.NOEMBED="noembed",e.NOSCRIPT="noscript",e.OBJECT="object",e.OL="ol",e.OPTGROUP="optgroup",e.OPTION="option",e.P="p",e.PARAM="param",e.PLAINTEXT="plaintext",e.PRE="pre",e.RB="rb",e.RP="rp",e.RT="rt",e.RTC="rtc",e.RUBY="ruby",e.S="s",e.SCRIPT="script",e.SEARCH="search",e.SECTION="section",e.SELECT="select",e.SOURCE="source",e.SMALL="small",e.SPAN="span",e.STRIKE="strike",e.STRONG="strong",e.STYLE="style",e.SUB="sub",e.SUMMARY="summary",e.SUP="sup",e.TABLE="table",e.TBODY="tbody",e.TEMPLATE="template",e.TEXTAREA="textarea",e.TFOOT="tfoot",e.TD="td",e.TH="th",e.THEAD="thead",e.TITLE="title",e.TR="tr",e.TRACK="track",e.TT="tt",e.U="u",e.UL="ul",e.SVG="svg",e.VAR="var",e.WBR="wbr",e.XMP="xmp"})(ke||(ke={}));var N;(function(e){e[e.UNKNOWN=0]="UNKNOWN",e[e.A=1]="A",e[e.ADDRESS=2]="ADDRESS",e[e.ANNOTATION_XML=3]="ANNOTATION_XML",e[e.APPLET=4]="APPLET",e[e.AREA=5]="AREA",e[e.ARTICLE=6]="ARTICLE",e[e.ASIDE=7]="ASIDE",e[e.B=8]="B",e[e.BASE=9]="BASE",e[e.BASEFONT=10]="BASEFONT",e[e.BGSOUND=11]="BGSOUND",e[e.BIG=12]="BIG",e[e.BLOCKQUOTE=13]="BLOCKQUOTE",e[e.BODY=14]="BODY",e[e.BR=15]="BR",e[e.BUTTON=16]="BUTTON",e[e.CAPTION=17]="CAPTION",e[e.CENTER=18]="CENTER",e[e.CODE=19]="CODE",e[e.COL=20]="COL",e[e.COLGROUP=21]="COLGROUP",e[e.DD=22]="DD",e[e.DESC=23]="DESC",e[e.DETAILS=24]="DETAILS",e[e.DIALOG=25]="DIALOG",e[e.DIR=26]="DIR",e[e.DIV=27]="DIV",e[e.DL=28]="DL",e[e.DT=29]="DT",e[e.EM=30]="EM",e[e.EMBED=31]="EMBED",e[e.FIELDSET=32]="FIELDSET",e[e.FIGCAPTION=33]="FIGCAPTION",e[e.FIGURE=34]="FIGURE",e[e.FONT=35]="FONT",e[e.FOOTER=36]="FOOTER",e[e.FOREIGN_OBJECT=37]="FOREIGN_OBJECT",e[e.FORM=38]="FORM",e[e.FRAME=39]="FRAME",e[e.FRAMESET=40]="FRAMESET",e[e.H1=41]="H1",e[e.H2=42]="H2",e[e.H3=43]="H3",e[e.H4=44]="H4",e[e.H5=45]="H5",e[e.H6=46]="H6",e[e.HEAD=47]="HEAD",e[e.HEADER=48]="HEADER",e[e.HGROUP=49]="HGROUP",e[e.HR=50]="HR",e[e.HTML=51]="HTML",e[e.I=52]="I",e[e.IMG=53]="IMG",e[e.IMAGE=54]="IMAGE",e[e.INPUT=55]="INPUT",e[e.IFRAME=56]="IFRAME",e[e.KEYGEN=57]="KEYGEN",e[e.LABEL=58]="LABEL",e[e.LI=59]="LI",e[e.LINK=60]="LINK",e[e.LISTING=61]="LISTING",e[e.MAIN=62]="MAIN",e[e.MALIGNMARK=63]="MALIGNMARK",e[e.MARQUEE=64]="MARQUEE",e[e.MATH=65]="MATH",e[e.MENU=66]="MENU",e[e.META=67]="META",e[e.MGLYPH=68]="MGLYPH",e[e.MI=69]="MI",e[e.MO=70]="MO",e[e.MN=71]="MN",e[e.MS=72]="MS",e[e.MTEXT=73]="MTEXT",e[e.NAV=74]="NAV",e[e.NOBR=75]="NOBR",e[e.NOFRAMES=76]="NOFRAMES",e[e.NOEMBED=77]="NOEMBED",e[e.NOSCRIPT=78]="NOSCRIPT",e[e.OBJECT=79]="OBJECT",e[e.OL=80]="OL",e[e.OPTGROUP=81]="OPTGROUP",e[e.OPTION=82]="OPTION",e[e.P=83]="P",e[e.PARAM=84]="PARAM",e[e.PLAINTEXT=85]="PLAINTEXT",e[e.PRE=86]="PRE",e[e.RB=87]="RB",e[e.RP=88]="RP",e[e.RT=89]="RT",e[e.RTC=90]="RTC",e[e.RUBY=91]="RUBY",e[e.S=92]="S",e[e.SCRIPT=93]="SCRIPT",e[e.SEARCH=94]="SEARCH",e[e.SECTION=95]="SECTION",e[e.SELECT=96]="SELECT",e[e.SOURCE=97]="SOURCE",e[e.SMALL=98]="SMALL",e[e.SPAN=99]="SPAN",e[e.STRIKE=100]="STRIKE",e[e.STRONG=101]="STRONG",e[e.STYLE=102]="STYLE",e[e.SUB=103]="SUB",e[e.SUMMARY=104]="SUMMARY",e[e.SUP=105]="SUP",e[e.TABLE=106]="TABLE",e[e.TBODY=107]="TBODY",e[e.TEMPLATE=108]="TEMPLATE",e[e.TEXTAREA=109]="TEXTAREA",e[e.TFOOT=110]="TFOOT",e[e.TD=111]="TD",e[e.TH=112]="TH",e[e.THEAD=113]="THEAD",e[e.TITLE=114]="TITLE",e[e.TR=115]="TR",e[e.TRACK=116]="TRACK",e[e.TT=117]="TT",e[e.U=118]="U",e[e.UL=119]="UL",e[e.SVG=120]="SVG",e[e.VAR=121]="VAR",e[e.WBR=122]="WBR",e[e.XMP=123]="XMP"})(N||(N={}));const D7e=new Map([[ke.A,N.A],[ke.ADDRESS,N.ADDRESS],[ke.ANNOTATION_XML,N.ANNOTATION_XML],[ke.APPLET,N.APPLET],[ke.AREA,N.AREA],[ke.ARTICLE,N.ARTICLE],[ke.ASIDE,N.ASIDE],[ke.B,N.B],[ke.BASE,N.BASE],[ke.BASEFONT,N.BASEFONT],[ke.BGSOUND,N.BGSOUND],[ke.BIG,N.BIG],[ke.BLOCKQUOTE,N.BLOCKQUOTE],[ke.BODY,N.BODY],[ke.BR,N.BR],[ke.BUTTON,N.BUTTON],[ke.CAPTION,N.CAPTION],[ke.CENTER,N.CENTER],[ke.CODE,N.CODE],[ke.COL,N.COL],[ke.COLGROUP,N.COLGROUP],[ke.DD,N.DD],[ke.DESC,N.DESC],[ke.DETAILS,N.DETAILS],[ke.DIALOG,N.DIALOG],[ke.DIR,N.DIR],[ke.DIV,N.DIV],[ke.DL,N.DL],[ke.DT,N.DT],[ke.EM,N.EM],[ke.EMBED,N.EMBED],[ke.FIELDSET,N.FIELDSET],[ke.FIGCAPTION,N.FIGCAPTION],[ke.FIGURE,N.FIGURE],[ke.FONT,N.FONT],[ke.FOOTER,N.FOOTER],[ke.FOREIGN_OBJECT,N.FOREIGN_OBJECT],[ke.FORM,N.FORM],[ke.FRAME,N.FRAME],[ke.FRAMESET,N.FRAMESET],[ke.H1,N.H1],[ke.H2,N.H2],[ke.H3,N.H3],[ke.H4,N.H4],[ke.H5,N.H5],[ke.H6,N.H6],[ke.HEAD,N.HEAD],[ke.HEADER,N.HEADER],[ke.HGROUP,N.HGROUP],[ke.HR,N.HR],[ke.HTML,N.HTML],[ke.I,N.I],[ke.IMG,N.IMG],[ke.IMAGE,N.IMAGE],[ke.INPUT,N.INPUT],[ke.IFRAME,N.IFRAME],[ke.KEYGEN,N.KEYGEN],[ke.LABEL,N.LABEL],[ke.LI,N.LI],[ke.LINK,N.LINK],[ke.LISTING,N.LISTING],[ke.MAIN,N.MAIN],[ke.MALIGNMARK,N.MALIGNMARK],[ke.MARQUEE,N.MARQUEE],[ke.MATH,N.MATH],[ke.MENU,N.MENU],[ke.META,N.META],[ke.MGLYPH,N.MGLYPH],[ke.MI,N.MI],[ke.MO,N.MO],[ke.MN,N.MN],[ke.MS,N.MS],[ke.MTEXT,N.MTEXT],[ke.NAV,N.NAV],[ke.NOBR,N.NOBR],[ke.NOFRAMES,N.NOFRAMES],[ke.NOEMBED,N.NOEMBED],[ke.NOSCRIPT,N.NOSCRIPT],[ke.OBJECT,N.OBJECT],[ke.OL,N.OL],[ke.OPTGROUP,N.OPTGROUP],[ke.OPTION,N.OPTION],[ke.P,N.P],[ke.PARAM,N.PARAM],[ke.PLAINTEXT,N.PLAINTEXT],[ke.PRE,N.PRE],[ke.RB,N.RB],[ke.RP,N.RP],[ke.RT,N.RT],[ke.RTC,N.RTC],[ke.RUBY,N.RUBY],[ke.S,N.S],[ke.SCRIPT,N.SCRIPT],[ke.SEARCH,N.SEARCH],[ke.SECTION,N.SECTION],[ke.SELECT,N.SELECT],[ke.SOURCE,N.SOURCE],[ke.SMALL,N.SMALL],[ke.SPAN,N.SPAN],[ke.STRIKE,N.STRIKE],[ke.STRONG,N.STRONG],[ke.STYLE,N.STYLE],[ke.SUB,N.SUB],[ke.SUMMARY,N.SUMMARY],[ke.SUP,N.SUP],[ke.TABLE,N.TABLE],[ke.TBODY,N.TBODY],[ke.TEMPLATE,N.TEMPLATE],[ke.TEXTAREA,N.TEXTAREA],[ke.TFOOT,N.TFOOT],[ke.TD,N.TD],[ke.TH,N.TH],[ke.THEAD,N.THEAD],[ke.TITLE,N.TITLE],[ke.TR,N.TR],[ke.TRACK,N.TRACK],[ke.TT,N.TT],[ke.U,N.U],[ke.UL,N.UL],[ke.SVG,N.SVG],[ke.VAR,N.VAR],[ke.WBR,N.WBR],[ke.XMP,N.XMP]]);function Ap(e){var t;return(t=D7e.get(e))!==null&&t!==void 0?t:N.UNKNOWN}const et=N,N7e={[Xe.HTML]:new Set([et.ADDRESS,et.APPLET,et.AREA,et.ARTICLE,et.ASIDE,et.BASE,et.BASEFONT,et.BGSOUND,et.BLOCKQUOTE,et.BODY,et.BR,et.BUTTON,et.CAPTION,et.CENTER,et.COL,et.COLGROUP,et.DD,et.DETAILS,et.DIR,et.DIV,et.DL,et.DT,et.EMBED,et.FIELDSET,et.FIGCAPTION,et.FIGURE,et.FOOTER,et.FORM,et.FRAME,et.FRAMESET,et.H1,et.H2,et.H3,et.H4,et.H5,et.H6,et.HEAD,et.HEADER,et.HGROUP,et.HR,et.HTML,et.IFRAME,et.IMG,et.INPUT,et.LI,et.LINK,et.LISTING,et.MAIN,et.MARQUEE,et.MENU,et.META,et.NAV,et.NOEMBED,et.NOFRAMES,et.NOSCRIPT,et.OBJECT,et.OL,et.P,et.PARAM,et.PLAINTEXT,et.PRE,et.SCRIPT,et.SECTION,et.SELECT,et.SOURCE,et.STYLE,et.SUMMARY,et.TABLE,et.TBODY,et.TD,et.TEMPLATE,et.TEXTAREA,et.TFOOT,et.TH,et.THEAD,et.TITLE,et.TR,et.TRACK,et.UL,et.WBR,et.XMP]),[Xe.MATHML]:new Set([et.MI,et.MO,et.MN,et.MS,et.MTEXT,et.ANNOTATION_XML]),[Xe.SVG]:new Set([et.TITLE,et.FOREIGN_OBJECT,et.DESC]),[Xe.XLINK]:new Set,[Xe.XML]:new Set,[Xe.XMLNS]:new Set},JR=new Set([et.H1,et.H2,et.H3,et.H4,et.H5,et.H6]);ke.STYLE,ke.SCRIPT,ke.XMP,ke.IFRAME,ke.NOEMBED,ke.NOFRAMES,ke.PLAINTEXT;var ie;(function(e){e[e.DATA=0]="DATA",e[e.RCDATA=1]="RCDATA",e[e.RAWTEXT=2]="RAWTEXT",e[e.SCRIPT_DATA=3]="SCRIPT_DATA",e[e.PLAINTEXT=4]="PLAINTEXT",e[e.TAG_OPEN=5]="TAG_OPEN",e[e.END_TAG_OPEN=6]="END_TAG_OPEN",e[e.TAG_NAME=7]="TAG_NAME",e[e.RCDATA_LESS_THAN_SIGN=8]="RCDATA_LESS_THAN_SIGN",e[e.RCDATA_END_TAG_OPEN=9]="RCDATA_END_TAG_OPEN",e[e.RCDATA_END_TAG_NAME=10]="RCDATA_END_TAG_NAME",e[e.RAWTEXT_LESS_THAN_SIGN=11]="RAWTEXT_LESS_THAN_SIGN",e[e.RAWTEXT_END_TAG_OPEN=12]="RAWTEXT_END_TAG_OPEN",e[e.RAWTEXT_END_TAG_NAME=13]="RAWTEXT_END_TAG_NAME",e[e.SCRIPT_DATA_LESS_THAN_SIGN=14]="SCRIPT_DATA_LESS_THAN_SIGN",e[e.SCRIPT_DATA_END_TAG_OPEN=15]="SCRIPT_DATA_END_TAG_OPEN",e[e.SCRIPT_DATA_END_TAG_NAME=16]="SCRIPT_DATA_END_TAG_NAME",e[e.SCRIPT_DATA_ESCAPE_START=17]="SCRIPT_DATA_ESCAPE_START",e[e.SCRIPT_DATA_ESCAPE_START_DASH=18]="SCRIPT_DATA_ESCAPE_START_DASH",e[e.SCRIPT_DATA_ESCAPED=19]="SCRIPT_DATA_ESCAPED",e[e.SCRIPT_DATA_ESCAPED_DASH=20]="SCRIPT_DATA_ESCAPED_DASH",e[e.SCRIPT_DATA_ESCAPED_DASH_DASH=21]="SCRIPT_DATA_ESCAPED_DASH_DASH",e[e.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN=22]="SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN",e[e.SCRIPT_DATA_ESCAPED_END_TAG_OPEN=23]="SCRIPT_DATA_ESCAPED_END_TAG_OPEN",e[e.SCRIPT_DATA_ESCAPED_END_TAG_NAME=24]="SCRIPT_DATA_ESCAPED_END_TAG_NAME",e[e.SCRIPT_DATA_DOUBLE_ESCAPE_START=25]="SCRIPT_DATA_DOUBLE_ESCAPE_START",e[e.SCRIPT_DATA_DOUBLE_ESCAPED=26]="SCRIPT_DATA_DOUBLE_ESCAPED",e[e.SCRIPT_DATA_DOUBLE_ESCAPED_DASH=27]="SCRIPT_DATA_DOUBLE_ESCAPED_DASH",e[e.SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH=28]="SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH",e[e.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN=29]="SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN",e[e.SCRIPT_DATA_DOUBLE_ESCAPE_END=30]="SCRIPT_DATA_DOUBLE_ESCAPE_END",e[e.BEFORE_ATTRIBUTE_NAME=31]="BEFORE_ATTRIBUTE_NAME",e[e.ATTRIBUTE_NAME=32]="ATTRIBUTE_NAME",e[e.AFTER_ATTRIBUTE_NAME=33]="AFTER_ATTRIBUTE_NAME",e[e.BEFORE_ATTRIBUTE_VALUE=34]="BEFORE_ATTRIBUTE_VALUE",e[e.ATTRIBUTE_VALUE_DOUBLE_QUOTED=35]="ATTRIBUTE_VALUE_DOUBLE_QUOTED",e[e.ATTRIBUTE_VALUE_SINGLE_QUOTED=36]="ATTRIBUTE_VALUE_SINGLE_QUOTED",e[e.ATTRIBUTE_VALUE_UNQUOTED=37]="ATTRIBUTE_VALUE_UNQUOTED",e[e.AFTER_ATTRIBUTE_VALUE_QUOTED=38]="AFTER_ATTRIBUTE_VALUE_QUOTED",e[e.SELF_CLOSING_START_TAG=39]="SELF_CLOSING_START_TAG",e[e.BOGUS_COMMENT=40]="BOGUS_COMMENT",e[e.MARKUP_DECLARATION_OPEN=41]="MARKUP_DECLARATION_OPEN",e[e.COMMENT_START=42]="COMMENT_START",e[e.COMMENT_START_DASH=43]="COMMENT_START_DASH",e[e.COMMENT=44]="COMMENT",e[e.COMMENT_LESS_THAN_SIGN=45]="COMMENT_LESS_THAN_SIGN",e[e.COMMENT_LESS_THAN_SIGN_BANG=46]="COMMENT_LESS_THAN_SIGN_BANG",e[e.COMMENT_LESS_THAN_SIGN_BANG_DASH=47]="COMMENT_LESS_THAN_SIGN_BANG_DASH",e[e.COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH=48]="COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH",e[e.COMMENT_END_DASH=49]="COMMENT_END_DASH",e[e.COMMENT_END=50]="COMMENT_END",e[e.COMMENT_END_BANG=51]="COMMENT_END_BANG",e[e.DOCTYPE=52]="DOCTYPE",e[e.BEFORE_DOCTYPE_NAME=53]="BEFORE_DOCTYPE_NAME",e[e.DOCTYPE_NAME=54]="DOCTYPE_NAME",e[e.AFTER_DOCTYPE_NAME=55]="AFTER_DOCTYPE_NAME",e[e.AFTER_DOCTYPE_PUBLIC_KEYWORD=56]="AFTER_DOCTYPE_PUBLIC_KEYWORD",e[e.BEFORE_DOCTYPE_PUBLIC_IDENTIFIER=57]="BEFORE_DOCTYPE_PUBLIC_IDENTIFIER",e[e.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED=58]="DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED",e[e.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED=59]="DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED",e[e.AFTER_DOCTYPE_PUBLIC_IDENTIFIER=60]="AFTER_DOCTYPE_PUBLIC_IDENTIFIER",e[e.BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS=61]="BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS",e[e.AFTER_DOCTYPE_SYSTEM_KEYWORD=62]="AFTER_DOCTYPE_SYSTEM_KEYWORD",e[e.BEFORE_DOCTYPE_SYSTEM_IDENTIFIER=63]="BEFORE_DOCTYPE_SYSTEM_IDENTIFIER",e[e.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED=64]="DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED",e[e.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED=65]="DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED",e[e.AFTER_DOCTYPE_SYSTEM_IDENTIFIER=66]="AFTER_DOCTYPE_SYSTEM_IDENTIFIER",e[e.BOGUS_DOCTYPE=67]="BOGUS_DOCTYPE",e[e.CDATA_SECTION=68]="CDATA_SECTION",e[e.CDATA_SECTION_BRACKET=69]="CDATA_SECTION_BRACKET",e[e.CDATA_SECTION_END=70]="CDATA_SECTION_END",e[e.CHARACTER_REFERENCE=71]="CHARACTER_REFERENCE",e[e.AMBIGUOUS_AMPERSAND=72]="AMBIGUOUS_AMPERSAND"})(ie||(ie={}));const Dn={DATA:ie.DATA,RCDATA:ie.RCDATA,RAWTEXT:ie.RAWTEXT,SCRIPT_DATA:ie.SCRIPT_DATA,PLAINTEXT:ie.PLAINTEXT,CDATA_SECTION:ie.CDATA_SECTION};function L7e(e){return e>=re.DIGIT_0&&e<=re.DIGIT_9}function om(e){return e>=re.LATIN_CAPITAL_A&&e<=re.LATIN_CAPITAL_Z}function M7e(e){return e>=re.LATIN_SMALL_A&&e<=re.LATIN_SMALL_Z}function hu(e){return M7e(e)||om(e)}function fz(e){return hu(e)||L7e(e)}function Qb(e){return e+32}function Dne(e){return e===re.SPACE||e===re.LINE_FEED||e===re.TABULATION||e===re.FORM_FEED}function mz(e){return Dne(e)||e===re.SOLIDUS||e===re.GREATER_THAN_SIGN}function I7e(e){return e===re.NULL?Be.nullCharacterReference:e>1114111?Be.characterReferenceOutsideUnicodeRange:Ane(e)?Be.surrogateCharacterReference:_ne(e)?Be.noncharacterCharacterReference:Fne(e)||e===re.CARRIAGE_RETURN?Be.controlCharacterReference:null}class O7e{constructor(t,r){this.options=t,this.handler=r,this.paused=!1,this.inLoop=!1,this.inForeignNode=!1,this.lastStartTagName="",this.active=!1,this.state=ie.DATA,this.returnState=ie.DATA,this.entityStartPos=0,this.consumedAfterSnapshot=-1,this.currentCharacterToken=null,this.currentToken=null,this.currentAttr={name:"",value:""},this.preprocessor=new E7e(r),this.currentLocation=this.getCurrentLocation(-1),this.entityDecoder=new _7e(x7e,(n,a)=>{this.preprocessor.pos=this.entityStartPos+a-1,this._flushCodePointConsumedAsCharacterReference(n)},r.onParseError?{missingSemicolonAfterCharacterReference:()=>{this._err(Be.missingSemicolonAfterCharacterReference,1)},absenceOfDigitsInNumericCharacterReference:n=>{this._err(Be.absenceOfDigitsInNumericCharacterReference,this.entityStartPos-this.preprocessor.pos+n)},validateNumericCharacterReference:n=>{const a=I7e(n);a&&this._err(a,1)}}:void 0)}_err(t,r=0){var n,a;(a=(n=this.handler).onParseError)===null||a===void 0||a.call(n,this.preprocessor.getError(t,r))}getCurrentLocation(t){return this.options.sourceCodeLocationInfo?{startLine:this.preprocessor.line,startCol:this.preprocessor.col-t,startOffset:this.preprocessor.offset-t,endLine:-1,endCol:-1,endOffset:-1}:null}_runParsingLoop(){if(!this.inLoop){for(this.inLoop=!0;this.active&&!this.paused;){this.consumedAfterSnapshot=0;const t=this._consume();this._ensureHibernation()||this._callState(t)}this.inLoop=!1}}pause(){this.paused=!0}resume(t){if(!this.paused)throw new Error("Parser was already resumed");this.paused=!1,!this.inLoop&&(this._runParsingLoop(),this.paused||t==null||t())}write(t,r,n){this.active=!0,this.preprocessor.write(t,r),this._runParsingLoop(),this.paused||n==null||n()}insertHtmlAtCurrentPos(t){this.active=!0,this.preprocessor.insertHtmlAtCurrentPos(t),this._runParsingLoop()}_ensureHibernation(){return this.preprocessor.endOfChunkHit?(this.preprocessor.retreat(this.consumedAfterSnapshot),this.consumedAfterSnapshot=0,this.active=!1,!0):!1}_consume(){return this.consumedAfterSnapshot++,this.preprocessor.advance()}_advanceBy(t){this.consumedAfterSnapshot+=t;for(let r=0;r<t;r++)this.preprocessor.advance()}_consumeSequenceIfMatch(t,r){return this.preprocessor.startsWith(t,r)?(this._advanceBy(t.length-1),!0):!1}_createStartTagToken(){this.currentToken={type:Er.START_TAG,tagName:"",tagID:N.UNKNOWN,selfClosing:!1,ackSelfClosing:!1,attrs:[],location:this.getCurrentLocation(1)}}_createEndTagToken(){this.currentToken={type:Er.END_TAG,tagName:"",tagID:N.UNKNOWN,selfClosing:!1,ackSelfClosing:!1,attrs:[],location:this.getCurrentLocation(2)}}_createCommentToken(t){this.currentToken={type:Er.COMMENT,data:"",location:this.getCurrentLocation(t)}}_createDoctypeToken(t){this.currentToken={type:Er.DOCTYPE,name:t,forceQuirks:!1,publicId:null,systemId:null,location:this.currentLocation}}_createCharacterToken(t,r){this.currentCharacterToken={type:t,chars:r,location:this.currentLocation}}_createAttr(t){this.currentAttr={name:t,value:""},this.currentLocation=this.getCurrentLocation(0)}_leaveAttrName(){var t,r;const n=this.currentToken;if(Rne(n,this.currentAttr.name)===null){if(n.attrs.push(this.currentAttr),n.location&&this.currentLocation){const a=(t=(r=n.location).attrs)!==null&&t!==void 0?t:r.attrs=Object.create(null);a[this.currentAttr.name]=this.currentLocation,this._leaveAttrValue()}}else this._err(Be.duplicateAttribute)}_leaveAttrValue(){this.currentLocation&&(this.currentLocation.endLine=this.preprocessor.line,this.currentLocation.endCol=this.preprocessor.col,this.currentLocation.endOffset=this.preprocessor.offset)}prepareToken(t){this._emitCurrentCharacterToken(t.location),this.currentToken=null,t.location&&(t.location.endLine=this.preprocessor.line,t.location.endCol=this.preprocessor.col+1,t.location.endOffset=this.preprocessor.offset+1),this.currentLocation=this.getCurrentLocation(-1)}emitCurrentTagToken(){const t=this.currentToken;this.prepareToken(t),t.tagID=Ap(t.tagName),t.type===Er.START_TAG?(this.lastStartTagName=t.tagName,this.handler.onStartTag(t)):(t.attrs.length>0&&this._err(Be.endTagWithAttributes),t.selfClosing&&this._err(Be.endTagWithTrailingSolidus),this.handler.onEndTag(t)),this.preprocessor.dropParsedChunk()}emitCurrentComment(t){this.prepareToken(t),this.handler.onComment(t),this.preprocessor.dropParsedChunk()}emitCurrentDoctype(t){this.prepareToken(t),this.handler.onDoctype(t),this.preprocessor.dropParsedChunk()}_emitCurrentCharacterToken(t){if(this.currentCharacterToken){switch(t&&this.currentCharacterToken.location&&(this.currentCharacterToken.location.endLine=t.startLine,this.currentCharacterToken.location.endCol=t.startCol,this.currentCharacterToken.location.endOffset=t.startOffset),this.currentCharacterToken.type){case Er.CHARACTER:{this.handler.onCharacter(this.currentCharacterToken);break}case Er.NULL_CHARACTER:{this.handler.onNullCharacter(this.currentCharacterToken);break}case Er.WHITESPACE_CHARACTER:{this.handler.onWhitespaceCharacter(this.currentCharacterToken);break}}this.currentCharacterToken=null}}_emitEOFToken(){const t=this.getCurrentLocation(0);t&&(t.endLine=t.startLine,t.endCol=t.startCol,t.endOffset=t.startOffset),this._emitCurrentCharacterToken(t),this.handler.onEof({type:Er.EOF,location:t}),this.active=!1}_appendCharToCurrentCharacterToken(t,r){if(this.currentCharacterToken)if(this.currentCharacterToken.type===t){this.currentCharacterToken.chars+=r;return}else this.currentLocation=this.getCurrentLocation(0),this._emitCurrentCharacterToken(this.currentLocation),this.preprocessor.dropParsedChunk();this._createCharacterToken(t,r)}_emitCodePoint(t){const r=Dne(t)?Er.WHITESPACE_CHARACTER:t===re.NULL?Er.NULL_CHARACTER:Er.CHARACTER;this._appendCharToCurrentCharacterToken(r,String.fromCodePoint(t))}_emitChars(t){this._appendCharToCurrentCharacterToken(Er.CHARACTER,t)}_startCharacterReference(){this.returnState=this.state,this.state=ie.CHARACTER_REFERENCE,this.entityStartPos=this.preprocessor.pos,this.entityDecoder.startEntity(this._isCharacterReferenceInAttribute()?il.Attribute:il.Legacy)}_isCharacterReferenceInAttribute(){return this.returnState===ie.ATTRIBUTE_VALUE_DOUBLE_QUOTED||this.returnState===ie.ATTRIBUTE_VALUE_SINGLE_QUOTED||this.returnState===ie.ATTRIBUTE_VALUE_UNQUOTED}_flushCodePointConsumedAsCharacterReference(t){this._isCharacterReferenceInAttribute()?this.currentAttr.value+=String.fromCodePoint(t):this._emitCodePoint(t)}_callState(t){switch(this.state){case ie.DATA:{this._stateData(t);break}case ie.RCDATA:{this._stateRcdata(t);break}case ie.RAWTEXT:{this._stateRawtext(t);break}case ie.SCRIPT_DATA:{this._stateScriptData(t);break}case ie.PLAINTEXT:{this._statePlaintext(t);break}case ie.TAG_OPEN:{this._stateTagOpen(t);break}case ie.END_TAG_OPEN:{this._stateEndTagOpen(t);break}case ie.TAG_NAME:{this._stateTagName(t);break}case ie.RCDATA_LESS_THAN_SIGN:{this._stateRcdataLessThanSign(t);break}case ie.RCDATA_END_TAG_OPEN:{this._stateRcdataEndTagOpen(t);break}case ie.RCDATA_END_TAG_NAME:{this._stateRcdataEndTagName(t);break}case ie.RAWTEXT_LESS_THAN_SIGN:{this._stateRawtextLessThanSign(t);break}case ie.RAWTEXT_END_TAG_OPEN:{this._stateRawtextEndTagOpen(t);break}case ie.RAWTEXT_END_TAG_NAME:{this._stateRawtextEndTagName(t);break}case ie.SCRIPT_DATA_LESS_THAN_SIGN:{this._stateScriptDataLessThanSign(t);break}case ie.SCRIPT_DATA_END_TAG_OPEN:{this._stateScriptDataEndTagOpen(t);break}case ie.SCRIPT_DATA_END_TAG_NAME:{this._stateScriptDataEndTagName(t);break}case ie.SCRIPT_DATA_ESCAPE_START:{this._stateScriptDataEscapeStart(t);break}case ie.SCRIPT_DATA_ESCAPE_START_DASH:{this._stateScriptDataEscapeStartDash(t);break}case ie.SCRIPT_DATA_ESCAPED:{this._stateScriptDataEscaped(t);break}case ie.SCRIPT_DATA_ESCAPED_DASH:{this._stateScriptDataEscapedDash(t);break}case ie.SCRIPT_DATA_ESCAPED_DASH_DASH:{this._stateScriptDataEscapedDashDash(t);break}case ie.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN:{this._stateScriptDataEscapedLessThanSign(t);break}case ie.SCRIPT_DATA_ESCAPED_END_TAG_OPEN:{this._stateScriptDataEscapedEndTagOpen(t);break}case ie.SCRIPT_DATA_ESCAPED_END_TAG_NAME:{this._stateScriptDataEscapedEndTagName(t);break}case ie.SCRIPT_DATA_DOUBLE_ESCAPE_START:{this._stateScriptDataDoubleEscapeStart(t);break}case ie.SCRIPT_DATA_DOUBLE_ESCAPED:{this._stateScriptDataDoubleEscaped(t);break}case ie.SCRIPT_DATA_DOUBLE_ESCAPED_DASH:{this._stateScriptDataDoubleEscapedDash(t);break}case ie.SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH:{this._stateScriptDataDoubleEscapedDashDash(t);break}case ie.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN:{this._stateScriptDataDoubleEscapedLessThanSign(t);break}case ie.SCRIPT_DATA_DOUBLE_ESCAPE_END:{this._stateScriptDataDoubleEscapeEnd(t);break}case ie.BEFORE_ATTRIBUTE_NAME:{this._stateBeforeAttributeName(t);break}case ie.ATTRIBUTE_NAME:{this._stateAttributeName(t);break}case ie.AFTER_ATTRIBUTE_NAME:{this._stateAfterAttributeName(t);break}case ie.BEFORE_ATTRIBUTE_VALUE:{this._stateBeforeAttributeValue(t);break}case ie.ATTRIBUTE_VALUE_DOUBLE_QUOTED:{this._stateAttributeValueDoubleQuoted(t);break}case ie.ATTRIBUTE_VALUE_SINGLE_QUOTED:{this._stateAttributeValueSingleQuoted(t);break}case ie.ATTRIBUTE_VALUE_UNQUOTED:{this._stateAttributeValueUnquoted(t);break}case ie.AFTER_ATTRIBUTE_VALUE_QUOTED:{this._stateAfterAttributeValueQuoted(t);break}case ie.SELF_CLOSING_START_TAG:{this._stateSelfClosingStartTag(t);break}case ie.BOGUS_COMMENT:{this._stateBogusComment(t);break}case ie.MARKUP_DECLARATION_OPEN:{this._stateMarkupDeclarationOpen(t);break}case ie.COMMENT_START:{this._stateCommentStart(t);break}case ie.COMMENT_START_DASH:{this._stateCommentStartDash(t);break}case ie.COMMENT:{this._stateComment(t);break}case ie.COMMENT_LESS_THAN_SIGN:{this._stateCommentLessThanSign(t);break}case ie.COMMENT_LESS_THAN_SIGN_BANG:{this._stateCommentLessThanSignBang(t);break}case ie.COMMENT_LESS_THAN_SIGN_BANG_DASH:{this._stateCommentLessThanSignBangDash(t);break}case ie.COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH:{this._stateCommentLessThanSignBangDashDash(t);break}case ie.COMMENT_END_DASH:{this._stateCommentEndDash(t);break}case ie.COMMENT_END:{this._stateCommentEnd(t);break}case ie.COMMENT_END_BANG:{this._stateCommentEndBang(t);break}case ie.DOCTYPE:{this._stateDoctype(t);break}case ie.BEFORE_DOCTYPE_NAME:{this._stateBeforeDoctypeName(t);break}case ie.DOCTYPE_NAME:{this._stateDoctypeName(t);break}case ie.AFTER_DOCTYPE_NAME:{this._stateAfterDoctypeName(t);break}case ie.AFTER_DOCTYPE_PUBLIC_KEYWORD:{this._stateAfterDoctypePublicKeyword(t);break}case ie.BEFORE_DOCTYPE_PUBLIC_IDENTIFIER:{this._stateBeforeDoctypePublicIdentifier(t);break}case ie.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED:{this._stateDoctypePublicIdentifierDoubleQuoted(t);break}case ie.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED:{this._stateDoctypePublicIdentifierSingleQuoted(t);break}case ie.AFTER_DOCTYPE_PUBLIC_IDENTIFIER:{this._stateAfterDoctypePublicIdentifier(t);break}case ie.BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS:{this._stateBetweenDoctypePublicAndSystemIdentifiers(t);break}case ie.AFTER_DOCTYPE_SYSTEM_KEYWORD:{this._stateAfterDoctypeSystemKeyword(t);break}case ie.BEFORE_DOCTYPE_SYSTEM_IDENTIFIER:{this._stateBeforeDoctypeSystemIdentifier(t);break}case ie.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED:{this._stateDoctypeSystemIdentifierDoubleQuoted(t);break}case ie.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED:{this._stateDoctypeSystemIdentifierSingleQuoted(t);break}case ie.AFTER_DOCTYPE_SYSTEM_IDENTIFIER:{this._stateAfterDoctypeSystemIdentifier(t);break}case ie.BOGUS_DOCTYPE:{this._stateBogusDoctype(t);break}case ie.CDATA_SECTION:{this._stateCdataSection(t);break}case ie.CDATA_SECTION_BRACKET:{this._stateCdataSectionBracket(t);break}case ie.CDATA_SECTION_END:{this._stateCdataSectionEnd(t);break}case ie.CHARACTER_REFERENCE:{this._stateCharacterReference();break}case ie.AMBIGUOUS_AMPERSAND:{this._stateAmbiguousAmpersand(t);break}default:throw new Error("Unknown state")}}_stateData(t){switch(t){case re.LESS_THAN_SIGN:{this.state=ie.TAG_OPEN;break}case re.AMPERSAND:{this._startCharacterReference();break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this._emitCodePoint(t);break}case re.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateRcdata(t){switch(t){case re.AMPERSAND:{this._startCharacterReference();break}case re.LESS_THAN_SIGN:{this.state=ie.RCDATA_LESS_THAN_SIGN;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this._emitChars(mn);break}case re.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateRawtext(t){switch(t){case re.LESS_THAN_SIGN:{this.state=ie.RAWTEXT_LESS_THAN_SIGN;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this._emitChars(mn);break}case re.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateScriptData(t){switch(t){case re.LESS_THAN_SIGN:{this.state=ie.SCRIPT_DATA_LESS_THAN_SIGN;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this._emitChars(mn);break}case re.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_statePlaintext(t){switch(t){case re.NULL:{this._err(Be.unexpectedNullCharacter),this._emitChars(mn);break}case re.EOF:{this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateTagOpen(t){if(hu(t))this._createStartTagToken(),this.state=ie.TAG_NAME,this._stateTagName(t);else switch(t){case re.EXCLAMATION_MARK:{this.state=ie.MARKUP_DECLARATION_OPEN;break}case re.SOLIDUS:{this.state=ie.END_TAG_OPEN;break}case re.QUESTION_MARK:{this._err(Be.unexpectedQuestionMarkInsteadOfTagName),this._createCommentToken(1),this.state=ie.BOGUS_COMMENT,this._stateBogusComment(t);break}case re.EOF:{this._err(Be.eofBeforeTagName),this._emitChars("<"),this._emitEOFToken();break}default:this._err(Be.invalidFirstCharacterOfTagName),this._emitChars("<"),this.state=ie.DATA,this._stateData(t)}}_stateEndTagOpen(t){if(hu(t))this._createEndTagToken(),this.state=ie.TAG_NAME,this._stateTagName(t);else switch(t){case re.GREATER_THAN_SIGN:{this._err(Be.missingEndTagName),this.state=ie.DATA;break}case re.EOF:{this._err(Be.eofBeforeTagName),this._emitChars("</"),this._emitEOFToken();break}default:this._err(Be.invalidFirstCharacterOfTagName),this._createCommentToken(2),this.state=ie.BOGUS_COMMENT,this._stateBogusComment(t)}}_stateTagName(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this.state=ie.BEFORE_ATTRIBUTE_NAME;break}case re.SOLIDUS:{this.state=ie.SELF_CLOSING_START_TAG;break}case re.GREATER_THAN_SIGN:{this.state=ie.DATA,this.emitCurrentTagToken();break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.tagName+=mn;break}case re.EOF:{this._err(Be.eofInTag),this._emitEOFToken();break}default:r.tagName+=String.fromCodePoint(om(t)?Qb(t):t)}}_stateRcdataLessThanSign(t){t===re.SOLIDUS?this.state=ie.RCDATA_END_TAG_OPEN:(this._emitChars("<"),this.state=ie.RCDATA,this._stateRcdata(t))}_stateRcdataEndTagOpen(t){hu(t)?(this.state=ie.RCDATA_END_TAG_NAME,this._stateRcdataEndTagName(t)):(this._emitChars("</"),this.state=ie.RCDATA,this._stateRcdata(t))}handleSpecialEndTag(t){if(!this.preprocessor.startsWith(this.lastStartTagName,!1))return!this._ensureHibernation();this._createEndTagToken();const r=this.currentToken;switch(r.tagName=this.lastStartTagName,this.preprocessor.peek(this.lastStartTagName.length)){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:return this._advanceBy(this.lastStartTagName.length),this.state=ie.BEFORE_ATTRIBUTE_NAME,!1;case re.SOLIDUS:return this._advanceBy(this.lastStartTagName.length),this.state=ie.SELF_CLOSING_START_TAG,!1;case re.GREATER_THAN_SIGN:return this._advanceBy(this.lastStartTagName.length),this.emitCurrentTagToken(),this.state=ie.DATA,!1;default:return!this._ensureHibernation()}}_stateRcdataEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=ie.RCDATA,this._stateRcdata(t))}_stateRawtextLessThanSign(t){t===re.SOLIDUS?this.state=ie.RAWTEXT_END_TAG_OPEN:(this._emitChars("<"),this.state=ie.RAWTEXT,this._stateRawtext(t))}_stateRawtextEndTagOpen(t){hu(t)?(this.state=ie.RAWTEXT_END_TAG_NAME,this._stateRawtextEndTagName(t)):(this._emitChars("</"),this.state=ie.RAWTEXT,this._stateRawtext(t))}_stateRawtextEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=ie.RAWTEXT,this._stateRawtext(t))}_stateScriptDataLessThanSign(t){switch(t){case re.SOLIDUS:{this.state=ie.SCRIPT_DATA_END_TAG_OPEN;break}case re.EXCLAMATION_MARK:{this.state=ie.SCRIPT_DATA_ESCAPE_START,this._emitChars("<!");break}default:this._emitChars("<"),this.state=ie.SCRIPT_DATA,this._stateScriptData(t)}}_stateScriptDataEndTagOpen(t){hu(t)?(this.state=ie.SCRIPT_DATA_END_TAG_NAME,this._stateScriptDataEndTagName(t)):(this._emitChars("</"),this.state=ie.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=ie.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEscapeStart(t){t===re.HYPHEN_MINUS?(this.state=ie.SCRIPT_DATA_ESCAPE_START_DASH,this._emitChars("-")):(this.state=ie.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEscapeStartDash(t){t===re.HYPHEN_MINUS?(this.state=ie.SCRIPT_DATA_ESCAPED_DASH_DASH,this._emitChars("-")):(this.state=ie.SCRIPT_DATA,this._stateScriptData(t))}_stateScriptDataEscaped(t){switch(t){case re.HYPHEN_MINUS:{this.state=ie.SCRIPT_DATA_ESCAPED_DASH,this._emitChars("-");break}case re.LESS_THAN_SIGN:{this.state=ie.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this._emitChars(mn);break}case re.EOF:{this._err(Be.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateScriptDataEscapedDash(t){switch(t){case re.HYPHEN_MINUS:{this.state=ie.SCRIPT_DATA_ESCAPED_DASH_DASH,this._emitChars("-");break}case re.LESS_THAN_SIGN:{this.state=ie.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.state=ie.SCRIPT_DATA_ESCAPED,this._emitChars(mn);break}case re.EOF:{this._err(Be.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=ie.SCRIPT_DATA_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataEscapedDashDash(t){switch(t){case re.HYPHEN_MINUS:{this._emitChars("-");break}case re.LESS_THAN_SIGN:{this.state=ie.SCRIPT_DATA_ESCAPED_LESS_THAN_SIGN;break}case re.GREATER_THAN_SIGN:{this.state=ie.SCRIPT_DATA,this._emitChars(">");break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.state=ie.SCRIPT_DATA_ESCAPED,this._emitChars(mn);break}case re.EOF:{this._err(Be.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=ie.SCRIPT_DATA_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataEscapedLessThanSign(t){t===re.SOLIDUS?this.state=ie.SCRIPT_DATA_ESCAPED_END_TAG_OPEN:hu(t)?(this._emitChars("<"),this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPE_START,this._stateScriptDataDoubleEscapeStart(t)):(this._emitChars("<"),this.state=ie.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataEscapedEndTagOpen(t){hu(t)?(this.state=ie.SCRIPT_DATA_ESCAPED_END_TAG_NAME,this._stateScriptDataEscapedEndTagName(t)):(this._emitChars("</"),this.state=ie.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataEscapedEndTagName(t){this.handleSpecialEndTag(t)&&(this._emitChars("</"),this.state=ie.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataDoubleEscapeStart(t){if(this.preprocessor.startsWith(ai.SCRIPT,!1)&&mz(this.preprocessor.peek(ai.SCRIPT.length))){this._emitCodePoint(t);for(let r=0;r<ai.SCRIPT.length;r++)this._emitCodePoint(this._consume());this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED}else this._ensureHibernation()||(this.state=ie.SCRIPT_DATA_ESCAPED,this._stateScriptDataEscaped(t))}_stateScriptDataDoubleEscaped(t){switch(t){case re.HYPHEN_MINUS:{this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED_DASH,this._emitChars("-");break}case re.LESS_THAN_SIGN:{this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN,this._emitChars("<");break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this._emitChars(mn);break}case re.EOF:{this._err(Be.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateScriptDataDoubleEscapedDash(t){switch(t){case re.HYPHEN_MINUS:{this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED_DASH_DASH,this._emitChars("-");break}case re.LESS_THAN_SIGN:{this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN,this._emitChars("<");break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitChars(mn);break}case re.EOF:{this._err(Be.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataDoubleEscapedDashDash(t){switch(t){case re.HYPHEN_MINUS:{this._emitChars("-");break}case re.LESS_THAN_SIGN:{this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED_LESS_THAN_SIGN,this._emitChars("<");break}case re.GREATER_THAN_SIGN:{this.state=ie.SCRIPT_DATA,this._emitChars(">");break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitChars(mn);break}case re.EOF:{this._err(Be.eofInScriptHtmlCommentLikeText),this._emitEOFToken();break}default:this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED,this._emitCodePoint(t)}}_stateScriptDataDoubleEscapedLessThanSign(t){t===re.SOLIDUS?(this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPE_END,this._emitChars("/")):(this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED,this._stateScriptDataDoubleEscaped(t))}_stateScriptDataDoubleEscapeEnd(t){if(this.preprocessor.startsWith(ai.SCRIPT,!1)&&mz(this.preprocessor.peek(ai.SCRIPT.length))){this._emitCodePoint(t);for(let r=0;r<ai.SCRIPT.length;r++)this._emitCodePoint(this._consume());this.state=ie.SCRIPT_DATA_ESCAPED}else this._ensureHibernation()||(this.state=ie.SCRIPT_DATA_DOUBLE_ESCAPED,this._stateScriptDataDoubleEscaped(t))}_stateBeforeAttributeName(t){switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.SOLIDUS:case re.GREATER_THAN_SIGN:case re.EOF:{this.state=ie.AFTER_ATTRIBUTE_NAME,this._stateAfterAttributeName(t);break}case re.EQUALS_SIGN:{this._err(Be.unexpectedEqualsSignBeforeAttributeName),this._createAttr("="),this.state=ie.ATTRIBUTE_NAME;break}default:this._createAttr(""),this.state=ie.ATTRIBUTE_NAME,this._stateAttributeName(t)}}_stateAttributeName(t){switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:case re.SOLIDUS:case re.GREATER_THAN_SIGN:case re.EOF:{this._leaveAttrName(),this.state=ie.AFTER_ATTRIBUTE_NAME,this._stateAfterAttributeName(t);break}case re.EQUALS_SIGN:{this._leaveAttrName(),this.state=ie.BEFORE_ATTRIBUTE_VALUE;break}case re.QUOTATION_MARK:case re.APOSTROPHE:case re.LESS_THAN_SIGN:{this._err(Be.unexpectedCharacterInAttributeName),this.currentAttr.name+=String.fromCodePoint(t);break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.currentAttr.name+=mn;break}default:this.currentAttr.name+=String.fromCodePoint(om(t)?Qb(t):t)}}_stateAfterAttributeName(t){switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.SOLIDUS:{this.state=ie.SELF_CLOSING_START_TAG;break}case re.EQUALS_SIGN:{this.state=ie.BEFORE_ATTRIBUTE_VALUE;break}case re.GREATER_THAN_SIGN:{this.state=ie.DATA,this.emitCurrentTagToken();break}case re.EOF:{this._err(Be.eofInTag),this._emitEOFToken();break}default:this._createAttr(""),this.state=ie.ATTRIBUTE_NAME,this._stateAttributeName(t)}}_stateBeforeAttributeValue(t){switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.QUOTATION_MARK:{this.state=ie.ATTRIBUTE_VALUE_DOUBLE_QUOTED;break}case re.APOSTROPHE:{this.state=ie.ATTRIBUTE_VALUE_SINGLE_QUOTED;break}case re.GREATER_THAN_SIGN:{this._err(Be.missingAttributeValue),this.state=ie.DATA,this.emitCurrentTagToken();break}default:this.state=ie.ATTRIBUTE_VALUE_UNQUOTED,this._stateAttributeValueUnquoted(t)}}_stateAttributeValueDoubleQuoted(t){switch(t){case re.QUOTATION_MARK:{this.state=ie.AFTER_ATTRIBUTE_VALUE_QUOTED;break}case re.AMPERSAND:{this._startCharacterReference();break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.currentAttr.value+=mn;break}case re.EOF:{this._err(Be.eofInTag),this._emitEOFToken();break}default:this.currentAttr.value+=String.fromCodePoint(t)}}_stateAttributeValueSingleQuoted(t){switch(t){case re.APOSTROPHE:{this.state=ie.AFTER_ATTRIBUTE_VALUE_QUOTED;break}case re.AMPERSAND:{this._startCharacterReference();break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.currentAttr.value+=mn;break}case re.EOF:{this._err(Be.eofInTag),this._emitEOFToken();break}default:this.currentAttr.value+=String.fromCodePoint(t)}}_stateAttributeValueUnquoted(t){switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this._leaveAttrValue(),this.state=ie.BEFORE_ATTRIBUTE_NAME;break}case re.AMPERSAND:{this._startCharacterReference();break}case re.GREATER_THAN_SIGN:{this._leaveAttrValue(),this.state=ie.DATA,this.emitCurrentTagToken();break}case re.NULL:{this._err(Be.unexpectedNullCharacter),this.currentAttr.value+=mn;break}case re.QUOTATION_MARK:case re.APOSTROPHE:case re.LESS_THAN_SIGN:case re.EQUALS_SIGN:case re.GRAVE_ACCENT:{this._err(Be.unexpectedCharacterInUnquotedAttributeValue),this.currentAttr.value+=String.fromCodePoint(t);break}case re.EOF:{this._err(Be.eofInTag),this._emitEOFToken();break}default:this.currentAttr.value+=String.fromCodePoint(t)}}_stateAfterAttributeValueQuoted(t){switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this._leaveAttrValue(),this.state=ie.BEFORE_ATTRIBUTE_NAME;break}case re.SOLIDUS:{this._leaveAttrValue(),this.state=ie.SELF_CLOSING_START_TAG;break}case re.GREATER_THAN_SIGN:{this._leaveAttrValue(),this.state=ie.DATA,this.emitCurrentTagToken();break}case re.EOF:{this._err(Be.eofInTag),this._emitEOFToken();break}default:this._err(Be.missingWhitespaceBetweenAttributes),this.state=ie.BEFORE_ATTRIBUTE_NAME,this._stateBeforeAttributeName(t)}}_stateSelfClosingStartTag(t){switch(t){case re.GREATER_THAN_SIGN:{const r=this.currentToken;r.selfClosing=!0,this.state=ie.DATA,this.emitCurrentTagToken();break}case re.EOF:{this._err(Be.eofInTag),this._emitEOFToken();break}default:this._err(Be.unexpectedSolidusInTag),this.state=ie.BEFORE_ATTRIBUTE_NAME,this._stateBeforeAttributeName(t)}}_stateBogusComment(t){const r=this.currentToken;switch(t){case re.GREATER_THAN_SIGN:{this.state=ie.DATA,this.emitCurrentComment(r);break}case re.EOF:{this.emitCurrentComment(r),this._emitEOFToken();break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.data+=mn;break}default:r.data+=String.fromCodePoint(t)}}_stateMarkupDeclarationOpen(t){this._consumeSequenceIfMatch(ai.DASH_DASH,!0)?(this._createCommentToken(ai.DASH_DASH.length+1),this.state=ie.COMMENT_START):this._consumeSequenceIfMatch(ai.DOCTYPE,!1)?(this.currentLocation=this.getCurrentLocation(ai.DOCTYPE.length+1),this.state=ie.DOCTYPE):this._consumeSequenceIfMatch(ai.CDATA_START,!0)?this.inForeignNode?this.state=ie.CDATA_SECTION:(this._err(Be.cdataInHtmlContent),this._createCommentToken(ai.CDATA_START.length+1),this.currentToken.data="[CDATA[",this.state=ie.BOGUS_COMMENT):this._ensureHibernation()||(this._err(Be.incorrectlyOpenedComment),this._createCommentToken(2),this.state=ie.BOGUS_COMMENT,this._stateBogusComment(t))}_stateCommentStart(t){switch(t){case re.HYPHEN_MINUS:{this.state=ie.COMMENT_START_DASH;break}case re.GREATER_THAN_SIGN:{this._err(Be.abruptClosingOfEmptyComment),this.state=ie.DATA;const r=this.currentToken;this.emitCurrentComment(r);break}default:this.state=ie.COMMENT,this._stateComment(t)}}_stateCommentStartDash(t){const r=this.currentToken;switch(t){case re.HYPHEN_MINUS:{this.state=ie.COMMENT_END;break}case re.GREATER_THAN_SIGN:{this._err(Be.abruptClosingOfEmptyComment),this.state=ie.DATA,this.emitCurrentComment(r);break}case re.EOF:{this._err(Be.eofInComment),this.emitCurrentComment(r),this._emitEOFToken();break}default:r.data+="-",this.state=ie.COMMENT,this._stateComment(t)}}_stateComment(t){const r=this.currentToken;switch(t){case re.HYPHEN_MINUS:{this.state=ie.COMMENT_END_DASH;break}case re.LESS_THAN_SIGN:{r.data+="<",this.state=ie.COMMENT_LESS_THAN_SIGN;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.data+=mn;break}case re.EOF:{this._err(Be.eofInComment),this.emitCurrentComment(r),this._emitEOFToken();break}default:r.data+=String.fromCodePoint(t)}}_stateCommentLessThanSign(t){const r=this.currentToken;switch(t){case re.EXCLAMATION_MARK:{r.data+="!",this.state=ie.COMMENT_LESS_THAN_SIGN_BANG;break}case re.LESS_THAN_SIGN:{r.data+="<";break}default:this.state=ie.COMMENT,this._stateComment(t)}}_stateCommentLessThanSignBang(t){t===re.HYPHEN_MINUS?this.state=ie.COMMENT_LESS_THAN_SIGN_BANG_DASH:(this.state=ie.COMMENT,this._stateComment(t))}_stateCommentLessThanSignBangDash(t){t===re.HYPHEN_MINUS?this.state=ie.COMMENT_LESS_THAN_SIGN_BANG_DASH_DASH:(this.state=ie.COMMENT_END_DASH,this._stateCommentEndDash(t))}_stateCommentLessThanSignBangDashDash(t){t!==re.GREATER_THAN_SIGN&&t!==re.EOF&&this._err(Be.nestedComment),this.state=ie.COMMENT_END,this._stateCommentEnd(t)}_stateCommentEndDash(t){const r=this.currentToken;switch(t){case re.HYPHEN_MINUS:{this.state=ie.COMMENT_END;break}case re.EOF:{this._err(Be.eofInComment),this.emitCurrentComment(r),this._emitEOFToken();break}default:r.data+="-",this.state=ie.COMMENT,this._stateComment(t)}}_stateCommentEnd(t){const r=this.currentToken;switch(t){case re.GREATER_THAN_SIGN:{this.state=ie.DATA,this.emitCurrentComment(r);break}case re.EXCLAMATION_MARK:{this.state=ie.COMMENT_END_BANG;break}case re.HYPHEN_MINUS:{r.data+="-";break}case re.EOF:{this._err(Be.eofInComment),this.emitCurrentComment(r),this._emitEOFToken();break}default:r.data+="--",this.state=ie.COMMENT,this._stateComment(t)}}_stateCommentEndBang(t){const r=this.currentToken;switch(t){case re.HYPHEN_MINUS:{r.data+="--!",this.state=ie.COMMENT_END_DASH;break}case re.GREATER_THAN_SIGN:{this._err(Be.incorrectlyClosedComment),this.state=ie.DATA,this.emitCurrentComment(r);break}case re.EOF:{this._err(Be.eofInComment),this.emitCurrentComment(r),this._emitEOFToken();break}default:r.data+="--!",this.state=ie.COMMENT,this._stateComment(t)}}_stateDoctype(t){switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this.state=ie.BEFORE_DOCTYPE_NAME;break}case re.GREATER_THAN_SIGN:{this.state=ie.BEFORE_DOCTYPE_NAME,this._stateBeforeDoctypeName(t);break}case re.EOF:{this._err(Be.eofInDoctype),this._createDoctypeToken(null);const r=this.currentToken;r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.missingWhitespaceBeforeDoctypeName),this.state=ie.BEFORE_DOCTYPE_NAME,this._stateBeforeDoctypeName(t)}}_stateBeforeDoctypeName(t){if(om(t))this._createDoctypeToken(String.fromCharCode(Qb(t))),this.state=ie.DOCTYPE_NAME;else switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.NULL:{this._err(Be.unexpectedNullCharacter),this._createDoctypeToken(mn),this.state=ie.DOCTYPE_NAME;break}case re.GREATER_THAN_SIGN:{this._err(Be.missingDoctypeName),this._createDoctypeToken(null);const r=this.currentToken;r.forceQuirks=!0,this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.EOF:{this._err(Be.eofInDoctype),this._createDoctypeToken(null);const r=this.currentToken;r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._createDoctypeToken(String.fromCodePoint(t)),this.state=ie.DOCTYPE_NAME}}_stateDoctypeName(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this.state=ie.AFTER_DOCTYPE_NAME;break}case re.GREATER_THAN_SIGN:{this.state=ie.DATA,this.emitCurrentDoctype(r);break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.name+=mn;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:r.name+=String.fromCodePoint(om(t)?Qb(t):t)}}_stateAfterDoctypeName(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.GREATER_THAN_SIGN:{this.state=ie.DATA,this.emitCurrentDoctype(r);break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._consumeSequenceIfMatch(ai.PUBLIC,!1)?this.state=ie.AFTER_DOCTYPE_PUBLIC_KEYWORD:this._consumeSequenceIfMatch(ai.SYSTEM,!1)?this.state=ie.AFTER_DOCTYPE_SYSTEM_KEYWORD:this._ensureHibernation()||(this._err(Be.invalidCharacterSequenceAfterDoctypeName),r.forceQuirks=!0,this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t))}}_stateAfterDoctypePublicKeyword(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this.state=ie.BEFORE_DOCTYPE_PUBLIC_IDENTIFIER;break}case re.QUOTATION_MARK:{this._err(Be.missingWhitespaceAfterDoctypePublicKeyword),r.publicId="",this.state=ie.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED;break}case re.APOSTROPHE:{this._err(Be.missingWhitespaceAfterDoctypePublicKeyword),r.publicId="",this.state=ie.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED;break}case re.GREATER_THAN_SIGN:{this._err(Be.missingDoctypePublicIdentifier),r.forceQuirks=!0,this.state=ie.DATA,this.emitCurrentDoctype(r);break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.missingQuoteBeforeDoctypePublicIdentifier),r.forceQuirks=!0,this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBeforeDoctypePublicIdentifier(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.QUOTATION_MARK:{r.publicId="",this.state=ie.DOCTYPE_PUBLIC_IDENTIFIER_DOUBLE_QUOTED;break}case re.APOSTROPHE:{r.publicId="",this.state=ie.DOCTYPE_PUBLIC_IDENTIFIER_SINGLE_QUOTED;break}case re.GREATER_THAN_SIGN:{this._err(Be.missingDoctypePublicIdentifier),r.forceQuirks=!0,this.state=ie.DATA,this.emitCurrentDoctype(r);break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.missingQuoteBeforeDoctypePublicIdentifier),r.forceQuirks=!0,this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateDoctypePublicIdentifierDoubleQuoted(t){const r=this.currentToken;switch(t){case re.QUOTATION_MARK:{this.state=ie.AFTER_DOCTYPE_PUBLIC_IDENTIFIER;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.publicId+=mn;break}case re.GREATER_THAN_SIGN:{this._err(Be.abruptDoctypePublicIdentifier),r.forceQuirks=!0,this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:r.publicId+=String.fromCodePoint(t)}}_stateDoctypePublicIdentifierSingleQuoted(t){const r=this.currentToken;switch(t){case re.APOSTROPHE:{this.state=ie.AFTER_DOCTYPE_PUBLIC_IDENTIFIER;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.publicId+=mn;break}case re.GREATER_THAN_SIGN:{this._err(Be.abruptDoctypePublicIdentifier),r.forceQuirks=!0,this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:r.publicId+=String.fromCodePoint(t)}}_stateAfterDoctypePublicIdentifier(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this.state=ie.BETWEEN_DOCTYPE_PUBLIC_AND_SYSTEM_IDENTIFIERS;break}case re.GREATER_THAN_SIGN:{this.state=ie.DATA,this.emitCurrentDoctype(r);break}case re.QUOTATION_MARK:{this._err(Be.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers),r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case re.APOSTROPHE:{this._err(Be.missingWhitespaceBetweenDoctypePublicAndSystemIdentifiers),r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.missingQuoteBeforeDoctypeSystemIdentifier),r.forceQuirks=!0,this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBetweenDoctypePublicAndSystemIdentifiers(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.GREATER_THAN_SIGN:{this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.QUOTATION_MARK:{r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case re.APOSTROPHE:{r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.missingQuoteBeforeDoctypeSystemIdentifier),r.forceQuirks=!0,this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateAfterDoctypeSystemKeyword(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:{this.state=ie.BEFORE_DOCTYPE_SYSTEM_IDENTIFIER;break}case re.QUOTATION_MARK:{this._err(Be.missingWhitespaceAfterDoctypeSystemKeyword),r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case re.APOSTROPHE:{this._err(Be.missingWhitespaceAfterDoctypeSystemKeyword),r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case re.GREATER_THAN_SIGN:{this._err(Be.missingDoctypeSystemIdentifier),r.forceQuirks=!0,this.state=ie.DATA,this.emitCurrentDoctype(r);break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.missingQuoteBeforeDoctypeSystemIdentifier),r.forceQuirks=!0,this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBeforeDoctypeSystemIdentifier(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.QUOTATION_MARK:{r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_DOUBLE_QUOTED;break}case re.APOSTROPHE:{r.systemId="",this.state=ie.DOCTYPE_SYSTEM_IDENTIFIER_SINGLE_QUOTED;break}case re.GREATER_THAN_SIGN:{this._err(Be.missingDoctypeSystemIdentifier),r.forceQuirks=!0,this.state=ie.DATA,this.emitCurrentDoctype(r);break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.missingQuoteBeforeDoctypeSystemIdentifier),r.forceQuirks=!0,this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateDoctypeSystemIdentifierDoubleQuoted(t){const r=this.currentToken;switch(t){case re.QUOTATION_MARK:{this.state=ie.AFTER_DOCTYPE_SYSTEM_IDENTIFIER;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.systemId+=mn;break}case re.GREATER_THAN_SIGN:{this._err(Be.abruptDoctypeSystemIdentifier),r.forceQuirks=!0,this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:r.systemId+=String.fromCodePoint(t)}}_stateDoctypeSystemIdentifierSingleQuoted(t){const r=this.currentToken;switch(t){case re.APOSTROPHE:{this.state=ie.AFTER_DOCTYPE_SYSTEM_IDENTIFIER;break}case re.NULL:{this._err(Be.unexpectedNullCharacter),r.systemId+=mn;break}case re.GREATER_THAN_SIGN:{this._err(Be.abruptDoctypeSystemIdentifier),r.forceQuirks=!0,this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:r.systemId+=String.fromCodePoint(t)}}_stateAfterDoctypeSystemIdentifier(t){const r=this.currentToken;switch(t){case re.SPACE:case re.LINE_FEED:case re.TABULATION:case re.FORM_FEED:break;case re.GREATER_THAN_SIGN:{this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.EOF:{this._err(Be.eofInDoctype),r.forceQuirks=!0,this.emitCurrentDoctype(r),this._emitEOFToken();break}default:this._err(Be.unexpectedCharacterAfterDoctypeSystemIdentifier),this.state=ie.BOGUS_DOCTYPE,this._stateBogusDoctype(t)}}_stateBogusDoctype(t){const r=this.currentToken;switch(t){case re.GREATER_THAN_SIGN:{this.emitCurrentDoctype(r),this.state=ie.DATA;break}case re.NULL:{this._err(Be.unexpectedNullCharacter);break}case re.EOF:{this.emitCurrentDoctype(r),this._emitEOFToken();break}}}_stateCdataSection(t){switch(t){case re.RIGHT_SQUARE_BRACKET:{this.state=ie.CDATA_SECTION_BRACKET;break}case re.EOF:{this._err(Be.eofInCdata),this._emitEOFToken();break}default:this._emitCodePoint(t)}}_stateCdataSectionBracket(t){t===re.RIGHT_SQUARE_BRACKET?this.state=ie.CDATA_SECTION_END:(this._emitChars("]"),this.state=ie.CDATA_SECTION,this._stateCdataSection(t))}_stateCdataSectionEnd(t){switch(t){case re.GREATER_THAN_SIGN:{this.state=ie.DATA;break}case re.RIGHT_SQUARE_BRACKET:{this._emitChars("]");break}default:this._emitChars("]]"),this.state=ie.CDATA_SECTION,this._stateCdataSection(t)}}_stateCharacterReference(){let t=this.entityDecoder.write(this.preprocessor.html,this.preprocessor.pos);if(t<0)if(this.preprocessor.lastChunkWritten)t=this.entityDecoder.end();else{this.active=!1,this.preprocessor.pos=this.preprocessor.html.length-1,this.consumedAfterSnapshot=0,this.preprocessor.endOfChunkHit=!0;return}t===0?(this.preprocessor.pos=this.entityStartPos,this._flushCodePointConsumedAsCharacterReference(re.AMPERSAND),this.state=!this._isCharacterReferenceInAttribute()&&fz(this.preprocessor.peek(1))?ie.AMBIGUOUS_AMPERSAND:this.returnState):this.state=this.returnState}_stateAmbiguousAmpersand(t){fz(t)?this._flushCodePointConsumedAsCharacterReference(t):(t===re.SEMICOLON&&this._err(Be.unknownNamedCharacterReference),this.state=this.returnState,this._callState(t))}}const Nne=new Set([N.DD,N.DT,N.LI,N.OPTGROUP,N.OPTION,N.P,N.RB,N.RP,N.RT,N.RTC]),gz=new Set([...Nne,N.CAPTION,N.COLGROUP,N.TBODY,N.TD,N.TFOOT,N.TH,N.THEAD,N.TR]),Ny=new Set([N.APPLET,N.CAPTION,N.HTML,N.MARQUEE,N.OBJECT,N.TABLE,N.TD,N.TEMPLATE,N.TH]),B7e=new Set([...Ny,N.OL,N.UL]),P7e=new Set([...Ny,N.BUTTON]),bz=new Set([N.ANNOTATION_XML,N.MI,N.MN,N.MO,N.MS,N.MTEXT]),yz=new Set([N.DESC,N.FOREIGN_OBJECT,N.TITLE]),z7e=new Set([N.TR,N.TEMPLATE,N.HTML]),H7e=new Set([N.TBODY,N.TFOOT,N.THEAD,N.TEMPLATE,N.HTML]),U7e=new Set([N.TABLE,N.TEMPLATE,N.HTML]),$7e=new Set([N.TD,N.TH]);class G7e{get currentTmplContentOrNode(){return this._isInTemplate()?this.treeAdapter.getTemplateContent(this.current):this.current}constructor(t,r,n){this.treeAdapter=r,this.handler=n,this.items=[],this.tagIDs=[],this.stackTop=-1,this.tmplCount=0,this.currentTagId=N.UNKNOWN,this.current=t}_indexOf(t){return this.items.lastIndexOf(t,this.stackTop)}_isInTemplate(){return this.currentTagId===N.TEMPLATE&&this.treeAdapter.getNamespaceURI(this.current)===Xe.HTML}_updateCurrentElement(){this.current=this.items[this.stackTop],this.currentTagId=this.tagIDs[this.stackTop]}push(t,r){this.stackTop++,this.items[this.stackTop]=t,this.current=t,this.tagIDs[this.stackTop]=r,this.currentTagId=r,this._isInTemplate()&&this.tmplCount++,this.handler.onItemPush(t,r,!0)}pop(){const t=this.current;this.tmplCount>0&&this._isInTemplate()&&this.tmplCount--,this.stackTop--,this._updateCurrentElement(),this.handler.onItemPop(t,!0)}replace(t,r){const n=this._indexOf(t);this.items[n]=r,n===this.stackTop&&(this.current=r)}insertAfter(t,r,n){const a=this._indexOf(t)+1;this.items.splice(a,0,r),this.tagIDs.splice(a,0,n),this.stackTop++,a===this.stackTop&&this._updateCurrentElement(),this.current&&this.currentTagId!==void 0&&this.handler.onItemPush(this.current,this.currentTagId,a===this.stackTop)}popUntilTagNamePopped(t){let r=this.stackTop+1;do r=this.tagIDs.lastIndexOf(t,r-1);while(r>0&&this.treeAdapter.getNamespaceURI(this.items[r])!==Xe.HTML);this.shortenToLength(Math.max(r,0))}shortenToLength(t){for(;this.stackTop>=t;){const r=this.current;this.tmplCount>0&&this._isInTemplate()&&(this.tmplCount-=1),this.stackTop--,this._updateCurrentElement(),this.handler.onItemPop(r,this.stackTop<t)}}popUntilElementPopped(t){const r=this._indexOf(t);this.shortenToLength(Math.max(r,0))}popUntilPopped(t,r){const n=this._indexOfTagNames(t,r);this.shortenToLength(Math.max(n,0))}popUntilNumberedHeaderPopped(){this.popUntilPopped(JR,Xe.HTML)}popUntilTableCellPopped(){this.popUntilPopped($7e,Xe.HTML)}popAllUpToHtmlElement(){this.tmplCount=0,this.shortenToLength(1)}_indexOfTagNames(t,r){for(let n=this.stackTop;n>=0;n--)if(t.has(this.tagIDs[n])&&this.treeAdapter.getNamespaceURI(this.items[n])===r)return n;return-1}clearBackTo(t,r){const n=this._indexOfTagNames(t,r);this.shortenToLength(n+1)}clearBackToTableContext(){this.clearBackTo(U7e,Xe.HTML)}clearBackToTableBodyContext(){this.clearBackTo(H7e,Xe.HTML)}clearBackToTableRowContext(){this.clearBackTo(z7e,Xe.HTML)}remove(t){const r=this._indexOf(t);r>=0&&(r===this.stackTop?this.pop():(this.items.splice(r,1),this.tagIDs.splice(r,1),this.stackTop--,this._updateCurrentElement(),this.handler.onItemPop(t,!1)))}tryPeekProperlyNestedBodyElement(){return this.stackTop>=1&&this.tagIDs[1]===N.BODY?this.items[1]:null}contains(t){return this._indexOf(t)>-1}getCommonAncestor(t){const r=this._indexOf(t)-1;return r>=0?this.items[r]:null}isRootHtmlElementCurrent(){return this.stackTop===0&&this.tagIDs[0]===N.HTML}hasInDynamicScope(t,r){for(let n=this.stackTop;n>=0;n--){const a=this.tagIDs[n];switch(this.treeAdapter.getNamespaceURI(this.items[n])){case Xe.HTML:{if(a===t)return!0;if(r.has(a))return!1;break}case Xe.SVG:{if(yz.has(a))return!1;break}case Xe.MATHML:{if(bz.has(a))return!1;break}}}return!0}hasInScope(t){return this.hasInDynamicScope(t,Ny)}hasInListItemScope(t){return this.hasInDynamicScope(t,B7e)}hasInButtonScope(t){return this.hasInDynamicScope(t,P7e)}hasNumberedHeaderInScope(){for(let t=this.stackTop;t>=0;t--){const r=this.tagIDs[t];switch(this.treeAdapter.getNamespaceURI(this.items[t])){case Xe.HTML:{if(JR.has(r))return!0;if(Ny.has(r))return!1;break}case Xe.SVG:{if(yz.has(r))return!1;break}case Xe.MATHML:{if(bz.has(r))return!1;break}}}return!0}hasInTableScope(t){for(let r=this.stackTop;r>=0;r--)if(this.treeAdapter.getNamespaceURI(this.items[r])===Xe.HTML)switch(this.tagIDs[r]){case t:return!0;case N.TABLE:case N.HTML:return!1}return!0}hasTableBodyContextInTableScope(){for(let t=this.stackTop;t>=0;t--)if(this.treeAdapter.getNamespaceURI(this.items[t])===Xe.HTML)switch(this.tagIDs[t]){case N.TBODY:case N.THEAD:case N.TFOOT:return!0;case N.TABLE:case N.HTML:return!1}return!0}hasInSelectScope(t){for(let r=this.stackTop;r>=0;r--)if(this.treeAdapter.getNamespaceURI(this.items[r])===Xe.HTML)switch(this.tagIDs[r]){case t:return!0;case N.OPTION:case N.OPTGROUP:break;default:return!1}return!0}generateImpliedEndTags(){for(;this.currentTagId!==void 0&&Nne.has(this.currentTagId);)this.pop()}generateImpliedEndTagsThoroughly(){for(;this.currentTagId!==void 0&&gz.has(this.currentTagId);)this.pop()}generateImpliedEndTagsWithExclusion(t){for(;this.currentTagId!==void 0&&this.currentTagId!==t&&gz.has(this.currentTagId);)this.pop()}}const wk=3;var ts;(function(e){e[e.Marker=0]="Marker",e[e.Element=1]="Element"})(ts||(ts={}));const vz={type:ts.Marker};class j7e{constructor(t){this.treeAdapter=t,this.entries=[],this.bookmark=null}_getNoahArkConditionCandidates(t,r){const n=[],a=r.length,i=this.treeAdapter.getTagName(t),o=this.treeAdapter.getNamespaceURI(t);for(let s=0;s<this.entries.length;s++){const l=this.entries[s];if(l.type===ts.Marker)break;const{element:u}=l;if(this.treeAdapter.getTagName(u)===i&&this.treeAdapter.getNamespaceURI(u)===o){const d=this.treeAdapter.getAttrList(u);d.length===a&&n.push({idx:s,attrs:d})}}return n}_ensureNoahArkCondition(t){if(this.entries.length<wk)return;const r=this.treeAdapter.getAttrList(t),n=this._getNoahArkConditionCandidates(t,r);if(n.length<wk)return;const a=new Map(r.map(o=>[o.name,o.value]));let i=0;for(let o=0;o<n.length;o++){const s=n[o];s.attrs.every(l=>a.get(l.name)===l.value)&&(i+=1,i>=wk&&this.entries.splice(s.idx,1))}}insertMarker(){this.entries.unshift(vz)}pushElement(t,r){this._ensureNoahArkCondition(t),this.entries.unshift({type:ts.Element,element:t,token:r})}insertElementAfterBookmark(t,r){const n=this.entries.indexOf(this.bookmark);this.entries.splice(n,0,{type:ts.Element,element:t,token:r})}removeEntry(t){const r=this.entries.indexOf(t);r!==-1&&this.entries.splice(r,1)}clearToLastMarker(){const t=this.entries.indexOf(vz);t===-1?this.entries.length=0:this.entries.splice(0,t+1)}getElementEntryInScopeWithTagName(t){const r=this.entries.find(n=>n.type===ts.Marker||this.treeAdapter.getTagName(n.element)===t);return r&&r.type===ts.Element?r:null}getElementEntry(t){return this.entries.find(r=>r.type===ts.Element&&r.element===t)}}const pu={createDocument(){return{nodeName:"#document",mode:ro.NO_QUIRKS,childNodes:[]}},createDocumentFragment(){return{nodeName:"#document-fragment",childNodes:[]}},createElement(e,t,r){return{nodeName:e,tagName:e,attrs:r,namespaceURI:t,childNodes:[],parentNode:null}},createCommentNode(e){return{nodeName:"#comment",data:e,parentNode:null}},createTextNode(e){return{nodeName:"#text",value:e,parentNode:null}},appendChild(e,t){e.childNodes.push(t),t.parentNode=e},insertBefore(e,t,r){const n=e.childNodes.indexOf(r);e.childNodes.splice(n,0,t),t.parentNode=e},setTemplateContent(e,t){e.content=t},getTemplateContent(e){return e.content},setDocumentType(e,t,r,n){const a=e.childNodes.find(i=>i.nodeName==="#documentType");if(a)a.name=t,a.publicId=r,a.systemId=n;else{const i={nodeName:"#documentType",name:t,publicId:r,systemId:n,parentNode:null};pu.appendChild(e,i)}},setDocumentMode(e,t){e.mode=t},getDocumentMode(e){return e.mode},detachNode(e){if(e.parentNode){const t=e.parentNode.childNodes.indexOf(e);e.parentNode.childNodes.splice(t,1),e.parentNode=null}},insertText(e,t){if(e.childNodes.length>0){const r=e.childNodes[e.childNodes.length-1];if(pu.isTextNode(r)){r.value+=t;return}}pu.appendChild(e,pu.createTextNode(t))},insertTextBefore(e,t,r){const n=e.childNodes[e.childNodes.indexOf(r)-1];n&&pu.isTextNode(n)?n.value+=t:pu.insertBefore(e,pu.createTextNode(t),r)},adoptAttributes(e,t){const r=new Set(e.attrs.map(n=>n.name));for(let n=0;n<t.length;n++)r.has(t[n].name)||e.attrs.push(t[n])},getFirstChild(e){return e.childNodes[0]},getChildNodes(e){return e.childNodes},getParentNode(e){return e.parentNode},getAttrList(e){return e.attrs},getTagName(e){return e.tagName},getNamespaceURI(e){return e.namespaceURI},getTextNodeContent(e){return e.value},getCommentNodeContent(e){return e.data},getDocumentTypeNodeName(e){return e.name},getDocumentTypeNodePublicId(e){return e.publicId},getDocumentTypeNodeSystemId(e){return e.systemId},isTextNode(e){return e.nodeName==="#text"},isCommentNode(e){return e.nodeName==="#comment"},isDocumentTypeNode(e){return e.nodeName==="#documentType"},isElementNode(e){return Object.prototype.hasOwnProperty.call(e,"tagName")},setNodeSourceCodeLocation(e,t){e.sourceCodeLocation=t},getNodeSourceCodeLocation(e){return e.sourceCodeLocation},updateNodeSourceCodeLocation(e,t){e.sourceCodeLocation={...e.sourceCodeLocation,...t}}},Lne="html",q7e="about:legacy-compat",W7e="http://www.ibm.com/data/dtd/v11/ibmxhtml1-transitional.dtd",Mne=["+//silmaril//dtd html pro v0r11 19970101//","-//as//dtd html 3.0 aswedit + extensions//","-//advasoft ltd//dtd html 3.0 aswedit + extensions//","-//ietf//dtd html 2.0 level 1//","-//ietf//dtd html 2.0 level 2//","-//ietf//dtd html 2.0 strict level 1//","-//ietf//dtd html 2.0 strict level 2//","-//ietf//dtd html 2.0 strict//","-//ietf//dtd html 2.0//","-//ietf//dtd html 2.1e//","-//ietf//dtd html 3.0//","-//ietf//dtd html 3.2 final//","-//ietf//dtd html 3.2//","-//ietf//dtd html 3//","-//ietf//dtd html level 0//","-//ietf//dtd html level 1//","-//ietf//dtd html level 2//","-//ietf//dtd html level 3//","-//ietf//dtd html strict level 0//","-//ietf//dtd html strict level 1//","-//ietf//dtd html strict level 2//","-//ietf//dtd html strict level 3//","-//ietf//dtd html strict//","-//ietf//dtd html//","-//metrius//dtd metrius presentational//","-//microsoft//dtd internet explorer 2.0 html strict//","-//microsoft//dtd internet explorer 2.0 html//","-//microsoft//dtd internet explorer 2.0 tables//","-//microsoft//dtd internet explorer 3.0 html strict//","-//microsoft//dtd internet explorer 3.0 html//","-//microsoft//dtd internet explorer 3.0 tables//","-//netscape comm. corp.//dtd html//","-//netscape comm. corp.//dtd strict html//","-//o'reilly and associates//dtd html 2.0//","-//o'reilly and associates//dtd html extended 1.0//","-//o'reilly and associates//dtd html extended relaxed 1.0//","-//sq//dtd html 2.0 hotmetal + extensions//","-//softquad software//dtd hotmetal pro 6.0::19990601::extensions to html 4.0//","-//softquad//dtd hotmetal pro 4.0::19971010::extensions to html 4.0//","-//spyglass//dtd html 2.0 extended//","-//sun microsystems corp.//dtd hotjava html//","-//sun microsystems corp.//dtd hotjava strict html//","-//w3c//dtd html 3 1995-03-24//","-//w3c//dtd html 3.2 draft//","-//w3c//dtd html 3.2 final//","-//w3c//dtd html 3.2//","-//w3c//dtd html 3.2s draft//","-//w3c//dtd html 4.0 frameset//","-//w3c//dtd html 4.0 transitional//","-//w3c//dtd html experimental 19960712//","-//w3c//dtd html experimental 970421//","-//w3c//dtd w3 html//","-//w3o//dtd w3 html 3.0//","-//webtechs//dtd mozilla html 2.0//","-//webtechs//dtd mozilla html//"],V7e=[...Mne,"-//w3c//dtd html 4.01 frameset//","-//w3c//dtd html 4.01 transitional//"],K7e=new Set(["-//w3o//dtd w3 html strict 3.0//en//","-/w3c/dtd html 4.0 transitional/en","html"]),Ine=["-//w3c//dtd xhtml 1.0 frameset//","-//w3c//dtd xhtml 1.0 transitional//"],Y7e=[...Ine,"-//w3c//dtd html 4.01 frameset//","-//w3c//dtd html 4.01 transitional//"];function Sz(e,t){return t.some(r=>e.startsWith(r))}function J7e(e){return e.name===Lne&&e.publicId===null&&(e.systemId===null||e.systemId===q7e)}function X7e(e){if(e.name!==Lne)return ro.QUIRKS;const{systemId:t}=e;if(t&&t.toLowerCase()===W7e)return ro.QUIRKS;let{publicId:r}=e;if(r!==null){if(r=r.toLowerCase(),K7e.has(r))return ro.QUIRKS;let n=t===null?V7e:Mne;if(Sz(r,n))return ro.QUIRKS;if(n=t===null?Ine:Y7e,Sz(r,n))return ro.LIMITED_QUIRKS}return ro.NO_QUIRKS}const Ez={TEXT_HTML:"text/html",APPLICATION_XML:"application/xhtml+xml"},Z7e="definitionurl",Q7e="definitionURL",eBe=new Map(["attributeName","attributeType","baseFrequency","baseProfile","calcMode","clipPathUnits","diffuseConstant","edgeMode","filterUnits","glyphRef","gradientTransform","gradientUnits","kernelMatrix","kernelUnitLength","keyPoints","keySplines","keyTimes","lengthAdjust","limitingConeAngle","markerHeight","markerUnits","markerWidth","maskContentUnits","maskUnits","numOctaves","pathLength","patternContentUnits","patternTransform","patternUnits","pointsAtX","pointsAtY","pointsAtZ","preserveAlpha","preserveAspectRatio","primitiveUnits","refX","refY","repeatCount","repeatDur","requiredExtensions","requiredFeatures","specularConstant","specularExponent","spreadMethod","startOffset","stdDeviation","stitchTiles","surfaceScale","systemLanguage","tableValues","targetX","targetY","textLength","viewBox","viewTarget","xChannelSelector","yChannelSelector","zoomAndPan"].map(e=>[e.toLowerCase(),e])),tBe=new Map([["xlink:actuate",{prefix:"xlink",name:"actuate",namespace:Xe.XLINK}],["xlink:arcrole",{prefix:"xlink",name:"arcrole",namespace:Xe.XLINK}],["xlink:href",{prefix:"xlink",name:"href",namespace:Xe.XLINK}],["xlink:role",{prefix:"xlink",name:"role",namespace:Xe.XLINK}],["xlink:show",{prefix:"xlink",name:"show",namespace:Xe.XLINK}],["xlink:title",{prefix:"xlink",name:"title",namespace:Xe.XLINK}],["xlink:type",{prefix:"xlink",name:"type",namespace:Xe.XLINK}],["xml:lang",{prefix:"xml",name:"lang",namespace:Xe.XML}],["xml:space",{prefix:"xml",name:"space",namespace:Xe.XML}],["xmlns",{prefix:"",name:"xmlns",namespace:Xe.XMLNS}],["xmlns:xlink",{prefix:"xmlns",name:"xlink",namespace:Xe.XMLNS}]]),rBe=new Map(["altGlyph","altGlyphDef","altGlyphItem","animateColor","animateMotion","animateTransform","clipPath","feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence","foreignObject","glyphRef","linearGradient","radialGradient","textPath"].map(e=>[e.toLowerCase(),e])),nBe=new Set([N.B,N.BIG,N.BLOCKQUOTE,N.BODY,N.BR,N.CENTER,N.CODE,N.DD,N.DIV,N.DL,N.DT,N.EM,N.EMBED,N.H1,N.H2,N.H3,N.H4,N.H5,N.H6,N.HEAD,N.HR,N.I,N.IMG,N.LI,N.LISTING,N.MENU,N.META,N.NOBR,N.OL,N.P,N.PRE,N.RUBY,N.S,N.SMALL,N.SPAN,N.STRONG,N.STRIKE,N.SUB,N.SUP,N.TABLE,N.TT,N.U,N.UL,N.VAR]);function aBe(e){const t=e.tagID;return t===N.FONT&&e.attrs.some(({name:n})=>n===Dc.COLOR||n===Dc.SIZE||n===Dc.FACE)||nBe.has(t)}function One(e){for(let t=0;t<e.attrs.length;t++)if(e.attrs[t].name===Z7e){e.attrs[t].name=Q7e;break}}function Bne(e){for(let t=0;t<e.attrs.length;t++){const r=eBe.get(e.attrs[t].name);r!=null&&(e.attrs[t].name=r)}}function aN(e){for(let t=0;t<e.attrs.length;t++){const r=tBe.get(e.attrs[t].name);r&&(e.attrs[t].prefix=r.prefix,e.attrs[t].name=r.name,e.attrs[t].namespace=r.namespace)}}function iBe(e){const t=rBe.get(e.tagName);t!=null&&(e.tagName=t,e.tagID=Ap(e.tagName))}function oBe(e,t){return t===Xe.MATHML&&(e===N.MI||e===N.MO||e===N.MN||e===N.MS||e===N.MTEXT)}function sBe(e,t,r){if(t===Xe.MATHML&&e===N.ANNOTATION_XML){for(let n=0;n<r.length;n++)if(r[n].name===Dc.ENCODING){const a=r[n].value.toLowerCase();return a===Ez.TEXT_HTML||a===Ez.APPLICATION_XML}}return t===Xe.SVG&&(e===N.FOREIGN_OBJECT||e===N.DESC||e===N.TITLE)}function lBe(e,t,r,n){return(!n||n===Xe.HTML)&&sBe(e,t,r)||(!n||n===Xe.MATHML)&&oBe(e,t)}const uBe="hidden",cBe=8,dBe=3;var ce;(function(e){e[e.INITIAL=0]="INITIAL",e[e.BEFORE_HTML=1]="BEFORE_HTML",e[e.BEFORE_HEAD=2]="BEFORE_HEAD",e[e.IN_HEAD=3]="IN_HEAD",e[e.IN_HEAD_NO_SCRIPT=4]="IN_HEAD_NO_SCRIPT",e[e.AFTER_HEAD=5]="AFTER_HEAD",e[e.IN_BODY=6]="IN_BODY",e[e.TEXT=7]="TEXT",e[e.IN_TABLE=8]="IN_TABLE",e[e.IN_TABLE_TEXT=9]="IN_TABLE_TEXT",e[e.IN_CAPTION=10]="IN_CAPTION",e[e.IN_COLUMN_GROUP=11]="IN_COLUMN_GROUP",e[e.IN_TABLE_BODY=12]="IN_TABLE_BODY",e[e.IN_ROW=13]="IN_ROW",e[e.IN_CELL=14]="IN_CELL",e[e.IN_SELECT=15]="IN_SELECT",e[e.IN_SELECT_IN_TABLE=16]="IN_SELECT_IN_TABLE",e[e.IN_TEMPLATE=17]="IN_TEMPLATE",e[e.AFTER_BODY=18]="AFTER_BODY",e[e.IN_FRAMESET=19]="IN_FRAMESET",e[e.AFTER_FRAMESET=20]="AFTER_FRAMESET",e[e.AFTER_AFTER_BODY=21]="AFTER_AFTER_BODY",e[e.AFTER_AFTER_FRAMESET=22]="AFTER_AFTER_FRAMESET"})(ce||(ce={}));const hBe={startLine:-1,startCol:-1,startOffset:-1,endLine:-1,endCol:-1,endOffset:-1},Pne=new Set([N.TABLE,N.TBODY,N.TFOOT,N.THEAD,N.TR]),xz={scriptingEnabled:!0,sourceCodeLocationInfo:!1,treeAdapter:pu,onParseError:null};class wz{constructor(t,r,n=null,a=null){this.fragmentContext=n,this.scriptHandler=a,this.currentToken=null,this.stopped=!1,this.insertionMode=ce.INITIAL,this.originalInsertionMode=ce.INITIAL,this.headElement=null,this.formElement=null,this.currentNotInHTML=!1,this.tmplInsertionModeStack=[],this.pendingCharacterTokens=[],this.hasNonWhitespacePendingCharacterToken=!1,this.framesetOk=!0,this.skipNextNewLine=!1,this.fosterParentingEnabled=!1,this.options={...xz,...t},this.treeAdapter=this.options.treeAdapter,this.onParseError=this.options.onParseError,this.onParseError&&(this.options.sourceCodeLocationInfo=!0),this.document=r??this.treeAdapter.createDocument(),this.tokenizer=new O7e(this.options,this),this.activeFormattingElements=new j7e(this.treeAdapter),this.fragmentContextID=n?Ap(this.treeAdapter.getTagName(n)):N.UNKNOWN,this._setContextModes(n??this.document,this.fragmentContextID),this.openElements=new G7e(this.document,this.treeAdapter,this)}static parse(t,r){const n=new this(r);return n.tokenizer.write(t,!0),n.document}static getFragmentParser(t,r){const n={...xz,...r};t??(t=n.treeAdapter.createElement(ke.TEMPLATE,Xe.HTML,[]));const a=n.treeAdapter.createElement("documentmock",Xe.HTML,[]),i=new this(n,a,t);return i.fragmentContextID===N.TEMPLATE&&i.tmplInsertionModeStack.unshift(ce.IN_TEMPLATE),i._initTokenizerForFragmentParsing(),i._insertFakeRootElement(),i._resetInsertionMode(),i._findFormInFragmentContext(),i}getFragment(){const t=this.treeAdapter.getFirstChild(this.document),r=this.treeAdapter.createDocumentFragment();return this._adoptNodes(t,r),r}_err(t,r,n){var a;if(!this.onParseError)return;const i=(a=t.location)!==null&&a!==void 0?a:hBe,o={code:r,startLine:i.startLine,startCol:i.startCol,startOffset:i.startOffset,endLine:n?i.startLine:i.endLine,endCol:n?i.startCol:i.endCol,endOffset:n?i.startOffset:i.endOffset};this.onParseError(o)}onItemPush(t,r,n){var a,i;(i=(a=this.treeAdapter).onItemPush)===null||i===void 0||i.call(a,t),n&&this.openElements.stackTop>0&&this._setContextModes(t,r)}onItemPop(t,r){var n,a;if(this.options.sourceCodeLocationInfo&&this._setEndLocation(t,this.currentToken),(a=(n=this.treeAdapter).onItemPop)===null||a===void 0||a.call(n,t,this.openElements.current),r){let i,o;this.openElements.stackTop===0&&this.fragmentContext?(i=this.fragmentContext,o=this.fragmentContextID):{current:i,currentTagId:o}=this.openElements,this._setContextModes(i,o)}}_setContextModes(t,r){const n=t===this.document||t&&this.treeAdapter.getNamespaceURI(t)===Xe.HTML;this.currentNotInHTML=!n,this.tokenizer.inForeignNode=!n&&t!==void 0&&r!==void 0&&!this._isIntegrationPoint(r,t)}_switchToTextParsing(t,r){this._insertElement(t,Xe.HTML),this.tokenizer.state=r,this.originalInsertionMode=this.insertionMode,this.insertionMode=ce.TEXT}switchToPlaintextParsing(){this.insertionMode=ce.TEXT,this.originalInsertionMode=ce.IN_BODY,this.tokenizer.state=Dn.PLAINTEXT}_getAdjustedCurrentElement(){return this.openElements.stackTop===0&&this.fragmentContext?this.fragmentContext:this.openElements.current}_findFormInFragmentContext(){let t=this.fragmentContext;for(;t;){if(this.treeAdapter.getTagName(t)===ke.FORM){this.formElement=t;break}t=this.treeAdapter.getParentNode(t)}}_initTokenizerForFragmentParsing(){if(!(!this.fragmentContext||this.treeAdapter.getNamespaceURI(this.fragmentContext)!==Xe.HTML))switch(this.fragmentContextID){case N.TITLE:case N.TEXTAREA:{this.tokenizer.state=Dn.RCDATA;break}case N.STYLE:case N.XMP:case N.IFRAME:case N.NOEMBED:case N.NOFRAMES:case N.NOSCRIPT:{this.tokenizer.state=Dn.RAWTEXT;break}case N.SCRIPT:{this.tokenizer.state=Dn.SCRIPT_DATA;break}case N.PLAINTEXT:{this.tokenizer.state=Dn.PLAINTEXT;break}}}_setDocumentType(t){const r=t.name||"",n=t.publicId||"",a=t.systemId||"";if(this.treeAdapter.setDocumentType(this.document,r,n,a),t.location){const o=this.treeAdapter.getChildNodes(this.document).find(s=>this.treeAdapter.isDocumentTypeNode(s));o&&this.treeAdapter.setNodeSourceCodeLocation(o,t.location)}}_attachElementToTree(t,r){if(this.options.sourceCodeLocationInfo){const n=r&&{...r,startTag:r};this.treeAdapter.setNodeSourceCodeLocation(t,n)}if(this._shouldFosterParentOnInsertion())this._fosterParentElement(t);else{const n=this.openElements.currentTmplContentOrNode;this.treeAdapter.appendChild(n??this.document,t)}}_appendElement(t,r){const n=this.treeAdapter.createElement(t.tagName,r,t.attrs);this._attachElementToTree(n,t.location)}_insertElement(t,r){const n=this.treeAdapter.createElement(t.tagName,r,t.attrs);this._attachElementToTree(n,t.location),this.openElements.push(n,t.tagID)}_insertFakeElement(t,r){const n=this.treeAdapter.createElement(t,Xe.HTML,[]);this._attachElementToTree(n,null),this.openElements.push(n,r)}_insertTemplate(t){const r=this.treeAdapter.createElement(t.tagName,Xe.HTML,t.attrs),n=this.treeAdapter.createDocumentFragment();this.treeAdapter.setTemplateContent(r,n),this._attachElementToTree(r,t.location),this.openElements.push(r,t.tagID),this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(n,null)}_insertFakeRootElement(){const t=this.treeAdapter.createElement(ke.HTML,Xe.HTML,[]);this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(t,null),this.treeAdapter.appendChild(this.openElements.current,t),this.openElements.push(t,N.HTML)}_appendCommentNode(t,r){const n=this.treeAdapter.createCommentNode(t.data);this.treeAdapter.appendChild(r,n),this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(n,t.location)}_insertCharacters(t){let r,n;if(this._shouldFosterParentOnInsertion()?({parent:r,beforeElement:n}=this._findFosterParentingLocation(),n?this.treeAdapter.insertTextBefore(r,t.chars,n):this.treeAdapter.insertText(r,t.chars)):(r=this.openElements.currentTmplContentOrNode,this.treeAdapter.insertText(r,t.chars)),!t.location)return;const a=this.treeAdapter.getChildNodes(r),i=n?a.lastIndexOf(n):a.length,o=a[i-1];if(this.treeAdapter.getNodeSourceCodeLocation(o)){const{endLine:l,endCol:u,endOffset:d}=t.location;this.treeAdapter.updateNodeSourceCodeLocation(o,{endLine:l,endCol:u,endOffset:d})}else this.options.sourceCodeLocationInfo&&this.treeAdapter.setNodeSourceCodeLocation(o,t.location)}_adoptNodes(t,r){for(let n=this.treeAdapter.getFirstChild(t);n;n=this.treeAdapter.getFirstChild(t))this.treeAdapter.detachNode(n),this.treeAdapter.appendChild(r,n)}_setEndLocation(t,r){if(this.treeAdapter.getNodeSourceCodeLocation(t)&&r.location){const n=r.location,a=this.treeAdapter.getTagName(t),i=r.type===Er.END_TAG&&a===r.tagName?{endTag:{...n},endLine:n.endLine,endCol:n.endCol,endOffset:n.endOffset}:{endLine:n.startLine,endCol:n.startCol,endOffset:n.startOffset};this.treeAdapter.updateNodeSourceCodeLocation(t,i)}}shouldProcessStartTagTokenInForeignContent(t){if(!this.currentNotInHTML)return!1;let r,n;return this.openElements.stackTop===0&&this.fragmentContext?(r=this.fragmentContext,n=this.fragmentContextID):{current:r,currentTagId:n}=this.openElements,t.tagID===N.SVG&&this.treeAdapter.getTagName(r)===ke.ANNOTATION_XML&&this.treeAdapter.getNamespaceURI(r)===Xe.MATHML?!1:this.tokenizer.inForeignNode||(t.tagID===N.MGLYPH||t.tagID===N.MALIGNMARK)&&n!==void 0&&!this._isIntegrationPoint(n,r,Xe.HTML)}_processToken(t){switch(t.type){case Er.CHARACTER:{this.onCharacter(t);break}case Er.NULL_CHARACTER:{this.onNullCharacter(t);break}case Er.COMMENT:{this.onComment(t);break}case Er.DOCTYPE:{this.onDoctype(t);break}case Er.START_TAG:{this._processStartTag(t);break}case Er.END_TAG:{this.onEndTag(t);break}case Er.EOF:{this.onEof(t);break}case Er.WHITESPACE_CHARACTER:{this.onWhitespaceCharacter(t);break}}}_isIntegrationPoint(t,r,n){const a=this.treeAdapter.getNamespaceURI(r),i=this.treeAdapter.getAttrList(r);return lBe(t,a,i,n)}_reconstructActiveFormattingElements(){const t=this.activeFormattingElements.entries.length;if(t){const r=this.activeFormattingElements.entries.findIndex(a=>a.type===ts.Marker||this.openElements.contains(a.element)),n=r===-1?t-1:r-1;for(let a=n;a>=0;a--){const i=this.activeFormattingElements.entries[a];this._insertElement(i.token,this.treeAdapter.getNamespaceURI(i.element)),i.element=this.openElements.current}}}_closeTableCell(){this.openElements.generateImpliedEndTags(),this.openElements.popUntilTableCellPopped(),this.activeFormattingElements.clearToLastMarker(),this.insertionMode=ce.IN_ROW}_closePElement(){this.openElements.generateImpliedEndTagsWithExclusion(N.P),this.openElements.popUntilTagNamePopped(N.P)}_resetInsertionMode(){for(let t=this.openElements.stackTop;t>=0;t--)switch(t===0&&this.fragmentContext?this.fragmentContextID:this.openElements.tagIDs[t]){case N.TR:{this.insertionMode=ce.IN_ROW;return}case N.TBODY:case N.THEAD:case N.TFOOT:{this.insertionMode=ce.IN_TABLE_BODY;return}case N.CAPTION:{this.insertionMode=ce.IN_CAPTION;return}case N.COLGROUP:{this.insertionMode=ce.IN_COLUMN_GROUP;return}case N.TABLE:{this.insertionMode=ce.IN_TABLE;return}case N.BODY:{this.insertionMode=ce.IN_BODY;return}case N.FRAMESET:{this.insertionMode=ce.IN_FRAMESET;return}case N.SELECT:{this._resetInsertionModeForSelect(t);return}case N.TEMPLATE:{this.insertionMode=this.tmplInsertionModeStack[0];return}case N.HTML:{this.insertionMode=this.headElement?ce.AFTER_HEAD:ce.BEFORE_HEAD;return}case N.TD:case N.TH:{if(t>0){this.insertionMode=ce.IN_CELL;return}break}case N.HEAD:{if(t>0){this.insertionMode=ce.IN_HEAD;return}break}}this.insertionMode=ce.IN_BODY}_resetInsertionModeForSelect(t){if(t>0)for(let r=t-1;r>0;r--){const n=this.openElements.tagIDs[r];if(n===N.TEMPLATE)break;if(n===N.TABLE){this.insertionMode=ce.IN_SELECT_IN_TABLE;return}}this.insertionMode=ce.IN_SELECT}_isElementCausesFosterParenting(t){return Pne.has(t)}_shouldFosterParentOnInsertion(){return this.fosterParentingEnabled&&this.openElements.currentTagId!==void 0&&this._isElementCausesFosterParenting(this.openElements.currentTagId)}_findFosterParentingLocation(){for(let t=this.openElements.stackTop;t>=0;t--){const r=this.openElements.items[t];switch(this.openElements.tagIDs[t]){case N.TEMPLATE:{if(this.treeAdapter.getNamespaceURI(r)===Xe.HTML)return{parent:this.treeAdapter.getTemplateContent(r),beforeElement:null};break}case N.TABLE:{const n=this.treeAdapter.getParentNode(r);return n?{parent:n,beforeElement:r}:{parent:this.openElements.items[t-1],beforeElement:null}}}}return{parent:this.openElements.items[0],beforeElement:null}}_fosterParentElement(t){const r=this._findFosterParentingLocation();r.beforeElement?this.treeAdapter.insertBefore(r.parent,t,r.beforeElement):this.treeAdapter.appendChild(r.parent,t)}_isSpecialElement(t,r){const n=this.treeAdapter.getNamespaceURI(t);return N7e[n].has(r)}onCharacter(t){if(this.skipNextNewLine=!1,this.tokenizer.inForeignNode){$Pe(this,t);return}switch(this.insertionMode){case ce.INITIAL:{Uf(this,t);break}case ce.BEFORE_HTML:{xm(this,t);break}case ce.BEFORE_HEAD:{wm(this,t);break}case ce.IN_HEAD:{km(this,t);break}case ce.IN_HEAD_NO_SCRIPT:{Cm(this,t);break}case ce.AFTER_HEAD:{Tm(this,t);break}case ce.IN_BODY:case ce.IN_CAPTION:case ce.IN_CELL:case ce.IN_TEMPLATE:{Hne(this,t);break}case ce.TEXT:case ce.IN_SELECT:case ce.IN_SELECT_IN_TABLE:{this._insertCharacters(t);break}case ce.IN_TABLE:case ce.IN_TABLE_BODY:case ce.IN_ROW:{kk(this,t);break}case ce.IN_TABLE_TEXT:{Wne(this,t);break}case ce.IN_COLUMN_GROUP:{Ly(this,t);break}case ce.AFTER_BODY:{My(this,t);break}case ce.AFTER_AFTER_BODY:{B1(this,t);break}}}onNullCharacter(t){if(this.skipNextNewLine=!1,this.tokenizer.inForeignNode){UPe(this,t);return}switch(this.insertionMode){case ce.INITIAL:{Uf(this,t);break}case ce.BEFORE_HTML:{xm(this,t);break}case ce.BEFORE_HEAD:{wm(this,t);break}case ce.IN_HEAD:{km(this,t);break}case ce.IN_HEAD_NO_SCRIPT:{Cm(this,t);break}case ce.AFTER_HEAD:{Tm(this,t);break}case ce.TEXT:{this._insertCharacters(t);break}case ce.IN_TABLE:case ce.IN_TABLE_BODY:case ce.IN_ROW:{kk(this,t);break}case ce.IN_COLUMN_GROUP:{Ly(this,t);break}case ce.AFTER_BODY:{My(this,t);break}case ce.AFTER_AFTER_BODY:{B1(this,t);break}}}onComment(t){if(this.skipNextNewLine=!1,this.currentNotInHTML){XR(this,t);return}switch(this.insertionMode){case ce.INITIAL:case ce.BEFORE_HTML:case ce.BEFORE_HEAD:case ce.IN_HEAD:case ce.IN_HEAD_NO_SCRIPT:case ce.AFTER_HEAD:case ce.IN_BODY:case ce.IN_TABLE:case ce.IN_CAPTION:case ce.IN_COLUMN_GROUP:case ce.IN_TABLE_BODY:case ce.IN_ROW:case ce.IN_CELL:case ce.IN_SELECT:case ce.IN_SELECT_IN_TABLE:case ce.IN_TEMPLATE:case ce.IN_FRAMESET:case ce.AFTER_FRAMESET:{XR(this,t);break}case ce.IN_TABLE_TEXT:{$f(this,t);break}case ce.AFTER_BODY:{vBe(this,t);break}case ce.AFTER_AFTER_BODY:case ce.AFTER_AFTER_FRAMESET:{SBe(this,t);break}}}onDoctype(t){switch(this.skipNextNewLine=!1,this.insertionMode){case ce.INITIAL:{EBe(this,t);break}case ce.BEFORE_HEAD:case ce.IN_HEAD:case ce.IN_HEAD_NO_SCRIPT:case ce.AFTER_HEAD:{this._err(t,Be.misplacedDoctype);break}case ce.IN_TABLE_TEXT:{$f(this,t);break}}}onStartTag(t){this.skipNextNewLine=!1,this.currentToken=t,this._processStartTag(t),t.selfClosing&&!t.ackSelfClosing&&this._err(t,Be.nonVoidHtmlElementStartTagWithTrailingSolidus)}_processStartTag(t){this.shouldProcessStartTagTokenInForeignContent(t)?GPe(this,t):this._startTagOutsideForeignContent(t)}_startTagOutsideForeignContent(t){switch(this.insertionMode){case ce.INITIAL:{Uf(this,t);break}case ce.BEFORE_HTML:{xBe(this,t);break}case ce.BEFORE_HEAD:{kBe(this,t);break}case ce.IN_HEAD:{Ho(this,t);break}case ce.IN_HEAD_NO_SCRIPT:{ABe(this,t);break}case ce.AFTER_HEAD:{_Be(this,t);break}case ce.IN_BODY:{Pa(this,t);break}case ce.IN_TABLE:{Jh(this,t);break}case ce.IN_TABLE_TEXT:{$f(this,t);break}case ce.IN_CAPTION:{CPe(this,t);break}case ce.IN_COLUMN_GROUP:{sN(this,t);break}case ce.IN_TABLE_BODY:{A2(this,t);break}case ce.IN_ROW:{F2(this,t);break}case ce.IN_CELL:{FPe(this,t);break}case ce.IN_SELECT:{Yne(this,t);break}case ce.IN_SELECT_IN_TABLE:{RPe(this,t);break}case ce.IN_TEMPLATE:{NPe(this,t);break}case ce.AFTER_BODY:{MPe(this,t);break}case ce.IN_FRAMESET:{IPe(this,t);break}case ce.AFTER_FRAMESET:{BPe(this,t);break}case ce.AFTER_AFTER_BODY:{zPe(this,t);break}case ce.AFTER_AFTER_FRAMESET:{HPe(this,t);break}}}onEndTag(t){this.skipNextNewLine=!1,this.currentToken=t,this.currentNotInHTML?jPe(this,t):this._endTagOutsideForeignContent(t)}_endTagOutsideForeignContent(t){switch(this.insertionMode){case ce.INITIAL:{Uf(this,t);break}case ce.BEFORE_HTML:{wBe(this,t);break}case ce.BEFORE_HEAD:{CBe(this,t);break}case ce.IN_HEAD:{TBe(this,t);break}case ce.IN_HEAD_NO_SCRIPT:{FBe(this,t);break}case ce.AFTER_HEAD:{RBe(this,t);break}case ce.IN_BODY:{T2(this,t);break}case ce.TEXT:{mPe(this,t);break}case ce.IN_TABLE:{Vm(this,t);break}case ce.IN_TABLE_TEXT:{$f(this,t);break}case ce.IN_CAPTION:{TPe(this,t);break}case ce.IN_COLUMN_GROUP:{APe(this,t);break}case ce.IN_TABLE_BODY:{ZR(this,t);break}case ce.IN_ROW:{Kne(this,t);break}case ce.IN_CELL:{_Pe(this,t);break}case ce.IN_SELECT:{Jne(this,t);break}case ce.IN_SELECT_IN_TABLE:{DPe(this,t);break}case ce.IN_TEMPLATE:{LPe(this,t);break}case ce.AFTER_BODY:{Zne(this,t);break}case ce.IN_FRAMESET:{OPe(this,t);break}case ce.AFTER_FRAMESET:{PPe(this,t);break}case ce.AFTER_AFTER_BODY:{B1(this,t);break}}}onEof(t){switch(this.insertionMode){case ce.INITIAL:{Uf(this,t);break}case ce.BEFORE_HTML:{xm(this,t);break}case ce.BEFORE_HEAD:{wm(this,t);break}case ce.IN_HEAD:{km(this,t);break}case ce.IN_HEAD_NO_SCRIPT:{Cm(this,t);break}case ce.AFTER_HEAD:{Tm(this,t);break}case ce.IN_BODY:case ce.IN_TABLE:case ce.IN_CAPTION:case ce.IN_COLUMN_GROUP:case ce.IN_TABLE_BODY:case ce.IN_ROW:case ce.IN_CELL:case ce.IN_SELECT:case ce.IN_SELECT_IN_TABLE:{jne(this,t);break}case ce.TEXT:{gPe(this,t);break}case ce.IN_TABLE_TEXT:{$f(this,t);break}case ce.IN_TEMPLATE:{Xne(this,t);break}case ce.AFTER_BODY:case ce.IN_FRAMESET:case ce.AFTER_FRAMESET:case ce.AFTER_AFTER_BODY:case ce.AFTER_AFTER_FRAMESET:{oN(this,t);break}}}onWhitespaceCharacter(t){if(this.skipNextNewLine&&(this.skipNextNewLine=!1,t.chars.charCodeAt(0)===re.LINE_FEED)){if(t.chars.length===1)return;t.chars=t.chars.substr(1)}if(this.tokenizer.inForeignNode){this._insertCharacters(t);return}switch(this.insertionMode){case ce.IN_HEAD:case ce.IN_HEAD_NO_SCRIPT:case ce.AFTER_HEAD:case ce.TEXT:case ce.IN_COLUMN_GROUP:case ce.IN_SELECT:case ce.IN_SELECT_IN_TABLE:case ce.IN_FRAMESET:case ce.AFTER_FRAMESET:{this._insertCharacters(t);break}case ce.IN_BODY:case ce.IN_CAPTION:case ce.IN_CELL:case ce.IN_TEMPLATE:case ce.AFTER_BODY:case ce.AFTER_AFTER_BODY:case ce.AFTER_AFTER_FRAMESET:{zne(this,t);break}case ce.IN_TABLE:case ce.IN_TABLE_BODY:case ce.IN_ROW:{kk(this,t);break}case ce.IN_TABLE_TEXT:{qne(this,t);break}}}}function pBe(e,t){let r=e.activeFormattingElements.getElementEntryInScopeWithTagName(t.tagName);return r?e.openElements.contains(r.element)?e.openElements.hasInScope(t.tagID)||(r=null):(e.activeFormattingElements.removeEntry(r),r=null):Gne(e,t),r}function fBe(e,t){let r=null,n=e.openElements.stackTop;for(;n>=0;n--){const a=e.openElements.items[n];if(a===t.element)break;e._isSpecialElement(a,e.openElements.tagIDs[n])&&(r=a)}return r||(e.openElements.shortenToLength(Math.max(n,0)),e.activeFormattingElements.removeEntry(t)),r}function mBe(e,t,r){let n=t,a=e.openElements.getCommonAncestor(t);for(let i=0,o=a;o!==r;i++,o=a){a=e.openElements.getCommonAncestor(o);const s=e.activeFormattingElements.getElementEntry(o),l=s&&i>=dBe;!s||l?(l&&e.activeFormattingElements.removeEntry(s),e.openElements.remove(o)):(o=gBe(e,s),n===t&&(e.activeFormattingElements.bookmark=s),e.treeAdapter.detachNode(n),e.treeAdapter.appendChild(o,n),n=o)}return n}function gBe(e,t){const r=e.treeAdapter.getNamespaceURI(t.element),n=e.treeAdapter.createElement(t.token.tagName,r,t.token.attrs);return e.openElements.replace(t.element,n),t.element=n,n}function bBe(e,t,r){const n=e.treeAdapter.getTagName(t),a=Ap(n);if(e._isElementCausesFosterParenting(a))e._fosterParentElement(r);else{const i=e.treeAdapter.getNamespaceURI(t);a===N.TEMPLATE&&i===Xe.HTML&&(t=e.treeAdapter.getTemplateContent(t)),e.treeAdapter.appendChild(t,r)}}function yBe(e,t,r){const n=e.treeAdapter.getNamespaceURI(r.element),{token:a}=r,i=e.treeAdapter.createElement(a.tagName,n,a.attrs);e._adoptNodes(t,i),e.treeAdapter.appendChild(t,i),e.activeFormattingElements.insertElementAfterBookmark(i,a),e.activeFormattingElements.removeEntry(r),e.openElements.remove(r.element),e.openElements.insertAfter(t,i,a.tagID)}function iN(e,t){for(let r=0;r<cBe;r++){const n=pBe(e,t);if(!n)break;const a=fBe(e,n);if(!a)break;e.activeFormattingElements.bookmark=n;const i=mBe(e,a,n.element),o=e.openElements.getCommonAncestor(n.element);e.treeAdapter.detachNode(i),o&&bBe(e,o,i),yBe(e,a,n)}}function XR(e,t){e._appendCommentNode(t,e.openElements.currentTmplContentOrNode)}function vBe(e,t){e._appendCommentNode(t,e.openElements.items[0])}function SBe(e,t){e._appendCommentNode(t,e.document)}function oN(e,t){if(e.stopped=!0,t.location){const r=e.fragmentContext?0:2;for(let n=e.openElements.stackTop;n>=r;n--)e._setEndLocation(e.openElements.items[n],t);if(!e.fragmentContext&&e.openElements.stackTop>=0){const n=e.openElements.items[0],a=e.treeAdapter.getNodeSourceCodeLocation(n);if(a&&!a.endTag&&(e._setEndLocation(n,t),e.openElements.stackTop>=1)){const i=e.openElements.items[1],o=e.treeAdapter.getNodeSourceCodeLocation(i);o&&!o.endTag&&e._setEndLocation(i,t)}}}}function EBe(e,t){e._setDocumentType(t);const r=t.forceQuirks?ro.QUIRKS:X7e(t);J7e(t)||e._err(t,Be.nonConformingDoctype),e.treeAdapter.setDocumentMode(e.document,r),e.insertionMode=ce.BEFORE_HTML}function Uf(e,t){e._err(t,Be.missingDoctype,!0),e.treeAdapter.setDocumentMode(e.document,ro.QUIRKS),e.insertionMode=ce.BEFORE_HTML,e._processToken(t)}function xBe(e,t){t.tagID===N.HTML?(e._insertElement(t,Xe.HTML),e.insertionMode=ce.BEFORE_HEAD):xm(e,t)}function wBe(e,t){const r=t.tagID;(r===N.HTML||r===N.HEAD||r===N.BODY||r===N.BR)&&xm(e,t)}function xm(e,t){e._insertFakeRootElement(),e.insertionMode=ce.BEFORE_HEAD,e._processToken(t)}function kBe(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.HEAD:{e._insertElement(t,Xe.HTML),e.headElement=e.openElements.current,e.insertionMode=ce.IN_HEAD;break}default:wm(e,t)}}function CBe(e,t){const r=t.tagID;r===N.HEAD||r===N.BODY||r===N.HTML||r===N.BR?wm(e,t):e._err(t,Be.endTagWithoutMatchingOpenElement)}function wm(e,t){e._insertFakeElement(ke.HEAD,N.HEAD),e.headElement=e.openElements.current,e.insertionMode=ce.IN_HEAD,e._processToken(t)}function Ho(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.BASE:case N.BASEFONT:case N.BGSOUND:case N.LINK:case N.META:{e._appendElement(t,Xe.HTML),t.ackSelfClosing=!0;break}case N.TITLE:{e._switchToTextParsing(t,Dn.RCDATA);break}case N.NOSCRIPT:{e.options.scriptingEnabled?e._switchToTextParsing(t,Dn.RAWTEXT):(e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_HEAD_NO_SCRIPT);break}case N.NOFRAMES:case N.STYLE:{e._switchToTextParsing(t,Dn.RAWTEXT);break}case N.SCRIPT:{e._switchToTextParsing(t,Dn.SCRIPT_DATA);break}case N.TEMPLATE:{e._insertTemplate(t),e.activeFormattingElements.insertMarker(),e.framesetOk=!1,e.insertionMode=ce.IN_TEMPLATE,e.tmplInsertionModeStack.unshift(ce.IN_TEMPLATE);break}case N.HEAD:{e._err(t,Be.misplacedStartTagForHeadElement);break}default:km(e,t)}}function TBe(e,t){switch(t.tagID){case N.HEAD:{e.openElements.pop(),e.insertionMode=ce.AFTER_HEAD;break}case N.BODY:case N.BR:case N.HTML:{km(e,t);break}case N.TEMPLATE:{sd(e,t);break}default:e._err(t,Be.endTagWithoutMatchingOpenElement)}}function sd(e,t){e.openElements.tmplCount>0?(e.openElements.generateImpliedEndTagsThoroughly(),e.openElements.currentTagId!==N.TEMPLATE&&e._err(t,Be.closingOfElementWithOpenChildElements),e.openElements.popUntilTagNamePopped(N.TEMPLATE),e.activeFormattingElements.clearToLastMarker(),e.tmplInsertionModeStack.shift(),e._resetInsertionMode()):e._err(t,Be.endTagWithoutMatchingOpenElement)}function km(e,t){e.openElements.pop(),e.insertionMode=ce.AFTER_HEAD,e._processToken(t)}function ABe(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.BASEFONT:case N.BGSOUND:case N.HEAD:case N.LINK:case N.META:case N.NOFRAMES:case N.STYLE:{Ho(e,t);break}case N.NOSCRIPT:{e._err(t,Be.nestedNoscriptInHead);break}default:Cm(e,t)}}function FBe(e,t){switch(t.tagID){case N.NOSCRIPT:{e.openElements.pop(),e.insertionMode=ce.IN_HEAD;break}case N.BR:{Cm(e,t);break}default:e._err(t,Be.endTagWithoutMatchingOpenElement)}}function Cm(e,t){const r=t.type===Er.EOF?Be.openElementsLeftAfterEof:Be.disallowedContentInNoscriptInHead;e._err(t,r),e.openElements.pop(),e.insertionMode=ce.IN_HEAD,e._processToken(t)}function _Be(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.BODY:{e._insertElement(t,Xe.HTML),e.framesetOk=!1,e.insertionMode=ce.IN_BODY;break}case N.FRAMESET:{e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_FRAMESET;break}case N.BASE:case N.BASEFONT:case N.BGSOUND:case N.LINK:case N.META:case N.NOFRAMES:case N.SCRIPT:case N.STYLE:case N.TEMPLATE:case N.TITLE:{e._err(t,Be.abandonedHeadElementChild),e.openElements.push(e.headElement,N.HEAD),Ho(e,t),e.openElements.remove(e.headElement);break}case N.HEAD:{e._err(t,Be.misplacedStartTagForHeadElement);break}default:Tm(e,t)}}function RBe(e,t){switch(t.tagID){case N.BODY:case N.HTML:case N.BR:{Tm(e,t);break}case N.TEMPLATE:{sd(e,t);break}default:e._err(t,Be.endTagWithoutMatchingOpenElement)}}function Tm(e,t){e._insertFakeElement(ke.BODY,N.BODY),e.insertionMode=ce.IN_BODY,C2(e,t)}function C2(e,t){switch(t.type){case Er.CHARACTER:{Hne(e,t);break}case Er.WHITESPACE_CHARACTER:{zne(e,t);break}case Er.COMMENT:{XR(e,t);break}case Er.START_TAG:{Pa(e,t);break}case Er.END_TAG:{T2(e,t);break}case Er.EOF:{jne(e,t);break}}}function zne(e,t){e._reconstructActiveFormattingElements(),e._insertCharacters(t)}function Hne(e,t){e._reconstructActiveFormattingElements(),e._insertCharacters(t),e.framesetOk=!1}function DBe(e,t){e.openElements.tmplCount===0&&e.treeAdapter.adoptAttributes(e.openElements.items[0],t.attrs)}function NBe(e,t){const r=e.openElements.tryPeekProperlyNestedBodyElement();r&&e.openElements.tmplCount===0&&(e.framesetOk=!1,e.treeAdapter.adoptAttributes(r,t.attrs))}function LBe(e,t){const r=e.openElements.tryPeekProperlyNestedBodyElement();e.framesetOk&&r&&(e.treeAdapter.detachNode(r),e.openElements.popAllUpToHtmlElement(),e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_FRAMESET)}function MBe(e,t){e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._insertElement(t,Xe.HTML)}function IBe(e,t){e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e.openElements.currentTagId!==void 0&&JR.has(e.openElements.currentTagId)&&e.openElements.pop(),e._insertElement(t,Xe.HTML)}function OBe(e,t){e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._insertElement(t,Xe.HTML),e.skipNextNewLine=!0,e.framesetOk=!1}function BBe(e,t){const r=e.openElements.tmplCount>0;(!e.formElement||r)&&(e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._insertElement(t,Xe.HTML),r||(e.formElement=e.openElements.current))}function PBe(e,t){e.framesetOk=!1;const r=t.tagID;for(let n=e.openElements.stackTop;n>=0;n--){const a=e.openElements.tagIDs[n];if(r===N.LI&&a===N.LI||(r===N.DD||r===N.DT)&&(a===N.DD||a===N.DT)){e.openElements.generateImpliedEndTagsWithExclusion(a),e.openElements.popUntilTagNamePopped(a);break}if(a!==N.ADDRESS&&a!==N.DIV&&a!==N.P&&e._isSpecialElement(e.openElements.items[n],a))break}e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._insertElement(t,Xe.HTML)}function zBe(e,t){e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._insertElement(t,Xe.HTML),e.tokenizer.state=Dn.PLAINTEXT}function HBe(e,t){e.openElements.hasInScope(N.BUTTON)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(N.BUTTON)),e._reconstructActiveFormattingElements(),e._insertElement(t,Xe.HTML),e.framesetOk=!1}function UBe(e,t){const r=e.activeFormattingElements.getElementEntryInScopeWithTagName(ke.A);r&&(iN(e,t),e.openElements.remove(r.element),e.activeFormattingElements.removeEntry(r)),e._reconstructActiveFormattingElements(),e._insertElement(t,Xe.HTML),e.activeFormattingElements.pushElement(e.openElements.current,t)}function $Be(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Xe.HTML),e.activeFormattingElements.pushElement(e.openElements.current,t)}function GBe(e,t){e._reconstructActiveFormattingElements(),e.openElements.hasInScope(N.NOBR)&&(iN(e,t),e._reconstructActiveFormattingElements()),e._insertElement(t,Xe.HTML),e.activeFormattingElements.pushElement(e.openElements.current,t)}function jBe(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Xe.HTML),e.activeFormattingElements.insertMarker(),e.framesetOk=!1}function qBe(e,t){e.treeAdapter.getDocumentMode(e.document)!==ro.QUIRKS&&e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._insertElement(t,Xe.HTML),e.framesetOk=!1,e.insertionMode=ce.IN_TABLE}function Une(e,t){e._reconstructActiveFormattingElements(),e._appendElement(t,Xe.HTML),e.framesetOk=!1,t.ackSelfClosing=!0}function $ne(e){const t=Rne(e,Dc.TYPE);return t!=null&&t.toLowerCase()===uBe}function WBe(e,t){e._reconstructActiveFormattingElements(),e._appendElement(t,Xe.HTML),$ne(t)||(e.framesetOk=!1),t.ackSelfClosing=!0}function VBe(e,t){e._appendElement(t,Xe.HTML),t.ackSelfClosing=!0}function KBe(e,t){e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._appendElement(t,Xe.HTML),e.framesetOk=!1,t.ackSelfClosing=!0}function YBe(e,t){t.tagName=ke.IMG,t.tagID=N.IMG,Une(e,t)}function JBe(e,t){e._insertElement(t,Xe.HTML),e.skipNextNewLine=!0,e.tokenizer.state=Dn.RCDATA,e.originalInsertionMode=e.insertionMode,e.framesetOk=!1,e.insertionMode=ce.TEXT}function XBe(e,t){e.openElements.hasInButtonScope(N.P)&&e._closePElement(),e._reconstructActiveFormattingElements(),e.framesetOk=!1,e._switchToTextParsing(t,Dn.RAWTEXT)}function ZBe(e,t){e.framesetOk=!1,e._switchToTextParsing(t,Dn.RAWTEXT)}function kz(e,t){e._switchToTextParsing(t,Dn.RAWTEXT)}function QBe(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Xe.HTML),e.framesetOk=!1,e.insertionMode=e.insertionMode===ce.IN_TABLE||e.insertionMode===ce.IN_CAPTION||e.insertionMode===ce.IN_TABLE_BODY||e.insertionMode===ce.IN_ROW||e.insertionMode===ce.IN_CELL?ce.IN_SELECT_IN_TABLE:ce.IN_SELECT}function ePe(e,t){e.openElements.currentTagId===N.OPTION&&e.openElements.pop(),e._reconstructActiveFormattingElements(),e._insertElement(t,Xe.HTML)}function tPe(e,t){e.openElements.hasInScope(N.RUBY)&&e.openElements.generateImpliedEndTags(),e._insertElement(t,Xe.HTML)}function rPe(e,t){e.openElements.hasInScope(N.RUBY)&&e.openElements.generateImpliedEndTagsWithExclusion(N.RTC),e._insertElement(t,Xe.HTML)}function nPe(e,t){e._reconstructActiveFormattingElements(),One(t),aN(t),t.selfClosing?e._appendElement(t,Xe.MATHML):e._insertElement(t,Xe.MATHML),t.ackSelfClosing=!0}function aPe(e,t){e._reconstructActiveFormattingElements(),Bne(t),aN(t),t.selfClosing?e._appendElement(t,Xe.SVG):e._insertElement(t,Xe.SVG),t.ackSelfClosing=!0}function Cz(e,t){e._reconstructActiveFormattingElements(),e._insertElement(t,Xe.HTML)}function Pa(e,t){switch(t.tagID){case N.I:case N.S:case N.B:case N.U:case N.EM:case N.TT:case N.BIG:case N.CODE:case N.FONT:case N.SMALL:case N.STRIKE:case N.STRONG:{$Be(e,t);break}case N.A:{UBe(e,t);break}case N.H1:case N.H2:case N.H3:case N.H4:case N.H5:case N.H6:{IBe(e,t);break}case N.P:case N.DL:case N.OL:case N.UL:case N.DIV:case N.DIR:case N.NAV:case N.MAIN:case N.MENU:case N.ASIDE:case N.CENTER:case N.FIGURE:case N.FOOTER:case N.HEADER:case N.HGROUP:case N.DIALOG:case N.DETAILS:case N.ADDRESS:case N.ARTICLE:case N.SEARCH:case N.SECTION:case N.SUMMARY:case N.FIELDSET:case N.BLOCKQUOTE:case N.FIGCAPTION:{MBe(e,t);break}case N.LI:case N.DD:case N.DT:{PBe(e,t);break}case N.BR:case N.IMG:case N.WBR:case N.AREA:case N.EMBED:case N.KEYGEN:{Une(e,t);break}case N.HR:{KBe(e,t);break}case N.RB:case N.RTC:{tPe(e,t);break}case N.RT:case N.RP:{rPe(e,t);break}case N.PRE:case N.LISTING:{OBe(e,t);break}case N.XMP:{XBe(e,t);break}case N.SVG:{aPe(e,t);break}case N.HTML:{DBe(e,t);break}case N.BASE:case N.LINK:case N.META:case N.STYLE:case N.TITLE:case N.SCRIPT:case N.BGSOUND:case N.BASEFONT:case N.TEMPLATE:{Ho(e,t);break}case N.BODY:{NBe(e,t);break}case N.FORM:{BBe(e,t);break}case N.NOBR:{GBe(e,t);break}case N.MATH:{nPe(e,t);break}case N.TABLE:{qBe(e,t);break}case N.INPUT:{WBe(e,t);break}case N.PARAM:case N.TRACK:case N.SOURCE:{VBe(e,t);break}case N.IMAGE:{YBe(e,t);break}case N.BUTTON:{HBe(e,t);break}case N.APPLET:case N.OBJECT:case N.MARQUEE:{jBe(e,t);break}case N.IFRAME:{ZBe(e,t);break}case N.SELECT:{QBe(e,t);break}case N.OPTION:case N.OPTGROUP:{ePe(e,t);break}case N.NOEMBED:case N.NOFRAMES:{kz(e,t);break}case N.FRAMESET:{LBe(e,t);break}case N.TEXTAREA:{JBe(e,t);break}case N.NOSCRIPT:{e.options.scriptingEnabled?kz(e,t):Cz(e,t);break}case N.PLAINTEXT:{zBe(e,t);break}case N.COL:case N.TH:case N.TD:case N.TR:case N.HEAD:case N.FRAME:case N.TBODY:case N.TFOOT:case N.THEAD:case N.CAPTION:case N.COLGROUP:break;default:Cz(e,t)}}function iPe(e,t){if(e.openElements.hasInScope(N.BODY)&&(e.insertionMode=ce.AFTER_BODY,e.options.sourceCodeLocationInfo)){const r=e.openElements.tryPeekProperlyNestedBodyElement();r&&e._setEndLocation(r,t)}}function oPe(e,t){e.openElements.hasInScope(N.BODY)&&(e.insertionMode=ce.AFTER_BODY,Zne(e,t))}function sPe(e,t){const r=t.tagID;e.openElements.hasInScope(r)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(r))}function lPe(e){const t=e.openElements.tmplCount>0,{formElement:r}=e;t||(e.formElement=null),(r||t)&&e.openElements.hasInScope(N.FORM)&&(e.openElements.generateImpliedEndTags(),t?e.openElements.popUntilTagNamePopped(N.FORM):r&&e.openElements.remove(r))}function uPe(e){e.openElements.hasInButtonScope(N.P)||e._insertFakeElement(ke.P,N.P),e._closePElement()}function cPe(e){e.openElements.hasInListItemScope(N.LI)&&(e.openElements.generateImpliedEndTagsWithExclusion(N.LI),e.openElements.popUntilTagNamePopped(N.LI))}function dPe(e,t){const r=t.tagID;e.openElements.hasInScope(r)&&(e.openElements.generateImpliedEndTagsWithExclusion(r),e.openElements.popUntilTagNamePopped(r))}function hPe(e){e.openElements.hasNumberedHeaderInScope()&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilNumberedHeaderPopped())}function pPe(e,t){const r=t.tagID;e.openElements.hasInScope(r)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(r),e.activeFormattingElements.clearToLastMarker())}function fPe(e){e._reconstructActiveFormattingElements(),e._insertFakeElement(ke.BR,N.BR),e.openElements.pop(),e.framesetOk=!1}function Gne(e,t){const r=t.tagName,n=t.tagID;for(let a=e.openElements.stackTop;a>0;a--){const i=e.openElements.items[a],o=e.openElements.tagIDs[a];if(n===o&&(n!==N.UNKNOWN||e.treeAdapter.getTagName(i)===r)){e.openElements.generateImpliedEndTagsWithExclusion(n),e.openElements.stackTop>=a&&e.openElements.shortenToLength(a);break}if(e._isSpecialElement(i,o))break}}function T2(e,t){switch(t.tagID){case N.A:case N.B:case N.I:case N.S:case N.U:case N.EM:case N.TT:case N.BIG:case N.CODE:case N.FONT:case N.NOBR:case N.SMALL:case N.STRIKE:case N.STRONG:{iN(e,t);break}case N.P:{uPe(e);break}case N.DL:case N.UL:case N.OL:case N.DIR:case N.DIV:case N.NAV:case N.PRE:case N.MAIN:case N.MENU:case N.ASIDE:case N.BUTTON:case N.CENTER:case N.FIGURE:case N.FOOTER:case N.HEADER:case N.HGROUP:case N.DIALOG:case N.ADDRESS:case N.ARTICLE:case N.DETAILS:case N.SEARCH:case N.SECTION:case N.SUMMARY:case N.LISTING:case N.FIELDSET:case N.BLOCKQUOTE:case N.FIGCAPTION:{sPe(e,t);break}case N.LI:{cPe(e);break}case N.DD:case N.DT:{dPe(e,t);break}case N.H1:case N.H2:case N.H3:case N.H4:case N.H5:case N.H6:{hPe(e);break}case N.BR:{fPe(e);break}case N.BODY:{iPe(e,t);break}case N.HTML:{oPe(e,t);break}case N.FORM:{lPe(e);break}case N.APPLET:case N.OBJECT:case N.MARQUEE:{pPe(e,t);break}case N.TEMPLATE:{sd(e,t);break}default:Gne(e,t)}}function jne(e,t){e.tmplInsertionModeStack.length>0?Xne(e,t):oN(e,t)}function mPe(e,t){var r;t.tagID===N.SCRIPT&&((r=e.scriptHandler)===null||r===void 0||r.call(e,e.openElements.current)),e.openElements.pop(),e.insertionMode=e.originalInsertionMode}function gPe(e,t){e._err(t,Be.eofInElementThatCanContainOnlyText),e.openElements.pop(),e.insertionMode=e.originalInsertionMode,e.onEof(t)}function kk(e,t){if(e.openElements.currentTagId!==void 0&&Pne.has(e.openElements.currentTagId))switch(e.pendingCharacterTokens.length=0,e.hasNonWhitespacePendingCharacterToken=!1,e.originalInsertionMode=e.insertionMode,e.insertionMode=ce.IN_TABLE_TEXT,t.type){case Er.CHARACTER:{Wne(e,t);break}case Er.WHITESPACE_CHARACTER:{qne(e,t);break}}else Pg(e,t)}function bPe(e,t){e.openElements.clearBackToTableContext(),e.activeFormattingElements.insertMarker(),e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_CAPTION}function yPe(e,t){e.openElements.clearBackToTableContext(),e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_COLUMN_GROUP}function vPe(e,t){e.openElements.clearBackToTableContext(),e._insertFakeElement(ke.COLGROUP,N.COLGROUP),e.insertionMode=ce.IN_COLUMN_GROUP,sN(e,t)}function SPe(e,t){e.openElements.clearBackToTableContext(),e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_TABLE_BODY}function EPe(e,t){e.openElements.clearBackToTableContext(),e._insertFakeElement(ke.TBODY,N.TBODY),e.insertionMode=ce.IN_TABLE_BODY,A2(e,t)}function xPe(e,t){e.openElements.hasInTableScope(N.TABLE)&&(e.openElements.popUntilTagNamePopped(N.TABLE),e._resetInsertionMode(),e._processStartTag(t))}function wPe(e,t){$ne(t)?e._appendElement(t,Xe.HTML):Pg(e,t),t.ackSelfClosing=!0}function kPe(e,t){!e.formElement&&e.openElements.tmplCount===0&&(e._insertElement(t,Xe.HTML),e.formElement=e.openElements.current,e.openElements.pop())}function Jh(e,t){switch(t.tagID){case N.TD:case N.TH:case N.TR:{EPe(e,t);break}case N.STYLE:case N.SCRIPT:case N.TEMPLATE:{Ho(e,t);break}case N.COL:{vPe(e,t);break}case N.FORM:{kPe(e,t);break}case N.TABLE:{xPe(e,t);break}case N.TBODY:case N.TFOOT:case N.THEAD:{SPe(e,t);break}case N.INPUT:{wPe(e,t);break}case N.CAPTION:{bPe(e,t);break}case N.COLGROUP:{yPe(e,t);break}default:Pg(e,t)}}function Vm(e,t){switch(t.tagID){case N.TABLE:{e.openElements.hasInTableScope(N.TABLE)&&(e.openElements.popUntilTagNamePopped(N.TABLE),e._resetInsertionMode());break}case N.TEMPLATE:{sd(e,t);break}case N.BODY:case N.CAPTION:case N.COL:case N.COLGROUP:case N.HTML:case N.TBODY:case N.TD:case N.TFOOT:case N.TH:case N.THEAD:case N.TR:break;default:Pg(e,t)}}function Pg(e,t){const r=e.fosterParentingEnabled;e.fosterParentingEnabled=!0,C2(e,t),e.fosterParentingEnabled=r}function qne(e,t){e.pendingCharacterTokens.push(t)}function Wne(e,t){e.pendingCharacterTokens.push(t),e.hasNonWhitespacePendingCharacterToken=!0}function $f(e,t){let r=0;if(e.hasNonWhitespacePendingCharacterToken)for(;r<e.pendingCharacterTokens.length;r++)Pg(e,e.pendingCharacterTokens[r]);else for(;r<e.pendingCharacterTokens.length;r++)e._insertCharacters(e.pendingCharacterTokens[r]);e.insertionMode=e.originalInsertionMode,e._processToken(t)}const Vne=new Set([N.CAPTION,N.COL,N.COLGROUP,N.TBODY,N.TD,N.TFOOT,N.TH,N.THEAD,N.TR]);function CPe(e,t){const r=t.tagID;Vne.has(r)?e.openElements.hasInTableScope(N.CAPTION)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(N.CAPTION),e.activeFormattingElements.clearToLastMarker(),e.insertionMode=ce.IN_TABLE,Jh(e,t)):Pa(e,t)}function TPe(e,t){const r=t.tagID;switch(r){case N.CAPTION:case N.TABLE:{e.openElements.hasInTableScope(N.CAPTION)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(N.CAPTION),e.activeFormattingElements.clearToLastMarker(),e.insertionMode=ce.IN_TABLE,r===N.TABLE&&Vm(e,t));break}case N.BODY:case N.COL:case N.COLGROUP:case N.HTML:case N.TBODY:case N.TD:case N.TFOOT:case N.TH:case N.THEAD:case N.TR:break;default:T2(e,t)}}function sN(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.COL:{e._appendElement(t,Xe.HTML),t.ackSelfClosing=!0;break}case N.TEMPLATE:{Ho(e,t);break}default:Ly(e,t)}}function APe(e,t){switch(t.tagID){case N.COLGROUP:{e.openElements.currentTagId===N.COLGROUP&&(e.openElements.pop(),e.insertionMode=ce.IN_TABLE);break}case N.TEMPLATE:{sd(e,t);break}case N.COL:break;default:Ly(e,t)}}function Ly(e,t){e.openElements.currentTagId===N.COLGROUP&&(e.openElements.pop(),e.insertionMode=ce.IN_TABLE,e._processToken(t))}function A2(e,t){switch(t.tagID){case N.TR:{e.openElements.clearBackToTableBodyContext(),e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_ROW;break}case N.TH:case N.TD:{e.openElements.clearBackToTableBodyContext(),e._insertFakeElement(ke.TR,N.TR),e.insertionMode=ce.IN_ROW,F2(e,t);break}case N.CAPTION:case N.COL:case N.COLGROUP:case N.TBODY:case N.TFOOT:case N.THEAD:{e.openElements.hasTableBodyContextInTableScope()&&(e.openElements.clearBackToTableBodyContext(),e.openElements.pop(),e.insertionMode=ce.IN_TABLE,Jh(e,t));break}default:Jh(e,t)}}function ZR(e,t){const r=t.tagID;switch(t.tagID){case N.TBODY:case N.TFOOT:case N.THEAD:{e.openElements.hasInTableScope(r)&&(e.openElements.clearBackToTableBodyContext(),e.openElements.pop(),e.insertionMode=ce.IN_TABLE);break}case N.TABLE:{e.openElements.hasTableBodyContextInTableScope()&&(e.openElements.clearBackToTableBodyContext(),e.openElements.pop(),e.insertionMode=ce.IN_TABLE,Vm(e,t));break}case N.BODY:case N.CAPTION:case N.COL:case N.COLGROUP:case N.HTML:case N.TD:case N.TH:case N.TR:break;default:Vm(e,t)}}function F2(e,t){switch(t.tagID){case N.TH:case N.TD:{e.openElements.clearBackToTableRowContext(),e._insertElement(t,Xe.HTML),e.insertionMode=ce.IN_CELL,e.activeFormattingElements.insertMarker();break}case N.CAPTION:case N.COL:case N.COLGROUP:case N.TBODY:case N.TFOOT:case N.THEAD:case N.TR:{e.openElements.hasInTableScope(N.TR)&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=ce.IN_TABLE_BODY,A2(e,t));break}default:Jh(e,t)}}function Kne(e,t){switch(t.tagID){case N.TR:{e.openElements.hasInTableScope(N.TR)&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=ce.IN_TABLE_BODY);break}case N.TABLE:{e.openElements.hasInTableScope(N.TR)&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=ce.IN_TABLE_BODY,ZR(e,t));break}case N.TBODY:case N.TFOOT:case N.THEAD:{(e.openElements.hasInTableScope(t.tagID)||e.openElements.hasInTableScope(N.TR))&&(e.openElements.clearBackToTableRowContext(),e.openElements.pop(),e.insertionMode=ce.IN_TABLE_BODY,ZR(e,t));break}case N.BODY:case N.CAPTION:case N.COL:case N.COLGROUP:case N.HTML:case N.TD:case N.TH:break;default:Vm(e,t)}}function FPe(e,t){const r=t.tagID;Vne.has(r)?(e.openElements.hasInTableScope(N.TD)||e.openElements.hasInTableScope(N.TH))&&(e._closeTableCell(),F2(e,t)):Pa(e,t)}function _Pe(e,t){const r=t.tagID;switch(r){case N.TD:case N.TH:{e.openElements.hasInTableScope(r)&&(e.openElements.generateImpliedEndTags(),e.openElements.popUntilTagNamePopped(r),e.activeFormattingElements.clearToLastMarker(),e.insertionMode=ce.IN_ROW);break}case N.TABLE:case N.TBODY:case N.TFOOT:case N.THEAD:case N.TR:{e.openElements.hasInTableScope(r)&&(e._closeTableCell(),Kne(e,t));break}case N.BODY:case N.CAPTION:case N.COL:case N.COLGROUP:case N.HTML:break;default:T2(e,t)}}function Yne(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.OPTION:{e.openElements.currentTagId===N.OPTION&&e.openElements.pop(),e._insertElement(t,Xe.HTML);break}case N.OPTGROUP:{e.openElements.currentTagId===N.OPTION&&e.openElements.pop(),e.openElements.currentTagId===N.OPTGROUP&&e.openElements.pop(),e._insertElement(t,Xe.HTML);break}case N.HR:{e.openElements.currentTagId===N.OPTION&&e.openElements.pop(),e.openElements.currentTagId===N.OPTGROUP&&e.openElements.pop(),e._appendElement(t,Xe.HTML),t.ackSelfClosing=!0;break}case N.INPUT:case N.KEYGEN:case N.TEXTAREA:case N.SELECT:{e.openElements.hasInSelectScope(N.SELECT)&&(e.openElements.popUntilTagNamePopped(N.SELECT),e._resetInsertionMode(),t.tagID!==N.SELECT&&e._processStartTag(t));break}case N.SCRIPT:case N.TEMPLATE:{Ho(e,t);break}}}function Jne(e,t){switch(t.tagID){case N.OPTGROUP:{e.openElements.stackTop>0&&e.openElements.currentTagId===N.OPTION&&e.openElements.tagIDs[e.openElements.stackTop-1]===N.OPTGROUP&&e.openElements.pop(),e.openElements.currentTagId===N.OPTGROUP&&e.openElements.pop();break}case N.OPTION:{e.openElements.currentTagId===N.OPTION&&e.openElements.pop();break}case N.SELECT:{e.openElements.hasInSelectScope(N.SELECT)&&(e.openElements.popUntilTagNamePopped(N.SELECT),e._resetInsertionMode());break}case N.TEMPLATE:{sd(e,t);break}}}function RPe(e,t){const r=t.tagID;r===N.CAPTION||r===N.TABLE||r===N.TBODY||r===N.TFOOT||r===N.THEAD||r===N.TR||r===N.TD||r===N.TH?(e.openElements.popUntilTagNamePopped(N.SELECT),e._resetInsertionMode(),e._processStartTag(t)):Yne(e,t)}function DPe(e,t){const r=t.tagID;r===N.CAPTION||r===N.TABLE||r===N.TBODY||r===N.TFOOT||r===N.THEAD||r===N.TR||r===N.TD||r===N.TH?e.openElements.hasInTableScope(r)&&(e.openElements.popUntilTagNamePopped(N.SELECT),e._resetInsertionMode(),e.onEndTag(t)):Jne(e,t)}function NPe(e,t){switch(t.tagID){case N.BASE:case N.BASEFONT:case N.BGSOUND:case N.LINK:case N.META:case N.NOFRAMES:case N.SCRIPT:case N.STYLE:case N.TEMPLATE:case N.TITLE:{Ho(e,t);break}case N.CAPTION:case N.COLGROUP:case N.TBODY:case N.TFOOT:case N.THEAD:{e.tmplInsertionModeStack[0]=ce.IN_TABLE,e.insertionMode=ce.IN_TABLE,Jh(e,t);break}case N.COL:{e.tmplInsertionModeStack[0]=ce.IN_COLUMN_GROUP,e.insertionMode=ce.IN_COLUMN_GROUP,sN(e,t);break}case N.TR:{e.tmplInsertionModeStack[0]=ce.IN_TABLE_BODY,e.insertionMode=ce.IN_TABLE_BODY,A2(e,t);break}case N.TD:case N.TH:{e.tmplInsertionModeStack[0]=ce.IN_ROW,e.insertionMode=ce.IN_ROW,F2(e,t);break}default:e.tmplInsertionModeStack[0]=ce.IN_BODY,e.insertionMode=ce.IN_BODY,Pa(e,t)}}function LPe(e,t){t.tagID===N.TEMPLATE&&sd(e,t)}function Xne(e,t){e.openElements.tmplCount>0?(e.openElements.popUntilTagNamePopped(N.TEMPLATE),e.activeFormattingElements.clearToLastMarker(),e.tmplInsertionModeStack.shift(),e._resetInsertionMode(),e.onEof(t)):oN(e,t)}function MPe(e,t){t.tagID===N.HTML?Pa(e,t):My(e,t)}function Zne(e,t){var r;if(t.tagID===N.HTML){if(e.fragmentContext||(e.insertionMode=ce.AFTER_AFTER_BODY),e.options.sourceCodeLocationInfo&&e.openElements.tagIDs[0]===N.HTML){e._setEndLocation(e.openElements.items[0],t);const n=e.openElements.items[1];n&&!(!((r=e.treeAdapter.getNodeSourceCodeLocation(n))===null||r===void 0)&&r.endTag)&&e._setEndLocation(n,t)}}else My(e,t)}function My(e,t){e.insertionMode=ce.IN_BODY,C2(e,t)}function IPe(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.FRAMESET:{e._insertElement(t,Xe.HTML);break}case N.FRAME:{e._appendElement(t,Xe.HTML),t.ackSelfClosing=!0;break}case N.NOFRAMES:{Ho(e,t);break}}}function OPe(e,t){t.tagID===N.FRAMESET&&!e.openElements.isRootHtmlElementCurrent()&&(e.openElements.pop(),!e.fragmentContext&&e.openElements.currentTagId!==N.FRAMESET&&(e.insertionMode=ce.AFTER_FRAMESET))}function BPe(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.NOFRAMES:{Ho(e,t);break}}}function PPe(e,t){t.tagID===N.HTML&&(e.insertionMode=ce.AFTER_AFTER_FRAMESET)}function zPe(e,t){t.tagID===N.HTML?Pa(e,t):B1(e,t)}function B1(e,t){e.insertionMode=ce.IN_BODY,C2(e,t)}function HPe(e,t){switch(t.tagID){case N.HTML:{Pa(e,t);break}case N.NOFRAMES:{Ho(e,t);break}}}function UPe(e,t){t.chars=mn,e._insertCharacters(t)}function $Pe(e,t){e._insertCharacters(t),e.framesetOk=!1}function Qne(e){for(;e.treeAdapter.getNamespaceURI(e.openElements.current)!==Xe.HTML&&e.openElements.currentTagId!==void 0&&!e._isIntegrationPoint(e.openElements.currentTagId,e.openElements.current);)e.openElements.pop()}function GPe(e,t){if(aBe(t))Qne(e),e._startTagOutsideForeignContent(t);else{const r=e._getAdjustedCurrentElement(),n=e.treeAdapter.getNamespaceURI(r);n===Xe.MATHML?One(t):n===Xe.SVG&&(iBe(t),Bne(t)),aN(t),t.selfClosing?e._appendElement(t,n):e._insertElement(t,n),t.ackSelfClosing=!0}}function jPe(e,t){if(t.tagID===N.P||t.tagID===N.BR){Qne(e),e._endTagOutsideForeignContent(t);return}for(let r=e.openElements.stackTop;r>0;r--){const n=e.openElements.items[r];if(e.treeAdapter.getNamespaceURI(n)===Xe.HTML){e._endTagOutsideForeignContent(t);break}const a=e.treeAdapter.getTagName(n);if(a.toLowerCase()===t.tagName){t.tagName=a,e.openElements.shortenToLength(r);break}}}ke.AREA,ke.BASE,ke.BASEFONT,ke.BGSOUND,ke.BR,ke.COL,ke.EMBED,ke.FRAME,ke.HR,ke.IMG,ke.INPUT,ke.KEYGEN,ke.LINK,ke.META,ke.PARAM,ke.SOURCE,ke.TRACK,ke.WBR;const qPe=/<(\/?)(iframe|noembed|noframes|plaintext|script|style|textarea|title|xmp)(?=[\t\n\f\r />])/gi,WPe=new Set(["mdxFlowExpression","mdxJsxFlowElement","mdxJsxTextElement","mdxTextExpression","mdxjsEsm"]),Tz={sourceCodeLocationInfo:!0,scriptingEnabled:!1};function eae(e,t){const r=rze(e),n=Kre("type",{handlers:{root:VPe,element:KPe,text:YPe,comment:rae,doctype:JPe,raw:ZPe},unknown:QPe}),a={parser:r?new wz(Tz):wz.getFragmentParser(void 0,Tz),handle(s){n(s,a)},stitches:!1,options:t||{}};n(e,a),Fp(a,vs());const i=r?a.parser.document:a.parser.getFragment(),o=n7e(i,{file:a.options.file});return a.stitches&&Ig(o,"comment",function(s,l,u){const d=s;if(d.value.stitch&&u&&l!==void 0){const p=u.children;return p[l]=d.value.stitch,l}}),o.type==="root"&&o.children.length===1&&o.children[0].type===e.type?o.children[0]:o}function tae(e,t){let r=-1;if(e)for(;++r<e.length;)t.handle(e[r])}function VPe(e,t){tae(e.children,t)}function KPe(e,t){eze(e,t),tae(e.children,t),tze(e,t)}function YPe(e,t){t.parser.tokenizer.state>4&&(t.parser.tokenizer.state=0);const r={type:Er.CHARACTER,chars:e.value,location:zg(e)};Fp(t,vs(e)),t.parser.currentToken=r,t.parser._processToken(t.parser.currentToken)}function JPe(e,t){const r={type:Er.DOCTYPE,name:"html",forceQuirks:!1,publicId:"",systemId:"",location:zg(e)};Fp(t,vs(e)),t.parser.currentToken=r,t.parser._processToken(t.parser.currentToken)}function XPe(e,t){t.stitches=!0;const r=nze(e);if("children"in e&&"children"in r){const n=eae({type:"root",children:e.children},t.options);r.children=n.children}rae({type:"comment",value:{stitch:r}},t)}function rae(e,t){const r=e.value,n={type:Er.COMMENT,data:r,location:zg(e)};Fp(t,vs(e)),t.parser.currentToken=n,t.parser._processToken(t.parser.currentToken)}function ZPe(e,t){if(t.parser.tokenizer.preprocessor.html="",t.parser.tokenizer.preprocessor.pos=-1,t.parser.tokenizer.preprocessor.lastGapPos=-2,t.parser.tokenizer.preprocessor.gapStack=[],t.parser.tokenizer.preprocessor.skipNextNewLine=!1,t.parser.tokenizer.preprocessor.lastChunkWritten=!1,t.parser.tokenizer.preprocessor.endOfChunkHit=!1,t.parser.tokenizer.preprocessor.isEol=!1,nae(t,vs(e)),t.parser.tokenizer.write(t.options.tagfilter?e.value.replace(qPe,"&lt;$1$2"):e.value,!1),t.parser.tokenizer._runParsingLoop(),t.parser.tokenizer.state===72||t.parser.tokenizer.state===78){t.parser.tokenizer.preprocessor.lastChunkWritten=!0;const r=t.parser.tokenizer._consume();t.parser.tokenizer._callState(r)}}function QPe(e,t){const r=e;if(t.options.passThrough&&t.options.passThrough.includes(r.type))XPe(r,t);else{let n="";throw WPe.has(r.type)&&(n=". It looks like you are using MDX nodes with `hast-util-raw` (or `rehype-raw`). If you use this because you are using remark or rehype plugins that inject `'html'` nodes, then please raise an issue with that plugin, as its a bad and slow idea. If you use this because you are using markdown syntax, then you have to configure this utility (or plugin) to pass through these nodes (see `passThrough` in docs), but you can also migrate to use the MDX syntax"),new Error("Cannot compile `"+r.type+"` node"+n)}}function Fp(e,t){nae(e,t);const r=e.parser.tokenizer.currentCharacterToken;r&&r.location&&(r.location.endLine=e.parser.tokenizer.preprocessor.line,r.location.endCol=e.parser.tokenizer.preprocessor.col+1,r.location.endOffset=e.parser.tokenizer.preprocessor.offset+1,e.parser.currentToken=r,e.parser._processToken(e.parser.currentToken)),e.parser.tokenizer.paused=!1,e.parser.tokenizer.inLoop=!1,e.parser.tokenizer.active=!1,e.parser.tokenizer.returnState=Dn.DATA,e.parser.tokenizer.charRefCode=-1,e.parser.tokenizer.consumedAfterSnapshot=-1,e.parser.tokenizer.currentLocation=null,e.parser.tokenizer.currentCharacterToken=null,e.parser.tokenizer.currentToken=null,e.parser.tokenizer.currentAttr={name:"",value:""}}function nae(e,t){if(t&&t.offset!==void 0){const r={startLine:t.line,startCol:t.column,startOffset:t.offset,endLine:-1,endCol:-1,endOffset:-1};e.parser.tokenizer.preprocessor.lineStartPos=-t.column+1,e.parser.tokenizer.preprocessor.droppedBufferSize=t.offset,e.parser.tokenizer.preprocessor.line=t.line,e.parser.tokenizer.currentLocation=r}}function eze(e,t){const r=e.tagName.toLowerCase();if(t.parser.tokenizer.state===Dn.PLAINTEXT)return;Fp(t,vs(e));const n=t.parser.openElements.current;let a="namespaceURI"in n?n.namespaceURI:xc.html;a===xc.html&&r==="svg"&&(a=xc.svg);const i=l7e({...e,children:[]},{space:a===xc.svg?"svg":"html"}),o={type:Er.START_TAG,tagName:r,tagID:Ap(r),selfClosing:!1,ackSelfClosing:!1,attrs:"attrs"in i?i.attrs:[],location:zg(e)};t.parser.currentToken=o,t.parser._processToken(t.parser.currentToken),t.parser.tokenizer.lastStartTagName=r}function tze(e,t){const r=e.tagName.toLowerCase();if(!t.parser.tokenizer.inForeignNode&&g7e.includes(r)||t.parser.tokenizer.state===Dn.PLAINTEXT)return;Fp(t,v2(e));const n={type:Er.END_TAG,tagName:r,tagID:Ap(r),selfClosing:!1,ackSelfClosing:!1,attrs:[],location:zg(e)};t.parser.currentToken=n,t.parser._processToken(t.parser.currentToken),r===t.parser.tokenizer.lastStartTagName&&(t.parser.tokenizer.state===Dn.RCDATA||t.parser.tokenizer.state===Dn.RAWTEXT||t.parser.tokenizer.state===Dn.SCRIPT_DATA)&&(t.parser.tokenizer.state=Dn.DATA)}function rze(e){const t=e.type==="root"?e.children[0]:e;return!!(t&&(t.type==="doctype"||t.type==="element"&&t.tagName.toLowerCase()==="html"))}function zg(e){const t=vs(e)||{line:void 0,column:void 0,offset:void 0},r=v2(e)||{line:void 0,column:void 0,offset:void 0};return{startLine:t.line,startCol:t.column,startOffset:t.offset,endLine:r.line,endCol:r.column,endOffset:r.offset}}function nze(e){return"children"in e?Yh({...e,children:[]}):Yh(e)}function Az(e){return function(t,r){return eae(t,{...e,file:r})}}function aze(){return{enter:{mathFlow:e,mathFlowFenceMeta:t,mathText:i},exit:{mathFlow:a,mathFlowFence:n,mathFlowFenceMeta:r,mathFlowValue:s,mathText:o,mathTextData:s}};function e(l){const u={type:"element",tagName:"code",properties:{className:["language-math","math-display"]},children:[]};this.enter({type:"math",meta:null,value:"",data:{hName:"pre",hChildren:[u]}},l)}function t(){this.buffer()}function r(){const l=this.resume(),u=this.stack[this.stack.length-1];u.type,u.meta=l}function n(){this.data.mathFlowInside||(this.buffer(),this.data.mathFlowInside=!0)}function a(l){const u=this.resume().replace(/^(\r?\n|\r)|(\r?\n|\r)$/g,""),d=this.stack[this.stack.length-1];d.type,this.exit(l),d.value=u;const p=d.data.hChildren[0];p.type,p.tagName,p.children.push({type:"text",value:u}),this.data.mathFlowInside=void 0}function i(l){this.enter({type:"inlineMath",value:"",data:{hName:"code",hProperties:{className:["language-math","math-inline"]},hChildren:[]}},l),this.buffer()}function o(l){const u=this.resume(),d=this.stack[this.stack.length-1];d.type,this.exit(l),d.value=u,d.data.hChildren.push({type:"text",value:u})}function s(l){this.config.enter.data.call(this,l),this.config.exit.data.call(this,l)}}function ize(e){let t=(e||{}).singleDollarTextMath;return t==null&&(t=!0),n.peek=a,{unsafe:[{character:"\r",inConstruct:"mathFlowMeta"},{character:`
`,inConstruct:"mathFlowMeta"},{character:"$",after:t?void 0:"\\$",inConstruct:"phrasing"},{character:"$",inConstruct:"mathFlowMeta"},{atBreak:!0,character:"$",after:"\\$"}],handlers:{math:r,inlineMath:n}};function r(i,o,s,l){const u=i.value||"",d=s.createTracker(l),p="$".repeat(Math.max(Yre(u,"$")+1,2)),f=s.enter("mathFlow");let m=d.move(p);if(i.meta){const b=s.enter("mathFlowMeta");m+=d.move(s.safe(i.meta,{after:`
`,before:m,encode:["$"],...d.current()})),b()}return m+=d.move(`
`),u&&(m+=d.move(u+`
`)),m+=d.move(p),f(),m}function n(i,o,s){let l=i.value||"",u=1;for(t||u++;new RegExp("(^|[^$])"+"\\$".repeat(u)+"([^$]|$)").test(l);)u++;const d="$".repeat(u);/[^ \r\n]/.test(l)&&(/^[ \r\n]/.test(l)&&/[ \r\n]$/.test(l)||/^\$|\$$/.test(l))&&(l=" "+l+" ");let p=-1;for(;++p<s.unsafe.length;){const f=s.unsafe[p];if(!f.atBreak)continue;const m=s.compilePattern(f);let b;for(;b=m.exec(l);){let y=b.index;l.codePointAt(y)===10&&l.codePointAt(y-1)===13&&y--,l=l.slice(0,y)+" "+l.slice(b.index+1)}}return d+l+d}function a(){return"$"}}const oze={tokenize:sze,concrete:!0,name:"mathFlow"},Fz={tokenize:lze,partial:!0};function sze(e,t,r){const n=this,a=n.events[n.events.length-1],i=a&&a[1].type==="linePrefix"?a[2].sliceSerialize(a[1],!0).length:0;let o=0;return s;function s(E){return e.enter("mathFlow"),e.enter("mathFlowFence"),e.enter("mathFlowFenceSequence"),l(E)}function l(E){return E===36?(e.consume(E),o++,l):o<2?r(E):(e.exit("mathFlowFenceSequence"),xr(e,u,"whitespace")(E))}function u(E){return E===null||Ot(E)?p(E):(e.enter("mathFlowFenceMeta"),e.enter("chunkString",{contentType:"string"}),d(E))}function d(E){return E===null||Ot(E)?(e.exit("chunkString"),e.exit("mathFlowFenceMeta"),p(E)):E===36?r(E):(e.consume(E),d)}function p(E){return e.exit("mathFlowFence"),n.interrupt?t(E):e.attempt(Fz,f,S)(E)}function f(E){return e.attempt({tokenize:w,partial:!0},S,m)(E)}function m(E){return(i?xr(e,b,"linePrefix",i+1):b)(E)}function b(E){return E===null?S(E):Ot(E)?e.attempt(Fz,f,S)(E):(e.enter("mathFlowValue"),y(E))}function y(E){return E===null||Ot(E)?(e.exit("mathFlowValue"),b(E)):(e.consume(E),y)}function S(E){return e.exit("mathFlow"),t(E)}function w(E,k,_){let R=0;return xr(E,C,"linePrefix",n.parser.constructs.disable.null.includes("codeIndented")?void 0:4);function C(I){return E.enter("mathFlowFence"),E.enter("mathFlowFenceSequence"),F(I)}function F(I){return I===36?(R++,E.consume(I),F):R<o?_(I):(E.exit("mathFlowFenceSequence"),xr(E,L,"whitespace")(I))}function L(I){return I===null||Ot(I)?(E.exit("mathFlowFence"),k(I)):_(I)}}}function lze(e,t,r){const n=this;return a;function a(o){return o===null?t(o):(e.enter("lineEnding"),e.consume(o),e.exit("lineEnding"),i)}function i(o){return n.parser.lazy[n.now().line]?r(o):t(o)}}function uze(e){let r=(e||{}).singleDollarTextMath;return r==null&&(r=!0),{tokenize:n,resolve:cze,previous:dze,name:"mathText"};function n(a,i,o){let s=0,l,u;return d;function d(y){return a.enter("mathText"),a.enter("mathTextSequence"),p(y)}function p(y){return y===36?(a.consume(y),s++,p):s<2&&!r?o(y):(a.exit("mathTextSequence"),f(y))}function f(y){return y===null?o(y):y===36?(u=a.enter("mathTextSequence"),l=0,b(y)):y===32?(a.enter("space"),a.consume(y),a.exit("space"),f):Ot(y)?(a.enter("lineEnding"),a.consume(y),a.exit("lineEnding"),f):(a.enter("mathTextData"),m(y))}function m(y){return y===null||y===32||y===36||Ot(y)?(a.exit("mathTextData"),f(y)):(a.consume(y),m)}function b(y){return y===36?(a.consume(y),l++,b):l===s?(a.exit("mathTextSequence"),a.exit("mathText"),i(y)):(u.type="mathTextData",m(y))}}}function cze(e){let t=e.length-4,r=3,n,a;if((e[r][1].type==="lineEnding"||e[r][1].type==="space")&&(e[t][1].type==="lineEnding"||e[t][1].type==="space")){for(n=r;++n<t;)if(e[n][1].type==="mathTextData"){e[t][1].type="mathTextPadding",e[r][1].type="mathTextPadding",r+=2,t-=2;break}}for(n=r-1,t++;++n<=t;)a===void 0?n!==t&&e[n][1].type!=="lineEnding"&&(a=n):(n===t||e[n][1].type==="lineEnding")&&(e[a][1].type="mathTextData",n!==a+2&&(e[a][1].end=e[n-1][1].end,e.splice(a+2,n-a-2),t-=n-a-2,n=a+2),a=void 0);return e}function dze(e){return e!==36||this.events[this.events.length-1][1].type==="characterEscape"}function hze(e){return{flow:{36:oze},text:{36:uze(e)}}}const pze={};function _z(e){const t=this,r=e||pze,n=t.data(),a=n.micromarkExtensions||(n.micromarkExtensions=[]),i=n.fromMarkdownExtensions||(n.fromMarkdownExtensions=[]),o=n.toMarkdownExtensions||(n.toMarkdownExtensions=[]);a.push(hze(r)),i.push(aze()),o.push(ize(r))}var Rz={name:"mermaid",version:"11.12.0",description:"Markdown-ish syntax for generating flowcharts, mindmaps, sequence diagrams, class diagrams, gantt charts, git graphs and more.",type:"module",module:"./dist/mermaid.core.mjs",types:"./dist/mermaid.d.ts",exports:{".":{types:"./dist/mermaid.d.ts",import:"./dist/mermaid.core.mjs",default:"./dist/mermaid.core.mjs"},"./*":"./*"},keywords:["diagram","markdown","flowchart","sequence diagram","gantt","class diagram","git graph","mindmap","packet diagram","c4 diagram","er diagram","pie chart","pie diagram","quadrant chart","requirement diagram","graph"],scripts:{clean:"rimraf dist",dev:"pnpm -w dev","docs:code":"typedoc src/defaultConfig.ts src/config.ts src/mermaid.ts && prettier --write ./src/docs/config/setup","docs:build":"rimraf ../../docs && pnpm docs:code && pnpm docs:spellcheck && tsx scripts/docs.cli.mts","docs:verify":"pnpm docs:code && pnpm docs:spellcheck && tsx scripts/docs.cli.mts --verify","docs:pre:vitepress":"pnpm --filter ./src/docs prefetch && rimraf src/vitepress && pnpm docs:code && tsx scripts/docs.cli.mts --vitepress && pnpm --filter ./src/vitepress install --no-frozen-lockfile --ignore-scripts","docs:build:vitepress":"pnpm docs:pre:vitepress && (cd src/vitepress && pnpm run build) && cpy --flat src/docs/landing/ ./src/vitepress/.vitepress/dist/landing","docs:dev":'pnpm docs:pre:vitepress && concurrently "pnpm --filter ./src/vitepress dev" "tsx scripts/docs.cli.mts --watch --vitepress"',"docs:dev:docker":'pnpm docs:pre:vitepress && concurrently "pnpm --filter ./src/vitepress dev:docker" "tsx scripts/docs.cli.mts --watch --vitepress"',"docs:serve":"pnpm docs:build:vitepress && vitepress serve src/vitepress","docs:spellcheck":'cspell "src/docs/**/*.md"',"docs:release-version":"tsx scripts/update-release-version.mts","docs:verify-version":"tsx scripts/update-release-version.mts --verify","types:build-config":"tsx scripts/create-types-from-json-schema.mts","types:verify-config":"tsx scripts/create-types-from-json-schema.mts --verify",checkCircle:"npx madge --circular ./src",prepublishOnly:"pnpm docs:verify-version"},repository:{type:"git",url:"https://github.com/mermaid-js/mermaid"},author:"Knut Sveidqvist",license:"MIT",standard:{ignore:["**/parser/*.js","dist/**/*.js","cypress/**/*.js"],globals:["page"]},dependencies:{"@braintree/sanitize-url":"^7.1.1","@iconify/utils":"^3.0.1","@mermaid-js/parser":"workspace:^","@types/d3":"^7.4.3",cytoscape:"^3.29.3","cytoscape-cose-bilkent":"^4.1.0","cytoscape-fcose":"^2.2.0",d3:"^7.9.0","d3-sankey":"^0.12.3","dagre-d3-es":"7.0.11",dayjs:"^1.11.18",dompurify:"^3.2.5",katex:"^0.16.22",khroma:"^2.1.0","lodash-es":"^4.17.21",marked:"^16.2.1",roughjs:"^4.6.6",stylis:"^4.3.6","ts-dedent":"^2.2.0",uuid:"^11.1.0"},devDependencies:{"@adobe/jsonschema2md":"^8.0.5","@iconify/types":"^2.0.0","@types/cytoscape":"^3.21.9","@types/cytoscape-fcose":"^2.2.4","@types/d3-sankey":"^0.12.4","@types/d3-scale":"^4.0.9","@types/d3-scale-chromatic":"^3.1.0","@types/d3-selection":"^3.0.11","@types/d3-shape":"^3.1.7","@types/jsdom":"^21.1.7","@types/katex":"^0.16.7","@types/lodash-es":"^4.17.12","@types/micromatch":"^4.0.9","@types/stylis":"^4.2.7","@types/uuid":"^10.0.0",ajv:"^8.17.1",canvas:"^3.1.2",chokidar:"3.6.0",concurrently:"^9.1.2","csstree-validator":"^4.0.1",globby:"^14.1.0",jison:"^0.4.18","js-base64":"^3.7.8",jsdom:"^26.1.0","json-schema-to-typescript":"^15.0.4",micromatch:"^4.0.8","path-browserify":"^1.0.1",prettier:"^3.5.3",remark:"^15.0.1","remark-frontmatter":"^5.0.0","remark-gfm":"^4.0.1",rimraf:"^6.0.1","start-server-and-test":"^2.0.13","type-fest":"^4.35.0",typedoc:"^0.28.12","typedoc-plugin-markdown":"^4.8.1",typescript:"~5.7.3","unist-util-flatmap":"^1.0.0","unist-util-visit":"^5.0.0",vitepress:"^1.6.4","vitepress-plugin-search":"1.0.4-alpha.22"},files:["dist/","README.md"],publishConfig:{access:"public"}},P1={exports:{}},fze=P1.exports,Dz;function mze(){return Dz||(Dz=1,(function(e,t){(function(r,n){e.exports=n()})(fze,(function(){var r=1e3,n=6e4,a=36e5,i="millisecond",o="second",s="minute",l="hour",u="day",d="week",p="month",f="quarter",m="year",b="date",y="Invalid Date",S=/^(\d{4})[-/]?(\d{1,2})?[-/]?(\d{0,2})[Tt\s]*(\d{1,2})?:?(\d{1,2})?:?(\d{1,2})?[.:]?(\d+)?$/,w=/\[([^\]]+)]|Y{1,4}|M{1,4}|D{1,2}|d{1,4}|H{1,2}|h{1,2}|a|A|m{1,2}|s{1,2}|Z{1,2}|SSS/g,E={name:"en",weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),ordinal:function($){var X=["th","st","nd","rd"],z=$%100;return"["+$+(X[(z-20)%10]||X[z]||X[0])+"]"}},k=function($,X,z){var H=String($);return!H||H.length>=X?$:""+Array(X+1-H.length).join(z)+$},_={s:k,z:function($){var X=-$.utcOffset(),z=Math.abs(X),H=Math.floor(z/60),B=z%60;return(X<=0?"+":"-")+k(H,2,"0")+":"+k(B,2,"0")},m:function $(X,z){if(X.date()<z.date())return-$(z,X);var H=12*(z.year()-X.year())+(z.month()-X.month()),B=X.clone().add(H,p),ee=z-B<0,J=X.clone().add(H+(ee?-1:1),p);return+(-(H+(z-B)/(ee?B-J:J-B))||0)},a:function($){return $<0?Math.ceil($)||0:Math.floor($)},p:function($){return{M:p,y:m,w:d,d:u,D:b,h:l,m:s,s:o,ms:i,Q:f}[$]||String($||"").toLowerCase().replace(/s$/,"")},u:function($){return $===void 0}},R="en",C={};C[R]=E;var F="$isDayjsObject",L=function($){return $ instanceof Y||!(!$||!$[F])},I=function $(X,z,H){var B;if(!X)return R;if(typeof X=="string"){var ee=X.toLowerCase();C[ee]&&(B=ee),z&&(C[ee]=z,B=ee);var J=X.split("-");if(!B&&J.length>1)return $(J[0])}else{var K=X.name;C[K]=X,B=K}return!H&&B&&(R=B),B||!H&&R},M=function($,X){if(L($))return $.clone();var z=typeof X=="object"?X:{};return z.date=$,z.args=arguments,new Y(z)},j=_;j.l=I,j.i=L,j.w=function($,X){return M($,{locale:X.$L,utc:X.$u,x:X.$x,$offset:X.$offset})};var Y=(function(){function $(z){this.$L=I(z.locale,null,!0),this.parse(z),this.$x=this.$x||z.x||{},this[F]=!0}var X=$.prototype;return X.parse=function(z){this.$d=(function(H){var B=H.date,ee=H.utc;if(B===null)return new Date(NaN);if(j.u(B))return new Date;if(B instanceof Date)return new Date(B);if(typeof B=="string"&&!/Z$/i.test(B)){var J=B.match(S);if(J){var K=J[2]-1||0,G=(J[7]||"0").substring(0,3);return ee?new Date(Date.UTC(J[1],K,J[3]||1,J[4]||0,J[5]||0,J[6]||0,G)):new Date(J[1],K,J[3]||1,J[4]||0,J[5]||0,J[6]||0,G)}}return new Date(B)})(z),this.init()},X.init=function(){var z=this.$d;this.$y=z.getFullYear(),this.$M=z.getMonth(),this.$D=z.getDate(),this.$W=z.getDay(),this.$H=z.getHours(),this.$m=z.getMinutes(),this.$s=z.getSeconds(),this.$ms=z.getMilliseconds()},X.$utils=function(){return j},X.isValid=function(){return this.$d.toString()!==y},X.isSame=function(z,H){var B=M(z);return this.startOf(H)<=B&&B<=this.endOf(H)},X.isAfter=function(z,H){return M(z)<this.startOf(H)},X.isBefore=function(z,H){return this.endOf(H)<M(z)},X.$g=function(z,H,B){return j.u(z)?this[H]:this.set(B,z)},X.unix=function(){return Math.floor(this.valueOf()/1e3)},X.valueOf=function(){return this.$d.getTime()},X.startOf=function(z,H){var B=this,ee=!!j.u(H)||H,J=j.p(z),K=function(Ee,fe){var Me=j.w(B.$u?Date.UTC(B.$y,fe,Ee):new Date(B.$y,fe,Ee),B);return ee?Me:Me.endOf(u)},G=function(Ee,fe){return j.w(B.toDate()[Ee].apply(B.toDate("s"),(ee?[0,0,0,0]:[23,59,59,999]).slice(fe)),B)},P=this.$W,ne=this.$M,oe=this.$D,pe="set"+(this.$u?"UTC":"");switch(J){case m:return ee?K(1,0):K(31,11);case p:return ee?K(1,ne):K(0,ne+1);case d:var ae=this.$locale().weekStart||0,se=(P<ae?P+7:P)-ae;return K(ee?oe-se:oe+(6-se),ne);case u:case b:return G(pe+"Hours",0);case l:return G(pe+"Minutes",1);case s:return G(pe+"Seconds",2);case o:return G(pe+"Milliseconds",3);default:return this.clone()}},X.endOf=function(z){return this.startOf(z,!1)},X.$set=function(z,H){var B,ee=j.p(z),J="set"+(this.$u?"UTC":""),K=(B={},B[u]=J+"Date",B[b]=J+"Date",B[p]=J+"Month",B[m]=J+"FullYear",B[l]=J+"Hours",B[s]=J+"Minutes",B[o]=J+"Seconds",B[i]=J+"Milliseconds",B)[ee],G=ee===u?this.$D+(H-this.$W):H;if(ee===p||ee===m){var P=this.clone().set(b,1);P.$d[K](G),P.init(),this.$d=P.set(b,Math.min(this.$D,P.daysInMonth())).$d}else K&&this.$d[K](G);return this.init(),this},X.set=function(z,H){return this.clone().$set(z,H)},X.get=function(z){return this[j.p(z)]()},X.add=function(z,H){var B,ee=this;z=Number(z);var J=j.p(H),K=function(ne){var oe=M(ee);return j.w(oe.date(oe.date()+Math.round(ne*z)),ee)};if(J===p)return this.set(p,this.$M+z);if(J===m)return this.set(m,this.$y+z);if(J===u)return K(1);if(J===d)return K(7);var G=(B={},B[s]=n,B[l]=a,B[o]=r,B)[J]||1,P=this.$d.getTime()+z*G;return j.w(P,this)},X.subtract=function(z,H){return this.add(-1*z,H)},X.format=function(z){var H=this,B=this.$locale();if(!this.isValid())return B.invalidDate||y;var ee=z||"YYYY-MM-DDTHH:mm:ssZ",J=j.z(this),K=this.$H,G=this.$m,P=this.$M,ne=B.weekdays,oe=B.months,pe=B.meridiem,ae=function(fe,Me,Oe,xe){return fe&&(fe[Me]||fe(H,ee))||Oe[Me].slice(0,xe)},se=function(fe){return j.s(K%12||12,fe,"0")},Ee=pe||function(fe,Me,Oe){var xe=fe<12?"AM":"PM";return Oe?xe.toLowerCase():xe};return ee.replace(w,(function(fe,Me){return Me||(function(Oe){switch(Oe){case"YY":return String(H.$y).slice(-2);case"YYYY":return j.s(H.$y,4,"0");case"M":return P+1;case"MM":return j.s(P+1,2,"0");case"MMM":return ae(B.monthsShort,P,oe,3);case"MMMM":return ae(oe,P);case"D":return H.$D;case"DD":return j.s(H.$D,2,"0");case"d":return String(H.$W);case"dd":return ae(B.weekdaysMin,H.$W,ne,2);case"ddd":return ae(B.weekdaysShort,H.$W,ne,3);case"dddd":return ne[H.$W];case"H":return String(K);case"HH":return j.s(K,2,"0");case"h":return se(1);case"hh":return se(2);case"a":return Ee(K,G,!0);case"A":return Ee(K,G,!1);case"m":return String(G);case"mm":return j.s(G,2,"0");case"s":return String(H.$s);case"ss":return j.s(H.$s,2,"0");case"SSS":return j.s(H.$ms,3,"0");case"Z":return J}return null})(fe)||J.replace(":","")}))},X.utcOffset=function(){return 15*-Math.round(this.$d.getTimezoneOffset()/15)},X.diff=function(z,H,B){var ee,J=this,K=j.p(H),G=M(z),P=(G.utcOffset()-this.utcOffset())*n,ne=this-G,oe=function(){return j.m(J,G)};switch(K){case m:ee=oe()/12;break;case p:ee=oe();break;case f:ee=oe()/3;break;case d:ee=(ne-P)/6048e5;break;case u:ee=(ne-P)/864e5;break;case l:ee=ne/a;break;case s:ee=ne/n;break;case o:ee=ne/r;break;default:ee=ne}return B?ee:j.a(ee)},X.daysInMonth=function(){return this.endOf(p).$D},X.$locale=function(){return C[this.$L]},X.locale=function(z,H){if(!z)return this.$L;var B=this.clone(),ee=I(z,H,!0);return ee&&(B.$L=ee),B},X.clone=function(){return j.w(this.$d,this)},X.toDate=function(){return new Date(this.valueOf())},X.toJSON=function(){return this.isValid()?this.toISOString():null},X.toISOString=function(){return this.$d.toISOString()},X.toString=function(){return this.$d.toUTCString()},$})(),W=Y.prototype;return M.prototype=W,[["$ms",i],["$s",o],["$m",s],["$H",l],["$W",u],["$M",p],["$y",m],["$D",b]].forEach((function($){W[$[1]]=function(X){return this.$g(X,$[0],$[1])}})),M.extend=function($,X){return $.$i||($(X,Y,M),$.$i=!0),M},M.locale=I,M.isDayjs=L,M.unix=function($){return M(1e3*$)},M.en=C[R],M.Ls=C,M.p={},M}))})(P1)),P1.exports}var gze=mze();const bze=jn(gze);var aae=Object.defineProperty,U=(e,t)=>aae(e,"name",{value:t,configurable:!0}),yze=(e,t)=>{for(var r in t)aae(e,r,{get:t[r],enumerable:!0})},Ks={trace:0,debug:1,info:2,warn:3,error:4,fatal:5},je={trace:U((...e)=>{},"trace"),debug:U((...e)=>{},"debug"),info:U((...e)=>{},"info"),warn:U((...e)=>{},"warn"),error:U((...e)=>{},"error"),fatal:U((...e)=>{},"fatal")},lN=U(function(e="fatal"){let t=Ks.fatal;typeof e=="string"?e.toLowerCase()in Ks&&(t=Ks[e]):typeof e=="number"&&(t=e),je.trace=()=>{},je.debug=()=>{},je.info=()=>{},je.warn=()=>{},je.error=()=>{},je.fatal=()=>{},t<=Ks.fatal&&(je.fatal=console.error?console.error.bind(console,eo("FATAL"),"color: orange"):console.log.bind(console,"\x1B[35m",eo("FATAL"))),t<=Ks.error&&(je.error=console.error?console.error.bind(console,eo("ERROR"),"color: orange"):console.log.bind(console,"\x1B[31m",eo("ERROR"))),t<=Ks.warn&&(je.warn=console.warn?console.warn.bind(console,eo("WARN"),"color: orange"):console.log.bind(console,"\x1B[33m",eo("WARN"))),t<=Ks.info&&(je.info=console.info?console.info.bind(console,eo("INFO"),"color: lightblue"):console.log.bind(console,"\x1B[34m",eo("INFO"))),t<=Ks.debug&&(je.debug=console.debug?console.debug.bind(console,eo("DEBUG"),"color: lightgreen"):console.log.bind(console,"\x1B[32m",eo("DEBUG"))),t<=Ks.trace&&(je.trace=console.debug?console.debug.bind(console,eo("TRACE"),"color: lightgreen"):console.log.bind(console,"\x1B[32m",eo("TRACE")))},"setLogLevel"),eo=U(e=>`%c${bze().format("ss.SSS")} : ${e} : `,"format");const z1={min:{r:0,g:0,b:0,s:0,l:0,a:0},max:{r:255,g:255,b:255,h:360,s:100,l:100,a:1},clamp:{r:e=>e>=255?255:e<0?0:e,g:e=>e>=255?255:e<0?0:e,b:e=>e>=255?255:e<0?0:e,h:e=>e%360,s:e=>e>=100?100:e<0?0:e,l:e=>e>=100?100:e<0?0:e,a:e=>e>=1?1:e<0?0:e},toLinear:e=>{const t=e/255;return e>.03928?Math.pow((t+.055)/1.055,2.4):t/12.92},hue2rgb:(e,t,r)=>(r<0&&(r+=1),r>1&&(r-=1),r<1/6?e+(t-e)*6*r:r<1/2?t:r<2/3?e+(t-e)*(2/3-r)*6:e),hsl2rgb:({h:e,s:t,l:r},n)=>{if(!t)return r*2.55;e/=360,t/=100,r/=100;const a=r<.5?r*(1+t):r+t-r*t,i=2*r-a;switch(n){case"r":return z1.hue2rgb(i,a,e+1/3)*255;case"g":return z1.hue2rgb(i,a,e)*255;case"b":return z1.hue2rgb(i,a,e-1/3)*255}},rgb2hsl:({r:e,g:t,b:r},n)=>{e/=255,t/=255,r/=255;const a=Math.max(e,t,r),i=Math.min(e,t,r),o=(a+i)/2;if(n==="l")return o*100;if(a===i)return 0;const s=a-i,l=o>.5?s/(2-a-i):s/(a+i);if(n==="s")return l*100;switch(a){case e:return((t-r)/s+(t<r?6:0))*60;case t:return((r-e)/s+2)*60;case r:return((e-t)/s+4)*60;default:return-1}}},vze={clamp:(e,t,r)=>t>r?Math.min(t,Math.max(r,e)):Math.min(r,Math.max(t,e)),round:e=>Math.round(e*1e10)/1e10},Sze={dec2hex:e=>{const t=Math.round(e).toString(16);return t.length>1?t:`0${t}`}},ir={channel:z1,lang:vze,unit:Sze},fu={};for(let e=0;e<=255;e++)fu[e]=ir.unit.dec2hex(e);const Aa={ALL:0,RGB:1,HSL:2};class Eze{constructor(){this.type=Aa.ALL}get(){return this.type}set(t){if(this.type&&this.type!==t)throw new Error("Cannot change both RGB and HSL channels at the same time");this.type=t}reset(){this.type=Aa.ALL}is(t){return this.type===t}}class xze{constructor(t,r){this.color=r,this.changed=!1,this.data=t,this.type=new Eze}set(t,r){return this.color=r,this.changed=!1,this.data=t,this.type.type=Aa.ALL,this}_ensureHSL(){const t=this.data,{h:r,s:n,l:a}=t;r===void 0&&(t.h=ir.channel.rgb2hsl(t,"h")),n===void 0&&(t.s=ir.channel.rgb2hsl(t,"s")),a===void 0&&(t.l=ir.channel.rgb2hsl(t,"l"))}_ensureRGB(){const t=this.data,{r,g:n,b:a}=t;r===void 0&&(t.r=ir.channel.hsl2rgb(t,"r")),n===void 0&&(t.g=ir.channel.hsl2rgb(t,"g")),a===void 0&&(t.b=ir.channel.hsl2rgb(t,"b"))}get r(){const t=this.data,r=t.r;return!this.type.is(Aa.HSL)&&r!==void 0?r:(this._ensureHSL(),ir.channel.hsl2rgb(t,"r"))}get g(){const t=this.data,r=t.g;return!this.type.is(Aa.HSL)&&r!==void 0?r:(this._ensureHSL(),ir.channel.hsl2rgb(t,"g"))}get b(){const t=this.data,r=t.b;return!this.type.is(Aa.HSL)&&r!==void 0?r:(this._ensureHSL(),ir.channel.hsl2rgb(t,"b"))}get h(){const t=this.data,r=t.h;return!this.type.is(Aa.RGB)&&r!==void 0?r:(this._ensureRGB(),ir.channel.rgb2hsl(t,"h"))}get s(){const t=this.data,r=t.s;return!this.type.is(Aa.RGB)&&r!==void 0?r:(this._ensureRGB(),ir.channel.rgb2hsl(t,"s"))}get l(){const t=this.data,r=t.l;return!this.type.is(Aa.RGB)&&r!==void 0?r:(this._ensureRGB(),ir.channel.rgb2hsl(t,"l"))}get a(){return this.data.a}set r(t){this.type.set(Aa.RGB),this.changed=!0,this.data.r=t}set g(t){this.type.set(Aa.RGB),this.changed=!0,this.data.g=t}set b(t){this.type.set(Aa.RGB),this.changed=!0,this.data.b=t}set h(t){this.type.set(Aa.HSL),this.changed=!0,this.data.h=t}set s(t){this.type.set(Aa.HSL),this.changed=!0,this.data.s=t}set l(t){this.type.set(Aa.HSL),this.changed=!0,this.data.l=t}set a(t){this.changed=!0,this.data.a=t}}const _2=new xze({r:0,g:0,b:0,a:0},"transparent"),Ch={re:/^#((?:[a-f0-9]{2}){2,4}|[a-f0-9]{3})$/i,parse:e=>{if(e.charCodeAt(0)!==35)return;const t=e.match(Ch.re);if(!t)return;const r=t[1],n=parseInt(r,16),a=r.length,i=a%4===0,o=a>4,s=o?1:17,l=o?8:4,u=i?0:-1,d=o?255:15;return _2.set({r:(n>>l*(u+3)&d)*s,g:(n>>l*(u+2)&d)*s,b:(n>>l*(u+1)&d)*s,a:i?(n&d)*s/255:1},e)},stringify:e=>{const{r:t,g:r,b:n,a}=e;return a<1?`#${fu[Math.round(t)]}${fu[Math.round(r)]}${fu[Math.round(n)]}${fu[Math.round(a*255)]}`:`#${fu[Math.round(t)]}${fu[Math.round(r)]}${fu[Math.round(n)]}`}},wc={re:/^hsla?\(\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?(?:deg|grad|rad|turn)?)\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?%)\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?%)(?:\s*?(?:,|\/)\s*?\+?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e-?\d+)?(%)?))?\s*?\)$/i,hueRe:/^(.+?)(deg|grad|rad|turn)$/i,_hue2deg:e=>{const t=e.match(wc.hueRe);if(t){const[,r,n]=t;switch(n){case"grad":return ir.channel.clamp.h(parseFloat(r)*.9);case"rad":return ir.channel.clamp.h(parseFloat(r)*180/Math.PI);case"turn":return ir.channel.clamp.h(parseFloat(r)*360)}}return ir.channel.clamp.h(parseFloat(e))},parse:e=>{const t=e.charCodeAt(0);if(t!==104&&t!==72)return;const r=e.match(wc.re);if(!r)return;const[,n,a,i,o,s]=r;return _2.set({h:wc._hue2deg(n),s:ir.channel.clamp.s(parseFloat(a)),l:ir.channel.clamp.l(parseFloat(i)),a:o?ir.channel.clamp.a(s?parseFloat(o)/100:parseFloat(o)):1},e)},stringify:e=>{const{h:t,s:r,l:n,a}=e;return a<1?`hsla(${ir.lang.round(t)}, ${ir.lang.round(r)}%, ${ir.lang.round(n)}%, ${a})`:`hsl(${ir.lang.round(t)}, ${ir.lang.round(r)}%, ${ir.lang.round(n)}%)`}},Am={colors:{aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyanaqua:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgreen:"#006400",darkgrey:"#a9a9a9",darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkslategrey:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",gray:"#808080",green:"#008000",greenyellow:"#adff2f",grey:"#808080",honeydew:"#f0fff0",hotpink:"#ff69b4",indianred:"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrodyellow:"#fafad2",lightgray:"#d3d3d3",lightgreen:"#90ee90",lightgrey:"#d3d3d3",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#b0c4de",lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370db",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#db7093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",rebeccapurple:"#663399",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",slategray:"#708090",slategrey:"#708090",snow:"#fffafa",springgreen:"#00ff7f",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",transparent:"#00000000",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"},parse:e=>{e=e.toLowerCase();const t=Am.colors[e];if(t)return Ch.parse(t)},stringify:e=>{const t=Ch.stringify(e);for(const r in Am.colors)if(Am.colors[r]===t)return r}},sm={re:/^rgba?\(\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?))\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?))\s*?(?:,|\s)\s*?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?))(?:\s*?(?:,|\/)\s*?\+?(-?(?:\d+(?:\.\d+)?|(?:\.\d+))(?:e\d+)?(%?)))?\s*?\)$/i,parse:e=>{const t=e.charCodeAt(0);if(t!==114&&t!==82)return;const r=e.match(sm.re);if(!r)return;const[,n,a,i,o,s,l,u,d]=r;return _2.set({r:ir.channel.clamp.r(a?parseFloat(n)*2.55:parseFloat(n)),g:ir.channel.clamp.g(o?parseFloat(i)*2.55:parseFloat(i)),b:ir.channel.clamp.b(l?parseFloat(s)*2.55:parseFloat(s)),a:u?ir.channel.clamp.a(d?parseFloat(u)/100:parseFloat(u)):1},e)},stringify:e=>{const{r:t,g:r,b:n,a}=e;return a<1?`rgba(${ir.lang.round(t)}, ${ir.lang.round(r)}, ${ir.lang.round(n)}, ${ir.lang.round(a)})`:`rgb(${ir.lang.round(t)}, ${ir.lang.round(r)}, ${ir.lang.round(n)})`}},cs={format:{keyword:Am,hex:Ch,rgb:sm,rgba:sm,hsl:wc,hsla:wc},parse:e=>{if(typeof e!="string")return e;const t=Ch.parse(e)||sm.parse(e)||wc.parse(e)||Am.parse(e);if(t)return t;throw new Error(`Unsupported color format: "${e}"`)},stringify:e=>!e.changed&&e.color?e.color:e.type.is(Aa.HSL)||e.data.r===void 0?wc.stringify(e):e.a<1||!Number.isInteger(e.r)||!Number.isInteger(e.g)||!Number.isInteger(e.b)?sm.stringify(e):Ch.stringify(e)},iae=(e,t)=>{const r=cs.parse(e);for(const n in t)r[n]=ir.channel.clamp[n](t[n]);return cs.stringify(r)},Fm=(e,t,r=0,n=1)=>{if(typeof e!="number")return iae(e,{a:t});const a=_2.set({r:ir.channel.clamp.r(e),g:ir.channel.clamp.g(t),b:ir.channel.clamp.b(r),a:ir.channel.clamp.a(n)});return cs.stringify(a)},wze=e=>{const{r:t,g:r,b:n}=cs.parse(e),a=.2126*ir.channel.toLinear(t)+.7152*ir.channel.toLinear(r)+.0722*ir.channel.toLinear(n);return ir.lang.round(a)},kze=e=>wze(e)>=.5,Hg=e=>!kze(e),oae=(e,t,r)=>{const n=cs.parse(e),a=n[t],i=ir.channel.clamp[t](a+r);return a!==i&&(n[t]=i),cs.stringify(n)},Ft=(e,t)=>oae(e,"l",t),Ut=(e,t)=>oae(e,"l",-t),Se=(e,t)=>{const r=cs.parse(e),n={};for(const a in t)t[a]&&(n[a]=r[a]+t[a]);return iae(e,n)},Cze=(e,t,r=50)=>{const{r:n,g:a,b:i,a:o}=cs.parse(e),{r:s,g:l,b:u,a:d}=cs.parse(t),p=r/100,f=p*2-1,m=o-d,y=((f*m===-1?f:(f+m)/(1+f*m))+1)/2,S=1-y,w=n*y+s*S,E=a*y+l*S,k=i*y+u*S,_=o*p+d*(1-p);return Fm(w,E,k,_)},dt=(e,t=100)=>{const r=cs.parse(e);return r.r=255-r.r,r.g=255-r.g,r.b=255-r.b,Cze(r,e,t)};/*! @license DOMPurify 3.2.5 | (c) Cure53 and other contributors | Released under the Apache license 2.0 and Mozilla Public License 2.0 | github.com/cure53/DOMPurify/blob/3.2.5/LICENSE */const{entries:sae,setPrototypeOf:Nz,isFrozen:Tze,getPrototypeOf:Aze,getOwnPropertyDescriptor:Fze}=Object;let{freeze:Ya,seal:lo,create:lae}=Object,{apply:QR,construct:e6}=typeof Reflect<"u"&&Reflect;Ya||(Ya=function(t){return t});lo||(lo=function(t){return t});QR||(QR=function(t,r,n){return t.apply(r,n)});e6||(e6=function(t,r){return new t(...r)});const e1=Ja(Array.prototype.forEach),_ze=Ja(Array.prototype.lastIndexOf),Lz=Ja(Array.prototype.pop),Gf=Ja(Array.prototype.push),Rze=Ja(Array.prototype.splice),H1=Ja(String.prototype.toLowerCase),Ck=Ja(String.prototype.toString),Mz=Ja(String.prototype.match),jf=Ja(String.prototype.replace),Dze=Ja(String.prototype.indexOf),Nze=Ja(String.prototype.trim),wo=Ja(Object.prototype.hasOwnProperty),$a=Ja(RegExp.prototype.test),qf=Lze(TypeError);function Ja(e){return function(t){t instanceof RegExp&&(t.lastIndex=0);for(var r=arguments.length,n=new Array(r>1?r-1:0),a=1;a<r;a++)n[a-1]=arguments[a];return QR(e,t,n)}}function Lze(e){return function(){for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return e6(e,r)}}function Cr(e,t){let r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:H1;Nz&&Nz(e,null);let n=t.length;for(;n--;){let a=t[n];if(typeof a=="string"){const i=r(a);i!==a&&(Tze(t)||(t[n]=i),a=i)}e[a]=!0}return e}function Mze(e){for(let t=0;t<e.length;t++)wo(e,t)||(e[t]=null);return e}function gc(e){const t=lae(null);for(const[r,n]of sae(e))wo(e,r)&&(Array.isArray(n)?t[r]=Mze(n):n&&typeof n=="object"&&n.constructor===Object?t[r]=gc(n):t[r]=n);return t}function Wf(e,t){for(;e!==null;){const n=Fze(e,t);if(n){if(n.get)return Ja(n.get);if(typeof n.value=="function")return Ja(n.value)}e=Aze(e)}function r(){return null}return r}const Iz=Ya(["a","abbr","acronym","address","area","article","aside","audio","b","bdi","bdo","big","blink","blockquote","body","br","button","canvas","caption","center","cite","code","col","colgroup","content","data","datalist","dd","decorator","del","details","dfn","dialog","dir","div","dl","dt","element","em","fieldset","figcaption","figure","font","footer","form","h1","h2","h3","h4","h5","h6","head","header","hgroup","hr","html","i","img","input","ins","kbd","label","legend","li","main","map","mark","marquee","menu","menuitem","meter","nav","nobr","ol","optgroup","option","output","p","picture","pre","progress","q","rp","rt","ruby","s","samp","section","select","shadow","small","source","spacer","span","strike","strong","style","sub","summary","sup","table","tbody","td","template","textarea","tfoot","th","thead","time","tr","track","tt","u","ul","var","video","wbr"]),Tk=Ya(["svg","a","altglyph","altglyphdef","altglyphitem","animatecolor","animatemotion","animatetransform","circle","clippath","defs","desc","ellipse","filter","font","g","glyph","glyphref","hkern","image","line","lineargradient","marker","mask","metadata","mpath","path","pattern","polygon","polyline","radialgradient","rect","stop","style","switch","symbol","text","textpath","title","tref","tspan","view","vkern"]),Ak=Ya(["feBlend","feColorMatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feDropShadow","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence"]),Ize=Ya(["animate","color-profile","cursor","discard","font-face","font-face-format","font-face-name","font-face-src","font-face-uri","foreignobject","hatch","hatchpath","mesh","meshgradient","meshpatch","meshrow","missing-glyph","script","set","solidcolor","unknown","use"]),Fk=Ya(["math","menclose","merror","mfenced","mfrac","mglyph","mi","mlabeledtr","mmultiscripts","mn","mo","mover","mpadded","mphantom","mroot","mrow","ms","mspace","msqrt","mstyle","msub","msup","msubsup","mtable","mtd","mtext","mtr","munder","munderover","mprescripts"]),Oze=Ya(["maction","maligngroup","malignmark","mlongdiv","mscarries","mscarry","msgroup","mstack","msline","msrow","semantics","annotation","annotation-xml","mprescripts","none"]),Oz=Ya(["#text"]),Bz=Ya(["accept","action","align","alt","autocapitalize","autocomplete","autopictureinpicture","autoplay","background","bgcolor","border","capture","cellpadding","cellspacing","checked","cite","class","clear","color","cols","colspan","controls","controlslist","coords","crossorigin","datetime","decoding","default","dir","disabled","disablepictureinpicture","disableremoteplayback","download","draggable","enctype","enterkeyhint","face","for","headers","height","hidden","high","href","hreflang","id","inputmode","integrity","ismap","kind","label","lang","list","loading","loop","low","max","maxlength","media","method","min","minlength","multiple","muted","name","nonce","noshade","novalidate","nowrap","open","optimum","pattern","placeholder","playsinline","popover","popovertarget","popovertargetaction","poster","preload","pubdate","radiogroup","readonly","rel","required","rev","reversed","role","rows","rowspan","spellcheck","scope","selected","shape","size","sizes","span","srclang","start","src","srcset","step","style","summary","tabindex","title","translate","type","usemap","valign","value","width","wrap","xmlns","slot"]),_k=Ya(["accent-height","accumulate","additive","alignment-baseline","amplitude","ascent","attributename","attributetype","azimuth","basefrequency","baseline-shift","begin","bias","by","class","clip","clippathunits","clip-path","clip-rule","color","color-interpolation","color-interpolation-filters","color-profile","color-rendering","cx","cy","d","dx","dy","diffuseconstant","direction","display","divisor","dur","edgemode","elevation","end","exponent","fill","fill-opacity","fill-rule","filter","filterunits","flood-color","flood-opacity","font-family","font-size","font-size-adjust","font-stretch","font-style","font-variant","font-weight","fx","fy","g1","g2","glyph-name","glyphref","gradientunits","gradienttransform","height","href","id","image-rendering","in","in2","intercept","k","k1","k2","k3","k4","kerning","keypoints","keysplines","keytimes","lang","lengthadjust","letter-spacing","kernelmatrix","kernelunitlength","lighting-color","local","marker-end","marker-mid","marker-start","markerheight","markerunits","markerwidth","maskcontentunits","maskunits","max","mask","media","method","mode","min","name","numoctaves","offset","operator","opacity","order","orient","orientation","origin","overflow","paint-order","path","pathlength","patterncontentunits","patterntransform","patternunits","points","preservealpha","preserveaspectratio","primitiveunits","r","rx","ry","radius","refx","refy","repeatcount","repeatdur","restart","result","rotate","scale","seed","shape-rendering","slope","specularconstant","specularexponent","spreadmethod","startoffset","stddeviation","stitchtiles","stop-color","stop-opacity","stroke-dasharray","stroke-dashoffset","stroke-linecap","stroke-linejoin","stroke-miterlimit","stroke-opacity","stroke","stroke-width","style","surfacescale","systemlanguage","tabindex","tablevalues","targetx","targety","transform","transform-origin","text-anchor","text-decoration","text-rendering","textlength","type","u1","u2","unicode","values","viewbox","visibility","version","vert-adv-y","vert-origin-x","vert-origin-y","width","word-spacing","wrap","writing-mode","xchannelselector","ychannelselector","x","x1","x2","xmlns","y","y1","y2","z","zoomandpan"]),Pz=Ya(["accent","accentunder","align","bevelled","close","columnsalign","columnlines","columnspan","denomalign","depth","dir","display","displaystyle","encoding","fence","frame","height","href","id","largeop","length","linethickness","lspace","lquote","mathbackground","mathcolor","mathsize","mathvariant","maxsize","minsize","movablelimits","notation","numalign","open","rowalign","rowlines","rowspacing","rowspan","rspace","rquote","scriptlevel","scriptminsize","scriptsizemultiplier","selection","separator","separators","stretchy","subscriptshift","supscriptshift","symmetric","voffset","width","xmlns"]),t1=Ya(["xlink:href","xml:id","xlink:title","xml:space","xmlns:xlink"]),Bze=lo(/\{\{[\w\W]*|[\w\W]*\}\}/gm),Pze=lo(/<%[\w\W]*|[\w\W]*%>/gm),zze=lo(/\$\{[\w\W]*/gm),Hze=lo(/^data-[\-\w.\u00B7-\uFFFF]+$/),Uze=lo(/^aria-[\-\w]+$/),uae=lo(/^(?:(?:(?:f|ht)tps?|mailto|tel|callto|sms|cid|xmpp):|[^a-z]|[a-z+.\-]+(?:[^a-z+.\-:]|$))/i),$ze=lo(/^(?:\w+script|data):/i),Gze=lo(/[\u0000-\u0020\u00A0\u1680\u180E\u2000-\u2029\u205F\u3000]/g),cae=lo(/^html$/i),jze=lo(/^[a-z][.\w]*(-[.\w]+)+$/i);var zz=Object.freeze({__proto__:null,ARIA_ATTR:Uze,ATTR_WHITESPACE:Gze,CUSTOM_ELEMENT:jze,DATA_ATTR:Hze,DOCTYPE_NAME:cae,ERB_EXPR:Pze,IS_ALLOWED_URI:uae,IS_SCRIPT_OR_DATA:$ze,MUSTACHE_EXPR:Bze,TMPLIT_EXPR:zze});const Vf={element:1,text:3,progressingInstruction:7,comment:8,document:9},qze=function(){return typeof window>"u"?null:window},Wze=function(t,r){if(typeof t!="object"||typeof t.createPolicy!="function")return null;let n=null;const a="data-tt-policy-suffix";r&&r.hasAttribute(a)&&(n=r.getAttribute(a));const i="dompurify"+(n?"#"+n:"");try{return t.createPolicy(i,{createHTML(o){return o},createScriptURL(o){return o}})}catch{return console.warn("TrustedTypes policy "+i+" could not be created."),null}},Hz=function(){return{afterSanitizeAttributes:[],afterSanitizeElements:[],afterSanitizeShadowDOM:[],beforeSanitizeAttributes:[],beforeSanitizeElements:[],beforeSanitizeShadowDOM:[],uponSanitizeAttribute:[],uponSanitizeElement:[],uponSanitizeShadowNode:[]}};function dae(){let e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:qze();const t=Bt=>dae(Bt);if(t.version="3.2.5",t.removed=[],!e||!e.document||e.document.nodeType!==Vf.document||!e.Element)return t.isSupported=!1,t;let{document:r}=e;const n=r,a=n.currentScript,{DocumentFragment:i,HTMLTemplateElement:o,Node:s,Element:l,NodeFilter:u,NamedNodeMap:d=e.NamedNodeMap||e.MozNamedAttrMap,HTMLFormElement:p,DOMParser:f,trustedTypes:m}=e,b=l.prototype,y=Wf(b,"cloneNode"),S=Wf(b,"remove"),w=Wf(b,"nextSibling"),E=Wf(b,"childNodes"),k=Wf(b,"parentNode");if(typeof o=="function"){const Bt=r.createElement("template");Bt.content&&Bt.content.ownerDocument&&(r=Bt.content.ownerDocument)}let _,R="";const{implementation:C,createNodeIterator:F,createDocumentFragment:L,getElementsByTagName:I}=r,{importNode:M}=n;let j=Hz();t.isSupported=typeof sae=="function"&&typeof k=="function"&&C&&C.createHTMLDocument!==void 0;const{MUSTACHE_EXPR:Y,ERB_EXPR:W,TMPLIT_EXPR:$,DATA_ATTR:X,ARIA_ATTR:z,IS_SCRIPT_OR_DATA:H,ATTR_WHITESPACE:B,CUSTOM_ELEMENT:ee}=zz;let{IS_ALLOWED_URI:J}=zz,K=null;const G=Cr({},[...Iz,...Tk,...Ak,...Fk,...Oz]);let P=null;const ne=Cr({},[...Bz,..._k,...Pz,...t1]);let oe=Object.seal(lae(null,{tagNameCheck:{writable:!0,configurable:!1,enumerable:!0,value:null},attributeNameCheck:{writable:!0,configurable:!1,enumerable:!0,value:null},allowCustomizedBuiltInElements:{writable:!0,configurable:!1,enumerable:!0,value:!1}})),pe=null,ae=null,se=!0,Ee=!0,fe=!1,Me=!0,Oe=!1,xe=!0,Te=!1,Ne=!1,he=!1,Ze=!1,Et=!1,qe=!1,Ye=!0,we=!1;const Qt="user-content-";let or=!0,Pe=!1,rt={},ge=null;const Re=Cr({},["annotation-xml","audio","colgroup","desc","foreignobject","head","iframe","math","mi","mn","mo","ms","mtext","noembed","noframes","noscript","plaintext","script","style","svg","template","thead","title","video","xmp"]);let We=null;const ht=Cr({},["audio","video","img","source","image","track"]);let xt=null;const sr=Cr({},["alt","class","for","id","label","name","pattern","placeholder","role","summary","title","value","style","xmlns"]),ar="http://www.w3.org/1998/Math/MathML",fr="http://www.w3.org/2000/svg",Mt="http://www.w3.org/1999/xhtml";let lr=Mt,qt=!1,zr=null;const nn=Cr({},[ar,fr,Mt],Ck);let dn=Cr({},["mi","mo","mn","ms","mtext"]),Ce=Cr({},["annotation-xml"]);const st=Cr({},["title","style","font","a","script"]);let ft=null;const vt=["application/xhtml+xml","text/html"],yr="text/html";let Qe=null,zt=null;const Nr=r.createElement("form"),yn=function(be){return be instanceof RegExp||be instanceof Function},hn=function(){let be=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};if(!(zt&&zt===be)){if((!be||typeof be!="object")&&(be={}),be=gc(be),ft=vt.indexOf(be.PARSER_MEDIA_TYPE)===-1?yr:be.PARSER_MEDIA_TYPE,Qe=ft==="application/xhtml+xml"?Ck:H1,K=wo(be,"ALLOWED_TAGS")?Cr({},be.ALLOWED_TAGS,Qe):G,P=wo(be,"ALLOWED_ATTR")?Cr({},be.ALLOWED_ATTR,Qe):ne,zr=wo(be,"ALLOWED_NAMESPACES")?Cr({},be.ALLOWED_NAMESPACES,Ck):nn,xt=wo(be,"ADD_URI_SAFE_ATTR")?Cr(gc(sr),be.ADD_URI_SAFE_ATTR,Qe):sr,We=wo(be,"ADD_DATA_URI_TAGS")?Cr(gc(ht),be.ADD_DATA_URI_TAGS,Qe):ht,ge=wo(be,"FORBID_CONTENTS")?Cr({},be.FORBID_CONTENTS,Qe):Re,pe=wo(be,"FORBID_TAGS")?Cr({},be.FORBID_TAGS,Qe):{},ae=wo(be,"FORBID_ATTR")?Cr({},be.FORBID_ATTR,Qe):{},rt=wo(be,"USE_PROFILES")?be.USE_PROFILES:!1,se=be.ALLOW_ARIA_ATTR!==!1,Ee=be.ALLOW_DATA_ATTR!==!1,fe=be.ALLOW_UNKNOWN_PROTOCOLS||!1,Me=be.ALLOW_SELF_CLOSE_IN_ATTR!==!1,Oe=be.SAFE_FOR_TEMPLATES||!1,xe=be.SAFE_FOR_XML!==!1,Te=be.WHOLE_DOCUMENT||!1,Ze=be.RETURN_DOM||!1,Et=be.RETURN_DOM_FRAGMENT||!1,qe=be.RETURN_TRUSTED_TYPE||!1,he=be.FORCE_BODY||!1,Ye=be.SANITIZE_DOM!==!1,we=be.SANITIZE_NAMED_PROPS||!1,or=be.KEEP_CONTENT!==!1,Pe=be.IN_PLACE||!1,J=be.ALLOWED_URI_REGEXP||uae,lr=be.NAMESPACE||Mt,dn=be.MATHML_TEXT_INTEGRATION_POINTS||dn,Ce=be.HTML_INTEGRATION_POINTS||Ce,oe=be.CUSTOM_ELEMENT_HANDLING||{},be.CUSTOM_ELEMENT_HANDLING&&yn(be.CUSTOM_ELEMENT_HANDLING.tagNameCheck)&&(oe.tagNameCheck=be.CUSTOM_ELEMENT_HANDLING.tagNameCheck),be.CUSTOM_ELEMENT_HANDLING&&yn(be.CUSTOM_ELEMENT_HANDLING.attributeNameCheck)&&(oe.attributeNameCheck=be.CUSTOM_ELEMENT_HANDLING.attributeNameCheck),be.CUSTOM_ELEMENT_HANDLING&&typeof be.CUSTOM_ELEMENT_HANDLING.allowCustomizedBuiltInElements=="boolean"&&(oe.allowCustomizedBuiltInElements=be.CUSTOM_ELEMENT_HANDLING.allowCustomizedBuiltInElements),Oe&&(Ee=!1),Et&&(Ze=!0),rt&&(K=Cr({},Oz),P=[],rt.html===!0&&(Cr(K,Iz),Cr(P,Bz)),rt.svg===!0&&(Cr(K,Tk),Cr(P,_k),Cr(P,t1)),rt.svgFilters===!0&&(Cr(K,Ak),Cr(P,_k),Cr(P,t1)),rt.mathMl===!0&&(Cr(K,Fk),Cr(P,Pz),Cr(P,t1))),be.ADD_TAGS&&(K===G&&(K=gc(K)),Cr(K,be.ADD_TAGS,Qe)),be.ADD_ATTR&&(P===ne&&(P=gc(P)),Cr(P,be.ADD_ATTR,Qe)),be.ADD_URI_SAFE_ATTR&&Cr(xt,be.ADD_URI_SAFE_ATTR,Qe),be.FORBID_CONTENTS&&(ge===Re&&(ge=gc(ge)),Cr(ge,be.FORBID_CONTENTS,Qe)),or&&(K["#text"]=!0),Te&&Cr(K,["html","head","body"]),K.table&&(Cr(K,["tbody"]),delete pe.tbody),be.TRUSTED_TYPES_POLICY){if(typeof be.TRUSTED_TYPES_POLICY.createHTML!="function")throw qf('TRUSTED_TYPES_POLICY configuration option must provide a "createHTML" hook.');if(typeof be.TRUSTED_TYPES_POLICY.createScriptURL!="function")throw qf('TRUSTED_TYPES_POLICY configuration option must provide a "createScriptURL" hook.');_=be.TRUSTED_TYPES_POLICY,R=_.createHTML("")}else _===void 0&&(_=Wze(m,a)),_!==null&&typeof R=="string"&&(R=_.createHTML(""));Ya&&Ya(be),zt=be}},Ea=Cr({},[...Tk,...Ak,...Ize]),xa=Cr({},[...Fk,...Oze]),Vn=function(be){let at=k(be);(!at||!at.tagName)&&(at={namespaceURI:lr,tagName:"template"});const wt=H1(be.tagName),Hr=H1(at.tagName);return zr[be.namespaceURI]?be.namespaceURI===fr?at.namespaceURI===Mt?wt==="svg":at.namespaceURI===ar?wt==="svg"&&(Hr==="annotation-xml"||dn[Hr]):!!Ea[wt]:be.namespaceURI===ar?at.namespaceURI===Mt?wt==="math":at.namespaceURI===fr?wt==="math"&&Ce[Hr]:!!xa[wt]:be.namespaceURI===Mt?at.namespaceURI===fr&&!Ce[Hr]||at.namespaceURI===ar&&!dn[Hr]?!1:!xa[wt]&&(st[wt]||!Ea[wt]):!!(ft==="application/xhtml+xml"&&zr[be.namespaceURI]):!1},kn=function(be){Gf(t.removed,{element:be});try{k(be).removeChild(be)}catch{S(be)}},Nl=function(be,at){try{Gf(t.removed,{attribute:at.getAttributeNode(be),from:at})}catch{Gf(t.removed,{attribute:null,from:at})}if(at.removeAttribute(be),be==="is")if(Ze||Et)try{kn(at)}catch{}else try{at.setAttribute(be,"")}catch{}},fd=function(be){let at=null,wt=null;if(he)be="<remove></remove>"+be;else{const pn=Mz(be,/^[\r\n\t ]+/);wt=pn&&pn[0]}ft==="application/xhtml+xml"&&lr===Mt&&(be='<html xmlns="http://www.w3.org/1999/xhtml"><head></head><body>'+be+"</body></html>");const Hr=_?_.createHTML(be):be;if(lr===Mt)try{at=new f().parseFromString(Hr,ft)}catch{}if(!at||!at.documentElement){at=C.createDocument(lr,"template",null);try{at.documentElement.innerHTML=qt?R:Hr}catch{}}const Cn=at.body||at.documentElement;return be&&wt&&Cn.insertBefore(r.createTextNode(wt),Cn.childNodes[0]||null),lr===Mt?I.call(at,Te?"html":"body")[0]:Te?at.documentElement:Cn},Ip=function(be){return F.call(be.ownerDocument||be,be,u.SHOW_ELEMENT|u.SHOW_COMMENT|u.SHOW_TEXT|u.SHOW_PROCESSING_INSTRUCTION|u.SHOW_CDATA_SECTION,null)},md=function(be){return be instanceof p&&(typeof be.nodeName!="string"||typeof be.textContent!="string"||typeof be.removeChild!="function"||!(be.attributes instanceof d)||typeof be.removeAttribute!="function"||typeof be.setAttribute!="function"||typeof be.namespaceURI!="string"||typeof be.insertBefore!="function"||typeof be.hasChildNodes!="function")},mo=function(be){return typeof s=="function"&&be instanceof s};function Xr(Bt,be,at){e1(Bt,wt=>{wt.call(t,be,at,zt)})}const ca=function(be){let at=null;if(Xr(j.beforeSanitizeElements,be,null),md(be))return kn(be),!0;const wt=Qe(be.nodeName);if(Xr(j.uponSanitizeElement,be,{tagName:wt,allowedTags:K}),be.hasChildNodes()&&!mo(be.firstElementChild)&&$a(/<[/\w!]/g,be.innerHTML)&&$a(/<[/\w!]/g,be.textContent)||be.nodeType===Vf.progressingInstruction||xe&&be.nodeType===Vf.comment&&$a(/<[/\w]/g,be.data))return kn(be),!0;if(!K[wt]||pe[wt]){if(!pe[wt]&&gd(wt)&&(oe.tagNameCheck instanceof RegExp&&$a(oe.tagNameCheck,wt)||oe.tagNameCheck instanceof Function&&oe.tagNameCheck(wt)))return!1;if(or&&!ge[wt]){const Hr=k(be)||be.parentNode,Cn=E(be)||be.childNodes;if(Cn&&Hr){const pn=Cn.length;for(let Mr=pn-1;Mr>=0;--Mr){const Ei=y(Cn[Mr],!0);Ei.__removalCount=(be.__removalCount||0)+1,Hr.insertBefore(Ei,w(be))}}}return kn(be),!0}return be instanceof l&&!Vn(be)||(wt==="noscript"||wt==="noembed"||wt==="noframes")&&$a(/<\/no(script|embed|frames)/i,be.innerHTML)?(kn(be),!0):(Oe&&be.nodeType===Vf.text&&(at=be.textContent,e1([Y,W,$],Hr=>{at=jf(at,Hr," ")}),be.textContent!==at&&(Gf(t.removed,{element:be.cloneNode()}),be.textContent=at)),Xr(j.afterSanitizeElements,be,null),!1)},ws=function(be,at,wt){if(Ye&&(at==="id"||at==="name")&&(wt in r||wt in Nr))return!1;if(!(Ee&&!ae[at]&&$a(X,at))){if(!(se&&$a(z,at))){if(!P[at]||ae[at]){if(!(gd(be)&&(oe.tagNameCheck instanceof RegExp&&$a(oe.tagNameCheck,be)||oe.tagNameCheck instanceof Function&&oe.tagNameCheck(be))&&(oe.attributeNameCheck instanceof RegExp&&$a(oe.attributeNameCheck,at)||oe.attributeNameCheck instanceof Function&&oe.attributeNameCheck(at))||at==="is"&&oe.allowCustomizedBuiltInElements&&(oe.tagNameCheck instanceof RegExp&&$a(oe.tagNameCheck,wt)||oe.tagNameCheck instanceof Function&&oe.tagNameCheck(wt))))return!1}else if(!xt[at]){if(!$a(J,jf(wt,B,""))){if(!((at==="src"||at==="xlink:href"||at==="href")&&be!=="script"&&Dze(wt,"data:")===0&&We[be])){if(!(fe&&!$a(H,jf(wt,B,"")))){if(wt)return!1}}}}}}return!0},gd=function(be){return be!=="annotation-xml"&&Mz(be,ee)},Xg=function(be){Xr(j.beforeSanitizeAttributes,be,null);const{attributes:at}=be;if(!at||md(be))return;const wt={attrName:"",attrValue:"",keepAttr:!0,allowedAttributes:P,forceKeepAttr:void 0};let Hr=at.length;for(;Hr--;){const Cn=at[Hr],{name:pn,namespaceURI:Mr,value:Ei}=Cn,Ll=Qe(pn);let vn=pn==="value"?Ei:Nze(Ei);if(wt.attrName=Ll,wt.attrValue=vn,wt.keepAttr=!0,wt.forceKeepAttr=void 0,Xr(j.uponSanitizeAttribute,be,wt),vn=wt.attrValue,we&&(Ll==="id"||Ll==="name")&&(Nl(pn,be),vn=Qt+vn),xe&&$a(/((--!?|])>)|<\/(style|title)/i,vn)){Nl(pn,be);continue}if(wt.forceKeepAttr||(Nl(pn,be),!wt.keepAttr))continue;if(!Me&&$a(/\/>/i,vn)){Nl(pn,be);continue}Oe&&e1([Y,W,$],Zg=>{vn=jf(vn,Zg," ")});const ks=Qe(be.nodeName);if(ws(ks,Ll,vn)){if(_&&typeof m=="object"&&typeof m.getAttributeType=="function"&&!Mr)switch(m.getAttributeType(ks,Ll)){case"TrustedHTML":{vn=_.createHTML(vn);break}case"TrustedScriptURL":{vn=_.createScriptURL(vn);break}}try{Mr?be.setAttributeNS(Mr,pn,vn):be.setAttribute(pn,vn),md(be)?kn(be):Lz(t.removed)}catch{}}}Xr(j.afterSanitizeAttributes,be,null)},sS=function Bt(be){let at=null;const wt=Ip(be);for(Xr(j.beforeSanitizeShadowDOM,be,null);at=wt.nextNode();)Xr(j.uponSanitizeShadowNode,at,null),ca(at),Xg(at),at.content instanceof i&&Bt(at.content);Xr(j.afterSanitizeShadowDOM,be,null)};return t.sanitize=function(Bt){let be=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},at=null,wt=null,Hr=null,Cn=null;if(qt=!Bt,qt&&(Bt="<!-->"),typeof Bt!="string"&&!mo(Bt))if(typeof Bt.toString=="function"){if(Bt=Bt.toString(),typeof Bt!="string")throw qf("dirty is not a string, aborting")}else throw qf("toString is not a function");if(!t.isSupported)return Bt;if(Ne||hn(be),t.removed=[],typeof Bt=="string"&&(Pe=!1),Pe){if(Bt.nodeName){const Ei=Qe(Bt.nodeName);if(!K[Ei]||pe[Ei])throw qf("root node is forbidden and cannot be sanitized in-place")}}else if(Bt instanceof s)at=fd("<!---->"),wt=at.ownerDocument.importNode(Bt,!0),wt.nodeType===Vf.element&&wt.nodeName==="BODY"||wt.nodeName==="HTML"?at=wt:at.appendChild(wt);else{if(!Ze&&!Oe&&!Te&&Bt.indexOf("<")===-1)return _&&qe?_.createHTML(Bt):Bt;if(at=fd(Bt),!at)return Ze?null:qe?R:""}at&&he&&kn(at.firstChild);const pn=Ip(Pe?Bt:at);for(;Hr=pn.nextNode();)ca(Hr),Xg(Hr),Hr.content instanceof i&&sS(Hr.content);if(Pe)return Bt;if(Ze){if(Et)for(Cn=L.call(at.ownerDocument);at.firstChild;)Cn.appendChild(at.firstChild);else Cn=at;return(P.shadowroot||P.shadowrootmode)&&(Cn=M.call(n,Cn,!0)),Cn}let Mr=Te?at.outerHTML:at.innerHTML;return Te&&K["!doctype"]&&at.ownerDocument&&at.ownerDocument.doctype&&at.ownerDocument.doctype.name&&$a(cae,at.ownerDocument.doctype.name)&&(Mr="<!DOCTYPE "+at.ownerDocument.doctype.name+`>
`+Mr),Oe&&e1([Y,W,$],Ei=>{Mr=jf(Mr,Ei," ")}),_&&qe?_.createHTML(Mr):Mr},t.setConfig=function(){let Bt=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};hn(Bt),Ne=!0},t.clearConfig=function(){zt=null,Ne=!1},t.isValidAttribute=function(Bt,be,at){zt||hn({});const wt=Qe(Bt),Hr=Qe(be);return ws(wt,Hr,at)},t.addHook=function(Bt,be){typeof be=="function"&&Gf(j[Bt],be)},t.removeHook=function(Bt,be){if(be!==void 0){const at=_ze(j[Bt],be);return at===-1?void 0:Rze(j[Bt],at,1)[0]}return Lz(j[Bt])},t.removeHooks=function(Bt){j[Bt]=[]},t.removeAllHooks=function(){j=Hz()},t}var Xh=dae(),hae=/^-{3}\s*[\n\r](.*?)[\n\r]-{3}\s*[\n\r]+/s,_m=/%{2}{\s*(?:(\w+)\s*:|(\w+))\s*(?:(\w+)|((?:(?!}%{2}).|\r?\n)*))?\s*(?:}%{2})?/gi,Vze=/\s*%%.*\n/gm,Fh,pae=(Fh=class extends Error{constructor(t){super(t),this.name="UnknownDiagramError"}},U(Fh,"UnknownDiagramError"),Fh),Vc={},uN=U(function(e,t){e=e.replace(hae,"").replace(_m,"").replace(Vze,`
`);for(const[r,{detector:n}]of Object.entries(Vc))if(n(e,t))return r;throw new pae(`No diagram type detected matching given configuration for text: ${e}`)},"detectType"),t6=U((...e)=>{for(const{id:t,detector:r,loader:n}of e)fae(t,r,n)},"registerLazyLoadedDiagrams"),fae=U((e,t,r)=>{Vc[e]&&je.warn(`Detector with key ${e} already exists. Overwriting.`),Vc[e]={detector:t,loader:r},je.debug(`Detector with key ${e} added${r?" with loader":""}`)},"addDetector"),Kze=U(e=>Vc[e].loader,"getDiagramLoader"),r6=U((e,t,{depth:r=2,clobber:n=!1}={})=>{const a={depth:r,clobber:n};return Array.isArray(t)&&!Array.isArray(e)?(t.forEach(i=>r6(e,i,a)),e):Array.isArray(t)&&Array.isArray(e)?(t.forEach(i=>{e.includes(i)||e.push(i)}),e):e===void 0||r<=0?e!=null&&typeof e=="object"&&typeof t=="object"?Object.assign(e,t):t:(t!==void 0&&typeof e=="object"&&typeof t=="object"&&Object.keys(t).forEach(i=>{typeof t[i]=="object"&&(e[i]===void 0||typeof e[i]=="object")?(e[i]===void 0&&(e[i]=Array.isArray(t[i])?[]:{}),e[i]=r6(e[i],t[i],{depth:r-1,clobber:n})):(n||typeof e[i]!="object"&&typeof t[i]!="object")&&(e[i]=t[i])}),e)},"assignWithDepth"),Qn=r6,R2="#ffffff",D2="#f2f2f2",Wa=U((e,t)=>t?Se(e,{s:-40,l:10}):Se(e,{s:-40,l:-10}),"mkBorder"),_h,Yze=(_h=class{constructor(){this.background="#f4f4f4",this.primaryColor="#fff4dd",this.noteBkgColor="#fff5ad",this.noteTextColor="#333",this.THEME_COLOR_LIMIT=12,this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px"}updateColors(){var r,n,a,i,o,s,l,u,d,p,f,m,b,y,S,w,E,k,_,R,C;if(this.primaryTextColor=this.primaryTextColor||(this.darkMode?"#eee":"#333"),this.secondaryColor=this.secondaryColor||Se(this.primaryColor,{h:-120}),this.tertiaryColor=this.tertiaryColor||Se(this.primaryColor,{h:180,l:5}),this.primaryBorderColor=this.primaryBorderColor||Wa(this.primaryColor,this.darkMode),this.secondaryBorderColor=this.secondaryBorderColor||Wa(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=this.tertiaryBorderColor||Wa(this.tertiaryColor,this.darkMode),this.noteBorderColor=this.noteBorderColor||Wa(this.noteBkgColor,this.darkMode),this.noteBkgColor=this.noteBkgColor||"#fff5ad",this.noteTextColor=this.noteTextColor||"#333",this.secondaryTextColor=this.secondaryTextColor||dt(this.secondaryColor),this.tertiaryTextColor=this.tertiaryTextColor||dt(this.tertiaryColor),this.lineColor=this.lineColor||dt(this.background),this.arrowheadColor=this.arrowheadColor||dt(this.background),this.textColor=this.textColor||this.primaryTextColor,this.border2=this.border2||this.tertiaryBorderColor,this.nodeBkg=this.nodeBkg||this.primaryColor,this.mainBkg=this.mainBkg||this.primaryColor,this.nodeBorder=this.nodeBorder||this.primaryBorderColor,this.clusterBkg=this.clusterBkg||this.tertiaryColor,this.clusterBorder=this.clusterBorder||this.tertiaryBorderColor,this.defaultLinkColor=this.defaultLinkColor||this.lineColor,this.titleColor=this.titleColor||this.tertiaryTextColor,this.edgeLabelBackground=this.edgeLabelBackground||(this.darkMode?Ut(this.secondaryColor,30):this.secondaryColor),this.nodeTextColor=this.nodeTextColor||this.primaryTextColor,this.actorBorder=this.actorBorder||this.primaryBorderColor,this.actorBkg=this.actorBkg||this.mainBkg,this.actorTextColor=this.actorTextColor||this.primaryTextColor,this.actorLineColor=this.actorLineColor||this.actorBorder,this.labelBoxBkgColor=this.labelBoxBkgColor||this.actorBkg,this.signalColor=this.signalColor||this.textColor,this.signalTextColor=this.signalTextColor||this.textColor,this.labelBoxBorderColor=this.labelBoxBorderColor||this.actorBorder,this.labelTextColor=this.labelTextColor||this.actorTextColor,this.loopTextColor=this.loopTextColor||this.actorTextColor,this.activationBorderColor=this.activationBorderColor||Ut(this.secondaryColor,10),this.activationBkgColor=this.activationBkgColor||this.secondaryColor,this.sequenceNumberColor=this.sequenceNumberColor||dt(this.lineColor),this.sectionBkgColor=this.sectionBkgColor||this.tertiaryColor,this.altSectionBkgColor=this.altSectionBkgColor||"white",this.sectionBkgColor=this.sectionBkgColor||this.secondaryColor,this.sectionBkgColor2=this.sectionBkgColor2||this.primaryColor,this.excludeBkgColor=this.excludeBkgColor||"#eeeeee",this.taskBorderColor=this.taskBorderColor||this.primaryBorderColor,this.taskBkgColor=this.taskBkgColor||this.primaryColor,this.activeTaskBorderColor=this.activeTaskBorderColor||this.primaryColor,this.activeTaskBkgColor=this.activeTaskBkgColor||Ft(this.primaryColor,23),this.gridColor=this.gridColor||"lightgrey",this.doneTaskBkgColor=this.doneTaskBkgColor||"lightgrey",this.doneTaskBorderColor=this.doneTaskBorderColor||"grey",this.critBorderColor=this.critBorderColor||"#ff8888",this.critBkgColor=this.critBkgColor||"red",this.todayLineColor=this.todayLineColor||"red",this.vertLineColor=this.vertLineColor||"navy",this.taskTextColor=this.taskTextColor||this.textColor,this.taskTextOutsideColor=this.taskTextOutsideColor||this.textColor,this.taskTextLightColor=this.taskTextLightColor||this.textColor,this.taskTextColor=this.taskTextColor||this.primaryTextColor,this.taskTextDarkColor=this.taskTextDarkColor||this.textColor,this.taskTextClickableColor=this.taskTextClickableColor||"#003163",this.personBorder=this.personBorder||this.primaryBorderColor,this.personBkg=this.personBkg||this.mainBkg,this.darkMode?(this.rowOdd=this.rowOdd||Ut(this.mainBkg,5)||"#ffffff",this.rowEven=this.rowEven||Ut(this.mainBkg,10)):(this.rowOdd=this.rowOdd||Ft(this.mainBkg,75)||"#ffffff",this.rowEven=this.rowEven||Ft(this.mainBkg,5)),this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||this.tertiaryColor,this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.nodeBorder,this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.transitionColor=this.transitionColor||this.lineColor,this.specialStateColor=this.lineColor,this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||Se(this.primaryColor,{h:30}),this.cScale4=this.cScale4||Se(this.primaryColor,{h:60}),this.cScale5=this.cScale5||Se(this.primaryColor,{h:90}),this.cScale6=this.cScale6||Se(this.primaryColor,{h:120}),this.cScale7=this.cScale7||Se(this.primaryColor,{h:150}),this.cScale8=this.cScale8||Se(this.primaryColor,{h:210,l:150}),this.cScale9=this.cScale9||Se(this.primaryColor,{h:270}),this.cScale10=this.cScale10||Se(this.primaryColor,{h:300}),this.cScale11=this.cScale11||Se(this.primaryColor,{h:330}),this.darkMode)for(let F=0;F<this.THEME_COLOR_LIMIT;F++)this["cScale"+F]=Ut(this["cScale"+F],75);else for(let F=0;F<this.THEME_COLOR_LIMIT;F++)this["cScale"+F]=Ut(this["cScale"+F],25);for(let F=0;F<this.THEME_COLOR_LIMIT;F++)this["cScaleInv"+F]=this["cScaleInv"+F]||dt(this["cScale"+F]);for(let F=0;F<this.THEME_COLOR_LIMIT;F++)this.darkMode?this["cScalePeer"+F]=this["cScalePeer"+F]||Ft(this["cScale"+F],10):this["cScalePeer"+F]=this["cScalePeer"+F]||Ut(this["cScale"+F],10);this.scaleLabelColor=this.scaleLabelColor||this.labelTextColor;for(let F=0;F<this.THEME_COLOR_LIMIT;F++)this["cScaleLabel"+F]=this["cScaleLabel"+F]||this.scaleLabelColor;const t=this.darkMode?-4:-1;for(let F=0;F<5;F++)this["surface"+F]=this["surface"+F]||Se(this.mainBkg,{h:180,s:-15,l:t*(5+F*3)}),this["surfacePeer"+F]=this["surfacePeer"+F]||Se(this.mainBkg,{h:180,s:-15,l:t*(8+F*3)});this.classText=this.classText||this.textColor,this.fillType0=this.fillType0||this.primaryColor,this.fillType1=this.fillType1||this.secondaryColor,this.fillType2=this.fillType2||Se(this.primaryColor,{h:64}),this.fillType3=this.fillType3||Se(this.secondaryColor,{h:64}),this.fillType4=this.fillType4||Se(this.primaryColor,{h:-64}),this.fillType5=this.fillType5||Se(this.secondaryColor,{h:-64}),this.fillType6=this.fillType6||Se(this.primaryColor,{h:128}),this.fillType7=this.fillType7||Se(this.secondaryColor,{h:128}),this.pie1=this.pie1||this.primaryColor,this.pie2=this.pie2||this.secondaryColor,this.pie3=this.pie3||this.tertiaryColor,this.pie4=this.pie4||Se(this.primaryColor,{l:-10}),this.pie5=this.pie5||Se(this.secondaryColor,{l:-10}),this.pie6=this.pie6||Se(this.tertiaryColor,{l:-10}),this.pie7=this.pie7||Se(this.primaryColor,{h:60,l:-10}),this.pie8=this.pie8||Se(this.primaryColor,{h:-60,l:-10}),this.pie9=this.pie9||Se(this.primaryColor,{h:120,l:0}),this.pie10=this.pie10||Se(this.primaryColor,{h:60,l:-20}),this.pie11=this.pie11||Se(this.primaryColor,{h:-60,l:-20}),this.pie12=this.pie12||Se(this.primaryColor,{h:120,l:-10}),this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOuterStrokeWidth=this.pieOuterStrokeWidth||"2px",this.pieOuterStrokeColor=this.pieOuterStrokeColor||"black",this.pieOpacity=this.pieOpacity||"0.7",this.radar={axisColor:((r=this.radar)==null?void 0:r.axisColor)||this.lineColor,axisStrokeWidth:((n=this.radar)==null?void 0:n.axisStrokeWidth)||2,axisLabelFontSize:((a=this.radar)==null?void 0:a.axisLabelFontSize)||12,curveOpacity:((i=this.radar)==null?void 0:i.curveOpacity)||.5,curveStrokeWidth:((o=this.radar)==null?void 0:o.curveStrokeWidth)||2,graticuleColor:((s=this.radar)==null?void 0:s.graticuleColor)||"#DEDEDE",graticuleStrokeWidth:((l=this.radar)==null?void 0:l.graticuleStrokeWidth)||1,graticuleOpacity:((u=this.radar)==null?void 0:u.graticuleOpacity)||.3,legendBoxSize:((d=this.radar)==null?void 0:d.legendBoxSize)||12,legendFontSize:((p=this.radar)==null?void 0:p.legendFontSize)||12},this.archEdgeColor=this.archEdgeColor||"#777",this.archEdgeArrowColor=this.archEdgeArrowColor||"#777",this.archEdgeWidth=this.archEdgeWidth||"3",this.archGroupBorderColor=this.archGroupBorderColor||"#000",this.archGroupBorderWidth=this.archGroupBorderWidth||"2px",this.quadrant1Fill=this.quadrant1Fill||this.primaryColor,this.quadrant2Fill=this.quadrant2Fill||Se(this.primaryColor,{r:5,g:5,b:5}),this.quadrant3Fill=this.quadrant3Fill||Se(this.primaryColor,{r:10,g:10,b:10}),this.quadrant4Fill=this.quadrant4Fill||Se(this.primaryColor,{r:15,g:15,b:15}),this.quadrant1TextFill=this.quadrant1TextFill||this.primaryTextColor,this.quadrant2TextFill=this.quadrant2TextFill||Se(this.primaryTextColor,{r:-5,g:-5,b:-5}),this.quadrant3TextFill=this.quadrant3TextFill||Se(this.primaryTextColor,{r:-10,g:-10,b:-10}),this.quadrant4TextFill=this.quadrant4TextFill||Se(this.primaryTextColor,{r:-15,g:-15,b:-15}),this.quadrantPointFill=this.quadrantPointFill||Hg(this.quadrant1Fill)?Ft(this.quadrant1Fill):Ut(this.quadrant1Fill),this.quadrantPointTextFill=this.quadrantPointTextFill||this.primaryTextColor,this.quadrantXAxisTextFill=this.quadrantXAxisTextFill||this.primaryTextColor,this.quadrantYAxisTextFill=this.quadrantYAxisTextFill||this.primaryTextColor,this.quadrantInternalBorderStrokeFill=this.quadrantInternalBorderStrokeFill||this.primaryBorderColor,this.quadrantExternalBorderStrokeFill=this.quadrantExternalBorderStrokeFill||this.primaryBorderColor,this.quadrantTitleFill=this.quadrantTitleFill||this.primaryTextColor,this.xyChart={backgroundColor:((f=this.xyChart)==null?void 0:f.backgroundColor)||this.background,titleColor:((m=this.xyChart)==null?void 0:m.titleColor)||this.primaryTextColor,xAxisTitleColor:((b=this.xyChart)==null?void 0:b.xAxisTitleColor)||this.primaryTextColor,xAxisLabelColor:((y=this.xyChart)==null?void 0:y.xAxisLabelColor)||this.primaryTextColor,xAxisTickColor:((S=this.xyChart)==null?void 0:S.xAxisTickColor)||this.primaryTextColor,xAxisLineColor:((w=this.xyChart)==null?void 0:w.xAxisLineColor)||this.primaryTextColor,yAxisTitleColor:((E=this.xyChart)==null?void 0:E.yAxisTitleColor)||this.primaryTextColor,yAxisLabelColor:((k=this.xyChart)==null?void 0:k.yAxisLabelColor)||this.primaryTextColor,yAxisTickColor:((_=this.xyChart)==null?void 0:_.yAxisTickColor)||this.primaryTextColor,yAxisLineColor:((R=this.xyChart)==null?void 0:R.yAxisLineColor)||this.primaryTextColor,plotColorPalette:((C=this.xyChart)==null?void 0:C.plotColorPalette)||"#FFF4DD,#FFD8B1,#FFA07A,#ECEFF1,#D6DBDF,#C3E0A8,#FFB6A4,#FFD74D,#738FA7,#FFFFF0"},this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||"1",this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||(this.darkMode?Ut(this.secondaryColor,30):this.secondaryColor),this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=this.git0||this.primaryColor,this.git1=this.git1||this.secondaryColor,this.git2=this.git2||this.tertiaryColor,this.git3=this.git3||Se(this.primaryColor,{h:-30}),this.git4=this.git4||Se(this.primaryColor,{h:-60}),this.git5=this.git5||Se(this.primaryColor,{h:-90}),this.git6=this.git6||Se(this.primaryColor,{h:60}),this.git7=this.git7||Se(this.primaryColor,{h:120}),this.darkMode?(this.git0=Ft(this.git0,25),this.git1=Ft(this.git1,25),this.git2=Ft(this.git2,25),this.git3=Ft(this.git3,25),this.git4=Ft(this.git4,25),this.git5=Ft(this.git5,25),this.git6=Ft(this.git6,25),this.git7=Ft(this.git7,25)):(this.git0=Ut(this.git0,25),this.git1=Ut(this.git1,25),this.git2=Ut(this.git2,25),this.git3=Ut(this.git3,25),this.git4=Ut(this.git4,25),this.git5=Ut(this.git5,25),this.git6=Ut(this.git6,25),this.git7=Ut(this.git7,25)),this.gitInv0=this.gitInv0||dt(this.git0),this.gitInv1=this.gitInv1||dt(this.git1),this.gitInv2=this.gitInv2||dt(this.git2),this.gitInv3=this.gitInv3||dt(this.git3),this.gitInv4=this.gitInv4||dt(this.git4),this.gitInv5=this.gitInv5||dt(this.git5),this.gitInv6=this.gitInv6||dt(this.git6),this.gitInv7=this.gitInv7||dt(this.git7),this.branchLabelColor=this.branchLabelColor||(this.darkMode?"black":this.labelTextColor),this.gitBranchLabel0=this.gitBranchLabel0||this.branchLabelColor,this.gitBranchLabel1=this.gitBranchLabel1||this.branchLabelColor,this.gitBranchLabel2=this.gitBranchLabel2||this.branchLabelColor,this.gitBranchLabel3=this.gitBranchLabel3||this.branchLabelColor,this.gitBranchLabel4=this.gitBranchLabel4||this.branchLabelColor,this.gitBranchLabel5=this.gitBranchLabel5||this.branchLabelColor,this.gitBranchLabel6=this.gitBranchLabel6||this.branchLabelColor,this.gitBranchLabel7=this.gitBranchLabel7||this.branchLabelColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||R2,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||D2}calculate(t){if(typeof t!="object"){this.updateColors();return}const r=Object.keys(t);r.forEach(n=>{this[n]=t[n]}),this.updateColors(),r.forEach(n=>{this[n]=t[n]})}},U(_h,"Theme"),_h),Jze=U(e=>{const t=new Yze;return t.calculate(e),t},"getThemeVariables"),Rh,Xze=(Rh=class{constructor(){this.background="#333",this.primaryColor="#1f2020",this.secondaryColor=Ft(this.primaryColor,16),this.tertiaryColor=Se(this.primaryColor,{h:-160}),this.primaryBorderColor=dt(this.background),this.secondaryBorderColor=Wa(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Wa(this.tertiaryColor,this.darkMode),this.primaryTextColor=dt(this.primaryColor),this.secondaryTextColor=dt(this.secondaryColor),this.tertiaryTextColor=dt(this.tertiaryColor),this.lineColor=dt(this.background),this.textColor=dt(this.background),this.mainBkg="#1f2020",this.secondBkg="calculated",this.mainContrastColor="lightgrey",this.darkTextColor=Ft(dt("#323D47"),10),this.lineColor="calculated",this.border1="#ccc",this.border2=Fm(255,255,255,.25),this.arrowheadColor="calculated",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.labelBackground="#181818",this.textColor="#ccc",this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="#F9FFFE",this.edgeLabelBackground="calculated",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="calculated",this.actorLineColor="calculated",this.signalColor="calculated",this.signalTextColor="calculated",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="calculated",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="#fff5ad",this.noteTextColor="calculated",this.activationBorderColor="calculated",this.activationBkgColor="calculated",this.sequenceNumberColor="black",this.sectionBkgColor=Ut("#EAE8D9",30),this.altSectionBkgColor="calculated",this.sectionBkgColor2="#EAE8D9",this.excludeBkgColor=Ut(this.sectionBkgColor,10),this.taskBorderColor=Fm(255,255,255,70),this.taskBkgColor="calculated",this.taskTextColor="calculated",this.taskTextLightColor="calculated",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor=Fm(255,255,255,50),this.activeTaskBkgColor="#81B1DB",this.gridColor="calculated",this.doneTaskBkgColor="calculated",this.doneTaskBorderColor="grey",this.critBorderColor="#E83737",this.critBkgColor="#E83737",this.taskTextDarkColor="calculated",this.todayLineColor="#DB5757",this.vertLineColor="#00BFFF",this.personBorder=this.primaryBorderColor,this.personBkg=this.mainBkg,this.archEdgeColor="calculated",this.archEdgeArrowColor="calculated",this.archEdgeWidth="3",this.archGroupBorderColor=this.primaryBorderColor,this.archGroupBorderWidth="2px",this.rowOdd=this.rowOdd||Ft(this.mainBkg,5)||"#ffffff",this.rowEven=this.rowEven||Ut(this.mainBkg,10),this.labelColor="calculated",this.errorBkgColor="#a44141",this.errorTextColor="#ddd"}updateColors(){var t,r,n,a,i,o,s,l,u,d,p,f,m,b,y,S,w,E,k,_,R;this.secondBkg=Ft(this.mainBkg,16),this.lineColor=this.mainContrastColor,this.arrowheadColor=this.mainContrastColor,this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.edgeLabelBackground=Ft(this.labelBackground,25),this.actorBorder=this.border1,this.actorBkg=this.mainBkg,this.actorTextColor=this.mainContrastColor,this.actorLineColor=this.actorBorder,this.signalColor=this.mainContrastColor,this.signalTextColor=this.mainContrastColor,this.labelBoxBkgColor=this.actorBkg,this.labelBoxBorderColor=this.actorBorder,this.labelTextColor=this.mainContrastColor,this.loopTextColor=this.mainContrastColor,this.noteBorderColor=this.secondaryBorderColor,this.noteBkgColor=this.secondBkg,this.noteTextColor=this.secondaryTextColor,this.activationBorderColor=this.border1,this.activationBkgColor=this.secondBkg,this.altSectionBkgColor=this.background,this.taskBkgColor=Ft(this.mainBkg,23),this.taskTextColor=this.darkTextColor,this.taskTextLightColor=this.mainContrastColor,this.taskTextOutsideColor=this.taskTextLightColor,this.gridColor=this.mainContrastColor,this.doneTaskBkgColor=this.mainContrastColor,this.taskTextDarkColor=this.darkTextColor,this.archEdgeColor=this.lineColor,this.archEdgeArrowColor=this.lineColor,this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#555",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.primaryBorderColor,this.specialStateColor="#f4f4f4",this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=Se(this.primaryColor,{h:64}),this.fillType3=Se(this.secondaryColor,{h:64}),this.fillType4=Se(this.primaryColor,{h:-64}),this.fillType5=Se(this.secondaryColor,{h:-64}),this.fillType6=Se(this.primaryColor,{h:128}),this.fillType7=Se(this.secondaryColor,{h:128}),this.cScale1=this.cScale1||"#0b0000",this.cScale2=this.cScale2||"#4d1037",this.cScale3=this.cScale3||"#3f5258",this.cScale4=this.cScale4||"#4f2f1b",this.cScale5=this.cScale5||"#6e0a0a",this.cScale6=this.cScale6||"#3b0048",this.cScale7=this.cScale7||"#995a01",this.cScale8=this.cScale8||"#154706",this.cScale9=this.cScale9||"#161722",this.cScale10=this.cScale10||"#00296f",this.cScale11=this.cScale11||"#01629c",this.cScale12=this.cScale12||"#010029",this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||Se(this.primaryColor,{h:30}),this.cScale4=this.cScale4||Se(this.primaryColor,{h:60}),this.cScale5=this.cScale5||Se(this.primaryColor,{h:90}),this.cScale6=this.cScale6||Se(this.primaryColor,{h:120}),this.cScale7=this.cScale7||Se(this.primaryColor,{h:150}),this.cScale8=this.cScale8||Se(this.primaryColor,{h:210}),this.cScale9=this.cScale9||Se(this.primaryColor,{h:270}),this.cScale10=this.cScale10||Se(this.primaryColor,{h:300}),this.cScale11=this.cScale11||Se(this.primaryColor,{h:330});for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleInv"+C]=this["cScaleInv"+C]||dt(this["cScale"+C]);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScalePeer"+C]=this["cScalePeer"+C]||Ft(this["cScale"+C],10);for(let C=0;C<5;C++)this["surface"+C]=this["surface"+C]||Se(this.mainBkg,{h:30,s:-30,l:-(-10+C*4)}),this["surfacePeer"+C]=this["surfacePeer"+C]||Se(this.mainBkg,{h:30,s:-30,l:-(-7+C*4)});this.scaleLabelColor=this.scaleLabelColor||(this.darkMode?"black":this.labelTextColor);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleLabel"+C]=this["cScaleLabel"+C]||this.scaleLabelColor;for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["pie"+C]=this["cScale"+C];this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOuterStrokeWidth=this.pieOuterStrokeWidth||"2px",this.pieOuterStrokeColor=this.pieOuterStrokeColor||"black",this.pieOpacity=this.pieOpacity||"0.7",this.quadrant1Fill=this.quadrant1Fill||this.primaryColor,this.quadrant2Fill=this.quadrant2Fill||Se(this.primaryColor,{r:5,g:5,b:5}),this.quadrant3Fill=this.quadrant3Fill||Se(this.primaryColor,{r:10,g:10,b:10}),this.quadrant4Fill=this.quadrant4Fill||Se(this.primaryColor,{r:15,g:15,b:15}),this.quadrant1TextFill=this.quadrant1TextFill||this.primaryTextColor,this.quadrant2TextFill=this.quadrant2TextFill||Se(this.primaryTextColor,{r:-5,g:-5,b:-5}),this.quadrant3TextFill=this.quadrant3TextFill||Se(this.primaryTextColor,{r:-10,g:-10,b:-10}),this.quadrant4TextFill=this.quadrant4TextFill||Se(this.primaryTextColor,{r:-15,g:-15,b:-15}),this.quadrantPointFill=this.quadrantPointFill||Hg(this.quadrant1Fill)?Ft(this.quadrant1Fill):Ut(this.quadrant1Fill),this.quadrantPointTextFill=this.quadrantPointTextFill||this.primaryTextColor,this.quadrantXAxisTextFill=this.quadrantXAxisTextFill||this.primaryTextColor,this.quadrantYAxisTextFill=this.quadrantYAxisTextFill||this.primaryTextColor,this.quadrantInternalBorderStrokeFill=this.quadrantInternalBorderStrokeFill||this.primaryBorderColor,this.quadrantExternalBorderStrokeFill=this.quadrantExternalBorderStrokeFill||this.primaryBorderColor,this.quadrantTitleFill=this.quadrantTitleFill||this.primaryTextColor,this.xyChart={backgroundColor:((t=this.xyChart)==null?void 0:t.backgroundColor)||this.background,titleColor:((r=this.xyChart)==null?void 0:r.titleColor)||this.primaryTextColor,xAxisTitleColor:((n=this.xyChart)==null?void 0:n.xAxisTitleColor)||this.primaryTextColor,xAxisLabelColor:((a=this.xyChart)==null?void 0:a.xAxisLabelColor)||this.primaryTextColor,xAxisTickColor:((i=this.xyChart)==null?void 0:i.xAxisTickColor)||this.primaryTextColor,xAxisLineColor:((o=this.xyChart)==null?void 0:o.xAxisLineColor)||this.primaryTextColor,yAxisTitleColor:((s=this.xyChart)==null?void 0:s.yAxisTitleColor)||this.primaryTextColor,yAxisLabelColor:((l=this.xyChart)==null?void 0:l.yAxisLabelColor)||this.primaryTextColor,yAxisTickColor:((u=this.xyChart)==null?void 0:u.yAxisTickColor)||this.primaryTextColor,yAxisLineColor:((d=this.xyChart)==null?void 0:d.yAxisLineColor)||this.primaryTextColor,plotColorPalette:((p=this.xyChart)==null?void 0:p.plotColorPalette)||"#3498db,#2ecc71,#e74c3c,#f1c40f,#bdc3c7,#ffffff,#34495e,#9b59b6,#1abc9c,#e67e22"},this.packet={startByteColor:this.primaryTextColor,endByteColor:this.primaryTextColor,labelColor:this.primaryTextColor,titleColor:this.primaryTextColor,blockStrokeColor:this.primaryTextColor,blockFillColor:this.background},this.radar={axisColor:((f=this.radar)==null?void 0:f.axisColor)||this.lineColor,axisStrokeWidth:((m=this.radar)==null?void 0:m.axisStrokeWidth)||2,axisLabelFontSize:((b=this.radar)==null?void 0:b.axisLabelFontSize)||12,curveOpacity:((y=this.radar)==null?void 0:y.curveOpacity)||.5,curveStrokeWidth:((S=this.radar)==null?void 0:S.curveStrokeWidth)||2,graticuleColor:((w=this.radar)==null?void 0:w.graticuleColor)||"#DEDEDE",graticuleStrokeWidth:((E=this.radar)==null?void 0:E.graticuleStrokeWidth)||1,graticuleOpacity:((k=this.radar)==null?void 0:k.graticuleOpacity)||.3,legendBoxSize:((_=this.radar)==null?void 0:_.legendBoxSize)||12,legendFontSize:((R=this.radar)==null?void 0:R.legendFontSize)||12},this.classText=this.primaryTextColor,this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||"1",this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||(this.darkMode?Ut(this.secondaryColor,30):this.secondaryColor),this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=Ft(this.secondaryColor,20),this.git1=Ft(this.pie2||this.secondaryColor,20),this.git2=Ft(this.pie3||this.tertiaryColor,20),this.git3=Ft(this.pie4||Se(this.primaryColor,{h:-30}),20),this.git4=Ft(this.pie5||Se(this.primaryColor,{h:-60}),20),this.git5=Ft(this.pie6||Se(this.primaryColor,{h:-90}),10),this.git6=Ft(this.pie7||Se(this.primaryColor,{h:60}),10),this.git7=Ft(this.pie8||Se(this.primaryColor,{h:120}),20),this.gitInv0=this.gitInv0||dt(this.git0),this.gitInv1=this.gitInv1||dt(this.git1),this.gitInv2=this.gitInv2||dt(this.git2),this.gitInv3=this.gitInv3||dt(this.git3),this.gitInv4=this.gitInv4||dt(this.git4),this.gitInv5=this.gitInv5||dt(this.git5),this.gitInv6=this.gitInv6||dt(this.git6),this.gitInv7=this.gitInv7||dt(this.git7),this.gitBranchLabel0=this.gitBranchLabel0||dt(this.labelTextColor),this.gitBranchLabel1=this.gitBranchLabel1||this.labelTextColor,this.gitBranchLabel2=this.gitBranchLabel2||this.labelTextColor,this.gitBranchLabel3=this.gitBranchLabel3||dt(this.labelTextColor),this.gitBranchLabel4=this.gitBranchLabel4||this.labelTextColor,this.gitBranchLabel5=this.gitBranchLabel5||this.labelTextColor,this.gitBranchLabel6=this.gitBranchLabel6||this.labelTextColor,this.gitBranchLabel7=this.gitBranchLabel7||this.labelTextColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||Ft(this.background,12),this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||Ft(this.background,2),this.nodeBorder=this.nodeBorder||"#999"}calculate(t){if(typeof t!="object"){this.updateColors();return}const r=Object.keys(t);r.forEach(n=>{this[n]=t[n]}),this.updateColors(),r.forEach(n=>{this[n]=t[n]})}},U(Rh,"Theme"),Rh),Zze=U(e=>{const t=new Xze;return t.calculate(e),t},"getThemeVariables"),Dh,Qze=(Dh=class{constructor(){this.background="#f4f4f4",this.primaryColor="#ECECFF",this.secondaryColor=Se(this.primaryColor,{h:120}),this.secondaryColor="#ffffde",this.tertiaryColor=Se(this.primaryColor,{h:-160}),this.primaryBorderColor=Wa(this.primaryColor,this.darkMode),this.secondaryBorderColor=Wa(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Wa(this.tertiaryColor,this.darkMode),this.primaryTextColor=dt(this.primaryColor),this.secondaryTextColor=dt(this.secondaryColor),this.tertiaryTextColor=dt(this.tertiaryColor),this.lineColor=dt(this.background),this.textColor=dt(this.background),this.background="white",this.mainBkg="#ECECFF",this.secondBkg="#ffffde",this.lineColor="#333333",this.border1="#9370DB",this.border2="#aaaa33",this.arrowheadColor="#333333",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.labelBackground="rgba(232,232,232, 0.8)",this.textColor="#333",this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="calculated",this.edgeLabelBackground="calculated",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="black",this.actorLineColor="calculated",this.signalColor="calculated",this.signalTextColor="calculated",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="calculated",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="#fff5ad",this.noteTextColor="calculated",this.activationBorderColor="#666",this.activationBkgColor="#f4f4f4",this.sequenceNumberColor="white",this.sectionBkgColor="calculated",this.altSectionBkgColor="calculated",this.sectionBkgColor2="calculated",this.excludeBkgColor="#eeeeee",this.taskBorderColor="calculated",this.taskBkgColor="calculated",this.taskTextLightColor="calculated",this.taskTextColor=this.taskTextLightColor,this.taskTextDarkColor="calculated",this.taskTextOutsideColor=this.taskTextDarkColor,this.taskTextClickableColor="calculated",this.activeTaskBorderColor="calculated",this.activeTaskBkgColor="calculated",this.gridColor="calculated",this.doneTaskBkgColor="calculated",this.doneTaskBorderColor="calculated",this.critBorderColor="calculated",this.critBkgColor="calculated",this.todayLineColor="calculated",this.vertLineColor="calculated",this.sectionBkgColor=Fm(102,102,255,.49),this.altSectionBkgColor="white",this.sectionBkgColor2="#fff400",this.taskBorderColor="#534fbc",this.taskBkgColor="#8a90dd",this.taskTextLightColor="white",this.taskTextColor="calculated",this.taskTextDarkColor="black",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor="#534fbc",this.activeTaskBkgColor="#bfc7ff",this.gridColor="lightgrey",this.doneTaskBkgColor="lightgrey",this.doneTaskBorderColor="grey",this.critBorderColor="#ff8888",this.critBkgColor="red",this.todayLineColor="red",this.vertLineColor="navy",this.personBorder=this.primaryBorderColor,this.personBkg=this.mainBkg,this.archEdgeColor="calculated",this.archEdgeArrowColor="calculated",this.archEdgeWidth="3",this.archGroupBorderColor=this.primaryBorderColor,this.archGroupBorderWidth="2px",this.rowOdd="calculated",this.rowEven="calculated",this.labelColor="black",this.errorBkgColor="#552222",this.errorTextColor="#552222",this.updateColors()}updateColors(){var t,r,n,a,i,o,s,l,u,d,p,f,m,b,y,S,w,E,k,_,R;this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||Se(this.primaryColor,{h:30}),this.cScale4=this.cScale4||Se(this.primaryColor,{h:60}),this.cScale5=this.cScale5||Se(this.primaryColor,{h:90}),this.cScale6=this.cScale6||Se(this.primaryColor,{h:120}),this.cScale7=this.cScale7||Se(this.primaryColor,{h:150}),this.cScale8=this.cScale8||Se(this.primaryColor,{h:210}),this.cScale9=this.cScale9||Se(this.primaryColor,{h:270}),this.cScale10=this.cScale10||Se(this.primaryColor,{h:300}),this.cScale11=this.cScale11||Se(this.primaryColor,{h:330}),this.cScalePeer1=this.cScalePeer1||Ut(this.secondaryColor,45),this.cScalePeer2=this.cScalePeer2||Ut(this.tertiaryColor,40);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScale"+C]=Ut(this["cScale"+C],10),this["cScalePeer"+C]=this["cScalePeer"+C]||Ut(this["cScale"+C],25);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleInv"+C]=this["cScaleInv"+C]||Se(this["cScale"+C],{h:180});for(let C=0;C<5;C++)this["surface"+C]=this["surface"+C]||Se(this.mainBkg,{h:30,l:-(5+C*5)}),this["surfacePeer"+C]=this["surfacePeer"+C]||Se(this.mainBkg,{h:30,l:-(7+C*5)});if(this.scaleLabelColor=this.scaleLabelColor!=="calculated"&&this.scaleLabelColor?this.scaleLabelColor:this.labelTextColor,this.labelTextColor!=="calculated"){this.cScaleLabel0=this.cScaleLabel0||dt(this.labelTextColor),this.cScaleLabel3=this.cScaleLabel3||dt(this.labelTextColor);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleLabel"+C]=this["cScaleLabel"+C]||this.labelTextColor}this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.titleColor=this.textColor,this.edgeLabelBackground=this.labelBackground,this.actorBorder=Ft(this.border1,23),this.actorBkg=this.mainBkg,this.labelBoxBkgColor=this.actorBkg,this.signalColor=this.textColor,this.signalTextColor=this.textColor,this.labelBoxBorderColor=this.actorBorder,this.labelTextColor=this.actorTextColor,this.loopTextColor=this.actorTextColor,this.noteBorderColor=this.border2,this.noteTextColor=this.actorTextColor,this.actorLineColor=this.actorBorder,this.taskTextColor=this.taskTextLightColor,this.taskTextOutsideColor=this.taskTextDarkColor,this.archEdgeColor=this.lineColor,this.archEdgeArrowColor=this.lineColor,this.rowOdd=this.rowOdd||Ft(this.primaryColor,75)||"#ffffff",this.rowEven=this.rowEven||Ft(this.primaryColor,1),this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#f0f0f0",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.nodeBorder,this.specialStateColor=this.lineColor,this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.transitionColor=this.transitionColor||this.lineColor,this.classText=this.primaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=Se(this.primaryColor,{h:64}),this.fillType3=Se(this.secondaryColor,{h:64}),this.fillType4=Se(this.primaryColor,{h:-64}),this.fillType5=Se(this.secondaryColor,{h:-64}),this.fillType6=Se(this.primaryColor,{h:128}),this.fillType7=Se(this.secondaryColor,{h:128}),this.pie1=this.pie1||this.primaryColor,this.pie2=this.pie2||this.secondaryColor,this.pie3=this.pie3||Se(this.tertiaryColor,{l:-40}),this.pie4=this.pie4||Se(this.primaryColor,{l:-10}),this.pie5=this.pie5||Se(this.secondaryColor,{l:-30}),this.pie6=this.pie6||Se(this.tertiaryColor,{l:-20}),this.pie7=this.pie7||Se(this.primaryColor,{h:60,l:-20}),this.pie8=this.pie8||Se(this.primaryColor,{h:-60,l:-40}),this.pie9=this.pie9||Se(this.primaryColor,{h:120,l:-40}),this.pie10=this.pie10||Se(this.primaryColor,{h:60,l:-40}),this.pie11=this.pie11||Se(this.primaryColor,{h:-90,l:-40}),this.pie12=this.pie12||Se(this.primaryColor,{h:120,l:-30}),this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOuterStrokeWidth=this.pieOuterStrokeWidth||"2px",this.pieOuterStrokeColor=this.pieOuterStrokeColor||"black",this.pieOpacity=this.pieOpacity||"0.7",this.quadrant1Fill=this.quadrant1Fill||this.primaryColor,this.quadrant2Fill=this.quadrant2Fill||Se(this.primaryColor,{r:5,g:5,b:5}),this.quadrant3Fill=this.quadrant3Fill||Se(this.primaryColor,{r:10,g:10,b:10}),this.quadrant4Fill=this.quadrant4Fill||Se(this.primaryColor,{r:15,g:15,b:15}),this.quadrant1TextFill=this.quadrant1TextFill||this.primaryTextColor,this.quadrant2TextFill=this.quadrant2TextFill||Se(this.primaryTextColor,{r:-5,g:-5,b:-5}),this.quadrant3TextFill=this.quadrant3TextFill||Se(this.primaryTextColor,{r:-10,g:-10,b:-10}),this.quadrant4TextFill=this.quadrant4TextFill||Se(this.primaryTextColor,{r:-15,g:-15,b:-15}),this.quadrantPointFill=this.quadrantPointFill||Hg(this.quadrant1Fill)?Ft(this.quadrant1Fill):Ut(this.quadrant1Fill),this.quadrantPointTextFill=this.quadrantPointTextFill||this.primaryTextColor,this.quadrantXAxisTextFill=this.quadrantXAxisTextFill||this.primaryTextColor,this.quadrantYAxisTextFill=this.quadrantYAxisTextFill||this.primaryTextColor,this.quadrantInternalBorderStrokeFill=this.quadrantInternalBorderStrokeFill||this.primaryBorderColor,this.quadrantExternalBorderStrokeFill=this.quadrantExternalBorderStrokeFill||this.primaryBorderColor,this.quadrantTitleFill=this.quadrantTitleFill||this.primaryTextColor,this.radar={axisColor:((t=this.radar)==null?void 0:t.axisColor)||this.lineColor,axisStrokeWidth:((r=this.radar)==null?void 0:r.axisStrokeWidth)||2,axisLabelFontSize:((n=this.radar)==null?void 0:n.axisLabelFontSize)||12,curveOpacity:((a=this.radar)==null?void 0:a.curveOpacity)||.5,curveStrokeWidth:((i=this.radar)==null?void 0:i.curveStrokeWidth)||2,graticuleColor:((o=this.radar)==null?void 0:o.graticuleColor)||"#DEDEDE",graticuleStrokeWidth:((s=this.radar)==null?void 0:s.graticuleStrokeWidth)||1,graticuleOpacity:((l=this.radar)==null?void 0:l.graticuleOpacity)||.3,legendBoxSize:((u=this.radar)==null?void 0:u.legendBoxSize)||12,legendFontSize:((d=this.radar)==null?void 0:d.legendFontSize)||12},this.xyChart={backgroundColor:((p=this.xyChart)==null?void 0:p.backgroundColor)||this.background,titleColor:((f=this.xyChart)==null?void 0:f.titleColor)||this.primaryTextColor,xAxisTitleColor:((m=this.xyChart)==null?void 0:m.xAxisTitleColor)||this.primaryTextColor,xAxisLabelColor:((b=this.xyChart)==null?void 0:b.xAxisLabelColor)||this.primaryTextColor,xAxisTickColor:((y=this.xyChart)==null?void 0:y.xAxisTickColor)||this.primaryTextColor,xAxisLineColor:((S=this.xyChart)==null?void 0:S.xAxisLineColor)||this.primaryTextColor,yAxisTitleColor:((w=this.xyChart)==null?void 0:w.yAxisTitleColor)||this.primaryTextColor,yAxisLabelColor:((E=this.xyChart)==null?void 0:E.yAxisLabelColor)||this.primaryTextColor,yAxisTickColor:((k=this.xyChart)==null?void 0:k.yAxisTickColor)||this.primaryTextColor,yAxisLineColor:((_=this.xyChart)==null?void 0:_.yAxisLineColor)||this.primaryTextColor,plotColorPalette:((R=this.xyChart)==null?void 0:R.plotColorPalette)||"#ECECFF,#8493A6,#FFC3A0,#DCDDE1,#B8E994,#D1A36F,#C3CDE6,#FFB6C1,#496078,#F8F3E3"},this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||"1",this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||this.labelBackground,this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=this.git0||this.primaryColor,this.git1=this.git1||this.secondaryColor,this.git2=this.git2||this.tertiaryColor,this.git3=this.git3||Se(this.primaryColor,{h:-30}),this.git4=this.git4||Se(this.primaryColor,{h:-60}),this.git5=this.git5||Se(this.primaryColor,{h:-90}),this.git6=this.git6||Se(this.primaryColor,{h:60}),this.git7=this.git7||Se(this.primaryColor,{h:120}),this.darkMode?(this.git0=Ft(this.git0,25),this.git1=Ft(this.git1,25),this.git2=Ft(this.git2,25),this.git3=Ft(this.git3,25),this.git4=Ft(this.git4,25),this.git5=Ft(this.git5,25),this.git6=Ft(this.git6,25),this.git7=Ft(this.git7,25)):(this.git0=Ut(this.git0,25),this.git1=Ut(this.git1,25),this.git2=Ut(this.git2,25),this.git3=Ut(this.git3,25),this.git4=Ut(this.git4,25),this.git5=Ut(this.git5,25),this.git6=Ut(this.git6,25),this.git7=Ut(this.git7,25)),this.gitInv0=this.gitInv0||Ut(dt(this.git0),25),this.gitInv1=this.gitInv1||dt(this.git1),this.gitInv2=this.gitInv2||dt(this.git2),this.gitInv3=this.gitInv3||dt(this.git3),this.gitInv4=this.gitInv4||dt(this.git4),this.gitInv5=this.gitInv5||dt(this.git5),this.gitInv6=this.gitInv6||dt(this.git6),this.gitInv7=this.gitInv7||dt(this.git7),this.gitBranchLabel0=this.gitBranchLabel0||dt(this.labelTextColor),this.gitBranchLabel1=this.gitBranchLabel1||this.labelTextColor,this.gitBranchLabel2=this.gitBranchLabel2||this.labelTextColor,this.gitBranchLabel3=this.gitBranchLabel3||dt(this.labelTextColor),this.gitBranchLabel4=this.gitBranchLabel4||this.labelTextColor,this.gitBranchLabel5=this.gitBranchLabel5||this.labelTextColor,this.gitBranchLabel6=this.gitBranchLabel6||this.labelTextColor,this.gitBranchLabel7=this.gitBranchLabel7||this.labelTextColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||R2,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||D2}calculate(t){if(Object.keys(this).forEach(n=>{this[n]==="calculated"&&(this[n]=void 0)}),typeof t!="object"){this.updateColors();return}const r=Object.keys(t);r.forEach(n=>{this[n]=t[n]}),this.updateColors(),r.forEach(n=>{this[n]=t[n]})}},U(Dh,"Theme"),Dh),eHe=U(e=>{const t=new Qze;return t.calculate(e),t},"getThemeVariables"),Nh,tHe=(Nh=class{constructor(){this.background="#f4f4f4",this.primaryColor="#cde498",this.secondaryColor="#cdffb2",this.background="white",this.mainBkg="#cde498",this.secondBkg="#cdffb2",this.lineColor="green",this.border1="#13540c",this.border2="#6eaa49",this.arrowheadColor="green",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.tertiaryColor=Ft("#cde498",10),this.primaryBorderColor=Wa(this.primaryColor,this.darkMode),this.secondaryBorderColor=Wa(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Wa(this.tertiaryColor,this.darkMode),this.primaryTextColor=dt(this.primaryColor),this.secondaryTextColor=dt(this.secondaryColor),this.tertiaryTextColor=dt(this.primaryColor),this.lineColor=dt(this.background),this.textColor=dt(this.background),this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="#333",this.edgeLabelBackground="#e8e8e8",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="black",this.actorLineColor="calculated",this.signalColor="#333",this.signalTextColor="#333",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="#326932",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="#fff5ad",this.noteTextColor="calculated",this.activationBorderColor="#666",this.activationBkgColor="#f4f4f4",this.sequenceNumberColor="white",this.sectionBkgColor="#6eaa49",this.altSectionBkgColor="white",this.sectionBkgColor2="#6eaa49",this.excludeBkgColor="#eeeeee",this.taskBorderColor="calculated",this.taskBkgColor="#487e3a",this.taskTextLightColor="white",this.taskTextColor="calculated",this.taskTextDarkColor="black",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor="calculated",this.activeTaskBkgColor="calculated",this.gridColor="lightgrey",this.doneTaskBkgColor="lightgrey",this.doneTaskBorderColor="grey",this.critBorderColor="#ff8888",this.critBkgColor="red",this.todayLineColor="red",this.vertLineColor="#00BFFF",this.personBorder=this.primaryBorderColor,this.personBkg=this.mainBkg,this.archEdgeColor="calculated",this.archEdgeArrowColor="calculated",this.archEdgeWidth="3",this.archGroupBorderColor=this.primaryBorderColor,this.archGroupBorderWidth="2px",this.labelColor="black",this.errorBkgColor="#552222",this.errorTextColor="#552222"}updateColors(){var t,r,n,a,i,o,s,l,u,d,p,f,m,b,y,S,w,E,k,_,R;this.actorBorder=Ut(this.mainBkg,20),this.actorBkg=this.mainBkg,this.labelBoxBkgColor=this.actorBkg,this.labelTextColor=this.actorTextColor,this.loopTextColor=this.actorTextColor,this.noteBorderColor=this.border2,this.noteTextColor=this.actorTextColor,this.actorLineColor=this.actorBorder,this.cScale0=this.cScale0||this.primaryColor,this.cScale1=this.cScale1||this.secondaryColor,this.cScale2=this.cScale2||this.tertiaryColor,this.cScale3=this.cScale3||Se(this.primaryColor,{h:30}),this.cScale4=this.cScale4||Se(this.primaryColor,{h:60}),this.cScale5=this.cScale5||Se(this.primaryColor,{h:90}),this.cScale6=this.cScale6||Se(this.primaryColor,{h:120}),this.cScale7=this.cScale7||Se(this.primaryColor,{h:150}),this.cScale8=this.cScale8||Se(this.primaryColor,{h:210}),this.cScale9=this.cScale9||Se(this.primaryColor,{h:270}),this.cScale10=this.cScale10||Se(this.primaryColor,{h:300}),this.cScale11=this.cScale11||Se(this.primaryColor,{h:330}),this.cScalePeer1=this.cScalePeer1||Ut(this.secondaryColor,45),this.cScalePeer2=this.cScalePeer2||Ut(this.tertiaryColor,40);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScale"+C]=Ut(this["cScale"+C],10),this["cScalePeer"+C]=this["cScalePeer"+C]||Ut(this["cScale"+C],25);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleInv"+C]=this["cScaleInv"+C]||Se(this["cScale"+C],{h:180});this.scaleLabelColor=this.scaleLabelColor!=="calculated"&&this.scaleLabelColor?this.scaleLabelColor:this.labelTextColor;for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleLabel"+C]=this["cScaleLabel"+C]||this.scaleLabelColor;for(let C=0;C<5;C++)this["surface"+C]=this["surface"+C]||Se(this.mainBkg,{h:30,s:-30,l:-(5+C*5)}),this["surfacePeer"+C]=this["surfacePeer"+C]||Se(this.mainBkg,{h:30,s:-30,l:-(8+C*5)});this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.taskBorderColor=this.border1,this.taskTextColor=this.taskTextLightColor,this.taskTextOutsideColor=this.taskTextDarkColor,this.activeTaskBorderColor=this.taskBorderColor,this.activeTaskBkgColor=this.mainBkg,this.archEdgeColor=this.lineColor,this.archEdgeArrowColor=this.lineColor,this.rowOdd=this.rowOdd||Ft(this.mainBkg,75)||"#ffffff",this.rowEven=this.rowEven||Ft(this.mainBkg,20),this.transitionColor=this.transitionColor||this.lineColor,this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#f0f0f0",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.compositeBorder=this.compositeBorder||this.nodeBorder,this.innerEndBackground=this.primaryBorderColor,this.specialStateColor=this.lineColor,this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.transitionColor=this.transitionColor||this.lineColor,this.classText=this.primaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=Se(this.primaryColor,{h:64}),this.fillType3=Se(this.secondaryColor,{h:64}),this.fillType4=Se(this.primaryColor,{h:-64}),this.fillType5=Se(this.secondaryColor,{h:-64}),this.fillType6=Se(this.primaryColor,{h:128}),this.fillType7=Se(this.secondaryColor,{h:128}),this.pie1=this.pie1||this.primaryColor,this.pie2=this.pie2||this.secondaryColor,this.pie3=this.pie3||this.tertiaryColor,this.pie4=this.pie4||Se(this.primaryColor,{l:-30}),this.pie5=this.pie5||Se(this.secondaryColor,{l:-30}),this.pie6=this.pie6||Se(this.tertiaryColor,{h:40,l:-40}),this.pie7=this.pie7||Se(this.primaryColor,{h:60,l:-10}),this.pie8=this.pie8||Se(this.primaryColor,{h:-60,l:-10}),this.pie9=this.pie9||Se(this.primaryColor,{h:120,l:0}),this.pie10=this.pie10||Se(this.primaryColor,{h:60,l:-50}),this.pie11=this.pie11||Se(this.primaryColor,{h:-60,l:-50}),this.pie12=this.pie12||Se(this.primaryColor,{h:120,l:-50}),this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOuterStrokeWidth=this.pieOuterStrokeWidth||"2px",this.pieOuterStrokeColor=this.pieOuterStrokeColor||"black",this.pieOpacity=this.pieOpacity||"0.7",this.quadrant1Fill=this.quadrant1Fill||this.primaryColor,this.quadrant2Fill=this.quadrant2Fill||Se(this.primaryColor,{r:5,g:5,b:5}),this.quadrant3Fill=this.quadrant3Fill||Se(this.primaryColor,{r:10,g:10,b:10}),this.quadrant4Fill=this.quadrant4Fill||Se(this.primaryColor,{r:15,g:15,b:15}),this.quadrant1TextFill=this.quadrant1TextFill||this.primaryTextColor,this.quadrant2TextFill=this.quadrant2TextFill||Se(this.primaryTextColor,{r:-5,g:-5,b:-5}),this.quadrant3TextFill=this.quadrant3TextFill||Se(this.primaryTextColor,{r:-10,g:-10,b:-10}),this.quadrant4TextFill=this.quadrant4TextFill||Se(this.primaryTextColor,{r:-15,g:-15,b:-15}),this.quadrantPointFill=this.quadrantPointFill||Hg(this.quadrant1Fill)?Ft(this.quadrant1Fill):Ut(this.quadrant1Fill),this.quadrantPointTextFill=this.quadrantPointTextFill||this.primaryTextColor,this.quadrantXAxisTextFill=this.quadrantXAxisTextFill||this.primaryTextColor,this.quadrantYAxisTextFill=this.quadrantYAxisTextFill||this.primaryTextColor,this.quadrantInternalBorderStrokeFill=this.quadrantInternalBorderStrokeFill||this.primaryBorderColor,this.quadrantExternalBorderStrokeFill=this.quadrantExternalBorderStrokeFill||this.primaryBorderColor,this.quadrantTitleFill=this.quadrantTitleFill||this.primaryTextColor,this.packet={startByteColor:this.primaryTextColor,endByteColor:this.primaryTextColor,labelColor:this.primaryTextColor,titleColor:this.primaryTextColor,blockStrokeColor:this.primaryTextColor,blockFillColor:this.mainBkg},this.radar={axisColor:((t=this.radar)==null?void 0:t.axisColor)||this.lineColor,axisStrokeWidth:((r=this.radar)==null?void 0:r.axisStrokeWidth)||2,axisLabelFontSize:((n=this.radar)==null?void 0:n.axisLabelFontSize)||12,curveOpacity:((a=this.radar)==null?void 0:a.curveOpacity)||.5,curveStrokeWidth:((i=this.radar)==null?void 0:i.curveStrokeWidth)||2,graticuleColor:((o=this.radar)==null?void 0:o.graticuleColor)||"#DEDEDE",graticuleStrokeWidth:((s=this.radar)==null?void 0:s.graticuleStrokeWidth)||1,graticuleOpacity:((l=this.radar)==null?void 0:l.graticuleOpacity)||.3,legendBoxSize:((u=this.radar)==null?void 0:u.legendBoxSize)||12,legendFontSize:((d=this.radar)==null?void 0:d.legendFontSize)||12},this.xyChart={backgroundColor:((p=this.xyChart)==null?void 0:p.backgroundColor)||this.background,titleColor:((f=this.xyChart)==null?void 0:f.titleColor)||this.primaryTextColor,xAxisTitleColor:((m=this.xyChart)==null?void 0:m.xAxisTitleColor)||this.primaryTextColor,xAxisLabelColor:((b=this.xyChart)==null?void 0:b.xAxisLabelColor)||this.primaryTextColor,xAxisTickColor:((y=this.xyChart)==null?void 0:y.xAxisTickColor)||this.primaryTextColor,xAxisLineColor:((S=this.xyChart)==null?void 0:S.xAxisLineColor)||this.primaryTextColor,yAxisTitleColor:((w=this.xyChart)==null?void 0:w.yAxisTitleColor)||this.primaryTextColor,yAxisLabelColor:((E=this.xyChart)==null?void 0:E.yAxisLabelColor)||this.primaryTextColor,yAxisTickColor:((k=this.xyChart)==null?void 0:k.yAxisTickColor)||this.primaryTextColor,yAxisLineColor:((_=this.xyChart)==null?void 0:_.yAxisLineColor)||this.primaryTextColor,plotColorPalette:((R=this.xyChart)==null?void 0:R.plotColorPalette)||"#CDE498,#FF6B6B,#A0D2DB,#D7BDE2,#F0F0F0,#FFC3A0,#7FD8BE,#FF9A8B,#FAF3E0,#FFF176"},this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||"1",this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||this.edgeLabelBackground,this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=this.git0||this.primaryColor,this.git1=this.git1||this.secondaryColor,this.git2=this.git2||this.tertiaryColor,this.git3=this.git3||Se(this.primaryColor,{h:-30}),this.git4=this.git4||Se(this.primaryColor,{h:-60}),this.git5=this.git5||Se(this.primaryColor,{h:-90}),this.git6=this.git6||Se(this.primaryColor,{h:60}),this.git7=this.git7||Se(this.primaryColor,{h:120}),this.darkMode?(this.git0=Ft(this.git0,25),this.git1=Ft(this.git1,25),this.git2=Ft(this.git2,25),this.git3=Ft(this.git3,25),this.git4=Ft(this.git4,25),this.git5=Ft(this.git5,25),this.git6=Ft(this.git6,25),this.git7=Ft(this.git7,25)):(this.git0=Ut(this.git0,25),this.git1=Ut(this.git1,25),this.git2=Ut(this.git2,25),this.git3=Ut(this.git3,25),this.git4=Ut(this.git4,25),this.git5=Ut(this.git5,25),this.git6=Ut(this.git6,25),this.git7=Ut(this.git7,25)),this.gitInv0=this.gitInv0||dt(this.git0),this.gitInv1=this.gitInv1||dt(this.git1),this.gitInv2=this.gitInv2||dt(this.git2),this.gitInv3=this.gitInv3||dt(this.git3),this.gitInv4=this.gitInv4||dt(this.git4),this.gitInv5=this.gitInv5||dt(this.git5),this.gitInv6=this.gitInv6||dt(this.git6),this.gitInv7=this.gitInv7||dt(this.git7),this.gitBranchLabel0=this.gitBranchLabel0||dt(this.labelTextColor),this.gitBranchLabel1=this.gitBranchLabel1||this.labelTextColor,this.gitBranchLabel2=this.gitBranchLabel2||this.labelTextColor,this.gitBranchLabel3=this.gitBranchLabel3||dt(this.labelTextColor),this.gitBranchLabel4=this.gitBranchLabel4||this.labelTextColor,this.gitBranchLabel5=this.gitBranchLabel5||this.labelTextColor,this.gitBranchLabel6=this.gitBranchLabel6||this.labelTextColor,this.gitBranchLabel7=this.gitBranchLabel7||this.labelTextColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||R2,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||D2}calculate(t){if(typeof t!="object"){this.updateColors();return}const r=Object.keys(t);r.forEach(n=>{this[n]=t[n]}),this.updateColors(),r.forEach(n=>{this[n]=t[n]})}},U(Nh,"Theme"),Nh),rHe=U(e=>{const t=new tHe;return t.calculate(e),t},"getThemeVariables"),Lh,nHe=(Lh=class{constructor(){this.primaryColor="#eee",this.contrast="#707070",this.secondaryColor=Ft(this.contrast,55),this.background="#ffffff",this.tertiaryColor=Se(this.primaryColor,{h:-160}),this.primaryBorderColor=Wa(this.primaryColor,this.darkMode),this.secondaryBorderColor=Wa(this.secondaryColor,this.darkMode),this.tertiaryBorderColor=Wa(this.tertiaryColor,this.darkMode),this.primaryTextColor=dt(this.primaryColor),this.secondaryTextColor=dt(this.secondaryColor),this.tertiaryTextColor=dt(this.tertiaryColor),this.lineColor=dt(this.background),this.textColor=dt(this.background),this.mainBkg="#eee",this.secondBkg="calculated",this.lineColor="#666",this.border1="#999",this.border2="calculated",this.note="#ffa",this.text="#333",this.critical="#d42",this.done="#bbb",this.arrowheadColor="#333333",this.fontFamily='"trebuchet ms", verdana, arial, sans-serif',this.fontSize="16px",this.THEME_COLOR_LIMIT=12,this.nodeBkg="calculated",this.nodeBorder="calculated",this.clusterBkg="calculated",this.clusterBorder="calculated",this.defaultLinkColor="calculated",this.titleColor="calculated",this.edgeLabelBackground="white",this.actorBorder="calculated",this.actorBkg="calculated",this.actorTextColor="calculated",this.actorLineColor=this.actorBorder,this.signalColor="calculated",this.signalTextColor="calculated",this.labelBoxBkgColor="calculated",this.labelBoxBorderColor="calculated",this.labelTextColor="calculated",this.loopTextColor="calculated",this.noteBorderColor="calculated",this.noteBkgColor="calculated",this.noteTextColor="calculated",this.activationBorderColor="#666",this.activationBkgColor="#f4f4f4",this.sequenceNumberColor="white",this.sectionBkgColor="calculated",this.altSectionBkgColor="white",this.sectionBkgColor2="calculated",this.excludeBkgColor="#eeeeee",this.taskBorderColor="calculated",this.taskBkgColor="calculated",this.taskTextLightColor="white",this.taskTextColor="calculated",this.taskTextDarkColor="calculated",this.taskTextOutsideColor="calculated",this.taskTextClickableColor="#003163",this.activeTaskBorderColor="calculated",this.activeTaskBkgColor="calculated",this.gridColor="calculated",this.doneTaskBkgColor="calculated",this.doneTaskBorderColor="calculated",this.critBkgColor="calculated",this.critBorderColor="calculated",this.todayLineColor="calculated",this.vertLineColor="calculated",this.personBorder=this.primaryBorderColor,this.personBkg=this.mainBkg,this.archEdgeColor="calculated",this.archEdgeArrowColor="calculated",this.archEdgeWidth="3",this.archGroupBorderColor=this.primaryBorderColor,this.archGroupBorderWidth="2px",this.rowOdd=this.rowOdd||Ft(this.mainBkg,75)||"#ffffff",this.rowEven=this.rowEven||"#f4f4f4",this.labelColor="black",this.errorBkgColor="#552222",this.errorTextColor="#552222"}updateColors(){var t,r,n,a,i,o,s,l,u,d,p,f,m,b,y,S,w,E,k,_,R;this.secondBkg=Ft(this.contrast,55),this.border2=this.contrast,this.actorBorder=Ft(this.border1,23),this.actorBkg=this.mainBkg,this.actorTextColor=this.text,this.actorLineColor=this.actorBorder,this.signalColor=this.text,this.signalTextColor=this.text,this.labelBoxBkgColor=this.actorBkg,this.labelBoxBorderColor=this.actorBorder,this.labelTextColor=this.text,this.loopTextColor=this.text,this.noteBorderColor="#999",this.noteBkgColor="#666",this.noteTextColor="#fff",this.cScale0=this.cScale0||"#555",this.cScale1=this.cScale1||"#F4F4F4",this.cScale2=this.cScale2||"#555",this.cScale3=this.cScale3||"#BBB",this.cScale4=this.cScale4||"#777",this.cScale5=this.cScale5||"#999",this.cScale6=this.cScale6||"#DDD",this.cScale7=this.cScale7||"#FFF",this.cScale8=this.cScale8||"#DDD",this.cScale9=this.cScale9||"#BBB",this.cScale10=this.cScale10||"#999",this.cScale11=this.cScale11||"#777";for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleInv"+C]=this["cScaleInv"+C]||dt(this["cScale"+C]);for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this.darkMode?this["cScalePeer"+C]=this["cScalePeer"+C]||Ft(this["cScale"+C],10):this["cScalePeer"+C]=this["cScalePeer"+C]||Ut(this["cScale"+C],10);this.scaleLabelColor=this.scaleLabelColor||(this.darkMode?"black":this.labelTextColor),this.cScaleLabel0=this.cScaleLabel0||this.cScale1,this.cScaleLabel2=this.cScaleLabel2||this.cScale1;for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["cScaleLabel"+C]=this["cScaleLabel"+C]||this.scaleLabelColor;for(let C=0;C<5;C++)this["surface"+C]=this["surface"+C]||Se(this.mainBkg,{l:-(5+C*5)}),this["surfacePeer"+C]=this["surfacePeer"+C]||Se(this.mainBkg,{l:-(8+C*5)});this.nodeBkg=this.mainBkg,this.nodeBorder=this.border1,this.clusterBkg=this.secondBkg,this.clusterBorder=this.border2,this.defaultLinkColor=this.lineColor,this.titleColor=this.text,this.sectionBkgColor=Ft(this.contrast,30),this.sectionBkgColor2=Ft(this.contrast,30),this.taskBorderColor=Ut(this.contrast,10),this.taskBkgColor=this.contrast,this.taskTextColor=this.taskTextLightColor,this.taskTextDarkColor=this.text,this.taskTextOutsideColor=this.taskTextDarkColor,this.activeTaskBorderColor=this.taskBorderColor,this.activeTaskBkgColor=this.mainBkg,this.gridColor=Ft(this.border1,30),this.doneTaskBkgColor=this.done,this.doneTaskBorderColor=this.lineColor,this.critBkgColor=this.critical,this.critBorderColor=Ut(this.critBkgColor,10),this.todayLineColor=this.critBkgColor,this.vertLineColor=this.critBkgColor,this.archEdgeColor=this.lineColor,this.archEdgeArrowColor=this.lineColor,this.transitionColor=this.transitionColor||"#000",this.transitionLabelColor=this.transitionLabelColor||this.textColor,this.stateLabelColor=this.stateLabelColor||this.stateBkg||this.primaryTextColor,this.stateBkg=this.stateBkg||this.mainBkg,this.labelBackgroundColor=this.labelBackgroundColor||this.stateBkg,this.compositeBackground=this.compositeBackground||this.background||this.tertiaryColor,this.altBackground=this.altBackground||"#f4f4f4",this.compositeTitleBackground=this.compositeTitleBackground||this.mainBkg,this.stateBorder=this.stateBorder||"#000",this.innerEndBackground=this.primaryBorderColor,this.specialStateColor="#222",this.errorBkgColor=this.errorBkgColor||this.tertiaryColor,this.errorTextColor=this.errorTextColor||this.tertiaryTextColor,this.classText=this.primaryTextColor,this.fillType0=this.primaryColor,this.fillType1=this.secondaryColor,this.fillType2=Se(this.primaryColor,{h:64}),this.fillType3=Se(this.secondaryColor,{h:64}),this.fillType4=Se(this.primaryColor,{h:-64}),this.fillType5=Se(this.secondaryColor,{h:-64}),this.fillType6=Se(this.primaryColor,{h:128}),this.fillType7=Se(this.secondaryColor,{h:128});for(let C=0;C<this.THEME_COLOR_LIMIT;C++)this["pie"+C]=this["cScale"+C];this.pie12=this.pie0,this.pieTitleTextSize=this.pieTitleTextSize||"25px",this.pieTitleTextColor=this.pieTitleTextColor||this.taskTextDarkColor,this.pieSectionTextSize=this.pieSectionTextSize||"17px",this.pieSectionTextColor=this.pieSectionTextColor||this.textColor,this.pieLegendTextSize=this.pieLegendTextSize||"17px",this.pieLegendTextColor=this.pieLegendTextColor||this.taskTextDarkColor,this.pieStrokeColor=this.pieStrokeColor||"black",this.pieStrokeWidth=this.pieStrokeWidth||"2px",this.pieOuterStrokeWidth=this.pieOuterStrokeWidth||"2px",this.pieOuterStrokeColor=this.pieOuterStrokeColor||"black",this.pieOpacity=this.pieOpacity||"0.7",this.quadrant1Fill=this.quadrant1Fill||this.primaryColor,this.quadrant2Fill=this.quadrant2Fill||Se(this.primaryColor,{r:5,g:5,b:5}),this.quadrant3Fill=this.quadrant3Fill||Se(this.primaryColor,{r:10,g:10,b:10}),this.quadrant4Fill=this.quadrant4Fill||Se(this.primaryColor,{r:15,g:15,b:15}),this.quadrant1TextFill=this.quadrant1TextFill||this.primaryTextColor,this.quadrant2TextFill=this.quadrant2TextFill||Se(this.primaryTextColor,{r:-5,g:-5,b:-5}),this.quadrant3TextFill=this.quadrant3TextFill||Se(this.primaryTextColor,{r:-10,g:-10,b:-10}),this.quadrant4TextFill=this.quadrant4TextFill||Se(this.primaryTextColor,{r:-15,g:-15,b:-15}),this.quadrantPointFill=this.quadrantPointFill||Hg(this.quadrant1Fill)?Ft(this.quadrant1Fill):Ut(this.quadrant1Fill),this.quadrantPointTextFill=this.quadrantPointTextFill||this.primaryTextColor,this.quadrantXAxisTextFill=this.quadrantXAxisTextFill||this.primaryTextColor,this.quadrantYAxisTextFill=this.quadrantYAxisTextFill||this.primaryTextColor,this.quadrantInternalBorderStrokeFill=this.quadrantInternalBorderStrokeFill||this.primaryBorderColor,this.quadrantExternalBorderStrokeFill=this.quadrantExternalBorderStrokeFill||this.primaryBorderColor,this.quadrantTitleFill=this.quadrantTitleFill||this.primaryTextColor,this.xyChart={backgroundColor:((t=this.xyChart)==null?void 0:t.backgroundColor)||this.background,titleColor:((r=this.xyChart)==null?void 0:r.titleColor)||this.primaryTextColor,xAxisTitleColor:((n=this.xyChart)==null?void 0:n.xAxisTitleColor)||this.primaryTextColor,xAxisLabelColor:((a=this.xyChart)==null?void 0:a.xAxisLabelColor)||this.primaryTextColor,xAxisTickColor:((i=this.xyChart)==null?void 0:i.xAxisTickColor)||this.primaryTextColor,xAxisLineColor:((o=this.xyChart)==null?void 0:o.xAxisLineColor)||this.primaryTextColor,yAxisTitleColor:((s=this.xyChart)==null?void 0:s.yAxisTitleColor)||this.primaryTextColor,yAxisLabelColor:((l=this.xyChart)==null?void 0:l.yAxisLabelColor)||this.primaryTextColor,yAxisTickColor:((u=this.xyChart)==null?void 0:u.yAxisTickColor)||this.primaryTextColor,yAxisLineColor:((d=this.xyChart)==null?void 0:d.yAxisLineColor)||this.primaryTextColor,plotColorPalette:((p=this.xyChart)==null?void 0:p.plotColorPalette)||"#EEE,#6BB8E4,#8ACB88,#C7ACD6,#E8DCC2,#FFB2A8,#FFF380,#7E8D91,#FFD8B1,#FAF3E0"},this.radar={axisColor:((f=this.radar)==null?void 0:f.axisColor)||this.lineColor,axisStrokeWidth:((m=this.radar)==null?void 0:m.axisStrokeWidth)||2,axisLabelFontSize:((b=this.radar)==null?void 0:b.axisLabelFontSize)||12,curveOpacity:((y=this.radar)==null?void 0:y.curveOpacity)||.5,curveStrokeWidth:((S=this.radar)==null?void 0:S.curveStrokeWidth)||2,graticuleColor:((w=this.radar)==null?void 0:w.graticuleColor)||"#DEDEDE",graticuleStrokeWidth:((E=this.radar)==null?void 0:E.graticuleStrokeWidth)||1,graticuleOpacity:((k=this.radar)==null?void 0:k.graticuleOpacity)||.3,legendBoxSize:((_=this.radar)==null?void 0:_.legendBoxSize)||12,legendFontSize:((R=this.radar)==null?void 0:R.legendFontSize)||12},this.requirementBackground=this.requirementBackground||this.primaryColor,this.requirementBorderColor=this.requirementBorderColor||this.primaryBorderColor,this.requirementBorderSize=this.requirementBorderSize||"1",this.requirementTextColor=this.requirementTextColor||this.primaryTextColor,this.relationColor=this.relationColor||this.lineColor,this.relationLabelBackground=this.relationLabelBackground||this.edgeLabelBackground,this.relationLabelColor=this.relationLabelColor||this.actorTextColor,this.git0=Ut(this.pie1,25)||this.primaryColor,this.git1=this.pie2||this.secondaryColor,this.git2=this.pie3||this.tertiaryColor,this.git3=this.pie4||Se(this.primaryColor,{h:-30}),this.git4=this.pie5||Se(this.primaryColor,{h:-60}),this.git5=this.pie6||Se(this.primaryColor,{h:-90}),this.git6=this.pie7||Se(this.primaryColor,{h:60}),this.git7=this.pie8||Se(this.primaryColor,{h:120}),this.gitInv0=this.gitInv0||dt(this.git0),this.gitInv1=this.gitInv1||dt(this.git1),this.gitInv2=this.gitInv2||dt(this.git2),this.gitInv3=this.gitInv3||dt(this.git3),this.gitInv4=this.gitInv4||dt(this.git4),this.gitInv5=this.gitInv5||dt(this.git5),this.gitInv6=this.gitInv6||dt(this.git6),this.gitInv7=this.gitInv7||dt(this.git7),this.branchLabelColor=this.branchLabelColor||this.labelTextColor,this.gitBranchLabel0=this.branchLabelColor,this.gitBranchLabel1="white",this.gitBranchLabel2=this.branchLabelColor,this.gitBranchLabel3="white",this.gitBranchLabel4=this.branchLabelColor,this.gitBranchLabel5=this.branchLabelColor,this.gitBranchLabel6=this.branchLabelColor,this.gitBranchLabel7=this.branchLabelColor,this.tagLabelColor=this.tagLabelColor||this.primaryTextColor,this.tagLabelBackground=this.tagLabelBackground||this.primaryColor,this.tagLabelBorder=this.tagBorder||this.primaryBorderColor,this.tagLabelFontSize=this.tagLabelFontSize||"10px",this.commitLabelColor=this.commitLabelColor||this.secondaryTextColor,this.commitLabelBackground=this.commitLabelBackground||this.secondaryColor,this.commitLabelFontSize=this.commitLabelFontSize||"10px",this.attributeBackgroundColorOdd=this.attributeBackgroundColorOdd||R2,this.attributeBackgroundColorEven=this.attributeBackgroundColorEven||D2}calculate(t){if(typeof t!="object"){this.updateColors();return}const r=Object.keys(t);r.forEach(n=>{this[n]=t[n]}),this.updateColors(),r.forEach(n=>{this[n]=t[n]})}},U(Lh,"Theme"),Lh),aHe=U(e=>{const t=new nHe;return t.calculate(e),t},"getThemeVariables"),dl={base:{getThemeVariables:Jze},dark:{getThemeVariables:Zze},default:{getThemeVariables:eHe},forest:{getThemeVariables:rHe},neutral:{getThemeVariables:aHe}},Yo={flowchart:{useMaxWidth:!0,titleTopMargin:25,subGraphTitleMargin:{top:0,bottom:0},diagramPadding:8,htmlLabels:!0,nodeSpacing:50,rankSpacing:50,curve:"basis",padding:15,defaultRenderer:"dagre-wrapper",wrappingWidth:200,inheritDir:!1},sequence:{useMaxWidth:!0,hideUnusedParticipants:!1,activationWidth:10,diagramMarginX:50,diagramMarginY:10,actorMargin:50,width:150,height:65,boxMargin:10,boxTextMargin:5,noteMargin:10,messageMargin:35,messageAlign:"center",mirrorActors:!0,forceMenus:!1,bottomMarginAdj:1,rightAngles:!1,showSequenceNumbers:!1,actorFontSize:14,actorFontFamily:'"Open Sans", sans-serif',actorFontWeight:400,noteFontSize:14,noteFontFamily:'"trebuchet ms", verdana, arial, sans-serif',noteFontWeight:400,noteAlign:"center",messageFontSize:16,messageFontFamily:'"trebuchet ms", verdana, arial, sans-serif',messageFontWeight:400,wrap:!1,wrapPadding:10,labelBoxWidth:50,labelBoxHeight:20},gantt:{useMaxWidth:!0,titleTopMargin:25,barHeight:20,barGap:4,topPadding:50,rightPadding:75,leftPadding:75,gridLineStartPadding:35,fontSize:11,sectionFontSize:11,numberSectionStyles:4,axisFormat:"%Y-%m-%d",topAxis:!1,displayMode:"",weekday:"sunday"},journey:{useMaxWidth:!0,diagramMarginX:50,diagramMarginY:10,leftMargin:150,maxLabelWidth:360,width:150,height:50,boxMargin:10,boxTextMargin:5,noteMargin:10,messageMargin:35,messageAlign:"center",bottomMarginAdj:1,rightAngles:!1,taskFontSize:14,taskFontFamily:'"Open Sans", sans-serif',taskMargin:50,activationWidth:10,textPlacement:"fo",actorColours:["#8FBC8F","#7CFC00","#00FFFF","#20B2AA","#B0E0E6","#FFFFE0"],sectionFills:["#191970","#8B008B","#4B0082","#2F4F4F","#800000","#8B4513","#00008B"],sectionColours:["#fff"],titleColor:"",titleFontFamily:'"trebuchet ms", verdana, arial, sans-serif',titleFontSize:"4ex"},class:{useMaxWidth:!0,titleTopMargin:25,arrowMarkerAbsolute:!1,dividerMargin:10,padding:5,textHeight:10,defaultRenderer:"dagre-wrapper",htmlLabels:!1,hideEmptyMembersBox:!1},state:{useMaxWidth:!0,titleTopMargin:25,dividerMargin:10,sizeUnit:5,padding:8,textHeight:10,titleShift:-15,noteMargin:10,forkWidth:70,forkHeight:7,miniPadding:2,fontSizeFactor:5.02,fontSize:24,labelHeight:16,edgeLengthFactor:"20",compositTitleSize:35,radius:5,defaultRenderer:"dagre-wrapper"},er:{useMaxWidth:!0,titleTopMargin:25,diagramPadding:20,layoutDirection:"TB",minEntityWidth:100,minEntityHeight:75,entityPadding:15,nodeSpacing:140,rankSpacing:80,stroke:"gray",fill:"honeydew",fontSize:12},pie:{useMaxWidth:!0,textPosition:.75},quadrantChart:{useMaxWidth:!0,chartWidth:500,chartHeight:500,titleFontSize:20,titlePadding:10,quadrantPadding:5,xAxisLabelPadding:5,yAxisLabelPadding:5,xAxisLabelFontSize:16,yAxisLabelFontSize:16,quadrantLabelFontSize:16,quadrantTextTopPadding:5,pointTextPadding:5,pointLabelFontSize:12,pointRadius:5,xAxisPosition:"top",yAxisPosition:"left",quadrantInternalBorderStrokeWidth:1,quadrantExternalBorderStrokeWidth:2},xyChart:{useMaxWidth:!0,width:700,height:500,titleFontSize:20,titlePadding:10,showDataLabel:!1,showTitle:!0,xAxis:{$ref:"#/$defs/XYChartAxisConfig",showLabel:!0,labelFontSize:14,labelPadding:5,showTitle:!0,titleFontSize:16,titlePadding:5,showTick:!0,tickLength:5,tickWidth:2,showAxisLine:!0,axisLineWidth:2},yAxis:{$ref:"#/$defs/XYChartAxisConfig",showLabel:!0,labelFontSize:14,labelPadding:5,showTitle:!0,titleFontSize:16,titlePadding:5,showTick:!0,tickLength:5,tickWidth:2,showAxisLine:!0,axisLineWidth:2},chartOrientation:"vertical",plotReservedSpacePercent:50},requirement:{useMaxWidth:!0,rect_fill:"#f9f9f9",text_color:"#333",rect_border_size:"0.5px",rect_border_color:"#bbb",rect_min_width:200,rect_min_height:200,fontSize:14,rect_padding:10,line_height:20},mindmap:{useMaxWidth:!0,padding:10,maxNodeWidth:200,layoutAlgorithm:"cose-bilkent"},kanban:{useMaxWidth:!0,padding:8,sectionWidth:200,ticketBaseUrl:""},timeline:{useMaxWidth:!0,diagramMarginX:50,diagramMarginY:10,leftMargin:150,width:150,height:50,boxMargin:10,boxTextMargin:5,noteMargin:10,messageMargin:35,messageAlign:"center",bottomMarginAdj:1,rightAngles:!1,taskFontSize:14,taskFontFamily:'"Open Sans", sans-serif',taskMargin:50,activationWidth:10,textPlacement:"fo",actorColours:["#8FBC8F","#7CFC00","#00FFFF","#20B2AA","#B0E0E6","#FFFFE0"],sectionFills:["#191970","#8B008B","#4B0082","#2F4F4F","#800000","#8B4513","#00008B"],sectionColours:["#fff"],disableMulticolor:!1},gitGraph:{useMaxWidth:!0,titleTopMargin:25,diagramPadding:8,nodeLabel:{width:75,height:100,x:-25,y:0},mainBranchName:"main",mainBranchOrder:0,showCommitLabel:!0,showBranches:!0,rotateCommitLabel:!0,parallelCommits:!1,arrowMarkerAbsolute:!1},c4:{useMaxWidth:!0,diagramMarginX:50,diagramMarginY:10,c4ShapeMargin:50,c4ShapePadding:20,width:216,height:60,boxMargin:10,c4ShapeInRow:4,nextLinePaddingX:0,c4BoundaryInRow:2,personFontSize:14,personFontFamily:'"Open Sans", sans-serif',personFontWeight:"normal",external_personFontSize:14,external_personFontFamily:'"Open Sans", sans-serif',external_personFontWeight:"normal",systemFontSize:14,systemFontFamily:'"Open Sans", sans-serif',systemFontWeight:"normal",external_systemFontSize:14,external_systemFontFamily:'"Open Sans", sans-serif',external_systemFontWeight:"normal",system_dbFontSize:14,system_dbFontFamily:'"Open Sans", sans-serif',system_dbFontWeight:"normal",external_system_dbFontSize:14,external_system_dbFontFamily:'"Open Sans", sans-serif',external_system_dbFontWeight:"normal",system_queueFontSize:14,system_queueFontFamily:'"Open Sans", sans-serif',system_queueFontWeight:"normal",external_system_queueFontSize:14,external_system_queueFontFamily:'"Open Sans", sans-serif',external_system_queueFontWeight:"normal",boundaryFontSize:14,boundaryFontFamily:'"Open Sans", sans-serif',boundaryFontWeight:"normal",messageFontSize:12,messageFontFamily:'"Open Sans", sans-serif',messageFontWeight:"normal",containerFontSize:14,containerFontFamily:'"Open Sans", sans-serif',containerFontWeight:"normal",external_containerFontSize:14,external_containerFontFamily:'"Open Sans", sans-serif',external_containerFontWeight:"normal",container_dbFontSize:14,container_dbFontFamily:'"Open Sans", sans-serif',container_dbFontWeight:"normal",external_container_dbFontSize:14,external_container_dbFontFamily:'"Open Sans", sans-serif',external_container_dbFontWeight:"normal",container_queueFontSize:14,container_queueFontFamily:'"Open Sans", sans-serif',container_queueFontWeight:"normal",external_container_queueFontSize:14,external_container_queueFontFamily:'"Open Sans", sans-serif',external_container_queueFontWeight:"normal",componentFontSize:14,componentFontFamily:'"Open Sans", sans-serif',componentFontWeight:"normal",external_componentFontSize:14,external_componentFontFamily:'"Open Sans", sans-serif',external_componentFontWeight:"normal",component_dbFontSize:14,component_dbFontFamily:'"Open Sans", sans-serif',component_dbFontWeight:"normal",external_component_dbFontSize:14,external_component_dbFontFamily:'"Open Sans", sans-serif',external_component_dbFontWeight:"normal",component_queueFontSize:14,component_queueFontFamily:'"Open Sans", sans-serif',component_queueFontWeight:"normal",external_component_queueFontSize:14,external_component_queueFontFamily:'"Open Sans", sans-serif',external_component_queueFontWeight:"normal",wrap:!0,wrapPadding:10,person_bg_color:"#08427B",person_border_color:"#073B6F",external_person_bg_color:"#686868",external_person_border_color:"#8A8A8A",system_bg_color:"#1168BD",system_border_color:"#3C7FC0",system_db_bg_color:"#1168BD",system_db_border_color:"#3C7FC0",system_queue_bg_color:"#1168BD",system_queue_border_color:"#3C7FC0",external_system_bg_color:"#999999",external_system_border_color:"#8A8A8A",external_system_db_bg_color:"#999999",external_system_db_border_color:"#8A8A8A",external_system_queue_bg_color:"#999999",external_system_queue_border_color:"#8A8A8A",container_bg_color:"#438DD5",container_border_color:"#3C7FC0",container_db_bg_color:"#438DD5",container_db_border_color:"#3C7FC0",container_queue_bg_color:"#438DD5",container_queue_border_color:"#3C7FC0",external_container_bg_color:"#B3B3B3",external_container_border_color:"#A6A6A6",external_container_db_bg_color:"#B3B3B3",external_container_db_border_color:"#A6A6A6",external_container_queue_bg_color:"#B3B3B3",external_container_queue_border_color:"#A6A6A6",component_bg_color:"#85BBF0",component_border_color:"#78A8D8",component_db_bg_color:"#85BBF0",component_db_border_color:"#78A8D8",component_queue_bg_color:"#85BBF0",component_queue_border_color:"#78A8D8",external_component_bg_color:"#CCCCCC",external_component_border_color:"#BFBFBF",external_component_db_bg_color:"#CCCCCC",external_component_db_border_color:"#BFBFBF",external_component_queue_bg_color:"#CCCCCC",external_component_queue_border_color:"#BFBFBF"},sankey:{useMaxWidth:!0,width:600,height:400,linkColor:"gradient",nodeAlignment:"justify",showValues:!0,prefix:"",suffix:""},block:{useMaxWidth:!0,padding:8},packet:{useMaxWidth:!0,rowHeight:32,bitWidth:32,bitsPerRow:32,showBits:!0,paddingX:5,paddingY:5},architecture:{useMaxWidth:!0,padding:40,iconSize:80,fontSize:16},radar:{useMaxWidth:!0,width:600,height:600,marginTop:50,marginRight:50,marginBottom:50,marginLeft:50,axisScaleFactor:1,axisLabelFactor:1.05,curveTension:.17},theme:"default",look:"classic",handDrawnSeed:0,layout:"dagre",maxTextSize:5e4,maxEdges:500,darkMode:!1,fontFamily:'"trebuchet ms", verdana, arial, sans-serif;',logLevel:5,securityLevel:"strict",startOnLoad:!0,arrowMarkerAbsolute:!1,secure:["secure","securityLevel","startOnLoad","maxTextSize","suppressErrorRendering","maxEdges"],legacyMathML:!1,forceLegacyMathML:!1,deterministicIds:!1,fontSize:16,markdownAutoWrap:!0,suppressErrorRendering:!1},mae={...Yo,deterministicIDSeed:void 0,elk:{mergeEdges:!1,nodePlacementStrategy:"BRANDES_KOEPF",forceNodeModelOrder:!1,considerModelOrder:"NODES_AND_EDGES"},themeCSS:void 0,themeVariables:dl.default.getThemeVariables(),sequence:{...Yo.sequence,messageFont:U(function(){return{fontFamily:this.messageFontFamily,fontSize:this.messageFontSize,fontWeight:this.messageFontWeight}},"messageFont"),noteFont:U(function(){return{fontFamily:this.noteFontFamily,fontSize:this.noteFontSize,fontWeight:this.noteFontWeight}},"noteFont"),actorFont:U(function(){return{fontFamily:this.actorFontFamily,fontSize:this.actorFontSize,fontWeight:this.actorFontWeight}},"actorFont")},class:{hideEmptyMembersBox:!1},gantt:{...Yo.gantt,tickInterval:void 0,useWidth:void 0},c4:{...Yo.c4,useWidth:void 0,personFont:U(function(){return{fontFamily:this.personFontFamily,fontSize:this.personFontSize,fontWeight:this.personFontWeight}},"personFont"),flowchart:{...Yo.flowchart,inheritDir:!1},external_personFont:U(function(){return{fontFamily:this.external_personFontFamily,fontSize:this.external_personFontSize,fontWeight:this.external_personFontWeight}},"external_personFont"),systemFont:U(function(){return{fontFamily:this.systemFontFamily,fontSize:this.systemFontSize,fontWeight:this.systemFontWeight}},"systemFont"),external_systemFont:U(function(){return{fontFamily:this.external_systemFontFamily,fontSize:this.external_systemFontSize,fontWeight:this.external_systemFontWeight}},"external_systemFont"),system_dbFont:U(function(){return{fontFamily:this.system_dbFontFamily,fontSize:this.system_dbFontSize,fontWeight:this.system_dbFontWeight}},"system_dbFont"),external_system_dbFont:U(function(){return{fontFamily:this.external_system_dbFontFamily,fontSize:this.external_system_dbFontSize,fontWeight:this.external_system_dbFontWeight}},"external_system_dbFont"),system_queueFont:U(function(){return{fontFamily:this.system_queueFontFamily,fontSize:this.system_queueFontSize,fontWeight:this.system_queueFontWeight}},"system_queueFont"),external_system_queueFont:U(function(){return{fontFamily:this.external_system_queueFontFamily,fontSize:this.external_system_queueFontSize,fontWeight:this.external_system_queueFontWeight}},"external_system_queueFont"),containerFont:U(function(){return{fontFamily:this.containerFontFamily,fontSize:this.containerFontSize,fontWeight:this.containerFontWeight}},"containerFont"),external_containerFont:U(function(){return{fontFamily:this.external_containerFontFamily,fontSize:this.external_containerFontSize,fontWeight:this.external_containerFontWeight}},"external_containerFont"),container_dbFont:U(function(){return{fontFamily:this.container_dbFontFamily,fontSize:this.container_dbFontSize,fontWeight:this.container_dbFontWeight}},"container_dbFont"),external_container_dbFont:U(function(){return{fontFamily:this.external_container_dbFontFamily,fontSize:this.external_container_dbFontSize,fontWeight:this.external_container_dbFontWeight}},"external_container_dbFont"),container_queueFont:U(function(){return{fontFamily:this.container_queueFontFamily,fontSize:this.container_queueFontSize,fontWeight:this.container_queueFontWeight}},"container_queueFont"),external_container_queueFont:U(function(){return{fontFamily:this.external_container_queueFontFamily,fontSize:this.external_container_queueFontSize,fontWeight:this.external_container_queueFontWeight}},"external_container_queueFont"),componentFont:U(function(){return{fontFamily:this.componentFontFamily,fontSize:this.componentFontSize,fontWeight:this.componentFontWeight}},"componentFont"),external_componentFont:U(function(){return{fontFamily:this.external_componentFontFamily,fontSize:this.external_componentFontSize,fontWeight:this.external_componentFontWeight}},"external_componentFont"),component_dbFont:U(function(){return{fontFamily:this.component_dbFontFamily,fontSize:this.component_dbFontSize,fontWeight:this.component_dbFontWeight}},"component_dbFont"),external_component_dbFont:U(function(){return{fontFamily:this.external_component_dbFontFamily,fontSize:this.external_component_dbFontSize,fontWeight:this.external_component_dbFontWeight}},"external_component_dbFont"),component_queueFont:U(function(){return{fontFamily:this.component_queueFontFamily,fontSize:this.component_queueFontSize,fontWeight:this.component_queueFontWeight}},"component_queueFont"),external_component_queueFont:U(function(){return{fontFamily:this.external_component_queueFontFamily,fontSize:this.external_component_queueFontSize,fontWeight:this.external_component_queueFontWeight}},"external_component_queueFont"),boundaryFont:U(function(){return{fontFamily:this.boundaryFontFamily,fontSize:this.boundaryFontSize,fontWeight:this.boundaryFontWeight}},"boundaryFont"),messageFont:U(function(){return{fontFamily:this.messageFontFamily,fontSize:this.messageFontSize,fontWeight:this.messageFontWeight}},"messageFont")},pie:{...Yo.pie,useWidth:984},xyChart:{...Yo.xyChart,useWidth:void 0},requirement:{...Yo.requirement,useWidth:void 0},packet:{...Yo.packet},radar:{...Yo.radar},treemap:{useMaxWidth:!0,padding:10,diagramPadding:8,showValues:!0,nodeWidth:100,nodeHeight:40,borderWidth:1,valueFontSize:12,labelFontSize:14,valueFormat:","}},gae=U((e,t="")=>Object.keys(e).reduce((r,n)=>Array.isArray(e[n])?r:typeof e[n]=="object"&&e[n]!==null?[...r,t+n,...gae(e[n],"")]:[...r,t+n],[]),"keyify"),iHe=new Set(gae(mae,"")),bae=mae,Iy=U(e=>{if(je.debug("sanitizeDirective called with",e),!(typeof e!="object"||e==null)){if(Array.isArray(e)){e.forEach(t=>Iy(t));return}for(const t of Object.keys(e)){if(je.debug("Checking key",t),t.startsWith("__")||t.includes("proto")||t.includes("constr")||!iHe.has(t)||e[t]==null){je.debug("sanitize deleting key: ",t),delete e[t];continue}if(typeof e[t]=="object"){je.debug("sanitizing object",t),Iy(e[t]);continue}const r=["themeCSS","fontFamily","altFontFamily"];for(const n of r)t.includes(n)&&(je.debug("sanitizing css option",t),e[t]=oHe(e[t]))}if(e.themeVariables)for(const t of Object.keys(e.themeVariables)){const r=e.themeVariables[t];r!=null&&r.match&&!r.match(/^[\d "#%(),.;A-Za-z]+$/)&&(e.themeVariables[t]="")}je.debug("After sanitization",e)}},"sanitizeDirective"),oHe=U(e=>{let t=0,r=0;for(const n of e){if(t<r)return"{ /* ERROR: Unbalanced CSS */ }";n==="{"?t++:n==="}"&&r++}return t!==r?"{ /* ERROR: Unbalanced CSS */ }":e},"sanitizeCss"),Zh=Object.freeze(bae),ci=Qn({},Zh),Oy,Kc=[],Rm=Qn({},Zh),N2=U((e,t)=>{let r=Qn({},e),n={};for(const a of t)Sae(a),n=Qn(n,a);if(r=Qn(r,n),n.theme&&n.theme in dl){const a=Qn({},Oy),i=Qn(a.themeVariables||{},n.themeVariables);r.theme&&r.theme in dl&&(r.themeVariables=dl[r.theme].getThemeVariables(i))}return Rm=r,Eae(Rm),Rm},"updateCurrentConfig"),sHe=U(e=>(ci=Qn({},Zh),ci=Qn(ci,e),e.theme&&dl[e.theme]&&(ci.themeVariables=dl[e.theme].getThemeVariables(e.themeVariables)),N2(ci,Kc),ci),"setSiteConfig"),lHe=U(e=>{Oy=Qn({},e)},"saveConfigFromInitialize"),uHe=U(e=>(ci=Qn(ci,e),N2(ci,Kc),ci),"updateSiteConfig"),yae=U(()=>Qn({},ci),"getSiteConfig"),vae=U(e=>(Eae(e),Qn(Rm,e),Ma()),"setConfig"),Ma=U(()=>Qn({},Rm),"getConfig"),Sae=U(e=>{e&&(["secure",...ci.secure??[]].forEach(t=>{Object.hasOwn(e,t)&&(je.debug(`Denied attempt to modify a secure key ${t}`,e[t]),delete e[t])}),Object.keys(e).forEach(t=>{t.startsWith("__")&&delete e[t]}),Object.keys(e).forEach(t=>{typeof e[t]=="string"&&(e[t].includes("<")||e[t].includes(">")||e[t].includes("url(data:"))&&delete e[t],typeof e[t]=="object"&&Sae(e[t])}))},"sanitize"),cHe=U(e=>{var t;Iy(e),e.fontFamily&&!((t=e.themeVariables)!=null&&t.fontFamily)&&(e.themeVariables={...e.themeVariables,fontFamily:e.fontFamily}),Kc.push(e),N2(ci,Kc)},"addDirective"),By=U((e=ci)=>{Kc=[],N2(e,Kc)},"reset"),dHe={LAZY_LOAD_DEPRECATED:"The configuration options lazyLoadedDiagrams and loadExternalDiagramsAtStartup are deprecated. Please use registerExternalDiagrams instead."},Uz={},hHe=U(e=>{Uz[e]||(je.warn(dHe[e]),Uz[e]=!0)},"issueWarning"),Eae=U(e=>{e&&(e.lazyLoadedDiagrams||e.loadExternalDiagramsAtStartup)&&hHe("LAZY_LOAD_DEPRECATED")},"checkConfig"),wdt=U(()=>{let e={};Oy&&(e=Qn(e,Oy));for(const t of Kc)e=Qn(e,t);return e},"getUserDefinedConfig"),Ug=/<br\s*\/?>/gi,pHe=U(e=>e?kae(e).replace(/\\n/g,"#br#").split("#br#"):[""],"getRows"),fHe=(()=>{let e=!1;return()=>{e||(xae(),e=!0)}})();function xae(){const e="data-temp-href-target";Xh.addHook("beforeSanitizeAttributes",t=>{t.tagName==="A"&&t.hasAttribute("target")&&t.setAttribute(e,t.getAttribute("target")??"")}),Xh.addHook("afterSanitizeAttributes",t=>{t.tagName==="A"&&t.hasAttribute(e)&&(t.setAttribute("target",t.getAttribute(e)??""),t.removeAttribute(e),t.getAttribute("target")==="_blank"&&t.setAttribute("rel","noopener"))})}U(xae,"setupDompurifyHooks");var wae=U(e=>(fHe(),Xh.sanitize(e)),"removeScript"),$z=U((e,t)=>{var r;if(((r=t.flowchart)==null?void 0:r.htmlLabels)!==!1){const n=t.securityLevel;n==="antiscript"||n==="strict"?e=wae(e):n!=="loose"&&(e=kae(e),e=e.replace(/</g,"&lt;").replace(/>/g,"&gt;"),e=e.replace(/=/g,"&equals;"),e=yHe(e))}return e},"sanitizeMore"),uo=U((e,t)=>e&&(t.dompurifyConfig?e=Xh.sanitize($z(e,t),t.dompurifyConfig).toString():e=Xh.sanitize($z(e,t),{FORBID_TAGS:["style"]}).toString(),e),"sanitizeText"),mHe=U((e,t)=>typeof e=="string"?uo(e,t):e.flat().map(r=>uo(r,t)),"sanitizeTextOrArray"),gHe=U(e=>Ug.test(e),"hasBreaks"),bHe=U(e=>e.split(Ug),"splitBreaks"),yHe=U(e=>e.replace(/#br#/g,"<br/>"),"placeholderToBreak"),kae=U(e=>e.replace(Ug,"#br#"),"breakToPlaceholder"),vHe=U(e=>{let t="";return e&&(t=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,t=CSS.escape(t)),t},"getUrl"),ra=U(e=>!(e===!1||["false","null","0"].includes(String(e).trim().toLowerCase())),"evaluate"),SHe=U(function(...e){const t=e.filter(r=>!isNaN(r));return Math.max(...t)},"getMax"),EHe=U(function(...e){const t=e.filter(r=>!isNaN(r));return Math.min(...t)},"getMin"),Gz=U(function(e){const t=e.split(/(,)/),r=[];for(let n=0;n<t.length;n++){let a=t[n];if(a===","&&n>0&&n+1<t.length){const i=t[n-1],o=t[n+1];xHe(i,o)&&(a=i+","+o,n++,r.pop())}r.push(wHe(a))}return r.join("")},"parseGenericTypes"),n6=U((e,t)=>Math.max(0,e.split(t).length-1),"countOccurrence"),xHe=U((e,t)=>{const r=n6(e,"~"),n=n6(t,"~");return r===1&&n===1},"shouldCombineSets"),wHe=U(e=>{const t=n6(e,"~");let r=!1;if(t<=1)return e;t%2!==0&&e.startsWith("~")&&(e=e.substring(1),r=!0);const n=[...e];let a=n.indexOf("~"),i=n.lastIndexOf("~");for(;a!==-1&&i!==-1&&a!==i;)n[a]="<",n[i]=">",a=n.indexOf("~"),i=n.lastIndexOf("~");return r&&n.unshift("~"),n.join("")},"processSet"),jz=U(()=>window.MathMLElement!==void 0,"isMathMLSupported"),a6=/\$\$(.*)\$\$/g,Qh=U(e=>{var t;return(((t=e.match(a6))==null?void 0:t.length)??0)>0},"hasKatex"),kdt=U(async(e,t)=>{const r=document.createElement("div");r.innerHTML=await cN(e,t),r.id="katex-temp",r.style.visibility="hidden",r.style.position="absolute",r.style.top="0";const n=document.querySelector("body");n==null||n.insertAdjacentElement("beforeend",r);const a={width:r.clientWidth,height:r.clientHeight};return r.remove(),a},"calculateMathMLDimensions"),kHe=U(async(e,t)=>{if(!Qh(e))return e;if(!(jz()||t.legacyMathML||t.forceLegacyMathML))return e.replace(a6,"MathML is unsupported in this environment.");{const{default:r}=await Pr(async()=>{const{default:a}=await Promise.resolve().then(()=>ame);return{default:a}},void 0),n=t.forceLegacyMathML||!jz()&&t.legacyMathML?"htmlAndMathml":"mathml";return e.split(Ug).map(a=>Qh(a)?`<div style="display: flex; align-items: center; justify-content: center; white-space: nowrap;">${a}</div>`:`<div>${a}</div>`).join("").replace(a6,(a,i)=>r.renderToString(i,{throwOnError:!0,displayMode:!0,output:n}).replace(/\n/g," ").replace(/<annotation.*<\/annotation>/g,""))}},"renderKatexUnsanitized"),cN=U(async(e,t)=>uo(await kHe(e,t),t),"renderKatexSanitized"),_p={getRows:pHe,sanitizeText:uo,sanitizeTextOrArray:mHe,hasBreaks:gHe,splitBreaks:bHe,lineBreakRegex:Ug,removeScript:wae,getUrl:vHe,evaluate:ra,getMax:SHe,getMin:EHe},CHe=U(function(e,t){for(let r of t)e.attr(r[0],r[1])},"d3Attrs"),THe=U(function(e,t,r){let n=new Map;return r?(n.set("width","100%"),n.set("style",`max-width: ${t}px;`)):(n.set("height",e),n.set("width",t)),n},"calculateSvgSizeAttrs"),Cae=U(function(e,t,r,n){const a=THe(t,r,n);CHe(e,a)},"configureSvgSize"),AHe=U(function(e,t,r,n){const a=t.node().getBBox(),i=a.width,o=a.height;je.info(`SVG bounds: ${i}x${o}`,a);let s=0,l=0;je.info(`Graph bounds: ${s}x${l}`,e),s=i+r*2,l=o+r*2,je.info(`Calculated bounds: ${s}x${l}`),Cae(t,l,s,n);const u=`${a.x-r} ${a.y-r} ${a.width+2*r} ${a.height+2*r}`;t.attr("viewBox",u)},"setupGraphViewbox"),U1={},FHe=U((e,t,r)=>{let n="";return e in U1&&U1[e]?n=U1[e](r):je.warn(`No theme found for ${e}`),` & {
    font-family: ${r.fontFamily};
    font-size: ${r.fontSize};
    fill: ${r.textColor}
  }
  @keyframes edge-animation-frame {
    from {
      stroke-dashoffset: 0;
    }
  }
  @keyframes dash {
    to {
      stroke-dashoffset: 0;
    }
  }
  & .edge-animation-slow {
    stroke-dasharray: 9,5 !important;
    stroke-dashoffset: 900;
    animation: dash 50s linear infinite;
    stroke-linecap: round;
  }
  & .edge-animation-fast {
    stroke-dasharray: 9,5 !important;
    stroke-dashoffset: 900;
    animation: dash 20s linear infinite;
    stroke-linecap: round;
  }
  /* Classes common for multiple diagrams */

  & .error-icon {
    fill: ${r.errorBkgColor};
  }
  & .error-text {
    fill: ${r.errorTextColor};
    stroke: ${r.errorTextColor};
  }

  & .edge-thickness-normal {
    stroke-width: 1px;
  }
  & .edge-thickness-thick {
    stroke-width: 3.5px
  }
  & .edge-pattern-solid {
    stroke-dasharray: 0;
  }
  & .edge-thickness-invisible {
    stroke-width: 0;
    fill: none;
  }
  & .edge-pattern-dashed{
    stroke-dasharray: 3;
  }
  .edge-pattern-dotted {
    stroke-dasharray: 2;
  }

  & .marker {
    fill: ${r.lineColor};
    stroke: ${r.lineColor};
  }
  & .marker.cross {
    stroke: ${r.lineColor};
  }

  & svg {
    font-family: ${r.fontFamily};
    font-size: ${r.fontSize};
  }
   & p {
    margin: 0
   }

  ${n}

  ${t}
`},"getStyles"),_He=U((e,t)=>{t!==void 0&&(U1[e]=t)},"addStylesForDiagram"),RHe=FHe,Tae={};yze(Tae,{clear:()=>DHe,getAccDescription:()=>IHe,getAccTitle:()=>LHe,getDiagramTitle:()=>BHe,setAccDescription:()=>MHe,setAccTitle:()=>NHe,setDiagramTitle:()=>OHe});var dN="",hN="",pN="",fN=U(e=>uo(e,Ma()),"sanitizeText"),DHe=U(()=>{dN="",pN="",hN=""},"clear"),NHe=U(e=>{dN=fN(e).replace(/^\s+/g,"")},"setAccTitle"),LHe=U(()=>dN,"getAccTitle"),MHe=U(e=>{pN=fN(e).replace(/\n\s+/g,`
`)},"setAccDescription"),IHe=U(()=>pN,"getAccDescription"),OHe=U(e=>{hN=fN(e)},"setDiagramTitle"),BHe=U(()=>hN,"getDiagramTitle"),qz=je,PHe=lN,Gr=Ma,Cdt=vae,Tdt=Zh,mN=U(e=>uo(e,Gr()),"sanitizeText"),zHe=AHe,HHe=U(()=>Tae,"getCommonDb"),Py={},zy=U((e,t,r)=>{var n;Py[e]&&qz.warn(`Diagram with id ${e} already registered. Overwriting.`),Py[e]=t,r&&fae(e,r),_He(e,t.styles),(n=t.injectUtils)==null||n.call(t,qz,PHe,Gr,mN,zHe,HHe(),()=>{})},"registerDiagram"),i6=U(e=>{if(e in Py)return Py[e];throw new UHe(e)},"getDiagram"),Mh,UHe=(Mh=class extends Error{constructor(t){super(`Diagram ${t} not found.`)}},U(Mh,"DiagramNotFoundError"),Mh),$He={value:()=>{}};function Aae(){for(var e=0,t=arguments.length,r={},n;e<t;++e){if(!(n=arguments[e]+"")||n in r||/[\s.]/.test(n))throw new Error("illegal type: "+n);r[n]=[]}return new $1(r)}function $1(e){this._=e}function GHe(e,t){return e.trim().split(/^|\s+/).map(function(r){var n="",a=r.indexOf(".");if(a>=0&&(n=r.slice(a+1),r=r.slice(0,a)),r&&!t.hasOwnProperty(r))throw new Error("unknown type: "+r);return{type:r,name:n}})}$1.prototype=Aae.prototype={constructor:$1,on:function(e,t){var r=this._,n=GHe(e+"",r),a,i=-1,o=n.length;if(arguments.length<2){for(;++i<o;)if((a=(e=n[i]).type)&&(a=jHe(r[a],e.name)))return a;return}if(t!=null&&typeof t!="function")throw new Error("invalid callback: "+t);for(;++i<o;)if(a=(e=n[i]).type)r[a]=Wz(r[a],e.name,t);else if(t==null)for(a in r)r[a]=Wz(r[a],e.name,null);return this},copy:function(){var e={},t=this._;for(var r in t)e[r]=t[r].slice();return new $1(e)},call:function(e,t){if((a=arguments.length-2)>0)for(var r=new Array(a),n=0,a,i;n<a;++n)r[n]=arguments[n+2];if(!this._.hasOwnProperty(e))throw new Error("unknown type: "+e);for(i=this._[e],n=0,a=i.length;n<a;++n)i[n].value.apply(t,r)},apply:function(e,t,r){if(!this._.hasOwnProperty(e))throw new Error("unknown type: "+e);for(var n=this._[e],a=0,i=n.length;a<i;++a)n[a].value.apply(t,r)}};function jHe(e,t){for(var r=0,n=e.length,a;r<n;++r)if((a=e[r]).name===t)return a.value}function Wz(e,t,r){for(var n=0,a=e.length;n<a;++n)if(e[n].name===t){e[n]=$He,e=e.slice(0,n).concat(e.slice(n+1));break}return r!=null&&e.push({name:t,value:r}),e}var o6="http://www.w3.org/1999/xhtml";const Vz={svg:"http://www.w3.org/2000/svg",xhtml:o6,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function L2(e){var t=e+="",r=t.indexOf(":");return r>=0&&(t=e.slice(0,r))!=="xmlns"&&(e=e.slice(r+1)),Vz.hasOwnProperty(t)?{space:Vz[t],local:e}:e}function qHe(e){return function(){var t=this.ownerDocument,r=this.namespaceURI;return r===o6&&t.documentElement.namespaceURI===o6?t.createElement(e):t.createElementNS(r,e)}}function WHe(e){return function(){return this.ownerDocument.createElementNS(e.space,e.local)}}function Fae(e){var t=L2(e);return(t.local?WHe:qHe)(t)}function VHe(){}function gN(e){return e==null?VHe:function(){return this.querySelector(e)}}function KHe(e){typeof e!="function"&&(e=gN(e));for(var t=this._groups,r=t.length,n=new Array(r),a=0;a<r;++a)for(var i=t[a],o=i.length,s=n[a]=new Array(o),l,u,d=0;d<o;++d)(l=i[d])&&(u=e.call(l,l.__data__,d,i))&&("__data__"in l&&(u.__data__=l.__data__),s[d]=u);return new Hi(n,this._parents)}function YHe(e){return e==null?[]:Array.isArray(e)?e:Array.from(e)}function JHe(){return[]}function _ae(e){return e==null?JHe:function(){return this.querySelectorAll(e)}}function XHe(e){return function(){return YHe(e.apply(this,arguments))}}function ZHe(e){typeof e=="function"?e=XHe(e):e=_ae(e);for(var t=this._groups,r=t.length,n=[],a=[],i=0;i<r;++i)for(var o=t[i],s=o.length,l,u=0;u<s;++u)(l=o[u])&&(n.push(e.call(l,l.__data__,u,o)),a.push(l));return new Hi(n,a)}function Rae(e){return function(){return this.matches(e)}}function Dae(e){return function(t){return t.matches(e)}}var QHe=Array.prototype.find;function eUe(e){return function(){return QHe.call(this.children,e)}}function tUe(){return this.firstElementChild}function rUe(e){return this.select(e==null?tUe:eUe(typeof e=="function"?e:Dae(e)))}var nUe=Array.prototype.filter;function aUe(){return Array.from(this.children)}function iUe(e){return function(){return nUe.call(this.children,e)}}function oUe(e){return this.selectAll(e==null?aUe:iUe(typeof e=="function"?e:Dae(e)))}function sUe(e){typeof e!="function"&&(e=Rae(e));for(var t=this._groups,r=t.length,n=new Array(r),a=0;a<r;++a)for(var i=t[a],o=i.length,s=n[a]=[],l,u=0;u<o;++u)(l=i[u])&&e.call(l,l.__data__,u,i)&&s.push(l);return new Hi(n,this._parents)}function Nae(e){return new Array(e.length)}function lUe(){return new Hi(this._enter||this._groups.map(Nae),this._parents)}function Hy(e,t){this.ownerDocument=e.ownerDocument,this.namespaceURI=e.namespaceURI,this._next=null,this._parent=e,this.__data__=t}Hy.prototype={constructor:Hy,appendChild:function(e){return this._parent.insertBefore(e,this._next)},insertBefore:function(e,t){return this._parent.insertBefore(e,t)},querySelector:function(e){return this._parent.querySelector(e)},querySelectorAll:function(e){return this._parent.querySelectorAll(e)}};function uUe(e){return function(){return e}}function cUe(e,t,r,n,a,i){for(var o=0,s,l=t.length,u=i.length;o<u;++o)(s=t[o])?(s.__data__=i[o],n[o]=s):r[o]=new Hy(e,i[o]);for(;o<l;++o)(s=t[o])&&(a[o]=s)}function dUe(e,t,r,n,a,i,o){var s,l,u=new Map,d=t.length,p=i.length,f=new Array(d),m;for(s=0;s<d;++s)(l=t[s])&&(f[s]=m=o.call(l,l.__data__,s,t)+"",u.has(m)?a[s]=l:u.set(m,l));for(s=0;s<p;++s)m=o.call(e,i[s],s,i)+"",(l=u.get(m))?(n[s]=l,l.__data__=i[s],u.delete(m)):r[s]=new Hy(e,i[s]);for(s=0;s<d;++s)(l=t[s])&&u.get(f[s])===l&&(a[s]=l)}function hUe(e){return e.__data__}function pUe(e,t){if(!arguments.length)return Array.from(this,hUe);var r=t?dUe:cUe,n=this._parents,a=this._groups;typeof e!="function"&&(e=uUe(e));for(var i=a.length,o=new Array(i),s=new Array(i),l=new Array(i),u=0;u<i;++u){var d=n[u],p=a[u],f=p.length,m=fUe(e.call(d,d&&d.__data__,u,n)),b=m.length,y=s[u]=new Array(b),S=o[u]=new Array(b),w=l[u]=new Array(f);r(d,p,y,S,w,m,t);for(var E=0,k=0,_,R;E<b;++E)if(_=y[E]){for(E>=k&&(k=E+1);!(R=S[k])&&++k<b;);_._next=R||null}}return o=new Hi(o,n),o._enter=s,o._exit=l,o}function fUe(e){return typeof e=="object"&&"length"in e?e:Array.from(e)}function mUe(){return new Hi(this._exit||this._groups.map(Nae),this._parents)}function gUe(e,t,r){var n=this.enter(),a=this,i=this.exit();return typeof e=="function"?(n=e(n),n&&(n=n.selection())):n=n.append(e+""),t!=null&&(a=t(a),a&&(a=a.selection())),r==null?i.remove():r(i),n&&a?n.merge(a).order():a}function bUe(e){for(var t=e.selection?e.selection():e,r=this._groups,n=t._groups,a=r.length,i=n.length,o=Math.min(a,i),s=new Array(a),l=0;l<o;++l)for(var u=r[l],d=n[l],p=u.length,f=s[l]=new Array(p),m,b=0;b<p;++b)(m=u[b]||d[b])&&(f[b]=m);for(;l<a;++l)s[l]=r[l];return new Hi(s,this._parents)}function yUe(){for(var e=this._groups,t=-1,r=e.length;++t<r;)for(var n=e[t],a=n.length-1,i=n[a],o;--a>=0;)(o=n[a])&&(i&&o.compareDocumentPosition(i)^4&&i.parentNode.insertBefore(o,i),i=o);return this}function vUe(e){e||(e=SUe);function t(p,f){return p&&f?e(p.__data__,f.__data__):!p-!f}for(var r=this._groups,n=r.length,a=new Array(n),i=0;i<n;++i){for(var o=r[i],s=o.length,l=a[i]=new Array(s),u,d=0;d<s;++d)(u=o[d])&&(l[d]=u);l.sort(t)}return new Hi(a,this._parents).order()}function SUe(e,t){return e<t?-1:e>t?1:e>=t?0:NaN}function EUe(){var e=arguments[0];return arguments[0]=this,e.apply(null,arguments),this}function xUe(){return Array.from(this)}function wUe(){for(var e=this._groups,t=0,r=e.length;t<r;++t)for(var n=e[t],a=0,i=n.length;a<i;++a){var o=n[a];if(o)return o}return null}function kUe(){let e=0;for(const t of this)++e;return e}function CUe(){return!this.node()}function TUe(e){for(var t=this._groups,r=0,n=t.length;r<n;++r)for(var a=t[r],i=0,o=a.length,s;i<o;++i)(s=a[i])&&e.call(s,s.__data__,i,a);return this}function AUe(e){return function(){this.removeAttribute(e)}}function FUe(e){return function(){this.removeAttributeNS(e.space,e.local)}}function _Ue(e,t){return function(){this.setAttribute(e,t)}}function RUe(e,t){return function(){this.setAttributeNS(e.space,e.local,t)}}function DUe(e,t){return function(){var r=t.apply(this,arguments);r==null?this.removeAttribute(e):this.setAttribute(e,r)}}function NUe(e,t){return function(){var r=t.apply(this,arguments);r==null?this.removeAttributeNS(e.space,e.local):this.setAttributeNS(e.space,e.local,r)}}function LUe(e,t){var r=L2(e);if(arguments.length<2){var n=this.node();return r.local?n.getAttributeNS(r.space,r.local):n.getAttribute(r)}return this.each((t==null?r.local?FUe:AUe:typeof t=="function"?r.local?NUe:DUe:r.local?RUe:_Ue)(r,t))}function Lae(e){return e.ownerDocument&&e.ownerDocument.defaultView||e.document&&e||e.defaultView}function MUe(e){return function(){this.style.removeProperty(e)}}function IUe(e,t,r){return function(){this.style.setProperty(e,t,r)}}function OUe(e,t,r){return function(){var n=t.apply(this,arguments);n==null?this.style.removeProperty(e):this.style.setProperty(e,n,r)}}function BUe(e,t,r){return arguments.length>1?this.each((t==null?MUe:typeof t=="function"?OUe:IUe)(e,t,r??"")):ep(this.node(),e)}function ep(e,t){return e.style.getPropertyValue(t)||Lae(e).getComputedStyle(e,null).getPropertyValue(t)}function PUe(e){return function(){delete this[e]}}function zUe(e,t){return function(){this[e]=t}}function HUe(e,t){return function(){var r=t.apply(this,arguments);r==null?delete this[e]:this[e]=r}}function UUe(e,t){return arguments.length>1?this.each((t==null?PUe:typeof t=="function"?HUe:zUe)(e,t)):this.node()[e]}function Mae(e){return e.trim().split(/^|\s+/)}function bN(e){return e.classList||new Iae(e)}function Iae(e){this._node=e,this._names=Mae(e.getAttribute("class")||"")}Iae.prototype={add:function(e){var t=this._names.indexOf(e);t<0&&(this._names.push(e),this._node.setAttribute("class",this._names.join(" ")))},remove:function(e){var t=this._names.indexOf(e);t>=0&&(this._names.splice(t,1),this._node.setAttribute("class",this._names.join(" ")))},contains:function(e){return this._names.indexOf(e)>=0}};function Oae(e,t){for(var r=bN(e),n=-1,a=t.length;++n<a;)r.add(t[n])}function Bae(e,t){for(var r=bN(e),n=-1,a=t.length;++n<a;)r.remove(t[n])}function $Ue(e){return function(){Oae(this,e)}}function GUe(e){return function(){Bae(this,e)}}function jUe(e,t){return function(){(t.apply(this,arguments)?Oae:Bae)(this,e)}}function qUe(e,t){var r=Mae(e+"");if(arguments.length<2){for(var n=bN(this.node()),a=-1,i=r.length;++a<i;)if(!n.contains(r[a]))return!1;return!0}return this.each((typeof t=="function"?jUe:t?$Ue:GUe)(r,t))}function WUe(){this.textContent=""}function VUe(e){return function(){this.textContent=e}}function KUe(e){return function(){var t=e.apply(this,arguments);this.textContent=t??""}}function YUe(e){return arguments.length?this.each(e==null?WUe:(typeof e=="function"?KUe:VUe)(e)):this.node().textContent}function JUe(){this.innerHTML=""}function XUe(e){return function(){this.innerHTML=e}}function ZUe(e){return function(){var t=e.apply(this,arguments);this.innerHTML=t??""}}function QUe(e){return arguments.length?this.each(e==null?JUe:(typeof e=="function"?ZUe:XUe)(e)):this.node().innerHTML}function e$e(){this.nextSibling&&this.parentNode.appendChild(this)}function t$e(){return this.each(e$e)}function r$e(){this.previousSibling&&this.parentNode.insertBefore(this,this.parentNode.firstChild)}function n$e(){return this.each(r$e)}function a$e(e){var t=typeof e=="function"?e:Fae(e);return this.select(function(){return this.appendChild(t.apply(this,arguments))})}function i$e(){return null}function o$e(e,t){var r=typeof e=="function"?e:Fae(e),n=t==null?i$e:typeof t=="function"?t:gN(t);return this.select(function(){return this.insertBefore(r.apply(this,arguments),n.apply(this,arguments)||null)})}function s$e(){var e=this.parentNode;e&&e.removeChild(this)}function l$e(){return this.each(s$e)}function u$e(){var e=this.cloneNode(!1),t=this.parentNode;return t?t.insertBefore(e,this.nextSibling):e}function c$e(){var e=this.cloneNode(!0),t=this.parentNode;return t?t.insertBefore(e,this.nextSibling):e}function d$e(e){return this.select(e?c$e:u$e)}function h$e(e){return arguments.length?this.property("__data__",e):this.node().__data__}function p$e(e){return function(t){e.call(this,t,this.__data__)}}function f$e(e){return e.trim().split(/^|\s+/).map(function(t){var r="",n=t.indexOf(".");return n>=0&&(r=t.slice(n+1),t=t.slice(0,n)),{type:t,name:r}})}function m$e(e){return function(){var t=this.__on;if(t){for(var r=0,n=-1,a=t.length,i;r<a;++r)i=t[r],(!e.type||i.type===e.type)&&i.name===e.name?this.removeEventListener(i.type,i.listener,i.options):t[++n]=i;++n?t.length=n:delete this.__on}}}function g$e(e,t,r){return function(){var n=this.__on,a,i=p$e(t);if(n){for(var o=0,s=n.length;o<s;++o)if((a=n[o]).type===e.type&&a.name===e.name){this.removeEventListener(a.type,a.listener,a.options),this.addEventListener(a.type,a.listener=i,a.options=r),a.value=t;return}}this.addEventListener(e.type,i,r),a={type:e.type,name:e.name,value:t,listener:i,options:r},n?n.push(a):this.__on=[a]}}function b$e(e,t,r){var n=f$e(e+""),a,i=n.length,o;if(arguments.length<2){var s=this.node().__on;if(s){for(var l=0,u=s.length,d;l<u;++l)for(a=0,d=s[l];a<i;++a)if((o=n[a]).type===d.type&&o.name===d.name)return d.value}return}for(s=t?g$e:m$e,a=0;a<i;++a)this.each(s(n[a],t,r));return this}function Pae(e,t,r){var n=Lae(e),a=n.CustomEvent;typeof a=="function"?a=new a(t,r):(a=n.document.createEvent("Event"),r?(a.initEvent(t,r.bubbles,r.cancelable),a.detail=r.detail):a.initEvent(t,!1,!1)),e.dispatchEvent(a)}function y$e(e,t){return function(){return Pae(this,e,t)}}function v$e(e,t){return function(){return Pae(this,e,t.apply(this,arguments))}}function S$e(e,t){return this.each((typeof t=="function"?v$e:y$e)(e,t))}function*E$e(){for(var e=this._groups,t=0,r=e.length;t<r;++t)for(var n=e[t],a=0,i=n.length,o;a<i;++a)(o=n[a])&&(yield o)}var zae=[null];function Hi(e,t){this._groups=e,this._parents=t}function $g(){return new Hi([[document.documentElement]],zae)}function x$e(){return this}Hi.prototype=$g.prototype={constructor:Hi,select:KHe,selectAll:ZHe,selectChild:rUe,selectChildren:oUe,filter:sUe,data:pUe,enter:lUe,exit:mUe,join:gUe,merge:bUe,selection:x$e,order:yUe,sort:vUe,call:EUe,nodes:xUe,node:wUe,size:kUe,empty:CUe,each:TUe,attr:LUe,style:BUe,property:UUe,classed:qUe,text:YUe,html:QUe,raise:t$e,lower:n$e,append:a$e,insert:o$e,remove:l$e,clone:d$e,datum:h$e,on:b$e,dispatch:S$e,[Symbol.iterator]:E$e};function Lr(e){return typeof e=="string"?new Hi([[document.querySelector(e)]],[document.documentElement]):new Hi([[e]],zae)}function yN(e,t,r){e.prototype=t.prototype=r,r.constructor=e}function Hae(e,t){var r=Object.create(e.prototype);for(var n in t)r[n]=t[n];return r}function Gg(){}var Km=.7,Uy=1/Km,Th="\\s*([+-]?\\d+)\\s*",Ym="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)\\s*",ls="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)%\\s*",w$e=/^#([0-9a-f]{3,8})$/,k$e=new RegExp(`^rgb\\(${Th},${Th},${Th}\\)$`),C$e=new RegExp(`^rgb\\(${ls},${ls},${ls}\\)$`),T$e=new RegExp(`^rgba\\(${Th},${Th},${Th},${Ym}\\)$`),A$e=new RegExp(`^rgba\\(${ls},${ls},${ls},${Ym}\\)$`),F$e=new RegExp(`^hsl\\(${Ym},${ls},${ls}\\)$`),_$e=new RegExp(`^hsla\\(${Ym},${ls},${ls},${Ym}\\)$`),Kz={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};yN(Gg,Jm,{copy(e){return Object.assign(new this.constructor,this,e)},displayable(){return this.rgb().displayable()},hex:Yz,formatHex:Yz,formatHex8:R$e,formatHsl:D$e,formatRgb:Jz,toString:Jz});function Yz(){return this.rgb().formatHex()}function R$e(){return this.rgb().formatHex8()}function D$e(){return Uae(this).formatHsl()}function Jz(){return this.rgb().formatRgb()}function Jm(e){var t,r;return e=(e+"").trim().toLowerCase(),(t=w$e.exec(e))?(r=t[1].length,t=parseInt(t[1],16),r===6?Xz(t):r===3?new hi(t>>8&15|t>>4&240,t>>4&15|t&240,(t&15)<<4|t&15,1):r===8?r1(t>>24&255,t>>16&255,t>>8&255,(t&255)/255):r===4?r1(t>>12&15|t>>8&240,t>>8&15|t>>4&240,t>>4&15|t&240,((t&15)<<4|t&15)/255):null):(t=k$e.exec(e))?new hi(t[1],t[2],t[3],1):(t=C$e.exec(e))?new hi(t[1]*255/100,t[2]*255/100,t[3]*255/100,1):(t=T$e.exec(e))?r1(t[1],t[2],t[3],t[4]):(t=A$e.exec(e))?r1(t[1]*255/100,t[2]*255/100,t[3]*255/100,t[4]):(t=F$e.exec(e))?eH(t[1],t[2]/100,t[3]/100,1):(t=_$e.exec(e))?eH(t[1],t[2]/100,t[3]/100,t[4]):Kz.hasOwnProperty(e)?Xz(Kz[e]):e==="transparent"?new hi(NaN,NaN,NaN,0):null}function Xz(e){return new hi(e>>16&255,e>>8&255,e&255,1)}function r1(e,t,r,n){return n<=0&&(e=t=r=NaN),new hi(e,t,r,n)}function N$e(e){return e instanceof Gg||(e=Jm(e)),e?(e=e.rgb(),new hi(e.r,e.g,e.b,e.opacity)):new hi}function s6(e,t,r,n){return arguments.length===1?N$e(e):new hi(e,t,r,n??1)}function hi(e,t,r,n){this.r=+e,this.g=+t,this.b=+r,this.opacity=+n}yN(hi,s6,Hae(Gg,{brighter(e){return e=e==null?Uy:Math.pow(Uy,e),new hi(this.r*e,this.g*e,this.b*e,this.opacity)},darker(e){return e=e==null?Km:Math.pow(Km,e),new hi(this.r*e,this.g*e,this.b*e,this.opacity)},rgb(){return this},clamp(){return new hi(Nc(this.r),Nc(this.g),Nc(this.b),$y(this.opacity))},displayable(){return-.5<=this.r&&this.r<255.5&&-.5<=this.g&&this.g<255.5&&-.5<=this.b&&this.b<255.5&&0<=this.opacity&&this.opacity<=1},hex:Zz,formatHex:Zz,formatHex8:L$e,formatRgb:Qz,toString:Qz}));function Zz(){return`#${kc(this.r)}${kc(this.g)}${kc(this.b)}`}function L$e(){return`#${kc(this.r)}${kc(this.g)}${kc(this.b)}${kc((isNaN(this.opacity)?1:this.opacity)*255)}`}function Qz(){const e=$y(this.opacity);return`${e===1?"rgb(":"rgba("}${Nc(this.r)}, ${Nc(this.g)}, ${Nc(this.b)}${e===1?")":`, ${e})`}`}function $y(e){return isNaN(e)?1:Math.max(0,Math.min(1,e))}function Nc(e){return Math.max(0,Math.min(255,Math.round(e)||0))}function kc(e){return e=Nc(e),(e<16?"0":"")+e.toString(16)}function eH(e,t,r,n){return n<=0?e=t=r=NaN:r<=0||r>=1?e=t=NaN:t<=0&&(e=NaN),new Fo(e,t,r,n)}function Uae(e){if(e instanceof Fo)return new Fo(e.h,e.s,e.l,e.opacity);if(e instanceof Gg||(e=Jm(e)),!e)return new Fo;if(e instanceof Fo)return e;e=e.rgb();var t=e.r/255,r=e.g/255,n=e.b/255,a=Math.min(t,r,n),i=Math.max(t,r,n),o=NaN,s=i-a,l=(i+a)/2;return s?(t===i?o=(r-n)/s+(r<n)*6:r===i?o=(n-t)/s+2:o=(t-r)/s+4,s/=l<.5?i+a:2-i-a,o*=60):s=l>0&&l<1?0:o,new Fo(o,s,l,e.opacity)}function M$e(e,t,r,n){return arguments.length===1?Uae(e):new Fo(e,t,r,n??1)}function Fo(e,t,r,n){this.h=+e,this.s=+t,this.l=+r,this.opacity=+n}yN(Fo,M$e,Hae(Gg,{brighter(e){return e=e==null?Uy:Math.pow(Uy,e),new Fo(this.h,this.s,this.l*e,this.opacity)},darker(e){return e=e==null?Km:Math.pow(Km,e),new Fo(this.h,this.s,this.l*e,this.opacity)},rgb(){var e=this.h%360+(this.h<0)*360,t=isNaN(e)||isNaN(this.s)?0:this.s,r=this.l,n=r+(r<.5?r:1-r)*t,a=2*r-n;return new hi(Rk(e>=240?e-240:e+120,a,n),Rk(e,a,n),Rk(e<120?e+240:e-120,a,n),this.opacity)},clamp(){return new Fo(tH(this.h),n1(this.s),n1(this.l),$y(this.opacity))},displayable(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1},formatHsl(){const e=$y(this.opacity);return`${e===1?"hsl(":"hsla("}${tH(this.h)}, ${n1(this.s)*100}%, ${n1(this.l)*100}%${e===1?")":`, ${e})`}`}}));function tH(e){return e=(e||0)%360,e<0?e+360:e}function n1(e){return Math.max(0,Math.min(1,e||0))}function Rk(e,t,r){return(e<60?t+(r-t)*e/60:e<180?r:e<240?t+(r-t)*(240-e)/60:t)*255}const vN=e=>()=>e;function $ae(e,t){return function(r){return e+r*t}}function I$e(e,t,r){return e=Math.pow(e,r),t=Math.pow(t,r)-e,r=1/r,function(n){return Math.pow(e+n*t,r)}}function Adt(e,t){var r=t-e;return r?$ae(e,r>180||r<-180?r-360*Math.round(r/360):r):vN(isNaN(e)?t:e)}function O$e(e){return(e=+e)==1?Gae:function(t,r){return r-t?I$e(t,r,e):vN(isNaN(t)?r:t)}}function Gae(e,t){var r=t-e;return r?$ae(e,r):vN(isNaN(e)?t:e)}const rH=(function e(t){var r=O$e(t);function n(a,i){var o=r((a=s6(a)).r,(i=s6(i)).r),s=r(a.g,i.g),l=r(a.b,i.b),u=Gae(a.opacity,i.opacity);return function(d){return a.r=o(d),a.g=s(d),a.b=l(d),a.opacity=u(d),a+""}}return n.gamma=e,n})(1);function mu(e,t){return e=+e,t=+t,function(r){return e*(1-r)+t*r}}var l6=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,Dk=new RegExp(l6.source,"g");function B$e(e){return function(){return e}}function P$e(e){return function(t){return e(t)+""}}function z$e(e,t){var r=l6.lastIndex=Dk.lastIndex=0,n,a,i,o=-1,s=[],l=[];for(e=e+"",t=t+"";(n=l6.exec(e))&&(a=Dk.exec(t));)(i=a.index)>r&&(i=t.slice(r,i),s[o]?s[o]+=i:s[++o]=i),(n=n[0])===(a=a[0])?s[o]?s[o]+=a:s[++o]=a:(s[++o]=null,l.push({i:o,x:mu(n,a)})),r=Dk.lastIndex;return r<t.length&&(i=t.slice(r),s[o]?s[o]+=i:s[++o]=i),s.length<2?l[0]?P$e(l[0].x):B$e(t):(t=l.length,function(u){for(var d=0,p;d<t;++d)s[(p=l[d]).i]=p.x(u);return s.join("")})}var nH=180/Math.PI,u6={translateX:0,translateY:0,rotate:0,skewX:0,scaleX:1,scaleY:1};function jae(e,t,r,n,a,i){var o,s,l;return(o=Math.sqrt(e*e+t*t))&&(e/=o,t/=o),(l=e*r+t*n)&&(r-=e*l,n-=t*l),(s=Math.sqrt(r*r+n*n))&&(r/=s,n/=s,l/=s),e*n<t*r&&(e=-e,t=-t,l=-l,o=-o),{translateX:a,translateY:i,rotate:Math.atan2(t,e)*nH,skewX:Math.atan(l)*nH,scaleX:o,scaleY:s}}var a1;function H$e(e){const t=new(typeof DOMMatrix=="function"?DOMMatrix:WebKitCSSMatrix)(e+"");return t.isIdentity?u6:jae(t.a,t.b,t.c,t.d,t.e,t.f)}function U$e(e){return e==null||(a1||(a1=document.createElementNS("http://www.w3.org/2000/svg","g")),a1.setAttribute("transform",e),!(e=a1.transform.baseVal.consolidate()))?u6:(e=e.matrix,jae(e.a,e.b,e.c,e.d,e.e,e.f))}function qae(e,t,r,n){function a(u){return u.length?u.pop()+" ":""}function i(u,d,p,f,m,b){if(u!==p||d!==f){var y=m.push("translate(",null,t,null,r);b.push({i:y-4,x:mu(u,p)},{i:y-2,x:mu(d,f)})}else(p||f)&&m.push("translate("+p+t+f+r)}function o(u,d,p,f){u!==d?(u-d>180?d+=360:d-u>180&&(u+=360),f.push({i:p.push(a(p)+"rotate(",null,n)-2,x:mu(u,d)})):d&&p.push(a(p)+"rotate("+d+n)}function s(u,d,p,f){u!==d?f.push({i:p.push(a(p)+"skewX(",null,n)-2,x:mu(u,d)}):d&&p.push(a(p)+"skewX("+d+n)}function l(u,d,p,f,m,b){if(u!==p||d!==f){var y=m.push(a(m)+"scale(",null,",",null,")");b.push({i:y-4,x:mu(u,p)},{i:y-2,x:mu(d,f)})}else(p!==1||f!==1)&&m.push(a(m)+"scale("+p+","+f+")")}return function(u,d){var p=[],f=[];return u=e(u),d=e(d),i(u.translateX,u.translateY,d.translateX,d.translateY,p,f),o(u.rotate,d.rotate,p,f),s(u.skewX,d.skewX,p,f),l(u.scaleX,u.scaleY,d.scaleX,d.scaleY,p,f),u=d=null,function(m){for(var b=-1,y=f.length,S;++b<y;)p[(S=f[b]).i]=S.x(m);return p.join("")}}}var $$e=qae(H$e,"px, ","px)","deg)"),G$e=qae(U$e,", ",")",")"),tp=0,lm=0,Kf=0,Wae=1e3,Gy,um,jy=0,Yc=0,M2=0,Xm=typeof performance=="object"&&performance.now?performance:Date,Vae=typeof window=="object"&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(e){setTimeout(e,17)};function SN(){return Yc||(Vae(j$e),Yc=Xm.now()+M2)}function j$e(){Yc=0}function qy(){this._call=this._time=this._next=null}qy.prototype=Kae.prototype={constructor:qy,restart:function(e,t,r){if(typeof e!="function")throw new TypeError("callback is not a function");r=(r==null?SN():+r)+(t==null?0:+t),!this._next&&um!==this&&(um?um._next=this:Gy=this,um=this),this._call=e,this._time=r,c6()},stop:function(){this._call&&(this._call=null,this._time=1/0,c6())}};function Kae(e,t,r){var n=new qy;return n.restart(e,t,r),n}function q$e(){SN(),++tp;for(var e=Gy,t;e;)(t=Yc-e._time)>=0&&e._call.call(void 0,t),e=e._next;--tp}function aH(){Yc=(jy=Xm.now())+M2,tp=lm=0;try{q$e()}finally{tp=0,V$e(),Yc=0}}function W$e(){var e=Xm.now(),t=e-jy;t>Wae&&(M2-=t,jy=e)}function V$e(){for(var e,t=Gy,r,n=1/0;t;)t._call?(n>t._time&&(n=t._time),e=t,t=t._next):(r=t._next,t._next=null,t=e?e._next=r:Gy=r);um=e,c6(n)}function c6(e){if(!tp){lm&&(lm=clearTimeout(lm));var t=e-Yc;t>24?(e<1/0&&(lm=setTimeout(aH,e-Xm.now()-M2)),Kf&&(Kf=clearInterval(Kf))):(Kf||(jy=Xm.now(),Kf=setInterval(W$e,Wae)),tp=1,Vae(aH))}}function iH(e,t,r){var n=new qy;return t=t==null?0:+t,n.restart(a=>{n.stop(),e(a+t)},t,r),n}var K$e=Aae("start","end","cancel","interrupt"),Y$e=[],Yae=0,oH=1,d6=2,G1=3,sH=4,h6=5,j1=6;function I2(e,t,r,n,a,i){var o=e.__transition;if(!o)e.__transition={};else if(r in o)return;J$e(e,r,{name:t,index:n,group:a,on:K$e,tween:Y$e,time:i.time,delay:i.delay,duration:i.duration,ease:i.ease,timer:null,state:Yae})}function EN(e,t){var r=Uo(e,t);if(r.state>Yae)throw new Error("too late; already scheduled");return r}function Es(e,t){var r=Uo(e,t);if(r.state>G1)throw new Error("too late; already running");return r}function Uo(e,t){var r=e.__transition;if(!r||!(r=r[t]))throw new Error("transition not found");return r}function J$e(e,t,r){var n=e.__transition,a;n[t]=r,r.timer=Kae(i,0,r.time);function i(u){r.state=oH,r.timer.restart(o,r.delay,r.time),r.delay<=u&&o(u-r.delay)}function o(u){var d,p,f,m;if(r.state!==oH)return l();for(d in n)if(m=n[d],m.name===r.name){if(m.state===G1)return iH(o);m.state===sH?(m.state=j1,m.timer.stop(),m.on.call("interrupt",e,e.__data__,m.index,m.group),delete n[d]):+d<t&&(m.state=j1,m.timer.stop(),m.on.call("cancel",e,e.__data__,m.index,m.group),delete n[d])}if(iH(function(){r.state===G1&&(r.state=sH,r.timer.restart(s,r.delay,r.time),s(u))}),r.state=d6,r.on.call("start",e,e.__data__,r.index,r.group),r.state===d6){for(r.state=G1,a=new Array(f=r.tween.length),d=0,p=-1;d<f;++d)(m=r.tween[d].value.call(e,e.__data__,r.index,r.group))&&(a[++p]=m);a.length=p+1}}function s(u){for(var d=u<r.duration?r.ease.call(null,u/r.duration):(r.timer.restart(l),r.state=h6,1),p=-1,f=a.length;++p<f;)a[p].call(e,d);r.state===h6&&(r.on.call("end",e,e.__data__,r.index,r.group),l())}function l(){r.state=j1,r.timer.stop(),delete n[t];for(var u in n)return;delete e.__transition}}function X$e(e,t){var r=e.__transition,n,a,i=!0,o;if(r){t=t==null?null:t+"";for(o in r){if((n=r[o]).name!==t){i=!1;continue}a=n.state>d6&&n.state<h6,n.state=j1,n.timer.stop(),n.on.call(a?"interrupt":"cancel",e,e.__data__,n.index,n.group),delete r[o]}i&&delete e.__transition}}function Z$e(e){return this.each(function(){X$e(this,e)})}function Q$e(e,t){var r,n;return function(){var a=Es(this,e),i=a.tween;if(i!==r){n=r=i;for(var o=0,s=n.length;o<s;++o)if(n[o].name===t){n=n.slice(),n.splice(o,1);break}}a.tween=n}}function eGe(e,t,r){var n,a;if(typeof r!="function")throw new Error;return function(){var i=Es(this,e),o=i.tween;if(o!==n){a=(n=o).slice();for(var s={name:t,value:r},l=0,u=a.length;l<u;++l)if(a[l].name===t){a[l]=s;break}l===u&&a.push(s)}i.tween=a}}function tGe(e,t){var r=this._id;if(e+="",arguments.length<2){for(var n=Uo(this.node(),r).tween,a=0,i=n.length,o;a<i;++a)if((o=n[a]).name===e)return o.value;return null}return this.each((t==null?Q$e:eGe)(r,e,t))}function xN(e,t,r){var n=e._id;return e.each(function(){var a=Es(this,n);(a.value||(a.value={}))[t]=r.apply(this,arguments)}),function(a){return Uo(a,n).value[t]}}function Jae(e,t){var r;return(typeof t=="number"?mu:t instanceof Jm?rH:(r=Jm(t))?(t=r,rH):z$e)(e,t)}function rGe(e){return function(){this.removeAttribute(e)}}function nGe(e){return function(){this.removeAttributeNS(e.space,e.local)}}function aGe(e,t,r){var n,a=r+"",i;return function(){var o=this.getAttribute(e);return o===a?null:o===n?i:i=t(n=o,r)}}function iGe(e,t,r){var n,a=r+"",i;return function(){var o=this.getAttributeNS(e.space,e.local);return o===a?null:o===n?i:i=t(n=o,r)}}function oGe(e,t,r){var n,a,i;return function(){var o,s=r(this),l;return s==null?void this.removeAttribute(e):(o=this.getAttribute(e),l=s+"",o===l?null:o===n&&l===a?i:(a=l,i=t(n=o,s)))}}function sGe(e,t,r){var n,a,i;return function(){var o,s=r(this),l;return s==null?void this.removeAttributeNS(e.space,e.local):(o=this.getAttributeNS(e.space,e.local),l=s+"",o===l?null:o===n&&l===a?i:(a=l,i=t(n=o,s)))}}function lGe(e,t){var r=L2(e),n=r==="transform"?G$e:Jae;return this.attrTween(e,typeof t=="function"?(r.local?sGe:oGe)(r,n,xN(this,"attr."+e,t)):t==null?(r.local?nGe:rGe)(r):(r.local?iGe:aGe)(r,n,t))}function uGe(e,t){return function(r){this.setAttribute(e,t.call(this,r))}}function cGe(e,t){return function(r){this.setAttributeNS(e.space,e.local,t.call(this,r))}}function dGe(e,t){var r,n;function a(){var i=t.apply(this,arguments);return i!==n&&(r=(n=i)&&cGe(e,i)),r}return a._value=t,a}function hGe(e,t){var r,n;function a(){var i=t.apply(this,arguments);return i!==n&&(r=(n=i)&&uGe(e,i)),r}return a._value=t,a}function pGe(e,t){var r="attr."+e;if(arguments.length<2)return(r=this.tween(r))&&r._value;if(t==null)return this.tween(r,null);if(typeof t!="function")throw new Error;var n=L2(e);return this.tween(r,(n.local?dGe:hGe)(n,t))}function fGe(e,t){return function(){EN(this,e).delay=+t.apply(this,arguments)}}function mGe(e,t){return t=+t,function(){EN(this,e).delay=t}}function gGe(e){var t=this._id;return arguments.length?this.each((typeof e=="function"?fGe:mGe)(t,e)):Uo(this.node(),t).delay}function bGe(e,t){return function(){Es(this,e).duration=+t.apply(this,arguments)}}function yGe(e,t){return t=+t,function(){Es(this,e).duration=t}}function vGe(e){var t=this._id;return arguments.length?this.each((typeof e=="function"?bGe:yGe)(t,e)):Uo(this.node(),t).duration}function SGe(e,t){if(typeof t!="function")throw new Error;return function(){Es(this,e).ease=t}}function EGe(e){var t=this._id;return arguments.length?this.each(SGe(t,e)):Uo(this.node(),t).ease}function xGe(e,t){return function(){var r=t.apply(this,arguments);if(typeof r!="function")throw new Error;Es(this,e).ease=r}}function wGe(e){if(typeof e!="function")throw new Error;return this.each(xGe(this._id,e))}function kGe(e){typeof e!="function"&&(e=Rae(e));for(var t=this._groups,r=t.length,n=new Array(r),a=0;a<r;++a)for(var i=t[a],o=i.length,s=n[a]=[],l,u=0;u<o;++u)(l=i[u])&&e.call(l,l.__data__,u,i)&&s.push(l);return new kl(n,this._parents,this._name,this._id)}function CGe(e){if(e._id!==this._id)throw new Error;for(var t=this._groups,r=e._groups,n=t.length,a=r.length,i=Math.min(n,a),o=new Array(n),s=0;s<i;++s)for(var l=t[s],u=r[s],d=l.length,p=o[s]=new Array(d),f,m=0;m<d;++m)(f=l[m]||u[m])&&(p[m]=f);for(;s<n;++s)o[s]=t[s];return new kl(o,this._parents,this._name,this._id)}function TGe(e){return(e+"").trim().split(/^|\s+/).every(function(t){var r=t.indexOf(".");return r>=0&&(t=t.slice(0,r)),!t||t==="start"})}function AGe(e,t,r){var n,a,i=TGe(t)?EN:Es;return function(){var o=i(this,e),s=o.on;s!==n&&(a=(n=s).copy()).on(t,r),o.on=a}}function FGe(e,t){var r=this._id;return arguments.length<2?Uo(this.node(),r).on.on(e):this.each(AGe(r,e,t))}function _Ge(e){return function(){var t=this.parentNode;for(var r in this.__transition)if(+r!==e)return;t&&t.removeChild(this)}}function RGe(){return this.on("end.remove",_Ge(this._id))}function DGe(e){var t=this._name,r=this._id;typeof e!="function"&&(e=gN(e));for(var n=this._groups,a=n.length,i=new Array(a),o=0;o<a;++o)for(var s=n[o],l=s.length,u=i[o]=new Array(l),d,p,f=0;f<l;++f)(d=s[f])&&(p=e.call(d,d.__data__,f,s))&&("__data__"in d&&(p.__data__=d.__data__),u[f]=p,I2(u[f],t,r,f,u,Uo(d,r)));return new kl(i,this._parents,t,r)}function NGe(e){var t=this._name,r=this._id;typeof e!="function"&&(e=_ae(e));for(var n=this._groups,a=n.length,i=[],o=[],s=0;s<a;++s)for(var l=n[s],u=l.length,d,p=0;p<u;++p)if(d=l[p]){for(var f=e.call(d,d.__data__,p,l),m,b=Uo(d,r),y=0,S=f.length;y<S;++y)(m=f[y])&&I2(m,t,r,y,f,b);i.push(f),o.push(d)}return new kl(i,o,t,r)}var LGe=$g.prototype.constructor;function MGe(){return new LGe(this._groups,this._parents)}function IGe(e,t){var r,n,a;return function(){var i=ep(this,e),o=(this.style.removeProperty(e),ep(this,e));return i===o?null:i===r&&o===n?a:a=t(r=i,n=o)}}function Xae(e){return function(){this.style.removeProperty(e)}}function OGe(e,t,r){var n,a=r+"",i;return function(){var o=ep(this,e);return o===a?null:o===n?i:i=t(n=o,r)}}function BGe(e,t,r){var n,a,i;return function(){var o=ep(this,e),s=r(this),l=s+"";return s==null&&(l=s=(this.style.removeProperty(e),ep(this,e))),o===l?null:o===n&&l===a?i:(a=l,i=t(n=o,s))}}function PGe(e,t){var r,n,a,i="style."+t,o="end."+i,s;return function(){var l=Es(this,e),u=l.on,d=l.value[i]==null?s||(s=Xae(t)):void 0;(u!==r||a!==d)&&(n=(r=u).copy()).on(o,a=d),l.on=n}}function zGe(e,t,r){var n=(e+="")=="transform"?$$e:Jae;return t==null?this.styleTween(e,IGe(e,n)).on("end.style."+e,Xae(e)):typeof t=="function"?this.styleTween(e,BGe(e,n,xN(this,"style."+e,t))).each(PGe(this._id,e)):this.styleTween(e,OGe(e,n,t),r).on("end.style."+e,null)}function HGe(e,t,r){return function(n){this.style.setProperty(e,t.call(this,n),r)}}function UGe(e,t,r){var n,a;function i(){var o=t.apply(this,arguments);return o!==a&&(n=(a=o)&&HGe(e,o,r)),n}return i._value=t,i}function $Ge(e,t,r){var n="style."+(e+="");if(arguments.length<2)return(n=this.tween(n))&&n._value;if(t==null)return this.tween(n,null);if(typeof t!="function")throw new Error;return this.tween(n,UGe(e,t,r??""))}function GGe(e){return function(){this.textContent=e}}function jGe(e){return function(){var t=e(this);this.textContent=t??""}}function qGe(e){return this.tween("text",typeof e=="function"?jGe(xN(this,"text",e)):GGe(e==null?"":e+""))}function WGe(e){return function(t){this.textContent=e.call(this,t)}}function VGe(e){var t,r;function n(){var a=e.apply(this,arguments);return a!==r&&(t=(r=a)&&WGe(a)),t}return n._value=e,n}function KGe(e){var t="text";if(arguments.length<1)return(t=this.tween(t))&&t._value;if(e==null)return this.tween(t,null);if(typeof e!="function")throw new Error;return this.tween(t,VGe(e))}function YGe(){for(var e=this._name,t=this._id,r=Zae(),n=this._groups,a=n.length,i=0;i<a;++i)for(var o=n[i],s=o.length,l,u=0;u<s;++u)if(l=o[u]){var d=Uo(l,t);I2(l,e,r,u,o,{time:d.time+d.delay+d.duration,delay:0,duration:d.duration,ease:d.ease})}return new kl(n,this._parents,e,r)}function JGe(){var e,t,r=this,n=r._id,a=r.size();return new Promise(function(i,o){var s={value:o},l={value:function(){--a===0&&i()}};r.each(function(){var u=Es(this,n),d=u.on;d!==e&&(t=(e=d).copy(),t._.cancel.push(s),t._.interrupt.push(s),t._.end.push(l)),u.on=t}),a===0&&i()})}var XGe=0;function kl(e,t,r,n){this._groups=e,this._parents=t,this._name=r,this._id=n}function Zae(){return++XGe}var Ys=$g.prototype;kl.prototype={constructor:kl,select:DGe,selectAll:NGe,selectChild:Ys.selectChild,selectChildren:Ys.selectChildren,filter:kGe,merge:CGe,selection:MGe,transition:YGe,call:Ys.call,nodes:Ys.nodes,node:Ys.node,size:Ys.size,empty:Ys.empty,each:Ys.each,on:FGe,attr:lGe,attrTween:pGe,style:zGe,styleTween:$Ge,text:qGe,textTween:KGe,remove:RGe,tween:tGe,delay:gGe,duration:vGe,ease:EGe,easeVarying:wGe,end:JGe,[Symbol.iterator]:Ys[Symbol.iterator]};function ZGe(e){return((e*=2)<=1?e*e*e:(e-=2)*e*e+2)/2}var QGe={time:null,delay:0,duration:250,ease:ZGe};function eje(e,t){for(var r;!(r=e.__transition)||!(r=r[t]);)if(!(e=e.parentNode))throw new Error(`transition ${t} not found`);return r}function tje(e){var t,r;e instanceof kl?(t=e._id,e=e._name):(t=Zae(),(r=QGe).time=SN(),e=e==null?null:e+"");for(var n=this._groups,a=n.length,i=0;i<a;++i)for(var o=n[i],s=o.length,l,u=0;u<s;++u)(l=o[u])&&I2(l,e,t,u,o,r||eje(l,t));return new kl(n,this._parents,e,t)}$g.prototype.interrupt=Z$e;$g.prototype.transition=tje;const p6=Math.PI,f6=2*p6,bc=1e-6,rje=f6-bc;function Qae(e){this._+=e[0];for(let t=1,r=e.length;t<r;++t)this._+=arguments[t]+e[t]}function nje(e){let t=Math.floor(e);if(!(t>=0))throw new Error(`invalid digits: ${e}`);if(t>15)return Qae;const r=10**t;return function(n){this._+=n[0];for(let a=1,i=n.length;a<i;++a)this._+=Math.round(arguments[a]*r)/r+n[a]}}class aje{constructor(t){this._x0=this._y0=this._x1=this._y1=null,this._="",this._append=t==null?Qae:nje(t)}moveTo(t,r){this._append`M${this._x0=this._x1=+t},${this._y0=this._y1=+r}`}closePath(){this._x1!==null&&(this._x1=this._x0,this._y1=this._y0,this._append`Z`)}lineTo(t,r){this._append`L${this._x1=+t},${this._y1=+r}`}quadraticCurveTo(t,r,n,a){this._append`Q${+t},${+r},${this._x1=+n},${this._y1=+a}`}bezierCurveTo(t,r,n,a,i,o){this._append`C${+t},${+r},${+n},${+a},${this._x1=+i},${this._y1=+o}`}arcTo(t,r,n,a,i){if(t=+t,r=+r,n=+n,a=+a,i=+i,i<0)throw new Error(`negative radius: ${i}`);let o=this._x1,s=this._y1,l=n-t,u=a-r,d=o-t,p=s-r,f=d*d+p*p;if(this._x1===null)this._append`M${this._x1=t},${this._y1=r}`;else if(f>bc)if(!(Math.abs(p*l-u*d)>bc)||!i)this._append`L${this._x1=t},${this._y1=r}`;else{let m=n-o,b=a-s,y=l*l+u*u,S=m*m+b*b,w=Math.sqrt(y),E=Math.sqrt(f),k=i*Math.tan((p6-Math.acos((y+f-S)/(2*w*E)))/2),_=k/E,R=k/w;Math.abs(_-1)>bc&&this._append`L${t+_*d},${r+_*p}`,this._append`A${i},${i},0,0,${+(p*m>d*b)},${this._x1=t+R*l},${this._y1=r+R*u}`}}arc(t,r,n,a,i,o){if(t=+t,r=+r,n=+n,o=!!o,n<0)throw new Error(`negative radius: ${n}`);let s=n*Math.cos(a),l=n*Math.sin(a),u=t+s,d=r+l,p=1^o,f=o?a-i:i-a;this._x1===null?this._append`M${u},${d}`:(Math.abs(this._x1-u)>bc||Math.abs(this._y1-d)>bc)&&this._append`L${u},${d}`,n&&(f<0&&(f=f%f6+f6),f>rje?this._append`A${n},${n},0,1,${p},${t-s},${r-l}A${n},${n},0,1,${p},${this._x1=u},${this._y1=d}`:f>bc&&this._append`A${n},${n},0,${+(f>=p6)},${p},${this._x1=t+n*Math.cos(i)},${this._y1=r+n*Math.sin(i)}`)}rect(t,r,n,a){this._append`M${this._x0=this._x1=+t},${this._y0=this._y1=+r}h${n=+n}v${+a}h${-n}Z`}toString(){return this._}}function ih(e){return function(){return e}}const Fdt=Math.abs,_dt=Math.atan2,Rdt=Math.cos,Ddt=Math.max,Ndt=Math.min,Ldt=Math.sin,Mdt=Math.sqrt,lH=1e-12,wN=Math.PI,uH=wN/2,Idt=2*wN;function Odt(e){return e>1?0:e<-1?wN:Math.acos(e)}function Bdt(e){return e>=1?uH:e<=-1?-uH:Math.asin(e)}function ije(e){let t=3;return e.digits=function(r){if(!arguments.length)return t;if(r==null)t=null;else{const n=Math.floor(r);if(!(n>=0))throw new RangeError(`invalid digits: ${r}`);t=n}return e},()=>new aje(t)}function oje(e){return typeof e=="object"&&"length"in e?e:Array.from(e)}function eie(e){this._context=e}eie.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;default:this._context.lineTo(e,t);break}}};function Wy(e){return new eie(e)}function sje(e){return e[0]}function lje(e){return e[1]}function uje(e,t){var r=ih(!0),n=null,a=Wy,i=null,o=ije(s);e=typeof e=="function"?e:e===void 0?sje:ih(e),t=typeof t=="function"?t:t===void 0?lje:ih(t);function s(l){var u,d=(l=oje(l)).length,p,f=!1,m;for(n==null&&(i=a(m=o())),u=0;u<=d;++u)!(u<d&&r(p=l[u],u,l))===f&&((f=!f)?i.lineStart():i.lineEnd()),f&&i.point(+e(p,u,l),+t(p,u,l));if(m)return i=null,m+""||null}return s.x=function(l){return arguments.length?(e=typeof l=="function"?l:ih(+l),s):e},s.y=function(l){return arguments.length?(t=typeof l=="function"?l:ih(+l),s):t},s.defined=function(l){return arguments.length?(r=typeof l=="function"?l:ih(!!l),s):r},s.curve=function(l){return arguments.length?(a=l,n!=null&&(i=a(n)),s):a},s.context=function(l){return arguments.length?(l==null?n=i=null:i=a(n=l),s):n},s}class tie{constructor(t,r){this._context=t,this._x=r}areaStart(){this._line=0}areaEnd(){this._line=NaN}lineStart(){this._point=0}lineEnd(){(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line}point(t,r){switch(t=+t,r=+r,this._point){case 0:{this._point=1,this._line?this._context.lineTo(t,r):this._context.moveTo(t,r);break}case 1:this._point=2;default:{this._x?this._context.bezierCurveTo(this._x0=(this._x0+t)/2,this._y0,this._x0,r,t,r):this._context.bezierCurveTo(this._x0,this._y0=(this._y0+r)/2,t,this._y0,t,r);break}}this._x0=t,this._y0=r}}function rie(e){return new tie(e,!0)}function nie(e){return new tie(e,!1)}function Lu(){}function Vy(e,t,r){e._context.bezierCurveTo((2*e._x0+e._x1)/3,(2*e._y0+e._y1)/3,(e._x0+2*e._x1)/3,(e._y0+2*e._y1)/3,(e._x0+4*e._x1+t)/6,(e._y0+4*e._y1+r)/6)}function O2(e){this._context=e}O2.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){switch(this._point){case 3:Vy(this,this._x1,this._y1);case 2:this._context.lineTo(this._x1,this._y1);break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;break;case 2:this._point=3,this._context.lineTo((5*this._x0+this._x1)/6,(5*this._y0+this._y1)/6);default:Vy(this,e,t);break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t}};function q1(e){return new O2(e)}function aie(e){this._context=e}aie.prototype={areaStart:Lu,areaEnd:Lu,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._y0=this._y1=this._y2=this._y3=this._y4=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:{this._context.moveTo(this._x2,this._y2),this._context.closePath();break}case 2:{this._context.moveTo((this._x2+2*this._x3)/3,(this._y2+2*this._y3)/3),this._context.lineTo((this._x3+2*this._x2)/3,(this._y3+2*this._y2)/3),this._context.closePath();break}case 3:{this.point(this._x2,this._y2),this.point(this._x3,this._y3),this.point(this._x4,this._y4);break}}},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._x2=e,this._y2=t;break;case 1:this._point=2,this._x3=e,this._y3=t;break;case 2:this._point=3,this._x4=e,this._y4=t,this._context.moveTo((this._x0+4*this._x1+e)/6,(this._y0+4*this._y1+t)/6);break;default:Vy(this,e,t);break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t}};function cje(e){return new aie(e)}function iie(e){this._context=e}iie.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===3)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3;var r=(this._x0+4*this._x1+e)/6,n=(this._y0+4*this._y1+t)/6;this._line?this._context.lineTo(r,n):this._context.moveTo(r,n);break;case 3:this._point=4;default:Vy(this,e,t);break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t}};function dje(e){return new iie(e)}function oie(e,t){this._basis=new O2(e),this._beta=t}oie.prototype={lineStart:function(){this._x=[],this._y=[],this._basis.lineStart()},lineEnd:function(){var e=this._x,t=this._y,r=e.length-1;if(r>0)for(var n=e[0],a=t[0],i=e[r]-n,o=t[r]-a,s=-1,l;++s<=r;)l=s/r,this._basis.point(this._beta*e[s]+(1-this._beta)*(n+l*i),this._beta*t[s]+(1-this._beta)*(a+l*o));this._x=this._y=null,this._basis.lineEnd()},point:function(e,t){this._x.push(+e),this._y.push(+t)}};const hje=(function e(t){function r(n){return t===1?new O2(n):new oie(n,t)}return r.beta=function(n){return e(+n)},r})(.85);function Ky(e,t,r){e._context.bezierCurveTo(e._x1+e._k*(e._x2-e._x0),e._y1+e._k*(e._y2-e._y0),e._x2+e._k*(e._x1-t),e._y2+e._k*(e._y1-r),e._x2,e._y2)}function kN(e,t){this._context=e,this._k=(1-t)/6}kN.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:Ky(this,this._x1,this._y1);break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2,this._x1=e,this._y1=t;break;case 2:this._point=3;default:Ky(this,e,t);break}this._x0=this._x1,this._x1=this._x2,this._x2=e,this._y0=this._y1,this._y1=this._y2,this._y2=t}};const sie=(function e(t){function r(n){return new kN(n,t)}return r.tension=function(n){return e(+n)},r})(0);function CN(e,t){this._context=e,this._k=(1-t)/6}CN.prototype={areaStart:Lu,areaEnd:Lu,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:{this._context.moveTo(this._x3,this._y3),this._context.closePath();break}case 2:{this._context.lineTo(this._x3,this._y3),this._context.closePath();break}case 3:{this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5);break}}},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._x3=e,this._y3=t;break;case 1:this._point=2,this._context.moveTo(this._x4=e,this._y4=t);break;case 2:this._point=3,this._x5=e,this._y5=t;break;default:Ky(this,e,t);break}this._x0=this._x1,this._x1=this._x2,this._x2=e,this._y0=this._y1,this._y1=this._y2,this._y2=t}};const pje=(function e(t){function r(n){return new CN(n,t)}return r.tension=function(n){return e(+n)},r})(0);function TN(e,t){this._context=e,this._k=(1-t)/6}TN.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===3)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:Ky(this,e,t);break}this._x0=this._x1,this._x1=this._x2,this._x2=e,this._y0=this._y1,this._y1=this._y2,this._y2=t}};const fje=(function e(t){function r(n){return new TN(n,t)}return r.tension=function(n){return e(+n)},r})(0);function AN(e,t,r){var n=e._x1,a=e._y1,i=e._x2,o=e._y2;if(e._l01_a>lH){var s=2*e._l01_2a+3*e._l01_a*e._l12_a+e._l12_2a,l=3*e._l01_a*(e._l01_a+e._l12_a);n=(n*s-e._x0*e._l12_2a+e._x2*e._l01_2a)/l,a=(a*s-e._y0*e._l12_2a+e._y2*e._l01_2a)/l}if(e._l23_a>lH){var u=2*e._l23_2a+3*e._l23_a*e._l12_a+e._l12_2a,d=3*e._l23_a*(e._l23_a+e._l12_a);i=(i*u+e._x1*e._l23_2a-t*e._l12_2a)/d,o=(o*u+e._y1*e._l23_2a-r*e._l12_2a)/d}e._context.bezierCurveTo(n,a,i,o,e._x2,e._y2)}function lie(e,t){this._context=e,this._alpha=t}lie.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x2,this._y2);break;case 3:this.point(this._x2,this._y2);break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){if(e=+e,t=+t,this._point){var r=this._x2-e,n=this._y2-t;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(r*r+n*n,this._alpha))}switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;break;case 2:this._point=3;default:AN(this,e,t);break}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=e,this._y0=this._y1,this._y1=this._y2,this._y2=t}};const uie=(function e(t){function r(n){return t?new lie(n,t):new kN(n,0)}return r.alpha=function(n){return e(+n)},r})(.5);function cie(e,t){this._context=e,this._alpha=t}cie.prototype={areaStart:Lu,areaEnd:Lu,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._x5=this._y0=this._y1=this._y2=this._y3=this._y4=this._y5=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){switch(this._point){case 1:{this._context.moveTo(this._x3,this._y3),this._context.closePath();break}case 2:{this._context.lineTo(this._x3,this._y3),this._context.closePath();break}case 3:{this.point(this._x3,this._y3),this.point(this._x4,this._y4),this.point(this._x5,this._y5);break}}},point:function(e,t){if(e=+e,t=+t,this._point){var r=this._x2-e,n=this._y2-t;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(r*r+n*n,this._alpha))}switch(this._point){case 0:this._point=1,this._x3=e,this._y3=t;break;case 1:this._point=2,this._context.moveTo(this._x4=e,this._y4=t);break;case 2:this._point=3,this._x5=e,this._y5=t;break;default:AN(this,e,t);break}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=e,this._y0=this._y1,this._y1=this._y2,this._y2=t}};const mje=(function e(t){function r(n){return t?new cie(n,t):new CN(n,0)}return r.alpha=function(n){return e(+n)},r})(.5);function die(e,t){this._context=e,this._alpha=t}die.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._x2=this._y0=this._y1=this._y2=NaN,this._l01_a=this._l12_a=this._l23_a=this._l01_2a=this._l12_2a=this._l23_2a=this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===3)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){if(e=+e,t=+t,this._point){var r=this._x2-e,n=this._y2-t;this._l23_a=Math.sqrt(this._l23_2a=Math.pow(r*r+n*n,this._alpha))}switch(this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3,this._line?this._context.lineTo(this._x2,this._y2):this._context.moveTo(this._x2,this._y2);break;case 3:this._point=4;default:AN(this,e,t);break}this._l01_a=this._l12_a,this._l12_a=this._l23_a,this._l01_2a=this._l12_2a,this._l12_2a=this._l23_2a,this._x0=this._x1,this._x1=this._x2,this._x2=e,this._y0=this._y1,this._y1=this._y2,this._y2=t}};const gje=(function e(t){function r(n){return t?new die(n,t):new TN(n,0)}return r.alpha=function(n){return e(+n)},r})(.5);function hie(e){this._context=e}hie.prototype={areaStart:Lu,areaEnd:Lu,lineStart:function(){this._point=0},lineEnd:function(){this._point&&this._context.closePath()},point:function(e,t){e=+e,t=+t,this._point?this._context.lineTo(e,t):(this._point=1,this._context.moveTo(e,t))}};function bje(e){return new hie(e)}function cH(e){return e<0?-1:1}function dH(e,t,r){var n=e._x1-e._x0,a=t-e._x1,i=(e._y1-e._y0)/(n||a<0&&-0),o=(r-e._y1)/(a||n<0&&-0),s=(i*a+o*n)/(n+a);return(cH(i)+cH(o))*Math.min(Math.abs(i),Math.abs(o),.5*Math.abs(s))||0}function hH(e,t){var r=e._x1-e._x0;return r?(3*(e._y1-e._y0)/r-t)/2:t}function Nk(e,t,r){var n=e._x0,a=e._y0,i=e._x1,o=e._y1,s=(i-n)/3;e._context.bezierCurveTo(n+s,a+s*t,i-s,o-s*r,i,o)}function Yy(e){this._context=e}Yy.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=this._t0=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x1,this._y1);break;case 3:Nk(this,this._t0,hH(this,this._t0));break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){var r=NaN;if(e=+e,t=+t,!(e===this._x1&&t===this._y1)){switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;break;case 2:this._point=3,Nk(this,hH(this,r=dH(this,e,t)),r);break;default:Nk(this,this._t0,r=dH(this,e,t));break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t,this._t0=r}}};function pie(e){this._context=new fie(e)}(pie.prototype=Object.create(Yy.prototype)).point=function(e,t){Yy.prototype.point.call(this,t,e)};function fie(e){this._context=e}fie.prototype={moveTo:function(e,t){this._context.moveTo(t,e)},closePath:function(){this._context.closePath()},lineTo:function(e,t){this._context.lineTo(t,e)},bezierCurveTo:function(e,t,r,n,a,i){this._context.bezierCurveTo(t,e,n,r,i,a)}};function mie(e){return new Yy(e)}function gie(e){return new pie(e)}function bie(e){this._context=e}bie.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=[],this._y=[]},lineEnd:function(){var e=this._x,t=this._y,r=e.length;if(r)if(this._line?this._context.lineTo(e[0],t[0]):this._context.moveTo(e[0],t[0]),r===2)this._context.lineTo(e[1],t[1]);else for(var n=pH(e),a=pH(t),i=0,o=1;o<r;++i,++o)this._context.bezierCurveTo(n[0][i],a[0][i],n[1][i],a[1][i],e[o],t[o]);(this._line||this._line!==0&&r===1)&&this._context.closePath(),this._line=1-this._line,this._x=this._y=null},point:function(e,t){this._x.push(+e),this._y.push(+t)}};function pH(e){var t,r=e.length-1,n,a=new Array(r),i=new Array(r),o=new Array(r);for(a[0]=0,i[0]=2,o[0]=e[0]+2*e[1],t=1;t<r-1;++t)a[t]=1,i[t]=4,o[t]=4*e[t]+2*e[t+1];for(a[r-1]=2,i[r-1]=7,o[r-1]=8*e[r-1]+e[r],t=1;t<r;++t)n=a[t]/i[t-1],i[t]-=n,o[t]-=n*o[t-1];for(a[r-1]=o[r-1]/i[r-1],t=r-2;t>=0;--t)a[t]=(o[t]-a[t+1])/i[t];for(i[r-1]=(e[r]+a[r-1])/2,t=0;t<r-1;++t)i[t]=2*e[t+1]-a[t+1];return[a,i]}function yie(e){return new bie(e)}function B2(e,t){this._context=e,this._t=t}B2.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=this._y=NaN,this._point=0},lineEnd:function(){0<this._t&&this._t<1&&this._point===2&&this._context.lineTo(this._x,this._y),(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line>=0&&(this._t=1-this._t,this._line=1-this._line)},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;default:{if(this._t<=0)this._context.lineTo(this._x,t),this._context.lineTo(e,t);else{var r=this._x*(1-this._t)+e*this._t;this._context.lineTo(r,this._y),this._context.lineTo(r,t)}break}}this._x=e,this._y=t}};function vie(e){return new B2(e,.5)}function Sie(e){return new B2(e,0)}function Eie(e){return new B2(e,1)}function cm(e,t,r){this.k=e,this.x=t,this.y=r}cm.prototype={constructor:cm,scale:function(e){return e===1?this:new cm(this.k*e,this.x,this.y)},translate:function(e,t){return e===0&t===0?this:new cm(this.k,this.x+this.k*e,this.y+this.k*t)},apply:function(e){return[e[0]*this.k+this.x,e[1]*this.k+this.y]},applyX:function(e){return e*this.k+this.x},applyY:function(e){return e*this.k+this.y},invert:function(e){return[(e[0]-this.x)/this.k,(e[1]-this.y)/this.k]},invertX:function(e){return(e-this.x)/this.k},invertY:function(e){return(e-this.y)/this.k},rescaleX:function(e){return e.copy().domain(e.range().map(this.invertX,this).map(e.invert,e))},rescaleY:function(e){return e.copy().domain(e.range().map(this.invertY,this).map(e.invert,e))},toString:function(){return"translate("+this.x+","+this.y+") scale("+this.k+")"}};cm.prototype;var yje=U(e=>{var a;const{securityLevel:t}=Gr();let r=Lr("body");if(t==="sandbox"){const o=((a=Lr(`#i${e}`).node())==null?void 0:a.contentDocument)??document;r=Lr(o.body)}return r.select(`#${e}`)},"selectSvgElement");function FN(e){return typeof e>"u"||e===null}U(FN,"isNothing");function xie(e){return typeof e=="object"&&e!==null}U(xie,"isObject");function wie(e){return Array.isArray(e)?e:FN(e)?[]:[e]}U(wie,"toArray");function kie(e,t){var r,n,a,i;if(t)for(i=Object.keys(t),r=0,n=i.length;r<n;r+=1)a=i[r],e[a]=t[a];return e}U(kie,"extend");function Cie(e,t){var r="",n;for(n=0;n<t;n+=1)r+=e;return r}U(Cie,"repeat");function Tie(e){return e===0&&Number.NEGATIVE_INFINITY===1/e}U(Tie,"isNegativeZero");var vje=FN,Sje=xie,Eje=wie,xje=Cie,wje=Tie,kje=kie,ea={isNothing:vje,isObject:Sje,toArray:Eje,repeat:xje,isNegativeZero:wje,extend:kje};function _N(e,t){var r="",n=e.reason||"(unknown reason)";return e.mark?(e.mark.name&&(r+='in "'+e.mark.name+'" '),r+="("+(e.mark.line+1)+":"+(e.mark.column+1)+")",!t&&e.mark.snippet&&(r+=`

`+e.mark.snippet),n+" "+r):n}U(_N,"formatError");function rp(e,t){Error.call(this),this.name="YAMLException",this.reason=e,this.mark=t,this.message=_N(this,!1),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=new Error().stack||""}U(rp,"YAMLException$1");rp.prototype=Object.create(Error.prototype);rp.prototype.constructor=rp;rp.prototype.toString=U(function(t){return this.name+": "+_N(this,t)},"toString");var di=rp;function W1(e,t,r,n,a){var i="",o="",s=Math.floor(a/2)-1;return n-t>s&&(i=" ... ",t=n-s+i.length),r-n>s&&(o=" ...",r=n+s-o.length),{str:i+e.slice(t,r).replace(/\t/g,"→")+o,pos:n-t+i.length}}U(W1,"getLine");function V1(e,t){return ea.repeat(" ",t-e.length)+e}U(V1,"padStart");function Aie(e,t){if(t=Object.create(t||null),!e.buffer)return null;t.maxLength||(t.maxLength=79),typeof t.indent!="number"&&(t.indent=1),typeof t.linesBefore!="number"&&(t.linesBefore=3),typeof t.linesAfter!="number"&&(t.linesAfter=2);for(var r=/\r?\n|\r|\0/g,n=[0],a=[],i,o=-1;i=r.exec(e.buffer);)a.push(i.index),n.push(i.index+i[0].length),e.position<=i.index&&o<0&&(o=n.length-2);o<0&&(o=n.length-1);var s="",l,u,d=Math.min(e.line+t.linesAfter,a.length).toString().length,p=t.maxLength-(t.indent+d+3);for(l=1;l<=t.linesBefore&&!(o-l<0);l++)u=W1(e.buffer,n[o-l],a[o-l],e.position-(n[o]-n[o-l]),p),s=ea.repeat(" ",t.indent)+V1((e.line-l+1).toString(),d)+" | "+u.str+`
`+s;for(u=W1(e.buffer,n[o],a[o],e.position,p),s+=ea.repeat(" ",t.indent)+V1((e.line+1).toString(),d)+" | "+u.str+`
`,s+=ea.repeat("-",t.indent+d+3+u.pos)+`^
`,l=1;l<=t.linesAfter&&!(o+l>=a.length);l++)u=W1(e.buffer,n[o+l],a[o+l],e.position-(n[o]-n[o+l]),p),s+=ea.repeat(" ",t.indent)+V1((e.line+l+1).toString(),d)+" | "+u.str+`
`;return s.replace(/\n$/,"")}U(Aie,"makeSnippet");var Cje=Aie,Tje=["kind","multi","resolve","construct","instanceOf","predicate","represent","representName","defaultStyle","styleAliases"],Aje=["scalar","sequence","mapping"];function Fie(e){var t={};return e!==null&&Object.keys(e).forEach(function(r){e[r].forEach(function(n){t[String(n)]=r})}),t}U(Fie,"compileStyleAliases");function _ie(e,t){if(t=t||{},Object.keys(t).forEach(function(r){if(Tje.indexOf(r)===-1)throw new di('Unknown option "'+r+'" is met in definition of "'+e+'" YAML type.')}),this.options=t,this.tag=e,this.kind=t.kind||null,this.resolve=t.resolve||function(){return!0},this.construct=t.construct||function(r){return r},this.instanceOf=t.instanceOf||null,this.predicate=t.predicate||null,this.represent=t.represent||null,this.representName=t.representName||null,this.defaultStyle=t.defaultStyle||null,this.multi=t.multi||!1,this.styleAliases=Fie(t.styleAliases||null),Aje.indexOf(this.kind)===-1)throw new di('Unknown kind "'+this.kind+'" is specified for "'+e+'" YAML type.')}U(_ie,"Type$1");var Ia=_ie;function m6(e,t){var r=[];return e[t].forEach(function(n){var a=r.length;r.forEach(function(i,o){i.tag===n.tag&&i.kind===n.kind&&i.multi===n.multi&&(a=o)}),r[a]=n}),r}U(m6,"compileList");function Rie(){var e={scalar:{},sequence:{},mapping:{},fallback:{},multi:{scalar:[],sequence:[],mapping:[],fallback:[]}},t,r;function n(a){a.multi?(e.multi[a.kind].push(a),e.multi.fallback.push(a)):e[a.kind][a.tag]=e.fallback[a.tag]=a}for(U(n,"collectType"),t=0,r=arguments.length;t<r;t+=1)arguments[t].forEach(n);return e}U(Rie,"compileMap");function Jy(e){return this.extend(e)}U(Jy,"Schema$1");Jy.prototype.extend=U(function(t){var r=[],n=[];if(t instanceof Ia)n.push(t);else if(Array.isArray(t))n=n.concat(t);else if(t&&(Array.isArray(t.implicit)||Array.isArray(t.explicit)))t.implicit&&(r=r.concat(t.implicit)),t.explicit&&(n=n.concat(t.explicit));else throw new di("Schema.extend argument should be a Type, [ Type ], or a schema definition ({ implicit: [...], explicit: [...] })");r.forEach(function(i){if(!(i instanceof Ia))throw new di("Specified list of YAML types (or a single Type object) contains a non-Type object.");if(i.loadKind&&i.loadKind!=="scalar")throw new di("There is a non-scalar type in the implicit list of a schema. Implicit resolving of such types is not supported.");if(i.multi)throw new di("There is a multi type in the implicit list of a schema. Multi tags can only be listed as explicit.")}),n.forEach(function(i){if(!(i instanceof Ia))throw new di("Specified list of YAML types (or a single Type object) contains a non-Type object.")});var a=Object.create(Jy.prototype);return a.implicit=(this.implicit||[]).concat(r),a.explicit=(this.explicit||[]).concat(n),a.compiledImplicit=m6(a,"implicit"),a.compiledExplicit=m6(a,"explicit"),a.compiledTypeMap=Rie(a.compiledImplicit,a.compiledExplicit),a},"extend");var Fje=Jy,_je=new Ia("tag:yaml.org,2002:str",{kind:"scalar",construct:U(function(e){return e!==null?e:""},"construct")}),Rje=new Ia("tag:yaml.org,2002:seq",{kind:"sequence",construct:U(function(e){return e!==null?e:[]},"construct")}),Dje=new Ia("tag:yaml.org,2002:map",{kind:"mapping",construct:U(function(e){return e!==null?e:{}},"construct")}),Nje=new Fje({explicit:[_je,Rje,Dje]});function Die(e){if(e===null)return!0;var t=e.length;return t===1&&e==="~"||t===4&&(e==="null"||e==="Null"||e==="NULL")}U(Die,"resolveYamlNull");function Nie(){return null}U(Nie,"constructYamlNull");function Lie(e){return e===null}U(Lie,"isNull");var Lje=new Ia("tag:yaml.org,2002:null",{kind:"scalar",resolve:Die,construct:Nie,predicate:Lie,represent:{canonical:U(function(){return"~"},"canonical"),lowercase:U(function(){return"null"},"lowercase"),uppercase:U(function(){return"NULL"},"uppercase"),camelcase:U(function(){return"Null"},"camelcase"),empty:U(function(){return""},"empty")},defaultStyle:"lowercase"});function Mie(e){if(e===null)return!1;var t=e.length;return t===4&&(e==="true"||e==="True"||e==="TRUE")||t===5&&(e==="false"||e==="False"||e==="FALSE")}U(Mie,"resolveYamlBoolean");function Iie(e){return e==="true"||e==="True"||e==="TRUE"}U(Iie,"constructYamlBoolean");function Oie(e){return Object.prototype.toString.call(e)==="[object Boolean]"}U(Oie,"isBoolean");var Mje=new Ia("tag:yaml.org,2002:bool",{kind:"scalar",resolve:Mie,construct:Iie,predicate:Oie,represent:{lowercase:U(function(e){return e?"true":"false"},"lowercase"),uppercase:U(function(e){return e?"TRUE":"FALSE"},"uppercase"),camelcase:U(function(e){return e?"True":"False"},"camelcase")},defaultStyle:"lowercase"});function Bie(e){return 48<=e&&e<=57||65<=e&&e<=70||97<=e&&e<=102}U(Bie,"isHexCode");function Pie(e){return 48<=e&&e<=55}U(Pie,"isOctCode");function zie(e){return 48<=e&&e<=57}U(zie,"isDecCode");function Hie(e){if(e===null)return!1;var t=e.length,r=0,n=!1,a;if(!t)return!1;if(a=e[r],(a==="-"||a==="+")&&(a=e[++r]),a==="0"){if(r+1===t)return!0;if(a=e[++r],a==="b"){for(r++;r<t;r++)if(a=e[r],a!=="_"){if(a!=="0"&&a!=="1")return!1;n=!0}return n&&a!=="_"}if(a==="x"){for(r++;r<t;r++)if(a=e[r],a!=="_"){if(!Bie(e.charCodeAt(r)))return!1;n=!0}return n&&a!=="_"}if(a==="o"){for(r++;r<t;r++)if(a=e[r],a!=="_"){if(!Pie(e.charCodeAt(r)))return!1;n=!0}return n&&a!=="_"}}if(a==="_")return!1;for(;r<t;r++)if(a=e[r],a!=="_"){if(!zie(e.charCodeAt(r)))return!1;n=!0}return!(!n||a==="_")}U(Hie,"resolveYamlInteger");function Uie(e){var t=e,r=1,n;if(t.indexOf("_")!==-1&&(t=t.replace(/_/g,"")),n=t[0],(n==="-"||n==="+")&&(n==="-"&&(r=-1),t=t.slice(1),n=t[0]),t==="0")return 0;if(n==="0"){if(t[1]==="b")return r*parseInt(t.slice(2),2);if(t[1]==="x")return r*parseInt(t.slice(2),16);if(t[1]==="o")return r*parseInt(t.slice(2),8)}return r*parseInt(t,10)}U(Uie,"constructYamlInteger");function $ie(e){return Object.prototype.toString.call(e)==="[object Number]"&&e%1===0&&!ea.isNegativeZero(e)}U($ie,"isInteger");var Ije=new Ia("tag:yaml.org,2002:int",{kind:"scalar",resolve:Hie,construct:Uie,predicate:$ie,represent:{binary:U(function(e){return e>=0?"0b"+e.toString(2):"-0b"+e.toString(2).slice(1)},"binary"),octal:U(function(e){return e>=0?"0o"+e.toString(8):"-0o"+e.toString(8).slice(1)},"octal"),decimal:U(function(e){return e.toString(10)},"decimal"),hexadecimal:U(function(e){return e>=0?"0x"+e.toString(16).toUpperCase():"-0x"+e.toString(16).toUpperCase().slice(1)},"hexadecimal")},defaultStyle:"decimal",styleAliases:{binary:[2,"bin"],octal:[8,"oct"],decimal:[10,"dec"],hexadecimal:[16,"hex"]}}),Oje=new RegExp("^(?:[-+]?(?:[0-9][0-9_]*)(?:\\.[0-9_]*)?(?:[eE][-+]?[0-9]+)?|\\.[0-9_]+(?:[eE][-+]?[0-9]+)?|[-+]?\\.(?:inf|Inf|INF)|\\.(?:nan|NaN|NAN))$");function Gie(e){return!(e===null||!Oje.test(e)||e[e.length-1]==="_")}U(Gie,"resolveYamlFloat");function jie(e){var t,r;return t=e.replace(/_/g,"").toLowerCase(),r=t[0]==="-"?-1:1,"+-".indexOf(t[0])>=0&&(t=t.slice(1)),t===".inf"?r===1?Number.POSITIVE_INFINITY:Number.NEGATIVE_INFINITY:t===".nan"?NaN:r*parseFloat(t,10)}U(jie,"constructYamlFloat");var Bje=/^[-+]?[0-9]+e/;function qie(e,t){var r;if(isNaN(e))switch(t){case"lowercase":return".nan";case"uppercase":return".NAN";case"camelcase":return".NaN"}else if(Number.POSITIVE_INFINITY===e)switch(t){case"lowercase":return".inf";case"uppercase":return".INF";case"camelcase":return".Inf"}else if(Number.NEGATIVE_INFINITY===e)switch(t){case"lowercase":return"-.inf";case"uppercase":return"-.INF";case"camelcase":return"-.Inf"}else if(ea.isNegativeZero(e))return"-0.0";return r=e.toString(10),Bje.test(r)?r.replace("e",".e"):r}U(qie,"representYamlFloat");function Wie(e){return Object.prototype.toString.call(e)==="[object Number]"&&(e%1!==0||ea.isNegativeZero(e))}U(Wie,"isFloat");var Pje=new Ia("tag:yaml.org,2002:float",{kind:"scalar",resolve:Gie,construct:jie,predicate:Wie,represent:qie,defaultStyle:"lowercase"}),Vie=Nje.extend({implicit:[Lje,Mje,Ije,Pje]}),zje=Vie,Kie=new RegExp("^([0-9][0-9][0-9][0-9])-([0-9][0-9])-([0-9][0-9])$"),Yie=new RegExp("^([0-9][0-9][0-9][0-9])-([0-9][0-9]?)-([0-9][0-9]?)(?:[Tt]|[ \\t]+)([0-9][0-9]?):([0-9][0-9]):([0-9][0-9])(?:\\.([0-9]*))?(?:[ \\t]*(Z|([-+])([0-9][0-9]?)(?::([0-9][0-9]))?))?$");function Jie(e){return e===null?!1:Kie.exec(e)!==null||Yie.exec(e)!==null}U(Jie,"resolveYamlTimestamp");function Xie(e){var t,r,n,a,i,o,s,l=0,u=null,d,p,f;if(t=Kie.exec(e),t===null&&(t=Yie.exec(e)),t===null)throw new Error("Date resolve error");if(r=+t[1],n=+t[2]-1,a=+t[3],!t[4])return new Date(Date.UTC(r,n,a));if(i=+t[4],o=+t[5],s=+t[6],t[7]){for(l=t[7].slice(0,3);l.length<3;)l+="0";l=+l}return t[9]&&(d=+t[10],p=+(t[11]||0),u=(d*60+p)*6e4,t[9]==="-"&&(u=-u)),f=new Date(Date.UTC(r,n,a,i,o,s,l)),u&&f.setTime(f.getTime()-u),f}U(Xie,"constructYamlTimestamp");function Zie(e){return e.toISOString()}U(Zie,"representYamlTimestamp");var Hje=new Ia("tag:yaml.org,2002:timestamp",{kind:"scalar",resolve:Jie,construct:Xie,instanceOf:Date,represent:Zie});function Qie(e){return e==="<<"||e===null}U(Qie,"resolveYamlMerge");var Uje=new Ia("tag:yaml.org,2002:merge",{kind:"scalar",resolve:Qie}),RN=`ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=
\r`;function eoe(e){if(e===null)return!1;var t,r,n=0,a=e.length,i=RN;for(r=0;r<a;r++)if(t=i.indexOf(e.charAt(r)),!(t>64)){if(t<0)return!1;n+=6}return n%8===0}U(eoe,"resolveYamlBinary");function toe(e){var t,r,n=e.replace(/[\r\n=]/g,""),a=n.length,i=RN,o=0,s=[];for(t=0;t<a;t++)t%4===0&&t&&(s.push(o>>16&255),s.push(o>>8&255),s.push(o&255)),o=o<<6|i.indexOf(n.charAt(t));return r=a%4*6,r===0?(s.push(o>>16&255),s.push(o>>8&255),s.push(o&255)):r===18?(s.push(o>>10&255),s.push(o>>2&255)):r===12&&s.push(o>>4&255),new Uint8Array(s)}U(toe,"constructYamlBinary");function roe(e){var t="",r=0,n,a,i=e.length,o=RN;for(n=0;n<i;n++)n%3===0&&n&&(t+=o[r>>18&63],t+=o[r>>12&63],t+=o[r>>6&63],t+=o[r&63]),r=(r<<8)+e[n];return a=i%3,a===0?(t+=o[r>>18&63],t+=o[r>>12&63],t+=o[r>>6&63],t+=o[r&63]):a===2?(t+=o[r>>10&63],t+=o[r>>4&63],t+=o[r<<2&63],t+=o[64]):a===1&&(t+=o[r>>2&63],t+=o[r<<4&63],t+=o[64],t+=o[64]),t}U(roe,"representYamlBinary");function noe(e){return Object.prototype.toString.call(e)==="[object Uint8Array]"}U(noe,"isBinary");var $je=new Ia("tag:yaml.org,2002:binary",{kind:"scalar",resolve:eoe,construct:toe,predicate:noe,represent:roe}),Gje=Object.prototype.hasOwnProperty,jje=Object.prototype.toString;function aoe(e){if(e===null)return!0;var t=[],r,n,a,i,o,s=e;for(r=0,n=s.length;r<n;r+=1){if(a=s[r],o=!1,jje.call(a)!=="[object Object]")return!1;for(i in a)if(Gje.call(a,i))if(!o)o=!0;else return!1;if(!o)return!1;if(t.indexOf(i)===-1)t.push(i);else return!1}return!0}U(aoe,"resolveYamlOmap");function ioe(e){return e!==null?e:[]}U(ioe,"constructYamlOmap");var qje=new Ia("tag:yaml.org,2002:omap",{kind:"sequence",resolve:aoe,construct:ioe}),Wje=Object.prototype.toString;function ooe(e){if(e===null)return!0;var t,r,n,a,i,o=e;for(i=new Array(o.length),t=0,r=o.length;t<r;t+=1){if(n=o[t],Wje.call(n)!=="[object Object]"||(a=Object.keys(n),a.length!==1))return!1;i[t]=[a[0],n[a[0]]]}return!0}U(ooe,"resolveYamlPairs");function soe(e){if(e===null)return[];var t,r,n,a,i,o=e;for(i=new Array(o.length),t=0,r=o.length;t<r;t+=1)n=o[t],a=Object.keys(n),i[t]=[a[0],n[a[0]]];return i}U(soe,"constructYamlPairs");var Vje=new Ia("tag:yaml.org,2002:pairs",{kind:"sequence",resolve:ooe,construct:soe}),Kje=Object.prototype.hasOwnProperty;function loe(e){if(e===null)return!0;var t,r=e;for(t in r)if(Kje.call(r,t)&&r[t]!==null)return!1;return!0}U(loe,"resolveYamlSet");function uoe(e){return e!==null?e:{}}U(uoe,"constructYamlSet");var Yje=new Ia("tag:yaml.org,2002:set",{kind:"mapping",resolve:loe,construct:uoe}),coe=zje.extend({implicit:[Hje,Uje],explicit:[$je,qje,Vje,Yje]}),Mu=Object.prototype.hasOwnProperty,Xy=1,doe=2,hoe=3,Zy=4,Lk=1,Jje=2,fH=3,Xje=/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/,Zje=/[\x85\u2028\u2029]/,Qje=/[,\[\]\{\}]/,poe=/^(?:!|!!|![a-z\-]+!)$/i,foe=/^(?:!|[^,\[\]\{\}])(?:%[0-9a-f]{2}|[0-9a-z\-#;\/\?:@&=\+\$,_\.!~\*'\(\)\[\]])*$/i;function g6(e){return Object.prototype.toString.call(e)}U(g6,"_class");function Do(e){return e===10||e===13}U(Do,"is_EOL");function ku(e){return e===9||e===32}U(ku,"is_WHITE_SPACE");function Va(e){return e===9||e===32||e===10||e===13}U(Va,"is_WS_OR_EOL");function Cc(e){return e===44||e===91||e===93||e===123||e===125}U(Cc,"is_FLOW_INDICATOR");function moe(e){var t;return 48<=e&&e<=57?e-48:(t=e|32,97<=t&&t<=102?t-97+10:-1)}U(moe,"fromHexCode");function goe(e){return e===120?2:e===117?4:e===85?8:0}U(goe,"escapedHexLen");function boe(e){return 48<=e&&e<=57?e-48:-1}U(boe,"fromDecimalCode");function b6(e){return e===48?"\0":e===97?"\x07":e===98?"\b":e===116||e===9?"	":e===110?`
`:e===118?"\v":e===102?"\f":e===114?"\r":e===101?"\x1B":e===32?" ":e===34?'"':e===47?"/":e===92?"\\":e===78?"":e===95?" ":e===76?"\u2028":e===80?"\u2029":""}U(b6,"simpleEscapeSequence");function yoe(e){return e<=65535?String.fromCharCode(e):String.fromCharCode((e-65536>>10)+55296,(e-65536&1023)+56320)}U(yoe,"charFromCodepoint");var voe=new Array(256),Soe=new Array(256);for(mc=0;mc<256;mc++)voe[mc]=b6(mc)?1:0,Soe[mc]=b6(mc);var mc;function Eoe(e,t){this.input=e,this.filename=t.filename||null,this.schema=t.schema||coe,this.onWarning=t.onWarning||null,this.legacy=t.legacy||!1,this.json=t.json||!1,this.listener=t.listener||null,this.implicitTypes=this.schema.compiledImplicit,this.typeMap=this.schema.compiledTypeMap,this.length=e.length,this.position=0,this.line=0,this.lineStart=0,this.lineIndent=0,this.firstTabInLine=-1,this.documents=[]}U(Eoe,"State$1");function DN(e,t){var r={name:e.filename,buffer:e.input.slice(0,-1),position:e.position,line:e.line,column:e.position-e.lineStart};return r.snippet=Cje(r),new di(t,r)}U(DN,"generateError");function Pt(e,t){throw DN(e,t)}U(Pt,"throwError");function Zm(e,t){e.onWarning&&e.onWarning.call(null,DN(e,t))}U(Zm,"throwWarning");var mH={YAML:U(function(t,r,n){var a,i,o;t.version!==null&&Pt(t,"duplication of %YAML directive"),n.length!==1&&Pt(t,"YAML directive accepts exactly one argument"),a=/^([0-9]+)\.([0-9]+)$/.exec(n[0]),a===null&&Pt(t,"ill-formed argument of the YAML directive"),i=parseInt(a[1],10),o=parseInt(a[2],10),i!==1&&Pt(t,"unacceptable YAML version of the document"),t.version=n[0],t.checkLineBreaks=o<2,o!==1&&o!==2&&Zm(t,"unsupported YAML version of the document")},"handleYamlDirective"),TAG:U(function(t,r,n){var a,i;n.length!==2&&Pt(t,"TAG directive accepts exactly two arguments"),a=n[0],i=n[1],poe.test(a)||Pt(t,"ill-formed tag handle (first argument) of the TAG directive"),Mu.call(t.tagMap,a)&&Pt(t,'there is a previously declared suffix for "'+a+'" tag handle'),foe.test(i)||Pt(t,"ill-formed tag prefix (second argument) of the TAG directive");try{i=decodeURIComponent(i)}catch{Pt(t,"tag prefix is malformed: "+i)}t.tagMap[a]=i},"handleTagDirective")};function hl(e,t,r,n){var a,i,o,s;if(t<r){if(s=e.input.slice(t,r),n)for(a=0,i=s.length;a<i;a+=1)o=s.charCodeAt(a),o===9||32<=o&&o<=1114111||Pt(e,"expected valid JSON character");else Xje.test(s)&&Pt(e,"the stream contains non-printable characters");e.result+=s}}U(hl,"captureSegment");function y6(e,t,r,n){var a,i,o,s;for(ea.isObject(r)||Pt(e,"cannot merge mappings; the provided source object is unacceptable"),a=Object.keys(r),o=0,s=a.length;o<s;o+=1)i=a[o],Mu.call(t,i)||(t[i]=r[i],n[i]=!0)}U(y6,"mergeMappings");function Tc(e,t,r,n,a,i,o,s,l){var u,d;if(Array.isArray(a))for(a=Array.prototype.slice.call(a),u=0,d=a.length;u<d;u+=1)Array.isArray(a[u])&&Pt(e,"nested arrays are not supported inside keys"),typeof a=="object"&&g6(a[u])==="[object Object]"&&(a[u]="[object Object]");if(typeof a=="object"&&g6(a)==="[object Object]"&&(a="[object Object]"),a=String(a),t===null&&(t={}),n==="tag:yaml.org,2002:merge")if(Array.isArray(i))for(u=0,d=i.length;u<d;u+=1)y6(e,t,i[u],r);else y6(e,t,i,r);else!e.json&&!Mu.call(r,a)&&Mu.call(t,a)&&(e.line=o||e.line,e.lineStart=s||e.lineStart,e.position=l||e.position,Pt(e,"duplicated mapping key")),a==="__proto__"?Object.defineProperty(t,a,{configurable:!0,enumerable:!0,writable:!0,value:i}):t[a]=i,delete r[a];return t}U(Tc,"storeMappingPair");function P2(e){var t;t=e.input.charCodeAt(e.position),t===10?e.position++:t===13?(e.position++,e.input.charCodeAt(e.position)===10&&e.position++):Pt(e,"a line break is expected"),e.line+=1,e.lineStart=e.position,e.firstTabInLine=-1}U(P2,"readLineBreak");function Nn(e,t,r){for(var n=0,a=e.input.charCodeAt(e.position);a!==0;){for(;ku(a);)a===9&&e.firstTabInLine===-1&&(e.firstTabInLine=e.position),a=e.input.charCodeAt(++e.position);if(t&&a===35)do a=e.input.charCodeAt(++e.position);while(a!==10&&a!==13&&a!==0);if(Do(a))for(P2(e),a=e.input.charCodeAt(e.position),n++,e.lineIndent=0;a===32;)e.lineIndent++,a=e.input.charCodeAt(++e.position);else break}return r!==-1&&n!==0&&e.lineIndent<r&&Zm(e,"deficient indentation"),n}U(Nn,"skipSeparationSpace");function jg(e){var t=e.position,r;return r=e.input.charCodeAt(t),!!((r===45||r===46)&&r===e.input.charCodeAt(t+1)&&r===e.input.charCodeAt(t+2)&&(t+=3,r=e.input.charCodeAt(t),r===0||Va(r)))}U(jg,"testDocumentSeparator");function z2(e,t){t===1?e.result+=" ":t>1&&(e.result+=ea.repeat(`
`,t-1))}U(z2,"writeFoldedLines");function xoe(e,t,r){var n,a,i,o,s,l,u,d,p=e.kind,f=e.result,m;if(m=e.input.charCodeAt(e.position),Va(m)||Cc(m)||m===35||m===38||m===42||m===33||m===124||m===62||m===39||m===34||m===37||m===64||m===96||(m===63||m===45)&&(a=e.input.charCodeAt(e.position+1),Va(a)||r&&Cc(a)))return!1;for(e.kind="scalar",e.result="",i=o=e.position,s=!1;m!==0;){if(m===58){if(a=e.input.charCodeAt(e.position+1),Va(a)||r&&Cc(a))break}else if(m===35){if(n=e.input.charCodeAt(e.position-1),Va(n))break}else{if(e.position===e.lineStart&&jg(e)||r&&Cc(m))break;if(Do(m))if(l=e.line,u=e.lineStart,d=e.lineIndent,Nn(e,!1,-1),e.lineIndent>=t){s=!0,m=e.input.charCodeAt(e.position);continue}else{e.position=o,e.line=l,e.lineStart=u,e.lineIndent=d;break}}s&&(hl(e,i,o,!1),z2(e,e.line-l),i=o=e.position,s=!1),ku(m)||(o=e.position+1),m=e.input.charCodeAt(++e.position)}return hl(e,i,o,!1),e.result?!0:(e.kind=p,e.result=f,!1)}U(xoe,"readPlainScalar");function woe(e,t){var r,n,a;if(r=e.input.charCodeAt(e.position),r!==39)return!1;for(e.kind="scalar",e.result="",e.position++,n=a=e.position;(r=e.input.charCodeAt(e.position))!==0;)if(r===39)if(hl(e,n,e.position,!0),r=e.input.charCodeAt(++e.position),r===39)n=e.position,e.position++,a=e.position;else return!0;else Do(r)?(hl(e,n,a,!0),z2(e,Nn(e,!1,t)),n=a=e.position):e.position===e.lineStart&&jg(e)?Pt(e,"unexpected end of the document within a single quoted scalar"):(e.position++,a=e.position);Pt(e,"unexpected end of the stream within a single quoted scalar")}U(woe,"readSingleQuotedScalar");function koe(e,t){var r,n,a,i,o,s;if(s=e.input.charCodeAt(e.position),s!==34)return!1;for(e.kind="scalar",e.result="",e.position++,r=n=e.position;(s=e.input.charCodeAt(e.position))!==0;){if(s===34)return hl(e,r,e.position,!0),e.position++,!0;if(s===92){if(hl(e,r,e.position,!0),s=e.input.charCodeAt(++e.position),Do(s))Nn(e,!1,t);else if(s<256&&voe[s])e.result+=Soe[s],e.position++;else if((o=goe(s))>0){for(a=o,i=0;a>0;a--)s=e.input.charCodeAt(++e.position),(o=moe(s))>=0?i=(i<<4)+o:Pt(e,"expected hexadecimal character");e.result+=yoe(i),e.position++}else Pt(e,"unknown escape sequence");r=n=e.position}else Do(s)?(hl(e,r,n,!0),z2(e,Nn(e,!1,t)),r=n=e.position):e.position===e.lineStart&&jg(e)?Pt(e,"unexpected end of the document within a double quoted scalar"):(e.position++,n=e.position)}Pt(e,"unexpected end of the stream within a double quoted scalar")}U(koe,"readDoubleQuotedScalar");function Coe(e,t){var r=!0,n,a,i,o=e.tag,s,l=e.anchor,u,d,p,f,m,b=Object.create(null),y,S,w,E;if(E=e.input.charCodeAt(e.position),E===91)d=93,m=!1,s=[];else if(E===123)d=125,m=!0,s={};else return!1;for(e.anchor!==null&&(e.anchorMap[e.anchor]=s),E=e.input.charCodeAt(++e.position);E!==0;){if(Nn(e,!0,t),E=e.input.charCodeAt(e.position),E===d)return e.position++,e.tag=o,e.anchor=l,e.kind=m?"mapping":"sequence",e.result=s,!0;r?E===44&&Pt(e,"expected the node content, but found ','"):Pt(e,"missed comma between flow collection entries"),S=y=w=null,p=f=!1,E===63&&(u=e.input.charCodeAt(e.position+1),Va(u)&&(p=f=!0,e.position++,Nn(e,!0,t))),n=e.line,a=e.lineStart,i=e.position,Jc(e,t,Xy,!1,!0),S=e.tag,y=e.result,Nn(e,!0,t),E=e.input.charCodeAt(e.position),(f||e.line===n)&&E===58&&(p=!0,E=e.input.charCodeAt(++e.position),Nn(e,!0,t),Jc(e,t,Xy,!1,!0),w=e.result),m?Tc(e,s,b,S,y,w,n,a,i):p?s.push(Tc(e,null,b,S,y,w,n,a,i)):s.push(y),Nn(e,!0,t),E=e.input.charCodeAt(e.position),E===44?(r=!0,E=e.input.charCodeAt(++e.position)):r=!1}Pt(e,"unexpected end of the stream within a flow collection")}U(Coe,"readFlowCollection");function Toe(e,t){var r,n,a=Lk,i=!1,o=!1,s=t,l=0,u=!1,d,p;if(p=e.input.charCodeAt(e.position),p===124)n=!1;else if(p===62)n=!0;else return!1;for(e.kind="scalar",e.result="";p!==0;)if(p=e.input.charCodeAt(++e.position),p===43||p===45)Lk===a?a=p===43?fH:Jje:Pt(e,"repeat of a chomping mode identifier");else if((d=boe(p))>=0)d===0?Pt(e,"bad explicit indentation width of a block scalar; it cannot be less than one"):o?Pt(e,"repeat of an indentation width identifier"):(s=t+d-1,o=!0);else break;if(ku(p)){do p=e.input.charCodeAt(++e.position);while(ku(p));if(p===35)do p=e.input.charCodeAt(++e.position);while(!Do(p)&&p!==0)}for(;p!==0;){for(P2(e),e.lineIndent=0,p=e.input.charCodeAt(e.position);(!o||e.lineIndent<s)&&p===32;)e.lineIndent++,p=e.input.charCodeAt(++e.position);if(!o&&e.lineIndent>s&&(s=e.lineIndent),Do(p)){l++;continue}if(e.lineIndent<s){a===fH?e.result+=ea.repeat(`
`,i?1+l:l):a===Lk&&i&&(e.result+=`
`);break}for(n?ku(p)?(u=!0,e.result+=ea.repeat(`
`,i?1+l:l)):u?(u=!1,e.result+=ea.repeat(`
`,l+1)):l===0?i&&(e.result+=" "):e.result+=ea.repeat(`
`,l):e.result+=ea.repeat(`
`,i?1+l:l),i=!0,o=!0,l=0,r=e.position;!Do(p)&&p!==0;)p=e.input.charCodeAt(++e.position);hl(e,r,e.position,!1)}return!0}U(Toe,"readBlockScalar");function v6(e,t){var r,n=e.tag,a=e.anchor,i=[],o,s=!1,l;if(e.firstTabInLine!==-1)return!1;for(e.anchor!==null&&(e.anchorMap[e.anchor]=i),l=e.input.charCodeAt(e.position);l!==0&&(e.firstTabInLine!==-1&&(e.position=e.firstTabInLine,Pt(e,"tab characters must not be used in indentation")),!(l!==45||(o=e.input.charCodeAt(e.position+1),!Va(o))));){if(s=!0,e.position++,Nn(e,!0,-1)&&e.lineIndent<=t){i.push(null),l=e.input.charCodeAt(e.position);continue}if(r=e.line,Jc(e,t,hoe,!1,!0),i.push(e.result),Nn(e,!0,-1),l=e.input.charCodeAt(e.position),(e.line===r||e.lineIndent>t)&&l!==0)Pt(e,"bad indentation of a sequence entry");else if(e.lineIndent<t)break}return s?(e.tag=n,e.anchor=a,e.kind="sequence",e.result=i,!0):!1}U(v6,"readBlockSequence");function Aoe(e,t,r){var n,a,i,o,s,l,u=e.tag,d=e.anchor,p={},f=Object.create(null),m=null,b=null,y=null,S=!1,w=!1,E;if(e.firstTabInLine!==-1)return!1;for(e.anchor!==null&&(e.anchorMap[e.anchor]=p),E=e.input.charCodeAt(e.position);E!==0;){if(!S&&e.firstTabInLine!==-1&&(e.position=e.firstTabInLine,Pt(e,"tab characters must not be used in indentation")),n=e.input.charCodeAt(e.position+1),i=e.line,(E===63||E===58)&&Va(n))E===63?(S&&(Tc(e,p,f,m,b,null,o,s,l),m=b=y=null),w=!0,S=!0,a=!0):S?(S=!1,a=!0):Pt(e,"incomplete explicit mapping pair; a key node is missed; or followed by a non-tabulated empty line"),e.position+=1,E=n;else{if(o=e.line,s=e.lineStart,l=e.position,!Jc(e,r,doe,!1,!0))break;if(e.line===i){for(E=e.input.charCodeAt(e.position);ku(E);)E=e.input.charCodeAt(++e.position);if(E===58)E=e.input.charCodeAt(++e.position),Va(E)||Pt(e,"a whitespace character is expected after the key-value separator within a block mapping"),S&&(Tc(e,p,f,m,b,null,o,s,l),m=b=y=null),w=!0,S=!1,a=!1,m=e.tag,b=e.result;else if(w)Pt(e,"can not read an implicit mapping pair; a colon is missed");else return e.tag=u,e.anchor=d,!0}else if(w)Pt(e,"can not read a block mapping entry; a multiline key may not be an implicit key");else return e.tag=u,e.anchor=d,!0}if((e.line===i||e.lineIndent>t)&&(S&&(o=e.line,s=e.lineStart,l=e.position),Jc(e,t,Zy,!0,a)&&(S?b=e.result:y=e.result),S||(Tc(e,p,f,m,b,y,o,s,l),m=b=y=null),Nn(e,!0,-1),E=e.input.charCodeAt(e.position)),(e.line===i||e.lineIndent>t)&&E!==0)Pt(e,"bad indentation of a mapping entry");else if(e.lineIndent<t)break}return S&&Tc(e,p,f,m,b,null,o,s,l),w&&(e.tag=u,e.anchor=d,e.kind="mapping",e.result=p),w}U(Aoe,"readBlockMapping");function Foe(e){var t,r=!1,n=!1,a,i,o;if(o=e.input.charCodeAt(e.position),o!==33)return!1;if(e.tag!==null&&Pt(e,"duplication of a tag property"),o=e.input.charCodeAt(++e.position),o===60?(r=!0,o=e.input.charCodeAt(++e.position)):o===33?(n=!0,a="!!",o=e.input.charCodeAt(++e.position)):a="!",t=e.position,r){do o=e.input.charCodeAt(++e.position);while(o!==0&&o!==62);e.position<e.length?(i=e.input.slice(t,e.position),o=e.input.charCodeAt(++e.position)):Pt(e,"unexpected end of the stream within a verbatim tag")}else{for(;o!==0&&!Va(o);)o===33&&(n?Pt(e,"tag suffix cannot contain exclamation marks"):(a=e.input.slice(t-1,e.position+1),poe.test(a)||Pt(e,"named tag handle cannot contain such characters"),n=!0,t=e.position+1)),o=e.input.charCodeAt(++e.position);i=e.input.slice(t,e.position),Qje.test(i)&&Pt(e,"tag suffix cannot contain flow indicator characters")}i&&!foe.test(i)&&Pt(e,"tag name cannot contain such characters: "+i);try{i=decodeURIComponent(i)}catch{Pt(e,"tag name is malformed: "+i)}return r?e.tag=i:Mu.call(e.tagMap,a)?e.tag=e.tagMap[a]+i:a==="!"?e.tag="!"+i:a==="!!"?e.tag="tag:yaml.org,2002:"+i:Pt(e,'undeclared tag handle "'+a+'"'),!0}U(Foe,"readTagProperty");function _oe(e){var t,r;if(r=e.input.charCodeAt(e.position),r!==38)return!1;for(e.anchor!==null&&Pt(e,"duplication of an anchor property"),r=e.input.charCodeAt(++e.position),t=e.position;r!==0&&!Va(r)&&!Cc(r);)r=e.input.charCodeAt(++e.position);return e.position===t&&Pt(e,"name of an anchor node must contain at least one character"),e.anchor=e.input.slice(t,e.position),!0}U(_oe,"readAnchorProperty");function Roe(e){var t,r,n;if(n=e.input.charCodeAt(e.position),n!==42)return!1;for(n=e.input.charCodeAt(++e.position),t=e.position;n!==0&&!Va(n)&&!Cc(n);)n=e.input.charCodeAt(++e.position);return e.position===t&&Pt(e,"name of an alias node must contain at least one character"),r=e.input.slice(t,e.position),Mu.call(e.anchorMap,r)||Pt(e,'unidentified alias "'+r+'"'),e.result=e.anchorMap[r],Nn(e,!0,-1),!0}U(Roe,"readAlias");function Jc(e,t,r,n,a){var i,o,s,l=1,u=!1,d=!1,p,f,m,b,y,S;if(e.listener!==null&&e.listener("open",e),e.tag=null,e.anchor=null,e.kind=null,e.result=null,i=o=s=Zy===r||hoe===r,n&&Nn(e,!0,-1)&&(u=!0,e.lineIndent>t?l=1:e.lineIndent===t?l=0:e.lineIndent<t&&(l=-1)),l===1)for(;Foe(e)||_oe(e);)Nn(e,!0,-1)?(u=!0,s=i,e.lineIndent>t?l=1:e.lineIndent===t?l=0:e.lineIndent<t&&(l=-1)):s=!1;if(s&&(s=u||a),(l===1||Zy===r)&&(Xy===r||doe===r?y=t:y=t+1,S=e.position-e.lineStart,l===1?s&&(v6(e,S)||Aoe(e,S,y))||Coe(e,y)?d=!0:(o&&Toe(e,y)||woe(e,y)||koe(e,y)?d=!0:Roe(e)?(d=!0,(e.tag!==null||e.anchor!==null)&&Pt(e,"alias node should not have any properties")):xoe(e,y,Xy===r)&&(d=!0,e.tag===null&&(e.tag="?")),e.anchor!==null&&(e.anchorMap[e.anchor]=e.result)):l===0&&(d=s&&v6(e,S))),e.tag===null)e.anchor!==null&&(e.anchorMap[e.anchor]=e.result);else if(e.tag==="?"){for(e.result!==null&&e.kind!=="scalar"&&Pt(e,'unacceptable node kind for !<?> tag; it should be "scalar", not "'+e.kind+'"'),p=0,f=e.implicitTypes.length;p<f;p+=1)if(b=e.implicitTypes[p],b.resolve(e.result)){e.result=b.construct(e.result),e.tag=b.tag,e.anchor!==null&&(e.anchorMap[e.anchor]=e.result);break}}else if(e.tag!=="!"){if(Mu.call(e.typeMap[e.kind||"fallback"],e.tag))b=e.typeMap[e.kind||"fallback"][e.tag];else for(b=null,m=e.typeMap.multi[e.kind||"fallback"],p=0,f=m.length;p<f;p+=1)if(e.tag.slice(0,m[p].tag.length)===m[p].tag){b=m[p];break}b||Pt(e,"unknown tag !<"+e.tag+">"),e.result!==null&&b.kind!==e.kind&&Pt(e,"unacceptable node kind for !<"+e.tag+'> tag; it should be "'+b.kind+'", not "'+e.kind+'"'),b.resolve(e.result,e.tag)?(e.result=b.construct(e.result,e.tag),e.anchor!==null&&(e.anchorMap[e.anchor]=e.result)):Pt(e,"cannot resolve a node with !<"+e.tag+"> explicit tag")}return e.listener!==null&&e.listener("close",e),e.tag!==null||e.anchor!==null||d}U(Jc,"composeNode");function Doe(e){var t=e.position,r,n,a,i=!1,o;for(e.version=null,e.checkLineBreaks=e.legacy,e.tagMap=Object.create(null),e.anchorMap=Object.create(null);(o=e.input.charCodeAt(e.position))!==0&&(Nn(e,!0,-1),o=e.input.charCodeAt(e.position),!(e.lineIndent>0||o!==37));){for(i=!0,o=e.input.charCodeAt(++e.position),r=e.position;o!==0&&!Va(o);)o=e.input.charCodeAt(++e.position);for(n=e.input.slice(r,e.position),a=[],n.length<1&&Pt(e,"directive name must not be less than one character in length");o!==0;){for(;ku(o);)o=e.input.charCodeAt(++e.position);if(o===35){do o=e.input.charCodeAt(++e.position);while(o!==0&&!Do(o));break}if(Do(o))break;for(r=e.position;o!==0&&!Va(o);)o=e.input.charCodeAt(++e.position);a.push(e.input.slice(r,e.position))}o!==0&&P2(e),Mu.call(mH,n)?mH[n](e,n,a):Zm(e,'unknown document directive "'+n+'"')}if(Nn(e,!0,-1),e.lineIndent===0&&e.input.charCodeAt(e.position)===45&&e.input.charCodeAt(e.position+1)===45&&e.input.charCodeAt(e.position+2)===45?(e.position+=3,Nn(e,!0,-1)):i&&Pt(e,"directives end mark is expected"),Jc(e,e.lineIndent-1,Zy,!1,!0),Nn(e,!0,-1),e.checkLineBreaks&&Zje.test(e.input.slice(t,e.position))&&Zm(e,"non-ASCII line breaks are interpreted as content"),e.documents.push(e.result),e.position===e.lineStart&&jg(e)){e.input.charCodeAt(e.position)===46&&(e.position+=3,Nn(e,!0,-1));return}if(e.position<e.length-1)Pt(e,"end of the stream or a document separator is expected");else return}U(Doe,"readDocument");function NN(e,t){e=String(e),t=t||{},e.length!==0&&(e.charCodeAt(e.length-1)!==10&&e.charCodeAt(e.length-1)!==13&&(e+=`
`),e.charCodeAt(0)===65279&&(e=e.slice(1)));var r=new Eoe(e,t),n=e.indexOf("\0");for(n!==-1&&(r.position=n,Pt(r,"null byte is not allowed in input")),r.input+="\0";r.input.charCodeAt(r.position)===32;)r.lineIndent+=1,r.position+=1;for(;r.position<r.length-1;)Doe(r);return r.documents}U(NN,"loadDocuments");function eqe(e,t,r){t!==null&&typeof t=="object"&&typeof r>"u"&&(r=t,t=null);var n=NN(e,r);if(typeof t!="function")return n;for(var a=0,i=n.length;a<i;a+=1)t(n[a])}U(eqe,"loadAll$1");function Noe(e,t){var r=NN(e,t);if(r.length!==0){if(r.length===1)return r[0];throw new di("expected a single document in the stream, but found more")}}U(Noe,"load$1");var tqe=Noe,rqe={load:tqe},Loe=Object.prototype.toString,Moe=Object.prototype.hasOwnProperty,LN=65279,nqe=9,Qm=10,aqe=13,iqe=32,oqe=33,sqe=34,S6=35,lqe=37,uqe=38,cqe=39,dqe=42,Ioe=44,hqe=45,Qy=58,pqe=61,fqe=62,mqe=63,gqe=64,Ooe=91,Boe=93,bqe=96,Poe=123,yqe=124,zoe=125,za={};za[0]="\\0";za[7]="\\a";za[8]="\\b";za[9]="\\t";za[10]="\\n";za[11]="\\v";za[12]="\\f";za[13]="\\r";za[27]="\\e";za[34]='\\"';za[92]="\\\\";za[133]="\\N";za[160]="\\_";za[8232]="\\L";za[8233]="\\P";var vqe=["y","Y","yes","Yes","YES","on","On","ON","n","N","no","No","NO","off","Off","OFF"],Sqe=/^[-+]?[0-9_]+(?::[0-9_]+)+(?:\.[0-9_]*)?$/;function Hoe(e,t){var r,n,a,i,o,s,l;if(t===null)return{};for(r={},n=Object.keys(t),a=0,i=n.length;a<i;a+=1)o=n[a],s=String(t[o]),o.slice(0,2)==="!!"&&(o="tag:yaml.org,2002:"+o.slice(2)),l=e.compiledTypeMap.fallback[o],l&&Moe.call(l.styleAliases,s)&&(s=l.styleAliases[s]),r[o]=s;return r}U(Hoe,"compileStyleMap");function Uoe(e){var t,r,n;if(t=e.toString(16).toUpperCase(),e<=255)r="x",n=2;else if(e<=65535)r="u",n=4;else if(e<=4294967295)r="U",n=8;else throw new di("code point within a string may not be greater than 0xFFFFFFFF");return"\\"+r+ea.repeat("0",n-t.length)+t}U(Uoe,"encodeHex");var Eqe=1,eg=2;function $oe(e){this.schema=e.schema||coe,this.indent=Math.max(1,e.indent||2),this.noArrayIndent=e.noArrayIndent||!1,this.skipInvalid=e.skipInvalid||!1,this.flowLevel=ea.isNothing(e.flowLevel)?-1:e.flowLevel,this.styleMap=Hoe(this.schema,e.styles||null),this.sortKeys=e.sortKeys||!1,this.lineWidth=e.lineWidth||80,this.noRefs=e.noRefs||!1,this.noCompatMode=e.noCompatMode||!1,this.condenseFlow=e.condenseFlow||!1,this.quotingType=e.quotingType==='"'?eg:Eqe,this.forceQuotes=e.forceQuotes||!1,this.replacer=typeof e.replacer=="function"?e.replacer:null,this.implicitTypes=this.schema.compiledImplicit,this.explicitTypes=this.schema.compiledExplicit,this.tag=null,this.result="",this.duplicates=[],this.usedDuplicates=null}U($oe,"State");function E6(e,t){for(var r=ea.repeat(" ",t),n=0,a=-1,i="",o,s=e.length;n<s;)a=e.indexOf(`
`,n),a===-1?(o=e.slice(n),n=s):(o=e.slice(n,a+1),n=a+1),o.length&&o!==`
`&&(i+=r),i+=o;return i}U(E6,"indentString");function ev(e,t){return`
`+ea.repeat(" ",e.indent*t)}U(ev,"generateNextLine");function Goe(e,t){var r,n,a;for(r=0,n=e.implicitTypes.length;r<n;r+=1)if(a=e.implicitTypes[r],a.resolve(t))return!0;return!1}U(Goe,"testImplicitResolving");function tg(e){return e===iqe||e===nqe}U(tg,"isWhitespace");function np(e){return 32<=e&&e<=126||161<=e&&e<=55295&&e!==8232&&e!==8233||57344<=e&&e<=65533&&e!==LN||65536<=e&&e<=1114111}U(np,"isPrintable");function x6(e){return np(e)&&e!==LN&&e!==aqe&&e!==Qm}U(x6,"isNsCharOrWhitespace");function w6(e,t,r){var n=x6(e),a=n&&!tg(e);return(r?n:n&&e!==Ioe&&e!==Ooe&&e!==Boe&&e!==Poe&&e!==zoe)&&e!==S6&&!(t===Qy&&!a)||x6(t)&&!tg(t)&&e===S6||t===Qy&&a}U(w6,"isPlainSafe");function joe(e){return np(e)&&e!==LN&&!tg(e)&&e!==hqe&&e!==mqe&&e!==Qy&&e!==Ioe&&e!==Ooe&&e!==Boe&&e!==Poe&&e!==zoe&&e!==S6&&e!==uqe&&e!==dqe&&e!==oqe&&e!==yqe&&e!==pqe&&e!==fqe&&e!==cqe&&e!==sqe&&e!==lqe&&e!==gqe&&e!==bqe}U(joe,"isPlainSafeFirst");function qoe(e){return!tg(e)&&e!==Qy}U(qoe,"isPlainSafeLast");function gh(e,t){var r=e.charCodeAt(t),n;return r>=55296&&r<=56319&&t+1<e.length&&(n=e.charCodeAt(t+1),n>=56320&&n<=57343)?(r-55296)*1024+n-56320+65536:r}U(gh,"codePointAt");function MN(e){var t=/^\n* /;return t.test(e)}U(MN,"needIndentIndicator");var Woe=1,k6=2,Voe=3,Koe=4,ph=5;function Yoe(e,t,r,n,a,i,o,s){var l,u=0,d=null,p=!1,f=!1,m=n!==-1,b=-1,y=joe(gh(e,0))&&qoe(gh(e,e.length-1));if(t||o)for(l=0;l<e.length;u>=65536?l+=2:l++){if(u=gh(e,l),!np(u))return ph;y=y&&w6(u,d,s),d=u}else{for(l=0;l<e.length;u>=65536?l+=2:l++){if(u=gh(e,l),u===Qm)p=!0,m&&(f=f||l-b-1>n&&e[b+1]!==" ",b=l);else if(!np(u))return ph;y=y&&w6(u,d,s),d=u}f=f||m&&l-b-1>n&&e[b+1]!==" "}return!p&&!f?y&&!o&&!a(e)?Woe:i===eg?ph:k6:r>9&&MN(e)?ph:o?i===eg?ph:k6:f?Koe:Voe}U(Yoe,"chooseScalarStyle");function Joe(e,t,r,n,a){e.dump=(function(){if(t.length===0)return e.quotingType===eg?'""':"''";if(!e.noCompatMode&&(vqe.indexOf(t)!==-1||Sqe.test(t)))return e.quotingType===eg?'"'+t+'"':"'"+t+"'";var i=e.indent*Math.max(1,r),o=e.lineWidth===-1?-1:Math.max(Math.min(e.lineWidth,40),e.lineWidth-i),s=n||e.flowLevel>-1&&r>=e.flowLevel;function l(u){return Goe(e,u)}switch(U(l,"testAmbiguity"),Yoe(t,s,e.indent,o,l,e.quotingType,e.forceQuotes&&!n,a)){case Woe:return t;case k6:return"'"+t.replace(/'/g,"''")+"'";case Voe:return"|"+C6(t,e.indent)+T6(E6(t,i));case Koe:return">"+C6(t,e.indent)+T6(E6(Xoe(t,o),i));case ph:return'"'+Zoe(t)+'"';default:throw new di("impossible error: invalid scalar style")}})()}U(Joe,"writeScalar");function C6(e,t){var r=MN(e)?String(t):"",n=e[e.length-1]===`
`,a=n&&(e[e.length-2]===`
`||e===`
`),i=a?"+":n?"":"-";return r+i+`
`}U(C6,"blockHeader");function T6(e){return e[e.length-1]===`
`?e.slice(0,-1):e}U(T6,"dropEndingNewline");function Xoe(e,t){for(var r=/(\n+)([^\n]*)/g,n=(function(){var u=e.indexOf(`
`);return u=u!==-1?u:e.length,r.lastIndex=u,A6(e.slice(0,u),t)})(),a=e[0]===`
`||e[0]===" ",i,o;o=r.exec(e);){var s=o[1],l=o[2];i=l[0]===" ",n+=s+(!a&&!i&&l!==""?`
`:"")+A6(l,t),a=i}return n}U(Xoe,"foldString");function A6(e,t){if(e===""||e[0]===" ")return e;for(var r=/ [^ ]/g,n,a=0,i,o=0,s=0,l="";n=r.exec(e);)s=n.index,s-a>t&&(i=o>a?o:s,l+=`
`+e.slice(a,i),a=i+1),o=s;return l+=`
`,e.length-a>t&&o>a?l+=e.slice(a,o)+`
`+e.slice(o+1):l+=e.slice(a),l.slice(1)}U(A6,"foldLine");function Zoe(e){for(var t="",r=0,n,a=0;a<e.length;r>=65536?a+=2:a++)r=gh(e,a),n=za[r],!n&&np(r)?(t+=e[a],r>=65536&&(t+=e[a+1])):t+=n||Uoe(r);return t}U(Zoe,"escapeString");function Qoe(e,t,r){var n="",a=e.tag,i,o,s;for(i=0,o=r.length;i<o;i+=1)s=r[i],e.replacer&&(s=e.replacer.call(r,String(i),s)),(ds(e,t,s,!1,!1)||typeof s>"u"&&ds(e,t,null,!1,!1))&&(n!==""&&(n+=","+(e.condenseFlow?"":" ")),n+=e.dump);e.tag=a,e.dump="["+n+"]"}U(Qoe,"writeFlowSequence");function F6(e,t,r,n){var a="",i=e.tag,o,s,l;for(o=0,s=r.length;o<s;o+=1)l=r[o],e.replacer&&(l=e.replacer.call(r,String(o),l)),(ds(e,t+1,l,!0,!0,!1,!0)||typeof l>"u"&&ds(e,t+1,null,!0,!0,!1,!0))&&((!n||a!=="")&&(a+=ev(e,t)),e.dump&&Qm===e.dump.charCodeAt(0)?a+="-":a+="- ",a+=e.dump);e.tag=i,e.dump=a||"[]"}U(F6,"writeBlockSequence");function ese(e,t,r){var n="",a=e.tag,i=Object.keys(r),o,s,l,u,d;for(o=0,s=i.length;o<s;o+=1)d="",n!==""&&(d+=", "),e.condenseFlow&&(d+='"'),l=i[o],u=r[l],e.replacer&&(u=e.replacer.call(r,l,u)),ds(e,t,l,!1,!1)&&(e.dump.length>1024&&(d+="? "),d+=e.dump+(e.condenseFlow?'"':"")+":"+(e.condenseFlow?"":" "),ds(e,t,u,!1,!1)&&(d+=e.dump,n+=d));e.tag=a,e.dump="{"+n+"}"}U(ese,"writeFlowMapping");function tse(e,t,r,n){var a="",i=e.tag,o=Object.keys(r),s,l,u,d,p,f;if(e.sortKeys===!0)o.sort();else if(typeof e.sortKeys=="function")o.sort(e.sortKeys);else if(e.sortKeys)throw new di("sortKeys must be a boolean or a function");for(s=0,l=o.length;s<l;s+=1)f="",(!n||a!=="")&&(f+=ev(e,t)),u=o[s],d=r[u],e.replacer&&(d=e.replacer.call(r,u,d)),ds(e,t+1,u,!0,!0,!0)&&(p=e.tag!==null&&e.tag!=="?"||e.dump&&e.dump.length>1024,p&&(e.dump&&Qm===e.dump.charCodeAt(0)?f+="?":f+="? "),f+=e.dump,p&&(f+=ev(e,t)),ds(e,t+1,d,!0,p)&&(e.dump&&Qm===e.dump.charCodeAt(0)?f+=":":f+=": ",f+=e.dump,a+=f));e.tag=i,e.dump=a||"{}"}U(tse,"writeBlockMapping");function _6(e,t,r){var n,a,i,o,s,l;for(a=r?e.explicitTypes:e.implicitTypes,i=0,o=a.length;i<o;i+=1)if(s=a[i],(s.instanceOf||s.predicate)&&(!s.instanceOf||typeof t=="object"&&t instanceof s.instanceOf)&&(!s.predicate||s.predicate(t))){if(r?s.multi&&s.representName?e.tag=s.representName(t):e.tag=s.tag:e.tag="?",s.represent){if(l=e.styleMap[s.tag]||s.defaultStyle,Loe.call(s.represent)==="[object Function]")n=s.represent(t,l);else if(Moe.call(s.represent,l))n=s.represent[l](t,l);else throw new di("!<"+s.tag+'> tag resolver accepts not "'+l+'" style');e.dump=n}return!0}return!1}U(_6,"detectType");function ds(e,t,r,n,a,i,o){e.tag=null,e.dump=r,_6(e,r,!1)||_6(e,r,!0);var s=Loe.call(e.dump),l=n,u;n&&(n=e.flowLevel<0||e.flowLevel>t);var d=s==="[object Object]"||s==="[object Array]",p,f;if(d&&(p=e.duplicates.indexOf(r),f=p!==-1),(e.tag!==null&&e.tag!=="?"||f||e.indent!==2&&t>0)&&(a=!1),f&&e.usedDuplicates[p])e.dump="*ref_"+p;else{if(d&&f&&!e.usedDuplicates[p]&&(e.usedDuplicates[p]=!0),s==="[object Object]")n&&Object.keys(e.dump).length!==0?(tse(e,t,e.dump,a),f&&(e.dump="&ref_"+p+e.dump)):(ese(e,t,e.dump),f&&(e.dump="&ref_"+p+" "+e.dump));else if(s==="[object Array]")n&&e.dump.length!==0?(e.noArrayIndent&&!o&&t>0?F6(e,t-1,e.dump,a):F6(e,t,e.dump,a),f&&(e.dump="&ref_"+p+e.dump)):(Qoe(e,t,e.dump),f&&(e.dump="&ref_"+p+" "+e.dump));else if(s==="[object String]")e.tag!=="?"&&Joe(e,e.dump,t,i,l);else{if(s==="[object Undefined]")return!1;if(e.skipInvalid)return!1;throw new di("unacceptable kind of an object to dump "+s)}e.tag!==null&&e.tag!=="?"&&(u=encodeURI(e.tag[0]==="!"?e.tag.slice(1):e.tag).replace(/!/g,"%21"),e.tag[0]==="!"?u="!"+u:u.slice(0,18)==="tag:yaml.org,2002:"?u="!!"+u.slice(18):u="!<"+u+">",e.dump=u+" "+e.dump)}return!0}U(ds,"writeNode");function rse(e,t){var r=[],n=[],a,i;for(tv(e,r,n),a=0,i=n.length;a<i;a+=1)t.duplicates.push(r[n[a]]);t.usedDuplicates=new Array(i)}U(rse,"getDuplicateReferences");function tv(e,t,r){var n,a,i;if(e!==null&&typeof e=="object")if(a=t.indexOf(e),a!==-1)r.indexOf(a)===-1&&r.push(a);else if(t.push(e),Array.isArray(e))for(a=0,i=e.length;a<i;a+=1)tv(e[a],t,r);else for(n=Object.keys(e),a=0,i=n.length;a<i;a+=1)tv(e[n[a]],t,r)}U(tv,"inspectNode");function xqe(e,t){t=t||{};var r=new $oe(t);r.noRefs||rse(e,r);var n=e;return r.replacer&&(n=r.replacer.call({"":n},"",n)),ds(r,0,n,!0,!0)?r.dump+`
`:""}U(xqe,"dump$1");function wqe(e,t){return function(){throw new Error("Function yaml."+e+" is removed in js-yaml 4. Use yaml."+t+" instead, which is now safe by default.")}}U(wqe,"renamed");var kqe=Vie,Cqe=rqe.load;/*! Bundled license information:

js-yaml/dist/js-yaml.mjs:
  (*! js-yaml 4.1.0 https://github.com/nodeca/js-yaml @license MIT *)
*/var Fa={aggregation:17.25,extension:17.25,composition:17.25,dependency:6,lollipop:13.5,arrow_point:4},gH={arrow_point:9,arrow_cross:12.5,arrow_circle:12.5};function dm(e,t){if(e===void 0||t===void 0)return{angle:0,deltaX:0,deltaY:0};e=gn(e),t=gn(t);const[r,n]=[e.x,e.y],[a,i]=[t.x,t.y],o=a-r,s=i-n;return{angle:Math.atan(s/o),deltaX:o,deltaY:s}}U(dm,"calculateDeltaAndAngle");var gn=U(e=>Array.isArray(e)?{x:e[0],y:e[1]}:e,"pointTransformer"),Tqe=U(e=>({x:U(function(t,r,n){let a=0;const i=gn(n[0]).x<gn(n[n.length-1]).x?"left":"right";if(r===0&&Object.hasOwn(Fa,e.arrowTypeStart)){const{angle:m,deltaX:b}=dm(n[0],n[1]);a=Fa[e.arrowTypeStart]*Math.cos(m)*(b>=0?1:-1)}else if(r===n.length-1&&Object.hasOwn(Fa,e.arrowTypeEnd)){const{angle:m,deltaX:b}=dm(n[n.length-1],n[n.length-2]);a=Fa[e.arrowTypeEnd]*Math.cos(m)*(b>=0?1:-1)}const o=Math.abs(gn(t).x-gn(n[n.length-1]).x),s=Math.abs(gn(t).y-gn(n[n.length-1]).y),l=Math.abs(gn(t).x-gn(n[0]).x),u=Math.abs(gn(t).y-gn(n[0]).y),d=Fa[e.arrowTypeStart],p=Fa[e.arrowTypeEnd],f=1;if(o<p&&o>0&&s<p){let m=p+f-o;m*=i==="right"?-1:1,a-=m}if(l<d&&l>0&&u<d){let m=d+f-l;m*=i==="right"?-1:1,a+=m}return gn(t).x+a},"x"),y:U(function(t,r,n){let a=0;const i=gn(n[0]).y<gn(n[n.length-1]).y?"down":"up";if(r===0&&Object.hasOwn(Fa,e.arrowTypeStart)){const{angle:m,deltaY:b}=dm(n[0],n[1]);a=Fa[e.arrowTypeStart]*Math.abs(Math.sin(m))*(b>=0?1:-1)}else if(r===n.length-1&&Object.hasOwn(Fa,e.arrowTypeEnd)){const{angle:m,deltaY:b}=dm(n[n.length-1],n[n.length-2]);a=Fa[e.arrowTypeEnd]*Math.abs(Math.sin(m))*(b>=0?1:-1)}const o=Math.abs(gn(t).y-gn(n[n.length-1]).y),s=Math.abs(gn(t).x-gn(n[n.length-1]).x),l=Math.abs(gn(t).y-gn(n[0]).y),u=Math.abs(gn(t).x-gn(n[0]).x),d=Fa[e.arrowTypeStart],p=Fa[e.arrowTypeEnd],f=1;if(o<p&&o>0&&s<p){let m=p+f-o;m*=i==="up"?-1:1,a-=m}if(l<d&&l>0&&u<d){let m=d+f-l;m*=i==="up"?-1:1,a+=m}return gn(t).y+a},"y")}),"getLineFunctionsWithOffset"),IN=U(({flowchart:e})=>{var a,i;const t=((a=e==null?void 0:e.subGraphTitleMargin)==null?void 0:a.top)??0,r=((i=e==null?void 0:e.subGraphTitleMargin)==null?void 0:i.bottom)??0,n=t+r;return{subGraphTitleTopMargin:t,subGraphTitleBottomMargin:r,subGraphTitleTotalMargin:n}},"getSubGraphTitleMargins"),Aqe=U(e=>{const{handDrawnSeed:t}=Gr();return{fill:e,hachureAngle:120,hachureGap:4,fillWeight:2,roughness:.7,stroke:e,seed:t}},"solidStateFill"),Rp=U(e=>{const t=Fqe([...e.cssCompiledStyles||[],...e.cssStyles||[],...e.labelStyle||[]]);return{stylesMap:t,stylesArray:[...t]}},"compileStyles"),Fqe=U(e=>{const t=new Map;return e.forEach(r=>{const[n,a]=r.split(":");t.set(n.trim(),a==null?void 0:a.trim())}),t},"styles2Map"),nse=U(e=>e==="color"||e==="font-size"||e==="font-family"||e==="font-weight"||e==="font-style"||e==="text-decoration"||e==="text-align"||e==="text-transform"||e==="line-height"||e==="letter-spacing"||e==="word-spacing"||e==="text-shadow"||e==="text-overflow"||e==="white-space"||e==="word-wrap"||e==="word-break"||e==="overflow-wrap"||e==="hyphens","isLabelStyle"),Nt=U(e=>{const{stylesArray:t}=Rp(e),r=[],n=[],a=[],i=[];return t.forEach(o=>{const s=o[0];nse(s)?r.push(o.join(":")+" !important"):(n.push(o.join(":")+" !important"),s.includes("stroke")&&a.push(o.join(":")+" !important"),s==="fill"&&i.push(o.join(":")+" !important"))}),{labelStyles:r.join(";"),nodeStyles:n.join(";"),stylesArray:t,borderStyles:a,backgroundStyles:i}},"styles2String"),Dt=U((e,t)=>{var l;const{themeVariables:r,handDrawnSeed:n}=Gr(),{nodeBorder:a,mainBkg:i}=r,{stylesMap:o}=Rp(e);return Object.assign({roughness:.7,fill:o.get("fill")||i,fillStyle:"hachure",fillWeight:4,hachureGap:5.2,stroke:o.get("stroke")||a,seed:n,strokeWidth:((l=o.get("stroke-width"))==null?void 0:l.replace("px",""))||1.3,fillLineDash:[0,0],strokeLineDash:_qe(o.get("stroke-dasharray"))},t)},"userNodeOverrides"),_qe=U(e=>{if(!e)return[0,0];const t=e.trim().split(/\s+/).map(Number);if(t.length===1){const a=isNaN(t[0])?0:t[0];return[a,a]}const r=isNaN(t[0])?0:t[0],n=isNaN(t[1])?0:t[1];return[r,n]},"getStrokeDashArray"),Yf={},Jn={},bH;function Rqe(){return bH||(bH=1,Object.defineProperty(Jn,"__esModule",{value:!0}),Jn.BLANK_URL=Jn.relativeFirstCharacters=Jn.whitespaceEscapeCharsRegex=Jn.urlSchemeRegex=Jn.ctrlCharactersRegex=Jn.htmlCtrlEntityRegex=Jn.htmlEntitiesRegex=Jn.invalidProtocolRegex=void 0,Jn.invalidProtocolRegex=/^([^\w]*)(javascript|data|vbscript)/im,Jn.htmlEntitiesRegex=/&#(\w+)(^\w|;)?/g,Jn.htmlCtrlEntityRegex=/&(newline|tab);/gi,Jn.ctrlCharactersRegex=/[\u0000-\u001F\u007F-\u009F\u2000-\u200D\uFEFF]/gim,Jn.urlSchemeRegex=/^.+(:|&colon;)/gim,Jn.whitespaceEscapeCharsRegex=/(\\|%5[cC])((%(6[eE]|72|74))|[nrt])/g,Jn.relativeFirstCharacters=[".","/"],Jn.BLANK_URL="about:blank"),Jn}var yH;function Dqe(){if(yH)return Yf;yH=1,Object.defineProperty(Yf,"__esModule",{value:!0}),Yf.sanitizeUrl=void 0;var e=Rqe();function t(o){return e.relativeFirstCharacters.indexOf(o[0])>-1}function r(o){var s=o.replace(e.ctrlCharactersRegex,"");return s.replace(e.htmlEntitiesRegex,function(l,u){return String.fromCharCode(u)})}function n(o){return URL.canParse(o)}function a(o){try{return decodeURIComponent(o)}catch{return o}}function i(o){if(!o)return e.BLANK_URL;var s,l=a(o.trim());do l=r(l).replace(e.htmlCtrlEntityRegex,"").replace(e.ctrlCharactersRegex,"").replace(e.whitespaceEscapeCharsRegex,"").trim(),l=a(l),s=l.match(e.ctrlCharactersRegex)||l.match(e.htmlEntitiesRegex)||l.match(e.htmlCtrlEntityRegex)||l.match(e.whitespaceEscapeCharsRegex);while(s&&s.length>0);var u=l;if(!u)return e.BLANK_URL;if(t(u))return u;var d=u.trimStart(),p=d.match(e.urlSchemeRegex);if(!p)return u;var f=p[0].toLowerCase().trim();if(e.invalidProtocolRegex.test(f))return e.BLANK_URL;var m=d.replace(/\\/g,"/");if(f==="mailto:"||f.includes("://"))return m;if(f==="http:"||f==="https:"){if(!n(m))return e.BLANK_URL;var b=new URL(m);return b.protocol=b.protocol.toLowerCase(),b.hostname=b.hostname.toLowerCase(),b.toString()}return m}return Yf.sanitizeUrl=i,Yf}var Nqe=Dqe(),ase=typeof global=="object"&&global&&global.Object===Object&&global,Lqe=typeof self=="object"&&self&&self.Object===Object&&self,xs=ase||Lqe||Function("return this")(),rv=xs.Symbol,ise=Object.prototype,Mqe=ise.hasOwnProperty,Iqe=ise.toString,Jf=rv?rv.toStringTag:void 0;function Oqe(e){var t=Mqe.call(e,Jf),r=e[Jf];try{e[Jf]=void 0;var n=!0}catch{}var a=Iqe.call(e);return n&&(t?e[Jf]=r:delete e[Jf]),a}var Bqe=Object.prototype,Pqe=Bqe.toString;function zqe(e){return Pqe.call(e)}var Hqe="[object Null]",Uqe="[object Undefined]",vH=rv?rv.toStringTag:void 0;function Dp(e){return e==null?e===void 0?Uqe:Hqe:vH&&vH in Object(e)?Oqe(e):zqe(e)}function ld(e){var t=typeof e;return e!=null&&(t=="object"||t=="function")}var $qe="[object AsyncFunction]",Gqe="[object Function]",jqe="[object GeneratorFunction]",qqe="[object Proxy]";function ON(e){if(!ld(e))return!1;var t=Dp(e);return t==Gqe||t==jqe||t==$qe||t==qqe}var Mk=xs["__core-js_shared__"],SH=(function(){var e=/[^.]+$/.exec(Mk&&Mk.keys&&Mk.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""})();function Wqe(e){return!!SH&&SH in e}var Vqe=Function.prototype,Kqe=Vqe.toString;function ud(e){if(e!=null){try{return Kqe.call(e)}catch{}try{return e+""}catch{}}return""}var Yqe=/[\\^$.*+?()[\]{}|]/g,Jqe=/^\[object .+?Constructor\]$/,Xqe=Function.prototype,Zqe=Object.prototype,Qqe=Xqe.toString,eWe=Zqe.hasOwnProperty,tWe=RegExp("^"+Qqe.call(eWe).replace(Yqe,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function rWe(e){if(!ld(e)||Wqe(e))return!1;var t=ON(e)?tWe:Jqe;return t.test(ud(e))}function nWe(e,t){return e==null?void 0:e[t]}function cd(e,t){var r=nWe(e,t);return rWe(r)?r:void 0}var rg=cd(Object,"create");function aWe(){this.__data__=rg?rg(null):{},this.size=0}function iWe(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var oWe="__lodash_hash_undefined__",sWe=Object.prototype,lWe=sWe.hasOwnProperty;function uWe(e){var t=this.__data__;if(rg){var r=t[e];return r===oWe?void 0:r}return lWe.call(t,e)?t[e]:void 0}var cWe=Object.prototype,dWe=cWe.hasOwnProperty;function hWe(e){var t=this.__data__;return rg?t[e]!==void 0:dWe.call(t,e)}var pWe="__lodash_hash_undefined__";function fWe(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=rg&&t===void 0?pWe:t,this}function Xc(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1])}}Xc.prototype.clear=aWe;Xc.prototype.delete=iWe;Xc.prototype.get=uWe;Xc.prototype.has=hWe;Xc.prototype.set=fWe;function mWe(){this.__data__=[],this.size=0}function H2(e,t){return e===t||e!==e&&t!==t}function U2(e,t){for(var r=e.length;r--;)if(H2(e[r][0],t))return r;return-1}var gWe=Array.prototype,bWe=gWe.splice;function yWe(e){var t=this.__data__,r=U2(t,e);if(r<0)return!1;var n=t.length-1;return r==n?t.pop():bWe.call(t,r,1),--this.size,!0}function vWe(e){var t=this.__data__,r=U2(t,e);return r<0?void 0:t[r][1]}function SWe(e){return U2(this.__data__,e)>-1}function EWe(e,t){var r=this.__data__,n=U2(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this}function Dl(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1])}}Dl.prototype.clear=mWe;Dl.prototype.delete=yWe;Dl.prototype.get=vWe;Dl.prototype.has=SWe;Dl.prototype.set=EWe;var ng=cd(xs,"Map");function xWe(){this.size=0,this.__data__={hash:new Xc,map:new(ng||Dl),string:new Xc}}function wWe(e){var t=typeof e;return t=="string"||t=="number"||t=="symbol"||t=="boolean"?e!=="__proto__":e===null}function $2(e,t){var r=e.__data__;return wWe(t)?r[typeof t=="string"?"string":"hash"]:r.map}function kWe(e){var t=$2(this,e).delete(e);return this.size-=t?1:0,t}function CWe(e){return $2(this,e).get(e)}function TWe(e){return $2(this,e).has(e)}function AWe(e,t){var r=$2(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this}function $u(e){var t=-1,r=e==null?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1])}}$u.prototype.clear=xWe;$u.prototype.delete=kWe;$u.prototype.get=CWe;$u.prototype.has=TWe;$u.prototype.set=AWe;var FWe="Expected a function";function qg(e,t){if(typeof e!="function"||t!=null&&typeof t!="function")throw new TypeError(FWe);var r=function(){var n=arguments,a=t?t.apply(this,n):n[0],i=r.cache;if(i.has(a))return i.get(a);var o=e.apply(this,n);return r.cache=i.set(a,o)||i,o};return r.cache=new(qg.Cache||$u),r}qg.Cache=$u;function _We(){this.__data__=new Dl,this.size=0}function RWe(e){var t=this.__data__,r=t.delete(e);return this.size=t.size,r}function DWe(e){return this.__data__.get(e)}function NWe(e){return this.__data__.has(e)}var LWe=200;function MWe(e,t){var r=this.__data__;if(r instanceof Dl){var n=r.__data__;if(!ng||n.length<LWe-1)return n.push([e,t]),this.size=++r.size,this;r=this.__data__=new $u(n)}return r.set(e,t),this.size=r.size,this}function Np(e){var t=this.__data__=new Dl(e);this.size=t.size}Np.prototype.clear=_We;Np.prototype.delete=RWe;Np.prototype.get=DWe;Np.prototype.has=NWe;Np.prototype.set=MWe;var nv=(function(){try{var e=cd(Object,"defineProperty");return e({},"",{}),e}catch{}})();function BN(e,t,r){t=="__proto__"&&nv?nv(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r}function R6(e,t,r){(r!==void 0&&!H2(e[t],r)||r===void 0&&!(t in e))&&BN(e,t,r)}function IWe(e){return function(t,r,n){for(var a=-1,i=Object(t),o=n(t),s=o.length;s--;){var l=o[++a];if(r(i[l],l,i)===!1)break}return t}}var OWe=IWe(),ose=typeof exports=="object"&&exports&&!exports.nodeType&&exports,EH=ose&&typeof module=="object"&&module&&!module.nodeType&&module,BWe=EH&&EH.exports===ose,xH=BWe?xs.Buffer:void 0,wH=xH?xH.allocUnsafe:void 0;function PWe(e,t){if(t)return e.slice();var r=e.length,n=wH?wH(r):new e.constructor(r);return e.copy(n),n}var kH=xs.Uint8Array;function zWe(e){var t=new e.constructor(e.byteLength);return new kH(t).set(new kH(e)),t}function HWe(e,t){var r=t?zWe(e.buffer):e.buffer;return new e.constructor(r,e.byteOffset,e.length)}function UWe(e,t){var r=-1,n=e.length;for(t||(t=Array(n));++r<n;)t[r]=e[r];return t}var CH=Object.create,$We=(function(){function e(){}return function(t){if(!ld(t))return{};if(CH)return CH(t);e.prototype=t;var r=new e;return e.prototype=void 0,r}})();function sse(e,t){return function(r){return e(t(r))}}var lse=sse(Object.getPrototypeOf,Object),GWe=Object.prototype;function G2(e){var t=e&&e.constructor,r=typeof t=="function"&&t.prototype||GWe;return e===r}function jWe(e){return typeof e.constructor=="function"&&!G2(e)?$We(lse(e)):{}}function Wg(e){return e!=null&&typeof e=="object"}var qWe="[object Arguments]";function TH(e){return Wg(e)&&Dp(e)==qWe}var use=Object.prototype,WWe=use.hasOwnProperty,VWe=use.propertyIsEnumerable,av=TH((function(){return arguments})())?TH:function(e){return Wg(e)&&WWe.call(e,"callee")&&!VWe.call(e,"callee")},iv=Array.isArray,KWe=9007199254740991;function cse(e){return typeof e=="number"&&e>-1&&e%1==0&&e<=KWe}function j2(e){return e!=null&&cse(e.length)&&!ON(e)}function YWe(e){return Wg(e)&&j2(e)}function JWe(){return!1}var dse=typeof exports=="object"&&exports&&!exports.nodeType&&exports,AH=dse&&typeof module=="object"&&module&&!module.nodeType&&module,XWe=AH&&AH.exports===dse,FH=XWe?xs.Buffer:void 0,ZWe=FH?FH.isBuffer:void 0,PN=ZWe||JWe,QWe="[object Object]",eVe=Function.prototype,tVe=Object.prototype,hse=eVe.toString,rVe=tVe.hasOwnProperty,nVe=hse.call(Object);function aVe(e){if(!Wg(e)||Dp(e)!=QWe)return!1;var t=lse(e);if(t===null)return!0;var r=rVe.call(t,"constructor")&&t.constructor;return typeof r=="function"&&r instanceof r&&hse.call(r)==nVe}var iVe="[object Arguments]",oVe="[object Array]",sVe="[object Boolean]",lVe="[object Date]",uVe="[object Error]",cVe="[object Function]",dVe="[object Map]",hVe="[object Number]",pVe="[object Object]",fVe="[object RegExp]",mVe="[object Set]",gVe="[object String]",bVe="[object WeakMap]",yVe="[object ArrayBuffer]",vVe="[object DataView]",SVe="[object Float32Array]",EVe="[object Float64Array]",xVe="[object Int8Array]",wVe="[object Int16Array]",kVe="[object Int32Array]",CVe="[object Uint8Array]",TVe="[object Uint8ClampedArray]",AVe="[object Uint16Array]",FVe="[object Uint32Array]",cn={};cn[SVe]=cn[EVe]=cn[xVe]=cn[wVe]=cn[kVe]=cn[CVe]=cn[TVe]=cn[AVe]=cn[FVe]=!0;cn[iVe]=cn[oVe]=cn[yVe]=cn[sVe]=cn[vVe]=cn[lVe]=cn[uVe]=cn[cVe]=cn[dVe]=cn[hVe]=cn[pVe]=cn[fVe]=cn[mVe]=cn[gVe]=cn[bVe]=!1;function _Ve(e){return Wg(e)&&cse(e.length)&&!!cn[Dp(e)]}function RVe(e){return function(t){return e(t)}}var pse=typeof exports=="object"&&exports&&!exports.nodeType&&exports,Dm=pse&&typeof module=="object"&&module&&!module.nodeType&&module,DVe=Dm&&Dm.exports===pse,Ik=DVe&&ase.process,_H=(function(){try{var e=Dm&&Dm.require&&Dm.require("util").types;return e||Ik&&Ik.binding&&Ik.binding("util")}catch{}})(),RH=_H&&_H.isTypedArray,zN=RH?RVe(RH):_Ve;function D6(e,t){if(!(t==="constructor"&&typeof e[t]=="function")&&t!="__proto__")return e[t]}var NVe=Object.prototype,LVe=NVe.hasOwnProperty;function MVe(e,t,r){var n=e[t];(!(LVe.call(e,t)&&H2(n,r))||r===void 0&&!(t in e))&&BN(e,t,r)}function IVe(e,t,r,n){var a=!r;r||(r={});for(var i=-1,o=t.length;++i<o;){var s=t[i],l=void 0;l===void 0&&(l=e[s]),a?BN(r,s,l):MVe(r,s,l)}return r}function OVe(e,t){for(var r=-1,n=Array(e);++r<e;)n[r]=t(r);return n}var BVe=9007199254740991,PVe=/^(?:0|[1-9]\d*)$/;function fse(e,t){var r=typeof e;return t=t??BVe,!!t&&(r=="number"||r!="symbol"&&PVe.test(e))&&e>-1&&e%1==0&&e<t}var zVe=Object.prototype,HVe=zVe.hasOwnProperty;function UVe(e,t){var r=iv(e),n=!r&&av(e),a=!r&&!n&&PN(e),i=!r&&!n&&!a&&zN(e),o=r||n||a||i,s=o?OVe(e.length,String):[],l=s.length;for(var u in e)(t||HVe.call(e,u))&&!(o&&(u=="length"||a&&(u=="offset"||u=="parent")||i&&(u=="buffer"||u=="byteLength"||u=="byteOffset")||fse(u,l)))&&s.push(u);return s}function $Ve(e){var t=[];if(e!=null)for(var r in Object(e))t.push(r);return t}var GVe=Object.prototype,jVe=GVe.hasOwnProperty;function qVe(e){if(!ld(e))return $Ve(e);var t=G2(e),r=[];for(var n in e)n=="constructor"&&(t||!jVe.call(e,n))||r.push(n);return r}function mse(e){return j2(e)?UVe(e,!0):qVe(e)}function WVe(e){return IVe(e,mse(e))}function VVe(e,t,r,n,a,i,o){var s=D6(e,r),l=D6(t,r),u=o.get(l);if(u){R6(e,r,u);return}var d=i?i(s,l,r+"",e,t,o):void 0,p=d===void 0;if(p){var f=iv(l),m=!f&&PN(l),b=!f&&!m&&zN(l);d=l,f||m||b?iv(s)?d=s:YWe(s)?d=UWe(s):m?(p=!1,d=PWe(l,!0)):b?(p=!1,d=HWe(l,!0)):d=[]:aVe(l)||av(l)?(d=s,av(s)?d=WVe(s):(!ld(s)||ON(s))&&(d=jWe(l))):p=!1}p&&(o.set(l,d),a(d,l,n,i,o),o.delete(l)),R6(e,r,d)}function gse(e,t,r,n,a){e!==t&&OWe(t,function(i,o){if(a||(a=new Np),ld(i))VVe(e,t,o,r,gse,n,a);else{var s=n?n(D6(e,o),i,o+"",e,t,a):void 0;s===void 0&&(s=i),R6(e,o,s)}},mse)}function bse(e){return e}function KVe(e,t,r){switch(r.length){case 0:return e.call(t);case 1:return e.call(t,r[0]);case 2:return e.call(t,r[0],r[1]);case 3:return e.call(t,r[0],r[1],r[2])}return e.apply(t,r)}var DH=Math.max;function YVe(e,t,r){return t=DH(t===void 0?e.length-1:t,0),function(){for(var n=arguments,a=-1,i=DH(n.length-t,0),o=Array(i);++a<i;)o[a]=n[t+a];a=-1;for(var s=Array(t+1);++a<t;)s[a]=n[a];return s[t]=r(o),KVe(e,this,s)}}function JVe(e){return function(){return e}}var XVe=nv?function(e,t){return nv(e,"toString",{configurable:!0,enumerable:!1,value:JVe(t),writable:!0})}:bse,ZVe=800,QVe=16,eKe=Date.now;function tKe(e){var t=0,r=0;return function(){var n=eKe(),a=QVe-(n-r);if(r=n,a>0){if(++t>=ZVe)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}var rKe=tKe(XVe);function nKe(e,t){return rKe(YVe(e,t,bse),e+"")}function aKe(e,t,r){if(!ld(r))return!1;var n=typeof t;return(n=="number"?j2(r)&&fse(t,r.length):n=="string"&&t in r)?H2(r[t],e):!1}function iKe(e){return nKe(function(t,r){var n=-1,a=r.length,i=a>1?r[a-1]:void 0,o=a>2?r[2]:void 0;for(i=e.length>3&&typeof i=="function"?(a--,i):void 0,o&&aKe(r[0],r[1],o)&&(i=a<3?void 0:i,a=1),t=Object(t);++n<a;){var s=r[n];s&&e(t,s,n,i)}return t})}var oKe=iKe(function(e,t,r){gse(e,t,r)}),sKe="​",lKe={curveBasis:q1,curveBasisClosed:cje,curveBasisOpen:dje,curveBumpX:rie,curveBumpY:nie,curveBundle:hje,curveCardinalClosed:pje,curveCardinalOpen:fje,curveCardinal:sie,curveCatmullRomClosed:mje,curveCatmullRomOpen:gje,curveCatmullRom:uie,curveLinear:Wy,curveLinearClosed:bje,curveMonotoneX:mie,curveMonotoneY:gie,curveNatural:yie,curveStep:vie,curveStepAfter:Eie,curveStepBefore:Sie},uKe=/\s*(?:(\w+)(?=:):|(\w+))\s*(?:(\w+)|((?:(?!}%{2}).|\r?\n)*))?\s*(?:}%{2})?/gi,cKe=U(function(e,t){const r=yse(e,/(?:init\b)|(?:initialize\b)/);let n={};if(Array.isArray(r)){const o=r.map(s=>s.args);Iy(o),n=Qn(n,[...o])}else n=r.args;if(!n)return;let a=uN(e,t);const i="config";return n[i]!==void 0&&(a==="flowchart-v2"&&(a="flowchart"),n[a]=n[i],delete n[i]),n},"detectInit"),yse=U(function(e,t=null){var r,n;try{const a=new RegExp(`[%]{2}(?![{]${uKe.source})(?=[}][%]{2}).*
`,"ig");e=e.trim().replace(a,"").replace(/'/gm,'"'),je.debug(`Detecting diagram directive${t!==null?" type:"+t:""} based on the text:${e}`);let i;const o=[];for(;(i=_m.exec(e))!==null;)if(i.index===_m.lastIndex&&_m.lastIndex++,i&&!t||t&&((r=i[1])!=null&&r.match(t))||t&&((n=i[2])!=null&&n.match(t))){const s=i[1]?i[1]:i[2],l=i[3]?i[3].trim():i[4]?JSON.parse(i[4].trim()):null;o.push({type:s,args:l})}return o.length===0?{type:e,args:null}:o.length===1?o[0]:o}catch(a){return je.error(`ERROR: ${a.message} - Unable to parse directive type: '${t}' based on the text: '${e}'`),{type:void 0,args:null}}},"detectDirective"),dKe=U(function(e){return e.replace(_m,"")},"removeDirectives"),hKe=U(function(e,t){for(const[r,n]of t.entries())if(n.match(e))return r;return-1},"isSubstringInArray");function HN(e,t){if(!e)return t;const r=`curve${e.charAt(0).toUpperCase()+e.slice(1)}`;return lKe[r]??t}U(HN,"interpolateToCurve");function vse(e,t){const r=e.trim();if(r)return t.securityLevel!=="loose"?Nqe.sanitizeUrl(r):r}U(vse,"formatUrl");var pKe=U((e,...t)=>{const r=e.split("."),n=r.length-1,a=r[n];let i=window;for(let o=0;o<n;o++)if(i=i[r[o]],!i){je.error(`Function name: ${e} not found in window`);return}i[a](...t)},"runFunc");function UN(e,t){return!e||!t?0:Math.sqrt(Math.pow(t.x-e.x,2)+Math.pow(t.y-e.y,2))}U(UN,"distance");function Sse(e){let t,r=0;e.forEach(a=>{r+=UN(a,t),t=a});const n=r/2;return $N(e,n)}U(Sse,"traverseEdge");function Ese(e){return e.length===1?e[0]:Sse(e)}U(Ese,"calcLabelPosition");var NH=U((e,t=2)=>{const r=Math.pow(10,t);return Math.round(e*r)/r},"roundNumber"),$N=U((e,t)=>{let r,n=t;for(const a of e){if(r){const i=UN(a,r);if(i===0)return r;if(i<n)n-=i;else{const o=n/i;if(o<=0)return r;if(o>=1)return{x:a.x,y:a.y};if(o>0&&o<1)return{x:NH((1-o)*r.x+o*a.x,5),y:NH((1-o)*r.y+o*a.y,5)}}}r=a}throw new Error("Could not find a suitable point for the given distance")},"calculatePoint"),fKe=U((e,t,r)=>{je.info(`our points ${JSON.stringify(t)}`),t[0]!==r&&(t=t.reverse());const a=$N(t,25),i=e?10:5,o=Math.atan2(t[0].y-a.y,t[0].x-a.x),s={x:0,y:0};return s.x=Math.sin(o)*i+(t[0].x+a.x)/2,s.y=-Math.cos(o)*i+(t[0].y+a.y)/2,s},"calcCardinalityPosition");function xse(e,t,r){const n=structuredClone(r);je.info("our points",n),t!=="start_left"&&t!=="start_right"&&n.reverse();const a=25+e,i=$N(n,a),o=10+e*.5,s=Math.atan2(n[0].y-i.y,n[0].x-i.x),l={x:0,y:0};return t==="start_left"?(l.x=Math.sin(s+Math.PI)*o+(n[0].x+i.x)/2,l.y=-Math.cos(s+Math.PI)*o+(n[0].y+i.y)/2):t==="end_right"?(l.x=Math.sin(s-Math.PI)*o+(n[0].x+i.x)/2-5,l.y=-Math.cos(s-Math.PI)*o+(n[0].y+i.y)/2-5):t==="end_left"?(l.x=Math.sin(s)*o+(n[0].x+i.x)/2-5,l.y=-Math.cos(s)*o+(n[0].y+i.y)/2-5):(l.x=Math.sin(s)*o+(n[0].x+i.x)/2,l.y=-Math.cos(s)*o+(n[0].y+i.y)/2),l}U(xse,"calcTerminalLabelPosition");function wse(e){let t="",r="";for(const n of e)n!==void 0&&(n.startsWith("color:")||n.startsWith("text-align:")?r=r+n+";":t=t+n+";");return{style:t,labelStyle:r}}U(wse,"getStylesFromArray");var LH=0,mKe=U(()=>(LH++,"id-"+Math.random().toString(36).substr(2,12)+"-"+LH),"generateId");function kse(e){let t="";const r="0123456789abcdef",n=r.length;for(let a=0;a<e;a++)t+=r.charAt(Math.floor(Math.random()*n));return t}U(kse,"makeRandomHex");var gKe=U(e=>kse(e.length),"random"),bKe=U(function(){return{x:0,y:0,fill:void 0,anchor:"start",style:"#666",width:100,height:100,textMargin:0,rx:0,ry:0,valign:void 0,text:""}},"getTextObj"),yKe=U(function(e,t){const r=t.text.replace(_p.lineBreakRegex," "),[,n]=q2(t.fontSize),a=e.append("text");a.attr("x",t.x),a.attr("y",t.y),a.style("text-anchor",t.anchor),a.style("font-family",t.fontFamily),a.style("font-size",n),a.style("font-weight",t.fontWeight),a.attr("fill",t.fill),t.class!==void 0&&a.attr("class",t.class);const i=a.append("tspan");return i.attr("x",t.x+t.textMargin*2),i.attr("fill",t.fill),i.text(r),a},"drawSimpleText"),vKe=qg((e,t,r)=>{if(!e||(r=Object.assign({fontSize:12,fontWeight:400,fontFamily:"Arial",joinWith:"<br/>"},r),_p.lineBreakRegex.test(e)))return e;const n=e.split(" ").filter(Boolean),a=[];let i="";return n.forEach((o,s)=>{const l=Cl(`${o} `,r),u=Cl(i,r);if(l>t){const{hyphenatedStrings:f,remainingWord:m}=SKe(o,t,"-",r);a.push(i,...f),i=m}else u+l>=t?(a.push(i),i=o):i=[i,o].filter(Boolean).join(" ");s+1===n.length&&a.push(i)}),a.filter(o=>o!=="").join(r.joinWith)},(e,t,r)=>`${e}${t}${r.fontSize}${r.fontWeight}${r.fontFamily}${r.joinWith}`),SKe=qg((e,t,r="-",n)=>{n=Object.assign({fontSize:12,fontWeight:400,fontFamily:"Arial",margin:0},n);const a=[...e],i=[];let o="";return a.forEach((s,l)=>{const u=`${o}${s}`;if(Cl(u,n)>=t){const p=l+1,f=a.length===p,m=`${u}${r}`;i.push(f?u:m),o=""}else o=u}),{hyphenatedStrings:i,remainingWord:o}},(e,t,r="-",n)=>`${e}${t}${r}${n.fontSize}${n.fontWeight}${n.fontFamily}`);function Cse(e,t){return GN(e,t).height}U(Cse,"calculateTextHeight");function Cl(e,t){return GN(e,t).width}U(Cl,"calculateTextWidth");var GN=qg((e,t)=>{const{fontSize:r=12,fontFamily:n="Arial",fontWeight:a=400}=t;if(!e)return{width:0,height:0};const[,i]=q2(r),o=["sans-serif",n],s=e.split(_p.lineBreakRegex),l=[],u=Lr("body");if(!u.remove)return{width:0,height:0,lineHeight:0};const d=u.append("svg");for(const f of o){let m=0;const b={width:0,height:0,lineHeight:0};for(const y of s){const S=bKe();S.text=y||sKe;const w=yKe(d,S).style("font-size",i).style("font-weight",a).style("font-family",f),E=(w._groups||w)[0][0].getBBox();if(E.width===0&&E.height===0)throw new Error("svg element not in render tree");b.width=Math.round(Math.max(b.width,E.width)),m=Math.round(E.height),b.height+=m,b.lineHeight=Math.round(Math.max(b.lineHeight,m))}l.push(b)}d.remove();const p=isNaN(l[1].height)||isNaN(l[1].width)||isNaN(l[1].lineHeight)||l[0].height>l[1].height&&l[0].width>l[1].width&&l[0].lineHeight>l[1].lineHeight?0:1;return l[p]},(e,t)=>`${e}${t.fontSize}${t.fontWeight}${t.fontFamily}`),Ih,EKe=(Ih=class{constructor(t=!1,r){this.count=0,this.count=r?r.length:0,this.next=t?()=>this.count++:()=>Date.now()}},U(Ih,"InitIDGenerator"),Ih),i1,xKe=U(function(e){return i1=i1||document.createElement("div"),e=escape(e).replace(/%26/g,"&").replace(/%23/g,"#").replace(/%3B/g,";"),i1.innerHTML=e,unescape(i1.textContent)},"entityDecode");function jN(e){return"str"in e}U(jN,"isDetailedError");var wKe=U((e,t,r,n)=>{var i;if(!n)return;const a=(i=e.node())==null?void 0:i.getBBox();a&&e.append("text").text(n).attr("text-anchor","middle").attr("x",a.x+a.width/2).attr("y",-r).attr("class",t)},"insertTitle"),q2=U(e=>{if(typeof e=="number")return[e,e+"px"];const t=parseInt(e??"",10);return Number.isNaN(t)?[void 0,void 0]:e===String(t)?[t,e+"px"]:[t,e]},"parseFontSize");function qN(e,t){return oKe({},e,t)}U(qN,"cleanAndMerge");var _o={assignWithDepth:Qn,wrapLabel:vKe,calculateTextHeight:Cse,calculateTextWidth:Cl,calculateTextDimensions:GN,cleanAndMerge:qN,detectInit:cKe,detectDirective:yse,isSubstringInArray:hKe,interpolateToCurve:HN,calcLabelPosition:Ese,calcCardinalityPosition:fKe,calcTerminalLabelPosition:xse,formatUrl:vse,getStylesFromArray:wse,generateId:mKe,random:gKe,runFunc:pKe,entityDecode:xKe,insertTitle:wKe,isLabelCoordinateInPath:Tse,parseFontSize:q2,InitIDGenerator:EKe},kKe=U(function(e){let t=e;return t=t.replace(/style.*:\S*#.*;/g,function(r){return r.substring(0,r.length-1)}),t=t.replace(/classDef.*:\S*#.*;/g,function(r){return r.substring(0,r.length-1)}),t=t.replace(/#\w+;/g,function(r){const n=r.substring(1,r.length-1);return/^\+?\d+$/.test(n)?"ﬂ°°"+n+"¶ß":"ﬂ°"+n+"¶ß"}),t},"encodeEntities"),dd=U(function(e){return e.replace(/ﬂ°°/g,"&#").replace(/ﬂ°/g,"&").replace(/¶ß/g,";")},"decodeEntities"),Pdt=U((e,t,{counter:r=0,prefix:n,suffix:a},i)=>i||`${n?`${n}_`:""}${e}_${t}_${r}${a?`_${a}`:""}`,"getEdgeId");function Oa(e){return e??null}U(Oa,"handleUndefinedAttr");function Tse(e,t){const r=Math.round(e.x),n=Math.round(e.y),a=t.replace(/(\d+\.\d+)/g,i=>Math.round(parseFloat(i)).toString());return a.includes(r.toString())||a.includes(n.toString())}U(Tse,"isLabelCoordinateInPath");const CKe=Object.freeze({left:0,top:0,width:16,height:16}),ov=Object.freeze({rotate:0,vFlip:!1,hFlip:!1}),Ase=Object.freeze({...CKe,...ov}),TKe=Object.freeze({...Ase,body:"",hidden:!1}),AKe=Object.freeze({width:null,height:null}),FKe=Object.freeze({...AKe,...ov}),_Ke=(e,t,r,n="")=>{const a=e.split(":");if(e.slice(0,1)==="@"){if(a.length<2||a.length>3)return null;n=a.shift().slice(1)}if(a.length>3||!a.length)return null;if(a.length>1){const s=a.pop(),l=a.pop(),u={provider:a.length>0?a[0]:n,prefix:l,name:s};return Ok(u)?u:null}const i=a[0],o=i.split("-");if(o.length>1){const s={provider:n,prefix:o.shift(),name:o.join("-")};return Ok(s)?s:null}if(r&&n===""){const s={provider:n,prefix:"",name:i};return Ok(s,r)?s:null}return null},Ok=(e,t)=>e?!!((t&&e.prefix===""||e.prefix)&&e.name):!1;function RKe(e,t){const r={};!e.hFlip!=!t.hFlip&&(r.hFlip=!0),!e.vFlip!=!t.vFlip&&(r.vFlip=!0);const n=((e.rotate||0)+(t.rotate||0))%4;return n&&(r.rotate=n),r}function MH(e,t){const r=RKe(e,t);for(const n in TKe)n in ov?n in e&&!(n in r)&&(r[n]=ov[n]):n in t?r[n]=t[n]:n in e&&(r[n]=e[n]);return r}function DKe(e,t){const r=e.icons,n=e.aliases||Object.create(null),a=Object.create(null);function i(o){if(r[o])return a[o]=[];if(!(o in a)){a[o]=null;const s=n[o]&&n[o].parent,l=s&&i(s);l&&(a[o]=[s].concat(l))}return a[o]}return(t||Object.keys(r).concat(Object.keys(n))).forEach(i),a}function IH(e,t,r){const n=e.icons,a=e.aliases||Object.create(null);let i={};function o(s){i=MH(n[s]||a[s],i)}return o(t),r.forEach(o),MH(e,i)}function NKe(e,t){if(e.icons[t])return IH(e,t,[]);const r=DKe(e,[t])[t];return r?IH(e,t,r):null}const LKe=/(-?[0-9.]*[0-9]+[0-9.]*)/g,MKe=/^-?[0-9.]*[0-9]+[0-9.]*$/g;function OH(e,t,r){if(t===1)return e;if(r=r||100,typeof e=="number")return Math.ceil(e*t*r)/r;if(typeof e!="string")return e;const n=e.split(LKe);if(n===null||!n.length)return e;const a=[];let i=n.shift(),o=MKe.test(i);for(;;){if(o){const s=parseFloat(i);isNaN(s)?a.push(i):a.push(Math.ceil(s*t*r)/r)}else a.push(i);if(i=n.shift(),i===void 0)return a.join("");o=!o}}function IKe(e,t="defs"){let r="";const n=e.indexOf("<"+t);for(;n>=0;){const a=e.indexOf(">",n),i=e.indexOf("</"+t);if(a===-1||i===-1)break;const o=e.indexOf(">",i);if(o===-1)break;r+=e.slice(a+1,i).trim(),e=e.slice(0,n).trim()+e.slice(o+1)}return{defs:r,content:e}}function OKe(e,t){return e?"<defs>"+e+"</defs>"+t:t}function BKe(e,t,r){const n=IKe(e);return OKe(n.defs,t+n.content+r)}const PKe=e=>e==="unset"||e==="undefined"||e==="none";function zKe(e,t){const r={...Ase,...e},n={...FKe,...t},a={left:r.left,top:r.top,width:r.width,height:r.height};let i=r.body;[r,n].forEach(y=>{const S=[],w=y.hFlip,E=y.vFlip;let k=y.rotate;w?E?k+=2:(S.push("translate("+(a.width+a.left).toString()+" "+(0-a.top).toString()+")"),S.push("scale(-1 1)"),a.top=a.left=0):E&&(S.push("translate("+(0-a.left).toString()+" "+(a.height+a.top).toString()+")"),S.push("scale(1 -1)"),a.top=a.left=0);let _;switch(k<0&&(k-=Math.floor(k/4)*4),k=k%4,k){case 1:_=a.height/2+a.top,S.unshift("rotate(90 "+_.toString()+" "+_.toString()+")");break;case 2:S.unshift("rotate(180 "+(a.width/2+a.left).toString()+" "+(a.height/2+a.top).toString()+")");break;case 3:_=a.width/2+a.left,S.unshift("rotate(-90 "+_.toString()+" "+_.toString()+")");break}k%2===1&&(a.left!==a.top&&(_=a.left,a.left=a.top,a.top=_),a.width!==a.height&&(_=a.width,a.width=a.height,a.height=_)),S.length&&(i=BKe(i,'<g transform="'+S.join(" ")+'">',"</g>"))});const o=n.width,s=n.height,l=a.width,u=a.height;let d,p;o===null?(p=s===null?"1em":s==="auto"?u:s,d=OH(p,l/u)):(d=o==="auto"?l:o,p=s===null?OH(d,u/l):s==="auto"?u:s);const f={},m=(y,S)=>{PKe(S)||(f[y]=S.toString())};m("width",d),m("height",p);const b=[a.left,a.top,l,u];return f.viewBox=b.join(" "),{attributes:f,viewBox:b,body:i}}const HKe=/\sid="(\S+)"/g,UKe="IconifyId"+Date.now().toString(16)+(Math.random()*16777216|0).toString(16);let $Ke=0;function GKe(e,t=UKe){const r=[];let n;for(;n=HKe.exec(e);)r.push(n[1]);if(!r.length)return e;const a="suffix"+(Math.random()*16777216|Date.now()).toString(16);return r.forEach(i=>{const o=typeof t=="function"?t(i):t+($Ke++).toString(),s=i.replace(/[.*+?^${}()|[\]\\]/g,"\\$&");e=e.replace(new RegExp('([#;"])('+s+')([")]|\\.[a-z])',"g"),"$1"+o+a+"$3")}),e=e.replace(new RegExp(a,"g"),""),e}function jKe(e,t){let r=e.indexOf("xlink:")===-1?"":' xmlns:xlink="http://www.w3.org/1999/xlink"';for(const n in t)r+=" "+n+'="'+t[n]+'"';return'<svg xmlns="http://www.w3.org/2000/svg"'+r+">"+e+"</svg>"}function WN(){return{async:!1,breaks:!1,extensions:null,gfm:!0,hooks:null,pedantic:!1,renderer:null,silent:!1,tokenizer:null,walkTokens:null}}var hd=WN();function Fse(e){hd=e}var Nm={exec:()=>null};function $r(e,t=""){let r=typeof e=="string"?e:e.source,n={replace:(a,i)=>{let o=typeof i=="string"?i:i.source;return o=o.replace(Ka.caret,"$1"),r=r.replace(a,o),n},getRegex:()=>new RegExp(r,t)};return n}var Ka={codeRemoveIndent:/^(?: {1,4}| {0,3}\t)/gm,outputLinkReplace:/\\([\[\]])/g,indentCodeCompensation:/^(\s+)(?:```)/,beginningSpace:/^\s+/,endingHash:/#$/,startingSpaceChar:/^ /,endingSpaceChar:/ $/,nonSpaceChar:/[^ ]/,newLineCharGlobal:/\n/g,tabCharGlobal:/\t/g,multipleSpaceGlobal:/\s+/g,blankLine:/^[ \t]*$/,doubleBlankLine:/\n[ \t]*\n[ \t]*$/,blockquoteStart:/^ {0,3}>/,blockquoteSetextReplace:/\n {0,3}((?:=+|-+) *)(?=\n|$)/g,blockquoteSetextReplace2:/^ {0,3}>[ \t]?/gm,listReplaceTabs:/^\t+/,listReplaceNesting:/^ {1,4}(?=( {4})*[^ ])/g,listIsTask:/^\[[ xX]\] /,listReplaceTask:/^\[[ xX]\] +/,anyLine:/\n.*\n/,hrefBrackets:/^<(.*)>$/,tableDelimiter:/[:|]/,tableAlignChars:/^\||\| *$/g,tableRowBlankLine:/\n[ \t]*$/,tableAlignRight:/^ *-+: *$/,tableAlignCenter:/^ *:-+: *$/,tableAlignLeft:/^ *:-+ *$/,startATag:/^<a /i,endATag:/^<\/a>/i,startPreScriptTag:/^<(pre|code|kbd|script)(\s|>)/i,endPreScriptTag:/^<\/(pre|code|kbd|script)(\s|>)/i,startAngleBracket:/^</,endAngleBracket:/>$/,pedanticHrefTitle:/^([^'"]*[^\s])\s+(['"])(.*)\2/,unicodeAlphaNumeric:/[\p{L}\p{N}]/u,escapeTest:/[&<>"']/,escapeReplace:/[&<>"']/g,escapeTestNoEncode:/[<>"']|&(?!(#\d{1,7}|#[Xx][a-fA-F0-9]{1,6}|\w+);)/,escapeReplaceNoEncode:/[<>"']|&(?!(#\d{1,7}|#[Xx][a-fA-F0-9]{1,6}|\w+);)/g,unescapeTest:/&(#(?:\d+)|(?:#x[0-9A-Fa-f]+)|(?:\w+));?/ig,caret:/(^|[^\[])\^/g,percentDecode:/%25/g,findPipe:/\|/g,splitPipe:/ \|/,slashPipe:/\\\|/g,carriageReturn:/\r\n|\r/g,spaceLine:/^ +$/gm,notSpaceStart:/^\S*/,endingNewline:/\n$/,listItemRegex:e=>new RegExp(`^( {0,3}${e})((?:[	 ][^\\n]*)?(?:\\n|$))`),nextBulletRegex:e=>new RegExp(`^ {0,${Math.min(3,e-1)}}(?:[*+-]|\\d{1,9}[.)])((?:[ 	][^\\n]*)?(?:\\n|$))`),hrRegex:e=>new RegExp(`^ {0,${Math.min(3,e-1)}}((?:- *){3,}|(?:_ *){3,}|(?:\\* *){3,})(?:\\n+|$)`),fencesBeginRegex:e=>new RegExp(`^ {0,${Math.min(3,e-1)}}(?:\`\`\`|~~~)`),headingBeginRegex:e=>new RegExp(`^ {0,${Math.min(3,e-1)}}#`),htmlBeginRegex:e=>new RegExp(`^ {0,${Math.min(3,e-1)}}<(?:[a-z].*>|!--)`,"i")},qKe=/^(?:[ \t]*(?:\n|$))+/,WKe=/^((?: {4}| {0,3}\t)[^\n]+(?:\n(?:[ \t]*(?:\n|$))*)?)+/,VKe=/^ {0,3}(`{3,}(?=[^`\n]*(?:\n|$))|~{3,})([^\n]*)(?:\n|$)(?:|([\s\S]*?)(?:\n|$))(?: {0,3}\1[~`]* *(?=\n|$)|$)/,Vg=/^ {0,3}((?:-[\t ]*){3,}|(?:_[ \t]*){3,}|(?:\*[ \t]*){3,})(?:\n+|$)/,KKe=/^ {0,3}(#{1,6})(?=\s|$)(.*)(?:\n+|$)/,VN=/(?:[*+-]|\d{1,9}[.)])/,_se=/^(?!bull |blockCode|fences|blockquote|heading|html|table)((?:.|\n(?!\s*?\n|bull |blockCode|fences|blockquote|heading|html|table))+?)\n {0,3}(=+|-+) *(?:\n+|$)/,Rse=$r(_se).replace(/bull/g,VN).replace(/blockCode/g,/(?: {4}| {0,3}\t)/).replace(/fences/g,/ {0,3}(?:`{3,}|~{3,})/).replace(/blockquote/g,/ {0,3}>/).replace(/heading/g,/ {0,3}#{1,6}/).replace(/html/g,/ {0,3}<[^\n>]+>\n/).replace(/\|table/g,"").getRegex(),YKe=$r(_se).replace(/bull/g,VN).replace(/blockCode/g,/(?: {4}| {0,3}\t)/).replace(/fences/g,/ {0,3}(?:`{3,}|~{3,})/).replace(/blockquote/g,/ {0,3}>/).replace(/heading/g,/ {0,3}#{1,6}/).replace(/html/g,/ {0,3}<[^\n>]+>\n/).replace(/table/g,/ {0,3}\|?(?:[:\- ]*\|)+[\:\- ]*\n/).getRegex(),KN=/^([^\n]+(?:\n(?!hr|heading|lheading|blockquote|fences|list|html|table| +\n)[^\n]+)*)/,JKe=/^[^\n]+/,YN=/(?!\s*\])(?:\\[\s\S]|[^\[\]\\])+/,XKe=$r(/^ {0,3}\[(label)\]: *(?:\n[ \t]*)?([^<\s][^\s]*|<.*?>)(?:(?: +(?:\n[ \t]*)?| *\n[ \t]*)(title))? *(?:\n+|$)/).replace("label",YN).replace("title",/(?:"(?:\\"?|[^"\\])*"|'[^'\n]*(?:\n[^'\n]+)*\n?'|\([^()]*\))/).getRegex(),ZKe=$r(/^( {0,3}bull)([ \t][^\n]+?)?(?:\n|$)/).replace(/bull/g,VN).getRegex(),W2="address|article|aside|base|basefont|blockquote|body|caption|center|col|colgroup|dd|details|dialog|dir|div|dl|dt|fieldset|figcaption|figure|footer|form|frame|frameset|h[1-6]|head|header|hr|html|iframe|legend|li|link|main|menu|menuitem|meta|nav|noframes|ol|optgroup|option|p|param|search|section|summary|table|tbody|td|tfoot|th|thead|title|tr|track|ul",JN=/<!--(?:-?>|[\s\S]*?(?:-->|$))/,QKe=$r("^ {0,3}(?:<(script|pre|style|textarea)[\\s>][\\s\\S]*?(?:</\\1>[^\\n]*\\n+|$)|comment[^\\n]*(\\n+|$)|<\\?[\\s\\S]*?(?:\\?>\\n*|$)|<![A-Z][\\s\\S]*?(?:>\\n*|$)|<!\\[CDATA\\[[\\s\\S]*?(?:\\]\\]>\\n*|$)|</?(tag)(?: +|\\n|/?>)[\\s\\S]*?(?:(?:\\n[ 	]*)+\\n|$)|<(?!script|pre|style|textarea)([a-z][\\w-]*)(?:attribute)*? */?>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:(?:\\n[ 	]*)+\\n|$)|</(?!script|pre|style|textarea)[a-z][\\w-]*\\s*>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:(?:\\n[ 	]*)+\\n|$))","i").replace("comment",JN).replace("tag",W2).replace("attribute",/ +[a-zA-Z:_][\w.:-]*(?: *= *"[^"\n]*"| *= *'[^'\n]*'| *= *[^\s"'=<>`]+)?/).getRegex(),Dse=$r(KN).replace("hr",Vg).replace("heading"," {0,3}#{1,6}(?:\\s|$)").replace("|lheading","").replace("|table","").replace("blockquote"," {0,3}>").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag",W2).getRegex(),eYe=$r(/^( {0,3}> ?(paragraph|[^\n]*)(?:\n|$))+/).replace("paragraph",Dse).getRegex(),XN={blockquote:eYe,code:WKe,def:XKe,fences:VKe,heading:KKe,hr:Vg,html:QKe,lheading:Rse,list:ZKe,newline:qKe,paragraph:Dse,table:Nm,text:JKe},BH=$r("^ *([^\\n ].*)\\n {0,3}((?:\\| *)?:?-+:? *(?:\\| *:?-+:? *)*(?:\\| *)?)(?:\\n((?:(?! *\\n|hr|heading|blockquote|code|fences|list|html).*(?:\\n|$))*)\\n*|$)").replace("hr",Vg).replace("heading"," {0,3}#{1,6}(?:\\s|$)").replace("blockquote"," {0,3}>").replace("code","(?: {4}| {0,3}	)[^\\n]").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag",W2).getRegex(),tYe={...XN,lheading:YKe,table:BH,paragraph:$r(KN).replace("hr",Vg).replace("heading"," {0,3}#{1,6}(?:\\s|$)").replace("|lheading","").replace("table",BH).replace("blockquote"," {0,3}>").replace("fences"," {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list"," {0,3}(?:[*+-]|1[.)]) ").replace("html","</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag",W2).getRegex()},rYe={...XN,html:$r(`^ *(?:comment *(?:\\n|\\s*$)|<(tag)[\\s\\S]+?</\\1> *(?:\\n{2,}|\\s*$)|<tag(?:"[^"]*"|'[^']*'|\\s[^'"/>\\s]*)*?/?> *(?:\\n{2,}|\\s*$))`).replace("comment",JN).replace(/tag/g,"(?!(?:a|em|strong|small|s|cite|q|dfn|abbr|data|time|code|var|samp|kbd|sub|sup|i|b|u|mark|ruby|rt|rp|bdi|bdo|span|br|wbr|ins|del|img)\\b)\\w+(?!:|[^\\w\\s@]*@)\\b").getRegex(),def:/^ *\[([^\]]+)\]: *<?([^\s>]+)>?(?: +(["(][^\n]+[")]))? *(?:\n+|$)/,heading:/^(#{1,6})(.*)(?:\n+|$)/,fences:Nm,lheading:/^(.+?)\n {0,3}(=+|-+) *(?:\n+|$)/,paragraph:$r(KN).replace("hr",Vg).replace("heading",` *#{1,6} *[^
]`).replace("lheading",Rse).replace("|table","").replace("blockquote"," {0,3}>").replace("|fences","").replace("|list","").replace("|html","").replace("|tag","").getRegex()},nYe=/^\\([!"#$%&'()*+,\-./:;<=>?@\[\]\\^_`{|}~])/,aYe=/^(`+)([^`]|[^`][\s\S]*?[^`])\1(?!`)/,Nse=/^( {2,}|\\)\n(?!\s*$)/,iYe=/^(`+|[^`])(?:(?= {2,}\n)|[\s\S]*?(?:(?=[\\<!\[`*_]|\b_|$)|[^ ](?= {2,}\n)))/,V2=/[\p{P}\p{S}]/u,ZN=/[\s\p{P}\p{S}]/u,Lse=/[^\s\p{P}\p{S}]/u,oYe=$r(/^((?![*_])punctSpace)/,"u").replace(/punctSpace/g,ZN).getRegex(),Mse=/(?!~)[\p{P}\p{S}]/u,sYe=/(?!~)[\s\p{P}\p{S}]/u,lYe=/(?:[^\s\p{P}\p{S}]|~)/u,uYe=/\[(?:[^\[\]`]|`[^`]*?`)*?\]\((?:\\[\s\S]|[^\\\(\)]|\((?:\\[\s\S]|[^\\\(\)])*\))*\)|`[^`]*?`|<(?! )[^<>]*?>/g,Ise=/^(?:\*+(?:((?!\*)punct)|[^\s*]))|^_+(?:((?!_)punct)|([^\s_]))/,cYe=$r(Ise,"u").replace(/punct/g,V2).getRegex(),dYe=$r(Ise,"u").replace(/punct/g,Mse).getRegex(),Ose="^[^_*]*?__[^_*]*?\\*[^_*]*?(?=__)|[^*]+(?=[^*])|(?!\\*)punct(\\*+)(?=[\\s]|$)|notPunctSpace(\\*+)(?!\\*)(?=punctSpace|$)|(?!\\*)punctSpace(\\*+)(?=notPunctSpace)|[\\s](\\*+)(?!\\*)(?=punct)|(?!\\*)punct(\\*+)(?!\\*)(?=punct)|notPunctSpace(\\*+)(?=notPunctSpace)",hYe=$r(Ose,"gu").replace(/notPunctSpace/g,Lse).replace(/punctSpace/g,ZN).replace(/punct/g,V2).getRegex(),pYe=$r(Ose,"gu").replace(/notPunctSpace/g,lYe).replace(/punctSpace/g,sYe).replace(/punct/g,Mse).getRegex(),fYe=$r("^[^_*]*?\\*\\*[^_*]*?_[^_*]*?(?=\\*\\*)|[^_]+(?=[^_])|(?!_)punct(_+)(?=[\\s]|$)|notPunctSpace(_+)(?!_)(?=punctSpace|$)|(?!_)punctSpace(_+)(?=notPunctSpace)|[\\s](_+)(?!_)(?=punct)|(?!_)punct(_+)(?!_)(?=punct)","gu").replace(/notPunctSpace/g,Lse).replace(/punctSpace/g,ZN).replace(/punct/g,V2).getRegex(),mYe=$r(/\\(punct)/,"gu").replace(/punct/g,V2).getRegex(),gYe=$r(/^<(scheme:[^\s\x00-\x1f<>]*|email)>/).replace("scheme",/[a-zA-Z][a-zA-Z0-9+.-]{1,31}/).replace("email",/[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+(@)[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+(?![-_])/).getRegex(),bYe=$r(JN).replace("(?:-->|$)","-->").getRegex(),yYe=$r("^comment|^</[a-zA-Z][\\w:-]*\\s*>|^<[a-zA-Z][\\w-]*(?:attribute)*?\\s*/?>|^<\\?[\\s\\S]*?\\?>|^<![a-zA-Z]+\\s[\\s\\S]*?>|^<!\\[CDATA\\[[\\s\\S]*?\\]\\]>").replace("comment",bYe).replace("attribute",/\s+[a-zA-Z:_][\w.:-]*(?:\s*=\s*"[^"]*"|\s*=\s*'[^']*'|\s*=\s*[^\s"'=<>`]+)?/).getRegex(),sv=/(?:\[(?:\\[\s\S]|[^\[\]\\])*\]|\\[\s\S]|`+[^`]*?`+(?!`)|[^\[\]\\`])*?/,vYe=$r(/^!?\[(label)\]\(\s*(href)(?:(?:[ \t]*(?:\n[ \t]*)?)(title))?\s*\)/).replace("label",sv).replace("href",/<(?:\\.|[^\n<>\\])+>|[^ \t\n\x00-\x1f]*/).replace("title",/"(?:\\"?|[^"\\])*"|'(?:\\'?|[^'\\])*'|\((?:\\\)?|[^)\\])*\)/).getRegex(),Bse=$r(/^!?\[(label)\]\[(ref)\]/).replace("label",sv).replace("ref",YN).getRegex(),Pse=$r(/^!?\[(ref)\](?:\[\])?/).replace("ref",YN).getRegex(),SYe=$r("reflink|nolink(?!\\()","g").replace("reflink",Bse).replace("nolink",Pse).getRegex(),PH=/[hH][tT][tT][pP][sS]?|[fF][tT][pP]/,QN={_backpedal:Nm,anyPunctuation:mYe,autolink:gYe,blockSkip:uYe,br:Nse,code:aYe,del:Nm,emStrongLDelim:cYe,emStrongRDelimAst:hYe,emStrongRDelimUnd:fYe,escape:nYe,link:vYe,nolink:Pse,punctuation:oYe,reflink:Bse,reflinkSearch:SYe,tag:yYe,text:iYe,url:Nm},EYe={...QN,link:$r(/^!?\[(label)\]\((.*?)\)/).replace("label",sv).getRegex(),reflink:$r(/^!?\[(label)\]\s*\[([^\]]*)\]/).replace("label",sv).getRegex()},N6={...QN,emStrongRDelimAst:pYe,emStrongLDelim:dYe,url:$r(/^((?:protocol):\/\/|www\.)(?:[a-zA-Z0-9\-]+\.?)+[^\s<]*|^email/).replace("protocol",PH).replace("email",/[A-Za-z0-9._+-]+(@)[a-zA-Z0-9-_]+(?:\.[a-zA-Z0-9-_]*[a-zA-Z0-9])+(?![-_])/).getRegex(),_backpedal:/(?:[^?!.,:;*_'"~()&]+|\([^)]*\)|&(?![a-zA-Z0-9]+;$)|[?!.,:;*_'"~)]+(?!$))+/,del:/^(~~?)(?=[^\s~])((?:\\[\s\S]|[^\\])*?(?:\\[\s\S]|[^\s~\\]))\1(?=[^~]|$)/,text:$r(/^([`~]+|[^`~])(?:(?= {2,}\n)|(?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@)|[\s\S]*?(?:(?=[\\<!\[`*~_]|\b_|protocol:\/\/|www\.|$)|[^ ](?= {2,}\n)|[^a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-](?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@)))/).replace("protocol",PH).getRegex()},xYe={...N6,br:$r(Nse).replace("{2,}","*").getRegex(),text:$r(N6.text).replace("\\b_","\\b_| {2,}\\n").replace(/\{2,\}/g,"*").getRegex()},o1={normal:XN,gfm:tYe,pedantic:rYe},Xf={normal:QN,gfm:N6,breaks:xYe,pedantic:EYe},wYe={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"},zH=e=>wYe[e];function Jo(e,t){if(t){if(Ka.escapeTest.test(e))return e.replace(Ka.escapeReplace,zH)}else if(Ka.escapeTestNoEncode.test(e))return e.replace(Ka.escapeReplaceNoEncode,zH);return e}function HH(e){try{e=encodeURI(e).replace(Ka.percentDecode,"%")}catch{return null}return e}function UH(e,t){var i;let r=e.replace(Ka.findPipe,(o,s,l)=>{let u=!1,d=s;for(;--d>=0&&l[d]==="\\";)u=!u;return u?"|":" |"}),n=r.split(Ka.splitPipe),a=0;if(n[0].trim()||n.shift(),n.length>0&&!((i=n.at(-1))!=null&&i.trim())&&n.pop(),t)if(n.length>t)n.splice(t);else for(;n.length<t;)n.push("");for(;a<n.length;a++)n[a]=n[a].trim().replace(Ka.slashPipe,"|");return n}function Zf(e,t,r){let n=e.length;if(n===0)return"";let a=0;for(;a<n&&e.charAt(n-a-1)===t;)a++;return e.slice(0,n-a)}function kYe(e,t){if(e.indexOf(t[1])===-1)return-1;let r=0;for(let n=0;n<e.length;n++)if(e[n]==="\\")n++;else if(e[n]===t[0])r++;else if(e[n]===t[1]&&(r--,r<0))return n;return r>0?-2:-1}function $H(e,t,r,n,a){let i=t.href,o=t.title||null,s=e[1].replace(a.other.outputLinkReplace,"$1");n.state.inLink=!0;let l={type:e[0].charAt(0)==="!"?"image":"link",raw:r,href:i,title:o,text:s,tokens:n.inlineTokens(s)};return n.state.inLink=!1,l}function CYe(e,t,r){let n=e.match(r.other.indentCodeCompensation);if(n===null)return t;let a=n[1];return t.split(`
`).map(i=>{let o=i.match(r.other.beginningSpace);if(o===null)return i;let[s]=o;return s.length>=a.length?i.slice(a.length):i}).join(`
`)}var lv=class{constructor(t){mt(this,"options");mt(this,"rules");mt(this,"lexer");this.options=t||hd}space(t){let r=this.rules.block.newline.exec(t);if(r&&r[0].length>0)return{type:"space",raw:r[0]}}code(t){let r=this.rules.block.code.exec(t);if(r){let n=r[0].replace(this.rules.other.codeRemoveIndent,"");return{type:"code",raw:r[0],codeBlockStyle:"indented",text:this.options.pedantic?n:Zf(n,`
`)}}}fences(t){let r=this.rules.block.fences.exec(t);if(r){let n=r[0],a=CYe(n,r[3]||"",this.rules);return{type:"code",raw:n,lang:r[2]?r[2].trim().replace(this.rules.inline.anyPunctuation,"$1"):r[2],text:a}}}heading(t){let r=this.rules.block.heading.exec(t);if(r){let n=r[2].trim();if(this.rules.other.endingHash.test(n)){let a=Zf(n,"#");(this.options.pedantic||!a||this.rules.other.endingSpaceChar.test(a))&&(n=a.trim())}return{type:"heading",raw:r[0],depth:r[1].length,text:n,tokens:this.lexer.inline(n)}}}hr(t){let r=this.rules.block.hr.exec(t);if(r)return{type:"hr",raw:Zf(r[0],`
`)}}blockquote(t){let r=this.rules.block.blockquote.exec(t);if(r){let n=Zf(r[0],`
`).split(`
`),a="",i="",o=[];for(;n.length>0;){let s=!1,l=[],u;for(u=0;u<n.length;u++)if(this.rules.other.blockquoteStart.test(n[u]))l.push(n[u]),s=!0;else if(!s)l.push(n[u]);else break;n=n.slice(u);let d=l.join(`
`),p=d.replace(this.rules.other.blockquoteSetextReplace,`
    $1`).replace(this.rules.other.blockquoteSetextReplace2,"");a=a?`${a}
${d}`:d,i=i?`${i}
${p}`:p;let f=this.lexer.state.top;if(this.lexer.state.top=!0,this.lexer.blockTokens(p,o,!0),this.lexer.state.top=f,n.length===0)break;let m=o.at(-1);if((m==null?void 0:m.type)==="code")break;if((m==null?void 0:m.type)==="blockquote"){let b=m,y=b.raw+`
`+n.join(`
`),S=this.blockquote(y);o[o.length-1]=S,a=a.substring(0,a.length-b.raw.length)+S.raw,i=i.substring(0,i.length-b.text.length)+S.text;break}else if((m==null?void 0:m.type)==="list"){let b=m,y=b.raw+`
`+n.join(`
`),S=this.list(y);o[o.length-1]=S,a=a.substring(0,a.length-m.raw.length)+S.raw,i=i.substring(0,i.length-b.raw.length)+S.raw,n=y.substring(o.at(-1).raw.length).split(`
`);continue}}return{type:"blockquote",raw:a,tokens:o,text:i}}}list(t){let r=this.rules.block.list.exec(t);if(r){let n=r[1].trim(),a=n.length>1,i={type:"list",raw:"",ordered:a,start:a?+n.slice(0,-1):"",loose:!1,items:[]};n=a?`\\d{1,9}\\${n.slice(-1)}`:`\\${n}`,this.options.pedantic&&(n=a?n:"[*+-]");let o=this.rules.other.listItemRegex(n),s=!1;for(;t;){let u=!1,d="",p="";if(!(r=o.exec(t))||this.rules.block.hr.test(t))break;d=r[0],t=t.substring(d.length);let f=r[2].split(`
`,1)[0].replace(this.rules.other.listReplaceTabs,E=>" ".repeat(3*E.length)),m=t.split(`
`,1)[0],b=!f.trim(),y=0;if(this.options.pedantic?(y=2,p=f.trimStart()):b?y=r[1].length+1:(y=r[2].search(this.rules.other.nonSpaceChar),y=y>4?1:y,p=f.slice(y),y+=r[1].length),b&&this.rules.other.blankLine.test(m)&&(d+=m+`
`,t=t.substring(m.length+1),u=!0),!u){let E=this.rules.other.nextBulletRegex(y),k=this.rules.other.hrRegex(y),_=this.rules.other.fencesBeginRegex(y),R=this.rules.other.headingBeginRegex(y),C=this.rules.other.htmlBeginRegex(y);for(;t;){let F=t.split(`
`,1)[0],L;if(m=F,this.options.pedantic?(m=m.replace(this.rules.other.listReplaceNesting,"  "),L=m):L=m.replace(this.rules.other.tabCharGlobal,"    "),_.test(m)||R.test(m)||C.test(m)||E.test(m)||k.test(m))break;if(L.search(this.rules.other.nonSpaceChar)>=y||!m.trim())p+=`
`+L.slice(y);else{if(b||f.replace(this.rules.other.tabCharGlobal,"    ").search(this.rules.other.nonSpaceChar)>=4||_.test(f)||R.test(f)||k.test(f))break;p+=`
`+m}!b&&!m.trim()&&(b=!0),d+=F+`
`,t=t.substring(F.length+1),f=L.slice(y)}}i.loose||(s?i.loose=!0:this.rules.other.doubleBlankLine.test(d)&&(s=!0));let S=null,w;this.options.gfm&&(S=this.rules.other.listIsTask.exec(p),S&&(w=S[0]!=="[ ] ",p=p.replace(this.rules.other.listReplaceTask,""))),i.items.push({type:"list_item",raw:d,task:!!S,checked:w,loose:!1,text:p,tokens:[]}),i.raw+=d}let l=i.items.at(-1);if(l)l.raw=l.raw.trimEnd(),l.text=l.text.trimEnd();else return;i.raw=i.raw.trimEnd();for(let u=0;u<i.items.length;u++)if(this.lexer.state.top=!1,i.items[u].tokens=this.lexer.blockTokens(i.items[u].text,[]),!i.loose){let d=i.items[u].tokens.filter(f=>f.type==="space"),p=d.length>0&&d.some(f=>this.rules.other.anyLine.test(f.raw));i.loose=p}if(i.loose)for(let u=0;u<i.items.length;u++)i.items[u].loose=!0;return i}}html(t){let r=this.rules.block.html.exec(t);if(r)return{type:"html",block:!0,raw:r[0],pre:r[1]==="pre"||r[1]==="script"||r[1]==="style",text:r[0]}}def(t){let r=this.rules.block.def.exec(t);if(r){let n=r[1].toLowerCase().replace(this.rules.other.multipleSpaceGlobal," "),a=r[2]?r[2].replace(this.rules.other.hrefBrackets,"$1").replace(this.rules.inline.anyPunctuation,"$1"):"",i=r[3]?r[3].substring(1,r[3].length-1).replace(this.rules.inline.anyPunctuation,"$1"):r[3];return{type:"def",tag:n,raw:r[0],href:a,title:i}}}table(t){var s;let r=this.rules.block.table.exec(t);if(!r||!this.rules.other.tableDelimiter.test(r[2]))return;let n=UH(r[1]),a=r[2].replace(this.rules.other.tableAlignChars,"").split("|"),i=(s=r[3])!=null&&s.trim()?r[3].replace(this.rules.other.tableRowBlankLine,"").split(`
`):[],o={type:"table",raw:r[0],header:[],align:[],rows:[]};if(n.length===a.length){for(let l of a)this.rules.other.tableAlignRight.test(l)?o.align.push("right"):this.rules.other.tableAlignCenter.test(l)?o.align.push("center"):this.rules.other.tableAlignLeft.test(l)?o.align.push("left"):o.align.push(null);for(let l=0;l<n.length;l++)o.header.push({text:n[l],tokens:this.lexer.inline(n[l]),header:!0,align:o.align[l]});for(let l of i)o.rows.push(UH(l,o.header.length).map((u,d)=>({text:u,tokens:this.lexer.inline(u),header:!1,align:o.align[d]})));return o}}lheading(t){let r=this.rules.block.lheading.exec(t);if(r)return{type:"heading",raw:r[0],depth:r[2].charAt(0)==="="?1:2,text:r[1],tokens:this.lexer.inline(r[1])}}paragraph(t){let r=this.rules.block.paragraph.exec(t);if(r){let n=r[1].charAt(r[1].length-1)===`
`?r[1].slice(0,-1):r[1];return{type:"paragraph",raw:r[0],text:n,tokens:this.lexer.inline(n)}}}text(t){let r=this.rules.block.text.exec(t);if(r)return{type:"text",raw:r[0],text:r[0],tokens:this.lexer.inline(r[0])}}escape(t){let r=this.rules.inline.escape.exec(t);if(r)return{type:"escape",raw:r[0],text:r[1]}}tag(t){let r=this.rules.inline.tag.exec(t);if(r)return!this.lexer.state.inLink&&this.rules.other.startATag.test(r[0])?this.lexer.state.inLink=!0:this.lexer.state.inLink&&this.rules.other.endATag.test(r[0])&&(this.lexer.state.inLink=!1),!this.lexer.state.inRawBlock&&this.rules.other.startPreScriptTag.test(r[0])?this.lexer.state.inRawBlock=!0:this.lexer.state.inRawBlock&&this.rules.other.endPreScriptTag.test(r[0])&&(this.lexer.state.inRawBlock=!1),{type:"html",raw:r[0],inLink:this.lexer.state.inLink,inRawBlock:this.lexer.state.inRawBlock,block:!1,text:r[0]}}link(t){let r=this.rules.inline.link.exec(t);if(r){let n=r[2].trim();if(!this.options.pedantic&&this.rules.other.startAngleBracket.test(n)){if(!this.rules.other.endAngleBracket.test(n))return;let o=Zf(n.slice(0,-1),"\\");if((n.length-o.length)%2===0)return}else{let o=kYe(r[2],"()");if(o===-2)return;if(o>-1){let s=(r[0].indexOf("!")===0?5:4)+r[1].length+o;r[2]=r[2].substring(0,o),r[0]=r[0].substring(0,s).trim(),r[3]=""}}let a=r[2],i="";if(this.options.pedantic){let o=this.rules.other.pedanticHrefTitle.exec(a);o&&(a=o[1],i=o[3])}else i=r[3]?r[3].slice(1,-1):"";return a=a.trim(),this.rules.other.startAngleBracket.test(a)&&(this.options.pedantic&&!this.rules.other.endAngleBracket.test(n)?a=a.slice(1):a=a.slice(1,-1)),$H(r,{href:a&&a.replace(this.rules.inline.anyPunctuation,"$1"),title:i&&i.replace(this.rules.inline.anyPunctuation,"$1")},r[0],this.lexer,this.rules)}}reflink(t,r){let n;if((n=this.rules.inline.reflink.exec(t))||(n=this.rules.inline.nolink.exec(t))){let a=(n[2]||n[1]).replace(this.rules.other.multipleSpaceGlobal," "),i=r[a.toLowerCase()];if(!i){let o=n[0].charAt(0);return{type:"text",raw:o,text:o}}return $H(n,i,n[0],this.lexer,this.rules)}}emStrong(t,r,n=""){let a=this.rules.inline.emStrongLDelim.exec(t);if(!(!a||a[3]&&n.match(this.rules.other.unicodeAlphaNumeric))&&(!(a[1]||a[2])||!n||this.rules.inline.punctuation.exec(n))){let i=[...a[0]].length-1,o,s,l=i,u=0,d=a[0][0]==="*"?this.rules.inline.emStrongRDelimAst:this.rules.inline.emStrongRDelimUnd;for(d.lastIndex=0,r=r.slice(-1*t.length+i);(a=d.exec(r))!=null;){if(o=a[1]||a[2]||a[3]||a[4]||a[5]||a[6],!o)continue;if(s=[...o].length,a[3]||a[4]){l+=s;continue}else if((a[5]||a[6])&&i%3&&!((i+s)%3)){u+=s;continue}if(l-=s,l>0)continue;s=Math.min(s,s+l+u);let p=[...a[0]][0].length,f=t.slice(0,i+a.index+p+s);if(Math.min(i,s)%2){let b=f.slice(1,-1);return{type:"em",raw:f,text:b,tokens:this.lexer.inlineTokens(b)}}let m=f.slice(2,-2);return{type:"strong",raw:f,text:m,tokens:this.lexer.inlineTokens(m)}}}}codespan(t){let r=this.rules.inline.code.exec(t);if(r){let n=r[2].replace(this.rules.other.newLineCharGlobal," "),a=this.rules.other.nonSpaceChar.test(n),i=this.rules.other.startingSpaceChar.test(n)&&this.rules.other.endingSpaceChar.test(n);return a&&i&&(n=n.substring(1,n.length-1)),{type:"codespan",raw:r[0],text:n}}}br(t){let r=this.rules.inline.br.exec(t);if(r)return{type:"br",raw:r[0]}}del(t){let r=this.rules.inline.del.exec(t);if(r)return{type:"del",raw:r[0],text:r[2],tokens:this.lexer.inlineTokens(r[2])}}autolink(t){let r=this.rules.inline.autolink.exec(t);if(r){let n,a;return r[2]==="@"?(n=r[1],a="mailto:"+n):(n=r[1],a=n),{type:"link",raw:r[0],text:n,href:a,tokens:[{type:"text",raw:n,text:n}]}}}url(t){var n;let r;if(r=this.rules.inline.url.exec(t)){let a,i;if(r[2]==="@")a=r[0],i="mailto:"+a;else{let o;do o=r[0],r[0]=((n=this.rules.inline._backpedal.exec(r[0]))==null?void 0:n[0])??"";while(o!==r[0]);a=r[0],r[1]==="www."?i="http://"+r[0]:i=r[0]}return{type:"link",raw:r[0],text:a,href:i,tokens:[{type:"text",raw:a,text:a}]}}}inlineText(t){let r=this.rules.inline.text.exec(t);if(r){let n=this.lexer.state.inRawBlock;return{type:"text",raw:r[0],text:r[0],escaped:n}}}},Co=class L6{constructor(t){mt(this,"tokens");mt(this,"options");mt(this,"state");mt(this,"tokenizer");mt(this,"inlineQueue");this.tokens=[],this.tokens.links=Object.create(null),this.options=t||hd,this.options.tokenizer=this.options.tokenizer||new lv,this.tokenizer=this.options.tokenizer,this.tokenizer.options=this.options,this.tokenizer.lexer=this,this.inlineQueue=[],this.state={inLink:!1,inRawBlock:!1,top:!0};let r={other:Ka,block:o1.normal,inline:Xf.normal};this.options.pedantic?(r.block=o1.pedantic,r.inline=Xf.pedantic):this.options.gfm&&(r.block=o1.gfm,this.options.breaks?r.inline=Xf.breaks:r.inline=Xf.gfm),this.tokenizer.rules=r}static get rules(){return{block:o1,inline:Xf}}static lex(t,r){return new L6(r).lex(t)}static lexInline(t,r){return new L6(r).inlineTokens(t)}lex(t){t=t.replace(Ka.carriageReturn,`
`),this.blockTokens(t,this.tokens);for(let r=0;r<this.inlineQueue.length;r++){let n=this.inlineQueue[r];this.inlineTokens(n.src,n.tokens)}return this.inlineQueue=[],this.tokens}blockTokens(t,r=[],n=!1){var a,i,o;for(this.options.pedantic&&(t=t.replace(Ka.tabCharGlobal,"    ").replace(Ka.spaceLine,""));t;){let s;if((i=(a=this.options.extensions)==null?void 0:a.block)!=null&&i.some(u=>(s=u.call({lexer:this},t,r))?(t=t.substring(s.raw.length),r.push(s),!0):!1))continue;if(s=this.tokenizer.space(t)){t=t.substring(s.raw.length);let u=r.at(-1);s.raw.length===1&&u!==void 0?u.raw+=`
`:r.push(s);continue}if(s=this.tokenizer.code(t)){t=t.substring(s.raw.length);let u=r.at(-1);(u==null?void 0:u.type)==="paragraph"||(u==null?void 0:u.type)==="text"?(u.raw+=(u.raw.endsWith(`
`)?"":`
`)+s.raw,u.text+=`
`+s.text,this.inlineQueue.at(-1).src=u.text):r.push(s);continue}if(s=this.tokenizer.fences(t)){t=t.substring(s.raw.length),r.push(s);continue}if(s=this.tokenizer.heading(t)){t=t.substring(s.raw.length),r.push(s);continue}if(s=this.tokenizer.hr(t)){t=t.substring(s.raw.length),r.push(s);continue}if(s=this.tokenizer.blockquote(t)){t=t.substring(s.raw.length),r.push(s);continue}if(s=this.tokenizer.list(t)){t=t.substring(s.raw.length),r.push(s);continue}if(s=this.tokenizer.html(t)){t=t.substring(s.raw.length),r.push(s);continue}if(s=this.tokenizer.def(t)){t=t.substring(s.raw.length);let u=r.at(-1);(u==null?void 0:u.type)==="paragraph"||(u==null?void 0:u.type)==="text"?(u.raw+=(u.raw.endsWith(`
`)?"":`
`)+s.raw,u.text+=`
`+s.raw,this.inlineQueue.at(-1).src=u.text):this.tokens.links[s.tag]||(this.tokens.links[s.tag]={href:s.href,title:s.title},r.push(s));continue}if(s=this.tokenizer.table(t)){t=t.substring(s.raw.length),r.push(s);continue}if(s=this.tokenizer.lheading(t)){t=t.substring(s.raw.length),r.push(s);continue}let l=t;if((o=this.options.extensions)!=null&&o.startBlock){let u=1/0,d=t.slice(1),p;this.options.extensions.startBlock.forEach(f=>{p=f.call({lexer:this},d),typeof p=="number"&&p>=0&&(u=Math.min(u,p))}),u<1/0&&u>=0&&(l=t.substring(0,u+1))}if(this.state.top&&(s=this.tokenizer.paragraph(l))){let u=r.at(-1);n&&(u==null?void 0:u.type)==="paragraph"?(u.raw+=(u.raw.endsWith(`
`)?"":`
`)+s.raw,u.text+=`
`+s.text,this.inlineQueue.pop(),this.inlineQueue.at(-1).src=u.text):r.push(s),n=l.length!==t.length,t=t.substring(s.raw.length);continue}if(s=this.tokenizer.text(t)){t=t.substring(s.raw.length);let u=r.at(-1);(u==null?void 0:u.type)==="text"?(u.raw+=(u.raw.endsWith(`
`)?"":`
`)+s.raw,u.text+=`
`+s.text,this.inlineQueue.pop(),this.inlineQueue.at(-1).src=u.text):r.push(s);continue}if(t){let u="Infinite loop on byte: "+t.charCodeAt(0);if(this.options.silent){console.error(u);break}else throw new Error(u)}}return this.state.top=!0,r}inline(t,r=[]){return this.inlineQueue.push({src:t,tokens:r}),r}inlineTokens(t,r=[]){var s,l,u,d,p;let n=t,a=null;if(this.tokens.links){let f=Object.keys(this.tokens.links);if(f.length>0)for(;(a=this.tokenizer.rules.inline.reflinkSearch.exec(n))!=null;)f.includes(a[0].slice(a[0].lastIndexOf("[")+1,-1))&&(n=n.slice(0,a.index)+"["+"a".repeat(a[0].length-2)+"]"+n.slice(this.tokenizer.rules.inline.reflinkSearch.lastIndex))}for(;(a=this.tokenizer.rules.inline.anyPunctuation.exec(n))!=null;)n=n.slice(0,a.index)+"++"+n.slice(this.tokenizer.rules.inline.anyPunctuation.lastIndex);for(;(a=this.tokenizer.rules.inline.blockSkip.exec(n))!=null;)n=n.slice(0,a.index)+"["+"a".repeat(a[0].length-2)+"]"+n.slice(this.tokenizer.rules.inline.blockSkip.lastIndex);n=((l=(s=this.options.hooks)==null?void 0:s.emStrongMask)==null?void 0:l.call({lexer:this},n))??n;let i=!1,o="";for(;t;){i||(o=""),i=!1;let f;if((d=(u=this.options.extensions)==null?void 0:u.inline)!=null&&d.some(b=>(f=b.call({lexer:this},t,r))?(t=t.substring(f.raw.length),r.push(f),!0):!1))continue;if(f=this.tokenizer.escape(t)){t=t.substring(f.raw.length),r.push(f);continue}if(f=this.tokenizer.tag(t)){t=t.substring(f.raw.length),r.push(f);continue}if(f=this.tokenizer.link(t)){t=t.substring(f.raw.length),r.push(f);continue}if(f=this.tokenizer.reflink(t,this.tokens.links)){t=t.substring(f.raw.length);let b=r.at(-1);f.type==="text"&&(b==null?void 0:b.type)==="text"?(b.raw+=f.raw,b.text+=f.text):r.push(f);continue}if(f=this.tokenizer.emStrong(t,n,o)){t=t.substring(f.raw.length),r.push(f);continue}if(f=this.tokenizer.codespan(t)){t=t.substring(f.raw.length),r.push(f);continue}if(f=this.tokenizer.br(t)){t=t.substring(f.raw.length),r.push(f);continue}if(f=this.tokenizer.del(t)){t=t.substring(f.raw.length),r.push(f);continue}if(f=this.tokenizer.autolink(t)){t=t.substring(f.raw.length),r.push(f);continue}if(!this.state.inLink&&(f=this.tokenizer.url(t))){t=t.substring(f.raw.length),r.push(f);continue}let m=t;if((p=this.options.extensions)!=null&&p.startInline){let b=1/0,y=t.slice(1),S;this.options.extensions.startInline.forEach(w=>{S=w.call({lexer:this},y),typeof S=="number"&&S>=0&&(b=Math.min(b,S))}),b<1/0&&b>=0&&(m=t.substring(0,b+1))}if(f=this.tokenizer.inlineText(m)){t=t.substring(f.raw.length),f.raw.slice(-1)!=="_"&&(o=f.raw.slice(-1)),i=!0;let b=r.at(-1);(b==null?void 0:b.type)==="text"?(b.raw+=f.raw,b.text+=f.text):r.push(f);continue}if(t){let b="Infinite loop on byte: "+t.charCodeAt(0);if(this.options.silent){console.error(b);break}else throw new Error(b)}}return r}},uv=class{constructor(t){mt(this,"options");mt(this,"parser");this.options=t||hd}space(t){return""}code({text:t,lang:r,escaped:n}){var o;let a=(o=(r||"").match(Ka.notSpaceStart))==null?void 0:o[0],i=t.replace(Ka.endingNewline,"")+`
`;return a?'<pre><code class="language-'+Jo(a)+'">'+(n?i:Jo(i,!0))+`</code></pre>
`:"<pre><code>"+(n?i:Jo(i,!0))+`</code></pre>
`}blockquote({tokens:t}){return`<blockquote>
${this.parser.parse(t)}</blockquote>
`}html({text:t}){return t}def(t){return""}heading({tokens:t,depth:r}){return`<h${r}>${this.parser.parseInline(t)}</h${r}>
`}hr(t){return`<hr>
`}list(t){let r=t.ordered,n=t.start,a="";for(let s=0;s<t.items.length;s++){let l=t.items[s];a+=this.listitem(l)}let i=r?"ol":"ul",o=r&&n!==1?' start="'+n+'"':"";return"<"+i+o+`>
`+a+"</"+i+`>
`}listitem(t){var n;let r="";if(t.task){let a=this.checkbox({checked:!!t.checked});t.loose?((n=t.tokens[0])==null?void 0:n.type)==="paragraph"?(t.tokens[0].text=a+" "+t.tokens[0].text,t.tokens[0].tokens&&t.tokens[0].tokens.length>0&&t.tokens[0].tokens[0].type==="text"&&(t.tokens[0].tokens[0].text=a+" "+Jo(t.tokens[0].tokens[0].text),t.tokens[0].tokens[0].escaped=!0)):t.tokens.unshift({type:"text",raw:a+" ",text:a+" ",escaped:!0}):r+=a+" "}return r+=this.parser.parse(t.tokens,!!t.loose),`<li>${r}</li>
`}checkbox({checked:t}){return"<input "+(t?'checked="" ':"")+'disabled="" type="checkbox">'}paragraph({tokens:t}){return`<p>${this.parser.parseInline(t)}</p>
`}table(t){let r="",n="";for(let i=0;i<t.header.length;i++)n+=this.tablecell(t.header[i]);r+=this.tablerow({text:n});let a="";for(let i=0;i<t.rows.length;i++){let o=t.rows[i];n="";for(let s=0;s<o.length;s++)n+=this.tablecell(o[s]);a+=this.tablerow({text:n})}return a&&(a=`<tbody>${a}</tbody>`),`<table>
<thead>
`+r+`</thead>
`+a+`</table>
`}tablerow({text:t}){return`<tr>
${t}</tr>
`}tablecell(t){let r=this.parser.parseInline(t.tokens),n=t.header?"th":"td";return(t.align?`<${n} align="${t.align}">`:`<${n}>`)+r+`</${n}>
`}strong({tokens:t}){return`<strong>${this.parser.parseInline(t)}</strong>`}em({tokens:t}){return`<em>${this.parser.parseInline(t)}</em>`}codespan({text:t}){return`<code>${Jo(t,!0)}</code>`}br(t){return"<br>"}del({tokens:t}){return`<del>${this.parser.parseInline(t)}</del>`}link({href:t,title:r,tokens:n}){let a=this.parser.parseInline(n),i=HH(t);if(i===null)return a;t=i;let o='<a href="'+t+'"';return r&&(o+=' title="'+Jo(r)+'"'),o+=">"+a+"</a>",o}image({href:t,title:r,text:n,tokens:a}){a&&(n=this.parser.parseInline(a,this.parser.textRenderer));let i=HH(t);if(i===null)return Jo(n);t=i;let o=`<img src="${t}" alt="${n}"`;return r&&(o+=` title="${Jo(r)}"`),o+=">",o}text(t){return"tokens"in t&&t.tokens?this.parser.parseInline(t.tokens):"escaped"in t&&t.escaped?t.text:Jo(t.text)}},eL=class{strong({text:t}){return t}em({text:t}){return t}codespan({text:t}){return t}del({text:t}){return t}html({text:t}){return t}text({text:t}){return t}link({text:t}){return""+t}image({text:t}){return""+t}br(){return""}},To=class M6{constructor(t){mt(this,"options");mt(this,"renderer");mt(this,"textRenderer");this.options=t||hd,this.options.renderer=this.options.renderer||new uv,this.renderer=this.options.renderer,this.renderer.options=this.options,this.renderer.parser=this,this.textRenderer=new eL}static parse(t,r){return new M6(r).parse(t)}static parseInline(t,r){return new M6(r).parseInline(t)}parse(t,r=!0){var a,i;let n="";for(let o=0;o<t.length;o++){let s=t[o];if((i=(a=this.options.extensions)==null?void 0:a.renderers)!=null&&i[s.type]){let u=s,d=this.options.extensions.renderers[u.type].call({parser:this},u);if(d!==!1||!["space","hr","heading","code","table","blockquote","list","html","def","paragraph","text"].includes(u.type)){n+=d||"";continue}}let l=s;switch(l.type){case"space":{n+=this.renderer.space(l);continue}case"hr":{n+=this.renderer.hr(l);continue}case"heading":{n+=this.renderer.heading(l);continue}case"code":{n+=this.renderer.code(l);continue}case"table":{n+=this.renderer.table(l);continue}case"blockquote":{n+=this.renderer.blockquote(l);continue}case"list":{n+=this.renderer.list(l);continue}case"html":{n+=this.renderer.html(l);continue}case"def":{n+=this.renderer.def(l);continue}case"paragraph":{n+=this.renderer.paragraph(l);continue}case"text":{let u=l,d=this.renderer.text(u);for(;o+1<t.length&&t[o+1].type==="text";)u=t[++o],d+=`
`+this.renderer.text(u);r?n+=this.renderer.paragraph({type:"paragraph",raw:d,text:d,tokens:[{type:"text",raw:d,text:d,escaped:!0}]}):n+=d;continue}default:{let u='Token with "'+l.type+'" type was not found.';if(this.options.silent)return console.error(u),"";throw new Error(u)}}}return n}parseInline(t,r=this.renderer){var a,i;let n="";for(let o=0;o<t.length;o++){let s=t[o];if((i=(a=this.options.extensions)==null?void 0:a.renderers)!=null&&i[s.type]){let u=this.options.extensions.renderers[s.type].call({parser:this},s);if(u!==!1||!["escape","html","link","image","strong","em","codespan","br","del","text"].includes(s.type)){n+=u||"";continue}}let l=s;switch(l.type){case"escape":{n+=r.text(l);break}case"html":{n+=r.html(l);break}case"link":{n+=r.link(l);break}case"image":{n+=r.image(l);break}case"strong":{n+=r.strong(l);break}case"em":{n+=r.em(l);break}case"codespan":{n+=r.codespan(l);break}case"br":{n+=r.br(l);break}case"del":{n+=r.del(l);break}case"text":{n+=r.text(l);break}default:{let u='Token with "'+l.type+'" type was not found.';if(this.options.silent)return console.error(u),"";throw new Error(u)}}}return n}},g1,hm=(g1=class{constructor(t){mt(this,"options");mt(this,"block");this.options=t||hd}preprocess(t){return t}postprocess(t){return t}processAllTokens(t){return t}emStrongMask(t){return t}provideLexer(){return this.block?Co.lex:Co.lexInline}provideParser(){return this.block?To.parse:To.parseInline}},mt(g1,"passThroughHooks",new Set(["preprocess","postprocess","processAllTokens","emStrongMask"])),mt(g1,"passThroughHooksRespectAsync",new Set(["preprocess","postprocess","processAllTokens"])),g1),TYe=class{constructor(...t){mt(this,"defaults",WN());mt(this,"options",this.setOptions);mt(this,"parse",this.parseMarkdown(!0));mt(this,"parseInline",this.parseMarkdown(!1));mt(this,"Parser",To);mt(this,"Renderer",uv);mt(this,"TextRenderer",eL);mt(this,"Lexer",Co);mt(this,"Tokenizer",lv);mt(this,"Hooks",hm);this.use(...t)}walkTokens(t,r){var a,i;let n=[];for(let o of t)switch(n=n.concat(r.call(this,o)),o.type){case"table":{let s=o;for(let l of s.header)n=n.concat(this.walkTokens(l.tokens,r));for(let l of s.rows)for(let u of l)n=n.concat(this.walkTokens(u.tokens,r));break}case"list":{let s=o;n=n.concat(this.walkTokens(s.items,r));break}default:{let s=o;(i=(a=this.defaults.extensions)==null?void 0:a.childTokens)!=null&&i[s.type]?this.defaults.extensions.childTokens[s.type].forEach(l=>{let u=s[l].flat(1/0);n=n.concat(this.walkTokens(u,r))}):s.tokens&&(n=n.concat(this.walkTokens(s.tokens,r)))}}return n}use(...t){let r=this.defaults.extensions||{renderers:{},childTokens:{}};return t.forEach(n=>{let a={...n};if(a.async=this.defaults.async||a.async||!1,n.extensions&&(n.extensions.forEach(i=>{if(!i.name)throw new Error("extension name required");if("renderer"in i){let o=r.renderers[i.name];o?r.renderers[i.name]=function(...s){let l=i.renderer.apply(this,s);return l===!1&&(l=o.apply(this,s)),l}:r.renderers[i.name]=i.renderer}if("tokenizer"in i){if(!i.level||i.level!=="block"&&i.level!=="inline")throw new Error("extension level must be 'block' or 'inline'");let o=r[i.level];o?o.unshift(i.tokenizer):r[i.level]=[i.tokenizer],i.start&&(i.level==="block"?r.startBlock?r.startBlock.push(i.start):r.startBlock=[i.start]:i.level==="inline"&&(r.startInline?r.startInline.push(i.start):r.startInline=[i.start]))}"childTokens"in i&&i.childTokens&&(r.childTokens[i.name]=i.childTokens)}),a.extensions=r),n.renderer){let i=this.defaults.renderer||new uv(this.defaults);for(let o in n.renderer){if(!(o in i))throw new Error(`renderer '${o}' does not exist`);if(["options","parser"].includes(o))continue;let s=o,l=n.renderer[s],u=i[s];i[s]=(...d)=>{let p=l.apply(i,d);return p===!1&&(p=u.apply(i,d)),p||""}}a.renderer=i}if(n.tokenizer){let i=this.defaults.tokenizer||new lv(this.defaults);for(let o in n.tokenizer){if(!(o in i))throw new Error(`tokenizer '${o}' does not exist`);if(["options","rules","lexer"].includes(o))continue;let s=o,l=n.tokenizer[s],u=i[s];i[s]=(...d)=>{let p=l.apply(i,d);return p===!1&&(p=u.apply(i,d)),p}}a.tokenizer=i}if(n.hooks){let i=this.defaults.hooks||new hm;for(let o in n.hooks){if(!(o in i))throw new Error(`hook '${o}' does not exist`);if(["options","block"].includes(o))continue;let s=o,l=n.hooks[s],u=i[s];hm.passThroughHooks.has(o)?i[s]=d=>{if(this.defaults.async&&hm.passThroughHooksRespectAsync.has(o))return(async()=>{let f=await l.call(i,d);return u.call(i,f)})();let p=l.call(i,d);return u.call(i,p)}:i[s]=(...d)=>{if(this.defaults.async)return(async()=>{let f=await l.apply(i,d);return f===!1&&(f=await u.apply(i,d)),f})();let p=l.apply(i,d);return p===!1&&(p=u.apply(i,d)),p}}a.hooks=i}if(n.walkTokens){let i=this.defaults.walkTokens,o=n.walkTokens;a.walkTokens=function(s){let l=[];return l.push(o.call(this,s)),i&&(l=l.concat(i.call(this,s))),l}}this.defaults={...this.defaults,...a}}),this}setOptions(t){return this.defaults={...this.defaults,...t},this}lexer(t,r){return Co.lex(t,r??this.defaults)}parser(t,r){return To.parse(t,r??this.defaults)}parseMarkdown(t){return(r,n)=>{let a={...n},i={...this.defaults,...a},o=this.onError(!!i.silent,!!i.async);if(this.defaults.async===!0&&a.async===!1)return o(new Error("marked(): The async option was set to true by an extension. Remove async: false from the parse options object to return a Promise."));if(typeof r>"u"||r===null)return o(new Error("marked(): input parameter is undefined or null"));if(typeof r!="string")return o(new Error("marked(): input parameter is of type "+Object.prototype.toString.call(r)+", string expected"));if(i.hooks&&(i.hooks.options=i,i.hooks.block=t),i.async)return(async()=>{let s=i.hooks?await i.hooks.preprocess(r):r,l=await(i.hooks?await i.hooks.provideLexer():t?Co.lex:Co.lexInline)(s,i),u=i.hooks?await i.hooks.processAllTokens(l):l;i.walkTokens&&await Promise.all(this.walkTokens(u,i.walkTokens));let d=await(i.hooks?await i.hooks.provideParser():t?To.parse:To.parseInline)(u,i);return i.hooks?await i.hooks.postprocess(d):d})().catch(o);try{i.hooks&&(r=i.hooks.preprocess(r));let s=(i.hooks?i.hooks.provideLexer():t?Co.lex:Co.lexInline)(r,i);i.hooks&&(s=i.hooks.processAllTokens(s)),i.walkTokens&&this.walkTokens(s,i.walkTokens);let l=(i.hooks?i.hooks.provideParser():t?To.parse:To.parseInline)(s,i);return i.hooks&&(l=i.hooks.postprocess(l)),l}catch(s){return o(s)}}}onError(t,r){return n=>{if(n.message+=`
Please report this to https://github.com/markedjs/marked.`,t){let a="<p>An error occurred:</p><pre>"+Jo(n.message+"",!0)+"</pre>";return r?Promise.resolve(a):a}if(r)return Promise.reject(n);throw n}}},Zc=new TYe;function jr(e,t){return Zc.parse(e,t)}jr.options=jr.setOptions=function(e){return Zc.setOptions(e),jr.defaults=Zc.defaults,Fse(jr.defaults),jr};jr.getDefaults=WN;jr.defaults=hd;jr.use=function(...e){return Zc.use(...e),jr.defaults=Zc.defaults,Fse(jr.defaults),jr};jr.walkTokens=function(e,t){return Zc.walkTokens(e,t)};jr.parseInline=Zc.parseInline;jr.Parser=To;jr.parser=To.parse;jr.Renderer=uv;jr.TextRenderer=eL;jr.Lexer=Co;jr.lexer=Co.lex;jr.Tokenizer=lv;jr.Hooks=hm;jr.parse=jr;jr.options;jr.setOptions;jr.use;jr.walkTokens;jr.parseInline;To.parse;Co.lex;function zse(e){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];var n=Array.from(typeof e=="string"?[e]:e);n[n.length-1]=n[n.length-1].replace(/\r?\n([\t ]*)$/,"");var a=n.reduce(function(s,l){var u=l.match(/\n([\t ]+|(?!\s).)/g);return u?s.concat(u.map(function(d){var p,f;return(f=(p=d.match(/[\t ]/g))===null||p===void 0?void 0:p.length)!==null&&f!==void 0?f:0})):s},[]);if(a.length){var i=new RegExp(`
[	 ]{`+Math.min.apply(Math,a)+"}","g");n=n.map(function(s){return s.replace(i,`
`)})}n[0]=n[0].replace(/^\r?\n/,"");var o=n[0];return t.forEach(function(s,l){var u=o.match(/(?:^|\n)( *)$/),d=u?u[1]:"",p=s;typeof s=="string"&&s.includes(`
`)&&(p=String(s).split(`
`).map(function(f,m){return m===0?f:""+d+f}).join(`
`)),o+=p+n[l+1]}),o}var AYe={body:'<g><rect width="80" height="80" style="fill: #087ebf; stroke-width: 0px;"/><text transform="translate(21.16 64.67)" style="fill: #fff; font-family: ArialMT, Arial; font-size: 67.75px;"><tspan x="0" y="0">?</tspan></text></g>',height:80,width:80},I6=new Map,Hse=new Map,FYe=U(e=>{for(const t of e){if(!t.name)throw new Error('Invalid icon loader. Must have a "name" property with non-empty string value.');if(je.debug("Registering icon pack:",t.name),"loader"in t)Hse.set(t.name,t.loader);else if("icons"in t)I6.set(t.name,t.icons);else throw je.error("Invalid icon loader:",t),new Error('Invalid icon loader. Must have either "icons" or "loader" property.')}},"registerIconPacks"),Use=U(async(e,t)=>{const r=_Ke(e,!0,t!==void 0);if(!r)throw new Error(`Invalid icon name: ${e}`);const n=r.prefix||t;if(!n)throw new Error(`Icon name must contain a prefix: ${e}`);let a=I6.get(n);if(!a){const o=Hse.get(n);if(!o)throw new Error(`Icon set not found: ${r.prefix}`);try{a={...await o(),prefix:n},I6.set(n,a)}catch(s){throw je.error(s),new Error(`Failed to load icon set: ${r.prefix}`)}}const i=NKe(a,r.name);if(!i)throw new Error(`Icon not found: ${e}`);return i},"getRegisteredIconData"),_Ye=U(async e=>{try{return await Use(e),!0}catch{return!1}},"isIconAvailable"),Kg=U(async(e,t,r)=>{let n;try{n=await Use(e,t==null?void 0:t.fallbackPrefix)}catch(o){je.error(o),n=AYe}const a=zKe(n,t),i=jKe(GKe(a.body),{...a.attributes,...r});return uo(i,Ma())},"getIconSVG");function $se(e,{markdownAutoWrap:t}){const n=e.replace(/<br\/>/g,`
`).replace(/\n{2,}/g,`
`),a=zse(n);return t===!1?a.replace(/ /g,"&nbsp;"):a}U($se,"preprocessMarkdown");function Gse(e,t={}){const r=$se(e,t),n=jr.lexer(r),a=[[]];let i=0;function o(s,l="normal"){s.type==="text"?s.text.split(`
`).forEach((d,p)=>{p!==0&&(i++,a.push([])),d.split(" ").forEach(f=>{f=f.replace(/&#39;/g,"'"),f&&a[i].push({content:f,type:l})})}):s.type==="strong"||s.type==="em"?s.tokens.forEach(u=>{o(u,s.type)}):s.type==="html"&&a[i].push({content:s.text,type:"normal"})}return U(o,"processNode"),n.forEach(s=>{var l;s.type==="paragraph"?(l=s.tokens)==null||l.forEach(u=>{o(u)}):s.type==="html"?a[i].push({content:s.text,type:"normal"}):a[i].push({content:s.raw,type:"normal"})}),a}U(Gse,"markdownToLines");function jse(e,{markdownAutoWrap:t}={}){const r=jr.lexer(e);function n(a){var i,o,s;return a.type==="text"?t===!1?a.text.replace(/\n */g,"<br/>").replace(/ /g,"&nbsp;"):a.text.replace(/\n */g,"<br/>"):a.type==="strong"?`<strong>${(i=a.tokens)==null?void 0:i.map(n).join("")}</strong>`:a.type==="em"?`<em>${(o=a.tokens)==null?void 0:o.map(n).join("")}</em>`:a.type==="paragraph"?`<p>${(s=a.tokens)==null?void 0:s.map(n).join("")}</p>`:a.type==="space"?"":a.type==="html"?`${a.text}`:a.type==="escape"?a.text:(je.warn(`Unsupported markdown: ${a.type}`),a.raw)}return U(n,"output"),r.map(n).join("")}U(jse,"markdownToHTML");function qse(e){return Intl.Segmenter?[...new Intl.Segmenter().segment(e)].map(t=>t.segment):[...e]}U(qse,"splitTextToChars");function Wse(e,t){const r=qse(t.content);return tL(e,[],r,t.type)}U(Wse,"splitWordToFitWidth");function tL(e,t,r,n){if(r.length===0)return[{content:t.join(""),type:n},{content:"",type:n}];const[a,...i]=r,o=[...t,a];return e([{content:o.join(""),type:n}])?tL(e,o,i,n):(t.length===0&&a&&(t.push(a),r.shift()),[{content:t.join(""),type:n},{content:r.join(""),type:n}])}U(tL,"splitWordToFitWidthRecursion");function Vse(e,t){if(e.some(({content:r})=>r.includes(`
`)))throw new Error("splitLineToFitWidth does not support newlines in the line");return cv(e,t)}U(Vse,"splitLineToFitWidth");function cv(e,t,r=[],n=[]){if(e.length===0)return n.length>0&&r.push(n),r.length>0?r:[];let a="";e[0].content===" "&&(a=" ",e.shift());const i=e.shift()??{content:" ",type:"normal"},o=[...n];if(a!==""&&o.push({content:a,type:"normal"}),o.push(i),t(o))return cv(e,t,r,o);if(n.length>0)r.push(n),e.unshift(i);else if(i.content){const[s,l]=Wse(t,i);r.push([s]),l.content&&e.unshift(l)}return cv(e,t,r)}U(cv,"splitLineToFitWidthRecursion");function O6(e,t){t&&e.attr("style",t)}U(O6,"applyStyle");async function Kse(e,t,r,n,a=!1,i=Ma()){const o=e.append("foreignObject");o.attr("width",`${10*r}px`),o.attr("height",`${10*r}px`);const s=o.append("xhtml:div"),l=Qh(t.label)?await cN(t.label.replace(_p.lineBreakRegex,`
`),i):uo(t.label,i),u=t.isNode?"nodeLabel":"edgeLabel",d=s.append("span");d.html(l),O6(d,t.labelStyle),d.attr("class",`${u} ${n}`),O6(s,t.labelStyle),s.style("display","table-cell"),s.style("white-space","nowrap"),s.style("line-height","1.5"),s.style("max-width",r+"px"),s.style("text-align","center"),s.attr("xmlns","http://www.w3.org/1999/xhtml"),a&&s.attr("class","labelBkg");let p=s.node().getBoundingClientRect();return p.width===r&&(s.style("display","table"),s.style("white-space","break-spaces"),s.style("width",r+"px"),p=s.node().getBoundingClientRect()),o.node()}U(Kse,"addHtmlSpan");function K2(e,t,r){return e.append("tspan").attr("class","text-outer-tspan").attr("x",0).attr("y",t*r-.1+"em").attr("dy",r+"em")}U(K2,"createTspan");function Yse(e,t,r){const n=e.append("text"),a=K2(n,1,t);Y2(a,r);const i=a.node().getComputedTextLength();return n.remove(),i}U(Yse,"computeWidthOfText");function RYe(e,t,r){var o;const n=e.append("text"),a=K2(n,1,t);Y2(a,[{content:r,type:"normal"}]);const i=(o=a.node())==null?void 0:o.getBoundingClientRect();return i&&n.remove(),i}U(RYe,"computeDimensionOfText");function Jse(e,t,r,n=!1){const i=t.append("g"),o=i.insert("rect").attr("class","background").attr("style","stroke: none"),s=i.append("text").attr("y","-10.1");let l=0;for(const u of r){const d=U(f=>Yse(i,1.1,f)<=e,"checkWidth"),p=d(u)?[u]:Vse(u,d);for(const f of p){const m=K2(s,l,1.1);Y2(m,f),l++}}if(n){const u=s.node().getBBox(),d=2;return o.attr("x",u.x-d).attr("y",u.y-d).attr("width",u.width+2*d).attr("height",u.height+2*d),i.node()}else return s.node()}U(Jse,"createFormattedText");function Y2(e,t){e.text(""),t.forEach((r,n)=>{const a=e.append("tspan").attr("font-style",r.type==="em"?"italic":"normal").attr("class","text-inner-tspan").attr("font-weight",r.type==="strong"?"bold":"normal");n===0?a.text(r.content):a.text(" "+r.content)})}U(Y2,"updateTextContentAndStyles");async function Xse(e,t={}){const r=[];e.replace(/(fa[bklrs]?):fa-([\w-]+)/g,(a,i,o)=>(r.push((async()=>{const s=`${i}:${o}`;return await _Ye(s)?await Kg(s,void 0,{class:"label-icon"}):`<i class='${uo(a,t).replace(":"," ")}'></i>`})()),a));const n=await Promise.all(r);return e.replace(/(fa[bklrs]?):fa-([\w-]+)/g,()=>n.shift()??"")}U(Xse,"replaceIconSubstring");var Gu=U(async(e,t="",{style:r="",isTitle:n=!1,classes:a="",useHtmlLabels:i=!0,isNode:o=!0,width:s=200,addSvgBackground:l=!1}={},u)=>{if(je.debug("XYZ createText",t,r,n,a,i,o,"addSvgBackground: ",l),i){const d=jse(t,u),p=await Xse(dd(d),u),f=t.replace(/\\\\/g,"\\"),m={isNode:o,label:Qh(t)?f:p,labelStyle:r.replace("fill:","color:")};return await Kse(e,m,s,a,l,u)}else{const d=t.replace(/<br\s*\/?>/g,"<br/>"),p=Gse(d.replace("<br>","<br/>"),u),f=Jse(s,e,p,t?l:!1);if(o){/stroke:/.exec(r)&&(r=r.replace("stroke:","lineColor:"));const m=r.replace(/stroke:[^;]+;?/g,"").replace(/stroke-width:[^;]+;?/g,"").replace(/fill:[^;]+;?/g,"").replace(/color:/g,"fill:");Lr(f).attr("style",m)}else{const m=r.replace(/stroke:[^;]+;?/g,"").replace(/stroke-width:[^;]+;?/g,"").replace(/fill:[^;]+;?/g,"").replace(/background:/g,"fill:");Lr(f).select("rect").attr("style",m.replace(/background:/g,"fill:"));const b=r.replace(/stroke:[^;]+;?/g,"").replace(/stroke-width:[^;]+;?/g,"").replace(/fill:[^;]+;?/g,"").replace(/color:/g,"fill:");Lr(f).select("text").attr("style",b)}return f}},"createText");function Bk(e,t,r){if(e&&e.length){const[n,a]=t,i=Math.PI/180*r,o=Math.cos(i),s=Math.sin(i);for(const l of e){const[u,d]=l;l[0]=(u-n)*o-(d-a)*s+n,l[1]=(u-n)*s+(d-a)*o+a}}}function DYe(e,t){return e[0]===t[0]&&e[1]===t[1]}function NYe(e,t,r,n=1){const a=r,i=Math.max(t,.1),o=e[0]&&e[0][0]&&typeof e[0][0]=="number"?[e]:e,s=[0,0];if(a)for(const u of o)Bk(u,s,a);const l=(function(u,d,p){const f=[];for(const E of u){const k=[...E];DYe(k[0],k[k.length-1])||k.push([k[0][0],k[0][1]]),k.length>2&&f.push(k)}const m=[];d=Math.max(d,.1);const b=[];for(const E of f)for(let k=0;k<E.length-1;k++){const _=E[k],R=E[k+1];if(_[1]!==R[1]){const C=Math.min(_[1],R[1]);b.push({ymin:C,ymax:Math.max(_[1],R[1]),x:C===_[1]?_[0]:R[0],islope:(R[0]-_[0])/(R[1]-_[1])})}}if(b.sort(((E,k)=>E.ymin<k.ymin?-1:E.ymin>k.ymin?1:E.x<k.x?-1:E.x>k.x?1:E.ymax===k.ymax?0:(E.ymax-k.ymax)/Math.abs(E.ymax-k.ymax))),!b.length)return m;let y=[],S=b[0].ymin,w=0;for(;y.length||b.length;){if(b.length){let E=-1;for(let k=0;k<b.length&&!(b[k].ymin>S);k++)E=k;b.splice(0,E+1).forEach((k=>{y.push({s:S,edge:k})}))}if(y=y.filter((E=>!(E.edge.ymax<=S))),y.sort(((E,k)=>E.edge.x===k.edge.x?0:(E.edge.x-k.edge.x)/Math.abs(E.edge.x-k.edge.x))),(p!==1||w%d==0)&&y.length>1)for(let E=0;E<y.length;E+=2){const k=E+1;if(k>=y.length)break;const _=y[E].edge,R=y[k].edge;m.push([[Math.round(_.x),S],[Math.round(R.x),S]])}S+=p,y.forEach((E=>{E.edge.x=E.edge.x+p*E.edge.islope})),w++}return m})(o,i,n);if(a){for(const u of o)Bk(u,s,-a);(function(u,d,p){const f=[];u.forEach((m=>f.push(...m))),Bk(f,d,p)})(l,s,-a)}return l}function Yg(e,t){var r;const n=t.hachureAngle+90;let a=t.hachureGap;a<0&&(a=4*t.strokeWidth),a=Math.round(Math.max(a,.1));let i=1;return t.roughness>=1&&(((r=t.randomizer)===null||r===void 0?void 0:r.next())||Math.random())>.7&&(i=a),NYe(e,a,n,i||1)}class rL{constructor(t){this.helper=t}fillPolygons(t,r){return this._fillPolygons(t,r)}_fillPolygons(t,r){const n=Yg(t,r);return{type:"fillSketch",ops:this.renderLines(n,r)}}renderLines(t,r){const n=[];for(const a of t)n.push(...this.helper.doubleLineOps(a[0][0],a[0][1],a[1][0],a[1][1],r));return n}}function J2(e){const t=e[0],r=e[1];return Math.sqrt(Math.pow(t[0]-r[0],2)+Math.pow(t[1]-r[1],2))}class LYe extends rL{fillPolygons(t,r){let n=r.hachureGap;n<0&&(n=4*r.strokeWidth),n=Math.max(n,.1);const a=Yg(t,Object.assign({},r,{hachureGap:n})),i=Math.PI/180*r.hachureAngle,o=[],s=.5*n*Math.cos(i),l=.5*n*Math.sin(i);for(const[u,d]of a)J2([u,d])&&o.push([[u[0]-s,u[1]+l],[...d]],[[u[0]+s,u[1]-l],[...d]]);return{type:"fillSketch",ops:this.renderLines(o,r)}}}class MYe extends rL{fillPolygons(t,r){const n=this._fillPolygons(t,r),a=Object.assign({},r,{hachureAngle:r.hachureAngle+90}),i=this._fillPolygons(t,a);return n.ops=n.ops.concat(i.ops),n}}class IYe{constructor(t){this.helper=t}fillPolygons(t,r){const n=Yg(t,r=Object.assign({},r,{hachureAngle:0}));return this.dotsOnLines(n,r)}dotsOnLines(t,r){const n=[];let a=r.hachureGap;a<0&&(a=4*r.strokeWidth),a=Math.max(a,.1);let i=r.fillWeight;i<0&&(i=r.strokeWidth/2);const o=a/4;for(const s of t){const l=J2(s),u=l/a,d=Math.ceil(u)-1,p=l-d*a,f=(s[0][0]+s[1][0])/2-a/4,m=Math.min(s[0][1],s[1][1]);for(let b=0;b<d;b++){const y=m+p+b*a,S=f-o+2*Math.random()*o,w=y-o+2*Math.random()*o,E=this.helper.ellipse(S,w,i,i,r);n.push(...E.ops)}}return{type:"fillSketch",ops:n}}}class OYe{constructor(t){this.helper=t}fillPolygons(t,r){const n=Yg(t,r);return{type:"fillSketch",ops:this.dashedLine(n,r)}}dashedLine(t,r){const n=r.dashOffset<0?r.hachureGap<0?4*r.strokeWidth:r.hachureGap:r.dashOffset,a=r.dashGap<0?r.hachureGap<0?4*r.strokeWidth:r.hachureGap:r.dashGap,i=[];return t.forEach((o=>{const s=J2(o),l=Math.floor(s/(n+a)),u=(s+a-l*(n+a))/2;let d=o[0],p=o[1];d[0]>p[0]&&(d=o[1],p=o[0]);const f=Math.atan((p[1]-d[1])/(p[0]-d[0]));for(let m=0;m<l;m++){const b=m*(n+a),y=b+n,S=[d[0]+b*Math.cos(f)+u*Math.cos(f),d[1]+b*Math.sin(f)+u*Math.sin(f)],w=[d[0]+y*Math.cos(f)+u*Math.cos(f),d[1]+y*Math.sin(f)+u*Math.sin(f)];i.push(...this.helper.doubleLineOps(S[0],S[1],w[0],w[1],r))}})),i}}class BYe{constructor(t){this.helper=t}fillPolygons(t,r){const n=r.hachureGap<0?4*r.strokeWidth:r.hachureGap,a=r.zigzagOffset<0?n:r.zigzagOffset,i=Yg(t,r=Object.assign({},r,{hachureGap:n+a}));return{type:"fillSketch",ops:this.zigzagLines(i,a,r)}}zigzagLines(t,r,n){const a=[];return t.forEach((i=>{const o=J2(i),s=Math.round(o/(2*r));let l=i[0],u=i[1];l[0]>u[0]&&(l=i[1],u=i[0]);const d=Math.atan((u[1]-l[1])/(u[0]-l[0]));for(let p=0;p<s;p++){const f=2*p*r,m=2*(p+1)*r,b=Math.sqrt(2*Math.pow(r,2)),y=[l[0]+f*Math.cos(d),l[1]+f*Math.sin(d)],S=[l[0]+m*Math.cos(d),l[1]+m*Math.sin(d)],w=[y[0]+b*Math.cos(d+Math.PI/4),y[1]+b*Math.sin(d+Math.PI/4)];a.push(...this.helper.doubleLineOps(y[0],y[1],w[0],w[1],n),...this.helper.doubleLineOps(w[0],w[1],S[0],S[1],n))}})),a}}const ii={};class PYe{constructor(t){this.seed=t}next(){return this.seed?(2**31-1&(this.seed=Math.imul(48271,this.seed)))/2**31:Math.random()}}const zYe=0,Pk=1,GH=2,s1={A:7,a:7,C:6,c:6,H:1,h:1,L:2,l:2,M:2,m:2,Q:4,q:4,S:4,s:4,T:2,t:2,V:1,v:1,Z:0,z:0};function zk(e,t){return e.type===t}function nL(e){const t=[],r=(function(o){const s=new Array;for(;o!=="";)if(o.match(/^([ \t\r\n,]+)/))o=o.substr(RegExp.$1.length);else if(o.match(/^([aAcChHlLmMqQsStTvVzZ])/))s[s.length]={type:zYe,text:RegExp.$1},o=o.substr(RegExp.$1.length);else{if(!o.match(/^(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)/))return[];s[s.length]={type:Pk,text:`${parseFloat(RegExp.$1)}`},o=o.substr(RegExp.$1.length)}return s[s.length]={type:GH,text:""},s})(e);let n="BOD",a=0,i=r[a];for(;!zk(i,GH);){let o=0;const s=[];if(n==="BOD"){if(i.text!=="M"&&i.text!=="m")return nL("M0,0"+e);a++,o=s1[i.text],n=i.text}else zk(i,Pk)?o=s1[n]:(a++,o=s1[i.text],n=i.text);if(!(a+o<r.length))throw new Error("Path data ended short");for(let l=a;l<a+o;l++){const u=r[l];if(!zk(u,Pk))throw new Error("Param not a number: "+n+","+u.text);s[s.length]=+u.text}if(typeof s1[n]!="number")throw new Error("Bad segment: "+n);{const l={key:n,data:s};t.push(l),a+=o,i=r[a],n==="M"&&(n="L"),n==="m"&&(n="l")}}return t}function Zse(e){let t=0,r=0,n=0,a=0;const i=[];for(const{key:o,data:s}of e)switch(o){case"M":i.push({key:"M",data:[...s]}),[t,r]=s,[n,a]=s;break;case"m":t+=s[0],r+=s[1],i.push({key:"M",data:[t,r]}),n=t,a=r;break;case"L":i.push({key:"L",data:[...s]}),[t,r]=s;break;case"l":t+=s[0],r+=s[1],i.push({key:"L",data:[t,r]});break;case"C":i.push({key:"C",data:[...s]}),t=s[4],r=s[5];break;case"c":{const l=s.map(((u,d)=>d%2?u+r:u+t));i.push({key:"C",data:l}),t=l[4],r=l[5];break}case"Q":i.push({key:"Q",data:[...s]}),t=s[2],r=s[3];break;case"q":{const l=s.map(((u,d)=>d%2?u+r:u+t));i.push({key:"Q",data:l}),t=l[2],r=l[3];break}case"A":i.push({key:"A",data:[...s]}),t=s[5],r=s[6];break;case"a":t+=s[5],r+=s[6],i.push({key:"A",data:[s[0],s[1],s[2],s[3],s[4],t,r]});break;case"H":i.push({key:"H",data:[...s]}),t=s[0];break;case"h":t+=s[0],i.push({key:"H",data:[t]});break;case"V":i.push({key:"V",data:[...s]}),r=s[0];break;case"v":r+=s[0],i.push({key:"V",data:[r]});break;case"S":i.push({key:"S",data:[...s]}),t=s[2],r=s[3];break;case"s":{const l=s.map(((u,d)=>d%2?u+r:u+t));i.push({key:"S",data:l}),t=l[2],r=l[3];break}case"T":i.push({key:"T",data:[...s]}),t=s[0],r=s[1];break;case"t":t+=s[0],r+=s[1],i.push({key:"T",data:[t,r]});break;case"Z":case"z":i.push({key:"Z",data:[]}),t=n,r=a}return i}function Qse(e){const t=[];let r="",n=0,a=0,i=0,o=0,s=0,l=0;for(const{key:u,data:d}of e){switch(u){case"M":t.push({key:"M",data:[...d]}),[n,a]=d,[i,o]=d;break;case"C":t.push({key:"C",data:[...d]}),n=d[4],a=d[5],s=d[2],l=d[3];break;case"L":t.push({key:"L",data:[...d]}),[n,a]=d;break;case"H":n=d[0],t.push({key:"L",data:[n,a]});break;case"V":a=d[0],t.push({key:"L",data:[n,a]});break;case"S":{let p=0,f=0;r==="C"||r==="S"?(p=n+(n-s),f=a+(a-l)):(p=n,f=a),t.push({key:"C",data:[p,f,...d]}),s=d[0],l=d[1],n=d[2],a=d[3];break}case"T":{const[p,f]=d;let m=0,b=0;r==="Q"||r==="T"?(m=n+(n-s),b=a+(a-l)):(m=n,b=a);const y=n+2*(m-n)/3,S=a+2*(b-a)/3,w=p+2*(m-p)/3,E=f+2*(b-f)/3;t.push({key:"C",data:[y,S,w,E,p,f]}),s=m,l=b,n=p,a=f;break}case"Q":{const[p,f,m,b]=d,y=n+2*(p-n)/3,S=a+2*(f-a)/3,w=m+2*(p-m)/3,E=b+2*(f-b)/3;t.push({key:"C",data:[y,S,w,E,m,b]}),s=p,l=f,n=m,a=b;break}case"A":{const p=Math.abs(d[0]),f=Math.abs(d[1]),m=d[2],b=d[3],y=d[4],S=d[5],w=d[6];p===0||f===0?(t.push({key:"C",data:[n,a,S,w,S,w]}),n=S,a=w):(n!==S||a!==w)&&(ele(n,a,S,w,p,f,m,b,y).forEach((function(E){t.push({key:"C",data:E})})),n=S,a=w);break}case"Z":t.push({key:"Z",data:[]}),n=i,a=o}r=u}return t}function Qf(e,t,r){return[e*Math.cos(r)-t*Math.sin(r),e*Math.sin(r)+t*Math.cos(r)]}function ele(e,t,r,n,a,i,o,s,l,u){const d=(p=o,Math.PI*p/180);var p;let f=[],m=0,b=0,y=0,S=0;if(u)[m,b,y,S]=u;else{[e,t]=Qf(e,t,-d),[r,n]=Qf(r,n,-d);const W=(e-r)/2,$=(t-n)/2;let X=W*W/(a*a)+$*$/(i*i);X>1&&(X=Math.sqrt(X),a*=X,i*=X);const z=a*a,H=i*i,B=z*H-z*$*$-H*W*W,ee=z*$*$+H*W*W,J=(s===l?-1:1)*Math.sqrt(Math.abs(B/ee));y=J*a*$/i+(e+r)/2,S=J*-i*W/a+(t+n)/2,m=Math.asin(parseFloat(((t-S)/i).toFixed(9))),b=Math.asin(parseFloat(((n-S)/i).toFixed(9))),e<y&&(m=Math.PI-m),r<y&&(b=Math.PI-b),m<0&&(m=2*Math.PI+m),b<0&&(b=2*Math.PI+b),l&&m>b&&(m-=2*Math.PI),!l&&b>m&&(b-=2*Math.PI)}let w=b-m;if(Math.abs(w)>120*Math.PI/180){const W=b,$=r,X=n;b=l&&b>m?m+120*Math.PI/180*1:m+120*Math.PI/180*-1,f=ele(r=y+a*Math.cos(b),n=S+i*Math.sin(b),$,X,a,i,o,0,l,[b,W,y,S])}w=b-m;const E=Math.cos(m),k=Math.sin(m),_=Math.cos(b),R=Math.sin(b),C=Math.tan(w/4),F=4/3*a*C,L=4/3*i*C,I=[e,t],M=[e+F*k,t-L*E],j=[r+F*R,n-L*_],Y=[r,n];if(M[0]=2*I[0]-M[0],M[1]=2*I[1]-M[1],u)return[M,j,Y].concat(f);{f=[M,j,Y].concat(f);const W=[];for(let $=0;$<f.length;$+=3){const X=Qf(f[$][0],f[$][1],d),z=Qf(f[$+1][0],f[$+1][1],d),H=Qf(f[$+2][0],f[$+2][1],d);W.push([X[0],X[1],z[0],z[1],H[0],H[1]])}return W}}const HYe={randOffset:function(e,t){return tr(e,t)},randOffsetWithRange:function(e,t,r){return dv(e,t,r)},ellipse:function(e,t,r,n,a){const i=rle(r,n,a);return B6(e,t,a,i).opset},doubleLineOps:function(e,t,r,n,a){return Iu(e,t,r,n,a,!0)}};function tle(e,t,r,n,a){return{type:"path",ops:Iu(e,t,r,n,a)}}function K1(e,t,r){const n=(e||[]).length;if(n>2){const a=[];for(let i=0;i<n-1;i++)a.push(...Iu(e[i][0],e[i][1],e[i+1][0],e[i+1][1],r));return t&&a.push(...Iu(e[n-1][0],e[n-1][1],e[0][0],e[0][1],r)),{type:"path",ops:a}}return n===2?tle(e[0][0],e[0][1],e[1][0],e[1][1],r):{type:"path",ops:[]}}function UYe(e,t,r,n,a){return(function(i,o){return K1(i,!0,o)})([[e,t],[e+r,t],[e+r,t+n],[e,t+n]],a)}function jH(e,t){if(e.length){const r=typeof e[0][0]=="number"?[e]:e,n=l1(r[0],1*(1+.2*t.roughness),t),a=t.disableMultiStroke?[]:l1(r[0],1.5*(1+.22*t.roughness),VH(t));for(let i=1;i<r.length;i++){const o=r[i];if(o.length){const s=l1(o,1*(1+.2*t.roughness),t),l=t.disableMultiStroke?[]:l1(o,1.5*(1+.22*t.roughness),VH(t));for(const u of s)u.op!=="move"&&n.push(u);for(const u of l)u.op!=="move"&&a.push(u)}}return{type:"path",ops:n.concat(a)}}return{type:"path",ops:[]}}function rle(e,t,r){const n=Math.sqrt(2*Math.PI*Math.sqrt((Math.pow(e/2,2)+Math.pow(t/2,2))/2)),a=Math.ceil(Math.max(r.curveStepCount,r.curveStepCount/Math.sqrt(200)*n)),i=2*Math.PI/a;let o=Math.abs(e/2),s=Math.abs(t/2);const l=1-r.curveFitting;return o+=tr(o*l,r),s+=tr(s*l,r),{increment:i,rx:o,ry:s}}function B6(e,t,r,n){const[a,i]=KH(n.increment,e,t,n.rx,n.ry,1,n.increment*dv(.1,dv(.4,1,r),r),r);let o=hv(a,null,r);if(!r.disableMultiStroke&&r.roughness!==0){const[s]=KH(n.increment,e,t,n.rx,n.ry,1.5,0,r),l=hv(s,null,r);o=o.concat(l)}return{estimatedPoints:i,opset:{type:"path",ops:o}}}function qH(e,t,r,n,a,i,o,s,l){const u=e,d=t;let p=Math.abs(r/2),f=Math.abs(n/2);p+=tr(.01*p,l),f+=tr(.01*f,l);let m=a,b=i;for(;m<0;)m+=2*Math.PI,b+=2*Math.PI;b-m>2*Math.PI&&(m=0,b=2*Math.PI);const y=2*Math.PI/l.curveStepCount,S=Math.min(y/2,(b-m)/2),w=YH(S,u,d,p,f,m,b,1,l);if(!l.disableMultiStroke){const E=YH(S,u,d,p,f,m,b,1.5,l);w.push(...E)}return o&&(s?w.push(...Iu(u,d,u+p*Math.cos(m),d+f*Math.sin(m),l),...Iu(u,d,u+p*Math.cos(b),d+f*Math.sin(b),l)):w.push({op:"lineTo",data:[u,d]},{op:"lineTo",data:[u+p*Math.cos(m),d+f*Math.sin(m)]})),{type:"path",ops:w}}function WH(e,t){const r=Qse(Zse(nL(e))),n=[];let a=[0,0],i=[0,0];for(const{key:o,data:s}of r)switch(o){case"M":i=[s[0],s[1]],a=[s[0],s[1]];break;case"L":n.push(...Iu(i[0],i[1],s[0],s[1],t)),i=[s[0],s[1]];break;case"C":{const[l,u,d,p,f,m]=s;n.push(...$Ye(l,u,d,p,f,m,i,t)),i=[f,m];break}case"Z":n.push(...Iu(i[0],i[1],a[0],a[1],t)),i=[a[0],a[1]]}return{type:"path",ops:n}}function Hk(e,t){const r=[];for(const n of e)if(n.length){const a=t.maxRandomnessOffset||0,i=n.length;if(i>2){r.push({op:"move",data:[n[0][0]+tr(a,t),n[0][1]+tr(a,t)]});for(let o=1;o<i;o++)r.push({op:"lineTo",data:[n[o][0]+tr(a,t),n[o][1]+tr(a,t)]})}}return{type:"fillPath",ops:r}}function oh(e,t){return(function(r,n){let a=r.fillStyle||"hachure";if(!ii[a])switch(a){case"zigzag":ii[a]||(ii[a]=new LYe(n));break;case"cross-hatch":ii[a]||(ii[a]=new MYe(n));break;case"dots":ii[a]||(ii[a]=new IYe(n));break;case"dashed":ii[a]||(ii[a]=new OYe(n));break;case"zigzag-line":ii[a]||(ii[a]=new BYe(n));break;default:a="hachure",ii[a]||(ii[a]=new rL(n))}return ii[a]})(t,HYe).fillPolygons(e,t)}function VH(e){const t=Object.assign({},e);return t.randomizer=void 0,e.seed&&(t.seed=e.seed+1),t}function nle(e){return e.randomizer||(e.randomizer=new PYe(e.seed||0)),e.randomizer.next()}function dv(e,t,r,n=1){return r.roughness*n*(nle(r)*(t-e)+e)}function tr(e,t,r=1){return dv(-e,e,t,r)}function Iu(e,t,r,n,a,i=!1){const o=i?a.disableMultiStrokeFill:a.disableMultiStroke,s=P6(e,t,r,n,a,!0,!1);if(o)return s;const l=P6(e,t,r,n,a,!0,!0);return s.concat(l)}function P6(e,t,r,n,a,i,o){const s=Math.pow(e-r,2)+Math.pow(t-n,2),l=Math.sqrt(s);let u=1;u=l<200?1:l>500?.4:-.0016668*l+1.233334;let d=a.maxRandomnessOffset||0;d*d*100>s&&(d=l/10);const p=d/2,f=.2+.2*nle(a);let m=a.bowing*a.maxRandomnessOffset*(n-t)/200,b=a.bowing*a.maxRandomnessOffset*(e-r)/200;m=tr(m,a,u),b=tr(b,a,u);const y=[],S=()=>tr(p,a,u),w=()=>tr(d,a,u),E=a.preserveVertices;return o?y.push({op:"move",data:[e+(E?0:S()),t+(E?0:S())]}):y.push({op:"move",data:[e+(E?0:tr(d,a,u)),t+(E?0:tr(d,a,u))]}),o?y.push({op:"bcurveTo",data:[m+e+(r-e)*f+S(),b+t+(n-t)*f+S(),m+e+2*(r-e)*f+S(),b+t+2*(n-t)*f+S(),r+(E?0:S()),n+(E?0:S())]}):y.push({op:"bcurveTo",data:[m+e+(r-e)*f+w(),b+t+(n-t)*f+w(),m+e+2*(r-e)*f+w(),b+t+2*(n-t)*f+w(),r+(E?0:w()),n+(E?0:w())]}),y}function l1(e,t,r){if(!e.length)return[];const n=[];n.push([e[0][0]+tr(t,r),e[0][1]+tr(t,r)]),n.push([e[0][0]+tr(t,r),e[0][1]+tr(t,r)]);for(let a=1;a<e.length;a++)n.push([e[a][0]+tr(t,r),e[a][1]+tr(t,r)]),a===e.length-1&&n.push([e[a][0]+tr(t,r),e[a][1]+tr(t,r)]);return hv(n,null,r)}function hv(e,t,r){const n=e.length,a=[];if(n>3){const i=[],o=1-r.curveTightness;a.push({op:"move",data:[e[1][0],e[1][1]]});for(let s=1;s+2<n;s++){const l=e[s];i[0]=[l[0],l[1]],i[1]=[l[0]+(o*e[s+1][0]-o*e[s-1][0])/6,l[1]+(o*e[s+1][1]-o*e[s-1][1])/6],i[2]=[e[s+1][0]+(o*e[s][0]-o*e[s+2][0])/6,e[s+1][1]+(o*e[s][1]-o*e[s+2][1])/6],i[3]=[e[s+1][0],e[s+1][1]],a.push({op:"bcurveTo",data:[i[1][0],i[1][1],i[2][0],i[2][1],i[3][0],i[3][1]]})}}else n===3?(a.push({op:"move",data:[e[1][0],e[1][1]]}),a.push({op:"bcurveTo",data:[e[1][0],e[1][1],e[2][0],e[2][1],e[2][0],e[2][1]]})):n===2&&a.push(...P6(e[0][0],e[0][1],e[1][0],e[1][1],r,!0,!0));return a}function KH(e,t,r,n,a,i,o,s){const l=[],u=[];if(s.roughness===0){e/=4,u.push([t+n*Math.cos(-e),r+a*Math.sin(-e)]);for(let d=0;d<=2*Math.PI;d+=e){const p=[t+n*Math.cos(d),r+a*Math.sin(d)];l.push(p),u.push(p)}u.push([t+n*Math.cos(0),r+a*Math.sin(0)]),u.push([t+n*Math.cos(e),r+a*Math.sin(e)])}else{const d=tr(.5,s)-Math.PI/2;u.push([tr(i,s)+t+.9*n*Math.cos(d-e),tr(i,s)+r+.9*a*Math.sin(d-e)]);const p=2*Math.PI+d-.01;for(let f=d;f<p;f+=e){const m=[tr(i,s)+t+n*Math.cos(f),tr(i,s)+r+a*Math.sin(f)];l.push(m),u.push(m)}u.push([tr(i,s)+t+n*Math.cos(d+2*Math.PI+.5*o),tr(i,s)+r+a*Math.sin(d+2*Math.PI+.5*o)]),u.push([tr(i,s)+t+.98*n*Math.cos(d+o),tr(i,s)+r+.98*a*Math.sin(d+o)]),u.push([tr(i,s)+t+.9*n*Math.cos(d+.5*o),tr(i,s)+r+.9*a*Math.sin(d+.5*o)])}return[u,l]}function YH(e,t,r,n,a,i,o,s,l){const u=i+tr(.1,l),d=[];d.push([tr(s,l)+t+.9*n*Math.cos(u-e),tr(s,l)+r+.9*a*Math.sin(u-e)]);for(let p=u;p<=o;p+=e)d.push([tr(s,l)+t+n*Math.cos(p),tr(s,l)+r+a*Math.sin(p)]);return d.push([t+n*Math.cos(o),r+a*Math.sin(o)]),d.push([t+n*Math.cos(o),r+a*Math.sin(o)]),hv(d,null,l)}function $Ye(e,t,r,n,a,i,o,s){const l=[],u=[s.maxRandomnessOffset||1,(s.maxRandomnessOffset||1)+.3];let d=[0,0];const p=s.disableMultiStroke?1:2,f=s.preserveVertices;for(let m=0;m<p;m++)m===0?l.push({op:"move",data:[o[0],o[1]]}):l.push({op:"move",data:[o[0]+(f?0:tr(u[0],s)),o[1]+(f?0:tr(u[0],s))]}),d=f?[a,i]:[a+tr(u[m],s),i+tr(u[m],s)],l.push({op:"bcurveTo",data:[e+tr(u[m],s),t+tr(u[m],s),r+tr(u[m],s),n+tr(u[m],s),d[0],d[1]]});return l}function em(e){return[...e]}function JH(e,t=0){const r=e.length;if(r<3)throw new Error("A curve must have at least three points.");const n=[];if(r===3)n.push(em(e[0]),em(e[1]),em(e[2]),em(e[2]));else{const a=[];a.push(e[0],e[0]);for(let s=1;s<e.length;s++)a.push(e[s]),s===e.length-1&&a.push(e[s]);const i=[],o=1-t;n.push(em(a[0]));for(let s=1;s+2<a.length;s++){const l=a[s];i[0]=[l[0],l[1]],i[1]=[l[0]+(o*a[s+1][0]-o*a[s-1][0])/6,l[1]+(o*a[s+1][1]-o*a[s-1][1])/6],i[2]=[a[s+1][0]+(o*a[s][0]-o*a[s+2][0])/6,a[s+1][1]+(o*a[s][1]-o*a[s+2][1])/6],i[3]=[a[s+1][0],a[s+1][1]],n.push(i[1],i[2],i[3])}}return n}function Y1(e,t){return Math.pow(e[0]-t[0],2)+Math.pow(e[1]-t[1],2)}function GYe(e,t,r){const n=Y1(t,r);if(n===0)return Y1(e,t);let a=((e[0]-t[0])*(r[0]-t[0])+(e[1]-t[1])*(r[1]-t[1]))/n;return a=Math.max(0,Math.min(1,a)),Y1(e,yc(t,r,a))}function yc(e,t,r){return[e[0]+(t[0]-e[0])*r,e[1]+(t[1]-e[1])*r]}function z6(e,t,r,n){const a=n||[];if((function(s,l){const u=s[l+0],d=s[l+1],p=s[l+2],f=s[l+3];let m=3*d[0]-2*u[0]-f[0];m*=m;let b=3*d[1]-2*u[1]-f[1];b*=b;let y=3*p[0]-2*f[0]-u[0];y*=y;let S=3*p[1]-2*f[1]-u[1];return S*=S,m<y&&(m=y),b<S&&(b=S),m+b})(e,t)<r){const s=e[t+0];a.length?(i=a[a.length-1],o=s,Math.sqrt(Y1(i,o))>1&&a.push(s)):a.push(s),a.push(e[t+3])}else{const l=e[t+0],u=e[t+1],d=e[t+2],p=e[t+3],f=yc(l,u,.5),m=yc(u,d,.5),b=yc(d,p,.5),y=yc(f,m,.5),S=yc(m,b,.5),w=yc(y,S,.5);z6([l,f,y,w],0,r,a),z6([w,S,b,p],0,r,a)}var i,o;return a}function jYe(e,t){return pv(e,0,e.length,t)}function pv(e,t,r,n,a){const i=a||[],o=e[t],s=e[r-1];let l=0,u=1;for(let d=t+1;d<r-1;++d){const p=GYe(e[d],o,s);p>l&&(l=p,u=d)}return Math.sqrt(l)>n?(pv(e,t,u+1,n,i),pv(e,u,r,n,i)):(i.length||i.push(o),i.push(s)),i}function Uk(e,t=.15,r){const n=[],a=(e.length-1)/3;for(let i=0;i<a;i++)z6(e,3*i,t,n);return r&&r>0?pv(n,0,n.length,r):n}const Ri="none";class fv{constructor(t){this.defaultOptions={maxRandomnessOffset:2,roughness:1,bowing:1,stroke:"#000",strokeWidth:1,curveTightness:0,curveFitting:.95,curveStepCount:9,fillStyle:"hachure",fillWeight:-1,hachureAngle:-41,hachureGap:-1,dashOffset:-1,dashGap:-1,zigzagOffset:-1,seed:0,disableMultiStroke:!1,disableMultiStrokeFill:!1,preserveVertices:!1,fillShapeRoughnessGain:.8},this.config=t||{},this.config.options&&(this.defaultOptions=this._o(this.config.options))}static newSeed(){return Math.floor(Math.random()*2**31)}_o(t){return t?Object.assign({},this.defaultOptions,t):this.defaultOptions}_d(t,r,n){return{shape:t,sets:r||[],options:n||this.defaultOptions}}line(t,r,n,a,i){const o=this._o(i);return this._d("line",[tle(t,r,n,a,o)],o)}rectangle(t,r,n,a,i){const o=this._o(i),s=[],l=UYe(t,r,n,a,o);if(o.fill){const u=[[t,r],[t+n,r],[t+n,r+a],[t,r+a]];o.fillStyle==="solid"?s.push(Hk([u],o)):s.push(oh([u],o))}return o.stroke!==Ri&&s.push(l),this._d("rectangle",s,o)}ellipse(t,r,n,a,i){const o=this._o(i),s=[],l=rle(n,a,o),u=B6(t,r,o,l);if(o.fill)if(o.fillStyle==="solid"){const d=B6(t,r,o,l).opset;d.type="fillPath",s.push(d)}else s.push(oh([u.estimatedPoints],o));return o.stroke!==Ri&&s.push(u.opset),this._d("ellipse",s,o)}circle(t,r,n,a){const i=this.ellipse(t,r,n,n,a);return i.shape="circle",i}linearPath(t,r){const n=this._o(r);return this._d("linearPath",[K1(t,!1,n)],n)}arc(t,r,n,a,i,o,s=!1,l){const u=this._o(l),d=[],p=qH(t,r,n,a,i,o,s,!0,u);if(s&&u.fill)if(u.fillStyle==="solid"){const f=Object.assign({},u);f.disableMultiStroke=!0;const m=qH(t,r,n,a,i,o,!0,!1,f);m.type="fillPath",d.push(m)}else d.push((function(f,m,b,y,S,w,E){const k=f,_=m;let R=Math.abs(b/2),C=Math.abs(y/2);R+=tr(.01*R,E),C+=tr(.01*C,E);let F=S,L=w;for(;F<0;)F+=2*Math.PI,L+=2*Math.PI;L-F>2*Math.PI&&(F=0,L=2*Math.PI);const I=(L-F)/E.curveStepCount,M=[];for(let j=F;j<=L;j+=I)M.push([k+R*Math.cos(j),_+C*Math.sin(j)]);return M.push([k+R*Math.cos(L),_+C*Math.sin(L)]),M.push([k,_]),oh([M],E)})(t,r,n,a,i,o,u));return u.stroke!==Ri&&d.push(p),this._d("arc",d,u)}curve(t,r){const n=this._o(r),a=[],i=jH(t,n);if(n.fill&&n.fill!==Ri)if(n.fillStyle==="solid"){const o=jH(t,Object.assign(Object.assign({},n),{disableMultiStroke:!0,roughness:n.roughness?n.roughness+n.fillShapeRoughnessGain:0}));a.push({type:"fillPath",ops:this._mergedShape(o.ops)})}else{const o=[],s=t;if(s.length){const l=typeof s[0][0]=="number"?[s]:s;for(const u of l)u.length<3?o.push(...u):u.length===3?o.push(...Uk(JH([u[0],u[0],u[1],u[2]]),10,(1+n.roughness)/2)):o.push(...Uk(JH(u),10,(1+n.roughness)/2))}o.length&&a.push(oh([o],n))}return n.stroke!==Ri&&a.push(i),this._d("curve",a,n)}polygon(t,r){const n=this._o(r),a=[],i=K1(t,!0,n);return n.fill&&(n.fillStyle==="solid"?a.push(Hk([t],n)):a.push(oh([t],n))),n.stroke!==Ri&&a.push(i),this._d("polygon",a,n)}path(t,r){const n=this._o(r),a=[];if(!t)return this._d("path",a,n);t=(t||"").replace(/\n/g," ").replace(/(-\s)/g,"-").replace("/(ss)/g"," ");const i=n.fill&&n.fill!=="transparent"&&n.fill!==Ri,o=n.stroke!==Ri,s=!!(n.simplification&&n.simplification<1),l=(function(d,p,f){const m=Qse(Zse(nL(d))),b=[];let y=[],S=[0,0],w=[];const E=()=>{w.length>=4&&y.push(...Uk(w,p)),w=[]},k=()=>{E(),y.length&&(b.push(y),y=[])};for(const{key:R,data:C}of m)switch(R){case"M":k(),S=[C[0],C[1]],y.push(S);break;case"L":E(),y.push([C[0],C[1]]);break;case"C":if(!w.length){const F=y.length?y[y.length-1]:S;w.push([F[0],F[1]])}w.push([C[0],C[1]]),w.push([C[2],C[3]]),w.push([C[4],C[5]]);break;case"Z":E(),y.push([S[0],S[1]])}if(k(),!f)return b;const _=[];for(const R of b){const C=jYe(R,f);C.length&&_.push(C)}return _})(t,1,s?4-4*(n.simplification||1):(1+n.roughness)/2),u=WH(t,n);if(i)if(n.fillStyle==="solid")if(l.length===1){const d=WH(t,Object.assign(Object.assign({},n),{disableMultiStroke:!0,roughness:n.roughness?n.roughness+n.fillShapeRoughnessGain:0}));a.push({type:"fillPath",ops:this._mergedShape(d.ops)})}else a.push(Hk(l,n));else a.push(oh(l,n));return o&&(s?l.forEach((d=>{a.push(K1(d,!1,n))})):a.push(u)),this._d("path",a,n)}opsToPath(t,r){let n="";for(const a of t.ops){const i=typeof r=="number"&&r>=0?a.data.map((o=>+o.toFixed(r))):a.data;switch(a.op){case"move":n+=`M${i[0]} ${i[1]} `;break;case"bcurveTo":n+=`C${i[0]} ${i[1]}, ${i[2]} ${i[3]}, ${i[4]} ${i[5]} `;break;case"lineTo":n+=`L${i[0]} ${i[1]} `}}return n.trim()}toPaths(t){const r=t.sets||[],n=t.options||this.defaultOptions,a=[];for(const i of r){let o=null;switch(i.type){case"path":o={d:this.opsToPath(i),stroke:n.stroke,strokeWidth:n.strokeWidth,fill:Ri};break;case"fillPath":o={d:this.opsToPath(i),stroke:Ri,strokeWidth:0,fill:n.fill||Ri};break;case"fillSketch":o=this.fillSketch(i,n)}o&&a.push(o)}return a}fillSketch(t,r){let n=r.fillWeight;return n<0&&(n=r.strokeWidth/2),{d:this.opsToPath(t),stroke:r.fill||Ri,strokeWidth:n,fill:Ri}}_mergedShape(t){return t.filter(((r,n)=>n===0||r.op!=="move"))}}class qYe{constructor(t,r){this.canvas=t,this.ctx=this.canvas.getContext("2d"),this.gen=new fv(r)}draw(t){const r=t.sets||[],n=t.options||this.getDefaultOptions(),a=this.ctx,i=t.options.fixedDecimalPlaceDigits;for(const o of r)switch(o.type){case"path":a.save(),a.strokeStyle=n.stroke==="none"?"transparent":n.stroke,a.lineWidth=n.strokeWidth,n.strokeLineDash&&a.setLineDash(n.strokeLineDash),n.strokeLineDashOffset&&(a.lineDashOffset=n.strokeLineDashOffset),this._drawToContext(a,o,i),a.restore();break;case"fillPath":{a.save(),a.fillStyle=n.fill||"";const s=t.shape==="curve"||t.shape==="polygon"||t.shape==="path"?"evenodd":"nonzero";this._drawToContext(a,o,i,s),a.restore();break}case"fillSketch":this.fillSketch(a,o,n)}}fillSketch(t,r,n){let a=n.fillWeight;a<0&&(a=n.strokeWidth/2),t.save(),n.fillLineDash&&t.setLineDash(n.fillLineDash),n.fillLineDashOffset&&(t.lineDashOffset=n.fillLineDashOffset),t.strokeStyle=n.fill||"",t.lineWidth=a,this._drawToContext(t,r,n.fixedDecimalPlaceDigits),t.restore()}_drawToContext(t,r,n,a="nonzero"){t.beginPath();for(const i of r.ops){const o=typeof n=="number"&&n>=0?i.data.map((s=>+s.toFixed(n))):i.data;switch(i.op){case"move":t.moveTo(o[0],o[1]);break;case"bcurveTo":t.bezierCurveTo(o[0],o[1],o[2],o[3],o[4],o[5]);break;case"lineTo":t.lineTo(o[0],o[1])}}r.type==="fillPath"?t.fill(a):t.stroke()}get generator(){return this.gen}getDefaultOptions(){return this.gen.defaultOptions}line(t,r,n,a,i){const o=this.gen.line(t,r,n,a,i);return this.draw(o),o}rectangle(t,r,n,a,i){const o=this.gen.rectangle(t,r,n,a,i);return this.draw(o),o}ellipse(t,r,n,a,i){const o=this.gen.ellipse(t,r,n,a,i);return this.draw(o),o}circle(t,r,n,a){const i=this.gen.circle(t,r,n,a);return this.draw(i),i}linearPath(t,r){const n=this.gen.linearPath(t,r);return this.draw(n),n}polygon(t,r){const n=this.gen.polygon(t,r);return this.draw(n),n}arc(t,r,n,a,i,o,s=!1,l){const u=this.gen.arc(t,r,n,a,i,o,s,l);return this.draw(u),u}curve(t,r){const n=this.gen.curve(t,r);return this.draw(n),n}path(t,r){const n=this.gen.path(t,r);return this.draw(n),n}}const u1="http://www.w3.org/2000/svg";class WYe{constructor(t,r){this.svg=t,this.gen=new fv(r)}draw(t){const r=t.sets||[],n=t.options||this.getDefaultOptions(),a=this.svg.ownerDocument||window.document,i=a.createElementNS(u1,"g"),o=t.options.fixedDecimalPlaceDigits;for(const s of r){let l=null;switch(s.type){case"path":l=a.createElementNS(u1,"path"),l.setAttribute("d",this.opsToPath(s,o)),l.setAttribute("stroke",n.stroke),l.setAttribute("stroke-width",n.strokeWidth+""),l.setAttribute("fill","none"),n.strokeLineDash&&l.setAttribute("stroke-dasharray",n.strokeLineDash.join(" ").trim()),n.strokeLineDashOffset&&l.setAttribute("stroke-dashoffset",`${n.strokeLineDashOffset}`);break;case"fillPath":l=a.createElementNS(u1,"path"),l.setAttribute("d",this.opsToPath(s,o)),l.setAttribute("stroke","none"),l.setAttribute("stroke-width","0"),l.setAttribute("fill",n.fill||""),t.shape!=="curve"&&t.shape!=="polygon"||l.setAttribute("fill-rule","evenodd");break;case"fillSketch":l=this.fillSketch(a,s,n)}l&&i.appendChild(l)}return i}fillSketch(t,r,n){let a=n.fillWeight;a<0&&(a=n.strokeWidth/2);const i=t.createElementNS(u1,"path");return i.setAttribute("d",this.opsToPath(r,n.fixedDecimalPlaceDigits)),i.setAttribute("stroke",n.fill||""),i.setAttribute("stroke-width",a+""),i.setAttribute("fill","none"),n.fillLineDash&&i.setAttribute("stroke-dasharray",n.fillLineDash.join(" ").trim()),n.fillLineDashOffset&&i.setAttribute("stroke-dashoffset",`${n.fillLineDashOffset}`),i}get generator(){return this.gen}getDefaultOptions(){return this.gen.defaultOptions}opsToPath(t,r){return this.gen.opsToPath(t,r)}line(t,r,n,a,i){const o=this.gen.line(t,r,n,a,i);return this.draw(o)}rectangle(t,r,n,a,i){const o=this.gen.rectangle(t,r,n,a,i);return this.draw(o)}ellipse(t,r,n,a,i){const o=this.gen.ellipse(t,r,n,a,i);return this.draw(o)}circle(t,r,n,a){const i=this.gen.circle(t,r,n,a);return this.draw(i)}linearPath(t,r){const n=this.gen.linearPath(t,r);return this.draw(n)}polygon(t,r){const n=this.gen.polygon(t,r);return this.draw(n)}arc(t,r,n,a,i,o,s=!1,l){const u=this.gen.arc(t,r,n,a,i,o,s,l);return this.draw(u)}curve(t,r){const n=this.gen.curve(t,r);return this.draw(n)}path(t,r){const n=this.gen.path(t,r);return this.draw(n)}}var _t={canvas:(e,t)=>new qYe(e,t),svg:(e,t)=>new WYe(e,t),generator:e=>new fv(e),newSeed:()=>fv.newSeed()},Zt=U(async(e,t,r)=>{var p,f;let n;const a=t.useHtmlLabels||ra((p=Gr())==null?void 0:p.htmlLabels);r?n=r:n="node default";const i=e.insert("g").attr("class",n).attr("id",t.domId||t.id),o=i.insert("g").attr("class","label").attr("style",Oa(t.labelStyle));let s;t.label===void 0?s="":s=typeof t.label=="string"?t.label:t.label[0];const l=await Gu(o,uo(dd(s),Gr()),{useHtmlLabels:a,width:t.width||((f=Gr().flowchart)==null?void 0:f.wrappingWidth),cssClasses:"markdown-node-label",style:t.labelStyle,addSvgBackground:!!t.icon||!!t.img});let u=l.getBBox();const d=((t==null?void 0:t.padding)??0)/2;if(a){const m=l.children[0],b=Lr(l),y=m.getElementsByTagName("img");if(y){const S=s.replace(/<img[^>]*>/g,"").trim()==="";await Promise.all([...y].map(w=>new Promise(E=>{function k(){if(w.style.display="flex",w.style.flexDirection="column",S){const _=Gr().fontSize?Gr().fontSize:window.getComputedStyle(document.body).fontSize,R=5,[C=bae.fontSize]=q2(_),F=C*R+"px";w.style.minWidth=F,w.style.maxWidth=F}else w.style.width="100%";E(w)}U(k,"setupImage"),setTimeout(()=>{w.complete&&k()}),w.addEventListener("error",k),w.addEventListener("load",k)})))}u=m.getBoundingClientRect(),b.attr("width",u.width),b.attr("height",u.height)}return a?o.attr("transform","translate("+-u.width/2+", "+-u.height/2+")"):o.attr("transform","translate(0, "+-u.height/2+")"),t.centerLabel&&o.attr("transform","translate("+-u.width/2+", "+-u.height/2+")"),o.insert("rect",":first-child"),{shapeSvg:i,bbox:u,halfPadding:d,label:o}},"labelHelper"),$k=U(async(e,t,r)=>{var l,u,d,p,f,m;const n=r.useHtmlLabels||ra((u=(l=Gr())==null?void 0:l.flowchart)==null?void 0:u.htmlLabels),a=e.insert("g").attr("class","label").attr("style",r.labelStyle||""),i=await Gu(a,uo(dd(t),Gr()),{useHtmlLabels:n,width:r.width||((p=(d=Gr())==null?void 0:d.flowchart)==null?void 0:p.wrappingWidth),style:r.labelStyle,addSvgBackground:!!r.icon||!!r.img});let o=i.getBBox();const s=r.padding/2;if(ra((m=(f=Gr())==null?void 0:f.flowchart)==null?void 0:m.htmlLabels)){const b=i.children[0],y=Lr(i);o=b.getBoundingClientRect(),y.attr("width",o.width),y.attr("height",o.height)}return n?a.attr("transform","translate("+-o.width/2+", "+-o.height/2+")"):a.attr("transform","translate(0, "+-o.height/2+")"),r.centerLabel&&a.attr("transform","translate("+-o.width/2+", "+-o.height/2+")"),a.insert("rect",":first-child"),{shapeSvg:e,bbox:o,halfPadding:s,label:a}},"insertLabel"),Lt=U((e,t)=>{const r=t.node().getBBox();e.width=r.width,e.height=r.height},"updateNodeBounds"),jt=U((e,t)=>(e.look==="handDrawn"?"rough-node":"node")+" "+e.cssClasses+" "+(t||""),"getNodeClasses");function Ar(e){const t=e.map((r,n)=>`${n===0?"M":"L"}${r.x},${r.y}`);return t.push("Z"),t.join(" ")}U(Ar,"createPathFromPoints");function Ou(e,t,r,n,a,i){const o=[],l=r-e,u=n-t,d=l/i,p=2*Math.PI/d,f=t+u/2;for(let m=0;m<=50;m++){const b=m/50,y=e+b*l,S=f+a*Math.sin(p*(y-e));o.push({x:y,y:S})}return o}U(Ou,"generateFullSineWavePoints");function ag(e,t,r,n,a,i){const o=[],s=a*Math.PI/180,d=(i*Math.PI/180-s)/(n-1);for(let p=0;p<n;p++){const f=s+p*d,m=e+r*Math.cos(f),b=t+r*Math.sin(f);o.push({x:-m,y:-b})}return o}U(ag,"generateCirclePoints");var VYe=U((e,t)=>{var r=e.x,n=e.y,a=t.x-r,i=t.y-n,o=e.width/2,s=e.height/2,l,u;return Math.abs(i)*o>Math.abs(a)*s?(i<0&&(s=-s),l=i===0?0:s*a/i,u=s):(a<0&&(o=-o),l=o,u=a===0?0:o*i/a),{x:r+l,y:n+u}},"intersectRect"),Lp=VYe;function ale(e,t){t&&e.attr("style",t)}U(ale,"applyStyle");async function ile(e){const t=Lr(document.createElementNS("http://www.w3.org/2000/svg","foreignObject")),r=t.append("xhtml:div"),n=Gr();let a=e.label;e.label&&Qh(e.label)&&(a=await cN(e.label.replace(_p.lineBreakRegex,`
`),n));const o='<span class="'+(e.isNode?"nodeLabel":"edgeLabel")+'" '+(e.labelStyle?'style="'+e.labelStyle+'"':"")+">"+a+"</span>";return r.html(uo(o,n)),ale(r,e.labelStyle),r.style("display","inline-block"),r.style("padding-right","1px"),r.style("white-space","nowrap"),r.attr("xmlns","http://www.w3.org/1999/xhtml"),t.node()}U(ile,"addHtmlLabel");var KYe=U(async(e,t,r,n)=>{let a=e||"";if(typeof a=="object"&&(a=a[0]),ra(Gr().flowchart.htmlLabels)){a=a.replace(/\\n|\n/g,"<br />"),je.info("vertexText"+a);const i={isNode:n,label:dd(a).replace(/fa[blrs]?:fa-[\w-]+/g,s=>`<i class='${s.replace(":"," ")}'></i>`),labelStyle:t&&t.replace("fill:","color:")};return await ile(i)}else{const i=document.createElementNS("http://www.w3.org/2000/svg","text");i.setAttribute("style",t.replace("color:","fill:"));let o=[];typeof a=="string"?o=a.split(/\\n|\n|<br\s*\/?>/gi):Array.isArray(a)?o=a:o=[];for(const s of o){const l=document.createElementNS("http://www.w3.org/2000/svg","tspan");l.setAttributeNS("http://www.w3.org/XML/1998/namespace","xml:space","preserve"),l.setAttribute("dy","1em"),l.setAttribute("x","0"),r?l.setAttribute("class","title-row"):l.setAttribute("class","row"),l.textContent=s.trim(),i.appendChild(l)}return i}},"createLabel"),Ac=KYe,ju=U((e,t,r,n,a)=>["M",e+a,t,"H",e+r-a,"A",a,a,0,0,1,e+r,t+a,"V",t+n-a,"A",a,a,0,0,1,e+r-a,t+n,"H",e+a,"A",a,a,0,0,1,e,t+n-a,"V",t+a,"A",a,a,0,0,1,e+a,t,"Z"].join(" "),"createRoundedRectPathD"),ole=U(async(e,t)=>{je.info("Creating subgraph rect for ",t.id,t);const r=Gr(),{themeVariables:n,handDrawnSeed:a}=r,{clusterBkg:i,clusterBorder:o}=n,{labelStyles:s,nodeStyles:l,borderStyles:u,backgroundStyles:d}=Nt(t),p=e.insert("g").attr("class","cluster "+t.cssClasses).attr("id",t.id).attr("data-look",t.look),f=ra(r.flowchart.htmlLabels),m=p.insert("g").attr("class","cluster-label "),b=await Gu(m,t.label,{style:t.labelStyle,useHtmlLabels:f,isNode:!0});let y=b.getBBox();if(ra(r.flowchart.htmlLabels)){const F=b.children[0],L=Lr(b);y=F.getBoundingClientRect(),L.attr("width",y.width),L.attr("height",y.height)}const S=t.width<=y.width+t.padding?y.width+t.padding:t.width;t.width<=y.width+t.padding?t.diff=(S-t.width)/2-t.padding:t.diff=-t.padding;const w=t.height,E=t.x-S/2,k=t.y-w/2;je.trace("Data ",t,JSON.stringify(t));let _;if(t.look==="handDrawn"){const F=_t.svg(p),L=Dt(t,{roughness:.7,fill:i,stroke:o,fillWeight:3,seed:a}),I=F.path(ju(E,k,S,w,0),L);_=p.insert(()=>(je.debug("Rough node insert CXC",I),I),":first-child"),_.select("path:nth-child(2)").attr("style",u.join(";")),_.select("path").attr("style",d.join(";").replace("fill","stroke"))}else _=p.insert("rect",":first-child"),_.attr("style",l).attr("rx",t.rx).attr("ry",t.ry).attr("x",E).attr("y",k).attr("width",S).attr("height",w);const{subGraphTitleTopMargin:R}=IN(r);if(m.attr("transform",`translate(${t.x-y.width/2}, ${t.y-t.height/2+R})`),s){const F=m.select("span");F&&F.attr("style",s)}const C=_.node().getBBox();return t.offsetX=0,t.width=C.width,t.height=C.height,t.offsetY=y.height-t.padding/2,t.intersect=function(F){return Lp(t,F)},{cluster:p,labelBBox:y}},"rect"),YYe=U((e,t)=>{const r=e.insert("g").attr("class","note-cluster").attr("id",t.id),n=r.insert("rect",":first-child"),a=0*t.padding,i=a/2;n.attr("rx",t.rx).attr("ry",t.ry).attr("x",t.x-t.width/2-i).attr("y",t.y-t.height/2-i).attr("width",t.width+a).attr("height",t.height+a).attr("fill","none");const o=n.node().getBBox();return t.width=o.width,t.height=o.height,t.intersect=function(s){return Lp(t,s)},{cluster:r,labelBBox:{width:0,height:0}}},"noteGroup"),JYe=U(async(e,t)=>{const r=Gr(),{themeVariables:n,handDrawnSeed:a}=r,{altBackground:i,compositeBackground:o,compositeTitleBackground:s,nodeBorder:l}=n,u=e.insert("g").attr("class",t.cssClasses).attr("id",t.id).attr("data-id",t.id).attr("data-look",t.look),d=u.insert("g",":first-child"),p=u.insert("g").attr("class","cluster-label");let f=u.append("rect");const m=p.node().appendChild(await Ac(t.label,t.labelStyle,void 0,!0));let b=m.getBBox();if(ra(r.flowchart.htmlLabels)){const I=m.children[0],M=Lr(m);b=I.getBoundingClientRect(),M.attr("width",b.width),M.attr("height",b.height)}const y=0*t.padding,S=y/2,w=(t.width<=b.width+t.padding?b.width+t.padding:t.width)+y;t.width<=b.width+t.padding?t.diff=(w-t.width)/2-t.padding:t.diff=-t.padding;const E=t.height+y,k=t.height+y-b.height-6,_=t.x-w/2,R=t.y-E/2;t.width=w;const C=t.y-t.height/2-S+b.height+2;let F;if(t.look==="handDrawn"){const I=t.cssClasses.includes("statediagram-cluster-alt"),M=_t.svg(u),j=t.rx||t.ry?M.path(ju(_,R,w,E,10),{roughness:.7,fill:s,fillStyle:"solid",stroke:l,seed:a}):M.rectangle(_,R,w,E,{seed:a});F=u.insert(()=>j,":first-child");const Y=M.rectangle(_,C,w,k,{fill:I?i:o,fillStyle:I?"hachure":"solid",stroke:l,seed:a});F=u.insert(()=>j,":first-child"),f=u.insert(()=>Y)}else F=d.insert("rect",":first-child"),F.attr("class","outer").attr("x",_).attr("y",R).attr("width",w).attr("height",E).attr("data-look",t.look),f.attr("class","inner").attr("x",_).attr("y",C).attr("width",w).attr("height",k);p.attr("transform",`translate(${t.x-b.width/2}, ${R+1-(ra(r.flowchart.htmlLabels)?0:3)})`);const L=F.node().getBBox();return t.height=L.height,t.offsetX=0,t.offsetY=b.height-t.padding/2,t.labelBBox=b,t.intersect=function(I){return Lp(t,I)},{cluster:u,labelBBox:b}},"roundedWithTitle"),XYe=U(async(e,t)=>{je.info("Creating subgraph rect for ",t.id,t);const r=Gr(),{themeVariables:n,handDrawnSeed:a}=r,{clusterBkg:i,clusterBorder:o}=n,{labelStyles:s,nodeStyles:l,borderStyles:u,backgroundStyles:d}=Nt(t),p=e.insert("g").attr("class","cluster "+t.cssClasses).attr("id",t.id).attr("data-look",t.look),f=ra(r.flowchart.htmlLabels),m=p.insert("g").attr("class","cluster-label "),b=await Gu(m,t.label,{style:t.labelStyle,useHtmlLabels:f,isNode:!0,width:t.width});let y=b.getBBox();if(ra(r.flowchart.htmlLabels)){const F=b.children[0],L=Lr(b);y=F.getBoundingClientRect(),L.attr("width",y.width),L.attr("height",y.height)}const S=t.width<=y.width+t.padding?y.width+t.padding:t.width;t.width<=y.width+t.padding?t.diff=(S-t.width)/2-t.padding:t.diff=-t.padding;const w=t.height,E=t.x-S/2,k=t.y-w/2;je.trace("Data ",t,JSON.stringify(t));let _;if(t.look==="handDrawn"){const F=_t.svg(p),L=Dt(t,{roughness:.7,fill:i,stroke:o,fillWeight:4,seed:a}),I=F.path(ju(E,k,S,w,t.rx),L);_=p.insert(()=>(je.debug("Rough node insert CXC",I),I),":first-child"),_.select("path:nth-child(2)").attr("style",u.join(";")),_.select("path").attr("style",d.join(";").replace("fill","stroke"))}else _=p.insert("rect",":first-child"),_.attr("style",l).attr("rx",t.rx).attr("ry",t.ry).attr("x",E).attr("y",k).attr("width",S).attr("height",w);const{subGraphTitleTopMargin:R}=IN(r);if(m.attr("transform",`translate(${t.x-y.width/2}, ${t.y-t.height/2+R})`),s){const F=m.select("span");F&&F.attr("style",s)}const C=_.node().getBBox();return t.offsetX=0,t.width=C.width,t.height=C.height,t.offsetY=y.height-t.padding/2,t.intersect=function(F){return Lp(t,F)},{cluster:p,labelBBox:y}},"kanbanSection"),ZYe=U((e,t)=>{const r=Gr(),{themeVariables:n,handDrawnSeed:a}=r,{nodeBorder:i}=n,o=e.insert("g").attr("class",t.cssClasses).attr("id",t.id).attr("data-look",t.look),s=o.insert("g",":first-child"),l=0*t.padding,u=t.width+l;t.diff=-t.padding;const d=t.height+l,p=t.x-u/2,f=t.y-d/2;t.width=u;let m;if(t.look==="handDrawn"){const S=_t.svg(o).rectangle(p,f,u,d,{fill:"lightgrey",roughness:.5,strokeLineDash:[5],stroke:i,seed:a});m=o.insert(()=>S,":first-child")}else m=s.insert("rect",":first-child"),m.attr("class","divider").attr("x",p).attr("y",f).attr("width",u).attr("height",d).attr("data-look",t.look);const b=m.node().getBBox();return t.height=b.height,t.offsetX=0,t.offsetY=0,t.intersect=function(y){return Lp(t,y)},{cluster:o,labelBBox:{}}},"divider"),QYe=ole,eJe={rect:ole,squareRect:QYe,roundedWithTitle:JYe,noteGroup:YYe,divider:ZYe,kanbanSection:XYe},sle=new Map,tJe=U(async(e,t)=>{const r=t.shape||"rect",n=await eJe[r](e,t);return sle.set(t.id,n),n},"insertCluster"),jdt=U(()=>{sle=new Map},"clear");function lle(e,t){return e.intersect(t)}U(lle,"intersectNode");var rJe=lle;function ule(e,t,r,n){var a=e.x,i=e.y,o=a-n.x,s=i-n.y,l=Math.sqrt(t*t*s*s+r*r*o*o),u=Math.abs(t*r*o/l);n.x<a&&(u=-u);var d=Math.abs(t*r*s/l);return n.y<i&&(d=-d),{x:a+u,y:i+d}}U(ule,"intersectEllipse");var cle=ule;function dle(e,t,r){return cle(e,t,t,r)}U(dle,"intersectCircle");var nJe=dle;function hle(e,t,r,n){{const a=t.y-e.y,i=e.x-t.x,o=t.x*e.y-e.x*t.y,s=a*r.x+i*r.y+o,l=a*n.x+i*n.y+o,u=1e-6;if(s!==0&&l!==0&&H6(s,l))return;const d=n.y-r.y,p=r.x-n.x,f=n.x*r.y-r.x*n.y,m=d*e.x+p*e.y+f,b=d*t.x+p*t.y+f;if(Math.abs(m)<u&&Math.abs(b)<u&&H6(m,b))return;const y=a*p-d*i;if(y===0)return;const S=Math.abs(y/2);let w=i*f-p*o;const E=w<0?(w-S)/y:(w+S)/y;w=d*o-a*f;const k=w<0?(w-S)/y:(w+S)/y;return{x:E,y:k}}}U(hle,"intersectLine");function H6(e,t){return e*t>0}U(H6,"sameSign");var aJe=hle;function ple(e,t,r){let n=e.x,a=e.y,i=[],o=Number.POSITIVE_INFINITY,s=Number.POSITIVE_INFINITY;typeof t.forEach=="function"?t.forEach(function(d){o=Math.min(o,d.x),s=Math.min(s,d.y)}):(o=Math.min(o,t.x),s=Math.min(s,t.y));let l=n-e.width/2-o,u=a-e.height/2-s;for(let d=0;d<t.length;d++){let p=t[d],f=t[d<t.length-1?d+1:0],m=aJe(e,r,{x:l+p.x,y:u+p.y},{x:l+f.x,y:u+f.y});m&&i.push(m)}return i.length?(i.length>1&&i.sort(function(d,p){let f=d.x-r.x,m=d.y-r.y,b=Math.sqrt(f*f+m*m),y=p.x-r.x,S=p.y-r.y,w=Math.sqrt(y*y+S*S);return b<w?-1:b===w?0:1}),i[0]):e}U(ple,"intersectPolygon");var iJe=ple,yt={node:rJe,circle:nJe,ellipse:cle,polygon:iJe,rect:Lp};function fle(e,t){const{labelStyles:r}=Nt(t);t.labelStyle=r;const n=jt(t);let a=n;n||(a="anchor");const i=e.insert("g").attr("class",a).attr("id",t.domId||t.id),o=1,{cssStyles:s}=t,l=_t.svg(i),u=Dt(t,{fill:"black",stroke:"none",fillStyle:"solid"});t.look!=="handDrawn"&&(u.roughness=0);const d=l.circle(0,0,o*2,u),p=i.insert(()=>d,":first-child");return p.attr("class","anchor").attr("style",Oa(s)),Lt(t,p),t.intersect=function(f){return je.info("Circle intersect",t,o,f),yt.circle(t,o,f)},i}U(fle,"anchor");function U6(e,t,r,n,a,i,o){const l=(e+r)/2,u=(t+n)/2,d=Math.atan2(n-t,r-e),p=(r-e)/2,f=(n-t)/2,m=p/a,b=f/i,y=Math.sqrt(m**2+b**2);if(y>1)throw new Error("The given radii are too small to create an arc between the points.");const S=Math.sqrt(1-y**2),w=l+S*i*Math.sin(d)*(o?-1:1),E=u-S*a*Math.cos(d)*(o?-1:1),k=Math.atan2((t-E)/i,(e-w)/a);let R=Math.atan2((n-E)/i,(r-w)/a)-k;o&&R<0&&(R+=2*Math.PI),!o&&R>0&&(R-=2*Math.PI);const C=[];for(let F=0;F<20;F++){const L=F/19,I=k+L*R,M=w+a*Math.cos(I),j=E+i*Math.sin(I);C.push({x:M,y:j})}return C}U(U6,"generateArcPoints");async function mle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=i.width+t.padding+20,s=i.height+t.padding,l=s/2,u=l/(2.5+s/50),{cssStyles:d}=t,p=[{x:o/2,y:-s/2},{x:-o/2,y:-s/2},...U6(-o/2,-s/2,-o/2,s/2,u,l,!1),{x:o/2,y:s/2},...U6(o/2,s/2,o/2,-s/2,u,l,!0)],f=_t.svg(a),m=Dt(t,{});t.look!=="handDrawn"&&(m.roughness=0,m.fillStyle="solid");const b=Ar(p),y=f.path(b,m),S=a.insert(()=>y,":first-child");return S.attr("class","basic label-container"),d&&t.look!=="handDrawn"&&S.selectAll("path").attr("style",d),n&&t.look!=="handDrawn"&&S.selectAll("path").attr("style",n),S.attr("transform",`translate(${u/2}, 0)`),Lt(t,S),t.intersect=function(w){return yt.polygon(t,p,w)},a}U(mle,"bowTieRect");function qu(e,t,r,n){return e.insert("polygon",":first-child").attr("points",n.map(function(a){return a.x+","+a.y}).join(" ")).attr("class","label-container").attr("transform","translate("+-t/2+","+r/2+")")}U(qu,"insertPolygonShape");async function gle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=i.height+t.padding,s=12,l=i.width+t.padding+s,u=0,d=l,p=-o,f=0,m=[{x:u+s,y:p},{x:d,y:p},{x:d,y:f},{x:u,y:f},{x:u,y:p+s},{x:u+s,y:p}];let b;const{cssStyles:y}=t;if(t.look==="handDrawn"){const S=_t.svg(a),w=Dt(t,{}),E=Ar(m),k=S.path(E,w);b=a.insert(()=>k,":first-child").attr("transform",`translate(${-l/2}, ${o/2})`),y&&b.attr("style",y)}else b=qu(a,l,o,m);return n&&b.attr("style",n),Lt(t,b),t.intersect=function(S){return yt.polygon(t,m,S)},a}U(gle,"card");function ble(e,t){const{nodeStyles:r}=Nt(t);t.label="";const n=e.insert("g").attr("class",jt(t)).attr("id",t.domId??t.id),{cssStyles:a}=t,i=Math.max(28,t.width??0),o=[{x:0,y:i/2},{x:i/2,y:0},{x:0,y:-i/2},{x:-i/2,y:0}],s=_t.svg(n),l=Dt(t,{});t.look!=="handDrawn"&&(l.roughness=0,l.fillStyle="solid");const u=Ar(o),d=s.path(u,l),p=n.insert(()=>d,":first-child");return a&&t.look!=="handDrawn"&&p.selectAll("path").attr("style",a),r&&t.look!=="handDrawn"&&p.selectAll("path").attr("style",r),t.width=28,t.height=28,t.intersect=function(f){return yt.polygon(t,o,f)},n}U(ble,"choice");async function aL(e,t,r){const{labelStyles:n,nodeStyles:a}=Nt(t);t.labelStyle=n;const{shapeSvg:i,bbox:o,halfPadding:s}=await Zt(e,t,jt(t)),l=(r==null?void 0:r.padding)??s,u=o.width/2+l;let d;const{cssStyles:p}=t;if(t.look==="handDrawn"){const f=_t.svg(i),m=Dt(t,{}),b=f.circle(0,0,u*2,m);d=i.insert(()=>b,":first-child"),d.attr("class","basic label-container").attr("style",Oa(p))}else d=i.insert("circle",":first-child").attr("class","basic label-container").attr("style",a).attr("r",u).attr("cx",0).attr("cy",0);return Lt(t,d),t.calcIntersect=function(f,m){const b=f.width/2;return yt.circle(f,b,m)},t.intersect=function(f){return je.info("Circle intersect",t,u,f),yt.circle(t,u,f)},i}U(aL,"circle");function yle(e){const t=Math.cos(Math.PI/4),r=Math.sin(Math.PI/4),n=e*2,a={x:n/2*t,y:n/2*r},i={x:-(n/2)*t,y:n/2*r},o={x:-(n/2)*t,y:-(n/2)*r},s={x:n/2*t,y:-(n/2)*r};return`M ${i.x},${i.y} L ${s.x},${s.y}
                   M ${a.x},${a.y} L ${o.x},${o.y}`}U(yle,"createLine");function vle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r,t.label="";const a=e.insert("g").attr("class",jt(t)).attr("id",t.domId??t.id),i=Math.max(30,(t==null?void 0:t.width)??0),{cssStyles:o}=t,s=_t.svg(a),l=Dt(t,{});t.look!=="handDrawn"&&(l.roughness=0,l.fillStyle="solid");const u=s.circle(0,0,i*2,l),d=yle(i),p=s.path(d,l),f=a.insert(()=>u,":first-child");return f.insert(()=>p),o&&t.look!=="handDrawn"&&f.selectAll("path").attr("style",o),n&&t.look!=="handDrawn"&&f.selectAll("path").attr("style",n),Lt(t,f),t.intersect=function(m){return je.info("crossedCircle intersect",t,{radius:i,point:m}),yt.circle(t,i,m)},a}U(vle,"crossedCircle");function tl(e,t,r,n=100,a=0,i=180){const o=[],s=a*Math.PI/180,d=(i*Math.PI/180-s)/(n-1);for(let p=0;p<n;p++){const f=s+p*d,m=e+r*Math.cos(f),b=t+r*Math.sin(f);o.push({x:-m,y:-b})}return o}U(tl,"generateCirclePoints");async function Sle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=i.width+(t.padding??0),l=i.height+(t.padding??0),u=Math.max(5,l*.1),{cssStyles:d}=t,p=[...tl(s/2,-l/2,u,30,-90,0),{x:-s/2-u,y:u},...tl(s/2+u*2,-u,u,20,-180,-270),...tl(s/2+u*2,u,u,20,-90,-180),{x:-s/2-u,y:-l/2},...tl(s/2,l/2,u,20,0,90)],f=[{x:s/2,y:-l/2-u},{x:-s/2,y:-l/2-u},...tl(s/2,-l/2,u,20,-90,0),{x:-s/2-u,y:-u},...tl(s/2+s*.1,-u,u,20,-180,-270),...tl(s/2+s*.1,u,u,20,-90,-180),{x:-s/2-u,y:l/2},...tl(s/2,l/2,u,20,0,90),{x:-s/2,y:l/2+u},{x:s/2,y:l/2+u}],m=_t.svg(a),b=Dt(t,{fill:"none"});t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const S=Ar(p).replace("Z",""),w=m.path(S,b),E=Ar(f),k=m.path(E,{...b}),_=a.insert("g",":first-child");return _.insert(()=>k,":first-child").attr("stroke-opacity",0),_.insert(()=>w,":first-child"),_.attr("class","text"),d&&t.look!=="handDrawn"&&_.selectAll("path").attr("style",d),n&&t.look!=="handDrawn"&&_.selectAll("path").attr("style",n),_.attr("transform",`translate(${u}, 0)`),o.attr("transform",`translate(${-s/2+u-(i.x-(i.left??0))},${-l/2+(t.padding??0)/2-(i.y-(i.top??0))})`),Lt(t,_),t.intersect=function(R){return yt.polygon(t,f,R)},a}U(Sle,"curlyBraceLeft");function rl(e,t,r,n=100,a=0,i=180){const o=[],s=a*Math.PI/180,d=(i*Math.PI/180-s)/(n-1);for(let p=0;p<n;p++){const f=s+p*d,m=e+r*Math.cos(f),b=t+r*Math.sin(f);o.push({x:m,y:b})}return o}U(rl,"generateCirclePoints");async function Ele(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=i.width+(t.padding??0),l=i.height+(t.padding??0),u=Math.max(5,l*.1),{cssStyles:d}=t,p=[...rl(s/2,-l/2,u,20,-90,0),{x:s/2+u,y:-u},...rl(s/2+u*2,-u,u,20,-180,-270),...rl(s/2+u*2,u,u,20,-90,-180),{x:s/2+u,y:l/2},...rl(s/2,l/2,u,20,0,90)],f=[{x:-s/2,y:-l/2-u},{x:s/2,y:-l/2-u},...rl(s/2,-l/2,u,20,-90,0),{x:s/2+u,y:-u},...rl(s/2+u*2,-u,u,20,-180,-270),...rl(s/2+u*2,u,u,20,-90,-180),{x:s/2+u,y:l/2},...rl(s/2,l/2,u,20,0,90),{x:s/2,y:l/2+u},{x:-s/2,y:l/2+u}],m=_t.svg(a),b=Dt(t,{fill:"none"});t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const S=Ar(p).replace("Z",""),w=m.path(S,b),E=Ar(f),k=m.path(E,{...b}),_=a.insert("g",":first-child");return _.insert(()=>k,":first-child").attr("stroke-opacity",0),_.insert(()=>w,":first-child"),_.attr("class","text"),d&&t.look!=="handDrawn"&&_.selectAll("path").attr("style",d),n&&t.look!=="handDrawn"&&_.selectAll("path").attr("style",n),_.attr("transform",`translate(${-u}, 0)`),o.attr("transform",`translate(${-s/2+(t.padding??0)/2-(i.x-(i.left??0))},${-l/2+(t.padding??0)/2-(i.y-(i.top??0))})`),Lt(t,_),t.intersect=function(R){return yt.polygon(t,f,R)},a}U(Ele,"curlyBraceRight");function ma(e,t,r,n=100,a=0,i=180){const o=[],s=a*Math.PI/180,d=(i*Math.PI/180-s)/(n-1);for(let p=0;p<n;p++){const f=s+p*d,m=e+r*Math.cos(f),b=t+r*Math.sin(f);o.push({x:-m,y:-b})}return o}U(ma,"generateCirclePoints");async function xle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=i.width+(t.padding??0),l=i.height+(t.padding??0),u=Math.max(5,l*.1),{cssStyles:d}=t,p=[...ma(s/2,-l/2,u,30,-90,0),{x:-s/2-u,y:u},...ma(s/2+u*2,-u,u,20,-180,-270),...ma(s/2+u*2,u,u,20,-90,-180),{x:-s/2-u,y:-l/2},...ma(s/2,l/2,u,20,0,90)],f=[...ma(-s/2+u+u/2,-l/2,u,20,-90,-180),{x:s/2-u/2,y:u},...ma(-s/2-u/2,-u,u,20,0,90),...ma(-s/2-u/2,u,u,20,-90,0),{x:s/2-u/2,y:-u},...ma(-s/2+u+u/2,l/2,u,30,-180,-270)],m=[{x:s/2,y:-l/2-u},{x:-s/2,y:-l/2-u},...ma(s/2,-l/2,u,20,-90,0),{x:-s/2-u,y:-u},...ma(s/2+u*2,-u,u,20,-180,-270),...ma(s/2+u*2,u,u,20,-90,-180),{x:-s/2-u,y:l/2},...ma(s/2,l/2,u,20,0,90),{x:-s/2,y:l/2+u},{x:s/2-u-u/2,y:l/2+u},...ma(-s/2+u+u/2,-l/2,u,20,-90,-180),{x:s/2-u/2,y:u},...ma(-s/2-u/2,-u,u,20,0,90),...ma(-s/2-u/2,u,u,20,-90,0),{x:s/2-u/2,y:-u},...ma(-s/2+u+u/2,l/2,u,30,-180,-270)],b=_t.svg(a),y=Dt(t,{fill:"none"});t.look!=="handDrawn"&&(y.roughness=0,y.fillStyle="solid");const w=Ar(p).replace("Z",""),E=b.path(w,y),_=Ar(f).replace("Z",""),R=b.path(_,y),C=Ar(m),F=b.path(C,{...y}),L=a.insert("g",":first-child");return L.insert(()=>F,":first-child").attr("stroke-opacity",0),L.insert(()=>E,":first-child"),L.insert(()=>R,":first-child"),L.attr("class","text"),d&&t.look!=="handDrawn"&&L.selectAll("path").attr("style",d),n&&t.look!=="handDrawn"&&L.selectAll("path").attr("style",n),L.attr("transform",`translate(${u-u/4}, 0)`),o.attr("transform",`translate(${-s/2+(t.padding??0)/2-(i.x-(i.left??0))},${-l/2+(t.padding??0)/2-(i.y-(i.top??0))})`),Lt(t,L),t.intersect=function(I){return yt.polygon(t,m,I)},a}U(xle,"curlyBraces");async function wle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=80,s=20,l=Math.max(o,(i.width+(t.padding??0)*2)*1.25,(t==null?void 0:t.width)??0),u=Math.max(s,i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),d=u/2,{cssStyles:p}=t,f=_t.svg(a),m=Dt(t,{});t.look!=="handDrawn"&&(m.roughness=0,m.fillStyle="solid");const b=l,y=u,S=b-d,w=y/4,E=[{x:S,y:0},{x:w,y:0},{x:0,y:y/2},{x:w,y},{x:S,y},...ag(-S,-y/2,d,50,270,90)],k=Ar(E),_=f.path(k,m),R=a.insert(()=>_,":first-child");return R.attr("class","basic label-container"),p&&t.look!=="handDrawn"&&R.selectChildren("path").attr("style",p),n&&t.look!=="handDrawn"&&R.selectChildren("path").attr("style",n),R.attr("transform",`translate(${-l/2}, ${-u/2})`),Lt(t,R),t.intersect=function(C){return yt.polygon(t,E,C)},a}U(wle,"curvedTrapezoid");var oJe=U((e,t,r,n,a,i)=>[`M${e},${t+i}`,`a${a},${i} 0,0,0 ${r},0`,`a${a},${i} 0,0,0 ${-r},0`,`l0,${n}`,`a${a},${i} 0,0,0 ${r},0`,`l0,${-n}`].join(" "),"createCylinderPathD"),sJe=U((e,t,r,n,a,i)=>[`M${e},${t+i}`,`M${e+r},${t+i}`,`a${a},${i} 0,0,0 ${-r},0`,`l0,${n}`,`a${a},${i} 0,0,0 ${r},0`,`l0,${-n}`].join(" "),"createOuterCylinderPathD"),lJe=U((e,t,r,n,a,i)=>[`M${e-r/2},${-n/2}`,`a${a},${i} 0,0,0 ${r},0`].join(" "),"createInnerCylinderPathD");async function kle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+t.padding,t.width??0),l=s/2,u=l/(2.5+s/50),d=Math.max(i.height+u+t.padding,t.height??0);let p;const{cssStyles:f}=t;if(t.look==="handDrawn"){const m=_t.svg(a),b=sJe(0,0,s,d,l,u),y=lJe(0,u,s,d,l,u),S=m.path(b,Dt(t,{})),w=m.path(y,Dt(t,{fill:"none"}));p=a.insert(()=>w,":first-child"),p=a.insert(()=>S,":first-child"),p.attr("class","basic label-container"),f&&p.attr("style",f)}else{const m=oJe(0,0,s,d,l,u);p=a.insert("path",":first-child").attr("d",m).attr("class","basic label-container").attr("style",Oa(f)).attr("style",n)}return p.attr("label-offset-y",u),p.attr("transform",`translate(${-s/2}, ${-(d/2+u)})`),Lt(t,p),o.attr("transform",`translate(${-(i.width/2)-(i.x-(i.left??0))}, ${-(i.height/2)+(t.padding??0)/1.5-(i.y-(i.top??0))})`),t.intersect=function(m){const b=yt.rect(t,m),y=b.x-(t.x??0);if(l!=0&&(Math.abs(y)<(t.width??0)/2||Math.abs(y)==(t.width??0)/2&&Math.abs(b.y-(t.y??0))>(t.height??0)/2-u)){let S=u*u*(1-y*y/(l*l));S>0&&(S=Math.sqrt(S)),S=u-S,m.y-(t.y??0)>0&&(S=-S),b.y+=S}return b},a}U(kle,"cylinder");async function Cle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=i.width+t.padding,l=i.height+t.padding,u=l*.2,d=-s/2,p=-l/2-u/2,{cssStyles:f}=t,m=_t.svg(a),b=Dt(t,{});t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const y=[{x:d,y:p+u},{x:-d,y:p+u},{x:-d,y:-p},{x:d,y:-p},{x:d,y:p},{x:-d,y:p},{x:-d,y:p+u}],S=m.polygon(y.map(E=>[E.x,E.y]),b),w=a.insert(()=>S,":first-child");return w.attr("class","basic label-container"),f&&t.look!=="handDrawn"&&w.selectAll("path").attr("style",f),n&&t.look!=="handDrawn"&&w.selectAll("path").attr("style",n),o.attr("transform",`translate(${d+(t.padding??0)/2-(i.x-(i.left??0))}, ${p+u+(t.padding??0)/2-(i.y-(i.top??0))})`),Lt(t,w),t.intersect=function(E){return yt.rect(t,E)},a}U(Cle,"dividedRectangle");async function Tle(e,t){var f,m;const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,halfPadding:o}=await Zt(e,t,jt(t)),l=i.width/2+o+5,u=i.width/2+o;let d;const{cssStyles:p}=t;if(t.look==="handDrawn"){const b=_t.svg(a),y=Dt(t,{roughness:.2,strokeWidth:2.5}),S=Dt(t,{roughness:.2,strokeWidth:1.5}),w=b.circle(0,0,l*2,y),E=b.circle(0,0,u*2,S);d=a.insert("g",":first-child"),d.attr("class",Oa(t.cssClasses)).attr("style",Oa(p)),(f=d.node())==null||f.appendChild(w),(m=d.node())==null||m.appendChild(E)}else{d=a.insert("g",":first-child");const b=d.insert("circle",":first-child"),y=d.insert("circle");d.attr("class","basic label-container").attr("style",n),b.attr("class","outer-circle").attr("style",n).attr("r",l).attr("cx",0).attr("cy",0),y.attr("class","inner-circle").attr("style",n).attr("r",u).attr("cx",0).attr("cy",0)}return Lt(t,d),t.intersect=function(b){return je.info("DoubleCircle intersect",t,l,b),yt.circle(t,l,b)},a}U(Tle,"doublecircle");function Ale(e,t,{config:{themeVariables:r}}){const{labelStyles:n,nodeStyles:a}=Nt(t);t.label="",t.labelStyle=n;const i=e.insert("g").attr("class",jt(t)).attr("id",t.domId??t.id),o=7,{cssStyles:s}=t,l=_t.svg(i),{nodeBorder:u}=r,d=Dt(t,{fillStyle:"solid"});t.look!=="handDrawn"&&(d.roughness=0);const p=l.circle(0,0,o*2,d),f=i.insert(()=>p,":first-child");return f.selectAll("path").attr("style",`fill: ${u} !important;`),s&&s.length>0&&t.look!=="handDrawn"&&f.selectAll("path").attr("style",s),a&&t.look!=="handDrawn"&&f.selectAll("path").attr("style",a),Lt(t,f),t.intersect=function(m){return je.info("filledCircle intersect",t,{radius:o,point:m}),yt.circle(t,o,m)},i}U(Ale,"filledCircle");async function Fle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=i.width+(t.padding??0),l=s+i.height,u=s+i.height,d=[{x:0,y:-l},{x:u,y:-l},{x:u/2,y:0}],{cssStyles:p}=t,f=_t.svg(a),m=Dt(t,{});t.look!=="handDrawn"&&(m.roughness=0,m.fillStyle="solid");const b=Ar(d),y=f.path(b,m),S=a.insert(()=>y,":first-child").attr("transform",`translate(${-l/2}, ${l/2})`);return p&&t.look!=="handDrawn"&&S.selectChildren("path").attr("style",p),n&&t.look!=="handDrawn"&&S.selectChildren("path").attr("style",n),t.width=s,t.height=l,Lt(t,S),o.attr("transform",`translate(${-i.width/2-(i.x-(i.left??0))}, ${-l/2+(t.padding??0)/2+(i.y-(i.top??0))})`),t.intersect=function(w){return je.info("Triangle intersect",t,d,w),yt.polygon(t,d,w)},a}U(Fle,"flippedTriangle");function _le(e,t,{dir:r,config:{state:n,themeVariables:a}}){const{nodeStyles:i}=Nt(t);t.label="";const o=e.insert("g").attr("class",jt(t)).attr("id",t.domId??t.id),{cssStyles:s}=t;let l=Math.max(70,(t==null?void 0:t.width)??0),u=Math.max(10,(t==null?void 0:t.height)??0);r==="LR"&&(l=Math.max(10,(t==null?void 0:t.width)??0),u=Math.max(70,(t==null?void 0:t.height)??0));const d=-1*l/2,p=-1*u/2,f=_t.svg(o),m=Dt(t,{stroke:a.lineColor,fill:a.lineColor});t.look!=="handDrawn"&&(m.roughness=0,m.fillStyle="solid");const b=f.rectangle(d,p,l,u,m),y=o.insert(()=>b,":first-child");s&&t.look!=="handDrawn"&&y.selectAll("path").attr("style",s),i&&t.look!=="handDrawn"&&y.selectAll("path").attr("style",i),Lt(t,y);const S=(n==null?void 0:n.padding)??0;return t.width&&t.height&&(t.width+=S/2||0,t.height+=S/2||0),t.intersect=function(w){return yt.rect(t,w)},o}U(_le,"forkJoin");async function Rle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const a=80,i=50,{shapeSvg:o,bbox:s}=await Zt(e,t,jt(t)),l=Math.max(a,s.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),u=Math.max(i,s.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),d=u/2,{cssStyles:p}=t,f=_t.svg(o),m=Dt(t,{});t.look!=="handDrawn"&&(m.roughness=0,m.fillStyle="solid");const b=[{x:-l/2,y:-u/2},{x:l/2-d,y:-u/2},...ag(-l/2+d,0,d,50,90,270),{x:l/2-d,y:u/2},{x:-l/2,y:u/2}],y=Ar(b),S=f.path(y,m),w=o.insert(()=>S,":first-child");return w.attr("class","basic label-container"),p&&t.look!=="handDrawn"&&w.selectChildren("path").attr("style",p),n&&t.look!=="handDrawn"&&w.selectChildren("path").attr("style",n),Lt(t,w),t.intersect=function(E){return je.info("Pill intersect",t,{radius:d,point:E}),yt.polygon(t,b,E)},o}U(Rle,"halfRoundedRectangle");async function Dle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=i.height+(t.padding??0),s=i.width+(t.padding??0)*2.5,{cssStyles:l}=t,u=_t.svg(a),d=Dt(t,{});t.look!=="handDrawn"&&(d.roughness=0,d.fillStyle="solid");let p=s/2;const f=p/6;p=p+f;const m=o/2,b=m/2,y=p-b,S=[{x:-y,y:-m},{x:0,y:-m},{x:y,y:-m},{x:p,y:0},{x:y,y:m},{x:0,y:m},{x:-y,y:m},{x:-p,y:0}],w=Ar(S),E=u.path(w,d),k=a.insert(()=>E,":first-child");return k.attr("class","basic label-container"),l&&t.look!=="handDrawn"&&k.selectChildren("path").attr("style",l),n&&t.look!=="handDrawn"&&k.selectChildren("path").attr("style",n),t.width=s,t.height=o,Lt(t,k),t.intersect=function(_){return yt.polygon(t,S,_)},a}U(Dle,"hexagon");async function Nle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.label="",t.labelStyle=r;const{shapeSvg:a}=await Zt(e,t,jt(t)),i=Math.max(30,(t==null?void 0:t.width)??0),o=Math.max(30,(t==null?void 0:t.height)??0),{cssStyles:s}=t,l=_t.svg(a),u=Dt(t,{});t.look!=="handDrawn"&&(u.roughness=0,u.fillStyle="solid");const d=[{x:0,y:0},{x:i,y:0},{x:0,y:o},{x:i,y:o}],p=Ar(d),f=l.path(p,u),m=a.insert(()=>f,":first-child");return m.attr("class","basic label-container"),s&&t.look!=="handDrawn"&&m.selectChildren("path").attr("style",s),n&&t.look!=="handDrawn"&&m.selectChildren("path").attr("style",n),m.attr("transform",`translate(${-i/2}, ${-o/2})`),Lt(t,m),t.intersect=function(b){return je.info("Pill intersect",t,{points:d}),yt.polygon(t,d,b)},a}U(Nle,"hourglass");async function Lle(e,t,{config:{themeVariables:r,flowchart:n}}){const{labelStyles:a}=Nt(t);t.labelStyle=a;const i=t.assetHeight??48,o=t.assetWidth??48,s=Math.max(i,o),l=n==null?void 0:n.wrappingWidth;t.width=Math.max(s,l??0);const{shapeSvg:u,bbox:d,label:p}=await Zt(e,t,"icon-shape default"),f=t.pos==="t",m=s,b=s,{nodeBorder:y}=r,{stylesMap:S}=Rp(t),w=-b/2,E=-m/2,k=t.label?8:0,_=_t.svg(u),R=Dt(t,{stroke:"none",fill:"none"});t.look!=="handDrawn"&&(R.roughness=0,R.fillStyle="solid");const C=_.rectangle(w,E,b,m,R),F=Math.max(b,d.width),L=m+d.height+k,I=_.rectangle(-F/2,-L/2,F,L,{...R,fill:"transparent",stroke:"none"}),M=u.insert(()=>C,":first-child"),j=u.insert(()=>I);if(t.icon){const Y=u.append("g");Y.html(`<g>${await Kg(t.icon,{height:s,width:s,fallbackPrefix:""})}</g>`);const W=Y.node().getBBox(),$=W.width,X=W.height,z=W.x,H=W.y;Y.attr("transform",`translate(${-$/2-z},${f?d.height/2+k/2-X/2-H:-d.height/2-k/2-X/2-H})`),Y.attr("style",`color: ${S.get("stroke")??y};`)}return p.attr("transform",`translate(${-d.width/2-(d.x-(d.left??0))},${f?-L/2:L/2-d.height})`),M.attr("transform",`translate(0,${f?d.height/2+k/2:-d.height/2-k/2})`),Lt(t,j),t.intersect=function(Y){if(je.info("iconSquare intersect",t,Y),!t.label)return yt.rect(t,Y);const W=t.x??0,$=t.y??0,X=t.height??0;let z=[];return f?z=[{x:W-d.width/2,y:$-X/2},{x:W+d.width/2,y:$-X/2},{x:W+d.width/2,y:$-X/2+d.height+k},{x:W+b/2,y:$-X/2+d.height+k},{x:W+b/2,y:$+X/2},{x:W-b/2,y:$+X/2},{x:W-b/2,y:$-X/2+d.height+k},{x:W-d.width/2,y:$-X/2+d.height+k}]:z=[{x:W-b/2,y:$-X/2},{x:W+b/2,y:$-X/2},{x:W+b/2,y:$-X/2+m},{x:W+d.width/2,y:$-X/2+m},{x:W+d.width/2/2,y:$+X/2},{x:W-d.width/2,y:$+X/2},{x:W-d.width/2,y:$-X/2+m},{x:W-b/2,y:$-X/2+m}],yt.polygon(t,z,Y)},u}U(Lle,"icon");async function Mle(e,t,{config:{themeVariables:r,flowchart:n}}){const{labelStyles:a}=Nt(t);t.labelStyle=a;const i=t.assetHeight??48,o=t.assetWidth??48,s=Math.max(i,o),l=n==null?void 0:n.wrappingWidth;t.width=Math.max(s,l??0);const{shapeSvg:u,bbox:d,label:p}=await Zt(e,t,"icon-shape default"),f=20,m=t.label?8:0,b=t.pos==="t",{nodeBorder:y,mainBkg:S}=r,{stylesMap:w}=Rp(t),E=_t.svg(u),k=Dt(t,{});t.look!=="handDrawn"&&(k.roughness=0,k.fillStyle="solid");const _=w.get("fill");k.stroke=_??S;const R=u.append("g");t.icon&&R.html(`<g>${await Kg(t.icon,{height:s,width:s,fallbackPrefix:""})}</g>`);const C=R.node().getBBox(),F=C.width,L=C.height,I=C.x,M=C.y,j=Math.max(F,L)*Math.SQRT2+f*2,Y=E.circle(0,0,j,k),W=Math.max(j,d.width),$=j+d.height+m,X=E.rectangle(-W/2,-$/2,W,$,{...k,fill:"transparent",stroke:"none"}),z=u.insert(()=>Y,":first-child"),H=u.insert(()=>X);return R.attr("transform",`translate(${-F/2-I},${b?d.height/2+m/2-L/2-M:-d.height/2-m/2-L/2-M})`),R.attr("style",`color: ${w.get("stroke")??y};`),p.attr("transform",`translate(${-d.width/2-(d.x-(d.left??0))},${b?-$/2:$/2-d.height})`),z.attr("transform",`translate(0,${b?d.height/2+m/2:-d.height/2-m/2})`),Lt(t,H),t.intersect=function(B){return je.info("iconSquare intersect",t,B),yt.rect(t,B)},u}U(Mle,"iconCircle");async function Ile(e,t,{config:{themeVariables:r,flowchart:n}}){const{labelStyles:a}=Nt(t);t.labelStyle=a;const i=t.assetHeight??48,o=t.assetWidth??48,s=Math.max(i,o),l=n==null?void 0:n.wrappingWidth;t.width=Math.max(s,l??0);const{shapeSvg:u,bbox:d,halfPadding:p,label:f}=await Zt(e,t,"icon-shape default"),m=t.pos==="t",b=s+p*2,y=s+p*2,{nodeBorder:S,mainBkg:w}=r,{stylesMap:E}=Rp(t),k=-y/2,_=-b/2,R=t.label?8:0,C=_t.svg(u),F=Dt(t,{});t.look!=="handDrawn"&&(F.roughness=0,F.fillStyle="solid");const L=E.get("fill");F.stroke=L??w;const I=C.path(ju(k,_,y,b,5),F),M=Math.max(y,d.width),j=b+d.height+R,Y=C.rectangle(-M/2,-j/2,M,j,{...F,fill:"transparent",stroke:"none"}),W=u.insert(()=>I,":first-child").attr("class","icon-shape2"),$=u.insert(()=>Y);if(t.icon){const X=u.append("g");X.html(`<g>${await Kg(t.icon,{height:s,width:s,fallbackPrefix:""})}</g>`);const z=X.node().getBBox(),H=z.width,B=z.height,ee=z.x,J=z.y;X.attr("transform",`translate(${-H/2-ee},${m?d.height/2+R/2-B/2-J:-d.height/2-R/2-B/2-J})`),X.attr("style",`color: ${E.get("stroke")??S};`)}return f.attr("transform",`translate(${-d.width/2-(d.x-(d.left??0))},${m?-j/2:j/2-d.height})`),W.attr("transform",`translate(0,${m?d.height/2+R/2:-d.height/2-R/2})`),Lt(t,$),t.intersect=function(X){if(je.info("iconSquare intersect",t,X),!t.label)return yt.rect(t,X);const z=t.x??0,H=t.y??0,B=t.height??0;let ee=[];return m?ee=[{x:z-d.width/2,y:H-B/2},{x:z+d.width/2,y:H-B/2},{x:z+d.width/2,y:H-B/2+d.height+R},{x:z+y/2,y:H-B/2+d.height+R},{x:z+y/2,y:H+B/2},{x:z-y/2,y:H+B/2},{x:z-y/2,y:H-B/2+d.height+R},{x:z-d.width/2,y:H-B/2+d.height+R}]:ee=[{x:z-y/2,y:H-B/2},{x:z+y/2,y:H-B/2},{x:z+y/2,y:H-B/2+b},{x:z+d.width/2,y:H-B/2+b},{x:z+d.width/2/2,y:H+B/2},{x:z-d.width/2,y:H+B/2},{x:z-d.width/2,y:H-B/2+b},{x:z-y/2,y:H-B/2+b}],yt.polygon(t,ee,X)},u}U(Ile,"iconRounded");async function Ole(e,t,{config:{themeVariables:r,flowchart:n}}){const{labelStyles:a}=Nt(t);t.labelStyle=a;const i=t.assetHeight??48,o=t.assetWidth??48,s=Math.max(i,o),l=n==null?void 0:n.wrappingWidth;t.width=Math.max(s,l??0);const{shapeSvg:u,bbox:d,halfPadding:p,label:f}=await Zt(e,t,"icon-shape default"),m=t.pos==="t",b=s+p*2,y=s+p*2,{nodeBorder:S,mainBkg:w}=r,{stylesMap:E}=Rp(t),k=-y/2,_=-b/2,R=t.label?8:0,C=_t.svg(u),F=Dt(t,{});t.look!=="handDrawn"&&(F.roughness=0,F.fillStyle="solid");const L=E.get("fill");F.stroke=L??w;const I=C.path(ju(k,_,y,b,.1),F),M=Math.max(y,d.width),j=b+d.height+R,Y=C.rectangle(-M/2,-j/2,M,j,{...F,fill:"transparent",stroke:"none"}),W=u.insert(()=>I,":first-child"),$=u.insert(()=>Y);if(t.icon){const X=u.append("g");X.html(`<g>${await Kg(t.icon,{height:s,width:s,fallbackPrefix:""})}</g>`);const z=X.node().getBBox(),H=z.width,B=z.height,ee=z.x,J=z.y;X.attr("transform",`translate(${-H/2-ee},${m?d.height/2+R/2-B/2-J:-d.height/2-R/2-B/2-J})`),X.attr("style",`color: ${E.get("stroke")??S};`)}return f.attr("transform",`translate(${-d.width/2-(d.x-(d.left??0))},${m?-j/2:j/2-d.height})`),W.attr("transform",`translate(0,${m?d.height/2+R/2:-d.height/2-R/2})`),Lt(t,$),t.intersect=function(X){if(je.info("iconSquare intersect",t,X),!t.label)return yt.rect(t,X);const z=t.x??0,H=t.y??0,B=t.height??0;let ee=[];return m?ee=[{x:z-d.width/2,y:H-B/2},{x:z+d.width/2,y:H-B/2},{x:z+d.width/2,y:H-B/2+d.height+R},{x:z+y/2,y:H-B/2+d.height+R},{x:z+y/2,y:H+B/2},{x:z-y/2,y:H+B/2},{x:z-y/2,y:H-B/2+d.height+R},{x:z-d.width/2,y:H-B/2+d.height+R}]:ee=[{x:z-y/2,y:H-B/2},{x:z+y/2,y:H-B/2},{x:z+y/2,y:H-B/2+b},{x:z+d.width/2,y:H-B/2+b},{x:z+d.width/2/2,y:H+B/2},{x:z-d.width/2,y:H+B/2},{x:z-d.width/2,y:H-B/2+b},{x:z-y/2,y:H-B/2+b}],yt.polygon(t,ee,X)},u}U(Ole,"iconSquare");async function Ble(e,t,{config:{flowchart:r}}){const n=new Image;n.src=(t==null?void 0:t.img)??"",await n.decode();const a=Number(n.naturalWidth.toString().replace("px","")),i=Number(n.naturalHeight.toString().replace("px",""));t.imageAspectRatio=a/i;const{labelStyles:o}=Nt(t);t.labelStyle=o;const s=r==null?void 0:r.wrappingWidth;t.defaultWidth=r==null?void 0:r.wrappingWidth;const l=Math.max(t.label?s??0:0,(t==null?void 0:t.assetWidth)??a),u=t.constraint==="on"&&t!=null&&t.assetHeight?t.assetHeight*t.imageAspectRatio:l,d=t.constraint==="on"?u/t.imageAspectRatio:(t==null?void 0:t.assetHeight)??i;t.width=Math.max(u,s??0);const{shapeSvg:p,bbox:f,label:m}=await Zt(e,t,"image-shape default"),b=t.pos==="t",y=-u/2,S=-d/2,w=t.label?8:0,E=_t.svg(p),k=Dt(t,{});t.look!=="handDrawn"&&(k.roughness=0,k.fillStyle="solid");const _=E.rectangle(y,S,u,d,k),R=Math.max(u,f.width),C=d+f.height+w,F=E.rectangle(-R/2,-C/2,R,C,{...k,fill:"none",stroke:"none"}),L=p.insert(()=>_,":first-child"),I=p.insert(()=>F);if(t.img){const M=p.append("image");M.attr("href",t.img),M.attr("width",u),M.attr("height",d),M.attr("preserveAspectRatio","none"),M.attr("transform",`translate(${-u/2},${b?C/2-d:-C/2})`)}return m.attr("transform",`translate(${-f.width/2-(f.x-(f.left??0))},${b?-d/2-f.height/2-w/2:d/2-f.height/2+w/2})`),L.attr("transform",`translate(0,${b?f.height/2+w/2:-f.height/2-w/2})`),Lt(t,I),t.intersect=function(M){if(je.info("iconSquare intersect",t,M),!t.label)return yt.rect(t,M);const j=t.x??0,Y=t.y??0,W=t.height??0;let $=[];return b?$=[{x:j-f.width/2,y:Y-W/2},{x:j+f.width/2,y:Y-W/2},{x:j+f.width/2,y:Y-W/2+f.height+w},{x:j+u/2,y:Y-W/2+f.height+w},{x:j+u/2,y:Y+W/2},{x:j-u/2,y:Y+W/2},{x:j-u/2,y:Y-W/2+f.height+w},{x:j-f.width/2,y:Y-W/2+f.height+w}]:$=[{x:j-u/2,y:Y-W/2},{x:j+u/2,y:Y-W/2},{x:j+u/2,y:Y-W/2+d},{x:j+f.width/2,y:Y-W/2+d},{x:j+f.width/2/2,y:Y+W/2},{x:j-f.width/2,y:Y+W/2},{x:j-f.width/2,y:Y-W/2+d},{x:j-u/2,y:Y-W/2+d}],yt.polygon(t,$,M)},p}U(Ble,"imageSquare");async function Ple(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),s=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),l=[{x:0,y:0},{x:o,y:0},{x:o+3*s/6,y:-s},{x:-3*s/6,y:-s}];let u;const{cssStyles:d}=t;if(t.look==="handDrawn"){const p=_t.svg(a),f=Dt(t,{}),m=Ar(l),b=p.path(m,f);u=a.insert(()=>b,":first-child").attr("transform",`translate(${-o/2}, ${s/2})`),d&&u.attr("style",d)}else u=qu(a,o,s,l);return n&&u.attr("style",n),t.width=o,t.height=s,Lt(t,u),t.intersect=function(p){return yt.polygon(t,l,p)},a}U(Ple,"inv_trapezoid");async function X2(e,t,r){const{labelStyles:n,nodeStyles:a}=Nt(t);t.labelStyle=n;const{shapeSvg:i,bbox:o}=await Zt(e,t,jt(t)),s=Math.max(o.width+r.labelPaddingX*2,(t==null?void 0:t.width)||0),l=Math.max(o.height+r.labelPaddingY*2,(t==null?void 0:t.height)||0),u=-s/2,d=-l/2;let p,{rx:f,ry:m}=t;const{cssStyles:b}=t;if(r!=null&&r.rx&&r.ry&&(f=r.rx,m=r.ry),t.look==="handDrawn"){const y=_t.svg(i),S=Dt(t,{}),w=f||m?y.path(ju(u,d,s,l,f||0),S):y.rectangle(u,d,s,l,S);p=i.insert(()=>w,":first-child"),p.attr("class","basic label-container").attr("style",Oa(b))}else p=i.insert("rect",":first-child"),p.attr("class","basic label-container").attr("style",a).attr("rx",Oa(f)).attr("ry",Oa(m)).attr("x",u).attr("y",d).attr("width",s).attr("height",l);return Lt(t,p),t.calcIntersect=function(y,S){return yt.rect(y,S)},t.intersect=function(y){return yt.rect(t,y)},i}U(X2,"drawRect");async function zle(e,t){const{shapeSvg:r,bbox:n,label:a}=await Zt(e,t,"label"),i=r.insert("rect",":first-child");return i.attr("width",.1).attr("height",.1),r.attr("class","label edgeLabel"),a.attr("transform",`translate(${-(n.width/2)-(n.x-(n.left??0))}, ${-(n.height/2)-(n.y-(n.top??0))})`),Lt(t,i),t.intersect=function(l){return yt.rect(t,l)},r}U(zle,"labelRect");async function Hle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=Math.max(i.width+(t.padding??0),(t==null?void 0:t.width)??0),s=Math.max(i.height+(t.padding??0),(t==null?void 0:t.height)??0),l=[{x:0,y:0},{x:o+3*s/6,y:0},{x:o,y:-s},{x:-(3*s)/6,y:-s}];let u;const{cssStyles:d}=t;if(t.look==="handDrawn"){const p=_t.svg(a),f=Dt(t,{}),m=Ar(l),b=p.path(m,f);u=a.insert(()=>b,":first-child").attr("transform",`translate(${-o/2}, ${s/2})`),d&&u.attr("style",d)}else u=qu(a,o,s,l);return n&&u.attr("style",n),t.width=o,t.height=s,Lt(t,u),t.intersect=function(p){return yt.polygon(t,l,p)},a}U(Hle,"lean_left");async function Ule(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=Math.max(i.width+(t.padding??0),(t==null?void 0:t.width)??0),s=Math.max(i.height+(t.padding??0),(t==null?void 0:t.height)??0),l=[{x:-3*s/6,y:0},{x:o,y:0},{x:o+3*s/6,y:-s},{x:0,y:-s}];let u;const{cssStyles:d}=t;if(t.look==="handDrawn"){const p=_t.svg(a),f=Dt(t,{}),m=Ar(l),b=p.path(m,f);u=a.insert(()=>b,":first-child").attr("transform",`translate(${-o/2}, ${s/2})`),d&&u.attr("style",d)}else u=qu(a,o,s,l);return n&&u.attr("style",n),t.width=o,t.height=s,Lt(t,u),t.intersect=function(p){return yt.polygon(t,l,p)},a}U(Ule,"lean_right");function $le(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.label="",t.labelStyle=r;const a=e.insert("g").attr("class",jt(t)).attr("id",t.domId??t.id),{cssStyles:i}=t,o=Math.max(35,(t==null?void 0:t.width)??0),s=Math.max(35,(t==null?void 0:t.height)??0),l=7,u=[{x:o,y:0},{x:0,y:s+l/2},{x:o-2*l,y:s+l/2},{x:0,y:2*s},{x:o,y:s-l/2},{x:2*l,y:s-l/2}],d=_t.svg(a),p=Dt(t,{});t.look!=="handDrawn"&&(p.roughness=0,p.fillStyle="solid");const f=Ar(u),m=d.path(f,p),b=a.insert(()=>m,":first-child");return i&&t.look!=="handDrawn"&&b.selectAll("path").attr("style",i),n&&t.look!=="handDrawn"&&b.selectAll("path").attr("style",n),b.attr("transform",`translate(-${o/2},${-s})`),Lt(t,b),t.intersect=function(y){return je.info("lightningBolt intersect",t,y),yt.polygon(t,u,y)},a}U($le,"lightningBolt");var uJe=U((e,t,r,n,a,i,o)=>[`M${e},${t+i}`,`a${a},${i} 0,0,0 ${r},0`,`a${a},${i} 0,0,0 ${-r},0`,`l0,${n}`,`a${a},${i} 0,0,0 ${r},0`,`l0,${-n}`,`M${e},${t+i+o}`,`a${a},${i} 0,0,0 ${r},0`].join(" "),"createCylinderPathD"),cJe=U((e,t,r,n,a,i,o)=>[`M${e},${t+i}`,`M${e+r},${t+i}`,`a${a},${i} 0,0,0 ${-r},0`,`l0,${n}`,`a${a},${i} 0,0,0 ${r},0`,`l0,${-n}`,`M${e},${t+i+o}`,`a${a},${i} 0,0,0 ${r},0`].join(" "),"createOuterCylinderPathD"),dJe=U((e,t,r,n,a,i)=>[`M${e-r/2},${-n/2}`,`a${a},${i} 0,0,0 ${r},0`].join(" "),"createInnerCylinderPathD");async function Gle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0),t.width??0),l=s/2,u=l/(2.5+s/50),d=Math.max(i.height+u+(t.padding??0),t.height??0),p=d*.1;let f;const{cssStyles:m}=t;if(t.look==="handDrawn"){const b=_t.svg(a),y=cJe(0,0,s,d,l,u,p),S=dJe(0,u,s,d,l,u),w=Dt(t,{}),E=b.path(y,w),k=b.path(S,w);a.insert(()=>k,":first-child").attr("class","line"),f=a.insert(()=>E,":first-child"),f.attr("class","basic label-container"),m&&f.attr("style",m)}else{const b=uJe(0,0,s,d,l,u,p);f=a.insert("path",":first-child").attr("d",b).attr("class","basic label-container").attr("style",Oa(m)).attr("style",n)}return f.attr("label-offset-y",u),f.attr("transform",`translate(${-s/2}, ${-(d/2+u)})`),Lt(t,f),o.attr("transform",`translate(${-(i.width/2)-(i.x-(i.left??0))}, ${-(i.height/2)+u-(i.y-(i.top??0))})`),t.intersect=function(b){const y=yt.rect(t,b),S=y.x-(t.x??0);if(l!=0&&(Math.abs(S)<(t.width??0)/2||Math.abs(S)==(t.width??0)/2&&Math.abs(y.y-(t.y??0))>(t.height??0)/2-u)){let w=u*u*(1-S*S/(l*l));w>0&&(w=Math.sqrt(w)),w=u-w,b.y-(t.y??0)>0&&(w=-w),y.y+=w}return y},a}U(Gle,"linedCylinder");async function jle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),u=l/4,d=l+u,{cssStyles:p}=t,f=_t.svg(a),m=Dt(t,{});t.look!=="handDrawn"&&(m.roughness=0,m.fillStyle="solid");const b=[{x:-s/2-s/2*.1,y:-d/2},{x:-s/2-s/2*.1,y:d/2},...Ou(-s/2-s/2*.1,d/2,s/2+s/2*.1,d/2,u,.8),{x:s/2+s/2*.1,y:-d/2},{x:-s/2-s/2*.1,y:-d/2},{x:-s/2,y:-d/2},{x:-s/2,y:d/2*1.1},{x:-s/2,y:-d/2}],y=f.polygon(b.map(w=>[w.x,w.y]),m),S=a.insert(()=>y,":first-child");return S.attr("class","basic label-container"),p&&t.look!=="handDrawn"&&S.selectAll("path").attr("style",p),n&&t.look!=="handDrawn"&&S.selectAll("path").attr("style",n),S.attr("transform",`translate(0,${-u/2})`),o.attr("transform",`translate(${-s/2+(t.padding??0)+s/2*.1/2-(i.x-(i.left??0))},${-l/2+(t.padding??0)-u/2-(i.y-(i.top??0))})`),Lt(t,S),t.intersect=function(w){return yt.polygon(t,b,w)},a}U(jle,"linedWaveEdgedRect");async function qle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),u=5,d=-s/2,p=-l/2,{cssStyles:f}=t,m=_t.svg(a),b=Dt(t,{}),y=[{x:d-u,y:p+u},{x:d-u,y:p+l+u},{x:d+s-u,y:p+l+u},{x:d+s-u,y:p+l},{x:d+s,y:p+l},{x:d+s,y:p+l-u},{x:d+s+u,y:p+l-u},{x:d+s+u,y:p-u},{x:d+u,y:p-u},{x:d+u,y:p},{x:d,y:p},{x:d,y:p+u}],S=[{x:d,y:p+u},{x:d+s-u,y:p+u},{x:d+s-u,y:p+l},{x:d+s,y:p+l},{x:d+s,y:p},{x:d,y:p}];t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const w=Ar(y),E=m.path(w,b),k=Ar(S),_=m.path(k,{...b,fill:"none"}),R=a.insert(()=>_,":first-child");return R.insert(()=>E,":first-child"),R.attr("class","basic label-container"),f&&t.look!=="handDrawn"&&R.selectAll("path").attr("style",f),n&&t.look!=="handDrawn"&&R.selectAll("path").attr("style",n),o.attr("transform",`translate(${-(i.width/2)-u-(i.x-(i.left??0))}, ${-(i.height/2)+u-(i.y-(i.top??0))})`),Lt(t,R),t.intersect=function(C){return yt.polygon(t,y,C)},a}U(qle,"multiRect");async function Wle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),u=l/4,d=l+u,p=-s/2,f=-d/2,m=5,{cssStyles:b}=t,y=Ou(p-m,f+d+m,p+s-m,f+d+m,u,.8),S=y==null?void 0:y[y.length-1],w=[{x:p-m,y:f+m},{x:p-m,y:f+d+m},...y,{x:p+s-m,y:S.y-m},{x:p+s,y:S.y-m},{x:p+s,y:S.y-2*m},{x:p+s+m,y:S.y-2*m},{x:p+s+m,y:f-m},{x:p+m,y:f-m},{x:p+m,y:f},{x:p,y:f},{x:p,y:f+m}],E=[{x:p,y:f+m},{x:p+s-m,y:f+m},{x:p+s-m,y:S.y-m},{x:p+s,y:S.y-m},{x:p+s,y:f},{x:p,y:f}],k=_t.svg(a),_=Dt(t,{});t.look!=="handDrawn"&&(_.roughness=0,_.fillStyle="solid");const R=Ar(w),C=k.path(R,_),F=Ar(E),L=k.path(F,_),I=a.insert(()=>C,":first-child");return I.insert(()=>L),I.attr("class","basic label-container"),b&&t.look!=="handDrawn"&&I.selectAll("path").attr("style",b),n&&t.look!=="handDrawn"&&I.selectAll("path").attr("style",n),I.attr("transform",`translate(0,${-u/2})`),o.attr("transform",`translate(${-(i.width/2)-m-(i.x-(i.left??0))}, ${-(i.height/2)+m-u/2-(i.y-(i.top??0))})`),Lt(t,I),t.intersect=function(M){return yt.polygon(t,w,M)},a}U(Wle,"multiWaveEdgedRectangle");async function Vle(e,t,{config:{themeVariables:r}}){var E;const{labelStyles:n,nodeStyles:a}=Nt(t);t.labelStyle=n,t.useHtmlLabels||((E=Ma().flowchart)==null?void 0:E.htmlLabels)!==!1||(t.centerLabel=!0);const{shapeSvg:o,bbox:s,label:l}=await Zt(e,t,jt(t)),u=Math.max(s.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),d=Math.max(s.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),p=-u/2,f=-d/2,{cssStyles:m}=t,b=_t.svg(o),y=Dt(t,{fill:r.noteBkgColor,stroke:r.noteBorderColor});t.look!=="handDrawn"&&(y.roughness=0,y.fillStyle="solid");const S=b.rectangle(p,f,u,d,y),w=o.insert(()=>S,":first-child");return w.attr("class","basic label-container"),m&&t.look!=="handDrawn"&&w.selectAll("path").attr("style",m),a&&t.look!=="handDrawn"&&w.selectAll("path").attr("style",a),l.attr("transform",`translate(${-s.width/2-(s.x-(s.left??0))}, ${-(s.height/2)-(s.y-(s.top??0))})`),Lt(t,w),t.intersect=function(k){return yt.rect(t,k)},o}U(Vle,"note");var hJe=U((e,t,r)=>[`M${e+r/2},${t}`,`L${e+r},${t-r/2}`,`L${e+r/2},${t-r}`,`L${e},${t-r/2}`,"Z"].join(" "),"createDecisionBoxPathD");async function Kle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=i.width+t.padding,s=i.height+t.padding,l=o+s,u=.5,d=[{x:l/2,y:0},{x:l,y:-l/2},{x:l/2,y:-l},{x:0,y:-l/2}];let p;const{cssStyles:f}=t;if(t.look==="handDrawn"){const m=_t.svg(a),b=Dt(t,{}),y=hJe(0,0,l),S=m.path(y,b);p=a.insert(()=>S,":first-child").attr("transform",`translate(${-l/2+u}, ${l/2})`),f&&p.attr("style",f)}else p=qu(a,l,l,d),p.attr("transform",`translate(${-l/2+u}, ${l/2})`);return n&&p.attr("style",n),Lt(t,p),t.calcIntersect=function(m,b){const y=m.width,S=[{x:y/2,y:0},{x:y,y:-y/2},{x:y/2,y:-y},{x:0,y:-y/2}],w=yt.polygon(m,S,b);return{x:w.x-.5,y:w.y-.5}},t.intersect=function(m){return this.calcIntersect(t,m)},a}U(Kle,"question");async function Yle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0),(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0),(t==null?void 0:t.height)??0),u=-s/2,d=-l/2,p=d/2,f=[{x:u+p,y:d},{x:u,y:0},{x:u+p,y:-d},{x:-u,y:-d},{x:-u,y:d}],{cssStyles:m}=t,b=_t.svg(a),y=Dt(t,{});t.look!=="handDrawn"&&(y.roughness=0,y.fillStyle="solid");const S=Ar(f),w=b.path(S,y),E=a.insert(()=>w,":first-child");return E.attr("class","basic label-container"),m&&t.look!=="handDrawn"&&E.selectAll("path").attr("style",m),n&&t.look!=="handDrawn"&&E.selectAll("path").attr("style",n),E.attr("transform",`translate(${-p/2},0)`),o.attr("transform",`translate(${-p/2-i.width/2-(i.x-(i.left??0))}, ${-(i.height/2)-(i.y-(i.top??0))})`),Lt(t,E),t.intersect=function(k){return yt.polygon(t,f,k)},a}U(Yle,"rect_left_inv_arrow");async function Jle(e,t){var L,I;const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;let a;t.cssClasses?a="node "+t.cssClasses:a="node default";const i=e.insert("g").attr("class",a).attr("id",t.domId||t.id),o=i.insert("g"),s=i.insert("g").attr("class","label").attr("style",n),l=t.description,u=t.label,d=s.node().appendChild(await Ac(u,t.labelStyle,!0,!0));let p={width:0,height:0};if(ra((I=(L=Gr())==null?void 0:L.flowchart)==null?void 0:I.htmlLabels)){const M=d.children[0],j=Lr(d);p=M.getBoundingClientRect(),j.attr("width",p.width),j.attr("height",p.height)}je.info("Text 2",l);const f=l||[],m=d.getBBox(),b=s.node().appendChild(await Ac(f.join?f.join("<br/>"):f,t.labelStyle,!0,!0)),y=b.children[0],S=Lr(b);p=y.getBoundingClientRect(),S.attr("width",p.width),S.attr("height",p.height);const w=(t.padding||0)/2;Lr(b).attr("transform","translate( "+(p.width>m.width?0:(m.width-p.width)/2)+", "+(m.height+w+5)+")"),Lr(d).attr("transform","translate( "+(p.width<m.width?0:-(m.width-p.width)/2)+", 0)"),p=s.node().getBBox(),s.attr("transform","translate("+-p.width/2+", "+(-p.height/2-w+3)+")");const E=p.width+(t.padding||0),k=p.height+(t.padding||0),_=-p.width/2-w,R=-p.height/2-w;let C,F;if(t.look==="handDrawn"){const M=_t.svg(i),j=Dt(t,{}),Y=M.path(ju(_,R,E,k,t.rx||0),j),W=M.line(-p.width/2-w,-p.height/2-w+m.height+w,p.width/2+w,-p.height/2-w+m.height+w,j);F=i.insert(()=>(je.debug("Rough node insert CXC",Y),W),":first-child"),C=i.insert(()=>(je.debug("Rough node insert CXC",Y),Y),":first-child")}else C=o.insert("rect",":first-child"),F=o.insert("line"),C.attr("class","outer title-state").attr("style",n).attr("x",-p.width/2-w).attr("y",-p.height/2-w).attr("width",p.width+(t.padding||0)).attr("height",p.height+(t.padding||0)),F.attr("class","divider").attr("x1",-p.width/2-w).attr("x2",p.width/2+w).attr("y1",-p.height/2-w+m.height+w).attr("y2",-p.height/2-w+m.height+w);return Lt(t,C),t.intersect=function(M){return yt.rect(t,M)},i}U(Jle,"rectWithTitle");function pm(e,t,r,n,a,i,o){const l=(e+r)/2,u=(t+n)/2,d=Math.atan2(n-t,r-e),p=(r-e)/2,f=(n-t)/2,m=p/a,b=f/i,y=Math.sqrt(m**2+b**2);if(y>1)throw new Error("The given radii are too small to create an arc between the points.");const S=Math.sqrt(1-y**2),w=l+S*i*Math.sin(d)*(o?-1:1),E=u-S*a*Math.cos(d)*(o?-1:1),k=Math.atan2((t-E)/i,(e-w)/a);let R=Math.atan2((n-E)/i,(r-w)/a)-k;o&&R<0&&(R+=2*Math.PI),!o&&R>0&&(R-=2*Math.PI);const C=[];for(let F=0;F<20;F++){const L=F/19,I=k+L*R,M=w+a*Math.cos(I),j=E+i*Math.sin(I);C.push({x:M,y:j})}return C}U(pm,"generateArcPoints");async function Xle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=(t==null?void 0:t.padding)??0,s=(t==null?void 0:t.padding)??0,l=(t!=null&&t.width?t==null?void 0:t.width:i.width)+o*2,u=(t!=null&&t.height?t==null?void 0:t.height:i.height)+s*2,d=t.radius||5,p=t.taper||5,{cssStyles:f}=t,m=_t.svg(a),b=Dt(t,{});t.stroke&&(b.stroke=t.stroke),t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const y=[{x:-l/2+p,y:-u/2},{x:l/2-p,y:-u/2},...pm(l/2-p,-u/2,l/2,-u/2+p,d,d,!0),{x:l/2,y:-u/2+p},{x:l/2,y:u/2-p},...pm(l/2,u/2-p,l/2-p,u/2,d,d,!0),{x:l/2-p,y:u/2},{x:-l/2+p,y:u/2},...pm(-l/2+p,u/2,-l/2,u/2-p,d,d,!0),{x:-l/2,y:u/2-p},{x:-l/2,y:-u/2+p},...pm(-l/2,-u/2+p,-l/2+p,-u/2,d,d,!0)],S=Ar(y),w=m.path(S,b),E=a.insert(()=>w,":first-child");return E.attr("class","basic label-container outer-path"),f&&t.look!=="handDrawn"&&E.selectChildren("path").attr("style",f),n&&t.look!=="handDrawn"&&E.selectChildren("path").attr("style",n),Lt(t,E),t.intersect=function(k){return yt.polygon(t,y,k)},a}U(Xle,"roundedRect");async function Zle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=(t==null?void 0:t.padding)??0,l=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),u=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),d=-i.width/2-s,p=-i.height/2-s,{cssStyles:f}=t,m=_t.svg(a),b=Dt(t,{});t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const y=[{x:d,y:p},{x:d+l+8,y:p},{x:d+l+8,y:p+u},{x:d-8,y:p+u},{x:d-8,y:p},{x:d,y:p},{x:d,y:p+u}],S=m.polygon(y.map(E=>[E.x,E.y]),b),w=a.insert(()=>S,":first-child");return w.attr("class","basic label-container").attr("style",Oa(f)),n&&t.look!=="handDrawn"&&w.selectAll("path").attr("style",n),f&&t.look!=="handDrawn"&&w.selectAll("path").attr("style",n),o.attr("transform",`translate(${-l/2+4+(t.padding??0)-(i.x-(i.left??0))},${-u/2+(t.padding??0)-(i.y-(i.top??0))})`),Lt(t,w),t.intersect=function(E){return yt.rect(t,E)},a}U(Zle,"shadedProcess");async function Qle(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),u=-s/2,d=-l/2,{cssStyles:p}=t,f=_t.svg(a),m=Dt(t,{});t.look!=="handDrawn"&&(m.roughness=0,m.fillStyle="solid");const b=[{x:u,y:d},{x:u,y:d+l},{x:u+s,y:d+l},{x:u+s,y:d-l/2}],y=Ar(b),S=f.path(y,m),w=a.insert(()=>S,":first-child");return w.attr("class","basic label-container"),p&&t.look!=="handDrawn"&&w.selectChildren("path").attr("style",p),n&&t.look!=="handDrawn"&&w.selectChildren("path").attr("style",n),w.attr("transform",`translate(0, ${l/4})`),o.attr("transform",`translate(${-s/2+(t.padding??0)-(i.x-(i.left??0))}, ${-l/4+(t.padding??0)-(i.y-(i.top??0))})`),Lt(t,w),t.intersect=function(E){return yt.polygon(t,b,E)},a}U(Qle,"slopedRect");async function eue(e,t){const r={rx:0,ry:0,labelPaddingX:t.labelPaddingX??((t==null?void 0:t.padding)||0)*2,labelPaddingY:((t==null?void 0:t.padding)||0)*1};return X2(e,t,r)}U(eue,"squareRect");async function tue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=i.height+t.padding,s=i.width+o/4+t.padding,l=o/2,{cssStyles:u}=t,d=_t.svg(a),p=Dt(t,{});t.look!=="handDrawn"&&(p.roughness=0,p.fillStyle="solid");const f=[{x:-s/2+l,y:-o/2},{x:s/2-l,y:-o/2},...ag(-s/2+l,0,l,50,90,270),{x:s/2-l,y:o/2},...ag(s/2-l,0,l,50,270,450)],m=Ar(f),b=d.path(m,p),y=a.insert(()=>b,":first-child");return y.attr("class","basic label-container outer-path"),u&&t.look!=="handDrawn"&&y.selectChildren("path").attr("style",u),n&&t.look!=="handDrawn"&&y.selectChildren("path").attr("style",n),Lt(t,y),t.intersect=function(S){return yt.polygon(t,f,S)},a}U(tue,"stadium");async function rue(e,t){return X2(e,t,{rx:5,ry:5})}U(rue,"state");function nue(e,t,{config:{themeVariables:r}}){const{labelStyles:n,nodeStyles:a}=Nt(t);t.labelStyle=n;const{cssStyles:i}=t,{lineColor:o,stateBorder:s,nodeBorder:l}=r,u=e.insert("g").attr("class","node default").attr("id",t.domId||t.id),d=_t.svg(u),p=Dt(t,{});t.look!=="handDrawn"&&(p.roughness=0,p.fillStyle="solid");const f=d.circle(0,0,14,{...p,stroke:o,strokeWidth:2}),m=s??l,b=d.circle(0,0,5,{...p,fill:m,stroke:m,strokeWidth:2,fillStyle:"solid"}),y=u.insert(()=>f,":first-child");return y.insert(()=>b),i&&y.selectAll("path").attr("style",i),a&&y.selectAll("path").attr("style",a),Lt(t,y),t.intersect=function(S){return yt.circle(t,7,S)},u}U(nue,"stateEnd");function aue(e,t,{config:{themeVariables:r}}){const{lineColor:n}=r,a=e.insert("g").attr("class","node default").attr("id",t.domId||t.id);let i;if(t.look==="handDrawn"){const s=_t.svg(a).circle(0,0,14,Aqe(n));i=a.insert(()=>s),i.attr("class","state-start").attr("r",7).attr("width",14).attr("height",14)}else i=a.insert("circle",":first-child"),i.attr("class","state-start").attr("r",7).attr("width",14).attr("height",14);return Lt(t,i),t.intersect=function(o){return yt.circle(t,7,o)},a}U(aue,"stateStart");async function iue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=((t==null?void 0:t.padding)||0)/2,s=i.width+t.padding,l=i.height+t.padding,u=-i.width/2-o,d=-i.height/2-o,p=[{x:0,y:0},{x:s,y:0},{x:s,y:-l},{x:0,y:-l},{x:0,y:0},{x:-8,y:0},{x:s+8,y:0},{x:s+8,y:-l},{x:-8,y:-l},{x:-8,y:0}];if(t.look==="handDrawn"){const f=_t.svg(a),m=Dt(t,{}),b=f.rectangle(u-8,d,s+16,l,m),y=f.line(u,d,u,d+l,m),S=f.line(u+s,d,u+s,d+l,m);a.insert(()=>y,":first-child"),a.insert(()=>S,":first-child");const w=a.insert(()=>b,":first-child"),{cssStyles:E}=t;w.attr("class","basic label-container").attr("style",Oa(E)),Lt(t,w)}else{const f=qu(a,s,l,p);n&&f.attr("style",n),Lt(t,f)}return t.intersect=function(f){return yt.polygon(t,p,f)},a}U(iue,"subroutine");async function oue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),s=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),l=-o/2,u=-s/2,d=.2*s,p=.2*s,{cssStyles:f}=t,m=_t.svg(a),b=Dt(t,{}),y=[{x:l-d/2,y:u},{x:l+o+d/2,y:u},{x:l+o+d/2,y:u+s},{x:l-d/2,y:u+s}],S=[{x:l+o-d/2,y:u+s},{x:l+o+d/2,y:u+s},{x:l+o+d/2,y:u+s-p}];t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const w=Ar(y),E=m.path(w,b),k=Ar(S),_=m.path(k,{...b,fillStyle:"solid"}),R=a.insert(()=>_,":first-child");return R.insert(()=>E,":first-child"),R.attr("class","basic label-container"),f&&t.look!=="handDrawn"&&R.selectAll("path").attr("style",f),n&&t.look!=="handDrawn"&&R.selectAll("path").attr("style",n),Lt(t,R),t.intersect=function(C){return yt.polygon(t,y,C)},a}U(oue,"taggedRect");async function sue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),u=l/4,d=.2*s,p=.2*l,f=l+u,{cssStyles:m}=t,b=_t.svg(a),y=Dt(t,{});t.look!=="handDrawn"&&(y.roughness=0,y.fillStyle="solid");const S=[{x:-s/2-s/2*.1,y:f/2},...Ou(-s/2-s/2*.1,f/2,s/2+s/2*.1,f/2,u,.8),{x:s/2+s/2*.1,y:-f/2},{x:-s/2-s/2*.1,y:-f/2}],w=-s/2+s/2*.1,E=-f/2-p*.4,k=[{x:w+s-d,y:(E+l)*1.4},{x:w+s,y:E+l-p},{x:w+s,y:(E+l)*.9},...Ou(w+s,(E+l)*1.3,w+s-d,(E+l)*1.5,-l*.03,.5)],_=Ar(S),R=b.path(_,y),C=Ar(k),F=b.path(C,{...y,fillStyle:"solid"}),L=a.insert(()=>F,":first-child");return L.insert(()=>R,":first-child"),L.attr("class","basic label-container"),m&&t.look!=="handDrawn"&&L.selectAll("path").attr("style",m),n&&t.look!=="handDrawn"&&L.selectAll("path").attr("style",n),L.attr("transform",`translate(0,${-u/2})`),o.attr("transform",`translate(${-s/2+(t.padding??0)-(i.x-(i.left??0))},${-l/2+(t.padding??0)-u/2-(i.y-(i.top??0))})`),Lt(t,L),t.intersect=function(I){return yt.polygon(t,S,I)},a}U(sue,"taggedWaveEdgedRectangle");async function lue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=Math.max(i.width+t.padding,(t==null?void 0:t.width)||0),s=Math.max(i.height+t.padding,(t==null?void 0:t.height)||0),l=-o/2,u=-s/2,d=a.insert("rect",":first-child");return d.attr("class","text").attr("style",n).attr("rx",0).attr("ry",0).attr("x",l).attr("y",u).attr("width",o).attr("height",s),Lt(t,d),t.intersect=function(p){return yt.rect(t,p)},a}U(lue,"text");var pJe=U((e,t,r,n,a,i)=>`M${e},${t}
    a${a},${i} 0,0,1 0,${-n}
    l${r},0
    a${a},${i} 0,0,1 0,${n}
    M${r},${-n}
    a${a},${i} 0,0,0 0,${n}
    l${-r},0`,"createCylinderPathD"),fJe=U((e,t,r,n,a,i)=>[`M${e},${t}`,`M${e+r},${t}`,`a${a},${i} 0,0,0 0,${-n}`,`l${-r},0`,`a${a},${i} 0,0,0 0,${n}`,`l${r},0`].join(" "),"createOuterCylinderPathD"),mJe=U((e,t,r,n,a,i)=>[`M${e+r/2},${-n/2}`,`a${a},${i} 0,0,0 0,${n}`].join(" "),"createInnerCylinderPathD");async function uue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o,halfPadding:s}=await Zt(e,t,jt(t)),l=t.look==="neo"?s*2:s,u=i.height+l,d=u/2,p=d/(2.5+u/50),f=i.width+p+l,{cssStyles:m}=t;let b;if(t.look==="handDrawn"){const y=_t.svg(a),S=fJe(0,0,f,u,p,d),w=mJe(0,0,f,u,p,d),E=y.path(S,Dt(t,{})),k=y.path(w,Dt(t,{fill:"none"}));b=a.insert(()=>k,":first-child"),b=a.insert(()=>E,":first-child"),b.attr("class","basic label-container"),m&&b.attr("style",m)}else{const y=pJe(0,0,f,u,p,d);b=a.insert("path",":first-child").attr("d",y).attr("class","basic label-container").attr("style",Oa(m)).attr("style",n),b.attr("class","basic label-container"),m&&b.selectAll("path").attr("style",m),n&&b.selectAll("path").attr("style",n)}return b.attr("label-offset-x",p),b.attr("transform",`translate(${-f/2}, ${u/2} )`),o.attr("transform",`translate(${-(i.width/2)-p-(i.x-(i.left??0))}, ${-(i.height/2)-(i.y-(i.top??0))})`),Lt(t,b),t.intersect=function(y){const S=yt.rect(t,y),w=S.y-(t.y??0);if(d!=0&&(Math.abs(w)<(t.height??0)/2||Math.abs(w)==(t.height??0)/2&&Math.abs(S.x-(t.x??0))>(t.width??0)/2-p)){let E=p*p*(1-w*w/(d*d));E!=0&&(E=Math.sqrt(Math.abs(E))),E=p-E,y.x-(t.x??0)>0&&(E=-E),S.x+=E}return S},a}U(uue,"tiltedCylinder");async function cue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=i.width+t.padding,s=i.height+t.padding,l=[{x:-3*s/6,y:0},{x:o+3*s/6,y:0},{x:o,y:-s},{x:0,y:-s}];let u;const{cssStyles:d}=t;if(t.look==="handDrawn"){const p=_t.svg(a),f=Dt(t,{}),m=Ar(l),b=p.path(m,f);u=a.insert(()=>b,":first-child").attr("transform",`translate(${-o/2}, ${s/2})`),d&&u.attr("style",d)}else u=qu(a,o,s,l);return n&&u.attr("style",n),t.width=o,t.height=s,Lt(t,u),t.intersect=function(p){return yt.polygon(t,l,p)},a}U(cue,"trapezoid");async function due(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=60,s=20,l=Math.max(o,i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),u=Math.max(s,i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),{cssStyles:d}=t,p=_t.svg(a),f=Dt(t,{});t.look!=="handDrawn"&&(f.roughness=0,f.fillStyle="solid");const m=[{x:-l/2*.8,y:-u/2},{x:l/2*.8,y:-u/2},{x:l/2,y:-u/2*.6},{x:l/2,y:u/2},{x:-l/2,y:u/2},{x:-l/2,y:-u/2*.6}],b=Ar(m),y=p.path(b,f),S=a.insert(()=>y,":first-child");return S.attr("class","basic label-container"),d&&t.look!=="handDrawn"&&S.selectChildren("path").attr("style",d),n&&t.look!=="handDrawn"&&S.selectChildren("path").attr("style",n),Lt(t,S),t.intersect=function(w){return yt.polygon(t,m,w)},a}U(due,"trapezoidalPentagon");async function hue(e,t){var E;const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=ra((E=Gr().flowchart)==null?void 0:E.htmlLabels),l=i.width+(t.padding??0),u=l+i.height,d=l+i.height,p=[{x:0,y:0},{x:d,y:0},{x:d/2,y:-u}],{cssStyles:f}=t,m=_t.svg(a),b=Dt(t,{});t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const y=Ar(p),S=m.path(y,b),w=a.insert(()=>S,":first-child").attr("transform",`translate(${-u/2}, ${u/2})`);return f&&t.look!=="handDrawn"&&w.selectChildren("path").attr("style",f),n&&t.look!=="handDrawn"&&w.selectChildren("path").attr("style",n),t.width=l,t.height=u,Lt(t,w),o.attr("transform",`translate(${-i.width/2-(i.x-(i.left??0))}, ${u/2-(i.height+(t.padding??0)/(s?2:1)-(i.y-(i.top??0)))})`),t.intersect=function(k){return je.info("Triangle intersect",t,p,k),yt.polygon(t,p,k)},a}U(hue,"triangle");async function pue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),u=l/8,d=l+u,{cssStyles:p}=t,m=70-s,b=m>0?m/2:0,y=_t.svg(a),S=Dt(t,{});t.look!=="handDrawn"&&(S.roughness=0,S.fillStyle="solid");const w=[{x:-s/2-b,y:d/2},...Ou(-s/2-b,d/2,s/2+b,d/2,u,.8),{x:s/2+b,y:-d/2},{x:-s/2-b,y:-d/2}],E=Ar(w),k=y.path(E,S),_=a.insert(()=>k,":first-child");return _.attr("class","basic label-container"),p&&t.look!=="handDrawn"&&_.selectAll("path").attr("style",p),n&&t.look!=="handDrawn"&&_.selectAll("path").attr("style",n),_.attr("transform",`translate(0,${-u/2})`),o.attr("transform",`translate(${-s/2+(t.padding??0)-(i.x-(i.left??0))},${-l/2+(t.padding??0)-u-(i.y-(i.top??0))})`),Lt(t,_),t.intersect=function(R){return yt.polygon(t,w,R)},a}U(pue,"waveEdgedRectangle");async function fue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i}=await Zt(e,t,jt(t)),o=100,s=50,l=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),u=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),d=l/u;let p=l,f=u;p>f*d?f=p/d:p=f*d,p=Math.max(p,o),f=Math.max(f,s);const m=Math.min(f*.2,f/4),b=f+m*2,{cssStyles:y}=t,S=_t.svg(a),w=Dt(t,{});t.look!=="handDrawn"&&(w.roughness=0,w.fillStyle="solid");const E=[{x:-p/2,y:b/2},...Ou(-p/2,b/2,p/2,b/2,m,1),{x:p/2,y:-b/2},...Ou(p/2,-b/2,-p/2,-b/2,m,-1)],k=Ar(E),_=S.path(k,w),R=a.insert(()=>_,":first-child");return R.attr("class","basic label-container"),y&&t.look!=="handDrawn"&&R.selectAll("path").attr("style",y),n&&t.look!=="handDrawn"&&R.selectAll("path").attr("style",n),Lt(t,R),t.intersect=function(C){return yt.polygon(t,E,C)},a}U(fue,"waveRectangle");async function mue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,label:o}=await Zt(e,t,jt(t)),s=Math.max(i.width+(t.padding??0)*2,(t==null?void 0:t.width)??0),l=Math.max(i.height+(t.padding??0)*2,(t==null?void 0:t.height)??0),u=5,d=-s/2,p=-l/2,{cssStyles:f}=t,m=_t.svg(a),b=Dt(t,{}),y=[{x:d-u,y:p-u},{x:d-u,y:p+l},{x:d+s,y:p+l},{x:d+s,y:p-u}],S=`M${d-u},${p-u} L${d+s},${p-u} L${d+s},${p+l} L${d-u},${p+l} L${d-u},${p-u}
                M${d-u},${p} L${d+s},${p}
                M${d},${p-u} L${d},${p+l}`;t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const w=m.path(S,b),E=a.insert(()=>w,":first-child");return E.attr("transform",`translate(${u/2}, ${u/2})`),E.attr("class","basic label-container"),f&&t.look!=="handDrawn"&&E.selectAll("path").attr("style",f),n&&t.look!=="handDrawn"&&E.selectAll("path").attr("style",n),o.attr("transform",`translate(${-(i.width/2)+u/2-(i.x-(i.left??0))}, ${-(i.height/2)+u/2-(i.y-(i.top??0))})`),Lt(t,E),t.intersect=function(k){return yt.polygon(t,y,k)},a}U(mue,"windowPane");async function iL(e,t){var G,P,ne,oe;const r=t;if(r.alias&&(t.label=r.alias),t.look==="handDrawn"){const{themeVariables:pe}=Ma(),{background:ae}=pe,se={...t,id:t.id+"-background",look:"default",cssStyles:["stroke: none",`fill: ${ae}`]};await iL(e,se)}const n=Ma();t.useHtmlLabels=n.htmlLabels;let a=((G=n.er)==null?void 0:G.diagramPadding)??10,i=((P=n.er)==null?void 0:P.entityPadding)??6;const{cssStyles:o}=t,{labelStyles:s,nodeStyles:l}=Nt(t);if(r.attributes.length===0&&t.label){const pe={rx:0,ry:0,labelPaddingX:a,labelPaddingY:a*1.5};Cl(t.label,n)+pe.labelPaddingX*2<n.er.minEntityWidth&&(t.width=n.er.minEntityWidth);const ae=await X2(e,t,pe);if(!ra(n.htmlLabels)){const se=ae.select("text"),Ee=(ne=se.node())==null?void 0:ne.getBBox();se.attr("transform",`translate(${-Ee.width/2}, 0)`)}return ae}n.htmlLabels||(a*=1.25,i*=1.25);let u=jt(t);u||(u="node default");const d=e.insert("g").attr("class",u).attr("id",t.domId||t.id),p=await fh(d,t.label??"",n,0,0,["name"],s);p.height+=i;let f=0;const m=[],b=[];let y=0,S=0,w=0,E=0,k=!0,_=!0;for(const pe of r.attributes){const ae=await fh(d,pe.type,n,0,f,["attribute-type"],s);y=Math.max(y,ae.width+a);const se=await fh(d,pe.name,n,0,f,["attribute-name"],s);S=Math.max(S,se.width+a);const Ee=await fh(d,pe.keys.join(),n,0,f,["attribute-keys"],s);w=Math.max(w,Ee.width+a);const fe=await fh(d,pe.comment,n,0,f,["attribute-comment"],s);E=Math.max(E,fe.width+a);const Me=Math.max(ae.height,se.height,Ee.height,fe.height)+i;b.push({yOffset:f,rowHeight:Me}),f+=Me}let R=4;w<=a&&(k=!1,w=0,R--),E<=a&&(_=!1,E=0,R--);const C=d.node().getBBox();if(p.width+a*2-(y+S+w+E)>0){const pe=p.width+a*2-(y+S+w+E);y+=pe/R,S+=pe/R,w>0&&(w+=pe/R),E>0&&(E+=pe/R)}const F=y+S+w+E,L=_t.svg(d),I=Dt(t,{});t.look!=="handDrawn"&&(I.roughness=0,I.fillStyle="solid");let M=0;b.length>0&&(M=b.reduce((pe,ae)=>pe+((ae==null?void 0:ae.rowHeight)??0),0));const j=Math.max(C.width+a*2,(t==null?void 0:t.width)||0,F),Y=Math.max((M??0)+p.height,(t==null?void 0:t.height)||0),W=-j/2,$=-Y/2;d.selectAll("g:not(:first-child)").each((pe,ae,se)=>{const Ee=Lr(se[ae]),fe=Ee.attr("transform");let Me=0,Oe=0;if(fe){const Te=RegExp(/translate\(([^,]+),([^)]+)\)/).exec(fe);Te&&(Me=parseFloat(Te[1]),Oe=parseFloat(Te[2]),Ee.attr("class").includes("attribute-name")?Me+=y:Ee.attr("class").includes("attribute-keys")?Me+=y+S:Ee.attr("class").includes("attribute-comment")&&(Me+=y+S+w))}Ee.attr("transform",`translate(${W+a/2+Me}, ${Oe+$+p.height+i/2})`)}),d.select(".name").attr("transform","translate("+-p.width/2+", "+($+i/2)+")");const X=L.rectangle(W,$,j,Y,I),z=d.insert(()=>X,":first-child").attr("style",o.join("")),{themeVariables:H}=Ma(),{rowEven:B,rowOdd:ee,nodeBorder:J}=H;m.push(0);for(const[pe,ae]of b.entries()){const Ee=(pe+1)%2===0&&ae.yOffset!==0,fe=L.rectangle(W,p.height+$+(ae==null?void 0:ae.yOffset),j,ae==null?void 0:ae.rowHeight,{...I,fill:Ee?B:ee,stroke:J});d.insert(()=>fe,"g.label").attr("style",o.join("")).attr("class",`row-rect-${Ee?"even":"odd"}`)}let K=L.line(W,p.height+$,j+W,p.height+$,I);d.insert(()=>K).attr("class","divider"),K=L.line(y+W,p.height+$,y+W,Y+$,I),d.insert(()=>K).attr("class","divider"),k&&(K=L.line(y+S+W,p.height+$,y+S+W,Y+$,I),d.insert(()=>K).attr("class","divider")),_&&(K=L.line(y+S+w+W,p.height+$,y+S+w+W,Y+$,I),d.insert(()=>K).attr("class","divider"));for(const pe of m)K=L.line(W,p.height+$+pe,j+W,p.height+$+pe,I),d.insert(()=>K).attr("class","divider");if(Lt(t,z),l&&t.look!=="handDrawn"){const pe=l.split(";"),ae=(oe=pe==null?void 0:pe.filter(se=>se.includes("stroke")))==null?void 0:oe.map(se=>`${se}`).join("; ");d.selectAll("path").attr("style",ae??""),d.selectAll(".row-rect-even path").attr("style",l)}return t.intersect=function(pe){return yt.rect(t,pe)},d}U(iL,"erBox");async function fh(e,t,r,n=0,a=0,i=[],o=""){const s=e.insert("g").attr("class",`label ${i.join(" ")}`).attr("transform",`translate(${n}, ${a})`).attr("style",o);t!==Gz(t)&&(t=Gz(t),t=t.replaceAll("<","&lt;").replaceAll(">","&gt;"));const l=s.node().appendChild(await Gu(s,t,{width:Cl(t,r)+100,style:o,useHtmlLabels:r.htmlLabels},r));if(t.includes("&lt;")||t.includes("&gt;")){let d=l.children[0];for(d.textContent=d.textContent.replaceAll("&lt;","<").replaceAll("&gt;",">");d.childNodes[0];)d=d.childNodes[0],d.textContent=d.textContent.replaceAll("&lt;","<").replaceAll("&gt;",">")}let u=l.getBBox();if(ra(r.htmlLabels)){const d=l.children[0];d.style.textAlign="start";const p=Lr(l);u=d.getBoundingClientRect(),p.attr("width",u.width),p.attr("height",u.height)}return u}U(fh,"addText");async function gue(e,t,r,n,a=r.class.padding??12){const i=n?0:3,o=e.insert("g").attr("class",jt(t)).attr("id",t.domId||t.id);let s=null,l=null,u=null,d=null,p=0,f=0,m=0;if(s=o.insert("g").attr("class","annotation-group text"),t.annotations.length>0){const E=t.annotations[0];await fm(s,{text:`«${E}»`},0),p=s.node().getBBox().height}l=o.insert("g").attr("class","label-group text"),await fm(l,t,0,["font-weight: bolder"]);const b=l.node().getBBox();f=b.height,u=o.insert("g").attr("class","members-group text");let y=0;for(const E of t.members){const k=await fm(u,E,y,[E.parseClassifier()]);y+=k+i}m=u.node().getBBox().height,m<=0&&(m=a/2),d=o.insert("g").attr("class","methods-group text");let S=0;for(const E of t.methods){const k=await fm(d,E,S,[E.parseClassifier()]);S+=k+i}let w=o.node().getBBox();if(s!==null){const E=s.node().getBBox();s.attr("transform",`translate(${-E.width/2})`)}return l.attr("transform",`translate(${-b.width/2}, ${p})`),w=o.node().getBBox(),u.attr("transform",`translate(0, ${p+f+a*2})`),w=o.node().getBBox(),d.attr("transform",`translate(0, ${p+f+(m?m+a*4:a*2)})`),w=o.node().getBBox(),{shapeSvg:o,bbox:w}}U(gue,"textHelper");async function fm(e,t,r,n=[]){const a=e.insert("g").attr("class","label").attr("style",n.join("; ")),i=Ma();let o="useHtmlLabels"in t?t.useHtmlLabels:ra(i.htmlLabels)??!0,s="";"text"in t?s=t.text:s=t.label,!o&&s.startsWith("\\")&&(s=s.substring(1)),Qh(s)&&(o=!0);const l=await Gu(a,mN(dd(s)),{width:Cl(s,i)+50,classes:"markdown-node-label",useHtmlLabels:o},i);let u,d=1;if(o){const p=l.children[0],f=Lr(l);d=p.innerHTML.split("<br>").length,p.innerHTML.includes("</math>")&&(d+=p.innerHTML.split("<mrow>").length-1);const m=p.getElementsByTagName("img");if(m){const b=s.replace(/<img[^>]*>/g,"").trim()==="";await Promise.all([...m].map(y=>new Promise(S=>{function w(){var E;if(y.style.display="flex",y.style.flexDirection="column",b){const k=((E=i.fontSize)==null?void 0:E.toString())??window.getComputedStyle(document.body).fontSize,R=parseInt(k,10)*5+"px";y.style.minWidth=R,y.style.maxWidth=R}else y.style.width="100%";S(y)}U(w,"setupImage"),setTimeout(()=>{y.complete&&w()}),y.addEventListener("error",w),y.addEventListener("load",w)})))}u=p.getBoundingClientRect(),f.attr("width",u.width),f.attr("height",u.height)}else{n.includes("font-weight: bolder")&&Lr(l).selectAll("tspan").attr("font-weight",""),d=l.children.length;const p=l.children[0];(l.textContent===""||l.textContent.includes("&gt"))&&(p.textContent=s[0]+s.substring(1).replaceAll("&gt;",">").replaceAll("&lt;","<").trim(),s[1]===" "&&(p.textContent=p.textContent[0]+" "+p.textContent.substring(1))),p.textContent==="undefined"&&(p.textContent=""),u=l.getBBox()}return a.attr("transform","translate(0,"+(-u.height/(2*d)+r)+")"),u.height}U(fm,"addText");async function bue(e,t){var I,M;const r=Gr(),n=r.class.padding??12,a=n,i=t.useHtmlLabels??ra(r.htmlLabels)??!0,o=t;o.annotations=o.annotations??[],o.members=o.members??[],o.methods=o.methods??[];const{shapeSvg:s,bbox:l}=await gue(e,t,r,i,a),{labelStyles:u,nodeStyles:d}=Nt(t);t.labelStyle=u,t.cssStyles=o.styles||"";const p=((I=o.styles)==null?void 0:I.join(";"))||d||"";t.cssStyles||(t.cssStyles=p.replaceAll("!important","").split(";"));const f=o.members.length===0&&o.methods.length===0&&!((M=r.class)!=null&&M.hideEmptyMembersBox),m=_t.svg(s),b=Dt(t,{});t.look!=="handDrawn"&&(b.roughness=0,b.fillStyle="solid");const y=l.width;let S=l.height;o.members.length===0&&o.methods.length===0?S+=a:o.members.length>0&&o.methods.length===0&&(S+=a*2);const w=-y/2,E=-S/2,k=m.rectangle(w-n,E-n-(f?n:o.members.length===0&&o.methods.length===0?-n/2:0),y+2*n,S+2*n+(f?n*2:o.members.length===0&&o.methods.length===0?-n:0),b),_=s.insert(()=>k,":first-child");_.attr("class","basic label-container");const R=_.node().getBBox();s.selectAll(".text").each((j,Y,W)=>{var ee;const $=Lr(W[Y]),X=$.attr("transform");let z=0;if(X){const K=RegExp(/translate\(([^,]+),([^)]+)\)/).exec(X);K&&(z=parseFloat(K[2]))}let H=z+E+n-(f?n:o.members.length===0&&o.methods.length===0?-n/2:0);i||(H-=4);let B=w;($.attr("class").includes("label-group")||$.attr("class").includes("annotation-group"))&&(B=-((ee=$.node())==null?void 0:ee.getBBox().width)/2||0,s.selectAll("text").each(function(J,K,G){window.getComputedStyle(G[K]).textAnchor==="middle"&&(B=0)})),$.attr("transform",`translate(${B}, ${H})`)});const C=s.select(".annotation-group").node().getBBox().height-(f?n/2:0)||0,F=s.select(".label-group").node().getBBox().height-(f?n/2:0)||0,L=s.select(".members-group").node().getBBox().height-(f?n/2:0)||0;if(o.members.length>0||o.methods.length>0||f){const j=m.line(R.x,C+F+E+n,R.x+R.width,C+F+E+n,b);s.insert(()=>j).attr("class","divider").attr("style",p)}if(f||o.members.length>0||o.methods.length>0){const j=m.line(R.x,C+F+L+E+a*2+n,R.x+R.width,C+F+L+E+n+a*2,b);s.insert(()=>j).attr("class","divider").attr("style",p)}if(o.look!=="handDrawn"&&s.selectAll("path").attr("style",p),_.select(":nth-child(2)").attr("style",p),s.selectAll(".divider").select("path").attr("style",p),t.labelStyle?s.selectAll("span").attr("style",t.labelStyle):s.selectAll("span").attr("style",p),!i){const j=RegExp(/color\s*:\s*([^;]*)/),Y=j.exec(p);if(Y){const W=Y[0].replace("color","fill");s.selectAll("tspan").attr("style",W)}else if(u){const W=j.exec(u);if(W){const $=W[0].replace("color","fill");s.selectAll("tspan").attr("style",$)}}}return Lt(t,_),t.intersect=function(j){return yt.rect(t,j)},s}U(bue,"classBox");async function yue(e,t){var C,F;const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const a=t,i=t,o=20,s=20,l="verifyMethod"in t,u=jt(t),d=e.insert("g").attr("class",u).attr("id",t.domId??t.id);let p;l?p=await Xo(d,`&lt;&lt;${a.type}&gt;&gt;`,0,t.labelStyle):p=await Xo(d,"&lt;&lt;Element&gt;&gt;",0,t.labelStyle);let f=p;const m=await Xo(d,a.name,f,t.labelStyle+"; font-weight: bold;");if(f+=m+s,l){const L=await Xo(d,`${a.requirementId?`ID: ${a.requirementId}`:""}`,f,t.labelStyle);f+=L;const I=await Xo(d,`${a.text?`Text: ${a.text}`:""}`,f,t.labelStyle);f+=I;const M=await Xo(d,`${a.risk?`Risk: ${a.risk}`:""}`,f,t.labelStyle);f+=M,await Xo(d,`${a.verifyMethod?`Verification: ${a.verifyMethod}`:""}`,f,t.labelStyle)}else{const L=await Xo(d,`${i.type?`Type: ${i.type}`:""}`,f,t.labelStyle);f+=L,await Xo(d,`${i.docRef?`Doc Ref: ${i.docRef}`:""}`,f,t.labelStyle)}const b=(((C=d.node())==null?void 0:C.getBBox().width)??200)+o,y=(((F=d.node())==null?void 0:F.getBBox().height)??200)+o,S=-b/2,w=-y/2,E=_t.svg(d),k=Dt(t,{});t.look!=="handDrawn"&&(k.roughness=0,k.fillStyle="solid");const _=E.rectangle(S,w,b,y,k),R=d.insert(()=>_,":first-child");if(R.attr("class","basic label-container").attr("style",n),d.selectAll(".label").each((L,I,M)=>{const j=Lr(M[I]),Y=j.attr("transform");let W=0,$=0;if(Y){const B=RegExp(/translate\(([^,]+),([^)]+)\)/).exec(Y);B&&(W=parseFloat(B[1]),$=parseFloat(B[2]))}const X=$-y/2;let z=S+o/2;(I===0||I===1)&&(z=W),j.attr("transform",`translate(${z}, ${X+o})`)}),f>p+m+s){const L=E.line(S,w+p+m+s,S+b,w+p+m+s,k);d.insert(()=>L).attr("style",n)}return Lt(t,R),t.intersect=function(L){return yt.rect(t,L)},d}U(yue,"requirementBox");async function Xo(e,t,r,n=""){if(t==="")return 0;const a=e.insert("g").attr("class","label").attr("style",n),i=Gr(),o=i.htmlLabels??!0,s=await Gu(a,mN(dd(t)),{width:Cl(t,i)+50,classes:"markdown-node-label",useHtmlLabels:o,style:n},i);let l;if(o){const u=s.children[0],d=Lr(s);l=u.getBoundingClientRect(),d.attr("width",l.width),d.attr("height",l.height)}else{const u=s.children[0];for(const d of u.children)d.textContent=d.textContent.replaceAll("&gt;",">").replaceAll("&lt;","<"),n&&d.setAttribute("style",n);l=s.getBBox(),l.height+=6}return a.attr("transform",`translate(${-l.width/2},${-l.height/2+r})`),l.height}U(Xo,"addText");var gJe=U(e=>{switch(e){case"Very High":return"red";case"High":return"orange";case"Medium":return null;case"Low":return"blue";case"Very Low":return"lightblue"}},"colorFromPriority");async function vue(e,t,{config:r}){var Y,W;const{labelStyles:n,nodeStyles:a}=Nt(t);t.labelStyle=n||"";const i=10,o=t.width;t.width=(t.width??200)-10;const{shapeSvg:s,bbox:l,label:u}=await Zt(e,t,jt(t)),d=t.padding||10;let p="",f;"ticket"in t&&t.ticket&&((Y=r==null?void 0:r.kanban)!=null&&Y.ticketBaseUrl)&&(p=(W=r==null?void 0:r.kanban)==null?void 0:W.ticketBaseUrl.replace("#TICKET#",t.ticket),f=s.insert("svg:a",":first-child").attr("class","kanban-ticket-link").attr("xlink:href",p).attr("target","_blank"));const m={useHtmlLabels:t.useHtmlLabels,labelStyle:t.labelStyle||"",width:t.width,img:t.img,padding:t.padding||8,centerLabel:!1};let b,y;f?{label:b,bbox:y}=await $k(f,"ticket"in t&&t.ticket||"",m):{label:b,bbox:y}=await $k(s,"ticket"in t&&t.ticket||"",m);const{label:S,bbox:w}=await $k(s,"assigned"in t&&t.assigned||"",m);t.width=o;const E=10,k=(t==null?void 0:t.width)||0,_=Math.max(y.height,w.height)/2,R=Math.max(l.height+E*2,(t==null?void 0:t.height)||0)+_,C=-k/2,F=-R/2;u.attr("transform","translate("+(d-k/2)+", "+(-_-l.height/2)+")"),b.attr("transform","translate("+(d-k/2)+", "+(-_+l.height/2)+")"),S.attr("transform","translate("+(d+k/2-w.width-2*i)+", "+(-_+l.height/2)+")");let L;const{rx:I,ry:M}=t,{cssStyles:j}=t;if(t.look==="handDrawn"){const $=_t.svg(s),X=Dt(t,{}),z=I||M?$.path(ju(C,F,k,R,I||0),X):$.rectangle(C,F,k,R,X);L=s.insert(()=>z,":first-child"),L.attr("class","basic label-container").attr("style",j||null)}else{L=s.insert("rect",":first-child"),L.attr("class","basic label-container __APA__").attr("style",a).attr("rx",I??5).attr("ry",M??5).attr("x",C).attr("y",F).attr("width",k).attr("height",R);const $="priority"in t&&t.priority;if($){const X=s.append("line"),z=C+2,H=F+Math.floor((I??0)/2),B=F+R-Math.floor((I??0)/2);X.attr("x1",z).attr("y1",H).attr("x2",z).attr("y2",B).attr("stroke-width","4").attr("stroke",gJe($))}}return Lt(t,L),t.height=R,t.intersect=function($){return yt.rect(t,$)},s}U(vue,"kanbanItem");async function Sue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,halfPadding:o,label:s}=await Zt(e,t,jt(t)),l=i.width+10*o,u=i.height+8*o,d=.15*l,{cssStyles:p}=t,f=i.width+20,m=i.height+20,b=Math.max(l,f),y=Math.max(u,m);s.attr("transform",`translate(${-i.width/2}, ${-i.height/2})`);let S;const w=`M0 0 
    a${d},${d} 1 0,0 ${b*.25},${-1*y*.1}
    a${d},${d} 1 0,0 ${b*.25},0
    a${d},${d} 1 0,0 ${b*.25},0
    a${d},${d} 1 0,0 ${b*.25},${y*.1}

    a${d},${d} 1 0,0 ${b*.15},${y*.33}
    a${d*.8},${d*.8} 1 0,0 0,${y*.34}
    a${d},${d} 1 0,0 ${-1*b*.15},${y*.33}

    a${d},${d} 1 0,0 ${-1*b*.25},${y*.15}
    a${d},${d} 1 0,0 ${-1*b*.25},0
    a${d},${d} 1 0,0 ${-1*b*.25},0
    a${d},${d} 1 0,0 ${-1*b*.25},${-1*y*.15}

    a${d},${d} 1 0,0 ${-1*b*.1},${-1*y*.33}
    a${d*.8},${d*.8} 1 0,0 0,${-1*y*.34}
    a${d},${d} 1 0,0 ${b*.1},${-1*y*.33}
  H0 V0 Z`;if(t.look==="handDrawn"){const E=_t.svg(a),k=Dt(t,{}),_=E.path(w,k);S=a.insert(()=>_,":first-child"),S.attr("class","basic label-container").attr("style",Oa(p))}else S=a.insert("path",":first-child").attr("class","basic label-container").attr("style",n).attr("d",w);return S.attr("transform",`translate(${-b/2}, ${-y/2})`),Lt(t,S),t.calcIntersect=function(E,k){return yt.rect(E,k)},t.intersect=function(E){return je.info("Bang intersect",t,E),yt.rect(t,E)},a}U(Sue,"bang");async function Eue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,halfPadding:o,label:s}=await Zt(e,t,jt(t)),l=i.width+2*o,u=i.height+2*o,d=.15*l,p=.25*l,f=.35*l,m=.2*l,{cssStyles:b}=t;let y;const S=`M0 0 
    a${d},${d} 0 0,1 ${l*.25},${-1*l*.1}
    a${f},${f} 1 0,1 ${l*.4},${-1*l*.1}
    a${p},${p} 1 0,1 ${l*.35},${l*.2}

    a${d},${d} 1 0,1 ${l*.15},${u*.35}
    a${m},${m} 1 0,1 ${-1*l*.15},${u*.65}

    a${p},${d} 1 0,1 ${-1*l*.25},${l*.15}
    a${f},${f} 1 0,1 ${-1*l*.5},0
    a${d},${d} 1 0,1 ${-1*l*.25},${-1*l*.15}

    a${d},${d} 1 0,1 ${-1*l*.1},${-1*u*.35}
    a${m},${m} 1 0,1 ${l*.1},${-1*u*.65}
  H0 V0 Z`;if(t.look==="handDrawn"){const w=_t.svg(a),E=Dt(t,{}),k=w.path(S,E);y=a.insert(()=>k,":first-child"),y.attr("class","basic label-container").attr("style",Oa(b))}else y=a.insert("path",":first-child").attr("class","basic label-container").attr("style",n).attr("d",S);return s.attr("transform",`translate(${-i.width/2}, ${-i.height/2})`),y.attr("transform",`translate(${-l/2}, ${-u/2})`),Lt(t,y),t.calcIntersect=function(w,E){return yt.rect(w,E)},t.intersect=function(w){return je.info("Cloud intersect",t,w),yt.rect(t,w)},a}U(Eue,"cloud");async function xue(e,t){const{labelStyles:r,nodeStyles:n}=Nt(t);t.labelStyle=r;const{shapeSvg:a,bbox:i,halfPadding:o,label:s}=await Zt(e,t,jt(t)),l=i.width+8*o,u=i.height+2*o,d=5,p=`
    M${-l/2} ${u/2-d}
    v${-u+2*d}
    q0,-${d} ${d},-${d}
    h${l-2*d}
    q${d},0 ${d},${d}
    v${u-2*d}
    q0,${d} -${d},${d}
    h${-l+2*d}
    q-${d},0 -${d},-${d}
    Z
  `,f=a.append("path").attr("id","node-"+t.id).attr("class","node-bkg node-"+t.type).attr("style",n).attr("d",p);return a.append("line").attr("class","node-line-").attr("x1",-l/2).attr("y1",u/2).attr("x2",l/2).attr("y2",u/2),s.attr("transform",`translate(${-i.width/2}, ${-i.height/2})`),a.append(()=>s.node()),Lt(t,f),t.calcIntersect=function(m,b){return yt.rect(m,b)},t.intersect=function(m){return yt.rect(t,m)},a}U(xue,"defaultMindmapNode");async function wue(e,t){const r={padding:t.padding??0};return aL(e,t,r)}U(wue,"mindmapCircle");var bJe=[{semanticName:"Process",name:"Rectangle",shortName:"rect",description:"Standard process shape",aliases:["proc","process","rectangle"],internalAliases:["squareRect"],handler:eue},{semanticName:"Event",name:"Rounded Rectangle",shortName:"rounded",description:"Represents an event",aliases:["event"],internalAliases:["roundedRect"],handler:Xle},{semanticName:"Terminal Point",name:"Stadium",shortName:"stadium",description:"Terminal point",aliases:["terminal","pill"],handler:tue},{semanticName:"Subprocess",name:"Framed Rectangle",shortName:"fr-rect",description:"Subprocess",aliases:["subprocess","subproc","framed-rectangle","subroutine"],handler:iue},{semanticName:"Database",name:"Cylinder",shortName:"cyl",description:"Database storage",aliases:["db","database","cylinder"],handler:kle},{semanticName:"Start",name:"Circle",shortName:"circle",description:"Starting point",aliases:["circ"],handler:aL},{semanticName:"Bang",name:"Bang",shortName:"bang",description:"Bang",aliases:["bang"],handler:Sue},{semanticName:"Cloud",name:"Cloud",shortName:"cloud",description:"cloud",aliases:["cloud"],handler:Eue},{semanticName:"Decision",name:"Diamond",shortName:"diam",description:"Decision-making step",aliases:["decision","diamond","question"],handler:Kle},{semanticName:"Prepare Conditional",name:"Hexagon",shortName:"hex",description:"Preparation or condition step",aliases:["hexagon","prepare"],handler:Dle},{semanticName:"Data Input/Output",name:"Lean Right",shortName:"lean-r",description:"Represents input or output",aliases:["lean-right","in-out"],internalAliases:["lean_right"],handler:Ule},{semanticName:"Data Input/Output",name:"Lean Left",shortName:"lean-l",description:"Represents output or input",aliases:["lean-left","out-in"],internalAliases:["lean_left"],handler:Hle},{semanticName:"Priority Action",name:"Trapezoid Base Bottom",shortName:"trap-b",description:"Priority action",aliases:["priority","trapezoid-bottom","trapezoid"],handler:cue},{semanticName:"Manual Operation",name:"Trapezoid Base Top",shortName:"trap-t",description:"Represents a manual task",aliases:["manual","trapezoid-top","inv-trapezoid"],internalAliases:["inv_trapezoid"],handler:Ple},{semanticName:"Stop",name:"Double Circle",shortName:"dbl-circ",description:"Represents a stop point",aliases:["double-circle"],internalAliases:["doublecircle"],handler:Tle},{semanticName:"Text Block",name:"Text Block",shortName:"text",description:"Text block",handler:lue},{semanticName:"Card",name:"Notched Rectangle",shortName:"notch-rect",description:"Represents a card",aliases:["card","notched-rectangle"],handler:gle},{semanticName:"Lined/Shaded Process",name:"Lined Rectangle",shortName:"lin-rect",description:"Lined process shape",aliases:["lined-rectangle","lined-process","lin-proc","shaded-process"],handler:Zle},{semanticName:"Start",name:"Small Circle",shortName:"sm-circ",description:"Small starting point",aliases:["start","small-circle"],internalAliases:["stateStart"],handler:aue},{semanticName:"Stop",name:"Framed Circle",shortName:"fr-circ",description:"Stop point",aliases:["stop","framed-circle"],internalAliases:["stateEnd"],handler:nue},{semanticName:"Fork/Join",name:"Filled Rectangle",shortName:"fork",description:"Fork or join in process flow",aliases:["join"],internalAliases:["forkJoin"],handler:_le},{semanticName:"Collate",name:"Hourglass",shortName:"hourglass",description:"Represents a collate operation",aliases:["hourglass","collate"],handler:Nle},{semanticName:"Comment",name:"Curly Brace",shortName:"brace",description:"Adds a comment",aliases:["comment","brace-l"],handler:Sle},{semanticName:"Comment Right",name:"Curly Brace",shortName:"brace-r",description:"Adds a comment",handler:Ele},{semanticName:"Comment with braces on both sides",name:"Curly Braces",shortName:"braces",description:"Adds a comment",handler:xle},{semanticName:"Com Link",name:"Lightning Bolt",shortName:"bolt",description:"Communication link",aliases:["com-link","lightning-bolt"],handler:$le},{semanticName:"Document",name:"Document",shortName:"doc",description:"Represents a document",aliases:["doc","document"],handler:pue},{semanticName:"Delay",name:"Half-Rounded Rectangle",shortName:"delay",description:"Represents a delay",aliases:["half-rounded-rectangle"],handler:Rle},{semanticName:"Direct Access Storage",name:"Horizontal Cylinder",shortName:"h-cyl",description:"Direct access storage",aliases:["das","horizontal-cylinder"],handler:uue},{semanticName:"Disk Storage",name:"Lined Cylinder",shortName:"lin-cyl",description:"Disk storage",aliases:["disk","lined-cylinder"],handler:Gle},{semanticName:"Display",name:"Curved Trapezoid",shortName:"curv-trap",description:"Represents a display",aliases:["curved-trapezoid","display"],handler:wle},{semanticName:"Divided Process",name:"Divided Rectangle",shortName:"div-rect",description:"Divided process shape",aliases:["div-proc","divided-rectangle","divided-process"],handler:Cle},{semanticName:"Extract",name:"Triangle",shortName:"tri",description:"Extraction process",aliases:["extract","triangle"],handler:hue},{semanticName:"Internal Storage",name:"Window Pane",shortName:"win-pane",description:"Internal storage",aliases:["internal-storage","window-pane"],handler:mue},{semanticName:"Junction",name:"Filled Circle",shortName:"f-circ",description:"Junction point",aliases:["junction","filled-circle"],handler:Ale},{semanticName:"Loop Limit",name:"Trapezoidal Pentagon",shortName:"notch-pent",description:"Loop limit step",aliases:["loop-limit","notched-pentagon"],handler:due},{semanticName:"Manual File",name:"Flipped Triangle",shortName:"flip-tri",description:"Manual file operation",aliases:["manual-file","flipped-triangle"],handler:Fle},{semanticName:"Manual Input",name:"Sloped Rectangle",shortName:"sl-rect",description:"Manual input step",aliases:["manual-input","sloped-rectangle"],handler:Qle},{semanticName:"Multi-Document",name:"Stacked Document",shortName:"docs",description:"Multiple documents",aliases:["documents","st-doc","stacked-document"],handler:Wle},{semanticName:"Multi-Process",name:"Stacked Rectangle",shortName:"st-rect",description:"Multiple processes",aliases:["procs","processes","stacked-rectangle"],handler:qle},{semanticName:"Stored Data",name:"Bow Tie Rectangle",shortName:"bow-rect",description:"Stored data",aliases:["stored-data","bow-tie-rectangle"],handler:mle},{semanticName:"Summary",name:"Crossed Circle",shortName:"cross-circ",description:"Summary",aliases:["summary","crossed-circle"],handler:vle},{semanticName:"Tagged Document",name:"Tagged Document",shortName:"tag-doc",description:"Tagged document",aliases:["tag-doc","tagged-document"],handler:sue},{semanticName:"Tagged Process",name:"Tagged Rectangle",shortName:"tag-rect",description:"Tagged process",aliases:["tagged-rectangle","tag-proc","tagged-process"],handler:oue},{semanticName:"Paper Tape",name:"Flag",shortName:"flag",description:"Paper tape",aliases:["paper-tape"],handler:fue},{semanticName:"Odd",name:"Odd",shortName:"odd",description:"Odd shape",internalAliases:["rect_left_inv_arrow"],handler:Yle},{semanticName:"Lined Document",name:"Lined Document",shortName:"lin-doc",description:"Lined document",aliases:["lined-document"],handler:jle}],yJe=U(()=>{const t=[...Object.entries({state:rue,choice:ble,note:Vle,rectWithTitle:Jle,labelRect:zle,iconSquare:Ole,iconCircle:Mle,icon:Lle,iconRounded:Ile,imageSquare:Ble,anchor:fle,kanbanItem:vue,mindmapCircle:wue,defaultMindmapNode:xue,classBox:bue,erBox:iL,requirementBox:yue}),...bJe.flatMap(r=>[r.shortName,..."aliases"in r?r.aliases:[],..."internalAliases"in r?r.internalAliases:[]].map(a=>[a,r.handler]))];return Object.fromEntries(t)},"generateShapeMap"),kue=yJe();function vJe(e){return e in kue}U(vJe,"isValidShape");var Z2=new Map;async function Cue(e,t,r){let n,a;t.shape==="rect"&&(t.rx&&t.ry?t.shape="roundedRect":t.shape="squareRect");const i=t.shape?kue[t.shape]:void 0;if(!i)throw new Error(`No such shape: ${t.shape}. Please check your syntax.`);if(t.link){let o;r.config.securityLevel==="sandbox"?o="_top":t.linkTarget&&(o=t.linkTarget||"_blank"),n=e.insert("svg:a").attr("xlink:href",t.link).attr("target",o??null),a=await i(n,t,r)}else a=await i(e,t,r),n=a;return t.tooltip&&a.attr("title",t.tooltip),Z2.set(t.id,n),t.haveCallback&&n.attr("class",n.attr("class")+" clickable"),n}U(Cue,"insertNode");var qdt=U((e,t)=>{Z2.set(t.id,e)},"setNodeElem"),Wdt=U(()=>{Z2.clear()},"clear"),Vdt=U(e=>{const t=Z2.get(e.id);je.trace("Transforming node",e.diff,e,"translate("+(e.x-e.width/2-5)+", "+e.width/2+")");const r=8,n=e.diff||0;return e.clusterNode?t.attr("transform","translate("+(e.x+n-e.width/2)+", "+(e.y-e.height/2-r)+")"):t.attr("transform","translate("+e.x+", "+e.y+")"),n},"positionNode"),SJe=U((e,t,r,n,a,i)=>{t.arrowTypeStart&&XH(e,"start",t.arrowTypeStart,r,n,a,i),t.arrowTypeEnd&&XH(e,"end",t.arrowTypeEnd,r,n,a,i)},"addEdgeMarkers"),EJe={arrow_cross:{type:"cross",fill:!1},arrow_point:{type:"point",fill:!0},arrow_barb:{type:"barb",fill:!0},arrow_circle:{type:"circle",fill:!1},aggregation:{type:"aggregation",fill:!1},extension:{type:"extension",fill:!1},composition:{type:"composition",fill:!0},dependency:{type:"dependency",fill:!0},lollipop:{type:"lollipop",fill:!1},only_one:{type:"onlyOne",fill:!1},zero_or_one:{type:"zeroOrOne",fill:!1},one_or_more:{type:"oneOrMore",fill:!1},zero_or_more:{type:"zeroOrMore",fill:!1},requirement_arrow:{type:"requirement_arrow",fill:!1},requirement_contains:{type:"requirement_contains",fill:!1}},XH=U((e,t,r,n,a,i,o)=>{var p;const s=EJe[r];if(!s){je.warn(`Unknown arrow type: ${r}`);return}const l=s.type,d=`${a}_${i}-${l}${t==="start"?"Start":"End"}`;if(o&&o.trim()!==""){const f=o.replace(/[^\dA-Za-z]/g,"_"),m=`${d}_${f}`;if(!document.getElementById(m)){const b=document.getElementById(d);if(b){const y=b.cloneNode(!0);y.id=m,y.querySelectorAll("path, circle, line").forEach(w=>{w.setAttribute("stroke",o),s.fill&&w.setAttribute("fill",o)}),(p=b.parentNode)==null||p.appendChild(y)}}e.attr(`marker-${t}`,`url(${n}#${m})`)}else e.attr(`marker-${t}`,`url(${n}#${d})`)},"addEdgeMarker"),mv=new Map,ba=new Map,Kdt=U(()=>{mv.clear(),ba.clear()},"clear"),c1=U(e=>e?e.reduce((r,n)=>r+";"+n,""):"","getLabelStyles"),xJe=U(async(e,t)=>{let r=ra(Gr().flowchart.htmlLabels);const{labelStyles:n}=Nt(t);t.labelStyle=n;const a=await Gu(e,t.label,{style:t.labelStyle,useHtmlLabels:r,addSvgBackground:!0,isNode:!1});je.info("abc82",t,t.labelType);const i=e.insert("g").attr("class","edgeLabel"),o=i.insert("g").attr("class","label").attr("data-id",t.id);o.node().appendChild(a);let s=a.getBBox();if(r){const u=a.children[0],d=Lr(a);s=u.getBoundingClientRect(),d.attr("width",s.width),d.attr("height",s.height)}o.attr("transform","translate("+-s.width/2+", "+-s.height/2+")"),mv.set(t.id,i),t.width=s.width,t.height=s.height;let l;if(t.startLabelLeft){const u=await Ac(t.startLabelLeft,c1(t.labelStyle)),d=e.insert("g").attr("class","edgeTerminals"),p=d.insert("g").attr("class","inner");l=p.node().appendChild(u);const f=u.getBBox();p.attr("transform","translate("+-f.width/2+", "+-f.height/2+")"),ba.get(t.id)||ba.set(t.id,{}),ba.get(t.id).startLeft=d,mm(l,t.startLabelLeft)}if(t.startLabelRight){const u=await Ac(t.startLabelRight,c1(t.labelStyle)),d=e.insert("g").attr("class","edgeTerminals"),p=d.insert("g").attr("class","inner");l=d.node().appendChild(u),p.node().appendChild(u);const f=u.getBBox();p.attr("transform","translate("+-f.width/2+", "+-f.height/2+")"),ba.get(t.id)||ba.set(t.id,{}),ba.get(t.id).startRight=d,mm(l,t.startLabelRight)}if(t.endLabelLeft){const u=await Ac(t.endLabelLeft,c1(t.labelStyle)),d=e.insert("g").attr("class","edgeTerminals"),p=d.insert("g").attr("class","inner");l=p.node().appendChild(u);const f=u.getBBox();p.attr("transform","translate("+-f.width/2+", "+-f.height/2+")"),d.node().appendChild(u),ba.get(t.id)||ba.set(t.id,{}),ba.get(t.id).endLeft=d,mm(l,t.endLabelLeft)}if(t.endLabelRight){const u=await Ac(t.endLabelRight,c1(t.labelStyle)),d=e.insert("g").attr("class","edgeTerminals"),p=d.insert("g").attr("class","inner");l=p.node().appendChild(u);const f=u.getBBox();p.attr("transform","translate("+-f.width/2+", "+-f.height/2+")"),d.node().appendChild(u),ba.get(t.id)||ba.set(t.id,{}),ba.get(t.id).endRight=d,mm(l,t.endLabelRight)}return a},"insertEdgeLabel");function mm(e,t){Gr().flowchart.htmlLabels&&e&&(e.style.width=t.length*9+"px",e.style.height="12px")}U(mm,"setTerminalWidth");var wJe=U((e,t)=>{je.debug("Moving label abc88 ",e.id,e.label,mv.get(e.id),t);let r=t.updatedPath?t.updatedPath:t.originalPath;const n=Gr(),{subGraphTitleTotalMargin:a}=IN(n);if(e.label){const i=mv.get(e.id);let o=e.x,s=e.y;if(r){const l=_o.calcLabelPosition(r);je.debug("Moving label "+e.label+" from (",o,",",s,") to (",l.x,",",l.y,") abc88"),t.updatedPath&&(o=l.x,s=l.y)}i.attr("transform",`translate(${o}, ${s+a/2})`)}if(e.startLabelLeft){const i=ba.get(e.id).startLeft;let o=e.x,s=e.y;if(r){const l=_o.calcTerminalLabelPosition(e.arrowTypeStart?10:0,"start_left",r);o=l.x,s=l.y}i.attr("transform",`translate(${o}, ${s})`)}if(e.startLabelRight){const i=ba.get(e.id).startRight;let o=e.x,s=e.y;if(r){const l=_o.calcTerminalLabelPosition(e.arrowTypeStart?10:0,"start_right",r);o=l.x,s=l.y}i.attr("transform",`translate(${o}, ${s})`)}if(e.endLabelLeft){const i=ba.get(e.id).endLeft;let o=e.x,s=e.y;if(r){const l=_o.calcTerminalLabelPosition(e.arrowTypeEnd?10:0,"end_left",r);o=l.x,s=l.y}i.attr("transform",`translate(${o}, ${s})`)}if(e.endLabelRight){const i=ba.get(e.id).endRight;let o=e.x,s=e.y;if(r){const l=_o.calcTerminalLabelPosition(e.arrowTypeEnd?10:0,"end_right",r);o=l.x,s=l.y}i.attr("transform",`translate(${o}, ${s})`)}},"positionEdgeLabel"),kJe=U((e,t)=>{const r=e.x,n=e.y,a=Math.abs(t.x-r),i=Math.abs(t.y-n),o=e.width/2,s=e.height/2;return a>=o||i>=s},"outsideNode"),CJe=U((e,t,r)=>{je.debug(`intersection calc abc89:
  outsidePoint: ${JSON.stringify(t)}
  insidePoint : ${JSON.stringify(r)}
  node        : x:${e.x} y:${e.y} w:${e.width} h:${e.height}`);const n=e.x,a=e.y,i=Math.abs(n-r.x),o=e.width/2;let s=r.x<t.x?o-i:o+i;const l=e.height/2,u=Math.abs(t.y-r.y),d=Math.abs(t.x-r.x);if(Math.abs(a-t.y)*o>Math.abs(n-t.x)*l){let p=r.y<t.y?t.y-l-a:a-l-t.y;s=d*p/u;const f={x:r.x<t.x?r.x+s:r.x-d+s,y:r.y<t.y?r.y+u-p:r.y-u+p};return s===0&&(f.x=t.x,f.y=t.y),d===0&&(f.x=t.x),u===0&&(f.y=t.y),je.debug(`abc89 top/bottom calc, Q ${u}, q ${p}, R ${d}, r ${s}`,f),f}else{r.x<t.x?s=t.x-o-n:s=n-o-t.x;let p=u*s/d,f=r.x<t.x?r.x+d-s:r.x-d+s,m=r.y<t.y?r.y+p:r.y-p;return je.debug(`sides calc abc89, Q ${u}, q ${p}, R ${d}, r ${s}`,{_x:f,_y:m}),s===0&&(f=t.x,m=t.y),d===0&&(f=t.x),u===0&&(m=t.y),{x:f,y:m}}},"intersection"),ZH=U((e,t)=>{je.warn("abc88 cutPathAtIntersect",e,t);let r=[],n=e[0],a=!1;return e.forEach(i=>{if(je.info("abc88 checking point",i,t),!kJe(t,i)&&!a){const o=CJe(t,n,i);je.debug("abc88 inside",i,n,o),je.debug("abc88 intersection",o,t);let s=!1;r.forEach(l=>{s=s||l.x===o.x&&l.y===o.y}),r.some(l=>l.x===o.x&&l.y===o.y)?je.warn("abc88 no intersect",o,r):r.push(o),a=!0}else je.warn("abc88 outside",i,n),n=i,a||r.push(i)}),je.debug("returning points",r),r},"cutPathAtIntersect");function Tue(e){const t=[],r=[];for(let n=1;n<e.length-1;n++){const a=e[n-1],i=e[n],o=e[n+1];(a.x===i.x&&i.y===o.y&&Math.abs(i.x-o.x)>5&&Math.abs(i.y-a.y)>5||a.y===i.y&&i.x===o.x&&Math.abs(i.x-a.x)>5&&Math.abs(i.y-o.y)>5)&&(t.push(i),r.push(n))}return{cornerPoints:t,cornerPointPositions:r}}U(Tue,"extractCornerPoints");var QH=U(function(e,t,r){const n=t.x-e.x,a=t.y-e.y,i=Math.sqrt(n*n+a*a),o=r/i;return{x:t.x-o*n,y:t.y-o*a}},"findAdjacentPoint"),TJe=U(function(e){const{cornerPointPositions:t}=Tue(e),r=[];for(let n=0;n<e.length;n++)if(t.includes(n)){const a=e[n-1],i=e[n+1],o=e[n],s=QH(a,o,5),l=QH(i,o,5),u=l.x-s.x,d=l.y-s.y;r.push(s);const p=Math.sqrt(2)*2;let f={x:o.x,y:o.y};if(Math.abs(i.x-a.x)>10&&Math.abs(i.y-a.y)>=10){je.debug("Corner point fixing",Math.abs(i.x-a.x),Math.abs(i.y-a.y));const m=5;o.x===s.x?f={x:u<0?s.x-m+p:s.x+m-p,y:d<0?s.y-p:s.y+p}:f={x:u<0?s.x-p:s.x+p,y:d<0?s.y-m+p:s.y+m-p}}else je.debug("Corner point skipping fixing",Math.abs(i.x-a.x),Math.abs(i.y-a.y));r.push(f,l)}else r.push(e[n]);return r},"fixCorners"),AJe=U((e,t,r)=>{const n=e-t-r,a=2,i=2,o=a+i,s=Math.floor(n/o),l=Array(s).fill(`${a} ${i}`).join(" ");return`0 ${t} ${l} ${r}`},"generateDashArray"),FJe=U(function(e,t,r,n,a,i,o,s=!1){var $;const{handDrawnSeed:l}=Gr();let u=t.points,d=!1;const p=a;var f=i;const m=[];for(const X in t.cssCompiledStyles)nse(X)||m.push(t.cssCompiledStyles[X]);je.debug("UIO intersect check",t.points,f.x,p.x),f.intersect&&p.intersect&&!s&&(u=u.slice(1,t.points.length-1),u.unshift(p.intersect(u[0])),je.debug("Last point UIO",t.start,"-->",t.end,u[u.length-1],f,f.intersect(u[u.length-1])),u.push(f.intersect(u[u.length-1])));const b=btoa(JSON.stringify(u));t.toCluster&&(je.info("to cluster abc88",r.get(t.toCluster)),u=ZH(t.points,r.get(t.toCluster).node),d=!0),t.fromCluster&&(je.debug("from cluster abc88",r.get(t.fromCluster),JSON.stringify(u,null,2)),u=ZH(u.reverse(),r.get(t.fromCluster).node).reverse(),d=!0);let y=u.filter(X=>!Number.isNaN(X.y));y=TJe(y);let S=q1;switch(S=Wy,t.curve){case"linear":S=Wy;break;case"basis":S=q1;break;case"cardinal":S=sie;break;case"bumpX":S=rie;break;case"bumpY":S=nie;break;case"catmullRom":S=uie;break;case"monotoneX":S=mie;break;case"monotoneY":S=gie;break;case"natural":S=yie;break;case"step":S=vie;break;case"stepAfter":S=Eie;break;case"stepBefore":S=Sie;break;default:S=q1}const{x:w,y:E}=Tqe(t),k=uje().x(w).y(E).curve(S);let _;switch(t.thickness){case"normal":_="edge-thickness-normal";break;case"thick":_="edge-thickness-thick";break;case"invisible":_="edge-thickness-invisible";break;default:_="edge-thickness-normal"}switch(t.pattern){case"solid":_+=" edge-pattern-solid";break;case"dotted":_+=" edge-pattern-dotted";break;case"dashed":_+=" edge-pattern-dashed";break;default:_+=" edge-pattern-solid"}let R,C=t.curve==="rounded"?Aue(Fue(y,t),5):k(y);const F=Array.isArray(t.style)?t.style:[t.style];let L=F.find(X=>X==null?void 0:X.startsWith("stroke:")),I=!1;if(t.look==="handDrawn"){const X=_t.svg(e);Object.assign([],y);const z=X.path(C,{roughness:.3,seed:l});_+=" transition",R=Lr(z).select("path").attr("id",t.id).attr("class"," "+_+(t.classes?" "+t.classes:"")).attr("style",F?F.reduce((B,ee)=>B+";"+ee,""):"");let H=R.attr("d");R.attr("d",H),e.node().appendChild(R.node())}else{const X=m.join(";"),z=F?F.reduce((P,ne)=>P+ne+";",""):"";let H="";t.animate&&(H=" edge-animation-fast"),t.animation&&(H=" edge-animation-"+t.animation);const B=(X?X+";"+z+";":z)+";"+(F?F.reduce((P,ne)=>P+";"+ne,""):"");R=e.append("path").attr("d",C).attr("id",t.id).attr("class"," "+_+(t.classes?" "+t.classes:"")+(H??"")).attr("style",B),L=($=B.match(/stroke:([^;]+)/))==null?void 0:$[1],I=t.animate===!0||!!t.animation||X.includes("animation");const ee=R.node(),J=typeof ee.getTotalLength=="function"?ee.getTotalLength():0,K=gH[t.arrowTypeStart]||0,G=gH[t.arrowTypeEnd]||0;if(t.look==="neo"&&!I){const ne=`stroke-dasharray: ${t.pattern==="dotted"||t.pattern==="dashed"?AJe(J,K,G):`0 ${K} ${J-K-G} ${G}`}; stroke-dashoffset: 0;`;R.attr("style",ne+R.attr("style"))}}R.attr("data-edge",!0),R.attr("data-et","edge"),R.attr("data-id",t.id),R.attr("data-points",b),t.showPoints&&y.forEach(X=>{e.append("circle").style("stroke","red").style("fill","red").attr("r",1).attr("cx",X.x).attr("cy",X.y)});let M="";(Gr().flowchart.arrowMarkerAbsolute||Gr().state.arrowMarkerAbsolute)&&(M=window.location.protocol+"//"+window.location.host+window.location.pathname+window.location.search,M=M.replace(/\(/g,"\\(").replace(/\)/g,"\\)")),je.info("arrowTypeStart",t.arrowTypeStart),je.info("arrowTypeEnd",t.arrowTypeEnd),SJe(R,t,M,o,n,L);const j=Math.floor(u.length/2),Y=u[j];_o.isLabelCoordinateInPath(Y,R.attr("d"))||(d=!0);let W={};return d&&(W.updatedPath=u),W.originalPath=t.points,W},"insertEdge");function Aue(e,t){if(e.length<2)return"";let r="";const n=e.length,a=1e-5;for(let i=0;i<n;i++){const o=e[i],s=e[i-1],l=e[i+1];if(i===0)r+=`M${o.x},${o.y}`;else if(i===n-1)r+=`L${o.x},${o.y}`;else{const u=o.x-s.x,d=o.y-s.y,p=l.x-o.x,f=l.y-o.y,m=Math.hypot(u,d),b=Math.hypot(p,f);if(m<a||b<a){r+=`L${o.x},${o.y}`;continue}const y=u/m,S=d/m,w=p/b,E=f/b,k=y*w+S*E,_=Math.max(-1,Math.min(1,k)),R=Math.acos(_);if(R<a||Math.abs(Math.PI-R)<a){r+=`L${o.x},${o.y}`;continue}const C=Math.min(t/Math.sin(R/2),m/2,b/2),F=o.x-y*C,L=o.y-S*C,I=o.x+w*C,M=o.y+E*C;r+=`L${F},${L}`,r+=`Q${o.x},${o.y} ${I},${M}`}}return r}U(Aue,"generateRoundedPath");function $6(e,t){if(!e||!t)return{angle:0,deltaX:0,deltaY:0};const r=t.x-e.x,n=t.y-e.y;return{angle:Math.atan2(n,r),deltaX:r,deltaY:n}}U($6,"calculateDeltaAndAngle");function Fue(e,t){const r=e.map(a=>({...a}));if(e.length>=2&&Fa[t.arrowTypeStart]){const a=Fa[t.arrowTypeStart],i=e[0],o=e[1],{angle:s}=$6(i,o),l=a*Math.cos(s),u=a*Math.sin(s);r[0].x=i.x+l,r[0].y=i.y+u}const n=e.length;if(n>=2&&Fa[t.arrowTypeEnd]){const a=Fa[t.arrowTypeEnd],i=e[n-1],o=e[n-2],{angle:s}=$6(o,i),l=a*Math.cos(s),u=a*Math.sin(s);r[n-1].x=i.x-l,r[n-1].y=i.y-u}return r}U(Fue,"applyMarkerOffsetsToPoints");var _Je=U((e,t,r,n)=>{t.forEach(a=>{qJe[a](e,r,n)})},"insertMarkers"),RJe=U((e,t,r)=>{je.trace("Making markers for ",r),e.append("defs").append("marker").attr("id",r+"_"+t+"-extensionStart").attr("class","marker extension "+t).attr("refX",18).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 1,7 L18,13 V 1 Z"),e.append("defs").append("marker").attr("id",r+"_"+t+"-extensionEnd").attr("class","marker extension "+t).attr("refX",1).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 1,1 V 13 L18,7 Z")},"extension"),DJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-compositionStart").attr("class","marker composition "+t).attr("refX",18).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),e.append("defs").append("marker").attr("id",r+"_"+t+"-compositionEnd").attr("class","marker composition "+t).attr("refX",1).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z")},"composition"),NJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-aggregationStart").attr("class","marker aggregation "+t).attr("refX",18).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z"),e.append("defs").append("marker").attr("id",r+"_"+t+"-aggregationEnd").attr("class","marker aggregation "+t).attr("refX",1).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L1,7 L9,1 Z")},"aggregation"),LJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-dependencyStart").attr("class","marker dependency "+t).attr("refX",6).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("path").attr("d","M 5,7 L9,13 L1,7 L9,1 Z"),e.append("defs").append("marker").attr("id",r+"_"+t+"-dependencyEnd").attr("class","marker dependency "+t).attr("refX",13).attr("refY",7).attr("markerWidth",20).attr("markerHeight",28).attr("orient","auto").append("path").attr("d","M 18,7 L9,13 L14,7 L9,1 Z")},"dependency"),MJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-lollipopStart").attr("class","marker lollipop "+t).attr("refX",13).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("circle").attr("stroke","black").attr("fill","transparent").attr("cx",7).attr("cy",7).attr("r",6),e.append("defs").append("marker").attr("id",r+"_"+t+"-lollipopEnd").attr("class","marker lollipop "+t).attr("refX",1).attr("refY",7).attr("markerWidth",190).attr("markerHeight",240).attr("orient","auto").append("circle").attr("stroke","black").attr("fill","transparent").attr("cx",7).attr("cy",7).attr("r",6)},"lollipop"),IJe=U((e,t,r)=>{e.append("marker").attr("id",r+"_"+t+"-pointEnd").attr("class","marker "+t).attr("viewBox","0 0 10 10").attr("refX",5).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",8).attr("markerHeight",8).attr("orient","auto").append("path").attr("d","M 0 0 L 10 5 L 0 10 z").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0"),e.append("marker").attr("id",r+"_"+t+"-pointStart").attr("class","marker "+t).attr("viewBox","0 0 10 10").attr("refX",4.5).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",8).attr("markerHeight",8).attr("orient","auto").append("path").attr("d","M 0 5 L 10 10 L 10 0 z").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0")},"point"),OJe=U((e,t,r)=>{e.append("marker").attr("id",r+"_"+t+"-circleEnd").attr("class","marker "+t).attr("viewBox","0 0 10 10").attr("refX",11).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("circle").attr("cx","5").attr("cy","5").attr("r","5").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0"),e.append("marker").attr("id",r+"_"+t+"-circleStart").attr("class","marker "+t).attr("viewBox","0 0 10 10").attr("refX",-1).attr("refY",5).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("circle").attr("cx","5").attr("cy","5").attr("r","5").attr("class","arrowMarkerPath").style("stroke-width",1).style("stroke-dasharray","1,0")},"circle"),BJe=U((e,t,r)=>{e.append("marker").attr("id",r+"_"+t+"-crossEnd").attr("class","marker cross "+t).attr("viewBox","0 0 11 11").attr("refX",12).attr("refY",5.2).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("path").attr("d","M 1,1 l 9,9 M 10,1 l -9,9").attr("class","arrowMarkerPath").style("stroke-width",2).style("stroke-dasharray","1,0"),e.append("marker").attr("id",r+"_"+t+"-crossStart").attr("class","marker cross "+t).attr("viewBox","0 0 11 11").attr("refX",-1).attr("refY",5.2).attr("markerUnits","userSpaceOnUse").attr("markerWidth",11).attr("markerHeight",11).attr("orient","auto").append("path").attr("d","M 1,1 l 9,9 M 10,1 l -9,9").attr("class","arrowMarkerPath").style("stroke-width",2).style("stroke-dasharray","1,0")},"cross"),PJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-barbEnd").attr("refX",19).attr("refY",7).attr("markerWidth",20).attr("markerHeight",14).attr("markerUnits","userSpaceOnUse").attr("orient","auto").append("path").attr("d","M 19,7 L9,13 L14,7 L9,1 Z")},"barb"),zJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-onlyOneStart").attr("class","marker onlyOne "+t).attr("refX",0).attr("refY",9).attr("markerWidth",18).attr("markerHeight",18).attr("orient","auto").append("path").attr("d","M9,0 L9,18 M15,0 L15,18"),e.append("defs").append("marker").attr("id",r+"_"+t+"-onlyOneEnd").attr("class","marker onlyOne "+t).attr("refX",18).attr("refY",9).attr("markerWidth",18).attr("markerHeight",18).attr("orient","auto").append("path").attr("d","M3,0 L3,18 M9,0 L9,18")},"only_one"),HJe=U((e,t,r)=>{const n=e.append("defs").append("marker").attr("id",r+"_"+t+"-zeroOrOneStart").attr("class","marker zeroOrOne "+t).attr("refX",0).attr("refY",9).attr("markerWidth",30).attr("markerHeight",18).attr("orient","auto");n.append("circle").attr("fill","white").attr("cx",21).attr("cy",9).attr("r",6),n.append("path").attr("d","M9,0 L9,18");const a=e.append("defs").append("marker").attr("id",r+"_"+t+"-zeroOrOneEnd").attr("class","marker zeroOrOne "+t).attr("refX",30).attr("refY",9).attr("markerWidth",30).attr("markerHeight",18).attr("orient","auto");a.append("circle").attr("fill","white").attr("cx",9).attr("cy",9).attr("r",6),a.append("path").attr("d","M21,0 L21,18")},"zero_or_one"),UJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-oneOrMoreStart").attr("class","marker oneOrMore "+t).attr("refX",18).attr("refY",18).attr("markerWidth",45).attr("markerHeight",36).attr("orient","auto").append("path").attr("d","M0,18 Q 18,0 36,18 Q 18,36 0,18 M42,9 L42,27"),e.append("defs").append("marker").attr("id",r+"_"+t+"-oneOrMoreEnd").attr("class","marker oneOrMore "+t).attr("refX",27).attr("refY",18).attr("markerWidth",45).attr("markerHeight",36).attr("orient","auto").append("path").attr("d","M3,9 L3,27 M9,18 Q27,0 45,18 Q27,36 9,18")},"one_or_more"),$Je=U((e,t,r)=>{const n=e.append("defs").append("marker").attr("id",r+"_"+t+"-zeroOrMoreStart").attr("class","marker zeroOrMore "+t).attr("refX",18).attr("refY",18).attr("markerWidth",57).attr("markerHeight",36).attr("orient","auto");n.append("circle").attr("fill","white").attr("cx",48).attr("cy",18).attr("r",6),n.append("path").attr("d","M0,18 Q18,0 36,18 Q18,36 0,18");const a=e.append("defs").append("marker").attr("id",r+"_"+t+"-zeroOrMoreEnd").attr("class","marker zeroOrMore "+t).attr("refX",39).attr("refY",18).attr("markerWidth",57).attr("markerHeight",36).attr("orient","auto");a.append("circle").attr("fill","white").attr("cx",9).attr("cy",18).attr("r",6),a.append("path").attr("d","M21,18 Q39,0 57,18 Q39,36 21,18")},"zero_or_more"),GJe=U((e,t,r)=>{e.append("defs").append("marker").attr("id",r+"_"+t+"-requirement_arrowEnd").attr("refX",20).attr("refY",10).attr("markerWidth",20).attr("markerHeight",20).attr("orient","auto").append("path").attr("d",`M0,0
      L20,10
      M20,10
      L0,20`)},"requirement_arrow"),jJe=U((e,t,r)=>{const n=e.append("defs").append("marker").attr("id",r+"_"+t+"-requirement_containsStart").attr("refX",0).attr("refY",10).attr("markerWidth",20).attr("markerHeight",20).attr("orient","auto").append("g");n.append("circle").attr("cx",10).attr("cy",10).attr("r",9).attr("fill","none"),n.append("line").attr("x1",1).attr("x2",19).attr("y1",10).attr("y2",10),n.append("line").attr("y1",1).attr("y2",19).attr("x1",10).attr("x2",10)},"requirement_contains"),qJe={extension:RJe,composition:DJe,aggregation:NJe,dependency:LJe,lollipop:MJe,point:IJe,circle:OJe,cross:BJe,barb:PJe,only_one:zJe,zero_or_one:HJe,one_or_more:UJe,zero_or_more:$Je,requirement_arrow:GJe,requirement_contains:jJe},WJe=_Je,VJe={common:_p,getConfig:Ma,insertCluster:tJe,insertEdge:FJe,insertEdgeLabel:xJe,insertMarkers:WJe,insertNode:Cue,interpolateToCurve:HN,labelHelper:Zt,log:je,positionEdgeLabel:wJe},ig={},_ue=U(e=>{for(const t of e)ig[t.name]=t},"registerLayoutLoaders"),KJe=U(()=>{_ue([{name:"dagre",loader:U(async()=>await Pr(()=>import("./dagre-6UL2VRFP-CXCUh4Fa.js"),__vite__mapDeps([0,1,2,3,4,5])),"loader")},{name:"cose-bilkent",loader:U(async()=>await Pr(()=>import("./cose-bilkent-S5V4N54A-w8Z6_s03.js"),__vite__mapDeps([6,7])),"loader")}])},"registerDefaultLayoutLoaders");KJe();var Ydt=U(async(e,t)=>{if(!(e.layoutAlgorithm in ig))throw new Error(`Unknown layout algorithm: ${e.layoutAlgorithm}`);const r=ig[e.layoutAlgorithm];return(await r.loader()).render(e,t,VJe,{algorithm:r.algorithm})},"render"),Jdt=U((e="",{fallback:t="dagre"}={})=>{if(e in ig)return e;if(t in ig)return je.warn(`Layout algorithm ${e} is not registered. Using ${t} as fallback.`),t;throw new Error(`Both layout algorithms ${e} and ${t} are not registered.`)},"getRegisteredLayoutAlgorithm"),Rue="comm",Due="rule",Nue="decl",YJe="@import",JJe="@namespace",XJe="@keyframes",ZJe="@layer",Lue=Math.abs,oL=String.fromCharCode;function Mue(e){return e.trim()}function J1(e,t,r){return e.replace(t,r)}function QJe(e,t,r){return e.indexOf(t,r)}function Ah(e,t){return e.charCodeAt(t)|0}function ap(e,t,r){return e.slice(t,r)}function Qo(e){return e.length}function eXe(e){return e.length}function d1(e,t){return t.push(e),e}var Q2=1,ip=1,Iue=0,co=0,Gn=0,Mp="";function sL(e,t,r,n,a,i,o,s){return{value:e,root:t,parent:r,type:n,props:a,children:i,line:Q2,column:ip,length:o,return:"",siblings:s}}function tXe(){return Gn}function rXe(){return Gn=co>0?Ah(Mp,--co):0,ip--,Gn===10&&(ip=1,Q2--),Gn}function No(){return Gn=co<Iue?Ah(Mp,co++):0,ip++,Gn===10&&(ip=1,Q2++),Gn}function Eu(){return Ah(Mp,co)}function X1(){return co}function eS(e,t){return ap(Mp,e,t)}function og(e){switch(e){case 0:case 9:case 10:case 13:case 32:return 5;case 33:case 43:case 44:case 47:case 62:case 64:case 126:case 59:case 123:case 125:return 4;case 58:return 3;case 34:case 39:case 40:case 91:return 2;case 41:case 93:return 1}return 0}function nXe(e){return Q2=ip=1,Iue=Qo(Mp=e),co=0,[]}function aXe(e){return Mp="",e}function Gk(e){return Mue(eS(co-1,G6(e===91?e+2:e===40?e+1:e)))}function iXe(e){for(;(Gn=Eu())&&Gn<33;)No();return og(e)>2||og(Gn)>3?"":" "}function oXe(e,t){for(;--t&&No()&&!(Gn<48||Gn>102||Gn>57&&Gn<65||Gn>70&&Gn<97););return eS(e,X1()+(t<6&&Eu()==32&&No()==32))}function G6(e){for(;No();)switch(Gn){case e:return co;case 34:case 39:e!==34&&e!==39&&G6(Gn);break;case 40:e===41&&G6(e);break;case 92:No();break}return co}function sXe(e,t){for(;No()&&e+Gn!==57;)if(e+Gn===84&&Eu()===47)break;return"/*"+eS(t,co-1)+"*"+oL(e===47?e:No())}function lXe(e){for(;!og(Eu());)No();return eS(e,co)}function uXe(e){return aXe(Z1("",null,null,null,[""],e=nXe(e),0,[0],e))}function Z1(e,t,r,n,a,i,o,s,l){for(var u=0,d=0,p=o,f=0,m=0,b=0,y=1,S=1,w=1,E=0,k="",_=a,R=i,C=n,F=k;S;)switch(b=E,E=No()){case 40:if(b!=108&&Ah(F,p-1)==58){QJe(F+=J1(Gk(E),"&","&\f"),"&\f",Lue(u?s[u-1]:0))!=-1&&(w=-1);break}case 34:case 39:case 91:F+=Gk(E);break;case 9:case 10:case 13:case 32:F+=iXe(b);break;case 92:F+=oXe(X1()-1,7);continue;case 47:switch(Eu()){case 42:case 47:d1(cXe(sXe(No(),X1()),t,r,l),l),(og(b||1)==5||og(Eu()||1)==5)&&Qo(F)&&ap(F,-1,void 0)!==" "&&(F+=" ");break;default:F+="/"}break;case 123*y:s[u++]=Qo(F)*w;case 125*y:case 59:case 0:switch(E){case 0:case 125:S=0;case 59+d:w==-1&&(F=J1(F,/\f/g,"")),m>0&&(Qo(F)-p||y===0&&b===47)&&d1(m>32?tU(F+";",n,r,p-1,l):tU(J1(F," ","")+";",n,r,p-2,l),l);break;case 59:F+=";";default:if(d1(C=eU(F,t,r,u,d,a,s,k,_=[],R=[],p,i),i),E===123)if(d===0)Z1(F,t,C,C,_,i,p,s,R);else{switch(f){case 99:if(Ah(F,3)===110)break;case 108:if(Ah(F,2)===97)break;default:d=0;case 100:case 109:case 115:}d?Z1(e,C,C,n&&d1(eU(e,C,C,0,0,a,s,k,a,_=[],p,R),R),a,R,p,s,n?_:R):Z1(F,C,C,C,[""],R,0,s,R)}}u=d=m=0,y=w=1,k=F="",p=o;break;case 58:p=1+Qo(F),m=b;default:if(y<1){if(E==123)--y;else if(E==125&&y++==0&&rXe()==125)continue}switch(F+=oL(E),E*y){case 38:w=d>0?1:(F+="\f",-1);break;case 44:s[u++]=(Qo(F)-1)*w,w=1;break;case 64:Eu()===45&&(F+=Gk(No())),f=Eu(),d=p=Qo(k=F+=lXe(X1())),E++;break;case 45:b===45&&Qo(F)==2&&(y=0)}}return i}function eU(e,t,r,n,a,i,o,s,l,u,d,p){for(var f=a-1,m=a===0?i:[""],b=eXe(m),y=0,S=0,w=0;y<n;++y)for(var E=0,k=ap(e,f+1,f=Lue(S=o[y])),_=e;E<b;++E)(_=Mue(S>0?m[E]+" "+k:J1(k,/&\f/g,m[E])))&&(l[w++]=_);return sL(e,t,r,a===0?Due:s,l,u,d,p)}function cXe(e,t,r,n){return sL(e,t,r,Rue,oL(tXe()),ap(e,2,-2),0,n)}function tU(e,t,r,n,a){return sL(e,t,r,Nue,ap(e,0,n),ap(e,n+1,-1),n,a)}function j6(e,t){for(var r="",n=0;n<e.length;n++)r+=t(e[n],n,e,t)||"";return r}function dXe(e,t,r,n){switch(e.type){case ZJe:if(e.children.length)break;case YJe:case JJe:case Nue:return e.return=e.return||e.value;case Rue:return"";case XJe:return e.return=e.value+"{"+j6(e.children,n)+"}";case Due:if(!Qo(e.value=e.props.join(",")))return""}return Qo(r=j6(e.children,n))?e.return=e.value+"{"+r+"}":""}var hXe=sse(Object.keys,Object),pXe=Object.prototype,fXe=pXe.hasOwnProperty;function mXe(e){if(!G2(e))return hXe(e);var t=[];for(var r in Object(e))fXe.call(e,r)&&r!="constructor"&&t.push(r);return t}var q6=cd(xs,"DataView"),W6=cd(xs,"Promise"),V6=cd(xs,"Set"),K6=cd(xs,"WeakMap"),rU="[object Map]",gXe="[object Object]",nU="[object Promise]",aU="[object Set]",iU="[object WeakMap]",oU="[object DataView]",bXe=ud(q6),yXe=ud(ng),vXe=ud(W6),SXe=ud(V6),EXe=ud(K6),vc=Dp;(q6&&vc(new q6(new ArrayBuffer(1)))!=oU||ng&&vc(new ng)!=rU||W6&&vc(W6.resolve())!=nU||V6&&vc(new V6)!=aU||K6&&vc(new K6)!=iU)&&(vc=function(e){var t=Dp(e),r=t==gXe?e.constructor:void 0,n=r?ud(r):"";if(n)switch(n){case bXe:return oU;case yXe:return rU;case vXe:return nU;case SXe:return aU;case EXe:return iU}return t});var xXe="[object Map]",wXe="[object Set]",kXe=Object.prototype,CXe=kXe.hasOwnProperty;function sU(e){if(e==null)return!0;if(j2(e)&&(iv(e)||typeof e=="string"||typeof e.splice=="function"||PN(e)||zN(e)||av(e)))return!e.length;var t=vc(e);if(t==xXe||t==wXe)return!e.size;if(G2(e))return!mXe(e).length;for(var r in e)if(CXe.call(e,r))return!1;return!0}var Oue="c4",TXe=U(e=>/^\s*C4Context|C4Container|C4Component|C4Dynamic|C4Deployment/.test(e),"detector"),AXe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./c4Diagram-YG6GDRKO-wsxKHdY7.js");return{diagram:t}},__vite__mapDeps([8,9]));return{id:Oue,diagram:e}},"loader"),FXe={id:Oue,detector:TXe,loader:AXe},_Xe=FXe,Bue="flowchart",RXe=U((e,t)=>{var r,n;return((r=t==null?void 0:t.flowchart)==null?void 0:r.defaultRenderer)==="dagre-wrapper"||((n=t==null?void 0:t.flowchart)==null?void 0:n.defaultRenderer)==="elk"?!1:/^\s*graph/.test(e)},"detector"),DXe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./flowDiagram-NV44I4VS-Cea10LNu.js");return{diagram:t}},__vite__mapDeps([10,11,12,13,14]));return{id:Bue,diagram:e}},"loader"),NXe={id:Bue,detector:RXe,loader:DXe},LXe=NXe,Pue="flowchart-v2",MXe=U((e,t)=>{var r,n,a;return((r=t==null?void 0:t.flowchart)==null?void 0:r.defaultRenderer)==="dagre-d3"?!1:(((n=t==null?void 0:t.flowchart)==null?void 0:n.defaultRenderer)==="elk"&&(t.layout="elk"),/^\s*graph/.test(e)&&((a=t==null?void 0:t.flowchart)==null?void 0:a.defaultRenderer)==="dagre-wrapper"?!0:/^\s*flowchart/.test(e))},"detector"),IXe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./flowDiagram-NV44I4VS-Cea10LNu.js");return{diagram:t}},__vite__mapDeps([10,11,12,13,14]));return{id:Pue,diagram:e}},"loader"),OXe={id:Pue,detector:MXe,loader:IXe},BXe=OXe,zue="er",PXe=U(e=>/^\s*erDiagram/.test(e),"detector"),zXe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./erDiagram-Q2GNP2WA-Co43UztB.js");return{diagram:t}},__vite__mapDeps([15,12,13,14]));return{id:zue,diagram:e}},"loader"),HXe={id:zue,detector:PXe,loader:zXe},UXe=HXe,Hue="gitGraph",$Xe=U(e=>/^\s*gitGraph/.test(e),"detector"),GXe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./gitGraphDiagram-NY62KEGX-BoXPnSbQ.js");return{diagram:t}},__vite__mapDeps([16,17,18,19,2,4,5]));return{id:Hue,diagram:e}},"loader"),jXe={id:Hue,detector:$Xe,loader:GXe},qXe=jXe,Uue="gantt",WXe=U(e=>/^\s*gantt/.test(e),"detector"),VXe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./ganttDiagram-LVOFAZNH-K1v31eUf.js");return{diagram:t}},__vite__mapDeps([20,21,22,23]));return{id:Uue,diagram:e}},"loader"),KXe={id:Uue,detector:WXe,loader:VXe},YXe=KXe,$ue="info",JXe=U(e=>/^\s*info/.test(e),"detector"),XXe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./infoDiagram-F6ZHWCRC-CqiQjXdQ.js");return{diagram:t}},__vite__mapDeps([24,19,2,4,5]));return{id:$ue,diagram:e}},"loader"),ZXe={id:$ue,detector:JXe,loader:XXe},Gue="pie",QXe=U(e=>/^\s*pie/.test(e),"detector"),eZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./pieDiagram-ADFJNKIX-DVpm4HYO.js");return{diagram:t}},__vite__mapDeps([25,17,19,2,4,5,26,27,22]));return{id:Gue,diagram:e}},"loader"),tZe={id:Gue,detector:QXe,loader:eZe},jue="quadrantChart",rZe=U(e=>/^\s*quadrantChart/.test(e),"detector"),nZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./quadrantDiagram-AYHSOK5B-B_q4uP4u.js");return{diagram:t}},__vite__mapDeps([28,21,22,23]));return{id:jue,diagram:e}},"loader"),aZe={id:jue,detector:rZe,loader:nZe},iZe=aZe,que="xychart",oZe=U(e=>/^\s*xychart(-beta)?/.test(e),"detector"),sZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./xychartDiagram-PRI3JC2R-BQB5fhK1.js");return{diagram:t}},__vite__mapDeps([29,22,27,21,23]));return{id:que,diagram:e}},"loader"),lZe={id:que,detector:oZe,loader:sZe},uZe=lZe,Wue="requirement",cZe=U(e=>/^\s*requirement(Diagram)?/.test(e),"detector"),dZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./requirementDiagram-UZGBJVZJ-D3V1ZBb3.js");return{diagram:t}},__vite__mapDeps([30,12,13]));return{id:Wue,diagram:e}},"loader"),hZe={id:Wue,detector:cZe,loader:dZe},pZe=hZe,Vue="sequence",fZe=U(e=>/^\s*sequenceDiagram/.test(e),"detector"),mZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./sequenceDiagram-WL72ISMW-B4tO_SHN.js");return{diagram:t}},__vite__mapDeps([31,9,18]));return{id:Vue,diagram:e}},"loader"),gZe={id:Vue,detector:fZe,loader:mZe},bZe=gZe,Kue="class",yZe=U((e,t)=>{var r;return((r=t==null?void 0:t.class)==null?void 0:r.defaultRenderer)==="dagre-wrapper"?!1:/^\s*classDiagram/.test(e)},"detector"),vZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./classDiagram-2ON5EDUG-CKcdz_xq.js");return{diagram:t}},__vite__mapDeps([32,33,11,12,13]));return{id:Kue,diagram:e}},"loader"),SZe={id:Kue,detector:yZe,loader:vZe},EZe=SZe,Yue="classDiagram",xZe=U((e,t)=>{var r;return/^\s*classDiagram/.test(e)&&((r=t==null?void 0:t.class)==null?void 0:r.defaultRenderer)==="dagre-wrapper"?!0:/^\s*classDiagram-v2/.test(e)},"detector"),wZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./classDiagram-v2-WZHVMYZB-CKcdz_xq.js");return{diagram:t}},__vite__mapDeps([34,33,11,12,13]));return{id:Yue,diagram:e}},"loader"),kZe={id:Yue,detector:xZe,loader:wZe},CZe=kZe,Jue="state",TZe=U((e,t)=>{var r;return((r=t==null?void 0:t.state)==null?void 0:r.defaultRenderer)==="dagre-wrapper"?!1:/^\s*stateDiagram/.test(e)},"detector"),AZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./stateDiagram-FKZM4ZOC-DZeTsklT.js");return{diagram:t}},__vite__mapDeps([35,36,12,13,1,2,3,4]));return{id:Jue,diagram:e}},"loader"),FZe={id:Jue,detector:TZe,loader:AZe},_Ze=FZe,Xue="stateDiagram",RZe=U((e,t)=>{var r;return!!(/^\s*stateDiagram-v2/.test(e)||/^\s*stateDiagram/.test(e)&&((r=t==null?void 0:t.state)==null?void 0:r.defaultRenderer)==="dagre-wrapper")},"detector"),DZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./stateDiagram-v2-4FDKWEC3-BXv4Etpu.js");return{diagram:t}},__vite__mapDeps([37,36,12,13]));return{id:Xue,diagram:e}},"loader"),NZe={id:Xue,detector:RZe,loader:DZe},LZe=NZe,Zue="journey",MZe=U(e=>/^\s*journey/.test(e),"detector"),IZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./journeyDiagram-XKPGCS4Q-C3yL6hVc.js");return{diagram:t}},__vite__mapDeps([38,9,11,26]));return{id:Zue,diagram:e}},"loader"),OZe={id:Zue,detector:MZe,loader:IZe},BZe=OZe,PZe=U((e,t,r)=>{je.debug(`rendering svg for syntax error
`);const n=yje(t),a=n.append("g");n.attr("viewBox","0 0 2412 512"),Cae(n,100,512,!0),a.append("path").attr("class","error-icon").attr("d","m411.313,123.313c6.25-6.25 6.25-16.375 0-22.625s-16.375-6.25-22.625,0l-32,32-9.375,9.375-20.688-20.688c-12.484-12.5-32.766-12.5-45.25,0l-16,16c-1.261,1.261-2.304,2.648-3.31,4.051-21.739-8.561-45.324-13.426-70.065-13.426-105.867,0-192,86.133-192,192s86.133,192 192,192 192-86.133 192-192c0-24.741-4.864-48.327-13.426-70.065 1.402-1.007 2.79-2.049 4.051-3.31l16-16c12.5-12.492 12.5-32.758 0-45.25l-20.688-20.688 9.375-9.375 32.001-31.999zm-219.313,100.687c-52.938,0-96,43.063-96,96 0,8.836-7.164,16-16,16s-16-7.164-16-16c0-70.578 57.422-128 128-128 8.836,0 16,7.164 16,16s-7.164,16-16,16z"),a.append("path").attr("class","error-icon").attr("d","m459.02,148.98c-6.25-6.25-16.375-6.25-22.625,0s-6.25,16.375 0,22.625l16,16c3.125,3.125 7.219,4.688 11.313,4.688 4.094,0 8.188-1.563 11.313-4.688 6.25-6.25 6.25-16.375 0-22.625l-16.001-16z"),a.append("path").attr("class","error-icon").attr("d","m340.395,75.605c3.125,3.125 7.219,4.688 11.313,4.688 4.094,0 8.188-1.563 11.313-4.688 6.25-6.25 6.25-16.375 0-22.625l-16-16c-6.25-6.25-16.375-6.25-22.625,0s-6.25,16.375 0,22.625l15.999,16z"),a.append("path").attr("class","error-icon").attr("d","m400,64c8.844,0 16-7.164 16-16v-32c0-8.836-7.156-16-16-16-8.844,0-16,7.164-16,16v32c0,8.836 7.156,16 16,16z"),a.append("path").attr("class","error-icon").attr("d","m496,96.586h-32c-8.844,0-16,7.164-16,16 0,8.836 7.156,16 16,16h32c8.844,0 16-7.164 16-16 0-8.836-7.156-16-16-16z"),a.append("path").attr("class","error-icon").attr("d","m436.98,75.605c3.125,3.125 7.219,4.688 11.313,4.688 4.094,0 8.188-1.563 11.313-4.688l32-32c6.25-6.25 6.25-16.375 0-22.625s-16.375-6.25-22.625,0l-32,32c-6.251,6.25-6.251,16.375-0.001,22.625z"),a.append("text").attr("class","error-text").attr("x",1440).attr("y",250).attr("font-size","150px").style("text-anchor","middle").text("Syntax error in text"),a.append("text").attr("class","error-text").attr("x",1250).attr("y",400).attr("font-size","100px").style("text-anchor","middle").text(`mermaid version ${r}`)},"draw"),Que={draw:PZe},zZe=Que,HZe={db:{},renderer:Que,parser:{parse:U(()=>{},"parse")}},UZe=HZe,ece="flowchart-elk",$Ze=U((e,t={})=>{var r;return/^\s*flowchart-elk/.test(e)||/^\s*(flowchart|graph)/.test(e)&&((r=t==null?void 0:t.flowchart)==null?void 0:r.defaultRenderer)==="elk"?(t.layout="elk",!0):!1},"detector"),GZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./flowDiagram-NV44I4VS-Cea10LNu.js");return{diagram:t}},__vite__mapDeps([10,11,12,13,14]));return{id:ece,diagram:e}},"loader"),jZe={id:ece,detector:$Ze,loader:GZe},qZe=jZe,tce="timeline",WZe=U(e=>/^\s*timeline/.test(e),"detector"),VZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./timeline-definition-IT6M3QCI-EppKg-g9.js");return{diagram:t}},__vite__mapDeps([39,26]));return{id:tce,diagram:e}},"loader"),KZe={id:tce,detector:WZe,loader:VZe},YZe=KZe,rce="mindmap",JZe=U(e=>/^\s*mindmap/.test(e),"detector"),XZe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./mindmap-definition-VGOIOE7T-1BY12Tdp.js");return{diagram:t}},__vite__mapDeps([40,12,13]));return{id:rce,diagram:e}},"loader"),ZZe={id:rce,detector:JZe,loader:XZe},QZe=ZZe,nce="kanban",eQe=U(e=>/^\s*kanban/.test(e),"detector"),tQe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./kanban-definition-3W4ZIXB7-Dnras9Cb.js");return{diagram:t}},__vite__mapDeps([41,11]));return{id:nce,diagram:e}},"loader"),rQe={id:nce,detector:eQe,loader:tQe},nQe=rQe,ace="sankey",aQe=U(e=>/^\s*sankey(-beta)?/.test(e),"detector"),iQe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./sankeyDiagram-TZEHDZUN-D78eCDcZ.js");return{diagram:t}},__vite__mapDeps([42,27,22]));return{id:ace,diagram:e}},"loader"),oQe={id:ace,detector:aQe,loader:iQe},sQe=oQe,ice="packet",lQe=U(e=>/^\s*packet(-beta)?/.test(e),"detector"),uQe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./diagram-S2PKOQOG-BACuWjdy.js");return{diagram:t}},__vite__mapDeps([43,17,19,2,4,5]));return{id:ice,diagram:e}},"loader"),cQe={id:ice,detector:lQe,loader:uQe},oce="radar",dQe=U(e=>/^\s*radar-beta/.test(e),"detector"),hQe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./diagram-QEK2KX5R-BrWPzwYr.js");return{diagram:t}},__vite__mapDeps([44,17,19,2,4,5]));return{id:oce,diagram:e}},"loader"),pQe={id:oce,detector:dQe,loader:hQe},sce="block",fQe=U(e=>/^\s*block(-beta)?/.test(e),"detector"),mQe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./blockDiagram-VD42YOAC-q24NfDqw.js");return{diagram:t}},__vite__mapDeps([45,11,5,2,1,14]));return{id:sce,diagram:e}},"loader"),gQe={id:sce,detector:fQe,loader:mQe},bQe=gQe,lce="architecture",yQe=U(e=>/^\s*architecture/.test(e),"detector"),vQe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./architectureDiagram-VXUJARFQ-BYI4md1h.js");return{diagram:t}},__vite__mapDeps([46,17,19,2,4,5,7]));return{id:lce,diagram:e}},"loader"),SQe={id:lce,detector:yQe,loader:vQe},EQe=SQe,uce="treemap",xQe=U(e=>/^\s*treemap/.test(e),"detector"),wQe=U(async()=>{const{diagram:e}=await Pr(async()=>{const{diagram:t}=await import("./diagram-PSM6KHXK-BdzOSGFs.js");return{diagram:t}},__vite__mapDeps([47,13,17,19,2,4,5,23,27,22]));return{id:uce,diagram:e}},"loader"),kQe={id:uce,detector:xQe,loader:wQe},lU=!1,tS=U(()=>{lU||(lU=!0,zy("error",UZe,e=>e.toLowerCase().trim()==="error"),zy("---",{db:{clear:U(()=>{},"clear")},styles:{},renderer:{draw:U(()=>{},"draw")},parser:{parse:U(()=>{throw new Error("Diagrams beginning with --- are not valid. If you were trying to use a YAML front-matter, please ensure that you've correctly opened and closed the YAML front-matter with un-indented `---` blocks")},"parse")},init:U(()=>null,"init")},e=>e.toLowerCase().trimStart().startsWith("---")),t6(qZe,QZe,EQe),t6(_Xe,nQe,CZe,EZe,UXe,YXe,ZXe,tZe,pZe,bZe,BXe,LXe,YZe,qXe,LZe,_Ze,BZe,iZe,sQe,cQe,uZe,bQe,pQe,kQe))},"addDiagrams"),CQe=U(async()=>{je.debug("Loading registered diagrams");const t=(await Promise.allSettled(Object.entries(Vc).map(async([r,{detector:n,loader:a}])=>{if(a)try{i6(r)}catch{try{const{diagram:i,id:o}=await a();zy(o,i,n)}catch(i){throw je.error(`Failed to load external diagram with key ${r}. Removing from detectors.`),delete Vc[r],i}}}))).filter(r=>r.status==="rejected");if(t.length>0){je.error(`Failed to load ${t.length} external diagrams`);for(const r of t)je.error(r);throw new Error(`Failed to load ${t.length} external diagrams`)}},"loadRegisteredDiagrams"),TQe="graphics-document document";function cce(e,t){e.attr("role",TQe),t!==""&&e.attr("aria-roledescription",t)}U(cce,"setA11yDiagramInfo");function dce(e,t,r,n){if(e.insert!==void 0){if(r){const a=`chart-desc-${n}`;e.attr("aria-describedby",a),e.insert("desc",":first-child").attr("id",a).text(r)}if(t){const a=`chart-title-${n}`;e.attr("aria-labelledby",a),e.insert("title",":first-child").attr("id",a).text(t)}}}U(dce,"addSVGa11yTitleDescription");var Lc,Y6=(Lc=class{constructor(t,r,n,a,i){this.type=t,this.text=r,this.db=n,this.parser=a,this.renderer=i}static async fromText(t,r={}){var u,d;const n=Ma(),a=uN(t,n);t=kKe(t)+`
`;try{i6(a)}catch{const p=Kze(a);if(!p)throw new pae(`Diagram ${a} not found.`);const{id:f,diagram:m}=await p();zy(f,m)}const{db:i,parser:o,renderer:s,init:l}=i6(a);return o.parser&&(o.parser.yy=i),(u=i.clear)==null||u.call(i),l==null||l(n),r.title&&((d=i.setDiagramTitle)==null||d.call(i,r.title)),await o.parse(t),new Lc(a,t,i,o,s)}async render(t,r){await this.renderer.draw(this.text,t,r,this)}getParser(){return this.parser}getType(){return this.type}},U(Lc,"Diagram"),Lc),uU=[],AQe=U(()=>{uU.forEach(e=>{e()}),uU=[]},"attachFunctions"),FQe=U(e=>e.replace(/^\s*%%(?!{)[^\n]+\n?/gm,"").trimStart(),"cleanupComments");function hce(e){const t=e.match(hae);if(!t)return{text:e,metadata:{}};let r=Cqe(t[1],{schema:kqe})??{};r=typeof r=="object"&&!Array.isArray(r)?r:{};const n={};return r.displayMode&&(n.displayMode=r.displayMode.toString()),r.title&&(n.title=r.title.toString()),r.config&&(n.config=r.config),{text:e.slice(t[0].length),metadata:n}}U(hce,"extractFrontMatter");var _Qe=U(e=>e.replace(/\r\n?/g,`
`).replace(/<(\w+)([^>]*)>/g,(t,r,n)=>"<"+r+n.replace(/="([^"]*)"/g,"='$1'")+">"),"cleanupText"),RQe=U(e=>{const{text:t,metadata:r}=hce(e),{displayMode:n,title:a,config:i={}}=r;return n&&(i.gantt||(i.gantt={}),i.gantt.displayMode=n),{title:a,config:i,text:t}},"processFrontmatter"),DQe=U(e=>{const t=_o.detectInit(e)??{},r=_o.detectDirective(e,"wrap");return Array.isArray(r)?t.wrap=r.some(({type:n})=>n==="wrap"):(r==null?void 0:r.type)==="wrap"&&(t.wrap=!0),{text:dKe(e),directive:t}},"processDirectives");function lL(e){const t=_Qe(e),r=RQe(t),n=DQe(r.text),a=qN(r.config,n.directive);return e=FQe(n.text),{code:e,title:r.title,config:a}}U(lL,"preprocessDiagram");function pce(e){const t=new TextEncoder().encode(e),r=Array.from(t,n=>String.fromCodePoint(n)).join("");return btoa(r)}U(pce,"toBase64");var NQe=5e4,LQe="graph TB;a[Maximum text size in diagram exceeded];style a fill:#faa",MQe="sandbox",IQe="loose",OQe="http://www.w3.org/2000/svg",BQe="http://www.w3.org/1999/xlink",PQe="http://www.w3.org/1999/xhtml",zQe="100%",HQe="100%",UQe="border:0;margin:0;",$Qe="margin:0",GQe="allow-top-navigation-by-user-activation allow-popups",jQe='The "iframe" tag is not supported by your browser.',qQe=["foreignobject"],WQe=["dominant-baseline"];function uL(e){const t=lL(e);return By(),cHe(t.config??{}),t}U(uL,"processAndSetConfigs");async function fce(e,t){tS();try{const{code:r,config:n}=uL(e);return{diagramType:(await gce(r)).type,config:n}}catch(r){if(t!=null&&t.suppressErrors)return!1;throw r}}U(fce,"parse");var cU=U((e,t,r=[])=>`
.${e} ${t} { ${r.join(" !important; ")} !important; }`,"cssImportantStyles"),VQe=U((e,t=new Map)=>{var n;let r="";if(e.themeCSS!==void 0&&(r+=`
${e.themeCSS}`),e.fontFamily!==void 0&&(r+=`
:root { --mermaid-font-family: ${e.fontFamily}}`),e.altFontFamily!==void 0&&(r+=`
:root { --mermaid-alt-font-family: ${e.altFontFamily}}`),t instanceof Map){const s=e.htmlLabels??((n=e.flowchart)==null?void 0:n.htmlLabels)?["> *","span"]:["rect","polygon","ellipse","circle","path"];t.forEach(l=>{sU(l.styles)||s.forEach(u=>{r+=cU(l.id,u,l.styles)}),sU(l.textStyles)||(r+=cU(l.id,"tspan",((l==null?void 0:l.textStyles)||[]).map(u=>u.replace("color","fill"))))})}return r},"createCssStyles"),KQe=U((e,t,r,n)=>{const a=VQe(e,r),i=RHe(t,a,e.themeVariables);return j6(uXe(`${n}{${i}}`),dXe)},"createUserStyles"),YQe=U((e="",t,r)=>{let n=e;return!r&&!t&&(n=n.replace(/marker-end="url\([\d+./:=?A-Za-z-]*?#/g,'marker-end="url(#')),n=dd(n),n=n.replace(/<br>/g,"<br/>"),n},"cleanUpSvgCode"),JQe=U((e="",t)=>{var a,i;const r=(i=(a=t==null?void 0:t.viewBox)==null?void 0:a.baseVal)!=null&&i.height?t.viewBox.baseVal.height+"px":HQe,n=pce(`<body style="${$Qe}">${e}</body>`);return`<iframe style="width:${zQe};height:${r};${UQe}" src="data:text/html;charset=UTF-8;base64,${n}" sandbox="${GQe}">
  ${jQe}
</iframe>`},"putIntoIFrame"),dU=U((e,t,r,n,a)=>{const i=e.append("div");i.attr("id",r),n&&i.attr("style",n);const o=i.append("svg").attr("id",t).attr("width","100%").attr("xmlns",OQe);return a&&o.attr("xmlns:xlink",a),o.append("g"),e},"appendDivSvgG");function J6(e,t){return e.append("iframe").attr("id",t).attr("style","width: 100%; height: 100%;").attr("sandbox","")}U(J6,"sandboxedIframe");var XQe=U((e,t,r,n)=>{var a,i,o;(a=e.getElementById(t))==null||a.remove(),(i=e.getElementById(r))==null||i.remove(),(o=e.getElementById(n))==null||o.remove()},"removeExistingElements"),ZQe=U(async function(e,t,r){var Y,W,$,X,z,H;tS();const n=uL(t);t=n.code;const a=Ma();je.debug(a),t.length>((a==null?void 0:a.maxTextSize)??NQe)&&(t=LQe);const i="#"+e,o="i"+e,s="#"+o,l="d"+e,u="#"+l,d=U(()=>{const ee=Lr(f?s:u).node();ee&&"remove"in ee&&ee.remove()},"removeTempElements");let p=Lr("body");const f=a.securityLevel===MQe,m=a.securityLevel===IQe,b=a.fontFamily;if(r!==void 0){if(r&&(r.innerHTML=""),f){const B=J6(Lr(r),o);p=Lr(B.nodes()[0].contentDocument.body),p.node().style.margin=0}else p=Lr(r);dU(p,e,l,`font-family: ${b}`,BQe)}else{if(XQe(document,e,l,o),f){const B=J6(Lr("body"),o);p=Lr(B.nodes()[0].contentDocument.body),p.node().style.margin=0}else p=Lr("body");dU(p,e,l)}let y,S;try{y=await Y6.fromText(t,{title:n.title})}catch(B){if(a.suppressErrorRendering)throw d(),B;y=await Y6.fromText("error"),S=B}const w=p.select(u).node(),E=y.type,k=w.firstChild,_=k.firstChild,R=(W=(Y=y.renderer).getClasses)==null?void 0:W.call(Y,t,y),C=KQe(a,E,R,i),F=document.createElement("style");F.innerHTML=C,k.insertBefore(F,_);try{await y.renderer.draw(t,e,Rz.version,y)}catch(B){throw a.suppressErrorRendering?d():zZe.draw(t,e,Rz.version),B}const L=p.select(`${u} svg`),I=(X=($=y.db).getAccTitle)==null?void 0:X.call($),M=(H=(z=y.db).getAccDescription)==null?void 0:H.call(z);bce(E,L,I,M),p.select(`[id="${e}"]`).selectAll("foreignobject > *").attr("xmlns",PQe);let j=p.select(u).node().innerHTML;if(je.debug("config.arrowMarkerAbsolute",a.arrowMarkerAbsolute),j=YQe(j,f,ra(a.arrowMarkerAbsolute)),f){const B=p.select(u+" svg").node();j=JQe(j,B)}else m||(j=Xh.sanitize(j,{ADD_TAGS:qQe,ADD_ATTR:WQe,HTML_INTEGRATION_POINTS:{foreignobject:!0}}));if(AQe(),S)throw S;return d(),{diagramType:E,svg:j,bindFunctions:y.db.bindFunctions}},"render");function mce(e={}){var n;const t=Qn({},e);t!=null&&t.fontFamily&&!((n=t.themeVariables)!=null&&n.fontFamily)&&(t.themeVariables||(t.themeVariables={}),t.themeVariables.fontFamily=t.fontFamily),lHe(t),t!=null&&t.theme&&t.theme in dl?t.themeVariables=dl[t.theme].getThemeVariables(t.themeVariables):t&&(t.themeVariables=dl.default.getThemeVariables(t.themeVariables));const r=typeof t=="object"?sHe(t):yae();lN(r.logLevel),tS()}U(mce,"initialize");var gce=U((e,t={})=>{const{code:r}=lL(e);return Y6.fromText(r,t)},"getDiagramFromText");function bce(e,t,r,n){cce(t,e),dce(t,r,n,t.attr("id"))}U(bce,"addA11yInfo");var Qc=Object.freeze({render:ZQe,parse:fce,getDiagramFromText:gce,initialize:mce,getConfig:Ma,setConfig:vae,getSiteConfig:yae,updateSiteConfig:uHe,reset:U(()=>{By()},"reset"),globalReset:U(()=>{By(Zh)},"globalReset"),defaultConfig:Zh});lN(Ma().logLevel);By(Ma());var QQe=U((e,t,r)=>{je.warn(e),jN(e)?(r&&r(e.str,e.hash),t.push({...e,message:e.str,error:e})):(r&&r(e),e instanceof Error&&t.push({str:e.message,message:e.message,hash:e.name,error:e}))},"handleError"),yce=U(async function(e={querySelector:".mermaid"}){try{await eet(e)}catch(t){if(jN(t)&&je.error(t.str),zi.parseError&&zi.parseError(t),!e.suppressErrors)throw je.error("Use the suppressErrors option to suppress these errors"),t}},"run"),eet=U(async function({postRenderCallback:e,querySelector:t,nodes:r}={querySelector:".mermaid"}){const n=Qc.getConfig();je.debug(`${e?"":"No "}Callback function found`);let a;if(r)a=r;else if(t)a=document.querySelectorAll(t);else throw new Error("Nodes and querySelector are both undefined");je.debug(`Found ${a.length} diagrams`),(n==null?void 0:n.startOnLoad)!==void 0&&(je.debug("Start On Load: "+(n==null?void 0:n.startOnLoad)),Qc.updateSiteConfig({startOnLoad:n==null?void 0:n.startOnLoad}));const i=new _o.InitIDGenerator(n.deterministicIds,n.deterministicIDSeed);let o;const s=[];for(const l of Array.from(a)){if(je.info("Rendering diagram: "+l.id),l.getAttribute("data-processed"))continue;l.setAttribute("data-processed","true");const u=`mermaid-${i.next()}`;o=l.innerHTML,o=zse(_o.entityDecode(o)).trim().replace(/<br\s*\/?>/gi,"<br/>");const d=_o.detectInit(o);d&&je.debug("Detected early reinit: ",d);try{const{svg:p,bindFunctions:f}=await xce(u,o,l);l.innerHTML=p,e&&await e(u),f&&f(l)}catch(p){QQe(p,s,zi.parseError)}}if(s.length>0)throw s[0]},"runThrowsErrors"),vce=U(function(e){Qc.initialize(e)},"initialize"),tet=U(async function(e,t,r){je.warn("mermaid.init is deprecated. Please use run instead."),e&&vce(e);const n={postRenderCallback:r,querySelector:".mermaid"};typeof t=="string"?n.querySelector=t:t&&(t instanceof HTMLElement?n.nodes=[t]:n.nodes=t),await yce(n)},"init"),ret=U(async(e,{lazyLoad:t=!0}={})=>{tS(),t6(...e),t===!1&&await CQe()},"registerExternalDiagrams"),Sce=U(function(){if(zi.startOnLoad){const{startOnLoad:e}=Qc.getConfig();e&&zi.run().catch(t=>je.error("Mermaid failed to initialize",t))}},"contentLoaded");typeof document<"u"&&window.addEventListener("load",Sce,!1);var net=U(function(e){zi.parseError=e},"setParseErrorHandler"),gv=[],jk=!1,Ece=U(async()=>{if(!jk){for(jk=!0;gv.length>0;){const e=gv.shift();if(e)try{await e()}catch(t){je.error("Error executing queue",t)}}jk=!1}},"executeQueue"),aet=U(async(e,t)=>new Promise((r,n)=>{const a=U(()=>new Promise((i,o)=>{Qc.parse(e,t).then(s=>{i(s),r(s)},s=>{var l;je.error("Error parsing",s),(l=zi.parseError)==null||l.call(zi,s),o(s),n(s)})}),"performCall");gv.push(a),Ece().catch(n)}),"parse"),xce=U((e,t,r)=>new Promise((n,a)=>{const i=U(()=>new Promise((o,s)=>{Qc.render(e,t,r).then(l=>{o(l),n(l)},l=>{var u;je.error("Error parsing",l),(u=zi.parseError)==null||u.call(zi,l),s(l),a(l)})}),"performCall");gv.push(i),Ece().catch(a)}),"render"),iet=U(()=>Object.keys(Vc).map(e=>({id:e})),"getRegisteredDiagramsMetadata"),zi={startOnLoad:!0,mermaidAPI:Qc,parse:aet,render:xce,init:tet,run:yce,registerExternalDiagrams:ret,registerLayoutLoaders:_ue,initialize:vce,parseError:void 0,contentLoaded:Sce,setParseErrorHandler:net,detectType:uN,registerIconPacks:FYe,getRegisteredDiagramsMetadata:iet},hU=zi;/*! Check if previously processed *//*!
 * Wait for document loaded before starting the execution
 */const pU=()=>e=>{Ig(e,"text",(t,r,n)=>{if(!n||typeof r!="number")return;const a=t.value,i=/\[\^([^\]]+)\]/g;let o;const s=[];let l=0;for(;(o=i.exec(a))!==null;){const[u,d]=o,p=o.index;p>l&&s.push({type:"text",value:a.slice(l,p)});const f=p+u.length,m=a.slice(f),b=/^\[\^[^\]]+\]/.test(m),y=`<sup><a href="#footnote-${d}" class="footnote-ref">${d}</a></sup>`,S=b?y+"&nbsp;":y;s.push({type:"html",value:S}),l=p+u.length}l<a.length&&s.push({type:"text",value:a.slice(l)}),s.length>1&&n.children.splice(r,1,...s)})};function oet(e,t){if(e==null)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)!==-1)continue;r[n]=e[n]}return r}function set(e,t){if(e==null)return{};var r,n,a=oet(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)r=i[n],t.indexOf(r)===-1&&{}.propertyIsEnumerable.call(e,r)&&(a[r]=e[r])}return a}function X6(e,t){(t==null||t>e.length)&&(t=e.length);for(var r=0,n=Array(t);r<t;r++)n[r]=e[r];return n}function uet(e){if(Array.isArray(e))return X6(e)}function cet(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function det(e,t){if(e){if(typeof e=="string")return X6(e,t);var r={}.toString.call(e).slice(8,-1);return r==="Object"&&e.constructor&&(r=e.constructor.name),r==="Map"||r==="Set"?Array.from(e):r==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r)?X6(e,t):void 0}}function het(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function Z6(e){return uet(e)||cet(e)||det(e)||het()}function sg(e){"@babel/helpers - typeof";return sg=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},sg(e)}function pet(e,t){if(sg(e)!="object"||!e)return e;var r=e[Symbol.toPrimitive];if(r!==void 0){var n=r.call(e,t);if(sg(n)!="object")return n;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function fet(e){var t=pet(e,"string");return sg(t)=="symbol"?t:t+""}function wce(e,t,r){return(t=fet(t))in e?Object.defineProperty(e,t,{value:r,enumerable:!0,configurable:!0,writable:!0}):e[t]=r,e}function Q6(){return Q6=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},Q6.apply(null,arguments)}function fU(e,t){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),r.push.apply(r,n)}return r}function bh(e){for(var t=1;t<arguments.length;t++){var r=arguments[t]!=null?arguments[t]:{};t%2?fU(Object(r),!0).forEach(function(n){wce(e,n,r[n])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):fU(Object(r)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))})}return e}function met(e){var t=e.length;if(t===0||t===1)return e;if(t===2)return[e[0],e[1],"".concat(e[0],".").concat(e[1]),"".concat(e[1],".").concat(e[0])];if(t===3)return[e[0],e[1],e[2],"".concat(e[0],".").concat(e[1]),"".concat(e[0],".").concat(e[2]),"".concat(e[1],".").concat(e[0]),"".concat(e[1],".").concat(e[2]),"".concat(e[2],".").concat(e[0]),"".concat(e[2],".").concat(e[1]),"".concat(e[0],".").concat(e[1],".").concat(e[2]),"".concat(e[0],".").concat(e[2],".").concat(e[1]),"".concat(e[1],".").concat(e[0],".").concat(e[2]),"".concat(e[1],".").concat(e[2],".").concat(e[0]),"".concat(e[2],".").concat(e[0],".").concat(e[1]),"".concat(e[2],".").concat(e[1],".").concat(e[0])];if(t>=4)return[e[0],e[1],e[2],e[3],"".concat(e[0],".").concat(e[1]),"".concat(e[0],".").concat(e[2]),"".concat(e[0],".").concat(e[3]),"".concat(e[1],".").concat(e[0]),"".concat(e[1],".").concat(e[2]),"".concat(e[1],".").concat(e[3]),"".concat(e[2],".").concat(e[0]),"".concat(e[2],".").concat(e[1]),"".concat(e[2],".").concat(e[3]),"".concat(e[3],".").concat(e[0]),"".concat(e[3],".").concat(e[1]),"".concat(e[3],".").concat(e[2]),"".concat(e[0],".").concat(e[1],".").concat(e[2]),"".concat(e[0],".").concat(e[1],".").concat(e[3]),"".concat(e[0],".").concat(e[2],".").concat(e[1]),"".concat(e[0],".").concat(e[2],".").concat(e[3]),"".concat(e[0],".").concat(e[3],".").concat(e[1]),"".concat(e[0],".").concat(e[3],".").concat(e[2]),"".concat(e[1],".").concat(e[0],".").concat(e[2]),"".concat(e[1],".").concat(e[0],".").concat(e[3]),"".concat(e[1],".").concat(e[2],".").concat(e[0]),"".concat(e[1],".").concat(e[2],".").concat(e[3]),"".concat(e[1],".").concat(e[3],".").concat(e[0]),"".concat(e[1],".").concat(e[3],".").concat(e[2]),"".concat(e[2],".").concat(e[0],".").concat(e[1]),"".concat(e[2],".").concat(e[0],".").concat(e[3]),"".concat(e[2],".").concat(e[1],".").concat(e[0]),"".concat(e[2],".").concat(e[1],".").concat(e[3]),"".concat(e[2],".").concat(e[3],".").concat(e[0]),"".concat(e[2],".").concat(e[3],".").concat(e[1]),"".concat(e[3],".").concat(e[0],".").concat(e[1]),"".concat(e[3],".").concat(e[0],".").concat(e[2]),"".concat(e[3],".").concat(e[1],".").concat(e[0]),"".concat(e[3],".").concat(e[1],".").concat(e[2]),"".concat(e[3],".").concat(e[2],".").concat(e[0]),"".concat(e[3],".").concat(e[2],".").concat(e[1]),"".concat(e[0],".").concat(e[1],".").concat(e[2],".").concat(e[3]),"".concat(e[0],".").concat(e[1],".").concat(e[3],".").concat(e[2]),"".concat(e[0],".").concat(e[2],".").concat(e[1],".").concat(e[3]),"".concat(e[0],".").concat(e[2],".").concat(e[3],".").concat(e[1]),"".concat(e[0],".").concat(e[3],".").concat(e[1],".").concat(e[2]),"".concat(e[0],".").concat(e[3],".").concat(e[2],".").concat(e[1]),"".concat(e[1],".").concat(e[0],".").concat(e[2],".").concat(e[3]),"".concat(e[1],".").concat(e[0],".").concat(e[3],".").concat(e[2]),"".concat(e[1],".").concat(e[2],".").concat(e[0],".").concat(e[3]),"".concat(e[1],".").concat(e[2],".").concat(e[3],".").concat(e[0]),"".concat(e[1],".").concat(e[3],".").concat(e[0],".").concat(e[2]),"".concat(e[1],".").concat(e[3],".").concat(e[2],".").concat(e[0]),"".concat(e[2],".").concat(e[0],".").concat(e[1],".").concat(e[3]),"".concat(e[2],".").concat(e[0],".").concat(e[3],".").concat(e[1]),"".concat(e[2],".").concat(e[1],".").concat(e[0],".").concat(e[3]),"".concat(e[2],".").concat(e[1],".").concat(e[3],".").concat(e[0]),"".concat(e[2],".").concat(e[3],".").concat(e[0],".").concat(e[1]),"".concat(e[2],".").concat(e[3],".").concat(e[1],".").concat(e[0]),"".concat(e[3],".").concat(e[0],".").concat(e[1],".").concat(e[2]),"".concat(e[3],".").concat(e[0],".").concat(e[2],".").concat(e[1]),"".concat(e[3],".").concat(e[1],".").concat(e[0],".").concat(e[2]),"".concat(e[3],".").concat(e[1],".").concat(e[2],".").concat(e[0]),"".concat(e[3],".").concat(e[2],".").concat(e[0],".").concat(e[1]),"".concat(e[3],".").concat(e[2],".").concat(e[1],".").concat(e[0])]}var qk={};function get(e){if(e.length===0||e.length===1)return e;var t=e.join(".");return qk[t]||(qk[t]=met(e)),qk[t]}function bet(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},r=arguments.length>2?arguments[2]:void 0,n=e.filter(function(i){return i!=="token"}),a=get(n);return a.reduce(function(i,o){return bh(bh({},i),r[o])},t)}function mU(e){return e.join(" ")}function yet(e,t){var r=0;return function(n){return r+=1,n.map(function(a,i){return kce({node:a,stylesheet:e,useInlineStyles:t,key:"code-segment-".concat(r,"-").concat(i)})})}}function kce(e){var t=e.node,r=e.stylesheet,n=e.style,a=n===void 0?{}:n,i=e.useInlineStyles,o=e.key,s=t.properties,l=t.type,u=t.tagName,d=t.value;if(l==="text")return d;if(u){var p=yet(r,i),f;if(!i)f=bh(bh({},s),{},{className:mU(s.className)});else{var m=Object.keys(r).reduce(function(w,E){return E.split(".").forEach(function(k){w.includes(k)||w.push(k)}),w},[]),b=s.className&&s.className.includes("token")?["token"]:[],y=s.className&&b.concat(s.className.filter(function(w){return!m.includes(w)}));f=bh(bh({},s),{},{className:mU(y)||void 0,style:bet(s.className,Object.assign({},s.style,a),r)})}var S=p(t.children);return $e.createElement(u,Q6({key:o},f),S)}}const vet=(function(e,t){var r=e.listLanguages();return r.indexOf(t)!==-1});var Eet=["language","children","style","customStyle","codeTagProps","useInlineStyles","showLineNumbers","showInlineLineNumbers","startingLineNumber","lineNumberContainerStyle","lineNumberStyle","wrapLines","wrapLongLines","lineProps","renderer","PreTag","CodeTag","code","astGenerator"];function gU(e,t){var r=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter(function(a){return Object.getOwnPropertyDescriptor(e,a).enumerable})),r.push.apply(r,n)}return r}function xu(e){for(var t=1;t<arguments.length;t++){var r=arguments[t]!=null?arguments[t]:{};t%2?gU(Object(r),!0).forEach(function(n){wce(e,n,r[n])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):gU(Object(r)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(r,n))})}return e}var xet=/\n/g;function wet(e){return e.match(xet)}function ket(e){var t=e.lines,r=e.startingLineNumber,n=e.style;return t.map(function(a,i){var o=i+r;return $e.createElement("span",{key:"line-".concat(i),className:"react-syntax-highlighter-line-number",style:typeof n=="function"?n(o):n},"".concat(o,`
`))})}function Cet(e){var t=e.codeString,r=e.codeStyle,n=e.containerStyle,a=n===void 0?{float:"left",paddingRight:"10px"}:n,i=e.numberStyle,o=i===void 0?{}:i,s=e.startingLineNumber;return $e.createElement("code",{style:Object.assign({},r,a)},ket({lines:t.replace(/\n$/,"").split(`
`),style:o,startingLineNumber:s}))}function Tet(e){return"".concat(e.toString().length,".25em")}function Cce(e,t){return{type:"element",tagName:"span",properties:{key:"line-number--".concat(e),className:["comment","linenumber","react-syntax-highlighter-line-number"],style:t},children:[{type:"text",value:e}]}}function Tce(e,t,r){var n={display:"inline-block",minWidth:Tet(r),paddingRight:"1em",textAlign:"right",userSelect:"none"},a=typeof e=="function"?e(t):e,i=xu(xu({},n),a);return i}function Q1(e){var t=e.children,r=e.lineNumber,n=e.lineNumberStyle,a=e.largestLineNumber,i=e.showInlineLineNumbers,o=e.lineProps,s=o===void 0?{}:o,l=e.className,u=l===void 0?[]:l,d=e.showLineNumbers,p=e.wrapLongLines,f=e.wrapLines,m=f===void 0?!1:f,b=m?xu({},typeof s=="function"?s(r):s):{};if(b.className=b.className?[].concat(Z6(b.className.trim().split(/\s+/)),Z6(u)):u,r&&i){var y=Tce(n,r,a);t.unshift(Cce(r,y))}return p&d&&(b.style=xu({display:"flex"},b.style)),{type:"element",tagName:"span",properties:b,children:t}}function Ace(e){for(var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[],r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:[],n=0;n<e.length;n++){var a=e[n];if(a.type==="text")r.push(Q1({children:[a],className:Z6(new Set(t))}));else if(a.children){var i=t.concat(a.properties.className);Ace(a.children,i).forEach(function(o){return r.push(o)})}}return r}function Aet(e,t,r,n,a,i,o,s,l){var u,d=Ace(e.value),p=[],f=-1,m=0;function b(R,C){var F=arguments.length>2&&arguments[2]!==void 0?arguments[2]:[];return Q1({children:R,lineNumber:C,lineNumberStyle:s,largestLineNumber:o,showInlineLineNumbers:a,lineProps:r,className:F,showLineNumbers:n,wrapLongLines:l,wrapLines:t})}function y(R,C){if(n&&C&&a){var F=Tce(s,C,o);R.unshift(Cce(C,F))}return R}function S(R,C){var F=arguments.length>2&&arguments[2]!==void 0?arguments[2]:[];return t||F.length>0?b(R,C,F):y(R,C)}for(var w=function(){var C=d[m],F=C.children[0].value,L=wet(F);if(L){var I=F.split(`
`);I.forEach(function(M,j){var Y=n&&p.length+i,W={type:"text",value:"".concat(M,`
`)};if(j===0){var $=d.slice(f+1,m).concat(Q1({children:[W],className:C.properties.className})),X=S($,Y);p.push(X)}else if(j===I.length-1){var z=d[m+1]&&d[m+1].children&&d[m+1].children[0],H={type:"text",value:"".concat(M)};if(z){var B=Q1({children:[H],className:C.properties.className});d.splice(m+1,0,B)}else{var ee=[H],J=S(ee,Y,C.properties.className);p.push(J)}}else{var K=[W],G=S(K,Y,C.properties.className);p.push(G)}}),f=m}m++};m<d.length;)w();if(f!==d.length-1){var E=d.slice(f+1,d.length);if(E&&E.length){var k=n&&p.length+i,_=S(E,k);p.push(_)}}return t?p:(u=[]).concat.apply(u,p)}function Fet(e){var t=e.rows,r=e.stylesheet,n=e.useInlineStyles;return t.map(function(a,i){return kce({node:a,stylesheet:r,useInlineStyles:n,key:"code-segment-".concat(i)})})}function Fce(e){return e&&typeof e.highlightAuto<"u"}function _et(e){var t=e.astGenerator,r=e.language,n=e.code,a=e.defaultCodeValue;if(Fce(t)){var i=vet(t,r);return r==="text"?{value:a,language:"text"}:i?t.highlight(r,n):t.highlightAuto(n)}try{return r&&r!=="text"?{value:t.highlight(n,r)}:{value:a}}catch{return{value:a}}}function Ret(e,t){return function(n){var a,i,o=n.language,s=n.children,l=n.style,u=l===void 0?t:l,d=n.customStyle,p=d===void 0?{}:d,f=n.codeTagProps,m=f===void 0?{className:o?"language-".concat(o):void 0,style:xu(xu({},u['code[class*="language-"]']),u['code[class*="language-'.concat(o,'"]')])}:f,b=n.useInlineStyles,y=b===void 0?!0:b,S=n.showLineNumbers,w=S===void 0?!1:S,E=n.showInlineLineNumbers,k=E===void 0?!0:E,_=n.startingLineNumber,R=_===void 0?1:_,C=n.lineNumberContainerStyle,F=n.lineNumberStyle,L=F===void 0?{}:F,I=n.wrapLines,M=n.wrapLongLines,j=M===void 0?!1:M,Y=n.lineProps,W=Y===void 0?{}:Y,$=n.renderer,X=n.PreTag,z=X===void 0?"pre":X,H=n.CodeTag,B=H===void 0?"code":H,ee=n.code,J=ee===void 0?(Array.isArray(s)?s[0]:s)||"":ee,K=n.astGenerator,G=set(n,Eet);K=K||e;var P=w?$e.createElement(Cet,{containerStyle:C,codeStyle:m.style||{},numberStyle:L,startingLineNumber:R,codeString:J}):null,ne=u.hljs||u['pre[class*="language-"]']||{backgroundColor:"#fff"},oe=Fce(K)?"hljs":"prismjs",pe=y?Object.assign({},G,{style:Object.assign({},ne,p)}):Object.assign({},G,{className:G.className?"".concat(oe," ").concat(G.className):oe,style:Object.assign({},p)});if(j?m.style=xu({whiteSpace:"pre-wrap"},m.style):m.style=xu({whiteSpace:"pre"},m.style),!K)return $e.createElement(z,pe,P,$e.createElement(B,m,J));(I===void 0&&$||j)&&(I=!0),$=$||Fet;var ae=[{type:"text",value:J}],se=_et({astGenerator:K,language:o,code:J,defaultCodeValue:ae});se.language===null&&(se.value=ae);var Ee=(a=(i=J.match(/\n/g))===null||i===void 0?void 0:i.length)!==null&&a!==void 0?a:0,fe=R+Ee,Me=Aet(se,I,W,w,k,R,fe,L,j);return $e.createElement(z,pe,$e.createElement(B,m,!k&&P,$({rows:Me,stylesheet:u,useInlineStyles:y})))}}const Det=["abap","abnf","actionscript","ada","agda","al","antlr4","apacheconf","apex","apl","applescript","aql","arduino","arff","asciidoc","asm6502","asmatmel","aspnet","autohotkey","autoit","avisynth","avro-idl","bash","basic","batch","bbcode","bicep","birb","bison","bnf","brainfuck","brightscript","bro","bsl","c","cfscript","chaiscript","cil","clike","clojure","cmake","cobol","coffeescript","concurnas","coq","cpp","crystal","csharp","cshtml","csp","css-extras","css","csv","cypher","d","dart","dataweave","dax","dhall","diff","django","dns-zone-file","docker","dot","ebnf","editorconfig","eiffel","ejs","elixir","elm","erb","erlang","etlua","excel-formula","factor","false","firestore-security-rules","flow","fortran","fsharp","ftl","gap","gcode","gdscript","gedcom","gherkin","git","glsl","gml","gn","go-module","go","graphql","groovy","haml","handlebars","haskell","haxe","hcl","hlsl","hoon","hpkp","hsts","http","ichigojam","icon","icu-message-format","idris","iecst","ignore","inform7","ini","io","j","java","javadoc","javadoclike","javascript","javastacktrace","jexl","jolie","jq","js-extras","js-templates","jsdoc","json","json5","jsonp","jsstacktrace","jsx","julia","keepalived","keyman","kotlin","kumir","kusto","latex","latte","less","lilypond","liquid","lisp","livescript","llvm","log","lolcode","lua","magma","makefile","markdown","markup-templating","markup","matlab","maxscript","mel","mermaid","mizar","mongodb","monkey","moonscript","n1ql","n4js","nand2tetris-hdl","naniscript","nasm","neon","nevod","nginx","nim","nix","nsis","objectivec","ocaml","opencl","openqasm","oz","parigp","parser","pascal","pascaligo","pcaxis","peoplecode","perl","php-extras","php","phpdoc","plsql","powerquery","powershell","processing","prolog","promql","properties","protobuf","psl","pug","puppet","pure","purebasic","purescript","python","q","qml","qore","qsharp","r","racket","reason","regex","rego","renpy","rest","rip","roboconf","robotframework","ruby","rust","sas","sass","scala","scheme","scss","shell-session","smali","smalltalk","smarty","sml","solidity","solution-file","soy","sparql","splunk-spl","sqf","sql","squirrel","stan","stylus","swift","systemd","t4-cs","t4-templating","t4-vb","tap","tcl","textile","toml","tremor","tsx","tt2","turtle","twig","typescript","typoscript","unrealscript","uorazor","uri","v","vala","vbnet","velocity","verilog","vhdl","vim","visual-basic","warpscript","wasm","web-idl","wiki","wolfram","wren","xeora","xml-doc","xojo","xquery","yaml","yang","zig"];var Wk,bU;function Net(){if(bU)return Wk;bU=1,Wk=t;var e=Object.prototype.hasOwnProperty;function t(){for(var r={},n=0;n<arguments.length;n++){var a=arguments[n];for(var i in a)e.call(a,i)&&(r[i]=a[i])}return r}return Wk}var Vk,yU;function _ce(){if(yU)return Vk;yU=1,Vk=t;var e=t.prototype;e.space=null,e.normal={},e.property={};function t(r,n,a){this.property=r,this.normal=n,a&&(this.space=a)}return Vk}var Kk,vU;function Let(){if(vU)return Kk;vU=1;var e=Net(),t=_ce();Kk=r;function r(n){for(var a=n.length,i=[],o=[],s=-1,l,u;++s<a;)l=n[s],i.push(l.property),o.push(l.normal),u=l.space;return new t(e.apply(null,i),e.apply(null,o),u)}return Kk}var Yk,SU;function cL(){if(SU)return Yk;SU=1,Yk=e;function e(t){return t.toLowerCase()}return Yk}var Jk,EU;function Rce(){if(EU)return Jk;EU=1,Jk=t;var e=t.prototype;e.space=null,e.attribute=null,e.property=null,e.boolean=!1,e.booleanish=!1,e.overloadedBoolean=!1,e.number=!1,e.commaSeparated=!1,e.spaceSeparated=!1,e.commaOrSpaceSeparated=!1,e.mustUseProperty=!1,e.defined=!1;function t(r,n){this.property=r,this.attribute=n}return Jk}var Js={},xU;function dL(){if(xU)return Js;xU=1;var e=0;Js.boolean=t(),Js.booleanish=t(),Js.overloadedBoolean=t(),Js.number=t(),Js.spaceSeparated=t(),Js.commaSeparated=t(),Js.commaOrSpaceSeparated=t();function t(){return Math.pow(2,++e)}return Js}var Xk,wU;function Dce(){if(wU)return Xk;wU=1;var e=Rce(),t=dL();Xk=a,a.prototype=new e,a.prototype.defined=!0;var r=["boolean","booleanish","overloadedBoolean","number","commaSeparated","spaceSeparated","commaOrSpaceSeparated"],n=r.length;function a(o,s,l,u){var d=-1,p;for(i(this,"space",u),e.call(this,o,s);++d<n;)p=r[d],i(this,p,(l&t[p])===t[p])}function i(o,s,l){l&&(o[s]=l)}return Xk}var Zk,kU;function Jg(){if(kU)return Zk;kU=1;var e=cL(),t=_ce(),r=Dce();Zk=n;function n(a){var i=a.space,o=a.mustUseProperty||[],s=a.attributes||{},l=a.properties,u=a.transform,d={},p={},f,m;for(f in l)m=new r(f,u(s,f),l[f],i),o.indexOf(f)!==-1&&(m.mustUseProperty=!0),d[f]=m,p[e(f)]=f,p[e(m.attribute)]=f;return new t(d,p,i)}return Zk}var Qk,CU;function Met(){if(CU)return Qk;CU=1;var e=Jg();Qk=e({space:"xlink",transform:t,properties:{xLinkActuate:null,xLinkArcRole:null,xLinkHref:null,xLinkRole:null,xLinkShow:null,xLinkTitle:null,xLinkType:null}});function t(r,n){return"xlink:"+n.slice(5).toLowerCase()}return Qk}var eC,TU;function Iet(){if(TU)return eC;TU=1;var e=Jg();eC=e({space:"xml",transform:t,properties:{xmlLang:null,xmlBase:null,xmlSpace:null}});function t(r,n){return"xml:"+n.slice(3).toLowerCase()}return eC}var tC,AU;function Oet(){if(AU)return tC;AU=1,tC=e;function e(t,r){return r in t?t[r]:r}return tC}var rC,FU;function Nce(){if(FU)return rC;FU=1;var e=Oet();rC=t;function t(r,n){return e(r,n.toLowerCase())}return rC}var nC,_U;function Bet(){if(_U)return nC;_U=1;var e=Jg(),t=Nce();return nC=e({space:"xmlns",attributes:{xmlnsxlink:"xmlns:xlink"},transform:t,properties:{xmlns:null,xmlnsXLink:null}}),nC}var aC,RU;function Pet(){if(RU)return aC;RU=1;var e=dL(),t=Jg(),r=e.booleanish,n=e.number,a=e.spaceSeparated;aC=t({transform:i,properties:{ariaActiveDescendant:null,ariaAtomic:r,ariaAutoComplete:null,ariaBusy:r,ariaChecked:r,ariaColCount:n,ariaColIndex:n,ariaColSpan:n,ariaControls:a,ariaCurrent:null,ariaDescribedBy:a,ariaDetails:null,ariaDisabled:r,ariaDropEffect:a,ariaErrorMessage:null,ariaExpanded:r,ariaFlowTo:a,ariaGrabbed:r,ariaHasPopup:null,ariaHidden:r,ariaInvalid:null,ariaKeyShortcuts:null,ariaLabel:null,ariaLabelledBy:a,ariaLevel:n,ariaLive:null,ariaModal:r,ariaMultiLine:r,ariaMultiSelectable:r,ariaOrientation:null,ariaOwns:a,ariaPlaceholder:null,ariaPosInSet:n,ariaPressed:r,ariaReadOnly:r,ariaRelevant:null,ariaRequired:r,ariaRoleDescription:a,ariaRowCount:n,ariaRowIndex:n,ariaRowSpan:n,ariaSelected:r,ariaSetSize:n,ariaSort:null,ariaValueMax:n,ariaValueMin:n,ariaValueNow:n,ariaValueText:null,role:null}});function i(o,s){return s==="role"?s:"aria-"+s.slice(4).toLowerCase()}return aC}var iC,DU;function zet(){if(DU)return iC;DU=1;var e=dL(),t=Jg(),r=Nce(),n=e.boolean,a=e.overloadedBoolean,i=e.booleanish,o=e.number,s=e.spaceSeparated,l=e.commaSeparated;return iC=t({space:"html",attributes:{acceptcharset:"accept-charset",classname:"class",htmlfor:"for",httpequiv:"http-equiv"},transform:r,mustUseProperty:["checked","multiple","muted","selected"],properties:{abbr:null,accept:l,acceptCharset:s,accessKey:s,action:null,allow:null,allowFullScreen:n,allowPaymentRequest:n,allowUserMedia:n,alt:null,as:null,async:n,autoCapitalize:null,autoComplete:s,autoFocus:n,autoPlay:n,capture:n,charSet:null,checked:n,cite:null,className:s,cols:o,colSpan:null,content:null,contentEditable:i,controls:n,controlsList:s,coords:o|l,crossOrigin:null,data:null,dateTime:null,decoding:null,default:n,defer:n,dir:null,dirName:null,disabled:n,download:a,draggable:i,encType:null,enterKeyHint:null,form:null,formAction:null,formEncType:null,formMethod:null,formNoValidate:n,formTarget:null,headers:s,height:o,hidden:n,high:o,href:null,hrefLang:null,htmlFor:s,httpEquiv:s,id:null,imageSizes:null,imageSrcSet:l,inputMode:null,integrity:null,is:null,isMap:n,itemId:null,itemProp:s,itemRef:s,itemScope:n,itemType:s,kind:null,label:null,lang:null,language:null,list:null,loading:null,loop:n,low:o,manifest:null,max:null,maxLength:o,media:null,method:null,min:null,minLength:o,multiple:n,muted:n,name:null,nonce:null,noModule:n,noValidate:n,onAbort:null,onAfterPrint:null,onAuxClick:null,onBeforePrint:null,onBeforeUnload:null,onBlur:null,onCancel:null,onCanPlay:null,onCanPlayThrough:null,onChange:null,onClick:null,onClose:null,onContextMenu:null,onCopy:null,onCueChange:null,onCut:null,onDblClick:null,onDrag:null,onDragEnd:null,onDragEnter:null,onDragExit:null,onDragLeave:null,onDragOver:null,onDragStart:null,onDrop:null,onDurationChange:null,onEmptied:null,onEnded:null,onError:null,onFocus:null,onFormData:null,onHashChange:null,onInput:null,onInvalid:null,onKeyDown:null,onKeyPress:null,onKeyUp:null,onLanguageChange:null,onLoad:null,onLoadedData:null,onLoadedMetadata:null,onLoadEnd:null,onLoadStart:null,onMessage:null,onMessageError:null,onMouseDown:null,onMouseEnter:null,onMouseLeave:null,onMouseMove:null,onMouseOut:null,onMouseOver:null,onMouseUp:null,onOffline:null,onOnline:null,onPageHide:null,onPageShow:null,onPaste:null,onPause:null,onPlay:null,onPlaying:null,onPopState:null,onProgress:null,onRateChange:null,onRejectionHandled:null,onReset:null,onResize:null,onScroll:null,onSecurityPolicyViolation:null,onSeeked:null,onSeeking:null,onSelect:null,onSlotChange:null,onStalled:null,onStorage:null,onSubmit:null,onSuspend:null,onTimeUpdate:null,onToggle:null,onUnhandledRejection:null,onUnload:null,onVolumeChange:null,onWaiting:null,onWheel:null,open:n,optimum:o,pattern:null,ping:s,placeholder:null,playsInline:n,poster:null,preload:null,readOnly:n,referrerPolicy:null,rel:s,required:n,reversed:n,rows:o,rowSpan:o,sandbox:s,scope:null,scoped:n,seamless:n,selected:n,shape:null,size:o,sizes:null,slot:null,span:o,spellCheck:i,src:null,srcDoc:null,srcLang:null,srcSet:l,start:o,step:null,style:null,tabIndex:o,target:null,title:null,translate:null,type:null,typeMustMatch:n,useMap:null,value:i,width:o,wrap:null,align:null,aLink:null,archive:s,axis:null,background:null,bgColor:null,border:o,borderColor:null,bottomMargin:o,cellPadding:null,cellSpacing:null,char:null,charOff:null,classId:null,clear:null,code:null,codeBase:null,codeType:null,color:null,compact:n,declare:n,event:null,face:null,frame:null,frameBorder:null,hSpace:o,leftMargin:o,link:null,longDesc:null,lowSrc:null,marginHeight:o,marginWidth:o,noResize:n,noHref:n,noShade:n,noWrap:n,object:null,profile:null,prompt:null,rev:null,rightMargin:o,rules:null,scheme:null,scrolling:i,standby:null,summary:null,text:null,topMargin:o,valueType:null,version:null,vAlign:null,vLink:null,vSpace:o,allowTransparency:null,autoCorrect:null,autoSave:null,disablePictureInPicture:n,disableRemotePlayback:n,prefix:null,property:null,results:o,security:null,unselectable:null}}),iC}var oC,NU;function Het(){if(NU)return oC;NU=1;var e=Let(),t=Met(),r=Iet(),n=Bet(),a=Pet(),i=zet();return oC=e([r,t,n,a,i]),oC}var sC,LU;function Uet(){if(LU)return sC;LU=1;var e=cL(),t=Dce(),r=Rce(),n="data";sC=s;var a=/^data[-\w.:]+$/i,i=/-[a-z]/g,o=/[A-Z]/g;function s(f,m){var b=e(m),y=m,S=r;return b in f.normal?f.property[f.normal[b]]:(b.length>4&&b.slice(0,4)===n&&a.test(m)&&(m.charAt(4)==="-"?y=l(m):m=u(m),S=t),new S(y,m))}function l(f){var m=f.slice(5).replace(i,p);return n+m.charAt(0).toUpperCase()+m.slice(1)}function u(f){var m=f.slice(4);return i.test(m)?f:(m=m.replace(o,d),m.charAt(0)!=="-"&&(m="-"+m),n+m)}function d(f){return"-"+f.toLowerCase()}function p(f){return f.charAt(1).toUpperCase()}return sC}var lC,MU;function $et(){if(MU)return lC;MU=1,lC=t;var e=/[#.]/g;function t(r,n){for(var a=r||"",i=n||"div",o={},s=0,l,u,d;s<a.length;)e.lastIndex=s,d=e.exec(a),l=a.slice(s,d?d.index:a.length),l&&(u?u==="#"?o.id=l:o.className?o.className.push(l):o.className=[l]:i=l,s+=l.length),d&&(u=d[0],s++);return{type:"element",tagName:i,properties:o,children:[]}}return lC}var h1={},IU;function Get(){if(IU)return h1;IU=1,h1.parse=n,h1.stringify=a;var e="",t=" ",r=/[ \t\n\r\f]+/g;function n(i){var o=String(i||e).trim();return o===e?[]:o.split(r)}function a(i){return i.join(t).trim()}return h1}var p1={},OU;function jet(){if(OU)return p1;OU=1,p1.parse=n,p1.stringify=a;var e=",",t=" ",r="";function n(i){for(var o=[],s=String(i||r),l=s.indexOf(e),u=0,d=!1,p;!d;)l===-1&&(l=s.length,d=!0),p=s.slice(u,l).trim(),(p||!d)&&o.push(p),u=l+1,l=s.indexOf(e,u);return o}function a(i,o){var s=o||{},l=s.padLeft===!1?r:t,u=s.padRight?t:r;return i[i.length-1]===r&&(i=i.concat(r)),i.join(u+e+l).trim()}return p1}var uC,BU;function qet(){if(BU)return uC;BU=1;var e=Uet(),t=cL(),r=$et(),n=Get().parse,a=jet().parse;uC=o;var i={}.hasOwnProperty;function o(b,y,S){var w=S?m(S):null;return E;function E(_,R){var C=r(_,y),F=Array.prototype.slice.call(arguments,2),L=C.tagName.toLowerCase(),I;if(C.tagName=w&&i.call(w,L)?w[L]:L,R&&s(R,C)&&(F.unshift(R),R=null),R)for(I in R)k(C.properties,I,R[I]);return u(C.children,F),C.tagName==="template"&&(C.content={type:"root",children:C.children},C.children=[]),C}function k(_,R,C){var F,L,I;C==null||C!==C||(F=e(b,R),L=F.property,I=C,typeof I=="string"&&(F.spaceSeparated?I=n(I):F.commaSeparated?I=a(I):F.commaOrSpaceSeparated&&(I=n(a(I).join(" ")))),L==="style"&&typeof C!="string"&&(I=f(I)),L==="className"&&_.className&&(I=_.className.concat(I)),_[L]=d(F,L,I))}}function s(b,y){return typeof b=="string"||"length"in b||l(y.tagName,b)}function l(b,y){var S=y.type;return b==="input"||!S||typeof S!="string"?!1:typeof y.children=="object"&&"length"in y.children?!0:(S=S.toLowerCase(),b==="button"?S!=="menu"&&S!=="submit"&&S!=="reset"&&S!=="button":"value"in y)}function u(b,y){var S,w;if(typeof y=="string"||typeof y=="number"){b.push({type:"text",value:String(y)});return}if(typeof y=="object"&&"length"in y){for(S=-1,w=y.length;++S<w;)u(b,y[S]);return}if(typeof y!="object"||!("type"in y))throw new Error("Expected node, nodes, or string, got `"+y+"`");b.push(y)}function d(b,y,S){var w,E,k;if(typeof S!="object"||!("length"in S))return p(b,y,S);for(E=S.length,w=-1,k=[];++w<E;)k[w]=p(b,y,S[w]);return k}function p(b,y,S){var w=S;return b.number||b.positiveNumber?!isNaN(w)&&w!==""&&(w=Number(w)):(b.boolean||b.overloadedBoolean)&&typeof w=="string"&&(w===""||t(S)===t(y))&&(w=!0),w}function f(b){var y=[],S;for(S in b)y.push([S,b[S]].join(": "));return y.join("; ")}function m(b){for(var y=b.length,S=-1,w={},E;++S<y;)E=b[S],w[E.toLowerCase()]=E;return w}return uC}var cC,PU;function Wet(){if(PU)return cC;PU=1;var e=Het(),t=qet(),r=t(e,"div");return r.displayName="html",cC=r,cC}var dC,zU;function Vet(){return zU||(zU=1,dC=Wet()),dC}const Ket="Æ",Yet="&",Jet="Á",Xet="Â",Zet="À",Qet="Å",ett="Ã",ttt="Ä",rtt="©",ntt="Ç",att="Ð",itt="É",ott="Ê",stt="È",ltt="Ë",utt=">",ctt="Í",dtt="Î",htt="Ì",ptt="Ï",ftt="<",mtt="Ñ",gtt="Ó",btt="Ô",ytt="Ò",vtt="Ø",Stt="Õ",Ett="Ö",xtt='"',wtt="®",ktt="Þ",Ctt="Ú",Ttt="Û",Att="Ù",Ftt="Ü",_tt="Ý",Rtt="á",Dtt="â",Ntt="´",Ltt="æ",Mtt="à",Itt="&",Ott="å",Btt="ã",Ptt="ä",ztt="¦",Htt="ç",Utt="¸",$tt="¢",Gtt="©",jtt="¤",qtt="°",Wtt="÷",Vtt="é",Ktt="ê",Ytt="è",Jtt="ð",Xtt="ë",Ztt="½",Qtt="¼",ert="¾",trt=">",rrt="í",nrt="î",art="¡",irt="ì",ort="¿",srt="ï",lrt="«",urt="<",crt="¯",drt="µ",hrt="·",prt=" ",frt="¬",mrt="ñ",grt="ó",brt="ô",yrt="ò",vrt="ª",Srt="º",Ert="ø",xrt="õ",wrt="ö",krt="¶",Crt="±",Trt="£",Art='"',Frt="»",_rt="®",Rrt="§",Drt="­",Nrt="¹",Lrt="²",Mrt="³",Irt="ß",Ort="þ",Brt="×",Prt="ú",zrt="û",Hrt="ù",Urt="¨",$rt="ü",Grt="ý",jrt="¥",qrt="ÿ",Wrt={AElig:Ket,AMP:Yet,Aacute:Jet,Acirc:Xet,Agrave:Zet,Aring:Qet,Atilde:ett,Auml:ttt,COPY:rtt,Ccedil:ntt,ETH:att,Eacute:itt,Ecirc:ott,Egrave:stt,Euml:ltt,GT:utt,Iacute:ctt,Icirc:dtt,Igrave:htt,Iuml:ptt,LT:ftt,Ntilde:mtt,Oacute:gtt,Ocirc:btt,Ograve:ytt,Oslash:vtt,Otilde:Stt,Ouml:Ett,QUOT:xtt,REG:wtt,THORN:ktt,Uacute:Ctt,Ucirc:Ttt,Ugrave:Att,Uuml:Ftt,Yacute:_tt,aacute:Rtt,acirc:Dtt,acute:Ntt,aelig:Ltt,agrave:Mtt,amp:Itt,aring:Ott,atilde:Btt,auml:Ptt,brvbar:ztt,ccedil:Htt,cedil:Utt,cent:$tt,copy:Gtt,curren:jtt,deg:qtt,divide:Wtt,eacute:Vtt,ecirc:Ktt,egrave:Ytt,eth:Jtt,euml:Xtt,frac12:Ztt,frac14:Qtt,frac34:ert,gt:trt,iacute:rrt,icirc:nrt,iexcl:art,igrave:irt,iquest:ort,iuml:srt,laquo:lrt,lt:urt,macr:crt,micro:drt,middot:hrt,nbsp:prt,not:frt,ntilde:mrt,oacute:grt,ocirc:brt,ograve:yrt,ordf:vrt,ordm:Srt,oslash:Ert,otilde:xrt,ouml:wrt,para:krt,plusmn:Crt,pound:Trt,quot:Art,raquo:Frt,reg:_rt,sect:Rrt,shy:Drt,sup1:Nrt,sup2:Lrt,sup3:Mrt,szlig:Irt,thorn:Ort,times:Brt,uacute:Prt,ucirc:zrt,ugrave:Hrt,uml:Urt,uuml:$rt,yacute:Grt,yen:jrt,yuml:qrt},Vrt={0:"�",128:"€",130:"‚",131:"ƒ",132:"„",133:"…",134:"†",135:"‡",136:"ˆ",137:"‰",138:"Š",139:"‹",140:"Œ",142:"Ž",145:"‘",146:"’",147:"“",148:"”",149:"•",150:"–",151:"—",152:"˜",153:"™",154:"š",155:"›",156:"œ",158:"ž",159:"Ÿ"};var hC,HU;function Lce(){if(HU)return hC;HU=1,hC=e;function e(t){var r=typeof t=="string"?t.charCodeAt(0):t;return r>=48&&r<=57}return hC}var pC,UU;function Krt(){if(UU)return pC;UU=1,pC=e;function e(t){var r=typeof t=="string"?t.charCodeAt(0):t;return r>=97&&r<=102||r>=65&&r<=70||r>=48&&r<=57}return pC}var fC,$U;function Yrt(){if($U)return fC;$U=1,fC=e;function e(t){var r=typeof t=="string"?t.charCodeAt(0):t;return r>=97&&r<=122||r>=65&&r<=90}return fC}var mC,GU;function Jrt(){if(GU)return mC;GU=1;var e=Yrt(),t=Lce();mC=r;function r(n){return e(n)||t(n)}return mC}var gC,jU;function Xrt(){if(jU)return gC;jU=1;var e,t=59;gC=r;function r(n){var a="&"+n+";",i;return e=e||document.createElement("i"),e.innerHTML=a,i=e.textContent,i.charCodeAt(i.length-1)===t&&n!=="semi"||i===a?!1:i}return gC}var bC,qU;function Zrt(){if(qU)return bC;qU=1;var e=Wrt,t=Vrt,r=Lce(),n=Krt(),a=Jrt(),i=Xrt();bC=ee;var o={}.hasOwnProperty,s=String.fromCharCode,l=Function.prototype,u={warning:null,reference:null,text:null,warningContext:null,referenceContext:null,textContext:null,position:{},additional:null,attribute:!1,nonTerminated:!0},d=9,p=10,f=12,m=32,b=38,y=59,S=60,w=61,E=35,k=88,_=120,R=65533,C="named",F="hexadecimal",L="decimal",I={};I[F]=16,I[L]=10;var M={};M[C]=a,M[L]=r,M[F]=n;var j=1,Y=2,W=3,$=4,X=5,z=6,H=7,B={};B[j]="Named character references must be terminated by a semicolon",B[Y]="Numeric character references must be terminated by a semicolon",B[W]="Named character references cannot be empty",B[$]="Numeric character references cannot be empty",B[X]="Named character references must be known",B[z]="Numeric character references cannot be disallowed",B[H]="Numeric character references cannot be outside the permissible Unicode range";function ee(P,ne){var oe={},pe,ae;ne||(ne={});for(ae in u)pe=ne[ae],oe[ae]=pe??u[ae];return(oe.position.indent||oe.position.start)&&(oe.indent=oe.position.indent||[],oe.position=oe.position.start),J(P,oe)}function J(P,ne){var oe=ne.additional,pe=ne.nonTerminated,ae=ne.text,se=ne.reference,Ee=ne.warning,fe=ne.textContext,Me=ne.referenceContext,Oe=ne.warningContext,xe=ne.position,Te=ne.indent||[],Ne=P.length,he=0,Ze=-1,Et=xe.column||1,qe=xe.line||1,Ye="",we=[],Qt,or,Pe,rt,ge,Re,We,ht,xt,sr,ar,fr,Mt,lr,qt,zr,nn,dn,Ce;for(typeof oe=="string"&&(oe=oe.charCodeAt(0)),zr=st(),ht=Ee?ft:l,he--,Ne++;++he<Ne;)if(ge===p&&(Et=Te[Ze]||1),ge=P.charCodeAt(he),ge===b){if(We=P.charCodeAt(he+1),We===d||We===p||We===f||We===m||We===b||We===S||We!==We||oe&&We===oe){Ye+=s(ge),Et++;continue}for(Mt=he+1,fr=Mt,Ce=Mt,We===E?(Ce=++fr,We=P.charCodeAt(Ce),We===k||We===_?(lr=F,Ce=++fr):lr=L):lr=C,Qt="",ar="",rt="",qt=M[lr],Ce--;++Ce<Ne&&(We=P.charCodeAt(Ce),!!qt(We));)rt+=s(We),lr===C&&o.call(e,rt)&&(Qt=rt,ar=e[rt]);Pe=P.charCodeAt(Ce)===y,Pe&&(Ce++,or=lr===C?i(rt):!1,or&&(Qt=rt,ar=or)),dn=1+Ce-Mt,!Pe&&!pe||(rt?lr===C?(Pe&&!ar?ht(X,1):(Qt!==rt&&(Ce=fr+Qt.length,dn=1+Ce-fr,Pe=!1),Pe||(xt=Qt?j:W,ne.attribute?(We=P.charCodeAt(Ce),We===w?(ht(xt,dn),ar=null):a(We)?ar=null:ht(xt,dn)):ht(xt,dn))),Re=ar):(Pe||ht(Y,dn),Re=parseInt(rt,I[lr]),K(Re)?(ht(H,dn),Re=s(R)):Re in t?(ht(z,dn),Re=t[Re]):(sr="",G(Re)&&ht(z,dn),Re>65535&&(Re-=65536,sr+=s(Re>>>10|55296),Re=56320|Re&1023),Re=sr+s(Re))):lr!==C&&ht($,dn)),Re?(vt(),zr=st(),he=Ce-1,Et+=Ce-Mt+1,we.push(Re),nn=st(),nn.offset++,se&&se.call(Me,Re,{start:zr,end:nn},P.slice(Mt-1,Ce)),zr=nn):(rt=P.slice(Mt-1,Ce),Ye+=rt,Et+=rt.length,he=Ce-1)}else ge===10&&(qe++,Ze++,Et=0),ge===ge?(Ye+=s(ge),Et++):vt();return we.join("");function st(){return{line:qe,column:Et,offset:he+(xe.offset||0)}}function ft(yr,Qe){var zt=st();zt.column+=Qe,zt.offset+=Qe,Ee.call(Oe,B[yr],zt,yr)}function vt(){Ye&&(we.push(Ye),ae&&ae.call(fe,Ye,{start:zr,end:st()}),Ye="")}}function K(P){return P>=55296&&P<=57343||P>1114111}function G(P){return P>=1&&P<=8||P===11||P>=13&&P<=31||P>=127&&P<=159||P>=64976&&P<=65007||(P&65535)===65535||(P&65535)===65534}return bC}var yC={exports:{}},WU;function Qrt(){return WU||(WU=1,(function(e){var t=typeof window<"u"?window:typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope?self:{};/**
 * Prism: Lightweight, robust, elegant syntax highlighting
 *
 * @license MIT <https://opensource.org/licenses/MIT>
 * @author Lea Verou <https://lea.verou.me>
 * @namespace
 * @public
 */var r=(function(n){var a=/(?:^|\s)lang(?:uage)?-([\w-]+)(?=\s|$)/i,i=0,o={},s={manual:n.Prism&&n.Prism.manual,disableWorkerMessageHandler:n.Prism&&n.Prism.disableWorkerMessageHandler,util:{encode:function E(k){return k instanceof l?new l(k.type,E(k.content),k.alias):Array.isArray(k)?k.map(E):k.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/\u00a0/g," ")},type:function(E){return Object.prototype.toString.call(E).slice(8,-1)},objId:function(E){return E.__id||Object.defineProperty(E,"__id",{value:++i}),E.__id},clone:function E(k,_){_=_||{};var R,C;switch(s.util.type(k)){case"Object":if(C=s.util.objId(k),_[C])return _[C];R={},_[C]=R;for(var F in k)k.hasOwnProperty(F)&&(R[F]=E(k[F],_));return R;case"Array":return C=s.util.objId(k),_[C]?_[C]:(R=[],_[C]=R,k.forEach(function(L,I){R[I]=E(L,_)}),R);default:return k}},getLanguage:function(E){for(;E;){var k=a.exec(E.className);if(k)return k[1].toLowerCase();E=E.parentElement}return"none"},setLanguage:function(E,k){E.className=E.className.replace(RegExp(a,"gi"),""),E.classList.add("language-"+k)},currentScript:function(){if(typeof document>"u")return null;if("currentScript"in document)return document.currentScript;try{throw new Error}catch(R){var E=(/at [^(\r\n]*\((.*):[^:]+:[^:]+\)$/i.exec(R.stack)||[])[1];if(E){var k=document.getElementsByTagName("script");for(var _ in k)if(k[_].src==E)return k[_]}return null}},isActive:function(E,k,_){for(var R="no-"+k;E;){var C=E.classList;if(C.contains(k))return!0;if(C.contains(R))return!1;E=E.parentElement}return!!_}},languages:{plain:o,plaintext:o,text:o,txt:o,extend:function(E,k){var _=s.util.clone(s.languages[E]);for(var R in k)_[R]=k[R];return _},insertBefore:function(E,k,_,R){R=R||s.languages;var C=R[E],F={};for(var L in C)if(C.hasOwnProperty(L)){if(L==k)for(var I in _)_.hasOwnProperty(I)&&(F[I]=_[I]);_.hasOwnProperty(L)||(F[L]=C[L])}var M=R[E];return R[E]=F,s.languages.DFS(s.languages,function(j,Y){Y===M&&j!=E&&(this[j]=F)}),F},DFS:function E(k,_,R,C){C=C||{};var F=s.util.objId;for(var L in k)if(k.hasOwnProperty(L)){_.call(k,L,k[L],R||L);var I=k[L],M=s.util.type(I);M==="Object"&&!C[F(I)]?(C[F(I)]=!0,E(I,_,null,C)):M==="Array"&&!C[F(I)]&&(C[F(I)]=!0,E(I,_,L,C))}}},plugins:{},highlightAll:function(E,k){s.highlightAllUnder(document,E,k)},highlightAllUnder:function(E,k,_){var R={callback:_,container:E,selector:'code[class*="language-"], [class*="language-"] code, code[class*="lang-"], [class*="lang-"] code'};s.hooks.run("before-highlightall",R),R.elements=Array.prototype.slice.apply(R.container.querySelectorAll(R.selector)),s.hooks.run("before-all-elements-highlight",R);for(var C=0,F;F=R.elements[C++];)s.highlightElement(F,k===!0,R.callback)},highlightElement:function(E,k,_){var R=s.util.getLanguage(E),C=s.languages[R];s.util.setLanguage(E,R);var F=E.parentElement;F&&F.nodeName.toLowerCase()==="pre"&&s.util.setLanguage(F,R);var L=E.textContent,I={element:E,language:R,grammar:C,code:L};function M(Y){I.highlightedCode=Y,s.hooks.run("before-insert",I),I.element.innerHTML=I.highlightedCode,s.hooks.run("after-highlight",I),s.hooks.run("complete",I),_&&_.call(I.element)}if(s.hooks.run("before-sanity-check",I),F=I.element.parentElement,F&&F.nodeName.toLowerCase()==="pre"&&!F.hasAttribute("tabindex")&&F.setAttribute("tabindex","0"),!I.code){s.hooks.run("complete",I),_&&_.call(I.element);return}if(s.hooks.run("before-highlight",I),!I.grammar){M(s.util.encode(I.code));return}if(k&&n.Worker){var j=new Worker(s.filename);j.onmessage=function(Y){M(Y.data)},j.postMessage(JSON.stringify({language:I.language,code:I.code,immediateClose:!0}))}else M(s.highlight(I.code,I.grammar,I.language))},highlight:function(E,k,_){var R={code:E,grammar:k,language:_};if(s.hooks.run("before-tokenize",R),!R.grammar)throw new Error('The language "'+R.language+'" has no grammar.');return R.tokens=s.tokenize(R.code,R.grammar),s.hooks.run("after-tokenize",R),l.stringify(s.util.encode(R.tokens),R.language)},tokenize:function(E,k){var _=k.rest;if(_){for(var R in _)k[R]=_[R];delete k.rest}var C=new p;return f(C,C.head,E),d(E,C,k,C.head,0),b(C)},hooks:{all:{},add:function(E,k){var _=s.hooks.all;_[E]=_[E]||[],_[E].push(k)},run:function(E,k){var _=s.hooks.all[E];if(!(!_||!_.length))for(var R=0,C;C=_[R++];)C(k)}},Token:l};n.Prism=s;function l(E,k,_,R){this.type=E,this.content=k,this.alias=_,this.length=(R||"").length|0}l.stringify=function E(k,_){if(typeof k=="string")return k;if(Array.isArray(k)){var R="";return k.forEach(function(M){R+=E(M,_)}),R}var C={type:k.type,content:E(k.content,_),tag:"span",classes:["token",k.type],attributes:{},language:_},F=k.alias;F&&(Array.isArray(F)?Array.prototype.push.apply(C.classes,F):C.classes.push(F)),s.hooks.run("wrap",C);var L="";for(var I in C.attributes)L+=" "+I+'="'+(C.attributes[I]||"").replace(/"/g,"&quot;")+'"';return"<"+C.tag+' class="'+C.classes.join(" ")+'"'+L+">"+C.content+"</"+C.tag+">"};function u(E,k,_,R){E.lastIndex=k;var C=E.exec(_);if(C&&R&&C[1]){var F=C[1].length;C.index+=F,C[0]=C[0].slice(F)}return C}function d(E,k,_,R,C,F){for(var L in _)if(!(!_.hasOwnProperty(L)||!_[L])){var I=_[L];I=Array.isArray(I)?I:[I];for(var M=0;M<I.length;++M){if(F&&F.cause==L+","+M)return;var j=I[M],Y=j.inside,W=!!j.lookbehind,$=!!j.greedy,X=j.alias;if($&&!j.pattern.global){var z=j.pattern.toString().match(/[imsuy]*$/)[0];j.pattern=RegExp(j.pattern.source,z+"g")}for(var H=j.pattern||j,B=R.next,ee=C;B!==k.tail&&!(F&&ee>=F.reach);ee+=B.value.length,B=B.next){var J=B.value;if(k.length>E.length)return;if(!(J instanceof l)){var K=1,G;if($){if(G=u(H,ee,E,W),!G||G.index>=E.length)break;var pe=G.index,P=G.index+G[0].length,ne=ee;for(ne+=B.value.length;pe>=ne;)B=B.next,ne+=B.value.length;if(ne-=B.value.length,ee=ne,B.value instanceof l)continue;for(var oe=B;oe!==k.tail&&(ne<P||typeof oe.value=="string");oe=oe.next)K++,ne+=oe.value.length;K--,J=E.slice(ee,ne),G.index-=ee}else if(G=u(H,0,J,W),!G)continue;var pe=G.index,ae=G[0],se=J.slice(0,pe),Ee=J.slice(pe+ae.length),fe=ee+J.length;F&&fe>F.reach&&(F.reach=fe);var Me=B.prev;se&&(Me=f(k,Me,se),ee+=se.length),m(k,Me,K);var Oe=new l(L,Y?s.tokenize(ae,Y):ae,X,ae);if(B=f(k,Me,Oe),Ee&&f(k,B,Ee),K>1){var xe={cause:L+","+M,reach:fe};d(E,k,_,B.prev,ee,xe),F&&xe.reach>F.reach&&(F.reach=xe.reach)}}}}}}function p(){var E={value:null,prev:null,next:null},k={value:null,prev:E,next:null};E.next=k,this.head=E,this.tail=k,this.length=0}function f(E,k,_){var R=k.next,C={value:_,prev:k,next:R};return k.next=C,R.prev=C,E.length++,C}function m(E,k,_){for(var R=k.next,C=0;C<_&&R!==E.tail;C++)R=R.next;k.next=R,R.prev=k,E.length-=C}function b(E){for(var k=[],_=E.head.next;_!==E.tail;)k.push(_.value),_=_.next;return k}if(!n.document)return n.addEventListener&&(s.disableWorkerMessageHandler||n.addEventListener("message",function(E){var k=JSON.parse(E.data),_=k.language,R=k.code,C=k.immediateClose;n.postMessage(s.highlight(R,s.languages[_],_)),C&&n.close()},!1)),s;var y=s.util.currentScript();y&&(s.filename=y.src,y.hasAttribute("data-manual")&&(s.manual=!0));function S(){s.manual||s.highlightAll()}if(!s.manual){var w=document.readyState;w==="loading"||w==="interactive"&&y&&y.defer?document.addEventListener("DOMContentLoaded",S):window.requestAnimationFrame?window.requestAnimationFrame(S):window.setTimeout(S,16)}return s})(t);e.exports&&(e.exports=r),typeof ey<"u"&&(ey.Prism=r)})(yC)),yC.exports}var vC,VU;function ent(){if(VU)return vC;VU=1,vC=e,e.displayName="markup",e.aliases=["html","mathml","svg","xml","ssml","atom","rss"];function e(t){t.languages.markup={comment:{pattern:/<!--(?:(?!<!--)[\s\S])*?-->/,greedy:!0},prolog:{pattern:/<\?[\s\S]+?\?>/,greedy:!0},doctype:{pattern:/<!DOCTYPE(?:[^>"'[\]]|"[^"]*"|'[^']*')+(?:\[(?:[^<"'\]]|"[^"]*"|'[^']*'|<(?!!--)|<!--(?:[^-]|-(?!->))*-->)*\]\s*)?>/i,greedy:!0,inside:{"internal-subset":{pattern:/(^[^\[]*\[)[\s\S]+(?=\]>$)/,lookbehind:!0,greedy:!0,inside:null},string:{pattern:/"[^"]*"|'[^']*'/,greedy:!0},punctuation:/^<!|>$|[[\]]/,"doctype-tag":/^DOCTYPE/i,name:/[^\s<>'"]+/}},cdata:{pattern:/<!\[CDATA\[[\s\S]*?\]\]>/i,greedy:!0},tag:{pattern:/<\/?(?!\d)[^\s>\/=$<%]+(?:\s(?:\s*[^\s>\/=]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))|(?=[\s/>])))+)?\s*\/?>/,greedy:!0,inside:{tag:{pattern:/^<\/?[^\s>\/]+/,inside:{punctuation:/^<\/?/,namespace:/^[^\s>\/:]+:/}},"special-attr":[],"attr-value":{pattern:/=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+)/,inside:{punctuation:[{pattern:/^=/,alias:"attr-equals"},/"|'/]}},punctuation:/\/?>/,"attr-name":{pattern:/[^\s>\/]+/,inside:{namespace:/^[^\s>\/:]+:/}}}},entity:[{pattern:/&[\da-z]{1,8};/i,alias:"named-entity"},/&#x?[\da-f]{1,8};/i]},t.languages.markup.tag.inside["attr-value"].inside.entity=t.languages.markup.entity,t.languages.markup.doctype.inside["internal-subset"].inside=t.languages.markup,t.hooks.add("wrap",function(r){r.type==="entity"&&(r.attributes.title=r.content.value.replace(/&amp;/,"&"))}),Object.defineProperty(t.languages.markup.tag,"addInlined",{value:function(n,a){var i={};i["language-"+a]={pattern:/(^<!\[CDATA\[)[\s\S]+?(?=\]\]>$)/i,lookbehind:!0,inside:t.languages[a]},i.cdata=/^<!\[CDATA\[|\]\]>$/i;var o={"included-cdata":{pattern:/<!\[CDATA\[[\s\S]*?\]\]>/i,inside:i}};o["language-"+a]={pattern:/[\s\S]+/,inside:t.languages[a]};var s={};s[n]={pattern:RegExp(/(<__[^>]*>)(?:<!\[CDATA\[(?:[^\]]|\](?!\]>))*\]\]>|(?!<!\[CDATA\[)[\s\S])*?(?=<\/__>)/.source.replace(/__/g,function(){return n}),"i"),lookbehind:!0,greedy:!0,inside:o},t.languages.insertBefore("markup","cdata",s)}}),Object.defineProperty(t.languages.markup.tag,"addAttribute",{value:function(r,n){t.languages.markup.tag.inside["special-attr"].push({pattern:RegExp(/(^|["'\s])/.source+"(?:"+r+")"+/\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))/.source,"i"),lookbehind:!0,inside:{"attr-name":/^[^\s=]+/,"attr-value":{pattern:/=[\s\S]+/,inside:{value:{pattern:/(^=\s*(["']|(?!["'])))\S[\s\S]*(?=\2$)/,lookbehind:!0,alias:[n,"language-"+n],inside:t.languages[n]},punctuation:[{pattern:/^=/,alias:"attr-equals"},/"|'/]}}}})}}),t.languages.html=t.languages.markup,t.languages.mathml=t.languages.markup,t.languages.svg=t.languages.markup,t.languages.xml=t.languages.extend("markup",{}),t.languages.ssml=t.languages.xml,t.languages.atom=t.languages.xml,t.languages.rss=t.languages.xml}return vC}var SC,KU;function tnt(){if(KU)return SC;KU=1,SC=e,e.displayName="css",e.aliases=[];function e(t){(function(r){var n=/(?:"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"|'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n])*')/;r.languages.css={comment:/\/\*[\s\S]*?\*\//,atrule:{pattern:/@[\w-](?:[^;{\s]|\s+(?![\s{]))*(?:;|(?=\s*\{))/,inside:{rule:/^@[\w-]+/,"selector-function-argument":{pattern:/(\bselector\s*\(\s*(?![\s)]))(?:[^()\s]|\s+(?![\s)])|\((?:[^()]|\([^()]*\))*\))+(?=\s*\))/,lookbehind:!0,alias:"selector"},keyword:{pattern:/(^|[^\w-])(?:and|not|only|or)(?![\w-])/,lookbehind:!0}}},url:{pattern:RegExp("\\burl\\((?:"+n.source+"|"+/(?:[^\\\r\n()"']|\\[\s\S])*/.source+")\\)","i"),greedy:!0,inside:{function:/^url/i,punctuation:/^\(|\)$/,string:{pattern:RegExp("^"+n.source+"$"),alias:"url"}}},selector:{pattern:RegExp(`(^|[{}\\s])[^{}\\s](?:[^{};"'\\s]|\\s+(?![\\s{])|`+n.source+")*(?=\\s*\\{)"),lookbehind:!0},string:{pattern:n,greedy:!0},property:{pattern:/(^|[^-\w\xA0-\uFFFF])(?!\s)[-_a-z\xA0-\uFFFF](?:(?!\s)[-\w\xA0-\uFFFF])*(?=\s*:)/i,lookbehind:!0},important:/!important\b/i,function:{pattern:/(^|[^-a-z0-9])[-a-z0-9]+(?=\()/i,lookbehind:!0},punctuation:/[(){};:,]/},r.languages.css.atrule.inside.rest=r.languages.css;var a=r.languages.markup;a&&(a.tag.addInlined("style","css"),a.tag.addAttribute("style","css"))})(t)}return SC}var EC,YU;function rnt(){if(YU)return EC;YU=1,EC=e,e.displayName="clike",e.aliases=[];function e(t){t.languages.clike={comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],string:{pattern:/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0},"class-name":{pattern:/(\b(?:class|extends|implements|instanceof|interface|new|trait)\s+|\bcatch\s+\()[\w.\\]+/i,lookbehind:!0,inside:{punctuation:/[.\\]/}},keyword:/\b(?:break|catch|continue|do|else|finally|for|function|if|in|instanceof|new|null|return|throw|try|while)\b/,boolean:/\b(?:false|true)\b/,function:/\b\w+(?=\()/,number:/\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,operator:/[<>]=?|[!=]=?=?|--?|\+\+?|&&?|\|\|?|[?*/~^%]/,punctuation:/[{}[\];(),.:]/}}return EC}var xC,JU;function nnt(){if(JU)return xC;JU=1,xC=e,e.displayName="javascript",e.aliases=["js"];function e(t){t.languages.javascript=t.languages.extend("clike",{"class-name":[t.languages.clike["class-name"],{pattern:/(^|[^$\w\xA0-\uFFFF])(?!\s)[_$A-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\.(?:constructor|prototype))/,lookbehind:!0}],keyword:[{pattern:/((?:^|\})\s*)catch\b/,lookbehind:!0},{pattern:/(^|[^.]|\.\.\.\s*)\b(?:as|assert(?=\s*\{)|async(?=\s*(?:function\b|\(|[$\w\xA0-\uFFFF]|$))|await|break|case|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally(?=\s*(?:\{|$))|for|from(?=\s*(?:['"]|$))|function|(?:get|set)(?=\s*(?:[#\[$\w\xA0-\uFFFF]|$))|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)\b/,lookbehind:!0}],function:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*(?:\.\s*(?:apply|bind|call)\s*)?\()/,number:{pattern:RegExp(/(^|[^\w$])/.source+"(?:"+(/NaN|Infinity/.source+"|"+/0[bB][01]+(?:_[01]+)*n?/.source+"|"+/0[oO][0-7]+(?:_[0-7]+)*n?/.source+"|"+/0[xX][\dA-Fa-f]+(?:_[\dA-Fa-f]+)*n?/.source+"|"+/\d+(?:_\d+)*n/.source+"|"+/(?:\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\.\d+(?:_\d+)*)(?:[Ee][+-]?\d+(?:_\d+)*)?/.source)+")"+/(?![\w$])/.source),lookbehind:!0},operator:/--|\+\+|\*\*=?|=>|&&=?|\|\|=?|[!=]==|<<=?|>>>?=?|[-+*/%&|^!=<>]=?|\.{3}|\?\?=?|\?\.?|[~:]/}),t.languages.javascript["class-name"][0].pattern=/(\b(?:class|extends|implements|instanceof|interface|new)\s+)[\w.\\]+/,t.languages.insertBefore("javascript","keyword",{regex:{pattern:/((?:^|[^$\w\xA0-\uFFFF."'\])\s]|\b(?:return|yield))\s*)\/(?:\[(?:[^\]\\\r\n]|\\.)*\]|\\.|[^/\\\[\r\n])+\/[dgimyus]{0,7}(?=(?:\s|\/\*(?:[^*]|\*(?!\/))*\*\/)*(?:$|[\r\n,.;:})\]]|\/\/))/,lookbehind:!0,greedy:!0,inside:{"regex-source":{pattern:/^(\/)[\s\S]+(?=\/[a-z]*$)/,lookbehind:!0,alias:"language-regex",inside:t.languages.regex},"regex-delimiter":/^\/|\/$/,"regex-flags":/^[a-z]+$/}},"function-variable":{pattern:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*[=:]\s*(?:async\s*)?(?:\bfunction\b|(?:\((?:[^()]|\([^()]*\))*\)|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/,alias:"function"},parameter:[{pattern:/(function(?:\s+(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)?\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\))/,lookbehind:!0,inside:t.languages.javascript},{pattern:/(^|[^$\w\xA0-\uFFFF])(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*=>)/i,lookbehind:!0,inside:t.languages.javascript},{pattern:/(\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*=>)/,lookbehind:!0,inside:t.languages.javascript},{pattern:/((?:\b|\s|^)(?!(?:as|async|await|break|case|catch|class|const|continue|debugger|default|delete|do|else|enum|export|extends|finally|for|from|function|get|if|implements|import|in|instanceof|interface|let|new|null|of|package|private|protected|public|return|set|static|super|switch|this|throw|try|typeof|undefined|var|void|while|with|yield)(?![$\w\xA0-\uFFFF]))(?:(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*)\(\s*|\]\s*\(\s*)(?!\s)(?:[^()\s]|\s+(?![\s)])|\([^()]*\))+(?=\s*\)\s*\{)/,lookbehind:!0,inside:t.languages.javascript}],constant:/\b[A-Z](?:[A-Z_]|\dx?)*\b/}),t.languages.insertBefore("javascript","string",{hashbang:{pattern:/^#!.*/,greedy:!0,alias:"comment"},"template-string":{pattern:/`(?:\\[\s\S]|\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}|(?!\$\{)[^\\`])*`/,greedy:!0,inside:{"template-punctuation":{pattern:/^`|`$/,alias:"string"},interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,lookbehind:!0,inside:{"interpolation-punctuation":{pattern:/^\$\{|\}$/,alias:"punctuation"},rest:t.languages.javascript}},string:/[\s\S]+/}},"string-property":{pattern:/((?:^|[,{])[ \t]*)(["'])(?:\\(?:\r\n|[\s\S])|(?!\2)[^\\\r\n])*\2(?=\s*:)/m,lookbehind:!0,greedy:!0,alias:"property"}}),t.languages.insertBefore("javascript","operator",{"literal-property":{pattern:/((?:^|[,{])[ \t]*)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*:)/m,lookbehind:!0,alias:"property"}}),t.languages.markup&&(t.languages.markup.tag.addInlined("script","javascript"),t.languages.markup.tag.addAttribute(/on(?:abort|blur|change|click|composition(?:end|start|update)|dblclick|error|focus(?:in|out)?|key(?:down|up)|load|mouse(?:down|enter|leave|move|out|over|up)|reset|resize|scroll|select|slotchange|submit|unload|wheel)/.source,"javascript")),t.languages.js=t.languages.javascript}return xC}var wC,XU;function ant(){if(XU)return wC;XU=1;var e=typeof globalThis=="object"?globalThis:typeof self=="object"?self:typeof window=="object"?window:typeof ey=="object"?ey:{},t=R();e.Prism={manual:!0,disableWorkerMessageHandler:!0};var r=Vet(),n=Zrt(),a=Qrt(),i=ent(),o=tnt(),s=rnt(),l=nnt();t();var u={}.hasOwnProperty;function d(){}d.prototype=a;var p=new d;wC=p,p.highlight=b,p.register=f,p.alias=m,p.registered=y,p.listLanguages=S,f(i),f(o),f(s),f(l),p.util.encode=k,p.Token.stringify=w;function f(C){if(typeof C!="function"||!C.displayName)throw new Error("Expected `function` for `grammar`, got `"+C+"`");p.languages[C.displayName]===void 0&&C(p)}function m(C,F){var L=p.languages,I=C,M,j,Y,W;F&&(I={},I[C]=F);for(M in I)for(j=I[M],j=typeof j=="string"?[j]:j,Y=j.length,W=-1;++W<Y;)L[j[W]]=L[M]}function b(C,F){var L=a.highlight,I;if(typeof C!="string")throw new Error("Expected `string` for `value`, got `"+C+"`");if(p.util.type(F)==="Object")I=F,F=null;else{if(typeof F!="string")throw new Error("Expected `string` for `name`, got `"+F+"`");if(u.call(p.languages,F))I=p.languages[F];else throw new Error("Unknown language: `"+F+"` is not registered")}return L.call(this,C,I,F)}function y(C){if(typeof C!="string")throw new Error("Expected `string` for `language`, got `"+C+"`");return u.call(p.languages,C)}function S(){var C=p.languages,F=[],L;for(L in C)u.call(C,L)&&typeof C[L]=="object"&&F.push(L);return F}function w(C,F,L){var I;return typeof C=="string"?{type:"text",value:C}:p.util.type(C)==="Array"?E(C,F):(I={type:C.type,content:p.Token.stringify(C.content,F,L),tag:"span",classes:["token",C.type],attributes:{},language:F,parent:L},C.alias&&(I.classes=I.classes.concat(C.alias)),p.hooks.run("wrap",I),r(I.tag+"."+I.classes.join("."),_(I.attributes),I.content))}function E(C,F){for(var L=[],I=C.length,M=-1,j;++M<I;)j=C[M],j!==""&&j!==null&&j!==void 0&&L.push(j);for(M=-1,I=L.length;++M<I;)j=L[M],L[M]=p.Token.stringify(j,F,L);return L}function k(C){return C}function _(C){var F;for(F in C)C[F]=n(C[F]);return C}function R(){var C="Prism"in e,F=C?e.Prism:void 0;return L;function L(){C?e.Prism=F:delete e.Prism,C=void 0,F=void 0}}return wC}const int={'code[class*="language-"]':{color:"black",background:"none",textShadow:"0 1px white",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"black",background:"#f5f2f0",textShadow:"0 1px white",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#b3d4fc"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"]::selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#b3d4fc"},':not(pre) > code[class*="language-"]':{background:"#f5f2f0",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"slategray"},prolog:{color:"slategray"},doctype:{color:"slategray"},cdata:{color:"slategray"},punctuation:{color:"#999"},namespace:{Opacity:".7"},property:{color:"#905"},tag:{color:"#905"},boolean:{color:"#905"},number:{color:"#905"},constant:{color:"#905"},symbol:{color:"#905"},deleted:{color:"#905"},selector:{color:"#690"},"attr-name":{color:"#690"},string:{color:"#690"},char:{color:"#690"},builtin:{color:"#690"},inserted:{color:"#690"},operator:{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},entity:{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)",cursor:"help"},url:{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},".language-css .token.string":{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},".style .token.string":{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},atrule:{color:"#07a"},"attr-value":{color:"#07a"},keyword:{color:"#07a"},function:{color:"#DD4A68"},"class-name":{color:"#DD4A68"},regex:{color:"#e90"},important:{color:"#e90",fontWeight:"bold"},variable:{color:"#e90"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}};var kC,ZU;function ont(){if(ZU)return kC;ZU=1,kC=e,e.displayName="abap",e.aliases=[];function e(t){t.languages.abap={comment:/^\*.*/m,string:/(`|')(?:\\.|(?!\1)[^\\\r\n])*\1/,"string-template":{pattern:/([|}])(?:\\.|[^\\|{\r\n])*(?=[|{])/,lookbehind:!0,alias:"string"},"eol-comment":{pattern:/(^|\s)".*/m,lookbehind:!0,alias:"comment"},keyword:{pattern:/(\s|\.|^)(?:SCIENTIFIC_WITH_LEADING_ZERO|SCALE_PRESERVING_SCIENTIFIC|RMC_COMMUNICATION_FAILURE|END-ENHANCEMENT-SECTION|MULTIPLY-CORRESPONDING|SUBTRACT-CORRESPONDING|VERIFICATION-MESSAGE|DIVIDE-CORRESPONDING|ENHANCEMENT-SECTION|CURRENCY_CONVERSION|RMC_SYSTEM_FAILURE|START-OF-SELECTION|MOVE-CORRESPONDING|RMC_INVALID_STATUS|CUSTOMER-FUNCTION|END-OF-DEFINITION|ENHANCEMENT-POINT|SYSTEM-EXCEPTIONS|ADD-CORRESPONDING|SCALE_PRESERVING|SELECTION-SCREEN|CURSOR-SELECTION|END-OF-SELECTION|LOAD-OF-PROGRAM|SCROLL-BOUNDARY|SELECTION-TABLE|EXCEPTION-TABLE|IMPLEMENTATIONS|PARAMETER-TABLE|RIGHT-JUSTIFIED|UNIT_CONVERSION|AUTHORITY-CHECK|LIST-PROCESSING|SIGN_AS_POSTFIX|COL_BACKGROUND|IMPLEMENTATION|INTERFACE-POOL|TRANSFORMATION|IDENTIFICATION|ENDENHANCEMENT|LINE-SELECTION|INITIALIZATION|LEFT-JUSTIFIED|SELECT-OPTIONS|SELECTION-SETS|COMMUNICATION|CORRESPONDING|DECIMAL_SHIFT|PRINT-CONTROL|VALUE-REQUEST|CHAIN-REQUEST|FUNCTION-POOL|FIELD-SYMBOLS|FUNCTIONALITY|INVERTED-DATE|SELECTION-SET|CLASS-METHODS|OUTPUT-LENGTH|CLASS-CODING|COL_NEGATIVE|ERRORMESSAGE|FIELD-GROUPS|HELP-REQUEST|NO-EXTENSION|NO-TOPOFPAGE|REDEFINITION|DISPLAY-MODE|ENDINTERFACE|EXIT-COMMAND|FIELD-SYMBOL|NO-SCROLLING|SHORTDUMP-ID|ACCESSPOLICY|CLASS-EVENTS|COL_POSITIVE|DECLARATIONS|ENHANCEMENTS|FILTER-TABLE|SWITCHSTATES|SYNTAX-CHECK|TRANSPORTING|ASYNCHRONOUS|SYNTAX-TRACE|TOKENIZATION|USER-COMMAND|WITH-HEADING|ABAP-SOURCE|BREAK-POINT|CHAIN-INPUT|COMPRESSION|FIXED-POINT|NEW-SECTION|NON-UNICODE|OCCURRENCES|RESPONSIBLE|SYSTEM-CALL|TRACE-TABLE|ABBREVIATED|CHAR-TO-HEX|END-OF-FILE|ENDFUNCTION|ENVIRONMENT|ASSOCIATION|COL_HEADING|EDITOR-CALL|END-OF-PAGE|ENGINEERING|IMPLEMENTED|INTENSIFIED|RADIOBUTTON|SYSTEM-EXIT|TOP-OF-PAGE|TRANSACTION|APPLICATION|CONCATENATE|DESTINATION|ENHANCEMENT|IMMEDIATELY|NO-GROUPING|PRECOMPILED|REPLACEMENT|TITLE-LINES|ACTIVATION|BYTE-ORDER|CLASS-POOL|CONNECTION|CONVERSION|DEFINITION|DEPARTMENT|EXPIRATION|INHERITING|MESSAGE-ID|NO-HEADING|PERFORMING|QUEUE-ONLY|RIGHTSPACE|SCIENTIFIC|STATUSINFO|STRUCTURES|SYNCPOINTS|WITH-TITLE|ATTRIBUTES|BOUNDARIES|CLASS-DATA|COL_NORMAL|DD\/MM\/YYYY|DESCENDING|INTERFACES|LINE-COUNT|MM\/DD\/YYYY|NON-UNIQUE|PRESERVING|SELECTIONS|STATEMENTS|SUBROUTINE|TRUNCATION|TYPE-POOLS|ARITHMETIC|BACKGROUND|ENDPROVIDE|EXCEPTIONS|IDENTIFIER|INDEX-LINE|OBLIGATORY|PARAMETERS|PERCENTAGE|PUSHBUTTON|RESOLUTION|COMPONENTS|DEALLOCATE|DISCONNECT|DUPLICATES|FIRST-LINE|HEAD-LINES|NO-DISPLAY|OCCURRENCE|RESPECTING|RETURNCODE|SUBMATCHES|TRACE-FILE|ASCENDING|BYPASSING|ENDMODULE|EXCEPTION|EXCLUDING|EXPORTING|INCREMENT|MATCHCODE|PARAMETER|PARTIALLY|PREFERRED|REFERENCE|REPLACING|RETURNING|SELECTION|SEPARATED|SPECIFIED|STATEMENT|TIMESTAMP|TYPE-POOL|ACCEPTING|APPENDAGE|ASSIGNING|COL_GROUP|COMPARING|CONSTANTS|DANGEROUS|IMPORTING|INSTANCES|LEFTSPACE|LOG-POINT|QUICKINFO|READ-ONLY|SCROLLING|SQLSCRIPT|STEP-LOOP|TOP-LINES|TRANSLATE|APPENDING|AUTHORITY|CHARACTER|COMPONENT|CONDITION|DIRECTORY|DUPLICATE|MESSAGING|RECEIVING|SUBSCREEN|ACCORDING|COL_TOTAL|END-LINES|ENDMETHOD|ENDSELECT|EXPANDING|EXTENSION|INCLUDING|INFOTYPES|INTERFACE|INTERVALS|LINE-SIZE|PF-STATUS|PROCEDURE|PROTECTED|REQUESTED|RESUMABLE|RIGHTPLUS|SAP-SPOOL|SECONDARY|STRUCTURE|SUBSTRING|TABLEVIEW|NUMOFCHAR|ADJACENT|ANALYSIS|ASSIGNED|BACKWARD|CHANNELS|CHECKBOX|CONTINUE|CRITICAL|DATAINFO|DD\/MM\/YY|DURATION|ENCODING|ENDCLASS|FUNCTION|LEFTPLUS|LINEFEED|MM\/DD\/YY|OVERFLOW|RECEIVED|SKIPPING|SORTABLE|STANDARD|SUBTRACT|SUPPRESS|TABSTRIP|TITLEBAR|TRUNCATE|UNASSIGN|WHENEVER|ANALYZER|COALESCE|COMMENTS|CONDENSE|DECIMALS|DEFERRED|ENDWHILE|EXPLICIT|KEYWORDS|MESSAGES|POSITION|PRIORITY|RECEIVER|RENAMING|TIMEZONE|TRAILING|ALLOCATE|CENTERED|CIRCULAR|CONTROLS|CURRENCY|DELETING|DESCRIBE|DISTANCE|ENDCATCH|EXPONENT|EXTENDED|GENERATE|IGNORING|INCLUDES|INTERNAL|MAJOR-ID|MODIFIER|NEW-LINE|OPTIONAL|PROPERTY|ROLLBACK|STARTING|SUPPLIED|ABSTRACT|CHANGING|CONTEXTS|CREATING|CUSTOMER|DATABASE|DAYLIGHT|DEFINING|DISTINCT|DIVISION|ENABLING|ENDCHAIN|ESCAPING|HARMLESS|IMPLICIT|INACTIVE|LANGUAGE|MINOR-ID|MULTIPLY|NEW-PAGE|NO-TITLE|POS_HIGH|SEPARATE|TEXTPOOL|TRANSFER|SELECTOR|DBMAXLEN|ITERATOR|ARCHIVE|BIT-XOR|BYTE-CO|COLLECT|COMMENT|CURRENT|DEFAULT|DISPLAY|ENDFORM|EXTRACT|LEADING|LISTBOX|LOCATOR|MEMBERS|METHODS|NESTING|POS_LOW|PROCESS|PROVIDE|RAISING|RESERVE|SECONDS|SUMMARY|VISIBLE|BETWEEN|BIT-AND|BYTE-CS|CLEANUP|COMPUTE|CONTROL|CONVERT|DATASET|ENDCASE|FORWARD|HEADERS|HOTSPOT|INCLUDE|INVERSE|KEEPING|NO-ZERO|OBJECTS|OVERLAY|PADDING|PATTERN|PROGRAM|REFRESH|SECTION|SUMMING|TESTING|VERSION|WINDOWS|WITHOUT|BIT-NOT|BYTE-CA|BYTE-NA|CASTING|CONTEXT|COUNTRY|DYNAMIC|ENABLED|ENDLOOP|EXECUTE|FRIENDS|HANDLER|HEADING|INITIAL|\*-INPUT|LOGFILE|MAXIMUM|MINIMUM|NO-GAPS|NO-SIGN|PRAGMAS|PRIMARY|PRIVATE|REDUCED|REPLACE|REQUEST|RESULTS|UNICODE|WARNING|ALIASES|BYTE-CN|BYTE-NS|CALLING|COL_KEY|COLUMNS|CONNECT|ENDEXEC|ENTRIES|EXCLUDE|FILTERS|FURTHER|HELP-ID|LOGICAL|MAPPING|MESSAGE|NAMETAB|OPTIONS|PACKAGE|PERFORM|RECEIVE|STATICS|VARYING|BINDING|CHARLEN|GREATER|XSTRLEN|ACCEPT|APPEND|DETAIL|ELSEIF|ENDING|ENDTRY|FORMAT|FRAMES|GIVING|HASHED|HEADER|IMPORT|INSERT|MARGIN|MODULE|NATIVE|OBJECT|OFFSET|REMOTE|RESUME|SAVING|SIMPLE|SUBMIT|TABBED|TOKENS|UNIQUE|UNPACK|UPDATE|WINDOW|YELLOW|ACTUAL|ASPECT|CENTER|CURSOR|DELETE|DIALOG|DIVIDE|DURING|ERRORS|EVENTS|EXTEND|FILTER|HANDLE|HAVING|IGNORE|LITTLE|MEMORY|NO-GAP|OCCURS|OPTION|PERSON|PLACES|PUBLIC|REDUCE|REPORT|RESULT|SINGLE|SORTED|SWITCH|SYNTAX|TARGET|VALUES|WRITER|ASSERT|BLOCKS|BOUNDS|BUFFER|CHANGE|COLUMN|COMMIT|CONCAT|COPIES|CREATE|DDMMYY|DEFINE|ENDIAN|ESCAPE|EXPAND|KERNEL|LAYOUT|LEGACY|LEVELS|MMDDYY|NUMBER|OUTPUT|RANGES|READER|RETURN|SCREEN|SEARCH|SELECT|SHARED|SOURCE|STABLE|STATIC|SUBKEY|SUFFIX|TABLES|UNWIND|YYMMDD|ASSIGN|BACKUP|BEFORE|BINARY|BIT-OR|BLANKS|CLIENT|CODING|COMMON|DEMAND|DYNPRO|EXCEPT|EXISTS|EXPORT|FIELDS|GLOBAL|GROUPS|LENGTH|LOCALE|MEDIUM|METHOD|MODIFY|NESTED|OTHERS|REJECT|SCROLL|SUPPLY|SYMBOL|ENDFOR|STRLEN|ALIGN|BEGIN|BOUND|ENDAT|ENTRY|EVENT|FINAL|FLUSH|GRANT|INNER|SHORT|USING|WRITE|AFTER|BLACK|BLOCK|CLOCK|COLOR|COUNT|DUMMY|EMPTY|ENDDO|ENDON|GREEN|INDEX|INOUT|LEAVE|LEVEL|LINES|MODIF|ORDER|OUTER|RANGE|RESET|RETRY|RIGHT|SMART|SPLIT|STYLE|TABLE|THROW|UNDER|UNTIL|UPPER|UTF-8|WHERE|ALIAS|BLANK|CLEAR|CLOSE|EXACT|FETCH|FIRST|FOUND|GROUP|LLANG|LOCAL|OTHER|REGEX|SPOOL|TITLE|TYPES|VALID|WHILE|ALPHA|BOXED|CATCH|CHAIN|CHECK|CLASS|COVER|ENDIF|EQUIV|FIELD|FLOOR|FRAME|INPUT|LOWER|MATCH|NODES|PAGES|PRINT|RAISE|ROUND|SHIFT|SPACE|SPOTS|STAMP|STATE|TASKS|TIMES|TRMAC|ULINE|UNION|VALUE|WIDTH|EQUAL|LOG10|TRUNC|BLOB|CASE|CEIL|CLOB|COND|EXIT|FILE|GAPS|HOLD|INCL|INTO|KEEP|KEYS|LAST|LINE|LONG|LPAD|MAIL|MODE|OPEN|PINK|READ|ROWS|TEST|THEN|ZERO|AREA|BACK|BADI|BYTE|CAST|EDIT|EXEC|FAIL|FIND|FKEQ|FONT|FREE|GKEQ|HIDE|INIT|ITNO|LATE|LOOP|MAIN|MARK|MOVE|NEXT|NULL|RISK|ROLE|UNIT|WAIT|ZONE|BASE|CALL|CODE|DATA|DATE|FKGE|GKGE|HIGH|KIND|LEFT|LIST|MASK|MESH|NAME|NODE|PACK|PAGE|POOL|SEND|SIGN|SIZE|SOME|STOP|TASK|TEXT|TIME|USER|VARY|WITH|WORD|BLUE|CONV|COPY|DEEP|ELSE|FORM|FROM|HINT|ICON|JOIN|LIKE|LOAD|ONLY|PART|SCAN|SKIP|SORT|TYPE|UNIX|VIEW|WHEN|WORK|ACOS|ASIN|ATAN|COSH|EACH|FRAC|LESS|RTTI|SINH|SQRT|TANH|AVG|BIT|DIV|ISO|LET|OUT|PAD|SQL|ALL|CI_|CPI|END|LOB|LPI|MAX|MIN|NEW|OLE|RUN|SET|\?TO|YES|ABS|ADD|AND|BIG|FOR|HDB|JOB|LOW|NOT|SAP|TRY|VIA|XML|ANY|GET|IDS|KEY|MOD|OFF|PUT|RAW|RED|REF|SUM|TAB|XSD|CNT|COS|EXP|LOG|SIN|TAN|XOR|AT|CO|CP|DO|GT|ID|IF|NS|OR|BT|CA|CS|GE|NA|NB|EQ|IN|LT|NE|NO|OF|ON|PF|TO|AS|BY|CN|IS|LE|NP|UP|E|I|M|O|Z|C|X)\b/i,lookbehind:!0},number:/\b\d+\b/,operator:{pattern:/(\s)(?:\*\*?|<[=>]?|>=?|\?=|[-+\/=])(?=\s)/,lookbehind:!0},"string-operator":{pattern:/(\s)&&?(?=\s)/,lookbehind:!0,alias:"keyword"},"token-operator":[{pattern:/(\w)(?:->?|=>|[~|{}])(?=\w)/,lookbehind:!0,alias:"punctuation"},{pattern:/[|{}]/,alias:"punctuation"}],punctuation:/[,.:()]/}}return kC}var CC,QU;function snt(){if(QU)return CC;QU=1,CC=e,e.displayName="abnf",e.aliases=[];function e(t){(function(r){var n="(?:ALPHA|BIT|CHAR|CR|CRLF|CTL|DIGIT|DQUOTE|HEXDIG|HTAB|LF|LWSP|OCTET|SP|VCHAR|WSP)";r.languages.abnf={comment:/;.*/,string:{pattern:/(?:%[is])?"[^"\n\r]*"/,greedy:!0,inside:{punctuation:/^%[is]/}},range:{pattern:/%(?:b[01]+-[01]+|d\d+-\d+|x[A-F\d]+-[A-F\d]+)/i,alias:"number"},terminal:{pattern:/%(?:b[01]+(?:\.[01]+)*|d\d+(?:\.\d+)*|x[A-F\d]+(?:\.[A-F\d]+)*)/i,alias:"number"},repetition:{pattern:/(^|[^\w-])(?:\d*\*\d*|\d+)/,lookbehind:!0,alias:"operator"},definition:{pattern:/(^[ \t]*)(?:[a-z][\w-]*|<[^<>\r\n]*>)(?=\s*=)/m,lookbehind:!0,alias:"keyword",inside:{punctuation:/<|>/}},"core-rule":{pattern:RegExp("(?:(^|[^<\\w-])"+n+"|<"+n+">)(?![\\w-])","i"),lookbehind:!0,alias:["rule","constant"],inside:{punctuation:/<|>/}},rule:{pattern:/(^|[^<\w-])[a-z][\w-]*|<[^<>\r\n]*>/i,lookbehind:!0,inside:{punctuation:/<|>/}},operator:/=\/?|\//,punctuation:/[()\[\]]/}})(t)}return CC}var TC,e$;function lnt(){if(e$)return TC;e$=1,TC=e,e.displayName="actionscript",e.aliases=[];function e(t){t.languages.actionscript=t.languages.extend("javascript",{keyword:/\b(?:as|break|case|catch|class|const|default|delete|do|dynamic|each|else|extends|final|finally|for|function|get|if|implements|import|in|include|instanceof|interface|internal|is|namespace|native|new|null|override|package|private|protected|public|return|set|static|super|switch|this|throw|try|typeof|use|var|void|while|with)\b/,operator:/\+\+|--|(?:[+\-*\/%^]|&&?|\|\|?|<<?|>>?>?|[!=]=?)=?|[~?@]/}),t.languages.actionscript["class-name"].alias="function",delete t.languages.actionscript.parameter,delete t.languages.actionscript["literal-property"],t.languages.markup&&t.languages.insertBefore("actionscript","string",{xml:{pattern:/(^|[^.])<\/?\w+(?:\s+[^\s>\/=]+=("|')(?:\\[\s\S]|(?!\2)[^\\])*\2)*\s*\/?>/,lookbehind:!0,inside:t.languages.markup}})}return TC}var AC,t$;function unt(){if(t$)return AC;t$=1,AC=e,e.displayName="ada",e.aliases=[];function e(t){t.languages.ada={comment:/--.*/,string:/"(?:""|[^"\r\f\n])*"/,number:[{pattern:/\b\d(?:_?\d)*#[\dA-F](?:_?[\dA-F])*(?:\.[\dA-F](?:_?[\dA-F])*)?#(?:E[+-]?\d(?:_?\d)*)?/i},{pattern:/\b\d(?:_?\d)*(?:\.\d(?:_?\d)*)?(?:E[+-]?\d(?:_?\d)*)?\b/i}],"attr-name":/\b'\w+/,keyword:/\b(?:abort|abs|abstract|accept|access|aliased|all|and|array|at|begin|body|case|constant|declare|delay|delta|digits|do|else|elsif|end|entry|exception|exit|for|function|generic|goto|if|in|interface|is|limited|loop|mod|new|not|null|of|others|out|overriding|package|pragma|private|procedure|protected|raise|range|record|rem|renames|requeue|return|reverse|select|separate|some|subtype|synchronized|tagged|task|terminate|then|type|until|use|when|while|with|xor)\b/i,boolean:/\b(?:false|true)\b/i,operator:/<[=>]?|>=?|=>?|:=|\/=?|\*\*?|[&+-]/,punctuation:/\.\.?|[,;():]/,char:/'.'/,variable:/\b[a-z](?:\w)*\b/i}}return AC}var FC,r$;function cnt(){if(r$)return FC;r$=1,FC=e,e.displayName="agda",e.aliases=[];function e(t){(function(r){r.languages.agda={comment:/\{-[\s\S]*?(?:-\}|$)|--.*/,string:{pattern:/"(?:\\(?:\r\n|[\s\S])|[^\\\r\n"])*"/,greedy:!0},punctuation:/[(){}⦃⦄.;@]/,"class-name":{pattern:/((?:data|record) +)\S+/,lookbehind:!0},function:{pattern:/(^[ \t]*)(?!\s)[^:\r\n]+(?=:)/m,lookbehind:!0},operator:{pattern:/(^\s*|\s)(?:[=|:∀→λ\\?_]|->)(?=\s)/,lookbehind:!0},keyword:/\b(?:Set|abstract|constructor|data|eta-equality|field|forall|hiding|import|in|inductive|infix|infixl|infixr|instance|let|macro|module|mutual|no-eta-equality|open|overlap|pattern|postulate|primitive|private|public|quote|quoteContext|quoteGoal|quoteTerm|record|renaming|rewrite|syntax|tactic|unquote|unquoteDecl|unquoteDef|using|variable|where|with)\b/}})(t)}return FC}var _C,n$;function dnt(){if(n$)return _C;n$=1,_C=e,e.displayName="al",e.aliases=[];function e(t){t.languages.al={comment:/\/\/.*|\/\*[\s\S]*?\*\//,string:{pattern:/'(?:''|[^'\r\n])*'(?!')|"(?:""|[^"\r\n])*"(?!")/,greedy:!0},function:{pattern:/(\b(?:event|procedure|trigger)\s+|(?:^|[^.])\.\s*)[a-z_]\w*(?=\s*\()/i,lookbehind:!0},keyword:[/\b(?:array|asserterror|begin|break|case|do|downto|else|end|event|exit|for|foreach|function|if|implements|in|indataset|interface|internal|local|of|procedure|program|protected|repeat|runonclient|securityfiltering|suppressdispose|temporary|then|to|trigger|until|var|while|with|withevents)\b/i,/\b(?:action|actions|addafter|addbefore|addfirst|addlast|area|assembly|chartpart|codeunit|column|controladdin|cuegroup|customizes|dataitem|dataset|dotnet|elements|enum|enumextension|extends|field|fieldattribute|fieldelement|fieldgroup|fieldgroups|fields|filter|fixed|grid|group|key|keys|label|labels|layout|modify|moveafter|movebefore|movefirst|movelast|page|pagecustomization|pageextension|part|profile|query|repeater|report|requestpage|schema|separator|systempart|table|tableelement|tableextension|textattribute|textelement|type|usercontrol|value|xmlport)\b/i],number:/\b(?:0x[\da-f]+|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?)(?:F|LL?|U(?:LL?)?)?\b/i,boolean:/\b(?:false|true)\b/i,variable:/\b(?:Curr(?:FieldNo|Page|Report)|x?Rec|RequestOptionsPage)\b/,"class-name":/\b(?:automation|biginteger|bigtext|blob|boolean|byte|char|clienttype|code|completiontriggererrorlevel|connectiontype|database|dataclassification|datascope|date|dateformula|datetime|decimal|defaultlayout|dialog|dictionary|dotnetassembly|dotnettypedeclaration|duration|errorinfo|errortype|executioncontext|executionmode|fieldclass|fieldref|fieldtype|file|filterpagebuilder|guid|httpclient|httpcontent|httpheaders|httprequestmessage|httpresponsemessage|instream|integer|joker|jsonarray|jsonobject|jsontoken|jsonvalue|keyref|list|moduledependencyinfo|moduleinfo|none|notification|notificationscope|objecttype|option|outstream|pageresult|record|recordid|recordref|reportformat|securityfilter|sessionsettings|tableconnectiontype|tablefilter|testaction|testfield|testfilterfield|testpage|testpermissions|testrequestpage|text|textbuilder|textconst|textencoding|time|transactionmodel|transactiontype|variant|verbosity|version|view|views|webserviceactioncontext|webserviceactionresultcode|xmlattribute|xmlattributecollection|xmlcdata|xmlcomment|xmldeclaration|xmldocument|xmldocumenttype|xmlelement|xmlnamespacemanager|xmlnametable|xmlnode|xmlnodelist|xmlprocessinginstruction|xmlreadoptions|xmltext|xmlwriteoptions)\b/i,operator:/\.\.|:[=:]|[-+*/]=?|<>|[<>]=?|=|\b(?:and|div|mod|not|or|xor)\b/i,punctuation:/[()\[\]{}:.;,]/}}return _C}var RC,a$;function hnt(){if(a$)return RC;a$=1,RC=e,e.displayName="antlr4",e.aliases=["g4"];function e(t){t.languages.antlr4={comment:/\/\/.*|\/\*[\s\S]*?(?:\*\/|$)/,string:{pattern:/'(?:\\.|[^\\'\r\n])*'/,greedy:!0},"character-class":{pattern:/\[(?:\\.|[^\\\]\r\n])*\]/,greedy:!0,alias:"regex",inside:{range:{pattern:/([^[]|(?:^|[^\\])(?:\\\\)*\\\[)-(?!\])/,lookbehind:!0,alias:"punctuation"},escape:/\\(?:u(?:[a-fA-F\d]{4}|\{[a-fA-F\d]+\})|[pP]\{[=\w-]+\}|[^\r\nupP])/,punctuation:/[\[\]]/}},action:{pattern:/\{(?:[^{}]|\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\})*\}/,greedy:!0,inside:{content:{pattern:/(\{)[\s\S]+(?=\})/,lookbehind:!0},punctuation:/[{}]/}},command:{pattern:/(->\s*(?!\s))(?:\s*(?:,\s*)?\b[a-z]\w*(?:\s*\([^()\r\n]*\))?)+(?=\s*;)/i,lookbehind:!0,inside:{function:/\b\w+(?=\s*(?:[,(]|$))/,punctuation:/[,()]/}},annotation:{pattern:/@\w+(?:::\w+)*/,alias:"keyword"},label:{pattern:/#[ \t]*\w+/,alias:"punctuation"},keyword:/\b(?:catch|channels|finally|fragment|grammar|import|lexer|locals|mode|options|parser|returns|throws|tokens)\b/,definition:[{pattern:/\b[a-z]\w*(?=\s*:)/,alias:["rule","class-name"]},{pattern:/\b[A-Z]\w*(?=\s*:)/,alias:["token","constant"]}],constant:/\b[A-Z][A-Z_]*\b/,operator:/\.\.|->|[|~]|[*+?]\??/,punctuation:/[;:()=]/},t.languages.g4=t.languages.antlr4}return RC}var DC,i$;function pnt(){if(i$)return DC;i$=1,DC=e,e.displayName="apacheconf",e.aliases=[];function e(t){t.languages.apacheconf={comment:/#.*/,"directive-inline":{pattern:/(^[\t ]*)\b(?:AcceptFilter|AcceptPathInfo|AccessFileName|Action|Add(?:Alt|AltByEncoding|AltByType|Charset|DefaultCharset|Description|Encoding|Handler|Icon|IconByEncoding|IconByType|InputFilter|Language|ModuleInfo|OutputFilter|OutputFilterByType|Type)|Alias|AliasMatch|Allow(?:CONNECT|EncodedSlashes|Methods|Override|OverrideList)?|Anonymous(?:_LogEmail|_MustGiveEmail|_NoUserID|_VerifyEmail)?|AsyncRequestWorkerFactor|Auth(?:BasicAuthoritative|BasicFake|BasicProvider|BasicUseDigestAlgorithm|DBDUserPWQuery|DBDUserRealmQuery|DBMGroupFile|DBMType|DBMUserFile|Digest(?:Algorithm|Domain|NonceLifetime|Provider|Qop|ShmemSize)|Form(?:Authoritative|Body|DisableNoStore|FakeBasicAuth|Location|LoginRequiredLocation|LoginSuccessLocation|LogoutLocation|Method|Mimetype|Password|Provider|SitePassphrase|Size|Username)|GroupFile|LDAP(?:AuthorizePrefix|BindAuthoritative|BindDN|BindPassword|CharsetConfig|CompareAsUser|CompareDNOnServer|DereferenceAliases|GroupAttribute|GroupAttributeIsDN|InitialBindAsUser|InitialBindPattern|MaxSubGroupDepth|RemoteUserAttribute|RemoteUserIsDN|SearchAsUser|SubGroupAttribute|SubGroupClass|Url)|Merging|Name|nCache(?:Context|Enable|ProvideFor|SOCache|Timeout)|nzFcgiCheckAuthnProvider|nzFcgiDefineProvider|Type|UserFile|zDBDLoginToReferer|zDBDQuery|zDBDRedirectQuery|zDBMType|zSendForbiddenOnFailure)|BalancerGrowth|BalancerInherit|BalancerMember|BalancerPersist|BrowserMatch|BrowserMatchNoCase|BufferedLogs|BufferSize|Cache(?:DefaultExpire|DetailHeader|DirLength|DirLevels|Disable|Enable|File|Header|IgnoreCacheControl|IgnoreHeaders|IgnoreNoLastMod|IgnoreQueryString|IgnoreURLSessionIdentifiers|KeyBaseURL|LastModifiedFactor|Lock|LockMaxAge|LockPath|MaxExpire|MaxFileSize|MinExpire|MinFileSize|NegotiatedDocs|QuickHandler|ReadSize|ReadTime|Root|Socache(?:MaxSize|MaxTime|MinTime|ReadSize|ReadTime)?|StaleOnError|StoreExpired|StoreNoStore|StorePrivate)|CGIDScriptTimeout|CGIMapExtension|CharsetDefault|CharsetOptions|CharsetSourceEnc|CheckCaseOnly|CheckSpelling|ChrootDir|ContentDigest|CookieDomain|CookieExpires|CookieName|CookieStyle|CookieTracking|CoreDumpDirectory|CustomLog|Dav|DavDepthInfinity|DavGenericLockDB|DavLockDB|DavMinTimeout|DBDExptime|DBDInitSQL|DBDKeep|DBDMax|DBDMin|DBDParams|DBDPersist|DBDPrepareSQL|DBDriver|DefaultIcon|DefaultLanguage|DefaultRuntimeDir|DefaultType|Define|Deflate(?:BufferSize|CompressionLevel|FilterNote|InflateLimitRequestBody|InflateRatio(?:Burst|Limit)|MemLevel|WindowSize)|Deny|DirectoryCheckHandler|DirectoryIndex|DirectoryIndexRedirect|DirectorySlash|DocumentRoot|DTracePrivileges|DumpIOInput|DumpIOOutput|EnableExceptionHook|EnableMMAP|EnableSendfile|Error|ErrorDocument|ErrorLog|ErrorLogFormat|Example|ExpiresActive|ExpiresByType|ExpiresDefault|ExtendedStatus|ExtFilterDefine|ExtFilterOptions|FallbackResource|FileETag|FilterChain|FilterDeclare|FilterProtocol|FilterProvider|FilterTrace|ForceLanguagePriority|ForceType|ForensicLog|GprofDir|GracefulShutdownTimeout|Group|Header|HeaderName|Heartbeat(?:Address|Listen|MaxServers|Storage)|HostnameLookups|IdentityCheck|IdentityCheckTimeout|ImapBase|ImapDefault|ImapMenu|Include|IncludeOptional|Index(?:HeadInsert|Ignore|IgnoreReset|Options|OrderDefault|StyleSheet)|InputSed|ISAPI(?:AppendLogToErrors|AppendLogToQuery|CacheFile|FakeAsync|LogNotSupported|ReadAheadBuffer)|KeepAlive|KeepAliveTimeout|KeptBodySize|LanguagePriority|LDAP(?:CacheEntries|CacheTTL|ConnectionPoolTTL|ConnectionTimeout|LibraryDebug|OpCacheEntries|OpCacheTTL|ReferralHopLimit|Referrals|Retries|RetryDelay|SharedCacheFile|SharedCacheSize|Timeout|TrustedClientCert|TrustedGlobalCert|TrustedMode|VerifyServerCert)|Limit(?:InternalRecursion|Request(?:Body|Fields|FieldSize|Line)|XMLRequestBody)|Listen|ListenBackLog|LoadFile|LoadModule|LogFormat|LogLevel|LogMessage|LuaAuthzProvider|LuaCodeCache|Lua(?:Hook(?:AccessChecker|AuthChecker|CheckUserID|Fixups|InsertFilter|Log|MapToStorage|TranslateName|TypeChecker)|Inherit|InputFilter|MapHandler|OutputFilter|PackageCPath|PackagePath|QuickHandler|Root|Scope)|Max(?:ConnectionsPerChild|KeepAliveRequests|MemFree|RangeOverlaps|RangeReversals|Ranges|RequestWorkers|SpareServers|SpareThreads|Threads)|MergeTrailers|MetaDir|MetaFiles|MetaSuffix|MimeMagicFile|MinSpareServers|MinSpareThreads|MMapFile|ModemStandard|ModMimeUsePathInfo|MultiviewsMatch|Mutex|NameVirtualHost|NoProxy|NWSSLTrustedCerts|NWSSLUpgradeable|Options|Order|OutputSed|PassEnv|PidFile|PrivilegesMode|Protocol|ProtocolEcho|Proxy(?:AddHeaders|BadHeader|Block|Domain|ErrorOverride|ExpressDBMFile|ExpressDBMType|ExpressEnable|FtpDirCharset|FtpEscapeWildcards|FtpListOnWildcard|HTML(?:BufSize|CharsetOut|DocType|Enable|Events|Extended|Fixups|Interp|Links|Meta|StripComments|URLMap)|IOBufferSize|MaxForwards|Pass(?:Inherit|InterpolateEnv|Match|Reverse|ReverseCookieDomain|ReverseCookiePath)?|PreserveHost|ReceiveBufferSize|Remote|RemoteMatch|Requests|SCGIInternalRedirect|SCGISendfile|Set|SourceAddress|Status|Timeout|Via)|ReadmeName|ReceiveBufferSize|Redirect|RedirectMatch|RedirectPermanent|RedirectTemp|ReflectorHeader|RemoteIP(?:Header|InternalProxy|InternalProxyList|ProxiesHeader|TrustedProxy|TrustedProxyList)|RemoveCharset|RemoveEncoding|RemoveHandler|RemoveInputFilter|RemoveLanguage|RemoveOutputFilter|RemoveType|RequestHeader|RequestReadTimeout|Require|Rewrite(?:Base|Cond|Engine|Map|Options|Rule)|RLimitCPU|RLimitMEM|RLimitNPROC|Satisfy|ScoreBoardFile|Script(?:Alias|AliasMatch|InterpreterSource|Log|LogBuffer|LogLength|Sock)?|SecureListen|SeeRequestTail|SendBufferSize|Server(?:Admin|Alias|Limit|Name|Path|Root|Signature|Tokens)|Session(?:Cookie(?:Name|Name2|Remove)|Crypto(?:Cipher|Driver|Passphrase|PassphraseFile)|DBD(?:CookieName|CookieName2|CookieRemove|DeleteLabel|InsertLabel|PerUser|SelectLabel|UpdateLabel)|Env|Exclude|Header|Include|MaxAge)?|SetEnv|SetEnvIf|SetEnvIfExpr|SetEnvIfNoCase|SetHandler|SetInputFilter|SetOutputFilter|SSIEndTag|SSIErrorMsg|SSIETag|SSILastModified|SSILegacyExprParser|SSIStartTag|SSITimeFormat|SSIUndefinedEcho|SSL(?:CACertificateFile|CACertificatePath|CADNRequestFile|CADNRequestPath|CARevocationCheck|CARevocationFile|CARevocationPath|CertificateChainFile|CertificateFile|CertificateKeyFile|CipherSuite|Compression|CryptoDevice|Engine|FIPS|HonorCipherOrder|InsecureRenegotiation|OCSP(?:DefaultResponder|Enable|OverrideResponder|ResponderTimeout|ResponseMaxAge|ResponseTimeSkew|UseRequestNonce)|OpenSSLConfCmd|Options|PassPhraseDialog|Protocol|Proxy(?:CACertificateFile|CACertificatePath|CARevocation(?:Check|File|Path)|CheckPeer(?:CN|Expire|Name)|CipherSuite|Engine|MachineCertificate(?:ChainFile|File|Path)|Protocol|Verify|VerifyDepth)|RandomSeed|RenegBufferSize|Require|RequireSSL|Session(?:Cache|CacheTimeout|TicketKeyFile|Tickets)|SRPUnknownUserSeed|SRPVerifierFile|Stapling(?:Cache|ErrorCacheTimeout|FakeTryLater|ForceURL|ResponderTimeout|ResponseMaxAge|ResponseTimeSkew|ReturnResponderErrors|StandardCacheTimeout)|StrictSNIVHostCheck|UserName|UseStapling|VerifyClient|VerifyDepth)|StartServers|StartThreads|Substitute|Suexec|SuexecUserGroup|ThreadLimit|ThreadsPerChild|ThreadStackSize|TimeOut|TraceEnable|TransferLog|TypesConfig|UnDefine|UndefMacro|UnsetEnv|Use|UseCanonicalName|UseCanonicalPhysicalPort|User|UserDir|VHostCGIMode|VHostCGIPrivs|VHostGroup|VHostPrivs|VHostSecure|VHostUser|Virtual(?:DocumentRoot|ScriptAlias)(?:IP)?|WatchdogInterval|XBitHack|xml2EncAlias|xml2EncDefault|xml2StartParse)\b/im,lookbehind:!0,alias:"property"},"directive-block":{pattern:/<\/?\b(?:Auth[nz]ProviderAlias|Directory|DirectoryMatch|Else|ElseIf|Files|FilesMatch|If|IfDefine|IfModule|IfVersion|Limit|LimitExcept|Location|LocationMatch|Macro|Proxy|Require(?:All|Any|None)|VirtualHost)\b.*>/i,inside:{"directive-block":{pattern:/^<\/?\w+/,inside:{punctuation:/^<\/?/},alias:"tag"},"directive-block-parameter":{pattern:/.*[^>]/,inside:{punctuation:/:/,string:{pattern:/("|').*\1/,inside:{variable:/[$%]\{?(?:\w\.?[-+:]?)+\}?/}}},alias:"attr-value"},punctuation:/>/},alias:"tag"},"directive-flags":{pattern:/\[(?:[\w=],?)+\]/,alias:"keyword"},string:{pattern:/("|').*\1/,inside:{variable:/[$%]\{?(?:\w\.?[-+:]?)+\}?/}},variable:/[$%]\{?(?:\w\.?[-+:]?)+\}?/,regex:/\^?.*\$|\^.*\$?/}}return DC}var NC,o$;function hL(){if(o$)return NC;o$=1,NC=e,e.displayName="sql",e.aliases=[];function e(t){t.languages.sql={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:--|\/\/|#).*)/,lookbehind:!0},variable:[{pattern:/@(["'`])(?:\\[\s\S]|(?!\1)[^\\])+\1/,greedy:!0},/@[\w.$]+/],string:{pattern:/(^|[^@\\])("|')(?:\\[\s\S]|(?!\2)[^\\]|\2\2)*\2/,greedy:!0,lookbehind:!0},identifier:{pattern:/(^|[^@\\])`(?:\\[\s\S]|[^`\\]|``)*`/,greedy:!0,lookbehind:!0,inside:{punctuation:/^`|`$/}},function:/\b(?:AVG|COUNT|FIRST|FORMAT|LAST|LCASE|LEN|MAX|MID|MIN|MOD|NOW|ROUND|SUM|UCASE)(?=\s*\()/i,keyword:/\b(?:ACTION|ADD|AFTER|ALGORITHM|ALL|ALTER|ANALYZE|ANY|APPLY|AS|ASC|AUTHORIZATION|AUTO_INCREMENT|BACKUP|BDB|BEGIN|BERKELEYDB|BIGINT|BINARY|BIT|BLOB|BOOL|BOOLEAN|BREAK|BROWSE|BTREE|BULK|BY|CALL|CASCADED?|CASE|CHAIN|CHAR(?:ACTER|SET)?|CHECK(?:POINT)?|CLOSE|CLUSTERED|COALESCE|COLLATE|COLUMNS?|COMMENT|COMMIT(?:TED)?|COMPUTE|CONNECT|CONSISTENT|CONSTRAINT|CONTAINS(?:TABLE)?|CONTINUE|CONVERT|CREATE|CROSS|CURRENT(?:_DATE|_TIME|_TIMESTAMP|_USER)?|CURSOR|CYCLE|DATA(?:BASES?)?|DATE(?:TIME)?|DAY|DBCC|DEALLOCATE|DEC|DECIMAL|DECLARE|DEFAULT|DEFINER|DELAYED|DELETE|DELIMITERS?|DENY|DESC|DESCRIBE|DETERMINISTIC|DISABLE|DISCARD|DISK|DISTINCT|DISTINCTROW|DISTRIBUTED|DO|DOUBLE|DROP|DUMMY|DUMP(?:FILE)?|DUPLICATE|ELSE(?:IF)?|ENABLE|ENCLOSED|END|ENGINE|ENUM|ERRLVL|ERRORS|ESCAPED?|EXCEPT|EXEC(?:UTE)?|EXISTS|EXIT|EXPLAIN|EXTENDED|FETCH|FIELDS|FILE|FILLFACTOR|FIRST|FIXED|FLOAT|FOLLOWING|FOR(?: EACH ROW)?|FORCE|FOREIGN|FREETEXT(?:TABLE)?|FROM|FULL|FUNCTION|GEOMETRY(?:COLLECTION)?|GLOBAL|GOTO|GRANT|GROUP|HANDLER|HASH|HAVING|HOLDLOCK|HOUR|IDENTITY(?:COL|_INSERT)?|IF|IGNORE|IMPORT|INDEX|INFILE|INNER|INNODB|INOUT|INSERT|INT|INTEGER|INTERSECT|INTERVAL|INTO|INVOKER|ISOLATION|ITERATE|JOIN|KEYS?|KILL|LANGUAGE|LAST|LEAVE|LEFT|LEVEL|LIMIT|LINENO|LINES|LINESTRING|LOAD|LOCAL|LOCK|LONG(?:BLOB|TEXT)|LOOP|MATCH(?:ED)?|MEDIUM(?:BLOB|INT|TEXT)|MERGE|MIDDLEINT|MINUTE|MODE|MODIFIES|MODIFY|MONTH|MULTI(?:LINESTRING|POINT|POLYGON)|NATIONAL|NATURAL|NCHAR|NEXT|NO|NONCLUSTERED|NULLIF|NUMERIC|OFF?|OFFSETS?|ON|OPEN(?:DATASOURCE|QUERY|ROWSET)?|OPTIMIZE|OPTION(?:ALLY)?|ORDER|OUT(?:ER|FILE)?|OVER|PARTIAL|PARTITION|PERCENT|PIVOT|PLAN|POINT|POLYGON|PRECEDING|PRECISION|PREPARE|PREV|PRIMARY|PRINT|PRIVILEGES|PROC(?:EDURE)?|PUBLIC|PURGE|QUICK|RAISERROR|READS?|REAL|RECONFIGURE|REFERENCES|RELEASE|RENAME|REPEAT(?:ABLE)?|REPLACE|REPLICATION|REQUIRE|RESIGNAL|RESTORE|RESTRICT|RETURN(?:ING|S)?|REVOKE|RIGHT|ROLLBACK|ROUTINE|ROW(?:COUNT|GUIDCOL|S)?|RTREE|RULE|SAVE(?:POINT)?|SCHEMA|SECOND|SELECT|SERIAL(?:IZABLE)?|SESSION(?:_USER)?|SET(?:USER)?|SHARE|SHOW|SHUTDOWN|SIMPLE|SMALLINT|SNAPSHOT|SOME|SONAME|SQL|START(?:ING)?|STATISTICS|STATUS|STRIPED|SYSTEM_USER|TABLES?|TABLESPACE|TEMP(?:ORARY|TABLE)?|TERMINATED|TEXT(?:SIZE)?|THEN|TIME(?:STAMP)?|TINY(?:BLOB|INT|TEXT)|TOP?|TRAN(?:SACTIONS?)?|TRIGGER|TRUNCATE|TSEQUAL|TYPES?|UNBOUNDED|UNCOMMITTED|UNDEFINED|UNION|UNIQUE|UNLOCK|UNPIVOT|UNSIGNED|UPDATE(?:TEXT)?|USAGE|USE|USER|USING|VALUES?|VAR(?:BINARY|CHAR|CHARACTER|YING)|VIEW|WAITFOR|WARNINGS|WHEN|WHERE|WHILE|WITH(?: ROLLUP|IN)?|WORK|WRITE(?:TEXT)?|YEAR)\b/i,boolean:/\b(?:FALSE|NULL|TRUE)\b/i,number:/\b0x[\da-f]+\b|\b\d+(?:\.\d*)?|\B\.\d+\b/i,operator:/[-+*\/=%^~]|&&?|\|\|?|!=?|<(?:=>?|<|>)?|>[>=]?|\b(?:AND|BETWEEN|DIV|ILIKE|IN|IS|LIKE|NOT|OR|REGEXP|RLIKE|SOUNDS LIKE|XOR)\b/i,punctuation:/[;[\]()`,.]/}}return NC}var LC,s$;function fnt(){if(s$)return LC;s$=1;var e=hL();LC=t,t.displayName="apex",t.aliases=[];function t(r){r.register(e),(function(n){var a=/\b(?:(?:after|before)(?=\s+[a-z])|abstract|activate|and|any|array|as|asc|autonomous|begin|bigdecimal|blob|boolean|break|bulk|by|byte|case|cast|catch|char|class|collect|commit|const|continue|currency|date|datetime|decimal|default|delete|desc|do|double|else|end|enum|exception|exit|export|extends|final|finally|float|for|from|get(?=\s*[{};])|global|goto|group|having|hint|if|implements|import|in|inner|insert|instanceof|int|integer|interface|into|join|like|limit|list|long|loop|map|merge|new|not|null|nulls|number|object|of|on|or|outer|override|package|parallel|pragma|private|protected|public|retrieve|return|rollback|select|set|short|sObject|sort|static|string|super|switch|synchronized|system|testmethod|then|this|throw|time|transaction|transient|trigger|try|undelete|update|upsert|using|virtual|void|webservice|when|where|while|(?:inherited|with|without)\s+sharing)\b/i,i=/\b(?:(?=[a-z_]\w*\s*[<\[])|(?!<keyword>))[A-Z_]\w*(?:\s*\.\s*[A-Z_]\w*)*\b(?:\s*(?:\[\s*\]|<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>))*/.source.replace(/<keyword>/g,function(){return a.source});function o(l){return RegExp(l.replace(/<CLASS-NAME>/g,function(){return i}),"i")}var s={keyword:a,punctuation:/[()\[\]{};,:.<>]/};n.languages.apex={comment:n.languages.clike.comment,string:n.languages.clike.string,sql:{pattern:/((?:[=,({:]|\breturn)\s*)\[[^\[\]]*\]/i,lookbehind:!0,greedy:!0,alias:"language-sql",inside:n.languages.sql},annotation:{pattern:/@\w+\b/,alias:"punctuation"},"class-name":[{pattern:o(/(\b(?:class|enum|extends|implements|instanceof|interface|new|trigger\s+\w+\s+on)\s+)<CLASS-NAME>/.source),lookbehind:!0,inside:s},{pattern:o(/(\(\s*)<CLASS-NAME>(?=\s*\)\s*[\w(])/.source),lookbehind:!0,inside:s},{pattern:o(/<CLASS-NAME>(?=\s*\w+\s*[;=,(){:])/.source),inside:s}],trigger:{pattern:/(\btrigger\s+)\w+\b/i,lookbehind:!0,alias:"class-name"},keyword:a,function:/\b[a-z_]\w*(?=\s*\()/i,boolean:/\b(?:false|true)\b/i,number:/(?:\B\.\d+|\b\d+(?:\.\d+|L)?)\b/i,operator:/[!=](?:==?)?|\?\.?|&&|\|\||--|\+\+|[-+*/^&|]=?|:|<<?=?|>{1,3}=?/,punctuation:/[()\[\]{};,.]/}})(r)}return LC}var MC,l$;function mnt(){if(l$)return MC;l$=1,MC=e,e.displayName="apl",e.aliases=[];function e(t){t.languages.apl={comment:/(?:⍝|#[! ]).*$/m,string:{pattern:/'(?:[^'\r\n]|'')*'/,greedy:!0},number:/¯?(?:\d*\.?\b\d+(?:e[+¯]?\d+)?|¯|∞)(?:j¯?(?:(?:\d+(?:\.\d+)?|\.\d+)(?:e[+¯]?\d+)?|¯|∞))?/i,statement:/:[A-Z][a-z][A-Za-z]*\b/,"system-function":{pattern:/⎕[A-Z]+/i,alias:"function"},constant:/[⍬⌾#⎕⍞]/,function:/[-+×÷⌈⌊∣|⍳⍸?*⍟○!⌹<≤=>≥≠≡≢∊⍷∪∩~∨∧⍱⍲⍴,⍪⌽⊖⍉↑↓⊂⊃⊆⊇⌷⍋⍒⊤⊥⍕⍎⊣⊢⍁⍂≈⍯↗¤→]/,"monadic-operator":{pattern:/[\\\/⌿⍀¨⍨⌶&∥]/,alias:"operator"},"dyadic-operator":{pattern:/[.⍣⍠⍤∘⌸@⌺⍥]/,alias:"operator"},assignment:{pattern:/←/,alias:"keyword"},punctuation:/[\[;\]()◇⋄]/,dfn:{pattern:/[{}⍺⍵⍶⍹∇⍫:]/,alias:"builtin"}}}return MC}var IC,u$;function gnt(){if(u$)return IC;u$=1,IC=e,e.displayName="applescript",e.aliases=[];function e(t){t.languages.applescript={comment:[/\(\*(?:\(\*(?:[^*]|\*(?!\)))*\*\)|(?!\(\*)[\s\S])*?\*\)/,/--.+/,/#.+/],string:/"(?:\\.|[^"\\\r\n])*"/,number:/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e-?\d+)?\b/i,operator:[/[&=≠≤≥*+\-\/÷^]|[<>]=?/,/\b(?:(?:begin|end|start)s? with|(?:contains?|(?:does not|doesn't) contain)|(?:is|isn't|is not) (?:contained by|in)|(?:(?:is|isn't|is not) )?(?:greater|less) than(?: or equal)?(?: to)?|(?:comes|(?:does not|doesn't) come) (?:after|before)|(?:is|isn't|is not) equal(?: to)?|(?:(?:does not|doesn't) equal|equal to|equals|is not|isn't)|(?:a )?(?:ref(?: to)?|reference to)|(?:and|as|div|mod|not|or))\b/],keyword:/\b(?:about|above|after|against|apart from|around|aside from|at|back|before|beginning|behind|below|beneath|beside|between|but|by|considering|continue|copy|does|eighth|else|end|equal|error|every|exit|false|fifth|first|for|fourth|from|front|get|given|global|if|ignoring|in|instead of|into|is|it|its|last|local|me|middle|my|ninth|of|on|onto|out of|over|prop|property|put|repeat|return|returning|second|set|seventh|since|sixth|some|tell|tenth|that|the|then|third|through|thru|timeout|times|to|transaction|true|try|until|where|while|whose|with|without)\b/,"class-name":/\b(?:POSIX file|RGB color|alias|application|boolean|centimeters|centimetres|class|constant|cubic centimeters|cubic centimetres|cubic feet|cubic inches|cubic meters|cubic metres|cubic yards|date|degrees Celsius|degrees Fahrenheit|degrees Kelvin|feet|file|gallons|grams|inches|integer|kilograms|kilometers|kilometres|list|liters|litres|meters|metres|miles|number|ounces|pounds|quarts|real|record|reference|script|square feet|square kilometers|square kilometres|square meters|square metres|square miles|square yards|text|yards)\b/,punctuation:/[{}():,¬«»《》]/}}return IC}var OC,c$;function bnt(){if(c$)return OC;c$=1,OC=e,e.displayName="aql",e.aliases=[];function e(t){t.languages.aql={comment:/\/\/.*|\/\*[\s\S]*?\*\//,property:{pattern:/([{,]\s*)(?:(?!\d)\w+|(["'´`])(?:(?!\2)[^\\\r\n]|\\.)*\2)(?=\s*:)/,lookbehind:!0,greedy:!0},string:{pattern:/(["'])(?:(?!\1)[^\\\r\n]|\\.)*\1/,greedy:!0},identifier:{pattern:/([´`])(?:(?!\1)[^\\\r\n]|\\.)*\1/,greedy:!0},variable:/@@?\w+/,keyword:[{pattern:/(\bWITH\s+)COUNT(?=\s+INTO\b)/i,lookbehind:!0},/\b(?:AGGREGATE|ALL|AND|ANY|ASC|COLLECT|DESC|DISTINCT|FILTER|FOR|GRAPH|IN|INBOUND|INSERT|INTO|K_PATHS|K_SHORTEST_PATHS|LET|LIKE|LIMIT|NONE|NOT|NULL|OR|OUTBOUND|REMOVE|REPLACE|RETURN|SHORTEST_PATH|SORT|UPDATE|UPSERT|WINDOW|WITH)\b/i,{pattern:/(^|[^\w.[])(?:KEEP|PRUNE|SEARCH|TO)\b/i,lookbehind:!0},{pattern:/(^|[^\w.[])(?:CURRENT|NEW|OLD)\b/,lookbehind:!0},{pattern:/\bOPTIONS(?=\s*\{)/i}],function:/\b(?!\d)\w+(?=\s*\()/,boolean:/\b(?:false|true)\b/i,range:{pattern:/\.\./,alias:"operator"},number:[/\b0b[01]+/i,/\b0x[0-9a-f]+/i,/(?:\B\.\d+|\b(?:0|[1-9]\d*)(?:\.\d+)?)(?:e[+-]?\d+)?/i],operator:/\*{2,}|[=!]~|[!=<>]=?|&&|\|\||[-+*/%]/,punctuation:/::|[?.:,;()[\]{}]/}}return OC}var BC,d$;function pd(){if(d$)return BC;d$=1,BC=e,e.displayName="c",e.aliases=[];function e(t){t.languages.c=t.languages.extend("clike",{comment:{pattern:/\/\/(?:[^\r\n\\]|\\(?:\r\n?|\n|(?![\r\n])))*|\/\*[\s\S]*?(?:\*\/|$)/,greedy:!0},string:{pattern:/"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,greedy:!0},"class-name":{pattern:/(\b(?:enum|struct)\s+(?:__attribute__\s*\(\([\s\S]*?\)\)\s*)?)\w+|\b[a-z]\w*_t\b/,lookbehind:!0},keyword:/\b(?:_Alignas|_Alignof|_Atomic|_Bool|_Complex|_Generic|_Imaginary|_Noreturn|_Static_assert|_Thread_local|__attribute__|asm|auto|break|case|char|const|continue|default|do|double|else|enum|extern|float|for|goto|if|inline|int|long|register|return|short|signed|sizeof|static|struct|switch|typedef|typeof|union|unsigned|void|volatile|while)\b/,function:/\b[a-z_]\w*(?=\s*\()/i,number:/(?:\b0x(?:[\da-f]+(?:\.[\da-f]*)?|\.[\da-f]+)(?:p[+-]?\d+)?|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)[ful]{0,4}/i,operator:/>>=?|<<=?|->|([-+&|:])\1|[?:~]|[-+*/%&|^!=<>]=?/}),t.languages.insertBefore("c","string",{char:{pattern:/'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n]){0,32}'/,greedy:!0}}),t.languages.insertBefore("c","string",{macro:{pattern:/(^[\t ]*)#\s*[a-z](?:[^\r\n\\/]|\/(?!\*)|\/\*(?:[^*]|\*(?!\/))*\*\/|\\(?:\r\n|[\s\S]))*/im,lookbehind:!0,greedy:!0,alias:"property",inside:{string:[{pattern:/^(#\s*include\s*)<[^>]+>/,lookbehind:!0},t.languages.c.string],char:t.languages.c.char,comment:t.languages.c.comment,"macro-name":[{pattern:/(^#\s*define\s+)\w+\b(?!\()/i,lookbehind:!0},{pattern:/(^#\s*define\s+)\w+\b(?=\()/i,lookbehind:!0,alias:"function"}],directive:{pattern:/^(#\s*)[a-z]+/,lookbehind:!0,alias:"keyword"},"directive-hash":/^#/,punctuation:/##|\\(?=[\r\n])/,expression:{pattern:/\S[\s\S]*/,inside:t.languages.c}}}}),t.languages.insertBefore("c","function",{constant:/\b(?:EOF|NULL|SEEK_CUR|SEEK_END|SEEK_SET|__DATE__|__FILE__|__LINE__|__TIMESTAMP__|__TIME__|__func__|stderr|stdin|stdout)\b/}),delete t.languages.c.boolean}return BC}var PC,h$;function pL(){if(h$)return PC;h$=1;var e=pd();PC=t,t.displayName="cpp",t.aliases=[];function t(r){r.register(e),(function(n){var a=/\b(?:alignas|alignof|asm|auto|bool|break|case|catch|char|char16_t|char32_t|char8_t|class|co_await|co_return|co_yield|compl|concept|const|const_cast|consteval|constexpr|constinit|continue|decltype|default|delete|do|double|dynamic_cast|else|enum|explicit|export|extern|final|float|for|friend|goto|if|import|inline|int|int16_t|int32_t|int64_t|int8_t|long|module|mutable|namespace|new|noexcept|nullptr|operator|override|private|protected|public|register|reinterpret_cast|requires|return|short|signed|sizeof|static|static_assert|static_cast|struct|switch|template|this|thread_local|throw|try|typedef|typeid|typename|uint16_t|uint32_t|uint64_t|uint8_t|union|unsigned|using|virtual|void|volatile|wchar_t|while)\b/,i=/\b(?!<keyword>)\w+(?:\s*\.\s*\w+)*\b/.source.replace(/<keyword>/g,function(){return a.source});n.languages.cpp=n.languages.extend("c",{"class-name":[{pattern:RegExp(/(\b(?:class|concept|enum|struct|typename)\s+)(?!<keyword>)\w+/.source.replace(/<keyword>/g,function(){return a.source})),lookbehind:!0},/\b[A-Z]\w*(?=\s*::\s*\w+\s*\()/,/\b[A-Z_]\w*(?=\s*::\s*~\w+\s*\()/i,/\b\w+(?=\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>\s*::\s*\w+\s*\()/],keyword:a,number:{pattern:/(?:\b0b[01']+|\b0x(?:[\da-f']+(?:\.[\da-f']*)?|\.[\da-f']+)(?:p[+-]?[\d']+)?|(?:\b[\d']+(?:\.[\d']*)?|\B\.[\d']+)(?:e[+-]?[\d']+)?)[ful]{0,4}/i,greedy:!0},operator:/>>=?|<<=?|->|--|\+\+|&&|\|\||[?:~]|<=>|[-+*/%&|^!=<>]=?|\b(?:and|and_eq|bitand|bitor|not|not_eq|or|or_eq|xor|xor_eq)\b/,boolean:/\b(?:false|true)\b/}),n.languages.insertBefore("cpp","string",{module:{pattern:RegExp(/(\b(?:import|module)\s+)/.source+"(?:"+/"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"|<[^<>\r\n]*>/.source+"|"+/<mod-name>(?:\s*:\s*<mod-name>)?|:\s*<mod-name>/.source.replace(/<mod-name>/g,function(){return i})+")"),lookbehind:!0,greedy:!0,inside:{string:/^[<"][\s\S]+/,operator:/:/,punctuation:/\./}},"raw-string":{pattern:/R"([^()\\ ]{0,16})\([\s\S]*?\)\1"/,alias:"string",greedy:!0}}),n.languages.insertBefore("cpp","keyword",{"generic-function":{pattern:/\b(?!operator\b)[a-z_]\w*\s*<(?:[^<>]|<[^<>]*>)*>(?=\s*\()/i,inside:{function:/^\w+/,generic:{pattern:/<[\s\S]+/,alias:"class-name",inside:n.languages.cpp}}}}),n.languages.insertBefore("cpp","operator",{"double-colon":{pattern:/::/,alias:"punctuation"}}),n.languages.insertBefore("cpp","class-name",{"base-clause":{pattern:/(\b(?:class|struct)\s+\w+\s*:\s*)[^;{}"'\s]+(?:\s+[^;{}"'\s]+)*(?=\s*[;{])/,lookbehind:!0,greedy:!0,inside:n.languages.extend("cpp",{})}}),n.languages.insertBefore("inside","double-colon",{"class-name":/\b[a-z_]\w*\b(?!\s*::)/i},n.languages.cpp["base-clause"])})(r)}return PC}var zC,p$;function ynt(){if(p$)return zC;p$=1;var e=pL();zC=t,t.displayName="arduino",t.aliases=["ino"];function t(r){r.register(e),r.languages.arduino=r.languages.extend("cpp",{keyword:/\b(?:String|array|bool|boolean|break|byte|case|catch|continue|default|do|double|else|finally|for|function|goto|if|in|instanceof|int|integer|long|loop|new|null|return|setup|string|switch|throw|try|void|while|word)\b/,constant:/\b(?:ANALOG_MESSAGE|DEFAULT|DIGITAL_MESSAGE|EXTERNAL|FIRMATA_STRING|HIGH|INPUT|INPUT_PULLUP|INTERNAL|INTERNAL1V1|INTERNAL2V56|LED_BUILTIN|LOW|OUTPUT|REPORT_ANALOG|REPORT_DIGITAL|SET_PIN_MODE|SYSEX_START|SYSTEM_RESET)\b/,builtin:/\b(?:Audio|BSSID|Bridge|Client|Console|EEPROM|Esplora|EsploraTFT|Ethernet|EthernetClient|EthernetServer|EthernetUDP|File|FileIO|FileSystem|Firmata|GPRS|GSM|GSMBand|GSMClient|GSMModem|GSMPIN|GSMScanner|GSMServer|GSMVoiceCall|GSM_SMS|HttpClient|IPAddress|IRread|Keyboard|KeyboardController|LiquidCrystal|LiquidCrystal_I2C|Mailbox|Mouse|MouseController|PImage|Process|RSSI|RobotControl|RobotMotor|SD|SPI|SSID|Scheduler|Serial|Server|Servo|SoftwareSerial|Stepper|Stream|TFT|Task|USBHost|WiFi|WiFiClient|WiFiServer|WiFiUDP|Wire|YunClient|YunServer|abs|addParameter|analogRead|analogReadResolution|analogReference|analogWrite|analogWriteResolution|answerCall|attach|attachGPRS|attachInterrupt|attached|autoscroll|available|background|beep|begin|beginPacket|beginSD|beginSMS|beginSpeaker|beginTFT|beginTransmission|beginWrite|bit|bitClear|bitRead|bitSet|bitWrite|blink|blinkVersion|buffer|changePIN|checkPIN|checkPUK|checkReg|circle|cityNameRead|cityNameWrite|clear|clearScreen|click|close|compassRead|config|connect|connected|constrain|cos|countryNameRead|countryNameWrite|createChar|cursor|debugPrint|delay|delayMicroseconds|detach|detachInterrupt|digitalRead|digitalWrite|disconnect|display|displayLogos|drawBMP|drawCompass|encryptionType|end|endPacket|endSMS|endTransmission|endWrite|exists|exitValue|fill|find|findUntil|flush|gatewayIP|get|getAsynchronously|getBand|getButton|getCurrentCarrier|getIMEI|getKey|getModifiers|getOemKey|getPINUsed|getResult|getSignalStrength|getSocket|getVoiceCallStatus|getXChange|getYChange|hangCall|height|highByte|home|image|interrupts|isActionDone|isDirectory|isListening|isPIN|isPressed|isValid|keyPressed|keyReleased|keyboardRead|knobRead|leftToRight|line|lineFollowConfig|listen|listenOnLocalhost|loadImage|localIP|lowByte|macAddress|maintain|map|max|messageAvailable|micros|millis|min|mkdir|motorsStop|motorsWrite|mouseDragged|mouseMoved|mousePressed|mouseReleased|move|noAutoscroll|noBlink|noBuffer|noCursor|noDisplay|noFill|noInterrupts|noListenOnLocalhost|noStroke|noTone|onReceive|onRequest|open|openNextFile|overflow|parseCommand|parseFloat|parseInt|parsePacket|pauseMode|peek|pinMode|playFile|playMelody|point|pointTo|position|pow|prepare|press|print|printFirmwareVersion|printVersion|println|process|processInput|pulseIn|put|random|randomSeed|read|readAccelerometer|readBlue|readButton|readBytes|readBytesUntil|readGreen|readJoystickButton|readJoystickSwitch|readJoystickX|readJoystickY|readLightSensor|readMessage|readMicrophone|readNetworks|readRed|readSlider|readString|readStringUntil|readTemperature|ready|rect|release|releaseAll|remoteIP|remoteNumber|remotePort|remove|requestFrom|retrieveCallingNumber|rewindDirectory|rightToLeft|rmdir|robotNameRead|robotNameWrite|run|runAsynchronously|runShellCommand|runShellCommandAsynchronously|running|scanNetworks|scrollDisplayLeft|scrollDisplayRight|seek|sendAnalog|sendDigitalPortPair|sendDigitalPorts|sendString|sendSysex|serialEvent|setBand|setBitOrder|setClockDivider|setCursor|setDNS|setDataMode|setFirmwareVersion|setMode|setPINUsed|setSpeed|setTextSize|setTimeout|shiftIn|shiftOut|shutdown|sin|size|sqrt|startLoop|step|stop|stroke|subnetMask|switchPIN|tan|tempoWrite|text|tone|transfer|tuneWrite|turn|updateIR|userNameRead|userNameWrite|voiceCall|waitContinue|width|write|writeBlue|writeGreen|writeJSON|writeMessage|writeMicroseconds|writeRGB|writeRed|yield)\b/}),r.languages.ino=r.languages.arduino}return zC}var HC,f$;function vnt(){if(f$)return HC;f$=1,HC=e,e.displayName="arff",e.aliases=[];function e(t){t.languages.arff={comment:/%.*/,string:{pattern:/(["'])(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},keyword:/@(?:attribute|data|end|relation)\b/i,number:/\b\d+(?:\.\d+)?\b/,punctuation:/[{},]/}}return HC}var UC,m$;function Snt(){if(m$)return UC;m$=1,UC=e,e.displayName="asciidoc",e.aliases=["adoc"];function e(t){(function(r){var n={pattern:/(^[ \t]*)\[(?!\[)(?:(["'$`])(?:(?!\2)[^\\]|\\.)*\2|\[(?:[^\[\]\\]|\\.)*\]|[^\[\]\\"'$`]|\\.)*\]/m,lookbehind:!0,inside:{quoted:{pattern:/([$`])(?:(?!\1)[^\\]|\\.)*\1/,inside:{punctuation:/^[$`]|[$`]$/}},interpreted:{pattern:/'(?:[^'\\]|\\.)*'/,inside:{punctuation:/^'|'$/}},string:/"(?:[^"\\]|\\.)*"/,variable:/\w+(?==)/,punctuation:/^\[|\]$|,/,operator:/=/,"attr-value":/(?!^\s+$).+/}},a=r.languages.asciidoc={"comment-block":{pattern:/^(\/{4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1/m,alias:"comment"},table:{pattern:/^\|={3,}(?:(?:\r?\n|\r(?!\n)).*)*?(?:\r?\n|\r)\|={3,}$/m,inside:{specifiers:{pattern:/(?:(?:(?:\d+(?:\.\d+)?|\.\d+)[+*](?:[<^>](?:\.[<^>])?|\.[<^>])?|[<^>](?:\.[<^>])?|\.[<^>])[a-z]*|[a-z]+)(?=\|)/,alias:"attr-value"},punctuation:{pattern:/(^|[^\\])[|!]=*/,lookbehind:!0}}},"passthrough-block":{pattern:/^(\+{4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1$/m,inside:{punctuation:/^\++|\++$/}},"literal-block":{pattern:/^(-{4,}|\.{4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1$/m,inside:{punctuation:/^(?:-+|\.+)|(?:-+|\.+)$/}},"other-block":{pattern:/^(--|\*{4,}|_{4,}|={4,})(?:\r?\n|\r)(?:[\s\S]*(?:\r?\n|\r))??\1$/m,inside:{punctuation:/^(?:-+|\*+|_+|=+)|(?:-+|\*+|_+|=+)$/}},"list-punctuation":{pattern:/(^[ \t]*)(?:-|\*{1,5}|\.{1,5}|(?:[a-z]|\d+)\.|[xvi]+\))(?= )/im,lookbehind:!0,alias:"punctuation"},"list-label":{pattern:/(^[ \t]*)[a-z\d].+(?::{2,4}|;;)(?=\s)/im,lookbehind:!0,alias:"symbol"},"indented-block":{pattern:/((\r?\n|\r)\2)([ \t]+)\S.*(?:(?:\r?\n|\r)\3.+)*(?=\2{2}|$)/,lookbehind:!0},comment:/^\/\/.*/m,title:{pattern:/^.+(?:\r?\n|\r)(?:={3,}|-{3,}|~{3,}|\^{3,}|\+{3,})$|^={1,5} .+|^\.(?![\s.]).*/m,alias:"important",inside:{punctuation:/^(?:\.|=+)|(?:=+|-+|~+|\^+|\++)$/}},"attribute-entry":{pattern:/^:[^:\r\n]+:(?: .*?(?: \+(?:\r?\n|\r).*?)*)?$/m,alias:"tag"},attributes:n,hr:{pattern:/^'{3,}$/m,alias:"punctuation"},"page-break":{pattern:/^<{3,}$/m,alias:"punctuation"},admonition:{pattern:/^(?:CAUTION|IMPORTANT|NOTE|TIP|WARNING):/m,alias:"keyword"},callout:[{pattern:/(^[ \t]*)<?\d*>/m,lookbehind:!0,alias:"symbol"},{pattern:/<\d+>/,alias:"symbol"}],macro:{pattern:/\b[a-z\d][a-z\d-]*::?(?:[^\s\[\]]*\[(?:[^\]\\"']|(["'])(?:(?!\1)[^\\]|\\.)*\1|\\.)*\])/,inside:{function:/^[a-z\d-]+(?=:)/,punctuation:/^::?/,attributes:{pattern:/(?:\[(?:[^\]\\"']|(["'])(?:(?!\1)[^\\]|\\.)*\1|\\.)*\])/,inside:n.inside}}},inline:{pattern:/(^|[^\\])(?:(?:\B\[(?:[^\]\\"']|(["'])(?:(?!\2)[^\\]|\\.)*\2|\\.)*\])?(?:\b_(?!\s)(?: _|[^_\\\r\n]|\\.)+(?:(?:\r?\n|\r)(?: _|[^_\\\r\n]|\\.)+)*_\b|\B``(?!\s).+?(?:(?:\r?\n|\r).+?)*''\B|\B`(?!\s)(?:[^`'\s]|\s+\S)+['`]\B|\B(['*+#])(?!\s)(?: \3|(?!\3)[^\\\r\n]|\\.)+(?:(?:\r?\n|\r)(?: \3|(?!\3)[^\\\r\n]|\\.)+)*\3\B)|(?:\[(?:[^\]\\"']|(["'])(?:(?!\4)[^\\]|\\.)*\4|\\.)*\])?(?:(__|\*\*|\+\+\+?|##|\$\$|[~^]).+?(?:(?:\r?\n|\r).+?)*\5|\{[^}\r\n]+\}|\[\[\[?.+?(?:(?:\r?\n|\r).+?)*\]?\]\]|<<.+?(?:(?:\r?\n|\r).+?)*>>|\(\(\(?.+?(?:(?:\r?\n|\r).+?)*\)?\)\)))/m,lookbehind:!0,inside:{attributes:n,url:{pattern:/^(?:\[\[\[?.+?\]?\]\]|<<.+?>>)$/,inside:{punctuation:/^(?:\[\[\[?|<<)|(?:\]\]\]?|>>)$/}},"attribute-ref":{pattern:/^\{.+\}$/,inside:{variable:{pattern:/(^\{)[a-z\d,+_-]+/,lookbehind:!0},operator:/^[=?!#%@$]|!(?=[:}])/,punctuation:/^\{|\}$|::?/}},italic:{pattern:/^(['_])[\s\S]+\1$/,inside:{punctuation:/^(?:''?|__?)|(?:''?|__?)$/}},bold:{pattern:/^\*[\s\S]+\*$/,inside:{punctuation:/^\*\*?|\*\*?$/}},punctuation:/^(?:``?|\+{1,3}|##?|\$\$|[~^]|\(\(\(?)|(?:''?|\+{1,3}|##?|\$\$|[~^`]|\)?\)\))$/}},replacement:{pattern:/\((?:C|R|TM)\)/,alias:"builtin"},entity:/&#?[\da-z]{1,8};/i,"line-continuation":{pattern:/(^| )\+$/m,lookbehind:!0,alias:"punctuation"}};function i(o){o=o.split(" ");for(var s={},l=0,u=o.length;l<u;l++)s[o[l]]=a[o[l]];return s}n.inside.interpreted.inside.rest=i("macro inline replacement entity"),a["passthrough-block"].inside.rest=i("macro"),a["literal-block"].inside.rest=i("callout"),a.table.inside.rest=i("comment-block passthrough-block literal-block other-block list-punctuation indented-block comment title attribute-entry attributes hr page-break admonition list-label callout macro inline replacement entity line-continuation"),a["other-block"].inside.rest=i("table list-punctuation indented-block comment attribute-entry attributes hr page-break admonition list-label macro inline replacement entity line-continuation"),a.title.inside.rest=i("macro inline replacement entity"),r.hooks.add("wrap",function(o){o.type==="entity"&&(o.attributes.title=o.content.value.replace(/&amp;/,"&"))}),r.languages.adoc=r.languages.asciidoc})(t)}return UC}var $C,g$;function Ent(){if(g$)return $C;g$=1,$C=e,e.displayName="asm6502",e.aliases=[];function e(t){t.languages.asm6502={comment:/;.*/,directive:{pattern:/\.\w+(?= )/,alias:"property"},string:/(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,"op-code":{pattern:/\b(?:ADC|AND|ASL|BCC|BCS|BEQ|BIT|BMI|BNE|BPL|BRK|BVC|BVS|CLC|CLD|CLI|CLV|CMP|CPX|CPY|DEC|DEX|DEY|EOR|INC|INX|INY|JMP|JSR|LDA|LDX|LDY|LSR|NOP|ORA|PHA|PHP|PLA|PLP|ROL|ROR|RTI|RTS|SBC|SEC|SED|SEI|STA|STX|STY|TAX|TAY|TSX|TXA|TXS|TYA|adc|and|asl|bcc|bcs|beq|bit|bmi|bne|bpl|brk|bvc|bvs|clc|cld|cli|clv|cmp|cpx|cpy|dec|dex|dey|eor|inc|inx|iny|jmp|jsr|lda|ldx|ldy|lsr|nop|ora|pha|php|pla|plp|rol|ror|rti|rts|sbc|sec|sed|sei|sta|stx|sty|tax|tay|tsx|txa|txs|tya)\b/,alias:"keyword"},"hex-number":{pattern:/#?\$[\da-f]{1,4}\b/i,alias:"number"},"binary-number":{pattern:/#?%[01]+\b/,alias:"number"},"decimal-number":{pattern:/#?\b\d+\b/,alias:"number"},register:{pattern:/\b[xya]\b/i,alias:"variable"},punctuation:/[(),:]/}}return $C}var GC,b$;function xnt(){if(b$)return GC;b$=1,GC=e,e.displayName="asmatmel",e.aliases=[];function e(t){t.languages.asmatmel={comment:{pattern:/;.*/,greedy:!0},string:{pattern:/(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},constant:/\b(?:PORT[A-Z]|DDR[A-Z]|(?:DD|P)[A-Z](?:\d|[0-2]\d|3[01]))\b/,directive:{pattern:/\.\w+(?= )/,alias:"property"},"r-register":{pattern:/\br(?:\d|[12]\d|3[01])\b/,alias:"variable"},"op-code":{pattern:/\b(?:ADC|ADD|ADIW|AND|ANDI|ASR|BCLR|BLD|BRBC|BRBS|BRCC|BRCS|BREAK|BREQ|BRGE|BRHC|BRHS|BRID|BRIE|BRLO|BRLT|BRMI|BRNE|BRPL|BRSH|BRTC|BRTS|BRVC|BRVS|BSET|BST|CALL|CBI|CBR|CLC|CLH|CLI|CLN|CLR|CLS|CLT|CLV|CLZ|COM|CP|CPC|CPI|CPSE|DEC|DES|EICALL|EIJMP|ELPM|EOR|FMUL|FMULS|FMULSU|ICALL|IJMP|IN|INC|JMP|LAC|LAS|LAT|LD|LD[A-Za-z0-9]|LPM|LSL|LSR|MOV|MOVW|MUL|MULS|MULSU|NEG|NOP|OR|ORI|OUT|POP|PUSH|RCALL|RET|RETI|RJMP|ROL|ROR|SBC|SBCI|SBI|SBIC|SBIS|SBIW|SBR|SBRC|SBRS|SEC|SEH|SEI|SEN|SER|SES|SET|SEV|SEZ|SLEEP|SPM|ST|ST[A-Z0-9]|SUB|SUBI|SWAP|TST|WDR|XCH|adc|add|adiw|and|andi|asr|bclr|bld|brbc|brbs|brcc|brcs|break|breq|brge|brhc|brhs|brid|brie|brlo|brlt|brmi|brne|brpl|brsh|brtc|brts|brvc|brvs|bset|bst|call|cbi|cbr|clc|clh|cli|cln|clr|cls|clt|clv|clz|com|cp|cpc|cpi|cpse|dec|des|eicall|eijmp|elpm|eor|fmul|fmuls|fmulsu|icall|ijmp|in|inc|jmp|lac|las|lat|ld|ld[a-z0-9]|lpm|lsl|lsr|mov|movw|mul|muls|mulsu|neg|nop|or|ori|out|pop|push|rcall|ret|reti|rjmp|rol|ror|sbc|sbci|sbi|sbic|sbis|sbiw|sbr|sbrc|sbrs|sec|seh|sei|sen|ser|ses|set|sev|sez|sleep|spm|st|st[a-zA-Z0-9]|sub|subi|swap|tst|wdr|xch)\b/,alias:"keyword"},"hex-number":{pattern:/#?\$[\da-f]{2,4}\b/i,alias:"number"},"binary-number":{pattern:/#?%[01]+\b/,alias:"number"},"decimal-number":{pattern:/#?\b\d+\b/,alias:"number"},register:{pattern:/\b[acznvshtixy]\b/i,alias:"variable"},operator:/>>=?|<<=?|&&?|\|\|?|[-+*/%&|^!=<>?]=?/,punctuation:/[(),:]/}}return GC}var jC,y$;function rS(){if(y$)return jC;y$=1,jC=e,e.displayName="csharp",e.aliases=["dotnet","cs"];function e(t){(function(r){function n(K,G){return K.replace(/<<(\d+)>>/g,function(P,ne){return"(?:"+G[+ne]+")"})}function a(K,G,P){return RegExp(n(K,G),"")}function i(K,G){for(var P=0;P<G;P++)K=K.replace(/<<self>>/g,function(){return"(?:"+K+")"});return K.replace(/<<self>>/g,"[^\\s\\S]")}var o={type:"bool byte char decimal double dynamic float int long object sbyte short string uint ulong ushort var void",typeDeclaration:"class enum interface record struct",contextual:"add alias and ascending async await by descending from(?=\\s*(?:\\w|$)) get global group into init(?=\\s*;) join let nameof not notnull on or orderby partial remove select set unmanaged value when where with(?=\\s*{)",other:"abstract as base break case catch checked const continue default delegate do else event explicit extern finally fixed for foreach goto if implicit in internal is lock namespace new null operator out override params private protected public readonly ref return sealed sizeof stackalloc static switch this throw try typeof unchecked unsafe using virtual volatile while yield"};function s(K){return"\\b(?:"+K.trim().replace(/ /g,"|")+")\\b"}var l=s(o.typeDeclaration),u=RegExp(s(o.type+" "+o.typeDeclaration+" "+o.contextual+" "+o.other)),d=s(o.typeDeclaration+" "+o.contextual+" "+o.other),p=s(o.type+" "+o.typeDeclaration+" "+o.other),f=i(/<(?:[^<>;=+\-*/%&|^]|<<self>>)*>/.source,2),m=i(/\((?:[^()]|<<self>>)*\)/.source,2),b=/@?\b[A-Za-z_]\w*\b/.source,y=n(/<<0>>(?:\s*<<1>>)?/.source,[b,f]),S=n(/(?!<<0>>)<<1>>(?:\s*\.\s*<<1>>)*/.source,[d,y]),w=/\[\s*(?:,\s*)*\]/.source,E=n(/<<0>>(?:\s*(?:\?\s*)?<<1>>)*(?:\s*\?)?/.source,[S,w]),k=n(/[^,()<>[\];=+\-*/%&|^]|<<0>>|<<1>>|<<2>>/.source,[f,m,w]),_=n(/\(<<0>>+(?:,<<0>>+)+\)/.source,[k]),R=n(/(?:<<0>>|<<1>>)(?:\s*(?:\?\s*)?<<2>>)*(?:\s*\?)?/.source,[_,S,w]),C={keyword:u,punctuation:/[<>()?,.:[\]]/},F=/'(?:[^\r\n'\\]|\\.|\\[Uux][\da-fA-F]{1,8})'/.source,L=/"(?:\\.|[^\\"\r\n])*"/.source,I=/@"(?:""|\\[\s\S]|[^\\"])*"(?!")/.source;r.languages.csharp=r.languages.extend("clike",{string:[{pattern:a(/(^|[^$\\])<<0>>/.source,[I]),lookbehind:!0,greedy:!0},{pattern:a(/(^|[^@$\\])<<0>>/.source,[L]),lookbehind:!0,greedy:!0}],"class-name":[{pattern:a(/(\busing\s+static\s+)<<0>>(?=\s*;)/.source,[S]),lookbehind:!0,inside:C},{pattern:a(/(\busing\s+<<0>>\s*=\s*)<<1>>(?=\s*;)/.source,[b,R]),lookbehind:!0,inside:C},{pattern:a(/(\busing\s+)<<0>>(?=\s*=)/.source,[b]),lookbehind:!0},{pattern:a(/(\b<<0>>\s+)<<1>>/.source,[l,y]),lookbehind:!0,inside:C},{pattern:a(/(\bcatch\s*\(\s*)<<0>>/.source,[S]),lookbehind:!0,inside:C},{pattern:a(/(\bwhere\s+)<<0>>/.source,[b]),lookbehind:!0},{pattern:a(/(\b(?:is(?:\s+not)?|as)\s+)<<0>>/.source,[E]),lookbehind:!0,inside:C},{pattern:a(/\b<<0>>(?=\s+(?!<<1>>|with\s*\{)<<2>>(?:\s*[=,;:{)\]]|\s+(?:in|when)\b))/.source,[R,p,b]),inside:C}],keyword:u,number:/(?:\b0(?:x[\da-f_]*[\da-f]|b[01_]*[01])|(?:\B\.\d+(?:_+\d+)*|\b\d+(?:_+\d+)*(?:\.\d+(?:_+\d+)*)?)(?:e[-+]?\d+(?:_+\d+)*)?)(?:[dflmu]|lu|ul)?\b/i,operator:/>>=?|<<=?|[-=]>|([-+&|])\1|~|\?\?=?|[-+*/%&|^!=<>]=?/,punctuation:/\?\.?|::|[{}[\];(),.:]/}),r.languages.insertBefore("csharp","number",{range:{pattern:/\.\./,alias:"operator"}}),r.languages.insertBefore("csharp","punctuation",{"named-parameter":{pattern:a(/([(,]\s*)<<0>>(?=\s*:)/.source,[b]),lookbehind:!0,alias:"punctuation"}}),r.languages.insertBefore("csharp","class-name",{namespace:{pattern:a(/(\b(?:namespace|using)\s+)<<0>>(?:\s*\.\s*<<0>>)*(?=\s*[;{])/.source,[b]),lookbehind:!0,inside:{punctuation:/\./}},"type-expression":{pattern:a(/(\b(?:default|sizeof|typeof)\s*\(\s*(?!\s))(?:[^()\s]|\s(?!\s)|<<0>>)*(?=\s*\))/.source,[m]),lookbehind:!0,alias:"class-name",inside:C},"return-type":{pattern:a(/<<0>>(?=\s+(?:<<1>>\s*(?:=>|[({]|\.\s*this\s*\[)|this\s*\[))/.source,[R,S]),inside:C,alias:"class-name"},"constructor-invocation":{pattern:a(/(\bnew\s+)<<0>>(?=\s*[[({])/.source,[R]),lookbehind:!0,inside:C,alias:"class-name"},"generic-method":{pattern:a(/<<0>>\s*<<1>>(?=\s*\()/.source,[b,f]),inside:{function:a(/^<<0>>/.source,[b]),generic:{pattern:RegExp(f),alias:"class-name",inside:C}}},"type-list":{pattern:a(/\b((?:<<0>>\s+<<1>>|record\s+<<1>>\s*<<5>>|where\s+<<2>>)\s*:\s*)(?:<<3>>|<<4>>|<<1>>\s*<<5>>|<<6>>)(?:\s*,\s*(?:<<3>>|<<4>>|<<6>>))*(?=\s*(?:where|[{;]|=>|$))/.source,[l,y,b,R,u.source,m,/\bnew\s*\(\s*\)/.source]),lookbehind:!0,inside:{"record-arguments":{pattern:a(/(^(?!new\s*\()<<0>>\s*)<<1>>/.source,[y,m]),lookbehind:!0,greedy:!0,inside:r.languages.csharp},keyword:u,"class-name":{pattern:RegExp(R),greedy:!0,inside:C},punctuation:/[,()]/}},preprocessor:{pattern:/(^[\t ]*)#.*/m,lookbehind:!0,alias:"property",inside:{directive:{pattern:/(#)\b(?:define|elif|else|endif|endregion|error|if|line|nullable|pragma|region|undef|warning)\b/,lookbehind:!0,alias:"keyword"}}}});var M=L+"|"+F,j=n(/\/(?![*/])|\/\/[^\r\n]*[\r\n]|\/\*(?:[^*]|\*(?!\/))*\*\/|<<0>>/.source,[M]),Y=i(n(/[^"'/()]|<<0>>|\(<<self>>*\)/.source,[j]),2),W=/\b(?:assembly|event|field|method|module|param|property|return|type)\b/.source,$=n(/<<0>>(?:\s*\(<<1>>*\))?/.source,[S,Y]);r.languages.insertBefore("csharp","class-name",{attribute:{pattern:a(/((?:^|[^\s\w>)?])\s*\[\s*)(?:<<0>>\s*:\s*)?<<1>>(?:\s*,\s*<<1>>)*(?=\s*\])/.source,[W,$]),lookbehind:!0,greedy:!0,inside:{target:{pattern:a(/^<<0>>(?=\s*:)/.source,[W]),alias:"keyword"},"attribute-arguments":{pattern:a(/\(<<0>>*\)/.source,[Y]),inside:r.languages.csharp},"class-name":{pattern:RegExp(S),inside:{punctuation:/\./}},punctuation:/[:,]/}}});var X=/:[^}\r\n]+/.source,z=i(n(/[^"'/()]|<<0>>|\(<<self>>*\)/.source,[j]),2),H=n(/\{(?!\{)(?:(?![}:])<<0>>)*<<1>>?\}/.source,[z,X]),B=i(n(/[^"'/()]|\/(?!\*)|\/\*(?:[^*]|\*(?!\/))*\*\/|<<0>>|\(<<self>>*\)/.source,[M]),2),ee=n(/\{(?!\{)(?:(?![}:])<<0>>)*<<1>>?\}/.source,[B,X]);function J(K,G){return{interpolation:{pattern:a(/((?:^|[^{])(?:\{\{)*)<<0>>/.source,[K]),lookbehind:!0,inside:{"format-string":{pattern:a(/(^\{(?:(?![}:])<<0>>)*)<<1>>(?=\}$)/.source,[G,X]),lookbehind:!0,inside:{punctuation:/^:/}},punctuation:/^\{|\}$/,expression:{pattern:/[\s\S]+/,alias:"language-csharp",inside:r.languages.csharp}}},string:/[\s\S]+/}}r.languages.insertBefore("csharp","string",{"interpolation-string":[{pattern:a(/(^|[^\\])(?:\$@|@\$)"(?:""|\\[\s\S]|\{\{|<<0>>|[^\\{"])*"/.source,[H]),lookbehind:!0,greedy:!0,inside:J(H,z)},{pattern:a(/(^|[^@\\])\$"(?:\\.|\{\{|<<0>>|[^\\"{])*"/.source,[ee]),lookbehind:!0,greedy:!0,inside:J(ee,B)}],char:{pattern:RegExp(F),greedy:!0}}),r.languages.dotnet=r.languages.cs=r.languages.csharp})(t)}return jC}var qC,v$;function wnt(){if(v$)return qC;v$=1;var e=rS();qC=t,t.displayName="aspnet",t.aliases=[];function t(r){r.register(e),r.languages.aspnet=r.languages.extend("markup",{"page-directive":{pattern:/<%\s*@.*%>/,alias:"tag",inside:{"page-directive":{pattern:/<%\s*@\s*(?:Assembly|Control|Implements|Import|Master(?:Type)?|OutputCache|Page|PreviousPageType|Reference|Register)?|%>/i,alias:"tag"},rest:r.languages.markup.tag.inside}},directive:{pattern:/<%.*%>/,alias:"tag",inside:{directive:{pattern:/<%\s*?[$=%#:]{0,2}|%>/,alias:"tag"},rest:r.languages.csharp}}}),r.languages.aspnet.tag.pattern=/<(?!%)\/?[^\s>\/]+(?:\s+[^\s>\/=]+(?:=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">=]+))?)*\s*\/?>/,r.languages.insertBefore("inside","punctuation",{directive:r.languages.aspnet.directive},r.languages.aspnet.tag.inside["attr-value"]),r.languages.insertBefore("aspnet","comment",{"asp-comment":{pattern:/<%--[\s\S]*?--%>/,alias:["asp","comment"]}}),r.languages.insertBefore("aspnet",r.languages.javascript?"script":"tag",{"asp-script":{pattern:/(<script(?=.*runat=['"]?server\b)[^>]*>)[\s\S]*?(?=<\/script>)/i,lookbehind:!0,alias:["asp","script"],inside:r.languages.csharp||{}}})}return qC}var WC,S$;function knt(){if(S$)return WC;S$=1,WC=e,e.displayName="autohotkey",e.aliases=[];function e(t){t.languages.autohotkey={comment:[{pattern:/(^|\s);.*/,lookbehind:!0},{pattern:/(^[\t ]*)\/\*(?:[\r\n](?![ \t]*\*\/)|[^\r\n])*(?:[\r\n][ \t]*\*\/)?/m,lookbehind:!0,greedy:!0}],tag:{pattern:/^([ \t]*)[^\s,`":]+(?=:[ \t]*$)/m,lookbehind:!0},string:/"(?:[^"\n\r]|"")*"/,variable:/%\w+%/,number:/\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee]-?\d+)?/,operator:/\?|\/\/?=?|:=|\|[=|]?|&[=&]?|\+[=+]?|-[=-]?|\*[=*]?|<(?:<=?|>|=)?|>>?=?|[.^!=~]=?|\b(?:AND|NOT|OR)\b/,boolean:/\b(?:false|true)\b/,selector:/\b(?:AutoTrim|BlockInput|Break|Click|ClipWait|Continue|Control|ControlClick|ControlFocus|ControlGet|ControlGetFocus|ControlGetPos|ControlGetText|ControlMove|ControlSend|ControlSendRaw|ControlSetText|CoordMode|Critical|DetectHiddenText|DetectHiddenWindows|Drive|DriveGet|DriveSpaceFree|EnvAdd|EnvDiv|EnvGet|EnvMult|EnvSet|EnvSub|EnvUpdate|Exit|ExitApp|FileAppend|FileCopy|FileCopyDir|FileCreateDir|FileCreateShortcut|FileDelete|FileEncoding|FileGetAttrib|FileGetShortcut|FileGetSize|FileGetTime|FileGetVersion|FileInstall|FileMove|FileMoveDir|FileRead|FileReadLine|FileRecycle|FileRecycleEmpty|FileRemoveDir|FileSelectFile|FileSelectFolder|FileSetAttrib|FileSetTime|FormatTime|GetKeyState|Gosub|Goto|GroupActivate|GroupAdd|GroupClose|GroupDeactivate|Gui|GuiControl|GuiControlGet|Hotkey|ImageSearch|IniDelete|IniRead|IniWrite|Input|InputBox|KeyWait|ListHotkeys|ListLines|ListVars|Loop|Menu|MouseClick|MouseClickDrag|MouseGetPos|MouseMove|MsgBox|OnExit|OutputDebug|Pause|PixelGetColor|PixelSearch|PostMessage|Process|Progress|Random|RegDelete|RegRead|RegWrite|Reload|Repeat|Return|Run|RunAs|RunWait|Send|SendEvent|SendInput|SendMessage|SendMode|SendPlay|SendRaw|SetBatchLines|SetCapslockState|SetControlDelay|SetDefaultMouseSpeed|SetEnv|SetFormat|SetKeyDelay|SetMouseDelay|SetNumlockState|SetRegView|SetScrollLockState|SetStoreCapslockMode|SetTimer|SetTitleMatchMode|SetWinDelay|SetWorkingDir|Shutdown|Sleep|Sort|SoundBeep|SoundGet|SoundGetWaveVolume|SoundPlay|SoundSet|SoundSetWaveVolume|SplashImage|SplashTextOff|SplashTextOn|SplitPath|StatusBarGetText|StatusBarWait|StringCaseSense|StringGetPos|StringLeft|StringLen|StringLower|StringMid|StringReplace|StringRight|StringSplit|StringTrimLeft|StringTrimRight|StringUpper|Suspend|SysGet|Thread|ToolTip|Transform|TrayTip|URLDownloadToFile|WinActivate|WinActivateBottom|WinClose|WinGet|WinGetActiveStats|WinGetActiveTitle|WinGetClass|WinGetPos|WinGetText|WinGetTitle|WinHide|WinKill|WinMaximize|WinMenuSelectItem|WinMinimize|WinMinimizeAll|WinMinimizeAllUndo|WinMove|WinRestore|WinSet|WinSetTitle|WinShow|WinWait|WinWaitActive|WinWaitClose|WinWaitNotActive)\b/i,constant:/\b(?:a_ahkpath|a_ahkversion|a_appdata|a_appdatacommon|a_autotrim|a_batchlines|a_caretx|a_carety|a_computername|a_controldelay|a_cursor|a_dd|a_ddd|a_dddd|a_defaultmousespeed|a_desktop|a_desktopcommon|a_detecthiddentext|a_detecthiddenwindows|a_endchar|a_eventinfo|a_exitreason|a_fileencoding|a_formatfloat|a_formatinteger|a_gui|a_guicontrol|a_guicontrolevent|a_guievent|a_guiheight|a_guiwidth|a_guix|a_guiy|a_hour|a_iconfile|a_iconhidden|a_iconnumber|a_icontip|a_index|a_ipaddress1|a_ipaddress2|a_ipaddress3|a_ipaddress4|a_is64bitos|a_isadmin|a_iscompiled|a_iscritical|a_ispaused|a_issuspended|a_isunicode|a_keydelay|a_language|a_lasterror|a_linefile|a_linenumber|a_loopfield|a_loopfileattrib|a_loopfiledir|a_loopfileext|a_loopfilefullpath|a_loopfilelongpath|a_loopfilename|a_loopfileshortname|a_loopfileshortpath|a_loopfilesize|a_loopfilesizekb|a_loopfilesizemb|a_loopfiletimeaccessed|a_loopfiletimecreated|a_loopfiletimemodified|a_loopreadline|a_loopregkey|a_loopregname|a_loopregsubkey|a_loopregtimemodified|a_loopregtype|a_mday|a_min|a_mm|a_mmm|a_mmmm|a_mon|a_mousedelay|a_msec|a_mydocuments|a_now|a_nowutc|a_numbatchlines|a_ostype|a_osversion|a_priorhotkey|a_priorkey|a_programfiles|a_programs|a_programscommon|a_ptrsize|a_regview|a_screendpi|a_screenheight|a_screenwidth|a_scriptdir|a_scriptfullpath|a_scripthwnd|a_scriptname|a_sec|a_space|a_startmenu|a_startmenucommon|a_startup|a_startupcommon|a_stringcasesense|a_tab|a_temp|a_thisfunc|a_thishotkey|a_thislabel|a_thismenu|a_thismenuitem|a_thismenuitempos|a_tickcount|a_timeidle|a_timeidlephysical|a_timesincepriorhotkey|a_timesincethishotkey|a_titlematchmode|a_titlematchmodespeed|a_username|a_wday|a_windelay|a_windir|a_workingdir|a_yday|a_year|a_yweek|a_yyyy|clipboard|clipboardall|comspec|errorlevel|programfiles)\b/i,builtin:/\b(?:abs|acos|asc|asin|atan|ceil|chr|class|comobjactive|comobjarray|comobjconnect|comobjcreate|comobjerror|comobjflags|comobjget|comobjquery|comobjtype|comobjvalue|cos|dllcall|exp|fileexist|Fileopen|floor|format|il_add|il_create|il_destroy|instr|isfunc|islabel|IsObject|ln|log|ltrim|lv_add|lv_delete|lv_deletecol|lv_getcount|lv_getnext|lv_gettext|lv_insert|lv_insertcol|lv_modify|lv_modifycol|lv_setimagelist|mod|numget|numput|onmessage|regexmatch|regexreplace|registercallback|round|rtrim|sb_seticon|sb_setparts|sb_settext|sin|sqrt|strlen|strreplace|strsplit|substr|tan|tv_add|tv_delete|tv_get|tv_getchild|tv_getcount|tv_getnext|tv_getparent|tv_getprev|tv_getselection|tv_gettext|tv_modify|varsetcapacity|winactive|winexist|__Call|__Get|__New|__Set)\b/i,symbol:/\b(?:alt|altdown|altup|appskey|backspace|browser_back|browser_favorites|browser_forward|browser_home|browser_refresh|browser_search|browser_stop|bs|capslock|ctrl|ctrlbreak|ctrldown|ctrlup|del|delete|down|end|enter|esc|escape|f1|f10|f11|f12|f13|f14|f15|f16|f17|f18|f19|f2|f20|f21|f22|f23|f24|f3|f4|f5|f6|f7|f8|f9|home|ins|insert|joy1|joy10|joy11|joy12|joy13|joy14|joy15|joy16|joy17|joy18|joy19|joy2|joy20|joy21|joy22|joy23|joy24|joy25|joy26|joy27|joy28|joy29|joy3|joy30|joy31|joy32|joy4|joy5|joy6|joy7|joy8|joy9|joyaxes|joybuttons|joyinfo|joyname|joypov|joyr|joyu|joyv|joyx|joyy|joyz|lalt|launch_app1|launch_app2|launch_mail|launch_media|lbutton|lcontrol|lctrl|left|lshift|lwin|lwindown|lwinup|mbutton|media_next|media_play_pause|media_prev|media_stop|numlock|numpad0|numpad1|numpad2|numpad3|numpad4|numpad5|numpad6|numpad7|numpad8|numpad9|numpadadd|numpadclear|numpaddel|numpaddiv|numpaddot|numpaddown|numpadend|numpadenter|numpadhome|numpadins|numpadleft|numpadmult|numpadpgdn|numpadpgup|numpadright|numpadsub|numpadup|pgdn|pgup|printscreen|ralt|rbutton|rcontrol|rctrl|right|rshift|rwin|rwindown|rwinup|scrolllock|shift|shiftdown|shiftup|space|tab|up|volume_down|volume_mute|volume_up|wheeldown|wheelleft|wheelright|wheelup|xbutton1|xbutton2)\b/i,important:/#\b(?:AllowSameLineComments|ClipboardTimeout|CommentFlag|DerefChar|ErrorStdOut|EscapeChar|HotkeyInterval|HotkeyModifierTimeout|Hotstring|If|IfTimeout|IfWinActive|IfWinExist|IfWinNotActive|IfWinNotExist|Include|IncludeAgain|InputLevel|InstallKeybdHook|InstallMouseHook|KeyHistory|MaxHotkeysPerInterval|MaxMem|MaxThreads|MaxThreadsBuffer|MaxThreadsPerHotkey|MenuMaskKey|NoEnv|NoTrayIcon|Persistent|SingleInstance|UseHook|Warn|WinActivateForce)\b/i,keyword:/\b(?:Abort|AboveNormal|Add|ahk_class|ahk_exe|ahk_group|ahk_id|ahk_pid|All|Alnum|Alpha|AltSubmit|AltTab|AltTabAndMenu|AltTabMenu|AltTabMenuDismiss|AlwaysOnTop|AutoSize|Background|BackgroundTrans|BelowNormal|between|BitAnd|BitNot|BitOr|BitShiftLeft|BitShiftRight|BitXOr|Bold|Border|Button|ByRef|Catch|Checkbox|Checked|CheckedGray|Choose|ChooseString|Close|Color|ComboBox|Contains|ControlList|Count|Date|DateTime|Days|DDL|Default|DeleteAll|Delimiter|Deref|Destroy|Digit|Disable|Disabled|DropDownList|Edit|Eject|Else|Enable|Enabled|Error|Exist|Expand|ExStyle|FileSystem|Finally|First|Flash|Float|FloatFast|Focus|Font|for|global|Grid|Group|GroupBox|GuiClose|GuiContextMenu|GuiDropFiles|GuiEscape|GuiSize|Hdr|Hidden|Hide|High|HKCC|HKCR|HKCU|HKEY_CLASSES_ROOT|HKEY_CURRENT_CONFIG|HKEY_CURRENT_USER|HKEY_LOCAL_MACHINE|HKEY_USERS|HKLM|HKU|Hours|HScroll|Icon|IconSmall|ID|IDLast|If|IfEqual|IfExist|IfGreater|IfGreaterOrEqual|IfInString|IfLess|IfLessOrEqual|IfMsgBox|IfNotEqual|IfNotExist|IfNotInString|IfWinActive|IfWinExist|IfWinNotActive|IfWinNotExist|Ignore|ImageList|in|Integer|IntegerFast|Interrupt|is|italic|Join|Label|LastFound|LastFoundExist|Limit|Lines|List|ListBox|ListView|local|Lock|Logoff|Low|Lower|Lowercase|MainWindow|Margin|Maximize|MaximizeBox|MaxSize|Minimize|MinimizeBox|MinMax|MinSize|Minutes|MonthCal|Mouse|Move|Multi|NA|No|NoActivate|NoDefault|NoHide|NoIcon|NoMainWindow|norm|Normal|NoSort|NoSortHdr|NoStandard|Not|NoTab|NoTimers|Number|Off|Ok|On|OwnDialogs|Owner|Parse|Password|Picture|Pixel|Pos|Pow|Priority|ProcessName|Radio|Range|Read|ReadOnly|Realtime|Redraw|Region|REG_BINARY|REG_DWORD|REG_EXPAND_SZ|REG_MULTI_SZ|REG_SZ|Relative|Rename|Report|Resize|Restore|Retry|RGB|Screen|Seconds|Section|Serial|SetLabel|ShiftAltTab|Show|Single|Slider|SortDesc|Standard|static|Status|StatusBar|StatusCD|strike|Style|Submit|SysMenu|Tab2|TabStop|Text|Theme|Throw|Tile|ToggleCheck|ToggleEnable|ToolWindow|Top|Topmost|TransColor|Transparent|Tray|TreeView|Try|TryAgain|Type|UnCheck|underline|Unicode|Unlock|Until|UpDown|Upper|Uppercase|UseErrorLevel|Vis|VisFirst|Visible|VScroll|Wait|WaitClose|WantCtrlA|WantF2|WantReturn|While|Wrap|Xdigit|xm|xp|xs|Yes|ym|yp|ys)\b/i,function:/[^(); \t,\n+*\-=?>:\\\/<&%\[\]]+(?=\()/,punctuation:/[{}[\]():,]/}}return WC}var VC,E$;function Cnt(){if(E$)return VC;E$=1,VC=e,e.displayName="autoit",e.aliases=[];function e(t){t.languages.autoit={comment:[/;.*/,{pattern:/(^[\t ]*)#(?:comments-start|cs)[\s\S]*?^[ \t]*#(?:ce|comments-end)/m,lookbehind:!0}],url:{pattern:/(^[\t ]*#include\s+)(?:<[^\r\n>]+>|"[^\r\n"]+")/m,lookbehind:!0},string:{pattern:/(["'])(?:\1\1|(?!\1)[^\r\n])*\1/,greedy:!0,inside:{variable:/([%$@])\w+\1/}},directive:{pattern:/(^[\t ]*)#[\w-]+/m,lookbehind:!0,alias:"keyword"},function:/\b\w+(?=\()/,variable:/[$@]\w+/,keyword:/\b(?:Case|Const|Continue(?:Case|Loop)|Default|Dim|Do|Else(?:If)?|End(?:Func|If|Select|Switch|With)|Enum|Exit(?:Loop)?|For|Func|Global|If|In|Local|Next|Null|ReDim|Select|Static|Step|Switch|Then|To|Until|Volatile|WEnd|While|With)\b/i,number:/\b(?:0x[\da-f]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/i,boolean:/\b(?:False|True)\b/i,operator:/<[=>]?|[-+*\/=&>]=?|[?^]|\b(?:And|Not|Or)\b/i,punctuation:/[\[\]().,:]/}}return VC}var KC,x$;function Tnt(){if(x$)return KC;x$=1,KC=e,e.displayName="avisynth",e.aliases=["avs"];function e(t){(function(r){function n(d,p){return d.replace(/<<(\d+)>>/g,function(f,m){return p[+m]})}function a(d,p,f){return RegExp(n(d,p),f)}var i=/bool|clip|float|int|string|val/.source,o=[/is(?:bool|clip|float|int|string)|defined|(?:(?:internal)?function|var)?exists?/.source,/apply|assert|default|eval|import|nop|select|undefined/.source,/opt_(?:allowfloataudio|avipadscanlines|dwchannelmask|enable_(?:b64a|planartopackedrgb|v210|y3_10_10|y3_10_16)|usewaveextensible|vdubplanarhack)|set(?:cachemode|maxcpu|memorymax|planarlegacyalignment|workingdir)/.source,/hex(?:value)?|value/.source,/abs|ceil|continued(?:denominator|numerator)?|exp|floor|fmod|frac|log(?:10)?|max|min|muldiv|pi|pow|rand|round|sign|spline|sqrt/.source,/a?sinh?|a?cosh?|a?tan[2h]?/.source,/(?:bit(?:and|not|x?or|[lr]?shift[aslu]?|sh[lr]|sa[lr]|[lr]rotatel?|ro[rl]|te?st|set(?:count)?|cl(?:ea)?r|ch(?:an)?ge?))/.source,/average(?:[bgr]|chroma[uv]|luma)|(?:[rgb]|chroma[uv]|luma|rgb|[yuv](?=difference(?:fromprevious|tonext)))difference(?:fromprevious|tonext)?|[yuvrgb]plane(?:median|min|max|minmaxdifference)/.source,/getprocessinfo|logmsg|script(?:dir(?:utf8)?|file(?:utf8)?|name(?:utf8)?)|setlogparams/.source,/chr|(?:fill|find|left|mid|replace|rev|right)str|format|[lu]case|ord|str(?:cmpi?|fromutf8|len|toutf8)|time|trim(?:all|left|right)/.source,/isversionorgreater|version(?:number|string)/.source,/buildpixeltype|colorspacenametopixeltype/.source,/addautoloaddir|on(?:cpu|cuda)|prefetch|setfiltermtmode/.source].join("|"),s=[/has(?:audio|video)/.source,/height|width/.source,/frame(?:count|rate)|framerate(?:denominator|numerator)/.source,/getparity|is(?:field|frame)based/.source,/bitspercomponent|componentsize|hasalpha|is(?:planar(?:rgba?)?|interleaved|rgb(?:24|32|48|64)?|y(?:8|u(?:va?|y2))?|yv(?:12|16|24|411)|420|422|444|packedrgb)|numcomponents|pixeltype/.source,/audio(?:bits|channels|duration|length(?:[fs]|hi|lo)?|rate)|isaudio(?:float|int)/.source].join("|"),l=[/avi(?:file)?source|directshowsource|image(?:reader|source|sourceanim)|opendmlsource|segmented(?:avisource|directshowsource)|wavsource/.source,/coloryuv|convertbacktoyuy2|convertto(?:RGB(?:24|32|48|64)|(?:planar)?RGBA?|Y8?|YV(?:12|16|24|411)|YUVA?(?:411|420|422|444)|YUY2)|fixluminance|gr[ae]yscale|invert|levels|limiter|mergea?rgb|merge(?:chroma|luma)|rgbadjust|show(?:alpha|blue|green|red)|swapuv|tweak|[uv]toy8?|ytouv/.source,/(?:colorkey|reset)mask|layer|mask(?:hs)?|merge|overlay|subtract/.source,/addborders|(?:bicubic|bilinear|blackman|gauss|lanczos4|lanczos|point|sinc|spline(?:16|36|64))resize|crop(?:bottom)?|flip(?:horizontal|vertical)|(?:horizontal|vertical)?reduceby2|letterbox|skewrows|turn(?:180|left|right)/.source,/blur|fixbrokenchromaupsampling|generalconvolution|(?:spatial|temporal)soften|sharpen/.source,/trim|(?:un)?alignedsplice|(?:assume|assumescaled|change|convert)FPS|(?:delete|duplicate)frame|dissolve|fade(?:in|io|out)[02]?|freezeframe|interleave|loop|reverse|select(?:even|odd|(?:range)?every)/.source,/assume[bt]ff|assume(?:field|frame)based|bob|complementparity|doubleweave|peculiarblend|pulldown|separate(?:columns|fields|rows)|swapfields|weave(?:columns|rows)?/.source,/amplify(?:db)?|assumesamplerate|audiodub(?:ex)?|audiotrim|convertaudioto(?:(?:8|16|24|32)bit|float)|converttomono|delayaudio|ensurevbrmp3sync|get(?:left|right)?channel|kill(?:audio|video)|mergechannels|mixaudio|monotostereo|normalize|resampleaudio|ssrc|supereq|timestretch/.source,/animate|applyrange|conditional(?:filter|reader|select)|frameevaluate|scriptclip|tcp(?:server|source)|writefile(?:end|if|start)?/.source,/imagewriter/.source,/blackness|blankclip|colorbars(?:hd)?|compare|dumpfiltergraph|echo|histogram|info|messageclip|preroll|setgraphanalysis|show(?:framenumber|smpte|time)|showfiveversions|stack(?:horizontal|vertical)|subtitle|tone|version/.source].join("|"),u=[o,s,l].join("|");r.languages.avisynth={comment:[{pattern:/(^|[^\\])\[\*(?:[^\[*]|\[(?!\*)|\*(?!\])|\[\*(?:[^\[*]|\[(?!\*)|\*(?!\]))*\*\])*\*\]/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\$])#.*/,lookbehind:!0,greedy:!0}],argument:{pattern:a(/\b(?:<<0>>)\s+("?)\w+\1/.source,[i],"i"),inside:{keyword:/^\w+/}},"argument-label":{pattern:/([,(][\s\\]*)\w+\s*=(?!=)/,lookbehind:!0,inside:{"argument-name":{pattern:/^\w+/,alias:"punctuation"},punctuation:/=$/}},string:[{pattern:/"""[\s\S]*?"""/,greedy:!0},{pattern:/"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,greedy:!0,inside:{constant:{pattern:/\b(?:DEFAULT_MT_MODE|(?:MAINSCRIPT|PROGRAM|SCRIPT)DIR|(?:MACHINE|USER)_(?:CLASSIC|PLUS)_PLUGINS)\b/}}}],variable:/\b(?:last)\b/i,boolean:/\b(?:false|no|true|yes)\b/i,keyword:/\b(?:catch|else|for|function|global|if|return|try|while|__END__)\b/i,constant:/\bMT_(?:MULTI_INSTANCE|NICE_FILTER|SERIALIZED|SPECIAL_MT)\b/,"builtin-function":{pattern:a(/\b(?:<<0>>)\b/.source,[u],"i"),alias:"function"},"type-cast":{pattern:a(/\b(?:<<0>>)(?=\s*\()/.source,[i],"i"),alias:"keyword"},function:{pattern:/\b[a-z_]\w*(?=\s*\()|(\.)[a-z_]\w*\b/i,lookbehind:!0},"line-continuation":{pattern:/(^[ \t]*)\\|\\(?=[ \t]*$)/m,lookbehind:!0,alias:"punctuation"},number:/\B\$(?:[\da-f]{6}|[\da-f]{8})\b|(?:(?:\b|\B-)\d+(?:\.\d*)?\b|\B\.\d+\b)/i,operator:/\+\+?|[!=<>]=?|&&|\|\||[?:*/%-]/,punctuation:/[{}\[\]();,.]/},r.languages.avs=r.languages.avisynth})(t)}return KC}var YC,w$;function Ant(){if(w$)return YC;w$=1,YC=e,e.displayName="avroIdl",e.aliases=[];function e(t){t.languages["avro-idl"]={comment:{pattern:/\/\/.*|\/\*[\s\S]*?\*\//,greedy:!0},string:{pattern:/(^|[^\\])"(?:[^\r\n"\\]|\\.)*"/,lookbehind:!0,greedy:!0},annotation:{pattern:/@(?:[$\w.-]|`[^\r\n`]+`)+/,greedy:!0,alias:"function"},"function-identifier":{pattern:/`[^\r\n`]+`(?=\s*\()/,greedy:!0,alias:"function"},identifier:{pattern:/`[^\r\n`]+`/,greedy:!0},"class-name":{pattern:/(\b(?:enum|error|protocol|record|throws)\b\s+)[$\w]+/,lookbehind:!0,greedy:!0},keyword:/\b(?:array|boolean|bytes|date|decimal|double|enum|error|false|fixed|float|idl|import|int|local_timestamp_ms|long|map|null|oneway|protocol|record|schema|string|throws|time_ms|timestamp_ms|true|union|uuid|void)\b/,function:/\b[a-z_]\w*(?=\s*\()/i,number:[{pattern:/(^|[^\w.])-?(?:(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?|0x(?:[a-f0-9]+(?:\.[a-f0-9]*)?|\.[a-f0-9]+)(?:p[+-]?\d+)?)[dfl]?(?![\w.])/i,lookbehind:!0},/-?\b(?:Infinity|NaN)\b/],operator:/=/,punctuation:/[()\[\]{}<>.:,;-]/},t.languages.avdl=t.languages["avro-idl"]}return YC}var JC,k$;function Mce(){if(k$)return JC;k$=1,JC=e,e.displayName="bash",e.aliases=["shell"];function e(t){(function(r){var n="\\b(?:BASH|BASHOPTS|BASH_ALIASES|BASH_ARGC|BASH_ARGV|BASH_CMDS|BASH_COMPLETION_COMPAT_DIR|BASH_LINENO|BASH_REMATCH|BASH_SOURCE|BASH_VERSINFO|BASH_VERSION|COLORTERM|COLUMNS|COMP_WORDBREAKS|DBUS_SESSION_BUS_ADDRESS|DEFAULTS_PATH|DESKTOP_SESSION|DIRSTACK|DISPLAY|EUID|GDMSESSION|GDM_LANG|GNOME_KEYRING_CONTROL|GNOME_KEYRING_PID|GPG_AGENT_INFO|GROUPS|HISTCONTROL|HISTFILE|HISTFILESIZE|HISTSIZE|HOME|HOSTNAME|HOSTTYPE|IFS|INSTANCE|JOB|LANG|LANGUAGE|LC_ADDRESS|LC_ALL|LC_IDENTIFICATION|LC_MEASUREMENT|LC_MONETARY|LC_NAME|LC_NUMERIC|LC_PAPER|LC_TELEPHONE|LC_TIME|LESSCLOSE|LESSOPEN|LINES|LOGNAME|LS_COLORS|MACHTYPE|MAILCHECK|MANDATORY_PATH|NO_AT_BRIDGE|OLDPWD|OPTERR|OPTIND|ORBIT_SOCKETDIR|OSTYPE|PAPERSIZE|PATH|PIPESTATUS|PPID|PS1|PS2|PS3|PS4|PWD|RANDOM|REPLY|SECONDS|SELINUX_INIT|SESSION|SESSIONTYPE|SESSION_MANAGER|SHELL|SHELLOPTS|SHLVL|SSH_AUTH_SOCK|TERM|UID|UPSTART_EVENTS|UPSTART_INSTANCE|UPSTART_JOB|UPSTART_SESSION|USER|WINDOWID|XAUTHORITY|XDG_CONFIG_DIRS|XDG_CURRENT_DESKTOP|XDG_DATA_DIRS|XDG_GREETER_DATA_DIR|XDG_MENU_PREFIX|XDG_RUNTIME_DIR|XDG_SEAT|XDG_SEAT_PATH|XDG_SESSION_DESKTOP|XDG_SESSION_ID|XDG_SESSION_PATH|XDG_SESSION_TYPE|XDG_VTNR|XMODIFIERS)\\b",a={pattern:/(^(["']?)\w+\2)[ \t]+\S.*/,lookbehind:!0,alias:"punctuation",inside:null},i={bash:a,environment:{pattern:RegExp("\\$"+n),alias:"constant"},variable:[{pattern:/\$?\(\([\s\S]+?\)\)/,greedy:!0,inside:{variable:[{pattern:/(^\$\(\([\s\S]+)\)\)/,lookbehind:!0},/^\$\(\(/],number:/\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee]-?\d+)?/,operator:/--|\+\+|\*\*=?|<<=?|>>=?|&&|\|\||[=!+\-*/%<>^&|]=?|[?~:]/,punctuation:/\(\(?|\)\)?|,|;/}},{pattern:/\$\((?:\([^)]+\)|[^()])+\)|`[^`]+`/,greedy:!0,inside:{variable:/^\$\(|^`|\)$|`$/}},{pattern:/\$\{[^}]+\}/,greedy:!0,inside:{operator:/:[-=?+]?|[!\/]|##?|%%?|\^\^?|,,?/,punctuation:/[\[\]]/,environment:{pattern:RegExp("(\\{)"+n),lookbehind:!0,alias:"constant"}}},/\$(?:\w+|[#?*!@$])/],entity:/\\(?:[abceEfnrtv\\"]|O?[0-7]{1,3}|U[0-9a-fA-F]{8}|u[0-9a-fA-F]{4}|x[0-9a-fA-F]{1,2})/};r.languages.bash={shebang:{pattern:/^#!\s*\/.*/,alias:"important"},comment:{pattern:/(^|[^"{\\$])#.*/,lookbehind:!0},"function-name":[{pattern:/(\bfunction\s+)[\w-]+(?=(?:\s*\(?:\s*\))?\s*\{)/,lookbehind:!0,alias:"function"},{pattern:/\b[\w-]+(?=\s*\(\s*\)\s*\{)/,alias:"function"}],"for-or-select":{pattern:/(\b(?:for|select)\s+)\w+(?=\s+in\s)/,alias:"variable",lookbehind:!0},"assign-left":{pattern:/(^|[\s;|&]|[<>]\()\w+(?=\+?=)/,inside:{environment:{pattern:RegExp("(^|[\\s;|&]|[<>]\\()"+n),lookbehind:!0,alias:"constant"}},alias:"variable",lookbehind:!0},string:[{pattern:/((?:^|[^<])<<-?\s*)(\w+)\s[\s\S]*?(?:\r?\n|\r)\2/,lookbehind:!0,greedy:!0,inside:i},{pattern:/((?:^|[^<])<<-?\s*)(["'])(\w+)\2\s[\s\S]*?(?:\r?\n|\r)\3/,lookbehind:!0,greedy:!0,inside:{bash:a}},{pattern:/(^|[^\\](?:\\\\)*)"(?:\\[\s\S]|\$\([^)]+\)|\$(?!\()|`[^`]+`|[^"\\`$])*"/,lookbehind:!0,greedy:!0,inside:i},{pattern:/(^|[^$\\])'[^']*'/,lookbehind:!0,greedy:!0},{pattern:/\$'(?:[^'\\]|\\[\s\S])*'/,greedy:!0,inside:{entity:i.entity}}],environment:{pattern:RegExp("\\$?"+n),alias:"constant"},variable:i.variable,function:{pattern:/(^|[\s;|&]|[<>]\()(?:add|apropos|apt|apt-cache|apt-get|aptitude|aspell|automysqlbackup|awk|basename|bash|bc|bconsole|bg|bzip2|cal|cat|cfdisk|chgrp|chkconfig|chmod|chown|chroot|cksum|clear|cmp|column|comm|composer|cp|cron|crontab|csplit|curl|cut|date|dc|dd|ddrescue|debootstrap|df|diff|diff3|dig|dir|dircolors|dirname|dirs|dmesg|docker|docker-compose|du|egrep|eject|env|ethtool|expand|expect|expr|fdformat|fdisk|fg|fgrep|file|find|fmt|fold|format|free|fsck|ftp|fuser|gawk|git|gparted|grep|groupadd|groupdel|groupmod|groups|grub-mkconfig|gzip|halt|head|hg|history|host|hostname|htop|iconv|id|ifconfig|ifdown|ifup|import|install|ip|jobs|join|kill|killall|less|link|ln|locate|logname|logrotate|look|lpc|lpr|lprint|lprintd|lprintq|lprm|ls|lsof|lynx|make|man|mc|mdadm|mkconfig|mkdir|mke2fs|mkfifo|mkfs|mkisofs|mknod|mkswap|mmv|more|most|mount|mtools|mtr|mutt|mv|nano|nc|netstat|nice|nl|node|nohup|notify-send|npm|nslookup|op|open|parted|passwd|paste|pathchk|ping|pkill|pnpm|podman|podman-compose|popd|pr|printcap|printenv|ps|pushd|pv|quota|quotacheck|quotactl|ram|rar|rcp|reboot|remsync|rename|renice|rev|rm|rmdir|rpm|rsync|scp|screen|sdiff|sed|sendmail|seq|service|sftp|sh|shellcheck|shuf|shutdown|sleep|slocate|sort|split|ssh|stat|strace|su|sudo|sum|suspend|swapon|sync|tac|tail|tar|tee|time|timeout|top|touch|tr|traceroute|tsort|tty|umount|uname|unexpand|uniq|units|unrar|unshar|unzip|update-grub|uptime|useradd|userdel|usermod|users|uudecode|uuencode|v|vcpkg|vdir|vi|vim|virsh|vmstat|wait|watch|wc|wget|whereis|which|who|whoami|write|xargs|xdg-open|yarn|yes|zenity|zip|zsh|zypper)(?=$|[)\s;|&])/,lookbehind:!0},keyword:{pattern:/(^|[\s;|&]|[<>]\()(?:case|do|done|elif|else|esac|fi|for|function|if|in|select|then|until|while)(?=$|[)\s;|&])/,lookbehind:!0},builtin:{pattern:/(^|[\s;|&]|[<>]\()(?:\.|:|alias|bind|break|builtin|caller|cd|command|continue|declare|echo|enable|eval|exec|exit|export|getopts|hash|help|let|local|logout|mapfile|printf|pwd|read|readarray|readonly|return|set|shift|shopt|source|test|times|trap|type|typeset|ulimit|umask|unalias|unset)(?=$|[)\s;|&])/,lookbehind:!0,alias:"class-name"},boolean:{pattern:/(^|[\s;|&]|[<>]\()(?:false|true)(?=$|[)\s;|&])/,lookbehind:!0},"file-descriptor":{pattern:/\B&\d\b/,alias:"important"},operator:{pattern:/\d?<>|>\||\+=|=[=~]?|!=?|<<[<-]?|[&\d]?>>|\d[<>]&?|[<>][&=]?|&[>&]?|\|[&|]?/,inside:{"file-descriptor":{pattern:/^\d/,alias:"important"}}},punctuation:/\$?\(\(?|\)\)?|\.\.|[{}[\];\\]/,number:{pattern:/(^|\s)(?:[1-9]\d*|0)(?:[.,]\d+)?\b/,lookbehind:!0}},a.inside=r.languages.bash;for(var o=["comment","function-name","for-or-select","assign-left","string","environment","function","keyword","builtin","boolean","file-descriptor","operator","punctuation","number"],s=i.variable[1].inside,l=0;l<o.length;l++)s[o[l]]=r.languages.bash[o[l]];r.languages.shell=r.languages.bash})(t)}return JC}var XC,C$;function Ice(){if(C$)return XC;C$=1,XC=e,e.displayName="basic",e.aliases=[];function e(t){t.languages.basic={comment:{pattern:/(?:!|REM\b).+/i,inside:{keyword:/^REM/i}},string:{pattern:/"(?:""|[!#$%&'()*,\/:;<=>?^\w +\-.])*"/,greedy:!0},number:/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i,keyword:/\b(?:AS|BEEP|BLOAD|BSAVE|CALL(?: ABSOLUTE)?|CASE|CHAIN|CHDIR|CLEAR|CLOSE|CLS|COM|COMMON|CONST|DATA|DECLARE|DEF(?: FN| SEG|DBL|INT|LNG|SNG|STR)|DIM|DO|DOUBLE|ELSE|ELSEIF|END|ENVIRON|ERASE|ERROR|EXIT|FIELD|FILES|FOR|FUNCTION|GET|GOSUB|GOTO|IF|INPUT|INTEGER|IOCTL|KEY|KILL|LINE INPUT|LOCATE|LOCK|LONG|LOOP|LSET|MKDIR|NAME|NEXT|OFF|ON(?: COM| ERROR| KEY| TIMER)?|OPEN|OPTION BASE|OUT|POKE|PUT|READ|REDIM|REM|RESTORE|RESUME|RETURN|RMDIR|RSET|RUN|SELECT CASE|SHARED|SHELL|SINGLE|SLEEP|STATIC|STEP|STOP|STRING|SUB|SWAP|SYSTEM|THEN|TIMER|TO|TROFF|TRON|TYPE|UNLOCK|UNTIL|USING|VIEW PRINT|WAIT|WEND|WHILE|WRITE)(?:\$|\b)/i,function:/\b(?:ABS|ACCESS|ACOS|ANGLE|AREA|ARITHMETIC|ARRAY|ASIN|ASK|AT|ATN|BASE|BEGIN|BREAK|CAUSE|CEIL|CHR|CLIP|COLLATE|COLOR|CON|COS|COSH|COT|CSC|DATE|DATUM|DEBUG|DECIMAL|DEF|DEG|DEGREES|DELETE|DET|DEVICE|DISPLAY|DOT|ELAPSED|EPS|ERASABLE|EXLINE|EXP|EXTERNAL|EXTYPE|FILETYPE|FIXED|FP|GO|GRAPH|HANDLER|IDN|IMAGE|IN|INT|INTERNAL|IP|IS|KEYED|LBOUND|LCASE|LEFT|LEN|LENGTH|LET|LINE|LINES|LOG|LOG10|LOG2|LTRIM|MARGIN|MAT|MAX|MAXNUM|MID|MIN|MISSING|MOD|NATIVE|NUL|NUMERIC|OF|OPTION|ORD|ORGANIZATION|OUTIN|OUTPUT|PI|POINT|POINTER|POINTS|POS|PRINT|PROGRAM|PROMPT|RAD|RADIANS|RANDOMIZE|RECORD|RECSIZE|RECTYPE|RELATIVE|REMAINDER|REPEAT|REST|RETRY|REWRITE|RIGHT|RND|ROUND|RTRIM|SAME|SEC|SELECT|SEQUENTIAL|SET|SETTER|SGN|SIN|SINH|SIZE|SKIP|SQR|STANDARD|STATUS|STR|STREAM|STYLE|TAB|TAN|TANH|TEMPLATE|TEXT|THERE|TIME|TIMEOUT|TRACE|TRANSFORM|TRUNCATE|UBOUND|UCASE|USE|VAL|VARIABLE|VIEWPORT|WHEN|WINDOW|WITH|ZER|ZONEWIDTH)(?:\$|\b)/i,operator:/<[=>]?|>=?|[+\-*\/^=&]|\b(?:AND|EQV|IMP|NOT|OR|XOR)\b/i,punctuation:/[,;:()]/}}return XC}var ZC,T$;function Fnt(){if(T$)return ZC;T$=1,ZC=e,e.displayName="batch",e.aliases=[];function e(t){(function(r){var n=/%%?[~:\w]+%?|!\S+!/,a={pattern:/\/[a-z?]+(?=[ :]|$):?|-[a-z]\b|--[a-z-]+\b/im,alias:"attr-name",inside:{punctuation:/:/}},i=/"(?:[\\"]"|[^"])*"(?!")/,o=/(?:\b|-)\d+\b/;r.languages.batch={comment:[/^::.*/m,{pattern:/((?:^|[&(])[ \t]*)rem\b(?:[^^&)\r\n]|\^(?:\r\n|[\s\S]))*/im,lookbehind:!0}],label:{pattern:/^:.*/m,alias:"property"},command:[{pattern:/((?:^|[&(])[ \t]*)for(?: \/[a-z?](?:[ :](?:"[^"]*"|[^\s"/]\S*))?)* \S+ in \([^)]+\) do/im,lookbehind:!0,inside:{keyword:/\b(?:do|in)\b|^for\b/i,string:i,parameter:a,variable:n,number:o,punctuation:/[()',]/}},{pattern:/((?:^|[&(])[ \t]*)if(?: \/[a-z?](?:[ :](?:"[^"]*"|[^\s"/]\S*))?)* (?:not )?(?:cmdextversion \d+|defined \w+|errorlevel \d+|exist \S+|(?:"[^"]*"|(?!")(?:(?!==)\S)+)?(?:==| (?:equ|geq|gtr|leq|lss|neq) )(?:"[^"]*"|[^\s"]\S*))/im,lookbehind:!0,inside:{keyword:/\b(?:cmdextversion|defined|errorlevel|exist|not)\b|^if\b/i,string:i,parameter:a,variable:n,number:o,operator:/\^|==|\b(?:equ|geq|gtr|leq|lss|neq)\b/i}},{pattern:/((?:^|[&()])[ \t]*)else\b/im,lookbehind:!0,inside:{keyword:/^else\b/i}},{pattern:/((?:^|[&(])[ \t]*)set(?: \/[a-z](?:[ :](?:"[^"]*"|[^\s"/]\S*))?)* (?:[^^&)\r\n]|\^(?:\r\n|[\s\S]))*/im,lookbehind:!0,inside:{keyword:/^set\b/i,string:i,parameter:a,variable:[n,/\w+(?=(?:[*\/%+\-&^|]|<<|>>)?=)/],number:o,operator:/[*\/%+\-&^|]=?|<<=?|>>=?|[!~_=]/,punctuation:/[()',]/}},{pattern:/((?:^|[&(])[ \t]*@?)\w+\b(?:"(?:[\\"]"|[^"])*"(?!")|[^"^&)\r\n]|\^(?:\r\n|[\s\S]))*/m,lookbehind:!0,inside:{keyword:/^\w+\b/,string:i,parameter:a,label:{pattern:/(^\s*):\S+/m,lookbehind:!0,alias:"property"},variable:n,number:o,operator:/\^/}}],operator:/[&@]/,punctuation:/[()']/}})(t)}return ZC}var QC,A$;function _nt(){if(A$)return QC;A$=1,QC=e,e.displayName="bbcode",e.aliases=["shortcode"];function e(t){t.languages.bbcode={tag:{pattern:/\[\/?[^\s=\]]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+))?(?:\s+[^\s=\]]+\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+))*\s*\]/,inside:{tag:{pattern:/^\[\/?[^\s=\]]+/,inside:{punctuation:/^\[\/?/}},"attr-value":{pattern:/=\s*(?:"[^"]*"|'[^']*'|[^\s'"\]=]+)/,inside:{punctuation:[/^=/,{pattern:/^(\s*)["']|["']$/,lookbehind:!0}]}},punctuation:/\]/,"attr-name":/[^\s=\]]+/}}},t.languages.shortcode=t.languages.bbcode}return QC}var eT,F$;function Rnt(){if(F$)return eT;F$=1,eT=e,e.displayName="bicep",e.aliases=[];function e(t){t.languages.bicep={comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],property:[{pattern:/([\r\n][ \t]*)[a-z_]\w*(?=[ \t]*:)/i,lookbehind:!0},{pattern:/([\r\n][ \t]*)'(?:\\.|\$(?!\{)|[^'\\\r\n$])*'(?=[ \t]*:)/,lookbehind:!0,greedy:!0}],string:[{pattern:/'''[^'][\s\S]*?'''/,greedy:!0},{pattern:/(^|[^\\'])'(?:\\.|\$(?!\{)|[^'\\\r\n$])*'/,lookbehind:!0,greedy:!0}],"interpolated-string":{pattern:/(^|[^\\'])'(?:\\.|\$(?:(?!\{)|\{[^{}\r\n]*\})|[^'\\\r\n$])*'/,lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/\$\{[^{}\r\n]*\}/,inside:{expression:{pattern:/(^\$\{)[\s\S]+(?=\}$)/,lookbehind:!0},punctuation:/^\$\{|\}$/}},string:/[\s\S]+/}},datatype:{pattern:/(\b(?:output|param)\b[ \t]+\w+[ \t]+)\w+\b/,lookbehind:!0,alias:"class-name"},boolean:/\b(?:false|true)\b/,keyword:/\b(?:existing|for|if|in|module|null|output|param|resource|targetScope|var)\b/,decorator:/@\w+\b/,function:/\b[a-z_]\w*(?=[ \t]*\()/i,number:/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i,operator:/--|\+\+|\*\*=?|=>|&&=?|\|\|=?|[!=]==|<<=?|>>>?=?|[-+*/%&|^!=<>]=?|\.{3}|\?\?=?|\?\.?|[~:]/,punctuation:/[{}[\];(),.:]/},t.languages.bicep["interpolated-string"].inside.interpolation.inside.expression.inside=t.languages.bicep}return eT}var tT,_$;function Dnt(){if(_$)return tT;_$=1,tT=e,e.displayName="birb",e.aliases=[];function e(t){t.languages.birb=t.languages.extend("clike",{string:{pattern:/r?("|')(?:\\.|(?!\1)[^\\])*\1/,greedy:!0},"class-name":[/\b[A-Z](?:[\d_]*[a-zA-Z]\w*)?\b/,/\b(?:[A-Z]\w*|(?!(?:var|void)\b)[a-z]\w*)(?=\s+\w+\s*[;,=()])/],keyword:/\b(?:assert|break|case|class|const|default|else|enum|final|follows|for|grab|if|nest|new|next|noSeeb|return|static|switch|throw|var|void|while)\b/,operator:/\+\+|--|&&|\|\||<<=?|>>=?|~(?:\/=?)?|[+\-*\/%&^|=!<>]=?|\?|:/,variable:/\b[a-z_]\w*\b/}),t.languages.insertBefore("birb","function",{metadata:{pattern:/<\w+>/,greedy:!0,alias:"symbol"}})}return tT}var rT,R$;function Nnt(){if(R$)return rT;R$=1;var e=pd();rT=t,t.displayName="bison",t.aliases=[];function t(r){r.register(e),r.languages.bison=r.languages.extend("c",{}),r.languages.insertBefore("bison","comment",{bison:{pattern:/^(?:[^%]|%(?!%))*%%[\s\S]*?%%/,inside:{c:{pattern:/%\{[\s\S]*?%\}|\{(?:\{[^}]*\}|[^{}])*\}/,inside:{delimiter:{pattern:/^%?\{|%?\}$/,alias:"punctuation"},"bison-variable":{pattern:/[$@](?:<[^\s>]+>)?[\w$]+/,alias:"variable",inside:{punctuation:/<|>/}},rest:r.languages.c}},comment:r.languages.c.comment,string:r.languages.c.string,property:/\S+(?=:)/,keyword:/%\w+/,number:{pattern:/(^|[^@])\b(?:0x[\da-f]+|\d+)/i,lookbehind:!0},punctuation:/%[%?]|[|:;\[\]<>]/}}})}return rT}var nT,D$;function Lnt(){if(D$)return nT;D$=1,nT=e,e.displayName="bnf",e.aliases=["rbnf"];function e(t){t.languages.bnf={string:{pattern:/"[^\r\n"]*"|'[^\r\n']*'/},definition:{pattern:/<[^<>\r\n\t]+>(?=\s*::=)/,alias:["rule","keyword"],inside:{punctuation:/^<|>$/}},rule:{pattern:/<[^<>\r\n\t]+>/,inside:{punctuation:/^<|>$/}},operator:/::=|[|()[\]{}*+?]|\.{3}/},t.languages.rbnf=t.languages.bnf}return nT}var aT,N$;function Mnt(){if(N$)return aT;N$=1,aT=e,e.displayName="brainfuck",e.aliases=[];function e(t){t.languages.brainfuck={pointer:{pattern:/<|>/,alias:"keyword"},increment:{pattern:/\+/,alias:"inserted"},decrement:{pattern:/-/,alias:"deleted"},branching:{pattern:/\[|\]/,alias:"important"},operator:/[.,]/,comment:/\S+/}}return aT}var iT,L$;function Int(){if(L$)return iT;L$=1,iT=e,e.displayName="brightscript",e.aliases=[];function e(t){t.languages.brightscript={comment:/(?:\brem|').*/i,"directive-statement":{pattern:/(^[\t ]*)#(?:const|else(?:[\t ]+if)?|end[\t ]+if|error|if).*/im,lookbehind:!0,alias:"property",inside:{"error-message":{pattern:/(^#error).+/,lookbehind:!0},directive:{pattern:/^#(?:const|else(?:[\t ]+if)?|end[\t ]+if|error|if)/,alias:"keyword"},expression:{pattern:/[\s\S]+/,inside:null}}},property:{pattern:/([\r\n{,][\t ]*)(?:(?!\d)\w+|"(?:[^"\r\n]|"")*"(?!"))(?=[ \t]*:)/,lookbehind:!0,greedy:!0},string:{pattern:/"(?:[^"\r\n]|"")*"(?!")/,greedy:!0},"class-name":{pattern:/(\bAs[\t ]+)\w+/i,lookbehind:!0},keyword:/\b(?:As|Dim|Each|Else|Elseif|End|Exit|For|Function|Goto|If|In|Print|Return|Step|Stop|Sub|Then|To|While)\b/i,boolean:/\b(?:false|true)\b/i,function:/\b(?!\d)\w+(?=[\t ]*\()/,number:/(?:\b\d+(?:\.\d+)?(?:[ed][+-]\d+)?|&h[a-f\d]+)\b[%&!#]?/i,operator:/--|\+\+|>>=?|<<=?|<>|[-+*/\\<>]=?|[:^=?]|\b(?:and|mod|not|or)\b/i,punctuation:/[.,;()[\]{}]/,constant:/\b(?:LINE_NUM)\b/i},t.languages.brightscript["directive-statement"].inside.expression.inside=t.languages.brightscript}return iT}var oT,M$;function Ont(){if(M$)return oT;M$=1,oT=e,e.displayName="bro",e.aliases=[];function e(t){t.languages.bro={comment:{pattern:/(^|[^\\$])#.*/,lookbehind:!0,inside:{italic:/\b(?:FIXME|TODO|XXX)\b/}},string:{pattern:/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0},boolean:/\b[TF]\b/,function:{pattern:/(\b(?:event|function|hook)[ \t]+)\w+(?:::\w+)?/,lookbehind:!0},builtin:/(?:@(?:load(?:-(?:plugin|sigs))?|unload|prefixes|ifn?def|else|(?:end)?if|DIR|FILENAME))|(?:&?(?:add_func|create_expire|default|delete_func|encrypt|error_handler|expire_func|group|log|mergeable|optional|persistent|priority|raw_output|read_expire|redef|rotate_interval|rotate_size|synchronized|type_column|write_expire))/,constant:{pattern:/(\bconst[ \t]+)\w+/i,lookbehind:!0},keyword:/\b(?:add|addr|alarm|any|bool|break|const|continue|count|delete|double|else|enum|event|export|file|for|function|global|hook|if|in|int|interval|local|module|next|of|opaque|pattern|port|print|record|return|schedule|set|string|subnet|table|time|timeout|using|vector|when)\b/,operator:/--?|\+\+?|!=?=?|<=?|>=?|==?=?|&&|\|\|?|\?|\*|\/|~|\^|%/,number:/\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,punctuation:/[{}[\];(),.:]/}}return oT}var sT,I$;function Bnt(){if(I$)return sT;I$=1,sT=e,e.displayName="bsl",e.aliases=[];function e(t){t.languages.bsl={comment:/\/\/.*/,string:[{pattern:/"(?:[^"]|"")*"(?!")/,greedy:!0},{pattern:/'(?:[^'\r\n\\]|\\.)*'/}],keyword:[{pattern:/(^|[^\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])(?:пока|для|новый|прервать|попытка|исключение|вызватьисключение|иначе|конецпопытки|неопределено|функция|перем|возврат|конецфункции|если|иначеесли|процедура|конецпроцедуры|тогда|знач|экспорт|конецесли|из|каждого|истина|ложь|по|цикл|конеццикла|выполнить)(?![\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])/i,lookbehind:!0},{pattern:/\b(?:break|do|each|else|elseif|enddo|endfunction|endif|endprocedure|endtry|except|execute|export|false|for|function|if|in|new|null|procedure|raise|return|then|to|true|try|undefined|val|var|while)\b/i}],number:{pattern:/(^(?=\d)|[^\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])(?:\d+(?:\.\d*)?|\.\d+)(?:E[+-]?\d+)?/i,lookbehind:!0},operator:[/[<>+\-*/]=?|[%=]/,{pattern:/(^|[^\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])(?:и|или|не)(?![\w\u0400-\u0484\u0487-\u052f\u1d2b\u1d78\u2de0-\u2dff\ua640-\ua69f\ufe2e\ufe2f])/i,lookbehind:!0},{pattern:/\b(?:and|not|or)\b/i}],punctuation:/\(\.|\.\)|[()\[\]:;,.]/,directive:[{pattern:/^([ \t]*)&.*/m,lookbehind:!0,greedy:!0,alias:"important"},{pattern:/^([ \t]*)#.*/gm,lookbehind:!0,greedy:!0,alias:"important"}]},t.languages.oscript=t.languages.bsl}return sT}var lT,O$;function Pnt(){if(O$)return lT;O$=1,lT=e,e.displayName="cfscript",e.aliases=[];function e(t){t.languages.cfscript=t.languages.extend("clike",{comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,lookbehind:!0,inside:{annotation:{pattern:/(?:^|[^.])@[\w\.]+/,alias:"punctuation"}}},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],keyword:/\b(?:abstract|break|catch|component|continue|default|do|else|extends|final|finally|for|function|if|in|include|package|private|property|public|remote|required|rethrow|return|static|switch|throw|try|var|while|xml)\b(?!\s*=)/,operator:[/\+\+|--|&&|\|\||::|=>|[!=]==|<=?|>=?|[-+*/%&|^!=<>]=?|\?(?:\.|:)?|[?:]/,/\b(?:and|contains|eq|equal|eqv|gt|gte|imp|is|lt|lte|mod|not|or|xor)\b/],scope:{pattern:/\b(?:application|arguments|cgi|client|cookie|local|session|super|this|variables)\b/,alias:"global"},type:{pattern:/\b(?:any|array|binary|boolean|date|guid|numeric|query|string|struct|uuid|void|xml)\b/,alias:"builtin"}}),t.languages.insertBefore("cfscript","keyword",{"function-variable":{pattern:/[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*[=:]\s*(?:\bfunction\b|(?:\((?:[^()]|\([^()]*\))*\)|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/,alias:"function"}}),delete t.languages.cfscript["class-name"],t.languages.cfc=t.languages.cfscript}return lT}var uT,B$;function znt(){if(B$)return uT;B$=1;var e=pL();uT=t,t.displayName="chaiscript",t.aliases=[];function t(r){r.register(e),r.languages.chaiscript=r.languages.extend("clike",{string:{pattern:/(^|[^\\])'(?:[^'\\]|\\[\s\S])*'/,lookbehind:!0,greedy:!0},"class-name":[{pattern:/(\bclass\s+)\w+/,lookbehind:!0},{pattern:/(\b(?:attr|def)\s+)\w+(?=\s*::)/,lookbehind:!0}],keyword:/\b(?:attr|auto|break|case|catch|class|continue|def|default|else|finally|for|fun|global|if|return|switch|this|try|var|while)\b/,number:[r.languages.cpp.number,/\b(?:Infinity|NaN)\b/],operator:/>>=?|<<=?|\|\||&&|:[:=]?|--|\+\+|[=!<>+\-*/%|&^]=?|[?~]|`[^`\r\n]{1,4}`/}),r.languages.insertBefore("chaiscript","operator",{"parameter-type":{pattern:/([,(]\s*)\w+(?=\s+\w)/,lookbehind:!0,alias:"class-name"}}),r.languages.insertBefore("chaiscript","string",{"string-interpolation":{pattern:/(^|[^\\])"(?:[^"$\\]|\\[\s\S]|\$(?!\{)|\$\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\})*"/,lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\}/,lookbehind:!0,inside:{"interpolation-expression":{pattern:/(^\$\{)[\s\S]+(?=\}$)/,lookbehind:!0,inside:r.languages.chaiscript},"interpolation-punctuation":{pattern:/^\$\{|\}$/,alias:"punctuation"}}},string:/[\s\S]+/}}})}return uT}var cT,P$;function Hnt(){if(P$)return cT;P$=1,cT=e,e.displayName="cil",e.aliases=[];function e(t){t.languages.cil={comment:/\/\/.*/,string:{pattern:/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0},directive:{pattern:/(^|\W)\.[a-z]+(?=\s)/,lookbehind:!0,alias:"class-name"},variable:/\[[\w\.]+\]/,keyword:/\b(?:abstract|ansi|assembly|auto|autochar|beforefieldinit|bool|bstr|byvalstr|catch|char|cil|class|currency|date|decimal|default|enum|error|explicit|extends|extern|famandassem|family|famorassem|final(?:ly)?|float32|float64|hidebysig|u?int(?:8|16|32|64)?|iant|idispatch|implements|import|initonly|instance|interface|iunknown|literal|lpstr|lpstruct|lptstr|lpwstr|managed|method|native(?:Type)?|nested|newslot|object(?:ref)?|pinvokeimpl|private|privatescope|public|reqsecobj|rtspecialname|runtime|sealed|sequential|serializable|specialname|static|string|struct|syschar|tbstr|unicode|unmanagedexp|unsigned|value(?:type)?|variant|virtual|void)\b/,function:/\b(?:(?:constrained|no|readonly|tail|unaligned|volatile)\.)?(?:conv\.(?:[iu][1248]?|ovf\.[iu][1248]?(?:\.un)?|r\.un|r4|r8)|ldc\.(?:i4(?:\.\d+|\.[mM]1|\.s)?|i8|r4|r8)|ldelem(?:\.[iu][1248]?|\.r[48]|\.ref|a)?|ldind\.(?:[iu][1248]?|r[48]|ref)|stelem\.?(?:i[1248]?|r[48]|ref)?|stind\.(?:i[1248]?|r[48]|ref)?|end(?:fault|filter|finally)|ldarg(?:\.[0-3s]|a(?:\.s)?)?|ldloc(?:\.\d+|\.s)?|sub(?:\.ovf(?:\.un)?)?|mul(?:\.ovf(?:\.un)?)?|add(?:\.ovf(?:\.un)?)?|stloc(?:\.[0-3s])?|refany(?:type|val)|blt(?:\.un)?(?:\.s)?|ble(?:\.un)?(?:\.s)?|bgt(?:\.un)?(?:\.s)?|bge(?:\.un)?(?:\.s)?|unbox(?:\.any)?|init(?:blk|obj)|call(?:i|virt)?|brfalse(?:\.s)?|bne\.un(?:\.s)?|ldloca(?:\.s)?|brzero(?:\.s)?|brtrue(?:\.s)?|brnull(?:\.s)?|brinst(?:\.s)?|starg(?:\.s)?|leave(?:\.s)?|shr(?:\.un)?|rem(?:\.un)?|div(?:\.un)?|clt(?:\.un)?|alignment|castclass|ldvirtftn|beq(?:\.s)?|ckfinite|ldsflda|ldtoken|localloc|mkrefany|rethrow|cgt\.un|arglist|switch|stsfld|sizeof|newobj|newarr|ldsfld|ldnull|ldflda|isinst|throw|stobj|stfld|ldstr|ldobj|ldlen|ldftn|ldfld|cpobj|cpblk|break|br\.s|xor|shl|ret|pop|not|nop|neg|jmp|dup|cgt|ceq|box|and|or|br)\b/,boolean:/\b(?:false|true)\b/,number:/\b-?(?:0x[0-9a-f]+|\d+)(?:\.[0-9a-f]+)?\b/i,punctuation:/[{}[\];(),:=]|IL_[0-9A-Za-z]+/}}return cT}var dT,z$;function Unt(){if(z$)return dT;z$=1,dT=e,e.displayName="clojure",e.aliases=[];function e(t){t.languages.clojure={comment:{pattern:/;.*/,greedy:!0},string:{pattern:/"(?:[^"\\]|\\.)*"/,greedy:!0},char:/\\\w+/,symbol:{pattern:/(^|[\s()\[\]{},])::?[\w*+!?'<>=/.-]+/,lookbehind:!0},keyword:{pattern:/(\()(?:-|->|->>|\.|\.\.|\*|\/|\+|<|<=|=|==|>|>=|accessor|agent|agent-errors|aget|alength|all-ns|alter|and|append-child|apply|array-map|aset|aset-boolean|aset-byte|aset-char|aset-double|aset-float|aset-int|aset-long|aset-short|assert|assoc|await|await-for|bean|binding|bit-and|bit-not|bit-or|bit-shift-left|bit-shift-right|bit-xor|boolean|branch\?|butlast|byte|cast|char|children|class|clear-agent-errors|comment|commute|comp|comparator|complement|concat|cond|conj|cons|constantly|construct-proxy|contains\?|count|create-ns|create-struct|cycle|dec|declare|def|def-|definline|definterface|defmacro|defmethod|defmulti|defn|defn-|defonce|defproject|defprotocol|defrecord|defstruct|deftype|deref|difference|disj|dissoc|distinct|do|doall|doc|dorun|doseq|dosync|dotimes|doto|double|down|drop|drop-while|edit|end\?|ensure|eval|every\?|false\?|ffirst|file-seq|filter|find|find-doc|find-ns|find-var|first|float|flush|fn|fnseq|for|frest|gensym|get|get-proxy-class|hash-map|hash-set|identical\?|identity|if|if-let|if-not|import|in-ns|inc|index|insert-child|insert-left|insert-right|inspect-table|inspect-tree|instance\?|int|interleave|intersection|into|into-array|iterate|join|key|keys|keyword|keyword\?|last|lazy-cat|lazy-cons|left|lefts|let|line-seq|list|list\*|load|load-file|locking|long|loop|macroexpand|macroexpand-1|make-array|make-node|map|map-invert|map\?|mapcat|max|max-key|memfn|merge|merge-with|meta|min|min-key|monitor-enter|name|namespace|neg\?|new|newline|next|nil\?|node|not|not-any\?|not-every\?|not=|ns|ns-imports|ns-interns|ns-map|ns-name|ns-publics|ns-refers|ns-resolve|ns-unmap|nth|nthrest|or|parse|partial|path|peek|pop|pos\?|pr|pr-str|print|print-str|println|println-str|prn|prn-str|project|proxy|proxy-mappings|quot|quote|rand|rand-int|range|re-find|re-groups|re-matcher|re-matches|re-pattern|re-seq|read|read-line|recur|reduce|ref|ref-set|refer|rem|remove|remove-method|remove-ns|rename|rename-keys|repeat|replace|replicate|resolve|rest|resultset-seq|reverse|rfirst|right|rights|root|rrest|rseq|second|select|select-keys|send|send-off|seq|seq-zip|seq\?|set|set!|short|slurp|some|sort|sort-by|sorted-map|sorted-map-by|sorted-set|special-symbol\?|split-at|split-with|str|string\?|struct|struct-map|subs|subvec|symbol|symbol\?|sync|take|take-nth|take-while|test|throw|time|to-array|to-array-2d|tree-seq|true\?|try|union|up|update-proxy|val|vals|var|var-get|var-set|var\?|vector|vector-zip|vector\?|when|when-first|when-let|when-not|with-local-vars|with-meta|with-open|with-out-str|xml-seq|xml-zip|zero\?|zipmap|zipper)(?=[\s)]|$)/,lookbehind:!0},boolean:/\b(?:false|nil|true)\b/,number:{pattern:/(^|[^\w$@])(?:\d+(?:[/.]\d+)?(?:e[+-]?\d+)?|0x[a-f0-9]+|[1-9]\d?r[a-z0-9]+)[lmn]?(?![\w$@])/i,lookbehind:!0},function:{pattern:/((?:^|[^'])\()[\w*+!?'<>=/.-]+(?=[\s)]|$)/,lookbehind:!0},operator:/[#@^`~]/,punctuation:/[{}\[\](),]/}}return dT}var hT,H$;function $nt(){if(H$)return hT;H$=1,hT=e,e.displayName="cmake",e.aliases=[];function e(t){t.languages.cmake={comment:/#.*/,string:{pattern:/"(?:[^\\"]|\\.)*"/,greedy:!0,inside:{interpolation:{pattern:/\$\{(?:[^{}$]|\$\{[^{}$]*\})*\}/,inside:{punctuation:/\$\{|\}/,variable:/\w+/}}}},variable:/\b(?:CMAKE_\w+|\w+_(?:(?:BINARY|SOURCE)_DIR|DESCRIPTION|HOMEPAGE_URL|ROOT|VERSION(?:_MAJOR|_MINOR|_PATCH|_TWEAK)?)|(?:ANDROID|APPLE|BORLAND|BUILD_SHARED_LIBS|CACHE|CPACK_(?:ABSOLUTE_DESTINATION_FILES|COMPONENT_INCLUDE_TOPLEVEL_DIRECTORY|ERROR_ON_ABSOLUTE_INSTALL_DESTINATION|INCLUDE_TOPLEVEL_DIRECTORY|INSTALL_DEFAULT_DIRECTORY_PERMISSIONS|INSTALL_SCRIPT|PACKAGING_INSTALL_PREFIX|SET_DESTDIR|WARN_ON_ABSOLUTE_INSTALL_DESTINATION)|CTEST_(?:BINARY_DIRECTORY|BUILD_COMMAND|BUILD_NAME|BZR_COMMAND|BZR_UPDATE_OPTIONS|CHANGE_ID|CHECKOUT_COMMAND|CONFIGURATION_TYPE|CONFIGURE_COMMAND|COVERAGE_COMMAND|COVERAGE_EXTRA_FLAGS|CURL_OPTIONS|CUSTOM_(?:COVERAGE_EXCLUDE|ERROR_EXCEPTION|ERROR_MATCH|ERROR_POST_CONTEXT|ERROR_PRE_CONTEXT|MAXIMUM_FAILED_TEST_OUTPUT_SIZE|MAXIMUM_NUMBER_OF_(?:ERRORS|WARNINGS)|MAXIMUM_PASSED_TEST_OUTPUT_SIZE|MEMCHECK_IGNORE|POST_MEMCHECK|POST_TEST|PRE_MEMCHECK|PRE_TEST|TESTS_IGNORE|WARNING_EXCEPTION|WARNING_MATCH)|CVS_CHECKOUT|CVS_COMMAND|CVS_UPDATE_OPTIONS|DROP_LOCATION|DROP_METHOD|DROP_SITE|DROP_SITE_CDASH|DROP_SITE_PASSWORD|DROP_SITE_USER|EXTRA_COVERAGE_GLOB|GIT_COMMAND|GIT_INIT_SUBMODULES|GIT_UPDATE_CUSTOM|GIT_UPDATE_OPTIONS|HG_COMMAND|HG_UPDATE_OPTIONS|LABELS_FOR_SUBPROJECTS|MEMORYCHECK_(?:COMMAND|COMMAND_OPTIONS|SANITIZER_OPTIONS|SUPPRESSIONS_FILE|TYPE)|NIGHTLY_START_TIME|P4_CLIENT|P4_COMMAND|P4_OPTIONS|P4_UPDATE_OPTIONS|RUN_CURRENT_SCRIPT|SCP_COMMAND|SITE|SOURCE_DIRECTORY|SUBMIT_URL|SVN_COMMAND|SVN_OPTIONS|SVN_UPDATE_OPTIONS|TEST_LOAD|TEST_TIMEOUT|TRIGGER_SITE|UPDATE_COMMAND|UPDATE_OPTIONS|UPDATE_VERSION_ONLY|USE_LAUNCHERS)|CYGWIN|ENV|EXECUTABLE_OUTPUT_PATH|GHS-MULTI|IOS|LIBRARY_OUTPUT_PATH|MINGW|MSVC(?:10|11|12|14|60|70|71|80|90|_IDE|_TOOLSET_VERSION|_VERSION)?|MSYS|PROJECT_(?:BINARY_DIR|DESCRIPTION|HOMEPAGE_URL|NAME|SOURCE_DIR|VERSION|VERSION_(?:MAJOR|MINOR|PATCH|TWEAK))|UNIX|WIN32|WINCE|WINDOWS_PHONE|WINDOWS_STORE|XCODE|XCODE_VERSION))\b/,property:/\b(?:cxx_\w+|(?:ARCHIVE_OUTPUT_(?:DIRECTORY|NAME)|COMPILE_DEFINITIONS|COMPILE_PDB_NAME|COMPILE_PDB_OUTPUT_DIRECTORY|EXCLUDE_FROM_DEFAULT_BUILD|IMPORTED_(?:IMPLIB|LIBNAME|LINK_DEPENDENT_LIBRARIES|LINK_INTERFACE_LANGUAGES|LINK_INTERFACE_LIBRARIES|LINK_INTERFACE_MULTIPLICITY|LOCATION|NO_SONAME|OBJECTS|SONAME)|INTERPROCEDURAL_OPTIMIZATION|LIBRARY_OUTPUT_DIRECTORY|LIBRARY_OUTPUT_NAME|LINK_FLAGS|LINK_INTERFACE_LIBRARIES|LINK_INTERFACE_MULTIPLICITY|LOCATION|MAP_IMPORTED_CONFIG|OSX_ARCHITECTURES|OUTPUT_NAME|PDB_NAME|PDB_OUTPUT_DIRECTORY|RUNTIME_OUTPUT_DIRECTORY|RUNTIME_OUTPUT_NAME|STATIC_LIBRARY_FLAGS|VS_CSHARP|VS_DOTNET_REFERENCEPROP|VS_DOTNET_REFERENCE|VS_GLOBAL_SECTION_POST|VS_GLOBAL_SECTION_PRE|VS_GLOBAL|XCODE_ATTRIBUTE)_\w+|\w+_(?:CLANG_TIDY|COMPILER_LAUNCHER|CPPCHECK|CPPLINT|INCLUDE_WHAT_YOU_USE|OUTPUT_NAME|POSTFIX|VISIBILITY_PRESET)|ABSTRACT|ADDITIONAL_MAKE_CLEAN_FILES|ADVANCED|ALIASED_TARGET|ALLOW_DUPLICATE_CUSTOM_TARGETS|ANDROID_(?:ANT_ADDITIONAL_OPTIONS|API|API_MIN|ARCH|ASSETS_DIRECTORIES|GUI|JAR_DEPENDENCIES|NATIVE_LIB_DEPENDENCIES|NATIVE_LIB_DIRECTORIES|PROCESS_MAX|PROGUARD|PROGUARD_CONFIG_PATH|SECURE_PROPS_PATH|SKIP_ANT_STEP|STL_TYPE)|ARCHIVE_OUTPUT_DIRECTORY|ATTACHED_FILES|ATTACHED_FILES_ON_FAIL|AUTOGEN_(?:BUILD_DIR|ORIGIN_DEPENDS|PARALLEL|SOURCE_GROUP|TARGETS_FOLDER|TARGET_DEPENDS)|AUTOMOC|AUTOMOC_(?:COMPILER_PREDEFINES|DEPEND_FILTERS|EXECUTABLE|MACRO_NAMES|MOC_OPTIONS|SOURCE_GROUP|TARGETS_FOLDER)|AUTORCC|AUTORCC_EXECUTABLE|AUTORCC_OPTIONS|AUTORCC_SOURCE_GROUP|AUTOUIC|AUTOUIC_EXECUTABLE|AUTOUIC_OPTIONS|AUTOUIC_SEARCH_PATHS|BINARY_DIR|BUILDSYSTEM_TARGETS|BUILD_RPATH|BUILD_RPATH_USE_ORIGIN|BUILD_WITH_INSTALL_NAME_DIR|BUILD_WITH_INSTALL_RPATH|BUNDLE|BUNDLE_EXTENSION|CACHE_VARIABLES|CLEAN_NO_CUSTOM|COMMON_LANGUAGE_RUNTIME|COMPATIBLE_INTERFACE_(?:BOOL|NUMBER_MAX|NUMBER_MIN|STRING)|COMPILE_(?:DEFINITIONS|FEATURES|FLAGS|OPTIONS|PDB_NAME|PDB_OUTPUT_DIRECTORY)|COST|CPACK_DESKTOP_SHORTCUTS|CPACK_NEVER_OVERWRITE|CPACK_PERMANENT|CPACK_STARTUP_SHORTCUTS|CPACK_START_MENU_SHORTCUTS|CPACK_WIX_ACL|CROSSCOMPILING_EMULATOR|CUDA_EXTENSIONS|CUDA_PTX_COMPILATION|CUDA_RESOLVE_DEVICE_SYMBOLS|CUDA_SEPARABLE_COMPILATION|CUDA_STANDARD|CUDA_STANDARD_REQUIRED|CXX_EXTENSIONS|CXX_STANDARD|CXX_STANDARD_REQUIRED|C_EXTENSIONS|C_STANDARD|C_STANDARD_REQUIRED|DEBUG_CONFIGURATIONS|DEFINE_SYMBOL|DEFINITIONS|DEPENDS|DEPLOYMENT_ADDITIONAL_FILES|DEPLOYMENT_REMOTE_DIRECTORY|DISABLED|DISABLED_FEATURES|ECLIPSE_EXTRA_CPROJECT_CONTENTS|ECLIPSE_EXTRA_NATURES|ENABLED_FEATURES|ENABLED_LANGUAGES|ENABLE_EXPORTS|ENVIRONMENT|EXCLUDE_FROM_ALL|EXCLUDE_FROM_DEFAULT_BUILD|EXPORT_NAME|EXPORT_PROPERTIES|EXTERNAL_OBJECT|EchoString|FAIL_REGULAR_EXPRESSION|FIND_LIBRARY_USE_LIB32_PATHS|FIND_LIBRARY_USE_LIB64_PATHS|FIND_LIBRARY_USE_LIBX32_PATHS|FIND_LIBRARY_USE_OPENBSD_VERSIONING|FIXTURES_CLEANUP|FIXTURES_REQUIRED|FIXTURES_SETUP|FOLDER|FRAMEWORK|Fortran_FORMAT|Fortran_MODULE_DIRECTORY|GENERATED|GENERATOR_FILE_NAME|GENERATOR_IS_MULTI_CONFIG|GHS_INTEGRITY_APP|GHS_NO_SOURCE_GROUP_FILE|GLOBAL_DEPENDS_DEBUG_MODE|GLOBAL_DEPENDS_NO_CYCLES|GNUtoMS|HAS_CXX|HEADER_FILE_ONLY|HELPSTRING|IMPLICIT_DEPENDS_INCLUDE_TRANSFORM|IMPORTED|IMPORTED_(?:COMMON_LANGUAGE_RUNTIME|CONFIGURATIONS|GLOBAL|IMPLIB|LIBNAME|LINK_DEPENDENT_LIBRARIES|LINK_INTERFACE_(?:LANGUAGES|LIBRARIES|MULTIPLICITY)|LOCATION|NO_SONAME|OBJECTS|SONAME)|IMPORT_PREFIX|IMPORT_SUFFIX|INCLUDE_DIRECTORIES|INCLUDE_REGULAR_EXPRESSION|INSTALL_NAME_DIR|INSTALL_RPATH|INSTALL_RPATH_USE_LINK_PATH|INTERFACE_(?:AUTOUIC_OPTIONS|COMPILE_DEFINITIONS|COMPILE_FEATURES|COMPILE_OPTIONS|INCLUDE_DIRECTORIES|LINK_DEPENDS|LINK_DIRECTORIES|LINK_LIBRARIES|LINK_OPTIONS|POSITION_INDEPENDENT_CODE|SOURCES|SYSTEM_INCLUDE_DIRECTORIES)|INTERPROCEDURAL_OPTIMIZATION|IN_TRY_COMPILE|IOS_INSTALL_COMBINED|JOB_POOLS|JOB_POOL_COMPILE|JOB_POOL_LINK|KEEP_EXTENSION|LABELS|LANGUAGE|LIBRARY_OUTPUT_DIRECTORY|LINKER_LANGUAGE|LINK_(?:DEPENDS|DEPENDS_NO_SHARED|DIRECTORIES|FLAGS|INTERFACE_LIBRARIES|INTERFACE_MULTIPLICITY|LIBRARIES|OPTIONS|SEARCH_END_STATIC|SEARCH_START_STATIC|WHAT_YOU_USE)|LISTFILE_STACK|LOCATION|MACOSX_BUNDLE|MACOSX_BUNDLE_INFO_PLIST|MACOSX_FRAMEWORK_INFO_PLIST|MACOSX_PACKAGE_LOCATION|MACOSX_RPATH|MACROS|MANUALLY_ADDED_DEPENDENCIES|MEASUREMENT|MODIFIED|NAME|NO_SONAME|NO_SYSTEM_FROM_IMPORTED|OBJECT_DEPENDS|OBJECT_OUTPUTS|OSX_ARCHITECTURES|OUTPUT_NAME|PACKAGES_FOUND|PACKAGES_NOT_FOUND|PARENT_DIRECTORY|PASS_REGULAR_EXPRESSION|PDB_NAME|PDB_OUTPUT_DIRECTORY|POSITION_INDEPENDENT_CODE|POST_INSTALL_SCRIPT|PREDEFINED_TARGETS_FOLDER|PREFIX|PRE_INSTALL_SCRIPT|PRIVATE_HEADER|PROCESSORS|PROCESSOR_AFFINITY|PROJECT_LABEL|PUBLIC_HEADER|REPORT_UNDEFINED_PROPERTIES|REQUIRED_FILES|RESOURCE|RESOURCE_LOCK|RULE_LAUNCH_COMPILE|RULE_LAUNCH_CUSTOM|RULE_LAUNCH_LINK|RULE_MESSAGES|RUNTIME_OUTPUT_DIRECTORY|RUN_SERIAL|SKIP_AUTOGEN|SKIP_AUTOMOC|SKIP_AUTORCC|SKIP_AUTOUIC|SKIP_BUILD_RPATH|SKIP_RETURN_CODE|SOURCES|SOURCE_DIR|SOVERSION|STATIC_LIBRARY_FLAGS|STATIC_LIBRARY_OPTIONS|STRINGS|SUBDIRECTORIES|SUFFIX|SYMBOLIC|TARGET_ARCHIVES_MAY_BE_SHARED_LIBS|TARGET_MESSAGES|TARGET_SUPPORTS_SHARED_LIBS|TESTS|TEST_INCLUDE_FILE|TEST_INCLUDE_FILES|TIMEOUT|TIMEOUT_AFTER_MATCH|TYPE|USE_FOLDERS|VALUE|VARIABLES|VERSION|VISIBILITY_INLINES_HIDDEN|VS_(?:CONFIGURATION_TYPE|COPY_TO_OUT_DIR|DEBUGGER_(?:COMMAND|COMMAND_ARGUMENTS|ENVIRONMENT|WORKING_DIRECTORY)|DEPLOYMENT_CONTENT|DEPLOYMENT_LOCATION|DOTNET_REFERENCES|DOTNET_REFERENCES_COPY_LOCAL|GLOBAL_KEYWORD|GLOBAL_PROJECT_TYPES|GLOBAL_ROOTNAMESPACE|INCLUDE_IN_VSIX|IOT_STARTUP_TASK|KEYWORD|RESOURCE_GENERATOR|SCC_AUXPATH|SCC_LOCALPATH|SCC_PROJECTNAME|SCC_PROVIDER|SDK_REFERENCES|SHADER_(?:DISABLE_OPTIMIZATIONS|ENABLE_DEBUG|ENTRYPOINT|FLAGS|MODEL|OBJECT_FILE_NAME|OUTPUT_HEADER_FILE|TYPE|VARIABLE_NAME)|STARTUP_PROJECT|TOOL_OVERRIDE|USER_PROPS|WINRT_COMPONENT|WINRT_EXTENSIONS|WINRT_REFERENCES|XAML_TYPE)|WILL_FAIL|WIN32_EXECUTABLE|WINDOWS_EXPORT_ALL_SYMBOLS|WORKING_DIRECTORY|WRAP_EXCLUDE|XCODE_(?:EMIT_EFFECTIVE_PLATFORM_NAME|EXPLICIT_FILE_TYPE|FILE_ATTRIBUTES|LAST_KNOWN_FILE_TYPE|PRODUCT_TYPE|SCHEME_(?:ADDRESS_SANITIZER|ADDRESS_SANITIZER_USE_AFTER_RETURN|ARGUMENTS|DISABLE_MAIN_THREAD_CHECKER|DYNAMIC_LIBRARY_LOADS|DYNAMIC_LINKER_API_USAGE|ENVIRONMENT|EXECUTABLE|GUARD_MALLOC|MAIN_THREAD_CHECKER_STOP|MALLOC_GUARD_EDGES|MALLOC_SCRIBBLE|MALLOC_STACK|THREAD_SANITIZER(?:_STOP)?|UNDEFINED_BEHAVIOUR_SANITIZER(?:_STOP)?|ZOMBIE_OBJECTS))|XCTEST)\b/,keyword:/\b(?:add_compile_definitions|add_compile_options|add_custom_command|add_custom_target|add_definitions|add_dependencies|add_executable|add_library|add_link_options|add_subdirectory|add_test|aux_source_directory|break|build_command|build_name|cmake_host_system_information|cmake_minimum_required|cmake_parse_arguments|cmake_policy|configure_file|continue|create_test_sourcelist|ctest_build|ctest_configure|ctest_coverage|ctest_empty_binary_directory|ctest_memcheck|ctest_read_custom_files|ctest_run_script|ctest_sleep|ctest_start|ctest_submit|ctest_test|ctest_update|ctest_upload|define_property|else|elseif|enable_language|enable_testing|endforeach|endfunction|endif|endmacro|endwhile|exec_program|execute_process|export|export_library_dependencies|file|find_file|find_library|find_package|find_path|find_program|fltk_wrap_ui|foreach|function|get_cmake_property|get_directory_property|get_filename_component|get_property|get_source_file_property|get_target_property|get_test_property|if|include|include_directories|include_external_msproject|include_guard|include_regular_expression|install|install_files|install_programs|install_targets|link_directories|link_libraries|list|load_cache|load_command|macro|make_directory|mark_as_advanced|math|message|option|output_required_files|project|qt_wrap_cpp|qt_wrap_ui|remove|remove_definitions|return|separate_arguments|set|set_directory_properties|set_property|set_source_files_properties|set_target_properties|set_tests_properties|site_name|source_group|string|subdir_depends|subdirs|target_compile_definitions|target_compile_features|target_compile_options|target_include_directories|target_link_directories|target_link_libraries|target_link_options|target_sources|try_compile|try_run|unset|use_mangled_mesa|utility_source|variable_requires|variable_watch|while|write_file)(?=\s*\()\b/,boolean:/\b(?:FALSE|OFF|ON|TRUE)\b/,namespace:/\b(?:INTERFACE|PRIVATE|PROPERTIES|PUBLIC|SHARED|STATIC|TARGET_OBJECTS)\b/,operator:/\b(?:AND|DEFINED|EQUAL|GREATER|LESS|MATCHES|NOT|OR|STREQUAL|STRGREATER|STRLESS|VERSION_EQUAL|VERSION_GREATER|VERSION_LESS)\b/,inserted:{pattern:/\b\w+::\w+\b/,alias:"class-name"},number:/\b\d+(?:\.\d+)*\b/,function:/\b[a-z_]\w*(?=\s*\()\b/i,punctuation:/[()>}]|\$[<{]/}}return hT}var pT,U$;function Gnt(){if(U$)return pT;U$=1,pT=e,e.displayName="cobol",e.aliases=[];function e(t){t.languages.cobol={comment:{pattern:/\*>.*|(^[ \t]*)\*.*/m,lookbehind:!0,greedy:!0},string:{pattern:/[xzgn]?(?:"(?:[^\r\n"]|"")*"(?!")|'(?:[^\r\n']|'')*'(?!'))/i,greedy:!0},level:{pattern:/(^[ \t]*)\d+\b/m,lookbehind:!0,greedy:!0,alias:"number"},"class-name":{pattern:/(\bpic(?:ture)?\s+)(?:(?:[-\w$/,:*+<>]|\.(?!\s|$))(?:\(\d+\))?)+/i,lookbehind:!0,inside:{number:{pattern:/(\()\d+/,lookbehind:!0},punctuation:/[()]/}},keyword:{pattern:/(^|[^\w-])(?:ABORT|ACCEPT|ACCESS|ADD|ADDRESS|ADVANCING|AFTER|ALIGNED|ALL|ALPHABET|ALPHABETIC|ALPHABETIC-LOWER|ALPHABETIC-UPPER|ALPHANUMERIC|ALPHANUMERIC-EDITED|ALSO|ALTER|ALTERNATE|ANY|ARE|AREA|AREAS|AS|ASCENDING|ASCII|ASSIGN|ASSOCIATED-DATA|ASSOCIATED-DATA-LENGTH|AT|ATTRIBUTE|AUTHOR|AUTO|AUTO-SKIP|BACKGROUND-COLOR|BACKGROUND-COLOUR|BASIS|BEEP|BEFORE|BEGINNING|BELL|BINARY|BIT|BLANK|BLINK|BLOCK|BOTTOM|BOUNDS|BY|BYFUNCTION|BYTITLE|CALL|CANCEL|CAPABLE|CCSVERSION|CD|CF|CH|CHAINING|CHANGED|CHANNEL|CHARACTER|CHARACTERS|CLASS|CLASS-ID|CLOCK-UNITS|CLOSE|CLOSE-DISPOSITION|COBOL|CODE|CODE-SET|COL|COLLATING|COLUMN|COM-REG|COMMA|COMMITMENT|COMMON|COMMUNICATION|COMP|COMP-1|COMP-2|COMP-3|COMP-4|COMP-5|COMPUTATIONAL|COMPUTATIONAL-1|COMPUTATIONAL-2|COMPUTATIONAL-3|COMPUTATIONAL-4|COMPUTATIONAL-5|COMPUTE|CONFIGURATION|CONTAINS|CONTENT|CONTINUE|CONTROL|CONTROL-POINT|CONTROLS|CONVENTION|CONVERTING|COPY|CORR|CORRESPONDING|COUNT|CRUNCH|CURRENCY|CURSOR|DATA|DATA-BASE|DATE|DATE-COMPILED|DATE-WRITTEN|DAY|DAY-OF-WEEK|DBCS|DE|DEBUG-CONTENTS|DEBUG-ITEM|DEBUG-LINE|DEBUG-NAME|DEBUG-SUB-1|DEBUG-SUB-2|DEBUG-SUB-3|DEBUGGING|DECIMAL-POINT|DECLARATIVES|DEFAULT|DEFAULT-DISPLAY|DEFINITION|DELETE|DELIMITED|DELIMITER|DEPENDING|DESCENDING|DESTINATION|DETAIL|DFHRESP|DFHVALUE|DISABLE|DISK|DISPLAY|DISPLAY-1|DIVIDE|DIVISION|DONTCARE|DOUBLE|DOWN|DUPLICATES|DYNAMIC|EBCDIC|EGCS|EGI|ELSE|EMI|EMPTY-CHECK|ENABLE|END|END-ACCEPT|END-ADD|END-CALL|END-COMPUTE|END-DELETE|END-DIVIDE|END-EVALUATE|END-IF|END-MULTIPLY|END-OF-PAGE|END-PERFORM|END-READ|END-RECEIVE|END-RETURN|END-REWRITE|END-SEARCH|END-START|END-STRING|END-SUBTRACT|END-UNSTRING|END-WRITE|ENDING|ENTER|ENTRY|ENTRY-PROCEDURE|ENVIRONMENT|EOL|EOP|EOS|ERASE|ERROR|ESCAPE|ESI|EVALUATE|EVENT|EVERY|EXCEPTION|EXCLUSIVE|EXHIBIT|EXIT|EXPORT|EXTEND|EXTENDED|EXTERNAL|FD|FILE|FILE-CONTROL|FILLER|FINAL|FIRST|FOOTING|FOR|FOREGROUND-COLOR|FOREGROUND-COLOUR|FROM|FULL|FUNCTION|FUNCTION-POINTER|FUNCTIONNAME|GENERATE|GIVING|GLOBAL|GO|GOBACK|GRID|GROUP|HEADING|HIGH-VALUE|HIGH-VALUES|HIGHLIGHT|I-O|I-O-CONTROL|ID|IDENTIFICATION|IF|IMPLICIT|IMPORT|IN|INDEX|INDEXED|INDICATE|INITIAL|INITIALIZE|INITIATE|INPUT|INPUT-OUTPUT|INSPECT|INSTALLATION|INTEGER|INTO|INVALID|INVOKE|IS|JUST|JUSTIFIED|KANJI|KEPT|KEY|KEYBOARD|LABEL|LANGUAGE|LAST|LB|LD|LEADING|LEFT|LEFTLINE|LENGTH|LENGTH-CHECK|LIBACCESS|LIBPARAMETER|LIBRARY|LIMIT|LIMITS|LINAGE|LINAGE-COUNTER|LINE|LINE-COUNTER|LINES|LINKAGE|LIST|LOCAL|LOCAL-STORAGE|LOCK|LONG-DATE|LONG-TIME|LOW-VALUE|LOW-VALUES|LOWER|LOWLIGHT|MEMORY|MERGE|MESSAGE|MMDDYYYY|MODE|MODULES|MORE-LABELS|MOVE|MULTIPLE|MULTIPLY|NAMED|NATIONAL|NATIONAL-EDITED|NATIVE|NEGATIVE|NETWORK|NEXT|NO|NO-ECHO|NULL|NULLS|NUMBER|NUMERIC|NUMERIC-DATE|NUMERIC-EDITED|NUMERIC-TIME|OBJECT-COMPUTER|OCCURS|ODT|OF|OFF|OMITTED|ON|OPEN|OPTIONAL|ORDER|ORDERLY|ORGANIZATION|OTHER|OUTPUT|OVERFLOW|OVERLINE|OWN|PACKED-DECIMAL|PADDING|PAGE|PAGE-COUNTER|PASSWORD|PERFORM|PF|PH|PIC|PICTURE|PLUS|POINTER|PORT|POSITION|POSITIVE|PRINTER|PRINTING|PRIVATE|PROCEDURE|PROCEDURE-POINTER|PROCEDURES|PROCEED|PROCESS|PROGRAM|PROGRAM-ID|PROGRAM-LIBRARY|PROMPT|PURGE|QUEUE|QUOTE|QUOTES|RANDOM|RD|READ|READER|REAL|RECEIVE|RECEIVED|RECORD|RECORDING|RECORDS|RECURSIVE|REDEFINES|REEL|REF|REFERENCE|REFERENCES|RELATIVE|RELEASE|REMAINDER|REMARKS|REMOTE|REMOVAL|REMOVE|RENAMES|REPLACE|REPLACING|REPORT|REPORTING|REPORTS|REQUIRED|RERUN|RESERVE|RESET|RETURN|RETURN-CODE|RETURNING|REVERSE-VIDEO|REVERSED|REWIND|REWRITE|RF|RH|RIGHT|ROUNDED|RUN|SAME|SAVE|SCREEN|SD|SEARCH|SECTION|SECURE|SECURITY|SEGMENT|SEGMENT-LIMIT|SELECT|SEND|SENTENCE|SEPARATE|SEQUENCE|SEQUENTIAL|SET|SHARED|SHAREDBYALL|SHAREDBYRUNUNIT|SHARING|SHIFT-IN|SHIFT-OUT|SHORT-DATE|SIGN|SIZE|SORT|SORT-CONTROL|SORT-CORE-SIZE|SORT-FILE-SIZE|SORT-MERGE|SORT-MESSAGE|SORT-MODE-SIZE|SORT-RETURN|SOURCE|SOURCE-COMPUTER|SPACE|SPACES|SPECIAL-NAMES|STANDARD|STANDARD-1|STANDARD-2|START|STATUS|STOP|STRING|SUB-QUEUE-1|SUB-QUEUE-2|SUB-QUEUE-3|SUBTRACT|SUM|SUPPRESS|SYMBOL|SYMBOLIC|SYNC|SYNCHRONIZED|TABLE|TALLY|TALLYING|TAPE|TASK|TERMINAL|TERMINATE|TEST|TEXT|THEN|THREAD|THREAD-LOCAL|THROUGH|THRU|TIME|TIMER|TIMES|TITLE|TO|TODAYS-DATE|TODAYS-NAME|TOP|TRAILING|TRUNCATED|TYPE|TYPEDEF|UNDERLINE|UNIT|UNSTRING|UNTIL|UP|UPON|USAGE|USE|USING|VALUE|VALUES|VARYING|VIRTUAL|WAIT|WHEN|WHEN-COMPILED|WITH|WORDS|WORKING-STORAGE|WRITE|YEAR|YYYYDDD|YYYYMMDD|ZERO-FILL|ZEROES|ZEROS)(?![\w-])/i,lookbehind:!0},boolean:{pattern:/(^|[^\w-])(?:false|true)(?![\w-])/i,lookbehind:!0},number:{pattern:/(^|[^\w-])(?:[+-]?(?:(?:\d+(?:[.,]\d+)?|[.,]\d+)(?:e[+-]?\d+)?|zero))(?![\w-])/i,lookbehind:!0},operator:[/<>|[<>]=?|[=+*/&]/,{pattern:/(^|[^\w-])(?:-|and|equal|greater|less|not|or|than)(?![\w-])/i,lookbehind:!0}],punctuation:/[.:,()]/}}return pT}var fT,$$;function jnt(){if($$)return fT;$$=1,fT=e,e.displayName="coffeescript",e.aliases=["coffee"];function e(t){(function(r){var n=/#(?!\{).+/,a={pattern:/#\{[^}]+\}/,alias:"variable"};r.languages.coffeescript=r.languages.extend("javascript",{comment:n,string:[{pattern:/'(?:\\[\s\S]|[^\\'])*'/,greedy:!0},{pattern:/"(?:\\[\s\S]|[^\\"])*"/,greedy:!0,inside:{interpolation:a}}],keyword:/\b(?:and|break|by|catch|class|continue|debugger|delete|do|each|else|extend|extends|false|finally|for|if|in|instanceof|is|isnt|let|loop|namespace|new|no|not|null|of|off|on|or|own|return|super|switch|then|this|throw|true|try|typeof|undefined|unless|until|when|while|window|with|yes|yield)\b/,"class-member":{pattern:/@(?!\d)\w+/,alias:"variable"}}),r.languages.insertBefore("coffeescript","comment",{"multiline-comment":{pattern:/###[\s\S]+?###/,alias:"comment"},"block-regex":{pattern:/\/{3}[\s\S]*?\/{3}/,alias:"regex",inside:{comment:n,interpolation:a}}}),r.languages.insertBefore("coffeescript","string",{"inline-javascript":{pattern:/`(?:\\[\s\S]|[^\\`])*`/,inside:{delimiter:{pattern:/^`|`$/,alias:"punctuation"},script:{pattern:/[\s\S]+/,alias:"language-javascript",inside:r.languages.javascript}}},"multiline-string":[{pattern:/'''[\s\S]*?'''/,greedy:!0,alias:"string"},{pattern:/"""[\s\S]*?"""/,greedy:!0,alias:"string",inside:{interpolation:a}}]}),r.languages.insertBefore("coffeescript","keyword",{property:/(?!\d)\w+(?=\s*:(?!:))/}),delete r.languages.coffeescript["template-string"],r.languages.coffee=r.languages.coffeescript})(t)}return fT}var mT,G$;function qnt(){if(G$)return mT;G$=1,mT=e,e.displayName="concurnas",e.aliases=["conc"];function e(t){t.languages.concurnas={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?(?:\*\/|$)|\/\/.*)/,lookbehind:!0,greedy:!0},langext:{pattern:/\b\w+\s*\|\|[\s\S]+?\|\|/,greedy:!0,inside:{"class-name":/^\w+/,string:{pattern:/(^\s*\|\|)[\s\S]+(?=\|\|$)/,lookbehind:!0},punctuation:/\|\|/}},function:{pattern:/((?:^|\s)def[ \t]+)[a-zA-Z_]\w*(?=\s*\()/,lookbehind:!0},keyword:/\b(?:abstract|actor|also|annotation|assert|async|await|bool|boolean|break|byte|case|catch|changed|char|class|closed|constant|continue|def|default|del|double|elif|else|enum|every|extends|false|finally|float|for|from|global|gpudef|gpukernel|if|import|in|init|inject|int|lambda|local|long|loop|match|new|nodefault|null|of|onchange|open|out|override|package|parfor|parforsync|post|pre|private|protected|provide|provider|public|return|shared|short|single|size_t|sizeof|super|sync|this|throw|trait|trans|transient|true|try|typedef|unchecked|using|val|var|void|while|with)\b/,boolean:/\b(?:false|true)\b/,number:/\b0b[01][01_]*L?\b|\b0x(?:[\da-f_]*\.)?[\da-f_p+-]+\b|(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.\d[\d_]*)(?:e[+-]?\d[\d_]*)?[dfls]?/i,punctuation:/[{}[\];(),.:]/,operator:/<==|>==|=>|->|<-|<>|&==|&<>|\?:?|\.\?|\+\+|--|[-+*/=<>]=?|[!^~]|\b(?:and|as|band|bor|bxor|comp|is|isnot|mod|or)\b=?/,annotation:{pattern:/@(?:\w+:)?(?:\w+|\[[^\]]+\])?/,alias:"builtin"}},t.languages.insertBefore("concurnas","langext",{"regex-literal":{pattern:/\br("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,lookbehind:!0,inside:t.languages.concurnas},regex:/[\s\S]+/}},"string-literal":{pattern:/(?:\B|\bs)("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\{(?:[^{}]|\{(?:[^{}]|\{[^}]*\})*\})+\}/,lookbehind:!0,inside:t.languages.concurnas},string:/[\s\S]+/}}}),t.languages.conc=t.languages.concurnas}return mT}var gT,j$;function Wnt(){if(j$)return gT;j$=1,gT=e,e.displayName="coq",e.aliases=[];function e(t){(function(r){for(var n=/\(\*(?:[^(*]|\((?!\*)|\*(?!\))|<self>)*\*\)/.source,a=0;a<2;a++)n=n.replace(/<self>/g,function(){return n});n=n.replace(/<self>/g,"[]"),r.languages.coq={comment:RegExp(n),string:{pattern:/"(?:[^"]|"")*"(?!")/,greedy:!0},attribute:[{pattern:RegExp(/#\[(?:[^\[\]("]|"(?:[^"]|"")*"(?!")|\((?!\*)|<comment>)*\]/.source.replace(/<comment>/g,function(){return n})),greedy:!0,alias:"attr-name",inside:{comment:RegExp(n),string:{pattern:/"(?:[^"]|"")*"(?!")/,greedy:!0},operator:/=/,punctuation:/^#\[|\]$|[,()]/}},{pattern:/\b(?:Cumulative|Global|Local|Monomorphic|NonCumulative|Polymorphic|Private|Program)\b/,alias:"attr-name"}],keyword:/\b(?:Abort|About|Add|Admit|Admitted|All|Arguments|As|Assumptions|Axiom|Axioms|Back|BackTo|Backtrace|BinOp|BinOpSpec|BinRel|Bind|Blacklist|Canonical|Case|Cd|Check|Class|Classes|Close|CoFixpoint|CoInductive|Coercion|Coercions|Collection|Combined|Compute|Conjecture|Conjectures|Constant|Constants|Constraint|Constructors|Context|Corollary|Create|CstOp|Custom|Cut|Debug|Declare|Defined|Definition|Delimit|Dependencies|Dependent|Derive|Diffs|Drop|Elimination|End|Entry|Equality|Eval|Example|Existential|Existentials|Existing|Export|Extern|Extraction|Fact|Fail|Field|File|Firstorder|Fixpoint|Flags|Focus|From|Funclass|Function|Functional|GC|Generalizable|Goal|Grab|Grammar|Graph|Guarded|Haskell|Heap|Hide|Hint|HintDb|Hints|Hypotheses|Hypothesis|IF|Identity|Immediate|Implicit|Implicits|Import|Include|Induction|Inductive|Infix|Info|Initial|InjTyp|Inline|Inspect|Instance|Instances|Intro|Intros|Inversion|Inversion_clear|JSON|Language|Left|Lemma|Let|Lia|Libraries|Library|Load|LoadPath|Locate|Ltac|Ltac2|ML|Match|Method|Minimality|Module|Modules|Morphism|Next|NoInline|Notation|Number|OCaml|Obligation|Obligations|Opaque|Open|Optimize|Parameter|Parameters|Parametric|Path|Paths|Prenex|Preterm|Primitive|Print|Profile|Projections|Proof|Prop|PropBinOp|PropOp|PropUOp|Property|Proposition|Pwd|Qed|Quit|Rec|Record|Recursive|Redirect|Reduction|Register|Relation|Remark|Remove|Require|Reserved|Reset|Resolve|Restart|Rewrite|Right|Ring|Rings|SProp|Saturate|Save|Scheme|Scope|Scopes|Search|SearchHead|SearchPattern|SearchRewrite|Section|Separate|Set|Setoid|Show|Signatures|Solve|Solver|Sort|Sortclass|Sorted|Spec|Step|Strategies|Strategy|String|Structure|SubClass|Subgraph|SuchThat|Tactic|Term|TestCompile|Theorem|Time|Timeout|To|Transparent|Type|Typeclasses|Types|Typing|UnOp|UnOpSpec|Undelimit|Undo|Unfocus|Unfocused|Unfold|Universe|Universes|Unshelve|Variable|Variables|Variant|Verbose|View|Visibility|Zify|_|apply|as|at|by|cofix|else|end|exists|exists2|fix|for|forall|fun|if|in|let|match|measure|move|removed|return|struct|then|using|wf|where|with)\b/,number:/\b(?:0x[a-f0-9][a-f0-9_]*(?:\.[a-f0-9_]+)?(?:p[+-]?\d[\d_]*)?|\d[\d_]*(?:\.[\d_]+)?(?:e[+-]?\d[\d_]*)?)\b/i,punct:{pattern:/@\{|\{\||\[=|:>/,alias:"punctuation"},operator:/\/\\|\\\/|\.{2,3}|:{1,2}=|\*\*|[-=]>|<(?:->?|[+:=>]|<:)|>(?:=|->)|\|[-|]?|[-!%&*+/<=>?@^~']/,punctuation:/\.\(|`\(|@\{|`\{|\{\||\[=|:>|[:.,;(){}\[\]]/}})(t)}return gT}var bT,q$;function nS(){if(q$)return bT;q$=1,bT=e,e.displayName="ruby",e.aliases=["rb"];function e(t){(function(r){r.languages.ruby=r.languages.extend("clike",{comment:{pattern:/#.*|^=begin\s[\s\S]*?^=end/m,greedy:!0},"class-name":{pattern:/(\b(?:class|module)\s+|\bcatch\s+\()[\w.\\]+|\b[A-Z_]\w*(?=\s*\.\s*new\b)/,lookbehind:!0,inside:{punctuation:/[.\\]/}},keyword:/\b(?:BEGIN|END|alias|and|begin|break|case|class|def|define_method|defined|do|each|else|elsif|end|ensure|extend|for|if|in|include|module|new|next|nil|not|or|prepend|private|protected|public|raise|redo|require|rescue|retry|return|self|super|then|throw|undef|unless|until|when|while|yield)\b/,operator:/\.{2,3}|&\.|===|<?=>|[!=]?~|(?:&&|\|\||<<|>>|\*\*|[+\-*/%<>!^&|=])=?|[?:]/,punctuation:/[(){}[\].,;]/}),r.languages.insertBefore("ruby","operator",{"double-colon":{pattern:/::/,alias:"punctuation"}});var n={pattern:/((?:^|[^\\])(?:\\{2})*)#\{(?:[^{}]|\{[^{}]*\})*\}/,lookbehind:!0,inside:{content:{pattern:/^(#\{)[\s\S]+(?=\}$)/,lookbehind:!0,inside:r.languages.ruby},delimiter:{pattern:/^#\{|\}$/,alias:"punctuation"}}};delete r.languages.ruby.function;var a="(?:"+[/([^a-zA-Z0-9\s{(\[<=])(?:(?!\1)[^\\]|\\[\s\S])*\1/.source,/\((?:[^()\\]|\\[\s\S]|\((?:[^()\\]|\\[\s\S])*\))*\)/.source,/\{(?:[^{}\\]|\\[\s\S]|\{(?:[^{}\\]|\\[\s\S])*\})*\}/.source,/\[(?:[^\[\]\\]|\\[\s\S]|\[(?:[^\[\]\\]|\\[\s\S])*\])*\]/.source,/<(?:[^<>\\]|\\[\s\S]|<(?:[^<>\\]|\\[\s\S])*>)*>/.source].join("|")+")",i=/(?:"(?:\\.|[^"\\\r\n])*"|(?:\b[a-zA-Z_]\w*|[^\s\0-\x7F]+)[?!]?|\$.)/.source;r.languages.insertBefore("ruby","keyword",{"regex-literal":[{pattern:RegExp(/%r/.source+a+/[egimnosux]{0,6}/.source),greedy:!0,inside:{interpolation:n,regex:/[\s\S]+/}},{pattern:/(^|[^/])\/(?!\/)(?:\[[^\r\n\]]+\]|\\.|[^[/\\\r\n])+\/[egimnosux]{0,6}(?=\s*(?:$|[\r\n,.;})#]))/,lookbehind:!0,greedy:!0,inside:{interpolation:n,regex:/[\s\S]+/}}],variable:/[@$]+[a-zA-Z_]\w*(?:[?!]|\b)/,symbol:[{pattern:RegExp(/(^|[^:]):/.source+i),lookbehind:!0,greedy:!0},{pattern:RegExp(/([\r\n{(,][ \t]*)/.source+i+/(?=:(?!:))/.source),lookbehind:!0,greedy:!0}],"method-definition":{pattern:/(\bdef\s+)\w+(?:\s*\.\s*\w+)?/,lookbehind:!0,inside:{function:/\b\w+$/,keyword:/^self\b/,"class-name":/^\w+/,punctuation:/\./}}}),r.languages.insertBefore("ruby","string",{"string-literal":[{pattern:RegExp(/%[qQiIwWs]?/.source+a),greedy:!0,inside:{interpolation:n,string:/[\s\S]+/}},{pattern:/("|')(?:#\{[^}]+\}|#(?!\{)|\\(?:\r\n|[\s\S])|(?!\1)[^\\#\r\n])*\1/,greedy:!0,inside:{interpolation:n,string:/[\s\S]+/}},{pattern:/<<[-~]?([a-z_]\w*)[\r\n](?:.*[\r\n])*?[\t ]*\1/i,alias:"heredoc-string",greedy:!0,inside:{delimiter:{pattern:/^<<[-~]?[a-z_]\w*|\b[a-z_]\w*$/i,inside:{symbol:/\b\w+/,punctuation:/^<<[-~]?/}},interpolation:n,string:/[\s\S]+/}},{pattern:/<<[-~]?'([a-z_]\w*)'[\r\n](?:.*[\r\n])*?[\t ]*\1/i,alias:"heredoc-string",greedy:!0,inside:{delimiter:{pattern:/^<<[-~]?'[a-z_]\w*'|\b[a-z_]\w*$/i,inside:{symbol:/\b\w+/,punctuation:/^<<[-~]?'|'$/}},string:/[\s\S]+/}}],"command-literal":[{pattern:RegExp(/%x/.source+a),greedy:!0,inside:{interpolation:n,command:{pattern:/[\s\S]+/,alias:"string"}}},{pattern:/`(?:#\{[^}]+\}|#(?!\{)|\\(?:\r\n|[\s\S])|[^\\`#\r\n])*`/,greedy:!0,inside:{interpolation:n,command:{pattern:/[\s\S]+/,alias:"string"}}}]}),delete r.languages.ruby.string,r.languages.insertBefore("ruby","number",{builtin:/\b(?:Array|Bignum|Binding|Class|Continuation|Dir|Exception|FalseClass|File|Fixnum|Float|Hash|IO|Integer|MatchData|Method|Module|NilClass|Numeric|Object|Proc|Range|Regexp|Stat|String|Struct|Symbol|TMS|Thread|ThreadGroup|Time|TrueClass)\b/,constant:/\b[A-Z][A-Z0-9_]*(?:[?!]|\b)/}),r.languages.rb=r.languages.ruby})(t)}return bT}var yT,W$;function Vnt(){if(W$)return yT;W$=1;var e=nS();yT=t,t.displayName="crystal",t.aliases=[];function t(r){r.register(e),(function(n){n.languages.crystal=n.languages.extend("ruby",{keyword:[/\b(?:__DIR__|__END_LINE__|__FILE__|__LINE__|abstract|alias|annotation|as|asm|begin|break|case|class|def|do|else|elsif|end|ensure|enum|extend|for|fun|if|ifdef|include|instance_sizeof|lib|macro|module|next|of|out|pointerof|private|protected|ptr|require|rescue|return|select|self|sizeof|struct|super|then|type|typeof|undef|uninitialized|union|unless|until|when|while|with|yield)\b/,{pattern:/(\.\s*)(?:is_a|responds_to)\?/,lookbehind:!0}],number:/\b(?:0b[01_]*[01]|0o[0-7_]*[0-7]|0x[\da-fA-F_]*[\da-fA-F]|(?:\d(?:[\d_]*\d)?)(?:\.[\d_]*\d)?(?:[eE][+-]?[\d_]*\d)?)(?:_(?:[uif](?:8|16|32|64))?)?\b/,operator:[/->/,n.languages.ruby.operator],punctuation:/[(){}[\].,;\\]/}),n.languages.insertBefore("crystal","string-literal",{attribute:{pattern:/@\[.*?\]/,inside:{delimiter:{pattern:/^@\[|\]$/,alias:"punctuation"},attribute:{pattern:/^(\s*)\w+/,lookbehind:!0,alias:"class-name"},args:{pattern:/\S(?:[\s\S]*\S)?/,inside:n.languages.crystal}}},expansion:{pattern:/\{(?:\{.*?\}|%.*?%)\}/,inside:{content:{pattern:/^(\{.)[\s\S]+(?=.\}$)/,lookbehind:!0,inside:n.languages.crystal},delimiter:{pattern:/^\{[\{%]|[\}%]\}$/,alias:"operator"}}},char:{pattern:/'(?:[^\\\r\n]{1,2}|\\(?:.|u(?:[A-Fa-f0-9]{1,4}|\{[A-Fa-f0-9]{1,6}\})))'/,greedy:!0}})})(r)}return yT}var vT,V$;function Knt(){if(V$)return vT;V$=1;var e=rS();vT=t,t.displayName="cshtml",t.aliases=["razor"];function t(r){r.register(e),(function(n){var a=/\/(?![/*])|\/\/.*[\r\n]|\/\*[^*]*(?:\*(?!\/)[^*]*)*\*\//.source,i=/@(?!")|"(?:[^\r\n\\"]|\\.)*"|@"(?:[^\\"]|""|\\[\s\S])*"(?!")/.source+"|"+/'(?:(?:[^\r\n'\\]|\\.|\\[Uux][\da-fA-F]{1,8})'|(?=[^\\](?!')))/.source;function o(S,w){for(var E=0;E<w;E++)S=S.replace(/<self>/g,function(){return"(?:"+S+")"});return S.replace(/<self>/g,"[^\\s\\S]").replace(/<str>/g,"(?:"+i+")").replace(/<comment>/g,"(?:"+a+")")}var s=o(/\((?:[^()'"@/]|<str>|<comment>|<self>)*\)/.source,2),l=o(/\[(?:[^\[\]'"@/]|<str>|<comment>|<self>)*\]/.source,2),u=o(/\{(?:[^{}'"@/]|<str>|<comment>|<self>)*\}/.source,2),d=o(/<(?:[^<>'"@/]|<str>|<comment>|<self>)*>/.source,2),p=/(?:\s(?:\s*[^\s>\/=]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))|(?=[\s/>])))+)?/.source,f=/(?!\d)[^\s>\/=$<%]+/.source+p+/\s*\/?>/.source,m=/\B@?/.source+"(?:"+/<([a-zA-Z][\w:]*)/.source+p+/\s*>/.source+"(?:"+(/[^<]/.source+"|"+/<\/?(?!\1\b)/.source+f+"|"+o(/<\1/.source+p+/\s*>/.source+"(?:"+(/[^<]/.source+"|"+/<\/?(?!\1\b)/.source+f+"|<self>")+")*"+/<\/\1\s*>/.source,2))+")*"+/<\/\1\s*>/.source+"|"+/</.source+f+")";n.languages.cshtml=n.languages.extend("markup",{});var b=n.languages.insertBefore("csharp","string",{html:{pattern:RegExp(m),greedy:!0,inside:n.languages.cshtml}},{csharp:n.languages.extend("csharp",{})}),y={pattern:/\S[\s\S]*/,alias:"language-csharp",inside:b};n.languages.insertBefore("cshtml","prolog",{"razor-comment":{pattern:/@\*[\s\S]*?\*@/,greedy:!0,alias:"comment"},block:{pattern:RegExp(/(^|[^@])@/.source+"(?:"+[u,/(?:code|functions)\s*/.source+u,/(?:for|foreach|lock|switch|using|while)\s*/.source+s+/\s*/.source+u,/do\s*/.source+u+/\s*while\s*/.source+s+/(?:\s*;)?/.source,/try\s*/.source+u+/\s*catch\s*/.source+s+/\s*/.source+u+/\s*finally\s*/.source+u,/if\s*/.source+s+/\s*/.source+u+"(?:"+/\s*else/.source+"(?:"+/\s+if\s*/.source+s+")?"+/\s*/.source+u+")*"].join("|")+")"),lookbehind:!0,greedy:!0,inside:{keyword:/^@\w*/,csharp:y}},directive:{pattern:/^([ \t]*)@(?:addTagHelper|attribute|implements|inherits|inject|layout|model|namespace|page|preservewhitespace|removeTagHelper|section|tagHelperPrefix|using)(?=\s).*/m,lookbehind:!0,greedy:!0,inside:{keyword:/^@\w+/,csharp:y}},value:{pattern:RegExp(/(^|[^@])@/.source+/(?:await\b\s*)?/.source+"(?:"+/\w+\b/.source+"|"+s+")(?:"+/[?!]?\.\w+\b/.source+"|"+s+"|"+l+"|"+d+s+")*"),lookbehind:!0,greedy:!0,alias:"variable",inside:{keyword:/^@/,csharp:y}},"delegate-operator":{pattern:/(^|[^@])@(?=<)/,lookbehind:!0,alias:"operator"}}),n.languages.razor=n.languages.cshtml})(r)}return vT}var ST,K$;function Ynt(){if(K$)return ST;K$=1,ST=e,e.displayName="csp",e.aliases=[];function e(t){(function(r){function n(a){return RegExp(/([ \t])/.source+"(?:"+a+")"+/(?=[\s;]|$)/.source,"i")}r.languages.csp={directive:{pattern:/(^|[\s;])(?:base-uri|block-all-mixed-content|(?:child|connect|default|font|frame|img|manifest|media|object|prefetch|script|style|worker)-src|disown-opener|form-action|frame-(?:ancestors|options)|input-protection(?:-(?:clip|selectors))?|navigate-to|plugin-types|policy-uri|referrer|reflected-xss|report-(?:to|uri)|require-sri-for|sandbox|(?:script|style)-src-(?:attr|elem)|upgrade-insecure-requests)(?=[\s;]|$)/i,lookbehind:!0,alias:"property"},scheme:{pattern:n(/[a-z][a-z0-9.+-]*:/.source),lookbehind:!0},none:{pattern:n(/'none'/.source),lookbehind:!0,alias:"keyword"},nonce:{pattern:n(/'nonce-[-+/\w=]+'/.source),lookbehind:!0,alias:"number"},hash:{pattern:n(/'sha(?:256|384|512)-[-+/\w=]+'/.source),lookbehind:!0,alias:"number"},host:{pattern:n(/[a-z][a-z0-9.+-]*:\/\/[^\s;,']*/.source+"|"+/\*[^\s;,']*/.source+"|"+/[a-z0-9-]+(?:\.[a-z0-9-]+)+(?::[\d*]+)?(?:\/[^\s;,']*)?/.source),lookbehind:!0,alias:"url",inside:{important:/\*/}},keyword:[{pattern:n(/'unsafe-[a-z-]+'/.source),lookbehind:!0,alias:"unsafe"},{pattern:n(/'[a-z-]+'/.source),lookbehind:!0,alias:"safe"}],punctuation:/;/}})(t)}return ST}var ET,Y$;function Jnt(){if(Y$)return ET;Y$=1,ET=e,e.displayName="cssExtras",e.aliases=[];function e(t){(function(r){var n=/("|')(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,a;r.languages.css.selector={pattern:r.languages.css.selector.pattern,lookbehind:!0,inside:a={"pseudo-element":/:(?:after|before|first-letter|first-line|selection)|::[-\w]+/,"pseudo-class":/:[-\w]+/,class:/\.[-\w]+/,id:/#[-\w]+/,attribute:{pattern:RegExp(`\\[(?:[^[\\]"']|`+n.source+")*\\]"),greedy:!0,inside:{punctuation:/^\[|\]$/,"case-sensitivity":{pattern:/(\s)[si]$/i,lookbehind:!0,alias:"keyword"},namespace:{pattern:/^(\s*)(?:(?!\s)[-*\w\xA0-\uFFFF])*\|(?!=)/,lookbehind:!0,inside:{punctuation:/\|$/}},"attr-name":{pattern:/^(\s*)(?:(?!\s)[-\w\xA0-\uFFFF])+/,lookbehind:!0},"attr-value":[n,{pattern:/(=\s*)(?:(?!\s)[-\w\xA0-\uFFFF])+(?=\s*$)/,lookbehind:!0}],operator:/[|~*^$]?=/}},"n-th":[{pattern:/(\(\s*)[+-]?\d*[\dn](?:\s*[+-]\s*\d+)?(?=\s*\))/,lookbehind:!0,inside:{number:/[\dn]+/,operator:/[+-]/}},{pattern:/(\(\s*)(?:even|odd)(?=\s*\))/i,lookbehind:!0}],combinator:/>|\+|~|\|\|/,punctuation:/[(),]/}},r.languages.css.atrule.inside["selector-function-argument"].inside=a,r.languages.insertBefore("css","property",{variable:{pattern:/(^|[^-\w\xA0-\uFFFF])--(?!\s)[-_a-z\xA0-\uFFFF](?:(?!\s)[-\w\xA0-\uFFFF])*/i,lookbehind:!0}});var i={pattern:/(\b\d+)(?:%|[a-z]+(?![\w-]))/,lookbehind:!0},o={pattern:/(^|[^\w.-])-?(?:\d+(?:\.\d+)?|\.\d+)/,lookbehind:!0};r.languages.insertBefore("css","function",{operator:{pattern:/(\s)[+\-*\/](?=\s)/,lookbehind:!0},hexcode:{pattern:/\B#[\da-f]{3,8}\b/i,alias:"color"},color:[{pattern:/(^|[^\w-])(?:AliceBlue|AntiqueWhite|Aqua|Aquamarine|Azure|Beige|Bisque|Black|BlanchedAlmond|Blue|BlueViolet|Brown|BurlyWood|CadetBlue|Chartreuse|Chocolate|Coral|CornflowerBlue|Cornsilk|Crimson|Cyan|DarkBlue|DarkCyan|DarkGoldenRod|DarkGr[ae]y|DarkGreen|DarkKhaki|DarkMagenta|DarkOliveGreen|DarkOrange|DarkOrchid|DarkRed|DarkSalmon|DarkSeaGreen|DarkSlateBlue|DarkSlateGr[ae]y|DarkTurquoise|DarkViolet|DeepPink|DeepSkyBlue|DimGr[ae]y|DodgerBlue|FireBrick|FloralWhite|ForestGreen|Fuchsia|Gainsboro|GhostWhite|Gold|GoldenRod|Gr[ae]y|Green|GreenYellow|HoneyDew|HotPink|IndianRed|Indigo|Ivory|Khaki|Lavender|LavenderBlush|LawnGreen|LemonChiffon|LightBlue|LightCoral|LightCyan|LightGoldenRodYellow|LightGr[ae]y|LightGreen|LightPink|LightSalmon|LightSeaGreen|LightSkyBlue|LightSlateGr[ae]y|LightSteelBlue|LightYellow|Lime|LimeGreen|Linen|Magenta|Maroon|MediumAquaMarine|MediumBlue|MediumOrchid|MediumPurple|MediumSeaGreen|MediumSlateBlue|MediumSpringGreen|MediumTurquoise|MediumVioletRed|MidnightBlue|MintCream|MistyRose|Moccasin|NavajoWhite|Navy|OldLace|Olive|OliveDrab|Orange|OrangeRed|Orchid|PaleGoldenRod|PaleGreen|PaleTurquoise|PaleVioletRed|PapayaWhip|PeachPuff|Peru|Pink|Plum|PowderBlue|Purple|Red|RosyBrown|RoyalBlue|SaddleBrown|Salmon|SandyBrown|SeaGreen|SeaShell|Sienna|Silver|SkyBlue|SlateBlue|SlateGr[ae]y|Snow|SpringGreen|SteelBlue|Tan|Teal|Thistle|Tomato|Transparent|Turquoise|Violet|Wheat|White|WhiteSmoke|Yellow|YellowGreen)(?![\w-])/i,lookbehind:!0},{pattern:/\b(?:hsl|rgb)\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*\)\B|\b(?:hsl|rgb)a\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*,\s*(?:0|0?\.\d+|1)\s*\)\B/i,inside:{unit:i,number:o,function:/[\w-]+(?=\()/,punctuation:/[(),]/}}],entity:/\\[\da-f]{1,8}/i,unit:i,number:o})})(t)}return ET}var xT,J$;function Xnt(){if(J$)return xT;J$=1,xT=e,e.displayName="csv",e.aliases=[];function e(t){t.languages.csv={value:/[^\r\n,"]+|"(?:[^"]|"")*"(?!")/,punctuation:/,/}}return xT}var wT,X$;function Znt(){if(X$)return wT;X$=1,wT=e,e.displayName="cypher",e.aliases=[];function e(t){t.languages.cypher={comment:/\/\/.*/,string:{pattern:/"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\\\r\n]|\\.)*'/,greedy:!0},"class-name":{pattern:/(:\s*)(?:\w+|`(?:[^`\\\r\n])*`)(?=\s*[{):])/,lookbehind:!0,greedy:!0},relationship:{pattern:/(-\[\s*(?:\w+\s*|`(?:[^`\\\r\n])*`\s*)?:\s*|\|\s*:\s*)(?:\w+|`(?:[^`\\\r\n])*`)/,lookbehind:!0,greedy:!0,alias:"property"},identifier:{pattern:/`(?:[^`\\\r\n])*`/,greedy:!0},variable:/\$\w+/,keyword:/\b(?:ADD|ALL|AND|AS|ASC|ASCENDING|ASSERT|BY|CALL|CASE|COMMIT|CONSTRAINT|CONTAINS|CREATE|CSV|DELETE|DESC|DESCENDING|DETACH|DISTINCT|DO|DROP|ELSE|END|ENDS|EXISTS|FOR|FOREACH|IN|INDEX|IS|JOIN|KEY|LIMIT|LOAD|MANDATORY|MATCH|MERGE|NODE|NOT|OF|ON|OPTIONAL|OR|ORDER(?=\s+BY)|PERIODIC|REMOVE|REQUIRE|RETURN|SCALAR|SCAN|SET|SKIP|START|STARTS|THEN|UNION|UNIQUE|UNWIND|USING|WHEN|WHERE|WITH|XOR|YIELD)\b/i,function:/\b\w+\b(?=\s*\()/,boolean:/\b(?:false|null|true)\b/i,number:/\b(?:0x[\da-fA-F]+|\d+(?:\.\d+)?(?:[eE][+-]?\d+)?)\b/,operator:/:|<--?|--?>?|<>|=~?|[<>]=?|[+*/%^|]|\.\.\.?/,punctuation:/[()[\]{},;.]/}}return wT}var kT,Z$;function Qnt(){if(Z$)return kT;Z$=1,kT=e,e.displayName="d",e.aliases=[];function e(t){t.languages.d=t.languages.extend("clike",{comment:[{pattern:/^\s*#!.+/,greedy:!0},{pattern:RegExp(/(^|[^\\])/.source+"(?:"+[/\/\+(?:\/\+(?:[^+]|\+(?!\/))*\+\/|(?!\/\+)[\s\S])*?\+\//.source,/\/\/.*/.source,/\/\*[\s\S]*?\*\//.source].join("|")+")"),lookbehind:!0,greedy:!0}],string:[{pattern:RegExp([/\b[rx]"(?:\\[\s\S]|[^\\"])*"[cwd]?/.source,/\bq"(?:\[[\s\S]*?\]|\([\s\S]*?\)|<[\s\S]*?>|\{[\s\S]*?\})"/.source,/\bq"((?!\d)\w+)$[\s\S]*?^\1"/.source,/\bq"(.)[\s\S]*?\2"/.source,/(["`])(?:\\[\s\S]|(?!\3)[^\\])*\3[cwd]?/.source].join("|"),"m"),greedy:!0},{pattern:/\bq\{(?:\{[^{}]*\}|[^{}])*\}/,greedy:!0,alias:"token-string"}],keyword:/\$|\b(?:__(?:(?:DATE|EOF|FILE|FUNCTION|LINE|MODULE|PRETTY_FUNCTION|TIMESTAMP|TIME|VENDOR|VERSION)__|gshared|parameters|traits|vector)|abstract|alias|align|asm|assert|auto|body|bool|break|byte|case|cast|catch|cdouble|cent|cfloat|char|class|const|continue|creal|dchar|debug|default|delegate|delete|deprecated|do|double|dstring|else|enum|export|extern|false|final|finally|float|for|foreach|foreach_reverse|function|goto|idouble|if|ifloat|immutable|import|inout|int|interface|invariant|ireal|lazy|long|macro|mixin|module|new|nothrow|null|out|override|package|pragma|private|protected|ptrdiff_t|public|pure|real|ref|return|scope|shared|short|size_t|static|string|struct|super|switch|synchronized|template|this|throw|true|try|typedef|typeid|typeof|ubyte|ucent|uint|ulong|union|unittest|ushort|version|void|volatile|wchar|while|with|wstring)\b/,number:[/\b0x\.?[a-f\d_]+(?:(?!\.\.)\.[a-f\d_]*)?(?:p[+-]?[a-f\d_]+)?[ulfi]{0,4}/i,{pattern:/((?:\.\.)?)(?:\b0b\.?|\b|\.)\d[\d_]*(?:(?!\.\.)\.[\d_]*)?(?:e[+-]?\d[\d_]*)?[ulfi]{0,4}/i,lookbehind:!0}],operator:/\|[|=]?|&[&=]?|\+[+=]?|-[-=]?|\.?\.\.|=[>=]?|!(?:i[ns]\b|<>?=?|>=?|=)?|\bi[ns]\b|(?:<[<>]?|>>?>?|\^\^|[*\/%^~])=?/}),t.languages.insertBefore("d","string",{char:/'(?:\\(?:\W|\w+)|[^\\])'/}),t.languages.insertBefore("d","keyword",{property:/\B@\w*/}),t.languages.insertBefore("d","function",{register:{pattern:/\b(?:[ABCD][LHX]|E?(?:BP|DI|SI|SP)|[BS]PL|[ECSDGF]S|CR[0234]|[DS]IL|DR[012367]|E[ABCD]X|X?MM[0-7]|R(?:1[0-5]|[89])[BWD]?|R[ABCD]X|R[BS]P|R[DS]I|TR[3-7]|XMM(?:1[0-5]|[89])|YMM(?:1[0-5]|\d))\b|\bST(?:\([0-7]\)|\b)/,alias:"variable"}})}return kT}var CT,Q$;function eat(){if(Q$)return CT;Q$=1,CT=e,e.displayName="dart",e.aliases=[];function e(t){(function(r){var n=[/\b(?:async|sync|yield)\*/,/\b(?:abstract|assert|async|await|break|case|catch|class|const|continue|covariant|default|deferred|do|dynamic|else|enum|export|extends|extension|external|factory|final|finally|for|get|hide|if|implements|import|in|interface|library|mixin|new|null|on|operator|part|rethrow|return|set|show|static|super|switch|sync|this|throw|try|typedef|var|void|while|with|yield)\b/],a=/(^|[^\w.])(?:[a-z]\w*\s*\.\s*)*(?:[A-Z]\w*\s*\.\s*)*/.source,i={pattern:RegExp(a+/[A-Z](?:[\d_A-Z]*[a-z]\w*)?\b/.source),lookbehind:!0,inside:{namespace:{pattern:/^[a-z]\w*(?:\s*\.\s*[a-z]\w*)*(?:\s*\.)?/,inside:{punctuation:/\./}}}};r.languages.dart=r.languages.extend("clike",{"class-name":[i,{pattern:RegExp(a+/[A-Z]\w*(?=\s+\w+\s*[;,=()])/.source),lookbehind:!0,inside:i.inside}],keyword:n,operator:/\bis!|\b(?:as|is)\b|\+\+|--|&&|\|\||<<=?|>>=?|~(?:\/=?)?|[+\-*\/%&^|=!<>]=?|\?/}),r.languages.insertBefore("dart","string",{"string-literal":{pattern:/r?(?:("""|''')[\s\S]*?\1|(["'])(?:\\.|(?!\2)[^\\\r\n])*\2(?!\2))/,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$(?:\w+|\{(?:[^{}]|\{[^{}]*\})*\})/,lookbehind:!0,inside:{punctuation:/^\$\{?|\}$/,expression:{pattern:/[\s\S]+/,inside:r.languages.dart}}},string:/[\s\S]+/}},string:void 0}),r.languages.insertBefore("dart","class-name",{metadata:{pattern:/@\w+/,alias:"function"}}),r.languages.insertBefore("dart","class-name",{generics:{pattern:/<(?:[\w\s,.&?]|<(?:[\w\s,.&?]|<(?:[\w\s,.&?]|<[\w\s,.&?]*>)*>)*>)*>/,inside:{"class-name":i,keyword:n,punctuation:/[<>(),.:]/,operator:/[?&|]/}}})})(t)}return CT}var TT,eG;function tat(){if(eG)return TT;eG=1,TT=e,e.displayName="dataweave",e.aliases=[];function e(t){(function(r){r.languages.dataweave={url:/\b[A-Za-z]+:\/\/[\w/:.?=&-]+|\burn:[\w:.?=&-]+/,property:{pattern:/(?:\b\w+#)?(?:"(?:\\.|[^\\"\r\n])*"|\b\w+)(?=\s*[:@])/,greedy:!0},string:{pattern:/(["'`])(?:\\[\s\S]|(?!\1)[^\\])*\1/,greedy:!0},"mime-type":/\b(?:application|audio|image|multipart|text|video)\/[\w+-]+/,date:{pattern:/\|[\w:+-]+\|/,greedy:!0},comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],regex:{pattern:/\/(?:[^\\\/\r\n]|\\[^\r\n])+\//,greedy:!0},keyword:/\b(?:and|as|at|case|do|else|fun|if|input|is|match|not|ns|null|or|output|type|unless|update|using|var)\b/,function:/\b[A-Z_]\w*(?=\s*\()/i,number:/-?\b\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,punctuation:/[{}[\];(),.:@]/,operator:/<<|>>|->|[<>~=]=?|!=|--?-?|\+\+?|!|\?/,boolean:/\b(?:false|true)\b/}})(t)}return TT}var AT,tG;function rat(){if(tG)return AT;tG=1,AT=e,e.displayName="dax",e.aliases=[];function e(t){t.languages.dax={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:--|\/\/).*)/,lookbehind:!0},"data-field":{pattern:/'(?:[^']|'')*'(?!')(?:\[[ \w\xA0-\uFFFF]+\])?|\w+\[[ \w\xA0-\uFFFF]+\]/,alias:"symbol"},measure:{pattern:/\[[ \w\xA0-\uFFFF]+\]/,alias:"constant"},string:{pattern:/"(?:[^"]|"")*"(?!")/,greedy:!0},function:/\b(?:ABS|ACOS|ACOSH|ACOT|ACOTH|ADDCOLUMNS|ADDMISSINGITEMS|ALL|ALLCROSSFILTERED|ALLEXCEPT|ALLNOBLANKROW|ALLSELECTED|AND|APPROXIMATEDISTINCTCOUNT|ASIN|ASINH|ATAN|ATANH|AVERAGE|AVERAGEA|AVERAGEX|BETA\.DIST|BETA\.INV|BLANK|CALCULATE|CALCULATETABLE|CALENDAR|CALENDARAUTO|CEILING|CHISQ\.DIST|CHISQ\.DIST\.RT|CHISQ\.INV|CHISQ\.INV\.RT|CLOSINGBALANCEMONTH|CLOSINGBALANCEQUARTER|CLOSINGBALANCEYEAR|COALESCE|COMBIN|COMBINA|COMBINEVALUES|CONCATENATE|CONCATENATEX|CONFIDENCE\.NORM|CONFIDENCE\.T|CONTAINS|CONTAINSROW|CONTAINSSTRING|CONTAINSSTRINGEXACT|CONVERT|COS|COSH|COT|COTH|COUNT|COUNTA|COUNTAX|COUNTBLANK|COUNTROWS|COUNTX|CROSSFILTER|CROSSJOIN|CURRENCY|CURRENTGROUP|CUSTOMDATA|DATATABLE|DATE|DATEADD|DATEDIFF|DATESBETWEEN|DATESINPERIOD|DATESMTD|DATESQTD|DATESYTD|DATEVALUE|DAY|DEGREES|DETAILROWS|DISTINCT|DISTINCTCOUNT|DISTINCTCOUNTNOBLANK|DIVIDE|EARLIER|EARLIEST|EDATE|ENDOFMONTH|ENDOFQUARTER|ENDOFYEAR|EOMONTH|ERROR|EVEN|EXACT|EXCEPT|EXP|EXPON\.DIST|FACT|FALSE|FILTER|FILTERS|FIND|FIRSTDATE|FIRSTNONBLANK|FIRSTNONBLANKVALUE|FIXED|FLOOR|FORMAT|GCD|GENERATE|GENERATEALL|GENERATESERIES|GEOMEAN|GEOMEANX|GROUPBY|HASONEFILTER|HASONEVALUE|HOUR|IF|IF\.EAGER|IFERROR|IGNORE|INT|INTERSECT|ISBLANK|ISCROSSFILTERED|ISEMPTY|ISERROR|ISEVEN|ISFILTERED|ISINSCOPE|ISLOGICAL|ISNONTEXT|ISNUMBER|ISO\.CEILING|ISODD|ISONORAFTER|ISSELECTEDMEASURE|ISSUBTOTAL|ISTEXT|KEEPFILTERS|KEYWORDMATCH|LASTDATE|LASTNONBLANK|LASTNONBLANKVALUE|LCM|LEFT|LEN|LN|LOG|LOG10|LOOKUPVALUE|LOWER|MAX|MAXA|MAXX|MEDIAN|MEDIANX|MID|MIN|MINA|MINUTE|MINX|MOD|MONTH|MROUND|NATURALINNERJOIN|NATURALLEFTOUTERJOIN|NEXTDAY|NEXTMONTH|NEXTQUARTER|NEXTYEAR|NONVISUAL|NORM\.DIST|NORM\.INV|NORM\.S\.DIST|NORM\.S\.INV|NOT|NOW|ODD|OPENINGBALANCEMONTH|OPENINGBALANCEQUARTER|OPENINGBALANCEYEAR|OR|PARALLELPERIOD|PATH|PATHCONTAINS|PATHITEM|PATHITEMREVERSE|PATHLENGTH|PERCENTILE\.EXC|PERCENTILE\.INC|PERCENTILEX\.EXC|PERCENTILEX\.INC|PERMUT|PI|POISSON\.DIST|POWER|PREVIOUSDAY|PREVIOUSMONTH|PREVIOUSQUARTER|PREVIOUSYEAR|PRODUCT|PRODUCTX|QUARTER|QUOTIENT|RADIANS|RAND|RANDBETWEEN|RANK\.EQ|RANKX|RELATED|RELATEDTABLE|REMOVEFILTERS|REPLACE|REPT|RIGHT|ROLLUP|ROLLUPADDISSUBTOTAL|ROLLUPGROUP|ROLLUPISSUBTOTAL|ROUND|ROUNDDOWN|ROUNDUP|ROW|SAMEPERIODLASTYEAR|SAMPLE|SEARCH|SECOND|SELECTCOLUMNS|SELECTEDMEASURE|SELECTEDMEASUREFORMATSTRING|SELECTEDMEASURENAME|SELECTEDVALUE|SIGN|SIN|SINH|SQRT|SQRTPI|STARTOFMONTH|STARTOFQUARTER|STARTOFYEAR|STDEV\.P|STDEV\.S|STDEVX\.P|STDEVX\.S|SUBSTITUTE|SUBSTITUTEWITHINDEX|SUM|SUMMARIZE|SUMMARIZECOLUMNS|SUMX|SWITCH|T\.DIST|T\.DIST\.2T|T\.DIST\.RT|T\.INV|T\.INV\.2T|TAN|TANH|TIME|TIMEVALUE|TODAY|TOPN|TOPNPERLEVEL|TOPNSKIP|TOTALMTD|TOTALQTD|TOTALYTD|TREATAS|TRIM|TRUE|TRUNC|UNICHAR|UNICODE|UNION|UPPER|USERELATIONSHIP|USERNAME|USEROBJECTID|USERPRINCIPALNAME|UTCNOW|UTCTODAY|VALUE|VALUES|VAR\.P|VAR\.S|VARX\.P|VARX\.S|WEEKDAY|WEEKNUM|XIRR|XNPV|YEAR|YEARFRAC)(?=\s*\()/i,keyword:/\b(?:DEFINE|EVALUATE|MEASURE|ORDER\s+BY|RETURN|VAR|START\s+AT|ASC|DESC)\b/i,boolean:{pattern:/\b(?:FALSE|NULL|TRUE)\b/i,alias:"constant"},number:/\b\d+(?:\.\d*)?|\B\.\d+\b/,operator:/:=|[-+*\/=^]|&&?|\|\||<(?:=>?|<|>)?|>[>=]?|\b(?:IN|NOT)\b/i,punctuation:/[;\[\](){}`,.]/}}return AT}var FT,rG;function nat(){if(rG)return FT;rG=1,FT=e,e.displayName="dhall",e.aliases=[];function e(t){t.languages.dhall={comment:/--.*|\{-(?:[^-{]|-(?!\})|\{(?!-)|\{-(?:[^-{]|-(?!\})|\{(?!-))*-\})*-\}/,string:{pattern:/"(?:[^"\\]|\\.)*"|''(?:[^']|'(?!')|'''|''\$\{)*''(?!'|\$)/,greedy:!0,inside:{interpolation:{pattern:/\$\{[^{}]*\}/,inside:{expression:{pattern:/(^\$\{)[\s\S]+(?=\}$)/,lookbehind:!0,alias:"language-dhall",inside:null},punctuation:/\$\{|\}/}}}},label:{pattern:/`[^`]*`/,greedy:!0},url:{pattern:/\bhttps?:\/\/[\w.:%!$&'*+;=@~-]+(?:\/[\w.:%!$&'*+;=@~-]*)*(?:\?[/?\w.:%!$&'*+;=@~-]*)?/,greedy:!0},env:{pattern:/\benv:(?:(?!\d)\w+|"(?:[^"\\=]|\\.)*")/,greedy:!0,inside:{function:/^env/,operator:/^:/,variable:/[\s\S]+/}},hash:{pattern:/\bsha256:[\da-fA-F]{64}\b/,inside:{function:/sha256/,operator:/:/,number:/[\da-fA-F]{64}/}},keyword:/\b(?:as|assert|else|forall|if|in|let|merge|missing|then|toMap|using|with)\b|\u2200/,builtin:/\b(?:None|Some)\b/,boolean:/\b(?:False|True)\b/,number:/\bNaN\b|-?\bInfinity\b|[+-]?\b(?:0x[\da-fA-F]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/,operator:/\/\\|\/\/\\\\|&&|\|\||===|[!=]=|\/\/|->|\+\+|::|[+*#@=:?<>|\\\u2227\u2a53\u2261\u2afd\u03bb\u2192]/,punctuation:/\.\.|[{}\[\](),./]/,"class-name":/\b[A-Z]\w*\b/},t.languages.dhall.string.inside.interpolation.inside.expression.inside=t.languages.dhall}return FT}var _T,nG;function aat(){if(nG)return _T;nG=1,_T=e,e.displayName="diff",e.aliases=[];function e(t){(function(r){r.languages.diff={coord:[/^(?:\*{3}|-{3}|\+{3}).*$/m,/^@@.*@@$/m,/^\d.*$/m]};var n={"deleted-sign":"-","deleted-arrow":"<","inserted-sign":"+","inserted-arrow":">",unchanged:" ",diff:"!"};Object.keys(n).forEach(function(a){var i=n[a],o=[];/^\w+$/.test(a)||o.push(/\w+/.exec(a)[0]),a==="diff"&&o.push("bold"),r.languages.diff[a]={pattern:RegExp("^(?:["+i+`].*(?:\r
?|
|(?![\\s\\S])))+`,"m"),alias:o,inside:{line:{pattern:/(.)(?=[\s\S]).*(?:\r\n?|\n)?/,lookbehind:!0},prefix:{pattern:/[\s\S]/,alias:/\w+/.exec(a)[0]}}}}),Object.defineProperty(r.languages.diff,"PREFIXES",{value:n})})(t)}return _T}var RT,aG;function Si(){if(aG)return RT;aG=1,RT=e,e.displayName="markupTemplating",e.aliases=[];function e(t){(function(r){function n(a,i){return"___"+a.toUpperCase()+i+"___"}Object.defineProperties(r.languages["markup-templating"]={},{buildPlaceholders:{value:function(a,i,o,s){if(a.language===i){var l=a.tokenStack=[];a.code=a.code.replace(o,function(u){if(typeof s=="function"&&!s(u))return u;for(var d=l.length,p;a.code.indexOf(p=n(i,d))!==-1;)++d;return l[d]=u,p}),a.grammar=r.languages.markup}}},tokenizePlaceholders:{value:function(a,i){if(a.language!==i||!a.tokenStack)return;a.grammar=r.languages[i];var o=0,s=Object.keys(a.tokenStack);function l(u){for(var d=0;d<u.length&&!(o>=s.length);d++){var p=u[d];if(typeof p=="string"||p.content&&typeof p.content=="string"){var f=s[o],m=a.tokenStack[f],b=typeof p=="string"?p:p.content,y=n(i,f),S=b.indexOf(y);if(S>-1){++o;var w=b.substring(0,S),E=new r.Token(i,r.tokenize(m,a.grammar),"language-"+i,m),k=b.substring(S+y.length),_=[];w&&_.push.apply(_,l([w])),_.push(E),k&&_.push.apply(_,l([k])),typeof p=="string"?u.splice.apply(u,[d,1].concat(_)):p.content=_}}else p.content&&l(p.content)}return u}l(a.tokens)}}})})(t)}return RT}var DT,iG;function iat(){if(iG)return DT;iG=1;var e=Si();DT=t,t.displayName="django",t.aliases=["jinja2"];function t(r){r.register(e),(function(n){n.languages.django={comment:/^\{#[\s\S]*?#\}$/,tag:{pattern:/(^\{%[+-]?\s*)\w+/,lookbehind:!0,alias:"keyword"},delimiter:{pattern:/^\{[{%][+-]?|[+-]?[}%]\}$/,alias:"punctuation"},string:{pattern:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},filter:{pattern:/(\|)\w+/,lookbehind:!0,alias:"function"},test:{pattern:/(\bis\s+(?:not\s+)?)(?!not\b)\w+/,lookbehind:!0,alias:"function"},function:/\b[a-z_]\w+(?=\s*\()/i,keyword:/\b(?:and|as|by|else|for|if|import|in|is|loop|not|or|recursive|with|without)\b/,operator:/[-+%=]=?|!=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,number:/\b\d+(?:\.\d+)?\b/,boolean:/[Ff]alse|[Nn]one|[Tt]rue/,variable:/\b\w+\b/,punctuation:/[{}[\](),.:;]/};var a=/\{\{[\s\S]*?\}\}|\{%[\s\S]*?%\}|\{#[\s\S]*?#\}/g,i=n.languages["markup-templating"];n.hooks.add("before-tokenize",function(o){i.buildPlaceholders(o,"django",a)}),n.hooks.add("after-tokenize",function(o){i.tokenizePlaceholders(o,"django")}),n.languages.jinja2=n.languages.django,n.hooks.add("before-tokenize",function(o){i.buildPlaceholders(o,"jinja2",a)}),n.hooks.add("after-tokenize",function(o){i.tokenizePlaceholders(o,"jinja2")})})(r)}return DT}var NT,oG;function oat(){if(oG)return NT;oG=1,NT=e,e.displayName="dnsZoneFile",e.aliases=[];function e(t){t.languages["dns-zone-file"]={comment:/;.*/,string:{pattern:/"(?:\\.|[^"\\\r\n])*"/,greedy:!0},variable:[{pattern:/(^\$ORIGIN[ \t]+)\S+/m,lookbehind:!0},{pattern:/(^|\s)@(?=\s|$)/,lookbehind:!0}],keyword:/^\$(?:INCLUDE|ORIGIN|TTL)(?=\s|$)/m,class:{pattern:/(^|\s)(?:CH|CS|HS|IN)(?=\s|$)/,lookbehind:!0,alias:"keyword"},type:{pattern:/(^|\s)(?:A|A6|AAAA|AFSDB|APL|ATMA|CAA|CDNSKEY|CDS|CERT|CNAME|DHCID|DLV|DNAME|DNSKEY|DS|EID|GID|GPOS|HINFO|HIP|IPSECKEY|ISDN|KEY|KX|LOC|MAILA|MAILB|MB|MD|MF|MG|MINFO|MR|MX|NAPTR|NB|NBSTAT|NIMLOC|NINFO|NS|NSAP|NSAP-PTR|NSEC|NSEC3|NSEC3PARAM|NULL|NXT|OPENPGPKEY|PTR|PX|RKEY|RP|RRSIG|RT|SIG|SINK|SMIMEA|SOA|SPF|SRV|SSHFP|TA|TKEY|TLSA|TSIG|TXT|UID|UINFO|UNSPEC|URI|WKS|X25)(?=\s|$)/,lookbehind:!0,alias:"keyword"},punctuation:/[()]/},t.languages["dns-zone"]=t.languages["dns-zone-file"]}return NT}var LT,sG;function sat(){if(sG)return LT;sG=1,LT=e,e.displayName="docker",e.aliases=["dockerfile"];function e(t){(function(r){var n=/\\[\r\n](?:\s|\\[\r\n]|#.*(?!.))*(?![\s#]|\\[\r\n])/.source,a=/(?:[ \t]+(?![ \t])(?:<SP_BS>)?|<SP_BS>)/.source.replace(/<SP_BS>/g,function(){return n}),i=/"(?:[^"\\\r\n]|\\(?:\r\n|[\s\S]))*"|'(?:[^'\\\r\n]|\\(?:\r\n|[\s\S]))*'/.source,o=/--[\w-]+=(?:<STR>|(?!["'])(?:[^\s\\]|\\.)+)/.source.replace(/<STR>/g,function(){return i}),s={pattern:RegExp(i),greedy:!0},l={pattern:/(^[ \t]*)#.*/m,lookbehind:!0,greedy:!0};function u(d,p){return d=d.replace(/<OPT>/g,function(){return o}).replace(/<SP>/g,function(){return a}),RegExp(d,p)}r.languages.docker={instruction:{pattern:/(^[ \t]*)(?:ADD|ARG|CMD|COPY|ENTRYPOINT|ENV|EXPOSE|FROM|HEALTHCHECK|LABEL|MAINTAINER|ONBUILD|RUN|SHELL|STOPSIGNAL|USER|VOLUME|WORKDIR)(?=\s)(?:\\.|[^\r\n\\])*(?:\\$(?:\s|#.*$)*(?![\s#])(?:\\.|[^\r\n\\])*)*/im,lookbehind:!0,greedy:!0,inside:{options:{pattern:u(/(^(?:ONBUILD<SP>)?\w+<SP>)<OPT>(?:<SP><OPT>)*/.source,"i"),lookbehind:!0,greedy:!0,inside:{property:{pattern:/(^|\s)--[\w-]+/,lookbehind:!0},string:[s,{pattern:/(=)(?!["'])(?:[^\s\\]|\\.)+/,lookbehind:!0}],operator:/\\$/m,punctuation:/=/}},keyword:[{pattern:u(/(^(?:ONBUILD<SP>)?HEALTHCHECK<SP>(?:<OPT><SP>)*)(?:CMD|NONE)\b/.source,"i"),lookbehind:!0,greedy:!0},{pattern:u(/(^(?:ONBUILD<SP>)?FROM<SP>(?:<OPT><SP>)*(?!--)[^ \t\\]+<SP>)AS/.source,"i"),lookbehind:!0,greedy:!0},{pattern:u(/(^ONBUILD<SP>)\w+/.source,"i"),lookbehind:!0,greedy:!0},{pattern:/^\w+/,greedy:!0}],comment:l,string:s,variable:/\$(?:\w+|\{[^{}"'\\]*\})/,operator:/\\$/m}},comment:l},r.languages.dockerfile=r.languages.docker})(t)}return LT}var MT,lG;function lat(){if(lG)return MT;lG=1,MT=e,e.displayName="dot",e.aliases=["gv"];function e(t){(function(r){var n="(?:"+[/[a-zA-Z_\x80-\uFFFF][\w\x80-\uFFFF]*/.source,/-?(?:\.\d+|\d+(?:\.\d*)?)/.source,/"[^"\\]*(?:\\[\s\S][^"\\]*)*"/.source,/<(?:[^<>]|(?!<!--)<(?:[^<>"']|"[^"]*"|'[^']*')+>|<!--(?:[^-]|-(?!->))*-->)*>/.source].join("|")+")",a={markup:{pattern:/(^<)[\s\S]+(?=>$)/,lookbehind:!0,alias:["language-markup","language-html","language-xml"],inside:r.languages.markup}};function i(o,s){return RegExp(o.replace(/<ID>/g,function(){return n}),s)}r.languages.dot={comment:{pattern:/\/\/.*|\/\*[\s\S]*?\*\/|^#.*/m,greedy:!0},"graph-name":{pattern:i(/(\b(?:digraph|graph|subgraph)[ \t\r\n]+)<ID>/.source,"i"),lookbehind:!0,greedy:!0,alias:"class-name",inside:a},"attr-value":{pattern:i(/(=[ \t\r\n]*)<ID>/.source),lookbehind:!0,greedy:!0,inside:a},"attr-name":{pattern:i(/([\[;, \t\r\n])<ID>(?=[ \t\r\n]*=)/.source),lookbehind:!0,greedy:!0,inside:a},keyword:/\b(?:digraph|edge|graph|node|strict|subgraph)\b/i,"compass-point":{pattern:/(:[ \t\r\n]*)(?:[ewc_]|[ns][ew]?)(?![\w\x80-\uFFFF])/,lookbehind:!0,alias:"builtin"},node:{pattern:i(/(^|[^-.\w\x80-\uFFFF\\])<ID>/.source),lookbehind:!0,greedy:!0,inside:a},operator:/[=:]|-[->]/,punctuation:/[\[\]{};,]/},r.languages.gv=r.languages.dot})(t)}return MT}var IT,uG;function uat(){if(uG)return IT;uG=1,IT=e,e.displayName="ebnf",e.aliases=[];function e(t){t.languages.ebnf={comment:/\(\*[\s\S]*?\*\)/,string:{pattern:/"[^"\r\n]*"|'[^'\r\n]*'/,greedy:!0},special:{pattern:/\?[^?\r\n]*\?/,greedy:!0,alias:"class-name"},definition:{pattern:/^([\t ]*)[a-z]\w*(?:[ \t]+[a-z]\w*)*(?=\s*=)/im,lookbehind:!0,alias:["rule","keyword"]},rule:/\b[a-z]\w*(?:[ \t]+[a-z]\w*)*\b/i,punctuation:/\([:/]|[:/]\)|[.,;()[\]{}]/,operator:/[-=|*/!]/}}return IT}var OT,cG;function cat(){if(cG)return OT;cG=1,OT=e,e.displayName="editorconfig",e.aliases=[];function e(t){t.languages.editorconfig={comment:/[;#].*/,section:{pattern:/(^[ \t]*)\[.+\]/m,lookbehind:!0,alias:"selector",inside:{regex:/\\\\[\[\]{},!?.*]/,operator:/[!?]|\.\.|\*{1,2}/,punctuation:/[\[\]{},]/}},key:{pattern:/(^[ \t]*)[^\s=]+(?=[ \t]*=)/m,lookbehind:!0,alias:"attr-name"},value:{pattern:/=.*/,alias:"attr-value",inside:{punctuation:/^=/}}}}return OT}var BT,dG;function dat(){if(dG)return BT;dG=1,BT=e,e.displayName="eiffel",e.aliases=[];function e(t){t.languages.eiffel={comment:/--.*/,string:[{pattern:/"([^[]*)\[[\s\S]*?\]\1"/,greedy:!0},{pattern:/"([^{]*)\{[\s\S]*?\}\1"/,greedy:!0},{pattern:/"(?:%(?:(?!\n)\s)*\n\s*%|%\S|[^%"\r\n])*"/,greedy:!0}],char:/'(?:%.|[^%'\r\n])+'/,keyword:/\b(?:across|agent|alias|all|and|as|assign|attached|attribute|check|class|convert|create|Current|debug|deferred|detachable|do|else|elseif|end|ensure|expanded|export|external|feature|from|frozen|if|implies|inherit|inspect|invariant|like|local|loop|not|note|obsolete|old|once|or|Precursor|redefine|rename|require|rescue|Result|retry|select|separate|some|then|undefine|until|variant|Void|when|xor)\b/i,boolean:/\b(?:False|True)\b/i,"class-name":/\b[A-Z][\dA-Z_]*\b/,number:[/\b0[xcb][\da-f](?:_*[\da-f])*\b/i,/(?:\b\d(?:_*\d)*)?\.(?:(?:\d(?:_*\d)*)?e[+-]?)?\d(?:_*\d)*\b|\b\d(?:_*\d)*\b\.?/i],punctuation:/:=|<<|>>|\(\||\|\)|->|\.(?=\w)|[{}[\];(),:?]/,operator:/\\\\|\|\.\.\||\.\.|\/[~\/=]?|[><]=?|[-+*^=~]/}}return BT}var PT,hG;function hat(){if(hG)return PT;hG=1;var e=Si();PT=t,t.displayName="ejs",t.aliases=["eta"];function t(r){r.register(e),(function(n){n.languages.ejs={delimiter:{pattern:/^<%[-_=]?|[-_]?%>$/,alias:"punctuation"},comment:/^#[\s\S]*/,"language-javascript":{pattern:/[\s\S]+/,inside:n.languages.javascript}},n.hooks.add("before-tokenize",function(a){var i=/<%(?!%)[\s\S]+?%>/g;n.languages["markup-templating"].buildPlaceholders(a,"ejs",i)}),n.hooks.add("after-tokenize",function(a){n.languages["markup-templating"].tokenizePlaceholders(a,"ejs")}),n.languages.eta=n.languages.ejs})(r)}return PT}var zT,pG;function pat(){if(pG)return zT;pG=1,zT=e,e.displayName="elixir",e.aliases=[];function e(t){t.languages.elixir={doc:{pattern:/@(?:doc|moduledoc)\s+(?:("""|''')[\s\S]*?\1|("|')(?:\\(?:\r\n|[\s\S])|(?!\2)[^\\\r\n])*\2)/,inside:{attribute:/^@\w+/,string:/['"][\s\S]+/}},comment:{pattern:/#.*/,greedy:!0},regex:{pattern:/~[rR](?:("""|''')(?:\\[\s\S]|(?!\1)[^\\])+\1|([\/|"'])(?:\\.|(?!\2)[^\\\r\n])+\2|\((?:\\.|[^\\)\r\n])+\)|\[(?:\\.|[^\\\]\r\n])+\]|\{(?:\\.|[^\\}\r\n])+\}|<(?:\\.|[^\\>\r\n])+>)[uismxfr]*/,greedy:!0},string:[{pattern:/~[cCsSwW](?:("""|''')(?:\\[\s\S]|(?!\1)[^\\])+\1|([\/|"'])(?:\\.|(?!\2)[^\\\r\n])+\2|\((?:\\.|[^\\)\r\n])+\)|\[(?:\\.|[^\\\]\r\n])+\]|\{(?:\\.|#\{[^}]+\}|#(?!\{)|[^#\\}\r\n])+\}|<(?:\\.|[^\\>\r\n])+>)[csa]?/,greedy:!0,inside:{}},{pattern:/("""|''')[\s\S]*?\1/,greedy:!0,inside:{}},{pattern:/("|')(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0,inside:{}}],atom:{pattern:/(^|[^:]):\w+/,lookbehind:!0,alias:"symbol"},module:{pattern:/\b[A-Z]\w*\b/,alias:"class-name"},"attr-name":/\b\w+\??:(?!:)/,argument:{pattern:/(^|[^&])&\d+/,lookbehind:!0,alias:"variable"},attribute:{pattern:/@\w+/,alias:"variable"},function:/\b[_a-zA-Z]\w*[?!]?(?:(?=\s*(?:\.\s*)?\()|(?=\/\d))/,number:/\b(?:0[box][a-f\d_]+|\d[\d_]*)(?:\.[\d_]+)?(?:e[+-]?[\d_]+)?\b/i,keyword:/\b(?:after|alias|and|case|catch|cond|def(?:callback|delegate|exception|impl|macro|module|n|np|p|protocol|struct)?|do|else|end|fn|for|if|import|not|or|quote|raise|require|rescue|try|unless|unquote|use|when)\b/,boolean:/\b(?:false|nil|true)\b/,operator:[/\bin\b|&&?|\|[|>]?|\\\\|::|\.\.\.?|\+\+?|-[->]?|<[-=>]|>=|!==?|\B!|=(?:==?|[>~])?|[*\/^]/,{pattern:/([^<])<(?!<)/,lookbehind:!0},{pattern:/([^>])>(?!>)/,lookbehind:!0}],punctuation:/<<|>>|[.,%\[\]{}()]/},t.languages.elixir.string.forEach(function(r){r.inside={interpolation:{pattern:/#\{[^}]+\}/,inside:{delimiter:{pattern:/^#\{|\}$/,alias:"punctuation"},rest:t.languages.elixir}}}})}return zT}var HT,fG;function fat(){if(fG)return HT;fG=1,HT=e,e.displayName="elm",e.aliases=[];function e(t){t.languages.elm={comment:/--.*|\{-[\s\S]*?-\}/,char:{pattern:/'(?:[^\\'\r\n]|\\(?:[abfnrtv\\']|\d+|x[0-9a-fA-F]+|u\{[0-9a-fA-F]+\}))'/,greedy:!0},string:[{pattern:/"""[\s\S]*?"""/,greedy:!0},{pattern:/"(?:[^\\"\r\n]|\\.)*"/,greedy:!0}],"import-statement":{pattern:/(^[\t ]*)import\s+[A-Z]\w*(?:\.[A-Z]\w*)*(?:\s+as\s+(?:[A-Z]\w*)(?:\.[A-Z]\w*)*)?(?:\s+exposing\s+)?/m,lookbehind:!0,inside:{keyword:/\b(?:as|exposing|import)\b/}},keyword:/\b(?:alias|as|case|else|exposing|if|in|infixl|infixr|let|module|of|then|type)\b/,builtin:/\b(?:abs|acos|always|asin|atan|atan2|ceiling|clamp|compare|cos|curry|degrees|e|flip|floor|fromPolar|identity|isInfinite|isNaN|logBase|max|min|negate|never|not|pi|radians|rem|round|sin|sqrt|tan|toFloat|toPolar|toString|truncate|turns|uncurry|xor)\b/,number:/\b(?:\d+(?:\.\d+)?(?:e[+-]?\d+)?|0x[0-9a-f]+)\b/i,operator:/\s\.\s|[+\-/*=.$<>:&|^?%#@~!]{2,}|[+\-/*=$<>:&|^?%#@~!]/,hvariable:/\b(?:[A-Z]\w*\.)*[a-z]\w*\b/,constant:/\b(?:[A-Z]\w*\.)*[A-Z]\w*\b/,punctuation:/[{}[\]|(),.:]/}}return HT}var UT,mG;function mat(){if(mG)return UT;mG=1;var e=nS(),t=Si();UT=r,r.displayName="erb",r.aliases=[];function r(n){n.register(e),n.register(t),(function(a){a.languages.erb={delimiter:{pattern:/^(\s*)<%=?|%>(?=\s*$)/,lookbehind:!0,alias:"punctuation"},ruby:{pattern:/\s*\S[\s\S]*/,alias:"language-ruby",inside:a.languages.ruby}},a.hooks.add("before-tokenize",function(i){var o=/<%=?(?:[^\r\n]|[\r\n](?!=begin)|[\r\n]=begin\s(?:[^\r\n]|[\r\n](?!=end))*[\r\n]=end)+?%>/g;a.languages["markup-templating"].buildPlaceholders(i,"erb",o)}),a.hooks.add("after-tokenize",function(i){a.languages["markup-templating"].tokenizePlaceholders(i,"erb")})})(n)}return UT}var $T,gG;function gat(){if(gG)return $T;gG=1,$T=e,e.displayName="erlang",e.aliases=[];function e(t){t.languages.erlang={comment:/%.+/,string:{pattern:/"(?:\\.|[^\\"\r\n])*"/,greedy:!0},"quoted-function":{pattern:/'(?:\\.|[^\\'\r\n])+'(?=\()/,alias:"function"},"quoted-atom":{pattern:/'(?:\\.|[^\\'\r\n])+'/,alias:"atom"},boolean:/\b(?:false|true)\b/,keyword:/\b(?:after|case|catch|end|fun|if|of|receive|try|when)\b/,number:[/\$\\?./,/\b\d+#[a-z0-9]+/i,/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i],function:/\b[a-z][\w@]*(?=\()/,variable:{pattern:/(^|[^@])(?:\b|\?)[A-Z_][\w@]*/,lookbehind:!0},operator:[/[=\/<>:]=|=[:\/]=|\+\+?|--?|[=*\/!]|\b(?:and|andalso|band|bnot|bor|bsl|bsr|bxor|div|not|or|orelse|rem|xor)\b/,{pattern:/(^|[^<])<(?!<)/,lookbehind:!0},{pattern:/(^|[^>])>(?!>)/,lookbehind:!0}],atom:/\b[a-z][\w@]*/,punctuation:/[()[\]{}:;,.#|]|<<|>>/}}return $T}var GT,bG;function Oce(){if(bG)return GT;bG=1,GT=e,e.displayName="lua",e.aliases=[];function e(t){t.languages.lua={comment:/^#!.+|--(?:\[(=*)\[[\s\S]*?\]\1\]|.*)/m,string:{pattern:/(["'])(?:(?!\1)[^\\\r\n]|\\z(?:\r\n|\s)|\\(?:\r\n|[^z]))*\1|\[(=*)\[[\s\S]*?\]\2\]/,greedy:!0},number:/\b0x[a-f\d]+(?:\.[a-f\d]*)?(?:p[+-]?\d+)?\b|\b\d+(?:\.\B|(?:\.\d*)?(?:e[+-]?\d+)?\b)|\B\.\d+(?:e[+-]?\d+)?\b/i,keyword:/\b(?:and|break|do|else|elseif|end|false|for|function|goto|if|in|local|nil|not|or|repeat|return|then|true|until|while)\b/,function:/(?!\d)\w+(?=\s*(?:[({]))/,operator:[/[-+*%^&|#]|\/\/?|<[<=]?|>[>=]?|[=~]=?/,{pattern:/(^|[^.])\.\.(?!\.)/,lookbehind:!0}],punctuation:/[\[\](){},;]|\.+|:+/}}return GT}var jT,yG;function bat(){if(yG)return jT;yG=1;var e=Oce(),t=Si();jT=r,r.displayName="etlua",r.aliases=[];function r(n){n.register(e),n.register(t),(function(a){a.languages.etlua={delimiter:{pattern:/^<%[-=]?|-?%>$/,alias:"punctuation"},"language-lua":{pattern:/[\s\S]+/,inside:a.languages.lua}},a.hooks.add("before-tokenize",function(i){var o=/<%[\s\S]+?%>/g;a.languages["markup-templating"].buildPlaceholders(i,"etlua",o)}),a.hooks.add("after-tokenize",function(i){a.languages["markup-templating"].tokenizePlaceholders(i,"etlua")})})(n)}return jT}var qT,vG;function yat(){if(vG)return qT;vG=1,qT=e,e.displayName="excelFormula",e.aliases=[];function e(t){t.languages["excel-formula"]={comment:{pattern:/(\bN\(\s*)"(?:[^"]|"")*"(?=\s*\))/i,lookbehind:!0,greedy:!0},string:{pattern:/"(?:[^"]|"")*"(?!")/,greedy:!0},reference:{pattern:/(?:'[^']*'|(?:[^\s()[\]{}<>*?"';,$&]*\[[^^\s()[\]{}<>*?"']+\])?\w+)!/,greedy:!0,alias:"string",inside:{operator:/!$/,punctuation:/'/,sheet:{pattern:/[^[\]]+$/,alias:"function"},file:{pattern:/\[[^[\]]+\]$/,inside:{punctuation:/[[\]]/}},path:/[\s\S]+/}},"function-name":{pattern:/\b[A-Z]\w*(?=\()/i,alias:"keyword"},range:{pattern:/\$?\b(?:[A-Z]+\$?\d+:\$?[A-Z]+\$?\d+|[A-Z]+:\$?[A-Z]+|\d+:\$?\d+)\b/i,alias:"property",inside:{operator:/:/,cell:/\$?[A-Z]+\$?\d+/i,column:/\$?[A-Z]+/i,row:/\$?\d+/}},cell:{pattern:/\b[A-Z]+\d+\b|\$[A-Za-z]+\$?\d+\b|\b[A-Za-z]+\$\d+\b/,alias:"property"},number:/(?:\b\d+(?:\.\d+)?|\B\.\d+)(?:e[+-]?\d+)?\b/i,boolean:/\b(?:FALSE|TRUE)\b/i,operator:/[-+*/^%=&,]|<[=>]?|>=?/,punctuation:/[[\]();{}|]/},t.languages.xlsx=t.languages.xls=t.languages["excel-formula"]}return qT}var WT,SG;function vat(){if(SG)return WT;SG=1,WT=e,e.displayName="factor",e.aliases=[];function e(t){(function(r){var n={function:/\b(?:BUGS?|FIX(?:MES?)?|NOTES?|TODOS?|XX+|HACKS?|WARN(?:ING)?|\?{2,}|!{2,})\b/},a={number:/\\[^\s']|%\w/},i={comment:[{pattern:/(^|\s)(?:! .*|!$)/,lookbehind:!0,inside:n},{pattern:/(^|\s)\/\*\s[\s\S]*?\*\/(?=\s|$)/,lookbehind:!0,greedy:!0,inside:n},{pattern:/(^|\s)!\[(={0,6})\[\s[\s\S]*?\]\2\](?=\s|$)/,lookbehind:!0,greedy:!0,inside:n}],number:[{pattern:/(^|\s)[+-]?\d+(?=\s|$)/,lookbehind:!0},{pattern:/(^|\s)[+-]?0(?:b[01]+|o[0-7]+|d\d+|x[\dA-F]+)(?=\s|$)/i,lookbehind:!0},{pattern:/(^|\s)[+-]?\d+\/\d+\.?(?=\s|$)/,lookbehind:!0},{pattern:/(^|\s)\+?\d+\+\d+\/\d+(?=\s|$)/,lookbehind:!0},{pattern:/(^|\s)-\d+-\d+\/\d+(?=\s|$)/,lookbehind:!0},{pattern:/(^|\s)[+-]?(?:\d*\.\d+|\d+\.\d*|\d+)(?:e[+-]?\d+)?(?=\s|$)/i,lookbehind:!0},{pattern:/(^|\s)NAN:\s+[\da-fA-F]+(?=\s|$)/,lookbehind:!0},{pattern:/(^|\s)[+-]?0(?:b1\.[01]*|o1\.[0-7]*|d1\.\d*|x1\.[\dA-F]*)p\d+(?=\s|$)/i,lookbehind:!0}],regexp:{pattern:/(^|\s)R\/\s(?:\\\S|[^\\/])*\/(?:[idmsr]*|[idmsr]+-[idmsr]+)(?=\s|$)/,lookbehind:!0,alias:"number",inside:{variable:/\\\S/,keyword:/[+?*\[\]^$(){}.|]/,operator:{pattern:/(\/)[idmsr]+(?:-[idmsr]+)?/,lookbehind:!0}}},boolean:{pattern:/(^|\s)[tf](?=\s|$)/,lookbehind:!0},"custom-string":{pattern:/(^|\s)[A-Z0-9\-]+"\s(?:\\\S|[^"\\])*"/,lookbehind:!0,greedy:!0,alias:"string",inside:{number:/\\\S|%\w|\//}},"multiline-string":[{pattern:/(^|\s)STRING:\s+\S+(?:\n|\r\n).*(?:\n|\r\n)\s*;(?=\s|$)/,lookbehind:!0,greedy:!0,alias:"string",inside:{number:a.number,"semicolon-or-setlocal":{pattern:/([\r\n][ \t]*);(?=\s|$)/,lookbehind:!0,alias:"function"}}},{pattern:/(^|\s)HEREDOC:\s+\S+(?:\n|\r\n).*(?:\n|\r\n)\s*\S+(?=\s|$)/,lookbehind:!0,greedy:!0,alias:"string",inside:a},{pattern:/(^|\s)\[(={0,6})\[\s[\s\S]*?\]\2\](?=\s|$)/,lookbehind:!0,greedy:!0,alias:"string",inside:a}],"special-using":{pattern:/(^|\s)USING:(?:\s\S+)*(?=\s+;(?:\s|$))/,lookbehind:!0,alias:"function",inside:{string:{pattern:/(\s)[^:\s]+/,lookbehind:!0}}},"stack-effect-delimiter":[{pattern:/(^|\s)(?:call|eval|execute)?\((?=\s)/,lookbehind:!0,alias:"operator"},{pattern:/(\s)--(?=\s)/,lookbehind:!0,alias:"operator"},{pattern:/(\s)\)(?=\s|$)/,lookbehind:!0,alias:"operator"}],combinators:{pattern:null,lookbehind:!0,alias:"keyword"},"kernel-builtin":{pattern:null,lookbehind:!0,alias:"variable"},"sequences-builtin":{pattern:null,lookbehind:!0,alias:"variable"},"math-builtin":{pattern:null,lookbehind:!0,alias:"variable"},"constructor-word":{pattern:/(^|\s)<(?!=+>|-+>)\S+>(?=\s|$)/,lookbehind:!0,alias:"keyword"},"other-builtin-syntax":{pattern:null,lookbehind:!0,alias:"operator"},"conventionally-named-word":{pattern:/(^|\s)(?!")(?:(?:change|new|set|with)-\S+|\$\S+|>[^>\s]+|[^:>\s]+>|[^>\s]+>[^>\s]+|\+[^+\s]+\+|[^?\s]+\?|\?[^?\s]+|[^>\s]+>>|>>[^>\s]+|[^<\s]+<<|\([^()\s]+\)|[^!\s]+!|[^*\s]\S*\*|[^.\s]\S*\.)(?=\s|$)/,lookbehind:!0,alias:"keyword"},"colon-syntax":{pattern:/(^|\s)(?:[A-Z0-9\-]+#?)?:{1,2}\s+(?:;\S+|(?!;)\S+)(?=\s|$)/,lookbehind:!0,greedy:!0,alias:"function"},"semicolon-or-setlocal":{pattern:/(\s)(?:;|:>)(?=\s|$)/,lookbehind:!0,alias:"function"},"curly-brace-literal-delimiter":[{pattern:/(^|\s)[a-z]*\{(?=\s)/i,lookbehind:!0,alias:"operator"},{pattern:/(\s)\}(?=\s|$)/,lookbehind:!0,alias:"operator"}],"quotation-delimiter":[{pattern:/(^|\s)\[(?=\s)/,lookbehind:!0,alias:"operator"},{pattern:/(\s)\](?=\s|$)/,lookbehind:!0,alias:"operator"}],"normal-word":{pattern:/(^|\s)[^"\s]\S*(?=\s|$)/,lookbehind:!0},string:{pattern:/"(?:\\\S|[^"\\])*"/,greedy:!0,inside:a}},o=function(d){return(d+"").replace(/([.?*+\^$\[\]\\(){}|\-])/g,"\\$1")},s=function(d){return new RegExp("(^|\\s)(?:"+d.map(o).join("|")+")(?=\\s|$)")},l={"kernel-builtin":["or","2nipd","4drop","tuck","wrapper","nip","wrapper?","callstack>array","die","dupd","callstack","callstack?","3dup","hashcode","pick","4nip","build",">boolean","nipd","clone","5nip","eq?","?","=","swapd","2over","clear","2dup","get-retainstack","not","tuple?","dup","3nipd","call","-rotd","object","drop","assert=","assert?","-rot","execute","boa","get-callstack","curried?","3drop","pickd","overd","over","roll","3nip","swap","and","2nip","rotd","throw","(clone)","hashcode*","spin","reach","4dup","equal?","get-datastack","assert","2drop","<wrapper>","boolean?","identity-hashcode","identity-tuple?","null","composed?","new","5drop","rot","-roll","xor","identity-tuple","boolean"],"other-builtin-syntax":["=======","recursive","flushable",">>","<<<<<<","M\\","B","PRIVATE>","\\","======","final","inline","delimiter","deprecated","<PRIVATE",">>>>>>","<<<<<<<","parse-complex","malformed-complex","read-only",">>>>>>>","call-next-method","<<","foldable","$","$[","${"],"sequences-builtin":["member-eq?","mismatch","append","assert-sequence=","longer","repetition","clone-like","3sequence","assert-sequence?","last-index-from","reversed","index-from","cut*","pad-tail","join-as","remove-eq!","concat-as","but-last","snip","nths","nth","sequence","longest","slice?","<slice>","remove-nth","tail-slice","empty?","tail*","member?","virtual-sequence?","set-length","drop-prefix","iota","unclip","bounds-error?","unclip-last-slice","non-negative-integer-expected","non-negative-integer-expected?","midpoint@","longer?","?set-nth","?first","rest-slice","prepend-as","prepend","fourth","sift","subseq-start","new-sequence","?last","like","first4","1sequence","reverse","slice","virtual@","repetition?","set-last","index","4sequence","max-length","set-second","immutable-sequence","first2","first3","supremum","unclip-slice","suffix!","insert-nth","tail","3append","short","suffix","concat","flip","immutable?","reverse!","2sequence","sum","delete-all","indices","snip-slice","<iota>","check-slice","sequence?","head","append-as","halves","sequence=","collapse-slice","?second","slice-error?","product","bounds-check?","bounds-check","immutable","virtual-exemplar","harvest","remove","pad-head","last","set-fourth","cartesian-product","remove-eq","shorten","shorter","reversed?","shorter?","shortest","head-slice","pop*","tail-slice*","but-last-slice","iota?","append!","cut-slice","new-resizable","head-slice*","sequence-hashcode","pop","set-nth","?nth","second","join","immutable-sequence?","<reversed>","3append-as","virtual-sequence","subseq?","remove-nth!","length","last-index","lengthen","assert-sequence","copy","move","third","first","tail?","set-first","prefix","bounds-error","<repetition>","exchange","surround","cut","min-length","set-third","push-all","head?","subseq-start-from","delete-slice","rest","sum-lengths","head*","infimum","remove!","glue","slice-error","subseq","push","replace-slice","subseq-as","unclip-last"],"math-builtin":["number=","next-power-of-2","?1+","fp-special?","imaginary-part","float>bits","number?","fp-infinity?","bignum?","fp-snan?","denominator","gcd","*","+","fp-bitwise=","-","u>=","/",">=","bitand","power-of-2?","log2-expects-positive","neg?","<","log2",">","integer?","number","bits>double","2/","zero?","bits>float","float?","shift","ratio?","rect>","even?","ratio","fp-sign","bitnot",">fixnum","complex?","/i","integer>fixnum","/f","sgn",">bignum","next-float","u<","u>","mod","recip","rational",">float","2^","integer","fixnum?","neg","fixnum","sq","bignum",">rect","bit?","fp-qnan?","simple-gcd","complex","<fp-nan>","real",">fraction","double>bits","bitor","rem","fp-nan-payload","real-part","log2-expects-positive?","prev-float","align","unordered?","float","fp-nan?","abs","bitxor","integer>fixnum-strict","u<=","odd?","<=","/mod",">integer","real?","rational?","numerator"]};Object.keys(l).forEach(function(d){i[d].pattern=s(l[d])});var u=["2bi","while","2tri","bi*","4dip","both?","same?","tri@","curry","prepose","3bi","?if","tri*","2keep","3keep","curried","2keepd","when","2bi*","2tri*","4keep","bi@","keepdd","do","unless*","tri-curry","if*","loop","bi-curry*","when*","2bi@","2tri@","with","2with","either?","bi","until","3dip","3curry","tri-curry*","tri-curry@","bi-curry","keepd","compose","2dip","if","3tri","unless","tuple","keep","2curry","tri","most","while*","dip","composed","bi-curry@","find-last-from","trim-head-slice","map-as","each-from","none?","trim-tail","partition","if-empty","accumulate*","reject!","find-from","accumulate-as","collector-for-as","reject","map","map-sum","accumulate!","2each-from","follow","supremum-by","map!","unless-empty","collector","padding","reduce-index","replicate-as","infimum-by","trim-tail-slice","count","find-index","filter","accumulate*!","reject-as","map-integers","map-find","reduce","selector","interleave","2map","filter-as","binary-reduce","map-index-as","find","produce","filter!","replicate","cartesian-map","cartesian-each","find-index-from","map-find-last","3map-as","3map","find-last","selector-as","2map-as","2map-reduce","accumulate","each","each-index","accumulate*-as","when-empty","all?","collector-as","push-either","new-like","collector-for","2selector","push-if","2all?","map-reduce","3each","any?","trim-slice","2reduce","change-nth","produce-as","2each","trim","trim-head","cartesian-find","map-index","if-zero","each-integer","unless-zero","(find-integer)","when-zero","find-last-integer","(all-integers?)","times","(each-integer)","find-integer","all-integers?","unless-negative","if-positive","when-positive","when-negative","unless-positive","if-negative","case","2cleave","cond>quot","case>quot","3cleave","wrong-values","to-fixed-point","alist>quot","cond","cleave","call-effect","recursive-hashcode","spread","deep-spread>quot","2||","0||","n||","0&&","2&&","3||","1||","1&&","n&&","3&&","smart-unless*","keep-inputs","reduce-outputs","smart-when*","cleave>array","smart-with","smart-apply","smart-if","inputs/outputs","output>sequence-n","map-outputs","map-reduce-outputs","dropping","output>array","smart-map-reduce","smart-2map-reduce","output>array-n","nullary","input<sequence","append-outputs","drop-inputs","inputs","smart-2reduce","drop-outputs","smart-reduce","preserving","smart-when","outputs","append-outputs-as","smart-unless","smart-if*","sum-outputs","input<sequence-unsafe","output>sequence"];i.combinators.pattern=s(u),r.languages.factor=i})(t)}return WT}var VT,EG;function Sat(){if(EG)return VT;EG=1,VT=e,e.displayName="$false",e.aliases=[];function e(t){(function(r){r.languages.false={comment:{pattern:/\{[^}]*\}/},string:{pattern:/"[^"]*"/,greedy:!0},"character-code":{pattern:/'(?:[^\r]|\r\n?)/,alias:"number"},"assembler-code":{pattern:/\d+`/,alias:"important"},number:/\d+/,operator:/[-!#$%&'*+,./:;=>?@\\^_`|~ßø]/,punctuation:/\[|\]/,variable:/[a-z]/,"non-standard":{pattern:/[()<BDO®]/,alias:"bold"}}})(t)}return VT}var KT,xG;function Eat(){if(xG)return KT;xG=1,KT=e,e.displayName="firestoreSecurityRules",e.aliases=[];function e(t){t.languages["firestore-security-rules"]=t.languages.extend("clike",{comment:/\/\/.*/,keyword:/\b(?:allow|function|if|match|null|return|rules_version|service)\b/,operator:/&&|\|\||[<>!=]=?|[-+*/%]|\b(?:in|is)\b/}),delete t.languages["firestore-security-rules"]["class-name"],t.languages.insertBefore("firestore-security-rules","keyword",{path:{pattern:/(^|[\s(),])(?:\/(?:[\w\xA0-\uFFFF]+|\{[\w\xA0-\uFFFF]+(?:=\*\*)?\}|\$\([\w\xA0-\uFFFF.]+\)))+/,lookbehind:!0,greedy:!0,inside:{variable:{pattern:/\{[\w\xA0-\uFFFF]+(?:=\*\*)?\}|\$\([\w\xA0-\uFFFF.]+\)/,inside:{operator:/=/,keyword:/\*\*/,punctuation:/[.$(){}]/}},punctuation:/\//}},method:{pattern:/(\ballow\s+)[a-z]+(?:\s*,\s*[a-z]+)*(?=\s*[:;])/,lookbehind:!0,alias:"builtin",inside:{punctuation:/,/}}})}return KT}var YT,wG;function xat(){if(wG)return YT;wG=1,YT=e,e.displayName="flow",e.aliases=[];function e(t){(function(r){r.languages.flow=r.languages.extend("javascript",{}),r.languages.insertBefore("flow","keyword",{type:[{pattern:/\b(?:[Bb]oolean|Function|[Nn]umber|[Ss]tring|any|mixed|null|void)\b/,alias:"tag"}]}),r.languages.flow["function-variable"].pattern=/(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*=\s*(?:function\b|(?:\([^()]*\)(?:\s*:\s*\w+)?|(?!\s)[_$a-z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)\s*=>))/i,delete r.languages.flow.parameter,r.languages.insertBefore("flow","operator",{"flow-punctuation":{pattern:/\{\||\|\}/,alias:"punctuation"}}),Array.isArray(r.languages.flow.keyword)||(r.languages.flow.keyword=[r.languages.flow.keyword]),r.languages.flow.keyword.unshift({pattern:/(^|[^$]\b)(?:Class|declare|opaque|type)\b(?!\$)/,lookbehind:!0},{pattern:/(^|[^$]\B)\$(?:Diff|Enum|Exact|Keys|ObjMap|PropertyType|Record|Shape|Subtype|Supertype|await)\b(?!\$)/,lookbehind:!0})})(t)}return YT}var JT,kG;function wat(){if(kG)return JT;kG=1,JT=e,e.displayName="fortran",e.aliases=[];function e(t){t.languages.fortran={"quoted-number":{pattern:/[BOZ](['"])[A-F0-9]+\1/i,alias:"number"},string:{pattern:/(?:\b\w+_)?(['"])(?:\1\1|&(?:\r\n?|\n)(?:[ \t]*!.*(?:\r\n?|\n)|(?![ \t]*!))|(?!\1).)*(?:\1|&)/,inside:{comment:{pattern:/(&(?:\r\n?|\n)\s*)!.*/,lookbehind:!0}}},comment:{pattern:/!.*/,greedy:!0},boolean:/\.(?:FALSE|TRUE)\.(?:_\w+)?/i,number:/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[ED][+-]?\d+)?(?:_\w+)?/i,keyword:[/\b(?:CHARACTER|COMPLEX|DOUBLE ?PRECISION|INTEGER|LOGICAL|REAL)\b/i,/\b(?:END ?)?(?:BLOCK ?DATA|DO|FILE|FORALL|FUNCTION|IF|INTERFACE|MODULE(?! PROCEDURE)|PROGRAM|SELECT|SUBROUTINE|TYPE|WHERE)\b/i,/\b(?:ALLOCATABLE|ALLOCATE|BACKSPACE|CALL|CASE|CLOSE|COMMON|CONTAINS|CONTINUE|CYCLE|DATA|DEALLOCATE|DIMENSION|DO|END|EQUIVALENCE|EXIT|EXTERNAL|FORMAT|GO ?TO|IMPLICIT(?: NONE)?|INQUIRE|INTENT|INTRINSIC|MODULE PROCEDURE|NAMELIST|NULLIFY|OPEN|OPTIONAL|PARAMETER|POINTER|PRINT|PRIVATE|PUBLIC|READ|RETURN|REWIND|SAVE|SELECT|STOP|TARGET|WHILE|WRITE)\b/i,/\b(?:ASSIGNMENT|DEFAULT|ELEMENTAL|ELSE|ELSEIF|ELSEWHERE|ENTRY|IN|INCLUDE|INOUT|KIND|NULL|ONLY|OPERATOR|OUT|PURE|RECURSIVE|RESULT|SEQUENCE|STAT|THEN|USE)\b/i],operator:[/\*\*|\/\/|=>|[=\/]=|[<>]=?|::|[+\-*=%]|\.[A-Z]+\./i,{pattern:/(^|(?!\().)\/(?!\))/,lookbehind:!0}],punctuation:/\(\/|\/\)|[(),;:&]/}}return JT}var XT,CG;function kat(){if(CG)return XT;CG=1,XT=e,e.displayName="fsharp",e.aliases=[];function e(t){t.languages.fsharp=t.languages.extend("clike",{comment:[{pattern:/(^|[^\\])\(\*(?!\))[\s\S]*?\*\)/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],string:{pattern:/(?:"""[\s\S]*?"""|@"(?:""|[^"])*"|"(?:\\[\s\S]|[^\\"])*")B?/,greedy:!0},"class-name":{pattern:/(\b(?:exception|inherit|interface|new|of|type)\s+|\w\s*:\s*|\s:\??>\s*)[.\w]+\b(?:\s*(?:->|\*)\s*[.\w]+\b)*(?!\s*[:.])/,lookbehind:!0,inside:{operator:/->|\*/,punctuation:/\./}},keyword:/\b(?:let|return|use|yield)(?:!\B|\b)|\b(?:abstract|and|as|asr|assert|atomic|base|begin|break|checked|class|component|const|constraint|constructor|continue|default|delegate|do|done|downcast|downto|eager|elif|else|end|event|exception|extern|external|false|finally|fixed|for|fun|function|functor|global|if|in|include|inherit|inline|interface|internal|land|lazy|lor|lsl|lsr|lxor|match|member|method|mixin|mod|module|mutable|namespace|new|not|null|object|of|open|or|override|parallel|private|process|protected|public|pure|rec|sealed|select|sig|static|struct|tailcall|then|to|trait|true|try|type|upcast|val|virtual|void|volatile|when|while|with)\b/,number:[/\b0x[\da-fA-F]+(?:LF|lf|un)?\b/,/\b0b[01]+(?:uy|y)?\b/,/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[fm]|e[+-]?\d+)?\b/i,/\b\d+(?:[IlLsy]|UL|u[lsy]?)?\b/],operator:/([<>~&^])\1\1|([*.:<>&])\2|<-|->|[!=:]=|<?\|{1,3}>?|\??(?:<=|>=|<>|[-+*/%=<>])\??|[!?^&]|~[+~-]|:>|:\?>?/}),t.languages.insertBefore("fsharp","keyword",{preprocessor:{pattern:/(^[\t ]*)#.*/m,lookbehind:!0,alias:"property",inside:{directive:{pattern:/(^#)\b(?:else|endif|if|light|line|nowarn)\b/,lookbehind:!0,alias:"keyword"}}}}),t.languages.insertBefore("fsharp","punctuation",{"computation-expression":{pattern:/\b[_a-z]\w*(?=\s*\{)/i,alias:"keyword"}}),t.languages.insertBefore("fsharp","string",{annotation:{pattern:/\[<.+?>\]/,greedy:!0,inside:{punctuation:/^\[<|>\]$/,"class-name":{pattern:/^\w+$|(^|;\s*)[A-Z]\w*(?=\()/,lookbehind:!0},"annotation-content":{pattern:/[\s\S]+/,inside:t.languages.fsharp}}},char:{pattern:/'(?:[^\\']|\\(?:.|\d{3}|x[a-fA-F\d]{2}|u[a-fA-F\d]{4}|U[a-fA-F\d]{8}))'B?/,greedy:!0}})}return XT}var ZT,TG;function Cat(){if(TG)return ZT;TG=1;var e=Si();ZT=t,t.displayName="ftl",t.aliases=[];function t(r){r.register(e),(function(n){for(var a=/[^<()"']|\((?:<expr>)*\)|<(?!#--)|<#--(?:[^-]|-(?!->))*-->|"(?:[^\\"]|\\.)*"|'(?:[^\\']|\\.)*'/.source,i=0;i<2;i++)a=a.replace(/<expr>/g,function(){return a});a=a.replace(/<expr>/g,/[^\s\S]/.source);var o={comment:/<#--[\s\S]*?-->/,string:[{pattern:/\br("|')(?:(?!\1)[^\\]|\\.)*\1/,greedy:!0},{pattern:RegExp(/("|')(?:(?!\1|\$\{)[^\\]|\\.|\$\{(?:(?!\})(?:<expr>))*\})*\1/.source.replace(/<expr>/g,function(){return a})),greedy:!0,inside:{interpolation:{pattern:RegExp(/((?:^|[^\\])(?:\\\\)*)\$\{(?:(?!\})(?:<expr>))*\}/.source.replace(/<expr>/g,function(){return a})),lookbehind:!0,inside:{"interpolation-punctuation":{pattern:/^\$\{|\}$/,alias:"punctuation"},rest:null}}}}],keyword:/\b(?:as)\b/,boolean:/\b(?:false|true)\b/,"builtin-function":{pattern:/((?:^|[^?])\?\s*)\w+/,lookbehind:!0,alias:"function"},function:/\b\w+(?=\s*\()/,number:/\b\d+(?:\.\d+)?\b/,operator:/\.\.[<*!]?|->|--|\+\+|&&|\|\||\?{1,2}|[-+*/%!=<>]=?|\b(?:gt|gte|lt|lte)\b/,punctuation:/[,;.:()[\]{}]/};o.string[1].inside.interpolation.inside.rest=o,n.languages.ftl={"ftl-comment":{pattern:/^<#--[\s\S]*/,alias:"comment"},"ftl-directive":{pattern:/^<[\s\S]+>$/,inside:{directive:{pattern:/(^<\/?)[#@][a-z]\w*/i,lookbehind:!0,alias:"keyword"},punctuation:/^<\/?|\/?>$/,content:{pattern:/\s*\S[\s\S]*/,alias:"ftl",inside:o}}},"ftl-interpolation":{pattern:/^\$\{[\s\S]*\}$/,inside:{punctuation:/^\$\{|\}$/,content:{pattern:/\s*\S[\s\S]*/,alias:"ftl",inside:o}}}},n.hooks.add("before-tokenize",function(s){var l=RegExp(/<#--[\s\S]*?-->|<\/?[#@][a-zA-Z](?:<expr>)*?>|\$\{(?:<expr>)*?\}/.source.replace(/<expr>/g,function(){return a}),"gi");n.languages["markup-templating"].buildPlaceholders(s,"ftl",l)}),n.hooks.add("after-tokenize",function(s){n.languages["markup-templating"].tokenizePlaceholders(s,"ftl")})})(r)}return ZT}var QT,AG;function Tat(){if(AG)return QT;AG=1,QT=e,e.displayName="gap",e.aliases=[];function e(t){t.languages.gap={shell:{pattern:/^gap>[\s\S]*?(?=^gap>|$(?![\s\S]))/m,greedy:!0,inside:{gap:{pattern:/^(gap>).+(?:(?:\r(?:\n|(?!\n))|\n)>.*)*/,lookbehind:!0,inside:null},punctuation:/^gap>/}},comment:{pattern:/#.*/,greedy:!0},string:{pattern:/(^|[^\\'"])(?:'(?:[^\r\n\\']|\\.){1,10}'|"(?:[^\r\n\\"]|\\.)*"(?!")|"""[\s\S]*?""")/,lookbehind:!0,greedy:!0,inside:{continuation:{pattern:/([\r\n])>/,lookbehind:!0,alias:"punctuation"}}},keyword:/\b(?:Assert|Info|IsBound|QUIT|TryNextMethod|Unbind|and|atomic|break|continue|do|elif|else|end|fi|for|function|if|in|local|mod|not|od|or|quit|readonly|readwrite|rec|repeat|return|then|until|while)\b/,boolean:/\b(?:false|true)\b/,function:/\b[a-z_]\w*(?=\s*\()/i,number:{pattern:/(^|[^\w.]|\.\.)(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?(?:_[a-z]?)?(?=$|[^\w.]|\.\.)/,lookbehind:!0},continuation:{pattern:/([\r\n])>/,lookbehind:!0,alias:"punctuation"},operator:/->|[-+*/^~=!]|<>|[<>]=?|:=|\.\./,punctuation:/[()[\]{},;.:]/},t.languages.gap.shell.inside.gap.inside=t.languages.gap}return QT}var eA,FG;function Aat(){if(FG)return eA;FG=1,eA=e,e.displayName="gcode",e.aliases=[];function e(t){t.languages.gcode={comment:/;.*|\B\(.*?\)\B/,string:{pattern:/"(?:""|[^"])*"/,greedy:!0},keyword:/\b[GM]\d+(?:\.\d+)?\b/,property:/\b[A-Z]/,checksum:{pattern:/(\*)\d+/,lookbehind:!0,alias:"number"},punctuation:/[:*]/}}return eA}var tA,_G;function Fat(){if(_G)return tA;_G=1,tA=e,e.displayName="gdscript",e.aliases=[];function e(t){t.languages.gdscript={comment:/#.*/,string:{pattern:/@?(?:("|')(?:(?!\1)[^\n\\]|\\[\s\S])*\1(?!"|')|"""(?:[^\\]|\\[\s\S])*?""")/,greedy:!0},"class-name":{pattern:/(^(?:class|class_name|extends)[ \t]+|^export\([ \t]*|\bas[ \t]+|(?:\b(?:const|var)[ \t]|[,(])[ \t]*\w+[ \t]*:[ \t]*|->[ \t]*)[a-zA-Z_]\w*/m,lookbehind:!0},keyword:/\b(?:and|as|assert|break|breakpoint|class|class_name|const|continue|elif|else|enum|export|extends|for|func|if|in|is|master|mastersync|match|not|null|onready|or|pass|preload|puppet|puppetsync|remote|remotesync|return|self|setget|signal|static|tool|var|while|yield)\b/,function:/\b[a-z_]\w*(?=[ \t]*\()/i,variable:/\$\w+/,number:[/\b0b[01_]+\b|\b0x[\da-fA-F_]+\b|(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.[\d_]+)(?:e[+-]?[\d_]+)?\b/,/\b(?:INF|NAN|PI|TAU)\b/],constant:/\b[A-Z][A-Z_\d]*\b/,boolean:/\b(?:false|true)\b/,operator:/->|:=|&&|\|\||<<|>>|[-+*/%&|!<>=]=?|[~^]/,punctuation:/[.:,;()[\]{}]/}}return tA}var rA,RG;function _at(){if(RG)return rA;RG=1,rA=e,e.displayName="gedcom",e.aliases=[];function e(t){t.languages.gedcom={"line-value":{pattern:/(^[\t ]*\d+ +(?:@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@ +)?\w+ ).+/m,lookbehind:!0,inside:{pointer:{pattern:/^@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@$/,alias:"variable"}}},tag:{pattern:/(^[\t ]*\d+ +(?:@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@ +)?)\w+/m,lookbehind:!0,alias:"string"},level:{pattern:/(^[\t ]*)\d+/m,lookbehind:!0,alias:"number"},pointer:{pattern:/@\w[\w!"$%&'()*+,\-./:;<=>?[\\\]^`{|}~\x80-\xfe #]*@/,alias:"variable"}}}return rA}var nA,DG;function Rat(){if(DG)return nA;DG=1,nA=e,e.displayName="gherkin",e.aliases=[];function e(t){(function(r){var n=/(?:\r?\n|\r)[ \t]*\|.+\|(?:(?!\|).)*/.source;r.languages.gherkin={pystring:{pattern:/("""|''')[\s\S]+?\1/,alias:"string"},comment:{pattern:/(^[ \t]*)#.*/m,lookbehind:!0},tag:{pattern:/(^[ \t]*)@\S*/m,lookbehind:!0},feature:{pattern:/((?:^|\r?\n|\r)[ \t]*)(?:Ability|Ahoy matey!|Arwedd|Aspekt|Besigheid Behoefte|Business Need|Caracteristica|Característica|Egenskab|Egenskap|Eiginleiki|Feature|Fīča|Fitur|Fonctionnalité|Fonksyonalite|Funcionalidade|Funcionalitat|Functionalitate|Funcţionalitate|Funcționalitate|Functionaliteit|Fungsi|Funkcia|Funkcija|Funkcionalitāte|Funkcionalnost|Funkcja|Funksie|Funktionalität|Funktionalitéit|Funzionalità|Hwaet|Hwæt|Jellemző|Karakteristik|Lastnost|Mak|Mogucnost|laH|Mogućnost|Moznosti|Možnosti|OH HAI|Omadus|Ominaisuus|Osobina|Özellik|Potrzeba biznesowa|perbogh|poQbogh malja'|Požadavek|Požiadavka|Pretty much|Qap|Qu'meH 'ut|Savybė|Tính năng|Trajto|Vermoë|Vlastnosť|Właściwość|Značilnost|Δυνατότητα|Λειτουργία|Могућност|Мөмкинлек|Особина|Свойство|Үзенчәлеклелек|Функционал|Функционалност|Функция|Функціонал|תכונה|خاصية|خصوصیت|صلاحیت|کاروبار کی ضرورت|وِیژگی|रूप लेख|ਖਾਸੀਅਤ|ਨਕਸ਼ ਨੁਹਾਰ|ਮੁਹਾਂਦਰਾ|గుణము|ಹೆಚ್ಚಳ|ความต้องการทางธุรกิจ|ความสามารถ|โครงหลัก|기능|フィーチャ|功能|機能):(?:[^:\r\n]+(?:\r?\n|\r|$))*/,lookbehind:!0,inside:{important:{pattern:/(:)[^\r\n]+/,lookbehind:!0},keyword:/[^:\r\n]+:/}},scenario:{pattern:/(^[ \t]*)(?:Abstract Scenario|Abstrakt Scenario|Achtergrond|Aer|Ær|Agtergrond|All y'all|Antecedentes|Antecedents|Atburðarás|Atburðarásir|Awww, look mate|B4|Background|Baggrund|Bakgrund|Bakgrunn|Bakgrunnur|Beispiele|Beispiller|Bối cảnh|Cefndir|Cenario|Cenário|Cenario de Fundo|Cenário de Fundo|Cenarios|Cenários|Contesto|Context|Contexte|Contexto|Conto|Contoh|Contone|Dæmi|Dasar|Dead men tell no tales|Delineacao do Cenario|Delineação do Cenário|Dis is what went down|Dữ liệu|Dyagram Senaryo|Dyagram senaryo|Egzanp|Ejemplos|Eksempler|Ekzemploj|Enghreifftiau|Esbozo do escenario|Escenari|Escenario|Esempi|Esquema de l'escenari|Esquema del escenario|Esquema do Cenario|Esquema do Cenário|EXAMPLZ|Examples|Exempel|Exemple|Exemples|Exemplos|First off|Fono|Forgatókönyv|Forgatókönyv vázlat|Fundo|Geçmiş|Grundlage|Hannergrond|ghantoH|Háttér|Heave to|Istorik|Juhtumid|Keadaan|Khung kịch bản|Khung tình huống|Kịch bản|Koncept|Konsep skenario|Kontèks|Kontekst|Kontekstas|Konteksts|Kontext|Konturo de la scenaro|Latar Belakang|lut chovnatlh|lut|lutmey|Lýsing Atburðarásar|Lýsing Dæma|MISHUN SRSLY|MISHUN|Menggariskan Senario|mo'|Náčrt Scenára|Náčrt Scénáře|Náčrt Scenáru|Oris scenarija|Örnekler|Osnova|Osnova Scenára|Osnova scénáře|Osnutek|Ozadje|Paraugs|Pavyzdžiai|Példák|Piemēri|Plan du scénario|Plan du Scénario|Plan Senaryo|Plan senaryo|Plang vum Szenario|Pozadí|Pozadie|Pozadina|Príklady|Příklady|Primer|Primeri|Primjeri|Przykłady|Raamstsenaarium|Reckon it's like|Rerefons|Scenár|Scénář|Scenarie|Scenarij|Scenarijai|Scenarijaus šablonas|Scenariji|Scenārijs|Scenārijs pēc parauga|Scenarijus|Scenario|Scénario|Scenario Amlinellol|Scenario Outline|Scenario Template|Scenariomal|Scenariomall|Scenarios|Scenariu|Scenariusz|Scenaro|Schema dello scenario|Se ðe|Se the|Se þe|Senario|Senaryo Deskripsyon|Senaryo deskripsyon|Senaryo|Senaryo taslağı|Shiver me timbers|Situācija|Situai|Situasie Uiteensetting|Situasie|Skenario konsep|Skenario|Skica|Structura scenariu|Structură scenariu|Struktura scenarija|Stsenaarium|Swa hwaer swa|Swa|Swa hwær swa|Szablon scenariusza|Szenario|Szenariogrundriss|Tapaukset|Tapaus|Tapausaihio|Taust|Tausta|Template Keadaan|Template Senario|Template Situai|The thing of it is|Tình huống|Variantai|Voorbeelde|Voorbeelden|Wharrimean is|Yo-ho-ho|You'll wanna|Założenia|Παραδείγματα|Περιγραφή Σεναρίου|Σενάρια|Σενάριο|Υπόβαθρο|Кереш|Контекст|Концепт|Мисаллар|Мисоллар|Основа|Передумова|Позадина|Предистория|Предыстория|Приклади|Пример|Примери|Примеры|Рамка на сценарий|Скица|Структура сценарија|Структура сценария|Структура сценарію|Сценарий|Сценарий структураси|Сценарийның төзелеше|Сценарији|Сценарио|Сценарій|Тарих|Үрнәкләр|דוגמאות|רקע|תבנית תרחיש|תרחיש|الخلفية|الگوی سناریو|امثلة|پس منظر|زمینه|سناریو|سيناريو|سيناريو مخطط|مثالیں|منظر نامے کا خاکہ|منظرنامہ|نمونه ها|उदाहरण|परिदृश्य|परिदृश्य रूपरेखा|पृष्ठभूमि|ਉਦਾਹਰਨਾਂ|ਪਟਕਥਾ|ਪਟਕਥਾ ਢਾਂਚਾ|ਪਟਕਥਾ ਰੂਪ ਰੇਖਾ|ਪਿਛੋਕੜ|ఉదాహరణలు|కథనం|నేపథ్యం|సన్నివేశం|ಉದಾಹರಣೆಗಳು|ಕಥಾಸಾರಾಂಶ|ವಿವರಣೆ|ಹಿನ್ನೆಲೆ|โครงสร้างของเหตุการณ์|ชุดของตัวอย่าง|ชุดของเหตุการณ์|แนวคิด|สรุปเหตุการณ์|เหตุการณ์|배경|시나리오|시나리오 개요|예|サンプル|シナリオ|シナリオアウトライン|シナリオテンプレ|シナリオテンプレート|テンプレ|例|例子|剧本|剧本大纲|劇本|劇本大綱|场景|场景大纲|場景|場景大綱|背景):[^:\r\n]*/m,lookbehind:!0,inside:{important:{pattern:/(:)[^\r\n]*/,lookbehind:!0},keyword:/[^:\r\n]+:/}},"table-body":{pattern:RegExp("("+n+")(?:"+n+")+"),lookbehind:!0,inside:{outline:{pattern:/<[^>]+>/,alias:"variable"},td:{pattern:/\s*[^\s|][^|]*/,alias:"string"},punctuation:/\|/}},"table-head":{pattern:RegExp(n),inside:{th:{pattern:/\s*[^\s|][^|]*/,alias:"variable"},punctuation:/\|/}},atrule:{pattern:/(^[ \t]+)(?:'a|'ach|'ej|7|a|A také|A taktiež|A tiež|A zároveň|Aber|Ac|Adott|Akkor|Ak|Aleshores|Ale|Ali|Allora|Alors|Als|Ama|Amennyiben|Amikor|Ampak|an|AN|Ananging|And y'all|And|Angenommen|Anrhegedig a|An|Apabila|Atès|Atesa|Atunci|Avast!|Aye|A|awer|Bagi|Banjur|Bet|Biết|Blimey!|Buh|But at the end of the day I reckon|But y'all|But|BUT|Cal|Când|Cand|Cando|Ce|Cuando|Če|Ða ðe|Ða|Dadas|Dada|Dados|Dado|DaH ghu' bejlu'|dann|Dann|Dano|Dan|Dar|Dat fiind|Data|Date fiind|Date|Dati fiind|Dati|Daţi fiind|Dați fiind|DEN|Dato|De|Den youse gotta|Dengan|Diberi|Diyelim ki|Donada|Donat|Donitaĵo|Do|Dun|Duota|Ðurh|Eeldades|Ef|Eğer ki|Entao|Então|Entón|E|En|Entonces|Epi|És|Etant donnée|Etant donné|Et|Étant données|Étant donnée|Étant donné|Etant données|Etant donnés|Étant donnés|Fakat|Gangway!|Gdy|Gegeben seien|Gegeben sei|Gegeven|Gegewe|ghu' noblu'|Gitt|Given y'all|Given|Givet|Givun|Ha|Cho|I CAN HAZ|In|Ir|It's just unbelievable|I|Ja|Jeśli|Jeżeli|Kad|Kada|Kadar|Kai|Kaj|Když|Keď|Kemudian|Ketika|Khi|Kiedy|Ko|Kuid|Kui|Kun|Lan|latlh|Le sa a|Let go and haul|Le|Lè sa a|Lè|Logo|Lorsqu'<|Lorsque|mä|Maar|Mais|Mając|Ma|Majd|Maka|Manawa|Mas|Men|Menawa|Mutta|Nalika|Nalikaning|Nanging|Når|När|Nato|Nhưng|Niin|Njuk|O zaman|Och|Og|Oletetaan|Ond|Onda|Oraz|Pak|Pero|Però|Podano|Pokiaľ|Pokud|Potem|Potom|Privzeto|Pryd|Quan|Quand|Quando|qaSDI'|Så|Sed|Se|Siis|Sipoze ke|Sipoze Ke|Sipoze|Si|Şi|Și|Soit|Stel|Tada|Tad|Takrat|Tak|Tapi|Ter|Tetapi|Tha the|Tha|Then y'all|Then|Thì|Thurh|Toda|Too right|Un|Und|ugeholl|Và|vaj|Vendar|Ve|wann|Wanneer|WEN|Wenn|When y'all|When|Wtedy|Wun|Y'know|Yeah nah|Yna|Youse know like when|Youse know when youse got|Y|Za predpokladu|Za předpokladu|Zadan|Zadani|Zadano|Zadate|Zadato|Zakładając|Zaradi|Zatati|Þa þe|Þa|Þá|Þegar|Þurh|Αλλά|Δεδομένου|Και|Όταν|Τότε|А також|Агар|Але|Али|Аммо|А|Әгәр|Әйтик|Әмма|Бирок|Ва|Вә|Дадено|Дано|Допустим|Если|Задате|Задати|Задато|И|І|К тому же|Када|Кад|Когато|Когда|Коли|Ләкин|Лекин|Нәтиҗәдә|Нехай|Но|Онда|Припустимо, що|Припустимо|Пусть|Также|Та|Тогда|Тоді|То|Унда|Һәм|Якщо|אבל|אזי|אז|בהינתן|וגם|כאשר|آنگاه|اذاً|اگر|اما|اور|با فرض|بالفرض|بفرض|پھر|تب|ثم|جب|عندما|فرض کیا|لكن|لیکن|متى|هنگامی|و|अगर|और|कदा|किन्तु|चूंकि|जब|तथा|तदा|तब|परन्तु|पर|यदि|ਅਤੇ|ਜਦੋਂ|ਜਿਵੇਂ ਕਿ|ਜੇਕਰ|ਤਦ|ਪਰ|అప్పుడు|ఈ పరిస్థితిలో|కాని|చెప్పబడినది|మరియు|ಆದರೆ|ನಂತರ|ನೀಡಿದ|ಮತ್ತು|ಸ್ಥಿತಿಯನ್ನು|กำหนดให้|ดังนั้น|แต่|เมื่อ|และ|그러면<|그리고<|단<|만약<|만일<|먼저<|조건<|하지만<|かつ<|しかし<|ただし<|ならば<|もし<|並且<|但し<|但是<|假如<|假定<|假設<|假设<|前提<|同时<|同時<|并且<|当<|當<|而且<|那么<|那麼<)(?=[ \t])/m,lookbehind:!0},string:{pattern:/"(?:\\.|[^"\\\r\n])*"|'(?:\\.|[^'\\\r\n])*'/,inside:{outline:{pattern:/<[^>]+>/,alias:"variable"}}},outline:{pattern:/<[^>]+>/,alias:"variable"}}})(t)}return nA}var aA,NG;function Dat(){if(NG)return aA;NG=1,aA=e,e.displayName="git",e.aliases=[];function e(t){t.languages.git={comment:/^#.*/m,deleted:/^[-–].*/m,inserted:/^\+.*/m,string:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,command:{pattern:/^.*\$ git .*$/m,inside:{parameter:/\s--?\w+/}},coord:/^@@.*@@$/m,"commit-sha1":/^commit \w{40}$/m}}return aA}var iA,LG;function Nat(){if(LG)return iA;LG=1;var e=pd();iA=t,t.displayName="glsl",t.aliases=[];function t(r){r.register(e),r.languages.glsl=r.languages.extend("c",{keyword:/\b(?:active|asm|atomic_uint|attribute|[ibdu]?vec[234]|bool|break|buffer|case|cast|centroid|class|coherent|common|const|continue|d?mat[234](?:x[234])?|default|discard|do|double|else|enum|extern|external|false|filter|fixed|flat|float|for|fvec[234]|goto|half|highp|hvec[234]|[iu]?sampler2DMS(?:Array)?|[iu]?sampler2DRect|[iu]?samplerBuffer|[iu]?samplerCube|[iu]?samplerCubeArray|[iu]?sampler[123]D|[iu]?sampler[12]DArray|[iu]?image2DMS(?:Array)?|[iu]?image2DRect|[iu]?imageBuffer|[iu]?imageCube|[iu]?imageCubeArray|[iu]?image[123]D|[iu]?image[12]DArray|if|in|inline|inout|input|int|interface|invariant|layout|long|lowp|mediump|namespace|noinline|noperspective|out|output|partition|patch|precise|precision|public|readonly|resource|restrict|return|sample|sampler[12]DArrayShadow|sampler[12]DShadow|sampler2DRectShadow|sampler3DRect|samplerCubeArrayShadow|samplerCubeShadow|shared|short|sizeof|smooth|static|struct|subroutine|superp|switch|template|this|true|typedef|uint|uniform|union|unsigned|using|varying|void|volatile|while|writeonly)\b/})}return iA}var oA,MG;function Lat(){if(MG)return oA;MG=1,oA=e,e.displayName="gml",e.aliases=[];function e(t){t.languages.gamemakerlanguage=t.languages.gml=t.languages.extend("clike",{keyword:/\b(?:break|case|continue|default|do|else|enum|exit|for|globalvar|if|repeat|return|switch|until|var|while)\b/,number:/(?:\b0x[\da-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)[ulf]{0,4}/i,operator:/--|\+\+|[-+%/=]=?|!=|\*\*?=?|<[<=>]?|>[=>]?|&&?|\^\^?|\|\|?|~|\b(?:and|at|not|or|with|xor)\b/,constant:/\b(?:GM_build_date|GM_version|action_(?:continue|restart|reverse|stop)|all|gamespeed_(?:fps|microseconds)|global|local|noone|other|pi|pointer_(?:invalid|null)|self|timezone_(?:local|utc)|undefined|ev_(?:create|destroy|step|alarm|keyboard|mouse|collision|other|draw|draw_(?:begin|end|post|pre)|keypress|keyrelease|trigger|(?:left|middle|no|right)_button|(?:left|middle|right)_press|(?:left|middle|right)_release|mouse_(?:enter|leave|wheel_down|wheel_up)|global_(?:left|middle|right)_button|global_(?:left|middle|right)_press|global_(?:left|middle|right)_release|joystick(?:1|2)_(?:button1|button2|button3|button4|button5|button6|button7|button8|down|left|right|up)|outside|boundary|game_start|game_end|room_start|room_end|no_more_lives|animation_end|end_of_path|no_more_health|user\d|gui|gui_begin|gui_end|step_(?:begin|end|normal))|vk_(?:alt|anykey|backspace|control|delete|down|end|enter|escape|home|insert|left|nokey|pagedown|pageup|pause|printscreen|return|right|shift|space|tab|up|f\d|numpad\d|add|decimal|divide|lalt|lcontrol|lshift|multiply|ralt|rcontrol|rshift|subtract)|achievement_(?:filter_(?:all_players|favorites_only|friends_only)|friends_info|info|leaderboard_info|our_info|pic_loaded|show_(?:achievement|bank|friend_picker|leaderboard|profile|purchase_prompt|ui)|type_challenge|type_score_challenge)|asset_(?:font|object|path|room|script|shader|sound|sprite|tiles|timeline|unknown)|audio_(?:3d|falloff_(?:exponent_distance|exponent_distance_clamped|inverse_distance|inverse_distance_clamped|linear_distance|linear_distance_clamped|none)|mono|new_system|old_system|stereo)|bm_(?:add|complex|dest_alpha|dest_color|dest_colour|inv_dest_alpha|inv_dest_color|inv_dest_colour|inv_src_alpha|inv_src_color|inv_src_colour|max|normal|one|src_alpha|src_alpha_sat|src_color|src_colour|subtract|zero)|browser_(?:chrome|firefox|ie|ie_mobile|not_a_browser|opera|safari|safari_mobile|tizen|unknown|windows_store)|buffer_(?:bool|f16|f32|f64|fast|fixed|generalerror|grow|invalidtype|network|outofbounds|outofspace|s16|s32|s8|seek_end|seek_relative|seek_start|string|text|u16|u32|u64|u8|vbuffer|wrap)|c_(?:aqua|black|blue|dkgray|fuchsia|gray|green|lime|ltgray|maroon|navy|olive|orange|purple|red|silver|teal|white|yellow)|cmpfunc_(?:always|equal|greater|greaterequal|less|lessequal|never|notequal)|cr_(?:appstart|arrow|beam|cross|default|drag|handpoint|hourglass|none|size_all|size_nesw|size_ns|size_nwse|size_we|uparrow)|cull_(?:clockwise|counterclockwise|noculling)|device_(?:emulator|tablet)|device_ios_(?:ipad|ipad_retina|iphone|iphone5|iphone6|iphone6plus|iphone_retina|unknown)|display_(?:landscape|landscape_flipped|portrait|portrait_flipped)|dll_(?:cdecl|cdel|stdcall)|ds_type_(?:grid|list|map|priority|queue|stack)|ef_(?:cloud|ellipse|explosion|firework|flare|rain|ring|smoke|smokeup|snow|spark|star)|fa_(?:archive|bottom|center|directory|hidden|left|middle|readonly|right|sysfile|top|volumeid)|fb_login_(?:default|fallback_to_webview|forcing_safari|forcing_webview|no_fallback_to_webview|use_system_account)|iap_(?:available|canceled|ev_consume|ev_product|ev_purchase|ev_restore|ev_storeload|failed|purchased|refunded|status_available|status_loading|status_processing|status_restoring|status_unavailable|status_uninitialised|storeload_failed|storeload_ok|unavailable)|leaderboard_type_(?:number|time_mins_secs)|lighttype_(?:dir|point)|matrix_(?:projection|view|world)|mb_(?:any|left|middle|none|right)|network_(?:config_(?:connect_timeout|disable_reliable_udp|enable_reliable_udp|use_non_blocking_socket)|socket_(?:bluetooth|tcp|udp)|type_(?:connect|data|disconnect|non_blocking_connect))|of_challenge_(?:lose|tie|win)|os_(?:android|ios|linux|macosx|ps3|ps4|psvita|unknown|uwp|win32|win8native|windows|winphone|xboxone)|phy_debug_render_(?:aabb|collision_pairs|coms|core_shapes|joints|obb|shapes)|phy_joint_(?:anchor_1_x|anchor_1_y|anchor_2_x|anchor_2_y|angle|angle_limits|damping_ratio|frequency|length_1|length_2|lower_angle_limit|max_force|max_length|max_motor_force|max_motor_torque|max_torque|motor_force|motor_speed|motor_torque|reaction_force_x|reaction_force_y|reaction_torque|speed|translation|upper_angle_limit)|phy_particle_data_flag_(?:category|color|colour|position|typeflags|velocity)|phy_particle_flag_(?:colormixing|colourmixing|elastic|powder|spring|tensile|viscous|wall|water|zombie)|phy_particle_group_flag_(?:rigid|solid)|pr_(?:linelist|linestrip|pointlist|trianglefan|trianglelist|trianglestrip)|ps_(?:distr|shape)_(?:diamond|ellipse|gaussian|invgaussian|line|linear|rectangle)|pt_shape_(?:circle|cloud|disk|explosion|flare|line|pixel|ring|smoke|snow|spark|sphere|square|star)|ty_(?:real|string)|gp_(?:face\d|axislh|axislv|axisrh|axisrv|padd|padl|padr|padu|select|shoulderl|shoulderlb|shoulderr|shoulderrb|start|stickl|stickr)|lb_disp_(?:none|numeric|time_ms|time_sec)|lb_sort_(?:ascending|descending|none)|ov_(?:achievements|community|friends|gamegroup|players|settings)|ugc_(?:filetype_(?:community|microtrans)|list_(?:Favorited|Followed|Published|Subscribed|UsedOrPlayed|VotedDown|VotedOn|VotedUp|WillVoteLater)|match_(?:AllGuides|Artwork|Collections|ControllerBindings|IntegratedGuides|Items|Items_Mtx|Items_ReadyToUse|Screenshots|UsableInGame|Videos|WebGuides)|query_(?:AcceptedForGameRankedByAcceptanceDate|CreatedByFriendsRankedByPublicationDate|FavoritedByFriendsRankedByPublicationDate|NotYetRated)|query_RankedBy(?:NumTimesReported|PublicationDate|TextSearch|TotalVotesAsc|Trend|Vote|VotesUp)|result_success|sortorder_CreationOrder(?:Asc|Desc)|sortorder_(?:ForModeration|LastUpdatedDesc|SubscriptionDateDesc|TitleAsc|VoteScoreDesc)|visibility_(?:friends_only|private|public))|vertex_usage_(?:binormal|blendindices|blendweight|color|colour|depth|fog|normal|position|psize|sample|tangent|texcoord|textcoord)|vertex_type_(?:float\d|color|colour|ubyte4)|input_type|layerelementtype_(?:background|instance|oldtilemap|particlesystem|sprite|tile|tilemap|undefined)|se_(?:chorus|compressor|echo|equalizer|flanger|gargle|none|reverb)|text_type|tile_(?:flip|index_mask|mirror|rotate)|(?:obj|rm|scr|spr)\w+)\b/,variable:/\b(?:alarm|application_surface|async_load|background_(?:alpha|blend|color|colour|foreground|height|hspeed|htiled|index|showcolor|showcolour|visible|vspeed|vtiled|width|x|xscale|y|yscale)|bbox_(?:bottom|left|right|top)|browser_(?:height|width)|caption_(?:health|lives|score)|current_(?:day|hour|minute|month|second|time|weekday|year)|cursor_sprite|debug_mode|delta_time|direction|display_aa|error_(?:last|occurred)|event_(?:action|number|object|type)|fps|fps_real|friction|game_(?:display|project|save)_(?:id|name)|gamemaker_(?:pro|registered|version)|gravity|gravity_direction|(?:h|v)speed|health|iap_data|id|image_(?:alpha|angle|blend|depth|index|number|speed|xscale|yscale)|instance_(?:count|id)|keyboard_(?:key|lastchar|lastkey|string)|layer|lives|mask_index|mouse_(?:button|lastbutton|x|y)|object_index|os_(?:browser|device|type|version)|path_(?:endaction|index|orientation|position|positionprevious|scale|speed)|persistent|phy_(?:rotation|(?:col_normal|collision|com|linear_velocity|position|speed)_(?:x|y)|angular_(?:damping|velocity)|position_(?:x|y)previous|speed|linear_damping|bullet|fixed_rotation|active|mass|inertia|dynamic|kinematic|sleeping|collision_points)|pointer_(?:invalid|null)|room|room_(?:caption|first|height|last|persistent|speed|width)|score|secure_mode|show_(?:health|lives|score)|solid|speed|sprite_(?:height|index|width|xoffset|yoffset)|temp_directory|timeline_(?:index|loop|position|running|speed)|transition_(?:color|kind|steps)|undefined|view_(?:angle|current|enabled|(?:h|v)(?:border|speed)|(?:h|w|x|y)port|(?:h|w|x|y)view|object|surface_id|visible)|visible|webgl_enabled|working_directory|(?:x|y)(?:previous|start)|x|y|argument(?:_relitive|_count|\d)|argument|global|local|other|self)\b/})}return oA}var sA,IG;function Mat(){if(IG)return sA;IG=1,sA=e,e.displayName="gn",e.aliases=["gni"];function e(t){t.languages.gn={comment:{pattern:/#.*/,greedy:!0},"string-literal":{pattern:/(^|[^\\"])"(?:[^\r\n"\\]|\\.)*"/,lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$(?:\{[\s\S]*?\}|[a-zA-Z_]\w*|0x[a-fA-F0-9]{2})/,lookbehind:!0,inside:{number:/^\$0x[\s\S]{2}$/,variable:/^\$\w+$/,"interpolation-punctuation":{pattern:/^\$\{|\}$/,alias:"punctuation"},expression:{pattern:/[\s\S]+/,inside:null}}},string:/[\s\S]+/}},keyword:/\b(?:else|if)\b/,boolean:/\b(?:false|true)\b/,"builtin-function":{pattern:/\b(?:assert|defined|foreach|import|pool|print|template|tool|toolchain)(?=\s*\()/i,alias:"keyword"},function:/\b[a-z_]\w*(?=\s*\()/i,constant:/\b(?:current_cpu|current_os|current_toolchain|default_toolchain|host_cpu|host_os|root_build_dir|root_gen_dir|root_out_dir|target_cpu|target_gen_dir|target_os|target_out_dir)\b/,number:/-?\b\d+\b/,operator:/[-+!=<>]=?|&&|\|\|/,punctuation:/[(){}[\],.]/},t.languages.gn["string-literal"].inside.interpolation.inside.expression.inside=t.languages.gn,t.languages.gni=t.languages.gn}return sA}var lA,OG;function Iat(){if(OG)return lA;OG=1,lA=e,e.displayName="goModule",e.aliases=[];function e(t){t.languages["go-mod"]=t.languages["go-module"]={comment:{pattern:/\/\/.*/,greedy:!0},version:{pattern:/(^|[\s()[\],])v\d+\.\d+\.\d+(?:[+-][-+.\w]*)?(?![^\s()[\],])/,lookbehind:!0,alias:"number"},"go-version":{pattern:/((?:^|\s)go\s+)\d+(?:\.\d+){1,2}/,lookbehind:!0,alias:"number"},keyword:{pattern:/^([ \t]*)(?:exclude|go|module|replace|require|retract)\b/m,lookbehind:!0},operator:/=>/,punctuation:/[()[\],]/}}return lA}var uA,BG;function Oat(){if(BG)return uA;BG=1,uA=e,e.displayName="go",e.aliases=[];function e(t){t.languages.go=t.languages.extend("clike",{string:{pattern:/(^|[^\\])"(?:\\.|[^"\\\r\n])*"|`[^`]*`/,lookbehind:!0,greedy:!0},keyword:/\b(?:break|case|chan|const|continue|default|defer|else|fallthrough|for|func|go(?:to)?|if|import|interface|map|package|range|return|select|struct|switch|type|var)\b/,boolean:/\b(?:_|false|iota|nil|true)\b/,number:[/\b0(?:b[01_]+|o[0-7_]+)i?\b/i,/\b0x(?:[a-f\d_]+(?:\.[a-f\d_]*)?|\.[a-f\d_]+)(?:p[+-]?\d+(?:_\d+)*)?i?(?!\w)/i,/(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.\d[\d_]*)(?:e[+-]?[\d_]+)?i?(?!\w)/i],operator:/[*\/%^!=]=?|\+[=+]?|-[=-]?|\|[=|]?|&(?:=|&|\^=?)?|>(?:>=?|=)?|<(?:<=?|=|-)?|:=|\.\.\./,builtin:/\b(?:append|bool|byte|cap|close|complex|complex(?:64|128)|copy|delete|error|float(?:32|64)|u?int(?:8|16|32|64)?|imag|len|make|new|panic|print(?:ln)?|real|recover|rune|string|uintptr)\b/}),t.languages.insertBefore("go","string",{char:{pattern:/'(?:\\.|[^'\\\r\n]){0,10}'/,greedy:!0}}),delete t.languages.go["class-name"]}return uA}var cA,PG;function Bat(){if(PG)return cA;PG=1,cA=e,e.displayName="graphql",e.aliases=[];function e(t){t.languages.graphql={comment:/#.*/,description:{pattern:/(?:"""(?:[^"]|(?!""")")*"""|"(?:\\.|[^\\"\r\n])*")(?=\s*[a-z_])/i,greedy:!0,alias:"string",inside:{"language-markdown":{pattern:/(^"(?:"")?)(?!\1)[\s\S]+(?=\1$)/,lookbehind:!0,inside:t.languages.markdown}}},string:{pattern:/"""(?:[^"]|(?!""")")*"""|"(?:\\.|[^\\"\r\n])*"/,greedy:!0},number:/(?:\B-|\b)\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,boolean:/\b(?:false|true)\b/,variable:/\$[a-z_]\w*/i,directive:{pattern:/@[a-z_]\w*/i,alias:"function"},"attr-name":{pattern:/\b[a-z_]\w*(?=\s*(?:\((?:[^()"]|"(?:\\.|[^\\"\r\n])*")*\))?:)/i,greedy:!0},"atom-input":{pattern:/\b[A-Z]\w*Input\b/,alias:"class-name"},scalar:/\b(?:Boolean|Float|ID|Int|String)\b/,constant:/\b[A-Z][A-Z_\d]*\b/,"class-name":{pattern:/(\b(?:enum|implements|interface|on|scalar|type|union)\s+|&\s*|:\s*|\[)[A-Z_]\w*/,lookbehind:!0},fragment:{pattern:/(\bfragment\s+|\.{3}\s*(?!on\b))[a-zA-Z_]\w*/,lookbehind:!0,alias:"function"},"definition-mutation":{pattern:/(\bmutation\s+)[a-zA-Z_]\w*/,lookbehind:!0,alias:"function"},"definition-query":{pattern:/(\bquery\s+)[a-zA-Z_]\w*/,lookbehind:!0,alias:"function"},keyword:/\b(?:directive|enum|extend|fragment|implements|input|interface|mutation|on|query|repeatable|scalar|schema|subscription|type|union)\b/,operator:/[!=|&]|\.{3}/,"property-query":/\w+(?=\s*\()/,object:/\w+(?=\s*\{)/,punctuation:/[!(){}\[\]:=,]/,property:/\w+/},t.hooks.add("after-tokenize",function(n){if(n.language!=="graphql")return;var a=n.tokens.filter(function(w){return typeof w!="string"&&w.type!=="comment"&&w.type!=="scalar"}),i=0;function o(w){return a[i+w]}function s(w,E){E=E||0;for(var k=0;k<w.length;k++){var _=o(k+E);if(!_||_.type!==w[k])return!1}return!0}function l(w,E){for(var k=1,_=i;_<a.length;_++){var R=a[_],C=R.content;if(R.type==="punctuation"&&typeof C=="string"){if(w.test(C))k++;else if(E.test(C)&&(k--,k===0))return _}}return-1}function u(w,E){var k=w.alias;k?Array.isArray(k)||(w.alias=k=[k]):w.alias=k=[],k.push(E)}for(;i<a.length;){var d=a[i++];if(d.type==="keyword"&&d.content==="mutation"){var p=[];if(s(["definition-mutation","punctuation"])&&o(1).content==="("){i+=2;var f=l(/^\($/,/^\)$/);if(f===-1)continue;for(;i<f;i++){var m=o(0);m.type==="variable"&&(u(m,"variable-input"),p.push(m.content))}i=f+1}if(s(["punctuation","property-query"])&&o(0).content==="{"&&(i++,u(o(0),"property-mutation"),p.length>0)){var b=l(/^\{$/,/^\}$/);if(b===-1)continue;for(var y=i;y<b;y++){var S=a[y];S.type==="variable"&&p.indexOf(S.content)>=0&&u(S,"variable-input")}}}}})}return cA}var dA,zG;function Pat(){if(zG)return dA;zG=1,dA=e,e.displayName="groovy",e.aliases=[];function e(t){t.languages.groovy=t.languages.extend("clike",{string:[{pattern:/("""|''')(?:[^\\]|\\[\s\S])*?\1|\$\/(?:[^/$]|\$(?:[/$]|(?![/$]))|\/(?!\$))*\/\$/,greedy:!0},{pattern:/(["'/])(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0}],keyword:/\b(?:abstract|as|assert|boolean|break|byte|case|catch|char|class|const|continue|def|default|do|double|else|enum|extends|final|finally|float|for|goto|if|implements|import|in|instanceof|int|interface|long|native|new|package|private|protected|public|return|short|static|strictfp|super|switch|synchronized|this|throw|throws|trait|transient|try|void|volatile|while)\b/,number:/\b(?:0b[01_]+|0x[\da-f_]+(?:\.[\da-f_p\-]+)?|[\d_]+(?:\.[\d_]+)?(?:e[+-]?\d+)?)[glidf]?\b/i,operator:{pattern:/(^|[^.])(?:~|==?~?|\?[.:]?|\*(?:[.=]|\*=?)?|\.[@&]|\.\.<|\.\.(?!\.)|-[-=>]?|\+[+=]?|!=?|<(?:<=?|=>?)?|>(?:>>?=?|=)?|&[&=]?|\|[|=]?|\/=?|\^=?|%=?)/,lookbehind:!0},punctuation:/\.+|[{}[\];(),:$]/}),t.languages.insertBefore("groovy","string",{shebang:{pattern:/#!.+/,alias:"comment"}}),t.languages.insertBefore("groovy","punctuation",{"spock-block":/\b(?:and|cleanup|expect|given|setup|then|when|where):/}),t.languages.insertBefore("groovy","function",{annotation:{pattern:/(^|[^.])@\w+/,lookbehind:!0,alias:"punctuation"}}),t.hooks.add("wrap",function(r){if(r.language==="groovy"&&r.type==="string"){var n=r.content.value[0];if(n!="'"){var a=/([^\\])(?:\$(?:\{.*?\}|[\w.]+))/;n==="$"&&(a=/([^\$])(?:\$(?:\{.*?\}|[\w.]+))/),r.content.value=r.content.value.replace(/&lt;/g,"<").replace(/&amp;/g,"&"),r.content=t.highlight(r.content.value,{expression:{pattern:a,lookbehind:!0,inside:t.languages.groovy}}),r.classes.push(n==="/"?"regex":"gstring")}}})}return dA}var hA,HG;function zat(){if(HG)return hA;HG=1;var e=nS();hA=t,t.displayName="haml",t.aliases=[];function t(r){r.register(e),(function(n){n.languages.haml={"multiline-comment":{pattern:/((?:^|\r?\n|\r)([\t ]*))(?:\/|-#).*(?:(?:\r?\n|\r)\2[\t ].+)*/,lookbehind:!0,alias:"comment"},"multiline-code":[{pattern:/((?:^|\r?\n|\r)([\t ]*)(?:[~-]|[&!]?=)).*,[\t ]*(?:(?:\r?\n|\r)\2[\t ].*,[\t ]*)*(?:(?:\r?\n|\r)\2[\t ].+)/,lookbehind:!0,inside:n.languages.ruby},{pattern:/((?:^|\r?\n|\r)([\t ]*)(?:[~-]|[&!]?=)).*\|[\t ]*(?:(?:\r?\n|\r)\2[\t ].*\|[\t ]*)*/,lookbehind:!0,inside:n.languages.ruby}],filter:{pattern:/((?:^|\r?\n|\r)([\t ]*)):[\w-]+(?:(?:\r?\n|\r)(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/,lookbehind:!0,inside:{"filter-name":{pattern:/^:[\w-]+/,alias:"symbol"}}},markup:{pattern:/((?:^|\r?\n|\r)[\t ]*)<.+/,lookbehind:!0,inside:n.languages.markup},doctype:{pattern:/((?:^|\r?\n|\r)[\t ]*)!!!(?: .+)?/,lookbehind:!0},tag:{pattern:/((?:^|\r?\n|\r)[\t ]*)[%.#][\w\-#.]*[\w\-](?:\([^)]+\)|\{(?:\{[^}]+\}|[^{}])+\}|\[[^\]]+\])*[\/<>]*/,lookbehind:!0,inside:{attributes:[{pattern:/(^|[^#])\{(?:\{[^}]+\}|[^{}])+\}/,lookbehind:!0,inside:n.languages.ruby},{pattern:/\([^)]+\)/,inside:{"attr-value":{pattern:/(=\s*)(?:"(?:\\.|[^\\"\r\n])*"|[^)\s]+)/,lookbehind:!0},"attr-name":/[\w:-]+(?=\s*!?=|\s*[,)])/,punctuation:/[=(),]/}},{pattern:/\[[^\]]+\]/,inside:n.languages.ruby}],punctuation:/[<>]/}},code:{pattern:/((?:^|\r?\n|\r)[\t ]*(?:[~-]|[&!]?=)).+/,lookbehind:!0,inside:n.languages.ruby},interpolation:{pattern:/#\{[^}]+\}/,inside:{delimiter:{pattern:/^#\{|\}$/,alias:"punctuation"},ruby:{pattern:/[\s\S]+/,inside:n.languages.ruby}}},punctuation:{pattern:/((?:^|\r?\n|\r)[\t ]*)[~=\-&!]+/,lookbehind:!0}};for(var a="((?:^|\\r?\\n|\\r)([\\t ]*)):{{filter_name}}(?:(?:\\r?\\n|\\r)(?:\\2[\\t ].+|\\s*?(?=\\r?\\n|\\r)))+",i=["css",{filter:"coffee",language:"coffeescript"},"erb","javascript","less","markdown","ruby","scss","textile"],o={},s=0,l=i.length;s<l;s++){var u=i[s];u=typeof u=="string"?{filter:u,language:u}:u,n.languages[u.language]&&(o["filter-"+u.filter]={pattern:RegExp(a.replace("{{filter_name}}",function(){return u.filter})),lookbehind:!0,inside:{"filter-name":{pattern:/^:[\w-]+/,alias:"symbol"},text:{pattern:/[\s\S]+/,alias:[u.language,"language-"+u.language],inside:n.languages[u.language]}}})}n.languages.insertBefore("haml","filter",o)})(r)}return hA}var pA,UG;function Hat(){if(UG)return pA;UG=1;var e=Si();pA=t,t.displayName="handlebars",t.aliases=["hbs"];function t(r){r.register(e),(function(n){n.languages.handlebars={comment:/\{\{![\s\S]*?\}\}/,delimiter:{pattern:/^\{\{\{?|\}\}\}?$/,alias:"punctuation"},string:/(["'])(?:\\.|(?!\1)[^\\\r\n])*\1/,number:/\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][+-]?\d+)?/,boolean:/\b(?:false|true)\b/,block:{pattern:/^(\s*(?:~\s*)?)[#\/]\S+?(?=\s*(?:~\s*)?$|\s)/,lookbehind:!0,alias:"keyword"},brackets:{pattern:/\[[^\]]+\]/,inside:{punctuation:/\[|\]/,variable:/[\s\S]+/}},punctuation:/[!"#%&':()*+,.\/;<=>@\[\\\]^`{|}~]/,variable:/[^!"#%&'()*+,\/;<=>@\[\\\]^`{|}~\s]+/},n.hooks.add("before-tokenize",function(a){var i=/\{\{\{[\s\S]+?\}\}\}|\{\{[\s\S]+?\}\}/g;n.languages["markup-templating"].buildPlaceholders(a,"handlebars",i)}),n.hooks.add("after-tokenize",function(a){n.languages["markup-templating"].tokenizePlaceholders(a,"handlebars")}),n.languages.hbs=n.languages.handlebars})(r)}return pA}var fA,$G;function fL(){if($G)return fA;$G=1,fA=e,e.displayName="haskell",e.aliases=["hs"];function e(t){t.languages.haskell={comment:{pattern:/(^|[^-!#$%*+=?&@|~.:<>^\\\/])(?:--(?:(?=.)[^-!#$%*+=?&@|~.:<>^\\\/].*|$)|\{-[\s\S]*?-\})/m,lookbehind:!0},char:{pattern:/'(?:[^\\']|\\(?:[abfnrtv\\"'&]|\^[A-Z@[\]^_]|ACK|BEL|BS|CAN|CR|DC1|DC2|DC3|DC4|DEL|DLE|EM|ENQ|EOT|ESC|ETB|ETX|FF|FS|GS|HT|LF|NAK|NUL|RS|SI|SO|SOH|SP|STX|SUB|SYN|US|VT|\d+|o[0-7]+|x[0-9a-fA-F]+))'/,alias:"string"},string:{pattern:/"(?:[^\\"]|\\(?:\S|\s+\\))*"/,greedy:!0},keyword:/\b(?:case|class|data|deriving|do|else|if|in|infixl|infixr|instance|let|module|newtype|of|primitive|then|type|where)\b/,"import-statement":{pattern:/(^[\t ]*)import\s+(?:qualified\s+)?(?:[A-Z][\w']*)(?:\.[A-Z][\w']*)*(?:\s+as\s+(?:[A-Z][\w']*)(?:\.[A-Z][\w']*)*)?(?:\s+hiding\b)?/m,lookbehind:!0,inside:{keyword:/\b(?:as|hiding|import|qualified)\b/,punctuation:/\./}},builtin:/\b(?:abs|acos|acosh|all|and|any|appendFile|approxRational|asTypeOf|asin|asinh|atan|atan2|atanh|basicIORun|break|catch|ceiling|chr|compare|concat|concatMap|const|cos|cosh|curry|cycle|decodeFloat|denominator|digitToInt|div|divMod|drop|dropWhile|either|elem|encodeFloat|enumFrom|enumFromThen|enumFromThenTo|enumFromTo|error|even|exp|exponent|fail|filter|flip|floatDigits|floatRadix|floatRange|floor|fmap|foldl|foldl1|foldr|foldr1|fromDouble|fromEnum|fromInt|fromInteger|fromIntegral|fromRational|fst|gcd|getChar|getContents|getLine|group|head|id|inRange|index|init|intToDigit|interact|ioError|isAlpha|isAlphaNum|isAscii|isControl|isDenormalized|isDigit|isHexDigit|isIEEE|isInfinite|isLower|isNaN|isNegativeZero|isOctDigit|isPrint|isSpace|isUpper|iterate|last|lcm|length|lex|lexDigits|lexLitChar|lines|log|logBase|lookup|map|mapM|mapM_|max|maxBound|maximum|maybe|min|minBound|minimum|mod|negate|not|notElem|null|numerator|odd|or|ord|otherwise|pack|pi|pred|primExitWith|print|product|properFraction|putChar|putStr|putStrLn|quot|quotRem|range|rangeSize|read|readDec|readFile|readFloat|readHex|readIO|readInt|readList|readLitChar|readLn|readOct|readParen|readSigned|reads|readsPrec|realToFrac|recip|rem|repeat|replicate|return|reverse|round|scaleFloat|scanl|scanl1|scanr|scanr1|seq|sequence|sequence_|show|showChar|showInt|showList|showLitChar|showParen|showSigned|showString|shows|showsPrec|significand|signum|sin|sinh|snd|sort|span|splitAt|sqrt|subtract|succ|sum|tail|take|takeWhile|tan|tanh|threadToIOResult|toEnum|toInt|toInteger|toLower|toRational|toUpper|truncate|uncurry|undefined|unlines|until|unwords|unzip|unzip3|userError|words|writeFile|zip|zip3|zipWith|zipWith3)\b/,number:/\b(?:\d+(?:\.\d+)?(?:e[+-]?\d+)?|0o[0-7]+|0x[0-9a-f]+)\b/i,operator:[{pattern:/`(?:[A-Z][\w']*\.)*[_a-z][\w']*`/,greedy:!0},{pattern:/(\s)\.(?=\s)/,lookbehind:!0},/[-!#$%*+=?&@|~:<>^\\\/][-!#$%*+=?&@|~.:<>^\\\/]*|\.[-!#$%*+=?&@|~.:<>^\\\/]+/],hvariable:{pattern:/\b(?:[A-Z][\w']*\.)*[_a-z][\w']*/,inside:{punctuation:/\./}},constant:{pattern:/\b(?:[A-Z][\w']*\.)*[A-Z][\w']*/,inside:{punctuation:/\./}},punctuation:/[{}[\];(),.:]/},t.languages.hs=t.languages.haskell}return fA}var mA,GG;function Uat(){if(GG)return mA;GG=1,mA=e,e.displayName="haxe",e.aliases=[];function e(t){t.languages.haxe=t.languages.extend("clike",{string:{pattern:/"(?:[^"\\]|\\[\s\S])*"/,greedy:!0},"class-name":[{pattern:/(\b(?:abstract|class|enum|extends|implements|interface|new|typedef)\s+)[A-Z_]\w*/,lookbehind:!0},/\b[A-Z]\w*/],keyword:/\bthis\b|\b(?:abstract|as|break|case|cast|catch|class|continue|default|do|dynamic|else|enum|extends|extern|final|for|from|function|if|implements|import|in|inline|interface|macro|new|null|operator|overload|override|package|private|public|return|static|super|switch|throw|to|try|typedef|untyped|using|var|while)(?!\.)\b/,function:{pattern:/\b[a-z_]\w*(?=\s*(?:<[^<>]*>\s*)?\()/i,greedy:!0},operator:/\.{3}|\+\+|--|&&|\|\||->|=>|(?:<<?|>{1,3}|[-+*/%!=&|^])=?|[?:~]/}),t.languages.insertBefore("haxe","string",{"string-interpolation":{pattern:/'(?:[^'\\]|\\[\s\S])*'/,greedy:!0,inside:{interpolation:{pattern:/(^|[^\\])\$(?:\w+|\{[^{}]+\})/,lookbehind:!0,inside:{"interpolation-punctuation":{pattern:/^\$\{?|\}$/,alias:"punctuation"},expression:{pattern:/[\s\S]+/,inside:t.languages.haxe}}},string:/[\s\S]+/}}}),t.languages.insertBefore("haxe","class-name",{regex:{pattern:/~\/(?:[^\/\\\r\n]|\\.)+\/[a-z]*/,greedy:!0,inside:{"regex-flags":/\b[a-z]+$/,"regex-source":{pattern:/^(~\/)[\s\S]+(?=\/$)/,lookbehind:!0,alias:"language-regex",inside:t.languages.regex},"regex-delimiter":/^~\/|\/$/}}}),t.languages.insertBefore("haxe","keyword",{preprocessor:{pattern:/#(?:else|elseif|end|if)\b.*/,alias:"property"},metadata:{pattern:/@:?[\w.]+/,alias:"symbol"},reification:{pattern:/\$(?:\w+|(?=\{))/,alias:"important"}})}return mA}var gA,jG;function $at(){if(jG)return gA;jG=1,gA=e,e.displayName="hcl",e.aliases=[];function e(t){t.languages.hcl={comment:/(?:\/\/|#).*|\/\*[\s\S]*?(?:\*\/|$)/,heredoc:{pattern:/<<-?(\w+\b)[\s\S]*?^[ \t]*\1/m,greedy:!0,alias:"string"},keyword:[{pattern:/(?:data|resource)\s+(?:"(?:\\[\s\S]|[^\\"])*")(?=\s+"[\w-]+"\s+\{)/i,inside:{type:{pattern:/(resource|data|\s+)(?:"(?:\\[\s\S]|[^\\"])*")/i,lookbehind:!0,alias:"variable"}}},{pattern:/(?:backend|module|output|provider|provisioner|variable)\s+(?:[\w-]+|"(?:\\[\s\S]|[^\\"])*")\s+(?=\{)/i,inside:{type:{pattern:/(backend|module|output|provider|provisioner|variable)\s+(?:[\w-]+|"(?:\\[\s\S]|[^\\"])*")\s+/i,lookbehind:!0,alias:"variable"}}},/[\w-]+(?=\s+\{)/],property:[/[-\w\.]+(?=\s*=(?!=))/,/"(?:\\[\s\S]|[^\\"])+"(?=\s*[:=])/],string:{pattern:/"(?:[^\\$"]|\\[\s\S]|\$(?:(?=")|\$+(?!\$)|[^"${])|\$\{(?:[^{}"]|"(?:[^\\"]|\\[\s\S])*")*\})*"/,greedy:!0,inside:{interpolation:{pattern:/(^|[^$])\$\{(?:[^{}"]|"(?:[^\\"]|\\[\s\S])*")*\}/,lookbehind:!0,inside:{type:{pattern:/(\b(?:count|data|local|module|path|self|terraform|var)\b\.)[\w\*]+/i,lookbehind:!0,alias:"variable"},keyword:/\b(?:count|data|local|module|path|self|terraform|var)\b/i,function:/\w+(?=\()/,string:{pattern:/"(?:\\[\s\S]|[^\\"])*"/,greedy:!0},number:/\b0x[\da-f]+\b|\b\d+(?:\.\d*)?(?:e[+-]?\d+)?/i,punctuation:/[!\$#%&'()*+,.\/;<=>@\[\\\]^`{|}~?:]/}}}},number:/\b0x[\da-f]+\b|\b\d+(?:\.\d*)?(?:e[+-]?\d+)?/i,boolean:/\b(?:false|true)\b/i,punctuation:/[=\[\]{}]/}}return gA}var bA,qG;function Gat(){if(qG)return bA;qG=1;var e=pd();bA=t,t.displayName="hlsl",t.aliases=[];function t(r){r.register(e),r.languages.hlsl=r.languages.extend("c",{"class-name":[r.languages.c["class-name"],/\b(?:AppendStructuredBuffer|BlendState|Buffer|ByteAddressBuffer|CompileShader|ComputeShader|ConsumeStructuredBuffer|DepthStencilState|DepthStencilView|DomainShader|GeometryShader|Hullshader|InputPatch|LineStream|OutputPatch|PixelShader|PointStream|RWBuffer|RWByteAddressBuffer|RWStructuredBuffer|RWTexture(?:1D|1DArray|2D|2DArray|3D)|RasterizerState|RenderTargetView|SamplerComparisonState|SamplerState|StructuredBuffer|Texture(?:1D|1DArray|2D|2DArray|2DMS|2DMSArray|3D|Cube|CubeArray)|TriangleStream|VertexShader)\b/],keyword:[/\b(?:asm|asm_fragment|auto|break|case|catch|cbuffer|centroid|char|class|column_major|compile|compile_fragment|const|const_cast|continue|default|delete|discard|do|dynamic_cast|else|enum|explicit|export|extern|for|friend|fxgroup|goto|groupshared|if|in|inline|inout|interface|line|lineadj|linear|long|matrix|mutable|namespace|new|nointerpolation|noperspective|operator|out|packoffset|pass|pixelfragment|point|precise|private|protected|public|register|reinterpret_cast|return|row_major|sample|sampler|shared|short|signed|sizeof|snorm|stateblock|stateblock_state|static|static_cast|string|struct|switch|tbuffer|technique|technique10|technique11|template|texture|this|throw|triangle|triangleadj|try|typedef|typename|uniform|union|unorm|unsigned|using|vector|vertexfragment|virtual|void|volatile|while)\b/,/\b(?:bool|double|dword|float|half|int|min(?:10float|12int|16(?:float|int|uint))|uint)(?:[1-4](?:x[1-4])?)?\b/],number:/(?:(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[eE][+-]?\d+)?|\b0x[\da-fA-F]+)[fFhHlLuU]?\b/,boolean:/\b(?:false|true)\b/})}return bA}var yA,WG;function jat(){if(WG)return yA;WG=1,yA=e,e.displayName="hoon",e.aliases=[];function e(t){t.languages.hoon={comment:{pattern:/::.*/,greedy:!0},string:{pattern:/"[^"]*"|'[^']*'/,greedy:!0},constant:/%(?:\.[ny]|[\w-]+)/,"class-name":/@(?:[a-z0-9-]*[a-z0-9])?|\*/i,function:/(?:\+[-+] {2})?(?:[a-z](?:[a-z0-9-]*[a-z0-9])?)/,keyword:/\.[\^\+\*=\?]|![><:\.=\?!]|=[>|:,\.\-\^<+;/~\*\?]|\?[>|:\.\-\^<\+&~=@!]|\|[\$_%:\.\-\^~\*=@\?]|\+[|\$\+\*]|:[_\-\^\+~\*]|%[_:\.\-\^\+~\*=]|\^[|:\.\-\+&~\*=\?]|\$[|_%:<>\-\^&~@=\?]|;[:<\+;\/~\*=]|~[>|\$_%<\+\/&=\?!]|--|==/}}return yA}var vA,VG;function qat(){if(VG)return vA;VG=1,vA=e,e.displayName="hpkp",e.aliases=[];function e(t){t.languages.hpkp={directive:{pattern:/\b(?:includeSubDomains|max-age|pin-sha256|preload|report-to|report-uri|strict)(?=[\s;=]|$)/i,alias:"property"},operator:/=/,punctuation:/;/}}return vA}var SA,KG;function Wat(){if(KG)return SA;KG=1,SA=e,e.displayName="hsts",e.aliases=[];function e(t){t.languages.hsts={directive:{pattern:/\b(?:includeSubDomains|max-age|preload)(?=[\s;=]|$)/i,alias:"property"},operator:/=/,punctuation:/;/}}return SA}var EA,YG;function Vat(){if(YG)return EA;YG=1,EA=e,e.displayName="http",e.aliases=[];function e(t){(function(r){function n(p){return RegExp("(^(?:"+p+"):[ 	]*(?![ 	]))[^]+","i")}r.languages.http={"request-line":{pattern:/^(?:CONNECT|DELETE|GET|HEAD|OPTIONS|PATCH|POST|PRI|PUT|SEARCH|TRACE)\s(?:https?:\/\/|\/)\S*\sHTTP\/[\d.]+/m,inside:{method:{pattern:/^[A-Z]+\b/,alias:"property"},"request-target":{pattern:/^(\s)(?:https?:\/\/|\/)\S*(?=\s)/,lookbehind:!0,alias:"url",inside:r.languages.uri},"http-version":{pattern:/^(\s)HTTP\/[\d.]+/,lookbehind:!0,alias:"property"}}},"response-status":{pattern:/^HTTP\/[\d.]+ \d+ .+/m,inside:{"http-version":{pattern:/^HTTP\/[\d.]+/,alias:"property"},"status-code":{pattern:/^(\s)\d+(?=\s)/,lookbehind:!0,alias:"number"},"reason-phrase":{pattern:/^(\s).+/,lookbehind:!0,alias:"string"}}},header:{pattern:/^[\w-]+:.+(?:(?:\r\n?|\n)[ \t].+)*/m,inside:{"header-value":[{pattern:n(/Content-Security-Policy/.source),lookbehind:!0,alias:["csp","languages-csp"],inside:r.languages.csp},{pattern:n(/Public-Key-Pins(?:-Report-Only)?/.source),lookbehind:!0,alias:["hpkp","languages-hpkp"],inside:r.languages.hpkp},{pattern:n(/Strict-Transport-Security/.source),lookbehind:!0,alias:["hsts","languages-hsts"],inside:r.languages.hsts},{pattern:n(/[^:]+/.source),lookbehind:!0}],"header-name":{pattern:/^[^:]+/,alias:"keyword"},punctuation:/^:/}}};var a=r.languages,i={"application/javascript":a.javascript,"application/json":a.json||a.javascript,"application/xml":a.xml,"text/xml":a.xml,"text/html":a.html,"text/css":a.css,"text/plain":a.plain},o={"application/json":!0,"application/xml":!0};function s(p){var f=p.replace(/^[a-z]+\//,""),m="\\w+/(?:[\\w.-]+\\+)+"+f+"(?![+\\w.-])";return"(?:"+p+"|"+m+")"}var l;for(var u in i)if(i[u]){l=l||{};var d=o[u]?s(u):u;l[u.replace(/\//g,"-")]={pattern:RegExp("("+/content-type:\s*/.source+d+/(?:(?:\r\n?|\n)[\w-].*)*(?:\r(?:\n|(?!\n))|\n)/.source+")"+/[^ \t\w-][\s\S]*/.source,"i"),lookbehind:!0,inside:i[u]}}l&&r.languages.insertBefore("http","header",l)})(t)}return EA}var xA,JG;function Kat(){if(JG)return xA;JG=1,xA=e,e.displayName="ichigojam",e.aliases=[];function e(t){t.languages.ichigojam={comment:/(?:\B'|REM)(?:[^\n\r]*)/i,string:{pattern:/"(?:""|[!#$%&'()*,\/:;<=>?^\w +\-.])*"/,greedy:!0},number:/\B#[0-9A-F]+|\B`[01]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i,keyword:/\b(?:BEEP|BPS|CASE|CLEAR|CLK|CLO|CLP|CLS|CLT|CLV|CONT|COPY|ELSE|END|FILE|FILES|FOR|GOSUB|GOTO|GSB|IF|INPUT|KBD|LED|LET|LIST|LOAD|LOCATE|LRUN|NEW|NEXT|OUT|PLAY|POKE|PRINT|PWM|REM|RENUM|RESET|RETURN|RIGHT|RTN|RUN|SAVE|SCROLL|SLEEP|SRND|STEP|STOP|SUB|TEMPO|THEN|TO|UART|VIDEO|WAIT)(?:\$|\b)/i,function:/\b(?:ABS|ANA|ASC|BIN|BTN|DEC|END|FREE|HELP|HEX|I2CR|I2CW|IN|INKEY|LEN|LINE|PEEK|RND|SCR|SOUND|STR|TICK|USR|VER|VPEEK|ZER)(?:\$|\b)/i,label:/(?:\B@\S+)/,operator:/<[=>]?|>=?|\|\||&&|[+\-*\/=|&^~!]|\b(?:AND|NOT|OR)\b/i,punctuation:/[\[,;:()\]]/}}return xA}var wA,XG;function Yat(){if(XG)return wA;XG=1,wA=e,e.displayName="icon",e.aliases=[];function e(t){t.languages.icon={comment:/#.*/,string:{pattern:/(["'])(?:(?!\1)[^\\\r\n_]|\\.|_(?!\1)(?:\r\n|[\s\S]))*\1/,greedy:!0},number:/\b(?:\d+r[a-z\d]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b|\.\d+\b/i,"builtin-keyword":{pattern:/&(?:allocated|ascii|clock|collections|cset|current|date|dateline|digits|dump|e|error(?:number|text|value)?|errout|fail|features|file|host|input|lcase|letters|level|line|main|null|output|phi|pi|pos|progname|random|regions|source|storage|subject|time|trace|ucase|version)\b/,alias:"variable"},directive:{pattern:/\$\w+/,alias:"builtin"},keyword:/\b(?:break|by|case|create|default|do|else|end|every|fail|global|if|initial|invocable|link|local|next|not|of|procedure|record|repeat|return|static|suspend|then|to|until|while)\b/,function:/\b(?!\d)\w+(?=\s*[({]|\s*!\s*\[)/,operator:/[+-]:(?!=)|(?:[\/?@^%&]|\+\+?|--?|==?=?|~==?=?|\*\*?|\|\|\|?|<(?:->?|<?=?)|>>?=?)(?::=)?|:(?:=:?)?|[!.\\|~]/,punctuation:/[\[\](){},;]/}}return wA}var kA,ZG;function Jat(){if(ZG)return kA;ZG=1,kA=e,e.displayName="icuMessageFormat",e.aliases=[];function e(t){(function(r){function n(u,d){return d<=0?/[]/.source:u.replace(/<SELF>/g,function(){return n(u,d-1)})}var a=/'[{}:=,](?:[^']|'')*'(?!')/,i={pattern:/''/,greedy:!0,alias:"operator"},o={pattern:a,greedy:!0,inside:{escape:i}},s=n(/\{(?:[^{}']|'(?![{},'])|''|<STR>|<SELF>)*\}/.source.replace(/<STR>/g,function(){return a.source}),8),l={pattern:RegExp(s),inside:{message:{pattern:/^(\{)[\s\S]+(?=\}$)/,lookbehind:!0,inside:null},"message-delimiter":{pattern:/./,alias:"punctuation"}}};r.languages["icu-message-format"]={argument:{pattern:RegExp(s),greedy:!0,inside:{content:{pattern:/^(\{)[\s\S]+(?=\}$)/,lookbehind:!0,inside:{"argument-name":{pattern:/^(\s*)[^{}:=,\s]+/,lookbehind:!0},"choice-style":{pattern:/^(\s*,\s*choice\s*,\s*)\S(?:[\s\S]*\S)?/,lookbehind:!0,inside:{punctuation:/\|/,range:{pattern:/^(\s*)[+-]?(?:\d+(?:\.\d*)?|\u221e)\s*[<#\u2264]/,lookbehind:!0,inside:{operator:/[<#\u2264]/,number:/\S+/}},rest:null}},"plural-style":{pattern:/^(\s*,\s*(?:plural|selectordinal)\s*,\s*)\S(?:[\s\S]*\S)?/,lookbehind:!0,inside:{offset:/^offset:\s*\d+/,"nested-message":l,selector:{pattern:/=\d+|[^{}:=,\s]+/,inside:{keyword:/^(?:few|many|one|other|two|zero)$/}}}},"select-style":{pattern:/^(\s*,\s*select\s*,\s*)\S(?:[\s\S]*\S)?/,lookbehind:!0,inside:{"nested-message":l,selector:{pattern:/[^{}:=,\s]+/,inside:{keyword:/^other$/}}}},keyword:/\b(?:choice|plural|select|selectordinal)\b/,"arg-type":{pattern:/\b(?:date|duration|number|ordinal|spellout|time)\b/,alias:"keyword"},"arg-skeleton":{pattern:/(,\s*)::[^{}:=,\s]+/,lookbehind:!0},"arg-style":{pattern:/(,\s*)(?:currency|full|integer|long|medium|percent|short)(?=\s*$)/,lookbehind:!0},"arg-style-text":{pattern:RegExp(/(^\s*,\s*(?=\S))/.source+n(/(?:[^{}']|'[^']*'|\{(?:<SELF>)?\})+/.source,8)+"$"),lookbehind:!0,alias:"string"},punctuation:/,/}},"argument-delimiter":{pattern:/./,alias:"operator"}}},escape:i,string:o},l.inside.message.inside=r.languages["icu-message-format"],r.languages["icu-message-format"].argument.inside.content.inside["choice-style"].inside.rest=r.languages["icu-message-format"]})(t)}return kA}var CA,QG;function Xat(){if(QG)return CA;QG=1;var e=fL();CA=t,t.displayName="idris",t.aliases=["idr"];function t(r){r.register(e),r.languages.idris=r.languages.extend("haskell",{comment:{pattern:/(?:(?:--|\|\|\|).*$|\{-[\s\S]*?-\})/m},keyword:/\b(?:Type|case|class|codata|constructor|corecord|data|do|dsl|else|export|if|implementation|implicit|import|impossible|in|infix|infixl|infixr|instance|interface|let|module|mutual|namespace|of|parameters|partial|postulate|private|proof|public|quoteGoal|record|rewrite|syntax|then|total|using|where|with)\b/,builtin:void 0}),r.languages.insertBefore("idris","keyword",{"import-statement":{pattern:/(^\s*import\s+)(?:[A-Z][\w']*)(?:\.[A-Z][\w']*)*/m,lookbehind:!0,inside:{punctuation:/\./}}}),r.languages.idr=r.languages.idris}return CA}var TA,ej;function Zat(){if(ej)return TA;ej=1,TA=e,e.displayName="iecst",e.aliases=[];function e(t){t.languages.iecst={comment:[{pattern:/(^|[^\\])(?:\/\*[\s\S]*?(?:\*\/|$)|\(\*[\s\S]*?(?:\*\)|$)|\{[\s\S]*?(?:\}|$))/,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],string:{pattern:/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0},keyword:[/\b(?:END_)?(?:PROGRAM|CONFIGURATION|INTERFACE|FUNCTION_BLOCK|FUNCTION|ACTION|TRANSITION|TYPE|STRUCT|(?:INITIAL_)?STEP|NAMESPACE|LIBRARY|CHANNEL|FOLDER|RESOURCE|VAR_(?:ACCESS|CONFIG|EXTERNAL|GLOBAL|INPUT|IN_OUT|OUTPUT|TEMP)|VAR|METHOD|PROPERTY)\b/i,/\b(?:AT|BY|(?:END_)?(?:CASE|FOR|IF|REPEAT|WHILE)|CONSTANT|CONTINUE|DO|ELSE|ELSIF|EXIT|EXTENDS|FROM|GET|GOTO|IMPLEMENTS|JMP|NON_RETAIN|OF|PRIVATE|PROTECTED|PUBLIC|RETAIN|RETURN|SET|TASK|THEN|TO|UNTIL|USING|WITH|__CATCH|__ENDTRY|__FINALLY|__TRY)\b/],"class-name":/\b(?:ANY|ARRAY|BOOL|BYTE|U?(?:D|L|S)?INT|(?:D|L)?WORD|DATE(?:_AND_TIME)?|DT|L?REAL|POINTER|STRING|TIME(?:_OF_DAY)?|TOD)\b/,address:{pattern:/%[IQM][XBWDL][\d.]*|%[IQ][\d.]*/,alias:"symbol"},number:/\b(?:16#[\da-f]+|2#[01_]+|0x[\da-f]+)\b|\b(?:D|DT|T|TOD)#[\d_shmd:]*|\b[A-Z]*#[\d.,_]*|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,boolean:/\b(?:FALSE|NULL|TRUE)\b/,operator:/S?R?:?=>?|&&?|\*\*?|<[=>]?|>=?|[-:^/+#]|\b(?:AND|EQ|EXPT|GE|GT|LE|LT|MOD|NE|NOT|OR|XOR)\b/,function:/\b[a-z_]\w*(?=\s*\()/i,punctuation:/[()[\].,;]/}}return TA}var AA,tj;function Qat(){if(tj)return AA;tj=1,AA=e,e.displayName="ignore",e.aliases=["gitignore","hgignore","npmignore"];function e(t){(function(r){r.languages.ignore={comment:/^#.*/m,entry:{pattern:/\S(?:.*(?:(?:\\ )|\S))?/,alias:"string",inside:{operator:/^!|\*\*?|\?/,regex:{pattern:/(^|[^\\])\[[^\[\]]*\]/,lookbehind:!0},punctuation:/\//}}},r.languages.gitignore=r.languages.ignore,r.languages.hgignore=r.languages.ignore,r.languages.npmignore=r.languages.ignore})(t)}return AA}var FA,rj;function eit(){if(rj)return FA;rj=1,FA=e,e.displayName="inform7",e.aliases=[];function e(t){t.languages.inform7={string:{pattern:/"[^"]*"/,inside:{substitution:{pattern:/\[[^\[\]]+\]/,inside:{delimiter:{pattern:/\[|\]/,alias:"punctuation"}}}}},comment:{pattern:/\[[^\[\]]+\]/,greedy:!0},title:{pattern:/^[ \t]*(?:book|chapter|part(?! of)|section|table|volume)\b.+/im,alias:"important"},number:{pattern:/(^|[^-])(?:\b\d+(?:\.\d+)?(?:\^\d+)?(?:(?!\d)\w+)?|\b(?:eight|eleven|five|four|nine|one|seven|six|ten|three|twelve|two))\b(?!-)/i,lookbehind:!0},verb:{pattern:/(^|[^-])\b(?:answering|applying to|are|asking|attacking|be(?:ing)?|burning|buying|called|carries|carry(?! out)|carrying|climbing|closing|conceal(?:ing|s)?|consulting|contain(?:ing|s)?|cutting|drinking|dropping|eating|enclos(?:es?|ing)|entering|examining|exiting|getting|giving|going|ha(?:s|ve|ving)|hold(?:ing|s)?|impl(?:ies|y)|incorporat(?:es?|ing)|inserting|is|jumping|kissing|listening|locking|looking|mean(?:ing|s)?|opening|provid(?:es?|ing)|pulling|pushing|putting|relat(?:es?|ing)|removing|searching|see(?:ing|s)?|setting|showing|singing|sleeping|smelling|squeezing|support(?:ing|s)?|swearing|switching|taking|tasting|telling|thinking|throwing|touching|turning|tying|unlock(?:ing|s)?|var(?:ies|y|ying)|waiting|waking|waving|wear(?:ing|s)?)\b(?!-)/i,lookbehind:!0,alias:"operator"},keyword:{pattern:/(^|[^-])\b(?:after|before|carry out|check|continue the action|definition(?= *:)|do nothing|else|end (?:if|the story|unless)|every turn|if|include|instead(?: of)?|let|move|no|now|otherwise|repeat|report|resume the story|rule for|running through|say(?:ing)?|stop the action|test|try(?:ing)?|understand|unless|use|when|while|yes)\b(?!-)/i,lookbehind:!0},property:{pattern:/(^|[^-])\b(?:adjacent(?! to)|carried|closed|concealed|contained|dark|described|edible|empty|enclosed|enterable|even|female|fixed in place|full|handled|held|improper-named|incorporated|inedible|invisible|lighted|lit|lock(?:able|ed)|male|marked for listing|mentioned|negative|neuter|non-(?:empty|full|recurring)|odd|opaque|open(?:able)?|plural-named|portable|positive|privately-named|proper-named|provided|publically-named|pushable between rooms|recurring|related|rubbing|scenery|seen|singular-named|supported|swinging|switch(?:able|ed(?: off| on)?)|touch(?:able|ed)|transparent|unconcealed|undescribed|unlit|unlocked|unmarked for listing|unmentioned|unopenable|untouchable|unvisited|variable|visible|visited|wearable|worn)\b(?!-)/i,lookbehind:!0,alias:"symbol"},position:{pattern:/(^|[^-])\b(?:above|adjacent to|back side of|below|between|down|east|everywhere|front side|here|in|inside(?: from)?|north(?:east|west)?|nowhere|on(?: top of)?|other side|outside(?: from)?|parts? of|regionally in|south(?:east|west)?|through|up|west|within)\b(?!-)/i,lookbehind:!0,alias:"keyword"},type:{pattern:/(^|[^-])\b(?:actions?|activit(?:ies|y)|actors?|animals?|backdrops?|containers?|devices?|directions?|doors?|holders?|kinds?|lists?|m[ae]n|nobody|nothing|nouns?|numbers?|objects?|people|persons?|player(?:'s holdall)?|regions?|relations?|rooms?|rule(?:book)?s?|scenes?|someone|something|supporters?|tables?|texts?|things?|time|vehicles?|wom[ae]n)\b(?!-)/i,lookbehind:!0,alias:"variable"},punctuation:/[.,:;(){}]/},t.languages.inform7.string.inside.substitution.inside.rest=t.languages.inform7,t.languages.inform7.string.inside.substitution.inside.rest.text={pattern:/\S(?:\s*\S)*/,alias:"comment"}}return FA}var _A,nj;function tit(){if(nj)return _A;nj=1,_A=e,e.displayName="ini",e.aliases=[];function e(t){t.languages.ini={comment:{pattern:/(^[ \f\t\v]*)[#;][^\n\r]*/m,lookbehind:!0},section:{pattern:/(^[ \f\t\v]*)\[[^\n\r\]]*\]?/m,lookbehind:!0,inside:{"section-name":{pattern:/(^\[[ \f\t\v]*)[^ \f\t\v\]]+(?:[ \f\t\v]+[^ \f\t\v\]]+)*/,lookbehind:!0,alias:"selector"},punctuation:/\[|\]/}},key:{pattern:/(^[ \f\t\v]*)[^ \f\n\r\t\v=]+(?:[ \f\t\v]+[^ \f\n\r\t\v=]+)*(?=[ \f\t\v]*=)/m,lookbehind:!0,alias:"attr-name"},value:{pattern:/(=[ \f\t\v]*)[^ \f\n\r\t\v]+(?:[ \f\t\v]+[^ \f\n\r\t\v]+)*/,lookbehind:!0,alias:"attr-value",inside:{"inner-value":{pattern:/^("|').+(?=\1$)/,lookbehind:!0}}},punctuation:/=/}}return _A}var RA,aj;function rit(){if(aj)return RA;aj=1,RA=e,e.displayName="io",e.aliases=[];function e(t){t.languages.io={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?(?:\*\/|$)|\/\/.*|#.*)/,lookbehind:!0,greedy:!0},"triple-quoted-string":{pattern:/"""(?:\\[\s\S]|(?!""")[^\\])*"""/,greedy:!0,alias:"string"},string:{pattern:/"(?:\\.|[^\\\r\n"])*"/,greedy:!0},keyword:/\b(?:activate|activeCoroCount|asString|block|break|call|catch|clone|collectGarbage|compileString|continue|do|doFile|doMessage|doString|else|elseif|exit|for|foreach|forward|getEnvironmentVariable|getSlot|hasSlot|if|ifFalse|ifNil|ifNilEval|ifTrue|isActive|isNil|isResumable|list|message|method|parent|pass|pause|perform|performWithArgList|print|println|proto|raise|raiseResumable|removeSlot|resend|resume|schedulerSleepSeconds|self|sender|setSchedulerSleepSeconds|setSlot|shallowCopy|slotNames|super|system|then|thisBlock|thisContext|try|type|uniqueId|updateSlot|wait|while|write|yield)\b/,builtin:/\b(?:Array|AudioDevice|AudioMixer|BigNum|Block|Box|Buffer|CFunction|CGI|Color|Curses|DBM|DNSResolver|DOConnection|DOProxy|DOServer|Date|Directory|Duration|DynLib|Error|Exception|FFT|File|Fnmatch|Font|Future|GL|GLE|GLScissor|GLU|GLUCylinder|GLUQuadric|GLUSphere|GLUT|Host|Image|Importer|LinkList|List|Lobby|Locals|MD5|MP3Decoder|MP3Encoder|Map|Message|Movie|Notification|Number|Object|OpenGL|Point|Protos|Random|Regex|SGML|SGMLElement|SGMLParser|SQLite|Sequence|Server|ShowMessage|SleepyCat|SleepyCatCursor|Socket|SocketManager|Sound|Soup|Store|String|Tree|UDPSender|UPDReceiver|URL|User|Warning|WeakLink)\b/,boolean:/\b(?:false|nil|true)\b/,number:/\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e-?\d+)?/i,operator:/[=!*/%+\-^&|]=|>>?=?|<<?=?|:?:?=|\+\+?|--?|\*\*?|\/\/?|%|\|\|?|&&?|\b(?:and|not|or|return)\b|@@?|\?\??|\.\./,punctuation:/[{}[\];(),.:]/}}return RA}var DA,ij;function nit(){if(ij)return DA;ij=1,DA=e,e.displayName="j",e.aliases=[];function e(t){t.languages.j={comment:{pattern:/\bNB\..*/,greedy:!0},string:{pattern:/'(?:''|[^'\r\n])*'/,greedy:!0},keyword:/\b(?:(?:CR|LF|adverb|conjunction|def|define|dyad|monad|noun|verb)\b|(?:assert|break|case|catch[dt]?|continue|do|else|elseif|end|fcase|for|for_\w+|goto_\w+|if|label_\w+|return|select|throw|try|while|whilst)\.)/,verb:{pattern:/(?!\^:|;\.|[=!][.:])(?:\{(?:\.|::?)?|p(?:\.\.?|:)|[=!\]]|[<>+*\-%$|,#][.:]?|[?^]\.?|[;\[]:?|[~}"i][.:]|[ACeEIjLor]\.|(?:[_\/\\qsux]|_?\d):)/,alias:"keyword"},number:/\b_?(?:(?!\d:)\d+(?:\.\d+)?(?:(?:ad|ar|[ejpx])_?\d+(?:\.\d+)?)*(?:b_?[\da-z]+(?:\.[\da-z]+)?)?|_\b(?!\.))/,adverb:{pattern:/[~}]|[\/\\]\.?|[bfM]\.|t[.:]/,alias:"builtin"},operator:/[=a][.:]|_\./,conjunction:{pattern:/&(?:\.:?|:)?|[.:@][.:]?|[!D][.:]|[;dHT]\.|`:?|[\^LS]:|"/,alias:"variable"},punctuation:/[()]/}}return DA}var NA,oj;function mL(){if(oj)return NA;oj=1,NA=e,e.displayName="java",e.aliases=[];function e(t){(function(r){var n=/\b(?:abstract|assert|boolean|break|byte|case|catch|char|class|const|continue|default|do|double|else|enum|exports|extends|final|finally|float|for|goto|if|implements|import|instanceof|int|interface|long|module|native|new|non-sealed|null|open|opens|package|permits|private|protected|provides|public|record|requires|return|sealed|short|static|strictfp|super|switch|synchronized|this|throw|throws|to|transient|transitive|try|uses|var|void|volatile|while|with|yield)\b/,a=/(^|[^\w.])(?:[a-z]\w*\s*\.\s*)*(?:[A-Z]\w*\s*\.\s*)*/.source,i={pattern:RegExp(a+/[A-Z](?:[\d_A-Z]*[a-z]\w*)?\b/.source),lookbehind:!0,inside:{namespace:{pattern:/^[a-z]\w*(?:\s*\.\s*[a-z]\w*)*(?:\s*\.)?/,inside:{punctuation:/\./}},punctuation:/\./}};r.languages.java=r.languages.extend("clike",{string:{pattern:/(^|[^\\])"(?:\\.|[^"\\\r\n])*"/,lookbehind:!0,greedy:!0},"class-name":[i,{pattern:RegExp(a+/[A-Z]\w*(?=\s+\w+\s*[;,=()])/.source),lookbehind:!0,inside:i.inside}],keyword:n,function:[r.languages.clike.function,{pattern:/(::\s*)[a-z_]\w*/,lookbehind:!0}],number:/\b0b[01][01_]*L?\b|\b0x(?:\.[\da-f_p+-]+|[\da-f_]+(?:\.[\da-f_p+-]+)?)\b|(?:\b\d[\d_]*(?:\.[\d_]*)?|\B\.\d[\d_]*)(?:e[+-]?\d[\d_]*)?[dfl]?/i,operator:{pattern:/(^|[^.])(?:<<=?|>>>?=?|->|--|\+\+|&&|\|\||::|[?:~]|[-+*/%&|^!=<>]=?)/m,lookbehind:!0}}),r.languages.insertBefore("java","string",{"triple-quoted-string":{pattern:/"""[ \t]*[\r\n](?:(?:"|"")?(?:\\.|[^"\\]))*"""/,greedy:!0,alias:"string"},char:{pattern:/'(?:\\.|[^'\\\r\n]){1,6}'/,greedy:!0}}),r.languages.insertBefore("java","class-name",{annotation:{pattern:/(^|[^.])@\w+(?:\s*\.\s*\w+)*/,lookbehind:!0,alias:"punctuation"},generics:{pattern:/<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&)|<(?:[\w\s,.?]|&(?!&))*>)*>)*>)*>/,inside:{"class-name":i,keyword:n,punctuation:/[<>(),.:]/,operator:/[?&|]/}},namespace:{pattern:RegExp(/(\b(?:exports|import(?:\s+static)?|module|open|opens|package|provides|requires|to|transitive|uses|with)\s+)(?!<keyword>)[a-z]\w*(?:\.[a-z]\w*)*\.?/.source.replace(/<keyword>/g,function(){return n.source})),lookbehind:!0,inside:{punctuation:/\./}}})})(t)}return NA}var LA,sj;function aS(){if(sj)return LA;sj=1,LA=e,e.displayName="javadoclike",e.aliases=[];function e(t){(function(r){var n=r.languages.javadoclike={parameter:{pattern:/(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*@(?:arg|arguments|param)\s+)\w+/m,lookbehind:!0},keyword:{pattern:/(^[\t ]*(?:\/{3}|\*|\/\*\*)\s*|\{)@[a-z][a-zA-Z-]+\b/m,lookbehind:!0},punctuation:/[{}]/};function a(o,s){var l="doc-comment",u=r.languages[o];if(u){var d=u[l];if(!d){var p={};p[l]={pattern:/(^|[^\\])\/\*\*[^/][\s\S]*?(?:\*\/|$)/,lookbehind:!0,alias:"comment"},u=r.languages.insertBefore(o,"comment",p),d=u[l]}if(d instanceof RegExp&&(d=u[l]={pattern:d}),Array.isArray(d))for(var f=0,m=d.length;f<m;f++)d[f]instanceof RegExp&&(d[f]={pattern:d[f]}),s(d[f]);else s(d)}}function i(o,s){typeof o=="string"&&(o=[o]),o.forEach(function(l){a(l,function(u){u.inside||(u.inside={}),u.inside.rest=s})})}Object.defineProperty(n,"addSupport",{value:i}),n.addSupport(["java","javascript","php"],n)})(t)}return LA}var MA,lj;function ait(){if(lj)return MA;lj=1;var e=mL(),t=aS();MA=r,r.displayName="javadoc",r.aliases=[];function r(n){n.register(e),n.register(t),(function(a){var i=/(^(?:[\t ]*(?:\*\s*)*))[^*\s].*$/m,o=/#\s*\w+(?:\s*\([^()]*\))?/.source,s=/(?:\b[a-zA-Z]\w+\s*\.\s*)*\b[A-Z]\w*(?:\s*<mem>)?|<mem>/.source.replace(/<mem>/g,function(){return o});a.languages.javadoc=a.languages.extend("javadoclike",{}),a.languages.insertBefore("javadoc","keyword",{reference:{pattern:RegExp(/(@(?:exception|link|linkplain|see|throws|value)\s+(?:\*\s*)?)/.source+"(?:"+s+")"),lookbehind:!0,inside:{function:{pattern:/(#\s*)\w+(?=\s*\()/,lookbehind:!0},field:{pattern:/(#\s*)\w+/,lookbehind:!0},namespace:{pattern:/\b(?:[a-z]\w*\s*\.\s*)+/,inside:{punctuation:/\./}},"class-name":/\b[A-Z]\w*/,keyword:a.languages.java.keyword,punctuation:/[#()[\],.]/}},"class-name":{pattern:/(@param\s+)<[A-Z]\w*>/,lookbehind:!0,inside:{punctuation:/[.<>]/}},"code-section":[{pattern:/(\{@code\s+(?!\s))(?:[^\s{}]|\s+(?![\s}])|\{(?:[^{}]|\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})*\})*\})+(?=\s*\})/,lookbehind:!0,inside:{code:{pattern:i,lookbehind:!0,inside:a.languages.java,alias:"language-java"}}},{pattern:/(<(code|pre|tt)>(?!<code>)\s*)\S(?:\S|\s+\S)*?(?=\s*<\/\2>)/,lookbehind:!0,inside:{line:{pattern:i,lookbehind:!0,inside:{tag:a.languages.markup.tag,entity:a.languages.markup.entity,code:{pattern:/.+/,inside:a.languages.java,alias:"language-java"}}}}}],tag:a.languages.markup.tag,entity:a.languages.markup.entity}),a.languages.javadoclike.addSupport("java",a.languages.javadoc)})(n)}return MA}var IA,uj;function iit(){if(uj)return IA;uj=1,IA=e,e.displayName="javastacktrace",e.aliases=[];function e(t){t.languages.javastacktrace={summary:{pattern:/^([\t ]*)(?:(?:Caused by:|Suppressed:|Exception in thread "[^"]*")[\t ]+)?[\w$.]+(?::.*)?$/m,lookbehind:!0,inside:{keyword:{pattern:/^([\t ]*)(?:(?:Caused by|Suppressed)(?=:)|Exception in thread)/m,lookbehind:!0},string:{pattern:/^(\s*)"[^"]*"/,lookbehind:!0},exceptions:{pattern:/^(:?\s*)[\w$.]+(?=:|$)/,lookbehind:!0,inside:{"class-name":/[\w$]+$/,namespace:/\b[a-z]\w*\b/,punctuation:/\./}},message:{pattern:/(:\s*)\S.*/,lookbehind:!0,alias:"string"},punctuation:/:/}},"stack-frame":{pattern:/^([\t ]*)at (?:[\w$./]|@[\w$.+-]*\/)+(?:<init>)?\([^()]*\)/m,lookbehind:!0,inside:{keyword:{pattern:/^(\s*)at(?= )/,lookbehind:!0},source:[{pattern:/(\()\w+\.\w+:\d+(?=\))/,lookbehind:!0,inside:{file:/^\w+\.\w+/,punctuation:/:/,"line-number":{pattern:/\b\d+\b/,alias:"number"}}},{pattern:/(\()[^()]*(?=\))/,lookbehind:!0,inside:{keyword:/^(?:Native Method|Unknown Source)$/}}],"class-name":/[\w$]+(?=\.(?:<init>|[\w$]+)\()/,function:/(?:<init>|[\w$]+)(?=\()/,"class-loader":{pattern:/(\s)[a-z]\w*(?:\.[a-z]\w*)*(?=\/[\w@$.]*\/)/,lookbehind:!0,alias:"namespace",inside:{punctuation:/\./}},module:{pattern:/([\s/])[a-z]\w*(?:\.[a-z]\w*)*(?:@[\w$.+-]*)?(?=\/)/,lookbehind:!0,inside:{version:{pattern:/(@)[\s\S]+/,lookbehind:!0,alias:"number"},punctuation:/[@.]/}},namespace:{pattern:/(?:\b[a-z]\w*\.)+/,inside:{punctuation:/\./}},punctuation:/[()/.]/}},more:{pattern:/^([\t ]*)\.{3} \d+ [a-z]+(?: [a-z]+)*/m,lookbehind:!0,inside:{punctuation:/\.{3}/,number:/\d+/,keyword:/\b[a-z]+(?: [a-z]+)*\b/}}}}return IA}var OA,cj;function oit(){if(cj)return OA;cj=1,OA=e,e.displayName="jexl",e.aliases=[];function e(t){t.languages.jexl={string:/(["'])(?:\\[\s\S]|(?!\1)[^\\])*\1/,transform:{pattern:/(\|\s*)[a-zA-Zа-яА-Я_\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$][\wа-яА-Я\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$]*/,alias:"function",lookbehind:!0},function:/[a-zA-Zа-яА-Я_\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$][\wа-яА-Я\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF$]*\s*(?=\()/,number:/\b\d+(?:\.\d+)?\b|\B\.\d+\b/,operator:/[<>!]=?|-|\+|&&|==|\|\|?|\/\/?|[?:*^%]/,boolean:/\b(?:false|true)\b/,keyword:/\bin\b/,punctuation:/[{}[\](),.]/}}return OA}var BA,dj;function sit(){if(dj)return BA;dj=1,BA=e,e.displayName="jolie",e.aliases=[];function e(t){t.languages.jolie=t.languages.extend("clike",{string:{pattern:/(^|[^\\])"(?:\\[\s\S]|[^"\\])*"/,lookbehind:!0,greedy:!0},"class-name":{pattern:/((?:\b(?:as|courier|embed|in|inputPort|outputPort|service)\b|@)[ \t]*)\w+/,lookbehind:!0},keyword:/\b(?:as|cH|comp|concurrent|constants|courier|cset|csets|default|define|else|embed|embedded|execution|exit|extender|for|foreach|forward|from|global|if|import|in|include|init|inputPort|install|instanceof|interface|is_defined|linkIn|linkOut|main|new|nullProcess|outputPort|over|private|provide|public|scope|sequential|service|single|spawn|synchronized|this|throw|throws|type|undef|until|while|with)\b/,function:/\b[a-z_]\w*(?=[ \t]*[@(])/i,number:/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?l?/i,operator:/-[-=>]?|\+[+=]?|<[<=]?|[>=*!]=?|&&|\|\||[?\/%^@|]/,punctuation:/[()[\]{},;.:]/,builtin:/\b(?:Byte|any|bool|char|double|enum|float|int|length|long|ranges|regex|string|undefined|void)\b/}),t.languages.insertBefore("jolie","keyword",{aggregates:{pattern:/(\bAggregates\s*:\s*)(?:\w+(?:\s+with\s+\w+)?\s*,\s*)*\w+(?:\s+with\s+\w+)?/,lookbehind:!0,inside:{keyword:/\bwith\b/,"class-name":/\w+/,punctuation:/,/}},redirects:{pattern:/(\bRedirects\s*:\s*)(?:\w+\s*=>\s*\w+\s*,\s*)*(?:\w+\s*=>\s*\w+)/,lookbehind:!0,inside:{punctuation:/,/,"class-name":/\w+/,operator:/=>/}},property:{pattern:/\b(?:Aggregates|[Ii]nterfaces|Java|Javascript|Jolie|[Ll]ocation|OneWay|[Pp]rotocol|Redirects|RequestResponse)\b(?=[ \t]*:)/}})}return BA}var PA,hj;function lit(){if(hj)return PA;hj=1,PA=e,e.displayName="jq",e.aliases=[];function e(t){(function(r){var n=/\\\((?:[^()]|\([^()]*\))*\)/.source,a=RegExp(/(^|[^\\])"(?:[^"\r\n\\]|\\[^\r\n(]|__)*"/.source.replace(/__/g,function(){return n})),i={interpolation:{pattern:RegExp(/((?:^|[^\\])(?:\\{2})*)/.source+n),lookbehind:!0,inside:{content:{pattern:/^(\\\()[\s\S]+(?=\)$)/,lookbehind:!0,inside:null},punctuation:/^\\\(|\)$/}}},o=r.languages.jq={comment:/#.*/,property:{pattern:RegExp(a.source+/(?=\s*:(?!:))/.source),lookbehind:!0,greedy:!0,inside:i},string:{pattern:a,lookbehind:!0,greedy:!0,inside:i},function:{pattern:/(\bdef\s+)[a-z_]\w+/i,lookbehind:!0},variable:/\B\$\w+/,"property-literal":{pattern:/\b[a-z_]\w*(?=\s*:(?!:))/i,alias:"property"},keyword:/\b(?:as|break|catch|def|elif|else|end|foreach|if|import|include|label|module|modulemeta|null|reduce|then|try|while)\b/,boolean:/\b(?:false|true)\b/,number:/(?:\b\d+\.|\B\.)?\b\d+(?:[eE][+-]?\d+)?\b/,operator:[{pattern:/\|=?/,alias:"pipe"},/\.\.|[!=<>]?=|\?\/\/|\/\/=?|[-+*/%]=?|[<>?]|\b(?:and|not|or)\b/],"c-style-function":{pattern:/\b[a-z_]\w*(?=\s*\()/i,alias:"function"},punctuation:/::|[()\[\]{},:;]|\.(?=\s*[\[\w$])/,dot:{pattern:/\./,alias:"important"}};i.interpolation.inside.content.inside=o})(t)}return PA}var zA,pj;function uit(){if(pj)return zA;pj=1,zA=e,e.displayName="jsExtras",e.aliases=[];function e(t){(function(r){r.languages.insertBefore("javascript","function-variable",{"method-variable":{pattern:RegExp("(\\.\\s*)"+r.languages.javascript["function-variable"].pattern.source),lookbehind:!0,alias:["function-variable","method","function","property-access"]}}),r.languages.insertBefore("javascript","function",{method:{pattern:RegExp("(\\.\\s*)"+r.languages.javascript.function.source),lookbehind:!0,alias:["function","property-access"]}}),r.languages.insertBefore("javascript","constant",{"known-class-name":[{pattern:/\b(?:(?:Float(?:32|64)|(?:Int|Uint)(?:8|16|32)|Uint8Clamped)?Array|ArrayBuffer|BigInt|Boolean|DataView|Date|Error|Function|Intl|JSON|(?:Weak)?(?:Map|Set)|Math|Number|Object|Promise|Proxy|Reflect|RegExp|String|Symbol|WebAssembly)\b/,alias:"class-name"},{pattern:/\b(?:[A-Z]\w*)Error\b/,alias:"class-name"}]});function n(u,d){return RegExp(u.replace(/<ID>/g,function(){return/(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*/.source}),d)}r.languages.insertBefore("javascript","keyword",{imports:{pattern:n(/(\bimport\b\s*)(?:<ID>(?:\s*,\s*(?:\*\s*as\s+<ID>|\{[^{}]*\}))?|\*\s*as\s+<ID>|\{[^{}]*\})(?=\s*\bfrom\b)/.source),lookbehind:!0,inside:r.languages.javascript},exports:{pattern:n(/(\bexport\b\s*)(?:\*(?:\s*as\s+<ID>)?(?=\s*\bfrom\b)|\{[^{}]*\})/.source),lookbehind:!0,inside:r.languages.javascript}}),r.languages.javascript.keyword.unshift({pattern:/\b(?:as|default|export|from|import)\b/,alias:"module"},{pattern:/\b(?:await|break|catch|continue|do|else|finally|for|if|return|switch|throw|try|while|yield)\b/,alias:"control-flow"},{pattern:/\bnull\b/,alias:["null","nil"]},{pattern:/\bundefined\b/,alias:"nil"}),r.languages.insertBefore("javascript","operator",{spread:{pattern:/\.{3}/,alias:"operator"},arrow:{pattern:/=>/,alias:"operator"}}),r.languages.insertBefore("javascript","punctuation",{"property-access":{pattern:n(/(\.\s*)#?<ID>/.source),lookbehind:!0},"maybe-class-name":{pattern:/(^|[^$\w\xA0-\uFFFF])[A-Z][$\w\xA0-\uFFFF]+/,lookbehind:!0},dom:{pattern:/\b(?:document|(?:local|session)Storage|location|navigator|performance|window)\b/,alias:"variable"},console:{pattern:/\bconsole(?=\s*\.)/,alias:"class-name"}});for(var a=["function","function-variable","method","method-variable","property-access"],i=0;i<a.length;i++){var o=a[i],s=r.languages.javascript[o];r.util.type(s)==="RegExp"&&(s=r.languages.javascript[o]={pattern:s});var l=s.inside||{};s.inside=l,l["maybe-class-name"]=/^[A-Z][\s\S]*/}})(t)}return zA}var HA,fj;function cit(){if(fj)return HA;fj=1,HA=e,e.displayName="jsTemplates",e.aliases=[];function e(t){(function(r){var n=r.languages.javascript["template-string"],a=n.pattern.source,i=n.inside.interpolation,o=i.inside["interpolation-punctuation"],s=i.pattern.source;function l(y,S){if(r.languages[y])return{pattern:RegExp("((?:"+S+")\\s*)"+a),lookbehind:!0,greedy:!0,inside:{"template-punctuation":{pattern:/^`|`$/,alias:"string"},"embedded-code":{pattern:/[\s\S]+/,alias:y}}}}r.languages.javascript["template-string"]=[l("css",/\b(?:styled(?:\([^)]*\))?(?:\s*\.\s*\w+(?:\([^)]*\))*)*|css(?:\s*\.\s*(?:global|resolve))?|createGlobalStyle|keyframes)/.source),l("html",/\bhtml|\.\s*(?:inner|outer)HTML\s*\+?=/.source),l("svg",/\bsvg/.source),l("markdown",/\b(?:markdown|md)/.source),l("graphql",/\b(?:gql|graphql(?:\s*\.\s*experimental)?)/.source),l("sql",/\bsql/.source),n].filter(Boolean);function u(y,S){return"___"+S.toUpperCase()+"_"+y+"___"}function d(y,S,w){var E={code:y,grammar:S,language:w};return r.hooks.run("before-tokenize",E),E.tokens=r.tokenize(E.code,E.grammar),r.hooks.run("after-tokenize",E),E.tokens}function p(y){var S={};S["interpolation-punctuation"]=o;var w=r.tokenize(y,S);if(w.length===3){var E=[1,1];E.push.apply(E,d(w[1],r.languages.javascript,"javascript")),w.splice.apply(w,E)}return new r.Token("interpolation",w,i.alias,y)}function f(y,S,w){var E=r.tokenize(y,{interpolation:{pattern:RegExp(s),lookbehind:!0}}),k=0,_={},R=E.map(function(I){if(typeof I=="string")return I;for(var M=I.content,j;y.indexOf(j=u(k++,w))!==-1;);return _[j]=M,j}).join(""),C=d(R,S,w),F=Object.keys(_);k=0;function L(I){for(var M=0;M<I.length;M++){if(k>=F.length)return;var j=I[M];if(typeof j=="string"||typeof j.content=="string"){var Y=F[k],W=typeof j=="string"?j:j.content,$=W.indexOf(Y);if($!==-1){++k;var X=W.substring(0,$),z=p(_[Y]),H=W.substring($+Y.length),B=[];if(X&&B.push(X),B.push(z),H){var ee=[H];L(ee),B.push.apply(B,ee)}typeof j=="string"?(I.splice.apply(I,[M,1].concat(B)),M+=B.length-1):j.content=B}}else{var J=j.content;Array.isArray(J)?L(J):L([J])}}}return L(C),new r.Token(w,C,"language-"+w,y)}var m={javascript:!0,js:!0,typescript:!0,ts:!0,jsx:!0,tsx:!0};r.hooks.add("after-tokenize",function(y){if(!(y.language in m))return;function S(w){for(var E=0,k=w.length;E<k;E++){var _=w[E];if(typeof _!="string"){var R=_.content;if(!Array.isArray(R)){typeof R!="string"&&S([R]);continue}if(_.type==="template-string"){var C=R[1];if(R.length===3&&typeof C!="string"&&C.type==="embedded-code"){var F=b(C),L=C.alias,I=Array.isArray(L)?L[0]:L,M=r.languages[I];if(!M)continue;R[1]=f(F,M,I)}}else S(R)}}}S(y.tokens)});function b(y){return typeof y=="string"?y:Array.isArray(y)?y.map(b).join(""):b(y.content)}})(t)}return HA}var UA,mj;function gL(){if(mj)return UA;mj=1,UA=e,e.displayName="typescript",e.aliases=["ts"];function e(t){(function(r){r.languages.typescript=r.languages.extend("javascript",{"class-name":{pattern:/(\b(?:class|extends|implements|instanceof|interface|new|type)\s+)(?!keyof\b)(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?:\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>)?/,lookbehind:!0,greedy:!0,inside:null},builtin:/\b(?:Array|Function|Promise|any|boolean|console|never|number|string|symbol|unknown)\b/}),r.languages.typescript.keyword.push(/\b(?:abstract|declare|is|keyof|readonly|require)\b/,/\b(?:asserts|infer|interface|module|namespace|type)\b(?=\s*(?:[{_$a-zA-Z\xA0-\uFFFF]|$))/,/\btype\b(?=\s*(?:[\{*]|$))/),delete r.languages.typescript.parameter,delete r.languages.typescript["literal-property"];var n=r.languages.extend("typescript",{});delete n["class-name"],r.languages.typescript["class-name"].inside=n,r.languages.insertBefore("typescript","function",{decorator:{pattern:/@[$\w\xA0-\uFFFF]+/,inside:{at:{pattern:/^@/,alias:"operator"},function:/^[\s\S]+/}},"generic-function":{pattern:/#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*<(?:[^<>]|<(?:[^<>]|<[^<>]*>)*>)*>(?=\s*\()/,greedy:!0,inside:{function:/^#?(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*/,generic:{pattern:/<[\s\S]+/,alias:"class-name",inside:n}}}}),r.languages.ts=r.languages.typescript})(t)}return UA}var $A,gj;function dit(){if(gj)return $A;gj=1;var e=aS(),t=gL();$A=r,r.displayName="jsdoc",r.aliases=[];function r(n){n.register(e),n.register(t),(function(a){var i=a.languages.javascript,o=/\{(?:[^{}]|\{(?:[^{}]|\{[^{}]*\})*\})+\}/.source,s="(@(?:arg|argument|param|property)\\s+(?:"+o+"\\s+)?)";a.languages.jsdoc=a.languages.extend("javadoclike",{parameter:{pattern:RegExp(s+/(?:(?!\s)[$\w\xA0-\uFFFF.])+(?=\s|$)/.source),lookbehind:!0,inside:{punctuation:/\./}}}),a.languages.insertBefore("jsdoc","keyword",{"optional-parameter":{pattern:RegExp(s+/\[(?:(?!\s)[$\w\xA0-\uFFFF.])+(?:=[^[\]]+)?\](?=\s|$)/.source),lookbehind:!0,inside:{parameter:{pattern:/(^\[)[$\w\xA0-\uFFFF\.]+/,lookbehind:!0,inside:{punctuation:/\./}},code:{pattern:/(=)[\s\S]*(?=\]$)/,lookbehind:!0,inside:i,alias:"language-javascript"},punctuation:/[=[\]]/}},"class-name":[{pattern:RegExp(/(@(?:augments|class|extends|interface|memberof!?|template|this|typedef)\s+(?:<TYPE>\s+)?)[A-Z]\w*(?:\.[A-Z]\w*)*/.source.replace(/<TYPE>/g,function(){return o})),lookbehind:!0,inside:{punctuation:/\./}},{pattern:RegExp("(@[a-z]+\\s+)"+o),lookbehind:!0,inside:{string:i.string,number:i.number,boolean:i.boolean,keyword:a.languages.typescript.keyword,operator:/=>|\.\.\.|[&|?:*]/,punctuation:/[.,;=<>{}()[\]]/}}],example:{pattern:/(@example\s+(?!\s))(?:[^@\s]|\s+(?!\s))+?(?=\s*(?:\*\s*)?(?:@\w|\*\/))/,lookbehind:!0,inside:{code:{pattern:/^([\t ]*(?:\*\s*)?)\S.*$/m,lookbehind:!0,inside:i,alias:"language-javascript"}}}}),a.languages.javadoclike.addSupport("javascript",a.languages.jsdoc)})(n)}return $A}var GA,bj;function bL(){if(bj)return GA;bj=1,GA=e,e.displayName="json",e.aliases=["webmanifest"];function e(t){t.languages.json={property:{pattern:/(^|[^\\])"(?:\\.|[^\\"\r\n])*"(?=\s*:)/,lookbehind:!0,greedy:!0},string:{pattern:/(^|[^\\])"(?:\\.|[^\\"\r\n])*"(?!\s*:)/,lookbehind:!0,greedy:!0},comment:{pattern:/\/\/.*|\/\*[\s\S]*?(?:\*\/|$)/,greedy:!0},number:/-?\b\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,punctuation:/[{}[\],]/,operator:/:/,boolean:/\b(?:false|true)\b/,null:{pattern:/\bnull\b/,alias:"keyword"}},t.languages.webmanifest=t.languages.json}return GA}var jA,yj;function hit(){if(yj)return jA;yj=1;var e=bL();jA=t,t.displayName="json5",t.aliases=[];function t(r){r.register(e),(function(n){var a=/("|')(?:\\(?:\r\n?|\n|.)|(?!\1)[^\\\r\n])*\1/;n.languages.json5=n.languages.extend("json",{property:[{pattern:RegExp(a.source+"(?=\\s*:)"),greedy:!0},{pattern:/(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*:)/,alias:"unquoted"}],string:{pattern:a,greedy:!0},number:/[+-]?\b(?:NaN|Infinity|0x[a-fA-F\d]+)\b|[+-]?(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[eE][+-]?\d+\b)?/})})(r)}return jA}var qA,vj;function pit(){if(vj)return qA;vj=1;var e=bL();qA=t,t.displayName="jsonp",t.aliases=[];function t(r){r.register(e),r.languages.jsonp=r.languages.extend("json",{punctuation:/[{}[\]();,.]/}),r.languages.insertBefore("jsonp","punctuation",{function:/(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*(?=\s*\()/})}return qA}var WA,Sj;function fit(){if(Sj)return WA;Sj=1,WA=e,e.displayName="jsstacktrace",e.aliases=[];function e(t){t.languages.jsstacktrace={"error-message":{pattern:/^\S.*/m,alias:"string"},"stack-frame":{pattern:/(^[ \t]+)at[ \t].*/m,lookbehind:!0,inside:{"not-my-code":{pattern:/^at[ \t]+(?!\s)(?:node\.js|<unknown>|.*(?:node_modules|\(<anonymous>\)|\(<unknown>|<anonymous>$|\(internal\/|\(node\.js)).*/m,alias:"comment"},filename:{pattern:/(\bat\s+(?!\s)|\()(?:[a-zA-Z]:)?[^():]+(?=:)/,lookbehind:!0,alias:"url"},function:{pattern:/(\bat\s+(?:new\s+)?)(?!\s)[_$a-zA-Z\xA0-\uFFFF<][.$\w\xA0-\uFFFF<>]*/,lookbehind:!0,inside:{punctuation:/\./}},punctuation:/[()]/,keyword:/\b(?:at|new)\b/,alias:{pattern:/\[(?:as\s+)?(?!\s)[_$a-zA-Z\xA0-\uFFFF][$\w\xA0-\uFFFF]*\]/,alias:"variable"},"line-number":{pattern:/:\d+(?::\d+)?\b/,alias:"number",inside:{punctuation:/:/}}}}}}return WA}var VA,Ej;function Bce(){if(Ej)return VA;Ej=1,VA=e,e.displayName="jsx",e.aliases=[];function e(t){(function(r){var n=r.util.clone(r.languages.javascript),a=/(?:\s|\/\/.*(?!.)|\/\*(?:[^*]|\*(?!\/))\*\/)/.source,i=/(?:\{(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])*\})/.source,o=/(?:\{<S>*\.{3}(?:[^{}]|<BRACES>)*\})/.source;function s(d,p){return d=d.replace(/<S>/g,function(){return a}).replace(/<BRACES>/g,function(){return i}).replace(/<SPREAD>/g,function(){return o}),RegExp(d,p)}o=s(o).source,r.languages.jsx=r.languages.extend("markup",n),r.languages.jsx.tag.pattern=s(/<\/?(?:[\w.:-]+(?:<S>+(?:[\w.:$-]+(?:=(?:"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*'|[^\s{'"/>=]+|<BRACES>))?|<SPREAD>))*<S>*\/?)?>/.source),r.languages.jsx.tag.inside.tag.pattern=/^<\/?[^\s>\/]*/,r.languages.jsx.tag.inside["attr-value"].pattern=/=(?!\{)(?:"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*'|[^\s'">]+)/,r.languages.jsx.tag.inside.tag.inside["class-name"]=/^[A-Z]\w*(?:\.[A-Z]\w*)*$/,r.languages.jsx.tag.inside.comment=n.comment,r.languages.insertBefore("inside","attr-name",{spread:{pattern:s(/<SPREAD>/.source),inside:r.languages.jsx}},r.languages.jsx.tag),r.languages.insertBefore("inside","special-attr",{script:{pattern:s(/=<BRACES>/.source),alias:"language-javascript",inside:{"script-punctuation":{pattern:/^=(?=\{)/,alias:"punctuation"},rest:r.languages.jsx}}},r.languages.jsx.tag);var l=function(d){return d?typeof d=="string"?d:typeof d.content=="string"?d.content:d.content.map(l).join(""):""},u=function(d){for(var p=[],f=0;f<d.length;f++){var m=d[f],b=!1;if(typeof m!="string"&&(m.type==="tag"&&m.content[0]&&m.content[0].type==="tag"?m.content[0].content[0].content==="</"?p.length>0&&p[p.length-1].tagName===l(m.content[0].content[1])&&p.pop():m.content[m.content.length-1].content==="/>"||p.push({tagName:l(m.content[0].content[1]),openedBraces:0}):p.length>0&&m.type==="punctuation"&&m.content==="{"?p[p.length-1].openedBraces++:p.length>0&&p[p.length-1].openedBraces>0&&m.type==="punctuation"&&m.content==="}"?p[p.length-1].openedBraces--:b=!0),(b||typeof m=="string")&&p.length>0&&p[p.length-1].openedBraces===0){var y=l(m);f<d.length-1&&(typeof d[f+1]=="string"||d[f+1].type==="plain-text")&&(y+=l(d[f+1]),d.splice(f+1,1)),f>0&&(typeof d[f-1]=="string"||d[f-1].type==="plain-text")&&(y=l(d[f-1])+y,d.splice(f-1,1),f--),d[f]=new r.Token("plain-text",y,null,y)}m.content&&typeof m.content!="string"&&u(m.content)}};r.hooks.add("after-tokenize",function(d){d.language!=="jsx"&&d.language!=="tsx"||u(d.tokens)})})(t)}return VA}var KA,xj;function mit(){if(xj)return KA;xj=1,KA=e,e.displayName="julia",e.aliases=[];function e(t){t.languages.julia={comment:{pattern:/(^|[^\\])(?:#=(?:[^#=]|=(?!#)|#(?!=)|#=(?:[^#=]|=(?!#)|#(?!=))*=#)*=#|#.*)/,lookbehind:!0},regex:{pattern:/r"(?:\\.|[^"\\\r\n])*"[imsx]{0,4}/,greedy:!0},string:{pattern:/"""[\s\S]+?"""|(?:\b\w+)?"(?:\\.|[^"\\\r\n])*"|`(?:[^\\`\r\n]|\\.)*`/,greedy:!0},char:{pattern:/(^|[^\w'])'(?:\\[^\r\n][^'\r\n]*|[^\\\r\n])'/,lookbehind:!0,greedy:!0},keyword:/\b(?:abstract|baremodule|begin|bitstype|break|catch|ccall|const|continue|do|else|elseif|end|export|finally|for|function|global|if|immutable|import|importall|in|let|local|macro|module|print|println|quote|return|struct|try|type|typealias|using|while)\b/,boolean:/\b(?:false|true)\b/,number:/(?:\b(?=\d)|\B(?=\.))(?:0[box])?(?:[\da-f]+(?:_[\da-f]+)*(?:\.(?:\d+(?:_\d+)*)?)?|\.\d+(?:_\d+)*)(?:[efp][+-]?\d+(?:_\d+)*)?j?/i,operator:/&&|\|\||[-+*^%÷⊻&$\\]=?|\/[\/=]?|!=?=?|\|[=>]?|<(?:<=?|[=:|])?|>(?:=|>>?=?)?|==?=?|[~≠≤≥'√∛]/,punctuation:/::?|[{}[\]();,.?]/,constant:/\b(?:(?:Inf|NaN)(?:16|32|64)?|im|pi)\b|[πℯ]/}}return KA}var YA,wj;function git(){if(wj)return YA;wj=1,YA=e,e.displayName="keepalived",e.aliases=[];function e(t){t.languages.keepalived={comment:{pattern:/[#!].*/,greedy:!0},string:{pattern:/(^|[^\\])(?:"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"|'(?:\\(?:\r\n|[\s\S])|[^'\\\r\n])*')/,lookbehind:!0,greedy:!0},ip:{pattern:RegExp(/\b(?:(?:(?:[\da-f]{1,4}:){7}[\da-f]{1,4}|(?:[\da-f]{1,4}:){6}:[\da-f]{1,4}|(?:[\da-f]{1,4}:){5}:(?:[\da-f]{1,4}:)?[\da-f]{1,4}|(?:[\da-f]{1,4}:){4}:(?:[\da-f]{1,4}:){0,2}[\da-f]{1,4}|(?:[\da-f]{1,4}:){3}:(?:[\da-f]{1,4}:){0,3}[\da-f]{1,4}|(?:[\da-f]{1,4}:){2}:(?:[\da-f]{1,4}:){0,4}[\da-f]{1,4}|(?:[\da-f]{1,4}:){6}<ipv4>|(?:[\da-f]{1,4}:){0,5}:<ipv4>|::(?:[\da-f]{1,4}:){0,5}<ipv4>|[\da-f]{1,4}::(?:[\da-f]{1,4}:){0,5}[\da-f]{1,4}|::(?:[\da-f]{1,4}:){0,6}[\da-f]{1,4}|(?:[\da-f]{1,4}:){1,7}:)(?:\/\d{1,3})?|<ipv4>(?:\/\d{1,2})?)\b/.source.replace(/<ipv4>/g,function(){return/(?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d)\.){3}(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d|\d))/.source}),"i"),alias:"number"},path:{pattern:/(\s)\/(?:[^\/\s]+\/)*[^\/\s]*|\b[a-zA-Z]:\\(?:[^\\\s]+\\)*[^\\\s]*/,lookbehind:!0,alias:"string"},variable:/\$\{?\w+\}?/,email:{pattern:/[\w-]+@[\w-]+(?:\.[\w-]{2,3}){1,2}/,alias:"string"},"conditional-configuration":{pattern:/@\^?[\w-]+/,alias:"variable"},operator:/=/,property:/\b(?:BFD_CHECK|DNS_CHECK|FILE_CHECK|HTTP_GET|MISC_CHECK|NAME|PING_CHECK|SCRIPTS|SMTP_CHECK|SSL|SSL_GET|TCP_CHECK|UDP_CHECK|accept|advert_int|alpha|auth_pass|auth_type|authentication|bfd_cpu_affinity|bfd_instance|bfd_no_swap|bfd_priority|bfd_process_name|bfd_rlimit_rttime|bfd_rt_priority|bind_if|bind_port|bindto|ca|certificate|check_unicast_src|checker|checker_cpu_affinity|checker_log_all_failures|checker_no_swap|checker_priority|checker_rlimit_rttime|checker_rt_priority|child_wait_time|connect_ip|connect_port|connect_timeout|dbus_service_name|debug|default_interface|delay|delay_before_retry|delay_loop|digest|dont_track_primary|dynamic|dynamic_interfaces|enable_(?:dbus|script_security|sni|snmp_checker|snmp_rfc|snmp_rfcv2|snmp_rfcv3|snmp_vrrp|traps)|end|fall|fast_recovery|file|flag-[123]|fork_delay|full_command|fwmark|garp_group|garp_interval|garp_lower_prio_delay|garp_lower_prio_repeat|garp_master_delay|garp_master_refresh|garp_master_refresh_repeat|garp_master_repeat|global_defs|global_tracking|gna_interval|group|ha_suspend|hashed|helo_name|higher_prio_send_advert|hoplimit|http_protocol|hysteresis|idle_tx|include|inhibit_on_failure|init_fail|init_file|instance|interface|interfaces|interval|ip_family|ipvs_process_name|keepalived.conf|kernel_rx_buf_size|key|linkbeat_interfaces|linkbeat_use_polling|log_all_failures|log_unknown_vrids|lower_prio_no_advert|lthreshold|lvs_flush|lvs_flush_onstop|lvs_method|lvs_netlink_cmd_rcv_bufs|lvs_netlink_cmd_rcv_bufs_force|lvs_netlink_monitor_rcv_bufs|lvs_netlink_monitor_rcv_bufs_force|lvs_notify_fifo|lvs_notify_fifo_script|lvs_sched|lvs_sync_daemon|max_auto_priority|max_hops|mcast_src_ip|mh-fallback|mh-port|min_auto_priority_delay|min_rx|min_tx|misc_dynamic|misc_path|misc_timeout|multiplier|name|namespace_with_ipsets|native_ipv6|neighbor_ip|net_namespace|net_namespace_ipvs|nftables|nftables_counters|nftables_ifindex|nftables_priority|no_accept|no_checker_emails|no_email_faults|nopreempt|notification_email|notification_email_from|notify|notify_backup|notify_deleted|notify_down|notify_fault|notify_fifo|notify_fifo_script|notify_master|notify_master_rx_lower_pri|notify_priority_changes|notify_stop|notify_up|old_unicast_checksum|omega|ops|param_match|passive|password|path|persistence_engine|persistence_granularity|persistence_timeout|preempt|preempt_delay|priority|process|process_monitor_rcv_bufs|process_monitor_rcv_bufs_force|process_name|process_names|promote_secondaries|protocol|proxy_arp|proxy_arp_pvlan|quorum|quorum_down|quorum_max|quorum_up|random_seed|real_server|regex|regex_max_offset|regex_min_offset|regex_no_match|regex_options|regex_stack|reload_repeat|reload_time_file|require_reply|retry|rise|router_id|rs_init_notifies|script|script_user|sh-fallback|sh-port|shutdown_script|shutdown_script_timeout|skip_check_adv_addr|smtp_alert|smtp_alert_checker|smtp_alert_vrrp|smtp_connect_timeout|smtp_helo_name|smtp_server|snmp_socket|sorry_server|sorry_server_inhibit|sorry_server_lvs_method|source_ip|start|startup_script|startup_script_timeout|state|static_ipaddress|static_routes|static_rules|status_code|step|strict_mode|sync_group_tracking_weight|terminate_delay|timeout|track_bfd|track_file|track_group|track_interface|track_process|track_script|track_src_ip|ttl|type|umask|unicast_peer|unicast_src_ip|unicast_ttl|url|use_ipvlan|use_pid_dir|use_vmac|user|uthreshold|val[123]|version|virtual_ipaddress|virtual_ipaddress_excluded|virtual_router_id|virtual_routes|virtual_rules|virtual_server|virtual_server_group|virtualhost|vmac_xmit_base|vrrp|vrrp_(?:check_unicast_src|cpu_affinity|garp_interval|garp_lower_prio_delay|garp_lower_prio_repeat|garp_master_delay|garp_master_refresh|garp_master_refresh_repeat|garp_master_repeat|gna_interval|higher_prio_send_advert|instance|ipsets|iptables|lower_prio_no_advert|mcast_group4|mcast_group6|min_garp|netlink_cmd_rcv_bufs|netlink_cmd_rcv_bufs_force|netlink_monitor_rcv_bufs|netlink_monitor_rcv_bufs_force|no_swap|notify_fifo|notify_fifo_script|notify_priority_changes|priority|process_name|rlimit_rttime|rt_priority|rx_bufs_multiplier|rx_bufs_policy|script|skip_check_adv_addr|startup_delay|strict|sync_group|track_process|version)|warmup|weight)\b/,constant:/\b(?:A|AAAA|AH|BACKUP|CNAME|DR|MASTER|MX|NAT|NS|PASS|SCTP|SOA|TCP|TUN|TXT|UDP|dh|fo|lblc|lblcr|lc|mh|nq|ovf|rr|sed|sh|wlc|wrr)\b/,number:{pattern:/(^|[^\w.-])-?\d+(?:\.\d+)?/,lookbehind:!0},boolean:/\b(?:false|no|off|on|true|yes)\b/,punctuation:/[\{\}]/}}return YA}var JA,kj;function bit(){if(kj)return JA;kj=1,JA=e,e.displayName="keyman",e.aliases=[];function e(t){t.languages.keyman={comment:{pattern:/\bc .*/i,greedy:!0},string:{pattern:/"[^"\r\n]*"|'[^'\r\n]*'/,greedy:!0},"virtual-key":{pattern:/\[\s*(?:(?:ALT|CAPS|CTRL|LALT|LCTRL|NCAPS|RALT|RCTRL|SHIFT)\s+)*(?:[TKU]_[\w?]+|[A-E]\d\d?|"[^"\r\n]*"|'[^'\r\n]*')\s*\]/i,greedy:!0,alias:"function"},"header-keyword":{pattern:/&\w+/,alias:"bold"},"header-statement":{pattern:/\b(?:bitmap|bitmaps|caps always off|caps on only|copyright|hotkey|language|layout|message|name|shift frees caps|version)\b/i,alias:"bold"},"rule-keyword":{pattern:/\b(?:any|baselayout|beep|call|context|deadkey|dk|if|index|layer|notany|nul|outs|platform|reset|return|save|set|store|use)\b/i,alias:"keyword"},"structural-keyword":{pattern:/\b(?:ansi|begin|group|match|nomatch|unicode|using keys)\b/i,alias:"keyword"},"compile-target":{pattern:/\$(?:keyman|keymanonly|keymanweb|kmfl|weaver):/i,alias:"property"},number:/\b(?:U\+[\dA-F]+|d\d+|x[\da-f]+|\d+)\b/i,operator:/[+>\\$]|\.\./,punctuation:/[()=,]/}}return JA}var XA,Cj;function yit(){if(Cj)return XA;Cj=1,XA=e,e.displayName="kotlin",e.aliases=["kt","kts"];function e(t){(function(r){r.languages.kotlin=r.languages.extend("clike",{keyword:{pattern:/(^|[^.])\b(?:abstract|actual|annotation|as|break|by|catch|class|companion|const|constructor|continue|crossinline|data|do|dynamic|else|enum|expect|external|final|finally|for|fun|get|if|import|in|infix|init|inline|inner|interface|internal|is|lateinit|noinline|null|object|open|operator|out|override|package|private|protected|public|reified|return|sealed|set|super|suspend|tailrec|this|throw|to|try|typealias|val|var|vararg|when|where|while)\b/,lookbehind:!0},function:[{pattern:/(?:`[^\r\n`]+`|\b\w+)(?=\s*\()/,greedy:!0},{pattern:/(\.)(?:`[^\r\n`]+`|\w+)(?=\s*\{)/,lookbehind:!0,greedy:!0}],number:/\b(?:0[xX][\da-fA-F]+(?:_[\da-fA-F]+)*|0[bB][01]+(?:_[01]+)*|\d+(?:_\d+)*(?:\.\d+(?:_\d+)*)?(?:[eE][+-]?\d+(?:_\d+)*)?[fFL]?)\b/,operator:/\+[+=]?|-[-=>]?|==?=?|!(?:!|==?)?|[\/*%<>]=?|[?:]:?|\.\.|&&|\|\||\b(?:and|inv|or|shl|shr|ushr|xor)\b/}),delete r.languages.kotlin["class-name"];var n={"interpolation-punctuation":{pattern:/^\$\{?|\}$/,alias:"punctuation"},expression:{pattern:/[\s\S]+/,inside:r.languages.kotlin}};r.languages.insertBefore("kotlin","string",{"string-literal":[{pattern:/"""(?:[^$]|\$(?:(?!\{)|\{[^{}]*\}))*?"""/,alias:"multiline",inside:{interpolation:{pattern:/\$(?:[a-z_]\w*|\{[^{}]*\})/i,inside:n},string:/[\s\S]+/}},{pattern:/"(?:[^"\\\r\n$]|\\.|\$(?:(?!\{)|\{[^{}]*\}))*"/,alias:"singleline",inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$(?:[a-z_]\w*|\{[^{}]*\})/i,lookbehind:!0,inside:n},string:/[\s\S]+/}}],char:{pattern:/'(?:[^'\\\r\n]|\\(?:.|u[a-fA-F0-9]{0,4}))'/,greedy:!0}}),delete r.languages.kotlin.string,r.languages.insertBefore("kotlin","keyword",{annotation:{pattern:/\B@(?:\w+:)?(?:[A-Z]\w*|\[[^\]]+\])/,alias:"builtin"}}),r.languages.insertBefore("kotlin","function",{label:{pattern:/\b\w+@|@\w+\b/,alias:"symbol"}}),r.languages.kt=r.languages.kotlin,r.languages.kts=r.languages.kotlin})(t)}return XA}var ZA,Tj;function vit(){if(Tj)return ZA;Tj=1,ZA=e,e.displayName="kumir",e.aliases=["kum"];function e(t){(function(r){var n=/\s\x00-\x1f\x22-\x2f\x3a-\x3f\x5b-\x5e\x60\x7b-\x7e/.source;function a(i,o){return RegExp(i.replace(/<nonId>/g,n),o)}r.languages.kumir={comment:{pattern:/\|.*/},prolog:{pattern:/#.*/,greedy:!0},string:{pattern:/"[^\n\r"]*"|'[^\n\r']*'/,greedy:!0},boolean:{pattern:a(/(^|[<nonId>])(?:да|нет)(?=[<nonId>]|$)/.source),lookbehind:!0},"operator-word":{pattern:a(/(^|[<nonId>])(?:и|или|не)(?=[<nonId>]|$)/.source),lookbehind:!0,alias:"keyword"},"system-variable":{pattern:a(/(^|[<nonId>])знач(?=[<nonId>]|$)/.source),lookbehind:!0,alias:"keyword"},type:[{pattern:a(/(^|[<nonId>])(?:вещ|лит|лог|сим|цел)(?:\x20*таб)?(?=[<nonId>]|$)/.source),lookbehind:!0,alias:"builtin"},{pattern:a(/(^|[<nonId>])(?:компл|сканкод|файл|цвет)(?=[<nonId>]|$)/.source),lookbehind:!0,alias:"important"}],keyword:{pattern:a(/(^|[<nonId>])(?:алг|арг(?:\x20*рез)?|ввод|ВКЛЮЧИТЬ|вс[её]|выбор|вывод|выход|дано|для|до|дс|если|иначе|исп|использовать|кон(?:(?:\x20+|_)исп)?|кц(?:(?:\x20+|_)при)?|надо|нач|нс|нц|от|пауза|пока|при|раза?|рез|стоп|таб|то|утв|шаг)(?=[<nonId>]|$)/.source),lookbehind:!0},name:{pattern:a(/(^|[<nonId>])[^\d<nonId>][^<nonId>]*(?:\x20+[^<nonId>]+)*(?=[<nonId>]|$)/.source),lookbehind:!0},number:{pattern:a(/(^|[<nonId>])(?:\B\$[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)(?=[<nonId>]|$)/.source,"i"),lookbehind:!0},punctuation:/:=|[(),:;\[\]]/,"operator-char":{pattern:/\*\*?|<[=>]?|>=?|[-+/=]/,alias:"operator"}},r.languages.kum=r.languages.kumir})(t)}return ZA}var QA,Aj;function Sit(){if(Aj)return QA;Aj=1,QA=e,e.displayName="kusto",e.aliases=[];function e(t){t.languages.kusto={comment:{pattern:/\/\/.*/,greedy:!0},string:{pattern:/```[\s\S]*?```|[hH]?(?:"(?:[^\r\n\\"]|\\.)*"|'(?:[^\r\n\\']|\\.)*'|@(?:"[^\r\n"]*"|'[^\r\n']*'))/,greedy:!0},verb:{pattern:/(\|\s*)[a-z][\w-]*/i,lookbehind:!0,alias:"keyword"},command:{pattern:/\.[a-z][a-z\d-]*\b/,alias:"keyword"},"class-name":/\b(?:bool|datetime|decimal|dynamic|guid|int|long|real|string|timespan)\b/,keyword:/\b(?:access|alias|and|anti|as|asc|auto|between|by|(?:contains|(?:ends|starts)with|has(?:perfix|suffix)?)(?:_cs)?|database|declare|desc|external|from|fullouter|has_all|in|ingestion|inline|inner|innerunique|into|(?:left|right)(?:anti(?:semi)?|inner|outer|semi)?|let|like|local|not|of|on|or|pattern|print|query_parameters|range|restrict|schema|set|step|table|tables|to|view|where|with|matches\s+regex|nulls\s+(?:first|last))(?![\w-])/,boolean:/\b(?:false|null|true)\b/,function:/\b[a-z_]\w*(?=\s*\()/,datetime:[{pattern:/\b(?:(?:Fri|Friday|Mon|Monday|Sat|Saturday|Sun|Sunday|Thu|Thursday|Tue|Tuesday|Wed|Wednesday)\s*,\s*)?\d{1,2}(?:\s+|-)(?:Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep)(?:\s+|-)\d{2}\s+\d{2}:\d{2}(?::\d{2})?(?:\s*(?:\b(?:[A-Z]|(?:[ECMT][DS]|GM|U)T)|[+-]\d{4}))?\b/,alias:"number"},{pattern:/[+-]?\b(?:\d{4}-\d{2}-\d{2}(?:[ T]\d{2}:\d{2}(?::\d{2}(?:\.\d+)?)?)?|\d{2}:\d{2}(?::\d{2}(?:\.\d+)?)?)Z?/,alias:"number"}],number:/\b(?:0x[0-9A-Fa-f]+|\d+(?:\.\d+)?(?:[Ee][+-]?\d+)?)(?:(?:min|sec|[mnµ]s|[dhms]|microsecond|tick)\b)?|[+-]?\binf\b/,operator:/=>|[!=]~|[!=<>]=?|[-+*/%|]|\.\./,punctuation:/[()\[\]{},;.:]/}}return QA}var eF,Fj;function Eit(){if(Fj)return eF;Fj=1,eF=e,e.displayName="latex",e.aliases=["tex","context"];function e(t){(function(r){var n=/\\(?:[^a-z()[\]]|[a-z*]+)/i,a={"equation-command":{pattern:n,alias:"regex"}};r.languages.latex={comment:/%.*/,cdata:{pattern:/(\\begin\{((?:lstlisting|verbatim)\*?)\})[\s\S]*?(?=\\end\{\2\})/,lookbehind:!0},equation:[{pattern:/\$\$(?:\\[\s\S]|[^\\$])+\$\$|\$(?:\\[\s\S]|[^\\$])+\$|\\\([\s\S]*?\\\)|\\\[[\s\S]*?\\\]/,inside:a,alias:"string"},{pattern:/(\\begin\{((?:align|eqnarray|equation|gather|math|multline)\*?)\})[\s\S]*?(?=\\end\{\2\})/,lookbehind:!0,inside:a,alias:"string"}],keyword:{pattern:/(\\(?:begin|cite|documentclass|end|label|ref|usepackage)(?:\[[^\]]+\])?\{)[^}]+(?=\})/,lookbehind:!0},url:{pattern:/(\\url\{)[^}]+(?=\})/,lookbehind:!0},headline:{pattern:/(\\(?:chapter|frametitle|paragraph|part|section|subparagraph|subsection|subsubparagraph|subsubsection|subsubsubparagraph)\*?(?:\[[^\]]+\])?\{)[^}]+(?=\})/,lookbehind:!0,alias:"class-name"},function:{pattern:n,alias:"selector"},punctuation:/[[\]{}&]/},r.languages.tex=r.languages.latex,r.languages.context=r.languages.latex})(t)}return eF}var tF,_j;function iS(){if(_j)return tF;_j=1;var e=Si();tF=t,t.displayName="php",t.aliases=[];function t(r){r.register(e),(function(n){var a=/\/\*[\s\S]*?\*\/|\/\/.*|#(?!\[).*/,i=[{pattern:/\b(?:false|true)\b/i,alias:"boolean"},{pattern:/(::\s*)\b[a-z_]\w*\b(?!\s*\()/i,greedy:!0,lookbehind:!0},{pattern:/(\b(?:case|const)\s+)\b[a-z_]\w*(?=\s*[;=])/i,greedy:!0,lookbehind:!0},/\b(?:null)\b/i,/\b[A-Z_][A-Z0-9_]*\b(?!\s*\()/],o=/\b0b[01]+(?:_[01]+)*\b|\b0o[0-7]+(?:_[0-7]+)*\b|\b0x[\da-f]+(?:_[\da-f]+)*\b|(?:\b\d+(?:_\d+)*\.?(?:\d+(?:_\d+)*)?|\B\.\d+)(?:e[+-]?\d+)?/i,s=/<?=>|\?\?=?|\.{3}|\??->|[!=]=?=?|::|\*\*=?|--|\+\+|&&|\|\||<<|>>|[?~]|[/^|%*&<>.+-]=?/,l=/[{}\[\](),:;]/;n.languages.php={delimiter:{pattern:/\?>$|^<\?(?:php(?=\s)|=)?/i,alias:"important"},comment:a,variable:/\$+(?:\w+\b|(?=\{))/,package:{pattern:/(namespace\s+|use\s+(?:function\s+)?)(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,lookbehind:!0,inside:{punctuation:/\\/}},"class-name-definition":{pattern:/(\b(?:class|enum|interface|trait)\s+)\b[a-z_]\w*(?!\\)\b/i,lookbehind:!0,alias:"class-name"},"function-definition":{pattern:/(\bfunction\s+)[a-z_]\w*(?=\s*\()/i,lookbehind:!0,alias:"function"},keyword:[{pattern:/(\(\s*)\b(?:array|bool|boolean|float|int|integer|object|string)\b(?=\s*\))/i,alias:"type-casting",greedy:!0,lookbehind:!0},{pattern:/([(,?]\s*)\b(?:array(?!\s*\()|bool|callable|(?:false|null)(?=\s*\|)|float|int|iterable|mixed|object|self|static|string)\b(?=\s*\$)/i,alias:"type-hint",greedy:!0,lookbehind:!0},{pattern:/(\)\s*:\s*(?:\?\s*)?)\b(?:array(?!\s*\()|bool|callable|(?:false|null)(?=\s*\|)|float|int|iterable|mixed|object|self|static|string|void)\b/i,alias:"return-type",greedy:!0,lookbehind:!0},{pattern:/\b(?:array(?!\s*\()|bool|float|int|iterable|mixed|object|string|void)\b/i,alias:"type-declaration",greedy:!0},{pattern:/(\|\s*)(?:false|null)\b|\b(?:false|null)(?=\s*\|)/i,alias:"type-declaration",greedy:!0,lookbehind:!0},{pattern:/\b(?:parent|self|static)(?=\s*::)/i,alias:"static-context",greedy:!0},{pattern:/(\byield\s+)from\b/i,lookbehind:!0},/\bclass\b/i,{pattern:/((?:^|[^\s>:]|(?:^|[^-])>|(?:^|[^:]):)\s*)\b(?:abstract|and|array|as|break|callable|case|catch|clone|const|continue|declare|default|die|do|echo|else|elseif|empty|enddeclare|endfor|endforeach|endif|endswitch|endwhile|enum|eval|exit|extends|final|finally|fn|for|foreach|function|global|goto|if|implements|include|include_once|instanceof|insteadof|interface|isset|list|match|namespace|new|or|parent|print|private|protected|public|require|require_once|return|self|static|switch|throw|trait|try|unset|use|var|while|xor|yield|__halt_compiler)\b/i,lookbehind:!0}],"argument-name":{pattern:/([(,]\s+)\b[a-z_]\w*(?=\s*:(?!:))/i,lookbehind:!0},"class-name":[{pattern:/(\b(?:extends|implements|instanceof|new(?!\s+self|\s+static))\s+|\bcatch\s*\()\b[a-z_]\w*(?!\\)\b/i,greedy:!0,lookbehind:!0},{pattern:/(\|\s*)\b[a-z_]\w*(?!\\)\b/i,greedy:!0,lookbehind:!0},{pattern:/\b[a-z_]\w*(?!\\)\b(?=\s*\|)/i,greedy:!0},{pattern:/(\|\s*)(?:\\?\b[a-z_]\w*)+\b/i,alias:"class-name-fully-qualified",greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}},{pattern:/(?:\\?\b[a-z_]\w*)+\b(?=\s*\|)/i,alias:"class-name-fully-qualified",greedy:!0,inside:{punctuation:/\\/}},{pattern:/(\b(?:extends|implements|instanceof|new(?!\s+self\b|\s+static\b))\s+|\bcatch\s*\()(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,alias:"class-name-fully-qualified",greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}},{pattern:/\b[a-z_]\w*(?=\s*\$)/i,alias:"type-declaration",greedy:!0},{pattern:/(?:\\?\b[a-z_]\w*)+(?=\s*\$)/i,alias:["class-name-fully-qualified","type-declaration"],greedy:!0,inside:{punctuation:/\\/}},{pattern:/\b[a-z_]\w*(?=\s*::)/i,alias:"static-context",greedy:!0},{pattern:/(?:\\?\b[a-z_]\w*)+(?=\s*::)/i,alias:["class-name-fully-qualified","static-context"],greedy:!0,inside:{punctuation:/\\/}},{pattern:/([(,?]\s*)[a-z_]\w*(?=\s*\$)/i,alias:"type-hint",greedy:!0,lookbehind:!0},{pattern:/([(,?]\s*)(?:\\?\b[a-z_]\w*)+(?=\s*\$)/i,alias:["class-name-fully-qualified","type-hint"],greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}},{pattern:/(\)\s*:\s*(?:\?\s*)?)\b[a-z_]\w*(?!\\)\b/i,alias:"return-type",greedy:!0,lookbehind:!0},{pattern:/(\)\s*:\s*(?:\?\s*)?)(?:\\?\b[a-z_]\w*)+\b(?!\\)/i,alias:["class-name-fully-qualified","return-type"],greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}}],constant:i,function:{pattern:/(^|[^\\\w])\\?[a-z_](?:[\w\\]*\w)?(?=\s*\()/i,lookbehind:!0,inside:{punctuation:/\\/}},property:{pattern:/(->\s*)\w+/,lookbehind:!0},number:o,operator:s,punctuation:l};var u={pattern:/\{\$(?:\{(?:\{[^{}]+\}|[^{}]+)\}|[^{}])+\}|(^|[^\\{])\$+(?:\w+(?:\[[^\r\n\[\]]+\]|->\w+)?)/,lookbehind:!0,inside:n.languages.php},d=[{pattern:/<<<'([^']+)'[\r\n](?:.*[\r\n])*?\1;/,alias:"nowdoc-string",greedy:!0,inside:{delimiter:{pattern:/^<<<'[^']+'|[a-z_]\w*;$/i,alias:"symbol",inside:{punctuation:/^<<<'?|[';]$/}}}},{pattern:/<<<(?:"([^"]+)"[\r\n](?:.*[\r\n])*?\1;|([a-z_]\w*)[\r\n](?:.*[\r\n])*?\2;)/i,alias:"heredoc-string",greedy:!0,inside:{delimiter:{pattern:/^<<<(?:"[^"]+"|[a-z_]\w*)|[a-z_]\w*;$/i,alias:"symbol",inside:{punctuation:/^<<<"?|[";]$/}},interpolation:u}},{pattern:/`(?:\\[\s\S]|[^\\`])*`/,alias:"backtick-quoted-string",greedy:!0},{pattern:/'(?:\\[\s\S]|[^\\'])*'/,alias:"single-quoted-string",greedy:!0},{pattern:/"(?:\\[\s\S]|[^\\"])*"/,alias:"double-quoted-string",greedy:!0,inside:{interpolation:u}}];n.languages.insertBefore("php","variable",{string:d,attribute:{pattern:/#\[(?:[^"'\/#]|\/(?![*/])|\/\/.*$|#(?!\[).*$|\/\*(?:[^*]|\*(?!\/))*\*\/|"(?:\\[\s\S]|[^\\"])*"|'(?:\\[\s\S]|[^\\'])*')+\](?=\s*[a-z$#])/im,greedy:!0,inside:{"attribute-content":{pattern:/^(#\[)[\s\S]+(?=\]$)/,lookbehind:!0,inside:{comment:a,string:d,"attribute-class-name":[{pattern:/([^:]|^)\b[a-z_]\w*(?!\\)\b/i,alias:"class-name",greedy:!0,lookbehind:!0},{pattern:/([^:]|^)(?:\\?\b[a-z_]\w*)+/i,alias:["class-name","class-name-fully-qualified"],greedy:!0,lookbehind:!0,inside:{punctuation:/\\/}}],constant:i,number:o,operator:s,punctuation:l}},delimiter:{pattern:/^#\[|\]$/,alias:"punctuation"}}}}),n.hooks.add("before-tokenize",function(p){if(/<\?/.test(p.code)){var f=/<\?(?:[^"'/#]|\/(?![*/])|("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|(?:\/\/|#(?!\[))(?:[^?\n\r]|\?(?!>))*(?=$|\?>|[\r\n])|#\[|\/\*(?:[^*]|\*(?!\/))*(?:\*\/|$))*?(?:\?>|$)/g;n.languages["markup-templating"].buildPlaceholders(p,"php",f)}}),n.hooks.add("after-tokenize",function(p){n.languages["markup-templating"].tokenizePlaceholders(p,"php")})})(r)}return tF}var rF,Rj;function xit(){if(Rj)return rF;Rj=1;var e=Si(),t=iS();rF=r,r.displayName="latte",r.aliases=[];function r(n){n.register(e),n.register(t),(function(a){a.languages.latte={comment:/^\{\*[\s\S]*/,"latte-tag":{pattern:/(^\{(?:\/(?=[a-z]))?)(?:[=_]|[a-z]\w*\b(?!\())/i,lookbehind:!0,alias:"important"},delimiter:{pattern:/^\{\/?|\}$/,alias:"punctuation"},php:{pattern:/\S(?:[\s\S]*\S)?/,alias:"language-php",inside:a.languages.php}};var i=a.languages.extend("markup",{});a.languages.insertBefore("inside","attr-value",{"n-attr":{pattern:/n:[\w-]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+))?/,inside:{"attr-name":{pattern:/^[^\s=]+/,alias:"important"},"attr-value":{pattern:/=[\s\S]+/,inside:{punctuation:[/^=/,{pattern:/^(\s*)["']|["']$/,lookbehind:!0}],php:{pattern:/\S(?:[\s\S]*\S)?/,inside:a.languages.php}}}}}},i.tag),a.hooks.add("before-tokenize",function(o){if(o.language==="latte"){var s=/\{\*[\s\S]*?\*\}|\{[^'"\s{}*](?:[^"'/{}]|\/(?![*/])|("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|\/\*(?:[^*]|\*(?!\/))*\*\/)*\}/g;a.languages["markup-templating"].buildPlaceholders(o,"latte",s),o.grammar=i}}),a.hooks.add("after-tokenize",function(o){a.languages["markup-templating"].tokenizePlaceholders(o,"latte")})})(n)}return rF}var nF,Dj;function wit(){if(Dj)return nF;Dj=1,nF=e,e.displayName="less",e.aliases=[];function e(t){t.languages.less=t.languages.extend("css",{comment:[/\/\*[\s\S]*?\*\//,{pattern:/(^|[^\\])\/\/.*/,lookbehind:!0}],atrule:{pattern:/@[\w-](?:\((?:[^(){}]|\([^(){}]*\))*\)|[^(){};\s]|\s+(?!\s))*?(?=\s*\{)/,inside:{punctuation:/[:()]/}},selector:{pattern:/(?:@\{[\w-]+\}|[^{};\s@])(?:@\{[\w-]+\}|\((?:[^(){}]|\([^(){}]*\))*\)|[^(){};@\s]|\s+(?!\s))*?(?=\s*\{)/,inside:{variable:/@+[\w-]+/}},property:/(?:@\{[\w-]+\}|[\w-])+(?:\+_?)?(?=\s*:)/,operator:/[+\-*\/]/}),t.languages.insertBefore("less","property",{variable:[{pattern:/@[\w-]+\s*:/,inside:{punctuation:/:/}},/@@?[\w-]+/],"mixin-usage":{pattern:/([{;]\s*)[.#](?!\d)[\w-].*?(?=[(;])/,lookbehind:!0,alias:"function"}})}return nF}var aF,Nj;function yL(){if(Nj)return aF;Nj=1,aF=e,e.displayName="scheme",e.aliases=[];function e(t){(function(r){r.languages.scheme={comment:/;.*|#;\s*(?:\((?:[^()]|\([^()]*\))*\)|\[(?:[^\[\]]|\[[^\[\]]*\])*\])|#\|(?:[^#|]|#(?!\|)|\|(?!#)|#\|(?:[^#|]|#(?!\|)|\|(?!#))*\|#)*\|#/,string:{pattern:/"(?:[^"\\]|\\.)*"/,greedy:!0},symbol:{pattern:/'[^()\[\]#'\s]+/,greedy:!0},char:{pattern:/#\\(?:[ux][a-fA-F\d]+\b|[-a-zA-Z]+\b|[\uD800-\uDBFF][\uDC00-\uDFFF]|\S)/,greedy:!0},"lambda-parameter":[{pattern:/((?:^|[^'`#])[(\[]lambda\s+)(?:[^|()\[\]'\s]+|\|(?:[^\\|]|\\.)*\|)/,lookbehind:!0},{pattern:/((?:^|[^'`#])[(\[]lambda\s+[(\[])[^()\[\]']+/,lookbehind:!0}],keyword:{pattern:/((?:^|[^'`#])[(\[])(?:begin|case(?:-lambda)?|cond(?:-expand)?|define(?:-library|-macro|-record-type|-syntax|-values)?|defmacro|delay(?:-force)?|do|else|except|export|guard|if|import|include(?:-ci|-library-declarations)?|lambda|let(?:rec)?(?:-syntax|-values|\*)?|let\*-values|only|parameterize|prefix|(?:quasi-?)?quote|rename|set!|syntax-(?:case|rules)|unless|unquote(?:-splicing)?|when)(?=[()\[\]\s]|$)/,lookbehind:!0},builtin:{pattern:/((?:^|[^'`#])[(\[])(?:abs|and|append|apply|assoc|ass[qv]|binary-port\?|boolean=?\?|bytevector(?:-append|-copy|-copy!|-length|-u8-ref|-u8-set!|\?)?|caar|cadr|call-with-(?:current-continuation|port|values)|call\/cc|car|cdar|cddr|cdr|ceiling|char(?:->integer|-ready\?|\?|<\?|<=\?|=\?|>\?|>=\?)|close-(?:input-port|output-port|port)|complex\?|cons|current-(?:error|input|output)-port|denominator|dynamic-wind|eof-object\??|eq\?|equal\?|eqv\?|error|error-object(?:-irritants|-message|\?)|eval|even\?|exact(?:-integer-sqrt|-integer\?|\?)?|expt|features|file-error\?|floor(?:-quotient|-remainder|\/)?|flush-output-port|for-each|gcd|get-output-(?:bytevector|string)|inexact\??|input-port(?:-open\?|\?)|integer(?:->char|\?)|lcm|length|list(?:->string|->vector|-copy|-ref|-set!|-tail|\?)?|make-(?:bytevector|list|parameter|string|vector)|map|max|member|memq|memv|min|modulo|negative\?|newline|not|null\?|number(?:->string|\?)|numerator|odd\?|open-(?:input|output)-(?:bytevector|string)|or|output-port(?:-open\?|\?)|pair\?|peek-char|peek-u8|port\?|positive\?|procedure\?|quotient|raise|raise-continuable|rational\?|rationalize|read-(?:bytevector|bytevector!|char|error\?|line|string|u8)|real\?|remainder|reverse|round|set-c[ad]r!|square|string(?:->list|->number|->symbol|->utf8|->vector|-append|-copy|-copy!|-fill!|-for-each|-length|-map|-ref|-set!|\?|<\?|<=\?|=\?|>\?|>=\?)?|substring|symbol(?:->string|\?|=\?)|syntax-error|textual-port\?|truncate(?:-quotient|-remainder|\/)?|u8-ready\?|utf8->string|values|vector(?:->list|->string|-append|-copy|-copy!|-fill!|-for-each|-length|-map|-ref|-set!|\?)?|with-exception-handler|write-(?:bytevector|char|string|u8)|zero\?)(?=[()\[\]\s]|$)/,lookbehind:!0},operator:{pattern:/((?:^|[^'`#])[(\[])(?:[-+*%/]|[<>]=?|=>?)(?=[()\[\]\s]|$)/,lookbehind:!0},number:{pattern:RegExp(n({"<ureal dec>":/\d+(?:\/\d+)|(?:\d+(?:\.\d*)?|\.\d+)(?:[esfdl][+-]?\d+)?/.source,"<real dec>":/[+-]?<ureal dec>|[+-](?:inf|nan)\.0/.source,"<imaginary dec>":/[+-](?:<ureal dec>|(?:inf|nan)\.0)?i/.source,"<complex dec>":/<real dec>(?:@<real dec>|<imaginary dec>)?|<imaginary dec>/.source,"<num dec>":/(?:#d(?:#[ei])?|#[ei](?:#d)?)?<complex dec>/.source,"<ureal box>":/[0-9a-f]+(?:\/[0-9a-f]+)?/.source,"<real box>":/[+-]?<ureal box>|[+-](?:inf|nan)\.0/.source,"<imaginary box>":/[+-](?:<ureal box>|(?:inf|nan)\.0)?i/.source,"<complex box>":/<real box>(?:@<real box>|<imaginary box>)?|<imaginary box>/.source,"<num box>":/#[box](?:#[ei])?|(?:#[ei])?#[box]<complex box>/.source,"<number>":/(^|[()\[\]\s])(?:<num dec>|<num box>)(?=[()\[\]\s]|$)/.source}),"i"),lookbehind:!0},boolean:{pattern:/(^|[()\[\]\s])#(?:[ft]|false|true)(?=[()\[\]\s]|$)/,lookbehind:!0},function:{pattern:/((?:^|[^'`#])[(\[])(?:[^|()\[\]'\s]+|\|(?:[^\\|]|\\.)*\|)(?=[()\[\]\s]|$)/,lookbehind:!0},identifier:{pattern:/(^|[()\[\]\s])\|(?:[^\\|]|\\.)*\|(?=[()\[\]\s]|$)/,lookbehind:!0,greedy:!0},punctuation:/[()\[\]']/};function n(a){for(var i in a)a[i]=a[i].replace(/<[\w\s]+>/g,function(o){return"(?:"+a[o].trim()+")"});return a[i]}})(t)}return aF}var iF,Lj;function kit(){if(Lj)return iF;Lj=1;var e=yL();iF=t,t.displayName="lilypond",t.aliases=[];function t(r){r.register(e),(function(n){for(var a=/\((?:[^();"#\\]|\\[\s\S]|;.*(?!.)|"(?:[^"\\]|\\.)*"|#(?:\{(?:(?!#\})[\s\S])*#\}|[^{])|<expr>)*\)/.source,i=5,o=0;o<i;o++)a=a.replace(/<expr>/g,function(){return a});a=a.replace(/<expr>/g,/[^\s\S]/.source);var s=n.languages.lilypond={comment:/%(?:(?!\{).*|\{[\s\S]*?%\})/,"embedded-scheme":{pattern:RegExp(/(^|[=\s])#(?:"(?:[^"\\]|\\.)*"|[^\s()"]*(?:[^\s()]|<expr>))/.source.replace(/<expr>/g,function(){return a}),"m"),lookbehind:!0,greedy:!0,inside:{scheme:{pattern:/^(#)[\s\S]+$/,lookbehind:!0,alias:"language-scheme",inside:{"embedded-lilypond":{pattern:/#\{[\s\S]*?#\}/,greedy:!0,inside:{punctuation:/^#\{|#\}$/,lilypond:{pattern:/[\s\S]+/,alias:"language-lilypond",inside:null}}},rest:n.languages.scheme}},punctuation:/#/}},string:{pattern:/"(?:[^"\\]|\\.)*"/,greedy:!0},"class-name":{pattern:/(\\new\s+)[\w-]+/,lookbehind:!0},keyword:{pattern:/\\[a-z][-\w]*/i,inside:{punctuation:/^\\/}},operator:/[=|]|<<|>>/,punctuation:{pattern:/(^|[a-z\d])(?:'+|,+|[_^]?-[_^]?(?:[-+^!>._]|(?=\d))|[_^]\.?|[.!])|[{}()[\]<>^~]|\\[()[\]<>\\!]|--|__/,lookbehind:!0},number:/\b\d+(?:\/\d+)?\b/};s["embedded-scheme"].inside.scheme.inside["embedded-lilypond"].inside.lilypond.inside=s,n.languages.ly=s})(r)}return iF}var oF,Mj;function Cit(){if(Mj)return oF;Mj=1;var e=Si();oF=t,t.displayName="liquid",t.aliases=[];function t(r){r.register(e),r.languages.liquid={comment:{pattern:/(^\{%\s*comment\s*%\})[\s\S]+(?=\{%\s*endcomment\s*%\}$)/,lookbehind:!0},delimiter:{pattern:/^\{(?:\{\{|[%\{])-?|-?(?:\}\}|[%\}])\}$/,alias:"punctuation"},string:{pattern:/"[^"]*"|'[^']*'/,greedy:!0},keyword:/\b(?:as|assign|break|(?:end)?(?:capture|case|comment|for|form|if|paginate|raw|style|tablerow|unless)|continue|cycle|decrement|echo|else|elsif|in|include|increment|limit|liquid|offset|range|render|reversed|section|when|with)\b/,object:/\b(?:address|all_country_option_tags|article|block|blog|cart|checkout|collection|color|country|country_option_tags|currency|current_page|current_tags|customer|customer_address|date|discount_allocation|discount_application|external_video|filter|filter_value|font|forloop|fulfillment|generic_file|gift_card|group|handle|image|line_item|link|linklist|localization|location|measurement|media|metafield|model|model_source|order|page|page_description|page_image|page_title|part|policy|product|product_option|recommendations|request|robots|routes|rule|script|search|selling_plan|selling_plan_allocation|selling_plan_group|shipping_method|shop|shop_locale|sitemap|store_availability|tax_line|template|theme|transaction|unit_price_measurement|user_agent|variant|video|video_source)\b/,function:[{pattern:/(\|\s*)\w+/,lookbehind:!0,alias:"filter"},{pattern:/(\.\s*)(?:first|last|size)/,lookbehind:!0}],boolean:/\b(?:false|nil|true)\b/,range:{pattern:/\.\./,alias:"operator"},number:/\b\d+(?:\.\d+)?\b/,operator:/[!=]=|<>|[<>]=?|[|?:=-]|\b(?:and|contains(?=\s)|or)\b/,punctuation:/[.,\[\]()]/,empty:{pattern:/\bempty\b/,alias:"keyword"}},r.hooks.add("before-tokenize",function(n){var a=/\{%\s*comment\s*%\}[\s\S]*?\{%\s*endcomment\s*%\}|\{(?:%[\s\S]*?%|\{\{[\s\S]*?\}\}|\{[\s\S]*?\})\}/g,i=!1;r.languages["markup-templating"].buildPlaceholders(n,"liquid",a,function(o){var s=/^\{%-?\s*(\w+)/.exec(o);if(s){var l=s[1];if(l==="raw"&&!i)return i=!0,!0;if(l==="endraw")return i=!1,!0}return!i})}),r.hooks.add("after-tokenize",function(n){r.languages["markup-templating"].tokenizePlaceholders(n,"liquid")})}return oF}var sF,Ij;function Tit(){if(Ij)return sF;Ij=1,sF=e,e.displayName="lisp",e.aliases=[];function e(t){(function(r){function n(y){return RegExp(/(\()/.source+"(?:"+y+")"+/(?=[\s\)])/.source)}function a(y){return RegExp(/([\s([])/.source+"(?:"+y+")"+/(?=[\s)])/.source)}var i=/(?!\d)[-+*/~!@$%^=<>{}\w]+/.source,o="&"+i,s="(\\()",l="(?=\\))",u="(?=\\s)",d=/(?:[^()]|\((?:[^()]|\((?:[^()]|\((?:[^()]|\((?:[^()]|\([^()]*\))*\))*\))*\))*\))*/.source,p={heading:{pattern:/;;;.*/,alias:["comment","title"]},comment:/;.*/,string:{pattern:/"(?:[^"\\]|\\.)*"/,greedy:!0,inside:{argument:/[-A-Z]+(?=[.,\s])/,symbol:RegExp("`"+i+"'")}},"quoted-symbol":{pattern:RegExp("#?'"+i),alias:["variable","symbol"]},"lisp-property":{pattern:RegExp(":"+i),alias:"property"},splice:{pattern:RegExp(",@?"+i),alias:["symbol","variable"]},keyword:[{pattern:RegExp(s+"(?:and|(?:cl-)?letf|cl-loop|cond|cons|error|if|(?:lexical-)?let\\*?|message|not|null|or|provide|require|setq|unless|use-package|when|while)"+u),lookbehind:!0},{pattern:RegExp(s+"(?:append|by|collect|concat|do|finally|for|in|return)"+u),lookbehind:!0}],declare:{pattern:n(/declare/.source),lookbehind:!0,alias:"keyword"},interactive:{pattern:n(/interactive/.source),lookbehind:!0,alias:"keyword"},boolean:{pattern:a(/nil|t/.source),lookbehind:!0},number:{pattern:a(/[-+]?\d+(?:\.\d*)?/.source),lookbehind:!0},defvar:{pattern:RegExp(s+"def(?:const|custom|group|var)\\s+"+i),lookbehind:!0,inside:{keyword:/^def[a-z]+/,variable:RegExp(i)}},defun:{pattern:RegExp(s+/(?:cl-)?(?:defmacro|defun\*?)\s+/.source+i+/\s+\(/.source+d+/\)/.source),lookbehind:!0,greedy:!0,inside:{keyword:/^(?:cl-)?def\S+/,arguments:null,function:{pattern:RegExp("(^\\s)"+i),lookbehind:!0},punctuation:/[()]/}},lambda:{pattern:RegExp(s+"lambda\\s+\\(\\s*(?:&?"+i+"(?:\\s+&?"+i+")*\\s*)?\\)"),lookbehind:!0,greedy:!0,inside:{keyword:/^lambda/,arguments:null,punctuation:/[()]/}},car:{pattern:RegExp(s+i),lookbehind:!0},punctuation:[/(?:['`,]?\(|[)\[\]])/,{pattern:/(\s)\.(?=\s)/,lookbehind:!0}]},f={"lisp-marker":RegExp(o),varform:{pattern:RegExp(/\(/.source+i+/\s+(?=\S)/.source+d+/\)/.source),inside:p},argument:{pattern:RegExp(/(^|[\s(])/.source+i),lookbehind:!0,alias:"variable"},rest:p},m="\\S+(?:\\s+\\S+)*",b={pattern:RegExp(s+d+l),lookbehind:!0,inside:{"rest-vars":{pattern:RegExp("&(?:body|rest)\\s+"+m),inside:f},"other-marker-vars":{pattern:RegExp("&(?:aux|optional)\\s+"+m),inside:f},keys:{pattern:RegExp("&key\\s+"+m+"(?:\\s+&allow-other-keys)?"),inside:f},argument:{pattern:RegExp(i),alias:"variable"},punctuation:/[()]/}};p.lambda.inside.arguments=b,p.defun.inside.arguments=r.util.clone(b),p.defun.inside.arguments.inside.sublist=b,r.languages.lisp=p,r.languages.elisp=p,r.languages.emacs=p,r.languages["emacs-lisp"]=p})(t)}return sF}var lF,Oj;function Ait(){if(Oj)return lF;Oj=1,lF=e,e.displayName="livescript",e.aliases=[];function e(t){t.languages.livescript={comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?\*\//,lookbehind:!0},{pattern:/(^|[^\\])#.*/,lookbehind:!0}],"interpolated-string":{pattern:/(^|[^"])("""|")(?:\\[\s\S]|(?!\2)[^\\])*\2(?!")/,lookbehind:!0,greedy:!0,inside:{variable:{pattern:/(^|[^\\])#[a-z_](?:-?[a-z]|[\d_])*/m,lookbehind:!0},interpolation:{pattern:/(^|[^\\])#\{[^}]+\}/m,lookbehind:!0,inside:{"interpolation-punctuation":{pattern:/^#\{|\}$/,alias:"variable"}}},string:/[\s\S]+/}},string:[{pattern:/('''|')(?:\\[\s\S]|(?!\1)[^\\])*\1/,greedy:!0},{pattern:/<\[[\s\S]*?\]>/,greedy:!0},/\\[^\s,;\])}]+/],regex:[{pattern:/\/\/(?:\[[^\r\n\]]*\]|\\.|(?!\/\/)[^\\\[])+\/\/[gimyu]{0,5}/,greedy:!0,inside:{comment:{pattern:/(^|[^\\])#.*/,lookbehind:!0}}},{pattern:/\/(?:\[[^\r\n\]]*\]|\\.|[^/\\\r\n\[])+\/[gimyu]{0,5}/,greedy:!0}],keyword:{pattern:/(^|(?!-).)\b(?:break|case|catch|class|const|continue|default|do|else|extends|fallthrough|finally|for(?: ever)?|function|if|implements|it|let|loop|new|null|otherwise|own|return|super|switch|that|then|this|throw|try|unless|until|var|void|when|while|yield)(?!-)\b/m,lookbehind:!0},"keyword-operator":{pattern:/(^|[^-])\b(?:(?:delete|require|typeof)!|(?:and|by|delete|export|from|import(?: all)?|in|instanceof|is(?: not|nt)?|not|of|or|til|to|typeof|with|xor)(?!-)\b)/m,lookbehind:!0,alias:"operator"},boolean:{pattern:/(^|[^-])\b(?:false|no|off|on|true|yes)(?!-)\b/m,lookbehind:!0},argument:{pattern:/(^|(?!\.&\.)[^&])&(?!&)\d*/m,lookbehind:!0,alias:"variable"},number:/\b(?:\d+~[\da-z]+|\d[\d_]*(?:\.\d[\d_]*)?(?:[a-z]\w*)?)/i,identifier:/[a-z_](?:-?[a-z]|[\d_])*/i,operator:[{pattern:/( )\.(?= )/,lookbehind:!0},/\.(?:[=~]|\.\.?)|\.(?:[&|^]|<<|>>>?)\.|:(?:=|:=?)|&&|\|[|>]|<(?:<<?<?|--?!?|~~?!?|[|=?])?|>[>=?]?|-(?:->?|>)?|\+\+?|@@?|%%?|\*\*?|!(?:~?=|--?>|~?~>)?|~(?:~?>|=)?|==?|\^\^?|[\/?]/],punctuation:/[(){}\[\]|.,:;`]/},t.languages.livescript["interpolated-string"].inside.interpolation.inside.rest=t.languages.livescript}return lF}var uF,Bj;function Fit(){if(Bj)return uF;Bj=1,uF=e,e.displayName="llvm",e.aliases=[];function e(t){(function(r){r.languages.llvm={comment:/;.*/,string:{pattern:/"[^"]*"/,greedy:!0},boolean:/\b(?:false|true)\b/,variable:/[%@!#](?:(?!\d)(?:[-$.\w]|\\[a-f\d]{2})+|\d+)/i,label:/(?!\d)(?:[-$.\w]|\\[a-f\d]{2})+:/i,type:{pattern:/\b(?:double|float|fp128|half|i[1-9]\d*|label|metadata|ppc_fp128|token|void|x86_fp80|x86_mmx)\b/,alias:"class-name"},keyword:/\b[a-z_][a-z_0-9]*\b/,number:/[+-]?\b\d+(?:\.\d+)?(?:[eE][+-]?\d+)?\b|\b0x[\dA-Fa-f]+\b|\b0xK[\dA-Fa-f]{20}\b|\b0x[ML][\dA-Fa-f]{32}\b|\b0xH[\dA-Fa-f]{4}\b/,punctuation:/[{}[\];(),.!*=<>]/}})(t)}return uF}var cF,Pj;function _it(){if(Pj)return cF;Pj=1,cF=e,e.displayName="log",e.aliases=[];function e(t){t.languages.log={string:{pattern:/"(?:[^"\\\r\n]|\\.)*"|'(?![st] | \w)(?:[^'\\\r\n]|\\.)*'/,greedy:!0},exception:{pattern:/(^|[^\w.])[a-z][\w.]*(?:Error|Exception):.*(?:(?:\r\n?|\n)[ \t]*(?:at[ \t].+|\.{3}.*|Caused by:.*))+(?:(?:\r\n?|\n)[ \t]*\.\.\. .*)?/,lookbehind:!0,greedy:!0,alias:["javastacktrace","language-javastacktrace"],inside:t.languages.javastacktrace||{keyword:/\bat\b/,function:/[a-z_][\w$]*(?=\()/,punctuation:/[.:()]/}},level:[{pattern:/\b(?:ALERT|CRIT|CRITICAL|EMERG|EMERGENCY|ERR|ERROR|FAILURE|FATAL|SEVERE)\b/,alias:["error","important"]},{pattern:/\b(?:WARN|WARNING|WRN)\b/,alias:["warning","important"]},{pattern:/\b(?:DISPLAY|INF|INFO|NOTICE|STATUS)\b/,alias:["info","keyword"]},{pattern:/\b(?:DBG|DEBUG|FINE)\b/,alias:["debug","keyword"]},{pattern:/\b(?:FINER|FINEST|TRACE|TRC|VERBOSE|VRB)\b/,alias:["trace","comment"]}],property:{pattern:/((?:^|[\]|])[ \t]*)[a-z_](?:[\w-]|\b\/\b)*(?:[. ]\(?\w(?:[\w-]|\b\/\b)*\)?)*:(?=\s)/im,lookbehind:!0},separator:{pattern:/(^|[^-+])-{3,}|={3,}|\*{3,}|- - /m,lookbehind:!0,alias:"comment"},url:/\b(?:file|ftp|https?):\/\/[^\s|,;'"]*[^\s|,;'">.]/,email:{pattern:/(^|\s)[-\w+.]+@[a-z][a-z0-9-]*(?:\.[a-z][a-z0-9-]*)+(?=\s)/,lookbehind:!0,alias:"url"},"ip-address":{pattern:/\b(?:\d{1,3}(?:\.\d{1,3}){3})\b/,alias:"constant"},"mac-address":{pattern:/\b[a-f0-9]{2}(?::[a-f0-9]{2}){5}\b/i,alias:"constant"},domain:{pattern:/(^|\s)[a-z][a-z0-9-]*(?:\.[a-z][a-z0-9-]*)*\.[a-z][a-z0-9-]+(?=\s)/,lookbehind:!0,alias:"constant"},uuid:{pattern:/\b[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}\b/i,alias:"constant"},hash:{pattern:/\b(?:[a-f0-9]{32}){1,2}\b/i,alias:"constant"},"file-path":{pattern:/\b[a-z]:[\\/][^\s|,;:(){}\[\]"']+|(^|[\s:\[\](>|])\.{0,2}\/\w[^\s|,;:(){}\[\]"']*/i,lookbehind:!0,greedy:!0,alias:"string"},date:{pattern:RegExp(/\b\d{4}[-/]\d{2}[-/]\d{2}(?:T(?=\d{1,2}:)|(?=\s\d{1,2}:))/.source+"|"+/\b\d{1,4}[-/ ](?:\d{1,2}|Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep)[-/ ]\d{2,4}T?\b/.source+"|"+/\b(?:(?:Fri|Mon|Sat|Sun|Thu|Tue|Wed)(?:\s{1,2}(?:Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep))?|Apr|Aug|Dec|Feb|Jan|Jul|Jun|Mar|May|Nov|Oct|Sep)\s{1,2}\d{1,2}\b/.source,"i"),alias:"number"},time:{pattern:/\b\d{1,2}:\d{1,2}:\d{1,2}(?:[.,:]\d+)?(?:\s?[+-]\d{2}:?\d{2}|Z)?\b/,alias:"number"},boolean:/\b(?:false|null|true)\b/i,number:{pattern:/(^|[^.\w])(?:0x[a-f0-9]+|0o[0-7]+|0b[01]+|v?\d[\da-f]*(?:\.\d+)*(?:e[+-]?\d+)?[a-z]{0,3}\b)\b(?!\.\w)/i,lookbehind:!0},operator:/[;:?<=>~/@!$%&+\-|^(){}*#]/,punctuation:/[\[\].,]/}}return cF}var dF,zj;function Rit(){if(zj)return dF;zj=1,dF=e,e.displayName="lolcode",e.aliases=[];function e(t){t.languages.lolcode={comment:[/\bOBTW\s[\s\S]*?\sTLDR\b/,/\bBTW.+/],string:{pattern:/"(?::.|[^":])*"/,inside:{variable:/:\{[^}]+\}/,symbol:[/:\([a-f\d]+\)/i,/:\[[^\]]+\]/,/:[)>o":]/]},greedy:!0},number:/(?:\B-)?(?:\b\d+(?:\.\d*)?|\B\.\d+)/,symbol:{pattern:/(^|\s)(?:A )?(?:BUKKIT|NOOB|NUMBAR|NUMBR|TROOF|YARN)(?=\s|,|$)/,lookbehind:!0,inside:{keyword:/A(?=\s)/}},label:{pattern:/((?:^|\s)(?:IM IN YR|IM OUTTA YR) )[a-zA-Z]\w*/,lookbehind:!0,alias:"string"},function:{pattern:/((?:^|\s)(?:HOW IZ I|I IZ|IZ) )[a-zA-Z]\w*/,lookbehind:!0},keyword:[{pattern:/(^|\s)(?:AN|FOUND YR|GIMMEH|GTFO|HAI|HAS A|HOW IZ I|I HAS A|I IZ|IF U SAY SO|IM IN YR|IM OUTTA YR|IS NOW(?: A)?|ITZ(?: A)?|IZ|KTHX|KTHXBYE|LIEK(?: A)?|MAEK|MEBBE|MKAY|NERFIN|NO WAI|O HAI IM|O RLY\?|OIC|OMG|OMGWTF|R|SMOOSH|SRS|TIL|UPPIN|VISIBLE|WILE|WTF\?|YA RLY|YR)(?=\s|,|$)/,lookbehind:!0},/'Z(?=\s|,|$)/],boolean:{pattern:/(^|\s)(?:FAIL|WIN)(?=\s|,|$)/,lookbehind:!0},variable:{pattern:/(^|\s)IT(?=\s|,|$)/,lookbehind:!0},operator:{pattern:/(^|\s)(?:NOT|BOTH SAEM|DIFFRINT|(?:ALL|ANY|BIGGR|BOTH|DIFF|EITHER|MOD|PRODUKT|QUOSHUNT|SMALLR|SUM|WON) OF)(?=\s|,|$)/,lookbehind:!0},punctuation:/\.{3}|…|,|!/}}return dF}var hF,Hj;function Dit(){if(Hj)return hF;Hj=1,hF=e,e.displayName="magma",e.aliases=[];function e(t){t.languages.magma={output:{pattern:/^(>.*(?:\r(?:\n|(?!\n))|\n))(?!>)(?:.+|(?:\r(?:\n|(?!\n))|\n)(?!>).*)(?:(?:\r(?:\n|(?!\n))|\n)(?!>).*)*/m,lookbehind:!0,greedy:!0},comment:{pattern:/\/\/.*|\/\*[\s\S]*?\*\//,greedy:!0},string:{pattern:/(^|[^\\"])"(?:[^\r\n\\"]|\\.)*"/,lookbehind:!0,greedy:!0},keyword:/\b(?:_|adj|and|assert|assert2|assert3|assigned|break|by|case|cat|catch|clear|cmpeq|cmpne|continue|declare|default|delete|diff|div|do|elif|else|end|eq|error|eval|exists|exit|for|forall|forward|fprintf|freeze|function|ge|gt|if|iload|import|in|intrinsic|is|join|le|load|local|lt|meet|mod|ne|not|notadj|notin|notsubset|or|print|printf|procedure|quit|random|read|readi|repeat|require|requirege|requirerange|restore|return|save|sdiff|select|subset|then|time|to|try|until|vprint|vprintf|vtime|when|where|while|xor)\b/,boolean:/\b(?:false|true)\b/,generator:{pattern:/\b[a-z_]\w*(?=\s*<)/i,alias:"class-name"},function:/\b[a-z_]\w*(?=\s*\()/i,number:{pattern:/(^|[^\w.]|\.\.)(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?(?:_[a-z]?)?(?=$|[^\w.]|\.\.)/,lookbehind:!0},operator:/->|[-+*/^~!|#=]|:=|\.\./,punctuation:/[()[\]{}<>,;.:]/}}return hF}var pF,Uj;function Nit(){if(Uj)return pF;Uj=1,pF=e,e.displayName="makefile",e.aliases=[];function e(t){t.languages.makefile={comment:{pattern:/(^|[^\\])#(?:\\(?:\r\n|[\s\S])|[^\\\r\n])*/,lookbehind:!0},string:{pattern:/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0},"builtin-target":{pattern:/\.[A-Z][^:#=\s]+(?=\s*:(?!=))/,alias:"builtin"},target:{pattern:/^(?:[^:=\s]|[ \t]+(?![\s:]))+(?=\s*:(?!=))/m,alias:"symbol",inside:{variable:/\$+(?:(?!\$)[^(){}:#=\s]+|(?=[({]))/}},variable:/\$+(?:(?!\$)[^(){}:#=\s]+|\([@*%<^+?][DF]\)|(?=[({]))/,keyword:/-include\b|\b(?:define|else|endef|endif|export|ifn?def|ifn?eq|include|override|private|sinclude|undefine|unexport|vpath)\b/,function:{pattern:/(\()(?:abspath|addsuffix|and|basename|call|dir|error|eval|file|filter(?:-out)?|findstring|firstword|flavor|foreach|guile|if|info|join|lastword|load|notdir|or|origin|patsubst|realpath|shell|sort|strip|subst|suffix|value|warning|wildcard|word(?:list|s)?)(?=[ \t])/,lookbehind:!0},operator:/(?:::|[?:+!])?=|[|@]/,punctuation:/[:;(){}]/}}return pF}var fF,$j;function Lit(){if($j)return fF;$j=1,fF=e,e.displayName="markdown",e.aliases=["md"];function e(t){(function(r){var n=/(?:\\.|[^\\\n\r]|(?:\n|\r\n?)(?![\r\n]))/.source;function a(f){return f=f.replace(/<inner>/g,function(){return n}),RegExp(/((?:^|[^\\])(?:\\{2})*)/.source+"(?:"+f+")")}var i=/(?:\\.|``(?:[^`\r\n]|`(?!`))+``|`[^`\r\n]+`|[^\\|\r\n`])+/.source,o=/\|?__(?:\|__)+\|?(?:(?:\n|\r\n?)|(?![\s\S]))/.source.replace(/__/g,function(){return i}),s=/\|?[ \t]*:?-{3,}:?[ \t]*(?:\|[ \t]*:?-{3,}:?[ \t]*)+\|?(?:\n|\r\n?)/.source;r.languages.markdown=r.languages.extend("markup",{}),r.languages.insertBefore("markdown","prolog",{"front-matter-block":{pattern:/(^(?:\s*[\r\n])?)---(?!.)[\s\S]*?[\r\n]---(?!.)/,lookbehind:!0,greedy:!0,inside:{punctuation:/^---|---$/,"front-matter":{pattern:/\S+(?:\s+\S+)*/,alias:["yaml","language-yaml"],inside:r.languages.yaml}}},blockquote:{pattern:/^>(?:[\t ]*>)*/m,alias:"punctuation"},table:{pattern:RegExp("^"+o+s+"(?:"+o+")*","m"),inside:{"table-data-rows":{pattern:RegExp("^("+o+s+")(?:"+o+")*$"),lookbehind:!0,inside:{"table-data":{pattern:RegExp(i),inside:r.languages.markdown},punctuation:/\|/}},"table-line":{pattern:RegExp("^("+o+")"+s+"$"),lookbehind:!0,inside:{punctuation:/\||:?-{3,}:?/}},"table-header-row":{pattern:RegExp("^"+o+"$"),inside:{"table-header":{pattern:RegExp(i),alias:"important",inside:r.languages.markdown},punctuation:/\|/}}}},code:[{pattern:/((?:^|\n)[ \t]*\n|(?:^|\r\n?)[ \t]*\r\n?)(?: {4}|\t).+(?:(?:\n|\r\n?)(?: {4}|\t).+)*/,lookbehind:!0,alias:"keyword"},{pattern:/^```[\s\S]*?^```$/m,greedy:!0,inside:{"code-block":{pattern:/^(```.*(?:\n|\r\n?))[\s\S]+?(?=(?:\n|\r\n?)^```$)/m,lookbehind:!0},"code-language":{pattern:/^(```).+/,lookbehind:!0},punctuation:/```/}}],title:[{pattern:/\S.*(?:\n|\r\n?)(?:==+|--+)(?=[ \t]*$)/m,alias:"important",inside:{punctuation:/==+$|--+$/}},{pattern:/(^\s*)#.+/m,lookbehind:!0,alias:"important",inside:{punctuation:/^#+|#+$/}}],hr:{pattern:/(^\s*)([*-])(?:[\t ]*\2){2,}(?=\s*$)/m,lookbehind:!0,alias:"punctuation"},list:{pattern:/(^\s*)(?:[*+-]|\d+\.)(?=[\t ].)/m,lookbehind:!0,alias:"punctuation"},"url-reference":{pattern:/!?\[[^\]]+\]:[\t ]+(?:\S+|<(?:\\.|[^>\\])+>)(?:[\t ]+(?:"(?:\\.|[^"\\])*"|'(?:\\.|[^'\\])*'|\((?:\\.|[^)\\])*\)))?/,inside:{variable:{pattern:/^(!?\[)[^\]]+/,lookbehind:!0},string:/(?:"(?:\\.|[^"\\])*"|'(?:\\.|[^'\\])*'|\((?:\\.|[^)\\])*\))$/,punctuation:/^[\[\]!:]|[<>]/},alias:"url"},bold:{pattern:a(/\b__(?:(?!_)<inner>|_(?:(?!_)<inner>)+_)+__\b|\*\*(?:(?!\*)<inner>|\*(?:(?!\*)<inner>)+\*)+\*\*/.source),lookbehind:!0,greedy:!0,inside:{content:{pattern:/(^..)[\s\S]+(?=..$)/,lookbehind:!0,inside:{}},punctuation:/\*\*|__/}},italic:{pattern:a(/\b_(?:(?!_)<inner>|__(?:(?!_)<inner>)+__)+_\b|\*(?:(?!\*)<inner>|\*\*(?:(?!\*)<inner>)+\*\*)+\*/.source),lookbehind:!0,greedy:!0,inside:{content:{pattern:/(^.)[\s\S]+(?=.$)/,lookbehind:!0,inside:{}},punctuation:/[*_]/}},strike:{pattern:a(/(~~?)(?:(?!~)<inner>)+\2/.source),lookbehind:!0,greedy:!0,inside:{content:{pattern:/(^~~?)[\s\S]+(?=\1$)/,lookbehind:!0,inside:{}},punctuation:/~~?/}},"code-snippet":{pattern:/(^|[^\\`])(?:``[^`\r\n]+(?:`[^`\r\n]+)*``(?!`)|`[^`\r\n]+`(?!`))/,lookbehind:!0,greedy:!0,alias:["code","keyword"]},url:{pattern:a(/!?\[(?:(?!\])<inner>)+\](?:\([^\s)]+(?:[\t ]+"(?:\\.|[^"\\])*")?\)|[ \t]?\[(?:(?!\])<inner>)+\])/.source),lookbehind:!0,greedy:!0,inside:{operator:/^!/,content:{pattern:/(^\[)[^\]]+(?=\])/,lookbehind:!0,inside:{}},variable:{pattern:/(^\][ \t]?\[)[^\]]+(?=\]$)/,lookbehind:!0},url:{pattern:/(^\]\()[^\s)]+/,lookbehind:!0},string:{pattern:/(^[ \t]+)"(?:\\.|[^"\\])*"(?=\)$)/,lookbehind:!0}}}}),["url","bold","italic","strike"].forEach(function(f){["url","bold","italic","strike","code-snippet"].forEach(function(m){f!==m&&(r.languages.markdown[f].inside.content.inside[m]=r.languages.markdown[m])})}),r.hooks.add("after-tokenize",function(f){if(f.language!=="markdown"&&f.language!=="md")return;function m(b){if(!(!b||typeof b=="string"))for(var y=0,S=b.length;y<S;y++){var w=b[y];if(w.type!=="code"){m(w.content);continue}var E=w.content[1],k=w.content[3];if(E&&k&&E.type==="code-language"&&k.type==="code-block"&&typeof E.content=="string"){var _=E.content.replace(/\b#/g,"sharp").replace(/\b\+\+/g,"pp");_=(/[a-z][\w-]*/i.exec(_)||[""])[0].toLowerCase();var R="language-"+_;k.alias?typeof k.alias=="string"?k.alias=[k.alias,R]:k.alias.push(R):k.alias=[R]}}}m(f.tokens)}),r.hooks.add("wrap",function(f){if(f.type==="code-block"){for(var m="",b=0,y=f.classes.length;b<y;b++){var S=f.classes[b],w=/language-(.+)/.exec(S);if(w){m=w[1];break}}var E=r.languages[m];if(E)f.content=r.highlight(p(f.content.value),E,m);else if(m&&m!=="none"&&r.plugins.autoloader){var k="md-"+new Date().valueOf()+"-"+Math.floor(Math.random()*1e16);f.attributes.id=k,r.plugins.autoloader.loadLanguages(m,function(){var _=document.getElementById(k);_&&(_.innerHTML=r.highlight(_.textContent,r.languages[m],m))})}}});var l=RegExp(r.languages.markup.tag.pattern.source,"gi"),u={amp:"&",lt:"<",gt:">",quot:'"'},d=String.fromCodePoint||String.fromCharCode;function p(f){var m=f.replace(l,"");return m=m.replace(/&(\w{1,8}|#x?[\da-f]{1,8});/gi,function(b,y){if(y=y.toLowerCase(),y[0]==="#"){var S;return y[1]==="x"?S=parseInt(y.slice(2),16):S=Number(y.slice(1)),d(S)}else{var w=u[y];return w||b}}),m}r.languages.md=r.languages.markdown})(t)}return fF}var mF,Gj;function Mit(){if(Gj)return mF;Gj=1,mF=e,e.displayName="matlab",e.aliases=[];function e(t){t.languages.matlab={comment:[/%\{[\s\S]*?\}%/,/%.+/],string:{pattern:/\B'(?:''|[^'\r\n])*'/,greedy:!0},number:/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[eE][+-]?\d+)?(?:[ij])?|\b[ij]\b/,keyword:/\b(?:NaN|break|case|catch|continue|else|elseif|end|for|function|if|inf|otherwise|parfor|pause|pi|return|switch|try|while)\b/,function:/\b(?!\d)\w+(?=\s*\()/,operator:/\.?[*^\/\\']|[+\-:@]|[<>=~]=?|&&?|\|\|?/,punctuation:/\.{3}|[.,;\[\](){}!]/}}return mF}var gF,jj;function Iit(){if(jj)return gF;jj=1,gF=e,e.displayName="maxscript",e.aliases=[];function e(t){(function(r){var n=/\b(?:about|and|animate|as|at|attributes|by|case|catch|collect|continue|coordsys|do|else|exit|fn|for|from|function|global|if|in|local|macroscript|mapped|max|not|of|off|on|or|parameters|persistent|plugin|rcmenu|return|rollout|set|struct|then|throw|to|tool|try|undo|utility|when|where|while|with)\b/i;r.languages.maxscript={comment:{pattern:/\/\*[\s\S]*?(?:\*\/|$)|--.*/,greedy:!0},string:{pattern:/(^|[^"\\@])(?:"(?:[^"\\]|\\[\s\S])*"|@"[^"]*")/,lookbehind:!0,greedy:!0},path:{pattern:/\$(?:[\w/\\.*?]|'[^']*')*/,greedy:!0,alias:"string"},"function-call":{pattern:RegExp("((?:"+(/^/.source+"|"+/[;=<>+\-*/^({\[]/.source+"|"+/\b(?:and|by|case|catch|collect|do|else|if|in|not|or|return|then|to|try|where|while|with)\b/.source)+")[ 	]*)(?!"+n.source+")"+/[a-z_]\w*\b/.source+"(?=[ 	]*(?:"+("(?!"+n.source+")"+/[a-z_]/.source+"|"+/\d|-\.?\d/.source+"|"+/[({'"$@#?]/.source)+"))","im"),lookbehind:!0,greedy:!0,alias:"function"},"function-definition":{pattern:/(\b(?:fn|function)\s+)\w+\b/i,lookbehind:!0,alias:"function"},argument:{pattern:/\b[a-z_]\w*(?=:)/i,alias:"attr-name"},keyword:n,boolean:/\b(?:false|true)\b/,time:{pattern:/(^|[^\w.])(?:(?:(?:\d+(?:\.\d*)?|\.\d+)(?:[eEdD][+-]\d+|[LP])?[msft])+|\d+:\d+(?:\.\d*)?)(?![\w.:])/,lookbehind:!0,alias:"number"},number:[{pattern:/(^|[^\w.])(?:(?:\d+(?:\.\d*)?|\.\d+)(?:[eEdD][+-]\d+|[LP])?|0x[a-fA-F0-9]+)(?![\w.:])/,lookbehind:!0},/\b(?:e|pi)\b/],constant:/\b(?:dontcollect|ok|silentValue|undefined|unsupplied)\b/,color:{pattern:/\b(?:black|blue|brown|gray|green|orange|red|white|yellow)\b/i,alias:"constant"},operator:/[-+*/<>=!]=?|[&^?]|#(?!\()/,punctuation:/[()\[\]{}.:,;]|#(?=\()|\\$/m}})(t)}return gF}var bF,qj;function Oit(){if(qj)return bF;qj=1,bF=e,e.displayName="mel",e.aliases=[];function e(t){t.languages.mel={comment:/\/\/.*/,code:{pattern:/`(?:\\.|[^\\`\r\n])*`/,greedy:!0,alias:"italic",inside:{delimiter:{pattern:/^`|`$/,alias:"punctuation"}}},string:{pattern:/"(?:\\.|[^\\"\r\n])*"/,greedy:!0},variable:/\$\w+/,number:/\b0x[\da-fA-F]+\b|\b\d+(?:\.\d*)?|\B\.\d+/,flag:{pattern:/-[^\d\W]\w*/,alias:"operator"},keyword:/\b(?:break|case|continue|default|do|else|float|for|global|if|in|int|matrix|proc|return|string|switch|vector|while)\b/,function:/\b\w+(?=\()|\b(?:CBG|HfAddAttractorToAS|HfAssignAS|HfBuildEqualMap|HfBuildFurFiles|HfBuildFurImages|HfCancelAFR|HfConnectASToHF|HfCreateAttractor|HfDeleteAS|HfEditAS|HfPerformCreateAS|HfRemoveAttractorFromAS|HfSelectAttached|HfSelectAttractors|HfUnAssignAS|Mayatomr|about|abs|addAttr|addAttributeEditorNodeHelp|addDynamic|addNewShelfTab|addPP|addPanelCategory|addPrefixToName|advanceToNextDrivenKey|affectedNet|affects|aimConstraint|air|alias|aliasAttr|align|alignCtx|alignCurve|alignSurface|allViewFit|ambientLight|angle|angleBetween|animCone|animCurveEditor|animDisplay|animView|annotate|appendStringArray|applicationName|applyAttrPreset|applyTake|arcLenDimContext|arcLengthDimension|arclen|arrayMapper|art3dPaintCtx|artAttrCtx|artAttrPaintVertexCtx|artAttrSkinPaintCtx|artAttrTool|artBuildPaintMenu|artFluidAttrCtx|artPuttyCtx|artSelectCtx|artSetPaintCtx|artUserPaintCtx|assignCommand|assignInputDevice|assignViewportFactories|attachCurve|attachDeviceAttr|attachSurface|attrColorSliderGrp|attrCompatibility|attrControlGrp|attrEnumOptionMenu|attrEnumOptionMenuGrp|attrFieldGrp|attrFieldSliderGrp|attrNavigationControlGrp|attrPresetEditWin|attributeExists|attributeInfo|attributeMenu|attributeQuery|autoKeyframe|autoPlace|bakeClip|bakeFluidShading|bakePartialHistory|bakeResults|bakeSimulation|basename|basenameEx|batchRender|bessel|bevel|bevelPlus|binMembership|bindSkin|blend2|blendShape|blendShapeEditor|blendShapePanel|blendTwoAttr|blindDataType|boneLattice|boundary|boxDollyCtx|boxZoomCtx|bufferCurve|buildBookmarkMenu|buildKeyframeMenu|button|buttonManip|cacheFile|cacheFileCombine|cacheFileMerge|cacheFileTrack|camera|cameraView|canCreateManip|canvas|capitalizeString|catch|catchQuiet|ceil|changeSubdivComponentDisplayLevel|changeSubdivRegion|channelBox|character|characterMap|characterOutlineEditor|characterize|chdir|checkBox|checkBoxGrp|checkDefaultRenderGlobals|choice|circle|circularFillet|clamp|clear|clearCache|clip|clipEditor|clipEditorCurrentTimeCtx|clipSchedule|clipSchedulerOutliner|clipTrimBefore|closeCurve|closeSurface|cluster|cmdFileOutput|cmdScrollFieldExecuter|cmdScrollFieldReporter|cmdShell|coarsenSubdivSelectionList|collision|color|colorAtPoint|colorEditor|colorIndex|colorIndexSliderGrp|colorSliderButtonGrp|colorSliderGrp|columnLayout|commandEcho|commandLine|commandPort|compactHairSystem|componentEditor|compositingInterop|computePolysetVolume|condition|cone|confirmDialog|connectAttr|connectControl|connectDynamic|connectJoint|connectionInfo|constrain|constrainValue|constructionHistory|container|containsMultibyte|contextInfo|control|convertFromOldLayers|convertIffToPsd|convertLightmap|convertSolidTx|convertTessellation|convertUnit|copyArray|copyFlexor|copyKey|copySkinWeights|cos|cpButton|cpCache|cpClothSet|cpCollision|cpConstraint|cpConvClothToMesh|cpForces|cpGetSolverAttr|cpPanel|cpProperty|cpRigidCollisionFilter|cpSeam|cpSetEdit|cpSetSolverAttr|cpSolver|cpSolverTypes|cpTool|cpUpdateClothUVs|createDisplayLayer|createDrawCtx|createEditor|createLayeredPsdFile|createMotionField|createNewShelf|createNode|createRenderLayer|createSubdivRegion|cross|crossProduct|ctxAbort|ctxCompletion|ctxEditMode|ctxTraverse|currentCtx|currentTime|currentTimeCtx|currentUnit|curve|curveAddPtCtx|curveCVCtx|curveEPCtx|curveEditorCtx|curveIntersect|curveMoveEPCtx|curveOnSurface|curveSketchCtx|cutKey|cycleCheck|cylinder|dagPose|date|defaultLightListCheckBox|defaultNavigation|defineDataServer|defineVirtualDevice|deformer|deg_to_rad|delete|deleteAttr|deleteShadingGroupsAndMaterials|deleteShelfTab|deleteUI|deleteUnusedBrushes|delrandstr|detachCurve|detachDeviceAttr|detachSurface|deviceEditor|devicePanel|dgInfo|dgdirty|dgeval|dgtimer|dimWhen|directKeyCtx|directionalLight|dirmap|dirname|disable|disconnectAttr|disconnectJoint|diskCache|displacementToPoly|displayAffected|displayColor|displayCull|displayLevelOfDetail|displayPref|displayRGBColor|displaySmoothness|displayStats|displayString|displaySurface|distanceDimContext|distanceDimension|doBlur|dolly|dollyCtx|dopeSheetEditor|dot|dotProduct|doubleProfileBirailSurface|drag|dragAttrContext|draggerContext|dropoffLocator|duplicate|duplicateCurve|duplicateSurface|dynCache|dynControl|dynExport|dynExpression|dynGlobals|dynPaintEditor|dynParticleCtx|dynPref|dynRelEdPanel|dynRelEditor|dynamicLoad|editAttrLimits|editDisplayLayerGlobals|editDisplayLayerMembers|editRenderLayerAdjustment|editRenderLayerGlobals|editRenderLayerMembers|editor|editorTemplate|effector|emit|emitter|enableDevice|encodeString|endString|endsWith|env|equivalent|equivalentTol|erf|error|eval|evalDeferred|evalEcho|event|exactWorldBoundingBox|exclusiveLightCheckBox|exec|executeForEachObject|exists|exp|expression|expressionEditorListen|extendCurve|extendSurface|extrude|fcheck|fclose|feof|fflush|fgetline|fgetword|file|fileBrowserDialog|fileDialog|fileExtension|fileInfo|filetest|filletCurve|filter|filterCurve|filterExpand|filterStudioImport|findAllIntersections|findAnimCurves|findKeyframe|findMenuItem|findRelatedSkinCluster|finder|firstParentOf|fitBspline|flexor|floatEq|floatField|floatFieldGrp|floatScrollBar|floatSlider|floatSlider2|floatSliderButtonGrp|floatSliderGrp|floor|flow|fluidCacheInfo|fluidEmitter|fluidVoxelInfo|flushUndo|fmod|fontDialog|fopen|formLayout|format|fprint|frameLayout|fread|freeFormFillet|frewind|fromNativePath|fwrite|gamma|gauss|geometryConstraint|getApplicationVersionAsFloat|getAttr|getClassification|getDefaultBrush|getFileList|getFluidAttr|getInputDeviceRange|getMayaPanelTypes|getModifiers|getPanel|getParticleAttr|getPluginResource|getenv|getpid|glRender|glRenderEditor|globalStitch|gmatch|goal|gotoBindPose|grabColor|gradientControl|gradientControlNoAttr|graphDollyCtx|graphSelectContext|graphTrackCtx|gravity|grid|gridLayout|group|groupObjectsByName|hardenPointCurve|hardware|hardwareRenderPanel|headsUpDisplay|headsUpMessage|help|helpLine|hermite|hide|hilite|hitTest|hotBox|hotkey|hotkeyCheck|hsv_to_rgb|hudButton|hudSlider|hudSliderButton|hwReflectionMap|hwRender|hwRenderLoad|hyperGraph|hyperPanel|hyperShade|hypot|iconTextButton|iconTextCheckBox|iconTextRadioButton|iconTextRadioCollection|iconTextScrollList|iconTextStaticLabel|ikHandle|ikHandleCtx|ikHandleDisplayScale|ikSolver|ikSplineHandleCtx|ikSystem|ikSystemInfo|ikfkDisplayMethod|illustratorCurves|image|imfPlugins|inheritTransform|insertJoint|insertJointCtx|insertKeyCtx|insertKnotCurve|insertKnotSurface|instance|instanceable|instancer|intField|intFieldGrp|intScrollBar|intSlider|intSliderGrp|interToUI|internalVar|intersect|iprEngine|isAnimCurve|isConnected|isDirty|isParentOf|isSameObject|isTrue|isValidObjectName|isValidString|isValidUiName|isolateSelect|itemFilter|itemFilterAttr|itemFilterRender|itemFilterType|joint|jointCluster|jointCtx|jointDisplayScale|jointLattice|keyTangent|keyframe|keyframeOutliner|keyframeRegionCurrentTimeCtx|keyframeRegionDirectKeyCtx|keyframeRegionDollyCtx|keyframeRegionInsertKeyCtx|keyframeRegionMoveKeyCtx|keyframeRegionScaleKeyCtx|keyframeRegionSelectKeyCtx|keyframeRegionSetKeyCtx|keyframeRegionTrackCtx|keyframeStats|lassoContext|lattice|latticeDeformKeyCtx|launch|launchImageEditor|layerButton|layeredShaderPort|layeredTexturePort|layout|layoutDialog|lightList|lightListEditor|lightListPanel|lightlink|lineIntersection|linearPrecision|linstep|listAnimatable|listAttr|listCameras|listConnections|listDeviceAttachments|listHistory|listInputDeviceAxes|listInputDeviceButtons|listInputDevices|listMenuAnnotation|listNodeTypes|listPanelCategories|listRelatives|listSets|listTransforms|listUnselected|listerEditor|loadFluid|loadNewShelf|loadPlugin|loadPluginLanguageResources|loadPrefObjects|localizedPanelLabel|lockNode|loft|log|longNameOf|lookThru|ls|lsThroughFilter|lsType|lsUI|mag|makeIdentity|makeLive|makePaintable|makeRoll|makeSingleSurface|makeTubeOn|makebot|manipMoveContext|manipMoveLimitsCtx|manipOptions|manipRotateContext|manipRotateLimitsCtx|manipScaleContext|manipScaleLimitsCtx|marker|match|max|memory|menu|menuBarLayout|menuEditor|menuItem|menuItemToShelf|menuSet|menuSetPref|messageLine|min|minimizeApp|mirrorJoint|modelCurrentTimeCtx|modelEditor|modelPanel|mouse|movIn|movOut|move|moveIKtoFK|moveKeyCtx|moveVertexAlongDirection|multiProfileBirailSurface|mute|nParticle|nameCommand|nameField|namespace|namespaceInfo|newPanelItems|newton|nodeCast|nodeIconButton|nodeOutliner|nodePreset|nodeType|noise|nonLinear|normalConstraint|normalize|nurbsBoolean|nurbsCopyUVSet|nurbsCube|nurbsEditUV|nurbsPlane|nurbsSelect|nurbsSquare|nurbsToPoly|nurbsToPolygonsPref|nurbsToSubdiv|nurbsToSubdivPref|nurbsUVSet|nurbsViewDirectionVector|objExists|objectCenter|objectLayer|objectType|objectTypeUI|obsoleteProc|oceanNurbsPreviewPlane|offsetCurve|offsetCurveOnSurface|offsetSurface|openGLExtension|openMayaPref|optionMenu|optionMenuGrp|optionVar|orbit|orbitCtx|orientConstraint|outlinerEditor|outlinerPanel|overrideModifier|paintEffectsDisplay|pairBlend|palettePort|paneLayout|panel|panelConfiguration|panelHistory|paramDimContext|paramDimension|paramLocator|parent|parentConstraint|particle|particleExists|particleInstancer|particleRenderInfo|partition|pasteKey|pathAnimation|pause|pclose|percent|performanceOptions|pfxstrokes|pickWalk|picture|pixelMove|planarSrf|plane|play|playbackOptions|playblast|plugAttr|plugNode|pluginInfo|pluginResourceUtil|pointConstraint|pointCurveConstraint|pointLight|pointMatrixMult|pointOnCurve|pointOnSurface|pointPosition|poleVectorConstraint|polyAppend|polyAppendFacetCtx|polyAppendVertex|polyAutoProjection|polyAverageNormal|polyAverageVertex|polyBevel|polyBlendColor|polyBlindData|polyBoolOp|polyBridgeEdge|polyCacheMonitor|polyCheck|polyChipOff|polyClipboard|polyCloseBorder|polyCollapseEdge|polyCollapseFacet|polyColorBlindData|polyColorDel|polyColorPerVertex|polyColorSet|polyCompare|polyCone|polyCopyUV|polyCrease|polyCreaseCtx|polyCreateFacet|polyCreateFacetCtx|polyCube|polyCut|polyCutCtx|polyCylinder|polyCylindricalProjection|polyDelEdge|polyDelFacet|polyDelVertex|polyDuplicateAndConnect|polyDuplicateEdge|polyEditUV|polyEditUVShell|polyEvaluate|polyExtrudeEdge|polyExtrudeFacet|polyExtrudeVertex|polyFlipEdge|polyFlipUV|polyForceUV|polyGeoSampler|polyHelix|polyInfo|polyInstallAction|polyLayoutUV|polyListComponentConversion|polyMapCut|polyMapDel|polyMapSew|polyMapSewMove|polyMergeEdge|polyMergeEdgeCtx|polyMergeFacet|polyMergeFacetCtx|polyMergeUV|polyMergeVertex|polyMirrorFace|polyMoveEdge|polyMoveFacet|polyMoveFacetUV|polyMoveUV|polyMoveVertex|polyNormal|polyNormalPerVertex|polyNormalizeUV|polyOptUvs|polyOptions|polyOutput|polyPipe|polyPlanarProjection|polyPlane|polyPlatonicSolid|polyPoke|polyPrimitive|polyPrism|polyProjection|polyPyramid|polyQuad|polyQueryBlindData|polyReduce|polySelect|polySelectConstraint|polySelectConstraintMonitor|polySelectCtx|polySelectEditCtx|polySeparate|polySetToFaceNormal|polySewEdge|polyShortestPathCtx|polySmooth|polySoftEdge|polySphere|polySphericalProjection|polySplit|polySplitCtx|polySplitEdge|polySplitRing|polySplitVertex|polyStraightenUVBorder|polySubdivideEdge|polySubdivideFacet|polyToSubdiv|polyTorus|polyTransfer|polyTriangulate|polyUVSet|polyUnite|polyWedgeFace|popen|popupMenu|pose|pow|preloadRefEd|print|progressBar|progressWindow|projFileViewer|projectCurve|projectTangent|projectionContext|projectionManip|promptDialog|propModCtx|propMove|psdChannelOutliner|psdEditTextureFile|psdExport|psdTextureFile|putenv|pwd|python|querySubdiv|quit|rad_to_deg|radial|radioButton|radioButtonGrp|radioCollection|radioMenuItemCollection|rampColorPort|rand|randomizeFollicles|randstate|rangeControl|readTake|rebuildCurve|rebuildSurface|recordAttr|recordDevice|redo|reference|referenceEdit|referenceQuery|refineSubdivSelectionList|refresh|refreshAE|registerPluginResource|rehash|reloadImage|removeJoint|removeMultiInstance|removePanelCategory|rename|renameAttr|renameSelectionList|renameUI|render|renderGlobalsNode|renderInfo|renderLayerButton|renderLayerParent|renderLayerPostProcess|renderLayerUnparent|renderManip|renderPartition|renderQualityNode|renderSettings|renderThumbnailUpdate|renderWindowEditor|renderWindowSelectContext|renderer|reorder|reorderDeformers|requires|reroot|resampleFluid|resetAE|resetPfxToPolyCamera|resetTool|resolutionNode|retarget|reverseCurve|reverseSurface|revolve|rgb_to_hsv|rigidBody|rigidSolver|roll|rollCtx|rootOf|rot|rotate|rotationInterpolation|roundConstantRadius|rowColumnLayout|rowLayout|runTimeCommand|runup|sampleImage|saveAllShelves|saveAttrPreset|saveFluid|saveImage|saveInitialState|saveMenu|savePrefObjects|savePrefs|saveShelf|saveToolSettings|scale|scaleBrushBrightness|scaleComponents|scaleConstraint|scaleKey|scaleKeyCtx|sceneEditor|sceneUIReplacement|scmh|scriptCtx|scriptEditorInfo|scriptJob|scriptNode|scriptTable|scriptToShelf|scriptedPanel|scriptedPanelType|scrollField|scrollLayout|sculpt|searchPathArray|seed|selLoadSettings|select|selectContext|selectCurveCV|selectKey|selectKeyCtx|selectKeyframeRegionCtx|selectMode|selectPref|selectPriority|selectType|selectedNodes|selectionConnection|separator|setAttr|setAttrEnumResource|setAttrMapping|setAttrNiceNameResource|setConstraintRestPosition|setDefaultShadingGroup|setDrivenKeyframe|setDynamic|setEditCtx|setEditor|setFluidAttr|setFocus|setInfinity|setInputDeviceMapping|setKeyCtx|setKeyPath|setKeyframe|setKeyframeBlendshapeTargetWts|setMenuMode|setNodeNiceNameResource|setNodeTypeFlag|setParent|setParticleAttr|setPfxToPolyCamera|setPluginResource|setProject|setStampDensity|setStartupMessage|setState|setToolTo|setUITemplate|setXformManip|sets|shadingConnection|shadingGeometryRelCtx|shadingLightRelCtx|shadingNetworkCompare|shadingNode|shapeCompare|shelfButton|shelfLayout|shelfTabLayout|shellField|shortNameOf|showHelp|showHidden|showManipCtx|showSelectionInTitle|showShadingGroupAttrEditor|showWindow|sign|simplify|sin|singleProfileBirailSurface|size|sizeBytes|skinCluster|skinPercent|smoothCurve|smoothTangentSurface|smoothstep|snap2to2|snapKey|snapMode|snapTogetherCtx|snapshot|soft|softMod|softModCtx|sort|sound|soundControl|source|spaceLocator|sphere|sphrand|spotLight|spotLightPreviewPort|spreadSheetEditor|spring|sqrt|squareSurface|srtContext|stackTrace|startString|startsWith|stitchAndExplodeShell|stitchSurface|stitchSurfacePoints|strcmp|stringArrayCatenate|stringArrayContains|stringArrayCount|stringArrayInsertAtIndex|stringArrayIntersector|stringArrayRemove|stringArrayRemoveAtIndex|stringArrayRemoveDuplicates|stringArrayRemoveExact|stringArrayToString|stringToStringArray|strip|stripPrefixFromName|stroke|subdAutoProjection|subdCleanTopology|subdCollapse|subdDuplicateAndConnect|subdEditUV|subdListComponentConversion|subdMapCut|subdMapSewMove|subdMatchTopology|subdMirror|subdToBlind|subdToPoly|subdTransferUVsToCache|subdiv|subdivCrease|subdivDisplaySmoothness|substitute|substituteAllString|substituteGeometry|substring|surface|surfaceSampler|surfaceShaderList|swatchDisplayPort|switchTable|symbolButton|symbolCheckBox|sysFile|system|tabLayout|tan|tangentConstraint|texLatticeDeformContext|texManipContext|texMoveContext|texMoveUVShellContext|texRotateContext|texScaleContext|texSelectContext|texSelectShortestPathCtx|texSmudgeUVContext|texWinToolCtx|text|textCurves|textField|textFieldButtonGrp|textFieldGrp|textManip|textScrollList|textToShelf|textureDisplacePlane|textureHairColor|texturePlacementContext|textureWindow|threadCount|threePointArcCtx|timeControl|timePort|timerX|toNativePath|toggle|toggleAxis|toggleWindowVisibility|tokenize|tokenizeList|tolerance|tolower|toolButton|toolCollection|toolDropped|toolHasOptions|toolPropertyWindow|torus|toupper|trace|track|trackCtx|transferAttributes|transformCompare|transformLimits|translator|trim|trunc|truncateFluidCache|truncateHairCache|tumble|tumbleCtx|turbulence|twoPointArcCtx|uiRes|uiTemplate|unassignInputDevice|undo|undoInfo|ungroup|uniform|unit|unloadPlugin|untangleUV|untitledFileName|untrim|upAxis|updateAE|userCtx|uvLink|uvSnapshot|validateShelfName|vectorize|view2dToolCtx|viewCamera|viewClipPlane|viewFit|viewHeadOn|viewLookAt|viewManip|viewPlace|viewSet|visor|volumeAxis|vortex|waitCursor|warning|webBrowser|webBrowserPrefs|whatIs|window|windowPref|wire|wireContext|workspace|wrinkle|wrinkleContext|writeTake|xbmLangPathList|xform)\b/,operator:[/\+[+=]?|-[-=]?|&&|\|\||[<>]=|[*\/!=]=?|[%^]/,{pattern:/(^|[^<])<(?!<)/,lookbehind:!0},{pattern:/(^|[^>])>(?!>)/,lookbehind:!0}],punctuation:/<<|>>|[.,:;?\[\](){}]/},t.languages.mel.code.inside.rest=t.languages.mel}return bF}var yF,Wj;function Bit(){if(Wj)return yF;Wj=1,yF=e,e.displayName="mermaid",e.aliases=[];function e(t){t.languages.mermaid={comment:{pattern:/%%.*/,greedy:!0},style:{pattern:/^([ \t]*(?:classDef|linkStyle|style)[ \t]+[\w$-]+[ \t]+)\w.*[^\s;]/m,lookbehind:!0,inside:{property:/\b\w[\w-]*(?=[ \t]*:)/,operator:/:/,punctuation:/,/}},"inter-arrow-label":{pattern:/([^<>ox.=-])(?:-[-.]|==)(?![<>ox.=-])[ \t]*(?:"[^"\r\n]*"|[^\s".=-](?:[^\r\n.=-]*[^\s.=-])?)[ \t]*(?:\.+->?|--+[->]|==+[=>])(?![<>ox.=-])/,lookbehind:!0,greedy:!0,inside:{arrow:{pattern:/(?:\.+->?|--+[->]|==+[=>])$/,alias:"operator"},label:{pattern:/^([\s\S]{2}[ \t]*)\S(?:[\s\S]*\S)?/,lookbehind:!0,alias:"property"},"arrow-head":{pattern:/^\S+/,alias:["arrow","operator"]}}},arrow:[{pattern:/(^|[^{}|o.-])[|}][|o](?:--|\.\.)[|o][|{](?![{}|o.-])/,lookbehind:!0,alias:"operator"},{pattern:/(^|[^<>ox.=-])(?:[<ox](?:==+|--+|-\.*-)[>ox]?|(?:==+|--+|-\.*-)[>ox]|===+|---+|-\.+-)(?![<>ox.=-])/,lookbehind:!0,alias:"operator"},{pattern:/(^|[^<>()x-])(?:--?(?:>>|[x>)])(?![<>()x])|(?:<<|[x<(])--?(?!-))/,lookbehind:!0,alias:"operator"},{pattern:/(^|[^<>|*o.-])(?:[*o]--|--[*o]|<\|?(?:--|\.\.)|(?:--|\.\.)\|?>|--|\.\.)(?![<>|*o.-])/,lookbehind:!0,alias:"operator"}],label:{pattern:/(^|[^|<])\|(?:[^\r\n"|]|"[^"\r\n]*")+\|/,lookbehind:!0,greedy:!0,alias:"property"},text:{pattern:/(?:[(\[{]+|\b>)(?:[^\r\n"()\[\]{}]|"[^"\r\n]*")+(?:[)\]}]+|>)/,alias:"string"},string:{pattern:/"[^"\r\n]*"/,greedy:!0},annotation:{pattern:/<<(?:abstract|choice|enumeration|fork|interface|join|service)>>|\[\[(?:choice|fork|join)\]\]/i,alias:"important"},keyword:[{pattern:/(^[ \t]*)(?:action|callback|class|classDef|classDiagram|click|direction|erDiagram|flowchart|gantt|gitGraph|graph|journey|link|linkStyle|pie|requirementDiagram|sequenceDiagram|stateDiagram|stateDiagram-v2|style|subgraph)(?![\w$-])/m,lookbehind:!0,greedy:!0},{pattern:/(^[ \t]*)(?:activate|alt|and|as|autonumber|deactivate|else|end(?:[ \t]+note)?|loop|opt|par|participant|rect|state|note[ \t]+(?:over|(?:left|right)[ \t]+of))(?![\w$-])/im,lookbehind:!0,greedy:!0}],entity:/#[a-z0-9]+;/,operator:{pattern:/(\w[ \t]*)&(?=[ \t]*\w)|:::|:/,lookbehind:!0},punctuation:/[(){};]/}}return yF}var vF,Vj;function Pit(){if(Vj)return vF;Vj=1,vF=e,e.displayName="mizar",e.aliases=[];function e(t){t.languages.mizar={comment:/::.+/,keyword:/@proof\b|\b(?:according|aggregate|all|and|antonym|are|as|associativity|assume|asymmetry|attr|be|begin|being|by|canceled|case|cases|clusters?|coherence|commutativity|compatibility|connectedness|consider|consistency|constructors|contradiction|correctness|def|deffunc|define|definitions?|defpred|do|does|end|environ|equals|ex|exactly|existence|for|from|func|given|hence|hereby|holds|idempotence|identity|iff?|implies|involutiveness|irreflexivity|is|it|let|means|mode|non|not|notations?|now|of|or|otherwise|over|per|pred|prefix|projectivity|proof|provided|qua|reconsider|redefine|reduce|reducibility|reflexivity|registrations?|requirements|reserve|sch|schemes?|section|selector|set|sethood|st|struct|such|suppose|symmetry|synonym|take|that|the|then|theorems?|thesis|thus|to|transitivity|uniqueness|vocabular(?:ies|y)|when|where|with|wrt)\b/,parameter:{pattern:/\$(?:10|\d)/,alias:"variable"},variable:/\b\w+(?=:)/,number:/(?:\b|-)\d+\b/,operator:/\.\.\.|->|&|\.?=/,punctuation:/\(#|#\)|[,:;\[\](){}]/}}return vF}var SF,Kj;function zit(){if(Kj)return SF;Kj=1,SF=e,e.displayName="mongodb",e.aliases=[];function e(t){(function(r){var n=["$eq","$gt","$gte","$in","$lt","$lte","$ne","$nin","$and","$not","$nor","$or","$exists","$type","$expr","$jsonSchema","$mod","$regex","$text","$where","$geoIntersects","$geoWithin","$near","$nearSphere","$all","$elemMatch","$size","$bitsAllClear","$bitsAllSet","$bitsAnyClear","$bitsAnySet","$comment","$elemMatch","$meta","$slice","$currentDate","$inc","$min","$max","$mul","$rename","$set","$setOnInsert","$unset","$addToSet","$pop","$pull","$push","$pullAll","$each","$position","$slice","$sort","$bit","$addFields","$bucket","$bucketAuto","$collStats","$count","$currentOp","$facet","$geoNear","$graphLookup","$group","$indexStats","$limit","$listLocalSessions","$listSessions","$lookup","$match","$merge","$out","$planCacheStats","$project","$redact","$replaceRoot","$replaceWith","$sample","$set","$skip","$sort","$sortByCount","$unionWith","$unset","$unwind","$setWindowFields","$abs","$accumulator","$acos","$acosh","$add","$addToSet","$allElementsTrue","$and","$anyElementTrue","$arrayElemAt","$arrayToObject","$asin","$asinh","$atan","$atan2","$atanh","$avg","$binarySize","$bsonSize","$ceil","$cmp","$concat","$concatArrays","$cond","$convert","$cos","$dateFromParts","$dateToParts","$dateFromString","$dateToString","$dayOfMonth","$dayOfWeek","$dayOfYear","$degreesToRadians","$divide","$eq","$exp","$filter","$first","$floor","$function","$gt","$gte","$hour","$ifNull","$in","$indexOfArray","$indexOfBytes","$indexOfCP","$isArray","$isNumber","$isoDayOfWeek","$isoWeek","$isoWeekYear","$last","$last","$let","$literal","$ln","$log","$log10","$lt","$lte","$ltrim","$map","$max","$mergeObjects","$meta","$min","$millisecond","$minute","$mod","$month","$multiply","$ne","$not","$objectToArray","$or","$pow","$push","$radiansToDegrees","$range","$reduce","$regexFind","$regexFindAll","$regexMatch","$replaceOne","$replaceAll","$reverseArray","$round","$rtrim","$second","$setDifference","$setEquals","$setIntersection","$setIsSubset","$setUnion","$size","$sin","$slice","$split","$sqrt","$stdDevPop","$stdDevSamp","$strcasecmp","$strLenBytes","$strLenCP","$substr","$substrBytes","$substrCP","$subtract","$sum","$switch","$tan","$toBool","$toDate","$toDecimal","$toDouble","$toInt","$toLong","$toObjectId","$toString","$toLower","$toUpper","$trim","$trunc","$type","$week","$year","$zip","$count","$dateAdd","$dateDiff","$dateSubtract","$dateTrunc","$getField","$rand","$sampleRate","$setField","$unsetField","$comment","$explain","$hint","$max","$maxTimeMS","$min","$orderby","$query","$returnKey","$showDiskLoc","$natural"],a=["ObjectId","Code","BinData","DBRef","Timestamp","NumberLong","NumberDecimal","MaxKey","MinKey","RegExp","ISODate","UUID"];n=n.map(function(o){return o.replace("$","\\$")});var i="(?:"+n.join("|")+")\\b";r.languages.mongodb=r.languages.extend("javascript",{}),r.languages.insertBefore("mongodb","string",{property:{pattern:/(?:(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1|(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*)(?=\s*:)/,greedy:!0,inside:{keyword:RegExp(`^(['"])?`+i+"(?:\\1)?$")}}}),r.languages.mongodb.string.inside={url:{pattern:/https?:\/\/[-\w@:%.+~#=]{1,256}\.[a-z0-9()]{1,6}\b[-\w()@:%+.~#?&/=]*/i,greedy:!0},entity:{pattern:/\b(?:(?:[01]?\d\d?|2[0-4]\d|25[0-5])\.){3}(?:[01]?\d\d?|2[0-4]\d|25[0-5])\b/,greedy:!0}},r.languages.insertBefore("mongodb","constant",{builtin:{pattern:RegExp("\\b(?:"+a.join("|")+")\\b"),alias:"keyword"}})})(t)}return SF}var EF,Yj;function Hit(){if(Yj)return EF;Yj=1,EF=e,e.displayName="monkey",e.aliases=[];function e(t){t.languages.monkey={comment:{pattern:/^#Rem\s[\s\S]*?^#End|'.+/im,greedy:!0},string:{pattern:/"[^"\r\n]*"/,greedy:!0},preprocessor:{pattern:/(^[ \t]*)#.+/m,lookbehind:!0,greedy:!0,alias:"property"},function:/\b\w+(?=\()/,"type-char":{pattern:/\b[?%#$]/,alias:"class-name"},number:{pattern:/((?:\.\.)?)(?:(?:\b|\B-\.?|\B\.)\d+(?:(?!\.\.)\.\d*)?|\$[\da-f]+)/i,lookbehind:!0},keyword:/\b(?:Abstract|Array|Bool|Case|Catch|Class|Const|Continue|Default|Eachin|Else|ElseIf|End|EndIf|Exit|Extends|Extern|False|Field|Final|Float|For|Forever|Function|Global|If|Implements|Import|Inline|Int|Interface|Local|Method|Module|New|Next|Null|Object|Private|Property|Public|Repeat|Return|Select|Self|Step|Strict|String|Super|Then|Throw|To|True|Try|Until|Void|Wend|While)\b/i,operator:/\.\.|<[=>]?|>=?|:?=|(?:[+\-*\/&~|]|\b(?:Mod|Shl|Shr)\b)=?|\b(?:And|Not|Or)\b/i,punctuation:/[.,:;()\[\]]/}}return EF}var xF,Jj;function Uit(){if(Jj)return xF;Jj=1,xF=e,e.displayName="moonscript",e.aliases=["moon"];function e(t){t.languages.moonscript={comment:/--.*/,string:[{pattern:/'[^']*'|\[(=*)\[[\s\S]*?\]\1\]/,greedy:!0},{pattern:/"[^"]*"/,greedy:!0,inside:{interpolation:{pattern:/#\{[^{}]*\}/,inside:{moonscript:{pattern:/(^#\{)[\s\S]+(?=\})/,lookbehind:!0,inside:null},"interpolation-punctuation":{pattern:/#\{|\}/,alias:"punctuation"}}}}}],"class-name":[{pattern:/(\b(?:class|extends)[ \t]+)\w+/,lookbehind:!0},/\b[A-Z]\w*/],keyword:/\b(?:class|continue|do|else|elseif|export|extends|for|from|if|import|in|local|nil|return|self|super|switch|then|unless|using|when|while|with)\b/,variable:/@@?\w*/,property:{pattern:/\b(?!\d)\w+(?=:)|(:)(?!\d)\w+/,lookbehind:!0},function:{pattern:/\b(?:_G|_VERSION|assert|collectgarbage|coroutine\.(?:create|resume|running|status|wrap|yield)|debug\.(?:debug|getfenv|gethook|getinfo|getlocal|getmetatable|getregistry|getupvalue|setfenv|sethook|setlocal|setmetatable|setupvalue|traceback)|dofile|error|getfenv|getmetatable|io\.(?:close|flush|input|lines|open|output|popen|read|stderr|stdin|stdout|tmpfile|type|write)|ipairs|load|loadfile|loadstring|math\.(?:abs|acos|asin|atan|atan2|ceil|cos|cosh|deg|exp|floor|fmod|frexp|ldexp|log|log10|max|min|modf|pi|pow|rad|random|randomseed|sin|sinh|sqrt|tan|tanh)|module|next|os\.(?:clock|date|difftime|execute|exit|getenv|remove|rename|setlocale|time|tmpname)|package\.(?:cpath|loaded|loadlib|path|preload|seeall)|pairs|pcall|print|rawequal|rawget|rawset|require|select|setfenv|setmetatable|string\.(?:byte|char|dump|find|format|gmatch|gsub|len|lower|match|rep|reverse|sub|upper)|table\.(?:concat|insert|maxn|remove|sort)|tonumber|tostring|type|unpack|xpcall)\b/,inside:{punctuation:/\./}},boolean:/\b(?:false|true)\b/,number:/(?:\B\.\d+|\b\d+\.\d+|\b\d+(?=[eE]))(?:[eE][-+]?\d+)?\b|\b(?:0x[a-fA-F\d]+|\d+)(?:U?LL)?\b/,operator:/\.{3}|[-=]>|~=|(?:[-+*/%<>!=]|\.\.)=?|[:#^]|\b(?:and|or)\b=?|\b(?:not)\b/,punctuation:/[.,()[\]{}\\]/},t.languages.moonscript.string[1].inside.interpolation.inside.moonscript.inside=t.languages.moonscript,t.languages.moon=t.languages.moonscript}return xF}var wF,Xj;function $it(){if(Xj)return wF;Xj=1,wF=e,e.displayName="n1ql",e.aliases=[];function e(t){t.languages.n1ql={comment:{pattern:/\/\*[\s\S]*?(?:$|\*\/)|--.*/,greedy:!0},string:{pattern:/(["'])(?:\\[\s\S]|(?!\1)[^\\]|\1\1)*\1/,greedy:!0},identifier:{pattern:/`(?:\\[\s\S]|[^\\`]|``)*`/,greedy:!0},parameter:/\$[\w.]+/,keyword:/\b(?:ADVISE|ALL|ALTER|ANALYZE|AS|ASC|AT|BEGIN|BINARY|BOOLEAN|BREAK|BUCKET|BUILD|BY|CALL|CAST|CLUSTER|COLLATE|COLLECTION|COMMIT|COMMITTED|CONNECT|CONTINUE|CORRELATE|CORRELATED|COVER|CREATE|CURRENT|DATABASE|DATASET|DATASTORE|DECLARE|DECREMENT|DELETE|DERIVED|DESC|DESCRIBE|DISTINCT|DO|DROP|EACH|ELEMENT|EXCEPT|EXCLUDE|EXECUTE|EXPLAIN|FETCH|FILTER|FLATTEN|FLUSH|FOLLOWING|FOR|FORCE|FROM|FTS|FUNCTION|GOLANG|GRANT|GROUP|GROUPS|GSI|HASH|HAVING|IF|IGNORE|ILIKE|INCLUDE|INCREMENT|INDEX|INFER|INLINE|INNER|INSERT|INTERSECT|INTO|IS|ISOLATION|JAVASCRIPT|JOIN|KEY|KEYS|KEYSPACE|KNOWN|LANGUAGE|LAST|LEFT|LET|LETTING|LEVEL|LIMIT|LSM|MAP|MAPPING|MATCHED|MATERIALIZED|MERGE|MINUS|MISSING|NAMESPACE|NEST|NL|NO|NTH_VALUE|NULL|NULLS|NUMBER|OBJECT|OFFSET|ON|OPTION|OPTIONS|ORDER|OTHERS|OUTER|OVER|PARSE|PARTITION|PASSWORD|PATH|POOL|PRECEDING|PREPARE|PRIMARY|PRIVATE|PRIVILEGE|PROBE|PROCEDURE|PUBLIC|RANGE|RAW|REALM|REDUCE|RENAME|RESPECT|RETURN|RETURNING|REVOKE|RIGHT|ROLE|ROLLBACK|ROW|ROWS|SATISFIES|SAVEPOINT|SCHEMA|SCOPE|SELECT|SELF|SEMI|SET|SHOW|SOME|START|STATISTICS|STRING|SYSTEM|TIES|TO|TRAN|TRANSACTION|TRIGGER|TRUNCATE|UNBOUNDED|UNDER|UNION|UNIQUE|UNKNOWN|UNNEST|UNSET|UPDATE|UPSERT|USE|USER|USING|VALIDATE|VALUE|VALUES|VIA|VIEW|WHERE|WHILE|WINDOW|WITH|WORK|XOR)\b/i,function:/\b[a-z_]\w*(?=\s*\()/i,boolean:/\b(?:FALSE|TRUE)\b/i,number:/(?:\b\d+\.|\B\.)\d+e[+\-]?\d+\b|\b\d+(?:\.\d*)?|\B\.\d+\b/i,operator:/[-+*\/%]|!=|==?|\|\||<[>=]?|>=?|\b(?:AND|ANY|ARRAY|BETWEEN|CASE|ELSE|END|EVERY|EXISTS|FIRST|IN|LIKE|NOT|OR|THEN|VALUED|WHEN|WITHIN)\b/i,punctuation:/[;[\](),.{}:]/}}return wF}var kF,Zj;function Git(){if(Zj)return kF;Zj=1,kF=e,e.displayName="n4js",e.aliases=["n4jsd"];function e(t){t.languages.n4js=t.languages.extend("javascript",{keyword:/\b(?:Array|any|boolean|break|case|catch|class|const|constructor|continue|debugger|declare|default|delete|do|else|enum|export|extends|false|finally|for|from|function|get|if|implements|import|in|instanceof|interface|let|module|new|null|number|package|private|protected|public|return|set|static|string|super|switch|this|throw|true|try|typeof|var|void|while|with|yield)\b/}),t.languages.insertBefore("n4js","constant",{annotation:{pattern:/@+\w+/,alias:"operator"}}),t.languages.n4jsd=t.languages.n4js}return kF}var CF,Qj;function jit(){if(Qj)return CF;Qj=1,CF=e,e.displayName="nand2tetrisHdl",e.aliases=[];function e(t){t.languages["nand2tetris-hdl"]={comment:/\/\/.*|\/\*[\s\S]*?(?:\*\/|$)/,keyword:/\b(?:BUILTIN|CHIP|CLOCKED|IN|OUT|PARTS)\b/,boolean:/\b(?:false|true)\b/,function:/\b[A-Za-z][A-Za-z0-9]*(?=\()/,number:/\b\d+\b/,operator:/=|\.\./,punctuation:/[{}[\];(),:]/}}return CF}var TF,eq;function qit(){if(eq)return TF;eq=1,TF=e,e.displayName="naniscript",e.aliases=[];function e(t){(function(r){var n=/\{[^\r\n\[\]{}]*\}/,a={"quoted-string":{pattern:/"(?:[^"\\]|\\.)*"/,alias:"operator"},"command-param-id":{pattern:/(\s)\w+:/,lookbehind:!0,alias:"property"},"command-param-value":[{pattern:n,alias:"selector"},{pattern:/([\t ])\S+/,lookbehind:!0,greedy:!0,alias:"operator"},{pattern:/\S(?:.*\S)?/,alias:"operator"}]};r.languages.naniscript={comment:{pattern:/^([\t ]*);.*/m,lookbehind:!0},define:{pattern:/^>.+/m,alias:"tag",inside:{value:{pattern:/(^>\w+[\t ]+)(?!\s)[^{}\r\n]+/,lookbehind:!0,alias:"operator"},key:{pattern:/(^>)\w+/,lookbehind:!0}}},label:{pattern:/^([\t ]*)#[\t ]*\w+[\t ]*$/m,lookbehind:!0,alias:"regex"},command:{pattern:/^([\t ]*)@\w+(?=[\t ]|$).*/m,lookbehind:!0,alias:"function",inside:{"command-name":/^@\w+/,expression:{pattern:n,greedy:!0,alias:"selector"},"command-params":{pattern:/\s*\S[\s\S]*/,inside:a}}},"generic-text":{pattern:/(^[ \t]*)[^#@>;\s].*/m,lookbehind:!0,alias:"punctuation",inside:{"escaped-char":/\\[{}\[\]"]/,expression:{pattern:n,greedy:!0,alias:"selector"},"inline-command":{pattern:/\[[\t ]*\w[^\r\n\[\]]*\]/,greedy:!0,alias:"function",inside:{"command-params":{pattern:/(^\[[\t ]*\w+\b)[\s\S]+(?=\]$)/,lookbehind:!0,inside:a},"command-param-name":{pattern:/^(\[[\t ]*)\w+/,lookbehind:!0,alias:"name"},"start-stop-char":/[\[\]]/}}}}},r.languages.nani=r.languages.naniscript,r.hooks.add("after-tokenize",function(s){var l=s.tokens;l.forEach(function(u){if(typeof u!="string"&&u.type==="generic-text"){var d=o(u);i(d)||(u.type="bad-line",u.content=d)}})});function i(s){for(var l="[]{}",u=[],d=0;d<s.length;d++){var p=s[d],f=l.indexOf(p);if(f!==-1){if(f%2===0)u.push(f+1);else if(u.pop()!==f)return!1}}return u.length===0}function o(s){return typeof s=="string"?s:Array.isArray(s)?s.map(o).join(""):o(s.content)}})(t)}return TF}var AF,tq;function Wit(){if(tq)return AF;tq=1,AF=e,e.displayName="nasm",e.aliases=[];function e(t){t.languages.nasm={comment:/;.*$/m,string:/(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,label:{pattern:/(^\s*)[A-Za-z._?$][\w.?$@~#]*:/m,lookbehind:!0,alias:"function"},keyword:[/\[?BITS (?:16|32|64)\]?/,{pattern:/(^\s*)section\s*[a-z.]+:?/im,lookbehind:!0},/(?:extern|global)[^;\r\n]*/i,/(?:CPU|DEFAULT|FLOAT).*$/m],register:{pattern:/\b(?:st\d|[xyz]mm\d\d?|[cdt]r\d|r\d\d?[bwd]?|[er]?[abcd]x|[abcd][hl]|[er]?(?:bp|di|si|sp)|[cdefgs]s)\b/i,alias:"variable"},number:/(?:\b|(?=\$))(?:0[hx](?:\.[\da-f]+|[\da-f]+(?:\.[\da-f]+)?)(?:p[+-]?\d+)?|\d[\da-f]+[hx]|\$\d[\da-f]*|0[oq][0-7]+|[0-7]+[oq]|0[by][01]+|[01]+[by]|0[dt]\d+|(?:\d+(?:\.\d+)?|\.\d+)(?:\.?e[+-]?\d+)?[dt]?)\b/i,operator:/[\[\]*+\-\/%<>=&|$!]/}}return AF}var FF,rq;function Vit(){if(rq)return FF;rq=1,FF=e,e.displayName="neon",e.aliases=[];function e(t){t.languages.neon={comment:{pattern:/#.*/,greedy:!0},datetime:{pattern:/(^|[[{(=:,\s])\d\d\d\d-\d\d?-\d\d?(?:(?:[Tt]| +)\d\d?:\d\d:\d\d(?:\.\d*)? *(?:Z|[-+]\d\d?(?::?\d\d)?)?)?(?=$|[\]}),\s])/,lookbehind:!0,alias:"number"},key:{pattern:/(^|[[{(,\s])[^,:=[\]{}()'"\s]+(?=\s*:(?:$|[\]}),\s])|\s*=)/,lookbehind:!0,alias:"atrule"},number:{pattern:/(^|[[{(=:,\s])[+-]?(?:0x[\da-fA-F]+|0o[0-7]+|0b[01]+|(?:\d+(?:\.\d*)?|\.?\d+)(?:[eE][+-]?\d+)?)(?=$|[\]}),:=\s])/,lookbehind:!0},boolean:{pattern:/(^|[[{(=:,\s])(?:false|no|true|yes)(?=$|[\]}),:=\s])/i,lookbehind:!0},null:{pattern:/(^|[[{(=:,\s])(?:null)(?=$|[\]}),:=\s])/i,lookbehind:!0,alias:"keyword"},string:{pattern:/(^|[[{(=:,\s])(?:('''|""")\r?\n(?:(?:[^\r\n]|\r?\n(?![\t ]*\2))*\r?\n)?[\t ]*\2|'[^'\r\n]*'|"(?:\\.|[^\\"\r\n])*")/,lookbehind:!0,greedy:!0},literal:{pattern:/(^|[[{(=:,\s])(?:[^#"',:=[\]{}()\s`-]|[:-][^"',=[\]{}()\s])(?:[^,:=\]})(\s]|:(?![\s,\]})]|$)|[ \t]+[^#,:=\]})(\s])*/,lookbehind:!0,alias:"string"},punctuation:/[,:=[\]{}()-]/}}return FF}var _F,nq;function Kit(){if(nq)return _F;nq=1,_F=e,e.displayName="nevod",e.aliases=[];function e(t){t.languages.nevod={comment:/\/\/.*|(?:\/\*[\s\S]*?(?:\*\/|$))/,string:{pattern:/(?:"(?:""|[^"])*"(?!")|'(?:''|[^'])*'(?!'))!?\*?/,greedy:!0,inside:{"string-attrs":/!$|!\*$|\*$/}},namespace:{pattern:/(@namespace\s+)[a-zA-Z0-9\-.]+(?=\s*\{)/,lookbehind:!0},pattern:{pattern:/(@pattern\s+)?#?[a-zA-Z0-9\-.]+(?:\s*\(\s*(?:~\s*)?[a-zA-Z0-9\-.]+\s*(?:,\s*(?:~\s*)?[a-zA-Z0-9\-.]*)*\))?(?=\s*=)/,lookbehind:!0,inside:{"pattern-name":{pattern:/^#?[a-zA-Z0-9\-.]+/,alias:"class-name"},fields:{pattern:/\(.*\)/,inside:{"field-name":{pattern:/[a-zA-Z0-9\-.]+/,alias:"variable"},punctuation:/[,()]/,operator:{pattern:/~/,alias:"field-hidden-mark"}}}}},search:{pattern:/(@search\s+|#)[a-zA-Z0-9\-.]+(?:\.\*)?(?=\s*;)/,alias:"function",lookbehind:!0},keyword:/@(?:having|inside|namespace|outside|pattern|require|search|where)\b/,"standard-pattern":{pattern:/\b(?:Alpha|AlphaNum|Any|Blank|End|LineBreak|Num|NumAlpha|Punct|Space|Start|Symbol|Word|WordBreak)\b(?:\([a-zA-Z0-9\-.,\s+]*\))?/,inside:{"standard-pattern-name":{pattern:/^[a-zA-Z0-9\-.]+/,alias:"builtin"},quantifier:{pattern:/\b\d+(?:\s*\+|\s*-\s*\d+)?(?!\w)/,alias:"number"},"standard-pattern-attr":{pattern:/[a-zA-Z0-9\-.]+/,alias:"builtin"},punctuation:/[,()]/}},quantifier:{pattern:/\b\d+(?:\s*\+|\s*-\s*\d+)?(?!\w)/,alias:"number"},operator:[{pattern:/=/,alias:"pattern-def"},{pattern:/&/,alias:"conjunction"},{pattern:/~/,alias:"exception"},{pattern:/\?/,alias:"optionality"},{pattern:/[[\]]/,alias:"repetition"},{pattern:/[{}]/,alias:"variation"},{pattern:/[+_]/,alias:"sequence"},{pattern:/\.{2,3}/,alias:"span"}],"field-capture":[{pattern:/([a-zA-Z0-9\-.]+\s*\()\s*[a-zA-Z0-9\-.]+\s*:\s*[a-zA-Z0-9\-.]+(?:\s*,\s*[a-zA-Z0-9\-.]+\s*:\s*[a-zA-Z0-9\-.]+)*(?=\s*\))/,lookbehind:!0,inside:{"field-name":{pattern:/[a-zA-Z0-9\-.]+/,alias:"variable"},colon:/:/}},{pattern:/[a-zA-Z0-9\-.]+\s*:/,inside:{"field-name":{pattern:/[a-zA-Z0-9\-.]+/,alias:"variable"},colon:/:/}}],punctuation:/[:;,()]/,name:/[a-zA-Z0-9\-.]+/}}return _F}var RF,aq;function Yit(){if(aq)return RF;aq=1,RF=e,e.displayName="nginx",e.aliases=[];function e(t){(function(r){var n=/\$(?:\w[a-z\d]*(?:_[^\x00-\x1F\s"'\\()$]*)?|\{[^}\s"'\\]+\})/i;r.languages.nginx={comment:{pattern:/(^|[\s{};])#.*/,lookbehind:!0,greedy:!0},directive:{pattern:/(^|\s)\w(?:[^;{}"'\\\s]|\\.|"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*'|\s+(?:#.*(?!.)|(?![#\s])))*?(?=\s*[;{])/,lookbehind:!0,greedy:!0,inside:{string:{pattern:/((?:^|[^\\])(?:\\\\)*)(?:"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*')/,lookbehind:!0,greedy:!0,inside:{escape:{pattern:/\\["'\\nrt]/,alias:"entity"},variable:n}},comment:{pattern:/(\s)#.*/,lookbehind:!0,greedy:!0},keyword:{pattern:/^\S+/,greedy:!0},boolean:{pattern:/(\s)(?:off|on)(?!\S)/,lookbehind:!0},number:{pattern:/(\s)\d+[a-z]*(?!\S)/i,lookbehind:!0},variable:n}},punctuation:/[{};]/}})(t)}return RF}var DF,iq;function Jit(){if(iq)return DF;iq=1,DF=e,e.displayName="nim",e.aliases=[];function e(t){t.languages.nim={comment:{pattern:/#.*/,greedy:!0},string:{pattern:/(?:\b(?!\d)(?:\w|\\x[89a-fA-F][0-9a-fA-F])+)?(?:"""[\s\S]*?"""(?!")|"(?:\\[\s\S]|""|[^"\\])*")/,greedy:!0},char:{pattern:/'(?:\\(?:\d+|x[\da-fA-F]{0,2}|.)|[^'])'/,greedy:!0},function:{pattern:/(?:(?!\d)(?:\w|\\x[89a-fA-F][0-9a-fA-F])+|`[^`\r\n]+`)\*?(?:\[[^\]]+\])?(?=\s*\()/,greedy:!0,inside:{operator:/\*$/}},identifier:{pattern:/`[^`\r\n]+`/,greedy:!0,inside:{punctuation:/`/}},number:/\b(?:0[xXoObB][\da-fA-F_]+|\d[\d_]*(?:(?!\.\.)\.[\d_]*)?(?:[eE][+-]?\d[\d_]*)?)(?:'?[iuf]\d*)?/,keyword:/\b(?:addr|as|asm|atomic|bind|block|break|case|cast|concept|const|continue|converter|defer|discard|distinct|do|elif|else|end|enum|except|export|finally|for|from|func|generic|if|import|include|interface|iterator|let|macro|method|mixin|nil|object|out|proc|ptr|raise|ref|return|static|template|try|tuple|type|using|var|when|while|with|without|yield)\b/,operator:{pattern:/(^|[({\[](?=\.\.)|(?![({\[]\.).)(?:(?:[=+\-*\/<>@$~&%|!?^:\\]|\.\.|\.(?![)}\]]))+|\b(?:and|div|in|is|isnot|mod|not|notin|of|or|shl|shr|xor)\b)/m,lookbehind:!0},punctuation:/[({\[]\.|\.[)}\]]|[`(){}\[\],:]/}}return DF}var NF,oq;function Xit(){if(oq)return NF;oq=1,NF=e,e.displayName="nix",e.aliases=[];function e(t){t.languages.nix={comment:{pattern:/\/\*[\s\S]*?\*\/|#.*/,greedy:!0},string:{pattern:/"(?:[^"\\]|\\[\s\S])*"|''(?:(?!'')[\s\S]|''(?:'|\\|\$\{))*''/,greedy:!0,inside:{interpolation:{pattern:/(^|(?:^|(?!'').)[^\\])\$\{(?:[^{}]|\{[^}]*\})*\}/,lookbehind:!0,inside:null}}},url:[/\b(?:[a-z]{3,7}:\/\/)[\w\-+%~\/.:#=?&]+/,{pattern:/([^\/])(?:[\w\-+%~.:#=?&]*(?!\/\/)[\w\-+%~\/.:#=?&])?(?!\/\/)\/[\w\-+%~\/.:#=?&]*/,lookbehind:!0}],antiquotation:{pattern:/\$(?=\{)/,alias:"important"},number:/\b\d+\b/,keyword:/\b(?:assert|builtins|else|if|in|inherit|let|null|or|then|with)\b/,function:/\b(?:abort|add|all|any|attrNames|attrValues|baseNameOf|compareVersions|concatLists|currentSystem|deepSeq|derivation|dirOf|div|elem(?:At)?|fetch(?:Tarball|url)|filter(?:Source)?|fromJSON|genList|getAttr|getEnv|hasAttr|hashString|head|import|intersectAttrs|is(?:Attrs|Bool|Function|Int|List|Null|String)|length|lessThan|listToAttrs|map|mul|parseDrvName|pathExists|read(?:Dir|File)|removeAttrs|replaceStrings|seq|sort|stringLength|sub(?:string)?|tail|throw|to(?:File|JSON|Path|String|XML)|trace|typeOf)\b|\bfoldl'\B/,boolean:/\b(?:false|true)\b/,operator:/[=!<>]=?|\+\+?|\|\||&&|\/\/|->?|[?@]/,punctuation:/[{}()[\].,:;]/},t.languages.nix.string.inside.interpolation.inside=t.languages.nix}return NF}var LF,sq;function Zit(){if(sq)return LF;sq=1,LF=e,e.displayName="nsis",e.aliases=[];function e(t){t.languages.nsis={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|[#;].*)/,lookbehind:!0,greedy:!0},string:{pattern:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},keyword:{pattern:/(^[\t ]*)(?:Abort|Add(?:BrandingImage|Size)|AdvSplash|Allow(?:RootDirInstall|SkipFiles)|AutoCloseWindow|BG(?:Font|Gradient|Image)|Banner|BrandingText|BringToFront|CRCCheck|Call(?:InstDLL)?|Caption|ChangeUI|CheckBitmap|ClearErrors|CompletedText|ComponentText|CopyFiles|Create(?:Directory|Font|ShortCut)|Delete(?:INISec|INIStr|RegKey|RegValue)?|Detail(?:Print|sButtonText)|Dialer|Dir(?:Text|Var|Verify)|EnableWindow|Enum(?:RegKey|RegValue)|Exch|Exec(?:Shell(?:Wait)?|Wait)?|ExpandEnvStrings|File(?:BufSize|Close|ErrorText|Open|Read|ReadByte|ReadUTF16LE|ReadWord|Seek|Write|WriteByte|WriteUTF16LE|WriteWord)?|Find(?:Close|First|Next|Window)|FlushINI|Get(?:CurInstType|CurrentAddress|DLLVersion(?:Local)?|DlgItem|ErrorLevel|FileTime(?:Local)?|FullPathName|Function(?:Address|End)?|InstDirError|LabelAddress|TempFileName)|Goto|HideWindow|Icon|If(?:Abort|Errors|FileExists|RebootFlag|Silent)|InitPluginsDir|InstProgressFlags|Inst(?:Type(?:GetText|SetText)?)|Install(?:ButtonText|Colors|Dir(?:RegKey)?)|Int(?:64|Ptr)?CmpU?|Int(?:64)?Fmt|Int(?:Ptr)?Op|IsWindow|Lang(?:DLL|String)|License(?:BkColor|Data|ForceSelection|LangString|Text)|LoadLanguageFile|LockWindow|Log(?:Set|Text)|Manifest(?:DPIAware|SupportedOS)|Math|MessageBox|MiscButtonText|NSISdl|Name|Nop|OutFile|PE(?:DllCharacteristics|SubsysVer)|Page(?:Callbacks)?|Pop|Push|Quit|RMDir|Read(?:EnvStr|INIStr|RegDWORD|RegStr)|Reboot|RegDLL|Rename|RequestExecutionLevel|ReserveFile|Return|SearchPath|Section(?:End|GetFlags|GetInstTypes|GetSize|GetText|Group|In|SetFlags|SetInstTypes|SetSize|SetText)?|SendMessage|Set(?:AutoClose|BrandingImage|Compress|Compressor(?:DictSize)?|CtlColors|CurInstType|DatablockOptimize|DateSave|Details(?:Print|View)|ErrorLevel|Errors|FileAttributes|Font|OutPath|Overwrite|PluginUnload|RebootFlag|RegView|ShellVarContext|Silent)|Show(?:InstDetails|UninstDetails|Window)|Silent(?:Install|UnInstall)|Sleep|SpaceTexts|Splash|StartMenu|Str(?:CmpS?|Cpy|Len)|SubCaption|System|UnRegDLL|Unicode|UninstPage|Uninstall(?:ButtonText|Caption|Icon|SubCaption|Text)|UserInfo|VI(?:AddVersionKey|FileVersion|ProductVersion)|VPatch|Var|WindowIcon|Write(?:INIStr|Reg(?:Bin|DWORD|ExpandStr|MultiStr|None|Str)|Uninstaller)|XPStyle|ns(?:Dialogs|Exec))\b/m,lookbehind:!0},property:/\b(?:ARCHIVE|FILE_(?:ATTRIBUTE_ARCHIVE|ATTRIBUTE_NORMAL|ATTRIBUTE_OFFLINE|ATTRIBUTE_READONLY|ATTRIBUTE_SYSTEM|ATTRIBUTE_TEMPORARY)|HK(?:(?:CR|CU|LM)(?:32|64)?|DD|PD|U)|HKEY_(?:CLASSES_ROOT|CURRENT_CONFIG|CURRENT_USER|DYN_DATA|LOCAL_MACHINE|PERFORMANCE_DATA|USERS)|ID(?:ABORT|CANCEL|IGNORE|NO|OK|RETRY|YES)|MB_(?:ABORTRETRYIGNORE|DEFBUTTON1|DEFBUTTON2|DEFBUTTON3|DEFBUTTON4|ICONEXCLAMATION|ICONINFORMATION|ICONQUESTION|ICONSTOP|OK|OKCANCEL|RETRYCANCEL|RIGHT|RTLREADING|SETFOREGROUND|TOPMOST|USERICON|YESNO)|NORMAL|OFFLINE|READONLY|SHCTX|SHELL_CONTEXT|SYSTEM|TEMPORARY|admin|all|auto|both|colored|false|force|hide|highest|lastused|leave|listonly|none|normal|notset|off|on|open|print|show|silent|silentlog|smooth|textonly|true|user)\b/,constant:/\$\{[!\w\.:\^-]+\}|\$\([!\w\.:\^-]+\)/,variable:/\$\w[\w\.]*/,number:/\b0x[\dA-Fa-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee]-?\d+)?/,operator:/--?|\+\+?|<=?|>=?|==?=?|&&?|\|\|?|[?*\/~^%]/,punctuation:/[{}[\];(),.:]/,important:{pattern:/(^[\t ]*)!(?:addincludedir|addplugindir|appendfile|cd|define|delfile|echo|else|endif|error|execute|finalize|getdllversion|gettlbversion|if|ifdef|ifmacrodef|ifmacrondef|ifndef|include|insertmacro|macro|macroend|makensis|packhdr|pragma|searchparse|searchreplace|system|tempfile|undef|verbose|warning)\b/im,lookbehind:!0}}}return LF}var MF,lq;function Qit(){if(lq)return MF;lq=1;var e=pd();MF=t,t.displayName="objectivec",t.aliases=["objc"];function t(r){r.register(e),r.languages.objectivec=r.languages.extend("c",{string:{pattern:/@?"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,greedy:!0},keyword:/\b(?:asm|auto|break|case|char|const|continue|default|do|double|else|enum|extern|float|for|goto|if|in|inline|int|long|register|return|self|short|signed|sizeof|static|struct|super|switch|typedef|typeof|union|unsigned|void|volatile|while)\b|(?:@interface|@end|@implementation|@protocol|@class|@public|@protected|@private|@property|@try|@catch|@finally|@throw|@synthesize|@dynamic|@selector)\b/,operator:/-[->]?|\+\+?|!=?|<<?=?|>>?=?|==?|&&?|\|\|?|[~^%?*\/@]/}),delete r.languages.objectivec["class-name"],r.languages.objc=r.languages.objectivec}return MF}var IF,uq;function eot(){if(uq)return IF;uq=1,IF=e,e.displayName="ocaml",e.aliases=[];function e(t){t.languages.ocaml={comment:{pattern:/\(\*[\s\S]*?\*\)/,greedy:!0},char:{pattern:/'(?:[^\\\r\n']|\\(?:.|[ox]?[0-9a-f]{1,3}))'/i,greedy:!0},string:[{pattern:/"(?:\\(?:[\s\S]|\r\n)|[^\\\r\n"])*"/,greedy:!0},{pattern:/\{([a-z_]*)\|[\s\S]*?\|\1\}/,greedy:!0}],number:[/\b(?:0b[01][01_]*|0o[0-7][0-7_]*)\b/i,/\b0x[a-f0-9][a-f0-9_]*(?:\.[a-f0-9_]*)?(?:p[+-]?\d[\d_]*)?(?!\w)/i,/\b\d[\d_]*(?:\.[\d_]*)?(?:e[+-]?\d[\d_]*)?(?!\w)/i],directive:{pattern:/\B#\w+/,alias:"property"},label:{pattern:/\B~\w+/,alias:"property"},"type-variable":{pattern:/\B'\w+/,alias:"function"},variant:{pattern:/`\w+/,alias:"symbol"},keyword:/\b(?:as|assert|begin|class|constraint|do|done|downto|else|end|exception|external|for|fun|function|functor|if|in|include|inherit|initializer|lazy|let|match|method|module|mutable|new|nonrec|object|of|open|private|rec|sig|struct|then|to|try|type|val|value|virtual|when|where|while|with)\b/,boolean:/\b(?:false|true)\b/,"operator-like-punctuation":{pattern:/\[[<>|]|[>|]\]|\{<|>\}/,alias:"punctuation"},operator:/\.[.~]|:[=>]|[=<>@^|&+\-*\/$%!?~][!$%&*+\-.\/:<=>?@^|~]*|\b(?:and|asr|land|lor|lsl|lsr|lxor|mod|or)\b/,punctuation:/;;|::|[(){}\[\].,:;#]|\b_\b/}}return IF}var OF,cq;function tot(){if(cq)return OF;cq=1;var e=pd();OF=t,t.displayName="opencl",t.aliases=[];function t(r){r.register(e),(function(n){n.languages.opencl=n.languages.extend("c",{keyword:/\b(?:(?:__)?(?:constant|global|kernel|local|private|read_only|read_write|write_only)|__attribute__|auto|(?:bool|u?(?:char|int|long|short)|half|quad)(?:2|3|4|8|16)?|break|case|complex|const|continue|(?:double|float)(?:16(?:x(?:1|2|4|8|16))?|1x(?:1|2|4|8|16)|2(?:x(?:1|2|4|8|16))?|3|4(?:x(?:1|2|4|8|16))?|8(?:x(?:1|2|4|8|16))?)?|default|do|else|enum|extern|for|goto|if|imaginary|inline|packed|pipe|register|restrict|return|signed|sizeof|static|struct|switch|typedef|uniform|union|unsigned|void|volatile|while)\b/,number:/(?:\b0x(?:[\da-f]+(?:\.[\da-f]*)?|\.[\da-f]+)(?:p[+-]?\d+)?|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)[fuhl]{0,4}/i,boolean:/\b(?:false|true)\b/,"constant-opencl-kernel":{pattern:/\b(?:CHAR_(?:BIT|MAX|MIN)|CLK_(?:ADDRESS_(?:CLAMP(?:_TO_EDGE)?|NONE|REPEAT)|FILTER_(?:LINEAR|NEAREST)|(?:GLOBAL|LOCAL)_MEM_FENCE|NORMALIZED_COORDS_(?:FALSE|TRUE))|CL_(?:BGRA|(?:HALF_)?FLOAT|INTENSITY|LUMINANCE|A?R?G?B?[Ax]?|(?:(?:UN)?SIGNED|[US]NORM)_(?:INT(?:8|16|32))|UNORM_(?:INT_101010|SHORT_(?:555|565)))|(?:DBL|FLT|HALF)_(?:DIG|EPSILON|(?:MAX|MIN)(?:(?:_10)?_EXP)?|MANT_DIG)|FLT_RADIX|HUGE_VALF?|(?:INT|LONG|SCHAR|SHRT)_(?:MAX|MIN)|INFINITY|MAXFLOAT|M_(?:[12]_PI|2_SQRTPI|E|LN(?:2|10)|LOG(?:2|10)E?|PI(?:_[24])?|SQRT(?:1_2|2))(?:_F|_H)?|NAN|(?:UCHAR|UINT|ULONG|USHRT)_MAX)\b/,alias:"constant"}}),n.languages.insertBefore("opencl","class-name",{"builtin-type":{pattern:/\b(?:_cl_(?:command_queue|context|device_id|event|kernel|mem|platform_id|program|sampler)|cl_(?:image_format|mem_fence_flags)|clk_event_t|event_t|image(?:1d_(?:array_|buffer_)?t|2d_(?:array_(?:depth_|msaa_depth_|msaa_)?|depth_|msaa_depth_|msaa_)?t|3d_t)|intptr_t|ndrange_t|ptrdiff_t|queue_t|reserve_id_t|sampler_t|size_t|uintptr_t)\b/,alias:"keyword"}});var a={"type-opencl-host":{pattern:/\b(?:cl_(?:GLenum|GLint|GLuin|addressing_mode|bitfield|bool|buffer_create_type|build_status|channel_(?:order|type)|(?:u?(?:char|int|long|short)|double|float)(?:2|3|4|8|16)?|command_(?:queue(?:_info|_properties)?|type)|context(?:_info|_properties)?|device_(?:exec_capabilities|fp_config|id|info|local_mem_type|mem_cache_type|type)|(?:event|sampler)(?:_info)?|filter_mode|half|image_info|kernel(?:_info|_work_group_info)?|map_flags|mem(?:_flags|_info|_object_type)?|platform_(?:id|info)|profiling_info|program(?:_build_info|_info)?))\b/,alias:"keyword"},"boolean-opencl-host":{pattern:/\bCL_(?:FALSE|TRUE)\b/,alias:"boolean"},"constant-opencl-host":{pattern:/\bCL_(?:A|ABGR|ADDRESS_(?:CLAMP(?:_TO_EDGE)?|MIRRORED_REPEAT|NONE|REPEAT)|ARGB|BGRA|BLOCKING|BUFFER_CREATE_TYPE_REGION|BUILD_(?:ERROR|IN_PROGRESS|NONE|PROGRAM_FAILURE|SUCCESS)|COMMAND_(?:ACQUIRE_GL_OBJECTS|BARRIER|COPY_(?:BUFFER(?:_RECT|_TO_IMAGE)?|IMAGE(?:_TO_BUFFER)?)|FILL_(?:BUFFER|IMAGE)|MAP(?:_BUFFER|_IMAGE)|MARKER|MIGRATE(?:_SVM)?_MEM_OBJECTS|NATIVE_KERNEL|NDRANGE_KERNEL|READ_(?:BUFFER(?:_RECT)?|IMAGE)|RELEASE_GL_OBJECTS|SVM_(?:FREE|MAP|MEMCPY|MEMFILL|UNMAP)|TASK|UNMAP_MEM_OBJECT|USER|WRITE_(?:BUFFER(?:_RECT)?|IMAGE))|COMPILER_NOT_AVAILABLE|COMPILE_PROGRAM_FAILURE|COMPLETE|CONTEXT_(?:DEVICES|INTEROP_USER_SYNC|NUM_DEVICES|PLATFORM|PROPERTIES|REFERENCE_COUNT)|DEPTH(?:_STENCIL)?|DEVICE_(?:ADDRESS_BITS|AFFINITY_DOMAIN_(?:L[1-4]_CACHE|NEXT_PARTITIONABLE|NUMA)|AVAILABLE|BUILT_IN_KERNELS|COMPILER_AVAILABLE|DOUBLE_FP_CONFIG|ENDIAN_LITTLE|ERROR_CORRECTION_SUPPORT|EXECUTION_CAPABILITIES|EXTENSIONS|GLOBAL_(?:MEM_(?:CACHELINE_SIZE|CACHE_SIZE|CACHE_TYPE|SIZE)|VARIABLE_PREFERRED_TOTAL_SIZE)|HOST_UNIFIED_MEMORY|IL_VERSION|IMAGE(?:2D_MAX_(?:HEIGHT|WIDTH)|3D_MAX_(?:DEPTH|HEIGHT|WIDTH)|_BASE_ADDRESS_ALIGNMENT|_MAX_ARRAY_SIZE|_MAX_BUFFER_SIZE|_PITCH_ALIGNMENT|_SUPPORT)|LINKER_AVAILABLE|LOCAL_MEM_SIZE|LOCAL_MEM_TYPE|MAX_(?:CLOCK_FREQUENCY|COMPUTE_UNITS|CONSTANT_ARGS|CONSTANT_BUFFER_SIZE|GLOBAL_VARIABLE_SIZE|MEM_ALLOC_SIZE|NUM_SUB_GROUPS|ON_DEVICE_(?:EVENTS|QUEUES)|PARAMETER_SIZE|PIPE_ARGS|READ_IMAGE_ARGS|READ_WRITE_IMAGE_ARGS|SAMPLERS|WORK_GROUP_SIZE|WORK_ITEM_DIMENSIONS|WORK_ITEM_SIZES|WRITE_IMAGE_ARGS)|MEM_BASE_ADDR_ALIGN|MIN_DATA_TYPE_ALIGN_SIZE|NAME|NATIVE_VECTOR_WIDTH_(?:CHAR|DOUBLE|FLOAT|HALF|INT|LONG|SHORT)|NOT_(?:AVAILABLE|FOUND)|OPENCL_C_VERSION|PARENT_DEVICE|PARTITION_(?:AFFINITY_DOMAIN|BY_AFFINITY_DOMAIN|BY_COUNTS|BY_COUNTS_LIST_END|EQUALLY|FAILED|MAX_SUB_DEVICES|PROPERTIES|TYPE)|PIPE_MAX_(?:ACTIVE_RESERVATIONS|PACKET_SIZE)|PLATFORM|PREFERRED_(?:GLOBAL_ATOMIC_ALIGNMENT|INTEROP_USER_SYNC|LOCAL_ATOMIC_ALIGNMENT|PLATFORM_ATOMIC_ALIGNMENT|VECTOR_WIDTH_(?:CHAR|DOUBLE|FLOAT|HALF|INT|LONG|SHORT))|PRINTF_BUFFER_SIZE|PROFILE|PROFILING_TIMER_RESOLUTION|QUEUE_(?:ON_(?:DEVICE_(?:MAX_SIZE|PREFERRED_SIZE|PROPERTIES)|HOST_PROPERTIES)|PROPERTIES)|REFERENCE_COUNT|SINGLE_FP_CONFIG|SUB_GROUP_INDEPENDENT_FORWARD_PROGRESS|SVM_(?:ATOMICS|CAPABILITIES|COARSE_GRAIN_BUFFER|FINE_GRAIN_BUFFER|FINE_GRAIN_SYSTEM)|TYPE(?:_ACCELERATOR|_ALL|_CPU|_CUSTOM|_DEFAULT|_GPU)?|VENDOR(?:_ID)?|VERSION)|DRIVER_VERSION|EVENT_(?:COMMAND_(?:EXECUTION_STATUS|QUEUE|TYPE)|CONTEXT|REFERENCE_COUNT)|EXEC_(?:KERNEL|NATIVE_KERNEL|STATUS_ERROR_FOR_EVENTS_IN_WAIT_LIST)|FILTER_(?:LINEAR|NEAREST)|FLOAT|FP_(?:CORRECTLY_ROUNDED_DIVIDE_SQRT|DENORM|FMA|INF_NAN|ROUND_TO_INF|ROUND_TO_NEAREST|ROUND_TO_ZERO|SOFT_FLOAT)|GLOBAL|HALF_FLOAT|IMAGE_(?:ARRAY_SIZE|BUFFER|DEPTH|ELEMENT_SIZE|FORMAT|FORMAT_MISMATCH|FORMAT_NOT_SUPPORTED|HEIGHT|NUM_MIP_LEVELS|NUM_SAMPLES|ROW_PITCH|SLICE_PITCH|WIDTH)|INTENSITY|INVALID_(?:ARG_INDEX|ARG_SIZE|ARG_VALUE|BINARY|BUFFER_SIZE|BUILD_OPTIONS|COMMAND_QUEUE|COMPILER_OPTIONS|CONTEXT|DEVICE|DEVICE_PARTITION_COUNT|DEVICE_QUEUE|DEVICE_TYPE|EVENT|EVENT_WAIT_LIST|GLOBAL_OFFSET|GLOBAL_WORK_SIZE|GL_OBJECT|HOST_PTR|IMAGE_DESCRIPTOR|IMAGE_FORMAT_DESCRIPTOR|IMAGE_SIZE|KERNEL|KERNEL_ARGS|KERNEL_DEFINITION|KERNEL_NAME|LINKER_OPTIONS|MEM_OBJECT|MIP_LEVEL|OPERATION|PIPE_SIZE|PLATFORM|PROGRAM|PROGRAM_EXECUTABLE|PROPERTY|QUEUE_PROPERTIES|SAMPLER|VALUE|WORK_DIMENSION|WORK_GROUP_SIZE|WORK_ITEM_SIZE)|KERNEL_(?:ARG_(?:ACCESS_(?:NONE|QUALIFIER|READ_ONLY|READ_WRITE|WRITE_ONLY)|ADDRESS_(?:CONSTANT|GLOBAL|LOCAL|PRIVATE|QUALIFIER)|INFO_NOT_AVAILABLE|NAME|TYPE_(?:CONST|NAME|NONE|PIPE|QUALIFIER|RESTRICT|VOLATILE))|ATTRIBUTES|COMPILE_NUM_SUB_GROUPS|COMPILE_WORK_GROUP_SIZE|CONTEXT|EXEC_INFO_SVM_FINE_GRAIN_SYSTEM|EXEC_INFO_SVM_PTRS|FUNCTION_NAME|GLOBAL_WORK_SIZE|LOCAL_MEM_SIZE|LOCAL_SIZE_FOR_SUB_GROUP_COUNT|MAX_NUM_SUB_GROUPS|MAX_SUB_GROUP_SIZE_FOR_NDRANGE|NUM_ARGS|PREFERRED_WORK_GROUP_SIZE_MULTIPLE|PRIVATE_MEM_SIZE|PROGRAM|REFERENCE_COUNT|SUB_GROUP_COUNT_FOR_NDRANGE|WORK_GROUP_SIZE)|LINKER_NOT_AVAILABLE|LINK_PROGRAM_FAILURE|LOCAL|LUMINANCE|MAP_(?:FAILURE|READ|WRITE|WRITE_INVALIDATE_REGION)|MEM_(?:ALLOC_HOST_PTR|ASSOCIATED_MEMOBJECT|CONTEXT|COPY_HOST_PTR|COPY_OVERLAP|FLAGS|HOST_NO_ACCESS|HOST_PTR|HOST_READ_ONLY|HOST_WRITE_ONLY|KERNEL_READ_AND_WRITE|MAP_COUNT|OBJECT_(?:ALLOCATION_FAILURE|BUFFER|IMAGE1D|IMAGE1D_ARRAY|IMAGE1D_BUFFER|IMAGE2D|IMAGE2D_ARRAY|IMAGE3D|PIPE)|OFFSET|READ_ONLY|READ_WRITE|REFERENCE_COUNT|SIZE|SVM_ATOMICS|SVM_FINE_GRAIN_BUFFER|TYPE|USES_SVM_POINTER|USE_HOST_PTR|WRITE_ONLY)|MIGRATE_MEM_OBJECT_(?:CONTENT_UNDEFINED|HOST)|MISALIGNED_SUB_BUFFER_OFFSET|NONE|NON_BLOCKING|OUT_OF_(?:HOST_MEMORY|RESOURCES)|PIPE_(?:MAX_PACKETS|PACKET_SIZE)|PLATFORM_(?:EXTENSIONS|HOST_TIMER_RESOLUTION|NAME|PROFILE|VENDOR|VERSION)|PROFILING_(?:COMMAND_(?:COMPLETE|END|QUEUED|START|SUBMIT)|INFO_NOT_AVAILABLE)|PROGRAM_(?:BINARIES|BINARY_SIZES|BINARY_TYPE(?:_COMPILED_OBJECT|_EXECUTABLE|_LIBRARY|_NONE)?|BUILD_(?:GLOBAL_VARIABLE_TOTAL_SIZE|LOG|OPTIONS|STATUS)|CONTEXT|DEVICES|IL|KERNEL_NAMES|NUM_DEVICES|NUM_KERNELS|REFERENCE_COUNT|SOURCE)|QUEUED|QUEUE_(?:CONTEXT|DEVICE|DEVICE_DEFAULT|ON_DEVICE|ON_DEVICE_DEFAULT|OUT_OF_ORDER_EXEC_MODE_ENABLE|PROFILING_ENABLE|PROPERTIES|REFERENCE_COUNT|SIZE)|R|RA|READ_(?:ONLY|WRITE)_CACHE|RG|RGB|RGBA|RGBx|RGx|RUNNING|Rx|SAMPLER_(?:ADDRESSING_MODE|CONTEXT|FILTER_MODE|LOD_MAX|LOD_MIN|MIP_FILTER_MODE|NORMALIZED_COORDS|REFERENCE_COUNT)|(?:UN)?SIGNED_INT(?:8|16|32)|SNORM_INT(?:8|16)|SUBMITTED|SUCCESS|UNORM_INT(?:8|16|24|_101010|_101010_2)|UNORM_SHORT_(?:555|565)|VERSION_(?:1_0|1_1|1_2|2_0|2_1)|sBGRA|sRGB|sRGBA|sRGBx)\b/,alias:"constant"},"function-opencl-host":{pattern:/\bcl(?:BuildProgram|CloneKernel|CompileProgram|Create(?:Buffer|CommandQueue(?:WithProperties)?|Context|ContextFromType|Image|Image2D|Image3D|Kernel|KernelsInProgram|Pipe|ProgramWith(?:Binary|BuiltInKernels|IL|Source)|Sampler|SamplerWithProperties|SubBuffer|SubDevices|UserEvent)|Enqueue(?:(?:Barrier|Marker)(?:WithWaitList)?|Copy(?:Buffer(?:Rect|ToImage)?|Image(?:ToBuffer)?)|(?:Fill|Map)(?:Buffer|Image)|MigrateMemObjects|NDRangeKernel|NativeKernel|(?:Read|Write)(?:Buffer(?:Rect)?|Image)|SVM(?:Free|Map|MemFill|Memcpy|MigrateMem|Unmap)|Task|UnmapMemObject|WaitForEvents)|Finish|Flush|Get(?:CommandQueueInfo|ContextInfo|Device(?:AndHostTimer|IDs|Info)|Event(?:Profiling)?Info|ExtensionFunctionAddress(?:ForPlatform)?|HostTimer|ImageInfo|Kernel(?:ArgInfo|Info|SubGroupInfo|WorkGroupInfo)|MemObjectInfo|PipeInfo|Platform(?:IDs|Info)|Program(?:Build)?Info|SamplerInfo|SupportedImageFormats)|LinkProgram|(?:Release|Retain)(?:CommandQueue|Context|Device|Event|Kernel|MemObject|Program|Sampler)|SVM(?:Alloc|Free)|Set(?:CommandQueueProperty|DefaultDeviceCommandQueue|EventCallback|Kernel|Kernel(?:Arg(?:SVMPointer)?|ExecInfo)|MemObjectDestructorCallback|UserEventStatus)|Unload(?:Platform)?Compiler|WaitForEvents)\b/,alias:"function"}};n.languages.insertBefore("c","keyword",a),n.languages.cpp&&(a["type-opencl-host-cpp"]={pattern:/\b(?:Buffer|BufferGL|BufferRenderGL|CommandQueue|Context|Device|DeviceCommandQueue|EnqueueArgs|Event|Image|Image1D|Image1DArray|Image1DBuffer|Image2D|Image2DArray|Image2DGL|Image3D|Image3DGL|ImageFormat|ImageGL|Kernel|KernelFunctor|LocalSpaceArg|Memory|NDRange|Pipe|Platform|Program|SVMAllocator|SVMTraitAtomic|SVMTraitCoarse|SVMTraitFine|SVMTraitReadOnly|SVMTraitReadWrite|SVMTraitWriteOnly|Sampler|UserEvent)\b/,alias:"keyword"},n.languages.insertBefore("cpp","keyword",a))})(r)}return OF}var BF,dq;function rot(){if(dq)return BF;dq=1,BF=e,e.displayName="openqasm",e.aliases=["qasm"];function e(t){t.languages.openqasm={comment:/\/\*[\s\S]*?\*\/|\/\/.*/,string:{pattern:/"[^"\r\n\t]*"|'[^'\r\n\t]*'/,greedy:!0},keyword:/\b(?:CX|OPENQASM|U|barrier|boxas|boxto|break|const|continue|ctrl|def|defcal|defcalgrammar|delay|else|end|for|gate|gphase|if|in|include|inv|kernel|lengthof|let|measure|pow|reset|return|rotary|stretchinf|while)\b|#pragma\b/,"class-name":/\b(?:angle|bit|bool|creg|fixed|float|int|length|qreg|qubit|stretch|uint)\b/,function:/\b(?:cos|exp|ln|popcount|rotl|rotr|sin|sqrt|tan)\b(?=\s*\()/,constant:/\b(?:euler|pi|tau)\b|π|𝜏|ℇ/,number:{pattern:/(^|[^.\w$])(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?(?:dt|ns|us|µs|ms|s)?/i,lookbehind:!0},operator:/->|>>=?|<<=?|&&|\|\||\+\+|--|[!=<>&|~^+\-*/%]=?|@/,punctuation:/[(){}\[\];,:.]/},t.languages.qasm=t.languages.openqasm}return BF}var PF,hq;function not(){if(hq)return PF;hq=1,PF=e,e.displayName="oz",e.aliases=[];function e(t){t.languages.oz={comment:{pattern:/\/\*[\s\S]*?\*\/|%.*/,greedy:!0},string:{pattern:/"(?:[^"\\]|\\[\s\S])*"/,greedy:!0},atom:{pattern:/'(?:[^'\\]|\\[\s\S])*'/,greedy:!0,alias:"builtin"},keyword:/\$|\[\]|\b(?:_|at|attr|case|catch|choice|class|cond|declare|define|dis|else(?:case|if)?|end|export|fail|false|feat|finally|from|fun|functor|if|import|in|local|lock|meth|nil|not|of|or|prepare|proc|prop|raise|require|self|skip|then|thread|true|try|unit)\b/,function:[/\b[a-z][A-Za-z\d]*(?=\()/,{pattern:/(\{)[A-Z][A-Za-z\d]*\b/,lookbehind:!0}],number:/\b(?:0[bx][\da-f]+|\d+(?:\.\d*)?(?:e~?\d+)?)\b|&(?:[^\\]|\\(?:\d{3}|.))/i,variable:/`(?:[^`\\]|\\.)+`/,"attr-name":/\b\w+(?=[ \t]*:(?![:=]))/,operator:/:(?:=|::?)|<[-:=]?|=(?:=|<?:?)|>=?:?|\\=:?|!!?|[|#+\-*\/,~^@]|\b(?:andthen|div|mod|orelse)\b/,punctuation:/[\[\](){}.:;?]/}}return PF}var zF,pq;function aot(){if(pq)return zF;pq=1,zF=e,e.displayName="parigp",e.aliases=[];function e(t){t.languages.parigp={comment:/\/\*[\s\S]*?\*\/|\\\\.*/,string:{pattern:/"(?:[^"\\\r\n]|\\.)*"/,greedy:!0},keyword:(function(){var r=["breakpoint","break","dbg_down","dbg_err","dbg_up","dbg_x","forcomposite","fordiv","forell","forpart","forprime","forstep","forsubgroup","forvec","for","iferr","if","local","my","next","return","until","while"];return r=r.map(function(n){return n.split("").join(" *")}).join("|"),RegExp("\\b(?:"+r+")\\b")})(),function:/\b\w(?:[\w ]*\w)?(?= *\()/,number:{pattern:/((?:\. *\. *)?)(?:\b\d(?: *\d)*(?: *(?!\. *\.)\.(?: *\d)*)?|\. *\d(?: *\d)*)(?: *e *(?:[+-] *)?\d(?: *\d)*)?/i,lookbehind:!0},operator:/\. *\.|[*\/!](?: *=)?|%(?: *=|(?: *#)?(?: *')*)?|\+(?: *[+=])?|-(?: *[-=>])?|<(?: *>|(?: *<)?(?: *=)?)?|>(?: *>)?(?: *=)?|=(?: *=){0,2}|\\(?: *\/)?(?: *=)?|&(?: *&)?|\| *\||['#~^]/,punctuation:/[\[\]{}().,:;|]/}}return zF}var HF,fq;function iot(){if(fq)return HF;fq=1,HF=e,e.displayName="parser",e.aliases=[];function e(t){(function(r){var n=r.languages.parser=r.languages.extend("markup",{keyword:{pattern:/(^|[^^])(?:\^(?:case|eval|for|if|switch|throw)\b|@(?:BASE|CLASS|GET(?:_DEFAULT)?|OPTIONS|SET_DEFAULT|USE)\b)/,lookbehind:!0},variable:{pattern:/(^|[^^])\B\$(?:\w+|(?=[.{]))(?:(?:\.|::?)\w+)*(?:\.|::?)?/,lookbehind:!0,inside:{punctuation:/\.|:+/}},function:{pattern:/(^|[^^])\B[@^]\w+(?:(?:\.|::?)\w+)*(?:\.|::?)?/,lookbehind:!0,inside:{keyword:{pattern:/(^@)(?:GET_|SET_)/,lookbehind:!0},punctuation:/\.|:+/}},escape:{pattern:/\^(?:[$^;@()\[\]{}"':]|#[a-f\d]*)/i,alias:"builtin"},punctuation:/[\[\](){};]/});n=r.languages.insertBefore("parser","keyword",{"parser-comment":{pattern:/(\s)#.*/,lookbehind:!0,alias:"comment"},expression:{pattern:/(^|[^^])\((?:[^()]|\((?:[^()]|\((?:[^()])*\))*\))*\)/,greedy:!0,lookbehind:!0,inside:{string:{pattern:/(^|[^^])(["'])(?:(?!\2)[^^]|\^[\s\S])*\2/,lookbehind:!0},keyword:n.keyword,variable:n.variable,function:n.function,boolean:/\b(?:false|true)\b/,number:/\b(?:0x[a-f\d]+|\d+(?:\.\d*)?(?:e[+-]?\d+)?)\b/i,escape:n.escape,operator:/[~+*\/\\%]|!(?:\|\|?|=)?|&&?|\|\|?|==|<[<=]?|>[>=]?|-[fd]?|\b(?:def|eq|ge|gt|in|is|le|lt|ne)\b/,punctuation:n.punctuation}}}),r.languages.insertBefore("inside","punctuation",{expression:n.expression,keyword:n.keyword,variable:n.variable,function:n.function,escape:n.escape,"parser-punctuation":{pattern:n.punctuation,alias:"punctuation"}},n.tag.inside["attr-value"])})(t)}return HF}var UF,mq;function oot(){if(mq)return UF;mq=1,UF=e,e.displayName="pascal",e.aliases=["objectpascal"];function e(t){t.languages.pascal={directive:{pattern:/\{\$[\s\S]*?\}/,greedy:!0,alias:["marco","property"]},comment:{pattern:/\(\*[\s\S]*?\*\)|\{[\s\S]*?\}|\/\/.*/,greedy:!0},string:{pattern:/(?:'(?:''|[^'\r\n])*'(?!')|#[&$%]?[a-f\d]+)+|\^[a-z]/i,greedy:!0},asm:{pattern:/(\basm\b)[\s\S]+?(?=\bend\s*[;[])/i,lookbehind:!0,greedy:!0,inside:null},keyword:[{pattern:/(^|[^&])\b(?:absolute|array|asm|begin|case|const|constructor|destructor|do|downto|else|end|file|for|function|goto|if|implementation|inherited|inline|interface|label|nil|object|of|operator|packed|procedure|program|record|reintroduce|repeat|self|set|string|then|to|type|unit|until|uses|var|while|with)\b/i,lookbehind:!0},{pattern:/(^|[^&])\b(?:dispose|exit|false|new|true)\b/i,lookbehind:!0},{pattern:/(^|[^&])\b(?:class|dispinterface|except|exports|finalization|finally|initialization|inline|library|on|out|packed|property|raise|resourcestring|threadvar|try)\b/i,lookbehind:!0},{pattern:/(^|[^&])\b(?:absolute|abstract|alias|assembler|bitpacked|break|cdecl|continue|cppdecl|cvar|default|deprecated|dynamic|enumerator|experimental|export|external|far|far16|forward|generic|helper|implements|index|interrupt|iochecks|local|message|name|near|nodefault|noreturn|nostackframe|oldfpccall|otherwise|overload|override|pascal|platform|private|protected|public|published|read|register|reintroduce|result|safecall|saveregisters|softfloat|specialize|static|stdcall|stored|strict|unaligned|unimplemented|varargs|virtual|write)\b/i,lookbehind:!0}],number:[/(?:[&%]\d+|\$[a-f\d]+)/i,/\b\d+(?:\.\d+)?(?:e[+-]?\d+)?/i],operator:[/\.\.|\*\*|:=|<[<=>]?|>[>=]?|[+\-*\/]=?|[@^=]/,{pattern:/(^|[^&])\b(?:and|as|div|exclude|in|include|is|mod|not|or|shl|shr|xor)\b/,lookbehind:!0}],punctuation:/\(\.|\.\)|[()\[\]:;,.]/},t.languages.pascal.asm.inside=t.languages.extend("pascal",{asm:void 0,keyword:void 0,operator:void 0}),t.languages.objectpascal=t.languages.pascal}return UF}var $F,gq;function sot(){if(gq)return $F;gq=1,$F=e,e.displayName="pascaligo",e.aliases=[];function e(t){(function(r){var n=/\((?:[^()]|\((?:[^()]|\([^()]*\))*\))*\)/.source,a=/(?:\b\w+(?:<braces>)?|<braces>)/.source.replace(/<braces>/g,function(){return n}),i=r.languages.pascaligo={comment:/\(\*[\s\S]+?\*\)|\/\/.*/,string:{pattern:/(["'`])(?:\\[\s\S]|(?!\1)[^\\])*\1|\^[a-z]/i,greedy:!0},"class-name":[{pattern:RegExp(/(\btype\s+\w+\s+is\s+)<type>/.source.replace(/<type>/g,function(){return a}),"i"),lookbehind:!0,inside:null},{pattern:RegExp(/<type>(?=\s+is\b)/.source.replace(/<type>/g,function(){return a}),"i"),inside:null},{pattern:RegExp(/(:\s*)<type>/.source.replace(/<type>/g,function(){return a})),lookbehind:!0,inside:null}],keyword:{pattern:/(^|[^&])\b(?:begin|block|case|const|else|end|fail|for|from|function|if|is|nil|of|remove|return|skip|then|type|var|while|with)\b/i,lookbehind:!0},boolean:{pattern:/(^|[^&])\b(?:False|True)\b/i,lookbehind:!0},builtin:{pattern:/(^|[^&])\b(?:bool|int|list|map|nat|record|string|unit)\b/i,lookbehind:!0},function:/\b\w+(?=\s*\()/,number:[/%[01]+|&[0-7]+|\$[a-f\d]+/i,/\b\d+(?:\.\d+)?(?:e[+-]?\d+)?(?:mtz|n)?/i],operator:/->|=\/=|\.\.|\*\*|:=|<[<=>]?|>[>=]?|[+\-*\/]=?|[@^=|]|\b(?:and|mod|or)\b/,punctuation:/\(\.|\.\)|[()\[\]:;,.{}]/},o=["comment","keyword","builtin","operator","punctuation"].reduce(function(s,l){return s[l]=i[l],s},{});i["class-name"].forEach(function(s){s.inside=o})})(t)}return $F}var GF,bq;function lot(){if(bq)return GF;bq=1,GF=e,e.displayName="pcaxis",e.aliases=["px"];function e(t){t.languages.pcaxis={string:/"[^"]*"/,keyword:{pattern:/((?:^|;)\s*)[-A-Z\d]+(?:\s*\[[-\w]+\])?(?:\s*\("[^"]*"(?:,\s*"[^"]*")*\))?(?=\s*=)/,lookbehind:!0,greedy:!0,inside:{keyword:/^[-A-Z\d]+/,language:{pattern:/^(\s*)\[[-\w]+\]/,lookbehind:!0,inside:{punctuation:/^\[|\]$/,property:/[-\w]+/}},"sub-key":{pattern:/^(\s*)\S[\s\S]*/,lookbehind:!0,inside:{parameter:{pattern:/"[^"]*"/,alias:"property"},punctuation:/^\(|\)$|,/}}}},operator:/=/,tlist:{pattern:/TLIST\s*\(\s*\w+(?:(?:\s*,\s*"[^"]*")+|\s*,\s*"[^"]*"-"[^"]*")?\s*\)/,greedy:!0,inside:{function:/^TLIST/,property:{pattern:/^(\s*\(\s*)\w+/,lookbehind:!0},string:/"[^"]*"/,punctuation:/[(),]/,operator:/-/}},punctuation:/[;,]/,number:{pattern:/(^|\s)\d+(?:\.\d+)?(?!\S)/,lookbehind:!0},boolean:/NO|YES/},t.languages.px=t.languages.pcaxis}return GF}var jF,yq;function uot(){if(yq)return jF;yq=1,jF=e,e.displayName="peoplecode",e.aliases=["pcode"];function e(t){t.languages.peoplecode={comment:RegExp([/\/\*[\s\S]*?\*\//.source,/\bREM[^;]*;/.source,/<\*(?:[^<*]|\*(?!>)|<(?!\*)|<\*(?:(?!\*>)[\s\S])*\*>)*\*>/.source,/\/\+[\s\S]*?\+\//.source].join("|")),string:{pattern:/'(?:''|[^'\r\n])*'(?!')|"(?:""|[^"\r\n])*"(?!")/,greedy:!0},variable:/%\w+/,"function-definition":{pattern:/((?:^|[^\w-])(?:function|method)\s+)\w+/i,lookbehind:!0,alias:"function"},"class-name":{pattern:/((?:^|[^-\w])(?:as|catch|class|component|create|extends|global|implements|instance|local|of|property|returns)\s+)\w+(?::\w+)*/i,lookbehind:!0,inside:{punctuation:/:/}},keyword:/\b(?:abstract|alias|as|catch|class|component|constant|create|declare|else|end-(?:class|evaluate|for|function|get|if|method|set|try|while)|evaluate|extends|for|function|get|global|if|implements|import|instance|library|local|method|null|of|out|peopleCode|private|program|property|protected|readonly|ref|repeat|returns?|set|step|then|throw|to|try|until|value|when(?:-other)?|while)\b/i,"operator-keyword":{pattern:/\b(?:and|not|or)\b/i,alias:"operator"},function:/[_a-z]\w*(?=\s*\()/i,boolean:/\b(?:false|true)\b/i,number:/\b\d+(?:\.\d+)?\b/,operator:/<>|[<>]=?|!=|\*\*|[-+*/|=@]/,punctuation:/[:.;,()[\]]/},t.languages.pcode=t.languages.peoplecode}return jF}var qF,vq;function cot(){if(vq)return qF;vq=1,qF=e,e.displayName="perl",e.aliases=[];function e(t){(function(r){var n=/(?:\((?:[^()\\]|\\[\s\S])*\)|\{(?:[^{}\\]|\\[\s\S])*\}|\[(?:[^[\]\\]|\\[\s\S])*\]|<(?:[^<>\\]|\\[\s\S])*>)/.source;r.languages.perl={comment:[{pattern:/(^\s*)=\w[\s\S]*?=cut.*/m,lookbehind:!0,greedy:!0},{pattern:/(^|[^\\$])#.*/,lookbehind:!0,greedy:!0}],string:[{pattern:RegExp(/\b(?:q|qq|qw|qx)(?![a-zA-Z0-9])\s*/.source+"(?:"+[/([^a-zA-Z0-9\s{(\[<])(?:(?!\1)[^\\]|\\[\s\S])*\1/.source,/([a-zA-Z0-9])(?:(?!\2)[^\\]|\\[\s\S])*\2/.source,n].join("|")+")"),greedy:!0},{pattern:/("|`)(?:(?!\1)[^\\]|\\[\s\S])*\1/,greedy:!0},{pattern:/'(?:[^'\\\r\n]|\\.)*'/,greedy:!0}],regex:[{pattern:RegExp(/\b(?:m|qr)(?![a-zA-Z0-9])\s*/.source+"(?:"+[/([^a-zA-Z0-9\s{(\[<])(?:(?!\1)[^\\]|\\[\s\S])*\1/.source,/([a-zA-Z0-9])(?:(?!\2)[^\\]|\\[\s\S])*\2/.source,n].join("|")+")"+/[msixpodualngc]*/.source),greedy:!0},{pattern:RegExp(/(^|[^-])\b(?:s|tr|y)(?![a-zA-Z0-9])\s*/.source+"(?:"+[/([^a-zA-Z0-9\s{(\[<])(?:(?!\2)[^\\]|\\[\s\S])*\2(?:(?!\2)[^\\]|\\[\s\S])*\2/.source,/([a-zA-Z0-9])(?:(?!\3)[^\\]|\\[\s\S])*\3(?:(?!\3)[^\\]|\\[\s\S])*\3/.source,n+/\s*/.source+n].join("|")+")"+/[msixpodualngcer]*/.source),lookbehind:!0,greedy:!0},{pattern:/\/(?:[^\/\\\r\n]|\\.)*\/[msixpodualngc]*(?=\s*(?:$|[\r\n,.;})&|\-+*~<>!?^]|(?:and|cmp|eq|ge|gt|le|lt|ne|not|or|x|xor)\b))/,greedy:!0}],variable:[/[&*$@%]\{\^[A-Z]+\}/,/[&*$@%]\^[A-Z_]/,/[&*$@%]#?(?=\{)/,/[&*$@%]#?(?:(?:::)*'?(?!\d)[\w$]+(?![\w$]))+(?:::)*/,/[&*$@%]\d+/,/(?!%=)[$@%][!"#$%&'()*+,\-.\/:;<=>?@[\\\]^_`{|}~]/],filehandle:{pattern:/<(?![<=])\S*?>|\b_\b/,alias:"symbol"},"v-string":{pattern:/v\d+(?:\.\d+)*|\d+(?:\.\d+){2,}/,alias:"string"},function:{pattern:/(\bsub[ \t]+)\w+/,lookbehind:!0},keyword:/\b(?:any|break|continue|default|delete|die|do|else|elsif|eval|for|foreach|given|goto|if|last|local|my|next|our|package|print|redo|require|return|say|state|sub|switch|undef|unless|until|use|when|while)\b/,number:/\b(?:0x[\dA-Fa-f](?:_?[\dA-Fa-f])*|0b[01](?:_?[01])*|(?:(?:\d(?:_?\d)*)?\.)?\d(?:_?\d)*(?:[Ee][+-]?\d+)?)\b/,operator:/-[rwxoRWXOezsfdlpSbctugkTBMAC]\b|\+[+=]?|-[-=>]?|\*\*?=?|\/\/?=?|=[=~>]?|~[~=]?|\|\|?=?|&&?=?|<(?:=>?|<=?)?|>>?=?|![~=]?|[%^]=?|\.(?:=|\.\.?)?|[\\?]|\bx(?:=|\b)|\b(?:and|cmp|eq|ge|gt|le|lt|ne|not|or|xor)\b/,punctuation:/[{}[\];(),:]/}})(t)}return qF}var WF,Sq;function dot(){if(Sq)return WF;Sq=1;var e=iS();WF=t,t.displayName="phpExtras",t.aliases=[];function t(r){r.register(e),r.languages.insertBefore("php","variable",{this:{pattern:/\$this\b/,alias:"keyword"},global:/\$(?:GLOBALS|HTTP_RAW_POST_DATA|_(?:COOKIE|ENV|FILES|GET|POST|REQUEST|SERVER|SESSION)|argc|argv|http_response_header|php_errormsg)\b/,scope:{pattern:/\b[\w\\]+::/,inside:{keyword:/\b(?:parent|self|static)\b/,punctuation:/::|\\/}}})}return WF}var VF,Eq;function hot(){if(Eq)return VF;Eq=1;var e=iS(),t=aS();VF=r,r.displayName="phpdoc",r.aliases=[];function r(n){n.register(e),n.register(t),(function(a){var i=/(?:\b[a-zA-Z]\w*|[|\\[\]])+/.source;a.languages.phpdoc=a.languages.extend("javadoclike",{parameter:{pattern:RegExp("(@(?:global|param|property(?:-read|-write)?|var)\\s+(?:"+i+"\\s+)?)\\$\\w+"),lookbehind:!0}}),a.languages.insertBefore("phpdoc","keyword",{"class-name":[{pattern:RegExp("(@(?:global|package|param|property(?:-read|-write)?|return|subpackage|throws|var)\\s+)"+i),lookbehind:!0,inside:{keyword:/\b(?:array|bool|boolean|callback|double|false|float|int|integer|mixed|null|object|resource|self|string|true|void)\b/,punctuation:/[|\\[\]()]/}}]}),a.languages.javadoclike.addSupport("php",a.languages.phpdoc)})(n)}return VF}var KF,xq;function pot(){if(xq)return KF;xq=1;var e=hL();KF=t,t.displayName="plsql",t.aliases=[];function t(r){r.register(e),r.languages.plsql=r.languages.extend("sql",{comment:{pattern:/\/\*[\s\S]*?\*\/|--.*/,greedy:!0},keyword:/\b(?:A|ACCESSIBLE|ADD|AGENT|AGGREGATE|ALL|ALTER|AND|ANY|ARRAY|AS|ASC|AT|ATTRIBUTE|AUTHID|AVG|BEGIN|BETWEEN|BFILE_BASE|BINARY|BLOB_BASE|BLOCK|BODY|BOTH|BOUND|BULK|BY|BYTE|C|CALL|CALLING|CASCADE|CASE|CHAR|CHARACTER|CHARSET|CHARSETFORM|CHARSETID|CHAR_BASE|CHECK|CLOB_BASE|CLONE|CLOSE|CLUSTER|CLUSTERS|COLAUTH|COLLECT|COLUMNS|COMMENT|COMMIT|COMMITTED|COMPILED|COMPRESS|CONNECT|CONSTANT|CONSTRUCTOR|CONTEXT|CONTINUE|CONVERT|COUNT|CRASH|CREATE|CREDENTIAL|CURRENT|CURSOR|CUSTOMDATUM|DANGLING|DATA|DATE|DATE_BASE|DAY|DECLARE|DEFAULT|DEFINE|DELETE|DESC|DETERMINISTIC|DIRECTORY|DISTINCT|DOUBLE|DROP|DURATION|ELEMENT|ELSE|ELSIF|EMPTY|END|ESCAPE|EXCEPT|EXCEPTION|EXCEPTIONS|EXCLUSIVE|EXECUTE|EXISTS|EXIT|EXTERNAL|FETCH|FINAL|FIRST|FIXED|FLOAT|FOR|FORALL|FORCE|FROM|FUNCTION|GENERAL|GOTO|GRANT|GROUP|HASH|HAVING|HEAP|HIDDEN|HOUR|IDENTIFIED|IF|IMMEDIATE|IMMUTABLE|IN|INCLUDING|INDEX|INDEXES|INDICATOR|INDICES|INFINITE|INSERT|INSTANTIABLE|INT|INTERFACE|INTERSECT|INTERVAL|INTO|INVALIDATE|IS|ISOLATION|JAVA|LANGUAGE|LARGE|LEADING|LENGTH|LEVEL|LIBRARY|LIKE|LIKE2|LIKE4|LIKEC|LIMIT|LIMITED|LOCAL|LOCK|LONG|LOOP|MAP|MAX|MAXLEN|MEMBER|MERGE|MIN|MINUS|MINUTE|MOD|MODE|MODIFY|MONTH|MULTISET|MUTABLE|NAME|NAN|NATIONAL|NATIVE|NCHAR|NEW|NOCOMPRESS|NOCOPY|NOT|NOWAIT|NULL|NUMBER_BASE|OBJECT|OCICOLL|OCIDATE|OCIDATETIME|OCIDURATION|OCIINTERVAL|OCILOBLOCATOR|OCINUMBER|OCIRAW|OCIREF|OCIREFCURSOR|OCIROWID|OCISTRING|OCITYPE|OF|OLD|ON|ONLY|OPAQUE|OPEN|OPERATOR|OPTION|OR|ORACLE|ORADATA|ORDER|ORGANIZATION|ORLANY|ORLVARY|OTHERS|OUT|OVERLAPS|OVERRIDING|PACKAGE|PARALLEL_ENABLE|PARAMETER|PARAMETERS|PARENT|PARTITION|PASCAL|PERSISTABLE|PIPE|PIPELINED|PLUGGABLE|POLYMORPHIC|PRAGMA|PRECISION|PRIOR|PRIVATE|PROCEDURE|PUBLIC|RAISE|RANGE|RAW|READ|RECORD|REF|REFERENCE|RELIES_ON|REM|REMAINDER|RENAME|RESOURCE|RESULT|RESULT_CACHE|RETURN|RETURNING|REVERSE|REVOKE|ROLLBACK|ROW|SAMPLE|SAVE|SAVEPOINT|SB1|SB2|SB4|SECOND|SEGMENT|SELECT|SELF|SEPARATE|SEQUENCE|SERIALIZABLE|SET|SHARE|SHORT|SIZE|SIZE_T|SOME|SPARSE|SQL|SQLCODE|SQLDATA|SQLNAME|SQLSTATE|STANDARD|START|STATIC|STDDEV|STORED|STRING|STRUCT|STYLE|SUBMULTISET|SUBPARTITION|SUBSTITUTABLE|SUBTYPE|SUM|SYNONYM|TABAUTH|TABLE|TDO|THE|THEN|TIME|TIMESTAMP|TIMEZONE_ABBR|TIMEZONE_HOUR|TIMEZONE_MINUTE|TIMEZONE_REGION|TO|TRAILING|TRANSACTION|TRANSACTIONAL|TRUSTED|TYPE|UB1|UB2|UB4|UNDER|UNION|UNIQUE|UNPLUG|UNSIGNED|UNTRUSTED|UPDATE|USE|USING|VALIST|VALUE|VALUES|VARIABLE|VARIANCE|VARRAY|VARYING|VIEW|VIEWS|VOID|WHEN|WHERE|WHILE|WITH|WORK|WRAPPED|WRITE|YEAR|ZONE)\b/i,operator:/:=?|=>|[<>^~!]=|\.\.|\|\||\*\*|[-+*/%<>=@]/}),r.languages.insertBefore("plsql","operator",{label:{pattern:/<<\s*\w+\s*>>/,alias:"symbol"}})}return KF}var YF,wq;function fot(){if(wq)return YF;wq=1,YF=e,e.displayName="powerquery",e.aliases=[];function e(t){t.languages.powerquery={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,lookbehind:!0,greedy:!0},"quoted-identifier":{pattern:/#"(?:[^"\r\n]|"")*"(?!")/,greedy:!0},string:{pattern:/(?:#!)?"(?:[^"\r\n]|"")*"(?!")/,greedy:!0},constant:[/\bDay\.(?:Friday|Monday|Saturday|Sunday|Thursday|Tuesday|Wednesday)\b/,/\bTraceLevel\.(?:Critical|Error|Information|Verbose|Warning)\b/,/\bOccurrence\.(?:All|First|Last)\b/,/\bOrder\.(?:Ascending|Descending)\b/,/\bRoundingMode\.(?:AwayFromZero|Down|ToEven|TowardZero|Up)\b/,/\bMissingField\.(?:Error|Ignore|UseNull)\b/,/\bQuoteStyle\.(?:Csv|None)\b/,/\bJoinKind\.(?:FullOuter|Inner|LeftAnti|LeftOuter|RightAnti|RightOuter)\b/,/\bGroupKind\.(?:Global|Local)\b/,/\bExtraValues\.(?:Error|Ignore|List)\b/,/\bJoinAlgorithm\.(?:Dynamic|LeftHash|LeftIndex|PairwiseHash|RightHash|RightIndex|SortMerge)\b/,/\bJoinSide\.(?:Left|Right)\b/,/\bPrecision\.(?:Decimal|Double)\b/,/\bRelativePosition\.From(?:End|Start)\b/,/\bTextEncoding\.(?:Ascii|BigEndianUnicode|Unicode|Utf16|Utf8|Windows)\b/,/\b(?:Any|Binary|Date|DateTime|DateTimeZone|Duration|Function|Int16|Int32|Int64|Int8|List|Logical|None|Number|Record|Table|Text|Time)\.Type\b/,/\bnull\b/],boolean:/\b(?:false|true)\b/,keyword:/\b(?:and|as|each|else|error|if|in|is|let|meta|not|nullable|optional|or|otherwise|section|shared|then|try|type)\b|#(?:binary|date|datetime|datetimezone|duration|infinity|nan|sections|shared|table|time)\b/,function:{pattern:/(^|[^#\w.])[a-z_][\w.]*(?=\s*\()/i,lookbehind:!0},"data-type":{pattern:/\b(?:any|anynonnull|binary|date|datetime|datetimezone|duration|function|list|logical|none|number|record|table|text|time)\b/,alias:"class-name"},number:{pattern:/\b0x[\da-f]+\b|(?:[+-]?(?:\b\d+\.)?\b\d+|[+-]\.\d+|(^|[^.])\B\.\d+)(?:e[+-]?\d+)?\b/i,lookbehind:!0},operator:/[-+*\/&?@^]|<(?:=>?|>)?|>=?|=>?|\.\.\.?/,punctuation:/[,;\[\](){}]/},t.languages.pq=t.languages.powerquery,t.languages.mscript=t.languages.powerquery}return YF}var JF,kq;function mot(){if(kq)return JF;kq=1,JF=e,e.displayName="powershell",e.aliases=[];function e(t){(function(r){var n=r.languages.powershell={comment:[{pattern:/(^|[^`])<#[\s\S]*?#>/,lookbehind:!0},{pattern:/(^|[^`])#.*/,lookbehind:!0}],string:[{pattern:/"(?:`[\s\S]|[^`"])*"/,greedy:!0,inside:null},{pattern:/'(?:[^']|'')*'/,greedy:!0}],namespace:/\[[a-z](?:\[(?:\[[^\]]*\]|[^\[\]])*\]|[^\[\]])*\]/i,boolean:/\$(?:false|true)\b/i,variable:/\$\w+\b/,function:[/\b(?:Add|Approve|Assert|Backup|Block|Checkpoint|Clear|Close|Compare|Complete|Compress|Confirm|Connect|Convert|ConvertFrom|ConvertTo|Copy|Debug|Deny|Disable|Disconnect|Dismount|Edit|Enable|Enter|Exit|Expand|Export|Find|ForEach|Format|Get|Grant|Group|Hide|Import|Initialize|Install|Invoke|Join|Limit|Lock|Measure|Merge|Move|New|Open|Optimize|Out|Ping|Pop|Protect|Publish|Push|Read|Receive|Redo|Register|Remove|Rename|Repair|Request|Reset|Resize|Resolve|Restart|Restore|Resume|Revoke|Save|Search|Select|Send|Set|Show|Skip|Sort|Split|Start|Step|Stop|Submit|Suspend|Switch|Sync|Tee|Test|Trace|Unblock|Undo|Uninstall|Unlock|Unprotect|Unpublish|Unregister|Update|Use|Wait|Watch|Where|Write)-[a-z]+\b/i,/\b(?:ac|cat|chdir|clc|cli|clp|clv|compare|copy|cp|cpi|cpp|cvpa|dbp|del|diff|dir|ebp|echo|epal|epcsv|epsn|erase|fc|fl|ft|fw|gal|gbp|gc|gci|gcs|gdr|gi|gl|gm|gp|gps|group|gsv|gu|gv|gwmi|iex|ii|ipal|ipcsv|ipsn|irm|iwmi|iwr|kill|lp|ls|measure|mi|mount|move|mp|mv|nal|ndr|ni|nv|ogv|popd|ps|pushd|pwd|rbp|rd|rdr|ren|ri|rm|rmdir|rni|rnp|rp|rv|rvpa|rwmi|sal|saps|sasv|sbp|sc|select|set|shcm|si|sl|sleep|sls|sort|sp|spps|spsv|start|sv|swmi|tee|trcm|type|write)\b/i],keyword:/\b(?:Begin|Break|Catch|Class|Continue|Data|Define|Do|DynamicParam|Else|ElseIf|End|Exit|Filter|Finally|For|ForEach|From|Function|If|InlineScript|Parallel|Param|Process|Return|Sequence|Switch|Throw|Trap|Try|Until|Using|Var|While|Workflow)\b/i,operator:{pattern:/(^|\W)(?:!|-(?:b?(?:and|x?or)|as|(?:Not)?(?:Contains|In|Like|Match)|eq|ge|gt|is(?:Not)?|Join|le|lt|ne|not|Replace|sh[lr])\b|-[-=]?|\+[+=]?|[*\/%]=?)/i,lookbehind:!0},punctuation:/[|{}[\];(),.]/};n.string[0].inside={function:{pattern:/(^|[^`])\$\((?:\$\([^\r\n()]*\)|(?!\$\()[^\r\n)])*\)/,lookbehind:!0,inside:n},boolean:n.boolean,variable:n.variable}})(t)}return JF}var XF,Cq;function got(){if(Cq)return XF;Cq=1,XF=e,e.displayName="processing",e.aliases=[];function e(t){t.languages.processing=t.languages.extend("clike",{keyword:/\b(?:break|case|catch|class|continue|default|else|extends|final|for|if|implements|import|new|null|private|public|return|static|super|switch|this|try|void|while)\b/,function:/\b\w+(?=\s*\()/,operator:/<[<=]?|>[>=]?|&&?|\|\|?|[%?]|[!=+\-*\/]=?/}),t.languages.insertBefore("processing","number",{constant:/\b(?!XML\b)[A-Z][A-Z\d_]+\b/,type:{pattern:/\b(?:boolean|byte|char|color|double|float|int|[A-Z]\w*)\b/,alias:"class-name"}})}return XF}var ZF,Tq;function bot(){if(Tq)return ZF;Tq=1,ZF=e,e.displayName="prolog",e.aliases=[];function e(t){t.languages.prolog={comment:{pattern:/\/\*[\s\S]*?\*\/|%.*/,greedy:!0},string:{pattern:/(["'])(?:\1\1|\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1(?!\1)/,greedy:!0},builtin:/\b(?:fx|fy|xf[xy]?|yfx?)\b/,function:/\b[a-z]\w*(?:(?=\()|\/\d+)/,number:/\b\d+(?:\.\d*)?/,operator:/[:\\=><\-?*@\/;+^|!$.]+|\b(?:is|mod|not|xor)\b/,punctuation:/[(){}\[\],]/}}return ZF}var QF,Aq;function yot(){if(Aq)return QF;Aq=1,QF=e,e.displayName="promql",e.aliases=[];function e(t){(function(r){var n=["sum","min","max","avg","group","stddev","stdvar","count","count_values","bottomk","topk","quantile"],a=["on","ignoring","group_right","group_left","by","without"],i=["offset"],o=n.concat(a,i);r.languages.promql={comment:{pattern:/(^[ \t]*)#.*/m,lookbehind:!0},"vector-match":{pattern:new RegExp("((?:"+a.join("|")+")\\s*)\\([^)]*\\)"),lookbehind:!0,inside:{"label-key":{pattern:/\b[^,]+\b/,alias:"attr-name"},punctuation:/[(),]/}},"context-labels":{pattern:/\{[^{}]*\}/,inside:{"label-key":{pattern:/\b[a-z_]\w*(?=\s*(?:=|![=~]))/,alias:"attr-name"},"label-value":{pattern:/(["'`])(?:\\[\s\S]|(?!\1)[^\\])*\1/,greedy:!0,alias:"attr-value"},punctuation:/\{|\}|=~?|![=~]|,/}},"context-range":[{pattern:/\[[\w\s:]+\]/,inside:{punctuation:/\[|\]|:/,"range-duration":{pattern:/\b(?:\d+(?:[smhdwy]|ms))+\b/i,alias:"number"}}},{pattern:/(\boffset\s+)\w+/,lookbehind:!0,inside:{"range-duration":{pattern:/\b(?:\d+(?:[smhdwy]|ms))+\b/i,alias:"number"}}}],keyword:new RegExp("\\b(?:"+o.join("|")+")\\b","i"),function:/\b[a-z_]\w*(?=\s*\()/i,number:/[-+]?(?:(?:\b\d+(?:\.\d+)?|\B\.\d+)(?:e[-+]?\d+)?\b|\b(?:0x[0-9a-f]+|nan|inf)\b)/i,operator:/[\^*/%+-]|==|!=|<=|<|>=|>|\b(?:and|or|unless)\b/i,punctuation:/[{};()`,.[\]]/}})(t)}return QF}var e3,Fq;function vot(){if(Fq)return e3;Fq=1,e3=e,e.displayName="properties",e.aliases=[];function e(t){t.languages.properties={comment:/^[ \t]*[#!].*$/m,"attr-value":{pattern:/(^[ \t]*(?:\\(?:\r\n|[\s\S])|[^\\\s:=])+(?: *[=:] *(?! )| ))(?:\\(?:\r\n|[\s\S])|[^\\\r\n])+/m,lookbehind:!0},"attr-name":/^[ \t]*(?:\\(?:\r\n|[\s\S])|[^\\\s:=])+(?= *[=:]| )/m,punctuation:/[=:]/}}return e3}var t3,_q;function Sot(){if(_q)return t3;_q=1,t3=e,e.displayName="protobuf",e.aliases=[];function e(t){(function(r){var n=/\b(?:bool|bytes|double|s?fixed(?:32|64)|float|[su]?int(?:32|64)|string)\b/;r.languages.protobuf=r.languages.extend("clike",{"class-name":[{pattern:/(\b(?:enum|extend|message|service)\s+)[A-Za-z_]\w*(?=\s*\{)/,lookbehind:!0},{pattern:/(\b(?:rpc\s+\w+|returns)\s*\(\s*(?:stream\s+)?)\.?[A-Za-z_]\w*(?:\.[A-Za-z_]\w*)*(?=\s*\))/,lookbehind:!0}],keyword:/\b(?:enum|extend|extensions|import|message|oneof|option|optional|package|public|repeated|required|reserved|returns|rpc(?=\s+\w)|service|stream|syntax|to)\b(?!\s*=\s*\d)/,function:/\b[a-z_]\w*(?=\s*\()/i}),r.languages.insertBefore("protobuf","operator",{map:{pattern:/\bmap<\s*[\w.]+\s*,\s*[\w.]+\s*>(?=\s+[a-z_]\w*\s*[=;])/i,alias:"class-name",inside:{punctuation:/[<>.,]/,builtin:n}},builtin:n,"positional-class-name":{pattern:/(?:\b|\B\.)[a-z_]\w*(?:\.[a-z_]\w*)*(?=\s+[a-z_]\w*\s*[=;])/i,alias:"class-name",inside:{punctuation:/\./}},annotation:{pattern:/(\[\s*)[a-z_]\w*(?=\s*=)/i,lookbehind:!0}})})(t)}return t3}var r3,Rq;function Eot(){if(Rq)return r3;Rq=1,r3=e,e.displayName="psl",e.aliases=[];function e(t){t.languages.psl={comment:{pattern:/#.*/,greedy:!0},string:{pattern:/"(?:\\.|[^\\"])*"/,greedy:!0,inside:{symbol:/\\[ntrbA-Z"\\]/}},"heredoc-string":{pattern:/<<<([a-zA-Z_]\w*)[\r\n](?:.*[\r\n])*?\1\b/,alias:"string",greedy:!0},keyword:/\b(?:__multi|__single|case|default|do|else|elsif|exit|export|for|foreach|function|if|last|line|local|next|requires|return|switch|until|while|word)\b/,constant:/\b(?:ALARM|CHART_ADD_GRAPH|CHART_DELETE_GRAPH|CHART_DESTROY|CHART_LOAD|CHART_PRINT|EOF|OFFLINE|OK|PSL_PROF_LOG|R_CHECK_HORIZ|R_CHECK_VERT|R_CLICKER|R_COLUMN|R_FRAME|R_ICON|R_LABEL|R_LABEL_CENTER|R_LIST_MULTIPLE|R_LIST_MULTIPLE_ND|R_LIST_SINGLE|R_LIST_SINGLE_ND|R_MENU|R_POPUP|R_POPUP_SCROLLED|R_RADIO_HORIZ|R_RADIO_VERT|R_ROW|R_SCALE_HORIZ|R_SCALE_VERT|R_SEP_HORIZ|R_SEP_VERT|R_SPINNER|R_TEXT_FIELD|R_TEXT_FIELD_LABEL|R_TOGGLE|TRIM_LEADING|TRIM_LEADING_AND_TRAILING|TRIM_REDUNDANT|TRIM_TRAILING|VOID|WARN)\b/,boolean:/\b(?:FALSE|False|NO|No|TRUE|True|YES|Yes|false|no|true|yes)\b/,variable:/\b(?:PslDebug|errno|exit_status)\b/,builtin:{pattern:/\b(?:PslExecute|PslFunctionCall|PslFunctionExists|PslSetOptions|_snmp_debug|acos|add_diary|annotate|annotate_get|ascii_to_ebcdic|asctime|asin|atan|atexit|batch_set|blackout|cat|ceil|chan_exists|change_state|close|code_cvt|cond_signal|cond_wait|console_type|convert_base|convert_date|convert_locale_date|cos|cosh|create|date|dcget_text|destroy|destroy_lock|dget_text|difference|dump_hist|ebcdic_to_ascii|encrypt|event_archive|event_catalog_get|event_check|event_query|event_range_manage|event_range_query|event_report|event_schedule|event_trigger|event_trigger2|execute|exists|exp|fabs|file|floor|fmod|fopen|fseek|ftell|full_discovery|get|get_chan_info|get_ranges|get_text|get_vars|getenv|gethostinfo|getpid|getpname|grep|history|history_get_retention|in_transition|index|int|internal|intersection|is_var|isnumber|join|kill|length|lines|lock|lock_info|log|log10|loge|matchline|msg_check|msg_get_format|msg_get_severity|msg_printf|msg_sprintf|ntharg|nthargf|nthline|nthlinef|num_bytes|num_consoles|pconfig|popen|poplines|pow|print|printf|proc_exists|process|random|read|readln|refresh_parameters|remote_check|remote_close|remote_event_query|remote_event_trigger|remote_file_send|remote_open|remove|replace|rindex|sec_check_priv|sec_store_get|sec_store_set|set|set_alarm_ranges|set_locale|share|sin|sinh|sleep|snmp_agent_config|snmp_agent_start|snmp_agent_stop|snmp_close|snmp_config|snmp_get|snmp_get_next|snmp_h_get|snmp_h_get_next|snmp_h_set|snmp_open|snmp_set|snmp_trap_ignore|snmp_trap_listen|snmp_trap_raise_std_trap|snmp_trap_receive|snmp_trap_register_im|snmp_trap_send|snmp_walk|sopen|sort|splitline|sprintf|sqrt|srandom|str_repeat|strcasecmp|subset|substr|system|tail|tan|tanh|text_domain|time|tmpnam|tolower|toupper|trace_psl_process|trim|union|unique|unlock|unset|va_arg|va_start|write)\b/,alias:"builtin-function"},"foreach-variable":{pattern:/(\bforeach\s+(?:(?:\w+\b|"(?:\\.|[^\\"])*")\s+){0,2})[_a-zA-Z]\w*(?=\s*\()/,lookbehind:!0,greedy:!0},function:/\b[_a-z]\w*\b(?=\s*\()/i,number:/\b(?:0x[0-9a-f]+|\d+(?:\.\d+)?)\b/i,operator:/--|\+\+|&&=?|\|\|=?|<<=?|>>=?|[=!]~|[-+*/%&|^!=<>]=?|\.|[:?]/,punctuation:/[(){}\[\];,]/}}return r3}var n3,Dq;function xot(){if(Dq)return n3;Dq=1,n3=e,e.displayName="pug",e.aliases=[];function e(t){(function(r){r.languages.pug={comment:{pattern:/(^([\t ]*))\/\/.*(?:(?:\r?\n|\r)\2[\t ].+)*/m,lookbehind:!0},"multiline-script":{pattern:/(^([\t ]*)script\b.*\.[\t ]*)(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/m,lookbehind:!0,inside:r.languages.javascript},filter:{pattern:/(^([\t ]*)):.+(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/m,lookbehind:!0,inside:{"filter-name":{pattern:/^:[\w-]+/,alias:"variable"},text:/\S[\s\S]*/}},"multiline-plain-text":{pattern:/(^([\t ]*)[\w\-#.]+\.[\t ]*)(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/m,lookbehind:!0},markup:{pattern:/(^[\t ]*)<.+/m,lookbehind:!0,inside:r.languages.markup},doctype:{pattern:/((?:^|\n)[\t ]*)doctype(?: .+)?/,lookbehind:!0},"flow-control":{pattern:/(^[\t ]*)(?:case|default|each|else|if|unless|when|while)\b(?: .+)?/m,lookbehind:!0,inside:{each:{pattern:/^each .+? in\b/,inside:{keyword:/\b(?:each|in)\b/,punctuation:/,/}},branch:{pattern:/^(?:case|default|else|if|unless|when|while)\b/,alias:"keyword"},rest:r.languages.javascript}},keyword:{pattern:/(^[\t ]*)(?:append|block|extends|include|prepend)\b.+/m,lookbehind:!0},mixin:[{pattern:/(^[\t ]*)mixin .+/m,lookbehind:!0,inside:{keyword:/^mixin/,function:/\w+(?=\s*\(|\s*$)/,punctuation:/[(),.]/}},{pattern:/(^[\t ]*)\+.+/m,lookbehind:!0,inside:{name:{pattern:/^\+\w+/,alias:"function"},rest:r.languages.javascript}}],script:{pattern:/(^[\t ]*script(?:(?:&[^(]+)?\([^)]+\))*[\t ]).+/m,lookbehind:!0,inside:r.languages.javascript},"plain-text":{pattern:/(^[\t ]*(?!-)[\w\-#.]*[\w\-](?:(?:&[^(]+)?\([^)]+\))*\/?[\t ]).+/m,lookbehind:!0},tag:{pattern:/(^[\t ]*)(?!-)[\w\-#.]*[\w\-](?:(?:&[^(]+)?\([^)]+\))*\/?:?/m,lookbehind:!0,inside:{attributes:[{pattern:/&[^(]+\([^)]+\)/,inside:r.languages.javascript},{pattern:/\([^)]+\)/,inside:{"attr-value":{pattern:/(=\s*(?!\s))(?:\{[^}]*\}|[^,)\r\n]+)/,lookbehind:!0,inside:r.languages.javascript},"attr-name":/[\w-]+(?=\s*!?=|\s*[,)])/,punctuation:/[!=(),]+/}}],punctuation:/:/,"attr-id":/#[\w\-]+/,"attr-class":/\.[\w\-]+/}},code:[{pattern:/(^[\t ]*(?:-|!?=)).+/m,lookbehind:!0,inside:r.languages.javascript}],punctuation:/[.\-!=|]+/};for(var n=/(^([\t ]*)):<filter_name>(?:(?:\r?\n|\r(?!\n))(?:\2[\t ].+|\s*?(?=\r?\n|\r)))+/.source,a=[{filter:"atpl",language:"twig"},{filter:"coffee",language:"coffeescript"},"ejs","handlebars","less","livescript","markdown",{filter:"sass",language:"scss"},"stylus"],i={},o=0,s=a.length;o<s;o++){var l=a[o];l=typeof l=="string"?{filter:l,language:l}:l,r.languages[l.language]&&(i["filter-"+l.filter]={pattern:RegExp(n.replace("<filter_name>",function(){return l.filter}),"m"),lookbehind:!0,inside:{"filter-name":{pattern:/^:[\w-]+/,alias:"variable"},text:{pattern:/\S[\s\S]*/,alias:[l.language,"language-"+l.language],inside:r.languages[l.language]}}})}r.languages.insertBefore("pug","filter",i)})(t)}return n3}var a3,Nq;function wot(){if(Nq)return a3;Nq=1,a3=e,e.displayName="puppet",e.aliases=[];function e(t){(function(r){r.languages.puppet={heredoc:[{pattern:/(@\("([^"\r\n\/):]+)"(?:\/[nrts$uL]*)?\).*(?:\r?\n|\r))(?:.*(?:\r?\n|\r(?!\n)))*?[ \t]*(?:\|[ \t]*)?(?:-[ \t]*)?\2/,lookbehind:!0,alias:"string",inside:{punctuation:/(?=\S).*\S(?= *$)/}},{pattern:/(@\(([^"\r\n\/):]+)(?:\/[nrts$uL]*)?\).*(?:\r?\n|\r))(?:.*(?:\r?\n|\r(?!\n)))*?[ \t]*(?:\|[ \t]*)?(?:-[ \t]*)?\2/,lookbehind:!0,greedy:!0,alias:"string",inside:{punctuation:/(?=\S).*\S(?= *$)/}},{pattern:/@\("?(?:[^"\r\n\/):]+)"?(?:\/[nrts$uL]*)?\)/,alias:"string",inside:{punctuation:{pattern:/(\().+?(?=\))/,lookbehind:!0}}}],"multiline-comment":{pattern:/(^|[^\\])\/\*[\s\S]*?\*\//,lookbehind:!0,greedy:!0,alias:"comment"},regex:{pattern:/((?:\bnode\s+|[~=\(\[\{,]\s*|[=+]>\s*|^\s*))\/(?:[^\/\\]|\\[\s\S])+\/(?:[imx]+\b|\B)/,lookbehind:!0,greedy:!0,inside:{"extended-regex":{pattern:/^\/(?:[^\/\\]|\\[\s\S])+\/[im]*x[im]*$/,inside:{comment:/#.*/}}}},comment:{pattern:/(^|[^\\])#.*/,lookbehind:!0,greedy:!0},string:{pattern:/(["'])(?:\$\{(?:[^'"}]|(["'])(?:(?!\2)[^\\]|\\[\s\S])*\2)+\}|\$(?!\{)|(?!\1)[^\\$]|\\[\s\S])*\1/,greedy:!0,inside:{"double-quoted":{pattern:/^"[\s\S]*"$/,inside:{}}}},variable:{pattern:/\$(?:::)?\w+(?:::\w+)*/,inside:{punctuation:/::/}},"attr-name":/(?:\b\w+|\*)(?=\s*=>)/,function:[{pattern:/(\.)(?!\d)\w+/,lookbehind:!0},/\b(?:contain|debug|err|fail|include|info|notice|realize|require|tag|warning)\b|\b(?!\d)\w+(?=\()/],number:/\b(?:0x[a-f\d]+|\d+(?:\.\d+)?(?:e-?\d+)?)\b/i,boolean:/\b(?:false|true)\b/,keyword:/\b(?:application|attr|case|class|consumes|default|define|else|elsif|function|if|import|inherits|node|private|produces|type|undef|unless)\b/,datatype:{pattern:/\b(?:Any|Array|Boolean|Callable|Catalogentry|Class|Collection|Data|Default|Enum|Float|Hash|Integer|NotUndef|Numeric|Optional|Pattern|Regexp|Resource|Runtime|Scalar|String|Struct|Tuple|Type|Undef|Variant)\b/,alias:"symbol"},operator:/=[=~>]?|![=~]?|<(?:<\|?|[=~|-])?|>[>=]?|->?|~>|\|>?>?|[*\/%+?]|\b(?:and|in|or)\b/,punctuation:/[\[\]{}().,;]|:+/};var n=[{pattern:/(^|[^\\])\$\{(?:[^'"{}]|\{[^}]*\}|(["'])(?:(?!\2)[^\\]|\\[\s\S])*\2)+\}/,lookbehind:!0,inside:{"short-variable":{pattern:/(^\$\{)(?!\w+\()(?:::)?\w+(?:::\w+)*/,lookbehind:!0,alias:"variable",inside:{punctuation:/::/}},delimiter:{pattern:/^\$/,alias:"variable"},rest:r.languages.puppet}},{pattern:/(^|[^\\])\$(?:::)?\w+(?:::\w+)*/,lookbehind:!0,alias:"variable",inside:{punctuation:/::/}}];r.languages.puppet.heredoc[0].inside.interpolation=n,r.languages.puppet.string.inside["double-quoted"].inside.interpolation=n})(t)}return a3}var i3,Lq;function kot(){if(Lq)return i3;Lq=1,i3=e,e.displayName="pure",e.aliases=[];function e(t){(function(r){r.languages.pure={comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?\*\//,lookbehind:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0},/#!.+/],"inline-lang":{pattern:/%<[\s\S]+?%>/,greedy:!0,inside:{lang:{pattern:/(^%< *)-\*-.+?-\*-/,lookbehind:!0,alias:"comment"},delimiter:{pattern:/^%<.*|%>$/,alias:"punctuation"}}},string:{pattern:/"(?:\\.|[^"\\\r\n])*"/,greedy:!0},number:{pattern:/((?:\.\.)?)(?:\b(?:inf|nan)\b|\b0x[\da-f]+|(?:\b(?:0b)?\d+(?:\.\d+)?|\B\.\d+)(?:e[+-]?\d+)?L?)/i,lookbehind:!0},keyword:/\b(?:NULL|ans|break|bt|case|catch|cd|clear|const|def|del|dump|else|end|exit|extern|false|force|help|if|infix[lr]?|interface|let|ls|mem|namespace|nonfix|of|otherwise|outfix|override|postfix|prefix|private|public|pwd|quit|run|save|show|stats|then|throw|trace|true|type|underride|using|when|with)\b/,function:/\b(?:abs|add_(?:addr|constdef|(?:fundef|interface|macdef|typedef)(?:_at)?|vardef)|all|any|applp?|arity|bigintp?|blob(?:_crc|_size|p)?|boolp?|byte_c?string(?:_pointer)?|byte_(?:matrix|pointer)|calloc|cat|catmap|ceil|char[ps]?|check_ptrtag|chr|clear_sentry|clearsym|closurep?|cmatrixp?|cols?|colcat(?:map)?|colmap|colrev|colvector(?:p|seq)?|complex(?:_float_(?:matrix|pointer)|_matrix(?:_view)?|_pointer|p)?|conj|cookedp?|cst|cstring(?:_(?:dup|list|vector))?|curry3?|cyclen?|del_(?:constdef|fundef|interface|macdef|typedef|vardef)|delete|diag(?:mat)?|dim|dmatrixp?|do|double(?:_matrix(?:_view)?|_pointer|p)?|dowith3?|drop|dropwhile|eval(?:cmd)?|exactp|filter|fix|fixity|flip|float(?:_matrix|_pointer)|floor|fold[lr]1?|frac|free|funp?|functionp?|gcd|get(?:_(?:byte|constdef|double|float|fundef|int(?:64)?|interface(?:_typedef)?|long|macdef|pointer|ptrtag|sentry|short|string|typedef|vardef))?|globsym|hash|head|id|im|imatrixp?|index|inexactp|infp|init|insert|int(?:_matrix(?:_view)?|_pointer|p)?|int64_(?:matrix|pointer)|integerp?|iteraten?|iterwhile|join|keys?|lambdap?|last(?:err(?:pos)?)?|lcd|list[2p]?|listmap|make_ptrtag|malloc|map|matcat|matrixp?|max|member|min|nanp|nargs|nmatrixp?|null|numberp?|ord|pack(?:ed)?|pointer(?:_cast|_tag|_type|p)?|pow|pred|ptrtag|put(?:_(?:byte|double|float|int(?:64)?|long|pointer|short|string))?|rationalp?|re|realp?|realloc|recordp?|redim|reduce(?:_with)?|refp?|repeatn?|reverse|rlistp?|round|rows?|rowcat(?:map)?|rowmap|rowrev|rowvector(?:p|seq)?|same|scan[lr]1?|sentry|sgn|short_(?:matrix|pointer)|slice|smatrixp?|sort|split|str|strcat|stream|stride|string(?:_(?:dup|list|vector)|p)?|subdiag(?:mat)?|submat|subseq2?|substr|succ|supdiag(?:mat)?|symbolp?|tail|take|takewhile|thunkp?|transpose|trunc|tuplep?|typep|ubyte|uint(?:64)?|ulong|uncurry3?|unref|unzip3?|update|ushort|vals?|varp?|vector(?:p|seq)?|void|zip3?|zipwith3?)\b/,special:{pattern:/\b__[a-z]+__\b/i,alias:"builtin"},operator:/(?:[!"#$%&'*+,\-.\/:<=>?@\\^`|~\u00a1-\u00bf\u00d7-\u00f7\u20d0-\u2bff]|\b_+\b)+|\b(?:and|div|mod|not|or)\b/,punctuation:/[(){}\[\];,|]/};var n=["c",{lang:"c++",alias:"cpp"},"fortran"],a=/%< *-\*- *<lang>\d* *-\*-[\s\S]+?%>/.source;n.forEach(function(i){var o=i;if(typeof i!="string"&&(o=i.alias,i=i.lang),r.languages[o]){var s={};s["inline-lang-"+o]={pattern:RegExp(a.replace("<lang>",i.replace(/([.+*?\/\\(){}\[\]])/g,"\\$1")),"i"),inside:r.util.clone(r.languages.pure["inline-lang"].inside)},s["inline-lang-"+o].inside.rest=r.util.clone(r.languages[o]),r.languages.insertBefore("pure","inline-lang",s)}}),r.languages.c&&(r.languages.pure["inline-lang"].inside.rest=r.util.clone(r.languages.c))})(t)}return i3}var o3,Mq;function Cot(){if(Mq)return o3;Mq=1,o3=e,e.displayName="purebasic",e.aliases=[];function e(t){t.languages.purebasic=t.languages.extend("clike",{comment:/;.*/,keyword:/\b(?:align|and|as|break|calldebugger|case|compilercase|compilerdefault|compilerelse|compilerelseif|compilerendif|compilerendselect|compilererror|compilerif|compilerselect|continue|data|datasection|debug|debuglevel|declare|declarec|declarecdll|declaredll|declaremodule|default|define|dim|disableasm|disabledebugger|disableexplicit|else|elseif|enableasm|enabledebugger|enableexplicit|end|enddatasection|enddeclaremodule|endenumeration|endif|endimport|endinterface|endmacro|endmodule|endprocedure|endselect|endstructure|endstructureunion|endwith|enumeration|extends|fakereturn|for|foreach|forever|global|gosub|goto|if|import|importc|includebinary|includefile|includepath|interface|macro|module|newlist|newmap|next|not|or|procedure|procedurec|procedurecdll|proceduredll|procedurereturn|protected|prototype|prototypec|read|redim|repeat|restore|return|runtime|select|shared|static|step|structure|structureunion|swap|threaded|to|until|wend|while|with|xincludefile|xor)\b/i,function:/\b\w+(?:\.\w+)?\s*(?=\()/,number:/(?:\$[\da-f]+|\b-?(?:\d+(?:\.\d+)?|\.\d+)(?:e[+-]?\d+)?)\b/i,operator:/(?:@\*?|\?|\*)\w+|-[>-]?|\+\+?|!=?|<<?=?|>>?=?|==?|&&?|\|?\||[~^%?*/@]/}),t.languages.insertBefore("purebasic","keyword",{tag:/#\w+\$?/,asm:{pattern:/(^[\t ]*)!.*/m,lookbehind:!0,alias:"tag",inside:{comment:/;.*/,string:{pattern:/(["'`])(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},"label-reference-anonymous":{pattern:/(!\s*j[a-z]+\s+)@[fb]/i,lookbehind:!0,alias:"fasm-label"},"label-reference-addressed":{pattern:/(!\s*j[a-z]+\s+)[A-Z._?$@][\w.?$@~#]*/i,lookbehind:!0,alias:"fasm-label"},keyword:[/\b(?:extern|global)\b[^;\r\n]*/i,/\b(?:CPU|DEFAULT|FLOAT)\b.*/],function:{pattern:/^([\t ]*!\s*)[\da-z]+(?=\s|$)/im,lookbehind:!0},"function-inline":{pattern:/(:\s*)[\da-z]+(?=\s)/i,lookbehind:!0,alias:"function"},label:{pattern:/^([\t ]*!\s*)[A-Za-z._?$@][\w.?$@~#]*(?=:)/m,lookbehind:!0,alias:"fasm-label"},register:/\b(?:st\d|[xyz]mm\d\d?|[cdt]r\d|r\d\d?[bwd]?|[er]?[abcd]x|[abcd][hl]|[er]?(?:bp|di|si|sp)|[cdefgs]s|mm\d+)\b/i,number:/(?:\b|-|(?=\$))(?:0[hx](?:[\da-f]*\.)?[\da-f]+(?:p[+-]?\d+)?|\d[\da-f]+[hx]|\$\d[\da-f]*|0[oq][0-7]+|[0-7]+[oq]|0[by][01]+|[01]+[by]|0[dt]\d+|(?:\d+(?:\.\d+)?|\.\d+)(?:\.?e[+-]?\d+)?[dt]?)\b/i,operator:/[\[\]*+\-/%<>=&|$!,.:]/}}}),delete t.languages.purebasic["class-name"],delete t.languages.purebasic.boolean,t.languages.pbfasm=t.languages.purebasic}return o3}var s3,Iq;function Tot(){if(Iq)return s3;Iq=1;var e=fL();s3=t,t.displayName="purescript",t.aliases=["purs"];function t(r){r.register(e),r.languages.purescript=r.languages.extend("haskell",{keyword:/\b(?:ado|case|class|data|derive|do|else|forall|if|in|infixl|infixr|instance|let|module|newtype|of|primitive|then|type|where)\b|∀/,"import-statement":{pattern:/(^[\t ]*)import\s+[A-Z][\w']*(?:\.[A-Z][\w']*)*(?:\s+as\s+[A-Z][\w']*(?:\.[A-Z][\w']*)*)?(?:\s+hiding\b)?/m,lookbehind:!0,inside:{keyword:/\b(?:as|hiding|import)\b/,punctuation:/\./}},builtin:/\b(?:absurd|add|ap|append|apply|between|bind|bottom|clamp|compare|comparing|compose|conj|const|degree|discard|disj|div|eq|flap|flip|gcd|identity|ifM|join|lcm|liftA1|liftM1|map|max|mempty|min|mod|mul|negate|not|notEq|one|otherwise|recip|show|sub|top|unit|unless|unlessM|void|when|whenM|zero)\b/,operator:[r.languages.haskell.operator[0],r.languages.haskell.operator[2],/[\xa2-\xa6\xa8\xa9\xac\xae-\xb1\xb4\xb8\xd7\xf7\u02c2-\u02c5\u02d2-\u02df\u02e5-\u02eb\u02ed\u02ef-\u02ff\u0375\u0384\u0385\u03f6\u0482\u058d-\u058f\u0606-\u0608\u060b\u060e\u060f\u06de\u06e9\u06fd\u06fe\u07f6\u07fe\u07ff\u09f2\u09f3\u09fa\u09fb\u0af1\u0b70\u0bf3-\u0bfa\u0c7f\u0d4f\u0d79\u0e3f\u0f01-\u0f03\u0f13\u0f15-\u0f17\u0f1a-\u0f1f\u0f34\u0f36\u0f38\u0fbe-\u0fc5\u0fc7-\u0fcc\u0fce\u0fcf\u0fd5-\u0fd8\u109e\u109f\u1390-\u1399\u166d\u17db\u1940\u19de-\u19ff\u1b61-\u1b6a\u1b74-\u1b7c\u1fbd\u1fbf-\u1fc1\u1fcd-\u1fcf\u1fdd-\u1fdf\u1fed-\u1fef\u1ffd\u1ffe\u2044\u2052\u207a-\u207c\u208a-\u208c\u20a0-\u20bf\u2100\u2101\u2103-\u2106\u2108\u2109\u2114\u2116-\u2118\u211e-\u2123\u2125\u2127\u2129\u212e\u213a\u213b\u2140-\u2144\u214a-\u214d\u214f\u218a\u218b\u2190-\u2307\u230c-\u2328\u232b-\u2426\u2440-\u244a\u249c-\u24e9\u2500-\u2767\u2794-\u27c4\u27c7-\u27e5\u27f0-\u2982\u2999-\u29d7\u29dc-\u29fb\u29fe-\u2b73\u2b76-\u2b95\u2b97-\u2bff\u2ce5-\u2cea\u2e50\u2e51\u2e80-\u2e99\u2e9b-\u2ef3\u2f00-\u2fd5\u2ff0-\u2ffb\u3004\u3012\u3013\u3020\u3036\u3037\u303e\u303f\u309b\u309c\u3190\u3191\u3196-\u319f\u31c0-\u31e3\u3200-\u321e\u322a-\u3247\u3250\u3260-\u327f\u328a-\u32b0\u32c0-\u33ff\u4dc0-\u4dff\ua490-\ua4c6\ua700-\ua716\ua720\ua721\ua789\ua78a\ua828-\ua82b\ua836-\ua839\uaa77-\uaa79\uab5b\uab6a\uab6b\ufb29\ufbb2-\ufbc1\ufdfc\ufdfd\ufe62\ufe64-\ufe66\ufe69\uff04\uff0b\uff1c-\uff1e\uff3e\uff40\uff5c\uff5e\uffe0-\uffe6\uffe8-\uffee\ufffc\ufffd]/]}),r.languages.purs=r.languages.purescript}return s3}var l3,Oq;function Aot(){if(Oq)return l3;Oq=1,l3=e,e.displayName="python",e.aliases=["py"];function e(t){t.languages.python={comment:{pattern:/(^|[^\\])#.*/,lookbehind:!0,greedy:!0},"string-interpolation":{pattern:/(?:f|fr|rf)(?:("""|''')[\s\S]*?\1|("|')(?:\\.|(?!\2)[^\\\r\n])*\2)/i,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^{])(?:\{\{)*)\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}])+\})+\})+\}/,lookbehind:!0,inside:{"format-spec":{pattern:/(:)[^:(){}]+(?=\}$)/,lookbehind:!0},"conversion-option":{pattern:/![sra](?=[:}]$)/,alias:"punctuation"},rest:null}},string:/[\s\S]+/}},"triple-quoted-string":{pattern:/(?:[rub]|br|rb)?("""|''')[\s\S]*?\1/i,greedy:!0,alias:"string"},string:{pattern:/(?:[rub]|br|rb)?("|')(?:\\.|(?!\1)[^\\\r\n])*\1/i,greedy:!0},function:{pattern:/((?:^|\s)def[ \t]+)[a-zA-Z_]\w*(?=\s*\()/g,lookbehind:!0},"class-name":{pattern:/(\bclass\s+)\w+/i,lookbehind:!0},decorator:{pattern:/(^[\t ]*)@\w+(?:\.\w+)*/m,lookbehind:!0,alias:["annotation","punctuation"],inside:{punctuation:/\./}},keyword:/\b(?:_(?=\s*:)|and|as|assert|async|await|break|case|class|continue|def|del|elif|else|except|exec|finally|for|from|global|if|import|in|is|lambda|match|nonlocal|not|or|pass|print|raise|return|try|while|with|yield)\b/,builtin:/\b(?:__import__|abs|all|any|apply|ascii|basestring|bin|bool|buffer|bytearray|bytes|callable|chr|classmethod|cmp|coerce|compile|complex|delattr|dict|dir|divmod|enumerate|eval|execfile|file|filter|float|format|frozenset|getattr|globals|hasattr|hash|help|hex|id|input|int|intern|isinstance|issubclass|iter|len|list|locals|long|map|max|memoryview|min|next|object|oct|open|ord|pow|property|range|raw_input|reduce|reload|repr|reversed|round|set|setattr|slice|sorted|staticmethod|str|sum|super|tuple|type|unichr|unicode|vars|xrange|zip)\b/,boolean:/\b(?:False|None|True)\b/,number:/\b0(?:b(?:_?[01])+|o(?:_?[0-7])+|x(?:_?[a-f0-9])+)\b|(?:\b\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\B\.\d+(?:_\d+)*)(?:e[+-]?\d+(?:_\d+)*)?j?(?!\w)/i,operator:/[-+%=]=?|!=|:=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,punctuation:/[{}[\];(),.:]/},t.languages.python["string-interpolation"].inside.interpolation.inside.rest=t.languages.python,t.languages.py=t.languages.python}return l3}var u3,Bq;function Fot(){if(Bq)return u3;Bq=1,u3=e,e.displayName="q",e.aliases=[];function e(t){t.languages.q={string:/"(?:\\.|[^"\\\r\n])*"/,comment:[{pattern:/([\t )\]}])\/.*/,lookbehind:!0,greedy:!0},{pattern:/(^|\r?\n|\r)\/[\t ]*(?:(?:\r?\n|\r)(?:.*(?:\r?\n|\r(?!\n)))*?(?:\\(?=[\t ]*(?:\r?\n|\r))|$)|\S.*)/,lookbehind:!0,greedy:!0},{pattern:/^\\[\t ]*(?:\r?\n|\r)[\s\S]+/m,greedy:!0},{pattern:/^#!.+/m,greedy:!0}],symbol:/`(?::\S+|[\w.]*)/,datetime:{pattern:/0N[mdzuvt]|0W[dtz]|\d{4}\.\d\d(?:m|\.\d\d(?:T(?:\d\d(?::\d\d(?::\d\d(?:[.:]\d\d\d)?)?)?)?)?[dz]?)|\d\d:\d\d(?::\d\d(?:[.:]\d\d\d)?)?[uvt]?/,alias:"number"},number:/\b(?![01]:)(?:0N[hje]?|0W[hj]?|0[wn]|0x[\da-fA-F]+|\d+(?:\.\d*)?(?:e[+-]?\d+)?[hjfeb]?)/,keyword:/\\\w+\b|\b(?:abs|acos|aj0?|all|and|any|asc|asin|asof|atan|attr|avgs?|binr?|by|ceiling|cols|cor|cos|count|cov|cross|csv|cut|delete|deltas|desc|dev|differ|distinct|div|do|dsave|ej|enlist|eval|except|exec|exit|exp|fby|fills|first|fkeys|flip|floor|from|get|getenv|group|gtime|hclose|hcount|hdel|hopen|hsym|iasc|identity|idesc|if|ij|in|insert|inter|inv|keys?|last|like|list|ljf?|load|log|lower|lsq|ltime|ltrim|mavg|maxs?|mcount|md5|mdev|med|meta|mins?|mmax|mmin|mmu|mod|msum|neg|next|not|null|or|over|parse|peach|pj|plist|prds?|prev|prior|rand|rank|ratios|raze|read0|read1|reciprocal|reval|reverse|rload|rotate|rsave|rtrim|save|scan|scov|sdev|select|set|setenv|show|signum|sin|sqrt|ssr?|string|sublist|sums?|sv|svar|system|tables|tan|til|trim|txf|type|uj|ungroup|union|update|upper|upsert|value|var|views?|vs|wavg|where|while|within|wj1?|wsum|ww|xasc|xbar|xcols?|xdesc|xexp|xgroup|xkey|xlog|xprev|xrank)\b/,adverb:{pattern:/['\/\\]:?|\beach\b/,alias:"function"},verb:{pattern:/(?:\B\.\B|\b[01]:|<[=>]?|>=?|[:+\-*%,!?~=|$&#@^]):?|\b_\b:?/,alias:"operator"},punctuation:/[(){}\[\];.]/}}return u3}var c3,Pq;function _ot(){if(Pq)return c3;Pq=1,c3=e,e.displayName="qml",e.aliases=[];function e(t){(function(r){for(var n=/"(?:\\.|[^\\"\r\n])*"|'(?:\\.|[^\\'\r\n])*'/.source,a=/\/\/.*(?!.)|\/\*(?:[^*]|\*(?!\/))*\*\//.source,i=/(?:[^\\()[\]{}"'/]|<string>|\/(?![*/])|<comment>|\(<expr>*\)|\[<expr>*\]|\{<expr>*\}|\\[\s\S])/.source.replace(/<string>/g,function(){return n}).replace(/<comment>/g,function(){return a}),o=0;o<2;o++)i=i.replace(/<expr>/g,function(){return i});i=i.replace(/<expr>/g,"[^\\s\\S]"),r.languages.qml={comment:{pattern:/\/\/.*|\/\*[\s\S]*?\*\//,greedy:!0},"javascript-function":{pattern:RegExp(/((?:^|;)[ \t]*)function\s+(?!\s)[_$a-zA-Z\xA0-\uFFFF](?:(?!\s)[$\w\xA0-\uFFFF])*\s*\(<js>*\)\s*\{<js>*\}/.source.replace(/<js>/g,function(){return i}),"m"),lookbehind:!0,greedy:!0,alias:"language-javascript",inside:r.languages.javascript},"class-name":{pattern:/((?:^|[:;])[ \t]*)(?!\d)\w+(?=[ \t]*\{|[ \t]+on\b)/m,lookbehind:!0},property:[{pattern:/((?:^|[;{])[ \t]*)(?!\d)\w+(?:\.\w+)*(?=[ \t]*:)/m,lookbehind:!0},{pattern:/((?:^|[;{])[ \t]*)property[ \t]+(?!\d)\w+(?:\.\w+)*[ \t]+(?!\d)\w+(?:\.\w+)*(?=[ \t]*:)/m,lookbehind:!0,inside:{keyword:/^property/,property:/\w+(?:\.\w+)*/}}],"javascript-expression":{pattern:RegExp(/(:[ \t]*)(?![\s;}[])(?:(?!$|[;}])<js>)+/.source.replace(/<js>/g,function(){return i}),"m"),lookbehind:!0,greedy:!0,alias:"language-javascript",inside:r.languages.javascript},string:{pattern:/"(?:\\.|[^\\"\r\n])*"/,greedy:!0},keyword:/\b(?:as|import|on)\b/,punctuation:/[{}[\]:;,]/}})(t)}return c3}var d3,zq;function Rot(){if(zq)return d3;zq=1,d3=e,e.displayName="qore",e.aliases=[];function e(t){t.languages.qore=t.languages.extend("clike",{comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:\/\/|#).*)/,lookbehind:!0},string:{pattern:/("|')(?:\\[\s\S]|(?!\1)[^\\])*\1/,greedy:!0},keyword:/\b(?:abstract|any|assert|binary|bool|boolean|break|byte|case|catch|char|class|code|const|continue|data|default|do|double|else|enum|extends|final|finally|float|for|goto|hash|if|implements|import|inherits|instanceof|int|interface|long|my|native|new|nothing|null|object|our|own|private|reference|rethrow|return|short|soft(?:bool|date|float|int|list|number|string)|static|strictfp|string|sub|super|switch|synchronized|this|throw|throws|transient|try|void|volatile|while)\b/,boolean:/\b(?:false|true)\b/i,function:/\$?\b(?!\d)\w+(?=\()/,number:/\b(?:0b[01]+|0x(?:[\da-f]*\.)?[\da-fp\-]+|(?:\d+(?:\.\d+)?|\.\d+)(?:e\d+)?[df]|(?:\d+(?:\.\d+)?|\.\d+))\b/i,operator:{pattern:/(^|[^.])(?:\+[+=]?|-[-=]?|[!=](?:==?|~)?|>>?=?|<(?:=>?|<=?)?|&[&=]?|\|[|=]?|[*\/%^]=?|[~?])/,lookbehind:!0},variable:/\$(?!\d)\w+\b/})}return d3}var h3,Hq;function Dot(){if(Hq)return h3;Hq=1,h3=e,e.displayName="qsharp",e.aliases=["qs"];function e(t){(function(r){function n(b,y){return b.replace(/<<(\d+)>>/g,function(S,w){return"(?:"+y[+w]+")"})}function a(b,y,S){return RegExp(n(b,y),"")}function i(b,y){for(var S=0;S<y;S++)b=b.replace(/<<self>>/g,function(){return"(?:"+b+")"});return b.replace(/<<self>>/g,"[^\\s\\S]")}var o={type:"Adj BigInt Bool Ctl Double false Int One Pauli PauliI PauliX PauliY PauliZ Qubit Range Result String true Unit Zero",other:"Adjoint adjoint apply as auto body borrow borrowing Controlled controlled distribute elif else fail fixup for function if in internal intrinsic invert is let mutable namespace new newtype open operation repeat return self set until use using while within"};function s(b){return"\\b(?:"+b.trim().replace(/ /g,"|")+")\\b"}var l=RegExp(s(o.type+" "+o.other)),u=/\b[A-Za-z_]\w*\b/.source,d=n(/<<0>>(?:\s*\.\s*<<0>>)*/.source,[u]),p={keyword:l,punctuation:/[<>()?,.:[\]]/},f=/"(?:\\.|[^\\"])*"/.source;r.languages.qsharp=r.languages.extend("clike",{comment:/\/\/.*/,string:[{pattern:a(/(^|[^$\\])<<0>>/.source,[f]),lookbehind:!0,greedy:!0}],"class-name":[{pattern:a(/(\b(?:as|open)\s+)<<0>>(?=\s*(?:;|as\b))/.source,[d]),lookbehind:!0,inside:p},{pattern:a(/(\bnamespace\s+)<<0>>(?=\s*\{)/.source,[d]),lookbehind:!0,inside:p}],keyword:l,number:/(?:\b0(?:x[\da-f]+|b[01]+|o[0-7]+)|(?:\B\.\d+|\b\d+(?:\.\d*)?)(?:e[-+]?\d+)?)l?\b/i,operator:/\band=|\bor=|\band\b|\bnot\b|\bor\b|<[-=]|[-=]>|>>>=?|<<<=?|\^\^\^=?|\|\|\|=?|&&&=?|w\/=?|~~~|[*\/+\-^=!%]=?/,punctuation:/::|[{}[\];(),.:]/}),r.languages.insertBefore("qsharp","number",{range:{pattern:/\.\./,alias:"operator"}});var m=i(n(/\{(?:[^"{}]|<<0>>|<<self>>)*\}/.source,[f]),2);r.languages.insertBefore("qsharp","string",{"interpolation-string":{pattern:a(/\$"(?:\\.|<<0>>|[^\\"{])*"/.source,[m]),greedy:!0,inside:{interpolation:{pattern:a(/((?:^|[^\\])(?:\\\\)*)<<0>>/.source,[m]),lookbehind:!0,inside:{punctuation:/^\{|\}$/,expression:{pattern:/[\s\S]+/,alias:"language-qsharp",inside:r.languages.qsharp}}},string:/[\s\S]+/}}})})(t),t.languages.qs=t.languages.qsharp}return h3}var p3,Uq;function Not(){if(Uq)return p3;Uq=1,p3=e,e.displayName="r",e.aliases=[];function e(t){t.languages.r={comment:/#.*/,string:{pattern:/(['"])(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},"percent-operator":{pattern:/%[^%\s]*%/,alias:"operator"},boolean:/\b(?:FALSE|TRUE)\b/,ellipsis:/\.\.(?:\.|\d+)/,number:[/\b(?:Inf|NaN)\b/,/(?:\b0x[\dA-Fa-f]+(?:\.\d*)?|\b\d+(?:\.\d*)?|\B\.\d+)(?:[EePp][+-]?\d+)?[iL]?/],keyword:/\b(?:NA|NA_character_|NA_complex_|NA_integer_|NA_real_|NULL|break|else|for|function|if|in|next|repeat|while)\b/,operator:/->?>?|<(?:=|<?-)?|[>=!]=?|::?|&&?|\|\|?|[+*\/^$@~]/,punctuation:/[(){}\[\],;]/}}return p3}var f3,$q;function Lot(){if($q)return f3;$q=1;var e=yL();f3=t,t.displayName="racket",t.aliases=["rkt"];function t(r){r.register(e),r.languages.racket=r.languages.extend("scheme",{"lambda-parameter":{pattern:/([(\[]lambda\s+[(\[])[^()\[\]'\s]+/,lookbehind:!0}}),r.languages.insertBefore("racket","string",{lang:{pattern:/^#lang.+/m,greedy:!0,alias:"keyword"}}),r.languages.rkt=r.languages.racket}return f3}var m3,Gq;function Mot(){if(Gq)return m3;Gq=1,m3=e,e.displayName="reason",e.aliases=[];function e(t){t.languages.reason=t.languages.extend("clike",{string:{pattern:/"(?:\\(?:\r\n|[\s\S])|[^\\\r\n"])*"/,greedy:!0},"class-name":/\b[A-Z]\w*/,keyword:/\b(?:and|as|assert|begin|class|constraint|do|done|downto|else|end|exception|external|for|fun|function|functor|if|in|include|inherit|initializer|lazy|let|method|module|mutable|new|nonrec|object|of|open|or|private|rec|sig|struct|switch|then|to|try|type|val|virtual|when|while|with)\b/,operator:/\.{3}|:[:=]|\|>|->|=(?:==?|>)?|<=?|>=?|[|^?'#!~`]|[+\-*\/]\.?|\b(?:asr|land|lor|lsl|lsr|lxor|mod)\b/}),t.languages.insertBefore("reason","class-name",{char:{pattern:/'(?:\\x[\da-f]{2}|\\o[0-3][0-7][0-7]|\\\d{3}|\\.|[^'\\\r\n])'/,greedy:!0},constructor:/\b[A-Z]\w*\b(?!\s*\.)/,label:{pattern:/\b[a-z]\w*(?=::)/,alias:"symbol"}}),delete t.languages.reason.function}return m3}var g3,jq;function Iot(){if(jq)return g3;jq=1,g3=e,e.displayName="regex",e.aliases=[];function e(t){(function(r){var n={pattern:/\\[\\(){}[\]^$+*?|.]/,alias:"escape"},a=/\\(?:x[\da-fA-F]{2}|u[\da-fA-F]{4}|u\{[\da-fA-F]+\}|0[0-7]{0,2}|[123][0-7]{2}|c[a-zA-Z]|.)/,i={pattern:/\.|\\[wsd]|\\p\{[^{}]+\}/i,alias:"class-name"},o={pattern:/\\[wsd]|\\p\{[^{}]+\}/i,alias:"class-name"},s="(?:[^\\\\-]|"+a.source+")",l=RegExp(s+"-"+s),u={pattern:/(<|')[^<>']+(?=[>']$)/,lookbehind:!0,alias:"variable"};r.languages.regex={"char-class":{pattern:/((?:^|[^\\])(?:\\\\)*)\[(?:[^\\\]]|\\[\s\S])*\]/,lookbehind:!0,inside:{"char-class-negation":{pattern:/(^\[)\^/,lookbehind:!0,alias:"operator"},"char-class-punctuation":{pattern:/^\[|\]$/,alias:"punctuation"},range:{pattern:l,inside:{escape:a,"range-punctuation":{pattern:/-/,alias:"operator"}}},"special-escape":n,"char-set":o,escape:a}},"special-escape":n,"char-set":i,backreference:[{pattern:/\\(?![123][0-7]{2})[1-9]/,alias:"keyword"},{pattern:/\\k<[^<>']+>/,alias:"keyword",inside:{"group-name":u}}],anchor:{pattern:/[$^]|\\[ABbGZz]/,alias:"function"},escape:a,group:[{pattern:/\((?:\?(?:<[^<>']+>|'[^<>']+'|[>:]|<?[=!]|[idmnsuxU]+(?:-[idmnsuxU]+)?:?))?/,alias:"punctuation",inside:{"group-name":u}},{pattern:/\)/,alias:"punctuation"}],quantifier:{pattern:/(?:[+*?]|\{\d+(?:,\d*)?\})[?+]?/,alias:"number"},alternation:{pattern:/\|/,alias:"keyword"}}})(t)}return g3}var b3,qq;function Oot(){if(qq)return b3;qq=1,b3=e,e.displayName="rego",e.aliases=[];function e(t){t.languages.rego={comment:/#.*/,property:{pattern:/(^|[^\\.])(?:"(?:\\.|[^\\"\r\n])*"|`[^`]*`|\b[a-z_]\w*\b)(?=\s*:(?!=))/i,lookbehind:!0,greedy:!0},string:{pattern:/(^|[^\\])"(?:\\.|[^\\"\r\n])*"|`[^`]*`/,lookbehind:!0,greedy:!0},keyword:/\b(?:as|default|else|import|not|null|package|set(?=\s*\()|some|with)\b/,boolean:/\b(?:false|true)\b/,function:{pattern:/\b[a-z_]\w*\b(?:\s*\.\s*\b[a-z_]\w*\b)*(?=\s*\()/i,inside:{namespace:/\b\w+\b(?=\s*\.)/,punctuation:/\./}},number:/-?\b\d+(?:\.\d+)?(?:e[+-]?\d+)?\b/i,operator:/[-+*/%|&]|[<>:=]=?|!=|\b_\b/,punctuation:/[,;.\[\]{}()]/}}return b3}var y3,Wq;function Bot(){if(Wq)return y3;Wq=1,y3=e,e.displayName="renpy",e.aliases=["rpy"];function e(t){t.languages.renpy={comment:{pattern:/(^|[^\\])#.+/,lookbehind:!0},string:{pattern:/("""|''')[\s\S]+?\1|("|')(?:\\.|(?!\2)[^\\])*\2|(?:^#?(?:(?:[0-9a-fA-F]){3}|[0-9a-fA-F]{6})$)/m,greedy:!0},function:/\b[a-z_]\w*(?=\()/i,property:/\b(?:Update|UpdateVersion|action|activate_sound|adv_nvl_transition|after_load_transition|align|alpha|alt|anchor|antialias|area|auto|background|bar_invert|bar_resizing|bar_vertical|black_color|bold|bottom_bar|bottom_gutter|bottom_margin|bottom_padding|box_reverse|box_wrap|can_update|caret|child|color|crop|default_afm_enable|default_afm_time|default_fullscreen|default_text_cps|developer|directory_name|drag_handle|drag_joined|drag_name|drag_raise|draggable|dragged|drop_shadow|drop_shadow_color|droppable|dropped|easein|easeout|edgescroll|end_game_transition|end_splash_transition|enter_replay_transition|enter_sound|enter_transition|enter_yesno_transition|executable_name|exit_replay_transition|exit_sound|exit_transition|exit_yesno_transition|fadein|fadeout|first_indent|first_spacing|fit_first|focus|focus_mask|font|foreground|game_main_transition|get_installed_packages|google_play_key|google_play_salt|ground|has_music|has_sound|has_voice|height|help|hinting|hover|hover_background|hover_color|hover_sound|hovered|hyperlink_functions|idle|idle_color|image_style|include_update|insensitive|insensitive_background|insensitive_color|inside|intra_transition|italic|justify|kerning|keyboard_focus|language|layer_clipping|layers|layout|left_bar|left_gutter|left_margin|left_padding|length|line_leading|line_overlap_split|line_spacing|linear|main_game_transition|main_menu_music|maximum|min_width|minimum|minwidth|modal|mouse|mousewheel|name|narrator_menu|newline_indent|nvl_adv_transition|offset|order_reverse|outlines|overlay_functions|pos|position|prefix|radius|range|rest_indent|right_bar|right_gutter|right_margin|right_padding|rotate|rotate_pad|ruby_style|sample_sound|save_directory|say_attribute_transition|screen_height|screen_width|scrollbars|selected_hover|selected_hover_color|selected_idle|selected_idle_color|selected_insensitive|show_side_image|show_two_window|side_spacing|side_xpos|side_ypos|size|size_group|slow_cps|slow_cps_multiplier|spacing|strikethrough|subpixel|text_align|text_style|text_xpos|text_y_fudge|text_ypos|thumb|thumb_offset|thumb_shadow|thumbnail_height|thumbnail_width|time|top_bar|top_gutter|top_margin|top_padding|translations|underline|unscrollable|update|value|version|version_name|version_tuple|vertical|width|window_hide_transition|window_icon|window_left_padding|window_show_transition|window_title|windows_icon|xadjustment|xalign|xanchor|xanchoraround|xaround|xcenter|xfill|xinitial|xmargin|xmaximum|xminimum|xoffset|xofsset|xpadding|xpos|xsize|xzoom|yadjustment|yalign|yanchor|yanchoraround|yaround|ycenter|yfill|yinitial|ymargin|ymaximum|yminimum|yoffset|ypadding|ypos|ysize|ysizexysize|yzoom|zoom|zorder)\b/,tag:/\b(?:bar|block|button|buttoscreenn|drag|draggroup|fixed|frame|grid|[hv]box|hotbar|hotspot|image|imagebutton|imagemap|input|key|label|menu|mm_menu_frame|mousearea|nvl|parallel|screen|self|side|tag|text|textbutton|timer|vbar|viewport|window)\b|\$/,keyword:/\b(?:None|add|adjustment|alignaround|allow|angle|animation|around|as|assert|behind|box_layout|break|build|cache|call|center|changed|child_size|choice|circles|class|clear|clicked|clipping|clockwise|config|contains|continue|corner1|corner2|counterclockwise|def|default|define|del|delay|disabled|disabled_text|dissolve|elif|else|event|except|exclude|exec|expression|fade|finally|for|from|function|global|gm_root|has|hide|id|if|import|in|init|is|jump|knot|lambda|left|less_rounded|mm_root|movie|music|null|on|onlayer|pass|pause|persistent|play|print|python|queue|raise|random|renpy|repeat|return|right|rounded_window|scene|scope|set|show|slow|slow_abortable|slow_done|sound|stop|store|style|style_group|substitute|suffix|theme|transform|transform_anchor|transpose|try|ui|unhovered|updater|use|voice|while|widget|widget_hover|widget_selected|widget_text|yield)\b/,boolean:/\b(?:[Ff]alse|[Tt]rue)\b/,number:/(?:\b(?:0[bo])?(?:(?:\d|0x[\da-f])[\da-f]*(?:\.\d*)?)|\B\.\d+)(?:e[+-]?\d+)?j?/i,operator:/[-+%=]=?|!=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]|\b(?:and|at|not|or|with)\b/,punctuation:/[{}[\];(),.:]/},t.languages.rpy=t.languages.renpy}return y3}var v3,Vq;function Pot(){if(Vq)return v3;Vq=1,v3=e,e.displayName="rest",e.aliases=[];function e(t){t.languages.rest={table:[{pattern:/(^[\t ]*)(?:\+[=-]+)+\+(?:\r?\n|\r)(?:\1[+|].+[+|](?:\r?\n|\r))+\1(?:\+[=-]+)+\+/m,lookbehind:!0,inside:{punctuation:/\||(?:\+[=-]+)+\+/}},{pattern:/(^[\t ]*)=+ [ =]*=(?:(?:\r?\n|\r)\1.+)+(?:\r?\n|\r)\1=+ [ =]*=(?=(?:\r?\n|\r){2}|\s*$)/m,lookbehind:!0,inside:{punctuation:/[=-]+/}}],"substitution-def":{pattern:/(^[\t ]*\.\. )\|(?:[^|\s](?:[^|]*[^|\s])?)\| [^:]+::/m,lookbehind:!0,inside:{substitution:{pattern:/^\|(?:[^|\s]|[^|\s][^|]*[^|\s])\|/,alias:"attr-value",inside:{punctuation:/^\||\|$/}},directive:{pattern:/( )(?! )[^:]+::/,lookbehind:!0,alias:"function",inside:{punctuation:/::$/}}}},"link-target":[{pattern:/(^[\t ]*\.\. )\[[^\]]+\]/m,lookbehind:!0,alias:"string",inside:{punctuation:/^\[|\]$/}},{pattern:/(^[\t ]*\.\. )_(?:`[^`]+`|(?:[^:\\]|\\.)+):/m,lookbehind:!0,alias:"string",inside:{punctuation:/^_|:$/}}],directive:{pattern:/(^[\t ]*\.\. )[^:]+::/m,lookbehind:!0,alias:"function",inside:{punctuation:/::$/}},comment:{pattern:/(^[\t ]*\.\.)(?:(?: .+)?(?:(?:\r?\n|\r).+)+| .+)(?=(?:\r?\n|\r){2}|$)/m,lookbehind:!0},title:[{pattern:/^(([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\2+)(?:\r?\n|\r).+(?:\r?\n|\r)\1$/m,inside:{punctuation:/^[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]+|[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]+$/,important:/.+/}},{pattern:/(^|(?:\r?\n|\r){2}).+(?:\r?\n|\r)([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\2+(?=\r?\n|\r|$)/,lookbehind:!0,inside:{punctuation:/[!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]+$/,important:/.+/}}],hr:{pattern:/((?:\r?\n|\r){2})([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\2{3,}(?=(?:\r?\n|\r){2})/,lookbehind:!0,alias:"punctuation"},field:{pattern:/(^[\t ]*):[^:\r\n]+:(?= )/m,lookbehind:!0,alias:"attr-name"},"command-line-option":{pattern:/(^[\t ]*)(?:[+-][a-z\d]|(?:--|\/)[a-z\d-]+)(?:[ =](?:[a-z][\w-]*|<[^<>]+>))?(?:, (?:[+-][a-z\d]|(?:--|\/)[a-z\d-]+)(?:[ =](?:[a-z][\w-]*|<[^<>]+>))?)*(?=(?:\r?\n|\r)? {2,}\S)/im,lookbehind:!0,alias:"symbol"},"literal-block":{pattern:/::(?:\r?\n|\r){2}([ \t]+)(?![ \t]).+(?:(?:\r?\n|\r)\1.+)*/,inside:{"literal-block-punctuation":{pattern:/^::/,alias:"punctuation"}}},"quoted-literal-block":{pattern:/::(?:\r?\n|\r){2}([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~]).*(?:(?:\r?\n|\r)\1.*)*/,inside:{"literal-block-punctuation":{pattern:/^(?:::|([!"#$%&'()*+,\-.\/:;<=>?@\[\\\]^_`{|}~])\1*)/m,alias:"punctuation"}}},"list-bullet":{pattern:/(^[\t ]*)(?:[*+\-•‣⁃]|\(?(?:\d+|[a-z]|[ivxdclm]+)\)|(?:\d+|[a-z]|[ivxdclm]+)\.)(?= )/im,lookbehind:!0,alias:"punctuation"},"doctest-block":{pattern:/(^[\t ]*)>>> .+(?:(?:\r?\n|\r).+)*/m,lookbehind:!0,inside:{punctuation:/^>>>/}},inline:[{pattern:/(^|[\s\-:\/'"<(\[{])(?::[^:]+:`.*?`|`.*?`:[^:]+:|(\*\*?|``?|\|)(?!\s)(?:(?!\2).)*\S\2(?=[\s\-.,:;!?\\\/'")\]}]|$))/m,lookbehind:!0,inside:{bold:{pattern:/(^\*\*).+(?=\*\*$)/,lookbehind:!0},italic:{pattern:/(^\*).+(?=\*$)/,lookbehind:!0},"inline-literal":{pattern:/(^``).+(?=``$)/,lookbehind:!0,alias:"symbol"},role:{pattern:/^:[^:]+:|:[^:]+:$/,alias:"function",inside:{punctuation:/^:|:$/}},"interpreted-text":{pattern:/(^`).+(?=`$)/,lookbehind:!0,alias:"attr-value"},substitution:{pattern:/(^\|).+(?=\|$)/,lookbehind:!0,alias:"attr-value"},punctuation:/\*\*?|``?|\|/}}],link:[{pattern:/\[[^\[\]]+\]_(?=[\s\-.,:;!?\\\/'")\]}]|$)/,alias:"string",inside:{punctuation:/^\[|\]_$/}},{pattern:/(?:\b[a-z\d]+(?:[_.:+][a-z\d]+)*_?_|`[^`]+`_?_|_`[^`]+`)(?=[\s\-.,:;!?\\\/'")\]}]|$)/i,alias:"string",inside:{punctuation:/^_?`|`$|`?_?_$/}}],punctuation:{pattern:/(^[\t ]*)(?:\|(?= |$)|(?:---?|—|\.\.|__)(?= )|\.\.$)/m,lookbehind:!0}}}return v3}var S3,Kq;function zot(){if(Kq)return S3;Kq=1,S3=e,e.displayName="rip",e.aliases=[];function e(t){t.languages.rip={comment:{pattern:/#.*/,greedy:!0},char:{pattern:/\B`[^\s`'",.:;#\/\\()<>\[\]{}]\b/,greedy:!0},string:{pattern:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},regex:{pattern:/(^|[^/])\/(?!\/)(?:\[[^\n\r\]]*\]|\\.|[^/\\\r\n\[])+\/(?=\s*(?:$|[\r\n,.;})]))/,lookbehind:!0,greedy:!0},keyword:/(?:=>|->)|\b(?:case|catch|class|else|exit|finally|if|raise|return|switch|try)\b/,builtin:/@|\bSystem\b/,boolean:/\b(?:false|true)\b/,date:/\b\d{4}-\d{2}-\d{2}\b/,time:/\b\d{2}:\d{2}:\d{2}\b/,datetime:/\b\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\b/,symbol:/:[^\d\s`'",.:;#\/\\()<>\[\]{}][^\s`'",.:;#\/\\()<>\[\]{}]*/,number:/[+-]?\b(?:\d+\.\d+|\d+)\b/,punctuation:/(?:\.{2,3})|[`,.:;=\/\\()<>\[\]{}]/,reference:/[^\d\s`'",.:;#\/\\()<>\[\]{}][^\s`'",.:;#\/\\()<>\[\]{}]*/}}return S3}var E3,Yq;function Hot(){if(Yq)return E3;Yq=1,E3=e,e.displayName="roboconf",e.aliases=[];function e(t){t.languages.roboconf={comment:/#.*/,keyword:{pattern:/(^|\s)(?:(?:external|import)\b|(?:facet|instance of)(?=[ \t]+[\w-]+[ \t]*\{))/,lookbehind:!0},component:{pattern:/[\w-]+(?=[ \t]*\{)/,alias:"variable"},property:/[\w.-]+(?=[ \t]*:)/,value:{pattern:/(=[ \t]*(?![ \t]))[^,;]+/,lookbehind:!0,alias:"attr-value"},optional:{pattern:/\(optional\)/,alias:"builtin"},wildcard:{pattern:/(\.)\*/,lookbehind:!0,alias:"operator"},punctuation:/[{},.;:=]/}}return E3}var x3,Jq;function Uot(){if(Jq)return x3;Jq=1,x3=e,e.displayName="robotframework",e.aliases=[];function e(t){(function(r){var n={pattern:/(^[ \t]*| {2}|\t)#.*/m,lookbehind:!0,greedy:!0},a={pattern:/((?:^|[^\\])(?:\\{2})*)[$@&%]\{(?:[^{}\r\n]|\{[^{}\r\n]*\})*\}/,lookbehind:!0,inside:{punctuation:/^[$@&%]\{|\}$/}};function i(u,d){var p={};p["section-header"]={pattern:/^ ?\*{3}.+?\*{3}/,alias:"keyword"};for(var f in d)p[f]=d[f];return p.tag={pattern:/([\r\n](?: {2}|\t)[ \t]*)\[[-\w]+\]/,lookbehind:!0,inside:{punctuation:/\[|\]/}},p.variable=a,p.comment=n,{pattern:RegExp(/^ ?\*{3}[ \t]*<name>[ \t]*\*{3}(?:.|[\r\n](?!\*{3}))*/.source.replace(/<name>/g,function(){return u}),"im"),alias:"section",inside:p}}var o={pattern:/(\[Documentation\](?: {2}|\t)[ \t]*)(?![ \t]|#)(?:.|(?:\r\n?|\n)[ \t]*\.{3})+/,lookbehind:!0,alias:"string"},s={pattern:/([\r\n] ?)(?!#)(?:\S(?:[ \t]\S)*)+/,lookbehind:!0,alias:"function",inside:{variable:a}},l={pattern:/([\r\n](?: {2}|\t)[ \t]*)(?!\[|\.{3}|#)(?:\S(?:[ \t]\S)*)+/,lookbehind:!0,inside:{variable:a}};r.languages.robotframework={settings:i("Settings",{documentation:{pattern:/([\r\n] ?Documentation(?: {2}|\t)[ \t]*)(?![ \t]|#)(?:.|(?:\r\n?|\n)[ \t]*\.{3})+/,lookbehind:!0,alias:"string"},property:{pattern:/([\r\n] ?)(?!\.{3}|#)(?:\S(?:[ \t]\S)*)+/,lookbehind:!0}}),variables:i("Variables"),"test-cases":i("Test Cases",{"test-name":s,documentation:o,property:l}),keywords:i("Keywords",{"keyword-name":s,documentation:o,property:l}),tasks:i("Tasks",{"task-name":s,documentation:o,property:l}),comment:n},r.languages.robot=r.languages.robotframework})(t)}return x3}var w3,Xq;function $ot(){if(Xq)return w3;Xq=1,w3=e,e.displayName="rust",e.aliases=[];function e(t){(function(r){for(var n=/\/\*(?:[^*/]|\*(?!\/)|\/(?!\*)|<self>)*\*\//.source,a=0;a<2;a++)n=n.replace(/<self>/g,function(){return n});n=n.replace(/<self>/g,function(){return/[^\s\S]/.source}),r.languages.rust={comment:[{pattern:RegExp(/(^|[^\\])/.source+n),lookbehind:!0,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],string:{pattern:/b?"(?:\\[\s\S]|[^\\"])*"|b?r(#*)"(?:[^"]|"(?!\1))*"\1/,greedy:!0},char:{pattern:/b?'(?:\\(?:x[0-7][\da-fA-F]|u\{(?:[\da-fA-F]_*){1,6}\}|.)|[^\\\r\n\t'])'/,greedy:!0},attribute:{pattern:/#!?\[(?:[^\[\]"]|"(?:\\[\s\S]|[^\\"])*")*\]/,greedy:!0,alias:"attr-name",inside:{string:null}},"closure-params":{pattern:/([=(,:]\s*|\bmove\s*)\|[^|]*\||\|[^|]*\|(?=\s*(?:\{|->))/,lookbehind:!0,greedy:!0,inside:{"closure-punctuation":{pattern:/^\||\|$/,alias:"punctuation"},rest:null}},"lifetime-annotation":{pattern:/'\w+/,alias:"symbol"},"fragment-specifier":{pattern:/(\$\w+:)[a-z]+/,lookbehind:!0,alias:"punctuation"},variable:/\$\w+/,"function-definition":{pattern:/(\bfn\s+)\w+/,lookbehind:!0,alias:"function"},"type-definition":{pattern:/(\b(?:enum|struct|trait|type|union)\s+)\w+/,lookbehind:!0,alias:"class-name"},"module-declaration":[{pattern:/(\b(?:crate|mod)\s+)[a-z][a-z_\d]*/,lookbehind:!0,alias:"namespace"},{pattern:/(\b(?:crate|self|super)\s*)::\s*[a-z][a-z_\d]*\b(?:\s*::(?:\s*[a-z][a-z_\d]*\s*::)*)?/,lookbehind:!0,alias:"namespace",inside:{punctuation:/::/}}],keyword:[/\b(?:Self|abstract|as|async|await|become|box|break|const|continue|crate|do|dyn|else|enum|extern|final|fn|for|if|impl|in|let|loop|macro|match|mod|move|mut|override|priv|pub|ref|return|self|static|struct|super|trait|try|type|typeof|union|unsafe|unsized|use|virtual|where|while|yield)\b/,/\b(?:bool|char|f(?:32|64)|[ui](?:8|16|32|64|128|size)|str)\b/],function:/\b[a-z_]\w*(?=\s*(?:::\s*<|\())/,macro:{pattern:/\b\w+!/,alias:"property"},constant:/\b[A-Z_][A-Z_\d]+\b/,"class-name":/\b[A-Z]\w*\b/,namespace:{pattern:/(?:\b[a-z][a-z_\d]*\s*::\s*)*\b[a-z][a-z_\d]*\s*::(?!\s*<)/,inside:{punctuation:/::/}},number:/\b(?:0x[\dA-Fa-f](?:_?[\dA-Fa-f])*|0o[0-7](?:_?[0-7])*|0b[01](?:_?[01])*|(?:(?:\d(?:_?\d)*)?\.)?\d(?:_?\d)*(?:[Ee][+-]?\d+)?)(?:_?(?:f32|f64|[iu](?:8|16|32|64|size)?))?\b/,boolean:/\b(?:false|true)\b/,punctuation:/->|\.\.=|\.{1,3}|::|[{}[\];(),:]/,operator:/[-+*\/%!^]=?|=[=>]?|&[&=]?|\|[|=]?|<<?=?|>>?=?|[@?]/},r.languages.rust["closure-params"].inside.rest=r.languages.rust,r.languages.rust.attribute.inside.string=r.languages.rust.string})(t)}return w3}var k3,Zq;function Got(){if(Zq)return k3;Zq=1,k3=e,e.displayName="sas",e.aliases=[];function e(t){(function(r){var n=/(?:"(?:""|[^"])*"(?!")|'(?:''|[^'])*'(?!'))/.source,a=/\b(?:\d[\da-f]*x|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/i,i={pattern:RegExp(n+"[bx]"),alias:"number"},o={pattern:/&[a-z_]\w*/i},s={pattern:/((?:^|\s|=|\())%(?:ABORT|BY|CMS|COPY|DISPLAY|DO|ELSE|END|EVAL|GLOBAL|GO|GOTO|IF|INC|INCLUDE|INDEX|INPUT|KTRIM|LENGTH|LET|LIST|LOCAL|PUT|QKTRIM|QSCAN|QSUBSTR|QSYSFUNC|QUPCASE|RETURN|RUN|SCAN|SUBSTR|SUPERQ|SYMDEL|SYMEXIST|SYMGLOBL|SYMLOCAL|SYSCALL|SYSEVALF|SYSEXEC|SYSFUNC|SYSGET|SYSRPUT|THEN|TO|TSO|UNQUOTE|UNTIL|UPCASE|WHILE|WINDOW)\b/i,lookbehind:!0,alias:"keyword"},l={pattern:/(^|\s)(?:proc\s+\w+|data(?!=)|quit|run)\b/i,alias:"keyword",lookbehind:!0},u=[/\/\*[\s\S]*?\*\//,{pattern:/(^[ \t]*|;\s*)\*[^;]*;/m,lookbehind:!0}],d={pattern:RegExp(n),greedy:!0},p=/[$%@.(){}\[\];,\\]/,f={pattern:/%?\b\w+(?=\()/,alias:"keyword"},m={function:f,"arg-value":{pattern:/(=\s*)[A-Z\.]+/i,lookbehind:!0},operator:/=/,"macro-variable":o,arg:{pattern:/[A-Z]+/i,alias:"keyword"},number:a,"numeric-constant":i,punctuation:p,string:d},b={pattern:/\b(?:format|put)\b=?[\w'$.]+/i,inside:{keyword:/^(?:format|put)(?==)/i,equals:/=/,format:{pattern:/(?:\w|\$\d)+\.\d?/,alias:"number"}}},y={pattern:/\b(?:format|put)\s+[\w']+(?:\s+[$.\w]+)+(?=;)/i,inside:{keyword:/^(?:format|put)/i,format:{pattern:/[\w$]+\.\d?/,alias:"number"}}},S={pattern:/((?:^|\s)=?)(?:catname|checkpoint execute_always|dm|endsas|filename|footnote|%include|libname|%list|lock|missing|options|page|resetline|%run|sasfile|skip|sysecho|title\d?)\b/i,lookbehind:!0,alias:"keyword"},w={pattern:/(^|\s)(?:submit(?:\s+(?:load|norun|parseonly))?|endsubmit)\b/i,lookbehind:!0,alias:"keyword"},E=/aStore|accessControl|aggregation|audio|autotune|bayesianNetClassifier|bioMedImage|boolRule|builtins|cardinality|cdm|clustering|conditionalRandomFields|configuration|copula|countreg|dataDiscovery|dataPreprocess|dataSciencePilot|dataStep|decisionTree|deduplication|deepLearn|deepNeural|deepRnn|ds2|ecm|entityRes|espCluster|explainModel|factmac|fastKnn|fcmpact|fedSql|freqTab|gVarCluster|gam|gleam|graphSemiSupLearn|hiddenMarkovModel|hyperGroup|ica|image|iml|kernalPca|langModel|ldaTopic|loadStreams|mbc|mixed|mlTools|modelPublishing|network|neuralNet|nmf|nonParametricBayes|nonlinear|optNetwork|optimization|panel|pca|percentile|phreg|pls|qkb|qlim|quantreg|recommend|regression|reinforcementLearn|robustPca|ruleMining|sampling|sandwich|sccasl|search(?:Analytics)?|sentimentAnalysis|sequence|session(?:Prop)?|severity|simSystem|simple|smartData|sparkEmbeddedProcess|sparseML|spatialreg|spc|stabilityMonitoring|svDataDescription|svm|table|text(?:Filters|Frequency|Mining|Parse|Rule(?:Develop|Score)|Topic|Util)|timeData|transpose|tsInfo|tsReconcile|uniTimeSeries|varReduce/.source,k={pattern:RegExp(/(^|\s)(?:action\s+)?(?:<act>)\.[a-z]+\b[^;]+/.source.replace(/<act>/g,function(){return E}),"i"),lookbehind:!0,inside:{keyword:RegExp(/(?:<act>)\.[a-z]+\b/.source.replace(/<act>/g,function(){return E}),"i"),action:{pattern:/(?:action)/i,alias:"keyword"},comment:u,function:f,"arg-value":m["arg-value"],operator:m.operator,argument:m.arg,number:a,"numeric-constant":i,punctuation:p,string:d}},_={pattern:/((?:^|\s)=?)(?:after|analysis|and|array|barchart|barwidth|begingraph|by|call|cas|cbarline|cfill|class(?:lev)?|close|column|computed?|contains|continue|data(?==)|define|delete|describe|document|do\s+over|do|dol|drop|dul|else|end(?:comp|source)?|entryTitle|eval(?:uate)?|exec(?:ute)?|exit|file(?:name)?|fill(?:attrs)?|flist|fnc|function(?:list)?|global|goto|group(?:by)?|headline|headskip|histogram|if|infile|keep|keylabel|keyword|label|layout|leave|legendlabel|length|libname|loadactionset|merge|midpoints|_?null_|name|noobs|nowd|ods|options|or|otherwise|out(?:put)?|over(?:lay)?|plot|print|put|raise|ranexp|rannor|rbreak|retain|return|select|session|sessref|set|source|statgraph|sum|summarize|table|temp|terminate|then\s+do|then|title\d?|to|var|when|where|xaxisopts|y2axisopts|yaxisopts)\b/i,lookbehind:!0};r.languages.sas={datalines:{pattern:/^([ \t]*)(?:cards|(?:data)?lines);[\s\S]+?^[ \t]*;/im,lookbehind:!0,alias:"string",inside:{keyword:{pattern:/^(?:cards|(?:data)?lines)/i},punctuation:/;/}},"proc-sql":{pattern:/(^proc\s+(?:fed)?sql(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|data|quit|run);|(?![\s\S]))/im,lookbehind:!0,inside:{sql:{pattern:RegExp(/^[ \t]*(?:select|alter\s+table|(?:create|describe|drop)\s+(?:index|table(?:\s+constraints)?|view)|create\s+unique\s+index|insert\s+into|update)(?:<str>|[^;"'])+;/.source.replace(/<str>/g,function(){return n}),"im"),alias:"language-sql",inside:r.languages.sql},"global-statements":S,"sql-statements":{pattern:/(^|\s)(?:disconnect\s+from|begin|commit|exec(?:ute)?|reset|rollback|validate)\b/i,lookbehind:!0,alias:"keyword"},number:a,"numeric-constant":i,punctuation:p,string:d}},"proc-groovy":{pattern:/(^proc\s+groovy(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|data|quit|run);|(?![\s\S]))/im,lookbehind:!0,inside:{comment:u,groovy:{pattern:RegExp(/(^[ \t]*submit(?:\s+(?:load|norun|parseonly))?)(?:<str>|[^"'])+?(?=endsubmit;)/.source.replace(/<str>/g,function(){return n}),"im"),lookbehind:!0,alias:"language-groovy",inside:r.languages.groovy},keyword:_,"submit-statement":w,"global-statements":S,number:a,"numeric-constant":i,punctuation:p,string:d}},"proc-lua":{pattern:/(^proc\s+lua(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|data|quit|run);|(?![\s\S]))/im,lookbehind:!0,inside:{comment:u,lua:{pattern:RegExp(/(^[ \t]*submit(?:\s+(?:load|norun|parseonly))?)(?:<str>|[^"'])+?(?=endsubmit;)/.source.replace(/<str>/g,function(){return n}),"im"),lookbehind:!0,alias:"language-lua",inside:r.languages.lua},keyword:_,"submit-statement":w,"global-statements":S,number:a,"numeric-constant":i,punctuation:p,string:d}},"proc-cas":{pattern:/(^proc\s+cas(?:\s+[\w|=]+)?;)[\s\S]+?(?=^(?:proc\s+\w+|quit|data);|(?![\s\S]))/im,lookbehind:!0,inside:{comment:u,"statement-var":{pattern:/((?:^|\s)=?)saveresult\s[^;]+/im,lookbehind:!0,inside:{statement:{pattern:/^saveresult\s+\S+/i,inside:{keyword:/^(?:saveresult)/i}},rest:m}},"cas-actions":k,statement:{pattern:/((?:^|\s)=?)(?:default|(?:un)?set|on|output|upload)[^;]+/im,lookbehind:!0,inside:m},step:l,keyword:_,function:f,format:b,altformat:y,"global-statements":S,number:a,"numeric-constant":i,punctuation:p,string:d}},"proc-args":{pattern:RegExp(/(^proc\s+\w+\s+)(?!\s)(?:[^;"']|<str>)+;/.source.replace(/<str>/g,function(){return n}),"im"),lookbehind:!0,inside:m},"macro-keyword":s,"macro-variable":o,"macro-string-functions":{pattern:/((?:^|\s|=))%(?:BQUOTE|NRBQUOTE|NRQUOTE|NRSTR|QUOTE|STR)\(.*?(?:[^%]\))/i,lookbehind:!0,inside:{function:{pattern:/%(?:BQUOTE|NRBQUOTE|NRQUOTE|NRSTR|QUOTE|STR)/i,alias:"keyword"},"macro-keyword":s,"macro-variable":o,"escaped-char":{pattern:/%['"()<>=¬^~;,#]/},punctuation:p}},"macro-declaration":{pattern:/^%macro[^;]+(?=;)/im,inside:{keyword:/%macro/i}},"macro-end":{pattern:/^%mend[^;]+(?=;)/im,inside:{keyword:/%mend/i}},macro:{pattern:/%_\w+(?=\()/,alias:"keyword"},input:{pattern:/\binput\s[-\w\s/*.$&]+;/i,inside:{input:{alias:"keyword",pattern:/^input/i},comment:u,number:a,"numeric-constant":i}},"options-args":{pattern:/(^options)[-'"|/\\<>*+=:()\w\s]*(?=;)/im,lookbehind:!0,inside:m},"cas-actions":k,comment:u,function:f,format:b,altformat:y,"numeric-constant":i,datetime:{pattern:RegExp(n+"(?:dt?|t)"),alias:"number"},string:d,step:l,keyword:_,"operator-keyword":{pattern:/\b(?:eq|ge|gt|in|le|lt|ne|not)\b/i,alias:"operator"},number:a,operator:/\*\*?|\|\|?|!!?|¦¦?|<[>=]?|>[<=]?|[-+\/=&]|[~¬^]=?/,punctuation:p}})(t)}return k3}var C3,Qq;function jot(){if(Qq)return C3;Qq=1,C3=e,e.displayName="sass",e.aliases=[];function e(t){(function(r){r.languages.sass=r.languages.extend("css",{comment:{pattern:/^([ \t]*)\/[\/*].*(?:(?:\r?\n|\r)\1[ \t].+)*/m,lookbehind:!0,greedy:!0}}),r.languages.insertBefore("sass","atrule",{"atrule-line":{pattern:/^(?:[ \t]*)[@+=].+/m,greedy:!0,inside:{atrule:/(?:@[\w-]+|[+=])/}}}),delete r.languages.sass.atrule;var n=/\$[-\w]+|#\{\$[-\w]+\}/,a=[/[+*\/%]|[=!]=|<=?|>=?|\b(?:and|not|or)\b/,{pattern:/(\s)-(?=\s)/,lookbehind:!0}];r.languages.insertBefore("sass","property",{"variable-line":{pattern:/^[ \t]*\$.+/m,greedy:!0,inside:{punctuation:/:/,variable:n,operator:a}},"property-line":{pattern:/^[ \t]*(?:[^:\s]+ *:.*|:[^:\s].*)/m,greedy:!0,inside:{property:[/[^:\s]+(?=\s*:)/,{pattern:/(:)[^:\s]+/,lookbehind:!0}],punctuation:/:/,variable:n,operator:a,important:r.languages.sass.important}}}),delete r.languages.sass.property,delete r.languages.sass.important,r.languages.insertBefore("sass","punctuation",{selector:{pattern:/^([ \t]*)\S(?:,[^,\r\n]+|[^,\r\n]*)(?:,[^,\r\n]+)*(?:,(?:\r?\n|\r)\1[ \t]+\S(?:,[^,\r\n]+|[^,\r\n]*)(?:,[^,\r\n]+)*)*/m,lookbehind:!0,greedy:!0}})})(t)}return C3}var T3,eW;function qot(){if(eW)return T3;eW=1;var e=mL();T3=t,t.displayName="scala",t.aliases=[];function t(r){r.register(e),r.languages.scala=r.languages.extend("java",{"triple-quoted-string":{pattern:/"""[\s\S]*?"""/,greedy:!0,alias:"string"},string:{pattern:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,greedy:!0},keyword:/<-|=>|\b(?:abstract|case|catch|class|def|do|else|extends|final|finally|for|forSome|if|implicit|import|lazy|match|new|null|object|override|package|private|protected|return|sealed|self|super|this|throw|trait|try|type|val|var|while|with|yield)\b/,number:/\b0x(?:[\da-f]*\.)?[\da-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e\d+)?[dfl]?/i,builtin:/\b(?:Any|AnyRef|AnyVal|Boolean|Byte|Char|Double|Float|Int|Long|Nothing|Short|String|Unit)\b/,symbol:/'[^\d\s\\]\w*/}),r.languages.insertBefore("scala","triple-quoted-string",{"string-interpolation":{pattern:/\b[a-z]\w*(?:"""(?:[^$]|\$(?:[^{]|\{(?:[^{}]|\{[^{}]*\})*\}))*?"""|"(?:[^$"\r\n]|\$(?:[^{]|\{(?:[^{}]|\{[^{}]*\})*\}))*")/i,greedy:!0,inside:{id:{pattern:/^\w+/,greedy:!0,alias:"function"},escape:{pattern:/\\\$"|\$[$"]/,greedy:!0,alias:"symbol"},interpolation:{pattern:/\$(?:\w+|\{(?:[^{}]|\{[^{}]*\})*\})/,greedy:!0,inside:{punctuation:/^\$\{?|\}$/,expression:{pattern:/[\s\S]+/,inside:r.languages.scala}}},string:/[\s\S]+/}}}),delete r.languages.scala["class-name"],delete r.languages.scala.function}return T3}var A3,tW;function Wot(){if(tW)return A3;tW=1,A3=e,e.displayName="scss",e.aliases=[];function e(t){t.languages.scss=t.languages.extend("css",{comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,lookbehind:!0},atrule:{pattern:/@[\w-](?:\([^()]+\)|[^()\s]|\s+(?!\s))*?(?=\s+[{;])/,inside:{rule:/@[\w-]+/}},url:/(?:[-a-z]+-)?url(?=\()/i,selector:{pattern:/(?=\S)[^@;{}()]?(?:[^@;{}()\s]|\s+(?!\s)|#\{\$[-\w]+\})+(?=\s*\{(?:\}|\s|[^}][^:{}]*[:{][^}]))/,inside:{parent:{pattern:/&/,alias:"important"},placeholder:/%[-\w]+/,variable:/\$[-\w]+|#\{\$[-\w]+\}/}},property:{pattern:/(?:[-\w]|\$[-\w]|#\{\$[-\w]+\})+(?=\s*:)/,inside:{variable:/\$[-\w]+|#\{\$[-\w]+\}/}}}),t.languages.insertBefore("scss","atrule",{keyword:[/@(?:content|debug|each|else(?: if)?|extend|for|forward|function|if|import|include|mixin|return|use|warn|while)\b/i,{pattern:/( )(?:from|through)(?= )/,lookbehind:!0}]}),t.languages.insertBefore("scss","important",{variable:/\$[-\w]+|#\{\$[-\w]+\}/}),t.languages.insertBefore("scss","function",{"module-modifier":{pattern:/\b(?:as|hide|show|with)\b/i,alias:"keyword"},placeholder:{pattern:/%[-\w]+/,alias:"selector"},statement:{pattern:/\B!(?:default|optional)\b/i,alias:"keyword"},boolean:/\b(?:false|true)\b/,null:{pattern:/\bnull\b/,alias:"keyword"},operator:{pattern:/(\s)(?:[-+*\/%]|[=!]=|<=?|>=?|and|not|or)(?=\s)/,lookbehind:!0}}),t.languages.scss.atrule.inside.rest=t.languages.scss}return A3}var F3,rW;function Vot(){if(rW)return F3;rW=1;var e=Mce();F3=t,t.displayName="shellSession",t.aliases=[];function t(r){r.register(e),(function(n){var a=[/"(?:\\[\s\S]|\$\([^)]+\)|\$(?!\()|`[^`]+`|[^"\\`$])*"/.source,/'[^']*'/.source,/\$'(?:[^'\\]|\\[\s\S])*'/.source,/<<-?\s*(["']?)(\w+)\1\s[\s\S]*?[\r\n]\2/.source].join("|");n.languages["shell-session"]={command:{pattern:RegExp(/^/.source+"(?:"+(/[^\s@:$#%*!/\\]+@[^\r\n@:$#%*!/\\]+(?::[^\0-\x1F$#%*?"<>:;|]+)?/.source+"|"+/[/~.][^\0-\x1F$#%*?"<>@:;|]*/.source)+")?"+/[$#%](?=\s)/.source+/(?:[^\\\r\n \t'"<$]|[ \t](?:(?!#)|#.*$)|\\(?:[^\r]|\r\n?)|\$(?!')|<(?!<)|<<str>>)+/.source.replace(/<<str>>/g,function(){return a}),"m"),greedy:!0,inside:{info:{pattern:/^[^#$%]+/,alias:"punctuation",inside:{user:/^[^\s@:$#%*!/\\]+@[^\r\n@:$#%*!/\\]+/,punctuation:/:/,path:/[\s\S]+/}},bash:{pattern:/(^[$#%]\s*)\S[\s\S]*/,lookbehind:!0,alias:"language-bash",inside:n.languages.bash},"shell-symbol":{pattern:/^[$#%]/,alias:"important"}}},output:/.(?:.*(?:[\r\n]|.$))*/},n.languages["sh-session"]=n.languages.shellsession=n.languages["shell-session"]})(r)}return F3}var _3,nW;function Kot(){if(nW)return _3;nW=1,_3=e,e.displayName="smali",e.aliases=[];function e(t){t.languages.smali={comment:/#.*/,string:{pattern:/"(?:[^\r\n\\"]|\\.)*"|'(?:[^\r\n\\']|\\(?:.|u[\da-fA-F]{4}))'/,greedy:!0},"class-name":{pattern:/(^|[^L])L(?:(?:\w+|`[^`\r\n]*`)\/)*(?:[\w$]+|`[^`\r\n]*`)(?=\s*;)/,lookbehind:!0,inside:{"class-name":{pattern:/(^L|\/)(?:[\w$]+|`[^`\r\n]*`)$/,lookbehind:!0},namespace:{pattern:/^(L)(?:(?:\w+|`[^`\r\n]*`)\/)+/,lookbehind:!0,inside:{punctuation:/\//}},builtin:/^L/}},builtin:[{pattern:/([();\[])[BCDFIJSVZ]+/,lookbehind:!0},{pattern:/([\w$>]:)[BCDFIJSVZ]/,lookbehind:!0}],keyword:[{pattern:/(\.end\s+)[\w-]+/,lookbehind:!0},{pattern:/(^|[^\w.-])\.(?!\d)[\w-]+/,lookbehind:!0},{pattern:/(^|[^\w.-])(?:abstract|annotation|bridge|constructor|enum|final|interface|private|protected|public|runtime|static|synthetic|system|transient)(?![\w.-])/,lookbehind:!0}],function:{pattern:/(^|[^\w.-])(?:\w+|<[\w$-]+>)(?=\()/,lookbehind:!0},field:{pattern:/[\w$]+(?=:)/,alias:"variable"},register:{pattern:/(^|[^\w.-])[vp]\d(?![\w.-])/,lookbehind:!0,alias:"variable"},boolean:{pattern:/(^|[^\w.-])(?:false|true)(?![\w.-])/,lookbehind:!0},number:{pattern:/(^|[^/\w.-])-?(?:NAN|INFINITY|0x(?:[\dA-F]+(?:\.[\dA-F]*)?|\.[\dA-F]+)(?:p[+-]?[\dA-F]+)?|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?)[dflst]?(?![\w.-])/i,lookbehind:!0},label:{pattern:/(:)\w+/,lookbehind:!0,alias:"property"},operator:/->|\.\.|[\[=]/,punctuation:/[{}(),;:]/}}return _3}var R3,aW;function Yot(){if(aW)return R3;aW=1,R3=e,e.displayName="smalltalk",e.aliases=[];function e(t){t.languages.smalltalk={comment:{pattern:/"(?:""|[^"])*"/,greedy:!0},char:{pattern:/\$./,greedy:!0},string:{pattern:/'(?:''|[^'])*'/,greedy:!0},symbol:/#[\da-z]+|#(?:-|([+\/\\*~<>=@%|&?!])\1?)|#(?=\()/i,"block-arguments":{pattern:/(\[\s*):[^\[|]*\|/,lookbehind:!0,inside:{variable:/:[\da-z]+/i,punctuation:/\|/}},"temporary-variables":{pattern:/\|[^|]+\|/,inside:{variable:/[\da-z]+/i,punctuation:/\|/}},keyword:/\b(?:new|nil|self|super)\b/,boolean:/\b(?:false|true)\b/,number:[/\d+r-?[\dA-Z]+(?:\.[\dA-Z]+)?(?:e-?\d+)?/,/\b\d+(?:\.\d+)?(?:e-?\d+)?/],operator:/[<=]=?|:=|~[~=]|\/\/?|\\\\|>[>=]?|[!^+\-*&|,@]/,punctuation:/[.;:?\[\](){}]/}}return R3}var D3,iW;function Jot(){if(iW)return D3;iW=1;var e=Si();D3=t,t.displayName="smarty",t.aliases=[];function t(r){r.register(e),(function(n){n.languages.smarty={comment:{pattern:/^\{\*[\s\S]*?\*\}/,greedy:!0},"embedded-php":{pattern:/^\{php\}[\s\S]*?\{\/php\}/,greedy:!0,inside:{smarty:{pattern:/^\{php\}|\{\/php\}$/,inside:null},php:{pattern:/[\s\S]+/,alias:"language-php",inside:n.languages.php}}},string:[{pattern:/"(?:\\.|[^"\\\r\n])*"/,greedy:!0,inside:{interpolation:{pattern:/\{[^{}]*\}|`[^`]*`/,inside:{"interpolation-punctuation":{pattern:/^[{`]|[`}]$/,alias:"punctuation"},expression:{pattern:/[\s\S]+/,inside:null}}},variable:/\$\w+/}},{pattern:/'(?:\\.|[^'\\\r\n])*'/,greedy:!0}],keyword:{pattern:/(^\{\/?)[a-z_]\w*\b(?!\()/i,lookbehind:!0,greedy:!0},delimiter:{pattern:/^\{\/?|\}$/,greedy:!0,alias:"punctuation"},number:/\b0x[\dA-Fa-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][-+]?\d+)?/,variable:[/\$(?!\d)\w+/,/#(?!\d)\w+#/,{pattern:/(\.|->|\w\s*=)(?!\d)\w+\b(?!\()/,lookbehind:!0},{pattern:/(\[)(?!\d)\w+(?=\])/,lookbehind:!0}],function:{pattern:/(\|\s*)@?[a-z_]\w*|\b[a-z_]\w*(?=\()/i,lookbehind:!0},"attr-name":/\b[a-z_]\w*(?=\s*=)/i,boolean:/\b(?:false|no|off|on|true|yes)\b/,punctuation:/[\[\](){}.,:`]|->/,operator:[/[+\-*\/%]|==?=?|[!<>]=?|&&|\|\|?/,/\bis\s+(?:not\s+)?(?:div|even|odd)(?:\s+by)?\b/,/\b(?:and|eq|gt?e|gt|lt?e|lt|mod|neq?|not|or)\b/]},n.languages.smarty["embedded-php"].inside.smarty.inside=n.languages.smarty,n.languages.smarty.string[0].inside.interpolation.inside.expression.inside=n.languages.smarty;var a=/"(?:\\.|[^"\\\r\n])*"|'(?:\\.|[^'\\\r\n])*'/,i=RegExp(/\{\*[\s\S]*?\*\}/.source+"|"+/\{php\}[\s\S]*?\{\/php\}/.source+"|"+/\{(?:[^{}"']|<str>|\{(?:[^{}"']|<str>|\{(?:[^{}"']|<str>)*\})*\})*\}/.source.replace(/<str>/g,function(){return a.source}),"g");n.hooks.add("before-tokenize",function(o){var s="{literal}",l="{/literal}",u=!1;n.languages["markup-templating"].buildPlaceholders(o,"smarty",i,function(d){return d===l&&(u=!1),u?!1:(d===s&&(u=!0),!0)})}),n.hooks.add("after-tokenize",function(o){n.languages["markup-templating"].tokenizePlaceholders(o,"smarty")})})(r)}return D3}var N3,oW;function Xot(){if(oW)return N3;oW=1,N3=e,e.displayName="sml",e.aliases=["smlnj"];function e(t){(function(r){var n=/\b(?:abstype|and|andalso|as|case|datatype|do|else|end|eqtype|exception|fn|fun|functor|handle|if|in|include|infix|infixr|let|local|nonfix|of|op|open|orelse|raise|rec|sharing|sig|signature|struct|structure|then|type|val|where|while|with|withtype)\b/i;r.languages.sml={comment:/\(\*(?:[^*(]|\*(?!\))|\((?!\*)|\(\*(?:[^*(]|\*(?!\))|\((?!\*))*\*\))*\*\)/,string:{pattern:/#?"(?:[^"\\]|\\.)*"/,greedy:!0},"class-name":[{pattern:RegExp(/((?:^|[^:]):\s*)<TERMINAL>(?:\s*(?:(?:\*|->)\s*<TERMINAL>|,\s*<TERMINAL>(?:(?=<NOT-LAST>)|(?!<NOT-LAST>)\s+<LONG-ID>)))*/.source.replace(/<NOT-LAST>/g,function(){return/\s*(?:[*,]|->)/.source}).replace(/<TERMINAL>/g,function(){return/(?:'[\w']*|<LONG-ID>|\((?:[^()]|\([^()]*\))*\)|\{(?:[^{}]|\{[^{}]*\})*\})(?:\s+<LONG-ID>)*/.source}).replace(/<LONG-ID>/g,function(){return/(?!<KEYWORD>)[a-z\d_][\w'.]*/.source}).replace(/<KEYWORD>/g,function(){return n.source}),"i"),lookbehind:!0,greedy:!0,inside:null},{pattern:/((?:^|[^\w'])(?:datatype|exception|functor|signature|structure|type)\s+)[a-z_][\w'.]*/i,lookbehind:!0}],function:{pattern:/((?:^|[^\w'])fun\s+)[a-z_][\w'.]*/i,lookbehind:!0},keyword:n,variable:{pattern:/(^|[^\w'])'[\w']*/,lookbehind:!0},number:/~?\b(?:\d+(?:\.\d+)?(?:e~?\d+)?|0x[\da-f]+)\b/i,word:{pattern:/\b0w(?:\d+|x[\da-f]+)\b/i,alias:"constant"},boolean:/\b(?:false|true)\b/i,operator:/\.\.\.|:[>=:]|=>?|->|[<>]=?|[!+\-*/^#|@~]/,punctuation:/[(){}\[\].:,;]/},r.languages.sml["class-name"][0].inside=r.languages.sml,r.languages.smlnj=r.languages.sml})(t)}return N3}var L3,sW;function Zot(){if(sW)return L3;sW=1,L3=e,e.displayName="solidity",e.aliases=["sol"];function e(t){t.languages.solidity=t.languages.extend("clike",{"class-name":{pattern:/(\b(?:contract|enum|interface|library|new|struct|using)\s+)(?!\d)[\w$]+/,lookbehind:!0},keyword:/\b(?:_|anonymous|as|assembly|assert|break|calldata|case|constant|constructor|continue|contract|default|delete|do|else|emit|enum|event|external|for|from|function|if|import|indexed|inherited|interface|internal|is|let|library|mapping|memory|modifier|new|payable|pragma|private|public|pure|require|returns?|revert|selfdestruct|solidity|storage|struct|suicide|switch|this|throw|using|var|view|while)\b/,operator:/=>|->|:=|=:|\*\*|\+\+|--|\|\||&&|<<=?|>>=?|[-+*/%^&|<>!=]=?|[~?]/}),t.languages.insertBefore("solidity","keyword",{builtin:/\b(?:address|bool|byte|u?int(?:8|16|24|32|40|48|56|64|72|80|88|96|104|112|120|128|136|144|152|160|168|176|184|192|200|208|216|224|232|240|248|256)?|string|bytes(?:[1-9]|[12]\d|3[0-2])?)\b/}),t.languages.insertBefore("solidity","number",{version:{pattern:/([<>]=?|\^)\d+\.\d+\.\d+\b/,lookbehind:!0,alias:"number"}}),t.languages.sol=t.languages.solidity}return L3}var M3,lW;function Qot(){if(lW)return M3;lW=1,M3=e,e.displayName="solutionFile",e.aliases=[];function e(t){(function(r){var n={pattern:/\{[\da-f]{8}-[\da-f]{4}-[\da-f]{4}-[\da-f]{4}-[\da-f]{12}\}/i,alias:"constant",inside:{punctuation:/[{}]/}};r.languages["solution-file"]={comment:{pattern:/#.*/,greedy:!0},string:{pattern:/"[^"\r\n]*"|'[^'\r\n]*'/,greedy:!0,inside:{guid:n}},object:{pattern:/^([ \t]*)(?:([A-Z]\w*)\b(?=.*(?:\r\n?|\n)(?:\1[ \t].*(?:\r\n?|\n))*\1End\2(?=[ \t]*$))|End[A-Z]\w*(?=[ \t]*$))/m,lookbehind:!0,greedy:!0,alias:"keyword"},property:{pattern:/^([ \t]*)(?!\s)[^\r\n"#=()]*[^\s"#=()](?=\s*=)/m,lookbehind:!0,inside:{guid:n}},guid:n,number:/\b\d+(?:\.\d+)*\b/,boolean:/\b(?:FALSE|TRUE)\b/,operator:/=/,punctuation:/[(),]/},r.languages.sln=r.languages["solution-file"]})(t)}return M3}var I3,uW;function est(){if(uW)return I3;uW=1;var e=Si();I3=t,t.displayName="soy",t.aliases=[];function t(r){r.register(e),(function(n){var a=/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,i=/\b\d+(?:\.\d+)?(?:[eE][+-]?\d+)?\b|\b0x[\dA-F]+\b/;n.languages.soy={comment:[/\/\*[\s\S]*?\*\//,{pattern:/(\s)\/\/.*/,lookbehind:!0,greedy:!0}],"command-arg":{pattern:/(\{+\/?\s*(?:alias|call|delcall|delpackage|deltemplate|namespace|template)\s+)\.?[\w.]+/,lookbehind:!0,alias:"string",inside:{punctuation:/\./}},parameter:{pattern:/(\{+\/?\s*@?param\??\s+)\.?[\w.]+/,lookbehind:!0,alias:"variable"},keyword:[{pattern:/(\{+\/?[^\S\r\n]*)(?:\\[nrt]|alias|call|case|css|default|delcall|delpackage|deltemplate|else(?:if)?|fallbackmsg|for(?:each)?|if(?:empty)?|lb|let|literal|msg|namespace|nil|@?param\??|rb|sp|switch|template|xid)/,lookbehind:!0},/\b(?:any|as|attributes|bool|css|float|html|in|int|js|list|map|null|number|string|uri)\b/],delimiter:{pattern:/^\{+\/?|\/?\}+$/,alias:"punctuation"},property:/\w+(?==)/,variable:{pattern:/\$[^\W\d]\w*(?:\??(?:\.\w+|\[[^\]]+\]))*/,inside:{string:{pattern:a,greedy:!0},number:i,punctuation:/[\[\].?]/}},string:{pattern:a,greedy:!0},function:[/\w+(?=\()/,{pattern:/(\|[^\S\r\n]*)\w+/,lookbehind:!0}],boolean:/\b(?:false|true)\b/,number:i,operator:/\?:?|<=?|>=?|==?|!=|[+*/%-]|\b(?:and|not|or)\b/,punctuation:/[{}()\[\]|.,:]/},n.hooks.add("before-tokenize",function(o){var s=/\{\{.+?\}\}|\{.+?\}|\s\/\/.*|\/\*[\s\S]*?\*\//g,l="{literal}",u="{/literal}",d=!1;n.languages["markup-templating"].buildPlaceholders(o,"soy",s,function(p){return p===u&&(d=!1),d?!1:(p===l&&(d=!0),!0)})}),n.hooks.add("after-tokenize",function(o){n.languages["markup-templating"].tokenizePlaceholders(o,"soy")})})(r)}return I3}var O3,cW;function Pce(){if(cW)return O3;cW=1,O3=e,e.displayName="turtle",e.aliases=[];function e(t){t.languages.turtle={comment:{pattern:/#.*/,greedy:!0},"multiline-string":{pattern:/"""(?:(?:""?)?(?:[^"\\]|\\.))*"""|'''(?:(?:''?)?(?:[^'\\]|\\.))*'''/,greedy:!0,alias:"string",inside:{comment:/#.*/}},string:{pattern:/"(?:[^\\"\r\n]|\\.)*"|'(?:[^\\'\r\n]|\\.)*'/,greedy:!0},url:{pattern:/<(?:[^\x00-\x20<>"{}|^`\\]|\\(?:u[\da-fA-F]{4}|U[\da-fA-F]{8}))*>/,greedy:!0,inside:{punctuation:/[<>]/}},function:{pattern:/(?:(?![-.\d\xB7])[-.\w\xB7\xC0-\uFFFD]+)?:(?:(?![-.])(?:[-.:\w\xC0-\uFFFD]|%[\da-f]{2}|\\.)+)?/i,inside:{"local-name":{pattern:/([^:]*:)[\s\S]+/,lookbehind:!0},prefix:{pattern:/[\s\S]+/,inside:{punctuation:/:/}}}},number:/[+-]?\b\d+(?:\.\d*)?(?:e[+-]?\d+)?/i,punctuation:/[{}.,;()[\]]|\^\^/,boolean:/\b(?:false|true)\b/,keyword:[/(?:\ba|@prefix|@base)\b|=/,/\b(?:base|graph|prefix)\b/i],tag:{pattern:/@[a-z]+(?:-[a-z\d]+)*/i,inside:{punctuation:/@/}}},t.languages.trig=t.languages.turtle}return O3}var B3,dW;function tst(){if(dW)return B3;dW=1;var e=Pce();B3=t,t.displayName="sparql",t.aliases=["rq"];function t(r){r.register(e),r.languages.sparql=r.languages.extend("turtle",{boolean:/\b(?:false|true)\b/i,variable:{pattern:/[?$]\w+/,greedy:!0}}),r.languages.insertBefore("sparql","punctuation",{keyword:[/\b(?:A|ADD|ALL|AS|ASC|ASK|BNODE|BY|CLEAR|CONSTRUCT|COPY|CREATE|DATA|DEFAULT|DELETE|DESC|DESCRIBE|DISTINCT|DROP|EXISTS|FILTER|FROM|GROUP|HAVING|INSERT|INTO|LIMIT|LOAD|MINUS|MOVE|NAMED|NOT|NOW|OFFSET|OPTIONAL|ORDER|RAND|REDUCED|SELECT|SEPARATOR|SERVICE|SILENT|STRUUID|UNION|USING|UUID|VALUES|WHERE)\b/i,/\b(?:ABS|AVG|BIND|BOUND|CEIL|COALESCE|CONCAT|CONTAINS|COUNT|DATATYPE|DAY|ENCODE_FOR_URI|FLOOR|GROUP_CONCAT|HOURS|IF|IRI|isBLANK|isIRI|isLITERAL|isNUMERIC|isURI|LANG|LANGMATCHES|LCASE|MAX|MD5|MIN|MINUTES|MONTH|REGEX|REPLACE|ROUND|sameTerm|SAMPLE|SECONDS|SHA1|SHA256|SHA384|SHA512|STR|STRAFTER|STRBEFORE|STRDT|STRENDS|STRLANG|STRLEN|STRSTARTS|SUBSTR|SUM|TIMEZONE|TZ|UCASE|URI|YEAR)\b(?=\s*\()/i,/\b(?:BASE|GRAPH|PREFIX)\b/i]}),r.languages.rq=r.languages.sparql}return B3}var P3,hW;function rst(){if(hW)return P3;hW=1,P3=e,e.displayName="splunkSpl",e.aliases=[];function e(t){t.languages["splunk-spl"]={comment:/`comment\("(?:\\.|[^\\"])*"\)`/,string:{pattern:/"(?:\\.|[^\\"])*"/,greedy:!0},keyword:/\b(?:abstract|accum|addcoltotals|addinfo|addtotals|analyzefields|anomalies|anomalousvalue|anomalydetection|append|appendcols|appendcsv|appendlookup|appendpipe|arules|associate|audit|autoregress|bin|bucket|bucketdir|chart|cluster|cofilter|collect|concurrency|contingency|convert|correlate|datamodel|dbinspect|dedup|delete|delta|diff|erex|eval|eventcount|eventstats|extract|fieldformat|fields|fieldsummary|filldown|fillnull|findtypes|folderize|foreach|format|from|gauge|gentimes|geom|geomfilter|geostats|head|highlight|history|iconify|input|inputcsv|inputlookup|iplocation|join|kmeans|kv|kvform|loadjob|localize|localop|lookup|makecontinuous|makemv|makeresults|map|mcollect|metadata|metasearch|meventcollect|mstats|multikv|multisearch|mvcombine|mvexpand|nomv|outlier|outputcsv|outputlookup|outputtext|overlap|pivot|predict|rangemap|rare|regex|relevancy|reltime|rename|replace|rest|return|reverse|rex|rtorder|run|savedsearch|script|scrub|search|searchtxn|selfjoin|sendemail|set|setfields|sichart|sirare|sistats|sitimechart|sitop|sort|spath|stats|strcat|streamstats|table|tags|tail|timechart|timewrap|top|transaction|transpose|trendline|tscollect|tstats|typeahead|typelearner|typer|union|uniq|untable|where|x11|xmlkv|xmlunescape|xpath|xyseries)\b/i,"operator-word":{pattern:/\b(?:and|as|by|not|or|xor)\b/i,alias:"operator"},function:/\b\w+(?=\s*\()/,property:/\b\w+(?=\s*=(?!=))/,date:{pattern:/\b\d{1,2}\/\d{1,2}\/\d{1,4}(?:(?::\d{1,2}){3})?\b/,alias:"number"},number:/\b\d+(?:\.\d+)?\b/,boolean:/\b(?:f|false|t|true)\b/i,operator:/[<>=]=?|[-+*/%|]/,punctuation:/[()[\],]/}}return P3}var z3,pW;function nst(){if(pW)return z3;pW=1,z3=e,e.displayName="sqf",e.aliases=[];function e(t){t.languages.sqf=t.languages.extend("clike",{string:{pattern:/"(?:(?:"")?[^"])*"(?!")|'(?:[^'])*'/,greedy:!0},keyword:/\b(?:breakOut|breakTo|call|case|catch|default|do|echo|else|execFSM|execVM|exitWith|for|forEach|forEachMember|forEachMemberAgent|forEachMemberTeam|from|goto|if|nil|preprocessFile|preprocessFileLineNumbers|private|scopeName|spawn|step|switch|then|throw|to|try|while|with)\b/i,boolean:/\b(?:false|true)\b/i,function:/\b(?:abs|accTime|acos|action|actionIDs|actionKeys|actionKeysImages|actionKeysNames|actionKeysNamesArray|actionName|actionParams|activateAddons|activatedAddons|activateKey|add3DENConnection|add3DENEventHandler|add3DENLayer|addAction|addBackpack|addBackpackCargo|addBackpackCargoGlobal|addBackpackGlobal|addCamShake|addCuratorAddons|addCuratorCameraArea|addCuratorEditableObjects|addCuratorEditingArea|addCuratorPoints|addEditorObject|addEventHandler|addForce|addForceGeneratorRTD|addGoggles|addGroupIcon|addHandgunItem|addHeadgear|addItem|addItemCargo|addItemCargoGlobal|addItemPool|addItemToBackpack|addItemToUniform|addItemToVest|addLiveStats|addMagazine|addMagazineAmmoCargo|addMagazineCargo|addMagazineCargoGlobal|addMagazineGlobal|addMagazinePool|addMagazines|addMagazineTurret|addMenu|addMenuItem|addMissionEventHandler|addMPEventHandler|addMusicEventHandler|addOwnedMine|addPlayerScores|addPrimaryWeaponItem|addPublicVariableEventHandler|addRating|addResources|addScore|addScoreSide|addSecondaryWeaponItem|addSwitchableUnit|addTeamMember|addToRemainsCollector|addTorque|addUniform|addVehicle|addVest|addWaypoint|addWeapon|addWeaponCargo|addWeaponCargoGlobal|addWeaponGlobal|addWeaponItem|addWeaponPool|addWeaponTurret|admin|agent|agents|AGLToASL|aimedAtTarget|aimPos|airDensityCurveRTD|airDensityRTD|airplaneThrottle|airportSide|AISFinishHeal|alive|all3DENEntities|allAirports|allControls|allCurators|allCutLayers|allDead|allDeadMen|allDisplays|allGroups|allMapMarkers|allMines|allMissionObjects|allow3DMode|allowCrewInImmobile|allowCuratorLogicIgnoreAreas|allowDamage|allowDammage|allowFileOperations|allowFleeing|allowGetIn|allowSprint|allPlayers|allSimpleObjects|allSites|allTurrets|allUnits|allUnitsUAV|allVariables|ammo|ammoOnPylon|animate|animateBay|animateDoor|animatePylon|animateSource|animationNames|animationPhase|animationSourcePhase|animationState|append|apply|armoryPoints|arrayIntersect|asin|ASLToAGL|ASLToATL|assert|assignAsCargo|assignAsCargoIndex|assignAsCommander|assignAsDriver|assignAsGunner|assignAsTurret|assignCurator|assignedCargo|assignedCommander|assignedDriver|assignedGunner|assignedItems|assignedTarget|assignedTeam|assignedVehicle|assignedVehicleRole|assignItem|assignTeam|assignToAirport|atan|atan2|atg|ATLToASL|attachedObject|attachedObjects|attachedTo|attachObject|attachTo|attackEnabled|backpack|backpackCargo|backpackContainer|backpackItems|backpackMagazines|backpackSpaceFor|behaviour|benchmark|binocular|blufor|boundingBox|boundingBoxReal|boundingCenter|briefingName|buildingExit|buildingPos|buldozer_EnableRoadDiag|buldozer_IsEnabledRoadDiag|buldozer_LoadNewRoads|buldozer_reloadOperMap|buttonAction|buttonSetAction|cadetMode|callExtension|camCommand|camCommit|camCommitPrepared|camCommitted|camConstuctionSetParams|camCreate|camDestroy|cameraEffect|cameraEffectEnableHUD|cameraInterest|cameraOn|cameraView|campaignConfigFile|camPreload|camPreloaded|camPrepareBank|camPrepareDir|camPrepareDive|camPrepareFocus|camPrepareFov|camPrepareFovRange|camPreparePos|camPrepareRelPos|camPrepareTarget|camSetBank|camSetDir|camSetDive|camSetFocus|camSetFov|camSetFovRange|camSetPos|camSetRelPos|camSetTarget|camTarget|camUseNVG|canAdd|canAddItemToBackpack|canAddItemToUniform|canAddItemToVest|cancelSimpleTaskDestination|canFire|canMove|canSlingLoad|canStand|canSuspend|canTriggerDynamicSimulation|canUnloadInCombat|canVehicleCargo|captive|captiveNum|cbChecked|cbSetChecked|ceil|channelEnabled|cheatsEnabled|checkAIFeature|checkVisibility|civilian|className|clear3DENAttribute|clear3DENInventory|clearAllItemsFromBackpack|clearBackpackCargo|clearBackpackCargoGlobal|clearForcesRTD|clearGroupIcons|clearItemCargo|clearItemCargoGlobal|clearItemPool|clearMagazineCargo|clearMagazineCargoGlobal|clearMagazinePool|clearOverlay|clearRadio|clearVehicleInit|clearWeaponCargo|clearWeaponCargoGlobal|clearWeaponPool|clientOwner|closeDialog|closeDisplay|closeOverlay|collapseObjectTree|collect3DENHistory|collectiveRTD|combatMode|commandArtilleryFire|commandChat|commander|commandFire|commandFollow|commandFSM|commandGetOut|commandingMenu|commandMove|commandRadio|commandStop|commandSuppressiveFire|commandTarget|commandWatch|comment|commitOverlay|compile|compileFinal|completedFSM|composeText|configClasses|configFile|configHierarchy|configName|configNull|configProperties|configSourceAddonList|configSourceMod|configSourceModList|confirmSensorTarget|connectTerminalToUAV|controlNull|controlsGroupCtrl|copyFromClipboard|copyToClipboard|copyWaypoints|cos|count|countEnemy|countFriendly|countSide|countType|countUnknown|create3DENComposition|create3DENEntity|createAgent|createCenter|createDialog|createDiaryLink|createDiaryRecord|createDiarySubject|createDisplay|createGearDialog|createGroup|createGuardedPoint|createLocation|createMarker|createMarkerLocal|createMenu|createMine|createMissionDisplay|createMPCampaignDisplay|createSimpleObject|createSimpleTask|createSite|createSoundSource|createTask|createTeam|createTrigger|createUnit|createVehicle|createVehicleCrew|createVehicleLocal|crew|ctAddHeader|ctAddRow|ctClear|ctCurSel|ctData|ctFindHeaderRows|ctFindRowHeader|ctHeaderControls|ctHeaderCount|ctRemoveHeaders|ctRemoveRows|ctrlActivate|ctrlAddEventHandler|ctrlAngle|ctrlAutoScrollDelay|ctrlAutoScrollRewind|ctrlAutoScrollSpeed|ctrlChecked|ctrlClassName|ctrlCommit|ctrlCommitted|ctrlCreate|ctrlDelete|ctrlEnable|ctrlEnabled|ctrlFade|ctrlHTMLLoaded|ctrlIDC|ctrlIDD|ctrlMapAnimAdd|ctrlMapAnimClear|ctrlMapAnimCommit|ctrlMapAnimDone|ctrlMapCursor|ctrlMapMouseOver|ctrlMapScale|ctrlMapScreenToWorld|ctrlMapWorldToScreen|ctrlModel|ctrlModelDirAndUp|ctrlModelScale|ctrlParent|ctrlParentControlsGroup|ctrlPosition|ctrlRemoveAllEventHandlers|ctrlRemoveEventHandler|ctrlScale|ctrlSetActiveColor|ctrlSetAngle|ctrlSetAutoScrollDelay|ctrlSetAutoScrollRewind|ctrlSetAutoScrollSpeed|ctrlSetBackgroundColor|ctrlSetChecked|ctrlSetDisabledColor|ctrlSetEventHandler|ctrlSetFade|ctrlSetFocus|ctrlSetFont|ctrlSetFontH1|ctrlSetFontH1B|ctrlSetFontH2|ctrlSetFontH2B|ctrlSetFontH3|ctrlSetFontH3B|ctrlSetFontH4|ctrlSetFontH4B|ctrlSetFontH5|ctrlSetFontH5B|ctrlSetFontH6|ctrlSetFontH6B|ctrlSetFontHeight|ctrlSetFontHeightH1|ctrlSetFontHeightH2|ctrlSetFontHeightH3|ctrlSetFontHeightH4|ctrlSetFontHeightH5|ctrlSetFontHeightH6|ctrlSetFontHeightSecondary|ctrlSetFontP|ctrlSetFontPB|ctrlSetFontSecondary|ctrlSetForegroundColor|ctrlSetModel|ctrlSetModelDirAndUp|ctrlSetModelScale|ctrlSetPixelPrecision|ctrlSetPosition|ctrlSetScale|ctrlSetStructuredText|ctrlSetText|ctrlSetTextColor|ctrlSetTextColorSecondary|ctrlSetTextSecondary|ctrlSetTooltip|ctrlSetTooltipColorBox|ctrlSetTooltipColorShade|ctrlSetTooltipColorText|ctrlShow|ctrlShown|ctrlText|ctrlTextHeight|ctrlTextSecondary|ctrlTextWidth|ctrlType|ctrlVisible|ctRowControls|ctRowCount|ctSetCurSel|ctSetData|ctSetHeaderTemplate|ctSetRowTemplate|ctSetValue|ctValue|curatorAddons|curatorCamera|curatorCameraArea|curatorCameraAreaCeiling|curatorCoef|curatorEditableObjects|curatorEditingArea|curatorEditingAreaType|curatorMouseOver|curatorPoints|curatorRegisteredObjects|curatorSelected|curatorWaypointCost|current3DENOperation|currentChannel|currentCommand|currentMagazine|currentMagazineDetail|currentMagazineDetailTurret|currentMagazineTurret|currentMuzzle|currentNamespace|currentTask|currentTasks|currentThrowable|currentVisionMode|currentWaypoint|currentWeapon|currentWeaponMode|currentWeaponTurret|currentZeroing|cursorObject|cursorTarget|customChat|customRadio|cutFadeOut|cutObj|cutRsc|cutText|damage|date|dateToNumber|daytime|deActivateKey|debriefingText|debugFSM|debugLog|deg|delete3DENEntities|deleteAt|deleteCenter|deleteCollection|deleteEditorObject|deleteGroup|deleteGroupWhenEmpty|deleteIdentity|deleteLocation|deleteMarker|deleteMarkerLocal|deleteRange|deleteResources|deleteSite|deleteStatus|deleteTeam|deleteVehicle|deleteVehicleCrew|deleteWaypoint|detach|detectedMines|diag_activeMissionFSMs|diag_activeScripts|diag_activeSQFScripts|diag_activeSQSScripts|diag_captureFrame|diag_captureFrameToFile|diag_captureSlowFrame|diag_codePerformance|diag_drawMode|diag_dynamicSimulationEnd|diag_enable|diag_enabled|diag_fps|diag_fpsMin|diag_frameNo|diag_lightNewLoad|diag_list|diag_log|diag_logSlowFrame|diag_mergeConfigFile|diag_recordTurretLimits|diag_setLightNew|diag_tickTime|diag_toggle|dialog|diarySubjectExists|didJIP|didJIPOwner|difficulty|difficultyEnabled|difficultyEnabledRTD|difficultyOption|direction|directSay|disableAI|disableCollisionWith|disableConversation|disableDebriefingStats|disableMapIndicators|disableNVGEquipment|disableRemoteSensors|disableSerialization|disableTIEquipment|disableUAVConnectability|disableUserInput|displayAddEventHandler|displayCtrl|displayNull|displayParent|displayRemoveAllEventHandlers|displayRemoveEventHandler|displaySetEventHandler|dissolveTeam|distance|distance2D|distanceSqr|distributionRegion|do3DENAction|doArtilleryFire|doFire|doFollow|doFSM|doGetOut|doMove|doorPhase|doStop|doSuppressiveFire|doTarget|doWatch|drawArrow|drawEllipse|drawIcon|drawIcon3D|drawLine|drawLine3D|drawLink|drawLocation|drawPolygon|drawRectangle|drawTriangle|driver|drop|dynamicSimulationDistance|dynamicSimulationDistanceCoef|dynamicSimulationEnabled|dynamicSimulationSystemEnabled|east|edit3DENMissionAttributes|editObject|editorSetEventHandler|effectiveCommander|emptyPositions|enableAI|enableAIFeature|enableAimPrecision|enableAttack|enableAudioFeature|enableAutoStartUpRTD|enableAutoTrimRTD|enableCamShake|enableCaustics|enableChannel|enableCollisionWith|enableCopilot|enableDebriefingStats|enableDiagLegend|enableDynamicSimulation|enableDynamicSimulationSystem|enableEndDialog|enableEngineArtillery|enableEnvironment|enableFatigue|enableGunLights|enableInfoPanelComponent|enableIRLasers|enableMimics|enablePersonTurret|enableRadio|enableReload|enableRopeAttach|enableSatNormalOnDetail|enableSaving|enableSentences|enableSimulation|enableSimulationGlobal|enableStamina|enableStressDamage|enableTeamSwitch|enableTraffic|enableUAVConnectability|enableUAVWaypoints|enableVehicleCargo|enableVehicleSensor|enableWeaponDisassembly|endl|endLoadingScreen|endMission|engineOn|enginesIsOnRTD|enginesPowerRTD|enginesRpmRTD|enginesTorqueRTD|entities|environmentEnabled|estimatedEndServerTime|estimatedTimeLeft|evalObjectArgument|everyBackpack|everyContainer|exec|execEditorScript|exp|expectedDestination|exportJIPMessages|eyeDirection|eyePos|face|faction|fadeMusic|fadeRadio|fadeSound|fadeSpeech|failMission|fillWeaponsFromPool|find|findCover|findDisplay|findEditorObject|findEmptyPosition|findEmptyPositionReady|findIf|findNearestEnemy|finishMissionInit|finite|fire|fireAtTarget|firstBackpack|flag|flagAnimationPhase|flagOwner|flagSide|flagTexture|fleeing|floor|flyInHeight|flyInHeightASL|fog|fogForecast|fogParams|forceAddUniform|forceAtPositionRTD|forcedMap|forceEnd|forceFlagTexture|forceFollowRoad|forceGeneratorRTD|forceMap|forceRespawn|forceSpeed|forceWalk|forceWeaponFire|forceWeatherChange|forgetTarget|format|formation|formationDirection|formationLeader|formationMembers|formationPosition|formationTask|formatText|formLeader|freeLook|fromEditor|fuel|fullCrew|gearIDCAmmoCount|gearSlotAmmoCount|gearSlotData|get3DENActionState|get3DENAttribute|get3DENCamera|get3DENConnections|get3DENEntity|get3DENEntityID|get3DENGrid|get3DENIconsVisible|get3DENLayerEntities|get3DENLinesVisible|get3DENMissionAttribute|get3DENMouseOver|get3DENSelected|getAimingCoef|getAllEnvSoundControllers|getAllHitPointsDamage|getAllOwnedMines|getAllSoundControllers|getAmmoCargo|getAnimAimPrecision|getAnimSpeedCoef|getArray|getArtilleryAmmo|getArtilleryComputerSettings|getArtilleryETA|getAssignedCuratorLogic|getAssignedCuratorUnit|getBackpackCargo|getBleedingRemaining|getBurningValue|getCameraViewDirection|getCargoIndex|getCenterOfMass|getClientState|getClientStateNumber|getCompatiblePylonMagazines|getConnectedUAV|getContainerMaxLoad|getCursorObjectParams|getCustomAimCoef|getDammage|getDescription|getDir|getDirVisual|getDLCAssetsUsage|getDLCAssetsUsageByName|getDLCs|getDLCUsageTime|getEditorCamera|getEditorMode|getEditorObjectScope|getElevationOffset|getEngineTargetRpmRTD|getEnvSoundController|getFatigue|getFieldManualStartPage|getForcedFlagTexture|getFriend|getFSMVariable|getFuelCargo|getGroupIcon|getGroupIconParams|getGroupIcons|getHideFrom|getHit|getHitIndex|getHitPointDamage|getItemCargo|getMagazineCargo|getMarkerColor|getMarkerPos|getMarkerSize|getMarkerType|getMass|getMissionConfig|getMissionConfigValue|getMissionDLCs|getMissionLayerEntities|getMissionLayers|getModelInfo|getMousePosition|getMusicPlayedTime|getNumber|getObjectArgument|getObjectChildren|getObjectDLC|getObjectMaterials|getObjectProxy|getObjectTextures|getObjectType|getObjectViewDistance|getOxygenRemaining|getPersonUsedDLCs|getPilotCameraDirection|getPilotCameraPosition|getPilotCameraRotation|getPilotCameraTarget|getPlateNumber|getPlayerChannel|getPlayerScores|getPlayerUID|getPlayerUIDOld|getPos|getPosASL|getPosASLVisual|getPosASLW|getPosATL|getPosATLVisual|getPosVisual|getPosWorld|getPylonMagazines|getRelDir|getRelPos|getRemoteSensorsDisabled|getRepairCargo|getResolution|getRotorBrakeRTD|getShadowDistance|getShotParents|getSlingLoad|getSoundController|getSoundControllerResult|getSpeed|getStamina|getStatValue|getSuppression|getTerrainGrid|getTerrainHeightASL|getText|getTotalDLCUsageTime|getTrimOffsetRTD|getUnitLoadout|getUnitTrait|getUserMFDText|getUserMFDValue|getVariable|getVehicleCargo|getWeaponCargo|getWeaponSway|getWingsOrientationRTD|getWingsPositionRTD|getWPPos|glanceAt|globalChat|globalRadio|goggles|group|groupChat|groupFromNetId|groupIconSelectable|groupIconsVisible|groupId|groupOwner|groupRadio|groupSelectedUnits|groupSelectUnit|grpNull|gunner|gusts|halt|handgunItems|handgunMagazine|handgunWeapon|handsHit|hasInterface|hasPilotCamera|hasWeapon|hcAllGroups|hcGroupParams|hcLeader|hcRemoveAllGroups|hcRemoveGroup|hcSelected|hcSelectGroup|hcSetGroup|hcShowBar|hcShownBar|headgear|hideBody|hideObject|hideObjectGlobal|hideSelection|hint|hintC|hintCadet|hintSilent|hmd|hostMission|htmlLoad|HUDMovementLevels|humidity|image|importAllGroups|importance|in|inArea|inAreaArray|incapacitatedState|independent|inflame|inflamed|infoPanel|infoPanelComponentEnabled|infoPanelComponents|infoPanels|inGameUISetEventHandler|inheritsFrom|initAmbientLife|inPolygon|inputAction|inRangeOfArtillery|insertEditorObject|intersect|is3DEN|is3DENMultiplayer|isAbleToBreathe|isAgent|isAimPrecisionEnabled|isArray|isAutoHoverOn|isAutonomous|isAutoStartUpEnabledRTD|isAutotest|isAutoTrimOnRTD|isBleeding|isBurning|isClass|isCollisionLightOn|isCopilotEnabled|isDamageAllowed|isDedicated|isDLCAvailable|isEngineOn|isEqualTo|isEqualType|isEqualTypeAll|isEqualTypeAny|isEqualTypeArray|isEqualTypeParams|isFilePatchingEnabled|isFlashlightOn|isFlatEmpty|isForcedWalk|isFormationLeader|isGroupDeletedWhenEmpty|isHidden|isInRemainsCollector|isInstructorFigureEnabled|isIRLaserOn|isKeyActive|isKindOf|isLaserOn|isLightOn|isLocalized|isManualFire|isMarkedForCollection|isMultiplayer|isMultiplayerSolo|isNil|isNull|isNumber|isObjectHidden|isObjectRTD|isOnRoad|isPipEnabled|isPlayer|isRealTime|isRemoteExecuted|isRemoteExecutedJIP|isServer|isShowing3DIcons|isSimpleObject|isSprintAllowed|isStaminaEnabled|isSteamMission|isStreamFriendlyUIEnabled|isStressDamageEnabled|isText|isTouchingGround|isTurnedOut|isTutHintsEnabled|isUAVConnectable|isUAVConnected|isUIContext|isUniformAllowed|isVehicleCargo|isVehicleRadarOn|isVehicleSensorEnabled|isWalking|isWeaponDeployed|isWeaponRested|itemCargo|items|itemsWithMagazines|join|joinAs|joinAsSilent|joinSilent|joinString|kbAddDatabase|kbAddDatabaseTargets|kbAddTopic|kbHasTopic|kbReact|kbRemoveTopic|kbTell|kbWasSaid|keyImage|keyName|knowsAbout|land|landAt|landResult|language|laserTarget|lbAdd|lbClear|lbColor|lbColorRight|lbCurSel|lbData|lbDelete|lbIsSelected|lbPicture|lbPictureRight|lbSelection|lbSetColor|lbSetColorRight|lbSetCurSel|lbSetData|lbSetPicture|lbSetPictureColor|lbSetPictureColorDisabled|lbSetPictureColorSelected|lbSetPictureRight|lbSetPictureRightColor|lbSetPictureRightColorDisabled|lbSetPictureRightColorSelected|lbSetSelectColor|lbSetSelectColorRight|lbSetSelected|lbSetText|lbSetTextRight|lbSetTooltip|lbSetValue|lbSize|lbSort|lbSortByValue|lbText|lbTextRight|lbValue|leader|leaderboardDeInit|leaderboardGetRows|leaderboardInit|leaderboardRequestRowsFriends|leaderboardRequestRowsGlobal|leaderboardRequestRowsGlobalAroundUser|leaderboardsRequestUploadScore|leaderboardsRequestUploadScoreKeepBest|leaderboardState|leaveVehicle|libraryCredits|libraryDisclaimers|lifeState|lightAttachObject|lightDetachObject|lightIsOn|lightnings|limitSpeed|linearConversion|lineBreak|lineIntersects|lineIntersectsObjs|lineIntersectsSurfaces|lineIntersectsWith|linkItem|list|listObjects|listRemoteTargets|listVehicleSensors|ln|lnbAddArray|lnbAddColumn|lnbAddRow|lnbClear|lnbColor|lnbColorRight|lnbCurSelRow|lnbData|lnbDeleteColumn|lnbDeleteRow|lnbGetColumnsPosition|lnbPicture|lnbPictureRight|lnbSetColor|lnbSetColorRight|lnbSetColumnsPos|lnbSetCurSelRow|lnbSetData|lnbSetPicture|lnbSetPictureColor|lnbSetPictureColorRight|lnbSetPictureColorSelected|lnbSetPictureColorSelectedRight|lnbSetPictureRight|lnbSetText|lnbSetTextRight|lnbSetValue|lnbSize|lnbSort|lnbSortByValue|lnbText|lnbTextRight|lnbValue|load|loadAbs|loadBackpack|loadFile|loadGame|loadIdentity|loadMagazine|loadOverlay|loadStatus|loadUniform|loadVest|local|localize|locationNull|locationPosition|lock|lockCameraTo|lockCargo|lockDriver|locked|lockedCargo|lockedDriver|lockedTurret|lockIdentity|lockTurret|lockWP|log|logEntities|logNetwork|logNetworkTerminate|lookAt|lookAtPos|magazineCargo|magazines|magazinesAllTurrets|magazinesAmmo|magazinesAmmoCargo|magazinesAmmoFull|magazinesDetail|magazinesDetailBackpack|magazinesDetailUniform|magazinesDetailVest|magazinesTurret|magazineTurretAmmo|mapAnimAdd|mapAnimClear|mapAnimCommit|mapAnimDone|mapCenterOnCamera|mapGridPosition|markAsFinishedOnSteam|markerAlpha|markerBrush|markerColor|markerDir|markerPos|markerShape|markerSize|markerText|markerType|max|members|menuAction|menuAdd|menuChecked|menuClear|menuCollapse|menuData|menuDelete|menuEnable|menuEnabled|menuExpand|menuHover|menuPicture|menuSetAction|menuSetCheck|menuSetData|menuSetPicture|menuSetValue|menuShortcut|menuShortcutText|menuSize|menuSort|menuText|menuURL|menuValue|min|mineActive|mineDetectedBy|missionConfigFile|missionDifficulty|missionName|missionNamespace|missionStart|missionVersion|modelToWorld|modelToWorldVisual|modelToWorldVisualWorld|modelToWorldWorld|modParams|moonIntensity|moonPhase|morale|move|move3DENCamera|moveInAny|moveInCargo|moveInCommander|moveInDriver|moveInGunner|moveInTurret|moveObjectToEnd|moveOut|moveTime|moveTo|moveToCompleted|moveToFailed|musicVolume|name|nameSound|nearEntities|nearestBuilding|nearestLocation|nearestLocations|nearestLocationWithDubbing|nearestObject|nearestObjects|nearestTerrainObjects|nearObjects|nearObjectsReady|nearRoads|nearSupplies|nearTargets|needReload|netId|netObjNull|newOverlay|nextMenuItemIndex|nextWeatherChange|nMenuItems|numberOfEnginesRTD|numberToDate|objectCurators|objectFromNetId|objectParent|objNull|objStatus|onBriefingGear|onBriefingGroup|onBriefingNotes|onBriefingPlan|onBriefingTeamSwitch|onCommandModeChanged|onDoubleClick|onEachFrame|onGroupIconClick|onGroupIconOverEnter|onGroupIconOverLeave|onHCGroupSelectionChanged|onMapSingleClick|onPlayerConnected|onPlayerDisconnected|onPreloadFinished|onPreloadStarted|onShowNewObject|onTeamSwitch|openCuratorInterface|openDLCPage|openDSInterface|openMap|openSteamApp|openYoutubeVideo|opfor|orderGetIn|overcast|overcastForecast|owner|param|params|parseNumber|parseSimpleArray|parseText|parsingNamespace|particlesQuality|pi|pickWeaponPool|pitch|pixelGrid|pixelGridBase|pixelGridNoUIScale|pixelH|pixelW|playableSlotsNumber|playableUnits|playAction|playActionNow|player|playerRespawnTime|playerSide|playersNumber|playGesture|playMission|playMove|playMoveNow|playMusic|playScriptedMission|playSound|playSound3D|position|positionCameraToWorld|posScreenToWorld|posWorldToScreen|ppEffectAdjust|ppEffectCommit|ppEffectCommitted|ppEffectCreate|ppEffectDestroy|ppEffectEnable|ppEffectEnabled|ppEffectForceInNVG|precision|preloadCamera|preloadObject|preloadSound|preloadTitleObj|preloadTitleRsc|primaryWeapon|primaryWeaponItems|primaryWeaponMagazine|priority|processDiaryLink|processInitCommands|productVersion|profileName|profileNamespace|profileNameSteam|progressLoadingScreen|progressPosition|progressSetPosition|publicVariable|publicVariableClient|publicVariableServer|pushBack|pushBackUnique|putWeaponPool|queryItemsPool|queryMagazinePool|queryWeaponPool|rad|radioChannelAdd|radioChannelCreate|radioChannelRemove|radioChannelSetCallSign|radioChannelSetLabel|radioVolume|rain|rainbow|random|rank|rankId|rating|rectangular|registeredTasks|registerTask|reload|reloadEnabled|remoteControl|remoteExec|remoteExecCall|remoteExecutedOwner|remove3DENConnection|remove3DENEventHandler|remove3DENLayer|removeAction|removeAll3DENEventHandlers|removeAllActions|removeAllAssignedItems|removeAllContainers|removeAllCuratorAddons|removeAllCuratorCameraAreas|removeAllCuratorEditingAreas|removeAllEventHandlers|removeAllHandgunItems|removeAllItems|removeAllItemsWithMagazines|removeAllMissionEventHandlers|removeAllMPEventHandlers|removeAllMusicEventHandlers|removeAllOwnedMines|removeAllPrimaryWeaponItems|removeAllWeapons|removeBackpack|removeBackpackGlobal|removeCuratorAddons|removeCuratorCameraArea|removeCuratorEditableObjects|removeCuratorEditingArea|removeDrawIcon|removeDrawLinks|removeEventHandler|removeFromRemainsCollector|removeGoggles|removeGroupIcon|removeHandgunItem|removeHeadgear|removeItem|removeItemFromBackpack|removeItemFromUniform|removeItemFromVest|removeItems|removeMagazine|removeMagazineGlobal|removeMagazines|removeMagazinesTurret|removeMagazineTurret|removeMenuItem|removeMissionEventHandler|removeMPEventHandler|removeMusicEventHandler|removeOwnedMine|removePrimaryWeaponItem|removeSecondaryWeaponItem|removeSimpleTask|removeSwitchableUnit|removeTeamMember|removeUniform|removeVest|removeWeapon|removeWeaponAttachmentCargo|removeWeaponCargo|removeWeaponGlobal|removeWeaponTurret|reportRemoteTarget|requiredVersion|resetCamShake|resetSubgroupDirection|resistance|resize|resources|respawnVehicle|restartEditorCamera|reveal|revealMine|reverse|reversedMouseY|roadAt|roadsConnectedTo|roleDescription|ropeAttachedObjects|ropeAttachedTo|ropeAttachEnabled|ropeAttachTo|ropeCreate|ropeCut|ropeDestroy|ropeDetach|ropeEndPosition|ropeLength|ropes|ropeUnwind|ropeUnwound|rotorsForcesRTD|rotorsRpmRTD|round|runInitScript|safeZoneH|safeZoneW|safeZoneWAbs|safeZoneX|safeZoneXAbs|safeZoneY|save3DENInventory|saveGame|saveIdentity|saveJoysticks|saveOverlay|saveProfileNamespace|saveStatus|saveVar|savingEnabled|say|say2D|say3D|score|scoreSide|screenshot|screenToWorld|scriptDone|scriptName|scriptNull|scudState|secondaryWeapon|secondaryWeaponItems|secondaryWeaponMagazine|select|selectBestPlaces|selectDiarySubject|selectedEditorObjects|selectEditorObject|selectionNames|selectionPosition|selectLeader|selectMax|selectMin|selectNoPlayer|selectPlayer|selectRandom|selectRandomWeighted|selectWeapon|selectWeaponTurret|sendAUMessage|sendSimpleCommand|sendTask|sendTaskResult|sendUDPMessage|serverCommand|serverCommandAvailable|serverCommandExecutable|serverName|serverTime|set|set3DENAttribute|set3DENAttributes|set3DENGrid|set3DENIconsVisible|set3DENLayer|set3DENLinesVisible|set3DENLogicType|set3DENMissionAttribute|set3DENMissionAttributes|set3DENModelsVisible|set3DENObjectType|set3DENSelected|setAccTime|setActualCollectiveRTD|setAirplaneThrottle|setAirportSide|setAmmo|setAmmoCargo|setAmmoOnPylon|setAnimSpeedCoef|setAperture|setApertureNew|setArmoryPoints|setAttributes|setAutonomous|setBehaviour|setBleedingRemaining|setBrakesRTD|setCameraInterest|setCamShakeDefParams|setCamShakeParams|setCamUseTI|setCaptive|setCenterOfMass|setCollisionLight|setCombatMode|setCompassOscillation|setConvoySeparation|setCuratorCameraAreaCeiling|setCuratorCoef|setCuratorEditingAreaType|setCuratorWaypointCost|setCurrentChannel|setCurrentTask|setCurrentWaypoint|setCustomAimCoef|setCustomWeightRTD|setDamage|setDammage|setDate|setDebriefingText|setDefaultCamera|setDestination|setDetailMapBlendPars|setDir|setDirection|setDrawIcon|setDriveOnPath|setDropInterval|setDynamicSimulationDistance|setDynamicSimulationDistanceCoef|setEditorMode|setEditorObjectScope|setEffectCondition|setEngineRpmRTD|setFace|setFaceAnimation|setFatigue|setFeatureType|setFlagAnimationPhase|setFlagOwner|setFlagSide|setFlagTexture|setFog|setForceGeneratorRTD|setFormation|setFormationTask|setFormDir|setFriend|setFromEditor|setFSMVariable|setFuel|setFuelCargo|setGroupIcon|setGroupIconParams|setGroupIconsSelectable|setGroupIconsVisible|setGroupId|setGroupIdGlobal|setGroupOwner|setGusts|setHideBehind|setHit|setHitIndex|setHitPointDamage|setHorizonParallaxCoef|setHUDMovementLevels|setIdentity|setImportance|setInfoPanel|setLeader|setLightAmbient|setLightAttenuation|setLightBrightness|setLightColor|setLightDayLight|setLightFlareMaxDistance|setLightFlareSize|setLightIntensity|setLightnings|setLightUseFlare|setLocalWindParams|setMagazineTurretAmmo|setMarkerAlpha|setMarkerAlphaLocal|setMarkerBrush|setMarkerBrushLocal|setMarkerColor|setMarkerColorLocal|setMarkerDir|setMarkerDirLocal|setMarkerPos|setMarkerPosLocal|setMarkerShape|setMarkerShapeLocal|setMarkerSize|setMarkerSizeLocal|setMarkerText|setMarkerTextLocal|setMarkerType|setMarkerTypeLocal|setMass|setMimic|setMousePosition|setMusicEffect|setMusicEventHandler|setName|setNameSound|setObjectArguments|setObjectMaterial|setObjectMaterialGlobal|setObjectProxy|setObjectTexture|setObjectTextureGlobal|setObjectViewDistance|setOvercast|setOwner|setOxygenRemaining|setParticleCircle|setParticleClass|setParticleFire|setParticleParams|setParticleRandom|setPilotCameraDirection|setPilotCameraRotation|setPilotCameraTarget|setPilotLight|setPiPEffect|setPitch|setPlateNumber|setPlayable|setPlayerRespawnTime|setPos|setPosASL|setPosASL2|setPosASLW|setPosATL|setPosition|setPosWorld|setPylonLoadOut|setPylonsPriority|setRadioMsg|setRain|setRainbow|setRandomLip|setRank|setRectangular|setRepairCargo|setRotorBrakeRTD|setShadowDistance|setShotParents|setSide|setSimpleTaskAlwaysVisible|setSimpleTaskCustomData|setSimpleTaskDescription|setSimpleTaskDestination|setSimpleTaskTarget|setSimpleTaskType|setSimulWeatherLayers|setSize|setSkill|setSlingLoad|setSoundEffect|setSpeaker|setSpeech|setSpeedMode|setStamina|setStaminaScheme|setStatValue|setSuppression|setSystemOfUnits|setTargetAge|setTaskMarkerOffset|setTaskResult|setTaskState|setTerrainGrid|setText|setTimeMultiplier|setTitleEffect|setToneMapping|setToneMappingParams|setTrafficDensity|setTrafficDistance|setTrafficGap|setTrafficSpeed|setTriggerActivation|setTriggerArea|setTriggerStatements|setTriggerText|setTriggerTimeout|setTriggerType|setType|setUnconscious|setUnitAbility|setUnitLoadout|setUnitPos|setUnitPosWeak|setUnitRank|setUnitRecoilCoefficient|setUnitTrait|setUnloadInCombat|setUserActionText|setUserMFDText|setUserMFDValue|setVariable|setVectorDir|setVectorDirAndUp|setVectorUp|setVehicleAmmo|setVehicleAmmoDef|setVehicleArmor|setVehicleCargo|setVehicleId|setVehicleInit|setVehicleLock|setVehiclePosition|setVehicleRadar|setVehicleReceiveRemoteTargets|setVehicleReportOwnPosition|setVehicleReportRemoteTargets|setVehicleTIPars|setVehicleVarName|setVelocity|setVelocityModelSpace|setVelocityTransformation|setViewDistance|setVisibleIfTreeCollapsed|setWantedRpmRTD|setWaves|setWaypointBehaviour|setWaypointCombatMode|setWaypointCompletionRadius|setWaypointDescription|setWaypointForceBehaviour|setWaypointFormation|setWaypointHousePosition|setWaypointLoiterRadius|setWaypointLoiterType|setWaypointName|setWaypointPosition|setWaypointScript|setWaypointSpeed|setWaypointStatements|setWaypointTimeout|setWaypointType|setWaypointVisible|setWeaponReloadingTime|setWind|setWindDir|setWindForce|setWindStr|setWingForceScaleRTD|setWPPos|show3DIcons|showChat|showCinemaBorder|showCommandingMenu|showCompass|showCuratorCompass|showGPS|showHUD|showLegend|showMap|shownArtilleryComputer|shownChat|shownCompass|shownCuratorCompass|showNewEditorObject|shownGPS|shownHUD|shownMap|shownPad|shownRadio|shownScoretable|shownUAVFeed|shownWarrant|shownWatch|showPad|showRadio|showScoretable|showSubtitles|showUAVFeed|showWarrant|showWatch|showWaypoint|showWaypoints|side|sideAmbientLife|sideChat|sideEmpty|sideEnemy|sideFriendly|sideLogic|sideRadio|sideUnknown|simpleTasks|simulationEnabled|simulCloudDensity|simulCloudOcclusion|simulInClouds|simulWeatherSync|sin|size|sizeOf|skill|skillFinal|skipTime|sleep|sliderPosition|sliderRange|sliderSetPosition|sliderSetRange|sliderSetSpeed|sliderSpeed|slingLoadAssistantShown|soldierMagazines|someAmmo|sort|soundVolume|speaker|speed|speedMode|splitString|sqrt|squadParams|stance|startLoadingScreen|stop|stopEngineRTD|stopped|str|sunOrMoon|supportInfo|suppressFor|surfaceIsWater|surfaceNormal|surfaceType|swimInDepth|switchableUnits|switchAction|switchCamera|switchGesture|switchLight|switchMove|synchronizedObjects|synchronizedTriggers|synchronizedWaypoints|synchronizeObjectsAdd|synchronizeObjectsRemove|synchronizeTrigger|synchronizeWaypoint|systemChat|systemOfUnits|tan|targetKnowledge|targets|targetsAggregate|targetsQuery|taskAlwaysVisible|taskChildren|taskCompleted|taskCustomData|taskDescription|taskDestination|taskHint|taskMarkerOffset|taskNull|taskParent|taskResult|taskState|taskType|teamMember|teamMemberNull|teamName|teams|teamSwitch|teamSwitchEnabled|teamType|terminate|terrainIntersect|terrainIntersectASL|terrainIntersectAtASL|text|textLog|textLogFormat|tg|time|timeMultiplier|titleCut|titleFadeOut|titleObj|titleRsc|titleText|toArray|toFixed|toLower|toString|toUpper|triggerActivated|triggerActivation|triggerArea|triggerAttachedVehicle|triggerAttachObject|triggerAttachVehicle|triggerDynamicSimulation|triggerStatements|triggerText|triggerTimeout|triggerTimeoutCurrent|triggerType|turretLocal|turretOwner|turretUnit|tvAdd|tvClear|tvCollapse|tvCollapseAll|tvCount|tvCurSel|tvData|tvDelete|tvExpand|tvExpandAll|tvPicture|tvPictureRight|tvSetColor|tvSetCurSel|tvSetData|tvSetPicture|tvSetPictureColor|tvSetPictureColorDisabled|tvSetPictureColorSelected|tvSetPictureRight|tvSetPictureRightColor|tvSetPictureRightColorDisabled|tvSetPictureRightColorSelected|tvSetSelectColor|tvSetText|tvSetTooltip|tvSetValue|tvSort|tvSortByValue|tvText|tvTooltip|tvValue|type|typeName|typeOf|UAVControl|uiNamespace|uiSleep|unassignCurator|unassignItem|unassignTeam|unassignVehicle|underwater|uniform|uniformContainer|uniformItems|uniformMagazines|unitAddons|unitAimPosition|unitAimPositionVisual|unitBackpack|unitIsUAV|unitPos|unitReady|unitRecoilCoefficient|units|unitsBelowHeight|unlinkItem|unlockAchievement|unregisterTask|updateDrawIcon|updateMenuItem|updateObjectTree|useAIOperMapObstructionTest|useAISteeringComponent|useAudioTimeForMoves|userInputDisabled|vectorAdd|vectorCos|vectorCrossProduct|vectorDiff|vectorDir|vectorDirVisual|vectorDistance|vectorDistanceSqr|vectorDotProduct|vectorFromTo|vectorMagnitude|vectorMagnitudeSqr|vectorModelToWorld|vectorModelToWorldVisual|vectorMultiply|vectorNormalized|vectorUp|vectorUpVisual|vectorWorldToModel|vectorWorldToModelVisual|vehicle|vehicleCargoEnabled|vehicleChat|vehicleRadio|vehicleReceiveRemoteTargets|vehicleReportOwnPosition|vehicleReportRemoteTargets|vehicles|vehicleVarName|velocity|velocityModelSpace|verifySignature|vest|vestContainer|vestItems|vestMagazines|viewDistance|visibleCompass|visibleGPS|visibleMap|visiblePosition|visiblePositionASL|visibleScoretable|visibleWatch|waitUntil|waves|waypointAttachedObject|waypointAttachedVehicle|waypointAttachObject|waypointAttachVehicle|waypointBehaviour|waypointCombatMode|waypointCompletionRadius|waypointDescription|waypointForceBehaviour|waypointFormation|waypointHousePosition|waypointLoiterRadius|waypointLoiterType|waypointName|waypointPosition|waypoints|waypointScript|waypointsEnabledUAV|waypointShow|waypointSpeed|waypointStatements|waypointTimeout|waypointTimeoutCurrent|waypointType|waypointVisible|weaponAccessories|weaponAccessoriesCargo|weaponCargo|weaponDirection|weaponInertia|weaponLowered|weapons|weaponsItems|weaponsItemsCargo|weaponState|weaponsTurret|weightRTD|west|WFSideText|wind|windDir|windRTD|windStr|wingsForcesRTD|worldName|worldSize|worldToModel|worldToModelVisual|worldToScreen)\b/i,number:/(?:\$|\b0x)[\da-f]+\b|(?:\B\.\d+|\b\d+(?:\.\d+)?)(?:e[+-]?\d+)?\b/i,operator:/##|>>|&&|\|\||[!=<>]=?|[-+*/%#^]|\b(?:and|mod|not|or)\b/i,"magic-variable":{pattern:/\b(?:this|thisList|thisTrigger|_exception|_fnc_scriptName|_fnc_scriptNameParent|_forEachIndex|_this|_thisEventHandler|_thisFSM|_thisScript|_x)\b/i,alias:"keyword"},constant:/\bDIK(?:_[a-z\d]+)+\b/i}),t.languages.insertBefore("sqf","string",{macro:{pattern:/(^[ \t]*)#[a-z](?:[^\r\n\\]|\\(?:\r\n|[\s\S]))*/im,lookbehind:!0,greedy:!0,alias:"property",inside:{directive:{pattern:/#[a-z]+\b/i,alias:"keyword"},comment:t.languages.sqf.comment}}}),delete t.languages.sqf["class-name"]}return z3}var H3,fW;function ast(){if(fW)return H3;fW=1,H3=e,e.displayName="squirrel",e.aliases=[];function e(t){t.languages.squirrel=t.languages.extend("clike",{comment:[t.languages.clike.comment[0],{pattern:/(^|[^\\:])(?:\/\/|#).*/,lookbehind:!0,greedy:!0}],string:{pattern:/(^|[^\\"'@])(?:@"(?:[^"]|"")*"(?!")|"(?:[^\\\r\n"]|\\.)*")/,lookbehind:!0,greedy:!0},"class-name":{pattern:/(\b(?:class|enum|extends|instanceof)\s+)\w+(?:\.\w+)*/,lookbehind:!0,inside:{punctuation:/\./}},keyword:/\b(?:__FILE__|__LINE__|base|break|case|catch|class|clone|const|constructor|continue|default|delete|else|enum|extends|for|foreach|function|if|in|instanceof|local|null|resume|return|static|switch|this|throw|try|typeof|while|yield)\b/,number:/\b(?:0x[0-9a-fA-F]+|\d+(?:\.(?:\d+|[eE][+-]?\d+))?)\b/,operator:/\+\+|--|<=>|<[-<]|>>>?|&&?|\|\|?|[-+*/%!=<>]=?|[~^]|::?/,punctuation:/[(){}\[\],;.]/}),t.languages.insertBefore("squirrel","string",{char:{pattern:/(^|[^\\"'])'(?:[^\\']|\\(?:[xuU][0-9a-fA-F]{0,8}|[\s\S]))'/,lookbehind:!0,greedy:!0}}),t.languages.insertBefore("squirrel","operator",{"attribute-punctuation":{pattern:/<\/|\/>/,alias:"important"},lambda:{pattern:/@(?=\()/,alias:"operator"}})}return H3}var U3,mW;function ist(){if(mW)return U3;mW=1,U3=e,e.displayName="stan",e.aliases=[];function e(t){(function(r){var n=/\b(?:algebra_solver|algebra_solver_newton|integrate_1d|integrate_ode|integrate_ode_bdf|integrate_ode_rk45|map_rect|ode_(?:adams|bdf|ckrk|rk45)(?:_tol)?|ode_adjoint_tol_ctl|reduce_sum|reduce_sum_static)\b/;r.languages.stan={comment:/\/\/.*|\/\*[\s\S]*?\*\/|#(?!include).*/,string:{pattern:/"[\x20\x21\x23-\x5B\x5D-\x7E]*"/,greedy:!0},directive:{pattern:/^([ \t]*)#include\b.*/m,lookbehind:!0,alias:"property"},"function-arg":{pattern:RegExp("("+n.source+/\s*\(\s*/.source+")"+/[a-zA-Z]\w*/.source),lookbehind:!0,alias:"function"},constraint:{pattern:/(\b(?:int|matrix|real|row_vector|vector)\s*)<[^<>]*>/,lookbehind:!0,inside:{expression:{pattern:/(=\s*)\S(?:\S|\s+(?!\s))*?(?=\s*(?:>$|,\s*\w+\s*=))/,lookbehind:!0,inside:null},property:/\b[a-z]\w*(?=\s*=)/i,operator:/=/,punctuation:/^<|>$|,/}},keyword:[{pattern:/\bdata(?=\s*\{)|\b(?:functions|generated|model|parameters|quantities|transformed)\b/,alias:"program-block"},/\b(?:array|break|cholesky_factor_corr|cholesky_factor_cov|complex|continue|corr_matrix|cov_matrix|data|else|for|if|in|increment_log_prob|int|matrix|ordered|positive_ordered|print|real|reject|return|row_vector|simplex|target|unit_vector|vector|void|while)\b/,n],function:/\b[a-z]\w*(?=\s*\()/i,number:/(?:\b\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\B\.\d+(?:_\d+)*)(?:E[+-]?\d+(?:_\d+)*)?i?(?!\w)/i,boolean:/\b(?:false|true)\b/,operator:/<-|\.[*/]=?|\|\|?|&&|[!=<>+\-*/]=?|['^%~?:]/,punctuation:/[()\[\]{},;]/},r.languages.stan.constraint.inside.expression.inside=r.languages.stan})(t)}return U3}var $3,gW;function ost(){if(gW)return $3;gW=1,$3=e,e.displayName="stylus",e.aliases=[];function e(t){(function(r){var n={pattern:/(\b\d+)(?:%|[a-z]+)/,lookbehind:!0},a={pattern:/(^|[^\w.-])-?(?:\d+(?:\.\d+)?|\.\d+)/,lookbehind:!0},i={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,lookbehind:!0},url:{pattern:/\burl\((["']?).*?\1\)/i,greedy:!0},string:{pattern:/("|')(?:(?!\1)[^\\\r\n]|\\(?:\r\n|[\s\S]))*\1/,greedy:!0},interpolation:null,func:null,important:/\B!(?:important|optional)\b/i,keyword:{pattern:/(^|\s+)(?:(?:else|for|if|return|unless)(?=\s|$)|@[\w-]+)/,lookbehind:!0},hexcode:/#[\da-f]{3,6}/i,color:[/\b(?:AliceBlue|AntiqueWhite|Aqua|Aquamarine|Azure|Beige|Bisque|Black|BlanchedAlmond|Blue|BlueViolet|Brown|BurlyWood|CadetBlue|Chartreuse|Chocolate|Coral|CornflowerBlue|Cornsilk|Crimson|Cyan|DarkBlue|DarkCyan|DarkGoldenRod|DarkGr[ae]y|DarkGreen|DarkKhaki|DarkMagenta|DarkOliveGreen|DarkOrange|DarkOrchid|DarkRed|DarkSalmon|DarkSeaGreen|DarkSlateBlue|DarkSlateGr[ae]y|DarkTurquoise|DarkViolet|DeepPink|DeepSkyBlue|DimGr[ae]y|DodgerBlue|FireBrick|FloralWhite|ForestGreen|Fuchsia|Gainsboro|GhostWhite|Gold|GoldenRod|Gr[ae]y|Green|GreenYellow|HoneyDew|HotPink|IndianRed|Indigo|Ivory|Khaki|Lavender|LavenderBlush|LawnGreen|LemonChiffon|LightBlue|LightCoral|LightCyan|LightGoldenRodYellow|LightGr[ae]y|LightGreen|LightPink|LightSalmon|LightSeaGreen|LightSkyBlue|LightSlateGr[ae]y|LightSteelBlue|LightYellow|Lime|LimeGreen|Linen|Magenta|Maroon|MediumAquaMarine|MediumBlue|MediumOrchid|MediumPurple|MediumSeaGreen|MediumSlateBlue|MediumSpringGreen|MediumTurquoise|MediumVioletRed|MidnightBlue|MintCream|MistyRose|Moccasin|NavajoWhite|Navy|OldLace|Olive|OliveDrab|Orange|OrangeRed|Orchid|PaleGoldenRod|PaleGreen|PaleTurquoise|PaleVioletRed|PapayaWhip|PeachPuff|Peru|Pink|Plum|PowderBlue|Purple|Red|RosyBrown|RoyalBlue|SaddleBrown|Salmon|SandyBrown|SeaGreen|SeaShell|Sienna|Silver|SkyBlue|SlateBlue|SlateGr[ae]y|Snow|SpringGreen|SteelBlue|Tan|Teal|Thistle|Tomato|Transparent|Turquoise|Violet|Wheat|White|WhiteSmoke|Yellow|YellowGreen)\b/i,{pattern:/\b(?:hsl|rgb)\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*\)\B|\b(?:hsl|rgb)a\(\s*\d{1,3}\s*,\s*\d{1,3}%?\s*,\s*\d{1,3}%?\s*,\s*(?:0|0?\.\d+|1)\s*\)\B/i,inside:{unit:n,number:a,function:/[\w-]+(?=\()/,punctuation:/[(),]/}}],entity:/\\[\da-f]{1,8}/i,unit:n,boolean:/\b(?:false|true)\b/,operator:[/~|[+!\/%<>?=]=?|[-:]=|\*[*=]?|\.{2,3}|&&|\|\||\B-\B|\b(?:and|in|is(?: a| defined| not|nt)?|not|or)\b/],number:a,punctuation:/[{}()\[\];:,]/};i.interpolation={pattern:/\{[^\r\n}:]+\}/,alias:"variable",inside:{delimiter:{pattern:/^\{|\}$/,alias:"punctuation"},rest:i}},i.func={pattern:/[\w-]+\([^)]*\).*/,inside:{function:/^[^(]+/,rest:i}},r.languages.stylus={"atrule-declaration":{pattern:/(^[ \t]*)@.+/m,lookbehind:!0,inside:{atrule:/^@[\w-]+/,rest:i}},"variable-declaration":{pattern:/(^[ \t]*)[\w$-]+\s*.?=[ \t]*(?:\{[^{}]*\}|\S.*|$)/m,lookbehind:!0,inside:{variable:/^\S+/,rest:i}},statement:{pattern:/(^[ \t]*)(?:else|for|if|return|unless)[ \t].+/m,lookbehind:!0,inside:{keyword:/^\S+/,rest:i}},"property-declaration":{pattern:/((?:^|\{)([ \t]*))(?:[\w-]|\{[^}\r\n]+\})+(?:\s*:\s*|[ \t]+)(?!\s)[^{\r\n]*(?:;|[^{\r\n,]$(?!(?:\r?\n|\r)(?:\{|\2[ \t])))/m,lookbehind:!0,inside:{property:{pattern:/^[^\s:]+/,inside:{interpolation:i.interpolation}},rest:i}},selector:{pattern:/(^[ \t]*)(?:(?=\S)(?:[^{}\r\n:()]|::?[\w-]+(?:\([^)\r\n]*\)|(?![\w-]))|\{[^}\r\n]+\})+)(?:(?:\r?\n|\r)(?:\1(?:(?=\S)(?:[^{}\r\n:()]|::?[\w-]+(?:\([^)\r\n]*\)|(?![\w-]))|\{[^}\r\n]+\})+)))*(?:,$|\{|(?=(?:\r?\n|\r)(?:\{|\1[ \t])))/m,lookbehind:!0,inside:{interpolation:i.interpolation,comment:i.comment,punctuation:/[{},]/}},func:i.func,string:i.string,comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|\/\/.*)/,lookbehind:!0,greedy:!0},interpolation:i.interpolation,punctuation:/[{}()\[\];:.]/}})(t)}return $3}var G3,bW;function sst(){if(bW)return G3;bW=1,G3=e,e.displayName="swift",e.aliases=[];function e(t){t.languages.swift={comment:{pattern:/(^|[^\\:])(?:\/\/.*|\/\*(?:[^/*]|\/(?!\*)|\*(?!\/)|\/\*(?:[^*]|\*(?!\/))*\*\/)*\*\/)/,lookbehind:!0,greedy:!0},"string-literal":[{pattern:RegExp(/(^|[^"#])/.source+"(?:"+/"(?:\\(?:\((?:[^()]|\([^()]*\))*\)|\r\n|[^(])|[^\\\r\n"])*"/.source+"|"+/"""(?:\\(?:\((?:[^()]|\([^()]*\))*\)|[^(])|[^\\"]|"(?!""))*"""/.source+")"+/(?!["#])/.source),lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/(\\\()(?:[^()]|\([^()]*\))*(?=\))/,lookbehind:!0,inside:null},"interpolation-punctuation":{pattern:/^\)|\\\($/,alias:"punctuation"},punctuation:/\\(?=[\r\n])/,string:/[\s\S]+/}},{pattern:RegExp(/(^|[^"#])(#+)/.source+"(?:"+/"(?:\\(?:#+\((?:[^()]|\([^()]*\))*\)|\r\n|[^#])|[^\\\r\n])*?"/.source+"|"+/"""(?:\\(?:#+\((?:[^()]|\([^()]*\))*\)|[^#])|[^\\])*?"""/.source+")\\2"),lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/(\\#+\()(?:[^()]|\([^()]*\))*(?=\))/,lookbehind:!0,inside:null},"interpolation-punctuation":{pattern:/^\)|\\#+\($/,alias:"punctuation"},string:/[\s\S]+/}}],directive:{pattern:RegExp(/#/.source+"(?:"+(/(?:elseif|if)\b/.source+"(?:[ 	]*"+/(?:![ \t]*)?(?:\b\w+\b(?:[ \t]*\((?:[^()]|\([^()]*\))*\))?|\((?:[^()]|\([^()]*\))*\))(?:[ \t]*(?:&&|\|\|))?/.source+")+")+"|"+/(?:else|endif)\b/.source+")"),alias:"property",inside:{"directive-name":/^#\w+/,boolean:/\b(?:false|true)\b/,number:/\b\d+(?:\.\d+)*\b/,operator:/!|&&|\|\||[<>]=?/,punctuation:/[(),]/}},literal:{pattern:/#(?:colorLiteral|column|dsohandle|file(?:ID|Literal|Path)?|function|imageLiteral|line)\b/,alias:"constant"},"other-directive":{pattern:/#\w+\b/,alias:"property"},attribute:{pattern:/@\w+/,alias:"atrule"},"function-definition":{pattern:/(\bfunc\s+)\w+/,lookbehind:!0,alias:"function"},label:{pattern:/\b(break|continue)\s+\w+|\b[a-zA-Z_]\w*(?=\s*:\s*(?:for|repeat|while)\b)/,lookbehind:!0,alias:"important"},keyword:/\b(?:Any|Protocol|Self|Type|actor|as|assignment|associatedtype|associativity|async|await|break|case|catch|class|continue|convenience|default|defer|deinit|didSet|do|dynamic|else|enum|extension|fallthrough|fileprivate|final|for|func|get|guard|higherThan|if|import|in|indirect|infix|init|inout|internal|is|isolated|lazy|left|let|lowerThan|mutating|none|nonisolated|nonmutating|open|operator|optional|override|postfix|precedencegroup|prefix|private|protocol|public|repeat|required|rethrows|return|right|safe|self|set|some|static|struct|subscript|super|switch|throw|throws|try|typealias|unowned|unsafe|var|weak|where|while|willSet)\b/,boolean:/\b(?:false|true)\b/,nil:{pattern:/\bnil\b/,alias:"constant"},"short-argument":/\$\d+\b/,omit:{pattern:/\b_\b/,alias:"keyword"},number:/\b(?:[\d_]+(?:\.[\de_]+)?|0x[a-f0-9_]+(?:\.[a-f0-9p_]+)?|0b[01_]+|0o[0-7_]+)\b/i,"class-name":/\b[A-Z](?:[A-Z_\d]*[a-z]\w*)?\b/,function:/\b[a-z_]\w*(?=\s*\()/i,constant:/\b(?:[A-Z_]{2,}|k[A-Z][A-Za-z_]+)\b/,operator:/[-+*/%=!<>&|^~?]+|\.[.\-+*/%=!<>&|^~?]+/,punctuation:/[{}[\]();,.:\\]/},t.languages.swift["string-literal"].forEach(function(r){r.inside.interpolation.inside=t.languages.swift})}return G3}var j3,yW;function lst(){if(yW)return j3;yW=1,j3=e,e.displayName="systemd",e.aliases=[];function e(t){(function(r){var n={pattern:/^[;#].*/m,greedy:!0},a=/"(?:[^\r\n"\\]|\\(?:[^\r]|\r\n?))*"(?!\S)/.source;r.languages.systemd={comment:n,section:{pattern:/^\[[^\n\r\[\]]*\](?=[ \t]*$)/m,greedy:!0,inside:{punctuation:/^\[|\]$/,"section-name":{pattern:/[\s\S]+/,alias:"selector"}}},key:{pattern:/^[^\s=]+(?=[ \t]*=)/m,greedy:!0,alias:"attr-name"},value:{pattern:RegExp(/(=[ \t]*(?!\s))/.source+"(?:"+a+`|(?=[^"\r
]))(?:`+(/[^\s\\]/.source+'|[ 	]+(?:(?![ 	"])|'+a+")|"+/\\[\r\n]+(?:[#;].*[\r\n]+)*(?![#;])/.source)+")*"),lookbehind:!0,greedy:!0,alias:"attr-value",inside:{comment:n,quoted:{pattern:RegExp(/(^|\s)/.source+a),lookbehind:!0,greedy:!0},punctuation:/\\$/m,boolean:{pattern:/^(?:false|no|off|on|true|yes)$/,greedy:!0}}},punctuation:/=/}})(t)}return j3}var q3,vW;function vL(){if(vW)return q3;vW=1,q3=e,e.displayName="t4Templating",e.aliases=[];function e(t){(function(r){function n(i,o,s){return{pattern:RegExp("<#"+i+"[\\s\\S]*?#>"),alias:"block",inside:{delimiter:{pattern:RegExp("^<#"+i+"|#>$"),alias:"important"},content:{pattern:/[\s\S]+/,inside:o,alias:s}}}}function a(i){var o=r.languages[i],s="language-"+i;return{block:{pattern:/<#[\s\S]+?#>/,inside:{directive:n("@",{"attr-value":{pattern:/=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">=]+)/,inside:{punctuation:/^=|^["']|["']$/}},keyword:/\b\w+(?=\s)/,"attr-name":/\b\w+/}),expression:n("=",o,s),"class-feature":n("\\+",o,s),standard:n("",o,s)}}}}r.languages["t4-templating"]=Object.defineProperty({},"createT4",{value:a})})(t)}return q3}var W3,SW;function ust(){if(SW)return W3;SW=1;var e=vL(),t=rS();W3=r,r.displayName="t4Cs",r.aliases=[];function r(n){n.register(e),n.register(t),n.languages.t4=n.languages["t4-cs"]=n.languages["t4-templating"].createT4("csharp")}return W3}var V3,EW;function zce(){if(EW)return V3;EW=1;var e=Ice();V3=t,t.displayName="vbnet",t.aliases=[];function t(r){r.register(e),r.languages.vbnet=r.languages.extend("basic",{comment:[{pattern:/(?:!|REM\b).+/i,inside:{keyword:/^REM/i}},{pattern:/(^|[^\\:])'.*/,lookbehind:!0,greedy:!0}],string:{pattern:/(^|[^"])"(?:""|[^"])*"(?!")/,lookbehind:!0,greedy:!0},keyword:/(?:\b(?:ADDHANDLER|ADDRESSOF|ALIAS|AND|ANDALSO|AS|BEEP|BLOAD|BOOLEAN|BSAVE|BYREF|BYTE|BYVAL|CALL(?: ABSOLUTE)?|CASE|CATCH|CBOOL|CBYTE|CCHAR|CDATE|CDBL|CDEC|CHAIN|CHAR|CHDIR|CINT|CLASS|CLEAR|CLNG|CLOSE|CLS|COBJ|COM|COMMON|CONST|CONTINUE|CSBYTE|CSHORT|CSNG|CSTR|CTYPE|CUINT|CULNG|CUSHORT|DATA|DATE|DECIMAL|DECLARE|DEF(?: FN| SEG|DBL|INT|LNG|SNG|STR)|DEFAULT|DELEGATE|DIM|DIRECTCAST|DO|DOUBLE|ELSE|ELSEIF|END|ENUM|ENVIRON|ERASE|ERROR|EVENT|EXIT|FALSE|FIELD|FILES|FINALLY|FOR(?: EACH)?|FRIEND|FUNCTION|GET|GETTYPE|GETXMLNAMESPACE|GLOBAL|GOSUB|GOTO|HANDLES|IF|IMPLEMENTS|IMPORTS|IN|INHERITS|INPUT|INTEGER|INTERFACE|IOCTL|IS|ISNOT|KEY|KILL|LET|LIB|LIKE|LINE INPUT|LOCATE|LOCK|LONG|LOOP|LSET|ME|MKDIR|MOD|MODULE|MUSTINHERIT|MUSTOVERRIDE|MYBASE|MYCLASS|NAME|NAMESPACE|NARROWING|NEW|NEXT|NOT|NOTHING|NOTINHERITABLE|NOTOVERRIDABLE|OBJECT|OF|OFF|ON(?: COM| ERROR| KEY| TIMER)?|OPEN|OPERATOR|OPTION(?: BASE)?|OPTIONAL|OR|ORELSE|OUT|OVERLOADS|OVERRIDABLE|OVERRIDES|PARAMARRAY|PARTIAL|POKE|PRIVATE|PROPERTY|PROTECTED|PUBLIC|PUT|RAISEEVENT|READ|READONLY|REDIM|REM|REMOVEHANDLER|RESTORE|RESUME|RETURN|RMDIR|RSET|RUN|SBYTE|SELECT(?: CASE)?|SET|SHADOWS|SHARED|SHELL|SHORT|SINGLE|SLEEP|STATIC|STEP|STOP|STRING|STRUCTURE|SUB|SWAP|SYNCLOCK|SYSTEM|THEN|THROW|TIMER|TO|TROFF|TRON|TRUE|TRY|TRYCAST|TYPE|TYPEOF|UINTEGER|ULONG|UNLOCK|UNTIL|USHORT|USING|VIEW PRINT|WAIT|WEND|WHEN|WHILE|WIDENING|WITH|WITHEVENTS|WRITE|WRITEONLY|XOR)|\B(?:#CONST|#ELSE|#ELSEIF|#END|#IF))(?:\$|\b)/i,punctuation:/[,;:(){}]/})}return V3}var K3,xW;function cst(){if(xW)return K3;xW=1;var e=vL(),t=zce();K3=r,r.displayName="t4Vb",r.aliases=[];function r(n){n.register(e),n.register(t),n.languages["t4-vb"]=n.languages["t4-templating"].createT4("vbnet")}return K3}var Y3,wW;function Hce(){if(wW)return Y3;wW=1,Y3=e,e.displayName="yaml",e.aliases=["yml"];function e(t){(function(r){var n=/[*&][^\s[\]{},]+/,a=/!(?:<[\w\-%#;/?:@&=+$,.!~*'()[\]]+>|(?:[a-zA-Z\d-]*!)?[\w\-%#;/?:@&=+$.~*'()]+)?/,i="(?:"+a.source+"(?:[ 	]+"+n.source+")?|"+n.source+"(?:[ 	]+"+a.source+")?)",o=/(?:[^\s\x00-\x08\x0e-\x1f!"#%&'*,\-:>?@[\]`{|}\x7f-\x84\x86-\x9f\ud800-\udfff\ufffe\uffff]|[?:-]<PLAIN>)(?:[ \t]*(?:(?![#:])<PLAIN>|:<PLAIN>))*/.source.replace(/<PLAIN>/g,function(){return/[^\s\x00-\x08\x0e-\x1f,[\]{}\x7f-\x84\x86-\x9f\ud800-\udfff\ufffe\uffff]/.source}),s=/"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\\\r\n]|\\.)*'/.source;function l(u,d){d=(d||"").replace(/m/g,"")+"m";var p=/([:\-,[{]\s*(?:\s<<prop>>[ \t]+)?)(?:<<value>>)(?=[ \t]*(?:$|,|\]|\}|(?:[\r\n]\s*)?#))/.source.replace(/<<prop>>/g,function(){return i}).replace(/<<value>>/g,function(){return u});return RegExp(p,d)}r.languages.yaml={scalar:{pattern:RegExp(/([\-:]\s*(?:\s<<prop>>[ \t]+)?[|>])[ \t]*(?:((?:\r?\n|\r)[ \t]+)\S[^\r\n]*(?:\2[^\r\n]+)*)/.source.replace(/<<prop>>/g,function(){return i})),lookbehind:!0,alias:"string"},comment:/#.*/,key:{pattern:RegExp(/((?:^|[:\-,[{\r\n?])[ \t]*(?:<<prop>>[ \t]+)?)<<key>>(?=\s*:\s)/.source.replace(/<<prop>>/g,function(){return i}).replace(/<<key>>/g,function(){return"(?:"+o+"|"+s+")"})),lookbehind:!0,greedy:!0,alias:"atrule"},directive:{pattern:/(^[ \t]*)%.+/m,lookbehind:!0,alias:"important"},datetime:{pattern:l(/\d{4}-\d\d?-\d\d?(?:[tT]|[ \t]+)\d\d?:\d{2}:\d{2}(?:\.\d*)?(?:[ \t]*(?:Z|[-+]\d\d?(?::\d{2})?))?|\d{4}-\d{2}-\d{2}|\d\d?:\d{2}(?::\d{2}(?:\.\d*)?)?/.source),lookbehind:!0,alias:"number"},boolean:{pattern:l(/false|true/.source,"i"),lookbehind:!0,alias:"important"},null:{pattern:l(/null|~/.source,"i"),lookbehind:!0,alias:"important"},string:{pattern:l(s),lookbehind:!0,greedy:!0},number:{pattern:l(/[+-]?(?:0x[\da-f]+|0o[0-7]+|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?|\.inf|\.nan)/.source,"i"),lookbehind:!0},tag:a,important:n,punctuation:/---|[:[\]{}\-,|>?]|\.\.\./},r.languages.yml=r.languages.yaml})(t)}return Y3}var J3,kW;function dst(){if(kW)return J3;kW=1;var e=Hce();J3=t,t.displayName="tap",t.aliases=[];function t(r){r.register(e),r.languages.tap={fail:/not ok[^#{\n\r]*/,pass:/ok[^#{\n\r]*/,pragma:/pragma [+-][a-z]+/,bailout:/bail out!.*/i,version:/TAP version \d+/i,plan:/\b\d+\.\.\d+(?: +#.*)?/,subtest:{pattern:/# Subtest(?:: .*)?/,greedy:!0},punctuation:/[{}]/,directive:/#.*/,yamlish:{pattern:/(^[ \t]*)---[\s\S]*?[\r\n][ \t]*\.\.\.$/m,lookbehind:!0,inside:r.languages.yaml,alias:"language-yaml"}}}return J3}var X3,CW;function hst(){if(CW)return X3;CW=1,X3=e,e.displayName="tcl",e.aliases=[];function e(t){t.languages.tcl={comment:{pattern:/(^|[^\\])#.*/,lookbehind:!0},string:{pattern:/"(?:[^"\\\r\n]|\\(?:\r\n|[\s\S]))*"/,greedy:!0},variable:[{pattern:/(\$)(?:::)?(?:[a-zA-Z0-9]+::)*\w+/,lookbehind:!0},{pattern:/(\$)\{[^}]+\}/,lookbehind:!0},{pattern:/(^[\t ]*set[ \t]+)(?:::)?(?:[a-zA-Z0-9]+::)*\w+/m,lookbehind:!0}],function:{pattern:/(^[\t ]*proc[ \t]+)\S+/m,lookbehind:!0},builtin:[{pattern:/(^[\t ]*)(?:break|class|continue|error|eval|exit|for|foreach|if|proc|return|switch|while)\b/m,lookbehind:!0},/\b(?:else|elseif)\b/],scope:{pattern:/(^[\t ]*)(?:global|upvar|variable)\b/m,lookbehind:!0,alias:"constant"},keyword:{pattern:/(^[\t ]*|\[)(?:Safe_Base|Tcl|after|append|apply|array|auto_(?:execok|import|load|mkindex|qualify|reset)|automkindex_old|bgerror|binary|catch|cd|chan|clock|close|concat|dde|dict|encoding|eof|exec|expr|fblocked|fconfigure|fcopy|file(?:event|name)?|flush|gets|glob|history|http|incr|info|interp|join|lappend|lassign|lindex|linsert|list|llength|load|lrange|lrepeat|lreplace|lreverse|lsearch|lset|lsort|math(?:func|op)|memory|msgcat|namespace|open|package|parray|pid|pkg_mkIndex|platform|puts|pwd|re_syntax|read|refchan|regexp|registry|regsub|rename|scan|seek|set|socket|source|split|string|subst|tcl(?:_endOfWord|_findLibrary|startOf(?:Next|Previous)Word|test|vars|wordBreak(?:After|Before))|tell|time|tm|trace|unknown|unload|unset|update|uplevel|vwait)\b/m,lookbehind:!0},operator:/!=?|\*\*?|==|&&?|\|\|?|<[=<]?|>[=>]?|[-+~\/%?^]|\b(?:eq|in|ne|ni)\b/,punctuation:/[{}()\[\]]/}}return X3}var Z3,TW;function pst(){if(TW)return Z3;TW=1,Z3=e,e.displayName="textile",e.aliases=[];function e(t){(function(r){var n=/\([^|()\n]+\)|\[[^\]\n]+\]|\{[^}\n]+\}/.source,a=/\)|\((?![^|()\n]+\))/.source;function i(f,m){return RegExp(f.replace(/<MOD>/g,function(){return"(?:"+n+")"}).replace(/<PAR>/g,function(){return"(?:"+a+")"}),m||"")}var o={css:{pattern:/\{[^{}]+\}/,inside:{rest:r.languages.css}},"class-id":{pattern:/(\()[^()]+(?=\))/,lookbehind:!0,alias:"attr-value"},lang:{pattern:/(\[)[^\[\]]+(?=\])/,lookbehind:!0,alias:"attr-value"},punctuation:/[\\\/]\d+|\S/},s=r.languages.textile=r.languages.extend("markup",{phrase:{pattern:/(^|\r|\n)\S[\s\S]*?(?=$|\r?\n\r?\n|\r\r)/,lookbehind:!0,inside:{"block-tag":{pattern:i(/^[a-z]\w*(?:<MOD>|<PAR>|[<>=])*\./.source),inside:{modifier:{pattern:i(/(^[a-z]\w*)(?:<MOD>|<PAR>|[<>=])+(?=\.)/.source),lookbehind:!0,inside:o},tag:/^[a-z]\w*/,punctuation:/\.$/}},list:{pattern:i(/^[*#]+<MOD>*\s+\S.*/.source,"m"),inside:{modifier:{pattern:i(/(^[*#]+)<MOD>+/.source),lookbehind:!0,inside:o},punctuation:/^[*#]+/}},table:{pattern:i(/^(?:(?:<MOD>|<PAR>|[<>=^~])+\.\s*)?(?:\|(?:(?:<MOD>|<PAR>|[<>=^~_]|[\\/]\d+)+\.|(?!(?:<MOD>|<PAR>|[<>=^~_]|[\\/]\d+)+\.))[^|]*)+\|/.source,"m"),inside:{modifier:{pattern:i(/(^|\|(?:\r?\n|\r)?)(?:<MOD>|<PAR>|[<>=^~_]|[\\/]\d+)+(?=\.)/.source),lookbehind:!0,inside:o},punctuation:/\||^\./}},inline:{pattern:i(/(^|[^a-zA-Z\d])(\*\*|__|\?\?|[*_%@+\-^~])<MOD>*.+?\2(?![a-zA-Z\d])/.source),lookbehind:!0,inside:{bold:{pattern:i(/(^(\*\*?)<MOD>*).+?(?=\2)/.source),lookbehind:!0},italic:{pattern:i(/(^(__?)<MOD>*).+?(?=\2)/.source),lookbehind:!0},cite:{pattern:i(/(^\?\?<MOD>*).+?(?=\?\?)/.source),lookbehind:!0,alias:"string"},code:{pattern:i(/(^@<MOD>*).+?(?=@)/.source),lookbehind:!0,alias:"keyword"},inserted:{pattern:i(/(^\+<MOD>*).+?(?=\+)/.source),lookbehind:!0},deleted:{pattern:i(/(^-<MOD>*).+?(?=-)/.source),lookbehind:!0},span:{pattern:i(/(^%<MOD>*).+?(?=%)/.source),lookbehind:!0},modifier:{pattern:i(/(^\*\*|__|\?\?|[*_%@+\-^~])<MOD>+/.source),lookbehind:!0,inside:o},punctuation:/[*_%?@+\-^~]+/}},"link-ref":{pattern:/^\[[^\]]+\]\S+$/m,inside:{string:{pattern:/(^\[)[^\]]+(?=\])/,lookbehind:!0},url:{pattern:/(^\])\S+$/,lookbehind:!0},punctuation:/[\[\]]/}},link:{pattern:i(/"<MOD>*[^"]+":.+?(?=[^\w/]?(?:\s|$))/.source),inside:{text:{pattern:i(/(^"<MOD>*)[^"]+(?=")/.source),lookbehind:!0},modifier:{pattern:i(/(^")<MOD>+/.source),lookbehind:!0,inside:o},url:{pattern:/(:).+/,lookbehind:!0},punctuation:/[":]/}},image:{pattern:i(/!(?:<MOD>|<PAR>|[<>=])*(?![<>=])[^!\s()]+(?:\([^)]+\))?!(?::.+?(?=[^\w/]?(?:\s|$)))?/.source),inside:{source:{pattern:i(/(^!(?:<MOD>|<PAR>|[<>=])*)(?![<>=])[^!\s()]+(?:\([^)]+\))?(?=!)/.source),lookbehind:!0,alias:"url"},modifier:{pattern:i(/(^!)(?:<MOD>|<PAR>|[<>=])+/.source),lookbehind:!0,inside:o},url:{pattern:/(:).+/,lookbehind:!0},punctuation:/[!:]/}},footnote:{pattern:/\b\[\d+\]/,alias:"comment",inside:{punctuation:/\[|\]/}},acronym:{pattern:/\b[A-Z\d]+\([^)]+\)/,inside:{comment:{pattern:/(\()[^()]+(?=\))/,lookbehind:!0},punctuation:/[()]/}},mark:{pattern:/\b\((?:C|R|TM)\)/,alias:"comment",inside:{punctuation:/[()]/}}}}}),l=s.phrase.inside,u={inline:l.inline,link:l.link,image:l.image,footnote:l.footnote,acronym:l.acronym,mark:l.mark};s.tag.pattern=/<\/?(?!\d)[a-z0-9]+(?:\s+[^\s>\/=]+(?:=(?:("|')(?:\\[\s\S]|(?!\1)[^\\])*\1|[^\s'">=]+))?)*\s*\/?>/i;var d=l.inline.inside;d.bold.inside=u,d.italic.inside=u,d.inserted.inside=u,d.deleted.inside=u,d.span.inside=u;var p=l.table.inside;p.inline=u.inline,p.link=u.link,p.image=u.image,p.footnote=u.footnote,p.acronym=u.acronym,p.mark=u.mark})(t)}return Z3}var Q3,AW;function fst(){if(AW)return Q3;AW=1,Q3=e,e.displayName="toml",e.aliases=[];function e(t){(function(r){var n=/(?:[\w-]+|'[^'\n\r]*'|"(?:\\.|[^\\"\r\n])*")/.source;function a(i){return i.replace(/__/g,function(){return n})}r.languages.toml={comment:{pattern:/#.*/,greedy:!0},table:{pattern:RegExp(a(/(^[\t ]*\[\s*(?:\[\s*)?)__(?:\s*\.\s*__)*(?=\s*\])/.source),"m"),lookbehind:!0,greedy:!0,alias:"class-name"},key:{pattern:RegExp(a(/(^[\t ]*|[{,]\s*)__(?:\s*\.\s*__)*(?=\s*=)/.source),"m"),lookbehind:!0,greedy:!0,alias:"property"},string:{pattern:/"""(?:\\[\s\S]|[^\\])*?"""|'''[\s\S]*?'''|'[^'\n\r]*'|"(?:\\.|[^\\"\r\n])*"/,greedy:!0},date:[{pattern:/\b\d{4}-\d{2}-\d{2}(?:[T\s]\d{2}:\d{2}:\d{2}(?:\.\d+)?(?:Z|[+-]\d{2}:\d{2})?)?\b/i,alias:"number"},{pattern:/\b\d{2}:\d{2}:\d{2}(?:\.\d+)?\b/,alias:"number"}],number:/(?:\b0(?:x[\da-zA-Z]+(?:_[\da-zA-Z]+)*|o[0-7]+(?:_[0-7]+)*|b[10]+(?:_[10]+)*))\b|[-+]?\b\d+(?:_\d+)*(?:\.\d+(?:_\d+)*)?(?:[eE][+-]?\d+(?:_\d+)*)?\b|[-+]?\b(?:inf|nan)\b/,boolean:/\b(?:false|true)\b/,punctuation:/[.,=[\]{}]/}})(t)}return Q3}var e4,FW;function mst(){if(FW)return e4;FW=1,e4=e,e.displayName="tremor",e.aliases=[];function e(t){(function(r){r.languages.tremor={comment:{pattern:/(^|[^\\])(?:\/\*[\s\S]*?\*\/|(?:--|\/\/|#).*)/,lookbehind:!0},"interpolated-string":null,extractor:{pattern:/\b[a-z_]\w*\|(?:[^\r\n\\|]|\\(?:\r\n|[\s\S]))*\|/i,greedy:!0,inside:{regex:{pattern:/(^re)\|[\s\S]+/,lookbehind:!0},function:/^\w+/,value:/\|[\s\S]+/}},identifier:{pattern:/`[^`]*`/,greedy:!0},function:/\b[a-z_]\w*(?=\s*(?:::\s*<|\())\b/,keyword:/\b(?:args|as|by|case|config|connect|connector|const|copy|create|default|define|deploy|drop|each|emit|end|erase|event|flow|fn|for|from|group|having|insert|into|intrinsic|let|links|match|merge|mod|move|of|operator|patch|pipeline|recur|script|select|set|sliding|state|stream|to|tumbling|update|use|when|where|window|with)\b/,boolean:/\b(?:false|null|true)\b/i,number:/\b(?:0b[01_]*|0x[0-9a-fA-F_]*|\d[\d_]*(?:\.\d[\d_]*)?(?:[Ee][+-]?[\d_]+)?)\b/,"pattern-punctuation":{pattern:/%(?=[({[])/,alias:"punctuation"},operator:/[-+*\/%~!^]=?|=[=>]?|&[&=]?|\|[|=]?|<<?=?|>>?>?=?|(?:absent|and|not|or|present|xor)\b/,punctuation:/::|[;\[\]()\{\},.:]/};var n=/#\{(?:[^"{}]|\{[^{}]*\}|"(?:[^"\\\r\n]|\\(?:\r\n|[\s\S]))*")*\}/.source;r.languages.tremor["interpolated-string"]={pattern:RegExp(/(^|[^\\])/.source+'(?:"""(?:'+/[^"\\#]|\\[\s\S]|"(?!"")|#(?!\{)/.source+"|"+n+')*"""|"(?:'+/[^"\\\r\n#]|\\(?:\r\n|[\s\S])|#(?!\{)/.source+"|"+n+')*")'),lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:RegExp(n),inside:{punctuation:/^#\{|\}$/,expression:{pattern:/[\s\S]+/,inside:r.languages.tremor}}},string:/[\s\S]+/}},r.languages.troy=r.languages.tremor,r.languages.trickle=r.languages.tremor})(t)}return e4}var t4,_W;function gst(){if(_W)return t4;_W=1;var e=Bce(),t=gL();t4=r,r.displayName="tsx",r.aliases=[];function r(n){n.register(e),n.register(t),(function(a){var i=a.util.clone(a.languages.typescript);a.languages.tsx=a.languages.extend("jsx",i),delete a.languages.tsx.parameter,delete a.languages.tsx["literal-property"];var o=a.languages.tsx.tag;o.pattern=RegExp(/(^|[^\w$]|(?=<\/))/.source+"(?:"+o.pattern.source+")",o.pattern.flags),o.lookbehind=!0})(n)}return t4}var r4,RW;function bst(){if(RW)return r4;RW=1;var e=Si();r4=t,t.displayName="tt2",t.aliases=[];function t(r){r.register(e),(function(n){n.languages.tt2=n.languages.extend("clike",{comment:/#.*|\[%#[\s\S]*?%\]/,keyword:/\b(?:BLOCK|CALL|CASE|CATCH|CLEAR|DEBUG|DEFAULT|ELSE|ELSIF|END|FILTER|FINAL|FOREACH|GET|IF|IN|INCLUDE|INSERT|LAST|MACRO|META|NEXT|PERL|PROCESS|RAWPERL|RETURN|SET|STOP|SWITCH|TAGS|THROW|TRY|UNLESS|USE|WHILE|WRAPPER)\b/,punctuation:/[[\]{},()]/}),n.languages.insertBefore("tt2","number",{operator:/=[>=]?|!=?|<=?|>=?|&&|\|\|?|\b(?:and|not|or)\b/,variable:{pattern:/\b[a-z]\w*(?:\s*\.\s*(?:\d+|\$?[a-z]\w*))*\b/i}}),n.languages.insertBefore("tt2","keyword",{delimiter:{pattern:/^(?:\[%|%%)-?|-?%\]$/,alias:"punctuation"}}),n.languages.insertBefore("tt2","string",{"single-quoted-string":{pattern:/'[^\\']*(?:\\[\s\S][^\\']*)*'/,greedy:!0,alias:"string"},"double-quoted-string":{pattern:/"[^\\"]*(?:\\[\s\S][^\\"]*)*"/,greedy:!0,alias:"string",inside:{variable:{pattern:/\$(?:[a-z]\w*(?:\.(?:\d+|\$?[a-z]\w*))*)/i}}}}),delete n.languages.tt2.string,n.hooks.add("before-tokenize",function(a){var i=/\[%[\s\S]+?%\]/g;n.languages["markup-templating"].buildPlaceholders(a,"tt2",i)}),n.hooks.add("after-tokenize",function(a){n.languages["markup-templating"].tokenizePlaceholders(a,"tt2")})})(r)}return r4}var n4,DW;function yst(){if(DW)return n4;DW=1;var e=Si();n4=t,t.displayName="twig",t.aliases=[];function t(r){r.register(e),r.languages.twig={comment:/^\{#[\s\S]*?#\}$/,"tag-name":{pattern:/(^\{%-?\s*)\w+/,lookbehind:!0,alias:"keyword"},delimiter:{pattern:/^\{[{%]-?|-?[%}]\}$/,alias:"punctuation"},string:{pattern:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,inside:{punctuation:/^['"]|['"]$/}},keyword:/\b(?:even|if|odd)\b/,boolean:/\b(?:false|null|true)\b/,number:/\b0x[\dA-Fa-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][-+]?\d+)?/,operator:[{pattern:/(\s)(?:and|b-and|b-or|b-xor|ends with|in|is|matches|not|or|same as|starts with)(?=\s)/,lookbehind:!0},/[=<>]=?|!=|\*\*?|\/\/?|\?:?|[-+~%|]/],punctuation:/[()\[\]{}:.,]/},r.hooks.add("before-tokenize",function(n){if(n.language==="twig"){var a=/\{(?:#[\s\S]*?#|%[\s\S]*?%|\{[\s\S]*?\})\}/g;r.languages["markup-templating"].buildPlaceholders(n,"twig",a)}}),r.hooks.add("after-tokenize",function(n){r.languages["markup-templating"].tokenizePlaceholders(n,"twig")})}return n4}var a4,NW;function vst(){if(NW)return a4;NW=1,a4=e,e.displayName="typoscript",e.aliases=["tsconfig"];function e(t){(function(r){var n=/\b(?:ACT|ACTIFSUB|CARRAY|CASE|CLEARGIF|COA|COA_INT|CONSTANTS|CONTENT|CUR|EDITPANEL|EFFECT|EXT|FILE|FLUIDTEMPLATE|FORM|FRAME|FRAMESET|GIFBUILDER|GMENU|GMENU_FOLDOUT|GMENU_LAYERS|GP|HMENU|HRULER|HTML|IENV|IFSUB|IMAGE|IMGMENU|IMGMENUITEM|IMGTEXT|IMG_RESOURCE|INCLUDE_TYPOSCRIPT|JSMENU|JSMENUITEM|LLL|LOAD_REGISTER|NO|PAGE|RECORDS|RESTORE_REGISTER|TEMPLATE|TEXT|TMENU|TMENUITEM|TMENU_LAYERS|USER|USER_INT|_GIFBUILDER|global|globalString|globalVar)\b/;r.languages.typoscript={comment:[{pattern:/(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,lookbehind:!0},{pattern:/(^|[^\\:= \t]|(?:^|[^= \t])[ \t]+)\/\/.*/,lookbehind:!0,greedy:!0},{pattern:/(^|[^"'])#.*/,lookbehind:!0,greedy:!0}],function:[{pattern:/<INCLUDE_TYPOSCRIPT:\s*source\s*=\s*(?:"[^"\r\n]*"|'[^'\r\n]*')\s*>/,inside:{string:{pattern:/"[^"\r\n]*"|'[^'\r\n]*'/,inside:{keyword:n}},keyword:{pattern:/INCLUDE_TYPOSCRIPT/}}},{pattern:/@import\s*(?:"[^"\r\n]*"|'[^'\r\n]*')/,inside:{string:/"[^"\r\n]*"|'[^'\r\n]*'/}}],string:{pattern:/^([^=]*=[< ]?)(?:(?!\]\n).)*/,lookbehind:!0,inside:{function:/\{\$.*\}/,keyword:n,number:/^\d+$/,punctuation:/[,|:]/}},keyword:n,number:{pattern:/\b\d+\s*[.{=]/,inside:{operator:/[.{=]/}},tag:{pattern:/\.?[-\w\\]+\.?/,inside:{punctuation:/\./}},punctuation:/[{}[\];(),.:|]/,operator:/[<>]=?|[!=]=?=?|--?|\+\+?|&&?|\|\|?|[?*/~^%]/},r.languages.tsconfig=r.languages.typoscript})(t)}return a4}var i4,LW;function Sst(){if(LW)return i4;LW=1,i4=e,e.displayName="unrealscript",e.aliases=["uc","uscript"];function e(t){t.languages.unrealscript={comment:/\/\/.*|\/\*[\s\S]*?\*\//,string:{pattern:/(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,greedy:!0},category:{pattern:/(\b(?:(?:autoexpand|hide|show)categories|var)\s*\()[^()]+(?=\))/,lookbehind:!0,greedy:!0,alias:"property"},metadata:{pattern:/(\w\s*)<\s*\w+\s*=[^<>|=\r\n]+(?:\|\s*\w+\s*=[^<>|=\r\n]+)*>/,lookbehind:!0,greedy:!0,inside:{property:/\b\w+(?=\s*=)/,operator:/=/,punctuation:/[<>|]/}},macro:{pattern:/`\w+/,alias:"property"},"class-name":{pattern:/(\b(?:class|enum|extends|interface|state(?:\(\))?|struct|within)\s+)\w+/,lookbehind:!0},keyword:/\b(?:abstract|actor|array|auto|autoexpandcategories|bool|break|byte|case|class|classgroup|client|coerce|collapsecategories|config|const|continue|default|defaultproperties|delegate|dependson|deprecated|do|dontcollapsecategories|editconst|editinlinenew|else|enum|event|exec|export|extends|final|float|for|forcescriptorder|foreach|function|goto|guid|hidecategories|hidedropdown|if|ignores|implements|inherits|input|int|interface|iterator|latent|local|material|name|native|nativereplication|noexport|nontransient|noteditinlinenew|notplaceable|operator|optional|out|pawn|perobjectconfig|perobjectlocalized|placeable|postoperator|preoperator|private|protected|reliable|replication|return|server|showcategories|simulated|singular|state|static|string|struct|structdefault|structdefaultproperties|switch|texture|transient|travel|unreliable|until|var|vector|while|within)\b/,function:/\b[a-z_]\w*(?=\s*\()/i,boolean:/\b(?:false|true)\b/,number:/\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,operator:/>>|<<|--|\+\+|\*\*|[-+*/~!=<>$@]=?|&&?|\|\|?|\^\^?|[?:%]|\b(?:ClockwiseFrom|Cross|Dot)\b/,punctuation:/[()[\]{};,.]/},t.languages.uc=t.languages.uscript=t.languages.unrealscript}return i4}var o4,MW;function Est(){if(MW)return o4;MW=1,o4=e,e.displayName="uorazor",e.aliases=[];function e(t){t.languages.uorazor={"comment-hash":{pattern:/#.*/,alias:"comment",greedy:!0},"comment-slash":{pattern:/\/\/.*/,alias:"comment",greedy:!0},string:{pattern:/("|')(?:\\.|(?!\1)[^\\\r\n])*\1/,inside:{punctuation:/^['"]|['"]$/},greedy:!0},"source-layers":{pattern:/\b(?:arms|backpack|blue|bracelet|cancel|clear|cloak|criminal|earrings|enemy|facialhair|friend|friendly|gloves|gray|grey|ground|hair|head|innerlegs|innertorso|innocent|lefthand|middletorso|murderer|neck|nonfriendly|onehandedsecondary|outerlegs|outertorso|pants|red|righthand|ring|self|shirt|shoes|talisman|waist)\b/i,alias:"function"},"source-commands":{pattern:/\b(?:alliance|attack|cast|clearall|clearignore|clearjournal|clearlist|clearsysmsg|createlist|createtimer|dclick|dclicktype|dclickvar|dress|dressconfig|drop|droprelloc|emote|getlabel|guild|gumpclose|gumpresponse|hotkey|ignore|lasttarget|lift|lifttype|menu|menuresponse|msg|org|organize|organizer|overhead|pause|poplist|potion|promptresponse|pushlist|removelist|removetimer|rename|restock|say|scav|scavenger|script|setability|setlasttarget|setskill|settimer|setvar|sysmsg|target|targetloc|targetrelloc|targettype|undress|unignore|unsetvar|useobject|useonce|useskill|usetype|virtue|wait|waitforgump|waitformenu|waitforprompt|waitforstat|waitforsysmsg|waitfortarget|walk|wfsysmsg|wft|whisper|yell)\b/,alias:"function"},"tag-name":{pattern:/(^\{%-?\s*)\w+/,lookbehind:!0,alias:"keyword"},delimiter:{pattern:/^\{[{%]-?|-?[%}]\}$/,alias:"punctuation"},function:/\b(?:atlist|close|closest|count|counter|counttype|dead|dex|diffhits|diffmana|diffstam|diffweight|find|findbuff|finddebuff|findlayer|findtype|findtypelist|followers|gumpexists|hidden|hits|hp|hue|human|humanoid|ingump|inlist|insysmessage|insysmsg|int|invul|lhandempty|list|listexists|mana|maxhits|maxhp|maxmana|maxstam|maxweight|monster|mounted|name|next|noto|paralyzed|poisoned|position|prev|previous|queued|rand|random|rhandempty|skill|stam|str|targetexists|timer|timerexists|varexist|warmode|weight)\b/,keyword:/\b(?:and|as|break|continue|else|elseif|endfor|endif|endwhile|for|if|loop|not|or|replay|stop|while)\b/,boolean:/\b(?:false|null|true)\b/,number:/\b0x[\dA-Fa-f]+|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:[Ee][-+]?\d+)?/,operator:[{pattern:/(\s)(?:and|b-and|b-or|b-xor|ends with|in|is|matches|not|or|same as|starts with)(?=\s)/,lookbehind:!0},/[=<>]=?|!=|\*\*?|\/\/?|\?:?|[-+~%|]/],punctuation:/[()\[\]{}:.,]/}}return o4}var s4,IW;function xst(){if(IW)return s4;IW=1,s4=e,e.displayName="uri",e.aliases=["url"];function e(t){t.languages.uri={scheme:{pattern:/^[a-z][a-z0-9+.-]*:/im,greedy:!0,inside:{"scheme-delimiter":/:$/}},fragment:{pattern:/#[\w\-.~!$&'()*+,;=%:@/?]*/,inside:{"fragment-delimiter":/^#/}},query:{pattern:/\?[\w\-.~!$&'()*+,;=%:@/?]*/,inside:{"query-delimiter":{pattern:/^\?/,greedy:!0},"pair-delimiter":/[&;]/,pair:{pattern:/^[^=][\s\S]*/,inside:{key:/^[^=]+/,value:{pattern:/(^=)[\s\S]+/,lookbehind:!0}}}}},authority:{pattern:RegExp(/^\/\//.source+/(?:[\w\-.~!$&'()*+,;=%:]*@)?/.source+("(?:"+/\[(?:[0-9a-fA-F:.]{2,48}|v[0-9a-fA-F]+\.[\w\-.~!$&'()*+,;=]+)\]/.source+"|"+/[\w\-.~!$&'()*+,;=%]*/.source+")")+/(?::\d*)?/.source,"m"),inside:{"authority-delimiter":/^\/\//,"user-info-segment":{pattern:/^[\w\-.~!$&'()*+,;=%:]*@/,inside:{"user-info-delimiter":/@$/,"user-info":/^[\w\-.~!$&'()*+,;=%:]+/}},"port-segment":{pattern:/:\d*$/,inside:{"port-delimiter":/^:/,port:/^\d+/}},host:{pattern:/[\s\S]+/,inside:{"ip-literal":{pattern:/^\[[\s\S]+\]$/,inside:{"ip-literal-delimiter":/^\[|\]$/,"ipv-future":/^v[\s\S]+/,"ipv6-address":/^[\s\S]+/}},"ipv4-address":/^(?:(?:[03-9]\d?|[12]\d{0,2})\.){3}(?:[03-9]\d?|[12]\d{0,2})$/}}}},path:{pattern:/^[\w\-.~!$&'()*+,;=%:@/]+/m,inside:{"path-separator":/\//}}},t.languages.url=t.languages.uri}return s4}var l4,OW;function wst(){if(OW)return l4;OW=1,l4=e,e.displayName="v",e.aliases=[];function e(t){(function(r){var n={pattern:/[\s\S]+/,inside:null};r.languages.v=r.languages.extend("clike",{string:{pattern:/r?(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,alias:"quoted-string",greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)\$(?:\{[^{}]*\}|\w+(?:\.\w+(?:\([^\(\)]*\))?|\[[^\[\]]+\])*)/,lookbehind:!0,inside:{"interpolation-variable":{pattern:/^\$\w[\s\S]*$/,alias:"variable"},"interpolation-punctuation":{pattern:/^\$\{|\}$/,alias:"punctuation"},"interpolation-expression":n}}}},"class-name":{pattern:/(\b(?:enum|interface|struct|type)\s+)(?:C\.)?\w+/,lookbehind:!0},keyword:/(?:\b(?:__global|as|asm|assert|atomic|break|chan|const|continue|defer|else|embed|enum|fn|for|go(?:to)?|if|import|in|interface|is|lock|match|module|mut|none|or|pub|return|rlock|select|shared|sizeof|static|struct|type(?:of)?|union|unsafe)|\$(?:else|for|if)|#(?:flag|include))\b/,number:/\b(?:0x[a-f\d]+(?:_[a-f\d]+)*|0b[01]+(?:_[01]+)*|0o[0-7]+(?:_[0-7]+)*|\d+(?:_\d+)*(?:\.\d+(?:_\d+)*)?)\b/i,operator:/~|\?|[*\/%^!=]=?|\+[=+]?|-[=-]?|\|[=|]?|&(?:=|&|\^=?)?|>(?:>=?|=)?|<(?:<=?|=|-)?|:=|\.\.\.?/,builtin:/\b(?:any(?:_float|_int)?|bool|byte(?:ptr)?|charptr|f(?:32|64)|i(?:8|16|64|128|nt)|rune|size_t|string|u(?:16|32|64|128)|voidptr)\b/}),n.inside=r.languages.v,r.languages.insertBefore("v","string",{char:{pattern:/`(?:\\`|\\?[^`]{1,2})`/,alias:"rune"}}),r.languages.insertBefore("v","operator",{attribute:{pattern:/(^[\t ]*)\[(?:deprecated|direct_array_access|flag|inline|live|ref_only|typedef|unsafe_fn|windows_stdcall)\]/m,lookbehind:!0,alias:"annotation",inside:{punctuation:/[\[\]]/,keyword:/\w+/}},generic:{pattern:/<\w+>(?=\s*[\)\{])/,inside:{punctuation:/[<>]/,"class-name":/\w+/}}}),r.languages.insertBefore("v","function",{"generic-function":{pattern:/\b\w+\s*<\w+>(?=\()/,inside:{function:/^\w+/,generic:{pattern:/<\w+>/,inside:r.languages.v.generic.inside}}}})})(t)}return l4}var u4,BW;function kst(){if(BW)return u4;BW=1,u4=e,e.displayName="vala",e.aliases=[];function e(t){t.languages.vala=t.languages.extend("clike",{"class-name":[{pattern:/\b[A-Z]\w*(?:\.\w+)*\b(?=(?:\?\s+|\*?\s+\*?)\w)/,inside:{punctuation:/\./}},{pattern:/(\[)[A-Z]\w*(?:\.\w+)*\b/,lookbehind:!0,inside:{punctuation:/\./}},{pattern:/(\b(?:class|interface)\s+[A-Z]\w*(?:\.\w+)*\s*:\s*)[A-Z]\w*(?:\.\w+)*\b/,lookbehind:!0,inside:{punctuation:/\./}},{pattern:/((?:\b(?:class|enum|interface|new|struct)\s+)|(?:catch\s+\())[A-Z]\w*(?:\.\w+)*\b/,lookbehind:!0,inside:{punctuation:/\./}}],keyword:/\b(?:abstract|as|assert|async|base|bool|break|case|catch|char|class|const|construct|continue|default|delegate|delete|do|double|dynamic|else|ensures|enum|errordomain|extern|finally|float|for|foreach|get|if|in|inline|int|int16|int32|int64|int8|interface|internal|is|lock|long|namespace|new|null|out|override|owned|params|private|protected|public|ref|requires|return|set|short|signal|sizeof|size_t|ssize_t|static|string|struct|switch|this|throw|throws|try|typeof|uchar|uint|uint16|uint32|uint64|uint8|ulong|unichar|unowned|ushort|using|value|var|virtual|void|volatile|weak|while|yield)\b/i,function:/\b\w+(?=\s*\()/,number:/(?:\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?)(?:f|u?l?)?/i,operator:/\+\+|--|&&|\|\||<<=?|>>=?|=>|->|~|[+\-*\/%&^|=!<>]=?|\?\??|\.\.\./,punctuation:/[{}[\];(),.:]/,constant:/\b[A-Z0-9_]+\b/}),t.languages.insertBefore("vala","string",{"raw-string":{pattern:/"""[\s\S]*?"""/,greedy:!0,alias:"string"},"template-string":{pattern:/@"[\s\S]*?"/,greedy:!0,inside:{interpolation:{pattern:/\$(?:\([^)]*\)|[a-zA-Z]\w*)/,inside:{delimiter:{pattern:/^\$\(?|\)$/,alias:"punctuation"},rest:t.languages.vala}},string:/[\s\S]+/}}}),t.languages.insertBefore("vala","keyword",{regex:{pattern:/\/(?:\[(?:[^\]\\\r\n]|\\.)*\]|\\.|[^/\\\[\r\n])+\/[imsx]{0,4}(?=\s*(?:$|[\r\n,.;})\]]))/,greedy:!0,inside:{"regex-source":{pattern:/^(\/)[\s\S]+(?=\/[a-z]*$)/,lookbehind:!0,alias:"language-regex",inside:t.languages.regex},"regex-delimiter":/^\//,"regex-flags":/^[a-z]+$/}}})}return u4}var c4,PW;function Cst(){if(PW)return c4;PW=1,c4=e,e.displayName="velocity",e.aliases=[];function e(t){(function(r){r.languages.velocity=r.languages.extend("markup",{});var n={variable:{pattern:/(^|[^\\](?:\\\\)*)\$!?(?:[a-z][\w-]*(?:\([^)]*\))?(?:\.[a-z][\w-]*(?:\([^)]*\))?|\[[^\]]+\])*|\{[^}]+\})/i,lookbehind:!0,inside:{}},string:{pattern:/"[^"]*"|'[^']*'/,greedy:!0},number:/\b\d+\b/,boolean:/\b(?:false|true)\b/,operator:/[=!<>]=?|[+*/%-]|&&|\|\||\.\.|\b(?:eq|g[et]|l[et]|n(?:e|ot))\b/,punctuation:/[(){}[\]:,.]/};n.variable.inside={string:n.string,function:{pattern:/([^\w-])[a-z][\w-]*(?=\()/,lookbehind:!0},number:n.number,boolean:n.boolean,punctuation:n.punctuation},r.languages.insertBefore("velocity","comment",{unparsed:{pattern:/(^|[^\\])#\[\[[\s\S]*?\]\]#/,lookbehind:!0,greedy:!0,inside:{punctuation:/^#\[\[|\]\]#$/}},"velocity-comment":[{pattern:/(^|[^\\])#\*[\s\S]*?\*#/,lookbehind:!0,greedy:!0,alias:"comment"},{pattern:/(^|[^\\])##.*/,lookbehind:!0,greedy:!0,alias:"comment"}],directive:{pattern:/(^|[^\\](?:\\\\)*)#@?(?:[a-z][\w-]*|\{[a-z][\w-]*\})(?:\s*\((?:[^()]|\([^()]*\))*\))?/i,lookbehind:!0,inside:{keyword:{pattern:/^#@?(?:[a-z][\w-]*|\{[a-z][\w-]*\})|\bin\b/,inside:{punctuation:/[{}]/}},rest:n}},variable:n.variable}),r.languages.velocity.tag.inside["attr-value"].inside.rest=r.languages.velocity})(t)}return c4}var d4,zW;function Tst(){if(zW)return d4;zW=1,d4=e,e.displayName="verilog",e.aliases=[];function e(t){t.languages.verilog={comment:{pattern:/\/\/.*|\/\*[\s\S]*?\*\//,greedy:!0},string:{pattern:/"(?:\\(?:\r\n|[\s\S])|[^"\\\r\n])*"/,greedy:!0},"kernel-function":{pattern:/\B\$\w+\b/,alias:"property"},constant:/\B`\w+\b/,function:/\b\w+(?=\()/,keyword:/\b(?:alias|and|assert|assign|assume|automatic|before|begin|bind|bins|binsof|bit|break|buf|bufif0|bufif1|byte|case|casex|casez|cell|chandle|class|clocking|cmos|config|const|constraint|context|continue|cover|covergroup|coverpoint|cross|deassign|default|defparam|design|disable|dist|do|edge|else|end|endcase|endclass|endclocking|endconfig|endfunction|endgenerate|endgroup|endinterface|endmodule|endpackage|endprimitive|endprogram|endproperty|endsequence|endspecify|endtable|endtask|enum|event|expect|export|extends|extern|final|first_match|for|force|foreach|forever|fork|forkjoin|function|generate|genvar|highz0|highz1|if|iff|ifnone|ignore_bins|illegal_bins|import|incdir|include|initial|inout|input|inside|instance|int|integer|interface|intersect|join|join_any|join_none|large|liblist|library|local|localparam|logic|longint|macromodule|matches|medium|modport|module|nand|negedge|new|nmos|nor|noshowcancelled|not|notif0|notif1|null|or|output|package|packed|parameter|pmos|posedge|primitive|priority|program|property|protected|pull0|pull1|pulldown|pullup|pulsestyle_ondetect|pulsestyle_onevent|pure|rand|randc|randcase|randsequence|rcmos|real|realtime|ref|reg|release|repeat|return|rnmos|rpmos|rtran|rtranif0|rtranif1|scalared|sequence|shortint|shortreal|showcancelled|signed|small|solve|specify|specparam|static|string|strong0|strong1|struct|super|supply0|supply1|table|tagged|task|this|throughout|time|timeprecision|timeunit|tran|tranif0|tranif1|tri|tri0|tri1|triand|trior|trireg|type|typedef|union|unique|unsigned|use|uwire|var|vectored|virtual|void|wait|wait_order|wand|weak0|weak1|while|wildcard|wire|with|within|wor|xnor|xor)\b/,important:/\b(?:always|always_comb|always_ff|always_latch)\b(?: *@)?/,number:/\B##?\d+|(?:\b\d+)?'[odbh] ?[\da-fzx_?]+|\b(?:\d*[._])?\d+(?:e[-+]?\d+)?/i,operator:/[-+{}^~%*\/?=!<>&|]+/,punctuation:/[[\];(),.:]/}}return d4}var h4,HW;function Ast(){if(HW)return h4;HW=1,h4=e,e.displayName="vhdl",e.aliases=[];function e(t){t.languages.vhdl={comment:/--.+/,"vhdl-vectors":{pattern:/\b[oxb]"[\da-f_]+"|"[01uxzwlh-]+"/i,alias:"number"},"quoted-function":{pattern:/"\S+?"(?=\()/,alias:"function"},string:/"(?:[^\\"\r\n]|\\(?:\r\n|[\s\S]))*"/,constant:/\b(?:library|use)\b/i,keyword:/\b(?:'active|'ascending|'base|'delayed|'driving|'driving_value|'event|'high|'image|'instance_name|'last_active|'last_event|'last_value|'left|'leftof|'length|'low|'path_name|'pos|'pred|'quiet|'range|'reverse_range|'right|'rightof|'simple_name|'stable|'succ|'transaction|'val|'value|access|after|alias|all|architecture|array|assert|attribute|begin|block|body|buffer|bus|case|component|configuration|constant|disconnect|downto|else|elsif|end|entity|exit|file|for|function|generate|generic|group|guarded|if|impure|in|inertial|inout|is|label|library|linkage|literal|loop|map|new|next|null|of|on|open|others|out|package|port|postponed|procedure|process|pure|range|record|register|reject|report|return|select|severity|shared|signal|subtype|then|to|transport|type|unaffected|units|until|use|variable|wait|when|while|with)\b/i,boolean:/\b(?:false|true)\b/i,function:/\w+(?=\()/,number:/'[01uxzwlh-]'|\b(?:\d+#[\da-f_.]+#|\d[\d_.]*)(?:e[-+]?\d+)?/i,operator:/[<>]=?|:=|[-+*/&=]|\b(?:abs|and|mod|nand|nor|not|or|rem|rol|ror|sla|sll|sra|srl|xnor|xor)\b/i,punctuation:/[{}[\];(),.:]/}}return h4}var p4,UW;function Fst(){if(UW)return p4;UW=1,p4=e,e.displayName="vim",e.aliases=[];function e(t){t.languages.vim={string:/"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\r\n]|'')*'/,comment:/".*/,function:/\b\w+(?=\()/,keyword:/\b(?:N|Next|P|Print|X|XMLent|XMLns|ab|abbreviate|abc|abclear|abo|aboveleft|al|all|ar|arga|argadd|argd|argdelete|argdo|arge|argedit|argg|argglobal|argl|arglocal|args|argu|argument|as|ascii|b|bN|bNext|ba|bad|badd|ball|bd|bdelete|be|bel|belowright|bf|bfirst|bl|blast|bm|bmodified|bn|bnext|bo|botright|bp|bprevious|br|brea|break|breaka|breakadd|breakd|breakdel|breakl|breaklist|brewind|bro|browse|bufdo|buffer|buffers|bun|bunload|bw|bwipeout|c|cN|cNext|cNfcNfile|ca|cabbrev|cabc|cabclear|cad|caddb|caddbuffer|caddexpr|caddf|caddfile|cal|call|cat|catch|cb|cbuffer|cc|ccl|cclose|cd|ce|center|cex|cexpr|cf|cfile|cfir|cfirst|cg|cgetb|cgetbuffer|cgete|cgetexpr|cgetfile|change|changes|chd|chdir|che|checkpath|checkt|checktime|cl|cla|clast|clist|clo|close|cmapc|cmapclear|cn|cnew|cnewer|cnext|cnf|cnfile|cnorea|cnoreabbrev|co|col|colder|colo|colorscheme|comc|comclear|comp|compiler|con|conf|confirm|continue|cope|copen|copy|cp|cpf|cpfile|cprevious|cq|cquit|cr|crewind|cu|cuna|cunabbrev|cunmap|cw|cwindow|d|debugg|debuggreedy|delc|delcommand|delete|delf|delfunction|delm|delmarks|di|diffg|diffget|diffoff|diffpatch|diffpu|diffput|diffsplit|diffthis|diffu|diffupdate|dig|digraphs|display|dj|djump|dl|dlist|dr|drop|ds|dsearch|dsp|dsplit|e|earlier|echoe|echoerr|echom|echomsg|echon|edit|el|else|elsei|elseif|em|emenu|en|endf|endfo|endfor|endfun|endfunction|endif|endt|endtry|endw|endwhile|ene|enew|ex|exi|exit|exu|exusage|f|file|files|filetype|fin|fina|finally|find|fini|finish|fir|first|fix|fixdel|fo|fold|foldc|foldclose|foldd|folddoc|folddoclosed|folddoopen|foldo|foldopen|for|fu|fun|function|go|goto|gr|grep|grepa|grepadd|h|ha|hardcopy|help|helpf|helpfind|helpg|helpgrep|helpt|helptags|hid|hide|his|history|ia|iabbrev|iabc|iabclear|if|ij|ijump|il|ilist|imapc|imapclear|in|inorea|inoreabbrev|isearch|isp|isplit|iu|iuna|iunabbrev|iunmap|j|join|ju|jumps|k|kee|keepalt|keepj|keepjumps|keepmarks|l|lN|lNext|lNf|lNfile|la|lad|laddb|laddbuffer|laddexpr|laddf|laddfile|lan|language|last|later|lb|lbuffer|lc|lcd|lch|lchdir|lcl|lclose|left|lefta|leftabove|let|lex|lexpr|lf|lfile|lfir|lfirst|lg|lgetb|lgetbuffer|lgete|lgetexpr|lgetfile|lgr|lgrep|lgrepa|lgrepadd|lh|lhelpgrep|list|ll|lla|llast|lli|llist|lm|lmak|lmake|lmap|lmapc|lmapclear|ln|lne|lnew|lnewer|lnext|lnf|lnfile|lnoremap|lo|loadview|loc|lockmarks|lockv|lockvar|lol|lolder|lop|lopen|lp|lpf|lpfile|lprevious|lr|lrewind|ls|lt|ltag|lu|lunmap|lv|lvimgrep|lvimgrepa|lvimgrepadd|lw|lwindow|m|ma|mak|make|mark|marks|mat|match|menut|menutranslate|mk|mkexrc|mks|mksession|mksp|mkspell|mkv|mkvie|mkview|mkvimrc|mod|mode|move|mz|mzf|mzfile|mzscheme|n|nbkey|new|next|nmapc|nmapclear|noh|nohlsearch|norea|noreabbrev|nu|number|nun|nunmap|o|omapc|omapclear|on|only|open|opt|options|ou|ounmap|p|pc|pclose|pe|ped|pedit|perl|perld|perldo|po|pop|popu|popup|pp|ppop|pre|preserve|prev|previous|print|prof|profd|profdel|profile|promptf|promptfind|promptr|promptrepl|ps|psearch|ptN|ptNext|pta|ptag|ptf|ptfirst|ptj|ptjump|ptl|ptlast|ptn|ptnext|ptp|ptprevious|ptr|ptrewind|pts|ptselect|pu|put|pw|pwd|py|pyf|pyfile|python|q|qa|qall|quit|quita|quitall|r|read|rec|recover|red|redi|redir|redo|redr|redraw|redraws|redrawstatus|reg|registers|res|resize|ret|retab|retu|return|rew|rewind|ri|right|rightb|rightbelow|ru|rub|ruby|rubyd|rubydo|rubyf|rubyfile|runtime|rv|rviminfo|sN|sNext|sa|sal|sall|san|sandbox|sargument|sav|saveas|sb|sbN|sbNext|sba|sball|sbf|sbfirst|sbl|sblast|sbm|sbmodified|sbn|sbnext|sbp|sbprevious|sbr|sbrewind|sbuffer|scrip|scripte|scriptencoding|scriptnames|se|set|setf|setfiletype|setg|setglobal|setl|setlocal|sf|sfind|sfir|sfirst|sh|shell|sign|sil|silent|sim|simalt|sl|sla|slast|sleep|sm|smagic|smap|smapc|smapclear|sme|smenu|sn|snext|sni|sniff|sno|snomagic|snor|snoremap|snoreme|snoremenu|so|sor|sort|source|sp|spe|spelld|spelldump|spellgood|spelli|spellinfo|spellr|spellrepall|spellu|spellundo|spellw|spellwrong|split|spr|sprevious|sre|srewind|st|sta|stag|star|startg|startgreplace|startinsert|startr|startreplace|stj|stjump|stop|stopi|stopinsert|sts|stselect|sun|sunhide|sunm|sunmap|sus|suspend|sv|sview|syncbind|t|tN|tNext|ta|tab|tabN|tabNext|tabc|tabclose|tabd|tabdo|tabe|tabedit|tabf|tabfind|tabfir|tabfirst|tabl|tablast|tabm|tabmove|tabn|tabnew|tabnext|tabo|tabonly|tabp|tabprevious|tabr|tabrewind|tabs|tag|tags|tc|tcl|tcld|tcldo|tclf|tclfile|te|tearoff|tf|tfirst|th|throw|tj|tjump|tl|tlast|tm|tmenu|tn|tnext|to|topleft|tp|tprevious|tr|trewind|try|ts|tselect|tu|tunmenu|u|una|unabbreviate|undo|undoj|undojoin|undol|undolist|unh|unhide|unlet|unlo|unlockvar|unm|unmap|up|update|ve|verb|verbose|version|vert|vertical|vi|vie|view|vim|vimgrep|vimgrepa|vimgrepadd|visual|viu|viusage|vmapc|vmapclear|vne|vnew|vs|vsplit|vu|vunmap|w|wN|wNext|wa|wall|wh|while|win|winc|wincmd|windo|winp|winpos|winsize|wn|wnext|wp|wprevious|wq|wqa|wqall|write|ws|wsverb|wv|wviminfo|x|xa|xall|xit|xm|xmap|xmapc|xmapclear|xme|xmenu|xn|xnoremap|xnoreme|xnoremenu|xu|xunmap|y|yank)\b/,builtin:/\b(?:acd|ai|akm|aleph|allowrevins|altkeymap|ambiwidth|ambw|anti|antialias|arab|arabic|arabicshape|ari|arshape|autochdir|autocmd|autoindent|autoread|autowrite|autowriteall|aw|awa|background|backspace|backup|backupcopy|backupdir|backupext|backupskip|balloondelay|ballooneval|balloonexpr|bdir|bdlay|beval|bex|bexpr|bg|bh|bin|binary|biosk|bioskey|bk|bkc|bomb|breakat|brk|browsedir|bs|bsdir|bsk|bt|bufhidden|buflisted|buftype|casemap|ccv|cdpath|cedit|cfu|ch|charconvert|ci|cin|cindent|cink|cinkeys|cino|cinoptions|cinw|cinwords|clipboard|cmdheight|cmdwinheight|cmp|cms|columns|com|comments|commentstring|compatible|complete|completefunc|completeopt|consk|conskey|copyindent|cot|cpo|cpoptions|cpt|cscopepathcomp|cscopeprg|cscopequickfix|cscopetag|cscopetagorder|cscopeverbose|cspc|csprg|csqf|cst|csto|csverb|cuc|cul|cursorcolumn|cursorline|cwh|debug|deco|def|define|delcombine|dex|dg|dict|dictionary|diff|diffexpr|diffopt|digraph|dip|dir|directory|dy|ea|ead|eadirection|eb|ed|edcompatible|ef|efm|ei|ek|enc|encoding|endofline|eol|ep|equalalways|equalprg|errorbells|errorfile|errorformat|esckeys|et|eventignore|expandtab|exrc|fcl|fcs|fdc|fde|fdi|fdl|fdls|fdm|fdn|fdo|fdt|fen|fenc|fencs|fex|ff|ffs|fileencoding|fileencodings|fileformat|fileformats|fillchars|fk|fkmap|flp|fml|fmr|foldcolumn|foldenable|foldexpr|foldignore|foldlevel|foldlevelstart|foldmarker|foldmethod|foldminlines|foldnestmax|foldtext|formatexpr|formatlistpat|formatoptions|formatprg|fp|fs|fsync|ft|gcr|gd|gdefault|gfm|gfn|gfs|gfw|ghr|gp|grepformat|grepprg|gtl|gtt|guicursor|guifont|guifontset|guifontwide|guiheadroom|guioptions|guipty|guitablabel|guitabtooltip|helpfile|helpheight|helplang|hf|hh|hi|hidden|highlight|hk|hkmap|hkmapp|hkp|hl|hlg|hls|hlsearch|ic|icon|iconstring|ignorecase|im|imactivatekey|imak|imc|imcmdline|imd|imdisable|imi|iminsert|ims|imsearch|inc|include|includeexpr|incsearch|inde|indentexpr|indentkeys|indk|inex|inf|infercase|insertmode|invacd|invai|invakm|invallowrevins|invaltkeymap|invanti|invantialias|invar|invarab|invarabic|invarabicshape|invari|invarshape|invautochdir|invautoindent|invautoread|invautowrite|invautowriteall|invaw|invawa|invbackup|invballooneval|invbeval|invbin|invbinary|invbiosk|invbioskey|invbk|invbl|invbomb|invbuflisted|invcf|invci|invcin|invcindent|invcompatible|invconfirm|invconsk|invconskey|invcopyindent|invcp|invcscopetag|invcscopeverbose|invcst|invcsverb|invcuc|invcul|invcursorcolumn|invcursorline|invdeco|invdelcombine|invdg|invdiff|invdigraph|invdisable|invea|inveb|inved|invedcompatible|invek|invendofline|inveol|invequalalways|inverrorbells|invesckeys|invet|invex|invexpandtab|invexrc|invfen|invfk|invfkmap|invfoldenable|invgd|invgdefault|invguipty|invhid|invhidden|invhk|invhkmap|invhkmapp|invhkp|invhls|invhlsearch|invic|invicon|invignorecase|invim|invimc|invimcmdline|invimd|invincsearch|invinf|invinfercase|invinsertmode|invis|invjoinspaces|invjs|invlazyredraw|invlbr|invlinebreak|invlisp|invlist|invloadplugins|invlpl|invlz|invma|invmacatsui|invmagic|invmh|invml|invmod|invmodeline|invmodifiable|invmodified|invmore|invmousef|invmousefocus|invmousehide|invnu|invnumber|invodev|invopendevice|invpaste|invpi|invpreserveindent|invpreviewwindow|invprompt|invpvw|invreadonly|invremap|invrestorescreen|invrevins|invri|invrightleft|invrightleftcmd|invrl|invrlc|invro|invrs|invru|invruler|invsb|invsc|invscb|invscrollbind|invscs|invsecure|invsft|invshellslash|invshelltemp|invshiftround|invshortname|invshowcmd|invshowfulltag|invshowmatch|invshowmode|invsi|invsm|invsmartcase|invsmartindent|invsmarttab|invsmd|invsn|invsol|invspell|invsplitbelow|invsplitright|invspr|invsr|invssl|invsta|invstartofline|invstmp|invswapfile|invswf|invta|invtagbsearch|invtagrelative|invtagstack|invtbi|invtbidi|invtbs|invtermbidi|invterse|invtextauto|invtextmode|invtf|invtgst|invtildeop|invtimeout|invtitle|invto|invtop|invtr|invttimeout|invttybuiltin|invttyfast|invtx|invvb|invvisualbell|invwa|invwarn|invwb|invweirdinvert|invwfh|invwfw|invwildmenu|invwinfixheight|invwinfixwidth|invwiv|invwmnu|invwrap|invwrapscan|invwrite|invwriteany|invwritebackup|invws|isf|isfname|isi|isident|isk|iskeyword|isprint|joinspaces|js|key|keymap|keymodel|keywordprg|km|kmp|kp|langmap|langmenu|laststatus|lazyredraw|lbr|lcs|linebreak|lines|linespace|lisp|lispwords|listchars|loadplugins|lpl|lsp|lz|macatsui|magic|makeef|makeprg|matchpairs|matchtime|maxcombine|maxfuncdepth|maxmapdepth|maxmem|maxmempattern|maxmemtot|mco|mef|menuitems|mfd|mh|mis|mkspellmem|ml|mls|mm|mmd|mmp|mmt|modeline|modelines|modifiable|modified|more|mouse|mousef|mousefocus|mousehide|mousem|mousemodel|mouses|mouseshape|mouset|mousetime|mp|mps|msm|mzq|mzquantum|nf|noacd|noai|noakm|noallowrevins|noaltkeymap|noanti|noantialias|noar|noarab|noarabic|noarabicshape|noari|noarshape|noautochdir|noautoindent|noautoread|noautowrite|noautowriteall|noaw|noawa|nobackup|noballooneval|nobeval|nobin|nobinary|nobiosk|nobioskey|nobk|nobl|nobomb|nobuflisted|nocf|noci|nocin|nocindent|nocompatible|noconfirm|noconsk|noconskey|nocopyindent|nocp|nocscopetag|nocscopeverbose|nocst|nocsverb|nocuc|nocul|nocursorcolumn|nocursorline|nodeco|nodelcombine|nodg|nodiff|nodigraph|nodisable|noea|noeb|noed|noedcompatible|noek|noendofline|noeol|noequalalways|noerrorbells|noesckeys|noet|noex|noexpandtab|noexrc|nofen|nofk|nofkmap|nofoldenable|nogd|nogdefault|noguipty|nohid|nohidden|nohk|nohkmap|nohkmapp|nohkp|nohls|noic|noicon|noignorecase|noim|noimc|noimcmdline|noimd|noincsearch|noinf|noinfercase|noinsertmode|nois|nojoinspaces|nojs|nolazyredraw|nolbr|nolinebreak|nolisp|nolist|noloadplugins|nolpl|nolz|noma|nomacatsui|nomagic|nomh|noml|nomod|nomodeline|nomodifiable|nomodified|nomore|nomousef|nomousefocus|nomousehide|nonu|nonumber|noodev|noopendevice|nopaste|nopi|nopreserveindent|nopreviewwindow|noprompt|nopvw|noreadonly|noremap|norestorescreen|norevins|nori|norightleft|norightleftcmd|norl|norlc|noro|nors|noru|noruler|nosb|nosc|noscb|noscrollbind|noscs|nosecure|nosft|noshellslash|noshelltemp|noshiftround|noshortname|noshowcmd|noshowfulltag|noshowmatch|noshowmode|nosi|nosm|nosmartcase|nosmartindent|nosmarttab|nosmd|nosn|nosol|nospell|nosplitbelow|nosplitright|nospr|nosr|nossl|nosta|nostartofline|nostmp|noswapfile|noswf|nota|notagbsearch|notagrelative|notagstack|notbi|notbidi|notbs|notermbidi|noterse|notextauto|notextmode|notf|notgst|notildeop|notimeout|notitle|noto|notop|notr|nottimeout|nottybuiltin|nottyfast|notx|novb|novisualbell|nowa|nowarn|nowb|noweirdinvert|nowfh|nowfw|nowildmenu|nowinfixheight|nowinfixwidth|nowiv|nowmnu|nowrap|nowrapscan|nowrite|nowriteany|nowritebackup|nows|nrformats|numberwidth|nuw|odev|oft|ofu|omnifunc|opendevice|operatorfunc|opfunc|osfiletype|pa|para|paragraphs|paste|pastetoggle|patchexpr|patchmode|path|pdev|penc|pex|pexpr|pfn|ph|pheader|pi|pm|pmbcs|pmbfn|popt|preserveindent|previewheight|previewwindow|printdevice|printencoding|printexpr|printfont|printheader|printmbcharset|printmbfont|printoptions|prompt|pt|pumheight|pvh|pvw|qe|quoteescape|readonly|remap|report|restorescreen|revins|rightleft|rightleftcmd|rl|rlc|ro|rs|rtp|ruf|ruler|rulerformat|runtimepath|sbo|sc|scb|scr|scroll|scrollbind|scrolljump|scrolloff|scrollopt|scs|sect|sections|secure|sel|selection|selectmode|sessionoptions|sft|shcf|shellcmdflag|shellpipe|shellquote|shellredir|shellslash|shelltemp|shelltype|shellxquote|shiftround|shiftwidth|shm|shortmess|shortname|showbreak|showcmd|showfulltag|showmatch|showmode|showtabline|shq|si|sidescroll|sidescrolloff|siso|sj|slm|smartcase|smartindent|smarttab|smc|smd|softtabstop|sol|spc|spell|spellcapcheck|spellfile|spelllang|spellsuggest|spf|spl|splitbelow|splitright|sps|sr|srr|ss|ssl|ssop|stal|startofline|statusline|stl|stmp|su|sua|suffixes|suffixesadd|sw|swapfile|swapsync|swb|swf|switchbuf|sws|sxq|syn|synmaxcol|syntax|t_AB|t_AF|t_AL|t_CS|t_CV|t_Ce|t_Co|t_Cs|t_DL|t_EI|t_F1|t_F2|t_F3|t_F4|t_F5|t_F6|t_F7|t_F8|t_F9|t_IE|t_IS|t_K1|t_K3|t_K4|t_K5|t_K6|t_K7|t_K8|t_K9|t_KA|t_KB|t_KC|t_KD|t_KE|t_KF|t_KG|t_KH|t_KI|t_KJ|t_KK|t_KL|t_RI|t_RV|t_SI|t_Sb|t_Sf|t_WP|t_WS|t_ZH|t_ZR|t_al|t_bc|t_cd|t_ce|t_cl|t_cm|t_cs|t_da|t_db|t_dl|t_fs|t_k1|t_k2|t_k3|t_k4|t_k5|t_k6|t_k7|t_k8|t_k9|t_kB|t_kD|t_kI|t_kN|t_kP|t_kb|t_kd|t_ke|t_kh|t_kl|t_kr|t_ks|t_ku|t_le|t_mb|t_md|t_me|t_mr|t_ms|t_nd|t_op|t_se|t_so|t_sr|t_te|t_ti|t_ts|t_ue|t_us|t_ut|t_vb|t_ve|t_vi|t_vs|t_xs|tabline|tabpagemax|tabstop|tagbsearch|taglength|tagrelative|tagstack|tal|tb|tbi|tbidi|tbis|tbs|tenc|term|termbidi|termencoding|terse|textauto|textmode|textwidth|tgst|thesaurus|tildeop|timeout|timeoutlen|title|titlelen|titleold|titlestring|toolbar|toolbariconsize|top|tpm|tsl|tsr|ttimeout|ttimeoutlen|ttm|tty|ttybuiltin|ttyfast|ttym|ttymouse|ttyscroll|ttytype|tw|tx|uc|ul|undolevels|updatecount|updatetime|ut|vb|vbs|vdir|verbosefile|vfile|viewdir|viewoptions|viminfo|virtualedit|visualbell|vop|wak|warn|wb|wc|wcm|wd|weirdinvert|wfh|wfw|whichwrap|wi|wig|wildchar|wildcharm|wildignore|wildmenu|wildmode|wildoptions|wim|winaltkeys|window|winfixheight|winfixwidth|winheight|winminheight|winminwidth|winwidth|wiv|wiw|wm|wmh|wmnu|wmw|wop|wrap|wrapmargin|wrapscan|writeany|writebackup|writedelay|ww)\b/,number:/\b(?:0x[\da-f]+|\d+(?:\.\d+)?)\b/i,operator:/\|\||&&|[-+.]=?|[=!](?:[=~][#?]?)?|[<>]=?[#?]?|[*\/%?]|\b(?:is(?:not)?)\b/,punctuation:/[{}[\](),;:]/}}return p4}var f4,$W;function _st(){if($W)return f4;$W=1,f4=e,e.displayName="visualBasic",e.aliases=[];function e(t){t.languages["visual-basic"]={comment:{pattern:/(?:['‘’]|REM\b)(?:[^\r\n_]|_(?:\r\n?|\n)?)*/i,inside:{keyword:/^REM/i}},directive:{pattern:/#(?:Const|Else|ElseIf|End|ExternalChecksum|ExternalSource|If|Region)(?:\b_[ \t]*(?:\r\n?|\n)|.)+/i,alias:"property",greedy:!0},string:{pattern:/\$?["“”](?:["“”]{2}|[^"“”])*["“”]C?/i,greedy:!0},date:{pattern:/#[ \t]*(?:\d+([/-])\d+\1\d+(?:[ \t]+(?:\d+[ \t]*(?:AM|PM)|\d+:\d+(?::\d+)?(?:[ \t]*(?:AM|PM))?))?|\d+[ \t]*(?:AM|PM)|\d+:\d+(?::\d+)?(?:[ \t]*(?:AM|PM))?)[ \t]*#/i,alias:"number"},number:/(?:(?:\b\d+(?:\.\d+)?|\.\d+)(?:E[+-]?\d+)?|&[HO][\dA-F]+)(?:[FRD]|U?[ILS])?/i,boolean:/\b(?:False|Nothing|True)\b/i,keyword:/\b(?:AddHandler|AddressOf|Alias|And(?:Also)?|As|Boolean|ByRef|Byte|ByVal|Call|Case|Catch|C(?:Bool|Byte|Char|Date|Dbl|Dec|Int|Lng|Obj|SByte|Short|Sng|Str|Type|UInt|ULng|UShort)|Char|Class|Const|Continue|Currency|Date|Decimal|Declare|Default|Delegate|Dim|DirectCast|Do|Double|Each|Else(?:If)?|End(?:If)?|Enum|Erase|Error|Event|Exit|Finally|For|Friend|Function|Get(?:Type|XMLNamespace)?|Global|GoSub|GoTo|Handles|If|Implements|Imports|In|Inherits|Integer|Interface|Is|IsNot|Let|Lib|Like|Long|Loop|Me|Mod|Module|Must(?:Inherit|Override)|My(?:Base|Class)|Namespace|Narrowing|New|Next|Not(?:Inheritable|Overridable)?|Object|Of|On|Operator|Option(?:al)?|Or(?:Else)?|Out|Overloads|Overridable|Overrides|ParamArray|Partial|Private|Property|Protected|Public|RaiseEvent|ReadOnly|ReDim|RemoveHandler|Resume|Return|SByte|Select|Set|Shadows|Shared|short|Single|Static|Step|Stop|String|Structure|Sub|SyncLock|Then|Throw|To|Try|TryCast|Type|TypeOf|U(?:Integer|Long|Short)|Until|Using|Variant|Wend|When|While|Widening|With(?:Events)?|WriteOnly|Xor)\b/i,operator:/[+\-*/\\^<=>&#@$%!]|\b_(?=[ \t]*[\r\n])/,punctuation:/[{}().,:?]/},t.languages.vb=t.languages["visual-basic"],t.languages.vba=t.languages["visual-basic"]}return f4}var m4,GW;function Rst(){if(GW)return m4;GW=1,m4=e,e.displayName="warpscript",e.aliases=[];function e(t){t.languages.warpscript={comment:/#.*|\/\/.*|\/\*[\s\S]*?\*\//,string:{pattern:/"(?:[^"\\\r\n]|\\.)*"|'(?:[^'\\\r\n]|\\.)*'|<'(?:[^\\']|'(?!>)|\\.)*'>/,greedy:!0},variable:/\$\S+/,macro:{pattern:/@\S+/,alias:"property"},keyword:/\b(?:BREAK|CHECKMACRO|CONTINUE|CUDF|DEFINED|DEFINEDMACRO|EVAL|FAIL|FOR|FOREACH|FORSTEP|IFT|IFTE|MSGFAIL|NRETURN|RETHROW|RETURN|SWITCH|TRY|UDF|UNTIL|WHILE)\b/,number:/[+-]?\b(?:NaN|Infinity|\d+(?:\.\d*)?(?:[Ee][+-]?\d+)?|0x[\da-fA-F]+|0b[01]+)\b/,boolean:/\b(?:F|T|false|true)\b/,punctuation:/<%|%>|[{}[\]()]/,operator:/==|&&?|\|\|?|\*\*?|>>>?|<<|[<>!~]=?|[-/%^]|\+!?|\b(?:AND|NOT|OR)\b/}}return m4}var g4,jW;function Dst(){if(jW)return g4;jW=1,g4=e,e.displayName="wasm",e.aliases=[];function e(t){t.languages.wasm={comment:[/\(;[\s\S]*?;\)/,{pattern:/;;.*/,greedy:!0}],string:{pattern:/"(?:\\[\s\S]|[^"\\])*"/,greedy:!0},keyword:[{pattern:/\b(?:align|offset)=/,inside:{operator:/=/}},{pattern:/\b(?:(?:f32|f64|i32|i64)(?:\.(?:abs|add|and|ceil|clz|const|convert_[su]\/i(?:32|64)|copysign|ctz|demote\/f64|div(?:_[su])?|eqz?|extend_[su]\/i32|floor|ge(?:_[su])?|gt(?:_[su])?|le(?:_[su])?|load(?:(?:8|16|32)_[su])?|lt(?:_[su])?|max|min|mul|neg?|nearest|or|popcnt|promote\/f32|reinterpret\/[fi](?:32|64)|rem_[su]|rot[lr]|shl|shr_[su]|sqrt|store(?:8|16|32)?|sub|trunc(?:_[su]\/f(?:32|64))?|wrap\/i64|xor))?|memory\.(?:grow|size))\b/,inside:{punctuation:/\./}},/\b(?:anyfunc|block|br(?:_if|_table)?|call(?:_indirect)?|data|drop|elem|else|end|export|func|get_(?:global|local)|global|if|import|local|loop|memory|module|mut|nop|offset|param|result|return|select|set_(?:global|local)|start|table|tee_local|then|type|unreachable)\b/],variable:/\$[\w!#$%&'*+\-./:<=>?@\\^`|~]+/,number:/[+-]?\b(?:\d(?:_?\d)*(?:\.\d(?:_?\d)*)?(?:[eE][+-]?\d(?:_?\d)*)?|0x[\da-fA-F](?:_?[\da-fA-F])*(?:\.[\da-fA-F](?:_?[\da-fA-D])*)?(?:[pP][+-]?\d(?:_?\d)*)?)\b|\binf\b|\bnan(?::0x[\da-fA-F](?:_?[\da-fA-D])*)?\b/,punctuation:/[()]/}}return g4}var b4,qW;function Nst(){if(qW)return b4;qW=1,b4=e,e.displayName="webIdl",e.aliases=[];function e(t){(function(r){var n=/(?:\B-|\b_|\b)[A-Za-z][\w-]*(?![\w-])/.source,a="(?:"+/\b(?:unsigned\s+)?long\s+long(?![\w-])/.source+"|"+/\b(?:unrestricted|unsigned)\s+[a-z]+(?![\w-])/.source+"|"+/(?!(?:unrestricted|unsigned)\b)/.source+n+/(?:\s*<(?:[^<>]|<[^<>]*>)*>)?/.source+")"+/(?:\s*\?)?/.source,i={};r.languages["web-idl"]={comment:{pattern:/\/\/.*|\/\*[\s\S]*?\*\//,greedy:!0},string:{pattern:/"[^"]*"/,greedy:!0},namespace:{pattern:RegExp(/(\bnamespace\s+)/.source+n),lookbehind:!0},"class-name":[{pattern:/(^|[^\w-])(?:iterable|maplike|setlike)\s*<(?:[^<>]|<[^<>]*>)*>/,lookbehind:!0,inside:i},{pattern:RegExp(/(\b(?:attribute|const|deleter|getter|optional|setter)\s+)/.source+a),lookbehind:!0,inside:i},{pattern:RegExp("("+/\bcallback\s+/.source+n+/\s*=\s*/.source+")"+a),lookbehind:!0,inside:i},{pattern:RegExp(/(\btypedef\b\s*)/.source+a),lookbehind:!0,inside:i},{pattern:RegExp(/(\b(?:callback|dictionary|enum|interface(?:\s+mixin)?)\s+)(?!(?:interface|mixin)\b)/.source+n),lookbehind:!0},{pattern:RegExp(/(:\s*)/.source+n),lookbehind:!0},RegExp(n+/(?=\s+(?:implements|includes)\b)/.source),{pattern:RegExp(/(\b(?:implements|includes)\s+)/.source+n),lookbehind:!0},{pattern:RegExp(a+"(?="+/\s*(?:\.{3}\s*)?/.source+n+/\s*[(),;=]/.source+")"),inside:i}],builtin:/\b(?:ArrayBuffer|BigInt64Array|BigUint64Array|ByteString|DOMString|DataView|Float32Array|Float64Array|FrozenArray|Int16Array|Int32Array|Int8Array|ObservableArray|Promise|USVString|Uint16Array|Uint32Array|Uint8Array|Uint8ClampedArray)\b/,keyword:[/\b(?:async|attribute|callback|const|constructor|deleter|dictionary|enum|getter|implements|includes|inherit|interface|mixin|namespace|null|optional|or|partial|readonly|required|setter|static|stringifier|typedef|unrestricted)\b/,/\b(?:any|bigint|boolean|byte|double|float|iterable|long|maplike|object|octet|record|sequence|setlike|short|symbol|undefined|unsigned|void)\b/],boolean:/\b(?:false|true)\b/,number:{pattern:/(^|[^\w-])-?(?:0x[0-9a-f]+|(?:\d+(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?|NaN|Infinity)(?![\w-])/i,lookbehind:!0},operator:/\.{3}|[=:?<>-]/,punctuation:/[(){}[\].,;]/};for(var o in r.languages["web-idl"])o!=="class-name"&&(i[o]=r.languages["web-idl"][o]);r.languages.webidl=r.languages["web-idl"]})(t)}return b4}var y4,WW;function Lst(){if(WW)return y4;WW=1,y4=e,e.displayName="wiki",e.aliases=[];function e(t){t.languages.wiki=t.languages.extend("markup",{"block-comment":{pattern:/(^|[^\\])\/\*[\s\S]*?\*\//,lookbehind:!0,alias:"comment"},heading:{pattern:/^(=+)[^=\r\n].*?\1/m,inside:{punctuation:/^=+|=+$/,important:/.+/}},emphasis:{pattern:/('{2,5}).+?\1/,inside:{"bold-italic":{pattern:/(''''').+?(?=\1)/,lookbehind:!0,alias:["bold","italic"]},bold:{pattern:/(''')[^'](?:.*?[^'])?(?=\1)/,lookbehind:!0},italic:{pattern:/('')[^'](?:.*?[^'])?(?=\1)/,lookbehind:!0},punctuation:/^''+|''+$/}},hr:{pattern:/^-{4,}/m,alias:"punctuation"},url:[/ISBN +(?:97[89][ -]?)?(?:\d[ -]?){9}[\dx]\b|(?:PMID|RFC) +\d+/i,/\[\[.+?\]\]|\[.+?\]/],variable:[/__[A-Z]+__/,/\{{3}.+?\}{3}/,/\{\{.+?\}\}/],symbol:[/^#redirect/im,/~{3,5}/],"table-tag":{pattern:/((?:^|[|!])[|!])[^|\r\n]+\|(?!\|)/m,lookbehind:!0,inside:{"table-bar":{pattern:/\|$/,alias:"punctuation"},rest:t.languages.markup.tag.inside}},punctuation:/^(?:\{\||\|\}|\|-|[*#:;!|])|\|\||!!/m}),t.languages.insertBefore("wiki","tag",{nowiki:{pattern:/<(nowiki|pre|source)\b[^>]*>[\s\S]*?<\/\1>/i,inside:{tag:{pattern:/<(?:nowiki|pre|source)\b[^>]*>|<\/(?:nowiki|pre|source)>/i,inside:t.languages.markup.tag.inside}}}})}return y4}var v4,VW;function Mst(){if(VW)return v4;VW=1,v4=e,e.displayName="wolfram",e.aliases=["mathematica","wl","nb"];function e(t){t.languages.wolfram={comment:/\(\*(?:\(\*(?:[^*]|\*(?!\)))*\*\)|(?!\(\*)[\s\S])*?\*\)/,string:{pattern:/"(?:\\.|[^"\\\r\n])*"/,greedy:!0},keyword:/\b(?:Abs|AbsArg|Accuracy|Block|Do|For|Function|If|Manipulate|Module|Nest|NestList|None|Return|Switch|Table|Which|While)\b/,context:{pattern:/\b\w+`+\w*/,alias:"class-name"},blank:{pattern:/\b\w+_\b/,alias:"regex"},"global-variable":{pattern:/\$\w+/,alias:"variable"},boolean:/\b(?:False|True)\b/,number:/(?:\b(?=\d)|\B(?=\.))(?:0[bo])?(?:(?:\d|0x[\da-f])[\da-f]*(?:\.\d*)?|\.\d+)(?:e[+-]?\d+)?j?\b/i,operator:/\/\.|;|=\.|\^=|\^:=|:=|<<|>>|<\||\|>|:>|\|->|->|<-|@@@|@@|@|\/@|=!=|===|==|=|\+|-|\^|\[\/-+%=\]=?|!=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,punctuation:/[{}[\];(),.:]/},t.languages.mathematica=t.languages.wolfram,t.languages.wl=t.languages.wolfram,t.languages.nb=t.languages.wolfram}return v4}var S4,KW;function Ist(){if(KW)return S4;KW=1,S4=e,e.displayName="wren",e.aliases=[];function e(t){t.languages.wren={comment:[{pattern:/\/\*(?:[^*/]|\*(?!\/)|\/(?!\*)|\/\*(?:[^*/]|\*(?!\/)|\/(?!\*)|\/\*(?:[^*/]|\*(?!\/)|\/(?!\*))*\*\/)*\*\/)*\*\//,greedy:!0},{pattern:/(^|[^\\:])\/\/.*/,lookbehind:!0,greedy:!0}],"triple-quoted-string":{pattern:/"""[\s\S]*?"""/,greedy:!0,alias:"string"},"string-literal":null,hashbang:{pattern:/^#!\/.+/,greedy:!0,alias:"comment"},attribute:{pattern:/#!?[ \t\u3000]*\w+/,alias:"keyword"},"class-name":[{pattern:/(\bclass\s+)\w+/,lookbehind:!0},/\b[A-Z][a-z\d_]*\b/],constant:/\b[A-Z][A-Z\d_]*\b/,null:{pattern:/\bnull\b/,alias:"keyword"},keyword:/\b(?:as|break|class|construct|continue|else|for|foreign|if|import|in|is|return|static|super|this|var|while)\b/,boolean:/\b(?:false|true)\b/,number:/\b(?:0x[\da-f]+|\d+(?:\.\d+)?(?:e[+-]?\d+)?)\b/i,function:/\b[a-z_]\w*(?=\s*[({])/i,operator:/<<|>>|[=!<>]=?|&&|\|\||[-+*/%~^&|?:]|\.{2,3}/,punctuation:/[\[\](){}.,;]/},t.languages.wren["string-literal"]={pattern:/(^|[^\\"])"(?:[^\\"%]|\\[\s\S]|%(?!\()|%\((?:[^()]|\((?:[^()]|\([^)]*\))*\))*\))*"/,lookbehind:!0,greedy:!0,inside:{interpolation:{pattern:/((?:^|[^\\])(?:\\{2})*)%\((?:[^()]|\((?:[^()]|\([^)]*\))*\))*\)/,lookbehind:!0,inside:{expression:{pattern:/^(%\()[\s\S]+(?=\)$)/,lookbehind:!0,inside:t.languages.wren},"interpolation-punctuation":{pattern:/^%\(|\)$/,alias:"punctuation"}}},string:/[\s\S]+/}}}return S4}var E4,YW;function Ost(){if(YW)return E4;YW=1,E4=e,e.displayName="xeora",e.aliases=["xeoracube"];function e(t){(function(r){r.languages.xeora=r.languages.extend("markup",{constant:{pattern:/\$(?:DomainContents|PageRenderDuration)\$/,inside:{punctuation:{pattern:/\$/}}},variable:{pattern:/\$@?(?:#+|[-+*~=^])?[\w.]+\$/,inside:{punctuation:{pattern:/[$.]/},operator:{pattern:/#+|[-+*~=^@]/}}},"function-inline":{pattern:/\$F:[-\w.]+\?[-\w.]+(?:,(?:(?:@[-#]*\w+\.[\w+.]\.*)*\|)*(?:(?:[\w+]|[-#*.~^]+[\w+]|=\S)(?:[^$=]|=+[^=])*=*|(?:@[-#]*\w+\.[\w+.]\.*)+(?:(?:[\w+]|[-#*~^][-#*.~^]*[\w+]|=\S)(?:[^$=]|=+[^=])*=*)?)?)?\$/,inside:{variable:{pattern:/(?:[,|])@?(?:#+|[-+*~=^])?[\w.]+/,inside:{punctuation:{pattern:/[,.|]/},operator:{pattern:/#+|[-+*~=^@]/}}},punctuation:{pattern:/\$\w:|[$:?.,|]/}},alias:"function"},"function-block":{pattern:/\$XF:\{[-\w.]+\?[-\w.]+(?:,(?:(?:@[-#]*\w+\.[\w+.]\.*)*\|)*(?:(?:[\w+]|[-#*.~^]+[\w+]|=\S)(?:[^$=]|=+[^=])*=*|(?:@[-#]*\w+\.[\w+.]\.*)+(?:(?:[\w+]|[-#*~^][-#*.~^]*[\w+]|=\S)(?:[^$=]|=+[^=])*=*)?)?)?\}:XF\$/,inside:{punctuation:{pattern:/[$:{}?.,|]/}},alias:"function"},"directive-inline":{pattern:/\$\w(?:#\d+\+?)?(?:\[[-\w.]+\])?:[-\/\w.]+\$/,inside:{punctuation:{pattern:/\$(?:\w:|C(?:\[|#\d))?|[:{[\]]/,inside:{tag:{pattern:/#\d/}}}},alias:"function"},"directive-block-open":{pattern:/\$\w+:\{|\$\w(?:#\d+\+?)?(?:\[[-\w.]+\])?:[-\w.]+:\{(?:![A-Z]+)?/,inside:{punctuation:{pattern:/\$(?:\w:|C(?:\[|#\d))?|[:{[\]]/,inside:{tag:{pattern:/#\d/}}},attribute:{pattern:/![A-Z]+$/,inside:{punctuation:{pattern:/!/}},alias:"keyword"}},alias:"function"},"directive-block-separator":{pattern:/\}:[-\w.]+:\{/,inside:{punctuation:{pattern:/[:{}]/}},alias:"function"},"directive-block-close":{pattern:/\}:[-\w.]+\$/,inside:{punctuation:{pattern:/[:{}$]/}},alias:"function"}}),r.languages.insertBefore("inside","punctuation",{variable:r.languages.xeora["function-inline"].inside.variable},r.languages.xeora["function-block"]),r.languages.xeoracube=r.languages.xeora})(t)}return E4}var x4,JW;function Bst(){if(JW)return x4;JW=1,x4=e,e.displayName="xmlDoc",e.aliases=[];function e(t){(function(r){function n(s,l){r.languages[s]&&r.languages.insertBefore(s,"comment",{"doc-comment":l})}var a=r.languages.markup.tag,i={pattern:/\/\/\/.*/,greedy:!0,alias:"comment",inside:{tag:a}},o={pattern:/'''.*/,greedy:!0,alias:"comment",inside:{tag:a}};n("csharp",i),n("fsharp",i),n("vbnet",o)})(t)}return x4}var w4,XW;function Pst(){if(XW)return w4;XW=1,w4=e,e.displayName="xojo",e.aliases=[];function e(t){t.languages.xojo={comment:{pattern:/(?:'|\/\/|Rem\b).+/i,greedy:!0},string:{pattern:/"(?:""|[^"])*"/,greedy:!0},number:[/(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:E[+-]?\d+)?/i,/&[bchou][a-z\d]+/i],directive:{pattern:/#(?:Else|ElseIf|Endif|If|Pragma)\b/i,alias:"property"},keyword:/\b(?:AddHandler|App|Array|As(?:signs)?|Auto|Boolean|Break|By(?:Ref|Val)|Byte|Call|Case|Catch|CFStringRef|CGFloat|Class|Color|Const|Continue|CString|Currency|CurrentMethodName|Declare|Delegate|Dim|Do(?:uble|wnTo)?|Each|Else(?:If)?|End|Enumeration|Event|Exception|Exit|Extends|False|Finally|For|Function|Get|GetTypeInfo|Global|GOTO|If|Implements|In|Inherits|Int(?:8|16|32|64|eger|erface)?|Lib|Loop|Me|Module|Next|Nil|Object|Optional|OSType|ParamArray|Private|Property|Protected|PString|Ptr|Raise(?:Event)?|ReDim|RemoveHandler|Return|Select(?:or)?|Self|Set|Shared|Short|Single|Soft|Static|Step|String|Sub|Super|Text|Then|To|True|Try|Ubound|UInt(?:8|16|32|64|eger)?|Until|Using|Var(?:iant)?|Wend|While|WindowPtr|WString)\b/i,operator:/<[=>]?|>=?|[+\-*\/\\^=]|\b(?:AddressOf|And|Ctype|IsA?|Mod|New|Not|Or|WeakAddressOf|Xor)\b/i,punctuation:/[.,;:()]/}}return w4}var k4,ZW;function zst(){if(ZW)return k4;ZW=1,k4=e,e.displayName="xquery",e.aliases=[];function e(t){(function(r){r.languages.xquery=r.languages.extend("markup",{"xquery-comment":{pattern:/\(:[\s\S]*?:\)/,greedy:!0,alias:"comment"},string:{pattern:/(["'])(?:\1\1|(?!\1)[\s\S])*\1/,greedy:!0},extension:{pattern:/\(#.+?#\)/,alias:"symbol"},variable:/\$[-\w:]+/,axis:{pattern:/(^|[^-])(?:ancestor(?:-or-self)?|attribute|child|descendant(?:-or-self)?|following(?:-sibling)?|parent|preceding(?:-sibling)?|self)(?=::)/,lookbehind:!0,alias:"operator"},"keyword-operator":{pattern:/(^|[^:-])\b(?:and|castable as|div|eq|except|ge|gt|idiv|instance of|intersect|is|le|lt|mod|ne|or|union)\b(?=$|[^:-])/,lookbehind:!0,alias:"operator"},keyword:{pattern:/(^|[^:-])\b(?:as|ascending|at|base-uri|boundary-space|case|cast as|collation|construction|copy-namespaces|declare|default|descending|else|empty (?:greatest|least)|encoding|every|external|for|function|if|import|in|inherit|lax|let|map|module|namespace|no-inherit|no-preserve|option|order(?: by|ed|ing)?|preserve|return|satisfies|schema|some|stable|strict|strip|then|to|treat as|typeswitch|unordered|validate|variable|version|where|xquery)\b(?=$|[^:-])/,lookbehind:!0},function:/[\w-]+(?::[\w-]+)*(?=\s*\()/,"xquery-element":{pattern:/(element\s+)[\w-]+(?::[\w-]+)*/,lookbehind:!0,alias:"tag"},"xquery-attribute":{pattern:/(attribute\s+)[\w-]+(?::[\w-]+)*/,lookbehind:!0,alias:"attr-name"},builtin:{pattern:/(^|[^:-])\b(?:attribute|comment|document|element|processing-instruction|text|xs:(?:ENTITIES|ENTITY|ID|IDREFS?|NCName|NMTOKENS?|NOTATION|Name|QName|anyAtomicType|anyType|anyURI|base64Binary|boolean|byte|date|dateTime|dayTimeDuration|decimal|double|duration|float|gDay|gMonth|gMonthDay|gYear|gYearMonth|hexBinary|int|integer|language|long|negativeInteger|nonNegativeInteger|nonPositiveInteger|normalizedString|positiveInteger|short|string|time|token|unsigned(?:Byte|Int|Long|Short)|untyped(?:Atomic)?|yearMonthDuration))\b(?=$|[^:-])/,lookbehind:!0},number:/\b\d+(?:\.\d+)?(?:E[+-]?\d+)?/,operator:[/[+*=?|@]|\.\.?|:=|!=|<[=<]?|>[=>]?/,{pattern:/(\s)-(?=\s)/,lookbehind:!0}],punctuation:/[[\](){},;:/]/}),r.languages.xquery.tag.pattern=/<\/?(?!\d)[^\s>\/=$<%]+(?:\s+[^\s>\/=]+(?:=(?:("|')(?:\\[\s\S]|\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}|(?!\1)[^\\])*\1|[^\s'">=]+))?)*\s*\/?>/,r.languages.xquery.tag.inside["attr-value"].pattern=/=(?:("|')(?:\\[\s\S]|\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}|(?!\1)[^\\])*\1|[^\s'">=]+)/,r.languages.xquery.tag.inside["attr-value"].inside.punctuation=/^="|"$/,r.languages.xquery.tag.inside["attr-value"].inside.expression={pattern:/\{(?!\{)(?:\{(?:\{[^{}]*\}|[^{}])*\}|[^{}])+\}/,inside:r.languages.xquery,alias:"language-xquery"};var n=function(i){return typeof i=="string"?i:typeof i.content=="string"?i.content:i.content.map(n).join("")},a=function(i){for(var o=[],s=0;s<i.length;s++){var l=i[s],u=!1;if(typeof l!="string"&&(l.type==="tag"&&l.content[0]&&l.content[0].type==="tag"?l.content[0].content[0].content==="</"?o.length>0&&o[o.length-1].tagName===n(l.content[0].content[1])&&o.pop():l.content[l.content.length-1].content==="/>"||o.push({tagName:n(l.content[0].content[1]),openedBraces:0}):o.length>0&&l.type==="punctuation"&&l.content==="{"&&(!i[s+1]||i[s+1].type!=="punctuation"||i[s+1].content!=="{")&&(!i[s-1]||i[s-1].type!=="plain-text"||i[s-1].content!=="{")?o[o.length-1].openedBraces++:o.length>0&&o[o.length-1].openedBraces>0&&l.type==="punctuation"&&l.content==="}"?o[o.length-1].openedBraces--:l.type!=="comment"&&(u=!0)),(u||typeof l=="string")&&o.length>0&&o[o.length-1].openedBraces===0){var d=n(l);s<i.length-1&&(typeof i[s+1]=="string"||i[s+1].type==="plain-text")&&(d+=n(i[s+1]),i.splice(s+1,1)),s>0&&(typeof i[s-1]=="string"||i[s-1].type==="plain-text")&&(d=n(i[s-1])+d,i.splice(s-1,1),s--),/^\s+$/.test(d)?i[s]=d:i[s]=new r.Token("plain-text",d,null,d)}l.content&&typeof l.content!="string"&&a(l.content)}};r.hooks.add("after-tokenize",function(i){i.language==="xquery"&&a(i.tokens)})})(t)}return k4}var C4,QW;function Hst(){if(QW)return C4;QW=1,C4=e,e.displayName="yang",e.aliases=[];function e(t){t.languages.yang={comment:/\/\*[\s\S]*?\*\/|\/\/.*/,string:{pattern:/"(?:[^\\"]|\\.)*"|'[^']*'/,greedy:!0},keyword:{pattern:/(^|[{};\r\n][ \t]*)[a-z_][\w.-]*/i,lookbehind:!0},namespace:{pattern:/(\s)[a-z_][\w.-]*(?=:)/i,lookbehind:!0},boolean:/\b(?:false|true)\b/,operator:/\+/,punctuation:/[{};:]/}}return C4}var T4,eV;function Ust(){if(eV)return T4;eV=1,T4=e,e.displayName="zig",e.aliases=[];function e(t){(function(r){function n(d){return function(){return d}}var a=/\b(?:align|allowzero|and|anyframe|anytype|asm|async|await|break|cancel|catch|comptime|const|continue|defer|else|enum|errdefer|error|export|extern|fn|for|if|inline|linksection|nakedcc|noalias|nosuspend|null|or|orelse|packed|promise|pub|resume|return|stdcallcc|struct|suspend|switch|test|threadlocal|try|undefined|union|unreachable|usingnamespace|var|volatile|while)\b/,i="\\b(?!"+a.source+")(?!\\d)\\w+\\b",o=/align\s*\((?:[^()]|\([^()]*\))*\)/.source,s=/(?:\?|\bpromise->|(?:\[[^[\]]*\]|\*(?!\*)|\*\*)(?:\s*<ALIGN>|\s*const\b|\s*volatile\b|\s*allowzero\b)*)/.source.replace(/<ALIGN>/g,n(o)),l=/(?:\bpromise\b|(?:\berror\.)?<ID>(?:\.<ID>)*(?!\s+<ID>))/.source.replace(/<ID>/g,n(i)),u="(?!\\s)(?:!?\\s*(?:"+s+"\\s*)*"+l+")+";r.languages.zig={comment:[{pattern:/\/\/[/!].*/,alias:"doc-comment"},/\/{2}.*/],string:[{pattern:/(^|[^\\@])c?"(?:[^"\\\r\n]|\\.)*"/,lookbehind:!0,greedy:!0},{pattern:/([\r\n])([ \t]+c?\\{2}).*(?:(?:\r\n?|\n)\2.*)*/,lookbehind:!0,greedy:!0}],char:{pattern:/(^|[^\\])'(?:[^'\\\r\n]|[\uD800-\uDFFF]{2}|\\(?:.|x[a-fA-F\d]{2}|u\{[a-fA-F\d]{1,6}\}))'/,lookbehind:!0,greedy:!0},builtin:/\B@(?!\d)\w+(?=\s*\()/,label:{pattern:/(\b(?:break|continue)\s*:\s*)\w+\b|\b(?!\d)\w+\b(?=\s*:\s*(?:\{|while\b))/,lookbehind:!0},"class-name":[/\b(?!\d)\w+(?=\s*=\s*(?:(?:extern|packed)\s+)?(?:enum|struct|union)\s*[({])/,{pattern:RegExp(/(:\s*)<TYPE>(?=\s*(?:<ALIGN>\s*)?[=;,)])|<TYPE>(?=\s*(?:<ALIGN>\s*)?\{)/.source.replace(/<TYPE>/g,n(u)).replace(/<ALIGN>/g,n(o))),lookbehind:!0,inside:null},{pattern:RegExp(/(\)\s*)<TYPE>(?=\s*(?:<ALIGN>\s*)?;)/.source.replace(/<TYPE>/g,n(u)).replace(/<ALIGN>/g,n(o))),lookbehind:!0,inside:null}],"builtin-type":{pattern:/\b(?:anyerror|bool|c_u?(?:int|long|longlong|short)|c_longdouble|c_void|comptime_(?:float|int)|f(?:16|32|64|128)|[iu](?:8|16|32|64|128|size)|noreturn|type|void)\b/,alias:"keyword"},keyword:a,function:/\b(?!\d)\w+(?=\s*\()/,number:/\b(?:0b[01]+|0o[0-7]+|0x[a-fA-F\d]+(?:\.[a-fA-F\d]*)?(?:[pP][+-]?[a-fA-F\d]+)?|\d+(?:\.\d*)?(?:[eE][+-]?\d+)?)\b/,boolean:/\b(?:false|true)\b/,operator:/\.[*?]|\.{2,3}|[-=]>|\*\*|\+\+|\|\||(?:<<|>>|[-+*]%|[-+*/%^&|<>!=])=?|[?~]/,punctuation:/[.:,;(){}[\]]/},r.languages.zig["class-name"].forEach(function(d){d.inside===null&&(d.inside=r.languages.zig)})})(t)}return T4}var A4,tV;function $st(){if(tV)return A4;tV=1;var e=ant();return A4=e,e.register(ont()),e.register(snt()),e.register(lnt()),e.register(unt()),e.register(cnt()),e.register(dnt()),e.register(hnt()),e.register(pnt()),e.register(fnt()),e.register(mnt()),e.register(gnt()),e.register(bnt()),e.register(ynt()),e.register(vnt()),e.register(Snt()),e.register(Ent()),e.register(xnt()),e.register(wnt()),e.register(knt()),e.register(Cnt()),e.register(Tnt()),e.register(Ant()),e.register(Mce()),e.register(Ice()),e.register(Fnt()),e.register(_nt()),e.register(Rnt()),e.register(Dnt()),e.register(Nnt()),e.register(Lnt()),e.register(Mnt()),e.register(Int()),e.register(Ont()),e.register(Bnt()),e.register(pd()),e.register(Pnt()),e.register(znt()),e.register(Hnt()),e.register(Unt()),e.register($nt()),e.register(Gnt()),e.register(jnt()),e.register(qnt()),e.register(Wnt()),e.register(pL()),e.register(Vnt()),e.register(rS()),e.register(Knt()),e.register(Ynt()),e.register(Jnt()),e.register(Xnt()),e.register(Znt()),e.register(Qnt()),e.register(eat()),e.register(tat()),e.register(rat()),e.register(nat()),e.register(aat()),e.register(iat()),e.register(oat()),e.register(sat()),e.register(lat()),e.register(uat()),e.register(cat()),e.register(dat()),e.register(hat()),e.register(pat()),e.register(fat()),e.register(mat()),e.register(gat()),e.register(bat()),e.register(yat()),e.register(vat()),e.register(Sat()),e.register(Eat()),e.register(xat()),e.register(wat()),e.register(kat()),e.register(Cat()),e.register(Tat()),e.register(Aat()),e.register(Fat()),e.register(_at()),e.register(Rat()),e.register(Dat()),e.register(Nat()),e.register(Lat()),e.register(Mat()),e.register(Iat()),e.register(Oat()),e.register(Bat()),e.register(Pat()),e.register(zat()),e.register(Hat()),e.register(fL()),e.register(Uat()),e.register($at()),e.register(Gat()),e.register(jat()),e.register(qat()),e.register(Wat()),e.register(Vat()),e.register(Kat()),e.register(Yat()),e.register(Jat()),e.register(Xat()),e.register(Zat()),e.register(Qat()),e.register(eit()),e.register(tit()),e.register(rit()),e.register(nit()),e.register(mL()),e.register(ait()),e.register(aS()),e.register(iit()),e.register(oit()),e.register(sit()),e.register(lit()),e.register(uit()),e.register(cit()),e.register(dit()),e.register(bL()),e.register(hit()),e.register(pit()),e.register(fit()),e.register(Bce()),e.register(mit()),e.register(git()),e.register(bit()),e.register(yit()),e.register(vit()),e.register(Sit()),e.register(Eit()),e.register(xit()),e.register(wit()),e.register(kit()),e.register(Cit()),e.register(Tit()),e.register(Ait()),e.register(Fit()),e.register(_it()),e.register(Rit()),e.register(Oce()),e.register(Dit()),e.register(Nit()),e.register(Lit()),e.register(Si()),e.register(Mit()),e.register(Iit()),e.register(Oit()),e.register(Bit()),e.register(Pit()),e.register(zit()),e.register(Hit()),e.register(Uit()),e.register($it()),e.register(Git()),e.register(jit()),e.register(qit()),e.register(Wit()),e.register(Vit()),e.register(Kit()),e.register(Yit()),e.register(Jit()),e.register(Xit()),e.register(Zit()),e.register(Qit()),e.register(eot()),e.register(tot()),e.register(rot()),e.register(not()),e.register(aot()),e.register(iot()),e.register(oot()),e.register(sot()),e.register(lot()),e.register(uot()),e.register(cot()),e.register(dot()),e.register(iS()),e.register(hot()),e.register(pot()),e.register(fot()),e.register(mot()),e.register(got()),e.register(bot()),e.register(yot()),e.register(vot()),e.register(Sot()),e.register(Eot()),e.register(xot()),e.register(wot()),e.register(kot()),e.register(Cot()),e.register(Tot()),e.register(Aot()),e.register(Fot()),e.register(_ot()),e.register(Rot()),e.register(Dot()),e.register(Not()),e.register(Lot()),e.register(Mot()),e.register(Iot()),e.register(Oot()),e.register(Bot()),e.register(Pot()),e.register(zot()),e.register(Hot()),e.register(Uot()),e.register(nS()),e.register($ot()),e.register(Got()),e.register(jot()),e.register(qot()),e.register(yL()),e.register(Wot()),e.register(Vot()),e.register(Kot()),e.register(Yot()),e.register(Jot()),e.register(Xot()),e.register(Zot()),e.register(Qot()),e.register(est()),e.register(tst()),e.register(rst()),e.register(nst()),e.register(hL()),e.register(ast()),e.register(ist()),e.register(ost()),e.register(sst()),e.register(lst()),e.register(ust()),e.register(vL()),e.register(cst()),e.register(dst()),e.register(hst()),e.register(pst()),e.register(fst()),e.register(mst()),e.register(gst()),e.register(bst()),e.register(Pce()),e.register(yst()),e.register(gL()),e.register(vst()),e.register(Sst()),e.register(Est()),e.register(xst()),e.register(wst()),e.register(kst()),e.register(zce()),e.register(Cst()),e.register(Tst()),e.register(Ast()),e.register(Fst()),e.register(_st()),e.register(Rst()),e.register(Dst()),e.register(Nst()),e.register(Lst()),e.register(Mst()),e.register(Ist()),e.register(Ost()),e.register(Bst()),e.register(Pst()),e.register(zst()),e.register(Hce()),e.register(Hst()),e.register(Ust()),A4}var Gst=$st();const jst=jn(Gst);var eD=Ret(jst,int);eD.supportedLanguages=Det;var F4={},_4={exports:{}},rV;function qst(){return rV||(rV=1,(function(e){function t(r){return r&&r.__esModule?r:{default:r}}e.exports=t,e.exports.__esModule=!0,e.exports.default=e.exports})(_4)),_4.exports}var R4={},nV;function Wst(){return nV||(nV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"black",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",maxHeight:"inherit",height:"inherit",padding:"0 1em",display:"block",overflow:"auto"},'pre[class*="language-"]':{color:"black",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",position:"relative",margin:".5em 0",overflow:"visible",padding:"1px",backgroundColor:"#fdfdfd",WebkitBoxSizing:"border-box",MozBoxSizing:"border-box",boxSizing:"border-box",marginBottom:"1em"},'pre[class*="language-"] > code':{position:"relative",zIndex:"1",borderLeft:"10px solid #358ccb",boxShadow:"-1px 0px 0px 0px #358ccb, 0px 0px 0px 1px #dfdfdf",backgroundColor:"#fdfdfd",backgroundImage:"linear-gradient(transparent 50%, rgba(69, 142, 209, 0.04) 50%)",backgroundSize:"3em 3em",backgroundOrigin:"content-box",backgroundAttachment:"local"},':not(pre) > code[class*="language-"]':{backgroundColor:"#fdfdfd",WebkitBoxSizing:"border-box",MozBoxSizing:"border-box",boxSizing:"border-box",marginBottom:"1em",position:"relative",padding:".2em",borderRadius:"0.3em",color:"#c92c2c",border:"1px solid rgba(0, 0, 0, 0.1)",display:"inline",whiteSpace:"normal"},'pre[class*="language-"]:before':{content:"''",display:"block",position:"absolute",bottom:"0.75em",left:"0.18em",width:"40%",height:"20%",maxHeight:"13em",boxShadow:"0px 13px 8px #979797",WebkitTransform:"rotate(-2deg)",MozTransform:"rotate(-2deg)",msTransform:"rotate(-2deg)",OTransform:"rotate(-2deg)",transform:"rotate(-2deg)"},'pre[class*="language-"]:after':{content:"''",display:"block",position:"absolute",bottom:"0.75em",left:"auto",width:"40%",height:"20%",maxHeight:"13em",boxShadow:"0px 13px 8px #979797",WebkitTransform:"rotate(2deg)",MozTransform:"rotate(2deg)",msTransform:"rotate(2deg)",OTransform:"rotate(2deg)",transform:"rotate(2deg)",right:"0.75em"},comment:{color:"#7D8B99"},"block-comment":{color:"#7D8B99"},prolog:{color:"#7D8B99"},doctype:{color:"#7D8B99"},cdata:{color:"#7D8B99"},punctuation:{color:"#5F6364"},property:{color:"#c92c2c"},tag:{color:"#c92c2c"},boolean:{color:"#c92c2c"},number:{color:"#c92c2c"},"function-name":{color:"#c92c2c"},constant:{color:"#c92c2c"},symbol:{color:"#c92c2c"},deleted:{color:"#c92c2c"},selector:{color:"#2f9c0a"},"attr-name":{color:"#2f9c0a"},string:{color:"#2f9c0a"},char:{color:"#2f9c0a"},function:{color:"#2f9c0a"},builtin:{color:"#2f9c0a"},inserted:{color:"#2f9c0a"},operator:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},entity:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)",cursor:"help"},url:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},variable:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},atrule:{color:"#1990b8"},"attr-value":{color:"#1990b8"},keyword:{color:"#1990b8"},"class-name":{color:"#1990b8"},regex:{color:"#e90"},important:{color:"#e90",fontWeight:"normal"},".language-css .token.string":{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},".style .token.string":{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},namespace:{Opacity:".7"},'pre[class*="language-"].line-numbers.line-numbers':{paddingLeft:"0"},'pre[class*="language-"].line-numbers.line-numbers code':{paddingLeft:"3.8em"},'pre[class*="language-"].line-numbers.line-numbers .line-numbers-rows':{left:"0"},'pre[class*="language-"][data-line]':{paddingTop:"0",paddingBottom:"0",paddingLeft:"0"},"pre[data-line] code":{position:"relative",paddingLeft:"4em"},"pre .line-highlight":{marginTop:"0"}}})(R4)),R4}var D4={},aV;function Vst(){return aV||(aV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"white",background:"none",textShadow:"0 -.1em .2em black",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"white",background:"hsl(30, 20%, 25%)",textShadow:"0 -.1em .2em black",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",border:".3em solid hsl(30, 20%, 40%)",borderRadius:".5em",boxShadow:"1px 1px .5em black inset"},':not(pre) > code[class*="language-"]':{background:"hsl(30, 20%, 25%)",padding:".15em .2em .05em",borderRadius:".3em",border:".13em solid hsl(30, 20%, 40%)",boxShadow:"1px 1px .3em -.1em black inset",whiteSpace:"normal"},comment:{color:"hsl(30, 20%, 50%)"},prolog:{color:"hsl(30, 20%, 50%)"},doctype:{color:"hsl(30, 20%, 50%)"},cdata:{color:"hsl(30, 20%, 50%)"},punctuation:{Opacity:".7"},namespace:{Opacity:".7"},property:{color:"hsl(350, 40%, 70%)"},tag:{color:"hsl(350, 40%, 70%)"},boolean:{color:"hsl(350, 40%, 70%)"},number:{color:"hsl(350, 40%, 70%)"},constant:{color:"hsl(350, 40%, 70%)"},symbol:{color:"hsl(350, 40%, 70%)"},selector:{color:"hsl(75, 70%, 60%)"},"attr-name":{color:"hsl(75, 70%, 60%)"},string:{color:"hsl(75, 70%, 60%)"},char:{color:"hsl(75, 70%, 60%)"},builtin:{color:"hsl(75, 70%, 60%)"},inserted:{color:"hsl(75, 70%, 60%)"},operator:{color:"hsl(40, 90%, 60%)"},entity:{color:"hsl(40, 90%, 60%)",cursor:"help"},url:{color:"hsl(40, 90%, 60%)"},".language-css .token.string":{color:"hsl(40, 90%, 60%)"},".style .token.string":{color:"hsl(40, 90%, 60%)"},variable:{color:"hsl(40, 90%, 60%)"},atrule:{color:"hsl(350, 40%, 70%)"},"attr-value":{color:"hsl(350, 40%, 70%)"},keyword:{color:"hsl(350, 40%, 70%)"},regex:{color:"#e90"},important:{color:"#e90",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},deleted:{color:"red"}}})(D4)),D4}var N4={},iV;function Kst(){return iV||(iV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"black",color:"white",boxShadow:"-.3em 0 0 .3em black, .3em 0 0 .3em black"},'pre[class*="language-"]':{fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:".4em .8em",margin:".5em 0",overflow:"auto",background:`url('data:image/svg+xml;charset=utf-8,<svg%20version%3D"1.1"%20xmlns%3D"http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg"%20width%3D"100"%20height%3D"100"%20fill%3D"rgba(0%2C0%2C0%2C.2)">%0D%0A<polygon%20points%3D"0%2C50%2050%2C0%200%2C0"%20%2F>%0D%0A<polygon%20points%3D"0%2C100%2050%2C100%20100%2C50%20100%2C0"%20%2F>%0D%0A<%2Fsvg>')`,backgroundSize:"1em 1em"},':not(pre) > code[class*="language-"]':{padding:".2em",borderRadius:".3em",boxShadow:"none",whiteSpace:"normal"},comment:{color:"#aaa"},prolog:{color:"#aaa"},doctype:{color:"#aaa"},cdata:{color:"#aaa"},punctuation:{color:"#999"},namespace:{Opacity:".7"},property:{color:"#0cf"},tag:{color:"#0cf"},boolean:{color:"#0cf"},number:{color:"#0cf"},constant:{color:"#0cf"},symbol:{color:"#0cf"},selector:{color:"yellow"},"attr-name":{color:"yellow"},string:{color:"yellow"},char:{color:"yellow"},builtin:{color:"yellow"},operator:{color:"yellowgreen"},entity:{color:"yellowgreen",cursor:"help"},url:{color:"yellowgreen"},".language-css .token.string":{color:"yellowgreen"},variable:{color:"yellowgreen"},inserted:{color:"yellowgreen"},atrule:{color:"deeppink"},"attr-value":{color:"deeppink"},keyword:{color:"deeppink"},regex:{color:"orange"},important:{color:"orange",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},deleted:{color:"red"},"pre.diff-highlight.diff-highlight > code .token.deleted:not(.prefix)":{backgroundColor:"rgba(255, 0, 0, .3)",display:"inline"},"pre > code.diff-highlight.diff-highlight .token.deleted:not(.prefix)":{backgroundColor:"rgba(255, 0, 0, .3)",display:"inline"},"pre.diff-highlight.diff-highlight > code .token.inserted:not(.prefix)":{backgroundColor:"rgba(0, 255, 128, .3)",display:"inline"},"pre > code.diff-highlight.diff-highlight .token.inserted:not(.prefix)":{backgroundColor:"rgba(0, 255, 128, .3)",display:"inline"}}})(N4)),N4}var L4={},oV;function Yst(){return oV||(oV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#f8f8f2",background:"none",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#f8f8f2",background:"#272822",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",borderRadius:"0.3em"},':not(pre) > code[class*="language-"]':{background:"#272822",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"#8292a2"},prolog:{color:"#8292a2"},doctype:{color:"#8292a2"},cdata:{color:"#8292a2"},punctuation:{color:"#f8f8f2"},namespace:{Opacity:".7"},property:{color:"#f92672"},tag:{color:"#f92672"},constant:{color:"#f92672"},symbol:{color:"#f92672"},deleted:{color:"#f92672"},boolean:{color:"#ae81ff"},number:{color:"#ae81ff"},selector:{color:"#a6e22e"},"attr-name":{color:"#a6e22e"},string:{color:"#a6e22e"},char:{color:"#a6e22e"},builtin:{color:"#a6e22e"},inserted:{color:"#a6e22e"},operator:{color:"#f8f8f2"},entity:{color:"#f8f8f2",cursor:"help"},url:{color:"#f8f8f2"},".language-css .token.string":{color:"#f8f8f2"},".style .token.string":{color:"#f8f8f2"},variable:{color:"#f8f8f2"},atrule:{color:"#e6db74"},"attr-value":{color:"#e6db74"},function:{color:"#e6db74"},"class-name":{color:"#e6db74"},keyword:{color:"#66d9ef"},regex:{color:"#fd971f"},important:{color:"#fd971f",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(L4)),L4}var M4={},sV;function Jst(){return sV||(sV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#657b83",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#657b83",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",borderRadius:"0.3em",backgroundColor:"#fdf6e3"},'pre[class*="language-"]::-moz-selection':{background:"#073642"},'pre[class*="language-"] ::-moz-selection':{background:"#073642"},'code[class*="language-"]::-moz-selection':{background:"#073642"},'code[class*="language-"] ::-moz-selection':{background:"#073642"},'pre[class*="language-"]::selection':{background:"#073642"},'pre[class*="language-"] ::selection':{background:"#073642"},'code[class*="language-"]::selection':{background:"#073642"},'code[class*="language-"] ::selection':{background:"#073642"},':not(pre) > code[class*="language-"]':{backgroundColor:"#fdf6e3",padding:".1em",borderRadius:".3em"},comment:{color:"#93a1a1"},prolog:{color:"#93a1a1"},doctype:{color:"#93a1a1"},cdata:{color:"#93a1a1"},punctuation:{color:"#586e75"},namespace:{Opacity:".7"},property:{color:"#268bd2"},tag:{color:"#268bd2"},boolean:{color:"#268bd2"},number:{color:"#268bd2"},constant:{color:"#268bd2"},symbol:{color:"#268bd2"},deleted:{color:"#268bd2"},selector:{color:"#2aa198"},"attr-name":{color:"#2aa198"},string:{color:"#2aa198"},char:{color:"#2aa198"},builtin:{color:"#2aa198"},url:{color:"#2aa198"},inserted:{color:"#2aa198"},entity:{color:"#657b83",background:"#eee8d5",cursor:"help"},atrule:{color:"#859900"},"attr-value":{color:"#859900"},keyword:{color:"#859900"},function:{color:"#b58900"},"class-name":{color:"#b58900"},regex:{color:"#cb4b16"},important:{color:"#cb4b16",fontWeight:"bold"},variable:{color:"#cb4b16"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(M4)),M4}var I4={},lV;function Xst(){return lV||(lV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#ccc",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#ccc",background:"#2d2d2d",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto"},':not(pre) > code[class*="language-"]':{background:"#2d2d2d",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"#999"},"block-comment":{color:"#999"},prolog:{color:"#999"},doctype:{color:"#999"},cdata:{color:"#999"},punctuation:{color:"#ccc"},tag:{color:"#e2777a"},"attr-name":{color:"#e2777a"},namespace:{color:"#e2777a"},deleted:{color:"#e2777a"},"function-name":{color:"#6196cc"},boolean:{color:"#f08d49"},number:{color:"#f08d49"},function:{color:"#f08d49"},property:{color:"#f8c555"},"class-name":{color:"#f8c555"},constant:{color:"#f8c555"},symbol:{color:"#f8c555"},selector:{color:"#cc99cd"},important:{color:"#cc99cd",fontWeight:"bold"},atrule:{color:"#cc99cd"},keyword:{color:"#cc99cd"},builtin:{color:"#cc99cd"},string:{color:"#7ec699"},char:{color:"#7ec699"},"attr-value":{color:"#7ec699"},regex:{color:"#7ec699"},variable:{color:"#7ec699"},operator:{color:"#67cdcc"},entity:{color:"#67cdcc",cursor:"help"},url:{color:"#67cdcc"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},inserted:{color:"green"}}})(I4)),I4}var O4={},uV;function Zst(){return uV||(uV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"white",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",textShadow:"0 -.1em .2em black",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"white",background:"hsl(0, 0%, 8%)",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",textShadow:"0 -.1em .2em black",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",borderRadius:".5em",border:".3em solid hsl(0, 0%, 33%)",boxShadow:"1px 1px .5em black inset",margin:".5em 0",overflow:"auto",padding:"1em"},':not(pre) > code[class*="language-"]':{background:"hsl(0, 0%, 8%)",borderRadius:".3em",border:".13em solid hsl(0, 0%, 33%)",boxShadow:"1px 1px .3em -.1em black inset",padding:".15em .2em .05em",whiteSpace:"normal"},'pre[class*="language-"]::-moz-selection':{background:"hsla(0, 0%, 93%, 0.15)",textShadow:"none"},'pre[class*="language-"]::selection':{background:"hsla(0, 0%, 93%, 0.15)",textShadow:"none"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"hsla(0, 0%, 93%, 0.15)"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"hsla(0, 0%, 93%, 0.15)"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"hsla(0, 0%, 93%, 0.15)"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"hsla(0, 0%, 93%, 0.15)"},'code[class*="language-"]::selection':{textShadow:"none",background:"hsla(0, 0%, 93%, 0.15)"},'code[class*="language-"] ::selection':{textShadow:"none",background:"hsla(0, 0%, 93%, 0.15)"},comment:{color:"hsl(0, 0%, 47%)"},prolog:{color:"hsl(0, 0%, 47%)"},doctype:{color:"hsl(0, 0%, 47%)"},cdata:{color:"hsl(0, 0%, 47%)"},punctuation:{Opacity:".7"},namespace:{Opacity:".7"},tag:{color:"hsl(14, 58%, 55%)"},boolean:{color:"hsl(14, 58%, 55%)"},number:{color:"hsl(14, 58%, 55%)"},deleted:{color:"hsl(14, 58%, 55%)"},keyword:{color:"hsl(53, 89%, 79%)"},property:{color:"hsl(53, 89%, 79%)"},selector:{color:"hsl(53, 89%, 79%)"},constant:{color:"hsl(53, 89%, 79%)"},symbol:{color:"hsl(53, 89%, 79%)"},builtin:{color:"hsl(53, 89%, 79%)"},"attr-name":{color:"hsl(76, 21%, 52%)"},"attr-value":{color:"hsl(76, 21%, 52%)"},string:{color:"hsl(76, 21%, 52%)"},char:{color:"hsl(76, 21%, 52%)"},operator:{color:"hsl(76, 21%, 52%)"},entity:{color:"hsl(76, 21%, 52%)",cursor:"help"},url:{color:"hsl(76, 21%, 52%)"},".language-css .token.string":{color:"hsl(76, 21%, 52%)"},".style .token.string":{color:"hsl(76, 21%, 52%)"},variable:{color:"hsl(76, 21%, 52%)"},inserted:{color:"hsl(76, 21%, 52%)"},atrule:{color:"hsl(218, 22%, 55%)"},regex:{color:"hsl(42, 75%, 65%)"},important:{color:"hsl(42, 75%, 65%)",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},".language-markup .token.tag":{color:"hsl(33, 33%, 52%)"},".language-markup .token.attr-name":{color:"hsl(33, 33%, 52%)"},".language-markup .token.punctuation":{color:"hsl(33, 33%, 52%)"},"":{position:"relative",zIndex:"1"},".line-highlight.line-highlight":{background:"linear-gradient(to right, hsla(0, 0%, 33%, .1) 70%, hsla(0, 0%, 33%, 0))",borderBottom:"1px dashed hsl(0, 0%, 33%)",borderTop:"1px dashed hsl(0, 0%, 33%)",marginTop:"0.75em",zIndex:"0"},".line-highlight.line-highlight:before":{backgroundColor:"hsl(215, 15%, 59%)",color:"hsl(24, 20%, 95%)"},".line-highlight.line-highlight[data-end]:after":{backgroundColor:"hsl(215, 15%, 59%)",color:"hsl(24, 20%, 95%)"}}})(O4)),O4}var B4={},cV;function Qst(){return cV||(cV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"black",background:"none",textShadow:"0 1px white",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"black",background:"#f5f2f0",textShadow:"0 1px white",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#b3d4fc"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#b3d4fc"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"]::selection':{textShadow:"none",background:"#b3d4fc"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#b3d4fc"},':not(pre) > code[class*="language-"]':{background:"#f5f2f0",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"slategray"},prolog:{color:"slategray"},doctype:{color:"slategray"},cdata:{color:"slategray"},punctuation:{color:"#999"},namespace:{Opacity:".7"},property:{color:"#905"},tag:{color:"#905"},boolean:{color:"#905"},number:{color:"#905"},constant:{color:"#905"},symbol:{color:"#905"},deleted:{color:"#905"},selector:{color:"#690"},"attr-name":{color:"#690"},string:{color:"#690"},char:{color:"#690"},builtin:{color:"#690"},inserted:{color:"#690"},operator:{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},entity:{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)",cursor:"help"},url:{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},".language-css .token.string":{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},".style .token.string":{color:"#9a6e3a",background:"hsla(0, 0%, 100%, .5)"},atrule:{color:"#07a"},"attr-value":{color:"#07a"},keyword:{color:"#07a"},function:{color:"#DD4A68"},"class-name":{color:"#DD4A68"},regex:{color:"#e90"},important:{color:"#e90",fontWeight:"bold"},variable:{color:"#e90"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(B4)),B4}var P4={},dV;function elt(){return dV||(dV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#f8f8f2",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#f8f8f2",background:"#2b2b2b",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto",borderRadius:"0.3em"},':not(pre) > code[class*="language-"]':{background:"#2b2b2b",padding:"0.1em",borderRadius:"0.3em",whiteSpace:"normal"},comment:{color:"#d4d0ab"},prolog:{color:"#d4d0ab"},doctype:{color:"#d4d0ab"},cdata:{color:"#d4d0ab"},punctuation:{color:"#fefefe"},property:{color:"#ffa07a"},tag:{color:"#ffa07a"},constant:{color:"#ffa07a"},symbol:{color:"#ffa07a"},deleted:{color:"#ffa07a"},boolean:{color:"#00e0e0"},number:{color:"#00e0e0"},selector:{color:"#abe338"},"attr-name":{color:"#abe338"},string:{color:"#abe338"},char:{color:"#abe338"},builtin:{color:"#abe338"},inserted:{color:"#abe338"},operator:{color:"#00e0e0"},entity:{color:"#00e0e0",cursor:"help"},url:{color:"#00e0e0"},".language-css .token.string":{color:"#00e0e0"},".style .token.string":{color:"#00e0e0"},variable:{color:"#00e0e0"},atrule:{color:"#ffd700"},"attr-value":{color:"#ffd700"},function:{color:"#ffd700"},keyword:{color:"#00e0e0"},regex:{color:"#ffd700"},important:{color:"#ffd700",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(P4)),P4}var z4={},hV;function tlt(){return hV||(hV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#c5c8c6",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Inconsolata, Monaco, Consolas, 'Courier New', Courier, monospace",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#c5c8c6",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Inconsolata, Monaco, Consolas, 'Courier New', Courier, monospace",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",borderRadius:"0.3em",background:"#1d1f21"},':not(pre) > code[class*="language-"]':{background:"#1d1f21",padding:".1em",borderRadius:".3em"},comment:{color:"#7C7C7C"},prolog:{color:"#7C7C7C"},doctype:{color:"#7C7C7C"},cdata:{color:"#7C7C7C"},punctuation:{color:"#c5c8c6"},".namespace":{Opacity:".7"},property:{color:"#96CBFE"},keyword:{color:"#96CBFE"},tag:{color:"#96CBFE"},"class-name":{color:"#FFFFB6",textDecoration:"underline"},boolean:{color:"#99CC99"},constant:{color:"#99CC99"},symbol:{color:"#f92672"},deleted:{color:"#f92672"},number:{color:"#FF73FD"},selector:{color:"#A8FF60"},"attr-name":{color:"#A8FF60"},string:{color:"#A8FF60"},char:{color:"#A8FF60"},builtin:{color:"#A8FF60"},inserted:{color:"#A8FF60"},variable:{color:"#C6C5FE"},operator:{color:"#EDEDED"},entity:{color:"#FFFFB6",cursor:"help"},url:{color:"#96CBFE"},".language-css .token.string":{color:"#87C38A"},".style .token.string":{color:"#87C38A"},atrule:{color:"#F9EE98"},"attr-value":{color:"#F9EE98"},function:{color:"#DAD085"},regex:{color:"#E9C062"},important:{color:"#fd971f",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(z4)),z4}var H4={},pV;function rlt(){return pV||(pV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#f5f7ff",color:"#5e6687"},'pre[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#f5f7ff",color:"#5e6687",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#dfe2f1"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#dfe2f1"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#dfe2f1"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#dfe2f1"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#dfe2f1"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#dfe2f1"},'code[class*="language-"]::selection':{textShadow:"none",background:"#dfe2f1"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#dfe2f1"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#898ea4"},prolog:{color:"#898ea4"},doctype:{color:"#898ea4"},cdata:{color:"#898ea4"},punctuation:{color:"#5e6687"},namespace:{Opacity:".7"},operator:{color:"#c76b29"},boolean:{color:"#c76b29"},number:{color:"#c76b29"},property:{color:"#c08b30"},tag:{color:"#3d8fd1"},string:{color:"#22a2c9"},selector:{color:"#6679cc"},"attr-name":{color:"#c76b29"},entity:{color:"#22a2c9",cursor:"help"},url:{color:"#22a2c9"},".language-css .token.string":{color:"#22a2c9"},".style .token.string":{color:"#22a2c9"},"attr-value":{color:"#ac9739"},keyword:{color:"#ac9739"},control:{color:"#ac9739"},directive:{color:"#ac9739"},unit:{color:"#ac9739"},statement:{color:"#22a2c9"},regex:{color:"#22a2c9"},atrule:{color:"#22a2c9"},placeholder:{color:"#3d8fd1"},variable:{color:"#3d8fd1"},deleted:{textDecoration:"line-through"},inserted:{borderBottom:"1px dotted #202746",textDecoration:"none"},italic:{fontStyle:"italic"},important:{fontWeight:"bold",color:"#c94922"},bold:{fontWeight:"bold"},"pre > code.highlight":{Outline:"0.4em solid #c94922",OutlineOffset:".4em"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#dfe2f1"},".line-numbers .line-numbers-rows > span:before":{color:"#979db4"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(107, 115, 148, 0.2) 70%, rgba(107, 115, 148, 0))"}}})(H4)),H4}var U4={},fV;function nlt(){return fV||(fV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#fff",textShadow:"0 1px 1px #000",fontFamily:'Menlo, Monaco, "Courier New", monospace',direction:"ltr",textAlign:"left",wordSpacing:"normal",whiteSpace:"pre",wordWrap:"normal",lineHeight:"1.4",background:"none",border:"0",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#fff",textShadow:"0 1px 1px #000",fontFamily:'Menlo, Monaco, "Courier New", monospace',direction:"ltr",textAlign:"left",wordSpacing:"normal",whiteSpace:"pre",wordWrap:"normal",lineHeight:"1.4",background:"#222",border:"0",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"15px",margin:"1em 0",overflow:"auto",MozBorderRadius:"8px",WebkitBorderRadius:"8px",borderRadius:"8px"},'pre[class*="language-"] code':{float:"left",padding:"0 15px 0 0"},':not(pre) > code[class*="language-"]':{background:"#222",padding:"5px 10px",lineHeight:"1",MozBorderRadius:"3px",WebkitBorderRadius:"3px",borderRadius:"3px"},comment:{color:"#797979"},prolog:{color:"#797979"},doctype:{color:"#797979"},cdata:{color:"#797979"},selector:{color:"#fff"},operator:{color:"#fff"},punctuation:{color:"#fff"},namespace:{Opacity:".7"},tag:{color:"#ffd893"},boolean:{color:"#ffd893"},atrule:{color:"#B0C975"},"attr-value":{color:"#B0C975"},hex:{color:"#B0C975"},string:{color:"#B0C975"},property:{color:"#c27628"},entity:{color:"#c27628",cursor:"help"},url:{color:"#c27628"},"attr-name":{color:"#c27628"},keyword:{color:"#c27628"},regex:{color:"#9B71C6"},function:{color:"#e5a638"},constant:{color:"#e5a638"},variable:{color:"#fdfba8"},number:{color:"#8799B0"},important:{color:"#E45734"},deliminator:{color:"#E45734"},".line-highlight.line-highlight":{background:"rgba(255, 255, 255, .2)"},".line-highlight.line-highlight:before":{top:".3em",backgroundColor:"rgba(255, 255, 255, .3)",color:"#fff",MozBorderRadius:"8px",WebkitBorderRadius:"8px",borderRadius:"8px"},".line-highlight.line-highlight[data-end]:after":{top:".3em",backgroundColor:"rgba(255, 255, 255, .3)",color:"#fff",MozBorderRadius:"8px",WebkitBorderRadius:"8px",borderRadius:"8px"},".line-numbers .line-numbers-rows > span":{borderRight:"3px #d9d336 solid"}}})(U4)),U4}var $4={},mV;function alt(){return mV||(mV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#111b27",background:"none",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#111b27",background:"#e3eaf2",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto"},'pre[class*="language-"]::-moz-selection':{background:"#8da1b9"},'pre[class*="language-"] ::-moz-selection':{background:"#8da1b9"},'code[class*="language-"]::-moz-selection':{background:"#8da1b9"},'code[class*="language-"] ::-moz-selection':{background:"#8da1b9"},'pre[class*="language-"]::selection':{background:"#8da1b9"},'pre[class*="language-"] ::selection':{background:"#8da1b9"},'code[class*="language-"]::selection':{background:"#8da1b9"},'code[class*="language-"] ::selection':{background:"#8da1b9"},':not(pre) > code[class*="language-"]':{background:"#e3eaf2",padding:"0.1em 0.3em",borderRadius:"0.3em",whiteSpace:"normal"},comment:{color:"#3c526d"},prolog:{color:"#3c526d"},doctype:{color:"#3c526d"},cdata:{color:"#3c526d"},punctuation:{color:"#111b27"},"delimiter.important":{color:"#006d6d",fontWeight:"inherit"},"selector.parent":{color:"#006d6d"},tag:{color:"#006d6d"},"tag.punctuation":{color:"#006d6d"},"attr-name":{color:"#755f00"},boolean:{color:"#755f00"},"boolean.important":{color:"#755f00"},number:{color:"#755f00"},constant:{color:"#755f00"},"selector.attribute":{color:"#755f00"},"class-name":{color:"#005a8e"},key:{color:"#005a8e"},parameter:{color:"#005a8e"},property:{color:"#005a8e"},"property-access":{color:"#005a8e"},variable:{color:"#005a8e"},"attr-value":{color:"#116b00"},inserted:{color:"#116b00"},color:{color:"#116b00"},"selector.value":{color:"#116b00"},string:{color:"#116b00"},"string.url-link":{color:"#116b00"},builtin:{color:"#af00af"},"keyword-array":{color:"#af00af"},package:{color:"#af00af"},regex:{color:"#af00af"},function:{color:"#7c00aa"},"selector.class":{color:"#7c00aa"},"selector.id":{color:"#7c00aa"},"atrule.rule":{color:"#a04900"},combinator:{color:"#a04900"},keyword:{color:"#a04900"},operator:{color:"#a04900"},"pseudo-class":{color:"#a04900"},"pseudo-element":{color:"#a04900"},selector:{color:"#a04900"},unit:{color:"#a04900"},deleted:{color:"#c22f2e"},important:{color:"#c22f2e",fontWeight:"bold"},"keyword-this":{color:"#005a8e",fontWeight:"bold"},this:{color:"#005a8e",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},entity:{cursor:"help"},".language-markdown .token.title":{color:"#005a8e",fontWeight:"bold"},".language-markdown .token.title .token.punctuation":{color:"#005a8e",fontWeight:"bold"},".language-markdown .token.blockquote.punctuation":{color:"#af00af"},".language-markdown .token.code":{color:"#006d6d"},".language-markdown .token.hr.punctuation":{color:"#005a8e"},".language-markdown .token.url > .token.content":{color:"#116b00"},".language-markdown .token.url-link":{color:"#755f00"},".language-markdown .token.list.punctuation":{color:"#af00af"},".language-markdown .token.table-header":{color:"#111b27"},".language-json .token.operator":{color:"#111b27"},".language-scss .token.variable":{color:"#006d6d"},"token.tab:not(:empty):before":{color:"#3c526d"},"token.cr:before":{color:"#3c526d"},"token.lf:before":{color:"#3c526d"},"token.space:before":{color:"#3c526d"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a":{color:"#e3eaf2",background:"#005a8e"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button":{color:"#e3eaf2",background:"#005a8e"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:hover":{color:"#e3eaf2",background:"#005a8eda",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:focus":{color:"#e3eaf2",background:"#005a8eda",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:hover":{color:"#e3eaf2",background:"#005a8eda",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:focus":{color:"#e3eaf2",background:"#005a8eda",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span":{color:"#e3eaf2",background:"#3c526d"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:hover":{color:"#e3eaf2",background:"#3c526d"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:focus":{color:"#e3eaf2",background:"#3c526d"},".line-highlight.line-highlight":{background:"linear-gradient(to right, #8da1b92f 70%, #8da1b925)"},".line-highlight.line-highlight:before":{backgroundColor:"#3c526d",color:"#e3eaf2",boxShadow:"0 1px #8da1b9"},".line-highlight.line-highlight[data-end]:after":{backgroundColor:"#3c526d",color:"#e3eaf2",boxShadow:"0 1px #8da1b9"},"pre[id].linkable-line-numbers.linkable-line-numbers span.line-numbers-rows > span:hover:before":{backgroundColor:"#3c526d1f"},".line-numbers.line-numbers .line-numbers-rows":{borderRight:"1px solid #8da1b97a",background:"#d0dae77a"},".line-numbers .line-numbers-rows > span:before":{color:"#3c526dda"},".rainbow-braces .token.token.punctuation.brace-level-1":{color:"#755f00"},".rainbow-braces .token.token.punctuation.brace-level-5":{color:"#755f00"},".rainbow-braces .token.token.punctuation.brace-level-9":{color:"#755f00"},".rainbow-braces .token.token.punctuation.brace-level-2":{color:"#af00af"},".rainbow-braces .token.token.punctuation.brace-level-6":{color:"#af00af"},".rainbow-braces .token.token.punctuation.brace-level-10":{color:"#af00af"},".rainbow-braces .token.token.punctuation.brace-level-3":{color:"#005a8e"},".rainbow-braces .token.token.punctuation.brace-level-7":{color:"#005a8e"},".rainbow-braces .token.token.punctuation.brace-level-11":{color:"#005a8e"},".rainbow-braces .token.token.punctuation.brace-level-4":{color:"#7c00aa"},".rainbow-braces .token.token.punctuation.brace-level-8":{color:"#7c00aa"},".rainbow-braces .token.token.punctuation.brace-level-12":{color:"#7c00aa"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)":{backgroundColor:"#c22f2e1f"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)":{backgroundColor:"#c22f2e1f"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)":{backgroundColor:"#116b001f"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)":{backgroundColor:"#116b001f"},".command-line .command-line-prompt":{borderRight:"1px solid #8da1b97a"},".command-line .command-line-prompt > span:before":{color:"#3c526dda"}}})($4)),$4}var G4={},gV;function ilt(){return gV||(gV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#e3eaf2",background:"none",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#e3eaf2",background:"#111b27",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto"},'pre[class*="language-"]::-moz-selection':{background:"#3c526d"},'pre[class*="language-"] ::-moz-selection':{background:"#3c526d"},'code[class*="language-"]::-moz-selection':{background:"#3c526d"},'code[class*="language-"] ::-moz-selection':{background:"#3c526d"},'pre[class*="language-"]::selection':{background:"#3c526d"},'pre[class*="language-"] ::selection':{background:"#3c526d"},'code[class*="language-"]::selection':{background:"#3c526d"},'code[class*="language-"] ::selection':{background:"#3c526d"},':not(pre) > code[class*="language-"]':{background:"#111b27",padding:"0.1em 0.3em",borderRadius:"0.3em",whiteSpace:"normal"},comment:{color:"#8da1b9"},prolog:{color:"#8da1b9"},doctype:{color:"#8da1b9"},cdata:{color:"#8da1b9"},punctuation:{color:"#e3eaf2"},"delimiter.important":{color:"#66cccc",fontWeight:"inherit"},"selector.parent":{color:"#66cccc"},tag:{color:"#66cccc"},"tag.punctuation":{color:"#66cccc"},"attr-name":{color:"#e6d37a"},boolean:{color:"#e6d37a"},"boolean.important":{color:"#e6d37a"},number:{color:"#e6d37a"},constant:{color:"#e6d37a"},"selector.attribute":{color:"#e6d37a"},"class-name":{color:"#6cb8e6"},key:{color:"#6cb8e6"},parameter:{color:"#6cb8e6"},property:{color:"#6cb8e6"},"property-access":{color:"#6cb8e6"},variable:{color:"#6cb8e6"},"attr-value":{color:"#91d076"},inserted:{color:"#91d076"},color:{color:"#91d076"},"selector.value":{color:"#91d076"},string:{color:"#91d076"},"string.url-link":{color:"#91d076"},builtin:{color:"#f4adf4"},"keyword-array":{color:"#f4adf4"},package:{color:"#f4adf4"},regex:{color:"#f4adf4"},function:{color:"#c699e3"},"selector.class":{color:"#c699e3"},"selector.id":{color:"#c699e3"},"atrule.rule":{color:"#e9ae7e"},combinator:{color:"#e9ae7e"},keyword:{color:"#e9ae7e"},operator:{color:"#e9ae7e"},"pseudo-class":{color:"#e9ae7e"},"pseudo-element":{color:"#e9ae7e"},selector:{color:"#e9ae7e"},unit:{color:"#e9ae7e"},deleted:{color:"#cd6660"},important:{color:"#cd6660",fontWeight:"bold"},"keyword-this":{color:"#6cb8e6",fontWeight:"bold"},this:{color:"#6cb8e6",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},entity:{cursor:"help"},".language-markdown .token.title":{color:"#6cb8e6",fontWeight:"bold"},".language-markdown .token.title .token.punctuation":{color:"#6cb8e6",fontWeight:"bold"},".language-markdown .token.blockquote.punctuation":{color:"#f4adf4"},".language-markdown .token.code":{color:"#66cccc"},".language-markdown .token.hr.punctuation":{color:"#6cb8e6"},".language-markdown .token.url .token.content":{color:"#91d076"},".language-markdown .token.url-link":{color:"#e6d37a"},".language-markdown .token.list.punctuation":{color:"#f4adf4"},".language-markdown .token.table-header":{color:"#e3eaf2"},".language-json .token.operator":{color:"#e3eaf2"},".language-scss .token.variable":{color:"#66cccc"},"token.tab:not(:empty):before":{color:"#8da1b9"},"token.cr:before":{color:"#8da1b9"},"token.lf:before":{color:"#8da1b9"},"token.space:before":{color:"#8da1b9"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a":{color:"#111b27",background:"#6cb8e6"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button":{color:"#111b27",background:"#6cb8e6"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:hover":{color:"#111b27",background:"#6cb8e6da",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:focus":{color:"#111b27",background:"#6cb8e6da",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:hover":{color:"#111b27",background:"#6cb8e6da",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:focus":{color:"#111b27",background:"#6cb8e6da",textDecoration:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span":{color:"#111b27",background:"#8da1b9"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:hover":{color:"#111b27",background:"#8da1b9"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:focus":{color:"#111b27",background:"#8da1b9"},".line-highlight.line-highlight":{background:"linear-gradient(to right, #3c526d5f 70%, #3c526d55)"},".line-highlight.line-highlight:before":{backgroundColor:"#8da1b9",color:"#111b27",boxShadow:"0 1px #3c526d"},".line-highlight.line-highlight[data-end]:after":{backgroundColor:"#8da1b9",color:"#111b27",boxShadow:"0 1px #3c526d"},"pre[id].linkable-line-numbers.linkable-line-numbers span.line-numbers-rows > span:hover:before":{backgroundColor:"#8da1b918"},".line-numbers.line-numbers .line-numbers-rows":{borderRight:"1px solid #0b121b",background:"#0b121b7a"},".line-numbers .line-numbers-rows > span:before":{color:"#8da1b9da"},".rainbow-braces .token.token.punctuation.brace-level-1":{color:"#e6d37a"},".rainbow-braces .token.token.punctuation.brace-level-5":{color:"#e6d37a"},".rainbow-braces .token.token.punctuation.brace-level-9":{color:"#e6d37a"},".rainbow-braces .token.token.punctuation.brace-level-2":{color:"#f4adf4"},".rainbow-braces .token.token.punctuation.brace-level-6":{color:"#f4adf4"},".rainbow-braces .token.token.punctuation.brace-level-10":{color:"#f4adf4"},".rainbow-braces .token.token.punctuation.brace-level-3":{color:"#6cb8e6"},".rainbow-braces .token.token.punctuation.brace-level-7":{color:"#6cb8e6"},".rainbow-braces .token.token.punctuation.brace-level-11":{color:"#6cb8e6"},".rainbow-braces .token.token.punctuation.brace-level-4":{color:"#c699e3"},".rainbow-braces .token.token.punctuation.brace-level-8":{color:"#c699e3"},".rainbow-braces .token.token.punctuation.brace-level-12":{color:"#c699e3"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)":{backgroundColor:"#cd66601f"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)":{backgroundColor:"#cd66601f"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)":{backgroundColor:"#91d0761f"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)":{backgroundColor:"#91d0761f"},".command-line .command-line-prompt":{borderRight:"1px solid #0b121b"},".command-line .command-line-prompt > span:before":{color:"#8da1b9da"}}})(G4)),G4}var j4={},bV;function olt(){return bV||(bV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"black",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"black",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",position:"relative",borderLeft:"10px solid #358ccb",boxShadow:"-1px 0 0 0 #358ccb, 0 0 0 1px #dfdfdf",backgroundColor:"#fdfdfd",backgroundImage:"linear-gradient(transparent 50%, rgba(69, 142, 209, 0.04) 50%)",backgroundSize:"3em 3em",backgroundOrigin:"content-box",backgroundAttachment:"local",margin:".5em 0",padding:"0 1em"},'pre[class*="language-"] > code':{display:"block"},':not(pre) > code[class*="language-"]':{position:"relative",padding:".2em",borderRadius:"0.3em",color:"#c92c2c",border:"1px solid rgba(0, 0, 0, 0.1)",display:"inline",whiteSpace:"normal",backgroundColor:"#fdfdfd",WebkitBoxSizing:"border-box",MozBoxSizing:"border-box",boxSizing:"border-box"},comment:{color:"#7D8B99"},"block-comment":{color:"#7D8B99"},prolog:{color:"#7D8B99"},doctype:{color:"#7D8B99"},cdata:{color:"#7D8B99"},punctuation:{color:"#5F6364"},property:{color:"#c92c2c"},tag:{color:"#c92c2c"},boolean:{color:"#c92c2c"},number:{color:"#c92c2c"},"function-name":{color:"#c92c2c"},constant:{color:"#c92c2c"},symbol:{color:"#c92c2c"},deleted:{color:"#c92c2c"},selector:{color:"#2f9c0a"},"attr-name":{color:"#2f9c0a"},string:{color:"#2f9c0a"},char:{color:"#2f9c0a"},function:{color:"#2f9c0a"},builtin:{color:"#2f9c0a"},inserted:{color:"#2f9c0a"},operator:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},entity:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)",cursor:"help"},url:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},variable:{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},atrule:{color:"#1990b8"},"attr-value":{color:"#1990b8"},keyword:{color:"#1990b8"},"class-name":{color:"#1990b8"},regex:{color:"#e90"},important:{color:"#e90",fontWeight:"normal"},".language-css .token.string":{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},".style .token.string":{color:"#a67f59",background:"rgba(255, 255, 255, 0.5)"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},namespace:{Opacity:".7"}}})(j4)),j4}var q4={},yV;function slt(){return yV||(yV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#a9b7c6",fontFamily:"Consolas, Monaco, 'Andale Mono', monospace",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#a9b7c6",fontFamily:"Consolas, Monaco, 'Andale Mono', monospace",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",background:"#2b2b2b"},'pre[class*="language-"]::-moz-selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},'pre[class*="language-"] ::-moz-selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},'code[class*="language-"]::-moz-selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},'code[class*="language-"] ::-moz-selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},'pre[class*="language-"]::selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},'pre[class*="language-"] ::selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},'code[class*="language-"]::selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},'code[class*="language-"] ::selection':{color:"inherit",background:"rgba(33, 66, 131, .85)"},':not(pre) > code[class*="language-"]':{background:"#2b2b2b",padding:".1em",borderRadius:".3em"},comment:{color:"#808080"},prolog:{color:"#808080"},cdata:{color:"#808080"},delimiter:{color:"#cc7832"},boolean:{color:"#cc7832"},keyword:{color:"#cc7832"},selector:{color:"#cc7832"},important:{color:"#cc7832"},atrule:{color:"#cc7832"},operator:{color:"#a9b7c6"},punctuation:{color:"#a9b7c6"},"attr-name":{color:"#a9b7c6"},tag:{color:"#e8bf6a"},"tag.punctuation":{color:"#e8bf6a"},doctype:{color:"#e8bf6a"},builtin:{color:"#e8bf6a"},entity:{color:"#6897bb"},number:{color:"#6897bb"},symbol:{color:"#6897bb"},property:{color:"#9876aa"},constant:{color:"#9876aa"},variable:{color:"#9876aa"},string:{color:"#6a8759"},char:{color:"#6a8759"},"attr-value":{color:"#a5c261"},"attr-value.punctuation":{color:"#a5c261"},"attr-value.punctuation:first-child":{color:"#a9b7c6"},url:{color:"#287bde",textDecoration:"underline"},function:{color:"#ffc66d"},regex:{background:"#364135"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},inserted:{background:"#294436"},deleted:{background:"#484a4a"},"code.language-css .token.property":{color:"#a9b7c6"},"code.language-css .token.property + .token.punctuation":{color:"#a9b7c6"},"code.language-css .token.id":{color:"#ffc66d"},"code.language-css .token.selector > .token.class":{color:"#ffc66d"},"code.language-css .token.selector > .token.attribute":{color:"#ffc66d"},"code.language-css .token.selector > .token.pseudo-class":{color:"#ffc66d"},"code.language-css .token.selector > .token.pseudo-element":{color:"#ffc66d"}}})(q4)),q4}var W4={},vV;function llt(){return vV||(vV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#f8f8f2",background:"none",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#f8f8f2",background:"#282a36",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",borderRadius:"0.3em"},':not(pre) > code[class*="language-"]':{background:"#282a36",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"#6272a4"},prolog:{color:"#6272a4"},doctype:{color:"#6272a4"},cdata:{color:"#6272a4"},punctuation:{color:"#f8f8f2"},".namespace":{Opacity:".7"},property:{color:"#ff79c6"},tag:{color:"#ff79c6"},constant:{color:"#ff79c6"},symbol:{color:"#ff79c6"},deleted:{color:"#ff79c6"},boolean:{color:"#bd93f9"},number:{color:"#bd93f9"},selector:{color:"#50fa7b"},"attr-name":{color:"#50fa7b"},string:{color:"#50fa7b"},char:{color:"#50fa7b"},builtin:{color:"#50fa7b"},inserted:{color:"#50fa7b"},operator:{color:"#f8f8f2"},entity:{color:"#f8f8f2",cursor:"help"},url:{color:"#f8f8f2"},".language-css .token.string":{color:"#f8f8f2"},".style .token.string":{color:"#f8f8f2"},variable:{color:"#f8f8f2"},atrule:{color:"#f1fa8c"},"attr-value":{color:"#f1fa8c"},function:{color:"#f1fa8c"},"class-name":{color:"#f1fa8c"},keyword:{color:"#8be9fd"},regex:{color:"#ffb86c"},important:{color:"#ffb86c",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(W4)),W4}var V4={},SV;function ult(){return SV||(SV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#2a2734",color:"#9a86fd"},'pre[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#2a2734",color:"#9a86fd",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#6a51e6"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#6a51e6"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#6a51e6"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#6a51e6"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#6a51e6"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#6a51e6"},'code[class*="language-"]::selection':{textShadow:"none",background:"#6a51e6"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#6a51e6"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#6c6783"},prolog:{color:"#6c6783"},doctype:{color:"#6c6783"},cdata:{color:"#6c6783"},punctuation:{color:"#6c6783"},namespace:{Opacity:".7"},tag:{color:"#e09142"},operator:{color:"#e09142"},number:{color:"#e09142"},property:{color:"#9a86fd"},function:{color:"#9a86fd"},"tag-id":{color:"#eeebff"},selector:{color:"#eeebff"},"atrule-id":{color:"#eeebff"},"code.language-javascript":{color:"#c4b9fe"},"attr-name":{color:"#c4b9fe"},"code.language-css":{color:"#ffcc99"},"code.language-scss":{color:"#ffcc99"},boolean:{color:"#ffcc99"},string:{color:"#ffcc99"},entity:{color:"#ffcc99",cursor:"help"},url:{color:"#ffcc99"},".language-css .token.string":{color:"#ffcc99"},".language-scss .token.string":{color:"#ffcc99"},".style .token.string":{color:"#ffcc99"},"attr-value":{color:"#ffcc99"},keyword:{color:"#ffcc99"},control:{color:"#ffcc99"},directive:{color:"#ffcc99"},unit:{color:"#ffcc99"},statement:{color:"#ffcc99"},regex:{color:"#ffcc99"},atrule:{color:"#ffcc99"},placeholder:{color:"#ffcc99"},variable:{color:"#ffcc99"},deleted:{textDecoration:"line-through"},inserted:{borderBottom:"1px dotted #eeebff",textDecoration:"none"},italic:{fontStyle:"italic"},important:{fontWeight:"bold",color:"#c4b9fe"},bold:{fontWeight:"bold"},"pre > code.highlight":{Outline:".4em solid #8a75f5",OutlineOffset:".4em"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#2c2937"},".line-numbers .line-numbers-rows > span:before":{color:"#3c3949"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(224, 145, 66, 0.2) 70%, rgba(224, 145, 66, 0))"}}})(V4)),V4}var K4={},EV;function clt(){return EV||(EV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#322d29",color:"#88786d"},'pre[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#322d29",color:"#88786d",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#6f5849"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#6f5849"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#6f5849"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#6f5849"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#6f5849"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#6f5849"},'code[class*="language-"]::selection':{textShadow:"none",background:"#6f5849"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#6f5849"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#6a5f58"},prolog:{color:"#6a5f58"},doctype:{color:"#6a5f58"},cdata:{color:"#6a5f58"},punctuation:{color:"#6a5f58"},namespace:{Opacity:".7"},tag:{color:"#bfa05a"},operator:{color:"#bfa05a"},number:{color:"#bfa05a"},property:{color:"#88786d"},function:{color:"#88786d"},"tag-id":{color:"#fff3eb"},selector:{color:"#fff3eb"},"atrule-id":{color:"#fff3eb"},"code.language-javascript":{color:"#a48774"},"attr-name":{color:"#a48774"},"code.language-css":{color:"#fcc440"},"code.language-scss":{color:"#fcc440"},boolean:{color:"#fcc440"},string:{color:"#fcc440"},entity:{color:"#fcc440",cursor:"help"},url:{color:"#fcc440"},".language-css .token.string":{color:"#fcc440"},".language-scss .token.string":{color:"#fcc440"},".style .token.string":{color:"#fcc440"},"attr-value":{color:"#fcc440"},keyword:{color:"#fcc440"},control:{color:"#fcc440"},directive:{color:"#fcc440"},unit:{color:"#fcc440"},statement:{color:"#fcc440"},regex:{color:"#fcc440"},atrule:{color:"#fcc440"},placeholder:{color:"#fcc440"},variable:{color:"#fcc440"},deleted:{textDecoration:"line-through"},inserted:{borderBottom:"1px dotted #fff3eb",textDecoration:"none"},italic:{fontStyle:"italic"},important:{fontWeight:"bold",color:"#a48774"},bold:{fontWeight:"bold"},"pre > code.highlight":{Outline:".4em solid #816d5f",OutlineOffset:".4em"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#35302b"},".line-numbers .line-numbers-rows > span:before":{color:"#46403d"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(191, 160, 90, 0.2) 70%, rgba(191, 160, 90, 0))"}}})(K4)),K4}var Y4={},xV;function dlt(){return xV||(xV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#2a2d2a",color:"#687d68"},'pre[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#2a2d2a",color:"#687d68",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#435643"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#435643"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#435643"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#435643"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#435643"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#435643"},'code[class*="language-"]::selection':{textShadow:"none",background:"#435643"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#435643"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#535f53"},prolog:{color:"#535f53"},doctype:{color:"#535f53"},cdata:{color:"#535f53"},punctuation:{color:"#535f53"},namespace:{Opacity:".7"},tag:{color:"#a2b34d"},operator:{color:"#a2b34d"},number:{color:"#a2b34d"},property:{color:"#687d68"},function:{color:"#687d68"},"tag-id":{color:"#f0fff0"},selector:{color:"#f0fff0"},"atrule-id":{color:"#f0fff0"},"code.language-javascript":{color:"#b3d6b3"},"attr-name":{color:"#b3d6b3"},"code.language-css":{color:"#e5fb79"},"code.language-scss":{color:"#e5fb79"},boolean:{color:"#e5fb79"},string:{color:"#e5fb79"},entity:{color:"#e5fb79",cursor:"help"},url:{color:"#e5fb79"},".language-css .token.string":{color:"#e5fb79"},".language-scss .token.string":{color:"#e5fb79"},".style .token.string":{color:"#e5fb79"},"attr-value":{color:"#e5fb79"},keyword:{color:"#e5fb79"},control:{color:"#e5fb79"},directive:{color:"#e5fb79"},unit:{color:"#e5fb79"},statement:{color:"#e5fb79"},regex:{color:"#e5fb79"},atrule:{color:"#e5fb79"},placeholder:{color:"#e5fb79"},variable:{color:"#e5fb79"},deleted:{textDecoration:"line-through"},inserted:{borderBottom:"1px dotted #f0fff0",textDecoration:"none"},italic:{fontStyle:"italic"},important:{fontWeight:"bold",color:"#b3d6b3"},bold:{fontWeight:"bold"},"pre > code.highlight":{Outline:".4em solid #5c705c",OutlineOffset:".4em"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#2c302c"},".line-numbers .line-numbers-rows > span:before":{color:"#3b423b"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(162, 179, 77, 0.2) 70%, rgba(162, 179, 77, 0))"}}})(Y4)),Y4}var J4={},wV;function hlt(){return wV||(wV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#faf8f5",color:"#728fcb"},'pre[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#faf8f5",color:"#728fcb",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#faf8f5"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#faf8f5"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#faf8f5"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#faf8f5"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#faf8f5"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#faf8f5"},'code[class*="language-"]::selection':{textShadow:"none",background:"#faf8f5"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#faf8f5"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#b6ad9a"},prolog:{color:"#b6ad9a"},doctype:{color:"#b6ad9a"},cdata:{color:"#b6ad9a"},punctuation:{color:"#b6ad9a"},namespace:{Opacity:".7"},tag:{color:"#063289"},operator:{color:"#063289"},number:{color:"#063289"},property:{color:"#b29762"},function:{color:"#b29762"},"tag-id":{color:"#2d2006"},selector:{color:"#2d2006"},"atrule-id":{color:"#2d2006"},"code.language-javascript":{color:"#896724"},"attr-name":{color:"#896724"},"code.language-css":{color:"#728fcb"},"code.language-scss":{color:"#728fcb"},boolean:{color:"#728fcb"},string:{color:"#728fcb"},entity:{color:"#728fcb",cursor:"help"},url:{color:"#728fcb"},".language-css .token.string":{color:"#728fcb"},".language-scss .token.string":{color:"#728fcb"},".style .token.string":{color:"#728fcb"},"attr-value":{color:"#728fcb"},keyword:{color:"#728fcb"},control:{color:"#728fcb"},directive:{color:"#728fcb"},unit:{color:"#728fcb"},statement:{color:"#728fcb"},regex:{color:"#728fcb"},atrule:{color:"#728fcb"},placeholder:{color:"#93abdc"},variable:{color:"#93abdc"},deleted:{textDecoration:"line-through"},inserted:{borderBottom:"1px dotted #2d2006",textDecoration:"none"},italic:{fontStyle:"italic"},important:{fontWeight:"bold",color:"#896724"},bold:{fontWeight:"bold"},"pre > code.highlight":{Outline:".4em solid #896724",OutlineOffset:".4em"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#ece8de"},".line-numbers .line-numbers-rows > span:before":{color:"#cdc4b1"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(45, 32, 6, 0.2) 70%, rgba(45, 32, 6, 0))"}}})(J4)),J4}var X4={},kV;function plt(){return kV||(kV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#1d262f",color:"#57718e"},'pre[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#1d262f",color:"#57718e",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#004a9e"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#004a9e"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#004a9e"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#004a9e"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#004a9e"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#004a9e"},'code[class*="language-"]::selection':{textShadow:"none",background:"#004a9e"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#004a9e"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#4a5f78"},prolog:{color:"#4a5f78"},doctype:{color:"#4a5f78"},cdata:{color:"#4a5f78"},punctuation:{color:"#4a5f78"},namespace:{Opacity:".7"},tag:{color:"#0aa370"},operator:{color:"#0aa370"},number:{color:"#0aa370"},property:{color:"#57718e"},function:{color:"#57718e"},"tag-id":{color:"#ebf4ff"},selector:{color:"#ebf4ff"},"atrule-id":{color:"#ebf4ff"},"code.language-javascript":{color:"#7eb6f6"},"attr-name":{color:"#7eb6f6"},"code.language-css":{color:"#47ebb4"},"code.language-scss":{color:"#47ebb4"},boolean:{color:"#47ebb4"},string:{color:"#47ebb4"},entity:{color:"#47ebb4",cursor:"help"},url:{color:"#47ebb4"},".language-css .token.string":{color:"#47ebb4"},".language-scss .token.string":{color:"#47ebb4"},".style .token.string":{color:"#47ebb4"},"attr-value":{color:"#47ebb4"},keyword:{color:"#47ebb4"},control:{color:"#47ebb4"},directive:{color:"#47ebb4"},unit:{color:"#47ebb4"},statement:{color:"#47ebb4"},regex:{color:"#47ebb4"},atrule:{color:"#47ebb4"},placeholder:{color:"#47ebb4"},variable:{color:"#47ebb4"},deleted:{textDecoration:"line-through"},inserted:{borderBottom:"1px dotted #ebf4ff",textDecoration:"none"},italic:{fontStyle:"italic"},important:{fontWeight:"bold",color:"#7eb6f6"},bold:{fontWeight:"bold"},"pre > code.highlight":{Outline:".4em solid #34659d",OutlineOffset:".4em"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#1f2932"},".line-numbers .line-numbers-rows > span:before":{color:"#2c3847"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(10, 163, 112, 0.2) 70%, rgba(10, 163, 112, 0))"}}})(X4)),X4}var Z4={},CV;function flt(){return CV||(CV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#24242e",color:"#767693"},'pre[class*="language-"]':{fontFamily:'Consolas, Menlo, Monaco, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", "Courier New", Courier, monospace',fontSize:"14px",lineHeight:"1.375",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",background:"#24242e",color:"#767693",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"#5151e6"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#5151e6"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"#5151e6"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"#5151e6"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#5151e6"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"#5151e6"},'code[class*="language-"]::selection':{textShadow:"none",background:"#5151e6"},'code[class*="language-"] ::selection':{textShadow:"none",background:"#5151e6"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#5b5b76"},prolog:{color:"#5b5b76"},doctype:{color:"#5b5b76"},cdata:{color:"#5b5b76"},punctuation:{color:"#5b5b76"},namespace:{Opacity:".7"},tag:{color:"#dd672c"},operator:{color:"#dd672c"},number:{color:"#dd672c"},property:{color:"#767693"},function:{color:"#767693"},"tag-id":{color:"#ebebff"},selector:{color:"#ebebff"},"atrule-id":{color:"#ebebff"},"code.language-javascript":{color:"#aaaaca"},"attr-name":{color:"#aaaaca"},"code.language-css":{color:"#fe8c52"},"code.language-scss":{color:"#fe8c52"},boolean:{color:"#fe8c52"},string:{color:"#fe8c52"},entity:{color:"#fe8c52",cursor:"help"},url:{color:"#fe8c52"},".language-css .token.string":{color:"#fe8c52"},".language-scss .token.string":{color:"#fe8c52"},".style .token.string":{color:"#fe8c52"},"attr-value":{color:"#fe8c52"},keyword:{color:"#fe8c52"},control:{color:"#fe8c52"},directive:{color:"#fe8c52"},unit:{color:"#fe8c52"},statement:{color:"#fe8c52"},regex:{color:"#fe8c52"},atrule:{color:"#fe8c52"},placeholder:{color:"#fe8c52"},variable:{color:"#fe8c52"},deleted:{textDecoration:"line-through"},inserted:{borderBottom:"1px dotted #ebebff",textDecoration:"none"},italic:{fontStyle:"italic"},important:{fontWeight:"bold",color:"#aaaaca"},bold:{fontWeight:"bold"},"pre > code.highlight":{Outline:".4em solid #7676f4",OutlineOffset:".4em"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#262631"},".line-numbers .line-numbers-rows > span:before":{color:"#393949"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(221, 103, 44, 0.2) 70%, rgba(221, 103, 44, 0))"}}})(Z4)),Z4}var Q4={},TV;function mlt(){return TV||(TV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#393A34",fontFamily:'"Consolas", "Bitstream Vera Sans Mono", "Courier New", Courier, monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",fontSize:".9em",lineHeight:"1.2em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#393A34",fontFamily:'"Consolas", "Bitstream Vera Sans Mono", "Courier New", Courier, monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",fontSize:".9em",lineHeight:"1.2em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",border:"1px solid #dddddd",backgroundColor:"white"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{background:"#b3d4fc"},'pre[class*="language-"] ::-moz-selection':{background:"#b3d4fc"},'code[class*="language-"]::-moz-selection':{background:"#b3d4fc"},'code[class*="language-"] ::-moz-selection':{background:"#b3d4fc"},'pre[class*="language-"]::selection':{background:"#b3d4fc"},'pre[class*="language-"] ::selection':{background:"#b3d4fc"},'code[class*="language-"]::selection':{background:"#b3d4fc"},'code[class*="language-"] ::selection':{background:"#b3d4fc"},':not(pre) > code[class*="language-"]':{padding:".2em",paddingTop:"1px",paddingBottom:"1px",background:"#f8f8f8",border:"1px solid #dddddd"},comment:{color:"#999988",fontStyle:"italic"},prolog:{color:"#999988",fontStyle:"italic"},doctype:{color:"#999988",fontStyle:"italic"},cdata:{color:"#999988",fontStyle:"italic"},namespace:{Opacity:".7"},string:{color:"#e3116c"},"attr-value":{color:"#e3116c"},punctuation:{color:"#393A34"},operator:{color:"#393A34"},entity:{color:"#36acaa"},url:{color:"#36acaa"},symbol:{color:"#36acaa"},number:{color:"#36acaa"},boolean:{color:"#36acaa"},variable:{color:"#36acaa"},constant:{color:"#36acaa"},property:{color:"#36acaa"},regex:{color:"#36acaa"},inserted:{color:"#36acaa"},atrule:{color:"#00a4db"},keyword:{color:"#00a4db"},"attr-name":{color:"#00a4db"},".language-autohotkey .token.selector":{color:"#00a4db"},function:{color:"#9a050f",fontWeight:"bold"},deleted:{color:"#9a050f"},".language-autohotkey .token.tag":{color:"#9a050f"},tag:{color:"#00009f"},selector:{color:"#00009f"},".language-autohotkey .token.keyword":{color:"#00009f"},important:{fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(Q4)),Q4}var e_={},AV;function glt(){return AV||(AV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#ebdbb2",fontFamily:'Consolas, Monaco, "Andale Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#ebdbb2",fontFamily:'Consolas, Monaco, "Andale Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto",background:"#1d2021"},'pre[class*="language-"]::-moz-selection':{color:"#fbf1c7",background:"#7c6f64"},'pre[class*="language-"] ::-moz-selection':{color:"#fbf1c7",background:"#7c6f64"},'code[class*="language-"]::-moz-selection':{color:"#fbf1c7",background:"#7c6f64"},'code[class*="language-"] ::-moz-selection':{color:"#fbf1c7",background:"#7c6f64"},'pre[class*="language-"]::selection':{color:"#fbf1c7",background:"#7c6f64"},'pre[class*="language-"] ::selection':{color:"#fbf1c7",background:"#7c6f64"},'code[class*="language-"]::selection':{color:"#fbf1c7",background:"#7c6f64"},'code[class*="language-"] ::selection':{color:"#fbf1c7",background:"#7c6f64"},':not(pre) > code[class*="language-"]':{background:"#1d2021",padding:"0.1em",borderRadius:"0.3em"},comment:{color:"#a89984"},prolog:{color:"#a89984"},cdata:{color:"#a89984"},delimiter:{color:"#fb4934"},boolean:{color:"#fb4934"},keyword:{color:"#fb4934"},selector:{color:"#fb4934"},important:{color:"#fb4934"},atrule:{color:"#fb4934"},operator:{color:"#a89984"},punctuation:{color:"#a89984"},"attr-name":{color:"#a89984"},tag:{color:"#fabd2f"},"tag.punctuation":{color:"#fabd2f"},doctype:{color:"#fabd2f"},builtin:{color:"#fabd2f"},entity:{color:"#d3869b"},number:{color:"#d3869b"},symbol:{color:"#d3869b"},property:{color:"#fb4934"},constant:{color:"#fb4934"},variable:{color:"#fb4934"},string:{color:"#b8bb26"},char:{color:"#b8bb26"},"attr-value":{color:"#a89984"},"attr-value.punctuation":{color:"#a89984"},url:{color:"#b8bb26",textDecoration:"underline"},function:{color:"#fabd2f"},regex:{background:"#b8bb26"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},inserted:{background:"#a89984"},deleted:{background:"#fb4934"}}})(e_)),e_}var t_={},FV;function blt(){return FV||(FV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#3c3836",fontFamily:'Consolas, Monaco, "Andale Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#3c3836",fontFamily:'Consolas, Monaco, "Andale Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto",background:"#f9f5d7"},'pre[class*="language-"]::-moz-selection':{color:"#282828",background:"#a89984"},'pre[class*="language-"] ::-moz-selection':{color:"#282828",background:"#a89984"},'code[class*="language-"]::-moz-selection':{color:"#282828",background:"#a89984"},'code[class*="language-"] ::-moz-selection':{color:"#282828",background:"#a89984"},'pre[class*="language-"]::selection':{color:"#282828",background:"#a89984"},'pre[class*="language-"] ::selection':{color:"#282828",background:"#a89984"},'code[class*="language-"]::selection':{color:"#282828",background:"#a89984"},'code[class*="language-"] ::selection':{color:"#282828",background:"#a89984"},':not(pre) > code[class*="language-"]':{background:"#f9f5d7",padding:"0.1em",borderRadius:"0.3em"},comment:{color:"#7c6f64"},prolog:{color:"#7c6f64"},cdata:{color:"#7c6f64"},delimiter:{color:"#9d0006"},boolean:{color:"#9d0006"},keyword:{color:"#9d0006"},selector:{color:"#9d0006"},important:{color:"#9d0006"},atrule:{color:"#9d0006"},operator:{color:"#7c6f64"},punctuation:{color:"#7c6f64"},"attr-name":{color:"#7c6f64"},tag:{color:"#b57614"},"tag.punctuation":{color:"#b57614"},doctype:{color:"#b57614"},builtin:{color:"#b57614"},entity:{color:"#8f3f71"},number:{color:"#8f3f71"},symbol:{color:"#8f3f71"},property:{color:"#9d0006"},constant:{color:"#9d0006"},variable:{color:"#9d0006"},string:{color:"#797403"},char:{color:"#797403"},"attr-value":{color:"#7c6f64"},"attr-value.punctuation":{color:"#7c6f64"},url:{color:"#797403",textDecoration:"underline"},function:{color:"#b57614"},regex:{background:"#797403"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},inserted:{background:"#7c6f64"},deleted:{background:"#9d0006"}}})(t_)),t_}var r_={},_V;function ylt(){return _V||(_V=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={"code[class*='language-']":{color:"#d6e7ff",background:"#030314",textShadow:"none",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',fontSize:"1em",lineHeight:"1.5",letterSpacing:".2px",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",textAlign:"left",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},"pre[class*='language-']":{color:"#d6e7ff",background:"#030314",textShadow:"none",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',fontSize:"1em",lineHeight:"1.5",letterSpacing:".2px",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",textAlign:"left",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",border:"1px solid #2a4555",borderRadius:"5px",padding:"1.5em 1em",margin:"1em 0",overflow:"auto"},"pre[class*='language-']::-moz-selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},"pre[class*='language-'] ::-moz-selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},"code[class*='language-']::-moz-selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},"code[class*='language-'] ::-moz-selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},"pre[class*='language-']::selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},"pre[class*='language-'] ::selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},"code[class*='language-']::selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},"code[class*='language-'] ::selection":{color:"inherit",background:"#1d3b54",textShadow:"none"},":not(pre) > code[class*='language-']":{color:"#f0f6f6",background:"#2a4555",padding:"0.2em 0.3em",borderRadius:"0.2em",boxDecorationBreak:"clone"},comment:{color:"#446e69"},prolog:{color:"#446e69"},doctype:{color:"#446e69"},cdata:{color:"#446e69"},punctuation:{color:"#d6b007"},property:{color:"#d6e7ff"},tag:{color:"#d6e7ff"},boolean:{color:"#d6e7ff"},number:{color:"#d6e7ff"},constant:{color:"#d6e7ff"},symbol:{color:"#d6e7ff"},deleted:{color:"#d6e7ff"},selector:{color:"#e60067"},"attr-name":{color:"#e60067"},builtin:{color:"#e60067"},inserted:{color:"#e60067"},string:{color:"#49c6ec"},char:{color:"#49c6ec"},operator:{color:"#ec8e01",background:"transparent"},entity:{color:"#ec8e01",background:"transparent"},url:{color:"#ec8e01",background:"transparent"},".language-css .token.string":{color:"#ec8e01",background:"transparent"},".style .token.string":{color:"#ec8e01",background:"transparent"},atrule:{color:"#0fe468"},"attr-value":{color:"#0fe468"},keyword:{color:"#0fe468"},function:{color:"#78f3e9"},"class-name":{color:"#78f3e9"},regex:{color:"#d6e7ff"},important:{color:"#d6e7ff"},variable:{color:"#d6e7ff"}}})(r_)),r_}var n_={},RV;function vlt(){return RV||(RV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{fontFamily:'"Fira Mono", Menlo, Monaco, "Lucida Console", "Courier New", Courier, monospace',fontSize:"16px",lineHeight:"1.375",direction:"ltr",textAlign:"left",wordSpacing:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",whiteSpace:"pre-wrap",wordBreak:"break-all",wordWrap:"break-word",background:"#322931",color:"#b9b5b8"},'pre[class*="language-"]':{fontFamily:'"Fira Mono", Menlo, Monaco, "Lucida Console", "Courier New", Courier, monospace',fontSize:"16px",lineHeight:"1.375",direction:"ltr",textAlign:"left",wordSpacing:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",whiteSpace:"pre-wrap",wordBreak:"break-all",wordWrap:"break-word",background:"#322931",color:"#b9b5b8",padding:"1em",margin:".5em 0",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},':not(pre) > code[class*="language-"]':{padding:".1em",borderRadius:".3em"},comment:{color:"#797379"},prolog:{color:"#797379"},doctype:{color:"#797379"},cdata:{color:"#797379"},punctuation:{color:"#b9b5b8"},".namespace":{Opacity:".7"},null:{color:"#fd8b19"},operator:{color:"#fd8b19"},boolean:{color:"#fd8b19"},number:{color:"#fd8b19"},property:{color:"#fdcc59"},tag:{color:"#1290bf"},string:{color:"#149b93"},selector:{color:"#c85e7c"},"attr-name":{color:"#fd8b19"},entity:{color:"#149b93",cursor:"help"},url:{color:"#149b93"},".language-css .token.string":{color:"#149b93"},".style .token.string":{color:"#149b93"},"attr-value":{color:"#8fc13e"},keyword:{color:"#8fc13e"},control:{color:"#8fc13e"},directive:{color:"#8fc13e"},unit:{color:"#8fc13e"},statement:{color:"#149b93"},regex:{color:"#149b93"},atrule:{color:"#149b93"},placeholder:{color:"#1290bf"},variable:{color:"#1290bf"},important:{color:"#dd464c",fontWeight:"bold"},"pre > code.highlight":{Outline:".4em solid red",OutlineOffset:".4em"}}})(n_)),n_}var a_={},DV;function Slt(){return DV||(DV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#f8f8f2",background:"none",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Monaco, Consolas, 'Andale Mono', 'Ubuntu Mono', monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#f8f8f2",background:"#263E52",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Monaco, Consolas, 'Andale Mono', 'Ubuntu Mono', monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",borderRadius:"0.3em"},':not(pre) > code[class*="language-"]':{background:"#263E52",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"#5c98cd"},prolog:{color:"#5c98cd"},doctype:{color:"#5c98cd"},cdata:{color:"#5c98cd"},punctuation:{color:"#f8f8f2"},".namespace":{Opacity:".7"},property:{color:"#F05E5D"},tag:{color:"#F05E5D"},constant:{color:"#F05E5D"},symbol:{color:"#F05E5D"},deleted:{color:"#F05E5D"},boolean:{color:"#BC94F9"},number:{color:"#BC94F9"},selector:{color:"#FCFCD6"},"attr-name":{color:"#FCFCD6"},string:{color:"#FCFCD6"},char:{color:"#FCFCD6"},builtin:{color:"#FCFCD6"},inserted:{color:"#FCFCD6"},operator:{color:"#f8f8f2"},entity:{color:"#f8f8f2",cursor:"help"},url:{color:"#f8f8f2"},".language-css .token.string":{color:"#f8f8f2"},".style .token.string":{color:"#f8f8f2"},variable:{color:"#f8f8f2"},atrule:{color:"#66D8EF"},"attr-value":{color:"#66D8EF"},function:{color:"#66D8EF"},"class-name":{color:"#66D8EF"},keyword:{color:"#6EB26E"},regex:{color:"#F05E5D"},important:{color:"#F05E5D",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(a_)),a_}var i_={},NV;function Elt(){return NV||(NV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",color:"#eee",background:"#2f2f2f",fontFamily:"Roboto Mono, monospace",fontSize:"1em",lineHeight:"1.5em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",color:"#eee",background:"#2f2f2f",fontFamily:"Roboto Mono, monospace",fontSize:"1em",lineHeight:"1.5em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",overflow:"auto",position:"relative",margin:"0.5em 0",padding:"1.25em 1em"},'code[class*="language-"]::-moz-selection':{background:"#363636"},'pre[class*="language-"]::-moz-selection':{background:"#363636"},'code[class*="language-"] ::-moz-selection':{background:"#363636"},'pre[class*="language-"] ::-moz-selection':{background:"#363636"},'code[class*="language-"]::selection':{background:"#363636"},'pre[class*="language-"]::selection':{background:"#363636"},'code[class*="language-"] ::selection':{background:"#363636"},'pre[class*="language-"] ::selection':{background:"#363636"},':not(pre) > code[class*="language-"]':{whiteSpace:"normal",borderRadius:"0.2em",padding:"0.1em"},".language-css > code":{color:"#fd9170"},".language-sass > code":{color:"#fd9170"},".language-scss > code":{color:"#fd9170"},'[class*="language-"] .namespace':{Opacity:"0.7"},atrule:{color:"#c792ea"},"attr-name":{color:"#ffcb6b"},"attr-value":{color:"#a5e844"},attribute:{color:"#a5e844"},boolean:{color:"#c792ea"},builtin:{color:"#ffcb6b"},cdata:{color:"#80cbc4"},char:{color:"#80cbc4"},class:{color:"#ffcb6b"},"class-name":{color:"#f2ff00"},comment:{color:"#616161"},constant:{color:"#c792ea"},deleted:{color:"#ff6666"},doctype:{color:"#616161"},entity:{color:"#ff6666"},function:{color:"#c792ea"},hexcode:{color:"#f2ff00"},id:{color:"#c792ea",fontWeight:"bold"},important:{color:"#c792ea",fontWeight:"bold"},inserted:{color:"#80cbc4"},keyword:{color:"#c792ea"},number:{color:"#fd9170"},operator:{color:"#89ddff"},prolog:{color:"#616161"},property:{color:"#80cbc4"},"pseudo-class":{color:"#a5e844"},"pseudo-element":{color:"#a5e844"},punctuation:{color:"#89ddff"},regex:{color:"#f2ff00"},selector:{color:"#ff6666"},string:{color:"#a5e844"},symbol:{color:"#c792ea"},tag:{color:"#ff6666"},unit:{color:"#fd9170"},url:{color:"#ff6666"},variable:{color:"#ff6666"}}})(i_)),i_}var o_={},LV;function xlt(){return LV||(LV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",color:"#90a4ae",background:"#fafafa",fontFamily:"Roboto Mono, monospace",fontSize:"1em",lineHeight:"1.5em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",color:"#90a4ae",background:"#fafafa",fontFamily:"Roboto Mono, monospace",fontSize:"1em",lineHeight:"1.5em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",overflow:"auto",position:"relative",margin:"0.5em 0",padding:"1.25em 1em"},'code[class*="language-"]::-moz-selection':{background:"#cceae7",color:"#263238"},'pre[class*="language-"]::-moz-selection':{background:"#cceae7",color:"#263238"},'code[class*="language-"] ::-moz-selection':{background:"#cceae7",color:"#263238"},'pre[class*="language-"] ::-moz-selection':{background:"#cceae7",color:"#263238"},'code[class*="language-"]::selection':{background:"#cceae7",color:"#263238"},'pre[class*="language-"]::selection':{background:"#cceae7",color:"#263238"},'code[class*="language-"] ::selection':{background:"#cceae7",color:"#263238"},'pre[class*="language-"] ::selection':{background:"#cceae7",color:"#263238"},':not(pre) > code[class*="language-"]':{whiteSpace:"normal",borderRadius:"0.2em",padding:"0.1em"},".language-css > code":{color:"#f76d47"},".language-sass > code":{color:"#f76d47"},".language-scss > code":{color:"#f76d47"},'[class*="language-"] .namespace':{Opacity:"0.7"},atrule:{color:"#7c4dff"},"attr-name":{color:"#39adb5"},"attr-value":{color:"#f6a434"},attribute:{color:"#f6a434"},boolean:{color:"#7c4dff"},builtin:{color:"#39adb5"},cdata:{color:"#39adb5"},char:{color:"#39adb5"},class:{color:"#39adb5"},"class-name":{color:"#6182b8"},comment:{color:"#aabfc9"},constant:{color:"#7c4dff"},deleted:{color:"#e53935"},doctype:{color:"#aabfc9"},entity:{color:"#e53935"},function:{color:"#7c4dff"},hexcode:{color:"#f76d47"},id:{color:"#7c4dff",fontWeight:"bold"},important:{color:"#7c4dff",fontWeight:"bold"},inserted:{color:"#39adb5"},keyword:{color:"#7c4dff"},number:{color:"#f76d47"},operator:{color:"#39adb5"},prolog:{color:"#aabfc9"},property:{color:"#39adb5"},"pseudo-class":{color:"#f6a434"},"pseudo-element":{color:"#f6a434"},punctuation:{color:"#39adb5"},regex:{color:"#6182b8"},selector:{color:"#e53935"},string:{color:"#f6a434"},symbol:{color:"#7c4dff"},tag:{color:"#e53935"},unit:{color:"#f76d47"},url:{color:"#e53935"},variable:{color:"#e53935"}}})(o_)),o_}var s_={},MV;function wlt(){return MV||(MV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",color:"#c3cee3",background:"#263238",fontFamily:"Roboto Mono, monospace",fontSize:"1em",lineHeight:"1.5em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",color:"#c3cee3",background:"#263238",fontFamily:"Roboto Mono, monospace",fontSize:"1em",lineHeight:"1.5em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",overflow:"auto",position:"relative",margin:"0.5em 0",padding:"1.25em 1em"},'code[class*="language-"]::-moz-selection':{background:"#363636"},'pre[class*="language-"]::-moz-selection':{background:"#363636"},'code[class*="language-"] ::-moz-selection':{background:"#363636"},'pre[class*="language-"] ::-moz-selection':{background:"#363636"},'code[class*="language-"]::selection':{background:"#363636"},'pre[class*="language-"]::selection':{background:"#363636"},'code[class*="language-"] ::selection':{background:"#363636"},'pre[class*="language-"] ::selection':{background:"#363636"},':not(pre) > code[class*="language-"]':{whiteSpace:"normal",borderRadius:"0.2em",padding:"0.1em"},".language-css > code":{color:"#fd9170"},".language-sass > code":{color:"#fd9170"},".language-scss > code":{color:"#fd9170"},'[class*="language-"] .namespace':{Opacity:"0.7"},atrule:{color:"#c792ea"},"attr-name":{color:"#ffcb6b"},"attr-value":{color:"#c3e88d"},attribute:{color:"#c3e88d"},boolean:{color:"#c792ea"},builtin:{color:"#ffcb6b"},cdata:{color:"#80cbc4"},char:{color:"#80cbc4"},class:{color:"#ffcb6b"},"class-name":{color:"#f2ff00"},color:{color:"#f2ff00"},comment:{color:"#546e7a"},constant:{color:"#c792ea"},deleted:{color:"#f07178"},doctype:{color:"#546e7a"},entity:{color:"#f07178"},function:{color:"#c792ea"},hexcode:{color:"#f2ff00"},id:{color:"#c792ea",fontWeight:"bold"},important:{color:"#c792ea",fontWeight:"bold"},inserted:{color:"#80cbc4"},keyword:{color:"#c792ea",fontStyle:"italic"},number:{color:"#fd9170"},operator:{color:"#89ddff"},prolog:{color:"#546e7a"},property:{color:"#80cbc4"},"pseudo-class":{color:"#c3e88d"},"pseudo-element":{color:"#c3e88d"},punctuation:{color:"#89ddff"},regex:{color:"#f2ff00"},selector:{color:"#f07178"},string:{color:"#c3e88d"},symbol:{color:"#c792ea"},tag:{color:"#f07178"},unit:{color:"#f07178"},url:{color:"#fd9170"},variable:{color:"#f07178"}}})(s_)),s_}var l_={},IV;function klt(){return IV||(IV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#d6deeb",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",fontSize:"1em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"white",fontFamily:'Consolas, Monaco, "Andale Mono", "Ubuntu Mono", monospace',textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",fontSize:"1em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto",background:"#011627"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'pre[class*="language-"]::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"]::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"] ::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},':not(pre) > code[class*="language-"]':{color:"white",background:"#011627",padding:"0.1em",borderRadius:"0.3em",whiteSpace:"normal"},comment:{color:"rgb(99, 119, 119)",fontStyle:"italic"},prolog:{color:"rgb(99, 119, 119)",fontStyle:"italic"},cdata:{color:"rgb(99, 119, 119)",fontStyle:"italic"},punctuation:{color:"rgb(199, 146, 234)"},".namespace":{color:"rgb(178, 204, 214)"},deleted:{color:"rgba(239, 83, 80, 0.56)",fontStyle:"italic"},symbol:{color:"rgb(128, 203, 196)"},property:{color:"rgb(128, 203, 196)"},tag:{color:"rgb(127, 219, 202)"},operator:{color:"rgb(127, 219, 202)"},keyword:{color:"rgb(127, 219, 202)"},boolean:{color:"rgb(255, 88, 116)"},number:{color:"rgb(247, 140, 108)"},constant:{color:"rgb(130, 170, 255)"},function:{color:"rgb(130, 170, 255)"},builtin:{color:"rgb(130, 170, 255)"},char:{color:"rgb(130, 170, 255)"},selector:{color:"rgb(199, 146, 234)",fontStyle:"italic"},doctype:{color:"rgb(199, 146, 234)",fontStyle:"italic"},"attr-name":{color:"rgb(173, 219, 103)",fontStyle:"italic"},inserted:{color:"rgb(173, 219, 103)",fontStyle:"italic"},string:{color:"rgb(173, 219, 103)"},url:{color:"rgb(173, 219, 103)"},entity:{color:"rgb(173, 219, 103)"},".language-css .token.string":{color:"rgb(173, 219, 103)"},".style .token.string":{color:"rgb(173, 219, 103)"},"class-name":{color:"rgb(255, 203, 139)"},atrule:{color:"rgb(255, 203, 139)"},"attr-value":{color:"rgb(255, 203, 139)"},regex:{color:"rgb(214, 222, 235)"},important:{color:"rgb(214, 222, 235)",fontWeight:"bold"},variable:{color:"rgb(214, 222, 235)"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(l_)),l_}var u_={},OV;function Clt(){return OV||(OV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#f8f8f2",background:"none",fontFamily:`"Fira Code", Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace`,textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#f8f8f2",background:"#2E3440",fontFamily:`"Fira Code", Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace`,textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",borderRadius:"0.3em"},':not(pre) > code[class*="language-"]':{background:"#2E3440",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"#636f88"},prolog:{color:"#636f88"},doctype:{color:"#636f88"},cdata:{color:"#636f88"},punctuation:{color:"#81A1C1"},".namespace":{Opacity:".7"},property:{color:"#81A1C1"},tag:{color:"#81A1C1"},constant:{color:"#81A1C1"},symbol:{color:"#81A1C1"},deleted:{color:"#81A1C1"},number:{color:"#B48EAD"},boolean:{color:"#81A1C1"},selector:{color:"#A3BE8C"},"attr-name":{color:"#A3BE8C"},string:{color:"#A3BE8C"},char:{color:"#A3BE8C"},builtin:{color:"#A3BE8C"},inserted:{color:"#A3BE8C"},operator:{color:"#81A1C1"},entity:{color:"#81A1C1",cursor:"help"},url:{color:"#81A1C1"},".language-css .token.string":{color:"#81A1C1"},".style .token.string":{color:"#81A1C1"},variable:{color:"#81A1C1"},atrule:{color:"#88C0D0"},"attr-value":{color:"#88C0D0"},function:{color:"#88C0D0"},"class-name":{color:"#88C0D0"},keyword:{color:"#81A1C1"},regex:{color:"#EBCB8B"},important:{color:"#EBCB8B",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(u_)),u_}var c_={},BV;function Tlt(){return BV||(BV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{background:"hsl(220, 13%, 18%)",color:"hsl(220, 14%, 71%)",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:'"Fira Code", "Fira Mono", Menlo, Consolas, "DejaVu Sans Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"2",OTabSize:"2",tabSize:"2",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{background:"hsl(220, 13%, 18%)",color:"hsl(220, 14%, 71%)",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:'"Fira Code", "Fira Mono", Menlo, Consolas, "DejaVu Sans Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"2",OTabSize:"2",tabSize:"2",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto",borderRadius:"0.3em"},'code[class*="language-"]::-moz-selection':{background:"hsl(220, 13%, 28%)",color:"inherit",textShadow:"none"},'code[class*="language-"] *::-moz-selection':{background:"hsl(220, 13%, 28%)",color:"inherit",textShadow:"none"},'pre[class*="language-"] *::-moz-selection':{background:"hsl(220, 13%, 28%)",color:"inherit",textShadow:"none"},'code[class*="language-"]::selection':{background:"hsl(220, 13%, 28%)",color:"inherit",textShadow:"none"},'code[class*="language-"] *::selection':{background:"hsl(220, 13%, 28%)",color:"inherit",textShadow:"none"},'pre[class*="language-"] *::selection':{background:"hsl(220, 13%, 28%)",color:"inherit",textShadow:"none"},':not(pre) > code[class*="language-"]':{padding:"0.2em 0.3em",borderRadius:"0.3em",whiteSpace:"normal"},comment:{color:"hsl(220, 10%, 40%)",fontStyle:"italic"},prolog:{color:"hsl(220, 10%, 40%)"},cdata:{color:"hsl(220, 10%, 40%)"},doctype:{color:"hsl(220, 14%, 71%)"},punctuation:{color:"hsl(220, 14%, 71%)"},entity:{color:"hsl(220, 14%, 71%)",cursor:"help"},"attr-name":{color:"hsl(29, 54%, 61%)"},"class-name":{color:"hsl(29, 54%, 61%)"},boolean:{color:"hsl(29, 54%, 61%)"},constant:{color:"hsl(29, 54%, 61%)"},number:{color:"hsl(29, 54%, 61%)"},atrule:{color:"hsl(29, 54%, 61%)"},keyword:{color:"hsl(286, 60%, 67%)"},property:{color:"hsl(355, 65%, 65%)"},tag:{color:"hsl(355, 65%, 65%)"},symbol:{color:"hsl(355, 65%, 65%)"},deleted:{color:"hsl(355, 65%, 65%)"},important:{color:"hsl(355, 65%, 65%)"},selector:{color:"hsl(95, 38%, 62%)"},string:{color:"hsl(95, 38%, 62%)"},char:{color:"hsl(95, 38%, 62%)"},builtin:{color:"hsl(95, 38%, 62%)"},inserted:{color:"hsl(95, 38%, 62%)"},regex:{color:"hsl(95, 38%, 62%)"},"attr-value":{color:"hsl(95, 38%, 62%)"},"attr-value > .token.punctuation":{color:"hsl(95, 38%, 62%)"},variable:{color:"hsl(207, 82%, 66%)"},operator:{color:"hsl(207, 82%, 66%)"},function:{color:"hsl(207, 82%, 66%)"},url:{color:"hsl(187, 47%, 55%)"},"attr-value > .token.punctuation.attr-equals":{color:"hsl(220, 14%, 71%)"},"special-attr > .token.attr-value > .token.value.css":{color:"hsl(220, 14%, 71%)"},".language-css .token.selector":{color:"hsl(355, 65%, 65%)"},".language-css .token.property":{color:"hsl(220, 14%, 71%)"},".language-css .token.function":{color:"hsl(187, 47%, 55%)"},".language-css .token.url > .token.function":{color:"hsl(187, 47%, 55%)"},".language-css .token.url > .token.string.url":{color:"hsl(95, 38%, 62%)"},".language-css .token.important":{color:"hsl(286, 60%, 67%)"},".language-css .token.atrule .token.rule":{color:"hsl(286, 60%, 67%)"},".language-javascript .token.operator":{color:"hsl(286, 60%, 67%)"},".language-javascript .token.template-string > .token.interpolation > .token.interpolation-punctuation.punctuation":{color:"hsl(5, 48%, 51%)"},".language-json .token.operator":{color:"hsl(220, 14%, 71%)"},".language-json .token.null.keyword":{color:"hsl(29, 54%, 61%)"},".language-markdown .token.url":{color:"hsl(220, 14%, 71%)"},".language-markdown .token.url > .token.operator":{color:"hsl(220, 14%, 71%)"},".language-markdown .token.url-reference.url > .token.string":{color:"hsl(220, 14%, 71%)"},".language-markdown .token.url > .token.content":{color:"hsl(207, 82%, 66%)"},".language-markdown .token.url > .token.url":{color:"hsl(187, 47%, 55%)"},".language-markdown .token.url-reference.url":{color:"hsl(187, 47%, 55%)"},".language-markdown .token.blockquote.punctuation":{color:"hsl(220, 10%, 40%)",fontStyle:"italic"},".language-markdown .token.hr.punctuation":{color:"hsl(220, 10%, 40%)",fontStyle:"italic"},".language-markdown .token.code-snippet":{color:"hsl(95, 38%, 62%)"},".language-markdown .token.bold .token.content":{color:"hsl(29, 54%, 61%)"},".language-markdown .token.italic .token.content":{color:"hsl(286, 60%, 67%)"},".language-markdown .token.strike .token.content":{color:"hsl(355, 65%, 65%)"},".language-markdown .token.strike .token.punctuation":{color:"hsl(355, 65%, 65%)"},".language-markdown .token.list.punctuation":{color:"hsl(355, 65%, 65%)"},".language-markdown .token.title.important > .token.punctuation":{color:"hsl(355, 65%, 65%)"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},namespace:{Opacity:"0.8"},"token.tab:not(:empty):before":{color:"hsla(220, 14%, 71%, 0.15)",textShadow:"none"},"token.cr:before":{color:"hsla(220, 14%, 71%, 0.15)",textShadow:"none"},"token.lf:before":{color:"hsla(220, 14%, 71%, 0.15)",textShadow:"none"},"token.space:before":{color:"hsla(220, 14%, 71%, 0.15)",textShadow:"none"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item":{marginRight:"0.4em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button":{background:"hsl(220, 13%, 26%)",color:"hsl(220, 9%, 55%)",padding:"0.1em 0.4em",borderRadius:"0.3em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a":{background:"hsl(220, 13%, 26%)",color:"hsl(220, 9%, 55%)",padding:"0.1em 0.4em",borderRadius:"0.3em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span":{background:"hsl(220, 13%, 26%)",color:"hsl(220, 9%, 55%)",padding:"0.1em 0.4em",borderRadius:"0.3em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:hover":{background:"hsl(220, 13%, 28%)",color:"hsl(220, 14%, 71%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:focus":{background:"hsl(220, 13%, 28%)",color:"hsl(220, 14%, 71%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:hover":{background:"hsl(220, 13%, 28%)",color:"hsl(220, 14%, 71%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:focus":{background:"hsl(220, 13%, 28%)",color:"hsl(220, 14%, 71%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:hover":{background:"hsl(220, 13%, 28%)",color:"hsl(220, 14%, 71%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:focus":{background:"hsl(220, 13%, 28%)",color:"hsl(220, 14%, 71%)"},".line-highlight.line-highlight":{background:"hsla(220, 100%, 80%, 0.04)"},".line-highlight.line-highlight:before":{background:"hsl(220, 13%, 26%)",color:"hsl(220, 14%, 71%)",padding:"0.1em 0.6em",borderRadius:"0.3em",boxShadow:"0 2px 0 0 rgba(0, 0, 0, 0.2)"},".line-highlight.line-highlight[data-end]:after":{background:"hsl(220, 13%, 26%)",color:"hsl(220, 14%, 71%)",padding:"0.1em 0.6em",borderRadius:"0.3em",boxShadow:"0 2px 0 0 rgba(0, 0, 0, 0.2)"},"pre[id].linkable-line-numbers.linkable-line-numbers span.line-numbers-rows > span:hover:before":{backgroundColor:"hsla(220, 100%, 80%, 0.04)"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"hsla(220, 14%, 71%, 0.15)"},".command-line .command-line-prompt":{borderRightColor:"hsla(220, 14%, 71%, 0.15)"},".line-numbers .line-numbers-rows > span:before":{color:"hsl(220, 14%, 45%)"},".command-line .command-line-prompt > span:before":{color:"hsl(220, 14%, 45%)"},".rainbow-braces .token.token.punctuation.brace-level-1":{color:"hsl(355, 65%, 65%)"},".rainbow-braces .token.token.punctuation.brace-level-5":{color:"hsl(355, 65%, 65%)"},".rainbow-braces .token.token.punctuation.brace-level-9":{color:"hsl(355, 65%, 65%)"},".rainbow-braces .token.token.punctuation.brace-level-2":{color:"hsl(95, 38%, 62%)"},".rainbow-braces .token.token.punctuation.brace-level-6":{color:"hsl(95, 38%, 62%)"},".rainbow-braces .token.token.punctuation.brace-level-10":{color:"hsl(95, 38%, 62%)"},".rainbow-braces .token.token.punctuation.brace-level-3":{color:"hsl(207, 82%, 66%)"},".rainbow-braces .token.token.punctuation.brace-level-7":{color:"hsl(207, 82%, 66%)"},".rainbow-braces .token.token.punctuation.brace-level-11":{color:"hsl(207, 82%, 66%)"},".rainbow-braces .token.token.punctuation.brace-level-4":{color:"hsl(286, 60%, 67%)"},".rainbow-braces .token.token.punctuation.brace-level-8":{color:"hsl(286, 60%, 67%)"},".rainbow-braces .token.token.punctuation.brace-level-12":{color:"hsl(286, 60%, 67%)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)":{backgroundColor:"hsla(353, 100%, 66%, 0.15)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)":{backgroundColor:"hsla(353, 100%, 66%, 0.15)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix) *::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix) *::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)":{backgroundColor:"hsla(137, 100%, 55%, 0.15)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)":{backgroundColor:"hsla(137, 100%, 55%, 0.15)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix) *::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix) *::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},".prism-previewer.prism-previewer:before":{borderColor:"hsl(224, 13%, 17%)"},".prism-previewer-gradient.prism-previewer-gradient div":{borderColor:"hsl(224, 13%, 17%)",borderRadius:"0.3em"},".prism-previewer-color.prism-previewer-color:before":{borderRadius:"0.3em"},".prism-previewer-easing.prism-previewer-easing:before":{borderRadius:"0.3em"},".prism-previewer.prism-previewer:after":{borderTopColor:"hsl(224, 13%, 17%)"},".prism-previewer-flipped.prism-previewer-flipped.after":{borderBottomColor:"hsl(224, 13%, 17%)"},".prism-previewer-angle.prism-previewer-angle:before":{background:"hsl(219, 13%, 22%)"},".prism-previewer-time.prism-previewer-time:before":{background:"hsl(219, 13%, 22%)"},".prism-previewer-easing.prism-previewer-easing":{background:"hsl(219, 13%, 22%)"},".prism-previewer-angle.prism-previewer-angle circle":{stroke:"hsl(220, 14%, 71%)",strokeOpacity:"1"},".prism-previewer-time.prism-previewer-time circle":{stroke:"hsl(220, 14%, 71%)",strokeOpacity:"1"},".prism-previewer-easing.prism-previewer-easing circle":{stroke:"hsl(220, 14%, 71%)",fill:"transparent"},".prism-previewer-easing.prism-previewer-easing path":{stroke:"hsl(220, 14%, 71%)"},".prism-previewer-easing.prism-previewer-easing line":{stroke:"hsl(220, 14%, 71%)"}}})(c_)),c_}var d_={},PV;function Alt(){return PV||(PV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{background:"hsl(230, 1%, 98%)",color:"hsl(230, 8%, 24%)",fontFamily:'"Fira Code", "Fira Mono", Menlo, Consolas, "DejaVu Sans Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"2",OTabSize:"2",tabSize:"2",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{background:"hsl(230, 1%, 98%)",color:"hsl(230, 8%, 24%)",fontFamily:'"Fira Code", "Fira Mono", Menlo, Consolas, "DejaVu Sans Mono", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"2",OTabSize:"2",tabSize:"2",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:"0.5em 0",overflow:"auto",borderRadius:"0.3em"},'code[class*="language-"]::-moz-selection':{background:"hsl(230, 1%, 90%)",color:"inherit"},'code[class*="language-"] *::-moz-selection':{background:"hsl(230, 1%, 90%)",color:"inherit"},'pre[class*="language-"] *::-moz-selection':{background:"hsl(230, 1%, 90%)",color:"inherit"},'code[class*="language-"]::selection':{background:"hsl(230, 1%, 90%)",color:"inherit"},'code[class*="language-"] *::selection':{background:"hsl(230, 1%, 90%)",color:"inherit"},'pre[class*="language-"] *::selection':{background:"hsl(230, 1%, 90%)",color:"inherit"},':not(pre) > code[class*="language-"]':{padding:"0.2em 0.3em",borderRadius:"0.3em",whiteSpace:"normal"},comment:{color:"hsl(230, 4%, 64%)",fontStyle:"italic"},prolog:{color:"hsl(230, 4%, 64%)"},cdata:{color:"hsl(230, 4%, 64%)"},doctype:{color:"hsl(230, 8%, 24%)"},punctuation:{color:"hsl(230, 8%, 24%)"},entity:{color:"hsl(230, 8%, 24%)",cursor:"help"},"attr-name":{color:"hsl(35, 99%, 36%)"},"class-name":{color:"hsl(35, 99%, 36%)"},boolean:{color:"hsl(35, 99%, 36%)"},constant:{color:"hsl(35, 99%, 36%)"},number:{color:"hsl(35, 99%, 36%)"},atrule:{color:"hsl(35, 99%, 36%)"},keyword:{color:"hsl(301, 63%, 40%)"},property:{color:"hsl(5, 74%, 59%)"},tag:{color:"hsl(5, 74%, 59%)"},symbol:{color:"hsl(5, 74%, 59%)"},deleted:{color:"hsl(5, 74%, 59%)"},important:{color:"hsl(5, 74%, 59%)"},selector:{color:"hsl(119, 34%, 47%)"},string:{color:"hsl(119, 34%, 47%)"},char:{color:"hsl(119, 34%, 47%)"},builtin:{color:"hsl(119, 34%, 47%)"},inserted:{color:"hsl(119, 34%, 47%)"},regex:{color:"hsl(119, 34%, 47%)"},"attr-value":{color:"hsl(119, 34%, 47%)"},"attr-value > .token.punctuation":{color:"hsl(119, 34%, 47%)"},variable:{color:"hsl(221, 87%, 60%)"},operator:{color:"hsl(221, 87%, 60%)"},function:{color:"hsl(221, 87%, 60%)"},url:{color:"hsl(198, 99%, 37%)"},"attr-value > .token.punctuation.attr-equals":{color:"hsl(230, 8%, 24%)"},"special-attr > .token.attr-value > .token.value.css":{color:"hsl(230, 8%, 24%)"},".language-css .token.selector":{color:"hsl(5, 74%, 59%)"},".language-css .token.property":{color:"hsl(230, 8%, 24%)"},".language-css .token.function":{color:"hsl(198, 99%, 37%)"},".language-css .token.url > .token.function":{color:"hsl(198, 99%, 37%)"},".language-css .token.url > .token.string.url":{color:"hsl(119, 34%, 47%)"},".language-css .token.important":{color:"hsl(301, 63%, 40%)"},".language-css .token.atrule .token.rule":{color:"hsl(301, 63%, 40%)"},".language-javascript .token.operator":{color:"hsl(301, 63%, 40%)"},".language-javascript .token.template-string > .token.interpolation > .token.interpolation-punctuation.punctuation":{color:"hsl(344, 84%, 43%)"},".language-json .token.operator":{color:"hsl(230, 8%, 24%)"},".language-json .token.null.keyword":{color:"hsl(35, 99%, 36%)"},".language-markdown .token.url":{color:"hsl(230, 8%, 24%)"},".language-markdown .token.url > .token.operator":{color:"hsl(230, 8%, 24%)"},".language-markdown .token.url-reference.url > .token.string":{color:"hsl(230, 8%, 24%)"},".language-markdown .token.url > .token.content":{color:"hsl(221, 87%, 60%)"},".language-markdown .token.url > .token.url":{color:"hsl(198, 99%, 37%)"},".language-markdown .token.url-reference.url":{color:"hsl(198, 99%, 37%)"},".language-markdown .token.blockquote.punctuation":{color:"hsl(230, 4%, 64%)",fontStyle:"italic"},".language-markdown .token.hr.punctuation":{color:"hsl(230, 4%, 64%)",fontStyle:"italic"},".language-markdown .token.code-snippet":{color:"hsl(119, 34%, 47%)"},".language-markdown .token.bold .token.content":{color:"hsl(35, 99%, 36%)"},".language-markdown .token.italic .token.content":{color:"hsl(301, 63%, 40%)"},".language-markdown .token.strike .token.content":{color:"hsl(5, 74%, 59%)"},".language-markdown .token.strike .token.punctuation":{color:"hsl(5, 74%, 59%)"},".language-markdown .token.list.punctuation":{color:"hsl(5, 74%, 59%)"},".language-markdown .token.title.important > .token.punctuation":{color:"hsl(5, 74%, 59%)"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},namespace:{Opacity:"0.8"},"token.tab:not(:empty):before":{color:"hsla(230, 8%, 24%, 0.2)"},"token.cr:before":{color:"hsla(230, 8%, 24%, 0.2)"},"token.lf:before":{color:"hsla(230, 8%, 24%, 0.2)"},"token.space:before":{color:"hsla(230, 8%, 24%, 0.2)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item":{marginRight:"0.4em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button":{background:"hsl(230, 1%, 90%)",color:"hsl(230, 6%, 44%)",padding:"0.1em 0.4em",borderRadius:"0.3em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a":{background:"hsl(230, 1%, 90%)",color:"hsl(230, 6%, 44%)",padding:"0.1em 0.4em",borderRadius:"0.3em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span":{background:"hsl(230, 1%, 90%)",color:"hsl(230, 6%, 44%)",padding:"0.1em 0.4em",borderRadius:"0.3em"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:hover":{background:"hsl(230, 1%, 78%)",color:"hsl(230, 8%, 24%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > button:focus":{background:"hsl(230, 1%, 78%)",color:"hsl(230, 8%, 24%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:hover":{background:"hsl(230, 1%, 78%)",color:"hsl(230, 8%, 24%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > a:focus":{background:"hsl(230, 1%, 78%)",color:"hsl(230, 8%, 24%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:hover":{background:"hsl(230, 1%, 78%)",color:"hsl(230, 8%, 24%)"},"div.code-toolbar > .toolbar.toolbar > .toolbar-item > span:focus":{background:"hsl(230, 1%, 78%)",color:"hsl(230, 8%, 24%)"},".line-highlight.line-highlight":{background:"hsla(230, 8%, 24%, 0.05)"},".line-highlight.line-highlight:before":{background:"hsl(230, 1%, 90%)",color:"hsl(230, 8%, 24%)",padding:"0.1em 0.6em",borderRadius:"0.3em",boxShadow:"0 2px 0 0 rgba(0, 0, 0, 0.2)"},".line-highlight.line-highlight[data-end]:after":{background:"hsl(230, 1%, 90%)",color:"hsl(230, 8%, 24%)",padding:"0.1em 0.6em",borderRadius:"0.3em",boxShadow:"0 2px 0 0 rgba(0, 0, 0, 0.2)"},"pre[id].linkable-line-numbers.linkable-line-numbers span.line-numbers-rows > span:hover:before":{backgroundColor:"hsla(230, 8%, 24%, 0.05)"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"hsla(230, 8%, 24%, 0.2)"},".command-line .command-line-prompt":{borderRightColor:"hsla(230, 8%, 24%, 0.2)"},".line-numbers .line-numbers-rows > span:before":{color:"hsl(230, 1%, 62%)"},".command-line .command-line-prompt > span:before":{color:"hsl(230, 1%, 62%)"},".rainbow-braces .token.token.punctuation.brace-level-1":{color:"hsl(5, 74%, 59%)"},".rainbow-braces .token.token.punctuation.brace-level-5":{color:"hsl(5, 74%, 59%)"},".rainbow-braces .token.token.punctuation.brace-level-9":{color:"hsl(5, 74%, 59%)"},".rainbow-braces .token.token.punctuation.brace-level-2":{color:"hsl(119, 34%, 47%)"},".rainbow-braces .token.token.punctuation.brace-level-6":{color:"hsl(119, 34%, 47%)"},".rainbow-braces .token.token.punctuation.brace-level-10":{color:"hsl(119, 34%, 47%)"},".rainbow-braces .token.token.punctuation.brace-level-3":{color:"hsl(221, 87%, 60%)"},".rainbow-braces .token.token.punctuation.brace-level-7":{color:"hsl(221, 87%, 60%)"},".rainbow-braces .token.token.punctuation.brace-level-11":{color:"hsl(221, 87%, 60%)"},".rainbow-braces .token.token.punctuation.brace-level-4":{color:"hsl(301, 63%, 40%)"},".rainbow-braces .token.token.punctuation.brace-level-8":{color:"hsl(301, 63%, 40%)"},".rainbow-braces .token.token.punctuation.brace-level-12":{color:"hsl(301, 63%, 40%)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)":{backgroundColor:"hsla(353, 100%, 66%, 0.15)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)":{backgroundColor:"hsla(353, 100%, 66%, 0.15)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix)::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.deleted:not(.prefix) *::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix)::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre > code.diff-highlight .token.token.deleted:not(.prefix) *::selection":{backgroundColor:"hsla(353, 95%, 66%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)":{backgroundColor:"hsla(137, 100%, 55%, 0.15)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)":{backgroundColor:"hsla(137, 100%, 55%, 0.15)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix) *::-moz-selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix)::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre.diff-highlight > code .token.token.inserted:not(.prefix) *::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix)::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},"pre > code.diff-highlight .token.token.inserted:not(.prefix) *::selection":{backgroundColor:"hsla(135, 73%, 55%, 0.25)"},".prism-previewer.prism-previewer:before":{borderColor:"hsl(0, 0, 95%)"},".prism-previewer-gradient.prism-previewer-gradient div":{borderColor:"hsl(0, 0, 95%)",borderRadius:"0.3em"},".prism-previewer-color.prism-previewer-color:before":{borderRadius:"0.3em"},".prism-previewer-easing.prism-previewer-easing:before":{borderRadius:"0.3em"},".prism-previewer.prism-previewer:after":{borderTopColor:"hsl(0, 0, 95%)"},".prism-previewer-flipped.prism-previewer-flipped.after":{borderBottomColor:"hsl(0, 0, 95%)"},".prism-previewer-angle.prism-previewer-angle:before":{background:"hsl(0, 0%, 100%)"},".prism-previewer-time.prism-previewer-time:before":{background:"hsl(0, 0%, 100%)"},".prism-previewer-easing.prism-previewer-easing":{background:"hsl(0, 0%, 100%)"},".prism-previewer-angle.prism-previewer-angle circle":{stroke:"hsl(230, 8%, 24%)",strokeOpacity:"1"},".prism-previewer-time.prism-previewer-time circle":{stroke:"hsl(230, 8%, 24%)",strokeOpacity:"1"},".prism-previewer-easing.prism-previewer-easing circle":{stroke:"hsl(230, 8%, 24%)",fill:"transparent"},".prism-previewer-easing.prism-previewer-easing path":{stroke:"hsl(230, 8%, 24%)"},".prism-previewer-easing.prism-previewer-easing line":{stroke:"hsl(230, 8%, 24%)"}}})(d_)),d_}var h_={},zV;function Flt(){return zV||(zV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",whiteSpace:"pre-wrap",wordBreak:"break-all",wordWrap:"break-word",fontFamily:'Menlo, Monaco, "Courier New", monospace',fontSize:"15px",lineHeight:"1.5",color:"#dccf8f",textShadow:"0"},'pre[class*="language-"]':{MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",whiteSpace:"pre-wrap",wordBreak:"break-all",wordWrap:"break-word",fontFamily:'Menlo, Monaco, "Courier New", monospace',fontSize:"15px",lineHeight:"1.5",color:"#DCCF8F",textShadow:"0",borderRadius:"5px",border:"1px solid #000",background:"#181914 url('data:image/jpeg;base64,/9j/4AAQSkZJRgABAgAAZABkAAD/7AARRHVja3kAAQAEAAAAMAAA/+4ADkFkb2JlAGTAAAAAAf/bAIQACQYGBgcGCQcHCQ0IBwgNDwsJCQsPEQ4ODw4OERENDg4ODg0RERQUFhQUERoaHBwaGiYmJiYmKysrKysrKysrKwEJCAgJCgkMCgoMDwwODA8TDg4ODhMVDg4PDg4VGhMRERERExoXGhYWFhoXHR0aGh0dJCQjJCQrKysrKysrKysr/8AAEQgAjACMAwEiAAIRAQMRAf/EAF4AAQEBAAAAAAAAAAAAAAAAAAABBwEBAQAAAAAAAAAAAAAAAAAAAAIQAAEDAwIHAQEAAAAAAAAAAADwAREhYaExkUFRcYGxwdHh8REBAAAAAAAAAAAAAAAAAAAAAP/aAAwDAQACEQMRAD8AyGFEjHaBS2fDDs2zkhKmBKktb7km+ZwwCnXPkLVmCTMItj6AXFxRS465/BTnkAJvkLkJe+7AKKoi2AtRS2zuAWsCb5GOlBN8gKfmuGHZ8MFqIth3ALmFoFwbwKWyAlTAp17uKqBvgBD8sM4fTjhvAhkzhaRkBMKBrfs7jGPIpzy7gFrAqnC0C0gB0EWwBDW2cBVQwm+QtPpa3wBO3sVvszCnLAhkzgL5/RLf13cLQd8/AGlu0Cb5HTx9KuAEieGJEdcehS3eRTp2ATdt3CpIm+QtZwAhROXFeb7swp/ahaM3kBE/jSIUBc/AWrgBN8uNFAl+b7sAXFxFn2YLUU5Ns7gFX8C4ib+hN8gFWXwK3bZglxEJm+gKdciLPsFV/TClsgJUwKJ5FVA7tvIFrfZhVfGJDcsCKaYgAqv6YRbE+RWOWBtu7+AL3yRalXLyKqAIIfk+zARbDgFyEsncYwJvlgFRW+GEWntIi2P0BooyFxcNr8Ep3+ANLbMO+QyhvbiqdgC0kVvgUUiLYgBS2QtPbiVI1/sgOmG9uO+Y8DW+7jS2zAOnj6O2BndwuIAUtkdRN8gFoK3wwXMQyZwHVbClsuNLd4E3yAUR6FVDBR+BafQGt93LVMxJTv8ABts4CVLhcfYWsCb5kC9/BHdU8CLYFY5bMAd+eX9MGthhpbA1vu4B7+RKkaW2Yq4AQtVBBFsAJU/AuIXBhN8gGWnstefhiZyWvLAEnbYS1uzSFP6Jvn4Baxx70JKkQojLib5AVTey1jjgkKJGO0AKWyOm7N7cSpgSpAdPH0Tfd/gp1z5C1ZgKqN9J2wFxcUUuAFLZAm+QC0Fb4YUVRFsAOvj4KW2dwtYE3yAWk/wS/PLMKfmuGHZ8MAXF/Ja32Yi5haAKWz4Ydm2cSpgU693Atb7km+Zwwh+WGcPpxw3gAkzCLY+iYUDW/Z3Adc/gpzyFrAqnALkJe+7DoItgAtRS2zuKqGE3yAx0oJvkdvYrfZmALURbDuL5/RLf13cAuDeBS2RpbtAm+QFVA3wR+3fUtFHoBDJnC0jIXH0HWsgMY8inPLuOkd9chp4z20ALQLSA8cI9jYAIa2zjzjBd8gRafS1vgiUho/kAKcsCGTOGWvoOpkAtB3z8Hm8x2Ff5ADp4+lXAlIvcmwH/2Q==') repeat left top",padding:"12px",overflow:"auto"},'pre > code[class*="language-"]':{fontSize:"1em"},':not(pre) > code[class*="language-"]':{borderRadius:"5px",border:"1px solid #000",color:"#DCCF8F",background:"#181914 url('data:image/jpeg;base64,/9j/4AAQSkZJRgABAgAAZABkAAD/7AARRHVja3kAAQAEAAAAMAAA/+4ADkFkb2JlAGTAAAAAAf/bAIQACQYGBgcGCQcHCQ0IBwgNDwsJCQsPEQ4ODw4OERENDg4ODg0RERQUFhQUERoaHBwaGiYmJiYmKysrKysrKysrKwEJCAgJCgkMCgoMDwwODA8TDg4ODhMVDg4PDg4VGhMRERERExoXGhYWFhoXHR0aGh0dJCQjJCQrKysrKysrKysr/8AAEQgAjACMAwEiAAIRAQMRAf/EAF4AAQEBAAAAAAAAAAAAAAAAAAABBwEBAQAAAAAAAAAAAAAAAAAAAAIQAAEDAwIHAQEAAAAAAAAAAADwAREhYaExkUFRcYGxwdHh8REBAAAAAAAAAAAAAAAAAAAAAP/aAAwDAQACEQMRAD8AyGFEjHaBS2fDDs2zkhKmBKktb7km+ZwwCnXPkLVmCTMItj6AXFxRS465/BTnkAJvkLkJe+7AKKoi2AtRS2zuAWsCb5GOlBN8gKfmuGHZ8MFqIth3ALmFoFwbwKWyAlTAp17uKqBvgBD8sM4fTjhvAhkzhaRkBMKBrfs7jGPIpzy7gFrAqnC0C0gB0EWwBDW2cBVQwm+QtPpa3wBO3sVvszCnLAhkzgL5/RLf13cLQd8/AGlu0Cb5HTx9KuAEieGJEdcehS3eRTp2ATdt3CpIm+QtZwAhROXFeb7swp/ahaM3kBE/jSIUBc/AWrgBN8uNFAl+b7sAXFxFn2YLUU5Ns7gFX8C4ib+hN8gFWXwK3bZglxEJm+gKdciLPsFV/TClsgJUwKJ5FVA7tvIFrfZhVfGJDcsCKaYgAqv6YRbE+RWOWBtu7+AL3yRalXLyKqAIIfk+zARbDgFyEsncYwJvlgFRW+GEWntIi2P0BooyFxcNr8Ep3+ANLbMO+QyhvbiqdgC0kVvgUUiLYgBS2QtPbiVI1/sgOmG9uO+Y8DW+7jS2zAOnj6O2BndwuIAUtkdRN8gFoK3wwXMQyZwHVbClsuNLd4E3yAUR6FVDBR+BafQGt93LVMxJTv8ABts4CVLhcfYWsCb5kC9/BHdU8CLYFY5bMAd+eX9MGthhpbA1vu4B7+RKkaW2Yq4AQtVBBFsAJU/AuIXBhN8gGWnstefhiZyWvLAEnbYS1uzSFP6Jvn4Baxx70JKkQojLib5AVTey1jjgkKJGO0AKWyOm7N7cSpgSpAdPH0Tfd/gp1z5C1ZgKqN9J2wFxcUUuAFLZAm+QC0Fb4YUVRFsAOvj4KW2dwtYE3yAWk/wS/PLMKfmuGHZ8MAXF/Ja32Yi5haAKWz4Ydm2cSpgU693Atb7km+Zwwh+WGcPpxw3gAkzCLY+iYUDW/Z3Adc/gpzyFrAqnALkJe+7DoItgAtRS2zuKqGE3yAx0oJvkdvYrfZmALURbDuL5/RLf13cAuDeBS2RpbtAm+QFVA3wR+3fUtFHoBDJnC0jIXH0HWsgMY8inPLuOkd9chp4z20ALQLSA8cI9jYAIa2zjzjBd8gRafS1vgiUho/kAKcsCGTOGWvoOpkAtB3z8Hm8x2Ff5ADp4+lXAlIvcmwH/2Q==') repeat left top",padding:"2px 6px"},namespace:{Opacity:".7"},comment:{color:"#586e75",fontStyle:"italic"},prolog:{color:"#586e75",fontStyle:"italic"},doctype:{color:"#586e75",fontStyle:"italic"},cdata:{color:"#586e75",fontStyle:"italic"},number:{color:"#b89859"},string:{color:"#468966"},char:{color:"#468966"},builtin:{color:"#468966"},inserted:{color:"#468966"},"attr-name":{color:"#b89859"},operator:{color:"#dccf8f"},entity:{color:"#dccf8f",cursor:"help"},url:{color:"#dccf8f"},".language-css .token.string":{color:"#dccf8f"},".style .token.string":{color:"#dccf8f"},selector:{color:"#859900"},regex:{color:"#859900"},atrule:{color:"#cb4b16"},keyword:{color:"#cb4b16"},"attr-value":{color:"#468966"},function:{color:"#b58900"},variable:{color:"#b58900"},placeholder:{color:"#b58900"},property:{color:"#b89859"},tag:{color:"#ffb03b"},boolean:{color:"#b89859"},constant:{color:"#b89859"},symbol:{color:"#b89859"},important:{color:"#dc322f"},statement:{color:"#dc322f"},deleted:{color:"#dc322f"},punctuation:{color:"#dccf8f"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(h_)),h_}var p_={},HV;function _lt(){return HV||(HV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={"code[class*='language-']":{color:"#9efeff",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",fontFamily:"'Operator Mono', 'Fira Code', Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontWeight:"400",fontSize:"17px",lineHeight:"25px",letterSpacing:"0.5px",textShadow:"0 1px #222245"},"pre[class*='language-']":{color:"#9efeff",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",fontFamily:"'Operator Mono', 'Fira Code', Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontWeight:"400",fontSize:"17px",lineHeight:"25px",letterSpacing:"0.5px",textShadow:"0 1px #222245",padding:"2em",margin:"0.5em 0",overflow:"auto",background:"#1e1e3f"},"pre[class*='language-']::-moz-selection":{color:"inherit",background:"#a599e9"},"pre[class*='language-'] ::-moz-selection":{color:"inherit",background:"#a599e9"},"code[class*='language-']::-moz-selection":{color:"inherit",background:"#a599e9"},"code[class*='language-'] ::-moz-selection":{color:"inherit",background:"#a599e9"},"pre[class*='language-']::selection":{color:"inherit",background:"#a599e9"},"pre[class*='language-'] ::selection":{color:"inherit",background:"#a599e9"},"code[class*='language-']::selection":{color:"inherit",background:"#a599e9"},"code[class*='language-'] ::selection":{color:"inherit",background:"#a599e9"},":not(pre) > code[class*='language-']":{background:"#1e1e3f",padding:"0.1em",borderRadius:"0.3em"},"":{fontWeight:"400"},comment:{color:"#b362ff"},prolog:{color:"#b362ff"},cdata:{color:"#b362ff"},delimiter:{color:"#ff9d00"},keyword:{color:"#ff9d00"},selector:{color:"#ff9d00"},important:{color:"#ff9d00"},atrule:{color:"#ff9d00"},operator:{color:"rgb(255, 180, 84)",background:"none"},"attr-name":{color:"rgb(255, 180, 84)"},punctuation:{color:"#ffffff"},boolean:{color:"rgb(255, 98, 140)"},tag:{color:"rgb(255, 157, 0)"},"tag.punctuation":{color:"rgb(255, 157, 0)"},doctype:{color:"rgb(255, 157, 0)"},builtin:{color:"rgb(255, 157, 0)"},entity:{color:"#6897bb",background:"none"},symbol:{color:"#6897bb"},number:{color:"#ff628c"},property:{color:"#ff628c"},constant:{color:"#ff628c"},variable:{color:"#ff628c"},string:{color:"#a5ff90"},char:{color:"#a5ff90"},"attr-value":{color:"#a5c261"},"attr-value.punctuation":{color:"#a5c261"},"attr-value.punctuation:first-child":{color:"#a9b7c6"},url:{color:"#287bde",textDecoration:"underline",background:"none"},function:{color:"rgb(250, 208, 0)"},regex:{background:"#364135"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},inserted:{background:"#00ff00"},deleted:{background:"#ff000d"},"code.language-css .token.property":{color:"#a9b7c6"},"code.language-css .token.property + .token.punctuation":{color:"#a9b7c6"},"code.language-css .token.id":{color:"#ffc66d"},"code.language-css .token.selector > .token.class":{color:"#ffc66d"},"code.language-css .token.selector > .token.attribute":{color:"#ffc66d"},"code.language-css .token.selector > .token.pseudo-class":{color:"#ffc66d"},"code.language-css .token.selector > .token.pseudo-element":{color:"#ffc66d"},"class-name":{color:"#fb94ff"},".language-css .token.string":{background:"none"},".style .token.string":{background:"none"},".line-highlight.line-highlight":{marginTop:"36px",background:"linear-gradient(to right, rgba(179, 98, 255, 0.17), transparent)"},".line-highlight.line-highlight:before":{content:"''"},".line-highlight.line-highlight[data-end]:after":{content:"''"}}})(p_)),p_}var f_={},UV;function Rlt(){return UV||(UV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#839496",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Inconsolata, Monaco, Consolas, 'Courier New', Courier, monospace",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#839496",textShadow:"0 1px rgba(0, 0, 0, 0.3)",fontFamily:"Inconsolata, Monaco, Consolas, 'Courier New', Courier, monospace",direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",borderRadius:"0.3em",background:"#002b36"},':not(pre) > code[class*="language-"]':{background:"#002b36",padding:".1em",borderRadius:".3em"},comment:{color:"#586e75"},prolog:{color:"#586e75"},doctype:{color:"#586e75"},cdata:{color:"#586e75"},punctuation:{color:"#93a1a1"},".namespace":{Opacity:".7"},property:{color:"#268bd2"},keyword:{color:"#268bd2"},tag:{color:"#268bd2"},"class-name":{color:"#FFFFB6",textDecoration:"underline"},boolean:{color:"#b58900"},constant:{color:"#b58900"},symbol:{color:"#dc322f"},deleted:{color:"#dc322f"},number:{color:"#859900"},selector:{color:"#859900"},"attr-name":{color:"#859900"},string:{color:"#859900"},char:{color:"#859900"},builtin:{color:"#859900"},inserted:{color:"#859900"},variable:{color:"#268bd2"},operator:{color:"#EDEDED"},function:{color:"#268bd2"},regex:{color:"#E9C062"},important:{color:"#fd971f",fontWeight:"bold"},entity:{color:"#FFFFB6",cursor:"help"},url:{color:"#96CBFE"},".language-css .token.string":{color:"#87C38A"},".style .token.string":{color:"#87C38A"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},atrule:{color:"#F9EE98"},"attr-value":{color:"#F9EE98"}}})(f_)),f_}var m_={},$V;function Dlt(){return $V||($V=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#f92aad",textShadow:"0 0 2px #100c0f, 0 0 5px #dc078e33, 0 0 10px #fff3",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#f92aad",textShadow:"0 0 2px #100c0f, 0 0 5px #dc078e33, 0 0 10px #fff3",background:"none",fontFamily:"Consolas, Monaco, 'Andale Mono', 'Ubuntu Mono', monospace",fontSize:"1em",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",backgroundColor:"transparent !important",backgroundImage:"linear-gradient(to bottom, #2a2139 75%, #34294f)"},':not(pre) > code[class*="language-"]':{backgroundColor:"transparent !important",backgroundImage:"linear-gradient(to bottom, #2a2139 75%, #34294f)",padding:".1em",borderRadius:".3em",whiteSpace:"normal"},comment:{color:"#8e8e8e"},"block-comment":{color:"#8e8e8e"},prolog:{color:"#8e8e8e"},doctype:{color:"#8e8e8e"},cdata:{color:"#8e8e8e"},punctuation:{color:"#ccc"},tag:{color:"#e2777a"},"attr-name":{color:"#e2777a"},namespace:{color:"#e2777a"},number:{color:"#e2777a"},unit:{color:"#e2777a"},hexcode:{color:"#e2777a"},deleted:{color:"#e2777a"},property:{color:"#72f1b8",textShadow:"0 0 2px #100c0f, 0 0 10px #257c5575, 0 0 35px #21272475"},selector:{color:"#72f1b8",textShadow:"0 0 2px #100c0f, 0 0 10px #257c5575, 0 0 35px #21272475"},"function-name":{color:"#6196cc"},boolean:{color:"#fdfdfd",textShadow:"0 0 2px #001716, 0 0 3px #03edf975, 0 0 5px #03edf975, 0 0 8px #03edf975"},"selector.id":{color:"#fdfdfd",textShadow:"0 0 2px #001716, 0 0 3px #03edf975, 0 0 5px #03edf975, 0 0 8px #03edf975"},function:{color:"#fdfdfd",textShadow:"0 0 2px #001716, 0 0 3px #03edf975, 0 0 5px #03edf975, 0 0 8px #03edf975"},"class-name":{color:"#fff5f6",textShadow:"0 0 2px #000, 0 0 10px #fc1f2c75, 0 0 5px #fc1f2c75, 0 0 25px #fc1f2c75"},constant:{color:"#f92aad",textShadow:"0 0 2px #100c0f, 0 0 5px #dc078e33, 0 0 10px #fff3"},symbol:{color:"#f92aad",textShadow:"0 0 2px #100c0f, 0 0 5px #dc078e33, 0 0 10px #fff3"},important:{color:"#f4eee4",textShadow:"0 0 2px #393a33, 0 0 8px #f39f0575, 0 0 2px #f39f0575",fontWeight:"bold"},atrule:{color:"#f4eee4",textShadow:"0 0 2px #393a33, 0 0 8px #f39f0575, 0 0 2px #f39f0575"},keyword:{color:"#f4eee4",textShadow:"0 0 2px #393a33, 0 0 8px #f39f0575, 0 0 2px #f39f0575"},"selector.class":{color:"#f4eee4",textShadow:"0 0 2px #393a33, 0 0 8px #f39f0575, 0 0 2px #f39f0575"},builtin:{color:"#f4eee4",textShadow:"0 0 2px #393a33, 0 0 8px #f39f0575, 0 0 2px #f39f0575"},string:{color:"#f87c32"},char:{color:"#f87c32"},"attr-value":{color:"#f87c32"},regex:{color:"#f87c32"},variable:{color:"#f87c32"},operator:{color:"#67cdcc"},entity:{color:"#67cdcc",cursor:"help"},url:{color:"#67cdcc"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},inserted:{color:"green"}}})(m_)),m_}var g_={},GV;function Nlt(){return GV||(GV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#393A34",fontFamily:'"Consolas", "Bitstream Vera Sans Mono", "Courier New", Courier, monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",fontSize:".9em",lineHeight:"1.2em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]':{color:"#393A34",fontFamily:'"Consolas", "Bitstream Vera Sans Mono", "Courier New", Courier, monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",fontSize:".9em",lineHeight:"1.2em",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",border:"1px solid #dddddd",backgroundColor:"white"},'pre > code[class*="language-"]':{fontSize:"1em"},'pre[class*="language-"]::-moz-selection':{background:"#C1DEF1"},'pre[class*="language-"] ::-moz-selection':{background:"#C1DEF1"},'code[class*="language-"]::-moz-selection':{background:"#C1DEF1"},'code[class*="language-"] ::-moz-selection':{background:"#C1DEF1"},'pre[class*="language-"]::selection':{background:"#C1DEF1"},'pre[class*="language-"] ::selection':{background:"#C1DEF1"},'code[class*="language-"]::selection':{background:"#C1DEF1"},'code[class*="language-"] ::selection':{background:"#C1DEF1"},':not(pre) > code[class*="language-"]':{padding:".2em",paddingTop:"1px",paddingBottom:"1px",background:"#f8f8f8",border:"1px solid #dddddd"},comment:{color:"#008000",fontStyle:"italic"},prolog:{color:"#008000",fontStyle:"italic"},doctype:{color:"#008000",fontStyle:"italic"},cdata:{color:"#008000",fontStyle:"italic"},namespace:{Opacity:".7"},string:{color:"#A31515"},punctuation:{color:"#393A34"},operator:{color:"#393A34"},url:{color:"#36acaa"},symbol:{color:"#36acaa"},number:{color:"#36acaa"},boolean:{color:"#36acaa"},variable:{color:"#36acaa"},constant:{color:"#36acaa"},inserted:{color:"#36acaa"},atrule:{color:"#0000ff"},keyword:{color:"#0000ff"},"attr-value":{color:"#0000ff"},".language-autohotkey .token.selector":{color:"#0000ff"},".language-json .token.boolean":{color:"#0000ff"},".language-json .token.number":{color:"#0000ff"},'code[class*="language-css"]':{color:"#0000ff"},function:{color:"#393A34"},deleted:{color:"#9a050f"},".language-autohotkey .token.tag":{color:"#9a050f"},selector:{color:"#800000"},".language-autohotkey .token.keyword":{color:"#00009f"},important:{color:"#e90",fontWeight:"bold"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"},"class-name":{color:"#2B91AF"},".language-json .token.property":{color:"#2B91AF"},tag:{color:"#800000"},"attr-name":{color:"#ff0000"},property:{color:"#ff0000"},regex:{color:"#ff0000"},entity:{color:"#ff0000"},"directive.tag.tag":{background:"#ffff00",color:"#393A34"},".line-numbers.line-numbers .line-numbers-rows":{borderRightColor:"#a5a5a5"},".line-numbers .line-numbers-rows > span:before":{color:"#2B91AF"},".line-highlight.line-highlight":{background:"linear-gradient(to right, rgba(193, 222, 241, 0.2) 70%, rgba(221, 222, 241, 0))"}}})(g_)),g_}var b_={},jV;function Llt(){return jV||(jV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'pre[class*="language-"]':{color:"#d4d4d4",fontSize:"13px",textShadow:"none",fontFamily:'Menlo, Monaco, Consolas, "Andale Mono", "Ubuntu Mono", "Courier New", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",padding:"1em",margin:".5em 0",overflow:"auto",background:"#1e1e1e"},'code[class*="language-"]':{color:"#d4d4d4",fontSize:"13px",textShadow:"none",fontFamily:'Menlo, Monaco, Consolas, "Andale Mono", "Ubuntu Mono", "Courier New", monospace',direction:"ltr",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",lineHeight:"1.5",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none"},'pre[class*="language-"]::selection':{textShadow:"none",background:"#264F78"},'code[class*="language-"]::selection':{textShadow:"none",background:"#264F78"},'pre[class*="language-"] *::selection':{textShadow:"none",background:"#264F78"},'code[class*="language-"] *::selection':{textShadow:"none",background:"#264F78"},':not(pre) > code[class*="language-"]':{padding:".1em .3em",borderRadius:".3em",color:"#db4c69",background:"#1e1e1e"},".namespace":{Opacity:".7"},"doctype.doctype-tag":{color:"#569CD6"},"doctype.name":{color:"#9cdcfe"},comment:{color:"#6a9955"},prolog:{color:"#6a9955"},punctuation:{color:"#d4d4d4"},".language-html .language-css .token.punctuation":{color:"#d4d4d4"},".language-html .language-javascript .token.punctuation":{color:"#d4d4d4"},property:{color:"#9cdcfe"},tag:{color:"#569cd6"},boolean:{color:"#569cd6"},number:{color:"#b5cea8"},constant:{color:"#9cdcfe"},symbol:{color:"#b5cea8"},inserted:{color:"#b5cea8"},unit:{color:"#b5cea8"},selector:{color:"#d7ba7d"},"attr-name":{color:"#9cdcfe"},string:{color:"#ce9178"},char:{color:"#ce9178"},builtin:{color:"#ce9178"},deleted:{color:"#ce9178"},".language-css .token.string.url":{textDecoration:"underline"},operator:{color:"#d4d4d4"},entity:{color:"#569cd6"},"operator.arrow":{color:"#569CD6"},atrule:{color:"#ce9178"},"atrule.rule":{color:"#c586c0"},"atrule.url":{color:"#9cdcfe"},"atrule.url.function":{color:"#dcdcaa"},"atrule.url.punctuation":{color:"#d4d4d4"},keyword:{color:"#569CD6"},"keyword.module":{color:"#c586c0"},"keyword.control-flow":{color:"#c586c0"},function:{color:"#dcdcaa"},"function.maybe-class-name":{color:"#dcdcaa"},regex:{color:"#d16969"},important:{color:"#569cd6"},italic:{fontStyle:"italic"},"class-name":{color:"#4ec9b0"},"maybe-class-name":{color:"#4ec9b0"},console:{color:"#9cdcfe"},parameter:{color:"#9cdcfe"},interpolation:{color:"#9cdcfe"},"punctuation.interpolation-punctuation":{color:"#569cd6"},variable:{color:"#9cdcfe"},"imports.maybe-class-name":{color:"#9cdcfe"},"exports.maybe-class-name":{color:"#9cdcfe"},escape:{color:"#d7ba7d"},"tag.punctuation":{color:"#808080"},cdata:{color:"#808080"},"attr-value":{color:"#ce9178"},"attr-value.punctuation":{color:"#ce9178"},"attr-value.punctuation.attr-equals":{color:"#d4d4d4"},namespace:{color:"#4ec9b0"},'pre[class*="language-javascript"]':{color:"#9cdcfe"},'code[class*="language-javascript"]':{color:"#9cdcfe"},'pre[class*="language-jsx"]':{color:"#9cdcfe"},'code[class*="language-jsx"]':{color:"#9cdcfe"},'pre[class*="language-typescript"]':{color:"#9cdcfe"},'code[class*="language-typescript"]':{color:"#9cdcfe"},'pre[class*="language-tsx"]':{color:"#9cdcfe"},'code[class*="language-tsx"]':{color:"#9cdcfe"},'pre[class*="language-css"]':{color:"#ce9178"},'code[class*="language-css"]':{color:"#ce9178"},'pre[class*="language-html"]':{color:"#d4d4d4"},'code[class*="language-html"]':{color:"#d4d4d4"},".language-regex .token.anchor":{color:"#dcdcaa"},".language-html .token.punctuation":{color:"#808080"},'pre[class*="language-"] > code[class*="language-"]':{position:"relative",zIndex:"1"},".line-highlight.line-highlight":{background:"#f7ebc6",boxShadow:"inset 5px 0 0 #f7d87c",zIndex:"0"}}})(b_)),b_}var y_={},qV;function Mlt(){return qV||(qV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{MozTabSize:"2",OTabSize:"2",tabSize:"2",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",whiteSpace:"pre-wrap",wordWrap:"normal",fontFamily:'Menlo, Monaco, "Courier New", monospace',fontSize:"14px",color:"#76d9e6",textShadow:"none"},'pre[class*="language-"]':{MozTabSize:"2",OTabSize:"2",tabSize:"2",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",whiteSpace:"pre-wrap",wordWrap:"normal",fontFamily:'Menlo, Monaco, "Courier New", monospace',fontSize:"14px",color:"#76d9e6",textShadow:"none",background:"#2a2a2a",padding:"15px",borderRadius:"4px",border:"1px solid #e1e1e8",overflow:"auto",position:"relative"},'pre > code[class*="language-"]':{fontSize:"1em"},':not(pre) > code[class*="language-"]':{background:"#2a2a2a",padding:"0.15em 0.2em 0.05em",borderRadius:".3em",border:"0.13em solid #7a6652",boxShadow:"1px 1px 0.3em -0.1em #000 inset"},'pre[class*="language-"] code':{whiteSpace:"pre",display:"block"},namespace:{Opacity:".7"},comment:{color:"#6f705e"},prolog:{color:"#6f705e"},doctype:{color:"#6f705e"},cdata:{color:"#6f705e"},operator:{color:"#a77afe"},boolean:{color:"#a77afe"},number:{color:"#a77afe"},"attr-name":{color:"#e6d06c"},string:{color:"#e6d06c"},entity:{color:"#e6d06c",cursor:"help"},url:{color:"#e6d06c"},".language-css .token.string":{color:"#e6d06c"},".style .token.string":{color:"#e6d06c"},selector:{color:"#a6e22d"},inserted:{color:"#a6e22d"},atrule:{color:"#ef3b7d"},"attr-value":{color:"#ef3b7d"},keyword:{color:"#ef3b7d"},important:{color:"#ef3b7d",fontWeight:"bold"},deleted:{color:"#ef3b7d"},regex:{color:"#76d9e6"},statement:{color:"#76d9e6",fontWeight:"bold"},placeholder:{color:"#fff"},variable:{color:"#fff"},bold:{fontWeight:"bold"},punctuation:{color:"#bebec5"},italic:{fontStyle:"italic"},"code.language-markup":{color:"#f9f9f9"},"code.language-markup .token.tag":{color:"#ef3b7d"},"code.language-markup .token.attr-name":{color:"#a6e22d"},"code.language-markup .token.attr-value":{color:"#e6d06c"},"code.language-markup .token.style":{color:"#76d9e6"},"code.language-markup .token.script":{color:"#76d9e6"},"code.language-markup .token.script .token.keyword":{color:"#76d9e6"},".line-highlight.line-highlight":{padding:"0",background:"rgba(255, 255, 255, 0.08)"},".line-highlight.line-highlight:before":{padding:"0.2em 0.5em",backgroundColor:"rgba(255, 255, 255, 0.4)",color:"black",height:"1em",lineHeight:"1em",boxShadow:"0 1px 1px rgba(255, 255, 255, 0.7)"},".line-highlight.line-highlight[data-end]:after":{padding:"0.2em 0.5em",backgroundColor:"rgba(255, 255, 255, 0.4)",color:"black",height:"1em",lineHeight:"1em",boxShadow:"0 1px 1px rgba(255, 255, 255, 0.7)"}}})(y_)),y_}var v_={},WV;function Ilt(){return WV||(WV=1,(function(e){Object.defineProperty(e,"__esModule",{value:!0}),e.default=void 0,e.default={'code[class*="language-"]':{color:"#22da17",fontFamily:"monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",lineHeight:"25px",fontSize:"18px",margin:"5px 0"},'pre[class*="language-"]':{color:"white",fontFamily:"monospace",textAlign:"left",whiteSpace:"pre",wordSpacing:"normal",wordBreak:"normal",wordWrap:"normal",MozTabSize:"4",OTabSize:"4",tabSize:"4",WebkitHyphens:"none",MozHyphens:"none",msHyphens:"none",hyphens:"none",lineHeight:"25px",fontSize:"18px",margin:"0.5em 0",background:"#0a143c",padding:"1em",overflow:"auto"},'pre[class*="language-"] *':{fontFamily:"monospace"},':not(pre) > code[class*="language-"]':{color:"white",background:"#0a143c",padding:"0.1em",borderRadius:"0.3em",whiteSpace:"normal"},'pre[class*="language-"]::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'pre[class*="language-"] ::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"]::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"] ::-moz-selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'pre[class*="language-"]::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'pre[class*="language-"] ::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"]::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},'code[class*="language-"] ::selection':{textShadow:"none",background:"rgba(29, 59, 83, 0.99)"},comment:{color:"rgb(99, 119, 119)",fontStyle:"italic"},prolog:{color:"rgb(99, 119, 119)",fontStyle:"italic"},cdata:{color:"rgb(99, 119, 119)",fontStyle:"italic"},punctuation:{color:"rgb(199, 146, 234)"},".namespace":{color:"rgb(178, 204, 214)"},deleted:{color:"rgba(239, 83, 80, 0.56)",fontStyle:"italic"},symbol:{color:"rgb(128, 203, 196)"},property:{color:"rgb(128, 203, 196)"},tag:{color:"rgb(127, 219, 202)"},operator:{color:"rgb(127, 219, 202)"},keyword:{color:"rgb(127, 219, 202)"},boolean:{color:"rgb(255, 88, 116)"},number:{color:"rgb(247, 140, 108)"},constant:{color:"rgb(34 183 199)"},function:{color:"rgb(34 183 199)"},builtin:{color:"rgb(34 183 199)"},char:{color:"rgb(34 183 199)"},selector:{color:"rgb(199, 146, 234)",fontStyle:"italic"},doctype:{color:"rgb(199, 146, 234)",fontStyle:"italic"},"attr-name":{color:"rgb(173, 219, 103)",fontStyle:"italic"},inserted:{color:"rgb(173, 219, 103)",fontStyle:"italic"},string:{color:"rgb(173, 219, 103)"},url:{color:"rgb(173, 219, 103)"},entity:{color:"rgb(173, 219, 103)"},".language-css .token.string":{color:"rgb(173, 219, 103)"},".style .token.string":{color:"rgb(173, 219, 103)"},"class-name":{color:"rgb(255, 203, 139)"},atrule:{color:"rgb(255, 203, 139)"},"attr-value":{color:"rgb(255, 203, 139)"},regex:{color:"rgb(214, 222, 235)"},important:{color:"rgb(214, 222, 235)",fontWeight:"bold"},variable:{color:"rgb(214, 222, 235)"},bold:{fontWeight:"bold"},italic:{fontStyle:"italic"}}})(v_)),v_}var VV;function Olt(){return VV||(VV=1,(function(e){var t=qst();Object.defineProperty(e,"__esModule",{value:!0}),Object.defineProperty(e,"a11yDark",{enumerable:!0,get:function(){return d.default}}),Object.defineProperty(e,"atomDark",{enumerable:!0,get:function(){return p.default}}),Object.defineProperty(e,"base16AteliersulphurpoolLight",{enumerable:!0,get:function(){return f.default}}),Object.defineProperty(e,"cb",{enumerable:!0,get:function(){return m.default}}),Object.defineProperty(e,"coldarkCold",{enumerable:!0,get:function(){return b.default}}),Object.defineProperty(e,"coldarkDark",{enumerable:!0,get:function(){return y.default}}),Object.defineProperty(e,"coy",{enumerable:!0,get:function(){return r.default}}),Object.defineProperty(e,"coyWithoutShadows",{enumerable:!0,get:function(){return S.default}}),Object.defineProperty(e,"darcula",{enumerable:!0,get:function(){return w.default}}),Object.defineProperty(e,"dark",{enumerable:!0,get:function(){return n.default}}),Object.defineProperty(e,"dracula",{enumerable:!0,get:function(){return E.default}}),Object.defineProperty(e,"duotoneDark",{enumerable:!0,get:function(){return k.default}}),Object.defineProperty(e,"duotoneEarth",{enumerable:!0,get:function(){return _.default}}),Object.defineProperty(e,"duotoneForest",{enumerable:!0,get:function(){return R.default}}),Object.defineProperty(e,"duotoneLight",{enumerable:!0,get:function(){return C.default}}),Object.defineProperty(e,"duotoneSea",{enumerable:!0,get:function(){return F.default}}),Object.defineProperty(e,"duotoneSpace",{enumerable:!0,get:function(){return L.default}}),Object.defineProperty(e,"funky",{enumerable:!0,get:function(){return a.default}}),Object.defineProperty(e,"ghcolors",{enumerable:!0,get:function(){return I.default}}),Object.defineProperty(e,"gruvboxDark",{enumerable:!0,get:function(){return M.default}}),Object.defineProperty(e,"gruvboxLight",{enumerable:!0,get:function(){return j.default}}),Object.defineProperty(e,"holiTheme",{enumerable:!0,get:function(){return Y.default}}),Object.defineProperty(e,"hopscotch",{enumerable:!0,get:function(){return W.default}}),Object.defineProperty(e,"lucario",{enumerable:!0,get:function(){return $.default}}),Object.defineProperty(e,"materialDark",{enumerable:!0,get:function(){return X.default}}),Object.defineProperty(e,"materialLight",{enumerable:!0,get:function(){return z.default}}),Object.defineProperty(e,"materialOceanic",{enumerable:!0,get:function(){return H.default}}),Object.defineProperty(e,"nightOwl",{enumerable:!0,get:function(){return B.default}}),Object.defineProperty(e,"nord",{enumerable:!0,get:function(){return ee.default}}),Object.defineProperty(e,"okaidia",{enumerable:!0,get:function(){return i.default}}),Object.defineProperty(e,"oneDark",{enumerable:!0,get:function(){return J.default}}),Object.defineProperty(e,"oneLight",{enumerable:!0,get:function(){return K.default}}),Object.defineProperty(e,"pojoaque",{enumerable:!0,get:function(){return G.default}}),Object.defineProperty(e,"prism",{enumerable:!0,get:function(){return u.default}}),Object.defineProperty(e,"shadesOfPurple",{enumerable:!0,get:function(){return P.default}}),Object.defineProperty(e,"solarizedDarkAtom",{enumerable:!0,get:function(){return ne.default}}),Object.defineProperty(e,"solarizedlight",{enumerable:!0,get:function(){return o.default}}),Object.defineProperty(e,"synthwave84",{enumerable:!0,get:function(){return oe.default}}),Object.defineProperty(e,"tomorrow",{enumerable:!0,get:function(){return s.default}}),Object.defineProperty(e,"twilight",{enumerable:!0,get:function(){return l.default}}),Object.defineProperty(e,"vs",{enumerable:!0,get:function(){return pe.default}}),Object.defineProperty(e,"vscDarkPlus",{enumerable:!0,get:function(){return ae.default}}),Object.defineProperty(e,"xonokai",{enumerable:!0,get:function(){return se.default}}),Object.defineProperty(e,"zTouch",{enumerable:!0,get:function(){return Ee.default}});var r=t(Wst()),n=t(Vst()),a=t(Kst()),i=t(Yst()),o=t(Jst()),s=t(Xst()),l=t(Zst()),u=t(Qst()),d=t(elt()),p=t(tlt()),f=t(rlt()),m=t(nlt()),b=t(alt()),y=t(ilt()),S=t(olt()),w=t(slt()),E=t(llt()),k=t(ult()),_=t(clt()),R=t(dlt()),C=t(hlt()),F=t(plt()),L=t(flt()),I=t(mlt()),M=t(glt()),j=t(blt()),Y=t(ylt()),W=t(vlt()),$=t(Slt()),X=t(Elt()),z=t(xlt()),H=t(wlt()),B=t(klt()),ee=t(Clt()),J=t(Tlt()),K=t(Alt()),G=t(Flt()),P=t(_lt()),ne=t(Rlt()),oe=t(Dlt()),pe=t(Nlt()),ae=t(Llt()),se=t(Mlt()),Ee=t(Ilt())})(F4)),F4}var f1=Olt();const Blt=({message:e,isTabActive:t=!0})=>{const{t:r}=Sr(),{theme:n}=A5(),[a,i]=T.useState(null),[o,s]=T.useState(!1),{thinkingContent:l,displayContent:u,thinkingTime:d,isThinking:p}=e;T.useEffect(()=>{p&&s(!1)},[p,e.id]);const f=l,m=e.role==="user"?e.content:u!==void 0?u:e.content||"";T.useEffect(()=>{(async()=>{try{const{default:w}=await Pr(async()=>{const{default:E}=await import("./index-Dg_Ll5cH.js");return{default:E}},[]);i(()=>w)}catch(w){console.error("Failed to load KaTeX plugin:",w),i(null)}})()},[]);const b=T.useMemo(()=>({code:S=>{const{inline:w,className:E,children:k,..._}=S,R=/language-(\w+)/.exec(E||""),C=R?R[1]:void 0;return C==="math"&&!w?x.jsx("div",{className:"katex-display-wrapper my-4 overflow-x-auto",children:x.jsx("div",{className:"text-current",children:k})}):C==="math"&&w?x.jsx("span",{className:"katex-inline-wrapper",children:x.jsx("span",{className:"text-current",children:k})}):x.jsx(tD,{inline:w,className:E,..._,renderAsDiagram:e.mermaidRendered??!1,messageRole:e.role,children:k})},p:({children:S})=>x.jsx("div",{className:"my-2",children:S}),h1:({children:S})=>x.jsx("h1",{className:"text-xl font-bold mt-4 mb-2",children:S}),h2:({children:S})=>x.jsx("h2",{className:"text-lg font-bold mt-4 mb-2",children:S}),h3:({children:S})=>x.jsx("h3",{className:"text-base font-bold mt-3 mb-2",children:S}),h4:({children:S})=>x.jsx("h4",{className:"text-base font-semibold mt-3 mb-2",children:S}),ul:({children:S})=>x.jsx("ul",{className:"list-disc pl-5 my-2",children:S}),ol:({children:S})=>x.jsx("ol",{className:"list-decimal pl-5 my-2",children:S}),li:({children:S})=>x.jsx("li",{className:"my-1",children:S})}),[e.mermaidRendered,e.role]),y=T.useMemo(()=>({code:S=>x.jsx(tD,{...S,renderAsDiagram:e.mermaidRendered??!1,messageRole:e.role})}),[e.mermaidRendered,e.role]);return x.jsxs("div",{className:`${e.role==="user"?"max-w-[80%] bg-primary text-primary-foreground":e.isError?"w-[95%] bg-red-100 text-red-600 dark:bg-red-950 dark:text-red-400":"w-[95%] bg-muted"} rounded-lg px-4 py-2`,children:[e.role==="assistant"&&(p||d!==null)&&x.jsxs("div",{className:lt("mb-2",!t&&"opacity-50"),children:[x.jsxs("div",{className:"flex items-center text-gray-500 dark:text-gray-400 hover:text-gray-700 dark:hover:text-gray-200 transition-colors duration-200 text-sm cursor-pointer select-none",onClick:()=>{f&&f.trim()!==""&&s(!o)},children:[p?x.jsxs(x.Fragment,{children:[t&&x.jsx(a7,{className:"mr-2 size-4 animate-spin"}),x.jsx("span",{children:r("retrievePanel.chatMessage.thinking")})]}):typeof d=="number"&&x.jsx("span",{children:r("retrievePanel.chatMessage.thinkingTime",{time:d})}),f&&f.trim()!==""&&x.jsx(Zv,{className:`ml-2 size-4 shrink-0 transition-transform ${o?"rotate-180":""}`})]}),o&&f&&f.trim()!==""&&x.jsxs("div",{className:"mt-2 pl-4 border-l-2 border-primary/20 dark:border-primary/40 text-sm prose dark:prose-invert max-w-none break-words prose-p:my-1 prose-headings:my-2 [&_sup]:text-[0.75em] [&_sup]:align-[0.1em] [&_sup]:leading-[0] [&_sub]:text-[0.75em] [&_sub]:align-[-0.2em] [&_sub]:leading-[0] [&_mark]:bg-yellow-200 [&_mark]:dark:bg-yellow-800 [&_u]:underline [&_del]:line-through [&_ins]:underline [&_ins]:decoration-green-500 [&_.footnotes]:mt-6 [&_.footnotes]:pt-3 [&_.footnotes]:border-t [&_.footnotes]:border-border [&_.footnotes_ol]:text-xs [&_.footnotes_li]:my-0.5 [&_a[href^='#fn']]:text-primary [&_a[href^='#fn']]:no-underline [&_a[href^='#fn']]:hover:underline [&_a[href^='#fnref']]:text-primary [&_a[href^='#fnref']]:no-underline [&_a[href^='#fnref']]:hover:underline text-foreground",children:[p&&x.jsx("div",{className:"mb-2 text-xs text-gray-400 dark:text-gray-300 italic",children:r("retrievePanel.chatMessage.thinkingInProgress","Thinking in progress...")}),x.jsx(XP,{remarkPlugins:[oz,pU,_z],rehypePlugins:[Az,...a&&(e.latexRendered??!0)?[[a,{errorColor:n==="dark"?"#ef4444":"#dc2626",throwOnError:!1,displayMode:!1,strict:!1,trust:!0,errorCallback:(S,w)=>{}}]]:[],sz],skipHtml:!1,components:y,children:f})]})]}),m&&x.jsx("div",{className:"relative",children:x.jsx(XP,{className:`prose dark:prose-invert max-w-none text-sm break-words prose-headings:mt-4 prose-headings:mb-2 prose-p:my-2 prose-ul:my-2 prose-ol:my-2 prose-li:my-1 [&_.katex]:text-current [&_.katex-display]:my-4 [&_.katex-display]:max-w-full [&_.katex-display_>.base]:overflow-x-auto [&_sup]:text-[0.75em] [&_sup]:align-[0.1em] [&_sup]:leading-[0] [&_sub]:text-[0.75em] [&_sub]:align-[-0.2em] [&_sub]:leading-[0] [&_mark]:bg-yellow-200 [&_mark]:dark:bg-yellow-800 [&_u]:underline [&_del]:line-through [&_ins]:underline [&_ins]:decoration-green-500 [&_.footnotes]:mt-8 [&_.footnotes]:pt-4 [&_.footnotes]:border-t [&_.footnotes_ol]:text-sm [&_.footnotes_li]:my-1 ${e.role==="user"?"text-primary-foreground":"text-foreground"} ${e.role==="user"?'[&_.footnotes]:border-primary-foreground/30 [&_a[href^="#fn"]]:text-primary-foreground [&_a[href^="#fn"]]:no-underline [&_a[href^="#fn"]]:hover:underline [&_a[href^="#fnref"]]:text-primary-foreground [&_a[href^="#fnref"]]:no-underline [&_a[href^="#fnref"]]:hover:underline':'[&_.footnotes]:border-border [&_a[href^="#fn"]]:text-primary [&_a[href^="#fn"]]:no-underline [&_a[href^="#fn"]]:hover:underline [&_a[href^="#fnref"]]:text-primary [&_a[href^="#fnref"]]:no-underline [&_a[href^="#fnref"]]:hover:underline'}`,remarkPlugins:[oz,pU,_z],rehypePlugins:[Az,...a&&(e.latexRendered??!0)?[[a,{errorColor:n==="dark"?"#ef4444":"#dc2626",throwOnError:!1,displayMode:!1,strict:!1,trust:!0,errorCallback:(S,w)=>{}}]]:[],sz],skipHtml:!1,components:b,children:m})}),t&&(!(m&&m.trim()!=="")&&!p&&!d&&x.jsx(a7,{className:"animate-spin duration-2000"}))]})},Plt=(e,t)=>!t||e!=="json"?!1:t.length>5e3,tD=T.memo(({inline:e,className:t,children:r,renderAsDiagram:n=!1,messageRole:a,...i})=>{const{theme:o}=A5(),[s,l]=T.useState(!1),u=t==null?void 0:t.match(/language-(\w+)/),d=u?u[1]:void 0,p=T.useRef(null),f=T.useRef(null),m=String(r||"").replace(/\n$/,""),b=Plt(d,m);if(T.useEffect(()=>{if(n&&!s&&d==="mermaid"&&p.current){const w=p.current;f.current&&clearTimeout(f.current),f.current=setTimeout(()=>{if(w&&!s)try{hU.initialize({startOnLoad:!1,theme:o==="dark"?"dark":"default",securityLevel:"loose",suppressErrorRendering:!0}),w.innerHTML='<div class="flex justify-center items-center p-4"><svg class="animate-spin h-5 w-5 text-primary" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24"><circle class="opacity-25" cx="12" cy="12" r="10" stroke="currentColor" stroke-width="4"></circle><path class="opacity-75" fill="currentColor" d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"></path></svg></div>';const E=String(r).replace(/\n$/,"").trim();if(!(E.length>10&&(E.startsWith("graph")||E.startsWith("sequenceDiagram")||E.startsWith("classDiagram")||E.startsWith("stateDiagram")||E.startsWith("gantt")||E.startsWith("pie")||E.startsWith("flowchart")||E.startsWith("erDiagram")))){console.log("Mermaid content might be incomplete, skipping render attempt:",E);return}const _=E.split(`
`).map(C=>{const F=C.trim();if(F.startsWith("subgraph")){const L=F.split(" ");if(L.length>1)return`subgraph "${L.slice(1).join(" ").replace(/["']/g,"")}"`}return F}).filter(C=>!C.trim().startsWith("linkStyle")).join(`
`),R=`mermaid-${Date.now()}`;hU.render(R,_).then(({svg:C,bindFunctions:F})=>{if(p.current===w&&!s){if(w.innerHTML=C,l(!0),F)try{F(w)}catch(L){console.error("Mermaid bindFunctions error:",L),w.innerHTML+='<p class="text-orange-500 text-xs">Diagram interactions might be limited.</p>'}}else p.current!==w&&console.log("Mermaid container changed before rendering completed.")}).catch(C=>{if(console.error("Mermaid rendering promise error (debounced):",C),console.error("Failed content (debounced):",_),p.current===w){const F=C instanceof Error?C.message:String(C),L=document.createElement("pre");L.className="text-red-500 text-xs whitespace-pre-wrap break-words",L.textContent=`Mermaid diagram error: ${F}

Content:
${_}`,w.innerHTML="",w.appendChild(L)}})}catch(E){if(console.error("Mermaid synchronous error (debounced):",E),console.error("Failed content (debounced):",String(r)),p.current===w){const k=E instanceof Error?E.message:String(E),_=document.createElement("pre");_.className="text-red-500 text-xs whitespace-pre-wrap break-words",_.textContent=`Mermaid diagram setup error: ${k}`,w.innerHTML="",w.appendChild(_)}}},300)}return()=>{f.current&&clearTimeout(f.current)}},[n,s,d,r,o]),b)return x.jsx("pre",{className:"whitespace-pre-wrap break-words bg-muted p-4 rounded-md overflow-x-auto text-sm font-mono",children:m});if(d==="mermaid"&&!n)return x.jsx(eD,{style:o==="dark"?f1.oneDark:f1.oneLight,PreTag:"div",language:"text",...i,children:m});if(d==="mermaid")return x.jsx("div",{className:"mermaid-diagram-container my-4 overflow-x-auto",ref:p});const y=e??!(t!=null&&t.startsWith("language-")),S=()=>a==="user"?"bg-primary-foreground/20 text-primary-foreground border border-primary-foreground/30":o==="dark"?"bg-muted-foreground/20 text-muted-foreground border border-muted-foreground/30":"bg-slate-200 text-slate-800 border border-slate-300";return y?x.jsx("code",{className:lt(t,"mx-1 rounded-sm px-1 py-0.5 font-mono text-sm",S()),...i,children:r}):x.jsx(eD,{style:o==="dark"?f1.oneDark:f1.oneLight,PreTag:"div",language:d,...i,children:m})});tD.displayName="CodeHighlight";async function zlt(e){if(!e||e.trim()==="")return{success:!1,method:"fallback",error:"No text provided"};if(navigator.clipboard&&typeof navigator.clipboard.writeText=="function")try{return await navigator.clipboard.writeText(e),{success:!0,method:"clipboard-api"}}catch(t){console.warn("Clipboard API failed:",t)}try{const t=await Hlt(e);if(t.success)return t}catch(t){console.warn("execCommand failed:",t)}try{const t=await Ult(e);if(t.success)return t}catch(t){console.warn("Manual selection failed:",t)}return{success:!1,method:"fallback",error:"All copy methods failed. Please copy the text manually."}}async function Hlt(e){return new Promise(t=>{const r=document.createElement("textarea");r.value=e,r.style.position="fixed",r.style.left="-9999px",r.style.top="-9999px",r.style.opacity="0",r.setAttribute("readonly",""),document.body.appendChild(r);try{r.select(),r.setSelectionRange(0,e.length);const n=document.execCommand("copy");t(n?{success:!0,method:"execCommand"}:{success:!1,method:"execCommand",error:"execCommand returned false"})}catch(n){t({success:!1,method:"execCommand",error:n instanceof Error?n.message:"execCommand failed"})}finally{document.body.removeChild(r)}})}async function Ult(e){return new Promise(t=>{const r=document.createElement("textarea");r.value=e,r.style.position="absolute",r.style.left="-9999px",r.style.top="-9999px",r.style.opacity="0",r.style.pointerEvents="none",r.setAttribute("readonly",""),r.setAttribute("tabindex","-1"),document.body.appendChild(r);try{r.focus(),r.select(),r.setSelectionRange(0,e.length);const n=new ClipboardEvent("copy",{clipboardData:new DataTransfer});n.clipboardData?(n.clipboardData.setData("text/plain",e),document.dispatchEvent(n),t({success:!0,method:"manual-select"})):t({success:!1,method:"manual-select",error:"Manual selection prepared, but automatic copy failed"})}catch(n){t({success:!1,method:"manual-select",error:n instanceof Error?n.message:"Manual selection failed"})}finally{setTimeout(()=>{document.body.contains(r)&&document.body.removeChild(r)},100)}})}const KV=()=>typeof crypto<"u"&&typeof crypto.randomUUID=="function"?crypto.randomUUID():`id-${Date.now()}-${Math.random().toString(36).substring(2,9)}`,$lt=e=>{const r=(e.match(/\$\$/g)||[]).length%2!==0,i=(e.replace(/\$\$[\s\S]*?\$\$/g,"").match(new RegExp("(?<!\\$)\\$(?!\\$)","g"))||[]).length%2!==0;return!r&&!i},YV=e=>{const t="<think>",r="</think>",n=[],a=[];let i=0;for(;(i=e.indexOf(t,i))!==-1;)n.push(i),i+=t.length;let o=0;for(;(o=e.indexOf(r,o))!==-1;)a.push(o),o+=r.length;const s=n.length>0,l=a.length>0,u=s&&n.length>a.length;let d="",p=e;if(s){if(l&&n.length===a.length){const f=n[n.length-1],m=a[a.length-1];m>f&&(d=e.substring(f+t.length,m).trim(),p=e.substring(m+r.length).trim())}else if(u){const f=n[n.length-1];d=e.substring(f+t.length),p=""}}return{isThinking:u,thinkingContent:d,displayContent:p,hasValidThinkBlock:s&&l&&n.length===a.length}};function Glt(){const{t:e}=Sr(),r=Ve.use.currentTab()==="retrieval",[n,a]=T.useState(()=>{try{return(Ve.getState().retrievalHistory||[]).map((X,z)=>{try{const H=X;return{...X,id:H.id||`hist-${Date.now()}-${z}`,mermaidRendered:H.mermaidRendered??!0,latexRendered:H.latexRendered??!0}}catch(H){return console.error("Error processing message:",H),{role:"system",content:"Error loading message",id:`error-${Date.now()}-${z}`,isError:!0,mermaidRendered:!0}}})}catch($){return console.error("Error loading history:",$),[]}}),[i,o]=T.useState(""),[s,l]=T.useState(!1),[u,d]=T.useState(""),p=T.useRef(null),f=i.includes(`
`),m=T.useCallback($=>{o($.target.value),u&&d("")},[u]),b=T.useCallback($=>{requestAnimationFrame(()=>{$.style.height="auto",$.style.height=Math.min($.scrollHeight,120)+"px"})},[]),y=T.useCallback(()=>{F.current=!0,requestAnimationFrame(()=>{I.current&&I.current.scrollIntoView({behavior:"auto"})})},[]),S=T.useCallback(async $=>{if($.preventDefault(),!i.trim()||s)return;const X=["naive","local","global","hybrid","mix","bypass"],z=i.match(/^\/(\w+)\s+([\s\S]+)/);let H,B=i;if(/^\/\S+/.test(i)&&!z){d(e("retrievePanel.retrieval.queryModePrefixInvalid"));return}if(z){const se=z[1],Ee=z[2];if(!X.includes(se)){d(e("retrievePanel.retrieval.queryModeError",{modes:"naive, local, global, hybrid, mix, bypass"}));return}H=se,B=Ee}d(""),_.current=null,R.current=!1;const ee={id:KV(),content:i,role:"user"},J={id:KV(),content:"",role:"assistant",mermaidRendered:!1,latexRendered:!1,thinkingTime:null,thinkingContent:void 0,displayContent:void 0,isThinking:!1},K=[...n];a([...K,ee,J]),k.current=!0,L.current=!0,setTimeout(()=>{y()},0),o(""),l(!0),p.current&&"style"in p.current&&(p.current.style.height="40px");const G=(se,Ee)=>{J.content+=se,J.content.includes("<think>")&&!_.current&&(_.current=Date.now());const fe=YV(J.content);if(J.isThinking=fe.isThinking,fe.hasValidThinkBlock&&!R.current){if(_.current&&!J.thinkingTime){const Ne=(Date.now()-_.current)/1e3;J.thinkingTime=parseFloat(Ne.toFixed(2))}R.current=!0}J.thinkingContent=fe.thinkingContent,fe.isThinking?J.displayContent="":J.displayContent=fe.displayContent||J.content;const Me=/```mermaid\s+([\s\S]+?)```/g;let Oe=!1,xe;for(;(xe=Me.exec(J.content))!==null;)if(xe[1]&&xe[1].trim().length>10){Oe=!0;break}J.mermaidRendered=Oe;const Te=$lt(J.content);J.latexRendered=Te,a(Ne=>{const he=[...Ne],Ze=he[he.length-1];return Ze&&Ze.id===J.id&&Object.assign(Ze,{content:J.content,thinkingContent:J.thinkingContent,displayContent:J.displayContent,isThinking:J.isThinking,isError:Ee,mermaidRendered:J.mermaidRendered,latexRendered:J.latexRendered,thinkingTime:J.thinkingTime}),he}),k.current&&setTimeout(()=>{y()},30)},P=Ve.getState();P.querySettings.user_prompt&&P.querySettings.user_prompt.trim()&&P.addUserPromptToHistory(P.querySettings.user_prompt.trim());const ne=H||P.querySettings.mode,oe=P.querySettings.history_turns||0,pe=ne==="bypass"&&oe===0?3:oe,ae={...P.querySettings,query:B,conversation_history:pe>0?K.filter(se=>se.isError!==!0).slice(-pe*2).map(se=>({role:se.role,content:se.content})):[],...H?{mode:H}:{}};try{if(P.querySettings.stream){let se="";await l1e(ae,G,Ee=>{se+=Ee}),se&&(J.content&&(se=J.content+`
`+se),G(se,!0))}else{const se=await s1e(ae);G(se.response)}}catch(se){G(`${e("retrievePanel.retrieval.error")}
${Da(se)}`,!0)}finally{l(!1),L.current=!1;try{const se=YV(J.content);if(J.isThinking=!1,se.hasValidThinkBlock&&_.current&&!J.thinkingTime){const Ee=(Date.now()-_.current)/1e3;J.thinkingTime=parseFloat(Ee.toFixed(2))}se.displayContent!==void 0&&(J.displayContent=se.displayContent)}catch(se){console.error("Error in final COT state validation:",se),J.isThinking=!1}finally{_.current=null}try{Ve.getState().setRetrievalHistory([...K,ee,J])}catch(se){console.error("Error saving retrieval history:",se)}}},[i,s,n,a,e,y]),w=T.useCallback($=>{if($.key==="Enter"&&$.shiftKey){$.preventDefault();const X=$.target,z=X.selectionStart||0,H=X.selectionEnd||0,B=i.slice(0,z)+`
`+i.slice(H);o(B),setTimeout(()=>{X.setSelectionRange&&X.setSelectionRange(z+1,z+1),p.current&&p.current.tagName==="TEXTAREA"&&b(p.current)},0)}else $.key==="Enter"&&!$.shiftKey&&($.preventDefault(),S($))},[i,S,b]),E=T.useCallback($=>{const X=$.clipboardData.getData("text");if(X.includes(`
`)){$.preventDefault();const z=$.target,H=z.selectionStart||0,B=z.selectionEnd||0,ee=i.slice(0,H)+X+i.slice(B);o(ee),setTimeout(()=>{if(p.current&&p.current.setSelectionRange){const J=H+X.length;p.current.setSelectionRange(J,J)}},0)}},[i]);T.useEffect(()=>{if(p.current){const $=p.current,X=$.selectionStart||i.length;requestAnimationFrame(()=>{$.focus(),$.setSelectionRange&&$.setSelectionRange(X,X)})}},[f,i.length]),T.useEffect(()=>{f&&p.current&&p.current.tagName==="TEXTAREA"&&b(p.current)},[f,i,b]);const k=T.useRef(!0),_=T.useRef(null),R=T.useRef(!1),C=T.useRef(!1),F=T.useRef(!1),L=T.useRef(!1),I=T.useRef(null),M=T.useRef(null);T.useEffect(()=>()=>{_.current&&(_.current=null)},[]),T.useEffect(()=>{const $=M.current;if(!$)return;const X=H=>{Math.abs(H.deltaY)>10&&!C.current&&(k.current=!1)},z=c0e(()=>{if(F.current){F.current=!1;return}const H=M.current;H&&(H.scrollHeight-H.scrollTop-H.clientHeight<20?k.current=!0:!C.current&&!L.current&&(k.current=!1))},30);return $.addEventListener("wheel",X),$.addEventListener("scroll",z),()=>{$.removeEventListener("wheel",X),$.removeEventListener("scroll",z)}},[]),T.useEffect(()=>{const $=document.querySelector("form");if(!$)return;const X=()=>{C.current=!0,setTimeout(()=>{C.current=!1},500)};return $.addEventListener("mousedown",X),()=>{$.removeEventListener("mousedown",X)}},[]);const j=N5(n,150);T.useEffect(()=>{k.current&&y()},[j,y]);const Y=T.useCallback(()=>{a([]),Ve.getState().setRetrievalHistory([])},[a]),W=T.useCallback(async $=>{let X="";if($.role==="user"?X=$.content||"":X=$.displayContent!==void 0?$.displayContent:$.content||"",!X.trim()){er.error(e("retrievePanel.chatMessage.copyEmpty","No content to copy"));return}try{const z=await zlt(X);if(z.success){const H={"clipboard-api":e("retrievePanel.chatMessage.copySuccess","Content copied to clipboard"),execCommand:e("retrievePanel.chatMessage.copySuccessLegacy","Content copied (legacy method)"),"manual-select":e("retrievePanel.chatMessage.copySuccessManual","Content copied (manual method)"),fallback:e("retrievePanel.chatMessage.copySuccess","Content copied to clipboard")};er.success(H[z.method]||e("retrievePanel.chatMessage.copySuccess","Content copied to clipboard"))}else z.method==="fallback"?er.error(z.error||e("retrievePanel.chatMessage.copyFailed","Failed to copy content"),{description:e("retrievePanel.chatMessage.copyManualInstruction","Please select and copy the text manually")}):er.error(e("retrievePanel.chatMessage.copyFailed","Failed to copy content"),{description:z.error})}catch(z){console.error("Clipboard operation failed:",z),er.error(e("retrievePanel.chatMessage.copyError","Copy operation failed"),{description:z instanceof Error?z.message:"Unknown error occurred"})}},[e]);return x.jsxs("div",{className:"flex size-full gap-2 px-2 pb-12 overflow-hidden",children:[x.jsxs("div",{className:"flex grow flex-col gap-4",children:[x.jsx("div",{className:"relative grow",children:x.jsx("div",{ref:M,className:"bg-primary-foreground/60 absolute inset-0 flex flex-col overflow-auto rounded-lg border p-2",onClick:()=>{k.current&&(k.current=!1)},children:x.jsxs("div",{className:"flex min-h-0 flex-1 flex-col gap-2",children:[n.length===0?x.jsx("div",{className:"text-muted-foreground flex h-full items-center justify-center text-lg",children:e("retrievePanel.retrieval.startPrompt")}):n.map($=>x.jsxs("div",{className:`flex ${$.role==="user"?"justify-end":"justify-start"} items-end gap-2`,children:[$.role==="user"&&x.jsx(Rt,{onClick:()=>W($),className:"mb-2 size-6 rounded-md opacity-60 transition-opacity hover:opacity-100 shrink-0",tooltip:e("retrievePanel.chatMessage.copyTooltip"),variant:"ghost",size:"icon",children:x.jsx(n7,{className:"size-4"})}),x.jsx(Blt,{message:$,isTabActive:r}),$.role==="assistant"&&x.jsx(Rt,{onClick:()=>W($),className:"mb-2 size-6 rounded-md opacity-60 transition-opacity hover:opacity-100 shrink-0",tooltip:e("retrievePanel.chatMessage.copyTooltip"),variant:"ghost",size:"icon",children:x.jsx(n7,{className:"size-4"})})]},$.id)),x.jsx("div",{ref:I,className:"pb-1"})]})})}),x.jsxs("form",{onSubmit:S,className:"flex shrink-0 items-center gap-2",autoComplete:"on",method:"post",action:"#",role:"search",children:[x.jsx("input",{type:"submit",style:{display:"none"},tabIndex:-1}),x.jsxs(Rt,{type:"button",variant:"outline",onClick:Y,disabled:s,size:"sm",children:[x.jsx(sZ,{}),e("retrievePanel.retrieval.clear")]}),x.jsxs("div",{className:"flex-1 relative",children:[x.jsx("label",{htmlFor:"query-input",className:"sr-only",children:e("retrievePanel.retrieval.placeholder")}),f?x.jsx(nre,{ref:p,id:"query-input",autoComplete:"on",className:"w-full min-h-[40px] max-h-[120px] overflow-y-auto",value:i,onChange:m,onKeyDown:w,onPaste:E,placeholder:e("retrievePanel.retrieval.placeholder"),disabled:s,rows:1,style:{resize:"none",height:"auto",minHeight:"40px",maxHeight:"120px"},onInput:$=>{const X=$.target;requestAnimationFrame(()=>{X.style.height="auto",X.style.height=Math.min(X.scrollHeight,120)+"px"})}}):x.jsx(oa,{ref:p,id:"query-input",autoComplete:"on",className:"w-full",value:i,onChange:m,onKeyDown:w,onPaste:E,placeholder:e("retrievePanel.retrieval.placeholder"),disabled:s}),u&&x.jsx("div",{className:"absolute left-0 top-full mt-1 text-xs text-red-500",children:u})]}),x.jsxs(Rt,{type:"submit",variant:"default",disabled:s,size:"sm",children:[x.jsx(WEe,{}),e("retrievePanel.retrieval.send")]})]})]}),x.jsx(xNe,{})]})}const jlt=()=>{const e=T.useContext(CJ);if(!e)throw new Error("useTabVisibility must be used within a TabVisibilityProvider");return e};function qlt(){const{t:e}=Sr(),{isTabVisible:t}=jlt(),r=t("api"),[n,a]=T.useState(!1);return T.useEffect(()=>{n||a(!0)},[n]),x.jsx("div",{className:`size-full ${r?"":"hidden"}`,children:n?x.jsx("iframe",{src:ly+"/docs",className:"size-full w-full h-full",style:{width:"100%",height:"100%",border:"none"}},"api-docs-iframe"):x.jsx("div",{className:"flex h-full w-full items-center justify-center bg-background",children:x.jsxs("div",{className:"text-center",children:[x.jsx("div",{className:"mb-2 h-8 w-8 animate-spin rounded-full border-4 border-primary border-t-transparent"}),x.jsx("p",{children:e("apiSite.loading")})]})})})}function Wlt(){const e=Xn.use.message(),t=Ve.use.enableHealthCheck(),r=Ve.use.currentTab(),[n,a]=T.useState(!1),[i,o]=T.useState(!0),s=T.useRef(!1),l=T.useRef(!1),u=T.useCallback(f=>{a(f),f||Xn.getState().clear()},[]),d=T.useRef(!0);T.useEffect(()=>{d.current=!0;const f=()=>{d.current=!1};return window.addEventListener("beforeunload",f),()=>{d.current=!1,window.removeEventListener("beforeunload",f)}},[]),T.useEffect(()=>{const f=async()=>{try{d.current&&await Xn.getState().check()}catch(m){console.error("Health check error:",m)}};if(Xn.getState().setHealthCheckFunction(f),!t||n){Xn.getState().clearHealthCheckTimer();return}return l.current||(l.current=!0),Xn.getState().resetHealthCheckTimer(),()=>{Xn.getState().clearHealthCheckTimer()}},[t,n]),T.useEffect(()=>{(async()=>{if(s.current)return;if(s.current=!0,sessionStorage.getItem("VERSION_CHECKED_FROM_LOGIN")==="true"){o(!1);return}try{o(!0);const b=localStorage.getItem("LIGHTRAG-API-TOKEN"),y=await SJ();if(!y.auth_configured&&y.access_token)Oi.getState().login(y.access_token,!0,y.core_version,y.api_version,y.webui_title||null,y.webui_description||null);else if(b&&(y.core_version||y.api_version||y.webui_title||y.webui_description)){const S=y.auth_mode==="disabled"||Oi.getState().isGuestMode;Oi.getState().login(b,S,y.core_version,y.api_version,y.webui_title||null,y.webui_description||null)}sessionStorage.setItem("VERSION_CHECKED_FROM_LOGIN","true")}catch(b){console.error("Failed to get version info:",b)}finally{o(!1)}})()},[]);const p=T.useCallback(f=>Ve.getState().setCurrentTab(f),[]);return T.useEffect(()=>{e&&(e.includes(bJ)||e.includes(yJ))&&a(!0)},[e]),x.jsx(kJ,{children:x.jsx(X1e,{children:i?x.jsxs("div",{className:"flex h-screen w-screen flex-col",children:[x.jsxs("header",{className:"border-border/40 bg-background/95 supports-[backdrop-filter]:bg-background/60 sticky top-0 z-50 flex h-10 w-full border-b px-4 backdrop-blur",children:[x.jsx("div",{className:"min-w-[200px] w-auto flex items-center",children:x.jsxs("a",{href:fJ,className:"flex items-center gap-2",children:[x.jsx(l5,{className:"size-4 text-emerald-400","aria-hidden":"true"}),x.jsx("span",{className:"font-bold md:inline-block",children:j_.name})]})}),x.jsx("div",{className:"flex h-10 flex-1 items-center justify-center"}),x.jsx("nav",{className:"w-[200px] flex items-center justify-end"})]}),x.jsx("div",{className:"flex flex-1 items-center justify-center",children:x.jsxs("div",{className:"text-center",children:[x.jsx("div",{className:"mb-2 h-8 w-8 animate-spin rounded-full border-4 border-primary border-t-transparent mx-auto"}),x.jsx("p",{children:"Initializing..."})]})})]}):x.jsxs("main",{className:"flex h-screen w-screen overflow-hidden",children:[x.jsxs(gwe,{defaultValue:r,className:"!m-0 flex grow flex-col !p-0 overflow-hidden",onValueChange:p,children:[x.jsx(ywe,{}),x.jsxs("div",{className:"relative grow",children:[x.jsx(am,{value:"documents",className:"absolute top-0 right-0 bottom-0 left-0 overflow-auto",children:x.jsx(SNe,{})}),x.jsx(am,{value:"knowledge-graph",className:"absolute top-0 right-0 bottom-0 left-0 overflow-hidden",children:x.jsx(o5e,{})}),x.jsx(am,{value:"retrieval",className:"absolute top-0 right-0 bottom-0 left-0 overflow-hidden",children:x.jsx(Glt,{})}),x.jsx(am,{value:"api",className:"absolute top-0 right-0 bottom-0 left-0 overflow-hidden",children:x.jsx(qlt,{})})]})]}),t&&x.jsx(cxe,{}),x.jsx(NSe,{open:n,onOpenChange:u})]})})})}const Vlt=()=>{const e=CD(),{login:t,isAuthenticated:r}=Oi(),{t:n}=Sr(),[a,i]=T.useState(!1),[o,s]=T.useState(""),[l,u]=T.useState(""),[d,p]=T.useState(!0),f=T.useRef(!1);if(T.useEffect(()=>{console.log("LoginPage mounted")},[]),T.useEffect(()=>((async()=>{if(!f.current){f.current=!0;try{if(r){e("/");return}const y=await SJ();if((y.core_version||y.api_version)&&sessionStorage.setItem("VERSION_CHECKED_FROM_LOGIN","true"),!y.auth_configured&&y.access_token){t(y.access_token,!0,y.core_version,y.api_version,y.webui_title||null,y.webui_description||null),y.message&&er.info(y.message),e("/");return}p(!1)}catch(y){console.error("Failed to check auth configuration:",y),p(!1)}}})(),()=>{}),[r,t,e]),d)return null;const m=async b=>{if(b.preventDefault(),!o||!l){er.error(n("login.errorEmptyFields"));return}try{i(!0);const y=await m1e(o,l);localStorage.getItem("LIGHTRAG-PREVIOUS-USER")===o?console.log("Same user logging in, preserving chat history"):(console.log("Different user logging in, clearing chat history"),Ve.getState().setRetrievalHistory([])),localStorage.setItem("LIGHTRAG-PREVIOUS-USER",o);const E=y.auth_mode==="disabled";t(y.access_token,E,y.core_version,y.api_version,y.webui_title||null,y.webui_description||null),(y.core_version||y.api_version)&&sessionStorage.setItem("VERSION_CHECKED_FROM_LOGIN","true"),E?er.info(y.message||n("login.authDisabled","Authentication is disabled. Using guest access.")):er.success(n("login.successMessage")),e("/")}catch(y){console.error("Login failed...",y),er.error(n("login.errorInvalidCredentials")),Oi.getState().logout(),localStorage.removeItem("LIGHTRAG-API-TOKEN")}finally{i(!1)}};return x.jsxs("div",{className:"flex h-screen w-screen items-center justify-center bg-gradient-to-br from-emerald-50 to-teal-100 dark:from-gray-900 dark:to-gray-800",children:[x.jsx("div",{className:"absolute top-4 right-4 flex items-center gap-2",children:x.jsx(uQ,{className:"bg-white/30 dark:bg-gray-800/30 backdrop-blur-sm rounded-md"})}),x.jsxs(qc,{className:"w-full max-w-[480px] shadow-lg mx-4",children:[x.jsx(Hm,{className:"flex items-center justify-center space-y-2 pb-8 pt-6",children:x.jsxs("div",{className:"flex flex-col items-center space-y-4",children:[x.jsxs("div",{className:"flex items-center gap-3",children:[x.jsx("img",{src:"logo.svg",alt:"LightRAG Logo",className:"h-12 w-12"}),x.jsx(l5,{className:"size-10 text-emerald-400","aria-hidden":"true"})]}),x.jsxs("div",{className:"text-center space-y-2",children:[x.jsx("h1",{className:"text-3xl font-bold tracking-tight",children:"LightRAG"}),x.jsx("p",{className:"text-muted-foreground text-sm",children:n("login.description")})]})]})}),x.jsx($m,{className:"px-8 pb-8",children:x.jsxs("form",{onSubmit:m,className:"space-y-6",children:[x.jsxs("div",{className:"flex items-center gap-4",children:[x.jsx("label",{htmlFor:"username-input",className:"text-sm font-medium w-16 shrink-0",children:n("login.username")}),x.jsx(oa,{id:"username-input",placeholder:n("login.usernamePlaceholder"),value:o,onChange:b=>s(b.target.value),required:!0,className:"h-11 flex-1"})]}),x.jsxs("div",{className:"flex items-center gap-4",children:[x.jsx("label",{htmlFor:"password-input",className:"text-sm font-medium w-16 shrink-0",children:n("login.password")}),x.jsx(oa,{id:"password-input",type:"password",placeholder:n("login.passwordPlaceholder"),value:l,onChange:b=>u(b.target.value),required:!0,className:"h-11 flex-1"})]}),x.jsx(Rt,{type:"submit",className:"w-full h-11 text-base font-medium mt-2",disabled:a,children:n(a?"login.loggingIn":"login.loginButton")})]})})]})]})},Klt=()=>{const[e,t]=T.useState(!0),{isAuthenticated:r}=Oi(),n=CD();return T.useEffect(()=>{Bv.setNavigate(n)},[n]),T.useEffect(()=>((async()=>{try{const i=localStorage.getItem("LIGHTRAG-API-TOKEN");if(i&&r){t(!1);return}i||Oi.getState().logout()}catch(i){console.error("Auth initialization error:",i),r||Oi.getState().logout()}finally{t(!1)}})(),()=>{}),[r]),T.useEffect(()=>{!e&&!r&&window.location.hash.slice(1)!=="/login"&&(console.log("Not authenticated, redirecting to login"),n("/login"))},[e,r,n]),e?null:x.jsxs(Kme,{children:[x.jsx(D_,{path:"/login",element:x.jsx(Vlt,{})}),x.jsx(D_,{path:"/*",element:r?x.jsx(Wlt,{}):null})]})},Ylt=()=>x.jsx(kJ,{children:x.jsxs(bge,{children:[x.jsx(Klt,{}),x.jsx(K1e,{position:"bottom-center",theme:"system",closeButton:!0,richColors:!0})]})}),ur=e=>typeof e=="string",tm=()=>{let e,t;const r=new Promise((n,a)=>{e=n,t=a});return r.resolve=e,r.reject=t,r},JV=e=>e==null?"":""+e,Jlt=(e,t,r)=>{e.forEach(n=>{t[n]&&(r[n]=t[n])})},Xlt=/###/g,XV=e=>e&&e.indexOf("###")>-1?e.replace(Xlt,"."):e,ZV=e=>!e||ur(e),Lm=(e,t,r)=>{const n=ur(t)?t.split("."):t;let a=0;for(;a<n.length-1;){if(ZV(e))return{};const i=XV(n[a]);!e[i]&&r&&(e[i]=new r),Object.prototype.hasOwnProperty.call(e,i)?e=e[i]:e={},++a}return ZV(e)?{}:{obj:e,k:XV(n[a])}},QV=(e,t,r)=>{const{obj:n,k:a}=Lm(e,t,Object);if(n!==void 0||t.length===1){n[a]=r;return}let i=t[t.length-1],o=t.slice(0,t.length-1),s=Lm(e,o,Object);for(;s.obj===void 0&&o.length;)i=`${o[o.length-1]}.${i}`,o=o.slice(0,o.length-1),s=Lm(e,o,Object),s!=null&&s.obj&&typeof s.obj[`${s.k}.${i}`]<"u"&&(s.obj=void 0);s.obj[`${s.k}.${i}`]=r},Zlt=(e,t,r,n)=>{const{obj:a,k:i}=Lm(e,t,Object);a[i]=a[i]||[],a[i].push(r)},bv=(e,t)=>{const{obj:r,k:n}=Lm(e,t);if(r&&Object.prototype.hasOwnProperty.call(r,n))return r[n]},Qlt=(e,t,r)=>{const n=bv(e,r);return n!==void 0?n:bv(t,r)},Uce=(e,t,r)=>{for(const n in t)n!=="__proto__"&&n!=="constructor"&&(n in e?ur(e[n])||e[n]instanceof String||ur(t[n])||t[n]instanceof String?r&&(e[n]=t[n]):Uce(e[n],t[n],r):e[n]=t[n]);return e},sh=e=>e.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g,"\\$&");var eut={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","/":"&#x2F;"};const tut=e=>ur(e)?e.replace(/[&<>"'\/]/g,t=>eut[t]):e;class rut{constructor(t){this.capacity=t,this.regExpMap=new Map,this.regExpQueue=[]}getRegExp(t){const r=this.regExpMap.get(t);if(r!==void 0)return r;const n=new RegExp(t);return this.regExpQueue.length===this.capacity&&this.regExpMap.delete(this.regExpQueue.shift()),this.regExpMap.set(t,n),this.regExpQueue.push(t),n}}const nut=[" ",",","?","!",";"],aut=new rut(20),iut=(e,t,r)=>{t=t||"",r=r||"";const n=nut.filter(o=>t.indexOf(o)<0&&r.indexOf(o)<0);if(n.length===0)return!0;const a=aut.getRegExp(`(${n.map(o=>o==="?"?"\\?":o).join("|")})`);let i=!a.test(e);if(!i){const o=e.indexOf(r);o>0&&!a.test(e.substring(0,o))&&(i=!0)}return i},rD=function(e,t){let r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:".";if(!e)return;if(e[t])return Object.prototype.hasOwnProperty.call(e,t)?e[t]:void 0;const n=t.split(r);let a=e;for(let i=0;i<n.length;){if(!a||typeof a!="object")return;let o,s="";for(let l=i;l<n.length;++l)if(l!==i&&(s+=r),s+=n[l],o=a[s],o!==void 0){if(["string","number","boolean"].indexOf(typeof o)>-1&&l<n.length-1)continue;i+=l-i+1;break}a=o}return a},yv=e=>e==null?void 0:e.replace("_","-"),out={type:"logger",log(e){this.output("log",e)},warn(e){this.output("warn",e)},error(e){this.output("error",e)},output(e,t){var r,n;(n=(r=console==null?void 0:console[e])==null?void 0:r.apply)==null||n.call(r,console,t)}};class vv{constructor(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};this.init(t,r)}init(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};this.prefix=r.prefix||"i18next:",this.logger=t||out,this.options=r,this.debug=r.debug}log(){for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return this.forward(r,"log","",!0)}warn(){for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return this.forward(r,"warn","",!0)}error(){for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return this.forward(r,"error","")}deprecate(){for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return this.forward(r,"warn","WARNING DEPRECATED: ",!0)}forward(t,r,n,a){return a&&!this.debug?null:(ur(t[0])&&(t[0]=`${n}${this.prefix} ${t[0]}`),this.logger[r](t))}create(t){return new vv(this.logger,{prefix:`${this.prefix}:${t}:`,...this.options})}clone(t){return t=t||this.options,t.prefix=t.prefix||this.prefix,new vv(this.logger,t)}}var os=new vv;class oS{constructor(){this.observers={}}on(t,r){return t.split(" ").forEach(n=>{this.observers[n]||(this.observers[n]=new Map);const a=this.observers[n].get(r)||0;this.observers[n].set(r,a+1)}),this}off(t,r){if(this.observers[t]){if(!r){delete this.observers[t];return}this.observers[t].delete(r)}}emit(t){for(var r=arguments.length,n=new Array(r>1?r-1:0),a=1;a<r;a++)n[a-1]=arguments[a];this.observers[t]&&Array.from(this.observers[t].entries()).forEach(o=>{let[s,l]=o;for(let u=0;u<l;u++)s(...n)}),this.observers["*"]&&Array.from(this.observers["*"].entries()).forEach(o=>{let[s,l]=o;for(let u=0;u<l;u++)s.apply(s,[t,...n])})}}class eK extends oS{constructor(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{ns:["translation"],defaultNS:"translation"};super(),this.data=t||{},this.options=r,this.options.keySeparator===void 0&&(this.options.keySeparator="."),this.options.ignoreJSONStructure===void 0&&(this.options.ignoreJSONStructure=!0)}addNamespaces(t){this.options.ns.indexOf(t)<0&&this.options.ns.push(t)}removeNamespaces(t){const r=this.options.ns.indexOf(t);r>-1&&this.options.ns.splice(r,1)}getResource(t,r,n){var u,d;let a=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{};const i=a.keySeparator!==void 0?a.keySeparator:this.options.keySeparator,o=a.ignoreJSONStructure!==void 0?a.ignoreJSONStructure:this.options.ignoreJSONStructure;let s;t.indexOf(".")>-1?s=t.split("."):(s=[t,r],n&&(Array.isArray(n)?s.push(...n):ur(n)&&i?s.push(...n.split(i)):s.push(n)));const l=bv(this.data,s);return!l&&!r&&!n&&t.indexOf(".")>-1&&(t=s[0],r=s[1],n=s.slice(2).join(".")),l||!o||!ur(n)?l:rD((d=(u=this.data)==null?void 0:u[t])==null?void 0:d[r],n,i)}addResource(t,r,n,a){let i=arguments.length>4&&arguments[4]!==void 0?arguments[4]:{silent:!1};const o=i.keySeparator!==void 0?i.keySeparator:this.options.keySeparator;let s=[t,r];n&&(s=s.concat(o?n.split(o):n)),t.indexOf(".")>-1&&(s=t.split("."),a=r,r=s[1]),this.addNamespaces(r),QV(this.data,s,a),i.silent||this.emit("added",t,r,n,a)}addResources(t,r,n){let a=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{silent:!1};for(const i in n)(ur(n[i])||Array.isArray(n[i]))&&this.addResource(t,r,i,n[i],{silent:!0});a.silent||this.emit("added",t,r,n)}addResourceBundle(t,r,n,a,i){let o=arguments.length>5&&arguments[5]!==void 0?arguments[5]:{silent:!1,skipCopy:!1},s=[t,r];t.indexOf(".")>-1&&(s=t.split("."),a=n,n=r,r=s[1]),this.addNamespaces(r);let l=bv(this.data,s)||{};o.skipCopy||(n=JSON.parse(JSON.stringify(n))),a?Uce(l,n,i):l={...l,...n},QV(this.data,s,l),o.silent||this.emit("added",t,r,n)}removeResourceBundle(t,r){this.hasResourceBundle(t,r)&&delete this.data[t][r],this.removeNamespaces(r),this.emit("removed",t,r)}hasResourceBundle(t,r){return this.getResource(t,r)!==void 0}getResourceBundle(t,r){return r||(r=this.options.defaultNS),this.getResource(t,r)}getDataByLanguage(t){return this.data[t]}hasLanguageSomeTranslations(t){const r=this.getDataByLanguage(t);return!!(r&&Object.keys(r)||[]).find(a=>r[a]&&Object.keys(r[a]).length>0)}toJSON(){return this.data}}var $ce={processors:{},addPostProcessor(e){this.processors[e.name]=e},handle(e,t,r,n,a){return e.forEach(i=>{var o;t=((o=this.processors[i])==null?void 0:o.process(t,r,n,a))??t}),t}};const tK={},rK=e=>!ur(e)&&typeof e!="boolean"&&typeof e!="number";class Sv extends oS{constructor(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};super(),Jlt(["resourceStore","languageUtils","pluralResolver","interpolator","backendConnector","i18nFormat","utils"],t,this),this.options=r,this.options.keySeparator===void 0&&(this.options.keySeparator="."),this.logger=os.create("translator")}changeLanguage(t){t&&(this.language=t)}exists(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{interpolation:{}};if(t==null)return!1;const n=this.resolve(t,r);return(n==null?void 0:n.res)!==void 0}extractFromKey(t,r){let n=r.nsSeparator!==void 0?r.nsSeparator:this.options.nsSeparator;n===void 0&&(n=":");const a=r.keySeparator!==void 0?r.keySeparator:this.options.keySeparator;let i=r.ns||this.options.defaultNS||[];const o=n&&t.indexOf(n)>-1,s=!this.options.userDefinedKeySeparator&&!r.keySeparator&&!this.options.userDefinedNsSeparator&&!r.nsSeparator&&!iut(t,n,a);if(o&&!s){const l=t.match(this.interpolator.nestingRegexp);if(l&&l.length>0)return{key:t,namespaces:ur(i)?[i]:i};const u=t.split(n);(n!==a||n===a&&this.options.ns.indexOf(u[0])>-1)&&(i=u.shift()),t=u.join(a)}return{key:t,namespaces:ur(i)?[i]:i}}translate(t,r,n){if(typeof r!="object"&&this.options.overloadTranslationOptionHandler&&(r=this.options.overloadTranslationOptionHandler(arguments)),typeof r=="object"&&(r={...r}),r||(r={}),t==null)return"";Array.isArray(t)||(t=[String(t)]);const a=r.returnDetails!==void 0?r.returnDetails:this.options.returnDetails,i=r.keySeparator!==void 0?r.keySeparator:this.options.keySeparator,{key:o,namespaces:s}=this.extractFromKey(t[t.length-1],r),l=s[s.length-1],u=r.lng||this.language,d=r.appendNamespaceToCIMode||this.options.appendNamespaceToCIMode;if((u==null?void 0:u.toLowerCase())==="cimode"){if(d){const j=r.nsSeparator||this.options.nsSeparator;return a?{res:`${l}${j}${o}`,usedKey:o,exactUsedKey:o,usedLng:u,usedNS:l,usedParams:this.getUsedParamsDetails(r)}:`${l}${j}${o}`}return a?{res:o,usedKey:o,exactUsedKey:o,usedLng:u,usedNS:l,usedParams:this.getUsedParamsDetails(r)}:o}const p=this.resolve(t,r);let f=p==null?void 0:p.res;const m=(p==null?void 0:p.usedKey)||o,b=(p==null?void 0:p.exactUsedKey)||o,y=["[object Number]","[object Function]","[object RegExp]"],S=r.joinArrays!==void 0?r.joinArrays:this.options.joinArrays,w=!this.i18nFormat||this.i18nFormat.handleAsObject,E=r.count!==void 0&&!ur(r.count),k=Sv.hasDefaultValue(r),_=E?this.pluralResolver.getSuffix(u,r.count,r):"",R=r.ordinal&&E?this.pluralResolver.getSuffix(u,r.count,{ordinal:!1}):"",C=E&&!r.ordinal&&r.count===0,F=C&&r[`defaultValue${this.options.pluralSeparator}zero`]||r[`defaultValue${_}`]||r[`defaultValue${R}`]||r.defaultValue;let L=f;w&&!f&&k&&(L=F);const I=rK(L),M=Object.prototype.toString.apply(L);if(w&&L&&I&&y.indexOf(M)<0&&!(ur(S)&&Array.isArray(L))){if(!r.returnObjects&&!this.options.returnObjects){this.options.returnedObjectHandler||this.logger.warn("accessing an object - but returnObjects options is not enabled!");const j=this.options.returnedObjectHandler?this.options.returnedObjectHandler(m,L,{...r,ns:s}):`key '${o} (${this.language})' returned an object instead of string.`;return a?(p.res=j,p.usedParams=this.getUsedParamsDetails(r),p):j}if(i){const j=Array.isArray(L),Y=j?[]:{},W=j?b:m;for(const $ in L)if(Object.prototype.hasOwnProperty.call(L,$)){const X=`${W}${i}${$}`;k&&!f?Y[$]=this.translate(X,{...r,defaultValue:rK(F)?F[$]:void 0,joinArrays:!1,ns:s}):Y[$]=this.translate(X,{...r,joinArrays:!1,ns:s}),Y[$]===X&&(Y[$]=L[$])}f=Y}}else if(w&&ur(S)&&Array.isArray(f))f=f.join(S),f&&(f=this.extendTranslation(f,t,r,n));else{let j=!1,Y=!1;!this.isValidLookup(f)&&k&&(j=!0,f=F),this.isValidLookup(f)||(Y=!0,f=o);const $=(r.missingKeyNoValueFallbackToKey||this.options.missingKeyNoValueFallbackToKey)&&Y?void 0:f,X=k&&F!==f&&this.options.updateMissing;if(Y||j||X){if(this.logger.log(X?"updateKey":"missingKey",u,l,o,X?F:f),i){const ee=this.resolve(o,{...r,keySeparator:!1});ee&&ee.res&&this.logger.warn("Seems the loaded translations were in flat JSON format instead of nested. Either set keySeparator: false on init or make sure your translations are published in nested format.")}let z=[];const H=this.languageUtils.getFallbackCodes(this.options.fallbackLng,r.lng||this.language);if(this.options.saveMissingTo==="fallback"&&H&&H[0])for(let ee=0;ee<H.length;ee++)z.push(H[ee]);else this.options.saveMissingTo==="all"?z=this.languageUtils.toResolveHierarchy(r.lng||this.language):z.push(r.lng||this.language);const B=(ee,J,K)=>{var P;const G=k&&K!==f?K:$;this.options.missingKeyHandler?this.options.missingKeyHandler(ee,l,J,G,X,r):(P=this.backendConnector)!=null&&P.saveMissing&&this.backendConnector.saveMissing(ee,l,J,G,X,r),this.emit("missingKey",ee,l,J,f)};this.options.saveMissing&&(this.options.saveMissingPlurals&&E?z.forEach(ee=>{const J=this.pluralResolver.getSuffixes(ee,r);C&&r[`defaultValue${this.options.pluralSeparator}zero`]&&J.indexOf(`${this.options.pluralSeparator}zero`)<0&&J.push(`${this.options.pluralSeparator}zero`),J.forEach(K=>{B([ee],o+K,r[`defaultValue${K}`]||F)})}):B(z,o,F))}f=this.extendTranslation(f,t,r,p,n),Y&&f===o&&this.options.appendNamespaceToMissingKey&&(f=`${l}:${o}`),(Y||j)&&this.options.parseMissingKeyHandler&&(f=this.options.parseMissingKeyHandler(this.options.appendNamespaceToMissingKey?`${l}:${o}`:o,j?f:void 0))}return a?(p.res=f,p.usedParams=this.getUsedParamsDetails(r),p):f}extendTranslation(t,r,n,a,i){var u,d;var o=this;if((u=this.i18nFormat)!=null&&u.parse)t=this.i18nFormat.parse(t,{...this.options.interpolation.defaultVariables,...n},n.lng||this.language||a.usedLng,a.usedNS,a.usedKey,{resolved:a});else if(!n.skipInterpolation){n.interpolation&&this.interpolator.init({...n,interpolation:{...this.options.interpolation,...n.interpolation}});const p=ur(t)&&(((d=n==null?void 0:n.interpolation)==null?void 0:d.skipOnVariables)!==void 0?n.interpolation.skipOnVariables:this.options.interpolation.skipOnVariables);let f;if(p){const b=t.match(this.interpolator.nestingRegexp);f=b&&b.length}let m=n.replace&&!ur(n.replace)?n.replace:n;if(this.options.interpolation.defaultVariables&&(m={...this.options.interpolation.defaultVariables,...m}),t=this.interpolator.interpolate(t,m,n.lng||this.language||a.usedLng,n),p){const b=t.match(this.interpolator.nestingRegexp),y=b&&b.length;f<y&&(n.nest=!1)}!n.lng&&a&&a.res&&(n.lng=this.language||a.usedLng),n.nest!==!1&&(t=this.interpolator.nest(t,function(){for(var b=arguments.length,y=new Array(b),S=0;S<b;S++)y[S]=arguments[S];return(i==null?void 0:i[0])===y[0]&&!n.context?(o.logger.warn(`It seems you are nesting recursively key: ${y[0]} in key: ${r[0]}`),null):o.translate(...y,r)},n)),n.interpolation&&this.interpolator.reset()}const s=n.postProcess||this.options.postProcess,l=ur(s)?[s]:s;return t!=null&&(l!=null&&l.length)&&n.applyPostProcessor!==!1&&(t=$ce.handle(l,t,r,this.options&&this.options.postProcessPassResolved?{i18nResolved:{...a,usedParams:this.getUsedParamsDetails(n)},...n}:n,this)),t}resolve(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n,a,i,o,s;return ur(t)&&(t=[t]),t.forEach(l=>{if(this.isValidLookup(n))return;const u=this.extractFromKey(l,r),d=u.key;a=d;let p=u.namespaces;this.options.fallbackNS&&(p=p.concat(this.options.fallbackNS));const f=r.count!==void 0&&!ur(r.count),m=f&&!r.ordinal&&r.count===0,b=r.context!==void 0&&(ur(r.context)||typeof r.context=="number")&&r.context!=="",y=r.lngs?r.lngs:this.languageUtils.toResolveHierarchy(r.lng||this.language,r.fallbackLng);p.forEach(S=>{var w,E;this.isValidLookup(n)||(s=S,!tK[`${y[0]}-${S}`]&&((w=this.utils)!=null&&w.hasLoadedNamespace)&&!((E=this.utils)!=null&&E.hasLoadedNamespace(s))&&(tK[`${y[0]}-${S}`]=!0,this.logger.warn(`key "${a}" for languages "${y.join(", ")}" won't get resolved as namespace "${s}" was not yet loaded`,"This means something IS WRONG in your setup. You access the t function before i18next.init / i18next.loadNamespace / i18next.changeLanguage was done. Wait for the callback or Promise to resolve before accessing it!!!")),y.forEach(k=>{var C;if(this.isValidLookup(n))return;o=k;const _=[d];if((C=this.i18nFormat)!=null&&C.addLookupKeys)this.i18nFormat.addLookupKeys(_,d,k,S,r);else{let F;f&&(F=this.pluralResolver.getSuffix(k,r.count,r));const L=`${this.options.pluralSeparator}zero`,I=`${this.options.pluralSeparator}ordinal${this.options.pluralSeparator}`;if(f&&(_.push(d+F),r.ordinal&&F.indexOf(I)===0&&_.push(d+F.replace(I,this.options.pluralSeparator)),m&&_.push(d+L)),b){const M=`${d}${this.options.contextSeparator}${r.context}`;_.push(M),f&&(_.push(M+F),r.ordinal&&F.indexOf(I)===0&&_.push(M+F.replace(I,this.options.pluralSeparator)),m&&_.push(M+L))}}let R;for(;R=_.pop();)this.isValidLookup(n)||(i=R,n=this.getResource(k,S,R,r))}))})}),{res:n,usedKey:a,exactUsedKey:i,usedLng:o,usedNS:s}}isValidLookup(t){return t!==void 0&&!(!this.options.returnNull&&t===null)&&!(!this.options.returnEmptyString&&t==="")}getResource(t,r,n){var i;let a=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{};return(i=this.i18nFormat)!=null&&i.getResource?this.i18nFormat.getResource(t,r,n,a):this.resourceStore.getResource(t,r,n,a)}getUsedParamsDetails(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};const r=["defaultValue","ordinal","context","replace","lng","lngs","fallbackLng","ns","keySeparator","nsSeparator","returnObjects","returnDetails","joinArrays","postProcess","interpolation"],n=t.replace&&!ur(t.replace);let a=n?t.replace:t;if(n&&typeof t.count<"u"&&(a.count=t.count),this.options.interpolation.defaultVariables&&(a={...this.options.interpolation.defaultVariables,...a}),!n){a={...a};for(const i of r)delete a[i]}return a}static hasDefaultValue(t){const r="defaultValue";for(const n in t)if(Object.prototype.hasOwnProperty.call(t,n)&&r===n.substring(0,r.length)&&t[n]!==void 0)return!0;return!1}}class nK{constructor(t){this.options=t,this.supportedLngs=this.options.supportedLngs||!1,this.logger=os.create("languageUtils")}getScriptPartFromCode(t){if(t=yv(t),!t||t.indexOf("-")<0)return null;const r=t.split("-");return r.length===2||(r.pop(),r[r.length-1].toLowerCase()==="x")?null:this.formatLanguageCode(r.join("-"))}getLanguagePartFromCode(t){if(t=yv(t),!t||t.indexOf("-")<0)return t;const r=t.split("-");return this.formatLanguageCode(r[0])}formatLanguageCode(t){if(ur(t)&&t.indexOf("-")>-1){let r;try{r=Intl.getCanonicalLocales(t)[0]}catch{}return r&&this.options.lowerCaseLng&&(r=r.toLowerCase()),r||(this.options.lowerCaseLng?t.toLowerCase():t)}return this.options.cleanCode||this.options.lowerCaseLng?t.toLowerCase():t}isSupportedCode(t){return(this.options.load==="languageOnly"||this.options.nonExplicitSupportedLngs)&&(t=this.getLanguagePartFromCode(t)),!this.supportedLngs||!this.supportedLngs.length||this.supportedLngs.indexOf(t)>-1}getBestMatchFromCodes(t){if(!t)return null;let r;return t.forEach(n=>{if(r)return;const a=this.formatLanguageCode(n);(!this.options.supportedLngs||this.isSupportedCode(a))&&(r=a)}),!r&&this.options.supportedLngs&&t.forEach(n=>{if(r)return;const a=this.getLanguagePartFromCode(n);if(this.isSupportedCode(a))return r=a;r=this.options.supportedLngs.find(i=>{if(i===a)return i;if(!(i.indexOf("-")<0&&a.indexOf("-")<0)&&(i.indexOf("-")>0&&a.indexOf("-")<0&&i.substring(0,i.indexOf("-"))===a||i.indexOf(a)===0&&a.length>1))return i})}),r||(r=this.getFallbackCodes(this.options.fallbackLng)[0]),r}getFallbackCodes(t,r){if(!t)return[];if(typeof t=="function"&&(t=t(r)),ur(t)&&(t=[t]),Array.isArray(t))return t;if(!r)return t.default||[];let n=t[r];return n||(n=t[this.getScriptPartFromCode(r)]),n||(n=t[this.formatLanguageCode(r)]),n||(n=t[this.getLanguagePartFromCode(r)]),n||(n=t.default),n||[]}toResolveHierarchy(t,r){const n=this.getFallbackCodes(r||this.options.fallbackLng||[],t),a=[],i=o=>{o&&(this.isSupportedCode(o)?a.push(o):this.logger.warn(`rejecting language code not found in supportedLngs: ${o}`))};return ur(t)&&(t.indexOf("-")>-1||t.indexOf("_")>-1)?(this.options.load!=="languageOnly"&&i(this.formatLanguageCode(t)),this.options.load!=="languageOnly"&&this.options.load!=="currentOnly"&&i(this.getScriptPartFromCode(t)),this.options.load!=="currentOnly"&&i(this.getLanguagePartFromCode(t))):ur(t)&&i(this.formatLanguageCode(t)),n.forEach(o=>{a.indexOf(o)<0&&i(this.formatLanguageCode(o))}),a}}const aK={zero:0,one:1,two:2,few:3,many:4,other:5},iK={select:e=>e===1?"one":"other",resolvedOptions:()=>({pluralCategories:["one","other"]})};class sut{constructor(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};this.languageUtils=t,this.options=r,this.logger=os.create("pluralResolver"),this.pluralRulesCache={}}addRule(t,r){this.rules[t]=r}clearCache(){this.pluralRulesCache={}}getRule(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};const n=yv(t==="dev"?"en":t),a=r.ordinal?"ordinal":"cardinal",i=JSON.stringify({cleanedCode:n,type:a});if(i in this.pluralRulesCache)return this.pluralRulesCache[i];let o;try{o=new Intl.PluralRules(n,{type:a})}catch{if(!Intl)return this.logger.error("No Intl support, please use an Intl polyfill!"),iK;if(!t.match(/-|_/))return iK;const l=this.languageUtils.getLanguagePartFromCode(t);o=this.getRule(l,r)}return this.pluralRulesCache[i]=o,o}needsPlural(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=this.getRule(t,r);return n||(n=this.getRule("dev",r)),(n==null?void 0:n.resolvedOptions().pluralCategories.length)>1}getPluralFormsOfKey(t,r){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{};return this.getSuffixes(t,n).map(a=>`${r}${a}`)}getSuffixes(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=this.getRule(t,r);return n||(n=this.getRule("dev",r)),n?n.resolvedOptions().pluralCategories.sort((a,i)=>aK[a]-aK[i]).map(a=>`${this.options.prepend}${r.ordinal?`ordinal${this.options.prepend}`:""}${a}`):[]}getSuffix(t,r){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{};const a=this.getRule(t,n);return a?`${this.options.prepend}${n.ordinal?`ordinal${this.options.prepend}`:""}${a.select(r)}`:(this.logger.warn(`no plural rule found for: ${t}`),this.getSuffix("dev",r,n))}}const oK=function(e,t,r){let n=arguments.length>3&&arguments[3]!==void 0?arguments[3]:".",a=arguments.length>4&&arguments[4]!==void 0?arguments[4]:!0,i=Qlt(e,t,r);return!i&&a&&ur(r)&&(i=rD(e,r,n),i===void 0&&(i=rD(t,r,n))),i},S_=e=>e.replace(/\$/g,"$$$$");class lut{constructor(){var r;let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};this.logger=os.create("interpolator"),this.options=t,this.format=((r=t==null?void 0:t.interpolation)==null?void 0:r.format)||(n=>n),this.init(t)}init(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};t.interpolation||(t.interpolation={escapeValue:!0});const{escape:r,escapeValue:n,useRawValueToEscape:a,prefix:i,prefixEscaped:o,suffix:s,suffixEscaped:l,formatSeparator:u,unescapeSuffix:d,unescapePrefix:p,nestingPrefix:f,nestingPrefixEscaped:m,nestingSuffix:b,nestingSuffixEscaped:y,nestingOptionsSeparator:S,maxReplaces:w,alwaysFormat:E}=t.interpolation;this.escape=r!==void 0?r:tut,this.escapeValue=n!==void 0?n:!0,this.useRawValueToEscape=a!==void 0?a:!1,this.prefix=i?sh(i):o||"{{",this.suffix=s?sh(s):l||"}}",this.formatSeparator=u||",",this.unescapePrefix=d?"":p||"-",this.unescapeSuffix=this.unescapePrefix?"":d||"",this.nestingPrefix=f?sh(f):m||sh("$t("),this.nestingSuffix=b?sh(b):y||sh(")"),this.nestingOptionsSeparator=S||",",this.maxReplaces=w||1e3,this.alwaysFormat=E!==void 0?E:!1,this.resetRegExp()}reset(){this.options&&this.init(this.options)}resetRegExp(){const t=(r,n)=>(r==null?void 0:r.source)===n?(r.lastIndex=0,r):new RegExp(n,"g");this.regexp=t(this.regexp,`${this.prefix}(.+?)${this.suffix}`),this.regexpUnescape=t(this.regexpUnescape,`${this.prefix}${this.unescapePrefix}(.+?)${this.unescapeSuffix}${this.suffix}`),this.nestingRegexp=t(this.nestingRegexp,`${this.nestingPrefix}(.+?)${this.nestingSuffix}`)}interpolate(t,r,n,a){var m;let i,o,s;const l=this.options&&this.options.interpolation&&this.options.interpolation.defaultVariables||{},u=b=>{if(b.indexOf(this.formatSeparator)<0){const E=oK(r,l,b,this.options.keySeparator,this.options.ignoreJSONStructure);return this.alwaysFormat?this.format(E,void 0,n,{...a,...r,interpolationkey:b}):E}const y=b.split(this.formatSeparator),S=y.shift().trim(),w=y.join(this.formatSeparator).trim();return this.format(oK(r,l,S,this.options.keySeparator,this.options.ignoreJSONStructure),w,n,{...a,...r,interpolationkey:S})};this.resetRegExp();const d=(a==null?void 0:a.missingInterpolationHandler)||this.options.missingInterpolationHandler,p=((m=a==null?void 0:a.interpolation)==null?void 0:m.skipOnVariables)!==void 0?a.interpolation.skipOnVariables:this.options.interpolation.skipOnVariables;return[{regex:this.regexpUnescape,safeValue:b=>S_(b)},{regex:this.regexp,safeValue:b=>this.escapeValue?S_(this.escape(b)):S_(b)}].forEach(b=>{for(s=0;i=b.regex.exec(t);){const y=i[1].trim();if(o=u(y),o===void 0)if(typeof d=="function"){const w=d(t,i,a);o=ur(w)?w:""}else if(a&&Object.prototype.hasOwnProperty.call(a,y))o="";else if(p){o=i[0];continue}else this.logger.warn(`missed to pass in variable ${y} for interpolating ${t}`),o="";else!ur(o)&&!this.useRawValueToEscape&&(o=JV(o));const S=b.safeValue(o);if(t=t.replace(i[0],S),p?(b.regex.lastIndex+=o.length,b.regex.lastIndex-=i[0].length):b.regex.lastIndex=0,s++,s>=this.maxReplaces)break}}),t}nest(t,r){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},a,i,o;const s=(l,u)=>{const d=this.nestingOptionsSeparator;if(l.indexOf(d)<0)return l;const p=l.split(new RegExp(`${d}[ ]*{`));let f=`{${p[1]}`;l=p[0],f=this.interpolate(f,o);const m=f.match(/'/g),b=f.match(/"/g);(((m==null?void 0:m.length)??0)%2===0&&!b||b.length%2!==0)&&(f=f.replace(/'/g,'"'));try{o=JSON.parse(f),u&&(o={...u,...o})}catch(y){return this.logger.warn(`failed parsing options string in nesting for key ${l}`,y),`${l}${d}${f}`}return o.defaultValue&&o.defaultValue.indexOf(this.prefix)>-1&&delete o.defaultValue,l};for(;a=this.nestingRegexp.exec(t);){let l=[];o={...n},o=o.replace&&!ur(o.replace)?o.replace:o,o.applyPostProcessor=!1,delete o.defaultValue;let u=!1;if(a[0].indexOf(this.formatSeparator)!==-1&&!/{.*}/.test(a[1])){const d=a[1].split(this.formatSeparator).map(p=>p.trim());a[1]=d.shift(),l=d,u=!0}if(i=r(s.call(this,a[1].trim(),o),o),i&&a[0]===t&&!ur(i))return i;ur(i)||(i=JV(i)),i||(this.logger.warn(`missed to resolve ${a[1]} for nesting ${t}`),i=""),u&&(i=l.reduce((d,p)=>this.format(d,p,n.lng,{...n,interpolationkey:a[1].trim()}),i.trim())),t=t.replace(a[0],i),this.regexp.lastIndex=0}return t}}const uut=e=>{let t=e.toLowerCase().trim();const r={};if(e.indexOf("(")>-1){const n=e.split("(");t=n[0].toLowerCase().trim();const a=n[1].substring(0,n[1].length-1);t==="currency"&&a.indexOf(":")<0?r.currency||(r.currency=a.trim()):t==="relativetime"&&a.indexOf(":")<0?r.range||(r.range=a.trim()):a.split(";").forEach(o=>{if(o){const[s,...l]=o.split(":"),u=l.join(":").trim().replace(/^'+|'+$/g,""),d=s.trim();r[d]||(r[d]=u),u==="false"&&(r[d]=!1),u==="true"&&(r[d]=!0),isNaN(u)||(r[d]=parseInt(u,10))}})}return{formatName:t,formatOptions:r}},lh=e=>{const t={};return(r,n,a)=>{let i=a;a&&a.interpolationkey&&a.formatParams&&a.formatParams[a.interpolationkey]&&a[a.interpolationkey]&&(i={...i,[a.interpolationkey]:void 0});const o=n+JSON.stringify(i);let s=t[o];return s||(s=e(yv(n),a),t[o]=s),s(r)}};class cut{constructor(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};this.logger=os.create("formatter"),this.options=t,this.formats={number:lh((r,n)=>{const a=new Intl.NumberFormat(r,{...n});return i=>a.format(i)}),currency:lh((r,n)=>{const a=new Intl.NumberFormat(r,{...n,style:"currency"});return i=>a.format(i)}),datetime:lh((r,n)=>{const a=new Intl.DateTimeFormat(r,{...n});return i=>a.format(i)}),relativetime:lh((r,n)=>{const a=new Intl.RelativeTimeFormat(r,{...n});return i=>a.format(i,n.range||"day")}),list:lh((r,n)=>{const a=new Intl.ListFormat(r,{...n});return i=>a.format(i)})},this.init(t)}init(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{interpolation:{}};this.formatSeparator=r.interpolation.formatSeparator||","}add(t,r){this.formats[t.toLowerCase().trim()]=r}addCached(t,r){this.formats[t.toLowerCase().trim()]=lh(r)}format(t,r,n){let a=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{};const i=r.split(this.formatSeparator);if(i.length>1&&i[0].indexOf("(")>1&&i[0].indexOf(")")<0&&i.find(s=>s.indexOf(")")>-1)){const s=i.findIndex(l=>l.indexOf(")")>-1);i[0]=[i[0],...i.splice(1,s)].join(this.formatSeparator)}return i.reduce((s,l)=>{var p;const{formatName:u,formatOptions:d}=uut(l);if(this.formats[u]){let f=s;try{const m=((p=a==null?void 0:a.formatParams)==null?void 0:p[a.interpolationkey])||{},b=m.locale||m.lng||a.locale||a.lng||n;f=this.formats[u](s,b,{...d,...a,...m})}catch(m){this.logger.warn(m)}return f}else this.logger.warn(`there was no format function for ${u}`);return s},t)}}const dut=(e,t)=>{e.pending[t]!==void 0&&(delete e.pending[t],e.pendingCount--)};class hut extends oS{constructor(t,r,n){var i,o;let a=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{};super(),this.backend=t,this.store=r,this.services=n,this.languageUtils=n.languageUtils,this.options=a,this.logger=os.create("backendConnector"),this.waitingReads=[],this.maxParallelReads=a.maxParallelReads||10,this.readingCalls=0,this.maxRetries=a.maxRetries>=0?a.maxRetries:5,this.retryTimeout=a.retryTimeout>=1?a.retryTimeout:350,this.state={},this.queue=[],(o=(i=this.backend)==null?void 0:i.init)==null||o.call(i,n,a.backend,a)}queueLoad(t,r,n,a){const i={},o={},s={},l={};return t.forEach(u=>{let d=!0;r.forEach(p=>{const f=`${u}|${p}`;!n.reload&&this.store.hasResourceBundle(u,p)?this.state[f]=2:this.state[f]<0||(this.state[f]===1?o[f]===void 0&&(o[f]=!0):(this.state[f]=1,d=!1,o[f]===void 0&&(o[f]=!0),i[f]===void 0&&(i[f]=!0),l[p]===void 0&&(l[p]=!0)))}),d||(s[u]=!0)}),(Object.keys(i).length||Object.keys(o).length)&&this.queue.push({pending:o,pendingCount:Object.keys(o).length,loaded:{},errors:[],callback:a}),{toLoad:Object.keys(i),pending:Object.keys(o),toLoadLanguages:Object.keys(s),toLoadNamespaces:Object.keys(l)}}loaded(t,r,n){const a=t.split("|"),i=a[0],o=a[1];r&&this.emit("failedLoading",i,o,r),!r&&n&&this.store.addResourceBundle(i,o,n,void 0,void 0,{skipCopy:!0}),this.state[t]=r?-1:2,r&&n&&(this.state[t]=0);const s={};this.queue.forEach(l=>{Zlt(l.loaded,[i],o),dut(l,t),r&&l.errors.push(r),l.pendingCount===0&&!l.done&&(Object.keys(l.loaded).forEach(u=>{s[u]||(s[u]={});const d=l.loaded[u];d.length&&d.forEach(p=>{s[u][p]===void 0&&(s[u][p]=!0)})}),l.done=!0,l.errors.length?l.callback(l.errors):l.callback())}),this.emit("loaded",s),this.queue=this.queue.filter(l=>!l.done)}read(t,r,n){let a=arguments.length>3&&arguments[3]!==void 0?arguments[3]:0,i=arguments.length>4&&arguments[4]!==void 0?arguments[4]:this.retryTimeout,o=arguments.length>5?arguments[5]:void 0;if(!t.length)return o(null,{});if(this.readingCalls>=this.maxParallelReads){this.waitingReads.push({lng:t,ns:r,fcName:n,tried:a,wait:i,callback:o});return}this.readingCalls++;const s=(u,d)=>{if(this.readingCalls--,this.waitingReads.length>0){const p=this.waitingReads.shift();this.read(p.lng,p.ns,p.fcName,p.tried,p.wait,p.callback)}if(u&&d&&a<this.maxRetries){setTimeout(()=>{this.read.call(this,t,r,n,a+1,i*2,o)},i);return}o(u,d)},l=this.backend[n].bind(this.backend);if(l.length===2){try{const u=l(t,r);u&&typeof u.then=="function"?u.then(d=>s(null,d)).catch(s):s(null,u)}catch(u){s(u)}return}return l(t,r,s)}prepareLoading(t,r){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},a=arguments.length>3?arguments[3]:void 0;if(!this.backend)return this.logger.warn("No backend was added via i18next.use. Will not load resources."),a&&a();ur(t)&&(t=this.languageUtils.toResolveHierarchy(t)),ur(r)&&(r=[r]);const i=this.queueLoad(t,r,n,a);if(!i.toLoad.length)return i.pending.length||a(),null;i.toLoad.forEach(o=>{this.loadOne(o)})}load(t,r,n){this.prepareLoading(t,r,{},n)}reload(t,r,n){this.prepareLoading(t,r,{reload:!0},n)}loadOne(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"";const n=t.split("|"),a=n[0],i=n[1];this.read(a,i,"read",void 0,void 0,(o,s)=>{o&&this.logger.warn(`${r}loading namespace ${i} for language ${a} failed`,o),!o&&s&&this.logger.log(`${r}loaded namespace ${i} for language ${a}`,s),this.loaded(t,o,s)})}saveMissing(t,r,n,a,i){var l,u,d,p,f;let o=arguments.length>5&&arguments[5]!==void 0?arguments[5]:{},s=arguments.length>6&&arguments[6]!==void 0?arguments[6]:()=>{};if((u=(l=this.services)==null?void 0:l.utils)!=null&&u.hasLoadedNamespace&&!((p=(d=this.services)==null?void 0:d.utils)!=null&&p.hasLoadedNamespace(r))){this.logger.warn(`did not save key "${n}" as the namespace "${r}" was not yet loaded`,"This means something IS WRONG in your setup. You access the t function before i18next.init / i18next.loadNamespace / i18next.changeLanguage was done. Wait for the callback or Promise to resolve before accessing it!!!");return}if(!(n==null||n==="")){if((f=this.backend)!=null&&f.create){const m={...o,isUpdate:i},b=this.backend.create.bind(this.backend);if(b.length<6)try{let y;b.length===5?y=b(t,r,n,a,m):y=b(t,r,n,a),y&&typeof y.then=="function"?y.then(S=>s(null,S)).catch(s):s(null,y)}catch(y){s(y)}else b(t,r,n,a,s,m)}!t||!t[0]||this.store.addResource(t[0],r,n,a)}}}const sK=()=>({debug:!1,initAsync:!0,ns:["translation"],defaultNS:["translation"],fallbackLng:["dev"],fallbackNS:!1,supportedLngs:!1,nonExplicitSupportedLngs:!1,load:"all",preload:!1,simplifyPluralSuffix:!0,keySeparator:".",nsSeparator:":",pluralSeparator:"_",contextSeparator:"_",partialBundledLanguages:!1,saveMissing:!1,updateMissing:!1,saveMissingTo:"fallback",saveMissingPlurals:!0,missingKeyHandler:!1,missingInterpolationHandler:!1,postProcess:!1,postProcessPassResolved:!1,returnNull:!1,returnEmptyString:!0,returnObjects:!1,joinArrays:!1,returnedObjectHandler:!1,parseMissingKeyHandler:!1,appendNamespaceToMissingKey:!1,appendNamespaceToCIMode:!1,overloadTranslationOptionHandler:e=>{let t={};if(typeof e[1]=="object"&&(t=e[1]),ur(e[1])&&(t.defaultValue=e[1]),ur(e[2])&&(t.tDescription=e[2]),typeof e[2]=="object"||typeof e[3]=="object"){const r=e[3]||e[2];Object.keys(r).forEach(n=>{t[n]=r[n]})}return t},interpolation:{escapeValue:!0,format:e=>e,prefix:"{{",suffix:"}}",formatSeparator:",",unescapePrefix:"-",nestingPrefix:"$t(",nestingSuffix:")",nestingOptionsSeparator:",",maxReplaces:1e3,skipOnVariables:!0}}),lK=e=>{var t,r;return ur(e.ns)&&(e.ns=[e.ns]),ur(e.fallbackLng)&&(e.fallbackLng=[e.fallbackLng]),ur(e.fallbackNS)&&(e.fallbackNS=[e.fallbackNS]),((r=(t=e.supportedLngs)==null?void 0:t.indexOf)==null?void 0:r.call(t,"cimode"))<0&&(e.supportedLngs=e.supportedLngs.concat(["cimode"])),typeof e.initImmediate=="boolean"&&(e.initAsync=e.initImmediate),e},m1=()=>{},put=e=>{Object.getOwnPropertyNames(Object.getPrototypeOf(e)).forEach(r=>{typeof e[r]=="function"&&(e[r]=e[r].bind(e))})};class lg extends oS{constructor(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},r=arguments.length>1?arguments[1]:void 0;if(super(),this.options=lK(t),this.services={},this.logger=os,this.modules={external:[]},put(this),r&&!this.isInitialized&&!t.isClone){if(!this.options.initAsync)return this.init(t,r),this;setTimeout(()=>{this.init(t,r)},0)}}init(){var t=this;let r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},n=arguments.length>1?arguments[1]:void 0;this.isInitializing=!0,typeof r=="function"&&(n=r,r={}),r.defaultNS==null&&r.ns&&(ur(r.ns)?r.defaultNS=r.ns:r.ns.indexOf("translation")<0&&(r.defaultNS=r.ns[0]));const a=sK();this.options={...a,...this.options,...lK(r)},this.options.interpolation={...a.interpolation,...this.options.interpolation},r.keySeparator!==void 0&&(this.options.userDefinedKeySeparator=r.keySeparator),r.nsSeparator!==void 0&&(this.options.userDefinedNsSeparator=r.nsSeparator);const i=d=>d?typeof d=="function"?new d:d:null;if(!this.options.isClone){this.modules.logger?os.init(i(this.modules.logger),this.options):os.init(null,this.options);let d;this.modules.formatter?d=this.modules.formatter:d=cut;const p=new nK(this.options);this.store=new eK(this.options.resources,this.options);const f=this.services;f.logger=os,f.resourceStore=this.store,f.languageUtils=p,f.pluralResolver=new sut(p,{prepend:this.options.pluralSeparator,simplifyPluralSuffix:this.options.simplifyPluralSuffix}),d&&(!this.options.interpolation.format||this.options.interpolation.format===a.interpolation.format)&&(f.formatter=i(d),f.formatter.init(f,this.options),this.options.interpolation.format=f.formatter.format.bind(f.formatter)),f.interpolator=new lut(this.options),f.utils={hasLoadedNamespace:this.hasLoadedNamespace.bind(this)},f.backendConnector=new hut(i(this.modules.backend),f.resourceStore,f,this.options),f.backendConnector.on("*",function(m){for(var b=arguments.length,y=new Array(b>1?b-1:0),S=1;S<b;S++)y[S-1]=arguments[S];t.emit(m,...y)}),this.modules.languageDetector&&(f.languageDetector=i(this.modules.languageDetector),f.languageDetector.init&&f.languageDetector.init(f,this.options.detection,this.options)),this.modules.i18nFormat&&(f.i18nFormat=i(this.modules.i18nFormat),f.i18nFormat.init&&f.i18nFormat.init(this)),this.translator=new Sv(this.services,this.options),this.translator.on("*",function(m){for(var b=arguments.length,y=new Array(b>1?b-1:0),S=1;S<b;S++)y[S-1]=arguments[S];t.emit(m,...y)}),this.modules.external.forEach(m=>{m.init&&m.init(this)})}if(this.format=this.options.interpolation.format,n||(n=m1),this.options.fallbackLng&&!this.services.languageDetector&&!this.options.lng){const d=this.services.languageUtils.getFallbackCodes(this.options.fallbackLng);d.length>0&&d[0]!=="dev"&&(this.options.lng=d[0])}!this.services.languageDetector&&!this.options.lng&&this.logger.warn("init: no languageDetector is used and no lng is defined"),["getResource","hasResourceBundle","getResourceBundle","getDataByLanguage"].forEach(d=>{this[d]=function(){return t.store[d](...arguments)}}),["addResource","addResources","addResourceBundle","removeResourceBundle"].forEach(d=>{this[d]=function(){return t.store[d](...arguments),t}});const l=tm(),u=()=>{const d=(p,f)=>{this.isInitializing=!1,this.isInitialized&&!this.initializedStoreOnce&&this.logger.warn("init: i18next is already initialized. You should call init just once!"),this.isInitialized=!0,this.options.isClone||this.logger.log("initialized",this.options),this.emit("initialized",this.options),l.resolve(f),n(p,f)};if(this.languages&&!this.isInitialized)return d(null,this.t.bind(this));this.changeLanguage(this.options.lng,d)};return this.options.resources||!this.options.initAsync?u():setTimeout(u,0),l}loadResources(t){var i,o;let n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:m1;const a=ur(t)?t:this.language;if(typeof t=="function"&&(n=t),!this.options.resources||this.options.partialBundledLanguages){if((a==null?void 0:a.toLowerCase())==="cimode"&&(!this.options.preload||this.options.preload.length===0))return n();const s=[],l=u=>{if(!u||u==="cimode")return;this.services.languageUtils.toResolveHierarchy(u).forEach(p=>{p!=="cimode"&&s.indexOf(p)<0&&s.push(p)})};a?l(a):this.services.languageUtils.getFallbackCodes(this.options.fallbackLng).forEach(d=>l(d)),(o=(i=this.options.preload)==null?void 0:i.forEach)==null||o.call(i,u=>l(u)),this.services.backendConnector.load(s,this.options.ns,u=>{!u&&!this.resolvedLanguage&&this.language&&this.setResolvedLanguage(this.language),n(u)})}else n(null)}reloadResources(t,r,n){const a=tm();return typeof t=="function"&&(n=t,t=void 0),typeof r=="function"&&(n=r,r=void 0),t||(t=this.languages),r||(r=this.options.ns),n||(n=m1),this.services.backendConnector.reload(t,r,i=>{a.resolve(),n(i)}),a}use(t){if(!t)throw new Error("You are passing an undefined module! Please check the object you are passing to i18next.use()");if(!t.type)throw new Error("You are passing a wrong module! Please check the object you are passing to i18next.use()");return t.type==="backend"&&(this.modules.backend=t),(t.type==="logger"||t.log&&t.warn&&t.error)&&(this.modules.logger=t),t.type==="languageDetector"&&(this.modules.languageDetector=t),t.type==="i18nFormat"&&(this.modules.i18nFormat=t),t.type==="postProcessor"&&$ce.addPostProcessor(t),t.type==="formatter"&&(this.modules.formatter=t),t.type==="3rdParty"&&this.modules.external.push(t),this}setResolvedLanguage(t){if(!(!t||!this.languages)&&!(["cimode","dev"].indexOf(t)>-1))for(let r=0;r<this.languages.length;r++){const n=this.languages[r];if(!(["cimode","dev"].indexOf(n)>-1)&&this.store.hasLanguageSomeTranslations(n)){this.resolvedLanguage=n;break}}}changeLanguage(t,r){var n=this;this.isLanguageChangingTo=t;const a=tm();this.emit("languageChanging",t);const i=l=>{this.language=l,this.languages=this.services.languageUtils.toResolveHierarchy(l),this.resolvedLanguage=void 0,this.setResolvedLanguage(l)},o=(l,u)=>{u?(i(u),this.translator.changeLanguage(u),this.isLanguageChangingTo=void 0,this.emit("languageChanged",u),this.logger.log("languageChanged",u)):this.isLanguageChangingTo=void 0,a.resolve(function(){return n.t(...arguments)}),r&&r(l,function(){return n.t(...arguments)})},s=l=>{var d,p;!t&&!l&&this.services.languageDetector&&(l=[]);const u=ur(l)?l:this.services.languageUtils.getBestMatchFromCodes(l);u&&(this.language||i(u),this.translator.language||this.translator.changeLanguage(u),(p=(d=this.services.languageDetector)==null?void 0:d.cacheUserLanguage)==null||p.call(d,u)),this.loadResources(u,f=>{o(f,u)})};return!t&&this.services.languageDetector&&!this.services.languageDetector.async?s(this.services.languageDetector.detect()):!t&&this.services.languageDetector&&this.services.languageDetector.async?this.services.languageDetector.detect.length===0?this.services.languageDetector.detect().then(s):this.services.languageDetector.detect(s):s(t),a}getFixedT(t,r,n){var a=this;const i=function(o,s){let l;if(typeof s!="object"){for(var u=arguments.length,d=new Array(u>2?u-2:0),p=2;p<u;p++)d[p-2]=arguments[p];l=a.options.overloadTranslationOptionHandler([o,s].concat(d))}else l={...s};l.lng=l.lng||i.lng,l.lngs=l.lngs||i.lngs,l.ns=l.ns||i.ns,l.keyPrefix!==""&&(l.keyPrefix=l.keyPrefix||n||i.keyPrefix);const f=a.options.keySeparator||".";let m;return l.keyPrefix&&Array.isArray(o)?m=o.map(b=>`${l.keyPrefix}${f}${b}`):m=l.keyPrefix?`${l.keyPrefix}${f}${o}`:o,a.t(m,l)};return ur(t)?i.lng=t:i.lngs=t,i.ns=r,i.keyPrefix=n,i}t(){var a;for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return(a=this.translator)==null?void 0:a.translate(...r)}exists(){var a;for(var t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return(a=this.translator)==null?void 0:a.exists(...r)}setDefaultNamespace(t){this.options.defaultNS=t}hasLoadedNamespace(t){let r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};if(!this.isInitialized)return this.logger.warn("hasLoadedNamespace: i18next was not initialized",this.languages),!1;if(!this.languages||!this.languages.length)return this.logger.warn("hasLoadedNamespace: i18n.languages were undefined or empty",this.languages),!1;const n=r.lng||this.resolvedLanguage||this.languages[0],a=this.options?this.options.fallbackLng:!1,i=this.languages[this.languages.length-1];if(n.toLowerCase()==="cimode")return!0;const o=(s,l)=>{const u=this.services.backendConnector.state[`${s}|${l}`];return u===-1||u===0||u===2};if(r.precheck){const s=r.precheck(this,o);if(s!==void 0)return s}return!!(this.hasResourceBundle(n,t)||!this.services.backendConnector.backend||this.options.resources&&!this.options.partialBundledLanguages||o(n,t)&&(!a||o(i,t)))}loadNamespaces(t,r){const n=tm();return this.options.ns?(ur(t)&&(t=[t]),t.forEach(a=>{this.options.ns.indexOf(a)<0&&this.options.ns.push(a)}),this.loadResources(a=>{n.resolve(),r&&r(a)}),n):(r&&r(),Promise.resolve())}loadLanguages(t,r){const n=tm();ur(t)&&(t=[t]);const a=this.options.preload||[],i=t.filter(o=>a.indexOf(o)<0&&this.services.languageUtils.isSupportedCode(o));return i.length?(this.options.preload=a.concat(i),this.loadResources(o=>{n.resolve(),r&&r(o)}),n):(r&&r(),Promise.resolve())}dir(t){var a,i;if(t||(t=this.resolvedLanguage||(((a=this.languages)==null?void 0:a.length)>0?this.languages[0]:this.language)),!t)return"rtl";const r=["ar","shu","sqr","ssh","xaa","yhd","yud","aao","abh","abv","acm","acq","acw","acx","acy","adf","ads","aeb","aec","afb","ajp","apc","apd","arb","arq","ars","ary","arz","auz","avl","ayh","ayl","ayn","ayp","bbz","pga","he","iw","ps","pbt","pbu","pst","prp","prd","ug","ur","ydd","yds","yih","ji","yi","hbo","men","xmn","fa","jpr","peo","pes","prs","dv","sam","ckb"],n=((i=this.services)==null?void 0:i.languageUtils)||new nK(sK());return r.indexOf(n.getLanguagePartFromCode(t))>-1||t.toLowerCase().indexOf("-arab")>1?"rtl":"ltr"}static createInstance(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},r=arguments.length>1?arguments[1]:void 0;return new lg(t,r)}cloneInstance(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:m1;const n=t.forkResourceStore;n&&delete t.forkResourceStore;const a={...this.options,...t,isClone:!0},i=new lg(a);if((t.debug!==void 0||t.prefix!==void 0)&&(i.logger=i.logger.clone(t)),["store","services","language"].forEach(s=>{i[s]=this[s]}),i.services={...this.services},i.services.utils={hasLoadedNamespace:i.hasLoadedNamespace.bind(i)},n){const s=Object.keys(this.store.data).reduce((l,u)=>(l[u]={...this.store.data[u]},Object.keys(l[u]).reduce((d,p)=>(d[p]={...l[u][p]},d),{})),{});i.store=new eK(s,a),i.services.resourceStore=i.store}return i.translator=new Sv(i.services,a),i.translator.on("*",function(s){for(var l=arguments.length,u=new Array(l>1?l-1:0),d=1;d<l;d++)u[d-1]=arguments[d];i.emit(s,...u)}),i.init(a,r),i.translator.options=a,i.translator.backendConnector.services.utils={hasLoadedNamespace:i.hasLoadedNamespace.bind(i)},i}toJSON(){return{options:this.options,store:this.store,language:this.language,languages:this.languages,resolvedLanguage:this.resolvedLanguage}}}const na=lg.createInstance();na.createInstance=lg.createInstance;na.createInstance;na.dir;na.init;na.loadResources;na.reloadResources;na.use;na.changeLanguage;na.getFixedT;na.t;na.exists;na.setDefaultNamespace;na.hasLoadedNamespace;na.loadNamespaces;na.loadLanguages;const fut=Object.freeze(Object.defineProperty({__proto__:null,default:na},Symbol.toStringTag,{value:"Module"})),mut={language:"Language",theme:"Theme",light:"Light",dark:"Dark",system:"System"},gut={documents:"Documents",knowledgeGraph:"Knowledge Graph",retrieval:"Retrieval",api:"API",projectRepository:"Project Repository",logout:"Logout",themeToggle:{switchToLight:"Switch to light theme",switchToDark:"Switch to dark theme"}},but={description:"Please enter your account and password to log in to the system",username:"Username",usernamePlaceholder:"Please input a username",password:"Password",passwordPlaceholder:"Please input a password",loginButton:"Login",loggingIn:"Logging in...",successMessage:"Login succeeded",errorEmptyFields:"Please enter your username and password",errorInvalidCredentials:"Login failed, please check username and password",authDisabled:"Authentication is disabled. Using login free mode.",guestMode:"Login Free"},yut={cancel:"Cancel",save:"Save",saving:"Saving...",saveFailed:"Save failed"},vut={clearDocuments:{button:"Clear",tooltip:"Clear documents",title:"Clear Documents",description:"This will remove all documents from the system",warning:"WARNING: This action will permanently delete all documents and cannot be undone!",confirm:"Do you really want to clear all documents?",confirmPrompt:"Type 'yes' to confirm this action",confirmPlaceholder:"Type yes to confirm",clearCache:"Clear LLM cache",confirmButton:"YES",clearing:"Clearing...",timeout:"Clear operation timed out, please try again",success:"Documents cleared successfully",cacheCleared:"Cache cleared successfully",cacheClearFailed:`Failed to clear cache:
{{error}}`,failed:`Clear Documents Failed:
{{message}}`,error:`Clear Documents Failed:
{{error}}`},deleteDocuments:{button:"Delete",tooltip:"Delete selected documents",title:"Delete Documents",description:"This will permanently delete the selected documents from the system",warning:"WARNING: This action will permanently delete the selected documents and cannot be undone!",confirm:"Do you really want to delete {{count}} selected document(s)?",confirmPrompt:"Type 'yes' to confirm this action",confirmPlaceholder:"Type yes to confirm",confirmButton:"YES",deleteFileOption:"Also delete uploaded files",deleteFileTooltip:"Check this option to also delete the corresponding uploaded files on the server",deleteLLMCacheOption:"Also delete extracted LLM cache",success:"Document deletion pipeline started successfully",failed:`Delete Documents Failed:
{{message}}`,error:`Delete Documents Failed:
{{error}}`,busy:"Pipeline is busy, please try again later",notAllowed:"No permission to perform this operation"},selectDocuments:{selectCurrentPage:"Select Current Page ({{count}})",deselectAll:"Deselect All ({{count}})"},uploadDocuments:{button:"Upload",tooltip:"Upload documents",title:"Upload Documents",description:"Drag and drop your documents here or click to browse.",single:{uploading:"Uploading {{name}}: {{percent}}%",success:`Upload Success:
{{name}} uploaded successfully`,failed:`Upload Failed:
{{name}}
{{message}}`,error:`Upload Failed:
{{name}}
{{error}}`},batch:{uploading:"Uploading files...",success:"Files uploaded successfully",error:"Some files failed to upload"},generalError:`Upload Failed
{{error}}`,fileTypes:"Supported types: TXT, MD, DOCX, PDF, PPTX, XLSX, RTF, ODT, EPUB, HTML, HTM, TEX, JSON, XML, YAML, YML, CSV, LOG, CONF, INI, PROPERTIES, SQL, BAT, SH, C, CPP, PY, JAVA, JS, TS, SWIFT, GO, RB, PHP, CSS, SCSS, LESS",fileUploader:{singleFileLimit:"Cannot upload more than 1 file at a time",maxFilesLimit:"Cannot upload more than {{count}} files",fileRejected:"File {{name}} was rejected",unsupportedType:"Unsupported file type",fileTooLarge:"File too large, maximum size is {{maxSize}}",dropHere:"Drop the files here",dragAndDrop:"Drag and drop files here, or click to select files",removeFile:"Remove file",uploadDescription:"You can upload {{isMultiple ? 'multiple' : count}} files (up to {{maxSize}} each)",duplicateFile:"File name already exists in server cache"}},documentManager:{title:"Document Management",scanButton:"Scan/Retry",scanTooltip:"Scan and process documents in input folder, and also reprocess all failed documents",refreshTooltip:"Reset document list",pipelineStatusButton:"Pipeline",pipelineStatusTooltip:"View document processing pipeline status",uploadedTitle:"Uploaded Documents",uploadedDescription:"List of uploaded documents and their statuses.",emptyTitle:"No Documents",emptyDescription:"There are no uploaded documents yet.",columns:{id:"ID",fileName:"File Name",summary:"Summary",status:"Status",length:"Length",chunks:"Chunks",created:"Created",updated:"Updated",metadata:"Metadata",select:"Select"},status:{all:"All",completed:"Completed",preprocessed:"Preprocessed",processing:"Processing",pending:"Pending",failed:"Failed"},errors:{loadFailed:`Failed to load documents
{{error}}`,scanFailed:`Failed to scan documents
{{error}}`,scanProgressFailed:`Failed to get scan progress
{{error}}`},fileNameLabel:"File Name",showButton:"Show",hideButton:"Hide",showFileNameTooltip:"Show file name",hideFileNameTooltip:"Hide file name"},pipelineStatus:{title:"Pipeline Status",busy:"Pipeline Busy",requestPending:"Request Pending",cancellationRequested:"Cancellation Requested",jobName:"Job Name",startTime:"Start Time",progress:"Progress",unit:"Batch",pipelineMessages:"Pipeline Messages",cancelButton:"Cancel",cancelTooltip:"Cancel pipeline processing",cancelConfirmTitle:"Confirm Pipeline Cancellation",cancelConfirmDescription:"This will interrupt the ongoing pipeline processing. Are you sure you want to continue?",cancelConfirmButton:"Confirm Cancellation",cancelInProgress:"Cancellation in progress...",pipelineNotRunning:"Pipeline not running",cancelSuccess:"Pipeline cancellation requested",cancelFailed:`Failed to cancel pipeline
{{error}}`,cancelNotBusy:"Pipeline is not running, no need to cancel",errors:{fetchFailed:`Failed to fetch pipeline status
{{error}}`}}},Sut={dataIsTruncated:"Graph data is truncated to Max Nodes",statusDialog:{title:"LightRAG Server Settings",description:"View current system status and connection information"},legend:"Legend",nodeTypes:{person:"Person",category:"Category",geo:"Geographic",location:"Location",organization:"Organization",event:"Event",equipment:"Equipment",weapon:"Weapon",animal:"Animal",unknown:"Unknown",object:"Object",group:"Group",technology:"Technology",product:"Product",document:"Document",content:"Content",data:"Data",artifact:"Artifact",concept:"Concept",naturalobject:"Natural Object",method:"Method",creature:"Creature",plant:"Plant",disease:"Disease",drug:"Drug",food:"Food",other:"Other"},sideBar:{settings:{settings:"Settings",healthCheck:"Health Check",showPropertyPanel:"Show Property Panel",showSearchBar:"Show Search Bar",showNodeLabel:"Show Node Label",nodeDraggable:"Node Draggable",showEdgeLabel:"Show Edge Label",hideUnselectedEdges:"Hide Unselected Edges",edgeEvents:"Edge Events",maxQueryDepth:"Max Query Depth",maxNodes:"Max Nodes",maxLayoutIterations:"Max Layout Iterations",resetToDefault:"Reset to default",edgeSizeRange:"Edge Size Range",depth:"D",max:"Max",degree:"Degree",apiKey:"API Key",enterYourAPIkey:"Enter your API key",save:"Save",refreshLayout:"Refresh Layout"},zoomControl:{zoomIn:"Zoom In",zoomOut:"Zoom Out",resetZoom:"Reset Zoom",rotateCamera:"Clockwise Rotate",rotateCameraCounterClockwise:"Counter-Clockwise Rotate"},layoutsControl:{startAnimation:"Continue layout animation",stopAnimation:"Stop layout animation",layoutGraph:"Layout Graph",layouts:{Circular:"Circular",Circlepack:"Circlepack",Random:"Random",Noverlaps:"Noverlaps","Force Directed":"Force Directed","Force Atlas":"Force Atlas"}},fullScreenControl:{fullScreen:"Full Screen",windowed:"Windowed"},legendControl:{toggleLegend:"Toggle Legend"}},statusIndicator:{connected:"Connected",disconnected:"Disconnected"},statusCard:{unavailable:"Status information unavailable",serverInfo:"Server Info",workingDirectory:"Working Directory",inputDirectory:"Input Directory",maxParallelInsert:"Concurrent Doc Processing",summarySettings:"Summary Settings",llmConfig:"LLM Configuration",llmBinding:"LLM Binding",llmBindingHost:"LLM Endpoint",llmModel:"LLM Model",embeddingConfig:"Embedding Configuration",embeddingBinding:"Embedding Binding",embeddingBindingHost:"Embedding Endpoint",embeddingModel:"Embedding Model",storageConfig:"Storage Configuration",kvStorage:"KV Storage",docStatusStorage:"Doc Status Storage",graphStorage:"Graph Storage",vectorStorage:"Vector Storage",workspace:"Workspace",maxGraphNodes:"Max Graph Nodes",rerankerConfig:"Reranker Configuration",rerankerBindingHost:"Reranker Endpoint",rerankerModel:"Reranker Model",lockStatus:"Lock Status",threshold:"Threshold"},propertiesView:{editProperty:"Edit {{property}}",editPropertyDescription:"Edit the property value in the text area below.",errors:{duplicateName:"Node name already exists",updateFailed:"Failed to update node",tryAgainLater:"Please try again later",updateSuccessButMergeFailed:"Properties updated, but merge failed: {{error}}",mergeFailed:"Merge failed: {{error}}"},success:{entityUpdated:"Node updated successfully",relationUpdated:"Relation updated successfully",entityMerged:"Nodes merged successfully"},mergeOptionLabel:"Automatically merge when a duplicate name is found",mergeOptionDescription:"If enabled, renaming to an existing name will merge this node into the existing one instead of failing.",mergeDialog:{title:"Node merged",description:'"{{source}}" has been merged into "{{target}}".',refreshHint:"Refresh the graph to load the latest structure.",keepCurrentStart:"Refresh and keep current start node",useMergedStart:"Refresh and use merged node",refreshing:"Refreshing graph..."},node:{title:"Node",id:"ID",labels:"Labels",degree:"Degree",properties:"Properties",relationships:"Relations(within subgraph)",expandNode:"Expand Node",pruneNode:"Prune Node",deleteAllNodesError:"Refuse to delete all nodes in the graph",nodesRemoved:"{{count}} nodes removed, including orphan nodes",noNewNodes:"No expandable nodes found",propertyNames:{description:"Description",entity_id:"Name",entity_type:"Type",source_id:"C-ID",Neighbour:"Neigh",file_path:"File",keywords:"Keys",weight:"Weight"}},edge:{title:"Relationship",id:"ID",type:"Type",source:"Source",target:"Target",properties:"Properties"}},search:{placeholder:"Search nodes in page...",message:"And {{count}} others"},graphLabels:{selectTooltip:"Get subgraph of a node (label)",noLabels:"No matching nodes found",label:"Search node name",placeholder:"Search node name...",andOthers:"And {{count}} others",refreshGlobalTooltip:"Refresh global graph data and reset search history",refreshCurrentLabelTooltip:"Refresh current page graph data",refreshingTooltip:"Refreshing data..."},emptyGraph:"Empty(Try Reload Again)"},Eut={chatMessage:{copyTooltip:"Copy to clipboard",copyError:"Failed to copy text to clipboard",copyEmpty:"No content to copy",copySuccess:"Content copied to clipboard",copySuccessLegacy:"Content copied (legacy method)",copySuccessManual:"Content copied (manual method)",copyFailed:"Failed to copy content",copyManualInstruction:"Please select and copy the text manually",thinking:"Thinking...",thinkingTime:"Thinking time {{time}}s",thinkingInProgress:"Thinking in progress..."},retrieval:{startPrompt:"Start a retrieval by typing your query below",clear:"Clear",send:"Send",placeholder:"Enter your query (Support prefix: /<Query Mode>)",error:"Error: Failed to get response",queryModeError:"Only supports the following query modes: {{modes}}",queryModePrefixInvalid:"Invalid query mode prefix. Use: /<mode> [space] your query"},querySettings:{parametersTitle:"Parameters",parametersDescription:"Configure your query parameters",queryMode:"Query Mode",queryModeTooltip:`Select the retrieval strategy:
• Naive: Traditional text chunk vector retrieval
• Local: Focus on entity retrieval
• Global: Focus on relationship retrieval
• Hybrid: Local+Global
• Mix: Local+Global+Naive
• Bypass: Skip retrieval, send conversation history and current question to LLM`,queryModeOptions:{naive:"Naive",local:"Local",global:"Global",hybrid:"Hybrid",mix:"Mix",bypass:"Bypass"},responseFormat:"Response Format",responseFormatTooltip:`Defines the response format. Examples:
• Multiple Paragraphs
• Single Paragraph
• Bullet Points`,responseFormatOptions:{multipleParagraphs:"Multiple Paragraphs",singleParagraph:"Single Paragraph",bulletPoints:"Bullet Points"},topK:"KG Top K",topKTooltip:"Number of entities and relations to retrieve. Applicable for non-naive modes.",topKPlaceholder:"Enter top_k value",chunkTopK:"Chunk Top K",chunkTopKTooltip:"Number of text chunks to retrieve, applicable for all modes.",chunkTopKPlaceholder:"Enter chunk_top_k value",maxEntityTokens:"Max Entity Tokens",maxEntityTokensTooltip:"Maximum number of tokens allocated for entity context in unified token control system",maxRelationTokens:"Max Relation Tokens",maxRelationTokensTooltip:"Maximum number of tokens allocated for relationship context in unified token control system",maxTotalTokens:"Max Total Tokens",maxTotalTokensTooltip:"Maximum total tokens budget for the entire query context (entities + relations + chunks + system prompt)",historyTurns:"History Turns",historyTurnsTooltip:"Number of complete conversation turns (user-assistant pairs) to consider in the response context",historyTurnsPlaceholder:"Number of history turns",onlyNeedContext:"Only Need Context",onlyNeedContextTooltip:"If True, only returns the retrieved context without generating a response",onlyNeedPrompt:"Only Need Prompt",onlyNeedPromptTooltip:"If True, only returns the generated prompt without producing a response",streamResponse:"Stream Response",streamResponseTooltip:"If True, enables streaming output for real-time responses",userPrompt:"Additional Output Prompt",userPromptTooltip:"Provide additional response requirements to the LLM (unrelated to query content, only for output processing).",userPromptPlaceholder:"Enter custom prompt (optional)",enableRerank:"Enable Rerank",enableRerankTooltip:"Enable reranking for retrieved text chunks. If True but no rerank model is configured, a warning will be issued. Default is True."}},xut={loading:"Loading API Documentation..."},wut={title:"API Key is required",description:"Please enter your API key to access the service",placeholder:"Enter your API key",save:"Save"},kut={showing:"Showing {{start}} to {{end}} of {{total}} entries",page:"Page",pageSize:"Page Size",firstPage:"First Page",prevPage:"Previous Page",nextPage:"Next Page",lastPage:"Last Page"},Cut={settings:mut,header:gut,login:but,common:yut,documentPanel:vut,graphPanel:Sut,retrievePanel:Eut,apiSite:xut,apiKeyAlert:wut,pagination:kut},Tut={language:"语言",theme:"主题",light:"浅色",dark:"深色",system:"系统"},Aut={documents:"文档",knowledgeGraph:"知识图谱",retrieval:"检索",api:"API",projectRepository:"项目仓库",logout:"退出登录",themeToggle:{switchToLight:"切换到浅色主题",switchToDark:"切换到深色主题"}},Fut={description:"请输入您的账号和密码登录系统",username:"用户名",usernamePlaceholder:"请输入用户名",password:"密码",passwordPlaceholder:"请输入密码",loginButton:"登录",loggingIn:"登录中...",successMessage:"登录成功",errorEmptyFields:"请输入您的用户名和密码",errorInvalidCredentials:"登录失败，请检查用户名和密码",authDisabled:"认证已禁用，使用无需登陆模式。",guestMode:"无需登陆"},_ut={cancel:"取消",save:"保存",saving:"保存中...",saveFailed:"保存失败"},Rut={clearDocuments:{button:"清空",tooltip:"清空文档",title:"清空文档",description:"此操作将从系统中移除所有文档",warning:"警告：此操作将永久删除所有文档，无法恢复！",confirm:"确定要清空所有文档吗？",confirmPrompt:"请输入 yes 确认操作",confirmPlaceholder:"输入 yes 确认",clearCache:"清空LLM缓存",confirmButton:"确定",clearing:"正在清除...",timeout:"清除操作超时，请重试",success:"文档清空成功",cacheCleared:"缓存清空成功",cacheClearFailed:`清空缓存失败：
{{error}}`,failed:`清空文档失败：
{{message}}`,error:`清空文档失败：
{{error}}`},deleteDocuments:{button:"删除",tooltip:"删除选中的文档",title:"删除文档",description:"此操作将永久删除选中的文档",warning:"警告：此操作将永久删除选中的文档，无法恢复！",confirm:"确定要删除 {{count}} 个选中的文档吗？",confirmPrompt:"请输入 yes 确认操作",confirmPlaceholder:"输入 yes 确认",confirmButton:"确定",deleteFileOption:"同时删除上传文件",deleteFileTooltip:"选中此选项将同时删除服务器上对应的上传文件",deleteLLMCacheOption:"同时删除实体关系抽取 LLM 缓存",success:"文档删除流水线启动成功",failed:`删除文档失败：
{{message}}`,error:`删除文档失败：
{{error}}`,busy:"流水线被占用，请稍后再试",notAllowed:"没有操作权限"},selectDocuments:{selectCurrentPage:"全选当前页 ({{count}})",deselectAll:"取消全选 ({{count}})"},uploadDocuments:{button:"上传",tooltip:"上传文档",title:"上传文档",description:"拖拽文件到此处或点击浏览",single:{uploading:"正在上传 {{name}}：{{percent}}%",success:`上传成功：
{{name}} 上传完成`,failed:`上传失败：
{{name}}
{{message}}`,error:`上传失败：
{{name}}
{{error}}`},batch:{uploading:"正在上传文件...",success:"文件上传完成",error:"部分文件上传失败"},generalError:`上传失败
{{error}}`,fileTypes:"支持的文件类型：TXT, MD, DOCX, PDF, PPTX, XLSX, RTF, ODT, EPUB, HTML, HTM, TEX, JSON, XML, YAML, YML, CSV, LOG, CONF, INI, PROPERTIES, SQL, BAT, SH, C, CPP, PY, JAVA, JS, TS, SWIFT, GO, RB, PHP, CSS, SCSS, LESS",fileUploader:{singleFileLimit:"一次只能上传一个文件",maxFilesLimit:"最多只能上传 {{count}} 个文件",fileRejected:"文件 {{name}} 被拒绝",unsupportedType:"不支持的文件类型",fileTooLarge:"文件过大，最大允许 {{maxSize}}",dropHere:"将文件拖放到此处",dragAndDrop:"拖放文件到此处，或点击选择文件",removeFile:"移除文件",uploadDescription:"您可以上传{{isMultiple ? '多个' : count}}个文件（每个文件最大{{maxSize}}）",duplicateFile:"文件名与服务器上的缓存重复"}},documentManager:{title:"文档管理",scanButton:"扫描/重试",scanTooltip:"扫描处理输入目录中的文档，同时重新处理所有失败的文档",refreshTooltip:"复位文档清单",pipelineStatusButton:"流水线",pipelineStatusTooltip:"查看文档处理流水线状态",uploadedTitle:"已上传文档",uploadedDescription:"已上传文档列表及其状态",emptyTitle:"无文档",emptyDescription:"还没有上传任何文档",columns:{id:"ID",fileName:"文件名",summary:"摘要",status:"状态",length:"长度",chunks:"分块",created:"创建时间",updated:"更新时间",metadata:"元数据",select:"选择"},status:{all:"全部",completed:"已完成",preprocessed:"预处理",processing:"处理中",pending:"等待中",failed:"失败"},errors:{loadFailed:`加载文档失败
{{error}}`,scanFailed:`扫描文档失败
{{error}}`,scanProgressFailed:`获取扫描进度失败
{{error}}`},fileNameLabel:"文件名",showButton:"显示",hideButton:"隐藏",showFileNameTooltip:"显示文件名",hideFileNameTooltip:"隐藏文件名"},pipelineStatus:{title:"流水线状态",busy:"流水线忙碌",requestPending:"待处理请求",cancellationRequested:"取消请求",jobName:"作业名称",startTime:"开始时间",progress:"进度",unit:"批",pipelineMessages:"流水线消息",cancelButton:"中断",cancelTooltip:"中断流水线处理",cancelConfirmTitle:"确认中断流水线",cancelConfirmDescription:"此操作将中断正在进行的流水线处理。确定要继续吗？",cancelConfirmButton:"确认中断",cancelInProgress:"取消请求进行中...",pipelineNotRunning:"流水线未运行",cancelSuccess:"流水线中断请求已发送",cancelFailed:`中断流水线失败
{{error}}`,cancelNotBusy:"流水线未运行，无需中断",errors:{fetchFailed:`获取流水线状态失败
{{error}}`}}},Dut={dataIsTruncated:"图数据已截断至最大返回节点数",statusDialog:{title:"LightRAG 服务器设置",description:"查看当前系统状态和连接信息"},legend:"图例",nodeTypes:{person:"人物角色",category:"分类",geo:"地理名称",location:"位置",organization:"组织机构",event:"事件",equipment:"装备",weapon:"武器",animal:"动物",unknown:"未知",object:"物品",group:"群组",technology:"技术",product:"产品",document:"文档",content:"内容",data:"数据",artifact:"人工制品",concept:"概念",naturalobject:"自然物品",method:"方法",creature:"生物神怪",plant:"植物",disease:"疾病",drug:"药物",food:"食物",other:"其他"},sideBar:{settings:{settings:"设置",healthCheck:"健康检查",showPropertyPanel:"显示属性面板",showSearchBar:"显示搜索栏",showNodeLabel:"显示节点标签",nodeDraggable:"节点可拖动",showEdgeLabel:"显示边标签",hideUnselectedEdges:"隐藏未选中的边",edgeEvents:"边事件",maxQueryDepth:"最大查询深度",maxNodes:"最大返回节点数",maxLayoutIterations:"最大布局迭代次数",resetToDefault:"重置为默认值",edgeSizeRange:"边粗细范围",depth:"深",max:"Max",degree:"邻边",apiKey:"API密钥",enterYourAPIkey:"输入您的API密钥",save:"保存",refreshLayout:"刷新布局"},zoomControl:{zoomIn:"放大",zoomOut:"缩小",resetZoom:"重置缩放",rotateCamera:"顺时针旋转图形",rotateCameraCounterClockwise:"逆时针旋转图形"},layoutsControl:{startAnimation:"继续布局动画",stopAnimation:"停止布局动画",layoutGraph:"图布局",layouts:{Circular:"环形",Circlepack:"圆形打包",Random:"随机",Noverlaps:"无重叠","Force Directed":"力导向","Force Atlas":"力地图"}},fullScreenControl:{fullScreen:"全屏",windowed:"窗口"},legendControl:{toggleLegend:"切换图例显示"}},statusIndicator:{connected:"已连接",disconnected:"未连接"},statusCard:{unavailable:"状态信息不可用",serverInfo:"服务器信息",workingDirectory:"工作目录",inputDirectory:"输入目录",maxParallelInsert:"并行处理文档",summarySettings:"摘要设置",llmConfig:"LLM配置",llmBinding:"LLM绑定",llmBindingHost:"LLM端点",llmModel:"LLM模型",embeddingConfig:"嵌入配置",embeddingBinding:"嵌入绑定",embeddingBindingHost:"嵌入端点",embeddingModel:"嵌入模型",storageConfig:"存储配置",kvStorage:"KV存储",docStatusStorage:"文档状态存储",graphStorage:"图存储",vectorStorage:"向量存储",workspace:"工作空间",maxGraphNodes:"最大图节点数",rerankerConfig:"重排序配置",rerankerBindingHost:"重排序端点",rerankerModel:"重排序模型",lockStatus:"锁状态",threshold:"阈值"},propertiesView:{editProperty:"编辑{{property}}",editPropertyDescription:"在下方文本区域编辑属性值。",errors:{duplicateName:"节点名称已存在",updateFailed:"更新节点失败",tryAgainLater:"请稍后重试",updateSuccessButMergeFailed:"属性已更新，但合并失败：{{error}}",mergeFailed:"合并失败：{{error}}"},success:{entityUpdated:"节点更新成功",relationUpdated:"关系更新成功",entityMerged:"节点合并成功"},mergeOptionLabel:"重名时自动合并",mergeOptionDescription:"勾选后，重命名为已存在的名称会将当前节点自动合并过去，而不会报错。",mergeDialog:{title:"节点已合并",description:'"{{source}}" 已合并到 "{{target}}"。',refreshHint:"请刷新图谱以获取最新结构。",keepCurrentStart:"刷新并保持当前起始节点",useMergedStart:"刷新并以合并后的节点为起始节点",refreshing:"正在刷新图谱..."},node:{title:"节点",id:"ID",labels:"标签",degree:"度数",properties:"属性",relationships:"关系(子图内)",expandNode:"扩展节点",pruneNode:"修剪节点",deleteAllNodesError:"拒绝删除图中的所有节点",nodesRemoved:"已删除 {{count}} 个节点，包括孤立节点",noNewNodes:"没有发现可以扩展的节点",propertyNames:{description:"描述",entity_id:"名称",entity_type:"类型",source_id:"C-ID",Neighbour:"邻接",file_path:"文件",keywords:"Keys",weight:"权重"}},edge:{title:"关系",id:"ID",type:"类型",source:"源节点",target:"目标节点",properties:"属性"}},search:{placeholder:"页面内搜索节点...",message:"还有 {{count}} 个"},graphLabels:{selectTooltip:"获取节点(标签)子图",noLabels:"未找到匹配的节点",label:"搜索节点名称",placeholder:"搜索节点名称...",andOthers:"还有 {{count}} 个",refreshGlobalTooltip:"刷新全图数据和重置搜索历史",refreshCurrentLabelTooltip:"刷新当前页面图数据",refreshingTooltip:"正在刷新数据..."},emptyGraph:"无数据(请重载图形数据)"},Nut={chatMessage:{copyTooltip:"复制到剪贴板",copyError:"复制文本到剪贴板失败",copyEmpty:"没有内容可复制",copySuccess:"内容已复制到剪贴板",copySuccessLegacy:"内容已复制（传统方法）",copySuccessManual:"内容已复制（手动方法）",copyFailed:"复制内容失败",copyManualInstruction:"请手动选择并复制文本",thinking:"正在思考...",thinkingTime:"思考用时 {{time}} 秒",thinkingInProgress:"思考进行中..."},retrieval:{startPrompt:"输入查询开始检索",clear:"清空",send:"发送",placeholder:"输入查询内容 (支持模式前缀: /<Query Mode>)",error:"错误：获取响应失败",queryModeError:"仅支持以下查询模式：{{modes}}",queryModePrefixInvalid:"无效的查询模式前缀。请使用：/<模式> [空格] 查询内容"},querySettings:{parametersTitle:"参数",parametersDescription:"配置查询参数",queryMode:"查询模式",queryModeTooltip:`选择检索策略：
• Naive：传统文本块向量检索
• Local：侧重实体检索
• Global：侧重关系检索
• Hybrid：Local+Global
• Mix：Local+Global+Naive
• Bypass：跳过检索,把历史会话与当前问题送LLM`,queryModeOptions:{naive:"Naive",local:"Local",global:"Global",hybrid:"Hybrid",mix:"Mix",bypass:"Bypass"},responseFormat:"响应格式",responseFormatTooltip:`定义响应格式。例如：
• 多段落
• 单段落
• 要点`,responseFormatOptions:{multipleParagraphs:"多段落",singleParagraph:"单段落",bulletPoints:"要点"},topK:"KG Top K",topKTooltip:"实体关系检索数量, 适用于非naive模式",topKPlaceholder:"输入top_k值",chunkTopK:"文本块 Top K",chunkTopKTooltip:"文本块检索数量, 适用于所有模式",chunkTopKPlaceholder:"输入文本块chunk_top_k值",maxEntityTokens:"实体令牌数上限",maxEntityTokensTooltip:"统一令牌控制系统中分配给实体上下文的最大令牌数",maxRelationTokens:"关系令牌数上限",maxRelationTokensTooltip:"统一令牌控制系统中分配给关系上下文的最大令牌数",maxTotalTokens:"总令牌数上限",maxTotalTokensTooltip:"整个查询上下文的最大总令牌预算（实体+关系+文档块+系统提示）",historyTurns:"历史轮次",historyTurnsTooltip:"响应上下文中考虑的完整对话轮次（用户-助手对）数量",historyTurnsPlaceholder:"历史轮次数",onlyNeedContext:"仅需上下文",onlyNeedContextTooltip:"如果为True，仅返回检索到的上下文而不生成响应",onlyNeedPrompt:"仅需提示",onlyNeedPromptTooltip:"如果为True，仅返回生成的提示而不产生响应",streamResponse:"流式响应",streamResponseTooltip:"如果为True，启用实时流式输出响应",userPrompt:"附加输出提示词",userPromptTooltip:"向LLM提供额外的响应要求（与查询内容无关，仅用于处理输出）。",userPromptPlaceholder:"输入自定义提示词（可选）",enableRerank:"启用重排",enableRerankTooltip:"为检索到的文本块启用重排。如果为True但未配置重排模型，将发出警告。默认为True。"}},Lut={loading:"正在加载 API 文档..."},Mut={title:"需要 API Key",description:"请输入您的 API Key 以访问服务",placeholder:"请输入 API Key",save:"保存"},Iut={showing:"显示第 {{start}} 到 {{end}} 条，共 {{total}} 条记录",page:"页",pageSize:"每页显示",firstPage:"首页",prevPage:"上一页",nextPage:"下一页",lastPage:"末页"},Out={settings:Tut,header:Aut,login:Fut,common:_ut,documentPanel:Rut,graphPanel:Dut,retrievePanel:Nut,apiSite:Lut,apiKeyAlert:Mut,pagination:Iut},But={language:"Langue",theme:"Thème",light:"Clair",dark:"Sombre",system:"Système"},Put={documents:"Documents",knowledgeGraph:"Graphe de connaissances",retrieval:"Récupération",api:"API",projectRepository:"Référentiel du projet",logout:"Déconnexion",themeToggle:{switchToLight:"Passer au thème clair",switchToDark:"Passer au thème sombre"}},zut={description:"Veuillez entrer votre compte et mot de passe pour vous connecter au système",username:"Nom d'utilisateur",usernamePlaceholder:"Veuillez saisir un nom d'utilisateur",password:"Mot de passe",passwordPlaceholder:"Veuillez saisir un mot de passe",loginButton:"Connexion",loggingIn:"Connexion en cours...",successMessage:"Connexion réussie",errorEmptyFields:"Veuillez saisir votre nom d'utilisateur et mot de passe",errorInvalidCredentials:"Échec de la connexion, veuillez vérifier le nom d'utilisateur et le mot de passe",authDisabled:"L'authentification est désactivée. Utilisation du mode sans connexion.",guestMode:"Mode sans connexion"},Hut={cancel:"Annuler",save:"Sauvegarder",saving:"Sauvegarde en cours...",saveFailed:"Échec de la sauvegarde"},Uut={clearDocuments:{button:"Effacer",tooltip:"Effacer les documents",title:"Effacer les documents",description:"Cette action supprimera tous les documents du système",warning:"ATTENTION : Cette action supprimera définitivement tous les documents et ne peut pas être annulée !",confirm:"Voulez-vous vraiment effacer tous les documents ?",confirmPrompt:"Tapez 'yes' pour confirmer cette action",confirmPlaceholder:"Tapez yes pour confirmer",clearCache:"Effacer le cache LLM",confirmButton:"OUI",clearing:"Effacement en cours...",timeout:"L'opération d'effacement a expiré, veuillez réessayer",success:"Documents effacés avec succès",cacheCleared:"Cache effacé avec succès",cacheClearFailed:`Échec de l'effacement du cache :
{{error}}`,failed:`Échec de l'effacement des documents :
{{message}}`,error:`Échec de l'effacement des documents :
{{error}}`},deleteDocuments:{button:"Supprimer",tooltip:"Supprimer les documents sélectionnés",title:"Supprimer les documents",description:"Cette action supprimera définitivement les documents sélectionnés du système",warning:"ATTENTION : Cette action supprimera définitivement les documents sélectionnés et ne peut pas être annulée !",confirm:"Voulez-vous vraiment supprimer {{count}} document(s) sélectionné(s) ?",confirmPrompt:"Tapez 'yes' pour confirmer cette action",confirmPlaceholder:"Tapez yes pour confirmer",confirmButton:"OUI",deleteFileOption:"Supprimer également les fichiers téléchargés",deleteFileTooltip:"Cochez cette option pour supprimer également les fichiers téléchargés correspondants sur le serveur",deleteLLMCacheOption:"Supprimer également le cache LLM d'extraction",success:"Pipeline de suppression de documents démarré avec succès",failed:`Échec de la suppression des documents :
{{message}}`,error:`Échec de la suppression des documents :
{{error}}`,busy:"Le pipeline est occupé, veuillez réessayer plus tard",notAllowed:"Aucune autorisation pour effectuer cette opération"},selectDocuments:{selectCurrentPage:"Sélectionner la page actuelle ({{count}})",deselectAll:"Tout désélectionner ({{count}})"},uploadDocuments:{button:"Télécharger",tooltip:"Télécharger des documents",title:"Télécharger des documents",description:"Glissez-déposez vos documents ici ou cliquez pour parcourir.",single:{uploading:"Téléchargement de {{name}} : {{percent}}%",success:`Succès du téléchargement :
{{name}} téléchargé avec succès`,failed:`Échec du téléchargement :
{{name}}
{{message}}`,error:`Échec du téléchargement :
{{name}}
{{error}}`},batch:{uploading:"Téléchargement des fichiers...",success:"Fichiers téléchargés avec succès",error:"Certains fichiers n'ont pas pu être téléchargés"},generalError:`Échec du téléchargement
{{error}}`,fileTypes:"Types pris en charge : TXT, MD, DOCX, PDF, PPTX, XLSX, RTF, ODT, EPUB, HTML, HTM, TEX, JSON, XML, YAML, YML, CSV, LOG, CONF, INI, PROPERTIES, SQL, BAT, SH, C, CPP, PY, JAVA, JS, TS, SWIFT, GO, RB, PHP, CSS, SCSS, LESS",fileUploader:{singleFileLimit:"Impossible de télécharger plus d'un fichier à la fois",maxFilesLimit:"Impossible de télécharger plus de {{count}} fichiers",fileRejected:"Le fichier {{name}} a été rejeté",unsupportedType:"Type de fichier non pris en charge",fileTooLarge:"Fichier trop volumineux, taille maximale {{maxSize}}",dropHere:"Déposez les fichiers ici",dragAndDrop:"Glissez et déposez les fichiers ici, ou cliquez pour sélectionner",removeFile:"Supprimer le fichier",uploadDescription:"Vous pouvez télécharger {{isMultiple ? 'plusieurs' : count}} fichiers (jusqu'à {{maxSize}} chacun)",duplicateFile:"Le nom du fichier existe déjà dans le cache du serveur"}},documentManager:{title:"Gestion des documents",scanButton:"Scanner/Retraiter",scanTooltip:"Scanner et traiter les documents dans le dossier d'entrée, et retraiter également tous les documents échoués",refreshTooltip:"Réinitialiser la liste des documents",pipelineStatusButton:"Pipeline",pipelineStatusTooltip:"Voir l'état du pipeline de traitement des documents",uploadedTitle:"Documents téléchargés",uploadedDescription:"Liste des documents téléchargés et leurs statuts.",emptyTitle:"Aucun document",emptyDescription:"Il n'y a pas encore de documents téléchargés.",columns:{id:"ID",fileName:"Nom du fichier",summary:"Résumé",status:"Statut",length:"Longueur",chunks:"Fragments",created:"Créé",updated:"Mis à jour",metadata:"Métadonnées",select:"Sélectionner"},status:{all:"Tous",completed:"Terminé",preprocessed:"Prétraité",processing:"En traitement",pending:"En attente",failed:"Échoué"},errors:{loadFailed:`Échec du chargement des documents
{{error}}`,scanFailed:`Échec de la numérisation des documents
{{error}}`,scanProgressFailed:`Échec de l'obtention de la progression de la numérisation
{{error}}`},fileNameLabel:"Nom du fichier",showButton:"Afficher",hideButton:"Masquer",showFileNameTooltip:"Afficher le nom du fichier",hideFileNameTooltip:"Masquer le nom du fichier"},pipelineStatus:{title:"État du Pipeline",busy:"Pipeline Occupé",requestPending:"Demande en Attente",cancellationRequested:"Annulation Demandée",jobName:"Nom du Travail",startTime:"Heure de Début",progress:"Progrès",unit:"Lot",pipelineMessages:"Messages de Pipeline",cancelButton:"Annuler",cancelTooltip:"Annuler le traitement du pipeline",cancelConfirmTitle:"Confirmer l'Annulation du Pipeline",cancelConfirmDescription:"Cette action interrompra le traitement du pipeline en cours. Êtes-vous sûr de vouloir continuer ?",cancelConfirmButton:"Confirmer l'Annulation",cancelInProgress:"Annulation en cours...",pipelineNotRunning:"Le pipeline n'est pas en cours d'exécution",cancelSuccess:"Annulation du pipeline demandée",cancelFailed:`Échec de l'annulation du pipeline
{{error}}`,cancelNotBusy:"Le pipeline n'est pas en cours d'exécution, pas besoin d'annuler",errors:{fetchFailed:`Échec de la récupération de l'état du pipeline
{{error}}`}}},$ut={dataIsTruncated:"Les données du graphe sont tronquées au nombre maximum de nœuds",statusDialog:{title:"Paramètres du Serveur LightRAG",description:"Afficher l'état actuel du système et les informations de connexion"},legend:"Légende",nodeTypes:{person:"Personne",category:"Catégorie",geo:"Géographique",location:"Emplacement",organization:"Organisation",event:"Événement",equipment:"Équipement",weapon:"Arme",animal:"Animal",unknown:"Inconnu",object:"Objet",group:"Groupe",technology:"Technologie",product:"Produit",document:"Document",content:"Contenu",data:"Données",artifact:"Artefact",concept:"Concept",naturalobject:"Objet naturel",method:"Méthode",creature:"Créature",plant:"Plante",disease:"Maladie",drug:"Médicament",food:"Nourriture",other:"Autre"},sideBar:{settings:{settings:"Paramètres",healthCheck:"Vérification de l'état",showPropertyPanel:"Afficher le panneau des propriétés",showSearchBar:"Afficher la barre de recherche",showNodeLabel:"Afficher l'étiquette du nœud",nodeDraggable:"Nœud déplaçable",showEdgeLabel:"Afficher l'étiquette de l'arête",hideUnselectedEdges:"Masquer les arêtes non sélectionnées",edgeEvents:"Événements des arêtes",maxQueryDepth:"Profondeur maximale de la requête",maxNodes:"Nombre maximum de nœuds",maxLayoutIterations:"Itérations maximales de mise en page",resetToDefault:"Réinitialiser par défaut",edgeSizeRange:"Plage de taille des arêtes",depth:"D",max:"Max",degree:"Degré",apiKey:"Clé API",enterYourAPIkey:"Entrez votre clé API",save:"Sauvegarder",refreshLayout:"Actualiser la mise en page"},zoomControl:{zoomIn:"Zoom avant",zoomOut:"Zoom arrière",resetZoom:"Réinitialiser le zoom",rotateCamera:"Rotation horaire",rotateCameraCounterClockwise:"Rotation antihoraire"},layoutsControl:{startAnimation:"Démarrer l'animation de mise en page",stopAnimation:"Arrêter l'animation de mise en page",layoutGraph:"Mettre en page le graphe",layouts:{Circular:"Circulaire",Circlepack:"Paquet circulaire",Random:"Aléatoire",Noverlaps:"Sans chevauchement","Force Directed":"Dirigé par la force","Force Atlas":"Atlas de force"}},fullScreenControl:{fullScreen:"Plein écran",windowed:"Fenêtré"},legendControl:{toggleLegend:"Basculer la légende"}},statusIndicator:{connected:"Connecté",disconnected:"Déconnecté"},statusCard:{unavailable:"Informations sur l'état indisponibles",serverInfo:"Informations du serveur",workingDirectory:"Répertoire de travail",inputDirectory:"Répertoire d'entrée",maxParallelInsert:"Traitement simultané des documents",summarySettings:"Paramètres de résumé",llmConfig:"Configuration du modèle de langage",llmBinding:"Liaison du modèle de langage",llmBindingHost:"Point de terminaison LLM",llmModel:"Modèle de langage",embeddingConfig:"Configuration d'incorporation",embeddingBinding:"Liaison d'incorporation",embeddingBindingHost:"Point de terminaison d'incorporation",embeddingModel:"Modèle d'incorporation",storageConfig:"Configuration de stockage",kvStorage:"Stockage clé-valeur",docStatusStorage:"Stockage de l'état des documents",graphStorage:"Stockage du graphe",vectorStorage:"Stockage vectoriel",workspace:"Espace de travail",maxGraphNodes:"Nombre maximum de nœuds du graphe",rerankerConfig:"Configuration du reclassement",rerankerBindingHost:"Point de terminaison de reclassement",rerankerModel:"Modèle de reclassement",lockStatus:"État des verrous",threshold:"Seuil"},propertiesView:{editProperty:"Modifier {{property}}",editPropertyDescription:"Modifiez la valeur de la propriété dans la zone de texte ci-dessous.",errors:{duplicateName:"Le nom du nœud existe déjà",updateFailed:"Échec de la mise à jour du nœud",tryAgainLater:"Veuillez réessayer plus tard",updateSuccessButMergeFailed:"Propriétés mises à jour, mais la fusion a échoué : {{error}}",mergeFailed:"Échec de la fusion : {{error}}"},success:{entityUpdated:"Nœud mis à jour avec succès",relationUpdated:"Relation mise à jour avec succès",entityMerged:"Fusion des nœuds réussie"},mergeOptionLabel:"Fusionner automatiquement en cas de nom dupliqué",mergeOptionDescription:"Si activé, renommer vers un nom existant fusionnera automatiquement ce nœud avec celui-ci au lieu d'échouer.",mergeDialog:{title:"Nœud fusionné",description:'"{{source}}" a été fusionné dans "{{target}}".',refreshHint:"Actualisez le graphe pour charger la structure la plus récente.",keepCurrentStart:"Actualiser en conservant le nœud de départ actuel",useMergedStart:"Actualiser en utilisant le nœud fusionné",refreshing:"Actualisation du graphe..."},node:{title:"Nœud",id:"ID",labels:"Étiquettes",degree:"Degré",properties:"Propriétés",relationships:"Relations(dans le sous-graphe)",expandNode:"Développer le nœud",pruneNode:"Élaguer le nœud",deleteAllNodesError:"Refus de supprimer tous les nœuds du graphe",nodesRemoved:"{{count}} nœuds supprimés, y compris les nœuds orphelins",noNewNodes:"Aucun nœud développable trouvé",propertyNames:{description:"Description",entity_id:"Nom",entity_type:"Type",source_id:"C-ID",Neighbour:"Voisin",file_path:"File",keywords:"Keys",weight:"Poids"}},edge:{title:"Relation",id:"ID",type:"Type",source:"Source",target:"Cible",properties:"Propriétés"}},search:{placeholder:"Rechercher des nœuds dans la page...",message:"Et {{count}} autres"},graphLabels:{selectTooltip:"Obtenir le sous-graphe d'un nœud (étiquette)",noLabels:"Aucun nœud correspondant trouvé",label:"Rechercher le nom du nœud",placeholder:"Rechercher le nom du nœud...",andOthers:"Et {{count}} autres",refreshGlobalTooltip:"Actualiser les données du graphe global et réinitialiser l'historique de recherche",refreshCurrentLabelTooltip:"Actualiser les données du graphe de la page actuelle",refreshingTooltip:"Actualisation des données en cours..."},emptyGraph:"Vide (Essayez de recharger)"},Gut={chatMessage:{copyTooltip:"Copier dans le presse-papiers",copyError:"Échec de la copie du texte dans le presse-papiers",copyEmpty:"Aucun contenu à copier",copySuccess:"Contenu copié dans le presse-papiers",copySuccessLegacy:"Contenu copié (méthode héritée)",copySuccessManual:"Contenu copié (méthode manuelle)",copyFailed:"Échec de la copie du contenu",copyManualInstruction:"Veuillez sélectionner et copier le texte manuellement",thinking:"Réflexion en cours...",thinkingTime:"Temps de réflexion {{time}}s",thinkingInProgress:"Réflexion en cours..."},retrieval:{startPrompt:"Démarrez une récupération en tapant votre requête ci-dessous",clear:"Effacer",send:"Envoyer",placeholder:"Tapez votre requête (Préfixe de requête : /<Query Mode>)",error:"Erreur : Échec de l'obtention de la réponse",queryModeError:"Seuls les modes de requête suivants sont pris en charge : {{modes}}",queryModePrefixInvalid:"Préfixe de mode de requête invalide. Utilisez : /<mode> [espace] votre requête"},querySettings:{parametersTitle:"Paramètres",parametersDescription:"Configurez vos paramètres de requête",queryMode:"Mode de requête",queryModeTooltip:`Sélectionnez la stratégie de récupération :
• Naïf : Récupération vectorielle traditionnelle par blocs de texte
• Local : Axé sur la récupération d'entités
• Global : Axé sur la récupération de relations
• Hybride : Local+Global
• Mixte : Local+Global+Naïf
• Bypass : Ignorer la récupération, envoyer l'historique de conversation et la question actuelle au LLM`,queryModeOptions:{naive:"Naïf",local:"Local",global:"Global",hybrid:"Hybride",mix:"Mixte",bypass:"Bypass"},responseFormat:"Format de réponse",responseFormatTooltip:`Définit le format de la réponse. Exemples :
• Plusieurs paragraphes
• Paragraphe unique
• Points à puces`,responseFormatOptions:{multipleParagraphs:"Plusieurs paragraphes",singleParagraph:"Paragraphe unique",bulletPoints:"Points à puces"},topK:"KG Top K",topKTooltip:"Nombre d'entités et de relations à récupérer. Applicable pour les modes non-naïfs.",topKPlaceholder:"Entrez la valeur top_k",chunkTopK:"Top K des Chunks",chunkTopKTooltip:"Nombre de morceaux de texte à récupérer, applicable à tous les modes.",chunkTopKPlaceholder:"Entrez la valeur chunk_top_k",maxEntityTokens:"Limite de jetons d'entité",maxEntityTokensTooltip:"Nombre maximum de jetons alloués au contexte d'entité dans le système de contrôle de jetons unifié",maxRelationTokens:"Limite de jetons de relation",maxRelationTokensTooltip:"Nombre maximum de jetons alloués au contexte de relation dans le système de contrôle de jetons unifié",maxTotalTokens:"Limite totale de jetons",maxTotalTokensTooltip:"Budget total maximum de jetons pour l'ensemble du contexte de requête (entités + relations + blocs + prompt système)",historyTurns:"Tours d'historique",historyTurnsTooltip:"Nombre de tours complets de conversation (paires utilisateur-assistant) à prendre en compte dans le contexte de la réponse",historyTurnsPlaceholder:"Nombre de tours d'historique",onlyNeedContext:"Besoin uniquement du contexte",onlyNeedContextTooltip:"Si vrai, ne renvoie que le contexte récupéré sans générer de réponse",onlyNeedPrompt:"Besoin uniquement de l'invite",onlyNeedPromptTooltip:"Si vrai, ne renvoie que l'invite générée sans produire de réponse",streamResponse:"Réponse en flux",streamResponseTooltip:"Si vrai, active la sortie en flux pour des réponses en temps réel",userPrompt:"Invite de sortie supplémentaire",userPromptTooltip:"Fournir des exigences de réponse supplémentaires au LLM (sans rapport avec le contenu de la requête, uniquement pour le traitement de sortie).",userPromptPlaceholder:"Entrez une invite personnalisée (facultatif)",enableRerank:"Activer le Reclassement",enableRerankTooltip:"Active le reclassement pour les fragments de texte récupérés. Si True mais qu'aucun modèle de reclassement n'est configuré, un avertissement sera émis. True par défaut."}},jut={loading:"Chargement de la documentation de l'API..."},qut={title:"Clé API requise",description:"Veuillez entrer votre clé API pour accéder au service",placeholder:"Entrez votre clé API",save:"Sauvegarder"},Wut={showing:"Affichage de {{start}} à {{end}} sur {{total}} entrées",page:"Page",pageSize:"Taille de la page",firstPage:"Première page",prevPage:"Page précédente",nextPage:"Page suivante",lastPage:"Dernière page"},Vut={settings:But,header:Put,login:zut,common:Hut,documentPanel:Uut,graphPanel:$ut,retrievePanel:Gut,apiSite:jut,apiKeyAlert:qut,pagination:Wut},Kut={language:"اللغة",theme:"السمة",light:"فاتح",dark:"داكن",system:"النظام"},Yut={documents:"المستندات",knowledgeGraph:"شبكة المعرفة",retrieval:"الاسترجاع",api:"واجهة برمجة التطبيقات",projectRepository:"مستودع المشروع",logout:"تسجيل الخروج",themeToggle:{switchToLight:"التحويل إلى السمة الفاتحة",switchToDark:"التحويل إلى السمة الداكنة"}},Jut={description:"الرجاء إدخال حسابك وكلمة المرور لتسجيل الدخول إلى النظام",username:"اسم المستخدم",usernamePlaceholder:"الرجاء إدخال اسم المستخدم",password:"كلمة المرور",passwordPlaceholder:"الرجاء إدخال كلمة المرور",loginButton:"تسجيل الدخول",loggingIn:"جاري تسجيل الدخول...",successMessage:"تم تسجيل الدخول بنجاح",errorEmptyFields:"الرجاء إدخال اسم المستخدم وكلمة المرور",errorInvalidCredentials:"فشل تسجيل الدخول، يرجى التحقق من اسم المستخدم وكلمة المرور",authDisabled:"تم تعطيل المصادقة. استخدام وضع بدون تسجيل دخول.",guestMode:"وضع بدون تسجيل دخول"},Xut={cancel:"إلغاء",save:"حفظ",saving:"جارٍ الحفظ...",saveFailed:"فشل الحفظ"},Zut={clearDocuments:{button:"مسح",tooltip:"مسح المستندات",title:"مسح المستندات",description:"سيؤدي هذا إلى إزالة جميع المستندات من النظام",warning:"تحذير: سيؤدي هذا الإجراء إلى حذف جميع المستندات بشكل دائم ولا يمكن التراجع عنه!",confirm:"هل تريد حقًا مسح جميع المستندات؟",confirmPrompt:"اكتب 'yes' لتأكيد هذا الإجراء",confirmPlaceholder:"اكتب yes للتأكيد",clearCache:"مسح كاش نموذج اللغة",confirmButton:"نعم",clearing:"جارٍ المسح...",timeout:"انتهت مهلة عملية المسح، يرجى المحاولة مرة أخرى",success:"تم مسح المستندات بنجاح",cacheCleared:"تم مسح ذاكرة التخزين المؤقت بنجاح",cacheClearFailed:`فشل مسح ذاكرة التخزين المؤقت:
{{error}}`,failed:`فشل مسح المستندات:
{{message}}`,error:`فشل مسح المستندات:
{{error}}`},deleteDocuments:{button:"حذف",tooltip:"حذف المستندات المحددة",title:"حذف المستندات",description:"سيؤدي هذا إلى حذف المستندات المحددة نهائيًا من النظام",warning:"تحذير: سيؤدي هذا الإجراء إلى حذف المستندات المحددة نهائيًا ولا يمكن التراجع عنه!",confirm:"هل تريد حقًا حذف {{count}} مستند(ات) محدد(ة)؟",confirmPrompt:"اكتب 'yes' لتأكيد هذا الإجراء",confirmPlaceholder:"اكتب yes للتأكيد",confirmButton:"نعم",deleteFileOption:"حذف الملفات المرفوعة أيضًا",deleteFileTooltip:"حدد هذا الخيار لحذف الملفات المرفوعة المقابلة على الخادم أيضًا",deleteLLMCacheOption:"حذف ذاكرة LLM المؤقتة للاستخراج أيضًا",success:"تم بدء تشغيل خط معالجة حذف المستندات بنجاح",failed:`فشل حذف المستندات:
{{message}}`,error:`فشل حذف المستندات:
{{error}}`,busy:"خط المعالجة مشغول، يرجى المحاولة مرة أخرى لاحقًا",notAllowed:"لا توجد صلاحية لتنفيذ هذه العملية"},selectDocuments:{selectCurrentPage:"تحديد الصفحة الحالية ({{count}})",deselectAll:"إلغاء تحديد الكل ({{count}})"},uploadDocuments:{button:"رفع",tooltip:"رفع المستندات",title:"رفع المستندات",description:"اسحب وأفلت مستنداتك هنا أو انقر للتصفح.",single:{uploading:"جارٍ الرفع {{name}}: {{percent}}%",success:`نجاح الرفع:
تم رفع {{name}} بنجاح`,failed:`فشل الرفع:
{{name}}
{{message}}`,error:`فشل الرفع:
{{name}}
{{error}}`},batch:{uploading:"جارٍ رفع الملفات...",success:"تم رفع الملفات بنجاح",error:"فشل رفع بعض الملفات"},generalError:`فشل الرفع
{{error}}`,fileTypes:"الأنواع المدعومة: TXT، MD، DOCX، PDF، PPTX، XLSX، RTF، ODT، EPUB، HTML، HTM، TEX، JSON، XML، YAML، YML، CSV، LOG، CONF، INI، PROPERTIES، SQL، BAT، SH، C، CPP، PY، JAVA، JS، TS، SWIFT، GO، RB، PHP، CSS، SCSS، LESS",fileUploader:{singleFileLimit:"لا يمكن رفع أكثر من ملف واحد في المرة الواحدة",maxFilesLimit:"لا يمكن رفع أكثر من {{count}} ملفات",fileRejected:"تم رفض الملف {{name}}",unsupportedType:"نوع الملف غير مدعوم",fileTooLarge:"حجم الملف كبير جدًا، الحد الأقصى {{maxSize}}",dropHere:"أفلت الملفات هنا",dragAndDrop:"اسحب وأفلت الملفات هنا، أو انقر للاختيار",removeFile:"إزالة الملف",uploadDescription:"يمكنك رفع {{isMultiple ? 'عدة' : count}} ملفات (حتى {{maxSize}} لكل منها)",duplicateFile:"اسم الملف موجود بالفعل في ذاكرة التخزين المؤقت للخادم"}},documentManager:{title:"إدارة المستندات",scanButton:"مسح/إعادة محاولة",scanTooltip:"مسح ومعالجة المستندات في مجلد الإدخال، وإعادة معالجة جميع المستندات الفاشلة أيضًا",refreshTooltip:"إعادة تعيين قائمة المستندات",pipelineStatusButton:"خط المعالجة",pipelineStatusTooltip:"عرض حالة خط معالجة المستندات",uploadedTitle:"المستندات المرفوعة",uploadedDescription:"قائمة المستندات المرفوعة وحالاتها.",emptyTitle:"لا توجد مستندات",emptyDescription:"لا توجد مستندات مرفوعة بعد.",columns:{id:"المعرف",fileName:"اسم الملف",summary:"الملخص",status:"الحالة",length:"الطول",chunks:"الأجزاء",created:"تم الإنشاء",updated:"تم التحديث",metadata:"البيانات الوصفية",select:"اختيار"},status:{all:"الكل",completed:"مكتمل",preprocessed:"مُعالج مسبقًا",processing:"قيد المعالجة",pending:"معلق",failed:"فشل"},errors:{loadFailed:`فشل تحميل المستندات
{{error}}`,scanFailed:`فشل مسح المستندات
{{error}}`,scanProgressFailed:`فشل الحصول على تقدم المسح
{{error}}`},fileNameLabel:"اسم الملف",showButton:"عرض",hideButton:"إخفاء",showFileNameTooltip:"عرض اسم الملف",hideFileNameTooltip:"إخفاء اسم الملف"},pipelineStatus:{title:"حالة خط الأنابيب",busy:"خط الأنابيب مشغول",requestPending:"طلب معلق",cancellationRequested:"طلب الإلغاء",jobName:"اسم المهمة",startTime:"وقت البدء",progress:"التقدم",unit:"دفعة",pipelineMessages:"رسائل خط الأنابيب",cancelButton:"إلغاء",cancelTooltip:"إلغاء معالجة خط الأنابيب",cancelConfirmTitle:"تأكيد إلغاء خط الأنابيب",cancelConfirmDescription:"سيؤدي هذا الإجراء إلى إيقاف معالجة خط الأنابيب الجارية. هل أنت متأكد من أنك تريد المتابعة؟",cancelConfirmButton:"تأكيد الإلغاء",cancelInProgress:"الإلغاء قيد التقدم...",pipelineNotRunning:"خط الأنابيب غير قيد التشغيل",cancelSuccess:"تم طلب إلغاء خط الأنابيب",cancelFailed:`فشل إلغاء خط الأنابيب
{{error}}`,cancelNotBusy:"خط الأنابيب غير قيد التشغيل، لا حاجة للإلغاء",errors:{fetchFailed:`فشل في جلب حالة خط الأنابيب
{{error}}`}}},Qut={dataIsTruncated:"تم اقتصار بيانات الرسم البياني على الحد الأقصى للعقد",statusDialog:{title:"إعدادات خادم LightRAG",description:"عرض حالة النظام الحالية ومعلومات الاتصال"},legend:"المفتاح",nodeTypes:{person:"شخص",category:"فئة",geo:"كيان جغرافي",location:"موقع",organization:"منظمة",event:"حدث",equipment:"معدات",weapon:"سلاح",animal:"حيوان",unknown:"غير معروف",object:"مصنوع",group:"مجموعة",technology:"العلوم",product:"منتج",document:"وثيقة",content:"محتوى",data:"بيانات",artifact:"قطعة أثرية",concept:"مفهوم",naturalobject:"كائن طبيعي",method:"عملية",creature:"مخلوق",plant:"نبات",disease:"مرض",drug:"دواء",food:"طعام",other:"أخرى"},sideBar:{settings:{settings:"الإعدادات",healthCheck:"فحص الحالة",showPropertyPanel:"إظهار لوحة الخصائص",showSearchBar:"إظهار شريط البحث",showNodeLabel:"إظهار تسمية العقدة",nodeDraggable:"العقدة قابلة للسحب",showEdgeLabel:"إظهار تسمية الحافة",hideUnselectedEdges:"إخفاء الحواف غير المحددة",edgeEvents:"أحداث الحافة",maxQueryDepth:"أقصى عمق للاستعلام",maxNodes:"الحد الأقصى للعقد",maxLayoutIterations:"أقصى تكرارات التخطيط",resetToDefault:"إعادة التعيين إلى الافتراضي",edgeSizeRange:"نطاق حجم الحافة",depth:"D",max:"Max",degree:"الدرجة",apiKey:"مفتاح واجهة برمجة التطبيقات",enterYourAPIkey:"أدخل مفتاح واجهة برمجة التطبيقات الخاص بك",save:"حفظ",refreshLayout:"تحديث التخطيط"},zoomControl:{zoomIn:"تكبير",zoomOut:"تصغير",resetZoom:"إعادة تعيين التكبير",rotateCamera:"تدوير في اتجاه عقارب الساعة",rotateCameraCounterClockwise:"تدوير عكس اتجاه عقارب الساعة"},layoutsControl:{startAnimation:"بدء حركة التخطيط",stopAnimation:"إيقاف حركة التخطيط",layoutGraph:"تخطيط الرسم البياني",layouts:{Circular:"دائري",Circlepack:"حزمة دائرية",Random:"عشوائي",Noverlaps:"بدون تداخل","Force Directed":"موجه بالقوة","Force Atlas":"أطلس القوة"}},fullScreenControl:{fullScreen:"شاشة كاملة",windowed:"نوافذ"},legendControl:{toggleLegend:"تبديل المفتاح"}},statusIndicator:{connected:"متصل",disconnected:"غير متصل"},statusCard:{unavailable:"معلومات الحالة غير متوفرة",serverInfo:"معلومات الخادم",workingDirectory:"دليل العمل",inputDirectory:"دليل الإدخال",maxParallelInsert:"معالجة المستندات المتزامنة",summarySettings:"إعدادات الملخص",llmConfig:"تكوين نموذج اللغة الكبير",llmBinding:"ربط نموذج اللغة الكبير",llmBindingHost:"نقطة نهاية نموذج اللغة الكبير",llmModel:"نموذج اللغة الكبير",embeddingConfig:"تكوين التضمين",embeddingBinding:"ربط التضمين",embeddingBindingHost:"نقطة نهاية التضمين",embeddingModel:"نموذج التضمين",storageConfig:"تكوين التخزين",kvStorage:"تخزين المفتاح-القيمة",docStatusStorage:"تخزين حالة المستند",graphStorage:"تخزين الرسم البياني",vectorStorage:"تخزين المتجهات",workspace:"مساحة العمل",maxGraphNodes:"الحد الأقصى لعقد الرسم البياني",rerankerConfig:"تكوين إعادة الترتيب",rerankerBindingHost:"نقطة نهاية إعادة الترتيب",rerankerModel:"نموذج إعادة الترتيب",lockStatus:"حالة القفل",threshold:"العتبة"},propertiesView:{editProperty:"تعديل {{property}}",editPropertyDescription:"قم بتحرير قيمة الخاصية في منطقة النص أدناه.",errors:{duplicateName:"اسم العقدة موجود بالفعل",updateFailed:"فشل تحديث العقدة",tryAgainLater:"يرجى المحاولة مرة أخرى لاحقًا",updateSuccessButMergeFailed:"تم تحديث الخصائص، لكن الدمج فشل: {{error}}",mergeFailed:"فشل الدمج: {{error}}"},success:{entityUpdated:"تم تحديث العقدة بنجاح",relationUpdated:"تم تحديث العلاقة بنجاح",entityMerged:"تم دمج العقد بنجاح"},mergeOptionLabel:"دمج تلقائي عند العثور على اسم مكرر",mergeOptionDescription:"عند التفعيل، سيتم دمج هذه العقدة تلقائيًا في العقدة الموجودة بدلاً من ظهور خطأ عند إعادة التسمية بنفس الاسم.",mergeDialog:{title:"تم دمج العقدة",description:'"{{source}}" تم دمجها في "{{target}}".',refreshHint:"يجب تحديث الرسم البياني لتحميل البنية الأحدث.",keepCurrentStart:"تحديث مع الحفاظ على عقدة البدء الحالية",useMergedStart:"تحديث واستخدام العقدة المدمجة كنقطة بدء",refreshing:"جارٍ تحديث الرسم البياني..."},node:{title:"عقدة",id:"المعرف",labels:"التسميات",degree:"الدرجة",properties:"الخصائص",relationships:"العلاقات (داخل الرسم الفرعي)",expandNode:"توسيع العقدة",pruneNode:"تقليم العقدة",deleteAllNodesError:"رفض حذف جميع العقد في الرسم البياني",nodesRemoved:"تم إزالة {{count}} عقدة، بما في ذلك العقد اليتيمة",noNewNodes:"لم يتم العثور على عقد قابلة للتوسيع",propertyNames:{description:"الوصف",entity_id:"الاسم",entity_type:"النوع",source_id:"C-ID",Neighbour:"الجار",file_path:"File",keywords:"Keyword",weight:"الوزن"}},edge:{title:"علاقة",id:"المعرف",type:"النوع",source:"المصدر",target:"الهدف",properties:"الخصائص"}},search:{placeholder:"ابحث في العقد في الصفحة...",message:"و {{count}} آخرون"},graphLabels:{selectTooltip:"الحصول على الرسم البياني الفرعي لعقدة (تسمية)",noLabels:"لم يتم العثور على عقد مطابقة",label:"البحث عن اسم العقدة",placeholder:"البحث عن اسم العقدة...",andOthers:"و {{count}} آخرون",refreshGlobalTooltip:"تحديث بيانات الرسم البياني العالمي وإعادة تعيين سجل البحث",refreshCurrentLabelTooltip:"تحديث بيانات الرسم البياني للصفحة الحالية",refreshingTooltip:"جارٍ تحديث البيانات..."},emptyGraph:"فارغ (حاول إعادة التحميل)"},ect={chatMessage:{copyTooltip:"نسخ إلى الحافظة",copyError:"فشل نسخ النص إلى الحافظة",copyEmpty:"لا يوجد محتوى للنسخ",copySuccess:"تم نسخ المحتوى إلى الحافظة",copySuccessLegacy:"تم نسخ المحتوى (الطريقة التقليدية)",copySuccessManual:"تم نسخ المحتوى (الطريقة اليدوية)",copyFailed:"فشل نسخ المحتوى",copyManualInstruction:"يرجى تحديد ونسخ النص يدوياً",thinking:"جاري التفكير...",thinkingTime:"وقت التفكير {{time}} ثانية",thinkingInProgress:"التفكير قيد التقدم..."},retrieval:{startPrompt:"ابدأ الاسترجاع بكتابة استفسارك أدناه",clear:"مسح",send:"إرسال",placeholder:"اكتب استفسارك (بادئة وضع الاستعلام: /<Query Mode>)",error:"خطأ: فشل الحصول على الرد",queryModeError:"يُسمح فقط بأنماط الاستعلام التالية: {{modes}}",queryModePrefixInvalid:"بادئة وضع الاستعلام غير صالحة. استخدم: /<الوضع> [مسافة] استفسارك"},querySettings:{parametersTitle:"المعلمات",parametersDescription:"تكوين معلمات الاستعلام الخاص بك",queryMode:"وضع الاستعلام",queryModeTooltip:`حدد استراتيجية الاسترجاع:
• ساذج: استرجاع متجهي تقليدي لقطع النص
• محلي: يركز على استرجاع الكيانات
• عالمي: يركز على استرجاع العلاقات
• مختلط: محلي+عالمي
• مزيج: محلي+عالمي+ساذج
• تجاوز: تخطي الاسترجاع، إرسال تاريخ المحادثة والسؤال الحالي إلى LLM`,queryModeOptions:{naive:"ساذج",local:"محلي",global:"عالمي",hybrid:"مختلط",mix:"مزيج",bypass:"تجاوز"},responseFormat:"تنسيق الرد",responseFormatTooltip:`يحدد تنسيق الرد. أمثلة:
• فقرات متعددة
• فقرة واحدة
• نقاط نقطية`,responseFormatOptions:{multipleParagraphs:"فقرات متعددة",singleParagraph:"فقرة واحدة",bulletPoints:"نقاط نقطية"},topK:"KG أعلى K",topKTooltip:"عدد الكيانات والعلاقات المطلوب استردادها، لا ينطبق على الوضع наивный.",topKPlaceholder:"أدخل قيمة top_k",chunkTopK:"أعلى K للقطع",chunkTopKTooltip:"عدد أجزاء النص المطلوب استردادها، وينطبق على جميع الأوضاع.",chunkTopKPlaceholder:"أدخل قيمة chunk_top_k",maxEntityTokens:"الحد الأقصى لرموز الكيان",maxEntityTokensTooltip:"الحد الأقصى لعدد الرموز المخصصة لسياق الكيان في نظام التحكم الموحد في الرموز",maxRelationTokens:"الحد الأقصى لرموز العلاقة",maxRelationTokensTooltip:"الحد الأقصى لعدد الرموز المخصصة لسياق العلاقة في نظام التحكم الموحد في الرموز",maxTotalTokens:"إجمالي الحد الأقصى للرموز",maxTotalTokensTooltip:"الحد الأقصى الإجمالي لميزانية الرموز لسياق الاستعلام بالكامل (الكيانات + العلاقات + الأجزاء + موجه النظام)",historyTurns:"أدوار التاريخ",historyTurnsTooltip:"عدد الدورات الكاملة للمحادثة (أزواج المستخدم-المساعد) التي يجب مراعاتها في سياق الرد",historyTurnsPlaceholder:"عدد دورات التاريخ",onlyNeedContext:"تحتاج فقط إلى السياق",onlyNeedContextTooltip:"إذا كان صحيحًا، يتم إرجاع السياق المسترجع فقط دون إنشاء رد",onlyNeedPrompt:"تحتاج فقط إلى المطالبة",onlyNeedPromptTooltip:"إذا كان صحيحًا، يتم إرجاع المطالبة المولدة فقط دون إنتاج رد",streamResponse:"تدفق الرد",streamResponseTooltip:"إذا كان صحيحًا، يتيح إخراج التدفق للردود في الوقت الفعلي",userPrompt:"مطالبة إخراج إضافية",userPromptTooltip:"تقديم متطلبات استجابة إضافية إلى نموذج اللغة الكبير (غير متعلقة بمحتوى الاستعلام، فقط لمعالجة المخرجات).",userPromptPlaceholder:"أدخل مطالبة مخصصة (اختياري)",enableRerank:"تمكين إعادة الترتيب",enableRerankTooltip:"تمكين إعادة ترتيب أجزاء النص المسترجعة. إذا كان True ولكن لم يتم تكوين نموذج إعادة الترتيب، فسيتم إصدار تحذير. افتراضي True."}},tct={loading:"جارٍ تحميل وثائق واجهة برمجة التطبيقات..."},rct={title:"مفتاح واجهة برمجة التطبيقات مطلوب",description:"الرجاء إدخال مفتاح واجهة برمجة التطبيقات للوصول إلى الخدمة",placeholder:"أدخل مفتاح واجهة برمجة التطبيقات",save:"حفظ"},nct={showing:"عرض {{start}} إلى {{end}} من أصل {{total}} إدخالات",page:"الصفحة",pageSize:"حجم الصفحة",firstPage:"الصفحة الأولى",prevPage:"الصفحة السابقة",nextPage:"الصفحة التالية",lastPage:"الصفحة الأخيرة"},act={settings:Kut,header:Yut,login:Jut,common:Xut,documentPanel:Zut,graphPanel:Qut,retrievePanel:ect,apiSite:tct,apiKeyAlert:rct,pagination:nct},ict={language:"語言",theme:"主題",light:"淺色",dark:"深色",system:"系統"},oct={documents:"文件",knowledgeGraph:"知識圖譜",retrieval:"檢索",api:"API",projectRepository:"專案庫",logout:"登出",themeToggle:{switchToLight:"切換至淺色主題",switchToDark:"切換至深色主題"}},sct={description:"請輸入您的帳號和密碼登入系統",username:"帳號",usernamePlaceholder:"請輸入帳號",password:"密碼",passwordPlaceholder:"請輸入密碼",loginButton:"登入",loggingIn:"登入中...",successMessage:"登入成功",errorEmptyFields:"請輸入您的帳號和密碼",errorInvalidCredentials:"登入失敗，請檢查帳號和密碼",authDisabled:"認證已停用，使用免登入模式",guestMode:"免登入"},lct={cancel:"取消",save:"儲存",saving:"儲存中...",saveFailed:"儲存失敗"},uct={clearDocuments:{button:"清空",tooltip:"清空文件",title:"清空文件",description:"此操作將從系統中移除所有文件",warning:"警告：此操作將永久刪除所有文件，無法復原！",confirm:"確定要清空所有文件嗎？",confirmPrompt:"請輸入 yes 確認操作",confirmPlaceholder:"輸入 yes 以確認",clearCache:"清空 LLM 快取",confirmButton:"確定",clearing:"正在清除...",timeout:"清除操作逾時，請重試",success:"文件清空成功",cacheCleared:"快取清空成功",cacheClearFailed:`清空快取失敗：
{{error}}`,failed:`清空文件失敗：
{{message}}`,error:`清空文件失敗：
{{error}}`},deleteDocuments:{button:"刪除",tooltip:"刪除選取的文件",title:"刪除文件",description:"此操作將永久刪除選取的文件",warning:"警告：此操作將永久刪除選取的文件，無法復原！",confirm:"確定要刪除 {{count}} 個選取的文件嗎？",confirmPrompt:"請輸入 yes 確認操作",confirmPlaceholder:"輸入 yes 以確認",confirmButton:"確定",deleteFileOption:"同時刪除上傳檔案",deleteFileTooltip:"選取此選項將同時刪除伺服器上對應的上傳檔案",deleteLLMCacheOption:"同時刪除實體關係擷取 LLM 快取",success:"文件刪除流水線啟動成功",failed:`刪除文件失敗：
{{message}}`,error:`刪除文件失敗：
{{error}}`,busy:"pipeline 被佔用，請稍後再試",notAllowed:"沒有操作權限"},selectDocuments:{selectCurrentPage:"全選當前頁 ({{count}})",deselectAll:"取消全選 ({{count}})"},uploadDocuments:{button:"上傳",tooltip:"上傳文件",title:"上傳文件",description:"拖曳檔案至此處或點擊瀏覽",single:{uploading:"正在上傳 {{name}}：{{percent}}%",success:`上傳成功：
{{name}} 上傳完成`,failed:`上傳失敗：
{{name}}
{{message}}`,error:`上傳失敗：
{{name}}
{{error}}`},batch:{uploading:"正在上傳檔案...",success:"檔案上傳完成",error:"部分檔案上傳失敗"},generalError:`上傳失敗
{{error}}`,fileTypes:"支援的檔案類型：TXT, MD, DOCX, PDF, PPTX, XLSX, RTF, ODT, EPUB, HTML, HTM, TEX, JSON, XML, YAML, YML, CSV, LOG, CONF, INI, PROPERTIES, SQL, BAT, SH, C, CPP, PY, JAVA, JS, TS, SWIFT, GO, RB, PHP, CSS, SCSS, LESS",fileUploader:{singleFileLimit:"一次只能上傳一個檔案",maxFilesLimit:"最多只能上傳 {{count}} 個檔案",fileRejected:"檔案 {{name}} 被拒絕",unsupportedType:"不支援的檔案類型",fileTooLarge:"檔案過大，最大允許 {{maxSize}}",dropHere:"將檔案拖放至此處",dragAndDrop:"拖放檔案至此處，或點擊選擇檔案",removeFile:"移除檔案",uploadDescription:"您可以上傳{{isMultiple ? '多個' : count}}個檔案（每個檔案最大{{maxSize}}）",duplicateFile:"檔案名稱與伺服器上的快取重複"}},documentManager:{title:"文件管理",scanButton:"掃描/重試",scanTooltip:"掃描處理輸入目錄中的文件，同時重新處理所有失敗的文件",refreshTooltip:"重設文件清單",pipelineStatusButton:"管線狀態",pipelineStatusTooltip:"查看文件處理管線狀態",uploadedTitle:"已上傳文件",uploadedDescription:"已上傳文件清單及其狀態",emptyTitle:"無文件",emptyDescription:"尚未上傳任何文件",columns:{id:"ID",fileName:"檔案名稱",summary:"摘要",status:"狀態",length:"長度",chunks:"分塊",created:"建立時間",updated:"更新時間",metadata:"元資料",select:"選擇"},status:{all:"全部",completed:"已完成",preprocessed:"預處理",processing:"處理中",pending:"等待中",failed:"失敗"},errors:{loadFailed:`載入文件失敗
{{error}}`,scanFailed:`掃描文件失敗
{{error}}`,scanProgressFailed:`取得掃描進度失敗
{{error}}`},fileNameLabel:"檔案名稱",showButton:"顯示",hideButton:"隱藏",showFileNameTooltip:"顯示檔案名稱",hideFileNameTooltip:"隱藏檔案名稱"},pipelineStatus:{title:"流水線狀態",busy:"流水線忙碌",requestPending:"待處理請求",cancellationRequested:"取消請求",jobName:"作業名稱",startTime:"開始時間",progress:"進度",unit:"批",pipelineMessages:"流水線消息",cancelButton:"中斷",cancelTooltip:"中斷流水線處理",cancelConfirmTitle:"確認中斷流水線",cancelConfirmDescription:"此操作將中斷正在進行的流水線處理。確定要繼續嗎？",cancelConfirmButton:"確認中斷",cancelInProgress:"取消請求進行中...",pipelineNotRunning:"流水線未運行",cancelSuccess:"流水線中斷請求已發送",cancelFailed:`中斷流水線失敗
{{error}}`,cancelNotBusy:"流水線未運行，無需中斷",errors:{fetchFailed:`獲取流水線狀態失敗
{{error}}`}}},cct={dataIsTruncated:"圖資料已截斷至最大回傳節點數",statusDialog:{title:"LightRAG 伺服器設定",description:"查看目前系統狀態和連線資訊"},legend:"圖例",nodeTypes:{person:"人物角色",category:"分類",geo:"地理名稱",location:"位置",organization:"組織機構",event:"事件",equipment:"設備",weapon:"武器",animal:"動物",unknown:"未知",object:"物品",group:"群組",technology:"技術",product:"產品",document:"文檔",content:"內容",data:"資料",artifact:"人工製品",concept:"概念",naturalobject:"自然物品",method:"方法",creature:"生物神怪",plant:"植物",disease:"疾病",drug:"藥物",food:"食物",other:"其他"},sideBar:{settings:{settings:"設定",healthCheck:"健康檢查",showPropertyPanel:"顯示屬性面板",showSearchBar:"顯示搜尋列",showNodeLabel:"顯示節點標籤",nodeDraggable:"節點可拖曳",showEdgeLabel:"顯示 Edge 標籤",hideUnselectedEdges:"隱藏未選取的 Edge",edgeEvents:"Edge 事件",maxQueryDepth:"最大查詢深度",maxNodes:"最大回傳節點數",maxLayoutIterations:"最大版面配置迭代次數",resetToDefault:"重設為預設值",edgeSizeRange:"Edge 粗細範圍",depth:"深度",max:"最大值",degree:"鄰邊",apiKey:"API key",enterYourAPIkey:"輸入您的 API key",save:"儲存",refreshLayout:"重新整理版面配置"},zoomControl:{zoomIn:"放大",zoomOut:"縮小",resetZoom:"重設縮放",rotateCamera:"順時針旋轉圖形",rotateCameraCounterClockwise:"逆時針旋轉圖形"},layoutsControl:{startAnimation:"繼續版面配置動畫",stopAnimation:"停止版面配置動畫",layoutGraph:"圖形版面配置",layouts:{Circular:"環形",Circlepack:"圓形打包",Random:"隨機",Noverlaps:"無重疊","Force Directed":"力導向","Force Atlas":"力圖"}},fullScreenControl:{fullScreen:"全螢幕",windowed:"視窗"},legendControl:{toggleLegend:"切換圖例顯示"}},statusIndicator:{connected:"已連線",disconnected:"未連線"},statusCard:{unavailable:"狀態資訊不可用",serverInfo:"伺服器資訊",workingDirectory:"工作目錄",inputDirectory:"輸入目錄",maxParallelInsert:"並行處理文档",summarySettings:"摘要設定",llmConfig:"LLM 設定",llmBinding:"LLM 綁定",llmBindingHost:"LLM 端點",llmModel:"LLM 模型",embeddingConfig:"嵌入設定",embeddingBinding:"嵌入綁定",embeddingBindingHost:"嵌入端點",embeddingModel:"嵌入模型",storageConfig:"儲存設定",kvStorage:"KV 儲存",docStatusStorage:"文件狀態儲存",graphStorage:"圖形儲存",vectorStorage:"向量儲存",workspace:"工作空間",maxGraphNodes:"最大圖形節點數",rerankerConfig:"重排序設定",rerankerBindingHost:"重排序端點",rerankerModel:"重排序模型",lockStatus:"鎖定狀態",threshold:"閾值"},propertiesView:{editProperty:"編輯{{property}}",editPropertyDescription:"在下方文字區域編輯屬性值。",errors:{duplicateName:"節點名稱已存在",updateFailed:"更新節點失敗",tryAgainLater:"請稍後重試",updateSuccessButMergeFailed:"屬性已更新，但合併失敗：{{error}}",mergeFailed:"合併失敗：{{error}}"},success:{entityUpdated:"節點更新成功",relationUpdated:"關係更新成功",entityMerged:"節點合併成功"},mergeOptionLabel:"遇到重名時自動合併",mergeOptionDescription:"勾選後，重新命名為既有名稱時會自動將當前節點合併過去，不再報錯。",mergeDialog:{title:"節點已合併",description:'"{{source}}" 已合併到 "{{target}}"。',refreshHint:"請重新整理圖譜以取得最新結構。",keepCurrentStart:"重新整理並保留目前的起始節點",useMergedStart:"重新整理並以合併後的節點為起始節點",refreshing:"正在重新整理圖譜..."},node:{title:"節點",id:"ID",labels:"標籤",degree:"度數",properties:"屬性",relationships:"關係(子圖內)",expandNode:"展開節點",pruneNode:"修剪節點",deleteAllNodesError:"拒絕刪除圖中的所有節點",nodesRemoved:"已刪除 {{count}} 個節點，包括孤立節點",noNewNodes:"沒有發現可以展開的節點",propertyNames:{description:"描述",entity_id:"名稱",entity_type:"類型",source_id:"C-ID",Neighbour:"鄰接",file_path:"檔案",keywords:"Keys",weight:"權重"}},edge:{title:"關係",id:"ID",type:"類型",source:"來源節點",target:"目標節點",properties:"屬性"}},search:{placeholder:"頁面內搜尋節點...",message:"還有 {{count}} 個"},graphLabels:{selectTooltip:"獲取節點(標籤)子圖",noLabels:"未找到匹配的節點",label:"搜尋節點名稱",placeholder:"搜尋節點名稱...",andOthers:"還有 {{count}} 個",refreshGlobalTooltip:"重新整理全圖資料和重置搜尋歷史",refreshCurrentLabelTooltip:"重新整理目前頁面圖形資料",refreshingTooltip:"正在重新整理資料..."},emptyGraph:"無數據(請重載圖形數據)"},dct={chatMessage:{copyTooltip:"複製到剪貼簿",copyError:"複製文字到剪貼簿失敗",copyEmpty:"沒有內容可複製",copySuccess:"內容已複製到剪貼簿",copySuccessLegacy:"內容已複製（傳統方法）",copySuccessManual:"內容已複製（手動方法）",copyFailed:"複製內容失敗",copyManualInstruction:"請手動選取並複製文字",thinking:"正在思考...",thinkingTime:"思考用時 {{time}} 秒",thinkingInProgress:"思考進行中..."},retrieval:{startPrompt:"輸入查詢開始檢索",clear:"清空",send:"送出",placeholder:"輸入查詢內容 (支援模式前綴：/<Query Mode>)",error:"錯誤：取得回應失敗",queryModeError:"僅支援以下查詢模式：{{modes}}",queryModePrefixInvalid:"無效的查詢模式前綴。請使用：/<模式> [空格] 查詢內容"},querySettings:{parametersTitle:"參數",parametersDescription:"設定查詢參數",queryMode:"查詢模式",queryModeTooltip:`選擇檢索策略：
• Naive：傳統文字塊向量檢索
• Local：側重實體檢索
• Global：側重關係檢索
• Hybrid：Local+Global
• Mix：Local+Global+Naive
• Bypass：跳過檢索，把歷史會話與當前問題送LLM`,queryModeOptions:{naive:"Naive",local:"Local",global:"Global",hybrid:"Hybrid",mix:"Mix",bypass:"Bypass"},responseFormat:"回應格式",responseFormatTooltip:`定義回應格式。例如：
• 多段落
• 單段落
• 重點`,responseFormatOptions:{multipleParagraphs:"多段落",singleParagraph:"單段落",bulletPoints:"重點"},topK:"知識圖譜 Top K",topKTooltip:"實體關係檢索數量，適用於非 naive 模式。",topKPlaceholder:"輸入 top_k 值",chunkTopK:"文本區塊 Top K",chunkTopKTooltip:"文本區塊檢索數量，適用於所有模式。",chunkTopKPlaceholder:"輸入文本區塊 chunk_top_k 值",historyTurns:"歷史輪次",historyTurnsTooltip:"回應上下文中考慮的完整對話輪次（使用者-助手對）數量",historyTurnsPlaceholder:"歷史輪次數",onlyNeedContext:"僅需上下文",onlyNeedContextTooltip:"如果為True，僅回傳檢索到的上下文而不產生回應",onlyNeedPrompt:"僅需提示",onlyNeedPromptTooltip:"如果為True，僅回傳產生的提示而不產生回應",streamResponse:"串流回應",streamResponseTooltip:"如果為True，啟用即時串流輸出回應",userPrompt:"附加輸出提示詞",userPromptTooltip:"向LLM提供額外的響應要求（與查詢內容無關，僅用於處理輸出）。",userPromptPlaceholder:"輸入自定義提示詞（可選）",enableRerank:"啟用重排",enableRerankTooltip:"為檢索到的文本塊啟用重排。如果為True但未配置重排模型，將發出警告。默認為True。",maxEntityTokens:"實體令牌數上限",maxEntityTokensTooltip:"統一令牌控制系統中分配給實體上下文的最大令牌數",maxRelationTokens:"關係令牌數上限",maxRelationTokensTooltip:"統一令牌控制系統中分配給關係上下文的最大令牌數",maxTotalTokens:"總令牌數上限",maxTotalTokensTooltip:"整個查詢上下文的最大總令牌預算（實體+關係+文檔塊+系統提示）"}},hct={loading:"正在載入 API 文件..."},pct={title:"需要 API key",description:"請輸入您的 API key 以存取服務",placeholder:"請輸入 API key",save:"儲存"},fct={showing:"顯示第 {{start}} 到 {{end}} 筆，共 {{total}} 筆記錄",page:"頁",pageSize:"每頁顯示",firstPage:"第一頁",prevPage:"上一頁",nextPage:"下一頁",lastPage:"最後一頁"},mct={settings:ict,header:oct,login:sct,common:lct,documentPanel:uct,graphPanel:cct,retrievePanel:dct,apiSite:hct,apiKeyAlert:pct,pagination:fct},gct=()=>{var e;try{const t=localStorage.getItem("settings-storage");if(t)return((e=JSON.parse(t).state)==null?void 0:e.language)||"en"}catch(t){console.error("Failed to get stored language:",t)}return"en"};na.use(uye).init({resources:{en:{translation:Cut},zh:{translation:Out},fr:{translation:Vut},ar:{translation:act},zh_TW:{translation:mct}},lng:gct(),fallbackLng:"en",interpolation:{escapeValue:!1},returnEmptyString:!1,returnNull:!1});Ve.subscribe(e=>{const t=e.language;na.language!==t&&na.changeLanguage(t)});upe.createRoot(document.getElementById("root")).render(x.jsx(T.StrictMode,{children:x.jsx(Ylt,{})}));export{Tae as $,Fm as A,yze as B,zHe as C,qN as D,Ma as E,bae as F,gKe as G,yje as H,Rz as I,kqe as J,eHe as K,Qh as L,kdt as M,q2 as N,vHe as O,cN as P,Gz as Q,uje as R,q1 as S,mKe as T,Ug as U,AHe as V,Hg as W,Ft as X,Ut as Y,sKe as Z,U as _,LHe as a,nse as a$,ije as a0,uH as a1,lH as a2,Idt as a3,Rdt as a4,Ldt as a5,Ndt as a6,Fdt as a7,ih as a8,wN as a9,nKe as aA,YWe as aB,JVe as aC,ON as aD,sU as aE,RYe as aF,oje as aG,wdt as aH,jn as aI,Kg as aJ,FYe as aK,AYe as aL,vN as aM,mu as aN,rH as aO,z$e as aP,Jm as aQ,ld as aR,aKe as aS,mse as aT,H2 as aU,j2 as aV,iv as aW,bse as aX,fse as aY,MVe as aZ,Nt as a_,Mdt as aa,_dt as ab,Bdt as ac,Odt as ad,Ddt as ae,tJe as af,Cue as ag,Vdt as ah,Tqe as ai,ra as aj,Gu as ak,IN as al,Xse as am,dd as an,wse as ao,ir as ap,cs as aq,WJe as ar,Wdt as as,Kdt as at,jdt as au,Lt as av,qdt as aw,FJe as ax,wJe as ay,xJe as az,NHe as b,vP as b0,fP as b1,xc as b2,w2 as b3,$re as b4,JIe as b5,hi as b6,N$e as b7,yN as b8,Hae as b9,jWe as bA,Np as bB,$u as bC,kH as bD,zN as bE,cse as bF,V6 as bG,iKe as bH,G2 as bI,Pr as bJ,Gg as ba,Gae as bb,Adt as bc,bze as bd,rKe as be,YVe as bf,OWe as bg,BN as bh,RVe as bi,oKe as bj,Wg as bk,Dp as bl,rv as bm,UVe as bn,mXe as bo,qg as bp,av as bq,IVe as br,lse as bs,zWe as bt,HWe as bu,vc as bv,_H as bw,UWe as bx,PN as by,PWe as bz,Gr as c,Lr as d,Cae as e,Qn as f,IHe as g,Cl as h,uo as i,Nqe as j,_p as k,je as l,Cse as m,Cdt as n,Jdt as o,OHe as p,BHe as q,Ydt as r,MHe as s,Cqe as t,_o as u,vJe as v,vKe as w,Pdt as x,DHe as y,Tdt as z};
